#!/bin/sh
# $UFP$

# Copyright © 2021 Trond Endrestøl <trond.endrestol@ximalas.info>

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

#set -x
set -eu -o pipefail
#set -u -o pipefail

# These are our working copies.
WORKING_COPIES="/usr/src /usr/ports"
readonly WORKING_COPIES

# Determine which git executable to run.
GIT_CANDIDATES="/usr/local/bin/git /usr/bin/git /usr/bin/gitlite"
readonly GIT_CANDIDATES

for cand in ${GIT_CANDIDATES}; do
  if [ -x ${cand} ]; then
    GIT=${cand}
    break
  fi
done
unset cand

if [ -z "${GIT}" ]; then
  echo "${0}: neither of ${GIT_CANDIDATES} are useable" >/dev/stderr
  exit 69
fi

readonly GIT

if which -s parallel; then # Assuming parallel is GNU parallel or something compatible.
    parallel ${GIT} -C {} prune -v             ::: ${WORKING_COPIES}
    parallel ${GIT} -C {} remote prune freebsd ::: ${WORKING_COPIES}
    parallel ${GIT} -C {} gc --auto            ::: ${WORKING_COPIES}
else
  for wc in ${WORKING_COPIES}; do
    ${GIT} -C ${wc} prune -v
    ${GIT} -C ${wc} remote prune freebsd
    ${GIT} -C ${wc} gc --auto
  done
fi

# EOF
