#!/bin/sh

# Copyright © 2015 Trond Endrestøl <Trond.Endrestol@ximalas.info>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

#set -x

ZFS=/sbin/zfs
DATE=/bin/date
DATE_FLAGS=-u

if [ "${1}" = "-h" -o "${1}" = "--help" ]; then
  help=1;
fi

if [ ${#} -lt 2 -o -n "${help}" ]; then
  echo "Usage: ${0} <existing-dataset> <new-dataset>";

  if [ -n "${help}" ]; then
    exit 0;
  else
    exit 69;
  fi;
fi

if ! ${ZFS} list ${1} >/dev/null 2>&1; then
  echo "${0}: dataset ${1} not found" >/dev/stderr;
  exit 69;
fi

if ${ZFS} list ${2} >/dev/null 2>&1; then
  echo "${0}: dataset ${2} exists" >/dev/stderr;
  exit 69;
fi

SNAPSHOT="automatic-transfer-${$}-`${DATE} ${DATE_FLAGS} +%Y-%m-%dT%H:%M:%S`"

${ZFS} snapshot ${1}@${SNAPSHOT}                           || exit
${ZFS} send -p  ${1}@${SNAPSHOT} | ${ZFS} receive -uv ${2} || exit
${ZFS} destroy  ${1}@${SNAPSHOT}                           || exit
${ZFS} destroy  ${2}@${SNAPSHOT}                           || exit
