--
-- PostgreSQL database dump
--

-- Dumped from database version 13.0
-- Dumped by pg_dump version 13.0

SET statement_timeout = 0;
SET lock_timeout = 0;
SET idle_in_transaction_session_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SELECT pg_catalog.set_config('search_path', '', false);
SET check_function_bodies = false;
SET xmloption = content;
SET client_min_messages = warning;
SET row_security = off;

--
-- Name: pg_qualstats; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS pg_qualstats WITH SCHEMA public;


--
-- Name: EXTENSION pg_qualstats; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION pg_qualstats IS 'An extension collecting statistics about quals';


--
-- Name: pg_stat_statements; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS pg_stat_statements WITH SCHEMA public;


--
-- Name: EXTENSION pg_stat_statements; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION pg_stat_statements IS 'track planning and execution statistics of all SQL statements executed';


--
-- Name: pg_stat_kcache; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS pg_stat_kcache WITH SCHEMA public;


--
-- Name: EXTENSION pg_stat_kcache; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION pg_stat_kcache IS 'Kernel statistics gathering';


--
-- Name: create_logfilenames(timestamp with time zone, timestamp with time zone); Type: FUNCTION; Schema: public; Owner: trond
--

CREATE FUNCTION public.create_logfilenames(timestamp with time zone, timestamp with time zone) RETURNS SETOF text
    LANGUAGE plpgsql
    AS $_$
DECLARE
  r portsbuildstats%rowtype;
BEGIN
  FOR r IN SELECT * FROM portsbuildstats WHERE start >= $1 AND finished <= $2 ORDER BY category, portname, flavor LOOP
    IF r.flavor IS NULL THEN
      RETURN NEXT r.category || '___' || r.portname || '.log';
    ELSE
      RETURN NEXT r.category || '___' || r.portname || '@' || r.flavor || '.log';
    END IF;
  END LOOP;

  RETURN;
END;
$_$;


ALTER FUNCTION public.create_logfilenames(timestamp with time zone, timestamp with time zone) OWNER TO trond;

--
-- Name: insert_pkg(text, text, text, timestamp with time zone, timestamp with time zone); Type: FUNCTION; Schema: public; Owner: trond
--

CREATE FUNCTION public.insert_pkg(text, text, text, timestamp with time zone, timestamp with time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
BEGIN
  INSERT INTO portsbuildstats (category, portname, pkgname, start, finished) VALUES ($1, $2, $3, $4, $5);
END;
$_$;


ALTER FUNCTION public.insert_pkg(text, text, text, timestamp with time zone, timestamp with time zone) OWNER TO trond;

--
-- Name: insert_pkg(text, text, text, text, timestamp with time zone, timestamp with time zone); Type: FUNCTION; Schema: public; Owner: trond
--

CREATE FUNCTION public.insert_pkg(text, text, text, text, timestamp with time zone, timestamp with time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
BEGIN
  INSERT INTO portsbuildstats (category, portname, flavor, pkgname, start, finished) VALUES ($1, $2, $3, $4, $5, $6);
END;
$_$;


ALTER FUNCTION public.insert_pkg(text, text, text, text, timestamp with time zone, timestamp with time zone) OWNER TO trond;

SET default_tablespace = '';

SET default_table_access_method = heap;

--
-- Name: portsbuildstats; Type: TABLE; Schema: public; Owner: trond
--

CREATE TABLE public.portsbuildstats (
    start timestamp(0) with time zone NOT NULL,
    finished timestamp(0) with time zone NOT NULL,
    category text NOT NULL,
    portname text NOT NULL,
    flavor text,
    pkgname text NOT NULL,
    CONSTRAINT start_before_finished_ck CHECK ((finished > start))
);


ALTER TABLE public.portsbuildstats OWNER TO trond;

--
-- Name: portsbuildstats_with_duration; Type: VIEW; Schema: public; Owner: trond
--

CREATE VIEW public.portsbuildstats_with_duration AS
 SELECT p.category,
    p.portname,
    p.flavor,
    p.pkgname,
    p.start,
    p.finished,
    make_interval(0, 0, 0, 0, (((date_part('days'::text, (p.finished - p.start)))::integer * 24) + (date_part('hours'::text, (p.finished - p.start)))::integer), (date_part('minutes'::text, (p.finished - p.start)))::integer, ((date_part('seconds'::text, (p.finished - p.start)))::integer)::double precision) AS duration
   FROM public.portsbuildstats p;


ALTER TABLE public.portsbuildstats_with_duration OWNER TO trond;

--
-- Name: portsbuildstats_statistics; Type: VIEW; Schema: public; Owner: trond
--

CREATE VIEW public.portsbuildstats_statistics AS
 SELECT count(p.portname) AS count,
    min(p.duration) AS min,
    max(p.duration) AS max,
    avg(p.duration) AS avg,
    ( SELECT percentile_cont((0.5)::double precision) WITHIN GROUP (ORDER BY p.duration) AS percentile_cont) AS median,
    ( SELECT mode() WITHIN GROUP (ORDER BY p.duration) AS mode) AS mode,
    sum(p.duration) AS sum
   FROM public.portsbuildstats_with_duration p;


ALTER TABLE public.portsbuildstats_statistics OWNER TO trond;

--
-- Name: portsbuildstats_statistics_category; Type: VIEW; Schema: public; Owner: trond
--

CREATE VIEW public.portsbuildstats_statistics_category AS
 SELECT p.category,
    count(p.portname) AS count,
    min(p.duration) AS min,
    max(p.duration) AS max,
    avg(p.duration) AS avg,
    ( SELECT percentile_cont((0.5)::double precision) WITHIN GROUP (ORDER BY p.duration) AS percentile_cont) AS median,
    ( SELECT mode() WITHIN GROUP (ORDER BY p.duration) AS mode) AS mode,
    sum(p.duration) AS sum
   FROM public.portsbuildstats_with_duration p
  GROUP BY p.category
  ORDER BY p.category;


ALTER TABLE public.portsbuildstats_statistics_category OWNER TO trond;

--
-- Name: portsbuildstats_statistics_category_portname; Type: VIEW; Schema: public; Owner: trond
--

CREATE VIEW public.portsbuildstats_statistics_category_portname AS
 SELECT p.category,
    p.portname,
    count(p.portname) AS count,
    min(p.duration) AS min,
    max(p.duration) AS max,
    avg(p.duration) AS avg,
    ( SELECT percentile_cont((0.5)::double precision) WITHIN GROUP (ORDER BY p.duration) AS percentile_cont) AS median,
    ( SELECT mode() WITHIN GROUP (ORDER BY p.duration) AS mode) AS mode,
    sum(p.duration) AS sum
   FROM public.portsbuildstats_with_duration p
  GROUP BY p.category, p.portname
  ORDER BY p.category, p.portname;


ALTER TABLE public.portsbuildstats_statistics_category_portname OWNER TO trond;

--
-- Name: portsbuildstats_statistics_category_portname_flavor; Type: VIEW; Schema: public; Owner: trond
--

CREATE VIEW public.portsbuildstats_statistics_category_portname_flavor AS
 SELECT p.category,
    p.portname,
    p.flavor,
    count(p.portname) AS count,
    min(p.duration) AS min,
    max(p.duration) AS max,
    avg(p.duration) AS avg,
    ( SELECT percentile_cont((0.5)::double precision) WITHIN GROUP (ORDER BY p.duration) AS percentile_cont) AS median,
    ( SELECT mode() WITHIN GROUP (ORDER BY p.duration) AS mode) AS mode,
    sum(p.duration) AS sum
   FROM public.portsbuildstats_with_duration p
  GROUP BY p.category, p.portname, p.flavor
  ORDER BY p.category, p.portname, p.flavor;


ALTER TABLE public.portsbuildstats_statistics_category_portname_flavor OWNER TO trond;

--
-- Name: portsbuildstats_statistics_category_portname_flavor_pkgname; Type: VIEW; Schema: public; Owner: trond
--

CREATE VIEW public.portsbuildstats_statistics_category_portname_flavor_pkgname AS
 SELECT p.category,
    p.portname,
    p.flavor,
    p.pkgname,
    count(p.portname) AS count,
    min(p.duration) AS min,
    max(p.duration) AS max,
    avg(p.duration) AS avg,
    ( SELECT percentile_cont((0.5)::double precision) WITHIN GROUP (ORDER BY p.duration) AS percentile_cont) AS median,
    ( SELECT mode() WITHIN GROUP (ORDER BY p.duration) AS mode) AS mode,
    sum(p.duration) AS sum
   FROM public.portsbuildstats_with_duration p
  GROUP BY p.category, p.portname, p.flavor, p.pkgname
  ORDER BY p.category, p.portname, p.flavor, p.pkgname;


ALTER TABLE public.portsbuildstats_statistics_category_portname_flavor_pkgname OWNER TO trond;

--
-- Name: portsbuildstats_statistics_category_portname_pkgname; Type: VIEW; Schema: public; Owner: trond
--

CREATE VIEW public.portsbuildstats_statistics_category_portname_pkgname AS
 SELECT p.category,
    p.portname,
    p.pkgname,
    count(p.portname) AS count,
    min(p.duration) AS min,
    max(p.duration) AS max,
    avg(p.duration) AS avg,
    ( SELECT percentile_cont((0.5)::double precision) WITHIN GROUP (ORDER BY p.duration) AS percentile_cont) AS median,
    ( SELECT mode() WITHIN GROUP (ORDER BY p.duration) AS mode) AS mode,
    sum(p.duration) AS sum
   FROM public.portsbuildstats_with_duration p
  GROUP BY p.category, p.portname, p.pkgname
  ORDER BY p.category, p.portname, p.pkgname;


ALTER TABLE public.portsbuildstats_statistics_category_portname_pkgname OWNER TO trond;

--
-- Data for Name: portsbuildstats; Type: TABLE DATA; Schema: public; Owner: trond
--

COPY public.portsbuildstats (start, finished, category, portname, flavor, pkgname) FROM stdin;
2020-05-25 07:51:33+02	2020-05-25 07:53:38+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-05-25 22:48:02+02	2020-05-25 22:48:40+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.14.2
2020-05-25 22:53:29+02	2020-05-25 22:54:25+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-05-25 23:04:00+02	2020-05-25 23:04:38+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-05-24 16:52:44+02	2020-05-24 17:07:15+02	local	E590T-localbase	\N	E590T-localbase-2020052200
2020-05-25 23:40:18+02	2020-05-25 23:43:15+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-05-26 19:37:36+02	2020-05-26 19:39:50+02	devel	bzr	py27	bzr-2.7.0.6622
2020-05-20 06:42:17+02	2020-05-20 06:43:42+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-04-25 10:02:25+02	2020-04-25 10:07:13+02	graphics	libzmf	\N	libzmf-0.0.2_21
2020-05-16 15:54:42+02	2020-05-16 16:02:39+02	graphics	mesa-dri	\N	mesa-dri-19.0.8
2020-05-16 15:19:33+02	2020-05-16 15:21:49+02	graphics	mesa-libs	\N	mesa-libs-19.0.8
2020-05-16 17:44:23+02	2020-05-16 17:46:36+02	graphics	movit	\N	movit-1.6.2_2
2020-03-02 08:44:04+01	2020-03-02 08:44:31+01	graphics	mypaint-brushes	\N	mypaint-brushes-1.3.1
2020-05-10 00:29:19+02	2020-05-10 00:31:36+02	graphics	netpbm	\N	netpbm-10.90.02
2020-05-20 08:51:07+02	2020-05-20 09:18:46+02	graphics	opencollada	\N	opencollada-1.6.68_2
2020-04-04 01:05:33+02	2020-04-04 01:24:24+02	graphics	opencv-core	\N	opencv-core-3.4.1_31
2020-05-23 08:59:36+02	2020-05-23 09:03:22+02	graphics	opencv	\N	opencv-3.4.1_31
2020-05-14 08:58:40+02	2020-05-14 09:02:50+02	graphics	openexr	\N	openexr-2.5.1
2020-02-28 23:26:26+01	2020-02-28 23:27:17+01	graphics	openjpeg	\N	openjpeg-2.3.1
2020-02-28 23:51:10+01	2020-02-28 23:51:38+01	graphics	openjpeg15	\N	openjpeg15-1.5.2_1
2020-05-21 01:43:41+02	2020-05-21 01:44:30+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-04-26 17:02:11+02	2020-04-26 17:05:43+02	graphics	pecl-imagick	php73	php73-pecl-imagick-3.4.4
2020-05-16 10:57:28+02	2020-05-16 10:57:53+02	graphics	php73-gd	\N	php73-gd-7.3.18
2020-02-28 22:17:50+01	2020-02-28 22:18:17+01	graphics	png	\N	png-1.6.37
2020-05-09 13:23:33+02	2020-05-09 13:27:35+02	graphics	podofo	\N	podofo-0.9.6_1
2020-02-28 22:52:44+01	2020-02-28 22:53:03+01	graphics	poppler-data	\N	poppler-data-0.4.9_2
2020-05-21 16:21:44+02	2020-05-21 16:22:23+02	graphics	poppler-glib	\N	poppler-glib-0.88.0
2020-05-21 17:08:35+02	2020-05-21 17:09:38+02	graphics	poppler-qt5	\N	poppler-qt5-0.88.0
2020-05-21 16:22:24+02	2020-05-21 16:22:51+02	graphics	poppler-utils	\N	poppler-utils-0.88.0
2020-05-21 16:21:00+02	2020-05-21 16:21:35+02	graphics	poppler	\N	poppler-0.88.0
2020-05-20 04:12:23+02	2020-05-20 04:12:52+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-05-21 01:36:42+02	2020-05-21 01:36:59+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-05-24 16:31:31+02	2020-05-24 16:32:32+02	graphics	py-gimp	\N	py27-gimp-2.10.18_6
2020-03-02 08:44:10+01	2020-03-02 08:44:20+01	graphics	py-imagesize	py27	py27-imagesize-1.1.0
2020-04-26 15:42:24+02	2020-04-26 15:42:41+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-02-24 15:36:58+01	2020-02-24 15:37:52+01	graphics	py-pillow	py27	py27-pillow-6.2.2
2020-05-09 12:49:20+02	2020-05-09 12:50:25+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-03-16 08:11:48+01	2020-03-16 08:12:18+01	graphics	py-pillow6	py27	py27-pillow6-6.2.2
2020-05-20 08:52:40+02	2020-05-20 08:53:50+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-03-20 09:30:37+01	2020-03-20 09:31:43+01	graphics	py-qt5-svg	py27	py27-qt5-svg-5.13.1
2020-05-20 08:53:51+02	2020-05-20 08:55:53+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-05-21 17:53:37+02	2020-05-21 17:53:55+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-05-20 08:55:55+02	2020-05-20 09:28:57+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-05-20 09:18:47+02	2020-05-20 09:20:42+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-05-20 02:23:38+02	2020-05-20 02:25:08+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-05-21 01:38:08+02	2020-05-21 01:40:07+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-05-21 01:44:30+02	2020-05-21 01:45:37+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-02-29 17:13:32+01	2020-02-29 17:18:41+01	graphics	sane-backends	\N	sane-backends-1.0.29
2020-05-16 19:28:53+02	2020-05-16 19:29:23+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-05-16 16:24:14+02	2020-05-16 16:24:39+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-05-16 17:47:05+02	2020-05-16 17:47:25+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-05-16 17:46:36+02	2020-05-16 17:47:05+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-05-15 08:31:08+02	2020-05-15 08:31:50+02	graphics	shaderc	\N	shaderc-2020.0
2020-03-16 22:37:22+01	2020-03-16 22:58:17+01	graphics	tesseract-data	\N	tesseract-data-4.0.0
2020-05-20 05:23:27+02	2020-05-20 05:38:55+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-02-28 22:30:15+01	2020-02-28 22:30:59+01	graphics	tiff	\N	tiff-4.1.0
2020-05-14 09:55:25+02	2020-05-14 09:56:02+02	graphics	vigra	\N	vigra-1.11.1_9
2020-05-15 08:08:50+02	2020-05-15 08:09:39+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-03-05 23:53:37+01	2020-03-05 23:53:45+01	graphics	wayland-protocols	\N	wayland-protocols-1.20
2020-03-02 07:18:29+01	2020-03-02 07:18:53+01	graphics	wayland	\N	wayland-1.18.0_3
2020-02-28 22:45:00+01	2020-02-28 22:46:31+01	graphics	webp	\N	webp-1.1.0
2020-05-21 13:04:09+02	2020-05-21 13:05:26+02	graphics	xfig	\N	xfig-3.2.7a
2020-05-21 04:57:06+02	2020-05-21 05:01:04+02	graphics	xpdf	\N	xpdf-4.02,1
2020-03-01 00:01:43+01	2020-03-01 00:02:09+01	graphics	aalib	\N	aalib-1.4.r5_13
2020-05-21 05:01:06+02	2020-05-21 05:06:44+02	irc	irssi	\N	irssi-1.2.2,1
2020-05-21 17:11:35+02	2020-05-21 17:20:40+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-02-29 06:51:59+01	2020-02-29 06:54:49+01	java	bootstrap-openjdk11	\N	bootstrap-openjdk11-11.0.5.10.1
2020-02-29 02:02:46+01	2020-02-29 02:04:31+01	java	bootstrap-openjdk8	\N	bootstrap-openjdk8-r450802
2020-05-10 11:43:19+02	2020-05-10 11:43:26+02	java	java-zoneinfo	\N	java-zoneinfo-2020.a
2020-03-22 12:36:27+01	2020-03-22 12:36:34+01	java	javavmwrapper	\N	javavmwrapper-2.7.5
2020-05-09 13:32:12+02	2020-05-09 13:54:45+02	java	openjdk11	\N	openjdk11-11.0.7+10.2
2020-05-09 15:19:20+02	2020-05-09 15:42:05+02	java	openjdk12	\N	openjdk12-12.0.2+10.4
2020-04-18 17:09:31+02	2020-04-18 17:30:46+02	java	openjdk13	\N	openjdk13-13.0.3+3.1
2020-05-10 11:43:43+02	2020-05-10 12:05:19+02	java	openjdk8	\N	openjdk8-8.252.09.1
2020-05-21 05:06:45+02	2020-05-21 05:09:23+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-04-27 23:32:03+02	2020-04-27 23:36:25+02	lang	cython	py27	py27-cython-0.29.15
2020-04-26 15:39:57+02	2020-04-26 15:44:59+02	lang	cython	py37	py37-cython-0.29.15
2020-02-29 17:55:37+01	2020-02-29 17:55:54+01	lang	erlang-man	\N	erlang-man-21.3_2
2020-05-21 17:53:54+02	2020-05-21 18:33:50+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-05-21 17:53:56+02	2020-05-21 18:39:52+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.1
2020-05-23 19:20:47+02	2020-05-23 19:41:02+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-03-07 11:37:27+01	2020-03-07 11:37:47+01	lang	fpc-rtl-console	\N	fpc-rtl-console-3.0.4_4
2020-03-07 03:18:20+01	2020-03-07 03:18:34+01	lang	fpc-rtl-extra	\N	fpc-rtl-extra-3.0.4_4
2020-03-07 03:17:30+01	2020-03-07 03:17:48+01	lang	fpc-rtl-objpas	\N	fpc-rtl-objpas-3.0.4_5
2020-03-07 11:37:48+01	2020-03-07 11:41:20+01	lang	fpc-source	\N	fpc-source-3.0.4_2
2020-05-09 22:44:13+02	2020-05-10 00:02:34+02	math	suitesparse	\N	suitesparse-5.7.2_1
2020-05-20 23:57:09+02	2020-05-21 00:07:40+02	devel	qt5-core	\N	qt5-core-5.14.2_3
2020-03-07 11:41:20+01	2020-03-07 11:42:54+01	lang	fpc-utils	\N	fpc-utils-3.0.4_6
2020-03-07 03:15:56+01	2020-03-07 03:17:29+01	lang	fpc	\N	fpc-3.0.4_4
2020-04-26 05:14:54+02	2020-04-26 05:15:08+02	lang	gcc	\N	gcc-9_4
2020-05-21 17:55:56+02	2020-05-21 20:03:55+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200516
2020-03-08 20:14:18+01	2020-03-08 20:57:48+01	lang	gcc48	\N	gcc48-4.8.5_11
2020-03-08 15:26:22+01	2020-03-08 16:07:54+01	lang	gcc6-aux	\N	gcc6-aux-20180516_1,1
2020-04-27 00:34:14+02	2020-04-27 01:59:23+02	lang	gcc9	\N	gcc9-9.3.0_1
2020-05-22 01:08:33+02	2020-05-22 02:16:43+02	lang	ghc	\N	ghc-8.8.3
2020-05-16 10:50:19+02	2020-05-16 10:57:27+02	lang	go	\N	go-1.14.3,1
2020-02-29 09:11:13+01	2020-02-29 09:13:31+01	lang	go14	\N	go14-1.4.3_6
2020-05-24 11:27:18+02	2020-05-24 11:27:39+02	lang	julia	\N	julia-1.0.3_3
2020-05-24 11:14:07+02	2020-05-24 11:27:17+02	lang	julia11	\N	julia11-1.1.1_5
2020-04-12 01:53:26+02	2020-04-12 01:53:49+02	lang	lua51	\N	lua51-5.1.5_9
2020-04-09 11:02:30+02	2020-04-09 11:02:49+02	lang	lua52	\N	lua52-5.2.4
2020-04-09 12:28:48+02	2020-04-09 12:29:21+02	lang	lua53	\N	lua53-5.3.5_3
2020-03-01 02:17:48+01	2020-03-01 02:18:48+01	lang	luajit	\N	luajit-2.0.5_4
2020-03-01 10:25:46+01	2020-03-01 10:26:12+01	lang	mmix	\N	mmix-20160804
2020-05-09 15:33:01+02	2020-05-09 16:03:12+02	lang	mono	\N	mono-5.10.1.57_3
2020-03-20 04:41:28+01	2020-03-20 04:41:49+01	lang	p5-Error	\N	p5-Error-0.17029
2020-03-20 07:52:26+01	2020-03-20 07:52:50+01	lang	p5-Try-Tiny	\N	p5-Try-Tiny-0.30
2020-03-19 20:11:57+01	2020-03-19 20:15:48+01	lang	perl5.30	\N	perl5-5.30.2
2020-05-16 10:50:19+02	2020-05-16 10:56:11+02	lang	php73	\N	php73-7.3.18
2020-03-20 05:30:45+01	2020-03-20 05:33:13+01	lang	py-qt5-qml	py27	py27-qt5-qml-5.13.1
2020-05-20 07:14:06+02	2020-05-20 07:17:10+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-03-01 10:58:22+01	2020-03-01 10:58:30+01	lang	python-mode.el	\N	python-mode.el-1.0_2,1
2020-04-27 13:38:55+02	2020-04-27 13:39:22+02	lang	python	\N	python-3.7_3,2
2020-04-27 23:26:02+02	2020-04-27 23:30:22+02	lang	python27	\N	python27-2.7.18
2020-04-27 08:25:20+02	2020-04-27 08:25:47+02	lang	python3	\N	python3-3_3
2020-04-26 15:30:09+02	2020-04-26 15:38:57+02	lang	python37	\N	python37-3.7.7
2020-05-16 18:34:45+02	2020-05-16 18:38:08+02	lang	python38	\N	python38-3.8.3
2020-05-16 15:38:41+02	2020-05-16 15:51:17+02	lang	ruby27	\N	ruby-2.7.1,1
2020-05-01 09:22:20+02	2020-05-01 13:06:50+02	lang	rust-nightly	\N	rust-nightly-1.45.0.20200427
2020-05-20 02:39:51+02	2020-05-20 05:43:57+02	lang	rust	\N	rust-1.43.1
2020-04-25 01:37:48+02	2020-04-25 02:04:48+02	lang	spidermonkey60	\N	spidermonkey60-60.9.0_3
2020-02-28 22:57:49+01	2020-02-28 23:00:13+01	lang	tcl86	\N	tcl86-8.6.10
2020-05-21 00:19:16+02	2020-05-21 00:23:18+02	lang	vala	\N	vala-0.40.15,1
2020-03-01 21:57:58+01	2020-03-01 21:58:19+01	local	rustapps	\N	rustapps-2020030100
2020-04-26 15:23:52+02	2020-04-26 15:28:58+02	mail	alpine	\N	alpine-2.22
2020-04-17 14:21:39+02	2020-04-17 14:21:55+02	mail	bbmail	\N	bbmail-0.9.3
2020-05-16 17:00:29+02	2020-05-16 17:04:40+02	mail	mailutils	\N	mailutils-3.9
2020-02-29 18:09:47+01	2020-02-29 18:23:41+01	math	coinmp	\N	CoinMP-1.8.4_3
2020-04-25 11:09:57+02	2020-04-25 11:16:32+02	math	cryptominisat	\N	cryptominisat-5.6.8_6
2020-03-01 00:05:03+01	2020-03-01 00:05:26+01	math	eigen3	\N	eigen-3.3.7
2020-04-19 17:55:47+02	2020-04-19 17:59:48+02	math	fftw3-float	\N	fftw3-float-3.3.8_6
2020-04-19 17:55:47+02	2020-04-19 17:59:53+02	math	fftw3	\N	fftw3-3.3.8_6
2020-03-07 11:42:55+01	2020-03-07 11:43:13+01	math	fpc-numlib	\N	fpc-numlib-3.0.4_4
2020-02-29 09:13:32+01	2020-02-29 09:14:00+01	math	glm	\N	glm-0.9.8.5_4,1
2020-03-08 12:06:15+01	2020-03-08 12:07:48+01	math	gmp	\N	gmp-6.2.0
2020-05-20 04:12:24+02	2020-05-20 04:15:05+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-03-01 02:37:10+01	2020-03-01 02:43:06+01	math	gsl	\N	gsl-2.6
2020-02-29 18:24:00+01	2020-02-29 18:26:45+01	math	lp_solve	\N	lp_solve-5.5.2.5
2020-05-20 21:44:18+02	2020-05-20 21:46:26+02	math	mate-calc	\N	mate-calc-1.22.2
2020-02-29 06:52:00+01	2020-02-29 06:52:42+01	math	metis	\N	metis-5.1.0_8
2020-03-28 19:45:02+01	2020-03-28 19:45:23+01	math	minisat	\N	minisat-2.2.1_2
2020-03-08 12:45:09+01	2020-03-08 12:45:33+01	math	mpc	\N	mpc-1.1.0_2
2020-03-08 12:07:49+01	2020-03-08 12:08:33+01	math	mpfr	\N	mpfr-4.0.2
2020-05-08 22:05:56+02	2020-05-08 22:32:42+02	math	openblas	\N	openblas-0.3.9_2,1
2020-02-29 18:26:46+01	2020-02-29 18:27:13+01	math	openlibm	\N	openlibm-0.6.0_1
2020-05-16 10:59:23+02	2020-05-16 10:59:41+02	math	pecl-bitset	php73	php73-pecl-bitset-3.0.1
2020-05-16 10:59:34+02	2020-05-16 10:59:57+02	math	php73-bcmath	\N	php73-bcmath-7.3.18
2020-04-26 22:41:26+02	2020-04-26 22:44:14+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0
2020-05-18 08:20:04+02	2020-05-18 08:22:10+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-05-20 21:46:27+02	2020-05-20 21:47:42+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-05-16 18:36:16+02	2020-05-16 18:38:05+02	math	py-numpy	py27	py27-numpy-1.16.6,1
2020-05-16 16:40:56+02	2020-05-16 16:43:17+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-05-21 18:33:51+02	2020-05-21 18:52:04+02	math	R	\N	R-4.0.0_1
2020-04-19 17:40:59+02	2020-04-19 17:41:06+02	math	sc	\N	sc-7.16_3
2020-04-27 08:30:17+02	2020-04-27 08:31:57+02	math	stp	\N	stp-2.3.3_5,1
2020-05-25 07:53:39+02	2020-05-25 08:06:18+02	editors	libreoffice	\N	libreoffice-6.3.6
2020-05-14 10:42:54+02	2020-05-14 11:03:34+02	math	z3	\N	z3-4.8.8
2020-03-01 02:45:25+01	2020-03-01 02:45:38+01	misc	chmlib	\N	chmlib-0.40_1
2020-03-03 20:24:04+01	2020-03-03 20:24:10+01	misc	compat12x	\N	compat12x-amd64-12.1.1201000.20200220
2020-03-24 07:46:57+01	2020-03-24 07:49:29+01	misc	e2fsprogs-libuuid	\N	e2fsprogs-libuuid-1.45.6
2020-03-07 03:28:52+01	2020-03-07 03:29:14+01	misc	fpc-chm	\N	fpc-chm-3.0.4_4
2020-05-23 09:25:00+02	2020-05-23 09:25:08+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200522
2020-02-29 01:13:24+01	2020-02-29 01:13:32+01	misc	getopt	\N	getopt-1.1.6
2020-04-27 08:31:58+02	2020-04-27 08:43:30+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-04-26 15:36:06+02	2020-04-26 15:37:06+02	misc	gnu-watch	\N	gnu-watch-3.3.16
2020-04-30 07:56:57+02	2020-04-30 07:57:12+02	misc	help2man	\N	help2man-1.47.15
2020-02-28 22:40:58+01	2020-02-28 22:41:06+01	misc	hicolor-icon-theme	\N	hicolor-icon-theme-0.17
2020-03-01 02:52:35+01	2020-03-01 02:52:57+01	misc	ini_file_manager	\N	ini_file_manager-03_3
2020-02-28 23:28:37+01	2020-02-28 23:29:11+01	misc	iso-codes	\N	iso-codes-4.2
2020-05-20 09:20:43+02	2020-05-20 09:30:37+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-05-21 08:12:44+02	2020-05-21 08:24:57+02	misc	mc	\N	mc-nox11-4.8.24
2020-05-05 19:39:19+02	2020-05-05 19:39:26+02	misc	pciids	\N	pciids-20200421
2020-03-03 17:35:23+01	2020-03-03 17:36:08+01	misc	pinfo	\N	pinfo-0.6.10_5
2020-02-25 17:18:46+01	2020-02-25 17:19:01+01	misc	py-pexpect	py37	py37-pexpect-4.8.0
2020-02-25 17:14:50+01	2020-02-25 17:14:58+01	misc	py-tqdm	py37	py37-tqdm-4.41.1
2020-03-16 01:06:24+01	2020-03-16 01:06:31+01	misc	qtchooser	\N	qtchooser-66_2
2020-05-19 23:55:24+02	2020-05-19 23:56:12+02	misc	shared-mime-info	\N	shared-mime-info-1.10_2
2020-05-05 23:46:25+02	2020-05-05 23:46:40+02	misc	usbids	\N	usbids-20200329
2020-04-17 11:36:49+02	2020-04-17 11:37:17+02	misc	xosd	\N	xosd-2.2.14_3
2020-05-21 01:13:11+02	2020-05-21 01:21:32+02	multimedia	aom	\N	aom-2.0.0
2020-03-17 05:19:38+01	2020-03-17 05:38:54+01	multimedia	assimp	\N	assimp-5.0.1
2020-05-23 09:47:05+02	2020-05-23 09:48:11+02	multimedia	celluloid	\N	celluloid-0.19
2020-05-07 20:18:18+02	2020-05-07 20:19:26+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-05-21 16:14:56+02	2020-05-21 16:16:28+02	multimedia	dav1d	\N	dav1d-0.7.0
2020-05-23 08:47:49+02	2020-05-23 08:59:11+02	multimedia	ffmpeg	\N	ffmpeg-4.2.3,1
2020-03-17 05:38:55+01	2020-03-17 05:40:37+01	multimedia	gavl	\N	gavl-1.4.0_4
2020-03-29 19:12:26+02	2020-03-29 19:13:39+02	multimedia	gmmlib	\N	gmmlib-20.1.1
2020-02-29 06:53:14+01	2020-02-29 06:53:20+01	multimedia	gmp-api	\N	gmp-api-58
2020-05-21 23:50:15+02	2020-05-22 00:00:33+02	astro	stellarium	\N	stellarium-0.20.1
2020-05-21 01:50:05+02	2020-05-21 01:52:15+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-05-21 01:44:43+02	2020-05-21 01:46:41+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-05-23 08:59:12+02	2020-05-23 08:59:42+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-05-21 01:27:50+02	2020-05-21 01:31:22+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-05-23 08:59:42+02	2020-05-23 09:00:12+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-05-20 04:13:46+02	2020-05-20 04:15:08+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-05-10 16:15:04+02	2020-05-10 16:16:23+02	net	py-pyzmq	py37	py37-pyzmq-19.0.1
2020-05-20 04:15:05+02	2020-05-20 04:15:34+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-05-20 02:53:18+02	2020-05-20 03:00:53+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-05-20 04:15:08+02	2020-05-20 04:16:12+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-05-20 04:15:34+02	2020-05-20 04:16:19+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-05-20 03:00:54+02	2020-05-20 03:04:31+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-05-21 01:01:31+02	2020-05-21 01:05:02+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-05-21 00:58:41+02	2020-05-21 01:01:29+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-05-20 09:28:58+02	2020-05-20 09:30:03+02	multimedia	libass	\N	libass-0.14.0
2020-03-01 00:05:44+01	2020-03-01 00:06:17+01	multimedia	libbdplus	\N	libbdplus-0.1.2
2020-05-09 11:23:40+02	2020-05-09 11:23:58+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-02-29 02:22:46+01	2020-02-29 02:23:04+01	multimedia	libdca	\N	libdca-0.0.6_1
2020-03-01 02:53:16+01	2020-03-01 02:53:48+01	multimedia	libdvbpsi	\N	libdvbpsi-1.3.3
2020-02-29 01:49:59+01	2020-02-29 01:50:12+01	multimedia	libdvdcss	\N	libdvdcss-1.4.2
2020-03-28 15:45:54+01	2020-03-28 15:46:10+01	multimedia	libdvdnav	\N	libdvdnav-6.1.0
2020-03-28 15:45:25+01	2020-03-28 15:45:53+01	multimedia	libdvdread	\N	libdvdread-6.1.0
2020-03-01 02:53:48+01	2020-03-01 02:54:28+01	multimedia	libmatroska	\N	libmatroska-1.5.2_1
2020-03-01 02:54:28+01	2020-03-01 02:55:18+01	multimedia	libmpeg2	\N	libmpeg2-0.5.1_6
2020-03-01 02:55:19+01	2020-03-01 02:55:51+01	multimedia	libmtp	\N	libmtp-1.1.17
2020-05-23 09:25:08+02	2020-05-23 09:26:27+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-05-10 16:10:54+02	2020-05-10 16:11:04+02	multimedia	librtmp	\N	librtmp-2.4.20190330
2020-03-01 00:06:18+01	2020-03-01 00:06:39+01	multimedia	libtheora	\N	libtheora-1.1.1_7
2020-04-19 09:26:09+02	2020-04-19 09:30:24+02	multimedia	libv4l	\N	libv4l-1.18.0
2020-05-16 21:15:00+02	2020-05-16 21:15:14+02	multimedia	libva-glx	\N	libva-glx-2.7.1
2020-05-09 01:38:04+02	2020-05-09 01:39:01+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.0
2020-05-07 21:35:52+02	2020-05-07 21:36:50+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-05-08 01:38:22+02	2020-05-08 02:02:19+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.1.1
2020-05-09 01:39:02+02	2020-05-09 01:40:19+02	multimedia	libva-utils	\N	libva-utils-2.7.1
2020-05-23 09:48:10+02	2020-05-23 09:48:28+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-05-05 19:43:27+02	2020-05-05 19:43:59+02	multimedia	libva	\N	libva-2.7.1
2020-05-23 08:47:36+02	2020-05-23 08:47:48+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-03-01 00:20:09+01	2020-03-01 00:23:58+01	multimedia	libvpx	\N	libvpx-1.8.2_1
2020-05-05 19:39:42+02	2020-05-05 19:43:27+02	multimedia	libx264	\N	libx264-0.159.2999
2020-04-26 15:30:08+02	2020-04-26 15:30:35+02	multimedia	libaacs	\N	libaacs-0.9.0
2020-05-23 09:13:33+02	2020-05-23 09:15:35+02	multimedia	mlt	\N	mlt-6.20.0
2020-05-23 09:25:20+02	2020-05-23 09:27:00+02	multimedia	mpv	\N	mpv-0.32.0_2,1
2020-05-23 09:48:12+02	2020-05-23 09:50:19+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-05-23 08:48:11+02	2020-05-23 08:48:51+02	multimedia	openh264	\N	openh264-2.1.1,2
2020-05-23 09:48:28+02	2020-05-23 09:50:51+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-05-20 04:39:43+02	2020-05-20 04:42:26+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-05-20 04:40:49+02	2020-05-20 04:48:47+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-05-10 16:12:11+02	2020-05-10 16:12:31+02	multimedia	rtmpdump	\N	rtmpdump-2.4.20190330
2020-05-23 09:50:20+02	2020-05-23 09:52:39+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-05-16 16:12:19+02	2020-05-16 16:12:39+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-04-18 09:03:18+02	2020-04-18 09:03:42+02	multimedia	v4l_compat	\N	v4l_compat-1.18.0
2020-03-17 05:40:38+01	2020-03-17 05:41:00+01	multimedia	vid.stab	\N	vid.stab-0.98.2
2020-05-23 09:26:28+02	2020-05-23 09:30:43+02	multimedia	vlc	\N	vlc-3.0.10_4,4
2020-05-21 23:54:21+02	2020-05-21 23:55:56+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-05-23 09:27:01+02	2020-05-23 09:27:53+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-03-02 07:51:06+01	2020-03-02 08:05:34+01	multimedia	x265	\N	x265-3.2.1_2
2020-03-01 00:23:59+01	2020-03-01 00:24:47+01	multimedia	xvid	\N	xvid-1.3.7,1
2020-05-21 00:01:57+02	2020-05-21 00:07:33+02	net	avahi-app	\N	avahi-app-0.7_3
2020-05-20 02:25:09+02	2020-05-20 02:28:34+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-02-29 01:13:32+01	2020-02-29 01:13:39+01	net	avahi-header	\N	avahi-header-0.7
2020-04-23 18:31:30+02	2020-04-23 18:32:36+02	net	cppzmq	\N	cppzmq-4.6.0
2020-02-29 18:36:33+01	2020-02-29 18:37:02+01	net	fping	\N	fping-4.2
2020-05-23 09:15:36+02	2020-05-23 09:18:18+02	net	freerdp	\N	freerdp-2.0.0.r4_9
2020-05-21 01:50:13+02	2020-05-21 01:51:27+02	net	geoclue	\N	geoclue-2.5.5
2020-05-08 02:02:20+02	2020-05-08 02:03:10+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-05-21 01:02:38+02	2020-05-21 01:03:25+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-05-20 05:38:56+02	2020-05-20 05:41:54+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-05-20 07:18:43+02	2020-05-20 07:19:36+02	net	gssdp	\N	gssdp-1.2.2
2020-05-20 09:32:18+02	2020-05-20 09:32:54+02	net	gupnp	\N	gupnp-1.1.1_1
2020-05-16 18:42:31+02	2020-05-16 18:50:31+02	net	libcmis	\N	libcmis-0.5.2_1
2020-05-20 04:16:12+02	2020-05-20 04:25:45+02	net	libmateweather	\N	libmateweather-1.22.1
2020-02-29 18:56:37+01	2020-02-29 18:56:54+01	net	libmaxminddb	\N	libmaxminddb-1.4.2
2020-05-16 17:05:40+02	2020-05-16 17:05:59+02	net	liboauth	\N	liboauth-1.0.3_4
2020-03-01 02:56:04+01	2020-03-01 02:56:30+01	net	libpcap	\N	libpcap-1.9.1_1
2020-02-28 23:42:06+01	2020-02-28 23:42:41+01	net	libproxy	\N	libproxy-0.4.15
2020-02-29 10:14:37+01	2020-02-29 10:15:21+01	net	libvncserver	\N	libvncserver-0.9.12_2
2020-04-24 00:39:34+02	2020-04-24 00:45:51+02	net	libzmq4	\N	libzmq4-4.3.1_1
2020-05-17 04:47:54+02	2020-05-17 04:49:08+02	net	liveMedia	\N	liveMedia-2020.05.15,2
2020-03-20 14:46:31+01	2020-03-20 14:47:48+01	net	mosh	\N	mosh-1.3.2_12
2020-05-23 09:27:54+02	2020-05-23 09:28:20+02	net	ndpi	\N	ndpi-3.2.d20200403_1,1
2020-04-24 00:38:46+02	2020-04-24 00:39:33+02	net	norm	\N	norm-1.5r6_1
2020-05-23 09:50:51+02	2020-05-23 09:52:34+02	net	ntopng	\N	ntopng-4.0.d20200414_1,1
2020-04-29 23:19:57+02	2020-04-29 23:21:05+02	net	openldap24-client	\N	openldap-client-2.4.50
2020-04-29 23:18:14+02	2020-04-29 23:19:57+02	net	openldap24-sasl-client	\N	openldap-sasl-client-2.4.50
2020-02-29 06:16:27+01	2020-02-29 06:17:16+01	net	openpgm	\N	openpgm-5.2.122_6
2020-03-20 02:13:42+01	2020-03-20 02:14:05+01	net	p5-IO-Socket-INET6	\N	p5-IO-Socket-INET6-2.72_1
2020-04-23 11:44:08+02	2020-04-23 11:45:21+02	net	p5-Net-HTTP	\N	p5-Net-HTTP-6.19
2020-03-20 01:57:20+01	2020-03-20 01:57:46+01	net	p5-Socket6	\N	p5-Socket6-0.29
2020-03-21 00:32:17+01	2020-03-21 00:32:31+01	net	p5-URI	\N	p5-URI-1.76
2020-05-16 10:59:41+02	2020-05-16 11:01:44+02	net	php73-ldap	\N	php73-ldap-7.3.18
2020-05-16 10:59:57+02	2020-05-16 11:00:25+02	net	php73-soap	\N	php73-soap-7.3.18
2020-05-16 11:00:09+02	2020-05-16 11:00:32+02	net	php73-sockets	\N	php73-sockets-7.3.18
2020-04-27 23:34:14+02	2020-04-27 23:34:38+02	net	py-ipaddress	py27	py27-ipaddress-1.0.23
2020-05-20 23:57:25+02	2020-05-21 00:01:56+02	print	harfbuzz	\N	harfbuzz-2.6.6
2020-04-30 07:39:22+02	2020-04-30 07:41:57+02	ports-mgmt	pkg	\N	pkg-1.14.4
2020-05-24 19:15:21+02	2020-05-24 19:16:59+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-04-27 08:47:09+02	2020-04-27 08:47:51+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-03-03 06:49:43+01	2020-03-03 06:50:09+01	net	py-netifaces	py27	py27-netifaces-0.10.9
2020-04-27 08:47:52+02	2020-04-27 08:48:24+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-03-02 08:32:51+01	2020-03-02 08:33:02+01	net	py-pysocks	py27	py27-pysocks-1.7.1
2020-04-26 15:41:26+02	2020-04-26 15:41:52+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-03-20 03:57:05+01	2020-03-20 04:00:09+01	net	py-qt5-network	py27	py27-qt5-network-5.13.1
2020-05-20 04:02:06+02	2020-05-20 04:05:08+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-05-21 16:54:21+02	2020-05-21 16:54:33+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-03-04 21:50:44+01	2020-03-04 21:50:53+01	net	py-urllib3	py27	py27-urllib3-1.25.7,1
2020-05-18 22:05:41+02	2020-05-18 22:05:54+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-05-21 00:23:47+02	2020-05-21 00:28:47+02	net	qt5-network	\N	qt5-network-5.14.2
2020-04-17 14:44:27+02	2020-04-17 14:45:12+02	net	rdesktop	\N	rdesktop-1.8.6
2020-05-20 09:32:42+02	2020-05-20 09:33:38+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-05-20 21:55:44+02	2020-05-20 21:57:17+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-05-23 09:28:15+02	2020-05-23 09:29:01+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-05-20 21:59:21+02	2020-05-20 22:00:55+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-05-20 22:00:56+02	2020-05-20 22:03:09+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-05-20 22:03:10+02	2020-05-20 22:04:45+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-05-20 22:04:46+02	2020-05-20 22:05:48+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-05-20 22:05:49+02	2020-05-20 22:08:02+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-05-20 22:08:02+02	2020-05-20 22:12:03+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-05-23 09:52:34+02	2020-05-23 09:53:44+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-05-20 02:31:31+02	2020-05-20 02:32:54+02	net	remmina	\N	remmina-1.3.9
2020-05-23 08:49:07+02	2020-05-23 08:59:36+02	net	samba410	\N	samba410-4.10.15
2020-05-16 17:06:31+02	2020-05-16 17:06:48+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-03-01 11:34:05+01	2020-03-01 11:34:13+01	net	svnup	\N	svnup-1.08
2020-03-01 11:34:14+01	2020-03-01 11:34:32+01	net	torsocks	\N	torsocks-2.3.0_1
2020-05-20 05:43:58+02	2020-05-20 05:45:09+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-05-21 22:41:21+02	2020-05-21 22:41:36+02	net	vde2	\N	vde2-2.3.2_4
2020-03-23 06:59:16+01	2020-03-23 06:59:35+01	net	wireguard-go	\N	wireguard-go-0.0.20200320
2020-05-16 11:42:41+02	2020-05-16 11:42:54+02	net	wireguard	\N	wireguard-1.0.20200513
2020-05-23 09:52:40+02	2020-05-23 10:11:19+02	net	wireshark	\N	wireshark-3.2.4
2020-05-21 09:48:27+02	2020-05-21 09:49:38+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-05-23 09:53:45+02	2020-05-23 09:55:09+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-05-21 09:52:05+02	2020-05-21 09:53:08+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-05-21 09:53:08+02	2020-05-21 09:54:09+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-05-20 22:12:04+02	2020-05-20 22:15:07+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-05-21 09:54:10+02	2020-05-21 09:55:24+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-05-21 09:55:26+02	2020-05-21 09:56:32+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-05-21 09:56:33+02	2020-05-21 09:57:32+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-05-21 09:57:34+02	2020-05-21 09:58:36+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-05-21 09:58:36+02	2020-05-21 09:59:47+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-05-23 09:54:37+02	2020-05-23 09:56:13+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-05-20 07:19:37+02	2020-05-20 07:21:17+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-05-21 09:59:48+02	2020-05-21 10:01:04+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-05-23 09:27:09+02	2020-05-23 09:28:15+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-05-20 09:32:55+02	2020-05-20 09:33:51+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-05-23 09:28:20+02	2020-05-23 09:29:40+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-05-21 10:01:05+02	2020-05-21 10:02:05+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-05-21 01:51:28+02	2020-05-21 01:52:35+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-03-01 12:08:15+01	2020-03-01 12:08:26+01	net-mgmt	iftop	\N	iftop-1.0.p4
2020-03-20 15:19:55+01	2020-03-20 15:20:10+01	net-mgmt	ipcalc	\N	ipcalc-0.41_3
2020-03-20 15:20:11+01	2020-03-20 15:20:50+01	net-mgmt	ipv6calc	\N	ipv6calc-2.2.0
2020-03-01 03:09:18+01	2020-03-01 03:10:06+01	net-mgmt	libsmi	\N	libsmi-0.4.8_1
2020-05-23 09:29:02+02	2020-05-23 09:29:59+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-05-21 19:56:54+02	2020-05-21 19:57:49+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-04-17 15:04:19+02	2020-04-17 15:04:27+02	net-mgmt	nstat	\N	nstat-1.0_3
2020-04-27 23:40:20+02	2020-04-27 23:40:40+02	net-mgmt	py-ipcalc	py27	py27-ipcalc-1.99.0
2020-05-21 17:20:43+02	2020-05-21 17:20:58+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.7.1
2020-04-12 06:26:51+02	2020-04-12 06:27:03+02	net-mgmt	whatmask	\N	whatmask-1.2_1
2020-03-01 03:11:30+01	2020-03-01 03:11:37+01	ports-mgmt	dialog4ports	\N	dialog4ports-0.1.6
2020-03-20 15:20:18+01	2020-03-20 15:20:36+01	ports-mgmt	pkg_tree	\N	pkg_tree-1.1_4
2020-05-21 10:02:06+02	2020-05-21 10:03:12+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-05-23 09:55:09+02	2020-05-23 09:55:34+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-03-04 21:20:57+01	2020-03-04 21:21:40+01	ports-mgmt	synth	\N	synth-2.08
2020-05-09 12:25:25+02	2020-05-09 12:25:42+02	print	adobe-cmaps	\N	adobe-cmaps-20051217_4
2020-05-21 18:51:09+02	2020-05-21 18:53:59+02	print	auctex	full	auctex-emacs26-12.2
2020-05-21 16:32:58+02	2020-05-21 16:33:33+02	print	cups-filters	\N	cups-filters-1.27.4_1
2020-05-21 00:07:34+02	2020-05-21 00:09:12+02	print	cups	\N	cups-2.2.13
2020-03-20 04:00:10+01	2020-03-20 04:00:49+01	print	enscript-a4	\N	enscript-a4-1.6.6_3
2020-05-21 10:08:00+02	2020-05-21 10:09:08+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-05-21 10:09:09+02	2020-05-21 10:24:23+02	print	fontforge	\N	fontforge-20190801_2
2020-02-29 19:01:51+01	2020-02-29 19:02:17+01	print	freetype	\N	freetype-1.3.1_5
2020-02-28 22:11:44+01	2020-02-28 22:12:38+01	print	freetype2	\N	freetype2-2.10.1
2020-05-21 00:09:12+02	2020-05-21 00:13:27+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_4
2020-05-21 01:35:34+02	2020-05-21 01:35:58+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-05-24 16:31:31+02	2020-05-24 16:32:20+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-02-28 23:00:25+01	2020-02-28 23:00:35+01	print	gsfonts	\N	gsfonts-8.11_8
2020-05-21 16:54:33+02	2020-05-21 16:57:28+02	print	gutenprint	\N	gutenprint-5.3.3
2020-05-21 10:09:41+02	2020-05-21 10:10:32+02	print	gv	\N	gv-3.7.4_5
2020-05-21 00:51:00+02	2020-05-21 00:57:44+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.6_2
2020-05-25 22:47:57+02	2020-05-25 22:48:49+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-05-24 19:16:59+02	2020-05-24 19:17:07+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-02-28 21:56:29+01	2020-02-28 21:56:34+01	print	indexinfo	\N	indexinfo-0.3.1
2020-04-25 14:46:54+02	2020-04-25 14:48:00+02	print	libmspub01	\N	libmspub01-0.1.4_16
2020-04-17 05:18:21+02	2020-04-17 05:19:11+02	print	libotf	\N	libotf-0.9.16
2020-02-29 10:17:25+01	2020-02-29 10:18:02+01	print	libpagemaker	\N	libpagemaker-0.0.4_10
2020-02-28 22:40:07+01	2020-02-28 22:40:17+01	print	libpaper	\N	libpaper-1.1.24.4
2020-05-21 01:36:54+02	2020-05-21 01:37:08+02	print	libraqm	\N	libraqm-0.6.0
2020-05-20 09:33:51+02	2020-05-20 09:34:32+02	print	libspectre	\N	libspectre-0.2.8
2020-05-21 10:10:34+02	2020-05-21 10:32:14+02	print	lyx	\N	lyx-2.3.4.2_1
2020-02-28 22:56:58+01	2020-02-28 22:57:17+01	print	psutils	\N	psutils-1.17_5
2020-03-20 08:19:16+01	2020-03-20 08:20:56+01	print	py-qt5-printsupport	py27	py27-qt5-printsupport-5.13.1
2020-05-20 05:45:10+02	2020-05-20 05:46:22+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-04-14 18:54:11+02	2020-04-14 19:05:04+02	print	qpdf	\N	qpdf-10.0.1
2020-05-21 01:25:12+02	2020-05-21 01:27:19+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-05-24 16:42:58+02	2020-05-24 16:44:00+02	print	scribus-devel	\N	scribus-devel-1.5.5_13
2020-05-21 17:21:45+02	2020-05-21 17:26:54+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-05-21 01:09:27+02	2020-05-21 01:11:49+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-05-21 16:55:09+02	2020-05-21 17:00:25+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-05-21 16:42:44+02	2020-05-21 16:50:34+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-05-21 16:42:28+02	2020-05-21 16:46:09+02	print	tex-formats	\N	tex-formats-20150521_2
2020-05-21 17:21:58+02	2020-05-21 17:26:51+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-05-21 16:46:10+02	2020-05-21 16:54:19+02	print	tex-luatex	\N	tex-luatex-0.80.0_17
2020-02-28 23:52:25+01	2020-02-28 23:52:47+01	print	tex-ptexenc	\N	tex-ptexenc-1.3.3_2
2020-05-21 17:23:18+02	2020-05-21 17:24:53+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-05-21 17:24:54+02	2020-05-21 17:30:21+02	print	tex-xetex	\N	tex-xetex-0.99992_33
2020-05-21 16:57:28+02	2020-05-21 17:02:36+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-05-18 22:05:20+02	2020-05-18 22:06:09+02	print	texinfo	\N	texinfo-6.7_3,1
2020-05-21 16:21:35+02	2020-05-21 16:24:34+02	print	texlive-base	\N	texlive-base-20150521_51
2020-05-21 17:26:51+02	2020-05-21 17:53:37+02	print	texlive-docs	\N	texlive-docs-20150523
2020-05-21 18:52:05+02	2020-05-21 18:56:49+02	print	texlive-full	\N	texlive-full-20150521
2020-05-21 16:24:34+02	2020-05-21 16:42:27+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-03-21 00:12:57+01	2020-03-21 00:13:30+01	print	texlive-tlmgr	\N	texlive-tlmgr-20150523_2
2020-05-20 23:26:11+02	2020-05-20 23:26:54+02	print	transfig	\N	transfig-3.2.5e_4
2020-04-17 03:24:31+02	2020-04-17 03:25:12+02	print	xpdfopen	\N	xpdfopen-0.86
2020-03-31 19:11:40+02	2020-03-31 19:30:50+02	science	hdf5	\N	hdf5-1.10.6,1
2020-05-16 18:50:32+02	2020-05-16 18:51:05+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-05-16 17:51:12+02	2020-05-16 17:57:28+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-02-29 09:27:25+01	2020-02-29 09:27:49+01	science	szip	\N	hdf-szip-2.1.1
2020-05-21 10:32:16+02	2020-05-21 10:33:04+02	security	aide	\N	aide-0.16.1
2020-04-30 10:01:02+02	2020-04-30 10:01:08+02	security	base-audit	\N	base-audit-0.4
2020-05-02 12:56:30+02	2020-05-02 12:56:46+02	security	ca_root_nss	\N	ca_root_nss-3.52
2020-05-23 09:55:35+02	2020-05-23 09:56:41+02	security	clamav	\N	clamav-0.102.3_1,1
2020-03-02 07:12:05+01	2020-03-02 07:12:58+01	security	cyrus-sasl2	\N	cyrus-sasl-2.1.27_1
2020-04-24 16:37:07+02	2020-04-24 16:37:22+02	security	doas	\N	doas-6.2p4_1
2020-03-07 03:24:05+01	2020-03-07 03:24:23+01	security	fpc-openssl	\N	fpc-openssl-3.0.4_4
2020-05-20 02:31:47+02	2020-05-20 02:35:11+02	security	gcr	\N	gcr-3.28.0
2020-05-20 05:46:23+02	2020-05-20 05:55:24+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-05-21 16:17:31+02	2020-05-21 16:19:20+02	security	gnupg	\N	gnupg-2.2.20
2020-05-16 15:21:21+02	2020-05-16 15:25:44+02	security	gnutls	\N	gnutls-3.6.13_1
2020-05-21 16:20:10+02	2020-05-21 16:20:58+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-05-21 16:24:11+02	2020-05-21 16:24:39+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-05-21 16:19:21+02	2020-05-21 16:19:56+02	security	gpgme	\N	gpgme-1.13.1
2020-03-01 12:44:08+01	2020-03-01 12:44:46+01	security	ipv6toolkit	\N	ipv6toolkit-2.0
2020-05-21 10:32:19+02	2020-05-21 10:32:41+02	security	kc24	\N	kc24-2.4.2_1
2020-05-16 21:26:30+02	2020-05-16 21:27:31+02	security	klee	\N	klee-2.1
2020-02-28 23:37:11+01	2020-02-28 23:37:26+01	security	libargon2	\N	libargon2-20190702
2020-03-01 00:36:28+01	2020-03-01 00:36:41+01	security	libassuan	\N	libassuan-2.5.3
2020-02-28 22:01:34+01	2020-02-28 22:02:35+01	security	libgcrypt	\N	libgcrypt-1.8.5
2020-05-20 04:42:28+02	2020-05-20 04:43:57+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-02-28 21:59:16+01	2020-02-28 21:59:34+01	security	libgpg-error	\N	libgpg-error-1.37
2020-05-21 16:17:03+02	2020-05-21 16:17:31+02	security	libksba	\N	libksba-1.4.0
2020-03-01 00:36:41+01	2020-03-01 00:37:09+01	security	libmcrypt	\N	libmcrypt-2.5.8_3
2020-04-24 12:34:18+02	2020-04-24 12:35:09+02	security	libntlm	\N	libntlm-1.6
2020-03-01 03:29:47+01	2020-03-01 03:29:55+01	security	libscrypt	\N	libscrypt-1.21
2020-05-21 01:34:37+02	2020-05-21 01:35:34+02	security	libsecret	\N	libsecret-0.18.6_1
2020-03-01 00:31:04+01	2020-03-01 00:33:23+01	security	libsodium	\N	libsodium-1.0.18
2020-04-19 17:55:47+02	2020-04-19 17:56:41+02	security	libssh	\N	libssh-0.9.4
2020-02-29 01:13:40+01	2020-02-29 01:14:14+01	security	libssh2	\N	libssh2-1.8.2,3
2020-04-17 03:00:23+02	2020-04-17 03:01:22+02	security	libtasn1	\N	libtasn1-4.16.0
2020-04-30 09:37:20+02	2020-04-30 09:37:32+02	security	lynis	\N	lynis-2.7.5
2020-04-17 06:02:24+02	2020-04-17 06:03:19+02	security	mbedtls	\N	mbedtls-2.16.6
2020-03-01 03:29:55+01	2020-03-01 03:30:21+01	security	mhash	\N	mhash-0.9.9.9_5
2020-05-16 15:20:17+02	2020-05-16 15:21:21+02	security	nettle	\N	nettle-3.6
2020-05-20 23:26:55+02	2020-05-20 23:30:08+02	security	nmap	\N	nmap-7.80
2020-05-21 16:15:21+02	2020-05-21 16:21:00+02	security	nss	\N	nss-3.52.1
2020-04-27 17:21:09+02	2020-04-27 17:22:11+02	security	nyx	\N	nyx-2.1.0_1
2020-05-21 10:32:41+02	2020-05-21 10:34:02+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_3
2020-04-20 07:47:25+02	2020-04-20 07:48:27+02	security	openconnect	\N	openconnect-8.08
2020-05-02 12:59:12+02	2020-05-02 13:00:15+02	security	p11-kit	\N	p11-kit-0.23.20
2020-03-20 08:31:10+01	2020-03-20 08:31:26+01	security	p5-Authen-NTLM	\N	p5-Authen-NTLM-1.09_1
2020-03-20 04:51:11+01	2020-03-20 04:51:34+01	security	p5-Authen-SASL	\N	p5-Authen-SASL-2.16_1
2020-03-20 02:14:05+01	2020-03-20 02:14:31+01	security	p5-Digest-HMAC	\N	p5-Digest-HMAC-1.03_1
2020-03-20 11:40:15+01	2020-03-20 11:40:32+01	security	p5-Digest-SHA1	\N	p5-Digest-SHA1-2.13_1
2020-03-20 04:00:49+01	2020-03-20 04:01:17+01	security	p5-GSSAPI	\N	p5-GSSAPI-0.28_1
2020-04-24 00:37:51+02	2020-04-24 00:38:23+02	security	p5-IO-Socket-SSL	\N	p5-IO-Socket-SSL-2.068
2020-03-20 02:14:31+01	2020-03-20 02:15:25+01	security	p5-Net-SSLeay	\N	p5-Net-SSLeay-1.88
2020-02-29 19:13:09+01	2020-02-29 19:13:17+01	security	pam_helper	\N	pam_helper-1.1
2020-05-16 11:00:25+02	2020-05-16 11:00:40+02	security	pecl-mcrypt	php73	php73-pecl-mcrypt-1.0.3
2020-05-16 10:57:43+02	2020-05-16 10:58:14+02	security	php73-hash	\N	php73-hash-7.3.18
2020-05-16 10:57:53+02	2020-05-16 10:58:18+02	security	php73-openssl	\N	php73-openssl-7.3.18
2020-05-20 04:43:58+02	2020-05-20 04:46:27+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-05-20 05:59:46+02	2020-05-20 06:01:41+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-02-29 02:08:48+01	2020-02-29 02:09:00+01	security	pinentry-tty	\N	pinentry-tty-1.1.0
2020-02-29 02:23:47+01	2020-02-29 02:23:54+01	security	pinentry	\N	pinentry-1.1.0_6
2020-03-01 03:33:49+01	2020-03-01 03:33:56+01	security	pkesh	\N	pkesh-1.0
2020-04-27 23:35:09+02	2020-04-27 23:35:34+02	security	py-bcrypt	py27	py27-bcrypt-3.1.7
2020-05-25 08:11:00+02	2020-05-25 08:12:44+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-04-27 05:21:46+02	2020-04-27 05:22:42+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7
2020-03-02 08:33:47+01	2020-03-02 08:33:59+01	security	py-certifi	py27	py27-certifi-2019.11.28
2020-04-26 15:41:26+02	2020-04-26 15:41:52+02	security	py-certifi	py37	py37-certifi-2020.4.5.1
2020-05-18 22:04:50+02	2020-05-18 22:05:29+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-05-18 22:04:52+02	2020-05-18 22:05:30+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-03-04 21:50:35+01	2020-03-04 21:50:44+01	security	py-openssl	py27	py27-openssl-19.0.0
2020-05-18 22:05:31+02	2020-05-18 22:05:41+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-05-18 22:09:15+02	2020-05-18 22:09:29+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-05-18 22:08:59+02	2020-05-18 22:09:15+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-04-27 05:25:18+02	2020-04-27 05:27:54+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-05-21 18:54:00+02	2020-05-21 18:55:37+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.0
2020-05-18 22:07:24+02	2020-05-18 22:07:44+02	security	py-pynacl	py27	py27-pynacl-1.3.0
2020-05-18 22:07:03+02	2020-05-18 22:07:23+02	security	py-pynacl	py37	py37-pynacl-1.3.0
2020-05-21 16:33:33+02	2020-05-21 16:33:44+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-04-27 09:16:13+02	2020-04-27 09:16:43+02	security	py-stem	py37	py37-stem-1.8.0
2020-04-14 18:51:58+02	2020-04-14 18:52:29+02	security	rhash	\N	rhash-1.3.9
2020-05-21 10:33:05+02	2020-05-21 10:33:26+02	security	rkhunter	\N	rkhunter-1.4.6
2020-03-20 15:43:45+01	2020-03-20 15:43:59+01	security	ssl-admin	\N	ssl-admin-1.2.1
2020-03-20 15:43:59+01	2020-03-20 15:44:54+01	security	sudo	\N	sudo-1.8.31p1
2020-05-19 07:41:31+02	2020-05-19 07:42:25+02	security	tor	\N	tor-0.4.3.5
2020-03-08 12:09:27+01	2020-03-08 12:10:30+01	security	trousers	\N	trousers-0.3.14_2
2020-05-21 16:57:36+02	2020-05-21 16:58:20+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-04-26 01:31:21+02	2020-04-26 01:34:03+02	shells	bash	\N	bash-5.0.17
2020-03-01 12:56:59+01	2020-03-01 13:01:03+01	shells	zsh	\N	zsh-5.8
2020-04-26 15:05:58+02	2020-04-26 15:06:05+02	sysutils	915resolution	\N	915resolution-0.5.3_1,1
2020-05-20 07:22:50+02	2020-05-20 07:23:46+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-05-19 04:22:10+02	2020-05-19 04:24:48+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-03-03 19:01:06+01	2020-03-03 19:01:51+01	sysutils	apachetop	\N	apachetop-0.18.4_1
2020-03-01 13:05:08+01	2020-03-01 13:05:21+01	sysutils	atop	\N	atop-2.0.2.b3_3
2020-03-01 13:05:21+01	2020-03-01 13:05:29+01	sysutils	beadm	\N	beadm-1.3.0
2020-03-01 13:05:29+01	2020-03-01 13:05:37+01	sysutils	bsdcrashtar	\N	bsdcrashtar-0.1.3_1
2020-03-01 13:05:37+01	2020-03-01 13:05:48+01	sysutils	bsdhwmon	\N	bsdhwmon-20151206
2020-03-26 09:45:34+01	2020-03-26 09:45:42+01	sysutils	bsdinfo	\N	bsdinfo-0.22_2
2020-05-01 17:23:37+02	2020-05-01 17:23:49+02	sysutils	bsdstats	\N	bsdstats-7.0
2020-03-01 13:06:10+01	2020-03-01 13:10:34+01	sysutils	cdrtools	\N	cdrtools-3.01_1
2020-03-01 13:10:29+01	2020-03-01 13:10:37+01	sysutils	cmdwatch	\N	cmdwatch-0.2.0_2
2020-02-29 07:17:34+01	2020-02-29 07:17:53+01	sysutils	cmocka	\N	cmocka-1.1.5
2020-05-21 01:35:59+02	2020-05-21 01:36:27+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-03-01 13:10:34+01	2020-03-01 13:13:53+01	sysutils	coreutils	\N	coreutils-8.31
2020-05-21 10:33:26+02	2020-05-21 10:34:52+02	sysutils	cpu-x	\N	cpu-x-4.0.0
2020-03-01 13:11:40+01	2020-03-01 13:11:49+01	sysutils	cpupdate	\N	cpupdate-g20180513_1
2020-05-21 18:55:37+02	2020-05-21 18:55:55+02	sysutils	ddrescue	\N	ddrescue-1.25
2020-05-21 18:55:55+02	2020-05-21 18:56:05+02	sysutils	devcpu-data	\N	devcpu-data-1.30
2020-03-01 13:12:26+01	2020-03-01 13:12:46+01	sysutils	dfc	\N	dfc-3.1.1
2020-02-29 01:53:05+01	2020-02-29 01:53:15+01	sysutils	dmidecode	\N	dmidecode-3.2
2020-05-16 21:27:46+02	2020-05-16 21:28:05+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-04-27 17:25:32+02	2020-04-27 17:27:21+02	sysutils	DTraceToolkit	\N	dtrace-toolkit-1.0_5
2020-05-21 10:34:03+02	2020-05-21 10:37:39+02	sysutils	exa	\N	exa-0.9.0_10
2020-02-29 09:34:35+01	2020-02-29 09:35:21+01	sysutils	flock	\N	flock-2.30.1
2020-02-29 09:35:22+01	2020-02-29 09:35:54+01	sysutils	fusefs-libs	\N	fusefs-libs-2.9.9_1
2020-03-24 13:13:32+01	2020-03-24 13:14:02+01	sysutils	fusefs-ntfs	\N	fusefs-ntfs-2017.3.23
2020-05-21 10:34:53+02	2020-05-21 10:36:43+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-02-28 21:56:26+01	2020-02-28 21:56:32+01	sysutils	gnome_subr	\N	gnome_subr-1.0
2020-05-20 07:23:46+02	2020-05-20 07:24:52+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-05-20 09:42:17+02	2020-05-20 09:42:37+02	sysutils	hal-info	\N	hal-info-20091130
2020-05-20 02:32:54+02	2020-05-20 02:34:42+02	sysutils	hal	\N	hal-0.5.14_34
2020-04-25 21:00:13+02	2020-04-25 21:05:32+02	sysutils	hexyl	\N	hexyl-0.6.0_6
2020-05-21 18:56:05+02	2020-05-21 18:56:18+02	sysutils	hourglass	\N	hourglass-1.0.1
2020-04-30 09:37:55+02	2020-04-30 09:38:35+02	sysutils	htop	\N	htop-2.2.0_1
2020-04-17 12:19:00+02	2020-04-17 12:19:49+02	sysutils	intel-pcm	\N	intel-pcm-201902_1
2020-03-01 13:14:31+01	2020-03-01 13:14:40+01	sysutils	ioping	\N	ioping-1.2
2020-05-21 16:33:44+02	2020-05-21 16:34:50+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-02-29 10:27:03+01	2020-02-29 10:27:26+01	sysutils	libcdio-paranoia	\N	libcdio-paranoia-10.2+2.0.1
2020-02-29 09:35:55+01	2020-02-29 09:36:50+01	sysutils	libcdio	\N	libcdio-2.1.0
2020-03-01 03:34:34+01	2020-03-01 03:35:04+01	sysutils	libcpuid	\N	libcpuid-0.4.1
2020-02-29 07:17:59+01	2020-02-29 07:18:06+01	sysutils	libsunacl	\N	libsunacl-1.0.1
2020-04-30 09:31:37+02	2020-04-30 09:31:52+02	sysutils	lsof	\N	lsof-4.93.2_11,8
2020-03-01 13:14:40+01	2020-03-01 13:14:50+01	sysutils	lsop	\N	lsop-0.1
2020-05-21 16:58:21+02	2020-05-21 17:00:03+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-05-20 09:45:52+02	2020-05-20 09:46:56+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-05-20 23:30:09+02	2020-05-20 23:32:20+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-05-21 16:34:13+02	2020-05-21 16:35:19+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-05-20 23:32:20+02	2020-05-20 23:35:02+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-03-01 13:26:36+01	2020-03-01 13:26:49+01	sysutils	memtest86+	\N	memtest86+-5.01_3
2020-05-21 10:36:44+02	2020-05-21 10:37:12+02	sysutils	most	\N	most-5.1.0
2020-03-01 13:14:50+01	2020-03-01 13:14:59+01	sysutils	ncdu	\N	ncdu-1.14.2
2020-05-21 18:56:18+02	2020-05-21 18:56:26+02	sysutils	neofetch	\N	neofetch-7.0.0
2020-03-01 13:14:59+01	2020-03-01 13:15:05+01	sysutils	obliterate	\N	obliterate-0.5
2020-05-23 21:42:47+02	2020-05-23 21:43:04+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.04_1
2020-05-19 04:24:50+02	2020-05-19 04:25:04+02	sysutils	panicmail	\N	panicmail-1.1
2020-03-20 15:52:38+01	2020-03-20 15:53:14+01	sysutils	parallel	\N	parallel-20200122
2020-02-29 19:16:14+01	2020-02-29 19:16:59+01	sysutils	patchelf	\N	patchelf-0.10_1
2020-05-16 11:00:32+02	2020-05-16 11:00:50+02	sysutils	php73-posix	\N	php73-posix-7.3.18
2020-05-20 06:01:41+02	2020-05-20 06:02:49+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-05-20 02:09:37+02	2020-05-20 02:11:51+02	sysutils	policykit	\N	policykit-0.9_10
2020-05-20 02:34:43+02	2020-05-20 02:36:00+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-05-21 00:15:22+02	2020-05-21 00:16:59+02	sysutils	polkit	\N	polkit-0.116
2020-03-01 13:15:17+01	2020-03-01 13:15:28+01	sysutils	psmisc	\N	psmisc-22.16_1
2020-03-01 13:15:24+01	2020-03-01 13:15:32+01	sysutils	pv	\N	pv-1.6.6
2020-03-01 13:15:29+01	2020-03-01 13:15:36+01	sysutils	pwgen	\N	pwgen-2.08,2
2020-04-27 17:34:38+02	2020-04-27 17:35:24+02	sysutils	py-cpuinfo	py37	py37-py-cpuinfo-5.0.0
2020-02-25 17:16:14+01	2020-02-25 17:16:24+01	sysutils	py-execnet	py37	py37-execnet-1.6.0
2020-02-25 17:16:14+01	2020-02-25 17:16:24+01	sysutils	py-filelock	py37	py37-filelock-3.0.12
2020-04-27 17:34:54+02	2020-04-27 17:35:55+02	sysutils	py-glances	py37	py37-glances-3.1.3
2020-05-25 08:11:00+02	2020-05-25 08:12:47+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-02-25 17:14:59+01	2020-02-25 17:15:10+01	sysutils	py-pkginfo	py37	py37-pkginfo-1.5.0.1
2020-04-27 09:23:00+02	2020-04-27 09:23:33+02	sysutils	py-psutil	py37	py37-psutil-5.7.0
2020-02-25 17:16:24+01	2020-02-25 17:16:33+01	sysutils	py-ptyprocess	py37	py37-ptyprocess-0.6.0
2020-05-19 04:25:05+02	2020-05-19 04:25:14+02	sysutils	py-py-cpuinfo	py37	py37-py-cpuinfo-5.0.0
2020-03-01 13:15:41+01	2020-03-01 13:15:47+01	sysutils	reptyr	\N	reptyr-0.6.3a_2
2020-04-25 21:05:32+02	2020-04-25 21:19:40+02	sysutils	rsfetch	\N	rsfetch-2.0.0_2
2020-03-01 13:15:48+01	2020-03-01 13:16:19+01	sysutils	screen	\N	screen-4.8.0
2020-05-08 02:50:12+02	2020-05-08 02:50:45+02	sysutils	smartmontools	\N	smartmontools-7.1_1
2020-03-01 13:15:53+01	2020-03-01 13:16:01+01	sysutils	spinner	\N	spinner-1.2.4
2020-02-29 19:16:59+01	2020-02-29 19:17:07+01	sysutils	sysctlbyname-improved-kmod	\N	sysctlbyname-improved-kmod-20191124
2020-02-29 19:17:07+01	2020-02-29 19:17:16+01	sysutils	sysctlinfo-kmod	\N	sysctlinfo-kmod-20191005
2020-05-24 11:17:29+02	2020-05-24 11:18:12+02	sysutils	tmux	\N	tmux-3.1b
2020-03-01 13:16:20+01	2020-03-01 13:16:32+01	sysutils	tree	\N	tree-1.8.0
2020-05-20 04:31:19+02	2020-05-20 04:32:10+02	sysutils	upower	\N	upower-0.99.11
2020-02-29 19:17:16+01	2020-02-29 19:17:23+01	sysutils	vpnc-scripts	\N	vpnc-scripts-20190611
2020-05-08 02:50:45+02	2020-05-08 02:51:03+02	sysutils	x86info	\N	x86info-1.31.s03
2020-03-20 15:53:45+01	2020-03-20 15:54:07+01	sysutils	zfs-stats	\N	zfs-stats-1.3.0
2020-03-01 13:16:36+01	2020-03-01 13:17:19+01	sysutils	zrepl	\N	zrepl-0.2.1
2020-04-26 15:44:36+02	2020-04-26 15:45:52+02	textproc	asciidoc	\N	asciidoc-9.0.0rc2
2020-03-20 01:57:46+01	2020-03-20 02:00:29+01	textproc	aspell	\N	aspell-0.60.8,1
2020-05-21 10:37:14+02	2020-05-21 10:52:32+02	textproc	bat	\N	bat-0.15.0
2020-02-29 19:17:23+01	2020-02-29 19:20:08+01	textproc	clucene	\N	clucene-2.3.3.4_19
2020-04-19 19:30:10+02	2020-04-19 19:30:38+02	textproc	colordiff	\N	colordiff-1.0.19
2020-03-01 03:39:31+01	2020-03-01 03:40:11+01	textproc	diffutils	\N	diffutils-3.7
2020-02-28 22:16:22+01	2020-02-28 22:16:30+01	textproc	docbook-sgml	\N	docbook-sgml-4.5_1
2020-02-28 22:16:31+01	2020-02-28 22:16:39+01	textproc	docbook-xml	\N	docbook-xml-5.0_3
2020-02-28 22:17:02+01	2020-02-28 22:17:37+01	textproc	docbook-xsl	\N	docbook-xsl-1.79.1_1,1
2020-02-28 22:16:53+01	2020-02-28 22:17:00+01	textproc	docbook	\N	docbook-1.5
2020-05-20 09:46:13+02	2020-05-20 09:46:52+02	textproc	enchant	\N	enchant-1.6.0_9
2020-05-21 01:40:08+02	2020-05-21 01:41:20+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-02-29 06:06:23+01	2020-02-29 06:08:53+01	textproc	exempi	\N	exempi-2.2.2_1
2020-02-28 21:58:51+01	2020-02-28 21:59:08+01	textproc	expat2	\N	expat-2.2.8
2020-03-23 06:59:16+01	2020-03-23 06:59:43+01	textproc	flex	\N	flex-2.6.4_2
2020-05-10 12:07:11+02	2020-05-10 12:08:43+02	textproc	fop	\N	fop-2.3
2020-04-26 01:23:56+02	2020-04-26 01:31:20+02	textproc	groff	\N	groff-1.22.4_3
2020-02-29 10:28:58+01	2020-02-29 10:29:39+01	textproc	gsed	\N	gsed-4.8
2020-04-26 15:45:23+02	2020-04-26 15:46:34+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-05-14 11:14:05+02	2020-05-14 11:14:43+02	textproc	gtkspell	\N	gtkspell-2.0.16_6
2020-05-20 23:35:03+02	2020-05-20 23:36:19+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-04-09 11:06:07+02	2020-04-09 11:09:53+02	textproc	highlight	\N	highlight-3.43_8,3
2020-05-22 02:21:15+02	2020-05-22 03:00:45+02	textproc	hs-pandoc	\N	hs-pandoc-2.9.2_1
2020-02-28 22:25:25+01	2020-02-28 22:25:51+01	textproc	html2text	\N	html2text-1.3.2a
2020-03-21 00:07:29+01	2020-03-21 00:09:23+01	textproc	hunspell	\N	hunspell-1.7.0_2
2020-03-21 00:25:57+01	2020-03-21 00:26:27+01	textproc	hyphen	\N	hyphen-2.8.8
2020-03-19 23:07:44+01	2020-03-19 23:07:58+01	textproc	intltool	\N	intltool-0.51.0_1
2020-02-28 22:16:08+01	2020-02-28 22:16:15+01	textproc	iso8879	\N	iso8879-1986_3
2020-04-26 15:44:52+02	2020-04-26 15:45:19+02	textproc	itstool	\N	itstool-2.0.6
2020-05-20 02:35:11+02	2020-05-20 02:36:23+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.70.0
2020-05-20 04:31:20+02	2020-05-20 04:33:05+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-02-29 19:17:25+01	2020-02-29 19:18:56+01	textproc	libabw	\N	libabw-0.1.3
2020-05-19 23:56:12+02	2020-05-19 23:57:08+02	textproc	libcroco	\N	libcroco-0.6.13
2020-04-12 10:27:57+02	2020-04-12 10:28:04+02	textproc	libdiff	\N	libdiff-0.1.0
2020-04-25 14:49:25+02	2020-04-25 14:56:36+02	textproc	libe-book	\N	libe-book-0.1.3_18
2020-02-29 19:20:09+01	2020-02-29 19:20:36+01	textproc	libebml	\N	libebml-1.3.10
2020-02-29 19:20:37+01	2020-02-29 19:21:57+01	textproc	libepubgen	\N	libepubgen-0.1.0_9
2020-02-29 19:21:58+01	2020-02-29 19:22:08+01	textproc	libexttextcat	\N	libexttextcat-3.4.5
2020-02-29 19:22:09+01	2020-02-29 19:31:23+01	textproc	libmwaw03	\N	libmwaw03-0.3.15
2020-02-29 19:23:46+01	2020-02-29 19:24:36+01	textproc	libnumbertext	\N	libnumbertext-1.0.5_1
2020-02-29 19:24:39+01	2020-02-29 19:26:16+01	textproc	libodfgen01	\N	libodfgen01-0.1.7_3
2020-04-25 14:50:39+02	2020-04-25 14:52:52+02	textproc	libqxp	\N	libqxp-0.0.0_16
2020-03-01 00:24:48+01	2020-03-01 00:26:09+01	textproc	librevenge	\N	librevenge-0.0.4_13
2020-02-29 19:27:42+01	2020-02-29 19:31:05+01	textproc	libstaroffice	\N	libstaroffice-0.0.6_8
2020-03-01 03:42:27+01	2020-03-01 03:43:07+01	textproc	libuninameslist	\N	libuninameslist-20190701
2020-04-25 08:15:40+02	2020-04-25 08:19:29+02	textproc	libvisio01	\N	libvisio01-0.1.7_2
2020-05-20 06:02:50+02	2020-05-20 06:06:24+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-02-29 19:31:06+01	2020-02-29 19:33:39+01	textproc	libwps	\N	libwps-0.4.10_2
2020-05-20 03:04:33+02	2020-05-20 03:08:47+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-02-28 21:57:46+01	2020-02-28 21:58:51+01	textproc	libxml2	\N	libxml2-2.9.10
2020-02-28 22:02:36+01	2020-02-28 22:02:58+01	textproc	libxslt	\N	libxslt-1.1.34_1
2020-04-24 00:37:51+02	2020-04-24 00:38:22+02	textproc	libyaml	\N	libyaml-0.2.4
2020-04-12 10:28:05+02	2020-04-12 10:28:23+02	textproc	lowdown	\N	lowdown-0.6.4
2020-02-28 22:25:51+01	2020-02-28 22:26:01+01	textproc	minixmlto	\N	minixmlto-0.0.2_1
2020-03-20 05:42:26+01	2020-03-20 05:42:51+01	textproc	mythes	\N	mythes-1.2.4_7
2020-03-20 08:37:13+01	2020-03-20 08:37:49+01	textproc	p5-PDF-API2	\N	p5-PDF-API2-2.037
2020-03-20 08:37:24+01	2020-03-20 08:37:41+01	textproc	p5-Pod-Escapes	\N	p5-Pod-Escapes-1.07
2020-03-20 11:47:00+01	2020-03-20 11:47:19+01	textproc	p5-pod2pdf	\N	p5-pod2pdf-0.42_1
2020-03-19 20:16:42+01	2020-03-19 20:16:53+01	textproc	p5-Unicode-EastAsianWidth	\N	p5-Unicode-EastAsianWidth-12.0
2020-03-21 00:51:40+01	2020-03-21 00:51:58+01	textproc	p5-XML-NamespaceSupport	\N	p5-XML-NamespaceSupport-1.12
2020-03-19 23:07:29+01	2020-03-19 23:07:44+01	textproc	p5-XML-Parser	\N	p5-XML-Parser-2.44
2020-03-21 00:51:58+01	2020-03-21 00:52:15+01	textproc	p5-XML-SAX-Base	\N	p5-XML-SAX-Base-1.09
2020-03-20 01:25:02+01	2020-03-20 01:25:18+01	textproc	p5-XML-SAX-Expat	\N	p5-XML-SAX-Expat-0.51_1
2020-03-20 01:02:59+01	2020-03-20 01:03:21+01	textproc	p5-XML-SAX	\N	p5-XML-SAX-1.02
2020-03-20 01:48:56+01	2020-03-20 01:49:18+01	textproc	p5-XML-Simple	\N	p5-XML-Simple-2.25
2020-05-16 11:00:41+02	2020-05-16 11:00:58+02	textproc	php73-ctype	\N	php73-ctype-7.3.18
2020-05-16 11:00:50+02	2020-05-16 11:01:13+02	textproc	php73-dom	\N	php73-dom-7.3.18
2020-05-16 11:00:58+02	2020-05-16 11:01:19+02	textproc	php73-simplexml	\N	php73-simplexml-7.3.18
2020-05-16 10:58:00+02	2020-05-16 10:58:20+02	textproc	php73-xml	\N	php73-xml-7.3.18
2020-03-01 03:43:03+01	2020-03-01 03:43:29+01	textproc	pugixml	\N	pugixml-1.10
2020-03-02 09:17:39+01	2020-03-02 09:17:50+01	textproc	py-alabaster	py27	py27-alabaster-0.7.6
2020-04-26 15:42:40+02	2020-04-26 15:43:05+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-03-02 08:42:52+01	2020-03-02 08:43:03+01	textproc	py-chardet	py27	py27-chardet-3.0.4_3
2020-04-26 15:41:53+02	2020-04-26 15:42:23+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-04-26 15:44:08+02	2020-04-26 15:44:35+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-03-02 09:17:50+01	2020-03-02 09:18:08+01	textproc	py-docutils	py27	py27-docutils-0.15.2
2020-04-26 15:42:42+02	2020-04-26 15:46:43+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-02-29 23:59:14+01	2020-03-01 00:00:14+01	textproc	teckit	\N	teckit-2.5.7
2020-05-21 16:34:50+02	2020-05-21 16:35:01+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-05-21 18:56:26+02	2020-05-21 18:56:52+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-04-26 15:43:36+02	2020-04-26 15:44:51+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-05-21 16:24:41+02	2020-05-21 16:24:55+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-05-14 07:52:46+02	2020-05-14 07:52:59+02	textproc	py-mako	py27	py27-mako-1.0.14
2020-03-03 08:28:04+01	2020-03-03 08:28:19+01	textproc	py-markdown	py27	py27-markdown-2.6.11_1
2020-04-27 09:23:42+02	2020-04-27 09:24:02+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-04-27 23:31:26+02	2020-04-27 23:31:40+02	textproc	py-MarkupSafe	py27	py27-MarkupSafe-1.1.1
2020-04-26 15:41:53+02	2020-04-26 15:42:19+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-05-21 16:20:58+02	2020-05-21 16:21:12+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-05-21 17:00:03+02	2020-05-21 17:00:15+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-03-02 09:18:09+01	2020-03-02 09:18:30+01	textproc	py-pygments	py27	py27-pygments-2.5.2
2020-04-26 15:42:48+02	2020-04-26 15:43:35+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-05-21 16:35:01+02	2020-05-21 16:35:17+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-02-25 17:16:25+01	2020-02-25 17:16:35+01	textproc	py-pypa-docs-theme	py37	py37-pypa-docs-theme-g20180622
2020-03-02 08:43:04+01	2020-03-02 08:43:32+01	textproc	py-pystemmer	py27	py27-pystemmer-1.3.0_2
2020-04-26 15:45:01+02	2020-04-26 15:45:51+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-02-25 17:14:59+01	2020-02-25 17:15:07+01	textproc	py-python-docs-theme	py37	py37-python-docs-theme-2018.2
2020-03-20 11:47:20+01	2020-03-20 11:48:27+01	textproc	py-qt5-xmlpatterns	py27	py27-qt5-xmlpatterns-5.13.1
2020-05-20 23:36:19+02	2020-05-20 23:37:25+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-05-23 08:47:55+02	2020-05-23 08:48:11+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-03-03 08:29:52+01	2020-03-03 08:30:31+01	textproc	py-regex	py27	py27-regex-2018.02.21
2020-05-14 11:14:54+02	2020-05-14 11:15:22+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-04-27 23:40:52+02	2020-04-27 23:41:16+02	textproc	py-scour	py27	py27-scour-0.37
2020-05-15 21:33:20+02	2020-05-15 21:33:34+02	textproc	py-scour	py37	py37-scour-0.37
2020-03-02 09:18:31+01	2020-03-02 09:18:48+01	textproc	py-snowballstemmer	py27	py27-snowballstemmer-1.2.1
2020-04-26 15:45:51+02	2020-04-26 15:46:15+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-03-02 09:18:49+01	2020-03-02 09:19:04+01	textproc	py-sphinx_rtd_theme	py27	py27-sphinx_rtd_theme-0.4.3
2020-04-26 15:42:58+02	2020-04-26 15:43:34+02	textproc	py-sphinx_rtd_theme	py37	py37-sphinx_rtd_theme-0.4.3
2020-03-08 02:32:16+01	2020-03-08 02:32:41+01	textproc	py-sphinx	py27	py27-sphinx-1.6.5_2,1
2020-05-23 08:47:36+02	2020-05-23 08:47:55+02	textproc	py-sphinx	py37	py37-sphinx-3.0.3,1
2020-05-18 22:06:09+02	2020-05-18 22:06:39+02	textproc	py-sphinx18	py37	py37-sphinx18-1.8.5_1,1
2020-05-14 07:50:05+02	2020-05-14 07:50:15+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-05-14 07:50:15+02	2020-05-14 07:50:25+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-05-14 07:50:26+02	2020-05-14 07:50:37+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-05-14 07:50:37+02	2020-05-14 07:50:47+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-05-14 07:50:48+02	2020-05-14 07:50:57+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-05-14 07:50:49+02	2020-05-14 07:50:58+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-03-02 09:19:05+01	2020-03-02 09:19:17+01	textproc	py-sphinxcontrib-websupport	py27	py27-sphinxcontrib-websupport-1.1.2
2020-04-26 15:43:05+02	2020-04-26 15:43:34+02	textproc	py-sphinxcontrib-websupport	py37	py37-sphinxcontrib-websupport-1.1.2
2020-05-14 07:50:53+02	2020-05-14 07:51:02+02	textproc	py-sphinxcontrib-websupport11	py37	py37-sphinxcontrib-websupport11-1.1.2
2020-05-21 17:26:55+02	2020-05-21 17:27:06+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-03-02 08:34:17+01	2020-03-02 08:34:28+01	textproc	py-toml	py27	py27-toml-0.10.0
2020-04-26 19:14:38+02	2020-04-26 19:15:27+02	textproc	py-toml	py37	py37-toml-0.10.0
2020-02-25 17:15:07+01	2020-02-25 17:15:18+01	textproc	py-towncrier	py37	py37-towncrier-19.2.0
2020-05-21 17:00:15+02	2020-05-21 17:00:26+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-05-21 17:00:26+02	2020-05-21 17:00:37+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-05-21 01:00:24+02	2020-05-21 01:01:45+02	textproc	qt5-xml	\N	qt5-xml-5.14.2
2020-05-21 00:50:05+02	2020-05-21 00:58:59+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.14.2
2020-05-16 17:17:53+02	2020-05-16 17:20:36+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-05-20 07:26:46+02	2020-05-20 07:27:28+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-05-20 09:46:52+02	2020-05-20 09:49:30+02	textproc	redland	\N	redland-1.0.17_4
2020-05-14 11:15:18+02	2020-05-14 11:19:36+02	textproc	ripgrep	\N	ripgrep-12.1.0
2020-05-16 16:49:06+02	2020-05-16 16:49:19+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-02-28 22:16:47+01	2020-02-28 22:16:53+01	textproc	sdocbook-xml	\N	sdocbook-xml-1.1_2,2
2020-04-25 03:22:57+02	2020-04-25 03:31:37+02	textproc	source-highlight	\N	source-highlight-3.1.9_1
2020-05-25 08:12:44+02	2020-05-25 08:12:52+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-03-19 23:57:23+01	2020-03-19 23:57:59+01	textproc	texi2html	\N	texi2html-5.0_2,1
2020-03-13 04:37:27+01	2020-03-13 04:38:38+01	textproc	UCD	\N	UCD-13.0.0
2020-04-26 01:23:34+02	2020-04-26 01:23:56+02	textproc	uchardet	\N	uchardet-0.0.7
2020-05-24 11:13:59+02	2020-05-24 11:14:07+02	textproc	utf8proc	\N	utf8proc-2.5.0
2020-02-28 22:15:29+01	2020-02-28 22:15:39+01	textproc	xmlcatmgr	\N	xmlcatmgr-2.2_2
2020-02-28 22:16:15+01	2020-02-28 22:16:22+01	textproc	xmlcharent	\N	xmlcharent-0.3_2
2020-04-25 03:46:20+02	2020-04-25 03:47:07+02	textproc	xmlto	\N	xmlto-0.0.28
2020-04-26 21:05:01+02	2020-04-26 21:06:52+02	textproc	yelp-tools	\N	yelp-tools-3.28.0
2020-04-26 15:46:15+02	2020-04-26 15:47:00+02	textproc	yelp-xsl	\N	yelp-xsl-3.28.0
2020-05-20 07:27:28+02	2020-05-20 07:30:08+02	www	apache24	\N	apache24-2.4.43
2020-05-23 09:00:15+02	2020-05-23 09:27:06+02	www	chromium	\N	chromium-81.0.4044.138_1
2020-05-16 11:56:02+02	2020-05-16 11:56:28+02	www	dokuwiki	\N	dokuwiki-20180422c
2020-05-23 09:56:14+02	2020-05-23 10:34:17+02	www	firefox	\N	firefox-76.0.1_3,1
2020-03-07 03:26:26+01	2020-03-07 03:26:42+01	www	fpc-fastcgi	\N	fpc-fastcgi-3.0.4_4
2020-03-07 03:26:42+01	2020-03-07 03:27:01+01	www	fpc-httpd22	\N	fpc-httpd22-3.0.4_4
2020-03-07 03:27:01+01	2020-03-07 03:27:18+01	www	fpc-httpd24	\N	fpc-httpd24-3.0.4_4
2020-02-28 22:03:16+01	2020-02-28 22:03:54+01	www	libnghttp2	\N	libnghttp2-1.40.0
2020-05-16 15:57:28+02	2020-05-16 15:57:48+02	www	libwpe	\N	libwpe-1.4.0.1
2020-05-19 07:42:25+02	2020-05-19 07:42:53+02	www	links	\N	links-2.16_2,1
2020-03-01 13:17:08+01	2020-03-01 13:18:35+01	www	lynx	\N	lynx-2.8.9.1_1,1
2020-05-14 20:30:41+02	2020-05-14 20:31:55+02	www	mattermost-server	\N	mattermost-server-5.19.1
2020-05-14 20:29:31+02	2020-05-14 20:30:21+02	www	mattermost-webapp	\N	mattermost-webapp-5.19.1
2020-05-21 10:52:33+02	2020-05-21 10:53:34+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_1
2020-05-21 10:53:36+02	2020-05-21 11:07:31+02	www	mod_php73	\N	mod_php73-7.3.18
2020-05-21 11:07:34+02	2020-05-21 11:10:50+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-05-20 09:46:57+02	2020-05-20 10:55:09+02	www	node10	\N	node10-10.20.1_1
2020-05-20 07:30:09+02	2020-05-20 08:36:35+02	www	node12	\N	node12-12.16.2_1
2020-05-21 18:56:53+02	2020-05-21 18:57:24+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-05-20 23:33:25+02	2020-05-21 00:40:21+02	www	node	\N	node-13.13.0_1
2020-05-25 08:31:54+02	2020-05-25 08:46:15+02	local	E590T-localbase	\N	E590T-localbase-2020052200
2020-05-21 17:27:06+02	2020-05-21 17:27:28+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-05-02 12:56:30+02	2020-05-02 12:56:44+02	www	p5-CGI	\N	p5-CGI-4.47
2020-03-20 02:58:56+01	2020-03-20 02:59:23+01	www	p5-HTML-Parser	\N	p5-HTML-Parser-3.72
2020-03-20 02:16:36+01	2020-03-20 02:16:57+01	www	p5-HTML-Tagset	\N	p5-HTML-Tagset-3.20_1
2020-05-23 21:43:04+02	2020-05-23 21:43:24+02	www	p5-HTTP-Cookies	\N	p5-HTTP-Cookies-6.08
2020-05-23 21:43:25+02	2020-05-23 21:43:44+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.06
2020-05-23 21:42:08+02	2020-05-23 21:42:26+02	www	p5-HTTP-Date	\N	p5-HTTP-Date-6.05
2020-05-23 21:42:27+02	2020-05-23 21:42:46+02	www	p5-HTTP-Message	\N	p5-HTTP-Message-6.24
2020-05-23 21:43:44+02	2020-05-23 21:44:07+02	www	p5-HTTP-Negotiate	\N	p5-HTTP-Negotiate-6.01_1
2020-05-23 21:44:08+02	2020-05-23 21:44:51+02	www	p5-libwww	\N	p5-libwww-6.44
2020-03-20 03:23:43+01	2020-03-20 03:24:11+01	www	p5-LWP-MediaTypes	\N	p5-LWP-MediaTypes-6.04
2020-03-20 02:16:58+01	2020-03-20 02:17:21+01	www	p5-Mozilla-CA	\N	p5-Mozilla-CA-20180117
2020-03-20 08:38:06+01	2020-03-20 08:38:22+01	www	p5-WWW-RobotRules	\N	p5-WWW-RobotRules-6.02_1
2020-05-16 11:01:13+02	2020-05-16 11:01:45+02	www	php73-opcache	\N	php73-opcache-7.3.18
2020-05-16 10:56:11+02	2020-05-16 10:56:36+02	www	php73-session	\N	php73-session-7.3.18
2020-04-27 23:31:50+02	2020-04-27 23:32:08+02	www	py-beaker	py27	py27-beaker-1.11.0
2020-03-03 08:31:15+01	2020-03-03 08:31:54+01	www	py-beautifulsoup	py27	py27-beautifulsoup-4.8.2
2020-04-27 09:25:16+02	2020-04-27 09:25:53+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.0
2020-02-25 17:14:40+01	2020-02-25 17:14:50+01	www	py-bleach	py37	py37-bleach-3.1.0
2020-04-27 18:04:31+02	2020-04-27 18:05:28+02	www	py-bottle	py37	py37-bottle-0.12.15_1
2020-03-03 08:31:54+01	2020-03-03 08:32:07+01	www	py-css-parser	py27	py27-css-parser-1.0.4
2020-04-27 09:25:49+02	2020-04-27 09:26:06+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-03-03 08:32:08+01	2020-03-03 08:32:30+01	www	py-cssselect	py27	py27-cssselect-1.0.3
2020-05-14 11:15:38+02	2020-05-14 11:15:51+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-05-21 18:57:25+02	2020-05-21 18:57:41+02	www	py-dj22-django-cacheops	py37	py37-dj22-django-cacheops-4.2
2020-05-21 18:57:41+02	2020-05-21 18:57:57+02	www	py-dj22-django-cors-headers	py37	py37-dj22-django-cors-headers-3.2.1
2020-05-21 18:57:58+02	2020-05-21 18:58:14+02	www	py-dj22-django-debug-toolbar	py37	py37-dj22-django-debug-toolbar-2.1
2020-05-21 18:58:14+02	2020-05-21 18:58:30+02	www	py-dj22-django-filter	py37	py37-dj22-django-filter-2.2.0
2020-05-21 17:27:28+02	2020-05-21 17:27:40+02	www	py-dj22-django-js-asset	py37	py37-dj22-django-js-asset-1.2.2
2020-05-21 18:58:31+02	2020-05-21 18:58:50+02	www	py-dj22-django-mptt	py37	py37-dj22-django-mptt-0.9.1
2020-05-21 18:58:50+02	2020-05-21 18:59:08+02	www	py-dj22-django-prometheus	py37	py37-dj22-django-prometheus-1.1.0_1
2020-05-21 18:59:09+02	2020-05-21 18:59:26+02	www	py-dj22-django-tables2	py37	py37-dj22-django-tables2-2.2.1
2020-05-21 18:59:26+02	2020-05-21 18:59:41+02	www	py-dj22-django-taggit-serializer	py37	py37-dj22-django-taggit-serializer-0.1.7
2020-05-21 17:27:40+02	2020-05-21 17:27:53+02	www	py-dj22-django-taggit	py37	py37-dj22-django-taggit-1.2.0
2020-05-21 18:59:42+02	2020-05-21 18:59:57+02	www	py-dj22-django-timezone-field	py37	py37-dj22-django-timezone-field-4.0
2020-05-21 17:27:53+02	2020-05-21 17:28:08+02	www	py-dj22-djangorestframework	py37	py37-dj22-djangorestframework-3.11.0
2020-05-21 18:59:57+02	2020-05-21 19:00:22+02	www	py-dj22-drf-yasg	py37	py37-dj22-drf-yasg-1.17.0
2020-05-21 19:00:23+02	2020-05-21 19:00:36+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-02-25 17:18:49+01	2020-02-25 17:19:27+01	www	py-django111	py37	py37-django111-1.11.28
2020-05-21 16:16:06+02	2020-05-21 16:16:52+02	www	py-django22	py37	py37-django22-2.2.12
2020-05-21 19:00:36+02	2020-05-21 19:00:55+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-05-21 16:35:19+02	2020-05-21 16:35:34+02	www	py-h2	py37	py37-h2-3.2.0
2020-05-21 16:24:56+02	2020-05-21 16:25:07+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-03-02 09:58:01+01	2020-03-02 09:58:31+01	www	py-html5-parser	py27	py27-html5-parser-0.4.9
2020-04-27 05:35:43+02	2020-04-27 05:37:12+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-04-27 23:33:40+02	2020-04-27 23:33:55+02	www	py-html5lib	py27	py27-html5lib-1.0.1
2020-04-26 23:36:48+02	2020-04-26 23:37:34+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-05-21 16:25:08+02	2020-05-21 16:25:17+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-05-21 16:35:34+02	2020-05-21 16:35:46+02	www	py-hyperlink	py37	py37-hyperlink-19.0.0
2020-03-03 08:32:30+01	2020-03-03 08:32:45+01	www	py-mechanize	py27	py27-mechanize-0.4.5
2020-04-27 09:26:06+02	2020-04-27 09:26:27+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-05-21 16:35:46+02	2020-05-21 16:35:55+02	www	py-priority	py37	py37-priority-1.3.0
2020-03-20 08:38:13+01	2020-03-20 08:39:05+01	www	py-qt5-webchannel	py27	py27-qt5-webchannel-5.13.1
2020-05-20 09:51:06+02	2020-05-20 09:52:43+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-03-20 11:56:34+01	2020-03-20 11:57:44+01	www	py-qt5-webengine	py27	py27-qt5-webengine-5.12.1
2020-05-23 09:29:59+02	2020-05-23 09:30:52+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-05-23 09:18:19+02	2020-05-23 09:19:31+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-05-23 09:30:39+02	2020-05-23 09:31:34+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-02-25 17:15:11+01	2020-02-25 17:15:30+01	www	py-requests-toolbelt	py37	py37-requests-toolbelt-0.8.0_1
2020-03-04 21:50:53+01	2020-03-04 21:51:01+01	www	py-requests	py27	py27-requests-2.22.0
2020-05-18 22:05:54+02	2020-05-18 22:06:05+02	www	py-requests	py37	py37-requests-2.22.0
2020-03-02 09:58:31+01	2020-03-02 09:58:43+01	www	py-soupsieve	py27	py27-soupsieve-1.9_1
2020-04-27 05:37:15+02	2020-04-27 05:37:58+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-04-26 23:37:35+02	2020-04-26 23:38:37+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-05-20 02:36:00+02	2020-05-20 02:37:30+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-05-23 09:03:22+02	2020-05-23 09:20:41+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-05-23 09:00:13+02	2020-05-23 09:05:38+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-05-20 23:40:19+02	2020-05-20 23:41:18+02	www	qt5-websockets	\N	qt5-websockets-5.14.2
2020-04-03 11:14:23+02	2020-04-03 11:14:57+02	www	serf	\N	serf-1.3.9_4
2020-03-22 12:37:45+01	2020-03-22 12:38:32+01	www	w3m	\N	w3m-0.5.3.20200316
2020-05-20 01:54:32+02	2020-05-20 04:05:29+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-05-21 01:52:15+02	2020-05-21 01:52:32+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-05-21 11:15:43+02	2020-05-21 11:16:11+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-05-20 23:41:19+02	2020-05-20 23:41:36+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-05-23 09:19:32+02	2020-05-23 09:20:11+02	www	youtube_dl	\N	youtube_dl-2020.05.08
2020-05-09 16:11:19+02	2020-05-09 16:30:54+02	x11	alacritty	\N	alacritty-0.4.2_1
2020-04-17 09:00:36+02	2020-04-17 09:00:52+02	x11	appres	\N	appres-1.0.5
2020-05-20 04:48:47+02	2020-05-20 04:49:34+02	x11	babl	\N	babl-0.1.74
2020-04-17 09:00:52+02	2020-04-17 09:01:12+02	x11	bitmap	\N	bitmap-1.0.9
2020-05-21 11:16:14+02	2020-05-21 11:19:28+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-04-17 05:21:31+02	2020-04-17 05:21:57+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-02-29 19:34:58+01	2020-02-29 19:35:09+01	x11	iceauth	\N	iceauth-1.0.8_2
2020-05-20 07:40:31+02	2020-05-20 07:42:38+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-05-20 04:49:36+02	2020-05-20 04:52:09+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-05-20 02:11:40+02	2020-05-20 02:13:17+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-04-17 05:03:49+02	2020-04-17 05:04:13+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-02-28 22:19:24+01	2020-02-28 22:19:43+01	x11	libICE	\N	libICE-1.0.10,1
2020-05-25 22:33:59+02	2020-05-25 22:35:04+02	devel	bison	\N	bison-3.6.2,1
2020-05-21 00:23:15+02	2020-05-21 00:23:47+02	x11	libinput	\N	libinput-1.15.5
2020-05-20 04:52:10+02	2020-05-20 04:55:19+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-02-28 22:23:12+01	2020-02-28 22:23:27+01	x11	libSM	\N	libSM-1.2.3,1
2020-05-21 00:22:54+02	2020-05-21 00:23:14+02	x11	libwacom	\N	libwacom-1.3
2020-04-17 03:05:51+02	2020-04-17 03:11:22+02	x11	libX11	\N	libX11-1.6.9_1,1
2020-02-28 21:59:54+01	2020-02-28 22:00:04+01	x11	libXau	\N	libXau-1.0.9
2020-04-17 03:01:14+02	2020-04-17 03:05:50+02	x11	libxcb	\N	libxcb-1.13.1
2020-04-17 03:14:33+02	2020-04-17 03:15:03+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-04-17 03:14:21+02	2020-04-17 03:14:47+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-04-17 03:12:06+02	2020-04-17 03:12:29+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-04-17 03:00:41+02	2020-04-17 03:01:13+02	x11	libXdmcp	\N	libXdmcp-1.1.3
2020-04-17 03:11:23+02	2020-04-17 03:11:53+02	x11	libXext	\N	libXext-1.3.4,1
2020-04-17 03:11:41+02	2020-04-17 03:12:06+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-04-17 03:12:27+02	2020-04-17 03:13:06+02	x11	libXi	\N	libXi-1.7.10,1
2020-04-17 03:13:58+02	2020-04-17 03:14:32+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-04-17 03:06:31+02	2020-04-17 03:07:28+02	x11	libxkbcommon	\N	libxkbcommon-0.10.0_2
2020-04-17 03:12:39+02	2020-04-17 03:13:24+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-05-20 04:32:11+02	2020-05-20 04:33:01+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-04-17 03:13:27+02	2020-04-17 03:14:19+02	x11	libXpm	\N	libXpm-3.5.13
2020-04-17 03:13:07+02	2020-04-17 03:13:35+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-04-17 03:11:53+02	2020-04-17 03:12:24+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-04-17 03:44:56+02	2020-04-17 03:45:32+02	x11	libXres	\N	libXres-1.2.0_2
2020-04-17 03:15:45+02	2020-04-17 03:16:13+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-02-28 22:17:50+01	2020-02-28 22:18:00+01	x11	libxshmfence	\N	libxshmfence-1.3
2020-04-17 03:13:36+02	2020-04-17 03:14:20+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-04-17 03:20:32+02	2020-04-17 03:21:15+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-04-17 03:46:48+02	2020-04-17 03:47:19+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-04-17 03:47:19+02	2020-04-17 03:47:51+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-04-17 03:12:00+02	2020-04-17 03:12:26+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-04-17 06:21:24+02	2020-04-17 06:22:56+02	x11	luit	\N	luit-20190106_1
2020-05-20 23:41:33+02	2020-05-20 23:48:23+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-05-21 17:29:52+02	2020-05-21 17:30:51+02	x11	mate-base	\N	mate-base-1.22.2
2020-05-21 01:45:39+02	2020-05-21 01:46:56+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-05-20 02:36:23+02	2020-05-20 02:37:11+02	x11	mate-menus	\N	mate-menus-1.22.1
2020-05-20 04:33:02+02	2020-05-20 04:36:55+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-05-20 23:43:16+02	2020-05-20 23:46:12+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-05-20 07:42:39+02	2020-05-20 07:46:34+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-05-20 23:46:13+02	2020-05-20 23:48:18+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-05-23 09:56:41+02	2020-05-23 09:58:31+02	x11	mate	\N	mate-1.22.2
2020-04-17 17:12:31+02	2020-04-17 17:12:46+02	x11	numlockx	\N	numlockx-1.2
2020-05-08 22:05:56+02	2020-05-08 22:07:10+02	x11	pixman	\N	pixman-0.40.0
2020-05-21 01:36:16+02	2020-05-21 01:36:54+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-04-17 09:01:13+02	2020-04-17 09:01:28+02	x11	sessreg	\N	sessreg-1.1.2
2020-04-17 09:01:29+02	2020-04-17 09:01:44+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-04-17 09:01:45+02	2020-04-17 09:02:01+02	x11	smproxy	\N	smproxy-1.0.6
2020-04-17 03:40:45+02	2020-04-17 03:41:22+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-05-09 16:14:36+02	2020-05-09 16:14:59+02	x11	sterm	\N	sterm-0.8.2
2020-05-09 13:00:58+02	2020-05-09 13:01:11+02	x11	x11perf	\N	x11perf-1.6.1
2020-04-17 09:02:19+02	2020-04-17 09:02:38+02	x11	xauth	\N	xauth-1.1
2020-04-17 09:02:39+02	2020-04-17 09:02:54+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-02-29 02:13:13+01	2020-02-29 02:13:57+01	x11	xbitmaps	\N	xbitmaps-1.1.2
2020-05-09 13:01:12+02	2020-05-09 13:01:30+02	x11	xcalc	\N	xcalc-1.1.0
2020-04-26 15:39:08+02	2020-04-26 15:39:27+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-04-17 03:08:43+02	2020-04-17 03:09:36+02	x11	xcb-util-image	\N	xcb-util-image-0.4.0_1
2020-04-17 03:09:20+02	2020-04-17 03:10:11+02	x11	xcb-util-keysyms	\N	xcb-util-keysyms-0.4.0_1
2020-04-17 03:09:34+02	2020-04-17 03:10:29+02	x11	xcb-util-renderutil	\N	xcb-util-renderutil-0.3.9_1
2020-04-17 03:07:59+02	2020-04-17 03:08:43+02	x11	xcb-util-wm	\N	xcb-util-wm-0.4.1_3
2020-04-17 03:07:29+02	2020-04-17 03:07:57+02	x11	xcb-util	\N	xcb-util-0.4.0_2,1
2020-04-17 17:12:59+02	2020-04-17 17:13:24+02	x11	xclip	\N	xclip-0.13
2020-04-17 17:13:24+02	2020-04-17 17:13:39+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-04-17 09:03:22+02	2020-04-17 09:03:37+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-04-17 09:03:41+02	2020-04-17 09:03:56+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-04-17 06:29:11+02	2020-04-17 06:30:16+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-05-09 16:15:00+02	2020-05-09 16:16:22+02	x11	xdm	\N	xdm-1.1.12_3
2020-04-17 05:12:24+02	2020-04-17 05:12:39+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-05-16 18:54:18+02	2020-05-16 18:54:29+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-04-17 09:04:15+02	2020-04-17 09:04:28+02	x11	xev	\N	xev-1.2.3
2020-04-17 09:04:29+02	2020-04-17 09:04:48+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-04-17 09:04:48+02	2020-04-17 09:05:06+02	x11	xgamma	\N	xgamma-1.0.6
2020-04-17 09:05:06+02	2020-04-17 09:05:28+02	x11	xgc	\N	xgc-1.0.5
2020-04-17 09:05:30+02	2020-04-17 09:05:44+02	x11	xhost	\N	xhost-1.0.8
2020-04-17 09:05:45+02	2020-04-17 09:06:01+02	x11	xinit	\N	xinit-1.4.1,1
2020-04-17 09:06:02+02	2020-04-17 09:06:18+02	x11	xinput	\N	xinput-1.6.3
2020-04-17 03:13:27+02	2020-04-17 03:13:58+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-04-17 09:06:19+02	2020-04-17 09:06:33+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-04-17 09:06:33+02	2020-04-17 09:06:51+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-04-02 23:51:51+02	2020-04-02 23:54:04+02	x11	xkeyboard-config	\N	xkeyboard-config-2.29
2020-04-17 09:06:51+02	2020-04-17 09:07:08+02	x11	xkill	\N	xkill-1.0.5
2020-05-16 21:30:29+02	2020-05-16 21:30:46+02	x11	xlockmore	\N	xlockmore-5.63
2020-04-17 09:07:09+02	2020-04-17 09:07:23+02	x11	xlsatoms	\N	xlsatoms-1.1.3
2020-04-17 09:07:23+02	2020-04-17 09:07:38+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-04-17 17:14:39+02	2020-04-17 17:14:56+02	x11	xman	\N	xman-1.1.5
2020-04-17 09:07:38+02	2020-04-17 09:07:52+02	x11	xmessage	\N	xmessage-1.0.5
2020-04-17 09:07:53+02	2020-04-17 09:08:09+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-05-16 20:05:07+02	2020-05-16 20:05:30+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-02-29 02:24:28+01	2020-02-29 02:24:37+01	x11	xorg-cf-files	\N	xorg-cf-files-1.0.6
2020-03-01 03:51:49+01	2020-03-01 03:52:00+01	x11	xorg-docs	\N	xorg-docs-1.7.1,1
2020-05-09 14:15:07+02	2020-05-09 14:15:16+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-05-23 00:09:21+02	2020-05-23 00:12:31+02	x11	xorg	\N	xorg-7.7_3
2020-04-17 03:00:23+02	2020-04-17 03:00:41+02	x11	xorgproto	\N	xorgproto-2020.1
2020-04-17 09:08:10+02	2020-04-17 09:08:27+02	x11	xpr	\N	xpr-1.0.5
2020-04-17 03:15:55+02	2020-04-17 03:16:20+02	x11	xprop	\N	xprop-1.2.4
2020-04-17 09:08:27+02	2020-04-17 09:08:46+02	x11	xrandr	\N	xrandr-1.5.1
2020-04-17 05:12:35+02	2020-04-17 05:12:47+02	x11	xrdb	\N	xrdb-1.2.0
2020-04-17 09:08:46+02	2020-04-17 09:09:04+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-05-23 21:44:52+02	2020-05-23 21:50:19+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-04-17 03:16:13+02	2020-04-17 03:16:38+02	x11	xset	\N	xset-1.2.4_3
2020-04-17 09:09:04+02	2020-04-17 09:09:20+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-05-09 12:28:53+02	2020-05-09 12:29:12+02	x11	xterm	\N	xterm-356
2020-02-28 22:00:54+01	2020-02-28 22:01:02+01	x11	xtrans	\N	xtrans-1.4.0
2020-04-17 09:10:07+02	2020-04-17 09:10:23+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-04-17 09:10:23+02	2020-04-17 09:10:40+02	x11	xwd	\N	xwd-1.0.7
2020-04-17 09:10:43+02	2020-04-17 09:10:57+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-04-17 09:10:57+02	2020-04-17 09:11:14+02	x11	xwud	\N	xwud-1.0.5
2020-05-20 07:46:35+02	2020-05-20 07:51:55+02	x11	yelp	\N	yelp-3.28.1_2
2020-05-25 22:35:18+02	2020-05-25 22:35:35+02	www	serf	\N	serf-1.3.9_5
2020-05-20 06:06:24+02	2020-05-20 06:07:46+02	x11	zenity	\N	zenity-3.28.1
2020-05-09 13:01:31+02	2020-05-09 13:01:42+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-05-21 23:57:13+02	2020-05-21 23:58:23+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-05-16 18:54:35+02	2020-05-16 18:55:24+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-05-21 17:01:05+02	2020-05-21 17:01:52+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-05-16 18:55:25+02	2020-05-16 18:56:29+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-03-13 05:53:07+01	2020-03-13 05:54:25+01	x11-drivers	xf86-input-synaptics	\N	xf86-input-synaptics-1.9.1_5
2020-03-13 05:54:26+01	2020-03-13 05:55:48+01	x11-drivers	xf86-input-wacom	\N	xf86-input-wacom-0.39.0_1
2020-05-16 18:56:29+02	2020-05-16 18:57:37+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-05-16 18:56:30+02	2020-05-16 18:58:10+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-05-16 18:57:37+02	2020-05-16 18:58:42+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-05-16 18:58:11+02	2020-05-16 18:59:15+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-05-23 00:01:08+02	2020-05-23 00:01:52+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-05-20 04:55:19+02	2020-05-20 05:01:59+02	x11-fm	caja	\N	caja-1.22.3
2020-05-23 09:58:32+02	2020-05-23 10:05:15+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-05-21 11:54:11+02	2020-05-21 11:56:28+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-04-17 17:28:31+02	2020-04-17 17:28:55+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-05-16 21:34:32+02	2020-05-16 21:34:44+02	x11-fm	xdiskusage	\N	xdiskusage-1.51_1
2020-02-28 22:27:58+01	2020-02-28 22:28:09+01	x11-fonts	bdftopcf	\N	bdftopcf-1.1_2
2020-05-09 14:15:16+02	2020-05-09 14:15:23+02	x11-fonts	bitstream-vera	\N	bitstream-vera-1.10_8
2020-05-09 13:01:42+02	2020-05-09 13:01:49+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-05-09 13:01:50+02	2020-05-09 13:01:58+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-05-09 11:12:31+02	2020-05-09 11:12:49+02	x11-fonts	dejavu	\N	dejavu-2.37_1
2020-02-28 22:29:16+01	2020-02-28 22:29:25+01	x11-fonts	encodings	\N	encodings-1.0.5,1
2020-05-09 12:10:02+02	2020-05-09 12:10:39+02	x11-fonts	font-adobe-100dpi	\N	font-adobe-100dpi-1.0.3_4
2020-05-09 12:18:08+02	2020-05-09 12:18:32+02	x11-fonts	font-adobe-75dpi	\N	font-adobe-75dpi-1.0.3_4
2020-05-09 12:30:09+02	2020-05-09 12:30:22+02	x11-fonts	font-adobe-utopia-100dpi	\N	font-adobe-utopia-100dpi-1.0.4_4
2020-05-09 12:30:23+02	2020-05-09 12:30:41+02	x11-fonts	font-adobe-utopia-75dpi	\N	font-adobe-utopia-75dpi-1.0.4_4
2020-05-09 12:30:42+02	2020-05-09 12:30:51+02	x11-fonts	font-adobe-utopia-type1	\N	font-adobe-utopia-type1-1.0.4_4
2020-02-29 07:30:22+01	2020-02-29 07:30:33+01	x11-fonts	font-alias	\N	font-alias-1.0.3_3
2020-05-09 12:30:51+02	2020-05-09 12:31:03+02	x11-fonts	font-arabic-misc	\N	font-arabic-misc-1.0.3_4
2020-05-09 12:31:03+02	2020-05-09 12:31:32+02	x11-fonts	font-bh-100dpi	\N	font-bh-100dpi-1.0.3_4
2020-05-09 12:31:32+02	2020-05-09 12:31:59+02	x11-fonts	font-bh-75dpi	\N	font-bh-75dpi-1.0.3_4
2020-05-09 12:31:59+02	2020-05-09 12:32:20+02	x11-fonts	font-bh-lucidatypewriter-100dpi	\N	font-bh-lucidatypewriter-100dpi-1.0.3_4
2020-05-09 12:32:20+02	2020-05-09 12:32:36+02	x11-fonts	font-bh-lucidatypewriter-75dpi	\N	font-bh-lucidatypewriter-75dpi-1.0.3_4
2020-05-09 11:12:50+02	2020-05-09 11:13:01+02	x11-fonts	font-bh-ttf	\N	font-bh-ttf-1.0.3_4
2020-05-09 12:32:37+02	2020-05-09 12:32:48+02	x11-fonts	font-bh-type1	\N	font-bh-type1-1.0.3_4
2020-05-09 12:32:38+02	2020-05-09 12:32:49+02	x11-fonts	font-bitstream-100dpi	\N	font-bitstream-100dpi-1.0.3_4
2020-05-09 12:32:49+02	2020-05-09 12:32:59+02	x11-fonts	font-bitstream-75dpi	\N	font-bitstream-75dpi-1.0.3_4
2020-05-09 12:32:49+02	2020-05-09 12:33:00+02	x11-fonts	font-bitstream-type1	\N	font-bitstream-type1-1.0.3_4
2020-05-09 12:33:00+02	2020-05-09 12:33:11+02	x11-fonts	font-cronyx-cyrillic	\N	font-cronyx-cyrillic-1.0.3_4
2020-05-09 12:33:01+02	2020-05-09 12:33:12+02	x11-fonts	font-cursor-misc	\N	font-cursor-misc-1.0.3_4
2020-05-09 12:33:11+02	2020-05-09 12:33:26+02	x11-fonts	font-daewoo-misc	\N	font-daewoo-misc-1.0.3_4
2020-05-09 12:33:12+02	2020-05-09 12:33:25+02	x11-fonts	font-dec-misc	\N	font-dec-misc-1.0.3_4
2020-05-09 12:33:26+02	2020-05-09 12:33:37+02	x11-fonts	font-ibm-type1	\N	font-ibm-type1-1.0.3_4
2020-05-09 12:33:26+02	2020-05-09 12:33:39+02	x11-fonts	font-isas-misc	\N	font-isas-misc-1.0.3_4
2020-05-09 12:33:39+02	2020-05-09 12:33:51+02	x11-fonts	font-jis-misc	\N	font-jis-misc-1.0.3_4
2020-05-09 12:33:40+02	2020-05-09 12:33:51+02	x11-fonts	font-micro-misc	\N	font-micro-misc-1.0.3_4
2020-05-09 12:33:51+02	2020-05-09 12:34:02+02	x11-fonts	font-misc-cyrillic	\N	font-misc-cyrillic-1.0.3_4
2020-05-09 11:12:57+02	2020-05-09 11:13:10+02	x11-fonts	font-misc-ethiopic	\N	font-misc-ethiopic-1.0.3_4
2020-05-09 11:13:01+02	2020-05-09 11:13:18+02	x11-fonts	font-misc-meltho	\N	font-misc-meltho-1.0.3_4
2020-05-09 12:06:09+02	2020-05-09 12:06:32+02	x11-fonts	font-misc-misc	\N	font-misc-misc-1.1.2_4
2020-05-09 12:33:52+02	2020-05-09 12:34:03+02	x11-fonts	font-mutt-misc	\N	font-mutt-misc-1.0.3_4
2020-05-09 12:34:02+02	2020-05-09 12:34:15+02	x11-fonts	font-schumacher-misc	\N	font-schumacher-misc-1.1.2_4
2020-05-09 12:34:04+02	2020-05-09 12:34:14+02	x11-fonts	font-screen-cyrillic	\N	font-screen-cyrillic-1.0.4_4
2020-05-09 12:34:15+02	2020-05-09 12:34:25+02	x11-fonts	font-sony-misc	\N	font-sony-misc-1.0.3_4
2020-05-09 12:34:15+02	2020-05-09 12:34:26+02	x11-fonts	font-sun-misc	\N	font-sun-misc-1.0.3_4
2020-02-28 22:28:09+01	2020-02-28 22:28:19+01	x11-fonts	font-util	\N	font-util-1.3.2
2020-05-09 12:34:26+02	2020-05-09 12:34:36+02	x11-fonts	font-winitzki-cyrillic	\N	font-winitzki-cyrillic-1.0.3_4
2020-05-09 12:34:27+02	2020-05-09 12:34:37+02	x11-fonts	font-xfree86-type1	\N	font-xfree86-type1-1.0.4_4
2020-05-09 11:10:58+02	2020-05-09 11:11:58+02	x11-fonts	fontconfig	\N	fontconfig-2.13.92_2,1
2020-05-09 13:01:58+02	2020-05-09 13:02:05+02	x11-fonts	gentium-basic	\N	GentiumBasic-1102_1
2020-05-09 14:15:22+02	2020-05-09 14:15:29+02	x11-fonts	gnu-unifont	\N	gnu-unifont-13.0.02,1
2020-05-09 13:01:59+02	2020-05-09 13:02:08+02	x11-fonts	liberation-fonts-ttf	\N	liberation-fonts-ttf-2.1.0,2
2020-02-28 22:24:09+01	2020-02-28 22:24:23+01	x11-fonts	libfontenc	\N	libfontenc-1.1.4
2020-02-29 19:43:01+01	2020-02-29 19:43:17+01	x11-fonts	libFS	\N	libFS-1.0.8
2020-02-29 02:24:38+01	2020-02-29 02:25:08+01	x11-fonts	libXfont	\N	libXfont-1.5.4_2,2
2020-03-01 00:57:39+01	2020-03-01 00:58:08+01	x11-fonts	libXfont2	\N	libXfont2-2.0.4
2020-05-09 11:13:11+02	2020-05-09 11:13:31+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-05-09 13:02:06+02	2020-05-09 13:02:20+02	x11-fonts	linuxlibertine-g	\N	linuxlibertine-g-20120116_2
2020-02-28 22:24:23+01	2020-02-28 22:24:36+01	x11-fonts	mkfontscale	\N	mkfontscale-1.2.1
2020-05-09 12:18:14+02	2020-05-09 12:19:05+02	x11-fonts	noto-basic	\N	noto-basic-2.0
2020-03-20 06:15:35+01	2020-03-20 06:15:57+01	x11-fonts	p5-Font-TTF	\N	p5-Font-TTF-1.06
2020-05-09 13:02:08+02	2020-05-09 13:02:20+02	x11-fonts	twemoji-color-font-ttf	\N	twemoji-color-font-ttf-12.0.1
2020-05-09 13:02:20+02	2020-05-09 13:02:51+02	x11-fonts	xorg-fonts-100dpi	\N	xorg-fonts-100dpi-7.7
2020-05-09 13:02:20+02	2020-05-09 13:02:50+02	x11-fonts	xorg-fonts-75dpi	\N	xorg-fonts-75dpi-7.7
2020-05-09 13:02:51+02	2020-05-09 13:03:07+02	x11-fonts	xorg-fonts-cyrillic	\N	xorg-fonts-cyrillic-7.7
2020-05-09 13:02:51+02	2020-05-09 13:03:35+02	x11-fonts	xorg-fonts-miscbitmaps	\N	xorg-fonts-miscbitmaps-7.7
2020-05-09 11:13:19+02	2020-05-09 11:13:39+02	x11-fonts	xorg-fonts-truetype	\N	xorg-fonts-truetype-7.7_1
2020-05-09 13:03:07+02	2020-05-09 13:03:27+02	x11-fonts	xorg-fonts-type1	\N	xorg-fonts-type1-7.7
2020-05-09 14:15:23+02	2020-05-09 14:17:14+02	x11-fonts	xorg-fonts	\N	xorg-fonts-7.7_1
2020-05-16 18:58:43+02	2020-05-16 19:00:38+02	x11-servers	xephyr	\N	xephyr-1.20.8_1,1
2020-05-16 16:15:19+02	2020-05-16 16:16:43+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_1,1
2020-05-25 22:48:17+02	2020-05-25 22:48:54+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-05-25 22:35:35+02	2020-05-25 22:35:49+02	textproc	lowdown	\N	lowdown-0.6.5
2020-05-16 20:05:31+02	2020-05-16 20:06:48+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_1,1
2020-05-21 00:25:22+02	2020-05-21 00:29:21+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-05-20 07:51:56+02	2020-05-20 07:55:59+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-05-20 07:56:00+02	2020-05-20 07:58:40+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-03-20 02:02:39+01	2020-03-20 02:03:01+01	x11-themes	icon-naming-utils	\N	icon-naming-utils-0.8.90_1
2020-05-20 04:05:09+02	2020-05-20 04:07:44+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.70.0
2020-05-20 04:16:19+02	2020-05-20 04:17:59+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-05-20 11:03:14+02	2020-05-20 11:04:58+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.22.0
2020-05-20 04:05:30+02	2020-05-20 04:08:11+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.22.2
2020-05-20 11:04:59+02	2020-05-20 11:11:24+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-03-01 03:54:20+01	2020-03-01 03:54:33+01	x11-themes	xcursor-themes	\N	xcursor-themes-1.0.6
2020-05-16 20:06:12+02	2020-05-16 20:06:34+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-05-20 11:11:25+02	2020-05-20 11:13:13+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-05-20 23:48:24+02	2020-05-20 23:49:43+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-05-20 04:17:59+02	2020-05-20 04:18:55+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-05-21 00:40:44+02	2020-05-21 00:49:59+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-05-21 00:29:21+02	2020-05-21 00:48:18+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-05-20 23:49:29+02	2020-05-20 23:50:14+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-05-20 23:49:44+02	2020-05-20 23:58:33+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-05-20 06:06:28+02	2020-05-20 06:17:04+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-05-20 07:58:40+02	2020-05-20 08:05:06+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-05-20 05:02:01+02	2020-05-20 05:05:35+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-05-20 04:33:05+02	2020-05-20 04:34:15+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-05-20 04:07:45+02	2020-05-20 04:09:39+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-05-20 03:08:49+02	2020-05-20 03:11:52+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-05-20 04:08:11+02	2020-05-20 04:09:47+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-05-20 08:05:07+02	2020-05-20 08:07:37+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-05-20 05:05:36+02	2020-05-20 05:09:01+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-05-20 02:28:34+02	2020-05-20 02:31:30+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-05-20 06:07:46+02	2020-05-20 06:11:37+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-05-21 11:56:29+02	2020-05-21 11:57:25+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-05-20 05:09:02+02	2020-05-20 05:11:52+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-05-20 03:11:53+02	2020-05-20 03:17:32+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-04-17 03:14:22+02	2020-04-17 03:15:44+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-04-17 03:13:27+02	2020-04-17 03:14:22+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-04-17 03:12:24+02	2020-04-17 03:13:24+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-05-09 12:37:32+02	2020-05-09 12:47:32+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-05-21 00:07:41+02	2020-05-21 00:11:37+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-05-20 04:18:55+02	2020-05-20 04:20:40+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-05-21 01:52:33+02	2020-05-21 01:52:59+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-05-20 11:13:14+02	2020-05-20 11:18:25+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-03-20 03:30:23+01	2020-03-20 03:57:05+01	x11-toolkits	py-qt5-gui	py27	py27-qt5-gui-5.13.1
2020-05-20 03:17:33+02	2020-05-20 03:50:11+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-03-20 04:14:04+01	2020-03-20 04:23:43+01	x11-toolkits	py-qt5-widgets	py27	py27-qt5-widgets-5.13.1
2020-05-20 04:20:41+02	2020-05-20 04:30:35+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-03-16 06:18:20+01	2020-03-16 06:18:30+01	x11-toolkits	py-tkinter	py27	py27-tkinter-2.7.17_6
2020-05-09 12:06:48+02	2020-05-09 12:07:02+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.7_6
2020-05-16 21:35:13+02	2020-05-16 21:35:36+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.3_6
2020-05-21 00:59:00+02	2020-05-21 01:22:21+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-05-21 00:30:56+02	2020-05-21 00:48:02+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-05-20 11:18:26+02	2020-05-20 11:24:26+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-05-21 01:37:14+02	2020-05-21 01:37:43+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-05-21 00:48:02+02	2020-05-21 00:57:56+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-05-09 11:57:10+02	2020-05-09 11:58:20+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-05-21 01:56:44+02	2020-05-21 01:58:40+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-05-20 01:58:41+02	2020-05-20 02:00:08+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-05-20 06:11:38+02	2020-05-20 06:15:52+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-05-20 08:07:38+02	2020-05-20 08:16:11+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-05-20 02:11:51+02	2020-05-20 02:23:05+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-05-20 11:24:26+02	2020-05-20 12:08:18+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-04-26 15:30:44+02	2020-04-26 15:31:55+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-04-17 17:29:45+02	2020-04-17 17:30:27+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-04-17 17:30:27+02	2020-04-17 17:30:52+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-05-09 12:39:38+02	2020-05-09 12:40:38+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-05-20 08:16:12+02	2020-05-20 08:23:33+02	x11-wm	marco	\N	marco-1.22.4
2020-05-07 20:42:34+02	2020-05-07 20:43:18+02	x11-wm	twm	\N	twm-1.0.10_1
2020-05-23 22:58:10+02	2020-05-23 23:14:43+02	local	E590T-localbase	\N	E590T-localbase-2020052200
2020-05-25 22:48:49+02	2020-05-25 22:49:13+02	devel	qt5-script	\N	qt5-script-5.14.2
2020-05-24 09:09:44+02	2020-05-24 09:15:58+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-05-24 09:15:59+02	2020-05-24 09:16:06+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-05-24 09:34:05+02	2020-05-24 09:48:40+02	local	E590T-localbase	\N	E590T-localbase-2020052200
2020-05-25 22:53:48+02	2020-05-25 22:54:27+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-05-24 11:13:59+02	2020-05-24 11:17:29+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-05-25 23:04:23+02	2020-05-25 23:05:10+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-05-24 11:18:13+02	2020-05-24 11:18:23+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-05-25 23:46:21+02	2020-05-25 23:49:38+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-05-26 19:47:15+02	2020-05-26 19:51:58+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-05-24 11:48:10+02	2020-05-24 12:02:45+02	local	E590T-localbase	\N	E590T-localbase-2020052200
2020-05-24 12:56:21+02	2020-05-24 12:58:14+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-05-24 12:58:15+02	2020-05-24 12:58:22+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-05-24 13:17:27+02	2020-05-24 13:32:00+02	local	E590T-localbase	\N	E590T-localbase-2020052200
2020-05-25 22:36:02+02	2020-05-25 22:37:00+02	devel	subversion	\N	subversion-1.13.0_1
2020-05-25 22:48:40+02	2020-05-25 22:49:32+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.70.0_1
2020-05-25 22:54:25+02	2020-05-25 22:55:12+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-05-25 23:04:15+02	2020-05-25 23:05:15+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-05-25 23:26:06+02	2020-05-25 23:52:21+02	devel	RStudio	\N	RStudio-1.3.959
2020-05-21 00:24:44+02	2020-05-21 00:25:22+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.26.2
2020-05-21 00:11:38+02	2020-05-21 00:12:42+02	accessibility	at-spi2-core	\N	at-spi2-core-2.28.0
2020-05-21 00:13:51+02	2020-05-21 00:14:21+02	accessibility	atk	\N	atk-2.28.1
2020-05-20 04:09:40+02	2020-05-20 04:10:39+02	accessibility	atkmm	\N	atkmm-2.24.2_4
2020-05-20 03:50:12+02	2020-05-20 03:52:28+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-05-20 02:00:09+02	2020-05-20 02:00:56+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-05-10 10:07:21+02	2020-05-10 10:07:34+02	archivers	arc	\N	arc-5.21p
2020-05-10 10:07:21+02	2020-05-10 10:08:07+02	archivers	arj	\N	arj-3.10.22_8
2020-03-01 00:07:39+01	2020-03-01 00:08:37+01	archivers	brotli	\N	brotli-1.0.7_2,1
2020-05-20 12:08:21+02	2020-05-20 12:14:50+02	archivers	engrampa	\N	engrampa-1.22.2
2020-02-29 09:45:35+01	2020-02-29 09:46:36+01	archivers	gtar	\N	gtar-1.32
2020-05-20 02:00:56+02	2020-05-20 02:01:53+02	archivers	kf5-karchive	\N	kf5-karchive-5.70.0
2020-02-29 09:06:33+01	2020-02-29 09:06:40+01	devel	mdds	\N	mdds-1.4.1
2020-05-23 08:47:36+02	2020-05-23 08:48:50+02	archivers	libarchive	\N	libarchive-3.4.3,1
2020-02-28 22:01:03+01	2020-02-28 22:01:33+01	archivers	liblz4	\N	liblz4-1.9.2_1,1
2020-02-29 01:23:07+01	2020-02-29 01:23:30+01	archivers	libmspack	\N	libmspack-0.10.1
2020-02-29 19:54:02+01	2020-02-29 19:54:30+01	archivers	libunrar5	\N	libunrar5-5.6.8_1
2020-02-28 22:07:51+01	2020-02-28 22:08:34+01	archivers	lzo2	\N	lzo2-2.10_1
2020-03-16 22:40:07+01	2020-03-16 22:40:37+01	archivers	minizip	\N	minizip-1.2.11
2020-03-28 19:28:44+01	2020-03-28 19:29:06+01	archivers	p5-Archive-Zip	\N	p5-Archive-Zip-1.68
2020-03-01 03:58:12+01	2020-03-01 03:58:23+01	archivers	pbzip2	\N	pbzip2-1.1.13
2020-05-16 10:56:36+02	2020-05-16 10:57:04+02	archivers	php73-zlib	\N	php73-zlib-7.3.18
2020-02-29 02:07:14+01	2020-02-29 02:07:32+01	archivers	snappy	\N	snappy-1.1.8
2020-04-05 01:27:21+02	2020-04-05 01:27:49+02	archivers	unrar	\N	unrar-5.90,6
2020-02-29 08:38:41+01	2020-02-29 08:38:59+01	archivers	unzip	\N	unzip-6.0_8
2020-05-10 10:07:21+02	2020-05-10 10:07:29+02	archivers	unzoo	\N	unzoo-4.4_2
2020-02-28 23:37:26+01	2020-02-28 23:37:40+01	archivers	zip	\N	zip-3.0_1
2020-02-29 19:54:30+01	2020-02-29 19:56:35+01	archivers	zstd	\N	zstd-1.4.4_1
2020-05-25 22:33:59+02	2020-05-25 22:37:20+02	archivers	zstd	\N	zstd-1.4.5
2020-05-09 14:17:15+02	2020-05-09 14:19:27+02	astro	xephem	\N	xephem-3.7.7_5
2020-02-28 23:56:35+01	2020-02-28 23:57:24+01	audio	alsa-lib	\N	alsa-lib-1.1.2_2
2020-02-29 01:09:28+01	2020-02-29 01:09:42+01	audio	alsa-plugins	\N	alsa-plugins-1.1.1_6
2020-02-29 15:11:25+01	2020-02-29 15:11:51+01	audio	celt	\N	celt-0.11.3_3
2020-03-01 00:58:08+01	2020-03-01 00:58:30+01	audio	espeak	\N	espeak-1.48.04_7
2020-05-23 09:20:11+02	2020-05-23 09:20:52+02	audio	faudio	\N	FAudio-20.05
2020-02-29 19:56:35+01	2020-02-29 19:59:08+01	audio	fdk-aac	\N	fdk-aac-2.0.1
2020-02-28 23:42:07+01	2020-02-28 23:43:09+01	audio	flac	\N	flac-1.3.3
2020-02-29 19:56:42+01	2020-02-29 19:57:05+01	audio	freealut	\N	freealut-1.1.0_4
2020-02-29 19:54:49+01	2020-02-29 19:55:59+01	audio	faad	\N	faad2-2.8.8,1
2020-03-16 22:40:37+01	2020-03-16 22:40:50+01	audio	gsm	\N	gsm-1.0.13_2
2020-05-20 04:09:47+02	2020-05-20 04:10:36+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-05-20 04:10:37+02	2020-05-20 04:11:16+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-05-20 03:52:29+02	2020-05-20 03:59:01+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-05-20 04:10:40+02	2020-05-20 04:11:27+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-03-02 10:00:19+01	2020-03-02 10:01:38+01	audio	jack	\N	jackit-0.125.0_9
2020-03-17 05:42:00+01	2020-03-17 05:42:11+01	audio	ladspa	\N	ladspa-1.15
2020-03-16 22:40:50+01	2020-03-16 22:41:35+01	audio	lame	\N	lame-3.100_2
2020-02-29 02:14:33+01	2020-02-29 02:14:47+01	audio	liba52	\N	liba52-0.7.4_3
2020-03-16 22:41:35+01	2020-03-16 22:41:51+01	audio	libao	\N	libao-1.2.0_4
2020-05-21 01:52:35+02	2020-05-21 01:53:18+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-05-21 01:46:42+02	2020-05-21 01:47:19+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-02-29 06:09:45+01	2020-02-29 06:10:03+01	audio	libcddb	\N	libcddb-1.3.2_4
2020-03-16 22:41:52+01	2020-03-16 22:42:07+01	audio	libid3tag	\N	libid3tag-0.15.1b_2
2020-02-29 19:58:19+01	2020-02-29 19:58:46+01	audio	libmad	\N	libmad-0.15.1b_7
2020-05-20 04:34:16+02	2020-05-20 04:35:28+02	audio	libmatemixer	\N	libmatemixer-1.22.0
2020-02-29 01:53:59+01	2020-02-29 01:54:38+01	audio	libmikmod	\N	libmikmod-3.3.11.1_1
2020-02-29 09:45:52+01	2020-02-29 09:46:16+01	audio	libmodplug	\N	libmodplug-0.8.9.0
2020-02-29 09:04:51+01	2020-02-29 09:05:04+01	audio	libmysofa	\N	libmysofa-0.9
2020-02-28 23:27:18+01	2020-02-28 23:27:32+01	audio	libogg	\N	libogg-1.3.4,4
2020-03-08 18:13:49+01	2020-03-08 18:15:17+01	audio	libopenshot-audio	\N	libopenshot-audio-0.2.0.2
2020-02-29 09:46:17+01	2020-02-29 09:46:45+01	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-02-28 23:48:57+01	2020-02-28 23:49:55+01	audio	libsndfile	\N	libsndfile-1.0.28_2
2020-02-29 01:09:42+01	2020-02-29 01:10:03+01	audio	libsoxr	\N	libsoxr-0.1.3_2
2020-02-28 23:28:46+01	2020-02-28 23:29:11+01	audio	libvorbis	\N	libvorbis-1.3.6,3
2020-05-20 12:16:33+02	2020-05-20 12:20:12+02	audio	mate-media	\N	mate-media-1.22.2
2020-02-29 06:10:03+01	2020-02-29 06:10:45+01	audio	mpg123	\N	mpg123-1.25.13
2020-02-29 09:46:37+01	2020-02-29 09:47:53+01	audio	openal-soft	\N	openal-soft-1.20.1
2020-03-01 00:00:55+01	2020-03-01 00:01:43+01	audio	opus	\N	opus-1.3.1
2020-03-16 22:42:09+01	2020-03-16 22:42:52+01	audio	opusfile	\N	opusfile-0.10
2020-03-01 00:52:24+01	2020-03-01 00:53:42+01	audio	portaudio	\N	portaudio-19.6.0_3,1
2020-05-20 02:13:17+02	2020-05-20 02:23:12+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-05-16 16:39:24+02	2020-05-16 16:39:51+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-05-16 18:17:50+02	2020-05-16 18:18:23+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-03-01 21:12:35+01	2020-03-01 21:12:45+01	audio	sndio	\N	sndio-1.6.0
2020-03-17 02:55:55+01	2020-03-17 02:58:33+01	audio	sox	\N	sox-14.4.2_3
2020-02-29 02:14:47+01	2020-02-29 02:15:06+01	audio	speex	\N	speex-1.2.0,1
2020-03-01 00:08:38+01	2020-03-01 00:08:54+01	audio	speexdsp	\N	speexdsp-1.2.0
2020-04-19 18:37:28+02	2020-04-19 18:38:30+02	audio	swhplugins	\N	swhplugins-0.4.15_7
2020-05-03 12:10:29+02	2020-05-03 12:13:28+02	audio	taglib	\N	taglib-1.12.b.1
2020-02-29 01:54:38+01	2020-02-29 01:54:54+01	audio	timidity	\N	timidity-0.2i_1
2020-02-29 20:00:24+01	2020-02-29 20:00:45+01	audio	twolame	\N	twolame-0.4.0
2020-02-29 01:10:03+01	2020-02-29 01:13:06+01	audio	webrtc-audio-processing	\N	webrtc-audio-processing-0.3.1_2
2020-04-17 12:41:21+02	2020-04-17 12:41:34+02	audio	xmix	\N	xmix-2.1_3
2020-05-24 12:56:21+02	2020-05-24 12:57:26+02	benchmarks	fio	\N	fio-3.20
2020-03-01 04:08:33+01	2020-03-01 04:08:57+01	benchmarks	iperf3	\N	iperf3-3.7
2020-03-01 04:08:58+01	2020-03-01 04:09:50+01	comms	kermit	\N	kermit-9.0.304
2020-05-21 16:25:18+02	2020-05-21 16:25:29+02	comms	py-serial	py37	py37-serial-3.4_1
2020-05-20 04:25:46+02	2020-05-20 04:30:24+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-05-20 02:23:05+02	2020-05-20 02:23:38+02	comms	qt5-serialport	\N	qt5-serialport-5.14.2
2020-03-01 04:09:50+01	2020-03-01 04:10:00+01	converters	dosunix	\N	dosunix-1.0.14
2020-03-07 03:18:34+01	2020-03-07 03:18:47+01	converters	fpc-iconvenc	\N	fpc-iconvenc-3.0.4_4
2020-02-28 22:46:59+01	2020-02-28 22:47:14+01	converters	fribidi	\N	fribidi-0.19.7
2020-05-18 22:04:26+02	2020-05-18 22:04:59+02	converters	libiconv	\N	libiconv-1.16
2020-03-20 03:10:06+01	2020-03-20 03:10:35+01	converters	p5-Encode-Locale	\N	p5-Encode-Locale-1.05
2020-03-19 20:16:02+01	2020-03-19 20:16:13+01	converters	p5-Text-Unidecode	\N	p5-Text-Unidecode-1.30
2020-05-18 22:06:35+02	2020-05-18 22:07:03+02	converters	php73-iconv	\N	php73-iconv-7.3.18
2020-05-16 10:57:27+02	2020-05-16 10:57:59+02	converters	php73-mbstring	\N	php73-mbstring-7.3.18
2020-04-27 23:32:27+02	2020-04-27 23:32:46+02	converters	py-webencodings	py27	py27-webencodings-0.5.1
2020-04-26 20:07:02+02	2020-04-26 20:07:56+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-04-26 15:31:19+02	2020-04-26 15:33:06+02	converters	recode	\N	recode-3.6_14
2020-03-01 04:10:51+01	2020-03-01 04:10:59+01	converters	unix2dos	\N	unix2dos-1.3
2020-02-28 23:37:40+01	2020-02-28 23:41:17+01	databases	db6	\N	db6-6.2.32_1
2020-04-25 06:06:12+02	2020-04-25 06:38:26+02	databases	firebird25-client	\N	firebird25-client-2.5.8_6
2020-03-07 03:19:05+01	2020-03-07 03:19:18+01	databases	fpc-dblib	\N	fpc-dblib-3.0.4_4
2020-04-25 06:38:28+02	2020-04-25 06:38:59+02	databases	fpc-ibase	\N	fpc-ibase-3.0.4_5
2020-05-19 07:19:34+02	2020-05-19 07:20:00+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-03-07 03:20:08+01	2020-03-07 03:20:18+01	databases	fpc-odbc	\N	fpc-odbc-3.0.4_4
2020-03-07 03:20:19+01	2020-03-07 03:20:31+01	databases	fpc-oracle	\N	fpc-oracle-3.0.4_4
2020-05-18 08:26:36+02	2020-05-18 08:26:47+02	databases	fpc-postgres	\N	fpc-postgres-3.0.4_6
2020-03-07 03:20:57+01	2020-03-07 03:21:25+01	databases	fpc-pxlib	\N	fpc-pxlib-3.0.4_4
2020-04-25 06:26:54+02	2020-04-25 06:27:42+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-08-04 18:42:00+02	2020-08-04 18:42:20+02	x11	xauth	\N	xauth-1.1
2020-03-02 07:09:52+01	2020-03-02 07:10:26+01	databases	gdbm	\N	gdbm-1.18.1_1
2020-05-19 07:12:07+02	2020-05-19 07:25:15+02	databases	libmemcached	\N	libmemcached-1.0.18_7
2020-02-29 20:00:45+01	2020-02-29 20:01:02+01	databases	lmdb	\N	lmdb-0.9.24_1,1
2020-05-19 07:18:35+02	2020-05-19 07:19:33+02	databases	mysql57-client	\N	mysql57-client-5.7.30_1
2020-05-16 10:58:18+02	2020-05-16 10:58:36+02	databases	pecl-memcache	php73	php73-pecl-memcache-4.0.5.2
2020-05-19 07:25:16+02	2020-05-19 07:25:45+02	databases	pecl-memcached	php73	php73-pecl-memcached-3.1.5
2020-05-18 08:24:14+02	2020-05-18 08:24:37+02	databases	php73-pdo_pgsql	\N	php73-pdo_pgsql-7.3.18
2020-05-16 10:58:37+02	2020-05-16 10:59:00+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.18
2020-05-16 10:57:04+02	2020-05-16 10:57:27+02	databases	php73-pdo	\N	php73-pdo-7.3.18
2020-05-18 08:24:26+02	2020-05-18 08:24:49+02	databases	php73-pgsql	\N	php73-pgsql-7.3.18
2020-05-16 10:58:59+02	2020-05-16 10:59:22+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.18
2020-05-18 08:19:53+02	2020-05-18 08:22:23+02	databases	postgresql12-client	\N	postgresql12-client-12.3
2020-05-18 09:35:13+02	2020-05-18 09:40:18+02	databases	postgresql12-contrib	\N	postgresql12-contrib-12.3
2020-05-18 09:40:19+02	2020-05-18 09:54:28+02	databases	postgresql12-server	\N	postgresql12-server-12.3
2020-03-15 05:17:49+01	2020-03-15 05:18:21+01	databases	py-apsw	py27	py27-apsw-3.30.1
2020-04-27 06:14:44+02	2020-04-27 06:15:40+02	databases	py-apsw	py37	py37-apsw-3.31.1
2020-05-21 17:30:52+02	2020-05-21 17:31:13+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-05-21 16:25:29+02	2020-05-21 16:25:40+02	databases	py-redis	py37	py37-redis-3.3.11
2020-03-15 05:18:51+01	2020-03-15 05:19:21+01	databases	py-sqlite3	py27	py27-sqlite3-2.7.17_7
2020-04-26 21:28:56+02	2020-04-26 21:30:07+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.7_7
2020-05-21 16:15:52+02	2020-05-21 16:16:06+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-05-21 00:48:19+02	2020-05-21 00:49:29+02	databases	qt5-sql	\N	qt5-sql-5.14.2
2020-05-21 01:03:25+02	2020-05-21 01:04:50+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.14.2
2020-04-18 20:25:25+02	2020-04-18 20:26:03+02	databases	redis	\N	redis-5.0.9
2020-05-20 12:20:13+02	2020-05-20 12:25:35+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-04-26 01:23:34+02	2020-04-26 01:35:56+02	databases	sqlite3	\N	sqlite3-3.31.1_1,1
2020-04-26 19:20:54+02	2020-04-26 19:32:25+02	databases	tdb	\N	tdb-1.4.2,1
2020-02-29 15:12:34+01	2020-02-29 15:14:38+01	databases	unixODBC	\N	unixODBC-2.3.7
2020-05-20 12:25:35+02	2020-05-20 12:31:31+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-05-24 09:09:44+02	2020-05-24 09:14:29+02	deskutils	calibre	\N	calibre-4.17.0
2020-05-20 08:23:34+02	2020-05-20 08:29:39+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-05-20 08:29:39+02	2020-05-20 08:33:36+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-05-20 12:31:32+02	2020-05-20 12:44:18+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-05-20 12:44:19+02	2020-05-20 12:47:16+02	deskutils	mozo	\N	mozo-1.22.2
2020-05-20 08:33:36+02	2020-05-20 08:38:54+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-05-22 00:00:33+02	2020-05-22 00:01:51+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-04-17 12:42:11+02	2020-04-17 12:42:23+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-04-17 12:42:18+02	2020-04-17 12:42:32+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-02-29 20:06:45+01	2020-02-29 20:07:36+01	devel	adacurses	\N	adacurses-20180127
2020-05-10 12:05:19+02	2020-05-10 12:05:55+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-04-03 06:26:40+02	2020-04-03 06:29:20+02	devel	apr1	\N	apr-1.7.0.1.6.1_1
2020-04-18 09:03:18+02	2020-04-18 09:03:57+02	devel	argp-standalone	\N	argp-standalone-1.3_4
2020-04-24 00:38:24+02	2020-04-24 00:39:01+02	devel	autoconf-archive	\N	autoconf-archive-0.2019.01.06
2020-02-28 21:56:20+01	2020-02-28 21:56:26+01	devel	autoconf-wrapper	\N	autoconf-wrapper-20131203
2020-03-19 20:20:31+01	2020-03-19 20:20:56+01	devel	autoconf	\N	autoconf-2.69_3
2020-03-19 23:09:08+01	2020-03-19 23:09:24+01	devel	autoconf213	\N	autoconf213-2.13.000227_7
2020-04-26 15:34:55+02	2020-04-26 15:36:05+02	devel	automake	\N	automake-1.16.2
2020-02-29 15:26:22+01	2020-02-29 15:28:33+01	devel	binutils	mips64	mips64-binutils-2.33.1_2,1
2020-02-28 22:08:53+01	2020-02-28 22:12:45+01	devel	binutils	native	binutils-2.33.1_2,1
2020-02-29 15:28:34+01	2020-02-29 15:30:28+01	devel	binutils	powerpc64	powerpc64-binutils-2.33.1_2,1
2020-02-29 15:30:28+01	2020-02-29 15:32:15+01	devel	binutils	riscv64	riscv64-binutils-2.33.1_2,1
2020-02-29 15:32:16+01	2020-02-29 15:33:52+01	devel	binutils	sparc64	sparc64-binutils-2.33.1_2,1
2020-02-29 15:23:35+01	2020-02-29 15:26:22+01	devel	binutils	aarch64	aarch64-binutils-2.33.1_2,1
2020-04-09 11:02:19+02	2020-04-09 11:04:25+02	devel	bison	\N	bison-3.5.4,1
2020-02-29 06:37:40+01	2020-02-29 06:38:11+01	devel	boehm-gc-threaded	\N	boehm-gc-threaded-8.0.4_1
2020-03-01 00:26:09+01	2020-03-01 00:26:43+01	devel	boehm-gc	\N	boehm-gc-8.0.4_1
2020-02-28 22:33:50+01	2020-02-28 22:34:53+01	devel	boost-jam	\N	boost-jam-1.72.0
2020-04-25 01:36:40+02	2020-04-25 02:20:18+02	devel	boost-libs	\N	boost-libs-1.72.0_2
2020-05-19 02:45:41+02	2020-05-19 02:46:54+02	devel	bzr	py27	bzr-2.7.0.6622
2020-04-14 18:51:48+02	2020-04-14 18:51:57+02	devel	ccache	\N	ccache-3.7.1_1
2020-05-16 11:36:35+02	2020-05-16 11:37:34+02	devel	cdecl	\N	cdecl-7.0
2020-02-28 22:02:59+01	2020-02-28 22:03:29+01	devel	check	\N	check-0.14.0
2020-05-23 08:48:50+02	2020-05-23 08:50:28+02	devel	cmake	\N	cmake-3.17.2
2020-02-29 06:38:11+01	2020-02-29 06:39:35+01	devel	cppunit	\N	cppunit-1.14.0_8
2020-02-29 20:07:37+01	2020-02-29 20:07:57+01	devel	cscope	\N	cscope-15.8b_1
2020-03-01 00:58:30+01	2020-03-01 00:58:47+01	devel	ctags	\N	ctags-5.8
2020-03-01 04:23:22+01	2020-03-01 04:23:53+01	devel	cvs+ipv6	\N	cvs+ipv6-1.20120905_3
2020-02-29 15:33:53+01	2020-02-29 15:34:03+01	devel	cvsps	\N	cvsps-2.1_2
2020-02-29 20:07:57+01	2020-02-29 20:08:11+01	devel	cweb	\N	cweb-3.64a.a
2020-05-19 23:54:17+02	2020-05-19 23:55:22+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-04-17 03:12:26+02	2020-04-17 03:14:32+02	devel	dbus	\N	dbus-1.12.16
2020-05-21 00:57:44+02	2020-05-21 00:58:40+02	devel	dconf	\N	dconf-0.28.0
2020-05-21 00:40:22+02	2020-05-21 00:40:43+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-05-22 00:01:53+02	2020-05-22 00:03:30+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-02-29 15:34:03+01	2020-02-29 15:34:36+01	devel	dmake	\N	dmake-4.12.3,1
2020-03-01 00:58:48+01	2020-03-01 00:59:01+01	devel	dotconf	\N	dotconf-1.3_1
2020-05-15 21:32:32+02	2020-05-15 21:32:45+02	devel	double-conversion	\N	double-conversion-3.1.5.19
2020-05-21 16:50:35+02	2020-05-21 16:54:05+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-03-16 17:37:40+01	2020-03-16 17:47:14+01	devel	electron6	\N	electron6-6.1.9
2020-05-23 09:31:35+02	2020-05-23 09:54:34+02	devel	electron7	\N	electron7-7.3.0_1
2020-02-28 21:56:26+01	2020-02-28 21:56:32+01	devel	evdev-proto	\N	evdev-proto-5.3
2020-05-16 19:07:10+02	2020-05-16 19:07:30+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-03-07 03:22:20+01	2020-03-07 03:23:04+01	devel	fpc-fcl-async	\N	fpc-fcl-async-3.0.4_4
2020-03-07 03:18:02+01	2020-03-07 03:18:20+01	devel	fpc-fcl-base	\N	fpc-fcl-base-3.0.4_4
2020-03-07 03:24:24+01	2020-03-07 03:25:29+01	devel	fpc-fcl-db	\N	fpc-fcl-db-3.0.4_6
2020-03-07 03:25:30+01	2020-03-07 03:25:48+01	devel	fpc-fcl-fpcunit	\N	fpc-fcl-fpcunit-3.0.4_4
2020-03-07 03:27:43+01	2020-03-07 03:28:03+01	devel	fpc-fcl-image	\N	fpc-fcl-image-3.0.4_4
2020-03-07 03:23:05+01	2020-03-07 03:23:46+01	devel	fpc-fcl-json	\N	fpc-fcl-json-3.0.4_5
2020-03-07 03:25:49+01	2020-03-07 03:26:07+01	devel	fpc-fcl-net	\N	fpc-fcl-net-3.0.4_4
2020-03-07 03:23:46+01	2020-03-07 03:24:05+01	devel	fpc-fcl-passrc	\N	fpc-fcl-passrc-3.0.4_5
2020-03-07 11:36:38+01	2020-03-07 11:36:58+01	devel	fpc-fcl-registry	\N	fpc-fcl-registry-3.0.4_4
2020-03-07 03:17:48+01	2020-03-07 03:18:02+01	devel	fpc-fcl-res	\N	fpc-fcl-res-3.0.4_4
2020-03-07 03:28:04+01	2020-03-07 03:28:52+01	devel	fpc-fcl-web	\N	fpc-fcl-web-3.0.4_7
2020-03-07 03:18:47+01	2020-03-07 03:19:05+01	devel	fpc-fcl-xml	\N	fpc-fcl-xml-3.0.4_4
2020-03-07 03:29:15+01	2020-03-07 03:30:00+01	devel	fpc-fppkg	\N	fpc-fppkg-3.0.4_6
2020-03-07 12:59:30+01	2020-03-07 12:59:54+01	devel	fpc-fv	\N	fpc-fv-3.0.4_4
2020-03-07 12:59:54+01	2020-03-07 13:00:14+01	devel	fpc-gdbint	\N	fpc-gdbint-3.0.4_4
2020-03-07 03:30:00+01	2020-03-07 03:30:17+01	devel	fpc-regexpr	\N	fpc-regexpr-3.0.4_4
2020-05-19 07:21:41+02	2020-05-19 07:22:22+02	devel	fstrm	\N	fstrm-0.6.0
2020-05-20 04:35:29+02	2020-05-20 04:36:28+02	devel	gamin	\N	gamin-0.1.10_10
2020-02-29 02:16:42+01	2020-02-29 02:16:51+01	devel	gccmakedep	\N	gccmakedep-1.0.3
2020-05-21 01:33:07+02	2020-05-21 01:34:37+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-05-18 22:06:55+02	2020-05-18 22:20:37+02	devel	gdb	py37	gdb-9.1
2020-04-26 15:22:36+02	2020-04-26 15:23:52+02	devel	gettext-runtime	\N	gettext-runtime-0.20.2
2020-04-26 15:25:23+02	2020-04-26 15:30:08+02	devel	gettext-tools	\N	gettext-tools-0.20.2
2020-03-16 22:36:46+01	2020-03-16 22:37:21+01	devel	gflags	\N	gflags-2.2.2_2
2020-05-09 14:22:32+02	2020-05-09 15:17:44+02	devel	ghidra	\N	ghidra-9.1
2020-05-20 06:15:53+02	2020-05-20 06:20:38+02	devel	git	\N	git-2.26.2
2020-05-19 23:37:56+02	2020-05-19 23:51:35+02	devel	glib20	\N	glib-2.56.3_7,1
2020-05-20 02:23:12+02	2020-05-20 02:28:40+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-03-16 22:42:52+01	2020-03-16 22:44:03+01	devel	glog	\N	glog-0.4.0_1
2020-05-15 08:05:50+02	2020-05-15 08:08:50+02	devel	glslang	\N	glslang-8.13.3743
2020-05-21 16:14:56+02	2020-05-21 16:15:20+02	devel	gmake	\N	gmake-4.3_1
2020-04-24 00:39:02+02	2020-04-24 00:39:19+02	devel	gnome-common	\N	gnome-common-3.18.0
2020-05-19 23:55:51+02	2020-05-19 23:57:24+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1,1
2020-05-16 19:07:14+02	2020-05-16 19:09:20+02	devel	google-perftools	\N	google-perftools-2.7_1
2020-02-28 22:12:46+01	2020-02-28 22:13:03+01	devel	gperf	\N	gperf-3.1
2020-05-10 12:05:56+02	2020-05-10 12:07:10+02	devel	gradle5	\N	gradle5-5.0
2020-05-21 01:01:45+02	2020-05-21 01:02:37+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-04-25 15:06:07+02	2020-04-25 15:28:48+02	devel	gsoap	\N	gsoap-2.8.101
2020-05-23 09:05:39+02	2020-05-23 09:09:46+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-05-22 02:16:43+02	2020-05-22 02:21:14+02	devel	hs-cabal-install	\N	hs-cabal-install-3.0.2.0
2020-04-26 01:23:34+02	2020-04-26 01:36:40+02	devel	icu	\N	icu-67.1,1
2020-03-20 02:31:55+01	2020-03-20 02:32:22+01	devel	imake	\N	imake-1.0.8,1
2020-02-28 23:24:44+01	2020-02-28 23:25:03+01	devel	jam	\N	jam-2.6
2020-02-29 06:12:28+01	2020-02-29 06:12:41+01	devel	jansson	\N	jansson-2.12
2020-05-21 23:54:21+02	2020-05-21 23:54:45+02	devel	json-c	\N	json-c-0.14
2020-05-21 01:08:22+02	2020-05-21 01:09:10+02	devel	json-glib	\N	json-glib-1.4.4
2020-02-28 22:07:52+01	2020-02-28 22:08:21+01	devel	jsoncpp	\N	jsoncpp-1.9.2
2020-04-26 01:59:24+02	2020-04-26 02:06:16+02	devel	kBuild	\N	kBuild-0.1.9998_13
2020-05-21 01:24:10+02	2020-05-21 01:25:12+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.70.0_1
2020-05-20 02:37:12+02	2020-05-20 02:38:48+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-05-20 06:17:05+02	2020-05-20 06:19:56+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-05-21 01:52:59+02	2020-05-21 01:54:31+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-05-21 01:42:38+02	2020-05-21 01:43:41+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.70.0
2020-05-20 02:28:41+02	2020-05-20 02:29:53+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-05-20 02:29:55+02	2020-05-20 02:31:47+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-05-20 02:03:36+02	2020-05-20 02:06:21+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-05-20 02:01:54+02	2020-05-20 02:03:35+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-05-21 16:35:56+02	2020-05-21 16:38:33+02	devel	kf5-kio	\N	kf5-kio-5.70.1
2020-05-20 05:11:54+02	2020-05-20 05:14:28+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-05-20 02:37:30+02	2020-05-20 02:39:51+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-05-20 06:19:56+02	2020-05-20 06:22:45+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-03-01 00:10:07+01	2020-03-01 00:10:34+01	devel	libatomic_ops	\N	libatomic_ops-7.6.10
2020-02-28 22:40:17+01	2020-02-28 22:40:32+01	devel	libdaemon	\N	libdaemon-0.14_1
2020-05-20 04:36:29+02	2020-05-20 04:37:42+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-04-09 11:01:50+02	2020-04-09 11:02:19+02	devel	libedit	\N	libedit-3.1.20191231,1
2020-02-28 22:26:31+01	2020-02-28 22:26:49+01	devel	libepoll-shim	\N	libepoll-shim-0.0.20200223
2020-02-28 23:21:27+01	2020-02-28 23:21:46+01	devel	libevdev	\N	libevdev-1.5.9_1
2020-05-19 07:10:49+02	2020-05-19 07:12:07+02	devel	libevent	\N	libevent-2.1.11_1
2020-02-28 21:57:31+01	2020-02-28 21:57:46+01	devel	libffi	\N	libffi-3.2.1_3
2020-05-20 06:20:38+02	2020-05-20 06:24:56+02	devel	libgdata	\N	libgdata-0.17.9
2020-05-20 02:06:22+02	2020-05-20 02:08:04+02	devel	libgit2	\N	libgit2-1.0.0
2020-05-20 05:14:29+02	2020-05-20 05:15:16+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-05-21 00:16:59+02	2020-05-21 00:18:27+02	devel	libgsf	\N	libgsf-1.14.46
2020-05-20 04:11:17+02	2020-05-20 04:12:22+02	devel	libgtop	\N	libgtop-2.38.0_2
2020-05-21 00:18:28+02	2020-05-21 00:18:54+02	devel	libgudev	\N	libgudev-230_1
2020-05-21 01:27:20+02	2020-05-21 01:27:49+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-04-25 02:16:31+02	2020-04-25 02:17:06+02	devel	libinotify	\N	libinotify-20180201_2
2020-02-29 09:06:11+01	2020-02-29 09:06:33+01	devel	liblangtag	\N	liblangtag-0.6.2
2020-02-28 22:54:07+01	2020-02-28 22:54:21+01	devel	libltdl	\N	libltdl-2.4.6
2020-02-28 23:21:46+01	2020-02-28 23:21:56+01	devel	libmtdev	\N	libmtdev-1.1.5_2
2020-05-21 01:09:10+02	2020-05-21 01:09:38+02	devel	libnotify	\N	libnotify-0.7.8
2020-04-25 09:35:07+02	2020-04-25 09:52:26+02	devel	liborcus	\N	liborcus-0.14.1_4
2020-05-05 21:09:48+02	2020-05-05 21:10:08+02	devel	libpci	\N	libpci-3.6.4
2020-05-05 19:39:26+02	2020-05-05 19:39:40+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-05-20 06:22:46+02	2020-05-20 06:24:55+02	devel	libpeas	\N	libpeas-1.22.0
2020-02-28 21:59:46+01	2020-02-28 21:59:53+01	devel	libpthread-stubs	\N	libpthread-stubs-0.4
2020-05-20 04:36:56+02	2020-05-20 04:37:51+02	devel	librest	\N	rest-0.7.93
2020-02-29 01:08:00+01	2020-02-29 01:08:33+01	devel	libsigc++20	\N	libsigc++-2.10.2
2020-02-29 16:02:02+01	2020-02-29 16:02:18+01	devel	libsigsegv	\N	libsigsegv-2.12
2020-05-20 08:40:14+02	2020-05-20 08:44:33+02	devel	libslang2	\N	libslang2-2.3.2_2
2020-05-20 04:30:24+02	2020-05-20 04:31:20+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-05-21 01:05:02+02	2020-05-21 01:07:00+02	devel	libsoup	\N	libsoup-2.62.3
2020-03-20 08:58:38+01	2020-03-20 08:59:42+01	devel	libstatgrab	\N	libstatgrab-0.92
2020-02-29 20:16:35+01	2020-02-29 20:16:43+01	devel	libsysctlmibinfo2	\N	libsysctlmibinfo2-2.0.0_1
2020-02-29 16:05:38+01	2020-02-29 16:05:49+01	devel	libsysinfo	\N	libsysinfo-0.0.3_2
2020-04-26 15:22:36+02	2020-04-26 15:25:22+02	devel	libtextstyle	\N	libtextstyle-0.20.2
2020-02-28 22:02:14+01	2020-02-28 22:02:36+01	devel	libtool	\N	libtool-2.4.6_1
2020-02-29 20:16:45+01	2020-02-29 20:16:52+01	devel	libublio	\N	libublio-20070103_2
2020-04-19 09:25:05+02	2020-04-19 09:25:17+02	devel	libudev-devd	\N	libudev-devd-0.4.2
2020-02-28 22:30:31+01	2020-02-28 22:33:23+01	devel	libunistring	\N	libunistring-0.9.10_1
2020-05-16 15:19:02+02	2020-05-16 15:19:32+02	devel	libunwind	\N	libunwind-20200331
2020-05-19 23:32:27+02	2020-05-19 23:33:18+02	devel	libuv	\N	libuv-1.38.0
2020-02-29 01:48:36+01	2020-02-29 01:48:48+01	devel	libvolume_id	\N	libvolume_id-0.81.1
2020-04-27 09:46:23+02	2020-04-27 21:01:42+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200327
2020-04-27 09:47:16+02	2020-04-27 20:59:07+02	devel	llvm10	\N	llvm10-10.0.0_1
2020-03-05 06:49:01+01	2020-03-05 09:30:47+01	devel	llvm70	\N	llvm70-7.0.1_4
2020-04-26 22:07:02+02	2020-04-27 02:15:40+02	devel	llvm80	\N	llvm80-8.0.1_3
2020-04-26 22:09:57+02	2020-04-27 08:48:28+02	devel	llvm90	\N	llvm90-9.0.1_1
2020-02-29 06:41:48+01	2020-02-29 06:42:21+01	devel	m17n-db	\N	m17n-db-1.8.0
2020-05-09 12:13:48+02	2020-05-09 12:14:31+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-02-28 22:01:31+01	2020-02-28 22:02:13+01	devel	m4	\N	m4-1.4.18_1,1
2020-02-29 02:16:52+01	2020-02-29 02:17:01+01	devel	makedepend	\N	makedepend-1.0.6,1
2020-05-07 22:52:37+02	2020-05-07 22:54:36+02	devel	mercurial	py27	mercurial-5.1.2
2020-05-19 23:32:28+02	2020-05-19 23:32:43+02	devel	meson	\N	meson-0.54.2
2020-03-08 18:19:46+01	2020-03-08 18:46:22+01	devel	mips64-gcc	\N	mips64-gcc-6.4.0_8
2020-03-08 21:14:39+01	2020-03-08 21:14:47+01	devel	mips64-xtoolchain-gcc	\N	mips64-xtoolchain-gcc-0.4_1
2020-02-28 22:23:27+01	2020-02-28 22:24:09+01	devel	nasm	\N	nasm-2.14.02,1
2020-02-29 16:05:49+01	2020-02-29 16:07:42+01	devel	ncurses	\N	ncurses-6.1.20190525
2020-03-20 03:13:30+01	2020-03-20 03:14:00+01	devel	netsurf-buildsystem	\N	netsurf-buildsystem-1.8
2020-04-26 15:38:59+02	2020-04-26 15:40:57+02	devel	ninja	\N	ninja-1.10.0,2
2020-02-29 02:17:02+01	2020-02-29 02:17:11+01	devel	npth	\N	npth-1.6
2020-02-28 22:54:21+01	2020-02-28 22:54:57+01	devel	nspr	\N	nspr-4.25
2020-05-16 19:07:30+02	2020-05-16 19:09:14+02	devel	ode	\N	ode-0.13_4,1
2020-03-01 00:28:05+01	2020-03-01 00:28:49+01	devel	oniguruma	\N	oniguruma-6.9.4
2020-03-01 00:04:34+01	2020-03-01 00:04:43+01	devel	opencl	\N	opencl-2.2_1
2020-02-29 16:07:42+01	2020-02-29 16:08:57+01	devel	openmp	\N	openmp-9.0.0
2020-05-21 01:31:23+02	2020-05-21 01:33:06+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-02-28 23:29:11+01	2020-02-28 23:30:04+01	devel	orc	\N	orc-0.4.31
2020-05-14 09:12:19+02	2020-05-14 09:12:40+02	devel	p5-Clone	\N	p5-Clone-0.45
2020-03-20 06:54:43+01	2020-03-20 06:55:02+01	devel	p5-Getopt-ArgvFile	\N	p5-Getopt-ArgvFile-1.11_1
2020-03-20 03:14:00+01	2020-03-20 03:14:27+01	devel	p5-IO-HTML	\N	p5-IO-HTML-1.001_1
2020-03-20 04:26:15+01	2020-03-20 04:26:33+01	devel	p5-IO-String	\N	p5-IO-String-1.08_1
2020-04-26 15:30:08+02	2020-04-26 15:30:35+02	devel	p5-Locale-gettext	\N	p5-Locale-gettext-1.07
2020-05-18 22:04:59+02	2020-05-18 22:05:20+02	devel	p5-Locale-libintl	\N	p5-Locale-libintl-1.31
2020-03-20 03:40:00+01	2020-03-20 03:40:16+01	devel	p5-Parse-Yapp	\N	p5-Parse-Yapp-1.05_2
2020-05-23 21:41:52+02	2020-05-23 21:42:08+02	devel	p5-TimeDate	\N	p5-TimeDate-2.33,1
2020-05-19 23:32:28+02	2020-05-19 23:35:21+02	devel	pcre	\N	pcre-8.44
2020-05-14 07:51:23+02	2020-05-14 07:54:17+02	devel	pcre2	\N	pcre2-10.35
2020-05-16 10:59:00+02	2020-05-16 10:59:22+02	devel	php73-gettext	\N	php73-gettext-7.3.18
2020-05-16 10:57:24+02	2020-05-16 10:57:43+02	devel	php73-json	\N	php73-json-7.3.18
2020-03-02 09:19:31+01	2020-03-02 09:20:02+01	devel	physfs	\N	physfs-3.0.2
2020-02-28 21:56:32+01	2020-02-28 21:56:40+01	devel	pkgconf	\N	pkgconf-1.6.3,1
2020-02-29 04:43:16+01	2020-02-29 04:43:28+01	devel	popt	\N	popt-1.16_2
2020-03-08 18:34:33+01	2020-03-08 19:03:40+01	devel	powerpc64-gcc	\N	powerpc64-gcc-6.4.0_8
2020-03-08 21:14:47+01	2020-03-08 21:14:55+01	devel	powerpc64-xtoolchain-gcc	\N	powerpc64-xtoolchain-gcc-0.4_1
2020-02-29 21:07:03+01	2020-02-29 21:11:09+01	devel	protobuf-c	\N	protobuf-c-1.3.2_4
2020-02-28 23:46:33+01	2020-02-28 23:56:35+01	devel	protobuf	\N	protobuf-3.11.4,1
2020-02-25 17:14:42+01	2020-02-25 17:14:50+01	devel	py-apipkg	py37	py37-apipkg-1.5
2020-05-21 16:25:52+02	2020-05-21 16:26:06+02	devel	py-appdirs	py37	py37-appdirs-1.4.3
2020-04-27 23:33:42+02	2020-04-27 23:33:57+02	devel	py-asn1crypto	py27	py27-asn1crypto-1.3.0
2020-04-26 15:39:38+02	2020-04-26 15:39:55+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-02-25 17:14:08+01	2020-02-25 17:14:16+01	devel	py-atomicwrites	py37	py37-atomicwrites-1.3.0
2020-05-21 16:16:52+02	2020-05-21 16:17:03+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-05-21 16:25:41+02	2020-05-21 16:25:52+02	devel	py-Automat	py37	py37-Automat-0.8.0
2020-05-14 07:51:58+02	2020-05-14 07:52:46+02	devel	py-babel	py27	py27-Babel-2.8.0
2020-05-14 07:50:06+02	2020-05-14 07:50:34+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-05-21 16:38:33+02	2020-05-21 16:38:45+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-03-02 09:20:02+01	2020-03-02 09:20:13+01	devel	py-backports.functools_lru_cache	py27	py27-backports.functools_lru_cache-1.5
2020-03-02 08:43:39+01	2020-03-02 08:43:48+01	devel	py-backports	py27	py27-backports-1
2020-05-21 16:26:06+02	2020-05-21 16:26:14+02	devel	py-backports	py37	py37-backports-1
2020-05-18 22:04:35+02	2020-05-18 22:04:49+02	devel	py-cffi	py27	py27-cffi-1.14.0
2020-05-18 22:04:36+02	2020-05-18 22:04:52+02	devel	py-cffi	py37	py37-cffi-1.14.0
2020-05-21 16:21:13+02	2020-05-21 16:21:28+02	devel	py-click	py37	py37-click-7.0
2020-05-09 13:13:12+02	2020-05-09 13:13:24+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-05-21 16:26:15+02	2020-05-21 16:26:25+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-05-21 17:02:25+02	2020-05-21 17:02:43+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-05-21 16:38:45+02	2020-05-21 16:38:59+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-02-25 17:15:19+01	2020-02-25 17:15:31+01	devel	py-coverage	py37	py37-coverage-4.5.4
2020-04-26 22:10:05+02	2020-04-26 22:10:30+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-03-03 02:25:58+01	2020-03-03 02:26:13+01	devel	py-dateutil	py27	py27-dateutil-2.8.0
2020-05-18 08:19:53+02	2020-05-18 08:20:04+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-03-20 08:59:43+01	2020-03-20 09:00:23+01	devel	py-dbus	py27	py27-dbus-1.2.16
2020-05-20 12:47:17+02	2020-05-20 12:49:50+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-04-27 04:19:14+02	2020-04-27 04:19:57+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-05-21 16:26:25+02	2020-05-21 16:26:35+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-05-21 17:33:02+02	2020-05-21 17:33:15+02	devel	py-dj22-django-rq	py37	py37-dj22-django-rq-2.3.2
2020-04-27 23:33:55+02	2020-04-27 23:34:14+02	devel	py-enum34	py27	py27-enum34-1.1.6
2020-04-26 15:43:57+02	2020-04-26 15:44:29+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-02-25 17:16:34+01	2020-02-25 17:16:44+01	devel	py-freezegun	py37	py37-freezegun-0.3.12
2020-04-27 23:31:32+02	2020-04-27 23:31:50+02	devel	py-funcsigs	py27	py27-funcsigs-1.0.2
2020-05-21 17:02:37+02	2020-05-21 17:02:52+02	devel	py-funcy	py37	py37-funcy-1.12
2020-04-26 15:43:35+02	2020-04-26 15:44:07+02	devel	py-future	py37	py37-future-0.18.2
2020-05-16 19:09:15+02	2020-05-16 19:09:51+02	devel	py-game	py27	py27-game-1.9.6_1
2020-05-20 06:24:55+02	2020-05-20 06:25:39+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-05-20 08:44:34+02	2020-05-20 08:47:16+02	devel	py-gobject3	py27	py27-gobject3-3.28.3
2020-05-21 01:37:43+02	2020-05-21 01:38:07+02	devel	py-gobject3	py37	py37-gobject3-3.28.3
2020-02-25 17:16:35+01	2020-02-25 17:16:46+01	devel	py-hypothesis	py37	py37-hypothesis-4.57.1
2020-05-21 16:26:36+02	2020-05-21 16:26:49+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-0.23
2020-05-21 16:26:49+02	2020-05-21 16:27:02+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-05-21 17:02:43+02	2020-05-21 17:02:56+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-02-25 17:16:45+01	2020-02-25 17:16:57+01	devel	py-invoke	py37	py37-invoke-1.4.1
2020-04-26 19:33:23+02	2020-04-26 19:34:21+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-05-21 16:39:00+02	2020-05-21 16:39:10+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-05-21 16:27:03+02	2020-05-21 16:27:16+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-05-14 09:25:54+02	2020-05-14 09:26:04+02	devel	py-Jinja2	py27	py27-Jinja2-2.10.1
2020-05-14 07:50:35+02	2020-05-14 07:50:48+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-05-14 10:17:05+02	2020-05-14 10:17:17+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-05-21 16:39:11+02	2020-05-21 16:39:25+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-04-27 23:38:12+02	2020-04-27 23:45:23+02	devel	py-lxml	py27	py27-lxml-4.5.0
2020-04-26 15:46:34+02	2020-04-26 15:54:42+02	devel	py-lxml	py37	py37-lxml-4.5.0
2020-02-25 17:16:47+01	2020-02-25 17:16:57+01	devel	py-mock	py37	py37-mock-3.0.5
2020-05-21 16:19:57+02	2020-05-21 16:20:10+02	devel	py-more-itertools	py37	py37-more-itertools-8.3.0
2020-03-03 02:29:25+01	2020-03-03 02:29:43+01	devel	py-msgpack	py27	py27-msgpack-0.6.2
2020-04-27 06:34:29+02	2020-04-27 06:35:31+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-02-25 17:16:57+01	2020-02-25 17:17:17+01	devel	py-nose	py37	py37-nose-1.3.7_1
2020-05-21 16:39:26+02	2020-05-21 16:39:41+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-03-02 09:42:11+01	2020-03-02 09:42:24+01	devel	py-olefile	py27	py27-olefile-0.46
2020-04-26 22:11:56+02	2020-04-26 22:12:23+02	devel	py-olefile	py37	py37-olefile-0.46
2020-05-21 16:14:56+02	2020-05-21 16:15:06+02	devel	py-packaging	py37	py37-packaging-20.4
2020-02-25 17:16:57+01	2020-02-25 17:17:28+01	devel	py-pip	py37	py37-pip-19.1.1
2020-02-25 17:14:15+01	2020-02-25 17:14:22+01	devel	py-pluggy	py37	py37-pluggy-0.12.0
2020-04-27 23:33:26+02	2020-04-27 23:33:42+02	devel	py-ply	py27	py27-ply-3.11
2020-04-26 20:10:04+02	2020-04-26 20:11:07+02	devel	py-ply	py37	py37-ply-3.11
2020-02-25 17:17:17+01	2020-02-25 17:17:29+01	devel	py-pretend	py37	py37-pretend-1.0.9
2020-02-25 17:13:51+01	2020-02-25 17:13:59+01	devel	py-py	py37	py37-py-1.8.1
2020-05-21 16:21:29+02	2020-05-21 16:21:44+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-04-27 23:34:43+02	2020-04-27 23:35:08+02	devel	py-pyasn1	py27	py27-pyasn1-0.4.7
2020-04-27 04:23:26+02	2020-04-27 04:24:14+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-05-18 22:04:26+02	2020-05-18 22:04:35+02	devel	py-pycparser	py27	py27-pycparser-2.20
2020-05-18 22:04:26+02	2020-05-18 22:04:36+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-02-25 17:17:28+01	2020-02-25 17:17:45+01	devel	py-pympler	py37	py37-Pympler-0.4.2
2020-04-26 19:07:11+02	2020-04-26 19:07:59+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-05-21 16:27:17+02	2020-05-21 16:27:31+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-02-25 17:17:29+01	2020-02-25 17:17:45+01	devel	py-pytest-capturelog	py37	py37-pytest-capturelog-0.7
2020-02-25 17:15:31+01	2020-02-25 17:15:41+01	devel	py-pytest-forked	py37	py37-pytest-forked-1.0.2
2020-02-25 17:17:46+01	2020-02-25 17:18:03+01	devel	py-pytest-mock	py37	py37-pytest-mock-1.10.4
2020-02-25 17:17:46+01	2020-02-25 17:18:03+01	devel	py-pytest-rerunfailures	py37	py37-pytest-rerunfailures-7.0
2020-05-21 17:02:53+02	2020-05-21 17:03:09+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-02-25 17:18:03+01	2020-02-25 17:18:15+01	devel	py-pytest-timeout	py37	py37-pytest-timeout-1.3.4
2020-02-25 17:18:03+01	2020-02-25 17:18:16+01	devel	py-pytest-xdist	py37	py37-pytest-xdist-1.29.0
2020-02-25 17:14:24+01	2020-02-25 17:14:33+01	devel	py-pytest	py37	py37-pytest-4.5.0
2020-05-14 07:51:45+02	2020-05-14 07:51:58+02	devel	py-pytz	py27	py27-pytz-2020.1,1
2020-05-14 07:49:55+02	2020-05-14 07:50:06+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-04-26 15:44:30+02	2020-04-26 15:44:57+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-03-20 01:31:51+01	2020-03-20 01:40:22+01	devel	py-qt5-core	py27	py27-qt5-core-5.13.1
2020-05-20 02:02:06+02	2020-05-20 02:08:52+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-02-25 17:14:50+01	2020-02-25 17:14:58+01	devel	py-readme_renderer	py37	py37-readme_renderer-24.0_1
2020-05-21 17:02:56+02	2020-05-21 17:03:14+02	devel	py-rq	py37	py37-rq-1.3.0
2020-05-21 17:03:10+02	2020-05-21 17:03:48+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.87
2020-02-25 17:18:15+01	2020-02-25 17:18:28+01	devel	py-scripttest	py37	py37-scripttest-1.3.0
2020-02-25 17:15:41+01	2020-02-25 17:15:51+01	devel	py-semantic_version	py37	py37-semantic_version-2.8.4
2020-05-21 17:03:15+02	2020-05-21 17:03:29+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-03-02 08:41:10+01	2020-03-02 08:41:20+01	devel	py-setuptools_scm	py27	py27-setuptools_scm-3.4.3
2020-04-26 19:34:22+02	2020-04-26 19:35:10+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-04-27 23:30:23+02	2020-04-27 23:30:51+02	devel	py-setuptools	py27	py27-setuptools-44.0.0
2020-04-26 15:38:57+02	2020-04-26 15:39:15+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-05-16 19:09:20+02	2020-05-16 19:09:33+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-02-25 17:18:16+01	2020-02-25 17:18:32+01	devel	py-simplejson	py37	py37-simplejson-3.17.0
2020-03-02 08:14:26+01	2020-03-02 08:15:07+01	devel	py-sip	py27	py27-sip-4.19.19_1,1
2020-04-26 15:45:52+02	2020-04-26 15:46:51+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-04-27 23:31:41+02	2020-04-27 23:32:00+02	devel	py-six	py27	py27-six-1.14.0
2020-04-26 15:39:16+02	2020-04-26 15:39:37+02	devel	py-six	py37	py37-six-1.14.0
2020-02-25 17:15:42+01	2020-02-25 17:15:52+01	devel	py-sortedcontainers	py37	py37-sortedcontainers-2.1.0
2020-05-21 17:03:29+02	2020-05-21 17:03:50+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-04-27 06:36:27+02	2020-04-27 06:37:15+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-02-25 17:18:29+01	2020-02-25 17:18:45+01	devel	py-tox	py37	py37-tox-3.12.1
2020-02-25 17:15:51+01	2020-02-25 17:16:02+01	devel	py-twine	py37	py37-twine-2.0.0
2020-05-21 16:39:42+02	2020-05-21 16:40:49+02	devel	py-twisted	py37	py37-twisted-20.3.0
2020-03-02 08:44:00+01	2020-03-02 08:44:10+01	devel	py-typing	py27	py27-typing-3.7.4.1
2020-05-21 16:27:31+02	2020-05-21 16:27:43+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-02-25 17:15:52+01	2020-02-25 17:16:13+01	devel	py-virtualenv	py37	py37-virtualenv-16.7.5
2020-02-25 17:14:16+01	2020-02-25 17:14:24+01	devel	py-wcwidth	py37	py37-wcwidth-0.1.8
2020-02-25 17:16:03+01	2020-02-25 17:16:13+01	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-04-28 08:08:51+02	2020-04-28 08:09:40+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-05-21 16:22:52+02	2020-05-21 16:23:02+02	devel	py-zipp	py37	py37-zipp-0.6.0_3
2020-05-21 16:27:44+02	2020-05-21 16:27:59+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-04-27 04:24:15+02	2020-04-27 04:27:40+02	devel	pydbus-common	\N	pydbus-common-1.2.16
2020-05-21 01:36:59+02	2020-05-21 01:37:14+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3
2020-05-21 01:53:18+02	2020-05-21 01:56:43+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-05-20 05:15:17+02	2020-05-20 05:17:43+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-04-14 18:59:20+02	2020-04-14 19:07:44+02	devel	qt5-buildtools	\N	qt5-buildtools-5.14.2
2020-05-21 01:07:00+02	2020-05-21 01:08:21+02	devel	qt5-concurrent	\N	qt5-concurrent-5.14.2
2020-05-25 22:37:01+02	2020-05-25 22:38:15+02	dns	bind916	\N	bind916-9.16.3_2
2020-05-21 00:28:47+02	2020-05-21 00:30:55+02	devel	qt5-dbus	\N	qt5-dbus-5.14.2
2020-05-20 06:24:57+02	2020-05-20 06:36:46+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-05-21 01:04:51+02	2020-05-21 01:06:49+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-05-21 01:22:27+02	2020-05-21 01:24:10+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.14.2
2020-05-20 02:38:49+02	2020-05-20 02:53:17+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-04-14 18:51:58+02	2020-04-14 18:59:19+02	devel	qt5-qmake	\N	qt5-qmake-5.14.2
2020-05-21 01:46:56+02	2020-05-21 01:50:12+02	devel	qt5-script	\N	qt5-script-5.14.2
2020-05-21 00:49:30+02	2020-05-21 00:50:59+02	devel	qt5-testlib	\N	qt5-testlib-5.14.2
2020-05-20 02:08:04+02	2020-05-20 02:09:36+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-05-23 01:08:33+02	2020-05-23 01:09:25+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-04-09 12:11:01+02	2020-04-09 12:12:56+02	devel	re2	\N	re2-20200401
2020-03-02 07:07:49+01	2020-03-02 07:08:11+01	devel	readline	\N	readline-8.0.4
2020-03-08 18:48:56+01	2020-03-08 19:19:41+01	devel	riscv64-gcc	\N	riscv64-gcc-8.3.0
2020-03-08 21:14:56+01	2020-03-08 21:15:04+01	devel	riscv64-xtoolchain-gcc	\N	riscv64-xtoolchain-gcc-0.4_1
2020-05-23 21:41:52+02	2020-05-23 22:41:48+02	devel	RStudio	\N	RStudio-1.3.959
2020-05-16 16:13:29+02	2020-05-16 16:13:44+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-05-16 19:09:27+02	2020-05-16 19:09:37+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-05-16 19:09:33+02	2020-05-16 19:09:42+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-05-16 19:09:38+02	2020-05-16 19:09:47+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-05-20 12:49:51+02	2020-05-20 12:58:52+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.2
2020-04-27 23:32:47+02	2020-04-27 23:33:22+02	devel	scons	py27	scons-3.1.1
2020-05-16 15:51:55+02	2020-05-16 15:52:29+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-05-16 16:10:24+02	2020-05-16 16:11:38+02	devel	sdl20	\N	sdl2-2.0.10_1
2020-03-08 19:15:14+01	2020-03-08 19:41:47+01	devel	sparc64-gcc	\N	sparc64-gcc-6.4.0_8
2020-03-08 21:15:04+01	2020-03-08 21:15:12+01	devel	sparc64-xtoolchain-gcc	\N	sparc64-xtoolchain-gcc-0.4_1
2020-04-26 22:12:59+02	2020-04-26 22:13:40+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-05-15 08:05:50+02	2020-05-15 08:18:19+02	devel	spirv-tools	\N	spirv-tools-2020.2
2020-05-24 11:15:38+02	2020-05-24 11:19:39+02	devel	subversion	\N	subversion-1.13.0_1
2020-05-19 23:35:21+02	2020-05-19 23:37:56+02	devel	swig30	\N	swig30-3.0.12_1
2020-04-17 03:21:54+02	2020-04-17 03:23:20+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-04-26 19:07:56+02	2020-04-26 19:15:46+02	devel	talloc	\N	talloc-2.3.0
2020-04-03 07:50:31+02	2020-04-03 08:02:03+02	devel	tbb	\N	tbb-2020.2
2020-04-26 19:37:08+02	2020-04-26 19:45:22+02	devel	tevent	\N	tevent-0.10.1
2020-02-28 23:46:34+01	2020-02-28 23:46:59+01	devel	tex-kpathsea	\N	tex-kpathsea-6.2.1_2
2020-02-29 09:07:22+01	2020-02-29 09:07:41+01	devel	tex-libtexlua	\N	tex-libtexlua-5.2.4
2020-02-29 09:07:38+01	2020-02-29 09:08:36+01	devel	tex-libtexluajit	\N	tex-libtexluajit-2.0.3
2020-02-29 21:59:15+01	2020-02-29 22:00:01+01	devel	tex-synctex	\N	tex-synctex-1.17.0_1
2020-05-21 01:06:49+02	2020-05-21 01:09:27+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-05-22 00:04:03+02	2020-05-22 00:04:40+02	devel	tig	\N	tig-2.5.1
2020-02-29 02:07:32+01	2020-02-29 02:07:41+01	devel	tradcpp	\N	tradcpp-0.5.3
2020-02-29 16:15:58+01	2020-02-29 16:16:11+01	devel	ucpp	\N	ucpp-1.3.2,1
2020-02-29 22:00:01+01	2020-02-29 22:01:01+01	devel	unittest-cpp	\N	unittest-cpp-2.0.0
2020-05-15 08:05:50+02	2020-05-15 08:06:10+02	devel	vulkan-headers	\N	vulkan-headers-1.2.135.0
2020-03-01 00:26:43+01	2020-03-01 00:27:13+01	devel	woff2	\N	woff2-1.0.2_4
2020-04-17 03:16:39+02	2020-04-17 03:17:19+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-02-28 21:59:08+01	2020-02-28 21:59:15+01	devel	xorg-macros	\N	xorg-macros-1.19.2
2020-02-29 06:13:17+01	2020-02-29 06:13:30+01	devel	yajl	\N	yajl-2.1.0
2020-02-29 01:49:24+01	2020-02-29 01:49:58+01	devel	yasm	\N	yasm-1.3.0
2020-04-24 00:37:51+02	2020-04-24 00:38:25+02	devel	zziplib	\N	zziplib-0.13.71
2020-03-08 18:17:33+01	2020-03-08 18:34:32+01	devel	aarch64-gcc	\N	aarch64-gcc-6.4.0_8
2020-03-08 21:14:30+01	2020-03-08 21:14:38+01	devel	aarch64-xtoolchain-gcc	\N	aarch64-xtoolchain-gcc-0.4_1
2020-05-23 08:50:28+02	2020-05-23 08:51:43+02	dns	bind-tools	\N	bind-tools-9.16.3_1
2020-05-23 09:33:09+02	2020-05-23 09:34:50+02	dns	bind916	\N	bind916-9.16.3_1
2020-02-29 06:13:31+01	2020-02-29 06:15:15+01	dns	c-ares	\N	c-ares-1.15.0_1
2020-02-29 16:16:11+01	2020-02-29 16:18:09+01	dns	ldns	\N	ldns-1.7.1_1
2020-02-28 22:53:03+01	2020-02-28 22:54:06+01	dns	libidn	\N	libidn-1.35
2020-02-28 22:33:23+01	2020-02-28 22:33:49+01	dns	libidn2	\N	libidn2-2.3.0_1
2020-04-27 23:39:50+02	2020-04-27 23:40:10+02	dns	py-dnspython	py27	py27-dnspython-1.16.0
2020-04-27 06:43:26+02	2020-04-27 06:44:24+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-03-02 08:32:37+01	2020-03-02 08:32:51+01	dns	py-idna	py27	py27-idna-2.8
2020-04-26 15:40:58+02	2020-04-26 15:41:25+02	dns	py-idna	py37	py37-idna-2.8
2020-04-27 23:30:22+02	2020-04-27 23:31:51+02	dns	py-ldns	py27	py27-ldns-1.7.1_1
2020-04-27 23:45:24+02	2020-04-27 23:45:47+02	dns	sshfp	\N	sshfp-1.2.2_5
2020-05-22 00:10:43+02	2020-05-22 00:37:55+02	editors	codelite	\N	codelite-12.0_4
2020-05-20 06:35:14+02	2020-05-20 06:42:16+02	editors	emacs	full	emacs-26.3_4,3
2020-03-07 13:43:46+01	2020-03-07 13:44:28+01	editors	fpc-ide	\N	fpc-ide-3.0.4_4
2020-05-20 12:58:53+02	2020-05-20 13:39:13+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-05-21 17:33:15+02	2020-05-21 17:35:06+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-05-21 17:03:49+02	2020-05-21 17:23:17+02	editors	libreoffice	\N	libreoffice-6.3.6
2020-05-23 09:20:44+02	2020-05-23 09:23:59+02	editors	pluma	\N	pluma-1.22.2
2020-05-22 00:43:26+02	2020-05-22 00:50:45+02	editors	vim	\N	vim-8.2.0491_1
2020-05-23 10:05:16+02	2020-05-23 10:41:09+02	editors	vscode	\N	vscode-1.45.1_1
2020-05-16 19:17:37+02	2020-05-16 19:19:53+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-05-21 22:41:37+02	2020-05-21 22:46:49+02	emulators	qemu	\N	qemu-4.1.1_3
2020-03-08 12:08:34+01	2020-03-08 12:08:49+01	emulators	tpm-emulator	\N	tpm-emulator-0.7.4_2
2020-03-02 00:49:28+01	2020-03-02 00:50:45+01	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.34
2020-05-21 00:50:47+02	2020-05-21 02:12:37+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-03-01 06:32:24+01	2020-03-01 06:33:06+01	emulators	wine-gecko	\N	wine-gecko-2.47.1
2020-03-01 06:33:07+01	2020-03-01 06:34:21+01	emulators	wine-mono	\N	wine-mono-4.9.4_1
2020-05-23 09:34:50+02	2020-05-23 09:42:28+02	emulators	wine	\N	wine-5.0_1,1
2020-05-09 14:52:56+02	2020-05-09 14:53:40+02	emulators	x48	\N	x48-0.6.4_1
2020-05-16 15:19:02+02	2020-05-16 15:21:03+02	ftp	curl	\N	curl-7.70.0_1
2020-05-21 02:41:19+02	2020-05-21 03:03:04+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-05-16 17:38:35+02	2020-05-16 17:38:54+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-05-18 22:08:35+02	2020-05-18 22:08:58+02	ftp	libfilezilla	\N	libfilezilla-0.15.1_3
2020-05-16 15:54:21+02	2020-05-16 15:54:41+02	ftp	php73-curl	\N	php73-curl-7.3.18
2020-03-01 06:50:18+01	2020-03-01 06:51:58+01	ftp	wget	\N	wget-1.20.3
2020-04-17 13:33:16+02	2020-04-17 13:33:25+02	games	affenspiel	\N	affenspiel-1.0_2
2020-05-16 20:46:34+02	2020-05-16 20:46:51+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-03-01 00:51:33+01	2020-03-01 01:03:48+01	games	alienarena-data	\N	alienarena-data-2013.766
2020-05-16 20:46:51+02	2020-05-16 20:48:56+02	games	alienarena	\N	alienarena-2013.766_9
2020-05-16 20:47:43+02	2020-05-16 20:48:29+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-05-16 20:48:30+02	2020-05-16 20:50:35+02	games	battletanks	\N	btanks-0.9.8083_9
2020-04-25 15:14:57+02	2020-04-25 15:21:35+02	games	gnuchess	\N	gnuchess-6.2.6
2020-05-16 20:46:22+02	2020-05-16 20:46:34+02	games	KnightCap	\N	knightcap-3.6_8
2020-05-16 20:48:56+02	2020-05-16 20:49:54+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-05-21 03:03:05+02	2020-05-21 03:04:46+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-05-21 03:04:47+02	2020-05-21 03:06:51+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-05-16 20:50:36+02	2020-05-16 20:51:19+02	games	outerspace	\N	outerspace-0.2.2_9
2020-05-16 20:51:14+02	2020-05-16 20:51:28+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-03-30 01:09:35+02	2020-03-30 01:10:47+02	games	stockfish	\N	stockfish-11
2020-05-16 20:51:20+02	2020-05-16 20:52:11+02	games	xblast	\N	xblast-2.10.4_17
2020-05-21 03:06:53+02	2020-05-21 03:09:53+02	games	xboard	\N	xboard-4.9.1_1
2020-05-25 22:37:21+02	2020-05-25 22:38:41+02	devel	qt5-core	\N	qt5-core-5.14.2_3
2020-05-25 22:49:14+02	2020-05-25 22:49:32+02	comms	qt5-serialport	\N	qt5-serialport-5.14.2
2020-05-25 22:55:16+02	2020-05-25 22:56:14+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-05-25 23:05:16+02	2020-05-25 23:06:11+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-05-25 23:49:39+02	2020-05-25 23:54:30+02	net	wireshark	\N	wireshark-3.2.4
2020-05-25 22:38:42+02	2020-05-25 22:39:48+02	net	qt5-network	\N	qt5-network-5.14.2
2020-05-25 22:48:54+02	2020-05-25 22:49:42+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-05-25 22:55:13+02	2020-05-25 22:56:43+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.70.0
2020-05-25 23:05:11+02	2020-05-25 23:06:17+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-05-25 23:52:22+02	2020-05-25 23:54:52+02	print	lyx	\N	lyx-2.3.4.2_1
2020-05-25 22:39:49+02	2020-05-25 22:41:23+02	devel	qt5-dbus	\N	qt5-dbus-5.14.2
2020-05-25 22:49:32+02	2020-05-25 22:50:00+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.70.0
2020-05-25 22:56:14+02	2020-05-25 22:56:54+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-05-25 23:06:11+02	2020-05-25 23:06:58+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-05-25 23:54:31+02	2020-05-25 23:55:24+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_3
2020-05-25 22:38:16+02	2020-05-25 22:43:27+02	graphics	inkscape	\N	inkscape-1.0_1
2020-05-25 22:49:42+02	2020-05-25 22:50:06+02	archivers	kf5-karchive	\N	kf5-karchive-5.70.0
2020-05-25 22:56:28+02	2020-05-25 22:57:11+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-05-25 23:06:17+02	2020-05-25 23:07:02+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-05-25 23:55:39+02	2020-05-25 23:57:07+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-05-25 22:41:24+02	2020-05-25 22:44:14+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-05-25 22:49:33+02	2020-05-25 22:50:15+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-05-25 22:56:55+02	2020-05-25 22:57:54+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-05-25 23:06:59+02	2020-05-25 23:08:05+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-05-25 23:57:08+02	2020-05-25 23:57:23+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-05-25 22:43:29+02	2020-05-25 22:44:22+02	databases	qt5-sql	\N	qt5-sql-5.14.2
2020-05-25 22:50:16+02	2020-05-25 22:50:53+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-05-25 22:57:11+02	2020-05-25 22:58:01+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-05-25 23:07:03+02	2020-05-25 23:10:39+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-05-25 23:57:24+02	2020-05-25 23:58:46+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-05-25 22:44:22+02	2020-05-25 22:44:57+02	devel	qt5-testlib	\N	qt5-testlib-5.14.2
2020-05-25 22:50:01+02	2020-05-25 22:50:54+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-05-25 22:57:55+02	2020-05-25 22:58:36+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-05-25 23:10:41+02	2020-05-25 23:11:52+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-05-25 23:54:53+02	2020-05-26 00:00:19+02	security	tor	\N	tor-0.4.3.5
2020-05-25 22:37:24+02	2020-05-25 22:45:31+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-05-25 22:50:55+02	2020-05-25 22:51:34+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-05-25 22:58:02+02	2020-05-25 22:58:39+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-05-25 23:09:15+02	2020-05-25 23:12:07+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-05-25 23:36:54+02	2020-05-26 00:17:39+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-05-25 22:44:57+02	2020-05-25 22:45:37+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.14.2
2020-05-25 22:50:53+02	2020-05-25 22:51:34+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-05-25 22:58:36+02	2020-05-25 22:59:30+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-05-25 23:13:23+02	2020-05-25 23:14:36+02	graphics	poppler-qt5	\N	poppler-qt5-0.88.0
2020-05-26 00:36:31+02	2020-05-26 00:50:35+02	local	E590T-localbase	\N	E590T-localbase-2020052200
2020-05-25 22:44:15+02	2020-05-25 22:45:49+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-05-25 22:50:54+02	2020-05-25 22:51:49+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-05-25 22:58:40+02	2020-05-25 22:59:31+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-05-25 23:05:22+02	2020-05-25 23:16:59+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-05-26 18:14:01+02	2020-05-26 18:14:09+02	devel	scons	py27	scons-py27-3.1.2
2020-05-25 22:45:31+02	2020-05-25 22:46:07+02	textproc	qt5-xml	\N	qt5-xml-5.14.2
2020-05-25 22:51:34+02	2020-05-25 22:52:15+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-05-25 22:59:31+02	2020-05-25 23:00:50+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-05-25 23:17:00+02	2020-05-25 23:18:14+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-05-26 18:20:33+02	2020-05-26 18:20:42+02	devel	scons	py27	scons-py27-3.1.2
2020-05-25 22:45:37+02	2020-05-25 22:46:14+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.14.2
2020-05-25 22:51:35+02	2020-05-25 22:52:29+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-05-25 22:59:30+02	2020-05-25 23:01:04+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-05-25 23:15:49+02	2020-05-25 23:22:02+02	multimedia	vlc	\N	vlc-3.0.10_4,4
2020-05-26 18:29:21+02	2020-05-26 18:29:30+02	devel	scons	py27	scons-py27-3.1.2
2020-05-25 22:46:08+02	2020-05-25 22:46:43+02	devel	qt5-concurrent	\N	qt5-concurrent-5.14.2
2020-05-25 22:51:50+02	2020-05-25 22:52:30+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-05-25 23:00:51+02	2020-05-25 23:01:48+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-05-25 23:21:47+02	2020-05-25 23:22:20+02	www	qt5-websockets	\N	qt5-websockets-5.14.2
2020-05-26 19:36:52+02	2020-05-26 19:37:02+02	devel	py-wheel	py27	py27-wheel-0.30.0_1
2020-05-25 22:46:14+02	2020-05-25 22:47:01+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-05-25 22:52:16+02	2020-05-25 22:52:57+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-05-25 23:01:05+02	2020-05-25 23:02:08+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-05-25 23:22:21+02	2020-05-25 23:26:05+02	deskutils	calibre	\N	calibre-4.17.0
2020-05-26 19:36:52+02	2020-05-26 19:37:03+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-05-25 22:47:02+02	2020-05-25 22:47:46+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-05-25 22:52:30+02	2020-05-25 22:53:11+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-05-25 23:01:49+02	2020-05-25 23:02:55+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-05-25 23:22:03+02	2020-05-25 23:36:53+02	astro	stellarium	\N	stellarium-0.20.1
2020-05-26 19:37:03+02	2020-05-26 19:37:21+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-05-25 22:46:43+02	2020-05-25 22:47:57+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-05-25 22:52:57+02	2020-05-25 22:53:29+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.70.0
2020-05-25 23:02:09+02	2020-05-25 23:03:43+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-05-25 23:12:38+02	2020-05-25 23:36:57+02	editors	libreoffice	\N	libreoffice-6.3.6
2020-05-26 19:37:03+02	2020-05-26 19:37:24+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-05-25 22:45:49+02	2020-05-25 22:48:02+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-05-25 22:52:30+02	2020-05-25 22:53:48+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-05-25 23:03:44+02	2020-05-25 23:04:14+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-05-25 23:36:59+02	2020-05-25 23:39:12+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-05-26 19:37:22+02	2020-05-26 19:37:35+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-05-25 22:47:47+02	2020-05-25 22:48:17+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-05-25 22:53:12+02	2020-05-25 22:54:08+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-05-25 22:57:54+02	2020-05-25 23:04:22+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-05-25 23:39:12+02	2020-05-25 23:40:17+02	graphics	xpdf	\N	xpdf-4.02,1
2020-05-26 19:37:24+02	2020-05-26 19:37:39+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-05-21 03:09:55+02	2020-05-21 03:12:26+02	games	xcowsay	\N	xcowsay-1.5
2020-04-17 13:41:28+02	2020-04-17 13:41:59+02	games	xevil	\N	xevil-2.02.r2_2
2020-04-17 13:41:59+02	2020-04-17 13:42:13+02	games	xminesweep	\N	xminesweep-3.0_2
2020-04-17 03:16:22+02	2020-04-17 03:22:23+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-05-23 09:20:55+02	2020-05-23 09:23:41+02	graphics	atril	\N	atril-1.22.3_5
2020-05-19 23:51:37+02	2020-05-19 23:55:51+02	graphics	cairo	\N	cairo-1.16.0,2
2020-05-20 03:59:01+02	2020-05-20 04:02:05+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-05-21 00:18:55+02	2020-05-21 00:22:54+02	graphics	colord	\N	colord-1.3.5
2020-05-21 03:12:28+02	2020-05-21 03:24:56+02	graphics	dia	\N	dia-0.97.3_1,1
2020-05-20 08:47:17+02	2020-05-20 08:50:33+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-05-24 16:26:10+02	2020-05-24 16:28:02+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-05-24 16:28:02+02	2020-05-24 16:28:11+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-05-20 19:19:12+02	2020-05-20 19:27:27+02	graphics	eom	\N	eom-1.22.2
2020-04-26 19:08:05+02	2020-04-26 19:14:16+02	graphics	exiv2	\N	exiv2-0.27.2,1
2020-05-20 06:36:47+02	2020-05-20 06:37:12+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-05-16 18:31:22+02	2020-05-16 18:31:37+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-03-07 03:26:08+01	2020-03-07 03:26:25+01	graphics	fpc-pasjpeg	\N	fpc-pasjpeg-3.0.4_4
2020-05-16 16:24:10+02	2020-05-16 16:24:44+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-03-17 07:58:57+01	2020-03-17 07:59:21+01	graphics	frei0r-plugins-gavl	\N	frei0r-plugins-gavl-1.6.1
2020-05-23 09:09:48+02	2020-05-23 09:12:32+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-05-23 09:23:42+02	2020-05-23 09:24:59+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-05-21 01:11:50+02	2020-05-21 01:13:11+02	graphics	frei0r	\N	frei0r-1.6.1
2020-05-16 19:25:29+02	2020-05-16 19:25:47+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-05-09 11:14:32+02	2020-05-09 11:14:53+02	graphics	gd	\N	libgd-2.3.0,1
2020-05-21 00:12:43+02	2020-05-21 00:13:51+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-05-24 16:27:18+02	2020-05-24 16:28:17+02	graphics	gegl	\N	gegl-0.4.22_5
2020-05-24 16:26:10+02	2020-05-24 16:27:18+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-02-28 22:43:41+01	2020-02-28 22:43:51+01	graphics	giflib	\N	giflib-5.2.1
2020-05-24 16:28:17+02	2020-05-24 16:31:30+02	graphics	gimp-app	\N	gimp-app-2.10.18_6,1
2020-05-24 16:32:33+02	2020-05-24 16:33:18+02	graphics	gimp	\N	gimp-2.10.18,2
2020-05-16 19:25:47+02	2020-05-16 19:26:05+02	graphics	gle	\N	gle-3.1.0_7
2020-05-16 17:41:47+02	2020-05-16 17:44:23+02	graphics	glew	\N	glew-2.1.0
2020-05-16 19:26:06+02	2020-05-16 19:26:20+02	graphics	gltt	\N	gltt-2.5.2_14
2020-05-20 19:30:34+02	2020-05-20 19:34:38+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-05-07 21:15:56+02	2020-05-07 21:18:45+02	graphics	gpu-firmware-kmod	\N	gpu-firmware-kmod-g20200503
2020-05-21 01:43:30+02	2020-05-21 01:44:42+02	graphics	graphene	\N	graphene-1.10.0
2020-05-15 21:32:32+02	2020-05-15 21:35:18+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-04-01 07:55:16+02	2020-04-01 07:56:05+02	graphics	graphite2	\N	graphite2-1.3.14
2020-05-21 00:13:28+02	2020-05-21 00:19:16+02	graphics	graphviz	\N	graphviz-2.44.0_3
2020-05-21 01:47:19+02	2020-05-21 01:50:04+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-05-20 04:11:27+02	2020-05-20 04:12:23+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-05-21 00:14:22+02	2020-05-21 00:15:22+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-05-14 07:53:00+02	2020-05-14 07:54:26+02	graphics	ilmbase	\N	ilmbase-2.5.1
2020-04-26 16:49:56+02	2020-04-26 17:02:10+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-05-21 01:37:08+02	2020-05-21 01:42:37+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-05-23 00:01:52+02	2020-05-23 00:11:29+02	graphics	inkscape	\N	inkscape-1.0_1
2020-03-01 00:00:33+01	2020-03-01 00:00:55+01	graphics	jasper	\N	jasper-2.0.16_1
2020-03-28 15:43:19+01	2020-03-28 15:43:41+01	graphics	jbig2dec	\N	jbig2dec-0.18
2020-02-28 22:30:06+01	2020-02-28 22:30:15+01	graphics	jbigkit	\N	jbigkit-2.1_1
2020-05-21 17:08:26+02	2020-05-21 17:08:35+02	graphics	jgraph	\N	jgraph-8.3_1
2020-02-28 22:27:18+01	2020-02-28 22:27:58+01	graphics	jpeg-turbo	\N	jpeg-turbo-2.0.4
2020-03-01 00:10:35+01	2020-03-01 00:11:00+01	graphics	lcms	\N	lcms-1.19_6,1
2020-02-28 22:55:33+01	2020-02-28 22:56:18+01	graphics	lcms2	\N	lcms2-2.9
2020-03-16 22:36:46+01	2020-03-16 22:40:07+01	graphics	leptonica	\N	leptonica-1.76.0_1
2020-03-01 01:06:58+01	2020-03-01 01:07:31+01	graphics	libart_lgpl	\N	libart_lgpl-2.3.21_3,1
2020-04-25 07:56:01+02	2020-04-25 07:58:38+02	graphics	libcdr01	\N	libcdr01-0.1.5_8
2020-05-05 19:39:41+02	2020-05-05 19:40:23+02	graphics	libdrm	\N	libdrm-2.4.99,1
2020-05-16 15:27:17+02	2020-05-16 15:27:42+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-04-25 09:52:28+02	2020-04-25 10:14:39+02	graphics	libetonyek01	\N	libetonyek01-0.1.9_5,1
2020-03-01 00:53:42+01	2020-03-01 00:54:04+01	graphics	libexif	\N	libexif-0.6.21_5
2020-04-25 09:57:12+02	2020-04-25 10:02:25+02	graphics	libfreehand	\N	libfreehand-0.1.2_18
2020-05-16 18:31:41+02	2020-05-16 18:32:24+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-05-16 15:31:25+02	2020-05-16 15:31:46+02	graphics	libGLU	\N	libGLU-9.0.1
2020-05-09 12:22:07+02	2020-05-09 12:22:55+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-05-23 09:12:33+02	2020-05-23 09:13:33+02	graphics	libgxps	\N	libgxps-0.3.1
2020-05-21 01:36:28+02	2020-05-21 01:36:42+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-03-01 00:27:13+01	2020-03-01 00:27:46+01	graphics	libmng	\N	libmng-1.0.10_3
2020-05-23 08:48:52+02	2020-05-23 08:49:06+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-02-29 06:48:56+01	2020-02-29 06:49:11+01	graphics	libnsgif	\N	libnsgif-0.2.1
2020-05-16 18:32:45+02	2020-05-16 18:33:01+02	graphics	libplacebo	\N	libplacebo-2.43.1
2020-02-28 23:57:46+01	2020-02-28 23:58:00+01	graphics	libpotrace	\N	libpotrace-1.12
2020-02-28 23:58:00+01	2020-02-28 23:59:08+01	graphics	libraw	\N	libraw-0.19.5
2020-05-21 00:23:18+02	2020-05-21 00:24:43+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-02-29 06:15:16+01	2020-02-29 06:15:52+01	graphics	libspiro	\N	libspiro-20190731,1
2020-04-17 03:38:55+02	2020-04-17 03:40:17+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-05-26 19:39:51+02	2020-05-26 20:03:10+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.p3
2020-05-26 20:03:11+02	2020-05-26 20:03:21+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-05-26 19:37:40+02	2020-05-26 22:15:50+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200523
2020-05-26 19:52:00+02	2020-05-26 22:19:33+02	www	firefox	\N	firefox-77.0,1
2020-05-26 22:40:38+02	2020-05-26 22:55:43+02	local	E590T-localbase	\N	E590T-localbase-2020052200
2020-05-27 07:44:57+02	2020-05-27 07:45:06+02	dns	py-idna	py27	py27-idna-2.8
2020-05-27 07:44:57+02	2020-05-27 07:45:06+02	net	py-pysocks	py27	py27-pysocks-1.7.1
2020-05-27 07:45:06+02	2020-05-27 07:45:15+02	security	py-certifi	py27	py27-certifi-2020.4.5.1
2020-05-27 07:45:06+02	2020-05-27 07:45:15+02	security	py-openssl	py27	py27-openssl-19.0.0
2020-05-27 07:45:15+02	2020-05-27 07:45:24+02	net	py-urllib3	py27	py27-urllib3-1.25.7,1
2020-05-27 07:45:15+02	2020-05-27 07:45:26+02	devel	py-future	py27	py27-future-0.18.2
2020-05-27 07:45:26+02	2020-05-27 07:45:35+02	textproc	py-chardet	py27	py27-chardet-3.0.4_3
2020-05-27 07:45:24+02	2020-05-27 07:45:35+02	sysutils	py-psutil	py27	py27-psutil-5.7.0
2020-05-27 07:45:36+02	2020-05-27 07:46:24+02	devel	cdecl	\N	cdecl-7.1
2020-05-27 07:46:25+02	2020-05-27 07:46:38+02	net	py-netifaces	py27	py27-netifaces-0.10.9
2020-05-27 07:46:38+02	2020-05-27 07:46:56+02	sysutils	py-glances	py27	py27-glances-3.1.4
2020-05-27 07:46:56+02	2020-05-27 07:47:08+02	sysutils	py-py-cpuinfo	py27	py27-py-cpuinfo-5.0.0
2020-05-27 07:47:08+02	2020-05-27 07:47:19+02	www	py-bottle	py27	py27-bottle-0.12.15_1
2020-05-27 07:47:20+02	2020-05-27 07:47:31+02	www	py-requests	py27	py27-requests-2.22.0
2020-05-27 07:45:36+02	2020-05-27 07:48:23+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-05-27 07:48:23+02	2020-05-27 07:48:31+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-05-27 07:44:57+02	2020-05-27 08:52:33+02	lang	ghc	\N	ghc-8.8.3_1
2020-05-27 09:06:45+02	2020-05-27 09:20:55+02	local	E590T-localbase	\N	E590T-localbase-2020052600
2020-05-27 23:26:36+02	2020-05-27 23:28:41+02	devel	google-perftools	\N	google-perftools-2.7.90
2020-05-27 23:28:41+02	2020-05-27 23:32:41+02	security	klee	\N	klee-2.1
2020-05-27 23:26:36+02	2020-05-27 23:34:13+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-05-27 23:34:13+02	2020-05-27 23:34:20+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-05-27 23:26:36+02	2020-05-28 00:55:37+02	www	firefox	\N	firefox-77.0_1,1
2020-05-28 01:15:18+02	2020-05-28 01:29:42+02	local	E590T-localbase	\N	E590T-localbase-2020052600
2020-05-28 23:58:09+02	2020-05-28 23:58:22+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.10
2020-05-28 23:58:09+02	2020-05-28 23:58:31+02	sysutils	libcpuid	\N	libcpuid-0.5.0
2020-05-28 23:58:23+02	2020-05-28 23:58:37+02	www	p5-libwww	\N	p5-libwww-6.44
2020-05-28 23:58:32+02	2020-05-29 00:05:17+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-05-28 23:58:09+02	2020-05-29 00:05:58+02	devel	gsoap	\N	gsoap-2.8.103
2020-05-29 00:05:17+02	2020-05-29 00:06:07+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-05-29 00:05:59+02	2020-05-29 00:06:51+02	www	links	\N	links-2.20.2,1
2020-05-29 00:06:51+02	2020-05-29 00:06:59+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-05-29 00:06:07+02	2020-05-29 00:07:47+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-05-28 23:58:37+02	2020-05-29 00:23:21+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-05-29 00:44:03+02	2020-05-29 00:59:05+02	local	E590T-localbase	\N	E590T-localbase-2020052600
2020-05-30 11:16:14+02	2020-05-30 11:16:31+02	security	ca_root_nss	\N	ca_root_nss-3.53
2020-05-30 11:16:14+02	2020-05-30 11:16:51+02	devel	oniguruma	\N	oniguruma-6.9.5.r1
2020-05-30 11:16:31+02	2020-05-30 11:17:04+02	textproc	py-sphinx	py37	py37-sphinx-3.0.3,1
2020-05-30 11:17:05+02	2020-05-30 11:17:22+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-05-30 11:17:23+02	2020-05-30 11:18:27+02	security	p11-kit	\N	p11-kit-0.23.20
2020-05-30 11:16:14+02	2020-05-30 11:18:32+02	security	nss	\N	nss-3.53
2020-05-30 11:16:51+02	2020-05-30 11:19:07+02	ftp	curl	\N	curl-7.70.0_1
2020-05-30 11:18:34+02	2020-05-30 11:24:02+02	net	qt5-network	\N	qt5-network-5.14.2
2020-05-30 11:18:27+02	2020-05-30 11:24:57+02	security	gnutls	\N	gnutls-3.6.13_1
2020-05-30 11:24:57+02	2020-05-30 11:28:10+02	print	cups	\N	cups-2.2.13
2020-05-30 11:28:11+02	2020-05-30 11:37:30+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_4
2020-05-30 11:19:10+02	2020-05-30 11:41:07+02	devel	cmake	\N	cmake-3.17.2
2020-05-30 11:24:07+02	2020-05-30 11:43:47+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-05-30 11:41:10+02	2020-05-30 11:51:36+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-05-30 11:43:49+02	2020-05-30 11:53:24+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-05-30 11:37:32+02	2020-05-30 11:53:38+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-05-30 11:53:39+02	2020-05-30 11:54:40+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-05-30 11:53:24+02	2020-05-30 11:56:19+02	graphics	poppler	\N	poppler-0.88.0
2020-05-30 11:54:42+02	2020-05-30 11:57:08+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-05-30 11:56:21+02	2020-05-30 11:58:19+02	devel	libsoup	\N	libsoup-2.62.3
2020-05-30 11:51:37+02	2020-05-30 11:59:34+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.14.2
2020-05-30 11:58:20+02	2020-05-30 12:00:20+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-05-30 11:57:09+02	2020-05-30 12:07:17+02	print	texlive-base	\N	texlive-base-20150521_51
2020-05-30 12:00:21+02	2020-05-30 12:22:04+02	multimedia	ffmpeg	\N	ffmpeg-4.2.3,1
2020-05-30 12:22:04+02	2020-05-30 12:23:39+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-05-30 12:23:40+02	2020-05-30 12:24:19+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-05-30 12:24:20+02	2020-05-30 12:25:06+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-05-30 11:59:35+02	2020-05-30 12:25:07+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-05-30 12:25:08+02	2020-05-30 12:25:38+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-05-30 12:25:38+02	2020-05-30 12:27:32+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-05-30 12:27:34+02	2020-05-30 12:28:25+02	converters	php73-mbstring	\N	php73-mbstring-7.3.18
2020-05-30 12:28:26+02	2020-05-30 12:29:48+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-05-30 12:29:49+02	2020-05-30 12:30:19+02	ftp	php73-curl	\N	php73-curl-7.3.18
2020-05-30 12:25:06+02	2020-05-30 12:31:01+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-05-30 12:30:19+02	2020-05-30 12:31:32+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-05-30 12:31:02+02	2020-05-30 12:31:53+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-05-30 12:31:32+02	2020-05-30 12:32:07+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-05-30 12:07:19+02	2020-05-30 12:32:50+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-05-30 12:31:54+02	2020-05-30 12:32:54+02	net	geoclue	\N	geoclue-2.5.5
2020-05-30 12:32:08+02	2020-05-30 12:33:06+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-05-30 12:33:07+02	2020-05-30 12:34:04+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-05-30 12:34:04+02	2020-05-30 12:34:51+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-05-30 12:32:51+02	2020-05-30 12:35:16+02	print	tex-formats	\N	tex-formats-20150521_2
2020-05-30 12:34:52+02	2020-05-30 12:36:36+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-05-30 12:35:17+02	2020-05-30 12:38:59+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-05-30 12:32:54+02	2020-05-30 12:39:37+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-05-30 12:39:00+02	2020-05-30 12:41:13+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-05-30 12:41:14+02	2020-05-30 12:42:53+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-05-30 12:42:53+02	2020-05-30 12:50:29+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-05-30 12:50:30+02	2020-05-30 13:02:46+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-05-30 12:39:38+02	2020-05-30 13:03:40+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-05-30 13:02:47+02	2020-05-30 13:04:27+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-05-30 13:03:41+02	2020-05-30 13:05:17+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-05-30 13:04:28+02	2020-05-30 13:07:37+02	graphics	poppler-glib	\N	poppler-glib-0.88.0
2020-05-30 13:05:19+02	2020-05-30 13:07:53+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-05-30 13:07:40+02	2020-05-30 13:09:04+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-05-30 13:07:54+02	2020-05-30 13:09:37+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-05-30 13:09:04+02	2020-05-30 13:10:33+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-05-30 13:09:38+02	2020-05-30 13:13:10+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-05-30 13:13:12+02	2020-05-30 13:15:10+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-05-30 13:10:34+02	2020-05-30 13:16:25+02	databases	mysql57-client	\N	mysql57-client-5.7.30_1
2020-05-30 13:15:12+02	2020-05-30 13:16:45+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-05-30 13:16:45+02	2020-05-30 13:18:38+02	net	remmina	\N	remmina-1.3.9
2020-05-30 13:18:40+02	2020-05-30 13:28:20+02	security	gcr	\N	gcr-3.28.0
2020-05-30 13:28:23+02	2020-05-30 13:30:45+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-05-30 13:16:27+02	2020-05-30 13:30:57+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-05-30 13:30:45+02	2020-05-30 13:32:44+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-05-30 13:30:58+02	2020-05-30 13:33:51+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-05-30 13:33:52+02	2020-05-30 13:35:34+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-05-30 13:35:35+02	2020-05-30 13:37:56+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-05-30 13:37:57+02	2020-05-30 14:12:46+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-05-30 14:12:49+02	2020-05-30 14:14:41+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-05-30 14:14:42+02	2020-05-30 14:17:25+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-05-30 14:17:27+02	2020-05-30 14:20:51+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-05-30 14:20:54+02	2020-05-30 14:23:39+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-05-30 14:23:41+02	2020-05-30 14:25:57+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-05-30 14:25:58+02	2020-05-30 14:30:12+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-05-30 14:30:14+02	2020-05-30 14:35:40+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-05-30 14:35:42+02	2020-05-30 14:37:10+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-05-30 14:37:13+02	2020-05-30 14:50:30+02	net	libmateweather	\N	libmateweather-1.22.1
2020-05-30 14:50:33+02	2020-05-30 14:54:50+02	security	gnupg	\N	gnupg-2.2.20
2020-05-30 14:54:52+02	2020-05-30 14:57:59+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-05-30 12:36:37+02	2020-05-30 15:03:17+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-05-30 15:03:19+02	2020-05-30 15:04:58+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-05-30 15:05:01+02	2020-05-30 15:08:29+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-05-30 14:58:02+02	2020-05-30 15:08:52+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-05-30 15:08:32+02	2020-05-30 15:09:19+02	security	gpgme	\N	gpgme-1.13.1
2020-05-30 15:08:52+02	2020-05-30 15:11:20+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-05-30 15:11:21+02	2020-05-30 15:21:23+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-05-30 15:21:24+02	2020-05-30 15:23:45+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-05-30 15:23:48+02	2020-05-30 15:26:02+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-05-30 15:26:03+02	2020-05-30 15:28:24+02	devel	librest	\N	rest-0.7.93
2020-05-30 15:28:26+02	2020-05-30 15:32:12+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-05-30 15:32:13+02	2020-05-30 15:36:40+02	graphics	poppler-utils	\N	poppler-utils-0.88.0
2020-05-30 15:36:41+02	2020-05-30 15:39:30+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-05-30 15:39:32+02	2020-05-30 15:49:58+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-05-30 15:50:00+02	2020-05-30 15:51:33+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-05-30 15:51:37+02	2020-05-30 15:54:03+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-05-30 15:54:06+02	2020-05-30 17:41:44+02	www	node	\N	node-13.13.0_1
2020-05-30 17:41:47+02	2020-05-30 17:45:52+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-05-30 17:45:54+02	2020-05-30 17:50:29+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-05-30 17:50:36+02	2020-05-30 17:56:58+02	x11-fm	caja	\N	caja-1.22.3
2020-05-30 17:56:59+02	2020-05-30 17:59:42+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-05-30 17:59:43+02	2020-05-30 18:02:10+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-05-30 18:02:11+02	2020-05-30 18:03:49+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-05-30 18:03:51+02	2020-05-30 18:06:51+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-05-30 18:06:51+02	2020-05-30 18:10:29+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-05-30 18:10:30+02	2020-05-30 18:12:04+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-05-30 18:12:04+02	2020-05-30 18:25:12+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-05-30 18:25:14+02	2020-05-30 18:30:01+02	graphics	gegl	\N	gegl-0.4.22_5
2020-05-30 18:30:02+02	2020-05-30 18:34:56+02	mail	mailutils	\N	mailutils-3.9
2020-05-30 13:32:45+02	2020-05-30 18:36:22+02	lang	rust	\N	rust-1.43.1
2020-05-30 18:36:26+02	2020-05-30 18:36:47+02	net	liboauth	\N	liboauth-1.0.3_4
2020-05-30 18:34:57+02	2020-05-30 18:38:54+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-05-30 18:38:55+02	2020-05-30 18:40:15+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-05-30 18:40:16+02	2020-05-30 18:41:45+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-05-30 15:09:20+02	2020-05-30 18:48:40+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-05-30 18:41:46+02	2020-05-30 18:49:21+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-05-30 18:48:42+02	2020-05-30 18:51:39+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-05-30 18:49:24+02	2020-05-30 18:51:54+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-05-30 18:51:55+02	2020-05-30 18:54:26+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-05-30 18:51:40+02	2020-05-30 18:55:20+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-05-30 18:54:27+02	2020-05-30 18:56:51+02	x11	zenity	\N	zenity-3.28.1
2020-05-30 18:56:52+02	2020-05-30 19:01:52+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-05-30 19:01:55+02	2020-05-30 19:06:43+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-05-30 18:55:26+02	2020-05-30 19:08:54+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-05-30 18:36:47+02	2020-05-30 19:09:23+02	net	samba410	\N	samba410-4.10.15
2020-05-30 19:08:56+02	2020-05-30 19:10:46+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-05-30 19:06:47+02	2020-05-30 19:11:27+02	devel	git	\N	git-2.26.2
2020-05-30 19:09:28+02	2020-05-30 19:12:05+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-05-30 19:11:28+02	2020-05-30 19:13:01+02	devel	libpeas	\N	libpeas-1.22.0
2020-05-30 19:13:02+02	2020-05-30 19:13:21+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-05-30 19:10:46+02	2020-05-30 19:13:32+02	devel	libgdata	\N	libgdata-0.17.9
2020-05-30 19:12:06+02	2020-05-30 19:18:19+02	editors	emacs	full	emacs-26.3_4,3
2020-05-30 19:18:21+02	2020-05-30 19:21:33+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-05-30 19:21:38+02	2020-05-30 19:22:31+02	net	gssdp	\N	gssdp-1.2.2
2020-05-30 19:22:33+02	2020-05-30 19:24:38+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-05-30 19:24:40+02	2020-05-30 19:27:43+02	print	cups-filters	\N	cups-filters-1.27.4_1
2020-05-30 19:13:21+02	2020-05-30 19:33:09+02	graphics	gimp-app	\N	gimp-app-2.10.18_6,1
2020-05-30 19:33:10+02	2020-05-30 19:34:21+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-05-30 19:34:21+02	2020-05-30 19:36:42+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-05-30 19:36:43+02	2020-05-30 19:39:14+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-05-30 19:39:14+02	2020-05-30 19:40:26+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-05-30 19:13:33+02	2020-05-30 19:43:44+02	graphics	opencv	\N	opencv-3.4.1_31
2020-05-30 19:27:44+02	2020-05-30 19:53:23+02	print	tex-luatex	\N	tex-luatex-0.80.0_17
2020-05-30 19:43:44+02	2020-05-30 21:35:14+02	www	node12	\N	node12-12.16.2_1
2020-05-30 21:35:17+02	2020-05-30 21:38:14+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-05-30 21:38:16+02	2020-05-30 21:42:41+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-05-30 21:42:43+02	2020-05-30 21:47:56+02	x11	yelp	\N	yelp-3.28.1_2
2020-05-30 21:47:58+02	2020-05-30 21:51:55+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-05-30 21:51:56+02	2020-05-30 21:54:36+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-05-30 21:54:38+02	2020-05-30 22:02:13+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-05-30 22:02:14+02	2020-05-30 22:05:26+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-05-30 22:05:29+02	2020-05-30 22:12:25+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-05-30 22:12:27+02	2020-05-30 22:20:51+02	x11-wm	marco	\N	marco-1.22.4
2020-05-30 22:20:53+02	2020-05-30 22:27:47+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-05-30 22:27:49+02	2020-05-30 22:31:17+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-05-30 22:31:20+02	2020-05-30 22:43:57+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-05-30 22:44:01+02	2020-05-30 22:56:06+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-05-30 22:56:08+02	2020-05-30 23:21:45+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-05-30 23:21:55+02	2020-05-30 23:27:30+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-05-30 23:27:32+02	2020-05-30 23:29:48+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-05-30 23:29:50+02	2020-05-31 00:04:03+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-05-31 00:04:06+02	2020-05-31 00:05:51+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-05-31 00:05:53+02	2020-05-31 00:16:06+02	graphics	sane-backends	\N	sane-backends-1.0.30
2020-05-31 00:16:08+02	2020-05-31 00:52:01+02	math	R	\N	R-4.0.0_1
2020-05-31 00:52:04+02	2020-05-31 01:06:38+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-05-31 01:06:40+02	2020-05-31 01:11:13+02	multimedia	mlt	\N	mlt-6.20.0
2020-05-31 01:11:14+02	2020-05-31 01:15:49+02	net	freerdp	\N	freerdp-2.0.0.r4_9
2020-05-31 01:15:51+02	2020-05-31 01:17:04+02	net	gupnp	\N	gupnp-1.1.1_1
2020-05-31 01:17:05+02	2020-05-31 01:36:57+02	net	libcmis	\N	libcmis-0.5.2_1
2020-05-31 01:36:59+02	2020-05-31 01:39:07+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-05-31 01:39:08+02	2020-05-31 01:41:14+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-05-31 01:41:16+02	2020-05-31 01:52:06+02	print	gutenprint	\N	gutenprint-5.3.3
2020-05-31 01:52:08+02	2020-05-31 01:53:55+02	print	libspectre	\N	libspectre-0.2.8
2020-05-31 01:53:57+02	2020-05-31 02:08:41+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-05-31 02:08:44+02	2020-05-31 02:20:54+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-05-31 02:20:56+02	2020-05-31 02:24:46+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-05-31 02:24:48+02	2020-05-31 02:34:04+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-05-31 02:34:06+02	2020-05-31 02:36:40+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-05-31 02:36:41+02	2020-05-31 02:40:12+02	textproc	redland	\N	redland-1.0.17_4
2020-05-31 02:40:14+02	2020-05-31 03:52:50+02	www	node10	\N	node10-10.20.1_1
2020-05-31 03:52:51+02	2020-05-31 03:54:40+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-05-31 03:54:42+02	2020-05-31 03:57:52+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-05-31 03:59:11+02	2020-05-31 04:04:45+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-05-31 04:04:47+02	2020-05-31 04:06:33+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-05-31 04:06:35+02	2020-05-31 04:11:48+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-05-31 04:11:50+02	2020-05-31 04:55:37+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-05-31 04:55:40+02	2020-05-31 05:02:46+02	archivers	engrampa	\N	engrampa-1.22.2
2020-05-31 05:02:49+02	2020-05-31 05:04:34+02	audio	faudio	\N	FAudio-20.05
2020-05-31 05:04:36+02	2020-05-31 05:07:08+02	audio	mate-media	\N	mate-media-1.22.2
2020-05-31 05:07:09+02	2020-05-31 05:10:08+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-05-30 19:53:25+02	2020-05-31 05:13:42+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-05-31 05:10:10+02	2020-05-31 05:15:03+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-05-31 05:13:46+02	2020-05-31 05:15:09+02	deskutils	mozo	\N	mozo-1.22.2
2020-05-31 05:15:03+02	2020-05-31 05:16:14+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-05-31 05:15:09+02	2020-05-31 05:22:01+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.2
2020-05-31 05:16:16+02	2020-05-31 05:53:51+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-05-31 05:53:54+02	2020-05-31 06:04:36+02	editors	pluma	\N	pluma-1.22.2
2020-05-31 06:04:39+02	2020-05-31 06:16:04+02	graphics	atril	\N	atril-1.22.3_5
2020-05-31 06:16:06+02	2020-05-31 06:23:08+02	graphics	eom	\N	eom-1.22.2
2020-05-31 06:23:10+02	2020-05-31 06:26:15+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-05-31 06:26:18+02	2020-05-31 06:30:03+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-05-31 06:30:06+02	2020-05-31 06:38:24+02	graphics	poppler-qt5	\N	poppler-qt5-0.88.0
2020-05-31 06:38:26+02	2020-05-31 06:45:38+02	graphics	py-gimp	\N	py27-gimp-2.10.18_6
2020-05-31 06:45:40+02	2020-05-31 07:12:48+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-05-31 07:12:51+02	2020-05-31 09:32:19+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-05-31 05:22:02+02	2020-05-31 11:53:47+02	editors	libreoffice	\N	libreoffice-6.3.6
2020-05-31 11:53:49+02	2020-05-31 11:56:23+02	math	mate-calc	\N	mate-calc-1.22.2
2020-05-31 11:56:27+02	2020-05-31 11:56:35+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200529
2020-05-31 09:32:23+02	2020-05-31 11:57:25+02	lang	julia11	\N	julia11-1.1.1_5
2020-05-31 11:56:36+02	2020-05-31 12:02:58+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-05-31 12:02:59+02	2020-05-31 12:06:18+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-05-31 12:06:19+02	2020-05-31 12:08:24+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-05-31 12:08:25+02	2020-05-31 12:10:26+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-05-31 12:10:29+02	2020-05-31 12:12:57+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-05-31 12:12:59+02	2020-05-31 12:15:13+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-05-31 12:15:15+02	2020-05-31 12:16:26+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-05-31 12:16:29+02	2020-05-31 12:17:47+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-05-31 11:57:27+02	2020-05-31 12:18:35+02	multimedia	vlc	\N	vlc-3.0.10_4,4
2020-05-31 12:17:47+02	2020-05-31 12:19:26+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-05-31 12:18:36+02	2020-05-31 12:20:47+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-05-31 12:19:27+02	2020-05-31 12:20:56+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-05-31 12:20:50+02	2020-05-31 12:22:50+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-05-31 12:22:50+02	2020-05-31 12:32:54+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-05-31 12:20:56+02	2020-05-31 12:33:56+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-05-31 12:32:55+02	2020-05-31 12:36:11+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-05-31 12:33:57+02	2020-05-31 12:50:51+02	print	tex-xetex	\N	tex-xetex-0.99992_33
2020-05-31 12:50:52+02	2020-05-31 12:51:45+02	print	transfig	\N	transfig-3.2.5e_4
2020-05-31 12:51:46+02	2020-05-31 12:54:38+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-05-31 12:54:39+02	2020-05-31 12:58:27+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-05-31 12:58:28+02	2020-05-31 13:00:11+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-05-31 13:00:12+02	2020-05-31 13:01:36+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-05-31 13:01:37+02	2020-05-31 13:02:25+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-05-31 13:02:26+02	2020-05-31 13:05:31+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-05-31 13:05:32+02	2020-05-31 13:07:48+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-05-31 13:07:48+02	2020-05-31 13:08:50+02	www	qt5-websockets	\N	qt5-websockets-5.14.2
2020-05-31 13:08:51+02	2020-05-31 13:09:07+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-05-31 12:36:12+02	2020-05-31 13:10:49+02	print	texlive-docs	\N	texlive-docs-20150523
2020-05-31 13:10:58+02	2020-05-31 13:13:55+02	x11	mate-base	\N	mate-base-1.22.2
2020-05-31 13:13:57+02	2020-05-31 13:16:27+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-05-31 13:09:08+02	2020-05-31 13:17:36+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-05-31 13:16:27+02	2020-05-31 13:19:28+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-05-31 13:17:37+02	2020-05-31 13:19:54+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-05-31 13:19:29+02	2020-05-31 13:20:34+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-05-31 13:19:54+02	2020-05-31 13:32:29+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-05-31 13:40:55+02	2020-05-31 13:42:26+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-05-31 13:20:34+02	2020-05-31 13:44:57+02	astro	stellarium	\N	stellarium-0.20.1
2020-05-31 13:44:58+02	2020-05-31 13:49:14+02	devel	bzr	py27	bzr-2.7.0.6622
2020-05-31 13:49:19+02	2020-05-31 13:52:40+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-05-31 13:42:27+02	2020-05-31 16:06:21+02	devel	RStudio	\N	RStudio-1.3.959
2020-05-31 16:06:23+02	2020-05-31 16:10:14+02	devel	mercurial	py27	mercurial-5.1.2
2020-05-31 16:10:18+02	2020-05-31 16:11:47+02	devel	tig	\N	tig-2.5.1
2020-05-31 16:11:48+02	2020-05-31 18:17:27+02	editors	codelite	\N	codelite-12.0_4
2020-05-31 18:17:30+02	2020-05-31 18:24:27+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-05-31 18:24:29+02	2020-05-31 18:33:21+02	editors	vim	\N	vim-8.2.0491_1
2020-05-31 18:33:22+02	2020-05-31 20:08:37+02	emulators	qemu	\N	qemu-4.1.1_3
2020-05-31 21:06:54+02	2020-05-31 22:04:04+02	emulators	wine	\N	wine-5.0_1,1
2020-05-31 22:04:10+02	2020-05-31 22:33:58+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-05-31 22:34:01+02	2020-05-31 22:40:14+02	games	alienarena	\N	alienarena-2013.766_9
2020-05-31 22:40:16+02	2020-05-31 22:42:43+02	games	xcowsay	\N	xcowsay-1.5
2020-05-31 22:42:48+02	2020-05-31 22:56:32+02	graphics	dia	\N	dia-0.97.3_1,1
2020-05-31 22:56:38+02	2020-05-31 23:12:05+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-05-31 23:12:07+02	2020-05-31 23:15:08+02	graphics	gimp	\N	gimp-2.10.18,2
2020-05-31 23:15:10+02	2020-06-01 00:43:25+02	graphics	inkscape	\N	inkscape-1.0_1
2020-06-01 00:43:32+02	2020-06-01 00:48:04+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-01 00:48:07+02	2020-06-01 00:50:42+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-01 00:50:44+02	2020-06-01 03:08:17+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-05-31 13:52:46+02	2020-06-01 03:09:05+02	devel	electron7	\N	electron7-7.3.0_1
2020-06-01 03:09:08+02	2020-06-01 03:10:03+02	lang	julia	\N	julia-1.0.3_3
2020-06-01 03:10:05+02	2020-06-01 04:23:18+02	lang	mono	\N	mono-5.10.1.57_3
2020-06-01 03:08:18+02	2020-06-01 04:23:59+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-01 04:24:04+02	2020-06-01 04:28:05+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-01 04:23:20+02	2020-06-01 04:31:25+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-01 04:31:28+02	2020-06-01 04:32:33+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-06-01 04:32:35+02	2020-06-01 04:40:53+02	net	ntopng	\N	ntopng-4.0.d20200414_1,1
2020-06-01 04:28:08+02	2020-06-01 04:41:30+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-08-05 02:07:49+02	2020-08-05 02:08:10+02	x11	bitmap	\N	bitmap-1.0.9
2020-06-01 04:40:54+02	2020-06-01 04:42:52+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-01 04:42:54+02	2020-06-01 04:45:31+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-01 04:45:35+02	2020-06-01 04:47:50+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-01 04:47:54+02	2020-06-01 04:49:58+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-01 04:49:59+02	2020-06-01 04:52:02+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-01 04:52:04+02	2020-06-01 04:54:01+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-01 04:54:02+02	2020-06-01 04:56:18+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-01 04:56:20+02	2020-06-01 04:58:19+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-01 04:58:22+02	2020-06-01 05:00:25+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-01 05:00:28+02	2020-06-01 05:02:40+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-01 05:02:43+02	2020-06-01 05:04:34+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-01 05:04:36+02	2020-06-01 05:05:10+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-06-01 04:41:39+02	2020-06-01 05:06:47+02	net	wireshark	\N	wireshark-3.2.4
2020-06-01 05:06:50+02	2020-06-01 05:07:50+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-01 05:05:11+02	2020-06-01 05:08:49+02	print	auctex	full	auctex-emacs26-12.2
2020-06-01 05:08:50+02	2020-06-01 05:09:47+02	print	gv	\N	gv-3.7.4_5
2020-06-01 05:07:51+02	2020-06-01 05:21:30+02	print	fontforge	\N	fontforge-20190801_2
2020-06-01 05:21:32+02	2020-06-01 05:31:49+02	print	texlive-full	\N	texlive-full-20150521
2020-06-01 05:09:48+02	2020-06-01 05:35:30+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-01 05:31:50+02	2020-06-01 05:36:25+02	security	clamav	\N	clamav-0.102.3_1,1
2020-06-01 05:35:33+02	2020-06-01 05:37:16+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_3
2020-06-01 05:36:26+02	2020-06-01 05:38:00+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-01 05:37:17+02	2020-06-01 05:38:34+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-01 05:38:35+02	2020-06-01 05:39:05+02	www	dokuwiki	\N	dokuwiki-20180422c
2020-06-01 05:38:02+02	2020-06-01 05:53:35+02	textproc	bat	\N	bat-0.15.0
2020-06-01 05:53:37+02	2020-06-01 05:55:21+02	www	mattermost-server	\N	mattermost-server-5.19.1
2020-06-01 05:55:22+02	2020-06-01 05:56:38+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-06-01 05:56:38+02	2020-06-01 05:57:18+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-06-01 05:57:18+02	2020-06-01 05:57:37+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-05-30 19:40:27+02	2020-06-01 05:59:48+02	www	chromium	\N	chromium-81.0.4044.138_1
2020-06-01 05:57:38+02	2020-06-01 06:00:06+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-01 05:59:52+02	2020-06-01 06:01:38+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-01 06:00:07+02	2020-06-01 06:02:09+02	x11	mate	\N	mate-1.22.2
2020-06-01 06:01:39+02	2020-06-01 06:03:19+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-01 06:03:20+02	2020-06-01 06:05:04+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-01 06:02:10+02	2020-06-01 06:10:24+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-01 06:05:05+02	2020-06-01 06:12:52+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-01 06:10:24+02	2020-06-01 06:13:47+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-01 06:13:48+02	2020-06-01 06:14:06+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-01 06:14:08+02	2020-06-01 06:16:03+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-01 05:39:06+02	2020-06-01 06:48:03+02	www	firefox	\N	firefox-77.0_3,1
2020-06-01 06:12:56+02	2020-06-01 06:58:23+02	editors	vscode	\N	vscode-1.45.1_1
2020-06-01 07:20:01+02	2020-06-01 07:20:27+02	devel	yaml-cpp03	\N	yaml-cpp03-0.3.0_2
2020-06-01 07:20:48+02	2020-06-01 07:21:25+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-06-01 07:21:25+02	2020-06-01 07:21:58+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.70.0
2020-06-01 07:21:58+02	2020-06-01 07:22:07+02	astro	xearth	\N	xearth-1.2
2020-06-01 07:20:01+02	2020-06-01 07:22:34+02	textproc	tinyxml	\N	tinyxml-2.6.2_4
2020-06-01 07:22:34+02	2020-06-01 07:24:49+02	graphics	opencolorio	\N	opencolorio-1.1.1_1
2020-06-01 07:24:50+02	2020-06-01 08:35:03+02	graphics	krita	\N	krita-4.2.9_3
2020-06-01 09:59:40+02	2020-06-01 09:59:48+02	devel	libpci	\N	libpci-3.7.0
2020-06-01 09:59:48+02	2020-06-01 10:00:03+02	devel	scons	py37	scons-py37-3.1.2
2020-06-01 09:59:40+02	2020-06-01 10:00:07+02	devel	oniguruma	\N	oniguruma-6.9.5.r1_1
2020-06-01 10:00:08+02	2020-06-01 10:00:40+02	converters	php73-mbstring	\N	php73-mbstring-7.3.18
2020-06-01 09:59:40+02	2020-06-01 10:00:49+02	www	w3m	\N	w3m-0.5.3.20200507
2020-06-01 10:00:50+02	2020-06-01 10:01:11+02	textproc	xmlto	\N	xmlto-0.0.28
2020-06-01 10:00:41+02	2020-06-01 10:01:44+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-01 10:01:44+02	2020-06-01 10:02:29+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-01 10:02:29+02	2020-06-01 10:03:18+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-01 10:04:10+02	2020-06-01 10:05:52+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-01 10:01:12+02	2020-06-01 10:22:54+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-01 10:23:01+02	2020-06-01 10:25:04+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-01 10:00:03+02	2020-06-01 10:41:45+02	www	chromium	\N	chromium-81.0.4044.138_1
2020-06-01 10:41:48+02	2020-06-01 10:43:39+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-01 10:43:39+02	2020-06-01 10:45:14+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-01 10:05:53+02	2020-06-01 10:46:17+02	devel	electron7	\N	electron7-7.3.0_1
2020-06-01 10:46:20+02	2020-06-01 10:49:51+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-01 10:49:51+02	2020-06-01 10:51:16+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-01 10:51:17+02	2020-06-01 10:52:46+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-01 10:52:47+02	2020-06-01 10:54:07+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-01 10:54:08+02	2020-06-01 10:55:31+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-01 10:55:32+02	2020-06-01 10:56:49+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-01 10:56:50+02	2020-06-01 10:58:15+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-01 10:58:15+02	2020-06-01 10:59:31+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-01 10:59:32+02	2020-06-01 11:01:20+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-01 11:01:21+02	2020-06-01 11:02:35+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-01 11:02:36+02	2020-06-01 11:03:44+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-01 10:29:23+02	2020-06-01 11:03:52+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-01 11:03:53+02	2020-06-01 11:04:11+02	sysutils	ncdu	\N	ncdu-1.15
2020-06-01 11:04:12+02	2020-06-01 11:04:32+02	sysutils	x86info	\N	x86info-1.31.s03
2020-06-01 11:03:44+02	2020-06-01 11:05:10+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-01 11:05:10+02	2020-06-01 11:05:40+02	www	dokuwiki	\N	dokuwiki-20180422c
2020-06-01 11:04:32+02	2020-06-01 11:21:51+02	textproc	bat	\N	bat-0.15.0
2020-06-01 11:21:52+02	2020-06-01 11:22:08+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-01 11:05:40+02	2020-06-01 11:54:46+02	www	firefox	\N	firefox-77.0_4,1
2020-06-01 11:20:14+02	2020-06-01 11:58:06+02	editors	vscode	\N	vscode-1.45.1_1
2020-06-01 12:28:04+02	2020-06-01 12:47:58+02	local	E590T-localbase	\N	E590T-localbase-2020060100
2020-06-01 14:02:10+02	2020-06-01 14:02:18+02	sysutils	lsop	\N	lsop-0.2
2020-06-01 14:01:36+02	2020-06-01 14:04:10+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-01 14:04:10+02	2020-06-01 14:04:21+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-01 14:01:36+02	2020-06-01 14:04:57+02	editors	vim	\N	vim-8.2.0869
2020-06-01 14:29:12+02	2020-06-01 14:46:46+02	local	E590T-localbase	\N	E590T-localbase-2020060100
2020-06-01 15:10:34+02	2020-06-01 15:10:41+02	textproc	py-toml	py27	py27-toml-0.10.0
2020-06-01 15:10:42+02	2020-06-01 15:10:49+02	devel	py-setuptools_scm	py27	py27-setuptools_scm-3.4.3
2020-06-01 15:10:49+02	2020-06-01 15:10:57+02	devel	py-dateutil	py27	py27-dateutil-2.8.1
2020-06-01 15:10:34+02	2020-06-01 15:12:38+02	devel	py-lxml	py27	py27-lxml-4.5.1
2020-06-01 16:59:08+02	2020-06-01 17:00:29+02	games	gnuchess	\N	gnuchess-6.2.7
2020-06-01 17:00:30+02	2020-06-01 17:02:49+02	games	xboard	\N	xboard-4.9.1_1
2020-06-01 16:59:43+02	2020-06-01 17:03:37+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-01 17:03:37+02	2020-06-01 17:03:48+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-01 16:59:08+02	2020-06-01 17:05:38+02	textproc	ripgrep	\N	ripgrep-12.1.1
2020-06-01 17:27:16+02	2020-06-01 17:42:42+02	local	E590T-localbase	\N	E590T-localbase-2020060100
2020-06-01 23:52:51+02	2020-06-01 23:54:18+02	print	freetype2	\N	freetype2-2.10.2
2020-06-01 23:52:51+02	2020-06-01 23:54:24+02	graphics	vigra	\N	vigra-1.11.1_10
2020-06-01 23:54:25+02	2020-06-01 23:54:43+02	x11-fonts	mkfontscale	\N	mkfontscale-1.2.1
2020-06-01 23:54:19+02	2020-06-01 23:55:15+02	x11-fonts	fontconfig	\N	fontconfig-2.13.92_2,1
2020-06-01 23:54:44+02	2020-06-01 23:59:23+02	print	harfbuzz	\N	harfbuzz-2.6.6
2020-06-01 23:59:24+02	2020-06-01 23:59:43+02	x11-fonts	dejavu	\N	dejavu-2.37_1
2020-06-01 23:59:44+02	2020-06-01 23:59:58+02	x11-fonts	font-bh-ttf	\N	font-bh-ttf-1.0.3_4
2020-06-01 23:55:16+02	2020-06-02 00:00:07+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-01 23:59:59+02	2020-06-02 00:00:14+02	x11-fonts	font-misc-ethiopic	\N	font-misc-ethiopic-1.0.3_4
2020-06-02 00:00:08+02	2020-06-02 00:00:23+02	x11-fonts	font-misc-meltho	\N	font-misc-meltho-1.0.3_4
2020-06-02 00:00:14+02	2020-06-02 00:00:40+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-06-02 00:00:23+02	2020-06-02 00:00:48+02	x11-fonts	xorg-fonts-truetype	\N	xorg-fonts-truetype-7.7_1
2020-06-02 00:00:41+02	2020-06-02 00:02:00+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_4
2020-06-02 00:02:01+02	2020-06-02 00:02:29+02	graphics	gd	\N	libgd-2.3.0,1
2020-06-01 23:52:51+02	2020-06-02 00:02:36+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-02 00:00:48+02	2020-06-02 00:03:29+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-02 00:02:36+02	2020-06-02 00:05:10+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.6_2
2020-06-02 00:02:29+02	2020-06-02 00:05:13+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-02 00:05:10+02	2020-06-02 00:06:00+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-02 00:05:14+02	2020-06-02 00:06:57+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-02 00:06:01+02	2020-06-02 00:07:40+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-02 00:03:30+02	2020-06-02 00:08:17+02	graphics	graphviz	\N	graphviz-2.44.0_3
2020-06-02 00:07:41+02	2020-06-02 00:11:00+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-02 00:06:58+02	2020-06-02 00:11:02+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-02 00:11:01+02	2020-06-02 00:11:40+02	graphics	poppler	\N	poppler-0.88.0
2020-06-02 00:08:18+02	2020-06-02 00:12:16+02	lang	vala	\N	vala-0.40.15,1
2020-06-02 00:11:41+02	2020-06-02 00:12:42+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-02 00:12:17+02	2020-06-02 00:13:36+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-02 00:11:02+02	2020-06-02 00:14:13+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-06-02 00:12:43+02	2020-06-02 00:15:08+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-02 00:14:14+02	2020-06-02 00:15:40+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-02 00:15:10+02	2020-06-02 00:18:08+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-02 00:18:09+02	2020-06-02 00:18:43+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-06-02 00:13:37+02	2020-06-02 00:19:57+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-02 00:18:44+02	2020-06-02 00:20:26+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-02 00:15:40+02	2020-06-02 00:22:02+02	print	texlive-base	\N	texlive-base-20150521_51
2020-06-02 00:20:28+02	2020-06-02 00:23:34+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-06-02 00:23:34+02	2020-06-02 00:24:15+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-06-02 00:24:16+02	2020-06-02 00:25:43+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-02 00:25:43+02	2020-06-02 00:26:23+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-06-02 00:26:23+02	2020-06-02 00:27:12+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-02 00:27:12+02	2020-06-02 00:27:40+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-02 00:27:41+02	2020-06-02 00:27:59+02	print	libraqm	\N	libraqm-0.6.0
2020-06-02 00:19:58+02	2020-06-02 00:29:28+02	multimedia	ffmpeg	\N	ffmpeg-4.2.3,1
2020-06-02 00:29:28+02	2020-06-02 00:30:01+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-02 00:27:59+02	2020-06-02 00:30:21+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-02 00:30:21+02	2020-06-02 00:30:45+02	x11-fonts	libXfont2	\N	libXfont2-2.0.4
2020-06-02 00:30:02+02	2020-06-02 00:31:02+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-02 00:30:45+02	2020-06-02 00:31:16+02	devel	py-gobject3	py37	py37-gobject3-3.28.3
2020-06-02 00:31:17+02	2020-06-02 00:31:39+02	graphics	php73-gd	\N	php73-gd-7.3.18
2020-06-02 00:31:03+02	2020-06-02 00:31:47+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-02 00:31:39+02	2020-06-02 00:32:15+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-02 00:31:47+02	2020-06-02 00:33:03+02	graphics	graphene	\N	graphene-1.10.0
2020-06-02 00:32:16+02	2020-06-02 00:33:03+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-02 00:33:03+02	2020-06-02 00:33:33+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-02 00:33:03+02	2020-06-02 00:34:00+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-06-02 00:33:33+02	2020-06-02 00:34:19+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-02 00:34:01+02	2020-06-02 00:34:44+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-02 00:34:45+02	2020-06-02 00:35:56+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-02 00:34:19+02	2020-06-02 00:36:16+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-02 00:35:56+02	2020-06-02 00:37:04+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-02 00:36:16+02	2020-06-02 00:37:17+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-02 00:37:04+02	2020-06-02 00:37:56+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-02 00:37:56+02	2020-06-02 00:39:43+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-02 00:39:44+02	2020-06-02 00:41:16+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-02 00:22:03+02	2020-06-02 00:41:39+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-02 00:41:16+02	2020-06-02 00:42:09+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-02 00:41:40+02	2020-06-02 00:44:51+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-02 00:44:52+02	2020-06-02 00:45:35+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-02 00:37:18+02	2020-06-02 00:46:05+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-02 00:45:36+02	2020-06-02 00:46:20+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-02 00:46:20+02	2020-06-02 00:47:03+02	graphics	poppler-glib	\N	poppler-glib-0.88.0
2020-06-02 00:46:05+02	2020-06-02 00:48:16+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-02 00:47:04+02	2020-06-02 00:48:17+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-02 00:42:10+02	2020-06-02 00:48:44+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-02 00:48:17+02	2020-06-02 00:49:00+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-06-02 00:48:18+02	2020-06-02 00:49:10+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-02 00:49:01+02	2020-06-02 00:49:43+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-06-02 00:49:11+02	2020-06-02 00:49:52+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-02 00:49:52+02	2020-06-02 00:50:58+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-02 00:50:59+02	2020-06-02 00:52:02+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-02 00:48:45+02	2020-06-02 00:52:24+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-02 00:52:03+02	2020-06-02 00:52:56+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-02 00:52:25+02	2020-06-02 00:53:32+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-02 00:52:57+02	2020-06-02 00:54:19+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-02 00:53:32+02	2020-06-02 00:55:38+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-02 00:55:39+02	2020-06-02 00:57:28+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-06-02 00:54:19+02	2020-06-02 00:59:04+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-06-02 00:57:29+02	2020-06-02 00:59:18+02	net	remmina	\N	remmina-1.3.9
2020-06-02 00:59:04+02	2020-06-02 01:01:38+02	security	gcr	\N	gcr-3.28.0
2020-06-02 01:01:39+02	2020-06-02 01:03:07+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-02 01:03:08+02	2020-06-02 01:04:26+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-02 00:59:18+02	2020-06-02 01:05:21+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_1,1
2020-06-02 01:04:27+02	2020-06-02 01:05:42+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-06-02 01:05:22+02	2020-06-02 01:06:24+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-02 01:05:43+02	2020-06-02 01:06:28+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-02 01:06:25+02	2020-06-02 01:07:34+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-02 00:49:43+02	2020-06-02 01:07:48+02	java	openjdk8	\N	openjdk8-8.252.09.1
2020-06-02 01:07:35+02	2020-06-02 01:07:53+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-06-02 01:07:48+02	2020-06-02 01:08:17+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-06-02 01:07:54+02	2020-06-02 01:08:28+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-06-02 01:06:28+02	2020-06-02 01:08:54+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-02 01:08:18+02	2020-06-02 01:08:56+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-02 01:08:29+02	2020-06-02 01:09:00+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-06-02 01:09:01+02	2020-06-02 01:09:22+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-06-02 01:08:54+02	2020-06-02 01:09:27+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-02 01:08:57+02	2020-06-02 01:09:31+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-06-02 01:09:28+02	2020-06-02 01:09:55+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-06-02 01:09:22+02	2020-06-02 01:10:02+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-06-02 01:09:32+02	2020-06-02 01:10:19+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-02 01:09:55+02	2020-06-02 01:10:31+02	x11-fonts	libXfont	\N	libXfont-1.5.4_2,2
2020-06-02 01:10:03+02	2020-06-02 01:11:02+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-02 01:10:31+02	2020-06-02 01:11:14+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-02 01:11:15+02	2020-06-02 01:11:34+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-02 01:11:03+02	2020-06-02 01:11:54+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-02 01:10:19+02	2020-06-02 01:12:05+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.22.2
2020-06-02 01:11:55+02	2020-06-02 01:12:25+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-02 01:11:35+02	2020-06-02 01:13:08+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-02 01:12:25+02	2020-06-02 01:13:09+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-02 01:13:08+02	2020-06-02 01:13:57+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-02 01:13:09+02	2020-06-02 01:14:32+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-02 01:14:33+02	2020-06-02 01:14:49+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.7_6
2020-06-02 01:13:58+02	2020-06-02 01:15:29+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-02 01:14:50+02	2020-06-02 01:15:45+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-02 01:15:30+02	2020-06-02 01:16:16+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-02 01:12:05+02	2020-06-02 01:16:42+02	net	libmateweather	\N	libmateweather-1.22.1
2020-06-02 01:15:45+02	2020-06-02 01:16:58+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-02 01:16:43+02	2020-06-02 01:17:38+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-02 01:17:41+02	2020-06-02 01:18:16+02	x11-fonts	font-misc-misc	\N	font-misc-misc-1.1.2_4
2020-06-02 01:18:17+02	2020-06-02 01:19:31+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-02 01:16:59+02	2020-06-02 01:19:39+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-02 01:19:31+02	2020-06-02 01:21:05+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-02 01:19:39+02	2020-06-02 01:21:19+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-02 01:21:20+02	2020-06-02 01:22:07+02	graphics	poppler-utils	\N	poppler-utils-0.88.0
2020-06-02 01:21:06+02	2020-06-02 01:22:18+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-02 01:22:08+02	2020-06-02 01:22:25+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-06-02 01:22:18+02	2020-06-02 01:22:37+02	print	libotf	\N	libotf-0.9.16
2020-06-02 01:16:17+02	2020-06-02 01:22:51+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-02 01:22:26+02	2020-06-02 01:23:06+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-02 01:22:51+02	2020-06-02 01:23:06+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-06-02 01:22:38+02	2020-06-02 01:23:17+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-02 01:23:18+02	2020-06-02 01:23:42+02	x11-fonts	font-adobe-100dpi	\N	font-adobe-100dpi-1.0.3_4
2020-06-02 01:23:06+02	2020-06-02 01:24:03+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-06-02 01:23:07+02	2020-06-02 01:24:32+02	x11-fm	caja	\N	caja-1.22.3
2020-06-02 01:23:42+02	2020-06-02 01:24:49+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-02 01:24:04+02	2020-06-02 01:24:56+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-02 01:24:32+02	2020-06-02 01:25:28+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-02 01:24:56+02	2020-06-02 01:25:47+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-02 01:24:50+02	2020-06-02 01:25:50+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-02 01:25:29+02	2020-06-02 01:26:08+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-02 01:25:50+02	2020-06-02 01:27:20+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-02 01:26:08+02	2020-06-02 01:27:28+02	graphics	gegl	\N	gegl-0.4.22_5
2020-06-02 01:25:48+02	2020-06-02 01:27:35+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-06-02 01:27:35+02	2020-06-02 01:28:27+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-02 01:27:29+02	2020-06-02 01:28:56+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-02 01:28:27+02	2020-06-02 01:29:27+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-02 01:28:57+02	2020-06-02 01:30:52+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-02 01:29:28+02	2020-06-02 01:31:21+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-02 01:30:52+02	2020-06-02 01:32:02+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-02 01:31:22+02	2020-06-02 01:32:33+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-02 01:32:34+02	2020-06-02 01:33:12+02	x11-fonts	font-adobe-75dpi	\N	font-adobe-75dpi-1.0.3_4
2020-06-02 01:32:03+02	2020-06-02 01:33:41+02	x11	zenity	\N	zenity-3.28.1
2020-06-02 01:33:13+02	2020-06-02 01:34:31+02	x11-fonts	noto-basic	\N	noto-basic-2.0
2020-06-02 01:33:42+02	2020-06-02 01:36:56+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_1,1
2020-06-02 01:36:56+02	2020-06-02 01:38:06+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-02 01:27:21+02	2020-06-02 01:38:31+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-02 01:38:32+02	2020-06-02 01:39:25+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-06-02 01:38:07+02	2020-06-02 01:39:38+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-02 01:39:39+02	2020-06-02 01:41:59+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-02 01:34:31+02	2020-06-02 01:42:09+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-02 01:42:10+02	2020-06-02 01:43:06+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-02 01:39:26+02	2020-06-02 01:43:11+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-02 01:41:59+02	2020-06-02 01:43:22+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-02 01:43:12+02	2020-06-02 01:43:31+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-02 01:43:31+02	2020-06-02 01:44:19+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-06-02 01:43:07+02	2020-06-02 01:47:10+02	editors	emacs	full	emacs-26.3_4,3
2020-06-02 01:47:11+02	2020-06-02 01:48:04+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-06-02 01:44:20+02	2020-06-02 01:48:20+02	graphics	opencv	\N	opencv-3.4.1_31
2020-06-02 01:48:21+02	2020-06-02 01:48:37+02	print	adobe-cmaps	\N	adobe-cmaps-20051217_4
2020-06-02 01:48:04+02	2020-06-02 01:48:48+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-02 01:43:22+02	2020-06-02 01:48:49+02	graphics	gimp-app	\N	gimp-app-2.10.18_6,1
2020-06-02 01:48:37+02	2020-06-02 01:49:12+02	print	cups-filters	\N	cups-filters-1.27.4_1
2020-06-02 01:48:49+02	2020-06-02 01:49:33+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-02 01:49:12+02	2020-06-02 01:50:22+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-06-02 01:48:48+02	2020-06-02 01:58:17+02	print	tex-luatex	\N	tex-luatex-0.80.0_17
2020-06-02 01:58:18+02	2020-06-02 02:01:35+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-06-02 02:01:37+02	2020-06-02 02:03:06+02	x11	xterm	\N	xterm-356
2020-06-02 02:03:07+02	2020-06-02 02:05:26+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-02 02:05:27+02	2020-06-02 02:05:49+02	x11-fonts	font-adobe-utopia-100dpi	\N	font-adobe-utopia-100dpi-1.0.4_4
2020-06-02 02:05:50+02	2020-06-02 02:06:09+02	x11-fonts	font-adobe-utopia-75dpi	\N	font-adobe-utopia-75dpi-1.0.4_4
2020-06-02 02:06:10+02	2020-06-02 02:06:31+02	x11-fonts	font-adobe-utopia-type1	\N	font-adobe-utopia-type1-1.0.4_4
2020-06-02 02:06:32+02	2020-06-02 02:06:43+02	x11-fonts	font-arabic-misc	\N	font-arabic-misc-1.0.3_4
2020-06-02 02:06:48+02	2020-06-02 02:07:27+02	x11-fonts	font-bh-100dpi	\N	font-bh-100dpi-1.0.3_4
2020-06-02 02:07:28+02	2020-06-02 02:08:02+02	x11-fonts	font-bh-75dpi	\N	font-bh-75dpi-1.0.3_4
2020-06-02 02:08:03+02	2020-06-02 02:08:21+02	x11-fonts	font-bh-lucidatypewriter-100dpi	\N	font-bh-lucidatypewriter-100dpi-1.0.3_4
2020-06-02 02:08:22+02	2020-06-02 02:08:42+02	x11-fonts	font-bh-lucidatypewriter-75dpi	\N	font-bh-lucidatypewriter-75dpi-1.0.3_4
2020-06-02 02:08:42+02	2020-06-02 02:08:59+02	x11-fonts	font-bh-type1	\N	font-bh-type1-1.0.3_4
2020-06-02 02:09:01+02	2020-06-02 02:09:15+02	x11-fonts	font-bitstream-100dpi	\N	font-bitstream-100dpi-1.0.3_4
2020-06-02 02:09:15+02	2020-06-02 02:09:30+02	x11-fonts	font-bitstream-75dpi	\N	font-bitstream-75dpi-1.0.3_4
2020-06-02 02:09:31+02	2020-06-02 02:09:43+02	x11-fonts	font-bitstream-type1	\N	font-bitstream-type1-1.0.3_4
2020-06-02 02:09:46+02	2020-06-02 02:10:02+02	x11-fonts	font-cronyx-cyrillic	\N	font-cronyx-cyrillic-1.0.3_4
2020-06-02 02:10:03+02	2020-06-02 02:10:16+02	x11-fonts	font-cursor-misc	\N	font-cursor-misc-1.0.3_4
2020-06-02 01:50:23+02	2020-06-02 02:10:16+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-02 02:10:18+02	2020-06-02 02:10:31+02	x11-fonts	font-dec-misc	\N	font-dec-misc-1.0.3_4
2020-06-02 02:10:18+02	2020-06-02 02:10:32+02	x11-fonts	font-daewoo-misc	\N	font-daewoo-misc-1.0.3_4
2020-06-02 02:10:32+02	2020-06-02 02:10:47+02	x11-fonts	font-ibm-type1	\N	font-ibm-type1-1.0.3_4
2020-06-02 02:10:33+02	2020-06-02 02:10:48+02	x11-fonts	font-isas-misc	\N	font-isas-misc-1.0.3_4
2020-06-02 02:10:47+02	2020-06-02 02:11:00+02	x11-fonts	font-jis-misc	\N	font-jis-misc-1.0.3_4
2020-06-02 02:10:49+02	2020-06-02 02:11:02+02	x11-fonts	font-micro-misc	\N	font-micro-misc-1.0.3_4
2020-06-02 02:11:00+02	2020-06-02 02:11:14+02	x11-fonts	font-misc-cyrillic	\N	font-misc-cyrillic-1.0.3_4
2020-06-02 02:11:02+02	2020-06-02 02:11:16+02	x11-fonts	font-mutt-misc	\N	font-mutt-misc-1.0.3_4
2020-06-02 02:11:14+02	2020-06-02 02:11:28+02	x11-fonts	font-schumacher-misc	\N	font-schumacher-misc-1.1.2_4
2020-06-02 02:11:17+02	2020-06-02 02:11:29+02	x11-fonts	font-screen-cyrillic	\N	font-screen-cyrillic-1.0.4_4
2020-06-02 02:11:29+02	2020-06-02 02:11:38+02	x11-fonts	font-sony-misc	\N	font-sony-misc-1.0.3_4
2020-06-02 02:11:30+02	2020-06-02 02:11:39+02	x11-fonts	font-sun-misc	\N	font-sun-misc-1.0.3_4
2020-06-02 02:11:39+02	2020-06-02 02:11:48+02	x11-fonts	font-winitzki-cyrillic	\N	font-winitzki-cyrillic-1.0.3_4
2020-06-02 02:11:40+02	2020-06-02 02:11:50+02	x11-fonts	font-xfree86-type1	\N	font-xfree86-type1-1.0.4_4
2020-06-02 02:11:51+02	2020-06-02 02:12:51+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-02 02:11:49+02	2020-06-02 02:13:02+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-02 02:12:52+02	2020-06-02 02:14:32+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-02 02:14:35+02	2020-06-02 02:15:44+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-02 02:15:45+02	2020-06-02 02:16:35+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-06-02 02:16:36+02	2020-06-02 02:17:51+02	x11-wm	marco	\N	marco-1.22.4
2020-06-02 02:17:52+02	2020-06-02 02:19:28+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-02 01:49:34+02	2020-06-02 02:19:47+02	www	chromium	\N	chromium-81.0.4044.138_1
2020-06-02 02:19:29+02	2020-06-02 02:20:25+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-06-02 02:19:50+02	2020-06-02 02:21:47+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-02 02:20:25+02	2020-06-02 02:21:57+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-02 02:21:48+02	2020-06-02 02:22:15+02	devel	py-gobject3	py27	py27-gobject3-3.28.3
2020-06-02 02:13:03+02	2020-06-02 02:22:37+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-06-02 02:22:38+02	2020-06-02 02:23:15+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-02 02:21:57+02	2020-06-02 02:23:30+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-02 02:22:16+02	2020-06-02 02:23:51+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-02 02:23:30+02	2020-06-02 02:23:57+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-02 02:23:17+02	2020-06-02 02:24:30+02	graphics	poppler-qt5	\N	poppler-qt5-0.88.0
2020-06-02 02:23:52+02	2020-06-02 02:24:36+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-02 02:24:30+02	2020-06-02 02:25:16+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-02 02:24:36+02	2020-06-02 02:25:40+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-02 02:23:57+02	2020-06-02 02:25:49+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-02 02:27:08+02	2020-06-02 02:28:36+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-02 02:28:36+02	2020-06-02 02:29:57+02	net	freerdp	\N	freerdp-2.0.0.r4_9
2020-06-02 02:29:58+02	2020-06-02 02:30:45+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-02 02:25:50+02	2020-06-02 02:31:34+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-06-02 02:30:45+02	2020-06-02 02:31:55+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-02 02:31:35+02	2020-06-02 02:32:24+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-02 02:32:25+02	2020-06-02 02:32:47+02	print	libspectre	\N	libspectre-0.2.8
2020-06-02 02:31:55+02	2020-06-02 02:35:18+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-02 02:32:48+02	2020-06-02 02:38:18+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-02 02:25:17+02	2020-06-02 02:38:36+02	math	R	\N	R-4.0.0_1
2020-06-02 02:38:37+02	2020-06-02 02:39:27+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-06-02 02:38:19+02	2020-06-02 02:40:02+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-06-02 02:40:03+02	2020-06-02 02:40:16+02	x11	x11perf	\N	x11perf-1.6.1
2020-06-02 02:35:19+02	2020-06-02 02:40:24+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-02 02:39:27+02	2020-06-02 02:40:26+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-02 02:40:16+02	2020-06-02 02:40:36+02	x11	xcalc	\N	xcalc-1.1.0
2020-06-02 02:40:25+02	2020-06-02 02:40:36+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-06-02 02:40:27+02	2020-06-02 02:41:16+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-02 02:40:36+02	2020-06-02 02:41:21+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-02 02:40:37+02	2020-06-02 02:41:26+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-02 02:41:17+02	2020-06-02 02:42:02+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-02 02:41:21+02	2020-06-02 02:42:12+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-02 02:42:03+02	2020-06-02 02:42:55+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-02 02:42:56+02	2020-06-02 02:43:09+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-06-02 02:42:12+02	2020-06-02 02:43:11+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-02 02:41:27+02	2020-06-02 02:43:16+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-02 02:43:09+02	2020-06-02 02:43:19+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-06-02 02:43:12+02	2020-06-02 02:43:19+02	x11-fonts	gentium-basic	\N	GentiumBasic-1102_1
2020-06-02 02:43:16+02	2020-06-02 02:43:25+02	x11-fonts	liberation-fonts-ttf	\N	liberation-fonts-ttf-2.1.0,2
2020-06-02 02:43:20+02	2020-06-02 02:43:31+02	x11-fonts	twemoji-color-font-ttf	\N	twemoji-color-font-ttf-12.0.1
2020-06-02 02:43:19+02	2020-06-02 02:43:33+02	x11-fonts	linuxlibertine-g	\N	linuxlibertine-g-20120116_2
2020-06-02 02:43:33+02	2020-06-02 02:43:48+02	x11-fonts	xorg-fonts-cyrillic	\N	xorg-fonts-cyrillic-7.7
2020-06-02 02:43:25+02	2020-06-02 02:43:50+02	x11-fonts	xorg-fonts-100dpi	\N	xorg-fonts-100dpi-7.7
2020-06-02 02:43:31+02	2020-06-02 02:43:56+02	x11-fonts	xorg-fonts-75dpi	\N	xorg-fonts-75dpi-7.7
2020-06-02 02:43:51+02	2020-06-02 02:44:08+02	x11-fonts	xorg-fonts-type1	\N	xorg-fonts-type1-7.7
2020-06-02 02:43:48+02	2020-06-02 02:44:26+02	x11-fonts	xorg-fonts-miscbitmaps	\N	xorg-fonts-miscbitmaps-7.7
2020-06-02 02:44:08+02	2020-06-02 02:45:04+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-02 02:44:26+02	2020-06-02 02:45:13+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-02 02:43:56+02	2020-06-02 02:45:22+02	x11-servers	xephyr	\N	xephyr-1.20.8_1,1
2020-06-02 02:45:04+02	2020-06-02 02:45:55+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-02 02:45:55+02	2020-06-02 02:46:42+02	audio	faudio	\N	FAudio-20.05
2020-06-02 02:45:23+02	2020-06-02 02:47:16+02	archivers	engrampa	\N	engrampa-1.22.2
2020-06-02 02:46:44+02	2020-06-02 02:48:10+02	audio	mate-media	\N	mate-media-1.22.2
2020-06-02 02:47:17+02	2020-06-02 02:48:56+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-06-02 02:48:11+02	2020-06-02 02:49:43+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-02 02:45:13+02	2020-06-02 02:50:04+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-02 02:49:43+02	2020-06-02 02:50:42+02	deskutils	mozo	\N	mozo-1.22.2
2020-06-02 02:50:05+02	2020-06-02 02:51:01+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-02 02:48:56+02	2020-06-02 02:51:26+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-06-02 02:50:42+02	2020-06-02 02:51:58+02	devel	gradle5	\N	gradle5-5.0
2020-06-02 02:51:01+02	2020-06-02 02:52:21+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-02 02:51:27+02	2020-06-02 02:52:43+02	devel	py-game	py27	py27-game-1.9.6_1
2020-06-02 02:52:44+02	2020-06-02 02:55:12+02	editors	pluma	\N	pluma-1.22.2
2020-06-02 02:55:13+02	2020-06-02 02:58:08+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-06-02 02:58:08+02	2020-06-02 03:01:18+02	graphics	atril	\N	atril-1.22.3_5
2020-06-02 03:01:19+02	2020-06-02 03:03:58+02	graphics	eom	\N	eom-1.22.2
2020-06-02 03:03:58+02	2020-06-02 03:06:04+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-02 03:06:05+02	2020-06-02 03:06:39+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-06-02 03:06:39+02	2020-06-02 03:07:30+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-02 03:07:30+02	2020-06-02 03:12:31+02	graphics	podofo	\N	podofo-0.9.6_1
2020-06-02 03:12:34+02	2020-06-02 03:14:20+02	graphics	py-gimp	\N	py27-gimp-2.10.18_6
2020-06-02 02:52:22+02	2020-06-02 03:16:09+02	editors	libreoffice	\N	libreoffice-6.3.6
2020-06-02 02:51:59+02	2020-06-02 03:19:34+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-02 03:14:20+02	2020-06-02 03:34:53+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-02 03:34:55+02	2020-06-02 03:39:26+02	math	mate-calc	\N	mate-calc-1.22.2
2020-06-02 03:39:27+02	2020-06-02 03:40:49+02	multimedia	libass	\N	libass-0.14.0
2020-06-02 03:40:50+02	2020-06-02 03:42:43+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-02 03:16:10+02	2020-06-02 03:44:39+02	java	openjdk11	\N	openjdk11-11.0.7+10.2
2020-06-02 03:44:40+02	2020-06-02 03:45:59+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-02 03:46:00+02	2020-06-02 03:47:09+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-02 03:47:10+02	2020-06-02 03:48:26+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-02 03:48:27+02	2020-06-02 03:49:29+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-02 03:42:44+02	2020-06-02 03:49:41+02	multimedia	vlc	\N	vlc-3.0.10_4,4
2020-06-02 03:49:29+02	2020-06-02 03:50:16+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-02 03:49:41+02	2020-06-02 03:50:22+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-02 03:50:17+02	2020-06-02 03:50:59+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-02 03:50:23+02	2020-06-02 03:51:18+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-02 03:51:18+02	2020-06-02 03:52:10+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-02 03:50:59+02	2020-06-02 03:52:17+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-02 03:52:18+02	2020-06-02 03:53:22+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-02 03:52:11+02	2020-06-02 03:53:24+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-02 03:19:36+02	2020-06-02 03:54:12+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-02 03:54:13+02	2020-06-02 03:55:57+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-02 03:53:25+02	2020-06-02 03:58:31+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-02 03:53:23+02	2020-06-02 03:58:38+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-02 03:58:39+02	2020-06-02 03:59:03+02	print	transfig	\N	transfig-3.2.5e_4
2020-06-02 03:59:03+02	2020-06-02 04:00:26+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-06-02 04:00:27+02	2020-06-02 04:02:17+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-06-02 03:55:58+02	2020-06-02 04:02:48+02	print	tex-xetex	\N	tex-xetex-0.99992_33
2020-06-02 04:02:49+02	2020-06-02 04:03:43+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-02 04:02:18+02	2020-06-02 04:03:44+02	textproc	fop	\N	fop-2.3
2020-06-02 04:03:43+02	2020-06-02 04:04:18+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-02 04:03:44+02	2020-06-02 04:04:46+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-02 04:04:46+02	2020-06-02 04:05:50+02	x11	mate-base	\N	mate-base-1.22.2
2020-06-02 04:05:51+02	2020-06-02 04:07:13+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-06-02 04:07:14+02	2020-06-02 04:08:47+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-06-02 04:04:44+02	2020-06-02 04:08:51+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-06-02 04:08:52+02	2020-06-02 04:09:01+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-06-02 04:08:47+02	2020-06-02 04:09:07+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-06-02 04:09:08+02	2020-06-02 04:09:14+02	x11-fonts	bitstream-vera	\N	bitstream-vera-1.10_8
2020-06-02 04:09:15+02	2020-06-02 04:09:22+02	x11-fonts	gnu-unifont	\N	gnu-unifont-13.0.02,1
2020-06-02 04:09:01+02	2020-06-02 04:09:43+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-02 04:09:43+02	2020-06-02 04:10:48+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-06-02 04:09:22+02	2020-06-02 04:11:20+02	x11-fonts	xorg-fonts	\N	xorg-fonts-7.7_1
2020-06-02 04:10:49+02	2020-06-02 04:11:37+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-02 04:11:21+02	2020-06-02 04:11:53+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-02 04:11:38+02	2020-06-02 04:17:25+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-02 03:58:32+02	2020-06-02 04:18:52+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-02 04:17:26+02	2020-06-02 04:19:14+02	astro	xephem	\N	xephem-3.7.7_5
2020-06-02 04:11:54+02	2020-06-02 04:19:25+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-02 04:18:53+02	2020-06-02 04:19:47+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-02 04:19:25+02	2020-06-02 04:20:33+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-02 04:19:15+02	2020-06-02 04:47:33+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-02 04:19:48+02	2020-06-02 04:48:32+02	devel	electron7	\N	electron7-7.3.0_1
2020-06-02 04:48:37+02	2020-06-02 04:52:22+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-06-02 04:52:23+02	2020-06-02 04:54:36+02	editors	vim	\N	vim-8.2.0869
2020-06-02 04:47:34+02	2020-06-02 04:55:46+02	editors	codelite	\N	codelite-12.0_4
2020-06-02 04:55:46+02	2020-06-02 05:01:42+02	emulators	wine	\N	wine-5.0_1,1
2020-06-02 05:01:43+02	2020-06-02 05:02:14+02	emulators	x48	\N	x48-0.6.4_1
2020-06-02 04:54:37+02	2020-06-02 05:02:53+02	emulators	qemu	\N	qemu-4.1.1_3
2020-06-02 05:02:54+02	2020-06-02 05:04:32+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-02 05:04:32+02	2020-06-02 05:05:12+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-02 05:05:13+02	2020-06-02 05:05:48+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-02 05:05:49+02	2020-06-02 05:06:40+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-02 05:06:40+02	2020-06-02 05:07:21+02	games	outerspace	\N	outerspace-0.2.2_9
2020-06-02 05:07:22+02	2020-06-02 05:08:32+02	games	xblast	\N	xblast-2.10.4_17
2020-06-02 05:08:32+02	2020-06-02 05:09:28+02	games	xboard	\N	xboard-4.9.1_1
2020-06-02 05:09:28+02	2020-06-02 05:10:12+02	games	xcowsay	\N	xcowsay-1.5
2020-06-02 05:02:14+02	2020-06-02 05:11:04+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-02 05:10:12+02	2020-06-02 05:11:42+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-02 05:11:05+02	2020-06-02 05:12:04+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-02 04:20:33+02	2020-06-02 05:16:28+02	devel	ghidra	\N	ghidra-9.1
2020-06-02 05:16:30+02	2020-06-02 05:18:37+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-02 05:12:04+02	2020-06-02 05:34:53+02	graphics	krita	\N	krita-4.2.9_3
2020-06-02 05:34:55+02	2020-06-02 05:37:55+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-02 05:18:39+02	2020-06-02 05:53:51+02	java	openjdk12	\N	openjdk12-12.0.2+10.4
2020-06-02 05:11:43+02	2020-06-02 05:59:46+02	graphics	inkscape	\N	inkscape-1.0_1
2020-06-02 05:37:57+02	2020-06-02 06:10:13+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-02 06:10:13+02	2020-06-02 06:11:59+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-02 06:12:00+02	2020-06-02 06:14:02+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-02 06:14:02+02	2020-06-02 06:17:01+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-02 06:17:02+02	2020-06-02 06:21:45+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-02 06:21:46+02	2020-06-02 06:23:05+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-02 06:23:06+02	2020-06-02 06:24:03+02	net	v2ray	\N	v2ray-4.23.2
2020-06-02 06:24:03+02	2020-06-02 06:24:17+02	net	vnstat	\N	vnstat-2.6_1
2020-06-02 05:53:52+02	2020-06-02 06:24:38+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-02 06:24:39+02	2020-06-02 06:25:21+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-02 06:25:22+02	2020-06-02 06:26:28+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-02 06:26:29+02	2020-06-02 06:27:12+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-02 06:24:18+02	2020-06-02 06:27:29+02	net	wireshark	\N	wireshark-3.2.4
2020-06-02 06:27:12+02	2020-06-02 06:27:55+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-02 05:59:47+02	2020-06-02 06:28:08+02	lang	mono	\N	mono-5.10.1.57_3
2020-06-02 06:27:30+02	2020-06-02 06:28:11+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-02 06:27:55+02	2020-06-02 06:28:36+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-02 06:28:09+02	2020-06-02 06:28:50+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-02 06:28:11+02	2020-06-02 06:28:52+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-02 06:28:50+02	2020-06-02 06:29:34+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-02 06:28:52+02	2020-06-02 06:29:35+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-02 06:28:36+02	2020-06-02 06:29:38+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-02 06:29:35+02	2020-06-02 06:29:58+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-02 06:29:58+02	2020-06-02 06:30:27+02	print	gv	\N	gv-3.7.4_5
2020-06-02 06:29:34+02	2020-06-02 06:31:46+02	print	auctex	full	auctex-emacs26-12.2
2020-06-02 06:29:39+02	2020-06-02 06:32:01+02	print	fontforge	\N	fontforge-20190801_2
2020-06-02 06:30:28+02	2020-06-02 06:32:22+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-02 06:32:01+02	2020-06-02 06:32:42+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_3
2020-06-02 06:32:22+02	2020-06-02 06:33:06+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-02 06:32:42+02	2020-06-02 06:33:11+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-02 06:33:11+02	2020-06-02 06:33:29+02	www	dokuwiki	\N	dokuwiki-20180422c
2020-06-02 06:33:06+02	2020-06-02 06:33:41+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-02 06:33:42+02	2020-06-02 06:34:35+02	www	links	\N	links-2.20.2,1
2020-06-02 06:31:47+02	2020-06-02 06:35:29+02	print	texlive-full	\N	texlive-full-20150521
2020-06-02 06:34:36+02	2020-06-02 06:35:30+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-02 06:35:31+02	2020-06-02 06:36:12+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-02 06:36:12+02	2020-06-02 06:38:00+02	x11	mate	\N	mate-1.22.2
2020-06-02 06:38:02+02	2020-06-02 06:38:18+02	x11	sterm	\N	sterm-0.8.2
2020-06-02 06:38:19+02	2020-06-02 06:39:31+02	x11	xdm	\N	xdm-1.1.12_3
2020-06-02 06:39:32+02	2020-06-02 06:41:03+02	x11	xlockmore	\N	xlockmore-5.63
2020-06-02 06:41:03+02	2020-06-02 06:45:10+02	x11	xorg	\N	xorg-7.7_3
2020-06-02 06:45:10+02	2020-06-02 06:47:27+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-02 06:47:27+02	2020-06-02 06:53:35+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-02 06:35:30+02	2020-06-02 06:54:15+02	x11	alacritty	\N	alacritty-0.4.2_1
2020-06-02 06:54:16+02	2020-06-02 06:54:30+02	x11-fm	xdiskusage	\N	xdiskusage-1.51_1
2020-06-02 06:53:36+02	2020-06-02 06:55:35+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-02 06:54:30+02	2020-06-02 06:55:52+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-02 06:55:38+02	2020-06-02 06:56:04+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.3_6
2020-06-02 06:58:10+02	2020-06-02 06:58:21+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-02 06:58:22+02	2020-06-02 06:59:14+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-02 06:59:15+02	2020-06-02 06:59:54+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-06-02 06:33:29+02	2020-06-02 07:13:06+02	www	firefox	\N	firefox-77.0_4,1
2020-06-02 06:56:05+02	2020-06-02 07:33:56+02	editors	vscode	\N	vscode-1.45.1_1
2020-06-02 07:39:11+02	2020-06-02 07:43:06+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-06-02 07:43:06+02	2020-06-02 07:43:52+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-02 07:43:53+02	2020-06-02 07:44:33+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-02 19:50:23+02	2020-06-02 19:50:45+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-06-02 19:50:46+02	2020-06-02 19:51:12+02	graphics	poppler-data	\N	poppler-data-0.4.9_3
2020-06-02 19:51:13+02	2020-06-02 19:53:59+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-06-02 19:54:01+02	2020-06-02 19:54:54+02	multimedia	libva	\N	libva-2.7.1
2020-06-02 19:54:56+02	2020-06-02 19:56:46+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-06-02 19:50:23+02	2020-06-02 20:01:02+02	lang	perl5.30	\N	perl5-5.30.3
2020-06-02 20:01:02+02	2020-06-02 20:01:30+02	devel	p5-Locale-gettext	\N	p5-Locale-gettext-1.07
2020-06-02 20:01:31+02	2020-06-02 20:01:56+02	converters	p5-Text-Unidecode	\N	p5-Text-Unidecode-1.30
2020-06-02 20:01:56+02	2020-06-02 20:02:40+02	devel	p5-Locale-libintl	\N	p5-Locale-libintl-1.31
2020-06-02 20:02:41+02	2020-06-02 20:03:08+02	misc	help2man	\N	help2man-1.47.15
2020-06-02 20:03:08+02	2020-06-02 20:03:30+02	textproc	p5-Unicode-EastAsianWidth	\N	p5-Unicode-EastAsianWidth-12.0
2020-06-02 19:56:47+02	2020-06-02 20:04:42+02	lang	go	\N	go-1.14.4,1
2020-06-02 20:04:43+02	2020-06-02 20:05:21+02	textproc	p5-XML-Parser	\N	p5-XML-Parser-2.44
2020-06-02 20:03:31+02	2020-06-02 20:05:36+02	print	texinfo	\N	texinfo-6.7_3,1
2020-06-02 20:05:22+02	2020-06-02 20:05:48+02	textproc	intltool	\N	intltool-0.51.0_1
2020-06-02 20:05:37+02	2020-06-02 20:06:41+02	devel	autoconf	\N	autoconf-2.69_3
2020-06-02 20:05:49+02	2020-06-02 20:06:42+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-06-02 20:06:41+02	2020-06-02 20:07:28+02	devel	automake	\N	automake-1.16.2
2020-06-02 20:07:30+02	2020-06-02 20:08:01+02	devel	autoconf213	\N	autoconf213-2.13.000227_7
2020-06-02 20:08:01+02	2020-06-02 20:12:13+02	devel	qt5-buildtools	\N	qt5-buildtools-5.14.2
2020-06-02 19:50:23+02	2020-06-02 20:12:16+02	devel	cmake	\N	cmake-3.17.3
2020-06-02 20:06:43+02	2020-06-02 20:12:21+02	textproc	groff	\N	groff-1.22.4_3
2020-06-02 20:12:14+02	2020-06-02 20:12:40+02	print	texlive-tlmgr	\N	texlive-tlmgr-20150523_2
2020-06-02 20:12:22+02	2020-06-02 20:12:51+02	textproc	texi2html	\N	texi2html-5.0_2,1
2020-06-02 20:12:52+02	2020-06-02 20:13:17+02	textproc	hyphen	\N	hyphen-2.8.8
2020-06-02 20:13:18+02	2020-06-02 20:13:45+02	net	p5-URI	\N	p5-URI-1.76
2020-06-02 20:13:45+02	2020-06-02 20:14:13+02	www	w3m	\N	w3m-0.5.3.20200507
2020-06-02 20:12:41+02	2020-06-02 20:14:14+02	textproc	hunspell	\N	hunspell-1.7.0_2
2020-06-02 20:14:14+02	2020-06-02 20:14:39+02	textproc	p5-XML-NamespaceSupport	\N	p5-XML-NamespaceSupport-1.12
2020-06-02 20:14:39+02	2020-06-02 20:15:02+02	textproc	p5-XML-SAX-Base	\N	p5-XML-SAX-Base-1.09
2020-06-02 20:15:03+02	2020-06-02 20:15:37+02	textproc	xmlto	\N	xmlto-0.0.28
2020-06-02 20:14:14+02	2020-06-02 20:15:52+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-06-02 20:15:37+02	2020-06-02 20:16:05+02	textproc	p5-XML-SAX	\N	p5-XML-SAX-1.02
2020-06-02 20:16:05+02	2020-06-02 20:16:32+02	textproc	p5-XML-SAX-Expat	\N	p5-XML-SAX-Expat-0.51_1
2020-06-02 20:16:32+02	2020-06-02 20:19:56+02	databases	mysql57-client	\N	mysql57-client-5.7.30_1
2020-06-02 20:15:52+02	2020-06-02 20:24:03+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-06-02 20:19:59+02	2020-06-02 20:24:06+02	textproc	aspell	\N	aspell-0.60.8,1
2020-06-02 20:24:04+02	2020-06-02 20:24:28+02	textproc	p5-XML-Simple	\N	p5-XML-Simple-2.25
2020-06-02 20:24:07+02	2020-06-02 20:24:34+02	devel	imake	\N	imake-1.0.8,1
2020-06-02 20:24:29+02	2020-06-02 20:24:58+02	net	p5-Socket6	\N	p5-Socket6-0.29
2020-06-02 20:24:59+02	2020-06-02 20:25:37+02	x11-themes	icon-naming-utils	\N	icon-naming-utils-0.8.90_1
2020-06-02 20:25:38+02	2020-06-02 20:26:19+02	devel	p5-TimeDate	\N	p5-TimeDate-2.33,1
2020-06-02 20:24:35+02	2020-06-02 20:26:28+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-06-02 20:26:20+02	2020-06-02 20:26:46+02	net	p5-IO-Socket-INET6	\N	p5-IO-Socket-INET6-2.72_1
2020-06-02 20:26:29+02	2020-06-02 20:26:53+02	security	p5-Digest-HMAC	\N	p5-Digest-HMAC-1.03_1
2020-06-02 20:26:54+02	2020-06-02 20:27:14+02	www	p5-HTML-Tagset	\N	p5-HTML-Tagset-3.20_1
2020-06-02 20:27:14+02	2020-06-02 20:27:39+02	www	p5-Mozilla-CA	\N	p5-Mozilla-CA-20180117
2020-06-02 20:26:46+02	2020-06-02 20:27:43+02	security	p5-Net-SSLeay	\N	p5-Net-SSLeay-1.88
2020-06-02 20:27:43+02	2020-06-02 20:28:09+02	security	p5-IO-Socket-SSL	\N	p5-IO-Socket-SSL-2.068
2020-06-02 20:28:10+02	2020-06-02 20:28:48+02	www	p5-HTML-Parser	\N	p5-HTML-Parser-3.72
2020-06-02 20:27:40+02	2020-06-02 20:28:50+02	accessibility	atkmm	\N	atkmm-2.24.2_4
2020-06-02 20:28:49+02	2020-06-02 20:29:12+02	www	p5-HTTP-Date	\N	p5-HTTP-Date-6.05
2020-06-02 20:28:51+02	2020-06-02 20:29:14+02	converters	p5-Encode-Locale	\N	p5-Encode-Locale-1.05
2020-06-02 20:29:16+02	2020-06-02 20:29:38+02	devel	netsurf-buildsystem	\N	netsurf-buildsystem-1.8
2020-06-02 20:29:13+02	2020-06-02 20:29:56+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-06-02 20:29:39+02	2020-06-02 20:30:03+02	devel	p5-Clone	\N	p5-Clone-0.45
2020-06-02 20:29:57+02	2020-06-02 20:30:18+02	devel	p5-IO-HTML	\N	p5-IO-HTML-1.001_1
2020-06-02 20:30:04+02	2020-06-02 20:30:28+02	www	p5-LWP-MediaTypes	\N	p5-LWP-MediaTypes-6.04
2020-06-02 20:30:18+02	2020-06-02 20:30:45+02	devel	p5-Parse-Yapp	\N	p5-Parse-Yapp-1.05_2
2020-06-02 20:30:46+02	2020-06-02 20:32:25+02	print	enscript-a4	\N	enscript-a4-1.6.6_3
2020-06-02 20:32:27+02	2020-06-02 20:33:05+02	security	p5-GSSAPI	\N	p5-GSSAPI-0.28_1
2020-06-02 20:30:28+02	2020-06-02 20:33:19+02	graphics	netpbm	\N	netpbm-10.90.02
2020-06-02 20:33:06+02	2020-06-02 20:33:32+02	www	p5-HTTP-Message	\N	p5-HTTP-Message-6.24
2020-06-02 20:33:21+02	2020-06-02 20:33:40+02	devel	p5-IO-String	\N	p5-IO-String-1.08_1
2020-06-02 20:33:40+02	2020-06-02 20:34:06+02	lang	p5-Error	\N	p5-Error-0.17029
2020-06-02 20:33:33+02	2020-06-02 20:34:29+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-06-02 20:34:06+02	2020-06-02 20:34:33+02	security	p5-Authen-SASL	\N	p5-Authen-SASL-2.16_1
2020-06-02 20:34:31+02	2020-06-02 20:34:53+02	www	p5-CGI	\N	p5-CGI-4.47
2020-06-02 20:34:54+02	2020-06-02 20:37:02+02	devel	git	\N	git-2.26.2
2020-06-02 20:37:03+02	2020-06-02 20:38:58+02	textproc	mythes	\N	mythes-1.2.4_7
2020-06-02 20:38:59+02	2020-06-02 20:55:22+02	www	apache24	\N	apache24-2.4.43
2020-06-02 20:55:24+02	2020-06-02 20:56:15+02	x11-fonts	p5-Font-TTF	\N	p5-Font-TTF-1.06
2020-06-02 20:56:21+02	2020-06-02 20:56:58+02	archivers	p5-Archive-Zip	\N	p5-Archive-Zip-1.68
2020-06-02 20:57:01+02	2020-06-02 20:57:46+02	devel	p5-Getopt-ArgvFile	\N	p5-Getopt-ArgvFile-1.11_1
2020-06-02 20:57:47+02	2020-06-02 20:58:24+02	lang	p5-Try-Tiny	\N	p5-Try-Tiny-0.30
2020-06-02 20:58:26+02	2020-06-02 21:02:24+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-06-02 21:02:28+02	2020-06-02 21:03:17+02	net	p5-Net-HTTP	\N	p5-Net-HTTP-6.19
2020-06-02 21:03:19+02	2020-06-02 21:04:12+02	security	p5-Authen-NTLM	\N	p5-Authen-NTLM-1.09_1
2020-06-02 21:04:16+02	2020-06-02 21:04:59+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.04_1
2020-06-02 21:05:02+02	2020-06-02 21:08:17+02	textproc	enchant	\N	enchant-1.6.0_9
2020-06-02 21:08:21+02	2020-06-02 21:09:47+02	textproc	p5-PDF-API2	\N	p5-PDF-API2-2.037
2020-06-02 21:09:48+02	2020-06-02 21:10:33+02	textproc	p5-Pod-Escapes	\N	p5-Pod-Escapes-1.07
2020-06-02 21:10:35+02	2020-06-02 21:11:15+02	www	p5-HTTP-Cookies	\N	p5-HTTP-Cookies-6.08
2020-06-02 21:11:17+02	2020-06-02 21:11:59+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.10
2020-06-02 21:12:01+02	2020-06-02 21:12:42+02	www	p5-HTTP-Negotiate	\N	p5-HTTP-Negotiate-6.01_1
2020-06-02 21:12:44+02	2020-06-02 21:13:26+02	www	p5-WWW-RobotRules	\N	p5-WWW-RobotRules-6.02_1
2020-06-02 21:13:28+02	2020-06-02 21:17:47+02	devel	libstatgrab	\N	libstatgrab-0.92
2020-06-02 21:17:50+02	2020-06-02 22:03:53+02	lang	julia11	\N	julia11-1.1.1_5
2020-06-02 22:03:55+02	2020-06-02 22:07:42+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-06-02 22:07:44+02	2020-06-02 22:14:33+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-06-02 22:14:39+02	2020-06-02 22:16:11+02	print	transfig	\N	transfig-3.2.5e_4
2020-06-02 22:16:13+02	2020-06-02 22:17:04+02	security	p5-Digest-SHA1	\N	p5-Digest-SHA1-2.13_1
2020-06-02 22:17:06+02	2020-06-02 22:17:41+02	textproc	p5-pod2pdf	\N	p5-pod2pdf-0.42_1
2020-06-02 22:17:48+02	2020-06-02 22:18:35+02	www	p5-libwww	\N	p5-libwww-6.44
2020-06-02 22:18:37+02	2020-06-02 22:21:35+02	converters	recode	\N	recode-3.6_14
2020-06-02 20:12:17+02	2020-06-03 02:52:37+02	devel	llvm80	\N	llvm80-8.0.1_3
2020-06-03 02:52:48+02	2020-06-03 03:05:01+02	graphics	mesa-libs	\N	mesa-libs-19.0.8
2020-06-03 03:05:03+02	2020-06-03 03:12:38+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-03 03:12:40+02	2020-06-03 03:18:50+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-03 03:18:53+02	2020-06-03 03:37:46+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-03 03:37:46+02	2020-06-03 03:38:59+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-03 03:38:59+02	2020-06-03 03:43:01+02	lang	vala	\N	vala-0.40.15,1
2020-06-03 03:43:02+02	2020-06-03 03:44:12+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-06-03 03:44:13+02	2020-06-03 03:45:20+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-03 03:45:20+02	2020-06-03 03:48:01+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-03 03:48:02+02	2020-06-03 03:51:58+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-02 20:34:34+02	2020-06-03 04:01:04+02	devel	llvm90	\N	llvm90-9.0.1_1
2020-06-03 03:51:59+02	2020-06-03 04:02:29+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-03 04:02:30+02	2020-06-03 04:04:06+02	graphics	libGLU	\N	libGLU-9.0.1
2020-06-03 04:01:07+02	2020-06-03 04:04:16+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-03 04:04:16+02	2020-06-03 04:08:50+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-03 04:08:51+02	2020-06-03 04:11:34+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-03 04:04:07+02	2020-06-03 04:12:46+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-03 04:11:35+02	2020-06-03 04:12:51+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-03 04:12:55+02	2020-06-03 04:13:31+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-03 04:12:47+02	2020-06-03 04:14:20+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-03 04:13:34+02	2020-06-03 04:15:24+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-03 04:14:21+02	2020-06-03 04:15:40+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-03 04:15:25+02	2020-06-03 04:20:07+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-03 04:15:41+02	2020-06-03 04:28:04+02	multimedia	ffmpeg	\N	ffmpeg-4.2.3,1
2020-06-03 04:28:05+02	2020-06-03 04:29:30+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-03 04:29:31+02	2020-06-03 04:30:28+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-03 04:30:29+02	2020-06-03 04:30:50+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-03 04:30:50+02	2020-06-03 04:35:16+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-03 04:35:17+02	2020-06-03 04:35:57+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-03 04:35:57+02	2020-06-03 04:37:20+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-06-03 04:20:08+02	2020-06-03 04:49:58+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-03 04:50:00+02	2020-06-03 04:54:21+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-03 04:37:20+02	2020-06-03 05:02:28+02	graphics	mesa-dri	\N	mesa-dri-19.0.8
2020-06-03 05:02:29+02	2020-06-03 05:03:58+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-03 04:54:22+02	2020-06-03 05:04:16+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-03 05:04:00+02	2020-06-03 05:04:29+02	devel	py-gobject3	py37	py37-gobject3-3.28.3
2020-06-03 05:04:17+02	2020-06-03 05:05:14+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-03 05:04:30+02	2020-06-03 05:05:20+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-03 05:05:21+02	2020-06-03 05:07:23+02	graphics	graphene	\N	graphene-1.10.0
2020-06-03 05:07:24+02	2020-06-03 05:08:36+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-03 05:08:37+02	2020-06-03 05:09:05+02	www	libwpe	\N	libwpe-1.4.0.1
2020-06-03 05:05:15+02	2020-06-03 05:09:54+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-03 05:09:55+02	2020-06-03 05:10:30+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-03 05:09:06+02	2020-06-03 05:10:32+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-06-03 05:10:33+02	2020-06-03 05:11:34+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-03 05:10:31+02	2020-06-03 05:11:35+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-03 05:11:34+02	2020-06-03 05:12:56+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-03 05:11:35+02	2020-06-03 05:13:08+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-03 05:12:57+02	2020-06-03 05:13:31+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-03 05:13:32+02	2020-06-03 05:14:57+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-03 05:13:09+02	2020-06-03 05:14:59+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-03 05:14:57+02	2020-06-03 05:16:13+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-03 05:16:13+02	2020-06-03 05:19:55+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-03 05:19:58+02	2020-06-03 05:21:30+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-03 05:21:31+02	2020-06-03 05:22:55+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-03 05:22:56+02	2020-06-03 05:23:55+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-03 05:15:00+02	2020-06-03 05:24:46+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-03 05:23:56+02	2020-06-03 05:25:02+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-03 05:25:02+02	2020-06-03 05:27:56+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-03 05:24:47+02	2020-06-03 05:28:03+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_1,1
2020-06-03 05:27:56+02	2020-06-03 05:30:47+02	devel	sdl20	\N	sdl2-2.0.10_1
2020-06-03 05:28:03+02	2020-06-03 05:31:03+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-03 05:31:04+02	2020-06-03 05:31:50+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-03 05:30:47+02	2020-06-03 05:31:51+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-03 05:31:51+02	2020-06-03 05:32:31+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-06-03 05:31:52+02	2020-06-03 05:32:34+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-03 05:32:32+02	2020-06-03 05:33:05+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-06-03 05:32:34+02	2020-06-03 05:33:30+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-03 05:33:05+02	2020-06-03 05:33:49+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-03 05:33:31+02	2020-06-03 05:34:11+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-03 05:33:49+02	2020-06-03 05:34:35+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-03 05:34:12+02	2020-06-03 05:35:08+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-03 05:34:36+02	2020-06-03 05:36:07+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-03 05:35:08+02	2020-06-03 05:36:08+02	net	remmina	\N	remmina-1.3.9
2020-06-03 05:36:09+02	2020-06-03 05:36:51+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-03 05:36:08+02	2020-06-03 05:37:29+02	security	gcr	\N	gcr-3.28.0
2020-06-03 05:36:52+02	2020-06-03 05:37:39+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-03 05:37:30+02	2020-06-03 05:38:23+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-03 05:37:40+02	2020-06-03 05:38:24+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-02 22:21:37+02	2020-06-03 05:38:35+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200327
2020-06-03 05:38:35+02	2020-06-03 05:38:58+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-03 05:38:24+02	2020-06-03 05:39:34+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-03 05:38:58+02	2020-06-03 05:39:42+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-06-03 05:39:35+02	2020-06-03 05:39:54+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-06-03 05:39:43+02	2020-06-03 05:40:20+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-03 05:39:55+02	2020-06-03 05:40:46+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-03 05:38:25+02	2020-06-03 05:40:59+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-03 05:40:21+02	2020-06-03 05:41:21+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-03 05:41:00+02	2020-06-03 05:41:37+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-03 05:41:38+02	2020-06-03 05:41:52+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-03 05:41:21+02	2020-06-03 05:42:11+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-03 05:40:46+02	2020-06-03 05:42:26+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.22.2
2020-06-03 05:42:11+02	2020-06-03 05:42:40+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-03 05:42:41+02	2020-06-03 05:43:25+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-03 05:41:53+02	2020-06-03 05:43:26+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-03 05:43:27+02	2020-06-03 05:44:02+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-03 05:43:26+02	2020-06-03 05:44:10+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-03 05:44:11+02	2020-06-03 05:44:30+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-06-03 05:44:31+02	2020-06-03 05:45:22+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-03 05:44:02+02	2020-06-03 05:45:27+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-03 05:45:23+02	2020-06-03 05:46:08+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-03 05:45:27+02	2020-06-03 05:46:40+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-03 05:42:26+02	2020-06-03 05:47:01+02	net	libmateweather	\N	libmateweather-1.22.1
2020-06-03 05:46:41+02	2020-06-03 05:47:39+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-03 05:47:39+02	2020-06-03 05:48:53+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-03 05:47:02+02	2020-06-03 05:49:50+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-03 05:48:55+02	2020-06-03 05:50:42+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-03 05:49:51+02	2020-06-03 05:51:29+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-03 05:50:46+02	2020-06-03 05:51:41+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-03 05:46:09+02	2020-06-03 05:52:11+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-03 05:51:42+02	2020-06-03 05:52:33+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-03 05:52:12+02	2020-06-03 05:53:10+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-03 05:53:11+02	2020-06-03 05:53:25+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-06-03 05:51:29+02	2020-06-03 05:53:43+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-03 05:52:34+02	2020-06-03 05:53:48+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-06-03 05:53:25+02	2020-06-03 05:54:19+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-03 05:53:44+02	2020-06-03 05:54:29+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-03 05:53:49+02	2020-06-03 05:54:35+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-03 05:54:20+02	2020-06-03 05:55:04+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-03 05:54:29+02	2020-06-03 05:55:20+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-03 05:55:05+02	2020-06-03 05:55:51+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-03 05:55:21+02	2020-06-03 05:56:20+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-03 05:54:36+02	2020-06-03 05:56:28+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-03 05:56:29+02	2020-06-03 05:57:19+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-03 05:56:21+02	2020-06-03 05:57:34+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-03 05:55:53+02	2020-06-03 05:57:37+02	x11-fm	caja	\N	caja-1.22.3
2020-06-03 05:57:19+02	2020-06-03 05:58:15+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-03 05:57:38+02	2020-06-03 05:58:28+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-03 05:57:34+02	2020-06-03 05:58:37+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-03 05:58:15+02	2020-06-03 05:58:54+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-03 05:58:29+02	2020-06-03 05:59:54+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-03 05:58:37+02	2020-06-03 05:59:57+02	graphics	gegl	\N	gegl-0.4.22_6
2020-06-03 05:59:58+02	2020-06-03 06:00:16+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-06-03 06:00:17+02	2020-06-03 06:01:04+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-03 05:59:54+02	2020-06-03 06:01:14+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-03 06:01:05+02	2020-06-03 06:01:49+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-03 06:01:15+02	2020-06-03 06:02:36+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-03 05:58:54+02	2020-06-03 06:03:00+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-03 06:01:50+02	2020-06-03 06:03:12+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-03 06:02:36+02	2020-06-03 06:03:24+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-03 06:03:13+02	2020-06-03 06:03:33+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-06-03 06:03:00+02	2020-06-03 06:03:51+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-03 06:03:25+02	2020-06-03 06:04:23+02	x11	zenity	\N	zenity-3.28.1
2020-06-03 06:03:33+02	2020-06-03 06:04:24+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-03 06:03:51+02	2020-06-03 06:05:11+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_1,1
2020-06-03 06:04:24+02	2020-06-03 06:05:14+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-03 06:04:23+02	2020-06-03 06:05:49+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-03 06:05:12+02	2020-06-03 06:06:19+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-03 06:05:50+02	2020-06-03 06:07:24+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-03 06:06:20+02	2020-06-03 06:07:48+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-03 06:07:25+02	2020-06-03 06:08:11+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-03 06:05:15+02	2020-06-03 06:08:23+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-03 06:08:11+02	2020-06-03 06:08:30+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-03 06:08:31+02	2020-06-03 06:10:33+02	graphics	glew	\N	glew-2.1.0
2020-06-03 06:07:50+02	2020-06-03 06:12:15+02	editors	emacs	full	emacs-26.3_4,3
2020-06-03 06:10:33+02	2020-06-03 06:13:32+02	graphics	movit	\N	movit-1.6.2_2
2020-06-03 06:13:33+02	2020-06-03 06:14:02+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-06-03 06:14:03+02	2020-06-03 06:14:15+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-06-03 06:08:23+02	2020-06-03 06:15:03+02	graphics	gimp-app	\N	gimp-app-2.10.18_7,1
2020-06-03 06:14:15+02	2020-06-03 06:15:07+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-03 06:15:04+02	2020-06-03 06:15:42+02	print	cups-filters	\N	cups-filters-1.27.4_2
2020-06-03 06:12:16+02	2020-06-03 06:16:18+02	graphics	opencv	\N	opencv-3.4.1_31
2020-06-03 06:15:43+02	2020-06-03 06:16:27+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-03 06:16:19+02	2020-06-03 06:17:32+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-06-03 06:15:11+02	2020-06-03 06:23:48+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-03 06:23:49+02	2020-06-03 06:26:50+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-06-03 06:26:51+02	2020-06-03 06:31:19+02	x11	xorg	\N	xorg-7.7_3
2020-06-03 06:31:23+02	2020-06-03 06:33:11+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-03 06:33:12+02	2020-06-03 06:34:31+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-03 06:34:32+02	2020-06-03 06:35:40+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-03 06:17:32+02	2020-06-03 06:35:51+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-03 06:35:53+02	2020-06-03 06:37:11+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-03 06:35:40+02	2020-06-03 06:37:25+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-03 06:37:30+02	2020-06-03 06:38:12+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-06-03 06:37:12+02	2020-06-03 06:39:00+02	x11-wm	marco	\N	marco-1.22.4
2020-06-03 06:39:01+02	2020-06-03 06:40:22+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-06-03 06:38:13+02	2020-06-03 06:40:29+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-03 06:40:30+02	2020-06-03 06:42:22+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-03 06:42:25+02	2020-06-03 06:43:11+02	devel	py-gobject3	py27	py27-gobject3-3.28.3
2020-06-03 06:40:23+02	2020-06-03 06:43:32+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-03 06:43:36+02	2020-06-03 06:44:30+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-06-03 06:43:12+02	2020-06-03 06:46:29+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-03 06:16:28+02	2020-06-03 06:47:42+02	www	chromium	\N	chromium-81.0.4044.138_1
2020-06-03 06:44:31+02	2020-06-03 06:47:53+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-03 06:46:30+02	2020-06-03 06:48:02+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-06-03 06:47:46+02	2020-06-03 06:48:16+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-03 06:48:03+02	2020-06-03 06:48:48+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-03 06:48:16+02	2020-06-03 06:50:23+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-03 06:50:26+02	2020-06-03 06:52:06+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-03 06:48:52+02	2020-06-03 06:53:03+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-03 06:52:08+02	2020-06-03 06:53:51+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-03 06:47:54+02	2020-06-03 06:55:55+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-03 06:55:59+02	2020-06-03 07:00:27+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-03 07:00:28+02	2020-06-03 07:02:23+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-03 07:02:25+02	2020-06-03 07:04:03+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-03 06:53:54+02	2020-06-03 07:04:21+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-06-03 07:04:04+02	2020-06-03 07:04:50+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-03 07:04:51+02	2020-06-03 07:05:10+02	print	libspectre	\N	libspectre-0.2.8
2020-06-03 07:04:21+02	2020-06-03 07:07:46+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-03 07:05:11+02	2020-06-03 07:11:01+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-03 06:53:03+02	2020-06-03 07:12:15+02	math	R	\N	R-4.0.0_1
2020-06-03 07:11:01+02	2020-06-03 07:12:52+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-06-03 07:12:16+02	2020-06-03 07:13:13+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-06-03 07:07:47+02	2020-06-03 07:13:14+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-03 07:12:53+02	2020-06-03 07:13:47+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-03 07:13:14+02	2020-06-03 07:14:13+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-03 07:13:48+02	2020-06-03 07:14:38+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-03 07:13:13+02	2020-06-03 07:14:39+02	x11-servers	xephyr	\N	xephyr-1.20.8_1,1
2020-06-03 07:14:13+02	2020-06-03 07:15:08+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-03 07:15:08+02	2020-06-03 07:15:41+02	audio	faudio	\N	FAudio-20.05
2020-06-03 07:14:39+02	2020-06-03 07:16:34+02	archivers	engrampa	\N	engrampa-1.22.2
2020-06-03 07:15:41+02	2020-06-03 07:17:20+02	audio	mate-media	\N	mate-media-1.22.2
2020-06-03 07:16:35+02	2020-06-03 07:17:58+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-06-03 07:17:21+02	2020-06-03 07:19:03+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-03 07:14:39+02	2020-06-03 07:20:04+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-03 07:19:04+02	2020-06-03 07:20:16+02	deskutils	mozo	\N	mozo-1.22.2
2020-06-03 07:20:18+02	2020-06-03 07:20:36+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-06-03 07:17:58+02	2020-06-03 07:20:55+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-06-03 07:20:05+02	2020-06-03 07:20:58+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-03 07:20:36+02	2020-06-03 07:21:50+02	devel	ode	\N	ode-0.13_4,1
2020-06-03 07:20:56+02	2020-06-03 07:21:55+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-03 07:21:55+02	2020-06-03 07:24:35+02	editors	pluma	\N	pluma-1.22.2
2020-06-03 07:24:36+02	2020-06-03 07:27:36+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-06-03 07:27:36+02	2020-06-03 07:33:00+02	graphics	atril	\N	atril-1.22.3_6
2020-06-03 07:33:02+02	2020-06-03 07:35:35+02	graphics	eom	\N	eom-1.22.2
2020-06-03 07:35:36+02	2020-06-03 07:37:51+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-03 07:37:51+02	2020-06-03 07:38:13+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-06-03 07:38:13+02	2020-06-03 07:38:32+02	graphics	gle	\N	gle-3.1.0_7
2020-06-03 07:38:32+02	2020-06-03 07:38:48+02	graphics	gltt	\N	gltt-2.5.2_14
2020-06-03 07:38:48+02	2020-06-03 07:39:31+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-03 07:39:32+02	2020-06-03 07:39:54+02	graphics	libplacebo	\N	libplacebo-2.43.1
2020-06-03 07:39:54+02	2020-06-03 07:41:23+02	graphics	py-gimp	\N	py27-gimp-2.10.18_7
2020-06-03 07:41:24+02	2020-06-03 07:41:55+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-06-03 07:21:51+02	2020-06-03 07:45:48+02	editors	libreoffice	\N	libreoffice-6.3.6_1
2020-06-03 07:20:59+02	2020-06-03 07:47:10+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-03 07:47:11+02	2020-06-03 07:48:47+02	math	mate-calc	\N	mate-calc-1.22.2
2020-06-03 07:48:48+02	2020-06-03 07:49:52+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-03 07:41:55+02	2020-06-03 07:50:26+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-03 07:49:53+02	2020-06-03 07:51:20+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-03 07:51:21+02	2020-06-03 07:52:32+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-03 07:52:32+02	2020-06-03 07:53:18+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-03 07:53:19+02	2020-06-03 07:54:34+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-03 07:54:35+02	2020-06-03 07:55:22+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-03 07:50:26+02	2020-06-03 07:55:59+02	multimedia	vlc	\N	vlc-3.0.10_4,4
2020-06-03 07:55:23+02	2020-06-03 07:56:20+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-03 07:56:00+02	2020-06-03 07:56:45+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-03 07:56:20+02	2020-06-03 07:57:07+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-03 07:56:46+02	2020-06-03 07:57:42+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-03 07:57:08+02	2020-06-03 07:58:32+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-03 07:57:42+02	2020-06-03 07:58:40+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-03 07:58:32+02	2020-06-03 07:59:52+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-03 07:58:40+02	2020-06-03 07:59:53+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-03 07:45:49+02	2020-06-03 08:02:24+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-03 08:02:25+02	2020-06-03 08:04:12+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-03 07:59:54+02	2020-06-03 08:05:19+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-03 07:59:52+02	2020-06-03 08:05:36+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-03 08:05:37+02	2020-06-03 08:07:09+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-06-03 08:07:10+02	2020-06-03 08:08:38+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-06-03 08:08:39+02	2020-06-03 08:09:32+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-03 08:09:32+02	2020-06-03 08:10:09+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-03 08:04:13+02	2020-06-03 08:10:31+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-03 08:10:10+02	2020-06-03 08:11:13+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-03 08:11:13+02	2020-06-03 08:12:29+02	x11	mate-base	\N	mate-base-1.22.2
2020-06-03 08:12:29+02	2020-06-03 08:13:57+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-06-03 08:10:57+02	2020-06-03 08:15:30+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-06-03 08:13:58+02	2020-06-03 08:15:37+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-06-03 08:15:31+02	2020-06-03 08:15:53+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-06-03 08:15:54+02	2020-06-03 08:16:28+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-03 08:15:38+02	2020-06-03 08:16:34+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-03 08:16:29+02	2020-06-03 08:17:42+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-03 08:17:42+02	2020-06-03 08:18:50+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-03 08:16:34+02	2020-06-03 08:25:30+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-03 08:25:31+02	2020-06-03 08:27:13+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-03 08:05:20+02	2020-06-03 08:29:01+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-03 08:18:51+02	2020-06-03 08:58:08+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-03 08:58:14+02	2020-06-03 09:06:22+02	devel	subversion	\N	subversion-1.14.0
2020-06-03 09:06:23+02	2020-06-03 09:06:54+02	devel	tig	\N	tig-2.5.1
2020-06-03 08:27:13+02	2020-06-03 09:10:32+02	devel	electron7	\N	electron7-7.3.0_1
2020-06-03 09:10:36+02	2020-06-03 09:16:45+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-06-03 09:06:54+02	2020-06-03 09:19:36+02	editors	codelite	\N	codelite-12.0_4
2020-06-03 09:16:48+02	2020-06-03 09:20:07+02	editors	vim	\N	vim-8.2.0869
2020-06-03 09:20:08+02	2020-06-03 09:32:12+02	emulators	wine	\N	wine-5.0_1,1
2020-06-03 09:19:37+02	2020-06-03 09:36:23+02	emulators	qemu	\N	qemu-4.1.1_3
2020-06-03 09:36:26+02	2020-06-03 09:36:55+02	games	KnightCap	\N	knightcap-3.6_8
2020-06-03 09:36:59+02	2020-06-03 09:38:24+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-06-03 09:38:27+02	2020-06-03 09:43:53+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-03 09:43:56+02	2020-06-03 09:46:52+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-06-03 09:32:13+02	2020-06-03 09:49:07+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-03 09:49:09+02	2020-06-03 09:50:35+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-06-03 09:50:38+02	2020-06-03 09:52:34+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-03 09:52:39+02	2020-06-03 09:53:57+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-03 09:46:53+02	2020-06-03 09:54:29+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-03 09:53:58+02	2020-06-03 09:54:43+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-03 09:54:29+02	2020-06-03 09:55:29+02	games	xblast	\N	xblast-2.10.4_17
2020-06-03 09:54:44+02	2020-06-03 09:55:55+02	games	xboard	\N	xboard-4.9.1_1
2020-06-03 09:55:29+02	2020-06-03 09:56:29+02	games	xcowsay	\N	xcowsay-1.5
2020-06-03 09:55:55+02	2020-06-03 10:03:16+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-03 10:03:20+02	2020-06-03 10:06:19+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-03 09:56:29+02	2020-06-03 10:08:22+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-03 10:06:21+02	2020-06-03 10:12:17+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-03 10:12:19+02	2020-06-03 10:14:52+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-03 10:14:53+02	2020-06-03 10:19:52+02	irc	irssi	\N	irssi-1.2.2,1
2020-06-03 10:19:55+02	2020-06-03 10:21:28+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-03 10:08:23+02	2020-06-03 10:24:03+02	graphics	krita	\N	krita-4.2.9_4
2020-06-03 10:21:29+02	2020-06-03 10:57:55+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-03 10:57:56+02	2020-06-03 10:58:45+02	lang	julia	\N	julia-1.0.3_3
2020-06-03 10:58:46+02	2020-06-03 11:05:14+02	mail	alpine	\N	alpine-2.22
2020-06-03 11:05:15+02	2020-06-03 11:08:43+02	misc	mc	\N	mc-nox11-4.8.24
2020-06-03 11:08:44+02	2020-06-03 11:09:04+02	multimedia	libva-glx	\N	libva-glx-2.7.1
2020-06-03 11:09:04+02	2020-06-03 11:10:15+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.0
2020-06-03 10:24:04+02	2020-06-03 11:10:46+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-03 11:10:48+02	2020-06-03 11:12:22+02	multimedia	libva-utils	\N	libva-utils-2.7.1
2020-06-03 11:12:24+02	2020-06-03 11:14:01+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-06-03 11:14:05+02	2020-06-03 11:19:56+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-03 11:19:59+02	2020-06-03 11:26:08+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-03 11:26:10+02	2020-06-03 11:33:55+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-03 11:33:57+02	2020-06-03 11:35:56+02	net	mosh	\N	mosh-1.3.2_12
2020-06-03 11:10:16+02	2020-06-03 11:38:10+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.p3
2020-06-03 11:35:57+02	2020-06-03 11:38:52+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-03 11:38:10+02	2020-06-03 11:40:18+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-03 11:40:19+02	2020-06-03 11:42:22+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-03 11:38:52+02	2020-06-03 11:45:10+02	net	wireshark	\N	wireshark-3.2.4
2020-06-03 11:45:11+02	2020-06-03 11:46:21+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-03 11:46:21+02	2020-06-03 11:47:32+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-03 11:47:33+02	2020-06-03 11:48:53+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-03 11:48:54+02	2020-06-03 11:50:06+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-03 11:50:07+02	2020-06-03 11:51:16+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-03 11:51:17+02	2020-06-03 11:52:24+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-03 11:52:25+02	2020-06-03 11:53:28+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-03 11:53:28+02	2020-06-03 11:55:53+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-03 11:55:54+02	2020-06-03 11:57:08+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-03 11:57:08+02	2020-06-03 11:58:19+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-03 11:58:20+02	2020-06-03 11:58:40+02	net-mgmt	ipcalc	\N	ipcalc-0.41_3
2020-06-03 11:58:40+02	2020-06-03 11:59:37+02	net-mgmt	ipv6calc	\N	ipv6calc-2.2.0
2020-06-03 11:59:40+02	2020-06-03 12:00:00+02	ports-mgmt	pkg_tree	\N	pkg_tree-1.1_4
2020-06-03 12:00:03+02	2020-06-03 12:02:18+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-03 11:42:23+02	2020-06-03 12:07:07+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-06-03 12:02:19+02	2020-06-03 12:07:32+02	print	auctex	full	auctex-emacs26-12.2
2020-06-03 12:07:08+02	2020-06-03 12:07:49+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-03 12:07:50+02	2020-06-03 12:08:36+02	print	gv	\N	gv-3.7.4_5
2020-06-03 12:08:37+02	2020-06-03 12:12:02+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-03 12:07:33+02	2020-06-03 12:12:04+02	print	fontforge	\N	fontforge-20190801_2
2020-06-03 12:12:07+02	2020-06-03 12:17:40+02	security	klee	\N	klee-2.1
2020-06-03 12:17:41+02	2020-06-03 12:18:54+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_3
2020-06-03 12:18:55+02	2020-06-03 12:19:20+02	security	rkhunter	\N	rkhunter-1.4.6
2020-06-03 12:19:20+02	2020-06-03 12:19:43+02	security	ssl-admin	\N	ssl-admin-1.2.1
2020-06-03 12:12:02+02	2020-06-03 12:19:52+02	print	texlive-full	\N	texlive-full-20150521
2020-06-03 12:19:53+02	2020-06-03 12:20:25+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-06-03 12:19:44+02	2020-06-03 12:20:59+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-03 12:20:25+02	2020-06-03 12:21:11+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-03 12:21:12+02	2020-06-03 12:21:31+02	sysutils	zfs-stats	\N	zfs-stats-1.3.0
2020-06-03 12:21:01+02	2020-06-03 12:21:34+02	sysutils	parallel	\N	parallel-20200122
2020-06-03 12:21:32+02	2020-06-03 12:21:53+02	textproc	colordiff	\N	colordiff-1.0.19
2020-06-03 12:21:35+02	2020-06-03 12:22:32+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-03 12:22:33+02	2020-06-03 12:23:08+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_1
2020-06-03 12:23:09+02	2020-06-03 12:29:09+02	www	mod_php73	\N	mod_php73-7.3.18
2020-06-03 12:29:10+02	2020-06-03 12:30:10+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-06-03 12:30:10+02	2020-06-03 12:32:28+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-03 12:32:30+02	2020-06-03 12:33:45+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-03 12:33:45+02	2020-06-03 12:38:15+02	x11	mate	\N	mate-1.22.2
2020-06-03 12:38:17+02	2020-06-03 12:39:45+02	x11	xlockmore	\N	xlockmore-5.63
2020-06-03 12:39:47+02	2020-06-03 12:48:26+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-03 12:48:28+02	2020-06-03 13:00:32+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-03 13:00:35+02	2020-06-03 13:04:24+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-03 13:04:28+02	2020-06-03 13:05:10+02	x11-fm	xdiskusage	\N	xdiskusage-1.51_1
2020-06-03 13:05:14+02	2020-06-03 13:07:21+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-03 13:13:19+02	2020-06-03 14:40:07+02	editors	vscode	\N	vscode-1.45.1_1
2020-06-03 14:40:08+02	2020-06-03 14:40:24+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-03 14:40:24+02	2020-06-03 14:41:30+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-03 08:29:04+02	2020-06-03 14:46:27+02	devel	llvm10	\N	llvm10-10.0.0_1
2020-06-03 12:21:54+02	2020-06-03 15:21:45+02	www	firefox	\N	firefox-77.0_4,1
2020-06-03 15:43:31+02	2020-06-03 15:59:24+02	local	E590T-localbase	\N	E590T-localbase-2020060200
2020-06-03 20:23:20+02	2020-06-03 20:23:38+02	devel	libepoll-shim	\N	libepoll-shim-0.0.20200602
2020-06-03 20:23:38+02	2020-06-03 20:24:05+02	graphics	wayland	\N	wayland-1.18.0_3
2020-06-03 20:23:38+02	2020-06-03 20:24:25+02	devel	libevdev	\N	libevdev-1.5.9_1
2020-06-03 20:24:06+02	2020-06-03 20:24:51+02	multimedia	libva	\N	libva-2.7.1
2020-06-03 20:24:06+02	2020-06-03 20:25:23+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-06-03 20:24:27+02	2020-06-03 20:25:46+02	x11	libinput	\N	libinput-1.15.5
2020-06-03 20:24:52+02	2020-06-03 20:27:25+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-06-03 20:25:25+02	2020-06-03 20:28:05+02	net	freerdp	\N	freerdp-2.0.0.r4_9
2020-06-03 20:28:06+02	2020-06-03 20:28:37+02	multimedia	libva-utils	\N	libva-utils-2.7.1
2020-06-03 20:23:20+02	2020-06-03 20:28:38+02	print	harfbuzz	\N	harfbuzz-2.6.7
2020-06-03 20:27:27+02	2020-06-03 20:28:56+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-06-03 20:28:39+02	2020-06-03 20:29:24+02	print	libraqm	\N	libraqm-0.6.0
2020-06-03 20:29:25+02	2020-06-03 20:30:39+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-06-03 20:25:48+02	2020-06-03 20:31:02+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.p3
2020-06-03 20:24:06+02	2020-06-03 20:31:18+02	graphics	mesa-libs	\N	mesa-libs-19.0.8
2020-06-03 20:28:57+02	2020-06-03 20:31:49+02	multimedia	libass	\N	libass-0.14.0
2020-06-03 20:31:21+02	2020-06-03 20:32:20+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-06-03 20:31:51+02	2020-06-03 20:32:34+02	graphics	libGLU	\N	libGLU-9.0.1
2020-06-03 20:31:21+02	2020-06-03 20:33:04+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-03 20:32:35+02	2020-06-03 20:33:24+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-06-03 20:28:39+02	2020-06-03 20:33:39+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.7_2
2020-06-03 20:23:20+02	2020-06-03 20:34:11+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-03 20:33:25+02	2020-06-03 20:34:24+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-03 20:34:13+02	2020-06-03 20:34:37+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-03 20:34:24+02	2020-06-03 20:34:45+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-03 20:33:40+02	2020-06-03 20:34:49+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-03 20:31:21+02	2020-06-03 20:34:52+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-03 20:34:46+02	2020-06-03 20:35:27+02	devel	py-gobject3	py37	py37-gobject3-3.28.3
2020-06-03 20:34:50+02	2020-06-03 20:36:03+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-03 20:33:04+02	2020-06-03 20:36:33+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-03 20:36:04+02	2020-06-03 20:37:10+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-03 20:34:54+02	2020-06-03 20:37:23+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-03 20:37:11+02	2020-06-03 20:38:18+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-03 20:32:20+02	2020-06-03 20:38:29+02	graphics	mesa-dri	\N	mesa-dri-19.0.8
2020-06-03 20:37:24+02	2020-06-03 20:40:52+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-03 20:36:34+02	2020-06-03 20:41:49+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-03 20:40:53+02	2020-06-03 20:42:06+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-03 20:38:19+02	2020-06-03 20:42:17+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-03 20:42:18+02	2020-06-03 20:43:07+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-03 20:35:28+02	2020-06-03 20:43:20+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-03 20:42:07+02	2020-06-03 20:44:14+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-03 20:41:50+02	2020-06-03 20:44:29+02	lang	vala	\N	vala-0.40.15,1
2020-06-03 20:43:08+02	2020-06-03 20:44:48+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-03 20:38:31+02	2020-06-03 20:45:05+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-03 20:44:15+02	2020-06-03 20:45:09+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-03 20:44:29+02	2020-06-03 20:45:22+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-03 20:45:10+02	2020-06-03 20:45:54+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-03 20:45:08+02	2020-06-03 20:46:24+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-03 20:34:38+02	2020-06-03 20:46:31+02	multimedia	ffmpeg	\N	ffmpeg-4.2.3,1
2020-06-03 20:44:49+02	2020-06-03 20:46:37+02	graphics	graphene	\N	graphene-1.10.0
2020-06-03 20:46:25+02	2020-06-03 20:46:46+02	www	libwpe	\N	libwpe-1.4.0.1
2020-06-03 20:46:31+02	2020-06-03 20:47:09+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-03 20:47:10+02	2020-06-03 20:47:35+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-03 20:46:38+02	2020-06-03 20:47:43+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-03 20:46:47+02	2020-06-03 20:48:38+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-03 20:47:45+02	2020-06-03 20:49:18+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-03 20:47:35+02	2020-06-03 20:49:51+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-03 20:48:39+02	2020-06-03 20:50:03+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-03 20:49:51+02	2020-06-03 20:50:47+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-03 20:45:55+02	2020-06-03 20:50:47+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-03 20:49:20+02	2020-06-03 20:50:51+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-03 20:50:04+02	2020-06-03 20:51:16+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-03 20:50:51+02	2020-06-03 20:51:52+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-03 20:50:51+02	2020-06-03 20:51:55+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-03 20:45:23+02	2020-06-03 20:52:10+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-03 20:51:53+02	2020-06-03 20:53:08+02	devel	sdl20	\N	sdl2-2.0.10_1
2020-06-03 20:51:55+02	2020-06-03 20:53:36+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-03 20:51:17+02	2020-06-03 20:54:03+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-03 20:50:51+02	2020-06-03 20:54:08+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_1,1
2020-06-03 20:52:11+02	2020-06-03 20:54:08+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-06-03 20:53:09+02	2020-06-03 20:54:23+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-03 20:54:08+02	2020-06-03 20:54:44+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-03 20:54:09+02	2020-06-03 20:55:21+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-03 20:54:24+02	2020-06-03 20:55:22+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-03 20:54:45+02	2020-06-03 20:55:53+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-06-03 20:54:03+02	2020-06-03 20:55:58+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-03 20:55:23+02	2020-06-03 20:55:58+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-06-03 20:55:21+02	2020-06-03 20:56:45+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-03 20:56:02+02	2020-06-03 20:57:18+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-03 20:56:02+02	2020-06-03 20:57:23+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-03 20:56:47+02	2020-06-03 20:58:41+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-03 20:55:55+02	2020-06-03 20:58:44+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-28 21:32:49+02	2020-09-28 21:34:07+02	net	gupnp	\N	gupnp-1.2.4
2020-06-03 20:57:24+02	2020-06-03 20:59:26+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-03 20:57:19+02	2020-06-03 21:00:10+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-03 20:59:29+02	2020-06-03 21:01:19+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-03 20:58:47+02	2020-06-03 21:01:29+02	net	remmina	\N	remmina-1.3.9
2020-06-03 21:00:12+02	2020-06-03 21:01:58+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-03 21:01:31+02	2020-06-03 21:03:46+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-03 21:01:20+02	2020-06-03 21:04:28+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-03 21:02:00+02	2020-06-03 21:06:43+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-03 21:04:35+02	2020-06-03 21:07:06+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-03 20:58:42+02	2020-06-03 21:07:22+02	security	gcr	\N	gcr-3.28.0
2020-06-03 21:06:45+02	2020-06-03 21:07:35+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-06-03 21:07:08+02	2020-06-03 21:08:20+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-06-03 21:07:38+02	2020-06-03 21:09:21+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-03 21:03:47+02	2020-06-03 21:09:45+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-03 21:07:23+02	2020-06-03 21:09:54+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-03 21:08:25+02	2020-06-03 21:11:01+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-03 21:11:02+02	2020-06-03 21:11:50+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-03 21:09:55+02	2020-06-03 21:12:09+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-03 20:43:21+02	2020-06-03 21:14:24+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-03 21:09:46+02	2020-06-03 21:14:26+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-03 21:11:58+02	2020-06-03 21:16:34+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-03 21:16:37+02	2020-06-03 21:17:53+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-03 21:09:22+02	2020-06-03 21:19:27+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.22.2
2020-06-03 21:17:54+02	2020-06-03 21:19:45+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-03 21:14:26+02	2020-06-03 21:20:07+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-03 21:19:32+02	2020-06-03 21:21:08+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-03 21:19:52+02	2020-06-03 21:21:57+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-03 21:20:10+02	2020-06-03 21:22:47+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-03 21:21:09+02	2020-06-03 21:22:56+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-06-03 21:22:00+02	2020-06-03 21:23:33+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-03 21:12:11+02	2020-06-03 21:24:01+02	net	libmateweather	\N	libmateweather-1.22.1
2020-06-03 21:22:51+02	2020-06-03 21:24:25+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-03 21:22:57+02	2020-06-03 21:25:21+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-03 21:24:02+02	2020-06-03 21:26:13+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-03 21:14:28+02	2020-06-03 21:26:33+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-03 21:25:23+02	2020-06-03 21:26:49+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-03 21:26:14+02	2020-06-03 21:28:52+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-03 21:26:56+02	2020-06-03 21:30:21+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-03 21:28:55+02	2020-06-03 21:31:22+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-03 21:30:25+02	2020-06-03 21:32:06+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-03 21:24:27+02	2020-06-03 21:32:18+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-03 21:31:25+02	2020-06-03 21:33:30+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-03 21:32:07+02	2020-06-03 21:34:10+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-03 21:33:32+02	2020-06-03 21:34:18+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-06-03 21:26:35+02	2020-06-03 21:34:53+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-03 21:23:34+02	2020-06-03 21:35:09+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-03 21:32:21+02	2020-06-03 21:35:32+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-06-03 21:34:19+02	2020-06-03 21:36:23+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-03 21:34:11+02	2020-06-03 21:36:24+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-03 21:34:55+02	2020-06-03 21:36:51+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-03 21:35:11+02	2020-06-03 21:37:01+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-03 21:35:33+02	2020-06-03 21:37:44+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-03 21:36:28+02	2020-06-03 21:38:10+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-03 21:36:52+02	2020-06-03 21:38:30+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-03 21:36:24+02	2020-06-03 21:39:17+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-03 21:37:45+02	2020-06-03 21:39:51+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-03 21:38:11+02	2020-06-03 21:39:57+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-03 21:37:02+02	2020-06-03 21:41:00+02	x11-fm	caja	\N	caja-1.22.3
2020-06-03 21:39:18+02	2020-06-03 21:41:07+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-03 21:38:31+02	2020-06-03 21:41:17+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-03 21:39:58+02	2020-06-03 21:41:20+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-03 21:39:52+02	2020-06-03 21:41:58+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-03 21:41:22+02	2020-06-03 21:42:01+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-06-03 21:41:03+02	2020-06-03 21:43:49+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-03 21:42:02+02	2020-06-03 21:43:51+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-03 21:41:08+02	2020-06-03 21:44:07+02	graphics	gegl	\N	gegl-0.4.22_6
2020-06-03 21:41:59+02	2020-06-03 21:44:18+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-03 21:44:19+02	2020-06-03 21:46:00+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-03 21:44:08+02	2020-06-03 21:46:33+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-03 21:43:52+02	2020-06-03 21:46:34+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-03 21:46:02+02	2020-06-03 21:46:56+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-06-03 21:46:37+02	2020-06-03 21:48:17+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-03 21:46:35+02	2020-06-03 21:49:05+02	x11	zenity	\N	zenity-3.28.1
2020-06-03 21:43:50+02	2020-06-03 21:49:24+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-03 21:49:06+02	2020-06-03 21:50:52+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-03 21:41:19+02	2020-06-03 21:51:10+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-03 21:46:57+02	2020-06-03 21:51:37+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_1,1
2020-06-03 21:48:19+02	2020-06-03 21:52:08+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-03 21:49:25+02	2020-06-03 21:52:16+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-03 21:51:39+02	2020-06-03 21:52:41+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-03 21:52:17+02	2020-06-03 21:52:50+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-03 21:51:11+02	2020-06-03 21:53:00+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-03 21:52:51+02	2020-06-03 21:53:09+02	graphics	glew	\N	glew-2.1.0
2020-06-03 21:53:02+02	2020-06-03 21:55:10+02	graphics	movit	\N	movit-1.6.2_2
2020-06-03 21:50:53+02	2020-06-03 21:55:48+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-03 21:55:12+02	2020-06-03 21:55:56+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-06-03 21:55:50+02	2020-06-03 21:56:18+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-06-03 21:55:57+02	2020-06-03 21:57:29+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-03 21:56:20+02	2020-06-03 21:58:54+02	print	cups-filters	\N	cups-filters-1.27.4_2
2020-06-03 21:52:10+02	2020-06-03 22:01:08+02	editors	emacs	full	emacs-26.3_4,3
2020-06-03 21:53:10+02	2020-06-03 22:01:13+02	graphics	opencv	\N	opencv-3.4.1_31
2020-06-03 21:58:56+02	2020-06-03 22:01:43+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-03 22:01:10+02	2020-06-03 22:05:01+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-06-03 22:05:03+02	2020-06-03 22:08:54+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-06-03 22:08:55+02	2020-06-03 22:16:15+02	x11	xorg	\N	xorg-7.7_3
2020-06-03 21:57:30+02	2020-06-03 22:19:04+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-03 21:52:41+02	2020-06-03 22:19:15+02	graphics	gimp-app	\N	gimp-app-2.10.18_7,1
2020-06-03 22:16:19+02	2020-06-03 22:20:27+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-03 22:19:08+02	2020-06-03 22:21:08+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-03 22:20:28+02	2020-06-03 22:22:51+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-03 20:53:37+02	2020-06-03 22:23:18+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-03 22:19:17+02	2020-06-03 22:23:43+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-03 22:22:53+02	2020-06-03 22:25:22+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-06-03 22:21:10+02	2020-06-03 22:26:08+02	x11-wm	marco	\N	marco-1.22.4
2020-06-03 22:23:44+02	2020-06-03 22:27:04+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-03 22:23:21+02	2020-06-03 22:28:25+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-03 22:26:13+02	2020-06-03 22:28:30+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-06-03 22:25:24+02	2020-06-03 22:29:12+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-03 22:28:31+02	2020-06-03 22:29:38+02	devel	py-gobject3	py27	py27-gobject3-3.28.3
2020-06-03 22:29:39+02	2020-06-03 22:30:14+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-06-03 22:29:13+02	2020-06-03 22:30:48+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-03 22:01:44+02	2020-06-03 22:31:57+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-03 22:28:26+02	2020-06-03 22:32:13+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-03 22:30:49+02	2020-06-03 22:32:15+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-06-03 22:31:59+02	2020-06-03 22:33:05+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-03 22:30:14+02	2020-06-03 22:33:09+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-03 22:27:06+02	2020-06-03 22:33:18+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-03 22:33:08+02	2020-06-03 22:33:52+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-03 22:33:11+02	2020-06-03 22:34:36+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-03 22:32:16+02	2020-06-03 22:34:37+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-03 22:33:53+02	2020-06-03 22:35:27+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-03 22:34:37+02	2020-06-03 22:37:12+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-03 22:33:19+02	2020-06-03 22:38:26+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-03 22:32:16+02	2020-06-03 22:39:40+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-03 22:38:29+02	2020-06-03 22:41:13+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-03 22:37:15+02	2020-06-03 22:41:20+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-03 22:39:42+02	2020-06-03 22:42:09+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-03 22:41:23+02	2020-06-03 22:43:33+02	print	libspectre	\N	libspectre-0.2.8
2020-06-03 22:35:29+02	2020-06-03 22:48:14+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-06-03 22:41:15+02	2020-06-03 22:50:26+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-03 22:50:30+02	2020-06-03 22:53:25+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-06-03 22:43:34+02	2020-06-03 22:54:39+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-03 22:42:11+02	2020-06-03 22:54:56+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-03 22:48:15+02	2020-06-03 22:55:05+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-06-03 22:53:27+02	2020-06-03 22:55:52+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-03 22:55:06+02	2020-06-03 22:56:45+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-03 22:55:53+02	2020-06-03 22:57:38+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-03 22:55:00+02	2020-06-03 22:57:48+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-03 22:57:49+02	2020-06-03 22:59:00+02	audio	faudio	\N	FAudio-20.05
2020-06-03 22:54:41+02	2020-06-03 22:59:35+02	x11-servers	xephyr	\N	xephyr-1.20.8_1,1
2020-06-03 22:59:00+02	2020-06-03 23:03:18+02	audio	mate-media	\N	mate-media-1.22.2
2020-06-03 22:57:41+02	2020-06-03 23:03:21+02	archivers	engrampa	\N	engrampa-1.22.2
2020-06-03 22:59:37+02	2020-06-03 23:04:39+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-06-03 22:34:37+02	2020-06-03 23:06:11+02	math	R	\N	R-4.0.0_1
2020-06-03 22:56:46+02	2020-06-03 23:06:37+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-03 23:04:42+02	2020-06-03 23:07:02+02	deskutils	mozo	\N	mozo-1.22.2
2020-06-03 23:03:22+02	2020-06-03 23:07:26+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-03 23:06:39+02	2020-06-03 23:07:47+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-06-03 23:06:13+02	2020-06-03 23:08:01+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-03 23:07:27+02	2020-06-03 23:09:11+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-03 23:07:03+02	2020-06-03 23:09:20+02	devel	ode	\N	ode-0.13_4,1
2020-06-03 23:03:22+02	2020-06-03 23:10:15+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-06-03 23:09:20+02	2020-06-03 23:10:15+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-06-03 23:10:17+02	2020-06-03 23:14:55+02	graphics	eom	\N	eom-1.22.2
2020-06-03 23:09:12+02	2020-06-03 23:15:30+02	editors	pluma	\N	pluma-1.22.2
2020-06-03 23:15:32+02	2020-06-03 23:16:48+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-06-03 23:10:17+02	2020-06-03 23:17:01+02	graphics	atril	\N	atril-1.22.3_6
2020-06-03 23:14:56+02	2020-06-03 23:17:41+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-03 23:16:50+02	2020-06-03 23:18:00+02	graphics	gle	\N	gle-3.1.0_7
2020-06-03 23:17:03+02	2020-06-03 23:18:02+02	graphics	gltt	\N	gltt-2.5.2_14
2020-06-03 23:18:02+02	2020-06-03 23:18:42+02	graphics	libplacebo	\N	libplacebo-2.43.1
2020-06-03 23:17:43+02	2020-06-03 23:20:05+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-03 23:18:43+02	2020-06-03 23:21:26+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-06-03 23:18:03+02	2020-06-03 23:27:23+02	graphics	py-gimp	\N	py27-gimp-2.10.18_7
2020-06-03 23:27:25+02	2020-06-03 23:35:44+02	math	mate-calc	\N	mate-calc-1.22.2
2020-06-03 23:35:45+02	2020-06-03 23:38:30+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-03 23:38:32+02	2020-06-03 23:42:46+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-03 23:20:06+02	2020-06-03 23:45:47+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-03 23:45:49+02	2020-06-03 23:47:52+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-03 23:47:53+02	2020-06-03 23:49:18+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-03 23:49:19+02	2020-06-03 23:50:53+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-03 23:07:49+02	2020-06-03 23:51:25+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-03 23:50:55+02	2020-06-03 23:52:34+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-03 23:51:27+02	2020-06-03 23:53:19+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-03 23:52:35+02	2020-06-03 23:54:08+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-03 23:53:20+02	2020-06-03 23:54:47+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-03 23:54:09+02	2020-06-03 23:56:00+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-03 23:54:48+02	2020-06-03 23:57:11+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-03 23:56:04+02	2020-06-03 23:57:35+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-03 23:08:03+02	2020-06-03 23:58:10+02	editors	libreoffice	\N	libreoffice-6.3.6_1
2020-06-03 23:57:12+02	2020-06-03 23:59:27+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-03 23:42:49+02	2020-06-04 00:00:57+02	multimedia	vlc	\N	vlc-3.0.10_4,4
2020-06-03 23:59:27+02	2020-06-04 00:03:20+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-03 23:58:12+02	2020-06-04 00:07:58+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-03 23:57:36+02	2020-06-04 00:08:52+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-04 00:07:59+02	2020-06-04 00:12:11+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-06-04 00:08:54+02	2020-06-04 00:12:26+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-06-04 00:12:28+02	2020-06-04 00:13:41+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-04 00:12:12+02	2020-06-04 00:14:13+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-04 00:13:42+02	2020-06-04 00:15:50+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-04 00:00:58+02	2020-06-04 00:15:51+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-04 00:15:51+02	2020-06-04 00:18:14+02	x11	mate-base	\N	mate-base-1.22.2
2020-06-04 00:15:53+02	2020-06-04 00:18:36+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-06-04 00:18:38+02	2020-06-04 00:20:19+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-06-04 00:18:15+02	2020-06-04 00:22:38+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-06-04 00:20:20+02	2020-06-04 00:22:54+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-04 00:22:39+02	2020-06-04 00:23:47+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-04 00:22:55+02	2020-06-04 00:26:25+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-04 00:26:27+02	2020-06-04 00:28:37+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-04 00:15:15+02	2020-06-04 00:29:16+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-06-04 00:23:48+02	2020-06-04 00:34:50+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-04 00:03:21+02	2020-06-04 00:40:18+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-04 00:40:19+02	2020-06-04 00:47:48+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-06-04 00:47:50+02	2020-06-04 00:55:11+02	editors	vim	\N	vim-8.2.0869
2020-06-04 00:34:52+02	2020-06-04 00:59:13+02	editors	codelite	\N	codelite-12.0_4
2020-06-03 23:21:28+02	2020-06-04 01:15:49+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-04 01:15:51+02	2020-06-04 01:20:12+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-04 00:55:13+02	2020-06-04 01:24:52+02	emulators	qemu	\N	qemu-4.1.1_3
2020-06-04 00:59:14+02	2020-06-04 01:25:09+02	emulators	wine	\N	wine-5.0_1,1
2020-06-04 01:24:54+02	2020-06-04 01:25:28+02	games	KnightCap	\N	knightcap-3.6_8
2020-06-04 01:25:11+02	2020-06-04 01:26:22+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-06-04 01:26:23+02	2020-06-04 01:27:25+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-06-04 00:28:38+02	2020-06-04 01:27:59+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-04 01:28:04+02	2020-06-04 01:29:23+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-06-04 01:27:26+02	2020-06-04 01:30:15+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-04 01:25:30+02	2020-06-04 01:30:37+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-04 01:29:24+02	2020-06-04 01:31:36+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-04 01:30:19+02	2020-06-04 01:32:16+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-04 01:30:41+02	2020-06-04 01:32:38+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-04 01:31:38+02	2020-06-04 01:34:41+02	games	xblast	\N	xblast-2.10.4_17
2020-06-04 01:32:40+02	2020-06-04 01:35:21+02	games	xcowsay	\N	xcowsay-1.5
2020-06-04 01:32:18+02	2020-06-04 01:35:23+02	games	xboard	\N	xboard-4.9.1_1
2020-06-04 00:29:18+02	2020-06-04 01:35:28+02	devel	electron7	\N	electron7-7.3.0_1
2020-06-04 01:35:23+02	2020-06-04 01:38:20+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-04 01:38:22+02	2020-06-04 01:40:44+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-04 01:20:15+02	2020-06-04 01:40:52+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-04 01:34:43+02	2020-06-04 01:41:43+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-04 01:40:46+02	2020-06-04 01:43:35+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-04 01:35:25+02	2020-06-04 01:43:49+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-04 01:43:42+02	2020-06-04 01:44:25+02	multimedia	libva-glx	\N	libva-glx-2.7.1
2020-06-04 01:43:58+02	2020-06-04 01:44:43+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-06-04 01:44:27+02	2020-06-04 01:50:09+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-04 01:44:44+02	2020-06-04 01:50:39+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-04 01:50:40+02	2020-06-04 01:53:46+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-04 01:35:34+02	2020-06-04 01:54:31+02	graphics	krita	\N	krita-4.2.9_4
2020-06-04 01:50:10+02	2020-06-04 01:55:04+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-04 01:53:47+02	2020-06-04 01:56:21+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-04 01:55:07+02	2020-06-04 01:56:38+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-04 01:56:22+02	2020-06-04 01:58:02+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-04 01:56:39+02	2020-06-04 01:58:39+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-04 01:58:04+02	2020-06-04 01:59:51+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-04 01:58:41+02	2020-06-04 02:00:08+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-04 01:59:52+02	2020-06-04 02:01:17+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-04 02:00:09+02	2020-06-04 02:01:28+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-04 01:54:32+02	2020-06-04 02:01:41+02	net	wireshark	\N	wireshark-3.2.4
2020-06-04 02:01:18+02	2020-06-04 02:02:44+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-04 02:01:30+02	2020-06-04 02:03:03+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-04 02:01:43+02	2020-06-04 02:03:08+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-04 02:02:45+02	2020-06-04 02:04:16+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-04 02:03:04+02	2020-06-04 02:07:46+02	print	auctex	full	auctex-emacs26-12.2
2020-06-04 02:04:17+02	2020-06-04 02:09:19+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-04 02:03:09+02	2020-06-04 02:10:15+02	print	fontforge	\N	fontforge-20190801_2
2020-06-04 02:09:20+02	2020-06-04 02:10:48+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_3
2020-06-04 02:10:48+02	2020-06-04 02:11:57+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-04 02:10:16+02	2020-06-04 02:12:04+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-04 02:11:58+02	2020-06-04 02:13:21+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-04 02:07:50+02	2020-06-04 02:15:18+02	print	texlive-full	\N	texlive-full-20150521
2020-06-04 02:13:25+02	2020-06-04 02:15:37+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-04 02:15:21+02	2020-06-04 02:16:46+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-04 02:16:47+02	2020-06-04 02:17:34+02	x11	xlockmore	\N	xlockmore-5.63
2020-06-04 02:15:39+02	2020-06-04 02:18:28+02	x11	mate	\N	mate-1.22.2
2020-06-04 02:17:35+02	2020-06-04 02:22:49+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-04 02:22:53+02	2020-06-04 02:25:25+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-04 02:25:28+02	2020-06-04 02:26:10+02	x11-fm	xdiskusage	\N	xdiskusage-1.51_1
2020-06-04 02:26:12+02	2020-06-04 02:28:29+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-04 02:18:29+02	2020-06-04 02:28:31+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-04 02:34:27+02	2020-06-04 02:34:50+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-04 01:41:44+02	2020-06-04 03:14:02+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-04 01:40:53+02	2020-06-04 03:43:55+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-04 02:28:34+02	2020-06-04 03:56:31+02	editors	vscode	\N	vscode-1.45.1_1
2020-06-04 02:12:06+02	2020-06-04 05:31:33+02	www	firefox	\N	firefox-77.0.1,1
2020-06-03 22:01:16+02	2020-06-04 19:42:24+02	www	chromium	\N	chromium-83.0.4103.61
2020-06-04 19:42:25+02	2020-06-04 19:43:19+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-04 19:43:20+02	2020-06-04 19:44:15+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-04 19:44:15+02	2020-06-04 19:45:09+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-04 20:05:50+02	2020-06-04 20:20:50+02	local	E590T-localbase	\N	E590T-localbase-2020060200
2020-06-05 01:09:48+02	2020-06-05 01:09:56+02	misc	pciids	\N	pciids-20200529
2020-06-05 01:09:56+02	2020-06-05 01:10:19+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-06-05 01:09:48+02	2020-06-05 01:10:27+02	graphics	lcms2	\N	lcms2-2.10
2020-06-05 01:09:48+02	2020-06-05 01:10:55+02	www	py-django22	py37	py37-django22-2.2.13
2020-06-05 01:10:20+02	2020-06-05 01:10:56+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-06-05 01:09:48+02	2020-06-05 01:11:28+02	www	libnghttp2	\N	libnghttp2-1.41.0
2020-06-05 01:10:56+02	2020-06-05 01:13:12+02	graphics	libraw	\N	libraw-0.19.5
2020-06-05 01:13:12+02	2020-06-05 01:13:43+02	multimedia	libva	\N	libva-2.7.1
2020-06-05 01:10:57+02	2020-06-05 01:16:34+02	graphics	mesa-libs	\N	mesa-libs-19.0.8
2020-06-05 01:13:45+02	2020-06-05 01:17:05+02	sysutils	hal	\N	hal-0.5.14_34
2020-06-05 01:10:28+02	2020-06-05 01:17:22+02	graphics	colord	\N	colord-1.3.5
2020-06-05 01:11:31+02	2020-06-05 01:17:24+02	ftp	curl	\N	curl-7.70.0_1
2020-06-05 01:17:06+02	2020-06-05 01:17:50+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-06-05 01:09:48+02	2020-06-05 01:20:07+02	security	gnutls	\N	gnutls-3.6.14
2020-06-05 01:17:51+02	2020-06-05 01:20:30+02	graphics	libGLU	\N	libGLU-9.0.1
2020-06-05 01:16:35+02	2020-06-05 01:22:21+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-05 01:20:32+02	2020-06-05 01:22:29+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-06-05 01:09:48+02	2020-06-05 01:24:17+02	devel	protobuf	\N	protobuf-3.12.2,1
2020-06-05 01:17:23+02	2020-06-05 01:24:27+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-05 01:24:18+02	2020-06-05 01:25:12+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-05 01:22:32+02	2020-06-05 01:26:01+02	devel	libsoup	\N	libsoup-2.62.3
2020-06-05 01:20:12+02	2020-06-05 01:26:03+02	print	cups	\N	cups-2.2.13
2020-06-05 01:25:15+02	2020-06-05 01:26:22+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-05 01:22:25+02	2020-06-05 01:27:20+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-05 01:24:29+02	2020-06-05 01:27:34+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-05 01:26:02+02	2020-06-05 01:29:15+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-06-05 01:27:20+02	2020-06-05 01:29:56+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-05 01:26:03+02	2020-06-05 01:30:46+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-06-05 01:29:16+02	2020-06-05 01:30:56+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-05 01:27:38+02	2020-06-05 01:32:56+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-05 01:32:57+02	2020-06-05 01:35:12+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-05 01:29:57+02	2020-06-05 01:36:09+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-05 01:35:13+02	2020-06-05 01:36:27+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-06-05 01:36:12+02	2020-06-05 01:37:16+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-05 01:37:17+02	2020-06-05 01:38:15+02	devel	py-gobject3	py37	py37-gobject3-3.28.3
2020-06-05 01:36:28+02	2020-06-05 01:38:38+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-05 01:30:50+02	2020-06-05 01:39:09+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-05 01:38:16+02	2020-06-05 01:39:28+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-05 01:38:39+02	2020-06-05 01:39:28+02	ftp	php73-curl	\N	php73-curl-7.3.18
2020-06-05 01:26:23+02	2020-06-05 01:39:53+02	graphics	mesa-dri	\N	mesa-dri-19.0.8
2020-06-05 01:30:57+02	2020-06-05 01:39:59+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-05 01:39:29+02	2020-06-05 01:40:16+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-05 01:39:10+02	2020-06-05 01:40:51+02	lang	vala	\N	vala-0.40.15,1
2020-06-05 01:17:26+02	2020-06-05 01:40:57+02	devel	cmake	\N	cmake-3.17.3
2020-06-05 01:39:54+02	2020-06-05 01:41:09+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-05 01:40:16+02	2020-06-05 01:41:10+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-05 01:40:00+02	2020-06-05 01:41:20+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-05 01:39:29+02	2020-06-05 01:41:30+02	graphics	graphene	\N	graphene-1.10.0
2020-06-05 01:41:09+02	2020-06-05 01:41:49+02	graphics	openjpeg15	\N	openjpeg15-1.5.2_1
2020-06-05 01:40:58+02	2020-06-05 01:41:54+02	graphics	openjpeg	\N	openjpeg-2.3.1
2020-06-05 01:41:11+02	2020-06-05 01:41:57+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-05 01:41:31+02	2020-06-05 01:42:02+02	www	libwpe	\N	libwpe-1.4.0.1
2020-06-05 01:40:52+02	2020-06-05 01:42:06+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-05 01:41:55+02	2020-06-05 01:42:56+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-05 01:41:58+02	2020-06-05 01:43:18+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-05 01:41:51+02	2020-06-05 01:43:29+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-05 01:42:02+02	2020-06-05 01:44:35+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-05 01:43:19+02	2020-06-05 01:45:22+02	net	geoclue	\N	geoclue-2.5.5
2020-06-05 01:43:30+02	2020-06-05 01:45:41+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-05 01:45:23+02	2020-06-05 01:45:55+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-05 01:45:56+02	2020-06-05 01:48:22+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-05 01:41:25+02	2020-06-05 01:49:07+02	graphics	opencv-core	\N	opencv-core-3.4.1_32
2020-06-05 01:45:42+02	2020-06-05 01:49:19+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-05 01:48:22+02	2020-06-05 01:49:59+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-05 01:42:56+02	2020-06-05 01:50:33+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-05 01:49:20+02	2020-06-05 01:50:38+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-05 01:50:01+02	2020-06-05 01:50:58+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-05 01:50:33+02	2020-06-05 01:51:39+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-05 01:50:39+02	2020-06-05 01:51:44+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-05 01:44:36+02	2020-06-05 01:52:19+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-05 01:42:06+02	2020-06-05 01:52:24+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-05 01:51:40+02	2020-06-05 01:53:18+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-05 01:51:00+02	2020-06-05 01:53:47+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_1,1
2020-06-05 01:52:28+02	2020-06-05 01:54:48+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-06-05 01:51:44+02	2020-06-05 01:54:57+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-05 01:54:58+02	2020-06-05 01:55:10+02	devel	libpci	\N	libpci-3.7.0
2020-06-05 01:53:19+02	2020-06-05 01:55:11+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-05 01:55:13+02	2020-06-05 01:56:29+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-05 01:55:11+02	2020-06-05 01:57:11+02	devel	sdl20	\N	sdl2-2.0.10_1
2020-06-05 01:56:30+02	2020-06-05 01:58:59+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-05 01:54:50+02	2020-06-05 01:59:26+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-05 01:57:14+02	2020-06-05 01:59:26+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-05 01:59:27+02	2020-06-05 01:59:59+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-06-05 01:59:01+02	2020-06-05 02:00:15+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-06-05 01:59:27+02	2020-06-05 02:00:34+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-05 02:00:15+02	2020-06-05 02:01:11+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-05 02:00:01+02	2020-06-05 02:01:14+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-05 02:00:35+02	2020-06-05 02:01:27+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-05 01:49:08+02	2020-06-05 02:01:47+02	multimedia	ffmpeg	\N	ffmpeg-4.2.3,1
2020-06-05 02:01:15+02	2020-06-05 02:02:37+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-05 02:01:27+02	2020-06-05 02:03:01+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-05 02:01:11+02	2020-06-05 02:03:31+02	databases	mysql57-client	\N	mysql57-client-5.7.30_2
2020-06-05 02:01:48+02	2020-06-05 02:03:56+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-05 02:02:37+02	2020-06-05 02:04:02+02	net	remmina	\N	remmina-1.3.9
2020-06-05 02:03:32+02	2020-06-05 02:04:18+02	x11	mate-menus	\N	mate-menus-1.22.1_1
2020-06-05 01:53:48+02	2020-06-05 02:04:19+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-05 02:03:58+02	2020-06-05 02:04:59+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-05 02:04:03+02	2020-06-05 02:04:59+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-05 02:03:02+02	2020-06-05 02:05:07+02	security	gcr	\N	gcr-3.28.0
2020-06-05 02:04:20+02	2020-06-05 02:05:27+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-05 02:05:28+02	2020-06-05 02:06:41+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-05 02:04:59+02	2020-06-05 02:06:42+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-05 02:04:59+02	2020-06-05 02:07:07+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-05 02:06:41+02	2020-06-05 02:07:16+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-06-05 02:06:43+02	2020-06-05 02:07:21+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-06-05 02:07:08+02	2020-06-05 02:08:23+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-05 02:07:17+02	2020-06-05 02:08:25+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-05 02:05:08+02	2020-06-05 02:09:27+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-05 02:07:22+02	2020-06-05 02:10:31+02	security	gnupg	\N	gnupg-2.2.20
2020-06-05 02:08:23+02	2020-06-05 02:10:36+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-05 02:10:38+02	2020-06-05 02:11:07+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-05 02:09:28+02	2020-06-05 02:11:49+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-05 02:10:32+02	2020-06-05 02:12:04+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-05 02:11:51+02	2020-06-05 02:12:57+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-05 02:08:26+02	2020-06-05 02:13:09+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.22.2
2020-06-05 02:11:08+02	2020-06-05 02:13:59+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-05 02:12:57+02	2020-06-05 02:14:11+02	security	gpgme	\N	gpgme-1.13.1
2020-06-05 02:13:10+02	2020-06-05 02:14:55+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-05 02:14:02+02	2020-06-05 02:15:32+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-05 02:14:12+02	2020-06-05 02:15:58+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-05 02:16:00+02	2020-06-05 02:17:17+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-06-05 02:15:32+02	2020-06-05 02:17:23+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-06-05 02:14:57+02	2020-06-05 02:17:28+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-05 02:17:18+02	2020-06-05 02:18:50+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-05 02:17:30+02	2020-06-05 02:19:15+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-05 02:17:25+02	2020-06-05 02:19:43+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-06-05 02:18:52+02	2020-06-05 02:22:07+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-05 02:19:44+02	2020-06-05 02:23:06+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-06-05 01:52:19+02	2020-06-05 02:23:14+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-05 02:12:05+02	2020-06-05 02:24:52+02	net	libmateweather	\N	libmateweather-1.22.1
2020-06-05 02:19:15+02	2020-06-05 02:25:08+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-06-05 02:23:08+02	2020-06-05 02:25:45+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-05 02:25:47+02	2020-06-05 02:27:44+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-05 02:23:19+02	2020-06-05 02:29:46+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-05 02:27:45+02	2020-06-05 02:31:01+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-05 02:29:52+02	2020-06-05 02:33:17+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-05 02:31:04+02	2020-06-05 02:33:25+02	devel	librest	\N	rest-0.7.93
2020-06-05 02:25:10+02	2020-06-05 02:33:58+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-05 02:33:18+02	2020-06-05 02:35:16+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-05 02:33:59+02	2020-06-05 02:36:02+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-05 02:22:10+02	2020-06-05 02:36:04+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-05 02:33:27+02	2020-06-05 02:36:41+02	graphics	netpbm	\N	netpbm-10.90.03
2020-06-05 02:35:18+02	2020-06-05 02:38:10+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-06-05 02:36:05+02	2020-06-05 02:38:19+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-05 02:36:04+02	2020-06-05 02:38:28+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-05 02:38:12+02	2020-06-05 02:39:39+02	x11	babl	\N	babl-0.1.74
2020-06-05 02:38:30+02	2020-06-05 02:39:41+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-06-05 02:24:55+02	2020-06-05 02:40:37+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-05 02:39:44+02	2020-06-05 02:43:07+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-05 02:38:21+02	2020-06-05 02:43:23+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-06-05 02:39:44+02	2020-06-05 02:43:37+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-05 02:43:25+02	2020-06-05 02:46:41+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-05 02:43:11+02	2020-06-05 02:46:48+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-05 02:43:39+02	2020-06-05 02:47:03+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-05 02:40:41+02	2020-06-05 02:49:45+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-05 02:46:50+02	2020-06-05 02:50:12+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-05 02:47:06+02	2020-06-05 02:50:14+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-05 02:46:47+02	2020-06-05 02:53:33+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-05 02:50:17+02	2020-06-05 02:53:46+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-05 02:50:14+02	2020-06-05 02:53:50+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-05 02:53:51+02	2020-06-05 02:57:59+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-05 02:53:55+02	2020-06-05 02:58:40+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-05 02:49:50+02	2020-06-05 03:01:23+02	x11-fm	caja	\N	caja-1.22.3
2020-06-05 02:58:02+02	2020-06-05 03:01:53+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-05 02:53:35+02	2020-06-05 03:03:36+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-05 02:58:43+02	2020-06-05 03:04:06+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-05 03:01:29+02	2020-06-05 03:06:44+02	graphics	gegl	\N	gegl-0.4.22_6
2020-06-05 03:01:56+02	2020-06-05 03:08:23+02	graphics	libcdr01	\N	libcdr01-0.1.5_8
2020-06-05 03:08:28+02	2020-06-05 03:10:58+02	net	liboauth	\N	liboauth-1.0.3_4
2020-06-05 03:06:48+02	2020-06-05 03:21:38+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-05 03:21:42+02	2020-06-05 03:24:34+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-06-05 03:03:38+02	2020-06-05 03:26:20+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-05 03:26:25+02	2020-06-05 03:29:20+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-05 03:24:37+02	2020-06-05 03:30:14+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-05 03:30:20+02	2020-06-05 03:39:44+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-05 03:40:00+02	2020-06-05 03:59:59+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-05 03:29:22+02	2020-06-05 04:06:00+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-05 04:00:11+02	2020-06-05 04:06:45+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-06-05 03:04:10+02	2020-06-05 04:06:58+02	mail	mailutils	\N	mailutils-3.9
2020-06-05 04:06:47+02	2020-06-05 04:08:10+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-06-05 03:11:03+02	2020-06-05 04:09:18+02	net	samba410	\N	samba410-4.10.15
2020-06-05 04:06:04+02	2020-06-05 04:09:21+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-05 04:08:13+02	2020-06-05 04:10:59+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-05 04:07:00+02	2020-06-05 04:11:50+02	x11	zenity	\N	zenity-3.28.1
2020-06-05 04:11:03+02	2020-06-05 04:12:50+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-05 02:36:42+02	2020-06-05 04:13:41+02	www	node	\N	node-13.14.0
2020-06-05 04:09:33+02	2020-06-05 04:14:21+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-05 04:11:51+02	2020-06-05 04:16:59+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-05 04:09:33+02	2020-06-05 04:17:17+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_1,1
2020-06-05 04:12:52+02	2020-06-05 04:19:43+02	devel	git	\N	git-2.27.0
2020-06-05 04:17:23+02	2020-06-05 04:20:18+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-05 04:20:22+02	2020-06-05 04:20:50+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-06-05 04:14:23+02	2020-06-05 04:20:58+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-05 04:13:45+02	2020-06-05 04:21:00+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-05 04:17:02+02	2020-06-05 04:21:20+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-05 04:21:02+02	2020-06-05 04:21:23+02	graphics	glew	\N	glew-2.1.0
2020-06-05 04:20:51+02	2020-06-05 04:21:27+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-05 04:21:24+02	2020-06-05 04:22:58+02	graphics	movit	\N	movit-1.6.2_2
2020-06-05 04:21:21+02	2020-06-05 04:23:08+02	graphics	libfreehand	\N	libfreehand-0.1.2_18
2020-06-05 04:23:09+02	2020-06-05 04:24:02+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-06-05 04:22:59+02	2020-06-05 04:24:41+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-06-05 04:24:04+02	2020-06-05 04:24:56+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-06-05 04:24:41+02	2020-06-05 04:25:24+02	net	gssdp	\N	gssdp-1.2.2
2020-06-05 04:24:57+02	2020-06-05 04:26:51+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-05 04:25:25+02	2020-06-05 04:29:48+02	print	cups-filters	\N	cups-filters-1.27.4_2
2020-06-05 04:19:44+02	2020-06-05 04:29:55+02	editors	emacs	full	emacs-26.3_4,3
2020-06-05 04:21:28+02	2020-06-05 04:31:31+02	graphics	opencv	\N	opencv-3.4.1_32
2020-06-05 04:29:50+02	2020-06-05 04:32:08+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-05 04:31:32+02	2020-06-05 04:33:32+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-06-05 04:29:56+02	2020-06-05 04:34:02+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-06-05 04:32:09+02	2020-06-05 04:45:33+02	www	apache24	\N	apache24-2.4.43
2020-06-05 04:26:53+02	2020-06-05 04:45:45+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-05 04:21:00+02	2020-06-05 04:46:49+02	graphics	gimp-app	\N	gimp-app-2.10.18_7,1
2020-06-05 04:45:48+02	2020-06-05 04:53:38+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-06-05 04:46:52+02	2020-06-05 04:54:14+02	x11	xorg	\N	xorg-7.7_3
2020-06-05 04:54:17+02	2020-06-05 05:01:45+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-05 04:53:42+02	2020-06-05 05:03:14+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-05 05:01:55+02	2020-06-05 05:05:53+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-05 05:05:55+02	2020-06-05 05:10:26+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-05 05:03:18+02	2020-06-05 05:12:52+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-05 05:12:54+02	2020-06-05 05:18:41+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-06-05 05:10:34+02	2020-06-05 05:21:42+02	x11-wm	marco	\N	marco-1.22.4
2020-06-05 05:21:45+02	2020-06-05 05:27:26+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-06-05 05:18:42+02	2020-06-05 05:28:37+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-05 04:45:38+02	2020-06-05 05:34:53+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-05 05:34:58+02	2020-06-05 05:38:19+02	devel	py-gobject3	py27	py27-gobject3-3.28.3
2020-06-05 05:28:41+02	2020-06-05 05:40:04+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-05 05:27:30+02	2020-06-05 05:40:33+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-05 05:40:07+02	2020-06-05 05:41:17+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-06-05 05:38:21+02	2020-06-05 05:42:05+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-05 05:42:07+02	2020-06-05 05:43:45+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-05 05:41:21+02	2020-06-05 05:44:10+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-06-05 05:40:37+02	2020-06-05 05:45:03+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-05 05:44:12+02	2020-06-05 05:45:49+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-05 05:45:09+02	2020-06-05 05:47:42+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-05 05:43:49+02	2020-06-05 05:48:21+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-05 05:47:45+02	2020-06-05 05:50:04+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-05 05:48:23+02	2020-06-05 05:51:01+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-05 05:45:52+02	2020-06-05 05:51:31+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-05 05:51:35+02	2020-06-05 05:55:34+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-06-05 05:55:41+02	2020-06-05 06:01:03+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-05 06:01:05+02	2020-06-05 06:05:16+02	net	freerdp	\N	freerdp-2.0.0.r4_9
2020-06-05 06:05:20+02	2020-06-05 06:07:03+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-05 04:33:33+02	2020-06-05 06:11:35+02	www	chromium	\N	chromium-83.0.4103.61
2020-06-05 06:11:41+02	2020-06-05 06:15:06+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-05 05:51:02+02	2020-06-05 06:15:16+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-06-05 06:15:19+02	2020-06-05 06:18:14+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-05 06:15:09+02	2020-06-05 06:20:14+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-05 06:20:21+02	2020-06-05 06:24:08+02	print	libspectre	\N	libspectre-0.2.8
2020-06-05 04:34:04+02	2020-06-05 06:30:16+02	www	node12	\N	node12-12.16.3
2020-06-05 06:07:06+02	2020-06-05 06:33:28+02	net	libcmis	\N	libcmis-0.5.2_1
2020-06-05 06:18:17+02	2020-06-05 06:34:02+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-05 06:33:30+02	2020-06-05 06:34:27+02	sysutils	hal-info	\N	hal-info-20091130
2020-06-05 05:50:10+02	2020-06-05 06:35:19+02	math	R	\N	R-4.0.0_1
2020-06-05 06:34:28+02	2020-06-05 06:36:24+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-06-05 06:35:21+02	2020-06-05 06:37:58+02	textproc	redland	\N	redland-1.0.17_4
2020-06-05 06:24:13+02	2020-06-05 06:40:34+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-05 06:37:59+02	2020-06-05 06:41:18+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-05 06:34:03+02	2020-06-05 06:42:43+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-06-05 06:30:19+02	2020-06-05 06:43:55+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-05 06:42:46+02	2020-06-05 06:44:38+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-05 06:41:21+02	2020-06-05 06:47:03+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-05 06:43:57+02	2020-06-05 06:47:05+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-05 06:47:09+02	2020-06-05 06:48:47+02	audio	faudio	\N	FAudio-20.06
2020-06-05 06:40:37+02	2020-06-05 06:49:28+02	x11-servers	xephyr	\N	xephyr-1.20.8_1,1
2020-06-05 06:48:50+02	2020-06-05 06:55:05+02	audio	mate-media	\N	mate-media-1.22.2
2020-06-05 06:47:04+02	2020-06-05 06:56:31+02	archivers	engrampa	\N	engrampa-1.22.2
2020-06-05 06:49:31+02	2020-06-05 06:57:25+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-06-05 06:44:41+02	2020-06-05 06:58:44+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-05 06:57:30+02	2020-06-05 07:00:21+02	deskutils	mozo	\N	mozo-1.22.2
2020-06-05 06:58:47+02	2020-06-05 07:00:35+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-05 06:55:11+02	2020-06-05 07:01:06+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-05 07:00:22+02	2020-06-05 07:01:50+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-06-05 07:01:53+02	2020-06-05 07:04:36+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-05 07:00:37+02	2020-06-05 07:05:43+02	devel	ode	\N	ode-0.13_4,1
2020-06-05 07:01:08+02	2020-06-05 07:09:52+02	devel	protobuf-c	\N	protobuf-c-1.3.2_5
2020-06-05 06:56:35+02	2020-06-05 07:10:50+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-06-05 07:10:51+02	2020-06-05 07:12:38+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-06-05 07:09:55+02	2020-06-05 07:24:38+02	editors	pluma	\N	pluma-1.22.2
2020-06-05 07:12:41+02	2020-06-05 07:25:15+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-06-05 07:25:17+02	2020-06-05 07:36:48+02	graphics	eom	\N	eom-1.22.2
2020-06-05 07:24:42+02	2020-06-05 07:40:40+02	graphics	atril	\N	atril-1.22.3_6
2020-06-05 07:36:52+02	2020-06-05 07:40:49+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-05 07:40:50+02	2020-06-05 07:44:32+02	graphics	gle	\N	gle-3.1.0_7
2020-06-05 07:40:42+02	2020-06-05 07:44:44+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-06-05 07:44:33+02	2020-06-05 07:46:39+02	graphics	gltt	\N	gltt-2.5.2_14
2020-06-05 07:46:44+02	2020-06-05 07:48:06+02	graphics	libplacebo	\N	libplacebo-2.43.1
2020-06-05 07:44:47+02	2020-06-05 07:49:02+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-05 06:36:27+02	2020-06-05 07:49:49+02	www	node10	\N	node10-10.20.1_1
2020-06-05 07:49:03+02	2020-06-05 07:50:44+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-06-05 07:48:08+02	2020-06-05 07:54:21+02	graphics	py-gimp	\N	py27-gimp-2.10.18_7
2020-06-05 07:04:39+02	2020-06-05 08:08:23+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-05 07:05:46+02	2020-06-05 08:13:53+02	editors	libreoffice	\N	libreoffice-6.3.6_1
2020-06-05 07:50:45+02	2020-06-05 08:36:10+02	java	openjdk11	\N	openjdk11-11.0.7+10.2
2020-06-05 08:36:12+02	2020-06-05 08:38:11+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-05 08:38:12+02	2020-06-05 08:40:25+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-05 08:40:26+02	2020-06-05 08:41:20+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-06-05 02:04:19+02	2020-06-05 08:45:34+02	lang	rust	\N	rust-1.44.0
2020-06-05 08:45:36+02	2020-06-05 08:52:18+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.2_1
2020-06-05 08:52:19+02	2020-06-05 08:53:52+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-05 08:53:53+02	2020-06-05 08:54:57+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-05 08:41:20+02	2020-06-05 08:55:21+02	multimedia	vlc	\N	vlc-3.0.10_5,4
2020-06-05 08:54:58+02	2020-06-05 08:56:05+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-05 08:55:22+02	2020-06-05 08:56:27+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-05 08:56:06+02	2020-06-05 08:57:16+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-05 07:54:32+02	2020-06-05 08:57:23+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-05 08:56:28+02	2020-06-05 08:57:27+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-05 08:57:16+02	2020-06-05 08:58:15+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-05 08:57:24+02	2020-06-05 08:58:45+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-05 08:57:28+02	2020-06-05 08:59:15+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-05 08:58:16+02	2020-06-05 08:59:25+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-05 08:58:45+02	2020-06-05 09:00:31+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-05 08:59:16+02	2020-06-05 09:00:55+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-05 09:00:55+02	2020-06-05 09:03:44+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-05 08:59:25+02	2020-06-05 09:08:04+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-05 09:00:32+02	2020-06-05 09:08:34+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-05 09:08:35+02	2020-06-05 09:09:21+02	print	transfig	\N	transfig-3.2.5e_4
2020-06-05 09:09:22+02	2020-06-05 09:11:35+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-06-05 09:03:45+02	2020-06-05 09:13:34+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-05 09:11:35+02	2020-06-05 09:13:52+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-06-05 09:13:35+02	2020-06-05 09:15:09+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-05 09:15:13+02	2020-06-05 09:16:08+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-06-05 09:16:11+02	2020-06-05 09:16:29+02	www	py-dj22-django-js-asset	py37	py37-dj22-django-js-asset-1.2.2
2020-06-05 09:16:29+02	2020-06-05 09:16:55+02	www	py-dj22-django-taggit	py37	py37-dj22-django-taggit-1.3.0
2020-06-05 09:16:58+02	2020-06-05 09:17:39+02	www	py-dj22-djangorestframework	py37	py37-dj22-djangorestframework-3.11.0
2020-06-05 09:17:42+02	2020-06-05 09:18:58+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-05 09:13:53+02	2020-06-05 09:19:52+02	textproc	ripgrep	\N	ripgrep-12.1.1_1
2020-06-05 09:19:52+02	2020-06-05 09:20:05+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-06-05 09:18:58+02	2020-06-05 09:20:16+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-05 09:20:35+02	2020-06-05 09:21:52+02	x11	mate-base	\N	mate-base-1.22.2
2020-06-05 09:21:53+02	2020-06-05 09:23:23+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-06-05 09:20:17+02	2020-06-05 09:25:02+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-06-05 09:23:24+02	2020-06-05 09:25:04+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-06-05 09:25:03+02	2020-06-05 09:25:26+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-06-05 09:25:05+02	2020-06-05 09:26:00+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-05 09:25:26+02	2020-06-05 09:26:01+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-05 09:26:01+02	2020-06-05 09:27:11+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-05 09:27:12+02	2020-06-05 09:28:27+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-05 09:26:01+02	2020-06-05 09:33:21+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-05 09:08:05+02	2020-06-05 09:34:37+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-05 09:33:22+02	2020-06-05 09:34:49+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-05 08:08:28+02	2020-06-05 09:35:32+02	lang	julia11	\N	julia11-1.1.1_5
2020-06-05 09:28:28+02	2020-06-05 10:05:20+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-05 10:05:22+02	2020-06-05 10:05:43+02	devel	py-dj22-django-rq	py37	py37-dj22-django-rq-2.3.2
2020-06-05 10:05:43+02	2020-06-05 10:06:32+02	devel	tig	\N	tig-2.5.1
2020-06-05 10:06:33+02	2020-06-05 10:16:14+02	dns	bind916	\N	bind916-9.16.3_2
2020-06-05 09:34:38+02	2020-06-05 10:24:21+02	devel	electron7	\N	electron7-7.3.1
2020-06-05 10:16:14+02	2020-06-05 10:27:11+02	editors	codelite	\N	codelite-12.0_4
2020-06-05 10:24:26+02	2020-06-05 10:27:41+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-06-05 10:27:12+02	2020-06-05 10:29:38+02	editors	vim	\N	vim-8.2.0869
2020-06-05 09:34:50+02	2020-06-05 10:44:34+02	devel	ghidra	\N	ghidra-9.1
2020-06-05 10:27:42+02	2020-06-05 10:45:42+02	emulators	qemu	\N	qemu-4.1.1_3
2020-06-05 10:45:46+02	2020-06-05 10:46:15+02	games	KnightCap	\N	knightcap-3.6_8
2020-06-05 10:46:18+02	2020-06-05 10:47:07+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-06-05 10:47:09+02	2020-06-05 10:52:05+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-05 10:52:07+02	2020-06-05 10:53:09+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-06-05 10:53:12+02	2020-06-05 10:56:18+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-05 10:56:20+02	2020-06-05 10:57:30+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-06-05 10:57:31+02	2020-06-05 10:59:45+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-05 10:59:48+02	2020-06-05 11:01:44+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-05 11:01:46+02	2020-06-05 11:03:49+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-05 11:03:53+02	2020-06-05 11:06:18+02	games	xblast	\N	xblast-2.10.4_17
2020-06-05 10:44:36+02	2020-06-05 11:06:43+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-05 11:06:19+02	2020-06-05 11:07:48+02	games	xboard	\N	xboard-4.9.1_1
2020-06-05 11:06:45+02	2020-06-05 11:08:02+02	games	xcowsay	\N	xcowsay-1.5
2020-09-28 21:30:44+02	2020-09-28 21:34:46+02	x11	yelp	\N	yelp-3.38.0
2020-06-05 10:29:39+02	2020-06-05 11:12:16+02	emulators	wine	\N	wine-5.0.1,1
2020-06-05 11:12:19+02	2020-06-05 11:15:05+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-05 11:07:49+02	2020-06-05 11:15:17+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-05 11:08:03+02	2020-06-05 11:18:13+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-05 11:18:15+02	2020-06-05 11:20:20+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-05 11:15:06+02	2020-06-05 11:22:24+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-05 11:22:27+02	2020-06-05 11:25:07+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-05 11:15:20+02	2020-06-05 11:44:14+02	graphics	krita	\N	krita-4.2.9_4
2020-06-05 11:20:23+02	2020-06-05 12:00:39+02	java	openjdk12	\N	openjdk12-12.0.2+10.4
2020-06-05 11:25:09+02	2020-06-05 12:52:33+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-05 12:52:37+02	2020-06-05 12:53:48+02	lang	julia	\N	julia-1.0.3_3
2020-06-05 11:44:16+02	2020-06-05 13:01:58+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-05 13:02:01+02	2020-06-05 13:02:51+02	multimedia	libva-glx	\N	libva-glx-2.7.1
2020-06-05 13:02:54+02	2020-06-05 13:03:50+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-06-05 13:03:53+02	2020-06-05 13:10:19+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.p3
2020-06-05 13:10:22+02	2020-06-05 13:11:17+02	multimedia	libva-utils	\N	libva-utils-2.7.1
2020-06-05 13:11:19+02	2020-06-05 13:12:57+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-06-05 13:13:00+02	2020-06-05 13:19:41+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-05 13:19:43+02	2020-06-05 13:26:28+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-05 13:26:30+02	2020-06-05 13:33:33+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-05 13:33:35+02	2020-06-05 13:36:31+02	net	mosh	\N	mosh-1.3.2_13
2020-06-05 13:36:33+02	2020-06-05 13:41:16+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-05 13:41:20+02	2020-06-05 13:44:35+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-05 13:44:38+02	2020-06-05 13:52:51+02	net	wireshark	\N	wireshark-3.2.4
2020-06-05 13:52:52+02	2020-06-05 13:54:03+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-05 13:54:04+02	2020-06-05 13:55:23+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-05 13:55:24+02	2020-06-05 13:56:49+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-05 13:56:50+02	2020-06-05 13:57:58+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-05 13:57:59+02	2020-06-05 13:59:44+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-05 13:59:47+02	2020-06-05 14:01:34+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-05 14:01:36+02	2020-06-05 14:03:30+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-05 14:03:31+02	2020-06-05 14:05:28+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-05 14:05:30+02	2020-06-05 14:08:52+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-05 14:08:54+02	2020-06-05 14:11:23+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-05 14:11:26+02	2020-06-05 14:13:30+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-05 14:13:31+02	2020-06-05 14:15:33+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-05 14:15:35+02	2020-06-05 14:21:39+02	print	auctex	full	auctex-emacs26-12.2
2020-06-05 14:21:42+02	2020-06-05 14:23:42+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-05 12:53:50+02	2020-06-05 14:29:50+02	lang	mono	\N	mono-5.10.1.57_3
2020-06-05 14:29:54+02	2020-06-05 14:33:17+02	print	gv	\N	gv-3.7.4_5
2020-06-05 14:23:44+02	2020-06-05 14:37:40+02	print	fontforge	\N	fontforge-20190801_2
2020-06-05 14:33:21+02	2020-06-05 14:39:23+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-05 14:39:25+02	2020-06-05 14:41:28+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_3
2020-06-05 14:37:42+02	2020-06-05 14:42:56+02	security	clamav	\N	clamav-0.102.3_1,1
2020-06-05 14:41:29+02	2020-06-05 14:43:06+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-05 14:43:07+02	2020-06-05 14:44:19+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-05 14:42:57+02	2020-06-05 14:47:27+02	sysutils	exa	\N	exa-0.9.0_11
2020-06-05 14:44:19+02	2020-06-05 14:47:46+02	sysutils	hexyl	\N	hexyl-0.6.0_7
2020-06-05 14:47:47+02	2020-06-05 14:48:24+02	sysutils	x86info	\N	x86info-1.31.s03
2020-06-05 14:47:27+02	2020-06-05 14:59:44+02	sysutils	rsfetch	\N	rsfetch-2.0.0_3
2020-06-05 14:59:46+02	2020-06-05 15:01:15+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-05 14:48:26+02	2020-06-05 15:10:10+02	textproc	bat	\N	bat-0.15.0_1
2020-06-05 15:10:13+02	2020-06-05 15:11:13+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_1
2020-06-05 15:11:16+02	2020-06-05 15:23:46+02	www	mod_php73	\N	mod_php73-7.3.18
2020-06-05 15:23:49+02	2020-06-05 15:26:25+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-06-05 15:26:31+02	2020-06-05 15:27:27+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-06-05 15:27:30+02	2020-06-05 15:28:08+02	www	py-dj22-django-cacheops	py37	py37-dj22-django-cacheops-4.2
2020-06-05 15:28:13+02	2020-06-05 15:28:45+02	www	py-dj22-django-cors-headers	py37	py37-dj22-django-cors-headers-3.2.1
2020-06-05 15:28:48+02	2020-06-05 15:29:24+02	www	py-dj22-django-debug-toolbar	py37	py37-dj22-django-debug-toolbar-2.1
2020-06-05 15:29:26+02	2020-06-05 15:30:07+02	www	py-dj22-django-filter	py37	py37-dj22-django-filter-2.2.0
2020-06-05 15:30:11+02	2020-06-05 15:30:54+02	www	py-dj22-django-mptt	py37	py37-dj22-django-mptt-0.9.1
2020-06-05 15:30:55+02	2020-06-05 15:31:43+02	www	py-dj22-django-prometheus	py37	py37-dj22-django-prometheus-1.1.0_1
2020-06-05 15:31:46+02	2020-06-05 15:32:20+02	www	py-dj22-django-tables2	py37	py37-dj22-django-tables2-2.2.1
2020-06-05 15:32:22+02	2020-06-05 15:32:58+02	www	py-dj22-django-taggit-serializer	py37	py37-dj22-django-taggit-serializer-0.1.7
2020-06-05 12:00:40+02	2020-06-05 15:33:01+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200530
2020-06-05 15:33:00+02	2020-06-05 15:33:31+02	www	py-dj22-django-timezone-field	py37	py37-dj22-django-timezone-field-4.0
2020-06-05 15:33:03+02	2020-06-05 15:33:54+02	www	py-dj22-drf-yasg	py37	py37-dj22-drf-yasg-1.17.0
2020-06-05 15:33:57+02	2020-06-05 15:34:24+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-06-05 15:33:35+02	2020-06-05 15:36:22+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-05 15:36:23+02	2020-06-05 15:38:21+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-05 15:38:23+02	2020-06-05 15:40:04+02	x11	xlockmore	\N	xlockmore-5.63
2020-06-05 15:40:06+02	2020-06-05 15:49:03+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-05 15:49:06+02	2020-06-05 15:54:30+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-05 15:54:31+02	2020-06-05 15:55:23+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-05 15:55:23+02	2020-06-05 15:55:34+02	x11-fm	xdiskusage	\N	xdiskusage-1.51_1
2020-06-05 15:55:34+02	2020-06-05 15:55:53+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-05 15:34:26+02	2020-06-05 15:57:06+02	x11	alacritty	\N	alacritty-0.4.3
2020-06-05 15:57:30+02	2020-06-05 15:57:39+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-05 15:57:40+02	2020-06-05 15:58:25+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-05 09:35:34+02	2020-06-05 15:58:37+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200519
2020-06-05 15:58:26+02	2020-06-05 15:59:07+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-06-05 15:01:19+02	2020-06-05 16:01:27+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-05 15:57:07+02	2020-06-05 16:22:22+02	editors	vscode	\N	vscode-1.45.1_2
2020-06-05 22:33:55+02	2020-06-05 22:38:09+02	math	mate-calc	\N	mate-calc-1.22.2
2020-06-05 22:39:37+02	2020-06-05 22:43:28+02	x11	mate	\N	mate-1.22.2
2020-06-05 22:43:30+02	2020-06-05 22:45:13+02	x11	xlockmore	\N	xlockmore-5.63
2020-06-05 22:33:55+02	2020-06-05 22:48:04+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-05 22:48:04+02	2020-06-05 22:50:45+02	print	texlive-full	\N	texlive-full-20150521
2020-06-05 23:00:29+02	2020-06-05 23:15:11+02	local	E590T-localbase	\N	E590T-localbase-2020060200
2020-06-06 10:12:23+02	2020-06-06 10:12:34+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-06-06 10:12:35+02	2020-06-06 10:12:51+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-06-06 10:12:52+02	2020-06-06 10:20:44+02	graphics	mesa-libs	\N	mesa-libs-19.0.8
2020-06-06 10:20:46+02	2020-06-06 10:21:32+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-06-06 10:20:47+02	2020-06-06 10:22:05+02	graphics	libGLU	\N	libGLU-9.0.1
2020-06-06 10:12:23+02	2020-06-06 10:22:59+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-06 10:23:02+02	2020-06-06 10:23:52+02	www	libwpe	\N	libwpe-1.4.0.1
2020-06-06 10:20:47+02	2020-06-06 10:24:31+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-06 10:23:53+02	2020-06-06 10:24:54+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-06 10:20:46+02	2020-06-06 10:25:29+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-06 10:22:06+02	2020-06-06 10:26:22+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-06-06 10:24:56+02	2020-06-06 10:26:53+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-06 10:26:54+02	2020-06-06 10:28:05+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-06 10:24:34+02	2020-06-06 10:29:57+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-06 10:28:08+02	2020-06-06 10:30:14+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-06 10:25:32+02	2020-06-06 10:30:18+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-06 10:26:25+02	2020-06-06 10:31:56+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-06 10:31:57+02	2020-06-06 10:33:26+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-06 10:30:16+02	2020-06-06 10:33:38+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-06 10:12:23+02	2020-06-06 10:34:04+02	devel	gdb	py37	gdb-9.2
2020-06-06 10:30:20+02	2020-06-06 10:34:58+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-06 10:33:27+02	2020-06-06 10:35:18+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-06 10:21:34+02	2020-06-06 10:36:36+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_2
2020-06-06 10:29:59+02	2020-06-06 10:36:49+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-06 10:33:40+02	2020-06-06 10:36:55+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-06 10:35:19+02	2020-06-06 10:36:57+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-06 10:36:37+02	2020-06-06 10:36:59+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-06 10:37:00+02	2020-06-06 10:37:21+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_1
2020-06-06 10:36:55+02	2020-06-06 10:37:34+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-06 10:37:22+02	2020-06-06 10:37:51+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_1
2020-06-06 10:36:50+02	2020-06-06 10:38:19+02	lang	vala	\N	vala-0.40.15,1
2020-06-06 10:36:58+02	2020-06-06 10:38:19+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-06 10:34:06+02	2020-06-06 10:38:25+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-06 10:37:38+02	2020-06-06 10:38:27+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-06 10:38:20+02	2020-06-06 10:38:52+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-06 10:38:20+02	2020-06-06 10:39:07+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-06 10:38:28+02	2020-06-06 10:39:11+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-06 10:34:59+02	2020-06-06 10:39:26+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-06 10:38:26+02	2020-06-06 10:39:33+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-06 10:37:51+02	2020-06-06 10:39:38+02	graphics	graphene	\N	graphene-1.10.0
2020-06-06 10:38:53+02	2020-06-06 10:39:52+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-06 10:39:53+02	2020-06-06 10:41:08+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-06 10:39:38+02	2020-06-06 10:41:16+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-06 10:39:33+02	2020-06-06 10:41:19+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-06 10:41:20+02	2020-06-06 10:42:17+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-06 10:41:09+02	2020-06-06 10:42:23+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-06 10:41:17+02	2020-06-06 10:42:42+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-06 10:39:12+02	2020-06-06 10:43:18+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-06 10:42:18+02	2020-06-06 10:43:24+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-06 10:42:24+02	2020-06-06 10:43:37+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-06 10:43:37+02	2020-06-06 10:44:23+02	devel	sdl20	\N	sdl2-2.0.10_1
2020-06-06 10:43:19+02	2020-06-06 10:44:23+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-06 10:44:23+02	2020-06-06 10:45:17+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-06 10:39:08+02	2020-06-06 10:45:20+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-06 10:42:43+02	2020-06-06 10:45:35+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-06-06 10:44:24+02	2020-06-06 10:46:02+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-06 10:43:24+02	2020-06-06 10:46:09+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-06 10:45:18+02	2020-06-06 10:46:37+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-06 10:45:37+02	2020-06-06 10:46:46+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-06 10:45:21+02	2020-06-06 10:46:48+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-06-06 10:46:38+02	2020-06-06 10:47:30+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-06 10:46:49+02	2020-06-06 10:47:38+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-06 10:46:47+02	2020-06-06 10:47:39+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-06-06 10:46:09+02	2020-06-06 10:47:42+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-06 10:47:31+02	2020-06-06 10:47:47+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-06-06 10:47:43+02	2020-06-06 10:49:03+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-06 10:47:40+02	2020-06-06 10:49:05+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-06 10:47:48+02	2020-06-06 10:49:33+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-06 10:47:39+02	2020-06-06 10:49:51+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-06 10:49:06+02	2020-06-06 10:50:30+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-06 10:49:04+02	2020-06-06 10:51:06+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-06 10:49:53+02	2020-06-06 10:51:20+02	net	remmina	\N	remmina-1.3.9
2020-06-06 10:50:32+02	2020-06-06 10:51:42+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-06 10:49:35+02	2020-06-06 10:51:54+02	security	gcr	\N	gcr-3.28.0
2020-06-06 10:51:07+02	2020-06-06 10:52:02+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-06 10:51:42+02	2020-06-06 10:53:14+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-06 10:51:54+02	2020-06-06 10:53:15+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-06 10:52:03+02	2020-06-06 10:53:44+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-06 10:53:15+02	2020-06-06 10:54:09+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-06 10:53:45+02	2020-06-06 10:54:44+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-06-06 10:46:03+02	2020-06-06 10:54:47+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-06 10:54:09+02	2020-06-06 10:54:50+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-06-06 10:54:45+02	2020-06-06 10:55:51+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-06 10:54:48+02	2020-06-06 10:56:21+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-06 10:53:15+02	2020-06-06 10:56:57+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-06 10:55:52+02	2020-06-06 10:57:15+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-06 10:57:00+02	2020-06-06 10:57:33+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-06 10:56:22+02	2020-06-06 10:57:45+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-06 10:54:51+02	2020-06-06 10:58:18+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.22.2
2020-06-06 12:16:11+02	2020-06-06 12:17:26+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-06-06 10:57:35+02	2020-06-06 10:58:28+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-06 10:58:19+02	2020-06-06 10:59:36+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-06 10:58:29+02	2020-06-06 10:59:45+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-06 10:57:16+02	2020-06-06 10:59:47+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-06 10:59:48+02	2020-06-06 11:00:30+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-06-06 10:59:38+02	2020-06-06 11:01:36+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-06 11:00:31+02	2020-06-06 11:02:14+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-06 10:59:45+02	2020-06-06 11:02:15+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-06 11:01:38+02	2020-06-06 11:03:16+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-06 11:02:14+02	2020-06-06 11:04:42+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-06 10:39:27+02	2020-06-06 11:05:19+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-06 11:03:17+02	2020-06-06 11:05:32+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-06 11:04:45+02	2020-06-06 11:06:21+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-06 10:57:47+02	2020-06-06 11:08:14+02	net	libmateweather	\N	libmateweather-1.22.1
2020-06-06 11:06:24+02	2020-06-06 11:09:19+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-06 11:05:22+02	2020-06-06 11:11:45+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-06 11:09:21+02	2020-06-06 11:12:19+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-06 11:11:46+02	2020-06-06 11:13:50+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-06 11:12:23+02	2020-06-06 11:14:01+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-06 11:02:17+02	2020-06-06 11:14:23+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-06 11:08:16+02	2020-06-06 11:14:59+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-06 11:13:52+02	2020-06-06 11:15:11+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-06 11:14:02+02	2020-06-06 11:15:20+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-06 11:15:01+02	2020-06-06 11:15:29+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-06-06 11:15:21+02	2020-06-06 11:16:53+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-06 11:14:25+02	2020-06-06 11:16:57+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-06-06 11:15:12+02	2020-06-06 11:17:02+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-06 11:15:30+02	2020-06-06 11:17:07+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-06 11:05:33+02	2020-06-06 11:18:14+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-06 11:16:54+02	2020-06-06 11:18:36+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-06 11:16:58+02	2020-06-06 11:18:39+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-06 11:17:08+02	2020-06-06 11:18:43+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-06 11:18:37+02	2020-06-06 11:20:15+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-06 11:17:03+02	2020-06-06 11:20:25+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-06 11:18:44+02	2020-06-06 11:20:53+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-06 11:20:18+02	2020-06-06 11:22:03+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-06 11:20:54+02	2020-06-06 11:22:48+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-06 11:18:40+02	2020-06-06 11:22:50+02	x11-fm	caja	\N	caja-1.22.3
2020-06-06 11:20:26+02	2020-06-06 11:23:18+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-06 11:22:05+02	2020-06-06 11:23:45+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-06 11:18:15+02	2020-06-06 11:23:51+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-06 11:22:50+02	2020-06-06 11:24:08+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-06 11:24:10+02	2020-06-06 11:25:08+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-06-06 11:22:51+02	2020-06-06 11:25:17+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-06 11:23:19+02	2020-06-06 11:26:00+02	graphics	gegl	\N	gegl-0.4.22_6
2020-06-06 11:25:18+02	2020-06-06 11:26:37+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-06 11:25:09+02	2020-06-06 11:27:08+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-06 11:23:52+02	2020-06-06 11:27:35+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-06 11:26:38+02	2020-06-06 11:29:06+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-06 11:27:36+02	2020-06-06 11:29:13+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-06 11:27:09+02	2020-06-06 11:29:22+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-06 11:29:09+02	2020-06-06 11:29:55+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-06-06 11:29:23+02	2020-06-06 11:30:51+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-06 11:26:02+02	2020-06-06 11:31:17+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-06 11:29:13+02	2020-06-06 11:31:38+02	x11	zenity	\N	zenity-3.28.1
2020-06-06 11:23:46+02	2020-06-06 11:32:04+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-06 11:31:18+02	2020-06-06 11:32:43+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-06 11:29:57+02	2020-06-06 11:33:50+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-06-06 11:30:52+02	2020-06-06 11:34:10+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-06 11:31:38+02	2020-06-06 11:34:37+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-06 11:32:04+02	2020-06-06 11:35:21+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-06 11:34:11+02	2020-06-06 11:35:45+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-06 11:35:23+02	2020-06-06 11:36:29+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-06 11:33:51+02	2020-06-06 11:37:20+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-06 11:36:32+02	2020-06-06 11:37:22+02	graphics	glew	\N	glew-2.1.0
2020-06-06 11:32:44+02	2020-06-06 11:37:50+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-06 11:37:51+02	2020-06-06 11:38:21+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-06-06 11:37:22+02	2020-06-06 11:38:22+02	graphics	movit	\N	movit-1.6.2_2
2020-06-06 11:38:21+02	2020-06-06 11:38:48+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-06-06 11:38:23+02	2020-06-06 11:39:33+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-06 11:38:48+02	2020-06-06 11:39:52+02	print	cups-filters	\N	cups-filters-1.27.4_2
2020-06-06 11:39:53+02	2020-06-06 11:41:50+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-06 11:34:38+02	2020-06-06 11:42:10+02	editors	emacs	full	emacs-26.3_4,3
2020-06-06 11:37:23+02	2020-06-06 11:43:58+02	graphics	opencv	\N	opencv-3.4.1_32
2020-06-06 11:41:51+02	2020-06-06 11:46:11+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-06-06 11:46:13+02	2020-06-06 11:49:15+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-06-06 11:49:17+02	2020-06-06 11:55:10+02	x11	xorg	\N	xorg-7.7_3
2020-06-06 11:35:46+02	2020-06-06 11:56:11+02	graphics	gimp-app	\N	gimp-app-2.10.18_7,1
2020-06-06 11:39:33+02	2020-06-06 11:59:37+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-06 11:56:15+02	2020-06-06 12:01:11+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-06 11:55:13+02	2020-06-06 12:01:32+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-06 11:59:39+02	2020-06-06 12:02:53+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-06 12:01:34+02	2020-06-06 12:04:26+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-06 12:01:17+02	2020-06-06 12:07:02+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-06 12:04:28+02	2020-06-06 12:07:46+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-06-06 12:02:55+02	2020-06-06 12:09:00+02	x11-wm	marco	\N	marco-1.22.4
2020-06-06 12:07:50+02	2020-06-06 12:11:27+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-06-06 12:07:05+02	2020-06-06 12:13:08+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-06 12:13:10+02	2020-06-06 12:14:43+02	devel	py-gobject3	py27	py27-gobject3-3.28.3_1
2020-06-06 11:43:59+02	2020-06-06 12:16:06+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-06 12:14:45+02	2020-06-06 12:18:11+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-06 12:09:04+02	2020-06-06 12:18:16+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-06 12:11:29+02	2020-06-06 12:19:12+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-06 12:18:13+02	2020-06-06 12:19:57+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-06-06 12:18:19+02	2020-06-06 12:20:02+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-06-06 12:19:14+02	2020-06-06 12:20:25+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-06 12:17:30+02	2020-06-06 12:20:58+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-06 12:20:04+02	2020-06-06 12:21:07+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-06 12:20:28+02	2020-06-06 12:22:18+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-06 12:21:09+02	2020-06-06 12:22:54+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-06 12:20:00+02	2020-06-06 12:22:54+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-06 12:22:21+02	2020-06-06 12:24:50+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-06 12:21:00+02	2020-06-06 12:25:59+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-06 12:26:00+02	2020-06-06 12:28:37+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-06 12:24:53+02	2020-06-06 12:28:42+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-06 12:28:38+02	2020-06-06 12:31:23+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-06 12:31:26+02	2020-06-06 12:32:43+02	print	libspectre	\N	libspectre-0.2.8
2020-06-06 12:22:57+02	2020-06-06 12:35:42+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-06-06 12:28:47+02	2020-06-06 12:39:31+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-06 12:39:34+02	2020-06-06 12:45:15+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-06-06 12:32:45+02	2020-06-06 12:47:16+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-06 12:45:17+02	2020-06-06 12:47:53+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-06-06 11:42:12+02	2020-06-06 12:48:08+02	www	chromium	\N	chromium-83.0.4103.61
2020-06-06 12:35:48+02	2020-06-06 12:48:33+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-06 12:47:18+02	2020-06-06 12:49:45+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-06 12:48:12+02	2020-06-06 12:50:52+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-06 12:49:47+02	2020-06-06 12:51:11+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-06 12:48:37+02	2020-06-06 12:51:19+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-06 12:47:55+02	2020-06-06 12:51:34+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-06-06 12:51:35+02	2020-06-06 12:52:44+02	audio	faudio	\N	FAudio-20.06
2020-06-06 12:50:53+02	2020-06-06 12:52:48+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-06 12:52:46+02	2020-06-06 12:55:20+02	audio	mate-media	\N	mate-media-1.22.2
2020-06-06 12:51:20+02	2020-06-06 12:55:31+02	archivers	engrampa	\N	engrampa-1.22.2
2020-06-06 12:22:57+02	2020-06-06 12:55:43+02	math	R	\N	R-4.0.0_1
2020-06-06 12:52:49+02	2020-06-06 12:56:04+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-06-06 12:56:07+02	2020-06-06 12:58:32+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-06 12:55:50+02	2020-06-06 12:59:03+02	deskutils	mozo	\N	mozo-1.22.2
2020-06-06 12:58:36+02	2020-06-06 13:00:11+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-06-06 12:55:22+02	2020-06-06 13:00:51+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-06 12:51:12+02	2020-06-06 13:01:48+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-06 12:59:06+02	2020-06-06 13:02:25+02	devel	ode	\N	ode-0.13_4,1
2020-06-06 13:00:13+02	2020-06-06 13:02:31+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-06 13:02:31+02	2020-06-06 13:03:23+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-06-06 12:55:34+02	2020-06-06 13:04:40+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-06-06 13:02:26+02	2020-06-06 13:10:07+02	editors	pluma	\N	pluma-1.22.2
2020-06-06 13:04:41+02	2020-06-06 13:10:36+02	graphics	eom	\N	eom-1.22.2
2020-06-06 13:03:26+02	2020-06-06 13:11:27+02	graphics	atril	\N	atril-1.22.3_6
2020-06-06 13:10:37+02	2020-06-06 13:11:31+02	graphics	gle	\N	gle-3.1.0_7
2020-06-06 13:11:28+02	2020-06-06 13:12:25+02	graphics	gltt	\N	gltt-2.5.2_14
2020-06-06 13:10:10+02	2020-06-06 13:13:06+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-06 13:12:26+02	2020-06-06 13:13:07+02	graphics	libplacebo	\N	libplacebo-2.43.1
2020-06-06 13:11:33+02	2020-06-06 13:13:27+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-06 13:13:08+02	2020-06-06 13:14:17+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-06-06 13:13:06+02	2020-06-06 13:19:14+02	graphics	py-gimp	\N	py27-gimp-2.10.18_7
2020-06-06 13:19:17+02	2020-06-06 13:24:58+02	math	mate-calc	\N	mate-calc-1.22.2
2020-06-06 13:25:01+02	2020-06-06 13:27:25+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-06 13:27:28+02	2020-06-06 13:30:28+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-06 13:13:28+02	2020-06-06 13:35:04+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-06 13:35:06+02	2020-06-06 13:37:25+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-06 13:37:27+02	2020-06-06 13:39:35+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-06 13:39:36+02	2020-06-06 13:41:23+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-06 13:41:26+02	2020-06-06 13:42:31+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-06 13:01:51+02	2020-06-06 13:42:51+02	editors	libreoffice	\N	libreoffice-6.3.6_1
2020-06-06 13:00:53+02	2020-06-06 13:43:28+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-06 13:42:32+02	2020-06-06 13:43:43+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-06 13:42:53+02	2020-06-06 13:43:55+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-06 13:30:29+02	2020-06-06 13:44:17+02	multimedia	vlc	\N	vlc-3.0.10_5,4
2020-06-06 13:43:29+02	2020-06-06 13:44:31+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-06 13:43:43+02	2020-06-06 13:45:04+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-06 13:44:18+02	2020-06-06 13:45:29+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-06 13:43:56+02	2020-06-06 13:45:44+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-06 13:44:32+02	2020-06-06 13:46:15+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-06 13:45:05+02	2020-06-06 13:46:46+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-06 13:46:15+02	2020-06-06 13:49:09+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-06 13:45:45+02	2020-06-06 13:55:20+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-06 13:45:30+02	2020-06-06 13:56:36+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-06 13:55:21+02	2020-06-06 13:57:42+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-06-06 13:46:46+02	2020-06-06 13:58:54+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-06 13:56:37+02	2020-06-06 13:58:56+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-06-06 13:57:43+02	2020-06-06 13:59:09+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-06 13:58:55+02	2020-06-06 13:59:44+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-06 13:58:56+02	2020-06-06 14:00:25+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-06 13:59:45+02	2020-06-06 14:01:36+02	x11	mate-base	\N	mate-base-1.22.2
2020-06-06 14:00:26+02	2020-06-06 14:03:18+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-06-06 14:03:19+02	2020-06-06 14:04:25+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-06-06 14:01:38+02	2020-06-06 14:04:43+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-06-06 14:04:44+02	2020-06-06 14:05:28+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-06 14:04:26+02	2020-06-06 14:05:38+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-06 13:14:19+02	2020-06-06 14:06:04+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-06 13:59:10+02	2020-06-06 14:06:05+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-06-06 14:05:28+02	2020-06-06 14:07:13+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-06 14:06:05+02	2020-06-06 14:07:19+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-06 14:07:14+02	2020-06-06 14:09:38+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-06 10:51:21+02	2020-06-06 14:14:34+02	lang	rust	\N	rust-1.44.0
2020-06-06 14:05:39+02	2020-06-06 14:18:43+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-06 14:14:36+02	2020-06-06 14:25:37+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.2_1
2020-06-06 14:18:44+02	2020-06-06 14:26:47+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-06-06 14:25:39+02	2020-06-06 14:30:28+02	editors	vim	\N	vim-8.2.0869
2020-06-06 14:09:39+02	2020-06-06 14:32:21+02	editors	codelite	\N	codelite-12.0_4
2020-06-06 14:26:48+02	2020-06-06 14:53:32+02	emulators	qemu	\N	qemu-4.1.1_3
2020-06-06 14:53:34+02	2020-06-06 14:53:53+02	games	KnightCap	\N	knightcap-3.6_8
2020-06-06 14:53:54+02	2020-06-06 14:54:22+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-06-06 13:49:10+02	2020-06-06 14:55:21+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-06 14:30:30+02	2020-06-06 14:55:40+02	emulators	wine	\N	wine-5.0.1,1
2020-06-06 14:32:22+02	2020-06-06 14:55:48+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-06 14:55:22+02	2020-06-06 14:55:53+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-06-06 14:55:51+02	2020-06-06 14:56:18+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-06-06 14:55:54+02	2020-06-06 14:56:54+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-06 14:56:19+02	2020-06-06 14:57:05+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-06 14:54:23+02	2020-06-06 14:57:10+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-06 14:55:42+02	2020-06-06 14:57:12+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-06 14:56:55+02	2020-06-06 14:57:34+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-06 14:57:05+02	2020-06-06 14:57:51+02	games	xblast	\N	xblast-2.10.4_17
2020-06-06 14:57:13+02	2020-06-06 14:58:04+02	games	xcowsay	\N	xcowsay-1.5
2020-06-06 14:57:11+02	2020-06-06 14:58:08+02	games	xboard	\N	xboard-4.9.1_1
2020-06-06 14:06:06+02	2020-06-06 14:58:55+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-06 14:57:52+02	2020-06-06 14:59:21+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-06 14:57:35+02	2020-06-06 14:59:53+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-06 14:58:58+02	2020-06-06 15:00:34+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-06 14:59:22+02	2020-06-06 15:01:07+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-06 15:01:08+02	2020-06-06 15:01:34+02	multimedia	libva-glx	\N	libva-glx-2.7.1
2020-06-06 15:01:35+02	2020-06-06 15:02:11+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-06-06 14:58:04+02	2020-06-06 15:03:58+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-06 14:07:21+02	2020-06-06 15:04:07+02	devel	electron7	\N	electron7-7.3.1
2020-06-06 15:02:12+02	2020-06-06 15:06:45+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-06 15:03:59+02	2020-06-06 15:09:05+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-06 15:04:13+02	2020-06-06 15:09:56+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-06 15:06:46+02	2020-06-06 15:10:08+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-06 15:09:07+02	2020-06-06 15:11:07+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-06 15:10:09+02	2020-06-06 15:11:12+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-06 14:58:09+02	2020-06-06 15:12:19+02	graphics	krita	\N	krita-4.2.9_4
2020-06-06 15:11:08+02	2020-06-06 15:12:35+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-06 15:11:12+02	2020-06-06 15:12:38+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-06 15:12:21+02	2020-06-06 15:13:34+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-06 15:12:36+02	2020-06-06 15:13:40+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-06 15:12:40+02	2020-06-06 15:13:42+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-06 15:13:35+02	2020-06-06 15:14:36+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-06 15:13:41+02	2020-06-06 15:14:41+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-06 15:09:57+02	2020-06-06 15:14:58+02	net	wireshark	\N	wireshark-3.2.4
2020-06-06 15:13:43+02	2020-06-06 15:15:19+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-06 15:14:36+02	2020-06-06 15:15:45+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-06 15:14:42+02	2020-06-06 15:15:48+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-06 15:14:59+02	2020-06-06 15:16:06+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-06 15:15:20+02	2020-06-06 15:18:45+02	print	auctex	full	auctex-emacs26-12.2
2020-06-06 15:15:49+02	2020-06-06 15:18:55+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-06 15:18:46+02	2020-06-06 15:19:46+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_3
2020-06-06 15:15:45+02	2020-06-06 15:19:48+02	print	fontforge	\N	fontforge-20190801_2
2020-06-06 15:18:56+02	2020-06-06 15:20:07+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-06 15:19:49+02	2020-06-06 15:20:13+02	sysutils	panicmail	\N	panicmail-1.1
2020-06-06 15:19:46+02	2020-06-06 15:20:31+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-06 15:20:07+02	2020-06-06 15:20:54+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-06 15:16:07+02	2020-06-06 15:21:46+02	print	texlive-full	\N	texlive-full-20150521
2020-06-06 15:20:32+02	2020-06-06 15:21:53+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-06 15:20:55+02	2020-06-06 15:21:58+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-06 15:21:55+02	2020-06-06 15:22:27+02	x11	xlockmore	\N	xlockmore-5.63
2020-06-06 15:21:47+02	2020-06-06 15:23:55+02	x11	mate	\N	mate-1.22.2
2020-06-06 15:21:59+02	2020-06-06 15:24:27+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-06 15:24:27+02	2020-06-06 15:24:40+02	x11-fm	xdiskusage	\N	xdiskusage-1.51_1
2020-06-06 15:23:56+02	2020-06-06 15:25:00+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-06 15:24:41+02	2020-06-06 15:25:09+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-06 15:27:41+02	2020-06-06 15:27:57+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-06 15:22:28+02	2020-06-06 15:29:15+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-06 14:59:54+02	2020-06-06 15:42:28+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-06 15:00:37+02	2020-06-06 15:49:27+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-06 15:20:13+02	2020-06-06 16:04:03+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-06 15:25:09+02	2020-06-06 16:06:45+02	editors	vscode	\N	vscode-1.45.1_2
2020-06-06 16:11:46+02	2020-06-06 16:11:54+02	x11	sterm	\N	sterm-0.8.3
2020-06-06 16:22:18+02	2020-06-06 16:37:09+02	local	E590T-localbase	\N	E590T-localbase-2020060200
2020-06-06 18:06:48+02	2020-06-06 18:08:09+02	graphics	mesa-libs	\N	mesa-libs-19.0.8
2020-06-06 18:08:09+02	2020-06-06 18:08:34+02	graphics	libGLU	\N	libGLU-9.0.1
2020-06-06 18:08:10+02	2020-06-06 18:08:35+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-06-06 18:08:36+02	2020-06-06 18:08:58+02	www	libwpe	\N	libwpe-1.4.0.1
2020-06-06 18:08:10+02	2020-06-06 18:09:13+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-06 18:06:48+02	2020-06-06 18:09:14+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-06 18:08:34+02	2020-06-06 18:09:15+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-06-06 18:08:59+02	2020-06-06 18:09:20+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-06 18:09:22+02	2020-06-06 18:10:11+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-06 18:09:15+02	2020-06-06 18:10:29+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-06 18:10:14+02	2020-06-06 18:10:50+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-06 18:08:10+02	2020-06-06 18:11:34+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-06 18:09:15+02	2020-06-06 18:11:36+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-06 18:10:54+02	2020-06-06 18:11:52+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_1
2020-06-06 18:10:32+02	2020-06-06 18:12:26+02	devel	sdl20	\N	sdl2-2.0.10_1
2020-06-06 18:11:37+02	2020-06-06 18:13:00+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-06 18:08:10+02	2020-06-06 18:13:07+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_2
2020-06-06 18:09:14+02	2020-06-06 18:13:09+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-06 18:12:28+02	2020-06-06 18:13:19+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-06 18:11:35+02	2020-06-06 18:14:04+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-06 18:13:07+02	2020-06-06 18:14:22+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-06 18:13:20+02	2020-06-06 18:14:34+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-06 18:13:01+02	2020-06-06 18:15:06+02	graphics	graphene	\N	graphene-1.10.0
2020-06-06 18:14:23+02	2020-06-06 18:15:33+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-06 18:15:34+02	2020-06-06 18:17:30+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-06 18:14:05+02	2020-06-06 18:17:45+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-06 18:14:35+02	2020-06-06 18:17:48+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-06 18:13:10+02	2020-06-06 18:17:59+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-06 18:17:31+02	2020-06-06 18:18:22+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-06 18:11:53+02	2020-06-06 18:18:22+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-06 18:17:49+02	2020-06-06 18:19:03+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-06 18:18:28+02	2020-06-06 18:19:12+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-06 18:17:45+02	2020-06-06 18:19:30+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-06 18:18:00+02	2020-06-06 18:19:53+02	lang	vala	\N	vala-0.40.15,1
2020-06-06 18:15:10+02	2020-06-06 18:20:24+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-06 18:19:13+02	2020-06-06 18:20:45+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-06 18:19:05+02	2020-06-06 18:20:48+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-06 18:19:30+02	2020-06-06 18:20:54+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-06 18:19:53+02	2020-06-06 18:21:01+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-06 18:20:46+02	2020-06-06 18:21:34+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-06 18:20:25+02	2020-06-06 18:21:44+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-06 18:20:55+02	2020-06-06 18:22:15+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-06 18:20:49+02	2020-06-06 18:22:17+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-06 18:22:15+02	2020-06-06 18:23:10+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-06 18:22:18+02	2020-06-06 18:23:21+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-06 18:21:44+02	2020-06-06 18:23:39+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-06 18:23:22+02	2020-06-06 18:24:23+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-06 18:24:24+02	2020-06-06 18:26:00+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-06 18:21:35+02	2020-06-06 18:26:54+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-06 18:23:11+02	2020-06-06 18:27:18+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-06-06 18:23:40+02	2020-06-06 18:27:19+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-06 18:26:02+02	2020-06-06 18:27:27+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-06 18:27:28+02	2020-06-06 18:27:43+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-06-06 18:26:55+02	2020-06-06 18:27:58+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-06 18:27:19+02	2020-06-06 18:28:13+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-06 18:27:19+02	2020-06-06 18:28:16+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-06-06 18:21:01+02	2020-06-06 18:28:30+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-06 18:27:44+02	2020-06-06 18:28:41+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-06 18:28:14+02	2020-06-06 18:29:11+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-06 18:27:59+02	2020-06-06 18:29:24+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-06 18:28:16+02	2020-06-06 18:29:38+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-06 18:28:41+02	2020-06-06 18:29:49+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-06 18:28:31+02	2020-06-06 18:29:59+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-06-06 18:29:49+02	2020-06-06 18:30:58+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-06 18:29:39+02	2020-06-06 18:31:06+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-06 18:29:26+02	2020-06-06 18:31:29+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-06 18:30:01+02	2020-06-06 18:32:29+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-06 18:31:06+02	2020-06-06 18:33:03+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-06 18:31:31+02	2020-06-06 18:33:36+02	net	remmina	\N	remmina-1.3.9
2020-06-06 18:32:30+02	2020-06-06 18:33:45+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-06 18:33:04+02	2020-06-06 18:34:54+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-06 18:33:37+02	2020-06-06 18:34:58+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-06 18:30:58+02	2020-06-06 18:35:06+02	security	gcr	\N	gcr-3.28.0
2020-06-06 18:33:46+02	2020-06-06 18:35:28+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-06 18:35:07+02	2020-06-06 18:35:31+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-06-06 18:34:59+02	2020-06-06 18:35:43+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-06 18:35:31+02	2020-06-06 18:36:16+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-06-06 18:35:32+02	2020-06-06 18:36:55+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-06 18:35:44+02	2020-06-06 18:37:38+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-06 18:36:55+02	2020-06-06 18:37:57+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-06 18:37:58+02	2020-06-06 18:38:21+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-06 18:37:40+02	2020-06-06 18:38:49+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-06 18:34:55+02	2020-06-06 18:38:54+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-06 18:29:12+02	2020-06-06 18:39:17+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-06 18:36:17+02	2020-06-06 18:39:24+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.22.2
2020-06-06 18:38:49+02	2020-06-06 18:39:31+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-06 18:38:25+02	2020-06-06 18:40:24+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-06 18:39:18+02	2020-06-06 18:40:24+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-06 18:39:25+02	2020-06-06 18:40:28+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-06 18:39:32+02	2020-06-06 18:40:29+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-06 18:40:25+02	2020-06-06 18:40:47+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-06-06 18:40:29+02	2020-06-06 18:41:24+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-06 18:40:30+02	2020-06-06 18:41:34+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-06 18:40:25+02	2020-06-06 18:42:37+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-06 18:40:47+02	2020-06-06 18:42:43+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-06 18:41:36+02	2020-06-06 18:43:20+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-06 18:42:37+02	2020-06-06 18:43:37+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-06 18:42:44+02	2020-06-06 18:44:16+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-06 18:43:37+02	2020-06-06 18:45:14+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-06 18:43:21+02	2020-06-06 18:45:38+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-06 18:44:17+02	2020-06-06 18:45:39+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-06 18:45:15+02	2020-06-06 18:47:11+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-06 18:45:40+02	2020-06-06 18:47:30+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-06 18:18:28+02	2020-06-06 18:47:44+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-06 18:47:12+02	2020-06-06 18:47:45+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-06-06 18:38:55+02	2020-06-06 18:48:29+02	net	libmateweather	\N	libmateweather-1.22.1
2020-06-06 18:45:40+02	2020-06-06 18:48:38+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-06-06 18:47:32+02	2020-06-06 18:49:38+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-06 18:48:40+02	2020-06-06 18:50:28+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-06 18:41:26+02	2020-06-06 18:51:35+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-06 18:49:38+02	2020-06-06 18:51:42+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-06 18:50:29+02	2020-06-06 18:52:04+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-06 18:48:29+02	2020-06-06 18:52:08+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-06 18:51:36+02	2020-06-06 18:52:55+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-06 18:47:45+02	2020-06-06 18:53:02+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-06 18:52:04+02	2020-06-06 18:53:20+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-06 18:52:08+02	2020-06-06 18:53:24+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-06 18:51:43+02	2020-06-06 18:53:47+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-06 18:53:21+02	2020-06-06 18:54:47+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-06 18:53:03+02	2020-06-06 18:54:53+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-06 18:53:48+02	2020-06-06 18:55:18+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-06 18:53:25+02	2020-06-06 18:55:23+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-06 18:52:57+02	2020-06-06 18:55:42+02	x11-fm	caja	\N	caja-1.22.3
2020-06-06 18:54:48+02	2020-06-06 18:55:49+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-06 18:54:53+02	2020-06-06 18:56:34+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-06 18:56:40+02	2020-06-06 18:57:25+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-06-06 18:55:18+02	2020-06-06 18:57:29+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-06 18:55:24+02	2020-06-06 18:57:46+02	graphics	gegl	\N	gegl-0.4.22_6
2020-06-06 18:55:50+02	2020-06-06 18:58:36+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-06 18:47:46+02	2020-06-06 18:58:44+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-06 18:57:30+02	2020-06-06 18:58:49+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-06 18:57:25+02	2020-06-06 18:58:53+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-06 18:57:47+02	2020-06-06 18:59:57+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-06 18:58:49+02	2020-06-06 19:00:09+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-06 18:58:54+02	2020-06-06 19:00:20+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-06 18:59:58+02	2020-06-06 19:00:38+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-06-06 18:58:37+02	2020-06-06 19:00:48+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-06 19:00:20+02	2020-06-06 19:01:46+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-06 19:00:10+02	2020-06-06 19:01:46+02	x11	zenity	\N	zenity-3.28.1
2020-06-06 18:55:43+02	2020-06-06 19:02:51+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-06 19:01:47+02	2020-06-06 19:03:12+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-06 19:00:49+02	2020-06-06 19:03:15+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-06 19:00:39+02	2020-06-06 19:03:20+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-06-06 19:01:47+02	2020-06-06 19:03:35+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-06 19:03:20+02	2020-06-06 19:03:52+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-06 18:58:45+02	2020-06-06 19:04:00+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-06 19:03:53+02	2020-06-06 19:04:19+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-06 19:03:16+02	2020-06-06 19:04:49+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-06 19:02:52+02	2020-06-06 19:04:57+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-06 19:04:50+02	2020-06-06 19:06:46+02	graphics	movit	\N	movit-1.6.2_2
2020-06-06 19:06:47+02	2020-06-06 19:07:32+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-06-06 19:03:14+02	2020-06-06 19:07:56+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-06 19:07:32+02	2020-06-06 19:08:04+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-06-06 19:04:20+02	2020-06-06 19:09:11+02	graphics	glew	\N	glew-2.2.0
2020-06-06 19:07:58+02	2020-06-06 19:09:18+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-06 19:08:05+02	2020-06-06 19:09:18+02	print	cups-filters	\N	cups-filters-1.27.4_2
2020-06-06 19:09:18+02	2020-06-06 19:11:01+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-06 19:03:36+02	2020-06-06 19:11:35+02	editors	emacs	full	emacs-26.3_4,3
2020-06-06 19:09:19+02	2020-06-06 19:12:02+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-06-06 19:04:58+02	2020-06-06 19:12:20+02	graphics	opencv	\N	opencv-3.4.1_32
2020-06-06 19:12:02+02	2020-06-06 19:13:34+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-06-06 19:04:01+02	2020-06-06 19:15:02+02	graphics	gimp-app	\N	gimp-app-2.10.18_7,1
2020-06-06 19:13:35+02	2020-06-06 19:15:33+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-06 19:12:21+02	2020-06-06 19:16:15+02	x11	xorg	\N	xorg-7.7_3
2020-06-06 19:15:03+02	2020-06-06 19:16:48+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-06 19:15:38+02	2020-06-06 19:17:14+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-06 19:16:48+02	2020-06-06 19:18:36+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-06 19:16:16+02	2020-06-06 19:19:10+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-06 19:18:37+02	2020-06-06 19:20:48+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-06-06 19:17:15+02	2020-06-06 19:21:48+02	x11-wm	marco	\N	marco-1.22.4
2020-06-06 19:09:11+02	2020-06-06 19:22:59+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-06 19:20:49+02	2020-06-06 19:24:14+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-06-06 19:19:12+02	2020-06-06 19:24:39+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-06 19:24:15+02	2020-06-06 19:25:42+02	devel	py-gobject3	py27	py27-gobject3-3.28.3_1
2020-06-06 19:25:43+02	2020-06-06 19:26:28+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-06-06 19:24:41+02	2020-06-06 19:26:45+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-06 19:26:48+02	2020-06-06 19:28:30+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-06-06 19:23:01+02	2020-06-06 19:29:01+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-06 19:21:50+02	2020-06-06 19:29:20+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-06 19:26:29+02	2020-06-06 19:29:23+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-06 19:29:02+02	2020-06-06 19:29:51+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-06 19:28:31+02	2020-06-06 19:30:11+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-06-06 19:29:24+02	2020-06-06 19:30:22+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-06 19:29:54+02	2020-06-06 19:31:24+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-06 19:29:22+02	2020-06-06 19:31:33+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-06 19:30:23+02	2020-06-06 19:32:00+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-06 19:31:25+02	2020-06-06 19:33:40+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-06 19:30:12+02	2020-06-06 19:34:50+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-06 19:11:35+02	2020-06-06 19:36:25+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-06 19:34:53+02	2020-06-06 19:37:14+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-06 19:33:41+02	2020-06-06 19:37:21+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-06 19:36:28+02	2020-06-06 19:38:22+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-06 19:37:22+02	2020-06-06 19:38:48+02	print	libspectre	\N	libspectre-0.2.8
2020-06-06 19:32:02+02	2020-06-06 19:43:39+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-06-06 19:37:15+02	2020-06-06 19:45:59+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-04 22:24:44+02	2020-10-04 22:29:25+02	print	cups	\N	cups-2.3.3_1
2020-06-06 19:43:40+02	2020-06-06 19:47:42+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-06-06 19:46:00+02	2020-06-06 19:48:02+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-06-06 19:47:43+02	2020-06-06 19:49:46+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-06 19:38:50+02	2020-06-06 19:50:13+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-06 19:38:23+02	2020-06-06 19:50:33+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-06 19:48:03+02	2020-06-06 19:51:31+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-06-06 19:50:15+02	2020-06-06 19:51:37+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-06 19:49:47+02	2020-06-06 19:52:05+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-06 19:50:35+02	2020-06-06 19:52:20+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-06 19:52:10+02	2020-06-06 19:53:02+02	audio	faudio	\N	FAudio-20.06
2020-06-06 19:52:21+02	2020-06-06 19:54:19+02	audio	mate-media	\N	mate-media-1.22.2
2020-06-06 19:51:38+02	2020-06-06 19:54:36+02	archivers	engrampa	\N	engrampa-1.22.2
2020-06-06 19:53:04+02	2020-06-06 19:55:41+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-06-06 19:54:21+02	2020-06-06 19:57:39+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-06 19:55:43+02	2020-06-06 19:58:02+02	deskutils	mozo	\N	mozo-1.22.2
2020-06-06 19:57:42+02	2020-06-06 19:58:57+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-06-06 19:58:04+02	2020-06-06 19:59:32+02	devel	ode	\N	ode-0.13_4,1
2020-06-06 19:51:32+02	2020-06-06 19:59:40+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-06 19:54:39+02	2020-06-06 20:00:07+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-06-06 19:11:02+02	2020-06-06 20:00:14+02	www	chromium	\N	chromium-83.0.4103.61
2020-06-06 19:59:00+02	2020-06-06 20:00:14+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-06 19:31:35+02	2020-06-06 20:00:23+02	math	R	\N	R-4.0.0_1
2020-06-06 20:00:16+02	2020-06-06 20:00:38+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-06-06 20:00:24+02	2020-06-06 20:01:25+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-06 20:00:16+02	2020-06-06 20:01:48+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-06 20:01:49+02	2020-06-06 20:04:11+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-06 20:00:08+02	2020-06-06 20:04:18+02	editors	pluma	\N	pluma-1.22.2
2020-06-06 20:01:25+02	2020-06-06 20:04:19+02	graphics	eom	\N	eom-1.22.2
2020-06-06 20:00:39+02	2020-06-06 20:04:24+02	graphics	atril	\N	atril-1.22.3_6
2020-06-06 20:04:12+02	2020-06-06 20:04:28+02	graphics	gle	\N	gle-3.1.0_7
2020-06-06 20:04:19+02	2020-06-06 20:04:35+02	graphics	gltt	\N	gltt-2.5.2_14
2020-06-06 20:04:24+02	2020-06-06 20:04:43+02	graphics	libplacebo	\N	libplacebo-2.43.1
2020-06-06 20:04:35+02	2020-06-06 20:05:11+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-06-06 20:04:19+02	2020-06-06 20:05:19+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-06 20:04:28+02	2020-06-06 20:06:54+02	graphics	py-gimp	\N	py27-gimp-2.10.18_7
2020-06-06 20:05:19+02	2020-06-06 20:07:54+02	math	mate-calc	\N	mate-calc-1.22.2
2020-06-06 20:06:56+02	2020-06-06 20:08:32+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-06 20:07:55+02	2020-06-06 20:10:44+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-06 20:10:45+02	2020-06-06 20:12:52+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-06 20:12:54+02	2020-06-06 20:14:15+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-06 20:14:16+02	2020-06-06 20:15:49+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-06 20:15:51+02	2020-06-06 20:17:13+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-06 20:17:14+02	2020-06-06 20:18:16+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-06 20:04:43+02	2020-06-06 20:19:10+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-06 20:18:17+02	2020-06-06 20:19:14+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-06 20:08:33+02	2020-06-06 20:19:32+02	multimedia	vlc	\N	vlc-3.0.10_5,4
2020-06-06 20:19:11+02	2020-06-06 20:20:09+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-06 20:19:14+02	2020-06-06 20:20:29+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-06 20:19:33+02	2020-06-06 20:21:06+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-06 20:20:10+02	2020-06-06 20:21:08+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-06 20:20:30+02	2020-06-06 20:21:59+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-06 20:21:07+02	2020-06-06 20:22:36+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-06 20:22:37+02	2020-06-06 20:25:00+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-06 19:59:40+02	2020-06-06 20:25:55+02	editors	libreoffice	\N	libreoffice-6.3.6_1
2020-06-06 20:21:09+02	2020-06-06 20:28:45+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-06 19:59:33+02	2020-06-06 20:29:08+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-06 20:21:59+02	2020-06-06 20:29:08+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-06 20:29:09+02	2020-06-06 20:30:31+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-06 20:28:46+02	2020-06-06 20:31:03+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-06-06 20:29:09+02	2020-06-06 20:31:12+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-06-06 20:30:31+02	2020-06-06 20:31:22+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-06 20:31:03+02	2020-06-06 20:32:25+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-06 20:31:23+02	2020-06-06 20:32:51+02	x11	mate-base	\N	mate-base-1.22.2
2020-06-06 20:25:01+02	2020-06-06 20:33:54+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-06 20:05:12+02	2020-06-06 20:34:26+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-06 20:32:26+02	2020-06-06 20:34:36+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-06-06 20:33:56+02	2020-06-06 20:34:59+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-06-06 20:32:52+02	2020-06-06 20:35:36+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-06-06 20:34:38+02	2020-06-06 20:35:47+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-06 20:34:27+02	2020-06-06 20:36:07+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-06 20:35:01+02	2020-06-06 20:36:38+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-06 20:35:47+02	2020-06-06 20:36:50+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-06 20:31:13+02	2020-06-06 20:37:11+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-06-06 20:36:40+02	2020-06-06 20:38:47+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-06 20:38:48+02	2020-06-06 20:44:04+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-06-06 20:35:37+02	2020-06-06 20:47:14+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-06 20:44:06+02	2020-06-06 20:49:13+02	editors	vim	\N	vim-8.2.0869
2020-06-06 20:37:12+02	2020-06-06 20:54:42+02	editors	codelite	\N	codelite-12.0_4
2020-06-06 20:47:16+02	2020-06-06 21:10:57+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-06 21:11:00+02	2020-06-06 21:11:30+02	games	KnightCap	\N	knightcap-3.6_8
2020-06-06 20:49:16+02	2020-06-06 21:11:31+02	emulators	wine	\N	wine-5.0.1,1
2020-06-06 21:11:32+02	2020-06-06 21:12:10+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-06-06 21:12:12+02	2020-06-06 21:12:46+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-06-06 21:12:46+02	2020-06-06 21:14:22+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-06 21:11:33+02	2020-06-06 21:14:23+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-06 20:54:43+02	2020-06-06 21:14:34+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-06 21:14:23+02	2020-06-06 21:14:49+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-06-06 21:14:24+02	2020-06-06 21:15:28+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-06 21:14:36+02	2020-06-06 21:15:30+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-06 21:14:52+02	2020-06-06 21:15:44+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-06 21:15:30+02	2020-06-06 21:16:48+02	games	xblast	\N	xblast-2.10.4_17
2020-06-06 21:15:31+02	2020-06-06 21:17:03+02	games	xboard	\N	xboard-4.9.1_1
2020-06-06 21:15:45+02	2020-06-06 21:17:09+02	games	xcowsay	\N	xcowsay-1.5
2020-06-06 21:17:03+02	2020-06-06 21:18:41+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-06 21:16:50+02	2020-06-06 21:19:33+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-06 21:19:33+02	2020-06-06 21:20:53+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-06 20:36:08+02	2020-06-06 21:21:53+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-06 21:17:10+02	2020-06-06 21:22:17+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-06 21:20:57+02	2020-06-06 21:22:50+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-06 21:22:54+02	2020-06-06 21:23:23+02	multimedia	libva-glx	\N	libva-glx-2.7.1
2020-06-06 21:23:24+02	2020-06-06 21:24:10+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-06-06 20:25:56+02	2020-06-06 21:24:43+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-06 20:36:50+02	2020-06-06 21:26:53+02	devel	electron7	\N	electron7-7.3.1
2020-06-06 21:24:11+02	2020-06-06 21:28:35+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-06 21:24:44+02	2020-06-06 21:29:12+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-06 21:28:36+02	2020-06-06 21:30:43+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-06 21:26:57+02	2020-06-06 21:31:08+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-06 21:29:13+02	2020-06-06 21:31:16+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-06 21:18:42+02	2020-06-06 21:31:52+02	graphics	krita	\N	krita-4.2.9_4
2020-06-06 21:31:09+02	2020-06-06 21:32:05+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-06 21:31:17+02	2020-06-06 21:32:17+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-06 21:31:53+02	2020-06-06 21:33:15+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-06 21:32:06+02	2020-06-06 21:33:28+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-06 21:32:18+02	2020-06-06 21:33:46+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-06 21:33:16+02	2020-06-06 21:34:17+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-06 21:33:29+02	2020-06-06 21:34:27+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-06 21:33:46+02	2020-06-06 21:34:43+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-06 21:34:28+02	2020-06-06 21:35:29+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-06 21:30:44+02	2020-06-06 21:35:33+02	net	wireshark	\N	wireshark-3.2.4
2020-06-06 21:34:44+02	2020-06-06 21:35:45+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-06 21:34:17+02	2020-06-06 21:35:51+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-06 21:35:30+02	2020-06-06 21:36:37+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-06 21:35:52+02	2020-06-06 21:38:39+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-06 21:35:34+02	2020-06-06 21:38:44+02	print	auctex	full	auctex-emacs26-12.2
2020-06-06 21:35:46+02	2020-06-06 21:39:27+02	print	fontforge	\N	fontforge-20190801_2
2020-06-06 21:38:39+02	2020-06-06 21:39:35+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_3
2020-06-06 21:38:45+02	2020-06-06 21:39:50+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-06 21:39:28+02	2020-06-06 21:40:07+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-06 21:39:35+02	2020-06-06 21:40:20+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-06 21:40:21+02	2020-06-06 21:41:16+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-06 21:40:08+02	2020-06-06 21:41:21+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-06 21:36:38+02	2020-06-06 21:41:40+02	print	texlive-full	\N	texlive-full-20150521
2020-06-06 21:41:21+02	2020-06-06 21:41:49+02	x11	xlockmore	\N	xlockmore-5.63
2020-06-06 21:41:17+02	2020-06-06 21:43:07+02	x11	mate	\N	mate-1.22.2
2020-06-06 21:41:41+02	2020-06-06 21:43:51+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-06 21:43:52+02	2020-06-06 21:44:04+02	x11-fm	xdiskusage	\N	xdiskusage-1.51_1
2020-06-06 21:43:08+02	2020-06-06 21:44:10+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-06 21:44:05+02	2020-06-06 21:44:29+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-06 21:41:50+02	2020-06-06 21:48:19+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-06 21:48:20+02	2020-06-06 21:48:31+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-06 21:21:56+02	2020-06-06 22:01:27+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-06 21:22:18+02	2020-06-06 22:08:29+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-06 21:39:51+02	2020-06-06 22:20:53+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-06 21:44:30+02	2020-06-06 22:23:10+02	editors	vscode	\N	vscode-1.45.1_2
2020-06-06 22:33:50+02	2020-06-06 22:48:33+02	local	E590T-localbase	\N	E590T-localbase-2020060200
2020-06-07 00:18:38+02	2020-06-07 05:13:06+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-07 05:18:26+02	2020-06-07 05:19:43+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-07 05:18:27+02	2020-06-07 05:20:13+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-07 05:18:27+02	2020-06-07 05:20:53+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-07 05:20:54+02	2020-06-07 05:21:03+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-07 05:20:14+02	2020-06-07 05:21:22+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-07 05:21:22+02	2020-06-07 05:22:22+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-07 05:19:44+02	2020-06-07 05:22:29+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-07 05:22:30+02	2020-06-07 05:23:19+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-07 05:23:20+02	2020-06-07 05:24:05+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-07 05:44:06+02	2020-06-07 05:58:26+02	local	E590T-localbase	\N	E590T-localbase-2020060200
2020-06-07 17:53:02+02	2020-06-07 17:53:18+02	print	t1utils	\N	t1utils-1.32
2020-06-07 17:53:02+02	2020-06-07 17:53:20+02	audio	libdssialsacompat	\N	libdssialsacompat-1.0.8a_2
2020-06-07 17:53:02+02	2020-06-07 17:53:31+02	graphics	potrace	\N	potrace-1.16
2020-06-07 17:53:02+02	2020-06-07 17:53:56+02	audio	liblo	\N	liblo-0.31_1
2020-06-07 17:53:19+02	2020-06-07 17:54:00+02	net	rsync	\N	rsync-3.1.3_1
2020-06-07 17:54:01+02	2020-06-07 17:54:44+02	audio	dssi	\N	dssi-1.1.1_8
2020-06-07 17:54:44+02	2020-06-07 17:56:04+02	audio	wavpack	\N	wavpack-5.3.0
2020-06-07 17:53:21+02	2020-06-07 17:57:01+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-07 17:53:32+02	2020-06-07 17:57:51+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-07 17:56:06+02	2020-06-07 17:58:22+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-06-07 17:54:45+02	2020-06-07 17:59:05+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-07 17:59:06+02	2020-06-07 17:59:19+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-07 17:53:02+02	2020-06-07 17:59:44+02	lang	guile1	\N	guile1-1.8.8
2020-06-07 17:53:02+02	2020-06-07 18:00:39+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-07 18:00:40+02	2020-06-07 18:11:02+02	print	lilypond	\N	lilypond-2.20.0
2020-06-07 18:11:03+02	2020-06-07 18:27:02+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-07 18:52:07+02	2020-06-07 19:26:59+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-06-07 20:05:08+02	2020-06-07 20:05:54+02	audio	liblo	\N	liblo-0.31_1
2020-06-07 20:05:55+02	2020-06-07 20:06:24+02	audio	dssi	\N	dssi-1.1.1_8
2020-06-07 20:05:08+02	2020-06-07 20:06:36+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-06-07 20:05:08+02	2020-06-07 20:07:41+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-07 20:07:42+02	2020-06-07 20:07:52+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-07 20:05:08+02	2020-06-07 20:07:54+02	lang	guile1	\N	guile1-1.8.8
2020-06-07 20:07:55+02	2020-06-07 20:13:03+02	print	lilypond	\N	lilypond-2.20.0
2020-06-07 20:13:04+02	2020-06-07 20:16:32+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-07 20:05:08+02	2020-06-07 20:18:42+02	math	R	\N	R-4.0.1
2020-06-07 20:18:43+02	2020-06-07 20:19:20+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-07 20:19:20+02	2020-06-07 20:55:52+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-07 21:06:59+02	2020-06-07 21:22:11+02	local	E590T-localbase	\N	E590T-localbase-2020060701
2020-06-07 22:32:57+02	2020-06-07 22:33:08+02	databases	pgtcl	\N	pgtcl-2.1.1_2
2020-06-07 22:32:57+02	2020-06-07 22:33:14+02	devel	p5-Term-ReadKey	\N	p5-Term-ReadKey-2.38_1
2020-06-07 22:33:08+02	2020-06-07 22:33:20+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-06-07 22:32:57+02	2020-06-07 22:33:20+02	databases	p5-DBI	\N	p5-DBI-1.643
2020-06-07 22:33:20+02	2020-06-07 22:33:38+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.12.0
2020-06-07 22:33:38+02	2020-06-07 22:33:51+02	databases	pgtop	\N	pgtop-0.05_5
2020-06-07 22:32:57+02	2020-06-07 22:36:21+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-07 22:36:21+02	2020-06-07 22:40:46+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-07 22:40:47+02	2020-06-07 22:44:15+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-07 22:44:16+02	2020-06-07 22:58:58+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-07 23:14:57+02	2020-06-07 23:17:04+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-07 23:22:30+02	2020-06-07 23:22:38+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-07 23:32:28+02	2020-06-07 23:47:10+02	local	E590T-localbase	\N	E590T-localbase-2020060702
2020-06-09 07:03:26+02	2020-06-09 07:03:36+02	x11-fonts	liberation-fonts-ttf	\N	liberation-fonts-ttf-2.1.1,2
2020-06-09 07:03:26+02	2020-06-09 07:03:47+02	devel	zziplib	\N	zziplib-0.13.71_1
2020-06-09 07:03:48+02	2020-06-09 07:06:02+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-09 07:03:26+02	2020-06-09 07:07:06+02	net	freerdp	\N	freerdp-2.1.1
2020-06-09 07:06:04+02	2020-06-09 07:11:55+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-09 07:11:57+02	2020-06-09 07:14:30+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-09 07:14:32+02	2020-06-09 07:15:47+02	textproc	lowdown	\N	lowdown-0.7.0
2020-06-09 07:03:26+02	2020-06-09 07:17:28+02	devel	gdb	py37	gdb-9.2_1
2020-06-09 07:03:26+02	2020-06-09 07:36:42+02	java	openjdk11	\N	openjdk11-11.0.7+10.2_1
2020-06-09 07:03:26+02	2020-06-09 07:40:28+02	multimedia	vlc	\N	vlc-3.0.10_6,4
2020-06-09 07:07:08+02	2020-06-09 07:40:39+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-09 07:40:30+02	2020-06-09 07:53:16+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-09 07:03:37+02	2020-06-09 07:57:41+02	editors	libreoffice	\N	libreoffice-6.3.6_1
2020-06-09 07:57:45+02	2020-06-09 08:04:06+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-06-09 08:04:08+02	2020-06-09 08:09:46+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-09 07:18:39+02	2020-06-09 08:16:46+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-09 07:40:41+02	2020-06-09 08:29:52+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-09 07:53:18+02	2020-06-09 08:31:53+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-06-09 08:29:53+02	2020-06-09 08:33:33+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-09 08:31:55+02	2020-06-09 08:39:53+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-09 07:36:44+02	2020-06-09 08:41:39+02	devel	ghidra	\N	ghidra-9.1
2020-06-09 08:33:34+02	2020-06-09 08:43:14+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-09 08:41:40+02	2020-06-09 08:44:35+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-09 08:39:54+02	2020-06-09 08:44:36+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-09 08:44:38+02	2020-06-09 08:47:18+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-09 08:43:16+02	2020-06-09 08:51:21+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-09 08:44:37+02	2020-06-09 08:52:27+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-09 08:51:22+02	2020-06-09 08:59:30+02	print	lilypond	\N	lilypond-2.20.0
2020-06-09 08:47:19+02	2020-06-09 09:00:18+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-09 08:52:28+02	2020-06-09 09:02:08+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-09 09:00:19+02	2020-06-09 09:02:56+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-09 08:59:31+02	2020-06-09 09:07:08+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-09 09:02:10+02	2020-06-09 09:11:36+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-09 09:07:09+02	2020-06-09 09:12:03+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-09 09:12:05+02	2020-06-09 09:12:26+02	sysutils	panicmail	\N	panicmail-1.1
2020-06-09 09:11:37+02	2020-06-09 09:14:46+02	print	auctex	full	auctex-emacs26-12.2
2020-06-09 09:14:45+02	2020-06-09 09:14:54+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-09 09:02:57+02	2020-06-09 09:27:02+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-09 09:27:04+02	2020-06-09 09:31:20+02	print	texlive-full	\N	texlive-full-20150521
2020-06-09 07:17:41+02	2020-06-09 09:55:46+02	lang	rust	\N	rust-1.44.0
2020-06-09 09:55:47+02	2020-06-09 09:59:20+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.2_1
2020-06-09 21:37:12+02	2020-06-09 21:38:37+02	net	v2ray	\N	v2ray-4.23.4
2020-06-09 21:37:12+02	2020-06-09 21:39:08+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-09 21:37:12+02	2020-06-09 21:39:36+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-09 21:39:37+02	2020-06-09 21:39:45+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-09 21:53:24+02	2020-06-09 22:08:22+02	local	E590T-localbase	\N	E590T-localbase-2020060702
2020-06-09 23:30:09+02	2020-06-09 23:30:41+02	textproc	py-sphinx	py37	py37-sphinx-3.0.4,1
2020-06-09 23:30:42+02	2020-06-09 23:31:03+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-06-09 23:30:09+02	2020-06-09 23:32:01+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_1
2020-06-09 23:32:02+02	2020-06-09 23:32:37+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-06-09 23:30:09+02	2020-06-09 23:32:41+02	sysutils	hexyl	\N	hexyl-0.8.0
2020-06-09 23:32:02+02	2020-06-09 23:32:55+02	graphics	libGLU	\N	libGLU-9.0.1
2020-06-09 23:32:43+02	2020-06-09 23:33:09+02	www	libwpe	\N	libwpe-1.4.0.1
2020-06-09 23:33:09+02	2020-06-09 23:33:43+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-09 23:32:02+02	2020-06-09 23:34:20+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-09 23:32:56+02	2020-06-09 23:35:22+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-06-09 23:32:02+02	2020-06-09 23:35:55+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-09 23:33:44+02	2020-06-09 23:36:14+02	devel	sdl20	\N	sdl2-2.0.10_1
2020-06-09 23:35:23+02	2020-06-09 23:36:39+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-09 23:32:37+02	2020-06-09 23:36:45+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_4
2020-06-09 23:36:41+02	2020-06-09 23:37:19+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-09 23:37:20+02	2020-06-09 23:37:46+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-09 23:36:47+02	2020-06-09 23:37:49+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-09 23:34:22+02	2020-06-09 23:38:18+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-09 23:36:15+02	2020-06-09 23:38:19+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-09 23:37:47+02	2020-06-09 23:38:31+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_1
2020-06-09 23:35:58+02	2020-06-09 23:38:34+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-09 23:38:20+02	2020-06-09 23:39:19+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-09 23:38:32+02	2020-06-09 23:39:21+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-09 23:30:09+02	2020-06-09 23:39:30+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-09 23:39:31+02	2020-06-09 23:40:43+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-09 23:37:51+02	2020-06-09 23:40:47+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-06-09 23:38:35+02	2020-06-09 23:41:40+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-09 23:39:20+02	2020-06-09 23:42:10+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-09 23:40:48+02	2020-06-09 23:42:13+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.70.0_2
2020-06-09 23:38:19+02	2020-06-09 23:42:22+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-09 23:42:11+02	2020-06-09 23:42:49+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-09 23:41:41+02	2020-06-09 23:43:04+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-09 23:39:21+02	2020-06-09 23:43:25+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-09 23:42:14+02	2020-06-09 23:43:29+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-09 23:42:50+02	2020-06-09 23:43:45+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-09 23:42:23+02	2020-06-09 23:43:53+02	lang	vala	\N	vala-0.40.15,1
2020-06-09 23:43:05+02	2020-06-09 23:44:36+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.70.0
2020-06-09 23:43:47+02	2020-06-09 23:44:47+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-09 23:43:26+02	2020-06-09 23:44:53+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-09 23:43:54+02	2020-06-09 23:44:53+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-09 23:44:37+02	2020-06-09 23:45:11+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-09 23:40:43+02	2020-06-09 23:45:33+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-09 23:43:30+02	2020-06-09 23:45:33+02	graphics	graphene	\N	graphene-1.10.0
2020-06-09 23:44:48+02	2020-06-09 23:45:44+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-09 23:45:44+02	2020-06-09 23:46:47+02	archivers	kf5-karchive	\N	kf5-karchive-5.70.0
2020-06-09 23:45:11+02	2020-06-09 23:47:03+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-09 23:45:34+02	2020-06-09 23:47:07+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-09 23:47:08+02	2020-06-09 23:48:05+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-09 23:46:47+02	2020-06-09 23:48:13+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-09 23:47:04+02	2020-06-09 23:48:20+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-09 23:44:54+02	2020-06-09 23:49:05+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-09 23:48:06+02	2020-06-09 23:49:11+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-09 23:48:13+02	2020-06-09 23:49:38+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-09 23:48:21+02	2020-06-09 23:49:40+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-09 23:49:06+02	2020-06-09 23:50:14+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-09 23:49:39+02	2020-06-09 23:50:15+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-09 23:49:41+02	2020-06-09 23:51:12+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-09 23:44:54+02	2020-06-09 23:51:14+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-09 23:50:15+02	2020-06-09 23:51:37+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-09 23:49:12+02	2020-06-09 23:51:54+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-09 23:50:15+02	2020-06-09 23:51:58+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-09 23:51:13+02	2020-06-09 23:52:10+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-06-09 23:51:38+02	2020-06-09 23:52:37+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-09 23:51:15+02	2020-06-09 23:52:39+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-06-09 23:52:37+02	2020-06-09 23:52:51+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-06-09 23:52:11+02	2020-06-09 23:53:01+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-09 23:51:59+02	2020-06-09 23:53:28+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-09 23:52:51+02	2020-06-09 23:53:57+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-09 23:53:02+02	2020-06-09 23:54:12+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.70.0
2020-06-09 23:53:29+02	2020-06-09 23:54:51+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-09 23:52:40+02	2020-06-09 23:54:53+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-09 23:53:58+02	2020-06-09 23:55:30+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-09 23:54:13+02	2020-06-09 23:55:56+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-09 23:54:54+02	2020-06-09 23:56:40+02	net	remmina	\N	remmina-1.3.9
2020-06-09 23:54:52+02	2020-06-09 23:56:55+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-09 23:55:57+02	2020-06-09 23:57:11+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-09 23:56:42+02	2020-06-09 23:57:38+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-09 23:55:31+02	2020-06-09 23:57:57+02	security	gcr	\N	gcr-3.28.0
2020-06-09 23:57:12+02	2020-06-09 23:58:34+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-09 23:57:39+02	2020-06-09 23:58:40+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-09 23:57:58+02	2020-06-09 23:59:19+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-09 23:58:41+02	2020-06-09 23:59:20+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-09 23:56:55+02	2020-06-09 23:59:45+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.70.0
2020-06-09 23:59:20+02	2020-06-09 23:59:57+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-06-09 23:59:21+02	2020-06-10 00:00:00+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-06-09 23:51:55+02	2020-06-10 00:00:30+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-09 23:59:46+02	2020-06-10 00:00:43+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-09 23:59:57+02	2020-06-10 00:01:12+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-10 00:00:31+02	2020-06-10 00:01:22+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-10 00:01:13+02	2020-06-10 00:01:34+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-10 00:00:44+02	2020-06-10 00:01:41+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-09 23:58:35+02	2020-06-10 00:01:48+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-10 00:01:35+02	2020-06-10 00:02:15+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-10 00:00:02+02	2020-06-10 00:02:24+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.22.2
2020-06-10 00:01:49+02	2020-06-10 00:02:43+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-10 00:02:15+02	2020-06-10 00:03:09+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-10 00:01:23+02	2020-06-10 00:03:10+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-10 00:02:24+02	2020-06-10 00:03:12+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-10 00:03:09+02	2020-06-10 00:03:29+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-06-10 00:03:11+02	2020-06-10 00:04:15+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-10 00:02:43+02	2020-06-10 00:04:22+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-10 00:03:12+02	2020-06-10 00:04:22+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-10 00:03:30+02	2020-06-10 00:05:13+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-10 00:04:23+02	2020-06-10 00:05:24+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-10 00:04:23+02	2020-06-10 00:05:36+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-10 00:05:13+02	2020-06-10 00:06:38+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-10 00:05:36+02	2020-06-10 00:06:40+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-10 00:05:24+02	2020-06-10 00:07:03+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-10 00:06:38+02	2020-06-10 00:07:38+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-10 00:06:41+02	2020-06-10 00:07:59+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-10 00:07:04+02	2020-06-10 00:08:18+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-10 00:08:00+02	2020-06-10 00:08:22+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-06-10 00:01:42+02	2020-06-10 00:08:45+02	net	libmateweather	\N	libmateweather-1.22.1
2020-06-10 00:07:40+02	2020-06-10 00:09:39+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-06-10 00:08:23+02	2020-06-10 00:09:48+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-09 23:45:34+02	2020-06-10 00:09:51+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-10 00:08:19+02	2020-06-10 00:09:55+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-10 00:09:40+02	2020-06-10 00:11:20+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-10 00:09:50+02	2020-06-10 00:11:20+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-10 00:08:46+02	2020-06-10 00:11:40+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-10 00:04:16+02	2020-06-10 00:12:07+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-10-05 00:50:09+02	2020-10-05 00:50:35+02	archivers	gcab	\N	gcab-1.4
2020-06-10 00:11:21+02	2020-06-10 00:12:25+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-10 00:11:41+02	2020-06-10 00:12:45+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-10 00:11:21+02	2020-06-10 00:12:56+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-10 00:12:08+02	2020-06-10 00:13:10+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-10 00:09:53+02	2020-06-10 00:13:37+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-10 00:12:56+02	2020-06-10 00:13:59+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-10 00:12:45+02	2020-06-10 00:14:13+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-10 00:12:26+02	2020-06-10 00:14:24+02	x11-fm	caja	\N	caja-1.22.3
2020-06-10 00:13:10+02	2020-06-10 00:14:27+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-10 00:13:38+02	2020-06-10 00:14:49+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-10 00:14:01+02	2020-06-10 00:14:51+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-10 00:14:13+02	2020-06-10 00:15:31+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-10 00:15:33+02	2020-06-10 00:16:04+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-06-10 00:14:24+02	2020-06-10 00:16:10+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-10 00:14:28+02	2020-06-10 00:16:16+02	graphics	gegl	\N	gegl-0.4.22_6
2020-06-10 00:14:52+02	2020-06-10 00:16:51+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-10 00:16:05+02	2020-06-10 00:17:11+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-10 00:16:11+02	2020-06-10 00:17:11+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-10 00:09:56+02	2020-06-10 00:17:55+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-10 00:16:17+02	2020-06-10 00:18:02+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-10 00:17:11+02	2020-06-10 00:18:19+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-10 00:17:12+02	2020-06-10 00:18:19+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-10 00:16:52+02	2020-06-10 00:18:31+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-10 00:18:03+02	2020-06-10 00:18:32+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-06-10 00:18:20+02	2020-06-10 00:19:28+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-10 00:18:20+02	2020-06-10 00:19:35+02	x11	zenity	\N	zenity-3.28.1
2020-06-10 00:14:50+02	2020-06-10 00:20:15+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-10 00:18:33+02	2020-06-10 00:20:26+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-10 00:18:32+02	2020-06-10 00:20:33+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-06-10 00:19:28+02	2020-06-10 00:20:34+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-10 00:19:36+02	2020-06-10 00:20:59+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-10 00:20:34+02	2020-06-10 00:21:04+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-10 00:21:05+02	2020-06-10 00:21:30+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-10 00:20:27+02	2020-06-10 00:21:46+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-10 00:21:47+02	2020-06-10 00:22:01+02	graphics	glew	\N	glew-2.2.0
2020-06-10 00:20:15+02	2020-06-10 00:22:03+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-10 00:17:56+02	2020-06-10 00:22:05+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-10 00:22:06+02	2020-06-10 00:22:48+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-06-10 00:22:02+02	2020-06-10 00:23:03+02	graphics	movit	\N	movit-1.6.2_2
2020-06-10 00:22:49+02	2020-06-10 00:23:15+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-06-10 00:20:35+02	2020-06-10 00:24:13+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-10 00:23:04+02	2020-06-10 00:24:23+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-10 00:23:16+02	2020-06-10 00:24:25+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-10 00:24:25+02	2020-06-10 00:25:41+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-10 00:20:59+02	2020-06-10 00:26:44+02	editors	emacs	full	emacs-26.3_4,3
2020-06-10 00:22:04+02	2020-06-10 00:27:26+02	graphics	opencv	\N	opencv-3.4.1_32
2020-06-10 00:25:42+02	2020-06-10 00:27:49+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-06-10 00:24:14+02	2020-06-10 00:28:13+02	print	fontforge	\N	fontforge-20190801_2
2020-06-10 00:27:50+02	2020-06-10 00:29:07+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-06-10 00:21:30+02	2020-06-10 00:29:51+02	graphics	gimp-app	\N	gimp-app-2.10.18_7,1
2020-06-10 00:29:08+02	2020-06-10 00:30:37+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-10 00:29:52+02	2020-06-10 00:31:21+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-10 00:30:38+02	2020-06-10 00:31:45+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-10 00:28:13+02	2020-06-10 00:32:03+02	x11	xorg	\N	xorg-7.7_3
2020-06-10 00:31:46+02	2020-06-10 00:33:37+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-10 00:31:21+02	2020-06-10 00:34:08+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-10 00:24:24+02	2020-06-10 00:34:56+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-10 00:34:10+02	2020-06-10 00:36:14+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-06-10 00:33:38+02	2020-06-10 00:39:12+02	x11-wm	marco	\N	marco-1.22.4
2020-06-10 00:36:16+02	2020-06-10 00:40:36+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-06-10 00:34:57+02	2020-06-10 00:41:29+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-10 00:32:04+02	2020-06-10 00:42:31+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-10 00:41:32+02	2020-06-10 00:42:55+02	devel	py-gobject3	py27	py27-gobject3-3.28.3_1
2020-06-10 00:42:57+02	2020-06-10 00:44:19+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-10 00:44:21+02	2020-06-10 00:45:18+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-06-10 00:40:38+02	2020-06-10 00:45:21+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-10 00:39:14+02	2020-06-10 00:45:27+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-10 00:45:23+02	2020-06-10 00:46:16+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-06-10 00:45:28+02	2020-06-10 00:46:23+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-06-10 00:46:16+02	2020-06-10 00:47:08+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-10 00:45:20+02	2020-06-10 00:47:44+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-10 00:47:10+02	2020-06-10 00:47:54+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-10 00:46:25+02	2020-06-10 00:48:18+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-10 00:27:27+02	2020-06-10 00:48:30+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-10 00:47:46+02	2020-06-10 00:48:57+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-10 00:42:32+02	2020-06-10 00:49:14+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-10 00:48:20+02	2020-06-10 00:49:31+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-10 00:48:32+02	2020-06-10 00:50:04+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-10 00:47:56+02	2020-06-10 00:51:32+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-10 00:50:06+02	2020-06-10 00:52:05+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-10 00:49:32+02	2020-06-10 00:52:32+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-10 00:51:33+02	2020-06-10 00:53:19+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-10 00:52:34+02	2020-06-10 00:53:45+02	print	libspectre	\N	libspectre-0.2.8
2020-06-10 00:53:21+02	2020-06-10 00:57:13+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-10 00:49:16+02	2020-06-10 00:58:17+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-06-10 00:52:07+02	2020-06-10 00:59:22+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-10 00:59:23+02	2020-06-10 01:01:04+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-06-10 00:58:18+02	2020-06-10 01:01:33+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-06-10 01:01:36+02	2020-06-10 01:03:20+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-10 00:53:47+02	2020-06-10 01:03:35+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-10 01:01:06+02	2020-06-10 01:04:00+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-10 01:04:02+02	2020-06-10 01:05:05+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-10 01:03:37+02	2020-06-10 01:05:11+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-10 01:03:21+02	2020-06-10 01:05:42+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-06-10 00:57:15+02	2020-06-10 01:05:44+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-10 01:05:07+02	2020-06-10 01:06:23+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-10 01:05:44+02	2020-06-10 01:06:25+02	audio	faudio	\N	FAudio-20.06
2020-06-10 01:05:46+02	2020-06-10 01:07:27+02	audio	mate-media	\N	mate-media-1.22.2
2020-06-10 01:05:13+02	2020-06-10 01:07:34+02	archivers	engrampa	\N	engrampa-1.22.2
2020-06-10 01:06:24+02	2020-06-10 01:07:50+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-06-10 01:06:27+02	2020-06-10 01:08:11+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-10 01:07:52+02	2020-06-10 01:08:12+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-06-10 01:07:35+02	2020-06-10 01:08:43+02	deskutils	mozo	\N	mozo-1.22.2
2020-06-10 00:26:45+02	2020-06-10 01:09:08+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-10 01:08:13+02	2020-06-10 01:09:15+02	devel	ode	\N	ode-0.13_4,1
2020-06-10 01:08:12+02	2020-06-10 01:09:21+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.70.0
2020-06-10 01:08:44+02	2020-06-10 01:10:12+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-10 01:07:28+02	2020-06-10 01:10:45+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-06-10 01:09:10+02	2020-06-10 01:11:03+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-10 01:10:46+02	2020-06-10 01:11:09+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-06-10 00:48:58+02	2020-06-10 01:11:33+02	math	R	\N	R-4.0.1
2020-06-10 01:11:34+02	2020-06-10 01:12:45+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-10 01:11:09+02	2020-06-10 01:13:40+02	graphics	eom	\N	eom-1.22.2
2020-06-10 01:10:13+02	2020-06-10 01:13:51+02	editors	pluma	\N	pluma-1.22.2
2020-06-10 01:13:42+02	2020-06-10 01:14:09+02	graphics	gle	\N	gle-3.1.0_7
2020-06-10 01:13:52+02	2020-06-10 01:14:13+02	graphics	gltt	\N	gltt-2.5.2_14
2020-06-10 01:11:03+02	2020-06-10 01:14:27+02	graphics	atril	\N	atril-1.22.3_6
2020-06-10 01:14:13+02	2020-06-10 01:14:34+02	graphics	libplacebo	\N	libplacebo-2.43.1
2020-06-10 01:12:46+02	2020-06-10 01:14:47+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-10 01:14:10+02	2020-06-10 01:15:21+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-10 01:14:35+02	2020-06-10 01:15:27+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-06-10 01:14:27+02	2020-06-10 01:17:43+02	graphics	py-gimp	\N	py27-gimp-2.10.18_7
2020-06-10 01:15:28+02	2020-06-10 01:18:14+02	math	mate-calc	\N	mate-calc-1.22.2
2020-06-10 01:17:44+02	2020-06-10 01:19:17+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-10 01:18:15+02	2020-06-10 01:20:42+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-10 01:20:43+02	2020-06-10 01:22:27+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-10 01:22:29+02	2020-06-10 01:23:43+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-10 01:23:43+02	2020-06-10 01:25:01+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-10 01:25:02+02	2020-06-10 01:26:08+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-10 01:26:09+02	2020-06-10 01:27:12+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-10 01:27:13+02	2020-06-10 01:28:10+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-10 01:19:18+02	2020-06-10 01:28:13+02	multimedia	vlc	\N	vlc-3.0.10_6,4
2020-06-10 01:14:48+02	2020-06-10 01:28:23+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-10 01:28:10+02	2020-06-10 01:29:04+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-10 01:28:13+02	2020-06-10 01:29:24+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-10 01:28:24+02	2020-06-10 01:29:57+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-10 01:29:05+02	2020-06-10 01:30:06+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-10 01:29:25+02	2020-06-10 01:30:55+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-10 01:29:58+02	2020-06-10 01:31:25+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-10 01:09:22+02	2020-06-10 01:34:09+02	editors	libreoffice	\N	libreoffice-6.3.6_1
2020-06-10 01:34:11+02	2020-06-10 01:36:54+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-10 01:30:07+02	2020-06-10 01:37:15+02	print	lilypond	\N	lilypond-2.20.0
2020-06-10 01:09:16+02	2020-06-10 01:38:06+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-10 01:31:26+02	2020-06-10 01:39:08+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-10 01:30:56+02	2020-06-10 01:39:18+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-10 01:38:07+02	2020-06-10 01:40:00+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-06-10 01:39:20+02	2020-06-10 01:40:47+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-10 01:40:02+02	2020-06-10 01:40:56+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-10 01:39:09+02	2020-06-10 01:41:14+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-06-10 01:40:48+02	2020-06-10 01:42:11+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-10 01:41:15+02	2020-06-10 01:42:46+02	x11	mate-base	\N	mate-base-1.22.2
2020-06-10 01:42:12+02	2020-06-10 01:44:05+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-06-10 01:44:06+02	2020-06-10 01:45:13+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-06-10 01:42:46+02	2020-06-10 01:45:26+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-06-10 01:15:22+02	2020-06-10 01:45:43+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-10 01:45:27+02	2020-06-10 01:46:09+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-10 01:45:14+02	2020-06-10 01:46:23+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-10 01:36:55+02	2020-06-10 01:46:29+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-10 01:40:56+02	2020-06-10 01:46:42+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-06-10 01:45:44+02	2020-06-10 01:47:10+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-10 01:46:43+02	2020-06-10 01:48:27+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-10 01:48:28+02	2020-06-10 01:50:52+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-10 01:46:30+02	2020-06-10 01:52:54+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-10 01:46:23+02	2020-06-10 01:53:10+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-10 01:46:10+02	2020-06-10 01:56:11+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-10 01:53:13+02	2020-06-10 01:57:40+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-06-10 01:56:12+02	2020-06-10 02:00:58+02	editors	vim	\N	vim-8.2.0869
2020-06-10 01:52:56+02	2020-06-10 02:07:32+02	editors	codelite	\N	codelite-12.0_4
2020-06-10 01:57:42+02	2020-06-10 02:21:05+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-10 02:07:33+02	2020-06-10 02:29:59+02	emulators	wine	\N	wine-5.0.1,1
2020-06-10 02:30:03+02	2020-06-10 02:30:34+02	games	KnightCap	\N	knightcap-3.6_8
2020-06-10 02:30:36+02	2020-06-10 02:31:37+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-06-10 02:31:38+02	2020-06-10 02:36:30+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-10 02:36:32+02	2020-06-10 02:37:27+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-06-10 01:47:10+02	2020-06-10 02:38:06+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-10 02:38:07+02	2020-06-10 02:38:40+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-06-10 02:37:29+02	2020-06-10 02:39:14+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-10 02:38:41+02	2020-06-10 02:39:48+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-10 02:21:09+02	2020-06-10 02:40:06+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-10 02:39:16+02	2020-06-10 02:40:15+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-10 02:39:49+02	2020-06-10 02:40:40+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-10 02:40:07+02	2020-06-10 02:41:12+02	games	xblast	\N	xblast-2.10.4_17
2020-06-10 01:37:16+02	2020-06-10 02:41:28+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-10 02:40:16+02	2020-06-10 02:41:34+02	games	xboard	\N	xboard-4.9.1_1
2020-06-10 02:40:41+02	2020-06-10 02:42:02+02	games	xcowsay	\N	xcowsay-1.5
2020-06-10 02:41:31+02	2020-06-10 02:43:17+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-10 02:43:18+02	2020-06-10 02:45:28+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-10 01:50:54+02	2020-06-10 02:45:28+02	devel	electron7	\N	electron7-7.3.1
2020-06-10 02:41:13+02	2020-06-10 02:46:30+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-10 02:45:32+02	2020-06-10 02:47:48+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-10 02:41:36+02	2020-06-10 02:48:18+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-10 02:47:51+02	2020-06-10 02:48:38+02	multimedia	libva-glx	\N	libva-glx-2.7.1
2020-06-10 02:48:20+02	2020-06-10 02:49:26+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-06-10 02:48:41+02	2020-06-10 02:53:54+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-10 02:49:27+02	2020-06-10 02:54:47+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-10 02:54:49+02	2020-06-10 02:58:08+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-10 02:42:03+02	2020-06-10 02:58:23+02	graphics	krita	\N	krita-4.2.9_4
2020-06-10 02:53:56+02	2020-06-10 02:58:27+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-10 02:58:29+02	2020-06-10 02:59:55+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-10 02:58:09+02	2020-06-10 03:00:20+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-10 02:59:57+02	2020-06-10 03:01:22+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-10 03:00:21+02	2020-06-10 03:02:11+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-10 03:01:24+02	2020-06-10 03:02:57+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-10 03:02:12+02	2020-06-10 03:03:18+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-10 03:02:57+02	2020-06-10 03:04:03+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-10 03:03:18+02	2020-06-10 03:04:24+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-10 02:58:24+02	2020-06-10 03:04:38+02	net	wireshark	\N	wireshark-3.2.4
2020-06-10 03:04:03+02	2020-06-10 03:05:05+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-10 03:04:39+02	2020-06-10 03:05:46+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-10 03:04:25+02	2020-06-10 03:05:59+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-10 03:05:06+02	2020-06-10 03:06:09+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-10 03:05:47+02	2020-06-10 03:06:54+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-10 03:06:10+02	2020-06-10 03:09:05+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-10 03:06:00+02	2020-06-10 03:09:16+02	print	auctex	full	auctex-emacs26-12.2
2020-06-10 03:09:06+02	2020-06-10 03:10:07+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-06-10 03:09:17+02	2020-06-10 03:10:30+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-10 03:10:07+02	2020-06-10 03:10:50+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-10 03:10:30+02	2020-06-10 03:11:18+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-10 02:00:59+02	2020-06-10 03:12:39+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-06-10 03:06:55+02	2020-06-10 03:12:43+02	print	texlive-full	\N	texlive-full-20150521
2020-06-10 03:11:19+02	2020-06-10 03:12:50+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-10 03:12:51+02	2020-06-10 03:13:21+02	x11	xlockmore	\N	xlockmore-5.63
2020-06-10 03:12:40+02	2020-06-10 03:13:41+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-10 03:12:44+02	2020-06-10 03:14:44+02	x11	mate	\N	mate-1.22.2
2020-06-10 03:13:22+02	2020-06-10 03:15:47+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-10 03:14:45+02	2020-06-10 03:15:53+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-10 03:15:48+02	2020-06-10 03:16:01+02	x11-fm	xdiskusage	\N	xdiskusage-1.51_1
2020-06-10 03:15:54+02	2020-06-10 03:16:24+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-10 03:18:52+02	2020-06-10 03:19:04+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-10 03:13:42+02	2020-06-10 03:20:27+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-10 02:45:32+02	2020-06-10 03:34:36+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-10 02:46:32+02	2020-06-10 03:38:50+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-10 03:10:51+02	2020-06-10 03:52:43+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-10 03:16:25+02	2020-06-10 03:56:45+02	editors	vscode	\N	vscode-1.45.1_2
2020-06-10 04:13:21+02	2020-06-10 04:27:58+02	local	E590T-localbase	\N	E590T-localbase-2020060702
2020-06-10 08:06:26+02	2020-06-10 08:16:28+02	www	node10	\N	node10-10.20.1_2
2020-06-10 08:16:28+02	2020-06-10 08:17:01+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-06-10 08:06:26+02	2020-06-10 08:17:11+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-10 08:17:02+02	2020-06-10 08:17:15+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-06-10 08:17:12+02	2020-06-10 08:17:22+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-10 08:06:26+02	2020-06-10 08:59:51+02	www	node	\N	node-14.0.0
2020-06-10 21:53:37+02	2020-06-10 21:53:46+02	sysutils	devcpu-data	\N	devcpu-data-1.31
2020-06-10 21:53:37+02	2020-06-10 21:54:00+02	benchmarks	iperf3	\N	iperf3-3.8.1
2020-06-10 21:53:37+02	2020-06-10 21:54:13+02	devel	gmake	\N	gmake-4.3_2
2020-06-10 21:53:37+02	2020-06-10 21:54:53+02	audio	mpg123	\N	mpg123-1.26.1
2020-06-10 21:54:14+02	2020-06-10 21:55:03+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-06-10 21:54:54+02	2020-06-10 21:55:47+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-06-10 21:55:51+02	2020-06-10 21:56:28+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-06-10 21:55:48+02	2020-06-10 21:56:34+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-10 21:53:37+02	2020-06-10 21:56:48+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-10 21:56:49+02	2020-06-10 21:56:58+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-10 21:56:35+02	2020-06-10 22:04:03+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-10 22:04:03+02	2020-06-10 22:04:59+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-10 22:04:04+02	2020-06-10 22:05:05+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-10 22:04:59+02	2020-06-10 22:06:07+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-10 22:05:06+02	2020-06-10 22:07:24+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-10 22:06:07+02	2020-06-10 22:08:05+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-10 21:54:14+02	2020-06-10 22:09:25+02	math	R	\N	R-4.0.1
2020-06-10 22:09:25+02	2020-06-10 22:10:02+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-10 22:10:03+02	2020-06-10 22:26:30+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-10 22:47:07+02	2020-06-10 23:02:07+02	local	E590T-localbase	\N	E590T-localbase-2020060702
2020-06-11 20:52:35+02	2020-06-11 20:54:37+02	graphics	opencv-core	\N	opencv-core-3.4.1_33
2020-06-11 20:54:38+02	2020-06-11 21:03:17+02	multimedia	ffmpeg	\N	ffmpeg-4.2.3,1
2020-06-11 21:03:17+02	2020-06-11 21:18:58+02	devel	electron7	\N	electron7-7.3.1
2020-06-11 21:19:00+02	2020-06-11 21:39:40+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-11 21:45:23+02	2020-06-11 21:46:05+02	audio	faudio	\N	FAudio-20.06
2020-06-11 21:45:24+02	2020-06-11 21:46:15+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-11 21:45:23+02	2020-06-11 21:47:01+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-11 21:46:17+02	2020-06-11 21:47:07+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-11 21:45:23+02	2020-06-11 21:47:11+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-11 21:46:06+02	2020-06-11 21:48:13+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-11 21:47:12+02	2020-06-11 21:49:24+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-11 21:45:23+02	2020-06-11 21:52:52+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-11 21:49:26+02	2020-06-11 21:54:45+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-11 21:52:54+02	2020-06-11 21:56:51+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-11 21:54:46+02	2020-06-11 21:58:34+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-11 21:47:08+02	2020-06-11 21:59:21+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-11 21:56:52+02	2020-06-11 22:00:24+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-11 21:59:23+02	2020-06-11 22:02:11+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-11 22:00:25+02	2020-06-11 22:03:25+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-11 22:02:14+02	2020-06-11 22:04:43+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-11 21:47:01+02	2020-06-11 22:07:02+02	multimedia	vlc	\N	vlc-3.0.10_6,4
2020-06-11 21:49:23+02	2020-06-11 22:07:43+02	emulators	wine	\N	wine-5.0.1,1
2020-06-11 22:03:28+02	2020-06-11 22:07:54+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-11 22:04:44+02	2020-06-11 22:08:13+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-11 22:08:14+02	2020-06-11 22:08:29+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-11 21:45:23+02	2020-06-11 22:09:48+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-11 22:07:02+02	2020-06-11 22:11:01+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-11 22:11:06+02	2020-06-11 22:13:44+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-11 22:07:45+02	2020-06-11 22:43:18+02	x11	alacritty	\N	alacritty-0.4.3_1
2020-06-11 22:09:49+02	2020-06-11 22:59:03+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-11 21:58:36+02	2020-06-11 23:05:20+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-11 22:07:55+02	2020-06-11 23:09:00+02	editors	vscode	\N	vscode-1.45.1_2
2020-06-11 23:25:27+02	2020-06-11 23:41:06+02	local	E590T-localbase	\N	E590T-localbase-2020060702
2020-06-12 07:45:09+02	2020-06-12 07:45:18+02	x11	sterm	\N	sterm-0.8.3
2020-06-12 07:45:09+02	2020-06-12 07:45:50+02	sysutils	smartmontools	\N	smartmontools-7.1_2
2020-06-12 07:45:09+02	2020-06-12 07:48:12+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-12 07:48:13+02	2020-06-12 07:48:23+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-12 07:45:09+02	2020-06-12 07:51:26+02	lang	python37	\N	python37-3.7.7
2020-06-12 07:51:27+02	2020-06-12 07:51:54+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-06-12 07:51:27+02	2020-06-12 07:52:00+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-06-12 07:51:56+02	2020-06-12 07:52:17+02	devel	py-six	py37	py37-six-1.14.0
2020-06-12 07:51:27+02	2020-06-12 07:52:20+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-06-12 07:52:02+02	2020-06-12 07:52:27+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-06-12 07:52:18+02	2020-06-12 07:52:40+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-06-12 07:52:22+02	2020-06-12 07:52:44+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-06-12 07:52:27+02	2020-06-12 07:52:46+02	dns	py-idna	py37	py37-idna-2.8
2020-06-12 07:52:47+02	2020-06-12 07:53:08+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-06-12 07:52:41+02	2020-06-12 07:53:14+02	devel	py-cffi	py37	py37-cffi-1.14.0
2020-06-12 07:51:27+02	2020-06-12 07:53:35+02	devel	ninja	\N	ninja-1.10.0,2
2020-06-12 07:52:44+02	2020-06-12 07:53:54+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-06-12 07:53:55+02	2020-06-12 07:54:29+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-06-12 07:53:17+02	2020-06-12 07:54:57+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-06-12 07:54:31+02	2020-06-12 07:54:58+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-06-12 07:55:01+02	2020-06-12 07:55:33+02	security	py-certifi	py37	py37-certifi-2020.4.5.1
2020-06-12 07:54:59+02	2020-06-12 07:55:33+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-06-12 07:55:36+02	2020-06-12 07:56:03+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-06-12 07:55:36+02	2020-06-12 07:56:04+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-06-12 07:56:06+02	2020-06-12 07:56:43+02	devel	py-packaging	py37	py37-packaging-20.4
2020-06-12 07:56:06+02	2020-06-12 07:56:45+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-06-12 07:53:09+02	2020-06-12 07:57:07+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-06-12 07:56:46+02	2020-06-12 07:57:15+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-06-12 07:56:46+02	2020-06-12 07:57:15+02	www	py-requests	py37	py37-requests-2.22.0
2020-06-12 07:57:08+02	2020-06-12 07:57:34+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-06-12 07:57:20+02	2020-06-12 07:57:43+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-06-12 07:57:20+02	2020-06-12 07:58:00+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-06-12 07:57:36+02	2020-06-12 07:58:03+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-06-12 07:57:45+02	2020-06-12 07:58:10+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-06-12 07:53:37+02	2020-06-12 07:58:19+02	lang	cython	py37	py37-cython-0.29.15
2020-06-12 07:58:01+02	2020-06-12 07:58:21+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-06-12 07:58:04+02	2020-06-12 07:58:24+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-06-12 07:58:11+02	2020-06-12 07:58:34+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-06-12 07:58:27+02	2020-06-12 07:59:03+02	devel	py-future	py37	py37-future-0.18.2
2020-06-12 07:58:25+02	2020-06-12 07:59:06+02	devel	meson	\N	meson-0.54.2
2020-06-12 07:58:35+02	2020-06-12 07:59:07+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-06-12 07:58:21+02	2020-06-12 07:59:16+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-06-12 07:59:06+02	2020-06-12 07:59:34+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-06-12 07:59:08+02	2020-06-12 07:59:40+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-06-12 07:59:20+02	2020-06-12 07:59:48+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-06-12 07:59:36+02	2020-06-12 08:00:08+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-06-12 07:59:42+02	2020-06-12 08:00:12+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-06-12 08:00:13+02	2020-06-12 08:00:44+02	textproc	py-toml	py37	py37-toml-0.10.0
2020-06-12 08:00:09+02	2020-06-12 08:00:49+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.7_7
2020-06-12 07:59:54+02	2020-06-12 08:00:51+02	textproc	py-sphinx	py37	py37-sphinx-3.0.4,1
2020-06-12 07:59:08+02	2020-06-12 08:00:51+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-06-12 08:00:45+02	2020-06-12 08:01:06+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-06-12 08:00:50+02	2020-06-12 08:01:13+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-06-12 08:00:52+02	2020-06-12 08:01:15+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-06-12 08:00:53+02	2020-06-12 08:01:17+02	textproc	itstool	\N	itstool-2.0.6
2020-06-12 07:51:27+02	2020-06-12 08:02:22+02	devel	glib20	\N	glib-2.56.3_7,1
2020-06-12 08:01:13+02	2020-06-12 08:02:23+02	www	py-django22	py37	py37-django22-2.2.13
2020-06-12 08:01:20+02	2020-06-12 08:02:51+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-06-12 08:02:29+02	2020-06-12 08:03:50+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-06-12 08:02:54+02	2020-06-12 08:04:28+02	misc	shared-mime-info	\N	shared-mime-info-1.10_2
2020-06-12 08:03:51+02	2020-06-12 08:05:59+02	textproc	libcroco	\N	libcroco-0.6.13
2020-06-12 08:02:28+02	2020-06-12 08:06:09+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-12 08:06:02+02	2020-06-12 08:07:18+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-06-12 08:07:22+02	2020-06-12 08:09:33+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-12 08:06:12+02	2020-06-12 08:11:59+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1,1
2020-06-12 08:01:08+02	2020-06-12 08:12:35+02	devel	gdb	py37	gdb-9.2_1
2020-06-12 08:09:36+02	2020-06-12 08:16:54+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-10-05 00:55:17+02	2020-10-05 00:58:39+02	x11	xorg	\N	xorg-7.7_3
2020-06-12 08:16:57+02	2020-06-12 08:19:13+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-06-12 08:12:05+02	2020-06-12 08:21:46+02	print	harfbuzz	\N	harfbuzz-2.6.7
2020-06-12 08:04:29+02	2020-06-12 08:22:30+02	devel	qt5-core	\N	qt5-core-5.14.2_3
2020-06-12 08:12:37+02	2020-06-12 08:22:43+02	net	avahi-app	\N	avahi-app-0.7_3
2020-06-12 08:19:15+02	2020-06-12 08:22:51+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-06-12 08:22:35+02	2020-06-12 08:23:41+02	accessibility	atk	\N	atk-2.36.0
2020-06-12 08:21:48+02	2020-06-12 08:27:47+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-12 08:23:44+02	2020-06-12 08:30:42+02	devel	libgsf	\N	libgsf-1.14.46
2020-06-12 08:22:53+02	2020-06-12 08:31:49+02	sysutils	polkit	\N	polkit-0.116
2020-06-12 08:30:44+02	2020-06-12 08:32:25+02	devel	libgudev	\N	libgudev-230_1
2020-06-12 08:32:28+02	2020-06-12 08:33:13+02	x11	libwacom	\N	libwacom-1.3
2020-06-12 08:22:46+02	2020-06-12 08:33:20+02	print	cups	\N	cups-2.2.13
2020-06-12 08:33:15+02	2020-06-12 08:34:35+02	x11	libinput	\N	libinput-1.15.5
2020-06-12 08:28:09+02	2020-06-12 08:34:45+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-12 08:34:47+02	2020-06-12 08:36:21+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-06-12 08:34:37+02	2020-06-12 08:43:33+02	net	qt5-network	\N	qt5-network-5.14.2
2020-06-12 08:31:51+02	2020-06-12 08:43:49+02	graphics	colord	\N	colord-1.3.5
2020-06-12 08:33:22+02	2020-06-12 08:44:26+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-06-12 08:43:53+02	2020-06-12 08:46:59+02	databases	qt5-sql	\N	qt5-sql-5.14.2
2020-06-12 08:43:36+02	2020-06-12 08:48:15+02	devel	qt5-dbus	\N	qt5-dbus-5.14.2
2020-06-12 08:48:18+02	2020-06-12 08:56:22+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-12 08:36:25+02	2020-06-12 08:59:24+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-12 08:59:31+02	2020-06-12 09:04:45+02	devel	qt5-testlib	\N	qt5-testlib-5.14.2
2020-06-12 08:56:29+02	2020-06-12 09:04:55+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-12 09:04:52+02	2020-06-12 09:20:22+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.14.2
2020-06-12 09:05:07+02	2020-06-12 09:20:42+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.7_2
2020-06-12 08:44:31+02	2020-06-12 09:21:33+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-12 07:45:09+02	2020-06-12 09:21:48+02	www	node12	\N	node12-12.17.0
2020-06-12 09:21:49+02	2020-06-12 09:24:35+02	textproc	qt5-xml	\N	qt5-xml-5.14.2
2020-06-12 09:20:23+02	2020-06-12 09:26:13+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-12 09:24:43+02	2020-06-12 09:26:59+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.14.2
2020-06-12 09:26:15+02	2020-06-12 09:27:54+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-06-12 09:20:44+02	2020-06-12 09:28:16+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-06-12 09:27:03+02	2020-06-12 09:28:54+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-12 09:27:59+02	2020-06-12 09:28:58+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-06-12 09:21:36+02	2020-06-12 09:29:06+02	lang	vala	\N	vala-0.40.15,1
2020-06-12 08:47:01+02	2020-06-12 09:30:46+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-12 09:28:56+02	2020-06-12 09:32:39+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.14.2
2020-06-12 09:28:58+02	2020-06-12 09:32:45+02	devel	libsoup	\N	libsoup-2.62.3
2020-06-12 09:29:07+02	2020-06-12 09:33:12+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-12 09:30:58+02	2020-06-12 09:33:28+02	devel	dconf	\N	dconf-0.28.0
2020-06-12 09:33:30+02	2020-06-12 09:35:36+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.70.0_2
2020-06-12 09:32:41+02	2020-06-12 09:35:36+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-12 09:28:17+02	2020-06-12 09:35:46+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-06-12 09:35:48+02	2020-06-12 09:36:40+02	devel	json-glib	\N	json-glib-1.4.4
2020-06-12 09:35:37+02	2020-06-12 09:37:09+02	devel	qt5-concurrent	\N	qt5-concurrent-5.14.2
2020-06-12 09:36:43+02	2020-06-12 09:37:25+02	devel	libnotify	\N	libnotify-0.7.8
2020-06-12 09:37:10+02	2020-06-12 09:37:56+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-12 09:35:37+02	2020-06-12 09:38:05+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-12 09:37:26+02	2020-06-12 09:38:36+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-06-12 09:38:06+02	2020-06-12 09:39:31+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-06-12 09:39:31+02	2020-06-12 09:42:30+02	security	libsecret	\N	libsecret-0.18.6_1
2020-06-12 09:38:37+02	2020-06-12 09:42:52+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-06-12 09:42:31+02	2020-06-12 09:45:26+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-06-12 09:37:57+02	2020-06-12 09:46:05+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-06-12 09:42:53+02	2020-06-12 09:46:06+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-12 09:45:27+02	2020-06-12 09:46:25+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-06-12 09:46:28+02	2020-06-12 09:47:03+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-12 09:46:07+02	2020-06-12 09:47:05+02	print	libraqm	\N	libraqm-0.6.0
2020-06-12 09:46:08+02	2020-06-12 09:47:26+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-12 09:47:05+02	2020-06-12 09:48:10+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_1
2020-06-12 09:47:27+02	2020-06-12 09:48:24+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-12 09:48:25+02	2020-06-12 09:49:45+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_1
2020-06-12 09:48:11+02	2020-06-12 09:50:25+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-12 09:49:46+02	2020-06-12 09:51:25+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-12 09:51:27+02	2020-06-12 09:52:46+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.70.0
2020-06-12 09:50:26+02	2020-06-12 09:54:17+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-06-12 09:32:46+02	2020-06-12 09:55:14+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-12 09:52:48+02	2020-06-12 09:55:44+02	graphics	graphene	\N	graphene-1.10.0
2020-06-12 09:47:06+02	2020-06-12 09:58:10+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-12 09:55:47+02	2020-06-12 09:58:31+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-12 09:54:18+02	2020-06-12 09:59:56+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-06-12 09:33:14+02	2020-06-12 10:01:28+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-12 09:59:58+02	2020-06-12 10:03:36+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-12 09:58:13+02	2020-06-12 10:04:14+02	devel	qt5-script	\N	qt5-script-5.14.2
2020-06-12 10:04:14+02	2020-06-12 10:05:06+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-12 09:58:33+02	2020-06-12 10:05:34+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-12 10:03:38+02	2020-06-12 10:06:20+02	net	geoclue	\N	geoclue-2.5.5
2020-06-12 10:05:36+02	2020-06-12 10:06:53+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-12 10:06:56+02	2020-06-12 10:08:56+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-12 10:06:23+02	2020-06-12 10:09:02+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-12 10:01:30+02	2020-06-12 10:09:13+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-06-12 10:05:08+02	2020-06-12 10:09:16+02	x11	mate-desktop	\N	mate-desktop-1.22.2
2020-06-12 10:08:58+02	2020-06-12 10:11:08+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-12 10:09:03+02	2020-06-12 10:11:43+02	sysutils	policykit	\N	policykit-0.9_10
2020-06-12 10:11:44+02	2020-06-12 10:12:47+02	archivers	kf5-karchive	\N	kf5-karchive-5.70.0
2020-06-12 10:11:09+02	2020-06-12 10:13:30+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-06-12 10:09:19+02	2020-06-12 10:13:49+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-12 10:12:49+02	2020-06-12 10:15:46+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-12 10:13:34+02	2020-06-12 10:16:06+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-12 10:15:49+02	2020-06-12 10:19:07+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-12 10:16:09+02	2020-06-12 10:20:15+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-12 10:19:11+02	2020-06-12 10:21:37+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-12 10:13:52+02	2020-06-12 10:21:49+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-12 10:21:39+02	2020-06-12 10:24:08+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-12 10:20:20+02	2020-06-12 10:24:49+02	sysutils	hal	\N	hal-0.5.14_34
2020-06-12 10:24:14+02	2020-06-12 10:25:17+02	comms	qt5-serialport	\N	qt5-serialport-5.14.2
2020-06-12 10:24:51+02	2020-06-12 10:26:20+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-12 10:25:19+02	2020-06-12 10:26:22+02	textproc	asciidoc	\N	asciidoc-9.0.0rc2
2020-06-12 10:26:22+02	2020-06-12 10:29:57+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-12 10:21:53+02	2020-06-12 10:34:28+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-12 10:34:38+02	2020-06-12 10:39:04+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-12 09:55:15+02	2020-06-12 10:40:58+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-12 10:39:07+02	2020-06-12 10:43:53+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-06-12 10:29:59+02	2020-06-12 10:47:15+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-06-12 10:09:15+02	2020-06-12 10:47:31+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-12 10:47:17+02	2020-06-12 10:48:34+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-12 10:26:27+02	2020-06-12 10:48:57+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-06-12 10:41:06+02	2020-06-12 10:48:59+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-12 10:49:02+02	2020-06-12 10:49:54+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.70.0
2020-06-12 10:48:35+02	2020-06-12 10:50:07+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-12 10:47:33+02	2020-06-12 10:50:16+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-12 10:49:03+02	2020-06-12 10:50:18+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-12 10:50:12+02	2020-06-12 10:52:04+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-12 10:49:55+02	2020-06-12 10:52:17+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-12 10:50:17+02	2020-06-12 10:53:33+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-12 10:52:06+02	2020-06-12 10:55:37+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-06-12 10:52:19+02	2020-06-12 10:56:14+02	net	remmina	\N	remmina-1.3.9
2020-06-12 10:50:20+02	2020-06-12 10:57:23+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-06-12 10:55:39+02	2020-06-12 10:57:28+02	x11	mate-menus	\N	mate-menus-1.22.1_1
2020-06-12 10:56:15+02	2020-06-12 10:57:59+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-12 10:44:07+02	2020-06-12 10:58:07+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-12 10:57:24+02	2020-06-12 10:59:02+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-12 10:57:30+02	2020-06-12 11:00:41+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-06-12 10:53:35+02	2020-06-12 11:00:52+02	security	gcr	\N	gcr-3.28.0
2020-06-12 10:59:03+02	2020-06-12 11:02:08+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-06-12 11:00:55+02	2020-06-12 11:04:02+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-12 11:02:09+02	2020-06-12 11:04:19+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-12 10:58:13+02	2020-06-12 11:07:13+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-12 11:04:03+02	2020-06-12 11:08:35+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-12 11:07:14+02	2020-06-12 11:10:07+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-06-12 11:04:23+02	2020-06-12 11:11:25+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-12 11:11:28+02	2020-06-12 11:12:01+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-06-12 11:08:38+02	2020-06-12 11:12:14+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-06-12 11:12:03+02	2020-06-12 11:12:29+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-06-12 11:12:16+02	2020-06-12 11:12:54+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-06-12 11:00:44+02	2020-06-12 11:12:58+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.70.0
2020-06-12 11:10:09+02	2020-06-12 11:14:33+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-06-12 11:12:33+02	2020-06-12 11:15:27+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-12 11:12:57+02	2020-06-12 11:18:29+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-06-12 11:14:35+02	2020-06-12 11:18:32+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-12 11:18:31+02	2020-06-12 11:21:38+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-06-12 11:13:05+02	2020-06-12 11:23:16+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-06-12 11:15:31+02	2020-06-12 11:23:59+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-06-12 11:23:19+02	2020-06-12 11:25:37+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-12 11:18:37+02	2020-06-12 11:26:52+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-06-12 11:24:03+02	2020-06-12 11:27:30+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-12 11:21:41+02	2020-06-12 11:27:32+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-06-12 11:27:35+02	2020-06-12 11:30:40+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-12 11:27:33+02	2020-06-12 11:31:01+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-06-12 11:26:56+02	2020-06-12 11:33:19+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-12 11:33:22+02	2020-06-12 11:34:41+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-12 11:30:42+02	2020-06-12 11:36:24+02	devel	libgtop	\N	libgtop-2.38.0_2
2020-06-12 11:36:25+02	2020-06-12 11:38:26+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-12 11:31:05+02	2020-06-12 11:39:43+02	devel	py-lxml	py37	py37-lxml-4.5.1
2020-06-12 11:34:44+02	2020-06-12 11:40:59+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-12 11:39:49+02	2020-06-12 11:42:49+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-12 11:41:02+02	2020-06-12 11:44:02+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-12 11:25:40+02	2020-06-12 11:45:14+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.22.2
2020-06-12 11:42:52+02	2020-06-12 11:46:19+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-12 11:45:16+02	2020-06-12 11:46:25+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.7_6
2020-06-12 11:44:04+02	2020-06-12 11:48:11+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-12 11:46:21+02	2020-06-12 11:48:59+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-12 11:46:32+02	2020-06-12 11:51:17+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-06-12 11:49:04+02	2020-06-12 11:51:51+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-12 11:48:13+02	2020-06-12 11:54:36+02	devel	talloc	\N	talloc-2.3.0
2020-06-12 11:54:38+02	2020-06-12 11:55:49+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7
2020-06-12 11:51:19+02	2020-06-12 11:55:58+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-12 11:51:53+02	2020-06-12 11:58:57+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-06-12 11:38:28+02	2020-06-12 11:59:49+02	net	libmateweather	\N	libmateweather-1.22.1
2020-06-12 11:55:52+02	2020-06-12 12:00:50+02	sysutils	upower	\N	upower-0.99.11
2020-06-12 11:59:54+02	2020-06-12 12:02:17+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-06-12 11:59:05+02	2020-06-12 12:02:43+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-12 12:02:20+02	2020-06-12 12:05:26+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-12 12:02:48+02	2020-06-12 12:07:34+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-12 12:05:29+02	2020-06-12 12:12:09+02	audio	libmatemixer	\N	libmatemixer-1.22.0
2020-06-12 12:12:14+02	2020-06-12 12:14:28+02	devel	gamin	\N	gamin-0.1.10_10
2020-06-12 11:56:00+02	2020-06-12 12:15:06+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-12 12:00:53+02	2020-06-12 12:15:38+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-12 12:07:39+02	2020-06-12 12:15:54+02	databases	tdb	\N	tdb-1.4.2,1
2020-06-12 12:15:42+02	2020-06-12 12:16:07+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-06-12 12:15:56+02	2020-06-12 12:16:26+02	devel	py-more-itertools	py37	py37-more-itertools-8.3.0
2020-06-12 12:15:15+02	2020-06-12 12:17:24+02	devel	librest	\N	rest-0.7.93
2020-06-12 12:14:33+02	2020-06-12 12:17:59+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-12 12:16:09+02	2020-06-12 12:18:27+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-12 12:18:01+02	2020-06-12 12:20:12+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-12 12:17:25+02	2020-06-12 12:20:41+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-06-12 12:16:28+02	2020-06-12 12:22:12+02	devel	tevent	\N	tevent-0.10.1
2020-06-12 12:20:14+02	2020-06-12 12:23:43+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-06-12 12:18:29+02	2020-06-12 12:23:46+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-06-12 12:20:43+02	2020-06-12 12:24:17+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-12 12:23:51+02	2020-06-12 12:24:26+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-06-12 12:22:17+02	2020-06-12 12:25:31+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-12 12:24:29+02	2020-06-12 12:26:17+02	x11	babl	\N	babl-0.1.74
2020-06-12 12:23:47+02	2020-06-12 12:32:29+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-06-12 12:25:34+02	2020-06-12 12:35:13+02	x11	libmatekbd	\N	libmatekbd-1.22.0
2020-06-12 12:32:31+02	2020-06-12 12:40:21+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-12 12:35:20+02	2020-06-12 12:42:58+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-12 12:26:19+02	2020-06-12 12:46:08+02	x11-fm	caja	\N	caja-1.22.3
2020-06-12 12:43:04+02	2020-06-12 12:47:53+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-12 12:46:14+02	2020-06-12 12:49:58+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-12 12:40:28+02	2020-06-12 12:50:29+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-12 12:50:01+02	2020-06-12 12:50:47+02	devel	py-click	py37	py37-click-7.0
2020-06-12 12:50:32+02	2020-06-12 12:51:25+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-06-12 12:50:48+02	2020-06-12 12:51:29+02	devel	py-olefile	py37	py37-olefile-0.46
2020-06-12 12:47:56+02	2020-06-12 12:52:04+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-12 12:51:29+02	2020-06-12 12:52:13+02	devel	py-ply	py37	py37-ply-3.11
2020-06-12 12:51:31+02	2020-06-12 12:52:23+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-06-12 12:52:09+02	2020-06-12 12:52:47+02	devel	py-zipp	py37	py37-zipp-0.6.0_3
2020-06-12 12:52:27+02	2020-06-12 12:53:31+02	devel	scons	py37	scons-py37-3.1.2
2020-06-12 12:53:33+02	2020-06-12 12:56:08+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-06-12 12:52:16+02	2020-06-12 12:57:57+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-12 12:52:49+02	2020-06-12 12:58:35+02	graphics	gegl	\N	gegl-0.4.22_6
2020-06-12 12:56:13+02	2020-06-12 13:00:18+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-06-12 12:58:42+02	2020-06-12 13:12:40+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-12 13:12:46+02	2020-06-12 13:17:47+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-12 13:17:49+02	2020-06-12 13:21:34+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-12 12:58:00+02	2020-06-12 13:23:06+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-12 13:23:08+02	2020-06-12 13:32:04+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-12 12:24:20+02	2020-06-12 15:00:13+02	www	node	\N	node-14.3.0
2020-06-12 15:00:14+02	2020-06-12 15:00:25+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-06-12 15:00:26+02	2020-06-12 15:00:37+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-06-12 15:00:38+02	2020-06-12 15:00:49+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-06-12 08:01:16+02	2020-06-12 15:01:14+02	devel	llvm80	\N	llvm80-8.0.1_3
2020-06-12 15:00:49+02	2020-06-12 15:01:44+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-12 15:01:45+02	2020-06-12 15:03:18+02	x11	zenity	\N	zenity-3.28.1
2020-06-12 15:01:15+02	2020-06-12 15:03:47+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_5
2020-06-12 15:03:48+02	2020-06-12 15:09:37+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-06-12 15:09:38+02	2020-06-12 15:11:51+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-12 15:11:52+02	2020-06-12 15:14:04+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-12 15:03:18+02	2020-06-12 15:14:20+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-12 15:14:05+02	2020-06-12 15:15:02+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-12 15:14:21+02	2020-06-12 15:15:16+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-12 15:15:03+02	2020-06-12 15:15:52+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-12 15:15:17+02	2020-06-12 15:16:33+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-12 15:15:52+02	2020-06-12 15:16:45+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-12 15:16:34+02	2020-06-12 15:17:23+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-12 15:17:24+02	2020-06-12 15:18:51+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-12 15:16:46+02	2020-06-12 15:18:51+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-06-12 15:18:52+02	2020-06-12 15:19:41+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-12 15:18:52+02	2020-06-12 15:19:52+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-12 15:19:42+02	2020-06-12 15:19:54+02	comms	py-serial	py37	py37-serial-3.4_1
2020-06-12 15:19:53+02	2020-06-12 15:20:04+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-06-12 15:19:55+02	2020-06-12 15:20:06+02	databases	py-redis	py37	py37-redis-3.3.11
2020-06-12 15:20:04+02	2020-06-12 15:22:33+02	devel	git	\N	git-2.27.0
2020-06-12 15:20:06+02	2020-06-12 15:23:41+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-12 15:22:34+02	2020-06-12 15:24:20+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-12 15:23:42+02	2020-06-12 15:24:56+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-12 15:24:57+02	2020-06-12 15:25:08+02	devel	py-Automat	py37	py37-Automat-0.8.0
2020-06-12 15:25:08+02	2020-06-12 15:25:20+02	devel	py-appdirs	py37	py37-appdirs-1.4.3
2020-06-12 10:58:00+02	2020-06-12 15:25:27+02	lang	rust	\N	rust-1.44.0
2020-06-12 15:25:20+02	2020-06-12 15:25:30+02	devel	py-backports	py37	py37-backports-1
2020-06-12 15:25:29+02	2020-06-12 15:25:39+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-06-12 15:25:31+02	2020-06-12 15:25:41+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-06-12 15:25:40+02	2020-06-12 15:25:50+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-06-12 15:25:51+02	2020-06-12 15:26:08+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-0.23
2020-06-12 15:25:42+02	2020-06-12 15:26:14+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-12 15:26:08+02	2020-06-12 15:26:22+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-06-12 15:26:14+02	2020-06-12 15:26:28+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-06-12 15:26:22+02	2020-06-12 15:26:41+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-06-12 15:26:29+02	2020-06-12 15:26:46+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-06-12 15:26:42+02	2020-06-12 15:26:57+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-06-12 15:26:46+02	2020-06-12 15:27:04+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-06-12 15:26:58+02	2020-06-12 15:27:13+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-06-12 15:27:05+02	2020-06-12 15:33:22+02	dns	bind-tools	\N	bind-tools-9.16.3_1
2020-06-12 15:33:23+02	2020-06-12 15:33:58+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-12 15:27:13+02	2020-06-12 15:34:39+02	editors	emacs	full	emacs-26.3_4,3
2020-06-12 15:34:43+02	2020-06-12 15:41:49+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-12 15:41:51+02	2020-06-12 15:43:28+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-06-12 15:43:31+02	2020-06-12 15:45:05+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0
2020-06-12 15:45:07+02	2020-06-12 15:46:11+02	net	gssdp	\N	gssdp-1.2.2
2020-06-12 15:46:12+02	2020-06-12 15:47:55+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-12 15:47:56+02	2020-06-12 15:50:21+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-12 15:33:58+02	2020-06-12 15:53:19+02	graphics	gimp-app	\N	gimp-app-2.10.18_7,1
2020-06-12 15:50:21+02	2020-06-12 15:54:54+02	print	fontforge	\N	fontforge-20190801_2
2020-06-12 15:53:20+02	2020-06-12 16:05:47+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-12 16:05:48+02	2020-06-12 16:06:08+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-06-12 16:06:08+02	2020-06-12 16:07:18+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-06-12 16:07:19+02	2020-06-12 16:11:30+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.22.1
2020-06-12 16:11:31+02	2020-06-12 16:11:47+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-06-12 16:11:48+02	2020-06-12 16:12:06+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-06-12 15:54:55+02	2020-06-12 16:24:47+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-06-12 16:24:50+02	2020-06-12 16:25:22+02	www	py-h2	py37	py37-h2-3.2.0
2020-06-12 16:25:27+02	2020-06-12 16:25:56+02	www	py-hyperlink	py37	py37-hyperlink-19.0.0
2020-06-12 16:26:03+02	2020-06-12 16:26:29+02	www	py-priority	py37	py37-priority-1.3.0
2020-06-12 16:26:30+02	2020-06-12 16:27:06+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-06-12 16:27:08+02	2020-06-12 17:05:01+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-12 17:05:10+02	2020-06-12 17:09:49+02	x11	mate-session-manager	\N	mate-session-manager-1.22.3
2020-06-12 17:09:52+02	2020-06-12 17:15:54+02	x11	xorg	\N	xorg-7.7_3
2020-06-12 17:15:56+02	2020-06-12 17:21:02+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-12 17:21:04+02	2020-06-12 17:24:43+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-12 17:24:46+02	2020-06-12 17:27:18+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-12 17:27:24+02	2020-06-12 17:31:52+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-12 17:31:54+02	2020-06-12 17:33:55+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-12 16:12:07+02	2020-06-12 17:49:11+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-12 17:33:58+02	2020-06-12 17:49:36+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-12 17:49:18+02	2020-06-12 17:52:28+02	x11-wm	marco	\N	marco-1.22.4
2020-06-12 17:49:37+02	2020-06-12 17:53:14+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-12 17:52:30+02	2020-06-12 17:54:12+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.22.1
2020-06-12 17:54:13+02	2020-06-12 17:54:27+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-06-12 17:54:28+02	2020-06-12 17:54:48+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-06-12 17:54:49+02	2020-06-12 17:55:03+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-06-12 17:55:09+02	2020-06-12 17:55:52+02	devel	py-gobject3	py27	py27-gobject3-3.28.3_1
2020-06-12 17:55:53+02	2020-06-12 17:56:14+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-06-12 17:56:15+02	2020-06-12 17:56:39+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-06-12 17:56:40+02	2020-06-12 17:56:57+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-06-12 17:53:14+02	2020-06-12 17:57:02+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-12 17:56:58+02	2020-06-12 17:58:16+02	devel	py-twisted	py37	py37-twisted-20.3.0
2020-06-12 17:58:17+02	2020-06-12 17:59:33+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-12 17:57:02+02	2020-06-12 18:01:53+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-12 17:59:35+02	2020-06-12 18:02:11+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-12 18:01:55+02	2020-06-12 18:02:29+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-12 18:02:30+02	2020-06-12 18:03:09+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-12 18:02:12+02	2020-06-12 18:03:48+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-12 18:03:09+02	2020-06-12 18:04:29+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-12 18:04:30+02	2020-06-12 18:05:45+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-12 18:03:49+02	2020-06-12 18:07:25+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-12 18:05:46+02	2020-06-12 18:07:33+02	graphics	vigra	\N	vigra-1.11.1_10
2020-06-12 18:07:26+02	2020-06-12 18:08:47+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-12 18:08:50+02	2020-06-12 18:17:01+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-06-12 18:17:03+02	2020-06-12 18:27:23+02	misc	mate-user-guide	\N	mate-user-guide-1.22.3
2020-06-12 18:27:24+02	2020-06-12 18:29:08+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-12 18:29:09+02	2020-06-12 18:30:54+02	net	freerdp	\N	freerdp-2.1.1
2020-06-12 18:30:55+02	2020-06-12 18:31:39+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-12 18:31:40+02	2020-06-12 18:32:03+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-06-12 18:32:04+02	2020-06-12 18:32:18+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-06-12 18:07:34+02	2020-06-12 18:32:53+02	math	R	\N	R-4.0.1
2020-06-12 18:32:19+02	2020-06-12 18:33:31+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-12 18:33:32+02	2020-06-12 18:34:47+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-12 18:32:56+02	2020-06-12 18:34:55+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-12 18:34:57+02	2020-06-12 18:35:25+02	print	libspectre	\N	libspectre-0.2.8
2020-06-12 18:35:26+02	2020-06-12 18:38:38+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-12 18:34:49+02	2020-06-12 18:40:18+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-12 18:38:39+02	2020-06-12 18:47:25+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-12 18:40:19+02	2020-06-12 18:48:41+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-12 18:47:26+02	2020-06-12 18:48:41+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-06-12 18:48:43+02	2020-06-12 18:49:10+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-06-12 18:49:13+02	2020-06-12 18:49:39+02	sysutils	hal-info	\N	hal-info-20091130
2020-06-12 18:48:43+02	2020-06-12 18:49:47+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-06-12 18:49:48+02	2020-06-12 18:51:12+02	sysutils	mate-polkit	\N	mate-polkit-1.22.0
2020-06-12 18:49:40+02	2020-06-12 18:53:57+02	sysutils	mate-control-center	\N	mate-control-center-1.22.2
2020-06-12 18:51:13+02	2020-06-12 18:54:48+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-12 18:53:57+02	2020-06-12 18:54:56+02	textproc	enchant	\N	enchant-1.6.0_9
2020-06-12 18:54:49+02	2020-06-12 18:55:09+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-06-12 18:54:57+02	2020-06-12 18:55:16+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-06-12 18:55:09+02	2020-06-12 18:55:24+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-06-12 18:55:16+02	2020-06-12 18:55:34+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-06-12 18:55:25+02	2020-06-12 18:55:46+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-06-12 18:55:35+02	2020-06-12 18:57:09+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-12 18:57:09+02	2020-06-12 18:58:12+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.22.0
2020-06-12 18:55:47+02	2020-06-12 18:58:27+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-06-12 18:58:28+02	2020-06-12 18:59:40+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-12 18:58:13+02	2020-06-12 18:59:49+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-12 18:59:41+02	2020-06-12 19:03:14+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-12 18:59:50+02	2020-06-12 19:03:26+02	archivers	engrampa	\N	engrampa-1.22.2
2020-06-12 19:03:15+02	2020-06-12 19:04:28+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-06-12 19:03:27+02	2020-06-12 19:05:12+02	audio	mate-media	\N	mate-media-1.22.2
2020-06-12 19:04:32+02	2020-06-12 19:05:57+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-06-12 19:05:15+02	2020-06-12 19:07:16+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-12 19:07:16+02	2020-06-12 19:09:28+02	deskutils	mozo	\N	mozo-1.22.2
2020-06-12 19:05:57+02	2020-06-12 19:10:56+02	deskutils	mate-utils	\N	mate-utils-1.22.2
2020-06-12 19:09:28+02	2020-06-12 19:10:56+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-05 00:51:50+02	2020-10-05 01:05:38+02	math	vtk8	\N	vtk8-8.2.0
2020-06-12 19:10:57+02	2020-06-12 19:11:11+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-06-12 19:11:11+02	2020-06-12 19:11:33+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-06-12 19:10:57+02	2020-06-12 19:11:38+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.70.0
2020-06-12 19:11:34+02	2020-06-12 19:11:49+02	devel	py-funcy	py37	py37-funcy-1.12
2020-06-12 19:11:49+02	2020-06-12 19:12:06+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-06-12 19:12:07+02	2020-06-12 19:12:23+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-06-12 19:12:23+02	2020-06-12 19:12:44+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-06-12 19:11:39+02	2020-06-12 19:12:53+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-12 19:12:44+02	2020-06-12 19:13:05+02	devel	py-rq	py37	py37-rq-1.3.0
2020-06-12 19:13:05+02	2020-06-12 19:13:20+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-06-12 19:13:21+02	2020-06-12 19:13:39+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-06-12 19:12:54+02	2020-06-12 19:13:42+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.87
2020-06-12 19:13:39+02	2020-06-12 19:13:56+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-06-12 19:13:56+02	2020-06-12 19:14:21+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-06-12 19:13:43+02	2020-06-12 19:21:39+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.2_1
2020-06-12 19:14:22+02	2020-06-12 19:52:35+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-12 19:21:40+02	2020-06-12 19:55:24+02	editors	libreoffice	\N	libreoffice-6.3.6_1
2020-06-12 15:24:20+02	2020-06-12 20:37:56+02	devel	llvm90	\N	llvm90-9.0.1_1
2020-06-12 21:02:50+02	2020-06-12 21:03:11+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-06-12 21:03:12+02	2020-06-12 21:03:21+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-06-12 21:03:21+02	2020-06-12 21:22:28+02	devel	electron7	\N	electron7-7.3.1
2020-06-12 21:27:50+02	2020-06-12 21:29:05+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-12 21:27:50+02	2020-06-12 21:30:13+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-12 21:27:50+02	2020-06-12 21:31:00+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-12 21:29:05+02	2020-06-12 21:31:01+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-12 21:27:50+02	2020-06-12 21:31:03+02	graphics	eom	\N	eom-1.22.2
2020-06-12 21:30:14+02	2020-06-12 21:32:58+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-12 21:33:00+02	2020-06-12 21:33:14+02	lang	python3	\N	python3-3_3
2020-06-12 21:27:50+02	2020-06-12 21:33:17+02	graphics	atril	\N	atril-1.22.3_6
2020-06-12 21:31:01+02	2020-06-12 21:34:13+02	graphics	py-gimp	\N	py27-gimp-2.10.18_7
2020-06-12 21:33:18+02	2020-06-12 21:35:33+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-12 21:33:15+02	2020-06-12 21:39:51+02	math	mate-calc	\N	mate-calc-1.22.2
2020-06-12 21:34:14+02	2020-06-12 21:40:02+02	math	stp	\N	stp-2.3.3_5,1
2020-06-12 21:39:52+02	2020-06-12 21:41:21+02	multimedia	libass	\N	libass-0.14.0
2020-06-12 21:40:03+02	2020-06-12 21:43:17+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-12 21:43:18+02	2020-06-12 21:45:40+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-12 21:27:50+02	2020-06-12 21:46:03+02	net	samba410	\N	samba410-4.10.15
2020-06-12 21:45:41+02	2020-06-12 21:46:37+02	net	py-pyzmq	py37	py37-pyzmq-19.0.1
2020-06-12 21:35:37+02	2020-06-12 21:47:27+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-06-12 21:46:38+02	2020-06-12 21:47:57+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-12 21:47:27+02	2020-06-12 21:49:10+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-12 21:46:04+02	2020-06-12 21:49:14+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-12 21:47:58+02	2020-06-12 21:49:26+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-12 21:49:10+02	2020-06-12 21:50:25+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-12 21:49:28+02	2020-06-12 21:50:33+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-12 21:31:02+02	2020-06-12 21:51:05+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-12 21:50:26+02	2020-06-12 21:51:42+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-12 21:50:34+02	2020-06-12 21:52:17+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-12 21:41:22+02	2020-06-12 21:52:42+02	multimedia	vlc	\N	vlc-3.0.10_6,4
2020-06-12 21:51:43+02	2020-06-12 21:53:21+02	net	vde2	\N	vde2-2.3.2_4
2020-06-12 21:51:08+02	2020-06-12 21:53:40+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-12 21:53:42+02	2020-06-12 21:54:04+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.7.1
2020-06-12 21:52:17+02	2020-06-12 21:54:06+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-12 21:49:14+02	2020-06-12 21:54:31+02	editors	pluma	\N	pluma-1.22.2
2020-06-12 21:52:45+02	2020-06-12 21:55:05+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-12 21:53:22+02	2020-06-12 21:55:29+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-06-12 21:54:05+02	2020-06-12 21:56:05+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-12 21:55:30+02	2020-06-12 21:58:25+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-12 21:54:06+02	2020-06-12 22:03:12+02	print	lilypond	\N	lilypond-2.20.0
2020-06-12 22:03:14+02	2020-06-12 22:04:32+02	print	transfig	\N	transfig-3.2.5e_4
2020-06-12 22:04:33+02	2020-06-12 22:04:55+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-06-12 22:04:59+02	2020-06-12 22:05:17+02	security	py-stem	py37	py37-stem-1.8.0
2020-06-12 21:55:06+02	2020-06-12 22:05:36+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-12 21:54:32+02	2020-06-12 22:07:17+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-12 22:07:19+02	2020-06-12 22:10:37+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-12 22:05:18+02	2020-06-12 22:10:37+02	sysutils	mate-power-manager	\N	mate-power-manager-1.22.2
2020-06-12 22:05:37+02	2020-06-12 22:10:37+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.22.2
2020-06-12 21:56:06+02	2020-06-12 22:10:47+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-12 22:10:39+02	2020-06-12 22:10:54+02	textproc	py-scour	py37	py37-scour-0.37
2020-06-12 22:10:38+02	2020-06-12 22:10:55+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-06-12 22:10:38+02	2020-06-12 22:10:55+02	www	py-dj22-django-js-asset	py37	py37-dj22-django-js-asset-1.2.2
2020-06-12 22:10:48+02	2020-06-12 22:11:03+02	www	py-dj22-django-taggit	py37	py37-dj22-django-taggit-1.3.0
2020-06-12 22:10:54+02	2020-06-12 22:11:16+02	www	py-dj22-djangorestframework	py37	py37-dj22-djangorestframework-3.11.0
2020-06-12 22:10:55+02	2020-06-12 22:11:27+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-06-12 22:11:17+02	2020-06-12 22:11:30+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-06-12 22:10:56+02	2020-06-12 22:11:46+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-12 22:11:29+02	2020-06-12 22:12:24+02	www	qt5-websockets	\N	qt5-websockets-5.14.2
2020-06-12 22:11:31+02	2020-06-12 22:12:27+02	www	youtube_dl	\N	youtube_dl-2020.06.06
2020-06-12 22:11:04+02	2020-06-12 22:12:42+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-12 21:31:04+02	2020-06-12 22:13:46+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-12 22:12:25+02	2020-06-12 22:14:17+02	x11	mate-base	\N	mate-base-1.22.2
2020-06-12 22:12:28+02	2020-06-12 22:15:01+02	x11	mate-screensaver	\N	mate-screensaver-1.22.2
2020-06-12 22:14:19+02	2020-06-12 22:15:58+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-12 22:12:42+02	2020-06-12 22:15:58+02	x11	mate-terminal	\N	mate-terminal-1.22.1
2020-06-12 22:13:47+02	2020-06-12 22:16:32+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-12 22:16:00+02	2020-06-12 22:25:48+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-12 22:16:33+02	2020-06-12 22:25:58+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-12 22:25:50+02	2020-06-12 22:26:19+02	databases	py-apsw	py37	py37-apsw-3.31.1
2020-06-12 22:15:03+02	2020-06-12 22:26:30+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-12 22:25:59+02	2020-06-12 22:26:37+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-06-12 22:11:50+02	2020-06-12 22:27:13+02	x11	mate-applets	\N	mate-applets-1.22.2
2020-06-12 22:26:19+02	2020-06-12 22:27:51+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-12 22:26:37+02	2020-06-12 22:28:21+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-12 22:15:59+02	2020-06-12 22:28:53+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-12 22:28:21+02	2020-06-12 22:28:59+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-06-12 22:28:57+02	2020-06-12 22:29:17+02	devel	py-dj22-django-rq	py37	py37-dj22-django-rq-2.3.2
2020-06-12 22:29:01+02	2020-06-12 22:29:32+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-06-12 22:29:18+02	2020-06-12 22:30:37+02	devel	tig	\N	tig-2.5.1
2020-06-12 22:29:34+02	2020-06-12 22:47:04+02	dns	bind916	\N	bind916-9.16.3_2
2020-06-12 21:58:26+02	2020-06-12 22:49:25+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-12 22:47:07+02	2020-06-12 22:57:51+02	editors	libreoffice-nb	\N	nb-libreoffice-6.3.6
2020-06-12 22:30:40+02	2020-06-12 23:00:04+02	editors	codelite	\N	codelite-12.0_4
2020-06-12 22:49:34+02	2020-06-12 23:01:23+02	editors	vim	\N	vim-8.2.0869
2020-06-12 23:01:25+02	2020-06-12 23:36:46+02	emulators	wine	\N	wine-5.0.1,1
2020-06-12 22:57:54+02	2020-06-12 23:38:41+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-12 23:38:45+02	2020-06-12 23:41:28+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-12 23:41:30+02	2020-06-12 23:43:39+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-12 23:43:40+02	2020-06-12 23:45:50+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-12 23:45:52+02	2020-06-12 23:49:07+02	games	xboard	\N	xboard-4.9.1_1
2020-06-12 23:49:08+02	2020-06-12 23:52:09+02	games	xcowsay	\N	xcowsay-1.5
2020-06-12 22:26:31+02	2020-06-12 23:54:02+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-12 23:54:04+02	2020-06-13 00:03:46+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-12 23:52:11+02	2020-06-13 00:07:49+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-13 00:03:48+02	2020-06-13 00:08:38+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-12 23:36:49+02	2020-06-13 00:13:18+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-13 00:13:20+02	2020-06-13 00:13:59+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-06-13 00:14:02+02	2020-06-13 00:17:57+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-12 23:00:06+02	2020-06-13 01:20:25+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-06-13 01:20:28+02	2020-06-13 01:23:23+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-13 00:07:51+02	2020-06-13 01:31:08+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-13 01:31:11+02	2020-06-13 03:03:04+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-13 01:23:25+02	2020-06-13 03:24:59+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-13 03:25:02+02	2020-06-13 03:25:26+02	lang	python	\N	python-3.7_3,2
2020-06-13 03:25:27+02	2020-06-13 03:37:22+02	misc	mc	\N	mc-nox11-4.8.24
2020-06-13 03:37:25+02	2020-06-13 03:43:25+02	multimedia	mpv	\N	mpv-0.32.0_2,1
2020-06-13 03:43:27+02	2020-06-13 03:48:22+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-13 03:48:24+02	2020-06-13 03:53:49+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-13 03:53:52+02	2020-06-13 04:00:11+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-13 04:00:13+02	2020-06-13 04:05:03+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-13 04:05:05+02	2020-06-13 04:05:40+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-06-13 04:05:43+02	2020-06-13 04:09:08+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-13 04:09:11+02	2020-06-13 04:20:34+02	net	wireshark	\N	wireshark-3.2.4
2020-06-13 04:20:36+02	2020-06-13 04:23:54+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-13 03:03:06+02	2020-06-13 04:42:02+02	lang	mono	\N	mono-5.10.1.57_3
2020-06-13 04:42:05+02	2020-06-13 04:44:16+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-13 04:44:18+02	2020-06-13 04:46:18+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-13 04:46:19+02	2020-06-13 04:48:16+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-13 04:48:17+02	2020-06-13 04:50:16+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-13 04:50:19+02	2020-06-13 04:52:31+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-13 04:52:33+02	2020-06-13 04:54:52+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-13 04:54:54+02	2020-06-13 04:56:52+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-13 04:23:56+02	2020-06-13 04:58:07+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-06-13 04:56:55+02	2020-06-13 04:59:57+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-13 04:58:09+02	2020-06-13 05:00:17+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-13 04:59:59+02	2020-06-13 05:01:56+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-13 05:00:19+02	2020-06-13 05:02:24+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-13 05:02:27+02	2020-06-13 05:04:22+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-13 05:04:25+02	2020-06-13 05:06:52+02	print	gv	\N	gv-3.7.4_5
2020-06-13 05:01:58+02	2020-06-13 05:07:58+02	print	auctex	full	auctex-emacs26-12.2
2020-06-13 05:08:01+02	2020-06-13 05:18:08+02	print	texlive-full	\N	texlive-full-20150521
2020-06-13 05:06:54+02	2020-06-13 05:18:19+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-13 05:18:21+02	2020-06-13 05:18:51+02	security	nyx	\N	nyx-2.1.0_1
2020-06-13 05:18:53+02	2020-06-13 05:20:44+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-06-13 05:20:46+02	2020-06-13 05:23:30+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.0
2020-06-13 05:18:11+02	2020-06-13 05:25:06+02	security	klee	\N	klee-2.1
2020-06-13 05:25:09+02	2020-06-13 05:27:18+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-13 05:27:20+02	2020-06-13 05:28:14+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-06-13 05:23:33+02	2020-06-13 05:28:43+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-06-13 05:28:45+02	2020-06-13 05:29:21+02	sysutils	panicmail	\N	panicmail-1.1
2020-06-13 05:28:16+02	2020-06-13 05:30:07+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-13 05:29:23+02	2020-06-13 05:30:09+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-06-13 05:30:11+02	2020-06-13 05:31:30+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-06-13 05:30:09+02	2020-06-13 05:31:49+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-13 05:31:52+02	2020-06-13 05:32:50+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-06-13 05:32:52+02	2020-06-13 05:33:55+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.0
2020-06-13 05:33:57+02	2020-06-13 05:34:28+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-06-13 05:34:30+02	2020-06-13 05:35:09+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-06-13 05:35:11+02	2020-06-13 05:35:45+02	www	py-dj22-django-cacheops	py37	py37-dj22-django-cacheops-4.2
2020-06-13 05:35:47+02	2020-06-13 05:36:19+02	www	py-dj22-django-cors-headers	py37	py37-dj22-django-cors-headers-3.2.1
2020-06-13 05:36:21+02	2020-06-13 05:36:51+02	www	py-dj22-django-debug-toolbar	py37	py37-dj22-django-debug-toolbar-2.1
2020-06-13 05:36:53+02	2020-06-13 05:37:26+02	www	py-dj22-django-filter	py37	py37-dj22-django-filter-2.2.0
2020-06-13 05:37:29+02	2020-06-13 05:38:04+02	www	py-dj22-django-mptt	py37	py37-dj22-django-mptt-0.9.1
2020-06-13 05:38:07+02	2020-06-13 05:38:45+02	www	py-dj22-django-prometheus	py37	py37-dj22-django-prometheus-1.1.0_1
2020-06-13 05:38:47+02	2020-06-13 05:39:22+02	www	py-dj22-django-tables2	py37	py37-dj22-django-tables2-2.2.1
2020-06-13 05:39:24+02	2020-06-13 05:39:57+02	www	py-dj22-django-taggit-serializer	py37	py37-dj22-django-taggit-serializer-0.1.7
2020-06-13 05:40:00+02	2020-06-13 05:40:31+02	www	py-dj22-django-timezone-field	py37	py37-dj22-django-timezone-field-4.0
2020-10-05 01:48:39+02	2020-10-05 01:51:11+02	x11-fm	caja	\N	caja-1.24.0
2020-06-13 05:40:34+02	2020-06-13 05:41:16+02	www	py-dj22-drf-yasg	py37	py37-dj22-drf-yasg-1.17.0
2020-06-13 05:41:18+02	2020-06-13 05:41:46+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-06-13 05:41:48+02	2020-06-13 05:42:20+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-06-13 05:42:22+02	2020-06-13 05:42:54+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-06-13 05:42:56+02	2020-06-13 05:45:54+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-13 05:45:56+02	2020-06-13 05:47:51+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-13 05:47:54+02	2020-06-13 05:52:07+02	x11	mate	\N	mate-1.22.2
2020-06-13 05:52:09+02	2020-06-13 05:56:53+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-13 05:56:54+02	2020-06-13 06:05:14+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-13 06:05:16+02	2020-06-13 06:07:36+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-13 06:07:37+02	2020-06-13 06:08:14+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-12 22:27:14+02	2020-06-13 06:08:37+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200519
2020-06-13 06:08:15+02	2020-06-13 06:11:47+02	deskutils	calibre	\N	calibre-4.18.0
2020-06-13 06:11:49+02	2020-06-13 06:12:02+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-12 22:27:51+02	2020-06-13 06:12:07+02	devel	llvm10	\N	llvm10-10.0.0_1
2020-06-13 06:12:03+02	2020-06-13 06:13:31+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-13 06:12:09+02	2020-06-13 06:13:58+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-13 06:13:32+02	2020-06-13 06:14:32+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-06-13 06:08:39+02	2020-06-13 06:52:38+02	editors	vscode	\N	vscode-1.45.1_2
2020-06-13 05:31:32+02	2020-06-13 07:37:38+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-13 08:22:16+02	2020-06-13 08:24:02+02	irc	irssi	\N	irssi-1.2.2,1
2020-06-13 08:22:16+02	2020-06-13 08:39:21+02	graphics	krita	\N	krita-4.2.9_4
2020-06-13 08:58:53+02	2020-06-13 09:43:09+02	local	E590T-localbase	\N	E590T-localbase-2020060702
2020-06-13 10:20:52+02	2020-06-13 10:28:08+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-13 10:28:08+02	2020-06-13 10:28:16+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-13 10:46:08+02	2020-06-13 11:04:29+02	local	E590T-localbase	\N	E590T-localbase-2020061300
2020-06-13 12:29:23+02	2020-06-13 12:30:35+02	devel	libsigc++20	\N	libsigc++-2.10.3
2020-06-13 12:29:23+02	2020-06-13 12:31:32+02	devel	libgtop	\N	libgtop-2.40.0
2020-06-13 12:29:23+02	2020-06-13 12:32:20+02	multimedia	dav1d	\N	dav1d-0.7.0_1
2020-06-13 12:31:32+02	2020-06-13 12:33:33+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-06-13 12:30:37+02	2020-06-13 12:33:35+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-06-13 12:29:23+02	2020-06-13 12:33:46+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_6
2020-06-13 12:33:36+02	2020-06-13 12:34:40+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-06-13 12:29:23+02	2020-06-13 12:35:53+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-06-13 12:34:43+02	2020-06-13 12:36:20+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-13 12:35:54+02	2020-06-13 12:37:16+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-06-13 12:37:19+02	2020-06-13 12:37:31+02	devel	mdds	\N	mdds-1.5.0
2020-06-13 12:36:22+02	2020-06-13 12:38:20+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-13 12:37:31+02	2020-06-13 12:41:38+02	graphics	libcdr01	\N	libcdr01-0.1.6
2020-06-13 12:33:47+02	2020-06-13 12:41:53+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-06-13 12:29:23+02	2020-06-13 12:42:03+02	databases	sqlite3	\N	sqlite3-3.32.2,1
2020-06-13 12:41:55+02	2020-06-13 12:45:57+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-13 12:38:21+02	2020-06-13 12:50:37+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-06-13 12:46:03+02	2020-06-13 12:53:16+02	security	nss	\N	nss-3.53
2020-06-13 12:50:40+02	2020-06-13 12:54:08+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.14.2
2020-06-13 12:42:04+02	2020-06-13 12:55:10+02	graphics	colord	\N	colord-1.3.5
2020-06-13 12:53:18+02	2020-06-13 12:56:02+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-13 12:54:10+02	2020-06-13 12:56:30+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-13 12:32:21+02	2020-06-13 12:58:27+02	multimedia	ffmpeg	\N	ffmpeg-4.2.3,1
2020-06-13 12:58:29+02	2020-06-13 12:59:26+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.7_7
2020-06-13 12:56:03+02	2020-06-13 13:00:30+02	devel	libsoup	\N	libsoup-2.62.3
2020-06-13 12:59:27+02	2020-06-13 13:01:27+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.18
2020-06-13 13:00:32+02	2020-06-13 13:02:38+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.18
2020-06-13 13:01:28+02	2020-06-13 13:04:40+02	net	geoclue	\N	geoclue-2.5.5
2020-06-13 13:02:40+02	2020-06-13 13:06:08+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-13 13:04:51+02	2020-06-13 13:07:00+02	www	py-django22	py37	py37-django22-2.2.13
2020-06-13 13:06:10+02	2020-06-13 13:10:12+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-13 13:07:04+02	2020-06-13 13:13:14+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-13 13:13:20+02	2020-06-13 13:17:14+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-13 13:17:23+02	2020-06-13 13:19:09+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-06-13 13:10:17+02	2020-06-13 13:22:38+02	security	gnupg	\N	gnupg-2.2.20
2020-06-13 13:19:13+02	2020-06-13 13:29:29+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-06-13 13:22:53+02	2020-06-13 13:31:45+02	security	gpgme	\N	gpgme-1.13.1
2020-06-13 13:31:57+02	2020-06-13 13:38:28+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-06-13 13:38:35+02	2020-06-13 13:46:50+02	devel	librest	\N	rest-0.7.93
2020-06-13 13:47:00+02	2020-06-13 13:52:40+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-13 13:52:54+02	2020-06-13 13:58:28+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-13 13:29:37+02	2020-06-13 14:04:00+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-13 13:58:33+02	2020-06-13 14:06:57+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-06-13 14:04:11+02	2020-06-13 14:09:27+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-13 14:07:04+02	2020-06-13 14:11:54+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-13 14:09:30+02	2020-06-13 14:12:51+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-13 14:11:57+02	2020-06-13 14:16:08+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-13 14:16:14+02	2020-06-13 14:19:35+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-13 14:12:53+02	2020-06-13 14:20:40+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-13 14:19:38+02	2020-06-13 14:21:54+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-13 12:55:13+02	2020-06-13 14:22:55+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-13 12:41:39+02	2020-06-13 14:23:20+02	www	node12	\N	node12-12.18.0
2020-06-13 14:20:42+02	2020-06-13 14:23:57+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-13 12:56:33+02	2020-06-13 14:24:05+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-13 14:21:55+02	2020-06-13 14:25:37+02	graphics	gegl	\N	gegl-0.4.22_6
2020-06-13 14:23:28+02	2020-06-13 14:26:03+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-13 14:23:00+02	2020-06-13 14:26:22+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-13 14:25:38+02	2020-06-13 14:29:33+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-13 14:26:05+02	2020-06-13 14:29:51+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-13 14:29:52+02	2020-06-13 14:32:07+02	net	remmina	\N	remmina-1.3.9
2020-06-13 12:33:34+02	2020-06-13 14:32:07+02	www	node	\N	node-14.4.0
2020-06-13 14:26:24+02	2020-06-13 14:32:24+02	security	gcr	\N	gcr-3.28.0
2020-06-13 14:29:34+02	2020-06-13 14:32:29+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-13 14:32:26+02	2020-06-13 14:34:08+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-13 14:32:11+02	2020-06-13 14:34:23+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-13 14:32:30+02	2020-06-13 14:35:54+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-13 14:34:24+02	2020-06-13 14:36:29+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-05 01:37:28+02	2020-10-05 01:51:59+02	cad	kicad	\N	kicad-5.1.7,2
2020-06-13 14:35:55+02	2020-06-13 14:38:28+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-13 14:32:11+02	2020-06-13 14:39:47+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-13 14:38:50+02	2020-06-13 14:41:39+02	x11	zenity	\N	zenity-3.28.1
2020-06-13 14:38:29+02	2020-06-13 14:41:39+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-13 14:36:30+02	2020-06-13 14:43:21+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-13 14:41:40+02	2020-06-13 14:43:26+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-13 14:43:27+02	2020-06-13 14:47:55+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-13 14:39:47+02	2020-06-13 14:48:35+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-13 14:41:43+02	2020-06-13 14:49:44+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-13 14:43:26+02	2020-06-13 14:50:20+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-13 14:50:21+02	2020-06-13 14:51:09+02	net	gssdp	\N	gssdp-1.2.2
2020-06-13 14:51:10+02	2020-06-13 14:52:57+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-13 14:52:58+02	2020-06-13 14:55:59+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-13 14:49:45+02	2020-06-13 14:58:43+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-13 14:47:57+02	2020-06-13 14:59:11+02	editors	emacs	full	emacs-26.3_4,3
2020-06-13 14:24:10+02	2020-06-13 15:00:42+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-13 14:58:45+02	2020-06-13 15:05:06+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-13 14:56:02+02	2020-06-13 15:06:18+02	print	fontforge	\N	fontforge-20190801_2
2020-06-13 15:00:44+02	2020-06-13 15:06:53+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-13 14:48:37+02	2020-06-13 15:18:59+02	graphics	gimp-app	\N	gimp-app-2.10.18_7,1
2020-06-13 15:19:04+02	2020-06-13 15:25:11+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-13 15:05:07+02	2020-06-13 15:28:15+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-13 15:25:14+02	2020-06-13 15:32:36+02	x11	xorg	\N	xorg-7.7_3
2020-06-13 15:06:54+02	2020-06-13 15:37:45+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-13 15:32:39+02	2020-06-13 15:39:02+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-13 15:28:19+02	2020-06-13 15:39:26+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-13 15:37:47+02	2020-06-13 15:44:34+02	x11-wm	marco	\N	marco-1.22.4
2020-06-13 15:39:33+02	2020-06-13 15:44:58+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-13 15:06:19+02	2020-06-13 15:46:16+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-13 15:39:04+02	2020-06-13 15:47:07+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-13 15:46:20+02	2020-06-13 15:51:47+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-13 15:51:51+02	2020-06-13 15:53:49+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-06-13 15:44:37+02	2020-06-13 16:00:04+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-13 15:53:53+02	2020-06-13 16:00:12+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-13 16:00:08+02	2020-06-13 16:00:50+02	graphics	qr-code-generator	\N	qr-code-generator-1.6.0
2020-06-13 15:45:01+02	2020-06-13 16:01:29+02	devel	liborcus	\N	liborcus-0.15.4
2020-06-13 16:00:52+02	2020-06-13 16:02:17+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-13 14:23:59+02	2020-06-13 16:02:38+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-13 16:01:37+02	2020-06-13 16:04:31+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-13 16:00:14+02	2020-06-13 16:04:46+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-13 16:02:20+02	2020-06-13 16:05:07+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-13 15:47:09+02	2020-06-13 16:06:53+02	graphics	libetonyek01	\N	libetonyek01-0.1.9_6,1
2020-06-13 16:04:34+02	2020-06-13 16:08:04+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-13 16:02:41+02	2020-06-13 16:08:13+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-13 16:06:54+02	2020-06-13 16:13:13+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-13 16:08:15+02	2020-06-13 16:18:32+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-13 16:04:51+02	2020-06-13 16:19:47+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-13 16:05:12+02	2020-06-13 16:20:11+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-13 16:20:12+02	2020-06-13 16:22:30+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-06-13 16:08:06+02	2020-06-13 16:23:15+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-13 16:18:37+02	2020-06-13 16:24:37+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-13 16:23:16+02	2020-06-13 16:25:41+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-13 16:13:15+02	2020-06-13 16:28:24+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-13 16:24:39+02	2020-06-13 16:29:13+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-13 16:22:32+02	2020-06-13 16:30:03+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-13 16:29:14+02	2020-06-13 16:30:24+02	www	youtube_dl	\N	youtube_dl-2020.06.06
2020-06-13 16:28:27+02	2020-06-13 16:30:59+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-13 16:30:29+02	2020-06-13 16:32:32+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-06-13 16:19:50+02	2020-06-13 16:32:36+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-13 16:31:01+02	2020-06-13 16:33:03+02	audio	faudio	\N	FAudio-20.06
2020-06-13 16:32:38+02	2020-06-13 16:33:28+02	databases	py-apsw	py37	py37-apsw-3.31.1
2020-06-13 16:33:29+02	2020-06-13 16:36:53+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-13 16:32:34+02	2020-06-13 16:37:14+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-13 16:30:05+02	2020-06-13 16:42:10+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-06-13 16:42:13+02	2020-06-13 16:52:36+02	graphics	eom	\N	eom-1.24.0
2020-06-13 16:37:18+02	2020-06-13 16:52:53+02	editors	pluma	\N	pluma-1.24.0
2020-06-13 16:33:05+02	2020-06-13 16:53:11+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-13 16:52:38+02	2020-06-13 16:56:41+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-13 16:52:55+02	2020-06-13 16:57:24+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-13 16:53:15+02	2020-06-13 17:05:43+02	graphics	py-gimp	\N	py27-gimp-2.10.18_7
2020-06-13 16:57:28+02	2020-06-13 17:10:25+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-13 17:05:46+02	2020-06-13 17:11:35+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-13 17:10:28+02	2020-06-13 17:19:10+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-13 17:19:13+02	2020-06-13 17:24:38+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-13 17:24:39+02	2020-06-13 17:28:43+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-13 17:28:49+02	2020-06-13 17:33:47+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-13 16:56:46+02	2020-06-13 17:38:37+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-13 17:33:51+02	2020-06-13 17:38:58+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-13 17:39:04+02	2020-06-13 17:44:09+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-13 17:38:47+02	2020-06-13 17:44:53+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-13 17:44:14+02	2020-06-13 17:50:06+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-13 17:44:55+02	2020-06-13 17:51:44+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-13 17:51:45+02	2020-06-13 17:55:05+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-13 17:50:15+02	2020-06-13 17:56:20+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-13 17:55:07+02	2020-06-13 18:02:00+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-13 16:25:43+02	2020-06-13 18:03:14+02	www	node10	\N	node10-10.21.0
2020-06-13 17:56:23+02	2020-06-13 18:17:27+02	print	lilypond	\N	lilypond-2.20.0
2020-06-13 17:11:38+02	2020-06-13 18:17:54+02	multimedia	vlc	\N	vlc-3.0.10_7,4
2020-06-13 18:03:17+02	2020-06-13 18:25:03+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-13 18:17:32+02	2020-06-13 18:25:30+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-13 18:02:02+02	2020-06-13 18:29:10+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-13 18:25:35+02	2020-06-13 18:35:43+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.0
2020-06-13 18:29:12+02	2020-06-13 18:38:35+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-13 18:38:37+02	2020-06-13 18:39:55+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-06-13 18:35:46+02	2020-06-13 18:39:59+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-13 18:39:57+02	2020-06-13 18:40:38+02	www	py-dj22-django-js-asset	py37	py37-dj22-django-js-asset-1.2.2
2020-06-13 18:40:05+02	2020-06-13 18:40:40+02	www	py-dj22-django-taggit	py37	py37-dj22-django-taggit-1.3.0
2020-06-13 18:40:41+02	2020-06-13 18:41:27+02	www	py-dj22-djangorestframework	py37	py37-dj22-djangorestframework-3.11.0
2020-06-13 18:40:42+02	2020-06-13 18:44:02+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-13 18:44:07+02	2020-06-13 18:44:46+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-06-13 18:41:29+02	2020-06-13 18:45:03+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-13 18:17:58+02	2020-06-13 18:46:47+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-13 18:45:05+02	2020-06-13 18:50:49+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-13 18:46:50+02	2020-06-13 18:55:44+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-13 18:50:51+02	2020-06-13 18:57:09+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-13 18:55:50+02	2020-06-13 19:06:51+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-13 18:57:12+02	2020-06-13 19:10:21+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-13 19:06:57+02	2020-06-13 19:18:56+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-13 19:10:24+02	2020-06-13 19:20:20+02	deskutils	calibre	\N	calibre-4.18.0
2020-06-13 19:18:58+02	2020-06-13 19:22:03+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-13 19:22:05+02	2020-06-13 19:26:02+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-13 18:44:49+02	2020-06-13 19:29:19+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-13 19:29:23+02	2020-06-13 19:30:20+02	devel	py-dj22-django-rq	py37	py37-dj22-django-rq-2.3.2
2020-06-13 18:25:10+02	2020-06-13 19:31:42+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-13 19:30:24+02	2020-06-13 19:58:47+02	devel	subversion	\N	subversion-1.14.0
2020-06-13 19:58:50+02	2020-06-13 20:09:02+02	editors	vim	\N	vim-8.2.0869
2020-06-13 19:31:44+02	2020-06-13 20:12:24+02	editors	codelite	\N	codelite-12.0_4
2020-06-13 20:12:27+02	2020-06-13 21:03:23+02	emulators	wine	\N	wine-5.0.1,1
2020-06-13 19:20:24+02	2020-06-13 21:06:30+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-13 21:06:33+02	2020-06-13 21:16:18+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-13 21:16:20+02	2020-06-13 21:20:37+02	games	xcowsay	\N	xcowsay-1.5
2020-06-13 21:20:47+02	2020-06-13 21:30:39+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-13 21:30:43+02	2020-06-13 21:35:43+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-13 21:03:31+02	2020-06-13 21:39:39+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-13 19:26:08+02	2020-06-13 21:46:02+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-13 21:46:09+02	2020-06-13 21:51:21+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-13 21:39:42+02	2020-06-13 22:07:47+02	graphics	krita	\N	krita-4.2.9_4
2020-06-13 14:59:15+02	2020-06-13 22:10:06+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-13 22:23:00+02	2020-06-14 00:04:24+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-14 00:09:28+02	2020-06-14 00:11:19+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-14 00:09:28+02	2020-06-14 00:12:34+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-14 00:09:28+02	2020-06-14 00:13:06+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-14 00:11:23+02	2020-06-14 00:13:59+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-14 00:11:20+02	2020-06-14 00:14:34+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-14 00:12:35+02	2020-06-14 00:17:20+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-14 00:13:08+02	2020-06-14 00:18:41+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-14 00:18:43+02	2020-06-14 00:19:34+02	net	vnstat	\N	vnstat-2.6_1
2020-06-14 00:14:03+02	2020-06-14 00:20:38+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-14 00:17:22+02	2020-06-14 00:20:54+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-14 00:09:28+02	2020-06-14 00:21:16+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-14 00:19:36+02	2020-06-14 00:21:24+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-14 00:14:36+02	2020-06-14 00:21:48+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-14 00:20:44+02	2020-06-14 00:22:00+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-14 00:20:55+02	2020-06-14 00:22:03+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-14 00:21:19+02	2020-06-14 00:22:16+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-14 00:21:25+02	2020-06-14 00:22:23+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-14 00:21:53+02	2020-06-14 00:22:49+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-14 00:22:00+02	2020-06-14 00:22:55+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-14 00:22:05+02	2020-06-14 00:22:58+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-14 00:22:24+02	2020-06-14 00:23:23+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-14 00:22:17+02	2020-06-14 00:23:43+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-14 00:22:49+02	2020-06-14 00:23:48+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-14 00:23:44+02	2020-06-14 00:24:03+02	security	nyx	\N	nyx-2.1.0_1
2020-06-14 00:23:49+02	2020-06-14 00:25:28+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-14 00:23:23+02	2020-06-14 00:25:30+02	security	klee	\N	klee-2.1
2020-06-14 00:25:30+02	2020-06-14 00:25:45+02	www	py-dj22-django-cacheops	py37	py37-dj22-django-cacheops-4.2
2020-06-14 00:25:29+02	2020-06-14 00:26:03+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-06-14 00:25:45+02	2020-06-14 00:26:04+02	www	py-dj22-django-cors-headers	py37	py37-dj22-django-cors-headers-3.2.1
2020-06-14 00:22:55+02	2020-06-14 00:26:18+02	print	auctex	full	auctex-emacs26-12.2
2020-06-14 00:26:03+02	2020-06-14 00:26:18+02	www	py-dj22-django-debug-toolbar	py37	py37-dj22-django-debug-toolbar-2.1
2020-06-14 00:26:04+02	2020-06-14 00:26:20+02	www	py-dj22-django-filter	py37	py37-dj22-django-filter-2.2.0
2020-06-14 00:26:19+02	2020-06-14 00:26:34+02	www	py-dj22-django-mptt	py37	py37-dj22-django-mptt-0.9.1
2020-06-14 00:26:19+02	2020-06-14 00:26:35+02	www	py-dj22-django-prometheus	py37	py37-dj22-django-prometheus-1.1.0_1
2020-06-14 00:26:21+02	2020-06-14 00:26:36+02	www	py-dj22-django-tables2	py37	py37-dj22-django-tables2-2.2.1
2020-06-14 00:09:28+02	2020-06-14 00:26:40+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-14 00:26:35+02	2020-06-14 00:26:49+02	www	py-dj22-django-taggit-serializer	py37	py37-dj22-django-taggit-serializer-0.1.7
2020-06-14 00:26:36+02	2020-06-14 00:26:50+02	www	py-dj22-django-timezone-field	py37	py37-dj22-django-timezone-field-4.0
2020-06-14 00:26:41+02	2020-06-14 00:26:54+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-06-14 00:26:37+02	2020-06-14 00:26:57+02	www	py-dj22-drf-yasg	py37	py37-dj22-drf-yasg-1.17.0
2020-06-14 00:26:57+02	2020-06-14 00:27:11+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-14 00:26:50+02	2020-06-14 00:27:48+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-14 00:27:12+02	2020-06-14 00:27:50+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-06-14 00:26:51+02	2020-06-14 00:27:59+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-14 00:22:59+02	2020-06-14 00:28:14+02	print	texlive-full	\N	texlive-full-20150521
2020-06-14 00:24:04+02	2020-06-14 01:02:03+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-14 00:26:54+02	2020-06-14 01:02:56+02	editors	vscode	\N	vscode-1.45.1_2
2020-06-14 08:39:14+02	2020-06-14 08:39:28+02	sysutils	ncdu	\N	ncdu-1.15.1
2020-06-14 08:39:14+02	2020-06-14 08:42:49+02	graphics	netpbm	\N	netpbm-10.90.04
2020-06-14 08:39:14+02	2020-06-14 08:46:31+02	lang	python37	\N	python37-3.7.7_1
2020-06-14 08:46:32+02	2020-06-14 08:47:17+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-06-14 08:46:32+02	2020-06-14 08:47:36+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-06-14 08:47:19+02	2020-06-14 08:48:02+02	devel	py-six	py37	py37-six-1.14.0
2020-06-14 08:47:39+02	2020-06-14 08:48:21+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-06-14 08:48:05+02	2020-06-14 08:48:30+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-06-14 08:48:22+02	2020-06-14 08:48:43+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-06-14 08:48:31+02	2020-06-14 08:49:13+02	devel	py-cffi	py37	py37-cffi-1.14.0
2020-06-14 08:46:32+02	2020-06-14 08:49:13+02	devel	ninja	\N	ninja-1.10.0,2
2020-06-14 08:48:45+02	2020-06-14 08:49:17+02	dns	py-idna	py37	py37-idna-2.8
2020-06-14 08:49:19+02	2020-06-14 08:49:45+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-06-14 08:49:16+02	2020-06-14 08:50:21+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-06-14 08:39:14+02	2020-06-14 08:50:25+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-14 08:49:16+02	2020-06-14 08:50:43+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-06-14 08:50:22+02	2020-06-14 08:50:56+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-06-14 08:50:46+02	2020-06-14 08:51:19+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-06-14 08:50:57+02	2020-06-14 08:51:30+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-06-14 08:51:22+02	2020-06-14 08:51:54+02	security	py-certifi	py37	py37-certifi-2020.4.5.1
2020-06-14 08:51:32+02	2020-06-14 08:52:09+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-06-14 08:51:59+02	2020-06-14 08:52:34+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-06-14 08:52:11+02	2020-06-14 08:52:38+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-06-14 08:52:36+02	2020-06-14 08:53:08+02	devel	py-packaging	py37	py37-packaging-20.4
2020-06-14 08:52:40+02	2020-06-14 08:53:20+02	www	py-requests	py37	py37-requests-2.22.0
2020-06-14 08:53:08+02	2020-06-14 08:53:41+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-06-14 08:53:21+02	2020-06-14 08:53:44+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-06-14 08:49:47+02	2020-06-14 08:53:48+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-06-14 08:50:27+02	2020-06-14 08:53:53+02	lang	cython	py37	py37-cython-0.29.15
2020-06-14 08:53:48+02	2020-06-14 08:54:19+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-06-14 08:53:49+02	2020-06-14 08:54:22+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-06-14 08:53:43+02	2020-06-14 08:54:26+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-06-14 08:53:57+02	2020-06-14 08:54:50+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-06-14 08:54:23+02	2020-06-14 08:54:51+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-06-14 08:54:23+02	2020-06-14 08:54:51+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-06-14 08:54:30+02	2020-06-14 08:55:00+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-06-14 08:54:53+02	2020-06-14 08:55:21+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-06-14 08:54:53+02	2020-06-14 08:55:23+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-06-14 08:55:02+02	2020-06-14 08:55:38+02	devel	py-future	py37	py37-future-0.18.2
2020-06-14 08:54:57+02	2020-06-14 08:55:39+02	devel	meson	\N	meson-0.54.2
2020-06-14 08:55:25+02	2020-06-14 08:55:59+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-06-14 08:55:40+02	2020-06-14 08:56:13+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-06-14 08:55:25+02	2020-06-14 08:56:31+02	textproc	py-sphinx	py37	py37-sphinx-3.0.4,1
2020-06-14 08:56:01+02	2020-06-14 08:56:31+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-06-14 08:56:15+02	2020-06-14 08:56:43+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-06-14 08:56:34+02	2020-06-14 08:57:09+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-06-14 08:56:34+02	2020-06-14 08:57:09+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-06-14 08:56:46+02	2020-06-14 08:57:19+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.7_7
2020-06-14 08:55:40+02	2020-06-14 08:57:22+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-06-14 08:57:11+02	2020-06-14 08:57:30+02	textproc	py-toml	py37	py37-toml-0.10.0
2020-06-14 08:57:19+02	2020-06-14 08:57:42+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-06-14 08:57:24+02	2020-06-14 08:57:49+02	textproc	itstool	\N	itstool-2.0.6
2020-06-14 08:57:31+02	2020-06-14 08:57:54+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-06-14 08:46:32+02	2020-06-14 08:58:36+02	devel	glib20	\N	glib-2.56.3_7,1
2020-06-14 08:57:44+02	2020-06-14 08:58:38+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-06-14 08:57:53+02	2020-06-14 08:58:58+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-06-14 08:57:54+02	2020-06-14 08:59:03+02	www	py-django22	py37	py37-django22-2.2.13
2020-06-14 08:58:40+02	2020-06-14 08:59:41+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-06-14 08:58:59+02	2020-06-14 09:00:29+02	misc	shared-mime-info	\N	shared-mime-info-1.10_2
2020-06-14 08:59:04+02	2020-06-14 09:00:47+02	textproc	libcroco	\N	libcroco-0.6.13
2020-06-14 09:00:31+02	2020-06-14 09:01:21+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-06-14 09:00:48+02	2020-06-14 09:02:37+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-06-14 09:01:24+02	2020-06-14 09:03:24+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-06-14 08:58:38+02	2020-06-14 09:06:22+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-14 09:02:40+02	2020-06-14 09:09:42+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-06-14 08:59:42+02	2020-06-14 09:09:53+02	devel	qt5-core	\N	qt5-core-5.14.2_3
2020-06-14 09:03:25+02	2020-06-14 09:10:33+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-06-14 09:06:26+02	2020-06-14 09:10:52+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1,1
2020-06-14 09:09:43+02	2020-06-14 09:10:57+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-14 09:10:34+02	2020-06-14 09:14:26+02	devel	qt5-dbus	\N	qt5-dbus-5.14.2
2020-06-14 09:09:55+02	2020-06-14 09:14:50+02	net	qt5-network	\N	qt5-network-5.14.2
2020-06-14 09:14:29+02	2020-06-14 09:15:39+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-06-14 09:10:53+02	2020-06-14 09:15:52+02	print	harfbuzz	\N	harfbuzz-2.6.7
2020-06-14 09:15:40+02	2020-06-14 09:16:44+02	accessibility	atk	\N	atk-2.36.0
2020-06-14 09:14:53+02	2020-06-14 09:16:52+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-06-14 09:10:58+02	2020-06-14 09:17:55+02	net	avahi-app	\N	avahi-app-0.7_3
2020-06-14 09:15:53+02	2020-06-14 09:21:09+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-14 09:16:53+02	2020-06-14 09:21:57+02	devel	libgsf	\N	libgsf-1.14.46
2020-06-14 09:21:59+02	2020-06-14 09:23:03+02	devel	libgudev	\N	libgudev-230_1
2020-06-14 09:16:45+02	2020-06-14 09:23:24+02	sysutils	polkit	\N	polkit-0.116
2020-06-14 09:23:04+02	2020-06-14 09:23:37+02	x11	libwacom	\N	libwacom-1.3
2020-06-14 09:17:59+02	2020-06-14 09:23:43+02	print	cups	\N	cups-2.2.13
2020-06-14 09:21:14+02	2020-06-14 09:24:19+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-14 09:23:39+02	2020-06-14 09:24:30+02	x11	libinput	\N	libinput-1.15.5
2020-06-14 09:24:22+02	2020-06-14 09:25:22+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-06-14 09:23:44+02	2020-06-14 09:32:46+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-06-14 09:23:25+02	2020-06-14 09:34:11+02	graphics	colord	\N	colord-1.3.5
2020-06-14 09:34:17+02	2020-06-14 09:38:38+02	databases	qt5-sql	\N	qt5-sql-5.14.2
2020-06-14 09:25:24+02	2020-06-14 09:45:06+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-14 09:24:30+02	2020-06-14 09:46:45+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-14 09:46:48+02	2020-06-14 09:52:25+02	devel	qt5-testlib	\N	qt5-testlib-5.14.2
2020-06-14 09:45:08+02	2020-06-14 10:03:52+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-14 09:52:45+02	2020-06-14 10:07:46+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.14.2
2020-06-14 10:03:57+02	2020-06-14 10:28:44+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.7_2
2020-06-14 09:32:53+02	2020-06-14 10:34:29+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-14 10:08:02+02	2020-06-14 10:40:17+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-14 10:40:34+02	2020-06-14 10:43:06+02	textproc	qt5-xml	\N	qt5-xml-5.14.2
2020-06-14 10:28:47+02	2020-06-14 10:44:15+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-06-14 10:43:07+02	2020-06-14 10:44:51+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.14.2
2020-06-14 10:44:52+02	2020-06-14 10:45:48+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-06-14 10:34:46+02	2020-06-14 10:45:49+02	lang	vala	\N	vala-0.40.15,1
2020-06-14 09:38:54+02	2020-06-14 10:46:19+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-14 10:45:50+02	2020-06-14 10:46:39+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-06-14 10:46:22+02	2020-06-14 10:48:07+02	devel	dconf	\N	dconf-0.28.0
2020-06-14 10:46:42+02	2020-06-14 10:48:26+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-14 10:45:50+02	2020-06-14 10:48:41+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-14 10:48:09+02	2020-06-14 10:50:36+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.14.2
2020-06-14 10:44:17+02	2020-06-14 10:50:48+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-06-14 10:48:27+02	2020-06-14 10:52:19+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-14 10:50:51+02	2020-06-14 10:53:01+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.70.0_2
2020-06-14 10:50:37+02	2020-06-14 10:53:07+02	devel	libsoup	\N	libsoup-2.62.3
2020-06-14 10:53:08+02	2020-06-14 10:54:36+02	devel	qt5-concurrent	\N	qt5-concurrent-5.14.2
2020-06-14 10:52:20+02	2020-06-14 10:55:40+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-14 10:55:42+02	2020-06-14 10:56:32+02	devel	json-glib	\N	json-glib-1.4.4
2020-06-14 10:56:32+02	2020-06-14 10:57:26+02	devel	libnotify	\N	libnotify-0.7.8
2020-06-14 10:54:37+02	2020-06-14 10:57:46+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-14 10:57:28+02	2020-06-14 10:58:51+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-14 10:58:52+02	2020-06-14 11:01:18+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-14 11:01:24+02	2020-06-14 11:02:36+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-06-14 10:57:47+02	2020-06-14 11:02:55+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-06-14 11:02:56+02	2020-06-14 11:03:54+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-06-14 11:02:36+02	2020-06-14 11:04:01+02	security	libsecret	\N	libsecret-0.18.6_1
2020-06-14 11:03:54+02	2020-06-14 11:04:27+02	print	libraqm	\N	libraqm-0.6.0
2020-06-14 08:57:11+02	2020-06-14 11:04:51+02	devel	llvm80	\N	llvm80-8.0.1_3
2020-06-14 11:04:02+02	2020-06-14 11:04:58+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-14 11:04:54+02	2020-06-14 11:05:25+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-14 10:48:44+02	2020-06-14 11:05:31+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-14 10:53:02+02	2020-06-14 11:05:40+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-14 11:05:00+02	2020-06-14 11:05:44+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-14 11:05:26+02	2020-06-14 11:05:51+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_1
2020-06-14 11:05:51+02	2020-06-14 11:06:43+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_1
2020-06-14 11:05:32+02	2020-06-14 11:08:37+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-14 11:06:44+02	2020-06-14 11:09:22+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-14 11:08:43+02	2020-06-14 11:10:59+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.70.0
2020-06-14 11:05:45+02	2020-06-14 11:11:21+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_6
2020-06-14 11:11:23+02	2020-06-14 11:12:50+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-14 11:09:24+02	2020-06-14 11:12:50+02	graphics	graphene	\N	graphene-1.10.0
2020-06-14 11:11:01+02	2020-06-14 11:13:42+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-06-14 11:04:28+02	2020-06-14 11:14:17+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-14 11:12:51+02	2020-06-14 11:15:36+02	devel	qt5-script	\N	qt5-script-5.14.2
2020-06-14 11:14:18+02	2020-06-14 11:17:05+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-14 11:15:37+02	2020-06-14 11:17:56+02	net	geoclue	\N	geoclue-2.5.5
2020-06-14 11:17:07+02	2020-06-14 11:18:15+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-14 11:12:51+02	2020-06-14 11:18:37+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-14 11:13:43+02	2020-06-14 11:19:18+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-06-14 11:18:21+02	2020-06-14 11:19:22+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-14 11:18:38+02	2020-06-14 11:20:41+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-14 11:17:58+02	2020-06-14 11:20:55+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-14 11:19:24+02	2020-06-14 11:21:51+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-14 11:19:19+02	2020-06-14 11:21:53+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-14 11:21:55+02	2020-06-14 11:23:27+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-06-14 11:20:44+02	2020-06-14 11:23:32+02	sysutils	policykit	\N	policykit-0.9_10
2020-06-14 11:23:27+02	2020-06-14 11:24:33+02	archivers	kf5-karchive	\N	kf5-karchive-5.70.0
2020-06-14 11:23:33+02	2020-06-14 11:26:28+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-14 11:21:53+02	2020-06-14 11:27:49+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-14 11:24:39+02	2020-06-14 11:29:38+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-14 11:29:42+02	2020-06-14 11:33:57+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-14 11:26:30+02	2020-06-14 11:38:21+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-14 11:38:26+02	2020-06-14 11:41:49+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-14 11:34:10+02	2020-06-14 11:43:03+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-14 11:05:40+02	2020-06-14 11:46:06+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-14 11:43:15+02	2020-06-14 11:47:42+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-14 11:41:56+02	2020-06-14 11:49:49+02	sysutils	hal	\N	hal-0.5.14_34
2020-06-14 11:46:11+02	2020-06-14 11:56:45+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-14 11:27:52+02	2020-06-14 12:02:08+02	devel	gdb	py37	gdb-9.2_1
2020-06-14 12:02:10+02	2020-06-14 12:03:50+02	comms	qt5-serialport	\N	qt5-serialport-5.14.2
2020-06-14 11:49:51+02	2020-06-14 12:06:06+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-06-14 12:03:52+02	2020-06-14 12:06:44+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-14 12:06:11+02	2020-06-14 12:08:07+02	textproc	asciidoc	\N	asciidoc-9.0.0rc2
2020-06-14 11:56:48+02	2020-06-14 12:10:10+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-14 11:47:51+02	2020-06-14 12:10:27+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-14 12:06:49+02	2020-06-14 12:10:48+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-14 12:10:49+02	2020-06-14 12:14:00+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-14 12:14:02+02	2020-06-14 12:17:05+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-06-14 12:10:12+02	2020-06-14 12:17:16+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-06-14 12:17:06+02	2020-06-14 12:19:20+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-14 12:10:30+02	2020-06-14 12:19:49+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-14 12:19:52+02	2020-06-14 12:21:50+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.70.0
2020-06-14 12:19:22+02	2020-06-14 12:22:10+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-14 12:17:20+02	2020-06-14 12:22:15+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-14 12:08:09+02	2020-06-14 12:22:45+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-06-14 12:21:50+02	2020-06-14 12:23:48+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-14 12:22:11+02	2020-06-14 12:24:42+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-14 12:22:19+02	2020-06-14 12:25:18+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-14 15:04:54+02	2020-06-14 15:09:16+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-14 12:24:44+02	2020-06-14 12:28:25+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-06-14 12:25:22+02	2020-06-14 12:29:38+02	net	remmina	\N	remmina-1.3.9
2020-06-14 12:22:47+02	2020-06-14 12:30:27+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-14 12:29:44+02	2020-06-14 12:31:22+02	x11	mate-menus	\N	mate-menus-1.22.1_1
2020-06-14 12:23:49+02	2020-06-14 12:31:53+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-06-14 12:30:32+02	2020-06-14 12:32:11+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-14 11:20:58+02	2020-06-14 12:32:36+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-14 12:31:23+02	2020-06-14 12:32:59+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-14 12:31:54+02	2020-06-14 12:33:21+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-06-14 12:32:37+02	2020-06-14 12:33:27+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-06-14 12:28:27+02	2020-06-14 12:33:32+02	security	gcr	\N	gcr-3.28.0
2020-06-14 12:33:27+02	2020-06-14 12:35:16+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-14 12:33:22+02	2020-06-14 12:35:39+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-14 12:33:35+02	2020-06-14 12:36:39+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-14 12:35:43+02	2020-06-14 12:37:59+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-06-14 12:33:00+02	2020-06-14 12:39:55+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.70.0
2020-06-14 12:39:56+02	2020-06-14 12:40:24+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-06-14 12:36:42+02	2020-06-14 12:40:46+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-06-14 12:40:26+02	2020-06-14 12:41:01+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-06-14 12:40:48+02	2020-06-14 12:41:24+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-06-14 12:38:04+02	2020-06-14 12:42:21+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-06-14 12:41:03+02	2020-06-14 12:43:36+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-14 12:42:22+02	2020-06-14 12:46:08+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-06-14 12:41:25+02	2020-06-14 12:46:39+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-06-14 12:43:39+02	2020-06-14 12:47:36+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-14 12:35:17+02	2020-06-14 12:48:29+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-14 12:46:43+02	2020-06-14 12:49:21+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-06-14 12:46:14+02	2020-06-14 12:50:08+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-06-14 12:48:32+02	2020-06-14 12:50:12+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-06-14 12:49:23+02	2020-06-14 12:50:46+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-14 12:47:39+02	2020-06-14 12:51:12+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-06-14 12:51:14+02	2020-06-14 12:52:19+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-06-14 12:50:09+02	2020-06-14 12:52:27+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-14 12:50:52+02	2020-06-14 12:53:16+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-14 12:52:20+02	2020-06-14 12:54:28+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-14 12:53:17+02	2020-06-14 12:54:59+02	devel	py-lxml	py37	py37-lxml-4.5.1
2020-06-14 12:52:28+02	2020-06-14 12:55:04+02	devel	libgtop	\N	libgtop-2.40.0
2020-06-14 08:39:14+02	2020-06-14 12:55:24+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200606
2020-06-14 12:55:03+02	2020-06-14 12:56:08+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-14 12:50:15+02	2020-06-14 12:56:51+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-06-14 12:55:27+02	2020-06-14 12:57:03+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-14 12:54:29+02	2020-06-14 12:57:39+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-14 12:56:08+02	2020-06-14 12:57:46+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-14 12:57:41+02	2020-06-14 12:58:13+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.7_6
2020-06-14 12:56:55+02	2020-06-14 12:59:15+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-14 12:57:50+02	2020-06-14 13:00:14+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-14 12:57:06+02	2020-06-14 13:01:15+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-14 12:58:15+02	2020-06-14 13:01:31+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-06-14 13:00:17+02	2020-06-14 13:02:17+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-14 13:02:17+02	2020-06-14 13:02:51+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7
2020-06-14 12:55:08+02	2020-06-14 13:03:22+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-14 12:59:18+02	2020-06-14 13:04:40+02	devel	talloc	\N	talloc-2.3.0
2020-06-14 13:01:32+02	2020-06-14 13:06:07+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-06-14 13:01:16+02	2020-06-14 13:06:10+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-14 13:02:54+02	2020-06-14 13:06:31+02	sysutils	upower	\N	upower-0.99.11
2020-06-14 13:06:07+02	2020-06-14 13:07:01+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-06-14 13:04:45+02	2020-06-14 13:07:19+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-14 13:06:33+02	2020-06-14 13:08:31+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-14 13:07:03+02	2020-06-14 13:10:03+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-14 13:07:21+02	2020-06-14 13:10:39+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-06-14 13:10:06+02	2020-06-14 13:12:30+02	devel	gamin	\N	gamin-0.1.10_10
2020-06-14 13:08:31+02	2020-06-14 13:14:18+02	databases	tdb	\N	tdb-1.4.2,1
2020-06-14 13:10:41+02	2020-06-14 13:14:30+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-14 13:06:11+02	2020-06-14 13:14:52+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-14 13:14:21+02	2020-06-14 13:14:54+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-06-14 13:12:32+02	2020-06-14 13:15:03+02	devel	librest	\N	rest-0.7.93
2020-06-14 13:14:32+02	2020-06-14 13:15:05+02	devel	py-more-itertools	py37	py37-more-itertools-8.3.0
2020-06-14 13:15:07+02	2020-06-14 13:16:46+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-14 13:14:54+02	2020-06-14 13:17:09+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-14 13:15:07+02	2020-06-14 13:17:41+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-06-14 13:17:11+02	2020-06-14 13:18:10+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-06-14 13:16:52+02	2020-06-14 13:18:27+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-06-14 13:14:56+02	2020-06-14 13:18:40+02	devel	tevent	\N	tevent-0.10.1
2020-06-14 13:03:26+02	2020-06-14 13:18:44+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-14 13:18:42+02	2020-06-14 13:19:04+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-06-14 13:17:42+02	2020-06-14 13:19:08+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-14 13:18:46+02	2020-06-14 13:19:40+02	x11	babl	\N	babl-0.1.74
2020-06-14 13:18:11+02	2020-06-14 13:19:53+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-14 13:18:29+02	2020-06-14 13:20:47+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-06-14 13:19:09+02	2020-06-14 13:21:52+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-14 13:19:42+02	2020-06-14 13:21:54+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-14 13:19:55+02	2020-06-14 13:22:14+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-14 13:19:05+02	2020-06-14 13:22:18+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-14 13:20:49+02	2020-06-14 13:22:45+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-14 13:21:54+02	2020-06-14 13:24:27+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-14 13:22:17+02	2020-06-14 13:24:37+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-14 13:22:18+02	2020-06-14 13:24:37+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-14 13:21:56+02	2020-06-14 13:26:02+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-14 13:26:04+02	2020-06-14 13:26:37+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-06-14 13:24:43+02	2020-06-14 13:27:26+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-14 13:24:31+02	2020-06-14 13:27:46+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-14 13:27:51+02	2020-06-14 13:28:19+02	devel	py-click	py37	py37-click-7.0
2020-06-14 13:28:19+02	2020-06-14 13:28:42+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-06-14 13:24:43+02	2020-06-14 13:28:52+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-14 13:26:38+02	2020-06-14 13:29:20+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-14 13:28:51+02	2020-06-14 13:29:20+02	devel	py-olefile	py37	py37-olefile-0.46
2020-06-14 13:28:55+02	2020-06-14 13:29:23+02	devel	py-ply	py37	py37-ply-3.11
2020-06-14 13:27:29+02	2020-06-14 13:29:28+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-14 13:29:21+02	2020-06-14 13:29:48+02	devel	py-zipp	py37	py37-zipp-0.6.0_3
2020-06-14 13:29:21+02	2020-06-14 13:29:49+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-06-14 13:29:29+02	2020-06-14 13:30:12+02	devel	scons	py37	scons-py37-3.1.2
2020-06-14 13:27:50+02	2020-06-14 13:30:39+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-14 13:29:54+02	2020-06-14 13:30:57+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-06-14 13:30:40+02	2020-06-14 13:31:20+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-14 13:30:13+02	2020-06-14 13:31:58+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-06-14 13:29:24+02	2020-06-14 13:33:15+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-14 13:29:54+02	2020-06-14 13:33:47+02	graphics	gegl	\N	gegl-0.4.22_6
2020-06-14 13:33:49+02	2020-06-14 13:35:36+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-14 13:33:17+02	2020-06-14 13:36:05+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-14 13:31:23+02	2020-06-14 13:37:12+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-14 13:36:07+02	2020-06-14 13:37:47+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-06-14 13:37:13+02	2020-06-14 13:40:32+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-14 13:37:51+02	2020-06-14 13:40:49+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-14 13:40:33+02	2020-06-14 13:40:56+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-06-14 13:40:52+02	2020-06-14 13:41:23+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-06-14 13:41:02+02	2020-06-14 13:41:29+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-06-14 13:35:42+02	2020-06-14 13:42:28+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-14 13:30:59+02	2020-06-14 13:42:55+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-14 13:41:25+02	2020-06-14 13:44:24+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-14 13:42:32+02	2020-06-14 13:45:08+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-14 13:41:34+02	2020-06-14 13:47:05+02	x11	zenity	\N	zenity-3.28.1
2020-06-14 13:45:10+02	2020-06-14 13:48:37+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-14 13:48:39+02	2020-06-14 13:49:16+02	comms	py-serial	py37	py37-serial-3.4_1
2020-06-14 13:49:18+02	2020-06-14 13:50:01+02	databases	py-redis	py37	py37-redis-3.3.11
2020-06-14 13:44:27+02	2020-06-14 13:55:00+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-14 13:42:57+02	2020-06-14 13:55:25+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-06-14 13:47:07+02	2020-06-14 13:55:59+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-14 13:50:05+02	2020-06-14 13:57:09+02	devel	git	\N	git-2.27.0
2020-06-14 13:32:02+02	2020-06-14 13:58:35+02	net	samba410	\N	samba410-4.10.15
2020-06-14 13:58:36+02	2020-06-14 13:59:00+02	devel	py-Automat	py37	py37-Automat-0.8.0
2020-06-14 13:59:02+02	2020-06-14 13:59:52+02	devel	py-appdirs	py37	py37-appdirs-1.4.3
2020-06-14 13:56:04+02	2020-06-14 14:00:20+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-14 13:59:56+02	2020-06-14 14:00:45+02	devel	py-backports	py37	py37-backports-1
2020-06-14 14:00:23+02	2020-06-14 14:01:12+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-06-14 14:00:50+02	2020-06-14 14:01:31+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-06-14 14:01:16+02	2020-06-14 14:02:10+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-06-14 14:02:13+02	2020-06-14 14:02:53+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-0.23
2020-06-14 13:55:27+02	2020-06-14 14:03:02+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-14 13:55:05+02	2020-06-14 14:03:21+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-14 14:02:58+02	2020-06-14 14:03:40+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-06-14 14:03:04+02	2020-06-14 14:03:47+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-06-14 14:03:25+02	2020-06-14 14:04:17+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-06-14 14:03:43+02	2020-06-14 14:04:33+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-06-14 14:03:50+02	2020-06-14 14:04:53+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-06-14 14:04:21+02	2020-06-14 14:05:23+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-06-14 14:04:36+02	2020-06-14 14:05:59+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-06-14 14:01:35+02	2020-06-14 14:06:06+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-14 14:06:01+02	2020-06-14 14:07:47+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-14 14:04:55+02	2020-06-14 14:13:20+02	dns	bind-tools	\N	bind-tools-9.16.3_1
2020-06-14 14:13:22+02	2020-06-14 14:14:52+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-06-14 14:14:56+02	2020-06-14 14:17:46+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-14 14:17:48+02	2020-06-14 14:18:23+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0
2020-06-14 14:07:54+02	2020-06-14 14:19:34+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-14 14:18:26+02	2020-06-14 14:19:36+02	net	gssdp	\N	gssdp-1.2.2
2020-06-14 14:05:28+02	2020-06-14 14:21:20+02	editors	emacs	full	emacs-26.3_4,3
2020-06-14 14:19:36+02	2020-06-14 14:22:14+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-14 14:19:38+02	2020-06-14 14:25:46+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-14 14:25:50+02	2020-06-14 14:39:55+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-06-14 14:21:22+02	2020-06-14 14:40:19+02	print	fontforge	\N	fontforge-20190801_2
2020-06-14 14:40:00+02	2020-06-14 14:40:41+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-06-14 14:40:20+02	2020-06-14 14:43:01+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-14 14:43:02+02	2020-06-14 14:43:32+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-06-14 14:43:32+02	2020-06-14 14:43:56+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-06-14 14:43:57+02	2020-06-14 14:44:33+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-06-14 14:06:10+02	2020-06-14 14:46:17+02	graphics	gimp-app	\N	gimp-app-2.10.18_7,1
2020-06-14 14:46:18+02	2020-06-14 14:46:51+02	www	py-h2	py37	py37-h2-3.2.0
2020-06-14 14:40:43+02	2020-06-14 14:46:54+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-14 14:46:54+02	2020-06-14 14:47:32+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-06-14 14:46:56+02	2020-06-14 14:47:32+02	www	py-hyperlink	py37	py37-hyperlink-19.0.0
2020-06-14 14:47:33+02	2020-06-14 14:48:07+02	www	py-priority	py37	py37-priority-1.3.0
2020-06-14 14:47:37+02	2020-06-14 14:48:20+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-06-14 14:22:16+02	2020-06-14 14:53:22+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-14 14:48:21+02	2020-06-14 14:54:00+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-14 14:53:24+02	2020-06-14 14:59:45+02	x11	xorg	\N	xorg-7.7_3
2020-06-14 14:54:03+02	2020-06-14 15:00:41+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-14 15:00:44+02	2020-06-14 15:03:55+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-14 14:59:50+02	2020-06-14 15:04:51+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-19 13:29:45+02	2020-06-19 13:45:11+02	math	R	\N	R-4.0.1
2020-06-14 15:03:56+02	2020-06-14 15:12:45+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-14 15:12:49+02	2020-06-14 15:21:15+02	x11-wm	marco	\N	marco-1.22.4
2020-06-14 15:09:18+02	2020-06-14 15:28:38+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-14 15:21:23+02	2020-06-14 15:29:16+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-14 14:48:10+02	2020-06-14 15:31:37+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-14 15:28:48+02	2020-06-14 15:33:03+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-14 15:33:04+02	2020-06-14 15:33:53+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-06-14 15:33:56+02	2020-06-14 15:34:42+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-06-14 15:34:43+02	2020-06-14 15:35:07+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-06-14 15:35:08+02	2020-06-14 15:35:43+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-06-14 15:35:48+02	2020-06-14 15:36:31+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-06-14 15:36:32+02	2020-06-14 15:36:57+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-06-14 15:36:58+02	2020-06-14 15:40:24+02	devel	py-twisted	py37	py37-twisted-20.3.0
2020-06-14 15:29:20+02	2020-06-14 15:40:50+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-14 15:31:40+02	2020-06-14 15:41:44+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-14 15:40:54+02	2020-06-14 15:44:24+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-14 15:44:27+02	2020-06-14 15:45:37+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-14 15:41:48+02	2020-06-14 15:46:12+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-14 15:46:18+02	2020-06-14 15:48:10+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-14 15:45:40+02	2020-06-14 15:50:41+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-14 15:48:20+02	2020-06-14 15:51:08+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-14 15:51:13+02	2020-06-14 15:54:57+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-14 13:57:14+02	2020-06-14 15:55:15+02	devel	llvm90	\N	llvm90-9.0.1_1
2020-06-14 15:55:03+02	2020-06-14 15:57:42+02	graphics	vigra	\N	vigra-1.11.1_10
2020-06-14 15:50:42+02	2020-06-14 16:01:00+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-14 15:57:46+02	2020-06-14 16:08:40+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-06-14 15:40:26+02	2020-06-14 16:10:49+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-14 16:08:43+02	2020-06-14 16:13:34+02	multimedia	libass	\N	libass-0.14.0
2020-06-14 16:10:53+02	2020-06-14 16:15:46+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-14 16:13:36+02	2020-06-14 16:17:30+02	net	freerdp	\N	freerdp-2.1.1
2020-06-14 16:15:48+02	2020-06-14 16:17:50+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-14 16:17:35+02	2020-06-14 16:18:25+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-06-14 16:17:53+02	2020-06-14 16:18:30+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-06-14 16:18:32+02	2020-06-14 16:21:08+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-14 16:18:30+02	2020-06-14 16:21:09+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-14 16:01:08+02	2020-06-14 16:21:38+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-14 16:21:11+02	2020-06-14 16:24:27+02	print	libspectre	\N	libspectre-0.2.8
2020-06-14 16:21:41+02	2020-06-14 16:29:44+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-14 16:21:09+02	2020-06-14 16:43:54+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-14 16:44:00+02	2020-06-14 16:46:36+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-06-14 16:46:42+02	2020-06-14 16:49:14+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-06-14 15:55:18+02	2020-06-14 16:50:18+02	math	R	\N	R-4.0.1
2020-06-14 16:49:19+02	2020-06-14 16:50:24+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-06-14 16:50:27+02	2020-06-14 16:52:09+02	sysutils	hal-info	\N	hal-info-20091130
2020-06-14 16:24:30+02	2020-06-14 16:53:02+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-14 16:29:50+02	2020-06-14 16:54:54+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-14 16:52:13+02	2020-06-14 16:55:22+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-14 16:55:25+02	2020-06-14 16:56:06+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-06-14 16:56:09+02	2020-06-14 16:56:48+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-06-14 16:56:56+02	2020-06-14 16:57:38+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-06-14 16:54:57+02	2020-06-14 16:58:02+02	textproc	enchant	\N	enchant-1.6.0_9
2020-06-14 16:53:06+02	2020-06-14 16:58:23+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-14 16:57:41+02	2020-06-14 16:58:34+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-06-14 16:58:35+02	2020-06-14 16:59:08+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-06-14 16:58:05+02	2020-06-14 17:00:40+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-14 16:59:11+02	2020-06-14 17:01:23+02	www	youtube_dl	\N	youtube_dl-2020.06.06
2020-06-14 16:58:26+02	2020-06-14 17:02:28+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-14 16:50:27+02	2020-06-14 17:03:21+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-14 17:01:27+02	2020-06-14 17:03:56+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-06-14 17:03:26+02	2020-06-14 17:05:23+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-14 17:05:28+02	2020-06-14 17:06:55+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-06-14 17:04:00+02	2020-06-14 17:06:55+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-14 17:06:57+02	2020-06-14 17:07:30+02	databases	py-apsw	py37	py37-apsw-3.31.1
2020-06-14 17:02:29+02	2020-06-14 17:07:42+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-14 17:00:42+02	2020-06-14 17:11:53+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-06-14 17:06:57+02	2020-06-14 17:12:04+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-14 17:07:32+02	2020-06-14 17:13:18+02	databases	rrdtool	\N	rrdtool-1.7.2_2
2020-06-14 17:12:05+02	2020-06-14 17:15:09+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-14 17:11:55+02	2020-06-14 17:15:47+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-14 17:15:13+02	2020-06-14 17:15:57+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-06-14 17:13:27+02	2020-06-14 17:16:02+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.70.0
2020-06-14 17:15:49+02	2020-06-14 17:16:46+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-06-14 17:16:06+02	2020-06-14 17:16:48+02	devel	py-funcy	py37	py37-funcy-1.12
2020-06-14 17:16:52+02	2020-06-14 17:17:09+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-06-14 17:17:14+02	2020-06-14 17:18:04+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-06-14 17:16:01+02	2020-06-14 17:18:14+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-06-14 17:18:07+02	2020-06-14 17:18:33+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-06-14 17:18:16+02	2020-06-14 17:18:41+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-06-14 17:18:35+02	2020-06-14 17:19:08+02	devel	py-rq	py37	py37-rq-1.3.0
2020-06-14 17:16:48+02	2020-06-14 17:19:14+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-14 17:18:47+02	2020-06-14 17:19:24+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.87
2020-06-14 17:19:09+02	2020-06-14 17:19:34+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-06-14 17:19:19+02	2020-06-14 17:19:36+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-06-14 17:19:25+02	2020-06-14 17:19:54+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-06-14 17:19:35+02	2020-06-14 17:20:07+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-06-14 17:07:43+02	2020-06-14 17:20:39+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-14 17:20:41+02	2020-06-14 17:27:00+02	graphics	eom	\N	eom-1.24.0
2020-06-14 17:20:08+02	2020-06-14 17:31:10+02	editors	pluma	\N	pluma-1.24.0
2020-06-14 17:27:02+02	2020-06-14 17:31:12+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-14 17:31:11+02	2020-06-14 17:33:27+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-14 17:31:13+02	2020-06-14 17:39:44+02	graphics	py-gimp	\N	py27-gimp-2.10.18_7
2020-06-14 12:32:12+02	2020-06-14 17:51:43+02	lang	rust	\N	rust-1.44.0
2020-06-14 17:51:51+02	2020-06-14 18:03:19+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.2_1
2020-06-14 18:03:21+02	2020-06-14 18:03:38+02	lang	python3	\N	python3-3_3
2020-06-14 17:33:32+02	2020-06-14 18:05:14+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-14 18:05:16+02	2020-06-14 18:07:39+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-14 18:03:42+02	2020-06-14 18:07:51+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-14 18:07:40+02	2020-06-14 18:09:11+02	math	stp	\N	stp-2.3.3_5,1
2020-06-14 18:09:13+02	2020-06-14 18:12:54+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-14 17:19:39+02	2020-06-14 18:14:30+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-14 17:19:56+02	2020-06-14 18:18:16+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-14 18:07:54+02	2020-06-14 18:18:29+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-06-14 18:12:55+02	2020-06-14 18:18:32+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-14 18:18:30+02	2020-06-14 18:18:57+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-06-14 18:18:33+02	2020-06-14 18:19:02+02	net	py-pyzmq	py37	py37-pyzmq-19.0.1
2020-06-14 18:18:19+02	2020-06-14 18:20:33+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-14 18:18:58+02	2020-06-14 18:20:41+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-14 18:19:04+02	2020-06-14 18:20:44+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-14 18:20:35+02	2020-06-14 18:22:29+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-14 18:20:45+02	2020-06-14 18:22:29+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-14 18:20:42+02	2020-06-14 18:22:41+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-14 18:22:30+02	2020-06-14 18:24:17+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-14 18:22:30+02	2020-06-14 18:24:49+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-14 18:22:42+02	2020-06-14 18:25:41+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-14 18:24:18+02	2020-06-14 18:26:20+02	net	vde2	\N	vde2-2.3.2_4
2020-06-14 18:26:21+02	2020-06-14 18:26:48+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.7.1
2020-06-14 18:24:50+02	2020-06-14 18:26:50+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-14 18:25:43+02	2020-06-14 18:28:52+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-06-14 18:26:49+02	2020-06-14 18:29:50+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-14 18:26:51+02	2020-06-14 18:38:08+02	print	lilypond	\N	lilypond-2.20.0
2020-06-14 18:14:34+02	2020-06-14 18:41:02+02	multimedia	vlc	\N	vlc-3.0.10_7,4
2020-06-14 18:29:51+02	2020-06-14 18:41:40+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-14 18:28:55+02	2020-06-14 18:42:20+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-14 18:38:10+02	2020-06-14 18:43:15+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-14 18:42:22+02	2020-06-14 18:43:48+02	print	transfig	\N	transfig-3.2.5e_4
2020-06-14 18:43:18+02	2020-06-14 18:43:50+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-06-14 18:43:49+02	2020-06-14 18:44:27+02	security	py-stem	py37	py37-stem-1.8.0
2020-06-14 18:44:29+02	2020-06-14 18:51:28+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-14 18:43:53+02	2020-06-14 18:51:37+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.0
2020-06-14 18:51:41+02	2020-06-14 18:53:39+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-14 18:53:40+02	2020-06-14 18:54:09+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-06-14 18:54:09+02	2020-06-14 18:54:37+02	textproc	py-scour	py37	py37-scour-0.37
2020-06-14 18:51:29+02	2020-06-14 18:54:57+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-14 18:54:39+02	2020-06-14 18:55:25+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-06-14 18:54:59+02	2020-06-14 18:56:15+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-06-14 18:55:28+02	2020-06-14 18:56:56+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.0
2020-06-14 18:56:20+02	2020-06-14 18:57:04+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-06-14 18:57:10+02	2020-06-14 18:58:02+02	www	py-dj22-django-js-asset	py37	py37-dj22-django-js-asset-1.2.2
2020-06-14 18:56:58+02	2020-06-14 18:58:03+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-06-14 18:58:05+02	2020-06-14 18:58:47+02	www	py-dj22-django-taggit	py37	py37-dj22-django-taggit-1.3.0
2020-06-14 18:58:05+02	2020-06-14 18:58:57+02	www	py-dj22-djangorestframework	py37	py37-dj22-djangorestframework-3.11.0
2020-06-14 18:58:51+02	2020-06-14 18:59:35+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-06-14 18:58:59+02	2020-06-14 19:02:16+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-14 18:59:37+02	2020-06-14 19:03:01+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-14 19:02:21+02	2020-06-14 19:03:20+02	www	qt5-websockets	\N	qt5-websockets-5.14.2
2020-06-14 18:41:05+02	2020-06-14 19:03:57+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-14 19:03:21+02	2020-06-14 19:06:23+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-14 19:04:00+02	2020-06-14 19:09:53+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-14 19:06:26+02	2020-06-14 19:10:40+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-14 19:09:55+02	2020-06-14 19:11:47+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-14 19:10:45+02	2020-06-14 19:15:09+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-14 19:03:02+02	2020-06-14 19:21:25+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-14 19:15:13+02	2020-06-14 19:23:58+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-14 19:24:01+02	2020-06-14 19:24:30+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-06-14 19:11:47+02	2020-06-14 19:24:33+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-14 19:24:35+02	2020-06-14 19:27:40+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-14 18:41:42+02	2020-06-14 19:29:50+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-14 19:21:29+02	2020-06-14 19:30:25+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-14 17:39:48+02	2020-06-14 19:31:28+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-14 19:24:31+02	2020-06-14 19:32:17+02	deskutils	calibre	\N	calibre-4.18.0
2020-06-14 19:31:30+02	2020-06-14 19:35:10+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-14 19:35:11+02	2020-06-14 19:35:55+02	devel	py-dj22-django-rq	py37	py37-dj22-django-rq-2.3.2
2020-06-14 19:35:58+02	2020-06-14 19:37:42+02	devel	tig	\N	tig-2.5.1
2020-06-14 19:37:46+02	2020-06-14 19:49:48+02	dns	bind916	\N	bind916-9.16.3_2
2020-06-14 19:27:44+02	2020-06-14 20:52:50+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-14 19:30:27+02	2020-06-14 20:56:14+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200519
2020-06-14 14:44:38+02	2020-06-14 20:58:34+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-14 19:29:54+02	2020-06-14 21:02:31+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-14 21:18:19+02	2020-06-14 21:19:32+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-14 21:19:32+02	2020-06-14 21:22:34+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-14 21:22:36+02	2020-06-14 21:29:15+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-14 21:19:25+02	2020-06-14 21:31:50+02	editors	codelite	\N	codelite-12.0_4
2020-06-14 21:29:18+02	2020-06-14 21:32:11+02	editors	vim	\N	vim-8.2.0869
2020-06-14 21:18:19+02	2020-06-14 21:47:07+02	devel	llvm10	\N	llvm10-10.0.0_1
2020-06-14 21:31:50+02	2020-06-14 21:48:50+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-14 21:48:53+02	2020-06-14 22:12:39+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-14 22:12:39+02	2020-06-14 22:13:14+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-14 22:13:14+02	2020-06-14 22:13:43+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-14 21:47:10+02	2020-06-14 22:14:01+02	emulators	wine	\N	wine-5.0.1,1
2020-06-14 22:13:43+02	2020-06-14 22:14:13+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-14 22:14:02+02	2020-06-14 22:14:43+02	games	xboard	\N	xboard-4.9.1_1
2020-06-14 22:14:13+02	2020-06-14 22:14:52+02	games	xcowsay	\N	xcowsay-1.5
2020-06-14 22:14:43+02	2020-06-14 22:16:56+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-14 22:14:53+02	2020-06-14 22:17:08+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-14 22:16:56+02	2020-06-14 22:17:54+02	graphics	gimp	\N	gimp-2.10.18,2
2020-06-14 21:32:12+02	2020-06-14 22:22:15+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-06-14 22:22:16+02	2020-06-14 22:22:35+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-06-14 22:17:08+02	2020-06-14 22:23:26+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-14 22:22:36+02	2020-06-14 22:24:42+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-14 22:23:28+02	2020-06-14 22:26:30+02	irc	irssi	\N	irssi-1.2.2,1
2020-06-14 22:24:44+02	2020-06-14 22:26:51+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-14 22:17:54+02	2020-06-14 22:29:10+02	graphics	krita	\N	krita-4.2.9_4
2020-06-14 22:26:31+02	2020-06-14 22:56:28+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-14 22:56:29+02	2020-06-14 22:56:43+02	lang	python	\N	python-3.7_3,2
2020-06-14 22:56:43+02	2020-06-14 22:57:41+02	misc	mc	\N	mc-nox11-4.8.24
2020-06-14 22:57:41+02	2020-06-14 22:58:38+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-14 22:58:39+02	2020-06-14 23:00:42+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-14 22:29:11+02	2020-06-14 23:02:23+02	lang	mono	\N	mono-5.10.1.57_3
2020-06-14 22:26:51+02	2020-06-14 23:02:39+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-14 23:00:42+02	2020-06-14 23:02:40+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-14 23:02:40+02	2020-06-14 23:04:20+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-14 23:02:40+02	2020-06-14 23:04:29+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-14 23:02:24+02	2020-06-14 23:04:51+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-14 23:04:29+02	2020-06-14 23:05:11+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-14 23:04:52+02	2020-06-14 23:06:25+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-06-14 23:05:12+02	2020-06-14 23:06:37+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-14 23:06:26+02	2020-06-14 23:07:12+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-14 23:06:38+02	2020-06-14 23:07:19+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-14 23:07:12+02	2020-06-14 23:07:53+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-14 23:04:21+02	2020-06-14 23:07:57+02	net	wireshark	\N	wireshark-3.2.4
2020-06-14 23:07:20+02	2020-06-14 23:08:03+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-14 23:07:54+02	2020-06-14 23:08:37+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-14 23:07:58+02	2020-06-14 23:08:43+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-14 23:08:04+02	2020-06-14 23:09:08+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-14 23:08:38+02	2020-06-14 23:09:24+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-14 23:08:43+02	2020-06-14 23:09:29+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-14 23:09:31+02	2020-06-14 23:09:53+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-14 23:09:08+02	2020-06-14 23:09:55+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-14 23:09:53+02	2020-06-14 23:10:23+02	print	gv	\N	gv-3.7.4_5
2020-06-14 23:09:25+02	2020-06-14 23:11:50+02	print	auctex	full	auctex-emacs26-12.2
2020-06-14 23:09:55+02	2020-06-14 23:12:11+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-14 23:12:12+02	2020-06-14 23:12:22+02	security	nyx	\N	nyx-2.1.0_1
2020-06-14 23:11:51+02	2020-06-14 23:12:52+02	security	klee	\N	klee-2.1
2020-06-14 23:12:22+02	2020-06-14 23:13:06+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-06-14 23:12:52+02	2020-06-14 23:13:23+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.0
2020-06-14 23:13:24+02	2020-06-14 23:14:12+02	sysutils	cpu-x	\N	cpu-x-4.0.0_1
2020-06-14 23:10:23+02	2020-06-14 23:14:18+02	print	texlive-full	\N	texlive-full-20150521
2020-06-14 23:14:12+02	2020-06-14 23:14:33+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-06-14 23:14:34+02	2020-06-14 23:14:48+02	sysutils	panicmail	\N	panicmail-1.1
2020-06-14 23:14:19+02	2020-06-14 23:14:50+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-14 23:13:07+02	2020-06-14 23:14:56+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-06-14 23:14:49+02	2020-06-14 23:15:06+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-06-14 23:15:06+02	2020-06-14 23:15:18+02	www	py-dj22-django-cacheops	py37	py37-dj22-django-cacheops-4.2
2020-06-14 23:14:57+02	2020-06-14 23:15:21+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-06-14 23:15:19+02	2020-06-14 23:15:29+02	www	py-dj22-django-cors-headers	py37	py37-dj22-django-cors-headers-3.2.1
2020-06-14 23:15:22+02	2020-06-14 23:15:33+02	www	py-dj22-django-debug-toolbar	py37	py37-dj22-django-debug-toolbar-2.1
2020-06-14 23:15:30+02	2020-06-14 23:15:41+02	www	py-dj22-django-filter	py37	py37-dj22-django-filter-2.2.0
2020-06-14 23:15:33+02	2020-06-14 23:15:46+02	www	py-dj22-django-mptt	py37	py37-dj22-django-mptt-0.9.1
2020-06-14 23:15:41+02	2020-06-14 23:15:55+02	www	py-dj22-django-prometheus	py37	py37-dj22-django-prometheus-1.1.0_1
2020-06-14 23:15:46+02	2020-06-14 23:15:58+02	www	py-dj22-django-tables2	py37	py37-dj22-django-tables2-2.2.1
2020-06-14 23:15:55+02	2020-06-14 23:16:07+02	www	py-dj22-django-taggit-serializer	py37	py37-dj22-django-taggit-serializer-0.1.7
2020-06-14 23:15:58+02	2020-06-14 23:16:10+02	www	py-dj22-django-timezone-field	py37	py37-dj22-django-timezone-field-4.0
2020-06-14 23:16:10+02	2020-06-14 23:16:20+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-06-14 23:16:08+02	2020-06-14 23:16:23+02	www	py-dj22-drf-yasg	py37	py37-dj22-drf-yasg-1.17.0
2020-06-14 23:16:21+02	2020-06-14 23:16:31+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-06-14 23:16:24+02	2020-06-14 23:17:04+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-14 23:16:32+02	2020-06-14 23:18:05+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-14 23:18:06+02	2020-06-14 23:18:58+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-14 23:18:59+02	2020-06-14 23:19:33+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-14 23:17:05+02	2020-06-14 23:24:15+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-14 23:24:18+02	2020-06-14 23:24:39+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-14 23:24:41+02	2020-06-14 23:27:20+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-14 23:27:22+02	2020-06-14 23:29:33+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-06-14 23:19:35+02	2020-06-15 00:43:45+02	editors	vscode	\N	vscode-1.45.1_2
2020-06-14 23:14:51+02	2020-06-15 00:57:29+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-15 07:13:09+02	2020-06-15 07:13:29+02	security	libgpg-error	\N	libgpg-error-1.38
2020-06-15 07:13:09+02	2020-06-15 07:13:48+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-06-15 07:13:30+02	2020-06-15 07:14:51+02	security	libgcrypt	\N	libgcrypt-1.8.5
2020-06-15 07:13:09+02	2020-06-15 07:14:52+02	devel	bison	\N	bison-3.6.3,1
2020-06-15 07:14:52+02	2020-06-15 07:15:23+02	textproc	libxslt	\N	libxslt-1.1.34_1
2020-06-15 07:15:24+02	2020-06-15 07:15:37+02	textproc	minixmlto	\N	minixmlto-0.0.2_1
2020-06-15 07:15:38+02	2020-06-15 07:17:03+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-06-15 07:17:04+02	2020-06-15 07:18:23+02	devel	libsoup	\N	libsoup-2.62.3
2020-06-15 07:18:24+02	2020-06-15 07:18:45+02	multimedia	libaacs	\N	libaacs-0.9.0
2020-06-15 07:14:52+02	2020-06-15 07:18:50+02	lang	php73	\N	php73-7.3.19
2020-06-15 07:18:45+02	2020-06-15 07:19:16+02	multimedia	libbdplus	\N	libbdplus-0.1.2
2020-06-15 07:19:16+02	2020-06-15 07:19:37+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-06-15 07:18:50+02	2020-06-15 07:19:43+02	security	libsecret	\N	libsecret-0.18.6_1
2020-06-15 07:13:50+02	2020-06-15 07:19:44+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-15 07:19:44+02	2020-06-15 07:20:10+02	www	php73-session	\N	php73-session-7.3.19
2020-06-15 07:19:43+02	2020-06-15 07:20:12+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-06-15 07:20:10+02	2020-06-15 07:20:30+02	archivers	php73-zlib	\N	php73-zlib-7.3.19
2020-06-15 07:20:13+02	2020-06-15 07:20:35+02	databases	php73-pdo	\N	php73-pdo-7.3.19
2020-06-15 07:20:36+02	2020-06-15 07:20:47+02	security	libassuan	\N	libassuan-2.5.3
2020-06-15 07:20:31+02	2020-06-15 07:20:52+02	devel	php73-json	\N	php73-json-7.3.19
2020-06-15 07:20:48+02	2020-06-15 07:21:22+02	converters	php73-mbstring	\N	php73-mbstring-7.3.19
2020-06-15 07:20:53+02	2020-06-15 07:21:23+02	graphics	php73-gd	\N	php73-gd-7.3.19
2020-06-15 07:21:22+02	2020-06-15 07:21:50+02	security	php73-hash	\N	php73-hash-7.3.19
2020-06-15 07:21:23+02	2020-06-15 07:21:50+02	security	php73-openssl	\N	php73-openssl-7.3.19
2020-06-15 07:21:50+02	2020-06-15 07:22:09+02	textproc	php73-xml	\N	php73-xml-7.3.19
2020-06-15 07:21:50+02	2020-06-15 07:22:12+02	converters	php73-iconv	\N	php73-iconv-7.3.19
2020-06-15 07:22:10+02	2020-06-15 07:22:54+02	databases	pecl-memcache	php73	php73-pecl-memcache-4.0.5.2
2020-06-15 07:22:12+02	2020-06-15 07:23:26+02	databases	pecl-memcached	php73	php73-pecl-memcached-3.1.5
2020-06-15 07:22:56+02	2020-06-15 07:24:20+02	databases	php73-pdo_pgsql	\N	php73-pdo_pgsql-7.3.19
2020-06-15 07:23:29+02	2020-06-15 07:24:50+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.19
2020-06-15 07:24:21+02	2020-06-15 07:26:11+02	databases	php73-pgsql	\N	php73-pgsql-7.3.19
2020-06-15 07:24:53+02	2020-06-15 07:26:30+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.19
2020-06-15 07:26:15+02	2020-06-15 07:27:42+02	devel	php73-gettext	\N	php73-gettext-7.3.19
2020-06-15 07:26:34+02	2020-06-15 07:28:24+02	ftp	php73-curl	\N	php73-curl-7.3.19
2020-06-15 07:28:27+02	2020-06-15 07:29:57+02	math	pecl-bitset	php73	php73-pecl-bitset-3.0.1
2020-06-15 07:27:43+02	2020-06-15 07:30:06+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-15 07:29:59+02	2020-06-15 07:30:24+02	math	php73-bcmath	\N	php73-bcmath-7.3.19
2020-06-15 07:30:07+02	2020-06-15 07:31:00+02	net	geoclue	\N	geoclue-2.5.5
2020-06-15 07:31:00+02	2020-06-15 07:31:22+02	net	php73-soap	\N	php73-soap-7.3.19
2020-06-15 07:31:23+02	2020-06-15 07:31:42+02	net	php73-sockets	\N	php73-sockets-7.3.19
2020-06-15 07:19:38+02	2020-06-15 07:31:54+02	multimedia	ffmpeg	\N	ffmpeg-4.2.3,1
2020-06-15 07:31:42+02	2020-06-15 07:32:00+02	security	pecl-mcrypt	php73	php73-pecl-mcrypt-1.0.3
2020-06-15 07:31:55+02	2020-06-15 07:32:12+02	sysutils	php73-posix	\N	php73-posix-7.3.19
2020-06-15 07:32:00+02	2020-06-15 07:32:17+02	textproc	php73-ctype	\N	php73-ctype-7.3.19
2020-06-15 07:30:25+02	2020-06-15 07:32:26+02	net	php73-ldap	\N	php73-ldap-7.3.19
2020-06-15 07:32:13+02	2020-06-15 07:32:35+02	textproc	php73-dom	\N	php73-dom-7.3.19
2020-06-15 07:32:18+02	2020-06-15 07:32:40+02	textproc	php73-simplexml	\N	php73-simplexml-7.3.19
2020-06-15 07:32:27+02	2020-06-15 07:33:02+02	www	php73-opcache	\N	php73-opcache-7.3.19
2020-06-15 07:32:41+02	2020-06-15 07:33:34+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-15 07:32:35+02	2020-06-15 07:33:42+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-15 07:33:02+02	2020-06-15 07:34:00+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-15 07:34:01+02	2020-06-15 07:35:02+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-15 07:33:43+02	2020-06-15 07:35:02+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-15 07:35:06+02	2020-06-15 07:35:50+02	textproc	xmlto	\N	xmlto-0.0.28
2020-06-15 07:35:03+02	2020-06-15 07:36:14+02	sysutils	hal	\N	hal-0.5.14_34
2020-06-15 07:36:14+02	2020-06-15 07:37:24+02	textproc	asciidoc	\N	asciidoc-9.0.0rc2
2020-06-15 07:35:52+02	2020-06-15 07:38:29+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-06-15 07:37:25+02	2020-06-15 07:38:52+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-06-15 07:38:30+02	2020-06-15 07:39:23+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-15 07:38:53+02	2020-06-15 07:40:03+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-15 07:39:24+02	2020-06-15 07:40:26+02	net	remmina	\N	remmina-1.3.9
2020-06-15 07:40:27+02	2020-06-15 07:40:36+02	security	pinentry-tty	\N	pinentry-tty-1.1.0
2020-06-15 07:33:35+02	2020-06-15 07:40:53+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-15 07:40:36+02	2020-06-15 07:41:08+02	security	libksba	\N	libksba-1.4.0
2020-06-15 07:40:54+02	2020-06-15 07:41:08+02	security	pinentry	\N	pinentry-1.1.0_6
2020-06-15 07:40:04+02	2020-06-15 07:41:39+02	security	gcr	\N	gcr-3.28.0
2020-06-15 07:41:11+02	2020-06-15 07:42:17+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-15 07:41:11+02	2020-06-15 07:42:17+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-15 07:41:40+02	2020-06-15 07:42:18+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-15 07:42:18+02	2020-06-15 07:42:37+02	textproc	yelp-xsl	\N	yelp-xsl-3.28.0
2020-06-15 07:42:37+02	2020-06-15 07:43:27+02	devel	py-lxml	py37	py37-lxml-4.5.1
2020-06-15 07:42:18+02	2020-06-15 07:43:39+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-15 07:42:17+02	2020-06-15 07:43:53+02	security	gnupg	\N	gnupg-2.2.20
2020-06-15 07:43:28+02	2020-06-15 07:44:03+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-15 07:43:53+02	2020-06-15 07:44:15+02	security	gpgme	\N	gpgme-1.13.1
2020-06-15 07:44:03+02	2020-06-15 07:44:53+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-06-15 07:44:16+02	2020-06-15 07:45:59+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-06-15 07:44:56+02	2020-06-15 07:46:12+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-06-15 07:46:03+02	2020-06-15 07:46:39+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-06-15 07:46:39+02	2020-06-15 07:47:30+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-15 07:43:39+02	2020-06-15 07:47:40+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-15 07:47:30+02	2020-06-15 07:49:39+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-15 07:49:40+02	2020-06-15 07:51:06+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-06-15 07:47:40+02	2020-06-15 07:51:20+02	x11	mate-panel	\N	mate-panel-1.22.2
2020-06-15 07:51:20+02	2020-06-15 07:51:46+02	devel	librest	\N	rest-0.7.93
2020-06-15 07:46:12+02	2020-06-15 07:51:49+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-15 07:51:07+02	2020-06-15 07:52:05+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-15 07:51:50+02	2020-06-15 07:52:41+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-06-15 07:51:47+02	2020-06-15 07:52:53+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-06-15 07:52:06+02	2020-06-15 07:53:10+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-15 07:52:43+02	2020-06-15 07:53:27+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-15 07:52:53+02	2020-06-15 07:53:49+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-15 07:53:10+02	2020-06-15 07:53:58+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-15 07:53:27+02	2020-06-15 07:54:13+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-15 07:53:50+02	2020-06-15 07:54:40+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-15 07:53:58+02	2020-06-15 07:54:46+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-15 07:54:14+02	2020-06-15 07:55:03+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-15 07:54:47+02	2020-06-15 07:55:33+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-15 07:55:03+02	2020-06-15 07:55:50+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-15 07:54:40+02	2020-06-15 07:55:52+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-15 07:55:52+02	2020-06-15 07:56:41+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-15 07:55:51+02	2020-06-15 07:56:55+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-15 07:56:37+02	2020-06-15 07:57:51+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-15 07:56:55+02	2020-06-15 07:57:55+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-15 07:57:52+02	2020-06-15 07:58:40+02	net	uhttpmock	\N	uhttpmock-0.5.1_1
2020-06-28 14:46:36+02	2020-06-28 14:46:57+02	net	gssdp	\N	gssdp-1.2.3
2020-06-15 07:58:41+02	2020-06-15 07:59:28+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-06-15 07:57:56+02	2020-06-15 07:59:28+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-15 07:59:29+02	2020-06-15 08:00:07+02	textproc	yelp-tools	\N	yelp-tools-3.28.0
2020-06-15 08:00:07+02	2020-06-15 08:00:38+02	www	p5-CGI	\N	p5-CGI-4.48
2020-06-15 08:00:39+02	2020-06-15 08:02:21+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-15 07:59:29+02	2020-06-15 08:02:22+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-15 08:02:23+02	2020-06-15 08:03:07+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-15 08:02:21+02	2020-06-15 08:03:26+02	x11	zenity	\N	zenity-3.28.1
2020-06-15 08:03:09+02	2020-06-15 08:06:19+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.22.2
2020-06-15 08:03:27+02	2020-06-15 08:06:27+02	devel	git	\N	git-2.27.0
2020-06-15 07:56:42+02	2020-06-15 08:06:32+02	net	samba410	\N	samba410-4.10.15
2020-06-15 08:06:33+02	2020-06-15 08:07:53+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-15 08:06:28+02	2020-06-15 08:08:28+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-15 08:06:19+02	2020-06-15 08:09:06+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-15 08:08:29+02	2020-06-15 08:10:42+02	graphics	movit	\N	movit-1.6.2_2
2020-06-15 08:10:43+02	2020-06-15 08:11:10+02	net	gssdp	\N	gssdp-1.2.2
2020-06-15 08:11:11+02	2020-06-15 08:11:41+02	net	libvncserver	\N	libvncserver-0.9.12_2
2020-06-15 08:11:42+02	2020-06-15 08:12:33+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-15 08:07:55+02	2020-06-15 08:13:08+02	editors	emacs	full	emacs-26.3_4,3
2020-06-15 08:09:06+02	2020-06-15 08:13:41+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-15 08:13:08+02	2020-06-15 08:14:00+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-15 08:14:01+02	2020-06-15 08:14:42+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-06-15 08:12:33+02	2020-06-15 08:15:10+02	print	fontforge	\N	fontforge-20190801_2
2020-06-15 08:13:42+02	2020-06-15 08:15:26+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-15 08:15:27+02	2020-06-15 08:16:22+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-15 08:16:22+02	2020-06-15 08:20:35+02	x11	xorg	\N	xorg-7.7_3
2020-06-15 08:20:42+02	2020-06-15 08:25:31+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-15 08:25:33+02	2020-06-15 08:30:11+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.8_3
2020-06-15 08:15:10+02	2020-06-15 08:34:41+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-15 08:30:13+02	2020-06-15 08:39:11+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-15 08:34:44+02	2020-06-15 08:39:20+02	x11-wm	marco	\N	marco-1.22.4
2020-06-15 08:39:24+02	2020-06-15 08:41:10+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-15 08:39:15+02	2020-06-15 08:42:24+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-15 08:41:11+02	2020-06-15 08:44:29+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-15 08:42:25+02	2020-06-15 08:44:37+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-15 08:44:38+02	2020-06-15 08:44:59+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-06-15 08:44:29+02	2020-06-15 08:47:02+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-15 08:47:06+02	2020-06-15 08:49:23+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-15 08:49:25+02	2020-06-15 08:49:48+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-15 08:45:00+02	2020-06-15 08:50:04+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-15 08:14:43+02	2020-06-15 08:50:05+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-15 08:49:49+02	2020-06-15 08:50:33+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-15 08:50:06+02	2020-06-15 08:50:52+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-15 08:50:06+02	2020-06-15 08:51:19+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-15 08:50:53+02	2020-06-15 08:51:21+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-06-15 08:51:21+02	2020-06-15 08:51:42+02	sysutils	hal-info	\N	hal-info-20091130
2020-06-15 08:51:43+02	2020-06-15 08:52:29+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-15 08:50:33+02	2020-06-15 08:52:41+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-15 08:52:41+02	2020-06-15 08:53:47+02	textproc	redland	\N	redland-1.0.17_4
2020-06-15 08:51:22+02	2020-06-15 08:54:00+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-15 08:54:00+02	2020-06-15 08:54:29+02	www	youtube_dl	\N	youtube_dl-2020.06.06
2020-06-15 08:52:29+02	2020-06-15 08:54:35+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-15 08:53:49+02	2020-06-15 08:54:46+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-15 08:54:36+02	2020-06-15 08:55:08+02	audio	faudio	\N	FAudio-20.06
2020-06-15 08:55:08+02	2020-06-15 08:55:38+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.12.1
2020-06-15 08:54:47+02	2020-06-15 08:55:54+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-15 08:54:30+02	2020-06-15 08:55:56+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-06-15 08:55:38+02	2020-06-15 08:56:29+02	databases	rrdtool	\N	rrdtool-1.7.2_3
2020-06-15 08:55:57+02	2020-06-15 08:56:40+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-15 08:56:29+02	2020-06-15 08:56:52+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-06-15 08:55:55+02	2020-06-15 08:58:10+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-15 08:58:11+02	2020-06-15 09:00:19+02	graphics	eom	\N	eom-1.24.0
2020-06-15 08:56:54+02	2020-06-15 09:00:28+02	editors	pluma	\N	pluma-1.24.0
2020-06-15 09:00:29+02	2020-06-15 09:01:12+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-15 09:00:19+02	2020-06-15 09:01:32+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-15 09:01:33+02	2020-06-15 09:01:53+02	multimedia	libmtp	\N	libmtp-1.1.17
2020-06-15 09:01:12+02	2020-06-15 09:02:33+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-15 09:02:34+02	2020-06-15 09:07:08+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-15 09:01:53+02	2020-06-15 09:07:38+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-15 09:07:39+02	2020-06-15 09:10:08+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-15 09:10:09+02	2020-06-15 09:11:26+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-15 09:11:27+02	2020-06-15 09:12:20+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-15 09:12:21+02	2020-06-15 09:13:18+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-15 09:07:10+02	2020-06-15 09:14:06+02	multimedia	vlc	\N	vlc-3.0.10_7,4
2020-06-15 09:13:18+02	2020-06-15 09:14:09+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-15 09:14:06+02	2020-06-15 09:14:44+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-15 09:14:09+02	2020-06-15 09:14:48+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-15 09:14:44+02	2020-06-15 09:15:37+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-15 09:14:48+02	2020-06-15 09:16:00+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-15 09:15:38+02	2020-06-15 09:16:26+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-15 09:16:01+02	2020-06-15 09:17:08+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-15 08:56:41+02	2020-06-15 09:18:18+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-15 09:17:08+02	2020-06-15 09:18:35+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.0
2020-06-15 09:18:36+02	2020-06-15 09:20:19+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-15 09:18:19+02	2020-06-15 09:20:25+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-15 09:20:25+02	2020-06-15 09:20:54+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.0
2020-06-15 09:20:20+02	2020-06-15 09:20:56+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-06-15 09:16:26+02	2020-06-15 09:21:40+02	print	lilypond	\N	lilypond-2.20.0
2020-06-15 09:20:56+02	2020-06-15 09:21:57+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-15 09:21:00+02	2020-06-15 09:22:07+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-15 09:21:58+02	2020-06-15 09:23:31+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-15 09:22:08+02	2020-06-15 09:24:46+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-15 09:23:33+02	2020-06-15 09:25:15+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-15 09:21:41+02	2020-06-15 09:26:36+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-15 09:26:37+02	2020-06-15 09:27:22+02	databases	pgtop	\N	pgtop-0.05_5
2020-06-15 09:25:15+02	2020-06-15 09:28:51+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-15 09:28:52+02	2020-06-15 09:29:40+02	deskutils	sysctlview	\N	sysctlview-2.0
2020-06-15 09:24:48+02	2020-06-15 09:29:41+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-15 09:27:25+02	2020-06-15 09:30:25+02	deskutils	calibre	\N	calibre-4.18.0
2020-06-15 09:29:42+02	2020-06-15 09:30:50+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-15 09:30:50+02	2020-06-15 09:31:13+02	devel	tig	\N	tig-2.5.1
2020-06-15 09:31:13+02	2020-06-15 09:41:35+02	editors	codelite	\N	codelite-12.0_4
2020-06-15 09:41:36+02	2020-06-15 09:44:38+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-15 09:44:40+02	2020-06-15 09:47:41+02	editors	vim	\N	vim-8.2.0869
2020-06-15 09:47:42+02	2020-06-15 10:04:51+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-15 09:30:25+02	2020-06-15 10:05:32+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-15 09:29:40+02	2020-06-15 10:07:25+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-15 10:05:38+02	2020-06-15 10:25:06+02	emulators	wine	\N	wine-5.0.1,1
2020-06-15 10:07:26+02	2020-06-15 10:27:41+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-15 10:25:10+02	2020-06-15 10:27:52+02	games	xcowsay	\N	xcowsay-1.5
2020-06-15 10:27:54+02	2020-06-15 10:33:24+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-15 10:27:44+02	2020-06-15 10:39:38+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-15 10:33:26+02	2020-06-15 10:44:19+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-15 10:44:22+02	2020-06-15 10:46:21+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-15 10:46:21+02	2020-06-15 10:47:58+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-15 10:48:02+02	2020-06-15 10:52:07+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-15 10:39:41+02	2020-06-15 10:53:40+02	graphics	krita	\N	krita-4.2.9_4
2020-06-15 10:52:09+02	2020-06-15 10:54:14+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-15 10:54:15+02	2020-06-15 10:56:19+02	net	ntopng	\N	ntopng-4.0.d20200530,1
2020-06-15 10:53:40+02	2020-06-15 10:56:29+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-15 10:04:52+02	2020-06-15 10:57:10+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-06-15 10:56:19+02	2020-06-15 10:57:54+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-15 10:57:11+02	2020-06-15 10:59:04+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-15 10:57:57+02	2020-06-15 10:59:28+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-15 10:59:05+02	2020-06-15 10:59:48+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-15 10:59:29+02	2020-06-15 11:00:15+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-15 10:56:30+02	2020-06-15 11:00:27+02	net	wireshark	\N	wireshark-3.2.4
2020-06-15 10:59:48+02	2020-06-15 11:00:33+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-15 11:00:16+02	2020-06-15 11:00:58+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-15 11:00:28+02	2020-06-15 11:01:09+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-15 11:00:33+02	2020-06-15 11:01:17+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-15 11:01:10+02	2020-06-15 11:01:55+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-15 11:01:17+02	2020-06-15 11:02:02+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-15 11:00:59+02	2020-06-15 11:02:07+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-15 11:02:07+02	2020-06-15 11:02:16+02	sysutils	py-py-cpuinfo	py27	py27-py-cpuinfo-6.0.0
2020-06-15 11:02:16+02	2020-06-15 11:02:36+02	www	dokuwiki	\N	dokuwiki-20180422c
2020-06-15 11:02:03+02	2020-06-15 11:02:55+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-06-15 11:01:56+02	2020-06-15 11:04:09+02	print	auctex	full	auctex-emacs26-12.2
2020-06-15 11:04:11+02	2020-06-15 11:04:38+02	www	py-dj22-drf-yasg	py37	py37-dj22-drf-yasg-1.17.0
2020-06-15 11:02:56+02	2020-06-15 11:06:40+02	www	mod_php73	\N	mod_php73-7.3.19
2020-06-15 11:04:43+02	2020-06-15 11:12:09+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-15 11:12:13+02	2020-06-15 11:12:30+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-15 11:12:33+02	2020-06-15 11:14:57+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-06-15 11:02:37+02	2020-06-15 11:40:20+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-15 11:06:41+02	2020-06-15 11:50:21+02	editors	vscode	\N	vscode-1.45.1_2
2020-06-15 22:16:15+02	2020-06-15 22:16:26+02	security	py-certifi	py37	py37-certifi-2020.4.5.2
2020-06-15 22:16:15+02	2020-06-15 22:16:31+02	textproc	libyaml	\N	libyaml-0.2.5
2020-06-15 22:16:27+02	2020-06-15 22:16:38+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-06-15 22:16:39+02	2020-06-15 22:17:03+02	www	py-requests	py37	py37-requests-2.22.0
2020-06-15 22:16:15+02	2020-06-15 22:17:11+02	devel	libgsf	\N	libgsf-1.14.47
2020-06-15 22:17:04+02	2020-06-15 22:17:29+02	textproc	py-sphinx	py37	py37-sphinx-3.0.4,1
2020-06-15 22:17:29+02	2020-06-15 22:17:41+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-06-15 22:17:13+02	2020-06-15 22:17:55+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-15 22:17:42+02	2020-06-15 22:18:12+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-06-15 22:18:12+02	2020-06-15 22:21:37+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-15 22:17:56+02	2020-06-15 22:22:22+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-15 22:21:37+02	2020-06-15 22:22:24+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-15 22:22:25+02	2020-06-15 22:23:18+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-15 22:22:23+02	2020-06-15 22:23:28+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-15 22:23:20+02	2020-06-15 22:24:25+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-15 22:23:29+02	2020-06-15 22:24:50+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-15 22:24:26+02	2020-06-15 22:25:24+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-15 22:24:50+02	2020-06-15 22:26:20+02	security	gcr	\N	gcr-3.28.0
2020-06-15 22:25:24+02	2020-06-15 22:26:22+02	net	remmina	\N	remmina-1.3.9
2020-06-15 22:26:20+02	2020-06-15 22:27:19+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-15 22:27:19+02	2020-06-15 22:28:23+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-06-15 22:28:24+02	2020-06-15 22:29:31+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-15 22:16:31+02	2020-06-15 22:29:38+02	lang	ruby27	\N	ruby-2.7.1,1
2020-06-15 22:26:23+02	2020-06-15 22:29:44+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-15 22:29:47+02	2020-06-15 22:30:00+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-06-15 22:29:31+02	2020-06-15 22:30:15+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-15 22:30:16+02	2020-06-15 22:30:54+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-06-15 22:30:54+02	2020-06-15 22:31:06+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-06-15 22:31:07+02	2020-06-15 22:33:09+02	x11	babl	\N	babl-0.1.78
2020-06-15 22:30:01+02	2020-06-15 22:34:24+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-06-15 22:33:10+02	2020-06-15 22:34:47+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-15 22:34:48+02	2020-06-15 22:35:47+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-15 22:35:47+02	2020-06-15 22:36:21+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-06-15 22:29:38+02	2020-06-15 22:38:16+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-15 22:35:42+02	2020-06-15 22:38:32+02	graphics	gegl	\N	gegl-0.4.24
2020-06-15 22:36:22+02	2020-06-15 22:38:47+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-15 22:38:47+02	2020-06-15 22:39:01+02	www	p5-CGI	\N	p5-CGI-4.49
2020-06-15 22:38:17+02	2020-06-15 22:39:44+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-15 22:38:32+02	2020-06-15 22:39:52+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-15 22:39:01+02	2020-06-15 22:40:11+02	x11	zenity	\N	zenity-3.32.0
2020-06-15 22:39:55+02	2020-06-15 22:41:46+02	devel	git	\N	git-2.27.0
2020-06-15 22:40:11+02	2020-06-15 22:46:23+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-15 22:39:45+02	2020-06-15 22:48:06+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-06-15 22:41:49+02	2020-06-15 22:48:09+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-15 22:46:25+02	2020-06-15 22:48:26+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-15 22:48:26+02	2020-06-15 22:49:13+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-15 22:48:07+02	2020-06-15 22:52:51+02	editors	emacs	full	emacs-26.3_4,3
2020-06-15 22:49:13+02	2020-06-15 22:53:41+02	print	fontforge	\N	fontforge-20190801_2
2020-06-15 22:52:53+02	2020-06-15 22:58:04+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-15 22:58:06+02	2020-06-15 22:59:46+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-15 22:59:48+02	2020-06-15 23:01:26+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-15 22:48:09+02	2020-06-15 23:03:36+02	graphics	gimp-app	\N	gimp-app-2.10.20,1
2020-06-15 23:01:26+02	2020-06-15 23:06:13+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-06-15 23:06:14+02	2020-06-15 23:13:12+02	x11-wm	marco	\N	marco-1.22.4
2020-06-15 23:03:37+02	2020-06-15 23:14:25+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-15 23:14:27+02	2020-06-15 23:16:42+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-15 23:13:14+02	2020-06-15 23:17:28+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-15 23:17:32+02	2020-06-15 23:21:04+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-15 23:16:47+02	2020-06-15 23:21:23+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-15 23:21:09+02	2020-06-15 23:27:18+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-15 23:21:24+02	2020-06-15 23:27:24+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-15 23:27:19+02	2020-06-15 23:28:05+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-15 23:27:25+02	2020-06-15 23:28:13+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-15 23:28:06+02	2020-06-15 23:28:33+02	print	libspectre	\N	libspectre-0.2.9
2020-06-15 23:28:34+02	2020-06-15 23:28:47+02	security	py-certifi	py27	py27-certifi-2020.4.5.2
2020-06-15 22:53:42+02	2020-06-15 23:29:54+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-15 23:28:14+02	2020-06-15 23:30:34+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-15 23:28:51+02	2020-06-15 23:31:20+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-15 23:29:57+02	2020-06-15 23:31:24+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-15 23:31:20+02	2020-06-15 23:31:34+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.12
2020-06-15 23:30:34+02	2020-06-15 23:31:34+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-15 23:31:35+02	2020-06-15 23:31:49+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-06-15 23:31:34+02	2020-06-15 23:31:55+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.12.2
2020-06-15 23:31:24+02	2020-06-15 23:32:18+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-15 23:32:18+02	2020-06-15 23:32:30+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-06-15 23:32:30+02	2020-06-15 23:32:41+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-06-15 23:31:56+02	2020-06-15 23:32:54+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-15 23:32:41+02	2020-06-15 23:32:58+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-06-15 23:32:54+02	2020-06-15 23:33:15+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-06-15 23:31:50+02	2020-06-15 23:34:10+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-15 23:34:10+02	2020-06-15 23:35:54+02	graphics	eom	\N	eom-1.24.0
2020-06-15 23:33:15+02	2020-06-15 23:36:55+02	editors	pluma	\N	pluma-1.24.0
2020-06-15 23:35:56+02	2020-06-15 23:37:11+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-15 23:36:55+02	2020-06-15 23:38:36+02	graphics	py-gimp	\N	py27-gimp-2.10.20
2020-06-15 23:37:12+02	2020-06-15 23:38:36+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-15 23:38:36+02	2020-06-15 23:38:45+02	net	py-urllib3	py27	py27-urllib3-1.25.7,1
2020-06-15 23:38:37+02	2020-06-15 23:39:21+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-15 23:38:46+02	2020-06-15 23:39:36+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-15 23:39:22+02	2020-06-15 23:40:44+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-15 23:39:36+02	2020-06-15 23:41:28+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-15 23:40:45+02	2020-06-15 23:42:33+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-15 23:41:29+02	2020-06-15 23:43:22+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-15 23:42:35+02	2020-06-15 23:44:50+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-15 23:43:24+02	2020-06-15 23:46:16+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-15 23:44:51+02	2020-06-15 23:46:33+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-15 23:46:17+02	2020-06-15 23:47:26+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-15 23:46:34+02	2020-06-15 23:47:31+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-15 23:47:32+02	2020-06-15 23:49:03+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-06-15 23:49:03+02	2020-06-15 23:51:39+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-15 23:51:39+02	2020-06-15 23:52:50+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-15 23:52:51+02	2020-06-15 23:53:07+02	www	p5-libwww	\N	p5-libwww-6.45
2020-06-15 23:47:26+02	2020-06-15 23:53:18+02	print	lilypond	\N	lilypond-2.20.0
2020-06-15 23:53:19+02	2020-06-15 23:54:38+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-15 23:32:58+02	2020-06-15 23:55:32+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-15 23:54:40+02	2020-06-15 23:56:59+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-15 23:55:36+02	2020-06-15 23:57:05+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-15 23:53:08+02	2020-06-15 23:57:45+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-15 23:57:46+02	2020-06-15 23:58:29+02	databases	pgtop	\N	pgtop-0.05_5
2020-06-15 23:57:05+02	2020-06-16 00:00:56+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-16 00:00:56+02	2020-06-16 00:01:54+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-06-15 23:56:59+02	2020-06-16 00:02:04+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-15 23:58:30+02	2020-06-16 00:02:10+02	deskutils	calibre	\N	calibre-4.18.0
2020-06-16 00:02:10+02	2020-06-16 00:02:36+02	devel	tig	\N	tig-2.5.1
2020-06-16 00:01:55+02	2020-06-16 00:03:03+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-16 00:03:04+02	2020-06-16 00:07:43+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-16 00:02:36+02	2020-06-16 00:09:40+02	editors	codelite	\N	codelite-12.0_4
2020-06-16 00:07:46+02	2020-06-16 00:10:06+02	editors	vim	\N	vim-8.2.0869
2020-06-16 00:10:06+02	2020-06-16 00:24:30+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-16 00:24:31+02	2020-06-16 00:24:55+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-06-16 00:24:57+02	2020-06-16 00:26:53+02	games	xboard	\N	xboard-4.9.1_1
2020-06-16 00:26:54+02	2020-06-16 00:27:49+02	games	xcowsay	\N	xcowsay-1.5
2020-06-16 00:09:41+02	2020-06-16 00:28:49+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-16 00:27:50+02	2020-06-16 00:29:56+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-16 00:28:51+02	2020-06-16 00:29:59+02	graphics	gimp	\N	gimp-2.10.20,2
2020-06-16 00:02:05+02	2020-06-16 00:33:58+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-16 00:34:05+02	2020-06-16 00:41:03+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-06-16 00:41:07+02	2020-06-16 00:44:58+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-16 00:45:03+02	2020-06-16 00:48:38+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-16 00:29:59+02	2020-06-16 00:50:20+02	graphics	krita	\N	krita-4.2.9_4
2020-06-16 00:48:39+02	2020-06-16 00:56:26+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-16 00:50:25+02	2020-06-16 00:59:02+02	net	ntopng	\N	ntopng-4.0.d20200615,1
2020-06-16 00:56:28+02	2020-06-16 01:01:29+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-16 00:59:05+02	2020-06-16 01:02:26+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-16 01:01:33+02	2020-06-16 01:02:57+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-16 01:02:27+02	2020-06-16 01:03:12+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-16 00:29:57+02	2020-06-16 01:03:34+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-16 01:02:57+02	2020-06-16 01:03:39+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-16 01:03:12+02	2020-06-16 01:03:54+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-16 01:03:35+02	2020-06-16 01:04:17+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-16 01:03:39+02	2020-06-16 01:04:21+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-16 01:03:55+02	2020-06-16 01:04:37+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-16 01:04:22+02	2020-06-16 01:05:09+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-16 01:04:38+02	2020-06-16 01:05:21+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-16 01:04:17+02	2020-06-16 01:05:28+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-16 01:05:28+02	2020-06-16 01:05:48+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-06-16 01:05:21+02	2020-06-16 01:06:15+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-06-16 01:06:15+02	2020-06-16 01:06:37+02	www	py-dj22-drf-yasg	py37	py37-dj22-drf-yasg-1.17.0
2020-06-16 01:06:37+02	2020-06-16 01:06:46+02	www	py-requests	py27	py27-requests-2.22.0
2020-06-16 01:05:10+02	2020-06-16 01:07:37+02	print	auctex	full	auctex-emacs26-12.2
2020-06-16 01:06:47+02	2020-06-16 01:08:16+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-16 01:08:17+02	2020-06-16 01:08:24+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-16 01:08:25+02	2020-06-16 01:08:49+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-06-16 01:05:48+02	2020-06-16 01:39:24+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-16 01:07:38+02	2020-06-16 01:50:41+02	editors	vscode	\N	vscode-1.46.0
2020-06-16 17:24:43+02	2020-06-16 17:25:02+02	shells	bash-completion	\N	bash-completion-2.10,2
2020-06-16 17:25:02+02	2020-06-16 17:25:37+02	devel	dconf	\N	dconf-0.36.0
2020-06-16 17:25:37+02	2020-06-16 17:43:10+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-16 17:43:11+02	2020-06-16 18:06:53+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-16 20:29:23+02	2020-06-16 20:30:17+02	devel	hwloc	\N	hwloc-1.11.11
2020-06-16 20:29:23+02	2020-06-16 20:30:27+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-16 20:29:23+02	2020-06-16 20:30:41+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-16 20:30:41+02	2020-06-16 20:31:10+02	textproc	man2html	\N	man2html-3.0.1_2
2020-06-16 20:30:17+02	2020-06-16 20:31:20+02	security	munge	\N	munge-0.5.14_1
2020-06-16 20:30:27+02	2020-06-16 20:31:59+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-16 20:31:11+02	2020-06-16 20:38:42+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-16 20:31:59+02	2020-06-16 20:38:58+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-16 20:38:49+02	2020-06-16 20:39:07+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-06-16 20:39:07+02	2020-06-16 20:44:00+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-16 20:44:03+02	2020-06-16 20:44:43+02	print	gl2ps	\N	gl2ps-1.4.0
2020-06-16 20:38:58+02	2020-06-16 20:44:51+02	graphics	freeimage	\N	freeimage-3.18.0_1
2020-06-16 20:31:20+02	2020-06-16 20:45:18+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-06-16 20:44:52+02	2020-06-16 20:48:25+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-06-16 20:44:43+02	2020-06-16 20:50:29+02	cad	ngspice_rework	shlib	ngspice_rework-shlib-31
2020-06-16 20:51:50+02	2020-06-16 20:53:11+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-16 20:53:11+02	2020-06-16 20:53:27+02	x11-wm	twm	\N	twm-1.0.11
2020-06-16 20:53:28+02	2020-06-16 20:54:44+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-06-16 20:54:45+02	2020-06-16 20:56:28+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-16 20:55:54+02	2020-06-16 20:56:30+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-06-16 20:56:29+02	2020-06-16 20:59:18+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-16 20:59:19+02	2020-06-16 21:00:54+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-16 20:45:27+02	2020-06-16 21:01:45+02	net	openmpi	\N	openmpi-4.0.3
2020-06-16 21:00:55+02	2020-06-16 21:02:30+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-16 20:56:31+02	2020-06-16 21:02:34+02	editors	emacs	full	emacs-26.3_4,3
2020-06-16 21:02:30+02	2020-06-16 21:03:37+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-16 21:03:37+02	2020-06-16 21:05:12+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-16 21:02:34+02	2020-06-16 21:05:38+02	x11	xorg	\N	xorg-7.7_3
2020-06-16 21:05:13+02	2020-06-16 21:06:33+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-16 21:06:34+02	2020-06-16 21:08:29+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-06-16 21:08:32+02	2020-06-16 21:14:36+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-16 21:05:38+02	2020-06-16 21:17:23+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-16 21:17:25+02	2020-06-16 21:36:39+02	editors	pluma	\N	pluma-1.24.0
2020-06-16 21:14:37+02	2020-06-16 21:39:10+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-16 21:01:46+02	2020-06-16 22:07:41+02	math	vtk8	\N	vtk8-8.2.0
2020-06-16 23:19:45+02	2020-06-16 23:34:33+02	graphics	eom	\N	eom-1.24.0
2020-06-16 23:15:12+02	2020-06-17 00:07:42+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-06-17 00:07:45+02	2020-06-17 00:15:32+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-17 00:15:35+02	2020-06-17 00:20:03+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-17 00:20:06+02	2020-06-17 00:28:19+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-06-17 00:28:22+02	2020-06-17 00:49:25+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-17 00:49:25+02	2020-06-17 00:50:33+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-16 23:15:12+02	2020-06-17 00:51:04+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-06-17 00:50:33+02	2020-06-17 00:52:02+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-17 00:52:03+02	2020-06-17 00:57:10+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-17 00:57:12+02	2020-06-17 00:59:46+02	devel	cdecl	\N	cdecl-7.2
2020-06-17 00:59:49+02	2020-06-17 01:14:46+02	editors	codelite	\N	codelite-12.0_4
2020-06-17 01:14:49+02	2020-06-17 01:21:16+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-16 23:34:40+02	2020-06-17 01:31:23+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-17 01:31:26+02	2020-06-17 01:36:36+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-17 01:21:19+02	2020-06-17 01:46:41+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-17 01:36:38+02	2020-06-17 01:49:25+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-17 01:46:46+02	2020-06-17 01:49:34+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-17 00:51:05+02	2020-06-17 01:58:49+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-06-17 01:58:50+02	2020-06-17 02:03:45+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-06-17 02:03:46+02	2020-06-17 02:06:08+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-06-17 01:49:29+02	2020-06-17 02:14:48+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-17 02:14:49+02	2020-06-17 02:16:42+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-06-17 02:16:44+02	2020-06-17 02:18:20+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-06-17 01:49:40+02	2020-06-17 02:21:36+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-17 02:21:38+02	2020-06-17 02:25:17+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-17 02:25:19+02	2020-06-17 02:28:53+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-17 02:28:55+02	2020-06-17 02:32:24+02	print	auctex	full	auctex-emacs26-12.2
2020-06-17 02:32:24+02	2020-06-17 02:32:33+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-17 02:18:20+02	2020-06-17 03:56:23+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200613
2020-06-17 02:06:09+02	2020-06-17 03:58:30+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-06-17 07:53:49+02	2020-06-17 07:54:22+02	devel	ocl-icd	\N	ocl-icd-2.2.12
2020-06-17 07:54:23+02	2020-06-17 07:54:47+02	databases	lmdb	\N	lmdb-0.9.24_2,1
2020-06-17 07:53:49+02	2020-06-17 07:54:50+02	devel	dbus	\N	dbus-1.12.18
2020-06-17 07:54:48+02	2020-06-17 07:54:54+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200616
2020-06-17 07:54:54+02	2020-06-17 07:56:11+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-06-17 07:53:49+02	2020-06-17 07:56:15+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-06-17 07:54:50+02	2020-06-17 07:56:15+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-06-17 07:56:12+02	2020-06-17 07:57:45+02	sysutils	polkit	\N	polkit-0.116
2020-06-17 07:56:16+02	2020-06-17 07:57:48+02	net	avahi-app	\N	avahi-app-0.7_3
2020-06-17 07:57:45+02	2020-06-17 07:58:03+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-06-17 07:56:15+02	2020-06-17 07:58:18+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-17 07:57:48+02	2020-06-17 07:58:48+02	print	cups	\N	cups-2.2.13
2020-06-17 07:58:04+02	2020-06-17 08:01:15+02	graphics	colord	\N	colord-1.3.5
2020-06-17 07:58:50+02	2020-06-17 08:01:16+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-06-17 07:58:19+02	2020-06-17 08:01:26+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-17 08:01:16+02	2020-06-17 08:01:44+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-06-17 08:01:17+02	2020-06-17 08:02:43+02	devel	qt5-dbus	\N	qt5-dbus-5.14.2
2020-06-17 08:01:45+02	2020-06-17 08:02:53+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-2.24.32
2020-06-17 08:02:54+02	2020-06-17 08:08:04+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-17 08:08:06+02	2020-06-17 08:10:08+02	graphics	libGLU	\N	libGLU-9.0.1
2020-06-17 08:01:26+02	2020-06-17 08:10:18+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-17 08:02:44+02	2020-06-17 08:11:45+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-17 08:10:20+02	2020-06-17 08:13:10+02	lang	vala	\N	vala-0.40.15,1
2020-06-17 08:13:14+02	2020-06-17 08:16:43+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-17 08:11:46+02	2020-06-17 08:16:54+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-17 08:10:09+02	2020-06-17 08:17:27+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-17 08:16:54+02	2020-06-17 08:17:42+02	devel	dconf	\N	dconf-0.36.0
2020-06-17 08:17:43+02	2020-06-17 08:19:44+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-17 08:19:47+02	2020-06-17 08:20:51+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-06-17 08:20:52+02	2020-06-17 08:24:13+02	devel	libsoup	\N	libsoup-2.62.3
2020-06-17 08:24:14+02	2020-06-17 08:26:01+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-17 08:17:28+02	2020-06-17 08:26:45+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-17 08:16:45+02	2020-06-17 08:27:05+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-17 08:26:03+02	2020-06-17 08:27:34+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-17 08:27:06+02	2020-06-17 08:28:08+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-17 08:27:34+02	2020-06-17 08:28:20+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-17 08:28:09+02	2020-06-17 08:28:45+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-17 08:28:20+02	2020-06-17 08:29:45+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-17 08:29:45+02	2020-06-17 08:30:26+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-17 08:26:45+02	2020-06-17 08:30:27+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-17 08:30:27+02	2020-06-17 08:30:52+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-17 08:30:52+02	2020-06-17 08:31:17+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_1
2020-06-17 08:31:18+02	2020-06-17 08:31:48+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-17 08:31:49+02	2020-06-17 08:35:28+02	graphics	graphene	\N	graphene-1.10.0
2020-06-17 08:35:33+02	2020-06-17 08:36:34+02	www	libwpe	\N	libwpe-1.4.0.1
2020-06-17 08:36:36+02	2020-06-17 08:42:15+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-17 08:42:16+02	2020-06-17 08:43:13+02	net	geoclue	\N	geoclue-2.5.5
2020-06-17 08:43:13+02	2020-06-17 08:43:39+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-06-17 08:28:46+02	2020-06-17 08:43:51+02	multimedia	ffmpeg	\N	ffmpeg-4.3,1
2020-06-17 08:43:40+02	2020-06-17 08:44:07+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-06-17 08:43:51+02	2020-06-17 08:44:08+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-17 08:44:07+02	2020-06-17 08:44:51+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-06-17 08:44:53+02	2020-06-17 08:45:20+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-17 08:45:21+02	2020-06-17 08:49:17+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-17 08:49:17+02	2020-06-17 08:50:36+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-17 08:44:08+02	2020-06-17 08:51:25+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-17 08:50:37+02	2020-06-17 08:51:44+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-17 08:51:27+02	2020-06-17 08:55:12+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_7
2020-06-17 08:51:45+02	2020-06-17 08:55:26+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-17 08:55:14+02	2020-06-17 08:55:57+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-17 08:55:27+02	2020-06-17 08:56:01+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-17 08:55:58+02	2020-06-17 08:56:42+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-17 08:56:02+02	2020-06-17 08:57:02+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-17 08:30:27+02	2020-06-17 08:57:09+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-17 08:56:43+02	2020-06-17 08:57:31+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-17 08:57:02+02	2020-06-17 08:57:53+02	devel	kf5-kconfig	\N	kf5-kconfig-5.70.0
2020-06-17 08:57:53+02	2020-06-17 08:58:55+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-17 08:58:55+02	2020-06-17 08:59:34+02	sysutils	policykit	\N	policykit-0.9_10
2020-06-17 08:57:09+02	2020-06-17 08:59:48+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-17 08:59:35+02	2020-06-17 09:00:35+02	devel	kf5-ki18n	\N	kf5-ki18n-5.70.0
2020-06-17 08:59:48+02	2020-06-17 09:00:41+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-17 09:00:35+02	2020-06-17 09:01:08+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-06-17 09:01:08+02	2020-06-17 09:02:11+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.70.0
2020-06-17 09:00:41+02	2020-06-17 09:02:14+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-17 09:02:14+02	2020-06-17 09:02:59+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-17 09:02:11+02	2020-06-17 09:03:03+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-17 08:57:32+02	2020-06-17 09:03:27+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-17 09:03:00+02	2020-06-17 09:03:37+02	sysutils	hal	\N	hal-0.5.14_34
2020-06-17 09:03:04+02	2020-06-17 09:03:55+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.70.0
2020-06-17 09:03:56+02	2020-06-17 09:05:49+02	devel	sdl20	\N	sdl2-2.0.10_1
2020-06-17 09:05:49+02	2020-06-17 09:06:29+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-17 09:03:37+02	2020-06-17 09:06:32+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-06-17 09:06:30+02	2020-06-17 09:07:44+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.70.0
2020-06-17 09:03:28+02	2020-06-17 09:08:02+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-17 09:06:33+02	2020-06-17 09:08:06+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-06-17 09:07:45+02	2020-06-17 09:08:30+02	devel	kf5-kcrash	\N	kf5-kcrash-5.70.0
2020-06-17 09:08:03+02	2020-06-17 09:08:49+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.70.0
2020-07-05 06:24:22+02	2020-07-05 06:25:50+02	misc	mc	\N	mc-nox11-4.8.24
2020-06-17 09:08:07+02	2020-06-17 09:08:49+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-17 09:08:31+02	2020-06-17 09:09:02+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-06-17 09:08:50+02	2020-06-17 09:09:35+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-17 09:09:03+02	2020-06-17 09:09:42+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-17 09:08:50+02	2020-06-17 09:09:46+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-17 09:09:42+02	2020-06-17 09:10:30+02	devel	kf5-kauth	\N	kf5-kauth-5.70.0
2020-06-17 09:09:36+02	2020-06-17 09:10:33+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-17 09:09:46+02	2020-06-17 09:10:48+02	devel	kf5-kservice	\N	kf5-kservice-5.70.0
2020-06-17 09:10:33+02	2020-06-17 09:10:55+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-06-17 09:10:31+02	2020-06-17 09:11:55+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-17 09:10:48+02	2020-06-17 09:11:59+02	net	remmina	\N	remmina-1.3.9
2020-06-17 09:10:55+02	2020-06-17 09:12:37+02	security	gcr	\N	gcr-3.28.0
2020-06-17 09:11:57+02	2020-06-17 09:12:50+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.70.0
2020-06-17 09:12:00+02	2020-06-17 09:13:14+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-17 09:12:50+02	2020-06-17 09:13:39+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-17 09:13:15+02	2020-06-17 09:13:40+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-06-17 09:13:41+02	2020-06-17 09:16:43+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.70.0
2020-06-17 09:12:37+02	2020-06-17 09:17:52+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-17 09:16:45+02	2020-06-17 09:18:08+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.70.0
2020-06-17 09:13:39+02	2020-06-17 09:18:25+02	graphics	glew	\N	glew-2.2.0
2020-06-17 09:18:29+02	2020-06-17 09:19:16+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-17 09:17:53+02	2020-06-17 09:19:18+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-17 09:19:17+02	2020-06-17 09:20:03+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-17 09:19:19+02	2020-06-17 09:20:10+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-17 09:18:08+02	2020-06-17 09:20:44+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-17 09:20:05+02	2020-06-17 09:21:07+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.70.0
2020-06-17 09:20:45+02	2020-06-17 09:21:23+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-17 09:21:07+02	2020-06-17 09:21:59+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-17 09:20:11+02	2020-06-17 09:22:08+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-06-17 09:22:00+02	2020-06-17 09:23:32+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-17 09:23:32+02	2020-06-17 09:24:05+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-17 09:22:08+02	2020-06-17 09:27:31+02	math	vtk8	\N	vtk8-8.2.0
2020-06-17 09:27:32+02	2020-06-17 09:27:49+02	print	gl2ps	\N	gl2ps-1.4.0
2020-06-17 09:24:06+02	2020-06-17 09:28:11+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-17 09:27:49+02	2020-06-17 09:28:40+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.70.0
2020-06-17 09:28:12+02	2020-06-17 09:29:11+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-06-17 09:28:40+02	2020-06-17 09:29:27+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.70.0
2020-06-17 09:29:12+02	2020-06-17 09:30:02+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-17 09:30:03+02	2020-06-17 09:30:52+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-06-17 09:29:28+02	2020-06-17 09:30:55+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-17 09:30:56+02	2020-06-17 09:31:46+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-17 09:31:47+02	2020-06-17 09:32:22+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-06-17 09:32:23+02	2020-06-17 09:33:41+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-17 09:33:42+02	2020-06-17 09:35:13+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-17 09:35:13+02	2020-06-17 09:36:37+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-06-17 09:36:38+02	2020-06-17 09:37:13+02	sysutils	upower	\N	upower-0.99.11
2020-06-17 09:30:53+02	2020-06-17 09:37:16+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-06-17 09:37:17+02	2020-06-17 09:38:15+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.70.0
2020-06-17 09:38:15+02	2020-06-17 09:42:42+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-06-17 09:37:13+02	2020-06-17 09:43:14+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-17 09:42:43+02	2020-06-17 09:43:36+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.70.0
2020-06-17 09:21:24+02	2020-06-17 09:43:55+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-17 09:43:15+02	2020-06-17 09:44:22+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.70.0
2020-06-17 09:43:55+02	2020-06-17 09:44:27+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-06-17 09:44:28+02	2020-06-17 09:46:50+02	devel	librest	\N	rest-0.7.93
2020-06-17 09:44:23+02	2020-06-17 09:47:48+02	devel	kf5-knotifications	\N	kf5-knotifications-5.70.0
2020-06-17 09:46:52+02	2020-06-17 09:49:16+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-17 09:47:50+02	2020-06-17 09:49:17+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-17 09:49:17+02	2020-06-17 09:49:32+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-06-17 09:49:17+02	2020-06-17 09:49:45+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-06-17 09:43:37+02	2020-06-17 09:49:54+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-06-17 09:49:32+02	2020-06-17 09:50:15+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-17 09:49:46+02	2020-06-17 09:50:24+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-17 09:50:24+02	2020-06-17 09:50:37+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-06-17 09:50:16+02	2020-06-17 09:51:17+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-17 09:50:37+02	2020-06-17 09:51:29+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-17 09:51:17+02	2020-06-17 09:53:40+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-06-17 09:51:29+02	2020-06-17 09:54:28+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-17 09:53:42+02	2020-06-17 09:57:15+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-06-17 09:54:30+02	2020-06-17 09:57:36+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-06-17 09:57:38+02	2020-06-17 09:59:12+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-06-17 09:57:16+02	2020-06-17 09:59:34+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-06-17 09:59:12+02	2020-06-17 10:01:55+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-06-17 10:01:58+02	2020-06-17 10:03:22+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.70.0
2020-06-17 10:02:43+02	2020-06-17 10:03:34+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-17 09:49:55+02	2020-06-17 10:03:42+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-06-17 10:03:44+02	2020-06-17 10:04:19+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-17 10:03:23+02	2020-06-17 10:04:28+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.70.0
2020-06-17 10:03:34+02	2020-06-17 10:04:39+02	devel	kf5-solid	\N	kf5-solid-5.70.0
2020-06-17 10:04:39+02	2020-06-17 10:04:55+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-17 10:04:21+02	2020-06-17 10:05:57+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-17 10:04:32+02	2020-06-17 10:06:08+02	graphics	gegl	\N	gegl-0.4.24
2020-06-17 10:06:08+02	2020-06-17 10:06:32+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-06-17 10:05:58+02	2020-06-17 10:07:26+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-17 10:06:32+02	2020-06-17 10:07:30+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-06-17 10:07:27+02	2020-06-17 10:08:10+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-17 10:07:31+02	2020-06-17 10:09:01+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-17 10:08:10+02	2020-06-17 10:09:02+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-06-17 10:04:55+02	2020-06-17 10:09:33+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-07-12 16:51:03+02	2020-07-12 16:51:47+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-17 10:09:02+02	2020-06-17 10:09:55+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-17 10:09:01+02	2020-06-17 10:10:15+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.70.0
2020-06-17 10:09:55+02	2020-06-17 10:10:16+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-06-17 10:09:34+02	2020-06-17 10:10:27+02	x11	kf5-kded	\N	kf5-kded-5.70.0_1
2020-06-17 10:10:17+02	2020-06-17 10:11:15+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-17 10:10:15+02	2020-06-17 10:11:20+02	x11	zenity	\N	zenity-3.32.0
2020-06-17 10:10:28+02	2020-06-17 10:11:58+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-06-17 10:11:21+02	2020-06-17 10:12:18+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.70.0
2020-06-17 10:11:15+02	2020-06-17 10:12:42+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-06-17 10:11:59+02	2020-06-17 10:13:20+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-17 10:12:42+02	2020-06-17 10:13:44+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-17 10:13:21+02	2020-06-17 10:13:47+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-17 10:13:49+02	2020-06-17 10:14:59+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-17 10:12:19+02	2020-06-17 10:15:31+02	devel	kf5-kio	\N	kf5-kio-5.70.1_1
2020-06-17 10:15:31+02	2020-06-17 10:16:07+02	graphics	movit	\N	movit-1.6.2_2
2020-06-17 10:13:44+02	2020-06-17 10:19:08+02	editors	emacs	full	emacs-26.3_4,3
2020-06-17 10:19:12+02	2020-06-17 10:19:53+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-06-17 10:19:54+02	2020-06-17 10:20:48+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-17 10:16:08+02	2020-06-17 10:20:55+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-17 10:20:48+02	2020-06-17 10:21:07+02	net	gssdp	\N	gssdp-1.2.2
2020-06-17 10:14:59+02	2020-06-17 10:21:13+02	graphics	gimp-app	\N	gimp-app-2.10.20,1
2020-06-17 10:20:56+02	2020-06-17 10:21:43+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-17 10:21:07+02	2020-06-17 10:21:45+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-17 10:21:46+02	2020-06-17 10:22:47+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-17 10:21:14+02	2020-06-17 10:23:56+02	print	fontforge	\N	fontforge-20190801_2
2020-06-17 10:22:48+02	2020-06-17 10:24:28+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-17 10:21:43+02	2020-06-17 10:33:51+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-17 10:33:56+02	2020-06-17 10:39:06+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-17 10:39:08+02	2020-06-17 10:45:33+02	x11	xorg	\N	xorg-7.7_3
2020-06-17 10:24:29+02	2020-06-17 10:46:41+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-17 10:45:34+02	2020-06-17 10:48:19+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-17 10:46:48+02	2020-06-17 10:49:07+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-17 10:48:20+02	2020-06-17 10:50:08+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-17 10:49:08+02	2020-06-17 10:51:56+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-06-17 10:50:13+02	2020-06-17 10:52:03+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-17 10:52:05+02	2020-06-17 10:53:10+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-06-17 10:52:01+02	2020-06-17 10:54:23+02	x11-wm	marco	\N	marco-1.22.4
2020-06-17 10:54:25+02	2020-06-17 10:55:24+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-17 10:53:14+02	2020-06-17 10:55:25+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-17 10:55:28+02	2020-06-17 10:57:10+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-17 10:55:25+02	2020-06-17 10:57:46+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-17 10:57:11+02	2020-06-17 10:58:07+02	devel	rebar	\N	rebar-2.6.4,1
2020-06-17 10:23:57+02	2020-06-17 10:59:00+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-17 10:57:46+02	2020-06-17 10:59:21+02	devel	rebar3	\N	rebar3-3.13.2,1
2020-06-17 10:59:21+02	2020-06-17 10:59:46+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-06-17 10:59:05+02	2020-06-17 11:00:14+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-17 10:58:08+02	2020-06-17 11:01:00+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-17 11:00:14+02	2020-06-17 11:01:25+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-06-17 11:01:01+02	2020-06-17 11:01:35+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-17 10:59:48+02	2020-06-17 11:01:48+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-17 11:01:26+02	2020-06-17 11:01:48+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-06-17 11:01:48+02	2020-06-17 11:02:18+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-17 11:01:36+02	2020-06-17 11:02:37+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-17 11:01:49+02	2020-06-17 11:02:43+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-17 11:02:37+02	2020-06-17 11:03:34+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-17 11:02:18+02	2020-06-17 11:04:41+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-17 11:04:47+02	2020-06-17 11:08:31+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-17 11:08:31+02	2020-06-17 11:09:47+02	net	freerdp	\N	freerdp-2.1.1
2020-06-17 11:03:35+02	2020-06-17 11:09:57+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-17 11:09:47+02	2020-06-17 11:10:17+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-17 11:09:57+02	2020-06-17 11:10:52+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-17 11:10:17+02	2020-06-17 11:11:32+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-17 11:10:52+02	2020-06-17 11:11:42+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-17 11:11:42+02	2020-06-17 11:12:03+02	print	libspectre	\N	libspectre-0.2.9
2020-06-17 11:12:03+02	2020-06-17 11:14:44+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-17 11:11:33+02	2020-06-17 11:15:39+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-17 11:02:44+02	2020-06-17 11:19:49+02	math	R	\N	R-4.0.1
2020-06-17 11:19:50+02	2020-06-17 11:20:11+02	sysutils	hal-info	\N	hal-info-20091130
2020-06-17 11:14:45+02	2020-06-17 11:20:54+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-17 11:15:40+02	2020-06-17 11:21:28+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-17 11:20:55+02	2020-06-17 11:21:56+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-17 11:20:12+02	2020-06-17 11:22:47+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-17 11:21:57+02	2020-06-17 11:22:51+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-17 11:22:51+02	2020-06-17 11:23:24+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-06-17 11:21:28+02	2020-06-17 11:23:38+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-17 11:22:48+02	2020-06-17 11:23:53+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-17 11:23:53+02	2020-06-17 11:24:41+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-17 11:23:39+02	2020-06-17 11:24:42+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-17 11:23:24+02	2020-06-17 11:24:53+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-06-17 11:24:54+02	2020-06-17 11:25:26+02	audio	faudio	\N	FAudio-20.06
2020-06-17 11:24:42+02	2020-06-17 11:25:38+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-17 11:25:27+02	2020-06-17 11:29:43+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-17 11:25:38+02	2020-06-17 11:30:19+02	databases	rrdtool	\N	rrdtool-1.7.2_3
2020-06-17 11:30:22+02	2020-06-17 11:33:49+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-17 11:33:51+02	2020-06-17 11:37:02+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-17 11:37:09+02	2020-06-17 11:40:04+02	devel	erlang-cl	\N	erlang-cl-1.2.4
2020-06-17 11:40:07+02	2020-06-17 11:42:35+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-06-17 11:42:36+02	2020-06-17 11:44:37+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-06-17 11:29:45+02	2020-06-17 11:44:53+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-17 11:24:43+02	2020-06-17 11:45:29+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-06-17 11:44:54+02	2020-06-17 11:45:53+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-06-17 11:44:37+02	2020-06-17 11:46:00+02	devel	ode	\N	ode-0.13_4,1
2020-06-17 11:45:35+02	2020-06-17 11:46:30+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-17 11:46:31+02	2020-06-17 11:49:58+02	editors	pluma	\N	pluma-1.24.0
2020-06-17 11:49:58+02	2020-06-17 11:52:16+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-06-17 11:52:16+02	2020-06-17 11:56:35+02	graphics	eom	\N	eom-1.24.0
2020-06-17 11:56:36+02	2020-06-17 12:00:10+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-17 12:00:13+02	2020-06-17 12:01:11+02	graphics	gle	\N	gle-3.1.0_7
2020-06-17 12:01:11+02	2020-06-17 12:01:33+02	graphics	gltt	\N	gltt-2.5.2_14
2020-06-17 12:01:33+02	2020-06-17 12:02:21+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-17 12:02:22+02	2020-06-17 12:03:47+02	graphics	py-gimp	\N	py27-gimp-2.10.20
2020-06-17 12:03:48+02	2020-06-17 12:04:29+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-06-17 11:46:01+02	2020-06-17 12:08:54+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-17 11:45:54+02	2020-06-17 12:12:53+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-17 12:04:29+02	2020-06-17 12:13:18+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-17 12:12:53+02	2020-06-17 12:14:36+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-17 12:13:18+02	2020-06-17 12:14:50+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-17 12:08:55+02	2020-06-17 12:18:18+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-06-17 12:14:51+02	2020-06-17 12:18:29+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-17 12:14:37+02	2020-06-17 12:19:05+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-17 12:18:30+02	2020-06-17 12:19:35+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-17 12:19:05+02	2020-06-17 12:19:51+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-17 12:19:36+02	2020-06-17 12:20:22+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-17 12:19:51+02	2020-06-17 12:20:34+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-17 12:20:23+02	2020-06-17 12:21:32+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-17 12:20:37+02	2020-06-17 12:21:49+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-17 12:21:34+02	2020-06-17 12:22:35+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-17 12:21:49+02	2020-06-17 12:22:59+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-17 12:18:18+02	2020-06-17 12:23:27+02	multimedia	vlc	\N	vlc-3.0.10_7,4
2020-06-17 12:22:59+02	2020-06-17 12:23:51+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-17 12:22:35+02	2020-06-17 12:24:02+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-17 12:23:27+02	2020-06-17 12:24:40+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-17 12:23:55+02	2020-06-17 12:24:57+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-17 12:24:03+02	2020-06-17 12:29:26+02	print	lilypond	\N	lilypond-2.20.0
2020-06-17 12:29:26+02	2020-06-17 12:31:21+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-17 12:24:41+02	2020-06-17 12:31:40+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-17 12:24:57+02	2020-06-17 12:31:41+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-17 12:31:42+02	2020-06-17 12:32:09+02	print	transfig	\N	transfig-3.2.5e_4
2020-06-17 12:32:10+02	2020-06-17 12:33:44+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-06-17 12:33:44+02	2020-06-17 12:35:22+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-17 12:35:24+02	2020-06-17 12:36:22+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-17 12:36:23+02	2020-06-17 12:37:00+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-17 12:37:00+02	2020-06-17 12:38:07+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-17 12:31:21+02	2020-06-17 12:38:26+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-17 12:38:07+02	2020-06-17 12:39:12+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-17 12:39:13+02	2020-06-17 12:41:32+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-17 12:41:33+02	2020-06-17 12:43:20+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-17 12:38:27+02	2020-06-17 12:43:34+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-17 12:43:21+02	2020-06-17 12:44:33+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-06-17 12:43:35+02	2020-06-17 12:45:04+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-17 12:44:34+02	2020-06-17 12:45:08+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-17 12:45:04+02	2020-06-17 12:46:26+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-17 12:46:28+02	2020-06-17 12:51:03+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-17 12:45:08+02	2020-06-17 12:52:00+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-17 12:52:04+02	2020-06-17 12:54:21+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-06-17 12:31:41+02	2020-06-17 12:55:55+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-17 12:51:04+02	2020-06-17 12:56:38+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-06-17 12:55:57+02	2020-06-17 12:57:59+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-06-17 12:56:39+02	2020-06-17 12:58:28+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-06-17 12:58:00+02	2020-06-17 13:01:53+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-17 12:58:29+02	2020-06-17 13:02:37+02	deskutils	calibre	\N	calibre-4.18.0
2020-06-17 13:01:53+02	2020-06-17 13:02:46+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-06-17 13:02:47+02	2020-06-17 13:04:03+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-17 13:04:03+02	2020-06-17 13:40:58+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-17 13:02:38+02	2020-06-17 13:41:58+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-17 13:41:02+02	2020-06-17 13:42:28+02	dns	bind916	\N	bind916-9.16.3_2
2020-06-17 13:42:29+02	2020-06-17 13:47:27+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-17 13:42:00+02	2020-06-17 13:50:49+02	editors	codelite	\N	codelite-12.0_4
2020-06-17 13:47:30+02	2020-06-17 13:50:55+02	editors	vim	\N	vim-8.2.0869
2020-06-17 13:50:50+02	2020-06-17 14:11:45+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-17 14:11:50+02	2020-06-17 14:37:18+02	emulators	wine	\N	wine-5.0.1,1
2020-06-17 14:37:19+02	2020-06-17 14:48:37+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-17 14:48:38+02	2020-06-17 14:49:02+02	games	KnightCap	\N	knightcap-3.6_8
2020-06-17 13:50:55+02	2020-06-17 14:49:08+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-06-17 14:49:02+02	2020-06-17 14:49:32+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-06-17 14:49:32+02	2020-06-17 14:51:31+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-06-17 14:49:09+02	2020-06-17 14:51:38+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-17 14:51:39+02	2020-06-17 14:51:58+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-06-17 14:51:58+02	2020-06-17 14:54:37+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-17 14:54:40+02	2020-06-17 14:55:57+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-17 14:51:31+02	2020-06-17 14:56:10+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-17 14:55:57+02	2020-06-17 14:56:48+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-17 14:56:11+02	2020-06-17 14:57:16+02	games	xblast	\N	xblast-2.10.4_17
2020-06-17 14:56:49+02	2020-06-17 14:57:43+02	games	xboard	\N	xboard-4.9.1_1
2020-06-17 14:57:16+02	2020-06-17 14:58:05+02	games	xcowsay	\N	xcowsay-1.5
2020-06-17 14:57:43+02	2020-06-17 15:00:30+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-17 14:58:05+02	2020-06-17 15:00:47+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-17 15:00:30+02	2020-06-17 15:01:44+02	graphics	gimp	\N	gimp-2.10.20,2
2020-06-17 15:00:48+02	2020-06-17 15:05:08+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-17 15:07:17+02	2020-06-17 15:10:40+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-17 15:10:41+02	2020-06-17 15:11:51+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-17 15:01:44+02	2020-06-17 15:13:23+02	graphics	krita	\N	krita-4.2.9_4
2020-06-17 15:11:52+02	2020-06-17 15:39:14+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-17 15:39:14+02	2020-06-17 15:40:19+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-17 15:40:21+02	2020-06-17 15:40:45+02	multimedia	libva-glx	\N	libva-glx-2.7.1
2020-06-17 15:40:46+02	2020-06-17 15:41:07+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-10-05 01:52:21+02	2020-10-05 01:52:38+02	databases	gom	\N	gom-0.4
2020-06-17 15:41:07+02	2020-06-17 15:43:25+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-17 15:43:25+02	2020-06-17 15:46:05+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-17 15:13:24+02	2020-06-17 15:48:39+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-17 15:46:05+02	2020-06-17 15:49:14+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-17 15:48:40+02	2020-06-17 15:50:52+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-06-17 15:49:15+02	2020-06-17 15:51:11+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-17 15:51:12+02	2020-06-17 15:52:05+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-17 15:52:05+02	2020-06-17 15:53:54+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-17 15:53:54+02	2020-06-17 15:54:50+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-17 12:54:22+02	2020-06-17 15:55:20+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-06-17 15:50:53+02	2020-06-17 15:55:22+02	net	wireshark	\N	wireshark-3.2.4
2020-06-17 15:54:52+02	2020-06-17 15:55:43+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-17 15:55:21+02	2020-06-17 15:56:04+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-17 15:55:23+02	2020-06-17 15:56:07+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-17 15:55:43+02	2020-06-17 15:56:27+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-17 15:56:05+02	2020-06-17 15:56:50+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-17 15:56:28+02	2020-06-17 15:57:21+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-17 15:56:07+02	2020-06-17 15:57:22+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-17 15:57:22+02	2020-06-17 15:57:39+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-06-17 15:56:51+02	2020-06-17 15:57:45+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-17 15:57:45+02	2020-06-17 15:58:16+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-17 15:57:21+02	2020-06-17 15:58:25+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-17 15:58:16+02	2020-06-17 15:58:49+02	print	gv	\N	gv-3.7.4_5
2020-06-17 15:57:39+02	2020-06-17 16:00:39+02	print	auctex	full	auctex-emacs26-12.2
2020-06-17 15:58:26+02	2020-06-17 16:01:09+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-17 16:00:40+02	2020-06-17 16:01:32+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-06-17 16:01:10+02	2020-06-17 16:02:08+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-06-17 16:01:32+02	2020-06-17 16:02:11+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-17 16:02:12+02	2020-06-17 16:03:03+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-17 15:58:49+02	2020-06-17 16:04:00+02	print	texlive-full	\N	texlive-full-20150521
2020-06-17 16:03:05+02	2020-06-17 16:04:05+02	x11	xlockmore	\N	xlockmore-5.63
2020-06-17 16:04:01+02	2020-06-17 16:06:02+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-17 16:06:02+02	2020-06-17 16:07:06+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-17 16:07:06+02	2020-06-17 16:07:18+02	x11-fm	xdiskusage	\N	xdiskusage-1.51_1
2020-06-17 16:07:19+02	2020-06-17 16:07:51+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-17 16:04:08+02	2020-06-17 16:10:35+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-17 16:10:42+02	2020-06-17 16:11:04+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-17 16:11:06+02	2020-06-17 16:13:52+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-17 16:02:08+02	2020-06-17 16:43:20+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-17 16:07:51+02	2020-06-17 16:53:39+02	editors	vscode	\N	vscode-1.46.0
2020-06-18 07:19:28+02	2020-06-18 07:20:03+02	devel	swig	\N	swig-4.0.1
2020-06-18 07:19:28+02	2020-06-18 07:20:22+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.71.0_2
2020-06-18 07:20:04+02	2020-06-18 07:20:30+02	textproc	asciidoc	\N	asciidoc-9.0.0
2020-06-18 07:20:22+02	2020-06-18 07:21:10+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.71.0
2020-06-18 07:21:11+02	2020-06-18 07:21:57+02	archivers	kf5-karchive	\N	kf5-karchive-5.71.0
2020-06-18 07:20:30+02	2020-06-18 07:21:57+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-06-18 07:21:57+02	2020-06-18 07:22:59+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-06-18 07:21:57+02	2020-06-18 07:23:09+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-06-18 07:23:09+02	2020-06-18 07:24:31+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-06-18 07:22:59+02	2020-06-18 07:24:52+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-06-18 07:24:33+02	2020-06-18 07:25:23+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-06-18 07:24:52+02	2020-06-18 07:25:35+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-06-18 07:25:24+02	2020-06-18 07:25:59+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.71.0
2020-06-18 07:25:35+02	2020-06-18 07:26:35+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-06-18 07:26:00+02	2020-06-18 07:27:17+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-06-18 07:26:36+02	2020-06-18 07:27:40+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-06-18 07:27:41+02	2020-06-18 07:28:46+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-06-18 07:27:17+02	2020-06-18 07:28:50+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.71.0
2020-06-18 07:28:47+02	2020-06-18 07:29:40+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-06-18 07:28:51+02	2020-06-18 07:29:59+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-06-18 07:29:41+02	2020-06-18 07:30:47+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-06-18 07:29:59+02	2020-06-18 07:30:47+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-06-18 07:30:48+02	2020-06-18 07:31:29+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-18 07:30:48+02	2020-06-18 07:31:53+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-06-18 07:31:30+02	2020-06-18 07:32:53+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-06-18 07:32:54+02	2020-06-18 07:33:08+02	devel	py-more-itertools	py37	py37-more-itertools-8.4.0
2020-06-18 07:31:53+02	2020-06-18 07:33:14+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-06-18 07:33:09+02	2020-06-18 07:35:04+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-06-18 07:35:06+02	2020-06-18 07:35:24+02	devel	py-zipp	py37	py37-zipp-0.6.0_3
2020-06-18 07:33:15+02	2020-06-18 07:35:26+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-06-18 07:35:25+02	2020-06-18 07:36:17+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-06-18 07:35:27+02	2020-06-18 07:36:26+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-06-18 07:45:52+02	2020-06-18 07:46:01+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-0.23
2020-06-18 07:46:02+02	2020-06-18 07:46:15+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-06-18 07:46:14+02	2020-06-18 07:46:35+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.12.3
2020-06-18 07:45:52+02	2020-06-18 07:46:47+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-06-18 07:46:37+02	2020-06-18 07:46:52+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-06-18 07:46:15+02	2020-06-18 07:46:53+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0
2020-06-18 07:46:48+02	2020-06-18 07:47:01+02	databases	pgtop	\N	pgtop-0.05_5
2020-06-18 07:46:54+02	2020-06-18 07:47:12+02	sysutils	devcpu-data	\N	devcpu-data-1.34
2020-06-18 07:47:02+02	2020-06-18 07:47:27+02	www	py-dj22-drf-yasg	py37	py37-dj22-drf-yasg-1.17.0
2020-06-18 07:46:53+02	2020-06-18 07:48:16+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-18 07:48:16+02	2020-06-18 07:48:24+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-18 07:52:47+02	2020-06-18 07:53:06+02	graphics	lcms2	\N	lcms2-2.11
2020-06-18 07:53:06+02	2020-06-18 07:53:32+02	graphics	openjpeg	\N	openjpeg-2.3.1
2020-06-18 07:53:33+02	2020-06-18 07:53:59+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-18 07:53:06+02	2020-06-18 07:54:14+02	graphics	colord	\N	colord-1.3.5
2020-06-18 08:03:38+02	2020-06-18 08:03:58+02	graphics	openjpeg15	\N	openjpeg15-1.5.2_1
2020-06-18 08:03:38+02	2020-06-18 08:04:35+02	graphics	libraw	\N	libraw-0.19.5
2020-06-18 08:03:59+02	2020-06-18 08:04:57+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-18 08:05:01+02	2020-06-18 08:06:55+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-18 08:03:38+02	2020-06-18 08:07:24+02	x11-toolkits	gtk30	\N	gtk3-3.24.10_1
2020-06-18 08:04:35+02	2020-06-18 08:07:25+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-18 08:07:26+02	2020-06-18 08:08:06+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-18 08:08:07+02	2020-06-18 08:09:44+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-18 08:06:56+02	2020-06-18 08:10:42+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-18 08:09:45+02	2020-06-18 08:11:00+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-18 08:11:00+02	2020-06-18 08:12:03+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-18 08:07:25+02	2020-06-18 08:12:54+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-18 08:12:03+02	2020-06-18 08:13:04+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-18 08:12:56+02	2020-06-18 08:13:27+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-18 08:13:05+02	2020-06-18 08:13:54+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-18 08:13:28+02	2020-06-18 08:14:40+02	security	gcr	\N	gcr-3.28.0
2020-06-18 08:13:54+02	2020-06-18 08:14:41+02	net	remmina	\N	remmina-1.3.9
2020-06-18 08:14:42+02	2020-06-18 08:15:31+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-18 08:14:40+02	2020-06-18 08:18:37+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-18 08:18:37+02	2020-06-18 08:19:29+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-06-18 08:15:32+02	2020-06-18 08:20:07+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-18 08:20:07+02	2020-06-18 08:25:00+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-06-18 08:19:29+02	2020-06-18 08:25:09+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-06-18 08:25:00+02	2020-06-18 08:26:10+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-06-18 08:26:11+02	2020-06-18 08:26:46+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-18 08:26:46+02	2020-06-18 08:29:04+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-18 08:29:06+02	2020-06-18 08:30:56+02	x11	babl	\N	babl-0.1.78
2020-06-18 08:30:59+02	2020-06-18 08:34:06+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-18 08:10:43+02	2020-06-18 08:34:42+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-18 08:34:44+02	2020-06-18 08:37:36+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-18 08:37:36+02	2020-06-18 08:38:18+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-18 08:25:09+02	2020-06-18 08:38:29+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-06-18 08:38:19+02	2020-06-18 08:39:30+02	graphics	gegl	\N	gegl-0.4.24
2020-06-18 08:38:30+02	2020-06-18 08:39:42+02	graphics	libcdr01	\N	libcdr01-0.1.6
2020-06-18 08:39:31+02	2020-06-18 08:40:49+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-18 08:39:42+02	2020-06-18 08:40:54+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-18 08:36:02+02	2020-06-18 08:41:17+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-18 08:40:55+02	2020-06-18 08:41:45+02	x11	zenity	\N	zenity-3.32.0
2020-06-18 08:40:50+02	2020-06-18 08:42:18+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-06-18 08:41:45+02	2020-06-18 08:43:06+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-06-18 08:41:18+02	2020-06-18 08:45:59+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-18 08:43:07+02	2020-06-18 08:46:52+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-18 08:46:02+02	2020-06-18 08:49:10+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-18 08:42:21+02	2020-06-18 08:49:15+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-06-18 08:49:15+02	2020-06-18 08:49:59+02	graphics	libfreehand	\N	libfreehand-0.1.2_18
2020-06-18 08:50:00+02	2020-06-18 08:50:18+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-06-18 08:50:18+02	2020-06-18 08:50:58+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-18 08:46:54+02	2020-06-18 08:51:36+02	editors	emacs	full	emacs-26.3_4,3
2020-06-18 08:50:58+02	2020-06-18 08:51:54+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-18 08:49:10+02	2020-06-18 08:54:14+02	graphics	gimp-app	\N	gimp-app-2.10.20,1
2020-06-18 08:51:37+02	2020-06-18 08:54:15+02	print	fontforge	\N	fontforge-20190801_2
2020-06-18 08:54:15+02	2020-06-18 08:55:44+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-18 08:51:56+02	2020-06-18 09:00:28+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-18 09:00:31+02	2020-06-18 09:05:54+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-18 09:05:59+02	2020-06-18 09:10:11+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-18 09:10:16+02	2020-06-18 09:13:25+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-06-18 09:13:26+02	2020-06-18 09:16:00+02	x11-wm	marco	\N	marco-1.22.4
2020-06-18 08:55:44+02	2020-06-18 09:16:31+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-18 09:16:39+02	2020-06-18 09:18:20+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-18 09:16:01+02	2020-06-18 09:19:03+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-18 09:19:04+02	2020-06-18 09:20:34+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-18 09:18:21+02	2020-06-18 09:20:38+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-18 09:20:39+02	2020-06-18 09:21:00+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-18 09:21:00+02	2020-06-18 09:21:18+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-06-18 09:21:19+02	2020-06-18 09:22:47+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-18 09:20:35+02	2020-06-18 09:23:00+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-18 08:54:15+02	2020-06-18 09:23:35+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-18 09:23:01+02	2020-06-18 09:25:20+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-18 09:23:41+02	2020-06-18 09:26:32+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-18 09:25:22+02	2020-06-18 09:26:42+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-18 09:22:47+02	2020-06-18 09:27:21+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-18 09:26:43+02	2020-06-18 09:28:33+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-18 09:26:32+02	2020-06-18 09:29:17+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-18 09:29:18+02	2020-06-18 09:31:25+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-18 09:31:26+02	2020-06-18 09:32:08+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-18 09:27:22+02	2020-06-18 09:32:24+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-18 09:28:33+02	2020-06-18 09:33:24+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-18 09:32:08+02	2020-06-18 09:34:12+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-18 09:33:26+02	2020-06-18 09:35:44+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-18 09:32:24+02	2020-06-18 09:36:07+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-06-18 09:35:44+02	2020-06-18 09:36:56+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-18 09:34:13+02	2020-06-18 09:37:15+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-18 09:37:16+02	2020-06-18 09:39:42+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-06-18 09:36:56+02	2020-06-18 09:41:02+02	editors	pluma	\N	pluma-1.24.0
2020-06-18 09:39:42+02	2020-06-18 09:41:25+02	graphics	eom	\N	eom-1.24.0
2020-06-18 09:41:03+02	2020-06-18 09:41:51+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-18 09:41:25+02	2020-06-18 09:42:45+02	graphics	py-gimp	\N	py27-gimp-2.10.20
2020-06-18 09:41:52+02	2020-06-18 10:08:06+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-18 09:36:07+02	2020-06-18 10:11:41+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-18 10:11:42+02	2020-06-18 10:13:07+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-18 09:42:45+02	2020-06-18 10:13:51+02	java	openjdk11	\N	openjdk11-11.0.7+10.2_1
2020-06-18 10:13:52+02	2020-06-18 10:14:39+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-18 10:13:07+02	2020-06-18 10:14:49+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-18 10:14:39+02	2020-06-18 10:15:23+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-18 10:14:49+02	2020-06-18 10:15:33+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-18 10:08:08+02	2020-06-18 10:15:35+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-06-18 10:15:23+02	2020-06-18 10:16:06+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-18 10:15:33+02	2020-06-18 10:16:11+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-18 10:15:35+02	2020-06-18 10:16:14+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-18 10:16:06+02	2020-06-18 10:16:54+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-18 10:16:14+02	2020-06-18 10:16:57+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-18 10:16:12+02	2020-06-18 10:17:17+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-18 10:16:58+02	2020-06-18 10:17:53+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-18 10:16:55+02	2020-06-18 10:17:56+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-18 10:17:18+02	2020-06-18 10:21:47+02	print	lilypond	\N	lilypond-2.20.0
2020-06-18 10:21:48+02	2020-06-18 10:23:20+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-18 10:17:57+02	2020-06-18 10:23:28+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-18 10:17:54+02	2020-06-18 10:23:39+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-18 10:23:40+02	2020-06-18 10:24:56+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-06-18 10:24:56+02	2020-06-18 10:26:22+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-18 10:26:22+02	2020-06-18 10:27:09+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-18 10:27:11+02	2020-06-18 10:28:01+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-18 10:23:21+02	2020-06-18 10:29:58+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-18 10:30:01+02	2020-06-18 10:31:27+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-18 10:28:02+02	2020-06-18 10:32:05+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-18 10:31:28+02	2020-06-18 10:32:46+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-18 10:32:06+02	2020-06-18 10:33:10+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-18 10:32:47+02	2020-06-18 10:36:03+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-18 10:33:10+02	2020-06-18 10:37:53+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-06-18 10:36:03+02	2020-06-18 10:37:54+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-06-18 10:37:57+02	2020-06-18 10:39:45+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-06-18 10:39:46+02	2020-06-18 10:41:23+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-06-18 10:23:29+02	2020-06-18 10:43:22+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-18 10:41:24+02	2020-06-18 10:44:33+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-18 10:44:34+02	2020-06-18 10:45:17+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-06-18 10:43:24+02	2020-06-18 10:46:02+02	deskutils	calibre	\N	calibre-4.18.0
2020-06-18 10:46:03+02	2020-06-18 10:47:10+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-18 10:47:15+02	2020-06-18 11:17:13+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-18 10:45:18+02	2020-06-18 11:17:53+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-18 11:17:53+02	2020-06-18 11:26:02+02	editors	codelite	\N	codelite-12.0_4
2020-06-18 11:26:03+02	2020-06-18 11:27:45+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-18 11:27:46+02	2020-06-18 11:29:17+02	editors	vim	\N	vim-8.2.0869
2020-06-18 11:29:18+02	2020-06-18 11:37:50+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-18 11:37:51+02	2020-06-18 11:45:14+02	emulators	wine	\N	wine-5.0.1,1
2020-06-18 11:45:14+02	2020-06-18 11:54:10+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-18 11:54:11+02	2020-06-18 11:54:56+02	games	xcowsay	\N	xcowsay-1.5
2020-06-18 11:54:57+02	2020-06-18 11:56:38+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-18 11:56:38+02	2020-06-18 11:57:38+02	graphics	gimp	\N	gimp-2.10.20,2
2020-06-18 11:57:39+02	2020-06-18 12:00:55+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-18 11:17:15+02	2020-06-18 12:09:25+02	devel	ghidra	\N	ghidra-9.1
2020-06-18 12:12:04+02	2020-06-18 13:21:12+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-06-18 13:21:14+02	2020-06-18 13:24:26+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-18 12:00:56+02	2020-06-18 13:38:05+02	graphics	krita	\N	krita-4.3.0
2020-06-18 13:38:07+02	2020-06-18 13:39:19+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-18 13:39:22+02	2020-06-18 13:41:10+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-18 13:41:11+02	2020-06-18 13:42:10+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-18 13:42:10+02	2020-06-18 13:43:09+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-18 13:43:10+02	2020-06-18 13:44:03+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-18 13:44:05+02	2020-06-18 13:44:54+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-18 13:44:54+02	2020-06-18 13:45:46+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-18 13:45:47+02	2020-06-18 13:46:39+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-18 13:46:39+02	2020-06-18 13:47:35+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-18 10:37:54+02	2020-06-18 13:48:18+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-06-18 13:47:37+02	2020-06-18 13:48:28+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-18 13:48:28+02	2020-06-18 13:49:46+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-18 13:48:19+02	2020-06-18 13:50:15+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-18 13:49:47+02	2020-06-18 13:51:00+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-18 13:50:15+02	2020-06-18 13:54:24+02	print	auctex	full	auctex-emacs26-12.2
2020-06-18 13:54:25+02	2020-06-18 13:55:21+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-06-18 13:51:02+02	2020-06-18 13:56:26+02	print	texlive-full	\N	texlive-full-20150521
2020-06-18 13:24:29+02	2020-06-18 14:01:29+02	lang	mono	\N	mono-5.10.1.57_3
2020-06-18 14:01:30+02	2020-06-18 14:01:41+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-18 14:01:41+02	2020-06-18 14:03:08+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-06-18 13:55:22+02	2020-06-18 14:30:52+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-18 13:56:26+02	2020-06-18 14:38:42+02	editors	vscode	\N	vscode-1.46.0
2020-06-19 07:15:20+02	2020-06-19 07:15:37+02	devel	meson	\N	meson-0.54.3
2020-06-19 07:15:38+02	2020-06-19 07:16:04+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-06-19 07:16:05+02	2020-06-19 07:16:47+02	textproc	py-sparqlwrapper	py37	py37-sparqlwrapper-1.8.5
2020-06-19 07:15:20+02	2020-06-19 07:18:22+02	security	nss	\N	nss-3.53.1
2020-06-19 07:18:24+02	2020-06-19 07:19:28+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-06-19 07:16:48+02	2020-06-19 07:22:00+02	textproc	py-rdflib	py37	py37-rdflib-4.2.2_1
2020-06-19 07:15:20+02	2020-06-19 07:26:01+02	devel	glib20	\N	glib-2.56.3_8,1
2020-06-19 07:26:05+02	2020-06-19 07:30:37+02	graphics	cairo	\N	cairo-1.16.0,2
2020-06-19 07:30:40+02	2020-06-19 07:36:16+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1,1
2020-06-19 07:36:19+02	2020-06-19 07:43:14+02	print	harfbuzz	\N	harfbuzz-2.6.7
2020-06-19 07:43:15+02	2020-06-19 07:45:17+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-06-19 07:45:20+02	2020-06-19 07:50:24+02	net	avahi-app	\N	avahi-app-0.7_3
2020-06-19 07:50:27+02	2020-06-19 07:57:28+02	print	cups	\N	cups-2.2.13
2020-06-19 07:57:30+02	2020-06-19 08:03:03+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-19 08:03:05+02	2020-06-19 08:05:42+02	misc	shared-mime-info	\N	shared-mime-info-1.10_2
2020-06-19 08:05:46+02	2020-06-19 08:08:17+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-06-19 08:08:20+02	2020-06-19 08:15:25+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-06-19 08:15:28+02	2020-06-19 08:16:53+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-06-19 08:16:56+02	2020-06-19 08:18:06+02	accessibility	atk	\N	atk-2.36.0
2020-06-19 08:18:09+02	2020-06-19 08:19:29+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-06-19 08:19:33+02	2020-06-19 08:43:04+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-19 08:43:07+02	2020-06-19 08:51:17+02	lang	vala	\N	vala-0.40.15,1
2020-06-19 08:51:19+02	2020-06-19 08:53:06+02	sysutils	polkit	\N	polkit-0.116
2020-06-19 08:53:06+02	2020-06-19 08:55:18+02	devel	libgsf	\N	libgsf-1.14.47
2020-06-19 08:55:19+02	2020-06-19 08:56:44+02	textproc	libcroco	\N	libcroco-0.6.13
2020-06-19 08:56:47+02	2020-06-19 08:59:34+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-19 07:22:02+02	2020-06-19 09:04:20+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-06-19 09:04:25+02	2020-06-19 09:06:53+02	devel	libgudev	\N	libgudev-230_1
2020-06-19 08:59:37+02	2020-06-19 09:11:21+02	devel	qt5-core	\N	qt5-core-5.14.2_3
2020-06-19 09:11:27+02	2020-06-19 09:12:12+02	x11	libwacom	\N	libwacom-1.3
2020-06-19 09:12:15+02	2020-06-19 09:13:36+02	x11	libinput	\N	libinput-1.15.5
2020-06-19 09:13:43+02	2020-06-19 09:15:13+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-06-19 09:06:58+02	2020-06-19 09:19:12+02	graphics	colord	\N	colord-1.3.5
2020-06-19 09:15:15+02	2020-06-19 09:24:47+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-19 09:19:16+02	2020-06-19 09:30:29+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-19 09:30:29+02	2020-06-19 09:35:09+02	net	qt5-network	\N	qt5-network-5.14.2
2020-06-19 09:35:11+02	2020-06-19 09:37:42+02	devel	qt5-dbus	\N	qt5-dbus-5.14.2
2020-06-19 09:37:45+02	2020-06-19 09:51:05+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-19 09:51:11+02	2020-06-19 09:58:33+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-19 09:58:36+02	2020-06-19 09:59:47+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-06-19 09:59:50+02	2020-06-19 10:01:24+02	databases	qt5-sql	\N	qt5-sql-5.14.2
2020-06-19 09:24:50+02	2020-06-19 10:02:13+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-06-19 10:01:25+02	2020-06-19 10:03:02+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.7_2
2020-06-19 10:02:15+02	2020-06-19 10:03:34+02	devel	qt5-testlib	\N	qt5-testlib-5.14.2
2020-06-19 10:03:35+02	2020-06-19 10:06:08+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-19 10:06:12+02	2020-06-19 10:09:56+02	devel	dconf	\N	dconf-0.36.0
2020-06-19 10:03:07+02	2020-06-19 10:10:13+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.14.2
2020-06-19 10:09:59+02	2020-06-19 10:12:41+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-06-19 10:10:15+02	2020-06-19 10:14:30+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-19 10:14:32+02	2020-06-19 10:15:47+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-06-19 10:15:48+02	2020-06-19 10:16:24+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-06-19 10:12:42+02	2020-06-19 10:18:00+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-06-19 10:16:24+02	2020-06-19 10:18:02+02	textproc	qt5-xml	\N	qt5-xml-5.14.2
2020-06-19 10:18:01+02	2020-06-19 10:19:07+02	devel	libsoup	\N	libsoup-2.62.3
2020-06-19 10:18:02+02	2020-06-19 10:21:33+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-19 10:19:09+02	2020-06-19 10:21:40+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.14.2
2020-06-19 10:21:36+02	2020-06-19 10:26:51+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-19 10:26:52+02	2020-06-19 10:27:52+02	devel	json-glib	\N	json-glib-1.4.4
2020-06-19 10:27:54+02	2020-06-19 10:28:37+02	devel	libnotify	\N	libnotify-0.7.8
2020-06-19 10:28:38+02	2020-06-19 10:29:32+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-19 10:29:33+02	2020-06-19 10:31:37+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.14.2
2020-06-19 10:21:43+02	2020-06-19 10:32:26+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-19 10:31:37+02	2020-06-19 10:35:29+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-06-19 10:35:30+02	2020-06-19 10:36:46+02	audio	lv2	\N	lv2-1.18.0
2020-06-19 10:36:47+02	2020-06-19 10:38:20+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-06-19 10:38:23+02	2020-06-19 10:41:14+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-06-19 10:41:18+02	2020-06-19 10:42:51+02	graphics	frei0r	\N	frei0r-1.6.1
2020-06-19 10:42:53+02	2020-06-19 10:46:13+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-06-19 10:46:14+02	2020-06-19 10:58:17+02	multimedia	ffmpeg	\N	ffmpeg-4.3,1
2020-06-19 10:58:17+02	2020-06-19 10:59:31+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-19 10:59:32+02	2020-06-19 11:00:21+02	security	libsecret	\N	libsecret-0.18.6_1
2020-06-19 11:00:21+02	2020-06-19 11:01:16+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.71.0_2
2020-06-19 11:01:16+02	2020-06-19 11:01:52+02	devel	qt5-concurrent	\N	qt5-concurrent-5.14.2
2020-06-19 11:01:53+02	2020-06-19 11:03:22+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-19 10:32:27+02	2020-06-19 11:03:45+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-19 11:03:23+02	2020-06-19 11:03:59+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-19 11:03:45+02	2020-06-19 11:06:13+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-19 11:06:14+02	2020-06-19 11:06:29+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-06-19 11:06:29+02	2020-06-19 11:06:52+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_1
2020-06-19 11:06:52+02	2020-06-19 11:07:18+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_1
2020-06-19 11:07:19+02	2020-06-19 11:07:54+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-19 11:07:54+02	2020-06-19 11:08:40+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-06-19 07:19:30+02	2020-06-19 11:08:43+02	lang	rust	\N	rust-1.44.1_1
2020-06-19 11:08:45+02	2020-06-19 11:09:40+02	net	geoclue	\N	geoclue-2.5.5
2020-06-19 11:04:00+02	2020-06-19 11:09:40+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-19 11:08:40+02	2020-06-19 11:09:50+02	graphics	graphene	\N	graphene-1.10.0
2020-06-19 11:09:41+02	2020-06-19 11:10:13+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-06-19 11:09:40+02	2020-06-19 11:10:14+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-06-19 11:10:13+02	2020-06-19 11:10:33+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-06-19 11:10:14+02	2020-06-19 11:10:34+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-06-19 11:10:33+02	2020-06-19 11:10:51+02	print	libraqm	\N	libraqm-0.6.0
2020-06-19 11:09:51+02	2020-06-19 11:10:59+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-06-19 11:10:52+02	2020-06-19 11:14:18+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-19 11:10:35+02	2020-06-19 11:15:16+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-19 11:14:21+02	2020-06-19 11:15:48+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-19 11:15:17+02	2020-06-19 11:16:03+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-19 11:15:48+02	2020-06-19 11:16:23+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.71.0
2020-06-19 11:16:04+02	2020-06-19 11:17:02+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-19 11:16:24+02	2020-06-19 11:17:43+02	devel	qt5-script	\N	qt5-script-5.14.2
2020-06-19 11:17:03+02	2020-06-19 11:20:39+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-19 11:17:45+02	2020-06-19 11:23:13+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-19 11:20:40+02	2020-06-19 11:23:24+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-19 11:23:14+02	2020-06-19 11:24:26+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-06-19 11:23:25+02	2020-06-19 11:25:02+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-06-19 11:11:00+02	2020-06-19 11:25:23+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-19 11:24:27+02	2020-06-19 11:25:33+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-19 11:25:03+02	2020-06-19 11:25:36+02	sysutils	policykit	\N	policykit-0.9_10
2020-06-19 11:25:23+02	2020-06-19 11:25:57+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-06-19 11:25:34+02	2020-06-19 11:26:02+02	archivers	kf5-karchive	\N	kf5-karchive-5.71.0
2020-06-19 11:25:36+02	2020-06-19 11:26:50+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-06-19 11:25:57+02	2020-06-19 11:26:58+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-19 11:26:52+02	2020-06-19 11:27:22+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-19 11:26:02+02	2020-06-19 11:27:42+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-06-19 11:27:22+02	2020-06-19 11:28:21+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-06-19 11:26:59+02	2020-06-19 11:28:26+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-19 11:27:43+02	2020-06-19 11:28:44+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-19 11:28:27+02	2020-06-19 11:29:00+02	sysutils	hal	\N	hal-0.5.14_34
2020-06-19 11:28:21+02	2020-06-19 11:29:04+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-19 11:29:05+02	2020-06-19 11:29:26+02	comms	qt5-serialport	\N	qt5-serialport-5.14.2
2020-06-19 11:28:45+02	2020-06-19 11:29:26+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-06-19 11:29:26+02	2020-06-19 11:29:53+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-19 11:29:00+02	2020-06-19 11:30:02+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-19 11:29:27+02	2020-06-19 11:30:24+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-06-19 11:30:25+02	2020-06-19 11:31:45+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-06-19 11:30:03+02	2020-06-19 11:32:16+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-06-19 11:29:54+02	2020-06-19 11:32:16+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-06-19 11:31:45+02	2020-06-19 11:32:39+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-06-19 11:32:17+02	2020-06-19 11:32:54+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-19 11:32:17+02	2020-06-19 11:33:08+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-19 11:32:55+02	2020-06-19 11:33:19+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.71.0
2020-06-19 11:32:39+02	2020-06-19 11:33:23+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-19 11:33:08+02	2020-06-19 11:33:49+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-19 11:33:23+02	2020-06-19 11:34:12+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-06-19 11:33:20+02	2020-06-19 11:35:14+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-19 11:33:49+02	2020-06-19 11:35:17+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-06-19 11:35:17+02	2020-06-19 11:35:59+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-06-19 11:34:13+02	2020-06-19 11:36:07+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-19 11:35:15+02	2020-06-19 11:36:48+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-06-19 11:35:59+02	2020-06-19 11:37:23+02	net	remmina	\N	remmina-1.3.9
2020-06-19 11:36:49+02	2020-06-19 11:37:25+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-06-19 11:36:08+02	2020-06-19 11:38:12+02	security	gcr	\N	gcr-3.28.0
2020-06-19 11:37:24+02	2020-06-19 11:38:25+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-06-19 11:38:13+02	2020-06-19 11:39:00+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-19 11:38:26+02	2020-06-19 11:40:30+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-06-19 11:39:02+02	2020-06-19 11:41:23+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-06-19 11:41:25+02	2020-06-19 11:44:07+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-06-19 11:37:26+02	2020-06-19 11:44:45+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-19 11:44:08+02	2020-06-19 11:45:03+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-06-19 11:40:32+02	2020-06-19 11:45:13+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.71.0
2020-06-19 11:45:14+02	2020-06-19 11:45:37+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-06-19 11:44:46+02	2020-06-19 11:46:05+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-19 11:45:38+02	2020-06-19 11:46:27+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-06-19 11:46:08+02	2020-06-19 11:46:38+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-06-19 11:46:28+02	2020-06-19 11:46:54+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-06-19 11:46:39+02	2020-06-19 11:47:08+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-06-19 11:45:03+02	2020-06-19 11:47:33+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-19 11:46:55+02	2020-06-19 11:47:34+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-19 11:47:09+02	2020-06-19 11:47:44+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-06-19 11:47:34+02	2020-06-19 11:47:50+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-06-19 11:47:34+02	2020-06-19 11:48:09+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-06-19 11:47:45+02	2020-06-19 11:48:14+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-06-19 11:47:51+02	2020-06-19 11:48:35+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-19 11:48:09+02	2020-06-19 11:49:03+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-06-19 11:48:35+02	2020-06-19 11:49:11+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-19 11:49:03+02	2020-06-19 11:49:29+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-06-19 11:49:12+02	2020-06-19 11:50:06+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-19 11:48:14+02	2020-06-19 11:50:08+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-06-19 11:49:29+02	2020-06-19 11:50:10+02	devel	libgtop	\N	libgtop-2.40.0
2020-06-19 11:50:11+02	2020-06-19 11:50:41+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-19 11:50:07+02	2020-06-19 11:51:31+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-19 11:51:31+02	2020-06-19 11:52:21+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-06-19 11:52:21+02	2020-06-19 11:53:43+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-06-19 11:50:42+02	2020-06-19 11:54:10+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-19 11:53:43+02	2020-06-19 11:54:28+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-06-19 11:50:09+02	2020-06-19 11:54:42+02	math	vtk8	\N	vtk8-8.2.0
2020-06-19 11:54:10+02	2020-06-19 11:54:56+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-19 11:54:56+02	2020-06-19 11:55:08+02	devel	jansson	\N	jansson-2.13.1
2020-06-19 11:54:29+02	2020-06-19 11:55:46+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-19 11:55:08+02	2020-06-19 11:56:13+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-19 11:55:47+02	2020-06-19 11:56:19+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-06-19 11:56:13+02	2020-06-19 11:56:59+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-19 11:56:20+02	2020-06-19 11:58:04+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-19 11:57:00+02	2020-06-19 11:59:04+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-06-19 11:58:05+02	2020-06-19 11:59:11+02	sysutils	upower	\N	upower-0.99.11
2020-06-19 11:59:12+02	2020-06-19 12:00:05+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-06-19 12:00:05+02	2020-06-19 12:00:23+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-06-19 11:54:43+02	2020-06-19 12:01:29+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-06-19 12:01:31+02	2020-06-19 12:03:16+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-06-19 12:03:21+02	2020-06-19 12:05:13+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-06-19 12:00:24+02	2020-06-19 12:05:15+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-06-19 11:59:05+02	2020-06-19 12:05:15+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-19 12:05:16+02	2020-06-19 12:05:31+02	devel	gamin	\N	gamin-0.1.10_10
2020-06-19 12:05:15+02	2020-06-19 12:05:43+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-06-19 12:05:43+02	2020-06-19 12:06:40+02	devel	librest	\N	rest-0.7.93
2020-06-19 12:05:32+02	2020-06-19 12:07:57+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-06-19 12:06:42+02	2020-06-19 12:09:06+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-19 12:07:59+02	2020-06-19 12:10:35+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-06-19 12:09:08+02	2020-06-19 12:10:40+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-19 12:10:36+02	2020-06-19 12:11:06+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-06-19 12:10:40+02	2020-06-19 12:11:06+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-06-19 12:05:14+02	2020-06-19 12:11:16+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-06-19 12:11:06+02	2020-06-19 12:11:43+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-19 12:11:07+02	2020-06-19 12:11:45+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-19 12:11:43+02	2020-06-19 12:12:19+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-06-19 12:11:45+02	2020-06-19 12:12:21+02	x11	babl	\N	babl-0.1.78
2020-06-19 12:12:22+02	2020-06-19 12:14:14+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-19 12:12:20+02	2020-06-19 12:14:52+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-19 12:14:17+02	2020-06-19 12:17:28+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-06-19 12:17:31+02	2020-06-19 12:20:42+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-06-19 12:14:54+02	2020-06-19 12:21:07+02	x11-fm	caja	\N	caja-1.22.3_1
2020-06-19 12:20:43+02	2020-06-19 12:23:26+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-19 12:21:07+02	2020-06-19 12:23:39+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-06-19 12:23:41+02	2020-06-19 12:24:21+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-19 12:23:29+02	2020-06-19 12:24:25+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-06-19 12:11:16+02	2020-06-19 12:24:45+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-06-19 12:24:45+02	2020-06-19 12:25:00+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-06-19 12:24:23+02	2020-06-19 12:25:47+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-19 12:25:00+02	2020-06-19 12:25:51+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-06-19 12:24:25+02	2020-06-19 12:25:54+02	graphics	gegl	\N	gegl-0.4.24
2020-06-19 12:25:48+02	2020-06-19 12:26:03+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-06-19 12:25:54+02	2020-06-19 12:27:48+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-19 12:27:49+02	2020-06-19 12:28:37+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-06-19 12:28:37+02	2020-06-19 12:29:45+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-19 12:25:52+02	2020-06-19 12:31:02+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-19 12:31:06+02	2020-06-19 12:32:24+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-06-19 12:29:45+02	2020-06-19 12:32:30+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-19 12:32:30+02	2020-06-19 12:33:20+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-19 12:32:25+02	2020-06-19 12:34:14+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-06-19 12:33:21+02	2020-06-19 12:35:13+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-06-19 12:34:15+02	2020-06-19 12:35:47+02	x11	zenity	\N	zenity-3.32.0
2020-06-19 12:35:13+02	2020-06-19 12:36:06+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-19 12:26:03+02	2020-06-19 12:36:11+02	net	samba410	\N	samba410-4.10.15
2020-06-19 12:36:06+02	2020-06-19 12:37:12+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-06-19 12:36:11+02	2020-06-19 12:40:55+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-19 12:35:48+02	2020-06-19 12:43:33+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-06-19 12:37:14+02	2020-06-19 12:43:41+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-06-19 12:41:03+02	2020-06-19 12:43:47+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-19 12:43:33+02	2020-06-19 12:44:01+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-06-19 12:44:01+02	2020-06-19 12:44:22+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-06-19 12:43:42+02	2020-06-19 12:47:26+02	dns	bind-tools	\N	bind-tools-9.16.4
2020-06-19 12:43:47+02	2020-06-19 12:48:30+02	editors	emacs	full	emacs-26.3_4,3
2020-06-19 12:48:31+02	2020-06-19 12:49:19+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-19 12:49:20+02	2020-06-19 12:49:41+02	net	gssdp	\N	gssdp-1.2.2
2020-06-19 12:44:22+02	2020-06-19 12:50:25+02	graphics	gimp-app	\N	gimp-app-2.10.20,1
2020-06-19 12:49:42+02	2020-06-19 12:50:38+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-19 12:50:25+02	2020-06-19 12:50:59+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-19 12:47:27+02	2020-06-19 12:51:20+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-19 12:51:20+02	2020-06-19 12:52:11+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-19 12:50:39+02	2020-06-19 12:53:02+02	print	fontforge	\N	fontforge-20190801_2
2020-06-19 12:52:12+02	2020-06-19 12:54:04+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-19 12:51:00+02	2020-06-19 13:01:28+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-19 13:01:35+02	2020-06-19 13:06:55+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-19 13:06:57+02	2020-06-19 13:13:04+02	x11	xorg	\N	xorg-7.7_3
2020-06-19 12:54:05+02	2020-06-19 13:15:46+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-19 13:13:06+02	2020-06-19 13:16:38+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-19 13:15:49+02	2020-06-19 13:18:02+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-19 13:16:39+02	2020-06-19 13:18:10+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-19 13:18:11+02	2020-06-19 13:19:55+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-19 13:18:03+02	2020-06-19 13:21:00+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-06-19 13:19:58+02	2020-06-19 13:21:47+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-19 13:21:02+02	2020-06-19 13:23:07+02	x11-wm	marco	\N	marco-1.22.4
2020-06-19 13:21:49+02	2020-06-19 13:23:56+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-19 13:23:08+02	2020-06-19 13:24:20+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-19 13:24:21+02	2020-06-19 13:26:02+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-19 13:23:57+02	2020-06-19 13:26:09+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-19 12:53:02+02	2020-06-19 13:26:21+02	www	chromium	\N	chromium-83.0.4103.97
2020-06-19 13:26:03+02	2020-06-19 13:26:24+02	devel	rebar	\N	rebar-2.6.4,1
2020-06-19 13:26:10+02	2020-06-19 13:27:06+02	devel	rebar3	\N	rebar3-3.13.2,1
2020-06-19 13:26:24+02	2020-06-19 13:27:10+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-19 13:27:10+02	2020-06-19 13:27:35+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-19 13:27:35+02	2020-06-19 13:29:00+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-19 13:27:06+02	2020-06-19 13:29:04+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-19 13:26:24+02	2020-06-19 13:29:07+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-19 13:29:01+02	2020-06-19 13:29:26+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-19 13:29:04+02	2020-06-19 13:29:44+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-19 13:29:26+02	2020-06-19 13:30:18+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-19 13:29:07+02	2020-06-19 13:30:58+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-19 13:31:00+02	2020-06-19 13:34:36+02	multimedia	libass	\N	libass-0.14.0
2020-06-19 13:34:39+02	2020-06-19 13:35:57+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-19 13:30:19+02	2020-06-19 13:36:15+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-19 13:36:16+02	2020-06-19 13:36:40+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-19 13:35:58+02	2020-06-19 13:36:53+02	net	freerdp	\N	freerdp-2.1.1
2020-06-19 13:36:45+02	2020-06-19 13:37:26+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-19 13:36:53+02	2020-06-19 13:37:59+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-19 13:37:27+02	2020-06-19 13:38:12+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-19 13:38:13+02	2020-06-19 13:38:31+02	print	libspectre	\N	libspectre-0.2.9
2020-06-19 13:38:32+02	2020-06-19 13:40:57+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-19 13:38:00+02	2020-06-19 13:41:36+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-19 13:45:12+02	2020-06-19 13:45:31+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-06-19 13:45:31+02	2020-06-19 13:45:47+02	sysutils	hal-info	\N	hal-info-20091130
2020-06-19 13:40:58+02	2020-06-19 13:46:20+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-19 13:41:37+02	2020-06-19 13:46:47+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-19 13:46:21+02	2020-06-19 13:47:23+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-19 13:47:24+02	2020-06-19 13:48:16+02	textproc	enchant	\N	enchant-1.6.0_9
2020-06-19 13:45:48+02	2020-06-19 13:48:36+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-19 13:48:17+02	2020-06-19 13:48:52+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-19 13:46:48+02	2020-06-19 13:49:04+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-19 13:48:53+02	2020-06-19 13:49:24+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-06-19 13:48:37+02	2020-06-19 13:49:33+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-19 13:49:05+02	2020-06-19 13:49:37+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-06-19 13:49:34+02	2020-06-19 13:50:18+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-19 13:49:25+02	2020-06-19 13:50:18+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-19 13:49:37+02	2020-06-19 13:50:26+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-19 13:50:18+02	2020-06-19 13:51:16+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-19 13:51:16+02	2020-06-19 13:53:02+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-06-19 13:50:26+02	2020-06-19 13:54:29+02	archivers	engrampa	\N	engrampa-1.24.0
2020-06-19 13:53:04+02	2020-06-19 13:54:54+02	audio	faudio	\N	FAudio-20.06
2020-06-19 13:54:31+02	2020-06-19 13:57:34+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-19 13:54:56+02	2020-06-19 13:57:39+02	databases	rrdtool	\N	rrdtool-1.7.2_3
2020-06-19 13:50:19+02	2020-06-19 13:57:55+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-06-19 13:57:55+02	2020-06-19 13:59:02+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-19 13:57:36+02	2020-06-19 13:59:18+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-19 13:59:22+02	2020-06-19 13:59:44+02	devel	erlang-cl	\N	erlang-cl-1.2.4
2020-06-19 13:59:44+02	2020-06-19 14:00:14+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-06-19 13:59:03+02	2020-06-19 14:00:19+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-19 13:57:40+02	2020-06-19 14:00:39+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-19 14:00:19+02	2020-06-19 14:00:44+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-06-19 14:00:14+02	2020-06-19 14:00:46+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0
2020-06-19 14:00:39+02	2020-06-19 14:05:25+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.2_1
2020-06-19 14:05:25+02	2020-06-19 14:09:19+02	editors	pluma	\N	pluma-1.24.0
2020-06-19 14:09:20+02	2020-06-19 14:18:20+02	graphics	atril	\N	atril-1.22.3_6
2020-06-19 14:18:26+02	2020-06-19 14:20:59+02	graphics	eom	\N	eom-1.24.0
2020-06-19 14:20:59+02	2020-06-19 14:22:21+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-19 14:22:21+02	2020-06-19 14:23:34+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-19 14:23:34+02	2020-06-19 14:25:03+02	graphics	py-gimp	\N	py27-gimp-2.10.20
2020-06-19 14:00:46+02	2020-06-19 14:27:13+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-19 14:00:44+02	2020-06-19 14:30:41+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-19 14:30:42+02	2020-06-19 14:32:23+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-19 14:32:23+02	2020-06-19 14:33:29+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-19 14:25:04+02	2020-06-19 14:33:34+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-19 14:27:15+02	2020-06-19 14:33:41+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-06-19 14:33:35+02	2020-06-19 14:35:35+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-19 14:33:29+02	2020-06-19 14:35:56+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-19 14:35:35+02	2020-06-19 14:36:34+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-19 14:35:58+02	2020-06-19 14:36:54+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-19 14:36:35+02	2020-06-19 14:37:53+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-19 14:36:55+02	2020-06-19 14:38:02+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-19 14:37:53+02	2020-06-19 14:38:46+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-19 14:38:02+02	2020-06-19 14:38:48+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-19 14:33:42+02	2020-06-19 14:38:51+02	multimedia	vlc	\N	vlc-3.0.10_7,4
2020-06-19 14:38:46+02	2020-06-19 14:39:24+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-19 14:38:49+02	2020-06-19 14:39:41+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-19 14:38:51+02	2020-06-19 14:40:02+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-19 14:39:25+02	2020-06-19 14:40:12+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-19 14:40:03+02	2020-06-19 14:40:51+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-06-19 14:39:41+02	2020-06-19 14:40:52+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-19 14:40:12+02	2020-06-19 14:41:18+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-19 14:40:51+02	2020-06-19 14:45:49+02	print	lilypond	\N	lilypond-2.20.0
2020-06-19 14:40:52+02	2020-06-19 14:47:10+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-19 14:41:18+02	2020-06-19 14:47:15+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-19 14:45:50+02	2020-06-19 14:47:32+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-19 14:47:32+02	2020-06-19 14:47:56+02	print	transfig	\N	transfig-3.2.5e_4
2020-06-19 14:47:56+02	2020-06-19 14:49:30+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-06-19 14:49:31+02	2020-06-19 14:51:29+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-19 14:51:30+02	2020-06-19 14:52:31+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-19 14:52:32+02	2020-06-19 14:53:07+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-19 14:53:07+02	2020-06-19 14:54:04+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-19 14:47:11+02	2020-06-19 14:54:16+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-19 14:54:16+02	2020-06-19 14:54:38+02	www	qt5-websockets	\N	qt5-websockets-5.14.2
2020-06-19 14:54:05+02	2020-06-19 14:55:12+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-19 14:55:13+02	2020-06-19 14:57:10+02	x11	mate-base	\N	mate-base-1.24.0
2020-06-19 14:57:11+02	2020-06-19 14:58:56+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-19 14:54:39+02	2020-06-19 14:59:36+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-19 14:58:57+02	2020-06-19 15:00:41+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-19 14:59:37+02	2020-06-19 15:00:56+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-19 15:00:42+02	2020-06-19 15:01:22+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-19 15:00:57+02	2020-06-19 15:02:04+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-19 15:02:05+02	2020-06-19 15:06:37+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-19 15:01:23+02	2020-06-19 15:08:06+02	astro	stellarium	\N	stellarium-0.20.1
2020-06-19 15:08:07+02	2020-06-19 15:10:16+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-06-19 14:47:16+02	2020-06-19 15:10:47+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-19 15:06:37+02	2020-06-19 15:11:51+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-06-19 15:10:48+02	2020-06-19 15:12:40+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-06-19 15:11:53+02	2020-06-19 15:13:42+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-06-19 15:12:40+02	2020-06-19 15:16:52+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-19 15:13:43+02	2020-06-19 15:17:44+02	deskutils	calibre	\N	calibre-4.18.0
2020-06-19 15:16:52+02	2020-06-19 15:17:51+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-06-19 15:17:56+02	2020-06-19 15:19:08+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-19 15:19:09+02	2020-06-19 15:55:46+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-19 15:17:45+02	2020-06-19 15:56:21+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-19 15:55:49+02	2020-06-19 16:03:49+02	dns	bind916	\N	bind916-9.16.4
2020-06-19 15:56:21+02	2020-06-19 16:05:48+02	editors	codelite	\N	codelite-12.0_4
2020-06-19 16:03:52+02	2020-06-19 16:06:28+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-19 16:05:50+02	2020-06-19 16:09:16+02	editors	vim	\N	vim-8.2.0869
2020-06-19 16:06:28+02	2020-06-19 16:23:26+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-19 16:23:33+02	2020-06-19 16:54:49+02	emulators	wine	\N	wine-5.0.1,1
2020-06-19 16:54:49+02	2020-06-19 17:06:09+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-19 17:06:09+02	2020-06-19 17:06:45+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-19 16:09:16+02	2020-06-19 17:06:51+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-06-19 17:06:45+02	2020-06-19 17:07:24+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-19 17:06:51+02	2020-06-19 17:07:45+02	games	xboard	\N	xboard-4.9.1_1
2020-06-19 17:07:24+02	2020-06-19 17:08:14+02	games	xcowsay	\N	xcowsay-1.5
2020-06-19 17:07:46+02	2020-06-19 17:14:04+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-19 17:14:05+02	2020-06-19 17:16:04+02	graphics	gimp	\N	gimp-2.10.20,2
2020-06-19 17:08:15+02	2020-06-19 17:16:28+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-19 17:16:28+02	2020-06-19 17:42:58+02	graphics	krita	\N	krita-4.3.0
2020-06-19 17:43:00+02	2020-06-19 17:55:20+02	graphics	wings	\N	wings-2.2.5
2020-06-19 17:16:05+02	2020-06-19 17:55:40+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-19 17:55:21+02	2020-06-19 17:56:36+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-19 17:55:42+02	2020-06-19 17:56:45+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-06-19 17:56:37+02	2020-06-19 17:57:39+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-19 17:56:46+02	2020-06-19 18:23:25+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.1
2020-06-19 18:23:26+02	2020-06-19 18:24:43+02	misc	mc	\N	mc-nox11-4.8.24
2020-06-19 18:24:43+02	2020-06-19 18:25:58+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-19 18:25:58+02	2020-06-19 18:29:18+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-19 18:29:18+02	2020-06-19 18:32:03+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-19 17:57:40+02	2020-06-19 18:32:49+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-19 18:32:49+02	2020-06-19 18:34:58+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-06-19 18:32:04+02	2020-06-19 18:35:16+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-19 18:34:59+02	2020-06-19 18:36:31+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-19 18:36:33+02	2020-06-19 18:38:18+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-19 18:38:18+02	2020-06-19 18:39:49+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-06-19 18:35:17+02	2020-06-19 18:39:55+02	net	wireshark	\N	wireshark-3.2.4
2020-06-19 15:10:17+02	2020-06-19 18:40:12+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-06-19 18:39:50+02	2020-06-19 18:40:37+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-19 18:39:56+02	2020-06-19 18:40:40+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-19 18:40:12+02	2020-06-19 18:40:51+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-19 18:40:37+02	2020-06-19 18:41:18+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-19 18:40:40+02	2020-06-19 18:41:20+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-19 18:40:52+02	2020-06-19 18:41:31+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-19 18:41:18+02	2020-06-19 18:41:58+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-19 18:41:32+02	2020-06-19 18:42:15+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-19 18:41:20+02	2020-06-19 18:42:30+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-19 18:41:58+02	2020-06-19 18:42:46+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-19 18:42:17+02	2020-06-19 18:43:12+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-19 18:42:46+02	2020-06-19 18:43:13+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-19 18:43:12+02	2020-06-19 18:43:39+02	print	gv	\N	gv-3.7.4_5
2020-06-19 18:42:31+02	2020-06-19 18:44:48+02	print	auctex	full	auctex-emacs26-12.2
2020-06-19 18:44:48+02	2020-06-19 18:44:58+02	security	lynis	\N	lynis-3.0.0
2020-06-19 18:43:14+02	2020-06-19 18:45:07+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-19 18:44:59+02	2020-06-19 18:45:45+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-06-19 18:45:08+02	2020-06-19 18:46:44+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.7
2020-06-19 18:45:45+02	2020-06-19 18:46:46+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-06-19 18:46:45+02	2020-06-19 18:47:22+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-19 18:43:39+02	2020-06-19 18:47:53+02	print	texlive-full	\N	texlive-full-20150521
2020-06-19 18:47:23+02	2020-06-19 18:48:06+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-19 18:47:54+02	2020-06-19 18:49:23+02	x11	mate	\N	mate-1.22.2
2020-06-19 18:48:06+02	2020-06-19 18:49:51+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-19 18:49:52+02	2020-06-19 18:50:53+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-19 18:50:54+02	2020-06-19 18:51:17+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-06-19 18:49:24+02	2020-06-19 18:57:43+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-19 18:57:49+02	2020-06-19 18:58:12+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-19 18:58:16+02	2020-06-19 19:01:44+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-19 19:01:46+02	2020-06-19 19:05:02+02	net-mgmt	netbox	\N	netbox-2.7.12
2020-06-19 18:46:47+02	2020-06-19 19:31:19+02	www	firefox	\N	firefox-77.0.1_1,1
2020-06-19 18:51:17+02	2020-06-19 19:42:04+02	editors	vscode	\N	vscode-1.46.0
2020-06-19 19:57:52+02	2020-06-19 20:15:10+02	local	E590T-localbase	\N	E590T-localbase-2020061700
2020-06-20 01:57:06+02	2020-06-20 01:57:19+02	audio	lv2	\N	lv2-1.18.0
2020-06-20 01:57:06+02	2020-06-20 01:57:45+02	devel	bison	\N	bison-3.6.4,1
2020-06-20 01:57:06+02	2020-06-20 01:58:52+02	dns	c-ares	\N	c-ares-1.16.1
2020-06-20 01:57:45+02	2020-06-20 01:59:46+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-20 01:57:19+02	2020-06-20 02:53:45+02	multimedia	vlc	\N	vlc-3.0.11,4
2020-06-20 01:58:52+02	2020-06-20 03:27:52+02	www	node	\N	node-14.4.0
2020-06-20 03:27:55+02	2020-06-20 03:32:41+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-20 03:32:45+02	2020-06-20 03:36:36+02	security	sudo	\N	sudo-1.9.1
2020-06-20 02:53:48+02	2020-06-20 03:39:56+02	www	node10	\N	node10-10.21.0
2020-06-20 03:40:01+02	2020-06-20 03:40:22+02	sysutils	parallel	\N	parallel-20200422
2020-06-20 03:36:37+02	2020-06-20 03:40:25+02	sysutils	coreutils	\N	coreutils-8.32
2020-06-20 03:40:26+02	2020-06-20 03:40:35+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-06-20 03:40:22+02	2020-06-20 03:40:44+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-06-20 03:40:35+02	2020-06-20 03:40:48+02	x11	sterm	\N	sterm-0.8.4
2020-06-20 03:40:47+02	2020-06-20 03:40:57+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-06-20 03:40:49+02	2020-06-20 03:40:57+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-20 10:29:53+02	2020-06-20 10:31:17+02	www	node12	\N	node12-12.18.1
2020-06-20 10:31:18+02	2020-06-20 10:31:37+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-06-20 10:31:37+02	2020-06-20 10:31:46+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-06-20 10:44:17+02	2020-06-20 10:59:51+02	local	E590T-localbase	\N	E590T-localbase-2020061700
2020-06-20 13:27:21+02	2020-06-20 13:37:38+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-20 13:37:41+02	2020-06-20 13:38:07+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-20 13:27:21+02	2020-06-20 13:40:56+02	deskutils	calibre	\N	calibre-4.19.0
2020-06-20 13:27:21+02	2020-06-20 14:03:03+02	print	scribus-devel	\N	scribus-devel-1.5.5_14
2020-06-20 14:17:45+02	2020-06-20 14:35:10+02	local	E590T-localbase	\N	E590T-localbase-2020061700
2020-06-20 18:52:22+02	2020-06-20 18:52:37+02	devel	pkgconf	\N	pkgconf-1.7.3,1
2020-06-20 18:52:38+02	2020-06-20 18:53:40+02	devel	libgit2	\N	libgit2-1.0.1
2020-06-20 18:52:22+02	2020-06-20 18:54:26+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-20 18:52:37+02	2020-06-20 19:08:24+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-20 19:08:26+02	2020-06-20 19:11:54+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-20 18:54:27+02	2020-06-20 19:14:41+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-20 19:11:56+02	2020-06-20 19:15:16+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-20 19:15:18+02	2020-06-20 19:17:45+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-20 19:17:47+02	2020-06-20 19:21:23+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-20 19:21:25+02	2020-06-20 19:24:45+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-20 19:14:45+02	2020-06-20 19:33:15+02	editors	emacs	full	emacs-26.3_5,3
2020-06-20 19:33:18+02	2020-06-20 19:37:49+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-20 19:37:55+02	2020-06-20 19:44:49+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-20 19:44:53+02	2020-06-20 19:49:54+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-20 19:49:55+02	2020-06-20 19:54:45+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-20 19:54:47+02	2020-06-20 19:58:49+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-20 19:58:51+02	2020-06-20 20:02:31+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-20 20:02:33+02	2020-06-20 20:06:00+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-20 20:06:02+02	2020-06-20 20:09:56+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-20 20:09:58+02	2020-06-20 20:13:38+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-20 20:13:40+02	2020-06-20 20:17:22+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-20 20:17:25+02	2020-06-20 20:20:55+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-20 20:20:59+02	2020-06-20 20:26:22+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-20 20:26:24+02	2020-06-20 20:29:58+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-20 20:30:00+02	2020-06-20 20:33:06+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-20 20:33:10+02	2020-06-20 20:45:49+02	print	auctex	full	auctex-emacs26-12.2
2020-06-20 20:45:52+02	2020-06-20 20:46:30+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-20 19:24:47+02	2020-06-20 21:22:44+02	lang	julia11	\N	julia11-1.1.1_5
2020-06-20 21:22:51+02	2020-06-20 21:24:06+02	lang	julia	\N	julia-1.0.3_3
2020-06-20 18:53:40+02	2020-06-20 22:15:55+02	lang	rust	\N	rust-1.44.1_1
2020-06-20 22:15:56+02	2020-06-20 22:18:48+02	sysutils	exa	\N	exa-0.9.0_11
2020-06-20 22:15:56+02	2020-06-20 22:22:06+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.2_1
2020-06-20 22:15:56+02	2020-06-20 22:25:31+02	textproc	bat	\N	bat-0.15.0_1
2020-06-20 22:44:28+02	2020-06-20 23:00:14+02	local	E590T-localbase	\N	E590T-localbase-2020061700
2020-06-21 10:37:43+02	2020-06-21 10:42:21+02	ports-mgmt	pkg	\N	pkg-1.14.4_1
2020-06-21 11:14:14+02	2020-06-21 11:14:26+02	security	base-audit	\N	base-audit-0.4
2020-06-21 11:14:14+02	2020-06-21 11:24:51+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-21 11:24:54+02	2020-06-21 11:25:09+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-21 11:59:45+02	2020-06-21 12:15:26+02	local	E590T-localbase	\N	E590T-localbase-2020061700
2020-06-25 00:20:05+02	2020-06-25 00:20:49+02	devel	check	\N	check-0.15.0
2020-06-25 00:20:05+02	2020-06-25 00:20:51+02	devel	swig	\N	swig-4.0.2
2020-06-25 00:20:52+02	2020-06-25 00:22:38+02	print	cups	\N	cups-2.3.3_1
2020-06-25 00:20:05+02	2020-06-25 00:22:39+02	ftp	curl	\N	curl-7.71.0
2020-06-25 00:20:49+02	2020-06-25 00:22:59+02	print	harfbuzz	\N	harfbuzz-2.6.8
2020-06-25 00:22:39+02	2020-06-25 00:28:04+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-06-25 00:23:01+02	2020-06-25 00:28:26+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-06-25 00:28:06+02	2020-06-25 00:29:56+02	devel	nspr	\N	nspr-4.26
2020-06-25 00:28:28+02	2020-06-25 00:31:01+02	graphics	lcms2	\N	lcms2-2.11_1
2020-06-25 00:22:39+02	2020-06-25 00:35:14+02	devel	cmake	\N	cmake-3.17.3
2020-06-25 00:33:17+02	2020-06-25 00:35:27+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-06-25 00:35:14+02	2020-06-25 00:35:48+02	graphics	openjpeg	\N	openjpeg-2.3.1
2020-06-25 00:29:59+02	2020-06-25 00:37:22+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-06-25 00:35:49+02	2020-06-25 00:37:35+02	security	nss	\N	nss-3.53.1
2020-06-25 00:37:24+02	2020-06-25 00:46:01+02	lang	vala	\N	vala-0.40.15,1
2020-06-25 00:46:02+02	2020-06-25 00:47:44+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-06-25 00:35:27+02	2020-06-25 00:47:55+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-25 00:47:46+02	2020-06-25 00:50:07+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.8_2
2020-06-25 00:37:35+02	2020-06-25 00:50:22+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-25 00:50:29+02	2020-06-25 00:51:26+02	graphics	openjpeg15	\N	openjpeg15-1.5.2_1
2020-06-25 00:50:08+02	2020-06-25 00:53:31+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-25 00:53:32+02	2020-06-25 00:56:16+02	multimedia	dav1d	\N	dav1d-0.7.1
2020-06-25 00:47:57+02	2020-06-25 00:56:26+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-25 00:51:31+02	2020-06-25 00:57:11+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-06-25 00:57:12+02	2020-06-25 01:02:46+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-06-25 00:56:29+02	2020-06-25 01:16:22+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-25 00:56:17+02	2020-06-25 01:16:56+02	multimedia	ffmpeg	\N	ffmpeg-4.3,1
2020-06-25 01:16:23+02	2020-06-25 01:17:56+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-25 01:17:56+02	2020-06-25 01:18:44+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-25 01:16:58+02	2020-06-25 01:18:57+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-25 01:18:57+02	2020-06-25 01:19:32+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-25 01:18:45+02	2020-06-25 01:19:43+02	graphics	libraw	\N	libraw-0.19.5
2020-06-25 01:02:50+02	2020-06-25 01:19:49+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-25 01:19:32+02	2020-06-25 01:19:57+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-06-25 01:19:43+02	2020-06-25 01:19:58+02	print	libraqm	\N	libraqm-0.6.0
2020-06-25 01:19:57+02	2020-06-25 01:21:09+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-25 01:21:10+02	2020-06-25 01:22:10+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-25 01:19:59+02	2020-06-25 01:22:15+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-25 01:22:15+02	2020-06-25 01:22:34+02	ftp	php73-curl	\N	php73-curl-7.3.19
2020-06-25 01:22:10+02	2020-06-25 01:22:44+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-25 01:22:34+02	2020-06-25 01:23:13+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-25 01:22:45+02	2020-06-25 01:23:54+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-06-25 01:23:13+02	2020-06-25 01:24:21+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-25 01:23:54+02	2020-06-25 01:24:54+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-06-25 01:24:54+02	2020-06-25 01:26:21+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-06-25 01:24:22+02	2020-06-25 01:26:50+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-25 01:26:22+02	2020-06-25 01:27:30+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-06-25 01:27:31+02	2020-06-25 01:29:54+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-25 01:26:51+02	2020-06-25 01:30:29+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-25 01:29:56+02	2020-06-25 01:32:04+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-06-25 01:30:30+02	2020-06-25 01:32:10+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-25 01:32:10+02	2020-06-25 01:33:16+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-06-25 01:32:04+02	2020-06-25 01:33:49+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-06-25 01:33:19+02	2020-06-25 01:34:17+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-06-25 01:33:49+02	2020-06-25 01:34:38+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-25 01:34:18+02	2020-06-25 01:35:23+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-25 01:34:38+02	2020-06-25 01:37:15+02	databases	mysql57-client	\N	mysql57-client-5.7.30_2
2020-06-25 01:35:25+02	2020-06-25 01:37:19+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-06-25 01:37:20+02	2020-06-25 01:38:06+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-06-25 01:38:08+02	2020-06-25 01:40:02+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-25 01:19:50+02	2020-06-25 01:40:08+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-25 01:37:15+02	2020-06-25 01:44:06+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-25 01:40:11+02	2020-06-25 01:45:04+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-25 01:45:04+02	2020-06-25 01:46:16+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-06-25 01:44:07+02	2020-06-25 01:58:04+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-25 01:46:18+02	2020-06-25 02:15:42+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-25 02:15:51+02	2020-06-25 02:19:56+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-25 01:58:11+02	2020-06-25 02:23:23+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-25 02:19:59+02	2020-06-25 02:23:26+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-25 02:23:31+02	2020-06-25 02:30:29+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-06-25 02:30:32+02	2020-06-25 02:33:09+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-25 02:23:31+02	2020-06-25 02:33:27+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-06-25 02:33:27+02	2020-06-25 02:34:01+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-25 02:34:02+02	2020-06-25 02:36:45+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-06-25 02:36:48+02	2020-06-25 02:37:09+02	www	p5-CGI	\N	p5-CGI-4.50
2020-06-25 02:37:09+02	2020-06-25 02:38:22+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-06-25 02:38:27+02	2020-06-25 02:42:58+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-06-25 02:43:01+02	2020-06-25 02:55:46+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-25 02:55:50+02	2020-06-25 02:57:34+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-06-25 02:57:40+02	2020-06-25 03:06:34+02	devel	git	\N	git-2.27.0
2020-06-25 03:06:35+02	2020-06-25 03:09:58+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-25 03:10:02+02	2020-06-25 03:15:22+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-25 03:15:25+02	2020-06-25 03:28:29+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-25 03:28:31+02	2020-06-25 03:31:59+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-06-25 02:33:10+02	2020-06-25 03:48:52+02	math	vtk8	\N	vtk8-8.2.0
2020-06-25 01:40:05+02	2020-06-25 04:45:31+02	lang	rust	\N	rust-1.44.1_1
2020-06-25 04:45:38+02	2020-06-25 04:51:51+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-06-25 04:51:55+02	2020-06-25 04:57:09+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-06-25 04:57:14+02	2020-06-25 05:02:57+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-25 03:48:56+02	2020-06-25 05:23:31+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-06-25 05:23:35+02	2020-06-25 05:26:06+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-25 05:26:08+02	2020-06-25 05:28:01+02	x11	babl	\N	babl-0.1.78
2020-06-25 05:28:04+02	2020-06-25 05:31:59+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-06-25 05:32:01+02	2020-06-25 05:34:56+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-25 05:34:59+02	2020-06-25 05:47:01+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-25 05:47:08+02	2020-06-25 05:51:50+02	graphics	gegl	\N	gegl-0.4.24
2020-06-25 03:32:01+02	2020-06-25 05:52:19+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-25 05:52:20+02	2020-06-25 05:52:36+02	net	liboauth	\N	liboauth-1.0.3_4
2020-06-25 05:51:51+02	2020-06-25 05:52:49+02	graphics	libcdr01	\N	libcdr01-0.1.6
2020-06-25 05:52:36+02	2020-06-25 05:53:21+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-06-25 05:52:50+02	2020-06-25 05:53:36+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-25 05:53:21+02	2020-06-25 05:54:17+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-06-25 05:53:36+02	2020-06-25 05:54:23+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-06-25 05:54:18+02	2020-06-25 05:54:32+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-06-25 05:54:32+02	2020-06-25 05:55:07+02	graphics	libfreehand	\N	libfreehand-0.1.2_18
2020-06-25 05:55:07+02	2020-06-25 05:55:45+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-06-25 05:55:45+02	2020-06-25 05:57:08+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-25 05:57:09+02	2020-06-25 05:57:41+02	net	gssdp	\N	gssdp-1.2.3
2020-06-25 05:57:42+02	2020-06-25 05:58:47+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-25 05:54:23+02	2020-06-25 06:04:07+02	graphics	gimp-app	\N	gimp-app-2.10.20,1
2020-06-25 06:04:08+02	2020-06-25 06:04:45+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-06-25 05:58:49+02	2020-06-25 06:16:44+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-25 06:16:47+02	2020-06-25 06:20:08+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-25 06:20:09+02	2020-06-25 06:22:31+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-25 06:22:32+02	2020-06-25 06:26:55+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-25 06:26:57+02	2020-06-25 06:43:28+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-25 06:43:31+02	2020-06-25 06:47:33+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-06-25 06:47:35+02	2020-06-25 06:48:52+02	graphics	libgxps	\N	libgxps-0.3.1
2020-06-25 06:48:54+02	2020-06-25 06:50:06+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-06-25 06:50:08+02	2020-06-25 06:56:22+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-25 06:56:24+02	2020-06-25 06:57:33+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-06-25 06:57:35+02	2020-06-25 06:59:21+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-25 06:59:23+02	2020-06-25 07:26:07+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-25 07:26:10+02	2020-06-25 07:28:09+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-25 07:28:12+02	2020-06-25 08:06:45+02	math	R	\N	R-4.0.1
2020-06-25 08:06:47+02	2020-06-25 08:09:43+02	multimedia	libass	\N	libass-0.14.0
2020-06-25 08:09:45+02	2020-06-25 08:14:42+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-25 08:14:44+02	2020-06-25 08:18:55+02	net	freerdp	\N	freerdp-2.1.1
2020-06-25 08:18:57+02	2020-06-25 08:20:18+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-25 08:20:21+02	2020-06-25 08:37:36+02	net	libcmis	\N	libcmis-0.5.2_1
2020-06-25 08:37:39+02	2020-06-25 08:39:55+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-25 08:39:59+02	2020-06-25 08:42:26+02	print	libspectre	\N	libspectre-0.2.9
2020-06-25 08:42:32+02	2020-06-25 08:56:09+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-25 08:56:13+02	2020-06-25 09:07:38+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-25 09:07:41+02	2020-06-25 09:12:26+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-06-25 09:12:29+02	2020-06-25 09:16:32+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-25 09:16:35+02	2020-06-25 09:20:53+02	textproc	redland	\N	redland-1.0.17_4
2020-06-25 09:20:55+02	2020-06-25 09:23:05+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-25 09:23:08+02	2020-06-25 09:26:32+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-25 09:26:37+02	2020-06-25 09:28:18+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-06-25 09:28:21+02	2020-06-25 09:35:25+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-25 09:35:28+02	2020-06-25 09:37:22+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-25 09:37:25+02	2020-06-25 09:42:57+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-25 09:42:59+02	2020-06-25 09:44:47+02	audio	faudio	\N	FAudio-20.06
2020-06-25 09:44:50+02	2020-06-25 09:45:50+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.13.0
2020-06-25 09:45:52+02	2020-06-25 09:51:31+02	databases	rrdtool	\N	rrdtool-1.7.2_3
2020-06-25 09:51:32+02	2020-06-25 09:53:32+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-25 09:53:34+02	2020-06-25 10:00:37+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3
2020-06-25 06:04:46+02	2020-06-25 10:03:22+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-25 10:03:23+02	2020-06-25 10:05:19+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-06-25 10:05:19+02	2020-06-25 10:06:28+02	graphics	py-gimp	\N	py27-gimp-2.10.20
2020-06-25 10:06:29+02	2020-06-25 10:13:55+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-25 10:00:37+02	2020-06-25 10:17:52+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-25 10:13:56+02	2020-06-25 10:36:12+02	java	openjdk11	\N	openjdk11-11.0.7+10.2_1
2020-06-25 10:36:12+02	2020-06-25 10:36:38+02	multimedia	gmmlib	\N	gmmlib-20.2.2
2020-06-25 10:36:39+02	2020-06-25 10:39:36+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.4
2020-06-25 10:39:36+02	2020-06-25 10:41:21+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-25 10:17:54+02	2020-06-25 10:42:08+02	lang	julia11	\N	julia11-1.1.1_5
2020-06-25 10:42:09+02	2020-06-25 10:43:16+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-25 10:43:17+02	2020-06-25 10:44:13+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-25 10:41:22+02	2020-06-25 10:49:01+02	multimedia	vlc	\N	vlc-3.0.11,4
2020-06-25 10:44:15+02	2020-06-25 10:51:33+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-25 10:51:34+02	2020-06-25 10:52:56+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-25 10:49:02+02	2020-06-25 10:52:59+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-25 10:52:57+02	2020-06-25 10:59:16+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-25 10:59:17+02	2020-06-25 10:59:39+02	print	transfig	\N	transfig-3.2.5e_4
2020-06-25 10:59:40+02	2020-06-25 11:00:02+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-06-25 11:00:02+02	2020-06-25 11:00:42+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-25 11:00:42+02	2020-06-25 11:00:55+02	www	p5-libwww	\N	p5-libwww-6.46
2020-06-25 11:00:56+02	2020-06-25 11:01:05+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-06-25 11:01:05+02	2020-06-25 11:01:14+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-06-25 11:01:15+02	2020-06-25 11:01:26+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-06-25 11:01:26+02	2020-06-25 11:02:34+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-25 11:02:35+02	2020-06-25 11:03:38+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-25 11:03:39+02	2020-06-25 11:04:10+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-25 11:04:11+02	2020-06-25 11:08:07+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-25 10:52:59+02	2020-06-25 11:09:48+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-25 11:09:54+02	2020-06-25 11:10:32+02	databases	pgtop	\N	pgtop-0.05_5
2020-06-25 11:08:08+02	2020-06-25 11:29:40+02	astro	stellarium	\N	stellarium-0.20.2
2020-06-25 11:10:34+02	2020-06-25 12:05:03+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-25 12:05:04+02	2020-06-25 12:05:15+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-06-25 12:05:16+02	2020-06-25 12:05:34+02	devel	tig	\N	tig-2.5.1
2020-06-25 11:29:42+02	2020-06-25 12:34:18+02	devel	ghidra	\N	ghidra-9.1
2020-06-25 12:05:35+02	2020-06-25 12:38:39+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-06-25 12:34:19+02	2020-06-25 12:39:36+02	emulators	wine	\N	wine-5.0.1,1
2020-06-25 12:38:40+02	2020-06-25 12:40:01+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-25 12:39:37+02	2020-06-25 12:40:04+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-06-25 12:40:01+02	2020-06-25 12:40:34+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-06-25 12:40:05+02	2020-06-25 12:40:52+02	games	xboard	\N	xboard-4.9.1_1
2020-06-25 12:40:35+02	2020-06-25 12:46:40+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-25 12:40:53+02	2020-06-25 12:47:04+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-25 12:46:40+02	2020-06-25 12:48:05+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-25 12:47:05+02	2020-06-25 13:18:48+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-06-25 13:18:49+02	2020-06-25 13:19:18+02	lang	julia	\N	julia-1.0.3_3
2020-06-25 13:19:21+02	2020-06-25 14:26:21+02	lang	mono	\N	mono-5.10.1.57_3
2020-06-25 14:26:22+02	2020-06-25 14:40:14+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.p3
2020-06-25 14:40:15+02	2020-06-25 14:42:40+02	multimedia	openshot	py37	openshot-2.5.1.7
2020-06-25 14:42:43+02	2020-06-25 14:48:49+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-06-25 12:48:05+02	2020-06-25 14:54:34+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200620
2020-06-25 14:54:37+02	2020-06-25 14:57:05+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-25 14:57:07+02	2020-06-25 14:59:18+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-25 14:59:20+02	2020-06-25 15:01:26+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-25 15:01:26+02	2020-06-25 15:02:04+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-25 14:48:52+02	2020-06-25 15:02:38+02	net	wireshark	\N	wireshark-3.2.4
2020-06-25 15:02:05+02	2020-06-25 15:02:42+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-25 15:02:39+02	2020-06-25 15:03:16+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-25 15:02:42+02	2020-06-25 15:03:19+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-25 15:03:16+02	2020-06-25 15:03:55+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-25 15:03:19+02	2020-06-25 15:03:55+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-25 15:03:56+02	2020-06-25 15:04:20+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-25 15:03:56+02	2020-06-25 15:04:28+02	print	gv	\N	gv-3.7.4_5
2020-06-25 15:04:30+02	2020-06-25 15:10:33+02	print	texlive-full	\N	texlive-full-20150521
2020-06-25 15:10:39+02	2020-06-25 15:14:36+02	security	clamav	\N	clamav-0.102.3_1,1
2020-06-25 15:14:36+02	2020-06-25 15:14:43+02	security	doas	\N	doas-6.3
2020-06-25 15:04:20+02	2020-06-25 15:15:07+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-25 15:14:43+02	2020-06-25 15:15:30+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-06-25 15:15:08+02	2020-06-25 15:15:50+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-25 15:15:31+02	2020-06-25 15:15:52+02	sysutils	parallel	\N	parallel-20200622
2020-06-25 15:15:53+02	2020-06-25 15:16:29+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-06-25 15:16:29+02	2020-06-25 15:16:39+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-06-25 15:16:39+02	2020-06-25 15:16:49+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-06-25 15:16:50+02	2020-06-25 15:17:07+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-06-25 15:17:08+02	2020-06-25 15:17:37+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-06-25 15:17:40+02	2020-06-25 15:18:10+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-06-25 15:18:10+02	2020-06-25 15:18:27+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-06-25 15:18:28+02	2020-06-25 15:18:42+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-06-25 15:18:42+02	2020-06-25 15:18:52+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-06-25 15:18:52+02	2020-06-25 15:19:02+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-06-25 15:19:02+02	2020-06-25 15:19:20+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-06-25 15:19:21+02	2020-06-25 15:20:37+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-25 15:15:51+02	2020-06-25 15:22:47+02	textproc	bat	\N	bat-0.15.4
2020-06-25 15:20:38+02	2020-06-25 15:24:18+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-25 15:24:19+02	2020-06-25 15:25:24+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-25 15:25:25+02	2020-06-25 15:25:32+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-25 15:25:33+02	2020-06-25 15:26:14+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-25 15:22:47+02	2020-06-25 15:26:22+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-25 15:26:15+02	2020-06-25 15:26:37+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-06-25 15:26:23+02	2020-06-25 15:29:18+02	print	scribus-devel	\N	scribus-devel-1.5.5_14
2020-06-25 22:19:41+02	2020-06-25 22:20:26+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-25 22:21:03+02	2020-06-25 22:28:56+02	deskutils	calibre	\N	calibre-4.19.0
2020-06-25 22:20:27+02	2020-06-25 22:36:35+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-06-27 00:09:17+02	2020-06-27 00:11:01+02	ports-mgmt	pkg	\N	pkg-1.14.5
2020-06-26 00:18:53+02	2020-06-26 00:24:54+02	cad	ngspice_rework	shlib	ngspice_rework-shlib-32
2020-06-26 00:24:55+02	2020-06-26 00:25:02+02	security	base-audit	\N	base-audit-0.4
2020-06-26 00:19:49+02	2020-06-26 00:29:02+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-26 00:29:03+02	2020-06-26 00:29:11+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-26 00:18:53+02	2020-06-26 00:40:47+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-26 00:45:07+02	2020-06-26 00:46:09+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-26 23:35:36+02	2020-06-26 23:36:14+02	sysutils	polkit	\N	polkit-0.116
2020-06-26 23:35:36+02	2020-06-26 23:36:27+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-06-26 23:35:36+02	2020-06-26 23:36:43+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-26 23:36:43+02	2020-06-26 23:37:14+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-06-26 23:36:27+02	2020-06-26 23:37:32+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-26 23:36:14+02	2020-06-26 23:37:56+02	graphics	colord	\N	colord-1.3.5
2020-06-26 23:37:15+02	2020-06-26 23:38:01+02	sysutils	hal	\N	hal-0.5.14_34
2020-06-26 23:37:32+02	2020-06-26 23:38:30+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-26 23:38:31+02	2020-06-26 23:39:47+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-26 23:38:01+02	2020-06-26 23:40:13+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-06-26 23:40:13+02	2020-06-26 23:41:46+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-06-26 23:41:47+02	2020-06-26 23:43:44+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-06-26 23:43:44+02	2020-06-26 23:45:25+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-06-26 23:37:56+02	2020-06-26 23:49:27+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-06-26 23:45:26+02	2020-06-26 23:50:17+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-06-26 23:39:47+02	2020-06-26 23:50:51+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-26 23:50:18+02	2020-06-26 23:51:37+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-26 23:50:52+02	2020-06-26 23:51:44+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-26 23:51:45+02	2020-06-26 23:52:59+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-26 23:51:38+02	2020-06-26 23:53:16+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-26 23:53:16+02	2020-06-26 23:54:33+02	net	remmina	\N	remmina-1.3.9
2020-06-26 23:53:00+02	2020-06-26 23:55:41+02	security	gcr	\N	gcr-3.28.0
2020-06-26 23:54:33+02	2020-06-26 23:56:03+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-26 23:55:41+02	2020-06-26 23:58:00+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-26 23:58:01+02	2020-06-26 23:59:02+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-06-26 23:56:03+02	2020-06-26 23:59:41+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-26 23:59:03+02	2020-06-26 23:59:48+02	sysutils	upower	\N	upower-0.99.11
2020-06-26 23:59:48+02	2020-06-27 00:01:21+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-06-27 00:01:22+02	2020-06-27 00:02:22+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-06-26 23:59:42+02	2020-06-27 00:03:15+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-06-27 00:03:15+02	2020-06-27 00:04:09+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-27 00:02:22+02	2020-06-27 00:04:10+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-06-27 00:04:11+02	2020-06-27 00:05:16+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-27 00:04:09+02	2020-06-27 00:05:28+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-27 00:05:29+02	2020-06-27 00:08:09+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-06-27 00:05:17+02	2020-06-27 00:09:08+02	x11-fm	caja	\N	caja-1.22.3_1
2020-06-27 00:08:10+02	2020-06-27 00:09:14+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-27 00:09:08+02	2020-06-27 00:10:41+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-06-27 00:10:41+02	2020-06-27 00:11:42+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-06-27 00:09:15+02	2020-06-27 00:11:59+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-27 00:11:59+02	2020-06-27 00:13:03+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-27 00:11:43+02	2020-06-27 00:13:45+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-06-27 00:13:03+02	2020-06-27 00:14:27+02	x11	zenity	\N	zenity-3.32.0
2020-06-27 00:13:46+02	2020-06-27 00:14:43+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-27 00:14:27+02	2020-06-27 00:28:09+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-06-27 00:14:43+02	2020-06-27 00:28:40+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-06-27 00:28:10+02	2020-06-27 00:29:25+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-27 00:29:25+02	2020-06-27 00:30:51+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-27 00:28:43+02	2020-06-27 00:33:02+02	editors	emacs	full	emacs-26.3_5,3
2020-06-27 00:33:02+02	2020-06-27 00:34:14+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-27 00:34:15+02	2020-06-27 00:36:55+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-27 00:30:51+02	2020-06-27 00:39:28+02	print	fontforge	\N	fontforge-20190801_2
2020-06-27 00:39:28+02	2020-06-27 00:40:54+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-27 00:40:55+02	2020-06-27 00:45:17+02	x11	xorg	\N	xorg-7.7_3
2020-06-27 00:45:17+02	2020-06-27 00:47:39+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-06-27 00:47:40+02	2020-06-27 00:49:01+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-27 00:49:02+02	2020-06-27 00:52:25+02	x11-wm	marco	\N	marco-1.22.4
2020-06-27 00:52:27+02	2020-06-27 00:55:05+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-27 00:55:06+02	2020-06-27 00:56:29+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-27 00:56:30+02	2020-06-27 01:01:41+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-27 01:01:42+02	2020-06-27 01:02:42+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-27 01:02:43+02	2020-06-27 01:08:08+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-27 01:08:09+02	2020-06-27 01:11:02+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-27 01:11:03+02	2020-06-27 01:11:24+02	sysutils	hal-info	\N	hal-info-20091130
2020-06-27 01:11:25+02	2020-06-27 01:16:14+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-27 01:16:15+02	2020-06-27 01:17:23+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-27 01:17:24+02	2020-06-27 01:20:55+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-27 01:20:56+02	2020-06-27 01:24:10+02	archivers	engrampa	\N	engrampa-1.24.0
2020-06-27 01:24:11+02	2020-06-27 01:25:39+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-27 01:25:39+02	2020-06-27 01:27:30+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-27 01:27:30+02	2020-06-27 01:32:29+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-27 01:32:35+02	2020-06-27 01:33:35+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-26 23:49:28+02	2020-06-27 02:09:32+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-27 02:09:33+02	2020-06-27 02:46:49+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-27 02:46:49+02	2020-06-27 03:21:03+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-06-27 03:21:04+02	2020-06-27 05:09:34+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-06-27 05:09:35+02	2020-06-27 05:13:13+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-27 05:13:14+02	2020-06-27 05:17:23+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-27 05:17:24+02	2020-06-27 05:19:32+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-27 05:19:33+02	2020-06-27 05:23:23+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-27 05:23:24+02	2020-06-27 05:32:23+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-27 05:32:24+02	2020-06-27 06:06:54+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-06-27 06:06:55+02	2020-06-27 06:12:33+02	editors	pluma	\N	pluma-1.24.0
2020-06-27 06:12:34+02	2020-06-27 06:16:42+02	graphics	atril	\N	atril-1.22.3_6
2020-06-27 06:16:44+02	2020-06-27 06:19:27+02	graphics	eom	\N	eom-1.24.0
2020-06-27 06:19:28+02	2020-06-27 06:21:18+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-27 06:21:19+02	2020-06-27 06:30:42+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-06-27 06:30:43+02	2020-06-27 06:33:01+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-27 06:33:02+02	2020-06-27 06:34:00+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-27 06:34:01+02	2020-06-27 06:35:03+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-27 06:35:04+02	2020-06-27 06:36:01+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-27 06:36:02+02	2020-06-27 06:37:06+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-27 06:37:07+02	2020-06-27 06:38:02+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-27 06:38:02+02	2020-06-27 06:38:59+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-27 06:39:00+02	2020-06-27 06:40:13+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-27 06:40:14+02	2020-06-27 06:41:53+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-27 06:41:54+02	2020-06-27 06:43:29+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-27 06:43:30+02	2020-06-27 07:02:59+02	print	lilypond	\N	lilypond-2.20.0
2020-06-27 07:03:03+02	2020-06-27 07:05:25+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-06-27 07:05:26+02	2020-06-27 07:08:28+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-27 07:08:29+02	2020-06-27 07:09:45+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-27 07:09:46+02	2020-06-27 07:16:58+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-27 07:17:00+02	2020-06-27 07:18:18+02	x11	mate-base	\N	mate-base-1.24.0
2020-06-27 07:18:19+02	2020-06-27 07:19:53+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-27 07:19:54+02	2020-06-27 07:22:15+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-27 07:22:16+02	2020-06-27 07:23:40+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-27 07:23:41+02	2020-06-27 07:54:33+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-27 07:54:34+02	2020-06-27 08:00:24+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-06-27 08:00:25+02	2020-06-27 08:03:12+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-06-27 01:33:35+02	2020-06-27 08:14:14+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-27 08:14:16+02	2020-06-27 08:16:06+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-06-27 08:16:07+02	2020-06-27 08:17:49+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-06-27 08:17:49+02	2020-06-27 08:40:48+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-27 08:40:49+02	2020-06-27 08:41:54+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-06-27 08:41:54+02	2020-06-27 08:43:01+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-27 08:03:13+02	2020-06-27 09:37:46+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-06-27 09:37:47+02	2020-06-27 10:55:55+02	editors	codelite	\N	codelite-12.0_4
2020-06-27 10:56:00+02	2020-06-27 10:58:38+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-27 10:58:39+02	2020-06-27 11:03:35+02	editors	vim	\N	vim-8.2.0869
2020-06-27 11:03:36+02	2020-06-27 12:17:29+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-27 12:17:31+02	2020-06-27 12:35:37+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-27 12:35:38+02	2020-06-27 12:36:42+02	games	xcowsay	\N	xcowsay-1.5
2020-06-27 12:36:42+02	2020-06-27 12:49:59+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-27 12:50:00+02	2020-06-27 12:51:39+02	graphics	gimp	\N	gimp-2.10.20,2
2020-06-27 12:51:39+02	2020-06-27 14:10:15+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-27 14:10:16+02	2020-06-27 16:40:41+02	graphics	krita	\N	krita-4.3.0
2020-06-27 16:40:45+02	2020-06-27 16:46:36+02	graphics	wings	\N	wings-2.2.5
2020-06-27 16:46:36+02	2020-06-27 16:48:20+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-27 16:48:21+02	2020-06-27 17:47:45+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.2
2020-06-27 17:47:46+02	2020-06-27 18:50:52+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-27 18:50:53+02	2020-06-27 18:52:52+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-27 18:52:53+02	2020-06-27 19:03:57+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-27 19:03:58+02	2020-06-27 19:19:01+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-27 19:19:03+02	2020-06-27 19:21:24+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-27 19:21:25+02	2020-06-27 19:22:56+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-27 19:22:57+02	2020-06-27 19:24:26+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-06-27 19:24:27+02	2020-06-27 19:29:02+02	print	auctex	full	auctex-emacs26-12.2
2020-06-27 19:29:04+02	2020-06-27 19:30:32+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-06-27 08:43:02+02	2020-06-27 20:47:48+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-27 20:47:52+02	2020-06-27 20:50:30+02	x11	mate	\N	mate-1.22.2
2020-06-27 20:50:31+02	2020-06-27 21:58:04+02	editors	vscode	\N	vscode-1.46.1
2020-06-27 21:58:06+02	2020-06-27 21:58:17+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-27 19:30:32+02	2020-06-27 23:15:00+02	www	firefox	\N	firefox-78.0_2,1
2020-06-27 23:22:13+02	2020-06-27 23:23:29+02	ports-mgmt	synth	\N	synth-2.08
2020-06-27 23:30:35+02	2020-06-27 23:31:10+02	sysutils	polkit	\N	polkit-0.116
2020-06-27 23:31:11+02	2020-06-27 23:32:05+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-27 23:31:10+02	2020-06-27 23:32:12+02	graphics	colord	\N	colord-1.3.5
2020-06-27 23:32:05+02	2020-06-27 23:32:28+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-06-27 23:30:35+02	2020-06-27 23:32:31+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-27 23:32:29+02	2020-06-27 23:32:54+02	sysutils	hal	\N	hal-0.5.14_34
2020-06-27 23:32:31+02	2020-06-27 23:33:38+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-27 23:32:55+02	2020-06-27 23:34:24+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-06-27 23:33:39+02	2020-06-27 23:34:39+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-27 23:34:39+02	2020-06-27 23:35:28+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-06-27 23:32:12+02	2020-06-27 23:35:58+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-06-27 23:34:24+02	2020-06-27 23:36:19+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-27 23:35:29+02	2020-06-27 23:36:29+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-06-27 23:36:30+02	2020-06-27 23:37:11+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-27 23:36:19+02	2020-06-27 23:37:13+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-27 23:37:14+02	2020-06-27 23:38:26+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-27 23:37:12+02	2020-06-27 23:38:49+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-27 23:38:50+02	2020-06-27 23:39:46+02	net	remmina	\N	remmina-1.3.9
2020-06-27 23:38:27+02	2020-06-27 23:39:58+02	security	gcr	\N	gcr-3.28.0
2020-06-27 23:39:46+02	2020-06-27 23:40:42+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-27 23:39:59+02	2020-06-27 23:40:51+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-06-27 23:35:59+02	2020-06-27 23:41:32+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-27 23:40:42+02	2020-06-27 23:42:02+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-27 23:42:03+02	2020-06-27 23:42:54+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-06-27 23:40:52+02	2020-06-27 23:43:48+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-27 23:43:49+02	2020-06-27 23:44:35+02	sysutils	upower	\N	upower-0.99.11
2020-06-27 23:42:55+02	2020-06-27 23:44:43+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-06-27 23:44:43+02	2020-06-27 23:45:55+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-06-27 23:41:32+02	2020-06-27 23:46:28+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-27 23:45:56+02	2020-06-27 23:46:31+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-06-27 23:44:35+02	2020-06-27 23:46:45+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-06-27 23:46:32+02	2020-06-27 23:47:56+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-06-27 23:46:46+02	2020-06-27 23:47:58+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-27 23:47:59+02	2020-06-27 23:50:39+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-06-27 23:47:59+02	2020-06-27 23:50:43+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-27 23:46:29+02	2020-06-27 23:51:09+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-06-27 23:50:44+02	2020-06-27 23:51:46+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-06-27 23:50:40+02	2020-06-27 23:51:59+02	x11-fm	caja	\N	caja-1.22.3_1
2020-06-27 23:51:46+02	2020-06-27 23:52:28+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-27 23:52:00+02	2020-06-27 23:52:58+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-06-27 23:52:29+02	2020-06-27 23:53:58+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-27 23:52:59+02	2020-06-27 23:54:41+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-27 23:53:59+02	2020-06-27 23:54:43+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-06-27 23:54:44+02	2020-06-27 23:55:40+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-27 23:54:42+02	2020-06-27 23:56:10+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-06-27 23:55:41+02	2020-06-27 23:56:54+02	x11	zenity	\N	zenity-3.32.0
2020-06-27 23:56:10+02	2020-06-27 23:57:14+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-06-27 23:56:55+02	2020-06-27 23:58:33+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-06-27 23:58:34+02	2020-06-28 00:01:03+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-27 23:57:14+02	2020-06-28 00:01:24+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-06-28 00:01:05+02	2020-06-28 00:02:14+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-28 00:02:15+02	2020-06-28 00:03:04+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-28 00:01:25+02	2020-06-28 00:05:34+02	editors	emacs	full	emacs-26.3_5,3
2020-06-27 23:51:10+02	2020-06-28 00:05:37+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-06-28 00:03:04+02	2020-06-28 00:05:45+02	print	fontforge	\N	fontforge-20190801_2
2020-06-28 00:05:35+02	2020-06-28 00:06:24+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-28 00:05:38+02	2020-06-28 00:07:01+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-28 00:06:25+02	2020-06-28 00:07:16+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-28 00:07:17+02	2020-06-28 00:08:16+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-28 00:08:17+02	2020-06-28 00:10:09+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-06-28 00:07:01+02	2020-06-28 00:10:19+02	x11	xorg	\N	xorg-7.7_3
2020-06-28 00:10:10+02	2020-06-28 00:13:12+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-28 00:10:22+02	2020-06-28 00:13:48+02	x11-wm	marco	\N	marco-1.22.4
2020-06-28 00:13:51+02	2020-06-28 00:18:11+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-28 00:13:13+02	2020-06-28 00:20:31+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-28 00:18:13+02	2020-06-28 00:28:50+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-28 00:20:33+02	2020-06-28 00:28:52+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-28 00:28:53+02	2020-06-28 00:31:10+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-28 00:31:11+02	2020-06-28 00:36:26+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-28 00:28:51+02	2020-06-28 00:38:27+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-28 00:38:28+02	2020-06-28 00:38:47+02	sysutils	hal-info	\N	hal-info-20091130
2020-06-28 00:36:27+02	2020-06-28 00:39:08+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-28 00:39:08+02	2020-06-28 00:40:09+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-28 00:40:10+02	2020-06-28 00:41:36+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-28 00:38:48+02	2020-06-28 00:41:55+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-28 00:41:55+02	2020-06-28 00:44:27+02	archivers	engrampa	\N	engrampa-1.24.0
2020-06-28 00:44:28+02	2020-06-28 00:46:02+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-28 00:46:03+02	2020-06-28 00:47:34+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-28 00:41:36+02	2020-06-28 00:47:54+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-06-28 00:47:55+02	2020-06-28 00:48:52+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-28 00:47:34+02	2020-06-28 00:50:51+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-28 00:50:51+02	2020-06-28 00:54:57+02	editors	pluma	\N	pluma-1.24.0
2020-06-28 00:54:59+02	2020-06-28 00:58:25+02	graphics	atril	\N	atril-1.22.3_6
2020-06-28 00:58:27+02	2020-06-28 01:01:21+02	graphics	eom	\N	eom-1.24.0
2020-06-28 01:01:23+02	2020-06-28 01:02:54+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-28 01:02:56+02	2020-06-28 01:11:30+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-06-28 01:11:31+02	2020-06-28 01:13:22+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-28 01:13:24+02	2020-06-28 01:14:12+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-28 00:48:53+02	2020-06-28 01:14:29+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-28 01:14:12+02	2020-06-28 01:15:02+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-28 01:14:31+02	2020-06-28 01:15:20+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-28 01:15:02+02	2020-06-28 01:15:57+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-28 01:15:21+02	2020-06-28 01:16:09+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-28 01:15:58+02	2020-06-28 01:16:45+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-28 01:16:10+02	2020-06-28 01:17:11+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-28 01:16:46+02	2020-06-28 01:18:11+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-28 01:17:11+02	2020-06-28 01:18:28+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-28 01:18:29+02	2020-06-28 01:20:13+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-06-28 01:20:14+02	2020-06-28 01:22:24+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-28 01:22:25+02	2020-06-28 01:23:33+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-28 01:18:12+02	2020-06-28 01:25:16+02	print	lilypond	\N	lilypond-2.20.0
2020-06-28 01:25:18+02	2020-06-28 01:26:45+02	x11	mate-base	\N	mate-base-1.24.0
2020-06-28 01:26:46+02	2020-06-28 01:28:13+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-28 01:23:34+02	2020-06-28 01:29:33+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-28 01:28:14+02	2020-06-28 01:30:02+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-28 01:29:33+02	2020-06-28 01:30:39+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-28 01:30:03+02	2020-06-28 01:34:06+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-28 01:30:39+02	2020-06-28 01:36:29+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-06-28 01:34:07+02	2020-06-28 01:36:38+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-06-28 01:36:40+02	2020-06-28 01:39:08+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-06-28 01:39:09+02	2020-06-28 01:41:35+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-06-28 01:41:36+02	2020-06-28 01:46:06+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-28 01:46:07+02	2020-06-28 01:47:05+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-06-28 01:47:05+02	2020-06-28 01:48:13+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-28 01:48:13+02	2020-06-28 02:16:15+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-28 02:16:19+02	2020-06-28 02:26:06+02	editors	codelite	\N	codelite-12.0_4
2020-06-28 02:26:07+02	2020-06-28 02:28:16+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-28 02:28:16+02	2020-06-28 02:30:14+02	editors	vim	\N	vim-8.2.0869
2020-06-28 02:30:14+02	2020-06-28 02:39:41+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-28 02:39:43+02	2020-06-28 02:52:28+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-28 02:52:29+02	2020-06-28 02:53:12+02	games	xcowsay	\N	xcowsay-1.5
2020-06-28 02:53:12+02	2020-06-28 02:54:21+02	graphics	gimp	\N	gimp-2.10.20,2
2020-06-28 02:54:21+02	2020-06-28 02:58:20+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-28 02:58:21+02	2020-06-28 03:10:18+02	graphics	krita	\N	krita-4.3.0
2020-06-28 03:10:19+02	2020-06-28 03:14:15+02	graphics	wings	\N	wings-2.2.5
2020-06-28 03:14:16+02	2020-06-28 03:15:25+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-28 01:36:30+02	2020-06-28 03:15:38+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-06-28 03:15:27+02	2020-06-28 03:40:10+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.2
2020-06-28 03:40:11+02	2020-06-28 03:41:07+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-28 03:41:08+02	2020-06-28 03:43:04+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-28 03:43:05+02	2020-06-28 03:44:33+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-28 03:44:34+02	2020-06-28 03:45:23+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-28 03:15:40+02	2020-06-28 03:46:55+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-28 03:46:56+02	2020-06-28 03:47:53+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-06-28 03:45:24+02	2020-06-28 03:48:00+02	print	auctex	full	auctex-emacs26-12.2
2020-06-28 03:48:01+02	2020-06-28 03:49:33+02	x11	mate	\N	mate-1.22.2
2020-06-28 03:49:34+02	2020-06-28 04:38:09+02	editors	vscode	\N	vscode-1.46.1
2020-06-28 04:38:11+02	2020-06-28 04:38:21+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-28 03:47:53+02	2020-06-28 04:41:03+02	www	firefox	\N	firefox-78.0_2,1
2020-06-28 00:05:46+02	2020-06-28 09:09:04+02	www	chromium	\N	chromium-83.0.4103.106
2020-06-28 09:09:05+02	2020-06-28 09:09:54+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-28 09:09:55+02	2020-06-28 09:10:43+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-28 09:10:43+02	2020-06-28 09:11:31+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-28 09:25:26+02	2020-06-28 09:26:07+02	ports-mgmt	synth	\N	synth-2.08
2020-06-28 09:31:50+02	2020-06-28 09:33:31+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-28 09:33:31+02	2020-06-28 09:33:39+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-28 09:31:50+02	2020-06-28 09:47:18+02	local	E590T-localbase	\N	E590T-localbase-2020062500
2020-06-28 11:34:13+02	2020-06-28 11:34:21+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-06-28 11:34:13+02	2020-06-28 11:34:27+02	security	ca_root_nss	\N	ca_root_nss-3.54
2020-06-28 11:34:22+02	2020-06-28 11:34:33+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-06-28 11:34:34+02	2020-06-28 11:34:45+02	www	py-requests	py37	py37-requests-2.22.0
2020-06-28 11:34:45+02	2020-06-28 11:35:09+02	textproc	py-sphinx	py37	py37-sphinx-3.0.4,1
2020-06-28 11:35:09+02	2020-06-28 11:35:22+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-06-28 11:34:28+02	2020-06-28 11:35:49+02	ftp	curl	\N	curl-7.71.0
2020-06-28 11:34:13+02	2020-06-28 11:36:00+02	security	nss	\N	nss-3.54
2020-06-28 11:35:23+02	2020-06-28 11:36:10+02	security	p11-kit	\N	p11-kit-0.23.20
2020-06-28 11:35:50+02	2020-06-28 11:37:41+02	devel	cmake	\N	cmake-3.17.3
2020-06-28 11:37:42+02	2020-06-28 11:38:22+02	graphics	poppler	\N	poppler-0.89.0_1
2020-06-28 11:36:11+02	2020-06-28 11:39:45+02	security	gnutls	\N	gnutls-3.6.14
2020-06-28 11:36:00+02	2020-06-28 11:39:53+02	net	qt5-network	\N	qt5-network-5.14.2
2020-06-28 11:39:46+02	2020-06-28 11:40:22+02	print	cups	\N	cups-2.3.3_1
2020-06-28 11:40:23+02	2020-06-28 11:41:38+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-06-28 11:39:54+02	2020-06-28 11:42:05+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-06-28 11:42:06+02	2020-06-28 11:44:08+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-06-28 11:41:38+02	2020-06-28 11:46:29+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-06-28 11:44:08+02	2020-06-28 11:47:15+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-06-28 11:47:16+02	2020-06-28 11:47:52+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-06-28 11:38:23+02	2020-06-28 11:48:59+02	devel	protobuf	\N	protobuf-3.12.3,1
2020-06-28 11:49:00+02	2020-06-28 11:49:25+02	multimedia	libva	\N	libva-2.8.0
2020-06-28 11:47:53+02	2020-06-28 11:49:29+02	devel	libsoup	\N	libsoup-2.62.3
2020-06-28 11:49:29+02	2020-06-28 11:50:19+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-06-28 11:49:25+02	2020-06-28 11:52:34+02	print	texlive-base	\N	texlive-base-20150521_52
2020-06-28 11:46:31+02	2020-06-28 11:54:15+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.14.2
2020-06-28 11:54:16+02	2020-06-28 11:57:04+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-06-28 11:57:04+02	2020-06-28 11:57:57+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-06-28 11:50:20+02	2020-06-28 11:58:21+02	multimedia	ffmpeg	\N	ffmpeg-4.3,1
2020-06-28 11:57:57+02	2020-06-28 11:58:49+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-06-28 11:58:22+02	2020-06-28 11:58:54+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-06-28 11:58:50+02	2020-06-28 11:59:17+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-06-28 11:59:18+02	2020-06-28 11:59:42+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-06-28 11:58:54+02	2020-06-28 11:59:51+02	net	geoclue	\N	geoclue-2.5.5
2020-06-28 11:59:43+02	2020-06-28 12:02:12+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-06-28 12:02:12+02	2020-06-28 12:03:12+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-06-28 12:03:13+02	2020-06-28 12:03:51+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-06-28 12:03:51+02	2020-06-28 12:04:26+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-28 11:59:51+02	2020-06-28 12:05:01+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-06-28 12:04:26+02	2020-06-28 12:05:10+02	audio	portaudio	\N	portaudio-19.6.0_4,1
2020-06-28 12:05:01+02	2020-06-28 12:05:20+02	ftp	php73-curl	\N	php73-curl-7.3.19
2020-06-28 12:05:11+02	2020-06-28 12:05:49+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-06-28 12:05:50+02	2020-06-28 12:06:09+02	audio	espeak	\N	espeak-1.48.04_7
2020-06-28 12:05:20+02	2020-06-28 12:06:12+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-06-28 12:06:10+02	2020-06-28 12:06:49+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-28 12:06:13+02	2020-06-28 12:06:50+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-06-28 12:06:51+02	2020-06-28 12:07:13+02	textproc	asciidoc	\N	asciidoc-9.0.1
2020-06-28 12:06:49+02	2020-06-28 12:07:41+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-06-28 12:07:13+02	2020-06-28 12:07:42+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-06-28 11:52:35+02	2020-06-28 12:08:09+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-06-28 12:07:42+02	2020-06-28 12:08:28+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-06-28 12:07:42+02	2020-06-28 12:08:33+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-06-28 12:08:33+02	2020-06-28 12:09:31+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-06-28 12:08:09+02	2020-06-28 12:10:08+02	print	tex-formats	\N	tex-formats-20150521_2
2020-06-28 12:09:32+02	2020-06-28 12:10:24+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-06-28 12:10:09+02	2020-06-28 12:10:55+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-06-28 12:10:24+02	2020-06-28 12:10:59+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-06-28 12:10:55+02	2020-06-28 12:11:34+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-06-28 12:10:59+02	2020-06-28 12:11:52+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-06-28 12:11:35+02	2020-06-28 12:12:00+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-06-28 12:12:00+02	2020-06-28 12:12:39+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-06-28 12:11:53+02	2020-06-28 12:12:41+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-06-28 12:08:29+02	2020-06-28 12:12:46+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-06-28 12:12:39+02	2020-06-28 12:13:16+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-06-28 12:12:42+02	2020-06-28 12:13:18+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-06-28 12:13:18+02	2020-06-28 12:13:56+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-06-28 12:13:16+02	2020-06-28 12:14:00+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-06-28 12:13:56+02	2020-06-28 12:14:33+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-06-28 12:12:47+02	2020-06-28 12:15:19+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-06-28 12:14:34+02	2020-06-28 12:15:40+02	databases	mysql57-client	\N	mysql57-client-5.7.30_2
2020-06-28 12:14:01+02	2020-06-28 12:15:45+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-06-28 12:15:20+02	2020-06-28 12:16:06+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-06-28 12:15:41+02	2020-06-28 12:16:32+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-06-28 12:16:06+02	2020-06-28 12:16:53+02	net	remmina	\N	remmina-1.3.9
2020-06-28 12:15:45+02	2020-06-28 12:16:58+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-06-28 12:16:54+02	2020-06-28 12:17:33+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-06-28 12:16:32+02	2020-06-28 12:17:47+02	security	gcr	\N	gcr-3.28.0
2020-06-28 12:17:33+02	2020-06-28 12:18:11+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-28 12:18:12+02	2020-06-28 12:19:33+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-06-28 12:19:33+02	2020-06-28 12:20:36+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-06-28 12:20:36+02	2020-06-28 12:21:46+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-06-28 12:16:58+02	2020-06-28 12:21:55+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-06-28 12:21:56+02	2020-06-28 12:22:39+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-28 12:21:46+02	2020-06-28 12:24:29+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-06-28 12:24:30+02	2020-06-28 12:25:20+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-06-28 12:25:20+02	2020-06-28 12:27:43+02	security	gnupg	\N	gnupg-2.2.20
2020-06-28 12:27:43+02	2020-06-28 12:28:50+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-06-28 12:28:50+02	2020-06-28 12:29:54+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-06-28 12:29:54+02	2020-06-28 12:31:51+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-06-28 12:31:52+02	2020-06-28 12:32:29+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-28 12:32:32+02	2020-06-28 12:37:01+02	net	libmateweather	\N	libmateweather-1.24.0
2020-06-28 12:37:02+02	2020-06-28 12:37:33+02	security	gpgme	\N	gpgme-1.13.1
2020-06-28 12:22:40+02	2020-06-28 12:38:03+02	net	openmpi	\N	openmpi-4.0.4
2020-06-28 12:37:33+02	2020-06-28 12:38:30+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-06-28 12:38:31+02	2020-06-28 12:39:31+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-06-28 12:39:32+02	2020-06-28 12:40:21+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-06-28 12:40:21+02	2020-06-28 12:42:17+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-06-28 12:42:18+02	2020-06-28 12:42:57+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-06-28 12:38:04+02	2020-06-28 12:44:10+02	math	vtk8	\N	vtk8-8.2.0
2020-06-28 12:42:57+02	2020-06-28 12:44:25+02	devel	git	\N	git-2.27.0
2020-06-28 12:44:26+02	2020-06-28 12:45:13+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-06-28 12:45:13+02	2020-06-28 12:46:05+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-06-28 12:46:06+02	2020-06-28 12:47:14+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-06-28 12:47:18+02	2020-06-28 12:49:00+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-06-28 12:49:00+02	2020-06-28 12:49:52+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-06-28 12:49:53+02	2020-06-28 12:50:19+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-06-28 12:44:11+02	2020-06-28 12:50:41+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-06-28 12:50:43+02	2020-06-28 12:51:45+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-06-28 12:51:46+02	2020-06-28 12:57:02+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-06-28 12:57:05+02	2020-06-28 12:58:08+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-06-28 12:50:19+02	2020-06-28 12:59:12+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-28 12:58:09+02	2020-06-28 12:59:25+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-06-28 12:59:26+02	2020-06-28 13:01:35+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-06-28 13:01:38+02	2020-06-28 13:04:38+02	devel	librest	\N	rest-0.7.93
2020-06-28 12:59:13+02	2020-06-28 13:05:21+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-06-28 13:04:38+02	2020-06-28 13:05:29+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-06-28 13:05:30+02	2020-06-28 13:06:01+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-06-28 13:06:01+02	2020-06-28 13:07:48+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-06-28 13:07:49+02	2020-06-28 13:08:44+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-06-28 13:08:45+02	2020-06-28 13:09:40+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-06-28 13:05:22+02	2020-06-28 13:26:49+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-06-28 13:26:51+02	2020-06-28 13:28:08+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-06-28 13:28:09+02	2020-06-28 13:30:27+02	x11-fm	caja	\N	caja-1.22.3_1
2020-06-28 13:30:28+02	2020-06-28 13:31:58+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-06-28 13:31:59+02	2020-06-28 13:32:59+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-06-28 13:32:59+02	2020-06-28 13:34:19+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-06-28 13:34:20+02	2020-06-28 13:35:31+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-06-28 13:35:32+02	2020-06-28 13:36:17+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-06-28 13:36:17+02	2020-06-28 13:38:20+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-06-28 13:38:20+02	2020-06-28 13:40:09+02	graphics	gegl	\N	gegl-0.4.24
2020-06-28 13:40:10+02	2020-06-28 13:44:23+02	mail	mailutils	\N	mailutils-3.9
2020-06-28 13:44:23+02	2020-06-28 13:46:12+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-06-28 13:46:13+02	2020-06-28 13:46:28+02	net	liboauth	\N	liboauth-1.0.3_4
2020-06-28 13:46:28+02	2020-06-28 14:14:27+02	net	samba410	\N	samba410-4.10.15
2020-06-28 14:14:27+02	2020-06-28 14:15:33+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-06-28 14:15:33+02	2020-06-28 14:16:32+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-06-28 14:16:32+02	2020-06-28 14:18:35+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-06-28 14:18:36+02	2020-06-28 14:20:14+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-06-28 14:20:15+02	2020-06-28 14:21:22+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-06-28 14:21:23+02	2020-06-28 14:22:31+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-06-28 14:22:31+02	2020-06-28 14:23:59+02	x11	zenity	\N	zenity-3.32.0
2020-06-28 14:23:59+02	2020-06-28 14:25:53+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-06-28 14:25:54+02	2020-06-28 14:26:54+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-06-28 14:26:55+02	2020-06-28 14:27:09+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-06-28 14:27:10+02	2020-06-28 14:31:12+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-06-28 14:31:12+02	2020-06-28 14:32:56+02	devel	libgdata	\N	libgdata-0.17.9
2020-06-28 14:32:57+02	2020-06-28 14:34:13+02	devel	libpeas	\N	libpeas-1.22.0
2020-06-28 14:34:13+02	2020-06-28 14:34:29+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-06-28 14:34:29+02	2020-06-28 14:39:08+02	editors	emacs	full	emacs-26.3_5,3
2020-06-28 14:39:09+02	2020-06-28 14:39:28+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-06-28 13:09:41+02	2020-06-28 14:40:59+02	www	node	\N	node-14.4.0
2020-06-28 14:39:28+02	2020-06-28 14:45:48+02	graphics	gimp-app	\N	gimp-app-2.10.20,1
2020-06-28 14:40:59+02	2020-06-28 14:46:36+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-28 14:45:49+02	2020-06-28 14:46:40+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-06-28 14:46:40+02	2020-06-28 14:47:29+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-06-28 14:46:57+02	2020-06-28 14:47:38+02	print	cups-filters	\N	cups-filters-1.27.5
2020-06-28 14:47:30+02	2020-06-28 14:50:23+02	print	fontforge	\N	fontforge-20190801_2
2020-06-28 14:50:23+02	2020-06-28 14:51:05+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-06-28 14:51:06+02	2020-06-28 14:52:21+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-28 14:52:21+02	2020-06-28 14:52:43+02	textproc	libebml	\N	libebml-1.4.0
2020-06-28 14:52:43+02	2020-06-28 14:53:13+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-06-28 14:47:38+02	2020-06-28 14:54:15+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-06-28 12:17:47+02	2020-06-28 15:16:21+02	lang	rust	\N	rust-1.44.1_1
2020-06-28 15:16:22+02	2020-06-28 15:35:47+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-28 14:53:14+02	2020-06-28 15:36:15+02	www	chromium	\N	chromium-83.0.4103.106
2020-06-28 15:35:49+02	2020-06-28 15:36:52+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-06-28 15:36:18+02	2020-06-28 15:37:23+02	x11	yelp	\N	yelp-3.28.1_2
2020-06-28 15:36:52+02	2020-06-28 15:37:44+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-06-28 15:37:24+02	2020-06-28 15:38:08+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-06-28 15:37:45+02	2020-06-28 15:38:53+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-06-28 15:38:08+02	2020-06-28 15:39:08+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-06-28 15:38:53+02	2020-06-28 15:39:55+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-06-28 15:39:09+02	2020-06-28 15:40:25+02	x11-wm	marco	\N	marco-1.22.4
2020-06-28 15:39:56+02	2020-06-28 15:41:26+02	audio	libopenshot-audio	\N	libopenshot-audio-0.2.0.10
2020-06-28 15:40:25+02	2020-06-28 15:42:04+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-06-28 15:41:27+02	2020-06-28 15:42:25+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-28 15:42:25+02	2020-06-28 15:44:06+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-28 15:42:05+02	2020-06-28 15:44:13+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-28 15:44:07+02	2020-06-28 15:44:56+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-06-28 15:44:56+02	2020-06-28 15:46:38+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-28 15:44:14+02	2020-06-28 15:46:55+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-06-28 15:46:55+02	2020-06-28 15:47:46+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-06-28 15:46:39+02	2020-06-28 15:47:49+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-06-28 15:47:50+02	2020-06-28 15:48:34+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-06-28 15:47:47+02	2020-06-28 15:50:01+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-06-28 15:50:02+02	2020-06-28 15:57:17+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-06-28 15:57:18+02	2020-06-28 15:58:32+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-06-28 15:58:32+02	2020-06-28 15:59:03+02	multimedia	libmatroska	\N	libmatroska-1.6.0
2020-06-28 15:59:04+02	2020-06-28 16:00:21+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-28 16:00:22+02	2020-06-28 16:01:26+02	net	freerdp	\N	freerdp-2.1.1
2020-06-28 16:01:26+02	2020-06-28 16:01:49+02	net	gupnp	\N	gupnp-1.1.1_1
2020-06-28 14:54:16+02	2020-06-28 16:02:35+02	www	node12	\N	node12-12.18.1
2020-06-28 16:01:49+02	2020-06-28 16:02:43+02	net	libcmis	\N	libcmis-0.5.2_1
2020-06-28 16:02:36+02	2020-06-28 16:03:20+02	net	liveMedia	\N	liveMedia-2020.06.25,2
2020-06-28 16:02:44+02	2020-06-28 16:03:27+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-06-28 16:03:27+02	2020-06-28 16:04:06+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-06-28 16:03:21+02	2020-06-28 16:04:19+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-28 16:04:19+02	2020-06-28 16:04:35+02	print	libspectre	\N	libspectre-0.2.9
2020-06-28 15:48:34+02	2020-06-28 16:04:37+02	math	R	\N	R-4.0.2
2020-06-28 16:04:36+02	2020-06-28 16:06:18+02	print	mftrace	\N	mftrace-1.2.18_2
2020-06-28 16:04:06+02	2020-06-28 16:06:39+02	print	gutenprint	\N	gutenprint-5.3.3
2020-06-28 16:06:39+02	2020-06-28 16:06:47+02	security	py-certifi	py27	py27-certifi-2020.6.20
2020-06-28 16:06:48+02	2020-06-28 16:07:10+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-06-28 16:04:37+02	2020-06-28 16:09:07+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-06-28 16:07:10+02	2020-06-28 16:09:11+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-28 16:09:08+02	2020-06-28 16:09:48+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-06-28 16:09:49+02	2020-06-28 16:10:14+02	textproc	redland	\N	redland-1.0.17_4
2020-06-28 16:09:11+02	2020-06-28 16:10:34+02	textproc	dblatex	\N	dblatex-0.3.11
2020-06-28 16:06:18+02	2020-06-28 16:10:36+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-06-28 16:10:35+02	2020-06-28 16:11:14+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-06-28 16:10:37+02	2020-06-28 16:11:36+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-28 16:11:14+02	2020-06-28 16:11:48+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-06-28 16:11:49+02	2020-06-28 16:12:35+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-06-28 16:11:36+02	2020-06-28 16:12:35+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-06-28 16:12:35+02	2020-06-28 16:13:33+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-06-28 16:12:35+02	2020-06-28 16:13:45+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-06-28 16:13:46+02	2020-06-28 16:16:00+02	archivers	engrampa	\N	engrampa-1.24.0
2020-06-28 16:16:00+02	2020-06-28 16:16:42+02	audio	faudio	\N	FAudio-20.06
2020-06-28 16:16:43+02	2020-06-28 16:17:59+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-28 16:13:34+02	2020-06-28 16:19:00+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-06-28 16:18:00+02	2020-06-28 16:19:14+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-06-28 16:19:15+02	2020-06-28 16:20:04+02	deskutils	mozo	\N	mozo-1.24.0
2020-06-28 16:20:04+02	2020-06-28 16:21:03+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-06-28 16:19:01+02	2020-06-28 16:21:54+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-28 16:21:04+02	2020-06-28 16:22:19+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0_1
2020-06-28 16:22:20+02	2020-06-28 16:22:33+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-06-28 16:21:55+02	2020-06-28 16:25:33+02	devel	protobuf-c	\N	protobuf-c-1.3.2_5
2020-06-28 16:22:33+02	2020-06-28 16:28:41+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3
2020-06-28 16:25:33+02	2020-06-28 16:49:17+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-06-28 16:28:41+02	2020-06-28 16:50:58+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-28 16:49:17+02	2020-06-28 16:53:16+02	editors	pluma	\N	pluma-1.24.0
2020-06-28 16:51:00+02	2020-06-28 16:53:28+02	graphics	atril	\N	atril-1.22.3_6
2020-06-28 16:10:14+02	2020-06-28 16:53:32+02	www	node10	\N	node10-10.21.0
2020-06-28 16:53:33+02	2020-06-28 16:54:16+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-06-28 16:53:16+02	2020-06-28 16:54:35+02	graphics	eom	\N	eom-1.24.0
2020-06-28 16:53:29+02	2020-06-28 16:54:42+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-28 16:54:17+02	2020-06-28 16:55:27+02	graphics	py-gimp	\N	py27-gimp-2.10.20
2020-06-28 16:54:42+02	2020-06-28 17:00:26+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-06-28 17:00:27+02	2020-06-28 17:01:50+02	math	mate-calc	\N	mate-calc-1.24.0
2020-06-28 16:54:35+02	2020-06-28 17:01:54+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-06-28 17:01:55+02	2020-06-28 17:02:32+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-06-28 17:02:32+02	2020-06-28 17:04:31+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-28 17:01:51+02	2020-06-28 17:05:44+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106
2020-06-28 17:05:45+02	2020-06-28 17:06:37+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-28 17:06:38+02	2020-06-28 17:06:47+02	net	py-urllib3	py27	py27-urllib3-1.25.7,1
2020-06-28 17:06:48+02	2020-06-28 17:07:36+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-06-28 17:07:36+02	2020-06-28 17:08:26+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-06-28 17:08:26+02	2020-06-28 17:09:06+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-06-28 17:04:32+02	2020-06-28 17:09:07+02	multimedia	vlc	\N	vlc-3.0.11_1,4
2020-06-28 17:09:07+02	2020-06-28 17:09:45+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-06-28 17:09:06+02	2020-06-28 17:09:48+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-06-28 16:55:28+02	2020-06-28 17:10:03+02	lang	julia11	\N	julia11-1.1.1_5
2020-06-28 17:09:45+02	2020-06-28 17:10:22+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-06-28 17:09:49+02	2020-06-28 17:10:35+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-06-28 17:10:23+02	2020-06-28 17:11:03+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-06-28 17:10:04+02	2020-06-28 17:11:04+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-06-28 17:11:04+02	2020-06-28 17:11:15+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.7.1
2020-06-28 17:10:36+02	2020-06-28 17:11:35+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-28 17:11:04+02	2020-06-28 17:11:58+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-06-28 17:11:15+02	2020-06-28 17:15:52+02	print	lilypond	\N	lilypond-2.20.0
2020-06-28 17:11:35+02	2020-06-28 17:16:33+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-06-28 17:11:58+02	2020-06-28 17:16:44+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-06-28 17:15:53+02	2020-06-28 17:17:21+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-06-28 17:17:22+02	2020-06-28 17:17:42+02	print	transfig	\N	transfig-3.2.5e_4
2020-06-28 17:17:43+02	2020-06-28 17:18:59+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-06-28 17:18:59+02	2020-06-28 17:20:21+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-06-28 17:20:21+02	2020-06-28 17:21:07+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-06-28 17:21:08+02	2020-06-28 17:21:31+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-06-28 17:16:34+02	2020-06-28 17:21:54+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-06-28 17:21:31+02	2020-06-28 17:22:05+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-06-28 17:21:54+02	2020-06-28 17:22:14+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-06-28 17:22:05+02	2020-06-28 17:22:53+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-28 17:22:15+02	2020-06-28 17:23:09+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-28 17:23:10+02	2020-06-28 17:23:21+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-06-28 17:22:53+02	2020-06-28 17:23:28+02	www	qt5-websockets	\N	qt5-websockets-5.14.2
2020-06-28 17:23:29+02	2020-06-28 17:24:26+02	x11	mate-base	\N	mate-base-1.24.0
2020-06-28 17:24:26+02	2020-06-28 17:25:32+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-06-28 17:25:32+02	2020-06-28 17:27:03+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-06-28 17:23:21+02	2020-06-28 17:27:31+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-06-28 17:27:04+02	2020-06-28 17:27:51+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-06-28 17:27:32+02	2020-06-28 17:28:03+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-06-28 17:27:51+02	2020-06-28 17:28:47+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-06-28 17:28:48+02	2020-06-28 17:32:08+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-28 17:28:03+02	2020-06-28 17:33:32+02	astro	stellarium	\N	stellarium-0.20.2
2020-06-28 17:16:44+02	2020-06-28 17:33:42+02	print	texlive-docs	\N	texlive-docs-20150523
2020-06-28 17:33:32+02	2020-06-28 17:35:19+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-06-28 17:32:09+02	2020-06-28 17:36:12+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-06-28 17:35:20+02	2020-06-28 17:36:53+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-06-28 17:36:12+02	2020-06-28 17:37:38+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-06-28 17:37:39+02	2020-06-28 17:40:17+02	deskutils	calibre	\N	calibre-4.19.0
2020-06-28 17:36:54+02	2020-06-28 17:40:49+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-06-28 17:40:18+02	2020-06-28 17:41:21+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-06-28 17:41:21+02	2020-06-28 17:42:52+02	devel	bzr	py27	bzr-2.7.0.6622
2020-06-28 17:42:53+02	2020-06-28 17:44:51+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-06-28 17:40:50+02	2020-06-28 18:07:20+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-28 18:07:21+02	2020-06-28 18:09:29+02	devel	mercurial	py27	mercurial-5.1.2
2020-06-28 18:09:30+02	2020-06-28 18:09:52+02	devel	tig	\N	tig-2.5.1
2020-06-28 18:09:53+02	2020-06-28 18:13:16+02	dns	bind916	\N	bind916-9.16.4
2020-06-28 17:44:52+02	2020-06-28 18:13:17+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-28 18:13:22+02	2020-06-28 18:15:17+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-28 18:15:17+02	2020-06-28 18:17:29+02	editors	vim	\N	vim-8.2.0869
2020-06-28 18:13:17+02	2020-06-28 18:21:44+02	editors	codelite	\N	codelite-12.0_4
2020-06-28 18:17:29+02	2020-06-28 18:26:13+02	emulators	qemu	\N	qemu-4.1.1_4
2020-06-28 18:26:14+02	2020-06-28 18:34:12+02	emulators	wine	\N	wine-5.0.1,1
2020-06-28 18:34:13+02	2020-06-28 18:48:12+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-06-28 18:48:13+02	2020-06-28 18:49:50+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-28 18:49:50+02	2020-06-28 18:50:39+02	games	xcowsay	\N	xcowsay-1.5
2020-06-28 18:50:40+02	2020-06-28 18:52:49+02	graphics	dia	\N	dia-0.97.3_1,1
2020-06-28 18:52:50+02	2020-06-28 18:55:32+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-28 18:55:33+02	2020-06-28 18:56:40+02	graphics	gimp	\N	gimp-2.10.20,2
2020-06-28 18:56:41+02	2020-06-28 19:00:31+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-28 19:00:32+02	2020-06-28 19:10:38+02	graphics	krita	\N	krita-4.3.0
2020-06-28 19:10:39+02	2020-06-28 19:12:56+02	graphics	wings	\N	wings-2.2.5
2020-06-28 19:12:57+02	2020-06-28 19:13:43+02	graphics	xpdf	\N	xpdf-4.02,1
2020-06-28 17:33:43+02	2020-06-28 19:13:44+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-06-28 19:13:44+02	2020-06-28 19:14:30+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-06-28 18:21:45+02	2020-06-28 19:15:29+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-06-28 19:15:30+02	2020-06-28 19:15:57+02	lang	julia	\N	julia-1.0.3_3
2020-06-28 19:13:45+02	2020-06-28 19:32:10+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.2
2020-06-28 19:32:11+02	2020-06-28 19:33:02+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-28 19:33:03+02	2020-06-28 19:33:16+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-06-28 19:33:16+02	2020-06-28 19:34:14+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-06-28 19:14:31+02	2020-06-28 19:40:12+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-06-28 19:40:13+02	2020-06-28 19:40:52+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-06-28 19:40:52+02	2020-06-28 19:42:49+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-28 19:15:57+02	2020-06-28 19:43:13+02	lang	mono	\N	mono-5.10.1.57_3
2020-06-28 19:42:50+02	2020-06-28 19:45:24+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-06-28 19:45:24+02	2020-06-28 19:45:52+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-06-28 19:43:14+02	2020-06-28 19:45:59+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-28 19:45:53+02	2020-06-28 19:47:04+02	net	mosh	\N	mosh-1.3.2_13
2020-06-28 19:46:00+02	2020-06-28 19:48:07+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-06-28 19:47:04+02	2020-06-28 19:48:35+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-06-28 19:48:35+02	2020-06-28 19:49:29+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-28 19:49:30+02	2020-06-28 19:50:40+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-06-28 19:50:41+02	2020-06-28 19:51:22+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-06-28 19:34:15+02	2020-06-28 19:51:49+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.p3
2020-06-28 19:48:08+02	2020-06-28 19:51:54+02	net	wireshark	\N	wireshark-3.2.4
2020-06-28 19:51:22+02	2020-06-28 19:52:02+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-06-28 19:51:49+02	2020-06-28 19:52:26+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-06-28 19:51:55+02	2020-06-28 19:52:31+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-06-28 19:52:02+02	2020-06-28 19:52:39+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-06-28 19:52:26+02	2020-06-28 19:53:03+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-06-28 19:52:40+02	2020-06-28 19:53:20+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-06-28 19:52:32+02	2020-06-28 19:53:29+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-28 19:53:20+02	2020-06-28 19:53:36+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-06-28 19:53:04+02	2020-06-28 19:53:42+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-06-28 19:53:37+02	2020-06-28 19:53:59+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-06-28 19:53:43+02	2020-06-28 19:54:09+02	print	gv	\N	gv-3.7.4_5
2020-06-28 19:53:29+02	2020-06-28 19:55:26+02	print	auctex	full	auctex-emacs26-12.2
2020-06-28 19:53:59+02	2020-06-28 19:55:41+02	print	lyx	\N	lyx-2.3.4.2_1
2020-06-28 19:55:42+02	2020-06-28 19:56:31+02	security	clamav	\N	clamav-0.102.3_1,1
2020-06-28 19:56:32+02	2020-06-28 19:57:12+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-06-28 19:54:10+02	2020-06-28 19:57:42+02	print	scribus-devel	\N	scribus-devel-1.5.5_14
2020-06-28 19:57:12+02	2020-06-28 19:58:16+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-06-28 19:57:43+02	2020-06-28 19:58:26+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-06-28 19:58:16+02	2020-06-28 19:58:45+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-06-28 19:55:27+02	2020-06-28 19:58:51+02	print	texlive-full	\N	texlive-full-20150521
2020-06-28 19:58:52+02	2020-06-28 19:59:26+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_2
2020-06-28 19:59:27+02	2020-06-28 19:59:49+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-06-28 19:58:45+02	2020-06-28 19:59:53+02	www	mattermost-server	\N	mattermost-server-5.19.1
2020-06-28 19:59:53+02	2020-06-28 20:00:05+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-06-28 19:59:50+02	2020-06-28 20:00:11+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-06-28 20:00:06+02	2020-06-28 20:00:19+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-06-28 20:00:11+02	2020-06-28 20:00:20+02	www	py-requests	py27	py27-requests-2.22.0
2020-06-28 20:00:19+02	2020-06-28 20:00:28+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-06-28 20:00:20+02	2020-06-28 20:00:57+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-06-28 20:00:29+02	2020-06-28 20:01:32+02	x11	mate	\N	mate-1.22.2
2020-06-28 20:00:57+02	2020-06-28 20:02:22+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-06-28 20:02:22+02	2020-06-28 20:03:14+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-06-28 20:01:33+02	2020-06-28 20:06:09+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-28 20:06:10+02	2020-06-28 20:06:20+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-28 20:06:20+02	2020-06-28 20:07:04+02	graphics	xfig	\N	xfig-3.2.7a
2020-06-28 20:07:04+02	2020-06-28 20:07:36+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-06-28 20:03:15+02	2020-06-28 20:36:00+02	editors	vscode	\N	vscode-1.46.1
2020-06-28 19:58:27+02	2020-06-28 20:41:13+02	www	firefox	\N	firefox-78.0_2,1
2020-06-28 20:41:14+02	2020-06-28 20:54:57+02	local	E590T-localbase	\N	E590T-localbase-2020062500
2020-06-29 07:05:27+02	2020-06-29 07:07:35+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-29 07:07:36+02	2020-06-29 07:07:44+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-29 07:05:27+02	2020-06-29 07:17:42+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.p4
2020-06-29 07:17:43+02	2020-06-29 07:31:13+02	local	E590T-localbase	\N	E590T-localbase-2020062500
2020-06-29 17:07:57+02	2020-06-29 17:08:19+02	audio	libvorbis	\N	libvorbis-1.3.6_1,3
2020-06-29 17:08:19+02	2020-06-29 17:08:43+02	multimedia	libtheora	\N	libtheora-1.1.1_7
2020-06-29 17:08:19+02	2020-06-29 17:09:07+02	audio	libsndfile	\N	libsndfile-1.0.28_2
2020-06-29 17:08:44+02	2020-06-29 17:09:27+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-06-29 17:09:07+02	2020-06-29 17:09:33+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-06-29 17:09:27+02	2020-06-29 17:10:04+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-06-29 17:09:33+02	2020-06-29 17:10:22+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-06-29 17:10:05+02	2020-06-29 17:10:33+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-06-29 17:10:23+02	2020-06-29 17:10:48+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-06-29 17:07:57+02	2020-06-29 17:11:05+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-29 17:11:05+02	2020-06-29 17:11:43+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-06-29 17:10:49+02	2020-06-29 17:11:50+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-06-29 17:11:43+02	2020-06-29 17:12:11+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-06-29 17:11:50+02	2020-06-29 17:12:16+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-06-29 17:12:12+02	2020-06-29 17:12:30+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-06-29 17:12:17+02	2020-06-29 17:13:11+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-06-29 17:12:31+02	2020-06-29 17:13:12+02	audio	jack	\N	jackit-0.125.0_9
2020-06-29 17:13:12+02	2020-06-29 17:13:46+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-06-29 17:13:13+02	2020-06-29 17:14:24+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-06-29 17:13:47+02	2020-06-29 17:15:05+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-06-29 17:14:25+02	2020-06-29 17:15:46+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-06-29 17:15:06+02	2020-06-29 17:16:13+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-06-29 17:15:46+02	2020-06-29 17:17:09+02	audio	sox	\N	sox-14.4.2_3
2020-06-29 17:10:34+02	2020-06-29 17:17:50+02	multimedia	ffmpeg	\N	ffmpeg-4.3,1
2020-06-29 17:17:51+02	2020-06-29 17:18:37+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-29 17:17:10+02	2020-06-29 17:18:50+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-06-29 17:16:14+02	2020-06-29 17:19:06+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-06-29 17:18:37+02	2020-06-29 17:19:15+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-29 17:19:07+02	2020-06-29 17:22:59+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-29 17:18:51+02	2020-06-29 17:24:21+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-06-29 17:19:15+02	2020-06-29 17:27:00+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-29 17:24:22+02	2020-06-29 17:45:10+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-06-29 17:45:11+02	2020-06-29 17:46:33+02	x11-wm	marco	\N	marco-1.22.4
2020-06-29 17:46:34+02	2020-06-29 17:47:11+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-06-29 17:47:11+02	2020-06-29 17:47:31+02	audio	twolame	\N	twolame-0.4.0
2020-06-29 17:27:01+02	2020-06-29 17:47:58+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-29 17:47:31+02	2020-06-29 17:48:29+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-06-29 17:48:00+02	2020-06-29 17:49:58+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-06-29 17:48:29+02	2020-06-29 17:49:57+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-29 17:49:58+02	2020-06-29 17:51:10+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-29 17:51:11+02	2020-06-29 17:52:27+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-29 17:49:58+02	2020-06-29 17:52:47+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-06-29 17:52:27+02	2020-06-29 17:53:04+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-06-29 17:52:48+02	2020-06-29 17:53:08+02	audio	dssi	\N	dssi-1.1.1_8
2020-06-29 17:53:05+02	2020-06-29 17:53:32+02	audio	faudio	\N	FAudio-20.06
2020-06-29 17:53:08+02	2020-06-29 17:53:58+02	audio	mate-media	\N	mate-media-1.24.0
2020-06-29 17:53:59+02	2020-06-29 17:54:14+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-06-29 17:54:15+02	2020-06-29 17:55:45+02	devel	py-game	py37	py37-game-1.9.6_1
2020-06-29 17:53:33+02	2020-06-29 17:55:50+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-06-29 17:23:00+02	2020-06-29 17:56:51+02	www	chromium	\N	chromium-83.0.4103.106
2020-06-29 17:55:51+02	2020-06-29 17:57:50+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-06-29 17:56:53+02	2020-06-29 17:58:00+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-29 17:57:51+02	2020-06-29 17:58:55+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-29 17:58:01+02	2020-06-29 17:59:02+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106
2020-06-29 17:58:55+02	2020-06-29 18:00:16+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-29 18:00:16+02	2020-06-29 18:01:17+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-29 18:01:18+02	2020-06-29 18:02:48+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-29 17:59:03+02	2020-06-29 18:03:58+02	multimedia	vlc	\N	vlc-3.0.11_1,4
2020-06-29 18:02:49+02	2020-06-29 18:04:27+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-06-29 18:03:58+02	2020-06-29 18:04:59+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-29 18:04:28+02	2020-06-29 18:05:36+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-29 18:05:01+02	2020-06-29 18:06:09+02	x11	mate-base	\N	mate-base-1.24.0
2020-06-29 18:05:37+02	2020-06-29 18:08:41+02	audio	rosegarden	\N	rosegarden-20.06
2020-06-29 18:08:41+02	2020-06-29 18:10:22+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-06-29 18:06:09+02	2020-06-29 18:10:24+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-06-29 18:10:25+02	2020-06-29 18:11:54+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-06-29 18:11:55+02	2020-06-29 18:13:19+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-06-29 17:55:45+02	2020-06-29 18:13:22+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-06-29 18:13:20+02	2020-06-29 18:15:16+02	deskutils	calibre	\N	calibre-4.19.0
2020-06-29 18:13:23+02	2020-06-29 18:36:25+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-29 18:36:25+02	2020-06-29 18:38:01+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-06-29 18:15:17+02	2020-06-29 18:39:18+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-29 18:39:21+02	2020-06-29 18:39:39+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-06-29 18:39:40+02	2020-06-29 18:41:15+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-29 18:41:15+02	2020-06-29 18:42:17+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-06-29 18:38:02+02	2020-06-29 18:43:20+02	emulators	wine	\N	wine-5.0.1,1
2020-06-29 18:43:20+02	2020-06-29 18:43:56+02	games	monsterz	\N	monsterz-0.7.1_15
2020-06-29 18:43:57+02	2020-06-29 18:45:00+02	games	xblast	\N	xblast-2.10.4_17
2020-06-29 18:42:17+02	2020-06-29 18:47:06+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-29 18:47:06+02	2020-06-29 18:48:27+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-29 18:48:28+02	2020-06-29 18:50:53+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-29 18:50:53+02	2020-06-29 18:52:52+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-06-29 18:45:00+02	2020-06-29 18:53:20+02	graphics	krita	\N	krita-4.3.0
2020-06-29 18:53:21+02	2020-06-29 18:54:21+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-29 18:52:52+02	2020-06-29 18:54:46+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-29 18:54:47+02	2020-06-29 18:55:54+02	x11	mate	\N	mate-1.22.2
2020-06-29 18:55:55+02	2020-06-29 19:29:43+02	editors	vscode	\N	vscode-1.46.1
2020-06-29 19:29:44+02	2020-06-29 19:29:51+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-29 18:54:21+02	2020-06-29 19:31:46+02	www	firefox	\N	firefox-78.0_2,1
2020-06-29 18:10:22+02	2020-06-29 19:34:14+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-06-29 19:34:15+02	2020-06-29 19:48:05+02	local	E590T-localbase	\N	E590T-localbase-2020062500
2020-06-29 22:55:43+02	2020-06-29 22:57:33+02	ports-mgmt	pkg	\N	pkg-1.14.6
2020-06-29 23:03:21+02	2020-06-29 23:07:49+02	multimedia	ffmpeg	\N	ffmpeg-4.3_1,1
2020-06-29 23:07:50+02	2020-06-29 23:19:16+02	devel	electron7	\N	electron7-7.3.1_1
2020-06-29 23:19:18+02	2020-06-29 23:32:57+02	www	chromium	\N	chromium-83.0.4103.106
2020-06-29 23:37:28+02	2020-06-29 23:37:45+02	audio	libmysofa	\N	libmysofa-1.1
2020-06-29 23:37:28+02	2020-06-29 23:37:55+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-06-29 23:37:55+02	2020-06-29 23:38:28+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-06-29 23:37:45+02	2020-06-29 23:38:44+02	audio	openal-soft	\N	openal-soft-1.20.1_1
2020-06-29 23:37:28+02	2020-06-29 23:42:13+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-06-29 23:38:45+02	2020-06-29 23:42:18+02	graphics	opencv	\N	opencv-3.4.1_33
2020-06-29 23:38:28+02	2020-06-29 23:46:15+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-06-29 23:46:16+02	2020-06-29 23:48:05+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-06-29 23:48:05+02	2020-06-29 23:49:30+02	multimedia	mlt	\N	mlt-6.20.0
2020-06-29 23:49:31+02	2020-06-29 23:50:45+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-06-29 23:50:46+02	2020-06-29 23:51:28+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-06-29 23:51:29+02	2020-06-29 23:52:11+02	audio	faudio	\N	FAudio-20.06
2020-06-29 23:52:12+02	2020-06-29 23:52:34+02	audio	freealut	\N	freealut-1.1.0_4
2020-06-29 23:52:35+02	2020-06-29 23:54:07+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-06-29 23:54:08+02	2020-06-29 23:55:42+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106
2020-06-29 23:42:18+02	2020-06-29 23:56:06+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-29 23:42:14+02	2020-06-29 23:56:46+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-06-29 23:55:42+02	2020-06-29 23:57:06+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-06-29 23:56:47+02	2020-06-29 23:57:36+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-06-29 23:57:07+02	2020-06-29 23:57:53+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-29 23:57:37+02	2020-06-29 23:58:36+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-06-29 23:56:07+02	2020-06-30 00:00:02+02	multimedia	vlc	\N	vlc-3.0.11_1,4
2020-06-29 23:58:37+02	2020-06-30 00:00:28+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-06-29 23:57:53+02	2020-06-30 00:02:00+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-06-30 00:00:29+02	2020-06-30 00:02:25+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-06-30 00:02:01+02	2020-06-30 00:03:35+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-06-30 00:02:25+02	2020-06-30 00:04:06+02	deskutils	calibre	\N	calibre-4.19.0
2020-06-30 00:04:07+02	2020-06-30 00:04:35+02	devel	cdecl	\N	cdecl-7.3
2020-06-30 00:04:36+02	2020-06-30 00:12:11+02	emulators	wine	\N	wine-5.0.1,1
2020-06-30 00:12:12+02	2020-06-30 00:13:39+02	games	alienarena	\N	alienarena-2013.766_9
2020-06-30 00:13:39+02	2020-06-30 00:14:37+02	games	battletanks	\N	btanks-0.9.8083_9
2020-06-30 00:14:38+02	2020-06-30 00:17:19+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-30 00:17:19+02	2020-06-30 00:18:13+02	multimedia	celluloid	\N	celluloid-0.19
2020-06-30 00:18:14+02	2020-06-30 00:19:57+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-06-30 00:19:58+02	2020-06-30 00:21:45+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-06-30 00:03:35+02	2020-06-30 00:22:28+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-30 00:22:28+02	2020-06-30 00:22:35+02	security	base-audit	\N	base-audit-0.4
2020-06-30 00:21:46+02	2020-06-30 00:23:39+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-06-30 00:23:40+02	2020-06-30 00:55:46+02	editors	vscode	\N	vscode-1.46.1
2020-06-30 00:55:47+02	2020-06-30 00:55:55+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-30 00:22:35+02	2020-06-30 00:58:26+02	www	firefox	\N	firefox-78.0_2,1
2020-06-30 00:00:02+02	2020-06-30 01:07:50+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-06-30 01:07:51+02	2020-06-30 01:21:05+02	local	E590T-localbase	\N	E590T-localbase-2020062500
2020-06-30 07:30:17+02	2020-06-30 07:30:38+02	devel	jsoncpp	\N	jsoncpp-1.9.3
2020-06-30 07:30:38+02	2020-06-30 07:34:00+02	devel	cmake	\N	cmake-3.17.3
2020-06-30 07:34:01+02	2020-06-30 07:47:50+02	www	chromium	\N	chromium-83.0.4103.106
2020-06-30 07:52:25+02	2020-06-30 07:52:35+02	devel	popt	\N	popt-1.18
2020-06-30 07:52:25+02	2020-06-30 07:56:16+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106
2020-06-30 07:56:16+02	2020-06-30 07:59:11+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-30 07:59:11+02	2020-06-30 08:03:20+02	graphics	inkscape	\N	inkscape-1.0_2
2020-06-30 07:52:35+02	2020-06-30 08:03:33+02	net	samba410	\N	samba410-4.10.15
2020-06-30 08:03:34+02	2020-06-30 08:04:47+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-06-30 08:03:21+02	2020-06-30 08:05:12+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-06-30 07:52:25+02	2020-06-30 08:05:20+02	www	qt5-webengine	\N	qt5-webengine-5.14.2
2020-06-30 08:05:12+02	2020-06-30 08:05:49+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-06-30 08:05:21+02	2020-06-30 08:06:12+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-30 08:04:48+02	2020-06-30 08:07:23+02	editors	pluma	\N	pluma-1.24.0
2020-06-30 08:06:13+02	2020-06-30 08:08:35+02	deskutils	calibre	\N	calibre-4.19.0
2020-06-30 08:07:23+02	2020-06-30 08:09:18+02	x11	mate	\N	mate-1.22.2
2020-06-30 08:09:19+02	2020-06-30 08:09:29+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-30 08:08:36+02	2020-06-30 08:13:20+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.8_2
2020-06-30 08:05:50+02	2020-06-30 08:21:12+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-30 08:21:13+02	2020-06-30 08:34:42+02	local	E590T-localbase	\N	E590T-localbase-2020062500
2020-06-30 23:04:55+02	2020-06-30 23:06:05+02	devel	cmake	\N	cmake-3.17.3_1
2020-06-30 23:06:06+02	2020-06-30 23:20:07+02	www	chromium	\N	chromium-83.0.4103.106_1
2020-06-30 23:24:47+02	2020-06-30 23:24:57+02	databases	py-redis	py37	py37-redis-3.5.3
2020-06-30 23:24:57+02	2020-06-30 23:25:05+02	devel	py-rq	py37	py37-rq-1.3.0
2020-06-30 23:24:47+02	2020-06-30 23:25:42+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-06-30 23:25:06+02	2020-06-30 23:26:09+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-06-30 23:26:10+02	2020-06-30 23:26:23+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-06-30 23:25:43+02	2020-06-30 23:26:56+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-06-30 23:26:56+02	2020-06-30 23:29:32+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-06-30 23:26:23+02	2020-06-30 23:29:52+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-06-30 23:29:53+02	2020-06-30 23:30:06+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-06-30 23:30:06+02	2020-06-30 23:30:15+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-06-30 23:30:16+02	2020-06-30 23:30:47+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-06-30 23:29:32+02	2020-06-30 23:30:47+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-06-30 23:24:47+02	2020-06-30 23:34:17+02	www	qt5-webengine	\N	qt5-webengine-5.14.2_1
2020-06-30 23:34:18+02	2020-06-30 23:35:01+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-06-30 23:35:02+02	2020-06-30 23:37:28+02	deskutils	calibre	\N	calibre-4.19.0
2020-06-30 23:34:19+02	2020-06-30 23:48:37+02	devel	RStudio	\N	RStudio-1.3.959
2020-06-30 23:48:38+02	2020-07-01 00:02:23+02	local	E590T-localbase	\N	E590T-localbase-2020062500
2020-07-01 07:23:08+02	2020-07-01 07:28:45+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-01 07:28:46+02	2020-07-01 07:28:56+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-01 07:23:08+02	2020-07-01 08:21:48+02	www	node12	\N	node12-12.18.2
2020-07-01 08:21:48+02	2020-07-01 08:22:00+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-07-01 08:21:48+02	2020-07-01 08:22:13+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-07-01 07:23:08+02	2020-07-01 09:15:34+02	www	firefox	\N	firefox-78.0.1,1
2020-07-01 09:15:36+02	2020-07-01 09:28:53+02	local	E590T-localbase	\N	E590T-localbase-2020062500
2020-07-01 21:22:06+02	2020-07-01 21:23:11+02	graphics	glfw	\N	glfw-3.3.2
2020-07-01 21:23:11+02	2020-07-01 21:23:35+02	archivers	c-blosc	\N	c-blosc-1.15.0_2
2020-07-01 21:22:06+02	2020-07-01 21:23:58+02	graphics	netpbm	\N	netpbm-10.91.00
2020-07-01 21:22:06+02	2020-07-01 21:24:30+02	multimedia	libde265	\N	libde265-1.0.2_5
2020-07-01 21:23:59+02	2020-07-01 21:24:34+02	devel	libfmt	\N	libfmt-6.2.1
2020-07-01 21:24:30+02	2020-07-01 21:24:37+02	devel	robin-map	\N	robin-map-0.6.2
2020-07-01 21:24:37+02	2020-07-01 21:26:27+02	graphics	libheif	\N	libheif-1.6.2
2020-07-01 21:26:28+02	2020-07-01 21:26:47+02	graphics	squish	\N	squish-1.10_2
2020-07-01 21:26:48+02	2020-07-01 21:27:10+02	x11-fonts	droid-fonts-ttf	\N	droid-fonts-ttf-20131024_5
2020-07-01 21:27:10+02	2020-07-01 21:28:05+02	audio	faudio	\N	FAudio-20.07
2020-07-01 21:23:36+02	2020-07-01 21:28:16+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-07-01 21:28:06+02	2020-07-01 21:38:01+02	graphics	alembic	\N	alembic-1.7.12_2
2020-07-01 21:38:02+02	2020-07-01 21:45:47+02	graphics	oidn	\N	oidn-1.1.0_2
2020-07-01 21:45:50+02	2020-07-01 21:50:36+02	graphics	opensubdiv	\N	opensubdiv-3.4.3
2020-07-01 21:24:34+02	2020-07-01 21:52:01+02	graphics	dcmtk	\N	dcmtk-3.6.5_2
2020-07-01 21:28:17+02	2020-07-01 22:00:20+02	graphics	embree	\N	embree-3.10.0
2020-07-01 22:00:23+02	2020-07-01 22:00:57+02	print	transfig	\N	transfig-3.2.5e_4
2020-07-01 22:00:58+02	2020-07-01 22:05:31+02	editors	vim	\N	vim-8.2.1110
2020-07-01 21:50:37+02	2020-07-01 22:09:08+02	misc	openvdb	\N	openvdb-7.0.0_2
2020-07-01 21:52:02+02	2020-07-01 22:10:33+02	graphics	openimageio	\N	openimageio-2.1.15.0
2020-07-01 22:05:32+02	2020-07-01 22:12:55+02	emulators	wine	\N	wine-5.0.1,1
2020-07-01 22:12:56+02	2020-07-01 22:17:38+02	graphics	inkscape	\N	inkscape-1.0_2
2020-07-01 22:09:11+02	2020-07-01 22:18:03+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-01 22:17:39+02	2020-07-01 22:18:07+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-07-01 22:18:08+02	2020-07-01 22:18:17+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-01 22:18:17+02	2020-07-01 22:18:57+02	graphics	xfig	\N	xfig-3.2.7a
2020-07-01 22:18:04+02	2020-07-01 22:19:44+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-07-01 22:10:34+02	2020-07-01 22:47:46+02	graphics	blender	\N	blender-2.83.1
2020-07-01 22:47:46+02	2020-07-01 23:01:59+02	local	E590T-localbase	\N	E590T-localbase-2020070100
2020-07-02 07:57:26+02	2020-07-02 07:57:49+02	x11	libinput	\N	libinput-1.15.6
2020-07-02 07:57:50+02	2020-07-02 08:02:08+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-07-02 08:02:08+02	2020-07-02 08:04:27+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-07-02 07:57:26+02	2020-07-02 08:04:48+02	multimedia	aom	\N	aom-2.0.0_1
2020-07-02 08:04:49+02	2020-07-02 08:05:32+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-07-02 08:05:32+02	2020-07-02 08:06:44+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-07-02 08:06:45+02	2020-07-02 08:07:23+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-07-02 08:04:28+02	2020-07-02 08:07:45+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-07-02 08:07:24+02	2020-07-02 08:07:54+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-07-02 07:57:26+02	2020-07-02 08:07:55+02	multimedia	x265	\N	x265-3.2.1_3
2020-07-02 08:07:45+02	2020-07-02 08:08:37+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-07-02 08:08:38+02	2020-07-02 08:09:21+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-07-02 08:09:22+02	2020-07-02 08:10:11+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-07-02 08:10:12+02	2020-07-02 08:11:27+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-07-02 08:11:27+02	2020-07-02 08:12:35+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-02 08:12:36+02	2020-07-02 08:14:03+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-07-02 08:14:04+02	2020-07-02 08:15:13+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-02 08:15:13+02	2020-07-02 08:15:59+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-07-02 08:07:55+02	2020-07-02 08:16:25+02	multimedia	ffmpeg	\N	ffmpeg-4.3_2,1
2020-07-02 08:15:59+02	2020-07-02 08:16:49+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-07-02 08:16:26+02	2020-07-02 08:17:28+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-07-02 08:16:50+02	2020-07-02 08:17:42+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-07-02 08:17:28+02	2020-07-02 08:18:13+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-07-02 08:17:43+02	2020-07-02 08:18:26+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-07-02 08:18:13+02	2020-07-02 08:18:59+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-02 08:18:27+02	2020-07-02 08:19:11+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-07-02 08:19:00+02	2020-07-02 08:19:51+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-07-02 08:19:12+02	2020-07-02 08:20:13+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-07-02 08:07:54+02	2020-07-02 08:20:15+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_7
2020-07-02 08:20:13+02	2020-07-02 08:20:53+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-07-02 08:19:51+02	2020-07-02 08:21:07+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-07-02 08:20:53+02	2020-07-02 08:21:32+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-07-02 08:21:07+02	2020-07-02 08:22:05+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-07-02 08:21:33+02	2020-07-02 08:22:21+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-07-02 08:22:21+02	2020-07-02 08:23:02+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-02 08:20:16+02	2020-07-02 08:23:19+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-07-02 08:23:03+02	2020-07-02 08:23:44+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-02 08:23:19+02	2020-07-02 08:24:08+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-07-02 08:22:06+02	2020-07-02 08:24:13+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-02 08:23:45+02	2020-07-02 08:24:26+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-07-02 08:24:14+02	2020-07-02 08:24:43+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-02 08:24:26+02	2020-07-02 08:25:07+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-07-02 08:24:43+02	2020-07-02 08:25:20+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-07-02 08:25:21+02	2020-07-02 08:25:57+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-02 08:25:08+02	2020-07-02 08:26:16+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-02 08:25:58+02	2020-07-02 08:26:46+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-02 08:26:16+02	2020-07-02 08:27:32+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-07-02 08:27:33+02	2020-07-02 08:28:25+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-07-02 08:24:09+02	2020-07-02 08:28:48+02	math	vtk8	\N	vtk8-8.2.0
2020-07-02 08:28:25+02	2020-07-02 08:29:13+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-07-02 08:29:13+02	2020-07-02 08:30:23+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-02 08:30:24+02	2020-07-02 08:31:37+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-02 08:26:46+02	2020-07-02 08:32:17+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-07-02 08:31:44+02	2020-07-02 08:32:29+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-07-02 08:32:17+02	2020-07-02 08:32:54+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-02 08:32:29+02	2020-07-02 08:33:09+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-02 08:32:54+02	2020-07-02 08:33:31+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-07-02 08:28:49+02	2020-07-02 08:33:33+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-02 08:33:09+02	2020-07-02 08:33:49+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-07-02 08:33:32+02	2020-07-02 08:34:10+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-07-02 08:33:50+02	2020-07-02 08:34:32+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-07-02 08:34:32+02	2020-07-02 08:35:18+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-07-02 08:34:10+02	2020-07-02 08:36:04+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-07-02 08:35:18+02	2020-07-02 08:36:18+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-07-02 08:36:04+02	2020-07-02 08:37:21+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-02 08:36:18+02	2020-07-02 08:37:24+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-07-02 08:37:21+02	2020-07-02 08:38:31+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-02 08:37:25+02	2020-07-02 08:38:58+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-07-02 08:38:31+02	2020-07-02 08:39:22+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-02 08:38:58+02	2020-07-02 08:40:21+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-02 08:39:23+02	2020-07-02 08:40:21+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-07-02 08:40:22+02	2020-07-02 08:41:17+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-02 08:41:17+02	2020-07-02 08:42:15+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-07-02 08:40:22+02	2020-07-02 08:43:18+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-07-02 08:42:15+02	2020-07-02 08:45:51+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-02 08:45:51+02	2020-07-02 08:46:38+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-02 08:33:33+02	2020-07-02 08:46:44+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-02 08:43:19+02	2020-07-02 08:47:09+02	graphics	opencv	\N	opencv-3.4.1_33
2020-07-02 08:47:10+02	2020-07-02 08:49:35+02	x11	xorg	\N	xorg-7.7_3
2020-07-02 08:49:36+02	2020-07-02 08:50:31+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-07-02 08:50:31+02	2020-07-02 08:51:54+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-02 08:51:54+02	2020-07-02 08:52:29+02	graphics	libheif	\N	libheif-1.6.2
2020-07-02 08:52:30+02	2020-07-02 08:53:29+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-07-02 08:53:30+02	2020-07-02 08:54:08+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-02 08:54:08+02	2020-07-02 08:55:42+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-07-02 08:46:45+02	2020-07-02 08:55:51+02	www	qt5-webengine	\N	qt5-webengine-5.14.2_1
2020-07-02 08:55:42+02	2020-07-02 08:56:17+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-07-02 08:55:52+02	2020-07-02 08:56:38+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-02 08:56:17+02	2020-07-02 08:56:48+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-02 08:56:48+02	2020-07-02 08:57:10+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-02 08:56:38+02	2020-07-02 08:57:22+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-02 08:57:23+02	2020-07-02 08:57:44+02	audio	faudio	\N	FAudio-20.07
2020-07-02 08:57:10+02	2020-07-02 08:58:20+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-07-02 08:57:45+02	2020-07-02 08:58:23+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0_1
2020-07-02 08:58:23+02	2020-07-02 08:59:19+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-02 08:59:19+02	2020-07-02 08:59:53+02	graphics	openimageio	\N	openimageio-2.1.15.0
2020-07-02 08:59:53+02	2020-07-02 09:00:53+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-02 09:00:54+02	2020-07-02 09:02:02+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-07-02 09:02:02+02	2020-07-02 09:07:24+02	multimedia	vlc	\N	vlc-3.0.11_1,4
2020-07-02 09:07:27+02	2020-07-02 09:08:51+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-02 09:08:51+02	2020-07-02 09:10:41+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-02 09:10:41+02	2020-07-02 09:11:19+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-07-02 09:11:19+02	2020-07-02 09:12:21+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-02 09:12:21+02	2020-07-02 09:13:35+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-02 08:58:21+02	2020-07-02 09:18:35+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-07-02 09:13:35+02	2020-07-02 09:21:01+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-02 09:18:36+02	2020-07-02 09:22:42+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-02 09:22:43+02	2020-07-02 09:25:11+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-02 09:21:02+02	2020-07-02 09:26:18+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-02 09:26:19+02	2020-07-02 09:28:42+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-02 09:28:43+02	2020-07-02 09:30:29+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-02 09:30:30+02	2020-07-02 09:32:43+02	deskutils	calibre	\N	calibre-4.19.0
2020-07-02 09:32:44+02	2020-07-02 09:58:42+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-02 09:58:43+02	2020-07-02 09:59:23+02	devel	cdecl	\N	cdecl-7.4
2020-07-02 09:59:24+02	2020-07-02 10:24:39+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-02 10:24:42+02	2020-07-02 10:26:54+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-07-02 08:46:38+02	2020-07-02 11:05:35+02	www	chromium	\N	chromium-83.0.4103.116
2020-07-02 11:05:38+02	2020-07-02 11:06:59+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-02 11:07:00+02	2020-07-02 11:08:20+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-02 09:25:12+02	2020-07-02 11:09:08+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-02 11:09:09+02	2020-07-02 11:15:00+02	graphics	blender	\N	blender-2.83.1
2020-07-02 11:08:21+02	2020-07-02 11:15:34+02	emulators	wine	\N	wine-5.0.1,1
2020-07-02 11:15:01+02	2020-07-02 11:17:28+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-02 11:17:29+02	2020-07-02 11:18:39+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-02 11:18:39+02	2020-07-02 11:20:07+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-02 11:20:08+02	2020-07-02 11:22:19+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-07-02 11:15:35+02	2020-07-02 11:24:04+02	graphics	krita	\N	krita-4.3.0
2020-07-02 11:22:19+02	2020-07-02 11:24:07+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-02 11:24:07+02	2020-07-02 11:25:14+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-02 10:26:55+02	2020-07-02 11:25:13+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-07-02 11:24:04+02	2020-07-02 11:25:57+02	multimedia	shotcut	\N	shotcut-20.04.12_1
2020-07-02 11:25:14+02	2020-07-02 11:26:14+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-02 11:25:58+02	2020-07-02 11:27:49+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-02 11:25:14+02	2020-07-02 11:28:29+02	net	wireshark	\N	wireshark-3.2.4
2020-07-02 11:27:50+02	2020-07-02 11:28:38+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-02 11:28:39+02	2020-07-02 11:29:26+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-02 11:26:14+02	2020-07-02 11:30:13+02	print	scribus-devel	\N	scribus-devel-1.5.5_14
2020-07-02 11:30:13+02	2020-07-02 11:30:21+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-02 11:29:27+02	2020-07-02 11:59:16+02	editors	vscode	\N	vscode-1.46.1
2020-07-02 11:28:30+02	2020-07-02 12:01:29+02	www	firefox	\N	firefox-78.0.1,1
2020-07-02 12:06:17+02	2020-07-02 12:19:49+02	local	E590T-localbase	\N	E590T-localbase-2020070100
2020-07-02 22:03:33+02	2020-07-02 22:03:46+02	textproc	lowdown	\N	lowdown-0.7.1
2020-07-02 22:03:33+02	2020-07-02 22:03:46+02	devel	p5-Parse-Yapp	\N	p5-Parse-Yapp-1.21
2020-07-02 22:03:33+02	2020-07-02 22:05:12+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-02 22:05:12+02	2020-07-02 22:05:19+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-02 22:03:46+02	2020-07-02 22:06:41+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-07-02 22:06:42+02	2020-07-02 22:20:02+02	local	E590T-localbase	\N	E590T-localbase-2020070100
2020-07-03 07:08:49+02	2020-07-03 07:10:51+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-03 07:10:52+02	2020-07-03 07:10:59+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-03 07:08:49+02	2020-07-03 07:12:05+02	lang	python37	\N	python37-3.7.8
2020-07-03 07:12:05+02	2020-07-03 07:12:16+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-07-03 07:12:05+02	2020-07-03 07:12:16+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-07-03 07:12:16+02	2020-07-03 07:12:25+02	devel	py-six	py37	py37-six-1.14.0
2020-07-03 07:12:25+02	2020-07-03 07:12:35+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-07-03 07:12:35+02	2020-07-03 07:12:50+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-07-03 07:12:05+02	2020-07-03 07:12:53+02	devel	ninja	\N	ninja-1.10.0,2
2020-07-03 07:12:54+02	2020-07-03 07:13:05+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-07-03 07:12:51+02	2020-07-03 07:13:07+02	devel	py-cffi	py37	py37-cffi-1.14.0
2020-07-03 07:13:06+02	2020-07-03 07:13:17+02	dns	py-idna	py37	py37-idna-2.8
2020-07-03 07:13:17+02	2020-07-03 07:13:50+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-07-03 07:13:08+02	2020-07-03 07:13:50+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-07-03 07:13:50+02	2020-07-03 07:14:02+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-07-03 07:13:50+02	2020-07-03 07:14:02+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-07-03 07:14:02+02	2020-07-03 07:14:14+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-07-03 07:14:14+02	2020-07-03 07:14:28+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-07-03 07:12:16+02	2020-07-03 07:15:29+02	devel	glib20	\N	glib-2.56.3_8,1
2020-07-03 07:14:03+02	2020-07-03 07:15:32+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-07-03 07:15:30+02	2020-07-03 07:15:38+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-07-03 07:15:32+02	2020-07-03 07:15:41+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-07-03 07:15:39+02	2020-07-03 07:15:48+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-07-03 07:15:42+02	2020-07-03 07:15:52+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-07-03 07:15:48+02	2020-07-03 07:16:02+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-07-03 07:15:53+02	2020-07-03 07:16:02+02	www	py-requests	py37	py37-requests-2.22.0
2020-07-03 07:16:03+02	2020-07-03 07:16:11+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-07-03 07:16:03+02	2020-07-03 07:16:12+02	devel	py-packaging	py37	py37-packaging-20.4
2020-07-03 07:14:29+02	2020-07-03 07:16:13+02	lang	cython	py37	py37-cython-0.29.15
2020-07-03 07:16:12+02	2020-07-03 07:16:21+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-07-03 07:16:12+02	2020-07-03 07:16:21+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-07-03 07:16:13+02	2020-07-03 07:16:29+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-07-03 07:16:21+02	2020-07-03 07:16:30+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-07-03 07:16:22+02	2020-07-03 07:16:31+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-07-03 07:16:29+02	2020-07-03 07:16:38+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-07-03 07:16:30+02	2020-07-03 07:16:39+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-07-03 07:16:31+02	2020-07-03 07:16:40+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-07-03 07:16:38+02	2020-07-03 07:16:47+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-07-03 07:16:40+02	2020-07-03 07:16:51+02	devel	py-future	py37	py37-future-0.18.2
2020-07-03 07:16:39+02	2020-07-03 07:16:51+02	devel	meson	\N	meson-0.54.3
2020-07-03 07:16:51+02	2020-07-03 07:17:00+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-07-03 07:16:52+02	2020-07-03 07:17:00+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-07-03 07:16:47+02	2020-07-03 07:17:04+02	textproc	py-sphinx	py37	py37-sphinx-3.0.4,1
2020-07-03 07:17:01+02	2020-07-03 07:17:11+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-07-03 07:17:05+02	2020-07-03 07:17:15+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-07-03 07:17:11+02	2020-07-03 07:17:37+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-07-03 07:17:38+02	2020-07-03 07:18:04+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-07-03 07:18:05+02	2020-07-03 07:18:17+02	textproc	itstool	\N	itstool-2.0.6
2020-07-03 07:18:17+02	2020-07-03 07:18:46+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-07-03 07:17:01+02	2020-07-03 07:18:50+02	graphics	cairo	\N	cairo-1.16.0,2
2020-07-03 07:18:46+02	2020-07-03 07:19:29+02	misc	shared-mime-info	\N	shared-mime-info-1.10_2
2020-07-03 07:19:30+02	2020-07-03 07:20:04+02	textproc	libcroco	\N	libcroco-0.6.13
2020-07-03 07:18:50+02	2020-07-03 07:20:08+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1,1
2020-07-03 07:20:08+02	2020-07-03 07:23:12+02	print	harfbuzz	\N	harfbuzz-2.6.8
2020-07-03 07:23:12+02	2020-07-03 07:24:34+02	net	avahi-app	\N	avahi-app-0.7_3
2020-07-03 07:24:34+02	2020-07-03 07:25:29+02	print	cups	\N	cups-2.3.3_1
2020-07-03 07:25:29+02	2020-07-03 07:28:26+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-07-03 07:20:04+02	2020-07-03 07:29:55+02	devel	qt5-core	\N	qt5-core-5.14.2_3
2020-07-03 07:28:27+02	2020-07-03 07:29:59+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-07-03 07:29:56+02	2020-07-03 07:30:25+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-07-03 07:29:59+02	2020-07-03 07:31:02+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-07-03 07:31:02+02	2020-07-03 07:31:28+02	accessibility	atk	\N	atk-2.36.0
2020-07-03 07:31:28+02	2020-07-03 07:31:52+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-07-03 07:31:53+02	2020-07-03 07:32:46+02	sysutils	polkit	\N	polkit-0.116
2020-07-03 07:32:47+02	2020-07-03 07:33:59+02	devel	libgsf	\N	libgsf-1.14.47
2020-07-03 07:33:59+02	2020-07-03 07:34:19+02	devel	libgudev	\N	libgudev-230_1
2020-07-03 07:30:25+02	2020-07-03 07:34:54+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-07-03 07:34:20+02	2020-07-03 07:35:49+02	graphics	colord	\N	colord-1.3.5
2020-07-03 07:35:50+02	2020-07-03 07:36:04+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-07-03 07:36:05+02	2020-07-03 07:36:19+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-07-03 07:36:19+02	2020-07-03 07:36:39+02	x11	libwacom	\N	libwacom-1.4.1
2020-07-03 07:36:39+02	2020-07-03 07:37:06+02	x11	libinput	\N	libinput-1.15.6
2020-07-03 07:37:07+02	2020-07-03 07:37:45+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-07-03 07:34:55+02	2020-07-03 07:38:19+02	lang	vala	\N	vala-0.40.15,1
2020-07-03 07:38:20+02	2020-07-03 07:39:19+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-07-03 07:37:46+02	2020-07-03 07:39:21+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-07-03 07:39:19+02	2020-07-03 07:40:30+02	net	qt5-network	\N	qt5-network-5.14.2
2020-07-03 07:40:30+02	2020-07-03 07:42:52+02	devel	qt5-dbus	\N	qt5-dbus-5.14.2
2020-07-03 07:39:21+02	2020-07-03 07:44:46+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-07-03 07:44:46+02	2020-07-03 07:45:00+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-07-03 07:42:53+02	2020-07-03 07:45:02+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-07-03 07:45:03+02	2020-07-03 07:47:04+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-07-03 07:45:01+02	2020-07-03 07:47:49+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.8_2
2020-07-03 07:47:50+02	2020-07-03 07:49:05+02	databases	qt5-sql	\N	qt5-sql-5.14.2
2020-07-03 07:49:06+02	2020-07-03 07:50:02+02	graphics	poppler	\N	poppler-0.89.0_1
2020-07-03 07:47:05+02	2020-07-03 07:50:40+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-07-03 07:50:03+02	2020-07-03 07:51:15+02	devel	qt5-testlib	\N	qt5-testlib-5.14.2
2020-07-03 07:50:40+02	2020-07-03 07:51:48+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.14.2
2020-07-03 07:51:16+02	2020-07-03 07:52:11+02	devel	dconf	\N	dconf-0.36.0
2020-07-03 07:52:12+02	2020-07-03 07:54:43+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-07-03 07:51:49+02	2020-07-03 07:54:53+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-07-03 07:54:54+02	2020-07-03 07:55:21+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-07-03 07:55:22+02	2020-07-03 07:57:27+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-07-03 07:54:43+02	2020-07-03 07:57:43+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-07-03 07:57:28+02	2020-07-03 07:57:48+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-07-03 07:57:44+02	2020-07-03 07:59:37+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-07-03 07:59:38+02	2020-07-03 08:00:33+02	textproc	qt5-xml	\N	qt5-xml-5.14.2
2020-07-03 08:00:34+02	2020-07-03 08:01:17+02	devel	libsoup	\N	libsoup-2.62.3
2020-07-03 07:57:49+02	2020-07-03 08:01:29+02	print	texlive-base	\N	texlive-base-20150521_52
2020-07-03 08:01:17+02	2020-07-03 08:04:40+02	devel	py-lxml	py37	py37-lxml-4.5.1
2020-07-03 08:04:40+02	2020-07-03 08:04:51+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-07-03 08:04:52+02	2020-07-03 08:05:39+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.14.2
2020-07-03 08:05:39+02	2020-07-03 08:05:52+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-07-03 08:05:52+02	2020-07-03 08:06:20+02	devel	json-glib	\N	json-glib-1.4.4
2020-07-03 08:06:21+02	2020-07-03 08:06:32+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-07-03 08:06:32+02	2020-07-03 08:06:46+02	textproc	py-sparqlwrapper	py37	py37-sparqlwrapper-1.8.5
2020-07-03 08:06:46+02	2020-07-03 08:07:08+02	devel	libnotify	\N	libnotify-0.7.8
2020-07-03 08:07:08+02	2020-07-03 08:07:55+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-07-03 08:07:56+02	2020-07-03 08:09:55+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.14.2
2020-07-03 08:09:56+02	2020-07-03 08:11:34+02	textproc	py-rdflib	py37	py37-rdflib-4.2.2_1
2020-07-03 08:11:34+02	2020-07-03 08:11:50+02	audio	lv2	\N	lv2-1.18.0
2020-07-03 08:11:51+02	2020-07-03 08:14:48+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-07-03 08:14:48+02	2020-07-03 08:15:14+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-07-03 08:15:15+02	2020-07-03 08:16:33+02	graphics	frei0r	\N	frei0r-1.6.1
2020-07-03 08:16:33+02	2020-07-03 08:17:56+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-07-03 08:17:56+02	2020-07-03 08:18:59+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-07-03 08:01:29+02	2020-07-03 08:19:49+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-07-03 08:18:59+02	2020-07-03 08:20:02+02	security	libsecret	\N	libsecret-0.18.6_1
2020-07-03 08:19:50+02	2020-07-03 08:22:26+02	print	tex-formats	\N	tex-formats-20150521_2
2020-07-03 08:22:27+02	2020-07-03 08:23:29+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.71.0_2
2020-07-03 08:23:30+02	2020-07-03 08:24:14+02	devel	qt5-concurrent	\N	qt5-concurrent-5.14.2
2020-07-03 08:24:14+02	2020-07-03 08:25:20+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-07-03 08:20:02+02	2020-07-03 08:25:42+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-07-03 08:25:42+02	2020-07-03 08:26:02+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-07-03 08:25:21+02	2020-07-03 08:26:01+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-07-03 08:26:02+02	2020-07-03 08:26:20+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_1
2020-07-03 08:26:21+02	2020-07-03 08:26:52+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_1
2020-07-03 08:26:52+02	2020-07-03 08:27:27+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-07-03 08:27:27+02	2020-07-03 08:28:26+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-07-03 08:26:02+02	2020-07-03 08:29:36+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-07-03 08:28:26+02	2020-07-03 08:29:56+02	graphics	graphene	\N	graphene-1.10.0
2020-07-03 08:29:36+02	2020-07-03 08:30:40+02	net	geoclue	\N	geoclue-2.5.5
2020-07-03 08:30:41+02	2020-07-03 08:31:12+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-07-03 08:31:13+02	2020-07-03 08:31:47+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-07-03 08:31:47+02	2020-07-03 08:32:10+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-07-03 10:29:19+02	2020-07-03 10:30:30+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-07-03 08:29:57+02	2020-07-03 08:32:16+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-07-03 08:32:11+02	2020-07-03 08:32:29+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-07-03 08:32:17+02	2020-07-03 08:32:34+02	print	libraqm	\N	libraqm-0.6.0
2020-07-03 08:32:35+02	2020-07-03 08:36:37+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-07-03 08:36:39+02	2020-07-03 08:38:12+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-07-03 08:38:12+02	2020-07-03 08:38:34+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-07-03 08:38:34+02	2020-07-03 08:39:28+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-07-03 08:39:29+02	2020-07-03 08:41:19+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.71.0
2020-07-03 08:41:20+02	2020-07-03 08:41:33+02	textproc	py-toml	py37	py37-toml-0.10.0
2020-07-03 08:41:34+02	2020-07-03 08:42:21+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-07-03 08:42:22+02	2020-07-03 08:42:35+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-07-03 08:42:35+02	2020-07-03 08:47:40+02	devel	qt5-script	\N	qt5-script-5.14.2
2020-07-03 08:47:41+02	2020-07-03 08:48:42+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-07-03 08:48:42+02	2020-07-03 08:49:53+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-07-03 08:49:53+02	2020-07-03 08:50:42+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-07-03 08:32:30+02	2020-07-03 08:50:46+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-07-03 08:50:42+02	2020-07-03 08:50:54+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-07-03 08:50:47+02	2020-07-03 08:51:36+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-07-03 08:51:38+02	2020-07-03 08:52:53+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-03 08:50:55+02	2020-07-03 08:53:10+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-07-03 08:52:54+02	2020-07-03 08:53:41+02	sysutils	policykit	\N	policykit-0.9_10
2020-07-03 08:53:11+02	2020-07-03 08:53:43+02	textproc	asciidoc	\N	asciidoc-9.0.1
2020-07-03 08:53:41+02	2020-07-03 08:54:11+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-07-03 08:53:43+02	2020-07-03 08:54:34+02	archivers	kf5-karchive	\N	kf5-karchive-5.71.0
2020-07-03 08:54:35+02	2020-07-03 08:55:08+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-07-03 08:54:12+02	2020-07-03 08:55:18+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-07-03 08:55:08+02	2020-07-03 08:56:08+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-03 08:56:09+02	2020-07-03 08:56:53+02	www	py-django22	py37	py37-django22-2.2.13
2020-07-03 08:56:54+02	2020-07-03 08:57:30+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-07-03 08:55:19+02	2020-07-03 08:58:09+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-07-03 08:57:31+02	2020-07-03 08:58:54+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-07-03 08:58:54+02	2020-07-03 09:00:23+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-07-03 09:00:23+02	2020-07-03 09:08:12+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-03 09:08:13+02	2020-07-03 09:09:06+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-07-03 09:09:06+02	2020-07-03 09:10:02+02	sysutils	hal	\N	hal-0.5.14_34
2020-07-03 09:10:03+02	2020-07-03 09:11:06+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-07-03 09:11:07+02	2020-07-03 09:12:35+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-07-03 09:12:36+02	2020-07-03 09:13:12+02	comms	qt5-serialport	\N	qt5-serialport-5.14.2
2020-07-03 09:13:13+02	2020-07-03 09:14:07+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-07-03 09:14:08+02	2020-07-03 09:15:17+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-07-03 08:58:10+02	2020-07-03 09:15:47+02	devel	gdb	py37	gdb-9.2_1
2020-07-03 09:15:18+02	2020-07-03 09:16:43+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-07-03 09:16:44+02	2020-07-03 09:17:41+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-07-03 09:17:41+02	2020-07-03 09:18:35+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-07-03 09:18:36+02	2020-07-03 09:19:32+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-07-03 09:15:48+02	2020-07-03 09:21:24+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-07-03 09:21:25+02	2020-07-03 09:22:26+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-07-03 09:22:26+02	2020-07-03 09:23:14+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-03 09:19:32+02	2020-07-03 09:23:35+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-07-03 09:23:15+02	2020-07-03 09:24:12+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.71.0
2020-07-03 09:23:35+02	2020-07-03 09:24:25+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-07-03 09:24:26+02	2020-07-03 09:25:20+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-07-03 09:25:20+02	2020-07-03 09:26:41+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-07-03 09:24:12+02	2020-07-03 09:27:02+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-07-03 09:26:42+02	2020-07-03 09:28:28+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-07-03 09:28:29+02	2020-07-03 09:29:13+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-07-03 09:27:02+02	2020-07-03 09:30:08+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-07-03 09:29:14+02	2020-07-03 09:30:15+02	net	remmina	\N	remmina-1.3.9
2020-07-03 09:30:15+02	2020-07-03 09:30:47+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-07-03 09:30:48+02	2020-07-03 09:31:39+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-07-03 09:30:09+02	2020-07-03 09:31:46+02	security	gcr	\N	gcr-3.28.0
2020-07-03 09:31:47+02	2020-07-03 09:32:30+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-07-03 09:32:30+02	2020-07-03 09:33:16+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-07-03 09:31:40+02	2020-07-03 09:39:57+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-07-03 09:39:58+02	2020-07-03 09:41:48+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-07-03 09:41:48+02	2020-07-03 09:44:22+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.71.0
2020-07-03 09:44:23+02	2020-07-03 09:45:39+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-07-03 09:45:40+02	2020-07-03 09:46:40+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-07-03 09:46:41+02	2020-07-03 09:48:00+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-07-03 09:48:01+02	2020-07-03 10:13:37+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-03 10:13:38+02	2020-07-03 10:14:06+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-07-03 10:14:07+02	2020-07-03 10:14:46+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-07-03 10:14:47+02	2020-07-03 10:15:26+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-07-03 10:15:27+02	2020-07-03 10:15:40+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-07-03 10:15:41+02	2020-07-03 10:15:57+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-07-03 10:15:58+02	2020-07-03 10:16:11+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-07-03 10:16:12+02	2020-07-03 10:17:05+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-07-03 10:17:06+02	2020-07-03 10:17:45+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-07-03 10:17:45+02	2020-07-03 10:18:36+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-03 10:18:36+02	2020-07-03 10:19:27+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-07-03 10:19:28+02	2020-07-03 10:19:56+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-07-03 10:19:56+02	2020-07-03 10:20:59+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-07-03 10:21:00+02	2020-07-03 10:21:39+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-07-03 10:21:40+02	2020-07-03 10:24:30+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-03 10:24:31+02	2020-07-03 10:25:45+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-07-03 10:25:46+02	2020-07-03 10:28:19+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-07-03 10:28:20+02	2020-07-03 10:29:19+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-07-04 21:38:18+02	2020-07-04 21:41:42+02	devel	git	\N	git-2.27.0
2020-07-03 10:30:31+02	2020-07-03 10:31:37+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-07-03 10:31:38+02	2020-07-03 10:32:38+02	devel	libgtop	\N	libgtop-2.40.0
2020-07-03 10:32:39+02	2020-07-03 10:32:52+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-07-03 10:32:53+02	2020-07-03 10:34:43+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-07-03 10:34:44+02	2020-07-03 10:42:38+02	math	vtk8	\N	vtk8-8.2.0
2020-07-03 10:42:39+02	2020-07-03 10:43:20+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-03 10:43:20+02	2020-07-03 10:47:43+02	net	libmateweather	\N	libmateweather-1.24.0
2020-07-03 10:47:44+02	2020-07-03 10:48:49+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-07-03 10:48:50+02	2020-07-03 10:50:06+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-07-03 10:50:06+02	2020-07-03 10:51:04+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-07-03 10:51:05+02	2020-07-03 10:53:17+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-07-03 10:53:17+02	2020-07-03 11:02:43+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-03 11:02:44+02	2020-07-03 11:03:06+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.8_6
2020-07-03 11:03:06+02	2020-07-03 11:10:46+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-03 11:10:47+02	2020-07-03 11:13:02+02	devel	git	\N	git-2.27.0
2020-07-03 11:13:03+02	2020-07-03 11:13:57+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-03 11:13:58+02	2020-07-03 11:14:52+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-07-03 11:14:53+02	2020-07-03 11:16:27+02	devel	talloc	\N	talloc-2.3.0
2020-07-03 11:16:28+02	2020-07-03 11:17:31+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-03 11:17:32+02	2020-07-03 11:18:51+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-07-03 11:18:52+02	2020-07-03 11:22:19+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-07-03 11:22:20+02	2020-07-03 11:22:34+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7
2020-07-03 11:22:35+02	2020-07-03 11:23:10+02	sysutils	upower	\N	upower-0.99.11
2020-07-03 11:23:11+02	2020-07-03 11:29:42+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-07-03 11:29:43+02	2020-07-03 11:30:40+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-07-03 11:30:40+02	2020-07-03 11:31:07+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-07-03 11:31:07+02	2020-07-03 11:33:22+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-07-03 07:17:15+02	2020-07-03 11:33:43+02	devel	llvm80	\N	llvm80-8.0.1_3
2020-07-03 11:33:23+02	2020-07-03 11:34:11+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-07-03 11:34:11+02	2020-07-03 11:35:25+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-03 11:33:44+02	2020-07-03 11:36:01+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_7
2020-07-03 11:36:02+02	2020-07-03 11:53:49+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-07-03 11:53:58+02	2020-07-03 11:59:01+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-07-03 11:35:26+02	2020-07-03 12:03:24+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-07-03 11:59:04+02	2020-07-03 12:03:58+02	databases	tdb	\N	tdb-1.4.2,1
2020-07-03 12:04:02+02	2020-07-03 12:04:26+02	devel	gamin	\N	gamin-0.1.10_10
2020-07-03 12:04:26+02	2020-07-03 12:05:34+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-03 12:05:34+02	2020-07-03 12:06:21+02	devel	librest	\N	rest-0.7.93
2020-07-03 12:06:22+02	2020-07-03 12:06:35+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-07-03 12:06:35+02	2020-07-03 12:06:49+02	devel	py-more-itertools	py37	py37-more-itertools-8.4.0
2020-07-03 12:06:49+02	2020-07-03 12:07:53+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-07-03 12:07:54+02	2020-07-03 12:09:37+02	devel	tevent	\N	tevent-0.10.1
2020-07-03 12:09:38+02	2020-07-03 12:11:03+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-07-03 12:11:04+02	2020-07-03 12:13:35+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-07-03 12:13:35+02	2020-07-03 12:15:24+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-07-03 12:15:25+02	2020-07-03 12:16:11+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-07-03 12:16:11+02	2020-07-03 12:17:05+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-07-03 12:17:06+02	2020-07-03 12:17:58+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-03 12:17:59+02	2020-07-03 12:19:59+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-07-03 12:20:00+02	2020-07-03 12:20:10+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-07-03 12:20:10+02	2020-07-03 12:20:52+02	x11	babl	\N	babl-0.1.78
2020-07-03 12:20:53+02	2020-07-03 12:21:45+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-07-03 12:03:30+02	2020-07-03 12:22:21+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-03 12:21:46+02	2020-07-03 12:22:31+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-03 12:22:22+02	2020-07-03 12:23:09+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-07-03 12:22:32+02	2020-07-03 12:23:23+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-07-03 12:23:10+02	2020-07-03 12:23:54+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-07-03 12:23:24+02	2020-07-03 12:24:11+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-07-03 12:24:12+02	2020-07-03 12:25:02+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-07-03 12:23:54+02	2020-07-03 12:25:15+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-07-03 12:25:02+02	2020-07-03 12:25:51+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-07-03 12:25:51+02	2020-07-03 12:27:12+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-03 12:25:16+02	2020-07-03 12:27:50+02	x11-fm	caja	\N	caja-1.22.3_1
2020-07-03 12:27:13+02	2020-07-03 12:28:14+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-07-03 12:27:51+02	2020-07-03 12:28:55+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-03 12:28:14+02	2020-07-03 12:29:17+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-07-03 12:29:17+02	2020-07-03 12:29:26+02	devel	py-click	py37	py37-click-7.0
2020-07-03 12:28:56+02	2020-07-03 12:29:35+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-07-03 12:29:27+02	2020-07-03 12:29:37+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-07-03 12:29:35+02	2020-07-03 12:29:45+02	devel	py-olefile	py37	py37-olefile-0.46
2020-07-03 12:29:37+02	2020-07-03 12:29:47+02	devel	py-ply	py37	py37-ply-3.11
2020-07-03 12:29:45+02	2020-07-03 12:29:55+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-07-03 12:29:47+02	2020-07-03 12:29:56+02	devel	py-zipp	py37	py37-zipp-0.6.0_3
2020-07-03 12:29:56+02	2020-07-03 12:30:08+02	devel	scons	py37	scons-py37-3.1.2
2020-07-03 12:29:55+02	2020-07-03 12:31:14+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-07-03 12:31:14+02	2020-07-03 12:31:35+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-07-03 12:31:35+02	2020-07-03 12:32:33+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-07-03 12:30:09+02	2020-07-03 12:32:38+02	graphics	gegl	\N	gegl-0.4.24
2020-07-03 12:32:34+02	2020-07-03 12:32:51+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-07-03 12:32:52+02	2020-07-03 12:34:38+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-07-03 09:33:17+02	2020-07-03 12:38:04+02	lang	rust	\N	rust-1.44.1_1
2020-07-03 12:38:05+02	2020-07-03 12:39:14+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-07-03 12:39:15+02	2020-07-03 12:40:27+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-03 12:32:39+02	2020-07-03 12:42:25+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-07-03 12:40:27+02	2020-07-03 12:42:39+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-07-03 12:42:26+02	2020-07-03 12:43:08+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-07-03 12:42:39+02	2020-07-03 12:43:43+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-03 12:43:09+02	2020-07-03 12:43:52+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-07-03 12:43:44+02	2020-07-03 12:43:54+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-07-03 12:43:52+02	2020-07-03 12:44:02+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-07-03 12:43:54+02	2020-07-03 12:44:05+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-07-03 12:44:03+02	2020-07-03 12:45:25+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-07-03 12:44:05+02	2020-07-03 12:45:35+02	x11	zenity	\N	zenity-3.32.0
2020-07-03 12:45:26+02	2020-07-03 12:46:05+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-03 12:34:38+02	2020-07-03 12:46:15+02	net	samba410	\N	samba410-4.10.15
2020-07-03 12:45:36+02	2020-07-03 12:46:39+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-07-03 12:46:40+02	2020-07-03 12:46:49+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-10-05 01:52:39+02	2020-10-05 01:53:02+02	devel	dee	\N	dee-1.2.7_17
2020-07-03 12:46:15+02	2020-07-03 12:46:59+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-07-03 12:46:49+02	2020-07-03 12:47:00+02	databases	py-redis	py37	py37-redis-3.5.3
2020-07-03 12:47:01+02	2020-07-03 12:49:55+02	devel	libgdata	\N	libgdata-0.17.9
2020-07-03 12:47:00+02	2020-07-03 12:50:35+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-03 12:49:57+02	2020-07-03 12:51:15+02	devel	libpeas	\N	libpeas-1.22.0
2020-07-03 12:51:16+02	2020-07-03 12:51:28+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-07-03 12:51:29+02	2020-07-03 12:51:40+02	devel	py-appdirs	py37	py37-appdirs-1.4.3
2020-07-03 12:51:40+02	2020-07-03 12:51:50+02	devel	py-backports	py37	py37-backports-1
2020-07-03 12:51:50+02	2020-07-03 12:51:59+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-07-03 12:46:06+02	2020-07-03 12:51:59+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-07-03 12:52:00+02	2020-07-03 12:52:10+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-07-03 12:52:00+02	2020-07-03 12:52:11+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-07-03 12:52:11+02	2020-07-03 12:52:22+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-0.23
2020-07-03 12:52:23+02	2020-07-03 12:52:34+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-07-03 12:52:35+02	2020-07-03 12:52:46+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-07-03 12:52:11+02	2020-07-03 12:52:47+02	devel	py-gobject	py27	py27-gobject-2.28.6_8
2020-07-03 12:52:47+02	2020-07-03 12:52:59+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-07-03 12:52:48+02	2020-07-03 12:52:59+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-07-03 12:52:59+02	2020-07-03 12:53:11+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-07-03 12:52:59+02	2020-07-03 12:53:13+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-07-03 12:53:11+02	2020-07-03 12:53:23+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-07-03 12:53:14+02	2020-07-03 12:57:09+02	dns	bind-tools	\N	bind-tools-9.16.4
2020-07-03 12:57:09+02	2020-07-03 12:57:29+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-07-03 12:53:24+02	2020-07-03 12:58:37+02	editors	emacs	full	emacs-26.3_5,3
2020-07-03 12:58:38+02	2020-07-03 13:03:05+02	graphics	opencv	\N	opencv-3.4.1_33
2020-07-03 13:03:06+02	2020-07-03 13:04:16+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-07-03 13:04:16+02	2020-07-03 13:06:30+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-03 13:06:31+02	2020-07-03 13:07:14+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0
2020-07-03 13:07:15+02	2020-07-03 13:07:40+02	net	gssdp	\N	gssdp-1.2.3
2020-07-03 13:07:41+02	2020-07-03 13:08:37+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-03 13:08:37+02	2020-07-03 13:09:49+02	print	cups-filters	\N	cups-filters-1.27.5
2020-07-03 12:57:30+02	2020-07-03 13:10:44+02	graphics	gimp-app	\N	gimp-app-2.10.20,1
2020-07-03 13:09:50+02	2020-07-03 13:15:59+02	print	fontforge	\N	fontforge-20190801_2
2020-07-03 13:10:44+02	2020-07-03 13:23:24+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-07-03 13:23:25+02	2020-07-03 13:23:37+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-07-03 13:23:37+02	2020-07-03 13:24:32+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-07-03 13:24:33+02	2020-07-03 13:26:22+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-07-03 13:26:22+02	2020-07-03 13:26:33+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-07-03 13:26:33+02	2020-07-03 13:26:45+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-07-03 13:26:46+02	2020-07-03 13:26:58+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-07-03 13:15:59+02	2020-07-03 13:33:47+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-07-03 13:33:48+02	2020-07-03 13:34:01+02	www	py-h2	py37	py37-h2-3.2.0
2020-07-03 13:34:02+02	2020-07-03 13:34:15+02	www	py-hyperlink	py37	py37-hyperlink-19.0.0
2020-07-03 13:34:16+02	2020-07-03 13:34:32+02	www	py-priority	py37	py37-priority-1.3.0
2020-07-03 13:34:32+02	2020-07-03 13:34:50+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-07-03 13:34:50+02	2020-07-03 13:54:01+02	www	qt5-webengine	\N	qt5-webengine-5.14.2_1
2020-07-03 13:54:03+02	2020-07-03 13:55:38+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-07-03 13:55:39+02	2020-07-03 13:58:49+02	x11	xorg	\N	xorg-7.7_3
2020-07-03 13:58:50+02	2020-07-03 14:00:19+02	x11	yelp	\N	yelp-3.28.1_2
2020-07-03 14:00:20+02	2020-07-03 14:01:28+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-07-03 13:26:58+02	2020-07-03 14:01:33+02	www	chromium	\N	chromium-83.0.4103.116
2020-07-03 14:01:29+02	2020-07-03 14:02:17+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-07-03 14:01:35+02	2020-07-03 14:03:05+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-07-03 14:02:17+02	2020-07-03 14:03:21+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-07-03 14:03:06+02	2020-07-03 14:04:11+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-07-03 14:03:22+02	2020-07-03 14:04:30+02	x11-wm	marco	\N	marco-1.22.4
2020-07-03 14:04:31+02	2020-07-03 14:05:24+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-07-03 14:04:11+02	2020-07-03 14:05:54+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-07-03 14:05:25+02	2020-07-03 14:07:47+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-07-03 14:07:48+02	2020-07-03 14:09:59+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-03 14:10:00+02	2020-07-03 14:10:15+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-07-03 14:10:15+02	2020-07-03 14:10:30+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-07-03 14:05:55+02	2020-07-03 14:10:41+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-07-03 14:10:30+02	2020-07-03 14:10:42+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-07-03 14:10:42+02	2020-07-03 14:10:53+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-07-03 14:10:43+02	2020-07-03 14:10:55+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-07-03 14:10:53+02	2020-07-03 14:11:05+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-07-03 14:10:55+02	2020-07-03 14:11:40+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-07-03 14:11:41+02	2020-07-03 14:13:27+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-07-03 14:13:27+02	2020-07-03 14:15:25+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-03 14:15:26+02	2020-07-03 14:16:01+02	graphics	libgxps	\N	libgxps-0.3.1
2020-07-03 14:16:02+02	2020-07-03 14:16:44+02	graphics	libheif	\N	libheif-1.6.2
2020-07-03 14:11:06+02	2020-07-03 14:17:07+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-07-03 14:17:08+02	2020-07-03 14:17:38+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-07-03 14:16:45+02	2020-07-03 14:17:48+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-07-03 14:17:40+02	2020-07-03 14:18:34+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-03 14:18:35+02	2020-07-03 14:19:31+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-07-03 14:17:49+02	2020-07-03 14:20:00+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-07-03 14:19:32+02	2020-07-03 14:20:38+02	graphics	vigra	\N	vigra-1.11.1_10
2020-07-03 14:20:39+02	2020-07-03 14:26:01+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-07-03 14:26:01+02	2020-07-03 14:33:41+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-07-03 14:33:42+02	2020-07-03 14:34:36+02	multimedia	libass	\N	libass-0.14.0
2020-07-03 14:34:37+02	2020-07-03 14:35:45+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-03 14:20:02+02	2020-07-03 14:37:27+02	math	R	\N	R-4.0.2
2020-07-03 14:35:46+02	2020-07-03 14:37:49+02	net	freerdp	\N	freerdp-2.1.1
2020-07-03 14:37:28+02	2020-07-03 14:37:58+02	net	gupnp	\N	gupnp-1.1.1_1
2020-07-03 14:37:50+02	2020-07-03 14:38:06+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-07-03 14:37:59+02	2020-07-03 14:38:11+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-07-03 14:38:07+02	2020-07-03 14:38:52+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-07-03 14:38:11+02	2020-07-03 14:39:24+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-03 14:38:52+02	2020-07-03 14:39:40+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-03 14:39:41+02	2020-07-03 14:40:10+02	print	libspectre	\N	libspectre-0.2.9
2020-07-03 14:40:11+02	2020-07-03 14:42:18+02	print	mftrace	\N	mftrace-1.2.18_2
2020-07-03 14:39:25+02	2020-07-03 14:43:15+02	print	gutenprint	\N	gutenprint-5.3.3
2020-07-03 14:42:19+02	2020-07-03 14:48:25+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-07-03 14:43:16+02	2020-07-03 14:48:53+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-07-03 14:48:26+02	2020-07-03 14:49:14+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-07-03 14:48:54+02	2020-07-03 14:49:32+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-07-03 14:49:15+02	2020-07-03 14:49:32+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-07-03 14:49:33+02	2020-07-03 14:49:51+02	sysutils	hal-info	\N	hal-info-20091130
2020-07-03 14:49:52+02	2020-07-03 14:50:45+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-07-03 14:49:33+02	2020-07-03 14:52:15+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-07-03 14:50:45+02	2020-07-03 14:52:46+02	textproc	dblatex	\N	dblatex-0.3.11
2020-07-03 14:52:16+02	2020-07-03 14:52:48+02	textproc	enchant	\N	enchant-1.6.0_9
2020-07-03 14:52:47+02	2020-07-03 14:52:59+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-07-03 14:52:49+02	2020-07-03 14:53:00+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-07-03 14:53:00+02	2020-07-03 14:53:12+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-07-03 14:53:01+02	2020-07-03 14:53:23+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-07-03 14:53:13+02	2020-07-03 14:53:55+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-03 14:53:56+02	2020-07-03 14:54:07+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-07-03 14:53:23+02	2020-07-03 14:54:44+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-03 14:54:07+02	2020-07-03 14:54:46+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-03 14:54:47+02	2020-07-03 14:55:20+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-07-03 14:54:45+02	2020-07-03 14:56:15+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-07-03 14:55:20+02	2020-07-03 14:56:24+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-07-03 14:56:16+02	2020-07-03 14:57:02+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-07-03 14:56:25+02	2020-07-03 14:58:23+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-07-03 14:57:03+02	2020-07-03 14:59:04+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-07-03 14:59:05+02	2020-07-03 15:01:48+02	archivers	engrampa	\N	engrampa-1.24.0
2020-07-03 15:01:49+02	2020-07-03 15:02:58+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-07-03 15:02:59+02	2020-07-03 15:04:15+02	audio	mate-media	\N	mate-media-1.24.0
2020-07-03 15:04:16+02	2020-07-03 15:04:37+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-07-03 15:04:38+02	2020-07-03 15:06:21+02	databases	rrdtool	\N	rrdtool-1.7.2_3
2020-07-03 15:06:21+02	2020-07-03 15:08:00+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-07-03 15:08:01+02	2020-07-03 15:11:52+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-07-03 15:11:53+02	2020-07-03 15:12:56+02	deskutils	mozo	\N	mozo-1.24.0
2020-07-03 15:12:57+02	2020-07-03 15:13:56+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-07-03 15:13:57+02	2020-07-03 15:14:57+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0_1
2020-07-03 15:14:57+02	2020-07-03 15:15:12+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-07-03 15:15:12+02	2020-07-03 15:15:28+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-07-03 15:15:28+02	2020-07-03 15:16:02+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-07-03 15:16:03+02	2020-07-03 15:16:18+02	devel	py-funcy	py37	py37-funcy-1.12
2020-07-03 14:58:24+02	2020-07-03 15:17:33+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-07-03 15:17:34+02	2020-07-03 15:17:46+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-07-03 15:16:19+02	2020-07-03 15:17:54+02	devel	py-game	py37	py37-game-1.9.6_1
2020-07-03 15:17:47+02	2020-07-03 15:17:58+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-07-03 15:17:58+02	2020-07-03 15:18:09+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-07-03 15:17:55+02	2020-07-03 15:18:12+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-07-03 15:18:09+02	2020-07-03 15:18:20+02	devel	py-rq	py37	py37-rq-1.3.0
2020-07-03 15:18:21+02	2020-07-03 15:18:33+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-07-03 15:18:13+02	2020-07-03 15:18:43+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-07-03 15:18:34+02	2020-07-03 15:18:46+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-07-03 15:18:43+02	2020-07-03 15:18:55+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-07-03 15:18:55+02	2020-07-03 15:19:08+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-07-03 15:18:46+02	2020-07-03 15:24:41+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3
2020-07-03 15:19:09+02	2020-07-03 15:42:28+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-07-03 15:42:29+02	2020-07-03 15:46:12+02	editors	pluma	\N	pluma-1.24.0
2020-07-03 15:24:42+02	2020-07-03 15:46:29+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-07-03 15:46:30+02	2020-07-03 15:48:36+02	graphics	eom	\N	eom-1.24.0
2020-07-03 15:46:13+02	2020-07-03 15:49:21+02	graphics	atril	\N	atril-1.22.3_6
2020-07-03 15:48:37+02	2020-07-03 15:50:12+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-03 15:49:22+02	2020-07-03 15:50:44+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-07-03 15:50:13+02	2020-07-03 15:51:08+02	graphics	openimageio	\N	openimageio-2.1.15.0
2020-07-03 15:50:44+02	2020-07-03 15:52:20+02	graphics	py-gimp	\N	py27-gimp-2.10.20
2020-07-03 15:51:09+02	2020-07-03 16:14:53+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-07-03 15:52:21+02	2020-07-03 16:17:32+02	java	openjdk11	\N	openjdk11-11.0.7+10.2_2
2020-07-03 16:17:33+02	2020-07-03 16:17:44+02	lang	python3	\N	python3-3_3
2020-07-03 16:17:44+02	2020-07-03 16:19:37+02	math	mate-calc	\N	mate-calc-1.24.0
2020-07-03 16:19:38+02	2020-07-03 16:20:41+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-07-03 16:14:54+02	2020-07-03 16:22:32+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-07-03 16:20:42+02	2020-07-03 16:23:45+02	math	stp	\N	stp-2.3.3_5,1
2020-07-03 16:22:33+02	2020-07-03 16:28:03+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-07-03 16:28:04+02	2020-07-03 16:30:58+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-03 16:23:45+02	2020-07-03 16:32:44+02	misc	openvdb	\N	openvdb-7.0.0_2
2020-07-03 16:30:58+02	2020-07-03 16:32:56+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-07-03 16:32:56+02	2020-07-03 16:34:03+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-03 16:34:04+02	2020-07-03 16:34:17+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-07-03 16:34:17+02	2020-07-03 16:34:53+02	net	py-pyzmq	py37	py37-pyzmq-19.0.1
2020-07-03 16:34:54+02	2020-07-03 16:35:44+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-07-03 16:35:44+02	2020-07-03 16:36:49+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-07-03 16:36:49+02	2020-07-03 16:37:37+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-07-03 16:32:46+02	2020-07-03 16:38:09+02	multimedia	vlc	\N	vlc-3.0.11_1,4
2020-07-03 16:37:38+02	2020-07-03 16:38:30+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-07-03 16:38:09+02	2020-07-03 16:38:54+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-07-03 16:38:31+02	2020-07-03 16:39:16+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-07-03 16:38:55+02	2020-07-03 16:39:52+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-07-03 16:39:16+02	2020-07-03 16:40:33+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-03 16:39:53+02	2020-07-03 16:40:33+02	net	vde2	\N	vde2-2.3.2_4
2020-07-03 16:40:33+02	2020-07-03 16:41:24+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-03 16:40:34+02	2020-07-03 16:41:47+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-03 16:41:48+02	2020-07-03 16:42:00+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.7.1
2020-07-03 16:41:24+02	2020-07-03 16:42:50+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-07-03 16:42:01+02	2020-07-03 16:43:21+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-03 16:43:21+02	2020-07-03 16:51:56+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-07-03 16:42:51+02	2020-07-03 16:54:02+02	print	lilypond	\N	lilypond-2.20.0
2020-07-03 16:54:03+02	2020-07-03 16:56:03+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-07-03 16:51:57+02	2020-07-03 16:57:48+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-07-03 16:56:04+02	2020-07-03 17:03:35+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-07-03 17:03:36+02	2020-07-03 17:04:02+02	print	transfig	\N	transfig-3.2.5e_4
2020-07-03 17:04:03+02	2020-07-03 17:04:15+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-07-03 17:04:15+02	2020-07-03 17:04:29+02	security	py-stem	py37	py37-stem-1.8.0
2020-07-03 17:04:29+02	2020-07-03 17:06:07+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-07-03 17:06:08+02	2020-07-03 17:08:03+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-07-03 17:08:04+02	2020-07-03 17:08:52+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-07-03 17:08:52+02	2020-07-03 17:10:30+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-07-03 17:10:31+02	2020-07-03 17:10:58+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-07-03 17:10:59+02	2020-07-03 17:11:08+02	textproc	py-scour	py37	py37-scour-0.37
2020-07-03 17:11:09+02	2020-07-03 17:11:18+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-07-03 17:11:19+02	2020-07-03 17:11:40+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-07-03 17:11:40+02	2020-07-03 17:11:58+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.0
2020-07-03 17:11:59+02	2020-07-03 17:12:09+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-07-03 17:12:09+02	2020-07-03 17:12:21+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-07-03 17:12:21+02	2020-07-03 17:12:31+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-07-03 17:12:31+02	2020-07-03 17:12:41+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-07-03 17:12:42+02	2020-07-03 17:12:53+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-07-03 17:12:53+02	2020-07-03 17:13:03+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-07-03 17:13:03+02	2020-07-03 17:14:24+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-03 17:14:25+02	2020-07-03 17:15:37+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-03 17:15:37+02	2020-07-03 17:16:04+02	www	qt5-websockets	\N	qt5-websockets-5.14.2
2020-07-03 16:57:48+02	2020-07-03 17:16:29+02	print	texlive-docs	\N	texlive-docs-20150523
2020-07-03 17:16:30+02	2020-07-03 17:17:30+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-03 12:50:35+02	2020-07-03 17:18:32+02	devel	llvm90	\N	llvm90-9.0.1_1
2020-07-03 17:17:31+02	2020-07-03 17:18:43+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-07-03 17:18:44+02	2020-07-03 17:19:40+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-07-03 17:18:34+02	2020-07-03 17:20:04+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-07-03 17:16:05+02	2020-07-03 17:20:09+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-07-03 17:19:41+02	2020-07-03 17:20:16+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-07-03 17:20:17+02	2020-07-03 17:24:17+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-03 17:20:04+02	2020-07-03 17:24:19+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-07-03 17:24:19+02	2020-07-03 17:26:05+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-03 17:20:09+02	2020-07-03 17:26:17+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-03 17:26:18+02	2020-07-03 17:27:45+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-03 17:24:17+02	2020-07-03 17:28:17+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-03 17:27:46+02	2020-07-03 17:29:16+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-03 17:29:16+02	2020-07-03 17:29:38+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-07-03 17:29:39+02	2020-07-03 17:33:48+02	deskutils	calibre	\N	calibre-4.19.0
2020-07-03 17:33:49+02	2020-07-03 17:34:54+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-07-03 17:28:17+02	2020-07-03 17:41:25+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-07-03 17:41:26+02	2020-07-03 17:42:38+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-07-03 17:34:54+02	2020-07-03 17:57:32+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-03 17:42:39+02	2020-07-03 18:03:45+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-03 17:57:33+02	2020-07-03 18:44:07+02	devel	ghidra	\N	ghidra-9.1
2020-07-03 17:26:06+02	2020-07-03 18:54:00+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-03 18:54:02+02	2020-07-03 18:54:17+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-07-03 18:54:18+02	2020-07-03 18:54:51+02	devel	tig	\N	tig-2.5.1
2020-07-03 18:54:52+02	2020-07-03 18:58:05+02	dns	bind916	\N	bind916-9.16.4
2020-07-03 18:58:05+02	2020-07-03 19:48:25+02	editors	codelite	\N	codelite-12.0_4
2020-07-03 19:48:26+02	2020-07-03 19:50:52+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-07-03 19:50:52+02	2020-07-03 19:53:15+02	editors	vim	\N	vim-8.2.1110
2020-07-03 19:53:16+02	2020-07-03 20:25:11+02	emulators	qemu	\N	qemu-4.1.1_4
2020-07-03 20:25:12+02	2020-07-03 21:35:10+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-07-03 21:35:11+02	2020-07-03 21:44:39+02	emulators	wine	\N	wine-5.0.1,1
2020-07-03 21:44:40+02	2020-07-03 21:59:24+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-07-03 21:59:25+02	2020-07-03 22:00:11+02	games	monsterz	\N	monsterz-0.7.1_15
2020-07-03 22:00:12+02	2020-07-03 22:00:49+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-07-03 22:00:50+02	2020-07-03 22:01:47+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-07-03 22:01:48+02	2020-07-03 22:03:10+02	games	xboard	\N	xboard-4.9.1_1
2020-07-03 22:03:11+02	2020-07-03 22:04:04+02	games	xcowsay	\N	xcowsay-1.5
2020-07-03 22:04:04+02	2020-07-03 22:15:02+02	graphics	blender	\N	blender-2.83.1
2020-07-03 22:15:03+02	2020-07-03 22:18:12+02	graphics	dia	\N	dia-0.97.3_1,1
2020-07-03 22:18:13+02	2020-07-03 22:19:29+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-03 22:19:30+02	2020-07-03 22:24:12+02	graphics	inkscape	\N	inkscape-1.0_2
2020-07-03 22:24:13+02	2020-07-03 22:36:56+02	graphics	krita	\N	krita-4.3.0
2020-07-03 22:36:56+02	2020-07-03 22:37:11+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-07-03 22:37:12+02	2020-07-03 22:41:12+02	graphics	wings	\N	wings-2.2.5
2020-07-03 22:41:13+02	2020-07-03 22:42:23+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-03 22:42:23+02	2020-07-03 22:44:34+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-07-03 22:44:35+02	2020-07-03 22:45:46+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-07-03 22:45:46+02	2020-07-03 23:17:46+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.2
2020-07-03 18:03:47+02	2020-07-03 23:46:06+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200519
2020-07-03 23:17:47+02	2020-07-03 23:51:50+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-07-03 23:51:51+02	2020-07-03 23:52:06+02	lang	python	\N	python-3.7_3,2
2020-07-03 23:52:06+02	2020-07-03 23:55:12+02	misc	mc	\N	mc-nox11-4.8.24
2020-07-03 23:55:13+02	2020-07-03 23:56:19+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-03 23:56:20+02	2020-07-03 23:59:26+02	multimedia	obs-studio	\N	obs-studio-25.0.8
2020-07-03 23:59:27+02	2020-07-04 00:02:05+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-04 00:02:05+02	2020-07-04 00:09:55+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-04 00:09:55+02	2020-07-04 00:13:06+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-07-04 00:13:07+02	2020-07-04 00:14:37+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-04 00:14:38+02	2020-07-04 00:20:33+02	net	wireshark	\N	wireshark-3.2.4
2020-07-04 00:20:34+02	2020-07-04 00:21:13+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-07-03 18:44:08+02	2020-07-04 00:24:45+02	devel	llvm10	\N	llvm10-10.0.0_1
2020-07-03 23:46:07+02	2020-07-04 00:25:03+02	lang	mono	\N	mono-5.10.1.57_3
2020-07-04 00:24:46+02	2020-07-04 00:25:25+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-04 00:25:04+02	2020-07-04 00:25:42+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-04 00:25:26+02	2020-07-04 00:26:04+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-04 00:25:42+02	2020-07-04 00:26:19+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-04 00:26:04+02	2020-07-04 00:26:44+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-04 00:26:19+02	2020-07-04 00:27:03+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-07-04 00:26:44+02	2020-07-04 00:27:25+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-04 00:27:03+02	2020-07-04 00:28:02+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-04 00:27:25+02	2020-07-04 00:28:06+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-04 00:28:02+02	2020-07-04 00:28:40+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-04 00:28:07+02	2020-07-04 00:28:48+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-07-04 00:28:49+02	2020-07-04 00:29:14+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-07-04 00:29:15+02	2020-07-04 00:29:51+02	print	gv	\N	gv-3.7.4_5
2020-07-04 00:28:40+02	2020-07-04 00:30:52+02	print	auctex	full	auctex-emacs26-12.2
2020-07-04 00:21:14+02	2020-07-04 00:32:48+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-07-04 00:29:51+02	2020-07-04 00:32:56+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-04 00:32:57+02	2020-07-04 00:37:01+02	security	klee	\N	klee-2.1
2020-07-04 00:37:02+02	2020-07-04 00:37:14+02	security	nyx	\N	nyx-2.1.0_1
2020-07-04 00:32:49+02	2020-07-04 00:37:29+02	print	texlive-full	\N	texlive-full-20150521
2020-07-04 00:37:15+02	2020-07-04 00:38:05+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-04 00:37:30+02	2020-07-04 00:38:22+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-07-04 00:38:22+02	2020-07-04 00:39:09+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-07-04 00:39:09+02	2020-07-04 00:39:26+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-07-04 00:30:53+02	2020-07-04 00:39:27+02	print	scribus-devel	\N	scribus-devel-1.5.5_14
2020-07-04 00:39:28+02	2020-07-04 00:39:42+02	sysutils	panicmail	\N	panicmail-1.1
2020-07-04 00:38:06+02	2020-07-04 00:39:43+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-07-04 00:39:42+02	2020-07-04 00:39:56+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-07-04 00:39:26+02	2020-07-04 00:40:05+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-07-04 00:40:05+02	2020-07-04 00:40:15+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-07-04 00:39:56+02	2020-07-04 00:40:18+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-07-04 00:40:16+02	2020-07-04 00:40:26+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-07-04 00:40:18+02	2020-07-04 00:40:29+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-07-04 00:40:27+02	2020-07-04 00:40:36+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-07-04 00:40:29+02	2020-07-04 00:40:39+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-07-04 00:40:37+02	2020-07-04 00:40:45+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-07-04 00:40:39+02	2020-07-04 00:40:50+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-07-04 00:40:46+02	2020-07-04 00:40:56+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-07-04 00:40:50+02	2020-07-04 00:41:00+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-07-04 00:40:56+02	2020-07-04 00:41:05+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-07-04 00:41:01+02	2020-07-04 00:41:14+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-07-04 00:41:06+02	2020-07-04 00:41:15+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-07-04 00:41:16+02	2020-07-04 00:41:58+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-04 00:41:59+02	2020-07-04 00:43:25+02	x11	mate	\N	mate-1.22.2
2020-07-04 00:43:26+02	2020-07-04 00:50:47+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-07-04 00:50:48+02	2020-07-04 01:03:54+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-07-04 01:03:57+02	2020-07-04 01:09:00+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-07-04 00:41:14+02	2020-07-04 01:10:25+02	x11	alacritty	\N	alacritty-0.4.3_1
2020-07-04 01:09:03+02	2020-07-04 01:12:15+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-07-04 01:12:18+02	2020-07-04 01:15:07+02	graphics	xfig	\N	xfig-3.2.7a
2020-07-04 01:15:10+02	2020-07-04 01:17:10+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-07-04 01:10:29+02	2020-07-04 01:48:57+02	editors	vscode	\N	vscode-1.46.1
2020-07-04 00:39:44+02	2020-07-04 02:41:41+02	www	firefox	\N	firefox-78.0.1,1
2020-07-04 02:41:43+02	2020-07-04 02:54:54+02	local	E590T-localbase	\N	E590T-localbase-2020070100
2020-07-04 11:07:46+02	2020-07-04 11:07:55+02	devel	opencl	\N	opencl-2.2_2
2020-07-04 11:07:46+02	2020-07-04 11:08:15+02	www	py-django22	py37	py37-django22-2.2.14
2020-07-04 11:08:16+02	2020-07-04 11:10:58+02	devel	sdl20	\N	sdl2-2.0.12
2020-07-04 11:07:55+02	2020-07-04 11:11:02+02	multimedia	libx264	\N	libx264-0.160.3011
2020-07-04 11:07:46+02	2020-07-04 11:11:59+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_8
2020-07-04 11:11:59+02	2020-07-04 11:13:50+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-07-04 11:13:50+02	2020-07-04 11:14:58+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-04 11:14:58+02	2020-07-04 11:16:00+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-07-04 11:16:02+02	2020-07-04 11:17:05+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-07-04 11:17:06+02	2020-07-04 11:18:08+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-07-04 11:18:09+02	2020-07-04 11:19:13+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-07-04 11:19:14+02	2020-07-04 11:20:52+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-07-04 11:20:53+02	2020-07-04 11:21:55+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-07-04 11:21:56+02	2020-07-04 11:22:58+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-07-04 11:22:59+02	2020-07-04 11:25:15+02	graphics	gegl	\N	gegl-0.4.24
2020-07-04 11:25:16+02	2020-07-04 11:26:10+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-04 11:11:02+02	2020-07-04 11:26:42+02	multimedia	ffmpeg	\N	ffmpeg-4.3_3,1
2020-07-04 11:26:43+02	2020-07-04 11:27:25+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-04 11:26:10+02	2020-07-04 11:27:39+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-07-04 11:27:26+02	2020-07-04 11:28:00+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-04 11:27:40+02	2020-07-04 11:34:18+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-04 11:28:01+02	2020-07-04 11:34:42+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-07-04 11:34:42+02	2020-07-04 11:44:33+02	graphics	gimp-app	\N	gimp-app-2.10.20,1
2020-07-04 11:44:34+02	2020-07-04 11:49:31+02	graphics	opencv	\N	opencv-3.4.1_33
2020-07-04 11:49:32+02	2020-07-04 11:50:04+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-07-04 11:34:19+02	2020-07-04 11:51:04+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-04 11:51:05+02	2020-07-04 12:09:11+02	www	qt5-webengine	\N	qt5-webengine-5.14.2_1
2020-07-04 12:09:12+02	2020-07-04 12:12:34+02	x11	xorg	\N	xorg-7.7_3
2020-07-04 12:12:35+02	2020-07-04 12:14:23+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-04 12:14:24+02	2020-07-04 12:15:54+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-04 12:15:55+02	2020-07-04 12:17:14+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-04 12:17:15+02	2020-07-04 12:18:02+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-04 12:18:03+02	2020-07-04 12:19:53+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-07-04 12:19:54+02	2020-07-04 12:20:37+02	audio	faudio	\N	FAudio-20.07
2020-07-04 12:20:38+02	2020-07-04 12:22:08+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-04 12:22:09+02	2020-07-04 12:23:44+02	graphics	py-gimp	\N	py27-gimp-2.10.20
2020-07-04 11:50:05+02	2020-07-04 12:23:45+02	www	chromium	\N	chromium-83.0.4103.116
2020-07-04 12:23:48+02	2020-07-04 12:25:03+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-04 12:23:45+02	2020-07-04 12:25:13+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-04 12:25:04+02	2020-07-04 12:26:59+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-07-04 12:27:00+02	2020-07-04 12:28:03+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-04 12:28:04+02	2020-07-04 12:29:37+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-04 12:25:13+02	2020-07-04 12:30:46+02	multimedia	vlc	\N	vlc-3.0.11_2,4
2020-07-04 12:29:38+02	2020-07-04 12:30:52+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-04 12:30:53+02	2020-07-04 12:31:06+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-07-04 12:31:06+02	2020-07-04 12:31:19+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-07-04 12:31:19+02	2020-07-04 12:31:32+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-07-04 12:30:46+02	2020-07-04 12:31:53+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-04 12:31:33+02	2020-07-04 12:32:31+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-04 12:31:54+02	2020-07-04 12:32:57+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-04 12:32:37+02	2020-07-04 12:34:24+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-04 12:34:25+02	2020-07-04 12:35:52+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-04 12:32:31+02	2020-07-04 12:36:29+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-04 12:35:53+02	2020-07-04 12:37:10+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-04 12:36:30+02	2020-07-04 12:40:36+02	deskutils	calibre	\N	calibre-4.20.0
2020-07-04 12:37:11+02	2020-07-04 12:58:51+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-04 12:58:52+02	2020-07-04 12:59:04+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-07-04 12:40:37+02	2020-07-04 13:03:27+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-04 13:03:31+02	2020-07-04 13:09:52+02	emulators	wine	\N	wine-5.0.1,1
2020-07-04 12:59:05+02	2020-07-04 13:13:22+02	emulators	qemu	\N	qemu-4.1.1_4
2020-07-04 13:13:23+02	2020-07-04 13:14:28+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-04 13:09:52+02	2020-07-04 13:17:08+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-04 13:17:08+02	2020-07-04 13:18:00+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-04 13:18:01+02	2020-07-04 13:20:12+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-04 13:20:13+02	2020-07-04 13:22:41+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-04 13:22:41+02	2020-07-04 13:25:26+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-04 13:25:26+02	2020-07-04 13:26:48+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-04 13:26:48+02	2020-07-04 13:28:02+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-04 13:28:02+02	2020-07-04 13:28:10+02	security	doas	\N	doas-6.3_1
2020-07-04 12:32:57+02	2020-07-04 14:05:52+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-04 14:05:53+02	2020-07-04 14:06:06+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-07-04 14:06:07+02	2020-07-04 14:06:18+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-07-04 14:06:19+02	2020-07-04 14:06:32+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-07-04 14:06:33+02	2020-07-04 14:06:45+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-07-04 14:06:46+02	2020-07-04 14:06:59+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-07-04 14:06:59+02	2020-07-04 14:07:14+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-07-04 14:07:14+02	2020-07-04 14:07:27+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-07-04 14:07:28+02	2020-07-04 14:07:40+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-07-04 14:07:40+02	2020-07-04 14:07:53+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-07-04 14:07:54+02	2020-07-04 14:08:11+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-07-04 13:28:10+02	2020-07-04 14:14:35+02	www	firefox	\N	firefox-78.0.1,1
2020-07-04 14:14:37+02	2020-07-04 14:14:46+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-04 14:14:47+02	2020-07-04 14:15:21+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-07-04 14:08:11+02	2020-07-04 14:39:22+02	editors	vscode	\N	vscode-1.46.1
2020-07-04 13:14:28+02	2020-07-04 15:04:28+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200627
2020-07-04 21:06:59+02	2020-07-04 21:07:16+02	devel	libffi	\N	libffi-3.3
2020-07-04 21:06:59+02	2020-07-04 21:07:32+02	devel	rlwrap	\N	rlwrap-0.42_2
2020-07-04 21:06:59+02	2020-07-04 21:08:10+02	devel	libuv	\N	libuv-1.38.1
2020-07-04 21:07:16+02	2020-07-04 21:08:47+02	security	p11-kit	\N	p11-kit-0.23.20_1
2020-07-04 21:06:59+02	2020-07-04 21:08:56+02	java	bootstrap-openjdk8	\N	bootstrap-openjdk8-r450802_1
2020-07-04 21:06:59+02	2020-07-04 21:09:28+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-04 21:07:16+02	2020-07-04 21:11:07+02	lang	python37	\N	python37-3.7.8_1
2020-07-04 21:11:08+02	2020-07-04 21:11:26+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-07-04 21:11:27+02	2020-07-04 21:11:43+02	devel	py-six	py37	py37-six-1.14.0
2020-07-04 21:07:33+02	2020-07-04 21:11:51+02	lang	python27	\N	python27-2.7.18_1
2020-07-04 21:11:44+02	2020-07-04 21:12:00+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-07-04 21:11:52+02	2020-07-04 21:12:08+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-07-04 21:12:02+02	2020-07-04 21:12:17+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-07-04 21:12:09+02	2020-07-04 21:12:30+02	devel	py-cffi	py37	py37-cffi-1.14.0_1
2020-07-04 21:12:18+02	2020-07-04 21:12:31+02	dns	py-idna	py37	py37-idna-2.8
2020-07-04 21:12:31+02	2020-07-04 21:12:53+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-07-04 21:08:57+02	2020-07-04 21:13:06+02	lang	python38	\N	python38-3.8.3_1
2020-07-04 21:12:53+02	2020-07-04 21:13:06+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-07-04 21:12:32+02	2020-07-04 21:13:11+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-07-04 21:13:07+02	2020-07-04 21:13:19+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-07-04 21:13:07+02	2020-07-04 21:13:19+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-07-04 21:08:48+02	2020-07-04 21:13:21+02	security	gnutls	\N	gnutls-3.6.14
2020-07-04 21:13:21+02	2020-07-04 21:13:33+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-07-04 21:13:19+02	2020-07-04 21:13:34+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-07-04 21:13:33+02	2020-07-04 21:13:45+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-07-04 21:13:34+02	2020-07-04 21:13:47+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-07-04 21:13:46+02	2020-07-04 21:13:59+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-07-04 21:13:48+02	2020-07-04 21:14:07+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-07-04 21:13:59+02	2020-07-04 21:14:13+02	www	py-requests	py37	py37-requests-2.22.0
2020-07-04 21:13:19+02	2020-07-04 21:14:18+02	lang	cython	py37	py37-cython-0.29.15
2020-07-04 21:14:08+02	2020-07-04 21:14:19+02	devel	py-packaging	py37	py37-packaging-20.4
2020-07-04 21:14:19+02	2020-07-04 21:14:31+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-07-04 21:14:18+02	2020-07-04 21:14:32+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-07-04 21:14:31+02	2020-07-04 21:14:43+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-07-04 21:14:33+02	2020-07-04 21:14:46+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-07-04 21:14:43+02	2020-07-04 21:14:55+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-07-04 21:14:47+02	2020-07-04 21:14:59+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-07-04 21:13:12+02	2020-07-04 21:14:59+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-07-04 21:14:56+02	2020-07-04 21:15:08+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-07-04 21:14:59+02	2020-07-04 21:15:11+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-07-04 21:14:59+02	2020-07-04 21:15:12+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-07-04 21:14:13+02	2020-07-04 21:15:27+02	devel	ninja	\N	ninja-1.10.0,2
2020-07-04 21:15:09+02	2020-07-04 21:15:28+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-07-04 21:15:12+02	2020-07-04 21:15:31+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-07-04 21:15:12+02	2020-07-04 21:15:34+02	devel	py-future	py37	py37-future-0.18.2
2020-07-04 21:15:32+02	2020-07-04 21:15:44+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-07-04 21:15:28+02	2020-07-04 21:15:46+02	devel	meson	\N	meson-0.54.3
2020-07-04 21:15:35+02	2020-07-04 21:15:46+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-07-04 21:15:29+02	2020-07-04 21:15:54+02	textproc	py-sphinx	py37	py37-sphinx-3.0.4,1
2020-07-04 21:15:44+02	2020-07-04 21:15:57+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-07-04 21:15:46+02	2020-07-04 21:16:07+02	graphics	wayland	\N	wayland-1.18.0_4
2020-07-04 21:15:58+02	2020-07-04 21:16:14+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-07-04 21:16:08+02	2020-07-04 21:16:47+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-07-04 21:09:28+02	2020-07-04 21:17:05+02	lang	ecl	\N	ecl-20.4.24_1
2020-07-04 21:16:48+02	2020-07-04 21:17:06+02	textproc	itstool	\N	itstool-2.0.6
2020-07-04 21:17:06+02	2020-07-04 21:17:23+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-07-04 21:17:24+02	2020-07-04 21:17:41+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-07-04 21:17:42+02	2020-07-04 21:18:02+02	devel	py-setuptools	py27	py27-setuptools-44.0.0
2020-07-04 21:17:06+02	2020-07-04 21:18:17+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-07-04 21:18:05+02	2020-07-04 21:18:40+02	multimedia	libva	\N	libva-2.8.0
2020-07-04 21:18:18+02	2020-07-04 21:19:10+02	devel	py-lxml	py37	py37-lxml-4.5.1
2020-07-04 21:18:41+02	2020-07-04 21:19:29+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-07-04 21:19:14+02	2020-07-04 21:20:01+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-07-04 21:19:33+02	2020-07-04 21:20:24+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-07-04 21:20:08+02	2020-07-04 21:20:47+02	textproc	py-sparqlwrapper	py37	py37-sparqlwrapper-1.8.5
2020-07-04 21:20:32+02	2020-07-04 21:21:18+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-07-04 21:21:22+02	2020-07-04 21:21:48+02	textproc	py-toml	py37	py37-toml-0.10.0
2020-07-04 21:21:48+02	2020-07-04 21:22:05+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-07-04 21:22:06+02	2020-07-04 21:22:32+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-07-04 21:15:47+02	2020-07-04 21:24:12+02	devel	glib20	\N	glib-2.56.3_9,1
2020-07-04 21:22:35+02	2020-07-04 21:24:14+02	textproc	asciidoc	\N	asciidoc-9.0.1
2020-07-04 21:20:50+02	2020-07-04 21:24:16+02	textproc	py-rdflib	py37	py37-rdflib-4.2.2_1
2020-07-04 21:24:19+02	2020-07-04 21:27:13+02	misc	shared-mime-info	\N	shared-mime-info-1.10_2
2020-07-04 21:24:17+02	2020-07-04 21:27:44+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-07-04 21:24:19+02	2020-07-04 21:27:48+02	textproc	libcroco	\N	libcroco-0.6.13
2020-07-04 21:27:51+02	2020-07-04 21:28:36+02	audio	lv2	\N	lv2-1.18.0
2020-07-04 21:27:46+02	2020-07-04 21:28:45+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-07-04 21:28:39+02	2020-07-04 21:30:11+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-07-04 21:08:11+02	2020-07-04 21:30:26+02	lang	ruby27	\N	ruby-2.7.1_1,1
2020-07-04 21:27:16+02	2020-07-04 21:31:53+02	devel	qt5-core	\N	qt5-core-5.14.2_3
2020-07-04 21:30:30+02	2020-07-04 21:32:13+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-07-04 21:30:15+02	2020-07-04 21:32:42+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-07-04 21:28:47+02	2020-07-04 21:32:47+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-07-04 21:31:56+02	2020-07-04 21:33:56+02	net	qt5-network	\N	qt5-network-5.14.2
2020-07-04 21:32:15+02	2020-07-04 21:34:06+02	devel	qt5-dbus	\N	qt5-dbus-5.14.2
2020-07-04 21:32:43+02	2020-07-04 21:34:15+02	databases	qt5-sql	\N	qt5-sql-5.14.2
2020-07-04 21:32:47+02	2020-07-04 21:34:15+02	devel	qt5-testlib	\N	qt5-testlib-5.14.2
2020-07-04 21:16:14+02	2020-07-04 21:34:41+02	lang	spidermonkey60	\N	spidermonkey60-60.9.0_4
2020-07-04 21:34:16+02	2020-07-04 21:35:12+02	devel	qt5-concurrent	\N	qt5-concurrent-5.14.2
2020-07-04 21:15:54+02	2020-07-04 21:35:13+02	devel	cmake	\N	cmake-3.17.3_1
2020-07-04 21:34:16+02	2020-07-04 21:35:14+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.14.2
2020-07-04 21:34:09+02	2020-07-04 21:35:17+02	textproc	qt5-xml	\N	qt5-xml-5.14.2
2020-07-04 21:33:57+02	2020-07-04 21:35:17+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.14.2
2020-07-04 21:34:43+02	2020-07-04 21:35:20+02	devel	qt5-script	\N	qt5-script-5.14.2
2020-07-04 21:35:18+02	2020-07-04 21:35:27+02	devel	py-six	py27	py27-six-1.14.0
2020-07-04 21:35:18+02	2020-07-04 21:35:31+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-07-04 21:35:15+02	2020-07-04 21:35:47+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-07-04 21:35:31+02	2020-07-04 21:35:49+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-07-04 21:35:13+02	2020-07-04 21:35:53+02	sysutils	policykit	\N	policykit-0.9_10
2020-07-04 21:35:21+02	2020-07-04 21:36:01+02	www	py-django22	py37	py37-django22-2.2.14
2020-07-04 21:36:01+02	2020-07-04 21:36:13+02	devel	py-pytz	py27	py27-pytz-2020.1,1
2020-07-04 21:35:48+02	2020-07-04 21:36:14+02	comms	qt5-serialport	\N	qt5-serialport-5.14.2
2020-07-04 21:36:14+02	2020-07-04 21:36:27+02	devel	py-pycparser	py27	py27-pycparser-2.20
2020-07-04 21:36:27+02	2020-07-04 21:36:41+02	textproc	py-MarkupSafe	py27	py27-MarkupSafe-1.1.1
2020-07-04 21:36:42+02	2020-07-04 21:36:58+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-07-04 21:36:14+02	2020-07-04 21:36:58+02	devel	py-babel	py27	py27-Babel-2.8.0
2020-07-04 21:35:54+02	2020-07-04 21:37:00+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-07-04 21:36:59+02	2020-07-04 21:37:18+02	devel	py-Jinja2	py27	py27-Jinja2-2.10.1
2020-07-04 21:36:58+02	2020-07-04 21:37:25+02	devel	py-cffi	py27	py27-cffi-1.14.0_1
2020-07-04 21:37:20+02	2020-07-04 21:37:37+02	devel	py-enum34	py27	py27-enum34-1.1.6
2020-07-04 21:37:26+02	2020-07-04 21:37:44+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-07-04 21:37:39+02	2020-07-04 21:37:58+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-07-04 21:37:01+02	2020-07-04 21:38:04+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-07-04 21:37:59+02	2020-07-04 21:38:17+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-07-04 21:38:05+02	2020-07-04 21:38:22+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-07-04 21:38:22+02	2020-07-04 21:38:35+02	devel	py-asn1crypto	py27	py27-asn1crypto-1.3.0
2020-07-04 21:38:18+02	2020-07-04 21:38:40+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.8_6
2020-07-04 21:38:41+02	2020-07-04 21:38:59+02	net	py-ipaddress	py27	py27-ipaddress-1.0.23
2020-07-04 21:35:28+02	2020-07-04 21:39:21+02	devel	gdb	py37	gdb-9.2_1
2020-07-04 21:39:02+02	2020-07-04 21:39:29+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7
2020-07-04 21:39:30+02	2020-07-04 21:39:55+02	converters	py-webencodings	py27	py27-webencodings-0.5.1
2020-07-04 21:37:45+02	2020-07-04 21:40:56+02	security	gnupg	\N	gnupg-2.2.20
2020-07-04 21:38:36+02	2020-07-04 21:41:55+02	devel	talloc	\N	talloc-2.3.0
2020-07-04 21:40:59+02	2020-07-04 21:42:04+02	security	gpgme	\N	gpgme-1.13.1
2020-07-04 21:41:56+02	2020-07-04 21:42:13+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-07-04 21:41:42+02	2020-07-04 21:42:22+02	devel	gamin	\N	gamin-0.1.10_10
2020-07-04 21:42:14+02	2020-07-04 21:42:29+02	devel	py-more-itertools	py37	py37-more-itertools-8.4.0
2020-07-04 21:39:57+02	2020-07-04 21:43:09+02	databases	tdb	\N	tdb-1.4.2,1
2020-10-05 01:56:55+02	2020-10-05 01:57:24+02	net	grilo	\N	grilo-0.3.13
2020-07-04 21:42:07+02	2020-07-04 21:43:32+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-07-04 21:43:12+02	2020-07-04 21:43:40+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-07-04 21:42:29+02	2020-07-04 21:44:05+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-07-04 21:44:06+02	2020-07-04 21:44:34+02	devel	py-click	py37	py37-click-7.0
2020-07-04 21:44:36+02	2020-07-04 21:45:13+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-07-04 21:45:14+02	2020-07-04 21:45:38+02	devel	py-olefile	py37	py37-olefile-0.46
2020-07-04 21:45:40+02	2020-07-04 21:46:08+02	devel	py-ply	py37	py37-ply-3.11
2020-07-04 21:43:35+02	2020-07-04 21:46:10+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-07-04 21:46:10+02	2020-07-04 21:46:33+02	devel	py-ply	py27	py27-ply-3.11
2020-07-04 21:42:23+02	2020-07-04 21:46:35+02	devel	tevent	\N	tevent-0.10.1
2020-07-04 21:46:12+02	2020-07-04 21:46:45+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-07-04 21:46:35+02	2020-07-04 21:47:10+02	devel	py-zipp	py37	py37-zipp-0.6.0_3
2020-07-04 21:46:37+02	2020-07-04 21:47:20+02	devel	scons	py37	scons-py37-3.1.2
2020-07-04 21:47:22+02	2020-07-04 21:47:52+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-07-04 21:47:54+02	2020-07-04 21:48:24+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-07-04 21:48:26+02	2020-07-04 21:48:51+02	www	py-html5lib	py27	py27-html5lib-1.0.1
2020-07-04 21:48:54+02	2020-07-04 21:49:19+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-07-04 21:49:21+02	2020-07-04 21:49:55+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-07-04 21:49:57+02	2020-07-04 21:50:27+02	databases	py-redis	py37	py37-redis-3.5.3
2020-07-04 21:47:11+02	2020-07-04 22:19:37+02	net	samba410	\N	samba410-4.10.15
2020-07-04 22:19:41+02	2020-07-04 22:20:21+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-07-04 22:20:24+02	2020-07-04 22:20:57+02	devel	py-appdirs	py37	py37-appdirs-1.4.3
2020-07-04 22:20:59+02	2020-07-04 22:21:28+02	devel	py-backports	py37	py37-backports-1
2020-07-04 21:46:47+02	2020-07-04 22:21:46+02	mail	mailutils	\N	mailutils-3.9
2020-07-04 22:21:31+02	2020-07-04 22:22:04+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-07-04 22:21:49+02	2020-07-04 22:22:22+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-07-04 22:22:06+02	2020-07-04 22:22:43+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-07-04 22:22:26+02	2020-07-04 22:23:01+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-0.23
2020-07-04 22:22:45+02	2020-07-04 22:23:19+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-07-04 22:23:04+02	2020-07-04 22:23:39+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-07-04 22:23:22+02	2020-07-04 22:23:55+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-07-04 22:23:43+02	2020-07-04 22:24:15+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-07-04 22:23:58+02	2020-07-04 22:24:33+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-07-04 22:24:21+02	2020-07-04 22:24:49+02	devel	py-wheel	py27	py27-wheel-0.30.0_1
2020-07-04 21:35:13+02	2020-07-04 22:24:57+02	devel	llvm80	\N	llvm80-8.0.1_3
2020-07-04 22:24:37+02	2020-07-04 22:25:13+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-07-04 22:24:52+02	2020-07-04 22:25:44+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-07-04 22:25:47+02	2020-07-04 22:27:04+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-07-04 22:27:06+02	2020-07-04 22:28:33+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-07-04 22:25:02+02	2020-07-04 22:29:53+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-07-04 22:25:17+02	2020-07-04 22:32:00+02	dns	bind-tools	\N	bind-tools-9.16.4
2020-07-04 22:28:36+02	2020-07-04 22:33:37+02	lang	cython	py27	py27-cython-0.29.15
2020-07-04 22:32:03+02	2020-07-04 22:33:52+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-07-04 22:33:55+02	2020-07-04 22:35:20+02	www	libwpe	\N	libwpe-1.4.0.1
2020-07-04 22:35:25+02	2020-07-04 22:37:27+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-07-04 22:33:40+02	2020-07-04 22:38:19+02	graphics	libGLU	\N	libGLU-9.0.1
2020-07-04 22:29:56+02	2020-07-04 22:38:34+02	graphics	cairo	\N	cairo-1.16.0,2
2020-07-04 21:50:29+02	2020-07-04 22:40:09+02	devel	llvm90	\N	llvm90-9.0.1_1
2020-07-04 22:38:35+02	2020-07-04 22:41:58+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-07-04 22:40:12+02	2020-07-04 22:42:32+02	graphics	poppler	\N	poppler-0.89.0_1
2020-07-04 22:38:21+02	2020-07-04 22:42:50+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-07-04 22:42:52+02	2020-07-04 22:44:00+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-07-04 22:37:32+02	2020-07-04 22:46:26+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_8
2020-07-04 22:44:07+02	2020-07-04 22:46:39+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-07-04 22:42:02+02	2020-07-04 22:47:30+02	print	harfbuzz	\N	harfbuzz-2.6.8
2020-07-04 22:46:29+02	2020-07-04 22:47:57+02	accessibility	atk	\N	atk-2.36.0
2020-07-04 22:46:45+02	2020-07-04 22:48:06+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-07-04 22:42:35+02	2020-07-04 22:50:51+02	net	avahi-app	\N	avahi-app-0.7_3
2020-07-04 22:47:33+02	2020-07-04 22:52:23+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-07-04 22:48:10+02	2020-07-04 22:53:00+02	devel	libgsf	\N	libgsf-1.14.47
2020-07-04 22:52:27+02	2020-07-04 22:54:27+02	devel	libgudev	\N	libgudev-230_1
2020-07-04 22:53:09+02	2020-07-04 22:54:39+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-07-04 22:54:29+02	2020-07-04 22:55:22+02	x11	libwacom	\N	libwacom-1.4.1
2020-07-04 22:48:04+02	2020-07-04 22:56:03+02	sysutils	polkit	\N	polkit-0.116
2020-07-04 22:55:26+02	2020-07-04 22:56:32+02	x11	libinput	\N	libinput-1.15.6
2020-07-04 22:50:57+02	2020-07-04 22:59:54+02	print	cups	\N	cups-2.3.3_1
2020-07-04 22:56:35+02	2020-07-04 23:04:32+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.14.2
2020-07-04 22:56:07+02	2020-07-04 23:08:21+02	graphics	colord	\N	colord-1.3.5
2020-07-04 22:59:56+02	2020-07-04 23:09:39+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_5
2020-07-04 23:04:37+02	2020-07-04 23:10:32+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.14.2
2020-07-04 23:08:23+02	2020-07-04 23:13:43+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.8_2
2020-07-04 22:54:44+02	2020-07-04 23:14:26+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-07-04 23:14:30+02	2020-07-04 23:21:16+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-07-04 23:13:46+02	2020-07-04 23:21:27+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.14.2
2020-07-04 23:21:28+02	2020-07-04 23:22:06+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-07-04 21:43:40+02	2020-07-04 23:22:37+02	www	node	\N	node-14.4.0
2020-07-04 23:09:44+02	2020-07-04 23:22:59+02	graphics	graphviz	\N	graphviz-2.44.0_4
2020-07-04 23:22:38+02	2020-07-04 23:23:12+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-07-04 23:22:07+02	2020-07-04 23:23:36+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-07-04 23:21:17+02	2020-07-04 23:23:46+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-07-04 23:10:36+02	2020-07-04 23:24:37+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-07-04 23:23:13+02	2020-07-04 23:24:39+02	devel	libsoup	\N	libsoup-2.62.3
2020-07-04 23:23:37+02	2020-07-04 23:25:17+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-07-04 23:24:39+02	2020-07-04 23:25:18+02	devel	libnotify	\N	libnotify-0.7.8
2020-07-04 23:24:38+02	2020-07-04 23:25:19+02	devel	json-glib	\N	json-glib-1.4.4
2020-07-04 23:23:01+02	2020-07-04 23:25:54+02	lang	vala	\N	vala-0.40.15,1
2020-07-04 23:25:18+02	2020-07-04 23:26:20+02	devel	qt5-help	\N	qt5-help-5.14.2
2020-07-04 23:25:21+02	2020-07-04 23:26:22+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.14.2
2020-07-04 23:26:22+02	2020-07-04 23:26:59+02	graphics	frei0r	\N	frei0r-1.6.1
2020-07-04 23:25:55+02	2020-07-04 23:27:14+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-07-04 23:26:21+02	2020-07-04 23:27:16+02	devel	dconf	\N	dconf-0.36.0
2020-07-04 23:25:21+02	2020-07-04 23:27:42+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-07-04 23:27:17+02	2020-07-04 23:28:35+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-07-04 23:27:43+02	2020-07-04 23:28:43+02	security	libsecret	\N	libsecret-0.18.6_1
2020-07-04 23:28:36+02	2020-07-04 23:29:57+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.71.0_2
2020-07-04 23:28:43+02	2020-07-04 23:30:25+02	print	qt5-printsupport	\N	qt5-printsupport-5.14.2
2020-07-04 23:30:25+02	2020-07-04 23:30:48+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-07-04 23:29:58+02	2020-07-04 23:30:54+02	x11	qt5-x11extras	\N	qt5-x11extras-5.14.2
2020-07-04 23:30:49+02	2020-07-04 23:31:25+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-07-04 23:23:48+02	2020-07-04 23:31:35+02	print	texlive-base	\N	texlive-base-20150521_52
2020-07-04 23:30:55+02	2020-07-04 23:31:40+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.14.2
2020-07-04 23:31:25+02	2020-07-04 23:32:41+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-07-04 23:31:41+02	2020-07-04 23:33:49+02	net	geoclue	\N	geoclue-2.5.5
2020-07-04 23:32:42+02	2020-07-04 23:35:46+02	graphics	graphene	\N	graphene-1.10.0
2020-07-04 23:33:52+02	2020-07-04 23:36:09+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-07-04 23:36:12+02	2020-07-04 23:37:43+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-07-04 23:37:44+02	2020-07-04 23:38:21+02	print	libraqm	\N	libraqm-0.6.0
2020-07-04 23:35:47+02	2020-07-04 23:39:19+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-07-04 23:27:00+02	2020-07-04 23:39:44+02	multimedia	ffmpeg	\N	ffmpeg-4.3_3,1
2020-07-04 23:39:45+02	2020-07-04 23:40:47+02	graphics	qt5-svg	\N	qt5-svg-5.14.2
2020-07-04 23:39:20+02	2020-07-04 23:41:13+02	graphics	qt5-opengl	\N	qt5-opengl-5.14.2
2020-07-04 23:40:48+02	2020-07-04 23:41:53+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.71.0
2020-07-04 23:41:15+02	2020-07-04 23:42:30+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-07-04 23:27:15+02	2020-07-04 23:43:24+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-07-04 23:41:54+02	2020-07-04 23:43:29+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-07-04 23:42:31+02	2020-07-04 23:43:38+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-07-04 23:38:22+02	2020-07-04 23:44:32+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-07-04 23:43:39+02	2020-07-04 23:44:56+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-07-04 23:43:30+02	2020-07-04 23:45:20+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-07-04 23:44:33+02	2020-07-04 23:45:43+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-07-04 23:45:21+02	2020-07-04 23:47:28+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-07-04 23:47:30+02	2020-07-04 23:48:38+02	archivers	kf5-karchive	\N	kf5-karchive-5.71.0
2020-07-04 23:45:45+02	2020-07-04 23:48:56+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-04 23:44:57+02	2020-07-04 23:49:37+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-07-04 23:49:42+02	2020-07-04 23:50:40+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-07-04 23:48:40+02	2020-07-04 23:50:49+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-07-04 23:48:58+02	2020-07-04 23:50:54+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-04 23:50:57+02	2020-07-04 23:53:41+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-07-04 23:50:50+02	2020-07-04 23:54:29+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-07-04 23:50:41+02	2020-07-04 23:54:31+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-07-04 23:53:43+02	2020-07-04 23:55:07+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-04 23:54:30+02	2020-07-04 23:55:36+02	devel	qt5-uitools	\N	qt5-uitools-5.14.2
2020-07-04 23:54:33+02	2020-07-04 23:56:15+02	devel	sdl20	\N	sdl2-2.0.12
2020-07-04 23:55:09+02	2020-07-04 23:56:48+02	sysutils	hal	\N	hal-0.5.14_34
2020-07-04 23:55:37+02	2020-07-04 23:57:18+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-07-04 23:56:50+02	2020-07-04 23:58:28+02	graphics	poppler-glib	\N	poppler-glib-0.89.0_1
2020-07-04 23:57:22+02	2020-07-04 23:59:13+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-07-04 23:43:25+02	2020-07-04 23:59:50+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-07-04 23:31:36+02	2020-07-05 00:00:25+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-07-04 23:56:16+02	2020-07-05 00:01:00+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-07-04 23:59:51+02	2020-07-05 00:01:18+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-07-04 23:58:28+02	2020-07-05 00:02:22+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-07-05 00:01:19+02	2020-07-05 00:02:47+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-07-05 00:02:51+02	2020-07-05 00:04:21+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.14.2
2020-07-05 00:00:27+02	2020-07-05 00:04:59+02	print	tex-formats	\N	tex-formats-20150521_2
2020-07-05 00:05:02+02	2020-07-05 00:06:06+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-07-05 00:02:23+02	2020-07-05 00:06:08+02	graphics	glew	\N	glew-2.2.0
2020-07-04 23:59:14+02	2020-07-05 00:07:17+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-07-05 00:06:09+02	2020-07-05 00:07:45+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-05 00:07:18+02	2020-07-05 00:08:18+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.71.0
2020-07-05 00:06:07+02	2020-07-05 00:09:21+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-07-05 00:07:47+02	2020-07-05 00:09:24+02	www	qt5-webchannel	\N	qt5-webchannel-5.14.2
2020-07-05 00:08:19+02	2020-07-05 00:10:36+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-07-05 00:10:38+02	2020-07-05 00:11:45+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-07-05 00:09:24+02	2020-07-05 00:11:54+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-07-05 00:09:27+02	2020-07-05 00:12:18+02	devel	qt5-location	\N	qt5-location-5.14.2
2020-07-05 00:01:03+02	2020-07-05 00:12:35+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-07-05 00:11:58+02	2020-07-05 00:12:44+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-07-05 00:12:19+02	2020-07-05 00:13:39+02	net	remmina	\N	remmina-1.3.9
2020-07-05 00:11:47+02	2020-07-05 00:13:53+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-07-05 00:13:41+02	2020-07-05 00:14:36+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-07-05 00:13:54+02	2020-07-05 00:15:06+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-07-05 00:12:44+02	2020-07-05 00:16:02+02	security	gcr	\N	gcr-3.28.0
2020-07-05 00:15:07+02	2020-07-05 00:16:19+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-07-05 00:12:36+02	2020-07-05 00:17:59+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-07-05 00:16:21+02	2020-07-05 00:18:06+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-07-05 00:16:04+02	2020-07-05 00:18:44+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-07-05 00:18:49+02	2020-07-05 00:20:30+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-07-05 00:18:11+02	2020-07-05 00:20:47+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-07-05 00:14:37+02	2020-07-05 00:23:40+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-07-05 00:20:33+02	2020-07-05 00:23:52+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-07-05 00:23:42+02	2020-07-05 00:25:02+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-07-05 00:23:54+02	2020-07-05 00:25:31+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-07-05 00:20:50+02	2020-07-05 00:25:57+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-05 00:18:02+02	2020-07-05 00:26:11+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.71.0
2020-07-05 00:25:05+02	2020-07-05 00:26:11+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-07-05 00:25:34+02	2020-07-05 00:26:23+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-07-05 00:25:58+02	2020-07-05 00:26:53+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-07-05 00:26:12+02	2020-07-05 00:27:08+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-05 00:26:25+02	2020-07-05 00:27:14+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-07-05 00:26:54+02	2020-07-05 00:27:17+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-07-05 00:27:15+02	2020-07-05 00:27:43+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-10-05 01:59:27+02	2020-10-05 01:59:57+02	x11	babl	\N	babl-0.1.78_1
2020-07-05 00:27:09+02	2020-07-05 00:27:45+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-07-05 00:27:18+02	2020-07-05 00:28:05+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-05 00:26:12+02	2020-07-05 00:28:09+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-07-05 00:04:24+02	2020-07-05 00:28:15+02	java	openjdk8	\N	openjdk8-8.252.09.1_1
2020-07-05 00:28:10+02	2020-07-05 00:28:31+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-07-05 00:27:44+02	2020-07-05 00:28:46+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-07-05 00:28:05+02	2020-07-05 00:28:48+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-07-05 00:28:16+02	2020-07-05 00:29:06+02	comms	qt5-sensors	\N	qt5-sensors-5.14.2
2020-07-05 00:28:31+02	2020-07-05 00:29:13+02	devel	libgtop	\N	libgtop-2.40.0
2020-07-05 00:28:49+02	2020-07-05 00:29:27+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-05 00:29:14+02	2020-07-05 00:29:33+02	print	gl2ps	\N	gl2ps-1.4.0
2020-07-05 00:27:46+02	2020-07-05 00:29:51+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-07-05 00:29:28+02	2020-07-05 00:30:33+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-07-05 00:29:34+02	2020-07-05 00:31:01+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-07-05 00:29:52+02	2020-07-05 00:31:08+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-07-05 00:31:09+02	2020-07-05 00:31:32+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-07-05 00:30:36+02	2020-07-05 00:31:35+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-07-05 00:31:33+02	2020-07-05 00:32:27+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-05 00:31:36+02	2020-07-05 00:32:29+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-07-05 00:31:02+02	2020-07-05 00:32:32+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-05 00:29:07+02	2020-07-05 00:33:23+02	net	libmateweather	\N	libmateweather-1.24.0
2020-07-05 00:32:27+02	2020-07-05 00:33:38+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-05 00:32:30+02	2020-07-05 00:34:21+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.14.2
2020-07-05 00:32:33+02	2020-07-05 00:34:32+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-07-05 00:33:24+02	2020-07-05 00:34:36+02	sysutils	upower	\N	upower-0.99.11
2020-07-05 00:34:33+02	2020-07-05 00:34:51+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-07-05 00:28:47+02	2020-07-05 00:35:15+02	math	vtk8	\N	vtk8-8.2.0
2020-07-05 00:34:22+02	2020-07-05 00:35:20+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-07-05 00:34:52+02	2020-07-05 00:35:45+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-07-05 00:35:21+02	2020-07-05 00:37:08+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-05 00:34:36+02	2020-07-05 00:39:20+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-07-04 21:39:23+02	2020-07-05 00:39:31+02	lang	rust	\N	rust-1.44.1_1
2020-07-05 00:37:09+02	2020-07-05 00:41:05+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-07-05 00:39:35+02	2020-07-05 00:41:55+02	devel	librest	\N	rest-0.7.93
2020-07-05 00:39:23+02	2020-07-05 00:42:18+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-05 00:41:09+02	2020-07-05 00:42:32+02	devel	qt5-assistant	\N	qt5-assistant-5.14.2
2020-07-05 00:35:46+02	2020-07-05 00:42:40+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-07-05 00:42:19+02	2020-07-05 00:42:47+02	graphics	poppler-utils	\N	poppler-utils-0.89.0_1
2020-07-05 00:42:33+02	2020-07-05 00:42:50+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-07-05 00:41:56+02	2020-07-05 00:43:01+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-07-05 00:33:40+02	2020-07-05 00:43:02+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-07-05 00:42:41+02	2020-07-05 00:43:11+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-07-05 00:42:48+02	2020-07-05 00:43:31+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-07-05 00:42:51+02	2020-07-05 00:43:35+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-05 00:43:03+02	2020-07-05 00:43:37+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-07-05 00:43:03+02	2020-07-05 00:43:39+02	x11	babl	\N	babl-0.1.78
2020-07-05 00:43:32+02	2020-07-05 00:43:44+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-07-05 00:43:12+02	2020-07-05 00:44:15+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-07-05 00:35:17+02	2020-07-05 00:44:27+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-05 00:43:37+02	2020-07-05 00:44:29+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-07-05 00:43:36+02	2020-07-05 00:44:33+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-05 00:43:40+02	2020-07-05 00:44:34+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-07-05 00:43:44+02	2020-07-05 00:44:36+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-07-05 00:44:16+02	2020-07-05 00:45:08+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-07-05 00:44:33+02	2020-07-05 00:45:24+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-07-05 00:44:35+02	2020-07-05 00:45:24+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-07-05 00:44:30+02	2020-07-05 00:45:52+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-07-05 00:45:24+02	2020-07-05 00:46:27+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-07-05 00:45:09+02	2020-07-05 00:46:31+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-05 00:45:25+02	2020-07-05 00:46:32+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-07-05 00:44:37+02	2020-07-05 00:46:34+02	x11-fm	caja	\N	caja-1.22.3_1
2020-07-05 00:45:52+02	2020-07-05 00:46:37+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-07-05 00:46:35+02	2020-07-05 00:47:09+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-07-05 00:46:38+02	2020-07-05 00:47:18+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-07-05 00:47:10+02	2020-07-05 00:47:34+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-07-05 00:46:31+02	2020-07-05 00:48:27+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-05 00:46:28+02	2020-07-05 00:48:39+02	devel	qt5-designer	\N	qt5-designer-5.14.2
2020-07-05 00:48:29+02	2020-07-05 00:49:10+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-07-05 00:46:32+02	2020-07-05 00:49:19+02	graphics	gegl	\N	gegl-0.4.24
2020-07-05 00:49:10+02	2020-07-05 00:50:22+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-05 00:48:40+02	2020-07-05 00:50:34+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-07-05 00:47:36+02	2020-07-05 00:50:51+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-07-05 00:50:23+02	2020-07-05 00:51:24+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-07-05 00:50:52+02	2020-07-05 00:52:33+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-07-05 00:50:35+02	2020-07-05 00:52:35+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-05 00:51:26+02	2020-07-05 00:52:49+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-07-05 00:49:20+02	2020-07-05 00:52:49+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-07-05 00:52:34+02	2020-07-05 00:53:14+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-07-05 00:52:50+02	2020-07-05 00:54:11+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-05 00:47:19+02	2020-07-05 00:54:25+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-07-05 00:52:36+02	2020-07-05 00:54:27+02	x11	zenity	\N	zenity-3.32.0
2020-07-05 00:54:26+02	2020-07-05 00:55:25+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-07-05 00:54:12+02	2020-07-05 00:55:43+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-07-05 00:52:50+02	2020-07-05 00:56:53+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-07-05 00:53:14+02	2020-07-05 00:56:57+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-07-05 00:55:27+02	2020-07-05 00:57:43+02	devel	libpeas	\N	libpeas-1.22.0
2020-07-05 00:56:55+02	2020-07-05 00:57:47+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-07-05 00:54:28+02	2020-07-05 00:57:49+02	devel	libgdata	\N	libgdata-0.17.9
2020-07-05 00:57:44+02	2020-07-05 00:58:18+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-07-05 00:57:50+02	2020-07-05 00:58:57+02	graphics	glfw	\N	glfw-3.3.2
2020-07-05 00:55:45+02	2020-07-05 00:59:54+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-05 00:44:28+02	2020-07-05 01:00:05+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-05 00:59:55+02	2020-07-05 01:00:41+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-07-05 01:01:00+02	2020-07-05 01:01:18+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0
2020-07-05 00:58:21+02	2020-07-05 01:01:30+02	graphics	movit	\N	movit-1.6.2_2
2020-07-05 01:01:19+02	2020-07-05 01:01:41+02	net	gssdp	\N	gssdp-1.2.3
2020-07-05 01:00:42+02	2020-07-05 01:01:48+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-05 01:01:30+02	2020-07-05 01:02:26+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-05 01:01:41+02	2020-07-05 01:02:47+02	print	cups-filters	\N	cups-filters-1.27.5
2020-07-05 01:02:48+02	2020-07-05 01:03:02+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-07-05 00:56:59+02	2020-07-05 01:03:12+02	editors	emacs	full	emacs-26.3_5,3
2020-07-05 00:58:58+02	2020-07-05 01:03:54+02	graphics	opencv	\N	opencv-3.4.1_33
2020-07-05 01:03:55+02	2020-07-05 01:04:11+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-07-05 01:03:03+02	2020-07-05 01:04:28+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-07-05 01:04:13+02	2020-07-05 01:04:29+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-07-05 01:04:29+02	2020-07-05 01:04:44+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-07-05 01:03:13+02	2020-07-05 01:05:53+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-07-05 01:05:53+02	2020-07-05 01:06:12+02	www	py-h2	py37	py37-h2-3.2.0
2020-07-05 01:06:12+02	2020-07-05 01:06:28+02	www	py-hyperlink	py37	py37-hyperlink-19.0.0
2020-07-05 01:01:48+02	2020-07-05 01:06:32+02	print	fontforge	\N	fontforge-20190801_2
2020-07-05 01:06:29+02	2020-07-05 01:06:47+02	www	py-priority	py37	py37-priority-1.3.0
2020-07-05 01:06:34+02	2020-07-05 01:06:57+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-07-05 01:06:58+02	2020-07-05 01:08:38+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-07-05 00:57:47+02	2020-07-05 01:10:53+02	graphics	gimp-app	\N	gimp-app-2.10.20,1
2020-07-05 01:08:39+02	2020-07-05 01:12:38+02	x11	xorg	\N	xorg-7.7_3
2020-07-05 01:10:55+02	2020-07-05 01:15:33+02	x11	yelp	\N	yelp-3.28.1_2
2020-07-05 01:12:40+02	2020-07-05 01:16:06+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-07-05 01:15:38+02	2020-07-05 01:18:16+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-07-05 01:02:27+02	2020-07-05 01:19:01+02	print	tex-luatex	\N	tex-luatex-0.80.0_18
2020-07-05 01:18:19+02	2020-07-05 01:21:10+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.14.2
2020-07-05 01:16:10+02	2020-07-05 01:22:07+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-07-05 01:22:10+02	2020-07-05 01:24:46+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-07-05 01:19:03+02	2020-07-05 01:25:49+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-07-05 01:21:12+02	2020-07-05 01:27:42+02	x11-wm	marco	\N	marco-1.22.4
2020-07-05 01:25:56+02	2020-07-05 01:31:22+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-07-05 01:24:52+02	2020-07-05 01:33:56+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-07-05 01:33:59+02	2020-07-05 01:34:35+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-07-05 01:34:39+02	2020-07-05 01:35:15+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-07-05 01:35:17+02	2020-07-05 01:35:53+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-07-05 01:35:56+02	2020-07-05 01:36:36+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-07-05 01:36:40+02	2020-07-05 01:37:24+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-07-05 01:37:26+02	2020-07-05 01:38:29+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-07-05 01:38:33+02	2020-07-05 01:39:21+02	devel	py-pyasn1	py27	py27-pyasn1-0.4.7
2020-07-05 01:39:31+02	2020-07-05 01:42:10+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-07-05 01:42:13+02	2020-07-05 01:42:59+02	dns	py-idna	py27	py27-idna-2.8
2020-07-05 01:27:46+02	2020-07-05 01:43:26+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-07-05 01:31:24+02	2020-07-05 01:45:31+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-05 01:45:34+02	2020-07-05 01:46:39+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-07-05 01:43:29+02	2020-07-05 01:47:23+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-07-05 01:46:43+02	2020-07-05 01:50:28+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-05 01:47:26+02	2020-07-05 01:51:16+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-07-05 01:50:33+02	2020-07-05 01:52:42+02	graphics	libgxps	\N	libgxps-0.3.1
2020-07-05 01:52:48+02	2020-07-05 01:55:04+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-07-05 01:51:20+02	2020-07-05 01:56:42+02	graphics	libheif	\N	libheif-1.6.2
2020-07-05 01:56:50+02	2020-07-05 01:58:38+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-07-05 01:55:08+02	2020-07-05 01:59:56+02	graphics	poppler-qt5	\N	poppler-qt5-0.89.0_1
2020-07-05 01:58:45+02	2020-07-05 02:01:29+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-05 02:01:33+02	2020-07-05 02:04:16+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.14.2
2020-07-05 02:04:20+02	2020-07-05 02:06:01+02	graphics	vigra	\N	vigra-1.11.1_10
2020-07-05 02:00:01+02	2020-07-05 02:06:29+02	graphics	qt5-3d	\N	qt5-3d-5.14.2
2020-07-05 01:43:02+02	2020-07-05 02:10:58+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-07-05 02:11:02+02	2020-07-05 02:15:27+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-07-05 02:15:31+02	2020-07-05 02:18:57+02	multimedia	libass	\N	libass-0.14.0
2020-07-05 01:04:46+02	2020-07-05 02:19:43+02	www	node12	\N	node12-12.18.2_1
2020-07-05 02:06:33+02	2020-07-05 02:21:22+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-07-05 02:19:00+02	2020-07-05 02:21:50+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-05 02:19:44+02	2020-07-05 02:21:56+02	net	freerdp	\N	freerdp-2.1.1
2020-07-05 02:21:24+02	2020-07-05 02:22:12+02	net	gupnp	\N	gupnp-1.1.1_1
2020-07-05 02:21:57+02	2020-07-05 02:22:14+02	net	py-pysocks	py27	py27-pysocks-1.7.1
2020-07-05 02:21:51+02	2020-07-05 02:22:17+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-07-05 02:22:14+02	2020-07-05 02:22:34+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-07-05 02:22:18+02	2020-07-05 02:23:34+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-05 02:22:18+02	2020-07-05 02:23:36+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-07-05 01:06:48+02	2020-07-05 02:24:29+02	www	qt5-webengine	\N	qt5-webengine-5.14.2_1
2020-07-05 02:23:36+02	2020-07-05 02:25:04+02	print	libspectre	\N	libspectre-0.2.9
2020-07-05 02:23:37+02	2020-07-05 02:27:22+02	print	mftrace	\N	mftrace-1.2.18_2
2020-07-05 02:27:25+02	2020-07-05 02:27:54+02	security	py-bcrypt	py27	py27-bcrypt-3.1.7
2020-07-05 02:27:57+02	2020-07-05 02:28:22+02	security	py-certifi	py27	py27-certifi-2020.6.20
2020-07-05 02:28:25+02	2020-07-05 02:28:46+02	security	py-openssl	py27	py27-openssl-19.0.0
2020-07-05 02:28:47+02	2020-07-05 02:29:40+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-07-05 02:22:35+02	2020-07-05 02:29:55+02	print	gutenprint	\N	gutenprint-5.3.3
2020-07-05 02:29:43+02	2020-07-05 02:30:11+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-07-05 02:29:58+02	2020-07-05 02:30:25+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-07-05 02:30:12+02	2020-07-05 02:30:59+02	sysutils	hal-info	\N	hal-info-20091130
2020-07-05 02:31:01+02	2020-07-05 02:32:45+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-07-05 02:25:06+02	2020-07-05 02:34:01+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-07-05 02:24:33+02	2020-07-05 02:34:35+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-07-05 02:34:39+02	2020-07-05 02:35:09+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-07-05 02:34:02+02	2020-07-05 02:35:26+02	textproc	enchant	\N	enchant-1.6.0_9
2020-07-05 02:35:10+02	2020-07-05 02:35:30+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-07-05 02:35:27+02	2020-07-05 02:35:45+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-07-05 02:30:27+02	2020-07-05 02:36:03+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-07-05 02:06:05+02	2020-07-05 02:36:12+02	math	R	\N	R-4.0.2
2020-07-05 02:32:48+02	2020-07-05 02:36:15+02	textproc	dblatex	\N	dblatex-0.3.11
2020-07-05 02:35:46+02	2020-07-05 02:36:16+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-07-05 02:36:19+02	2020-07-05 02:36:49+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-07-05 02:36:19+02	2020-07-05 02:37:46+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-05 02:36:06+02	2020-07-05 02:38:02+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-05 02:36:16+02	2020-07-05 02:39:15+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-05 02:37:49+02	2020-07-05 02:39:26+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-07-05 02:39:18+02	2020-07-05 02:40:57+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-07-05 02:39:28+02	2020-07-05 02:42:21+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-07-05 02:38:05+02	2020-07-05 02:43:25+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-07-05 02:36:53+02	2020-07-05 02:45:38+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-07-05 02:45:41+02	2020-07-05 02:47:11+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-07-05 02:41:00+02	2020-07-05 02:47:23+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-07-05 02:47:16+02	2020-07-05 02:49:18+02	audio	faudio	\N	FAudio-20.07
2020-07-05 02:49:23+02	2020-07-05 02:50:11+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-07-05 02:43:27+02	2020-07-05 02:51:14+02	archivers	engrampa	\N	engrampa-1.24.0
2020-07-05 02:47:31+02	2020-07-05 02:53:15+02	audio	mate-media	\N	mate-media-1.24.0
2020-07-05 02:42:24+02	2020-07-05 02:54:52+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-07-05 02:50:14+02	2020-07-05 02:56:26+02	databases	rrdtool	\N	rrdtool-1.7.2_3
2020-07-05 02:54:54+02	2020-07-05 02:58:21+02	deskutils	mozo	\N	mozo-1.24.0
2020-07-05 02:51:17+02	2020-07-05 02:58:20+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-07-05 02:56:30+02	2020-07-05 02:58:41+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-07-05 02:58:26+02	2020-07-05 02:59:40+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-07-05 02:58:26+02	2020-07-05 03:00:47+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-07-05 02:58:45+02	2020-07-05 03:01:44+02	devel	gradle5	\N	gradle5-5.0
2020-07-05 02:59:42+02	2020-07-05 03:02:05+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0_1
2020-07-05 03:01:47+02	2020-07-05 03:02:18+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-07-05 03:02:07+02	2020-07-05 03:02:46+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-07-05 03:02:48+02	2020-07-05 03:03:18+02	devel	py-funcy	py37	py37-funcy-1.12
2020-07-05 03:03:21+02	2020-07-05 03:03:55+02	devel	py-future	py27	py27-future-0.18.2
2020-07-05 03:03:57+02	2020-07-05 03:04:33+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-07-05 03:02:23+02	2020-07-05 03:04:49+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-07-05 03:04:37+02	2020-07-05 03:05:07+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-07-05 03:04:52+02	2020-07-05 03:05:26+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-07-05 03:05:10+02	2020-07-05 03:05:45+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-07-05 03:05:29+02	2020-07-05 03:06:12+02	devel	py-rq	py37	py37-rq-1.3.0
2020-07-05 03:05:48+02	2020-07-05 03:06:40+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-07-05 03:06:14+02	2020-07-05 03:06:54+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-07-05 03:00:50+02	2020-07-05 03:07:14+02	devel	ode	\N	ode-0.13_4,1
2020-07-05 03:06:45+02	2020-07-05 03:07:22+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-07-05 03:06:58+02	2020-07-05 03:07:31+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-07-05 03:07:18+02	2020-07-05 03:07:48+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-07-05 03:07:26+02	2020-07-05 03:07:55+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-07-05 03:07:33+02	2020-07-05 03:08:02+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-07-05 03:07:50+02	2020-07-05 03:08:23+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-07-05 03:08:05+02	2020-07-05 03:08:41+02	devel	scons	py27	scons-py27-3.1.2
2020-07-05 03:08:25+02	2020-07-05 03:09:05+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-07-05 03:08:44+02	2020-07-05 03:09:18+02	dns	py-dnspython	py27	py27-dnspython-1.16.0
2020-07-05 02:53:19+02	2020-07-05 03:11:49+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-07-05 03:07:58+02	2020-07-05 03:17:24+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3
2020-07-05 03:09:09+02	2020-07-05 03:18:04+02	dns	py-ldns	py27	py27-ldns-1.7.1_1
2020-07-05 03:18:08+02	2020-07-05 03:24:28+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-07-05 03:17:28+02	2020-07-05 03:31:19+02	editors	pluma	\N	pluma-1.24.0
2020-07-05 02:35:31+02	2020-07-05 03:32:25+02	www	node10	\N	node10-10.21.0
2020-07-05 03:32:27+02	2020-07-05 03:35:04+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-05 03:24:30+02	2020-07-05 03:36:34+02	graphics	atril	\N	atril-1.22.3_6
2020-07-05 03:35:09+02	2020-07-05 03:37:08+02	graphics	gle	\N	gle-3.1.0_7
2020-07-05 03:31:23+02	2020-07-05 03:37:41+02	graphics	eom	\N	eom-1.24.0
2020-07-05 03:36:37+02	2020-07-05 03:37:49+02	graphics	gltt	\N	gltt-2.5.2_14
2020-07-05 03:37:10+02	2020-07-05 03:41:08+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-07-05 03:37:52+02	2020-07-05 03:46:27+02	graphics	opensubdiv	\N	opensubdiv-3.4.3
2020-07-05 03:46:32+02	2020-07-05 03:49:58+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-07-05 03:37:44+02	2020-07-05 03:51:11+02	graphics	openimageio	\N	openimageio-2.1.15.0
2020-07-05 03:41:11+02	2020-07-05 03:52:12+02	graphics	py-gimp	\N	py27-gimp-2.10.20
2020-07-05 03:52:16+02	2020-07-05 03:52:46+02	lang	python3	\N	python3-3_3
2020-07-05 03:09:22+02	2020-07-05 03:57:38+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-07-05 03:57:42+02	2020-07-05 03:59:39+02	math	stp	\N	stp-2.3.3_5,1
2020-07-05 03:52:50+02	2020-07-05 04:01:24+02	math	mate-calc	\N	mate-calc-1.24.0
2020-07-05 04:01:28+02	2020-07-05 04:05:22+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-05 04:05:25+02	2020-07-05 04:07:03+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-07-05 03:59:41+02	2020-07-05 04:09:14+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-07-05 04:07:04+02	2020-07-05 04:10:17+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-07-05 03:50:01+02	2020-07-05 04:11:29+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-07-05 04:11:31+02	2020-07-05 04:11:52+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-07-05 04:11:54+02	2020-07-05 04:12:18+02	net	py-pyzmq	py37	py37-pyzmq-19.0.1
2020-07-05 04:10:18+02	2020-07-05 04:12:18+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-05 04:12:20+02	2020-07-05 04:12:36+02	net	py-urllib3	py27	py27-urllib3-1.25.7,1
2020-07-05 04:12:20+02	2020-07-05 04:13:32+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-07-05 04:12:38+02	2020-07-05 04:13:57+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-07-05 04:13:34+02	2020-07-05 04:14:53+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-07-05 04:13:59+02	2020-07-05 04:15:33+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-07-05 04:14:55+02	2020-07-05 04:16:18+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-07-05 04:15:33+02	2020-07-05 04:16:56+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-07-05 03:51:15+02	2020-07-05 04:16:57+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-07-05 04:16:20+02	2020-07-05 04:18:03+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-07-05 04:16:58+02	2020-07-05 04:18:25+02	net	vde2	\N	vde2-2.3.2_4
2020-07-05 03:11:52+02	2020-07-05 04:18:26+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-07-05 04:18:30+02	2020-07-05 04:18:45+02	net-mgmt	py-ipcalc	py27	py27-ipcalc-1.99.0
2020-07-05 04:16:58+02	2020-07-05 04:19:06+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-05 04:18:46+02	2020-07-05 04:19:08+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.7.1
2020-07-05 04:18:04+02	2020-07-05 04:19:20+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-05 04:18:26+02	2020-07-05 04:20:28+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-07-05 04:19:08+02	2020-07-05 04:21:32+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-05 04:21:33+02	2020-07-05 04:25:53+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-07-05 04:20:29+02	2020-07-05 04:29:55+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-07-05 04:19:09+02	2020-07-05 04:29:55+02	print	lilypond	\N	lilypond-2.20.0
2020-07-05 04:09:16+02	2020-07-05 04:30:34+02	multimedia	vlc	\N	vlc-3.0.11_2,4
2020-07-05 04:29:58+02	2020-07-05 04:30:41+02	print	transfig	\N	transfig-3.2.5e_4
2020-07-05 04:30:37+02	2020-07-05 04:31:01+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-07-05 04:30:44+02	2020-07-05 04:31:04+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-07-05 04:19:21+02	2020-07-05 04:31:16+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-07-05 04:31:02+02	2020-07-05 04:31:23+02	security	py-stem	py37	py37-stem-1.8.0
2020-07-05 04:31:24+02	2020-07-05 04:31:43+02	sysutils	py-psutil	py27	py27-psutil-5.7.0
2020-07-05 04:31:18+02	2020-07-05 04:33:52+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-07-05 04:31:06+02	2020-07-05 04:33:51+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-07-05 04:31:44+02	2020-07-05 04:34:03+02	textproc	fop	\N	fop-2.3
2020-07-05 04:33:53+02	2020-07-05 04:34:09+02	textproc	py-chardet	py27	py27-chardet-3.0.4_3
2020-07-05 04:34:10+02	2020-07-05 04:34:28+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-07-05 04:34:29+02	2020-07-05 04:34:47+02	textproc	py-scour	py37	py37-scour-0.37
2020-07-05 04:34:04+02	2020-07-05 04:34:52+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-07-05 04:33:53+02	2020-07-05 04:35:04+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-07-05 04:34:49+02	2020-07-05 04:35:07+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-07-05 04:35:09+02	2020-07-05 04:35:27+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-07-05 04:34:54+02	2020-07-05 04:35:29+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-07-05 04:35:06+02	2020-07-05 04:35:39+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.0
2020-07-05 04:35:29+02	2020-07-05 04:35:51+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-07-05 04:35:29+02	2020-07-05 04:35:53+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-07-05 04:35:41+02	2020-07-05 04:35:59+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-07-05 04:35:53+02	2020-07-05 04:36:15+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-07-05 04:35:55+02	2020-07-05 04:36:13+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-07-05 04:36:16+02	2020-07-05 04:37:04+02	www	qt5-websockets	\N	qt5-websockets-5.14.2
2020-07-05 04:25:56+02	2020-07-05 04:37:08+02	print	tex-xetex	\N	tex-xetex-0.99992_34
2020-07-05 01:04:30+02	2020-07-05 04:37:15+02	www	chromium	\N	chromium-83.0.4103.116
2020-07-05 04:36:01+02	2020-07-05 04:37:20+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-05 04:37:06+02	2020-07-05 04:37:22+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-07-05 04:36:16+02	2020-07-05 04:37:35+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-05 04:37:21+02	2020-07-05 04:38:43+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-05 04:37:19+02	2020-07-05 04:38:45+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-05 04:37:22+02	2020-07-05 04:38:49+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-07-05 04:37:36+02	2020-07-05 04:39:55+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-07-05 04:38:44+02	2020-07-05 04:41:01+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-07-05 04:38:50+02	2020-07-05 04:41:07+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-07-05 04:39:57+02	2020-07-05 04:41:10+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-07-05 04:38:46+02	2020-07-05 04:41:10+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-05 04:41:02+02	2020-07-05 04:45:03+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-07-05 04:37:10+02	2020-07-05 04:45:41+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-07-05 04:41:11+02	2020-07-05 04:47:11+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-05 04:45:04+02	2020-07-05 04:47:41+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-05 04:41:11+02	2020-07-05 04:48:11+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-05 04:41:08+02	2020-07-05 04:49:11+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-05 04:47:12+02	2020-07-05 04:49:24+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-05 04:49:12+02	2020-07-05 04:49:25+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-07-05 04:47:42+02	2020-07-05 04:49:36+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-05 04:49:26+02	2020-07-05 04:50:36+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-07-05 04:49:37+02	2020-07-05 04:51:51+02	devel	bzr	py27	bzr-2.7.0.6622
2020-07-05 04:50:37+02	2020-07-05 04:52:02+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-07-05 04:48:12+02	2020-07-05 04:52:10+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-07-05 04:49:25+02	2020-07-05 04:52:32+02	deskutils	calibre	\N	calibre-4.20.0
2020-07-05 04:52:33+02	2020-07-05 04:54:33+02	devel	mercurial	py27	mercurial-5.1.2
2020-07-05 04:54:35+02	2020-07-05 04:56:06+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-07-05 04:56:08+02	2020-07-05 04:57:45+02	devel	tig	\N	tig-2.5.1
2020-07-05 04:57:46+02	2020-07-05 05:03:39+02	dns	bind916	\N	bind916-9.16.4
2020-07-05 05:03:40+02	2020-07-05 05:03:53+02	dns	sshfp	\N	sshfp-1.2.2_5
2020-07-05 04:29:58+02	2020-07-05 05:07:33+02	print	texlive-docs	\N	texlive-docs-20150523
2020-07-05 05:07:36+02	2020-07-05 05:12:12+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-07-05 05:03:54+02	2020-07-05 05:15:01+02	editors	codelite	\N	codelite-12.0_4
2020-07-05 05:12:15+02	2020-07-05 05:15:16+02	editors	vim	\N	vim-8.2.1110
2020-07-05 04:52:11+02	2020-07-05 05:20:57+02	devel	llvm10	\N	llvm10-10.0.0_1
2020-07-05 04:52:03+02	2020-07-05 05:23:53+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200519
2020-07-05 05:23:57+02	2020-07-05 05:48:32+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-07-05 05:48:36+02	2020-07-05 05:49:24+02	games	KnightCap	\N	knightcap-3.6_8
2020-07-05 05:15:02+02	2020-07-05 05:50:23+02	emulators	qemu	\N	qemu-4.1.1_4
2020-07-05 05:49:27+02	2020-07-05 05:50:44+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-07-05 05:50:46+02	2020-07-05 05:53:22+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-07-05 05:21:00+02	2020-07-05 05:54:56+02	emulators	wine	\N	wine-5.0.1,1
2020-07-05 05:50:27+02	2020-07-05 05:55:03+02	games	alienarena	\N	alienarena-2013.766_9
2020-07-05 05:54:59+02	2020-07-05 05:56:33+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-07-05 05:55:06+02	2020-07-05 05:57:06+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-07-05 05:57:09+02	2020-07-05 05:57:43+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-07-05 05:56:36+02	2020-07-05 05:58:40+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-07-05 05:53:24+02	2020-07-05 05:59:50+02	games	battletanks	\N	btanks-0.9.8083_9
2020-07-05 05:57:47+02	2020-07-05 06:00:02+02	games	xblast	\N	xblast-2.10.4_17
2020-07-05 05:58:41+02	2020-07-05 06:00:11+02	games	xboard	\N	xboard-4.9.1_1
2020-07-05 05:59:50+02	2020-07-05 06:00:48+02	games	xcowsay	\N	xcowsay-1.5
2020-07-05 04:51:51+02	2020-07-05 06:00:56+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-05 06:00:59+02	2020-07-05 06:01:14+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-07-05 06:00:12+02	2020-07-05 06:01:38+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-05 06:01:39+02	2020-07-05 06:03:20+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-05 06:00:03+02	2020-07-05 06:03:43+02	graphics	dia	\N	dia-0.97.3_1,1
2020-07-05 06:03:44+02	2020-07-05 06:05:54+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-07-05 06:03:21+02	2020-07-05 06:06:31+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-07-05 06:01:15+02	2020-07-05 06:11:36+02	graphics	wings	\N	wings-2.2.5
2020-07-05 06:00:49+02	2020-07-05 06:24:09+02	graphics	krita	\N	krita-4.3.0
2020-07-05 06:24:10+02	2020-07-05 06:24:21+02	lang	python	\N	python-3.7_3,2
2020-07-05 05:15:17+02	2020-07-05 06:24:21+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-07-05 06:25:51+02	2020-07-05 06:27:06+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-05 06:27:06+02	2020-07-05 06:27:26+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-07-05 06:27:27+02	2020-07-05 06:28:24+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-07-05 06:28:25+02	2020-07-05 06:29:18+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-07-05 06:29:19+02	2020-07-05 06:30:14+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-07-05 06:30:16+02	2020-07-05 06:33:30+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-05 06:33:31+02	2020-07-05 06:37:06+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-05 06:37:07+02	2020-07-05 06:43:08+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-05 06:43:09+02	2020-07-05 06:47:10+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-07-05 06:47:11+02	2020-07-05 06:47:27+02	net	py-netifaces	py27	py27-netifaces-0.10.9
2020-07-05 06:27:43+02	2020-07-05 06:48:20+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.p4
2020-07-05 06:47:28+02	2020-07-05 06:49:16+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-05 06:49:17+02	2020-07-05 06:50:33+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-07-05 06:50:34+02	2020-07-05 07:02:23+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-07-05 07:02:24+02	2020-07-05 07:03:31+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-05 07:03:32+02	2020-07-05 07:04:39+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-05 06:06:34+02	2020-07-05 07:04:49+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-07-05 06:05:56+02	2020-07-05 07:04:52+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.2
2020-07-05 06:11:37+02	2020-07-05 07:05:06+02	lang	mono	\N	mono-5.10.1.57_3
2020-07-05 07:04:40+02	2020-07-05 07:05:43+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-05 07:04:50+02	2020-07-05 07:05:53+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-05 07:04:53+02	2020-07-05 07:05:55+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-05 07:05:07+02	2020-07-05 07:06:08+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-07-05 07:05:44+02	2020-07-05 07:06:46+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-05 07:05:57+02	2020-07-05 07:07:01+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-05 07:06:09+02	2020-07-05 07:07:06+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-05 07:05:53+02	2020-07-05 07:07:22+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-05 07:07:01+02	2020-07-05 07:07:36+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-07-05 07:07:08+02	2020-07-05 07:07:48+02	print	gv	\N	gv-3.7.4_5
2020-07-05 06:48:22+02	2020-07-05 07:08:28+02	net	wireshark	\N	wireshark-3.2.5
2020-07-05 07:06:46+02	2020-07-05 07:09:44+02	print	auctex	full	auctex-emacs26-12.2
2020-07-05 07:09:46+02	2020-07-05 07:10:19+02	security	nyx	\N	nyx-2.1.0_1
2020-07-05 07:10:22+02	2020-07-05 07:12:51+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-05 07:08:29+02	2020-07-05 07:13:07+02	security	klee	\N	klee-2.1
2020-07-05 07:12:53+02	2020-07-05 07:15:12+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-07-05 07:07:49+02	2020-07-05 07:16:00+02	print	texlive-full	\N	texlive-full-20150521
2020-07-05 07:16:04+02	2020-07-05 07:17:01+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-07-05 07:15:15+02	2020-07-05 07:17:54+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-07-05 07:13:10+02	2020-07-05 07:17:57+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-07-05 07:18:00+02	2020-07-05 07:18:41+02	sysutils	py-glances	py27	py27-glances-3.1.4
2020-07-05 07:17:57+02	2020-07-05 07:18:43+02	sysutils	panicmail	\N	panicmail-1.1
2020-07-05 07:17:03+02	2020-07-05 07:18:49+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-07-05 07:18:42+02	2020-07-05 07:19:00+02	sysutils	py-py-cpuinfo	py27	py27-py-cpuinfo-6.0.0
2020-07-05 07:18:45+02	2020-07-05 07:19:13+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-07-05 07:19:14+02	2020-07-05 07:19:27+02	www	py-bottle	py27	py27-bottle-0.12.15_1
2020-07-05 07:19:01+02	2020-07-05 07:19:39+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-07-05 07:19:28+02	2020-07-05 07:19:46+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-07-05 07:19:40+02	2020-07-05 07:19:56+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-07-05 07:19:47+02	2020-07-05 07:20:05+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-07-05 07:19:57+02	2020-07-05 07:20:15+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-07-05 07:07:22+02	2020-07-05 07:20:18+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-05 07:20:06+02	2020-07-05 07:20:24+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-07-05 07:20:15+02	2020-07-05 07:20:30+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-07-05 07:20:19+02	2020-07-05 07:20:38+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-07-05 07:20:25+02	2020-07-05 07:20:43+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-07-05 07:20:31+02	2020-07-05 07:20:48+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-07-05 07:20:39+02	2020-07-05 07:20:55+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-07-05 07:20:48+02	2020-07-05 07:21:06+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-07-05 07:20:44+02	2020-07-05 07:21:07+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-07-05 07:20:56+02	2020-07-05 07:21:11+02	www	py-requests	py27	py27-requests-2.22.0
2020-07-05 07:21:06+02	2020-07-05 07:21:21+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-07-05 07:21:22+02	2020-07-05 07:23:03+02	x11	xlockmore	\N	xlockmore-5.63
2020-07-05 07:21:08+02	2020-07-05 07:23:07+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-05 07:21:11+02	2020-07-05 07:23:57+02	x11	mate	\N	mate-1.22.2
2020-07-05 07:23:58+02	2020-07-05 07:25:50+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-07-05 07:25:51+02	2020-07-05 07:26:10+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-07-05 07:26:11+02	2020-07-05 07:28:10+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-07-05 07:28:12+02	2020-07-05 07:28:54+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.3_6
2020-07-05 07:23:03+02	2020-07-05 07:30:26+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-07-05 07:30:29+02	2020-07-05 07:30:47+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-05 07:23:07+02	2020-07-05 07:31:13+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-07-05 07:31:16+02	2020-07-05 07:33:49+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-07-05 07:30:50+02	2020-07-05 07:33:50+02	graphics	xfig	\N	xfig-3.2.7a
2020-07-05 07:07:36+02	2020-07-05 07:39:18+02	print	scribus-devel	\N	scribus-devel-1.5.5_14
2020-07-05 07:28:55+02	2020-07-05 08:16:17+02	editors	vscode	\N	vscode-1.46.1
2020-07-05 04:45:42+02	2020-07-05 08:22:53+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-05 07:18:50+02	2020-07-05 09:04:19+02	www	firefox	\N	firefox-78.0.1_1,1
2020-07-05 09:09:07+02	2020-07-05 09:19:50+02	math	suitesparse	\N	suitesparse-5.8.0
2020-07-05 09:19:50+02	2020-07-05 09:23:03+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-07-05 09:23:03+02	2020-07-05 09:41:08+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-07-05 09:41:09+02	2020-07-05 09:46:00+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-07-05 09:46:02+02	2020-07-05 09:54:06+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-07-05 09:54:08+02	2020-07-05 09:55:35+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-07-05 09:55:37+02	2020-07-05 09:57:54+02	devel	py-game	py37	py37-game-1.9.6_1
2020-07-05 09:57:55+02	2020-07-05 09:59:46+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-07-05 09:59:48+02	2020-07-05 10:10:40+02	misc	openvdb	\N	openvdb-7.0.0_2
2020-07-05 10:10:40+02	2020-07-05 10:11:06+02	games	monsterz	\N	monsterz-0.7.1_15
2020-07-05 09:09:07+02	2020-07-05 10:36:35+02	lang	ghc	\N	ghc-8.8.3_2
2020-07-05 10:11:06+02	2020-07-05 10:38:35+02	graphics	blender	\N	blender-2.83.1
2020-07-05 10:38:36+02	2020-07-05 11:05:43+02	graphics	inkscape	\N	inkscape-1.0_2
2020-07-05 11:07:53+02	2020-07-05 11:08:48+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-07-05 10:36:37+02	2020-07-05 11:10:54+02	devel	hs-cabal-install	\N	hs-cabal-install-3.0.2.0_1
2020-07-05 11:10:55+02	2020-07-05 11:46:33+02	textproc	hs-pandoc	\N	hs-pandoc-2.9.2_2
2020-07-05 11:46:34+02	2020-07-05 12:22:31+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-05 12:27:11+02	2020-07-05 12:42:39+02	local	E590T-localbase	\N	E590T-localbase-2020070400
2020-07-05 13:06:31+02	2020-07-05 13:06:56+02	lang	cmucl	\N	cmucl-21.b
2020-07-05 13:08:26+02	2020-07-05 13:08:33+02	devel	cl-asdf	\N	cl-asdf-3.3.1.7,2
2020-07-05 13:09:34+02	2020-07-05 13:14:19+02	lang	sbcl	\N	sbcl-2.0.6,1
2020-07-05 13:14:20+02	2020-07-05 13:17:01+02	math	maxima	\N	maxima-5.44.0_1
2020-07-05 13:23:20+02	2020-07-05 13:26:55+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-05 13:26:55+02	2020-07-05 13:27:03+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-05 13:23:20+02	2020-07-05 13:39:09+02	local	E590T-localbase	\N	E590T-localbase-2020070401
2020-07-05 14:20:52+02	2020-07-05 14:23:49+02	graphics	opencv-core	\N	opencv-core-3.4.1_34
2020-07-05 14:23:50+02	2020-07-05 14:28:44+02	multimedia	ffmpeg	\N	ffmpeg-4.3_3,1
2020-07-05 14:28:45+02	2020-07-05 14:46:26+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-05 14:46:28+02	2020-07-05 15:00:46+02	www	chromium	\N	chromium-83.0.4103.116
2020-07-05 15:05:26+02	2020-07-05 15:06:57+02	devel	talloc	\N	talloc-2.3.1
2020-07-05 15:05:26+02	2020-07-05 15:07:45+02	databases	tdb	\N	tdb-1.4.3,1
2020-07-05 15:05:26+02	2020-07-05 15:07:53+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-05 15:07:53+02	2020-07-05 15:08:03+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-05 15:06:58+02	2020-07-05 15:08:20+02	devel	tevent	\N	tevent-0.10.2
2020-07-05 15:05:26+02	2020-07-05 15:09:09+02	graphics	opencv	\N	opencv-3.4.1_34
2020-07-05 15:09:09+02	2020-07-05 15:10:20+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-05 15:10:20+02	2020-07-05 15:11:37+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-05 15:11:37+02	2020-07-05 15:14:27+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-05 15:08:20+02	2020-07-05 15:26:23+02	net	samba410	\N	samba410-4.10.17
2020-07-05 15:26:23+02	2020-07-05 15:28:05+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-05 15:26:23+02	2020-07-05 15:31:12+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-07-05 15:28:05+02	2020-07-05 15:31:30+02	editors	pluma	\N	pluma-1.24.0
2020-07-05 15:31:30+02	2020-07-05 15:32:59+02	x11	mate	\N	mate-1.22.2
2020-07-05 15:32:59+02	2020-07-05 16:01:11+02	local	E590T-localbase	\N	E590T-localbase-2020070401
2020-07-05 15:05:26+02	2020-07-05 16:20:19+02	lang	ghc	\N	ghc-8.8.3_3
2020-07-05 16:30:38+02	2020-07-05 16:30:56+02	security	py-xkcdpass	py37	py37-xkcdpass-1.17.4
2020-07-05 16:30:38+02	2020-07-05 16:32:08+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-05 16:32:09+02	2020-07-05 16:32:16+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-05 16:30:56+02	2020-07-05 16:44:59+02	local	E590T-localbase	\N	E590T-localbase-2020070402
2020-07-05 20:25:25+02	2020-07-05 20:25:49+02	audio	libvorbis	\N	libvorbis-1.3.7,3
2020-07-05 20:25:50+02	2020-07-05 20:26:22+02	multimedia	libtheora	\N	libtheora-1.1.1_7
2020-07-05 20:25:50+02	2020-07-05 20:26:43+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-07-05 20:25:50+02	2020-07-05 20:27:09+02	audio	libsndfile	\N	libsndfile-1.0.28_2
2020-07-05 20:25:49+02	2020-07-05 20:27:17+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-07-05 20:25:50+02	2020-07-05 20:27:31+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-07-05 20:26:45+02	2020-07-05 20:28:04+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-07-05 20:27:17+02	2020-07-05 20:28:12+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-07-05 20:27:09+02	2020-07-05 20:28:14+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-07-05 20:26:23+02	2020-07-05 20:28:18+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-07-05 20:28:14+02	2020-07-05 20:29:07+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-07-05 20:27:32+02	2020-07-05 20:29:36+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-07-05 20:28:05+02	2020-07-05 20:29:36+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-07-05 20:28:19+02	2020-07-05 20:30:25+02	audio	jack	\N	jackit-0.125.0_9
2020-07-05 20:29:37+02	2020-07-05 20:31:08+02	accessibility	qt5-speech	\N	qt5-speech-5.14.2
2020-07-05 20:31:09+02	2020-07-05 20:33:46+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-05 20:29:37+02	2020-07-05 20:33:58+02	audio	sox	\N	sox-14.4.2_3
2020-07-05 20:30:27+02	2020-07-05 20:34:36+02	x11-wm	marco	\N	marco-1.22.4
2020-07-05 20:28:13+02	2020-07-05 20:34:37+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-07-05 20:34:37+02	2020-07-05 20:34:54+02	audio	twolame	\N	twolame-0.4.0
2020-07-05 20:25:25+02	2020-07-05 20:35:01+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-05 20:33:46+02	2020-07-05 20:36:23+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-05 20:34:37+02	2020-07-05 20:37:19+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-07-05 20:33:59+02	2020-07-05 20:37:29+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-05 20:34:54+02	2020-07-05 20:37:44+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-07-05 20:37:45+02	2020-07-05 20:38:22+02	audio	dssi	\N	dssi-1.1.1_8
2020-07-05 20:36:25+02	2020-07-05 20:39:58+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-05 20:37:33+02	2020-07-05 20:40:02+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-05 20:38:22+02	2020-07-05 20:41:12+02	audio	mate-media	\N	mate-media-1.24.0
2020-07-05 20:41:13+02	2020-07-05 20:41:49+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-07-05 20:37:19+02	2020-07-05 20:41:51+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-07-05 20:35:02+02	2020-07-05 20:45:23+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-07-05 20:41:51+02	2020-07-05 20:46:08+02	devel	py-game	py37	py37-game-1.9.6_1
2020-07-05 20:45:27+02	2020-07-05 20:52:29+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-07-05 20:41:52+02	2020-07-05 20:53:27+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-07-05 20:46:10+02	2020-07-05 20:53:34+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-07-05 20:40:00+02	2020-07-05 20:53:39+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-07-05 20:53:34+02	2020-07-05 20:54:03+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-07-05 20:40:03+02	2020-07-05 20:54:48+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-05 20:53:28+02	2020-07-05 20:55:56+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-05 20:29:08+02	2020-07-05 20:55:58+02	multimedia	ffmpeg	\N	ffmpeg-4.3_3,1
2020-07-05 20:54:05+02	2020-07-05 20:56:41+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-07-05 20:53:40+02	2020-07-05 20:57:23+02	games	alienarena	\N	alienarena-2013.766_9
2020-07-05 20:55:59+02	2020-07-05 20:59:18+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-05 20:59:21+02	2020-07-05 21:01:34+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-05 20:55:57+02	2020-07-05 21:03:45+02	games	battletanks	\N	btanks-0.9.8083_9
2020-07-05 20:57:25+02	2020-07-05 22:18:14+02	graphics	opencv	\N	opencv-3.4.1_34
2020-07-05 20:56:41+02	2020-07-06 00:55:28+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-05 21:01:37+02	2020-07-06 04:00:09+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_2
2020-07-06 04:00:13+02	2020-07-06 04:06:39+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-06 04:06:41+02	2020-07-06 04:14:24+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-06 04:14:26+02	2020-07-06 04:19:37+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-06 04:19:41+02	2020-07-06 04:21:52+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-10-05 02:06:01+02	2020-10-05 02:06:41+02	math	fcl05	\N	fcl05-0.5.0
2020-07-06 00:55:31+02	2020-07-06 04:23:12+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-06 04:21:55+02	2020-07-06 04:24:27+02	audio	faudio	\N	FAudio-20.07
2020-07-06 04:23:15+02	2020-07-06 04:27:50+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-06 04:27:54+02	2020-07-06 04:36:07+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-07-06 04:24:30+02	2020-07-06 04:36:56+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-06 04:37:00+02	2020-07-06 04:43:04+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-06 04:43:08+02	2020-07-06 04:48:15+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-06 04:48:19+02	2020-07-06 05:06:40+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-06 05:06:44+02	2020-07-06 05:18:17+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-06 05:50:40+02	2020-07-06 09:28:33+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-06 09:28:35+02	2020-07-06 10:13:22+02	editors	libreoffice	\N	libreoffice-6.4.4
2020-07-06 10:13:23+02	2020-07-06 20:58:55+02	www	chromium	\N	chromium-83.0.4103.116
2020-07-06 21:03:40+02	2020-07-06 21:04:38+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-06 21:04:38+02	2020-07-06 21:05:47+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-06 21:03:40+02	2020-07-06 21:08:50+02	multimedia	vlc	\N	vlc-3.0.11_2,4
2020-07-06 21:05:48+02	2020-07-06 21:09:40+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-06 21:09:41+02	2020-07-06 21:11:48+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-06 21:11:49+02	2020-07-06 21:13:31+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-06 21:13:31+02	2020-07-06 21:15:31+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.4
2020-07-06 21:15:32+02	2020-07-06 21:22:18+02	emulators	wine	\N	wine-5.0.1,1
2020-07-06 21:22:18+02	2020-07-06 21:22:53+02	games	monsterz	\N	monsterz-0.7.1_15
2020-07-06 21:22:54+02	2020-07-06 21:23:51+02	games	xblast	\N	xblast-2.10.4_17
2020-07-06 21:23:52+02	2020-07-06 22:39:38+02	graphics	blender	\N	blender-2.83.1
2020-07-06 22:39:40+02	2020-07-06 22:48:26+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-06 21:08:51+02	2020-07-06 22:58:41+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-06 22:58:43+02	2020-07-06 23:00:12+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-06 23:00:13+02	2020-07-06 23:08:30+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-06 23:08:30+02	2020-07-06 23:11:27+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-06 23:11:28+02	2020-07-06 23:23:20+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-06 23:23:21+02	2020-07-06 23:24:55+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-06 22:48:26+02	2020-07-07 00:39:12+02	graphics	krita	\N	krita-4.3.0
2020-07-07 00:39:13+02	2020-07-07 00:41:03+02	x11	mate	\N	mate-1.22.2
2020-07-06 21:03:40+02	2020-07-07 01:18:47+02	www	qt5-webengine	\N	qt5-webengine-5.14.2_1
2020-07-07 01:18:48+02	2020-07-07 01:20:49+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-07 01:20:49+02	2020-07-07 01:25:16+02	deskutils	calibre	\N	calibre-4.20.0
2020-07-07 00:41:05+02	2020-07-07 01:31:07+02	editors	vscode	\N	vscode-1.46.1
2020-07-07 01:31:09+02	2020-07-07 01:31:20+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-06 23:24:56+02	2020-07-07 02:15:08+02	www	firefox	\N	firefox-78.0.1_1,1
2020-07-07 01:25:17+02	2020-07-07 02:26:35+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-07 02:26:36+02	2020-07-07 02:40:25+02	local	E590T-localbase	\N	E590T-localbase-2020070402
2020-07-07 21:04:36+02	2020-07-07 21:04:49+02	graphics	poppler-data	\N	poppler-data-0.4.9_4
2020-07-07 21:04:36+02	2020-07-07 21:05:47+02	devel	dbus	\N	dbus-1.12.20
2020-07-07 21:05:47+02	2020-07-07 21:06:24+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-07-07 21:06:24+02	2020-07-07 21:07:24+02	net	avahi-app	\N	avahi-app-0.7_3
2020-07-07 21:04:51+02	2020-07-07 21:07:38+02	devel	qt5-qmake	\N	qt5-qmake-5.15.0
2020-07-07 21:07:39+02	2020-07-07 21:08:09+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-07-07 21:07:24+02	2020-07-07 21:08:31+02	print	cups	\N	cups-2.3.3_1
2020-07-07 21:08:09+02	2020-07-07 21:08:56+02	sysutils	polkit	\N	polkit-0.116
2020-07-07 21:04:36+02	2020-07-07 21:10:35+02	databases	sqlite3	\N	sqlite3-3.32.3,1
2020-07-07 21:08:56+02	2020-07-07 21:12:18+02	devel	qt5-buildtools	\N	qt5-buildtools-5.15.0
2020-07-07 21:08:32+02	2020-07-07 21:12:20+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_6
2020-07-07 21:10:35+02	2020-07-07 21:12:50+02	graphics	colord	\N	colord-1.3.5
2020-07-07 21:12:51+02	2020-07-07 21:13:15+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-07-07 21:12:21+02	2020-07-07 21:17:14+02	graphics	graphviz	\N	graphviz-2.44.0_5
2020-07-07 21:17:15+02	2020-07-07 21:21:55+02	lang	vala	\N	vala-0.40.15,1
2020-07-07 21:21:56+02	2020-07-07 21:22:13+02	audio	libvorbis	\N	libvorbis-1.3.7_1,3
2020-07-07 21:12:19+02	2020-07-07 21:23:00+02	devel	qt5-core	\N	qt5-core-5.15.0
2020-07-07 21:22:14+02	2020-07-07 21:25:14+02	security	nss	\N	nss-3.54
2020-07-07 21:13:16+02	2020-07-07 21:25:42+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-07-07 21:25:43+02	2020-07-07 21:27:32+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-07-07 21:25:15+02	2020-07-07 21:28:06+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-07-07 21:23:01+02	2020-07-07 21:28:21+02	net	qt5-network	\N	qt5-network-5.15.0
2020-07-07 21:28:06+02	2020-07-07 21:28:26+02	audio	libsndfile	\N	libsndfile-1.0.28_2
2020-07-07 21:28:27+02	2020-07-07 21:31:14+02	graphics	poppler	\N	poppler-0.90.0_1
2020-07-07 21:31:15+02	2020-07-07 21:32:52+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-07-07 21:32:56+02	2020-07-07 21:34:05+02	devel	dconf	\N	dconf-0.36.0
2020-07-07 21:34:06+02	2020-07-07 21:34:42+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-07-07 21:27:33+02	2020-07-07 21:36:35+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-07-07 21:36:36+02	2020-07-07 21:37:37+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-07-07 21:37:38+02	2020-07-07 21:39:34+02	devel	libsoup	\N	libsoup-2.62.3
2020-07-07 21:39:34+02	2020-07-07 21:40:33+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-07-07 21:40:34+02	2020-07-07 21:40:51+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-07-07 21:40:53+02	2020-07-07 21:41:06+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-07-07 21:41:07+02	2020-07-07 21:41:20+02	multimedia	libtheora	\N	libtheora-1.1.1_7
2020-07-07 21:41:21+02	2020-07-07 21:41:38+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-07-07 21:34:43+02	2020-07-07 21:42:41+02	print	texlive-base	\N	texlive-base-20150521_53
2020-07-07 21:28:21+02	2020-07-07 21:46:10+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-07-07 21:41:39+02	2020-07-07 21:50:01+02	multimedia	ffmpeg	\N	ffmpeg-4.3_3,1
2020-07-07 21:50:02+02	2020-07-07 21:51:23+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-07-07 21:51:24+02	2020-07-07 21:52:10+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-07-07 21:52:11+02	2020-07-07 21:52:56+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-07-07 21:52:57+02	2020-07-07 21:54:11+02	net	geoclue	\N	geoclue-2.5.5
2020-07-07 21:46:10+02	2020-07-07 21:54:20+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-07-07 21:54:12+02	2020-07-07 21:54:37+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-07-07 21:54:37+02	2020-07-07 21:56:24+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-07-07 21:56:25+02	2020-07-07 21:58:25+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-07-07 21:58:25+02	2020-07-07 21:59:02+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-07-07 21:59:03+02	2020-07-07 21:59:44+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-07-07 21:42:42+02	2020-07-07 22:01:54+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-07-07 22:01:58+02	2020-07-07 22:05:28+02	print	tex-formats	\N	tex-formats-20150521_2
2020-07-07 22:05:28+02	2020-07-07 22:13:16+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-07-07 21:54:20+02	2020-07-07 22:22:33+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-07-07 22:22:34+02	2020-07-07 22:24:45+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-07-07 22:24:46+02	2020-07-07 22:25:48+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.71.0_2
2020-07-07 22:13:17+02	2020-07-07 22:29:40+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-07-07 22:29:41+02	2020-07-07 22:32:25+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-07-07 22:32:26+02	2020-07-07 22:32:51+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-07-07 22:25:49+02	2020-07-07 22:34:05+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-07-07 22:32:52+02	2020-07-07 22:34:09+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-07-07 22:34:10+02	2020-07-07 22:34:58+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-07-07 22:34:59+02	2020-07-07 22:35:32+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.19
2020-07-07 22:34:06+02	2020-07-07 22:35:45+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.71.0
2020-07-07 22:35:32+02	2020-07-07 22:35:59+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.19
2020-07-07 22:35:59+02	2020-07-07 22:37:07+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-07-07 22:37:08+02	2020-07-07 22:39:07+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-07-07 22:39:08+02	2020-07-07 22:40:11+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-07-07 22:35:46+02	2020-07-07 22:40:28+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-07-07 22:40:29+02	2020-07-07 22:41:43+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-07 22:40:12+02	2020-07-07 22:41:59+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-07-07 22:41:43+02	2020-07-07 22:42:34+02	sysutils	policykit	\N	policykit-0.9_10
2020-07-07 22:41:59+02	2020-07-07 22:42:41+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-07-07 22:42:35+02	2020-07-07 22:43:27+02	archivers	kf5-karchive	\N	kf5-karchive-5.71.0
2020-07-07 22:42:43+02	2020-07-07 22:44:10+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-07-07 22:44:10+02	2020-07-07 22:45:02+02	www	py-django22	py37	py37-django22-2.2.14
2020-07-07 22:45:03+02	2020-07-07 22:47:17+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-07-07 22:43:28+02	2020-07-07 22:48:14+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-07 22:47:18+02	2020-07-07 22:48:41+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-07-07 22:48:15+02	2020-07-07 22:49:38+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-07-07 22:49:40+02	2020-07-07 22:51:13+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-07-07 22:51:14+02	2020-07-07 22:52:38+02	sysutils	hal	\N	hal-0.5.14_34
2020-07-07 22:52:38+02	2020-07-07 22:54:38+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-07-07 22:54:39+02	2020-07-07 22:55:19+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-07-07 22:48:41+02	2020-07-07 22:56:41+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-07 22:55:19+02	2020-07-07 22:58:30+02	graphics	poppler-glib	\N	poppler-glib-0.90.0_1
2020-07-07 22:56:42+02	2020-07-07 23:00:06+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-07-07 23:00:08+02	2020-07-07 23:02:21+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-07-07 23:02:22+02	2020-07-07 23:03:28+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-07-07 23:03:29+02	2020-07-07 23:04:51+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-07-07 23:04:52+02	2020-07-07 23:06:20+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-07-07 23:06:20+02	2020-07-07 23:07:44+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-07-07 23:07:45+02	2020-07-07 23:09:11+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-07 22:58:31+02	2020-07-07 23:10:11+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-07-07 23:09:12+02	2020-07-07 23:10:23+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.71.0
2020-07-07 23:10:13+02	2020-07-07 23:11:36+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-07-07 23:10:24+02	2020-07-07 23:11:51+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-07-07 23:11:36+02	2020-07-07 23:14:03+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-07-07 23:14:04+02	2020-07-07 23:15:31+02	net	remmina	\N	remmina-1.3.9
2020-07-07 23:15:32+02	2020-07-07 23:19:26+02	security	gcr	\N	gcr-3.28.0
2020-07-07 23:19:26+02	2020-07-07 23:20:44+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-07-07 23:20:45+02	2020-07-07 23:21:55+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-07-07 23:21:56+02	2020-07-07 23:23:04+02	audio	mpg123	\N	mpg123-1.26.2
2020-07-07 23:23:05+02	2020-07-07 23:25:36+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.71.0
2020-07-07 23:11:52+02	2020-07-07 23:26:01+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-07-07 23:25:37+02	2020-07-07 23:27:17+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-07-07 23:26:01+02	2020-07-07 23:27:20+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-07-07 23:27:18+02	2020-07-07 23:28:52+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-07-07 23:28:52+02	2020-07-07 23:29:40+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-07-07 23:29:41+02	2020-07-07 23:30:24+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-07-07 23:30:24+02	2020-07-07 23:33:11+02	math	gnuplot	\N	gnuplot-5.2.8_1
2020-07-07 23:33:11+02	2020-07-07 23:34:05+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-07 23:34:06+02	2020-07-07 23:34:41+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-07-07 23:34:41+02	2020-07-07 23:37:07+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-07 23:37:07+02	2020-07-07 23:39:24+02	security	gnupg	\N	gnupg-2.2.20
2020-07-07 23:39:24+02	2020-07-07 23:40:46+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-07-07 23:40:47+02	2020-07-07 23:43:50+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-07-07 23:43:50+02	2020-07-07 23:44:02+02	devel	popt	\N	popt-1.18_1
2020-07-07 23:44:03+02	2020-07-07 23:46:22+02	graphics	exiv2	\N	exiv2-0.27.3,1
2020-07-07 23:27:21+02	2020-07-07 23:50:00+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-07 23:50:01+02	2020-07-07 23:50:41+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-07 23:50:41+02	2020-07-07 23:55:28+02	net	libmateweather	\N	libmateweather-1.24.0
2020-07-07 23:55:29+02	2020-07-07 23:56:09+02	security	gpgme	\N	gpgme-1.13.1
2020-07-07 23:56:09+02	2020-07-07 23:58:15+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-07-07 23:58:16+02	2020-07-07 23:59:40+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-07-07 23:59:41+02	2020-07-08 00:01:11+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-07-08 00:01:12+02	2020-07-08 00:11:00+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-08 00:11:01+02	2020-07-08 00:11:24+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-07-08 00:11:25+02	2020-07-08 00:11:46+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-07-08 00:11:46+02	2020-07-08 00:12:57+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-07 21:59:44+02	2020-07-08 00:13:45+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-07-08 00:12:58+02	2020-07-08 00:13:47+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-07-08 07:15:46+02	2020-07-08 07:17:54+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-08 07:17:54+02	2020-07-08 07:29:03+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-07-08 07:29:05+02	2020-07-08 07:36:44+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-07-08 07:36:46+02	2020-07-08 07:42:04+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-07-08 07:42:08+02	2020-07-08 07:46:35+02	sysutils	upower	\N	upower-0.99.11
2020-07-08 07:15:46+02	2020-07-08 08:03:01+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-07-08 08:03:02+02	2020-07-08 08:04:58+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-07-08 08:04:59+02	2020-07-08 08:09:47+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-07-08 08:09:47+02	2020-07-08 08:10:40+02	x11	xterm	\N	xterm-357
2020-07-08 08:10:41+02	2020-07-08 08:13:20+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-07-08 08:13:20+02	2020-07-08 08:15:10+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-08 08:15:10+02	2020-07-08 08:50:28+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-07-08 08:50:31+02	2020-07-08 08:51:20+02	audio	jack	\N	jackit-0.125.0_9
2020-07-08 08:51:20+02	2020-07-08 08:51:54+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-07-08 07:15:46+02	2020-07-08 08:52:01+02	math	vtk8	\N	vtk8-8.2.0
2020-07-08 08:51:54+02	2020-07-08 08:53:38+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-08 08:53:39+02	2020-07-08 08:54:20+02	devel	librest	\N	rest-0.7.93
2020-07-08 08:54:20+02	2020-07-08 08:56:51+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-07-08 08:56:52+02	2020-07-08 08:57:57+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-07-08 08:52:02+02	2020-07-08 08:58:13+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-08 08:57:58+02	2020-07-08 09:00:38+02	graphics	poppler-utils	\N	poppler-utils-0.90.0_1
2020-07-08 09:00:39+02	2020-07-08 09:02:40+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-07-08 09:02:40+02	2020-07-08 09:03:26+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-07-08 09:03:27+02	2020-07-08 09:04:23+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-07-08 09:04:24+02	2020-07-08 09:05:26+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-08 08:58:14+02	2020-07-08 09:18:07+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-08 09:18:09+02	2020-07-08 09:19:42+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-07-08 09:19:43+02	2020-07-08 09:21:11+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-08 09:21:12+02	2020-07-08 09:25:43+02	x11-fm	caja	\N	caja-1.22.3_1
2020-07-08 09:25:43+02	2020-07-08 09:28:45+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-08 09:28:46+02	2020-07-08 09:29:55+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-07-08 09:29:56+02	2020-07-08 09:31:49+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-08 09:31:50+02	2020-07-08 09:35:23+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-07-08 09:35:23+02	2020-07-08 09:36:20+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-07-08 09:36:21+02	2020-07-08 09:51:25+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-07-08 09:51:26+02	2020-07-08 09:55:38+02	graphics	gegl	\N	gegl-0.4.24_1
2020-07-08 09:55:39+02	2020-07-08 09:58:59+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-07-08 09:59:00+02	2020-07-08 10:28:12+02	net	samba410	\N	samba410-4.10.17
2020-07-08 10:28:12+02	2020-07-08 10:29:29+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-07-08 10:29:29+02	2020-07-08 10:30:55+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-08 10:30:56+02	2020-07-08 10:34:32+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-07-08 10:34:32+02	2020-07-08 10:35:30+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-07-08 10:35:31+02	2020-07-08 10:37:58+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-08 10:37:59+02	2020-07-08 10:39:17+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-07-08 10:39:17+02	2020-07-08 10:40:35+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-07-08 10:40:35+02	2020-07-08 10:41:05+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-07-08 10:41:05+02	2020-07-08 10:42:31+02	x11	zenity	\N	zenity-3.32.0
2020-07-08 10:42:31+02	2020-07-08 10:43:26+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-08 09:05:27+02	2020-07-08 10:43:28+02	www	node	\N	node-14.5.0
2020-07-08 10:43:29+02	2020-07-08 10:44:42+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-07-08 10:44:43+02	2020-07-08 10:46:13+02	audio	sox	\N	sox-14.4.2_3
2020-07-08 10:43:27+02	2020-07-08 10:55:25+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-07-08 10:46:14+02	2020-07-08 10:59:22+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-08 10:55:26+02	2020-07-08 10:59:28+02	devel	libgdata	\N	libgdata-0.17.9
2020-07-08 10:59:23+02	2020-07-08 11:00:35+02	devel	libpeas	\N	libpeas-1.22.0
2020-07-08 10:59:29+02	2020-07-08 11:05:49+02	editors	emacs	full	emacs-26.3_5,3
2020-07-08 11:05:50+02	2020-07-08 11:11:01+02	graphics	opencv	\N	opencv-3.4.1_34
2020-07-08 11:11:02+02	2020-07-08 11:13:26+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-08 11:13:27+02	2020-07-08 11:13:53+02	net	gssdp	\N	gssdp-1.2.3
2020-07-08 11:13:53+02	2020-07-08 11:14:55+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-08 11:14:55+02	2020-07-08 11:16:46+02	print	cups-filters	\N	cups-filters-1.27.5_1
2020-07-08 11:00:36+02	2020-07-08 11:18:28+02	graphics	gimp-app	\N	gimp-app-2.10.20_1,1
2020-07-08 07:46:38+02	2020-07-08 11:19:29+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-08 11:19:31+02	2020-07-08 11:20:02+02	security	mbedtls	\N	mbedtls-2.16.7
2020-07-08 11:20:03+02	2020-07-08 11:20:59+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-07-08 11:21:00+02	2020-07-08 11:22:36+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-07-08 11:16:47+02	2020-07-08 11:23:53+02	print	fontforge	\N	fontforge-20190801_2
2020-07-08 11:22:36+02	2020-07-08 11:30:48+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-07-08 11:18:29+02	2020-07-08 11:37:37+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-07-08 11:37:38+02	2020-07-08 11:39:53+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-07-08 11:39:54+02	2020-07-08 11:44:08+02	x11	xorg	\N	xorg-7.7_3
2020-07-08 11:44:09+02	2020-07-08 11:46:56+02	x11	yelp	\N	yelp-3.28.1_2
2020-07-08 11:46:57+02	2020-07-08 11:48:33+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-07-08 11:48:34+02	2020-07-08 11:49:40+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-07-08 11:49:41+02	2020-07-08 11:52:02+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-07-08 11:52:03+02	2020-07-08 11:57:23+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-07-08 11:57:27+02	2020-07-08 11:58:43+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-07-08 11:58:44+02	2020-07-08 12:00:04+02	x11-wm	marco	\N	marco-1.22.4
2020-07-08 12:00:05+02	2020-07-08 12:00:32+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-07-08 12:00:32+02	2020-07-08 12:00:46+02	audio	twolame	\N	twolame-0.4.0
2020-07-08 11:23:53+02	2020-07-08 12:02:53+02	www	chromium	\N	chromium-83.0.4103.116
2020-07-08 12:00:46+02	2020-07-08 12:02:56+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-07-08 12:02:55+02	2020-07-08 12:03:53+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-07-08 12:02:57+02	2020-07-08 12:05:08+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-07-08 12:03:53+02	2020-07-08 12:06:44+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-08 12:06:45+02	2020-07-08 12:08:51+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-08 12:05:08+02	2020-07-08 12:14:10+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-07-08 12:14:11+02	2020-07-08 12:14:49+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-07-08 12:08:51+02	2020-07-08 12:15:06+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.0_1
2020-07-08 12:14:49+02	2020-07-08 12:15:46+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-08 12:15:47+02	2020-07-08 12:16:50+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-07-08 12:16:51+02	2020-07-08 12:40:41+02	math	R	\N	R-4.0.2
2020-07-08 12:15:06+02	2020-07-08 12:44:19+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-07-08 12:44:20+02	2020-07-08 12:45:52+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-08 12:45:53+02	2020-07-08 12:49:07+02	net	freerdp	\N	freerdp-2.1.1
2020-07-08 12:40:44+02	2020-07-08 12:49:18+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-07-08 12:49:08+02	2020-07-08 12:49:39+02	net	gupnp	\N	gupnp-1.1.1_1
2020-07-08 12:49:18+02	2020-07-08 12:50:09+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-07-08 12:49:39+02	2020-07-08 12:50:59+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-08 11:30:49+02	2020-07-08 21:09:37+02	www	qt5-webengine	\N	qt5-webengine-5.15.0
2020-07-08 12:50:10+02	2020-07-08 12:51:02+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-08 12:51:03+02	2020-07-08 12:51:31+02	print	libspectre	\N	libspectre-0.2.9
2020-07-08 12:51:31+02	2020-07-08 12:53:58+02	print	mftrace	\N	mftrace-1.2.18_2
2020-07-08 12:51:00+02	2020-07-08 12:55:20+02	print	gutenprint	\N	gutenprint-5.3.3
2020-07-08 12:53:59+02	2020-07-08 13:00:50+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-07-08 12:55:21+02	2020-07-08 13:01:31+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-07-08 13:00:50+02	2020-07-08 13:01:52+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-07-08 13:01:33+02	2020-07-08 13:01:53+02	sysutils	hal-info	\N	hal-info-20091130
2020-07-08 13:01:53+02	2020-07-08 13:02:49+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-07-08 13:01:52+02	2020-07-08 13:04:50+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-07-08 13:02:49+02	2020-07-08 13:05:07+02	textproc	dblatex	\N	dblatex-0.3.11
2020-07-08 13:04:51+02	2020-07-08 13:05:37+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-08 13:05:37+02	2020-07-08 13:06:17+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-08 13:05:08+02	2020-07-08 13:06:37+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-08 13:06:17+02	2020-07-08 13:07:26+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-07-08 13:06:38+02	2020-07-08 13:07:26+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-07-08 13:07:26+02	2020-07-08 13:09:56+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-07-08 13:07:27+02	2020-07-08 13:10:07+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-07-08 13:10:07+02	2020-07-08 13:12:54+02	archivers	engrampa	\N	engrampa-1.24.0
2020-07-08 13:12:55+02	2020-07-08 13:14:09+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-07-08 13:14:11+02	2020-07-08 13:14:30+02	audio	dssi	\N	dssi-1.1.1_8
2020-07-08 13:14:31+02	2020-07-08 13:15:13+02	audio	faudio	\N	FAudio-20.07
2020-07-08 13:15:13+02	2020-07-08 13:16:29+02	audio	mate-media	\N	mate-media-1.24.0
2020-07-08 13:16:30+02	2020-07-08 13:16:51+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-07-08 13:16:52+02	2020-07-08 13:18:38+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-07-08 13:18:38+02	2020-07-08 13:22:47+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-07-08 13:22:47+02	2020-07-08 13:23:52+02	deskutils	mozo	\N	mozo-1.24.0
2020-07-08 13:23:53+02	2020-07-08 13:24:57+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-07-08 13:24:58+02	2020-07-08 13:25:19+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-07-08 13:25:20+02	2020-07-08 13:26:57+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0_1
2020-07-08 13:26:58+02	2020-07-08 13:27:33+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-07-08 13:27:35+02	2020-07-08 13:28:36+02	devel	py-game	py37	py37-game-1.9.6_1
2020-07-08 13:09:57+02	2020-07-08 13:39:45+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-07-08 13:28:37+02	2020-07-08 13:54:35+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-07-08 13:54:36+02	2020-07-08 14:00:53+02	editors	pluma	\N	pluma-1.24.0
2020-07-08 14:00:54+02	2020-07-08 14:01:23+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-07-08 14:01:23+02	2020-07-08 14:06:05+02	graphics	atril	\N	atril-1.22.3_7
2020-07-08 14:06:06+02	2020-07-08 14:09:14+02	graphics	eom	\N	eom-1.24.0
2020-07-08 14:09:15+02	2020-07-08 14:11:23+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-08 14:11:24+02	2020-07-08 14:13:35+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-07-08 14:13:36+02	2020-07-08 14:16:02+02	graphics	py-gimp	\N	py27-gimp-2.10.20_1
2020-07-08 14:16:04+02	2020-07-08 14:31:13+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-07-08 14:31:14+02	2020-07-08 14:41:54+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-07-08 14:41:55+02	2020-07-08 16:42:55+02	lang	julia11	\N	julia11-1.1.1_5
2020-07-08 16:42:56+02	2020-07-08 16:45:34+02	math	mate-calc	\N	mate-calc-1.24.0
2020-07-08 16:45:35+02	2020-07-08 16:46:57+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-07-08 16:46:58+02	2020-07-08 16:53:00+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-08 16:53:06+02	2020-07-08 16:55:49+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-07-08 16:55:50+02	2020-07-08 17:09:32+02	multimedia	vlc	\N	vlc-3.0.11_2,4
2020-07-08 17:09:33+02	2020-07-08 17:11:55+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-08 17:11:55+02	2020-07-08 17:12:57+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-07-08 17:12:57+02	2020-07-08 17:14:09+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-07-08 17:14:10+02	2020-07-08 17:15:20+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-07-08 17:15:20+02	2020-07-08 17:16:31+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-07-08 17:16:32+02	2020-07-08 17:17:34+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-07-08 17:17:35+02	2020-07-08 17:18:49+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-07-08 17:18:54+02	2020-07-08 17:21:44+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-07-08 17:21:45+02	2020-07-08 17:25:13+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-08 17:25:18+02	2020-07-08 17:28:06+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-08 17:28:07+02	2020-07-08 17:31:08+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-08 17:31:09+02	2020-07-08 17:32:57+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-08 17:32:57+02	2020-07-08 17:55:08+02	print	lilypond	\N	lilypond-2.20.0
2020-07-08 17:55:09+02	2020-07-08 18:05:38+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-07-08 18:05:39+02	2020-07-08 18:13:35+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-07-08 18:13:36+02	2020-07-08 18:16:48+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-07-08 18:16:49+02	2020-07-08 18:30:37+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-07-08 18:30:38+02	2020-07-08 18:58:03+02	print	texlive-docs	\N	texlive-docs-20150523
2020-07-08 18:58:04+02	2020-07-08 18:59:01+02	print	transfig	\N	transfig-3.2.5e_4
2020-07-08 18:59:02+02	2020-07-08 19:01:35+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-07-08 19:01:36+02	2020-07-08 19:05:21+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-07-08 19:05:22+02	2020-07-08 19:06:53+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-07-08 19:06:54+02	2020-07-08 19:08:23+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-07-08 19:08:23+02	2020-07-08 19:08:40+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-07-08 19:08:41+02	2020-07-08 19:08:57+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-07-08 19:08:57+02	2020-07-08 19:09:23+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-07-08 19:09:24+02	2020-07-08 19:11:44+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-08 19:11:45+02	2020-07-08 19:12:55+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-07-08 19:12:57+02	2020-07-08 19:21:50+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-07-08 19:21:52+02	2020-07-08 19:23:35+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-08 19:23:36+02	2020-07-08 19:25:50+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-07-08 19:25:51+02	2020-07-08 19:30:20+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-07-08 19:30:21+02	2020-07-08 19:32:47+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-07-08 19:32:48+02	2020-07-08 19:34:26+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-07-08 19:34:28+02	2020-07-08 19:49:18+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-07-08 13:39:46+02	2020-07-08 20:15:53+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-08 19:49:19+02	2020-07-08 20:23:37+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-08 20:23:38+02	2020-07-08 20:30:51+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-08 20:30:52+02	2020-07-08 20:34:18+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-08 20:15:54+02	2020-07-08 20:46:42+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-08 20:46:43+02	2020-07-08 20:49:10+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-08 20:49:11+02	2020-07-08 20:51:07+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-08 21:09:38+02	2020-07-08 21:11:35+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-08 21:11:36+02	2020-07-08 21:14:27+02	deskutils	calibre	\N	calibre-4.20.0_1
2020-07-08 20:51:08+02	2020-07-08 21:15:15+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-07-08 21:14:28+02	2020-07-08 21:15:26+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-07-08 21:15:26+02	2020-07-08 21:16:20+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-07-08 21:15:16+02	2020-07-08 21:43:13+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-08 21:16:21+02	2020-07-08 21:53:50+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-08 21:53:53+02	2020-07-08 21:54:07+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-07-08 21:54:07+02	2020-07-08 21:59:48+02	devel	subversion	\N	subversion-1.14.0
2020-07-08 20:34:20+02	2020-07-08 22:33:50+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-08 22:33:52+02	2020-07-08 22:36:29+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-08 22:36:29+02	2020-07-08 22:41:31+02	editors	vim	\N	vim-8.2.1110
2020-07-08 21:59:48+02	2020-07-08 23:28:32+02	editors	codelite	\N	codelite-12.0_4
2020-07-08 22:41:32+02	2020-07-08 23:48:52+02	emulators	qemu	\N	qemu-4.1.1_4
2020-07-08 23:48:53+02	2020-07-09 00:14:52+02	emulators	wine	\N	wine-5.0.1,1
2020-07-09 00:14:52+02	2020-07-09 00:31:02+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-07-09 00:31:02+02	2020-07-09 00:31:21+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-07-09 00:31:22+02	2020-07-09 00:33:33+02	games	alienarena	\N	alienarena-2013.766_9
2020-07-09 00:33:33+02	2020-07-09 00:34:36+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-07-09 00:34:37+02	2020-07-09 00:39:09+02	games	battletanks	\N	btanks-0.9.8083_9
2020-07-09 00:39:10+02	2020-07-09 00:39:49+02	games	monsterz	\N	monsterz-0.7.1_15
2020-07-09 00:39:34+02	2020-07-09 00:40:21+02	games	xblast	\N	xblast-2.10.4_17
2020-07-09 00:39:50+02	2020-07-09 00:40:35+02	games	xcowsay	\N	xcowsay-1.5
2020-07-08 23:28:41+02	2020-07-09 00:39:33+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-07-09 00:40:36+02	2020-07-09 00:45:37+02	graphics	dia	\N	dia-0.97.3_1,1
2020-07-09 00:45:38+02	2020-07-09 00:57:06+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-09 00:57:07+02	2020-07-09 00:58:29+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-09 00:40:21+02	2020-07-09 01:06:42+02	graphics	blender	\N	blender-2.83.1
2020-07-09 00:58:30+02	2020-07-09 02:02:18+02	graphics	inkscape	\N	inkscape-1.0_3
2020-07-09 02:02:19+02	2020-07-09 02:06:19+02	graphics	wings	\N	wings-2.2.5
2020-07-09 02:06:20+02	2020-07-09 02:08:52+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-09 02:08:53+02	2020-07-09 02:10:03+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-07-09 02:10:04+02	2020-07-09 02:49:58+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.2
2020-07-09 01:06:42+02	2020-07-09 02:50:20+02	graphics	krita	\N	krita-4.3.0_1
2020-07-09 02:50:21+02	2020-07-09 02:50:51+02	lang	julia	\N	julia-1.0.3_3
2020-07-09 02:54:40+02	2020-07-09 02:55:40+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-09 02:55:40+02	2020-07-09 03:01:23+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-09 03:01:24+02	2020-07-09 03:03:58+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-09 03:03:59+02	2020-07-09 03:14:10+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-09 03:14:11+02	2020-07-09 03:19:24+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-07-09 03:19:25+02	2020-07-09 03:21:04+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-09 03:21:05+02	2020-07-09 03:21:22+02	net	vnstat	\N	vnstat-2.6_1
2020-07-09 02:49:59+02	2020-07-09 03:32:19+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-07-09 03:32:19+02	2020-07-09 03:33:30+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-07-09 03:33:30+02	2020-07-09 03:34:47+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-09 03:34:48+02	2020-07-09 03:35:45+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-09 03:35:46+02	2020-07-09 03:36:43+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-09 03:36:44+02	2020-07-09 03:37:41+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-09 03:37:41+02	2020-07-09 03:38:39+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-09 03:38:40+02	2020-07-09 03:39:35+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-07-09 03:39:35+02	2020-07-09 03:40:25+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-09 03:40:25+02	2020-07-09 03:41:46+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-09 03:41:47+02	2020-07-09 03:42:29+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-09 03:21:22+02	2020-07-09 03:42:53+02	net	wireshark	\N	wireshark-3.2.5
2020-07-09 03:42:30+02	2020-07-09 03:43:08+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-09 03:42:54+02	2020-07-09 03:43:36+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-07-09 03:43:36+02	2020-07-09 03:44:04+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-07-09 03:44:05+02	2020-07-09 03:44:34+02	print	gv	\N	gv-3.7.4_5
2020-07-09 03:43:08+02	2020-07-09 03:44:58+02	print	auctex	full	auctex-emacs26-12.2
2020-07-08 21:43:13+02	2020-07-09 03:50:30+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200706
2020-07-09 03:50:31+02	2020-07-09 03:55:29+02	print	texlive-full	\N	texlive-full-20150521
2020-07-09 03:55:30+02	2020-07-09 04:00:29+02	security	klee	\N	klee-2.1
2020-07-09 04:00:29+02	2020-07-09 04:00:42+02	security	nyx	\N	nyx-2.1.0_1
2020-07-09 04:00:43+02	2020-07-09 04:01:57+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-09 03:44:34+02	2020-07-09 04:02:30+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-09 04:02:30+02	2020-07-09 04:02:37+02	sysutils	cpupdate	\N	cpupdate-g20180513_2
2020-07-09 04:01:57+02	2020-07-09 04:02:56+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-07-09 04:02:37+02	2020-07-09 04:03:31+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-07-09 04:03:31+02	2020-07-09 04:03:44+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-07-09 04:03:44+02	2020-07-09 04:03:57+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-07-09 04:03:57+02	2020-07-09 04:04:11+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-07-09 04:04:12+02	2020-07-09 04:04:25+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-07-09 04:04:25+02	2020-07-09 04:04:39+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-07-09 04:04:40+02	2020-07-09 04:04:56+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-07-09 04:04:56+02	2020-07-09 04:05:09+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-07-09 04:05:10+02	2020-07-09 04:05:22+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-07-09 04:05:22+02	2020-07-09 04:05:34+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-07-09 04:05:35+02	2020-07-09 04:05:52+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-07-09 04:05:52+02	2020-07-09 04:07:48+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-09 04:07:48+02	2020-07-09 04:09:39+02	x11	mate	\N	mate-1.22.2
2020-07-09 04:09:40+02	2020-07-09 04:16:40+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-07-09 04:16:40+02	2020-07-09 04:22:36+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-07-09 04:22:37+02	2020-07-09 04:24:42+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-07-09 03:44:59+02	2020-07-09 04:30:26+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-07-09 04:30:26+02	2020-07-09 04:30:35+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-09 04:30:36+02	2020-07-09 04:31:43+02	graphics	xfig	\N	xfig-3.2.7a
2020-07-09 04:31:43+02	2020-07-09 04:32:18+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-07-09 04:24:42+02	2020-07-09 04:59:19+02	editors	vscode	\N	vscode-1.46.1
2020-07-09 04:02:57+02	2020-07-09 05:09:59+02	www	firefox	\N	firefox-78.0.1_1,1
2020-07-09 11:48:04+02	2020-07-09 11:48:18+02	misc	help2man	\N	help2man-1.47.16
2020-07-09 11:48:04+02	2020-07-09 11:48:22+02	textproc	py-sphinx	py37	py37-sphinx-3.1.2,1
2020-07-09 11:48:18+02	2020-07-09 11:48:30+02	www	p5-HTTP-Message	\N	p5-HTTP-Message-6.25
2020-07-09 11:48:23+02	2020-07-09 11:48:33+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-07-09 11:48:31+02	2020-07-09 11:48:43+02	www	p5-HTTP-Cookies	\N	p5-HTTP-Cookies-6.08
2020-10-05 02:06:59+02	2020-10-05 02:07:17+02	net	gssdp	\N	gssdp-1.2.3
2020-07-09 11:48:34+02	2020-07-09 11:48:46+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.12
2020-07-09 11:48:43+02	2020-07-09 11:48:55+02	www	p5-HTTP-Negotiate	\N	p5-HTTP-Negotiate-6.01_1
2020-07-09 11:48:04+02	2020-07-09 11:49:38+02	ftp	curl	\N	curl-7.71.1
2020-07-09 11:48:46+02	2020-07-09 11:52:13+02	devel	google-perftools	\N	google-perftools-2.8
2020-07-09 11:52:15+02	2020-07-09 11:54:10+02	ftp	php73-curl	\N	php73-curl-7.3.19
2020-07-09 11:54:17+02	2020-07-09 11:57:20+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-09 11:57:23+02	2020-07-09 12:08:50+02	devel	git	\N	git-2.27.0
2020-07-09 12:08:51+02	2020-07-09 12:12:03+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-07-09 12:12:06+02	2020-07-09 12:13:37+02	net	liboauth	\N	liboauth-1.0.3_4
2020-07-09 12:13:39+02	2020-07-09 12:18:37+02	devel	libgdata	\N	libgdata-0.17.9
2020-07-09 12:18:38+02	2020-07-09 12:18:56+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-07-09 12:18:57+02	2020-07-09 12:19:46+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-09 12:19:47+02	2020-07-09 12:20:42+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-07-09 11:48:55+02	2020-07-09 12:21:23+02	java	openjdk11	\N	openjdk11-11.0.7+10.2_3
2020-07-09 11:49:39+02	2020-07-09 12:21:51+02	devel	cmake	\N	cmake-3.17.3_1
2020-07-09 12:20:43+02	2020-07-09 12:23:17+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-09 12:21:53+02	2020-07-09 12:25:42+02	databases	mysql57-client	\N	mysql57-client-5.7.30_2
2020-07-09 12:25:43+02	2020-07-09 12:26:18+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-07-09 12:21:23+02	2020-07-09 12:43:02+02	math	R	\N	R-4.0.2
2020-07-09 12:43:03+02	2020-07-09 12:58:42+02	net	libcmis	\N	libcmis-0.5.2_1
2020-07-09 12:58:42+02	2020-07-09 13:00:31+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-09 13:00:32+02	2020-07-09 13:01:34+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-09 13:01:34+02	2020-07-09 13:02:47+02	textproc	redland	\N	redland-1.0.17_4
2020-07-09 13:02:47+02	2020-07-09 13:03:49+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-07-09 12:26:19+02	2020-07-09 14:13:21+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-09 14:13:22+02	2020-07-09 14:19:25+02	editors	pluma	\N	pluma-1.24.0
2020-07-09 14:19:25+02	2020-07-09 16:20:17+02	lang	julia11	\N	julia11-1.1.1_5
2020-07-09 16:20:52+02	2020-07-09 16:22:15+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-09 16:22:16+02	2020-07-09 16:23:59+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-09 16:24:00+02	2020-07-09 16:24:49+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-07-09 16:24:50+02	2020-07-09 16:25:15+02	www	p5-libwww	\N	p5-libwww-6.46
2020-07-09 16:25:16+02	2020-07-09 16:25:48+02	archivers	unrar	\N	unrar-5.91,6
2020-07-09 16:25:48+02	2020-07-09 17:01:38+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-09 17:01:42+02	2020-07-09 17:08:08+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-09 17:08:09+02	2020-07-09 17:11:09+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-09 12:23:18+02	2020-07-09 18:23:03+02	lang	rust	\N	rust-1.44.1_1
2020-07-09 18:23:05+02	2020-07-09 18:29:52+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3
2020-07-09 18:29:53+02	2020-07-09 18:32:03+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-09 18:32:04+02	2020-07-09 18:33:59+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-09 17:11:10+02	2020-07-09 19:03:52+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-09 13:03:50+02	2020-07-09 19:32:46+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-09 19:32:47+02	2020-07-09 19:33:18+02	devel	tig	\N	tig-2.5.1
2020-07-09 20:25:14+02	2020-07-09 20:26:23+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-09 20:26:23+02	2020-07-09 20:26:30+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-09 20:32:02+02	2020-07-09 21:08:08+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-09 20:32:02+02	2020-07-09 21:22:47+02	devel	ghidra	\N	ghidra-9.1
2020-07-09 21:22:47+02	2020-07-09 21:25:16+02	games	alienarena	\N	alienarena-2013.766_9
2020-07-09 20:32:02+02	2020-07-09 21:27:07+02	emulators	qemu	\N	qemu-4.1.1_4
2020-07-09 21:08:09+02	2020-07-09 22:15:43+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-07-09 22:15:44+02	2020-07-09 22:16:28+02	lang	julia	\N	julia-1.0.3_3
2020-07-09 22:21:10+02	2020-07-09 22:29:48+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-09 22:29:50+02	2020-07-09 22:35:56+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-07-09 22:35:57+02	2020-07-09 22:36:57+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-09 22:36:58+02	2020-07-09 22:38:00+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-09 22:38:00+02	2020-07-09 22:38:58+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-09 22:38:59+02	2020-07-09 22:39:56+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-09 22:39:57+02	2020-07-09 22:40:56+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-09 22:40:57+02	2020-07-09 22:41:55+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-07-09 22:41:56+02	2020-07-09 22:42:56+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-09 22:42:56+02	2020-07-09 22:44:32+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-09 22:44:33+02	2020-07-09 22:45:35+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-09 22:45:36+02	2020-07-09 22:46:34+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-09 22:46:34+02	2020-07-09 22:50:31+02	security	clamav	\N	clamav-0.102.3_1,1
2020-07-09 22:50:31+02	2020-07-09 22:55:46+02	security	klee	\N	klee-2.1
2020-07-09 21:25:16+02	2020-07-09 23:12:12+02	graphics	krita	\N	krita-4.3.0_1
2020-07-09 23:12:13+02	2020-07-09 23:13:26+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-07-09 23:13:27+02	2020-07-09 23:15:15+02	x11	mate	\N	mate-1.22.2
2020-07-09 23:15:16+02	2020-07-09 23:22:09+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-07-09 21:27:08+02	2020-07-10 00:07:26+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200704
2020-07-09 22:55:47+02	2020-07-10 00:54:57+02	www	firefox	\N	firefox-78.0.2,1
2020-07-10 08:13:56+02	2020-07-10 08:16:46+02	math	maxima	\N	maxima-5.44.0_2
2020-07-10 08:16:46+02	2020-07-10 08:41:40+02	local	E590T-localbase	\N	E590T-localbase-2020070402
2020-07-11 19:33:09+02	2020-07-11 19:37:10+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-11 19:37:10+02	2020-07-11 19:37:17+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-11 19:42:56+02	2020-07-11 19:44:07+02	shells	bash	\N	bash-5.0.18
2020-07-11 19:44:07+02	2020-07-11 19:44:36+02	shells	bash-completion	\N	bash-completion-2.10,2
2020-07-11 19:42:55+02	2020-07-11 19:46:46+02	lang	php73	\N	php73-7.3.20
2020-07-11 19:46:47+02	2020-07-11 19:47:18+02	www	php73-session	\N	php73-session-7.3.20
2020-07-11 19:47:18+02	2020-07-11 19:47:49+02	archivers	php73-zlib	\N	php73-zlib-7.3.20
2020-07-11 19:47:49+02	2020-07-11 19:48:23+02	databases	php73-pdo	\N	php73-pdo-7.3.20
2020-07-11 19:48:23+02	2020-07-11 19:48:56+02	devel	php73-json	\N	php73-json-7.3.20
2020-07-11 19:48:59+02	2020-07-11 19:49:56+02	converters	php73-mbstring	\N	php73-mbstring-7.3.20
2020-07-11 19:49:57+02	2020-07-11 19:50:33+02	graphics	php73-gd	\N	php73-gd-7.3.20
2020-07-11 19:50:33+02	2020-07-11 19:51:20+02	security	php73-hash	\N	php73-hash-7.3.20
2020-07-11 19:51:21+02	2020-07-11 19:51:59+02	security	php73-openssl	\N	php73-openssl-7.3.20
2020-07-11 19:52:00+02	2020-07-11 19:52:29+02	textproc	php73-xml	\N	php73-xml-7.3.20
2020-07-11 19:52:30+02	2020-07-11 19:53:00+02	textproc	xmlto	\N	xmlto-0.0.28
2020-07-11 19:53:01+02	2020-07-11 19:53:29+02	converters	php73-iconv	\N	php73-iconv-7.3.20
2020-10-05 02:12:37+02	2020-10-05 02:13:39+02	x11	yelp	\N	yelp-3.38.0
2020-07-11 19:42:56+02	2020-07-11 19:53:35+02	devel	qt5-core	\N	qt5-core-5.15.0_1
2020-07-11 19:53:30+02	2020-07-11 19:53:55+02	databases	pecl-memcache	php73	php73-pecl-memcache-4.0.5.2
2020-07-11 19:53:55+02	2020-07-11 19:56:37+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-07-11 19:56:38+02	2020-07-11 19:58:03+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-07-11 19:53:35+02	2020-07-11 19:58:57+02	net	qt5-network	\N	qt5-network-5.15.0
2020-07-11 19:58:04+02	2020-07-11 19:59:29+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-07-11 19:59:30+02	2020-07-11 20:00:27+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-07-11 19:44:37+02	2020-07-11 20:01:21+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-07-11 20:00:28+02	2020-07-11 20:01:22+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-07-11 20:01:22+02	2020-07-11 20:01:48+02	databases	pecl-memcached	php73	php73-pecl-memcached-3.1.5
2020-07-11 20:01:22+02	2020-07-11 20:02:07+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-07-11 20:01:49+02	2020-07-11 20:02:14+02	databases	php73-pdo_pgsql	\N	php73-pdo_pgsql-7.3.20
2020-07-11 20:02:08+02	2020-07-11 20:02:32+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.20
2020-07-11 20:02:15+02	2020-07-11 20:02:43+02	databases	php73-pgsql	\N	php73-pgsql-7.3.20
2020-07-11 20:02:33+02	2020-07-11 20:02:58+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.20
2020-07-11 20:02:43+02	2020-07-11 20:03:05+02	devel	php73-gettext	\N	php73-gettext-7.3.20
2020-07-11 20:03:06+02	2020-07-11 20:03:36+02	ftp	php73-curl	\N	php73-curl-7.3.20
2020-07-11 20:03:36+02	2020-07-11 20:04:43+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-07-11 20:04:44+02	2020-07-11 20:05:10+02	math	pecl-bitset	php73	php73-pecl-bitset-3.0.1
2020-07-11 20:05:10+02	2020-07-11 20:05:46+02	math	php73-bcmath	\N	php73-bcmath-7.3.20
2020-07-11 20:02:58+02	2020-07-11 20:07:22+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-07-11 20:07:22+02	2020-07-11 20:07:52+02	net	php73-soap	\N	php73-soap-7.3.20
2020-07-11 20:05:47+02	2020-07-11 20:07:56+02	net	php73-ldap	\N	php73-ldap-7.3.20
2020-07-11 20:07:54+02	2020-07-11 20:08:18+02	net	php73-sockets	\N	php73-sockets-7.3.20
2020-07-11 20:07:57+02	2020-07-11 20:08:18+02	security	pecl-mcrypt	php73	php73-pecl-mcrypt-1.0.3
2020-07-11 20:08:19+02	2020-07-11 20:08:43+02	sysutils	php73-posix	\N	php73-posix-7.3.20
2020-07-11 20:08:19+02	2020-07-11 20:08:43+02	textproc	php73-ctype	\N	php73-ctype-7.3.20
2020-07-11 20:08:44+02	2020-07-11 20:09:10+02	textproc	php73-simplexml	\N	php73-simplexml-7.3.20
2020-07-11 20:08:43+02	2020-07-11 20:09:15+02	textproc	php73-dom	\N	php73-dom-7.3.20
2020-07-11 20:09:10+02	2020-07-11 20:09:55+02	www	php73-opcache	\N	php73-opcache-7.3.20
2020-07-11 20:09:56+02	2020-07-11 20:11:10+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-11 20:11:10+02	2020-07-11 20:11:47+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-07-11 20:09:16+02	2020-07-11 20:13:37+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-07-11 20:11:48+02	2020-07-11 20:14:06+02	security	gnupg	\N	gnupg-2.2.21
2020-07-11 20:14:06+02	2020-07-11 20:14:27+02	security	gpgme	\N	gpgme-1.13.1
2020-07-11 20:13:38+02	2020-07-11 20:14:31+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-11 19:58:58+02	2020-07-11 20:14:40+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-07-11 20:14:28+02	2020-07-11 20:15:01+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-07-11 20:14:32+02	2020-07-11 20:15:16+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-07-11 20:15:02+02	2020-07-11 20:15:36+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-07-11 20:15:17+02	2020-07-11 20:16:12+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-07-11 20:15:36+02	2020-07-11 20:16:32+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-07-11 20:16:32+02	2020-07-11 20:17:38+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-07-11 20:16:13+02	2020-07-11 20:17:49+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-07-11 20:17:38+02	2020-07-11 20:18:33+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-07-11 20:17:49+02	2020-07-11 20:18:49+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-07-11 20:18:50+02	2020-07-11 20:19:42+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-07-11 20:19:43+02	2020-07-11 20:20:46+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-07-11 20:18:34+02	2020-07-11 20:20:47+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-07-11 20:20:47+02	2020-07-11 20:21:42+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-11 20:21:43+02	2020-07-11 20:22:36+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-11 20:14:41+02	2020-07-11 20:22:41+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-07-11 20:20:46+02	2020-07-11 20:23:20+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-07-11 20:23:21+02	2020-07-11 20:25:03+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-07-11 20:22:36+02	2020-07-11 20:25:29+02	x11	xorg	\N	xorg-7.7_3
2020-07-11 20:25:29+02	2020-07-11 20:26:02+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-07-11 20:25:04+02	2020-07-11 20:26:53+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-07-11 20:26:02+02	2020-07-11 20:28:01+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-07-11 20:26:53+02	2020-07-11 20:28:15+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-07-11 20:28:15+02	2020-07-11 20:29:31+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-07-11 20:29:32+02	2020-07-11 20:30:49+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-11 20:28:02+02	2020-07-11 20:32:28+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-11 20:32:28+02	2020-07-11 20:41:00+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-11 20:22:42+02	2020-07-11 20:52:57+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-07-11 20:52:58+02	2020-07-11 20:55:13+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-07-11 20:55:14+02	2020-07-11 20:56:23+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.71.0_2
2020-07-11 20:56:23+02	2020-07-11 20:58:05+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.71.0
2020-07-11 20:58:06+02	2020-07-11 20:59:55+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-07-11 20:59:56+02	2020-07-11 21:00:54+02	archivers	kf5-karchive	\N	kf5-karchive-5.71.0
2020-07-11 21:00:55+02	2020-07-11 21:02:27+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-07-11 21:02:28+02	2020-07-11 21:04:01+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-07-11 21:04:02+02	2020-07-11 21:05:44+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-07-11 20:41:01+02	2020-07-11 21:06:14+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-11 21:06:15+02	2020-07-11 21:07:27+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-07-11 21:07:28+02	2020-07-11 21:08:50+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-07-11 21:05:46+02	2020-07-11 21:09:10+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-07-11 21:08:50+02	2020-07-11 21:09:56+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.71.0
2020-07-11 21:09:11+02	2020-07-11 21:10:35+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-07-11 21:09:57+02	2020-07-11 21:11:27+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-07-11 21:10:36+02	2020-07-11 21:12:56+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-07-11 21:12:57+02	2020-07-11 21:14:18+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-07-11 21:14:19+02	2020-07-11 21:16:55+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.71.0
2020-07-11 21:16:56+02	2020-07-11 21:19:00+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-07-11 21:19:01+02	2020-07-11 21:20:27+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-07-11 21:20:28+02	2020-07-11 21:23:29+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-11 21:23:30+02	2020-07-11 21:25:09+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-07-11 21:11:27+02	2020-07-11 21:25:54+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-07-11 21:25:55+02	2020-07-11 21:27:59+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-07-11 21:25:09+02	2020-07-11 21:28:33+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-07-11 21:27:59+02	2020-07-11 21:29:13+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-07-11 21:29:13+02	2020-07-11 21:30:32+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-11 21:30:32+02	2020-07-11 21:33:21+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-11 21:28:34+02	2020-07-11 21:37:39+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-11 21:33:22+02	2020-07-11 21:43:04+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-07-11 21:43:05+02	2020-07-11 21:45:01+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-07-11 21:45:02+02	2020-07-11 21:47:45+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-07-11 21:47:46+02	2020-07-11 21:49:37+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-11 21:49:38+02	2020-07-11 21:51:52+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-11 21:51:53+02	2020-07-11 21:54:45+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-07-11 21:54:45+02	2020-07-11 21:55:49+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-11 21:55:50+02	2020-07-11 21:58:36+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-11 20:30:49+02	2020-07-11 21:59:11+02	math	vtk8	\N	vtk8-8.2.0
2020-07-11 21:58:37+02	2020-07-11 22:00:10+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-11 22:00:11+02	2020-07-11 22:03:24+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-07-11 22:03:24+02	2020-07-11 22:19:02+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-07-11 22:19:03+02	2020-07-11 22:20:32+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-11 22:20:33+02	2020-07-11 22:23:08+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-11 22:23:09+02	2020-07-11 22:24:25+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-07-11 22:24:26+02	2020-07-11 22:25:58+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-07-11 22:25:59+02	2020-07-11 22:40:01+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-11 22:40:02+02	2020-07-11 22:42:25+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-11 22:42:26+02	2020-07-11 22:53:54+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-07-11 21:59:19+02	2020-07-12 00:13:36+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-12 00:13:38+02	2020-07-12 00:34:46+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-12 00:34:47+02	2020-07-12 00:39:46+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-07-12 00:39:46+02	2020-07-12 00:46:21+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.0_1
2020-07-12 00:46:22+02	2020-07-12 00:47:28+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-11 21:37:40+02	2020-07-12 01:13:23+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-12 01:13:24+02	2020-07-12 01:14:28+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-07-12 01:14:29+02	2020-07-12 01:16:57+02	math	cryptominisat	\N	cryptominisat-5.7.1
2020-07-12 01:16:58+02	2020-07-12 01:18:13+02	net	liveMedia	\N	liveMedia-2020.07.09,2
2020-07-12 00:47:29+02	2020-07-12 01:18:37+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-07-12 01:18:38+02	2020-07-12 01:19:27+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-12 01:18:13+02	2020-07-12 01:19:29+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-12 01:19:27+02	2020-07-12 01:20:13+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-12 01:19:30+02	2020-07-12 01:20:56+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-12 01:20:57+02	2020-07-12 01:21:58+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-07-12 01:20:13+02	2020-07-12 01:22:26+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-07-12 01:22:27+02	2020-07-12 01:23:42+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0_1
2020-07-12 01:21:58+02	2020-07-12 01:27:10+02	devel	gsoap	\N	gsoap-2.8.104
2020-07-12 01:27:10+02	2020-07-12 01:30:17+02	math	stp	\N	stp-2.3.3_6,1
2020-07-12 01:30:18+02	2020-07-12 01:36:07+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-12 01:23:42+02	2020-07-12 01:48:54+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-12 01:36:08+02	2020-07-12 01:50:11+02	multimedia	vlc	\N	vlc-3.0.11_2,4
2020-07-12 01:48:55+02	2020-07-12 01:50:44+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-12 01:50:12+02	2020-07-12 01:51:34+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-12 01:50:45+02	2020-07-12 01:51:37+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-12 01:51:38+02	2020-07-12 01:52:40+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-07-12 01:51:35+02	2020-07-12 01:52:56+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-12 01:52:56+02	2020-07-12 01:53:42+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-07-12 01:52:41+02	2020-07-12 01:54:21+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-12 01:54:22+02	2020-07-12 01:59:02+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-12 01:59:03+02	2020-07-12 02:04:51+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-12 02:04:52+02	2020-07-12 02:07:52+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-12 01:53:43+02	2020-07-12 02:15:23+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-12 02:15:24+02	2020-07-12 02:17:21+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-12 02:17:22+02	2020-07-12 02:19:09+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-12 02:19:09+02	2020-07-12 03:07:13+02	devel	ghidra	\N	ghidra-9.1
2020-07-12 03:07:14+02	2020-07-12 03:09:29+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-12 02:07:53+02	2020-07-12 03:34:22+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-12 03:34:23+02	2020-07-12 03:47:23+02	graphics	krita	\N	krita-4.3.0_1
2020-07-12 03:47:25+02	2020-07-12 03:50:07+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-12 03:09:30+02	2020-07-12 04:12:16+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-07-12 04:12:17+02	2020-07-12 04:16:37+02	mail	alpine	\N	alpine-2.23.2
2020-07-12 03:50:07+02	2020-07-12 04:26:08+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.3
2020-07-12 04:26:09+02	2020-07-12 04:28:29+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-12 04:28:30+02	2020-07-12 04:31:21+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-12 04:16:38+02	2020-07-12 04:38:39+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.0
2020-07-12 04:38:40+02	2020-07-12 04:40:20+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-12 04:40:20+02	2020-07-12 04:40:53+02	net	wireguard	\N	wireguard-1.0.20200513
2020-07-12 04:31:22+02	2020-07-12 04:41:34+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-12 04:41:34+02	2020-07-12 05:03:12+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-07-12 05:03:12+02	2020-07-12 05:04:02+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-12 04:40:53+02	2020-07-12 05:04:33+02	net	wireshark	\N	wireshark-3.2.5
2020-07-12 05:04:02+02	2020-07-12 05:04:50+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-12 05:04:34+02	2020-07-12 05:05:21+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-12 05:04:50+02	2020-07-12 05:05:38+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-12 05:05:21+02	2020-07-12 05:06:08+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-12 05:05:38+02	2020-07-12 05:06:27+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-07-12 05:06:09+02	2020-07-12 05:06:56+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-12 05:06:27+02	2020-07-12 05:07:42+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-12 05:06:56+02	2020-07-12 05:07:49+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-12 05:07:42+02	2020-07-12 05:08:31+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-12 05:08:31+02	2020-07-12 05:14:22+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-07-12 05:14:23+02	2020-07-12 05:15:43+02	security	klee	\N	klee-2.1_1
2020-07-12 05:15:44+02	2020-07-12 05:17:07+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-12 05:17:08+02	2020-07-12 05:23:51+02	security	tor	\N	tor-0.4.3.6
2020-07-12 05:23:51+02	2020-07-12 05:23:59+02	sysutils	neofetch	\N	neofetch-7.0.0
2020-07-12 05:24:00+02	2020-07-12 05:24:26+02	www	dokuwiki	\N	dokuwiki-20180422c
2020-07-12 05:07:50+02	2020-07-12 05:27:31+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-12 05:24:27+02	2020-07-12 05:28:45+02	www	mod_php73	\N	mod_php73-7.3.20
2020-07-12 05:27:32+02	2020-07-12 05:29:31+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-12 05:28:45+02	2020-07-12 06:05:02+02	editors	vscode	\N	vscode-1.46.1
2020-07-11 22:53:55+02	2020-07-12 06:36:45+02	www	qt5-webengine	\N	qt5-webengine-5.15.0
2020-07-12 06:36:47+02	2020-07-12 06:38:01+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-12 06:38:01+02	2020-07-12 06:42:55+02	deskutils	calibre	\N	calibre-4.20.0_1
2020-07-12 06:36:47+02	2020-07-12 06:52:10+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-12 06:52:11+02	2020-07-12 07:06:00+02	local	E590T-localbase	\N	E590T-localbase-2020070402
2020-07-12 09:41:17+02	2020-07-12 09:42:26+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-12 09:42:27+02	2020-07-12 09:42:34+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-12 09:47:52+02	2020-07-12 09:47:59+02	misc	pciids	\N	pciids-20200624
2020-07-12 09:48:00+02	2020-07-12 09:48:17+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-07-12 09:48:19+02	2020-07-12 09:48:45+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-07-12 09:48:45+02	2020-07-12 09:49:13+02	audio	libvorbis	\N	libvorbis-1.3.7_2,3
2020-07-12 09:49:14+02	2020-07-12 09:50:02+02	audio	libsndfile	\N	libsndfile-1.0.28_2
2020-07-12 09:47:52+02	2020-07-12 09:50:06+02	x11	libX11	\N	libX11-1.6.9_2,1
2020-07-12 09:50:07+02	2020-07-12 09:50:21+02	x11	libXext	\N	libXext-1.3.4,1
2020-07-12 09:50:02+02	2020-07-12 09:50:24+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-07-12 09:50:22+02	2020-07-12 09:50:34+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-07-12 09:50:24+02	2020-07-12 09:50:35+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-07-12 09:50:35+02	2020-07-12 09:50:47+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-07-12 09:50:36+02	2020-07-12 09:50:48+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-07-12 09:50:47+02	2020-07-12 09:50:58+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-07-12 09:50:49+02	2020-07-12 09:51:19+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-07-12 09:51:19+02	2020-07-12 09:52:16+02	devel	dbus	\N	dbus-1.12.20
2020-07-12 09:52:18+02	2020-07-12 09:52:40+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-07-12 09:50:58+02	2020-07-12 09:52:54+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-07-12 09:52:41+02	2020-07-12 09:53:02+02	x11	libXi	\N	libXi-1.7.10,1
2020-07-12 09:47:52+02	2020-07-12 09:53:02+02	databases	sqlite3	\N	sqlite3-3.32.3_1,1
2020-07-12 09:53:02+02	2020-07-12 09:53:19+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-07-12 09:53:02+02	2020-07-12 09:53:30+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-07-12 09:53:19+02	2020-07-12 09:53:39+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-07-12 09:53:40+02	2020-07-12 09:53:57+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-07-12 09:53:57+02	2020-07-12 09:54:27+02	x11	xkeyboard-config	\N	xkeyboard-config-2.29_1
2020-07-12 09:53:32+02	2020-07-12 09:54:38+02	net	avahi-app	\N	avahi-app-0.7_3
2020-07-12 09:54:27+02	2020-07-12 09:54:47+02	x11	libXpm	\N	libXpm-3.5.13
2020-07-12 09:54:47+02	2020-07-12 09:55:00+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-07-12 09:52:56+02	2020-07-12 09:55:01+02	graphics	cairo	\N	cairo-1.16.0,2
2020-07-12 09:55:03+02	2020-07-12 09:55:36+02	x11	libxkbcommon	\N	libxkbcommon-0.10.0_2
2020-07-12 09:55:02+02	2020-07-12 09:55:52+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-07-12 09:54:38+02	2020-07-12 09:55:52+02	print	cups	\N	cups-2.3.3_1
2020-07-12 09:55:53+02	2020-07-12 09:56:17+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-07-12 09:56:18+02	2020-07-12 09:57:19+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-07-12 09:57:20+02	2020-07-12 09:57:33+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-07-12 09:57:33+02	2020-07-12 09:57:49+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-07-12 09:55:37+02	2020-07-12 09:57:55+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-07-12 09:57:53+02	2020-07-12 09:58:07+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-07-12 09:57:56+02	2020-07-12 09:58:21+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-07-12 09:58:21+02	2020-07-12 09:58:35+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-07-12 09:58:08+02	2020-07-12 09:58:56+02	sysutils	polkit	\N	polkit-0.116
2020-07-12 09:55:53+02	2020-07-12 09:59:01+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_6
2020-07-12 09:58:35+02	2020-07-12 09:59:28+02	devel	libgsf	\N	libgsf-1.14.47
2020-07-12 09:58:56+02	2020-07-12 09:59:33+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-07-12 09:59:33+02	2020-07-12 09:59:44+02	x11	xprop	\N	xprop-1.2.4
2020-07-12 09:59:45+02	2020-07-12 09:59:57+02	x11	xset	\N	xset-1.2.4_3
2020-07-12 09:59:58+02	2020-07-12 10:00:33+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-07-12 10:00:33+02	2020-07-12 10:02:01+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-07-12 09:59:28+02	2020-07-12 10:02:07+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-07-12 10:02:02+02	2020-07-12 10:02:18+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-07-12 10:02:19+02	2020-07-12 10:02:56+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-07-12 09:59:02+02	2020-07-12 10:02:58+02	graphics	graphviz	\N	graphviz-2.44.0_5
2020-07-12 10:02:08+02	2020-07-12 10:04:00+02	graphics	colord	\N	colord-1.3.5
2020-07-12 10:04:01+02	2020-07-12 10:04:44+02	graphics	libGLU	\N	libGLU-9.0.1
2020-07-12 10:02:57+02	2020-07-12 10:04:54+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-07-12 10:02:59+02	2020-07-12 10:06:18+02	lang	vala	\N	vala-0.40.15,1
2020-07-12 10:04:55+02	2020-07-12 10:06:48+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-07-12 10:04:44+02	2020-07-12 10:07:05+02	security	nss	\N	nss-3.54
2020-07-12 10:06:18+02	2020-07-12 10:07:16+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-07-12 10:07:06+02	2020-07-12 10:09:25+02	graphics	poppler	\N	poppler-0.90.0_1
2020-07-12 10:09:26+02	2020-07-12 10:12:48+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-07-12 10:12:49+02	2020-07-12 10:13:23+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-07-12 10:13:24+02	2020-07-12 10:14:20+02	devel	dconf	\N	dconf-0.36.0
2020-07-12 10:06:48+02	2020-07-12 10:14:45+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-07-12 10:14:21+02	2020-07-12 10:15:44+02	devel	libsoup	\N	libsoup-2.62.3
2020-07-12 10:15:45+02	2020-07-12 10:16:31+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-07-12 10:16:32+02	2020-07-12 10:16:48+02	print	xpdfopen	\N	xpdfopen-0.86
2020-07-12 10:14:46+02	2020-07-12 10:16:58+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-07-12 10:16:49+02	2020-07-12 10:17:05+02	devel	libnotify	\N	libnotify-0.7.8
2020-07-12 10:07:17+02	2020-07-12 10:17:19+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-07-12 10:17:19+02	2020-07-12 10:17:38+02	multimedia	libva	\N	libva-2.8.0
2020-07-12 10:17:39+02	2020-07-12 10:17:53+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-07-12 10:17:54+02	2020-07-12 10:18:28+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-07-12 10:16:58+02	2020-07-12 10:18:33+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-07-12 10:18:34+02	2020-07-12 10:19:05+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-07-12 10:18:28+02	2020-07-12 10:19:12+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-07-12 10:19:05+02	2020-07-12 10:19:25+02	multimedia	libtheora	\N	libtheora-1.1.1_7
2020-07-12 10:19:12+02	2020-07-12 10:19:27+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-07-12 10:19:26+02	2020-07-12 10:20:10+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-07-12 10:20:11+02	2020-07-12 10:20:40+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-07-12 10:19:27+02	2020-07-12 10:21:01+02	graphics	frei0r	\N	frei0r-1.6.1
2020-07-12 10:20:41+02	2020-07-12 10:22:00+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-07-12 10:22:01+02	2020-07-12 10:22:28+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-07-12 10:17:05+02	2020-07-12 10:23:17+02	print	texlive-base	\N	texlive-base-20150521_53
2020-07-12 10:22:29+02	2020-07-12 10:23:40+02	graphics	graphene	\N	graphene-1.10.0
2020-07-12 10:23:40+02	2020-07-12 10:23:59+02	www	libwpe	\N	libwpe-1.4.0.1
2020-07-12 10:24:00+02	2020-07-12 10:26:44+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-07-12 10:26:45+02	2020-07-12 10:28:01+02	net	geoclue	\N	geoclue-2.5.5
2020-07-12 10:28:02+02	2020-07-12 10:28:25+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-07-12 10:28:25+02	2020-07-12 10:29:33+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-07-12 10:21:02+02	2020-07-12 10:37:52+02	multimedia	ffmpeg	\N	ffmpeg-4.3_4,1
2020-07-12 10:37:52+02	2020-07-12 10:38:36+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-07-12 10:38:36+02	2020-07-12 10:39:17+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-07-12 10:39:17+02	2020-07-12 10:39:50+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-07-12 10:39:51+02	2020-07-12 10:40:23+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-07-12 10:40:23+02	2020-07-12 10:41:03+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-07-12 10:41:03+02	2020-07-12 10:41:57+02	devel	sdl12	\N	sdl-1.2.15_14,2
2020-07-12 10:23:18+02	2020-07-12 10:42:53+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-07-12 10:42:54+02	2020-07-12 10:46:14+02	print	tex-formats	\N	tex-formats-20150521_2
2020-07-12 10:41:57+02	2020-07-12 10:49:24+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-07-12 10:49:24+02	2020-07-12 10:49:38+02	x11	libXres	\N	libXres-1.2.0_2
2020-07-12 10:49:38+02	2020-07-12 10:49:53+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-07-12 10:49:54+02	2020-07-12 10:50:08+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-07-12 10:50:09+02	2020-07-12 10:51:05+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-07-12 10:51:05+02	2020-07-12 10:52:11+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-07-12 10:52:12+02	2020-07-12 10:52:30+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-07-12 10:46:15+02	2020-07-12 10:52:53+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-07-12 10:52:30+02	2020-07-12 10:54:48+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-07-12 10:54:49+02	2020-07-12 10:55:04+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-07-12 10:55:04+02	2020-07-12 10:55:49+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-07-12 10:55:50+02	2020-07-12 10:56:44+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-07-12 10:56:45+02	2020-07-12 10:57:09+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.20
2020-07-12 10:57:10+02	2020-07-12 10:57:32+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.20
2020-07-12 10:57:33+02	2020-07-12 10:58:23+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-07-12 10:58:24+02	2020-07-12 10:59:50+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-07-12 10:59:50+02	2020-07-12 11:00:43+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-07-12 11:00:43+02	2020-07-12 11:01:33+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-07-12 11:01:34+02	2020-07-12 11:02:41+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-12 11:02:41+02	2020-07-12 11:03:29+02	sysutils	policykit	\N	policykit-0.9_10
2020-07-12 11:03:30+02	2020-07-12 11:04:11+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-07-12 11:04:12+02	2020-07-12 11:05:17+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-07-12 11:05:20+02	2020-07-12 11:06:20+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-12 10:52:54+02	2020-07-12 11:06:35+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-07-12 11:06:20+02	2020-07-12 11:06:59+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-07-12 11:06:36+02	2020-07-12 11:07:10+02	security	libssh2	\N	libssh2-1.9.0,3
2020-07-12 11:06:59+02	2020-07-12 11:07:38+02	www	py-django22	py37	py37-django22-2.2.14
2020-07-12 11:07:39+02	2020-07-12 11:08:50+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-07-12 11:07:11+02	2020-07-12 11:08:50+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-07-12 11:08:51+02	2020-07-12 11:09:35+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-07-12 11:08:50+02	2020-07-12 11:09:45+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-12 11:09:46+02	2020-07-12 11:10:59+02	sysutils	hal	\N	hal-0.5.14_34
2020-07-12 11:11:00+02	2020-07-12 11:12:01+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-07-12 11:09:35+02	2020-07-12 11:12:07+02	devel	sdl20	\N	sdl2-2.0.12
2020-07-12 11:12:07+02	2020-07-12 11:13:05+02	devel	libgit2	\N	libgit2-1.0.1
2020-07-12 11:13:06+02	2020-07-12 11:13:13+02	devel	libpci	\N	libpci-3.7.0
2020-07-12 11:12:02+02	2020-07-12 11:14:07+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-07-12 11:14:08+02	2020-07-12 11:15:31+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-07-12 11:13:14+02	2020-07-12 11:16:00+02	graphics	poppler-glib	\N	poppler-glib-0.90.0_1
2020-07-12 11:16:00+02	2020-07-12 11:16:54+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-07-12 11:16:54+02	2020-07-12 11:17:47+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-07-12 11:15:32+02	2020-07-12 11:17:48+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-07-12 11:17:48+02	2020-07-12 11:18:39+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-07-12 11:17:48+02	2020-07-12 11:20:59+02	graphics	glew	\N	glew-2.2.0
2020-07-12 11:21:00+02	2020-07-12 11:21:50+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-07-12 11:21:51+02	2020-07-12 11:23:09+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-07-12 11:23:09+02	2020-07-12 11:24:07+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-12 11:24:08+02	2020-07-12 11:25:04+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-07-12 11:25:05+02	2020-07-12 11:25:22+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-07-12 11:25:23+02	2020-07-12 11:26:26+02	devel	hwloc	\N	hwloc-1.11.11
2020-07-12 11:26:27+02	2020-07-12 11:27:32+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-07-12 11:27:32+02	2020-07-12 11:29:04+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-07-12 11:29:05+02	2020-07-12 11:31:24+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-07-12 11:31:25+02	2020-07-12 11:31:48+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-07-12 11:31:49+02	2020-07-12 11:33:19+02	net	remmina	\N	remmina-1.3.9
2020-07-12 11:33:20+02	2020-07-12 11:36:46+02	security	gcr	\N	gcr-3.28.0
2020-07-12 11:36:47+02	2020-07-12 11:37:35+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-07-12 11:37:35+02	2020-07-12 11:38:18+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-07-12 11:18:39+02	2020-07-12 11:38:24+02	java	openjdk8	\N	openjdk8-8.252.09.1_1
2020-07-12 11:38:19+02	2020-07-12 11:38:52+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-07-12 11:38:52+02	2020-07-12 11:46:19+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-07-12 11:46:20+02	2020-07-12 11:46:37+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-07-12 11:46:38+02	2020-07-12 11:46:55+02	x11	xrdb	\N	xrdb-1.2.0
2020-07-12 11:46:55+02	2020-07-12 11:48:14+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-07-12 11:48:14+02	2020-07-12 11:49:14+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-07-12 11:49:15+02	2020-07-12 11:50:57+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-07-12 11:50:58+02	2020-07-12 11:54:29+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-12 11:54:30+02	2020-07-12 11:55:18+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-07-12 11:55:19+02	2020-07-12 11:56:14+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-07-12 11:56:14+02	2020-07-12 11:57:14+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-12 16:32:43+02	2020-07-12 16:38:41+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-07-12 11:57:15+02	2020-07-12 11:57:59+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-07-12 11:58:00+02	2020-07-12 12:24:23+02	net	openmpi	\N	openmpi-4.0.4
2020-07-12 12:24:24+02	2020-07-12 12:25:28+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-12 12:25:29+02	2020-07-12 12:26:35+02	security	gnupg	\N	gnupg-2.2.21
2020-07-12 12:26:36+02	2020-07-12 12:27:55+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-07-12 12:27:56+02	2020-07-12 12:30:48+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-07-12 12:30:49+02	2020-07-12 12:31:53+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-07-12 12:31:54+02	2020-07-12 12:33:06+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-07-12 10:29:34+02	2020-07-12 12:39:23+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-07-12 12:33:06+02	2020-07-12 12:39:51+02	math	vtk8	\N	vtk8-8.2.0
2020-07-12 12:39:52+02	2020-07-12 12:40:27+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-12 12:40:28+02	2020-07-12 12:45:06+02	net	libmateweather	\N	libmateweather-1.24.0
2020-07-12 12:45:07+02	2020-07-12 12:45:31+02	print	gl2ps	\N	gl2ps-1.4.0
2020-07-12 12:45:32+02	2020-07-12 12:45:55+02	security	gpgme	\N	gpgme-1.13.1
2020-07-12 12:45:56+02	2020-07-12 12:47:03+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-07-12 12:47:04+02	2020-07-12 12:47:21+02	x11	luit	\N	luit-20190106_1
2020-07-12 12:47:21+02	2020-07-12 12:48:40+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-07-12 12:48:41+02	2020-07-12 12:49:43+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-07-12 12:49:44+02	2020-07-12 12:51:57+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-07-12 12:51:57+02	2020-07-12 12:54:09+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-12 12:54:10+02	2020-07-12 12:54:32+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.8_6
2020-07-12 12:54:33+02	2020-07-12 12:55:10+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-07-12 12:55:10+02	2020-07-12 13:03:27+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-12 13:03:28+02	2020-07-12 13:03:48+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-07-12 13:03:48+02	2020-07-12 13:04:51+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-12 13:04:51+02	2020-07-12 13:06:09+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-07-12 13:06:10+02	2020-07-12 13:10:43+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-07-12 13:10:44+02	2020-07-12 13:11:56+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-12 12:39:23+02	2020-07-12 13:12:17+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-07-12 13:11:57+02	2020-07-12 13:13:51+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-07-12 13:12:18+02	2020-07-12 13:15:38+02	math	gnuplot	\N	gnuplot-5.2.8_2
2020-07-12 13:15:39+02	2020-07-12 13:16:06+02	security	gpgme-cpp	\N	gpgme-cpp-1.13.1
2020-07-12 13:16:06+02	2020-07-12 13:17:19+02	sysutils	upower	\N	upower-0.99.11
2020-07-12 13:13:51+02	2020-07-12 13:18:55+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-07-12 13:18:56+02	2020-07-12 13:20:21+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-07-12 13:20:21+02	2020-07-12 13:21:02+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-07-12 13:17:20+02	2020-07-12 13:25:23+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-12 13:21:03+02	2020-07-12 13:25:28+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-07-12 13:25:24+02	2020-07-12 13:25:37+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-07-12 13:25:28+02	2020-07-12 13:26:59+02	x11	xterm	\N	xterm-357
2020-07-12 13:25:37+02	2020-07-12 13:27:24+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-07-12 13:27:27+02	2020-07-12 13:28:08+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-07-12 13:27:00+02	2020-07-12 13:28:51+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-12 13:28:51+02	2020-07-12 13:35:21+02	audio	jack	\N	jackit-0.125.0_9
2020-07-12 13:35:23+02	2020-07-12 13:41:08+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-07-12 13:41:12+02	2020-07-12 13:46:12+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-12 13:46:15+02	2020-07-12 13:50:05+02	devel	librest	\N	rest-0.7.93
2020-07-12 13:50:07+02	2020-07-12 13:54:11+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-07-12 13:54:13+02	2020-07-12 14:01:46+02	graphics	poppler-utils	\N	poppler-utils-0.90.0_1
2020-07-12 14:01:49+02	2020-07-12 14:03:37+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-07-12 14:03:39+02	2020-07-12 14:05:37+02	print	libotf	\N	libotf-0.9.16
2020-07-12 14:05:39+02	2020-07-12 14:09:19+02	security	gpgme-qt5	\N	gpgme-qt5-1.13.1
2020-07-12 14:09:21+02	2020-07-12 14:12:16+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-07-12 14:12:16+02	2020-07-12 14:13:04+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-07-12 13:28:08+02	2020-07-12 14:13:07+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-07-12 14:13:05+02	2020-07-12 14:13:50+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-12 14:13:51+02	2020-07-12 14:16:15+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-07-12 14:16:16+02	2020-07-12 14:16:30+02	x11	appres	\N	appres-1.0.5
2020-07-12 14:16:31+02	2020-07-12 14:16:49+02	x11	bitmap	\N	bitmap-1.0.9
2020-07-12 14:16:49+02	2020-07-12 14:17:12+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-07-12 14:17:12+02	2020-07-12 14:18:42+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-07-12 14:18:42+02	2020-07-12 14:18:58+02	x11	sessreg	\N	sessreg-1.1.2
2020-07-12 14:18:58+02	2020-07-12 14:19:14+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-07-12 14:19:14+02	2020-07-12 14:19:30+02	x11	smproxy	\N	smproxy-1.0.6
2020-07-12 14:19:30+02	2020-07-12 14:19:51+02	x11	x11perf	\N	x11perf-1.6.1
2020-07-12 14:19:51+02	2020-07-12 14:20:08+02	x11	xauth	\N	xauth-1.1
2020-07-12 14:20:09+02	2020-07-12 14:20:24+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-07-12 14:20:24+02	2020-07-12 14:20:51+02	x11	xcalc	\N	xcalc-1.1.0
2020-07-12 14:20:52+02	2020-07-12 14:21:07+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-07-12 14:21:08+02	2020-07-12 14:21:23+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-07-12 14:21:24+02	2020-07-12 14:21:39+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-07-12 14:21:40+02	2020-07-12 14:21:53+02	x11	xev	\N	xev-1.2.3
2020-07-12 14:21:53+02	2020-07-12 14:22:06+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-07-12 14:22:06+02	2020-07-12 14:22:19+02	x11	xgamma	\N	xgamma-1.0.6
2020-07-12 14:22:19+02	2020-07-12 14:22:39+02	x11	xgc	\N	xgc-1.0.5
2020-07-12 14:22:40+02	2020-07-12 14:22:56+02	x11	xhost	\N	xhost-1.0.8
2020-07-12 14:22:56+02	2020-07-12 14:23:12+02	x11	xinit	\N	xinit-1.4.1,1
2020-07-12 14:23:12+02	2020-07-12 14:23:30+02	x11	xinput	\N	xinput-1.6.3
2020-07-12 14:23:30+02	2020-07-12 14:23:48+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-07-12 14:23:49+02	2020-07-12 14:24:07+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-07-12 14:24:07+02	2020-07-12 14:24:22+02	x11	xkill	\N	xkill-1.0.5
2020-07-12 14:24:23+02	2020-07-12 14:24:39+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-07-12 14:24:40+02	2020-07-12 14:24:55+02	x11	xmessage	\N	xmessage-1.0.5
2020-07-12 14:24:56+02	2020-07-12 14:25:10+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-07-12 14:25:11+02	2020-07-12 14:25:26+02	x11	xpr	\N	xpr-1.0.5
2020-07-12 14:25:27+02	2020-07-12 14:25:43+02	x11	xrandr	\N	xrandr-1.5.1
2020-07-12 14:25:43+02	2020-07-12 14:25:58+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-07-12 14:25:58+02	2020-07-12 14:26:13+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-07-12 14:26:14+02	2020-07-12 14:26:29+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-07-12 14:26:29+02	2020-07-12 14:26:44+02	x11	xwd	\N	xwd-1.0.7
2020-07-12 14:26:45+02	2020-07-12 14:27:02+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-07-12 14:27:02+02	2020-07-12 14:27:19+02	x11	xwud	\N	xwud-1.0.5
2020-07-12 14:27:20+02	2020-07-12 14:27:38+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-07-12 14:27:39+02	2020-07-12 14:28:52+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-12 14:28:52+02	2020-07-12 14:29:46+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-07-12 14:13:08+02	2020-07-12 14:30:04+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-12 14:29:47+02	2020-07-12 14:30:41+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-07-12 14:30:05+02	2020-07-12 14:30:57+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-07-12 14:30:42+02	2020-07-12 14:31:38+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-07-12 14:30:58+02	2020-07-12 14:32:22+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-07-12 14:31:38+02	2020-07-12 14:32:34+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-07-12 14:32:23+02	2020-07-12 14:33:13+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-07-12 14:33:14+02	2020-07-12 14:34:41+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-12 14:34:42+02	2020-07-12 14:35:47+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-07-12 14:32:34+02	2020-07-12 14:35:51+02	x11-fm	caja	\N	caja-1.22.3_1
2020-07-12 14:35:52+02	2020-07-12 14:36:10+02	x11-wm	twm	\N	twm-1.0.11
2020-07-12 14:35:48+02	2020-07-12 14:36:59+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-07-12 14:36:10+02	2020-07-12 14:37:18+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-12 14:37:00+02	2020-07-12 14:38:02+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-07-12 14:37:19+02	2020-07-12 14:38:09+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-07-12 14:38:09+02	2020-07-12 14:39:48+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-07-12 14:38:03+02	2020-07-12 14:40:04+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-07-12 14:40:04+02	2020-07-12 14:40:42+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-07-12 14:40:42+02	2020-07-12 14:41:12+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-07-12 14:39:49+02	2020-07-12 14:43:01+02	graphics	gegl	\N	gegl-0.4.24_1
2020-07-12 14:43:02+02	2020-07-12 14:43:09+02	misc	usbids	\N	usbids-20200622
2020-07-12 14:43:09+02	2020-07-12 14:45:35+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-07-12 14:45:36+02	2020-07-12 14:45:50+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-07-12 14:45:51+02	2020-07-12 14:47:06+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-07-12 14:47:07+02	2020-07-12 14:48:00+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-12 14:48:00+02	2020-07-12 14:50:49+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-07-12 14:50:49+02	2020-07-12 14:51:31+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-07-12 14:41:12+02	2020-07-12 14:52:43+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-07-12 14:51:32+02	2020-07-12 14:53:02+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-12 14:52:43+02	2020-07-12 14:53:48+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-07-12 14:53:03+02	2020-07-12 14:54:14+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-07-12 14:53:48+02	2020-07-12 14:54:21+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-07-12 14:54:15+02	2020-07-12 14:54:25+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-07-12 14:54:25+02	2020-07-12 14:55:14+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-12 14:55:14+02	2020-07-12 14:55:21+02	x11-fonts	gnu-unifont	\N	gnu-unifont-13.0.03,1
2020-07-12 14:54:21+02	2020-07-12 14:55:23+02	x11	zenity	\N	zenity-3.32.0
2020-07-12 14:55:22+02	2020-07-12 14:56:42+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-07-12 14:56:43+02	2020-07-12 14:57:46+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-07-12 14:57:47+02	2020-07-12 14:59:52+02	audio	sox	\N	sox-14.4.2_3
2020-07-12 14:59:53+02	2020-07-12 15:00:41+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-07-12 15:00:42+02	2020-07-12 15:04:28+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-12 14:55:24+02	2020-07-12 15:04:35+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-07-12 11:38:25+02	2020-07-12 15:05:42+02	lang	rust	\N	rust-1.44.1_1
2020-07-12 15:04:36+02	2020-07-12 15:05:44+02	devel	libpeas	\N	libpeas-1.22.0
2020-07-12 15:04:28+02	2020-07-12 15:06:00+02	devel	libgdata	\N	libgdata-0.17.9
2020-07-12 15:05:44+02	2020-07-12 15:06:15+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-07-12 15:06:00+02	2020-07-12 15:06:19+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-07-12 15:06:20+02	2020-07-12 15:07:10+02	graphics	glfw	\N	glfw-3.3.2
2020-07-12 15:07:11+02	2020-07-12 15:10:22+02	graphics	movit	\N	movit-1.6.2_2
2020-07-12 15:05:45+02	2020-07-12 15:11:46+02	editors	emacs	full	emacs-26.3_5,3
2020-07-12 15:11:46+02	2020-07-12 15:12:51+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-07-12 15:12:52+02	2020-07-12 15:13:38+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-07-12 15:13:39+02	2020-07-12 15:14:16+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-07-12 15:14:16+02	2020-07-12 15:15:24+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-12 15:15:25+02	2020-07-12 15:15:55+02	net	gssdp	\N	gssdp-1.2.3
2020-07-12 15:15:55+02	2020-07-12 15:16:58+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-12 15:16:59+02	2020-07-12 15:18:53+02	print	cups-filters	\N	cups-filters-1.27.5_1
2020-07-12 15:06:16+02	2020-07-12 15:24:31+02	graphics	gimp-app	\N	gimp-app-2.10.20_1,1
2020-07-12 15:18:53+02	2020-07-12 15:29:01+02	print	fontforge	\N	fontforge-20190801_2
2020-07-12 15:29:02+02	2020-07-12 15:29:58+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-07-12 15:29:58+02	2020-07-12 15:31:54+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-07-12 15:31:55+02	2020-07-12 15:32:54+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-07-12 15:24:32+02	2020-07-12 15:43:43+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-07-12 15:10:22+02	2020-07-12 15:44:31+02	graphics	opencv	\N	opencv-3.4.1_34
2020-07-12 15:44:32+02	2020-07-12 15:45:56+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-07-12 15:45:57+02	2020-07-12 15:49:37+02	x11	xorg	\N	xorg-7.7_3
2020-07-12 15:49:38+02	2020-07-12 15:51:48+02	x11	yelp	\N	yelp-3.28.1_2
2020-07-12 15:51:48+02	2020-07-12 15:53:18+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-07-12 15:53:18+02	2020-07-12 15:54:21+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-07-12 15:54:22+02	2020-07-12 15:56:45+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-07-12 15:43:43+02	2020-07-12 16:01:23+02	www	qt5-webengine	\N	qt5-webengine-5.15.0
2020-07-12 16:01:25+02	2020-07-12 16:02:34+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-07-12 15:56:46+02	2020-07-12 16:08:13+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-07-12 16:08:14+02	2020-07-12 16:09:40+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-07-12 16:09:41+02	2020-07-12 16:11:09+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-07-12 16:02:34+02	2020-07-12 16:14:29+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-07-12 16:11:10+02	2020-07-12 16:14:30+02	x11-wm	marco	\N	marco-1.22.4
2020-07-12 16:14:31+02	2020-07-12 16:14:50+02	audio	twolame	\N	twolame-0.4.0
2020-07-12 16:14:29+02	2020-07-12 16:15:08+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-07-12 16:15:09+02	2020-07-12 16:16:26+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-07-12 16:14:51+02	2020-07-12 16:17:12+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-07-12 16:17:13+02	2020-07-12 16:19:51+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-12 16:16:27+02	2020-07-12 16:20:59+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-07-12 16:21:00+02	2020-07-12 16:23:17+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-07-12 16:23:18+02	2020-07-12 16:23:47+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-07-12 16:23:48+02	2020-07-12 16:26:15+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-12 16:26:16+02	2020-07-12 16:27:48+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-07-12 16:27:48+02	2020-07-12 16:28:21+02	graphics	libgxps	\N	libgxps-0.3.1
2020-07-12 16:28:23+02	2020-07-12 16:28:52+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-07-12 16:19:52+02	2020-07-12 16:28:53+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-07-12 16:28:53+02	2020-07-12 16:29:27+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-07-12 16:28:53+02	2020-07-12 16:30:05+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.0_1
2020-07-12 16:29:27+02	2020-07-12 16:30:19+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-12 16:30:19+02	2020-07-12 16:31:09+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-07-12 16:30:06+02	2020-07-12 16:32:42+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-07-14 01:25:06+02	2020-07-14 02:38:55+02	math	vtk8	\N	vtk8-8.2.0
2020-07-12 16:38:42+02	2020-07-12 16:47:19+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-07-12 16:47:19+02	2020-07-12 16:48:29+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-07-12 16:31:10+02	2020-07-12 16:50:24+02	math	R	\N	R-4.0.2
2020-07-12 16:48:30+02	2020-07-12 16:51:03+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-12 16:51:48+02	2020-07-12 16:52:52+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-07-12 16:50:25+02	2020-07-12 16:53:38+02	net	freerdp	\N	freerdp-2.1.1
2020-07-12 16:52:52+02	2020-07-12 16:53:56+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-12 16:53:57+02	2020-07-12 16:54:32+02	print	libspectre	\N	libspectre-0.2.9
2020-07-12 16:54:33+02	2020-07-12 16:57:01+02	print	mftrace	\N	mftrace-1.2.18_2
2020-07-12 16:53:39+02	2020-07-12 16:58:17+02	print	gutenprint	\N	gutenprint-5.3.3
2020-07-12 16:57:03+02	2020-07-12 17:04:28+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-07-12 16:58:18+02	2020-07-12 17:04:54+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-07-12 17:04:55+02	2020-07-12 17:05:21+02	sysutils	hal-info	\N	hal-info-20091130
2020-07-12 17:04:30+02	2020-07-12 17:05:38+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-07-12 17:05:38+02	2020-07-12 17:06:37+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-07-12 17:06:38+02	2020-07-12 17:09:04+02	textproc	dblatex	\N	dblatex-0.3.11
2020-07-12 17:05:22+02	2020-07-12 17:09:18+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-07-12 17:09:05+02	2020-07-12 17:09:50+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-12 17:09:51+02	2020-07-12 17:10:31+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-12 17:09:19+02	2020-07-12 17:10:33+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-12 17:10:31+02	2020-07-12 17:10:42+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-07-12 17:10:33+02	2020-07-12 17:10:45+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-07-12 17:10:46+02	2020-07-12 17:11:58+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-07-12 17:11:58+02	2020-07-12 17:12:42+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-07-12 17:10:43+02	2020-07-12 17:12:43+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-07-12 17:12:42+02	2020-07-12 17:13:32+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-07-12 17:12:43+02	2020-07-12 17:14:58+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-07-12 17:13:33+02	2020-07-12 17:16:10+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-07-12 17:16:11+02	2020-07-12 17:19:27+02	archivers	engrampa	\N	engrampa-1.24.0
2020-07-12 17:19:28+02	2020-07-12 17:19:54+02	audio	dssi	\N	dssi-1.1.1_8
2020-07-12 17:19:54+02	2020-07-12 17:20:51+02	audio	faudio	\N	FAudio-20.07
2020-07-12 17:20:51+02	2020-07-12 17:22:24+02	audio	mate-media	\N	mate-media-1.24.0
2020-07-12 17:22:25+02	2020-07-12 17:22:48+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-07-12 17:22:49+02	2020-07-12 17:24:43+02	databases	rrdtool	\N	rrdtool-1.7.2_3
2020-07-12 17:24:44+02	2020-07-12 17:26:41+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-07-12 17:26:41+02	2020-07-12 17:31:53+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-07-12 17:31:54+02	2020-07-12 17:33:04+02	deskutils	mozo	\N	mozo-1.24.0
2020-07-12 17:33:05+02	2020-07-12 17:34:08+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-07-12 17:34:08+02	2020-07-12 17:34:39+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-07-12 17:34:40+02	2020-07-12 17:35:05+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-07-12 17:35:05+02	2020-07-12 17:36:44+02	devel	gradle5	\N	gradle5-5.0
2020-07-12 17:36:45+02	2020-07-12 17:37:49+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0_1
2020-07-12 17:37:49+02	2020-07-12 17:40:00+02	devel	ode	\N	ode-0.13_4,1
2020-07-12 17:40:02+02	2020-07-12 17:40:38+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-07-12 17:40:39+02	2020-07-12 17:42:46+02	devel	py-game	py37	py37-game-1.9.6_1
2020-07-12 17:14:59+02	2020-07-12 17:47:06+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-07-12 17:42:47+02	2020-07-12 17:49:57+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3
2020-07-12 17:47:07+02	2020-07-12 18:15:55+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-07-12 18:15:58+02	2020-07-12 18:20:27+02	editors	pluma	\N	pluma-1.24.0
2020-07-12 18:20:28+02	2020-07-12 18:23:25+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-07-12 17:49:57+02	2020-07-12 18:25:38+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-12 18:23:26+02	2020-07-12 18:27:04+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-07-12 18:25:40+02	2020-07-12 18:29:44+02	graphics	atril	\N	atril-1.22.3_7
2020-07-12 18:27:04+02	2020-07-12 18:29:55+02	graphics	eom	\N	eom-1.24.0
2020-07-12 18:29:55+02	2020-07-12 18:30:20+02	graphics	gle	\N	gle-3.1.0_7
2020-07-12 18:30:20+02	2020-07-12 18:30:44+02	graphics	gltt	\N	gltt-2.5.2_14
2020-07-12 18:29:45+02	2020-07-12 18:31:30+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-12 18:30:44+02	2020-07-12 18:32:25+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-07-12 18:32:25+02	2020-07-12 18:34:14+02	graphics	py-gimp	\N	py27-gimp-2.10.20_1
2020-07-12 18:34:15+02	2020-07-12 18:35:13+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-07-12 18:31:31+02	2020-07-12 18:36:00+02	graphics	opensubdiv	\N	opensubdiv-3.4.3
2020-07-12 18:35:14+02	2020-07-12 19:03:23+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-07-12 18:36:01+02	2020-07-12 19:05:45+02	java	openjdk11	\N	openjdk11-11.0.7+10.2_3
2020-07-12 19:03:24+02	2020-07-12 19:16:10+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-07-12 19:16:11+02	2020-07-12 19:19:45+02	math	mate-calc	\N	mate-calc-1.24.0
2020-07-12 19:19:45+02	2020-07-12 19:21:54+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-07-12 19:21:55+02	2020-07-12 19:49:51+02	misc	openvdb	\N	openvdb-7.0.0_2
2020-07-12 19:49:54+02	2020-07-12 19:50:19+02	misc	xosd	\N	xosd-2.2.14_3
2020-07-12 19:50:19+02	2020-07-12 19:56:24+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-12 19:56:25+02	2020-07-12 19:57:39+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-07-12 19:57:39+02	2020-07-12 20:01:32+02	multimedia	mpv	\N	mpv-0.32.0_3,1
2020-07-12 20:01:33+02	2020-07-12 20:10:31+02	multimedia	vlc	\N	vlc-3.0.11_2,4
2020-07-12 20:10:32+02	2020-07-12 20:11:54+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-12 20:11:54+02	2020-07-12 20:12:45+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-07-12 20:12:45+02	2020-07-12 20:13:44+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-07-12 20:13:44+02	2020-07-12 20:14:36+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-07-12 20:14:36+02	2020-07-12 20:15:31+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-07-12 20:15:31+02	2020-07-12 20:16:21+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-07-12 20:16:21+02	2020-07-12 20:17:13+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-07-12 20:17:14+02	2020-07-12 20:18:20+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-07-12 20:18:21+02	2020-07-12 20:19:58+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-12 20:19:59+02	2020-07-12 20:20:52+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-12 20:20:52+02	2020-07-12 20:21:10+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-07-12 20:21:10+02	2020-07-12 20:22:29+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-12 19:05:47+02	2020-07-12 20:23:12+02	lang	julia11	\N	julia11-1.1.1_5
2020-07-12 20:23:13+02	2020-07-12 20:33:15+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-07-12 20:22:29+02	2020-07-12 20:39:10+02	print	lilypond	\N	lilypond-2.20.0
2020-07-12 20:33:17+02	2020-07-12 20:41:06+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-07-12 20:39:13+02	2020-07-12 20:41:39+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-07-12 20:41:07+02	2020-07-12 20:51:43+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-07-12 20:51:44+02	2020-07-12 20:52:24+02	print	transfig	\N	transfig-3.2.5e_4
2020-07-12 20:52:25+02	2020-07-12 20:54:27+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-07-12 20:54:27+02	2020-07-12 20:57:11+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-07-12 20:57:12+02	2020-07-12 20:59:11+02	textproc	fop	\N	fop-2.3
2020-07-12 20:59:11+02	2020-07-12 21:00:21+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-07-12 21:00:21+02	2020-07-12 21:01:27+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-10-05 02:49:22+02	2020-10-05 02:49:44+02	net	gupnp	\N	gupnp-1.2.4
2020-07-12 21:01:27+02	2020-07-12 21:01:40+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-07-12 21:01:41+02	2020-07-12 21:01:57+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-07-12 21:01:57+02	2020-07-12 21:02:19+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-07-12 21:02:19+02	2020-07-12 21:03:30+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-12 20:41:40+02	2020-07-12 21:04:36+02	print	texlive-docs	\N	texlive-docs-20150523
2020-07-12 21:03:31+02	2020-07-12 21:05:07+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-12 21:05:08+02	2020-07-12 21:06:30+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-12 21:06:30+02	2020-07-12 21:08:20+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-07-12 21:08:20+02	2020-07-12 21:10:43+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-07-12 21:04:37+02	2020-07-12 21:11:19+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-07-12 21:10:43+02	2020-07-12 21:12:24+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-07-12 21:11:19+02	2020-07-12 21:12:45+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-07-12 21:12:25+02	2020-07-12 21:13:10+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-07-12 21:12:46+02	2020-07-12 21:23:39+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-07-12 21:23:39+02	2020-07-12 21:23:53+02	astro	xearth	\N	xearth-1.2
2020-07-12 21:23:53+02	2020-07-12 21:27:21+02	astro	xephem	\N	xephem-3.7.7_5
2020-07-12 21:13:10+02	2020-07-12 21:36:09+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-12 21:36:10+02	2020-07-12 21:36:36+02	audio	xmix	\N	xmix-2.1_3
2020-07-12 21:36:36+02	2020-07-12 21:43:15+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-12 21:43:16+02	2020-07-12 21:46:10+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-12 21:27:23+02	2020-07-12 21:46:28+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-12 21:46:29+02	2020-07-12 21:48:58+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-12 21:48:59+02	2020-07-12 21:51:20+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-12 21:51:21+02	2020-07-12 21:51:33+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-07-12 21:51:34+02	2020-07-12 22:17:31+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-07-12 22:17:32+02	2020-07-12 22:20:01+02	deskutils	calibre	\N	calibre-4.20.0_1
2020-07-12 22:20:02+02	2020-07-12 22:21:09+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-07-12 22:21:09+02	2020-07-12 22:21:24+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-07-12 22:21:24+02	2020-07-12 22:21:43+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-07-12 22:21:43+02	2020-07-12 22:49:11+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-12 22:49:13+02	2020-07-12 22:50:23+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-07-12 21:46:11+02	2020-07-12 23:28:14+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-12 23:28:15+02	2020-07-13 00:24:27+02	devel	ghidra	\N	ghidra-9.1
2020-07-13 00:24:28+02	2020-07-13 00:24:44+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-07-13 00:24:45+02	2020-07-13 00:31:43+02	devel	subversion	\N	subversion-1.14.0
2020-07-13 00:31:44+02	2020-07-13 01:59:00+02	editors	codelite	\N	codelite-12.0_4
2020-07-13 01:59:01+02	2020-07-13 02:01:28+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-13 02:01:30+02	2020-07-13 02:06:13+02	editors	vim	\N	vim-8.2.1110
2020-07-13 02:06:13+02	2020-07-13 03:12:30+02	emulators	qemu	\N	qemu-4.1.1_4
2020-07-13 03:12:32+02	2020-07-13 04:27:51+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-07-13 04:27:53+02	2020-07-13 05:15:04+02	emulators	wine	\N	wine-5.0.1,1
2020-07-13 05:15:06+02	2020-07-13 05:15:55+02	emulators	x48	\N	x48-0.6.4_1
2020-07-13 05:15:56+02	2020-07-13 05:31:34+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-07-13 05:31:35+02	2020-07-13 05:31:58+02	games	KnightCap	\N	knightcap-3.6_8
2020-07-13 05:31:59+02	2020-07-13 05:32:10+02	games	affenspiel	\N	affenspiel-1.0_2
2020-07-13 05:32:10+02	2020-07-13 05:32:33+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-07-13 05:32:34+02	2020-07-13 05:35:13+02	games	alienarena	\N	alienarena-2013.766_9
2020-07-13 05:35:14+02	2020-07-13 05:37:04+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-07-13 05:37:05+02	2020-07-13 05:44:42+02	games	battletanks	\N	btanks-0.9.8083_9
2020-07-13 05:44:43+02	2020-07-13 05:45:10+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-07-13 05:45:10+02	2020-07-13 05:45:57+02	games	monsterz	\N	monsterz-0.7.1_15
2020-07-13 05:45:58+02	2020-07-13 05:46:37+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-07-13 05:46:38+02	2020-07-13 05:47:37+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-07-13 05:47:38+02	2020-07-13 05:48:51+02	games	xblast	\N	xblast-2.10.4_17
2020-07-13 05:48:52+02	2020-07-13 05:50:21+02	games	xboard	\N	xboard-4.9.1_1
2020-07-13 05:50:22+02	2020-07-13 05:51:16+02	games	xcowsay	\N	xcowsay-1.5
2020-07-13 05:51:17+02	2020-07-13 05:52:05+02	games	xevil	\N	xevil-2.02.r2_2
2020-07-13 05:52:06+02	2020-07-13 05:52:28+02	games	xminesweep	\N	xminesweep-3.0_2
2020-07-13 05:52:28+02	2020-07-13 07:08:46+02	graphics	blender	\N	blender-2.83.1
2020-07-13 07:08:47+02	2020-07-13 07:14:24+02	graphics	dia	\N	dia-0.97.3_1,1
2020-07-13 07:14:25+02	2020-07-13 07:15:44+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-13 07:15:45+02	2020-07-13 08:21:34+02	graphics	inkscape	\N	inkscape-1.0_3
2020-07-12 22:50:24+02	2020-07-13 09:15:04+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-13 09:15:06+02	2020-07-13 09:19:16+02	graphics	wings	\N	wings-2.2.5
2020-07-13 09:19:16+02	2020-07-13 09:21:52+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-13 09:21:53+02	2020-07-13 10:03:22+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-07-13 10:03:24+02	2020-07-13 10:04:36+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-07-13 08:21:35+02	2020-07-13 10:23:37+02	graphics	krita	\N	krita-4.3.0_1
2020-07-13 10:04:37+02	2020-07-13 10:39:56+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.3
2020-07-13 10:39:57+02	2020-07-13 10:40:32+02	lang	julia	\N	julia-1.0.3_3
2020-07-13 10:23:38+02	2020-07-13 11:04:26+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-07-13 11:04:27+02	2020-07-13 11:04:52+02	mail	bbmail	\N	bbmail-0.9.3
2020-07-13 11:04:52+02	2020-07-13 11:11:13+02	math	maxima	\N	maxima-5.44.0_2
2020-07-13 11:11:45+02	2020-07-13 11:13:02+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-13 11:13:02+02	2020-07-13 11:13:25+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-07-13 11:13:26+02	2020-07-13 11:14:22+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-07-13 10:40:33+02	2020-07-13 11:34:42+02	lang	mono	\N	mono-5.10.1.57_3
2020-07-13 11:34:43+02	2020-07-13 11:35:20+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-07-13 11:35:21+02	2020-07-13 11:35:37+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-07-13 11:14:22+02	2020-07-13 11:36:42+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.2.0
2020-07-13 11:36:43+02	2020-07-13 11:39:34+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-13 11:35:38+02	2020-07-13 11:42:23+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-13 11:42:24+02	2020-07-13 11:48:14+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-07-13 11:48:15+02	2020-07-13 11:48:56+02	net	rdesktop	\N	rdesktop-1.8.6
2020-07-13 11:39:35+02	2020-07-13 11:50:08+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-13 11:50:09+02	2020-07-13 11:50:24+02	net	vnstat	\N	vnstat-2.6_1
2020-07-13 11:48:57+02	2020-07-13 11:50:25+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-13 11:50:26+02	2020-07-13 11:51:23+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-07-13 11:51:24+02	2020-07-13 11:52:21+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-14 00:43:02+02	2020-07-14 01:25:03+02	net	openmpi	\N	openmpi-4.0.4
2020-07-13 11:52:22+02	2020-07-13 11:53:20+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-13 11:53:21+02	2020-07-13 11:54:19+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-14 02:39:56+02	2020-07-14 03:50:11+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-13 11:54:20+02	2020-07-13 11:55:19+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-13 11:55:20+02	2020-07-13 11:56:19+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-13 11:56:20+02	2020-07-13 11:57:19+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-07-13 11:57:20+02	2020-07-13 11:58:18+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-13 11:58:19+02	2020-07-13 11:59:23+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-13 11:59:23+02	2020-07-13 12:00:22+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-13 12:00:23+02	2020-07-13 12:01:28+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-07-13 12:01:28+02	2020-07-13 12:04:53+02	print	auctex	full	auctex-emacs26-12.2
2020-07-13 12:04:54+02	2020-07-13 12:05:45+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-07-13 12:05:45+02	2020-07-13 12:06:39+02	print	gv	\N	gv-3.7.4_5
2020-07-13 11:50:25+02	2020-07-13 12:13:07+02	net	wireshark	\N	wireshark-3.2.5
2020-07-13 12:06:40+02	2020-07-13 12:27:49+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-13 12:27:50+02	2020-07-13 12:33:21+02	print	texlive-full	\N	texlive-full-20150521
2020-07-13 12:33:22+02	2020-07-13 12:38:29+02	security	klee	\N	klee-2.1_1
2020-07-13 12:38:30+02	2020-07-13 12:38:46+02	security	nyx	\N	nyx-2.1.0_1
2020-07-13 12:38:47+02	2020-07-13 12:40:12+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-13 12:40:13+02	2020-07-13 12:41:27+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-07-13 12:41:27+02	2020-07-13 12:45:41+02	sysutils	exa	\N	exa-0.9.0_11
2020-07-13 12:45:42+02	2020-07-13 12:46:53+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-07-13 12:46:54+02	2020-07-13 12:47:15+02	sysutils	x86info	\N	x86info-1.31.s03
2020-07-13 12:47:15+02	2020-07-13 13:01:18+02	textproc	bat	\N	bat-0.15.4
2020-07-13 12:13:08+02	2020-07-13 13:06:13+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-07-13 13:06:14+02	2020-07-13 13:07:24+02	www	links	\N	links-2.20.2,1
2020-07-13 13:07:25+02	2020-07-13 13:07:41+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-07-13 13:07:41+02	2020-07-13 13:07:57+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-07-13 13:07:58+02	2020-07-13 13:08:15+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-07-13 13:08:15+02	2020-07-13 13:08:31+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-07-13 13:08:32+02	2020-07-13 13:08:48+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-07-13 13:08:49+02	2020-07-13 13:09:07+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-07-13 13:09:08+02	2020-07-13 13:09:22+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-07-13 13:09:23+02	2020-07-13 13:09:36+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-07-13 13:09:37+02	2020-07-13 13:09:51+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-07-13 13:09:52+02	2020-07-13 13:10:15+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-07-13 13:10:16+02	2020-07-13 13:12:35+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-13 13:12:36+02	2020-07-13 13:14:30+02	x11	mate	\N	mate-1.22.2
2020-07-13 13:14:31+02	2020-07-13 13:14:48+02	x11	numlockx	\N	numlockx-1.2
2020-07-13 13:14:49+02	2020-07-13 13:15:03+02	x11	sterm	\N	sterm-0.8.4
2020-07-13 13:15:04+02	2020-07-13 13:15:32+02	x11	xclip	\N	xclip-0.13
2020-07-13 13:15:33+02	2020-07-13 13:15:48+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-07-13 13:15:49+02	2020-07-13 13:16:21+02	x11	xdm	\N	xdm-1.1.12_3
2020-07-13 13:16:21+02	2020-07-13 13:18:01+02	x11	xlockmore	\N	xlockmore-5.63
2020-07-13 13:18:02+02	2020-07-13 13:18:22+02	x11	xman	\N	xman-1.1.5
2020-07-13 13:18:22+02	2020-07-13 13:25:43+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-07-13 13:25:44+02	2020-07-13 13:31:55+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-07-13 13:31:57+02	2020-07-13 13:34:04+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-07-13 13:34:04+02	2020-07-13 13:34:27+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-07-13 13:34:28+02	2020-07-13 13:34:42+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-07-13 13:34:43+02	2020-07-13 13:35:45+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-07-13 13:35:46+02	2020-07-13 13:36:10+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.3_6
2020-07-13 13:36:11+02	2020-07-13 13:36:55+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-07-13 13:36:56+02	2020-07-13 13:37:20+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-07-13 13:37:21+02	2020-07-13 14:29:12+02	editors	vscode	\N	vscode-1.46.1
2020-07-13 14:29:13+02	2020-07-13 14:30:46+02	graphics	xfig	\N	xfig-3.2.7a
2020-07-13 14:30:47+02	2020-07-13 14:31:29+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-07-13 13:01:19+02	2020-07-13 15:46:56+02	www	firefox	\N	firefox-78.0.2,1
2020-07-12 15:32:55+02	2020-07-13 20:53:16+02	www	chromium	\N	chromium-83.0.4103.116
2020-07-13 20:53:17+02	2020-07-13 20:54:08+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-13 20:54:10+02	2020-07-13 20:55:00+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-13 20:55:01+02	2020-07-13 20:55:51+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-13 21:16:45+02	2020-07-13 21:17:25+02	math	mpfr	\N	mpfr-4.1.0
2020-07-13 21:17:26+02	2020-07-13 21:18:42+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-07-13 21:18:43+02	2020-07-13 21:18:57+02	math	mpc	\N	mpc-1.1.0_2
2020-07-13 21:18:57+02	2020-07-13 21:21:19+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-07-13 21:16:45+02	2020-07-13 21:28:50+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1,1
2020-07-13 21:21:21+02	2020-07-13 21:33:23+02	print	texlive-base	\N	texlive-base-20150521_53
2020-07-13 21:33:25+02	2020-07-13 21:58:25+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-07-13 21:58:28+02	2020-07-13 22:04:11+02	print	tex-formats	\N	tex-formats-20150521_2
2020-07-13 22:04:14+02	2020-07-13 22:13:16+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-07-13 22:13:17+02	2020-07-13 22:31:18+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-07-13 22:31:19+02	2020-07-13 22:31:57+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-07-13 22:31:58+02	2020-07-13 22:33:03+02	audio	portaudio	\N	portaudio-19.6.0_5,1
2020-07-13 22:33:03+02	2020-07-13 22:33:20+02	audio	espeak	\N	espeak-1.48.04_7
2020-07-13 22:33:20+02	2020-07-13 22:34:19+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-07-13 22:34:20+02	2020-07-13 22:34:47+02	security	libssh2	\N	libssh2-1.9.0_1,3
2020-07-13 22:34:48+02	2020-07-13 22:45:58+02	devel	gdb	py37	gdb-9.2_1
2020-07-13 22:46:02+02	2020-07-13 22:49:35+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-07-13 22:49:36+02	2020-07-13 22:50:11+02	devel	libgit2	\N	libgit2-1.0.1
2020-07-13 21:28:50+02	2020-07-13 22:52:28+02	lang	gcc9	\N	gcc9-9.3.0_1
2020-07-13 22:50:11+02	2020-07-13 23:11:44+02	java	openjdk8	\N	openjdk8-8.252.09.1_2
2020-07-13 23:11:46+02	2020-07-13 23:13:06+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-07-13 23:13:07+02	2020-07-13 23:13:26+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-07-13 23:13:26+02	2020-07-13 23:14:27+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-07-13 22:52:31+02	2020-07-13 23:14:42+02	math	openblas	\N	openblas-0.3.9_2,1
2020-07-13 23:14:28+02	2020-07-13 23:14:59+02	audio	mpg123	\N	mpg123-1.26.2_1
2020-07-13 23:14:43+02	2020-07-14 00:28:19+02	math	suitesparse	\N	suitesparse-5.8.0
2020-07-14 00:28:21+02	2020-07-14 00:35:41+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-07-14 00:35:44+02	2020-07-14 00:40:05+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-07-14 00:40:07+02	2020-07-14 00:43:00+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-14 02:38:56+02	2020-07-14 02:39:38+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-14 02:39:39+02	2020-07-14 02:39:56+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-07-13 23:14:59+02	2020-07-14 03:16:47+02	lang	rust	\N	rust-1.44.1_1
2020-07-14 03:50:14+02	2020-07-14 03:53:23+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-14 03:53:25+02	2020-07-14 03:58:40+02	audio	jack	\N	jackit-0.125.0_10
2020-07-14 03:58:42+02	2020-07-14 05:24:53+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-14 05:24:58+02	2020-07-14 05:28:36+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-14 05:28:37+02	2020-07-14 05:30:05+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-07-14 05:30:10+02	2020-07-14 05:32:40+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-14 03:16:51+02	2020-07-14 05:32:43+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-14 05:32:40+02	2020-07-14 05:32:53+02	audio	libao	\N	libao-1.2.0_5
2020-07-14 05:32:46+02	2020-07-14 05:33:37+02	audio	openal-soft	\N	openal-soft-1.20.1_2
2020-07-14 05:32:54+02	2020-07-14 05:34:18+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-14 05:34:18+02	2020-07-14 05:35:03+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-07-14 05:33:37+02	2020-07-14 05:35:41+02	graphics	gegl	\N	gegl-0.4.24_1
2020-07-14 05:35:42+02	2020-07-14 05:37:02+02	audio	sox	\N	sox-14.4.2_4
2020-07-14 05:35:04+02	2020-07-14 05:37:05+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-14 05:37:03+02	2020-07-14 05:37:33+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-07-14 05:37:05+02	2020-07-14 05:43:19+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-14 05:37:33+02	2020-07-14 06:04:41+02	graphics	gimp-app	\N	gimp-app-2.10.20_1,1
2020-07-14 05:43:20+02	2020-07-14 06:04:50+02	graphics	opencv	\N	opencv-3.4.1_34
2020-07-14 06:04:42+02	2020-07-14 06:05:08+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-07-14 06:04:51+02	2020-07-14 06:22:03+02	lang	gcc6-aux	\N	gcc6-aux-20180516_1,1
2020-07-14 06:05:08+02	2020-07-14 06:25:51+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-07-14 06:25:52+02	2020-07-14 06:28:03+02	www	apache24	\N	apache24-2.4.43
2020-07-14 06:22:03+02	2020-07-14 06:39:07+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-07-14 06:39:09+02	2020-07-14 13:52:46+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_1
2020-07-14 13:52:48+02	2020-07-14 13:53:01+02	audio	freealut	\N	freealut-1.1.0_4
2020-07-14 13:53:02+02	2020-07-14 13:53:23+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-07-14 06:28:03+02	2020-07-14 13:53:25+02	www	chromium	\N	chromium-83.0.4103.116_1
2020-07-14 13:53:28+02	2020-07-14 13:54:53+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-14 13:53:24+02	2020-07-14 13:55:24+02	devel	kBuild	\N	kBuild-0.1.9998_13
2020-07-14 13:55:25+02	2020-07-14 13:59:36+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-07-14 13:59:36+02	2020-07-14 14:01:28+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-14 14:01:29+02	2020-07-14 14:02:33+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-14 14:02:34+02	2020-07-14 14:04:08+02	print	mftrace	\N	mftrace-1.2.18_2
2020-07-14 13:54:55+02	2020-07-14 14:05:36+02	math	R	\N	R-4.0.2
2020-07-14 14:04:09+02	2020-07-14 14:08:23+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-07-14 14:08:24+02	2020-07-14 14:08:56+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-07-14 14:05:37+02	2020-07-14 14:09:32+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-07-14 14:08:56+02	2020-07-14 14:10:11+02	textproc	dblatex	\N	dblatex-0.3.11
2020-07-14 14:09:33+02	2020-07-14 14:10:28+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-14 14:10:11+02	2020-07-14 14:10:40+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-14 14:10:28+02	2020-07-14 14:10:41+02	audio	dssi	\N	dssi-1.1.1_8
2020-07-14 14:10:42+02	2020-07-14 14:10:54+02	audio	sndio	\N	sndio-1.7.0
2020-07-14 14:10:40+02	2020-07-14 14:11:07+02	audio	faudio	\N	FAudio-20.07
2020-07-14 14:10:54+02	2020-07-14 14:11:33+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-07-14 14:11:34+02	2020-07-14 14:11:58+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-07-14 14:11:59+02	2020-07-14 14:12:13+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-07-14 14:12:14+02	2020-07-14 14:14:04+02	devel	gradle5	\N	gradle5-5.0
2020-07-14 14:11:07+02	2020-07-14 14:17:02+02	devel	aarch64-gcc	\N	aarch64-gcc-6.4.0_8
2020-07-14 14:14:06+02	2020-07-14 14:25:07+02	devel	mips64-gcc	\N	mips64-gcc-6.4.0_8
2020-07-14 14:25:11+02	2020-07-14 14:28:24+02	devel	py-game	py37	py37-game-1.9.6_1
2020-07-14 14:17:04+02	2020-07-14 14:30:12+02	devel	powerpc64-gcc	\N	powerpc64-gcc-6.4.0_8
2020-07-14 14:30:15+02	2020-07-14 14:37:42+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3
2020-07-14 14:28:26+02	2020-07-14 14:41:00+02	devel	riscv64-gcc	\N	riscv64-gcc-8.3.0
2020-07-14 14:37:45+02	2020-07-14 14:45:49+02	devel	sparc64-gcc	\N	sparc64-gcc-6.4.0_8
2020-07-14 14:41:03+02	2020-07-14 14:58:13+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-07-14 14:59:23+02	2020-07-14 15:03:43+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-07-14 15:03:45+02	2020-07-14 15:10:56+02	graphics	alembic	\N	alembic-1.7.13
2020-07-14 15:10:57+02	2020-07-14 15:13:36+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-14 15:13:38+02	2020-07-14 15:20:58+02	graphics	py-gimp	\N	py27-gimp-2.10.20_1
2020-07-14 15:21:05+02	2020-07-14 15:23:28+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-07-14 15:23:29+02	2020-07-14 15:46:07+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-07-14 15:46:10+02	2020-07-14 17:00:27+02	lang	gcc48	\N	gcc48-4.8.5_11
2020-07-14 14:45:50+02	2020-07-14 18:05:12+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-14 18:05:14+02	2020-07-14 18:09:45+02	lang	sbcl	\N	sbcl-2.0.6,1
2020-07-14 18:09:46+02	2020-07-14 18:10:29+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-07-14 17:00:30+02	2020-07-14 18:11:15+02	lang	julia11	\N	julia11-1.1.1_5
2020-07-14 18:11:15+02	2020-07-14 18:15:36+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-14 18:15:39+02	2020-07-14 18:18:21+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-07-14 18:10:30+02	2020-07-14 18:20:00+02	misc	openvdb	\N	openvdb-7.0.0_2
2020-07-14 18:20:00+02	2020-07-14 18:21:51+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-14 18:21:52+02	2020-07-14 18:23:56+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-14 18:23:57+02	2020-07-14 18:25:13+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-14 18:18:21+02	2020-07-14 18:25:33+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-07-14 18:25:13+02	2020-07-14 18:32:49+02	print	lilypond	\N	lilypond-2.20.0
2020-07-14 18:25:33+02	2020-07-14 18:34:09+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-07-14 18:34:09+02	2020-07-14 18:35:29+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-07-14 18:32:50+02	2020-07-14 18:36:41+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-07-14 18:35:30+02	2020-07-14 18:41:42+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-07-14 18:41:42+02	2020-07-14 18:42:54+02	textproc	fop	\N	fop-2.3
2020-07-14 18:42:55+02	2020-07-14 18:44:01+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-14 18:44:01+02	2020-07-14 18:45:02+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-14 18:36:42+02	2020-07-14 18:56:40+02	print	texlive-docs	\N	texlive-docs-20150523
2020-07-14 18:45:03+02	2020-07-14 18:57:33+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-14 18:57:33+02	2020-07-14 18:59:06+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-15 11:45:55+02	2020-07-15 11:50:15+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-15 11:50:16+02	2020-07-15 11:50:22+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-15 11:56:02+02	2020-07-15 11:56:13+02	security	libssh2	\N	libssh2-1.9.0_2,3
2020-07-15 11:56:13+02	2020-07-15 11:56:28+02	devel	libgit2	\N	libgit2-1.0.1
2020-07-15 11:56:02+02	2020-07-15 11:59:21+02	databases	mysql57-client	\N	mysql57-client-5.7.31
2020-07-15 11:56:02+02	2020-07-15 11:59:27+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-07-15 11:59:28+02	2020-07-15 11:59:43+02	audio	lv2	\N	lv2-1.18.0
2020-07-15 11:59:22+02	2020-07-15 11:59:50+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-07-15 11:59:51+02	2020-07-15 12:00:03+02	devel	py-zipp	py37	py37-zipp-3.1.0
2020-07-15 12:00:03+02	2020-07-15 12:00:15+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-0.23
2020-07-15 11:59:43+02	2020-07-15 12:00:21+02	x11	xterm	\N	xterm-358
2020-07-15 12:00:15+02	2020-07-15 12:00:26+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-07-15 12:00:21+02	2020-07-15 12:00:46+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-07-15 12:00:46+02	2020-07-15 12:04:09+02	x11	xorg	\N	xorg-7.7_3
2020-07-15 12:00:27+02	2020-07-15 12:04:18+02	lang	python38	\N	python38-3.8.4
2020-07-15 12:04:10+02	2020-07-15 12:04:22+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-07-15 12:04:19+02	2020-07-15 12:04:33+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-07-15 12:04:33+02	2020-07-15 12:27:01+02	lang	julia11	\N	julia11-1.1.1_5
2020-07-15 12:27:02+02	2020-07-15 12:27:29+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.0
2020-07-15 12:04:22+02	2020-07-15 12:30:05+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-15 12:48:36+02	2020-07-15 12:52:28+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-15 12:52:29+02	2020-07-15 12:52:36+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-15 12:54:13+02	2020-07-15 16:54:48+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-15 16:59:25+02	2020-07-15 17:03:59+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-15 17:03:59+02	2020-07-15 17:05:57+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-15 17:05:58+02	2020-07-15 17:07:49+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-15 17:07:49+02	2020-07-15 17:12:27+02	deskutils	calibre	\N	calibre-4.20.0_1
2020-07-15 17:12:28+02	2020-07-15 18:31:37+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-15 18:31:38+02	2020-07-15 18:31:48+02	devel	aarch64-xtoolchain-gcc	\N	aarch64-xtoolchain-gcc-0.4_1
2020-07-15 16:59:25+02	2020-07-15 18:32:29+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-15 18:31:48+02	2020-07-15 18:32:31+02	devel	cdecl	\N	cdecl-7.4.1
2020-07-15 18:32:30+02	2020-07-15 18:32:40+02	devel	mips64-xtoolchain-gcc	\N	mips64-xtoolchain-gcc-0.4_1
2020-07-15 18:32:31+02	2020-07-15 18:32:41+02	devel	powerpc64-xtoolchain-gcc	\N	powerpc64-xtoolchain-gcc-0.4_1
2020-07-15 18:32:40+02	2020-07-15 18:32:51+02	devel	riscv64-xtoolchain-gcc	\N	riscv64-xtoolchain-gcc-0.4_1
2020-07-15 18:32:41+02	2020-07-15 18:32:52+02	devel	sparc64-xtoolchain-gcc	\N	sparc64-xtoolchain-gcc-0.4_1
2020-07-15 18:32:52+02	2020-07-15 18:35:11+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-15 18:35:12+02	2020-07-15 19:49:24+02	emulators	qemu	\N	qemu-4.1.1_4
2020-07-15 18:32:51+02	2020-07-15 20:07:00+02	editors	codelite	\N	codelite-12.0_4
2020-07-15 16:59:25+02	2020-07-15 20:16:37+02	lang	rust	\N	rust-1.44.1_1
2020-07-15 20:16:38+02	2020-07-15 20:25:20+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3
2020-07-15 20:25:21+02	2020-07-15 20:25:47+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-07-15 20:25:47+02	2020-07-15 20:28:35+02	games	alienarena	\N	alienarena-2013.766_9
2020-07-15 20:28:36+02	2020-07-15 20:30:30+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-07-15 20:30:30+02	2020-07-15 20:38:27+02	games	battletanks	\N	btanks-0.9.8083_9
2020-07-15 20:38:33+02	2020-07-15 20:39:04+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-07-15 20:39:05+02	2020-07-15 20:39:54+02	games	monsterz	\N	monsterz-0.7.1_15
2020-07-15 20:39:55+02	2020-07-15 20:40:35+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-07-15 20:40:36+02	2020-07-15 20:41:35+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-07-15 20:41:36+02	2020-07-15 20:42:48+02	games	xblast	\N	xblast-2.10.4_17
2020-07-15 20:07:01+02	2020-07-15 20:47:56+02	emulators	wine	\N	wine-5.0.1,1
2020-07-15 20:48:05+02	2020-07-15 20:49:18+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-15 19:49:25+02	2020-07-15 20:58:30+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.34_4
2020-07-15 20:49:19+02	2020-07-15 21:53:36+02	graphics	inkscape	\N	inkscape-1.0_3
2020-07-15 20:42:49+02	2020-07-15 21:54:56+02	graphics	blender	\N	blender-2.83.1
2020-07-15 21:54:57+02	2020-07-15 21:55:14+02	lang	gcc	\N	gcc-9_4
2020-07-15 21:53:37+02	2020-07-15 21:57:03+02	graphics	wings	\N	wings-2.2.5
2020-07-15 21:57:04+02	2020-07-15 21:57:42+02	lang	julia	\N	julia-1.0.3_3
2020-07-15 21:57:42+02	2020-07-15 22:05:10+02	math	maxima	\N	maxima-5.44.0_2
2020-07-15 22:05:10+02	2020-07-15 22:08:20+02	misc	mc	\N	mc-nox11-4.8.24
2020-07-15 22:08:21+02	2020-07-15 22:09:45+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-15 22:09:46+02	2020-07-15 22:37:25+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p2
2020-07-15 22:37:27+02	2020-07-15 22:45:42+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-15 22:45:42+02	2020-07-15 22:48:06+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-15 20:58:32+02	2020-07-15 22:48:27+02	graphics	krita	\N	krita-4.3.0_1
2020-07-15 22:48:29+02	2020-07-15 22:54:13+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-07-15 22:54:14+02	2020-07-15 22:55:46+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-15 22:55:47+02	2020-07-15 22:56:50+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-07-15 22:48:07+02	2020-07-15 22:58:05+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-15 22:56:51+02	2020-07-15 22:59:21+02	print	auctex	full	auctex-emacs26-12.2
2020-07-15 22:59:22+02	2020-07-15 23:04:23+02	print	texlive-full	\N	texlive-full-20150521
2020-07-15 23:04:24+02	2020-07-15 23:08:31+02	sysutils	exa	\N	exa-0.9.0_11
2020-07-15 23:08:32+02	2020-07-15 23:08:52+02	sysutils	panicmail	\N	panicmail-1.1
2020-07-15 23:08:53+02	2020-07-15 23:23:03+02	textproc	bat	\N	bat-0.15.4
2020-07-15 22:58:06+02	2020-07-15 23:49:57+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-07-15 23:49:58+02	2020-07-15 23:50:28+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_2
2020-07-15 23:50:29+02	2020-07-15 23:56:24+02	www	mod_php73	\N	mod_php73-7.3.20
2020-07-15 23:56:24+02	2020-07-15 23:57:35+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-07-15 23:57:35+02	2020-07-15 23:57:59+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-07-15 23:58:00+02	2020-07-15 23:58:26+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.4_6
2020-07-15 21:55:15+02	2020-07-16 00:44:58+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200711
2020-07-16 00:44:59+02	2020-07-16 00:45:32+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-07-15 23:58:26+02	2020-07-16 00:47:41+02	editors	vscode	\N	vscode-1.46.1
2020-07-15 23:23:04+02	2020-07-16 01:36:40+02	www	firefox	\N	firefox-78.0.2,1
2020-07-16 01:36:41+02	2020-07-16 01:51:07+02	local	E590T-localbase	\N	E590T-localbase-2020070402
2020-07-16 07:50:08+02	2020-07-16 07:53:55+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-16 07:53:56+02	2020-07-16 07:54:03+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-16 07:59:25+02	2020-07-16 08:01:05+02	shells	bash	\N	bash-5.0.18_1
2020-07-16 08:01:05+02	2020-07-16 08:01:24+02	shells	bash-completion	\N	bash-completion-2.10,2
2020-07-16 08:01:24+02	2020-07-16 08:01:47+02	textproc	xmlto	\N	xmlto-0.0.28
2020-07-16 07:59:25+02	2020-07-16 08:04:35+02	lang	go	\N	go-1.14.5,1
2020-07-16 08:01:47+02	2020-07-16 08:27:47+02	java	openjdk8	\N	openjdk8-8.262.10.1
2020-07-16 08:27:48+02	2020-07-16 08:28:43+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-07-16 08:28:44+02	2020-07-16 08:29:07+02	sysutils	lsof	\N	lsof-4.93.2_12,8
2020-07-16 08:29:08+02	2020-07-16 08:31:18+02	devel	gradle5	\N	gradle5-5.0
2020-07-16 08:31:19+02	2020-07-16 09:09:15+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-07-16 09:09:16+02	2020-07-16 09:11:34+02	textproc	fop	\N	fop-2.3
2020-07-16 09:11:35+02	2020-07-16 09:12:27+02	comms	kermit	\N	kermit-9.0.304.24
2020-07-16 09:12:27+02	2020-07-16 10:10:35+02	devel	ghidra	\N	ghidra-9.1
2020-07-16 08:04:36+02	2020-07-16 10:21:12+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-16 10:21:16+02	2020-07-16 12:19:05+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-16 12:19:06+02	2020-07-16 12:26:28+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-16 12:26:29+02	2020-07-16 12:29:58+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-16 07:59:25+02	2020-07-16 14:18:23+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-07-16 12:29:59+02	2020-07-16 14:20:27+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-16 14:18:24+02	2020-07-16 14:20:30+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-16 14:20:29+02	2020-07-16 14:22:25+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-16 14:22:26+02	2020-07-16 14:54:41+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p3
2020-07-16 14:54:43+02	2020-07-16 14:55:33+02	net	wireguard	\N	wireguard-1.0.20200513
2020-07-16 14:55:33+02	2020-07-16 15:19:01+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-07-16 15:19:02+02	2020-07-16 15:21:55+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-07-16 15:21:55+02	2020-07-16 15:27:32+02	security	klee	\N	klee-2.1_1
2020-07-16 15:27:36+02	2020-07-16 15:27:44+02	security	lynis	\N	lynis-3.0.0
2020-07-16 15:27:45+02	2020-07-16 15:28:07+02	security	rkhunter	\N	rkhunter-1.4.6
2020-07-16 15:28:08+02	2020-07-16 15:28:49+02	sysutils	htop	\N	htop-2.2.0_1
2020-07-16 15:28:50+02	2020-07-16 15:28:59+02	sysutils	neofetch	\N	neofetch-7.0.0
2020-07-16 15:29:00+02	2020-07-16 16:19:16+02	editors	vscode	\N	vscode-1.46.1
2020-07-16 10:10:36+02	2020-07-16 16:41:57+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200706_1
2020-07-16 14:20:31+02	2020-07-16 18:38:40+02	devel	llvm10	\N	llvm10-10.0.0_2
2020-07-16 18:38:41+02	2020-07-16 19:15:20+02	local	E590T-localbase	\N	E590T-localbase-2020071600
2020-07-16 19:57:16+02	2020-07-16 20:01:36+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-16 20:01:37+02	2020-07-16 20:01:44+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-16 20:02:06+02	2020-07-16 20:03:21+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-16 20:03:21+02	2020-07-16 20:03:29+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-16 20:09:19+02	2020-07-16 20:10:05+02	shells	bash	\N	bash-5.0.18_2
2020-07-16 20:10:05+02	2020-07-16 20:10:29+02	shells	bash-completion	\N	bash-completion-2.10,2
2020-07-16 20:09:19+02	2020-07-16 20:10:30+02	graphics	ilmbase	\N	ilmbase-2.5.2
2020-07-16 20:10:29+02	2020-07-16 20:10:49+02	textproc	xmlto	\N	xmlto-0.0.28
2020-07-16 20:10:31+02	2020-07-16 20:15:18+02	graphics	openexr	\N	openexr-2.5.2
2020-07-16 20:15:20+02	2020-07-16 20:16:22+02	archivers	gcab	\N	gcab-1.3
2020-07-16 20:16:23+02	2020-07-16 20:17:50+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-07-16 20:10:50+02	2020-07-16 20:18:10+02	cad	opencascade	\N	opencascade-7.4.0_6
2020-07-16 20:17:51+02	2020-07-16 20:21:51+02	graphics	gegl	\N	gegl-0.4.24_2
2020-07-16 20:21:52+02	2020-07-16 20:27:06+02	dns	bind-tools	\N	bind-tools-9.16.5
2020-07-16 20:18:11+02	2020-07-16 20:38:52+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-16 20:38:53+02	2020-07-16 20:39:46+02	x11-fonts	cantarell-fonts	\N	cantarell-fonts-0.101_1
2020-07-16 20:39:47+02	2020-07-16 20:41:03+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-07-16 20:41:04+02	2020-07-16 20:42:36+02	graphics	vigra	\N	vigra-1.11.1_11
2020-07-16 20:42:37+02	2020-07-16 20:47:37+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-07-16 20:27:07+02	2020-07-16 20:49:00+02	graphics	gimp-app	\N	gimp-app-2.10.20_2,1
2020-07-16 20:47:38+02	2020-07-16 20:49:05+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-07-16 20:49:01+02	2020-07-16 20:53:49+02	graphics	alembic	\N	alembic-1.7.13_1
2020-07-16 20:53:50+02	2020-07-16 20:55:57+02	graphics	py-gimp	\N	py27-gimp-2.10.20_2
2020-07-16 20:55:58+02	2020-07-16 21:03:25+02	misc	openvdb	\N	openvdb-7.0.0_3
2020-07-16 21:03:26+02	2020-07-16 21:05:30+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-07-16 21:05:30+02	2020-07-16 21:07:14+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-16 20:49:06+02	2020-07-16 21:08:21+02	graphics	openimageio	\N	openimageio-2.1.15.0_1
2020-07-16 21:07:14+02	2020-07-16 21:08:23+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-07-16 21:08:21+02	2020-07-16 21:11:05+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-07-16 21:11:06+02	2020-07-16 21:13:34+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-16 21:08:23+02	2020-07-16 21:14:05+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-16 21:14:05+02	2020-07-16 21:16:08+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-16 21:16:09+02	2020-07-16 21:18:09+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-16 21:18:10+02	2020-07-16 22:07:26+02	devel	ghidra	\N	ghidra-9.1
2020-07-16 22:07:27+02	2020-07-16 22:12:11+02	dns	bind916	\N	bind916-9.16.5
2020-07-16 22:12:12+02	2020-07-16 22:18:44+02	graphics	blender	\N	blender-2.83.1_1
2020-07-16 22:18:45+02	2020-07-16 22:20:34+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-16 22:20:35+02	2020-07-16 22:31:35+02	graphics	krita	\N	krita-4.3.0_2
2020-07-16 22:31:37+02	2020-07-16 22:31:45+02	net	wireguard	\N	wireguard-1.0.20200513
2020-07-16 22:31:45+02	2020-07-16 22:33:01+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-07-16 22:33:02+02	2020-07-16 22:34:06+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-07-16 22:34:06+02	2020-07-16 22:34:13+02	sysutils	neofetch	\N	neofetch-7.0.0
2020-07-16 21:13:35+02	2020-07-16 22:47:22+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-16 22:34:14+02	2020-07-16 23:09:04+02	editors	vscode	\N	vscode-1.46.1
2020-07-16 20:09:19+02	2020-07-16 23:52:37+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-07-16 23:52:37+02	2020-07-17 00:01:18+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-07-17 00:01:19+02	2020-07-17 00:04:16+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-07-17 00:01:19+02	2020-07-17 00:04:30+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-07-17 00:01:19+02	2020-07-17 00:05:01+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-07-17 00:04:30+02	2020-07-17 00:05:35+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-17 00:05:02+02	2020-07-17 00:05:43+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-07-17 00:05:02+02	2020-07-17 00:05:47+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-17 00:05:36+02	2020-07-17 00:06:17+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-07-17 00:05:43+02	2020-07-17 00:06:24+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-07-17 00:05:47+02	2020-07-17 00:06:31+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-07-17 00:06:24+02	2020-07-17 00:07:04+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-07-17 00:06:32+02	2020-07-17 00:07:11+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-07-17 00:06:17+02	2020-07-17 00:07:49+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-07-17 00:07:05+02	2020-07-17 00:08:10+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-17 00:07:49+02	2020-07-17 00:08:25+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-17 00:08:26+02	2020-07-17 00:10:36+02	x11	xorg	\N	xorg-7.7_3
2020-07-17 00:10:37+02	2020-07-17 00:24:58+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-17 09:01:25+02	2020-07-17 09:02:30+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-17 09:02:31+02	2020-07-17 09:02:38+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-17 09:08:04+02	2020-07-17 09:09:29+02	security	clamav	\N	clamav-0.102.4,1
2020-07-17 09:09:29+02	2020-07-17 09:23:47+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-17 12:06:30+02	2020-07-17 12:07:37+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-17 12:07:37+02	2020-07-17 12:07:44+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-17 12:13:21+02	2020-07-17 12:16:55+02	deskutils	calibre	\N	calibre-4.21.0
2020-07-17 12:13:21+02	2020-07-17 15:21:07+02	lang	rust	\N	rust-1.45.0
2020-07-17 15:21:09+02	2020-07-17 15:25:11+02	sysutils	exa	\N	exa-0.9.0_12
2020-07-17 15:25:12+02	2020-07-17 15:27:35+02	sysutils	hexyl	\N	hexyl-0.8.0_1
2020-07-17 15:21:09+02	2020-07-17 15:28:07+02	textproc	ripgrep	\N	ripgrep-12.1.1_2
2020-07-17 15:21:09+02	2020-07-17 15:28:12+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3_1
2020-07-17 15:27:36+02	2020-07-17 15:37:12+02	sysutils	rsfetch	\N	rsfetch-2.0.0_4
2020-07-17 15:28:08+02	2020-07-17 15:41:19+02	textproc	bat	\N	bat-0.15.4_1
2020-07-17 15:37:13+02	2020-07-17 15:57:31+02	x11	alacritty	\N	alacritty-0.4.3_2
2020-07-17 15:28:12+02	2020-07-17 17:15:15+02	www	firefox	\N	firefox-78.0.2_1,1
2020-07-17 17:15:17+02	2020-07-17 17:29:47+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-17 18:35:36+02	2020-07-17 18:36:43+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-17 18:36:44+02	2020-07-17 18:36:51+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-17 18:42:20+02	2020-07-17 18:44:37+02	graphics	poppler	\N	poppler-0.90.1
2020-07-17 18:42:20+02	2020-07-17 18:47:08+02	graphics	graphviz	\N	graphviz-2.44.0_6
2020-07-17 18:42:20+02	2020-07-17 18:49:00+02	graphics	opencv-core	\N	opencv-core-3.4.1_35
2020-07-17 18:47:09+02	2020-07-17 18:51:09+02	lang	vala	\N	vala-0.40.15,1
2020-07-17 18:44:37+02	2020-07-17 18:52:19+02	print	texlive-base	\N	texlive-base-20150521_53
2020-07-17 18:51:10+02	2020-07-17 18:53:48+02	graphics	poppler-glib	\N	poppler-glib-0.90.1
2020-07-17 18:53:49+02	2020-07-17 18:54:36+02	audio	mpg123	\N	mpg123-1.26.3
2020-07-17 18:54:37+02	2020-07-17 18:55:15+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-07-17 18:55:16+02	2020-07-17 19:00:53+02	lang	go	\N	go-1.14.6,1
2020-07-17 19:00:54+02	2020-07-17 19:03:35+02	graphics	poppler-utils	\N	poppler-utils-0.90.1
2020-07-17 19:03:36+02	2020-07-17 19:05:00+02	graphics	gegl	\N	gegl-0.4.24_2
2020-07-17 18:49:02+02	2020-07-17 19:06:53+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1,1
2020-07-17 19:06:54+02	2020-07-17 19:07:30+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-17 19:07:31+02	2020-07-17 19:08:07+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-17 19:05:00+02	2020-07-17 19:09:54+02	graphics	gimp-app	\N	gimp-app-2.10.20_2,1
2020-07-17 18:52:20+02	2020-07-17 19:12:35+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-07-17 19:12:36+02	2020-07-17 19:16:23+02	print	tex-formats	\N	tex-formats-20150521_2
2020-07-17 19:16:24+02	2020-07-17 19:24:30+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-07-17 19:24:31+02	2020-07-17 19:43:28+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-07-17 19:09:55+02	2020-07-17 19:44:38+02	graphics	opencv	\N	opencv-3.4.1_35
2020-07-17 19:44:39+02	2020-07-17 19:46:13+02	print	cups-filters	\N	cups-filters-1.27.5_1
2020-07-17 19:43:29+02	2020-07-17 19:51:33+02	devel	tbb	\N	tbb-2020.3
2020-07-17 19:46:13+02	2020-07-17 20:05:50+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-07-17 20:05:51+02	2020-07-17 20:14:32+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-07-17 20:14:33+02	2020-07-17 20:38:05+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-17 19:51:34+02	2020-07-17 21:50:49+02	math	suitesparse	\N	suitesparse-5.8.0_1
2020-07-17 21:50:51+02	2020-07-17 21:56:17+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-07-17 21:56:18+02	2020-07-17 22:20:08+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-07-17 19:08:08+02	2020-07-17 22:40:54+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-17 22:40:55+02	2020-07-17 22:43:06+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-17 22:43:07+02	2020-07-17 22:50:00+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.1
2020-07-17 22:50:02+02	2020-07-17 22:50:38+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-07-17 22:50:39+02	2020-07-17 22:57:41+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-07-17 22:57:42+02	2020-07-17 23:00:43+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-17 23:00:43+02	2020-07-17 23:06:08+02	print	gutenprint	\N	gutenprint-5.3.3
2020-07-17 23:06:09+02	2020-07-17 23:09:06+02	print	mftrace	\N	mftrace-1.2.18_2
2020-07-17 23:09:07+02	2020-07-17 23:17:30+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-07-17 23:17:31+02	2020-07-17 23:24:46+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-07-17 23:24:47+02	2020-07-17 23:25:50+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-07-17 23:25:51+02	2020-07-17 23:28:17+02	textproc	dblatex	\N	dblatex-0.3.11
2020-07-17 23:28:18+02	2020-07-17 23:30:11+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-17 23:30:11+02	2020-07-17 23:30:59+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-17 23:30:59+02	2020-07-17 23:31:49+02	audio	faudio	\N	FAudio-20.07
2020-07-17 23:31:50+02	2020-07-17 23:33:54+02	devel	py-game	py37	py37-game-1.9.6_1
2020-07-17 23:33:55+02	2020-07-18 06:36:51+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-18 06:36:52+02	2020-07-18 06:42:05+02	graphics	atril	\N	atril-1.22.3_7
2020-07-18 06:42:06+02	2020-07-18 07:20:13+02	graphics	embree	\N	embree-3.10.0_1
2020-07-18 07:20:13+02	2020-07-18 07:22:25+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-18 07:22:26+02	2020-07-18 07:30:02+02	graphics	oidn	\N	oidn-1.1.0_3
2020-07-18 07:30:03+02	2020-07-18 07:31:16+02	graphics	openimageio	\N	openimageio-2.1.15.0_2
2020-07-18 07:31:16+02	2020-07-18 07:36:16+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-07-18 07:36:17+02	2020-07-18 07:38:25+02	graphics	py-gimp	\N	py27-gimp-2.10.20_2
2020-07-18 07:38:26+02	2020-07-18 07:53:17+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-07-18 07:53:18+02	2020-07-18 07:54:43+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-07-18 07:54:44+02	2020-07-18 08:14:39+02	misc	openvdb	\N	openvdb-7.0.0_4
2020-07-18 08:14:41+02	2020-07-18 08:21:00+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-18 08:21:01+02	2020-07-18 08:24:51+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-07-18 08:24:52+02	2020-07-18 08:45:30+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-07-17 22:20:10+02	2020-07-18 08:46:35+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_1
2020-07-18 08:45:31+02	2020-07-18 08:47:22+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-18 08:46:36+02	2020-07-18 08:47:55+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-18 08:47:56+02	2020-07-18 08:56:53+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-07-18 08:47:23+02	2020-07-18 09:02:11+02	print	lilypond	\N	lilypond-2.20.0
2020-07-18 08:56:54+02	2020-07-18 09:03:40+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-07-18 09:02:12+02	2020-07-18 09:04:17+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-07-18 09:03:40+02	2020-07-18 09:13:31+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-07-18 09:13:33+02	2020-07-18 09:16:54+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-18 09:16:55+02	2020-07-18 09:20:51+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-18 09:04:18+02	2020-07-18 09:39:19+02	print	texlive-docs	\N	texlive-docs-20150523
2020-07-18 09:39:22+02	2020-07-18 09:46:15+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-18 09:46:16+02	2020-07-18 09:49:56+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-18 09:20:54+02	2020-07-18 10:00:13+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-18 10:00:14+02	2020-07-18 10:02:28+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-18 10:02:29+02	2020-07-18 10:04:21+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-18 10:04:22+02	2020-07-18 10:06:56+02	deskutils	calibre	\N	calibre-4.21.0
2020-07-18 10:06:56+02	2020-07-18 11:36:06+02	devel	RStudio	\N	RStudio-1.3.959
2020-07-18 09:49:57+02	2020-07-18 11:46:12+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-18 11:46:14+02	2020-07-18 11:50:46+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-18 11:50:47+02	2020-07-18 12:54:47+02	emulators	wine	\N	wine-5.0.1,1
2020-07-18 12:54:51+02	2020-07-18 12:55:51+02	games	monsterz	\N	monsterz-0.7.1_15
2020-07-18 12:55:52+02	2020-07-18 13:23:55+02	graphics	blender	\N	blender-2.83.1_2
2020-07-18 13:23:56+02	2020-07-18 13:25:31+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-18 13:25:32+02	2020-07-18 14:45:49+02	graphics	inkscape	\N	inkscape-1.0_3
2020-07-18 14:45:50+02	2020-07-18 15:01:41+02	graphics	krita	\N	krita-4.3.0_2
2020-07-18 15:01:43+02	2020-07-18 15:03:22+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-18 15:03:23+02	2020-07-18 15:13:01+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-18 15:13:03+02	2020-07-18 15:16:20+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-18 15:16:21+02	2020-07-18 15:29:25+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-18 15:29:26+02	2020-07-18 15:30:43+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-07-18 15:30:47+02	2020-07-18 15:34:34+02	print	auctex	full	auctex-emacs26-12.2
2020-07-18 15:34:36+02	2020-07-18 16:36:04+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-10-05 02:39:47+02	2020-10-05 02:52:49+02	math	R	\N	R-4.0.2
2020-07-18 16:36:05+02	2020-07-18 16:42:09+02	print	texlive-full	\N	texlive-full-20150521
2020-07-18 16:42:10+02	2020-07-18 17:46:21+02	www	firefox	\N	firefox-78.0.2_1,1
2020-07-18 17:46:26+02	2020-07-18 17:48:32+02	x11	mate	\N	mate-1.22.2
2020-07-18 11:36:08+02	2020-07-18 22:40:08+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-18 22:40:10+02	2020-07-18 23:13:41+02	editors	vscode	\N	vscode-1.46.1
2020-07-17 20:38:07+02	2020-07-19 04:19:46+02	www	chromium	\N	chromium-83.0.4103.116_1
2020-07-19 04:19:47+02	2020-07-19 04:20:37+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-19 04:20:38+02	2020-07-19 04:21:27+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-19 04:21:27+02	2020-07-19 04:22:16+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-19 04:22:16+02	2020-07-19 04:37:06+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-19 10:00:07+02	2020-07-19 10:04:10+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-19 10:04:11+02	2020-07-19 10:04:18+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-19 10:10:20+02	2020-07-19 10:10:29+02	x11	xev	\N	xev-1.2.4
2020-07-19 10:10:20+02	2020-07-19 10:10:33+02	devel	cscope	\N	cscope-15.9
2020-07-19 10:10:20+02	2020-07-19 10:10:42+02	security	gpgme	\N	gpgme-1.14.0
2020-07-19 10:10:30+02	2020-07-19 10:10:51+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-07-19 10:10:42+02	2020-07-19 10:11:17+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-07-19 10:11:18+02	2020-07-19 10:12:45+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-07-19 10:10:52+02	2020-07-19 10:13:56+02	x11	xorg	\N	xorg-7.7_3
2020-07-19 10:12:46+02	2020-07-19 10:15:36+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-19 10:15:37+02	2020-07-19 10:33:15+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-19 10:33:16+02	2020-07-19 11:06:18+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-19 11:06:24+02	2020-07-19 11:09:00+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-19 11:09:02+02	2020-07-19 11:14:06+02	editors	vim	\N	vim-8.2.1110
2020-07-19 11:14:37+02	2020-07-19 11:29:10+02	graphics	krita	\N	krita-4.3.0_2
2020-07-19 10:13:56+02	2020-07-19 11:42:06+02	editors	codelite	\N	codelite-12.0_4
2020-07-19 10:10:34+02	2020-07-19 11:43:03+02	devel	RStudio	\N	RStudio-1.3.1056
2020-07-19 11:53:07+02	2020-07-19 12:27:23+02	emulators	qemu	\N	qemu-4.2.1
2020-07-19 12:27:24+02	2020-07-19 12:42:44+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-19 14:43:04+02	2020-07-19 14:44:43+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-19 14:44:43+02	2020-07-19 14:44:50+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-19 14:50:36+02	2020-07-19 14:51:33+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-19 14:50:36+02	2020-07-19 14:56:42+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-07-19 14:56:43+02	2020-07-19 14:58:08+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-07-19 14:56:43+02	2020-07-19 14:59:22+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-07-19 14:58:09+02	2020-07-19 14:59:54+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-07-19 14:59:23+02	2020-07-19 15:00:29+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-07-19 14:59:54+02	2020-07-19 15:00:55+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-07-19 15:00:29+02	2020-07-19 15:01:26+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-07-19 14:56:43+02	2020-07-19 15:01:35+02	net	qt5-network	\N	qt5-network-5.15.0
2020-07-19 15:01:26+02	2020-07-19 15:01:59+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-07-19 15:01:59+02	2020-07-19 15:02:40+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-07-19 15:02:41+02	2020-07-19 15:03:59+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-07-19 15:04:00+02	2020-07-19 15:05:08+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-07-19 15:00:56+02	2020-07-19 15:05:41+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-07-19 15:01:35+02	2020-07-19 15:14:04+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-07-19 15:14:05+02	2020-07-19 15:14:40+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-07-19 15:14:05+02	2020-07-19 15:14:55+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-07-19 15:14:40+02	2020-07-19 15:15:29+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-07-19 15:14:05+02	2020-07-19 15:20:10+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-07-19 15:20:11+02	2020-07-19 15:22:17+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-07-19 15:20:11+02	2020-07-19 15:22:27+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-07-19 15:22:18+02	2020-07-19 15:22:59+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-07-19 15:23:00+02	2020-07-19 15:24:30+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-07-19 15:22:28+02	2020-07-19 15:24:39+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-07-19 15:24:40+02	2020-07-19 15:26:05+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-07-19 15:26:06+02	2020-07-19 15:27:32+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-19 15:24:30+02	2020-07-19 15:29:29+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-19 15:29:30+02	2020-07-19 15:38:47+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-19 15:20:11+02	2020-07-19 15:54:43+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-07-19 15:54:44+02	2020-07-19 15:57:05+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-07-19 15:57:05+02	2020-07-19 15:58:12+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.71.0_2
2020-07-19 15:58:13+02	2020-07-19 16:00:05+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.71.0
2020-07-19 16:00:07+02	2020-07-19 16:01:56+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-07-19 16:01:57+02	2020-07-19 16:02:51+02	archivers	kf5-karchive	\N	kf5-karchive-5.71.0
2020-07-19 16:02:52+02	2020-07-19 16:04:26+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-07-19 15:38:47+02	2020-07-19 16:05:49+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-19 16:04:27+02	2020-07-19 16:05:53+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-07-19 16:05:50+02	2020-07-19 16:07:46+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-07-19 16:07:47+02	2020-07-19 16:08:57+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-07-19 16:05:53+02	2020-07-19 16:09:22+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-07-19 16:08:58+02	2020-07-19 16:10:15+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-07-19 16:09:22+02	2020-07-19 16:10:32+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.71.0
2020-07-19 16:10:16+02	2020-07-19 16:11:39+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-07-19 16:10:33+02	2020-07-19 16:12:08+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-07-19 16:11:39+02	2020-07-19 16:14:14+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-07-19 16:14:14+02	2020-07-19 16:15:50+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-07-19 16:15:51+02	2020-07-19 16:18:36+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.71.0
2020-07-19 16:18:37+02	2020-07-19 16:20:34+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-07-19 16:20:35+02	2020-07-19 16:22:04+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-07-19 16:22:05+02	2020-07-19 16:25:15+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-19 16:25:16+02	2020-07-19 16:27:01+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-07-19 16:12:09+02	2020-07-19 16:27:17+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-07-19 16:27:18+02	2020-07-19 16:29:21+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-07-19 16:27:02+02	2020-07-19 16:30:39+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-07-19 16:29:22+02	2020-07-19 16:30:47+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-07-19 16:30:48+02	2020-07-19 16:32:05+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-19 16:32:06+02	2020-07-19 16:34:58+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-19 16:30:40+02	2020-07-19 16:40:32+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-19 16:34:59+02	2020-07-19 16:45:48+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-07-19 16:45:49+02	2020-07-19 16:48:01+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-07-19 16:48:02+02	2020-07-19 16:50:50+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-07-19 16:50:50+02	2020-07-19 16:52:42+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-19 16:52:42+02	2020-07-19 16:54:59+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-19 16:55:01+02	2020-07-19 16:58:04+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-07-19 16:58:05+02	2020-07-19 16:59:08+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-19 15:27:33+02	2020-07-19 17:01:30+02	math	vtk8	\N	vtk8-8.2.0
2020-07-19 16:59:09+02	2020-07-19 17:02:01+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-19 17:02:02+02	2020-07-19 17:03:47+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-19 17:03:48+02	2020-07-19 17:08:03+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-07-19 17:08:04+02	2020-07-19 17:24:13+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-07-19 17:24:13+02	2020-07-19 17:25:45+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-19 17:25:48+02	2020-07-19 17:27:36+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-19 17:27:37+02	2020-07-19 17:28:57+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-07-19 17:28:58+02	2020-07-19 17:30:32+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-07-19 17:30:32+02	2020-07-19 17:35:43+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-19 17:35:44+02	2020-07-19 17:39:06+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-19 17:39:07+02	2020-07-19 17:52:59+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-07-19 17:01:33+02	2020-07-19 18:51:12+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-07-19 17:53:00+02	2020-07-19 19:13:43+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-07-19 16:40:33+02	2020-07-19 19:17:56+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-19 19:13:45+02	2020-07-19 19:18:28+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-07-19 19:18:28+02	2020-07-19 19:19:33+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-19 19:17:57+02	2020-07-19 19:20:29+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.1
2020-07-19 19:20:30+02	2020-07-19 19:21:38+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-07-19 19:21:38+02	2020-07-19 19:22:35+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-19 19:22:36+02	2020-07-19 19:24:04+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-19 19:24:04+02	2020-07-19 19:25:39+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0_1
2020-07-19 19:19:34+02	2020-07-19 19:50:16+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-07-19 19:50:17+02	2020-07-19 19:51:50+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-19 19:25:39+02	2020-07-19 19:56:56+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-19 19:56:58+02	2020-07-19 19:58:19+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-19 19:51:51+02	2020-07-19 19:58:27+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-07-19 19:58:19+02	2020-07-19 19:59:13+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-07-19 19:58:28+02	2020-07-19 19:59:27+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-19 19:59:14+02	2020-07-19 20:00:16+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-19 18:51:15+02	2020-07-19 20:04:11+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-19 20:00:17+02	2020-07-19 20:08:06+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-19 20:08:09+02	2020-07-19 20:14:45+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-19 20:04:12+02	2020-07-19 20:16:14+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-19 20:16:17+02	2020-07-19 20:22:18+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-19 20:22:19+02	2020-07-19 20:24:14+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-19 20:24:14+02	2020-07-19 20:26:45+02	deskutils	calibre	\N	calibre-4.21.0
2020-07-19 19:59:27+02	2020-07-19 20:37:52+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-19 20:37:53+02	2020-07-19 20:40:08+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-19 20:26:45+02	2020-07-19 20:50:12+02	devel	RStudio	\N	RStudio-1.3.1056
2020-07-19 20:50:13+02	2020-07-19 21:01:56+02	graphics	krita	\N	krita-4.3.0_2
2020-07-19 21:01:57+02	2020-07-19 21:05:31+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-19 21:05:32+02	2020-07-19 21:08:34+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-19 21:08:34+02	2020-07-19 21:11:06+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-19 21:11:06+02	2020-07-19 21:14:11+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-19 21:14:12+02	2020-07-19 21:37:40+02	net	wireshark	\N	wireshark-3.2.5
2020-07-19 20:40:09+02	2020-07-19 21:38:34+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44
2020-07-19 21:37:41+02	2020-07-19 21:39:18+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-07-19 21:39:18+02	2020-07-19 21:48:31+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-07-19 21:48:32+02	2020-07-19 21:49:43+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-19 21:49:43+02	2020-07-19 21:51:40+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-19 21:38:35+02	2020-07-19 21:54:19+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-19 20:14:48+02	2020-07-19 21:56:03+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-19 21:56:04+02	2020-07-19 22:10:29+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-19 23:18:02+02	2020-07-19 23:21:54+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-19 23:21:54+02	2020-07-19 23:22:35+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-19 23:22:35+02	2020-07-19 23:22:43+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-19 23:50:30+02	2020-07-19 23:50:39+02	devel	ccache	\N	ccache-3.7.1_1
2020-07-19 23:50:39+02	2020-07-19 23:50:45+02	print	indexinfo	\N	indexinfo-0.3.1
2020-07-19 23:54:57+02	2020-07-19 23:55:03+02	devel	autoconf-wrapper	\N	autoconf-wrapper-20131203
2020-07-19 23:54:57+02	2020-07-19 23:55:03+02	misc	pciids	\N	pciids-20200624
2020-07-19 23:54:57+02	2020-07-19 23:55:06+02	devel	ccache	\N	ccache-3.7.1_1
2020-07-19 23:55:03+02	2020-07-19 23:55:09+02	sysutils	gnome_subr	\N	gnome_subr-1.0
2020-07-19 23:55:03+02	2020-07-19 23:55:09+02	devel	evdev-proto	\N	evdev-proto-5.3
2020-07-19 23:55:06+02	2020-07-19 23:55:12+02	print	indexinfo	\N	indexinfo-0.3.1
2020-07-19 23:55:10+02	2020-07-19 23:55:20+02	devel	pkgconf	\N	pkgconf-1.7.3,1
2020-07-19 23:55:12+02	2020-07-19 23:55:41+02	devel	gettext-runtime	\N	gettext-runtime-0.20.2
2020-07-19 23:55:42+02	2020-07-19 23:55:59+02	devel	gmake	\N	gmake-4.3_2
2020-07-19 23:55:59+02	2020-07-19 23:56:28+02	devel	readline	\N	readline-8.0.4
2020-07-19 23:56:28+02	2020-07-19 23:56:47+02	devel	libffi	\N	libffi-3.3
2020-07-19 23:55:20+02	2020-07-19 23:56:54+02	devel	libtextstyle	\N	libtextstyle-0.20.2
2020-07-19 23:56:47+02	2020-07-19 23:57:26+02	converters	libiconv	\N	libiconv-1.16
2020-07-19 23:57:26+02	2020-07-19 23:58:59+02	textproc	libxml2	\N	libxml2-2.9.10
2020-07-19 23:58:59+02	2020-07-19 23:59:23+02	textproc	expat2	\N	expat-2.2.8
2020-07-19 23:59:23+02	2020-07-19 23:59:31+02	devel	xorg-macros	\N	xorg-macros-1.19.2
2020-07-19 23:59:31+02	2020-07-19 23:59:50+02	security	libgpg-error	\N	libgpg-error-1.38
2020-07-19 23:59:50+02	2020-07-20 00:00:03+02	x11	xorgproto	\N	xorgproto-2020.1
2020-07-20 00:00:03+02	2020-07-20 00:00:39+02	archivers	liblz4	\N	liblz4-1.9.2_1,1
2020-07-19 23:55:10+02	2020-07-20 00:00:58+02	lang	perl5.30	\N	perl5-5.30.3
2020-07-20 00:00:59+02	2020-07-20 00:01:16+02	converters	p5-Text-Unidecode	\N	p5-Text-Unidecode-1.30
2020-07-20 00:00:39+02	2020-07-20 00:01:27+02	archivers	lzo2	\N	lzo2-2.10_1
2020-07-20 00:01:17+02	2020-07-20 00:01:29+02	textproc	p5-Unicode-EastAsianWidth	\N	p5-Unicode-EastAsianWidth-12.0
2020-07-19 23:56:54+02	2020-07-20 00:01:39+02	devel	gettext-tools	\N	gettext-tools-0.20.2
2020-07-20 00:01:27+02	2020-07-20 00:01:41+02	security	ca_root_nss	\N	ca_root_nss-3.54
2020-07-20 00:01:40+02	2020-07-20 00:01:53+02	devel	p5-Locale-gettext	\N	p5-Locale-gettext-1.07
2020-07-20 00:01:42+02	2020-07-20 00:02:00+02	devel	p5-Locale-libintl	\N	p5-Locale-libintl-1.31
2020-07-20 00:01:53+02	2020-07-20 00:02:10+02	misc	help2man	\N	help2man-1.47.16
2020-07-20 00:01:30+02	2020-07-20 00:02:33+02	archivers	libarchive	\N	libarchive-3.4.3,1
2020-07-20 00:02:33+02	2020-07-20 00:02:52+02	security	rhash	\N	rhash-1.3.9
2020-07-20 00:02:53+02	2020-07-20 00:03:02+02	devel	libpthread-stubs	\N	libpthread-stubs-0.4
2020-07-20 00:03:02+02	2020-07-20 00:03:16+02	x11	libXau	\N	libXau-1.0.9
2020-07-20 00:02:11+02	2020-07-20 00:03:24+02	print	texinfo	\N	texinfo-6.7_3,1
2020-07-20 00:03:17+02	2020-07-20 00:03:29+02	x11	libXdmcp	\N	libXdmcp-1.1.3
2020-07-20 00:03:25+02	2020-07-20 00:04:10+02	devel	m4	\N	m4-1.4.18_1,1
2020-07-20 00:03:29+02	2020-07-20 00:04:38+02	security	libgcrypt	\N	libgcrypt-1.8.5
2020-07-20 00:04:11+02	2020-07-20 00:04:42+02	devel	autoconf	\N	autoconf-2.69_3
2020-07-20 00:04:39+02	2020-07-20 00:04:59+02	devel	libtool	\N	libtool-2.4.6_1
2020-07-20 00:04:42+02	2020-07-20 00:05:06+02	devel	automake	\N	automake-1.16.2
2020-07-20 00:05:00+02	2020-07-20 00:05:25+02	textproc	libxslt	\N	libxslt-1.1.34_1
2020-07-20 00:05:07+02	2020-07-20 00:05:45+02	www	libnghttp2	\N	libnghttp2-1.41.0
2020-07-20 00:05:25+02	2020-07-20 00:06:00+02	devel	libuv	\N	libuv-1.38.1
2020-07-20 00:02:01+02	2020-07-20 00:06:07+02	lang	python37	\N	python37-3.7.8_1
2020-07-20 00:06:08+02	2020-07-20 00:06:20+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-07-20 00:06:20+02	2020-07-20 00:06:30+02	devel	py-six	py37	py37-six-1.14.0
2020-07-20 00:06:00+02	2020-07-20 00:06:33+02	devel	check	\N	check-0.15.0
2020-07-20 00:06:30+02	2020-07-20 00:06:40+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-07-20 00:06:34+02	2020-07-20 00:06:44+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-07-20 00:06:41+02	2020-07-20 00:06:52+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-07-20 00:06:44+02	2020-07-20 00:07:00+02	devel	py-cffi	py37	py37-cffi-1.14.0_1
2020-07-20 00:06:53+02	2020-07-20 00:07:04+02	dns	py-idna	py37	py37-idna-2.8
2020-07-20 00:05:45+02	2020-07-20 00:07:07+02	ftp	curl	\N	curl-7.71.1
2020-07-20 00:07:08+02	2020-07-20 00:07:18+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-07-20 00:07:18+02	2020-07-20 00:07:27+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-07-20 00:07:04+02	2020-07-20 00:07:30+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-07-20 00:07:00+02	2020-07-20 00:07:33+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-07-20 00:07:31+02	2020-07-20 00:07:41+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-07-20 00:07:34+02	2020-07-20 00:07:44+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-07-20 00:07:44+02	2020-07-20 00:07:53+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-07-20 00:07:53+02	2020-07-20 00:08:03+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-07-20 00:08:03+02	2020-07-20 00:08:14+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-07-20 00:08:14+02	2020-07-20 00:08:24+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-07-20 00:08:25+02	2020-07-20 00:08:39+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-07-20 00:07:28+02	2020-07-20 00:08:39+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-07-20 00:08:40+02	2020-07-20 00:08:49+02	devel	py-packaging	py37	py37-packaging-20.4
2020-07-20 00:08:40+02	2020-07-20 00:08:50+02	www	py-requests	py37	py37-requests-2.22.0
2020-07-20 00:08:50+02	2020-07-20 00:08:59+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-07-20 00:08:59+02	2020-07-20 00:09:08+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-07-20 00:07:42+02	2020-07-20 00:09:18+02	lang	cython	py37	py37-cython-0.29.15
2020-07-20 00:09:08+02	2020-07-20 00:09:18+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-07-20 00:09:18+02	2020-07-20 00:09:27+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-07-20 00:09:18+02	2020-07-20 00:09:42+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-07-20 00:09:28+02	2020-07-20 00:09:42+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-07-20 00:08:50+02	2020-07-20 00:09:43+02	devel	ninja	\N	ninja-1.10.0,2
2020-07-20 00:09:42+02	2020-07-20 00:09:51+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-07-20 00:09:42+02	2020-07-20 00:09:51+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-07-20 00:09:43+02	2020-07-20 00:09:52+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-07-20 00:09:51+02	2020-07-20 00:10:00+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-07-20 00:09:52+02	2020-07-20 00:10:02+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-07-20 00:09:51+02	2020-07-20 00:10:04+02	devel	meson	\N	meson-0.54.3
2020-07-20 00:10:00+02	2020-07-20 00:10:21+02	textproc	py-sphinx	py37	py37-sphinx-3.1.2,1
2020-07-20 00:10:21+02	2020-07-20 00:10:32+02	x11	xtrans	\N	xtrans-1.4.0
2020-07-20 00:10:05+02	2020-07-20 00:10:40+02	devel	jsoncpp	\N	jsoncpp-1.9.3
2020-07-20 00:10:03+02	2020-07-20 00:10:51+02	x11	libxcb	\N	libxcb-1.13.1
2020-07-20 00:10:33+02	2020-07-20 00:13:10+02	math	gmp	\N	gmp-6.2.0
2020-07-20 00:10:51+02	2020-07-20 00:13:22+02	x11	libX11	\N	libX11-1.6.9_2,1
2020-07-20 00:13:23+02	2020-07-20 00:13:43+02	x11	libXext	\N	libXext-1.3.4,1
2020-07-20 00:13:11+02	2020-07-20 00:14:24+02	math	mpfr	\N	mpfr-4.1.0
2020-07-20 00:13:43+02	2020-07-20 00:16:54+02	devel	pcre	\N	pcre-8.44
2020-07-20 00:16:55+02	2020-07-20 00:18:10+02	print	freetype2	\N	freetype2-2.10.2
2020-07-20 00:18:11+02	2020-07-20 00:18:40+02	devel	libedit	\N	libedit-3.1.20191231,1
2020-07-20 00:18:41+02	2020-07-20 00:18:59+02	devel	gperf	\N	gperf-3.1
2020-07-20 00:19:00+02	2020-07-20 00:19:16+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-07-20 00:19:17+02	2020-07-20 00:20:09+02	x11-fonts	fontconfig	\N	fontconfig-2.13.92_2,1
2020-07-20 00:20:09+02	2020-07-20 00:21:45+02	devel	bison	\N	bison-3.6.4,1
2020-07-20 00:21:46+02	2020-07-20 00:22:02+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-07-20 00:22:03+02	2020-07-20 00:22:49+02	devel	libunwind	\N	libunwind-20200331
2020-07-20 00:22:49+02	2020-07-20 00:23:04+02	lang	lua52	\N	lua52-5.2.4
2020-07-20 00:23:04+02	2020-07-20 00:23:21+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-07-20 00:23:21+02	2020-07-20 00:23:40+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-07-20 00:23:41+02	2020-07-20 00:23:58+02	devel	py-future	py37	py37-future-0.18.2
2020-07-20 00:23:59+02	2020-07-20 00:25:14+02	devel	swig	\N	swig-4.0.2
2020-07-20 00:25:15+02	2020-07-20 00:25:28+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-07-20 00:25:28+02	2020-07-20 00:25:59+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-07-20 00:25:59+02	2020-07-20 00:26:15+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-07-20 00:26:16+02	2020-07-20 00:26:28+02	textproc	xmlcatmgr	\N	xmlcatmgr-2.2_2
2020-07-20 00:26:29+02	2020-07-20 00:26:45+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-07-20 00:26:46+02	2020-07-20 00:26:53+02	textproc	iso8879	\N	iso8879-1986_3
2020-07-20 00:26:54+02	2020-07-20 00:27:01+02	textproc	xmlcharent	\N	xmlcharent-0.3_2
2020-07-20 00:27:02+02	2020-07-20 00:27:11+02	textproc	docbook-sgml	\N	docbook-sgml-4.5_1
2020-07-20 00:27:11+02	2020-07-20 00:27:21+02	textproc	docbook-xml	\N	docbook-xml-5.0_3
2020-07-20 00:27:21+02	2020-07-20 00:27:29+02	textproc	sdocbook-xml	\N	sdocbook-xml-1.1_2,2
2020-07-20 00:27:29+02	2020-07-20 00:27:43+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-07-20 00:27:43+02	2020-07-20 00:27:56+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-07-20 00:27:56+02	2020-07-20 00:28:03+02	textproc	docbook	\N	docbook-1.5
2020-07-20 00:28:04+02	2020-07-20 00:28:16+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-07-20 00:28:17+02	2020-07-20 00:28:28+02	x11	libxshmfence	\N	libxshmfence-1.3
2020-07-20 00:28:28+02	2020-07-20 00:29:08+02	textproc	docbook-xsl	\N	docbook-xsl-1.79.1_1,1
2020-07-20 00:29:09+02	2020-07-20 00:29:39+02	graphics	png	\N	png-1.6.37
2020-07-20 00:14:25+02	2020-07-20 00:29:41+02	devel	binutils	native	binutils-2.33.1_2,1
2020-07-20 00:10:41+02	2020-07-20 00:30:33+02	devel	cmake	\N	cmake-3.17.3_1
2020-07-20 00:29:42+02	2020-07-20 00:30:39+02	x11	pixman	\N	pixman-0.40.0
2020-10-05 03:24:21+02	2020-10-05 03:26:14+02	graphics	eom	\N	eom-1.24.1
2020-07-20 00:30:33+02	2020-07-20 00:30:52+02	devel	libepoll-shim	\N	libepoll-shim-0.0.20200602
2020-07-20 00:30:53+02	2020-07-20 00:31:13+02	graphics	wayland	\N	wayland-1.18.0_4
2020-07-20 00:31:13+02	2020-07-20 00:31:22+02	graphics	wayland-protocols	\N	wayland-protocols-1.20
2020-07-20 00:29:39+02	2020-07-20 00:33:59+02	devel	glib20	\N	glib-2.56.3_9,1
2020-07-20 00:34:00+02	2020-07-20 00:34:22+02	x11	libICE	\N	libICE-1.0.10,1
2020-07-20 00:34:23+02	2020-07-20 00:34:40+02	x11	libSM	\N	libSM-1.2.3,1
2020-07-20 00:34:40+02	2020-07-20 00:35:24+02	devel	nasm	\N	nasm-2.14.02,1
2020-07-20 00:35:25+02	2020-07-20 00:35:40+02	x11-fonts	libfontenc	\N	libfontenc-1.1.4
2020-07-20 00:35:41+02	2020-07-20 00:36:36+02	graphics	jpeg-turbo	\N	jpeg-turbo-2.0.4
2020-07-20 00:36:37+02	2020-07-20 00:36:51+02	x11-fonts	mkfontscale	\N	mkfontscale-1.2.1
2020-07-20 00:36:52+02	2020-07-20 00:37:45+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-07-20 00:37:46+02	2020-07-20 00:38:14+02	textproc	html2text	\N	html2text-1.3.2a
2020-07-20 00:38:14+02	2020-07-20 00:38:25+02	textproc	minixmlto	\N	minixmlto-0.0.2_1
2020-07-20 00:38:26+02	2020-07-20 00:39:53+02	devel	dbus	\N	dbus-1.12.20
2020-07-20 00:39:53+02	2020-07-20 00:40:09+02	x11-fonts	bdftopcf	\N	bdftopcf-1.1_2
2020-07-20 00:40:09+02	2020-07-20 00:40:23+02	x11-fonts	font-util	\N	font-util-1.3.2
2020-07-20 00:40:23+02	2020-07-20 00:40:53+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-07-20 00:40:54+02	2020-07-20 00:41:17+02	textproc	p5-XML-Parser	\N	p5-XML-Parser-2.44
2020-07-20 00:41:18+02	2020-07-20 00:41:35+02	textproc	intltool	\N	intltool-0.51.0_1
2020-07-20 00:41:36+02	2020-07-20 00:42:11+02	graphics	graphite2	\N	graphite2-1.3.14
2020-07-20 00:31:22+02	2020-07-20 00:42:21+02	devel	icu	\N	icu-67.1,1
2020-07-20 00:42:12+02	2020-07-20 00:42:22+02	x11-fonts	encodings	\N	encodings-1.0.5,1
2020-07-20 00:42:22+02	2020-07-20 00:42:32+02	graphics	jbigkit	\N	jbigkit-2.1_1
2020-07-20 00:42:22+02	2020-07-20 00:42:35+02	x11-fonts	dejavu	\N	dejavu-2.37_1
2020-07-20 00:42:36+02	2020-07-20 00:42:45+02	x11-fonts	font-bh-ttf	\N	font-bh-ttf-1.0.3_4
2020-07-20 00:42:46+02	2020-07-20 00:42:56+02	x11-fonts	font-misc-ethiopic	\N	font-misc-ethiopic-1.0.3_4
2020-07-20 00:42:57+02	2020-07-20 00:43:12+02	x11-fonts	font-misc-meltho	\N	font-misc-meltho-1.0.3_4
2020-07-20 00:42:32+02	2020-07-20 00:43:21+02	graphics	tiff	\N	tiff-4.1.0
2020-07-20 00:43:12+02	2020-07-20 00:43:30+02	x11-fonts	xorg-fonts-truetype	\N	xorg-fonts-truetype-7.7_1
2020-07-20 00:43:31+02	2020-07-20 00:43:53+02	x11	libXi	\N	libXi-1.7.10,1
2020-07-20 00:43:54+02	2020-07-20 00:44:55+02	devel	boost-jam	\N	boost-jam-1.72.0
2020-07-20 00:43:22+02	2020-07-20 00:45:59+02	devel	libunistring	\N	libunistring-0.9.10_1
2020-07-20 00:45:59+02	2020-07-20 00:46:35+02	dns	libidn2	\N	libidn2-2.3.0_1
2020-07-20 00:46:36+02	2020-07-20 00:47:03+02	emulators	tpm-emulator	\N	tpm-emulator-0.7.4_2
2020-07-20 00:47:03+02	2020-07-20 00:47:40+02	security	libtasn1	\N	libtasn1-4.16.0
2020-07-20 00:47:41+02	2020-07-20 00:49:07+02	security	nettle	\N	nettle-3.6
2020-07-20 00:49:07+02	2020-07-20 00:50:22+02	security	p11-kit	\N	p11-kit-0.23.20_1
2020-07-20 00:50:22+02	2020-07-20 00:52:58+02	security	trousers	\N	trousers-0.3.14_2
2020-07-20 00:52:59+02	2020-07-20 00:57:51+02	security	gnutls	\N	gnutls-3.6.14
2020-07-20 00:57:51+02	2020-07-20 00:58:03+02	print	libpaper	\N	libpaper-1.1.24.4
2020-07-20 00:58:04+02	2020-07-20 00:58:45+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-07-20 00:58:45+02	2020-07-20 00:59:19+02	databases	gdbm	\N	gdbm-1.18.1_1
2020-07-20 00:59:20+02	2020-07-20 01:06:53+02	databases	sqlite3	\N	sqlite3-3.32.3_1,1
2020-07-20 01:06:53+02	2020-07-20 01:07:10+02	devel	libdaemon	\N	libdaemon-0.14_1
2020-07-20 01:07:11+02	2020-07-20 01:07:36+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-07-20 01:07:37+02	2020-07-20 01:07:49+02	graphics	giflib	\N	giflib-5.2.1
2020-07-20 01:07:49+02	2020-07-20 01:08:22+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-07-20 01:08:23+02	2020-07-20 01:08:36+02	textproc	itstool	\N	itstool-2.0.6
2020-07-20 01:08:37+02	2020-07-20 01:09:01+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-07-20 01:09:02+02	2020-07-20 01:10:40+02	graphics	webp	\N	webp-1.1.0
2020-07-20 01:10:41+02	2020-07-20 01:11:00+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-07-20 01:11:01+02	2020-07-20 01:11:31+02	x11	xkeyboard-config	\N	xkeyboard-config-2.29_1
2020-07-20 01:11:32+02	2020-07-20 01:11:41+02	misc	hicolor-icon-theme	\N	hicolor-icon-theme-0.17
2020-07-20 01:11:42+02	2020-07-20 01:12:11+02	x11	libXpm	\N	libXpm-3.5.13
2020-07-20 01:12:11+02	2020-07-20 01:12:25+02	converters	fribidi	\N	fribidi-0.19.7
2020-07-20 01:12:25+02	2020-07-20 01:13:14+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-07-20 01:13:15+02	2020-07-20 01:13:28+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-07-20 01:13:28+02	2020-07-20 01:14:00+02	x11	libxkbcommon	\N	libxkbcommon-0.10.0_2
2020-07-20 01:14:00+02	2020-07-20 01:14:13+02	devel	libltdl	\N	libltdl-2.4.6
2020-07-20 00:44:55+02	2020-07-20 01:15:34+02	devel	boost-libs	\N	boost-libs-1.72.0_2
2020-07-20 01:14:13+02	2020-07-20 01:17:43+02	lang	python27	\N	python27-2.7.18_1
2020-07-20 01:15:35+02	2020-07-20 01:17:44+02	textproc	highlight	\N	highlight-3.43_8,3
2020-07-20 01:17:44+02	2020-07-20 01:18:18+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-07-20 01:17:43+02	2020-07-20 01:18:34+02	dns	libidn	\N	libidn-1.35
2020-07-20 01:18:19+02	2020-07-20 01:18:36+02	graphics	poppler-data	\N	poppler-data-0.4.9_4
2020-07-20 01:18:35+02	2020-07-20 01:18:50+02	print	psutils	\N	psutils-1.17_5
2020-07-20 01:18:37+02	2020-07-20 01:19:06+02	graphics	jbig2dec	\N	jbig2dec-0.18
2020-07-20 01:18:50+02	2020-07-20 01:19:24+02	graphics	lcms2	\N	lcms2-2.11_1
2020-07-20 01:19:07+02	2020-07-20 01:19:38+02	devel	nspr	\N	nspr-4.26
2020-07-20 01:19:25+02	2020-07-20 01:19:54+02	graphics	gd	\N	libgd-2.3.0,1
2020-07-20 01:19:55+02	2020-07-20 01:20:06+02	print	gsfonts	\N	gsfonts-8.11_8
2020-07-20 01:19:38+02	2020-07-20 01:20:19+02	misc	shared-mime-info	\N	shared-mime-info-1.10_2
2020-07-20 01:20:06+02	2020-07-20 01:20:26+02	textproc	uchardet	\N	uchardet-0.0.7
2020-07-20 01:20:20+02	2020-07-20 01:22:50+02	lang	tcl86	\N	tcl86-8.6.10
2020-07-20 01:20:26+02	2020-07-20 01:23:19+02	textproc	groff	\N	groff-1.22.4_3
2020-07-20 01:23:20+02	2020-07-20 01:23:35+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-07-20 01:23:36+02	2020-07-20 01:23:54+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-07-20 01:23:55+02	2020-07-20 01:24:24+02	multimedia	v4l_compat	\N	v4l_compat-1.18.0
2020-07-20 01:24:24+02	2020-07-20 01:24:41+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-07-20 01:24:41+02	2020-07-20 01:25:05+02	devel	autoconf213	\N	autoconf213-2.13.000227_7
2020-07-20 01:22:50+02	2020-07-20 01:25:47+02	devel	pcre2	\N	pcre2-10.35
2020-07-20 01:25:48+02	2020-07-20 01:26:01+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-07-20 01:26:02+02	2020-07-20 01:26:14+02	devel	libudev-devd	\N	libudev-devd-0.4.2
2020-07-20 01:26:15+02	2020-07-20 01:26:51+02	textproc	libcroco	\N	libcroco-0.6.13
2020-07-20 01:26:52+02	2020-07-20 01:27:08+02	x11	xcb-util	\N	xcb-util-0.4.0_2,1
2020-07-20 01:27:09+02	2020-07-20 01:28:45+02	shells	bash	\N	bash-5.0.18_2
2020-07-20 01:28:46+02	2020-07-20 01:29:05+02	devel	jam	\N	jam-2.6
2020-07-20 01:29:06+02	2020-07-20 01:33:12+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-07-20 01:33:13+02	2020-07-20 01:34:11+02	graphics	openjpeg	\N	openjpeg-2.3.1
2020-07-20 01:34:11+02	2020-07-20 01:38:44+02	archivers	zstd	\N	zstd-1.4.5
2020-07-20 01:38:45+02	2020-07-20 01:38:56+02	misc	qtchooser	\N	qtchooser-66_2
2020-07-20 01:38:56+02	2020-07-20 01:39:16+02	devel	double-conversion	\N	double-conversion-3.1.5.19
2020-07-20 01:25:06+02	2020-07-20 01:42:01+02	lang	spidermonkey60	\N	spidermonkey60-60.9.0_4
2020-07-20 01:42:02+02	2020-07-20 01:42:14+02	x11	xprop	\N	xprop-1.2.4
2020-07-20 01:42:14+02	2020-07-20 01:42:27+02	x11	xset	\N	xset-1.2.4_3
2020-07-20 01:39:17+02	2020-07-20 01:42:29+02	devel	qt5-qmake	\N	qt5-qmake-5.15.0
2020-07-20 01:42:27+02	2020-07-20 01:42:42+02	x11	xcb-util-wm	\N	xcb-util-wm-0.4.1_3
2020-07-20 01:42:42+02	2020-07-20 01:43:00+02	devel	libevdev	\N	libevdev-1.5.9_1
2020-07-20 01:43:00+02	2020-07-20 01:43:13+02	devel	libmtdev	\N	libmtdev-1.1.5_2
2020-07-20 01:43:14+02	2020-07-20 01:43:34+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-07-20 01:43:35+02	2020-07-20 01:43:49+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-07-20 01:43:49+02	2020-07-20 01:44:03+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-07-20 01:44:03+02	2020-07-20 01:44:21+02	devel	vulkan-headers	\N	vulkan-headers-1.2.135.0
2020-07-20 01:44:21+02	2020-07-20 01:44:38+02	x11	xcb-util-image	\N	xcb-util-image-0.4.0_1
2020-07-20 01:44:39+02	2020-07-20 01:44:53+02	x11	xcb-util-keysyms	\N	xcb-util-keysyms-0.4.0_1
2020-07-20 01:44:54+02	2020-07-20 01:45:10+02	x11	xcb-util-renderutil	\N	xcb-util-renderutil-0.3.9_1
2020-07-20 01:42:29+02	2020-07-20 01:45:44+02	devel	qt5-buildtools	\N	qt5-buildtools-5.15.0
2020-07-20 01:45:10+02	2020-07-20 01:46:03+02	textproc	gsed	\N	gsed-4.8
2020-07-20 01:46:03+02	2020-07-20 01:46:16+02	audio	libogg	\N	libogg-1.3.4,4
2020-07-20 01:46:17+02	2020-07-20 01:46:53+02	misc	iso-codes	\N	iso-codes-4.2
2020-07-20 01:46:54+02	2020-07-20 01:47:29+02	audio	libvorbis	\N	libvorbis-1.3.7_2,3
2020-07-20 01:47:30+02	2020-07-20 01:47:46+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-07-20 01:47:47+02	2020-07-20 01:48:58+02	devel	orc	\N	orc-0.4.31
2020-07-20 01:48:58+02	2020-07-20 01:53:17+02	math	fftw3	\N	fftw3-3.3.8_6
2020-07-20 01:53:17+02	2020-07-20 01:56:32+02	security	nss	\N	nss-3.54
2020-07-20 01:45:45+02	2020-07-20 01:56:39+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-07-20 01:56:33+02	2020-07-20 01:57:39+02	audio	flac	\N	flac-1.3.3
2020-07-20 01:57:40+02	2020-07-20 02:00:14+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-07-20 01:56:40+02	2020-07-20 02:01:21+02	net	qt5-network	\N	qt5-network-5.15.0
2020-07-20 02:00:14+02	2020-07-20 02:01:32+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-07-20 02:01:21+02	2020-07-20 02:01:36+02	archivers	zip	\N	zip-3.0_1
2020-07-20 02:01:32+02	2020-07-20 02:02:15+02	net	libproxy	\N	libproxy-0.4.15
2020-07-20 02:01:36+02	2020-07-20 02:02:30+02	audio	libsndfile	\N	libsndfile-1.0.28_2
2020-07-20 02:02:31+02	2020-07-20 02:02:56+02	devel	tex-kpathsea	\N	tex-kpathsea-6.2.1_2
2020-07-20 02:02:57+02	2020-07-20 02:03:25+02	shells	bash-completion	\N	bash-completion-2.10,2
2020-07-20 02:02:16+02	2020-07-20 02:03:31+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-07-20 02:03:25+02	2020-07-20 02:03:44+02	textproc	libyaml	\N	libyaml-0.2.5
2020-07-20 02:03:31+02	2020-07-20 02:03:50+02	devel	zziplib	\N	zziplib-0.13.71_1
2020-07-20 02:03:51+02	2020-07-20 02:04:15+02	graphics	openjpeg15	\N	openjpeg15-1.5.2_1
2020-07-20 02:04:15+02	2020-07-20 02:04:34+02	print	tex-ptexenc	\N	tex-ptexenc-1.3.3_2
2020-07-20 02:04:35+02	2020-07-20 02:15:40+02	devel	protobuf	\N	protobuf-3.12.3,1
2020-07-20 02:15:41+02	2020-07-20 02:16:17+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-07-20 02:16:17+02	2020-07-20 02:16:32+02	graphics	libpotrace	\N	libpotrace-1.12
2020-07-20 02:16:32+02	2020-07-20 02:16:53+02	math	mpc	\N	mpc-1.1.0_2
2020-07-20 02:16:53+02	2020-07-20 02:17:22+02	print	texlive-tlmgr	\N	texlive-tlmgr-20150523_2
2020-07-20 02:17:22+02	2020-07-20 02:17:39+02	print	xpdfopen	\N	xpdfopen-0.86
2020-07-20 02:17:39+02	2020-07-20 02:18:46+02	textproc	teckit	\N	teckit-2.5.7
2020-07-20 02:18:47+02	2020-07-20 02:18:59+02	devel	py-setuptools	py27	py27-setuptools-44.0.0
2020-07-20 02:03:44+02	2020-07-20 02:19:01+02	lang	ruby27	\N	ruby-2.7.1_1,1
2020-07-20 02:19:00+02	2020-07-20 02:19:38+02	textproc	flex	\N	flex-2.6.4_2
2020-07-20 02:19:01+02	2020-07-20 02:20:37+02	textproc	hunspell	\N	hunspell-1.7.0_2
2020-07-20 02:19:39+02	2020-07-20 02:20:40+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-07-20 02:20:38+02	2020-07-20 02:20:58+02	multimedia	libva	\N	libva-2.8.0
2020-07-20 02:20:58+02	2020-07-20 02:21:14+02	security	libargon2	\N	libargon2-20190702
2020-07-20 02:21:15+02	2020-07-20 02:22:42+02	devel	libevent	\N	libevent-2.1.11_1
2020-07-20 02:20:41+02	2020-07-20 02:24:23+02	databases	db6	\N	db6-6.2.32_1
2020-07-20 02:24:23+02	2020-07-20 02:24:37+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-07-20 02:22:43+02	2020-07-20 02:26:31+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-07-20 02:26:31+02	2020-07-20 02:27:54+02	archivers	brotli	\N	brotli-1.0.7_2,1
2020-07-20 02:27:54+02	2020-07-20 02:28:06+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-07-20 02:24:37+02	2020-07-20 02:28:31+02	lang	php73	\N	php73-7.3.20
2020-07-20 02:28:06+02	2020-07-20 02:28:36+02	devel	argp-standalone	\N	argp-standalone-1.3_4
2020-07-20 02:28:32+02	2020-07-20 02:28:40+02	audio	ladspa	\N	ladspa-1.15
2020-07-20 02:28:36+02	2020-07-20 02:29:29+02	audio	opus	\N	opus-1.3.1
2020-07-20 02:28:41+02	2020-07-20 02:29:39+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-07-20 02:29:30+02	2020-07-20 02:29:57+02	devel	woff2	\N	woff2-1.0.2_4
2020-07-20 02:29:58+02	2020-07-20 02:30:11+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-07-20 02:30:11+02	2020-07-20 02:30:43+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-07-20 02:29:39+02	2020-07-20 02:30:44+02	multimedia	libv4l	\N	libv4l-1.18.0
2020-07-20 02:30:43+02	2020-07-20 02:30:53+02	devel	opencl	\N	opencl-2.2_2
2020-07-20 02:30:54+02	2020-07-20 02:31:15+02	textproc	hyphen	\N	hyphen-2.8.8
2020-07-20 02:31:16+02	2020-07-20 02:31:41+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-07-20 02:31:41+02	2020-07-20 02:31:53+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-07-20 02:31:53+02	2020-07-20 02:32:18+02	math	eigen3	\N	eigen-3.3.7
2020-07-20 02:30:45+02	2020-07-20 02:32:27+02	lang	fpc	\N	fpc-3.0.4_4
2020-07-20 02:32:18+02	2020-07-20 02:32:30+02	textproc	py-sparqlwrapper	py37	py37-sparqlwrapper-1.8.5
2020-07-20 02:32:27+02	2020-07-20 02:33:01+02	textproc	texi2html	\N	texi2html-5.0_2,1
2020-07-20 02:33:01+02	2020-07-20 02:35:13+02	textproc	py-rdflib	py37	py37-rdflib-4.2.2_1
2020-07-20 02:35:13+02	2020-07-20 02:35:33+02	audio	lv2	\N	lv2-1.18.0
2020-07-20 02:35:33+02	2020-07-20 02:36:21+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-07-20 02:36:22+02	2020-07-20 02:38:14+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-07-20 02:38:14+02	2020-07-20 02:38:40+02	multimedia	libaacs	\N	libaacs-0.9.0
2020-07-20 02:38:40+02	2020-07-20 02:39:28+02	multimedia	libbdplus	\N	libbdplus-0.1.2
2020-07-20 02:39:28+02	2020-07-20 02:39:56+02	multimedia	libtheora	\N	libtheora-1.1.1_7
2020-07-20 02:39:57+02	2020-07-20 02:40:15+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-07-20 02:40:16+02	2020-07-20 02:40:51+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-07-20 02:32:31+02	2020-07-20 02:45:22+02	multimedia	x265	\N	x265-3.2.1_3
2020-07-20 02:40:51+02	2020-07-20 02:48:05+02	graphics	opencv-core	\N	opencv-core-3.4.1_35
2020-07-20 02:48:06+02	2020-07-20 02:50:40+02	multimedia	dav1d	\N	dav1d-0.7.1
2020-07-20 02:50:41+02	2020-07-20 02:51:12+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-07-20 02:45:23+02	2020-07-20 02:53:57+02	multimedia	aom	\N	aom-2.0.0_1
2020-07-20 02:51:13+02	2020-07-20 02:54:48+02	multimedia	libvpx	\N	libvpx-1.8.2_1
2020-07-20 02:54:48+02	2020-07-20 02:55:46+02	multimedia	xvid	\N	xvid-1.3.7,1
2020-07-20 02:53:57+02	2020-07-20 02:57:11+02	multimedia	libx264	\N	libx264-0.160.3011
2020-07-20 02:57:11+02	2020-07-20 02:58:11+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-07-20 02:55:47+02	2020-07-20 02:58:43+02	databases	postgresql12-client	\N	postgresql12-client-12.3
2020-07-20 02:58:12+02	2020-07-20 02:58:44+02	devel	libinotify	\N	libinotify-20180201_2
2020-07-20 02:58:44+02	2020-07-20 02:59:29+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-07-20 02:59:29+02	2020-07-20 03:00:37+02	security	cyrus-sasl2	\N	cyrus-sasl-2.1.27_1
2020-07-20 03:00:37+02	2020-07-20 03:01:29+02	audio	alsa-lib	\N	alsa-lib-1.1.2_2
2020-07-20 03:01:30+02	2020-07-20 03:02:27+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-07-20 03:02:28+02	2020-07-20 03:04:07+02	graphics	libraw	\N	libraw-0.19.5
2020-07-20 03:04:07+02	2020-07-20 03:04:37+02	archivers	libmspack	\N	libmspack-0.10.1
2020-07-20 03:04:37+02	2020-07-20 03:05:13+02	graphics	jasper	\N	jasper-2.0.16_1
2020-07-20 03:05:14+02	2020-07-20 03:05:51+02	devel	libatomic_ops	\N	libatomic_ops-7.6.10
2020-07-20 03:05:52+02	2020-07-20 03:06:33+02	graphics	aalib	\N	aalib-1.4.r5_13
2020-07-20 03:06:34+02	2020-07-20 03:07:18+02	devel	boehm-gc	\N	boehm-gc-8.0.4_1
2020-07-20 03:07:18+02	2020-07-20 03:07:43+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-07-20 03:07:44+02	2020-07-20 03:10:17+02	net	openldap24-client	\N	openldap-client-2.4.50
2020-07-20 03:10:18+02	2020-07-20 03:10:37+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-07-20 03:10:37+02	2020-07-20 03:11:10+02	devel	json-c	\N	json-c-0.14
2020-07-20 03:11:11+02	2020-07-20 03:11:32+02	net	p5-URI	\N	p5-URI-1.76
2020-07-20 03:11:32+02	2020-07-20 03:11:59+02	audio	speexdsp	\N	speexdsp-1.2.0
2020-07-20 03:12:00+02	2020-07-20 03:12:29+02	science	szip	\N	hdf-szip-2.1.1
2020-07-20 03:12:29+02	2020-07-20 03:14:01+02	textproc	librevenge	\N	librevenge-0.0.4_13
2020-07-20 03:14:01+02	2020-07-20 03:14:20+02	x11	libXres	\N	libXres-1.2.0_2
2020-07-20 03:14:20+02	2020-07-20 03:14:40+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-07-20 03:14:41+02	2020-07-20 03:15:50+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-07-20 03:15:50+02	2020-07-20 03:39:10+02	science	hdf5	\N	hdf5-1.10.6,1
2020-07-20 03:39:11+02	2020-07-20 03:39:33+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-07-20 03:39:33+02	2020-07-20 03:40:23+02	devel	oniguruma	\N	oniguruma-6.9.5.r1_1
2020-07-20 03:40:24+02	2020-07-20 03:40:42+02	lang	fpc-rtl-objpas	\N	fpc-rtl-objpas-3.0.4_5
2020-07-20 03:40:43+02	2020-07-20 03:41:14+02	www	php73-session	\N	php73-session-7.3.20
2020-07-20 03:41:15+02	2020-07-20 03:41:32+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-07-20 03:41:33+02	2020-07-20 03:42:15+02	x11-fonts	libXfont2	\N	libXfont2-2.0.4
2020-07-20 03:42:15+02	2020-07-20 03:42:44+02	archivers	php73-zlib	\N	php73-zlib-7.3.20
2020-07-20 03:42:45+02	2020-07-20 03:43:17+02	databases	php73-pdo	\N	php73-pdo-7.3.20
2020-07-20 03:43:17+02	2020-07-20 03:43:35+02	devel	fpc-fcl-res	\N	fpc-fcl-res-3.0.4_4
2020-07-20 03:43:36+02	2020-07-20 03:44:05+02	devel	php73-json	\N	php73-json-7.3.20
2020-07-20 03:44:06+02	2020-07-20 03:44:29+02	graphics	libexif	\N	libexif-0.6.21_5
2020-07-20 03:44:29+02	2020-07-20 03:48:50+02	math	fftw3-float	\N	fftw3-float-3.3.8_6
2020-07-20 03:48:51+02	2020-07-20 03:48:59+02	misc	getopt	\N	getopt-1.1.6
2020-07-20 03:48:59+02	2020-07-20 03:49:17+02	security	libassuan	\N	libassuan-2.5.3
2020-07-20 03:49:18+02	2020-07-20 03:52:31+02	security	libsodium	\N	libsodium-1.0.18
2020-07-20 03:52:31+02	2020-07-20 03:54:02+02	www	w3m	\N	w3m-0.5.3.20200507
2020-07-20 03:54:03+02	2020-07-20 03:54:57+02	converters	php73-mbstring	\N	php73-mbstring-7.3.20
2020-07-20 03:54:57+02	2020-07-20 04:09:19+02	databases	libmemcached	\N	libmemcached-1.0.18_7
2020-07-20 04:09:20+02	2020-07-20 04:09:40+02	devel	fpc-fcl-base	\N	fpc-fcl-base-3.0.4_4
2020-07-20 04:09:41+02	2020-07-20 04:10:22+02	graphics	lcms	\N	lcms-1.19_6,1
2020-07-20 04:10:23+02	2020-07-20 04:10:55+02	graphics	php73-gd	\N	php73-gd-7.3.20
2020-07-20 04:10:56+02	2020-07-20 04:11:42+02	security	libmcrypt	\N	libmcrypt-2.5.8_3
2020-07-20 04:11:42+02	2020-07-20 04:12:28+02	security	php73-hash	\N	php73-hash-7.3.20
2020-07-20 04:12:28+02	2020-07-20 04:13:01+02	security	php73-openssl	\N	php73-openssl-7.3.20
2020-07-20 04:13:02+02	2020-07-20 04:13:31+02	textproc	php73-xml	\N	php73-xml-7.3.20
2020-07-20 04:13:31+02	2020-07-20 04:13:44+02	textproc	py-toml	py37	py37-toml-0.10.0
2020-07-20 04:13:45+02	2020-07-20 04:14:13+02	textproc	xmlto	\N	xmlto-0.0.28
2020-07-20 04:14:14+02	2020-07-20 04:15:35+02	audio	portaudio	\N	portaudio-19.6.0_5,1
2020-07-20 04:15:36+02	2020-07-20 04:16:06+02	converters	php73-iconv	\N	php73-iconv-7.3.20
2020-07-20 04:16:06+02	2020-07-20 04:16:36+02	databases	pecl-memcache	php73	php73-pecl-memcache-4.0.5.2
2020-07-20 04:16:37+02	2020-07-20 04:17:10+02	databases	pecl-memcached	php73	php73-pecl-memcached-3.1.5
2020-07-20 04:17:11+02	2020-07-20 04:17:41+02	databases	php73-pdo_pgsql	\N	php73-pdo_pgsql-7.3.20
2020-07-20 04:17:42+02	2020-07-20 04:18:12+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.20
2020-07-20 04:18:13+02	2020-07-20 04:18:48+02	databases	php73-pgsql	\N	php73-pgsql-7.3.20
2020-07-20 04:18:49+02	2020-07-20 04:19:20+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.20
2020-07-20 04:19:20+02	2020-07-20 04:19:41+02	devel	ctags	\N	ctags-5.8
2020-07-20 04:19:41+02	2020-07-20 04:20:11+02	devel	php73-gettext	\N	php73-gettext-7.3.20
2020-07-20 04:20:11+02	2020-07-20 04:20:24+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-07-20 04:20:25+02	2020-07-20 04:25:48+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-07-20 04:25:49+02	2020-07-20 04:26:22+02	ftp	php73-curl	\N	php73-curl-7.3.20
2020-07-20 04:26:23+02	2020-07-20 04:26:53+02	graphics	libmng	\N	libmng-1.0.10_3
2020-07-20 04:26:54+02	2020-07-20 04:27:17+02	math	pecl-bitset	php73	php73-pecl-bitset-3.0.1
2020-07-20 04:27:17+02	2020-07-20 04:27:55+02	math	php73-bcmath	\N	php73-bcmath-7.3.20
2020-07-20 04:27:55+02	2020-07-20 04:28:11+02	multimedia	libdvdcss	\N	libdvdcss-1.4.2
2020-07-20 04:28:12+02	2020-07-20 04:30:21+02	net	php73-ldap	\N	php73-ldap-7.3.20
2020-07-20 04:30:22+02	2020-07-20 04:31:02+02	net	php73-soap	\N	php73-soap-7.3.20
2020-07-20 04:31:03+02	2020-07-20 04:31:30+02	net	php73-sockets	\N	php73-sockets-7.3.20
2020-07-20 04:31:31+02	2020-07-20 04:31:57+02	security	pecl-mcrypt	php73	php73-pecl-mcrypt-1.0.3
2020-07-20 04:31:58+02	2020-07-20 04:32:27+02	sysutils	php73-posix	\N	php73-posix-7.3.20
2020-07-20 04:32:28+02	2020-07-20 04:32:55+02	textproc	php73-ctype	\N	php73-ctype-7.3.20
2020-07-20 04:32:55+02	2020-07-20 04:33:40+02	textproc	php73-dom	\N	php73-dom-7.3.20
2020-07-20 04:33:41+02	2020-07-20 04:34:12+02	textproc	php73-simplexml	\N	php73-simplexml-7.3.20
2020-07-20 04:34:13+02	2020-07-20 04:35:13+02	www	php73-opcache	\N	php73-opcache-7.3.20
2020-07-20 04:35:14+02	2020-07-20 04:35:46+02	audio	espeak	\N	espeak-1.48.04_7
2020-07-20 04:35:46+02	2020-07-20 04:36:00+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-07-20 04:36:01+02	2020-07-20 04:36:16+02	devel	dotconf	\N	dotconf-1.3_1
2020-07-20 04:36:17+02	2020-07-20 04:36:39+02	multimedia	libdvdread	\N	libdvdread-6.1.0
2020-07-20 04:36:39+02	2020-07-20 04:37:43+02	sysutils	policykit	\N	policykit-0.9_10
2020-07-20 04:37:43+02	2020-07-20 04:38:21+02	textproc	asciidoc	\N	asciidoc-9.0.1
2020-07-20 04:38:21+02	2020-07-20 04:39:15+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-07-20 04:39:16+02	2020-07-20 04:39:39+02	archivers	snappy	\N	snappy-1.1.8
2020-07-20 04:39:40+02	2020-07-20 04:39:57+02	audio	alsa-plugins	\N	alsa-plugins-1.1.1_6
2020-07-20 04:39:57+02	2020-07-20 04:40:12+02	devel	libvolume_id	\N	libvolume_id-0.81.1
2020-07-20 04:40:12+02	2020-07-20 04:40:53+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-07-20 04:40:54+02	2020-07-20 04:41:04+02	devel	py-six	py27	py27-six-1.14.0
2020-07-20 04:41:04+02	2020-07-20 04:41:34+02	devel	yasm	\N	yasm-1.3.0
2020-07-20 04:41:34+02	2020-07-20 04:44:24+02	dns	c-ares	\N	c-ares-1.16.1
2020-07-20 04:44:24+02	2020-07-20 04:45:54+02	graphics	ilmbase	\N	ilmbase-2.5.2
2020-07-20 04:45:55+02	2020-07-20 04:46:01+02	java	javavmwrapper	\N	javavmwrapper-2.7.5
2020-07-20 04:46:02+02	2020-07-20 04:46:32+02	misc	e2fsprogs-libuuid	\N	e2fsprogs-libuuid-1.45.6
2020-07-20 04:46:33+02	2020-07-20 04:47:03+02	security	libssh2	\N	libssh2-1.9.0_2,3
2020-07-20 04:47:04+02	2020-07-20 04:47:14+02	sysutils	dmidecode	\N	dmidecode-3.2
2020-07-20 04:47:15+02	2020-07-20 04:52:54+02	textproc	source-highlight	\N	source-highlight-3.1.9_1
2020-07-20 04:52:55+02	2020-07-20 04:53:32+02	www	py-django22	py37	py37-django22-2.2.14
2020-07-20 00:30:40+02	2020-07-20 04:57:41+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-07-20 04:57:42+02	2020-07-20 05:04:15+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-07-20 05:04:16+02	2020-07-20 05:07:20+02	graphics	cairo	\N	cairo-1.16.0,2
2020-07-20 05:07:20+02	2020-07-20 05:08:55+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-07-20 04:53:32+02	2020-07-20 05:10:41+02	devel	gdb	py37	gdb-9.2_1
2020-07-20 05:10:41+02	2020-07-20 05:11:52+02	net	avahi-app	\N	avahi-app-0.7_3
2020-07-20 05:08:56+02	2020-07-20 05:12:07+02	print	harfbuzz	\N	harfbuzz-2.6.8
2020-07-20 05:11:53+02	2020-07-20 05:13:20+02	print	cups	\N	cups-2.3.3_1
2020-07-20 05:12:08+02	2020-07-20 05:14:51+02	x11-toolkits	pango	\N	pango-1.42.4_3
2020-07-20 05:14:52+02	2020-07-20 05:15:30+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-07-20 05:15:30+02	2020-07-20 05:16:50+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-07-20 05:16:51+02	2020-07-20 05:17:18+02	accessibility	atk	\N	atk-2.36.0
2020-07-20 05:13:21+02	2020-07-20 05:17:24+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_6
2020-07-20 05:17:18+02	2020-07-20 05:17:44+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-07-20 05:17:44+02	2020-07-20 05:18:39+02	sysutils	polkit	\N	polkit-0.116
2020-07-20 05:18:39+02	2020-07-20 05:19:32+02	devel	libgsf	\N	libgsf-1.14.47
2020-07-20 05:19:32+02	2020-07-20 05:20:10+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-07-20 02:58:45+02	2020-07-20 05:21:11+02	lang	gcc9	\N	gcc9-9.3.0_1
2020-07-20 05:17:25+02	2020-07-20 05:21:16+02	graphics	graphviz	\N	graphviz-2.44.0_6
2020-07-20 05:21:12+02	2020-07-20 05:21:35+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-07-20 05:20:10+02	2020-07-20 05:21:56+02	graphics	colord	\N	colord-1.3.5
2020-07-20 05:21:57+02	2020-07-20 05:22:12+02	devel	libgudev	\N	libgudev-230_1
2020-07-20 05:22:12+02	2020-07-20 05:22:29+02	x11	libwacom	\N	libwacom-1.4.1
2020-07-20 05:22:30+02	2020-07-20 05:22:54+02	x11	libinput	\N	libinput-1.15.6
2020-07-20 05:21:36+02	2020-07-20 05:22:58+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-07-20 05:22:58+02	2020-07-20 05:23:33+02	graphics	libGLU	\N	libGLU-9.0.1
2020-07-20 05:21:16+02	2020-07-20 05:24:08+02	lang	vala	\N	vala-0.40.15,1
2020-07-20 05:24:08+02	2020-07-20 05:25:29+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-07-20 05:23:34+02	2020-07-20 05:26:27+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.8_2
2020-07-20 05:26:27+02	2020-07-20 05:35:32+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-07-20 05:35:33+02	2020-07-20 05:36:04+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-07-20 05:25:30+02	2020-07-20 05:37:33+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-07-20 05:36:04+02	2020-07-20 05:38:32+02	graphics	poppler	\N	poppler-0.90.1
2020-07-20 05:38:33+02	2020-07-20 05:39:04+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-07-20 05:37:34+02	2020-07-20 05:39:54+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-07-20 05:39:04+02	2020-07-20 05:39:56+02	devel	dconf	\N	dconf-0.36.0
2020-07-20 05:22:55+02	2020-07-20 05:40:27+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-07-20 05:39:56+02	2020-07-20 05:41:09+02	devel	libsoup	\N	libsoup-2.62.3
2020-07-20 05:41:10+02	2020-07-20 05:41:33+02	devel	json-glib	\N	json-glib-1.4.4
2020-07-20 05:39:55+02	2020-07-20 05:42:18+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-07-20 05:42:18+02	2020-07-20 05:42:46+02	devel	libnotify	\N	libnotify-0.7.8
2020-07-20 05:41:34+02	2020-07-20 05:44:00+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-07-20 05:42:47+02	2020-07-20 05:46:11+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-07-20 05:44:00+02	2020-07-20 05:46:23+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-07-20 05:46:24+02	2020-07-20 05:47:32+02	security	libsecret	\N	libsecret-0.18.6_1
2020-07-20 05:47:33+02	2020-07-20 05:48:54+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-07-20 05:40:28+02	2020-07-20 05:49:32+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-07-20 05:48:55+02	2020-07-20 05:50:17+02	graphics	frei0r	\N	frei0r-1.6.1
2020-07-20 05:50:17+02	2020-07-20 05:52:18+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-07-20 05:52:19+02	2020-07-20 05:52:41+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-07-20 05:52:42+02	2020-07-20 05:53:01+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-07-20 05:46:11+02	2020-07-20 05:53:21+02	print	texlive-base	\N	texlive-base-20150521_53
2020-07-20 05:53:02+02	2020-07-20 06:11:10+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1,1
2020-07-20 06:11:11+02	2020-07-20 06:11:41+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-07-20 06:11:42+02	2020-07-20 06:13:12+02	graphics	graphene	\N	graphene-1.10.0
2020-07-20 05:53:22+02	2020-07-20 06:13:27+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-07-20 06:13:13+02	2020-07-20 06:13:28+02	www	libwpe	\N	libwpe-1.4.0.1
2020-07-20 05:49:33+02	2020-07-20 06:15:11+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-07-20 06:13:27+02	2020-07-20 06:15:46+02	print	tex-formats	\N	tex-formats-20150521_2
2020-07-20 06:15:12+02	2020-07-20 06:16:52+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-07-20 06:15:47+02	2020-07-20 06:17:38+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-07-20 06:16:53+02	2020-07-20 06:17:41+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.71.0_2
2020-07-20 06:17:41+02	2020-07-20 06:17:57+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-07-20 06:17:38+02	2020-07-20 06:18:32+02	net	geoclue	\N	geoclue-2.5.5
2020-07-20 06:13:29+02	2020-07-20 06:18:37+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-07-20 06:18:38+02	2020-07-20 06:19:14+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-07-20 06:17:57+02	2020-07-20 06:19:26+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-07-20 06:19:26+02	2020-07-20 06:19:55+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-07-20 06:19:56+02	2020-07-20 06:20:27+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-07-20 06:20:27+02	2020-07-20 06:21:05+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-07-20 06:21:06+02	2020-07-20 06:21:23+02	print	libraqm	\N	libraqm-0.6.0
2020-07-20 06:21:23+02	2020-07-20 06:22:16+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-07-20 06:22:16+02	2020-07-20 06:29:35+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-07-20 06:29:36+02	2020-07-20 06:31:27+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-07-20 06:19:15+02	2020-07-20 06:34:06+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-07-20 06:34:07+02	2020-07-20 06:42:59+02	devel	tbb	\N	tbb-2020.3
2020-07-20 06:43:00+02	2020-07-20 06:44:25+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-07-20 06:31:27+02	2020-07-20 06:44:32+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-07-20 06:44:32+02	2020-07-20 06:45:20+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-07-20 06:44:25+02	2020-07-20 06:45:56+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.71.0
2020-07-20 06:45:21+02	2020-07-20 06:46:20+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-07-20 06:46:20+02	2020-07-20 06:47:21+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-07-20 06:45:56+02	2020-07-20 06:47:38+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-07-20 06:47:22+02	2020-07-20 06:49:19+02	devel	kf5-kconfig	\N	kf5-kconfig-5.71.0
2020-07-20 06:49:20+02	2020-07-20 06:50:38+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-20 06:50:39+02	2020-07-20 06:51:41+02	archivers	kf5-karchive	\N	kf5-karchive-5.71.0
2020-07-20 06:51:42+02	2020-07-20 06:53:23+02	devel	kf5-ki18n	\N	kf5-ki18n-5.71.0
2020-07-20 06:53:23+02	2020-07-20 06:58:41+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-20 06:58:41+02	2020-07-20 06:59:28+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-07-20 06:59:28+02	2020-07-20 07:01:34+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-07-20 07:01:34+02	2020-07-20 07:03:12+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.71.0
2020-07-20 07:03:12+02	2020-07-20 07:03:50+02	devel	libsigc++20	\N	libsigc++-2.10.3
2020-07-20 07:03:50+02	2020-07-20 07:12:25+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-20 07:12:26+02	2020-07-20 07:13:50+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-07-20 07:13:50+02	2020-07-20 07:14:10+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-07-20 07:14:11+02	2020-07-20 07:17:17+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-07-20 07:17:17+02	2020-07-20 07:17:35+02	lang	fpc-rtl-extra	\N	fpc-rtl-extra-3.0.4_4
2020-07-20 07:17:36+02	2020-07-20 07:18:35+02	math	metis	\N	metis-5.1.0_8
2020-07-20 07:18:36+02	2020-07-20 07:18:54+02	multimedia	libdvdnav	\N	libdvdnav-6.1.0
2020-07-20 07:18:54+02	2020-07-20 07:20:12+02	sysutils	hal	\N	hal-0.5.14_34
2020-07-20 07:20:12+02	2020-07-20 07:20:32+02	textproc	p5-XML-NamespaceSupport	\N	p5-XML-NamespaceSupport-1.12
2020-07-20 07:20:32+02	2020-07-20 07:20:51+02	textproc	p5-XML-SAX-Base	\N	p5-XML-SAX-Base-1.09
2020-07-20 07:20:52+02	2020-07-20 07:22:40+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.71.0
2020-07-20 06:47:39+02	2020-07-20 07:24:51+02	math	openblas	\N	openblas-0.3.9_2,1
2020-07-20 07:24:52+02	2020-07-20 07:25:19+02	audio	libsoxr	\N	libsoxr-0.1.3_2
2020-07-20 07:22:41+02	2020-07-20 07:27:19+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-07-20 07:27:20+02	2020-07-20 07:27:56+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-07-20 07:27:57+02	2020-07-20 07:28:13+02	converters	fpc-iconvenc	\N	fpc-iconvenc-3.0.4_4
2020-07-20 07:25:19+02	2020-07-20 07:28:20+02	audio	webrtc-audio-processing	\N	webrtc-audio-processing-0.3.1_2
2020-07-20 07:28:21+02	2020-07-20 07:28:29+02	devel	libpci	\N	libpci-3.7.0
2020-07-20 07:28:29+02	2020-07-20 07:28:37+02	devel	libsysinfo	\N	libsysinfo-0.0.3_2
2020-07-20 07:28:14+02	2020-07-20 07:29:15+02	devel	libgit2	\N	libgit2-1.0.1
2020-07-20 07:29:15+02	2020-07-20 07:32:31+02	graphics	poppler-glib	\N	poppler-glib-0.90.1
2020-07-20 07:28:38+02	2020-07-20 07:33:16+02	graphics	openexr	\N	openexr-2.5.2
2020-07-20 07:33:16+02	2020-07-20 07:33:23+02	java	java-zoneinfo	\N	java-zoneinfo-2020.a
2020-07-20 07:32:32+02	2020-07-20 07:34:15+02	java	bootstrap-openjdk8	\N	bootstrap-openjdk8-r450802_1
2020-07-20 07:34:16+02	2020-07-20 07:34:22+02	net	avahi-header	\N	avahi-header-0.7
2020-07-20 07:34:23+02	2020-07-20 07:34:46+02	net	p5-Socket6	\N	p5-Socket6-0.29
2020-07-20 07:34:47+02	2020-07-20 07:35:49+02	security	libssh	\N	libssh-0.9.4
2020-07-20 07:35:50+02	2020-07-20 07:36:10+02	textproc	p5-XML-SAX	\N	p5-XML-SAX-1.02
2020-07-20 07:36:11+02	2020-07-20 07:36:23+02	x11	xbitmaps	\N	xbitmaps-1.1.2
2020-07-20 07:36:23+02	2020-07-20 07:40:02+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.71.0
2020-07-20 07:40:02+02	2020-07-20 07:40:21+02	audio	liba52	\N	liba52-0.7.4_3
2020-07-20 07:40:21+02	2020-07-20 07:43:43+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-07-20 07:43:44+02	2020-07-20 07:44:06+02	devel	fpc-fcl-xml	\N	fpc-fcl-xml-3.0.4_4
2020-07-20 07:44:07+02	2020-07-20 07:51:14+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-07-20 07:51:15+02	2020-07-20 07:52:17+02	devel	kf5-kcrash	\N	kf5-kcrash-5.71.0
2020-07-20 07:52:18+02	2020-07-20 07:53:34+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.71.0
2020-07-20 07:53:34+02	2020-07-20 07:53:45+02	devel	tradcpp	\N	tradcpp-0.5.3
2020-07-20 07:53:46+02	2020-07-20 07:57:18+02	graphics	glew	\N	glew-2.2.0
2020-07-20 07:57:18+02	2020-07-20 07:58:38+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-07-20 07:58:39+02	2020-07-20 08:23:09+02	java	openjdk8	\N	openjdk8-8.262.10.1
2020-07-20 08:23:10+02	2020-07-20 08:23:34+02	multimedia	libdca	\N	libdca-0.0.6_1
2020-07-20 08:23:34+02	2020-07-20 08:24:28+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-07-20 08:24:28+02	2020-07-20 08:25:44+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-07-20 08:25:45+02	2020-07-20 08:27:56+02	net	openldap24-sasl-client	\N	openldap-sasl-client-2.4.50
2020-07-20 08:27:57+02	2020-07-20 08:28:16+02	net	p5-IO-Socket-INET6	\N	p5-IO-Socket-INET6-2.72_1
2020-07-20 08:28:17+02	2020-07-20 08:28:36+02	security	p5-Digest-HMAC	\N	p5-Digest-HMAC-1.03_1
2020-07-20 08:28:37+02	2020-07-20 08:29:23+02	security	p5-Net-SSLeay	\N	p5-Net-SSLeay-1.88
2020-07-20 08:29:24+02	2020-07-20 08:30:45+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-20 08:30:46+02	2020-07-20 08:32:02+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.71.0
2020-07-20 08:32:03+02	2020-07-20 08:32:22+02	textproc	p5-XML-SAX-Expat	\N	p5-XML-SAX-Expat-0.51_1
2020-07-20 08:32:23+02	2020-07-20 08:32:42+02	www	p5-HTML-Tagset	\N	p5-HTML-Tagset-3.20_1
2020-07-20 08:32:42+02	2020-07-20 08:33:02+02	www	p5-Mozilla-CA	\N	p5-Mozilla-CA-20180117
2020-07-20 08:33:03+02	2020-07-20 08:34:32+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-07-20 08:34:33+02	2020-07-20 08:34:48+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-07-20 08:34:48+02	2020-07-20 08:35:27+02	x11-fonts	libXfont	\N	libXfont-1.5.4_2,2
2020-07-20 08:35:27+02	2020-07-20 08:35:58+02	audio	libmikmod	\N	libmikmod-3.3.11.1_1
2020-07-20 08:35:58+02	2020-07-20 08:36:14+02	audio	timidity	\N	timidity-0.2i_1
2020-07-20 06:18:33+02	2020-07-20 08:36:25+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-07-20 08:36:14+02	2020-07-20 08:40:15+02	databases	mysql57-client	\N	mysql57-client-5.7.31
2020-07-20 08:40:16+02	2020-07-20 08:40:28+02	devel	gccmakedep	\N	gccmakedep-1.0.3
2020-07-20 08:40:28+02	2020-07-20 08:41:29+02	devel	hwloc	\N	hwloc-1.11.11
2020-07-20 08:41:29+02	2020-07-20 08:43:11+02	devel	kf5-kauth	\N	kf5-kauth-5.71.0
2020-07-20 08:43:12+02	2020-07-20 08:45:48+02	devel	kf5-kservice	\N	kf5-kservice-5.71.0
2020-07-20 08:45:48+02	2020-07-20 08:46:02+02	devel	makedepend	\N	makedepend-1.0.6,1
2020-07-20 08:46:03+02	2020-07-20 08:46:15+02	devel	py-pytz	py27	py27-pytz-2020.1,1
2020-07-20 08:46:16+02	2020-07-20 09:02:10+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-07-20 09:02:11+02	2020-07-20 09:03:23+02	devel	re2	\N	re2-20200401
2020-07-20 09:03:24+02	2020-07-20 09:03:48+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-07-20 09:03:49+02	2020-07-20 09:04:24+02	math	glm	\N	glm-0.9.8.5_4,1
2020-07-20 09:04:24+02	2020-07-20 09:08:38+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-07-20 09:08:38+02	2020-07-20 09:09:23+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-07-20 09:09:23+02	2020-07-20 09:10:43+02	net	remmina	\N	remmina-1.3.9
2020-07-20 08:36:26+02	2020-07-20 09:10:49+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-07-20 09:10:50+02	2020-07-20 09:11:36+02	security	munge	\N	munge-0.5.14_1
2020-07-20 09:11:37+02	2020-07-20 09:11:57+02	security	p5-IO-Socket-SSL	\N	p5-IO-Socket-SSL-2.068
2020-07-20 09:11:57+02	2020-07-20 09:12:12+02	security	pinentry-tty	\N	pinentry-tty-1.1.0
2020-07-20 09:10:44+02	2020-07-20 09:13:20+02	security	gcr	\N	gcr-3.28.0
2020-07-20 09:13:20+02	2020-07-20 09:13:35+02	textproc	man2html	\N	man2html-3.0.1_2
2020-07-20 09:13:36+02	2020-07-20 09:13:51+02	textproc	p5-XML-Simple	\N	p5-XML-Simple-2.25
2020-07-20 09:12:13+02	2020-07-20 09:13:52+02	textproc	aspell	\N	aspell-0.60.8,1
2020-07-20 09:13:51+02	2020-07-20 09:14:06+02	www	p5-HTML-Parser	\N	p5-HTML-Parser-3.72
2020-07-20 07:33:23+02	2020-07-20 09:14:10+02	math	suitesparse	\N	suitesparse-5.8.0_1
2020-07-20 09:14:06+02	2020-07-20 09:14:16+02	x11	xorg-cf-files	\N	xorg-cf-files-1.0.6
2020-07-20 09:13:53+02	2020-07-20 09:14:19+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-07-20 09:14:19+02	2020-07-20 09:15:23+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-07-20 09:14:16+02	2020-07-20 09:15:26+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.71.0
2020-07-20 09:15:24+02	2020-07-20 09:16:00+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-07-20 09:15:26+02	2020-07-20 09:16:04+02	audio	mpg123	\N	mpg123-1.26.3
2020-07-20 09:16:00+02	2020-07-20 09:16:17+02	audio	speex	\N	speex-1.2.0,1
2020-07-20 09:16:05+02	2020-07-20 09:16:21+02	devel	imake	\N	imake-1.0.8,1
2020-07-20 09:16:22+02	2020-07-20 09:16:31+02	devel	npth	\N	npth-1.6
2020-07-20 09:16:18+02	2020-07-20 09:16:56+02	devel	libgtop	\N	libgtop-2.40.0
2020-07-20 09:16:31+02	2020-07-20 09:17:00+02	devel	py-babel	py27	py27-Babel-2.8.0
2020-07-20 09:16:57+02	2020-07-20 09:17:06+02	devel	py-pycparser	py27	py27-pycparser-2.20
2020-07-20 09:14:10+02	2020-07-20 09:17:12+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-07-20 09:17:06+02	2020-07-20 09:17:37+02	graphics	ftgl	\N	ftgl-2.1.3.r5_6,1
2020-07-20 09:17:13+02	2020-07-20 09:17:47+02	graphics	libspiro	\N	libspiro-20190731,1
2020-07-20 09:17:37+02	2020-07-20 09:17:51+02	lang	lua53	\N	lua53-5.3.5_3
2020-07-20 09:17:52+02	2020-07-20 09:18:27+02	security	libksba	\N	libksba-1.4.0
2020-07-20 09:18:27+02	2020-07-20 09:18:35+02	security	pinentry	\N	pinentry-1.1.0_6
2020-07-20 09:17:01+02	2020-07-20 09:19:19+02	graphics	exiv2	\N	exiv2-0.27.3,1
2020-07-20 09:19:19+02	2020-07-20 09:21:00+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-07-20 09:21:00+02	2020-07-20 09:21:10+02	textproc	py-MarkupSafe	py27	py27-MarkupSafe-1.1.1
2020-07-20 09:21:11+02	2020-07-20 09:21:24+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-07-20 09:21:24+02	2020-07-20 09:21:37+02	x11	xrdb	\N	xrdb-1.2.0
2020-07-20 09:21:38+02	2020-07-20 09:22:05+02	x11-fonts	font-misc-misc	\N	font-misc-misc-1.1.2_4
2020-07-20 09:22:06+02	2020-07-20 09:22:22+02	x11-themes	icon-naming-utils	\N	icon-naming-utils-0.8.90_1
2020-07-20 09:22:22+02	2020-07-20 09:24:33+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.71.0
2020-07-20 09:18:37+02	2020-07-20 09:24:49+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-07-20 09:24:49+02	2020-07-20 09:26:04+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.71.0
2020-07-20 09:24:33+02	2020-07-20 09:26:05+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.71.0
2020-07-20 09:26:04+02	2020-07-20 09:27:31+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-07-20 09:27:31+02	2020-07-20 09:27:58+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-07-20 09:27:58+02	2020-07-20 09:28:35+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-07-20 09:28:35+02	2020-07-20 09:29:19+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-07-20 09:29:20+02	2020-07-20 09:31:25+02	devel	apr1	\N	apr-1.7.0.1.6.1_1
2020-07-20 09:31:25+02	2020-07-20 09:31:45+02	devel	p5-TimeDate	\N	p5-TimeDate-2.33,1
2020-07-20 09:31:45+02	2020-07-20 09:31:59+02	devel	py-Jinja2	py27	py27-Jinja2-2.10.1
2020-07-20 09:32:00+02	2020-07-20 09:32:13+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-07-20 09:32:13+02	2020-07-20 09:32:34+02	devel	py-cffi	py27	py27-cffi-1.14.0_1
2020-07-20 09:32:35+02	2020-07-20 09:32:46+02	devel	py-enum34	py27	py27-enum34-1.1.6
2020-07-20 09:32:46+02	2020-07-20 09:32:58+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-07-20 09:32:59+02	2020-07-20 09:33:10+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-07-20 09:33:10+02	2020-07-20 09:33:25+02	devel	yajl	\N	yajl-2.1.0
2020-07-20 09:33:26+02	2020-07-20 09:34:05+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-07-20 09:34:06+02	2020-07-20 09:34:36+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-07-20 09:34:37+02	2020-07-20 09:34:48+02	lang	erlang-man	\N	erlang-man-21.3_2
2020-07-20 09:34:49+02	2020-07-20 09:37:14+02	math	gnuplot	\N	gnuplot-5.2.8_2
2020-07-20 09:37:15+02	2020-07-20 09:38:01+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-20 09:38:01+02	2020-07-20 09:38:40+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-07-20 09:38:41+02	2020-07-20 09:39:03+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-07-20 09:39:03+02	2020-07-20 09:39:52+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-07-20 09:39:53+02	2020-07-20 09:40:26+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-07-20 09:26:06+02	2020-07-20 09:47:32+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-20 09:47:32+02	2020-07-20 09:50:12+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-20 09:50:13+02	2020-07-20 09:52:55+02	security	gnupg	\N	gnupg-2.2.21
2020-07-20 09:52:55+02	2020-07-20 09:53:18+02	security	p5-GSSAPI	\N	p5-GSSAPI-0.28_1
2020-07-20 09:53:18+02	2020-07-20 09:53:32+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-07-20 09:53:32+02	2020-07-20 09:53:58+02	textproc	yelp-xsl	\N	yelp-xsl-3.28.0
2020-07-20 09:53:59+02	2020-07-20 09:54:38+02	x11-fonts	font-adobe-100dpi	\N	font-adobe-100dpi-1.0.3_4
2020-07-20 09:40:27+02	2020-07-20 09:55:48+02	net	openmpi	\N	openmpi-4.0.4
2020-07-20 09:54:39+02	2020-07-20 09:55:58+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.71.0
2020-07-20 09:55:59+02	2020-07-20 09:56:47+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-07-20 09:56:47+02	2020-07-20 09:57:40+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-07-20 09:55:48+02	2020-07-20 09:57:54+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-07-20 09:57:40+02	2020-07-20 09:57:56+02	audio	libcddb	\N	libcddb-1.3.2_4
2020-07-20 09:57:55+02	2020-07-20 10:00:54+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-07-20 10:00:55+02	2020-07-20 10:01:03+02	devel	cvsps	\N	cvsps-2.1_2
2020-07-20 10:01:04+02	2020-07-20 10:01:17+02	devel	popt	\N	popt-1.18_1
2020-07-20 10:01:17+02	2020-07-20 10:01:29+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-07-20 10:01:29+02	2020-07-20 10:06:24+02	graphics	freeimage	\N	freeimage-3.18.0_1
2020-07-20 10:06:25+02	2020-07-20 10:07:35+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-07-20 10:07:36+02	2020-07-20 10:09:29+02	graphics	netpbm	\N	netpbm-10.91.00
2020-07-20 09:57:56+02	2020-07-20 10:14:26+02	databases	firebird25-client	\N	firebird25-client-2.5.8_6
2020-07-20 10:14:26+02	2020-07-20 10:14:42+02	lang	p5-Error	\N	p5-Error-0.17029
2020-07-20 10:09:30+02	2020-07-20 10:36:35+02	lang	erlang	\N	erlang-21.3.8.16,4
2020-07-20 10:36:35+02	2020-07-20 10:36:43+02	multimedia	gmp-api	\N	gmp-api-58
2020-07-20 10:36:43+02	2020-07-20 10:39:50+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-07-20 10:39:51+02	2020-07-20 10:40:31+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-20 10:40:32+02	2020-07-20 10:45:09+02	net	libmateweather	\N	libmateweather-1.24.0
2020-07-20 10:45:10+02	2020-07-20 10:45:34+02	print	gl2ps	\N	gl2ps-1.4.0
2020-07-20 10:45:35+02	2020-07-20 10:46:14+02	security	gpgme	\N	gpgme-1.14.0
2020-07-20 10:46:15+02	2020-07-20 10:47:02+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-07-20 10:47:03+02	2020-07-20 10:47:23+02	security	p5-Authen-SASL	\N	p5-Authen-SASL-2.16_1
2020-07-20 10:47:23+02	2020-07-20 10:51:09+02	textproc	exempi	\N	exempi-2.2.2_1
2020-07-20 10:51:09+02	2020-07-20 10:53:15+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.71.0
2020-07-20 10:53:15+02	2020-07-20 10:53:37+02	www	p5-CGI	\N	p5-CGI-4.50
2020-07-20 10:53:37+02	2020-07-20 10:53:56+02	www	p5-HTTP-Date	\N	p5-HTTP-Date-6.05
2020-07-20 10:53:57+02	2020-07-20 10:54:14+02	x11	luit	\N	luit-20190106_1
2020-07-20 10:54:14+02	2020-07-20 10:54:30+02	x11	xauth	\N	xauth-1.1
2020-07-20 10:54:31+02	2020-07-20 10:55:12+02	x11-fonts	font-adobe-75dpi	\N	font-adobe-75dpi-1.0.3_4
2020-07-20 10:55:13+02	2020-07-20 10:55:23+02	x11-fonts	font-alias	\N	font-alias-1.0.3_3
2020-07-20 10:55:23+02	2020-07-20 10:56:44+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-07-20 10:56:45+02	2020-07-20 10:58:07+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.71.0
2020-07-20 10:58:07+02	2020-07-20 11:00:22+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-07-20 11:00:23+02	2020-07-20 11:10:03+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-20 11:10:03+02	2020-07-20 11:10:25+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.8_6
2020-07-20 11:10:26+02	2020-07-20 11:10:52+02	audio	celt	\N	celt-0.11.3_3
2020-07-20 11:10:53+02	2020-07-20 11:11:20+02	audio	libmad	\N	libmad-0.15.1b_7
2020-07-20 11:11:20+02	2020-07-20 11:11:55+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-07-20 11:11:56+02	2020-07-20 11:19:36+02	cad	ngspice_rework	shlib	ngspice_rework-shlib-32
2020-07-20 11:19:37+02	2020-07-20 11:19:56+02	converters	p5-Encode-Locale	\N	p5-Encode-Locale-1.05
2020-07-20 11:19:57+02	2020-07-20 11:20:16+02	databases	fpc-dblib	\N	fpc-dblib-3.0.4_4
2020-07-20 11:20:17+02	2020-07-20 11:20:36+02	databases	fpc-ibase	\N	fpc-ibase-3.0.4_5
2020-07-20 11:20:36+02	2020-07-20 11:21:14+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-07-20 11:21:14+02	2020-07-20 11:21:30+02	databases	fpc-odbc	\N	fpc-odbc-3.0.4_4
2020-07-20 11:21:31+02	2020-07-20 11:21:48+02	databases	fpc-oracle	\N	fpc-oracle-3.0.4_4
2020-07-20 11:21:49+02	2020-07-20 11:22:07+02	databases	fpc-postgres	\N	fpc-postgres-3.0.4_6
2020-07-20 11:22:08+02	2020-07-20 11:22:25+02	databases	fpc-pxlib	\N	fpc-pxlib-3.0.4_4
2020-07-20 11:22:26+02	2020-07-20 11:22:44+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-07-20 11:22:45+02	2020-07-20 11:23:10+02	devel	autoconf-archive	\N	autoconf-archive-0.2019.01.06
2020-07-20 11:23:11+02	2020-07-20 11:23:27+02	devel	fpc-fcl-async	\N	fpc-fcl-async-3.0.4_4
2020-07-20 11:23:28+02	2020-07-20 11:23:45+02	devel	fpc-fcl-json	\N	fpc-fcl-json-3.0.4_5
2020-07-20 11:23:45+02	2020-07-20 11:24:07+02	devel	fpc-fcl-passrc	\N	fpc-fcl-passrc-3.0.4_5
2020-07-20 11:24:08+02	2020-07-20 11:29:40+02	devel	git	\N	git-2.27.0
2020-07-20 11:29:41+02	2020-07-20 11:30:02+02	devel	jansson	\N	jansson-2.13.1
2020-07-20 11:30:02+02	2020-07-20 11:31:26+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-20 11:31:27+02	2020-07-20 11:32:21+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-07-20 11:32:21+02	2020-07-20 11:33:31+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-07-20 11:33:32+02	2020-07-20 11:33:51+02	devel	netsurf-buildsystem	\N	netsurf-buildsystem-1.8
2020-07-20 11:33:51+02	2020-07-20 11:34:12+02	devel	p5-Clone	\N	p5-Clone-0.45
2020-07-20 11:34:12+02	2020-07-20 11:34:32+02	devel	p5-IO-HTML	\N	p5-IO-HTML-1.001_1
2020-07-20 11:34:33+02	2020-07-20 11:34:45+02	devel	py-asn1crypto	py27	py27-asn1crypto-1.3.0
2020-07-20 11:34:45+02	2020-07-20 11:36:27+02	devel	talloc	\N	talloc-2.3.1
2020-07-20 11:36:28+02	2020-07-20 11:42:46+02	lang	go	\N	go-1.14.6,1
2020-07-20 10:14:42+02	2020-07-20 11:42:46+02	math	vtk8	\N	vtk8-8.2.0
2020-07-20 11:42:47+02	2020-07-20 11:46:06+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-07-20 11:46:06+02	2020-07-20 11:47:05+02	multimedia	openh264	\N	openh264-2.1.1,2
2020-07-20 11:47:05+02	2020-07-20 11:49:54+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-20 11:49:55+02	2020-07-20 12:00:26+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-07-20 12:00:27+02	2020-07-20 12:06:03+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-07-20 12:06:04+02	2020-07-20 12:06:41+02	net	norm	\N	norm-1.5r6_1
2020-07-20 12:06:41+02	2020-07-20 12:07:59+02	net	openpgm	\N	openpgm-5.2.122_6
2020-07-20 12:08:00+02	2020-07-20 12:08:11+02	net	py-ipaddress	py27	py27-ipaddress-1.0.23
2020-07-20 12:08:12+02	2020-07-20 12:08:31+02	security	fpc-openssl	\N	fpc-openssl-3.0.4_4
2020-07-20 12:08:31+02	2020-07-20 12:09:30+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-07-20 12:09:31+02	2020-07-20 12:09:48+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7
2020-07-20 12:09:49+02	2020-07-20 12:10:50+02	sysutils	upower	\N	upower-0.99.11
2020-07-20 12:10:50+02	2020-07-20 12:11:54+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-07-20 12:11:55+02	2020-07-20 12:12:14+02	www	p5-LWP-MediaTypes	\N	p5-LWP-MediaTypes-6.04
2020-07-20 11:42:47+02	2020-07-20 14:02:40+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-07-20 14:02:41+02	2020-07-20 14:04:40+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.71.0
2020-07-20 14:04:41+02	2020-07-20 14:05:19+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-07-20 14:05:20+02	2020-07-20 14:10:09+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-07-20 14:10:10+02	2020-07-20 14:10:24+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-07-20 14:10:25+02	2020-07-20 14:11:21+02	x11	xterm	\N	xterm-358
2020-07-20 14:11:22+02	2020-07-20 14:11:44+02	x11-fonts	font-adobe-utopia-100dpi	\N	font-adobe-utopia-100dpi-1.0.4_4
2020-07-20 14:11:45+02	2020-07-20 14:12:06+02	x11-fonts	font-adobe-utopia-75dpi	\N	font-adobe-utopia-75dpi-1.0.4_4
2020-07-20 14:12:06+02	2020-07-20 14:12:18+02	x11-fonts	font-adobe-utopia-type1	\N	font-adobe-utopia-type1-1.0.4_4
2020-07-20 14:12:18+02	2020-07-20 14:12:29+02	x11-fonts	font-arabic-misc	\N	font-arabic-misc-1.0.3_4
2020-07-20 14:12:30+02	2020-07-20 14:13:02+02	x11-fonts	font-bh-100dpi	\N	font-bh-100dpi-1.0.3_4
2020-07-20 14:13:03+02	2020-07-20 14:13:36+02	x11-fonts	font-bh-75dpi	\N	font-bh-75dpi-1.0.3_4
2020-07-20 14:13:37+02	2020-07-20 14:13:54+02	x11-fonts	font-bh-lucidatypewriter-100dpi	\N	font-bh-lucidatypewriter-100dpi-1.0.3_4
2020-07-20 14:13:55+02	2020-07-20 14:14:13+02	x11-fonts	font-bh-lucidatypewriter-75dpi	\N	font-bh-lucidatypewriter-75dpi-1.0.3_4
2020-07-20 14:14:13+02	2020-07-20 14:14:25+02	x11-fonts	font-bh-type1	\N	font-bh-type1-1.0.3_4
2020-07-20 14:14:26+02	2020-07-20 14:14:37+02	x11-fonts	font-bitstream-100dpi	\N	font-bitstream-100dpi-1.0.3_4
2020-07-20 14:14:38+02	2020-07-20 14:14:49+02	x11-fonts	font-bitstream-75dpi	\N	font-bitstream-75dpi-1.0.3_4
2020-07-20 14:14:50+02	2020-07-20 14:15:02+02	x11-fonts	font-bitstream-type1	\N	font-bitstream-type1-1.0.3_4
2020-07-20 14:15:03+02	2020-07-20 14:15:14+02	x11-fonts	font-cronyx-cyrillic	\N	font-cronyx-cyrillic-1.0.3_4
2020-07-20 14:15:15+02	2020-07-20 14:15:27+02	x11-fonts	font-cursor-misc	\N	font-cursor-misc-1.0.3_4
2020-07-20 14:15:28+02	2020-07-20 14:15:39+02	x11-fonts	font-daewoo-misc	\N	font-daewoo-misc-1.0.3_4
2020-07-20 14:15:40+02	2020-07-20 14:15:52+02	x11-fonts	font-dec-misc	\N	font-dec-misc-1.0.3_4
2020-07-20 14:15:52+02	2020-07-20 14:16:04+02	x11-fonts	font-ibm-type1	\N	font-ibm-type1-1.0.3_4
2020-07-20 14:16:04+02	2020-07-20 14:16:16+02	x11-fonts	font-isas-misc	\N	font-isas-misc-1.0.3_4
2020-07-20 14:16:17+02	2020-07-20 14:16:28+02	x11-fonts	font-jis-misc	\N	font-jis-misc-1.0.3_4
2020-07-20 14:16:29+02	2020-07-20 14:16:40+02	x11-fonts	font-micro-misc	\N	font-micro-misc-1.0.3_4
2020-07-20 14:16:40+02	2020-07-20 14:16:52+02	x11-fonts	font-misc-cyrillic	\N	font-misc-cyrillic-1.0.3_4
2020-07-20 14:16:53+02	2020-07-20 14:17:09+02	x11-fonts	font-mutt-misc	\N	font-mutt-misc-1.0.3_4
2020-07-20 14:17:09+02	2020-07-20 14:17:23+02	x11-fonts	font-schumacher-misc	\N	font-schumacher-misc-1.1.2_4
2020-07-20 14:17:23+02	2020-07-20 14:17:34+02	x11-fonts	font-screen-cyrillic	\N	font-screen-cyrillic-1.0.4_4
2020-07-20 14:17:35+02	2020-07-20 14:17:46+02	x11-fonts	font-sony-misc	\N	font-sony-misc-1.0.3_4
2020-07-20 14:17:46+02	2020-07-20 14:17:58+02	x11-fonts	font-sun-misc	\N	font-sun-misc-1.0.3_4
2020-07-20 14:17:58+02	2020-07-20 14:18:12+02	x11-fonts	font-winitzki-cyrillic	\N	font-winitzki-cyrillic-1.0.3_4
2020-07-20 14:18:13+02	2020-07-20 14:18:24+02	x11-fonts	font-xfree86-type1	\N	font-xfree86-type1-1.0.4_4
2020-07-20 14:18:25+02	2020-07-20 14:21:08+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.71.0
2020-07-20 14:21:08+02	2020-07-20 14:22:56+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.71.0
2020-07-20 14:22:56+02	2020-07-20 14:23:36+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-07-20 14:23:36+02	2020-07-20 14:58:22+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-07-20 14:58:23+02	2020-07-20 14:59:31+02	archivers	gcab	\N	gcab-1.3
2020-07-20 14:59:33+02	2020-07-20 15:00:51+02	audio	jack	\N	jackit-0.125.0_10
2020-07-20 15:00:52+02	2020-07-20 15:02:06+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-07-20 15:02:06+02	2020-07-20 15:02:43+02	audio	libmysofa	\N	libmysofa-1.1
2020-07-20 09:17:48+02	2020-07-20 15:41:38+02	lang	rust	\N	rust-1.45.0
2020-07-20 15:41:40+02	2020-07-20 15:41:51+02	converters	py-webencodings	py27	py27-webencodings-0.5.1
2020-07-20 15:41:52+02	2020-07-20 15:45:01+02	databases	tdb	\N	tdb-1.4.3,1
2020-07-20 15:45:04+02	2020-07-20 15:45:52+02	devel	boehm-gc-threaded	\N	boehm-gc-threaded-8.0.4_1
2020-07-20 15:45:53+02	2020-07-20 15:47:57+02	devel	cppunit	\N	cppunit-1.14.0_8
2020-07-20 15:47:58+02	2020-07-20 15:48:58+02	devel	fpc-fcl-db	\N	fpc-fcl-db-3.0.4_6
2020-07-20 15:48:59+02	2020-07-20 15:49:17+02	devel	fpc-fcl-fpcunit	\N	fpc-fcl-fpcunit-3.0.4_4
2020-07-20 15:49:18+02	2020-07-20 15:49:37+02	devel	fpc-fcl-net	\N	fpc-fcl-net-3.0.4_4
2020-07-20 15:49:37+02	2020-07-20 15:50:07+02	devel	gamin	\N	gamin-0.1.10_10
2020-07-20 15:50:07+02	2020-07-20 15:51:15+02	devel	gflags	\N	gflags-2.2.2_2
2020-07-20 15:51:15+02	2020-07-20 15:51:25+02	devel	gnome-common	\N	gnome-common-3.18.0
2020-07-20 15:51:26+02	2020-07-20 15:53:49+02	devel	kf5-knotifications	\N	kf5-knotifications-5.71.0
2020-07-20 15:53:50+02	2020-07-20 15:54:43+02	devel	librest	\N	rest-0.7.93
2020-07-20 15:54:44+02	2020-07-20 15:55:29+02	devel	m17n-db	\N	m17n-db-1.8.0
2020-07-20 15:55:30+02	2020-07-20 15:55:50+02	devel	p5-Parse-Yapp	\N	p5-Parse-Yapp-1.21
2020-07-20 15:55:50+02	2020-07-20 15:56:05+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-07-20 15:56:05+02	2020-07-20 15:58:57+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-07-20 15:58:58+02	2020-07-20 16:00:44+02	devel	tevent	\N	tevent-0.10.2
2020-07-20 16:00:45+02	2020-07-20 16:01:06+02	graphics	fpc-pasjpeg	\N	fpc-pasjpeg-3.0.4_4
2020-07-20 12:12:14+02	2020-07-20 16:03:25+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-20 16:03:27+02	2020-07-20 16:03:47+02	graphics	libnsgif	\N	libnsgif-0.2.1
2020-07-20 16:01:07+02	2020-07-20 16:06:31+02	graphics	leptonica	\N	leptonica-1.76.0_1
2020-07-20 16:06:32+02	2020-07-20 16:06:51+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-07-20 16:03:48+02	2020-07-20 16:07:16+02	graphics	poppler-utils	\N	poppler-utils-0.90.1
2020-07-20 16:07:17+02	2020-07-20 16:08:59+02	net	libzmq4	\N	libzmq4-4.3.1_1
2020-07-20 16:09:00+02	2020-07-20 16:09:36+02	print	enscript-a4	\N	enscript-a4-1.6.6_3
2020-07-20 16:09:36+02	2020-07-20 16:09:57+02	print	libotf	\N	libotf-0.9.16
2020-07-20 16:09:58+02	2020-07-20 16:11:40+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-10-05 04:16:28+02	2020-10-05 04:18:54+02	cad	cura	\N	Cura-4.7.1,2
2020-07-20 16:11:40+02	2020-07-20 16:12:26+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-07-20 16:12:27+02	2020-07-20 16:13:17+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-20 16:13:18+02	2020-07-20 16:14:05+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-07-20 16:14:05+02	2020-07-20 16:14:27+02	sysutils	cmocka	\N	cmocka-1.1.5
2020-07-20 16:14:28+02	2020-07-20 16:14:38+02	sysutils	libsunacl	\N	libsunacl-1.0.1
2020-07-20 16:14:38+02	2020-07-20 16:16:39+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-07-20 16:16:40+02	2020-07-20 16:16:51+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-07-20 16:16:51+02	2020-07-20 16:17:06+02	www	fpc-fastcgi	\N	fpc-fastcgi-3.0.4_4
2020-07-20 16:17:06+02	2020-07-20 16:17:21+02	www	fpc-httpd22	\N	fpc-httpd22-3.0.4_4
2020-07-20 16:17:22+02	2020-07-20 16:17:36+02	www	fpc-httpd24	\N	fpc-httpd24-3.0.4_4
2020-07-20 16:06:51+02	2020-07-20 16:25:16+02	graphics	tesseract-data	\N	tesseract-data-4.0.0
2020-07-20 16:25:16+02	2020-07-20 16:25:40+02	www	p5-HTTP-Message	\N	p5-HTTP-Message-6.25
2020-07-20 16:25:40+02	2020-07-20 16:25:54+02	x11	appres	\N	appres-1.0.5
2020-07-20 16:25:55+02	2020-07-20 16:26:58+02	x11	babl	\N	babl-0.1.78
2020-07-20 16:26:59+02	2020-07-20 16:27:20+02	x11	bitmap	\N	bitmap-1.0.9
2020-07-20 16:27:20+02	2020-07-20 16:27:41+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-07-20 16:27:41+02	2020-07-20 16:27:56+02	x11	iceauth	\N	iceauth-1.0.8_2
2020-07-20 16:27:56+02	2020-07-20 16:29:30+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-07-20 16:29:30+02	2020-07-20 16:29:48+02	x11	sessreg	\N	sessreg-1.1.2
2020-07-20 16:29:49+02	2020-07-20 16:30:04+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-07-20 16:30:05+02	2020-07-20 16:30:19+02	x11	smproxy	\N	smproxy-1.0.6
2020-07-20 16:30:19+02	2020-07-20 16:30:38+02	x11	x11perf	\N	x11perf-1.6.1
2020-07-20 16:30:38+02	2020-07-20 16:30:52+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-07-20 16:30:53+02	2020-07-20 16:31:20+02	x11	xcalc	\N	xcalc-1.1.0
2020-07-20 16:31:21+02	2020-07-20 16:31:36+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-07-20 16:31:36+02	2020-07-20 16:31:52+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-07-20 16:31:52+02	2020-07-20 16:32:08+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-07-20 16:32:09+02	2020-07-20 16:32:24+02	x11	xev	\N	xev-1.2.4
2020-07-20 16:32:25+02	2020-07-20 16:32:37+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-07-20 16:32:38+02	2020-07-20 16:32:51+02	x11	xgamma	\N	xgamma-1.0.6
2020-07-20 16:32:52+02	2020-07-20 16:33:11+02	x11	xgc	\N	xgc-1.0.5
2020-07-20 16:33:12+02	2020-07-20 16:33:26+02	x11	xhost	\N	xhost-1.0.8
2020-07-20 16:33:26+02	2020-07-20 16:33:38+02	x11	xinit	\N	xinit-1.4.1,1
2020-07-20 16:33:38+02	2020-07-20 16:33:51+02	x11	xinput	\N	xinput-1.6.3
2020-07-20 16:33:52+02	2020-07-20 16:34:04+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-07-20 16:34:05+02	2020-07-20 16:34:18+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-07-20 16:34:19+02	2020-07-20 16:34:34+02	x11	xkill	\N	xkill-1.0.5
2020-07-20 16:34:34+02	2020-07-20 16:34:45+02	x11	xlsatoms	\N	xlsatoms-1.1.3
2020-07-20 16:34:46+02	2020-07-20 16:34:57+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-07-20 16:34:58+02	2020-07-20 16:35:10+02	x11	xmessage	\N	xmessage-1.0.5
2020-07-20 16:35:11+02	2020-07-20 16:35:23+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-07-20 16:35:24+02	2020-07-20 16:35:37+02	x11	xpr	\N	xpr-1.0.5
2020-07-20 16:35:38+02	2020-07-20 16:35:55+02	x11	xrandr	\N	xrandr-1.5.1
2020-07-20 16:35:55+02	2020-07-20 16:36:09+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-07-20 16:36:10+02	2020-07-20 16:36:26+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-07-20 16:36:26+02	2020-07-20 16:36:42+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-07-20 16:36:42+02	2020-07-20 16:36:55+02	x11	xwd	\N	xwd-1.0.7
2020-07-20 16:36:55+02	2020-07-20 16:37:08+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-07-20 16:37:09+02	2020-07-20 16:37:20+02	x11	xwud	\N	xwud-1.0.5
2020-07-20 16:37:21+02	2020-07-20 16:37:35+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-07-20 15:02:43+02	2020-07-20 16:38:12+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-20 16:37:36+02	2020-07-20 16:38:41+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-20 16:38:13+02	2020-07-20 16:39:07+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-07-20 16:38:41+02	2020-07-20 16:39:37+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-07-20 16:39:08+02	2020-07-20 16:40:02+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-07-20 16:39:37+02	2020-07-20 16:40:39+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-07-20 16:40:40+02	2020-07-20 16:41:37+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-07-20 16:40:02+02	2020-07-20 16:42:20+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-07-20 16:41:38+02	2020-07-20 16:42:40+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-07-20 16:42:41+02	2020-07-20 16:42:56+02	x11-fonts	libFS	\N	libFS-1.0.8
2020-07-20 16:42:57+02	2020-07-20 16:43:29+02	x11-fonts	xorg-fonts-100dpi	\N	xorg-fonts-100dpi-7.7
2020-07-20 16:43:29+02	2020-07-20 16:44:02+02	x11-fonts	xorg-fonts-75dpi	\N	xorg-fonts-75dpi-7.7
2020-07-20 16:44:03+02	2020-07-20 16:44:20+02	x11-fonts	xorg-fonts-cyrillic	\N	xorg-fonts-cyrillic-7.7
2020-07-20 16:44:20+02	2020-07-20 16:45:06+02	x11-fonts	xorg-fonts-miscbitmaps	\N	xorg-fonts-miscbitmaps-7.7
2020-07-20 16:45:07+02	2020-07-20 16:45:25+02	x11-fonts	xorg-fonts-type1	\N	xorg-fonts-type1-7.7
2020-07-20 16:42:21+02	2020-07-20 16:45:25+02	x11-fm	caja	\N	caja-1.22.3_1
2020-07-20 16:45:26+02	2020-07-20 16:46:21+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-07-20 16:45:25+02	2020-07-20 16:47:47+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.71.0
2020-07-20 16:46:22+02	2020-07-20 16:47:49+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-07-20 16:47:48+02	2020-07-20 16:48:05+02	x11-wm	twm	\N	twm-1.0.11
2020-07-20 16:48:05+02	2020-07-20 16:48:19+02	archivers	unzip	\N	unzip-6.0_8
2020-07-20 16:47:49+02	2020-07-20 16:48:21+02	archivers	minizip	\N	minizip-1.2.11
2020-07-20 16:48:20+02	2020-07-20 16:48:30+02	audio	gsm	\N	gsm-1.0.13_2
2020-07-20 16:48:31+02	2020-07-20 16:48:45+02	audio	libao	\N	libao-1.2.0_5
2020-07-20 16:48:46+02	2020-07-20 16:49:02+02	audio	libid3tag	\N	libid3tag-0.15.1b_2
2020-07-20 16:48:21+02	2020-07-20 16:49:03+02	audio	lame	\N	lame-3.100_2
2020-07-20 16:49:03+02	2020-07-20 16:49:53+02	audio	opusfile	\N	opusfile-0.10
2020-07-20 16:49:03+02	2020-07-20 16:50:29+02	audio	openal-soft	\N	openal-soft-1.20.1_2
2020-07-20 16:50:29+02	2020-07-20 16:50:45+02	devel	fpc-fcl-image	\N	fpc-fcl-image-3.0.4_4
2020-07-20 16:49:53+02	2020-07-20 16:51:23+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-07-20 16:50:46+02	2020-07-20 16:51:27+02	devel	fpc-fcl-web	\N	fpc-fcl-web-3.0.4_7
2020-07-20 16:51:23+02	2020-07-20 16:52:46+02	devel	glog	\N	glog-0.4.0_1
2020-07-20 16:52:46+02	2020-07-20 16:54:35+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.71.0
2020-07-20 16:51:28+02	2020-07-20 16:54:39+02	devel	glslang	\N	glslang-8.13.3743
2020-07-20 16:54:40+02	2020-07-20 16:55:05+02	devel	liblangtag	\N	liblangtag-0.6.2
2020-07-20 16:55:05+02	2020-07-20 16:57:25+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-07-20 16:57:25+02	2020-07-20 16:57:34+02	devel	mdds	\N	mdds-1.5.0
2020-07-20 16:54:36+02	2020-07-20 16:57:38+02	devel	kf5-solid	\N	kf5-solid-5.71.0
2020-07-20 16:57:34+02	2020-07-20 16:57:54+02	devel	p5-IO-String	\N	p5-IO-String-1.08_1
2020-07-20 16:57:39+02	2020-07-20 16:57:54+02	devel	py-click	py37	py37-click-7.0
2020-07-20 16:57:54+02	2020-07-20 16:58:05+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-07-20 16:57:55+02	2020-07-20 16:58:06+02	devel	py-olefile	py37	py37-olefile-0.46
2020-07-20 16:58:06+02	2020-07-20 16:58:17+02	devel	py-ply	py27	py27-ply-3.11
2020-07-20 16:58:18+02	2020-07-20 16:58:29+02	devel	py-zipp	py37	py37-zipp-3.1.0
2020-07-20 16:58:18+02	2020-07-20 16:58:31+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-07-20 16:58:31+02	2020-07-20 16:58:46+02	devel	scons	py37	scons-py37-3.1.2
2020-07-20 16:58:29+02	2020-07-20 17:14:01+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-07-20 17:14:02+02	2020-07-20 17:14:40+02	devel	tex-libtexlua	\N	tex-libtexlua-5.2.4
2020-07-20 17:14:40+02	2020-07-20 17:16:12+02	devel	tex-libtexluajit	\N	tex-libtexluajit-2.0.3
2020-10-05 06:22:52+02	2020-10-05 06:24:56+02	misc	mc	\N	mc-nox11-4.8.24
2020-07-20 17:16:13+02	2020-07-20 17:17:04+02	devel	yaml-cpp03	\N	yaml-cpp03-0.3.0_2
2020-07-20 16:58:47+02	2020-07-20 17:17:57+02	devel	spirv-tools	\N	spirv-tools-2020.2
2020-07-20 17:17:58+02	2020-07-20 17:19:57+02	graphics	libcdr01	\N	libcdr01-0.1.6
2020-07-20 17:19:58+02	2020-07-20 17:20:26+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-07-20 17:17:04+02	2020-07-20 17:20:49+02	graphics	gegl	\N	gegl-0.4.24_2
2020-07-20 17:20:27+02	2020-07-20 17:21:06+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-07-20 17:20:50+02	2020-07-20 17:21:13+02	graphics	mypaint-brushes	\N	mypaint-brushes-1.3.1
2020-07-20 17:21:07+02	2020-07-20 17:21:30+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-07-20 17:21:30+02	2020-07-20 17:25:19+02	mail	mailutils	\N	mailutils-3.9
2020-07-20 17:25:19+02	2020-07-20 17:25:29+02	misc	compat12x	\N	compat12x-amd64-12.1.1201000.20200220
2020-07-20 17:25:30+02	2020-07-20 17:25:50+02	misc	fpc-chm	\N	fpc-chm-3.0.4_4
2020-07-20 17:25:51+02	2020-07-20 17:26:02+02	misc	usbids	\N	usbids-20200622
2020-07-20 17:26:02+02	2020-07-20 17:26:14+02	multimedia	librtmp	\N	librtmp-2.4.20190330
2020-07-20 17:26:15+02	2020-07-20 17:29:33+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-07-20 17:29:34+02	2020-07-20 17:29:52+02	net	liboauth	\N	liboauth-1.0.3_4
2020-07-20 17:21:13+02	2020-07-20 17:33:41+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-07-20 17:33:42+02	2020-07-20 17:33:58+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-07-20 17:33:59+02	2020-07-20 17:35:13+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-07-20 17:35:14+02	2020-07-20 17:36:41+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-20 17:36:42+02	2020-07-20 17:41:06+02	print	qpdf	\N	qpdf-10.0.1
2020-07-20 16:17:36+02	2020-07-20 17:43:18+02	www	node	\N	node-14.5.0
2020-07-20 17:41:07+02	2020-07-20 17:43:42+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-07-20 17:43:19+02	2020-07-20 17:44:08+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-07-20 17:43:42+02	2020-07-20 17:44:21+02	sysutils	flock	\N	flock-2.30.1
2020-07-20 17:44:08+02	2020-07-20 17:44:49+02	sysutils	fusefs-libs	\N	fusefs-libs-2.9.9_1
2020-07-20 17:44:49+02	2020-07-20 17:45:36+02	sysutils	libcdio	\N	libcdio-2.1.0
2020-07-20 17:44:21+02	2020-07-20 17:46:26+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.71.0
2020-07-20 17:45:37+02	2020-07-20 17:46:48+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-07-20 17:46:27+02	2020-07-20 17:47:13+02	textproc	libuninameslist	\N	libuninameslist-20190701
2020-07-20 17:47:13+02	2020-07-20 17:47:25+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-07-20 17:47:26+02	2020-07-20 17:47:45+02	textproc	tinyxml	\N	tinyxml-2.6.2_4
2020-07-20 17:47:46+02	2020-07-20 17:47:55+02	textproc	utf8proc	\N	utf8proc-2.5.0
2020-07-20 17:47:56+02	2020-07-20 17:48:24+02	textproc	yelp-tools	\N	yelp-tools-3.28.0
2020-07-20 17:48:25+02	2020-07-20 17:48:39+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-07-20 17:48:39+02	2020-07-20 17:48:51+02	www	py-html5lib	py27	py27-html5lib-1.0.1
2020-07-20 17:48:52+02	2020-07-20 17:49:02+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-07-20 17:46:48+02	2020-07-20 17:49:07+02	textproc	libvisio01	\N	libvisio01-0.1.7_2
2020-07-20 17:49:08+02	2020-07-20 17:49:24+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-07-20 17:49:25+02	2020-07-20 17:49:33+02	x11	xorg-docs	\N	xorg-docs-1.7.1,1
2020-07-20 17:49:33+02	2020-07-20 17:49:42+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-07-20 17:49:02+02	2020-07-20 17:49:52+02	x11	kf5-kded	\N	kf5-kded-5.71.0_1
2020-07-20 17:49:52+02	2020-07-20 17:51:23+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-20 17:49:43+02	2020-07-20 17:51:30+02	x11	zenity	\N	zenity-3.32.0
2020-07-20 17:51:24+02	2020-07-20 17:51:30+02	x11-fonts	gnu-unifont	\N	gnu-unifont-13.0.03,1
2020-07-20 17:29:53+02	2020-07-20 17:52:09+02	net	samba410	\N	samba410-4.10.17
2020-07-20 17:51:30+02	2020-07-20 17:52:11+02	x11-fonts	noto-basic	\N	noto-basic-2.0
2020-07-20 17:52:11+02	2020-07-20 17:52:22+02	x11-themes	xcursor-themes	\N	xcursor-themes-1.0.6
2020-07-20 17:51:31+02	2020-07-20 17:53:20+02	x11-fonts	xorg-fonts	\N	xorg-fonts-7.7_1
2020-07-20 17:52:09+02	2020-07-20 17:54:40+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-07-20 17:53:21+02	2020-07-20 17:54:41+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.71.0
2020-07-20 17:54:41+02	2020-07-20 17:55:07+02	audio	libmodplug	\N	libmodplug-0.8.9.0
2020-07-20 17:54:40+02	2020-07-20 17:55:43+02	archivers	gtar	\N	gtar-1.32
2020-07-20 17:55:43+02	2020-07-20 17:55:55+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-07-20 17:55:56+02	2020-07-20 17:56:09+02	databases	py-redis	py37	py37-redis-3.5.3
2020-07-20 17:55:07+02	2020-07-20 17:56:48+02	audio	sox	\N	sox-14.4.2_4
2020-07-20 17:56:10+02	2020-07-20 17:56:59+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-07-20 17:56:59+02	2020-07-20 17:57:13+02	devel	fpc-regexpr	\N	fpc-regexpr-3.0.4_4
2020-07-20 17:56:48+02	2020-07-20 17:57:22+02	devel	fpc-fppkg	\N	fpc-fppkg-3.0.4_6
2020-07-20 17:57:23+02	2020-07-20 18:01:01+02	devel	libgdata	\N	libgdata-0.17.9
2020-07-20 17:52:22+02	2020-07-20 18:01:16+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-07-20 18:01:01+02	2020-07-20 18:02:17+02	devel	libpeas	\N	libpeas-1.22.0
2020-07-20 18:02:18+02	2020-07-20 18:02:41+02	devel	physfs	\N	physfs-3.0.2
2020-07-20 18:02:41+02	2020-07-20 18:02:54+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-07-20 18:02:55+02	2020-07-20 18:03:08+02	devel	py-appdirs	py37	py37-appdirs-1.4.3
2020-07-20 18:03:09+02	2020-07-20 18:03:20+02	devel	py-backports	py37	py37-backports-1
2020-07-20 18:03:21+02	2020-07-20 18:03:33+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-07-20 18:03:33+02	2020-07-20 18:03:50+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-07-20 18:03:50+02	2020-07-20 18:04:04+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-07-20 18:04:05+02	2020-07-20 18:04:52+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-07-20 18:04:53+02	2020-07-20 18:05:10+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-0.23
2020-07-20 18:05:10+02	2020-07-20 18:05:23+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-07-20 18:05:24+02	2020-07-20 18:05:42+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-07-20 18:05:43+02	2020-07-20 18:06:04+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-07-20 18:06:04+02	2020-07-20 18:06:27+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-07-20 18:06:27+02	2020-07-20 18:06:41+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-07-20 18:06:42+02	2020-07-20 18:06:55+02	devel	py-wheel	py27	py27-wheel-0.30.0_1
2020-07-20 18:06:56+02	2020-07-20 18:07:13+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-07-20 18:07:14+02	2020-07-20 18:07:28+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-07-20 17:57:14+02	2020-07-20 18:08:19+02	devel	kf5-kio	\N	kf5-kio-5.71.0
2020-07-20 18:07:29+02	2020-07-20 18:11:57+02	dns	bind-tools	\N	bind-tools-9.16.5
2020-07-20 18:11:58+02	2020-07-20 18:12:16+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-07-20 18:12:17+02	2020-07-20 18:12:38+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-07-20 18:08:20+02	2020-07-20 18:14:19+02	editors	emacs	full	emacs-26.3_5,3
2020-07-20 18:14:20+02	2020-07-20 18:15:33+02	graphics	glfw	\N	glfw-3.3.2
2020-07-20 18:15:34+02	2020-07-20 18:16:35+02	graphics	libfreehand	\N	libfreehand-0.1.2_18
2020-07-20 18:16:35+02	2020-07-20 18:18:06+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-07-20 18:18:07+02	2020-07-20 18:21:57+02	graphics	movit	\N	movit-1.6.2_2
2020-07-20 18:21:57+02	2020-07-20 18:26:37+02	graphics	opencolorio	\N	opencolorio-1.1.1_1
2020-07-20 18:12:39+02	2020-07-20 18:32:11+02	graphics	gimp-app	\N	gimp-app-2.10.20_2,1
2020-07-20 18:32:12+02	2020-07-20 18:32:29+02	graphics	potrace	\N	potrace-1.16
2020-07-20 18:32:30+02	2020-07-20 18:33:44+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-07-20 18:33:44+02	2020-07-20 18:34:32+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-07-20 18:34:33+02	2020-07-20 18:35:31+02	graphics	shaderc	\N	shaderc-2020.0
2020-07-20 18:35:32+02	2020-07-20 18:36:03+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-07-21 02:13:40+02	2020-07-21 02:14:22+02	devel	ocl-icd	\N	ocl-icd-2.2.12
2020-07-20 18:36:04+02	2020-07-20 18:39:48+02	java	bootstrap-openjdk11	\N	bootstrap-openjdk11-11.0.5.10.1
2020-07-20 18:39:49+02	2020-07-20 18:42:55+02	lang	cython	py27	py27-cython-0.29.15
2020-07-20 18:26:38+02	2020-07-20 19:05:06+02	graphics	opencv	\N	opencv-3.4.1_35
2020-07-20 18:42:55+02	2020-07-20 19:14:39+02	lang	gcc6-aux	\N	gcc6-aux-20180516_1,1
2020-07-20 19:14:40+02	2020-07-20 19:16:41+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-20 19:16:42+02	2020-07-20 19:17:41+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0
2020-07-20 19:17:41+02	2020-07-20 19:33:26+02	multimedia	assimp	\N	assimp-5.0.1
2020-07-20 19:33:27+02	2020-07-20 19:34:56+02	multimedia	gavl	\N	gavl-1.4.0_4
2020-07-20 19:34:57+02	2020-07-20 19:38:34+02	multimedia	libde265	\N	libde265-1.0.2_5
2020-07-20 19:38:34+02	2020-07-20 19:38:44+02	multimedia	rtmpdump	\N	rtmpdump-2.4.20190330
2020-07-20 19:38:44+02	2020-07-20 19:39:03+02	multimedia	vid.stab	\N	vid.stab-0.98.2
2020-07-20 19:39:04+02	2020-07-20 19:39:27+02	net	gssdp	\N	gssdp-1.2.3
2020-07-20 19:39:28+02	2020-07-20 19:40:11+02	net	libvncserver	\N	libvncserver-0.9.12_2
2020-07-20 19:40:11+02	2020-07-20 19:41:07+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-20 19:41:07+02	2020-07-20 19:41:30+02	print	adobe-cmaps	\N	adobe-cmaps-20051217_4
2020-07-20 19:41:31+02	2020-07-20 19:43:16+02	print	cups-filters	\N	cups-filters-1.27.5_1
2020-07-20 19:43:16+02	2020-07-20 19:55:00+02	print	fontforge	\N	fontforge-20190801_2
2020-07-20 19:55:01+02	2020-07-20 19:56:06+02	print	libmspub01	\N	libmspub01-0.1.4_16
2020-07-20 19:56:07+02	2020-07-20 19:56:53+02	print	libpagemaker	\N	libpagemaker-0.0.4_10
2020-07-20 19:56:54+02	2020-07-20 19:57:08+02	print	t1utils	\N	t1utils-1.32
2020-07-20 19:57:08+02	2020-07-20 20:18:12+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-07-20 20:18:13+02	2020-07-20 20:43:59+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-07-20 20:44:00+02	2020-07-20 20:44:39+02	security	mbedtls	\N	mbedtls-2.16.7
2020-07-20 20:44:40+02	2020-07-20 20:44:55+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-07-20 20:44:56+02	2020-07-20 20:46:12+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-07-20 20:46:13+02	2020-07-20 20:46:54+02	sysutils	libcdio-paranoia	\N	libcdio-paranoia-10.2+2.0.1
2020-07-20 20:46:54+02	2020-07-20 20:47:13+02	sysutils	lsof	\N	lsof-4.93.2_12,8
2020-07-20 20:47:15+02	2020-07-20 20:49:58+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-07-20 20:49:59+02	2020-07-20 20:51:37+02	textproc	UCD	\N	UCD-13.0.0
2020-07-20 20:51:37+02	2020-07-20 20:52:08+02	textproc	libebml	\N	libebml-1.4.0
2020-07-20 20:52:09+02	2020-07-20 20:52:36+02	textproc	mythes	\N	mythes-1.2.4_7
2020-07-20 20:52:37+02	2020-07-20 20:52:54+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-07-20 20:52:54+02	2020-07-20 20:53:08+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-07-20 20:53:09+02	2020-07-20 20:53:24+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-07-20 20:53:25+02	2020-07-20 21:03:45+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-07-20 21:03:46+02	2020-07-20 21:04:42+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-07-20 21:04:42+02	2020-07-20 21:08:19+02	www	apache24	\N	apache24-2.4.43
2020-07-20 19:05:07+02	2020-07-20 21:24:26+02	lang	ghc	\N	ghc-8.8.3_3
2020-07-20 21:24:27+02	2020-07-20 22:52:24+02	www	node12	\N	node12-12.18.2_1
2020-07-20 22:52:25+02	2020-07-20 22:52:39+02	www	py-h2	py37	py37-h2-3.2.0
2020-07-20 22:52:40+02	2020-07-20 22:52:53+02	www	py-hyperlink	py37	py37-hyperlink-19.0.0
2020-07-20 22:52:53+02	2020-07-20 22:53:07+02	www	py-priority	py37	py37-priority-1.3.0
2020-07-20 22:53:08+02	2020-07-20 22:53:26+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-07-20 18:01:16+02	2020-07-21 00:05:24+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-07-21 00:05:26+02	2020-07-21 00:05:56+02	www	serf	\N	serf-1.3.9_5
2020-07-21 00:05:56+02	2020-07-21 00:07:33+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-07-21 00:07:34+02	2020-07-21 00:10:52+02	x11	xorg	\N	xorg-7.7_3
2020-07-21 00:10:53+02	2020-07-21 00:12:53+02	x11	yelp	\N	yelp-3.28.1_2
2020-07-21 00:12:54+02	2020-07-21 00:13:41+02	x11-fonts	cantarell-fonts	\N	cantarell-fonts-0.101_1
2020-07-21 00:13:42+02	2020-07-21 00:14:04+02	x11-fonts	p5-Font-TTF	\N	p5-Font-TTF-1.06
2020-07-21 00:14:04+02	2020-07-21 00:15:29+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-07-21 00:15:30+02	2020-07-21 00:16:31+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-07-21 00:16:31+02	2020-07-21 00:18:42+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-07-21 00:18:43+02	2020-07-21 00:30:58+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-07-21 00:30:58+02	2020-07-21 00:35:51+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-07-21 00:35:52+02	2020-07-21 00:49:07+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-07-21 00:49:07+02	2020-07-21 00:50:25+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-07-21 00:50:26+02	2020-07-21 00:51:37+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-07-21 00:51:38+02	2020-07-21 00:54:36+02	x11-wm	marco	\N	marco-1.22.4
2020-07-21 00:54:36+02	2020-07-21 00:54:58+02	archivers	c-blosc	\N	c-blosc-1.15.0_2
2020-07-21 00:54:58+02	2020-07-21 00:56:06+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-07-21 00:56:06+02	2020-07-21 00:56:26+02	archivers	p5-Archive-Zip	\N	p5-Archive-Zip-1.68
2020-07-21 00:56:27+02	2020-07-21 00:57:46+02	audio	faad	\N	faad2-2.8.8,1
2020-07-21 00:57:46+02	2020-07-21 00:58:07+02	audio	freealut	\N	freealut-1.1.0_4
2020-07-21 00:58:08+02	2020-07-21 00:58:22+02	audio	libdssialsacompat	\N	libdssialsacompat-1.0.8a_2
2020-07-21 00:58:23+02	2020-07-21 00:59:03+02	audio	liblo	\N	liblo-0.31_1
2020-07-21 00:59:03+02	2020-07-21 01:00:50+02	audio	libopenshot-audio	\N	libopenshot-audio-0.2.0.10
2020-07-21 01:00:51+02	2020-07-21 01:01:31+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-07-21 01:01:32+02	2020-07-21 01:03:35+02	audio	taglib	\N	taglib-1.12.b.1
2020-07-21 01:03:35+02	2020-07-21 01:03:58+02	audio	twolame	\N	twolame-0.4.0
2020-07-21 01:03:59+02	2020-07-21 01:04:32+02	databases	p5-DBI	\N	p5-DBI-1.643
2020-07-21 01:04:32+02	2020-07-21 01:06:21+02	databases	unixODBC	\N	unixODBC-2.3.7
2020-07-21 01:06:21+02	2020-07-21 01:08:49+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-07-21 01:08:50+02	2020-07-21 01:10:10+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-07-21 01:10:11+02	2020-07-21 01:14:59+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-07-21 01:14:59+02	2020-07-21 01:17:26+02	devel	binutils	aarch64	aarch64-binutils-2.33.1_2,1
2020-07-21 01:17:27+02	2020-07-21 01:19:35+02	devel	binutils	mips64	mips64-binutils-2.33.1_2,1
2020-07-21 01:19:36+02	2020-07-21 01:21:22+02	devel	binutils	powerpc64	powerpc64-binutils-2.33.1_2,1
2020-07-21 01:21:23+02	2020-07-21 01:22:58+02	devel	binutils	riscv64	riscv64-binutils-2.33.1_2,1
2020-07-21 01:22:59+02	2020-07-21 01:24:35+02	devel	binutils	sparc64	sparc64-binutils-2.33.1_2,1
2020-07-21 01:24:36+02	2020-07-21 01:30:50+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-07-21 01:30:51+02	2020-07-21 01:30:59+02	devel	cl-asdf	\N	cl-asdf-3.3.1.7,2
2020-07-21 01:31:00+02	2020-07-21 01:31:21+02	devel	cscope	\N	cscope-15.9
2020-07-21 01:31:21+02	2020-07-21 01:31:54+02	devel	dmake	\N	dmake-4.12.3,1
2020-07-21 01:31:55+02	2020-07-21 01:32:12+02	devel	fpc-fcl-registry	\N	fpc-fcl-registry-3.0.4_4
2020-07-21 01:32:12+02	2020-07-21 01:35:48+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-21 01:35:49+02	2020-07-21 01:50:41+02	devel	hs-cabal-install	\N	hs-cabal-install-3.0.2.0_1
2020-07-21 01:50:42+02	2020-07-21 01:54:18+02	devel	kBuild	\N	kBuild-0.1.9998_13
2020-07-21 01:54:19+02	2020-07-21 01:55:08+02	devel	libfmt	\N	libfmt-6.2.1
2020-07-21 01:55:08+02	2020-07-21 02:02:31+02	devel	liborcus	\N	liborcus-0.15.4
2020-07-21 02:02:32+02	2020-07-21 02:02:47+02	devel	libsigsegv	\N	libsigsegv-2.12
2020-07-21 02:02:48+02	2020-07-21 02:06:01+02	devel	libslang2	\N	libslang2-2.3.2_2
2020-07-21 02:06:02+02	2020-07-21 02:13:39+02	devel	ncurses	\N	ncurses-6.1.20190525
2020-07-25 12:20:15+02	2020-07-25 12:21:57+02	security	gcr	\N	gcr-3.28.0
2020-07-21 02:14:23+02	2020-07-21 02:14:41+02	devel	p5-Getopt-ArgvFile	\N	p5-Getopt-ArgvFile-1.11_1
2020-07-21 02:14:42+02	2020-07-21 02:14:54+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-07-21 02:14:55+02	2020-07-21 02:15:10+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-07-21 02:15:11+02	2020-07-21 02:15:23+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-07-21 02:15:24+02	2020-07-21 02:15:37+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-07-21 02:15:37+02	2020-07-21 02:15:51+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-07-21 02:15:52+02	2020-07-21 02:16:06+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-07-21 02:16:07+02	2020-07-21 02:16:19+02	devel	py-pyasn1	py27	py27-pyasn1-0.4.7
2020-07-21 02:16:19+02	2020-07-21 02:17:18+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-07-21 02:17:19+02	2020-07-21 02:17:51+02	devel	pydbus-common	\N	pydbus-common-1.2.16
2020-07-21 02:17:52+02	2020-07-21 02:18:12+02	devel	rebar	\N	rebar-2.6.4,1
2020-07-21 02:18:12+02	2020-07-21 02:19:28+02	devel	rebar3	\N	rebar3-3.13.2,1
2020-07-21 02:19:29+02	2020-07-21 02:19:36+02	devel	robin-map	\N	robin-map-0.6.2
2020-07-21 02:19:37+02	2020-07-21 02:19:47+02	devel	ucpp	\N	ucpp-1.3.2,1
2020-07-21 02:19:48+02	2020-07-21 02:20:48+02	devel	unittest-cpp	\N	unittest-cpp-2.0.0
2020-07-21 02:20:48+02	2020-07-21 02:23:29+02	dns	ldns	\N	ldns-1.7.1_1
2020-07-21 02:23:30+02	2020-07-21 02:23:42+02	dns	py-idna	py27	py27-idna-2.8
2020-07-21 02:23:42+02	2020-07-21 02:33:08+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-07-21 02:33:09+02	2020-07-21 03:01:12+02	graphics	dcmtk	\N	dcmtk-3.6.5_2
2020-07-21 03:01:13+02	2020-07-21 03:03:22+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-07-21 03:03:23+02	2020-07-21 03:03:48+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-07-21 03:03:49+02	2020-07-21 03:04:07+02	graphics	frei0r-plugins-gavl	\N	frei0r-plugins-gavl-1.6.1
2020-07-21 03:04:07+02	2020-07-21 03:06:21+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-21 03:06:22+02	2020-07-21 03:18:30+02	graphics	libetonyek01	\N	libetonyek01-0.1.9_6,1
2020-07-21 03:18:31+02	2020-07-21 03:20:03+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-07-21 03:20:04+02	2020-07-21 03:20:42+02	graphics	libgxps	\N	libgxps-0.3.1
2020-07-21 03:20:43+02	2020-07-21 03:23:05+02	graphics	libheif	\N	libheif-1.6.2
2020-07-21 03:23:06+02	2020-07-21 03:23:43+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-07-21 03:23:44+02	2020-07-21 03:24:52+02	graphics	libzmf	\N	libzmf-0.0.2_21
2020-07-21 03:24:53+02	2020-07-21 03:31:43+02	graphics	podofo	\N	podofo-0.9.6_1
2020-07-21 03:31:45+02	2020-07-21 03:38:58+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.1
2020-07-21 03:38:59+02	2020-07-21 03:39:37+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-07-21 03:39:37+02	2020-07-21 03:40:47+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-21 03:40:48+02	2020-07-21 03:41:07+02	graphics	qr-code-generator	\N	qr-code-generator-1.6.0
2020-07-21 03:41:08+02	2020-07-21 04:14:57+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-07-21 04:14:58+02	2020-07-21 04:16:09+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-07-21 04:16:10+02	2020-07-21 04:21:17+02	graphics	sane-backends	\N	sane-backends-1.0.30
2020-07-21 04:21:18+02	2020-07-21 04:21:37+02	graphics	squish	\N	squish-1.10_2
2020-07-21 04:21:38+02	2020-07-21 04:23:04+02	graphics	vigra	\N	vigra-1.11.1_11
2020-07-21 04:23:05+02	2020-07-21 04:23:24+02	lang	fpc-rtl-console	\N	fpc-rtl-console-3.0.4_4
2020-07-21 04:23:24+02	2020-07-21 04:26:33+02	lang	fpc-source	\N	fpc-source-3.0.4_2
2020-07-21 04:26:33+02	2020-07-21 04:27:55+02	lang	fpc-utils	\N	fpc-utils-3.0.4_6
2020-07-21 04:27:56+02	2020-07-21 04:34:16+02	lang	guile1	\N	guile1-1.8.8
2020-07-21 04:34:18+02	2020-07-21 04:34:46+02	lang	p5-Try-Tiny	\N	p5-Try-Tiny-0.30
2020-07-21 04:34:47+02	2020-07-21 04:40:16+02	lang	python38	\N	python38-3.8.4
2020-07-21 04:40:17+02	2020-07-21 05:02:10+02	math	R	\N	R-4.0.2
2020-07-21 05:02:12+02	2020-07-21 05:15:07+02	math	coinmp	\N	CoinMP-1.8.4_3
2020-07-21 05:15:08+02	2020-07-21 05:17:32+02	math	cryptominisat	\N	cryptominisat-5.7.1
2020-07-21 05:17:33+02	2020-07-21 05:17:51+02	math	fpc-numlib	\N	fpc-numlib-3.0.4_4
2020-07-21 05:17:51+02	2020-07-21 05:23:23+02	math	gsl	\N	gsl-2.6
2020-07-21 05:23:24+02	2020-07-21 05:26:06+02	math	lp_solve	\N	lp_solve-5.5.2.5
2020-07-21 05:26:07+02	2020-07-21 05:26:24+02	math	minisat	\N	minisat-2.2.1_2
2020-07-21 05:26:24+02	2020-07-21 05:26:45+02	math	openlibm	\N	openlibm-0.6.0_1
2020-07-21 05:26:46+02	2020-07-21 05:33:06+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-07-21 05:33:07+02	2020-07-21 05:42:06+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-07-21 05:42:07+02	2020-07-21 05:43:22+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-07-21 05:43:22+02	2020-07-21 05:44:34+02	multimedia	libass	\N	libass-0.14.0
2020-07-21 05:44:35+02	2020-07-21 05:45:08+02	multimedia	libdvbpsi	\N	libdvbpsi-1.3.3
2020-07-21 05:45:09+02	2020-07-21 05:45:44+02	multimedia	libmatroska	\N	libmatroska-1.6.0
2020-07-21 05:45:44+02	2020-07-21 05:46:31+02	multimedia	libmpeg2	\N	libmpeg2-0.5.1_6
2020-07-21 05:46:32+02	2020-07-21 05:49:19+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-21 05:49:20+02	2020-07-21 05:49:35+02	net	cppzmq	\N	cppzmq-4.6.0
2020-07-21 05:49:36+02	2020-07-21 05:50:04+02	net	fping	\N	fping-4.2
2020-07-21 05:50:05+02	2020-07-21 05:53:27+02	net	freerdp	\N	freerdp-2.1.1
2020-07-21 05:53:28+02	2020-07-21 05:54:05+02	net	gupnp	\N	gupnp-1.1.1_1
2020-07-21 05:54:05+02	2020-07-21 06:08:31+02	net	libcmis	\N	libcmis-0.5.2_1
2020-07-21 06:08:32+02	2020-07-21 06:08:49+02	net	libmaxminddb	\N	libmaxminddb-1.4.2
2020-07-21 06:08:50+02	2020-07-21 06:10:07+02	net	liveMedia	\N	liveMedia-2020.07.09,2
2020-07-21 06:10:07+02	2020-07-21 06:10:27+02	net	p5-Net-HTTP	\N	p5-Net-HTTP-6.19
2020-07-21 06:10:27+02	2020-07-21 06:10:46+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-07-21 06:10:47+02	2020-07-21 06:10:58+02	net	py-pysocks	py27	py27-pysocks-1.7.1
2020-07-21 06:10:59+02	2020-07-21 06:11:12+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-07-21 06:11:13+02	2020-07-21 06:12:09+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-07-21 06:12:10+02	2020-07-21 06:12:51+02	net	rsync	\N	rsync-3.1.3_1
2020-07-21 06:12:51+02	2020-07-21 06:13:50+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-21 06:13:51+02	2020-07-21 06:14:19+02	print	freetype	\N	freetype-1.3.1_5
2020-07-21 06:14:20+02	2020-07-21 06:19:25+02	print	gutenprint	\N	gutenprint-5.3.3
2020-07-21 06:19:26+02	2020-07-21 06:20:02+02	print	libspectre	\N	libspectre-0.2.9
2020-07-21 06:20:03+02	2020-07-21 06:22:36+02	print	mftrace	\N	mftrace-1.2.18_2
2020-07-21 06:22:37+02	2020-07-21 06:30:31+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-07-21 06:30:32+02	2020-07-21 06:37:26+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-07-21 06:37:27+02	2020-07-21 06:38:28+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-07-21 06:38:29+02	2020-07-21 06:38:48+02	security	p5-Authen-NTLM	\N	p5-Authen-NTLM-1.09_1
2020-07-21 06:38:49+02	2020-07-21 06:38:57+02	security	pam_helper	\N	pam_helper-1.1
2020-07-21 06:38:58+02	2020-07-21 06:39:14+02	security	py-bcrypt	py27	py27-bcrypt-3.1.7
2020-07-21 06:39:15+02	2020-07-21 06:39:26+02	security	py-certifi	py27	py27-certifi-2020.6.20
2020-07-21 06:39:26+02	2020-07-21 06:39:38+02	security	py-openssl	py27	py27-openssl-19.0.0
2020-07-21 06:39:39+02	2020-07-21 06:40:31+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-07-21 06:40:32+02	2020-07-21 06:40:53+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-07-21 06:40:54+02	2020-07-21 06:41:14+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-07-21 06:41:15+02	2020-07-21 06:42:46+02	security	sudo	\N	sudo-1.9.1
2020-07-21 06:42:47+02	2020-07-21 06:44:10+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-07-21 06:44:11+02	2020-07-21 06:44:33+02	sysutils	hal-info	\N	hal-info-20091130
2020-07-21 06:44:33+02	2020-07-21 06:49:07+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-07-21 06:49:07+02	2020-07-21 06:50:12+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-07-21 06:50:12+02	2020-07-21 06:50:32+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.04_1
2020-07-21 06:50:32+02	2020-07-21 06:51:11+02	sysutils	patchelf	\N	patchelf-0.10_1
2020-07-21 06:51:12+02	2020-07-21 06:51:20+02	sysutils	sysctlbyname-improved-kmod	\N	sysctlbyname-improved-kmod-20191124
2020-07-21 06:51:21+02	2020-07-21 06:51:29+02	sysutils	sysctlinfo-kmod	\N	sysctlinfo-kmod-20191005
2020-07-21 06:51:30+02	2020-07-21 06:56:31+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-07-21 06:56:32+02	2020-07-21 06:56:39+02	sysutils	vpnc-scripts	\N	vpnc-scripts-20190611
2020-07-21 06:56:40+02	2020-07-21 06:59:47+02	textproc	clucene	\N	clucene-2.3.3.4_19
2020-07-21 06:59:48+02	2020-07-21 07:02:09+02	textproc	dblatex	\N	dblatex-0.3.11
2020-07-21 07:02:10+02	2020-07-21 07:02:53+02	textproc	enchant	\N	enchant-1.6.0_9
2020-07-21 07:02:54+02	2020-07-21 07:04:31+02	textproc	libabw	\N	libabw-0.1.3
2020-07-21 07:04:32+02	2020-07-21 07:04:42+02	textproc	libdiff	\N	libdiff-0.1.0
2020-07-21 07:04:42+02	2020-07-21 07:09:25+02	textproc	libe-book	\N	libe-book-0.1.3_18
2020-07-21 07:09:27+02	2020-07-21 07:10:55+02	textproc	libepubgen	\N	libepubgen-0.1.0_9
2020-07-21 07:10:56+02	2020-07-21 07:11:10+02	textproc	libexttextcat	\N	libexttextcat-3.4.5
2020-07-21 07:11:11+02	2020-07-21 07:21:25+02	textproc	libmwaw03	\N	libmwaw03-0.3.15
2020-07-21 07:21:25+02	2020-07-21 07:22:15+02	textproc	libnumbertext	\N	libnumbertext-1.0.5_1
2020-07-21 07:22:15+02	2020-07-21 07:23:46+02	textproc	libodfgen01	\N	libodfgen01-0.1.7_3
2020-07-21 07:23:47+02	2020-07-21 07:25:12+02	textproc	libqxp	\N	libqxp-0.0.0_16
2020-07-21 07:25:13+02	2020-07-21 07:28:42+02	textproc	libstaroffice	\N	libstaroffice-0.0.6_8
2020-07-21 07:28:43+02	2020-07-21 07:32:06+02	textproc	libwps	\N	libwps-0.4.10_2
2020-07-21 07:32:07+02	2020-07-21 07:32:48+02	textproc	p5-PDF-API2	\N	p5-PDF-API2-2.037
2020-07-21 07:32:49+02	2020-07-21 07:33:08+02	textproc	p5-Pod-Escapes	\N	p5-Pod-Escapes-1.07
2020-07-21 07:33:09+02	2020-07-21 07:33:25+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-07-21 07:33:26+02	2020-07-21 07:33:40+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-07-21 07:33:41+02	2020-07-21 07:33:55+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-07-21 07:33:55+02	2020-07-21 07:34:58+02	textproc	redland	\N	redland-1.0.17_4
2020-07-20 22:53:27+02	2020-07-21 08:02:36+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-07-21 08:02:38+02	2020-07-21 08:02:57+02	www	p5-HTTP-Cookies	\N	p5-HTTP-Cookies-6.08
2020-07-21 08:02:58+02	2020-07-21 08:03:17+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.12
2020-07-21 08:03:17+02	2020-07-21 08:03:36+02	www	p5-HTTP-Negotiate	\N	p5-HTTP-Negotiate-6.01_1
2020-07-21 08:03:37+02	2020-07-21 08:03:56+02	www	p5-WWW-RobotRules	\N	p5-WWW-RobotRules-6.02_1
2020-07-21 08:03:57+02	2020-07-21 08:04:25+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-07-21 08:04:26+02	2020-07-21 08:05:20+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-21 08:05:21+02	2020-07-21 08:07:04+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-21 08:07:04+02	2020-07-21 08:07:17+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-07-21 08:07:18+02	2020-07-21 08:08:00+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-21 08:08:02+02	2020-07-21 08:09:46+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-07-21 08:09:47+02	2020-07-21 08:09:57+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-07-21 08:09:57+02	2020-07-21 08:10:09+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-07-21 08:10:09+02	2020-07-21 08:10:29+02	x11-fonts	droid-fonts-ttf	\N	droid-fonts-ttf-20131024_5
2020-07-21 08:10:29+02	2020-07-21 08:10:38+02	x11-fonts	gentium-basic	\N	GentiumBasic-1102_1
2020-07-21 08:10:39+02	2020-07-21 08:10:49+02	x11-fonts	liberation-fonts-ttf	\N	liberation-fonts-ttf-2.1.1,2
2020-07-21 08:10:50+02	2020-07-21 08:11:08+02	x11-fonts	linuxlibertine-g	\N	linuxlibertine-g-20120116_2
2020-07-21 08:11:09+02	2020-07-21 08:11:25+02	x11-fonts	twemoji-color-font-ttf	\N	twemoji-color-font-ttf-12.0.1
2020-07-21 08:11:26+02	2020-07-21 08:14:05+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-07-21 08:14:06+02	2020-07-21 08:14:47+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-07-21 08:14:47+02	2020-07-21 08:16:04+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-07-21 08:16:05+02	2020-07-21 08:16:43+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-07-21 08:16:44+02	2020-07-21 08:17:39+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-07-21 08:17:39+02	2020-07-21 08:20:10+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-07-21 08:20:11+02	2020-07-21 08:23:16+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-07-21 07:34:59+02	2020-07-21 08:32:08+02	www	node10	\N	node10-10.21.0
2020-07-21 08:32:09+02	2020-07-21 08:32:19+02	archivers	arc	\N	arc-5.21p
2020-07-21 08:32:19+02	2020-07-21 08:33:09+02	archivers	arj	\N	arj-3.10.22_8
2020-07-21 08:33:10+02	2020-07-21 08:36:11+02	archivers	engrampa	\N	engrampa-1.24.0
2020-07-21 08:36:11+02	2020-07-21 08:36:38+02	archivers	libunrar5	\N	libunrar5-5.6.8_1
2020-07-21 08:36:38+02	2020-07-21 08:37:47+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-07-21 08:37:47+02	2020-07-21 08:37:56+02	archivers	unzoo	\N	unzoo-4.4_2
2020-07-21 08:37:57+02	2020-07-21 08:38:19+02	audio	dssi	\N	dssi-1.1.1_8
2020-07-21 08:38:19+02	2020-07-21 08:39:10+02	audio	faudio	\N	FAudio-20.07
2020-07-21 08:39:10+02	2020-07-21 08:43:16+02	audio	fdk-aac	\N	fdk-aac-2.0.1
2020-07-21 08:43:16+02	2020-07-21 08:44:38+02	audio	mate-media	\N	mate-media-1.24.0
2020-07-21 08:44:39+02	2020-07-21 08:44:59+02	audio	sndio	\N	sndio-1.7.0
2020-07-21 08:44:59+02	2020-07-21 08:45:49+02	audio	swhplugins	\N	swhplugins-0.4.15_7
2020-07-21 08:45:50+02	2020-07-21 08:46:26+02	audio	wavpack	\N	wavpack-5.3.0
2020-07-21 08:46:26+02	2020-07-21 08:46:41+02	databases	lmdb	\N	lmdb-0.9.24_2,1
2020-07-21 08:46:42+02	2020-07-21 08:47:15+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.13.0
2020-07-21 08:47:16+02	2020-07-21 08:47:31+02	databases	pgtcl	\N	pgtcl-2.1.1_2
2020-07-21 08:47:31+02	2020-07-21 08:47:52+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-07-21 08:47:53+02	2020-07-21 08:49:35+02	databases	rrdtool	\N	rrdtool-1.7.2_3
2020-07-21 08:49:36+02	2020-07-21 08:51:20+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-07-21 08:23:17+02	2020-07-21 08:53:22+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-07-21 08:53:23+02	2020-07-21 08:54:21+02	deskutils	mozo	\N	mozo-1.24.0
2020-07-21 08:51:20+02	2020-07-21 08:55:09+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-07-21 08:54:22+02	2020-07-21 08:55:15+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-07-21 08:55:15+02	2020-07-21 08:55:58+02	devel	adacurses	\N	adacurses-20180127
2020-07-21 08:55:59+02	2020-07-21 08:56:13+02	devel	cweb	\N	cweb-3.64a.a
2020-07-21 08:56:14+02	2020-07-21 08:56:36+02	devel	erlang-cl	\N	erlang-cl-1.2.4
2020-07-21 08:56:36+02	2020-07-21 08:56:58+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-07-21 08:56:58+02	2020-07-21 08:57:17+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-07-21 08:57:17+02	2020-07-21 08:57:35+02	devel	fpc-fv	\N	fpc-fv-3.0.4_4
2020-07-21 08:57:36+02	2020-07-21 08:57:52+02	devel	fpc-gdbint	\N	fpc-gdbint-3.0.4_4
2020-07-21 08:57:53+02	2020-07-21 08:58:35+02	devel	fstrm	\N	fstrm-0.6.0
2020-07-21 08:58:36+02	2020-07-21 09:01:18+02	devel	google-perftools	\N	google-perftools-2.8
2020-07-21 09:01:19+02	2020-07-21 09:02:51+02	devel	gradle5	\N	gradle5-5.0
2020-07-21 08:55:09+02	2020-07-21 09:05:31+02	devel	aarch64-gcc	\N	aarch64-gcc-6.4.0_8
2020-07-21 09:05:32+02	2020-07-21 09:06:46+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.71.0_1
2020-07-21 09:06:47+02	2020-07-21 09:07:29+02	devel	libstatgrab	\N	libstatgrab-0.92
2020-07-21 09:07:30+02	2020-07-21 09:07:37+02	devel	libsysctlmibinfo2	\N	libsysctlmibinfo2-2.0.0_1
2020-07-21 09:07:37+02	2020-07-21 09:07:46+02	devel	libublio	\N	libublio-20070103_2
2020-07-21 09:02:52+02	2020-07-21 09:08:18+02	devel	gsoap	\N	gsoap-2.8.104
2020-07-21 09:08:18+02	2020-07-21 09:10:07+02	devel	ode	\N	ode-0.13_4,1
2020-07-21 09:10:08+02	2020-07-21 09:10:24+02	devel	p5-Term-ReadKey	\N	p5-Term-ReadKey-2.38_1
2020-07-21 09:07:47+02	2020-07-21 09:26:30+02	devel	mips64-gcc	\N	mips64-gcc-6.4.0_8
2020-07-21 09:10:25+02	2020-07-21 09:30:04+02	devel	powerpc64-gcc	\N	powerpc64-gcc-6.4.0_8
2020-07-21 09:26:31+02	2020-07-21 09:30:05+02	devel	protobuf-c	\N	protobuf-c-1.3.2_5
2020-07-21 09:30:04+02	2020-07-21 09:30:15+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-07-21 09:30:06+02	2020-07-21 09:30:18+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-07-21 09:30:19+02	2020-07-21 09:30:30+02	devel	py-funcy	py37	py37-funcy-1.12
2020-07-21 09:30:16+02	2020-07-21 09:30:34+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-07-21 09:30:30+02	2020-07-21 09:30:43+02	devel	py-future	py27	py27-future-0.18.2
2020-07-21 09:30:43+02	2020-07-21 09:30:54+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-07-21 09:30:55+02	2020-07-21 09:31:06+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-07-21 09:31:07+02	2020-07-21 09:31:24+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-07-21 09:31:25+02	2020-07-21 09:31:35+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-07-21 09:31:36+02	2020-07-21 09:31:47+02	devel	py-rq	py37	py37-rq-1.3.0
2020-07-21 09:30:35+02	2020-07-21 09:32:07+02	devel	py-game	py37	py37-game-1.9.6_1
2020-07-21 09:31:48+02	2020-07-21 09:32:19+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-07-21 09:32:07+02	2020-07-21 09:32:19+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-07-21 09:32:19+02	2020-07-21 09:32:31+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-07-21 09:32:19+02	2020-07-21 09:32:33+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-07-21 09:32:31+02	2020-07-21 09:32:42+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-07-21 09:32:42+02	2020-07-21 09:33:08+02	devel	rlwrap	\N	rlwrap-0.42_2
2020-07-21 09:33:08+02	2020-07-21 09:33:21+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-07-21 09:33:21+02	2020-07-21 09:33:34+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-07-21 09:33:35+02	2020-07-21 09:33:48+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-07-21 09:33:48+02	2020-07-21 09:41:54+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3_1
2020-07-21 09:41:55+02	2020-07-21 09:42:11+02	devel	scons	py27	scons-py27-3.1.2
2020-07-21 09:32:34+02	2020-07-21 09:53:51+02	devel	riscv64-gcc	\N	riscv64-gcc-8.3.0
2020-07-21 09:53:53+02	2020-07-21 09:54:38+02	devel	tex-synctex	\N	tex-synctex-1.17.0_1
2020-07-21 09:54:38+02	2020-07-21 09:54:53+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-07-21 09:54:54+02	2020-07-21 09:55:07+02	dns	py-dnspython	py27	py27-dnspython-1.16.0
2020-07-21 09:55:08+02	2020-07-21 09:57:10+02	dns	py-ldns	py27	py27-ldns-1.7.1_1
2020-07-21 09:42:12+02	2020-07-21 09:59:29+02	devel	sparc64-gcc	\N	sparc64-gcc-6.4.0_8
2020-07-21 09:57:11+02	2020-07-21 10:21:32+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-07-21 10:21:33+02	2020-07-21 10:27:22+02	editors	pluma	\N	pluma-1.24.0
2020-07-21 10:27:23+02	2020-07-21 10:30:53+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-07-21 10:30:54+02	2020-07-21 10:32:36+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-21 10:32:37+02	2020-07-21 10:33:41+02	ftp	libfilezilla	\N	libfilezilla-0.15.1_3
2020-07-21 10:33:42+02	2020-07-21 10:47:16+02	games	alienarena-data	\N	alienarena-data-2013.766
2020-07-21 10:47:17+02	2020-07-21 10:48:40+02	games	gnuchess	\N	gnuchess-6.2.7
2020-07-21 10:48:41+02	2020-07-21 10:53:17+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-07-21 10:53:17+02	2020-07-21 11:03:02+02	graphics	alembic	\N	alembic-1.7.13_1
2020-07-21 11:03:02+02	2020-07-21 11:07:18+02	graphics	atril	\N	atril-1.22.3_7
2020-07-21 11:07:19+02	2020-07-21 11:42:05+02	graphics	embree	\N	embree-3.10.0_1
2020-07-21 11:42:05+02	2020-07-21 11:44:58+02	graphics	eom	\N	eom-1.24.0
2020-07-21 11:44:58+02	2020-07-21 11:46:53+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-21 11:46:54+02	2020-07-21 11:47:28+02	graphics	gle	\N	gle-3.1.0_7
2020-07-21 11:47:28+02	2020-07-21 11:47:54+02	graphics	gltt	\N	gltt-2.5.2_14
2020-07-21 11:47:55+02	2020-07-21 11:49:52+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-07-21 11:49:52+02	2020-07-21 11:50:03+02	graphics	jgraph	\N	jgraph-8.3_1
2020-07-21 11:50:07+02	2020-07-21 11:50:41+02	graphics	libart_lgpl	\N	libart_lgpl-2.3.21_3,1
2020-07-21 11:50:41+02	2020-07-21 11:58:06+02	graphics	oidn	\N	oidn-1.1.0_3
2020-07-21 11:58:07+02	2020-07-21 12:25:47+02	graphics	opencollada	\N	opencollada-1.6.68_2
2020-07-21 12:25:48+02	2020-07-21 12:47:30+02	graphics	openimageio	\N	openimageio-2.1.15.0_2
2020-07-21 12:47:31+02	2020-07-21 12:52:28+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-07-21 12:52:29+02	2020-07-21 12:54:40+02	graphics	py-gimp	\N	py27-gimp-2.10.20_2
2020-07-21 12:54:41+02	2020-07-21 12:55:31+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-07-21 12:55:31+02	2020-07-21 13:10:18+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-07-21 13:10:19+02	2020-07-21 13:44:59+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-07-21 13:45:02+02	2020-07-21 13:54:39+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-07-21 13:54:40+02	2020-07-21 14:53:30+02	lang	gcc48	\N	gcc48-4.8.5_11
2020-07-20 16:58:06+02	2020-07-20 16:58:17+02	devel	py-ply	py37	py37-ply-3.11
2020-07-21 09:59:30+02	2020-07-21 16:17:18+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-21 16:17:21+02	2020-07-21 16:17:37+02	lang	lua51	\N	lua51-5.1.5_9
2020-07-21 16:17:38+02	2020-07-21 16:18:38+02	lang	luajit	\N	luajit-2.0.5_4
2020-07-21 16:18:39+02	2020-07-21 16:18:50+02	lang	python3	\N	python3-3_3
2020-07-21 16:18:51+02	2020-07-21 16:27:19+02	lang	sbcl	\N	sbcl-2.0.6,1
2020-07-21 16:27:19+02	2020-07-21 16:29:17+02	math	mate-calc	\N	mate-calc-1.24.0
2020-07-21 16:29:18+02	2020-07-21 16:30:25+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-07-21 16:30:26+02	2020-07-21 16:33:29+02	math	stp	\N	stp-2.3.3_6,1
2020-07-21 14:53:31+02	2020-07-21 16:41:24+02	lang	julia11	\N	julia11-1.1.1_5
2020-07-21 16:41:25+02	2020-07-21 16:41:39+02	misc	chmlib	\N	chmlib-0.40_1
2020-07-21 16:41:39+02	2020-07-21 16:41:47+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200616
2020-07-21 16:41:47+02	2020-07-21 16:48:08+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-07-21 16:48:09+02	2020-07-21 16:48:33+02	misc	ini_file_manager	\N	ini_file_manager-03_3
2020-07-21 16:33:30+02	2020-07-21 17:03:09+02	math	z3	\N	z3-4.8.8
2020-07-21 17:03:10+02	2020-07-21 17:03:29+02	misc	xosd	\N	xosd-2.2.14_3
2020-07-21 17:03:29+02	2020-07-21 17:04:15+02	multimedia	gmmlib	\N	gmmlib-20.2.2
2020-07-21 17:04:16+02	2020-07-21 17:04:45+02	multimedia	libmtp	\N	libmtp-1.1.17
2020-07-21 16:48:34+02	2020-07-21 17:07:04+02	misc	openvdb	\N	openvdb-7.0.0_4
2020-07-21 17:07:06+02	2020-07-21 17:08:10+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-07-21 17:04:45+02	2020-07-21 17:09:35+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-21 17:08:11+02	2020-07-21 17:10:59+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-07-21 17:10:59+02	2020-07-21 17:12:47+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-21 17:12:48+02	2020-07-21 17:13:56+02	net	ndpi	\N	ndpi-3.2.d20200403_1,1
2020-07-21 17:13:56+02	2020-07-21 17:14:12+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-07-21 17:14:13+02	2020-07-21 17:15:03+02	net	py-pyzmq	py37	py37-pyzmq-19.0.1
2020-07-21 17:15:04+02	2020-07-21 17:15:17+02	net	py-urllib3	py27	py27-urllib3-1.25.7,1
2020-07-21 17:15:18+02	2020-07-21 17:16:18+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-07-21 17:16:18+02	2020-07-21 17:17:24+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-07-21 17:17:25+02	2020-07-21 17:18:24+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-07-21 17:18:25+02	2020-07-21 17:19:31+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-07-21 17:19:32+02	2020-07-21 17:20:29+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-07-21 17:20:29+02	2020-07-21 17:21:29+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-07-21 17:21:30+02	2020-07-21 17:22:37+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-07-21 17:09:36+02	2020-07-21 17:23:21+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-07-21 17:22:38+02	2020-07-21 17:24:06+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-21 17:23:22+02	2020-07-21 17:24:07+02	net	vde2	\N	vde2-2.3.2_4
2020-10-05 06:47:10+02	2020-10-05 06:47:41+02	print	gv	\N	gv-3.7.4_5
2020-07-21 17:24:06+02	2020-07-21 17:24:29+02	net	wireguard-go	\N	wireguard-go-0.0.20200320
2020-07-21 17:24:07+02	2020-07-21 17:25:03+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-21 17:24:29+02	2020-07-21 17:25:16+02	net-mgmt	libsmi	\N	libsmi-0.4.8_1
2020-07-21 17:25:17+02	2020-07-21 17:25:26+02	net-mgmt	py-ipcalc	py27	py27-ipcalc-1.99.0
2020-07-21 17:25:27+02	2020-07-21 17:25:39+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-07-21 17:25:40+02	2020-07-21 17:25:47+02	ports-mgmt	dialog4ports	\N	dialog4ports-0.1.6
2020-07-21 17:25:04+02	2020-07-21 17:26:24+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-07-21 17:25:48+02	2020-07-21 17:27:08+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-21 17:27:09+02	2020-07-21 17:36:04+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-07-21 17:26:25+02	2020-07-21 17:40:58+02	print	lilypond	\N	lilypond-2.20.0
2020-07-21 17:36:05+02	2020-07-21 17:42:34+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-07-21 17:40:59+02	2020-07-21 17:43:05+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-07-21 17:42:35+02	2020-07-21 17:52:18+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-07-21 17:52:19+02	2020-07-21 17:52:54+02	print	transfig	\N	transfig-3.2.5e_4
2020-07-21 17:52:55+02	2020-07-21 17:53:18+02	security	libntlm	\N	libntlm-1.6
2020-07-21 17:53:18+02	2020-07-21 17:53:27+02	security	libscrypt	\N	libscrypt-1.21
2020-07-21 17:53:28+02	2020-07-21 17:53:56+02	security	mhash	\N	mhash-0.9.9.9_5
2020-07-21 17:53:57+02	2020-07-21 17:56:43+02	security	nmap	\N	nmap-7.80
2020-07-21 17:56:43+02	2020-07-21 17:57:17+02	security	openconnect	\N	openconnect-8.08
2020-07-21 17:57:18+02	2020-07-21 17:57:35+02	security	p5-Digest-SHA1	\N	p5-Digest-SHA1-2.13_1
2020-07-21 17:57:36+02	2020-07-21 17:57:43+02	security	pkesh	\N	pkesh-1.0
2020-07-21 17:57:43+02	2020-07-21 17:57:56+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-07-21 17:57:57+02	2020-07-21 17:58:09+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-07-21 17:58:10+02	2020-07-21 17:58:24+02	security	py-stem	py37	py37-stem-1.8.0
2020-07-21 17:58:25+02	2020-07-21 17:59:24+02	sysutils	intel-pcm	\N	intel-pcm-201902_1
2020-07-21 17:59:24+02	2020-07-21 17:59:54+02	sysutils	libcpuid	\N	libcpuid-0.5.0
2020-07-21 17:59:55+02	2020-07-21 18:00:51+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-07-21 18:00:52+02	2020-07-21 18:02:44+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-07-21 17:43:06+02	2020-07-21 18:03:48+02	print	texlive-docs	\N	texlive-docs-20150523
2020-07-21 18:03:49+02	2020-07-21 18:04:08+02	sysutils	py-psutil	py27	py27-psutil-5.7.0
2020-07-21 18:04:08+02	2020-07-21 18:05:04+02	textproc	diffutils	\N	diffutils-3.7
2020-07-21 18:02:45+02	2020-07-21 18:05:21+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-07-21 18:05:21+02	2020-07-21 18:06:44+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-07-21 18:05:04+02	2020-07-21 18:07:07+02	textproc	fop	\N	fop-2.3
2020-07-21 18:07:08+02	2020-07-21 18:07:44+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-07-21 18:07:45+02	2020-07-21 18:08:02+02	textproc	lowdown	\N	lowdown-0.7.1
2020-07-21 18:08:02+02	2020-07-21 18:08:19+02	textproc	p5-pod2pdf	\N	p5-pod2pdf-0.42_1
2020-07-21 18:08:20+02	2020-07-21 18:08:46+02	textproc	pugixml	\N	pugixml-1.10
2020-07-21 18:08:46+02	2020-07-21 18:08:57+02	textproc	py-chardet	py27	py27-chardet-3.0.4_3
2020-07-21 18:08:58+02	2020-07-21 18:10:02+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-07-21 18:10:02+02	2020-07-21 18:10:41+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-07-21 18:10:42+02	2020-07-21 18:10:53+02	textproc	py-scour	py37	py37-scour-0.37
2020-07-21 18:10:53+02	2020-07-21 18:11:06+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-07-21 18:11:07+02	2020-07-21 18:18:01+02	textproc	ripgrep	\N	ripgrep-12.1.1_2
2020-07-21 18:18:01+02	2020-07-21 18:18:52+02	www	mattermost-webapp	\N	mattermost-webapp-5.19.1
2020-07-21 18:18:52+02	2020-07-21 18:19:20+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-07-21 18:19:21+02	2020-07-21 18:19:40+02	www	p5-libwww	\N	p5-libwww-6.46
2020-07-21 18:19:40+02	2020-07-21 18:20:10+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.0
2020-07-21 18:20:10+02	2020-07-21 18:20:24+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-07-21 18:20:24+02	2020-07-21 18:20:41+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-07-21 18:20:42+02	2020-07-21 18:20:56+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-07-21 18:20:57+02	2020-07-21 18:21:11+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-07-21 18:21:12+02	2020-07-21 18:21:27+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-07-21 18:21:27+02	2020-07-21 18:21:39+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-07-21 18:21:39+02	2020-07-21 18:23:43+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-21 18:23:44+02	2020-07-21 18:25:37+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-21 18:25:38+02	2020-07-21 18:26:30+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-07-21 18:26:31+02	2020-07-21 18:26:46+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-07-21 18:26:47+02	2020-07-21 18:33:06+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-07-21 18:33:08+02	2020-07-21 18:34:21+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-21 18:34:22+02	2020-07-21 18:35:44+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-07-21 18:35:44+02	2020-07-21 18:37:49+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-07-21 18:37:50+02	2020-07-21 18:40:55+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-07-21 18:40:56+02	2020-07-21 18:41:04+02	x11-fonts	bitstream-vera	\N	bitstream-vera-1.10_8
2020-07-21 18:41:05+02	2020-07-21 18:42:48+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-07-21 18:42:48+02	2020-07-21 18:44:03+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-07-21 18:44:04+02	2020-07-21 18:44:45+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-07-21 18:44:45+02	2020-07-21 18:52:41+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-07-21 18:52:42+02	2020-07-21 18:52:55+02	archivers	pbzip2	\N	pbzip2-1.1.13
2020-07-21 18:52:56+02	2020-07-21 18:53:24+02	archivers	unrar	\N	unrar-5.91,6
2020-07-21 18:53:25+02	2020-07-21 19:23:13+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-21 19:23:14+02	2020-07-21 19:23:25+02	astro	xearth	\N	xearth-1.2
2020-07-21 19:23:26+02	2020-07-21 19:25:51+02	astro	xephem	\N	xephem-3.7.7_5
2020-07-21 18:06:44+02	2020-07-21 19:32:05+02	textproc	hs-pandoc	\N	hs-pandoc-2.9.2_2
2020-07-21 19:32:06+02	2020-07-21 19:32:30+02	audio	xmix	\N	xmix-2.1_3
2020-07-21 19:32:30+02	2020-07-21 19:34:29+02	benchmarks	fio	\N	fio-3.20
2020-07-21 19:34:30+02	2020-07-21 19:34:59+02	benchmarks	iperf3	\N	iperf3-3.8.1
2020-07-21 19:35:00+02	2020-07-21 19:41:32+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-21 19:41:33+02	2020-07-21 19:44:33+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-21 19:25:52+02	2020-07-21 19:53:43+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-21 19:53:45+02	2020-07-21 19:55:57+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-21 19:55:59+02	2020-07-21 19:57:57+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-21 19:57:58+02	2020-07-21 19:58:46+02	comms	kermit	\N	kermit-9.0.304.24
2020-07-21 19:58:46+02	2020-07-21 19:58:56+02	converters	dosunix	\N	dosunix-1.0.14
2020-07-21 19:58:56+02	2020-07-21 19:59:38+02	converters	recode	\N	recode-3.6_14
2020-07-21 19:59:38+02	2020-07-21 19:59:46+02	converters	unix2dos	\N	unix2dos-1.3
2020-07-21 19:59:47+02	2020-07-21 20:00:00+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-07-21 20:00:02+02	2020-07-21 20:27:37+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-07-21 20:27:38+02	2020-07-21 20:27:57+02	databases	pgtop	\N	pgtop-0.05_5
2020-07-21 20:27:58+02	2020-07-21 20:32:23+02	databases	postgresql12-contrib	\N	postgresql12-contrib-12.3
2020-07-21 20:32:24+02	2020-07-21 20:43:11+02	databases	postgresql12-server	\N	postgresql12-server-12.3
2020-07-21 20:43:12+02	2020-07-21 20:43:40+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-07-21 20:43:40+02	2020-07-21 20:44:28+02	databases	redis	\N	redis-5.0.9
2020-07-21 20:44:29+02	2020-07-21 20:49:32+02	deskutils	calibre	\N	calibre-4.21.0
2020-07-21 20:49:32+02	2020-07-21 20:50:43+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-07-21 20:50:44+02	2020-07-21 20:50:59+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-07-21 20:51:01+02	2020-07-21 20:51:21+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-07-21 19:44:34+02	2020-07-21 21:32:25+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-21 21:32:28+02	2020-07-21 21:32:44+02	devel	aarch64-xtoolchain-gcc	\N	aarch64-xtoolchain-gcc-0.4_1
2020-07-21 21:32:45+02	2020-07-21 21:35:10+02	devel	bzr	py27	bzr-2.7.0.6622
2020-07-21 21:35:10+02	2020-07-21 21:36:17+02	devel	cdecl	\N	cdecl-7.4.1
2020-07-21 21:36:18+02	2020-07-21 21:37:27+02	devel	cvs+ipv6	\N	cvs+ipv6-1.20120905_3
2020-07-21 21:37:29+02	2020-07-21 21:39:14+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-07-21 20:51:22+02	2020-07-21 22:30:58+02	devel	RStudio	\N	RStudio-1.3.1056
2020-07-21 22:31:00+02	2020-07-21 23:30:43+02	devel	ghidra	\N	ghidra-9.1
2020-07-21 23:30:45+02	2020-07-22 06:53:35+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200706_1
2020-07-20 21:08:20+02	2020-07-22 07:22:51+02	www	chromium	\N	chromium-83.0.4103.116_1
2020-07-22 07:22:56+02	2020-07-22 07:24:38+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-22 07:24:39+02	2020-07-22 07:26:31+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-22 07:26:32+02	2020-07-22 07:28:48+02	devel	mercurial	py27	mercurial-5.1.2
2020-07-22 07:28:49+02	2020-07-22 07:29:00+02	devel	mips64-xtoolchain-gcc	\N	mips64-xtoolchain-gcc-0.4_1
2020-07-22 07:29:02+02	2020-07-22 07:29:13+02	devel	powerpc64-xtoolchain-gcc	\N	powerpc64-xtoolchain-gcc-0.4_1
2020-07-22 07:29:14+02	2020-07-22 07:29:30+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-07-22 07:29:31+02	2020-07-22 07:29:41+02	devel	riscv64-xtoolchain-gcc	\N	riscv64-xtoolchain-gcc-0.4_1
2020-07-22 07:29:42+02	2020-07-22 07:29:53+02	devel	sparc64-xtoolchain-gcc	\N	sparc64-xtoolchain-gcc-0.4_1
2020-07-22 07:29:54+02	2020-07-22 07:37:25+02	devel	subversion	\N	subversion-1.14.0
2020-07-22 07:37:26+02	2020-07-22 07:38:07+02	devel	tig	\N	tig-2.5.1
2020-07-22 07:38:08+02	2020-07-22 07:43:36+02	dns	bind916	\N	bind916-9.16.5
2020-07-22 07:43:36+02	2020-07-22 07:43:47+02	dns	sshfp	\N	sshfp-1.2.2_5
2020-07-21 21:39:14+02	2020-07-22 08:42:18+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-22 08:42:20+02	2020-07-22 08:43:05+02	editors	fpc-ide	\N	fpc-ide-3.0.4_4
2020-07-22 08:43:05+02	2020-07-22 08:45:34+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-22 08:45:34+02	2020-07-22 08:50:20+02	editors	vim	\N	vim-8.2.1110
2020-07-22 07:43:48+02	2020-07-22 09:08:16+02	editors	codelite	\N	codelite-12.0_4
2020-07-22 08:50:20+02	2020-07-22 09:53:21+02	emulators	qemu	\N	qemu-4.2.1
2020-07-22 09:08:17+02	2020-07-22 10:16:31+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44
2020-07-22 10:16:33+02	2020-07-22 10:17:15+02	emulators	wine-gecko	\N	wine-gecko-2.47.1
2020-07-22 10:17:16+02	2020-07-22 10:18:29+02	emulators	wine-mono	\N	wine-mono-4.9.4_1
2020-07-22 10:18:30+02	2020-07-22 10:19:19+02	emulators	x48	\N	x48-0.6.4_1
2020-07-22 10:19:20+02	2020-07-22 10:35:03+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-07-22 09:53:23+02	2020-07-22 10:36:10+02	emulators	wine	\N	wine-5.0.1,1
2020-07-22 10:35:04+02	2020-07-22 10:36:16+02	ftp	wget	\N	wget-1.20.3
2020-07-22 10:36:16+02	2020-07-22 10:36:25+02	games	affenspiel	\N	affenspiel-1.0_2
2020-07-22 10:36:11+02	2020-07-22 10:36:29+02	games	KnightCap	\N	knightcap-3.6_8
2020-07-22 10:36:26+02	2020-07-22 10:36:50+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-07-22 10:36:50+02	2020-07-22 10:38:27+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-07-22 10:36:29+02	2020-07-22 10:38:43+02	games	alienarena	\N	alienarena-2013.766_9
2020-07-22 10:38:44+02	2020-07-22 10:39:08+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-07-22 10:39:09+02	2020-07-22 10:39:58+02	games	monsterz	\N	monsterz-0.7.1_15
2020-07-22 10:39:58+02	2020-07-22 10:40:38+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-07-22 10:40:39+02	2020-07-22 10:41:38+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-07-22 10:41:39+02	2020-07-22 10:41:53+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-07-22 10:41:53+02	2020-07-22 10:42:54+02	games	stockfish	\N	stockfish-11
2020-07-22 10:42:55+02	2020-07-22 10:44:11+02	games	xblast	\N	xblast-2.10.4_17
2020-07-22 10:38:28+02	2020-07-22 10:44:55+02	games	battletanks	\N	btanks-0.9.8083_9
2020-07-22 10:44:11+02	2020-07-22 10:45:24+02	games	xboard	\N	xboard-4.9.1_1
2020-07-22 10:44:56+02	2020-07-22 10:45:48+02	games	xcowsay	\N	xcowsay-1.5
2020-07-22 10:45:24+02	2020-07-22 10:46:04+02	games	xevil	\N	xevil-2.02.r2_2
2020-07-22 10:45:49+02	2020-07-22 10:46:08+02	games	xminesweep	\N	xminesweep-3.0_2
2020-07-22 10:46:09+02	2020-07-22 10:51:05+02	graphics	dia	\N	dia-0.97.3_1,1
2020-07-22 10:51:06+02	2020-07-22 10:52:27+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-22 10:52:27+02	2020-07-22 11:59:47+02	graphics	inkscape	\N	inkscape-1.0_3
2020-07-22 10:46:04+02	2020-07-22 12:02:16+02	graphics	blender	\N	blender-2.83.1_2
2020-07-22 12:02:17+02	2020-07-22 12:02:32+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-07-22 12:02:32+02	2020-07-22 12:07:12+02	graphics	wings	\N	wings-2.2.5
2020-07-22 12:07:13+02	2020-07-22 12:09:52+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-22 12:09:53+02	2020-07-22 12:12:17+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-07-22 12:12:17+02	2020-07-22 12:47:29+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-07-22 12:47:30+02	2020-07-22 12:48:42+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-07-22 06:53:37+02	2020-07-22 13:28:44+02	devel	llvm10	\N	llvm10-10.0.0_2
2020-07-22 12:48:43+02	2020-07-22 13:30:28+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.3
2020-07-22 13:30:29+02	2020-07-22 13:30:47+02	lang	gcc	\N	gcc-9_4
2020-07-22 11:59:48+02	2020-07-22 13:47:38+02	graphics	krita	\N	krita-4.3.0_2
2020-07-22 13:47:39+02	2020-07-22 13:48:15+02	lang	julia	\N	julia-1.0.3_3
2020-07-22 13:48:16+02	2020-07-22 13:48:40+02	lang	mmix	\N	mmix-20160804
2020-07-22 13:28:45+02	2020-07-22 14:11:42+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-07-22 14:11:43+02	2020-07-22 14:11:54+02	lang	python	\N	python-3.7_3,2
2020-07-22 14:11:55+02	2020-07-22 14:12:02+02	lang	python-mode.el	\N	python-mode.el-1.0_2,1
2020-07-22 14:12:03+02	2020-07-22 14:16:57+02	mail	alpine	\N	alpine-2.23.2
2020-07-22 14:16:57+02	2020-07-22 14:17:18+02	mail	bbmail	\N	bbmail-0.9.3
2020-07-22 14:17:19+02	2020-07-22 14:23:33+02	math	maxima	\N	maxima-5.44.0_2
2020-07-22 14:23:34+02	2020-07-22 14:23:45+02	math	sc	\N	sc-7.16_3
2020-07-22 14:23:46+02	2020-07-22 14:24:45+02	misc	gnu-watch	\N	gnu-watch-3.3.16
2020-07-22 14:24:45+02	2020-07-22 14:27:56+02	misc	mc	\N	mc-nox11-4.8.24
2020-07-22 14:27:57+02	2020-07-22 14:29:01+02	misc	pinfo	\N	pinfo-0.6.10_5
2020-07-22 14:29:02+02	2020-07-22 14:30:31+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-22 14:30:31+02	2020-07-22 14:30:50+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-07-22 14:30:51+02	2020-07-22 14:31:48+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-07-22 13:48:40+02	2020-07-22 14:39:27+02	lang	mono	\N	mono-5.10.1.57_3
2020-07-22 14:39:28+02	2020-07-22 14:40:20+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-07-22 14:40:21+02	2020-07-22 14:40:44+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-07-22 14:40:44+02	2020-07-22 14:49:28+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-22 14:49:29+02	2020-07-22 14:52:34+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-22 14:31:49+02	2020-07-22 14:58:17+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p3
2020-07-22 14:58:18+02	2020-07-22 14:58:52+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-07-22 14:58:53+02	2020-07-22 15:00:23+02	net	mosh	\N	mosh-1.3.2_13
2020-07-22 14:52:35+02	2020-07-22 15:03:53+02	multimedia	shotcut	\N	shotcut-20.06.28
2020-07-22 15:03:54+02	2020-07-22 15:04:08+02	net	py-netifaces	py27	py27-netifaces-0.10.9
2020-07-22 15:04:09+02	2020-07-22 15:04:52+02	net	rdesktop	\N	rdesktop-1.8.6
2020-07-22 15:00:24+02	2020-07-22 15:05:53+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-07-22 15:05:53+02	2020-07-22 15:06:02+02	net	svnup	\N	svnup-1.08
2020-07-22 15:06:02+02	2020-07-22 15:06:17+02	net	torsocks	\N	torsocks-2.3.0_1
2020-07-22 15:04:53+02	2020-07-22 15:06:19+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-22 15:06:20+02	2020-07-22 15:06:34+02	net	vnstat	\N	vnstat-2.6_1
2020-07-22 15:06:35+02	2020-07-22 15:07:15+02	net	wireguard	\N	wireguard-1.0.20200513
2020-07-22 15:06:18+02	2020-07-22 15:07:24+02	net	v2ray	\N	v2ray-4.23.4
2020-07-22 15:07:25+02	2020-07-22 15:08:22+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-07-22 15:08:23+02	2020-07-22 15:32:25+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-07-22 15:07:15+02	2020-07-22 15:33:18+02	net	wireshark	\N	wireshark-3.2.5
2020-07-22 15:32:26+02	2020-07-22 15:33:20+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-22 15:33:19+02	2020-07-22 15:34:11+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-22 15:33:20+02	2020-07-22 15:34:12+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-22 15:34:12+02	2020-07-22 15:35:04+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-22 15:34:13+02	2020-07-22 15:35:04+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-22 15:35:05+02	2020-07-22 15:35:56+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-22 15:35:05+02	2020-07-22 15:35:57+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-07-22 15:35:57+02	2020-07-22 15:36:53+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-22 15:35:56+02	2020-07-22 15:37:22+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-22 15:37:23+02	2020-07-22 15:37:34+02	net-mgmt	iftop	\N	iftop-1.0.p4
2020-07-22 15:37:34+02	2020-07-22 15:37:50+02	net-mgmt	ipcalc	\N	ipcalc-0.41_3
2020-07-22 15:36:54+02	2020-07-22 15:37:51+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-22 15:37:51+02	2020-07-22 15:38:03+02	net-mgmt	nstat	\N	nstat-1.0_3
2020-07-22 15:38:04+02	2020-07-22 15:38:13+02	net-mgmt	whatmask	\N	whatmask-1.2_1
2020-07-22 15:37:51+02	2020-07-22 15:38:32+02	net-mgmt	ipv6calc	\N	ipv6calc-2.2.0
2020-07-22 15:38:32+02	2020-07-22 15:38:47+02	ports-mgmt	pkg_tree	\N	pkg_tree-1.1_4
2020-07-22 15:38:48+02	2020-07-22 15:39:58+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-07-22 15:38:13+02	2020-07-22 15:40:33+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-07-22 15:39:59+02	2020-07-22 15:40:34+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-07-22 15:40:34+02	2020-07-22 15:42:13+02	ports-mgmt	synth	\N	synth-2.08
2020-07-22 15:42:14+02	2020-07-22 15:43:11+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-07-22 15:40:34+02	2020-07-22 15:43:25+02	print	auctex	full	auctex-emacs26-12.2
2020-07-22 15:43:12+02	2020-07-22 15:43:52+02	print	gv	\N	gv-3.7.4_5
2020-07-22 15:43:26+02	2020-07-22 16:07:33+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-22 16:07:33+02	2020-07-22 16:13:56+02	print	texlive-full	\N	texlive-full-20150521
2020-07-22 16:13:57+02	2020-07-22 16:14:36+02	security	aide	\N	aide-0.16.1
2020-07-22 16:14:37+02	2020-07-22 16:14:44+02	security	base-audit	\N	base-audit-0.4
2020-07-22 13:30:48+02	2020-07-22 16:17:12+02	lang	gcc10-devel	\N	gcc10-devel-10.1.1.s20200711
2020-07-22 16:17:14+02	2020-07-22 16:17:22+02	security	doas	\N	doas-6.3_1
2020-07-22 16:14:44+02	2020-07-22 16:18:02+02	security	clamav	\N	clamav-0.102.4,1
2020-07-22 16:17:23+02	2020-07-22 16:18:03+02	security	ipv6toolkit	\N	ipv6toolkit-2.0
2020-07-22 16:18:02+02	2020-07-22 16:18:22+02	security	kc24	\N	kc24-2.4.2_1
2020-07-22 16:18:23+02	2020-07-22 16:18:30+02	security	lynis	\N	lynis-3.0.0
2020-07-22 16:18:31+02	2020-07-22 16:18:48+02	security	nyx	\N	nyx-2.1.0_1
2020-07-22 16:18:48+02	2020-07-22 16:20:16+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-22 16:20:17+02	2020-07-22 16:21:55+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-07-22 16:21:56+02	2020-07-22 16:22:18+02	security	py-xkcdpass	py37	py37-xkcdpass-1.17.4
2020-07-22 16:22:18+02	2020-07-22 16:22:42+02	security	rkhunter	\N	rkhunter-1.4.6
2020-07-22 16:18:04+02	2020-07-22 16:23:00+02	security	klee	\N	klee-2.1_1
2020-07-22 16:22:42+02	2020-07-22 16:23:02+02	security	ssl-admin	\N	ssl-admin-1.2.1
2020-07-22 16:23:03+02	2020-07-22 16:26:53+02	shells	zsh	\N	zsh-5.8
2020-07-22 16:26:53+02	2020-07-22 16:27:02+02	sysutils	915resolution	\N	915resolution-0.5.3_1,1
2020-07-22 16:23:01+02	2020-07-22 16:29:31+02	security	tor	\N	tor-0.4.3.6
2020-07-22 16:29:32+02	2020-07-22 16:30:10+02	sysutils	apachetop	\N	apachetop-0.18.4_1
2020-07-22 16:30:11+02	2020-07-22 16:30:22+02	sysutils	atop	\N	atop-2.0.2.b3_3
2020-07-22 16:27:03+02	2020-07-22 16:30:26+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-07-22 16:30:22+02	2020-07-22 16:30:29+02	sysutils	beadm	\N	beadm-1.3.0
2020-07-22 16:30:26+02	2020-07-22 16:30:33+02	sysutils	bsdcrashtar	\N	bsdcrashtar-0.1.3_1
2020-07-22 16:30:30+02	2020-07-22 16:30:36+02	sysutils	bsdhwmon	\N	bsdhwmon-20151206
2020-07-22 16:30:34+02	2020-07-22 16:30:40+02	sysutils	bsdinfo	\N	bsdinfo-0.22_2
2020-07-22 16:30:37+02	2020-07-22 16:30:44+02	sysutils	bsdstats	\N	bsdstats-7.0
2020-07-22 16:30:44+02	2020-07-22 16:30:51+02	sysutils	cmdwatch	\N	cmdwatch-0.2.0_2
2020-07-22 16:30:51+02	2020-07-22 16:34:10+02	sysutils	coreutils	\N	coreutils-8.32
2020-07-22 16:34:11+02	2020-07-22 16:35:24+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-07-22 16:30:41+02	2020-07-22 16:35:32+02	sysutils	cdrtools	\N	cdrtools-3.01_1
2020-07-22 16:35:25+02	2020-07-22 16:35:36+02	sysutils	cpupdate	\N	cpupdate-g20180513_2
2020-07-22 16:35:36+02	2020-07-22 16:35:46+02	sysutils	devcpu-data	\N	devcpu-data-1.34
2020-07-22 16:35:32+02	2020-07-22 16:35:52+02	sysutils	ddrescue	\N	ddrescue-1.25
2020-07-22 16:35:47+02	2020-07-22 16:36:05+02	sysutils	dfc	\N	dfc-3.1.1
2020-07-22 16:35:53+02	2020-07-22 16:36:21+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-07-22 16:36:21+02	2020-07-22 16:37:18+02	sysutils	fusefs-ntfs	\N	fusefs-ntfs-2017.3.23
2020-07-22 16:37:18+02	2020-07-22 16:38:36+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-07-22 16:38:38+02	2020-07-22 16:39:47+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-07-22 16:36:05+02	2020-07-22 16:39:57+02	sysutils	exa	\N	exa-0.9.0_12
2020-07-22 16:39:58+02	2020-07-22 16:40:08+02	sysutils	hourglass	\N	hourglass-1.0.1
2020-07-22 15:43:53+02	2020-07-22 16:40:13+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-07-22 16:40:13+02	2020-07-22 16:40:20+02	sysutils	ioping	\N	ioping-1.2
2020-07-22 16:40:21+02	2020-07-22 16:40:28+02	sysutils	lsop	\N	lsop-0.2
2020-07-22 16:40:08+02	2020-07-22 16:40:35+02	sysutils	htop	\N	htop-2.2.0_1
2020-07-22 16:40:28+02	2020-07-22 16:40:44+02	sysutils	memtest86+	\N	memtest86+-5.01_3
2020-07-22 16:40:35+02	2020-07-22 16:40:47+02	sysutils	most	\N	most-5.1.0
2020-07-22 16:40:44+02	2020-07-22 16:40:53+02	sysutils	ncdu	\N	ncdu-1.15.1
2020-07-22 16:40:48+02	2020-07-22 16:40:54+02	sysutils	neofetch	\N	neofetch-7.0.0
2020-07-22 16:40:54+02	2020-07-22 16:41:00+02	sysutils	obliterate	\N	obliterate-0.5
2020-07-22 16:40:54+02	2020-07-22 16:41:09+02	sysutils	panicmail	\N	panicmail-1.1
2020-07-22 16:39:47+02	2020-07-22 16:41:15+02	sysutils	hexyl	\N	hexyl-0.8.0_1
2020-07-22 16:41:00+02	2020-07-22 16:41:19+02	sysutils	parallel	\N	parallel-20200622
2020-07-22 16:41:10+02	2020-07-22 16:41:22+02	sysutils	psmisc	\N	psmisc-22.16_1
2020-07-22 16:41:15+02	2020-07-22 16:41:23+02	sysutils	pv	\N	pv-1.6.6
2020-07-22 16:41:19+02	2020-07-22 16:41:26+02	sysutils	pwgen	\N	pwgen-2.08,2
2020-07-22 16:41:24+02	2020-07-22 16:41:32+02	sysutils	py-py-cpuinfo	py27	py27-py-cpuinfo-6.0.0
2020-07-22 16:41:27+02	2020-07-22 16:41:33+02	sysutils	reptyr	\N	reptyr-0.6.3a_2
2020-07-22 16:41:22+02	2020-07-22 16:41:34+02	sysutils	py-glances	py27	py27-glances-3.1.4
2020-07-22 16:41:35+02	2020-07-22 16:42:08+02	sysutils	smartmontools	\N	smartmontools-7.1_2
2020-07-22 16:41:33+02	2020-07-22 16:42:08+02	sysutils	screen	\N	screen-4.8.0
2020-07-22 16:42:08+02	2020-07-22 16:42:16+02	sysutils	spinner	\N	spinner-1.2.4
2020-07-22 16:42:17+02	2020-07-22 16:42:25+02	sysutils	tree	\N	tree-1.8.0
2020-07-22 16:42:09+02	2020-07-22 16:42:39+02	sysutils	tmux	\N	tmux-3.1b
2020-07-22 16:42:25+02	2020-07-22 16:42:42+02	sysutils	x86info	\N	x86info-1.31.s03
2020-07-22 16:42:39+02	2020-07-22 16:42:56+02	sysutils	zfs-stats	\N	zfs-stats-1.3.0
2020-07-22 16:42:42+02	2020-07-22 16:43:43+02	sysutils	zrepl	\N	zrepl-0.2.1
2020-07-22 16:43:43+02	2020-07-22 16:43:59+02	textproc	colordiff	\N	colordiff-1.0.19
2020-07-22 16:44:00+02	2020-07-22 16:44:22+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-07-22 16:44:23+02	2020-07-22 16:44:45+02	www	dokuwiki	\N	dokuwiki-20180422c
2020-07-22 16:41:32+02	2020-07-22 16:51:25+02	sysutils	rsfetch	\N	rsfetch-2.0.0_4
2020-07-22 16:51:26+02	2020-07-22 16:52:42+02	www	links	\N	links-2.20.2,1
2020-07-22 16:52:42+02	2020-07-22 16:54:46+02	www	lynx	\N	lynx-2.8.9.1_1,1
2020-07-22 16:54:46+02	2020-07-22 16:56:24+02	www	mattermost-server	\N	mattermost-server-5.19.1
2020-07-22 16:42:57+02	2020-07-22 16:56:42+02	textproc	bat	\N	bat-0.15.4_1
2020-07-22 16:56:25+02	2020-07-22 16:56:50+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_2
2020-07-22 16:56:50+02	2020-07-22 16:57:50+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-07-22 16:57:51+02	2020-07-22 16:58:23+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-07-22 16:58:24+02	2020-07-22 16:58:35+02	www	py-bottle	py27	py27-bottle-0.12.15_1
2020-07-22 16:58:36+02	2020-07-22 16:58:55+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-07-22 16:58:56+02	2020-07-22 16:59:14+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-07-22 16:59:17+02	2020-07-22 16:59:34+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-07-22 16:59:35+02	2020-07-22 16:59:52+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-07-22 16:59:52+02	2020-07-22 17:00:13+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-07-22 17:00:14+02	2020-07-22 17:00:31+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-07-22 17:00:31+02	2020-07-22 17:00:56+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-07-22 17:00:57+02	2020-07-22 17:01:17+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-07-22 17:01:18+02	2020-07-22 17:01:31+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-07-22 16:56:42+02	2020-07-22 17:01:47+02	www	mod_php73	\N	mod_php73-7.3.20
2020-07-22 17:01:31+02	2020-07-22 17:01:47+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-07-22 17:01:48+02	2020-07-22 17:02:04+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-07-22 17:01:48+02	2020-07-22 17:02:10+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-07-22 17:02:05+02	2020-07-22 17:02:16+02	www	py-requests	py27	py27-requests-2.22.0
2020-07-22 17:02:11+02	2020-07-22 17:02:23+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-07-22 17:02:24+02	2020-07-22 17:04:58+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-22 17:04:59+02	2020-07-22 17:07:26+02	x11	mate	\N	mate-1.22.2
2020-07-22 17:07:27+02	2020-07-22 17:07:43+02	x11	numlockx	\N	numlockx-1.2
2020-07-22 17:07:44+02	2020-07-22 17:07:59+02	x11	sterm	\N	sterm-0.8.4
2020-07-22 17:07:59+02	2020-07-22 17:08:33+02	x11	xclip	\N	xclip-0.13
2020-07-22 17:08:33+02	2020-07-22 17:08:52+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-07-22 17:08:53+02	2020-07-22 17:09:35+02	x11	xdm	\N	xdm-1.1.12_3
2020-07-22 17:09:36+02	2020-07-22 17:11:42+02	x11	xlockmore	\N	xlockmore-5.63
2020-07-22 17:11:43+02	2020-07-22 17:12:04+02	x11	xman	\N	xman-1.1.5
2020-07-22 17:12:04+02	2020-07-22 17:19:38+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-07-22 17:19:38+02	2020-07-22 17:26:22+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-07-22 17:26:23+02	2020-07-22 17:28:19+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-07-22 17:28:19+02	2020-07-22 17:28:40+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-07-22 17:28:41+02	2020-07-22 17:28:53+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-07-22 17:02:17+02	2020-07-22 17:29:10+02	x11	alacritty	\N	alacritty-0.4.3_2
2020-07-22 17:29:11+02	2020-07-22 17:29:36+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.4_6
2020-07-22 17:28:54+02	2020-07-22 17:29:47+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-07-22 17:29:47+02	2020-07-22 17:30:14+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-07-22 17:29:36+02	2020-07-22 17:30:20+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-07-22 17:30:20+02	2020-07-22 17:31:39+02	graphics	xfig	\N	xfig-3.2.7a
2020-07-22 17:31:39+02	2020-07-22 17:32:24+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-07-22 17:30:15+02	2020-07-22 18:17:24+02	editors	vscode	\N	vscode-1.46.1
2020-07-22 16:44:46+02	2020-07-22 19:10:25+02	www	firefox	\N	firefox-78.0.2_1,1
2020-07-22 19:10:26+02	2020-07-22 19:26:06+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-22 19:35:07+02	2020-07-22 19:36:16+02	graphics	gpu-firmware-kmod	\N	gpu-firmware-kmod-g20200503
2020-07-22 19:36:16+02	2020-07-22 19:40:31+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-22 19:40:32+02	2020-07-22 19:41:12+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-22 19:41:13+02	2020-07-22 19:41:20+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-22 20:38:42+02	2020-07-22 20:39:34+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-22 20:38:42+02	2020-07-22 20:40:13+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-22 20:40:14+02	2020-07-22 20:40:21+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-22 20:46:04+02	2020-07-22 20:46:11+02	misc	qtchooser	\N	qtchooser-66_3
2020-07-22 20:46:04+02	2020-07-22 20:46:24+02	x11	xkeyboard-config	\N	xkeyboard-config-2.30
2020-07-22 20:46:04+02	2020-07-22 20:46:34+02	devel	nasm	\N	nasm-2.15.03,1
2020-07-22 20:46:25+02	2020-07-22 20:46:39+02	x11	libxkbcommon	\N	libxkbcommon-0.10.0_2
2020-07-22 20:46:11+02	2020-07-22 20:46:46+02	devel	qt5-qmake	\N	qt5-qmake-5.15.0
2020-07-22 20:46:39+02	2020-07-22 20:47:03+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-07-22 20:46:34+02	2020-07-22 20:47:11+02	devel	libevent	\N	libevent-2.1.12
2020-07-22 20:46:46+02	2020-07-22 20:47:23+02	devel	qt5-buildtools	\N	qt5-buildtools-5.15.0
2020-07-22 20:47:03+02	2020-07-22 20:47:23+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-07-22 20:47:12+02	2020-07-22 20:47:26+02	www	libwpe	\N	libwpe-1.4.0.1
2020-07-22 20:47:27+02	2020-07-22 20:47:40+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-07-22 20:47:24+02	2020-07-22 20:48:15+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-07-22 20:48:15+02	2020-07-22 20:49:08+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-07-22 20:47:40+02	2020-07-22 20:49:36+02	databases	libmemcached	\N	libmemcached-1.0.18_7
2020-07-22 20:49:09+02	2020-07-22 20:50:03+02	net	qt5-network	\N	qt5-network-5.15.0
2020-07-22 20:49:37+02	2020-07-22 20:50:16+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-07-22 20:50:17+02	2020-07-22 20:50:48+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-07-22 20:47:24+02	2020-07-22 20:51:12+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-07-22 20:50:49+02	2020-07-22 20:51:29+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-07-22 20:51:13+02	2020-07-22 20:51:45+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-07-22 20:50:03+02	2020-07-22 20:51:52+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-07-22 20:51:29+02	2020-07-22 20:51:58+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-07-22 20:51:59+02	2020-07-22 20:52:33+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-07-22 20:52:33+02	2020-07-22 20:52:54+02	databases	pecl-memcached	php73	php73-pecl-memcached-3.1.5
2020-07-22 20:51:52+02	2020-07-22 20:53:38+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-07-22 20:52:54+02	2020-07-22 20:53:44+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-07-22 20:53:44+02	2020-07-22 20:54:40+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-07-22 20:54:40+02	2020-07-22 20:55:57+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-07-22 20:53:39+02	2020-07-22 20:56:23+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-07-22 20:55:57+02	2020-07-22 20:56:29+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-07-22 20:56:24+02	2020-07-22 20:57:05+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-07-22 20:56:30+02	2020-07-22 20:57:25+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-07-22 20:51:46+02	2020-07-22 20:57:35+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-07-22 20:57:06+02	2020-07-22 20:57:55+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.72.0
2020-07-22 20:57:25+02	2020-07-22 20:58:01+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-07-22 20:58:02+02	2020-07-22 20:59:11+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-07-22 20:57:55+02	2020-07-22 20:59:14+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.72.0
2020-07-22 20:59:12+02	2020-07-22 21:00:10+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-07-22 20:59:14+02	2020-07-22 21:01:05+02	devel	kf5-kconfig	\N	kf5-kconfig-5.72.0
2020-07-22 21:00:11+02	2020-07-22 21:01:30+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-22 21:01:06+02	2020-07-22 21:01:58+02	archivers	kf5-karchive	\N	kf5-karchive-5.72.0
2020-07-22 20:57:36+02	2020-07-22 21:02:38+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-07-22 21:01:31+02	2020-07-22 21:02:49+02	devel	kf5-ki18n	\N	kf5-ki18n-5.72.0
2020-07-22 21:01:58+02	2020-07-22 21:02:50+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-22 21:02:50+02	2020-07-22 21:03:37+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-22 21:02:39+02	2020-07-22 21:03:39+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-07-22 21:02:49+02	2020-07-22 21:03:51+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.72.0
2020-07-22 21:03:40+02	2020-07-22 21:04:12+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-07-22 21:03:38+02	2020-07-22 21:04:17+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-07-22 21:04:18+02	2020-07-22 21:04:37+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-07-22 21:03:52+02	2020-07-22 21:05:05+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.72.0
2020-07-22 21:04:12+02	2020-07-22 21:05:50+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_2,1
2020-07-22 21:05:06+02	2020-07-22 21:06:02+02	devel	kf5-kcrash	\N	kf5-kcrash-5.72.0
2020-07-22 21:06:03+02	2020-07-22 21:06:48+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-07-22 21:05:50+02	2020-07-22 21:06:50+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.72.0
2020-07-22 21:04:37+02	2020-07-22 21:07:05+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.72.0
2020-07-22 21:06:50+02	2020-07-22 21:07:32+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-22 21:06:50+02	2020-07-22 21:07:42+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-07-22 21:07:06+02	2020-07-22 21:07:48+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.72.0
2020-07-22 21:07:33+02	2020-07-22 21:08:13+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-07-22 21:07:43+02	2020-07-22 21:08:35+02	databases	mysql57-client	\N	mysql57-client-5.7.31
2020-07-22 21:07:49+02	2020-07-22 21:08:48+02	devel	kf5-kauth	\N	kf5-kauth-5.72.0
2020-07-22 21:08:49+02	2020-07-22 21:09:46+02	net	remmina	\N	remmina-1.3.9
2020-07-22 21:08:14+02	2020-07-22 21:09:58+02	devel	kf5-kservice	\N	kf5-kservice-5.72.0
2020-07-22 21:08:35+02	2020-07-22 21:10:05+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-07-22 21:10:05+02	2020-07-22 21:10:42+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-07-22 21:09:59+02	2020-07-22 21:10:54+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.72.0
2020-07-22 21:09:47+02	2020-07-22 21:11:05+02	security	gcr	\N	gcr-3.28.0
2020-07-22 21:11:05+02	2020-07-22 21:12:08+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.72.0
2020-07-22 21:10:54+02	2020-07-22 21:12:10+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.72.0
2020-07-22 21:10:43+02	2020-07-22 21:12:17+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.72.0
2020-07-22 21:12:09+02	2020-07-22 21:12:59+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-07-22 21:12:17+02	2020-07-22 21:13:51+02	math	gnuplot	\N	gnuplot-5.2.8_2
2020-07-22 21:12:10+02	2020-07-22 21:14:07+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-22 21:13:51+02	2020-07-22 21:14:31+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-22 21:14:08+02	2020-07-22 21:15:10+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.72.0
2020-07-22 21:14:32+02	2020-07-22 21:15:16+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-07-22 21:15:16+02	2020-07-22 21:16:40+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.72.0
2020-07-22 21:16:40+02	2020-07-22 21:17:35+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-07-22 21:15:11+02	2020-07-22 21:18:23+02	net	libmateweather	\N	libmateweather-1.24.0
2020-07-22 21:17:36+02	2020-07-22 21:18:41+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.72.0
2020-07-22 21:18:41+02	2020-07-22 21:19:09+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-07-22 21:18:24+02	2020-07-22 21:19:56+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-22 21:19:09+02	2020-07-22 21:20:03+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-22 21:19:56+02	2020-07-22 21:20:55+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-22 21:20:03+02	2020-07-22 21:21:28+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-07-22 21:21:28+02	2020-07-22 21:22:57+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.72.0
2020-07-22 21:13:00+02	2020-07-22 21:23:36+02	net	openmpi	\N	openmpi-4.0.4
2020-07-22 21:22:58+02	2020-07-22 21:25:22+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-07-22 21:20:56+02	2020-07-22 21:26:40+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-22 21:25:23+02	2020-07-22 21:27:14+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.72.0
2020-07-22 21:26:41+02	2020-07-22 21:27:59+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.72.0
2020-07-22 21:23:37+02	2020-07-22 21:29:29+02	math	vtk8	\N	vtk8-8.2.0
2020-07-22 21:28:00+02	2020-07-22 21:31:26+02	devel	kf5-knotifications	\N	kf5-knotifications-5.72.0
2020-07-22 21:27:15+02	2020-07-22 21:32:11+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-07-22 21:31:27+02	2020-07-22 21:32:14+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-07-22 21:32:11+02	2020-07-22 21:32:40+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-07-22 21:32:15+02	2020-07-22 21:32:55+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-07-22 21:32:41+02	2020-07-22 21:33:28+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-22 21:32:55+02	2020-07-22 21:33:48+02	sysutils	bsdisks	\N	bsdisks-0.17
2020-07-22 21:33:28+02	2020-07-22 21:34:19+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-07-22 21:33:49+02	2020-07-22 21:34:33+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-22 21:34:20+02	2020-07-22 21:34:59+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-07-22 21:34:33+02	2020-07-22 21:35:14+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-07-22 21:29:30+02	2020-07-22 21:35:18+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-07-22 21:34:59+02	2020-07-22 21:35:38+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-07-22 21:35:15+02	2020-07-22 21:35:52+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-07-22 21:35:53+02	2020-07-22 21:36:33+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-07-22 21:35:38+02	2020-07-22 21:36:37+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-07-22 21:36:34+02	2020-07-22 21:37:20+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-07-22 21:36:37+02	2020-07-22 21:38:14+02	x11-fm	caja	\N	caja-1.22.3_1
2020-07-22 21:38:14+02	2020-07-22 21:39:14+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-07-22 21:37:21+02	2020-07-22 21:39:41+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.72.0
2020-07-22 21:39:15+02	2020-07-22 21:40:23+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-07-22 21:39:42+02	2020-07-22 21:41:09+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.72.0
2020-07-22 21:40:23+02	2020-07-22 21:43:03+02	devel	kf5-solid	\N	kf5-solid-5.72.0
2020-07-22 21:41:09+02	2020-07-22 21:43:08+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-07-22 21:43:03+02	2020-07-22 21:44:24+02	graphics	gegl	\N	gegl-0.4.24_2
2020-10-05 06:47:23+02	2020-10-05 06:49:12+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-22 21:43:08+02	2020-07-22 21:44:38+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-07-22 21:44:25+02	2020-07-22 21:45:14+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-22 21:44:38+02	2020-07-22 21:46:05+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-07-22 21:45:15+02	2020-07-22 21:47:12+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.72.0
2020-07-22 21:46:06+02	2020-07-22 21:47:15+02	x11	kf5-kded	\N	kf5-kded-5.72.0_1
2020-07-22 21:47:16+02	2020-07-22 21:48:06+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-22 21:47:12+02	2020-07-22 21:48:11+02	x11	zenity	\N	zenity-3.32.0
2020-07-22 21:35:19+02	2020-07-22 21:48:49+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-22 21:48:07+02	2020-07-22 21:49:18+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_2,1
2020-07-22 21:48:12+02	2020-07-22 21:49:22+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-07-22 21:48:50+02	2020-07-22 21:49:45+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.72.0
2020-07-22 21:49:23+02	2020-07-22 21:50:15+02	devel	libpeas	\N	libpeas-1.22.0
2020-07-22 21:50:15+02	2020-07-22 21:50:25+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-1.7.0
2020-07-22 21:49:19+02	2020-07-22 21:50:25+02	devel	libgdata	\N	libgdata-0.17.9
2020-07-22 21:50:25+02	2020-07-22 21:54:32+02	editors	emacs	full	emacs-26.3_5,3
2020-07-22 21:54:33+02	2020-07-22 21:54:48+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-07-22 21:54:49+02	2020-07-22 21:55:44+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-22 21:50:26+02	2020-07-22 21:56:17+02	graphics	gimp-app	\N	gimp-app-2.10.20_2,1
2020-07-22 21:55:44+02	2020-07-22 21:56:34+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-22 21:56:34+02	2020-07-22 21:58:09+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-07-22 21:49:46+02	2020-07-22 21:58:10+02	devel	kf5-kio	\N	kf5-kio-5.72.0
2020-07-22 21:56:18+02	2020-07-22 21:58:46+02	print	fontforge	\N	fontforge-20190801_2
2020-07-22 21:58:10+02	2020-07-22 21:58:58+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-07-22 21:58:59+02	2020-07-22 21:59:49+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-07-22 21:59:49+02	2020-07-22 22:02:38+02	x11	xorg	\N	xorg-7.7_3
2020-07-22 22:02:38+02	2020-07-22 22:04:11+02	x11	yelp	\N	yelp-3.28.1_2
2020-07-22 22:04:11+02	2020-07-22 22:05:44+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-07-22 22:05:45+02	2020-07-22 22:07:06+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-07-22 22:07:07+02	2020-07-22 22:08:49+02	x11-wm	marco	\N	marco-1.22.4
2020-07-22 22:08:49+02	2020-07-22 22:09:55+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-07-22 22:09:56+02	2020-07-22 22:12:10+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-07-22 22:12:11+02	2020-07-22 22:13:12+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-07-22 21:58:47+02	2020-07-22 22:14:18+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-07-22 22:13:12+02	2020-07-22 22:15:28+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-07-22 22:15:29+02	2020-07-22 22:15:53+02	devel	libfmt	\N	libfmt-7.0.1
2020-07-22 22:14:19+02	2020-07-22 22:16:06+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-22 22:15:54+02	2020-07-22 22:16:07+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-07-22 22:16:07+02	2020-07-22 22:16:57+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-22 22:16:07+02	2020-07-22 22:17:17+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.1
2020-07-22 22:17:17+02	2020-07-22 22:18:05+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-07-22 22:16:58+02	2020-07-22 22:19:33+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-07-22 22:19:35+02	2020-07-22 22:21:15+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-22 22:21:15+02	2020-07-22 22:22:36+02	net	freerdp	\N	freerdp-2.1.1
2020-07-22 22:22:37+02	2020-07-22 22:23:26+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-07-22 22:23:28+02	2020-07-22 22:24:17+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-22 22:18:05+02	2020-07-22 22:24:59+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-07-22 22:25:01+02	2020-07-22 22:25:34+02	security	sudo	\N	sudo-1.9.2
2020-07-22 22:24:17+02	2020-07-22 22:26:08+02	print	mftrace	\N	mftrace-1.2.18_2
2020-07-22 21:58:10+02	2020-07-22 22:26:36+02	www	chromium	\N	chromium-83.0.4103.116_1
2020-07-22 22:26:09+02	2020-07-22 22:26:54+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-07-22 22:26:55+02	2020-07-22 22:27:30+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-22 22:25:35+02	2020-07-22 22:27:35+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-07-22 22:26:39+02	2020-07-22 22:27:41+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-22 22:27:31+02	2020-07-22 22:28:20+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-22 22:27:35+02	2020-07-22 22:28:32+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-07-22 22:27:41+02	2020-07-22 22:28:50+02	x11-servers	xephyr	\N	xephyr-1.20.8_2,1
2020-07-22 22:28:50+02	2020-07-22 22:29:09+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-07-22 22:29:10+02	2020-07-22 22:29:47+02	audio	faudio	\N	FAudio-20.07
2020-07-22 22:28:33+02	2020-07-22 22:30:20+02	archivers	engrampa	\N	engrampa-1.24.0
2020-07-22 22:29:47+02	2020-07-22 22:30:52+02	audio	mate-media	\N	mate-media-1.24.0
2020-07-22 22:30:21+02	2020-07-22 22:31:14+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-07-22 22:30:52+02	2020-07-22 22:32:03+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-07-22 22:28:21+02	2020-07-22 22:32:33+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-07-22 22:32:04+02	2020-07-22 22:32:53+02	deskutils	mozo	\N	mozo-1.24.0
2020-07-22 22:32:33+02	2020-07-22 22:33:02+02	devel	fstrm	\N	fstrm-0.6.0
2020-07-22 22:33:02+02	2020-07-22 22:33:14+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-07-22 22:31:14+02	2020-07-22 22:33:41+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-07-22 22:32:53+02	2020-07-22 22:33:52+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.72.0
2020-07-22 22:33:52+02	2020-07-22 22:35:40+02	graphics	atril	\N	atril-1.22.3_7
2020-07-22 22:33:42+02	2020-07-22 22:36:32+02	editors	pluma	\N	pluma-1.24.0
2020-07-22 22:35:41+02	2020-07-22 22:36:59+02	graphics	eom	\N	eom-1.24.0
2020-07-22 22:36:33+02	2020-07-22 22:37:14+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-07-22 22:37:15+02	2020-07-22 22:38:47+02	graphics	py-gimp	\N	py27-gimp-2.10.20_2
2020-07-22 22:38:48+02	2020-07-22 22:48:10+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-07-22 22:48:11+02	2020-07-22 22:49:51+02	math	mate-calc	\N	mate-calc-1.24.0
2020-07-22 22:49:51+02	2020-07-22 22:51:23+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-22 22:37:00+02	2020-07-22 22:51:46+02	graphics	openimageio	\N	openimageio-2.1.15.0_3
2020-07-22 22:51:23+02	2020-07-22 22:52:49+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-07-22 22:52:50+02	2020-07-22 22:53:42+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-22 22:53:42+02	2020-07-22 22:54:27+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-07-22 22:33:14+02	2020-07-22 22:54:52+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-22 22:54:27+02	2020-07-22 22:55:12+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-07-22 22:54:53+02	2020-07-22 22:55:31+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-07-22 22:51:47+02	2020-07-22 22:55:50+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-07-22 22:55:12+02	2020-07-22 22:55:54+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-07-22 22:55:32+02	2020-07-22 22:56:08+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-07-22 22:55:51+02	2020-07-22 22:56:27+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-07-22 22:55:54+02	2020-07-22 22:56:40+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-07-22 22:56:27+02	2020-07-22 22:57:06+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-22 22:56:09+02	2020-07-22 22:57:07+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-05 06:53:01+02	2020-10-05 06:54:13+02	x11	mate	\N	mate-1.22.2
2020-07-22 22:56:40+02	2020-07-22 22:57:38+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-22 22:57:07+02	2020-07-22 22:58:00+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-22 22:57:38+02	2020-07-22 22:58:17+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-07-22 22:58:01+02	2020-07-22 22:59:19+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-07-22 22:58:17+02	2020-07-22 22:59:51+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-07-22 22:59:20+02	2020-07-22 23:00:19+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-07-22 22:59:51+02	2020-07-22 23:00:31+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-07-22 23:00:20+02	2020-07-22 23:00:39+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-07-22 23:00:31+02	2020-07-22 23:01:25+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-22 22:57:07+02	2020-07-22 23:01:36+02	print	lilypond	\N	lilypond-2.20.0
2020-07-22 23:00:40+02	2020-07-22 23:01:39+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-22 23:01:26+02	2020-07-22 23:01:45+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-07-22 23:01:39+02	2020-07-22 23:02:35+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-22 23:01:45+02	2020-07-22 23:02:40+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-07-22 23:02:40+02	2020-07-22 23:04:04+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-07-22 23:02:35+02	2020-07-22 23:04:21+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-07-22 23:04:05+02	2020-07-22 23:05:00+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-07-22 23:01:37+02	2020-07-22 23:06:16+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-07-22 23:05:00+02	2020-07-22 23:08:41+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-22 23:08:43+02	2020-07-22 23:11:07+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-22 23:06:17+02	2020-07-22 23:11:31+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-22 23:11:32+02	2020-07-22 23:13:12+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-22 23:04:21+02	2020-07-22 23:13:45+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-22 23:13:13+02	2020-07-22 23:14:35+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-22 23:14:36+02	2020-07-22 23:16:52+02	deskutils	calibre	\N	calibre-4.21.0
2020-07-22 23:13:46+02	2020-07-22 23:17:02+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-07-22 23:16:52+02	2020-07-22 23:17:34+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-07-22 23:17:34+02	2020-07-22 23:18:26+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-07-22 23:17:02+02	2020-07-22 23:40:31+02	devel	RStudio	\N	RStudio-1.3.1056
2020-07-22 23:40:32+02	2020-07-22 23:41:33+02	dns	bind916	\N	bind916-9.16.5_1
2020-07-22 23:18:26+02	2020-07-22 23:43:39+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-22 23:43:43+02	2020-07-22 23:45:53+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-22 23:45:55+02	2020-07-22 23:47:53+02	editors	vim	\N	vim-8.2.1110
2020-07-22 23:41:34+02	2020-07-22 23:49:07+02	editors	codelite	\N	codelite-12.0_4
2020-07-22 23:47:54+02	2020-07-22 23:55:51+02	emulators	qemu	\N	qemu-4.2.1
2020-07-22 23:55:52+02	2020-07-23 00:03:29+02	emulators	wine	\N	wine-5.0.1,1
2020-07-23 00:03:29+02	2020-07-23 00:16:57+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-07-23 00:16:57+02	2020-07-23 00:17:44+02	games	xcowsay	\N	xcowsay-1.5
2020-07-22 23:11:07+02	2020-07-23 00:47:00+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-23 00:47:00+02	2020-07-23 00:47:59+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-23 00:48:00+02	2020-07-23 00:51:17+02	graphics	inkscape	\N	inkscape-1.0_3
2020-07-23 00:51:18+02	2020-07-23 01:08:19+02	graphics	krita	\N	krita-4.3.0_2
2020-07-23 00:17:44+02	2020-07-23 01:09:28+02	graphics	blender	\N	blender-2.83.2
2020-07-23 01:09:28+02	2020-07-23 01:10:13+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-23 01:08:20+02	2020-07-23 01:10:19+02	graphics	wings	\N	wings-2.2.5
2020-07-23 01:10:14+02	2020-07-23 01:10:56+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-07-23 01:10:56+02	2020-07-23 01:11:35+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-23 01:11:35+02	2020-07-23 01:12:56+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-23 01:10:20+02	2020-07-23 01:13:42+02	math	maxima	\N	maxima-5.44.0_2
2020-07-23 01:12:57+02	2020-07-23 01:14:33+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-23 01:14:33+02	2020-07-23 01:15:51+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-07-23 01:13:43+02	2020-07-23 01:16:14+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-07-23 01:15:51+02	2020-07-23 01:16:47+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-23 01:16:47+02	2020-07-23 01:17:39+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-07-23 01:17:39+02	2020-07-23 01:18:16+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-23 01:16:15+02	2020-07-23 01:18:47+02	net	wireshark	\N	wireshark-3.2.5
2020-07-23 01:18:16+02	2020-07-23 01:18:50+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-23 01:18:48+02	2020-07-23 01:19:23+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-23 01:18:51+02	2020-07-23 01:19:25+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-23 01:19:23+02	2020-07-23 01:19:56+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-23 01:19:26+02	2020-07-23 01:20:00+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-07-23 01:19:57+02	2020-07-23 01:20:31+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-23 01:20:00+02	2020-07-23 01:20:51+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-23 01:20:31+02	2020-07-23 01:21:08+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-23 01:20:52+02	2020-07-23 01:21:26+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-23 01:21:09+02	2020-07-23 01:21:41+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-07-23 01:21:26+02	2020-07-23 01:23:00+02	print	auctex	full	auctex-emacs26-12.2
2020-07-23 01:21:42+02	2020-07-23 01:23:04+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-23 01:23:05+02	2020-07-23 01:23:40+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-23 01:23:19+02	2020-07-23 01:24:09+02	security	tor	\N	tor-0.4.3.6
2020-07-23 01:23:41+02	2020-07-23 01:24:31+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-07-23 01:24:32+02	2020-07-23 01:24:41+02	sysutils	py-py-cpuinfo	py27	py27-py-cpuinfo-7.0.0
2020-07-23 01:24:10+02	2020-07-23 01:25:13+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-07-23 01:24:41+02	2020-07-23 01:25:15+02	sysutils	tmux	\N	tmux-3.1b
2020-07-23 01:25:15+02	2020-07-23 01:26:06+02	www	links	\N	links-2.20.2,1
2020-07-23 01:26:06+02	2020-07-23 01:26:19+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-07-23 01:23:01+02	2020-07-23 01:26:36+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-07-23 01:26:19+02	2020-07-23 01:26:57+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-23 01:26:37+02	2020-07-23 01:27:40+02	x11	mate	\N	mate-1.22.2
2020-07-23 01:27:40+02	2020-07-23 01:28:08+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-07-23 01:26:58+02	2020-07-23 02:00:30+02	editors	vscode	\N	vscode-1.46.1
2020-07-23 01:25:14+02	2020-07-23 03:04:46+02	www	firefox	\N	firefox-79.0,1
2020-07-23 08:48:32+02	2020-07-23 08:49:23+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-23 08:48:32+02	2020-07-23 08:50:05+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-23 08:50:06+02	2020-07-23 08:50:13+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-23 08:49:24+02	2020-07-23 09:22:02+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44
2020-07-23 09:23:20+02	2020-07-23 09:24:40+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-23 09:24:41+02	2020-07-23 09:24:48+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-23 09:30:34+02	2020-07-23 09:45:59+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-23 09:52:04+02	2020-07-23 09:52:56+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-23 09:52:04+02	2020-07-23 09:53:33+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-23 09:53:34+02	2020-07-23 09:53:41+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-23 09:59:21+02	2020-07-23 10:31:17+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44
2020-07-23 10:31:18+02	2020-07-23 10:45:53+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-23 12:27:07+02	2020-07-23 12:27:57+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-23 12:27:07+02	2020-07-23 12:28:33+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-23 12:28:33+02	2020-07-23 12:28:40+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-23 12:34:23+02	2020-07-23 12:35:05+02	net	v2ray	\N	v2ray-4.25.0
2020-07-23 12:35:06+02	2020-07-23 12:49:22+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-25 11:18:57+02	2020-07-25 11:20:16+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-25 11:18:57+02	2020-07-25 11:25:26+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-25 11:25:26+02	2020-07-25 11:25:34+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-25 11:32:43+02	2020-07-25 11:33:10+02	security	ca_root_nss	\N	ca_root_nss-3.55
2020-07-25 11:32:43+02	2020-07-25 11:33:43+02	devel	check	\N	check-0.15.1
2020-07-25 11:33:44+02	2020-07-25 11:34:08+02	textproc	py-sphinx	py37	py37-sphinx-3.1.2,1
2020-07-25 11:34:08+02	2020-07-25 11:34:21+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-07-25 11:33:11+02	2020-07-25 11:34:40+02	ftp	curl	\N	curl-7.71.1
2020-07-25 11:34:21+02	2020-07-25 11:34:42+02	security	p11-kit	\N	p11-kit-0.23.20_1
2020-07-25 11:32:43+02	2020-07-25 11:35:02+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-07-25 11:35:02+02	2020-07-25 11:35:16+02	devel	nspr	\N	nspr-4.27
2020-07-25 11:34:40+02	2020-07-25 11:36:28+02	devel	cmake	\N	cmake-3.17.3_1
2020-07-25 11:34:43+02	2020-07-25 11:36:34+02	security	gnutls	\N	gnutls-3.6.14
2020-07-25 11:35:17+02	2020-07-25 11:37:01+02	lang	spidermonkey60	\N	spidermonkey60-60.9.0_4
2020-07-25 11:36:35+02	2020-07-25 11:37:03+02	print	cups	\N	cups-2.3.3_1
2020-07-25 11:37:02+02	2020-07-25 11:37:42+02	sysutils	polkit	\N	polkit-0.116
2020-07-25 11:36:29+02	2020-07-25 11:37:50+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-07-25 11:37:04+02	2020-07-25 11:38:00+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_6
2020-07-25 11:37:50+02	2020-07-25 11:38:31+02	net	qt5-network	\N	qt5-network-5.15.0
2020-07-25 11:37:43+02	2020-07-25 11:38:41+02	graphics	colord	\N	colord-1.3.5
2020-07-25 11:38:42+02	2020-07-25 11:40:19+02	security	nss	\N	nss-3.55
2020-07-25 11:38:32+02	2020-07-25 11:40:37+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-07-25 11:38:00+02	2020-07-25 11:41:36+02	graphics	graphviz	\N	graphviz-2.44.1
2020-07-25 11:40:38+02	2020-07-25 11:42:24+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-07-25 11:42:25+02	2020-07-25 11:42:47+02	graphics	poppler	\N	poppler-0.90.1
2020-07-25 11:40:20+02	2020-07-25 11:42:55+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-07-25 11:42:48+02	2020-07-25 11:43:07+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-07-25 11:41:37+02	2020-07-25 11:43:23+02	lang	vala	\N	vala-0.40.15,1
2020-07-25 11:43:08+02	2020-07-25 11:43:38+02	devel	libsoup	\N	libsoup-2.62.3
2020-07-25 11:43:24+02	2020-07-25 11:44:05+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-07-25 11:42:56+02	2020-07-25 11:45:23+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-07-25 11:45:24+02	2020-07-25 11:46:18+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-07-25 11:43:38+02	2020-07-25 11:46:54+02	print	texlive-base	\N	texlive-base-20150521_53
2020-07-25 11:46:18+02	2020-07-25 11:47:20+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-07-25 11:44:06+02	2020-07-25 11:47:56+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-07-25 11:47:57+02	2020-07-25 11:48:49+02	net	geoclue	\N	geoclue-2.5.5
2020-07-25 11:48:50+02	2020-07-25 11:49:48+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-07-25 11:49:49+02	2020-07-25 11:57:39+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.1_1
2020-07-25 11:57:40+02	2020-07-25 11:58:22+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-07-25 11:58:23+02	2020-07-25 11:58:58+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-07-25 11:58:58+02	2020-07-25 11:59:30+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-07-25 11:59:30+02	2020-07-25 12:00:03+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-07-25 11:47:21+02	2020-07-25 12:01:03+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1,1
2020-07-25 12:01:04+02	2020-07-25 12:02:09+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-07-25 12:00:04+02	2020-07-25 12:02:13+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.6
2020-07-25 12:02:14+02	2020-07-25 12:02:50+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-07-25 12:02:50+02	2020-07-25 12:03:24+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-07-25 12:03:25+02	2020-07-25 12:03:53+02	ftp	php73-curl	\N	php73-curl-7.3.20
2020-07-25 12:03:54+02	2020-07-25 12:04:49+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-07-25 11:46:55+02	2020-07-25 12:06:04+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-07-25 12:04:50+02	2020-07-25 12:06:04+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-07-25 12:02:10+02	2020-07-25 12:06:50+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-07-25 12:06:51+02	2020-07-25 12:07:32+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-07-25 12:07:33+02	2020-07-25 12:08:15+02	devel	kf5-kconfig	\N	kf5-kconfig-5.72.0
2020-07-25 12:06:05+02	2020-07-25 12:08:17+02	print	tex-formats	\N	tex-formats-20150521_2
2020-07-25 12:08:18+02	2020-07-25 12:08:41+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-07-25 12:08:16+02	2020-07-25 12:09:08+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-25 12:08:41+02	2020-07-25 12:09:32+02	devel	kf5-ki18n	\N	kf5-ki18n-5.72.0
2020-07-25 12:09:08+02	2020-07-25 12:09:54+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-07-25 12:09:33+02	2020-07-25 12:10:32+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-07-25 12:09:55+02	2020-07-25 12:10:51+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.72.0
2020-07-25 12:06:05+02	2020-07-25 12:10:53+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-07-25 12:10:33+02	2020-07-25 12:11:19+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-07-25 12:10:51+02	2020-07-25 12:11:27+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-07-25 12:11:20+02	2020-07-25 12:11:48+02	sysutils	hal	\N	hal-0.5.14_34
2020-07-25 12:11:27+02	2020-07-25 12:12:10+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.72.0
2020-07-25 12:11:48+02	2020-07-25 12:12:12+02	graphics	poppler-glib	\N	poppler-glib-0.90.1
2020-07-25 12:12:12+02	2020-07-25 12:13:21+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.72.0
2020-07-25 12:10:54+02	2020-07-25 12:13:57+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-07-25 12:13:22+02	2020-07-25 12:14:52+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-07-25 12:13:58+02	2020-07-25 12:14:56+02	devel	kf5-kcrash	\N	kf5-kcrash-5.72.0
2020-07-25 12:14:53+02	2020-07-25 12:15:45+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.72.0
2020-07-25 12:14:57+02	2020-07-25 12:15:50+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-07-25 12:15:51+02	2020-07-25 12:16:40+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-07-25 12:15:46+02	2020-07-25 12:16:49+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-07-25 12:16:41+02	2020-07-25 12:17:34+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-07-25 12:16:50+02	2020-07-25 12:18:07+02	databases	mysql57-client	\N	mysql57-client-5.7.31
2020-07-25 12:17:35+02	2020-07-25 12:18:31+02	devel	kf5-kauth	\N	kf5-kauth-5.72.0
2020-07-25 12:18:08+02	2020-07-25 12:19:15+02	devel	kf5-kservice	\N	kf5-kservice-5.72.0
2020-07-25 12:18:32+02	2020-07-25 12:20:14+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-07-25 12:19:16+02	2020-07-25 12:20:21+02	net	remmina	\N	remmina-1.3.9
2020-07-25 12:20:21+02	2020-07-25 12:21:09+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.72.0
2020-07-25 12:21:10+02	2020-07-25 12:22:06+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-07-25 12:22:06+02	2020-07-25 12:23:34+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.72.0
2020-07-25 12:23:35+02	2020-07-25 12:24:40+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.72.0
2020-07-25 12:24:41+02	2020-07-25 12:25:59+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-07-25 12:25:59+02	2020-07-25 12:29:50+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-07-25 12:29:50+02	2020-07-25 12:32:05+02	math	gnuplot	\N	gnuplot-5.2.8_2
2020-07-25 12:32:05+02	2020-07-25 12:33:08+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-25 12:33:10+02	2020-07-25 12:34:16+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-07-25 12:34:17+02	2020-07-25 12:35:29+02	security	gnupg	\N	gnupg-2.2.21
2020-07-25 12:35:30+02	2020-07-25 12:36:53+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.72.0
2020-07-25 12:36:54+02	2020-07-25 12:39:42+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-07-25 12:39:42+02	2020-07-25 12:40:43+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-07-25 12:40:44+02	2020-07-25 12:42:03+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-07-25 12:42:03+02	2020-07-25 12:43:36+02	graphics	netpbm	\N	netpbm-10.91.01
2020-07-25 12:43:37+02	2020-07-25 12:51:44+02	math	vtk8	\N	vtk8-8.2.0
2020-07-25 12:51:45+02	2020-07-25 12:52:29+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-25 12:52:30+02	2020-07-25 12:57:26+02	net	libmateweather	\N	libmateweather-1.24.0
2020-07-25 12:57:26+02	2020-07-25 12:57:53+02	security	gpgme	\N	gpgme-1.14.0
2020-07-25 12:57:54+02	2020-07-25 12:59:06+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.72.0
2020-07-25 12:59:07+02	2020-07-25 13:00:27+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-07-25 13:00:28+02	2020-07-25 13:01:36+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.72.0
2020-07-25 13:01:37+02	2020-07-25 13:02:31+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-07-25 13:02:32+02	2020-07-25 13:04:46+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-07-25 13:04:47+02	2020-07-25 13:13:29+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-07-25 13:13:30+02	2020-07-25 13:14:12+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-07-25 13:14:13+02	2020-07-25 13:16:14+02	devel	git	\N	git-2.27.0
2020-07-25 13:16:15+02	2020-07-25 13:17:12+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-07-25 13:17:13+02	2020-07-25 13:17:59+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-07-25 13:17:59+02	2020-07-25 13:18:53+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-07-25 13:18:54+02	2020-07-25 13:20:05+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-07-25 13:20:06+02	2020-07-25 13:21:54+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-07-25 13:21:55+02	2020-07-25 13:24:12+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-07-25 13:24:12+02	2020-07-25 13:24:38+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-07-25 13:24:38+02	2020-07-25 13:25:23+02	sysutils	upower	\N	upower-0.99.11
2020-07-25 13:25:23+02	2020-07-25 13:25:58+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-07-25 13:25:59+02	2020-07-25 13:37:13+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-25 13:37:14+02	2020-07-25 13:38:34+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.72.0
2020-07-25 13:38:35+02	2020-07-25 13:41:55+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-07-25 13:41:56+02	2020-07-25 13:43:03+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.72.0
2020-07-25 13:43:04+02	2020-07-25 13:44:27+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.72.0
2020-07-25 13:44:31+02	2020-07-25 13:45:07+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-07-25 13:45:07+02	2020-07-25 13:53:28+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-07-25 13:53:29+02	2020-07-25 13:54:11+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-07-25 12:12:11+02	2020-07-25 13:55:17+02	math	suitesparse	\N	suitesparse-5.8.1
2020-07-25 13:55:18+02	2020-07-25 13:56:37+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-07-25 13:56:38+02	2020-07-25 13:58:08+02	devel	kf5-knotifications	\N	kf5-knotifications-5.72.0
2020-07-25 13:58:08+02	2020-07-25 13:58:55+02	devel	librest	\N	rest-0.7.93
2020-07-25 13:58:56+02	2020-07-25 14:00:09+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-07-25 14:00:10+02	2020-07-25 14:00:48+02	graphics	poppler-utils	\N	poppler-utils-0.90.1
2020-07-25 14:00:49+02	2020-07-25 14:01:43+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-07-25 14:01:43+02	2020-07-25 14:02:37+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-07-25 14:02:38+02	2020-07-25 14:03:33+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-07-25 14:03:33+02	2020-07-25 14:08:10+02	www	node	\N	node-14.5.0
2020-07-25 14:08:11+02	2020-07-25 14:09:39+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-07-25 14:09:39+02	2020-07-25 14:11:07+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-07-25 13:54:13+02	2020-07-25 14:12:59+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-25 14:11:07+02	2020-07-25 14:13:00+02	x11-fm	caja	\N	caja-1.22.3_1
2020-07-25 14:13:01+02	2020-07-25 14:13:54+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-07-25 14:12:59+02	2020-07-25 14:14:18+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.72.0
2020-07-25 14:13:54+02	2020-07-25 14:15:17+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-07-25 14:14:19+02	2020-07-25 14:15:25+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-07-25 14:15:25+02	2020-07-25 14:16:26+02	devel	kf5-solid	\N	kf5-solid-5.72.0
2020-07-25 14:15:18+02	2020-07-25 14:16:31+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.72.0
2020-07-25 14:16:32+02	2020-07-25 14:18:12+02	graphics	gegl	\N	gegl-0.4.24_2
2020-07-25 14:16:26+02	2020-07-25 14:18:17+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-07-25 14:18:17+02	2020-07-25 14:20:05+02	mail	mailutils	\N	mailutils-3.9
2020-07-25 14:20:05+02	2020-07-25 14:21:38+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-07-25 14:21:39+02	2020-07-25 14:21:53+02	net	liboauth	\N	liboauth-1.0.3_4
2020-07-25 14:18:13+02	2020-07-25 14:22:59+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-07-25 14:22:59+02	2020-07-25 14:24:05+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-07-25 14:24:05+02	2020-07-25 14:24:54+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-07-25 14:24:55+02	2020-07-25 14:27:17+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-07-25 14:27:18+02	2020-07-25 14:28:07+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-07-25 14:28:08+02	2020-07-25 14:29:38+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.72.0
2020-07-25 14:29:39+02	2020-07-25 14:31:59+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-07-25 14:31:59+02	2020-07-25 14:32:57+02	x11	kf5-kded	\N	kf5-kded-5.72.0_1
2020-07-25 14:21:53+02	2020-07-25 14:33:07+02	net	samba410	\N	samba410-4.10.17
2020-07-25 14:33:07+02	2020-07-25 14:34:03+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-07-25 14:32:58+02	2020-07-25 14:34:04+02	x11	zenity	\N	zenity-3.32.0
2020-07-25 14:34:05+02	2020-07-25 14:34:58+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.72.0
2020-07-25 14:34:03+02	2020-07-25 14:35:38+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-07-25 14:35:39+02	2020-07-25 14:37:29+02	devel	libgdata	\N	libgdata-0.17.9
2020-07-25 14:34:58+02	2020-07-25 14:39:00+02	devel	kf5-kio	\N	kf5-kio-5.72.0
2020-07-25 14:37:30+02	2020-07-25 14:39:02+02	devel	libpeas	\N	libpeas-1.22.0
2020-07-25 14:39:02+02	2020-07-25 14:39:19+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-07-25 14:39:01+02	2020-07-25 14:45:08+02	editors	emacs	full	emacs-26.3_5,3
2020-07-25 14:39:20+02	2020-07-25 14:48:01+02	graphics	gimp-app	\N	gimp-app-2.10.20_2,1
2020-07-25 14:48:02+02	2020-07-25 14:49:13+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-07-25 14:49:14+02	2020-07-25 14:49:39+02	net	gssdp	\N	gssdp-1.2.3
2020-07-25 14:45:09+02	2020-07-25 14:50:13+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-29 00:30:58+02	2020-08-29 00:31:26+02	devel	tig	\N	tig-2.5.1
2020-07-25 14:49:39+02	2020-07-25 14:50:37+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-25 14:50:14+02	2020-07-25 14:51:07+02	print	cups-filters	\N	cups-filters-1.27.5_1
2020-07-25 14:50:38+02	2020-07-25 14:54:15+02	print	fontforge	\N	fontforge-20190801_2
2020-07-25 14:54:16+02	2020-07-25 15:00:14+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-07-25 14:51:08+02	2020-07-25 15:01:56+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-07-25 15:00:17+02	2020-07-25 15:02:53+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-07-25 15:02:56+02	2020-07-25 15:05:14+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-07-25 15:01:58+02	2020-07-25 15:05:53+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-07-25 15:05:16+02	2020-07-25 15:06:23+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-07-25 15:06:25+02	2020-07-25 15:18:23+02	www	node12	\N	node12-12.18.2_1
2020-07-25 15:18:27+02	2020-07-25 16:00:12+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-07-25 16:00:14+02	2020-07-25 16:01:48+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0
2020-07-25 16:01:49+02	2020-07-25 16:05:38+02	x11	xorg	\N	xorg-7.7_3
2020-07-25 16:05:39+02	2020-07-25 16:07:29+02	x11	yelp	\N	yelp-3.28.1_2
2020-07-25 16:07:31+02	2020-07-25 16:08:44+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-07-25 16:08:44+02	2020-07-25 16:09:25+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-07-25 16:09:26+02	2020-07-25 16:10:28+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-07-25 16:10:29+02	2020-07-25 16:11:53+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-07-25 15:05:54+02	2020-07-25 16:12:31+02	www	chromium	\N	chromium-83.0.4103.116_1
2020-07-25 16:12:33+02	2020-07-25 16:13:37+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-07-25 16:11:53+02	2020-07-25 16:13:55+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-07-25 16:13:37+02	2020-07-25 16:14:35+02	x11-wm	marco	\N	marco-1.22.4
2020-07-25 16:13:55+02	2020-07-25 16:14:37+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-07-25 16:14:38+02	2020-07-25 16:15:22+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-07-25 16:14:36+02	2020-07-25 16:15:52+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-07-25 16:15:23+02	2020-07-25 16:17:05+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-07-25 16:15:53+02	2020-07-25 16:17:12+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-25 12:21:57+02	2020-07-25 16:17:27+02	lang	rust	\N	rust-1.45.0
2020-07-25 16:17:13+02	2020-07-25 16:17:44+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-07-25 16:17:45+02	2020-07-25 16:18:45+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.1
2020-07-25 16:17:28+02	2020-07-25 16:18:51+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-25 16:17:06+02	2020-07-25 16:19:10+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-07-25 16:18:45+02	2020-07-25 16:19:10+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-07-25 16:18:51+02	2020-07-25 16:19:30+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-07-25 16:19:10+02	2020-07-25 16:19:47+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-07-25 16:19:10+02	2020-07-25 16:21:25+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-07-25 16:19:30+02	2020-07-25 16:21:59+02	lang	python38	\N	python38-3.8.5
2020-07-25 16:21:26+02	2020-07-25 16:24:57+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-07-25 16:24:58+02	2020-07-25 16:26:13+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-25 16:26:13+02	2020-07-25 16:27:05+02	net	freerdp	\N	freerdp-2.1.1
2020-07-25 16:27:06+02	2020-07-25 16:27:26+02	net	gupnp	\N	gupnp-1.1.1_1
2020-07-25 16:21:59+02	2020-07-25 16:27:31+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-07-25 16:27:32+02	2020-07-25 16:28:16+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-07-25 16:27:26+02	2020-07-25 16:28:21+02	net	libcmis	\N	libcmis-0.5.2_1
2020-07-25 16:28:22+02	2020-07-25 16:29:04+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-25 16:28:16+02	2020-07-25 16:29:20+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-25 16:29:20+02	2020-07-25 16:29:37+02	print	libspectre	\N	libspectre-0.2.9
2020-07-25 16:29:38+02	2020-07-25 16:31:27+02	print	mftrace	\N	mftrace-1.2.18_2
2020-07-25 16:29:05+02	2020-07-25 16:31:48+02	print	gutenprint	\N	gutenprint-5.3.3
2020-07-25 16:19:48+02	2020-07-25 16:32:03+02	math	R	\N	R-4.0.2
2020-07-25 16:32:04+02	2020-07-25 16:32:34+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-07-25 16:32:34+02	2020-07-25 16:32:56+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-07-25 16:32:57+02	2020-07-25 16:33:11+02	sysutils	hal-info	\N	hal-info-20091130
2020-07-25 16:33:12+02	2020-07-25 16:35:18+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-07-25 16:35:19+02	2020-07-25 16:36:01+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-07-25 16:31:27+02	2020-07-25 16:36:12+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-07-25 16:31:48+02	2020-07-25 16:36:22+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-07-25 16:36:23+02	2020-07-25 16:36:53+02	textproc	redland	\N	redland-1.0.17_4
2020-07-25 16:36:02+02	2020-07-25 16:37:41+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-07-25 16:36:13+02	2020-07-25 16:37:56+02	textproc	dblatex	\N	dblatex-0.3.11
2020-07-25 16:37:41+02	2020-07-25 16:38:16+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-07-25 16:36:53+02	2020-07-25 16:38:22+02	www	node10	\N	node10-10.21.0
2020-07-25 16:38:17+02	2020-07-25 16:38:43+02	www	youtube_dl	\N	youtube_dl-2020.06.16.1
2020-07-25 16:37:56+02	2020-07-25 16:38:47+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-25 16:38:22+02	2020-07-25 16:39:18+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-07-25 16:38:47+02	2020-07-25 16:39:26+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-07-25 16:38:43+02	2020-07-25 16:39:31+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-07-25 16:39:19+02	2020-07-25 16:40:02+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-07-25 16:39:26+02	2020-07-25 16:40:18+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-07-25 16:40:18+02	2020-07-25 16:40:49+02	audio	faudio	\N	FAudio-20.07
2020-07-25 16:40:03+02	2020-07-25 16:42:03+02	archivers	engrampa	\N	engrampa-1.24.0
2020-07-25 16:40:49+02	2020-07-25 16:42:03+02	audio	mate-media	\N	mate-media-1.24.0
2020-07-25 16:42:04+02	2020-07-25 16:42:32+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.14.0
2020-07-25 16:42:04+02	2020-07-25 16:43:01+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-07-25 16:42:32+02	2020-07-25 16:43:40+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-07-25 16:39:31+02	2020-07-25 16:43:46+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-07-25 16:43:40+02	2020-07-25 16:44:26+02	deskutils	mozo	\N	mozo-1.24.0
2020-07-25 16:43:46+02	2020-07-25 16:44:35+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-07-25 16:44:26+02	2020-07-25 16:45:12+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.72.0
2020-07-25 16:44:36+02	2020-07-25 16:45:16+02	devel	py-game	py37	py37-game-1.9.6_1
2020-07-25 16:43:02+02	2020-07-25 16:45:23+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-07-25 16:45:12+02	2020-07-25 16:45:24+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-07-25 16:45:17+02	2020-07-25 16:49:58+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3_1
2020-07-25 16:49:58+02	2020-07-25 16:53:50+02	editors	pluma	\N	pluma-1.24.0
2020-07-25 16:53:51+02	2020-07-25 16:56:18+02	graphics	atril	\N	atril-1.22.3_7
2020-07-25 16:56:18+02	2020-07-25 16:58:16+02	graphics	eom	\N	eom-1.24.0
2020-07-25 16:58:16+02	2020-07-25 16:59:36+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-25 16:59:37+02	2020-07-25 17:00:16+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-07-25 17:00:17+02	2020-07-25 17:01:28+02	graphics	py-gimp	\N	py27-gimp-2.10.20_2
2020-07-25 16:45:24+02	2020-07-25 17:05:01+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-07-25 16:45:25+02	2020-07-25 17:05:27+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-25 17:01:28+02	2020-07-25 17:08:44+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-07-25 17:08:44+02	2020-07-25 17:10:11+02	math	mate-calc	\N	mate-calc-1.24.0
2020-07-25 17:05:02+02	2020-07-25 17:11:02+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-07-25 17:10:12+02	2020-07-25 17:11:08+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-07-25 17:11:09+02	2020-07-25 17:11:43+02	multimedia	gmmlib	\N	gmmlib-20.2.3
2020-07-25 17:11:44+02	2020-07-25 17:15:24+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-25 17:11:03+02	2020-07-25 17:15:42+02	misc	openvdb	\N	openvdb-7.0.0_4
2020-07-25 17:15:24+02	2020-07-25 17:17:02+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-07-25 17:17:02+02	2020-07-25 17:17:55+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-25 17:17:56+02	2020-07-25 17:18:44+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-07-25 17:18:45+02	2020-07-25 17:19:29+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-07-25 17:19:30+02	2020-07-25 17:20:10+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-07-25 17:15:42+02	2020-07-25 17:20:11+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-07-25 17:05:28+02	2020-07-25 17:20:32+02	lang	julia11	\N	julia11-1.1.1_5
2020-07-25 17:20:11+02	2020-07-25 17:20:48+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-07-25 17:20:10+02	2020-07-25 17:20:51+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-07-25 17:20:33+02	2020-07-25 17:21:09+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-07-25 17:20:48+02	2020-07-25 17:21:34+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-07-25 17:21:10+02	2020-07-25 17:21:51+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-25 17:20:52+02	2020-07-25 17:21:52+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-07-25 17:21:35+02	2020-07-25 17:22:33+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-25 17:21:51+02	2020-07-25 17:22:45+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-07-25 17:21:52+02	2020-07-25 17:26:22+02	print	lilypond	\N	lilypond-2.20.0
2020-07-25 17:22:45+02	2020-07-25 17:27:35+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-07-25 17:22:34+02	2020-07-25 17:27:37+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-07-25 17:26:23+02	2020-07-25 17:27:49+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-07-25 17:27:50+02	2020-07-25 17:28:13+02	print	transfig	\N	transfig-3.2.5e_4
2020-07-25 17:28:13+02	2020-07-25 17:28:54+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-07-25 17:28:54+02	2020-07-25 17:30:12+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-07-25 17:30:12+02	2020-07-25 17:31:34+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-07-25 17:31:35+02	2020-07-25 17:31:46+02	sysutils	py-psutil	py27	py27-psutil-5.7.2
2020-07-25 17:31:46+02	2020-07-25 17:32:33+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-07-25 17:32:33+02	2020-07-25 17:32:56+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-07-25 17:27:35+02	2020-07-25 17:32:58+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-07-25 17:32:57+02	2020-07-25 17:33:30+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-07-25 17:32:58+02	2020-07-25 17:33:36+02	www	mattermost-webapp	\N	mattermost-webapp-5.21.0
2020-07-25 17:33:31+02	2020-07-25 17:33:51+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-07-25 17:33:36+02	2020-07-25 17:34:24+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-25 17:34:25+02	2020-07-25 17:34:43+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-07-25 17:33:51+02	2020-07-25 17:34:46+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-25 17:34:44+02	2020-07-25 17:34:53+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-07-25 17:34:54+02	2020-07-25 17:35:51+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-25 17:35:52+02	2020-07-25 17:36:55+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-07-25 17:36:55+02	2020-07-25 17:38:29+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-07-25 17:34:46+02	2020-07-25 17:38:56+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-07-25 17:38:29+02	2020-07-25 17:39:31+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-07-25 17:38:57+02	2020-07-25 17:39:46+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-07-25 17:39:31+02	2020-07-25 17:40:03+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-07-25 17:39:47+02	2020-07-25 17:40:44+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-07-25 17:40:45+02	2020-07-25 17:45:37+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-25 17:27:37+02	2020-07-25 17:46:04+02	print	texlive-docs	\N	texlive-docs-20150523
2020-07-25 17:46:05+02	2020-07-25 17:48:21+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-25 17:40:04+02	2020-07-25 17:50:12+02	astro	stellarium	\N	stellarium-0.20.2
2020-07-25 17:45:38+02	2020-07-25 17:50:16+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-25 17:50:13+02	2020-07-25 17:51:54+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-25 17:50:17+02	2020-07-25 17:51:56+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-25 17:51:56+02	2020-07-25 17:52:09+02	databases	pgtop	\N	pgtop-0.05_5
2020-07-25 17:52:09+02	2020-07-25 17:54:19+02	deskutils	calibre	\N	calibre-4.21.0
2020-07-25 17:54:20+02	2020-07-25 17:55:10+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-07-25 17:51:54+02	2020-07-25 17:55:14+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-07-25 17:55:15+02	2020-07-25 17:55:52+02	devel	bzr	py27	bzr-2.7.0.6622
2020-07-25 17:55:52+02	2020-07-25 17:56:47+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-07-25 17:55:11+02	2020-07-25 18:21:54+02	devel	RStudio	\N	RStudio-1.3.1056
2020-07-25 18:21:55+02	2020-07-25 18:22:28+02	devel	mercurial	py27	mercurial-5.1.2
2020-07-25 18:22:29+02	2020-07-25 18:22:47+02	devel	tig	\N	tig-2.5.1
2020-07-25 17:56:47+02	2020-07-25 18:26:51+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-25 18:26:56+02	2020-07-25 18:29:24+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-25 18:22:47+02	2020-07-25 18:30:59+02	editors	codelite	\N	codelite-12.0_4
2020-07-25 18:29:24+02	2020-07-25 18:31:14+02	editors	vim	\N	vim-8.2.1110
2020-07-25 18:30:59+02	2020-07-25 18:40:03+02	emulators	qemu	\N	qemu-4.2.1
2020-07-25 18:40:05+02	2020-07-25 18:48:34+02	emulators	wine	\N	wine-5.0.1,1
2020-07-25 18:48:34+02	2020-07-25 19:03:48+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-07-25 19:03:49+02	2020-07-25 19:05:39+02	games	alienarena	\N	alienarena-2013.766_9
2020-07-25 19:05:40+02	2020-07-25 19:06:25+02	games	monsterz	\N	monsterz-0.7.1_15
2020-07-25 19:06:25+02	2020-07-25 19:07:04+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-07-25 19:07:05+02	2020-07-25 19:07:41+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-07-25 19:07:41+02	2020-07-25 19:08:37+02	games	xboard	\N	xboard-4.9.1_1
2020-07-25 19:08:37+02	2020-07-25 19:09:23+02	games	xcowsay	\N	xcowsay-1.5
2020-07-25 19:09:24+02	2020-07-25 19:15:21+02	graphics	blender	\N	blender-2.83.2
2020-07-25 19:15:22+02	2020-07-25 19:17:44+02	graphics	dia	\N	dia-0.97.3_1,1
2020-07-25 19:17:44+02	2020-07-25 19:19:00+02	graphics	gimp	\N	gimp-2.10.20,2
2020-07-25 19:19:01+02	2020-07-25 19:23:06+02	graphics	inkscape	\N	inkscape-1.0_3
2020-07-25 18:31:14+02	2020-07-25 19:27:33+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44
2020-07-25 19:27:34+02	2020-07-25 19:31:16+02	graphics	wings	\N	wings-2.2.5
2020-07-25 19:31:16+02	2020-07-25 19:32:07+02	graphics	xpdf	\N	xpdf-4.02,1
2020-07-25 19:23:07+02	2020-07-25 19:32:15+02	graphics	krita	\N	krita-4.3.0_2
2020-07-25 17:48:21+02	2020-07-25 19:32:38+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-25 19:32:08+02	2020-07-25 19:32:56+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-07-25 19:32:57+02	2020-07-25 19:33:23+02	lang	julia	\N	julia-1.0.3_3
2020-07-25 19:32:16+02	2020-07-25 19:52:32+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.3
2020-07-25 19:52:33+02	2020-07-25 19:57:20+02	math	maxima	\N	maxima-5.44.0_2
2020-07-25 19:57:20+02	2020-07-25 19:58:04+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-25 19:32:39+02	2020-07-25 19:58:25+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.2
2020-07-25 19:33:23+02	2020-07-25 20:00:34+02	lang	mono	\N	mono-5.10.1.57_3
2020-07-25 19:58:25+02	2020-07-25 20:00:36+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-28 17:19:00+02	2020-07-28 17:26:14+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-07-25 19:58:05+02	2020-07-25 20:02:32+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p3
2020-07-25 20:00:35+02	2020-07-25 20:02:37+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-25 20:00:37+02	2020-07-25 20:02:43+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-07-25 20:02:33+02	2020-07-25 20:02:52+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-07-25 20:02:43+02	2020-07-25 20:03:55+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-25 20:02:37+02	2020-07-25 20:04:16+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-07-25 20:03:55+02	2020-07-25 20:04:52+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-07-25 20:04:17+02	2020-07-25 20:05:12+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-25 20:04:52+02	2020-07-25 20:05:31+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-25 20:05:12+02	2020-07-25 20:05:50+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-25 20:02:53+02	2020-07-25 20:06:02+02	net	wireshark	\N	wireshark-3.2.5
2020-07-25 20:05:32+02	2020-07-25 20:06:10+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-25 20:05:51+02	2020-07-25 20:06:28+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-25 20:06:02+02	2020-07-25 20:06:39+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-07-25 20:06:10+02	2020-07-25 20:06:47+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-25 20:06:40+02	2020-07-25 20:07:19+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-25 20:06:48+02	2020-07-25 20:07:25+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-25 20:06:29+02	2020-07-25 20:07:26+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-25 20:07:26+02	2020-07-25 20:07:36+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-07-25 20:07:20+02	2020-07-25 20:07:56+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-07-25 20:07:26+02	2020-07-25 20:08:08+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-07-25 20:07:56+02	2020-07-25 20:08:17+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-07-25 20:08:08+02	2020-07-25 20:08:33+02	print	gv	\N	gv-3.7.4_5
2020-07-25 20:07:36+02	2020-07-25 20:09:29+02	print	auctex	full	auctex-emacs26-12.2
2020-07-25 20:08:18+02	2020-07-25 20:09:59+02	print	lyx	\N	lyx-2.3.4.2_1
2020-07-25 20:09:59+02	2020-07-25 20:10:56+02	security	clamav	\N	clamav-0.102.4,1
2020-07-25 20:10:57+02	2020-07-25 20:11:42+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-07-25 20:08:33+02	2020-07-25 20:12:27+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-07-25 20:11:42+02	2020-07-25 20:12:29+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-07-25 20:12:28+02	2020-07-25 20:12:56+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-07-25 20:12:57+02	2020-07-25 20:13:13+02	sysutils	parallel	\N	parallel-20200722
2020-07-25 20:09:30+02	2020-07-25 20:13:16+02	print	texlive-full	\N	texlive-full-20150521
2020-07-25 20:12:30+02	2020-07-25 20:13:17+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-07-25 20:13:14+02	2020-07-25 20:13:25+02	sysutils	py-glances	py27	py27-glances-3.1.4
2020-07-25 20:13:26+02	2020-07-25 20:14:07+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-07-25 20:14:08+02	2020-07-25 20:14:28+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-07-25 20:13:18+02	2020-07-25 20:14:29+02	www	mattermost-server	\N	mattermost-server-5.21.0
2020-07-25 20:14:29+02	2020-07-25 20:14:38+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-07-25 20:14:30+02	2020-07-25 20:15:10+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-07-25 20:15:10+02	2020-07-25 20:15:29+02	x11	xlockmore	\N	xlockmore-5.65
2020-07-25 20:14:39+02	2020-07-25 20:15:47+02	x11	mate	\N	mate-1.22.2
2020-07-25 20:15:29+02	2020-07-25 20:16:55+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-07-25 20:16:56+02	2020-07-25 20:17:45+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-07-25 20:17:45+02	2020-07-25 20:18:02+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-07-25 20:15:48+02	2020-07-25 20:20:42+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-07-25 20:20:43+02	2020-07-25 20:21:28+02	graphics	xfig	\N	xfig-3.2.7a
2020-07-25 20:18:02+02	2020-07-25 20:55:03+02	editors	vscode	\N	vscode-1.46.1
2020-07-25 20:13:17+02	2020-07-25 20:57:25+02	www	firefox	\N	firefox-79.0,1
2020-07-25 20:57:27+02	2020-07-25 21:11:51+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-26 09:30:16+02	2020-07-26 09:31:14+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-26 09:30:16+02	2020-07-26 09:34:58+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-26 09:34:59+02	2020-07-26 09:35:06+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-26 09:40:43+02	2020-07-26 09:41:01+02	net	liveMedia	\N	liveMedia-2020.07.21,2
2020-07-26 09:40:43+02	2020-07-26 09:44:00+02	www	mattermost-webapp	\N	mattermost-webapp-5.25.0
2020-07-26 09:41:02+02	2020-07-26 09:44:45+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-07-26 09:44:01+02	2020-07-26 09:45:25+02	www	mattermost-server	\N	mattermost-server-5.25.0
2020-07-26 09:44:46+02	2020-07-26 09:45:57+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-26 09:45:58+02	2020-07-26 10:00:13+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-28 10:48:30+02	2020-07-28 10:49:35+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-28 10:48:30+02	2020-07-28 10:53:35+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-28 10:53:36+02	2020-07-28 10:53:43+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-28 10:59:47+02	2020-07-28 10:59:58+02	devel	xxhash	\N	xxhash-0.7.4
2020-07-28 10:59:58+02	2020-07-28 11:00:26+02	net	rsync	\N	rsync-3.2.2
2020-07-28 10:59:47+02	2020-07-28 11:01:50+02	net	freerdp	\N	freerdp-2.2.0
2020-07-28 11:01:51+02	2020-07-28 11:02:32+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-07-28 11:02:32+02	2020-07-28 11:03:30+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-28 11:03:31+02	2020-07-28 11:19:35+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-28 15:03:20+02	2020-07-28 15:04:36+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-28 15:03:20+02	2020-07-28 15:05:30+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-28 15:05:31+02	2020-07-28 15:05:43+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-28 15:12:10+02	2020-07-28 15:12:32+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-07-28 15:12:32+02	2020-07-28 15:13:41+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-07-28 15:13:42+02	2020-07-28 15:14:19+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-28 15:14:19+02	2020-07-28 15:29:32+02	local	E590T-localbase	\N	E590T-localbase-2020071601
2020-07-28 00:34:57+02	2020-07-28 00:35:08+02	textproc	lowdown	\N	lowdown-0.7.2
2020-07-28 17:11:03+02	2020-07-28 17:12:11+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-28 17:11:03+02	2020-07-28 17:12:53+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-28 17:12:54+02	2020-07-28 17:13:01+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-28 17:19:00+02	2020-07-28 17:19:27+02	audio	libsndfile	\N	libsndfile-1.0.29.p.20200620
2020-07-28 17:19:28+02	2020-07-28 17:19:41+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-07-28 17:19:41+02	2020-07-28 17:19:58+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-07-28 17:19:58+02	2020-07-28 17:20:25+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-07-28 17:19:00+02	2020-07-28 17:20:59+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-07-28 17:20:59+02	2020-07-28 17:21:34+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-07-28 17:21:34+02	2020-07-28 17:22:20+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-07-28 17:22:21+02	2020-07-28 17:24:04+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-07-28 17:24:05+02	2020-07-28 17:25:26+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.72.0
2020-07-28 17:25:27+02	2020-07-28 17:26:18+02	audio	jack	\N	jackit-0.125.0_10
2020-07-28 17:26:19+02	2020-07-28 17:26:54+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-07-28 17:26:14+02	2020-07-28 17:27:09+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-07-28 17:26:55+02	2020-07-28 17:27:49+02	devel	kf5-knotifications	\N	kf5-knotifications-5.72.0
2020-07-28 17:20:26+02	2020-07-28 17:27:50+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1,1
2020-07-28 17:27:09+02	2020-07-28 17:28:02+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-07-28 17:27:50+02	2020-07-28 17:28:24+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-07-28 17:27:50+02	2020-07-28 17:28:51+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.72.0
2020-07-28 17:28:25+02	2020-07-28 17:28:54+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-07-28 17:28:03+02	2020-07-28 17:34:25+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-07-28 17:28:54+02	2020-07-28 17:35:56+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-07-28 17:35:57+02	2020-07-28 17:37:29+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.72.0
2020-07-28 17:37:30+02	2020-07-28 17:38:05+02	graphics	simage	\N	simage-1.7.0_7
2020-07-28 17:38:06+02	2020-07-28 17:40:13+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.72.0
2020-07-28 17:40:14+02	2020-07-28 17:42:05+02	audio	sox	\N	sox-14.4.2_4
2020-07-28 17:42:06+02	2020-07-28 17:46:27+02	devel	kf5-kio	\N	kf5-kio-5.72.0
2020-07-28 17:28:51+02	2020-07-28 17:47:20+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-07-28 17:46:28+02	2020-07-28 17:49:27+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-07-28 17:47:22+02	2020-07-28 17:50:06+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-07-28 17:34:27+02	2020-07-28 17:53:07+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-28 17:50:07+02	2020-07-28 17:54:30+02	editors	emacs	full	emacs-26.3_5,3
2020-07-28 17:49:28+02	2020-07-28 17:56:08+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-07-28 17:56:09+02	2020-07-28 17:57:08+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-07-28 17:54:31+02	2020-07-28 17:58:51+02	graphics	opencv	\N	opencv-3.4.1_35
2020-07-28 17:57:12+02	2020-07-28 17:58:58+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-07-28 17:53:08+02	2020-07-28 18:02:22+02	graphics	Coin	\N	Coin-4.0.0_1
2020-07-28 17:58:59+02	2020-07-28 18:05:03+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-07-28 18:05:04+02	2020-07-28 18:06:46+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-07-28 18:02:22+02	2020-07-28 18:09:09+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-07-28 18:09:10+02	2020-07-28 18:09:29+02	audio	twolame	\N	twolame-0.4.0
2020-07-28 18:09:29+02	2020-07-28 18:13:06+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-07-28 18:13:07+02	2020-07-28 18:15:17+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-07-28 18:15:20+02	2020-07-28 18:17:08+02	multimedia	mlt	\N	mlt-6.20.0
2020-07-28 18:17:08+02	2020-07-28 18:18:18+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-07-28 18:06:47+02	2020-07-28 18:19:19+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-07-28 18:18:18+02	2020-07-28 18:21:57+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-07-28 18:21:57+02	2020-07-28 18:23:36+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-07-28 18:23:37+02	2020-07-28 18:24:29+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-07-28 18:24:30+02	2020-07-28 18:27:31+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-07-28 18:27:32+02	2020-07-28 18:27:58+02	audio	dssi	\N	dssi-1.1.1_8
2020-07-28 18:27:59+02	2020-07-28 18:28:53+02	audio	faudio	\N	FAudio-20.07
2020-07-28 18:28:54+02	2020-07-28 18:30:32+02	audio	mate-media	\N	mate-media-1.24.0
2020-07-28 18:30:32+02	2020-07-28 18:32:40+02	devel	libarea	\N	libarea-20160313_13
2020-07-28 17:58:52+02	2020-07-28 18:37:45+02	www	chromium	\N	chromium-83.0.4103.116_1
2020-07-28 18:37:47+02	2020-07-28 18:39:18+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-07-28 18:39:19+02	2020-07-28 18:50:19+02	french	med	\N	fr-med-4.0.0
2020-07-28 18:50:20+02	2020-07-28 18:52:06+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-07-28 18:52:07+02	2020-07-28 18:59:40+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-07-28 18:32:41+02	2020-07-28 18:59:41+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-07-28 18:59:40+02	2020-07-28 19:00:21+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-07-28 18:59:43+02	2020-07-28 19:01:24+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-07-28 19:00:21+02	2020-07-28 19:02:43+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-07-28 19:02:44+02	2020-07-28 19:03:55+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-07-28 19:03:55+02	2020-07-28 19:04:56+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-07-28 19:04:57+02	2020-07-28 19:06:38+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-07-28 19:01:24+02	2020-07-28 19:08:18+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-07-28 19:08:19+02	2020-07-28 19:09:42+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-07-28 19:09:43+02	2020-07-28 19:11:13+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-07-28 19:06:39+02	2020-07-28 19:11:51+02	textproc	xerces-c3	\N	xerces-c3-3.2.3
2020-07-28 19:11:14+02	2020-07-28 19:12:33+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-28 19:11:51+02	2020-07-28 19:17:53+02	audio	rosegarden	\N	rosegarden-20.06
2020-07-28 19:12:33+02	2020-07-28 19:18:58+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-28 19:17:54+02	2020-07-28 19:20:25+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-28 19:20:27+02	2020-07-28 19:22:35+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-28 19:22:35+02	2020-07-28 19:24:30+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-28 19:24:30+02	2020-07-28 19:27:04+02	deskutils	calibre	\N	calibre-4.21.0
2020-07-28 18:19:20+02	2020-07-28 19:46:01+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-07-28 19:46:02+02	2020-07-28 19:51:49+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-07-28 19:27:05+02	2020-07-28 20:08:58+02	devel	RStudio	\N	RStudio-1.3.1056
2020-07-28 20:08:59+02	2020-07-28 20:10:30+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-07-28 20:10:31+02	2020-07-28 20:38:47+02	devel	electron7	\N	electron7-7.3.1_1
2020-07-28 20:38:50+02	2020-07-28 20:41:06+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-07-28 20:41:07+02	2020-07-28 20:49:29+02	emulators	wine	\N	wine-5.0.1,1
2020-07-28 20:49:30+02	2020-07-28 20:55:10+02	graphics	blender	\N	blender-2.83.2
2020-07-28 20:55:12+02	2020-07-28 21:07:04+02	graphics	krita	\N	krita-4.3.0_2
2020-07-28 21:07:04+02	2020-07-28 21:08:10+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-07-28 21:08:11+02	2020-07-28 21:09:16+02	multimedia	celluloid	\N	celluloid-0.19
2020-07-28 21:09:16+02	2020-07-28 21:11:24+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-07-28 21:11:25+02	2020-07-28 21:14:09+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-07-28 21:14:12+02	2020-07-28 21:17:11+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-07-28 21:17:11+02	2020-07-28 21:18:08+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-07-28 21:18:09+02	2020-07-28 21:19:05+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-07-28 21:19:05+02	2020-07-28 21:20:00+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-07-28 21:20:03+02	2020-07-28 21:20:58+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-07-28 21:20:59+02	2020-07-28 21:21:53+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-07-28 21:21:54+02	2020-07-28 21:22:50+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-02 01:55:32+02	2020-08-02 01:57:47+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-07-28 21:22:50+02	2020-07-28 21:23:47+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-07-28 19:18:59+02	2020-07-28 21:24:32+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-28 21:23:48+02	2020-07-28 21:25:14+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-07-28 21:24:32+02	2020-07-28 21:25:25+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-07-28 21:25:15+02	2020-07-28 21:26:08+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-07-28 21:25:26+02	2020-07-28 21:28:12+02	print	auctex	full	auctex-emacs26-12.2
2020-07-28 21:28:13+02	2020-07-28 21:29:48+02	x11	mate	\N	mate-1.22.2
2020-07-28 21:26:09+02	2020-07-28 22:16:27+02	www	firefox	\N	firefox-79.0,1
2020-07-28 21:29:49+02	2020-07-28 22:19:30+02	editors	vscode	\N	vscode-1.46.1
2020-07-28 19:51:54+02	2020-07-28 22:35:28+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-07-28 22:53:48+02	2020-07-28 23:08:39+02	local	E590T-localbase	\N	E590T-localbase-2020072800
2020-07-29 07:50:56+02	2020-07-29 07:52:20+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-07-29 07:50:56+02	2020-07-29 07:57:09+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-07-29 07:57:10+02	2020-07-29 07:57:17+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-07-29 08:02:53+02	2020-07-29 08:46:35+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-07-29 08:02:53+02	2020-07-29 10:04:04+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-07-29 10:04:05+02	2020-07-29 10:05:51+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-07-29 10:05:52+02	2020-07-29 10:07:57+02	devel	libgdata	\N	libgdata-0.17.9
2020-07-29 10:07:58+02	2020-07-29 10:09:30+02	x11	yelp	\N	yelp-3.28.1_2
2020-07-29 10:04:05+02	2020-07-29 10:10:21+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-07-29 10:09:31+02	2020-07-29 10:11:20+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-07-29 10:10:22+02	2020-07-29 10:14:00+02	math	gnuplot	\N	gnuplot-5.4.0
2020-07-29 10:11:21+02	2020-07-29 10:18:01+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-07-29 10:14:04+02	2020-07-29 10:24:17+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-07-29 10:24:18+02	2020-07-29 10:32:35+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-07-29 10:32:36+02	2020-07-29 10:37:43+02	editors	pluma	\N	pluma-1.24.0
2020-07-29 10:18:02+02	2020-07-29 10:38:59+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-07-29 10:37:44+02	2020-07-29 10:40:07+02	graphics	atril	\N	atril-1.22.3_7
2020-07-29 10:40:08+02	2020-07-29 10:41:12+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-07-29 10:41:13+02	2020-07-29 10:42:35+02	x11	mate-base	\N	mate-base-1.24.0
2020-07-29 10:39:00+02	2020-07-29 10:47:00+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-07-29 10:42:36+02	2020-07-29 10:48:18+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-07-29 10:47:01+02	2020-07-29 10:49:49+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-07-29 10:49:50+02	2020-07-29 10:52:22+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-07-29 10:52:23+02	2020-07-29 10:54:20+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-07-29 10:54:21+02	2020-07-29 10:59:05+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-07-29 10:59:06+02	2020-07-29 11:09:16+02	editors	codelite	\N	codelite-12.0_4
2020-07-29 11:09:17+02	2020-07-29 11:23:26+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-07-29 11:23:27+02	2020-07-29 11:26:40+02	graphics	wings	\N	wings-2.2.5
2020-07-29 11:26:41+02	2020-07-29 11:32:13+02	math	maxima	\N	maxima-5.44.0_2
2020-07-29 11:32:14+02	2020-07-29 11:33:51+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-07-29 11:33:52+02	2020-07-29 11:35:30+02	x11	mate	\N	mate-1.22.2
2020-07-29 10:48:19+02	2020-07-29 12:16:14+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-07-29 08:02:53+02	2020-07-29 12:45:56+02	devel	llvm10	\N	llvm10-10.0.1
2020-07-29 12:45:57+02	2020-07-29 13:01:52+02	local	E590T-localbase	\N	E590T-localbase-2020072800
2020-08-01 17:56:09+02	2020-08-01 18:06:43+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-01 18:06:44+02	2020-08-01 18:06:56+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-01 17:56:09+02	2020-08-01 18:09:52+02	devel	binutils	native	binutils-2.33.1_3,1
2020-08-01 18:09:53+02	2020-08-01 19:37:15+02	lang	gcc9	\N	gcc9-9.3.0_1
2020-08-01 19:37:16+02	2020-08-01 19:39:11+02	devel	kBuild	\N	kBuild-0.1.9998_13
2020-08-01 19:39:11+02	2020-08-01 19:39:59+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-01 19:46:17+02	2020-08-01 19:46:37+02	shells	bash-completion	\N	bash-completion-2.11,2
2020-08-01 19:46:17+02	2020-08-01 19:47:09+02	devel	check	\N	check-0.15.1_1
2020-08-01 19:47:10+02	2020-08-01 19:49:14+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-08-01 19:46:17+02	2020-08-01 19:50:35+02	multimedia	libvpx	\N	libvpx-1.9.0
2020-08-01 19:50:37+02	2020-08-01 20:04:27+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1,1
2020-08-01 20:04:28+02	2020-08-01 20:05:45+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-01 20:05:45+02	2020-08-01 20:06:35+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-01 19:46:38+02	2020-08-01 20:14:16+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-08-01 20:06:36+02	2020-08-01 20:15:33+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-01 20:14:16+02	2020-08-01 20:24:29+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-01 20:15:34+02	2020-08-01 20:44:53+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-01 20:44:55+02	2020-08-01 20:46:37+02	sysutils	bsdisks	\N	bsdisks-0.18
2020-08-01 20:24:30+02	2020-08-01 20:51:08+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-01 20:51:09+02	2020-08-01 20:52:41+02	devel	kf5-solid	\N	kf5-solid-5.72.0
2020-08-01 20:52:42+02	2020-08-01 20:53:00+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0,1
2020-08-01 20:53:01+02	2020-08-01 20:54:12+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-08-01 20:54:13+02	2020-08-01 20:59:37+02	devel	kf5-kio	\N	kf5-kio-5.72.0
2020-08-01 20:59:38+02	2020-08-01 21:06:08+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-01 21:06:08+02	2020-08-01 21:50:26+02	lang	gcc6-aux	\N	gcc6-aux-20180516_1,1
2020-08-01 21:50:28+02	2020-08-01 21:55:57+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-08-01 20:46:38+02	2020-08-01 22:57:52+02	www	node	\N	node-14.6.0_1
2020-08-01 21:55:58+02	2020-08-01 23:46:43+02	www	node12	\N	node12-12.18.3
2020-08-01 23:46:43+02	2020-08-01 23:48:24+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-01 23:48:25+02	2020-08-01 23:50:09+02	devel	binutils	aarch64	aarch64-binutils-2.33.1_3,1
2020-08-01 23:50:09+02	2020-08-01 23:52:08+02	devel	binutils	mips64	mips64-binutils-2.33.1_3,1
2020-08-01 23:52:08+02	2020-08-01 23:53:55+02	devel	binutils	powerpc64	powerpc64-binutils-2.33.1_3,1
2020-08-01 23:53:55+02	2020-08-01 23:55:34+02	devel	binutils	riscv64	riscv64-binutils-2.33.1_3,1
2020-08-01 23:55:35+02	2020-08-01 23:57:18+02	devel	binutils	sparc64	sparc64-binutils-2.33.1_3,1
2020-08-01 23:57:18+02	2020-08-01 23:57:53+02	devel	libfmt	\N	libfmt-7.0.2
2020-08-01 23:57:55+02	2020-08-02 00:13:45+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-02 00:13:46+02	2020-08-02 00:16:01+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-02 00:16:02+02	2020-08-02 00:17:40+02	multimedia	mlt	\N	mlt-6.20.0
2020-08-02 00:17:42+02	2020-08-02 01:34:59+02	www	node10	\N	node10-10.22.0
2020-08-02 01:35:01+02	2020-08-02 01:36:22+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-02 01:36:22+02	2020-08-02 01:37:10+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-02 01:37:11+02	2020-08-02 01:37:57+02	audio	faudio	\N	FAudio-20.07
2020-08-02 01:37:58+02	2020-08-02 01:43:59+02	devel	aarch64-gcc	\N	aarch64-gcc-6.4.0_8
2020-08-02 01:44:01+02	2020-08-02 01:45:48+02	devel	gradle5	\N	gradle5-5.0
2020-08-02 01:45:48+02	2020-08-02 01:50:20+02	devel	mips64-gcc	\N	mips64-gcc-6.4.0_8
2020-08-30 15:27:26+02	2020-08-30 15:29:00+02	graphics	eom	\N	eom-1.24.0
2020-08-02 01:50:21+02	2020-08-02 01:55:31+02	devel	powerpc64-gcc	\N	powerpc64-gcc-6.4.0_8
2020-08-02 01:57:48+02	2020-08-02 02:02:34+02	devel	riscv64-gcc	\N	riscv64-gcc-8.3.0
2020-08-02 02:02:35+02	2020-08-02 02:07:17+02	devel	sparc64-gcc	\N	sparc64-gcc-6.4.0_8
2020-08-02 02:07:18+02	2020-08-02 02:36:28+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-02 02:36:29+02	2020-08-02 03:09:02+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-08-02 03:09:04+02	2020-08-02 03:11:35+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-01 19:49:14+02	2020-08-02 03:27:16+02	lang	rust	\N	rust-1.45.1
2020-08-02 03:11:36+02	2020-08-02 03:34:28+02	graphics	openimageio	\N	openimageio-2.1.15.0_4
2020-08-02 03:27:19+02	2020-08-02 03:36:53+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3_2
2020-08-02 03:36:54+02	2020-08-02 03:38:28+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-02 03:38:29+02	2020-08-02 03:40:38+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-02 03:40:39+02	2020-08-02 03:43:51+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-02 03:43:52+02	2020-08-02 03:52:56+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-02 03:52:56+02	2020-08-02 03:54:24+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-02 03:54:24+02	2020-08-02 03:56:51+02	textproc	fop	\N	fop-2.3
2020-08-02 03:56:52+02	2020-08-02 04:05:23+02	textproc	ripgrep	\N	ripgrep-12.1.1_3
2020-08-02 04:05:24+02	2020-08-02 04:05:57+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-08-02 04:05:58+02	2020-08-02 04:07:17+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-02 04:07:17+02	2020-08-02 04:08:39+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-02 04:08:40+02	2020-08-02 04:08:53+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-08-02 04:08:53+02	2020-08-02 04:10:24+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-02 04:10:25+02	2020-08-02 04:11:49+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-02 03:34:29+02	2020-08-02 04:24:27+02	lang	gcc48	\N	gcc48-4.8.5_11
2020-08-02 04:24:28+02	2020-08-02 04:30:52+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-02 04:30:53+02	2020-08-02 04:33:18+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-02 04:11:50+02	2020-08-02 04:34:55+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-02 04:34:56+02	2020-08-02 04:37:36+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-02 04:37:37+02	2020-08-02 04:39:29+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-02 04:39:30+02	2020-08-02 04:41:48+02	deskutils	calibre	\N	calibre-4.21.0
2020-08-02 04:41:49+02	2020-08-02 05:07:53+02	devel	RStudio	\N	RStudio-1.3.1056
2020-08-02 05:07:54+02	2020-08-02 05:08:06+02	devel	aarch64-xtoolchain-gcc	\N	aarch64-xtoolchain-gcc-0.4_1
2020-08-02 05:08:06+02	2020-08-02 05:08:46+02	devel	cdecl	\N	cdecl-8.0
2020-08-02 05:08:47+02	2020-08-02 05:35:16+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-02 05:35:19+02	2020-08-02 05:35:30+02	devel	mips64-xtoolchain-gcc	\N	mips64-xtoolchain-gcc-0.4_1
2020-08-02 05:35:31+02	2020-08-02 05:35:42+02	devel	powerpc64-xtoolchain-gcc	\N	powerpc64-xtoolchain-gcc-0.4_1
2020-08-02 05:35:44+02	2020-08-02 05:35:54+02	devel	riscv64-xtoolchain-gcc	\N	riscv64-xtoolchain-gcc-0.4_1
2020-08-02 05:35:54+02	2020-08-02 05:36:04+02	devel	sparc64-xtoolchain-gcc	\N	sparc64-xtoolchain-gcc-0.4_1
2020-08-02 05:36:04+02	2020-08-02 05:38:18+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-08-02 05:38:19+02	2020-08-02 06:17:48+02	emulators	wine	\N	wine-5.0.1,1
2020-08-02 04:33:20+02	2020-08-02 06:25:22+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-02 06:17:48+02	2020-08-02 06:28:41+02	graphics	blender	\N	blender-2.83.2
2020-08-02 06:25:23+02	2020-08-02 06:36:37+02	graphics	krita	\N	krita-4.3.0_2
2020-08-02 06:36:39+02	2020-08-02 06:37:50+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-02 06:37:51+02	2020-08-02 06:40:13+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-02 06:40:14+02	2020-08-02 06:43:07+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-02 06:43:08+02	2020-08-02 06:46:37+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-02 06:46:38+02	2020-08-02 06:47:43+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-02 06:47:44+02	2020-08-02 06:51:52+02	sysutils	exa	\N	exa-0.9.0_13
2020-08-02 06:51:53+02	2020-08-02 06:54:13+02	sysutils	hexyl	\N	hexyl-0.8.0_2
2020-08-02 06:54:13+02	2020-08-02 07:05:28+02	sysutils	rsfetch	\N	rsfetch-2.0.0_5
2020-08-02 07:05:28+02	2020-08-02 07:20:00+02	textproc	bat	\N	bat-0.15.4_2
2020-08-02 07:20:02+02	2020-08-02 07:20:30+02	www	dokuwiki	\N	dokuwiki-20200729
2020-08-02 07:20:31+02	2020-08-02 08:19:24+02	www	firefox	\N	firefox-79.0_1,1
2020-08-02 08:19:26+02	2020-08-02 08:20:00+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-08-02 08:20:02+02	2020-08-02 08:20:16+02	www	yarn-node10	\N	yarn-node10-1.22.4_1
2020-08-02 08:20:17+02	2020-08-02 08:47:39+02	x11	alacritty	\N	alacritty-0.4.3_3
2020-08-02 08:47:40+02	2020-08-02 08:49:41+02	x11	mate	\N	mate-1.22.2
2020-08-02 08:49:42+02	2020-08-02 08:50:17+02	x11	xlockmore	\N	xlockmore-5.65
2020-08-02 08:50:18+02	2020-08-02 08:52:03+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-02 06:28:42+02	2020-08-02 09:26:39+02	lang	gcc10-devel	\N	gcc10-devel-10.2.1.s20200725
2020-08-02 08:52:04+02	2020-08-02 09:34:33+02	editors	vscode	\N	vscode-1.46.1
2020-08-02 11:33:12+02	2020-08-02 11:35:25+02	devel	binutils	native	binutils-2.33.1_3,1
2020-08-02 11:35:26+02	2020-08-02 20:37:31+02	www	chromium	\N	chromium-84.0.4147.105
2020-08-02 20:42:57+02	2020-08-02 20:44:03+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-02 20:44:03+02	2020-08-02 20:45:21+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-02 20:45:22+02	2020-08-02 20:46:38+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-02 20:42:57+02	2020-08-02 22:02:56+02	lang	gcc9	\N	gcc9-9.3.0_1
2020-08-02 22:02:57+02	2020-08-02 22:05:44+02	devel	kBuild	\N	kBuild-0.1.9998_13
2020-08-02 22:02:57+02	2020-08-02 22:09:12+02	math	openblas	\N	openblas-0.3.9_2,1
2020-08-02 22:02:57+02	2020-08-02 22:10:22+02	net	openmpi	\N	openmpi-4.0.4
2020-08-02 22:09:13+02	2020-08-02 22:12:03+02	math	suitesparse	\N	suitesparse-5.8.1
2020-08-02 22:12:04+02	2020-08-02 22:13:07+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-08-02 22:10:23+02	2020-08-02 22:15:19+02	math	vtk8	\N	vtk8-8.2.0
2020-08-02 22:13:07+02	2020-08-02 22:16:42+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-08-02 22:05:44+02	2020-08-02 22:18:31+02	math	R	\N	R-4.0.2
2020-08-02 22:18:32+02	2020-08-02 22:19:20+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-02 22:15:19+02	2020-08-02 22:19:57+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-02 22:19:20+02	2020-08-02 22:20:01+02	devel	py-game	py37	py37-game-1.9.6_1
2020-08-02 22:16:43+02	2020-08-02 22:20:25+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-08-02 22:20:26+02	2020-08-02 22:21:07+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-08-02 22:21:07+02	2020-08-02 22:21:35+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-08-02 22:20:02+02	2020-08-02 22:21:52+02	french	med	\N	fr-med-4.0.0
2020-08-02 22:21:52+02	2020-08-02 22:23:13+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-02 22:23:14+02	2020-08-02 22:29:26+02	misc	openvdb	\N	openvdb-7.0.0_4
2020-08-02 22:19:58+02	2020-08-02 22:34:46+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-02 22:21:36+02	2020-08-02 22:42:24+02	lang	julia11	\N	julia11-1.1.1_5
2020-08-02 22:29:27+02	2020-08-02 22:45:00+02	print	lilypond	\N	lilypond-2.20.0
2020-08-02 22:42:25+02	2020-08-02 22:48:22+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-02 22:48:23+02	2020-08-02 22:51:19+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-02 22:46:07+02	2020-08-02 22:52:49+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-02 22:52:50+02	2020-08-02 22:54:24+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-02 22:34:47+02	2020-08-02 22:54:42+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-13 02:34:40+02	2020-09-13 02:38:17+02	devel	git	\N	git-2.28.0
2020-08-02 22:54:25+02	2020-08-02 22:55:48+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-02 22:54:59+02	2020-08-02 23:21:30+02	devel	RStudio	\N	RStudio-1.3.1056
2020-08-02 23:21:31+02	2020-08-02 23:22:16+02	games	monsterz	\N	monsterz-0.7.1_15
2020-08-02 23:22:16+02	2020-08-02 23:27:48+02	graphics	blender	\N	blender-2.83.2
2020-08-02 22:55:48+02	2020-08-02 23:31:48+02	emulators	wine	\N	wine-5.0.1,1
2020-08-02 23:31:49+02	2020-08-02 23:32:06+02	lang	gcc	\N	gcc-9_4
2020-08-02 23:32:07+02	2020-08-02 23:32:46+02	lang	julia	\N	julia-1.0.3_3
2020-08-02 23:32:47+02	2020-08-02 23:33:46+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-02 23:27:49+02	2020-08-02 23:38:21+02	graphics	inkscape	\N	inkscape-1.0_3
2020-08-02 23:33:47+02	2020-08-03 00:10:41+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-08-02 22:51:20+02	2020-08-03 00:12:41+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-03 00:12:42+02	2020-08-03 00:27:12+02	local	E590T-localbase	\N	E590T-localbase-2020072800
2020-08-03 00:27:26+02	2020-08-03 00:28:17+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-03 00:27:26+02	2020-08-03 00:28:53+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-03 00:28:54+02	2020-08-03 00:29:01+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-03 07:04:20+02	2020-08-03 07:04:52+02	multimedia	libmatroska	\N	libmatroska-1.6.1
2020-08-03 07:04:52+02	2020-08-03 07:05:37+02	net	liveMedia	\N	liveMedia-2020.07.31,2
2020-08-03 07:04:20+02	2020-08-03 07:06:11+02	x11	libX11	\N	libX11-1.6.10,1
2020-08-03 07:05:37+02	2020-08-03 07:06:13+02	net	py-pyzmq	py37	py37-pyzmq-19.0.2
2020-08-03 07:06:11+02	2020-08-03 07:06:25+02	x11	libXext	\N	libXext-1.3.4,1
2020-08-03 07:06:14+02	2020-08-03 07:06:27+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-08-03 07:06:25+02	2020-08-03 07:06:37+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-08-03 07:06:28+02	2020-08-03 07:06:41+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-08-03 07:06:37+02	2020-08-03 07:06:49+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-08-03 07:06:41+02	2020-08-03 07:06:52+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-08-03 07:06:50+02	2020-08-03 07:07:11+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-08-03 07:07:12+02	2020-08-03 07:07:55+02	devel	dbus	\N	dbus-1.12.20
2020-08-03 07:04:20+02	2020-08-03 07:07:56+02	databases	sqlite3	\N	sqlite3-3.32.3_1,1
2020-08-03 07:07:56+02	2020-08-03 07:08:21+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-08-03 07:07:56+02	2020-08-03 07:08:24+02	x11	libXi	\N	libXi-1.7.10,1
2020-08-03 07:08:24+02	2020-08-03 07:08:42+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-08-03 07:08:22+02	2020-08-03 07:08:47+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-08-03 07:08:42+02	2020-08-03 07:08:55+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-08-03 07:08:56+02	2020-08-03 07:09:13+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-08-03 07:06:52+02	2020-08-03 07:09:16+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-08-03 07:09:14+02	2020-08-03 07:09:31+02	x11	libXpm	\N	libXpm-3.5.13
2020-08-03 07:08:48+02	2020-08-03 07:09:35+02	net	avahi-app	\N	avahi-app-0.7_3
2020-08-03 07:09:31+02	2020-08-03 07:09:42+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-08-03 07:09:42+02	2020-08-03 07:10:21+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-08-03 07:09:36+02	2020-08-03 07:10:35+02	print	cups	\N	cups-2.3.3_1
2020-08-03 07:09:17+02	2020-08-03 07:10:39+02	graphics	cairo	\N	cairo-1.16.0,2
2020-08-03 07:10:22+02	2020-08-03 07:10:44+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-08-03 07:10:45+02	2020-08-03 07:11:33+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-08-03 07:11:33+02	2020-08-03 07:11:49+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-08-03 07:11:49+02	2020-08-03 07:12:03+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-08-03 07:12:04+02	2020-08-03 07:12:15+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-08-03 07:10:35+02	2020-08-03 07:12:27+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_6
2020-08-03 07:10:40+02	2020-08-03 07:12:34+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-03 07:12:16+02	2020-08-03 07:12:35+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-08-03 07:12:28+02	2020-08-03 07:12:39+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-08-03 07:12:36+02	2020-08-03 07:13:17+02	sysutils	polkit	\N	polkit-0.116
2020-08-03 07:12:39+02	2020-08-03 07:13:18+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-08-03 07:13:19+02	2020-08-03 07:13:28+02	x11	xprop	\N	xprop-1.2.4
2020-08-03 07:13:17+02	2020-08-03 07:13:33+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-08-03 07:13:29+02	2020-08-03 07:13:40+02	x11	xset	\N	xset-1.2.4_3
2020-08-03 07:13:40+02	2020-08-03 07:13:58+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-08-03 07:13:59+02	2020-08-03 07:14:34+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-08-03 07:13:34+02	2020-08-03 07:14:59+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-08-03 07:12:34+02	2020-08-03 07:15:21+02	graphics	graphviz	\N	graphviz-2.44.1
2020-08-03 07:14:35+02	2020-08-03 07:15:55+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-03 07:15:01+02	2020-08-03 07:16:22+02	graphics	colord	\N	colord-1.3.5
2020-08-03 07:16:22+02	2020-08-03 07:16:53+02	graphics	libGLU	\N	libGLU-9.0.1
2020-08-03 07:15:22+02	2020-08-03 07:17:18+02	lang	vala	\N	vala-0.40.15,1
2020-08-03 07:15:55+02	2020-08-03 07:17:23+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-08-03 07:17:18+02	2020-08-03 07:18:14+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-03 07:16:53+02	2020-08-03 07:18:21+02	security	nss	\N	nss-3.55
2020-08-03 07:18:21+02	2020-08-03 07:24:16+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-03 07:17:23+02	2020-08-03 07:25:30+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-03 07:24:17+02	2020-08-03 07:25:32+02	graphics	poppler	\N	poppler-0.90.1
2020-08-03 07:25:33+02	2020-08-03 07:25:50+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-08-03 07:18:15+02	2020-08-03 07:25:53+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-03 07:25:51+02	2020-08-03 07:26:30+02	devel	dconf	\N	dconf-0.36.0
2020-08-03 07:25:54+02	2020-08-03 07:26:37+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-03 07:26:38+02	2020-08-03 07:27:05+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-08-03 07:27:05+02	2020-08-03 07:27:25+02	print	xpdfopen	\N	xpdfopen-0.86
2020-08-03 07:26:30+02	2020-08-03 07:27:32+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-08-03 07:27:25+02	2020-08-03 07:27:42+02	multimedia	libva	\N	libva-2.8.0
2020-08-03 07:27:32+02	2020-08-03 07:28:33+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-08-03 07:28:33+02	2020-08-03 07:29:01+02	devel	libnotify	\N	libnotify-0.7.8
2020-08-03 07:29:02+02	2020-08-03 07:29:14+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-08-03 07:25:31+02	2020-08-03 07:29:20+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-03 07:29:15+02	2020-08-03 07:29:51+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-08-03 07:29:52+02	2020-08-03 07:31:06+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-03 07:31:06+02	2020-08-03 07:31:22+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-08-03 07:31:23+02	2020-08-03 07:32:07+02	graphics	frei0r	\N	frei0r-1.6.1
2020-08-03 07:27:43+02	2020-08-03 07:32:23+02	print	texlive-base	\N	texlive-base-20150521_53
2020-08-03 07:29:21+02	2020-08-03 07:40:30+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-03 07:32:07+02	2020-08-03 07:41:00+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1,1
2020-08-03 07:41:01+02	2020-08-03 07:41:16+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-08-03 07:40:30+02	2020-08-03 07:41:24+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-03 07:41:24+02	2020-08-03 07:41:45+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-08-03 07:41:16+02	2020-08-03 07:42:25+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-03 07:42:25+02	2020-08-03 07:42:39+02	www	libwpe	\N	libwpe-1.4.0.1
2020-08-03 07:41:45+02	2020-08-03 07:42:52+02	graphics	graphene	\N	graphene-1.10.0
2020-08-03 07:42:40+02	2020-08-03 07:43:33+02	net	geoclue	\N	geoclue-2.5.5
2020-08-03 07:43:34+02	2020-08-03 07:43:49+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-08-03 07:42:52+02	2020-08-03 07:43:59+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-08-03 07:43:50+02	2020-08-03 07:44:25+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-03 07:44:25+02	2020-08-03 07:44:53+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-03 07:44:53+02	2020-08-03 07:45:15+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-08-03 07:45:16+02	2020-08-03 07:45:50+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-08-03 07:45:51+02	2020-08-03 07:46:23+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-08-03 07:46:24+02	2020-08-03 07:47:40+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-03 07:47:40+02	2020-08-03 07:47:54+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-08-03 07:47:55+02	2020-08-03 07:48:33+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-08-03 07:48:33+02	2020-08-03 07:49:15+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-08-03 07:49:16+02	2020-08-03 07:49:29+02	x11	libXres	\N	libXres-1.2.0_2
2020-08-03 07:49:30+02	2020-08-03 07:49:46+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-08-03 07:32:23+02	2020-08-03 07:50:19+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-03 07:50:19+02	2020-08-03 07:53:39+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-03 07:49:46+02	2020-08-03 07:54:06+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-03 07:54:08+02	2020-08-03 07:55:16+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-03 07:55:16+02	2020-08-03 08:00:55+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-08-03 07:53:40+02	2020-08-03 08:01:04+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-03 08:00:56+02	2020-08-03 08:01:12+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-08-03 08:01:13+02	2020-08-03 08:01:28+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-08-03 08:01:28+02	2020-08-03 08:02:09+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-03 08:02:10+02	2020-08-03 08:02:35+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.20
2020-08-03 08:02:36+02	2020-08-03 08:03:01+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.20
2020-08-03 08:03:01+02	2020-08-03 08:04:02+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-03 08:04:02+02	2020-08-03 08:04:55+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-03 08:04:55+02	2020-08-03 08:05:42+02	sysutils	policykit	\N	policykit-0.9_10
2020-08-03 08:05:43+02	2020-08-03 08:06:56+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-03 08:06:56+02	2020-08-03 08:07:50+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-03 08:01:05+02	2020-08-03 08:08:16+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-03 08:07:51+02	2020-08-03 08:09:07+02	devel	kf5-kconfig	\N	kf5-kconfig-5.72.0
2020-08-03 08:08:17+02	2020-08-03 08:09:30+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-08-03 08:09:07+02	2020-08-03 08:10:29+02	devel	kf5-ki18n	\N	kf5-ki18n-5.72.0
2020-08-03 08:10:30+02	2020-08-03 08:11:10+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-08-03 08:09:30+02	2020-08-03 08:11:57+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-03 08:11:11+02	2020-08-03 08:12:06+02	sysutils	hal	\N	hal-0.5.14_34
2020-08-03 08:11:58+02	2020-08-03 08:12:37+02	www	py-django22	py37	py37-django22-2.2.14
2020-08-03 08:12:07+02	2020-08-03 08:13:07+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-03 08:13:08+02	2020-08-03 08:14:55+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-03 08:12:38+02	2020-08-03 08:15:29+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_3,1
2020-08-03 08:14:55+02	2020-08-03 08:16:11+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.72.0
2020-08-03 08:15:30+02	2020-08-03 08:19:05+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-03 08:19:06+02	2020-08-03 08:20:57+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-08-03 08:20:58+02	2020-08-03 08:22:20+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.72.0
2020-08-03 08:16:12+02	2020-08-03 08:23:09+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-03 08:22:20+02	2020-08-03 08:23:13+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-03 08:23:14+02	2020-08-03 08:23:25+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-08-03 08:23:09+02	2020-08-03 08:24:08+02	graphics	poppler-glib	\N	poppler-glib-0.90.1
2020-08-03 07:43:59+02	2020-08-03 08:24:48+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-03 08:23:26+02	2020-08-03 08:25:13+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.72.0
2020-08-03 08:24:09+02	2020-08-03 08:25:42+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-08-03 08:25:13+02	2020-08-03 08:25:58+02	devel	kf5-kcrash	\N	kf5-kcrash-5.72.0
2020-08-03 08:25:43+02	2020-08-03 08:26:31+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.72.0
2020-08-03 08:26:31+02	2020-08-03 08:27:35+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-03 08:25:59+02	2020-08-03 08:29:08+02	graphics	glew	\N	glew-2.2.0
2020-08-03 08:29:08+02	2020-08-03 08:29:44+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-08-03 08:29:45+02	2020-08-03 08:31:09+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-03 08:31:10+02	2020-08-03 08:32:21+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-03 08:32:22+02	2020-08-03 08:34:27+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-03 08:34:27+02	2020-08-03 08:35:58+02	devel	kf5-kauth	\N	kf5-kauth-5.72.0
2020-08-03 08:35:59+02	2020-08-03 08:37:43+02	devel	kf5-kservice	\N	kf5-kservice-5.72.0
2020-08-03 08:24:49+02	2020-08-03 08:39:31+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-03 08:37:44+02	2020-08-03 08:39:47+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-03 08:39:48+02	2020-08-03 08:40:08+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-08-03 08:39:32+02	2020-08-03 08:40:39+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-08-03 08:40:09+02	2020-08-03 08:41:20+02	net	remmina	\N	remmina-1.3.9
2020-08-03 08:41:21+02	2020-08-03 08:41:35+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-08-03 08:41:35+02	2020-08-03 08:41:52+02	x11	xrdb	\N	xrdb-1.2.0
2020-08-03 08:40:40+02	2020-08-03 08:42:37+02	security	gcr	\N	gcr-3.28.0
2020-08-03 08:41:52+02	2020-08-03 08:42:56+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.72.0
2020-08-03 08:42:56+02	2020-08-03 08:43:49+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-03 08:27:36+02	2020-08-03 08:47:23+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-08-03 08:43:49+02	2020-08-03 08:47:55+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-03 08:42:37+02	2020-08-03 08:47:59+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-03 08:47:24+02	2020-08-03 08:48:12+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-03 08:48:00+02	2020-08-03 08:48:45+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.72.0
2020-08-03 08:47:55+02	2020-08-03 08:48:51+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.72.0
2020-08-03 08:48:12+02	2020-08-03 08:49:05+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-03 08:48:52+02	2020-08-03 08:49:17+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-03 08:49:05+02	2020-08-03 08:51:10+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-03 08:51:11+02	2020-08-03 08:51:43+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-03 08:51:43+02	2020-08-03 08:52:59+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-03 08:52:59+02	2020-08-03 08:53:15+02	print	gl2ps	\N	gl2ps-1.4.0
2020-08-03 08:49:17+02	2020-08-03 08:54:07+02	math	vtk8	\N	vtk8-8.2.0
2020-08-03 08:53:15+02	2020-08-03 08:54:15+02	security	gnupg	\N	gnupg-2.2.21
2020-08-03 08:54:08+02	2020-08-03 08:54:19+02	x11	luit	\N	luit-20190106_1
2020-08-03 08:54:16+02	2020-08-03 08:54:26+02	x11	xauth	\N	xauth-1.1
2020-08-03 08:54:19+02	2020-08-03 08:55:17+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.72.0
2020-08-03 08:55:18+02	2020-08-03 08:55:32+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.8_6
2020-08-03 08:48:45+02	2020-08-03 08:56:02+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-03 08:54:26+02	2020-08-03 08:56:03+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-03 08:56:03+02	2020-08-03 08:58:22+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-08-03 08:58:23+02	2020-08-03 08:58:48+02	security	gpgme	\N	gpgme-1.14.0
2020-08-03 08:58:49+02	2020-08-03 08:59:13+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-08-03 08:56:03+02	2020-08-03 08:59:36+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-03 08:59:14+02	2020-08-03 09:00:07+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.72.0
2020-08-03 08:55:33+02	2020-08-03 09:00:38+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-03 09:00:39+02	2020-08-03 09:00:50+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-08-03 09:00:51+02	2020-08-03 09:01:24+02	x11	xterm	\N	xterm-358
2020-08-03 09:01:24+02	2020-08-03 09:02:36+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-08-03 08:59:36+02	2020-08-03 09:02:36+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-03 09:02:36+02	2020-08-03 09:03:37+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.72.0
2020-08-03 09:02:37+02	2020-08-03 09:03:44+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-03 09:03:44+02	2020-08-03 09:04:06+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-08-03 09:04:06+02	2020-08-03 09:04:21+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-08-03 09:04:22+02	2020-08-03 09:05:18+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-03 09:05:18+02	2020-08-03 09:06:08+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-03 09:06:09+02	2020-08-03 09:07:08+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-03 09:03:38+02	2020-08-03 09:07:21+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-03 09:07:24+02	2020-08-03 09:09:02+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-03 09:07:09+02	2020-08-03 09:09:24+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-08-03 09:09:03+02	2020-08-03 09:09:36+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-08-03 09:09:27+02	2020-08-03 09:10:11+02	sysutils	upower	\N	upower-0.99.11
2020-08-03 09:10:11+02	2020-08-03 09:10:23+02	x11	appres	\N	appres-1.0.5
2020-08-03 09:10:23+02	2020-08-03 09:10:37+02	x11	bitmap	\N	bitmap-1.0.9
2020-08-03 09:10:37+02	2020-08-03 09:11:51+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.72.0
2020-08-03 09:11:52+02	2020-08-03 09:12:40+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-08-03 09:12:41+02	2020-08-03 09:16:41+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-03 09:16:42+02	2020-08-03 09:16:58+02	x11	sessreg	\N	sessreg-1.1.2
2020-08-03 09:16:59+02	2020-08-03 09:17:15+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-08-03 09:17:18+02	2020-08-03 09:17:33+02	x11	smproxy	\N	smproxy-1.0.6
2020-08-03 09:17:34+02	2020-08-03 09:17:53+02	x11	x11perf	\N	x11perf-1.6.1
2020-08-03 09:17:54+02	2020-08-03 09:18:11+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-08-03 09:18:12+02	2020-08-03 09:18:40+02	x11	xcalc	\N	xcalc-1.1.0
2020-08-03 09:18:40+02	2020-08-03 09:18:57+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-08-03 09:18:58+02	2020-08-03 09:19:15+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-08-03 09:19:16+02	2020-08-03 09:19:31+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-08-03 09:19:32+02	2020-08-03 09:19:52+02	x11	xev	\N	xev-1.2.4
2020-08-03 09:19:53+02	2020-08-03 09:20:12+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-08-03 09:20:13+02	2020-08-03 09:20:28+02	x11	xgamma	\N	xgamma-1.0.6
2020-08-03 09:20:28+02	2020-08-03 09:20:48+02	x11	xgc	\N	xgc-1.0.5
2020-08-03 09:20:48+02	2020-08-03 09:21:07+02	x11	xhost	\N	xhost-1.0.8
2020-08-03 09:21:08+02	2020-08-03 09:21:22+02	x11	xinit	\N	xinit-1.4.1,1
2020-08-03 09:21:23+02	2020-08-03 09:21:43+02	x11	xinput	\N	xinput-1.6.3
2020-08-03 09:21:43+02	2020-08-03 09:21:57+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-08-03 09:21:58+02	2020-08-03 09:22:17+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-08-03 09:22:18+02	2020-08-03 09:22:34+02	x11	xkill	\N	xkill-1.0.5
2020-08-03 09:22:34+02	2020-08-03 09:22:49+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-08-03 09:22:50+02	2020-08-03 09:23:08+02	x11	xmessage	\N	xmessage-1.0.5
2020-08-03 09:23:09+02	2020-08-03 09:23:26+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-08-03 09:23:26+02	2020-08-03 09:23:40+02	x11	xpr	\N	xpr-1.0.5
2020-08-03 09:23:41+02	2020-08-03 09:23:54+02	x11	xrandr	\N	xrandr-1.5.1
2020-08-03 09:23:54+02	2020-08-03 09:24:07+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-08-03 09:00:07+02	2020-08-03 09:24:15+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-03 09:24:08+02	2020-08-03 09:24:27+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-08-03 09:24:16+02	2020-08-03 09:24:31+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-08-03 09:24:27+02	2020-08-03 09:24:39+02	x11	xwd	\N	xwd-1.0.7
2020-08-03 09:24:32+02	2020-08-03 09:24:43+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-08-03 09:24:39+02	2020-08-03 09:24:51+02	x11	xwud	\N	xwud-1.0.5
2020-08-03 09:24:44+02	2020-08-03 09:24:57+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-08-03 09:24:51+02	2020-08-03 09:25:56+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-03 09:24:58+02	2020-08-03 09:25:59+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-03 09:25:57+02	2020-08-03 09:26:59+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-03 09:25:59+02	2020-08-03 09:27:01+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-03 09:26:59+02	2020-08-03 09:28:00+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-03 09:27:02+02	2020-08-03 09:28:35+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-08-03 09:28:01+02	2020-08-03 09:28:46+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-03 09:28:36+02	2020-08-03 09:29:26+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-03 09:28:47+02	2020-08-03 09:29:57+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.72.0
2020-08-03 09:29:57+02	2020-08-03 09:30:25+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-03 09:29:26+02	2020-08-03 09:30:26+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.72.0
2020-08-03 09:30:26+02	2020-08-03 09:30:39+02	x11-wm	twm	\N	twm-1.0.11
2020-08-03 09:09:36+02	2020-08-03 09:31:08+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-03 09:30:40+02	2020-08-03 09:31:10+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-08-03 09:31:10+02	2020-08-03 09:33:32+02	devel	librest	\N	rest-0.7.93
2020-08-03 09:31:09+02	2020-08-03 09:34:23+02	devel	kf5-knotifications	\N	kf5-knotifications-5.72.0
2020-08-03 09:33:37+02	2020-08-03 09:36:39+02	graphics	poppler-utils	\N	poppler-utils-0.90.1
2020-08-03 09:36:42+02	2020-08-03 09:38:36+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-08-03 09:38:39+02	2020-08-03 09:40:34+02	print	libotf	\N	libotf-0.9.16
2020-08-03 09:40:37+02	2020-08-03 09:41:57+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-08-03 09:30:25+02	2020-08-03 09:42:10+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-03 09:41:58+02	2020-08-03 09:42:43+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-03 09:42:43+02	2020-08-03 09:43:34+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-03 09:43:35+02	2020-08-03 09:44:06+02	sysutils	bsdisks	\N	bsdisks-0.18
2020-08-03 09:44:06+02	2020-08-03 09:45:42+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-08-03 09:45:42+02	2020-08-03 09:46:07+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-08-03 09:46:07+02	2020-08-03 09:47:26+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-03 09:34:25+02	2020-08-03 09:47:45+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-03 09:47:27+02	2020-08-03 09:47:51+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-08-03 09:47:46+02	2020-08-03 09:47:58+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-08-03 09:47:51+02	2020-08-03 09:48:48+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-03 09:47:58+02	2020-08-03 09:50:18+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-03 09:48:49+02	2020-08-03 09:51:07+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_3,1
2020-08-03 09:51:08+02	2020-08-03 09:52:10+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-03 09:50:19+02	2020-08-03 09:52:12+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.72.0
2020-08-03 09:52:11+02	2020-08-03 09:53:28+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-03 09:52:13+02	2020-08-03 09:53:28+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-03 09:53:29+02	2020-08-03 09:54:34+02	devel	kf5-solid	\N	kf5-solid-5.72.0
2020-08-03 09:53:28+02	2020-08-03 09:54:52+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.72.0
2020-08-03 09:54:34+02	2020-08-03 09:55:46+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-08-03 09:55:46+02	2020-08-03 09:56:25+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-08-03 09:56:26+02	2020-08-03 09:56:46+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-08-03 09:54:53+02	2020-08-03 09:56:47+02	graphics	gegl	\N	gegl-0.4.24_2
2020-08-03 09:42:11+02	2020-08-03 09:57:14+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-03 09:57:15+02	2020-08-03 09:57:26+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-08-03 09:56:47+02	2020-08-03 09:58:18+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-03 09:57:27+02	2020-08-03 09:58:31+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-03 09:58:19+02	2020-08-03 09:59:15+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-03 09:59:15+02	2020-08-03 10:00:06+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-08-03 09:58:32+02	2020-08-03 10:00:24+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-03 10:00:24+02	2020-08-03 10:01:11+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-03 10:00:06+02	2020-08-03 10:01:29+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.72.0
2020-08-03 10:01:11+02	2020-08-03 10:02:10+02	x11	kf5-kded	\N	kf5-kded-5.72.0_1
2020-08-03 09:56:47+02	2020-08-03 10:02:32+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-03 10:02:11+02	2020-08-03 10:03:01+02	x11	zenity	\N	zenity-3.32.0
2020-08-03 10:03:01+02	2020-08-03 10:03:49+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.72.0
2020-08-03 10:01:30+02	2020-08-03 10:04:03+02	x11	xorg	\N	xorg-7.7_3
2020-08-03 10:03:50+02	2020-08-03 10:04:29+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-08-03 10:02:32+02	2020-08-03 10:05:46+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-03 10:04:29+02	2020-08-03 10:06:39+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-03 10:06:40+02	2020-08-03 10:07:15+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-08-03 10:05:47+02	2020-08-03 10:07:17+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-03 10:04:04+02	2020-08-03 10:07:20+02	devel	kf5-kio	\N	kf5-kio-5.72.0
2020-08-03 10:07:16+02	2020-08-03 10:08:11+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-03 10:07:21+02	2020-08-03 10:08:25+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-03 10:07:17+02	2020-08-03 10:08:26+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-03 10:08:11+02	2020-08-03 10:10:49+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-03 10:10:50+02	2020-08-03 10:11:08+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-08-03 10:08:26+02	2020-08-03 10:12:50+02	editors	emacs	full	emacs-26.3_5,3
2020-08-03 10:12:51+02	2020-08-03 10:13:14+02	graphics	glfw	\N	glfw-3.3.2
2020-08-03 10:08:26+02	2020-08-03 10:13:25+02	graphics	Coin	\N	Coin-4.0.0_1
2020-08-03 10:13:14+02	2020-08-03 10:14:24+02	graphics	movit	\N	movit-1.6.2_2
2020-08-03 10:14:24+02	2020-08-03 10:15:00+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-08-03 10:15:00+02	2020-08-03 10:15:22+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-08-03 10:15:23+02	2020-08-03 10:15:48+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-08-03 10:15:49+02	2020-08-03 10:17:16+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-03 10:13:25+02	2020-08-03 10:17:35+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-03 10:17:36+02	2020-08-03 10:17:58+02	net	gssdp	\N	gssdp-1.2.3
2020-08-03 10:17:58+02	2020-08-03 10:18:43+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-08-03 10:18:43+02	2020-08-03 10:19:24+02	print	cups-filters	\N	cups-filters-1.27.5_1
2020-08-03 10:11:08+02	2020-08-03 10:19:42+02	graphics	gimp-app	\N	gimp-app-2.10.20_2,1
2020-08-03 10:17:17+02	2020-08-03 10:20:41+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-08-03 10:20:42+02	2020-08-03 10:21:36+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-03 10:19:24+02	2020-08-03 10:22:55+02	print	fontforge	\N	fontforge-20190801_2
2020-08-03 10:21:37+02	2020-08-03 10:23:08+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-08-03 10:23:09+02	2020-08-03 10:23:59+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-03 10:23:59+02	2020-08-03 10:25:01+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-03 10:25:02+02	2020-08-03 10:25:52+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-03 10:25:52+02	2020-08-03 10:26:35+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-03 10:19:43+02	2020-08-03 10:28:10+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-08-03 10:26:36+02	2020-08-03 10:28:11+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-03 10:28:12+02	2020-08-03 10:31:34+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-03 10:31:35+02	2020-08-03 10:34:27+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-03 10:28:11+02	2020-08-03 10:34:47+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-08-03 10:34:28+02	2020-08-03 10:35:27+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-03 10:34:47+02	2020-08-03 10:37:12+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-03 10:35:27+02	2020-08-03 10:41:11+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-03 10:41:12+02	2020-08-03 10:42:48+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-03 10:42:49+02	2020-08-03 10:43:30+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-08-03 10:37:13+02	2020-08-03 10:43:32+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-03 10:43:32+02	2020-08-03 10:44:30+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-03 10:44:31+02	2020-08-03 10:45:10+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-08-03 10:43:30+02	2020-08-03 10:45:30+02	x11-wm	marco	\N	marco-1.22.4
2020-08-03 10:45:30+02	2020-08-03 10:46:36+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-08-03 10:45:11+02	2020-08-03 10:47:06+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-03 10:47:07+02	2020-08-03 10:49:31+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-03 10:46:36+02	2020-08-03 10:50:32+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-03 10:50:33+02	2020-08-03 10:56:10+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-03 10:56:11+02	2020-08-03 10:57:04+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-03 10:57:06+02	2020-08-03 10:57:24+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-08-03 10:57:25+02	2020-08-03 10:58:48+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-03 10:49:32+02	2020-08-03 10:59:16+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-03 10:58:49+02	2020-08-03 10:59:31+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-08-03 10:22:56+02	2020-08-03 10:59:36+02	www	chromium	\N	chromium-84.0.4147.105
2020-08-03 10:59:17+02	2020-08-03 10:59:40+02	graphics	libgxps	\N	libgxps-0.3.1
2020-08-03 10:59:32+02	2020-08-03 10:59:51+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-08-03 10:59:40+02	2020-08-03 11:00:06+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-03 10:59:52+02	2020-08-03 11:00:37+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-03 11:00:06+02	2020-08-03 11:00:49+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-03 10:59:38+02	2020-08-03 11:01:42+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.1
2020-08-03 11:01:42+02	2020-08-03 11:02:26+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-08-03 11:02:26+02	2020-08-03 11:04:48+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-03 11:04:48+02	2020-08-03 11:06:08+02	net	freerdp	\N	freerdp-2.2.0
2020-08-03 11:06:09+02	2020-08-03 11:06:29+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-03 11:00:50+02	2020-08-03 11:07:09+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-03 11:06:30+02	2020-08-03 11:07:09+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-03 11:07:10+02	2020-08-03 11:07:59+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-03 11:07:10+02	2020-08-03 11:08:21+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-03 11:08:22+02	2020-08-03 11:08:44+02	print	libspectre	\N	libspectre-0.2.9
2020-08-03 11:08:45+02	2020-08-03 11:10:38+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-03 11:07:59+02	2020-08-03 11:10:56+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-03 11:00:38+02	2020-08-03 11:12:57+02	math	R	\N	R-4.0.2
2020-08-03 11:12:57+02	2020-08-03 11:13:24+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-08-03 11:13:24+02	2020-08-03 11:13:39+02	sysutils	hal-info	\N	hal-info-20091130
2020-08-03 11:10:39+02	2020-08-03 11:15:33+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-03 11:10:57+02	2020-08-03 11:15:40+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-03 11:13:40+02	2020-08-03 11:16:01+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-03 11:15:34+02	2020-08-03 11:16:18+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-03 11:16:18+02	2020-08-03 11:16:56+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-03 11:16:02+02	2020-08-03 11:17:31+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-03 11:15:41+02	2020-08-03 11:17:38+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-03 11:16:56+02	2020-08-03 11:17:47+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-03 11:17:47+02	2020-08-03 11:17:55+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-08-03 11:17:32+02	2020-08-03 11:17:58+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-03 11:17:55+02	2020-08-03 11:18:03+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-08-03 11:17:38+02	2020-08-03 11:18:37+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-03 11:18:04+02	2020-08-03 11:18:56+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-03 11:18:37+02	2020-08-03 11:18:59+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-08-03 11:17:59+02	2020-08-03 11:19:17+02	x11-servers	xephyr	\N	xephyr-1.20.8_3,1
2020-08-03 11:18:57+02	2020-08-03 11:19:39+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-03 11:19:00+02	2020-08-03 11:19:52+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-03 11:19:18+02	2020-08-03 11:20:19+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-03 11:19:39+02	2020-08-03 11:20:52+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-03 11:20:52+02	2020-08-03 11:21:08+02	audio	dssi	\N	dssi-1.1.1_8
2020-08-03 11:21:09+02	2020-08-03 11:22:02+02	audio	faudio	\N	FAudio-20.08
2020-08-03 11:20:19+02	2020-08-03 11:22:23+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-03 11:22:24+02	2020-08-03 11:22:41+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-08-03 11:22:03+02	2020-08-03 11:23:09+02	audio	mate-media	\N	mate-media-1.24.0
2020-08-03 11:22:41+02	2020-08-03 11:23:46+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-03 11:23:10+02	2020-08-03 11:24:25+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-03 11:24:25+02	2020-08-03 11:25:21+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-03 11:25:22+02	2020-08-03 11:26:25+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-03 11:26:26+02	2020-08-03 11:26:51+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-08-03 11:26:51+02	2020-08-03 11:27:09+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-08-03 11:23:47+02	2020-08-03 11:27:19+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-03 11:27:20+02	2020-08-03 11:28:18+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.72.0
2020-08-03 11:27:09+02	2020-08-03 11:28:33+02	devel	gradle5	\N	gradle5-5.0
2020-08-03 11:28:34+02	2020-08-03 11:29:06+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-08-03 11:28:18+02	2020-08-03 11:29:20+02	devel	ode	\N	ode-0.13_4,1
2020-08-03 11:29:07+02	2020-08-03 11:29:59+02	devel	py-game	py37	py37-game-1.9.6_1
2020-08-03 11:19:53+02	2020-08-03 11:30:31+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-03 11:29:21+02	2020-08-03 11:30:46+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-03 11:30:47+02	2020-08-03 11:33:45+02	editors	pluma	\N	pluma-1.24.0
2020-08-03 11:33:45+02	2020-08-03 11:34:22+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-08-03 11:34:23+02	2020-08-03 11:36:11+02	french	med	\N	fr-med-4.0.0
2020-08-03 11:36:12+02	2020-08-03 11:38:21+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-08-03 11:38:24+02	2020-08-03 11:41:20+02	graphics	atril	\N	atril-1.22.3_7
2020-08-03 11:41:20+02	2020-08-03 11:43:30+02	graphics	eom	\N	eom-1.24.0
2020-08-03 11:43:31+02	2020-08-03 11:44:54+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-03 11:44:55+02	2020-08-03 11:45:09+02	graphics	gle	\N	gle-3.1.0_7
2020-08-03 11:45:10+02	2020-08-03 11:45:22+02	graphics	gltt	\N	gltt-2.5.2_14
2020-08-03 11:45:23+02	2020-08-03 11:46:16+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-03 11:46:16+02	2020-08-03 11:48:40+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-08-03 11:30:00+02	2020-08-03 11:49:42+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-03 11:48:41+02	2020-08-03 11:49:55+02	graphics	py-gimp	\N	py27-gimp-2.10.20_2
2020-08-03 11:49:55+02	2020-08-03 11:50:20+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-08-03 11:30:32+02	2020-08-03 11:50:50+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-08-03 11:49:43+02	2020-08-03 11:59:59+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-03 11:50:20+02	2020-08-03 12:11:04+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-03 12:11:04+02	2020-08-03 12:12:32+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-03 11:50:51+02	2020-08-03 12:13:23+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-08-03 12:12:32+02	2020-08-03 12:13:30+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-03 12:00:04+02	2020-08-03 12:13:39+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-08-03 12:13:30+02	2020-08-03 12:13:42+02	misc	xosd	\N	xosd-2.2.14_3
2020-08-03 12:13:43+02	2020-08-03 12:14:17+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-08-03 12:13:40+02	2020-08-03 12:14:52+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-03 12:14:17+02	2020-08-03 12:15:54+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-03 12:15:54+02	2020-08-03 12:16:52+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-03 12:16:52+02	2020-08-03 12:17:33+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-03 12:13:24+02	2020-08-03 12:17:39+02	misc	openvdb	\N	openvdb-7.0.0_4
2020-08-03 12:17:34+02	2020-08-03 12:18:26+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-03 12:17:40+02	2020-08-03 12:18:28+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-03 12:18:29+02	2020-08-03 12:19:07+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-03 12:18:26+02	2020-08-03 12:19:10+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-03 12:14:53+02	2020-08-03 12:19:17+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-03 12:19:08+02	2020-08-03 12:19:44+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-03 12:19:10+02	2020-08-03 12:19:57+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-03 12:19:17+02	2020-08-03 12:20:18+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-03 12:19:45+02	2020-08-03 12:20:25+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-03 12:19:58+02	2020-08-03 12:20:57+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-03 12:20:18+02	2020-08-03 12:21:15+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-03 12:20:26+02	2020-08-03 12:24:59+02	print	lilypond	\N	lilypond-2.20.0
2020-08-03 12:21:16+02	2020-08-03 12:26:03+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-03 12:20:58+02	2020-08-03 12:26:14+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-03 12:25:00+02	2020-08-03 12:26:32+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-03 12:26:33+02	2020-08-03 12:26:56+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-03 12:26:57+02	2020-08-03 12:27:38+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-03 12:27:38+02	2020-08-03 12:28:58+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-03 12:28:59+02	2020-08-03 12:30:33+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-03 12:30:33+02	2020-08-03 12:31:53+02	textproc	fop	\N	fop-2.3
2020-08-03 12:26:04+02	2020-08-03 12:32:18+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-08-03 12:31:54+02	2020-08-03 12:32:41+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-03 12:32:41+02	2020-08-03 12:32:52+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-08-03 12:32:19+02	2020-08-03 12:32:56+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-03 12:32:53+02	2020-08-03 12:33:05+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-08-03 12:32:57+02	2020-08-03 12:33:11+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-08-03 12:33:05+02	2020-08-03 12:33:53+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-03 12:33:11+02	2020-08-03 12:34:05+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-03 12:34:06+02	2020-08-03 12:35:05+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-03 12:35:05+02	2020-08-03 12:36:12+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-03 12:36:12+02	2020-08-03 12:37:45+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-03 12:33:54+02	2020-08-03 12:38:04+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-03 12:38:04+02	2020-08-03 12:38:51+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-08-03 12:37:46+02	2020-08-03 12:39:18+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-03 12:38:52+02	2020-08-03 12:39:43+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-03 12:39:18+02	2020-08-03 12:39:50+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-03 12:39:43+02	2020-08-03 12:42:37+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-03 12:42:37+02	2020-08-03 12:42:47+02	astro	xearth	\N	xearth-1.2
2020-08-03 12:42:47+02	2020-08-03 12:43:52+02	astro	xephem	\N	xephem-3.7.7_5
2020-08-03 12:26:14+02	2020-08-03 12:44:34+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-03 12:44:35+02	2020-08-03 12:44:51+02	audio	xmix	\N	xmix-2.1_3
2020-08-03 12:43:53+02	2020-08-03 12:48:45+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-03 12:39:50+02	2020-08-03 12:53:40+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-03 12:48:46+02	2020-08-03 12:54:22+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-03 12:53:41+02	2020-08-03 12:56:13+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-03 12:56:14+02	2020-08-03 12:58:21+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-03 12:58:21+02	2020-08-03 13:00:04+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-03 13:00:05+02	2020-08-03 13:00:15+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-08-03 12:44:52+02	2020-08-03 13:03:15+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-03 13:03:16+02	2020-08-03 13:07:48+02	deskutils	calibre	\N	calibre-4.22.0
2020-08-03 13:07:49+02	2020-08-03 13:08:50+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-03 13:08:51+02	2020-08-03 13:09:04+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-08-03 13:00:15+02	2020-08-03 13:09:14+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-03 13:09:04+02	2020-08-03 13:09:18+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-08-03 13:09:19+02	2020-08-03 13:10:11+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-08-03 13:09:15+02	2020-08-03 13:35:10+02	devel	RStudio	\N	RStudio-1.3.1056
2020-08-03 13:10:12+02	2020-08-03 13:38:48+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-03 13:38:53+02	2020-08-03 13:39:14+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-08-03 13:39:15+02	2020-08-03 13:41:21+02	devel	subversion	\N	subversion-1.14.0
2020-08-03 13:41:21+02	2020-08-03 13:55:32+02	editors	codelite	\N	codelite-12.0_4
2020-08-03 13:55:33+02	2020-08-03 13:57:28+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-08-03 13:57:28+02	2020-08-03 14:00:14+02	editors	vim	\N	vim-8.2.1334
2020-08-03 14:00:15+02	2020-08-03 14:19:22+02	emulators	qemu	\N	qemu-4.2.1
2020-08-03 13:35:11+02	2020-08-03 14:20:36+02	devel	ghidra	\N	ghidra-9.1
2020-08-03 12:54:24+02	2020-08-03 14:29:12+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-03 14:29:13+02	2020-08-03 14:29:49+02	emulators	x48	\N	x48-0.6.4_1
2020-08-03 14:20:37+02	2020-08-03 14:30:13+02	emulators	wine	\N	wine-5.0.1,1
2020-08-03 14:30:14+02	2020-08-03 14:30:29+02	games	KnightCap	\N	knightcap-3.6_8
2020-08-03 14:30:30+02	2020-08-03 14:30:38+02	games	affenspiel	\N	affenspiel-1.0_2
2020-08-03 14:30:38+02	2020-08-03 14:30:57+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-08-03 14:30:58+02	2020-08-03 14:33:25+02	games	alienarena	\N	alienarena-2013.766_9
2020-08-03 14:33:26+02	2020-08-03 14:34:20+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-08-03 14:34:20+02	2020-08-03 14:38:08+02	games	battletanks	\N	btanks-0.9.8083_9
2020-08-03 14:38:08+02	2020-08-03 14:38:28+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-08-03 14:38:29+02	2020-08-03 14:39:22+02	games	monsterz	\N	monsterz-0.7.1_15
2020-08-03 14:39:23+02	2020-08-03 14:40:03+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-03 14:40:03+02	2020-08-03 14:40:52+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-03 14:40:53+02	2020-08-03 14:41:56+02	games	xblast	\N	xblast-2.10.4_17
2020-08-03 14:41:57+02	2020-08-03 14:42:56+02	games	xboard	\N	xboard-4.9.1_1
2020-08-03 14:42:57+02	2020-08-03 14:43:44+02	games	xcowsay	\N	xcowsay-1.5
2020-08-03 14:29:49+02	2020-08-03 14:43:54+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-03 14:43:45+02	2020-08-03 14:44:07+02	games	xevil	\N	xevil-2.02.r2_2
2020-08-03 14:43:55+02	2020-08-03 14:44:14+02	games	xminesweep	\N	xminesweep-3.0_2
2020-08-03 14:44:15+02	2020-08-03 14:47:22+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-03 14:47:23+02	2020-08-03 14:48:51+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-03 14:44:08+02	2020-08-03 14:54:44+02	graphics	blender	\N	blender-2.83.2
2020-08-03 14:48:52+02	2020-08-03 14:59:06+02	graphics	inkscape	\N	inkscape-1.0_3
2020-08-03 14:59:07+02	2020-08-03 15:05:06+02	graphics	wings	\N	wings-2.2.5
2020-08-03 15:05:09+02	2020-08-03 15:07:04+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-03 14:54:47+02	2020-08-03 15:17:20+02	graphics	krita	\N	krita-4.3.0_2
2020-08-03 15:17:22+02	2020-08-03 15:20:26+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-08-03 14:19:24+02	2020-08-03 15:28:11+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44
2020-08-03 15:07:04+02	2020-08-03 15:32:11+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-08-03 15:20:28+02	2020-08-03 15:49:15+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.3
2020-08-03 15:49:16+02	2020-08-03 15:49:29+02	mail	bbmail	\N	bbmail-0.9.3
2020-08-03 15:49:29+02	2020-08-03 15:54:56+02	math	maxima	\N	maxima-5.44.0_2
2020-08-03 15:54:57+02	2020-08-03 15:56:05+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-03 15:56:06+02	2020-08-03 15:56:24+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-08-03 15:56:25+02	2020-08-03 15:56:57+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-08-03 15:28:14+02	2020-08-03 15:58:32+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-03 15:58:32+02	2020-08-03 15:58:58+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-08-03 15:58:59+02	2020-08-03 15:59:14+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-08-03 15:59:14+02	2020-08-03 16:03:04+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-03 15:32:12+02	2020-08-03 16:05:04+02	lang	mono	\N	mono-5.10.1.57_3
2020-08-03 16:03:09+02	2020-08-03 16:05:17+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-03 15:56:57+02	2020-08-03 16:06:02+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p3
2020-08-03 16:06:03+02	2020-08-03 16:06:47+02	net	rdesktop	\N	rdesktop-1.8.6
2020-08-03 16:05:17+02	2020-08-03 16:08:06+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-08-03 16:08:06+02	2020-08-03 16:08:21+02	net	vnstat	\N	vnstat-2.6_1
2020-08-03 16:06:48+02	2020-08-03 16:08:21+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-03 16:08:22+02	2020-08-03 16:09:15+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-03 16:09:15+02	2020-08-03 16:10:15+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-03 16:10:16+02	2020-08-03 16:11:22+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-03 16:11:23+02	2020-08-03 16:12:20+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-03 16:05:05+02	2020-08-03 16:12:55+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-03 16:12:20+02	2020-08-03 16:13:25+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-03 16:12:56+02	2020-08-03 16:13:58+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-03 16:13:25+02	2020-08-03 16:14:15+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-03 16:13:58+02	2020-08-03 16:14:46+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-03 16:14:16+02	2020-08-03 16:15:36+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-03 16:14:46+02	2020-08-03 16:15:44+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-03 16:15:37+02	2020-08-03 16:16:20+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-03 16:15:44+02	2020-08-03 16:16:43+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-03 16:16:20+02	2020-08-03 16:17:10+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-03 16:08:21+02	2020-08-03 16:17:15+02	net	wireshark	\N	wireshark-3.2.5
2020-08-03 16:17:11+02	2020-08-03 16:17:36+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-03 16:17:16+02	2020-08-03 16:17:47+02	print	gv	\N	gv-3.7.4_5
2020-08-03 16:16:43+02	2020-08-03 16:18:39+02	print	auctex	full	auctex-emacs26-12.2
2020-08-03 16:18:40+02	2020-08-03 16:24:15+02	print	texlive-full	\N	texlive-full-20150521
2020-08-03 16:17:36+02	2020-08-03 16:25:05+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-03 16:25:06+02	2020-08-03 16:25:18+02	security	nyx	\N	nyx-2.1.0_1
2020-08-03 16:25:19+02	2020-08-03 16:26:29+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-03 16:24:16+02	2020-08-03 16:26:45+02	security	klee	\N	klee-2.1_1
2020-08-03 16:26:30+02	2020-08-03 16:27:28+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-03 16:26:46+02	2020-08-03 16:27:32+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-03 16:27:28+02	2020-08-03 16:28:34+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-03 16:28:34+02	2020-08-03 16:29:20+02	www	links	\N	links-2.20.2,1
2020-08-03 16:29:20+02	2020-08-03 16:29:36+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-08-03 16:29:36+02	2020-08-03 16:29:56+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-08-03 16:29:56+02	2020-08-03 16:30:12+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-08-03 16:30:12+02	2020-08-03 16:30:26+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-08-03 16:30:27+02	2020-08-03 16:30:42+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-08-03 16:30:42+02	2020-08-03 16:30:58+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-08-03 16:30:58+02	2020-08-03 16:31:15+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-08-03 16:31:15+02	2020-08-03 16:31:29+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-08-03 16:31:30+02	2020-08-03 16:31:43+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-08-03 16:31:43+02	2020-08-03 16:32:07+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-08-03 16:17:47+02	2020-08-03 16:34:59+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-08-03 16:35:02+02	2020-08-03 16:36:33+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-03 16:36:34+02	2020-08-03 16:38:33+02	x11	mate	\N	mate-1.22.2
2020-08-03 16:38:34+02	2020-08-03 16:38:49+02	x11	numlockx	\N	numlockx-1.2
2020-08-03 16:38:50+02	2020-08-03 16:39:00+02	x11	sterm	\N	sterm-0.8.4
2020-08-03 16:39:01+02	2020-08-03 16:39:29+02	x11	xclip	\N	xclip-0.13
2020-08-03 16:39:30+02	2020-08-03 16:39:45+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-08-03 16:39:46+02	2020-08-03 16:40:19+02	x11	xdm	\N	xdm-1.1.12_3
2020-08-03 16:40:19+02	2020-08-03 16:41:25+02	x11	xlockmore	\N	xlockmore-5.65
2020-08-03 16:41:26+02	2020-08-03 16:41:43+02	x11	xman	\N	xman-1.1.5
2020-08-03 16:41:43+02	2020-08-03 16:45:28+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-03 16:45:28+02	2020-08-03 16:51:25+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-03 16:51:26+02	2020-08-03 16:52:41+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-03 16:52:42+02	2020-08-03 16:53:04+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-08-03 16:53:04+02	2020-08-03 16:53:15+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-08-03 16:53:16+02	2020-08-03 16:53:49+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-08-03 16:53:50+02	2020-08-03 16:54:11+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-08-03 16:54:11+02	2020-08-03 16:54:37+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-08-03 16:54:37+02	2020-08-03 16:54:52+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-08-03 16:32:08+02	2020-08-03 16:54:58+02	x11	alacritty	\N	alacritty-0.5.0
2020-08-03 16:54:59+02	2020-08-03 16:55:54+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-03 16:55:54+02	2020-08-03 16:56:29+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-03 16:54:53+02	2020-08-03 17:30:45+02	editors	vscode	\N	vscode-1.46.1
2020-08-03 16:27:32+02	2020-08-03 17:33:28+02	www	firefox	\N	firefox-79.0_1,1
2020-08-03 17:33:29+02	2020-08-03 17:47:50+02	local	E590T-localbase	\N	E590T-localbase-2020072800
2020-08-03 17:48:04+02	2020-08-03 17:49:05+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-03 17:48:04+02	2020-08-03 17:52:49+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-03 17:52:50+02	2020-08-03 17:52:57+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-03 19:02:29+02	2020-08-03 19:09:23+02	lang	sbcl	\N	sbcl-2.0.7,1
2020-08-03 19:09:24+02	2020-08-03 19:15:18+02	math	maxima	\N	maxima-5.44.0_3
2020-08-03 19:15:19+02	2020-08-03 19:15:26+02	sysutils	neofetch	\N	neofetch-7.1.0
2020-08-03 19:02:29+02	2020-08-03 19:31:29+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.5
2020-08-03 19:02:29+02	2020-08-03 22:18:13+02	lang	rust	\N	rust-1.45.2
2020-08-03 22:19:52+02	2020-08-03 22:23:51+02	sysutils	exa	\N	exa-0.9.0_14
2020-08-03 22:23:52+02	2020-08-03 22:26:05+02	sysutils	hexyl	\N	hexyl-0.8.0_3
2020-08-03 22:19:52+02	2020-08-03 22:26:30+02	textproc	ripgrep	\N	ripgrep-12.1.1_4
2020-08-03 22:19:52+02	2020-08-03 22:26:34+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3_3
2020-08-03 22:26:06+02	2020-08-03 22:34:59+02	sysutils	rsfetch	\N	rsfetch-2.0.0_6
2020-08-03 22:26:30+02	2020-08-03 22:38:51+02	textproc	bat	\N	bat-0.15.4_3
2020-08-03 22:34:59+02	2020-08-03 22:56:34+02	x11	alacritty	\N	alacritty-0.5.0_1
2020-08-03 22:26:35+02	2020-08-03 23:08:23+02	www	firefox	\N	firefox-79.0_2,1
2020-08-03 23:08:24+02	2020-08-03 23:22:54+02	local	E590T-localbase	\N	E590T-localbase-2020072800
2020-08-03 23:23:07+02	2020-08-03 23:23:59+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-03 23:23:07+02	2020-08-03 23:24:36+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-03 23:24:36+02	2020-08-03 23:24:43+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-04 07:27:44+02	2020-08-04 07:31:50+02	multimedia	x265	\N	x265-3.2.1_4
2020-08-04 07:27:44+02	2020-08-04 07:32:01+02	databases	sqlite3	\N	sqlite3-3.32.3_2,1
2020-08-04 07:31:51+02	2020-08-04 07:32:48+02	graphics	libheif	\N	libheif-1.6.2
2020-08-04 07:32:02+02	2020-08-04 07:33:35+02	graphics	colord	\N	colord-1.3.5
2020-08-04 07:32:48+02	2020-08-04 07:34:14+02	security	nss	\N	nss-3.55
2020-08-04 07:34:14+02	2020-08-04 07:35:54+02	graphics	poppler	\N	poppler-0.90.1
2020-08-04 07:35:55+02	2020-08-04 07:37:08+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-04 07:33:36+02	2020-08-04 07:42:01+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-04 07:42:01+02	2020-08-04 07:42:52+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-08-04 07:37:09+02	2020-08-04 07:42:59+02	print	texlive-base	\N	texlive-base-20150521_53
2020-08-04 07:31:50+02	2020-08-04 07:43:16+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-04 07:42:52+02	2020-08-04 07:43:49+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-04 07:43:50+02	2020-08-04 07:44:05+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-08-04 07:43:17+02	2020-08-04 07:44:12+02	net	geoclue	\N	geoclue-2.5.5
2020-08-04 07:44:05+02	2020-08-04 07:44:23+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.20
2020-08-04 07:44:23+02	2020-08-04 07:44:41+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.20
2020-08-04 07:44:41+02	2020-08-04 07:45:47+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-04 07:45:47+02	2020-08-04 07:46:39+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-04 07:46:40+02	2020-08-04 07:47:45+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-08-04 07:47:45+02	2020-08-04 07:48:24+02	www	py-django22	py37	py37-django22-2.2.14
2020-08-04 07:48:24+02	2020-08-04 07:49:50+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-04 07:49:50+02	2020-08-04 07:51:16+02	graphics	poppler-glib	\N	poppler-glib-0.90.1
2020-08-04 07:51:17+02	2020-08-04 07:52:20+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-04 07:52:20+02	2020-08-04 07:53:48+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-04 07:53:48+02	2020-08-04 07:54:51+02	net	remmina	\N	remmina-1.3.9
2020-08-04 07:54:52+02	2020-08-04 07:57:17+02	security	gcr	\N	gcr-3.28.0
2020-08-04 07:42:59+02	2020-08-04 08:01:33+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-04 07:57:18+02	2020-08-04 08:03:42+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-04 08:01:34+02	2020-08-04 08:04:37+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-04 08:04:38+02	2020-08-04 08:05:22+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-04 08:05:23+02	2020-08-04 08:06:34+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-04 08:06:34+02	2020-08-04 08:07:09+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-04 08:07:10+02	2020-08-04 08:08:39+02	security	gnupg	\N	gnupg-2.2.21
2020-08-04 08:03:43+02	2020-08-04 08:09:58+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-04 08:09:58+02	2020-08-04 08:22:11+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-04 08:22:13+02	2020-08-04 08:26:50+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-04 08:26:51+02	2020-08-04 08:27:19+02	security	gpgme	\N	gpgme-1.14.0
2020-08-04 07:44:13+02	2020-08-04 08:51:43+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-04 08:08:40+02	2020-08-04 08:53:36+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-04 08:53:38+02	2020-08-04 08:54:55+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-08-04 08:54:56+02	2020-08-04 08:55:16+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-08-04 08:55:17+02	2020-08-04 08:56:15+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-04 08:56:16+02	2020-08-04 08:56:50+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-08-04 08:51:44+02	2020-08-04 09:10:39+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-04 09:10:40+02	2020-08-04 09:13:26+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-04 09:13:27+02	2020-08-04 09:17:06+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-04 09:17:07+02	2020-08-04 09:24:31+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-04 08:27:19+02	2020-08-04 09:33:10+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-04 09:33:11+02	2020-08-04 09:34:40+02	devel	kf5-knotifications	\N	kf5-knotifications-5.72.0
2020-08-04 09:34:41+02	2020-08-04 09:35:26+02	devel	librest	\N	rest-0.7.93
2020-08-04 09:35:27+02	2020-08-04 09:36:05+02	graphics	poppler-utils	\N	poppler-utils-0.90.1
2020-08-04 09:36:06+02	2020-08-04 09:36:57+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-08-04 09:36:58+02	2020-08-04 09:37:52+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-04 09:37:52+02	2020-08-04 09:39:16+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-04 09:39:16+02	2020-08-04 09:41:47+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-04 09:41:48+02	2020-08-04 09:42:41+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-04 09:24:32+02	2020-08-04 09:43:15+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-04 09:42:41+02	2020-08-04 09:43:48+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-04 09:43:15+02	2020-08-04 09:44:46+02	graphics	gegl	\N	gegl-0.4.24_2
2020-08-04 09:43:48+02	2020-08-04 09:45:23+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-04 09:44:47+02	2020-08-04 09:45:43+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-04 09:45:23+02	2020-08-04 09:46:53+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-04 09:45:43+02	2020-08-04 09:47:13+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.72.0
2020-08-04 09:46:54+02	2020-08-04 09:48:00+02	x11	zenity	\N	zenity-3.32.0
2020-08-04 09:47:14+02	2020-08-04 09:49:18+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-04 09:49:19+02	2020-08-04 09:51:44+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-04 09:48:01+02	2020-08-04 09:51:55+02	devel	kf5-kio	\N	kf5-kio-5.72.0
2020-08-04 09:51:45+02	2020-08-04 09:52:49+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-04 09:51:56+02	2020-08-04 09:56:30+02	editors	emacs	full	emacs-26.3_6,3
2020-08-04 09:52:50+02	2020-08-04 10:00:00+02	graphics	gimp-app	\N	gimp-app-2.10.20_2,1
2020-08-04 10:00:01+02	2020-08-04 10:00:23+02	net	gssdp	\N	gssdp-1.2.3
2020-08-04 09:56:32+02	2020-08-04 10:00:54+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-04 10:00:24+02	2020-08-04 10:01:15+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-08-04 10:00:55+02	2020-08-04 10:01:39+02	print	cups-filters	\N	cups-filters-1.27.5_1
2020-08-04 10:01:16+02	2020-08-04 10:04:28+02	print	fontforge	\N	fontforge-20190801_2
2020-08-04 10:04:28+02	2020-08-04 10:06:10+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-08-04 10:01:39+02	2020-08-04 10:09:47+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-08-04 10:09:49+02	2020-08-04 10:10:53+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-04 10:10:54+02	2020-08-04 10:12:29+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-04 10:12:31+02	2020-08-04 10:14:09+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-04 10:14:10+02	2020-08-04 10:15:56+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-04 10:15:57+02	2020-08-04 10:17:22+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-04 10:17:23+02	2020-08-04 10:19:27+02	x11-wm	marco	\N	marco-1.22.4
2020-08-04 10:19:28+02	2020-08-04 10:20:36+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-04 10:20:37+02	2020-08-04 10:23:04+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-04 10:23:05+02	2020-08-04 10:24:14+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-08-04 10:24:14+02	2020-08-04 10:26:53+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-04 08:56:51+02	2020-08-04 10:27:48+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-04 10:26:54+02	2020-08-04 10:28:42+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-04 10:28:42+02	2020-08-04 10:30:44+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-04 10:30:45+02	2020-08-04 10:32:16+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.1
2020-08-04 10:27:49+02	2020-08-04 10:39:45+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-04 10:32:16+02	2020-08-04 10:40:19+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-04 10:40:20+02	2020-08-04 10:40:43+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-04 10:39:46+02	2020-08-04 10:41:01+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-04 10:40:43+02	2020-08-04 10:41:34+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-04 10:41:02+02	2020-08-04 10:41:55+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-04 10:41:56+02	2020-08-04 10:44:24+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-04 10:41:34+02	2020-08-04 10:45:30+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-04 10:44:25+02	2020-08-04 10:51:17+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-04 10:51:18+02	2020-08-04 10:51:45+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-08-04 10:45:30+02	2020-08-04 10:51:51+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-04 10:51:52+02	2020-08-04 10:52:46+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-04 10:51:47+02	2020-08-04 10:54:19+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-04 10:52:47+02	2020-08-04 10:54:34+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-04 10:54:35+02	2020-08-04 10:55:29+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-04 10:54:19+02	2020-08-04 10:55:54+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-04 10:55:30+02	2020-08-04 10:55:59+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-04 10:06:11+02	2020-08-04 10:56:07+02	www	chromium	\N	chromium-84.0.4147.105
2020-08-04 10:55:55+02	2020-08-04 10:56:52+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-04 10:56:09+02	2020-08-04 10:57:14+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-04 10:57:14+02	2020-08-04 10:57:48+02	audio	faudio	\N	FAudio-20.08
2020-08-04 10:56:52+02	2020-08-04 10:58:54+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-04 10:57:48+02	2020-08-04 10:59:01+02	audio	mate-media	\N	mate-media-1.24.0
2020-08-04 10:58:55+02	2020-08-04 10:59:09+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-08-04 10:59:01+02	2020-08-04 11:00:06+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-04 10:56:00+02	2020-08-04 11:00:15+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-04 11:00:07+02	2020-08-04 11:00:58+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-04 10:59:09+02	2020-08-04 11:01:35+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-04 11:00:15+02	2020-08-04 11:01:40+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-04 11:01:41+02	2020-08-04 11:03:36+02	graphics	atril	\N	atril-1.22.3_7
2020-08-04 11:01:36+02	2020-08-04 11:04:31+02	editors	pluma	\N	pluma-1.24.0
2020-08-04 11:03:36+02	2020-08-04 11:05:00+02	graphics	eom	\N	eom-1.24.0
2020-08-04 11:04:32+02	2020-08-04 11:05:39+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-04 11:05:00+02	2020-08-04 11:05:41+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-04 11:05:41+02	2020-08-04 11:07:27+02	graphics	py-gimp	\N	py27-gimp-2.10.20_2
2020-08-04 11:05:39+02	2020-08-04 11:12:25+02	graphics	openimageio	\N	openimageio-2.1.15.0_4
2020-08-04 11:07:27+02	2020-08-04 11:16:50+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-04 11:12:26+02	2020-08-04 11:18:12+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-08-04 11:16:51+02	2020-08-04 11:18:14+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-04 11:18:13+02	2020-08-04 11:19:29+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-04 11:18:14+02	2020-08-04 11:19:45+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-04 11:19:45+02	2020-08-04 11:20:41+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-04 11:00:59+02	2020-08-04 11:21:08+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-08-04 11:20:41+02	2020-08-04 11:21:19+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-04 11:21:09+02	2020-08-04 11:21:50+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-04 11:21:20+02	2020-08-04 11:22:03+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-04 11:21:51+02	2020-08-04 11:22:42+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-04 11:22:04+02	2020-08-04 11:22:46+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-04 11:22:42+02	2020-08-04 11:23:19+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-04 11:22:47+02	2020-08-04 11:23:38+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-04 11:19:30+02	2020-08-04 11:23:41+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-04 11:23:38+02	2020-08-04 11:24:19+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-04 11:23:19+02	2020-08-04 11:24:22+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-04 11:23:42+02	2020-08-04 11:24:41+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-04 11:24:20+02	2020-08-04 11:25:15+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-04 11:24:23+02	2020-08-04 11:28:54+02	print	lilypond	\N	lilypond-2.20.0
2020-08-04 11:24:42+02	2020-08-04 11:29:44+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-04 11:25:16+02	2020-08-04 11:30:02+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-04 11:28:55+02	2020-08-04 11:30:24+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-04 11:30:25+02	2020-08-04 11:31:41+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-04 11:31:42+02	2020-08-04 11:33:05+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-04 11:33:06+02	2020-08-04 11:33:52+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-04 11:33:52+02	2020-08-04 11:34:03+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-08-04 11:34:03+02	2020-08-04 11:34:13+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-08-04 11:34:14+02	2020-08-04 11:34:25+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-08-04 11:29:45+02	2020-08-04 11:35:12+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-08-04 11:34:26+02	2020-08-04 11:35:16+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-04 11:35:13+02	2020-08-04 11:36:08+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-04 11:36:09+02	2020-08-04 11:37:16+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-04 11:37:17+02	2020-08-04 11:38:21+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-04 11:35:17+02	2020-08-04 11:39:23+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-04 11:38:22+02	2020-08-04 11:39:43+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-04 11:39:24+02	2020-08-04 11:40:33+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-04 11:39:44+02	2020-08-04 11:40:33+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-04 11:40:33+02	2020-08-04 11:44:07+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-04 11:30:02+02	2020-08-04 11:48:01+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-04 11:44:07+02	2020-08-04 11:49:20+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-04 11:48:03+02	2020-08-04 11:50:24+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-04 11:50:24+02	2020-08-04 11:52:56+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-04 11:52:57+02	2020-08-04 11:54:27+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-04 11:40:34+02	2020-08-04 11:56:58+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-04 11:54:27+02	2020-08-04 11:57:51+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-04 11:57:51+02	2020-08-04 11:58:33+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-04 11:56:59+02	2020-08-04 11:58:55+02	deskutils	calibre	\N	calibre-4.22.0
2020-08-04 11:58:56+02	2020-08-04 11:59:48+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-08-04 11:58:33+02	2020-08-04 12:24:48+02	devel	RStudio	\N	RStudio-1.3.1056
2020-08-04 12:24:49+02	2020-08-04 12:25:07+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-08-04 12:25:07+02	2020-08-04 12:26:40+02	devel	subversion	\N	subversion-1.14.0
2020-08-04 12:26:40+02	2020-08-04 12:37:35+02	editors	codelite	\N	codelite-12.0_4
2020-08-04 12:37:36+02	2020-08-04 12:39:21+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-08-04 11:59:48+02	2020-08-04 12:40:05+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-04 12:39:22+02	2020-08-04 12:40:59+02	editors	vim	\N	vim-8.2.1334
2020-08-04 12:41:00+02	2020-08-04 12:48:18+02	emulators	wine	\N	wine-5.0.1,1
2020-08-04 12:40:08+02	2020-08-04 12:51:11+02	emulators	qemu	\N	qemu-4.2.1
2020-08-04 12:51:11+02	2020-08-04 12:51:59+02	games	xcowsay	\N	xcowsay-1.5
2020-08-04 12:52:00+02	2020-08-04 13:00:07+02	graphics	blender	\N	blender-2.83.2
2020-08-04 12:48:19+02	2020-08-04 13:00:15+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-04 13:00:07+02	2020-08-04 13:01:02+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-04 13:00:15+02	2020-08-04 13:05:49+02	graphics	inkscape	\N	inkscape-1.0_3
2020-08-04 13:05:50+02	2020-08-04 13:09:23+02	graphics	wings	\N	wings-2.2.5
2020-08-04 13:09:23+02	2020-08-04 13:10:16+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-08-04 13:01:02+02	2020-08-04 13:10:18+02	graphics	krita	\N	krita-4.3.0_2
2020-08-04 13:10:19+02	2020-08-04 13:11:03+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-04 13:11:03+02	2020-08-04 13:13:00+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-04 13:10:17+02	2020-08-04 13:14:51+02	math	maxima	\N	maxima-5.44.0_3
2020-08-04 13:13:01+02	2020-08-04 13:15:07+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-04 13:15:08+02	2020-08-04 13:16:52+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-08-04 13:14:52+02	2020-08-04 13:17:10+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-04 13:17:11+02	2020-08-04 13:17:21+02	net	vnstat	\N	vnstat-2.6_1
2020-08-04 13:16:52+02	2020-08-04 13:18:02+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-04 13:17:21+02	2020-08-04 13:18:02+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-04 13:18:02+02	2020-08-04 13:18:43+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-04 13:18:03+02	2020-08-04 13:18:45+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-04 17:52:44+02	2020-08-04 17:52:54+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-08-04 13:18:44+02	2020-08-04 13:19:25+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-04 13:18:45+02	2020-08-04 13:19:27+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-04 13:19:26+02	2020-08-04 13:20:07+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-04 13:19:27+02	2020-08-04 13:20:09+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-04 13:20:07+02	2020-08-04 13:20:47+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-04 13:20:09+02	2020-08-04 13:21:12+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-04 13:20:50+02	2020-08-04 13:21:33+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-04 13:21:12+02	2020-08-04 13:21:54+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-04 13:21:33+02	2020-08-04 13:23:41+02	print	auctex	full	auctex-emacs26-12.2
2020-08-04 11:49:20+02	2020-08-04 13:25:45+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-04 13:25:46+02	2020-08-04 13:27:08+02	security	klee	\N	klee-2.1_1
2020-08-04 13:27:08+02	2020-08-04 13:27:19+02	security	nyx	\N	nyx-2.1.0_1
2020-08-04 13:27:19+02	2020-08-04 13:28:04+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-04 13:23:42+02	2020-08-04 13:28:07+02	print	texlive-full	\N	texlive-full-20150521
2020-08-04 13:21:55+02	2020-08-04 13:28:15+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-08-04 13:28:15+02	2020-08-04 13:28:26+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-08-04 13:28:26+02	2020-08-04 13:28:35+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-08-04 13:28:36+02	2020-08-04 13:28:46+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-08-04 13:28:05+02	2020-08-04 13:28:52+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-04 13:28:46+02	2020-08-04 13:28:57+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-08-04 13:28:52+02	2020-08-04 13:29:02+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-08-04 13:28:57+02	2020-08-04 13:29:09+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-08-04 13:29:03+02	2020-08-04 13:29:13+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-08-04 13:29:09+02	2020-08-04 13:29:19+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-08-04 13:29:14+02	2020-08-04 13:29:24+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-08-04 13:29:19+02	2020-08-04 13:29:33+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-08-04 13:29:24+02	2020-08-04 13:30:02+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-04 13:29:34+02	2020-08-04 13:30:40+02	x11	mate	\N	mate-1.22.2
2020-08-04 13:30:40+02	2020-08-04 13:31:06+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-04 13:30:03+02	2020-08-04 14:03:28+02	editors	vscode	\N	vscode-1.46.1
2020-08-04 13:28:08+02	2020-08-04 14:06:00+02	www	firefox	\N	firefox-79.0_2,1
2020-08-04 14:06:01+02	2020-08-04 14:20:25+02	local	E590T-localbase	\N	E590T-localbase-2020072800
2020-08-04 14:20:39+02	2020-08-04 14:21:30+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-04 14:20:39+02	2020-08-04 14:22:09+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-04 14:22:10+02	2020-08-04 14:22:17+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-04 17:41:42+02	2020-08-04 17:42:44+02	devel	gettext-runtime	\N	gettext-runtime-0.21
2020-08-04 17:41:42+02	2020-08-04 17:43:16+02	x11	libX11	\N	libX11-1.6.10_1,1
2020-08-04 17:43:16+02	2020-08-04 17:43:33+02	x11	libXext	\N	libXext-1.3.4,1
2020-08-04 17:41:42+02	2020-08-04 17:43:37+02	devel	libtextstyle	\N	libtextstyle-0.21
2020-08-04 17:43:33+02	2020-08-04 17:43:45+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-08-04 17:43:46+02	2020-08-04 17:43:57+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-08-04 17:43:57+02	2020-08-04 17:44:09+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-08-04 17:44:10+02	2020-08-04 17:44:21+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-08-04 17:44:21+02	2020-08-04 17:44:34+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-08-04 17:42:44+02	2020-08-04 17:45:03+02	mail	alpine	\N	alpine-2.23.2
2020-08-04 17:44:34+02	2020-08-04 17:45:07+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-08-04 17:45:04+02	2020-08-04 17:45:28+02	x11	libXi	\N	libXi-1.7.10,1
2020-08-04 17:45:08+02	2020-08-04 17:45:30+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-08-04 17:45:29+02	2020-08-04 17:45:45+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-08-04 17:45:30+02	2020-08-04 17:45:45+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-08-04 17:45:45+02	2020-08-04 17:45:57+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-08-04 17:45:46+02	2020-08-04 17:46:01+02	x11	libXpm	\N	libXpm-3.5.13
2020-08-04 17:45:57+02	2020-08-04 17:46:09+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-08-04 17:46:09+02	2020-08-04 17:46:21+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-08-04 17:46:01+02	2020-08-04 17:46:28+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-08-04 17:46:22+02	2020-08-04 17:46:35+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-08-04 17:46:29+02	2020-08-04 17:46:39+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-08-04 17:46:35+02	2020-08-04 17:46:46+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-08-04 17:46:40+02	2020-08-04 17:46:50+02	x11	xprop	\N	xprop-1.2.4
2020-08-04 17:46:46+02	2020-08-04 17:46:57+02	x11	xset	\N	xset-1.2.4_3
2020-08-04 17:46:57+02	2020-08-04 17:47:14+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-08-04 17:47:15+02	2020-08-04 17:47:52+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-08-04 17:47:53+02	2020-08-04 17:48:09+02	print	xpdfopen	\N	xpdfopen-0.86
2020-08-04 17:43:37+02	2020-08-04 17:48:11+02	devel	gettext-tools	\N	gettext-tools-0.21
2020-08-04 17:48:09+02	2020-08-04 17:48:20+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-08-04 17:48:11+02	2020-08-04 17:48:25+02	devel	p5-Locale-gettext	\N	p5-Locale-gettext-1.07
2020-08-04 17:46:51+02	2020-08-04 17:48:29+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-08-04 17:48:21+02	2020-08-04 17:48:38+02	devel	p5-Locale-libintl	\N	p5-Locale-libintl-1.31
2020-08-04 17:48:25+02	2020-08-04 17:48:39+02	misc	help2man	\N	help2man-1.47.16
2020-08-04 17:48:40+02	2020-08-04 17:49:55+02	print	texinfo	\N	texinfo-6.7_3,1
2020-08-04 17:49:55+02	2020-08-04 17:50:10+02	multimedia	libaacs	\N	libaacs-0.9.0
2020-08-04 17:50:11+02	2020-08-04 17:50:29+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-08-04 17:50:29+02	2020-08-04 17:50:52+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-08-04 17:48:39+02	2020-08-04 17:50:54+02	lang	python27	\N	python27-2.7.18_1
2020-08-04 17:50:52+02	2020-08-04 17:51:03+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-08-04 17:50:54+02	2020-08-04 17:51:05+02	devel	py-setuptools	py27	py27-setuptools-44.0.0
2020-08-04 17:51:05+02	2020-08-04 17:51:16+02	x11	libXres	\N	libXres-1.2.0_2
2020-08-04 17:51:17+02	2020-08-04 17:51:32+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-08-04 17:48:29+02	2020-08-04 17:51:38+02	lang	python37	\N	python37-3.7.8_1
2020-08-04 17:51:04+02	2020-08-04 17:51:40+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-08-04 17:51:33+02	2020-08-04 17:51:46+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-08-04 17:51:39+02	2020-08-04 17:51:51+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-08-04 17:51:46+02	2020-08-04 17:51:57+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-08-04 17:51:51+02	2020-08-04 17:52:00+02	devel	py-six	py37	py37-six-1.14.0
2020-08-04 17:51:57+02	2020-08-04 17:52:07+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-08-04 17:52:01+02	2020-08-04 17:52:11+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-08-04 17:52:08+02	2020-08-04 17:52:18+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-08-04 17:52:12+02	2020-08-04 17:52:31+02	devel	py-cffi	py37	py37-cffi-1.14.0_1
2020-08-04 17:51:40+02	2020-08-04 17:52:32+02	devel	ninja	\N	ninja-1.10.0,2
2020-08-04 17:52:18+02	2020-08-04 17:52:33+02	dns	py-idna	py37	py37-idna-2.8
2020-08-04 17:52:34+02	2020-08-04 17:52:43+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-08-04 17:52:33+02	2020-08-04 17:53:01+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-08-04 17:52:31+02	2020-08-04 17:53:06+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-08-04 17:53:02+02	2020-08-04 17:53:13+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-08-04 17:53:07+02	2020-08-04 17:53:17+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-08-04 17:53:17+02	2020-08-04 17:53:27+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-08-04 17:53:27+02	2020-08-04 17:53:37+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-08-04 17:53:38+02	2020-08-04 17:53:49+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-08-04 17:53:49+02	2020-08-04 17:54:01+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-08-04 17:52:54+02	2020-08-04 17:54:08+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-08-04 17:54:01+02	2020-08-04 17:54:12+02	www	py-requests	py37	py37-requests-2.22.0
2020-08-04 17:54:09+02	2020-08-04 17:54:19+02	devel	py-packaging	py37	py37-packaging-20.4
2020-08-04 17:54:12+02	2020-08-04 17:54:27+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-08-04 17:54:19+02	2020-08-04 17:54:29+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-08-04 17:54:27+02	2020-08-04 17:54:36+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-08-04 17:54:29+02	2020-08-04 17:54:39+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-08-04 17:54:37+02	2020-08-04 17:54:46+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-08-04 17:54:39+02	2020-08-04 17:54:49+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-08-04 17:53:14+02	2020-08-04 17:54:49+02	lang	cython	py37	py37-cython-0.29.15
2020-08-04 17:54:46+02	2020-08-04 17:54:55+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-08-04 17:54:49+02	2020-08-04 17:54:58+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-08-04 17:54:50+02	2020-08-04 17:55:03+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-08-04 17:54:56+02	2020-08-04 17:55:05+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-08-04 17:54:59+02	2020-08-04 17:55:13+02	devel	meson	\N	meson-0.54.3
2020-08-04 17:55:04+02	2020-08-04 17:55:13+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-08-04 17:55:06+02	2020-08-04 17:55:17+02	devel	py-future	py37	py37-future-0.18.2
2020-08-04 17:55:13+02	2020-08-04 17:55:23+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-08-04 17:55:18+02	2020-08-04 17:55:27+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-08-04 17:55:14+02	2020-08-04 17:55:32+02	textproc	py-sphinx	py37	py37-sphinx-3.1.2,1
2020-08-04 17:55:23+02	2020-08-04 17:55:34+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-08-04 17:55:32+02	2020-08-04 17:55:43+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-08-04 17:55:34+02	2020-08-04 17:56:24+02	devel	dbus	\N	dbus-1.12.20
2020-08-04 17:56:24+02	2020-08-04 17:56:46+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-08-04 17:56:47+02	2020-08-04 17:56:59+02	textproc	itstool	\N	itstool-2.0.6
2020-08-04 17:57:00+02	2020-08-04 17:57:45+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-08-04 17:57:46+02	2020-08-04 17:58:22+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-08-04 17:58:26+02	2020-08-04 17:58:49+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-08-04 17:58:51+02	2020-08-04 17:59:19+02	multimedia	libva	\N	libva-2.8.0
2020-08-04 17:55:27+02	2020-08-04 17:59:56+02	devel	glib20	\N	glib-2.56.3_9,1
2020-08-04 18:00:07+02	2020-08-04 18:00:42+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-08-04 18:00:42+02	2020-08-04 18:01:25+02	misc	shared-mime-info	\N	shared-mime-info-1.10_2
2020-08-04 18:01:25+02	2020-08-04 18:01:57+02	textproc	libcroco	\N	libcroco-0.6.13
2020-08-04 17:59:20+02	2020-08-04 18:02:16+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-08-04 18:02:17+02	2020-08-04 18:02:32+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-08-04 18:02:33+02	2020-08-04 18:02:47+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-08-04 18:02:47+02	2020-08-04 18:03:00+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-08-04 18:03:00+02	2020-08-04 18:03:14+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-08-04 18:03:14+02	2020-08-04 18:03:32+02	textproc	py-sparqlwrapper	py37	py37-sparqlwrapper-1.8.5
2020-08-04 18:03:33+02	2020-08-04 18:06:05+02	textproc	py-rdflib	py37	py37-rdflib-4.2.2_1
2020-08-04 18:06:06+02	2020-08-04 18:06:29+02	audio	lv2	\N	lv2-1.18.0
2020-08-04 18:06:29+02	2020-08-04 18:06:51+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-08-04 18:06:52+02	2020-08-04 18:07:28+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-08-04 18:07:29+02	2020-08-04 18:09:17+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-08-04 18:01:58+02	2020-08-04 18:10:10+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-08-04 18:09:17+02	2020-08-04 18:10:18+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-08-04 18:10:11+02	2020-08-04 18:12:03+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-08-04 18:12:04+02	2020-08-04 18:13:35+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-08-04 18:10:18+02	2020-08-04 18:13:54+02	net	qt5-network	\N	qt5-network-5.15.0
2020-08-04 18:13:36+02	2020-08-04 18:14:45+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-08-04 18:13:54+02	2020-08-04 18:14:56+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-08-04 18:14:46+02	2020-08-04 18:15:30+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-08-04 18:14:57+02	2020-08-04 18:15:44+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-08-04 18:15:32+02	2020-08-04 18:15:50+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-08-04 18:15:44+02	2020-08-04 18:15:56+02	textproc	py-toml	py37	py37-toml-0.10.0
2020-08-04 18:15:56+02	2020-08-04 18:16:08+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-08-04 18:16:09+02	2020-08-04 18:16:31+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-08-04 18:16:31+02	2020-08-04 18:17:25+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-08-04 18:17:25+02	2020-08-04 18:17:55+02	devel	php73-gettext	\N	php73-gettext-7.3.20
2020-08-04 18:17:56+02	2020-08-04 18:18:58+02	sysutils	policykit	\N	policykit-0.9_10
2020-08-04 18:15:50+02	2020-08-04 18:19:05+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-08-04 18:19:06+02	2020-08-04 18:19:19+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-08-04 18:18:59+02	2020-08-04 18:19:32+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-08-04 18:19:20+02	2020-08-04 18:19:52+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-08-04 18:19:53+02	2020-08-04 18:20:08+02	devel	py-six	py27	py27-six-1.14.0
2020-08-04 18:19:33+02	2020-08-04 18:20:10+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-08-04 18:20:10+02	2020-08-04 18:20:50+02	www	py-django22	py37	py37-django22-2.2.14
2020-08-04 18:20:08+02	2020-08-04 18:24:41+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-08-04 18:24:42+02	2020-08-04 18:25:03+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-08-04 18:25:04+02	2020-08-04 18:30:59+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-08-04 18:20:51+02	2020-08-04 18:35:00+02	devel	gdb	py37	gdb-9.2_1
2020-08-04 18:30:59+02	2020-08-04 18:35:13+02	devel	git	\N	git-2.27.0
2020-08-04 18:35:04+02	2020-08-04 18:35:15+02	devel	py-pytz	py27	py27-pytz-2020.1,1
2020-08-04 18:35:13+02	2020-08-04 18:35:29+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-08-04 18:35:15+02	2020-08-04 18:35:30+02	x11	xrdb	\N	xrdb-1.2.0
2020-08-04 18:35:30+02	2020-08-04 18:35:41+02	devel	py-pycparser	py27	py27-pycparser-2.20
2020-08-04 18:35:29+02	2020-08-04 18:36:09+02	devel	py-babel	py27	py27-Babel-2.8.0
2020-08-04 18:35:41+02	2020-08-04 18:38:07+02	graphics	exiv2	\N	exiv2-0.27.3,1
2020-08-04 18:38:07+02	2020-08-04 18:39:36+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-08-04 18:39:36+02	2020-08-04 18:39:49+02	textproc	py-MarkupSafe	py27	py27-MarkupSafe-1.1.1
2020-08-04 18:39:50+02	2020-08-04 18:40:08+02	devel	py-Jinja2	py27	py27-Jinja2-2.10.1
2020-08-04 18:40:08+02	2020-08-04 18:40:21+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-08-04 18:40:21+02	2020-08-04 18:40:34+02	devel	py-cffi	py27	py27-cffi-1.14.0_1
2020-08-04 18:40:35+02	2020-08-04 18:40:48+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-08-04 18:40:51+02	2020-08-04 18:41:05+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-08-04 18:41:06+02	2020-08-04 18:41:37+02	textproc	yelp-xsl	\N	yelp-xsl-3.28.0
2020-08-04 18:41:38+02	2020-08-04 18:41:59+02	x11	luit	\N	luit-20190106_1
2020-08-04 18:42:20+02	2020-08-04 18:42:44+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.8_6
2020-08-04 17:55:44+02	2020-08-04 21:55:29+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-08-04 21:55:33+02	2020-08-04 22:07:30+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-08-04 22:07:31+02	2020-08-04 22:10:21+02	graphics	cairo	\N	cairo-1.16.0,2
2020-08-04 22:10:22+02	2020-08-04 22:11:59+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-08-04 22:11:59+02	2020-08-04 22:15:15+02	print	harfbuzz	\N	harfbuzz-2.6.8
2020-08-04 22:15:16+02	2020-08-04 22:16:34+02	net	avahi-app	\N	avahi-app-0.7_3
2020-08-04 22:16:34+02	2020-08-04 22:18:08+02	print	cups	\N	cups-2.3.3_1
2020-08-04 22:18:09+02	2020-08-04 22:21:09+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-04 22:21:10+02	2020-08-04 22:21:48+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-08-04 22:21:49+02	2020-08-04 22:26:00+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_6
2020-08-04 22:26:01+02	2020-08-04 22:27:19+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-08-04 22:27:19+02	2020-08-04 22:32:02+02	graphics	graphviz	\N	graphviz-2.44.1
2020-08-04 22:32:03+02	2020-08-04 22:32:28+02	accessibility	atk	\N	atk-2.36.0
2020-08-04 22:32:29+02	2020-08-04 22:32:54+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-08-04 22:32:54+02	2020-08-04 22:35:40+02	lang	vala	\N	vala-0.40.15,1
2020-08-04 22:35:41+02	2020-08-04 22:36:57+02	sysutils	polkit	\N	polkit-0.116
2020-08-04 22:36:57+02	2020-08-04 22:38:14+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-08-04 22:38:15+02	2020-08-04 22:38:44+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-08-04 22:38:45+02	2020-08-04 22:39:59+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-04 22:40:00+02	2020-08-04 22:40:24+02	devel	libgudev	\N	libgudev-230_1
2020-08-04 22:40:25+02	2020-08-04 22:42:06+02	graphics	colord	\N	colord-1.3.5
2020-08-04 22:42:08+02	2020-08-04 22:42:29+02	x11	libwacom	\N	libwacom-1.4.1
2020-08-04 22:42:30+02	2020-08-04 22:43:05+02	x11	libinput	\N	libinput-1.15.6
2020-08-04 22:43:06+02	2020-08-04 22:44:02+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-08-04 22:44:04+02	2020-08-04 22:46:02+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-04 22:46:02+02	2020-08-04 22:55:28+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-04 18:36:11+02	2020-08-04 22:58:28+02	lang	rust	\N	rust-1.45.2
2020-08-04 22:58:31+02	2020-08-04 22:59:17+02	graphics	libGLU	\N	libGLU-9.0.1
2020-08-04 22:59:18+02	2020-08-04 23:01:00+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.6.8_2
2020-08-04 22:55:29+02	2020-08-04 23:07:22+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-04 23:01:01+02	2020-08-04 23:08:27+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-04 23:08:27+02	2020-08-04 23:09:06+02	graphics	poppler	\N	poppler-0.90.1
2020-08-04 23:09:07+02	2020-08-04 23:11:08+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-08-04 23:07:23+02	2020-08-04 23:13:07+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-04 23:11:09+02	2020-08-04 23:13:23+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-08-04 23:13:23+02	2020-08-04 23:13:42+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-08-04 23:13:42+02	2020-08-04 23:14:05+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-08-04 18:42:47+02	2020-08-04 23:14:43+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-08-04 23:14:06+02	2020-08-04 23:14:53+02	devel	dconf	\N	dconf-0.36.0
2020-08-04 23:14:54+02	2020-08-04 23:15:23+02	devel	json-glib	\N	json-glib-1.4.4
2020-08-04 23:14:44+02	2020-08-04 23:15:43+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-04 23:15:24+02	2020-08-04 23:16:31+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-08-04 23:16:31+02	2020-08-04 23:18:03+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-08-04 23:15:43+02	2020-08-04 23:18:32+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-08-04 23:18:33+02	2020-08-04 23:18:53+02	devel	libnotify	\N	libnotify-0.7.8
2020-08-04 23:18:53+02	2020-08-04 23:19:45+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-04 23:19:46+02	2020-08-04 23:20:52+02	security	libsecret	\N	libsecret-0.18.6_1
2020-08-04 23:18:03+02	2020-08-04 23:21:30+02	print	texlive-base	\N	texlive-base-20150521_53
2020-08-04 23:20:53+02	2020-08-04 23:22:02+02	graphics	frei0r	\N	frei0r-1.6.1
2020-08-04 23:22:02+02	2020-08-04 23:29:28+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-04 23:13:08+02	2020-08-04 23:29:48+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-04 23:29:28+02	2020-08-04 23:30:31+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-04 23:30:31+02	2020-08-04 23:30:49+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-08-04 23:30:50+02	2020-08-04 23:31:08+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-08-04 23:29:49+02	2020-08-04 23:31:08+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-08-04 23:31:08+02	2020-08-04 23:31:31+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-08-04 23:31:08+02	2020-08-04 23:32:31+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-04 23:31:31+02	2020-08-04 23:32:41+02	graphics	graphene	\N	graphene-1.10.0
2020-08-04 23:32:32+02	2020-08-04 23:32:45+02	www	libwpe	\N	libwpe-1.4.0.1
2020-08-04 23:32:41+02	2020-08-04 23:33:31+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.72.0
2020-08-04 23:32:45+02	2020-08-04 23:34:03+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-08-04 23:34:03+02	2020-08-04 23:34:21+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-08-04 23:33:32+02	2020-08-04 23:34:23+02	net	geoclue	\N	geoclue-2.5.5
2020-08-04 23:34:21+02	2020-08-04 23:34:55+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-04 23:34:56+02	2020-08-04 23:35:24+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-04 23:35:24+02	2020-08-04 23:36:23+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-08-04 23:36:23+02	2020-08-04 23:36:55+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-08-04 23:36:56+02	2020-08-04 23:38:26+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-04 23:38:27+02	2020-08-04 23:38:44+02	print	libraqm	\N	libraqm-0.6.0
2020-08-04 23:21:31+02	2020-08-04 23:39:14+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-04 23:38:45+02	2020-08-04 23:39:30+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-08-04 23:39:15+02	2020-08-04 23:42:04+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-04 23:39:30+02	2020-08-04 23:45:47+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-04 23:42:04+02	2020-08-04 23:47:25+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-04 23:47:25+02	2020-08-04 23:48:42+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-04 23:34:23+02	2020-08-04 23:51:13+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-04 23:45:51+02	2020-08-04 23:52:21+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-04 23:52:24+02	2020-08-04 23:53:50+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.72.0
2020-08-04 23:53:51+02	2020-08-04 23:54:43+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-04 23:54:43+02	2020-08-04 23:56:05+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-04 23:56:05+02	2020-08-04 23:57:11+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-04 23:50:19+02	2020-08-04 23:58:28+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-08-04 23:57:12+02	2020-08-04 23:58:42+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-04 23:58:28+02	2020-08-04 23:59:19+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-04 23:58:43+02	2020-08-05 00:00:18+02	devel	kf5-kconfig	\N	kf5-kconfig-5.72.0
2020-08-04 23:59:20+02	2020-08-05 00:00:30+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.7.3_3
2020-08-04 23:52:08+02	2020-08-05 00:00:51+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-05 00:00:18+02	2020-08-05 00:00:53+02	archivers	kf5-karchive	\N	kf5-karchive-5.72.0
2020-08-05 00:00:53+02	2020-08-05 00:01:20+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-08-05 00:00:30+02	2020-08-05 00:01:40+02	devel	kf5-ki18n	\N	kf5-ki18n-5.72.0
2020-08-05 00:01:21+02	2020-08-05 00:02:08+02	sysutils	hal	\N	hal-0.5.14_34
2020-08-05 00:01:41+02	2020-08-05 00:02:44+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-05 00:00:52+02	2020-08-05 00:03:39+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-05 00:02:45+02	2020-08-05 00:04:19+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-05 00:03:39+02	2020-08-05 00:04:56+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.72.0
2020-08-05 00:02:09+02	2020-08-05 00:05:15+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_3,1
2020-08-05 00:05:16+02	2020-08-05 00:07:31+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-08-05 00:07:31+02	2020-08-05 00:09:16+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.72.0
2020-08-05 00:04:19+02	2020-08-05 00:09:32+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-05 00:09:16+02	2020-08-05 00:10:14+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-05 00:09:33+02	2020-08-05 00:10:24+02	graphics	poppler-glib	\N	poppler-glib-0.90.1
2020-08-05 00:10:15+02	2020-08-05 00:12:50+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.72.0
2020-08-05 00:10:24+02	2020-08-05 00:13:15+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-08-05 00:12:51+02	2020-08-05 00:13:45+02	devel	kf5-kcrash	\N	kf5-kcrash-5.72.0
2020-08-05 00:04:56+02	2020-08-05 00:13:53+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-05 00:13:16+02	2020-08-05 00:14:13+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.72.0
2020-08-05 00:13:53+02	2020-08-05 00:14:49+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-05 00:13:46+02	2020-08-05 00:16:49+02	graphics	glew	\N	glew-2.2.0
2020-08-05 00:16:54+02	2020-08-05 00:17:45+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-08-05 00:18:25+02	2020-08-05 00:19:08+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-08-05 00:14:49+02	2020-08-05 00:19:33+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-08-05 00:19:15+02	2020-08-05 00:20:19+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-05 00:19:38+02	2020-08-05 00:20:41+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-05 00:20:19+02	2020-08-05 00:21:16+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.72.0
2020-08-05 00:20:42+02	2020-08-05 00:21:28+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-08-05 00:21:29+02	2020-08-05 00:22:58+02	devel	kf5-kauth	\N	kf5-kauth-5.72.0
2020-08-05 00:21:17+02	2020-08-05 00:24:00+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-05 00:22:59+02	2020-08-05 00:25:20+02	devel	kf5-kservice	\N	kf5-kservice-5.72.0
2020-08-05 00:24:00+02	2020-08-05 00:25:46+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-05 00:25:46+02	2020-08-05 00:26:03+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-08-05 00:25:20+02	2020-08-05 00:26:34+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-08-05 00:26:04+02	2020-08-05 00:27:23+02	net	remmina	\N	remmina-1.3.9
2020-08-05 00:27:24+02	2020-08-05 00:28:06+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-08-05 00:26:35+02	2020-08-05 00:29:00+02	security	gcr	\N	gcr-3.28.0
2020-08-05 00:28:07+02	2020-08-05 00:29:17+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.72.0
2020-08-05 00:29:01+02	2020-08-05 00:29:26+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-08-05 00:29:18+02	2020-08-05 00:29:44+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-08-05 00:29:26+02	2020-08-05 00:29:53+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-08-05 00:29:45+02	2020-08-05 00:30:22+02	devel	libgtop	\N	libgtop-2.40.0
2020-08-05 00:30:23+02	2020-08-05 00:30:49+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-08-05 00:14:13+02	2020-08-05 00:31:21+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-08-05 00:30:49+02	2020-08-05 00:31:31+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-05 00:31:32+02	2020-08-05 00:31:54+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-08-05 00:31:22+02	2020-08-05 00:32:00+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-08-05 00:32:01+02	2020-08-05 00:32:33+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-08-05 00:31:55+02	2020-08-05 00:32:44+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-08-05 00:29:54+02	2020-08-05 00:33:43+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-05 00:33:44+02	2020-08-05 00:34:32+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-05 00:32:45+02	2020-08-05 00:34:37+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.72.0
2020-08-05 00:34:38+02	2020-08-05 00:35:38+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.72.0
2020-08-05 00:34:33+02	2020-08-05 00:35:54+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.72.0
2020-08-05 00:35:38+02	2020-08-05 00:36:56+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-05 00:36:56+02	2020-08-05 00:37:48+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-05 00:32:34+02	2020-08-05 00:37:52+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-05 00:37:49+02	2020-08-05 00:40:03+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-05 00:40:05+02	2020-08-05 00:40:42+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-05 00:40:42+02	2020-08-05 00:42:27+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-05 00:42:28+02	2020-08-05 00:42:50+02	print	gl2ps	\N	gl2ps-1.4.0
2020-08-05 00:42:51+02	2020-08-05 00:44:08+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.72.0
2020-08-05 00:44:08+02	2020-08-05 00:46:09+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-05 00:46:09+02	2020-08-05 00:46:52+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-08-05 00:46:52+02	2020-08-05 00:47:04+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-08-05 00:47:05+02	2020-08-05 00:48:12+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-08-05 00:48:12+02	2020-08-05 00:50:22+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-08-05 00:35:55+02	2020-08-05 00:50:31+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-05 00:50:23+02	2020-08-05 00:53:53+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-08-05 00:53:54+02	2020-08-05 00:54:30+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-08-05 00:50:32+02	2020-08-05 00:55:00+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-05 00:54:31+02	2020-08-05 00:56:00+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.72.0
2020-08-05 00:55:01+02	2020-08-05 01:02:38+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-05 01:02:39+02	2020-08-05 01:02:55+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-08-05 01:02:55+02	2020-08-05 01:03:41+02	x11	xterm	\N	xterm-358
2020-08-05 01:03:42+02	2020-08-05 01:04:56+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.1.0
2020-08-05 01:04:57+02	2020-08-05 01:06:20+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.72.0
2020-08-05 01:06:20+02	2020-08-05 01:08:07+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-05 01:08:08+02	2020-08-05 01:14:27+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-05 01:14:28+02	2020-08-05 01:15:03+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-08-05 01:15:03+02	2020-08-05 01:16:21+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-05 01:16:21+02	2020-08-05 01:17:14+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-05 01:17:15+02	2020-08-05 01:18:12+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-05 01:18:13+02	2020-08-05 01:18:26+02	devel	py-asn1crypto	py27	py27-asn1crypto-1.3.0
2020-08-05 01:18:26+02	2020-08-05 01:18:43+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-08-05 01:18:43+02	2020-08-05 01:18:58+02	devel	py-enum34	py27	py27-enum34-1.1.6
2020-08-05 01:18:58+02	2020-08-05 01:20:39+02	devel	talloc	\N	talloc-2.3.1
2020-08-05 01:20:39+02	2020-08-05 01:24:01+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-08-05 01:24:02+02	2020-08-05 01:26:09+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-05 01:26:10+02	2020-08-05 01:26:23+02	net	py-ipaddress	py27	py27-ipaddress-1.0.23
2020-08-05 01:26:23+02	2020-08-05 01:26:41+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7
2020-08-05 01:26:41+02	2020-08-05 01:27:42+02	sysutils	upower	\N	upower-0.99.11
2020-08-05 00:37:53+02	2020-08-05 01:42:10+02	math	vtk8	\N	vtk8-8.2.0
2020-08-05 00:56:01+02	2020-08-05 02:07:31+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-05 02:07:32+02	2020-08-05 02:07:49+02	x11	appres	\N	appres-1.0.5
2020-08-05 02:08:11+02	2020-08-05 02:09:57+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.72.0
2020-08-05 02:09:58+02	2020-08-05 02:10:32+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-08-05 02:10:32+02	2020-08-05 02:14:45+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-05 02:14:46+02	2020-08-05 02:15:03+02	x11	sessreg	\N	sessreg-1.1.2
2020-08-05 02:15:04+02	2020-08-05 02:15:18+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-08-05 02:15:19+02	2020-08-05 02:15:35+02	x11	smproxy	\N	smproxy-1.0.6
2020-08-05 02:15:36+02	2020-08-05 02:15:55+02	x11	x11perf	\N	x11perf-1.6.1
2020-08-05 02:15:55+02	2020-08-05 02:16:14+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-08-05 02:16:14+02	2020-08-05 02:16:42+02	x11	xcalc	\N	xcalc-1.1.0
2020-08-05 02:16:42+02	2020-08-05 02:16:58+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-08-05 02:16:59+02	2020-08-05 02:17:16+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-08-05 02:17:16+02	2020-08-05 02:17:31+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-08-05 02:17:32+02	2020-08-05 02:17:50+02	x11	xev	\N	xev-1.2.4
2020-08-05 02:17:51+02	2020-08-05 02:18:05+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-08-05 02:18:05+02	2020-08-05 02:18:20+02	x11	xgamma	\N	xgamma-1.0.6
2020-08-05 02:18:20+02	2020-08-05 02:18:38+02	x11	xgc	\N	xgc-1.0.5
2020-08-05 02:18:40+02	2020-08-05 02:18:56+02	x11	xhost	\N	xhost-1.0.8
2020-08-05 02:18:56+02	2020-08-05 02:19:14+02	x11	xinit	\N	xinit-1.4.1,1
2020-08-05 02:19:15+02	2020-08-05 02:19:32+02	x11	xinput	\N	xinput-1.6.3
2020-08-05 02:19:33+02	2020-08-05 02:19:47+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-08-05 02:19:50+02	2020-08-05 02:20:09+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-08-05 02:20:09+02	2020-08-05 02:20:24+02	x11	xkill	\N	xkill-1.0.5
2020-08-05 02:20:24+02	2020-08-05 02:20:40+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-08-05 02:20:41+02	2020-08-05 02:20:58+02	x11	xmessage	\N	xmessage-1.0.5
2020-08-05 02:20:59+02	2020-08-05 02:21:17+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-08-05 02:21:17+02	2020-08-05 02:21:33+02	x11	xpr	\N	xpr-1.0.5
2020-08-05 02:21:34+02	2020-08-05 02:21:56+02	x11	xrandr	\N	xrandr-1.5.1
2020-08-05 02:21:57+02	2020-08-05 02:22:14+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-08-05 02:22:15+02	2020-08-05 02:22:32+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-08-05 02:22:32+02	2020-08-05 02:22:46+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-08-05 02:22:46+02	2020-08-05 02:23:04+02	x11	xwd	\N	xwd-1.0.7
2020-08-05 02:23:04+02	2020-08-05 02:23:22+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-08-05 02:23:22+02	2020-08-05 02:23:37+02	x11	xwud	\N	xwud-1.0.5
2020-08-05 02:23:38+02	2020-08-05 02:23:55+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-08-05 02:23:55+02	2020-08-05 02:25:10+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-05 02:25:11+02	2020-08-05 02:26:18+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-05 02:26:19+02	2020-08-05 02:27:28+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-05 02:27:29+02	2020-08-05 02:28:26+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-05 02:28:26+02	2020-08-05 02:29:23+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-05 01:42:12+02	2020-08-05 02:30:09+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-05 02:29:23+02	2020-08-05 02:31:19+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-08-05 02:30:26+02	2020-08-05 02:31:24+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-05 02:31:20+02	2020-08-05 02:32:13+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-05 02:31:24+02	2020-08-05 02:32:38+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.72.0
2020-08-05 02:32:39+02	2020-08-05 02:33:09+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-05 02:32:14+02	2020-08-05 02:33:32+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.72.0
2020-08-05 02:33:33+02	2020-08-05 02:33:49+02	x11-wm	twm	\N	twm-1.0.11
2020-08-05 02:33:49+02	2020-08-05 02:35:43+02	archivers	gcab	\N	gcab-1.3
2020-08-05 02:35:45+02	2020-08-05 02:40:41+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-08-05 02:40:44+02	2020-08-05 02:41:13+02	converters	py-webencodings	py27	py27-webencodings-0.5.1
2020-08-05 02:33:09+02	2020-08-05 02:46:15+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-05 02:41:16+02	2020-08-05 02:46:24+02	databases	tdb	\N	tdb-1.4.3,1
2020-08-05 02:46:25+02	2020-08-05 02:46:42+02	devel	gamin	\N	gamin-0.1.10_10
2020-08-05 02:46:43+02	2020-08-05 02:47:48+02	devel	kf5-knotifications	\N	kf5-knotifications-5.72.0
2020-08-05 02:47:48+02	2020-08-05 02:48:24+02	devel	librest	\N	rest-0.7.93
2020-08-05 02:48:24+02	2020-08-05 02:48:36+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-08-05 01:27:43+02	2020-08-05 02:49:42+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-05 02:48:37+02	2020-08-05 02:50:29+02	devel	tevent	\N	tevent-0.10.2
2020-08-05 02:50:32+02	2020-08-05 02:51:31+02	graphics	poppler-utils	\N	poppler-utils-0.90.1
2020-08-05 02:51:31+02	2020-08-05 02:51:49+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-08-05 02:51:50+02	2020-08-05 02:52:12+02	print	libotf	\N	libotf-0.9.16
2020-08-05 02:52:13+02	2020-08-05 02:53:39+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-08-05 02:53:39+02	2020-08-05 02:54:30+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-05 02:54:30+02	2020-08-05 02:55:29+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-05 02:55:30+02	2020-08-05 02:55:51+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-08-05 02:55:51+02	2020-08-05 02:56:39+02	sysutils	bsdisks	\N	bsdisks-0.18
2020-08-05 02:56:40+02	2020-08-05 02:58:22+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-08-05 02:58:28+02	2020-08-05 02:58:41+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-08-05 02:58:42+02	2020-08-05 02:59:42+02	x11	babl	\N	babl-0.1.78
2020-08-05 02:59:43+02	2020-08-05 03:00:06+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-08-05 03:00:07+02	2020-08-05 03:01:43+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-05 03:01:43+02	2020-08-05 03:02:20+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-08-05 03:02:20+02	2020-08-05 03:02:35+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-08-05 03:02:36+02	2020-08-05 03:03:57+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-05 02:50:32+02	2020-08-05 03:03:58+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-05 03:03:59+02	2020-08-05 03:07:40+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_3,1
2020-08-05 03:03:57+02	2020-08-05 03:08:25+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-05 03:09:00+02	2020-08-05 03:10:13+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-05 03:08:02+02	2020-08-05 03:10:48+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.72.0
2020-08-05 03:10:14+02	2020-08-05 03:11:47+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-05 03:10:49+02	2020-08-05 03:12:13+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-05 03:12:13+02	2020-08-05 03:13:59+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.72.0
2020-08-05 03:14:00+02	2020-08-05 03:16:22+02	devel	kf5-solid	\N	kf5-solid-5.72.0
2020-08-05 03:11:47+02	2020-08-05 03:17:26+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-08-05 03:17:27+02	2020-08-05 03:17:41+02	devel	py-click	py37	py37-click-7.0
2020-08-05 03:17:41+02	2020-08-05 03:17:54+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-08-05 03:17:55+02	2020-08-05 03:18:07+02	devel	py-olefile	py37	py37-olefile-0.46
2020-08-05 03:16:23+02	2020-08-05 03:18:12+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-08-05 03:18:08+02	2020-08-05 03:18:22+02	devel	py-ply	py37	py37-ply-3.11
2020-08-05 03:18:12+02	2020-08-05 03:18:24+02	devel	py-ply	py27	py27-ply-3.11
2020-08-05 03:18:25+02	2020-08-05 03:18:37+02	devel	py-zipp	py37	py37-zipp-3.1.0
2020-08-05 03:18:23+02	2020-08-05 03:18:38+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-08-05 03:18:38+02	2020-08-05 03:18:57+02	devel	scons	py37	scons-py37-3.1.2
2020-09-13 06:52:35+02	2020-09-13 06:52:50+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-08-05 03:18:57+02	2020-08-05 03:19:20+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-08-05 03:19:20+02	2020-08-05 03:20:06+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-08-05 03:20:07+02	2020-08-05 03:20:37+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-08-05 03:18:39+02	2020-08-05 03:22:21+02	graphics	gegl	\N	gegl-0.4.24_2
2020-08-05 03:22:22+02	2020-08-05 03:23:25+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0,1
2020-08-05 03:23:26+02	2020-08-05 03:25:57+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-05 02:46:16+02	2020-08-05 03:28:24+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-05 03:28:24+02	2020-08-05 03:28:39+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-08-05 03:28:39+02	2020-08-05 03:29:30+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-05 03:20:37+02	2020-08-05 03:29:41+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-05 03:29:31+02	2020-08-05 03:30:30+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-05 03:30:30+02	2020-08-05 03:31:26+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-08-05 03:29:41+02	2020-08-05 03:31:43+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-05 03:31:44+02	2020-08-05 03:32:48+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-05 03:32:48+02	2020-08-05 03:33:05+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-08-05 03:31:27+02	2020-08-05 03:33:08+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.72.0
2020-08-05 03:33:09+02	2020-08-05 03:33:20+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-08-05 03:33:06+02	2020-08-05 03:33:26+02	textproc	yelp-tools	\N	yelp-tools-3.28.0
2020-08-05 03:33:21+02	2020-08-05 03:33:31+02	www	py-html5lib	py27	py27-html5lib-1.0.1
2020-08-05 03:33:26+02	2020-08-05 03:33:38+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-08-05 03:33:32+02	2020-08-05 03:33:46+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-08-05 03:33:39+02	2020-08-05 03:34:23+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-08-05 03:33:47+02	2020-08-05 03:34:53+02	x11	kf5-kded	\N	kf5-kded-5.72.0_1
2020-08-05 03:34:53+02	2020-08-05 03:36:06+02	x11	zenity	\N	zenity-3.32.0
2020-08-05 03:34:23+02	2020-08-05 03:37:26+02	x11	xorg	\N	xorg-7.7_3
2020-08-05 03:37:27+02	2020-08-05 03:38:40+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.72.0
2020-08-05 03:38:41+02	2020-08-05 03:38:58+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-08-05 03:38:58+02	2020-08-05 03:39:14+02	databases	py-redis	py37	py37-redis-3.5.3
2020-08-05 03:39:15+02	2020-08-05 03:40:06+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-08-05 03:36:06+02	2020-08-05 03:40:28+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-05 03:40:28+02	2020-08-05 03:43:41+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-05 03:43:43+02	2020-08-05 03:45:14+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-05 03:25:58+02	2020-08-05 03:45:18+02	net	samba410	\N	samba410-4.10.17
2020-08-05 03:45:15+02	2020-08-05 03:45:27+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-08-05 03:40:07+02	2020-08-05 03:45:31+02	devel	kf5-kio	\N	kf5-kio-5.72.0
2020-08-05 03:45:19+02	2020-08-05 03:45:33+02	devel	py-appdirs	py37	py37-appdirs-1.4.3
2020-08-05 03:45:27+02	2020-08-05 03:45:35+02	devel	py-backports	py37	py37-backports-1
2020-08-05 03:45:32+02	2020-08-05 03:45:41+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-08-05 03:45:33+02	2020-08-05 03:45:43+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-08-05 03:45:42+02	2020-08-05 03:45:51+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-1.7.0
2020-08-05 03:45:43+02	2020-08-05 03:45:52+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-08-05 03:45:36+02	2020-08-05 03:45:56+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-08-05 03:45:52+02	2020-08-05 03:46:02+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-08-05 03:45:53+02	2020-08-05 03:46:02+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-08-05 03:45:57+02	2020-08-05 03:46:05+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-08-05 03:46:03+02	2020-08-05 03:46:11+02	devel	py-wheel	py27	py27-wheel-0.30.0_1
2020-08-05 03:46:02+02	2020-08-05 03:46:12+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-08-05 03:46:06+02	2020-08-05 03:46:15+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-08-05 03:46:16+02	2020-08-05 03:47:39+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-05 03:46:12+02	2020-08-05 03:47:49+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-05 03:46:12+02	2020-08-05 03:47:49+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-05 03:47:50+02	2020-08-05 03:48:00+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-08-05 03:47:50+02	2020-08-05 03:51:11+02	dns	bind-tools	\N	bind-tools-9.16.5
2020-08-05 03:47:40+02	2020-08-05 03:51:28+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-05 03:51:28+02	2020-08-05 03:51:46+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-08-05 03:48:00+02	2020-08-05 03:52:18+02	editors	emacs	full	emacs-26.3_6,3
2020-08-05 03:52:19+02	2020-08-05 03:52:53+02	graphics	glfw	\N	glfw-3.3.2
2020-08-05 03:52:53+02	2020-08-05 03:54:48+02	graphics	movit	\N	movit-1.6.2_2
2020-08-05 03:51:12+02	2020-08-05 03:57:41+02	graphics	Coin	\N	Coin-4.0.0_1
2020-08-05 03:57:42+02	2020-08-05 03:58:36+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-08-05 03:58:37+02	2020-08-05 03:59:02+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-08-05 03:59:02+02	2020-08-05 03:59:30+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-08-05 03:59:31+02	2020-08-05 04:02:36+02	lang	cython	py27	py27-cython-0.29.15
2020-08-05 03:54:48+02	2020-08-05 04:06:12+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-05 03:51:46+02	2020-08-05 04:05:54+02	graphics	gimp-app	\N	gimp-app-2.10.20_2,1
2020-08-05 04:08:01+02	2020-08-05 04:09:11+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-05 04:09:11+02	2020-08-05 04:09:27+02	net	gssdp	\N	gssdp-1.2.3
2020-08-05 04:09:28+02	2020-08-05 04:10:08+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-08-05 04:10:09+02	2020-08-05 04:10:57+02	print	cups-filters	\N	cups-filters-1.27.5_1
2020-08-05 04:08:01+02	2020-08-05 04:12:50+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-08-05 04:10:57+02	2020-08-05 04:19:34+02	print	fontforge	\N	fontforge-20190801_2
2020-08-05 04:12:50+02	2020-08-05 04:25:32+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-08-05 04:25:33+02	2020-08-05 04:25:48+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-08-05 04:25:48+02	2020-08-05 04:26:39+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-05 04:26:40+02	2020-08-05 04:28:52+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-08-05 04:28:53+02	2020-08-05 04:29:07+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-08-05 04:29:07+02	2020-08-05 04:29:20+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-08-05 04:29:20+02	2020-08-05 04:29:39+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-08-05 04:19:35+02	2020-08-05 04:38:21+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-08-05 04:38:22+02	2020-08-05 04:38:39+02	www	py-h2	py37	py37-h2-3.2.0
2020-08-05 04:38:40+02	2020-08-05 04:38:55+02	www	py-hyperlink	py37	py37-hyperlink-19.0.0
2020-08-05 04:38:56+02	2020-08-05 04:39:11+02	www	py-priority	py37	py37-priority-1.3.0
2020-08-05 04:39:12+02	2020-08-05 04:40:50+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-05 04:40:50+02	2020-08-05 04:43:04+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-05 04:43:05+02	2020-08-05 04:44:31+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-05 04:44:32+02	2020-08-05 04:45:36+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-05 04:45:37+02	2020-08-05 04:47:46+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-05 04:47:47+02	2020-08-05 04:59:47+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-08-05 04:59:47+02	2020-08-05 05:03:31+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-05 05:03:31+02	2020-08-05 05:07:06+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-05 05:07:07+02	2020-08-05 05:08:23+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-05 05:08:24+02	2020-08-05 05:12:46+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-05 05:12:47+02	2020-08-05 05:22:37+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-05 05:22:39+02	2020-08-05 05:33:14+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-05 07:23:43+02	2020-08-05 07:25:16+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-05 05:33:15+02	2020-08-05 05:34:38+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-05 05:34:38+02	2020-08-05 05:35:40+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-08-05 05:35:41+02	2020-08-05 05:38:26+02	x11-wm	marco	\N	marco-1.22.4
2020-08-05 05:38:27+02	2020-08-05 05:39:24+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-05 05:39:24+02	2020-08-05 05:39:59+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-08-05 05:40:00+02	2020-08-05 05:42:18+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-05 05:42:19+02	2020-08-05 05:43:28+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-08-05 05:43:29+02	2020-08-05 05:46:51+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-05 05:46:51+02	2020-08-05 05:50:09+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-05 05:50:39+02	2020-08-05 05:50:51+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-08-05 05:50:52+02	2020-08-05 05:51:05+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-08-05 05:51:05+02	2020-08-05 05:51:16+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-08-05 05:51:17+02	2020-08-05 05:51:28+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-08-05 05:51:29+02	2020-08-05 05:51:41+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-08-05 05:51:41+02	2020-08-05 05:51:55+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-08-05 05:51:55+02	2020-08-05 05:52:06+02	devel	py-pyasn1	py27	py27-pyasn1-0.4.7
2020-08-05 05:52:06+02	2020-08-05 05:52:59+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-08-05 05:52:59+02	2020-08-05 05:53:23+02	devel	pydbus-common	\N	pydbus-common-1.2.16
2020-08-05 04:02:37+02	2020-08-05 06:04:11+02	lang	ghc	\N	ghc-8.8.4
2020-08-05 06:04:12+02	2020-08-05 06:04:24+02	dns	py-idna	py27	py27-idna-2.8
2020-08-05 06:04:25+02	2020-08-05 06:12:32+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-05 06:12:33+02	2020-08-05 06:14:16+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-05 06:14:17+02	2020-08-05 06:14:38+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-08-05 06:14:39+02	2020-08-05 06:16:40+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-05 06:16:40+02	2020-08-05 06:17:59+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-08-05 06:18:00+02	2020-08-05 06:18:34+02	graphics	libgxps	\N	libgxps-0.3.1
2020-08-05 06:18:35+02	2020-08-05 06:20:47+02	graphics	libheif	\N	libheif-1.6.2
2020-08-05 06:20:47+02	2020-08-05 06:21:20+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-08-05 06:21:21+02	2020-08-05 06:24:26+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.1
2020-08-05 06:24:27+02	2020-08-05 06:24:59+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-05 06:25:00+02	2020-08-05 06:26:04+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-05 06:26:04+02	2020-08-05 06:27:11+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-05 06:27:11+02	2020-08-05 06:28:46+02	graphics	vigra	\N	vigra-1.11.1_11
2020-08-05 05:53:23+02	2020-08-05 06:32:49+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-05 06:28:47+02	2020-08-05 06:33:18+02	lang	python38	\N	python38-3.8.5
2020-08-05 06:33:18+02	2020-08-05 06:40:50+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-05 06:40:51+02	2020-08-05 06:41:43+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-08-05 06:41:43+02	2020-08-05 06:42:43+02	multimedia	libass	\N	libass-0.14.0
2020-08-05 06:42:43+02	2020-08-05 06:44:45+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-05 06:44:45+02	2020-08-05 06:47:23+02	net	freerdp	\N	freerdp-2.2.0
2020-08-05 06:47:24+02	2020-08-05 06:47:51+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-05 06:47:52+02	2020-08-05 06:48:09+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-08-05 06:48:09+02	2020-08-05 06:48:18+02	net	py-pysocks	py27	py27-pysocks-1.7.1
2020-08-05 06:48:19+02	2020-08-05 06:48:31+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-08-05 06:48:32+02	2020-08-05 06:49:19+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-05 06:49:20+02	2020-08-05 06:50:12+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-05 06:32:50+02	2020-08-05 06:51:52+02	math	R	\N	R-4.0.2
2020-08-05 06:51:53+02	2020-08-05 06:52:19+02	print	libspectre	\N	libspectre-0.2.9
2020-08-05 06:50:15+02	2020-08-05 06:54:05+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-05 06:52:19+02	2020-08-05 06:54:30+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-05 06:54:31+02	2020-08-05 07:00:24+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-05 06:54:06+02	2020-08-05 07:00:27+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-05 07:00:28+02	2020-08-05 07:00:40+02	security	py-bcrypt	py27	py27-bcrypt-3.1.7
2020-08-05 07:00:40+02	2020-08-05 07:00:52+02	security	py-certifi	py27	py27-certifi-2020.6.20
2020-08-05 07:00:52+02	2020-08-05 07:01:03+02	security	py-openssl	py27	py27-openssl-19.0.0
2020-08-05 07:00:25+02	2020-08-05 07:01:12+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-08-05 07:01:13+02	2020-08-05 07:01:31+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-08-05 07:01:04+02	2020-08-05 07:01:45+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-08-05 07:01:31+02	2020-08-05 07:01:46+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-08-05 07:01:46+02	2020-08-05 07:02:05+02	sysutils	hal-info	\N	hal-info-20091130
2020-08-05 07:02:05+02	2020-08-05 07:02:59+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-05 07:01:47+02	2020-08-05 07:05:09+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-05 07:03:00+02	2020-08-05 07:05:43+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-05 07:05:43+02	2020-08-05 07:06:19+02	textproc	enchant	\N	enchant-1.6.0_9
2020-08-05 07:06:20+02	2020-08-05 07:06:34+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-08-05 07:06:34+02	2020-08-05 07:06:47+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-08-05 07:06:48+02	2020-08-05 07:07:01+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-08-05 07:05:10+02	2020-08-05 07:07:06+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-05 07:07:02+02	2020-08-05 07:07:24+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-08-05 07:07:07+02	2020-08-05 07:07:50+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-05 07:07:51+02	2020-08-05 07:08:03+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-08-05 07:07:25+02	2020-08-05 07:08:28+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-05 07:08:03+02	2020-08-05 07:08:39+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-05 07:08:40+02	2020-08-05 07:08:48+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-08-05 07:08:49+02	2020-08-05 07:08:58+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-08-05 07:08:29+02	2020-08-05 07:09:51+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-05 07:09:51+02	2020-08-05 07:10:28+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-08-05 07:08:59+02	2020-08-05 07:11:03+02	x11-servers	xephyr	\N	xephyr-1.20.8_3,1
2020-08-05 07:11:03+02	2020-08-05 07:11:32+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-08-05 07:10:29+02	2020-08-05 07:11:41+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-05 07:11:33+02	2020-08-05 07:12:21+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-05 07:11:41+02	2020-08-05 07:13:48+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-05 07:12:22+02	2020-08-05 07:14:14+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-05 07:13:49+02	2020-08-05 07:16:34+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-05 07:16:35+02	2020-08-05 07:19:35+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-05 07:19:36+02	2020-08-05 07:20:55+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-08-05 07:20:56+02	2020-08-05 07:21:21+02	audio	dssi	\N	dssi-1.1.1_8
2020-08-05 07:21:23+02	2020-08-05 07:22:08+02	audio	faudio	\N	FAudio-20.08
2020-08-05 07:14:15+02	2020-08-05 07:23:28+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-05 07:22:08+02	2020-08-05 07:23:30+02	audio	mate-media	\N	mate-media-1.24.0
2020-08-05 07:23:29+02	2020-08-05 07:23:43+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-08-05 07:23:30+02	2020-08-05 07:24:51+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-15 15:24:59+02	2020-08-15 15:25:15+02	x11	sterm	\N	sterm-0.8.4
2020-08-05 07:25:17+02	2020-08-05 07:26:14+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-05 07:26:15+02	2020-08-05 07:27:21+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-05 07:27:22+02	2020-08-05 07:27:46+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-08-05 07:27:46+02	2020-08-05 07:28:13+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-08-05 07:24:52+02	2020-08-05 07:28:54+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-05 07:28:14+02	2020-08-05 07:29:38+02	devel	gradle5	\N	gradle5-5.0
2020-08-05 07:28:54+02	2020-08-05 07:30:02+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.72.0
2020-08-05 07:29:38+02	2020-08-05 07:31:18+02	devel	libarea	\N	libarea-20160313_13
2020-08-05 07:31:20+02	2020-08-05 07:31:35+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-08-05 07:31:36+02	2020-08-05 07:31:51+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-08-05 07:30:03+02	2020-08-05 07:31:53+02	devel	ode	\N	ode-0.13_4,1
2020-08-05 07:31:56+02	2020-08-05 07:32:09+02	devel	py-funcy	py37	py37-funcy-1.12
2020-08-05 07:31:52+02	2020-08-05 07:32:13+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-08-05 07:32:09+02	2020-08-05 07:32:23+02	devel	py-future	py27	py27-future-0.18.2
2020-08-05 07:32:24+02	2020-08-05 07:32:37+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-08-05 07:32:37+02	2020-08-05 07:32:49+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-08-05 07:32:50+02	2020-08-05 07:33:07+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-08-05 07:33:08+02	2020-08-05 07:33:19+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-08-05 07:33:20+02	2020-08-05 07:33:34+02	devel	py-rq	py37	py37-rq-1.3.0
2020-08-05 07:32:13+02	2020-08-05 07:33:42+02	devel	py-game	py37	py37-game-1.9.6_1
2020-08-05 07:33:42+02	2020-08-05 07:33:55+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-08-05 07:33:35+02	2020-08-05 07:34:05+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-08-05 07:33:56+02	2020-08-05 07:34:10+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-08-05 07:34:11+02	2020-08-05 07:34:22+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-08-05 07:34:06+02	2020-08-05 07:34:22+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-08-05 07:34:23+02	2020-08-05 07:36:25+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-05 07:36:26+02	2020-08-05 07:36:45+02	devel	scons	py27	scons-py27-3.1.2
2020-08-05 07:36:45+02	2020-08-05 07:37:00+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-08-05 07:37:00+02	2020-08-05 07:37:13+02	dns	py-dnspython	py27	py27-dnspython-1.16.0
2020-08-05 07:37:14+02	2020-08-05 07:40:13+02	dns	py-ldns	py27	py27-ldns-1.7.1_1
2020-08-05 07:34:25+02	2020-08-05 07:41:15+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3_3
2020-08-05 07:40:14+02	2020-08-05 08:06:46+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-05 08:06:47+02	2020-08-05 08:12:55+02	editors	pluma	\N	pluma-1.24.0
2020-08-05 08:12:56+02	2020-08-05 08:16:22+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-08-05 08:16:23+02	2020-08-05 08:21:40+02	french	med	\N	fr-med-4.0.0
2020-08-05 08:21:41+02	2020-08-05 08:26:06+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-08-05 08:26:07+02	2020-08-05 08:30:16+02	graphics	atril	\N	atril-1.22.3_7
2020-08-05 08:30:17+02	2020-08-05 08:33:03+02	graphics	eom	\N	eom-1.24.0
2020-08-05 08:33:04+02	2020-08-05 08:34:32+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-05 08:34:33+02	2020-08-05 08:34:57+02	graphics	gle	\N	gle-3.1.0_7
2020-08-05 08:34:58+02	2020-08-05 08:35:17+02	graphics	gltt	\N	gltt-2.5.2_14
2020-08-05 08:35:18+02	2020-08-05 08:36:33+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-05 07:41:16+02	2020-08-05 08:41:32+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-08-05 08:36:33+02	2020-08-05 08:44:48+02	graphics	openimageio	\N	openimageio-2.1.15.0_4
2020-08-05 08:41:34+02	2020-08-05 08:45:38+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-08-05 08:44:49+02	2020-08-05 08:46:28+02	graphics	py-gimp	\N	py27-gimp-2.10.20_2
2020-08-05 08:46:28+02	2020-08-05 08:47:04+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-08-05 08:47:05+02	2020-08-05 08:47:44+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-08-05 08:45:38+02	2020-08-05 08:52:28+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-05 08:47:44+02	2020-08-05 09:01:57+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-05 08:52:31+02	2020-08-05 09:20:04+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-08-05 09:20:05+02	2020-08-05 09:20:16+02	lang	python3	\N	python3-3_3
2020-08-05 09:02:01+02	2020-08-05 09:20:30+02	lang	erlang-wx	\N	erlang-wx-21.3.8.16,1
2020-08-05 09:20:31+02	2020-08-05 09:21:43+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-05 09:20:17+02	2020-08-05 09:22:11+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-05 09:21:44+02	2020-08-05 09:24:58+02	math	stp	\N	stp-2.3.3_6,1
2020-08-05 09:22:12+02	2020-08-05 09:28:02+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-08-05 09:28:02+02	2020-08-05 09:28:24+02	misc	xosd	\N	xosd-2.2.14_3
2020-08-05 09:28:24+02	2020-08-05 09:30:38+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-05 09:30:39+02	2020-08-05 09:31:45+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-08-05 09:31:46+02	2020-08-05 09:34:32+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-05 09:24:58+02	2020-08-05 09:41:21+02	misc	openvdb	\N	openvdb-7.0.0_4
2020-08-05 09:34:33+02	2020-08-05 09:41:52+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-05 09:41:53+02	2020-08-05 09:42:06+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-08-05 09:41:22+02	2020-08-05 09:42:29+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-05 09:42:30+02	2020-08-05 09:42:42+02	net	py-urllib3	py27	py27-urllib3-1.25.7,1
2020-08-05 09:42:06+02	2020-08-05 09:42:46+02	net	py-pyzmq	py37	py37-pyzmq-19.0.2
2020-08-05 09:42:42+02	2020-08-05 09:43:29+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-05 09:42:47+02	2020-08-05 09:43:39+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-05 09:43:30+02	2020-08-05 09:44:17+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-05 09:43:40+02	2020-08-05 09:44:31+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-05 09:44:18+02	2020-08-05 09:45:04+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-05 09:44:31+02	2020-08-05 09:45:20+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-05 09:45:05+02	2020-08-05 09:46:05+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-05 09:45:20+02	2020-08-05 09:46:46+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-05 09:46:05+02	2020-08-05 09:46:51+02	net	vde2	\N	vde2-2.3.2_4
2020-08-05 09:46:47+02	2020-08-05 09:47:40+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-05 09:47:41+02	2020-08-05 09:47:51+02	net-mgmt	py-ipcalc	py27	py27-ipcalc-1.99.0
2020-08-05 09:47:52+02	2020-08-05 09:48:11+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-08-05 09:46:52+02	2020-08-05 09:48:14+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-08-05 09:48:11+02	2020-08-05 09:49:22+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-05 09:48:14+02	2020-08-05 09:55:10+02	print	lilypond	\N	lilypond-2.20.0
2020-08-05 09:49:22+02	2020-08-05 09:56:27+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-05 09:56:28+02	2020-08-05 09:58:26+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-05 09:55:11+02	2020-08-05 10:01:02+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-05 09:58:26+02	2020-08-05 10:05:30+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-08-05 10:05:31+02	2020-08-05 10:06:06+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-05 10:06:07+02	2020-08-05 10:06:20+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-08-05 10:06:21+02	2020-08-05 10:06:32+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-08-05 10:06:33+02	2020-08-05 10:06:47+02	security	py-stem	py37	py37-stem-1.8.0
2020-08-05 10:06:48+02	2020-08-05 10:07:42+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-05 10:07:43+02	2020-08-05 10:09:32+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-15 15:25:15+02	2020-08-15 15:25:37+02	x11	xclip	\N	xclip-0.13
2020-08-05 10:09:33+02	2020-08-05 10:11:49+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-05 10:11:49+02	2020-08-05 10:12:04+02	sysutils	py-psutil	py27	py27-psutil-5.7.2
2020-08-05 10:12:04+02	2020-08-05 10:13:51+02	textproc	fop	\N	fop-2.3
2020-08-05 10:13:52+02	2020-08-05 10:14:54+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-05 10:14:55+02	2020-08-05 10:15:07+02	textproc	py-chardet	py27	py27-chardet-3.0.4_3
2020-08-05 10:15:07+02	2020-08-05 10:16:06+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-05 10:16:07+02	2020-08-05 10:16:46+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-08-05 10:16:46+02	2020-08-05 10:16:58+02	textproc	py-scour	py37	py37-scour-0.37
2020-08-05 10:16:59+02	2020-08-05 10:17:12+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-08-05 10:17:13+02	2020-08-05 10:17:39+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-08-05 10:17:40+02	2020-08-05 10:18:06+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-08-05 10:18:07+02	2020-08-05 10:18:19+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-08-05 10:18:19+02	2020-08-05 10:18:37+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-08-05 10:18:37+02	2020-08-05 10:18:50+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-08-05 10:18:50+02	2020-08-05 10:19:04+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-08-05 10:19:04+02	2020-08-05 10:19:20+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-08-05 10:19:20+02	2020-08-05 10:19:32+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-08-05 10:19:33+02	2020-08-05 10:20:41+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-05 10:01:02+02	2020-08-05 10:21:36+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-05 10:20:42+02	2020-08-05 10:21:55+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-05 10:21:55+02	2020-08-05 10:23:13+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-05 10:23:14+02	2020-08-05 10:24:43+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-05 10:24:44+02	2020-08-05 10:26:56+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-05 10:21:38+02	2020-08-05 10:27:35+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-05 10:27:36+02	2020-08-05 10:29:20+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-08-05 10:26:57+02	2020-08-05 10:29:23+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-05 10:29:23+02	2020-08-05 10:30:03+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-05 10:29:21+02	2020-08-05 10:30:26+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-05 10:30:04+02	2020-08-05 10:39:18+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-05 10:39:18+02	2020-08-05 10:39:31+02	astro	xearth	\N	xearth-1.2
2020-08-05 10:39:32+02	2020-08-05 10:42:22+02	astro	xephem	\N	xephem-3.7.7_5
2020-08-05 10:42:23+02	2020-08-05 10:49:44+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-05 10:49:46+02	2020-08-05 10:50:07+02	audio	xmix	\N	xmix-2.1_3
2020-08-05 10:30:26+02	2020-08-05 10:59:09+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-05 10:59:10+02	2020-08-05 11:05:20+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-05 11:05:21+02	2020-08-05 11:08:15+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-05 10:50:08+02	2020-08-05 11:24:57+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-05 11:24:59+02	2020-08-05 11:26:56+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-05 11:26:57+02	2020-08-05 11:28:48+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-05 11:28:49+02	2020-08-05 11:29:30+02	converters	recode	\N	recode-3.6_14
2020-08-05 11:29:31+02	2020-08-05 11:29:43+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-08-05 11:29:43+02	2020-08-05 11:35:24+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-05 11:35:25+02	2020-08-05 11:35:51+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-08-05 11:35:51+02	2020-08-05 11:38:19+02	deskutils	calibre	\N	calibre-4.22.0
2020-08-05 11:38:20+02	2020-08-05 11:39:10+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-05 11:39:11+02	2020-08-05 11:39:24+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-08-05 11:39:25+02	2020-08-05 11:39:45+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-08-05 11:39:46+02	2020-08-05 12:08:35+02	devel	RStudio	\N	RStudio-1.3.1056
2020-08-05 12:08:36+02	2020-08-05 12:10:17+02	devel	bzr	py27	bzr-2.7.0.6622
2020-08-05 12:10:17+02	2020-08-05 12:11:25+02	devel	distel	full	distel-emacs26-4.1.1_4
2020-08-05 11:08:16+02	2020-08-05 12:54:08+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-05 12:11:25+02	2020-08-05 13:23:19+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-05 12:54:09+02	2020-08-05 13:49:04+02	devel	ghidra	\N	ghidra-9.1
2020-08-05 13:23:22+02	2020-08-05 20:23:32+02	devel	llvm-devel	\N	llvm-devel-11.0.d20200706_1
2020-08-05 20:23:35+02	2020-08-05 20:25:55+02	devel	mercurial	py27	mercurial-5.1.2
2020-08-05 20:25:56+02	2020-08-05 20:26:13+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-08-05 20:26:14+02	2020-08-05 20:26:55+02	devel	tig	\N	tig-2.5.1
2020-08-05 20:26:56+02	2020-08-05 20:31:48+02	dns	bind916	\N	bind916-9.16.5_1
2020-08-05 20:31:49+02	2020-08-05 20:32:04+02	dns	sshfp	\N	sshfp-1.2.2_5
2020-08-05 13:49:06+02	2020-08-05 20:44:05+02	devel	llvm10	\N	llvm10-10.0.1
2020-08-05 20:44:07+02	2020-08-05 20:46:47+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-08-05 20:46:48+02	2020-08-05 20:50:00+02	editors	vim	\N	vim-8.2.1334
2020-08-05 20:32:07+02	2020-08-05 21:03:34+02	editors	codelite	\N	codelite-12.0_4
2020-08-05 20:50:02+02	2020-08-05 21:20:58+02	emulators	qemu	\N	qemu-4.2.1
2020-08-05 21:21:00+02	2020-08-05 21:43:23+02	emulators	wine	\N	wine-5.0.1,1
2020-08-05 21:43:23+02	2020-08-05 21:44:18+02	emulators	x48	\N	x48-0.6.4_1
2020-08-05 21:44:19+02	2020-08-05 22:02:49+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-05 22:02:55+02	2020-08-05 22:03:49+02	games	KnightCap	\N	knightcap-3.6_8
2020-08-05 22:03:54+02	2020-08-05 22:04:30+02	games	affenspiel	\N	affenspiel-1.0_2
2020-08-05 22:04:35+02	2020-08-05 22:05:48+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-08-05 22:05:51+02	2020-08-05 22:13:06+02	games	alienarena	\N	alienarena-2013.766_9
2020-08-05 22:13:09+02	2020-08-05 22:16:41+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-08-05 21:03:36+02	2020-08-05 22:21:26+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44
2020-08-05 22:21:31+02	2020-08-05 22:23:37+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-08-05 22:23:40+02	2020-08-05 22:26:46+02	games	monsterz	\N	monsterz-0.7.1_15
2020-08-05 22:26:49+02	2020-08-05 22:29:19+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-05 22:16:43+02	2020-08-05 22:31:10+02	games	battletanks	\N	btanks-0.9.8083_9
2020-08-05 22:29:21+02	2020-08-05 22:32:22+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-05 22:31:13+02	2020-08-05 22:35:27+02	games	xblast	\N	xblast-2.10.4_17
2020-08-05 22:32:25+02	2020-08-05 22:36:37+02	games	xboard	\N	xboard-4.9.1_1
2020-08-05 22:36:39+02	2020-08-05 22:38:26+02	games	xevil	\N	xevil-2.02.r2_2
2020-08-05 22:35:30+02	2020-08-05 22:39:00+02	games	xcowsay	\N	xcowsay-1.5
2020-08-05 22:38:32+02	2020-08-05 22:39:41+02	games	xminesweep	\N	xminesweep-3.0_2
2020-08-05 22:39:44+02	2020-08-05 22:56:31+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-05 22:56:37+02	2020-08-05 22:59:19+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-05 22:39:04+02	2020-08-05 23:32:34+02	graphics	blender	\N	blender-2.83.2
2020-08-05 22:59:21+02	2020-08-06 00:34:31+02	graphics	inkscape	\N	inkscape-1.0_3
2020-08-06 00:34:33+02	2020-08-06 00:34:56+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-08-05 23:32:39+02	2020-08-06 00:44:27+02	graphics	krita	\N	krita-4.3.0_2
2020-08-06 00:34:56+02	2020-08-06 00:49:15+02	graphics	wings	\N	wings-2.2.5
2020-08-06 00:44:29+02	2020-08-06 00:50:22+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-06 00:49:22+02	2020-08-06 00:56:01+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-08-06 00:56:06+02	2020-08-06 01:00:26+02	lang	csharp-mode.el	full	csharp-mode.el-emacs26-0.9.2_1
2020-08-06 00:50:26+02	2020-08-06 01:33:53+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-08-06 01:00:28+02	2020-08-06 01:52:12+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.5
2020-08-05 04:29:42+02	2020-08-06 01:52:53+02	www	chromium	\N	chromium-84.0.4147.105
2020-08-06 01:52:55+02	2020-08-06 01:53:58+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-06 01:53:59+02	2020-08-06 01:55:01+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-06 01:55:02+02	2020-08-06 01:55:10+02	lang	python	\N	python-3.7_3,2
2020-08-06 01:55:11+02	2020-08-06 01:55:29+02	mail	bbmail	\N	bbmail-0.9.3
2020-08-06 01:55:30+02	2020-08-06 02:01:10+02	math	maxima	\N	maxima-5.44.0_3
2020-08-06 02:01:10+02	2020-08-06 02:01:54+02	misc	gnu-watch	\N	gnu-watch-3.3.16
2020-08-06 02:01:54+02	2020-08-06 02:04:24+02	misc	mc	\N	mc-nox11-4.8.24
2020-08-06 02:04:28+02	2020-08-06 02:05:39+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-06 02:05:39+02	2020-08-06 02:05:56+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-08-06 02:05:57+02	2020-08-06 02:06:52+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-08-06 01:33:54+02	2020-08-06 02:08:59+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-06 02:09:04+02	2020-08-06 02:09:53+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-08-06 02:09:53+02	2020-08-06 02:10:12+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-08-06 02:10:12+02	2020-08-06 02:17:29+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-06 02:17:57+02	2020-08-06 02:20:45+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-06 02:20:48+02	2020-08-06 02:31:11+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-06 02:06:53+02	2020-08-06 02:31:52+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p3
2020-08-06 02:32:00+02	2020-08-06 02:32:18+02	net	py-netifaces	py27	py27-netifaces-0.10.9
2020-08-06 02:32:18+02	2020-08-06 02:32:59+02	net	rdesktop	\N	rdesktop-1.8.6
2020-08-06 02:32:59+02	2020-08-06 02:34:30+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-06 02:31:12+02	2020-08-06 02:35:39+02	net	ntopng	\N	ntopng-4.0.d20200615_1,1
2020-08-06 02:35:39+02	2020-08-06 02:36:35+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-06 01:52:15+02	2020-08-06 02:40:27+02	lang	mono	\N	mono-5.10.1.57_3
2020-08-06 02:40:28+02	2020-08-06 02:41:23+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-06 02:41:24+02	2020-08-06 02:42:16+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-06 02:42:17+02	2020-08-06 02:43:07+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-06 02:43:08+02	2020-08-06 02:44:10+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-06 02:44:10+02	2020-08-06 02:45:05+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-06 02:45:05+02	2020-08-06 02:45:59+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-06 02:46:00+02	2020-08-06 02:46:49+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-06 02:46:50+02	2020-08-06 02:48:07+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-06 02:48:08+02	2020-08-06 02:49:03+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-06 02:49:04+02	2020-08-06 02:50:02+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-06 02:50:03+02	2020-08-06 02:52:21+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-06 02:52:22+02	2020-08-06 02:53:11+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-06 02:34:30+02	2020-08-06 02:53:14+02	net	wireshark	\N	wireshark-3.2.5
2020-08-06 02:36:36+02	2020-08-06 02:53:15+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-08-06 02:53:15+02	2020-08-06 02:53:47+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-06 02:53:16+02	2020-08-06 02:53:52+02	print	gv	\N	gv-3.7.4_5
2020-08-06 02:53:12+02	2020-08-06 02:55:18+02	print	auctex	full	auctex-emacs26-12.2
2020-08-06 02:55:19+02	2020-08-06 03:00:32+02	print	texlive-full	\N	texlive-full-20150521
2020-08-06 03:00:33+02	2020-08-06 03:06:37+02	security	klee	\N	klee-2.1_1
2020-08-06 03:06:38+02	2020-08-06 03:06:56+02	security	nyx	\N	nyx-2.1.0_1
2020-08-06 03:06:57+02	2020-08-06 03:08:36+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-06 03:08:37+02	2020-08-06 03:10:27+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-08-06 03:10:27+02	2020-08-06 03:10:48+02	security	py-xkcdpass	py37	py37-xkcdpass-1.17.4
2020-08-06 03:10:49+02	2020-08-06 03:13:50+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-08-06 02:53:48+02	2020-08-06 03:14:43+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-06 03:13:51+02	2020-08-06 03:15:05+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-06 03:14:44+02	2020-08-06 03:15:13+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-08-06 03:15:05+02	2020-08-06 03:16:06+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-06 03:16:07+02	2020-08-06 03:16:29+02	sysutils	panicmail	\N	panicmail-1.1
2020-08-06 03:15:15+02	2020-08-06 03:16:32+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-06 03:16:32+02	2020-08-06 03:16:42+02	sysutils	py-py-cpuinfo	py27	py27-py-cpuinfo-7.0.0
2020-08-06 03:16:30+02	2020-08-06 03:16:46+02	sysutils	py-glances	py27	py27-glances-3.1.4
2020-08-06 03:16:43+02	2020-08-06 03:17:10+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-08-06 03:17:11+02	2020-08-06 03:18:27+02	www	links	\N	links-2.20.2,1
2020-08-06 03:18:28+02	2020-08-06 03:18:58+02	www	npm-node10	\N	npm-node10-6.12.1_1
2020-08-06 03:18:58+02	2020-08-06 03:19:13+02	www	py-bottle	py27	py27-bottle-0.12.15_1
2020-08-06 03:19:14+02	2020-08-06 03:19:30+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-08-06 03:19:31+02	2020-08-06 03:19:43+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-08-06 03:19:44+02	2020-08-06 03:20:01+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-08-06 03:20:01+02	2020-08-06 03:20:16+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-08-06 03:20:16+02	2020-08-06 03:20:31+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-08-06 03:20:31+02	2020-08-06 03:20:47+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-08-06 03:20:47+02	2020-08-06 03:21:06+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-08-06 03:21:06+02	2020-08-06 03:21:23+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-08-06 03:21:24+02	2020-08-06 03:21:38+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-08-06 03:21:39+02	2020-08-06 03:21:55+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-08-06 03:21:56+02	2020-08-06 03:22:19+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-08-06 03:22:19+02	2020-08-06 03:22:42+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-08-06 03:22:43+02	2020-08-06 03:22:56+02	www	py-requests	py27	py27-requests-2.22.0
2020-08-06 03:22:58+02	2020-08-06 03:25:40+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-06 03:25:41+02	2020-08-06 03:27:57+02	x11	mate	\N	mate-1.22.2
2020-08-06 03:27:58+02	2020-08-06 03:28:16+02	x11	numlockx	\N	numlockx-1.2
2020-08-06 03:28:16+02	2020-08-06 03:28:34+02	x11	sterm	\N	sterm-0.8.4
2020-08-06 03:28:37+02	2020-08-06 03:29:09+02	x11	xclip	\N	xclip-0.13
2020-08-06 03:29:09+02	2020-08-06 03:29:26+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-08-06 03:29:27+02	2020-08-06 03:30:10+02	x11	xdm	\N	xdm-1.1.12_3
2020-08-06 03:30:10+02	2020-08-06 03:32:20+02	x11	xlockmore	\N	xlockmore-5.65
2020-08-06 03:32:21+02	2020-08-06 03:32:40+02	x11	xman	\N	xman-1.1.5
2020-08-06 03:32:41+02	2020-08-06 03:40:58+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-06 02:53:52+02	2020-08-06 03:46:02+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-08-06 03:40:59+02	2020-08-06 03:46:39+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-06 03:46:40+02	2020-08-06 03:46:59+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-08-06 03:46:59+02	2020-08-06 03:47:14+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-08-06 03:46:02+02	2020-08-06 03:47:46+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-06 03:47:14+02	2020-08-06 03:48:05+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-08-06 03:47:47+02	2020-08-06 03:48:09+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-08-06 03:48:09+02	2020-08-06 03:48:29+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-08-06 03:48:05+02	2020-08-06 03:48:44+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-08-06 03:48:44+02	2020-08-06 03:49:48+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-06 03:49:49+02	2020-08-06 03:50:28+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-06 03:48:30+02	2020-08-06 04:24:02+02	editors	vscode	\N	vscode-1.46.1
2020-08-06 03:16:46+02	2020-08-06 05:14:15+02	www	firefox	\N	firefox-79.0_2,1
2020-08-06 05:14:16+02	2020-08-06 05:28:47+02	local	E590T-localbase	\N	E590T-localbase-2020072800
2020-08-06 06:50:10+02	2020-08-06 06:51:23+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-06 06:50:10+02	2020-08-06 06:57:12+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-06 06:57:13+02	2020-08-06 06:57:20+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-06 08:09:03+02	2020-08-06 08:09:14+02	www	p5-Mozilla-CA	\N	p5-Mozilla-CA-20200520
2020-08-06 08:09:14+02	2020-08-06 08:09:26+02	security	p5-IO-Socket-SSL	\N	p5-IO-Socket-SSL-2.068
2020-08-06 08:09:03+02	2020-08-06 08:09:28+02	www	py-django22	py37	py37-django22-2.2.15
2020-08-06 08:09:03+02	2020-08-06 08:09:54+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.2.0
2020-08-06 08:09:29+02	2020-08-06 08:13:00+02	www	apache24	\N	apache24-2.4.46
2020-08-06 08:09:55+02	2020-08-06 08:13:03+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-06 08:09:26+02	2020-08-06 08:13:08+02	devel	git	\N	git-2.28.0
2020-08-06 08:13:03+02	2020-08-06 08:13:16+02	net	p5-Net-HTTP	\N	p5-Net-HTTP-6.19
2020-08-06 08:13:01+02	2020-08-06 08:13:19+02	net	fping	\N	fping-4.4
2020-08-06 08:13:09+02	2020-08-06 08:13:54+02	audio	mate-media	\N	mate-media-1.24.0
2020-08-06 08:13:19+02	2020-08-06 08:13:58+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-08-06 08:13:59+02	2020-08-06 08:14:13+02	sysutils	py-psutil	py37	py37-psutil-5.7.2
2020-08-06 08:14:14+02	2020-08-06 08:14:31+02	www	p5-libwww	\N	p5-libwww-6.46
2020-08-06 08:14:31+02	2020-08-06 08:14:44+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-08-06 08:14:45+02	2020-08-06 08:14:58+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-08-06 08:14:59+02	2020-08-06 08:15:15+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-08-06 08:13:55+02	2020-08-06 08:15:26+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-06 08:13:16+02	2020-08-06 08:15:35+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-06 08:15:26+02	2020-08-06 08:16:26+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-06 08:15:15+02	2020-08-06 08:19:50+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-06 08:19:51+02	2020-08-06 08:20:06+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-08-06 08:20:07+02	2020-08-06 08:20:28+02	devel	tig	\N	tig-2.5.1
2020-08-06 08:20:29+02	2020-08-06 08:22:02+02	games	bsdgames	\N	bsdgames-0.75,2
2020-08-06 08:15:36+02	2020-08-06 08:35:06+02	x11-fonts	nerd-fonts	\N	nerd-fonts-2.1.0
2020-08-06 08:16:27+02	2020-08-06 08:38:24+02	devel	llvm10	\N	llvm10-10.0.1_1
2020-08-06 08:38:26+02	2020-08-06 08:39:54+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-08-06 08:39:54+02	2020-08-06 08:52:46+02	shells	starship	\N	starship-0.44.0
2020-08-06 08:52:47+02	2020-08-06 08:53:09+02	sysutils	py-glances	py37	py37-glances-3.1.4
2020-08-06 08:53:10+02	2020-08-06 08:53:22+02	sysutils	py-py-cpuinfo	py37	py37-py-cpuinfo-7.0.0
2020-08-06 08:53:23+02	2020-08-06 08:53:54+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_2
2020-08-06 08:53:54+02	2020-08-06 08:58:56+02	www	mod_php73	\N	mod_php73-7.3.20
2020-08-06 08:58:57+02	2020-08-06 08:59:51+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-08-06 08:59:52+02	2020-08-06 09:00:05+02	www	py-bottle	py37	py37-bottle-0.12.15_1
2020-08-06 08:35:06+02	2020-08-06 09:00:19+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p4
2020-08-06 09:00:06+02	2020-08-06 09:00:20+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-08-06 09:00:19+02	2020-08-06 09:00:33+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-08-06 09:00:20+02	2020-08-06 09:00:33+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-08-06 09:00:34+02	2020-08-06 09:00:47+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-08-06 09:00:34+02	2020-08-06 09:00:47+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-08-06 09:00:48+02	2020-08-06 09:01:03+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-08-06 09:00:47+02	2020-08-06 09:01:03+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-08-06 09:01:03+02	2020-08-06 09:01:16+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-08-06 09:01:04+02	2020-08-06 09:01:17+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-08-06 09:01:17+02	2020-08-06 09:01:34+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-08-06 09:01:18+02	2020-08-06 09:02:49+02	x11	mate	\N	mate-1.22.2
2020-08-06 09:02:49+02	2020-08-06 09:03:23+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-06 09:01:35+02	2020-08-06 09:03:23+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-06 08:22:03+02	2020-08-06 10:09:16+02	lang	gcc10-devel	\N	gcc10-devel-10.2.1.s20200801
2020-08-06 10:09:17+02	2020-08-06 10:24:45+02	local	E590T-localbase	\N	E590T-localbase-2020080600
2020-08-06 10:24:59+02	2020-08-06 10:25:50+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-06 10:24:59+02	2020-08-06 10:26:30+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-06 10:26:30+02	2020-08-06 10:26:37+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-06 22:50:57+02	2020-08-06 22:58:30+02	lang	go	\N	go-1.14.7,1
2020-08-06 22:50:57+02	2020-08-07 00:46:22+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200802
2020-08-07 00:46:23+02	2020-08-07 01:01:57+02	local	E590T-localbase	\N	E590T-localbase-2020080601
2020-08-07 01:02:09+02	2020-08-07 01:03:01+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-07 01:02:09+02	2020-08-07 01:03:38+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-07 01:03:38+02	2020-08-07 01:03:45+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-07 07:37:06+02	2020-08-07 07:40:13+02	editors	vim	\N	vim-8.2.1382
2020-08-07 07:37:06+02	2020-08-07 07:43:06+02	shells	starship	\N	starship-0.44.0_1
2020-08-07 07:43:07+02	2020-08-07 07:58:07+02	local	E590T-localbase	\N	E590T-localbase-2020080601
2020-08-07 07:58:21+02	2020-08-07 07:59:13+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-07 07:58:21+02	2020-08-07 07:59:51+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-07 07:59:51+02	2020-08-07 07:59:58+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-08 13:57:16+02	2020-08-08 13:57:43+02	net	rsync	\N	rsync-3.2.2_1
2020-08-08 13:57:16+02	2020-08-08 13:58:52+02	net	ndpi	\N	ndpi-3.2.d20200721,1
2020-08-08 13:57:43+02	2020-08-08 14:00:39+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-08 13:57:16+02	2020-08-08 14:02:48+02	devel	gsoap	\N	gsoap-2.8.105
2020-08-08 13:58:52+02	2020-08-08 14:03:11+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-08 14:03:12+02	2020-08-08 14:20:31+02	local	E590T-localbase	\N	E590T-localbase-2020080800
2020-08-08 14:20:45+02	2020-08-08 14:21:44+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-08 14:20:45+02	2020-08-08 14:26:02+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-08 14:26:02+02	2020-08-08 14:26:09+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-09 11:58:54+02	2020-08-09 11:59:09+02	devel	py-cffi	py27	py27-cffi-1.14.1
2020-08-09 11:58:54+02	2020-08-09 11:59:11+02	devel	py-cffi	py37	py37-cffi-1.14.1
2020-08-09 11:58:54+02	2020-08-09 11:59:21+02	security	libssh2	\N	libssh2-1.9.0_3,3
2020-08-09 11:59:12+02	2020-08-09 12:00:05+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-08-09 11:59:10+02	2020-08-09 12:00:09+02	sysutils	bsdisks	\N	bsdisks-0.19
2020-08-09 12:00:05+02	2020-08-09 12:00:16+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-08-09 11:59:21+02	2020-08-09 12:00:17+02	devel	libgit2	\N	libgit2-1.0.1
2020-08-09 12:00:17+02	2020-08-09 12:00:26+02	net	py-urllib3	py37	py37-urllib3-1.25.7,1
2020-08-09 12:00:26+02	2020-08-09 12:00:36+02	www	py-requests	py37	py37-requests-2.22.0
2020-08-09 12:00:36+02	2020-08-09 12:01:08+02	textproc	py-sphinx	py37	py37-sphinx-3.1.2,1
2020-08-09 12:01:09+02	2020-08-09 12:01:27+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-08-09 12:00:10+02	2020-08-09 12:01:33+02	devel	kf5-solid	\N	kf5-solid-5.72.0
2020-08-09 12:01:27+02	2020-08-09 12:01:37+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-08-09 12:01:38+02	2020-08-09 12:02:21+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-08-09 12:02:30+02	2020-08-09 12:02:46+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-08-09 12:02:46+02	2020-08-09 12:03:01+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-08-09 12:03:01+02	2020-08-09 12:03:16+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-08-09 12:03:16+02	2020-08-09 12:03:31+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-08-09 12:01:34+02	2020-08-09 12:06:19+02	devel	kf5-kio	\N	kf5-kio-5.72.0
2020-08-09 12:06:20+02	2020-08-09 12:35:24+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-08-09 12:35:26+02	2020-08-09 12:35:42+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-08-09 12:35:43+02	2020-08-09 12:36:00+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-08-09 12:36:00+02	2020-08-09 12:36:14+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-08-09 12:36:15+02	2020-08-09 12:37:15+02	devel	bzr	py27	bzr-2.7.0.6622
2020-08-09 12:37:16+02	2020-08-09 12:39:41+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-08-09 12:39:42+02	2020-08-09 13:00:04+02	emulators	wine	\N	wine-5.0.2,1
2020-08-09 13:00:05+02	2020-08-09 13:12:34+02	graphics	krita	\N	krita-4.3.0_2
2020-08-09 13:12:35+02	2020-08-09 13:13:59+02	misc	mc	\N	mc-nox11-4.8.24
2020-08-09 13:13:59+02	2020-08-09 13:16:44+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-09 13:16:45+02	2020-08-09 13:19:31+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-08-09 13:19:32+02	2020-08-09 13:19:49+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-08-09 13:19:51+02	2020-08-09 13:20:13+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-08-09 13:20:13+02	2020-08-09 13:20:52+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-09 12:03:31+02	2020-08-09 13:37:20+02	lang	julia11	\N	julia11-1.1.1_5
2020-08-09 13:37:20+02	2020-08-09 13:37:50+02	lang	julia	\N	julia-1.0.3_3
2020-08-09 12:00:18+02	2020-08-09 15:53:33+02	lang	rust	\N	rust-1.45.2
2020-08-09 15:53:35+02	2020-08-09 15:57:54+02	sysutils	exa	\N	exa-0.9.0_14
2020-08-09 15:53:35+02	2020-08-09 16:01:51+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3_3
2020-08-09 15:53:35+02	2020-08-09 16:06:03+02	shells	starship	\N	starship-0.44.0_1
2020-08-09 15:57:54+02	2020-08-09 16:08:41+02	textproc	bat	\N	bat-0.15.4_3
2020-08-09 16:49:02+02	2020-08-09 16:49:09+02	sysutils	smart	\N	smart-0.0.11
2020-08-09 16:49:02+02	2020-08-09 16:49:42+02	net	liveMedia	\N	liveMedia-2020.08.09,2
2020-08-09 16:49:09+02	2020-08-09 16:49:43+02	sysutils	tarsnap	\N	tarsnap-1.0.39
2020-08-09 16:49:43+02	2020-08-09 16:54:31+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-09 16:49:43+02	2020-08-09 16:54:55+02	shells	fish	\N	fish-3.1.2
2020-08-09 16:54:56+02	2020-08-09 16:55:05+02	sysutils	bashtop	\N	bashtop-0.9.25_1
2020-08-09 16:55:05+02	2020-08-09 16:55:14+02	sysutils	bpytop	\N	bpytop-1.0.7
2020-08-09 16:55:14+02	2020-08-09 16:55:25+02	sysutils	flashrom	\N	flashrom-1.2
2020-08-09 16:55:26+02	2020-08-09 16:55:32+02	sysutils	procmap	\N	procmap-1.0
2020-08-09 16:55:33+02	2020-08-09 16:55:43+02	sysutils	ssd_report	smart	ssd_report-0.4
2020-08-09 16:54:31+02	2020-08-09 16:56:00+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-09 16:56:01+02	2020-08-09 16:56:07+02	sysutils	tarsnap-periodic	\N	tarsnap-periodic-1.0.20170413
2020-08-09 16:55:43+02	2020-08-09 16:57:29+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-08-09 16:57:30+02	2020-08-09 17:15:15+02	local	E590T-localbase	\N	E590T-localbase-2020080900
2020-08-09 17:15:40+02	2020-08-09 17:16:48+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-09 17:15:40+02	2020-08-09 17:20:54+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-09 17:20:55+02	2020-08-09 17:21:02+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-09 18:54:44+02	2020-08-09 18:55:26+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.73.0
2020-08-09 18:55:27+02	2020-08-09 18:56:12+02	archivers	kf5-karchive	\N	kf5-karchive-5.73.0
2020-08-09 18:55:27+02	2020-08-09 18:56:46+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.73.0
2020-08-09 18:55:27+02	2020-08-09 18:56:56+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-08-09 18:56:12+02	2020-08-09 18:57:31+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-08-09 18:56:47+02	2020-08-09 18:58:15+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-08-09 18:57:31+02	2020-08-09 18:58:59+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-08-09 18:58:15+02	2020-08-09 18:59:14+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-08-09 18:56:57+02	2020-08-09 18:59:35+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-08-09 18:58:59+02	2020-08-09 19:00:03+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-08-09 18:59:15+02	2020-08-09 19:00:08+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.73.0
2020-08-09 18:59:36+02	2020-08-09 19:00:49+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-08-09 19:00:09+02	2020-08-09 19:01:06+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-08-09 19:00:04+02	2020-08-09 19:01:55+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-08-09 19:01:07+02	2020-08-09 19:02:30+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-08-09 19:00:49+02	2020-08-09 19:02:35+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.73.0
2020-08-09 19:01:56+02	2020-08-09 19:02:58+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-08-09 19:02:35+02	2020-08-09 19:03:38+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-08-09 19:02:30+02	2020-08-09 19:03:54+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-08-09 19:02:59+02	2020-08-09 19:04:09+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-08-09 19:03:55+02	2020-08-09 19:05:34+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-08-09 19:03:38+02	2020-08-09 19:05:43+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-09 19:04:09+02	2020-08-09 19:06:12+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-08-09 19:05:35+02	2020-08-09 19:06:47+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-08-09 19:05:44+02	2020-08-09 19:07:23+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-09 19:06:13+02	2020-08-09 19:08:28+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-08-09 19:06:47+02	2020-08-09 19:09:18+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-09 19:07:23+02	2020-08-09 19:09:29+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-09 19:08:29+02	2020-08-09 19:09:38+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-08-09 19:09:30+02	2020-08-09 19:10:34+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-08-09 19:09:19+02	2020-08-09 19:10:42+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-09 19:09:38+02	2020-08-09 19:10:48+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-08-09 19:10:42+02	2020-08-09 19:17:21+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-09 19:17:21+02	2020-08-09 19:40:53+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-08-09 19:40:54+02	2020-08-09 19:43:09+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-08-09 19:17:21+02	2020-08-09 19:51:15+02	graphics	krita	\N	krita-4.3.0_2
2020-08-09 19:51:16+02	2020-08-09 20:07:09+02	local	E590T-localbase	\N	E590T-localbase-2020080900
2020-08-09 20:07:23+02	2020-08-09 20:08:15+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-24 04:43:55+02	2020-08-24 04:44:08+02	dns	sshfp	\N	sshfp-1.2.2_5
2020-08-09 20:07:23+02	2020-08-09 20:08:54+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-09 20:08:55+02	2020-08-09 20:09:02+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-13 08:04:22+02	2020-08-13 08:04:43+02	x11-fonts	font-misc-ethiopic	\N	font-misc-ethiopic-1.0.4
2020-08-13 08:04:22+02	2020-08-13 08:04:46+02	devel	libgudev	\N	libgudev-233
2020-08-13 08:04:47+02	2020-08-13 08:05:01+02	x11	libwacom	\N	libwacom-1.4.1
2020-08-13 08:04:45+02	2020-08-13 08:05:02+02	x11-fonts	xorg-fonts-truetype	\N	xorg-fonts-truetype-7.7_1
2020-08-13 08:05:02+02	2020-08-13 08:05:12+02	x11-fonts	font-alias	\N	font-alias-1.0.4
2020-08-13 08:05:01+02	2020-08-13 08:05:27+02	x11	libinput	\N	libinput-1.15.6
2020-08-13 08:04:22+02	2020-08-13 08:06:26+02	x11	libX11	\N	libX11-1.6.11,1
2020-08-13 08:06:27+02	2020-08-13 08:06:56+02	x11	libXext	\N	libXext-1.3.4,1
2020-08-13 08:06:58+02	2020-08-13 08:07:20+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-08-13 08:07:20+02	2020-08-13 08:07:32+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-08-13 08:07:33+02	2020-08-13 08:07:46+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-08-13 08:07:46+02	2020-08-13 08:07:58+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-08-13 08:07:59+02	2020-08-13 08:08:10+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-08-13 08:05:27+02	2020-08-13 08:10:23+02	lang	go	\N	go-1.15,1
2020-08-13 08:10:24+02	2020-08-13 08:11:09+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-08-13 08:11:10+02	2020-08-13 08:12:27+02	devel	dbus	\N	dbus-1.12.20
2020-08-13 08:12:27+02	2020-08-13 08:12:55+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-08-13 08:12:55+02	2020-08-13 08:13:24+02	x11	libXi	\N	libXi-1.7.10,1
2020-08-13 08:08:10+02	2020-08-13 08:13:25+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-08-13 08:13:24+02	2020-08-13 08:13:51+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-08-13 08:13:52+02	2020-08-13 08:15:09+02	net	avahi-app	\N	avahi-app-0.7_3
2020-08-13 08:15:10+02	2020-08-13 08:15:30+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-08-13 08:13:25+02	2020-08-13 08:15:42+02	graphics	cairo	\N	cairo-1.16.0,2
2020-08-13 08:15:43+02	2020-08-13 08:16:01+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-08-13 08:16:02+02	2020-08-13 08:16:22+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-08-13 08:16:23+02	2020-08-13 08:16:37+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-08-13 08:15:30+02	2020-08-13 08:16:45+02	print	cups	\N	cups-2.3.3_1
2020-08-13 08:16:38+02	2020-08-13 08:16:58+02	x11	libXpm	\N	libXpm-3.5.13
2020-08-13 08:16:59+02	2020-08-13 08:17:47+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-08-13 08:17:48+02	2020-08-13 08:18:13+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-08-13 08:16:45+02	2020-08-13 08:18:55+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-13 08:18:56+02	2020-08-13 08:20:07+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-08-13 08:20:07+02	2020-08-13 08:20:22+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-08-13 08:20:23+02	2020-08-13 08:20:40+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-08-13 08:20:41+02	2020-08-13 08:20:56+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-08-13 08:20:56+02	2020-08-13 08:21:23+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-08-13 08:21:24+02	2020-08-13 08:21:36+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-08-13 08:18:14+02	2020-08-13 08:21:41+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_6
2020-08-13 08:21:36+02	2020-08-13 08:22:26+02	sysutils	polkit	\N	polkit-0.116
2020-08-13 08:22:26+02	2020-08-13 08:23:18+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-08-13 08:23:18+02	2020-08-13 08:24:03+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-08-13 08:24:07+02	2020-08-13 08:24:20+02	x11	xprop	\N	xprop-1.2.4
2020-08-13 08:24:25+02	2020-08-13 08:24:38+02	x11	xset	\N	xset-1.2.4_3
2020-08-13 08:21:42+02	2020-08-13 08:26:17+02	graphics	graphviz	\N	graphviz-2.44.1
2020-08-13 08:24:40+02	2020-08-13 08:28:08+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-08-13 08:05:13+02	2020-08-13 08:28:29+02	lang	erlang	\N	erlang-21.3.8.17,4
2020-08-13 08:28:15+02	2020-08-13 08:28:34+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-08-13 08:28:34+02	2020-08-13 08:29:11+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-08-13 08:26:18+02	2020-08-13 08:29:44+02	lang	vala	\N	vala-0.40.15,1
2020-08-13 08:28:30+02	2020-08-13 08:30:30+02	graphics	colord	\N	colord-1.3.5
2020-08-13 08:29:11+02	2020-08-13 08:30:31+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-13 08:29:48+02	2020-08-13 08:30:41+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-13 08:30:32+02	2020-08-13 08:31:06+02	graphics	libGLU	\N	libGLU-9.0.1
2020-08-13 08:30:30+02	2020-08-13 08:33:02+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-08-13 08:31:07+02	2020-08-13 08:40:57+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-13 08:30:42+02	2020-08-13 08:43:58+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-13 08:40:57+02	2020-08-13 08:44:03+02	graphics	poppler	\N	poppler-0.90.1
2020-08-13 08:43:59+02	2020-08-13 08:44:28+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-08-13 08:44:03+02	2020-08-13 08:45:01+02	devel	dconf	\N	dconf-0.36.0
2020-08-13 08:44:28+02	2020-08-13 08:46:16+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-13 08:46:17+02	2020-08-13 08:47:15+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-08-13 08:47:15+02	2020-08-13 08:47:36+02	print	xpdfopen	\N	xpdfopen-0.86
2020-08-13 08:45:02+02	2020-08-13 08:47:36+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-08-13 08:47:36+02	2020-08-13 08:47:51+02	multimedia	libva	\N	libva-2.8.0
2020-08-13 08:47:37+02	2020-08-13 08:49:56+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-08-13 08:49:56+02	2020-08-13 08:50:24+02	devel	libnotify	\N	libnotify-0.7.8
2020-08-13 08:50:25+02	2020-08-13 08:50:46+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-08-13 08:50:47+02	2020-08-13 08:51:08+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-08-13 08:51:09+02	2020-08-13 08:53:00+02	graphics	frei0r	\N	frei0r-1.6.1
2020-08-13 08:33:03+02	2020-08-13 08:54:21+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-13 08:47:52+02	2020-08-13 08:55:50+02	print	texlive-base	\N	texlive-base-20150521_53
2020-08-13 08:54:22+02	2020-08-13 09:05:12+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-13 08:53:00+02	2020-08-13 09:12:16+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-13 09:12:17+02	2020-08-13 09:14:21+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-13 09:14:21+02	2020-08-13 09:15:45+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-13 09:15:46+02	2020-08-13 09:16:14+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-08-13 09:16:14+02	2020-08-13 09:18:27+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-13 09:18:28+02	2020-08-13 09:19:05+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-08-13 08:55:51+02	2020-08-13 09:19:21+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-13 09:19:05+02	2020-08-13 09:21:04+02	graphics	graphene	\N	graphene-1.10.0
2020-08-13 09:19:22+02	2020-08-13 09:22:28+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-13 09:22:29+02	2020-08-13 09:22:47+02	www	libwpe	\N	libwpe-1.4.0.1
2020-08-13 09:22:48+02	2020-08-13 09:25:15+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-08-13 09:25:16+02	2020-08-13 09:26:30+02	net	geoclue	\N	geoclue-2.5.5
2020-08-13 09:26:31+02	2020-08-13 09:26:56+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-08-13 09:21:05+02	2020-08-13 09:27:35+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-13 09:26:56+02	2020-08-13 09:27:41+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-13 09:27:41+02	2020-08-13 09:28:18+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-13 09:05:13+02	2020-08-13 09:32:24+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-13 09:32:25+02	2020-08-13 09:33:28+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-08-13 09:33:28+02	2020-08-13 09:34:11+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-08-13 09:34:11+02	2020-08-13 09:34:46+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-08-13 09:34:47+02	2020-08-13 09:36:56+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-13 09:36:56+02	2020-08-13 09:37:13+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-08-13 09:37:14+02	2020-08-13 09:38:37+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-08-13 09:38:37+02	2020-08-13 09:39:46+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-08-13 09:39:46+02	2020-08-13 09:40:02+02	x11	libXres	\N	libXres-1.2.0_2
2020-08-13 09:40:02+02	2020-08-13 09:40:20+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-08-13 09:40:20+02	2020-08-13 09:41:44+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-13 09:28:19+02	2020-08-13 09:45:33+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-13 09:45:34+02	2020-08-13 09:45:59+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-08-13 09:46:00+02	2020-08-13 09:55:08+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-13 09:55:09+02	2020-08-13 09:56:15+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-13 09:56:16+02	2020-08-13 09:57:51+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-13 09:41:44+02	2020-08-13 09:58:06+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-08-13 09:57:51+02	2020-08-13 09:58:55+02	graphics	pecl-imagick	php73	php73-pecl-imagick-3.4.4
2020-08-13 09:58:06+02	2020-08-13 09:59:02+02	sysutils	policykit	\N	policykit-0.9_10
2020-08-13 09:59:03+02	2020-08-13 09:59:54+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-13 09:58:56+02	2020-08-13 10:00:32+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-13 09:59:55+02	2020-08-13 10:00:55+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-08-13 10:00:33+02	2020-08-13 10:01:53+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.1
2020-08-13 10:00:56+02	2020-08-13 10:02:23+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-08-13 10:02:24+02	2020-08-13 10:03:12+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-08-13 10:03:13+02	2020-08-13 10:04:44+02	sysutils	hal	\N	hal-0.5.14_34
2020-08-13 10:04:45+02	2020-08-13 10:06:27+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-13 10:01:53+02	2020-08-13 10:07:20+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-13 10:07:21+02	2020-08-13 10:09:32+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-13 10:09:33+02	2020-08-13 10:10:56+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-08-13 10:06:28+02	2020-08-13 10:11:26+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_3,1
2020-08-13 10:10:57+02	2020-08-13 10:20:44+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-13 10:20:45+02	2020-08-13 10:24:35+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-08-13 10:24:35+02	2020-08-13 10:25:40+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-08-13 10:25:41+02	2020-08-13 10:26:58+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-13 10:11:27+02	2020-08-13 10:28:22+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-13 10:28:23+02	2020-08-13 10:28:44+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-08-13 10:28:45+02	2020-08-13 10:30:38+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-08-13 10:26:58+02	2020-08-13 10:30:51+02	graphics	poppler-glib	\N	poppler-glib-0.90.1
2020-08-13 10:30:52+02	2020-08-13 10:31:44+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-08-13 10:31:45+02	2020-08-13 10:32:46+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-08-13 10:30:40+02	2020-08-13 10:33:16+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-08-13 10:33:17+02	2020-08-13 10:34:42+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-13 10:32:47+02	2020-08-13 10:36:15+02	graphics	glew	\N	glew-2.2.0
2020-08-13 10:36:15+02	2020-08-13 10:37:24+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-08-13 10:37:25+02	2020-08-13 10:38:48+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-13 10:38:49+02	2020-08-13 10:40:22+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-13 10:40:23+02	2020-08-13 10:44:35+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-13 10:44:36+02	2020-08-13 10:46:01+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-08-13 10:46:02+02	2020-08-13 10:47:50+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-08-13 10:47:51+02	2020-08-13 10:51:11+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-13 10:51:12+02	2020-08-13 10:53:01+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-08-13 10:53:04+02	2020-08-13 10:53:29+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-08-13 10:53:29+02	2020-08-13 10:55:21+02	net	remmina	\N	remmina-1.3.9
2020-08-13 10:34:43+02	2020-08-13 10:58:12+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-08-13 10:55:21+02	2020-08-13 10:58:17+02	security	gcr	\N	gcr-3.28.0
2020-08-13 10:58:13+02	2020-08-13 10:58:32+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-08-13 10:58:18+02	2020-08-13 10:58:34+02	x11	xrdb	\N	xrdb-1.2.0
2020-08-13 10:58:32+02	2020-08-13 10:59:23+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-08-13 10:59:23+02	2020-08-13 11:00:20+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-13 11:00:21+02	2020-08-13 11:01:30+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-13 11:01:30+02	2020-08-13 11:12:49+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-13 11:12:50+02	2020-08-13 11:13:10+02	x11	luit	\N	luit-20190106_1
2020-08-13 11:13:11+02	2020-08-13 11:14:20+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-13 10:58:35+02	2020-08-13 11:14:24+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-13 11:14:27+02	2020-08-13 11:15:17+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-08-13 11:14:21+02	2020-08-13 11:15:30+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-08-13 11:15:17+02	2020-08-13 11:16:55+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-13 11:15:30+02	2020-08-13 11:44:34+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-13 11:44:35+02	2020-08-13 11:45:21+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-13 11:45:22+02	2020-08-13 11:48:40+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-13 11:48:41+02	2020-08-13 11:49:11+02	print	gl2ps	\N	gl2ps-1.4.0
2020-08-13 11:49:11+02	2020-08-13 11:49:31+02	x11	xauth	\N	xauth-1.1
2020-08-13 11:49:32+02	2020-08-13 11:50:31+02	x11	xterm	\N	xterm-358
2020-08-13 11:50:31+02	2020-08-13 11:51:56+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-08-13 11:51:59+02	2020-08-13 11:54:53+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-13 11:54:53+02	2020-08-13 11:57:14+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-13 11:57:14+02	2020-08-13 11:57:35+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.8_6
2020-08-13 09:27:35+02	2020-08-13 11:58:35+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-13 11:57:35+02	2020-08-13 12:03:05+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-08-13 12:03:05+02	2020-08-13 12:08:21+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-13 12:08:22+02	2020-08-13 12:09:18+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-08-13 12:09:18+02	2020-08-13 12:10:31+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-08-13 12:10:31+02	2020-08-13 12:23:39+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-13 11:58:38+02	2020-08-13 12:39:09+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-13 12:39:10+02	2020-08-13 12:42:46+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-13 12:42:46+02	2020-08-13 12:43:05+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-08-13 12:43:06+02	2020-08-13 12:44:48+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-13 12:44:49+02	2020-08-13 12:45:53+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-08-13 12:45:54+02	2020-08-13 12:56:59+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-13 12:56:59+02	2020-08-13 12:57:38+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-08-13 12:57:39+02	2020-08-13 12:58:52+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-13 11:16:55+02	2020-08-13 12:59:28+02	math	vtk8	\N	vtk8-8.2.0
2020-08-13 12:58:53+02	2020-08-13 12:59:43+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-13 12:59:43+02	2020-08-13 13:00:41+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-13 13:00:41+02	2020-08-13 13:04:51+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-08-13 13:04:53+02	2020-08-13 13:06:04+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-13 13:06:04+02	2020-08-13 13:07:17+02	sysutils	upower	\N	upower-0.99.11
2020-08-13 12:59:30+02	2020-08-13 15:44:06+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-13 15:44:11+02	2020-08-13 15:44:32+02	x11	appres	\N	appres-1.0.5
2020-08-13 15:44:33+02	2020-08-13 15:44:55+02	x11	bitmap	\N	bitmap-1.0.9
2020-08-28 02:07:43+02	2020-08-28 02:09:41+02	x11	mate	\N	mate-1.22.2
2020-08-13 15:44:56+02	2020-08-13 15:46:24+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-08-13 15:46:24+02	2020-08-13 15:47:05+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-08-13 15:47:05+02	2020-08-13 15:51:16+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-13 15:51:17+02	2020-08-13 15:51:35+02	x11	sessreg	\N	sessreg-1.1.2
2020-08-13 15:51:36+02	2020-08-13 15:51:54+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-08-13 15:51:55+02	2020-08-13 15:52:15+02	x11	smproxy	\N	smproxy-1.0.6
2020-08-13 15:52:15+02	2020-08-13 15:52:40+02	x11	x11perf	\N	x11perf-1.6.1
2020-08-13 15:52:41+02	2020-08-13 15:53:02+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-08-13 15:53:03+02	2020-08-13 15:53:32+02	x11	xcalc	\N	xcalc-1.1.0
2020-08-13 15:53:32+02	2020-08-13 15:53:56+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-08-13 15:53:56+02	2020-08-13 15:54:13+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-08-13 15:54:14+02	2020-08-13 15:54:33+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-08-13 15:54:34+02	2020-08-13 15:54:52+02	x11	xev	\N	xev-1.2.4
2020-08-13 15:54:53+02	2020-08-13 15:55:12+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-08-13 15:55:12+02	2020-08-13 15:55:30+02	x11	xgamma	\N	xgamma-1.0.6
2020-08-13 15:55:31+02	2020-08-13 15:55:51+02	x11	xgc	\N	xgc-1.0.5
2020-08-13 15:55:52+02	2020-08-13 15:56:13+02	x11	xhost	\N	xhost-1.0.8
2020-08-13 15:56:13+02	2020-08-13 15:56:33+02	x11	xinit	\N	xinit-1.4.1,1
2020-08-13 15:56:34+02	2020-08-13 15:56:55+02	x11	xinput	\N	xinput-1.6.3
2020-08-13 15:56:55+02	2020-08-13 15:57:15+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-08-13 15:57:16+02	2020-08-13 15:57:39+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-08-13 15:57:40+02	2020-08-13 15:57:59+02	x11	xkill	\N	xkill-1.0.5
2020-08-13 15:58:00+02	2020-08-13 15:58:17+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-08-13 15:58:18+02	2020-08-13 15:58:36+02	x11	xmessage	\N	xmessage-1.0.5
2020-08-13 15:58:37+02	2020-08-13 15:58:58+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-08-13 15:58:58+02	2020-08-13 15:59:16+02	x11	xpr	\N	xpr-1.0.5
2020-08-13 15:59:17+02	2020-08-13 15:59:39+02	x11	xrandr	\N	xrandr-1.5.1
2020-08-13 15:59:39+02	2020-08-13 15:59:57+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-08-13 15:59:58+02	2020-08-13 16:00:18+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-08-13 16:00:19+02	2020-08-13 16:00:35+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-08-13 16:00:36+02	2020-08-13 16:00:56+02	x11	xwd	\N	xwd-1.0.7
2020-08-13 16:00:57+02	2020-08-13 16:01:15+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-08-13 16:01:16+02	2020-08-13 16:01:36+02	x11	xwud	\N	xwud-1.0.5
2020-08-13 16:01:37+02	2020-08-13 16:02:00+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-08-13 16:02:02+02	2020-08-13 16:03:38+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-13 16:03:39+02	2020-08-13 16:04:55+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-13 16:04:56+02	2020-08-13 16:06:18+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-13 16:06:19+02	2020-08-13 16:07:38+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-13 16:07:39+02	2020-08-13 16:09:12+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-13 16:09:13+02	2020-08-13 16:12:55+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-08-13 16:12:56+02	2020-08-13 16:14:10+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-13 16:14:10+02	2020-08-13 16:15:28+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-13 16:15:29+02	2020-08-13 16:16:40+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-08-13 16:16:40+02	2020-08-13 16:18:10+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-08-13 16:18:11+02	2020-08-13 16:18:55+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-13 12:23:40+02	2020-08-13 16:41:13+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-13 16:41:23+02	2020-08-13 16:43:05+02	x11-wm	twm	\N	twm-1.0.11
2020-08-13 16:43:07+02	2020-08-13 16:48:50+02	audio	libmatemixer	\N	libmatemixer-1.24.0
2020-08-13 16:48:53+02	2020-08-13 16:53:16+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-13 16:53:20+02	2020-08-13 16:54:20+02	devel	librest	\N	rest-0.7.93
2020-08-13 16:18:56+02	2020-08-13 16:54:50+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-13 16:54:21+02	2020-08-13 16:57:43+02	graphics	poppler-utils	\N	poppler-utils-0.90.1
2020-08-13 16:57:44+02	2020-08-13 17:37:53+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-13 17:37:54+02	2020-08-13 17:38:18+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-08-13 17:38:22+02	2020-08-13 17:38:53+02	print	libotf	\N	libotf-0.9.16
2020-08-13 17:38:53+02	2020-08-13 17:39:57+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-13 17:39:58+02	2020-08-13 17:41:08+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-13 17:41:08+02	2020-08-13 17:42:38+02	sysutils	bsdisks	\N	bsdisks-0.20
2020-08-13 17:42:38+02	2020-08-13 17:46:12+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-08-13 17:46:13+02	2020-08-13 17:46:37+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-08-13 17:46:38+02	2020-08-13 17:48:30+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-13 17:48:30+02	2020-08-13 17:49:12+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-08-13 17:49:12+02	2020-08-13 17:49:29+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-08-13 17:49:29+02	2020-08-13 17:50:50+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-13 17:50:51+02	2020-08-13 17:56:18+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-13 17:56:23+02	2020-08-13 17:58:51+02	x11-fonts	xorg-fonts	\N	xorg-fonts-7.7_1
2020-08-13 17:58:52+02	2020-08-13 18:04:10+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_3,1
2020-08-13 18:04:11+02	2020-08-13 18:19:55+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-13 18:19:56+02	2020-08-13 18:21:51+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-13 18:21:52+02	2020-08-13 18:23:17+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-13 18:23:18+02	2020-08-13 18:25:09+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-13 18:25:13+02	2020-08-13 18:27:17+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-13 18:27:17+02	2020-08-13 18:28:54+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-13 18:28:54+02	2020-08-13 18:30:22+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-08-13 18:30:23+02	2020-08-13 18:33:14+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-08-13 18:33:15+02	2020-08-13 18:45:19+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-13 18:45:20+02	2020-08-13 18:50:21+02	graphics	gegl	\N	gegl-0.4.24_2
2020-08-13 18:50:21+02	2020-08-13 18:51:06+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-08-13 18:51:06+02	2020-08-13 18:51:30+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-08-13 16:54:51+02	2020-08-13 18:56:07+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-13 18:56:08+02	2020-08-13 19:00:01+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-13 19:00:02+02	2020-08-13 19:00:23+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-08-13 19:00:23+02	2020-08-13 19:01:47+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-13 19:01:48+02	2020-08-13 19:03:08+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-13 18:51:31+02	2020-08-13 19:05:35+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-13 19:03:08+02	2020-08-13 19:05:57+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-13 19:05:36+02	2020-08-13 19:06:29+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-08-13 19:05:57+02	2020-08-13 19:07:32+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-13 19:06:29+02	2020-08-13 19:07:39+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-13 19:07:33+02	2020-08-13 19:08:32+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-08-13 19:08:32+02	2020-08-13 19:09:44+02	x11	zenity	\N	zenity-3.32.0
2020-08-13 19:07:39+02	2020-08-13 19:10:42+02	x11	xorg	\N	xorg-7.7_3
2020-08-13 19:09:45+02	2020-08-13 19:10:43+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-08-13 19:10:43+02	2020-08-13 19:11:30+02	devel	apache-ant	\N	apache-ant-1.10.7
2020-08-13 19:10:44+02	2020-08-13 19:15:10+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-13 19:11:30+02	2020-08-13 19:15:40+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-13 19:15:41+02	2020-08-13 19:16:25+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-08-13 19:15:11+02	2020-08-13 19:16:36+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-13 19:16:26+02	2020-08-13 19:19:22+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-13 19:16:37+02	2020-08-13 19:19:57+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-13 19:19:22+02	2020-08-13 19:22:03+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-13 19:19:58+02	2020-08-13 19:29:24+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-13 19:22:04+02	2020-08-13 19:30:29+02	editors	emacs	full	emacs-27.1,3
2020-08-13 19:30:29+02	2020-08-13 19:30:52+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-08-13 19:29:25+02	2020-08-13 19:42:24+02	graphics	Coin	\N	Coin-4.0.0_1
2020-08-13 19:42:25+02	2020-08-13 19:44:06+02	graphics	glfw	\N	glfw-3.3.2
2020-08-13 19:44:10+02	2020-08-13 19:48:30+02	graphics	movit	\N	movit-1.6.2_2
2020-08-13 19:30:53+02	2020-08-13 19:53:02+02	graphics	gimp-app	\N	gimp-app-2.10.20_2,1
2020-08-13 19:53:03+02	2020-08-13 19:54:48+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-08-13 19:54:48+02	2020-08-13 19:55:42+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-08-13 19:55:43+02	2020-08-13 19:56:19+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-08-13 19:56:19+02	2020-08-13 19:59:22+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-13 19:59:23+02	2020-08-13 20:07:09+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-08-13 20:07:10+02	2020-08-13 20:07:43+02	net	gssdp	\N	gssdp-1.2.3
2020-08-13 20:07:43+02	2020-08-13 20:09:06+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-08-13 20:09:07+02	2020-08-13 20:14:38+02	print	cups-filters	\N	cups-filters-1.27.5_1
2020-08-13 20:14:40+02	2020-08-13 20:41:58+02	print	fontforge	\N	fontforge-20190801_2
2020-08-13 19:48:31+02	2020-08-13 20:45:35+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-13 20:45:39+02	2020-08-13 20:49:30+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-13 20:49:33+02	2020-08-13 20:58:19+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-08-13 20:42:01+02	2020-08-13 21:15:19+02	print	tex-luatex	\N	tex-luatex-0.80.0_19
2020-08-13 21:15:20+02	2020-08-13 21:17:20+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-13 21:17:20+02	2020-08-13 21:19:45+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-13 21:19:46+02	2020-08-13 21:21:27+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-13 21:21:28+02	2020-08-13 21:22:39+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-13 21:22:40+02	2020-08-13 21:25:15+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-13 21:25:16+02	2020-08-13 21:40:22+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-08-13 21:40:22+02	2020-08-13 21:48:57+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-13 21:48:58+02	2020-08-13 21:56:53+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-13 21:56:54+02	2020-08-13 21:58:30+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-13 21:58:31+02	2020-08-13 22:04:27+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-13 22:04:28+02	2020-08-13 22:19:47+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-13 22:19:48+02	2020-08-13 22:36:40+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-13 22:36:41+02	2020-08-13 22:38:18+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-13 22:38:19+02	2020-08-13 22:39:48+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-08-13 22:39:49+02	2020-08-13 22:43:36+02	x11-wm	marco	\N	marco-1.22.4
2020-08-13 22:43:37+02	2020-08-13 22:44:56+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-13 22:44:57+02	2020-08-13 22:45:56+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-08-13 22:45:57+02	2020-08-13 22:48:55+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-13 22:48:58+02	2020-08-13 22:50:30+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-08-13 22:50:31+02	2020-08-13 22:56:46+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-13 22:56:47+02	2020-08-13 23:01:24+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-13 23:01:25+02	2020-08-13 23:01:47+02	devel	rebar	\N	rebar-2.6.4,1
2020-08-13 23:01:48+02	2020-08-13 23:03:27+02	devel	rebar3	\N	rebar3-3.13.2,1
2020-08-13 23:03:28+02	2020-08-13 23:05:58+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-13 23:05:59+02	2020-08-13 23:06:26+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-08-13 23:06:27+02	2020-08-13 23:09:01+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-13 23:09:02+02	2020-08-13 23:10:44+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-08-13 23:10:45+02	2020-08-13 23:11:32+02	graphics	libgxps	\N	libgxps-0.3.1
2020-08-13 23:11:33+02	2020-08-13 23:12:18+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-08-13 23:12:18+02	2020-08-13 23:20:23+02	graphics	poppler-qt5	\N	poppler-qt5-0.90.1
2020-08-13 23:20:24+02	2020-08-13 23:21:08+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-13 23:21:09+02	2020-08-13 23:22:27+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-13 23:22:28+02	2020-08-13 23:23:51+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-13 23:23:51+02	2020-08-13 23:53:17+02	math	R	\N	R-4.0.2
2020-08-13 23:53:19+02	2020-08-14 00:04:34+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-14 00:04:34+02	2020-08-14 00:06:13+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-08-14 00:06:14+02	2020-08-14 00:09:44+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-14 00:09:44+02	2020-08-14 00:14:14+02	net	freerdp	\N	freerdp-2.2.0
2020-08-14 00:14:15+02	2020-08-14 00:15:00+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-14 00:15:04+02	2020-08-14 00:16:15+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-14 00:16:16+02	2020-08-14 00:17:26+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-14 00:17:27+02	2020-08-14 00:23:50+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-14 00:23:54+02	2020-08-14 00:24:39+02	print	libspectre	\N	libspectre-0.2.9
2020-08-14 00:24:42+02	2020-08-14 00:28:02+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-14 00:28:03+02	2020-08-14 00:38:56+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-14 00:38:57+02	2020-08-14 00:47:50+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-14 00:47:51+02	2020-08-14 00:48:12+02	sysutils	hal-info	\N	hal-info-20091130
2020-08-13 13:07:17+02	2020-08-14 00:48:47+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-14 00:48:13+02	2020-08-14 00:53:08+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-14 00:53:08+02	2020-08-14 00:54:25+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-14 00:54:26+02	2020-08-14 00:55:12+02	sysutils	patchelf	\N	patchelf-0.11
2020-08-14 00:55:13+02	2020-08-14 01:01:30+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-14 01:01:30+02	2020-08-14 01:04:24+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-14 01:04:25+02	2020-08-14 01:05:32+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-14 01:05:32+02	2020-08-14 01:07:36+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-14 01:07:37+02	2020-08-14 01:08:26+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-14 01:08:27+02	2020-08-14 01:10:36+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-14 01:10:36+02	2020-08-14 01:10:46+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-08-14 01:10:47+02	2020-08-14 01:11:04+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-08-14 01:11:04+02	2020-08-14 01:14:31+02	x11-servers	xephyr	\N	xephyr-1.20.8_3,1
2020-08-14 01:14:32+02	2020-08-14 01:16:05+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-14 01:16:06+02	2020-08-14 01:16:52+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-08-14 01:16:52+02	2020-08-14 01:18:06+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-14 01:18:07+02	2020-08-14 01:21:24+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-14 01:21:24+02	2020-08-14 01:24:16+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-14 01:24:17+02	2020-08-14 01:28:07+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-14 01:28:08+02	2020-08-14 02:11:34+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-14 02:11:35+02	2020-08-14 02:15:23+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-14 02:15:24+02	2020-08-14 02:15:53+02	audio	dssi	\N	dssi-1.1.1_8
2020-08-14 02:15:54+02	2020-08-14 02:16:57+02	audio	faudio	\N	FAudio-20.08
2020-08-14 02:16:58+02	2020-08-14 02:18:27+02	audio	mate-media	\N	mate-media-1.24.0
2020-08-14 02:18:28+02	2020-08-14 02:20:28+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-14 02:20:32+02	2020-08-14 02:22:33+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-14 02:22:33+02	2020-08-14 02:27:18+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-14 02:27:19+02	2020-08-14 02:28:31+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-14 02:28:31+02	2020-08-14 02:29:40+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-14 02:29:41+02	2020-08-14 02:30:09+02	devel	erlang-cl	\N	erlang-cl-1.2.4
2020-08-14 02:30:10+02	2020-08-14 02:30:38+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-08-14 02:30:39+02	2020-08-14 02:31:07+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-08-14 02:31:07+02	2020-08-14 02:32:59+02	devel	gradle5	\N	gradle5-5.0
2020-08-14 02:33:00+02	2020-08-14 02:34:03+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-08-14 02:34:04+02	2020-08-14 02:36:41+02	devel	ode	\N	ode-0.13_4,1
2020-08-14 02:36:41+02	2020-08-14 02:37:19+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-08-14 00:48:49+02	2020-08-14 02:37:50+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-14 02:37:20+02	2020-08-14 02:39:10+02	devel	py-game	py37	py37-game-1.9.6_1
2020-08-14 02:37:53+02	2020-08-14 02:40:02+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-14 02:40:03+02	2020-08-14 03:08:41+02	editors	libreoffice	\N	libreoffice-6.4.5
2020-08-14 02:39:11+02	2020-08-14 03:09:06+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-14 03:09:07+02	2020-08-14 03:13:30+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-08-14 03:08:43+02	2020-08-14 03:15:53+02	editors	pluma	\N	pluma-1.24.0
2020-08-14 03:13:35+02	2020-08-14 03:20:25+02	french	med	\N	fr-med-4.0.0
2020-08-14 03:15:54+02	2020-08-14 03:21:27+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-08-14 03:21:28+02	2020-08-14 03:24:53+02	graphics	eom	\N	eom-1.24.0
2020-08-14 03:20:26+02	2020-08-14 03:25:31+02	graphics	atril	\N	atril-1.22.3_7
2020-08-14 03:25:32+02	2020-08-14 03:26:05+02	graphics	gle	\N	gle-3.1.0_7
2020-08-14 03:26:05+02	2020-08-14 03:26:30+02	graphics	gltt	\N	gltt-2.5.2_14
2020-08-14 03:24:54+02	2020-08-14 03:26:54+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-14 03:26:30+02	2020-08-14 03:28:29+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-14 03:28:30+02	2020-08-14 03:31:14+02	graphics	py-gimp	\N	py27-gimp-2.10.20_2
2020-08-14 03:26:55+02	2020-08-14 03:32:24+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-08-14 03:32:25+02	2020-08-14 03:33:17+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-08-14 03:31:15+02	2020-08-14 03:40:20+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-14 03:33:17+02	2020-08-14 03:57:39+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-14 03:40:20+02	2020-08-14 04:12:22+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-08-14 03:57:41+02	2020-08-14 04:14:05+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-14 04:12:23+02	2020-08-14 04:14:36+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-14 04:14:07+02	2020-08-14 04:15:23+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-14 04:15:24+02	2020-08-14 04:15:42+02	misc	xosd	\N	xosd-2.2.14_3
2020-08-14 04:15:43+02	2020-08-14 04:22:06+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-14 04:22:09+02	2020-08-14 04:23:25+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-08-14 04:23:25+02	2020-08-14 04:27:44+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-14 04:14:37+02	2020-08-14 04:33:58+02	misc	openvdb	\N	openvdb-7.0.0_4
2020-08-14 04:27:45+02	2020-08-14 04:35:52+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-14 04:33:59+02	2020-08-14 04:35:59+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-14 04:35:53+02	2020-08-14 04:36:43+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-14 04:36:00+02	2020-08-14 04:36:57+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-14 04:36:44+02	2020-08-14 04:37:36+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-14 04:36:57+02	2020-08-14 04:37:52+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-14 04:37:37+02	2020-08-14 04:38:26+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-14 04:37:53+02	2020-08-14 04:38:43+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-14 04:38:26+02	2020-08-14 04:39:31+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-14 04:38:43+02	2020-08-14 04:40:14+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-14 04:39:32+02	2020-08-14 04:40:33+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-14 04:40:15+02	2020-08-14 04:41:37+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-14 04:41:38+02	2020-08-14 04:51:44+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-14 04:40:33+02	2020-08-14 04:56:42+02	print	lilypond	\N	lilypond-2.20.0
2020-08-14 04:51:45+02	2020-08-14 04:58:44+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-14 04:56:43+02	2020-08-14 04:58:57+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-14 04:58:46+02	2020-08-14 05:09:19+02	print	tex-xetex	\N	tex-xetex-0.99992_35
2020-08-14 05:09:20+02	2020-08-14 05:09:59+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-14 05:09:59+02	2020-08-14 05:10:59+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-14 05:11:00+02	2020-08-14 05:12:49+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-14 05:12:50+02	2020-08-14 05:15:32+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-14 05:15:33+02	2020-08-14 05:17:30+02	textproc	fop	\N	fop-2.3
2020-08-14 05:17:30+02	2020-08-14 05:18:36+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-14 05:18:37+02	2020-08-14 05:19:41+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-14 04:58:58+02	2020-08-14 05:21:26+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-14 05:19:42+02	2020-08-14 05:21:46+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-14 05:21:28+02	2020-08-14 05:23:20+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-14 05:23:21+02	2020-08-14 05:24:50+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-14 05:24:51+02	2020-08-14 05:26:34+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-14 05:21:47+02	2020-08-14 05:27:54+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-14 05:26:35+02	2020-08-14 05:28:39+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-14 05:28:40+02	2020-08-14 05:30:51+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-08-14 05:27:55+02	2020-08-14 05:31:12+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-14 05:31:13+02	2020-08-14 05:31:53+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-14 05:30:52+02	2020-08-14 05:32:07+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-14 05:31:54+02	2020-08-14 05:42:38+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-14 05:42:39+02	2020-08-14 05:42:52+02	astro	xearth	\N	xearth-1.2
2020-08-14 05:42:52+02	2020-08-14 05:46:22+02	astro	xephem	\N	xephem-3.7.7_5
2020-08-14 05:32:07+02	2020-08-14 06:06:41+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-14 06:06:42+02	2020-08-14 06:07:07+02	audio	xmix	\N	xmix-2.1_3
2020-08-14 05:46:23+02	2020-08-14 06:19:15+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-14 06:19:16+02	2020-08-14 06:25:53+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-14 06:25:53+02	2020-08-14 06:29:05+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-14 06:29:07+02	2020-08-14 08:53:18+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-14 08:53:27+02	2020-08-14 08:56:29+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-14 08:56:30+02	2020-08-14 08:59:02+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-14 08:59:03+02	2020-08-14 08:59:21+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-08-14 08:59:22+02	2020-08-14 09:35:53+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-14 09:35:54+02	2020-08-14 09:42:41+02	deskutils	calibre	\N	calibre-4.22.0
2020-08-14 09:42:42+02	2020-08-14 09:44:09+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-14 09:44:10+02	2020-08-14 09:44:26+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-08-14 09:44:26+02	2020-08-14 09:44:46+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-08-14 06:07:07+02	2020-08-14 09:49:18+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-14 09:49:22+02	2020-08-14 09:51:41+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-14 09:44:47+02	2020-08-14 11:44:57+02	devel	RStudio	\N	RStudio-1.3.1056
2020-08-14 11:44:59+02	2020-08-14 12:49:57+02	devel	ghidra	\N	ghidra-9.1
2020-08-14 12:49:58+02	2020-08-14 22:20:13+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200806
2020-08-14 09:51:42+02	2020-08-14 23:38:17+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-14 23:38:21+02	2020-08-14 23:41:31+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.5
2020-08-14 23:41:33+02	2020-08-14 23:44:59+02	editors	vim	\N	vim-8.2.1382
2020-08-14 22:20:16+02	2020-08-15 00:09:36+02	editors	codelite	\N	codelite-12.0_4
2020-08-14 23:45:00+02	2020-08-15 01:06:06+02	emulators	qemu	\N	qemu-4.2.1
2020-08-15 01:06:07+02	2020-08-15 01:25:54+02	emulators	wine	\N	wine-5.0.2,1
2020-08-15 01:25:55+02	2020-08-15 01:26:44+02	emulators	x48	\N	x48-0.6.4_1
2020-08-15 00:09:37+02	2020-08-15 01:32:22+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44
2020-08-15 01:32:24+02	2020-08-15 01:32:50+02	games	KnightCap	\N	knightcap-3.6_8
2020-08-15 01:32:51+02	2020-08-15 01:33:07+02	games	affenspiel	\N	affenspiel-1.0_2
2020-08-15 01:33:08+02	2020-08-15 01:33:36+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-08-15 01:33:37+02	2020-08-15 01:37:10+02	games	alienarena	\N	alienarena-2013.766_9
2020-08-15 01:37:11+02	2020-08-15 01:39:22+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-08-15 01:26:44+02	2020-08-15 01:42:59+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-15 01:43:01+02	2020-08-15 01:43:39+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-08-15 01:43:39+02	2020-08-15 01:44:36+02	games	monsterz	\N	monsterz-0.7.1_15
2020-08-15 01:44:37+02	2020-08-15 01:45:27+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-15 01:45:27+02	2020-08-15 01:46:45+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-15 01:39:23+02	2020-08-15 01:47:22+02	games	battletanks	\N	btanks-0.9.8083_9
2020-08-15 01:46:45+02	2020-08-15 01:47:54+02	games	xblast	\N	xblast-2.10.4_17
2020-08-15 01:47:26+02	2020-08-15 01:48:48+02	games	xboard	\N	xboard-4.9.1_1
2020-08-15 01:47:54+02	2020-08-15 01:48:48+02	games	xcowsay	\N	xcowsay-1.5
2020-08-15 01:48:49+02	2020-08-15 01:49:12+02	games	xminesweep	\N	xminesweep-3.0_2
2020-08-15 01:48:48+02	2020-08-15 01:49:36+02	games	xevil	\N	xevil-2.02.r2_2
2020-08-15 01:49:37+02	2020-08-15 01:55:30+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-15 01:55:31+02	2020-08-15 01:57:12+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-15 01:57:13+02	2020-08-15 03:21:24+02	graphics	inkscape	\N	inkscape-1.0_3
2020-08-15 01:49:15+02	2020-08-15 03:23:44+02	graphics	blender	\N	blender-2.83.2
2020-08-15 03:23:46+02	2020-08-15 03:30:32+02	graphics	wings	\N	wings-2.2.5
2020-08-15 03:30:33+02	2020-08-15 03:34:15+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-15 03:34:16+02	2020-08-15 04:13:53+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-08-15 04:13:55+02	2020-08-15 04:15:14+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-15 04:15:15+02	2020-08-15 04:49:25+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.5
2020-08-15 04:49:26+02	2020-08-15 05:22:05+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-15 05:22:07+02	2020-08-15 08:23:11+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200809
2020-08-15 08:23:13+02	2020-08-15 09:10:54+02	lang	mono	\N	mono-5.10.1.57_3
2020-08-15 09:10:56+02	2020-08-15 09:11:17+02	mail	bbmail	\N	bbmail-0.9.3
2020-08-15 09:11:18+02	2020-08-15 09:16:48+02	math	maxima	\N	maxima-5.44.0_3
2020-08-15 09:16:49+02	2020-08-15 09:18:03+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-15 09:18:04+02	2020-08-15 09:18:21+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-08-15 09:18:23+02	2020-08-15 09:19:22+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-08-15 09:19:23+02	2020-08-15 09:44:02+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p4
2020-08-15 09:44:11+02	2020-08-15 09:45:39+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-08-15 09:45:44+02	2020-08-15 09:46:39+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-08-15 09:46:44+02	2020-08-15 09:58:00+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-15 09:58:07+02	2020-08-15 10:05:16+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-15 10:05:25+02	2020-08-15 10:35:17+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-15 10:35:20+02	2020-08-15 10:46:21+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-15 10:46:29+02	2020-08-15 10:48:57+02	net	rdesktop	\N	rdesktop-1.8.6
2020-08-15 10:48:58+02	2020-08-15 10:52:37+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-15 10:52:43+02	2020-08-15 11:44:41+02	net	wireshark	\N	wireshark-3.2.5
2020-08-15 11:44:47+02	2020-08-15 11:46:10+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-15 11:46:11+02	2020-08-15 11:47:18+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-15 11:47:19+02	2020-08-15 11:48:18+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-15 11:48:18+02	2020-08-15 11:50:32+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-15 11:50:33+02	2020-08-15 11:53:27+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-15 11:53:30+02	2020-08-15 11:56:20+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-15 11:56:22+02	2020-08-15 11:59:19+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-15 11:59:23+02	2020-08-15 12:02:07+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-15 12:02:14+02	2020-08-15 12:05:42+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-15 12:05:45+02	2020-08-15 12:08:52+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-15 12:08:54+02	2020-08-15 12:15:05+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-15 12:15:07+02	2020-08-15 12:18:44+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-15 12:18:48+02	2020-08-15 12:25:51+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-15 12:25:53+02	2020-08-15 12:26:38+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-15 12:26:40+02	2020-08-15 12:27:32+02	print	gv	\N	gv-3.7.4_5
2020-08-15 12:27:33+02	2020-08-15 12:48:13+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-15 12:48:14+02	2020-08-15 13:46:11+02	print	scribus-devel	\N	scribus-devel-1.5.5_15
2020-08-15 13:46:13+02	2020-08-15 13:51:33+02	print	texlive-full	\N	texlive-full-20150521
2020-08-15 13:51:34+02	2020-08-15 13:52:49+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-15 13:52:50+02	2020-08-15 13:53:02+02	sysutils	bpytop	\N	bpytop-1.0.12
2020-08-15 13:53:04+02	2020-08-15 13:54:09+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-15 13:54:10+02	2020-08-15 13:55:16+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-15 13:55:18+02	2020-08-15 13:56:39+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-15 13:56:41+02	2020-08-15 13:59:48+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-15 13:59:52+02	2020-08-15 14:02:33+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-08-13 20:58:21+02	2020-08-15 15:15:07+02	www	chromium	\N	chromium-84.0.4147.105
2020-08-15 15:15:09+02	2020-08-15 15:16:31+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-15 15:16:32+02	2020-08-15 15:17:57+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-15 15:17:58+02	2020-08-15 15:19:23+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-15 15:19:24+02	2020-08-15 15:20:39+02	www	links	\N	links-2.20.2,1
2020-08-15 15:20:41+02	2020-08-15 15:23:01+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-15 15:23:02+02	2020-08-15 15:24:47+02	x11	mate	\N	mate-1.22.2
2020-08-15 15:24:47+02	2020-08-15 15:24:58+02	x11	numlockx	\N	numlockx-1.2
2020-08-15 15:25:38+02	2020-08-15 15:25:51+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-08-15 15:25:51+02	2020-08-15 15:26:17+02	x11	xdm	\N	xdm-1.1.12_3
2020-08-15 15:26:18+02	2020-08-15 15:28:03+02	x11	xlockmore	\N	xlockmore-5.65
2020-08-15 15:28:03+02	2020-08-15 15:28:20+02	x11	xman	\N	xman-1.1.5
2020-08-15 15:28:20+02	2020-08-15 15:35:15+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-15 15:35:16+02	2020-08-15 15:40:54+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-15 15:40:55+02	2020-08-15 15:42:51+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-15 15:42:51+02	2020-08-15 15:43:15+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-08-15 15:43:16+02	2020-08-15 15:43:29+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-08-15 15:43:29+02	2020-08-15 15:44:25+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-08-15 15:44:25+02	2020-08-15 15:44:50+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-08-15 15:44:51+02	2020-08-15 15:45:29+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-08-15 15:45:30+02	2020-08-15 15:45:57+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-08-15 15:45:57+02	2020-08-15 16:17:32+02	editors	vscode	\N	vscode-1.46.1
2020-08-15 16:17:33+02	2020-08-15 16:18:53+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-15 16:18:54+02	2020-08-15 16:19:33+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-15 14:02:34+02	2020-08-15 16:57:35+02	www	firefox	\N	firefox-79.0_2,1
2020-08-15 17:17:13+02	2020-08-15 17:24:57+02	graphics	krita	\N	krita-4.3.0_2
2020-08-15 17:56:29+02	2020-08-15 17:57:28+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.1
2020-08-15 17:57:29+02	2020-08-15 17:58:26+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-08-15 17:57:29+02	2020-08-15 17:58:27+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-15 17:57:29+02	2020-08-15 17:58:59+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-15 17:58:28+02	2020-08-15 17:59:32+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-15 17:58:27+02	2020-08-15 17:59:34+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-15 17:59:33+02	2020-08-15 18:00:34+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-15 17:59:34+02	2020-08-15 18:00:36+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-15 17:58:59+02	2020-08-15 18:00:39+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-15 18:00:34+02	2020-08-15 18:01:34+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-15 18:00:36+02	2020-08-15 18:01:36+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-15 18:00:41+02	2020-08-15 18:01:40+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-15 18:01:35+02	2020-08-15 18:02:30+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-15 18:01:41+02	2020-08-15 18:02:43+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-15 18:01:36+02	2020-08-15 18:03:05+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-15 18:02:30+02	2020-08-15 18:03:20+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-15 18:03:21+02	2020-08-15 18:24:06+02	local	E590T-localbase	\N	E590T-localbase-2020081300
2020-08-15 18:24:20+02	2020-08-15 18:25:29+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-15 18:24:20+02	2020-08-15 18:29:54+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-15 18:29:55+02	2020-08-15 18:30:03+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-15 20:35:29+02	2020-08-15 20:35:36+02	misc	pciids	\N	pciids-20200721
2020-08-15 20:35:29+02	2020-08-15 20:35:43+02	graphics	poppler-data	\N	poppler-data-0.4.9_5
2020-08-15 20:35:37+02	2020-08-15 20:35:53+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-08-15 20:35:54+02	2020-08-15 20:36:18+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-08-15 20:35:29+02	2020-08-15 20:39:38+02	databases	postgresql12-client	\N	postgresql12-client-12.4
2020-08-15 20:39:38+02	2020-08-15 20:39:59+02	multimedia	libva	\N	libva-2.8.0
2020-08-15 20:35:44+02	2020-08-15 20:40:27+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_8
2020-08-15 20:40:28+02	2020-08-15 20:41:23+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-08-15 20:41:24+02	2020-08-15 20:42:02+02	databases	php73-pdo_pgsql	\N	php73-pdo_pgsql-7.3.20
2020-08-15 20:40:00+02	2020-08-15 20:42:15+02	net	openldap24-client	\N	openldap-client-2.4.51
2020-08-15 20:42:15+02	2020-08-15 20:42:24+02	devel	libpci	\N	libpci-3.7.0
2020-08-15 20:42:02+02	2020-08-15 20:42:35+02	databases	php73-pgsql	\N	php73-pgsql-7.3.20
2020-08-15 20:36:20+02	2020-08-15 20:42:55+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-08-15 20:42:36+02	2020-08-15 20:44:13+02	graphics	ilmbase	\N	ilmbase-2.5.3
2020-08-15 20:44:13+02	2020-08-15 20:44:40+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-08-15 20:42:56+02	2020-08-15 20:45:13+02	graphics	cairo	\N	cairo-1.16.0,2
2020-08-15 20:42:25+02	2020-08-15 20:45:19+02	net	php73-ldap	\N	php73-ldap-7.3.20
2020-08-15 20:45:19+02	2020-08-15 20:46:02+02	graphics	libGLU	\N	libGLU-9.0.1
2020-08-15 20:45:13+02	2020-08-15 20:48:18+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-15 20:46:03+02	2020-08-15 20:49:06+02	graphics	poppler	\N	poppler-20.08.0_1
2020-08-15 20:49:07+02	2020-08-15 20:50:56+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-15 20:48:18+02	2020-08-15 20:54:40+02	graphics	graphviz	\N	graphviz-2.44.1_1
2020-08-15 20:54:40+02	2020-08-15 21:00:06+02	lang	vala	\N	vala-0.40.15,1
2020-08-15 21:00:07+02	2020-08-15 21:01:46+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-15 20:50:56+02	2020-08-15 21:02:30+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-15 21:02:31+02	2020-08-15 21:05:18+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-08-15 20:44:41+02	2020-08-15 21:06:18+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-15 21:05:19+02	2020-08-15 21:08:00+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-08-15 21:01:47+02	2020-08-15 21:15:04+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-15 21:15:04+02	2020-08-15 21:16:54+02	graphics	frei0r	\N	frei0r-1.6.1
2020-08-15 21:08:02+02	2020-08-15 21:18:28+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-15 21:06:19+02	2020-08-15 21:18:29+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-15 21:16:54+02	2020-08-15 21:38:48+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-15 21:38:49+02	2020-08-15 21:40:55+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-15 21:40:56+02	2020-08-15 21:42:30+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-15 21:42:30+02	2020-08-15 21:42:53+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-08-15 21:42:53+02	2020-08-15 21:45:24+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-15 21:18:29+02	2020-08-15 21:45:27+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-15 21:45:24+02	2020-08-15 21:46:10+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-08-15 21:45:28+02	2020-08-15 21:49:07+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-15 21:18:29+02	2020-08-15 21:49:34+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-15 21:49:35+02	2020-08-15 21:49:55+02	www	libwpe	\N	libwpe-1.4.0.1
2020-08-15 21:49:56+02	2020-08-15 21:50:21+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-08-15 21:49:08+02	2020-08-15 21:50:35+02	graphics	graphene	\N	graphene-1.10.0
2020-08-15 21:50:21+02	2020-08-15 21:51:05+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-15 21:51:05+02	2020-08-15 21:51:40+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-15 21:50:35+02	2020-08-15 21:52:28+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-08-15 21:46:10+02	2020-08-15 21:52:46+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-25 04:07:07+02	2020-09-25 04:07:19+02	x11	xkill	\N	xkill-1.0.5
2020-08-15 21:51:40+02	2020-08-15 21:53:16+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-15 21:53:17+02	2020-08-15 21:53:59+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-08-15 21:54:00+02	2020-08-15 22:00:07+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-15 22:00:08+02	2020-08-15 22:01:39+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-15 21:52:47+02	2020-08-15 22:09:54+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-15 22:09:56+02	2020-08-15 22:11:01+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-15 22:11:02+02	2020-08-15 22:12:44+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-15 22:12:45+02	2020-08-15 22:14:05+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-15 22:01:39+02	2020-08-15 22:15:33+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-08-15 22:14:05+02	2020-08-15 22:15:51+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-15 22:15:34+02	2020-08-15 22:16:36+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-15 22:15:52+02	2020-08-15 22:17:41+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-08-15 22:16:37+02	2020-08-15 22:18:05+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.1
2020-08-15 22:17:43+02	2020-08-15 22:19:16+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-08-15 22:19:16+02	2020-08-15 22:20:13+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-08-15 22:20:15+02	2020-08-15 22:22:03+02	sysutils	hal	\N	hal-0.5.14_34
2020-08-15 22:18:05+02	2020-08-15 22:23:16+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-15 22:22:06+02	2020-08-15 22:23:43+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-15 22:23:43+02	2020-08-15 22:25:55+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-15 22:25:55+02	2020-08-15 22:27:45+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-08-15 22:23:16+02	2020-08-15 22:28:15+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_3,1
2020-08-15 22:27:46+02	2020-08-15 22:36:56+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-15 22:36:56+02	2020-08-15 22:40:32+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-08-15 22:40:32+02	2020-08-15 22:42:36+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-08-15 22:28:15+02	2020-08-15 22:43:27+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-15 22:42:37+02	2020-08-15 22:43:42+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-15 22:43:42+02	2020-08-15 22:47:34+02	graphics	poppler-glib	\N	poppler-glib-20.08.0_1
2020-08-15 22:43:28+02	2020-08-15 22:48:56+02	graphics	openexr	\N	openexr-2.5.3
2020-08-15 22:48:57+02	2020-08-15 22:50:14+02	devel	hwloc	\N	hwloc-1.11.11
2020-08-15 22:47:35+02	2020-08-15 22:51:05+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-08-15 22:50:15+02	2020-08-15 22:51:22+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-08-15 22:51:10+02	2020-08-15 22:52:21+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-08-15 22:52:22+02	2020-08-15 22:53:38+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-15 22:53:39+02	2020-08-15 22:54:29+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-08-15 22:51:22+02	2020-08-15 22:54:37+02	graphics	glew	\N	glew-2.2.0
2020-08-15 22:54:30+02	2020-08-15 22:55:57+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-15 22:54:38+02	2020-08-15 22:56:22+02	net	openldap24-sasl-client	\N	openldap-sasl-client-2.4.51
2020-08-15 22:55:59+02	2020-08-15 22:57:15+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-15 22:57:15+02	2020-08-15 22:58:59+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-08-15 22:56:23+02	2020-08-15 22:59:38+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-15 22:59:00+02	2020-08-15 23:01:33+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-08-15 22:59:39+02	2020-08-15 23:02:17+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-15 23:02:18+02	2020-08-15 23:02:44+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-08-15 23:01:34+02	2020-08-15 23:03:02+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-08-15 23:02:44+02	2020-08-15 23:04:03+02	net	remmina	\N	remmina-1.3.9
2020-08-15 23:03:02+02	2020-08-15 23:06:20+02	security	gcr	\N	gcr-3.28.0
2020-08-15 23:06:21+02	2020-08-15 23:07:35+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-08-15 23:04:03+02	2020-08-15 23:12:10+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-08-15 23:12:11+02	2020-08-15 23:13:16+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-15 23:13:16+02	2020-08-15 23:14:24+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-15 23:07:35+02	2020-08-15 23:23:11+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-15 23:14:24+02	2020-08-15 23:24:57+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-15 23:24:58+02	2020-08-15 23:25:50+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-15 23:25:51+02	2020-08-15 23:27:28+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-08-15 23:27:29+02	2020-08-15 23:28:58+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-08-15 23:28:59+02	2020-08-15 23:30:35+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-15 23:23:12+02	2020-08-15 23:42:02+02	net	openmpi	\N	openmpi-4.0.4
2020-08-15 23:30:36+02	2020-08-15 23:56:51+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-15 23:56:52+02	2020-08-15 23:57:38+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-15 23:57:38+02	2020-08-16 00:00:53+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-16 00:00:54+02	2020-08-16 00:01:19+02	print	gl2ps	\N	gl2ps-1.4.0
2020-08-16 00:01:20+02	2020-08-16 00:03:06+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-08-16 00:03:07+02	2020-08-16 00:05:57+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-16 00:05:57+02	2020-08-16 00:07:48+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-15 21:52:29+02	2020-08-16 00:08:00+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-16 00:07:48+02	2020-08-16 00:13:01+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-08-16 00:13:02+02	2020-08-16 00:18:02+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-16 00:18:03+02	2020-08-16 00:20:19+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-08-16 00:20:21+02	2020-08-16 00:32:29+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-16 00:08:03+02	2020-08-16 00:44:33+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-16 00:44:34+02	2020-08-16 00:47:34+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-16 00:47:34+02	2020-08-16 00:48:58+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-16 00:48:59+02	2020-08-16 00:50:24+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-08-16 00:50:25+02	2020-08-16 00:59:55+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-16 00:59:56+02	2020-08-16 01:00:32+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-08-16 01:00:33+02	2020-08-16 01:00:51+02	databases	fpc-postgres	\N	fpc-postgres-3.0.4_6
2020-08-16 01:00:52+02	2020-08-16 01:02:11+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-16 01:02:11+02	2020-08-16 01:03:12+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-16 01:03:12+02	2020-08-16 01:05:50+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-15 23:42:03+02	2020-08-16 01:15:06+02	math	vtk8	\N	vtk8-8.2.0
2020-08-16 01:15:07+02	2020-08-16 03:29:35+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-16 03:29:36+02	2020-08-16 03:31:32+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-08-16 03:31:33+02	2020-08-16 03:36:33+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-16 03:36:34+02	2020-08-16 03:36:53+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-08-16 03:36:53+02	2020-08-16 03:38:24+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-16 03:38:24+02	2020-08-16 03:39:42+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-16 03:39:43+02	2020-08-16 03:41:03+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-16 03:41:04+02	2020-08-16 03:42:20+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-16 03:42:20+02	2020-08-16 03:43:45+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-16 03:43:45+02	2020-08-16 03:47:07+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-08-16 03:47:08+02	2020-08-16 03:48:20+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-16 03:48:21+02	2020-08-16 03:49:36+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-16 03:49:37+02	2020-08-16 03:52:14+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-08-16 03:52:14+02	2020-08-16 03:54:12+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-08-16 03:54:13+02	2020-08-16 03:54:55+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-16 00:32:30+02	2020-08-16 04:01:51+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-16 04:01:58+02	2020-08-16 04:07:14+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-16 04:07:17+02	2020-08-16 04:14:26+02	graphics	poppler-utils	\N	poppler-utils-20.08.0_1
2020-08-16 03:54:56+02	2020-08-16 04:25:18+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-16 04:14:29+02	2020-08-16 04:44:44+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-16 04:44:45+02	2020-08-16 04:45:08+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-08-16 04:45:09+02	2020-08-16 04:46:05+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-16 04:46:06+02	2020-08-16 04:47:03+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-16 04:47:04+02	2020-08-16 04:48:31+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-16 04:48:31+02	2020-08-16 04:49:00+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-08-16 04:49:01+02	2020-08-16 04:50:10+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-16 04:50:11+02	2020-08-16 04:54:07+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-16 04:54:08+02	2020-08-16 04:57:22+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_3,1
2020-08-16 04:57:23+02	2020-08-16 05:07:17+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-16 05:07:18+02	2020-08-16 05:09:50+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-16 05:09:51+02	2020-08-16 05:10:58+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-16 05:10:59+02	2020-08-16 05:12:29+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-16 05:12:30+02	2020-08-16 05:14:07+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-16 05:14:07+02	2020-08-16 05:15:50+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-16 05:15:51+02	2020-08-16 05:18:24+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-08-16 05:18:25+02	2020-08-16 05:21:59+02	graphics	gegl	\N	gegl-0.4.24_4
2020-08-16 05:22:00+02	2020-08-16 05:22:25+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-08-16 05:22:26+02	2020-08-16 05:34:35+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-16 05:34:36+02	2020-08-16 05:34:44+02	misc	usbids	\N	usbids-20200707
2020-08-16 05:34:45+02	2020-08-16 05:37:47+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-16 04:25:19+02	2020-08-16 05:42:12+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-16 05:42:13+02	2020-08-16 05:42:30+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-08-16 05:42:30+02	2020-08-16 05:43:41+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-16 05:43:41+02	2020-08-16 05:44:52+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-16 05:44:52+02	2020-08-16 05:47:47+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-16 05:47:49+02	2020-08-16 05:49:59+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-16 05:50:00+02	2020-08-16 05:51:12+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-16 05:51:13+02	2020-08-16 05:52:23+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-08-16 05:52:24+02	2020-08-16 05:55:40+02	x11	xorg	\N	xorg-7.7_3
2020-08-16 05:55:41+02	2020-08-16 05:56:55+02	x11	zenity	\N	zenity-3.32.0
2020-08-16 05:56:55+02	2020-08-16 05:57:58+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-08-16 05:57:59+02	2020-08-16 05:59:20+02	devel	apache-ant	\N	apache-ant-1.10.8
2020-08-16 05:37:47+02	2020-08-16 06:00:47+02	net	samba410	\N	samba410-4.10.17
2020-08-16 06:00:47+02	2020-08-16 06:03:56+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-16 06:03:56+02	2020-08-16 06:05:27+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-16 06:05:27+02	2020-08-16 06:06:04+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-08-16 05:59:21+02	2020-08-16 06:06:40+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-16 06:06:04+02	2020-08-16 06:07:32+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-16 06:06:41+02	2020-08-16 06:08:18+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-16 06:07:33+02	2020-08-16 06:09:03+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-16 06:08:18+02	2020-08-16 06:12:23+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-16 06:09:04+02	2020-08-16 06:14:24+02	editors	emacs	full	emacs-27.1,3
2020-08-16 06:14:24+02	2020-08-16 06:14:45+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-08-16 06:12:23+02	2020-08-16 06:18:36+02	graphics	Coin	\N	Coin-4.0.0_1
2020-08-16 06:18:37+02	2020-08-16 06:19:26+02	graphics	glfw	\N	glfw-3.3.2
2020-08-16 06:19:26+02	2020-08-16 06:21:24+02	graphics	movit	\N	movit-1.6.2_2
2020-08-16 06:14:46+02	2020-08-16 06:26:58+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-08-16 06:26:59+02	2020-08-16 06:27:29+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-08-16 06:27:30+02	2020-08-16 06:28:58+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-16 06:28:59+02	2020-08-16 06:29:56+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.4.0_1
2020-08-16 06:29:57+02	2020-08-16 06:31:18+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-08-16 06:21:25+02	2020-08-16 06:37:27+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-16 06:31:19+02	2020-08-16 06:37:53+02	print	fontforge	\N	fontforge-20190801_2
2020-08-16 06:37:53+02	2020-08-16 06:38:46+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-16 06:38:46+02	2020-08-16 06:40:34+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.0
2020-08-16 06:37:28+02	2020-08-16 06:49:31+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-08-16 06:49:31+02	2020-08-16 06:50:59+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-16 06:51:00+02	2020-08-16 06:52:29+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-16 06:52:29+02	2020-08-16 06:53:40+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-16 06:53:41+02	2020-08-16 06:54:49+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-16 06:54:50+02	2020-08-16 06:56:41+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-16 06:56:42+02	2020-08-16 07:00:16+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-16 01:05:51+02	2020-08-16 07:01:25+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-16 07:01:26+02	2020-08-16 07:02:37+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-16 07:00:17+02	2020-08-16 07:03:00+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-16 07:02:37+02	2020-08-16 07:05:04+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-16 07:03:01+02	2020-08-16 07:08:27+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-16 07:08:28+02	2020-08-16 07:09:45+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-16 07:05:04+02	2020-08-16 07:11:10+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-16 07:09:46+02	2020-08-16 07:11:38+02	x11-wm	marco	\N	marco-1.22.4
2020-08-16 07:11:39+02	2020-08-16 07:12:05+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-08-16 07:11:10+02	2020-08-16 07:12:06+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-16 07:12:06+02	2020-08-16 07:13:07+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.0
2020-08-16 07:12:05+02	2020-08-16 07:13:48+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-16 07:13:07+02	2020-08-16 07:15:37+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-16 07:13:49+02	2020-08-16 07:15:58+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-16 07:15:58+02	2020-08-16 07:25:34+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-16 07:25:35+02	2020-08-16 07:27:02+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-16 07:27:03+02	2020-08-16 07:27:25+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-08-16 07:27:25+02	2020-08-16 07:29:30+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-16 07:29:31+02	2020-08-16 07:30:35+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-08-16 07:30:36+02	2020-08-16 07:31:11+02	graphics	libgxps	\N	libgxps-0.3.1
2020-08-16 07:31:11+02	2020-08-16 07:31:45+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-08-16 07:31:45+02	2020-08-16 07:38:16+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-08-16 07:38:17+02	2020-08-16 07:38:54+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-16 07:38:54+02	2020-08-16 07:39:51+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-16 07:39:52+02	2020-08-16 07:40:52+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-16 07:40:53+02	2020-08-16 07:42:16+02	graphics	vigra	\N	vigra-1.11.1_12
2020-08-16 07:15:38+02	2020-08-16 07:53:48+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-16 07:53:51+02	2020-08-16 08:01:12+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-16 08:01:13+02	2020-08-16 08:01:52+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-08-16 07:42:17+02	2020-08-16 08:02:10+02	math	R	\N	R-4.0.2
2020-08-16 08:02:11+02	2020-08-16 08:03:17+02	net	liveMedia	\N	liveMedia-2020.08.12,2
2020-08-16 08:01:53+02	2020-08-16 08:03:45+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-16 08:03:17+02	2020-08-16 08:04:06+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-16 08:03:46+02	2020-08-16 08:04:35+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-16 08:04:36+02	2020-08-16 08:05:03+02	print	libspectre	\N	libspectre-0.2.9
2020-08-16 08:05:04+02	2020-08-16 08:07:15+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-16 08:04:06+02	2020-08-16 08:07:51+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-16 08:07:16+02	2020-08-16 08:13:29+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-16 08:07:52+02	2020-08-16 08:13:43+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-16 08:13:29+02	2020-08-16 08:13:47+02	sysutils	hal-info	\N	hal-info-20091130
2020-08-16 08:13:48+02	2020-08-16 08:14:41+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-16 08:13:43+02	2020-08-16 08:16:55+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-16 08:14:42+02	2020-08-16 08:17:35+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-16 08:17:36+02	2020-08-16 08:18:17+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-16 08:16:56+02	2020-08-16 08:18:48+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-16 08:18:49+02	2020-08-16 08:19:27+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-16 08:18:18+02	2020-08-16 08:19:29+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-16 08:19:27+02	2020-08-16 08:20:47+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-16 08:19:29+02	2020-08-16 08:21:55+02	x11-servers	xephyr	\N	xephyr-1.20.8_3,1
2020-08-16 08:20:47+02	2020-08-16 08:22:02+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-16 08:21:56+02	2020-08-16 08:22:43+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-16 08:22:03+02	2020-08-16 08:23:06+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-16 08:22:43+02	2020-08-16 08:24:04+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-16 08:23:06+02	2020-08-16 08:24:42+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-16 08:24:42+02	2020-08-16 08:27:14+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-16 08:27:15+02	2020-08-16 08:27:59+02	audio	faudio	\N	FAudio-20.08
2020-08-16 08:27:59+02	2020-08-16 08:29:18+02	audio	mate-media	\N	mate-media-1.24.0
2020-08-16 08:29:18+02	2020-08-16 08:29:52+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.14.0
2020-08-16 08:29:52+02	2020-08-16 08:30:08+02	databases	pgtcl	\N	pgtcl-2.1.1_2
2020-08-16 08:30:09+02	2020-08-16 08:31:29+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-16 08:31:29+02	2020-08-16 08:33:00+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-16 08:33:02+02	2020-08-16 08:37:14+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-16 08:24:04+02	2020-08-16 08:37:42+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-16 08:37:15+02	2020-08-16 08:38:06+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-16 08:38:06+02	2020-08-16 08:38:22+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-08-16 08:37:42+02	2020-08-16 08:38:32+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-16 08:38:23+02	2020-08-16 08:38:39+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-08-16 08:38:33+02	2020-08-16 08:39:34+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-08-16 08:38:39+02	2020-08-16 08:39:35+02	devel	ode	\N	ode-0.13_4,1
2020-08-16 08:39:35+02	2020-08-16 08:40:41+02	devel	py-game	py37	py37-game-1.9.6_1
2020-08-16 08:39:36+02	2020-08-16 08:41:15+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-16 08:40:42+02	2020-08-16 09:06:05+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-16 09:06:06+02	2020-08-16 09:11:37+02	editors	pluma	\N	pluma-1.24.0
2020-08-16 09:11:38+02	2020-08-16 09:13:01+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-08-16 09:13:02+02	2020-08-16 09:22:55+02	graphics	alembic	\N	alembic-1.7.13_2
2020-08-16 09:22:56+02	2020-08-16 09:26:38+02	graphics	atril	\N	atril-1.22.3_8
2020-08-16 09:26:38+02	2020-08-16 09:29:04+02	graphics	eom	\N	eom-1.24.0
2020-08-16 09:29:05+02	2020-08-16 09:31:01+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-16 09:31:02+02	2020-08-16 09:31:31+02	graphics	gle	\N	gle-3.1.0_7
2020-08-16 09:31:32+02	2020-08-16 09:31:55+02	graphics	gltt	\N	gltt-2.5.2_14
2020-08-16 09:31:56+02	2020-08-16 09:33:24+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-16 09:33:24+02	2020-08-16 09:56:15+02	graphics	openimageio	\N	openimageio-2.1.15.0_5
2020-08-16 09:56:16+02	2020-08-16 10:00:49+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-08-16 10:00:50+02	2020-08-16 10:03:04+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-08-16 10:03:05+02	2020-08-16 10:06:25+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-16 10:06:25+02	2020-08-16 10:07:17+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-08-16 10:07:17+02	2020-08-16 10:20:26+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-16 10:20:27+02	2020-08-16 10:31:12+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-16 10:31:13+02	2020-08-16 10:33:49+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-16 10:33:50+02	2020-08-16 10:35:19+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-16 10:35:20+02	2020-08-16 10:49:57+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-08-16 10:49:58+02	2020-08-16 10:55:27+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-16 10:55:27+02	2020-08-16 10:56:39+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-08-16 10:56:42+02	2020-08-16 11:00:30+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-16 11:00:31+02	2020-08-16 11:20:46+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-16 11:20:47+02	2020-08-16 11:23:01+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-16 11:23:02+02	2020-08-16 11:24:06+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-16 11:24:06+02	2020-08-16 11:25:17+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-16 11:25:18+02	2020-08-16 11:26:20+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-16 11:26:21+02	2020-08-16 11:27:28+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-16 11:27:28+02	2020-08-16 11:28:35+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-16 11:28:35+02	2020-08-16 11:29:38+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-16 11:29:39+02	2020-08-16 11:31:00+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-16 11:31:01+02	2020-08-16 11:33:01+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-16 11:33:02+02	2020-08-16 11:34:09+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-16 11:34:09+02	2020-08-16 11:36:13+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-08-16 11:36:14+02	2020-08-16 11:37:57+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-16 11:37:58+02	2020-08-16 11:57:59+02	print	lilypond	\N	lilypond-2.20.0
2020-08-16 11:58:01+02	2020-08-16 12:08:25+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-16 12:08:26+02	2020-08-16 12:16:27+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-16 12:16:28+02	2020-08-16 12:19:26+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-16 12:19:27+02	2020-08-16 12:32:55+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-08-16 12:32:56+02	2020-08-16 13:01:17+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-16 13:01:22+02	2020-08-16 13:02:16+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-16 13:02:16+02	2020-08-16 13:03:34+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-16 13:03:34+02	2020-08-16 13:06:20+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-16 13:06:20+02	2020-08-16 13:09:47+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-16 13:09:48+02	2020-08-16 13:11:16+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-16 13:11:16+02	2020-08-16 13:12:42+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-16 13:12:42+02	2020-08-16 13:15:32+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-16 13:15:35+02	2020-08-16 13:17:52+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-16 13:17:53+02	2020-08-16 13:26:37+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-16 13:26:38+02	2020-08-16 13:28:11+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-16 13:28:12+02	2020-08-16 13:30:08+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-16 13:30:09+02	2020-08-16 13:32:57+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-16 13:32:58+02	2020-08-16 13:36:35+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-16 13:36:36+02	2020-08-16 13:38:59+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-08-16 13:39:00+02	2020-08-16 13:40:39+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-16 13:40:40+02	2020-08-16 13:41:32+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-16 13:41:33+02	2020-08-16 13:53:01+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-16 13:53:02+02	2020-08-16 14:30:55+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-16 14:30:57+02	2020-08-16 15:07:03+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-16 15:07:04+02	2020-08-16 15:08:44+02	benchmarks	fio	\N	fio-3.21
2020-08-16 08:41:16+02	2020-08-16 15:11:14+02	editors	libreoffice	\N	libreoffice-6.4.6
2020-08-16 15:11:23+02	2020-08-16 15:18:22+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-16 15:18:23+02	2020-08-16 15:21:56+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-16 06:40:35+02	2020-08-16 17:14:43+02	www	chromium	\N	chromium-84.0.4147.105
2020-08-16 17:14:45+02	2020-08-16 17:16:26+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-16 17:16:27+02	2020-08-16 17:17:55+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-16 17:17:56+02	2020-08-16 17:20:31+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-16 15:21:57+02	2020-08-16 17:22:18+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-16 17:20:32+02	2020-08-16 17:22:34+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-16 17:22:20+02	2020-08-16 17:22:36+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-08-16 17:22:37+02	2020-08-16 17:22:56+02	databases	pgtop	\N	pgtop-0.05_5
2020-08-16 17:22:57+02	2020-08-16 17:28:39+02	databases	postgresql12-contrib	\N	postgresql12-contrib-12.4
2020-08-16 17:28:40+02	2020-08-16 17:44:00+02	databases	postgresql12-server	\N	postgresql12-server-12.4
2020-08-16 17:44:02+02	2020-08-16 17:44:40+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-08-16 17:22:35+02	2020-08-16 17:50:02+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-16 17:44:41+02	2020-08-16 17:50:13+02	deskutils	calibre	\N	calibre-4.22.0_1
2020-08-16 15:08:45+02	2020-08-16 17:50:58+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-16 17:50:03+02	2020-08-16 17:51:00+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-16 17:50:59+02	2020-08-16 17:52:06+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-16 17:50:14+02	2020-08-16 19:43:35+02	devel	RStudio	\N	RStudio-1.3.1056
2020-08-16 19:43:36+02	2020-08-16 20:45:26+02	editors	codelite	\N	codelite-12.0_4
2020-08-16 20:45:27+02	2020-08-16 20:48:46+02	editors	libreoffice-nb	\N	nb-libreoffice-6.4.6
2020-08-16 20:48:47+02	2020-08-16 20:53:43+02	editors	vim	\N	vim-8.2.1382
2020-08-16 20:53:46+02	2020-08-16 21:30:27+02	emulators	qemu	\N	qemu-4.2.1
2020-08-16 21:30:28+02	2020-08-16 22:58:00+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-08-16 22:58:02+02	2020-08-16 23:17:52+02	emulators	wine	\N	wine-5.0.2,1
2020-08-16 23:17:53+02	2020-08-16 23:40:34+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-16 23:40:35+02	2020-08-16 23:40:55+02	games	KnightCap	\N	knightcap-3.6_8
2020-08-16 23:40:56+02	2020-08-16 23:41:18+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-08-16 23:41:18+02	2020-08-16 23:44:18+02	games	alienarena	\N	alienarena-2013.766_9
2020-08-16 23:44:19+02	2020-08-16 23:45:14+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-08-16 23:45:14+02	2020-08-16 23:49:00+02	games	battletanks	\N	btanks-0.9.8083_9
2020-08-16 23:49:07+02	2020-08-16 23:49:29+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-08-16 23:49:29+02	2020-08-16 23:50:36+02	games	monsterz	\N	monsterz-0.7.1_15
2020-08-16 23:50:37+02	2020-08-16 23:51:26+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-16 23:51:26+02	2020-08-16 23:52:30+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-16 23:52:30+02	2020-08-16 23:53:45+02	games	xblast	\N	xblast-2.10.4_17
2020-08-16 23:53:46+02	2020-08-16 23:55:11+02	games	xboard	\N	xboard-4.9.1_1
2020-08-16 23:55:12+02	2020-08-16 23:56:19+02	games	xcowsay	\N	xcowsay-1.5
2020-08-16 23:56:19+02	2020-08-17 00:23:19+02	graphics	blender	\N	blender-2.83.2_1
2020-08-17 00:23:21+02	2020-08-17 00:27:29+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-17 00:27:30+02	2020-08-17 00:29:09+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-17 00:29:09+02	2020-08-17 00:53:45+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-17 00:53:46+02	2020-08-17 01:18:45+02	graphics	krita	\N	krita-4.3.0_4
2020-08-17 01:18:46+02	2020-08-17 01:23:33+02	graphics	wings	\N	wings-2.2.5
2020-08-17 01:23:33+02	2020-08-17 01:25:25+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-17 01:25:26+02	2020-08-17 01:26:49+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-17 01:26:50+02	2020-08-17 02:15:10+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.6
2020-08-17 02:15:11+02	2020-08-17 02:59:15+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-17 02:59:16+02	2020-08-17 03:08:41+02	math	maxima	\N	maxima-5.44.0_3
2020-08-17 03:08:43+02	2020-08-17 03:10:15+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-17 03:10:18+02	2020-08-17 03:10:50+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-08-17 03:10:52+02	2020-08-17 03:11:29+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-08-17 03:11:30+02	2020-08-17 03:47:06+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p5
2020-08-17 03:47:08+02	2020-08-17 03:48:06+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-08-17 03:48:07+02	2020-08-17 03:48:35+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-08-17 03:48:36+02	2020-08-17 03:57:45+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-17 03:57:46+02	2020-08-17 04:01:36+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-17 04:01:36+02	2020-08-17 04:13:28+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-17 04:13:30+02	2020-08-17 04:20:18+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-17 04:20:19+02	2020-08-17 04:22:21+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-17 04:22:21+02	2020-08-17 04:46:35+02	net	wireshark	\N	wireshark-3.2.6
2020-08-16 17:51:00+02	2020-08-17 04:46:58+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-17 04:46:36+02	2020-08-17 04:47:24+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-17 04:47:24+02	2020-08-17 04:48:07+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-17 04:48:07+02	2020-08-17 04:48:48+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-18 07:32:39+02	2020-08-18 07:35:54+02	graphics	poppler	\N	poppler-20.08.0_1
2020-08-17 04:48:49+02	2020-08-17 04:49:29+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-17 04:49:29+02	2020-08-17 04:50:12+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-17 04:50:13+02	2020-08-17 04:50:54+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-09-26 01:00:33+02	2020-09-26 01:02:17+02	x11	yelp	\N	yelp-3.38.0
2020-08-17 04:50:54+02	2020-08-17 04:51:36+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-17 04:51:37+02	2020-08-17 04:52:16+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-17 04:52:17+02	2020-08-17 04:53:29+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-17 04:53:29+02	2020-08-17 04:54:15+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-17 04:54:15+02	2020-08-17 04:54:55+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-17 04:54:55+02	2020-08-17 04:56:43+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-17 04:56:43+02	2020-08-17 04:57:42+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-17 04:46:59+02	2020-08-17 04:59:48+02	net-mgmt	icinga2	\N	icinga2-2.11.3
2020-08-17 04:57:42+02	2020-08-17 05:00:07+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-17 04:59:48+02	2020-08-17 05:00:17+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-17 05:00:08+02	2020-08-17 05:00:39+02	print	gv	\N	gv-3.7.4_5
2020-08-16 17:52:07+02	2020-08-17 05:04:45+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200811
2020-08-17 05:04:50+02	2020-08-17 05:10:27+02	print	texlive-full	\N	texlive-full-20150521
2020-08-17 05:10:29+02	2020-08-17 05:12:08+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-17 05:12:09+02	2020-08-17 05:12:21+02	sysutils	bpytop	\N	bpytop-1.0.13
2020-08-17 05:12:22+02	2020-08-17 05:13:36+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-17 05:13:37+02	2020-08-17 05:14:08+02	sysutils	flashrom	\N	flashrom-1.2
2020-08-17 05:14:09+02	2020-08-17 05:15:16+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-17 05:15:17+02	2020-08-17 05:16:37+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-17 05:00:18+02	2020-08-17 05:18:14+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-17 05:16:38+02	2020-08-17 05:19:33+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-17 05:19:34+02	2020-08-17 05:19:56+02	sysutils	x86info	\N	x86info-1.31.s03
2020-08-17 05:18:16+02	2020-08-17 05:21:25+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-08-17 05:21:25+02	2020-08-17 05:23:27+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-17 05:23:27+02	2020-08-17 05:25:06+02	x11	mate	\N	mate-1.22.2
2020-08-17 05:25:07+02	2020-08-17 05:26:42+02	x11	xlockmore	\N	xlockmore-5.65
2020-08-17 05:26:43+02	2020-08-17 05:34:29+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-17 05:34:29+02	2020-08-17 05:40:55+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-17 05:40:56+02	2020-08-17 05:43:15+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-17 05:43:16+02	2020-08-17 05:43:30+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-08-17 05:43:31+02	2020-08-17 05:44:35+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-08-17 05:00:39+02	2020-08-17 05:47:58+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-08-17 05:47:59+02	2020-08-17 05:49:05+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-17 05:49:06+02	2020-08-17 05:49:44+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-17 05:44:36+02	2020-08-17 06:11:50+02	editors	vscode	\N	vscode-1.46.1
2020-08-17 05:19:56+02	2020-08-17 08:11:32+02	www	firefox	\N	firefox-79.0_2,1
2020-08-17 23:31:40+02	2020-08-17 23:32:10+02	misc	mbuffer	\N	mbuffer-20200505
2020-08-17 23:32:10+02	2020-08-18 00:30:24+02	local	E590T-localbase	\N	E590T-localbase-2020081500
2020-08-18 00:31:37+02	2020-08-18 00:35:07+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44
2020-08-18 00:31:37+02	2020-08-18 00:47:11+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-18 00:47:12+02	2020-08-18 00:47:30+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-18 06:38:20+02	2020-08-18 06:50:15+02	x11	libX11	\N	libX11-1.6.11_1,1
2020-08-18 06:50:18+02	2020-08-18 06:51:44+02	x11	libXext	\N	libXext-1.3.4,1
2020-08-18 06:51:49+02	2020-08-18 06:53:18+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-08-18 06:38:23+02	2020-08-18 06:53:20+02	databases	sqlite3	\N	sqlite3-3.32.3_3,1
2020-08-18 06:53:20+02	2020-08-18 06:54:13+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-08-18 06:53:22+02	2020-08-18 06:54:19+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-08-18 06:54:16+02	2020-08-18 06:55:09+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-08-18 06:54:20+02	2020-08-18 06:55:16+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-08-18 06:55:16+02	2020-08-18 06:57:33+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-08-18 06:57:34+02	2020-08-18 06:59:24+02	devel	dbus	\N	dbus-1.12.20
2020-08-18 06:59:24+02	2020-08-18 07:00:06+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-08-18 07:00:07+02	2020-08-18 07:00:48+02	x11	libXi	\N	libXi-1.7.10,1
2020-08-18 07:00:49+02	2020-08-18 07:01:38+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-08-18 06:55:19+02	2020-08-18 07:03:22+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-08-18 07:01:39+02	2020-08-18 07:03:23+02	net	avahi-app	\N	avahi-app-0.7_3
2020-08-18 07:03:23+02	2020-08-18 07:03:47+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-08-18 07:03:48+02	2020-08-18 07:05:56+02	print	cups	\N	cups-2.3.3_1
2020-08-18 07:06:01+02	2020-08-18 07:06:22+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-08-18 07:03:22+02	2020-08-18 07:06:37+02	graphics	cairo	\N	cairo-1.16.0,2
2020-08-18 07:06:22+02	2020-08-18 07:06:50+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-08-18 07:06:45+02	2020-08-18 07:06:59+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-08-18 07:06:50+02	2020-08-18 07:07:15+02	x11	libXpm	\N	libXpm-3.5.13
2020-08-18 07:07:16+02	2020-08-18 07:08:22+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-08-18 07:08:23+02	2020-08-18 07:08:57+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-08-18 07:06:59+02	2020-08-18 07:09:40+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-18 06:38:22+02	2020-08-18 07:09:42+02	devel	cmake	\N	cmake-3.18.1
2020-08-18 07:09:43+02	2020-08-18 07:09:56+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-08-18 07:09:57+02	2020-08-18 07:10:19+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-08-18 07:10:21+02	2020-08-18 07:10:43+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-08-18 07:09:40+02	2020-08-18 07:10:54+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-08-18 07:10:44+02	2020-08-18 07:10:58+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-08-18 07:10:54+02	2020-08-18 07:11:23+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-08-18 07:10:58+02	2020-08-18 07:12:04+02	sysutils	polkit	\N	polkit-0.116
2020-08-18 07:11:23+02	2020-08-18 07:12:31+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-08-18 07:08:57+02	2020-08-18 07:12:35+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_8
2020-08-18 07:12:05+02	2020-08-18 07:12:44+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-08-18 07:12:34+02	2020-08-18 07:12:45+02	x11	xprop	\N	xprop-1.2.4
2020-08-18 07:12:45+02	2020-08-18 07:12:57+02	x11	xset	\N	xset-1.2.4_3
2020-08-18 07:12:57+02	2020-08-18 07:13:15+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-08-18 07:13:16+02	2020-08-18 07:13:51+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-08-18 07:13:52+02	2020-08-18 07:15:30+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-18 07:12:45+02	2020-08-18 07:15:37+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-08-18 07:12:35+02	2020-08-18 07:16:37+02	graphics	graphviz	\N	graphviz-2.44.1_1
2020-08-18 07:15:30+02	2020-08-18 07:17:57+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-08-18 07:15:38+02	2020-08-18 07:18:02+02	graphics	colord	\N	colord-1.3.5
2020-08-18 07:18:03+02	2020-08-18 07:18:51+02	graphics	libGLU	\N	libGLU-9.0.1
2020-08-18 07:16:38+02	2020-08-18 07:20:30+02	lang	vala	\N	vala-0.40.15,1
2020-08-18 07:20:31+02	2020-08-18 07:21:59+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-18 07:18:52+02	2020-08-18 07:22:04+02	security	nss	\N	nss-3.55
2020-08-18 07:22:05+02	2020-08-18 07:32:38+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-18 09:57:23+02	2020-08-18 09:57:43+02	x11	xrdb	\N	xrdb-1.2.0
2020-08-18 07:21:59+02	2020-08-18 07:36:26+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-18 07:35:55+02	2020-08-18 07:36:31+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-08-18 07:36:26+02	2020-08-18 07:37:28+02	devel	dconf	\N	dconf-0.36.0
2020-08-18 07:36:31+02	2020-08-18 07:38:08+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-18 07:17:57+02	2020-08-18 07:38:47+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-18 07:38:08+02	2020-08-18 07:38:47+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-08-18 07:38:48+02	2020-08-18 07:39:09+02	print	xpdfopen	\N	xpdfopen-0.86
2020-08-18 07:39:09+02	2020-08-18 07:39:22+02	multimedia	libva	\N	libva-2.8.0
2020-08-18 07:37:31+02	2020-08-18 07:39:23+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-08-18 07:39:22+02	2020-08-18 07:40:08+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-08-18 07:39:23+02	2020-08-18 07:41:37+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-08-18 07:41:38+02	2020-08-18 07:42:05+02	devel	libnotify	\N	libnotify-0.7.8
2020-08-18 07:42:06+02	2020-08-18 07:42:23+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-08-18 07:42:24+02	2020-08-18 07:42:46+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-08-18 07:42:47+02	2020-08-18 07:44:33+02	graphics	frei0r	\N	frei0r-1.6.1
2020-08-18 07:38:48+02	2020-08-18 07:48:23+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-18 07:40:08+02	2020-08-18 07:48:40+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-18 07:44:34+02	2020-08-18 08:05:09+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-18 08:05:10+02	2020-08-18 08:07:19+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-18 08:07:20+02	2020-08-18 08:08:49+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-18 08:08:50+02	2020-08-18 08:09:15+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-08-18 08:09:16+02	2020-08-18 08:11:26+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-18 08:11:26+02	2020-08-18 08:12:01+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-08-18 07:48:41+02	2020-08-18 08:13:27+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-18 08:12:02+02	2020-08-18 08:13:50+02	graphics	graphene	\N	graphene-1.10.0
2020-08-18 08:13:28+02	2020-08-18 08:16:37+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-18 08:16:38+02	2020-08-18 08:17:01+02	www	libwpe	\N	libwpe-1.4.0.1
2020-08-18 07:48:24+02	2020-08-18 08:17:38+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-18 08:17:38+02	2020-08-18 08:18:46+02	net	geoclue	\N	geoclue-2.5.5
2020-08-18 08:17:02+02	2020-08-18 08:19:03+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-08-18 08:18:46+02	2020-08-18 08:19:05+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-08-18 08:19:03+02	2020-08-18 08:19:40+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-18 08:13:50+02	2020-08-18 08:20:01+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-18 08:19:40+02	2020-08-18 08:20:09+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-18 08:20:09+02	2020-08-18 08:20:59+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-08-18 08:21:00+02	2020-08-18 08:21:46+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-08-18 08:21:47+02	2020-08-18 08:22:26+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-08-18 08:22:27+02	2020-08-18 08:24:34+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-18 08:24:35+02	2020-08-18 08:25:08+02	devel	json-c	\N	json-c-0.15
2020-08-18 08:25:09+02	2020-08-18 08:25:26+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-08-18 08:25:26+02	2020-08-18 08:26:36+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-08-18 08:26:36+02	2020-08-18 08:27:41+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-08-18 08:27:41+02	2020-08-18 08:27:58+02	x11	libXres	\N	libXres-1.2.0_2
2020-08-18 08:27:58+02	2020-08-18 08:28:14+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-08-18 08:20:02+02	2020-08-18 08:35:51+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-18 08:28:14+02	2020-08-18 08:37:09+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-18 08:35:52+02	2020-08-18 08:37:21+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-18 08:37:21+02	2020-08-18 08:37:38+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-08-18 08:37:38+02	2020-08-18 08:38:04+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-08-18 08:38:07+02	2020-08-18 08:39:04+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-18 08:39:05+02	2020-08-18 08:39:44+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.20
2020-08-18 08:39:45+02	2020-08-18 08:40:25+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.20
2020-08-18 08:40:26+02	2020-08-18 08:42:10+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-18 08:42:12+02	2020-08-18 08:43:36+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-18 08:43:37+02	2020-08-18 08:44:57+02	sysutils	policykit	\N	policykit-0.9_10
2020-08-18 08:44:58+02	2020-08-18 08:47:16+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-18 08:47:16+02	2020-08-18 08:48:32+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-18 08:48:33+02	2020-08-18 08:51:13+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-08-18 08:37:10+02	2020-08-18 08:52:43+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-08-18 08:51:15+02	2020-08-18 08:52:57+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.1
2020-08-18 08:52:43+02	2020-08-18 08:54:26+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-08-18 08:54:27+02	2020-08-18 08:55:24+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-08-18 08:55:25+02	2020-08-18 08:57:02+02	sysutils	hal	\N	hal-0.5.14_34
2020-08-18 08:57:05+02	2020-08-18 08:58:01+02	www	py-django22	py37	py37-django22-2.2.15
2020-08-18 08:52:58+02	2020-08-18 08:58:25+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-18 08:58:01+02	2020-08-18 08:59:32+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-18 08:59:33+02	2020-08-18 09:02:00+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-18 08:58:26+02	2020-08-18 09:03:38+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_3,1
2020-08-18 09:02:01+02	2020-08-18 09:03:48+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-08-18 09:03:39+02	2020-08-18 09:13:21+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-18 09:13:22+02	2020-08-18 09:17:18+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-08-18 09:17:18+02	2020-08-18 09:19:34+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-08-18 09:19:37+02	2020-08-18 09:20:59+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-18 09:03:48+02	2020-08-18 09:21:29+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-18 09:21:30+02	2020-08-18 09:21:51+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-08-18 09:21:00+02	2020-08-18 09:24:34+02	graphics	poppler-glib	\N	poppler-glib-20.08.0_1
2020-08-18 09:21:52+02	2020-08-18 09:26:07+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-08-18 09:26:08+02	2020-08-18 09:27:23+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-08-18 09:24:35+02	2020-08-18 09:27:46+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-08-18 09:27:23+02	2020-08-18 09:28:44+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-08-18 09:28:45+02	2020-08-18 09:30:13+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-18 09:27:47+02	2020-08-18 09:31:21+02	graphics	glew	\N	glew-2.2.0
2020-08-18 09:31:24+02	2020-08-18 09:32:37+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-08-18 09:32:38+02	2020-08-18 09:34:20+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-18 09:34:21+02	2020-08-18 09:35:54+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-18 09:35:55+02	2020-08-18 09:40:13+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-18 09:40:13+02	2020-08-18 09:42:20+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-08-18 09:42:21+02	2020-08-18 09:46:05+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-08-18 09:46:06+02	2020-08-18 09:49:32+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-18 09:49:33+02	2020-08-18 09:51:22+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-08-18 09:51:23+02	2020-08-18 09:52:17+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-08-18 09:52:17+02	2020-08-18 09:53:56+02	net	remmina	\N	remmina-1.3.9
2020-08-18 09:30:14+02	2020-08-18 09:56:28+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-08-18 09:53:57+02	2020-08-18 09:57:00+02	security	gcr	\N	gcr-3.28.0
2020-08-18 09:57:01+02	2020-08-18 09:57:23+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-08-18 09:57:44+02	2020-08-18 09:59:24+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-08-18 09:56:29+02	2020-08-18 10:05:15+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-08-18 10:05:15+02	2020-08-18 10:06:21+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-18 10:06:21+02	2020-08-18 10:07:26+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-18 09:59:24+02	2020-08-18 10:15:16+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-18 10:07:26+02	2020-08-18 10:18:21+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-18 10:18:22+02	2020-08-18 10:18:39+02	x11	luit	\N	luit-20190106_1
2020-08-18 10:18:39+02	2020-08-18 10:19:43+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-18 10:19:44+02	2020-08-18 10:21:54+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-08-18 10:21:55+02	2020-08-18 10:23:33+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-08-18 10:23:33+02	2020-08-18 10:25:22+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-18 10:15:17+02	2020-08-18 10:36:52+02	net	openmpi	\N	openmpi-4.0.4
2020-08-18 08:19:06+02	2020-08-18 10:51:55+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-18 10:25:22+02	2020-08-18 10:53:35+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-18 10:53:35+02	2020-08-18 10:54:24+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-18 10:54:24+02	2020-08-18 10:57:46+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-18 10:57:47+02	2020-08-18 10:58:19+02	print	gl2ps	\N	gl2ps-1.4.0
2020-08-18 10:58:20+02	2020-08-18 11:02:12+02	security	gnupg	\N	gnupg-2.2.21
2020-08-18 11:02:13+02	2020-08-18 11:02:31+02	x11	xauth	\N	xauth-1.1
2020-08-18 11:02:34+02	2020-08-18 11:03:40+02	x11	xterm	\N	xterm-358
2020-08-18 11:03:41+02	2020-08-18 11:05:40+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-08-18 11:05:41+02	2020-08-18 11:08:59+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-18 11:09:00+02	2020-08-18 11:11:43+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-18 11:11:43+02	2020-08-18 11:12:13+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.8_6
2020-08-18 11:12:13+02	2020-08-18 11:18:37+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-08-18 11:18:38+02	2020-08-18 11:24:19+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-18 11:24:19+02	2020-08-18 11:25:14+02	security	gpgme	\N	gpgme-1.14.0
2020-08-18 11:25:15+02	2020-08-18 11:26:13+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-08-18 11:26:14+02	2020-08-18 11:28:27+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-08-18 10:51:56+02	2020-08-18 11:28:49+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-18 11:28:50+02	2020-08-18 11:32:13+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-18 11:28:28+02	2020-08-18 11:39:52+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-18 11:39:53+02	2020-08-18 11:40:13+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-08-18 11:40:13+02	2020-08-18 11:41:56+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-18 11:41:59+02	2020-08-18 11:43:39+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-08-18 11:43:39+02	2020-08-18 11:54:01+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-18 11:54:02+02	2020-08-18 11:54:51+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-08-18 11:54:51+02	2020-08-18 11:55:17+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-08-18 11:55:18+02	2020-08-18 11:56:43+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-18 11:56:44+02	2020-08-18 11:57:59+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-18 11:57:59+02	2020-08-18 11:59:21+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-18 11:59:23+02	2020-08-18 12:04:53+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-08-18 12:04:54+02	2020-08-18 12:08:04+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-18 12:08:05+02	2020-08-18 12:09:15+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-08-18 12:09:16+02	2020-08-18 12:10:34+02	sysutils	upower	\N	upower-0.99.11
2020-08-18 10:36:53+02	2020-08-18 12:12:29+02	math	vtk8	\N	vtk8-8.2.0
2020-08-18 12:12:30+02	2020-08-18 14:10:53+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-18 14:10:57+02	2020-08-18 14:11:17+02	x11	appres	\N	appres-1.0.5
2020-08-18 14:11:17+02	2020-08-18 14:11:46+02	x11	bitmap	\N	bitmap-1.0.9
2020-08-18 14:11:46+02	2020-08-18 14:13:38+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-08-18 14:13:39+02	2020-08-18 14:14:32+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-08-18 11:32:14+02	2020-08-18 14:17:48+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-18 14:17:54+02	2020-08-18 14:18:12+02	x11	sessreg	\N	sessreg-1.1.2
2020-08-18 14:18:14+02	2020-08-18 14:18:36+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-08-18 14:14:33+02	2020-08-18 14:18:41+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-18 14:18:36+02	2020-08-18 14:18:51+02	x11	smproxy	\N	smproxy-1.0.6
2020-08-18 14:18:42+02	2020-08-18 14:19:05+02	x11	x11perf	\N	x11perf-1.6.1
2020-08-18 14:18:51+02	2020-08-18 14:19:11+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-08-18 14:19:12+02	2020-08-18 14:19:27+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-08-18 14:19:06+02	2020-08-18 14:19:32+02	x11	xcalc	\N	xcalc-1.1.0
2020-08-18 14:19:28+02	2020-08-18 14:19:43+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-08-18 14:19:37+02	2020-08-18 14:19:51+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-08-18 14:19:44+02	2020-08-18 14:19:59+02	x11	xev	\N	xev-1.2.4
2020-08-18 14:19:52+02	2020-08-18 14:20:05+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-08-18 14:20:00+02	2020-08-18 14:20:14+02	x11	xgamma	\N	xgamma-1.0.6
2020-08-18 14:20:05+02	2020-08-18 14:20:24+02	x11	xgc	\N	xgc-1.0.5
2020-08-18 14:20:14+02	2020-08-18 14:20:33+02	x11	xhost	\N	xhost-1.0.8
2020-08-18 14:20:25+02	2020-08-18 14:20:39+02	x11	xinit	\N	xinit-1.4.1,1
2020-08-18 14:20:33+02	2020-08-18 14:20:48+02	x11	xinput	\N	xinput-1.6.3
2020-08-18 14:20:40+02	2020-08-18 14:20:56+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-08-18 14:20:48+02	2020-08-18 14:21:10+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-08-18 14:20:57+02	2020-08-18 14:21:12+02	x11	xkill	\N	xkill-1.0.5
2020-08-18 14:21:10+02	2020-08-18 14:21:27+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-08-18 14:21:13+02	2020-08-18 14:21:28+02	x11	xmessage	\N	xmessage-1.0.5
2020-08-18 14:21:27+02	2020-08-18 14:21:41+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-08-18 14:21:29+02	2020-08-18 14:21:43+02	x11	xpr	\N	xpr-1.0.5
2020-08-18 14:21:45+02	2020-08-18 14:22:02+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-08-18 14:21:41+02	2020-08-18 14:22:02+02	x11	xrandr	\N	xrandr-1.5.1
2020-08-18 14:22:03+02	2020-08-18 14:22:15+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-08-18 14:22:03+02	2020-08-18 14:22:16+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-08-18 14:22:16+02	2020-08-18 14:22:32+02	x11	xwd	\N	xwd-1.0.7
2020-08-18 14:22:16+02	2020-08-18 14:22:33+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-08-18 14:22:33+02	2020-08-18 14:22:47+02	x11	xwud	\N	xwud-1.0.5
2020-08-18 14:22:33+02	2020-08-18 14:22:53+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-08-18 14:22:53+02	2020-08-18 14:24:10+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-18 14:22:48+02	2020-08-18 14:24:15+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-18 14:24:10+02	2020-08-18 14:25:25+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-18 14:24:16+02	2020-08-18 14:25:27+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-18 14:25:26+02	2020-08-18 14:26:31+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-18 14:25:28+02	2020-08-18 14:27:31+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.20181203_1
2020-08-18 14:26:34+02	2020-08-18 14:27:45+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-18 14:27:31+02	2020-08-18 14:28:34+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-18 14:27:46+02	2020-08-18 14:28:57+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-08-18 14:28:58+02	2020-08-18 14:29:31+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-18 14:28:34+02	2020-08-18 14:30:02+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-08-18 14:30:03+02	2020-08-18 14:30:25+02	x11-wm	twm	\N	twm-1.0.11
2020-08-18 14:30:25+02	2020-08-18 14:35:46+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-08-18 14:35:48+02	2020-08-18 14:39:31+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-18 14:29:31+02	2020-08-18 14:40:07+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-18 14:39:32+02	2020-08-18 14:40:27+02	devel	librest	\N	rest-0.7.93
2020-08-18 14:40:27+02	2020-08-18 14:41:22+02	graphics	poppler-utils	\N	poppler-utils-20.08.0_1
2020-08-18 14:41:23+02	2020-08-18 14:48:07+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-18 14:48:08+02	2020-08-18 14:48:33+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-08-18 14:48:34+02	2020-08-18 14:49:08+02	print	libotf	\N	libotf-0.9.16
2020-08-18 14:49:09+02	2020-08-18 14:52:06+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-08-18 14:52:06+02	2020-08-18 14:53:17+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-18 14:53:17+02	2020-08-18 14:54:31+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-18 14:54:32+02	2020-08-18 14:56:12+02	sysutils	bsdisks	\N	bsdisks-0.20
2020-08-18 14:56:13+02	2020-08-18 14:59:52+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-08-18 14:59:55+02	2020-08-18 15:00:23+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-08-18 15:00:26+02	2020-08-18 15:02:17+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-18 15:02:18+02	2020-08-18 15:02:56+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-08-18 15:02:56+02	2020-08-18 15:03:18+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-08-18 15:03:18+02	2020-08-18 15:04:45+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-18 15:04:48+02	2020-08-18 15:08:36+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-18 15:08:37+02	2020-08-18 15:10:59+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_3,1
2020-08-18 14:40:08+02	2020-08-18 15:11:05+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-18 15:11:06+02	2020-08-18 15:13:02+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-18 15:13:03+02	2020-08-18 15:14:22+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-18 15:11:00+02	2020-08-18 15:15:48+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-18 15:14:24+02	2020-08-18 15:16:08+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-18 15:15:49+02	2020-08-18 15:17:17+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-18 15:16:09+02	2020-08-18 15:17:35+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-18 15:17:17+02	2020-08-18 15:19:02+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-08-18 15:17:35+02	2020-08-18 15:20:06+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-08-18 15:20:06+02	2020-08-18 15:20:40+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-08-18 15:19:03+02	2020-08-18 15:21:36+02	graphics	gegl	\N	gegl-0.4.24_4
2020-08-18 15:20:43+02	2020-08-18 15:21:42+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-08-18 15:21:37+02	2020-08-18 15:21:58+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-08-18 15:21:59+02	2020-08-18 15:27:10+02	mail	mailutils	\N	mailutils-3.10
2020-08-18 15:27:11+02	2020-08-18 15:29:17+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-18 15:29:18+02	2020-08-18 15:29:37+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-08-18 15:29:38+02	2020-08-18 15:30:56+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-18 15:21:43+02	2020-08-18 15:31:01+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-18 15:30:56+02	2020-08-18 15:31:56+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-18 15:31:56+02	2020-08-18 15:33:08+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-08-18 15:31:02+02	2020-08-18 15:33:21+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-18 15:33:22+02	2020-08-18 15:34:28+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-18 15:33:11+02	2020-08-18 15:35:08+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-18 15:34:28+02	2020-08-18 15:35:36+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-08-18 15:35:36+02	2020-08-18 15:37:00+02	x11	zenity	\N	zenity-3.32.0
2020-08-18 15:37:00+02	2020-08-18 15:38:01+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-08-18 15:35:08+02	2020-08-18 15:38:34+02	x11	xorg	\N	xorg-7.7_3
2020-08-18 15:38:02+02	2020-08-18 15:38:47+02	devel	apache-ant	\N	apache-ant-1.10.8
2020-08-18 15:38:47+02	2020-08-18 15:41:26+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-18 15:41:27+02	2020-08-18 15:43:20+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-18 15:43:20+02	2020-08-18 15:44:24+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-08-18 15:38:34+02	2020-08-18 15:45:14+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-18 15:44:24+02	2020-08-18 15:45:36+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-18 15:45:14+02	2020-08-18 15:46:18+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-18 15:45:36+02	2020-08-18 15:46:32+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-18 15:46:32+02	2020-08-18 15:46:49+02	devel	xxhash	\N	xxhash-0.8.0
2020-08-18 15:46:19+02	2020-08-18 15:49:43+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-18 15:46:52+02	2020-08-18 15:52:15+02	dns	bind-tools	\N	bind-tools-9.16.5
2020-08-18 15:49:43+02	2020-08-18 15:55:30+02	editors	emacs	full	emacs-27.1,3
2020-08-18 15:55:31+02	2020-08-18 15:55:57+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-08-18 15:52:16+02	2020-08-18 15:59:23+02	graphics	Coin	\N	Coin-4.0.0_1
2020-08-18 15:59:24+02	2020-08-18 16:00:27+02	graphics	glfw	\N	glfw-3.3.2
2020-08-18 16:00:27+02	2020-08-18 16:02:22+02	graphics	movit	\N	movit-1.6.2_2
2020-08-18 15:55:58+02	2020-08-18 16:07:30+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-08-18 16:07:31+02	2020-08-18 16:08:49+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-08-18 16:08:49+02	2020-08-18 16:09:15+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-08-18 16:09:16+02	2020-08-18 16:09:48+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-08-18 12:10:35+02	2020-08-18 16:10:34+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-18 16:10:40+02	2020-08-18 16:11:53+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-18 16:02:23+02	2020-08-18 16:15:40+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-18 16:15:41+02	2020-08-18 16:16:09+02	net	gssdp	\N	gssdp-1.2.3
2020-08-18 16:16:10+02	2020-08-18 16:17:19+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-08-18 16:17:20+02	2020-08-18 16:18:28+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-08-18 16:11:53+02	2020-08-18 16:18:54+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-08-18 16:18:29+02	2020-08-18 16:24:21+02	print	fontforge	\N	fontforge-20190801_2
2020-08-18 16:24:22+02	2020-08-18 16:25:37+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-18 16:25:38+02	2020-08-18 16:28:19+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-08-18 16:18:55+02	2020-08-18 16:32:40+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-08-18 16:32:40+02	2020-08-18 16:33:00+02	www	py-hyperlink	py37	py37-hyperlink-20.0.0
2020-08-18 16:33:00+02	2020-08-18 16:35:26+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-18 16:35:27+02	2020-08-18 16:37:25+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-18 16:37:26+02	2020-08-18 16:38:48+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-18 16:38:49+02	2020-08-18 16:39:52+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-18 16:39:52+02	2020-08-18 16:41:33+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-18 16:41:34+02	2020-08-18 17:00:21+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-08-18 17:00:22+02	2020-08-18 17:02:44+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-18 17:02:44+02	2020-08-18 17:04:45+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-18 17:04:46+02	2020-08-18 17:06:03+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-18 17:06:04+02	2020-08-18 17:07:59+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-18 17:08:00+02	2020-08-18 17:10:43+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-18 17:10:46+02	2020-08-18 17:15:38+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-18 17:15:39+02	2020-08-18 17:17:32+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-18 17:17:33+02	2020-08-18 17:19:20+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-08-18 17:19:21+02	2020-08-18 17:22:23+02	x11-wm	marco	\N	marco-1.22.4
2020-08-18 17:22:24+02	2020-08-18 17:24:01+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-18 17:24:02+02	2020-08-18 17:25:12+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-08-18 17:25:13+02	2020-08-18 17:28:37+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-05 19:26:17+02	2020-10-05 19:28:31+02	x11-fm	caja	\N	caja-1.24.0
2020-08-18 17:28:38+02	2020-08-18 17:30:38+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-08-18 17:30:39+02	2020-08-18 17:36:19+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-18 17:36:22+02	2020-08-18 17:40:09+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-18 17:40:10+02	2020-08-18 17:52:38+02	devel	ncurses	\N	ncurses-6.2.20200816
2020-08-18 17:52:39+02	2020-08-18 17:54:10+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-08-18 17:54:10+02	2020-08-18 18:33:08+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-18 18:33:12+02	2020-08-18 18:37:41+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-18 18:37:42+02	2020-08-18 18:39:32+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-18 18:39:32+02	2020-08-18 18:39:57+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-08-18 18:39:57+02	2020-08-18 18:42:16+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-18 18:42:17+02	2020-08-18 18:42:49+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-08-18 18:42:50+02	2020-08-18 18:43:19+02	graphics	libgxps	\N	libgxps-0.3.1
2020-08-18 18:43:20+02	2020-08-18 18:43:49+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-08-18 18:43:50+02	2020-08-18 18:45:17+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-08-18 18:45:17+02	2020-08-18 18:45:53+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-18 18:45:54+02	2020-08-18 18:46:51+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-18 18:46:52+02	2020-08-18 18:47:49+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-18 16:09:49+02	2020-08-18 19:05:43+02	lang	ghc	\N	ghc-8.10.1
2020-08-18 18:47:50+02	2020-08-18 19:09:02+02	math	R	\N	R-4.0.2
2020-08-18 19:09:03+02	2020-08-18 19:09:42+02	misc	box2d	\N	Box2D-2.3.1
2020-08-18 19:09:42+02	2020-08-18 19:21:04+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-18 19:05:44+02	2020-08-18 19:21:59+02	devel	hs-cabal-install	\N	hs-cabal-install-3.2.0.0
2020-08-18 19:21:05+02	2020-08-18 19:22:07+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-08-18 19:22:03+02	2020-08-18 19:22:41+02	multimedia	libmatroska	\N	libmatroska-1.6.2
2020-08-18 19:22:08+02	2020-08-18 19:23:47+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-18 19:23:48+02	2020-08-18 19:24:50+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-18 19:24:52+02	2020-08-18 19:26:12+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-18 19:22:42+02	2020-08-18 19:26:57+02	net	freerdp	\N	freerdp-2.2.0
2020-08-18 19:26:13+02	2020-08-18 19:27:23+02	net	rsync	\N	rsync-3.2.3
2020-08-18 19:27:03+02	2020-08-18 19:28:15+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-18 19:28:16+02	2020-08-18 19:28:49+02	print	libspectre	\N	libspectre-0.2.9
2020-08-18 19:28:50+02	2020-08-18 19:31:39+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-18 19:27:24+02	2020-08-18 19:32:26+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-18 19:31:40+02	2020-08-18 19:39:42+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-18 19:32:27+02	2020-08-18 19:40:09+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-18 19:40:13+02	2020-08-18 19:40:41+02	sysutils	hal-info	\N	hal-info-20091130
2020-08-18 19:39:45+02	2020-08-18 19:41:07+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-08-18 19:41:08+02	2020-08-18 19:42:17+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-18 19:40:42+02	2020-08-18 19:44:49+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-18 19:42:18+02	2020-08-18 19:45:34+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-18 19:44:51+02	2020-08-18 19:47:54+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-18 19:45:35+02	2020-08-18 19:49:36+02	textproc	libstaroffice	\N	libstaroffice-0.0.7
2020-08-18 19:49:36+02	2020-08-18 19:50:50+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-18 19:47:55+02	2020-08-18 19:52:17+02	textproc	libwps	\N	libwps-0.4.11
2020-08-18 19:50:56+02	2020-08-18 19:52:29+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-18 19:52:18+02	2020-08-18 19:53:09+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-18 19:53:09+02	2020-08-18 19:53:20+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-08-18 19:53:20+02	2020-08-18 19:53:32+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-08-18 19:52:30+02	2020-08-18 19:54:08+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-18 19:54:08+02	2020-08-18 19:55:47+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-18 19:53:33+02	2020-08-18 19:56:17+02	x11-servers	xephyr	\N	xephyr-1.20.8_3,1
2020-08-18 19:55:48+02	2020-08-18 19:56:39+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-08-18 19:56:18+02	2020-08-18 19:57:23+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-18 19:56:39+02	2020-08-18 19:57:48+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-18 19:57:24+02	2020-08-18 19:58:53+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-18 19:57:49+02	2020-08-18 19:59:34+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-18 19:59:35+02	2020-08-18 20:03:02+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-18 20:03:03+02	2020-08-18 20:03:34+02	audio	dssi	\N	dssi-1.1.1_8
2020-08-18 20:03:35+02	2020-08-18 20:04:34+02	audio	faudio	\N	FAudio-20.08
2020-08-18 20:04:35+02	2020-08-18 20:06:29+02	audio	mate-media	\N	mate-media-1.24.1
2020-08-18 20:06:30+02	2020-08-18 20:06:57+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-08-18 20:06:58+02	2020-08-18 20:08:37+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-18 19:58:53+02	2020-08-18 20:09:02+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-18 20:08:38+02	2020-08-18 20:10:19+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-18 20:10:21+02	2020-08-18 20:11:39+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-18 20:11:40+02	2020-08-18 20:12:55+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-18 20:09:03+02	2020-08-18 20:13:28+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-18 20:13:29+02	2020-08-18 20:13:56+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-08-18 20:12:55+02	2020-08-18 20:14:02+02	devel	adacurses	\N	adacurses-20180127
2020-08-18 20:13:57+02	2020-08-18 20:14:18+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-08-18 20:14:19+02	2020-08-18 20:15:25+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-08-18 20:14:02+02	2020-08-18 20:15:50+02	devel	gradle5	\N	gradle5-5.0
2020-08-18 20:15:25+02	2020-08-18 20:16:18+02	devel	ode	\N	ode-0.13_4,1
2020-08-18 20:15:51+02	2020-08-18 20:16:32+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-08-18 20:16:19+02	2020-08-18 20:17:28+02	devel	py-game	py37	py37-game-1.9.6_1
2020-08-18 20:16:33+02	2020-08-18 20:18:45+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-18 20:17:29+02	2020-08-18 20:53:18+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-18 20:53:19+02	2020-08-18 21:00:47+02	editors	pluma	\N	pluma-1.24.0
2020-08-18 21:00:48+02	2020-08-18 21:04:01+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-08-18 21:04:02+02	2020-08-18 21:06:37+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-18 21:06:38+02	2020-08-18 21:14:41+02	french	med	\N	fr-med-4.0.0
2020-08-18 21:14:42+02	2020-08-18 21:21:46+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-08-18 21:21:48+02	2020-08-18 21:26:43+02	graphics	atril	\N	atril-1.22.3_8
2020-08-18 21:26:44+02	2020-08-18 21:30:08+02	graphics	eom	\N	eom-1.24.0
2020-08-18 21:30:11+02	2020-08-18 21:32:59+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-18 21:33:00+02	2020-08-18 21:33:33+02	graphics	gle	\N	gle-3.1.0_7
2020-08-18 21:33:34+02	2020-08-18 21:34:15+02	graphics	gltt	\N	gltt-2.5.2_14
2020-08-18 21:34:15+02	2020-08-18 21:36:34+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-18 21:36:35+02	2020-08-18 21:47:13+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-08-18 21:47:20+02	2020-08-18 21:59:42+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-08-18 21:59:43+02	2020-08-18 22:07:22+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-18 22:07:28+02	2020-08-18 22:11:16+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-08-18 22:11:18+02	2020-08-18 22:46:21+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-18 22:46:24+02	2020-08-18 23:40:16+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-08-18 23:40:18+02	2020-08-18 23:52:04+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-18 23:52:05+02	2020-08-18 23:54:52+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-18 23:54:53+02	2020-08-18 23:56:32+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-18 23:56:33+02	2020-08-19 00:08:08+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-08-19 00:08:10+02	2020-08-19 00:08:35+02	misc	xosd	\N	xosd-2.2.14_3
2020-08-19 00:08:35+02	2020-08-19 00:11:37+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-19 00:11:42+02	2020-08-19 00:12:39+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-08-19 00:12:40+02	2020-08-19 00:15:48+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-19 00:15:48+02	2020-08-19 00:29:05+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-19 00:29:06+02	2020-08-19 00:30:59+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-19 00:31:00+02	2020-08-19 00:32:37+02	net	ndpi	\N	ndpi-3.2.d20200721,1
2020-08-19 00:32:37+02	2020-08-19 00:33:42+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-19 00:33:43+02	2020-08-19 00:34:54+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-19 00:34:55+02	2020-08-19 00:36:03+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-19 00:36:04+02	2020-08-19 00:37:20+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-19 00:37:21+02	2020-08-19 00:38:28+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-19 00:38:29+02	2020-08-19 00:39:32+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-19 00:39:33+02	2020-08-19 00:41:03+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-19 00:41:03+02	2020-08-19 00:43:02+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-19 00:43:03+02	2020-08-19 00:44:11+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-19 00:44:12+02	2020-08-19 00:45:54+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-08-19 00:45:55+02	2020-08-19 00:46:13+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-08-19 00:46:14+02	2020-08-19 00:47:59+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-19 00:48:00+02	2020-08-19 01:01:43+02	print	lilypond	\N	lilypond-2.20.0
2020-08-19 01:01:45+02	2020-08-19 01:10:39+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-19 01:10:40+02	2020-08-19 01:18:41+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-19 01:18:42+02	2020-08-19 01:21:30+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-19 01:21:32+02	2020-08-19 01:32:50+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-08-19 01:32:51+02	2020-08-19 01:58:59+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-19 01:59:03+02	2020-08-19 01:59:48+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-19 01:59:49+02	2020-08-19 02:02:45+02	sysutils	intel-pcm	\N	intel-pcm-202007_1
2020-08-19 02:02:46+02	2020-08-19 02:03:56+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-19 02:03:57+02	2020-08-19 02:06:23+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-19 02:06:24+02	2020-08-19 02:09:12+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-19 02:09:13+02	2020-08-19 02:11:44+02	textproc	fop	\N	fop-2.3
2020-08-19 02:11:45+02	2020-08-19 02:13:06+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-18 20:18:47+02	2020-08-19 03:42:39+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-08-19 03:42:41+02	2020-08-19 03:43:57+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-19 03:43:58+02	2020-08-19 03:44:12+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-08-19 03:44:13+02	2020-08-19 03:44:27+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-08-19 03:44:28+02	2020-08-19 03:44:45+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-08-19 03:44:46+02	2020-08-19 03:46:47+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-19 03:46:48+02	2020-08-19 03:48:32+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-19 03:48:32+02	2020-08-19 03:54:33+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-19 02:13:07+02	2020-08-19 03:55:05+02	textproc	hs-pandoc	\N	hs-pandoc-2.10
2020-08-19 03:54:33+02	2020-08-19 03:55:44+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-19 03:55:06+02	2020-08-19 03:56:25+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-19 03:55:45+02	2020-08-19 03:57:55+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-19 03:56:26+02	2020-08-19 03:59:36+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-19 03:57:56+02	2020-08-19 03:59:57+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-08-19 03:59:58+02	2020-08-19 04:00:41+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-19 03:59:37+02	2020-08-19 04:00:49+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-19 04:00:42+02	2020-08-19 04:10:30+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-19 04:10:31+02	2020-08-19 04:10:45+02	astro	xearth	\N	xearth-1.2
2020-08-19 04:10:46+02	2020-08-19 04:13:51+02	astro	xephem	\N	xephem-3.7.7_5
2020-08-19 04:00:50+02	2020-08-19 04:31:48+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-19 04:31:50+02	2020-08-19 04:32:12+02	audio	xmix	\N	xmix-2.1_3
2020-08-19 04:32:13+02	2020-08-19 04:32:41+02	benchmarks	iperf3	\N	iperf3-3.9
2020-08-19 04:13:52+02	2020-08-19 04:43:49+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-19 04:43:51+02	2020-08-19 04:49:51+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-19 04:49:52+02	2020-08-19 04:52:44+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-19 04:52:46+02	2020-08-19 06:30:40+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-19 06:30:42+02	2020-08-19 06:33:08+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-19 06:33:09+02	2020-08-19 06:35:21+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-19 06:35:23+02	2020-08-19 06:35:39+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-08-19 06:35:41+02	2020-08-19 07:07:06+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-19 07:07:07+02	2020-08-19 07:13:04+02	deskutils	calibre	\N	calibre-4.22.0_1
2020-08-19 07:13:05+02	2020-08-19 07:14:28+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-19 07:14:29+02	2020-08-19 07:14:45+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-08-19 07:14:46+02	2020-08-19 07:15:11+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-08-19 04:32:42+02	2020-08-19 07:19:32+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-19 07:19:35+02	2020-08-19 07:21:37+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-19 07:15:12+02	2020-08-19 09:47:41+02	devel	RStudio	\N	RStudio-1.3.1056
2020-08-19 09:47:42+02	2020-08-19 10:49:08+02	devel	ghidra	\N	ghidra-9.1
2020-08-19 10:49:10+02	2020-08-19 10:49:31+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-08-19 10:49:32+02	2020-08-19 10:58:14+02	devel	subversion	\N	subversion-1.14.0
2020-08-19 10:58:15+02	2020-08-19 11:04:23+02	dns	bind916	\N	bind916-9.16.5_1
2020-08-19 11:04:25+02	2020-08-19 12:39:08+02	editors	codelite	\N	codelite-12.0_4
2020-08-19 12:39:11+02	2020-08-19 12:42:20+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-08-19 12:42:21+02	2020-08-19 12:48:04+02	editors	vim	\N	vim-8.2.1382
2020-08-19 07:21:38+02	2020-08-19 13:06:24+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-19 12:48:06+02	2020-08-19 13:50:21+02	emulators	qemu	\N	qemu-4.2.1
2020-08-19 13:50:24+02	2020-08-19 14:15:59+02	emulators	wine	\N	wine-5.0.2,1
2020-08-19 14:16:00+02	2020-08-19 14:17:11+02	emulators	x48	\N	x48-0.6.4_1
2020-08-19 13:06:30+02	2020-08-19 14:26:31+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-08-19 14:26:33+02	2020-08-19 14:26:58+02	games	KnightCap	\N	knightcap-3.6_8
2020-08-19 14:26:59+02	2020-08-19 14:27:16+02	games	affenspiel	\N	affenspiel-1.0_2
2020-08-19 14:27:18+02	2020-08-19 14:27:43+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-08-19 14:27:44+02	2020-08-19 14:30:44+02	games	alienarena	\N	alienarena-2013.766_9
2020-08-19 14:30:45+02	2020-08-19 14:31:17+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-08-19 14:31:18+02	2020-08-19 14:33:07+02	games	battletanks	\N	btanks-0.9.8083_9
2020-08-19 14:17:11+02	2020-08-19 14:33:13+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-19 14:33:07+02	2020-08-19 14:33:28+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-08-19 14:33:15+02	2020-08-19 14:34:02+02	games	monsterz	\N	monsterz-0.7.1_15
2020-08-19 14:33:29+02	2020-08-19 14:34:10+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-19 14:34:02+02	2020-08-19 14:34:45+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-19 14:34:11+02	2020-08-19 14:34:59+02	games	xblast	\N	xblast-2.10.4_17
2020-08-19 14:34:46+02	2020-08-19 14:35:31+02	games	xboard	\N	xboard-4.9.1_1
2020-08-19 14:35:00+02	2020-08-19 14:35:38+02	games	xcowsay	\N	xcowsay-1.5
2020-08-19 14:35:39+02	2020-08-19 14:35:59+02	games	xminesweep	\N	xminesweep-3.0_2
2020-08-19 14:35:31+02	2020-08-19 14:36:04+02	games	xevil	\N	xevil-2.02.r2_2
2020-08-19 14:36:05+02	2020-08-19 14:38:07+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-18 16:28:20+02	2020-08-19 14:38:50+02	www	chromium	\N	chromium-84.0.4147.125
2020-08-19 14:38:08+02	2020-08-19 14:39:27+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-19 14:38:57+02	2020-08-19 14:40:21+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-19 14:40:22+02	2020-08-19 14:42:02+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-19 14:39:27+02	2020-08-19 14:57:02+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-19 14:36:01+02	2020-08-19 14:58:16+02	graphics	blender	\N	blender-2.83.2_1
2020-08-19 14:58:17+02	2020-08-19 14:59:54+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-19 14:57:05+02	2020-08-19 15:01:10+02	graphics	wings	\N	wings-2.2.5
2020-08-19 15:01:10+02	2020-08-19 15:03:09+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-19 14:42:03+02	2020-08-19 15:06:01+02	graphics	krita	\N	krita-4.3.0_4
2020-08-19 14:59:55+02	2020-08-19 15:24:37+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-08-19 15:03:11+02	2020-08-19 15:45:33+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.7
2020-08-19 15:45:34+02	2020-08-19 15:45:53+02	mail	bbmail	\N	bbmail-0.9.3
2020-08-19 15:06:06+02	2020-08-19 15:46:46+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-19 15:46:47+02	2020-08-19 15:47:44+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-19 15:47:45+02	2020-08-19 15:48:00+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-08-19 15:48:01+02	2020-08-19 15:48:29+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-08-19 15:45:54+02	2020-08-19 15:51:52+02	math	maxima	\N	maxima-5.44.0_3
2020-08-19 15:51:52+02	2020-08-19 15:52:12+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-08-19 15:52:13+02	2020-08-19 15:52:26+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-08-19 15:48:30+02	2020-08-19 15:53:03+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p5
2020-08-19 15:52:27+02	2020-08-19 15:54:55+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-19 15:53:04+02	2020-08-19 15:55:46+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-19 15:54:55+02	2020-08-19 15:59:40+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-19 15:59:41+02	2020-08-19 16:00:25+02	net	rdesktop	\N	rdesktop-1.8.6
2020-08-19 15:55:47+02	2020-08-19 16:02:03+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-19 16:02:04+02	2020-08-19 16:02:26+02	net	vnstat	\N	vnstat-2.6_1
2020-08-19 16:00:26+02	2020-08-19 16:02:26+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-19 16:02:27+02	2020-08-19 16:03:17+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-19 16:02:26+02	2020-08-19 16:09:26+02	net	wireshark	\N	wireshark-3.2.6
2020-08-19 16:09:27+02	2020-08-19 16:10:10+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-19 15:24:38+02	2020-08-19 16:10:37+02	lang	mono	\N	mono-5.10.1.57_3
2020-08-19 16:10:11+02	2020-08-19 16:10:53+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-19 16:10:38+02	2020-08-19 16:11:19+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-19 16:10:53+02	2020-08-19 16:11:34+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-19 16:11:19+02	2020-08-19 16:11:58+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-19 16:11:34+02	2020-08-19 16:12:14+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-19 16:11:59+02	2020-08-19 16:12:39+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-19 16:12:15+02	2020-08-19 16:13:18+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-19 16:12:39+02	2020-08-19 16:13:24+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-19 16:13:25+02	2020-08-19 16:13:33+02	net-mgmt	nstat	\N	nstat-1.0_3
2020-08-19 16:13:18+02	2020-08-19 16:14:02+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-19 16:13:34+02	2020-08-19 16:14:30+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-19 16:14:02+02	2020-08-19 16:15:04+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-19 16:15:04+02	2020-08-19 16:15:40+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-19 16:15:41+02	2020-08-19 16:16:23+02	print	gv	\N	gv-3.7.4_5
2020-08-19 16:14:33+02	2020-08-19 16:17:19+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-19 16:03:17+02	2020-08-19 16:19:37+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-08-19 16:16:23+02	2020-08-19 16:23:09+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-19 16:19:37+02	2020-08-19 16:25:38+02	print	texlive-full	\N	texlive-full-20150521
2020-08-19 16:23:09+02	2020-08-19 16:26:45+02	security	clamav	\N	clamav-0.102.4,1
2020-08-19 16:26:45+02	2020-08-19 16:26:59+02	security	nyx	\N	nyx-2.1.0_1
2020-08-19 16:27:00+02	2020-08-19 16:28:14+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-19 16:28:15+02	2020-08-19 16:29:37+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-19 16:29:40+02	2020-08-19 16:30:41+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-19 16:25:39+02	2020-08-19 16:30:58+02	security	klee	\N	klee-2.1_1
2020-08-19 16:30:42+02	2020-08-19 16:31:44+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-19 16:30:58+02	2020-08-19 16:32:20+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-19 16:17:20+02	2020-08-19 16:32:47+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-08-19 16:31:45+02	2020-08-19 16:33:01+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-08-19 16:33:02+02	2020-08-19 16:33:14+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-08-19 16:33:15+02	2020-08-19 16:33:28+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-08-19 16:32:48+02	2020-08-19 16:33:40+02	www	links	\N	links-2.20.2,1
2020-08-19 16:33:29+02	2020-08-19 16:33:42+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-08-19 16:33:41+02	2020-08-19 16:33:52+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-08-19 16:33:42+02	2020-08-19 16:33:53+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-08-19 16:33:53+02	2020-08-19 16:34:05+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-08-19 16:33:54+02	2020-08-19 16:34:06+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-08-19 16:34:06+02	2020-08-19 16:34:19+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-08-19 16:34:06+02	2020-08-19 16:34:19+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-08-19 16:34:19+02	2020-08-19 16:34:34+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-08-19 16:34:19+02	2020-08-19 16:35:10+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-19 16:35:11+02	2020-08-19 16:35:21+02	x11	numlockx	\N	numlockx-1.2
2020-08-19 16:35:22+02	2020-08-19 16:35:31+02	x11	sterm	\N	sterm-0.8.4
2020-08-19 16:34:34+02	2020-08-19 16:35:49+02	x11	mate	\N	mate-1.22.2
2020-08-19 16:35:32+02	2020-08-19 16:35:50+02	x11	xclip	\N	xclip-0.13
2020-08-19 16:35:50+02	2020-08-19 16:36:00+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-08-19 16:35:50+02	2020-08-19 16:36:10+02	x11	xdm	\N	xdm-1.1.12_3
2020-08-19 16:36:10+02	2020-08-19 16:36:26+02	x11	xman	\N	xman-1.1.5
2020-08-19 16:36:01+02	2020-08-19 16:36:31+02	x11	xlockmore	\N	xlockmore-5.65
2020-10-05 19:45:11+02	2020-10-05 19:47:09+02	graphics	eom	\N	eom-1.24.1
2020-08-19 16:36:26+02	2020-08-19 16:39:34+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-19 16:39:35+02	2020-08-19 16:40:50+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-19 16:40:50+02	2020-08-19 16:41:10+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-08-19 16:41:10+02	2020-08-19 16:41:26+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-08-19 16:41:26+02	2020-08-19 16:41:57+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-08-19 16:41:58+02	2020-08-19 16:42:20+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-08-19 16:36:31+02	2020-08-19 16:42:25+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-19 16:42:26+02	2020-08-19 16:42:51+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-08-19 16:42:20+02	2020-08-19 16:43:06+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-08-19 16:43:07+02	2020-08-19 16:43:58+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-19 16:43:59+02	2020-08-19 16:44:39+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-19 16:42:52+02	2020-08-19 17:09:45+02	editors	vscode	\N	vscode-1.46.1
2020-08-19 16:32:21+02	2020-08-19 17:24:47+02	www	firefox	\N	firefox-79.0_2,1
2020-08-19 17:24:48+02	2020-08-19 17:41:30+02	local	E590T-localbase	\N	E590T-localbase-2020081500
2020-08-19 17:41:45+02	2020-08-19 17:42:47+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-19 17:41:45+02	2020-08-19 17:46:54+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-19 17:46:55+02	2020-08-19 17:47:02+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-19 22:57:05+02	2020-08-19 23:03:28+02	ports-mgmt	pkg	\N	pkg-1.14.7
2020-08-19 23:31:14+02	2020-08-19 23:31:34+02	devel	py-six	py37	py37-six-1.15.0
2020-08-19 23:31:14+02	2020-08-19 23:31:51+02	devel	py-cffi	py37	py37-cffi-1.14.2
2020-08-19 23:31:37+02	2020-08-19 23:31:56+02	devel	py-packaging	py37	py37-packaging-20.4
2020-08-19 23:31:52+02	2020-08-19 23:33:25+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-08-19 23:33:26+02	2020-08-19 23:33:48+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-08-19 23:31:57+02	2020-08-19 23:33:57+02	devel	check	\N	check-0.15.2
2020-08-19 23:33:52+02	2020-08-19 23:34:15+02	net	py-urllib3	py37	py37-urllib3-1.25.10,1
2020-08-19 23:34:16+02	2020-08-19 23:34:40+02	www	py-requests	py37	py37-requests-2.22.0
2020-08-19 23:34:41+02	2020-08-19 23:35:39+02	textproc	py-sphinx	py37	py37-sphinx-3.1.2,1
2020-08-19 23:33:58+02	2020-08-19 23:35:52+02	devel	libunwind	\N	libunwind-20200331_1
2020-08-19 23:35:40+02	2020-08-19 23:36:05+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-08-19 23:31:14+02	2020-08-19 23:36:43+02	ftp	curl	\N	curl-7.72.0
2020-08-19 23:35:55+02	2020-08-19 23:42:32+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-08-19 23:36:07+02	2020-08-19 23:45:04+02	print	harfbuzz	\N	harfbuzz-2.7.1
2020-08-19 23:42:34+02	2020-08-19 23:47:31+02	graphics	cairo	\N	cairo-1.16.0,2
2020-08-19 23:36:53+02	2020-08-19 23:52:22+02	devel	cmake	\N	cmake-3.18.1
2020-08-19 23:47:33+02	2020-08-19 23:52:49+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-19 23:52:24+02	2020-08-19 23:53:14+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-08-19 23:45:08+02	2020-08-19 23:53:37+02	security	trousers	\N	trousers-0.3.14_3
2020-08-19 23:53:15+02	2020-08-19 23:53:39+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-08-19 23:52:50+02	2020-08-19 23:54:30+02	archivers	zstd	\N	zstd-1.4.5_1
2020-08-19 23:53:41+02	2020-08-19 23:54:55+02	x11	libinput	\N	libinput-1.15.6
2020-08-19 23:54:57+02	2020-08-19 23:57:12+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-19 23:57:12+02	2020-08-19 23:57:53+02	graphics	libGLU	\N	libGLU-9.0.1
2020-08-19 23:57:54+02	2020-08-19 23:58:33+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.1
2020-08-19 23:53:39+02	2020-08-19 23:59:05+02	security	gnutls	\N	gnutls-3.6.14
2020-08-19 23:58:33+02	2020-08-19 23:59:58+02	graphics	poppler	\N	poppler-20.08.0_1
2020-08-19 23:59:06+02	2020-08-20 00:00:18+02	print	cups	\N	cups-2.3.3_1
2020-08-19 23:59:58+02	2020-08-20 00:03:12+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-08-20 00:00:19+02	2020-08-20 00:03:12+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_8
2020-08-19 23:54:33+02	2020-08-20 00:06:23+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-08-20 00:03:13+02	2020-08-20 00:07:19+02	graphics	graphviz	\N	graphviz-2.44.1_1
2020-08-20 00:06:23+02	2020-08-20 00:07:43+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-08-20 00:03:12+02	2020-08-20 00:07:56+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-20 00:07:56+02	2020-08-20 00:09:17+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-08-20 00:07:20+02	2020-08-20 00:09:38+02	lang	vala	\N	vala-0.40.15,1
2020-08-20 00:09:38+02	2020-08-20 00:10:44+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-20 00:09:18+02	2020-08-20 00:10:46+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-08-20 00:07:44+02	2020-08-20 00:12:18+02	net	qt5-network	\N	qt5-network-5.15.0
2020-08-20 00:10:46+02	2020-08-20 00:14:05+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-08-20 00:14:05+02	2020-08-20 00:14:38+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-08-20 00:14:39+02	2020-08-20 00:15:57+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-20 00:10:45+02	2020-08-20 00:18:45+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-20 00:12:19+02	2020-08-20 00:19:22+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-20 00:19:22+02	2020-08-20 00:23:39+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-20 00:18:45+02	2020-08-20 00:30:46+02	devel	protobuf	\N	protobuf-3.12.4,1
2020-08-20 00:30:47+02	2020-08-20 00:31:56+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-08-20 00:15:58+02	2020-08-20 00:32:47+02	lang	ruby27	\N	ruby-2.7.1_1,1
2020-08-20 00:31:57+02	2020-08-20 00:32:54+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-08-20 00:32:48+02	2020-08-20 00:33:49+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-08-20 00:23:39+02	2020-08-20 00:36:41+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-20 00:33:49+02	2020-08-20 00:37:11+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-08-20 00:37:11+02	2020-08-20 00:37:23+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-08-20 00:36:42+02	2020-08-20 00:37:27+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-08-20 00:37:24+02	2020-08-20 00:37:33+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-08-20 00:37:33+02	2020-08-20 00:37:46+02	textproc	py-rdflib	py37	py37-rdflib-5.0.0
2020-08-20 00:37:47+02	2020-08-20 00:38:00+02	audio	lv2	\N	lv2-1.18.0
2020-08-20 00:32:54+02	2020-08-20 00:38:04+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-20 00:37:28+02	2020-08-20 00:38:40+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-20 00:38:01+02	2020-08-20 00:38:49+02	graphics	frei0r	\N	frei0r-1.6.1
2020-08-20 00:38:40+02	2020-08-20 00:40:24+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-08-20 00:40:24+02	2020-08-20 00:41:27+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-20 00:41:27+02	2020-08-20 00:41:43+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-08-20 00:41:43+02	2020-08-20 00:42:35+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-08-20 00:42:36+02	2020-08-20 00:44:14+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-20 00:44:15+02	2020-08-20 00:44:44+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-08-20 00:44:44+02	2020-08-20 00:46:17+02	graphics	graphene	\N	graphene-1.10.0
2020-08-20 00:46:18+02	2020-08-20 00:46:32+02	www	libwpe	\N	libwpe-1.4.0.1
2020-08-20 00:46:33+02	2020-08-20 00:47:25+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.73.0
2020-08-20 00:38:49+02	2020-08-20 00:47:29+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-20 00:47:29+02	2020-08-20 00:48:25+02	net	geoclue	\N	geoclue-2.5.5
2020-08-20 00:47:25+02	2020-08-20 00:48:32+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-08-20 00:48:25+02	2020-08-20 00:48:41+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-08-20 00:48:33+02	2020-08-20 00:49:08+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-20 00:49:08+02	2020-08-20 00:49:37+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-20 00:49:37+02	2020-08-20 00:50:05+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-10-05 19:49:52+02	2020-10-05 19:53:25+02	security	nmap	\N	nmap-7.90
2020-08-20 00:50:06+02	2020-08-20 00:51:30+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-20 00:51:31+02	2020-08-20 00:51:50+02	print	libraqm	\N	libraqm-0.6.0
2020-08-20 00:51:51+02	2020-08-20 00:52:31+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-08-20 00:52:32+02	2020-08-20 00:52:45+02	textproc	py-toml	py37	py37-toml-0.10.1
2020-08-20 00:52:45+02	2020-08-20 00:52:58+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-08-20 00:38:04+02	2020-08-20 00:56:40+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-20 00:52:58+02	2020-08-20 00:57:16+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-20 00:56:41+02	2020-08-20 00:59:50+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-20 00:59:51+02	2020-08-20 01:00:44+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-20 00:57:17+02	2020-08-20 01:04:36+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-20 01:00:45+02	2020-08-20 01:05:43+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-08-20 01:05:44+02	2020-08-20 01:12:42+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-08-20 01:12:43+02	2020-08-20 01:14:32+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.73.0
2020-08-20 01:14:32+02	2020-08-20 01:14:52+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-08-20 01:14:52+02	2020-08-20 01:15:37+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-20 01:04:38+02	2020-08-20 01:15:53+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-20 01:15:54+02	2020-08-20 01:16:21+02	ftp	php73-curl	\N	php73-curl-7.3.20
2020-08-20 01:15:38+02	2020-08-20 01:16:27+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-20 01:16:21+02	2020-08-20 01:17:19+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-20 01:16:27+02	2020-08-20 01:17:47+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-20 01:17:20+02	2020-08-20 01:18:18+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-20 01:17:47+02	2020-08-20 01:18:23+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-08-20 01:18:23+02	2020-08-20 01:19:41+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-08-20 01:18:19+02	2020-08-20 01:19:41+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-08-20 01:19:42+02	2020-08-20 01:20:33+02	archivers	kf5-karchive	\N	kf5-karchive-5.73.0
2020-08-20 01:19:42+02	2020-08-20 01:21:02+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-08-20 01:21:02+02	2020-08-20 01:21:25+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-08-20 01:21:26+02	2020-08-20 01:22:31+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-20 01:20:35+02	2020-08-20 01:22:54+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-20 01:22:54+02	2020-08-20 01:24:44+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-20 01:22:32+02	2020-08-20 01:26:00+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_3,1
2020-08-20 01:24:45+02	2020-08-20 01:26:17+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-08-20 01:26:02+02	2020-08-20 01:30:08+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-20 01:30:08+02	2020-08-20 01:31:27+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-08-20 00:48:41+02	2020-08-20 01:31:46+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-20 01:31:27+02	2020-08-20 01:32:42+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-08-20 01:26:17+02	2020-08-20 01:32:45+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-20 01:32:42+02	2020-08-20 01:33:29+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-20 01:33:30+02	2020-08-20 01:33:48+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-08-20 01:32:45+02	2020-08-20 01:33:55+02	graphics	poppler-glib	\N	poppler-glib-20.08.0_1
2020-08-20 01:33:56+02	2020-08-20 01:34:51+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-08-20 01:33:48+02	2020-08-20 01:35:45+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-08-20 01:34:52+02	2020-08-20 01:35:54+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-08-20 01:35:46+02	2020-08-20 01:35:56+02	devel	py-six	py27	py27-six-1.15.0
2020-08-20 01:35:57+02	2020-08-20 01:37:03+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-20 01:35:54+02	2020-08-20 01:39:23+02	graphics	glew	\N	glew-2.2.0
2020-08-20 01:39:23+02	2020-08-20 01:40:11+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-08-20 01:37:04+02	2020-08-20 01:40:41+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-08-20 01:40:11+02	2020-08-20 01:40:41+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-08-20 01:40:42+02	2020-08-20 01:41:42+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-20 01:40:42+02	2020-08-20 01:41:51+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-20 01:41:52+02	2020-08-20 01:42:41+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-08-20 01:41:42+02	2020-08-20 01:42:44+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.73.0
2020-08-20 01:31:47+02	2020-08-20 01:42:50+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-20 01:42:41+02	2020-08-20 01:44:55+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-20 01:44:56+02	2020-08-20 01:46:27+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-08-20 01:42:45+02	2020-08-20 01:46:51+02	databases	mysql57-client	\N	mysql57-client-5.7.31
2020-08-20 01:42:51+02	2020-08-20 01:47:13+02	devel	git	\N	git-2.28.0
2020-08-20 01:46:28+02	2020-08-20 01:47:43+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-08-20 01:47:43+02	2020-08-20 01:48:06+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-08-20 01:46:52+02	2020-08-20 01:48:07+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-20 01:47:13+02	2020-08-20 01:48:07+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-08-20 01:48:08+02	2020-08-20 01:49:12+02	net	remmina	\N	remmina-1.3.9
2020-08-20 01:48:08+02	2020-08-20 01:50:10+02	security	gcr	\N	gcr-3.28.0
2020-08-20 01:49:13+02	2020-08-20 01:50:22+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-08-20 01:50:11+02	2020-08-20 01:50:34+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-08-20 01:50:22+02	2020-08-20 01:50:52+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-08-20 01:50:34+02	2020-08-20 01:51:08+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-08-20 01:51:09+02	2020-08-20 01:51:44+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-20 01:51:46+02	2020-08-20 01:52:30+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-08-20 01:52:30+02	2020-08-20 01:53:27+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-20 01:53:28+02	2020-08-20 01:54:18+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-08-20 01:54:18+02	2020-08-20 01:54:48+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-08-20 01:54:49+02	2020-08-20 01:55:56+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-08-20 01:50:52+02	2020-08-20 01:56:33+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-20 01:55:56+02	2020-08-20 01:56:39+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-08-20 01:56:39+02	2020-08-20 01:59:08+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.73.0
2020-08-20 01:59:09+02	2020-08-20 02:00:07+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-20 01:56:33+02	2020-08-20 02:00:34+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-20 02:00:08+02	2020-08-20 02:01:28+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-08-20 02:00:35+02	2020-08-20 02:01:34+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-08-20 02:01:30+02	2020-08-20 02:02:49+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-20 02:02:49+02	2020-08-20 02:05:20+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-20 02:01:34+02	2020-08-20 02:09:31+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-20 02:09:31+02	2020-08-20 02:10:16+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-27 01:20:21+02	2020-08-27 01:20:53+02	net	fping	\N	fping-5.0_1
2020-08-20 02:10:16+02	2020-08-20 02:11:45+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-20 02:11:46+02	2020-08-20 02:12:13+02	print	gl2ps	\N	gl2ps-1.4.0
2020-08-20 02:12:13+02	2020-08-20 02:14:22+02	security	gnupg	\N	gnupg-2.2.21
2020-08-20 02:14:23+02	2020-08-20 02:15:45+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-08-20 02:15:45+02	2020-08-20 02:18:43+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-20 02:18:44+02	2020-08-20 02:20:11+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-20 02:20:12+02	2020-08-20 02:20:27+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-08-20 02:20:27+02	2020-08-20 02:25:24+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-20 02:25:25+02	2020-08-20 02:25:54+02	security	gpgme	\N	gpgme-1.14.0
2020-08-20 02:25:55+02	2020-08-20 02:27:32+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-08-20 02:27:32+02	2020-08-20 02:33:11+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-20 02:05:21+02	2020-08-20 02:43:17+02	math	vtk8	\N	vtk8-8.2.0
2020-08-20 02:43:18+02	2020-08-20 04:36:59+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-20 04:37:07+02	2020-08-20 04:39:03+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-20 04:39:04+02	2020-08-20 04:40:53+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-08-20 04:40:54+02	2020-08-20 04:51:53+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-20 04:51:53+02	2020-08-20 04:52:36+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-08-20 04:52:37+02	2020-08-20 04:53:28+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-08-20 04:53:29+02	2020-08-20 04:55:18+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-20 04:55:19+02	2020-08-20 04:56:29+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-20 04:56:30+02	2020-08-20 04:57:52+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-20 04:57:53+02	2020-08-20 04:58:12+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-08-20 04:58:13+02	2020-08-20 05:01:40+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-20 05:01:41+02	2020-08-20 05:02:56+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-08-20 05:02:57+02	2020-08-20 05:04:45+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-08-20 02:33:12+02	2020-08-20 05:43:44+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-20 05:43:45+02	2020-08-20 05:46:10+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-08-20 05:46:11+02	2020-08-20 05:52:10+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-20 05:52:11+02	2020-08-20 05:52:27+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-08-20 05:52:27+02	2020-08-20 05:54:07+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-20 05:54:08+02	2020-08-20 05:55:25+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-20 05:55:26+02	2020-08-20 05:56:40+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-20 05:56:41+02	2020-08-20 05:57:54+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-20 05:57:55+02	2020-08-20 05:59:16+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-20 05:59:17+02	2020-08-20 06:02:36+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-08-20 06:02:37+02	2020-08-20 06:03:41+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-20 06:03:42+02	2020-08-20 06:04:46+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-20 06:04:46+02	2020-08-20 06:07:16+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-08-20 06:07:17+02	2020-08-20 06:09:03+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-08-20 06:09:04+02	2020-08-20 06:09:46+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-20 06:09:46+02	2020-08-20 06:41:00+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-20 06:41:01+02	2020-08-20 08:49:32+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-20 08:49:35+02	2020-08-20 08:56:51+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-20 08:56:54+02	2020-08-20 09:01:36+02	devel	librest	\N	rest-0.7.93
2020-08-20 09:01:37+02	2020-08-20 09:02:28+02	devel	py-cffi	py27	py27-cffi-1.14.2
2020-08-20 09:02:30+02	2020-08-20 09:10:35+02	graphics	poppler-utils	\N	poppler-utils-20.08.0_1
2020-08-20 09:10:38+02	2020-08-20 09:59:14+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-20 09:59:15+02	2020-08-20 09:59:45+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-08-20 09:59:46+02	2020-08-20 10:02:21+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-08-20 10:02:22+02	2020-08-20 10:03:30+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-20 10:03:33+02	2020-08-20 10:06:33+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-20 10:06:37+02	2020-08-20 10:10:24+02	sysutils	bsdisks	\N	bsdisks-0.21
2020-08-20 10:10:26+02	2020-08-20 10:16:38+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-20 10:16:41+02	2020-08-20 10:18:17+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-08-20 10:18:21+02	2020-08-20 10:21:33+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-20 10:21:36+02	2020-08-20 10:34:48+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-20 10:34:51+02	2020-08-20 10:40:11+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_3,1
2020-08-20 01:48:06+02	2020-08-20 10:45:51+02	lang	rust	\N	rust-1.45.2
2020-08-20 10:45:57+02	2020-08-20 10:49:08+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-20 10:40:12+02	2020-08-20 10:49:25+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-20 10:49:08+02	2020-08-20 10:50:11+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-20 10:49:26+02	2020-08-20 10:50:43+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-20 10:50:11+02	2020-08-20 10:51:35+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-20 10:50:44+02	2020-08-20 10:52:16+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-20 10:52:17+02	2020-08-20 10:52:28+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-08-20 10:51:35+02	2020-08-20 10:54:11+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-08-20 10:54:11+02	2020-08-20 10:54:45+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-08-20 10:52:29+02	2020-08-20 10:56:15+02	graphics	gegl	\N	gegl-0.4.24_4
2020-08-20 10:56:15+02	2020-08-20 11:00:43+02	mail	mailutils	\N	mailutils-3.10
2020-08-20 11:00:44+02	2020-08-20 11:03:41+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-20 11:03:42+02	2020-08-20 11:04:01+02	net	liboauth	\N	liboauth-1.0.3_4
2020-08-20 10:54:46+02	2020-08-20 11:06:49+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-20 11:06:50+02	2020-08-20 11:07:05+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-08-20 11:07:05+02	2020-08-20 11:08:16+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-20 11:08:17+02	2020-08-20 11:09:33+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-20 11:09:34+02	2020-08-20 11:13:23+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-20 11:13:23+02	2020-08-20 11:16:05+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-20 11:16:06+02	2020-08-20 11:17:34+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-20 11:17:34+02	2020-08-20 11:17:54+02	www	py-html5lib	py27	py27-html5lib-1.0.1
2020-08-20 11:17:59+02	2020-08-20 11:19:09+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-08-20 11:19:11+02	2020-08-20 11:20:31+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-08-20 11:20:31+02	2020-08-20 11:24:15+02	x11	xorg	\N	xorg-7.7_3
2020-08-20 11:24:15+02	2020-08-20 11:25:57+02	x11	zenity	\N	zenity-3.32.0
2020-08-20 11:25:57+02	2020-08-20 11:27:20+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-08-20 11:04:01+02	2020-08-20 11:31:30+02	net	samba410	\N	samba410-4.10.17
2020-08-20 11:31:31+02	2020-08-20 11:35:47+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-20 11:35:49+02	2020-08-20 11:37:33+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-20 11:37:34+02	2020-08-20 11:37:54+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-08-20 11:37:55+02	2020-08-20 11:38:11+02	devel	py-appdirs	py37	py37-appdirs-1.4.4
2020-08-20 11:38:11+02	2020-08-20 11:39:12+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-08-20 11:39:13+02	2020-08-20 11:39:25+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-08-20 11:27:21+02	2020-08-20 11:39:25+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-20 11:39:26+02	2020-08-20 11:42:00+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-20 11:39:26+02	2020-08-20 11:42:10+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-20 11:42:01+02	2020-08-20 11:44:11+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-20 11:44:12+02	2020-08-20 11:44:28+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-08-20 11:42:11+02	2020-08-20 11:50:13+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-20 11:50:13+02	2020-08-20 11:50:32+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-08-20 11:44:29+02	2020-08-20 11:52:08+02	editors	emacs	full	emacs-27.1,3
2020-08-20 11:52:09+02	2020-08-20 11:52:34+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-08-20 11:50:33+02	2020-08-20 12:02:14+02	graphics	Coin	\N	Coin-4.0.0_1
2020-08-20 12:02:15+02	2020-08-20 12:03:37+02	graphics	glfw	\N	glfw-3.3.2
2020-08-20 12:03:37+02	2020-08-20 12:07:31+02	graphics	movit	\N	movit-1.6.2_2
2020-08-20 11:52:34+02	2020-08-20 12:11:50+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-08-20 12:11:51+02	2020-08-20 12:12:35+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-08-20 12:12:36+02	2020-08-20 12:14:57+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-20 12:14:58+02	2020-08-20 12:22:12+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-08-20 12:22:13+02	2020-08-20 12:22:47+02	net	gssdp	\N	gssdp-1.2.3
2020-08-20 12:22:47+02	2020-08-20 12:23:58+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-08-20 12:23:59+02	2020-08-20 12:25:48+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-08-20 12:25:49+02	2020-08-20 12:36:46+02	print	fontforge	\N	fontforge-20190801_2
2020-08-20 12:07:31+02	2020-08-20 12:39:46+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-20 12:39:48+02	2020-08-20 12:40:03+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-08-20 12:40:05+02	2020-08-20 12:41:07+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-20 12:41:08+02	2020-08-20 12:43:18+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-08-20 12:43:18+02	2020-08-20 12:44:22+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-08-20 12:36:46+02	2020-08-20 12:54:15+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-08-20 12:54:17+02	2020-08-20 12:54:36+02	www	py-hyperlink	py37	py37-hyperlink-20.0.1
2020-08-20 12:54:37+02	2020-08-20 12:56:39+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-20 12:56:40+02	2020-08-20 12:59:01+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-20 12:59:02+02	2020-08-20 13:00:39+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-20 13:00:41+02	2020-08-20 13:01:50+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-20 13:01:50+02	2020-08-20 13:04:23+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-20 13:04:27+02	2020-08-20 13:10:49+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-20 13:10:50+02	2020-08-20 13:17:12+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-20 13:17:13+02	2020-08-20 13:18:29+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-20 13:18:29+02	2020-08-20 13:23:14+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-20 13:23:15+02	2020-08-20 13:34:14+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-20 13:34:15+02	2020-08-20 13:46:19+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-20 13:46:20+02	2020-08-20 13:47:54+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-20 13:47:55+02	2020-08-20 13:51:01+02	x11-wm	marco	\N	marco-1.22.4
2020-08-20 13:51:02+02	2020-08-20 13:51:27+02	archivers	c-blosc	\N	c-blosc-1.15.0_2
2020-08-20 13:51:28+02	2020-08-20 13:52:47+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-20 13:52:48+02	2020-08-20 13:53:36+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-08-20 13:53:36+02	2020-08-20 13:56:44+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-20 13:56:45+02	2020-08-20 13:58:20+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-08-20 13:58:21+02	2020-08-20 14:04:06+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-20 14:04:07+02	2020-08-20 14:08:09+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-20 14:08:09+02	2020-08-20 14:08:29+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-08-20 14:08:29+02	2020-08-20 14:09:42+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-08-20 14:09:42+02	2020-08-20 14:17:55+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-20 14:17:56+02	2020-08-20 14:19:54+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-20 14:19:55+02	2020-08-20 14:20:25+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-08-20 14:20:26+02	2020-08-20 14:23:01+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-20 14:23:02+02	2020-08-20 14:24:17+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-08-20 14:24:18+02	2020-08-20 14:25:02+02	graphics	libgxps	\N	libgxps-0.3.1
2020-08-20 14:25:03+02	2020-08-20 14:25:38+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-08-20 14:25:39+02	2020-08-20 14:30:13+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-08-20 14:30:14+02	2020-08-20 14:31:00+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-20 14:31:01+02	2020-08-20 14:32:15+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-20 14:32:16+02	2020-08-20 14:33:24+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-20 14:33:24+02	2020-08-20 14:58:14+02	math	R	\N	R-4.0.2
2020-08-20 14:58:15+02	2020-08-20 15:09:24+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-20 15:09:25+02	2020-08-20 15:10:51+02	multimedia	libass	\N	libass-0.14.0
2020-08-20 15:10:52+02	2020-08-20 15:13:11+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-20 15:13:12+02	2020-08-20 15:13:42+02	net	fping	\N	fping-5.0
2020-08-20 15:13:43+02	2020-08-20 15:15:44+02	net	freerdp	\N	freerdp-2.2.0
2020-08-20 15:15:45+02	2020-08-20 15:16:23+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-20 15:16:24+02	2020-08-20 15:34:22+02	net	libcmis	\N	libcmis-0.5.2_1
2020-08-20 15:34:23+02	2020-08-20 15:35:30+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-20 15:35:31+02	2020-08-20 15:36:08+02	net	rsync	\N	rsync-3.2.3
2020-08-20 15:36:09+02	2020-08-20 15:37:06+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-20 05:04:46+02	2020-08-20 15:38:24+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-20 15:37:07+02	2020-08-20 15:41:44+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-20 15:41:45+02	2020-08-20 15:42:24+02	print	libspectre	\N	libspectre-0.2.9
2020-08-20 15:42:25+02	2020-08-20 15:45:27+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-20 15:45:28+02	2020-08-20 15:54:32+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-20 15:54:33+02	2020-08-20 16:03:03+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-20 16:03:04+02	2020-08-20 16:04:09+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-08-20 16:04:12+02	2020-08-20 16:04:39+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-08-20 16:04:40+02	2020-08-20 16:05:06+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-08-20 16:05:06+02	2020-08-20 16:10:01+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-20 16:10:02+02	2020-08-20 16:11:22+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-20 16:11:23+02	2020-08-20 16:15:22+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-20 16:15:26+02	2020-08-20 16:18:15+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-20 16:18:16+02	2020-08-20 16:19:32+02	textproc	redland	\N	redland-1.0.17_4
2020-08-20 16:19:32+02	2020-08-20 16:20:27+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-20 16:20:28+02	2020-08-20 16:22:04+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-20 16:22:05+02	2020-08-20 16:22:59+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-20 16:22:59+02	2020-08-20 16:24:56+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-20 16:24:56+02	2020-08-20 16:27:54+02	x11-servers	xephyr	\N	xephyr-1.20.8_3,1
2020-08-20 16:27:56+02	2020-08-20 16:29:32+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-20 16:29:34+02	2020-08-20 16:30:43+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-20 16:30:44+02	2020-08-20 16:32:27+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-20 16:32:28+02	2020-08-20 16:34:44+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-20 16:34:45+02	2020-08-20 16:37:19+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-20 15:38:26+02	2020-08-20 16:45:05+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-20 16:45:08+02	2020-08-20 16:48:26+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-20 16:48:28+02	2020-08-20 16:49:53+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-08-20 16:49:54+02	2020-08-20 16:50:51+02	audio	faudio	\N	FAudio-20.08
2020-08-20 16:50:51+02	2020-08-20 16:52:19+02	audio	mate-media	\N	mate-media-1.24.1
2020-08-20 16:37:19+02	2020-08-20 16:52:29+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-20 16:52:22+02	2020-08-20 16:52:51+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.14.2
2020-08-20 16:52:29+02	2020-08-20 16:53:45+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-20 16:52:52+02	2020-08-20 16:54:22+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-20 16:54:22+02	2020-08-20 16:55:28+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-20 16:55:28+02	2020-08-20 16:56:36+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-20 16:56:37+02	2020-08-20 16:57:05+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-08-20 16:57:05+02	2020-08-20 16:57:28+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-08-20 16:53:45+02	2020-08-20 16:57:39+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-20 16:57:39+02	2020-08-20 16:58:56+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-08-20 16:57:28+02	2020-08-20 17:00:17+02	devel	google-perftools	\N	google-perftools-2.8
2020-08-20 16:58:57+02	2020-08-20 17:00:19+02	devel	ode	\N	ode-0.13_4,1
2020-08-20 17:00:20+02	2020-08-20 17:00:33+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-08-20 17:00:34+02	2020-08-20 17:01:37+02	devel	py-game	py37	py37-game-1.9.6_1
2020-08-20 17:01:37+02	2020-08-20 17:01:51+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-08-20 17:01:52+02	2020-08-20 17:03:49+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-20 17:00:18+02	2020-08-20 17:04:04+02	devel	protobuf-c	\N	protobuf-c-1.3.2_5
2020-08-20 17:03:50+02	2020-08-20 17:04:05+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-08-20 17:04:05+02	2020-08-20 17:04:17+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-08-20 17:04:06+02	2020-08-20 17:04:17+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-08-20 17:04:17+02	2020-08-20 17:12:22+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.3_3
2020-08-20 17:04:18+02	2020-08-20 17:35:34+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-20 17:35:35+02	2020-08-20 17:42:16+02	editors	pluma	\N	pluma-1.24.0
2020-08-20 17:42:17+02	2020-08-20 17:45:21+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-08-20 17:45:22+02	2020-08-20 17:49:51+02	graphics	atril	\N	atril-1.22.3_8
2020-08-20 17:49:52+02	2020-08-20 17:53:00+02	graphics	eom	\N	eom-1.24.0
2020-08-20 17:53:01+02	2020-08-20 17:55:37+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-20 17:55:38+02	2020-08-20 17:56:08+02	graphics	gle	\N	gle-3.1.0_7
2020-08-20 17:56:09+02	2020-08-20 17:56:34+02	graphics	gltt	\N	gltt-2.5.2_14
2020-08-20 17:56:35+02	2020-08-20 17:58:38+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-20 17:58:39+02	2020-08-20 18:02:54+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-08-20 18:02:56+02	2020-08-20 18:04:50+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-08-20 18:04:51+02	2020-08-20 18:07:51+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-20 18:07:51+02	2020-08-20 18:08:34+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-08-20 18:08:35+02	2020-08-20 18:09:22+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-08-20 17:12:23+02	2020-08-20 18:10:12+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-08-20 18:10:13+02	2020-08-20 18:18:34+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-20 18:09:22+02	2020-08-20 18:21:31+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-20 18:21:32+02	2020-08-20 18:24:31+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-20 18:24:31+02	2020-08-20 18:26:16+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-20 18:26:17+02	2020-08-20 18:39:48+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-08-20 18:39:49+02	2020-08-20 18:41:04+02	multimedia	gmmlib	\N	gmmlib-20.2.4
2020-08-20 18:41:04+02	2020-08-20 18:43:55+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-20 18:43:56+02	2020-08-20 18:47:28+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-20 18:47:28+02	2020-08-20 19:03:19+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-20 19:03:20+02	2020-08-20 19:05:21+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-20 19:05:22+02	2020-08-20 19:06:41+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-20 19:06:42+02	2020-08-20 19:08:03+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-20 19:08:04+02	2020-08-20 19:09:22+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-20 19:09:22+02	2020-08-20 19:10:40+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-20 19:10:41+02	2020-08-20 19:11:48+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-20 19:11:49+02	2020-08-20 19:13:00+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-20 19:13:00+02	2020-08-20 19:14:52+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-20 19:14:53+02	2020-08-20 19:18:23+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-20 19:18:25+02	2020-08-20 19:21:05+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-20 19:21:10+02	2020-08-20 19:30:03+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-08-20 19:30:10+02	2020-08-20 19:31:01+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-08-20 19:31:06+02	2020-08-20 19:37:48+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-20 19:37:49+02	2020-08-20 20:05:54+02	print	lilypond	\N	lilypond-2.20.0
2020-08-20 20:06:00+02	2020-08-20 20:34:37+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-20 20:34:41+02	2020-08-20 20:53:05+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-20 20:53:07+02	2020-08-20 21:00:51+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-20 18:18:35+02	2020-08-20 21:10:53+02	lang	julia11	\N	julia11-1.1.1_5
2020-08-20 21:00:55+02	2020-08-20 21:15:20+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-08-20 21:15:21+02	2020-08-20 21:15:54+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-20 21:15:54+02	2020-08-20 21:16:12+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-08-20 21:16:13+02	2020-08-20 21:16:28+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-08-20 21:16:28+02	2020-08-20 21:17:25+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-20 21:17:26+02	2020-08-20 21:19:33+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-20 21:19:34+02	2020-08-20 21:22:05+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-20 21:22:06+02	2020-08-20 21:23:15+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-20 21:23:15+02	2020-08-20 21:23:57+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-08-20 21:23:57+02	2020-08-20 21:24:48+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-20 21:24:49+02	2020-08-20 21:25:03+02	textproc	py-scour	py37	py37-scour-0.38
2020-08-20 21:25:04+02	2020-08-20 21:25:34+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-08-20 21:25:35+02	2020-08-20 21:25:50+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-08-20 21:25:51+02	2020-08-20 21:27:24+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-20 21:27:25+02	2020-08-20 21:28:50+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-20 21:28:51+02	2020-08-20 21:35:35+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-20 21:10:54+02	2020-08-20 21:36:09+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-20 21:35:36+02	2020-08-20 21:36:58+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-20 21:36:12+02	2020-08-20 21:37:34+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-20 21:36:59+02	2020-08-20 21:39:22+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-20 21:37:35+02	2020-08-20 21:39:42+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-20 21:39:23+02	2020-08-20 21:40:14+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-08-20 21:40:15+02	2020-08-20 21:41:01+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-20 21:39:43+02	2020-08-20 21:41:04+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-05 20:09:47+02	2020-10-05 20:11:33+02	cad	cura	\N	Cura-4.7.1,2
2020-08-20 21:41:02+02	2020-08-20 21:45:38+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-20 21:45:39+02	2020-08-20 21:58:19+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-20 21:41:05+02	2020-08-20 21:59:43+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-20 21:58:20+02	2020-08-20 22:00:15+02	benchmarks	fio	\N	fio-3.22
2020-08-20 22:00:15+02	2020-08-20 22:11:33+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-20 22:11:40+02	2020-08-20 22:22:08+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-20 21:59:44+02	2020-08-20 23:11:41+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-20 23:11:44+02	2020-08-20 23:17:08+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-20 23:17:09+02	2020-08-20 23:21:44+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-20 23:21:50+02	2020-08-20 23:47:19+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-20 23:47:26+02	2020-08-20 23:48:24+02	databases	pgtop	\N	pgtop-0.05_5
2020-08-20 23:48:26+02	2020-08-21 00:02:38+02	deskutils	calibre	\N	calibre-4.22.0_1
2020-08-21 00:02:38+02	2020-08-21 00:03:42+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-21 00:03:43+02	2020-08-21 00:52:21+02	devel	RStudio	\N	RStudio-1.3.1056
2020-08-21 00:52:22+02	2020-08-21 00:54:20+02	devel	bzr	py27	bzr-2.7.0.6622
2020-08-21 00:54:21+02	2020-08-21 00:55:43+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-20 22:22:14+02	2020-08-21 01:13:18+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-21 01:13:19+02	2020-08-21 01:14:10+02	devel	tig	\N	tig-2.5.1
2020-08-21 01:14:10+02	2020-08-21 01:18:18+02	dns	bind916	\N	bind916-9.16.5_1
2020-08-21 01:18:19+02	2020-08-21 01:18:32+02	dns	sshfp	\N	sshfp-1.2.2_5
2020-08-21 01:18:33+02	2020-08-21 01:50:18+02	editors	codelite	\N	codelite-12.0_4
2020-08-21 01:50:20+02	2020-08-21 01:53:24+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-08-21 01:53:25+02	2020-08-21 01:57:27+02	editors	vim	\N	vim-8.2.1382
2020-08-21 01:57:27+02	2020-08-21 02:39:33+02	emulators	qemu	\N	qemu-4.2.1
2020-08-21 02:39:34+02	2020-08-21 04:11:48+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-08-21 04:11:49+02	2020-08-21 04:42:40+02	emulators	wine	\N	wine-5.0.2,1
2020-08-21 04:42:41+02	2020-08-21 05:04:31+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-21 05:04:31+02	2020-08-21 05:05:04+02	games	KnightCap	\N	knightcap-3.6_8
2020-08-21 05:05:07+02	2020-08-21 05:05:33+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-08-21 05:05:34+02	2020-08-21 05:09:05+02	games	alienarena	\N	alienarena-2013.766_9
2020-08-21 05:09:12+02	2020-08-21 05:10:24+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-08-21 05:10:27+02	2020-08-21 05:15:26+02	games	battletanks	\N	btanks-0.9.8083_9
2020-08-21 05:15:26+02	2020-08-21 05:15:55+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-08-21 05:15:56+02	2020-08-21 05:17:01+02	games	monsterz	\N	monsterz-0.7.1_15
2020-08-21 05:17:03+02	2020-08-21 05:17:56+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-21 05:17:57+02	2020-08-21 05:18:54+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-21 05:18:55+02	2020-08-21 05:19:21+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-08-21 05:19:22+02	2020-08-21 05:20:55+02	games	xblast	\N	xblast-2.10.4_17
2020-08-21 05:20:56+02	2020-08-21 05:22:31+02	games	xboard	\N	xboard-4.9.1_1
2020-08-21 05:22:32+02	2020-08-21 05:23:45+02	games	xcowsay	\N	xcowsay-1.5
2020-08-21 05:23:45+02	2020-08-21 06:09:06+02	graphics	blender	\N	blender-2.83.2_1
2020-08-21 06:09:08+02	2020-08-21 06:14:01+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-21 06:14:02+02	2020-08-21 06:15:44+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-21 06:15:44+02	2020-08-21 06:47:34+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-21 06:47:37+02	2020-08-21 08:07:14+02	graphics	krita	\N	krita-4.3.0_4
2020-08-21 08:07:17+02	2020-08-21 08:23:27+02	graphics	wings	\N	wings-2.2.5
2020-08-21 08:23:31+02	2020-08-21 08:27:29+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-21 08:27:29+02	2020-08-21 08:30:07+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-21 00:55:44+02	2020-08-21 08:32:09+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-21 08:30:10+02	2020-08-21 09:15:35+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.8
2020-08-21 08:32:14+02	2020-08-21 09:17:54+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-21 09:17:55+02	2020-08-21 09:18:46+02	lang	julia	\N	julia-1.0.3_3
2020-08-21 09:18:47+02	2020-08-21 09:27:29+02	math	maxima	\N	maxima-5.44.0_3
2020-08-21 09:27:31+02	2020-08-21 09:29:12+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-21 09:29:13+02	2020-08-21 09:29:40+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-08-21 09:29:42+02	2020-08-21 10:02:47+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p5
2020-08-21 10:02:49+02	2020-08-21 10:03:16+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-08-21 10:03:16+02	2020-08-21 10:10:38+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-21 10:10:40+02	2020-08-21 10:14:13+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-21 10:14:15+02	2020-08-21 10:24:23+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-21 10:24:24+02	2020-08-21 10:26:02+02	net	mosh	\N	mosh-1.3.2_13
2020-08-21 10:26:04+02	2020-08-21 10:33:23+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-21 10:33:25+02	2020-08-21 10:35:34+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-21 10:35:35+02	2020-08-21 11:01:37+02	net	wireshark	\N	wireshark-3.2.6
2020-08-21 11:01:42+02	2020-08-21 11:03:03+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-21 11:03:04+02	2020-08-21 11:15:49+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-08-21 11:15:50+02	2020-08-21 11:17:05+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-21 11:17:11+02	2020-08-21 11:18:19+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-21 11:18:19+02	2020-08-21 11:19:33+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-21 11:19:34+02	2020-08-21 11:20:45+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-21 11:20:47+02	2020-08-21 11:21:59+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-21 11:22:01+02	2020-08-21 11:23:16+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-21 11:23:17+02	2020-08-21 11:24:27+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-21 11:24:29+02	2020-08-21 11:25:45+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-21 11:25:46+02	2020-08-21 11:27:01+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-21 11:27:02+02	2020-08-21 11:29:21+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-21 11:29:22+02	2020-08-21 11:30:42+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-21 11:30:44+02	2020-08-21 11:34:33+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-21 11:34:34+02	2020-08-21 11:35:34+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-21 11:35:35+02	2020-08-21 11:36:37+02	print	gv	\N	gv-3.7.4_5
2020-08-21 11:36:38+02	2020-08-21 11:54:00+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-21 11:54:03+02	2020-08-21 12:53:26+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-08-21 12:53:28+02	2020-08-21 13:00:38+02	print	texlive-full	\N	texlive-full-20150521
2020-08-21 13:00:40+02	2020-08-21 13:00:50+02	security	base-audit	\N	base-audit-0.4
2020-08-21 13:00:50+02	2020-08-21 13:04:47+02	security	clamav	\N	clamav-0.102.4,1
2020-08-21 13:04:48+02	2020-08-21 13:10:11+02	security	klee	\N	klee-2.1_1
2020-08-21 13:10:11+02	2020-08-21 13:11:44+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-21 13:11:45+02	2020-08-21 13:19:52+02	security	tor	\N	tor-0.4.3.6
2020-08-21 13:19:54+02	2020-08-21 13:23:36+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-08-21 13:23:37+02	2020-08-21 13:25:08+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-21 13:25:09+02	2020-08-21 13:25:48+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-10-05 20:11:54+02	2020-10-05 20:13:15+02	x11	mate	\N	mate-1.24.1
2020-08-21 13:25:50+02	2020-08-21 13:27:14+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-21 13:27:15+02	2020-08-21 13:28:48+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-21 13:28:49+02	2020-08-21 13:32:19+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-21 13:32:20+02	2020-08-21 13:36:26+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-08-21 09:15:36+02	2020-08-21 13:46:18+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200816
2020-08-21 13:46:25+02	2020-08-21 13:48:00+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-08-21 13:48:01+02	2020-08-21 13:48:27+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-08-21 13:48:29+02	2020-08-21 13:48:46+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-08-21 13:48:48+02	2020-08-21 13:49:16+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-08-21 13:49:17+02	2020-08-21 13:49:39+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-08-21 13:49:39+02	2020-08-21 13:50:13+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-08-21 13:50:13+02	2020-08-21 13:52:38+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-21 13:52:39+02	2020-08-21 13:55:05+02	x11	mate	\N	mate-1.22.2
2020-08-21 13:55:06+02	2020-08-21 13:56:59+02	x11	xlockmore	\N	xlockmore-5.65
2020-08-21 13:57:04+02	2020-08-21 14:04:53+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-21 14:04:53+02	2020-08-21 14:12:37+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-21 14:12:39+02	2020-08-21 14:15:12+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-21 14:15:13+02	2020-08-21 14:15:32+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-08-21 14:15:35+02	2020-08-21 14:16:41+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-08-21 14:16:42+02	2020-08-21 15:04:44+02	editors	vscode	\N	vscode-1.46.1
2020-08-21 15:04:48+02	2020-08-21 15:06:33+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-21 15:06:34+02	2020-08-21 15:07:31+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-21 13:36:32+02	2020-08-21 16:48:23+02	www	firefox	\N	firefox-80.0,1
2020-08-20 12:44:24+02	2020-08-21 17:18:42+02	www	chromium	\N	chromium-84.0.4147.135
2020-08-21 17:18:43+02	2020-08-21 17:19:35+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-21 17:19:36+02	2020-08-21 17:20:26+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-21 17:20:27+02	2020-08-21 17:21:17+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-21 17:21:18+02	2020-08-21 17:38:12+02	local	E590T-localbase	\N	E590T-localbase-2020081500
2020-08-21 17:38:27+02	2020-08-21 17:39:31+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-21 17:38:27+02	2020-08-21 17:43:38+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-21 17:43:39+02	2020-08-21 17:43:46+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-22 01:27:48+02	2020-08-22 01:28:18+02	devel	libffi	\N	libffi-3.3_1
2020-08-22 01:27:48+02	2020-08-22 01:31:51+02	lang	php73	\N	php73-7.3.21
2020-08-22 01:31:51+02	2020-08-22 01:32:40+02	security	p11-kit	\N	p11-kit-0.23.20_1
2020-08-22 01:28:19+02	2020-08-22 01:33:39+02	lang	python37	\N	python37-3.7.8_1
2020-08-22 01:33:39+02	2020-08-22 01:33:57+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-08-22 01:27:48+02	2020-08-22 01:33:58+02	databases	sqlite3	\N	sqlite3-3.33.0,1
2020-08-22 01:33:57+02	2020-08-22 01:34:10+02	devel	py-six	py37	py37-six-1.15.0
2020-08-22 01:33:58+02	2020-08-22 01:34:12+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-08-22 01:34:11+02	2020-08-22 01:34:25+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-08-22 01:34:13+02	2020-08-22 01:34:27+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-08-22 01:34:28+02	2020-08-22 01:34:41+02	dns	py-idna	py37	py37-idna-2.8
2020-08-22 01:34:25+02	2020-08-22 01:34:47+02	devel	py-cffi	py37	py37-cffi-1.14.2
2020-08-22 01:34:42+02	2020-08-22 01:35:17+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-08-22 01:35:18+02	2020-08-22 01:35:27+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-08-22 01:34:47+02	2020-08-22 01:35:31+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-08-22 01:32:41+02	2020-08-22 01:35:36+02	security	gnutls	\N	gnutls-3.6.14
2020-08-22 01:35:28+02	2020-08-22 01:35:37+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-08-22 01:35:32+02	2020-08-22 01:35:42+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-08-22 01:35:36+02	2020-08-22 01:35:47+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-08-22 01:35:38+02	2020-08-22 01:35:48+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-08-22 01:35:43+02	2020-08-22 01:35:53+02	net	py-urllib3	py37	py37-urllib3-1.25.10,1
2020-08-22 01:35:47+02	2020-08-22 01:35:57+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-08-22 01:35:53+02	2020-08-22 01:36:05+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-08-22 01:36:06+02	2020-08-22 01:36:17+02	www	py-requests	py37	py37-requests-2.22.0
2020-08-22 01:35:48+02	2020-08-22 01:37:21+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-08-22 01:36:17+02	2020-08-22 01:37:22+02	devel	ninja	\N	ninja-1.10.0,2
2020-08-22 01:37:22+02	2020-08-22 01:37:31+02	devel	py-packaging	py37	py37-packaging-20.4
2020-08-22 01:37:23+02	2020-08-22 01:37:37+02	devel	meson	\N	meson-0.54.3
2020-08-22 01:37:32+02	2020-08-22 01:37:46+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-08-22 01:37:37+02	2020-08-22 01:37:47+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-08-22 01:35:58+02	2020-08-22 01:37:56+02	lang	cython	py37	py37-cython-0.29.15
2020-08-22 01:37:47+02	2020-08-22 01:37:57+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-08-22 01:37:47+02	2020-08-22 01:37:57+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-08-22 01:37:57+02	2020-08-22 01:38:06+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-08-22 01:37:57+02	2020-08-22 01:38:07+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-08-22 01:37:56+02	2020-08-22 01:38:14+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-08-22 01:38:07+02	2020-08-22 01:38:16+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-08-22 01:38:07+02	2020-08-22 01:38:16+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-08-22 01:38:15+02	2020-08-22 01:38:24+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-08-22 01:38:16+02	2020-08-22 01:38:26+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-08-22 01:38:17+02	2020-08-22 01:38:27+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-08-22 01:38:25+02	2020-08-22 01:38:36+02	devel	py-future	py37	py37-future-0.18.2
2020-08-22 01:38:27+02	2020-08-22 01:38:45+02	graphics	wayland	\N	wayland-1.18.0_4
2020-08-22 01:38:26+02	2020-08-22 01:38:47+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-08-22 01:38:37+02	2020-08-22 01:38:48+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-08-22 01:38:45+02	2020-08-22 01:38:55+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-08-22 01:38:48+02	2020-08-22 01:39:00+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-08-22 01:38:56+02	2020-08-22 01:39:07+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-08-22 01:39:07+02	2020-08-22 01:39:37+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-08-22 01:39:37+02	2020-08-22 01:39:51+02	textproc	itstool	\N	itstool-2.0.6
2020-08-22 01:39:53+02	2020-08-22 01:40:39+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-08-22 01:38:47+02	2020-08-22 01:44:17+02	devel	glib20	\N	glib-2.56.3_9,1
2020-08-22 01:44:17+02	2020-08-22 01:44:53+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-08-22 01:40:39+02	2020-08-22 01:45:29+02	lang	python27	\N	python27-2.7.18_1
2020-08-22 01:44:56+02	2020-08-22 01:45:41+02	misc	shared-mime-info	\N	shared-mime-info-1.10_2
2020-08-22 01:45:41+02	2020-08-22 01:46:22+02	textproc	libcroco	\N	libcroco-0.6.13
2020-08-22 01:46:22+02	2020-08-22 02:00:42+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-08-22 02:00:43+02	2020-08-22 02:00:57+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-08-22 02:00:59+02	2020-08-22 02:01:16+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-08-22 02:01:16+02	2020-08-22 02:04:31+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-08-22 02:04:32+02	2020-08-22 02:09:27+02	net	qt5-network	\N	qt5-network-5.15.0
2020-08-22 01:45:30+02	2020-08-22 02:09:28+02	lang	spidermonkey60	\N	spidermonkey60-60.9.0_4
2020-08-22 02:09:29+02	2020-08-22 02:11:01+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-08-22 02:11:04+02	2020-08-22 02:11:29+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-08-22 02:11:29+02	2020-08-22 02:13:13+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-08-22 02:09:29+02	2020-08-22 02:13:23+02	security	nss	\N	nss-3.55
2020-08-22 02:13:23+02	2020-08-22 02:14:35+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-08-22 02:14:36+02	2020-08-22 02:15:05+02	multimedia	libva	\N	libva-2.8.0
2020-08-22 02:15:06+02	2020-08-22 02:16:04+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-08-22 02:16:04+02	2020-08-22 02:20:24+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-08-22 02:20:24+02	2020-08-22 02:20:39+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-08-22 02:20:40+02	2020-08-22 02:20:57+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-08-22 02:20:58+02	2020-08-22 02:21:13+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-08-22 02:21:13+02	2020-08-22 02:21:31+02	textproc	py-rdflib	py37	py37-rdflib-5.0.0
2020-08-22 02:21:32+02	2020-08-22 02:21:53+02	audio	lv2	\N	lv2-1.18.0
2020-08-22 02:21:53+02	2020-08-22 02:22:28+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-08-22 02:22:28+02	2020-08-22 02:24:19+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-08-22 02:24:19+02	2020-08-22 02:24:33+02	devel	py-setuptools	py27	py27-setuptools-44.0.0
2020-08-22 02:24:33+02	2020-08-22 02:25:28+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-08-22 02:25:29+02	2020-08-22 02:26:50+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-08-22 02:26:51+02	2020-08-22 02:27:15+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-08-22 02:27:16+02	2020-08-22 02:27:34+02	textproc	py-toml	py37	py37-toml-0.10.1
2020-08-22 02:27:34+02	2020-08-22 02:27:47+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-08-22 02:13:14+02	2020-08-22 02:28:02+02	lang	ruby27	\N	ruby-2.7.1_1,1
2020-08-22 02:28:03+02	2020-08-22 02:28:31+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.8_7
2020-08-22 02:28:31+02	2020-08-22 02:29:14+02	www	php73-session	\N	php73-session-7.3.21
2020-08-22 02:29:15+02	2020-08-22 02:29:54+02	archivers	php73-zlib	\N	php73-zlib-7.3.21
2020-08-22 02:29:55+02	2020-08-22 02:30:34+02	databases	php73-pdo	\N	php73-pdo-7.3.21
2020-08-22 02:30:38+02	2020-08-22 02:31:16+02	devel	php73-json	\N	php73-json-7.3.21
2020-08-22 02:27:48+02	2020-08-22 02:31:23+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-08-22 02:31:16+02	2020-08-22 02:32:15+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-08-22 02:31:24+02	2020-08-22 02:32:16+02	converters	php73-mbstring	\N	php73-mbstring-7.3.21
2020-08-22 02:32:15+02	2020-08-22 02:32:38+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-08-22 02:32:17+02	2020-08-22 02:32:49+02	graphics	php73-gd	\N	php73-gd-7.3.21
2020-08-22 02:32:38+02	2020-08-22 02:33:19+02	security	php73-hash	\N	php73-hash-7.3.21
2020-08-22 02:32:50+02	2020-08-22 02:33:26+02	security	php73-openssl	\N	php73-openssl-7.3.21
2020-08-22 02:33:21+02	2020-08-22 02:33:52+02	textproc	php73-xml	\N	php73-xml-7.3.21
2020-08-22 02:33:27+02	2020-08-22 02:33:57+02	converters	php73-iconv	\N	php73-iconv-7.3.21
2020-08-22 02:33:52+02	2020-08-22 02:34:22+02	databases	pecl-memcache	php73	php73-pecl-memcache-4.0.5.2
2020-08-22 02:33:57+02	2020-08-22 02:34:35+02	databases	pecl-memcached	php73	php73-pecl-memcached-3.1.5
2020-08-22 02:34:23+02	2020-08-22 02:34:54+02	databases	php73-pdo_pgsql	\N	php73-pdo_pgsql-7.3.21
2020-08-22 02:34:36+02	2020-08-22 02:35:07+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.21
2020-08-22 02:34:54+02	2020-08-22 02:35:26+02	databases	php73-pgsql	\N	php73-pgsql-7.3.21
2020-08-22 02:35:08+02	2020-08-22 02:35:36+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.21
2020-08-22 02:35:27+02	2020-08-22 02:35:56+02	devel	php73-gettext	\N	php73-gettext-7.3.21
2020-08-22 02:35:36+02	2020-08-22 02:36:12+02	ftp	php73-curl	\N	php73-curl-7.3.21
2020-08-22 02:35:57+02	2020-08-22 02:36:22+02	math	pecl-bitset	php73	php73-pecl-bitset-3.0.1
2020-08-22 02:36:12+02	2020-08-22 02:36:45+02	math	php73-bcmath	\N	php73-bcmath-7.3.21
2020-08-22 02:36:46+02	2020-08-22 02:37:18+02	net	php73-soap	\N	php73-soap-7.3.21
2020-08-22 02:37:18+02	2020-08-22 02:37:44+02	net	php73-sockets	\N	php73-sockets-7.3.21
2020-08-22 02:37:44+02	2020-08-22 02:38:05+02	security	pecl-mcrypt	php73	php73-pecl-mcrypt-1.0.3
2020-08-22 02:38:06+02	2020-08-22 02:38:34+02	sysutils	php73-posix	\N	php73-posix-7.3.21
2020-08-22 02:36:23+02	2020-08-22 02:39:17+02	net	php73-ldap	\N	php73-ldap-7.3.21
2020-08-22 02:38:35+02	2020-08-22 02:39:28+02	sysutils	policykit	\N	policykit-0.9_10
2020-08-22 02:39:18+02	2020-08-22 02:39:46+02	textproc	php73-ctype	\N	php73-ctype-7.3.21
2020-08-22 02:39:29+02	2020-08-22 02:40:10+02	textproc	php73-dom	\N	php73-dom-7.3.21
2020-08-22 02:39:47+02	2020-08-22 02:40:20+02	textproc	php73-simplexml	\N	php73-simplexml-7.3.21
2020-08-22 02:40:21+02	2020-08-22 02:41:02+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-08-22 02:40:11+02	2020-08-22 02:41:04+02	www	php73-opcache	\N	php73-opcache-7.3.21
2020-08-22 02:41:02+02	2020-08-22 02:41:17+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-08-22 02:41:05+02	2020-08-22 02:41:39+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-08-22 02:41:40+02	2020-08-22 02:42:29+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-08-22 02:42:30+02	2020-08-22 02:48:59+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-08-22 02:49:00+02	2020-08-22 02:49:56+02	www	py-django22	py37	py37-django22-2.2.15
2020-08-22 02:49:57+02	2020-08-22 02:58:51+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-08-22 02:58:52+02	2020-08-22 02:59:14+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-08-22 02:59:14+02	2020-08-22 02:59:27+02	devel	py-six	py27	py27-six-1.15.0
2020-08-22 02:41:17+02	2020-08-22 03:05:19+02	devel	gdb	py37	gdb-9.2_1
2020-08-22 02:59:28+02	2020-08-22 03:05:22+02	devel	git	\N	git-2.28.0
2020-08-22 03:05:19+02	2020-08-22 03:05:37+02	devel	py-pytz	py27	py27-pytz-2020.1,1
2020-08-22 03:05:38+02	2020-08-22 03:08:00+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-08-22 03:08:01+02	2020-08-22 03:08:50+02	devel	py-babel	py27	py27-Babel-2.8.0
2020-08-22 03:08:53+02	2020-08-22 03:09:13+02	textproc	py-MarkupSafe	py27	py27-MarkupSafe-1.1.1
2020-08-22 03:09:14+02	2020-08-22 03:09:33+02	devel	py-Jinja2	py27	py27-Jinja2-2.10.1
2020-08-22 03:09:34+02	2020-08-22 03:09:50+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-08-22 03:09:51+02	2020-08-22 03:10:06+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-08-22 03:10:06+02	2020-08-22 03:10:26+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-08-22 03:10:26+02	2020-08-22 03:13:31+02	security	gnupg	\N	gnupg-2.2.21
2020-08-22 03:13:31+02	2020-08-22 03:14:08+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.8_6
2020-08-22 01:39:00+02	2020-08-22 09:29:36+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-08-22 09:29:45+02	2020-08-22 09:37:15+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-08-22 09:37:15+02	2020-08-22 09:41:26+02	graphics	cairo	\N	cairo-1.16.0,2
2020-08-22 09:41:27+02	2020-08-22 09:44:13+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-08-22 09:44:13+02	2020-08-22 09:49:38+02	print	harfbuzz	\N	harfbuzz-2.7.1
2020-08-22 09:49:45+02	2020-08-22 09:51:37+02	net	avahi-app	\N	avahi-app-0.7_3
2020-08-22 09:51:37+02	2020-08-22 09:53:27+02	print	cups	\N	cups-2.3.3_1
2020-08-22 09:53:27+02	2020-08-22 09:56:20+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-22 09:56:21+02	2020-08-22 09:56:57+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-08-22 03:05:23+02	2020-08-22 09:59:56+02	lang	rust	\N	rust-1.45.2
2020-08-22 10:00:03+02	2020-08-22 10:02:00+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-08-22 10:02:01+02	2020-08-22 10:02:33+02	accessibility	atk	\N	atk-2.36.0
2020-08-22 09:56:58+02	2020-08-22 10:02:35+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_8
2020-08-22 10:02:34+02	2020-08-22 10:03:09+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-08-22 10:03:09+02	2020-08-22 10:04:23+02	sysutils	polkit	\N	polkit-0.116
2020-08-22 10:04:23+02	2020-08-22 10:05:52+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-08-22 10:05:53+02	2020-08-22 10:06:55+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-08-22 10:06:56+02	2020-08-22 10:07:23+02	devel	libgudev	\N	libgudev-233
2020-08-22 10:02:35+02	2020-08-22 10:08:35+02	graphics	graphviz	\N	graphviz-2.44.1_1
2020-08-22 10:07:24+02	2020-08-22 10:10:41+02	graphics	colord	\N	colord-1.3.5
2020-08-22 10:10:42+02	2020-08-22 10:11:07+02	x11	libwacom	\N	libwacom-1.4.1
2020-08-22 10:11:08+02	2020-08-22 10:11:41+02	x11	libinput	\N	libinput-1.15.6
2020-08-22 10:11:42+02	2020-08-22 10:12:44+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-08-22 10:08:36+02	2020-08-22 10:13:16+02	lang	vala	\N	vala-0.40.15,1
2020-08-22 10:13:16+02	2020-08-22 10:14:33+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-22 10:12:44+02	2020-08-22 10:14:44+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-22 10:14:45+02	2020-08-22 10:33:39+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-22 10:33:40+02	2020-08-22 10:34:59+02	graphics	libGLU	\N	libGLU-9.0.1
2020-08-22 10:35:00+02	2020-08-22 10:35:39+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.1
2020-08-22 10:14:34+02	2020-08-22 10:37:53+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-22 10:35:40+02	2020-08-22 10:48:57+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-22 10:37:54+02	2020-08-22 10:51:26+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-22 10:48:58+02	2020-08-22 10:52:45+02	graphics	poppler	\N	poppler-20.08.0_1
2020-08-22 10:52:45+02	2020-08-22 10:56:53+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-08-22 10:56:56+02	2020-08-22 11:01:54+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-08-22 11:01:54+02	2020-08-22 11:02:30+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-08-22 11:02:31+02	2020-08-22 11:03:17+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-08-22 11:03:21+02	2020-08-22 11:04:37+02	devel	dconf	\N	dconf-0.36.0
2020-08-22 11:04:42+02	2020-08-22 11:06:42+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-22 11:06:42+02	2020-08-22 11:07:29+02	devel	json-glib	\N	json-glib-1.4.4
2020-08-22 11:07:29+02	2020-08-22 11:11:09+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-08-22 11:11:10+02	2020-08-22 11:14:36+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-08-22 11:14:37+02	2020-08-22 11:25:42+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-22 10:51:28+02	2020-08-22 11:27:42+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-22 11:27:43+02	2020-08-22 11:31:19+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-08-22 11:31:20+02	2020-08-22 11:31:47+02	devel	libnotify	\N	libnotify-0.7.8
2020-08-22 11:31:47+02	2020-08-22 11:34:00+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-22 11:34:05+02	2020-08-22 11:35:04+02	security	libsecret	\N	libsecret-0.18.6_1
2020-08-22 11:35:04+02	2020-08-22 11:36:21+02	graphics	frei0r	\N	frei0r-1.6.1
2020-08-22 11:36:22+02	2020-08-22 11:39:33+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-08-22 11:25:43+02	2020-08-22 11:49:20+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-22 03:14:09+02	2020-08-22 11:50:46+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-08-22 11:49:21+02	2020-08-22 11:52:55+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-22 11:52:56+02	2020-08-22 11:54:14+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-22 11:54:15+02	2020-08-22 11:54:36+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-08-22 11:39:34+02	2020-08-22 11:54:44+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-22 11:54:36+02	2020-08-22 11:54:52+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-08-22 11:54:52+02	2020-08-22 11:55:19+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-08-22 11:54:45+02	2020-08-22 11:56:32+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-22 11:55:20+02	2020-08-22 11:56:47+02	graphics	graphene	\N	graphene-1.10.0
2020-08-22 11:56:33+02	2020-08-22 11:56:47+02	www	libwpe	\N	libwpe-1.4.0.1
2020-08-22 11:50:59+02	2020-08-22 11:57:19+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-22 11:56:48+02	2020-08-22 11:57:46+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.73.0
2020-08-22 11:57:46+02	2020-08-22 11:58:07+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-08-22 11:57:20+02	2020-08-22 11:58:26+02	net	geoclue	\N	geoclue-2.5.5
2020-08-22 11:56:48+02	2020-08-22 11:58:36+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-08-22 11:58:07+02	2020-08-22 11:58:49+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-22 11:58:26+02	2020-08-22 11:58:58+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-22 11:58:58+02	2020-08-22 11:59:28+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-08-22 11:59:28+02	2020-08-22 11:59:57+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-08-22 11:59:58+02	2020-08-22 12:02:17+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-22 12:02:18+02	2020-08-22 12:02:39+02	print	libraqm	\N	libraqm-0.6.0
2020-08-22 12:02:40+02	2020-08-22 12:03:42+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-08-22 12:03:42+02	2020-08-22 12:12:02+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-22 12:12:02+02	2020-08-22 12:13:34+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-22 11:58:50+02	2020-08-22 12:15:26+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-22 12:15:27+02	2020-08-22 12:17:54+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.73.0
2020-08-22 12:17:57+02	2020-08-22 12:19:09+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-22 12:19:10+02	2020-08-22 12:20:41+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-22 12:20:42+02	2020-08-22 12:22:10+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-22 12:22:14+02	2020-08-22 12:24:35+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-22 12:24:35+02	2020-08-22 12:25:53+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-22 12:25:54+02	2020-08-22 12:28:32+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-08-22 12:13:35+02	2020-08-22 12:28:53+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-08-22 12:28:32+02	2020-08-22 12:30:14+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-08-22 12:28:54+02	2020-08-22 12:30:16+02	archivers	kf5-karchive	\N	kf5-karchive-5.73.0
2020-08-22 12:30:15+02	2020-08-22 12:32:01+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-08-22 12:32:02+02	2020-08-22 12:33:02+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-08-22 12:33:03+02	2020-08-22 12:34:56+02	sysutils	hal	\N	hal-0.5.14_34
2020-08-22 12:30:16+02	2020-08-22 12:35:36+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-22 12:34:57+02	2020-08-22 12:36:30+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-22 12:36:34+02	2020-08-22 12:39:07+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-22 12:35:38+02	2020-08-22 12:40:53+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_3,1
2020-08-22 12:39:07+02	2020-08-22 12:41:10+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-08-22 12:40:54+02	2020-08-22 12:51:06+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-22 12:51:07+02	2020-08-22 12:54:52+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-08-22 12:54:53+02	2020-08-22 12:57:22+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-08-22 12:41:10+02	2020-08-22 12:58:20+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-22 12:57:23+02	2020-08-22 12:58:40+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-22 12:58:25+02	2020-08-22 13:02:13+02	graphics	poppler-glib	\N	poppler-glib-20.08.0_1
2020-08-22 12:58:41+02	2020-08-22 13:02:46+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-08-22 13:02:46+02	2020-08-22 13:03:59+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-08-22 13:02:14+02	2020-08-22 13:05:17+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-08-22 13:04:00+02	2020-08-22 13:05:27+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-08-22 13:05:28+02	2020-08-22 13:07:06+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-22 13:05:17+02	2020-08-22 13:09:25+02	graphics	glew	\N	glew-2.2.0
2020-08-22 13:09:26+02	2020-08-22 13:10:19+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-08-22 13:07:06+02	2020-08-22 13:10:58+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-08-22 13:10:19+02	2020-08-22 13:11:10+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-08-22 13:10:59+02	2020-08-22 13:12:15+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-22 13:11:10+02	2020-08-22 13:12:25+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-22 13:12:25+02	2020-08-22 13:13:20+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-08-22 13:12:16+02	2020-08-22 13:13:33+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.73.0
2020-08-22 13:13:33+02	2020-08-22 13:15:26+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-08-22 13:13:21+02	2020-08-22 13:16:57+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-22 13:15:26+02	2020-08-22 13:18:19+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-08-22 13:16:57+02	2020-08-22 13:19:56+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-22 13:18:22+02	2020-08-22 13:20:07+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-08-22 13:19:57+02	2020-08-22 13:20:20+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-08-22 13:20:08+02	2020-08-22 13:21:33+02	net	remmina	\N	remmina-1.3.9
2020-08-22 13:21:33+02	2020-08-22 13:22:28+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-08-22 13:20:20+02	2020-08-22 13:23:23+02	security	gcr	\N	gcr-3.28.0
2020-08-22 13:22:28+02	2020-08-22 13:23:51+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-08-22 13:23:24+02	2020-08-22 13:23:54+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-08-22 13:23:53+02	2020-08-22 13:24:31+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-08-22 13:23:54+02	2020-08-22 13:24:38+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-08-22 13:24:32+02	2020-08-22 13:25:31+02	devel	libgtop	\N	libgtop-2.40.0
2020-08-22 13:25:32+02	2020-08-22 13:26:43+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-22 13:26:43+02	2020-08-22 13:27:31+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-08-22 13:27:31+02	2020-08-22 13:28:50+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-22 13:28:50+02	2020-08-22 13:29:55+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-08-22 13:29:56+02	2020-08-22 13:30:32+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-08-22 13:30:33+02	2020-08-22 13:31:48+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-08-22 13:31:50+02	2020-08-22 13:32:45+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-08-22 13:24:39+02	2020-08-22 13:38:43+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-22 13:38:44+02	2020-08-22 13:41:57+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.73.0
2020-08-22 13:32:46+02	2020-08-22 13:42:31+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-22 13:41:58+02	2020-08-22 13:43:04+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-22 13:42:31+02	2020-08-22 13:44:18+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-08-22 13:43:05+02	2020-08-22 13:44:37+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-08-22 13:44:18+02	2020-08-22 13:46:05+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-22 13:46:06+02	2020-08-22 13:47:41+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-08-22 13:44:38+02	2020-08-22 14:10:21+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-22 14:10:22+02	2020-08-22 14:11:12+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-22 14:11:12+02	2020-08-22 14:14:11+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-22 14:14:12+02	2020-08-22 14:14:41+02	print	gl2ps	\N	gl2ps-1.4.0
2020-08-22 14:14:43+02	2020-08-22 14:16:32+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-08-22 11:58:37+02	2020-08-22 14:18:15+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-22 14:16:33+02	2020-08-22 14:19:11+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-22 14:19:11+02	2020-08-22 14:21:28+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-22 14:21:30+02	2020-08-22 14:21:46+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-08-22 14:21:48+02	2020-08-22 14:23:45+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-08-22 14:23:46+02	2020-08-22 14:27:48+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-08-22 14:27:48+02	2020-08-22 14:34:56+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-08-22 14:34:57+02	2020-08-22 14:41:32+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-22 14:41:35+02	2020-08-22 14:42:31+02	security	gpgme	\N	gpgme-1.14.0
2020-08-22 14:42:31+02	2020-08-22 14:43:34+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-08-22 14:43:35+02	2020-08-22 14:46:20+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-08-22 14:18:16+02	2020-08-22 14:57:37+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-22 14:46:21+02	2020-08-22 14:59:18+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-22 14:57:38+02	2020-08-22 15:01:02+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-22 15:01:03+02	2020-08-22 15:02:55+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-22 15:02:56+02	2020-08-22 15:04:43+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-08-22 15:04:43+02	2020-08-22 15:13:47+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-22 15:13:48+02	2020-08-22 15:14:21+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-08-22 15:14:22+02	2020-08-22 15:14:48+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-08-22 15:14:48+02	2020-08-22 15:16:17+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-22 15:16:18+02	2020-08-22 15:17:16+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-22 15:17:16+02	2020-08-22 15:18:33+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-22 15:18:33+02	2020-08-22 15:18:51+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-08-22 15:18:51+02	2020-08-22 15:19:07+02	devel	py-pycparser	py27	py27-pycparser-2.20
2020-08-22 15:19:08+02	2020-08-22 15:20:33+02	devel	talloc	\N	talloc-2.3.1
2020-08-22 13:47:42+02	2020-08-22 15:20:39+02	math	vtk8	\N	vtk8-8.2.0
2020-08-22 15:20:36+02	2020-08-22 15:24:48+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-08-22 15:24:48+02	2020-08-22 15:27:34+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-22 15:27:35+02	2020-08-22 15:28:40+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-08-22 15:28:41+02	2020-08-22 15:29:06+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7
2020-08-22 15:29:07+02	2020-08-22 15:30:22+02	sysutils	upower	\N	upower-0.99.11
2020-08-22 15:20:41+02	2020-08-22 18:06:34+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-22 18:06:52+02	2020-08-22 18:09:15+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-08-22 18:09:16+02	2020-08-22 18:10:03+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-08-22 18:10:03+02	2020-08-22 18:16:19+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-22 18:16:21+02	2020-08-22 18:16:44+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-08-22 18:16:44+02	2020-08-22 18:18:23+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-22 18:18:24+02	2020-08-22 18:19:58+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-22 18:19:59+02	2020-08-22 18:21:34+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-22 18:21:35+02	2020-08-22 18:23:14+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-22 18:23:15+02	2020-08-22 18:24:48+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-22 18:24:49+02	2020-08-22 18:28:55+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-08-22 18:29:03+02	2020-08-22 18:30:26+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-22 18:30:29+02	2020-08-22 18:32:05+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-22 18:32:06+02	2020-08-22 18:35:19+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-08-22 18:35:19+02	2020-08-22 18:37:35+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-08-22 18:37:38+02	2020-08-22 18:38:27+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-22 18:38:28+02	2020-08-22 19:13:31+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-22 19:13:35+02	2020-08-22 19:15:14+02	archivers	gcab	\N	gcab-1.3
2020-08-22 19:15:19+02	2020-08-22 19:17:51+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-08-22 14:59:19+02	2020-08-22 19:25:38+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-22 19:26:05+02	2020-08-22 19:26:42+02	converters	py-webencodings	py27	py27-webencodings-0.5.1
2020-08-22 19:26:44+02	2020-08-22 19:39:39+02	databases	tdb	\N	tdb-1.4.3,1
2020-08-22 19:39:44+02	2020-08-22 19:41:33+02	devel	gamin	\N	gamin-0.1.10_10
2020-08-22 19:41:40+02	2020-08-22 19:49:20+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-22 19:49:24+02	2020-08-22 19:54:15+02	devel	librest	\N	rest-0.7.93
2020-08-22 19:54:17+02	2020-08-22 19:55:29+02	devel	py-cffi	py27	py27-cffi-1.14.2
2020-08-22 19:55:33+02	2020-08-22 19:56:30+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-08-22 19:56:33+02	2020-08-22 20:05:30+02	devel	tevent	\N	tevent-0.10.2
2020-08-22 20:05:36+02	2020-08-22 20:11:50+02	graphics	poppler-utils	\N	poppler-utils-20.08.0_1
2020-08-22 20:11:53+02	2020-08-22 21:08:00+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-22 21:08:05+02	2020-08-22 21:10:06+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-08-22 21:10:09+02	2020-08-22 21:15:56+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-08-22 21:15:58+02	2020-08-22 21:19:17+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-22 21:19:19+02	2020-08-22 21:22:36+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-22 21:22:41+02	2020-08-22 21:25:17+02	sysutils	bsdisks	\N	bsdisks-0.21
2020-08-22 21:25:19+02	2020-08-22 21:33:49+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-08-22 21:33:52+02	2020-08-22 21:34:37+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-08-22 19:17:52+02	2020-08-22 22:14:27+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-22 22:14:28+02	2020-08-22 22:15:59+02	x11	babl	\N	babl-0.1.78
2020-08-22 22:16:00+02	2020-08-22 22:17:45+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-22 22:17:45+02	2020-08-22 22:18:21+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-08-22 22:18:21+02	2020-08-22 22:19:42+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-22 22:19:43+02	2020-08-22 22:24:20+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-22 22:24:21+02	2020-08-22 22:28:19+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_3,1
2020-08-22 22:28:19+02	2020-08-22 22:41:15+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-22 22:41:16+02	2020-08-22 22:46:34+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-22 22:46:35+02	2020-08-22 22:50:53+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-22 22:50:55+02	2020-08-22 22:58:27+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-22 22:58:30+02	2020-08-22 23:04:25+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-22 23:04:28+02	2020-08-22 23:16:32+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-08-22 23:16:35+02	2020-08-22 23:21:26+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-22 23:21:28+02	2020-08-22 23:28:46+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-08-22 23:28:49+02	2020-08-22 23:29:33+02	devel	py-click	py37	py37-click-7.0
2020-08-22 23:29:36+02	2020-08-22 23:30:16+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-08-22 23:30:18+02	2020-08-22 23:30:51+02	devel	py-olefile	py37	py37-olefile-0.46
2020-08-22 23:30:55+02	2020-08-22 23:31:31+02	devel	py-ply	py37	py37-ply-3.11
2020-08-22 23:31:32+02	2020-08-22 23:32:04+02	devel	py-ply	py27	py27-ply-3.11
2020-08-22 23:32:07+02	2020-08-22 23:32:48+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-08-22 23:32:51+02	2020-08-22 23:33:25+02	devel	py-zipp	py37	py37-zipp-3.1.0
2020-08-22 23:33:27+02	2020-08-22 23:34:17+02	devel	scons	py37	scons-py37-3.1.2
2020-08-22 23:34:18+02	2020-08-22 23:43:08+02	graphics	gegl	\N	gegl-0.4.24_4
2020-08-22 23:43:10+02	2020-08-22 23:45:08+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-08-22 23:45:11+02	2020-08-22 23:47:54+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-08-22 23:47:56+02	2020-08-22 23:49:08+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-08-22 23:49:09+02	2020-08-23 00:16:46+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-22 21:34:41+02	2020-08-23 00:20:27+02	www	node	\N	node-14.8.0
2020-08-23 00:16:47+02	2020-08-23 00:20:39+02	mail	mailutils	\N	mailutils-3.10
2020-08-23 00:20:28+02	2020-08-23 00:21:13+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0,1
2020-08-23 00:20:39+02	2020-08-23 00:22:51+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-23 00:22:52+02	2020-08-23 00:23:11+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-08-23 00:23:12+02	2020-08-23 00:24:19+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-23 00:24:20+02	2020-08-23 00:25:28+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-23 00:25:28+02	2020-08-23 00:28:53+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-23 00:28:54+02	2020-08-23 00:29:56+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-08-23 00:29:59+02	2020-08-23 00:32:30+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-23 00:32:31+02	2020-08-23 00:33:50+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-23 00:33:51+02	2020-08-23 00:34:09+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-08-23 00:34:10+02	2020-08-23 00:34:23+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-08-23 00:34:24+02	2020-08-23 00:34:36+02	www	py-html5lib	py27	py27-html5lib-1.0.1
2020-08-23 00:34:37+02	2020-08-23 00:34:50+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-08-23 00:34:51+02	2020-08-23 00:35:10+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-08-23 00:35:15+02	2020-08-23 00:36:16+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-08-23 00:36:18+02	2020-08-23 00:37:41+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-08-23 00:37:42+02	2020-08-23 00:41:00+02	x11	xorg	\N	xorg-7.7_3
2020-08-23 00:41:01+02	2020-08-23 00:43:19+02	x11	zenity	\N	zenity-3.32.0
2020-08-23 00:43:20+02	2020-08-23 00:44:23+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-08-23 00:21:13+02	2020-08-23 00:44:23+02	net	samba410	\N	samba410-4.10.17
2020-08-23 00:44:24+02	2020-08-23 00:44:35+02	databases	py-redis	py37	py37-redis-3.5.3
2020-08-23 00:44:24+02	2020-08-23 00:44:36+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-08-23 00:44:38+02	2020-08-23 00:48:01+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-23 00:48:02+02	2020-08-23 00:49:32+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-23 00:49:33+02	2020-08-23 00:49:48+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-08-23 00:49:49+02	2020-08-23 00:50:05+02	devel	py-appdirs	py37	py37-appdirs-1.4.4
2020-08-23 00:50:05+02	2020-08-23 00:50:18+02	devel	py-asn1crypto	py27	py27-asn1crypto-1.3.0
2020-08-23 00:50:19+02	2020-08-23 00:50:31+02	devel	py-backports	py37	py37-backports-1
2020-08-23 00:50:32+02	2020-08-23 00:50:45+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-08-23 00:50:45+02	2020-08-23 00:51:02+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-08-23 00:51:03+02	2020-08-23 00:51:16+02	devel	py-enum34	py27	py27-enum34-1.1.6
2020-08-23 00:51:16+02	2020-08-23 00:52:08+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-08-23 00:52:09+02	2020-08-23 00:52:22+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-1.7.0
2020-08-23 00:52:23+02	2020-08-23 00:52:39+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-08-23 00:52:40+02	2020-08-23 00:52:53+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-08-23 00:52:53+02	2020-08-23 00:53:09+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-08-23 00:53:09+02	2020-08-23 00:53:23+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-08-23 00:53:24+02	2020-08-23 00:53:41+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-08-23 00:53:42+02	2020-08-23 00:53:54+02	devel	py-wheel	py27	py27-wheel-0.30.0_1
2020-08-23 00:53:54+02	2020-08-23 00:54:11+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-08-23 00:44:36+02	2020-08-23 00:54:37+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-10-06 07:44:51+02	2020-10-06 07:45:05+02	www	p5-CGI	\N	p5-CGI-4.51
2020-08-23 00:54:12+02	2020-08-23 00:56:20+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-23 00:54:38+02	2020-08-23 00:57:04+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-23 00:56:21+02	2020-08-23 00:58:31+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-23 00:58:31+02	2020-08-23 00:58:45+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-08-23 00:58:45+02	2020-08-23 01:03:48+02	dns	bind-tools	\N	bind-tools-9.16.6
2020-08-23 00:57:05+02	2020-08-23 01:03:59+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-23 01:03:49+02	2020-08-23 01:09:58+02	editors	emacs	full	emacs-27.1,3
2020-08-23 01:09:58+02	2020-08-23 01:10:22+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-08-23 01:04:00+02	2020-08-23 01:13:07+02	graphics	Coin	\N	Coin-4.0.0_1
2020-08-23 01:13:08+02	2020-08-23 01:14:02+02	graphics	glfw	\N	glfw-3.3.2
2020-08-23 01:14:02+02	2020-08-23 01:17:15+02	graphics	movit	\N	movit-1.6.2_2
2020-08-23 01:10:23+02	2020-08-23 01:29:08+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-08-23 01:29:09+02	2020-08-23 01:30:16+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-08-23 01:30:17+02	2020-08-23 01:31:37+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-08-23 01:31:40+02	2020-08-23 01:32:24+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-08-23 01:32:24+02	2020-08-23 01:35:19+02	lang	cython	py27	py27-cython-0.29.15
2020-08-22 15:30:28+02	2020-08-23 01:37:00+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-23 01:37:02+02	2020-08-23 01:38:44+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-23 01:38:44+02	2020-08-23 01:44:08+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-08-23 01:17:16+02	2020-08-23 01:44:29+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-23 01:44:09+02	2020-08-23 01:44:32+02	net	gssdp	\N	gssdp-1.2.3
2020-08-23 01:44:29+02	2020-08-23 01:44:39+02	net	py-ipaddress	py27	py27-ipaddress-1.0.23
2020-08-23 01:44:32+02	2020-08-23 01:45:27+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-08-23 01:44:40+02	2020-08-23 01:45:59+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-08-23 01:45:28+02	2020-08-23 01:53:25+02	print	fontforge	\N	fontforge-20190801_2
2020-08-23 01:46:00+02	2020-08-23 02:01:06+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-08-23 02:01:06+02	2020-08-23 02:01:21+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-08-23 02:01:22+02	2020-08-23 02:02:27+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-23 02:02:28+02	2020-08-23 02:04:28+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-08-23 02:04:28+02	2020-08-23 02:04:45+02	sysutils	py-psutil	py37	py37-psutil-5.7.2
2020-08-23 02:04:45+02	2020-08-23 02:04:56+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-08-23 02:04:57+02	2020-08-23 02:05:09+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-08-23 02:05:10+02	2020-08-23 02:05:21+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-08-23 01:53:26+02	2020-08-23 02:12:02+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-08-23 02:12:03+02	2020-08-23 02:12:16+02	www	py-h2	py37	py37-h2-3.2.0
2020-08-23 02:12:16+02	2020-08-23 02:12:30+02	www	py-hyperlink	py37	py37-hyperlink-20.0.1
2020-08-23 02:12:31+02	2020-08-23 02:12:44+02	www	py-priority	py37	py37-priority-1.3.0
2020-08-23 02:12:44+02	2020-08-23 02:14:20+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-23 02:14:21+02	2020-08-23 02:16:19+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-23 02:16:19+02	2020-08-23 02:17:39+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-23 02:17:40+02	2020-08-23 02:18:39+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-23 02:18:39+02	2020-08-23 02:20:45+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-23 02:20:46+02	2020-08-23 02:26:57+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-23 02:26:58+02	2020-08-23 02:32:59+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-23 02:32:59+02	2020-08-23 02:34:16+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-23 02:34:17+02	2020-08-23 02:38:39+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-23 02:38:43+02	2020-08-23 02:50:02+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-23 02:50:03+02	2020-08-23 03:02:10+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-23 03:02:11+02	2020-08-23 03:03:49+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-23 03:03:50+02	2020-08-23 03:07:00+02	x11-wm	marco	\N	marco-1.22.4
2020-08-23 03:07:01+02	2020-08-23 03:08:14+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-23 03:08:14+02	2020-08-23 03:08:57+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-08-23 03:08:58+02	2020-08-23 03:11:29+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-23 03:11:30+02	2020-08-23 03:12:55+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-08-23 03:12:55+02	2020-08-23 03:17:33+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-23 03:17:34+02	2020-08-23 03:21:20+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-23 03:21:20+02	2020-08-23 03:28:24+02	devel	ncurses	\N	ncurses-6.2.20200817
2020-08-23 03:28:25+02	2020-08-23 03:28:37+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-08-23 03:28:37+02	2020-08-23 03:28:50+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-08-23 03:28:51+02	2020-08-23 03:29:04+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-08-23 03:29:05+02	2020-08-23 03:29:17+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-08-23 03:29:18+02	2020-08-23 03:29:30+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-08-23 03:29:31+02	2020-08-23 03:29:45+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-08-23 03:29:45+02	2020-08-23 03:29:56+02	devel	py-pyasn1	py27	py27-pyasn1-0.4.7
2020-08-23 03:29:56+02	2020-08-23 03:30:47+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-08-23 01:35:20+02	2020-08-23 03:40:39+02	lang	ghc	\N	ghc-8.10.2
2020-08-23 03:40:40+02	2020-08-23 03:56:33+02	devel	hs-cabal-install	\N	hs-cabal-install-3.2.0.0
2020-08-23 03:56:33+02	2020-08-23 04:05:36+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-23 04:05:37+02	2020-08-23 04:07:32+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-23 04:07:32+02	2020-08-23 04:07:54+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-08-23 04:07:55+02	2020-08-23 04:09:51+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-23 04:09:51+02	2020-08-23 04:11:10+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-08-23 04:11:10+02	2020-08-23 04:11:46+02	graphics	libgxps	\N	libgxps-0.3.1
2020-08-23 04:11:46+02	2020-08-23 04:13:57+02	graphics	libheif	\N	libheif-1.6.2
2020-08-23 04:13:58+02	2020-08-23 04:14:31+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-08-23 04:14:32+02	2020-08-23 04:20:08+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-08-23 04:20:09+02	2020-08-23 04:20:44+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-23 04:20:44+02	2020-08-23 04:21:48+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-23 04:21:49+02	2020-08-23 04:22:53+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-23 04:22:53+02	2020-08-23 04:24:13+02	graphics	vigra	\N	vigra-1.11.1_12
2020-08-23 04:24:14+02	2020-08-23 04:28:58+02	lang	python38	\N	python38-3.8.5
2020-08-23 03:30:48+02	2020-08-23 04:45:32+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-23 04:28:58+02	2020-08-23 04:50:09+02	math	R	\N	R-4.0.2
2020-08-23 04:50:11+02	2020-08-23 04:51:21+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-08-23 04:51:22+02	2020-08-23 04:52:11+02	multimedia	libass	\N	libass-0.14.0
2020-08-23 04:45:33+02	2020-08-23 04:52:33+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-23 04:52:12+02	2020-08-23 04:54:32+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-23 04:54:32+02	2020-08-23 04:55:09+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-23 04:52:34+02	2020-08-23 04:55:16+02	net	freerdp	\N	freerdp-2.2.0
2020-08-23 04:55:10+02	2020-08-23 04:55:25+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-08-23 04:55:17+02	2020-08-23 04:55:28+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-08-23 04:55:26+02	2020-08-23 04:56:12+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-23 04:55:29+02	2020-08-23 04:56:16+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-23 04:56:17+02	2020-08-23 04:56:44+02	print	libspectre	\N	libspectre-0.2.9
2020-08-23 04:56:45+02	2020-08-23 04:58:57+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-23 04:56:12+02	2020-08-23 05:00:08+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-23 04:58:57+02	2020-08-23 05:05:18+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-23 05:00:09+02	2020-08-23 05:05:52+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-23 05:05:53+02	2020-08-23 05:06:07+02	security	py-bcrypt	py27	py27-bcrypt-3.1.7
2020-08-23 05:05:19+02	2020-08-23 05:06:08+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-08-23 05:06:09+02	2020-08-23 05:06:49+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-08-23 05:06:07+02	2020-08-23 05:06:53+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-08-23 05:06:49+02	2020-08-23 05:07:07+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-08-23 05:06:53+02	2020-08-23 05:07:10+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-08-23 05:07:10+02	2020-08-23 05:07:31+02	sysutils	hal-info	\N	hal-info-20091130
2020-08-23 05:07:08+02	2020-08-23 05:08:04+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-08-23 05:08:04+02	2020-08-23 05:09:00+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-23 05:07:31+02	2020-08-23 05:11:12+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-23 05:09:01+02	2020-08-23 05:12:52+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-23 05:11:13+02	2020-08-23 05:13:09+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-23 05:13:10+02	2020-08-23 05:13:24+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-08-23 05:12:53+02	2020-08-23 05:13:27+02	textproc	enchant	\N	enchant-1.6.0_9
2020-08-23 05:13:24+02	2020-08-23 05:13:36+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-08-23 05:13:27+02	2020-08-23 05:13:39+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-08-23 05:13:36+02	2020-08-23 05:13:58+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-08-23 05:13:39+02	2020-08-23 05:14:22+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-23 05:14:23+02	2020-08-23 05:14:34+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-08-23 05:14:34+02	2020-08-23 05:15:12+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-23 05:13:59+02	2020-08-23 05:15:21+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-23 05:15:12+02	2020-08-23 05:16:34+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-23 05:16:35+02	2020-08-23 05:17:15+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-08-23 05:15:22+02	2020-08-23 05:17:15+02	x11-servers	xephyr	\N	xephyr-1.20.8_3,1
2020-08-23 05:17:16+02	2020-08-23 05:18:04+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-23 05:17:16+02	2020-08-23 05:18:18+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-23 05:18:19+02	2020-08-23 05:20:12+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-23 05:18:05+02	2020-08-23 05:20:16+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-23 05:20:13+02	2020-08-23 05:22:52+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-23 05:22:52+02	2020-08-23 05:25:49+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-23 05:25:49+02	2020-08-23 05:26:53+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-08-23 05:26:54+02	2020-08-23 05:27:44+02	audio	faudio	\N	FAudio-20.08
2020-08-23 05:27:45+02	2020-08-23 05:29:07+02	audio	mate-media	\N	mate-media-1.24.1
2020-08-23 05:29:07+02	2020-08-23 05:29:28+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-08-23 05:29:29+02	2020-08-23 05:31:09+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-23 05:31:09+02	2020-08-23 05:32:50+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-23 05:32:51+02	2020-08-23 05:37:25+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-23 05:37:25+02	2020-08-23 05:38:25+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-23 05:38:26+02	2020-08-23 05:39:25+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-23 05:39:26+02	2020-08-23 05:40:21+02	devel	adacurses	\N	adacurses-20180127
2020-08-23 05:40:21+02	2020-08-23 05:40:45+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-08-23 05:40:46+02	2020-08-23 05:41:09+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-08-23 05:41:10+02	2020-08-23 05:42:38+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-08-23 05:42:38+02	2020-08-23 05:44:16+02	devel	libarea	\N	libarea-20160313_13
2020-08-23 05:44:16+02	2020-08-23 05:46:09+02	devel	ode	\N	ode-0.13_4,1
2020-08-23 05:46:09+02	2020-08-23 05:46:21+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-08-23 05:20:16+02	2020-08-23 05:46:25+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-23 05:46:21+02	2020-08-23 05:46:35+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-08-23 05:46:35+02	2020-08-23 05:46:47+02	devel	py-funcy	py37	py37-funcy-1.12
2020-08-23 05:46:26+02	2020-08-23 05:46:51+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-08-23 05:46:51+02	2020-08-23 05:47:03+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-08-23 05:47:03+02	2020-08-23 05:47:15+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-08-23 05:47:15+02	2020-08-23 05:47:34+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-08-23 05:47:34+02	2020-08-23 05:47:46+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-08-23 05:47:46+02	2020-08-23 05:47:58+02	devel	py-rq	py37	py37-rq-1.3.0
2020-08-23 05:46:48+02	2020-08-23 05:48:16+02	devel	py-game	py37	py37-game-1.9.6_1
2020-08-23 05:47:58+02	2020-08-23 05:48:28+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-08-23 05:48:17+02	2020-08-23 05:48:28+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-08-23 05:48:29+02	2020-08-23 05:48:41+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-08-23 05:48:29+02	2020-08-23 05:48:43+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-08-23 05:48:41+02	2020-08-23 05:48:53+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-08-23 05:48:53+02	2020-08-23 05:49:05+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-08-23 05:49:05+02	2020-08-23 05:49:16+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-08-23 05:49:16+02	2020-08-23 05:49:26+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-08-23 05:48:43+02	2020-08-23 05:50:56+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-23 05:50:57+02	2020-08-23 05:51:13+02	devel	scons	py27	scons-py27-3.1.2
2020-08-23 05:51:14+02	2020-08-23 05:51:29+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-08-23 05:51:30+02	2020-08-23 05:51:43+02	dns	py-dnspython	py27	py27-dnspython-1.16.0
2020-08-23 05:51:44+02	2020-08-23 05:54:29+02	dns	py-ldns	py27	py27-ldns-1.7.1_1
2020-08-23 05:49:27+02	2020-08-23 05:55:42+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.4
2020-08-23 05:54:30+02	2020-08-23 06:19:01+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-23 06:19:02+02	2020-08-23 06:24:46+02	editors	pluma	\N	pluma-1.24.0
2020-08-23 06:24:47+02	2020-08-23 06:28:01+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-08-23 06:28:02+02	2020-08-23 06:32:05+02	graphics	atril	\N	atril-1.22.3_8
2020-08-23 06:32:06+02	2020-08-23 06:34:47+02	graphics	eom	\N	eom-1.24.0
2020-08-23 06:34:48+02	2020-08-23 06:36:33+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-23 06:36:34+02	2020-08-23 06:37:06+02	graphics	gle	\N	gle-3.1.0_7
2020-08-23 06:37:06+02	2020-08-23 06:37:30+02	graphics	gltt	\N	gltt-2.5.2_14
2020-08-23 06:37:31+02	2020-08-23 06:39:13+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-23 06:39:14+02	2020-08-23 06:59:54+02	graphics	openimageio	\N	openimageio-2.1.15.0_5
2020-08-23 06:59:57+02	2020-08-23 07:04:28+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-08-23 07:04:29+02	2020-08-23 07:06:33+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-08-23 07:06:34+02	2020-08-23 07:14:31+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-23 07:14:32+02	2020-08-23 07:15:14+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-08-23 07:15:14+02	2020-08-23 07:16:03+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-08-23 07:16:03+02	2020-08-23 07:29:48+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-23 07:29:49+02	2020-08-23 07:39:30+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-23 07:39:31+02	2020-08-23 07:39:42+02	lang	python3	\N	python3-3_3
2020-08-23 07:39:43+02	2020-08-23 07:42:06+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-23 07:42:07+02	2020-08-23 07:43:23+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-23 07:43:24+02	2020-08-23 07:47:13+02	math	stp	\N	stp-2.3.3_6,1
2020-08-23 07:47:14+02	2020-08-23 07:53:26+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-08-23 07:53:26+02	2020-08-23 08:10:54+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-08-23 08:10:55+02	2020-08-23 08:16:38+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-23 08:16:38+02	2020-08-23 08:17:43+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-08-23 08:17:43+02	2020-08-23 08:21:08+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-23 08:21:08+02	2020-08-23 08:56:18+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-23 08:56:21+02	2020-08-23 09:00:11+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-23 09:00:12+02	2020-08-23 09:00:55+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-08-23 09:00:59+02	2020-08-23 09:02:32+02	net	py-pyzmq	py37	py37-pyzmq-19.0.2
2020-08-23 09:02:36+02	2020-08-23 09:05:40+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-23 09:05:43+02	2020-08-23 09:09:11+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-23 09:09:13+02	2020-08-23 09:12:11+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-23 09:12:13+02	2020-08-23 09:15:33+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-23 09:15:36+02	2020-08-23 09:18:22+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-23 09:18:24+02	2020-08-23 09:21:14+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-23 09:21:17+02	2020-08-23 09:24:30+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-23 09:24:31+02	2020-08-23 09:28:41+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-23 09:28:43+02	2020-08-23 09:33:05+02	net	vde2	\N	vde2-2.3.2_4
2020-08-23 09:33:06+02	2020-08-23 09:35:44+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-23 09:35:46+02	2020-08-23 09:43:37+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-08-23 09:43:40+02	2020-08-23 09:44:10+02	net-mgmt	py-ipcalc	py27	py27-ipcalc-1.99.0
2020-08-23 09:44:15+02	2020-08-23 09:44:56+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-08-23 09:45:00+02	2020-08-23 09:47:35+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-23 09:47:36+02	2020-08-23 10:07:38+02	print	lilypond	\N	lilypond-2.20.0
2020-08-23 10:07:40+02	2020-08-23 10:17:30+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-23 10:17:32+02	2020-08-23 10:25:03+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-23 10:25:04+02	2020-08-23 10:27:46+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-23 10:27:48+02	2020-08-23 10:40:32+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-08-23 10:40:33+02	2020-08-23 11:06:12+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-23 11:06:14+02	2020-08-23 11:07:10+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-23 11:07:10+02	2020-08-23 11:07:30+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-08-23 11:07:31+02	2020-08-23 11:07:44+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-08-23 11:07:44+02	2020-08-23 11:08:03+02	security	py-stem	py37	py37-stem-1.8.0
2020-08-23 11:08:03+02	2020-08-23 11:09:20+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-23 11:09:20+02	2020-08-23 11:11:52+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-23 11:11:53+02	2020-08-23 11:15:15+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-23 11:15:16+02	2020-08-23 11:16:37+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-23 05:55:42+02	2020-08-23 11:26:37+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-08-23 11:26:38+02	2020-08-23 11:27:38+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-23 11:27:39+02	2020-08-23 11:28:24+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-08-23 11:28:25+02	2020-08-23 11:28:38+02	textproc	py-scour	py37	py37-scour-0.38
2020-08-23 11:28:38+02	2020-08-23 11:28:53+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-08-23 11:28:53+02	2020-08-23 11:29:55+02	www	mattermost-webapp	\N	mattermost-webapp-5.25.3
2020-08-23 11:29:56+02	2020-08-23 11:30:23+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-08-23 11:30:24+02	2020-08-23 11:30:50+02	www	p5-libwww	\N	p5-libwww-6.47
2020-08-23 11:30:50+02	2020-08-23 11:31:19+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-08-23 11:31:20+02	2020-08-23 11:31:33+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-08-23 11:31:33+02	2020-08-23 11:31:50+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-08-23 11:31:51+02	2020-08-23 11:32:08+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-08-23 11:32:08+02	2020-08-23 11:32:21+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-08-23 11:32:21+02	2020-08-23 11:32:37+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-08-23 11:32:37+02	2020-08-23 11:32:52+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-08-23 11:32:53+02	2020-08-23 11:34:49+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-23 11:34:50+02	2020-08-23 11:36:34+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-23 11:36:35+02	2020-08-23 11:42:49+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-23 11:42:50+02	2020-08-23 11:44:07+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-23 11:44:08+02	2020-08-23 11:45:32+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-23 11:45:33+02	2020-08-23 11:47:35+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-23 11:47:35+02	2020-08-23 11:50:24+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-23 11:50:25+02	2020-08-23 11:52:01+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-08-23 11:52:01+02	2020-08-23 11:53:12+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-23 11:53:13+02	2020-08-23 11:53:53+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-23 11:53:53+02	2020-08-23 12:01:11+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-23 12:01:12+02	2020-08-23 12:30:16+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-23 11:16:37+02	2020-08-23 12:46:27+02	textproc	hs-pandoc	\N	hs-pandoc-2.10
2020-08-23 12:30:17+02	2020-08-23 13:01:38+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-23 13:01:40+02	2020-08-23 13:08:27+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-23 13:08:28+02	2020-08-23 13:11:43+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-23 13:11:44+02	2020-08-23 15:01:32+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-23 15:01:39+02	2020-08-23 15:04:15+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-23 15:04:16+02	2020-08-23 15:06:43+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-23 15:06:45+02	2020-08-23 15:41:25+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-23 15:41:26+02	2020-08-23 15:42:02+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-08-23 15:42:02+02	2020-08-23 15:48:27+02	deskutils	calibre	\N	calibre-4.23.0
2020-08-23 15:48:28+02	2020-08-23 15:49:36+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-23 12:46:33+02	2020-08-23 15:52:47+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-23 15:52:49+02	2020-08-23 15:55:54+02	devel	bzr	py27	bzr-2.7.0.6622
2020-08-23 15:55:54+02	2020-08-23 15:58:04+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-23 15:49:37+02	2020-08-23 17:42:12+02	devel	RStudio	\N	RStudio-1.3.1073
2020-08-23 15:58:05+02	2020-08-24 03:45:49+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-23 17:42:14+02	2020-08-24 04:27:58+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200811
2020-08-24 04:28:01+02	2020-08-24 04:30:11+02	devel	mercurial	py27	mercurial-5.1.2
2020-08-24 04:30:12+02	2020-08-24 04:30:29+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-08-24 04:30:31+02	2020-08-24 04:37:55+02	devel	subversion	\N	subversion-1.14.0
2020-08-24 04:37:56+02	2020-08-24 04:38:37+02	devel	tig	\N	tig-2.5.1
2020-08-24 04:38:38+02	2020-08-24 04:43:54+02	dns	bind916	\N	bind916-9.16.6
2020-08-24 04:44:09+02	2020-08-24 06:08:45+02	editors	codelite	\N	codelite-12.0_4
2020-08-24 06:08:47+02	2020-08-24 06:11:17+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-08-24 06:11:18+02	2020-08-24 06:16:01+02	editors	vim	\N	vim-8.2.1382
2020-08-24 06:16:02+02	2020-08-24 07:20:17+02	emulators	qemu	\N	qemu-4.2.1
2020-08-24 07:20:25+02	2020-08-24 08:28:52+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-08-24 08:28:55+02	2020-08-24 09:17:43+02	emulators	wine	\N	wine-5.0.2,1
2020-08-24 09:17:45+02	2020-08-24 09:33:51+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-24 09:33:53+02	2020-08-24 09:34:21+02	games	KnightCap	\N	knightcap-3.6_8
2020-08-24 09:34:22+02	2020-08-24 09:34:46+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-08-24 09:34:46+02	2020-08-24 09:37:37+02	games	alienarena	\N	alienarena-2013.766_9
2020-08-24 09:37:38+02	2020-08-24 09:39:35+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-08-24 09:39:36+02	2020-08-24 09:47:35+02	games	battletanks	\N	btanks-0.9.8083_9
2020-08-24 09:47:36+02	2020-08-24 09:48:04+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-08-24 09:48:05+02	2020-08-24 09:48:52+02	games	monsterz	\N	monsterz-0.7.1_15
2020-08-24 09:48:53+02	2020-08-24 09:49:35+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-24 09:49:36+02	2020-08-24 09:50:41+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-24 09:50:41+02	2020-08-24 09:50:58+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-08-24 09:51:00+02	2020-08-24 09:52:23+02	games	xblast	\N	xblast-2.10.4_17
2020-08-24 09:52:24+02	2020-08-24 09:53:57+02	games	xboard	\N	xboard-4.9.1_1
2020-08-24 09:53:57+02	2020-08-24 09:55:00+02	games	xcowsay	\N	xcowsay-1.5
2020-08-24 03:45:51+02	2020-08-24 10:18:00+02	devel	llvm10	\N	llvm10-10.0.1_1
2020-08-24 10:18:12+02	2020-08-24 10:24:15+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-24 10:24:16+02	2020-08-24 10:25:46+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-24 09:55:02+02	2020-08-24 11:12:14+02	graphics	blender	\N	blender-2.83.2_1
2020-08-24 10:25:47+02	2020-08-24 11:35:42+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-24 11:35:52+02	2020-08-24 11:36:09+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-08-24 11:36:10+02	2020-08-24 11:41:06+02	graphics	wings	\N	wings-2.2.5
2020-08-24 11:41:08+02	2020-08-24 11:43:59+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-24 11:44:00+02	2020-08-24 11:46:31+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-08-24 11:46:33+02	2020-08-24 11:47:57+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-24 11:47:58+02	2020-08-24 12:37:52+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.9
2020-08-24 11:12:29+02	2020-08-24 12:59:17+02	graphics	krita	\N	krita-4.3.0_4
2020-08-24 12:37:53+02	2020-08-24 13:25:48+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-24 13:25:49+02	2020-08-24 13:26:06+02	lang	python	\N	python-3.7_3,2
2020-08-24 13:26:08+02	2020-08-24 13:32:36+02	math	maxima	\N	maxima-5.44.0_3
2020-08-24 13:32:38+02	2020-08-24 13:35:30+02	misc	mc	\N	mc-nox11-4.8.24
2020-08-24 13:35:32+02	2020-08-24 13:37:09+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-24 13:37:10+02	2020-08-24 13:37:47+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-08-24 13:37:49+02	2020-08-24 13:39:09+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-08-24 12:59:19+02	2020-08-24 13:56:39+02	lang	mono	\N	mono-5.10.1.57_3
2020-08-24 13:56:41+02	2020-08-24 13:57:22+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-08-24 13:57:23+02	2020-08-24 13:57:42+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-08-23 02:05:22+02	2020-08-24 14:01:52+02	www	chromium	\N	chromium-84.0.4147.135
2020-08-24 14:02:04+02	2020-08-24 14:03:31+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-24 13:57:43+02	2020-08-24 14:04:23+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-24 14:03:32+02	2020-08-24 14:04:41+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-24 13:39:10+02	2020-08-24 14:04:44+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p5
2020-08-24 14:04:24+02	2020-08-24 14:06:32+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-24 14:06:33+02	2020-08-24 14:08:14+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-24 14:08:15+02	2020-08-24 14:08:37+02	net	vnstat	\N	vnstat-2.6_1
2020-08-24 14:08:38+02	2020-08-24 14:09:17+02	net	wireguard	\N	wireguard-1.0.20200820
2020-08-24 14:04:45+02	2020-08-24 14:09:24+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-24 14:09:26+02	2020-08-24 14:10:29+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-24 14:04:41+02	2020-08-24 14:13:49+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-24 14:13:50+02	2020-08-24 14:14:49+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-24 14:14:49+02	2020-08-24 14:15:49+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-24 14:15:49+02	2020-08-24 14:16:46+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-24 14:16:47+02	2020-08-24 14:17:38+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-24 14:17:39+02	2020-08-24 14:18:28+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-24 14:18:29+02	2020-08-24 14:19:22+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-24 14:19:23+02	2020-08-24 14:20:28+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-24 14:20:29+02	2020-08-24 14:21:50+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-24 14:21:52+02	2020-08-24 14:22:45+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-24 14:22:46+02	2020-08-24 14:23:43+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-24 14:23:46+02	2020-08-24 14:26:30+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-24 14:26:31+02	2020-08-24 14:27:40+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-24 14:09:18+02	2020-08-24 14:29:42+02	net	wireshark	\N	wireshark-3.2.6
2020-08-24 14:10:29+02	2020-08-24 14:29:44+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-08-24 14:27:41+02	2020-08-24 14:30:04+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-24 14:29:42+02	2020-08-24 14:30:12+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-24 14:29:45+02	2020-08-24 14:30:19+02	print	gv	\N	gv-3.7.4_5
2020-08-24 14:30:20+02	2020-08-24 14:35:25+02	print	texlive-full	\N	texlive-full-20150521
2020-08-24 14:35:26+02	2020-08-24 14:40:56+02	security	klee	\N	klee-2.1_1
2020-08-24 14:40:57+02	2020-08-24 14:41:15+02	security	nyx	\N	nyx-2.1.0_1
2020-08-24 14:41:15+02	2020-08-24 14:42:47+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-24 14:42:47+02	2020-08-24 14:44:20+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-08-24 14:44:21+02	2020-08-24 14:44:40+02	security	py-xkcdpass	py37	py37-xkcdpass-1.17.4
2020-08-24 14:44:41+02	2020-08-24 14:47:47+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-08-24 14:47:48+02	2020-08-24 14:48:04+02	sysutils	bashtop	\N	bashtop-0.9.25_1
2020-08-24 14:48:04+02	2020-08-24 14:48:15+02	sysutils	bpytop	\N	bpytop-1.0.16
2020-08-24 14:48:15+02	2020-08-24 14:49:24+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-24 14:49:25+02	2020-08-24 14:49:50+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-08-24 14:30:05+02	2020-08-24 14:50:31+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-24 14:49:51+02	2020-08-24 14:50:46+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-24 14:50:32+02	2020-08-24 14:51:42+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-24 14:51:42+02	2020-08-24 14:52:02+02	sysutils	panicmail	\N	panicmail-1.1
2020-08-24 14:52:03+02	2020-08-24 14:52:28+02	sysutils	py-glances	py37	py37-glances-3.1.4
2020-08-24 14:52:29+02	2020-08-24 14:52:40+02	sysutils	py-py-cpuinfo	py37	py37-py-cpuinfo-7.0.0
2020-08-24 14:50:46+02	2020-08-24 14:53:45+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-24 14:53:45+02	2020-08-24 14:54:11+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-08-24 14:54:12+02	2020-08-24 14:54:42+02	www	dokuwiki	\N	dokuwiki-20200729
2020-08-24 14:52:41+02	2020-08-24 14:56:11+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-08-24 14:56:11+02	2020-08-24 14:58:04+02	www	mattermost-server	\N	mattermost-server-5.25.3
2020-08-24 14:58:04+02	2020-08-24 15:02:20+02	www	mod_php73	\N	mod_php73-7.3.21
2020-08-24 15:02:21+02	2020-08-24 15:02:32+02	www	py-bottle	py37	py37-bottle-0.12.15_1
2020-08-24 15:02:33+02	2020-08-24 15:02:48+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-08-24 15:02:49+02	2020-08-24 15:03:02+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-08-24 15:03:02+02	2020-08-24 15:03:17+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-08-24 15:03:18+02	2020-08-24 15:03:31+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-08-24 15:03:31+02	2020-08-24 15:03:45+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-08-24 15:03:46+02	2020-08-24 15:04:00+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-08-24 15:04:01+02	2020-08-24 15:04:15+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-08-24 15:04:16+02	2020-08-24 15:04:32+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-08-24 15:04:33+02	2020-08-24 15:04:46+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-08-24 15:04:46+02	2020-08-24 15:04:59+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-08-24 15:05:00+02	2020-08-24 15:05:20+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-08-24 15:05:21+02	2020-08-24 15:05:35+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-08-24 15:05:36+02	2020-08-24 15:08:08+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-24 15:08:08+02	2020-08-24 15:10:32+02	x11	mate	\N	mate-1.22.2
2020-08-24 15:10:33+02	2020-08-24 15:12:28+02	x11	xlockmore	\N	xlockmore-5.65
2020-08-24 14:30:13+02	2020-08-24 15:19:21+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-08-24 15:12:29+02	2020-08-24 15:19:47+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-24 15:19:48+02	2020-08-24 15:21:39+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-24 15:21:39+02	2020-08-24 15:21:51+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-08-24 15:21:52+02	2020-08-24 15:22:41+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-08-24 15:22:41+02	2020-08-24 15:23:04+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-08-24 15:19:21+02	2020-08-24 15:25:16+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-24 15:25:17+02	2020-08-24 15:26:26+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-24 15:26:26+02	2020-08-24 15:27:04+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-24 15:23:04+02	2020-08-24 15:50:53+02	editors	vscode	\N	vscode-1.46.1
2020-08-24 14:54:43+02	2020-08-24 16:49:06+02	www	firefox	\N	firefox-80.0_2,1
2020-08-24 16:49:08+02	2020-08-24 17:05:25+02	local	E590T-localbase	\N	E590T-localbase-2020081500
2020-08-24 17:05:39+02	2020-08-24 17:06:41+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-24 17:05:39+02	2020-08-24 17:10:25+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-24 17:10:25+02	2020-08-24 17:10:32+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-26 07:48:46+02	2020-08-26 07:49:17+02	security	ca_root_nss	\N	ca_root_nss-3.56
2020-08-26 07:48:46+02	2020-08-26 07:49:41+02	security	libgpg-error	\N	libgpg-error-1.39
2020-08-26 07:48:46+02	2020-08-26 07:51:28+02	shells	bash	\N	bash-5.0.18_3
2020-08-26 07:51:30+02	2020-08-26 07:52:20+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-08-26 07:52:21+02	2020-08-26 07:52:44+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-08-26 07:49:42+02	2020-08-26 07:52:45+02	security	libgcrypt	\N	libgcrypt-1.8.5
2020-08-26 07:52:44+02	2020-08-26 07:53:34+02	shells	bash-completion	\N	bash-completion-2.11,2
2020-08-26 07:52:46+02	2020-08-26 07:53:46+02	textproc	libxslt	\N	libxslt-1.1.34_1
2020-08-26 07:49:18+02	2020-08-26 07:54:10+02	ftp	curl	\N	curl-7.72.0
2020-08-26 07:53:35+02	2020-08-26 07:54:32+02	devel	nspr	\N	nspr-4.28
2020-08-26 07:53:47+02	2020-08-26 07:54:50+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-08-26 07:54:33+02	2020-08-26 07:54:50+02	textproc	minixmlto	\N	minixmlto-0.0.2_1
2020-08-26 07:54:52+02	2020-08-26 07:55:46+02	textproc	xmlto	\N	xmlto-0.0.28
2020-08-26 07:54:52+02	2020-08-26 07:57:13+02	security	p11-kit	\N	p11-kit-0.23.21
2020-08-26 07:55:47+02	2020-08-26 07:57:53+02	misc	shared-mime-info	\N	shared-mime-info-2.0
2020-08-26 07:57:55+02	2020-08-26 07:59:55+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-08-26 07:59:57+02	2020-08-26 08:01:02+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-08-26 07:57:15+02	2020-08-26 08:12:23+02	security	gnutls	\N	gnutls-3.6.14
2020-08-26 08:12:27+02	2020-08-26 08:15:27+02	print	cups	\N	cups-2.3.3_1
2020-08-26 08:15:27+02	2020-08-26 08:20:21+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_8
2020-08-26 08:20:21+02	2020-08-26 08:21:46+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-08-26 07:54:14+02	2020-08-26 08:22:33+02	devel	cmake	\N	cmake-3.18.2
2020-08-26 08:21:47+02	2020-08-26 08:22:41+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-26 08:01:04+02	2020-08-26 08:23:18+02	lang	spidermonkey60	\N	spidermonkey60-60.9.0_4
2020-08-26 08:22:34+02	2020-08-26 08:23:44+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-26 08:23:19+02	2020-08-26 08:24:12+02	sysutils	polkit	\N	polkit-0.116
2020-08-26 08:23:45+02	2020-08-26 08:26:30+02	security	nss	\N	nss-3.56
2020-08-26 08:24:12+02	2020-08-26 08:26:47+02	graphics	colord	\N	colord-1.3.5
2020-08-26 08:22:42+02	2020-08-26 08:26:48+02	net	qt5-network	\N	qt5-network-5.15.0
2020-08-26 08:26:31+02	2020-08-26 08:34:47+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-26 08:34:48+02	2020-08-26 08:37:37+02	graphics	poppler	\N	poppler-20.08.0_1
2020-08-26 08:37:37+02	2020-08-26 08:38:06+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-08-26 08:26:47+02	2020-08-26 08:38:31+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-26 08:38:07+02	2020-08-26 08:39:24+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-26 08:39:25+02	2020-08-26 08:39:47+02	devel	libnotify	\N	libnotify-0.7.8
2020-08-26 08:39:48+02	2020-08-26 08:43:23+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-08-26 08:43:23+02	2020-08-26 08:44:20+02	security	libsecret	\N	libsecret-0.18.6_1
2020-08-26 08:26:48+02	2020-08-26 08:44:22+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-26 08:44:20+02	2020-08-26 08:44:34+02	audio	lv2	\N	lv2-1.18.0
2020-08-26 08:44:35+02	2020-08-26 08:44:51+02	multimedia	libaacs	\N	libaacs-0.9.0
2020-08-26 08:38:32+02	2020-08-26 08:45:05+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-26 08:44:51+02	2020-08-26 08:45:19+02	multimedia	libbdplus	\N	libbdplus-0.1.2
2020-08-26 08:45:20+02	2020-08-26 08:45:43+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-08-26 08:44:23+02	2020-08-26 08:52:37+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-26 08:45:43+02	2020-08-26 09:02:29+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-26 09:02:29+02	2020-08-26 09:04:18+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-26 08:45:05+02	2020-08-26 09:04:54+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-26 09:04:18+02	2020-08-26 09:05:37+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-26 09:04:55+02	2020-08-26 09:07:39+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-26 09:07:40+02	2020-08-26 09:09:36+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-26 09:09:36+02	2020-08-26 09:10:43+02	net	geoclue	\N	geoclue-2.5.5
2020-08-26 09:05:38+02	2020-08-26 09:11:31+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-26 09:11:31+02	2020-08-26 09:12:15+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-26 09:12:15+02	2020-08-26 09:12:53+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-26 08:52:38+02	2020-08-26 09:17:37+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-26 09:17:38+02	2020-08-26 09:18:17+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-08-26 09:18:18+02	2020-08-26 09:18:49+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-08-26 09:18:50+02	2020-08-26 09:20:41+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-26 09:12:53+02	2020-08-26 09:27:35+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-26 09:20:42+02	2020-08-26 09:27:47+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-26 09:27:48+02	2020-08-26 09:28:01+02	security	libassuan	\N	libassuan-2.5.3
2020-08-26 09:27:36+02	2020-08-26 09:28:53+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-26 09:28:02+02	2020-08-26 09:29:29+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.73.0
2020-08-26 09:28:54+02	2020-08-26 09:29:41+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-26 09:29:42+02	2020-08-26 09:30:07+02	ftp	php73-curl	\N	php73-curl-7.3.21
2020-08-26 09:29:30+02	2020-08-26 09:30:39+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-26 09:30:08+02	2020-08-26 09:31:05+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-26 09:31:06+02	2020-08-26 09:31:57+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-26 09:30:40+02	2020-08-26 09:32:03+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-26 09:32:03+02	2020-08-26 09:33:14+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-08-26 09:31:58+02	2020-08-26 09:33:37+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-08-26 09:33:14+02	2020-08-26 09:33:46+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-08-26 09:33:37+02	2020-08-26 09:35:06+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-08-26 09:35:07+02	2020-08-26 09:36:47+02	sysutils	hal	\N	hal-0.5.14_34
2020-08-26 09:36:48+02	2020-08-26 09:38:17+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-26 09:33:47+02	2020-08-26 09:38:26+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-26 09:38:27+02	2020-08-26 09:39:48+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-08-26 09:38:18+02	2020-08-26 09:40:00+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-26 09:39:50+02	2020-08-26 09:47:44+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-26 09:47:45+02	2020-08-26 09:49:35+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-08-26 09:49:36+02	2020-08-26 09:50:42+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-26 09:50:43+02	2020-08-26 09:53:40+02	graphics	poppler-glib	\N	poppler-glib-20.08.0_1
2020-08-26 09:40:03+02	2020-08-26 09:53:59+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-26 09:53:40+02	2020-08-26 09:56:52+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-08-26 09:53:59+02	2020-08-26 09:57:02+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-08-26 09:56:53+02	2020-08-26 09:57:44+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-08-26 09:57:03+02	2020-08-26 09:58:04+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-08-26 09:57:44+02	2020-08-26 09:58:48+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-26 09:58:04+02	2020-08-26 09:59:10+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-26 09:58:48+02	2020-08-26 09:59:53+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-26 09:59:10+02	2020-08-26 10:02:23+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-26 09:59:54+02	2020-08-26 10:04:12+02	databases	mysql57-client	\N	mysql57-client-5.7.31
2020-08-26 10:04:12+02	2020-08-26 10:05:47+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-08-26 10:02:23+02	2020-08-26 10:06:47+02	devel	git	\N	git-2.28.0
2020-08-26 10:05:48+02	2020-08-26 10:07:57+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-08-26 10:06:47+02	2020-08-26 10:09:11+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-26 10:09:14+02	2020-08-26 10:10:39+02	net	remmina	\N	remmina-1.3.9
2020-08-26 10:10:40+02	2020-08-26 10:13:48+02	security	gcr	\N	gcr-3.28.0
2020-08-26 10:13:49+02	2020-08-26 10:14:03+02	security	pinentry-tty	\N	pinentry-tty-1.1.0
2020-08-26 10:14:04+02	2020-08-26 10:15:22+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-08-26 10:15:23+02	2020-08-26 10:29:13+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-26 10:29:14+02	2020-08-26 10:30:10+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-26 10:30:11+02	2020-08-26 10:39:15+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-26 10:39:16+02	2020-08-26 10:40:00+02	security	libksba	\N	libksba-1.4.0
2020-08-26 10:40:02+02	2020-08-26 10:40:10+02	security	pinentry	\N	pinentry-1.1.0_6
2020-08-26 10:40:10+02	2020-08-26 10:41:57+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-08-26 10:41:58+02	2020-08-26 10:43:20+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-08-26 10:43:21+02	2020-08-26 10:45:02+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-26 10:45:03+02	2020-08-26 11:10:40+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-26 09:10:44+02	2020-08-26 11:17:51+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-26 11:17:52+02	2020-08-26 11:54:54+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-26 11:54:55+02	2020-08-26 11:57:52+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-26 11:57:53+02	2020-08-26 11:58:33+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-26 11:58:34+02	2020-08-26 12:01:22+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-26 12:01:23+02	2020-08-26 12:04:15+02	security	gnupg	\N	gnupg-2.2.21
2020-08-26 12:04:16+02	2020-08-26 12:04:43+02	textproc	yelp-xsl	\N	yelp-xsl-3.28.0
2020-08-26 12:04:44+02	2020-08-26 12:05:36+02	x11	xterm	\N	xterm-359
2020-08-26 12:05:36+02	2020-08-26 12:07:09+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-08-26 12:07:10+02	2020-08-26 12:09:40+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-26 12:09:40+02	2020-08-26 12:14:47+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-08-26 12:14:48+02	2020-08-26 12:19:20+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-26 12:19:21+02	2020-08-26 12:20:04+02	security	gpgme	\N	gpgme-1.14.0
2020-08-26 12:20:05+02	2020-08-26 12:20:49+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-08-26 12:20:50+02	2020-08-26 12:22:50+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-08-26 12:22:50+02	2020-08-26 12:34:09+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-26 11:10:41+02	2020-08-26 12:42:34+02	math	vtk8	\N	vtk8-8.2.0
2020-08-26 12:42:36+02	2020-08-26 15:00:19+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-26 15:00:21+02	2020-08-26 15:01:52+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-26 15:01:53+02	2020-08-26 15:03:16+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-08-26 15:03:16+02	2020-08-26 15:12:56+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-26 15:12:57+02	2020-08-26 15:13:33+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-08-26 15:13:34+02	2020-08-26 15:14:53+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-26 15:14:54+02	2020-08-26 15:15:49+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-26 15:15:50+02	2020-08-26 15:16:59+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-26 15:16:59+02	2020-08-26 15:19:41+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-26 15:19:42+02	2020-08-26 15:20:47+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-08-26 15:20:48+02	2020-08-26 15:21:49+02	sysutils	upower	\N	upower-0.99.11
2020-08-26 15:21:50+02	2020-08-26 15:22:56+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-08-26 12:34:10+02	2020-08-26 16:07:33+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-26 16:07:34+02	2020-08-26 16:09:28+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-08-26 16:09:28+02	2020-08-26 16:14:27+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-26 16:14:28+02	2020-08-26 16:16:16+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-26 16:16:16+02	2020-08-26 16:18:45+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-08-26 16:18:46+02	2020-08-26 16:20:11+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-08-26 10:07:58+02	2020-08-26 16:40:28+02	lang	rust	\N	rust-1.45.2
2020-08-26 16:40:38+02	2020-08-26 16:46:10+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-08-26 16:20:12+02	2020-08-26 16:48:48+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-26 16:46:12+02	2020-08-26 16:49:08+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-26 16:49:09+02	2020-08-26 16:49:47+02	devel	librest	\N	rest-0.7.93
2020-08-26 16:49:47+02	2020-08-26 16:53:01+02	graphics	poppler-utils	\N	poppler-utils-20.08.0_1
2020-08-26 16:53:02+02	2020-08-26 17:25:49+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-26 17:25:50+02	2020-08-26 17:27:55+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-08-26 17:27:55+02	2020-08-26 17:28:50+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-26 17:28:51+02	2020-08-26 17:29:48+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-26 17:29:49+02	2020-08-26 17:31:02+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-08-26 17:31:02+02	2020-08-26 17:33:46+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-08-26 16:48:49+02	2020-08-26 18:34:11+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-26 18:34:12+02	2020-08-26 18:35:43+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-26 18:35:43+02	2020-08-26 18:36:14+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-08-26 18:36:14+02	2020-08-26 18:37:33+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-26 18:37:34+02	2020-08-26 18:41:48+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-26 18:41:49+02	2020-08-26 18:54:20+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-26 18:54:20+02	2020-08-26 18:57:11+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-26 18:57:12+02	2020-08-26 18:58:18+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-26 18:58:19+02	2020-08-26 18:59:48+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-26 18:59:48+02	2020-08-26 19:01:29+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-26 19:01:29+02	2020-08-26 19:03:17+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-26 19:03:17+02	2020-08-26 19:06:25+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-08-26 19:06:25+02	2020-08-26 19:10:44+02	graphics	gegl	\N	gegl-0.4.24_4
2020-08-26 19:10:44+02	2020-08-26 19:11:29+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-08-26 19:11:30+02	2020-08-26 19:15:13+02	mail	mailutils	\N	mailutils-3.10
2020-08-26 17:33:47+02	2020-08-26 19:15:49+02	www	node	\N	node-14.8.0
2020-08-26 19:15:50+02	2020-08-26 19:16:05+02	net	liboauth	\N	liboauth-1.0.3_4
2020-08-26 19:15:14+02	2020-08-26 19:17:35+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-26 19:17:36+02	2020-08-26 19:18:38+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-26 19:18:38+02	2020-08-26 19:19:48+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-26 19:19:48+02	2020-08-26 19:23:06+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-26 19:23:07+02	2020-08-26 19:24:05+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-08-26 19:24:05+02	2020-08-26 19:26:37+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-26 19:26:38+02	2020-08-26 19:27:56+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-26 19:27:56+02	2020-08-26 19:28:24+02	textproc	yelp-tools	\N	yelp-tools-3.28.0
2020-08-26 19:28:24+02	2020-08-26 19:29:23+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-08-26 19:29:24+02	2020-08-26 19:30:39+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-08-26 19:30:39+02	2020-08-26 19:34:00+02	x11	xorg	\N	xorg-7.7_3
2020-08-26 19:34:02+02	2020-08-26 19:35:30+02	x11	zenity	\N	zenity-3.32.0
2020-08-26 19:35:31+02	2020-08-26 19:36:36+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-08-26 19:16:05+02	2020-08-26 19:40:01+02	net	samba410	\N	samba410-4.10.17
2020-08-26 19:40:02+02	2020-08-26 19:44:10+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-26 19:44:11+02	2020-08-26 19:45:42+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-26 19:45:43+02	2020-08-26 19:48:28+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-26 19:36:37+02	2020-08-26 19:49:06+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-26 19:48:28+02	2020-08-26 19:51:18+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-26 19:49:06+02	2020-08-26 19:51:34+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-26 19:51:35+02	2020-08-26 19:58:44+02	editors	emacs	full	emacs-27.1,3
2020-08-26 19:58:45+02	2020-08-26 19:59:02+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-08-26 19:51:18+02	2020-08-26 19:59:03+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-26 19:59:03+02	2020-08-26 20:20:47+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-08-26 20:20:48+02	2020-08-26 20:23:05+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-26 20:23:06+02	2020-08-26 20:23:36+02	net	gssdp	\N	gssdp-1.2.3
2020-08-26 20:23:37+02	2020-08-26 20:24:25+02	net	libvncserver	\N	libvncserver-0.9.12_2
2020-08-26 20:24:26+02	2020-08-26 20:25:28+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-08-26 20:25:28+02	2020-08-26 20:27:25+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-08-26 19:59:03+02	2020-08-26 20:36:59+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-26 20:27:26+02	2020-08-26 20:38:23+02	print	fontforge	\N	fontforge-20190801_2
2020-08-26 20:38:24+02	2020-08-26 20:39:23+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-26 20:39:23+02	2020-08-26 20:41:19+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-08-26 20:41:20+02	2020-08-26 20:42:14+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-08-26 20:37:02+02	2020-08-26 20:55:50+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-08-26 20:55:51+02	2020-08-26 22:44:32+02	www	node12	\N	node12-12.18.3
2020-08-26 22:44:33+02	2020-08-26 22:46:27+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-26 22:46:27+02	2020-08-26 22:48:43+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-26 22:48:44+02	2020-08-26 22:50:20+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-26 22:50:21+02	2020-08-26 22:51:21+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-26 22:51:22+02	2020-08-26 22:53:47+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-26 22:53:48+02	2020-08-26 23:01:24+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-26 23:01:25+02	2020-08-26 23:08:42+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-26 23:08:43+02	2020-08-26 23:10:08+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-26 23:10:12+02	2020-08-26 23:15:28+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-26 23:15:29+02	2020-08-26 23:28:58+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-26 23:28:59+02	2020-08-26 23:43:43+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-26 23:43:43+02	2020-08-26 23:45:09+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-26 23:45:11+02	2020-08-26 23:48:28+02	x11-wm	marco	\N	marco-1.22.4
2020-08-26 23:48:28+02	2020-08-26 23:49:37+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-26 23:49:39+02	2020-08-26 23:52:21+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-26 23:52:22+02	2020-08-26 23:53:45+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-08-26 23:53:45+02	2020-08-26 23:59:01+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-26 23:59:02+02	2020-08-27 00:02:58+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-27 00:02:59+02	2020-08-27 00:11:35+02	devel	ncurses	\N	ncurses-6.2.20200822
2020-08-27 00:11:36+02	2020-08-27 00:21:58+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-27 00:21:59+02	2020-08-27 00:24:12+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-27 00:24:13+02	2020-08-27 00:26:26+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-27 00:26:26+02	2020-08-27 00:28:49+02	graphics	libheif	\N	libheif-1.6.2
2020-08-27 00:28:51+02	2020-08-27 00:35:41+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-08-27 00:35:42+02	2020-08-27 00:36:53+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-27 00:36:53+02	2020-08-27 00:38:03+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-27 00:38:04+02	2020-08-27 00:43:33+02	graphics	sane-backends	\N	sane-backends-1.0.31
2020-08-27 00:43:34+02	2020-08-27 01:07:47+02	math	R	\N	R-4.0.2
2020-08-27 01:07:48+02	2020-08-27 01:17:21+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-27 01:17:22+02	2020-08-27 01:20:20+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-27 01:20:54+02	2020-08-27 01:24:45+02	net	freerdp	\N	freerdp-2.2.0
2020-08-27 01:24:46+02	2020-08-27 01:25:27+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-26 15:22:57+02	2020-08-27 01:34:45+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-10-06 07:45:05+02	2020-10-06 07:47:22+02	x11-fm	caja	\N	caja-1.24.0
2020-08-27 01:25:27+02	2020-08-27 01:39:32+02	net	libcmis	\N	libcmis-0.5.2_1
2020-08-27 01:39:33+02	2020-08-27 01:41:00+02	net	liveMedia	\N	liveMedia-2020.08.19,2
2020-08-27 01:41:01+02	2020-08-27 01:41:59+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-27 01:42:00+02	2020-08-27 01:43:04+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-27 01:43:05+02	2020-08-27 01:48:25+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-27 01:48:26+02	2020-08-27 01:49:05+02	print	libspectre	\N	libspectre-0.2.9
2020-08-27 01:49:06+02	2020-08-27 01:51:43+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-27 01:51:44+02	2020-08-27 02:00:38+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-27 02:00:39+02	2020-08-27 02:07:52+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-27 02:07:53+02	2020-08-27 02:09:18+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-08-27 02:09:18+02	2020-08-27 02:09:40+02	sysutils	hal-info	\N	hal-info-20091130
2020-08-27 02:09:41+02	2020-08-27 02:14:27+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-27 02:14:28+02	2020-08-27 02:15:34+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-27 02:15:35+02	2020-08-27 02:20:42+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-27 02:20:43+02	2020-08-27 02:23:05+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-27 02:23:06+02	2020-08-27 02:24:11+02	textproc	redland	\N	redland-1.0.17_4
2020-08-27 02:24:12+02	2020-08-27 02:25:05+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-27 02:25:06+02	2020-08-27 02:26:52+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-27 02:26:53+02	2020-08-27 02:27:41+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-27 02:27:41+02	2020-08-27 02:29:29+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-27 02:29:30+02	2020-08-27 02:30:48+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-27 02:30:49+02	2020-08-27 02:31:46+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-27 02:31:47+02	2020-08-27 02:34:27+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-27 02:34:28+02	2020-08-27 02:36:59+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-27 02:37:00+02	2020-08-27 02:40:18+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-27 01:34:51+02	2020-08-27 03:05:56+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-27 03:05:57+02	2020-08-27 03:09:40+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-27 03:09:40+02	2020-08-27 03:10:44+02	audio	faudio	\N	FAudio-20.08
2020-08-27 03:10:45+02	2020-08-27 03:12:18+02	audio	mate-media	\N	mate-media-1.24.1
2020-08-27 03:12:18+02	2020-08-27 03:14:22+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-27 02:40:19+02	2020-08-27 03:17:22+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-27 03:17:23+02	2020-08-27 03:18:39+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-27 03:14:22+02	2020-08-27 03:19:25+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-27 03:18:40+02	2020-08-27 03:19:43+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-27 03:19:26+02	2020-08-27 03:20:16+02	devel	adacurses	\N	adacurses-20180127
2020-08-27 03:19:43+02	2020-08-27 03:21:13+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-08-27 03:20:17+02	2020-08-27 03:22:50+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-27 03:21:13+02	2020-08-27 03:29:34+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.4
2020-08-27 03:22:51+02	2020-08-27 03:50:54+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-27 03:50:55+02	2020-08-27 03:57:13+02	editors	pluma	\N	pluma-1.24.0
2020-08-27 03:57:13+02	2020-08-27 04:07:32+02	graphics	alembic	\N	alembic-1.7.14
2020-08-27 04:07:32+02	2020-08-27 04:12:10+02	graphics	atril	\N	atril-1.22.3_8
2020-08-27 04:12:11+02	2020-08-27 04:15:07+02	graphics	eom	\N	eom-1.24.0
2020-08-27 04:15:07+02	2020-08-27 04:17:13+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-27 04:17:14+02	2020-08-27 04:19:10+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-27 04:19:11+02	2020-08-27 04:41:46+02	graphics	openimageio	\N	openimageio-2.1.15.0_5
2020-08-27 04:41:47+02	2020-08-27 04:44:00+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-08-27 04:44:01+02	2020-08-27 04:53:21+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-27 04:53:22+02	2020-08-27 05:09:04+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-27 05:09:05+02	2020-08-27 05:19:28+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-27 05:19:29+02	2020-08-27 07:40:44+02	lang	julia11	\N	julia11-1.1.1_5
2020-08-27 07:40:45+02	2020-08-27 07:43:39+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-27 07:43:40+02	2020-08-27 07:44:47+02	multimedia	gmmlib	\N	gmmlib-20.2.5
2020-08-27 07:44:48+02	2020-08-27 07:45:35+02	multimedia	libmtp	\N	libmtp-1.1.17
2020-08-27 07:45:35+02	2020-08-27 07:52:30+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-27 07:52:31+02	2020-08-27 07:56:47+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-27 07:56:48+02	2020-08-27 08:16:00+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-27 08:16:01+02	2020-08-27 08:18:23+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-27 08:18:23+02	2020-08-27 08:19:29+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-27 08:19:30+02	2020-08-27 08:20:38+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-27 08:20:39+02	2020-08-27 08:21:42+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-27 08:21:43+02	2020-08-27 08:22:52+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-27 08:22:53+02	2020-08-27 08:23:54+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-27 08:23:54+02	2020-08-27 08:24:56+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-27 08:24:56+02	2020-08-27 08:26:19+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-27 08:26:19+02	2020-08-27 08:28:14+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-27 08:28:15+02	2020-08-27 08:29:18+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-27 08:29:19+02	2020-08-27 08:31:24+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_7
2020-08-27 08:31:25+02	2020-08-27 08:33:06+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-27 08:33:07+02	2020-08-27 08:54:09+02	print	lilypond	\N	lilypond-2.20.0
2020-08-27 08:54:11+02	2020-08-27 09:04:19+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-27 09:04:20+02	2020-08-27 09:11:58+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-27 09:11:59+02	2020-08-27 09:14:54+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-27 09:14:58+02	2020-08-27 09:27:45+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-08-27 09:27:46+02	2020-08-27 09:53:26+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-27 09:53:28+02	2020-08-27 09:54:03+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-27 09:54:03+02	2020-08-27 09:54:36+02	security	openconnect	\N	openconnect-8.10
2020-08-27 09:54:37+02	2020-08-27 09:55:34+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-27 09:55:36+02	2020-08-27 09:57:30+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-27 09:57:31+02	2020-08-27 10:00:06+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-27 10:00:07+02	2020-08-27 10:01:15+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-27 10:01:15+02	2020-08-27 10:01:50+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-08-27 03:29:35+02	2020-08-27 10:02:03+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-08-27 10:02:05+02	2020-08-27 10:02:39+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-08-27 10:01:50+02	2020-08-27 10:03:02+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-27 10:02:40+02	2020-08-27 10:03:13+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-08-27 10:03:14+02	2020-08-27 10:05:06+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-27 10:03:02+02	2020-08-27 10:05:09+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-27 10:05:07+02	2020-08-27 10:05:19+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-08-27 10:05:20+02	2020-08-27 10:06:40+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-27 10:06:40+02	2020-08-27 10:08:19+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-27 10:08:19+02	2020-08-27 10:10:45+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-27 10:05:09+02	2020-08-27 10:11:28+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-27 10:11:28+02	2020-08-27 10:12:56+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-27 10:10:46+02	2020-08-27 10:13:22+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-27 10:12:57+02	2020-08-27 10:13:37+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-27 10:13:23+02	2020-08-27 10:23:38+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-27 10:13:38+02	2020-08-27 10:48:27+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-27 10:23:42+02	2020-08-27 10:56:40+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-27 10:56:41+02	2020-08-27 11:03:05+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-27 11:03:09+02	2020-08-27 11:06:08+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-27 11:06:09+02	2020-08-27 12:55:09+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-27 12:55:13+02	2020-08-27 12:57:41+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-27 12:57:42+02	2020-08-27 12:59:51+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-27 12:59:54+02	2020-08-27 13:33:58+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-27 13:33:59+02	2020-08-27 13:40:10+02	deskutils	calibre	\N	calibre-4.23.0
2020-08-27 13:40:11+02	2020-08-27 13:41:38+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-27 10:48:27+02	2020-08-27 13:54:01+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-27 13:54:06+02	2020-08-27 13:57:06+02	devel	bzr	py27	bzr-2.7.0.6622
2020-08-27 13:57:06+02	2020-08-27 15:07:15+02	devel	codeblocks	\N	codeblocks-20.03
2020-08-27 15:07:16+02	2020-08-27 15:08:59+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-27 13:41:39+02	2020-08-27 15:19:10+02	devel	RStudio	\N	RStudio-1.3.1073
2020-08-27 15:19:11+02	2020-08-27 16:20:33+02	devel	ghidra	\N	ghidra-9.1
2020-08-27 16:20:34+02	2020-08-27 16:22:14+02	devel	mercurial	py27	mercurial-5.1.2
2020-08-27 16:22:16+02	2020-08-27 16:22:59+02	devel	tig	\N	tig-2.5.1
2020-08-27 16:23:00+02	2020-08-27 16:56:20+02	editors	codelite	\N	codelite-12.0_4
2020-08-27 16:56:21+02	2020-08-27 16:59:15+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-08-27 16:59:16+02	2020-08-27 17:03:22+02	editors	vim	\N	vim-8.2.1382
2020-08-27 17:03:23+02	2020-08-27 17:43:49+02	emulators	qemu	\N	qemu-4.2.1
2020-08-27 17:43:52+02	2020-08-27 19:02:43+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-08-27 19:02:45+02	2020-08-27 19:32:13+02	emulators	wine	\N	wine-5.0.2,1
2020-08-27 19:32:17+02	2020-08-27 19:50:13+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-27 19:50:13+02	2020-08-27 19:53:03+02	games	alienarena	\N	alienarena-2013.766_9
2020-08-27 19:53:04+02	2020-08-27 19:53:43+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-27 19:53:44+02	2020-08-27 19:54:38+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-27 19:54:39+02	2020-08-27 19:56:15+02	games	xboard	\N	xboard-4.9.1_1
2020-08-27 19:56:17+02	2020-08-27 19:57:16+02	games	xcowsay	\N	xcowsay-1.5
2020-08-27 19:57:17+02	2020-08-27 20:33:24+02	graphics	blender	\N	blender-2.83.2_1
2020-08-27 20:33:25+02	2020-08-27 20:37:52+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-27 20:37:53+02	2020-08-27 20:39:26+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-27 20:39:27+02	2020-08-27 21:05:42+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-27 21:05:43+02	2020-08-27 22:04:47+02	graphics	krita	\N	krita-4.3.0_4
2020-08-27 22:04:57+02	2020-08-27 22:13:36+02	graphics	wings	\N	wings-2.2.5
2020-08-27 22:13:39+02	2020-08-27 22:15:43+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-27 22:15:44+02	2020-08-27 22:17:12+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-27 22:17:13+02	2020-08-27 22:58:05+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.9
2020-08-27 22:58:06+02	2020-08-27 23:44:16+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-27 23:44:17+02	2020-08-27 23:45:09+02	lang	julia	\N	julia-1.0.3_3
2020-08-27 23:45:10+02	2020-08-28 00:41:58+02	lang	mono	\N	mono-5.10.1.57_3
2020-08-28 00:41:59+02	2020-08-28 00:50:00+02	math	maxima	\N	maxima-5.44.0_3
2020-08-28 00:50:02+02	2020-08-28 00:51:21+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-28 00:51:22+02	2020-08-28 01:03:16+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p5
2020-08-28 01:03:17+02	2020-08-28 01:06:45+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-28 01:06:47+02	2020-08-28 01:10:05+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-28 01:10:06+02	2020-08-28 01:14:43+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-28 01:14:45+02	2020-08-28 01:15:23+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-08-28 01:15:26+02	2020-08-28 01:18:33+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-28 01:18:34+02	2020-08-28 01:20:22+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-28 01:20:23+02	2020-08-28 01:20:35+02	net	wireguard	\N	wireguard-1.0.20200820
2020-08-28 01:20:35+02	2020-08-28 01:29:51+02	net	wireshark	\N	wireshark-3.2.6
2020-08-28 01:29:52+02	2020-08-28 01:31:01+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-28 01:31:02+02	2020-08-28 01:32:46+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-08-28 01:32:48+02	2020-08-28 01:33:57+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-28 01:33:58+02	2020-08-28 01:35:07+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-28 01:35:08+02	2020-08-28 01:36:16+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-28 01:36:17+02	2020-08-28 01:37:27+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-28 01:37:28+02	2020-08-28 01:38:31+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-28 01:38:32+02	2020-08-28 01:39:37+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-28 01:39:38+02	2020-08-28 01:40:47+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-28 01:40:48+02	2020-08-28 01:41:44+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-28 01:41:45+02	2020-08-28 01:42:41+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-28 01:42:42+02	2020-08-28 01:43:42+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-28 01:43:43+02	2020-08-28 01:44:06+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-08-28 01:44:08+02	2020-08-28 01:47:07+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-28 01:47:08+02	2020-08-28 01:47:41+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-27 15:09:00+02	2020-08-28 01:47:52+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-28 01:47:41+02	2020-08-28 01:48:22+02	print	gv	\N	gv-3.7.4_5
2020-08-28 01:47:56+02	2020-08-28 01:52:07+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-28 01:48:23+02	2020-08-28 01:57:28+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-08-28 01:52:08+02	2020-08-28 01:57:49+02	print	texlive-full	\N	texlive-full-20150521
2020-08-28 01:57:50+02	2020-08-28 01:59:00+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-28 01:59:01+02	2020-08-28 01:59:18+02	sysutils	bpytop	\N	bpytop-1.0.19
2020-08-28 01:59:19+02	2020-08-28 02:00:31+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-28 01:57:29+02	2020-08-28 02:00:33+02	security	clamav	\N	clamav-0.102.4,1
2020-08-28 02:00:32+02	2020-08-28 02:01:12+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-28 02:00:34+02	2020-08-28 02:01:36+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-28 02:01:38+02	2020-08-28 02:01:46+02	sysutils	neofetch	\N	neofetch-7.1.0
2020-08-28 02:01:46+02	2020-08-28 02:02:13+02	sysutils	parallel	\N	parallel-20200822
2020-08-28 02:01:13+02	2020-08-28 02:02:41+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-28 02:02:15+02	2020-08-28 02:03:19+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-08-28 02:03:20+02	2020-08-28 02:05:32+02	www	mattermost-server	\N	mattermost-server-5.25.3
2020-08-28 02:05:33+02	2020-08-28 02:06:33+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-08-28 02:06:34+02	2020-08-28 02:07:41+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-28 02:09:42+02	2020-08-28 02:12:40+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-28 02:12:41+02	2020-08-28 02:19:29+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-28 02:19:31+02	2020-08-28 02:20:45+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-28 02:20:46+02	2020-08-28 03:00:19+02	editors	vscode	\N	vscode-1.46.1
2020-08-28 03:00:21+02	2020-08-28 03:01:30+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-28 02:02:42+02	2020-08-28 03:05:18+02	www	firefox	\N	firefox-80.0_3,1
2020-08-26 20:42:15+02	2020-08-28 04:13:00+02	www	chromium	\N	chromium-84.0.4147.135
2020-08-28 04:13:02+02	2020-08-28 04:13:50+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-28 04:13:51+02	2020-08-28 04:14:39+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-28 04:14:40+02	2020-08-28 04:15:28+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-28 04:15:29+02	2020-08-28 04:31:47+02	local	E590T-localbase	\N	E590T-localbase-2020082500
2020-08-28 04:32:03+02	2020-08-28 04:33:13+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-28 04:32:03+02	2020-08-28 04:36:49+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-28 04:36:49+02	2020-08-28 04:36:56+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-28 07:55:22+02	2020-08-28 07:55:50+02	net	libmaxminddb	\N	libmaxminddb-1.4.3
2020-08-28 07:55:52+02	2020-08-28 07:56:09+02	textproc	lowdown	\N	lowdown-0.7.3
2020-08-28 07:55:17+02	2020-08-28 07:56:13+02	archivers	brotli	\N	brotli-1.0.9,1
2020-08-28 08:00:37+02	2020-08-28 08:03:02+02	x11	libX11	\N	libX11-1.6.12,1
2020-08-28 08:03:02+02	2020-08-28 08:03:21+02	x11	libXext	\N	libXext-1.3.4,1
2020-08-28 08:03:21+02	2020-08-28 08:03:36+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-08-28 08:03:37+02	2020-08-28 08:03:51+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-08-28 08:03:51+02	2020-08-28 08:04:07+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-08-28 08:04:08+02	2020-08-28 08:04:22+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-08-28 08:04:23+02	2020-08-28 08:04:35+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-08-28 08:04:36+02	2020-08-28 08:05:17+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-08-28 08:05:17+02	2020-08-28 08:05:41+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-08-28 08:00:37+02	2020-08-28 08:05:46+02	lang	python37	\N	python37-3.7.9
2020-08-28 08:05:47+02	2020-08-28 08:06:03+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-08-28 08:05:42+02	2020-08-28 08:06:06+02	x11	libXi	\N	libXi-1.7.10,1
2020-08-28 08:06:04+02	2020-08-28 08:06:17+02	devel	py-six	py37	py37-six-1.15.0
2020-08-28 08:06:06+02	2020-08-28 08:06:21+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-08-28 08:06:18+02	2020-08-28 08:06:30+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-08-28 08:06:21+02	2020-08-28 08:06:33+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-08-28 08:06:34+02	2020-08-28 08:06:46+02	dns	py-idna	py37	py37-idna-2.8
2020-08-28 08:06:30+02	2020-08-28 08:06:48+02	devel	py-cffi	py37	py37-cffi-1.14.2
2020-08-28 08:06:47+02	2020-08-28 08:07:23+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-08-28 08:07:24+02	2020-08-28 08:07:36+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-08-28 08:06:48+02	2020-08-28 08:07:38+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-08-28 08:07:36+02	2020-08-28 08:07:48+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-08-28 08:07:38+02	2020-08-28 08:07:50+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-08-28 08:07:48+02	2020-08-28 08:08:00+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-08-28 08:08:01+02	2020-08-28 08:08:13+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-08-28 08:08:13+02	2020-08-28 08:08:27+02	net	py-urllib3	py37	py37-urllib3-1.25.10,1
2020-08-28 08:08:27+02	2020-08-28 08:08:39+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-08-28 08:07:50+02	2020-08-28 08:09:07+02	devel	ninja	\N	ninja-1.10.0,2
2020-08-28 08:09:08+02	2020-08-28 08:09:26+02	devel	meson	\N	meson-0.54.3
2020-08-28 08:09:27+02	2020-08-28 08:09:41+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-08-28 08:08:40+02	2020-08-28 08:10:36+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-08-28 08:10:37+02	2020-08-28 08:10:51+02	www	py-requests	py37	py37-requests-2.22.0
2020-08-28 08:10:52+02	2020-08-28 08:11:04+02	devel	py-packaging	py37	py37-packaging-20.4
2020-08-28 08:11:05+02	2020-08-28 08:11:25+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-08-28 08:11:25+02	2020-08-28 08:11:37+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-08-28 08:11:37+02	2020-08-28 08:11:50+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-08-28 08:11:50+02	2020-08-28 08:12:02+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-08-28 08:09:41+02	2020-08-28 08:12:06+02	lang	cython	py37	py37-cython-0.29.15
2020-08-28 08:12:03+02	2020-08-28 08:12:14+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-08-28 08:12:15+02	2020-08-28 08:12:26+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-08-28 08:12:07+02	2020-08-28 08:12:30+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-08-28 08:12:27+02	2020-08-28 08:12:38+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-08-28 08:12:30+02	2020-08-28 08:12:43+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-08-28 08:12:38+02	2020-08-28 08:12:50+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-08-28 08:12:43+02	2020-08-28 08:12:54+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-08-28 08:12:50+02	2020-08-28 08:13:03+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-08-28 08:13:04+02	2020-08-28 08:13:19+02	devel	py-future	py37	py37-future-0.18.2
2020-08-28 08:12:55+02	2020-08-28 08:13:20+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-08-28 08:13:20+02	2020-08-28 08:13:31+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-08-28 08:13:21+02	2020-08-28 08:13:32+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-08-28 08:13:32+02	2020-08-28 08:13:45+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-08-28 08:13:31+02	2020-08-28 08:13:45+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-08-28 08:13:46+02	2020-08-28 08:19:09+02	devel	glib20	\N	glib-2.56.3_9,1
2020-08-28 08:19:09+02	2020-08-28 08:19:38+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-08-28 08:19:39+02	2020-08-28 08:19:52+02	textproc	itstool	\N	itstool-2.0.6
2020-08-28 08:19:53+02	2020-08-28 08:20:43+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-08-28 08:20:43+02	2020-08-28 08:22:24+02	devel	dbus	\N	dbus-1.12.20
2020-08-28 08:22:25+02	2020-08-28 08:23:17+02	security	p11-kit	\N	p11-kit-0.23.21
2020-08-28 08:23:18+02	2020-08-28 08:26:04+02	security	gnutls	\N	gnutls-3.6.14
2020-08-28 08:26:05+02	2020-08-28 08:26:41+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-08-28 08:26:41+02	2020-08-28 08:27:12+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-08-28 08:27:12+02	2020-08-28 08:27:32+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-08-28 08:27:33+02	2020-08-28 08:28:03+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-08-28 08:28:03+02	2020-08-28 08:28:24+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-08-28 08:28:25+02	2020-08-28 08:28:57+02	x11	libXpm	\N	libXpm-3.5.13
2020-08-28 08:28:58+02	2020-08-28 08:30:11+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-08-28 08:30:11+02	2020-08-28 08:31:35+02	misc	shared-mime-info	\N	shared-mime-info-2.0
2020-08-28 08:31:37+02	2020-08-28 08:31:54+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-08-28 08:31:54+02	2020-08-28 08:32:13+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-08-28 08:32:14+02	2020-08-28 08:32:31+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-08-28 08:32:32+02	2020-08-28 08:32:48+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-08-28 08:32:49+02	2020-08-28 08:33:38+02	textproc	libcroco	\N	libcroco-0.6.13
2020-08-28 08:33:39+02	2020-08-28 08:33:56+02	x11	xprop	\N	xprop-1.2.4
2020-08-28 08:33:57+02	2020-08-28 08:34:15+02	x11	xset	\N	xset-1.2.4_3
2020-08-28 08:34:16+02	2020-08-28 08:46:55+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-08-28 08:46:56+02	2020-08-28 08:51:38+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-08-28 08:51:38+02	2020-08-28 08:52:02+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-08-28 08:52:02+02	2020-08-28 08:52:17+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-08-28 08:52:18+02	2020-08-28 08:52:33+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-08-28 08:52:34+02	2020-08-28 08:55:25+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-08-28 08:55:26+02	2020-08-28 08:56:40+02	net	qt5-network	\N	qt5-network-5.15.0
2020-08-28 08:56:41+02	2020-08-28 08:58:11+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-08-28 08:58:12+02	2020-08-28 08:58:31+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-08-28 08:58:32+02	2020-08-28 09:00:14+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-08-28 09:00:15+02	2020-08-28 09:01:11+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-08-28 09:01:12+02	2020-08-28 09:01:33+02	print	xpdfopen	\N	xpdfopen-0.86
2020-08-28 09:01:34+02	2020-08-28 09:02:47+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-08-28 09:02:48+02	2020-08-28 09:03:09+02	multimedia	libva	\N	libva-2.8.0
2020-08-28 09:03:10+02	2020-08-28 09:04:21+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-08-28 09:04:22+02	2020-08-28 09:09:00+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-08-28 09:09:02+02	2020-08-28 09:09:20+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-08-28 09:09:20+02	2020-08-28 09:09:34+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-08-28 09:09:34+02	2020-08-28 09:09:54+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-08-28 09:09:54+02	2020-08-28 09:10:32+02	devel	woff2	\N	woff2-1.0.2_4
2020-08-28 09:10:33+02	2020-08-28 09:10:47+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-08-28 09:10:47+02	2020-08-28 09:11:10+02	textproc	py-rdflib	py37	py37-rdflib-5.0.0
2020-08-28 09:11:11+02	2020-08-28 09:11:30+02	audio	lv2	\N	lv2-1.18.0
2020-08-28 09:11:31+02	2020-08-28 09:12:06+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-08-28 09:12:06+02	2020-08-28 09:12:30+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-08-28 09:12:31+02	2020-08-28 09:14:37+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-08-28 09:14:40+02	2020-08-28 09:15:41+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-08-28 09:15:42+02	2020-08-28 09:17:10+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-08-28 09:17:11+02	2020-08-28 09:18:17+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-08-28 09:18:18+02	2020-08-28 09:18:46+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-08-28 09:18:46+02	2020-08-28 09:19:05+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-08-28 09:19:06+02	2020-08-28 09:20:28+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-08-28 09:20:28+02	2020-08-28 09:20:43+02	textproc	py-toml	py37	py37-toml-0.10.1
2020-08-28 09:20:44+02	2020-08-28 09:21:02+02	x11	libXres	\N	libXres-1.2.0_2
2020-08-28 09:21:02+02	2020-08-28 09:21:22+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-08-28 09:21:23+02	2020-08-28 09:21:36+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3
2020-08-28 09:21:37+02	2020-08-28 09:27:37+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-08-28 09:27:38+02	2020-08-28 09:27:57+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-08-28 09:27:58+02	2020-08-28 09:28:23+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.9_7
2020-08-28 09:28:23+02	2020-08-28 09:29:19+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-08-28 09:29:20+02	2020-08-28 09:30:26+02	sysutils	policykit	\N	policykit-0.9_10
2020-08-28 09:30:26+02	2020-08-28 09:31:05+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-08-28 09:31:06+02	2020-08-28 09:31:20+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-08-28 09:31:20+02	2020-08-28 09:32:00+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-08-28 09:32:00+02	2020-08-28 09:55:20+02	devel	gdb	py37	gdb-9.2_1
2020-08-28 09:55:21+02	2020-08-28 09:56:08+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-08-28 09:56:08+02	2020-08-28 10:01:22+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-08-28 10:01:23+02	2020-08-28 10:02:13+02	www	py-django22	py37	py37-django22-2.2.15
2020-08-28 10:02:14+02	2020-08-28 10:09:34+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-08-28 10:09:36+02	2020-08-28 10:09:53+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-08-28 10:09:54+02	2020-08-28 10:11:43+02	devel	git	\N	git-2.28.0
2020-08-28 08:00:37+02	2020-08-28 12:03:14+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200823
2020-08-28 12:03:18+02	2020-08-28 12:05:38+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-08-28 12:05:38+02	2020-08-28 12:06:03+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-08-28 12:06:04+02	2020-08-28 12:06:24+02	x11	xrdb	\N	xrdb-1.2.0
2020-08-28 12:06:25+02	2020-08-28 12:06:44+02	x11	luit	\N	luit-20190106_1
2020-08-28 12:06:46+02	2020-08-28 12:07:00+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-08-28 12:07:02+02	2020-08-28 12:07:18+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-08-28 12:07:18+02	2020-08-28 12:07:36+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-08-28 12:07:36+02	2020-08-28 12:11:11+02	security	gnupg	\N	gnupg-2.2.22
2020-08-28 12:11:11+02	2020-08-28 12:11:33+02	x11	xauth	\N	xauth-1.1
2020-08-28 12:11:34+02	2020-08-28 12:12:07+02	x11	xterm	\N	xterm-359
2020-08-28 12:12:08+02	2020-08-28 12:12:32+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.9_6
2020-08-28 08:13:46+02	2020-08-28 14:17:12+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-08-28 14:17:13+02	2020-08-28 14:25:33+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-08-28 14:25:34+02	2020-08-28 14:29:25+02	graphics	cairo	\N	cairo-1.16.0,2
2020-08-28 14:29:26+02	2020-08-28 14:30:51+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-08-28 14:30:52+02	2020-08-28 14:35:47+02	print	harfbuzz	\N	harfbuzz-2.7.1
2020-08-28 14:35:48+02	2020-08-28 14:36:57+02	net	avahi-app	\N	avahi-app-0.7_3
2020-08-28 14:36:57+02	2020-08-28 14:37:38+02	print	cups	\N	cups-2.3.3_1
2020-08-28 14:37:38+02	2020-08-28 14:40:09+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-28 14:40:10+02	2020-08-28 14:40:42+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-08-28 10:11:44+02	2020-08-28 14:41:13+02	lang	rust	\N	rust-1.45.2
2020-08-28 14:41:15+02	2020-08-28 14:41:59+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-08-28 14:40:43+02	2020-08-28 14:41:59+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_8
2020-08-28 14:42:00+02	2020-08-28 14:42:30+02	accessibility	atk	\N	atk-2.36.0
2020-08-28 14:42:30+02	2020-08-28 14:42:56+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-08-28 14:42:57+02	2020-08-28 14:43:45+02	sysutils	polkit	\N	polkit-0.116
2020-08-28 14:43:45+02	2020-08-28 14:44:29+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-08-28 14:44:30+02	2020-08-28 14:45:30+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-08-28 14:42:00+02	2020-08-28 14:47:12+02	graphics	graphviz	\N	graphviz-2.44.1_1
2020-08-28 14:45:31+02	2020-08-28 14:47:13+02	graphics	colord	\N	colord-1.3.5
2020-08-28 14:47:13+02	2020-08-28 14:47:30+02	devel	libgudev	\N	libgudev-233
2020-08-28 14:47:31+02	2020-08-28 14:47:48+02	x11	libwacom	\N	libwacom-1.4.1
2020-08-28 14:47:51+02	2020-08-28 14:48:27+02	x11	libinput	\N	libinput-1.15.6
2020-08-28 14:48:29+02	2020-08-28 14:49:20+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-08-28 14:49:21+02	2020-08-28 14:51:18+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-28 14:47:13+02	2020-08-28 14:51:21+02	lang	vala	\N	vala-0.40.15,1
2020-08-28 14:51:22+02	2020-08-28 14:52:10+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-28 14:51:18+02	2020-08-28 14:53:47+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-28 14:53:48+02	2020-08-28 14:56:18+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-28 14:56:19+02	2020-08-28 14:57:08+02	graphics	libGLU	\N	libGLU-9.0.1
2020-08-28 14:57:08+02	2020-08-28 14:57:38+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.1
2020-08-28 14:52:11+02	2020-08-28 14:57:42+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-28 14:57:43+02	2020-08-28 15:01:20+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-28 14:57:38+02	2020-08-28 15:01:41+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-28 15:01:41+02	2020-08-28 15:02:22+02	graphics	poppler	\N	poppler-20.08.0_1
2020-08-28 15:02:24+02	2020-08-28 15:02:59+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-08-28 15:03:00+02	2020-08-28 15:03:25+02	net	glib-networking	\N	glib-networking-2.56.1_2
2020-08-28 15:01:20+02	2020-08-28 15:03:56+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-08-28 15:03:26+02	2020-08-28 15:04:27+02	devel	dconf	\N	dconf-0.36.0
2020-08-28 15:04:29+02	2020-08-28 15:05:42+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-28 15:05:42+02	2020-08-28 15:06:15+02	devel	json-glib	\N	json-glib-1.4.4
2020-08-28 15:03:57+02	2020-08-28 15:07:37+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-08-28 15:06:16+02	2020-08-28 15:08:45+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-08-28 15:07:38+02	2020-08-28 15:11:26+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-08-28 15:08:46+02	2020-08-28 15:11:30+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-08-28 15:11:31+02	2020-08-28 15:11:56+02	devel	libnotify	\N	libnotify-0.7.8
2020-08-28 15:11:56+02	2020-08-28 15:12:53+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-28 15:12:53+02	2020-08-28 15:13:50+02	security	libsecret	\N	libsecret-0.18.6_1
2020-08-28 15:13:50+02	2020-08-28 15:15:28+02	graphics	frei0r	\N	frei0r-1.6.1
2020-08-28 15:11:27+02	2020-08-28 15:15:28+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-28 15:15:29+02	2020-08-28 15:17:51+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-08-28 15:17:52+02	2020-08-28 15:25:42+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-28 15:25:43+02	2020-08-28 15:26:48+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-28 15:26:49+02	2020-08-28 15:27:12+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-08-28 15:27:12+02	2020-08-28 15:27:33+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-08-28 15:27:34+02	2020-08-28 15:28:09+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-08-28 15:28:09+02	2020-08-28 15:29:19+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-28 15:29:20+02	2020-08-28 15:30:59+02	graphics	graphene	\N	graphene-1.10.0
2020-08-28 15:31:00+02	2020-08-28 15:31:17+02	www	libwpe	\N	libwpe-1.4.0.1
2020-08-28 15:31:17+02	2020-08-28 15:33:48+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-08-28 15:33:49+02	2020-08-28 15:34:56+02	net	geoclue	\N	geoclue-2.5.5
2020-08-28 15:34:57+02	2020-08-28 15:35:23+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-08-28 15:15:29+02	2020-08-28 15:36:09+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-28 15:35:24+02	2020-08-28 15:36:29+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.73.0
2020-08-28 15:36:10+02	2020-08-28 15:39:08+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-28 15:36:29+02	2020-08-28 15:43:50+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-28 15:43:51+02	2020-08-28 15:44:42+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-28 15:44:42+02	2020-08-28 15:45:27+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-28 15:45:28+02	2020-08-28 15:49:36+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-28 15:49:37+02	2020-08-28 15:50:17+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-08-28 15:50:17+02	2020-08-28 15:50:48+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-08-28 15:39:09+02	2020-08-28 15:51:24+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-28 15:51:24+02	2020-08-28 15:51:41+02	print	libraqm	\N	libraqm-0.6.0
2020-08-28 15:50:49+02	2020-08-28 15:52:01+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-28 15:51:42+02	2020-08-28 15:52:47+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-08-28 15:52:48+02	2020-08-28 15:53:40+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-28 15:52:02+02	2020-08-28 15:55:19+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-28 15:55:20+02	2020-08-28 16:04:29+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-28 16:04:31+02	2020-08-28 16:05:29+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.73.0
2020-08-28 16:05:30+02	2020-08-28 16:06:21+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-28 16:06:22+02	2020-08-28 16:07:21+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-28 16:07:22+02	2020-08-28 16:08:30+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-28 15:53:41+02	2020-08-28 16:09:11+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-08-28 16:08:31+02	2020-08-28 16:09:39+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-28 16:09:12+02	2020-08-28 16:10:04+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-28 16:09:39+02	2020-08-28 16:10:34+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-08-28 16:10:05+02	2020-08-28 16:11:19+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-08-28 16:10:35+02	2020-08-28 16:11:26+02	archivers	kf5-karchive	\N	kf5-karchive-5.73.0
2020-08-28 16:11:19+02	2020-08-28 16:12:35+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-08-28 16:11:26+02	2020-08-28 16:12:38+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-28 16:12:36+02	2020-08-28 16:13:18+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-08-28 16:12:39+02	2020-08-28 16:13:24+02	sysutils	hal	\N	hal-0.5.14_34
2020-08-28 16:13:19+02	2020-08-28 16:14:08+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-28 16:14:08+02	2020-08-28 16:15:49+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-28 16:15:50+02	2020-08-28 16:17:21+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-08-28 16:13:25+02	2020-08-28 16:18:07+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_4,1
2020-08-28 16:17:22+02	2020-08-28 16:19:05+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-28 16:18:11+02	2020-08-28 16:20:11+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-28 16:20:11+02	2020-08-28 16:21:21+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-08-28 16:19:05+02	2020-08-28 16:22:06+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-08-28 16:21:21+02	2020-08-28 16:22:16+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-28 16:22:07+02	2020-08-28 16:22:42+02	graphics	poppler-glib	\N	poppler-glib-20.08.0_1
2020-08-28 16:22:16+02	2020-08-28 16:23:32+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-08-28 16:22:42+02	2020-08-28 16:24:22+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-08-28 16:23:32+02	2020-08-28 16:24:34+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-08-28 16:24:22+02	2020-08-28 16:25:17+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-08-28 16:25:18+02	2020-08-28 16:26:12+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-28 16:24:34+02	2020-08-28 16:28:02+02	graphics	glew	\N	glew-2.2.0
2020-08-28 16:28:03+02	2020-08-28 16:32:57+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-08-28 16:32:57+02	2020-08-28 16:33:49+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-08-28 16:33:49+02	2020-08-28 16:34:53+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-08-28 16:34:54+02	2020-08-28 16:36:19+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-28 16:36:20+02	2020-08-28 16:37:27+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-28 16:37:28+02	2020-08-28 16:39:12+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.73.0
2020-08-28 16:39:18+02	2020-08-28 16:40:50+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-08-28 16:40:50+02	2020-08-28 16:42:11+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-28 16:42:12+02	2020-08-28 16:43:19+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-08-28 16:43:19+02	2020-08-28 16:44:46+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-08-28 16:44:46+02	2020-08-28 16:46:19+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-28 16:46:19+02	2020-08-28 16:47:39+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-08-28 16:47:42+02	2020-08-28 16:48:00+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-08-28 16:48:01+02	2020-08-28 16:49:05+02	net	remmina	\N	remmina-1.3.9
2020-08-28 16:26:13+02	2020-08-28 16:49:09+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-08-28 16:49:10+02	2020-08-28 16:49:48+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-08-28 16:49:48+02	2020-08-28 16:50:49+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-08-28 16:49:05+02	2020-08-28 16:50:51+02	security	gcr	\N	gcr-3.28.0
2020-08-28 16:50:49+02	2020-08-28 16:51:13+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-08-28 16:50:52+02	2020-08-28 16:51:25+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-08-28 16:51:13+02	2020-08-28 16:51:55+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-08-28 16:51:25+02	2020-08-28 16:52:26+02	devel	libgtop	\N	libgtop-2.40.0
2020-08-28 16:52:27+02	2020-08-28 16:53:20+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-28 16:51:55+02	2020-08-28 16:53:48+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-28 16:53:21+02	2020-08-28 16:53:58+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-08-28 16:53:49+02	2020-08-28 16:54:32+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-28 16:53:59+02	2020-08-28 16:54:41+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-08-28 16:54:32+02	2020-08-28 16:54:57+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-08-28 16:54:57+02	2020-08-28 16:55:37+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-08-28 16:54:42+02	2020-08-28 16:55:38+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-08-28 16:55:38+02	2020-08-28 16:57:20+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-28 16:55:38+02	2020-08-28 16:57:54+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.73.0
2020-08-28 16:57:21+02	2020-08-28 16:58:10+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-28 16:57:55+02	2020-08-28 16:59:03+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-08-28 16:58:11+02	2020-08-28 16:59:06+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-08-28 16:59:04+02	2020-08-28 17:00:21+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-28 17:00:22+02	2020-08-28 17:01:23+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-08-28 16:59:07+02	2020-08-28 17:02:31+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-28 17:01:23+02	2020-08-28 17:03:17+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-28 17:03:18+02	2020-08-28 17:03:53+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-28 17:03:54+02	2020-08-28 17:04:45+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-28 17:04:45+02	2020-08-28 17:05:06+02	print	gl2ps	\N	gl2ps-1.4.0
2020-08-28 17:05:07+02	2020-08-28 17:06:17+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-08-28 17:06:18+02	2020-08-28 17:09:05+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-28 17:02:31+02	2020-08-28 17:09:07+02	math	vtk8	\N	vtk8-8.2.0
2020-08-28 17:09:06+02	2020-08-28 17:10:55+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-28 17:10:58+02	2020-08-28 17:11:12+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-08-28 17:11:12+02	2020-08-28 17:12:59+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-08-28 17:13:00+02	2020-08-28 17:15:38+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-08-28 17:09:08+02	2020-08-28 17:15:56+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-28 17:15:39+02	2020-08-28 17:17:32+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-08-28 17:17:32+02	2020-08-28 17:17:56+02	security	gpgme	\N	gpgme-1.14.0
2020-08-28 17:17:57+02	2020-08-28 17:18:27+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-08-28 17:18:28+02	2020-08-28 17:19:29+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-08-28 17:15:57+02	2020-08-28 17:20:04+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-28 17:19:30+02	2020-08-28 17:20:33+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-28 17:20:34+02	2020-08-28 17:20:45+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-08-28 17:20:46+02	2020-08-28 17:21:51+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-28 17:21:52+02	2020-08-28 17:23:02+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-08-28 17:23:03+02	2020-08-28 17:25:22+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-28 17:25:23+02	2020-08-28 17:26:10+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-08-28 17:26:11+02	2020-08-28 17:26:47+02	devel	gamin	\N	gamin-0.1.10_10
2020-08-28 17:26:47+02	2020-08-28 17:27:52+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-28 17:27:52+02	2020-08-28 17:28:39+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-28 17:20:05+02	2020-08-28 17:29:00+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-28 17:29:01+02	2020-08-28 17:29:15+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-08-28 17:28:40+02	2020-08-28 17:29:28+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-28 17:29:15+02	2020-08-28 17:30:59+02	devel	talloc	\N	talloc-2.3.1
2020-08-28 17:31:00+02	2020-08-28 17:32:09+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-28 17:32:10+02	2020-08-28 17:32:41+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-08-28 17:32:42+02	2020-08-28 17:32:59+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7
2020-08-28 17:29:29+02	2020-08-28 17:33:02+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-08-28 17:33:00+02	2020-08-28 17:33:39+02	sysutils	upower	\N	upower-0.99.11
2020-08-28 17:33:39+02	2020-08-28 17:33:51+02	x11	appres	\N	appres-1.0.5
2020-08-28 17:33:51+02	2020-08-28 17:34:07+02	x11	bitmap	\N	bitmap-1.0.9
2020-08-28 17:34:08+02	2020-08-28 17:35:09+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-08-28 17:35:10+02	2020-08-28 17:35:54+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-08-28 17:35:55+02	2020-08-28 17:39:23+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-28 17:39:24+02	2020-08-28 17:39:41+02	x11	sessreg	\N	sessreg-1.1.2
2020-08-28 17:39:42+02	2020-08-28 17:39:58+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-08-28 17:39:59+02	2020-08-28 17:40:14+02	x11	smproxy	\N	smproxy-1.0.6
2020-08-28 17:40:15+02	2020-08-28 17:40:42+02	x11	x11perf	\N	x11perf-1.6.1
2020-08-28 17:40:43+02	2020-08-28 17:40:59+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-08-28 17:41:00+02	2020-08-28 17:41:31+02	x11	xcalc	\N	xcalc-1.1.0
2020-08-28 17:41:32+02	2020-08-28 17:41:50+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-08-28 17:41:51+02	2020-08-28 17:42:12+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-08-28 17:42:12+02	2020-08-28 17:42:27+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-08-28 17:42:28+02	2020-08-28 17:42:44+02	x11	xev	\N	xev-1.2.4
2020-08-28 17:42:44+02	2020-08-28 17:43:02+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-08-28 17:43:03+02	2020-08-28 17:43:24+02	x11	xgamma	\N	xgamma-1.0.6
2020-08-28 17:43:24+02	2020-08-28 17:43:50+02	x11	xgc	\N	xgc-1.0.5
2020-08-28 17:43:50+02	2020-08-28 17:44:09+02	x11	xhost	\N	xhost-1.0.8
2020-08-28 17:44:10+02	2020-08-28 17:44:27+02	x11	xinit	\N	xinit-1.4.1,1
2020-08-28 17:44:27+02	2020-08-28 17:44:44+02	x11	xinput	\N	xinput-1.6.3
2020-08-28 17:44:45+02	2020-08-28 17:44:58+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-08-28 17:44:59+02	2020-08-28 17:45:16+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-08-28 17:45:17+02	2020-08-28 17:45:32+02	x11	xkill	\N	xkill-1.0.5
2020-08-28 17:45:33+02	2020-08-28 17:45:50+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-08-28 17:45:51+02	2020-08-28 17:46:06+02	x11	xmessage	\N	xmessage-1.0.5
2020-08-28 17:46:07+02	2020-08-28 17:46:24+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-08-28 17:46:26+02	2020-08-28 17:46:44+02	x11	xpr	\N	xpr-1.0.5
2020-08-28 17:46:44+02	2020-08-28 17:47:01+02	x11	xrandr	\N	xrandr-1.5.1
2020-08-28 17:47:02+02	2020-08-28 17:47:19+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-08-28 17:47:20+02	2020-08-28 17:48:21+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-08-28 17:48:25+02	2020-08-28 17:48:59+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-08-28 17:49:01+02	2020-08-28 17:49:16+02	x11	xwd	\N	xwd-1.0.7
2020-08-28 17:49:17+02	2020-08-28 17:49:28+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-08-28 17:49:28+02	2020-08-28 17:49:39+02	x11	xwud	\N	xwud-1.0.5
2020-08-28 17:49:40+02	2020-08-28 17:50:02+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-08-28 17:50:02+02	2020-08-28 17:51:08+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-28 18:36:04+02	2020-08-28 18:36:14+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-08-28 17:51:09+02	2020-08-28 17:51:52+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-28 17:51:53+02	2020-08-28 17:52:39+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-28 17:33:02+02	2020-08-28 17:52:46+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-10-06 07:50:18+02	2020-10-06 07:50:53+02	audio	lv2	\N	lv2-1.18.0_1
2020-08-28 17:52:39+02	2020-08-28 17:53:22+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-28 17:52:47+02	2020-08-28 17:53:36+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-28 17:53:37+02	2020-08-28 17:54:21+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-28 17:53:23+02	2020-08-28 17:55:16+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-08-28 17:54:22+02	2020-08-28 17:55:19+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-28 17:55:17+02	2020-08-28 17:55:57+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-08-28 17:55:20+02	2020-08-28 17:56:15+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-08-28 17:55:57+02	2020-08-28 17:56:27+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-28 17:56:28+02	2020-08-28 17:56:41+02	x11-wm	twm	\N	twm-1.0.11
2020-08-28 17:56:41+02	2020-08-28 17:58:28+02	archivers	gcab	\N	gcab-1.3
2020-08-28 12:12:33+02	2020-08-28 18:00:18+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-08-28 17:58:29+02	2020-08-28 18:00:57+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-08-28 17:56:15+02	2020-08-28 18:01:30+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-28 18:00:58+02	2020-08-28 18:02:00+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-28 18:02:01+02	2020-08-28 18:02:36+02	devel	librest	\N	rest-0.7.93
2020-08-28 18:00:24+02	2020-08-28 18:02:39+02	databases	tdb	\N	tdb-1.4.3,1
2020-08-28 18:02:36+02	2020-08-28 18:02:46+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-08-28 18:02:47+02	2020-08-28 18:03:11+02	graphics	poppler-utils	\N	poppler-utils-20.08.0_1
2020-08-28 18:02:39+02	2020-08-28 18:04:02+02	devel	tevent	\N	tevent-0.10.2
2020-08-28 18:04:03+02	2020-08-28 18:04:20+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-08-28 18:04:20+02	2020-08-28 18:04:53+02	print	libotf	\N	libotf-0.9.16
2020-08-28 18:04:53+02	2020-08-28 18:05:49+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-08-28 18:03:12+02	2020-08-28 18:06:00+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-28 18:05:49+02	2020-08-28 18:06:32+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-28 18:06:00+02	2020-08-28 18:06:47+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-28 18:06:33+02	2020-08-28 18:07:05+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-08-28 18:07:06+02	2020-08-28 18:07:19+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-08-28 18:06:48+02	2020-08-28 18:07:33+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-08-28 18:07:33+02	2020-08-28 18:07:51+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-08-28 18:07:19+02	2020-08-28 18:08:18+02	x11	babl	\N	babl-0.1.78
2020-08-28 18:08:18+02	2020-08-28 18:08:46+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-08-28 18:08:47+02	2020-08-28 18:08:58+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-08-28 18:07:51+02	2020-08-28 18:09:03+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-28 18:08:58+02	2020-08-28 18:09:58+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-28 18:09:05+02	2020-08-28 18:11:02+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-28 18:11:02+02	2020-08-28 18:13:05+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-28 18:09:58+02	2020-08-28 18:13:41+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_4,1
2020-08-28 18:13:06+02	2020-08-28 18:14:31+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-28 18:13:42+02	2020-08-28 18:14:39+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-28 18:14:32+02	2020-08-28 18:15:46+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-28 18:14:40+02	2020-08-28 18:15:48+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-28 18:01:30+02	2020-08-28 18:16:36+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-28 18:15:49+02	2020-08-28 18:16:41+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-28 18:16:37+02	2020-08-28 18:17:42+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-08-28 18:17:42+02	2020-08-28 18:17:54+02	devel	py-click	py37	py37-click-7.0
2020-08-28 18:17:55+02	2020-08-28 18:18:08+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-08-28 18:18:08+02	2020-08-28 18:18:22+02	devel	py-olefile	py37	py37-olefile-0.46
2020-08-28 18:18:23+02	2020-08-28 18:18:34+02	devel	py-ply	py37	py37-ply-3.11
2020-08-28 18:16:42+02	2020-08-28 18:18:43+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-08-28 18:18:34+02	2020-08-28 18:18:52+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-08-28 18:18:43+02	2020-08-28 18:18:57+02	devel	py-zipp	py37	py37-zipp-3.1.0
2020-08-28 18:18:52+02	2020-08-28 18:19:07+02	devel	scons	py37	scons-py37-3.1.2
2020-08-28 18:19:07+02	2020-08-28 18:19:28+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-08-28 18:15:47+02	2020-08-28 18:19:36+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-08-28 18:19:28+02	2020-08-28 18:19:49+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-08-28 18:19:37+02	2020-08-28 18:19:56+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-08-28 18:18:58+02	2020-08-28 18:20:13+02	graphics	gegl	\N	gegl-0.4.24_4
2020-08-28 18:20:13+02	2020-08-28 18:20:53+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0,1
2020-08-28 18:19:56+02	2020-08-28 18:21:33+02	mail	mailutils	\N	mailutils-3.10
2020-08-28 18:20:53+02	2020-08-28 18:22:29+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-28 18:22:29+02	2020-08-28 18:22:42+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-08-28 18:22:43+02	2020-08-28 18:23:49+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-28 18:23:49+02	2020-08-28 18:24:41+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-28 18:24:42+02	2020-08-28 18:26:50+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-28 18:26:50+02	2020-08-28 18:27:42+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-08-28 18:27:43+02	2020-08-28 18:29:15+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-28 18:29:16+02	2020-08-28 18:29:56+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-28 18:29:57+02	2020-08-28 18:30:08+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-08-28 18:30:08+02	2020-08-28 18:30:25+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-08-28 18:30:26+02	2020-08-28 18:30:47+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-08-28 18:19:50+02	2020-08-28 18:31:12+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-28 18:30:48+02	2020-08-28 18:31:16+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-08-28 18:31:13+02	2020-08-28 18:31:49+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-08-28 18:31:17+02	2020-08-28 18:32:16+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-08-28 18:21:34+02	2020-08-28 18:32:43+02	net	samba410	\N	samba410-4.10.17
2020-08-28 18:32:17+02	2020-08-28 18:33:06+02	x11	zenity	\N	zenity-3.32.0
2020-08-28 18:33:07+02	2020-08-28 18:33:15+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-08-28 18:32:43+02	2020-08-28 18:33:25+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-08-28 18:33:16+02	2020-08-28 18:33:25+02	databases	py-redis	py37	py37-redis-3.5.3
2020-08-28 18:33:26+02	2020-08-28 18:33:53+02	devel	apache-ant	\N	apache-ant-1.10.8
2020-08-28 18:31:50+02	2020-08-28 18:34:15+02	x11	xorg	\N	xorg-7.7_3
2020-08-28 18:33:54+02	2020-08-28 18:35:17+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-28 18:34:15+02	2020-08-28 18:35:23+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-28 18:35:18+02	2020-08-28 18:35:33+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-08-28 18:35:25+02	2020-08-28 18:35:41+02	devel	py-appdirs	py37	py37-appdirs-1.4.4
2020-08-28 18:35:34+02	2020-08-28 18:35:49+02	devel	py-backports	py37	py37-backports-1
2020-08-28 18:35:41+02	2020-08-28 18:35:54+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-08-28 18:35:49+02	2020-08-28 18:35:59+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-08-28 18:33:26+02	2020-08-28 18:36:03+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-28 18:36:00+02	2020-08-28 18:36:09+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-1.7.0
2020-08-28 18:36:10+02	2020-08-28 18:36:20+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-08-28 18:35:55+02	2020-08-28 18:36:23+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-08-28 18:36:14+02	2020-08-28 18:36:24+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-08-28 18:36:20+02	2020-08-28 18:36:29+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-08-28 18:36:24+02	2020-08-28 18:36:33+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-08-28 18:36:25+02	2020-08-28 18:36:35+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-08-28 18:36:30+02	2020-08-28 18:37:07+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-28 18:36:33+02	2020-08-28 18:37:16+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-28 18:36:36+02	2020-08-28 18:37:16+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-28 18:37:17+02	2020-08-28 18:37:27+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-08-28 18:37:08+02	2020-08-28 18:38:30+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-28 18:37:17+02	2020-08-28 18:40:13+02	dns	bind-tools	\N	bind-tools-9.16.6
2020-08-28 18:40:13+02	2020-08-28 18:40:30+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-08-28 18:37:27+02	2020-08-28 18:40:45+02	editors	emacs	full	emacs-27.1,3
2020-08-28 18:40:45+02	2020-08-28 18:41:55+02	graphics	glfw	\N	glfw-3.3.2
2020-08-28 18:41:55+02	2020-08-28 18:45:28+02	graphics	movit	\N	movit-1.6.2_2
2020-08-28 18:38:31+02	2020-08-28 18:47:11+02	graphics	Coin	\N	Coin-4.0.0_1
2020-08-28 18:40:31+02	2020-08-28 18:47:18+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-08-28 18:47:19+02	2020-08-28 18:47:46+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-08-28 18:47:11+02	2020-08-28 18:47:58+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-08-28 18:47:47+02	2020-08-28 18:48:13+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-08-28 18:47:58+02	2020-08-28 18:48:48+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-28 18:48:49+02	2020-08-28 18:49:04+02	net	gssdp	\N	gssdp-1.2.3
2020-08-28 18:45:28+02	2020-08-28 18:49:10+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-28 18:49:11+02	2020-08-28 18:49:42+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-08-28 18:49:05+02	2020-08-28 18:49:43+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-08-28 18:49:42+02	2020-08-28 18:51:53+02	print	fontforge	\N	fontforge-20190801_2
2020-08-28 18:48:13+02	2020-08-28 18:52:38+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-08-28 18:52:39+02	2020-08-28 18:52:49+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-08-28 18:52:50+02	2020-08-28 18:53:53+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-28 18:53:54+02	2020-08-28 18:55:15+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-08-28 18:55:15+02	2020-08-28 18:55:28+02	sysutils	py-psutil	py37	py37-psutil-5.7.2
2020-08-28 18:55:29+02	2020-08-28 18:55:38+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-08-28 18:55:39+02	2020-08-28 18:55:49+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-08-28 18:55:49+02	2020-08-28 18:56:01+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-08-28 18:49:43+02	2020-08-28 19:01:30+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-08-28 19:01:31+02	2020-08-28 19:01:45+02	www	py-h2	py37	py37-h2-3.2.0
2020-08-28 19:01:45+02	2020-08-28 19:01:57+02	www	py-hyperlink	py37	py37-hyperlink-20.0.1
2020-08-28 19:01:57+02	2020-08-28 19:02:09+02	www	py-priority	py37	py37-priority-1.3.0
2020-08-28 19:02:09+02	2020-08-28 19:03:36+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-28 19:03:37+02	2020-08-28 19:05:05+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-28 19:05:07+02	2020-08-28 19:06:04+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-28 19:06:04+02	2020-08-28 19:07:01+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-28 19:07:02+02	2020-08-28 19:08:32+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-28 18:51:54+02	2020-08-28 19:08:53+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-08-28 19:08:53+02	2020-08-28 19:10:24+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-28 19:10:25+02	2020-08-28 19:11:42+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-28 19:11:43+02	2020-08-28 19:12:30+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-28 19:12:31+02	2020-08-28 19:13:40+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-28 19:13:41+02	2020-08-28 19:15:19+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-28 19:15:20+02	2020-08-28 19:18:05+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-28 19:18:06+02	2020-08-28 19:19:24+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-28 19:08:32+02	2020-08-28 19:20:20+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-08-28 19:19:25+02	2020-08-28 19:20:31+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-08-28 19:20:32+02	2020-08-28 19:21:37+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-28 19:20:21+02	2020-08-28 19:21:45+02	x11-wm	marco	\N	marco-1.22.4
2020-08-28 19:21:38+02	2020-08-28 19:22:27+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-08-28 19:21:45+02	2020-08-28 19:23:56+02	audio	taglib	\N	taglib-1.12.b.1_1
2020-08-28 19:22:28+02	2020-08-28 19:24:44+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-28 19:23:57+02	2020-08-28 19:25:19+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-08-28 19:25:20+02	2020-08-28 19:27:24+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-28 19:24:44+02	2020-08-28 19:27:25+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-28 19:27:25+02	2020-08-28 19:27:37+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-08-28 19:27:26+02	2020-08-28 19:27:39+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-08-28 19:27:37+02	2020-08-28 19:27:55+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-08-28 19:27:39+02	2020-08-28 19:27:57+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-08-28 19:27:56+02	2020-08-28 19:28:09+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-08-28 19:27:58+02	2020-08-28 19:28:11+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-08-28 19:28:10+02	2020-08-28 19:29:09+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-08-28 19:29:10+02	2020-08-28 19:33:28+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-28 19:33:30+02	2020-08-28 19:34:28+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-28 19:34:29+02	2020-08-28 19:34:58+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-08-28 19:34:59+02	2020-08-28 19:37:26+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-28 19:37:28+02	2020-08-28 19:39:12+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-08-28 19:39:13+02	2020-08-28 19:40:01+02	graphics	libgxps	\N	libgxps-0.3.1
2020-08-28 19:40:04+02	2020-08-28 19:41:13+02	graphics	libheif	\N	libheif-1.6.2
2020-08-28 19:41:14+02	2020-08-28 19:41:54+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-08-28 19:41:56+02	2020-08-28 19:43:38+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-08-28 19:43:39+02	2020-08-28 19:44:22+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-28 19:44:23+02	2020-08-28 19:45:29+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-28 19:45:30+02	2020-08-28 19:46:45+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-28 19:46:47+02	2020-08-28 19:48:10+02	graphics	vigra	\N	vigra-1.11.1_12
2020-08-28 19:28:13+02	2020-08-28 19:48:15+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-28 18:56:02+02	2020-08-28 19:53:08+02	www	chromium	\N	chromium-84.0.4147.135
2020-08-28 19:53:11+02	2020-08-28 19:54:01+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-08-28 19:48:15+02	2020-08-28 19:54:34+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-28 19:54:02+02	2020-08-28 19:54:46+02	multimedia	libass	\N	libass-0.14.0
2020-08-28 19:54:46+02	2020-08-28 19:55:46+02	net	freerdp	\N	freerdp-2.2.0
2020-08-28 19:54:34+02	2020-08-28 19:55:49+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-28 19:55:50+02	2020-08-28 19:56:08+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-08-28 19:55:46+02	2020-08-28 19:56:10+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-28 19:56:08+02	2020-08-28 19:56:17+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-08-28 19:56:11+02	2020-08-28 19:56:48+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-28 19:56:18+02	2020-08-28 19:57:23+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-28 19:56:49+02	2020-08-28 19:57:36+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-28 19:57:36+02	2020-08-28 19:57:54+02	print	libspectre	\N	libspectre-0.2.9
2020-08-28 19:57:54+02	2020-08-28 19:59:42+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-28 19:57:24+02	2020-08-28 20:00:16+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-28 19:48:11+02	2020-08-28 20:00:54+02	math	R	\N	R-4.0.2
2020-08-28 20:00:55+02	2020-08-28 20:01:32+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-08-28 20:01:32+02	2020-08-28 20:02:02+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-08-28 20:02:02+02	2020-08-28 20:02:16+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-08-28 20:02:16+02	2020-08-28 20:02:31+02	sysutils	hal-info	\N	hal-info-20091130
2020-08-28 19:59:43+02	2020-08-28 20:04:36+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-28 20:02:32+02	2020-08-28 20:04:39+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-28 20:00:16+02	2020-08-28 20:04:55+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-28 20:04:37+02	2020-08-28 20:05:18+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-28 20:05:18+02	2020-08-28 20:05:47+02	textproc	enchant	\N	enchant-1.6.0_9
2020-08-28 20:05:47+02	2020-08-28 20:05:58+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-08-28 20:04:39+02	2020-08-28 20:06:06+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-28 20:05:59+02	2020-08-28 20:06:09+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-08-28 20:06:06+02	2020-08-28 20:06:16+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-08-28 20:04:56+02	2020-08-28 20:06:27+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-28 20:06:10+02	2020-08-28 20:06:28+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-08-28 20:06:28+02	2020-08-28 20:06:37+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-08-28 20:06:16+02	2020-08-28 20:06:50+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-28 20:06:37+02	2020-08-28 20:07:03+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-28 20:07:03+02	2020-08-28 20:07:11+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-08-28 20:06:28+02	2020-08-28 20:07:17+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-28 20:07:11+02	2020-08-28 20:07:19+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-08-28 20:06:50+02	2020-08-28 20:07:49+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-28 20:07:20+02	2020-08-28 20:07:51+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-08-28 20:07:51+02	2020-08-28 20:08:16+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-08-28 20:07:50+02	2020-08-28 20:08:49+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-28 20:07:17+02	2020-08-28 20:08:51+02	x11-servers	xephyr	\N	xephyr-1.20.8_4,1
2020-08-28 20:08:16+02	2020-08-28 20:09:00+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-28 20:08:50+02	2020-08-28 20:09:34+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-28 20:08:52+02	2020-08-28 20:09:40+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-28 20:09:00+02	2020-08-28 20:09:53+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-28 20:09:53+02	2020-08-28 20:10:36+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-08-28 20:10:36+02	2020-08-28 20:11:00+02	audio	dssi	\N	dssi-1.1.1_8
2020-08-28 20:09:40+02	2020-08-28 20:11:35+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-28 20:11:01+02	2020-08-28 20:11:45+02	audio	faudio	\N	FAudio-20.08
2020-08-28 20:11:46+02	2020-08-28 20:12:04+02	databases	py-apsw	py37	py37-apsw-3.32.2
2020-08-28 20:11:36+02	2020-08-28 20:12:39+02	audio	mate-media	\N	mate-media-1.24.1
2020-08-28 20:12:04+02	2020-08-28 20:13:24+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-28 20:12:40+02	2020-08-28 20:13:57+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-28 20:09:34+02	2020-08-28 20:14:09+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-28 20:13:58+02	2020-08-28 20:14:41+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-28 20:14:10+02	2020-08-28 20:14:59+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-28 20:14:41+02	2020-08-28 20:15:03+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-08-28 20:15:00+02	2020-08-28 20:15:18+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-08-28 20:13:25+02	2020-08-28 20:15:49+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-28 20:15:19+02	2020-08-28 20:15:59+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-08-28 20:15:04+02	2020-08-28 20:16:15+02	devel	gradle5	\N	gradle5-5.0
2020-08-28 20:16:16+02	2020-08-28 20:16:28+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-08-28 20:16:29+02	2020-08-28 20:16:46+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-08-28 20:15:49+02	2020-08-28 20:17:06+02	devel	libarea	\N	libarea-20160313_13
2020-08-28 20:16:47+02	2020-08-28 20:17:15+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-08-28 20:17:06+02	2020-08-28 20:17:20+02	devel	py-funcy	py37	py37-funcy-1.12
2020-08-28 20:15:59+02	2020-08-28 20:17:29+02	devel	ode	\N	ode-0.13_4,1
2020-08-28 20:17:21+02	2020-08-28 20:17:31+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-08-28 20:17:30+02	2020-08-28 20:17:39+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-08-28 20:17:31+02	2020-08-28 20:17:45+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-08-28 20:17:40+02	2020-08-28 20:17:49+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-08-28 20:17:46+02	2020-08-28 20:17:56+02	devel	py-rq	py37	py37-rq-1.3.0
2020-08-28 20:17:56+02	2020-08-28 20:18:07+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-08-28 20:17:50+02	2020-08-28 20:18:14+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-08-28 20:18:07+02	2020-08-28 20:18:17+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-08-28 20:18:14+02	2020-08-28 20:18:23+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-08-28 20:17:16+02	2020-08-28 20:18:26+02	devel	py-game	py37	py37-game-1.9.6_1
2020-08-28 20:18:26+02	2020-08-28 20:18:37+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-08-28 20:18:18+02	2020-08-28 20:19:47+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-28 20:18:24+02	2020-08-28 20:23:20+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.4
2020-08-28 20:23:20+02	2020-08-28 20:26:05+02	editors	pluma	\N	pluma-1.24.0
2020-08-28 20:26:06+02	2020-08-28 20:29:15+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-08-28 20:29:16+02	2020-08-28 20:33:42+02	french	med	\N	fr-med-4.0.0
2020-08-28 20:33:42+02	2020-08-28 20:36:30+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-08-28 20:36:30+02	2020-08-28 20:38:15+02	graphics	atril	\N	atril-1.22.3_8
2020-08-28 20:38:15+02	2020-08-28 20:39:34+02	graphics	eom	\N	eom-1.24.0
2020-08-28 20:18:37+02	2020-08-28 20:40:15+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-28 20:40:15+02	2020-08-28 20:40:34+02	graphics	gle	\N	gle-3.1.0_7
2020-08-28 20:39:35+02	2020-08-28 20:40:46+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-28 20:40:35+02	2020-08-28 20:40:50+02	graphics	gltt	\N	gltt-2.5.2_14
2020-08-28 20:40:46+02	2020-08-28 20:41:33+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-28 20:40:51+02	2020-08-28 20:41:38+02	graphics	openimageio	\N	openimageio-2.1.15.0_5
2020-08-28 20:19:47+02	2020-08-28 20:41:47+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-08-28 20:41:39+02	2020-08-28 20:42:46+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-08-28 20:42:47+02	2020-08-28 20:43:13+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-08-28 20:43:14+02	2020-08-28 20:43:55+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-08-28 20:41:48+02	2020-08-28 20:43:56+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-28 20:41:33+02	2020-08-28 20:44:39+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-08-28 20:43:56+02	2020-08-28 21:03:40+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-28 21:03:41+02	2020-08-28 21:03:49+02	lang	python3	\N	python3-3_3
2020-08-28 20:44:39+02	2020-08-28 21:04:43+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-28 21:03:50+02	2020-08-28 21:05:17+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-28 21:04:44+02	2020-08-28 21:05:36+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-28 20:43:57+02	2020-08-28 21:06:14+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-08-28 21:05:17+02	2020-08-28 21:07:49+02	math	stp	\N	stp-2.3.3_6,1
2020-08-28 21:07:49+02	2020-08-28 21:08:04+02	misc	xosd	\N	xosd-2.2.14_3
2020-08-28 21:08:04+02	2020-08-28 21:09:33+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-28 21:09:33+02	2020-08-28 21:10:26+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-08-28 21:05:37+02	2020-08-28 21:10:44+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-08-28 21:10:27+02	2020-08-28 21:12:35+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-28 21:12:36+02	2020-08-28 21:13:46+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-28 21:13:47+02	2020-08-28 21:14:01+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-08-28 21:14:02+02	2020-08-28 21:14:59+02	net	py-pyzmq	py37	py37-pyzmq-19.0.2
2020-08-28 21:15:00+02	2020-08-28 21:15:56+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-28 21:10:45+02	2020-08-28 21:16:48+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-28 21:15:57+02	2020-08-28 21:16:51+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-28 21:16:48+02	2020-08-28 21:17:32+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-28 21:16:51+02	2020-08-28 21:17:42+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-28 21:17:32+02	2020-08-28 21:18:14+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-28 21:17:43+02	2020-08-28 21:18:20+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-28 21:18:14+02	2020-08-28 21:19:02+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-28 21:18:21+02	2020-08-28 21:19:25+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-28 21:06:15+02	2020-08-28 21:19:29+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-08-28 21:19:03+02	2020-08-28 21:19:34+02	net	vde2	\N	vde2-2.3.2_4
2020-08-28 21:19:26+02	2020-08-28 21:20:07+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-28 21:20:08+02	2020-08-28 21:20:19+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-08-28 21:19:35+02	2020-08-28 21:20:21+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_8
2020-08-28 21:19:30+02	2020-08-28 21:20:33+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-28 21:20:20+02	2020-08-28 21:21:14+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-28 21:20:22+02	2020-08-28 21:25:05+02	print	lilypond	\N	lilypond-2.20.0
2020-08-28 21:20:33+02	2020-08-28 21:25:48+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-28 21:21:14+02	2020-08-28 21:26:11+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-28 21:25:06+02	2020-08-28 21:26:37+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-28 21:26:37+02	2020-08-28 21:27:05+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-28 21:27:05+02	2020-08-28 21:27:16+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-08-28 21:27:16+02	2020-08-28 21:27:28+02	security	py-stem	py37	py37-stem-1.8.0
2020-08-28 21:27:28+02	2020-08-28 21:28:07+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-28 21:28:08+02	2020-08-28 21:29:24+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-28 21:29:25+02	2020-08-28 21:30:49+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-28 21:25:49+02	2020-08-28 21:31:28+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-08-28 21:30:49+02	2020-08-28 21:32:11+02	textproc	fop	\N	fop-2.3
2020-08-28 21:31:29+02	2020-08-28 21:32:17+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-28 21:32:18+02	2020-08-28 21:32:45+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-08-28 21:32:12+02	2020-08-28 21:32:46+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-28 21:32:46+02	2020-08-28 21:32:55+02	textproc	py-scour	py37	py37-scour-0.38
2020-08-28 21:32:47+02	2020-08-28 21:32:56+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-08-28 21:32:57+02	2020-08-28 21:33:17+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-08-28 21:32:56+02	2020-08-28 21:33:17+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-08-28 21:33:17+02	2020-08-28 21:33:28+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-08-28 21:33:18+02	2020-08-28 21:33:30+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-08-28 21:33:28+02	2020-08-28 21:33:38+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-08-28 21:33:31+02	2020-08-28 21:33:41+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-08-28 21:33:38+02	2020-08-28 21:33:50+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-08-28 21:33:41+02	2020-08-28 21:33:52+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-08-28 21:33:50+02	2020-08-28 21:34:42+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-28 21:33:52+02	2020-08-28 21:34:49+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-28 21:34:49+02	2020-08-28 21:35:46+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-28 21:35:47+02	2020-08-28 21:36:55+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-28 21:36:55+02	2020-08-28 21:38:28+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-28 21:34:43+02	2020-08-28 21:39:00+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-28 21:38:29+02	2020-08-28 21:39:37+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-28 21:39:01+02	2020-08-28 21:40:15+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-08-28 21:39:38+02	2020-08-28 21:40:34+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-28 21:40:15+02	2020-08-28 21:40:48+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-28 21:40:34+02	2020-08-28 21:41:34+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-28 21:41:34+02	2020-08-28 21:41:43+02	astro	xearth	\N	xearth-1.2
2020-08-28 21:26:12+02	2020-08-28 21:46:00+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-28 21:41:43+02	2020-08-28 21:46:53+02	astro	xephem	\N	xephem-3.7.7_5
2020-08-28 21:46:55+02	2020-08-28 21:47:37+02	audio	xmix	\N	xmix-2.1_3
2020-08-28 21:46:04+02	2020-08-28 21:55:54+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-28 21:40:48+02	2020-08-28 22:01:11+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-28 22:01:14+02	2020-08-28 22:08:19+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-28 21:55:57+02	2020-08-28 22:08:21+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-28 22:08:23+02	2020-08-28 22:12:55+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-28 22:12:57+02	2020-08-28 22:17:25+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-28 22:17:28+02	2020-08-28 22:18:12+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-08-28 22:18:13+02	2020-08-28 22:30:07+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-28 22:30:12+02	2020-08-28 22:31:15+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.5
2020-08-28 22:31:16+02	2020-08-28 22:38:19+02	deskutils	calibre	\N	calibre-4.23.0
2020-08-28 22:38:23+02	2020-08-28 22:40:51+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-28 22:40:53+02	2020-08-28 22:42:09+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-08-28 22:42:10+02	2020-08-28 22:42:56+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-08-28 21:47:41+02	2020-08-28 23:11:02+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-28 23:11:06+02	2020-08-28 23:41:17+02	devel	codeblocks	\N	codeblocks-20.03
2020-08-28 22:43:02+02	2020-08-28 23:42:04+02	devel	RStudio	\N	RStudio-1.3.1073
2020-08-28 23:41:18+02	2020-08-28 23:42:16+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-28 23:42:05+02	2020-08-29 00:13:52+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-28 22:08:23+02	2020-08-29 00:29:41+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-28 23:42:17+02	2020-08-29 00:30:40+02	devel	ghidra	\N	ghidra-9.1
2020-08-29 00:30:41+02	2020-08-29 00:30:57+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-08-29 00:31:27+02	2020-08-29 00:36:04+02	dns	bind916	\N	bind916-9.16.6_1
2020-08-29 00:36:04+02	2020-08-29 00:50:30+02	editors	codelite	\N	codelite-12.0_4
2020-08-29 00:50:31+02	2020-08-29 00:53:23+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-08-29 00:53:23+02	2020-08-29 00:56:06+02	editors	vim	\N	vim-8.2.1382
2020-08-29 00:56:07+02	2020-08-29 01:13:56+02	emulators	qemu	\N	qemu-4.2.1
2020-08-29 01:13:58+02	2020-08-29 02:33:22+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-08-29 02:33:23+02	2020-08-29 02:44:49+02	emulators	wine	\N	wine-5.0.2,1
2020-08-29 02:44:50+02	2020-08-29 02:45:42+02	emulators	x48	\N	x48-0.6.4_1
2020-08-29 02:45:43+02	2020-08-29 03:03:58+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-29 03:03:59+02	2020-08-29 03:04:15+02	games	KnightCap	\N	knightcap-3.6_8
2020-08-29 03:04:17+02	2020-08-29 03:04:31+02	games	affenspiel	\N	affenspiel-1.0_2
2020-08-29 03:04:32+02	2020-08-29 03:04:54+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-08-29 03:04:55+02	2020-08-29 03:08:21+02	games	alienarena	\N	alienarena-2013.766_9
2020-08-29 03:08:21+02	2020-08-29 03:09:23+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-08-29 03:09:24+02	2020-08-29 03:13:21+02	games	battletanks	\N	btanks-0.9.8083_9
2020-08-29 03:13:21+02	2020-08-29 03:13:48+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-08-29 03:13:48+02	2020-08-29 03:14:40+02	games	monsterz	\N	monsterz-0.7.1_15
2020-08-29 03:14:42+02	2020-08-29 03:15:25+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-29 03:15:25+02	2020-08-29 03:16:18+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-29 03:16:19+02	2020-08-29 03:17:43+02	games	xblast	\N	xblast-2.10.4_17
2020-08-29 03:17:44+02	2020-08-29 03:19:11+02	games	xboard	\N	xboard-4.9.1_1
2020-08-29 03:19:12+02	2020-08-29 03:20:23+02	games	xcowsay	\N	xcowsay-1.5
2020-08-29 03:20:23+02	2020-08-29 03:29:57+02	graphics	blender	\N	blender-2.83.2_1
2020-08-29 03:29:59+02	2020-08-29 03:33:34+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-29 03:33:35+02	2020-08-29 03:35:22+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-29 03:35:22+02	2020-08-29 03:41:57+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-29 03:41:58+02	2020-08-29 03:58:32+02	graphics	krita	\N	krita-4.3.0_4
2020-08-29 03:58:33+02	2020-08-29 03:58:48+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-08-29 03:58:48+02	2020-08-29 04:03:27+02	graphics	wings	\N	wings-2.2.5
2020-08-29 04:03:28+02	2020-08-29 04:04:52+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-29 04:04:53+02	2020-08-29 04:06:38+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-08-29 04:06:38+02	2020-08-29 04:47:57+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-08-29 04:48:00+02	2020-08-29 04:49:03+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-29 00:29:42+02	2020-08-29 04:55:33+02	devel	llvm10	\N	llvm10-10.0.1_1
2020-08-29 04:49:04+02	2020-08-29 05:14:10+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.9
2020-08-29 04:55:36+02	2020-08-29 05:29:14+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-29 05:29:16+02	2020-08-29 05:29:26+02	lang	python	\N	python-3.7_3,2
2020-08-29 05:29:27+02	2020-08-29 05:29:51+02	mail	bbmail	\N	bbmail-0.9.3
2020-08-29 05:29:52+02	2020-08-29 05:36:47+02	math	maxima	\N	maxima-5.44.0_3
2020-08-29 05:36:47+02	2020-08-29 05:38:56+02	misc	mc	\N	mc-nox11-4.8.24
2020-08-29 05:38:57+02	2020-08-29 05:39:59+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-29 05:40:01+02	2020-08-29 05:40:27+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-08-29 05:40:28+02	2020-08-29 05:41:15+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-08-29 05:14:10+02	2020-08-29 05:53:14+02	lang	mono	\N	mono-5.10.1.57_3
2020-08-29 05:53:15+02	2020-08-29 05:53:41+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-08-29 05:53:42+02	2020-08-29 05:53:59+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-08-29 05:54:01+02	2020-08-29 05:56:44+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-29 05:56:45+02	2020-08-29 05:59:55+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-29 05:59:55+02	2020-08-29 06:03:44+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-29 06:03:45+02	2020-08-29 06:06:13+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-29 06:06:14+02	2020-08-29 06:06:50+02	net	rdesktop	\N	rdesktop-1.8.6
2020-08-29 05:41:16+02	2020-08-29 06:07:46+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p6
2020-08-29 06:06:51+02	2020-08-29 06:08:17+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-29 06:08:18+02	2020-08-29 06:09:06+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-29 06:09:06+02	2020-08-29 06:11:06+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-08-29 06:11:06+02	2020-08-29 06:11:59+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-29 06:07:46+02	2020-08-29 06:12:41+02	net	wireshark	\N	wireshark-3.2.6
2020-08-29 06:12:00+02	2020-08-29 06:12:50+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-29 06:12:47+02	2020-08-29 06:13:35+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-29 06:12:51+02	2020-08-29 06:13:39+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-29 06:13:36+02	2020-08-29 06:14:24+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-29 06:13:39+02	2020-08-29 06:14:27+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-29 06:14:24+02	2020-08-29 06:15:13+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-29 06:14:27+02	2020-08-29 06:15:44+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-29 06:15:13+02	2020-08-29 06:16:07+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-29 06:15:45+02	2020-08-29 06:16:35+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-29 06:16:08+02	2020-08-29 06:17:02+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-29 06:16:35+02	2020-08-29 06:17:29+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-29 06:17:30+02	2020-08-29 06:18:04+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-29 06:18:05+02	2020-08-29 06:18:45+02	print	gv	\N	gv-3.7.4_5
2020-08-29 06:17:03+02	2020-08-29 06:19:45+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-29 06:18:48+02	2020-08-29 06:21:52+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-29 06:21:53+02	2020-08-29 06:27:58+02	print	texlive-full	\N	texlive-full-20150521
2020-08-29 06:19:46+02	2020-08-29 06:28:39+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-08-29 06:28:40+02	2020-08-29 06:28:54+02	security	nyx	\N	nyx-2.1.0_1
2020-08-29 06:28:54+02	2020-08-29 06:29:54+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-29 06:28:03+02	2020-08-29 06:30:18+02	security	klee	\N	klee-2.1_1
2020-08-29 06:30:19+02	2020-08-29 06:30:37+02	security	py-xkcdpass	py37	py37-xkcdpass-1.17.4
2020-08-29 06:29:56+02	2020-08-29 06:30:47+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-08-29 06:30:47+02	2020-08-29 06:30:59+02	sysutils	bashtop	\N	bashtop-0.9.25_1
2020-08-29 06:30:59+02	2020-08-29 06:31:10+02	sysutils	bpytop	\N	bpytop-1.0.21
2020-08-29 06:31:10+02	2020-08-29 06:32:04+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-29 06:32:05+02	2020-08-29 06:32:28+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-08-29 06:30:38+02	2020-08-29 06:32:54+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-08-29 06:32:29+02	2020-08-29 06:33:08+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-29 06:32:55+02	2020-08-29 06:33:56+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-29 06:33:57+02	2020-08-29 06:34:16+02	sysutils	panicmail	\N	panicmail-1.1
2020-08-29 06:33:08+02	2020-08-29 06:34:33+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-29 06:34:17+02	2020-08-29 06:34:37+02	sysutils	py-glances	py37	py37-glances-3.1.4
2020-08-29 06:34:34+02	2020-08-29 06:34:45+02	sysutils	py-py-cpuinfo	py37	py37-py-cpuinfo-7.0.0
2020-08-29 06:34:46+02	2020-08-29 06:35:05+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-08-29 06:34:38+02	2020-08-29 06:35:55+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-08-29 06:35:55+02	2020-08-29 06:37:07+02	www	links	\N	links-2.20.2,1
2020-08-29 06:37:07+02	2020-08-29 06:37:22+02	www	py-bottle	py37	py37-bottle-0.12.15_1
2020-10-06 07:56:04+02	2020-10-06 08:00:00+02	devel	git	\N	git-2.28.0
2020-08-29 06:37:22+02	2020-08-29 06:37:38+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-08-29 06:37:38+02	2020-08-29 06:37:53+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-08-29 06:37:54+02	2020-08-29 06:38:11+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-08-29 06:38:11+02	2020-08-29 06:38:23+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-08-29 06:38:24+02	2020-08-29 06:38:38+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-08-29 06:38:39+02	2020-08-29 06:38:51+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-08-29 06:38:52+02	2020-08-29 06:39:07+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-08-29 06:39:08+02	2020-08-29 06:39:23+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-08-29 06:39:24+02	2020-08-29 06:39:39+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-08-29 06:39:39+02	2020-08-29 06:39:53+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-08-29 06:39:54+02	2020-08-29 06:40:15+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-08-29 06:40:16+02	2020-08-29 06:40:36+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-08-29 06:40:37+02	2020-08-29 06:41:56+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-29 06:41:57+02	2020-08-29 06:44:13+02	x11	mate	\N	mate-1.22.2
2020-08-29 06:44:14+02	2020-08-29 06:44:28+02	x11	numlockx	\N	numlockx-1.2
2020-08-29 06:44:28+02	2020-08-29 06:44:42+02	x11	sterm	\N	sterm-0.8.4
2020-08-29 06:44:43+02	2020-08-29 06:45:12+02	x11	xclip	\N	xclip-0.13
2020-08-29 06:45:12+02	2020-08-29 06:45:31+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-08-29 06:45:32+02	2020-08-29 06:46:05+02	x11	xdm	\N	xdm-1.1.12_3
2020-08-29 06:46:05+02	2020-08-29 06:47:00+02	x11	xlockmore	\N	xlockmore-5.65
2020-08-29 06:47:01+02	2020-08-29 06:47:21+02	x11	xman	\N	xman-1.1.5
2020-08-29 06:47:21+02	2020-08-29 06:50:42+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-29 06:50:43+02	2020-08-29 06:57:32+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-29 06:57:33+02	2020-08-29 06:59:04+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-29 06:59:05+02	2020-08-29 06:59:29+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-08-29 06:59:30+02	2020-08-29 06:59:44+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-08-29 06:59:45+02	2020-08-29 07:00:33+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-08-29 07:00:34+02	2020-08-29 07:00:56+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-08-29 07:00:56+02	2020-08-29 07:01:45+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-08-29 07:01:46+02	2020-08-29 07:02:12+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-08-29 07:02:13+02	2020-08-29 07:40:05+02	editors	vscode	\N	vscode-1.46.1
2020-08-29 07:40:07+02	2020-08-29 07:41:12+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-29 07:41:13+02	2020-08-29 07:41:59+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-08-29 06:35:06+02	2020-08-29 09:33:04+02	www	firefox	\N	firefox-80.0_3,1
2020-08-29 00:13:56+02	2020-08-29 09:35:31+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200811
2020-08-29 09:35:32+02	2020-08-29 09:52:27+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-08-29 09:52:42+02	2020-08-29 09:53:49+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-29 09:52:42+02	2020-08-29 09:58:00+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-29 09:58:01+02	2020-08-29 09:58:08+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-29 11:50:25+02	2020-08-29 11:55:39+02	lang	perl5.32	\N	perl5-5.32.0
2020-08-29 11:55:40+02	2020-08-29 11:55:54+02	converters	p5-Text-Unidecode	\N	p5-Text-Unidecode-1.30
2020-08-29 11:55:40+02	2020-08-29 11:55:54+02	devel	p5-Locale-gettext	\N	p5-Locale-gettext-1.07
2020-08-29 11:55:40+02	2020-08-29 11:55:58+02	devel	p5-Locale-libintl	\N	p5-Locale-libintl-1.31
2020-08-29 11:55:54+02	2020-08-29 11:56:07+02	textproc	p5-Unicode-EastAsianWidth	\N	p5-Unicode-EastAsianWidth-12.0
2020-08-29 11:55:54+02	2020-08-29 11:56:09+02	misc	help2man	\N	help2man-1.47.16
2020-08-29 11:56:08+02	2020-08-29 11:56:40+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-08-29 11:56:09+02	2020-08-29 11:57:27+02	print	texinfo	\N	texinfo-6.7_3,1
2020-08-29 11:57:27+02	2020-08-29 11:58:09+02	devel	autoconf	\N	autoconf-2.69_3
2020-08-29 11:58:09+02	2020-08-29 11:58:48+02	devel	automake	\N	automake-1.16.2
2020-08-29 11:56:41+02	2020-08-29 11:59:06+02	print	harfbuzz	\N	harfbuzz-2.7.2
2020-08-29 11:58:48+02	2020-08-29 11:59:12+02	textproc	p5-XML-Parser	\N	p5-XML-Parser-2.44
2020-08-29 11:59:13+02	2020-08-29 11:59:33+02	textproc	intltool	\N	intltool-0.51.0_1
2020-08-29 11:59:33+02	2020-08-29 12:01:11+02	www	w3m	\N	w3m-0.5.3.20200507
2020-08-29 12:01:12+02	2020-08-29 12:01:43+02	textproc	xmlto	\N	xmlto-0.0.28
2020-08-29 11:59:06+02	2020-08-29 12:01:48+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-29 12:01:49+02	2020-08-29 12:02:13+02	devel	autoconf213	\N	autoconf213-2.13.000227_7
2020-08-29 12:01:44+02	2020-08-29 12:06:20+02	textproc	groff	\N	groff-1.22.4_3
2020-08-29 12:02:14+02	2020-08-29 12:06:51+02	devel	qt5-buildtools	\N	qt5-buildtools-5.15.0
2020-08-29 12:06:55+02	2020-08-29 12:08:37+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-29 12:06:25+02	2020-08-29 12:11:06+02	graphics	graphviz	\N	graphviz-2.44.1_1
2020-08-29 12:11:07+02	2020-08-29 12:14:41+02	lang	vala	\N	vala-0.40.15,1
2020-08-29 12:14:41+02	2020-08-29 12:16:06+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-29 12:08:37+02	2020-08-29 12:20:48+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-08-29 12:16:07+02	2020-08-29 12:26:58+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-29 12:26:59+02	2020-08-29 12:27:44+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.2
2020-08-29 12:20:49+02	2020-08-29 12:27:53+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-08-29 12:27:45+02	2020-08-29 12:35:57+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-29 12:35:58+02	2020-08-29 12:36:42+02	print	texlive-tlmgr	\N	texlive-tlmgr-20150523_2
2020-08-29 12:36:42+02	2020-08-29 12:43:59+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-29 12:27:53+02	2020-08-29 12:46:40+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-08-29 12:46:41+02	2020-08-29 12:48:32+02	textproc	hunspell	\N	hunspell-1.7.0_2
2020-08-29 12:48:33+02	2020-08-29 12:49:51+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-08-29 12:49:52+02	2020-08-29 12:50:19+02	textproc	hyphen	\N	hyphen-2.8.8
2020-08-29 12:50:19+02	2020-08-29 12:50:56+02	textproc	texi2html	\N	texi2html-5.0_2,1
2020-08-29 12:50:56+02	2020-08-29 12:52:14+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-29 12:52:14+02	2020-08-29 12:53:59+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-29 12:53:59+02	2020-08-29 12:54:56+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-08-29 12:44:00+02	2020-08-29 13:10:47+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-29 13:10:49+02	2020-08-29 13:17:54+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-29 13:17:55+02	2020-08-29 13:26:22+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-29 13:26:23+02	2020-08-29 13:27:19+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-08-29 13:27:19+02	2020-08-29 13:28:01+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-08-29 13:28:02+02	2020-08-29 13:40:06+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-29 13:40:07+02	2020-08-29 13:42:25+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-08-29 13:42:26+02	2020-08-29 13:42:47+02	print	libraqm	\N	libraqm-0.6.0
2020-08-29 13:42:51+02	2020-08-29 13:43:12+02	net	p5-URI	\N	p5-URI-1.76
2020-08-29 13:43:13+02	2020-08-29 13:51:24+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-29 13:51:25+02	2020-08-29 13:52:46+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-08-29 13:52:46+02	2020-08-29 13:53:40+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-29 13:53:40+02	2020-08-29 13:55:03+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-08-29 13:55:04+02	2020-08-29 13:56:21+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-29 13:56:21+02	2020-08-29 13:56:52+02	net	p5-Socket6	\N	p5-Socket6-0.29
2020-08-29 13:56:53+02	2020-08-29 13:58:47+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-29 13:58:48+02	2020-08-29 13:59:58+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-29 13:59:59+02	2020-08-29 14:02:11+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-08-29 14:02:12+02	2020-08-29 14:02:37+02	net	p5-IO-Socket-INET6	\N	p5-IO-Socket-INET6-2.72_1
2020-08-29 14:02:38+02	2020-08-29 14:03:57+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-08-29 14:03:58+02	2020-08-29 14:04:21+02	security	p5-Digest-HMAC	\N	p5-Digest-HMAC-1.03_1
2020-08-29 14:04:21+02	2020-08-29 14:05:41+02	security	p5-Net-SSLeay	\N	p5-Net-SSLeay-1.88
2020-08-29 14:05:42+02	2020-08-29 14:06:41+02	www	p5-HTML-Tagset	\N	p5-HTML-Tagset-3.20_1
2020-08-29 14:06:43+02	2020-08-29 14:07:19+02	www	p5-Mozilla-CA	\N	p5-Mozilla-CA-20200520
2020-08-29 14:07:20+02	2020-08-29 14:09:18+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-08-29 12:54:57+02	2020-08-29 14:09:39+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-29 14:09:19+02	2020-08-29 14:13:28+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-29 14:13:29+02	2020-08-29 14:13:51+02	security	p5-IO-Socket-SSL	\N	p5-IO-Socket-SSL-2.068
2020-08-29 14:13:52+02	2020-08-29 14:14:17+02	www	p5-HTML-Parser	\N	p5-HTML-Parser-3.72
2020-08-29 14:14:18+02	2020-08-29 14:15:29+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-08-29 14:15:30+02	2020-08-29 14:17:11+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-29 14:17:11+02	2020-08-29 14:18:33+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-08-29 14:18:33+02	2020-08-29 14:22:40+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-29 11:55:58+02	2020-08-29 14:24:03+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-08-29 14:09:40+02	2020-08-29 14:26:24+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-29 14:26:25+02	2020-08-29 14:26:48+02	textproc	p5-XML-NamespaceSupport	\N	p5-XML-NamespaceSupport-1.12
2020-08-29 14:26:48+02	2020-08-29 14:27:12+02	textproc	p5-XML-SAX-Base	\N	p5-XML-SAX-Base-1.09
2020-08-29 14:27:13+02	2020-08-29 14:29:06+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-08-29 14:29:07+02	2020-08-29 14:30:13+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-08-29 14:22:41+02	2020-08-29 14:30:29+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-08-29 14:30:29+02	2020-08-29 14:30:54+02	security	p5-GSSAPI	\N	p5-GSSAPI-0.28_1
2020-08-29 14:30:55+02	2020-08-29 14:31:19+02	textproc	p5-XML-SAX	\N	p5-XML-SAX-1.02
2020-08-29 14:24:04+02	2020-08-29 14:34:08+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-08-29 14:30:14+02	2020-08-29 14:34:07+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-08-29 14:31:20+02	2020-08-29 14:34:11+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-08-29 14:34:10+02	2020-08-29 14:34:56+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-08-29 14:34:21+02	2020-08-29 14:35:04+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-08-29 14:35:04+02	2020-08-29 14:35:20+02	lang	p5-Error	\N	p5-Error-0.17029
2020-08-29 14:34:56+02	2020-08-29 14:35:55+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-08-29 14:35:21+02	2020-08-29 14:36:29+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-29 14:35:56+02	2020-08-29 14:36:51+02	security	munge	\N	munge-0.5.14_1
2020-08-29 14:36:31+02	2020-08-29 14:36:52+02	security	p5-Authen-SASL	\N	p5-Authen-SASL-2.16_1
2020-08-29 14:34:10+02	2020-08-29 14:36:57+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_4,1
2020-08-29 14:36:53+02	2020-08-29 14:37:05+02	textproc	man2html	\N	man2html-3.0.1_2
2020-08-29 14:36:58+02	2020-08-29 14:37:11+02	textproc	p5-XML-SAX-Expat	\N	p5-XML-SAX-Expat-0.51_1
2020-08-29 14:37:05+02	2020-08-29 14:37:19+02	www	p5-CGI	\N	p5-CGI-4.50
2020-08-29 14:36:52+02	2020-08-29 14:37:39+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-08-29 14:37:12+02	2020-08-29 14:39:36+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-08-29 14:37:20+02	2020-08-29 14:40:14+02	databases	mysql57-client	\N	mysql57-client-5.7.31
2020-08-29 14:39:37+02	2020-08-29 14:40:45+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-08-29 14:37:39+02	2020-08-29 14:40:48+02	devel	git	\N	git-2.28.0
2020-08-29 14:40:15+02	2020-08-29 14:41:43+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-08-29 14:40:49+02	2020-08-29 14:42:00+02	net	remmina	\N	remmina-1.3.9
2020-08-29 14:40:46+02	2020-08-29 14:42:20+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-29 14:42:20+02	2020-08-29 14:44:41+02	textproc	aspell	\N	aspell-0.60.8,1
2020-08-29 14:41:43+02	2020-08-29 14:44:49+02	security	gcr	\N	gcr-3.28.0
2020-08-29 14:44:50+02	2020-08-29 14:45:05+02	textproc	p5-XML-Simple	\N	p5-XML-Simple-2.25
2020-08-29 14:44:41+02	2020-08-29 14:45:33+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-08-29 14:45:06+02	2020-08-29 14:46:02+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-08-29 14:46:03+02	2020-08-29 14:46:47+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-29 14:42:01+02	2020-08-29 14:49:01+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-08-29 14:46:47+02	2020-08-29 14:52:16+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-08-29 14:52:16+02	2020-08-29 14:52:40+02	x11-themes	icon-naming-utils	\N	icon-naming-utils-0.8.90_1
2020-08-29 14:45:33+02	2020-08-29 14:53:22+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-29 14:52:40+02	2020-08-29 14:53:26+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-29 14:53:26+02	2020-08-29 14:54:29+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-08-29 14:53:22+02	2020-08-29 14:54:33+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-08-29 14:54:30+02	2020-08-29 14:55:48+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-29 14:55:49+02	2020-08-29 14:56:41+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-08-29 14:56:41+02	2020-08-29 14:57:08+02	devel	imake	\N	imake-1.0.8,1
2020-08-29 14:57:08+02	2020-08-29 14:57:34+02	devel	p5-TimeDate	\N	p5-TimeDate-2.33,1
2020-08-29 14:57:34+02	2020-08-29 15:00:28+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-29 15:00:28+02	2020-08-29 15:01:10+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-29 15:01:11+02	2020-08-29 15:02:57+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-29 15:02:58+02	2020-08-29 15:04:05+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-08-29 14:49:02+02	2020-08-29 15:04:35+02	net	openmpi	\N	openmpi-4.0.4
2020-08-29 15:04:06+02	2020-08-29 15:05:48+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-29 14:54:34+02	2020-08-29 15:05:53+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-29 15:05:49+02	2020-08-29 15:06:50+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-29 15:06:50+02	2020-08-29 15:09:21+02	graphics	netpbm	\N	netpbm-10.91.01
2020-08-29 15:09:21+02	2020-08-29 15:14:34+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-29 15:14:35+02	2020-08-29 15:16:27+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-08-29 15:16:28+02	2020-08-29 15:23:30+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-08-29 15:23:31+02	2020-08-29 15:23:52+02	www	p5-HTTP-Date	\N	p5-HTTP-Date-6.05
2020-08-29 15:04:36+02	2020-08-29 16:06:20+02	math	vtk8	\N	vtk8-8.2.0
2020-08-29 15:23:53+02	2020-08-29 17:47:05+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-29 17:47:36+02	2020-08-29 17:49:39+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-29 17:49:40+02	2020-08-29 17:52:14+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-08-29 16:06:36+02	2020-08-29 17:52:40+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-29 17:52:42+02	2020-08-29 17:53:30+02	converters	p5-Encode-Locale	\N	p5-Encode-Locale-1.05
2020-08-29 17:53:31+02	2020-08-29 17:54:53+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-08-29 17:54:54+02	2020-08-29 17:56:11+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-08-29 17:56:12+02	2020-08-29 17:57:09+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-29 17:57:10+02	2020-08-29 17:57:32+02	devel	netsurf-buildsystem	\N	netsurf-buildsystem-1.8
2020-08-29 17:57:33+02	2020-08-29 17:57:58+02	devel	p5-Clone	\N	p5-Clone-0.45
2020-08-29 17:57:59+02	2020-08-29 17:58:22+02	devel	p5-IO-HTML	\N	p5-IO-HTML-1.001_1
2020-08-29 17:52:15+02	2020-08-29 17:59:07+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-29 17:59:08+02	2020-08-29 17:59:28+02	www	p5-LWP-MediaTypes	\N	p5-LWP-MediaTypes-6.04
2020-10-06 08:09:34+02	2020-10-06 08:12:49+02	x11	xorg	\N	xorg-7.7_3
2020-08-29 17:58:22+02	2020-08-29 18:00:19+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-08-29 18:00:19+02	2020-08-29 18:01:47+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-08-29 18:01:47+02	2020-08-29 18:07:37+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-29 18:07:38+02	2020-08-29 18:09:08+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-29 18:09:09+02	2020-08-29 18:10:27+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-29 18:10:27+02	2020-08-29 18:11:46+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-29 18:11:46+02	2020-08-29 18:13:08+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-29 18:13:09+02	2020-08-29 18:14:31+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-29 18:14:31+02	2020-08-29 18:16:50+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-08-29 18:16:51+02	2020-08-29 18:18:07+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-29 18:18:08+02	2020-08-29 18:19:22+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-29 18:19:23+02	2020-08-29 18:21:05+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-08-29 18:21:06+02	2020-08-29 18:22:36+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-08-29 18:22:37+02	2020-08-29 18:23:12+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-29 18:23:13+02	2020-08-29 18:36:55+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-29 18:36:57+02	2020-08-29 19:12:49+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-29 19:12:50+02	2020-08-29 19:14:02+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-29 19:14:06+02	2020-08-29 19:14:22+02	devel	p5-Parse-Yapp	\N	p5-Parse-Yapp-1.21
2020-08-29 19:14:23+02	2020-08-29 19:20:29+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-08-29 19:20:29+02	2020-08-29 19:21:07+02	print	enscript-a4	\N	enscript-a4-1.6.6_3
2020-08-29 19:21:07+02	2020-08-29 19:21:52+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-29 19:21:53+02	2020-08-29 19:22:38+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-08-29 19:22:39+02	2020-08-29 19:22:56+02	www	p5-HTTP-Message	\N	p5-HTTP-Message-6.25
2020-08-29 19:22:56+02	2020-08-29 19:24:04+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-29 15:05:54+02	2020-08-29 19:24:05+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-08-29 19:24:04+02	2020-08-29 19:25:03+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-29 19:24:06+02	2020-08-29 19:26:49+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-29 19:25:04+02	2020-08-29 19:27:09+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_4,1
2020-08-29 19:27:10+02	2020-08-29 19:28:41+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-29 19:28:42+02	2020-08-29 19:29:49+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-29 19:26:49+02	2020-08-29 19:29:58+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-29 19:29:49+02	2020-08-29 19:31:05+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-29 19:29:59+02	2020-08-29 19:31:09+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-29 19:31:06+02	2020-08-29 19:32:24+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-29 19:31:10+02	2020-08-29 19:32:30+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-08-29 19:32:25+02	2020-08-29 19:32:43+02	devel	p5-IO-String	\N	p5-IO-String-1.08_1
2020-08-29 19:32:43+02	2020-08-29 19:33:12+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-08-29 19:32:31+02	2020-08-29 19:34:11+02	graphics	gegl	\N	gegl-0.4.24_4
2020-08-29 19:34:11+02	2020-08-29 19:36:10+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_2
2020-08-29 19:36:11+02	2020-08-29 19:37:11+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-29 19:37:12+02	2020-08-29 19:38:04+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-29 19:33:12+02	2020-08-29 19:39:13+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-29 19:38:04+02	2020-08-29 19:39:50+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_2
2020-08-29 19:39:51+02	2020-08-29 19:40:54+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-29 19:39:14+02	2020-08-29 19:40:58+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-29 19:40:55+02	2020-08-29 19:41:53+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-08-29 19:41:54+02	2020-08-29 19:43:02+02	x11	zenity	\N	zenity-3.32.0
2020-08-29 19:43:03+02	2020-08-29 19:43:53+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-08-29 19:40:59+02	2020-08-29 19:44:00+02	x11	xorg	\N	xorg-7.7_3
2020-08-29 19:44:01+02	2020-08-29 19:46:03+02	devel	libgdata	\N	libgdata-0.17.9
2020-08-29 19:46:04+02	2020-08-29 19:47:38+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-29 19:43:54+02	2020-08-29 19:48:55+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-29 19:47:39+02	2020-08-29 19:49:04+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-08-29 19:48:56+02	2020-08-29 19:50:06+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-08-29 19:49:04+02	2020-08-29 19:50:19+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-08-29 19:50:07+02	2020-08-29 19:53:30+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-08-29 19:50:20+02	2020-08-29 19:55:26+02	editors	emacs	full	emacs-27.1,3
2020-08-29 19:53:30+02	2020-08-29 20:01:56+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-08-29 19:55:27+02	2020-08-29 20:02:19+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-29 20:01:57+02	2020-08-29 20:02:53+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-29 20:02:20+02	2020-08-29 20:03:11+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-08-29 20:02:54+02	2020-08-29 20:06:30+02	print	fontforge	\N	fontforge-20190801_2
2020-08-29 20:06:31+02	2020-08-29 20:07:26+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-29 20:07:26+02	2020-08-29 20:09:07+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-08-29 20:09:07+02	2020-08-29 20:09:30+02	textproc	mythes	\N	mythes-1.2.4_7
2020-08-29 20:03:12+02	2020-08-29 20:11:44+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-08-29 20:09:30+02	2020-08-29 20:13:20+02	www	apache24	\N	apache24-2.4.46
2020-08-29 20:13:20+02	2020-08-29 20:14:22+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-29 20:14:22+02	2020-08-29 20:15:37+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-29 20:15:38+02	2020-08-29 20:15:56+02	x11-fonts	p5-Font-TTF	\N	p5-Font-TTF-1.06
2020-08-29 20:15:57+02	2020-08-29 20:17:05+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-29 20:17:06+02	2020-08-29 20:18:06+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-29 20:18:07+02	2020-08-29 20:19:43+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-29 20:19:44+02	2020-08-29 20:21:40+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-29 20:21:40+02	2020-08-29 20:23:10+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-29 20:23:10+02	2020-08-29 20:24:12+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-08-29 20:24:13+02	2020-08-29 20:25:38+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-08-29 20:25:38+02	2020-08-29 20:27:43+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-08-29 20:27:44+02	2020-08-29 20:31:08+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-29 20:31:08+02	2020-08-29 20:32:34+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-29 20:32:34+02	2020-08-29 20:34:17+02	x11-wm	marco	\N	marco-1.22.4
2020-08-29 20:34:18+02	2020-08-29 20:35:29+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-29 20:35:29+02	2020-08-29 20:35:51+02	archivers	p5-Archive-Zip	\N	p5-Archive-Zip-1.68
2020-08-29 20:35:51+02	2020-08-29 20:36:26+02	databases	p5-DBI	\N	p5-DBI-1.643
2020-08-29 20:36:29+02	2020-08-29 20:39:02+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-29 20:39:02+02	2020-08-29 20:40:24+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-08-29 20:40:24+02	2020-08-29 20:43:26+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-29 20:43:27+02	2020-08-29 20:46:09+02	devel	gvfs	\N	gvfs-1.30.4_3
2020-08-29 20:46:10+02	2020-08-29 20:46:33+02	devel	p5-Getopt-ArgvFile	\N	p5-Getopt-ArgvFile-1.11_1
2020-08-29 20:46:34+02	2020-08-29 20:51:06+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-29 20:51:07+02	2020-08-29 20:52:00+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-29 20:52:01+02	2020-08-29 20:54:12+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-29 20:54:15+02	2020-08-29 20:55:59+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-08-29 20:56:01+02	2020-08-29 20:56:36+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-29 20:56:37+02	2020-08-29 20:57:29+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-29 20:57:30+02	2020-08-29 20:58:21+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-08-29 20:58:22+02	2020-08-29 20:58:41+02	lang	p5-Try-Tiny	\N	p5-Try-Tiny-0.30
2020-08-29 20:11:45+02	2020-08-29 21:01:10+02	www	chromium	\N	chromium-84.0.4147.135
2020-08-29 21:01:13+02	2020-08-29 21:08:54+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-29 21:08:55+02	2020-08-29 21:09:37+02	multimedia	libass	\N	libass-0.14.0
2020-08-29 21:09:37+02	2020-08-29 21:10:55+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-29 21:10:55+02	2020-08-29 21:11:11+02	net	p5-Net-HTTP	\N	p5-Net-HTTP-6.19
2020-08-29 21:11:12+02	2020-08-29 21:11:58+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-29 21:11:59+02	2020-08-29 21:13:18+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-29 21:13:19+02	2020-08-29 21:14:09+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-29 20:58:42+02	2020-08-29 21:14:59+02	math	R	\N	R-4.0.2
2020-08-29 21:15:00+02	2020-08-29 21:17:13+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-29 21:14:10+02	2020-08-29 21:17:44+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-29 21:17:14+02	2020-08-29 21:23:17+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-29 21:23:18+02	2020-08-29 21:23:33+02	security	p5-Authen-NTLM	\N	p5-Authen-NTLM-1.09_1
2020-08-29 21:17:45+02	2020-08-29 21:23:33+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-29 21:23:34+02	2020-08-29 21:24:25+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-29 21:24:25+02	2020-08-29 21:24:43+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.04_1
2020-08-29 21:23:34+02	2020-08-29 21:26:27+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-29 21:24:44+02	2020-08-29 21:26:51+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-29 21:26:52+02	2020-08-29 21:27:20+02	textproc	enchant	\N	enchant-1.6.0_9
2020-08-29 21:27:20+02	2020-08-29 21:27:53+02	textproc	p5-PDF-API2	\N	p5-PDF-API2-2.037
2020-08-29 21:27:53+02	2020-08-29 21:28:10+02	textproc	p5-Pod-Escapes	\N	p5-Pod-Escapes-1.07
2020-08-29 21:26:28+02	2020-08-29 21:28:25+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-29 21:28:10+02	2020-08-29 21:28:27+02	www	p5-HTTP-Cookies	\N	p5-HTTP-Cookies-6.08
2020-08-29 21:28:26+02	2020-08-29 21:28:42+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.12
2020-08-29 21:28:27+02	2020-08-29 21:28:43+02	www	p5-HTTP-Negotiate	\N	p5-HTTP-Negotiate-6.01_1
2020-08-29 21:28:43+02	2020-08-29 21:28:59+02	www	p5-WWW-RobotRules	\N	p5-WWW-RobotRules-6.02_1
2020-08-29 21:28:44+02	2020-08-29 21:29:23+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-29 21:29:00+02	2020-08-29 21:30:02+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-29 21:29:24+02	2020-08-29 21:30:39+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-29 21:30:40+02	2020-08-29 21:31:52+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-29 21:30:02+02	2020-08-29 21:31:54+02	x11-servers	xephyr	\N	xephyr-1.20.8_4,1
2020-08-29 21:31:52+02	2020-08-29 21:32:42+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-29 21:31:55+02	2020-08-29 21:32:54+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-29 21:32:42+02	2020-08-29 21:33:41+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-08-29 21:32:55+02	2020-08-29 21:34:05+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-29 21:34:05+02	2020-08-29 21:36:21+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-29 21:36:22+02	2020-08-29 21:37:35+02	audio	mate-media	\N	mate-media-1.24.1
2020-08-29 21:37:36+02	2020-08-29 21:38:11+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.14.2
2020-08-29 21:38:11+02	2020-08-29 21:39:15+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-29 21:33:42+02	2020-08-29 21:39:19+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-29 21:39:15+02	2020-08-29 21:40:26+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-29 21:40:27+02	2020-08-29 21:41:26+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-29 21:39:20+02	2020-08-29 21:42:01+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-29 21:41:26+02	2020-08-29 21:42:09+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-29 21:42:02+02	2020-08-29 21:42:57+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-08-29 21:42:10+02	2020-08-29 21:43:07+02	devel	libstatgrab	\N	libstatgrab-0.92
2020-08-29 21:42:58+02	2020-08-29 21:43:17+02	devel	p5-Term-ReadKey	\N	p5-Term-ReadKey-2.38_1
2020-08-29 17:59:29+02	2020-08-29 21:44:53+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-29 21:43:07+02	2020-08-29 22:05:09+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-29 21:43:18+02	2020-08-29 22:06:08+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-08-29 22:06:09+02	2020-08-29 22:08:46+02	graphics	atril	\N	atril-1.22.3_8
2020-08-29 22:05:10+02	2020-08-29 22:09:04+02	editors	pluma	\N	pluma-1.24.0
2020-08-29 22:08:46+02	2020-08-29 22:10:26+02	graphics	eom	\N	eom-1.24.0
2020-08-29 22:09:04+02	2020-08-29 22:10:34+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-29 22:10:26+02	2020-08-29 22:11:17+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-29 22:10:35+02	2020-08-29 22:12:05+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-08-29 22:11:18+02	2020-08-29 22:13:47+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-08-29 22:13:47+02	2020-08-29 22:20:35+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-29 22:12:06+02	2020-08-29 22:20:42+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-29 22:20:43+02	2020-08-29 22:22:46+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-29 21:44:54+02	2020-08-29 22:23:22+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-29 22:22:47+02	2020-08-29 22:23:46+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-29 22:23:23+02	2020-08-29 22:24:57+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-29 22:23:47+02	2020-08-29 22:25:17+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-29 22:24:58+02	2020-08-29 22:26:57+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-29 22:26:58+02	2020-08-29 22:28:03+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-29 22:28:03+02	2020-08-29 22:28:56+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-29 22:28:57+02	2020-08-29 22:29:51+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-29 22:29:52+02	2020-08-29 22:30:51+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-29 22:25:17+02	2020-08-29 22:31:02+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-29 22:30:52+02	2020-08-29 22:32:22+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-29 22:31:03+02	2020-08-29 22:32:24+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-29 22:32:24+02	2020-08-29 22:33:30+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-29 22:32:26+02	2020-08-29 22:34:02+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-29 22:34:03+02	2020-08-29 22:35:51+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-29 22:33:31+02	2020-08-29 22:36:20+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-29 22:35:52+02	2020-08-29 22:38:11+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-29 22:36:21+02	2020-08-29 22:41:08+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_8
2020-08-29 22:38:13+02	2020-08-29 22:43:59+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-29 22:41:13+02	2020-08-29 23:01:31+02	print	lilypond	\N	lilypond-2.20.0
2020-08-29 22:44:07+02	2020-08-29 23:08:17+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-29 23:08:18+02	2020-08-29 23:11:17+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-29 23:01:33+02	2020-08-29 23:12:30+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-10-06 08:35:21+02	2020-10-06 08:35:54+02	net	grilo	\N	grilo-0.3.13
2020-08-29 23:11:17+02	2020-08-29 23:18:21+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-08-29 23:18:22+02	2020-08-29 23:18:50+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-29 23:18:51+02	2020-08-29 23:19:09+02	security	p5-Digest-SHA1	\N	p5-Digest-SHA1-2.13_1
2020-08-29 23:19:09+02	2020-08-29 23:19:59+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-29 23:19:59+02	2020-08-29 23:21:40+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-29 23:21:41+02	2020-08-29 23:23:14+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-29 23:23:15+02	2020-08-29 23:24:02+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-29 23:24:03+02	2020-08-29 23:24:16+02	textproc	p5-pod2pdf	\N	p5-pod2pdf-0.42_1
2020-08-29 23:24:16+02	2020-08-29 23:24:52+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-29 23:24:52+02	2020-08-29 23:25:07+02	www	p5-libwww	\N	p5-libwww-6.47
2020-08-29 23:25:08+02	2020-08-29 23:26:03+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-29 23:26:03+02	2020-08-29 23:27:00+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-29 23:27:01+02	2020-08-29 23:31:07+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-29 23:12:31+02	2020-08-29 23:31:25+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-29 23:31:08+02	2020-08-29 23:32:00+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-29 23:31:26+02	2020-08-29 23:32:18+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-29 22:20:36+02	2020-08-29 23:32:58+02	lang	julia11	\N	julia11-1.1.1_5
2020-08-29 23:32:01+02	2020-08-29 23:33:23+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-29 23:32:18+02	2020-08-29 23:33:40+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-29 23:33:24+02	2020-08-29 23:33:55+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-29 23:33:08+02	2020-08-29 23:33:57+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-29 23:33:40+02	2020-08-29 23:35:20+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-29 23:33:58+02	2020-08-29 23:39:32+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-29 23:39:33+02	2020-08-29 23:45:30+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-29 23:33:56+02	2020-08-29 23:46:20+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-29 23:45:32+02	2020-08-29 23:47:52+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-29 23:47:53+02	2020-08-29 23:49:50+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-29 23:49:51+02	2020-08-29 23:51:35+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-29 23:51:36+02	2020-08-29 23:52:16+02	converters	recode	\N	recode-3.6_14
2020-08-29 23:52:16+02	2020-08-29 23:59:26+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-29 23:59:27+02	2020-08-29 23:59:45+02	databases	pgtop	\N	pgtop-0.05_5
2020-08-29 23:59:46+02	2020-08-30 00:02:00+02	deskutils	calibre	\N	calibre-4.23.0
2020-08-30 00:02:01+02	2020-08-30 00:02:42+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-29 23:35:21+02	2020-08-30 00:04:43+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-30 00:04:45+02	2020-08-30 00:24:50+02	devel	codeblocks	\N	codeblocks-20.03
2020-08-30 00:24:51+02	2020-08-30 00:26:01+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-30 00:02:43+02	2020-08-30 00:29:23+02	devel	RStudio	\N	RStudio-1.3.1073
2020-08-29 23:46:21+02	2020-08-30 01:26:17+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-30 00:29:23+02	2020-08-30 01:41:17+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200811
2020-08-30 01:41:20+02	2020-08-30 01:41:54+02	devel	tig	\N	tig-2.5.1
2020-08-30 01:41:56+02	2020-08-30 02:09:32+02	editors	codelite	\N	codelite-12.0_4
2020-08-30 02:09:40+02	2020-08-30 02:11:48+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-08-30 02:11:49+02	2020-08-30 02:14:08+02	editors	vim	\N	vim-8.2.1382
2020-08-30 00:26:02+02	2020-08-30 02:14:54+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-30 02:14:08+02	2020-08-30 02:44:07+02	emulators	qemu	\N	qemu-4.2.1
2020-08-30 02:44:09+02	2020-08-30 03:00:30+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-30 03:00:31+02	2020-08-30 03:01:07+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-30 03:01:08+02	2020-08-30 03:01:56+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-30 03:01:57+02	2020-08-30 03:03:19+02	games	xboard	\N	xboard-4.9.1_1
2020-08-30 03:03:20+02	2020-08-30 03:04:25+02	games	xcowsay	\N	xcowsay-1.5
2020-08-30 03:04:26+02	2020-08-30 03:21:19+02	graphics	blender	\N	blender-2.83.2_1
2020-08-30 01:26:19+02	2020-08-30 03:22:03+02	devel	llvm10	\N	llvm10-10.0.1_1
2020-08-30 02:15:00+02	2020-08-30 03:22:36+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-08-30 03:22:04+02	2020-08-30 03:23:16+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-30 03:21:20+02	2020-08-30 03:23:24+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-30 03:23:25+02	2020-08-30 03:27:47+02	graphics	wings	\N	wings-2.2.5
2020-08-30 03:27:48+02	2020-08-30 03:29:37+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-30 03:29:38+02	2020-08-30 03:32:02+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-08-30 03:32:03+02	2020-08-30 03:33:32+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-30 03:22:38+02	2020-08-30 03:37:56+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-30 03:23:16+02	2020-08-30 03:49:56+02	graphics	krita	\N	krita-4.3.0_4
2020-08-30 03:49:59+02	2020-08-30 03:50:31+02	lang	julia	\N	julia-1.0.3_3
2020-08-30 03:50:31+02	2020-08-30 03:53:46+02	mail	alpine	\N	alpine-2.23.2
2020-08-30 03:33:32+02	2020-08-30 03:57:05+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.9
2020-08-30 03:57:05+02	2020-08-30 03:58:05+02	misc	mc	\N	mc-nox11-4.8.24
2020-08-30 03:53:46+02	2020-08-30 03:58:14+02	math	maxima	\N	maxima-5.44.0_3
2020-08-30 03:58:06+02	2020-08-30 03:58:53+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-30 03:58:15+02	2020-08-30 04:00:04+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-30 03:58:53+02	2020-08-30 04:00:52+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-30 04:00:53+02	2020-08-30 04:01:57+02	net	mosh	\N	mosh-1.3.2_13
2020-08-30 04:00:05+02	2020-08-30 04:02:41+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-30 04:01:58+02	2020-08-30 04:03:46+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-30 04:02:42+02	2020-08-30 04:03:58+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-30 04:03:58+02	2020-08-30 04:04:34+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-30 03:38:10+02	2020-08-30 04:04:51+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-30 04:04:51+02	2020-08-30 04:06:04+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-30 04:04:35+02	2020-08-30 04:06:07+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-08-30 04:06:04+02	2020-08-30 04:06:48+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-30 04:06:08+02	2020-08-30 04:06:52+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-30 04:06:49+02	2020-08-30 04:07:29+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-30 04:06:52+02	2020-08-30 04:07:30+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-30 04:03:47+02	2020-08-30 04:07:53+02	net	wireshark	\N	wireshark-3.2.6
2020-08-30 04:07:29+02	2020-08-30 04:08:08+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-30 04:07:31+02	2020-08-30 04:08:09+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-30 04:08:09+02	2020-08-30 04:08:46+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-30 04:08:08+02	2020-08-30 04:08:48+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-30 04:07:54+02	2020-08-30 04:08:51+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-30 04:08:47+02	2020-08-30 04:08:59+02	net-mgmt	ipcalc	\N	ipcalc-0.41_3
2020-08-30 04:08:59+02	2020-08-30 04:09:12+02	ports-mgmt	pkg_tree	\N	pkg_tree-1.1_4
2020-08-30 04:08:48+02	2020-08-30 04:09:13+02	net-mgmt	ipv6calc	\N	ipv6calc-2.2.0
2020-08-30 04:08:51+02	2020-08-30 04:09:36+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-08-30 04:09:13+02	2020-08-30 04:09:54+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-30 04:09:36+02	2020-08-30 04:09:58+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-30 04:09:14+02	2020-08-30 04:11:11+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-30 04:09:55+02	2020-08-30 04:13:06+02	print	lyx	\N	lyx-2.3.4.2_1
2020-08-30 04:13:06+02	2020-08-30 04:14:41+02	security	klee	\N	klee-2.1_1
2020-08-30 04:14:42+02	2020-08-30 04:15:40+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-30 04:11:12+02	2020-08-30 04:15:54+02	print	texlive-full	\N	texlive-full-20150521
2020-08-30 04:15:41+02	2020-08-30 04:15:58+02	security	rkhunter	\N	rkhunter-1.4.6
2020-08-30 04:15:55+02	2020-08-30 04:16:11+02	security	ssl-admin	\N	ssl-admin-1.2.1
2020-08-30 04:16:12+02	2020-08-30 04:16:33+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-08-30 04:15:59+02	2020-08-30 04:16:52+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-30 04:16:34+02	2020-08-30 04:17:07+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-30 04:16:52+02	2020-08-30 04:17:39+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-30 04:09:59+02	2020-08-30 04:17:55+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-08-30 04:17:07+02	2020-08-30 04:17:57+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-30 04:17:40+02	2020-08-30 04:17:59+02	sysutils	parallel	\N	parallel-20200822
2020-08-30 04:17:57+02	2020-08-30 04:18:09+02	sysutils	zfs-stats	\N	zfs-stats-1.3.0
2020-08-30 04:17:59+02	2020-08-30 04:18:11+02	textproc	colordiff	\N	colordiff-1.0.19
2020-08-30 04:18:11+02	2020-08-30 04:18:28+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_2
2020-08-30 04:17:56+02	2020-08-30 04:18:36+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-08-30 04:18:37+02	2020-08-30 04:19:10+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-08-30 04:19:10+02	2020-08-30 04:19:55+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-08-30 04:19:56+02	2020-08-30 04:21:25+02	x11	mate	\N	mate-1.22.2
2020-08-30 04:18:28+02	2020-08-30 04:21:39+02	www	mod_php73	\N	mod_php73-7.3.21
2020-08-30 04:21:26+02	2020-08-30 04:23:26+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-30 04:23:27+02	2020-08-30 04:24:35+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-30 04:21:40+02	2020-08-30 04:26:53+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-30 04:26:54+02	2020-08-30 04:27:43+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-30 04:24:36+02	2020-08-30 04:47:35+02	editors	vscode	\N	vscode-1.46.1
2020-08-30 04:18:10+02	2020-08-30 05:55:16+02	www	firefox	\N	firefox-80.0_3,1
2020-08-30 05:55:18+02	2020-08-30 06:11:38+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-08-30 06:11:52+02	2020-08-30 06:12:54+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-30 06:11:52+02	2020-08-30 06:16:38+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-30 06:16:39+02	2020-08-30 06:16:46+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-30 10:52:27+02	2020-08-30 10:52:47+02	shells	bash-completion	\N	bash-completion-2.11,2
2020-08-30 10:52:27+02	2020-08-30 10:52:50+02	net	wireguard	\N	wireguard-1.0.20200827
2020-08-30 10:52:27+02	2020-08-30 10:53:16+02	devel	ninja	\N	ninja-1.10.1,2
2020-08-30 10:53:16+02	2020-08-30 10:53:28+02	devel	meson	\N	meson-0.54.3
2020-08-30 10:53:28+02	2020-08-30 10:54:09+02	security	p11-kit	\N	p11-kit-0.23.21
2020-08-30 10:54:10+02	2020-08-30 10:56:32+02	security	gnutls	\N	gnutls-3.6.14_1
2020-08-30 10:56:32+02	2020-08-30 10:57:04+02	net	glib-networking	\N	glib-networking-2.56.1_3
2020-08-30 10:56:32+02	2020-08-30 10:58:00+02	print	cups	\N	cups-2.3.3_1
2020-08-30 10:57:05+02	2020-08-30 10:58:38+02	devel	libsoup	\N	libsoup-2.62.3
2020-08-30 10:58:01+02	2020-08-30 11:03:06+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_8
2020-08-30 10:58:39+02	2020-08-30 11:07:47+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-30 11:03:06+02	2020-08-30 11:09:08+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-30 11:09:09+02	2020-08-30 11:10:33+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-30 11:10:34+02	2020-08-30 11:12:34+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-08-30 11:07:48+02	2020-08-30 11:13:21+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-30 11:12:35+02	2020-08-30 11:14:11+02	net	geoclue	\N	geoclue-2.5.5
2020-08-30 10:56:32+02	2020-08-30 11:17:19+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-30 11:17:20+02	2020-08-30 11:18:04+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-08-30 11:18:05+02	2020-08-30 11:22:36+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-30 11:22:36+02	2020-08-30 11:23:25+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-30 11:23:26+02	2020-08-30 11:24:44+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-30 11:24:44+02	2020-08-30 11:26:06+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-30 11:26:06+02	2020-08-30 11:26:57+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-30 11:26:58+02	2020-08-30 11:28:13+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-08-30 11:28:14+02	2020-08-30 11:29:35+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-08-30 11:29:35+02	2020-08-30 11:31:00+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-30 11:31:01+02	2020-08-30 11:32:35+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-08-30 11:32:36+02	2020-08-30 11:33:47+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-30 11:33:48+02	2020-08-30 11:34:50+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-08-30 11:34:51+02	2020-08-30 11:35:56+02	net	remmina	\N	remmina-1.3.9
2020-08-30 11:13:22+02	2020-08-30 11:36:35+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-30 11:35:57+02	2020-08-30 11:38:02+02	security	gcr	\N	gcr-3.28.0_1
2020-08-30 11:36:36+02	2020-08-30 11:39:51+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-30 11:39:52+02	2020-08-30 11:41:51+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-08-30 11:41:52+02	2020-08-30 11:42:48+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-30 11:42:49+02	2020-08-30 11:44:03+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-30 11:38:03+02	2020-08-30 11:45:34+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-30 11:44:04+02	2020-08-30 11:47:16+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-08-30 11:47:17+02	2020-08-30 11:47:57+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-30 11:47:57+02	2020-08-30 11:48:56+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-08-30 11:45:35+02	2020-08-30 11:50:03+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-30 11:48:57+02	2020-08-30 11:51:55+02	security	gnupg	\N	gnupg-2.2.22
2020-08-30 11:51:55+02	2020-08-30 11:56:40+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-30 11:56:43+02	2020-08-30 11:57:43+02	security	gpgme	\N	gpgme-1.14.0
2020-08-30 11:14:11+02	2020-08-30 12:19:03+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-30 12:19:04+02	2020-08-30 12:31:20+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-30 11:50:04+02	2020-08-30 12:32:52+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-30 12:31:21+02	2020-08-30 12:33:32+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-30 12:32:52+02	2020-08-30 12:34:07+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-30 12:34:08+02	2020-08-30 12:34:55+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-30 12:33:33+02	2020-08-30 12:35:23+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-08-30 12:34:55+02	2020-08-30 12:35:57+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-08-30 12:35:24+02	2020-08-30 12:36:28+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-08-30 12:36:29+02	2020-08-30 12:40:26+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-30 12:40:27+02	2020-08-30 12:50:09+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-30 11:57:44+02	2020-08-30 12:53:31+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-30 12:53:32+02	2020-08-30 12:55:30+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-30 12:55:31+02	2020-08-30 12:56:35+02	devel	librest	\N	rest-0.7.93
2020-08-30 12:56:36+02	2020-08-30 12:58:25+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-08-30 12:58:26+02	2020-08-30 12:59:29+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-30 12:59:30+02	2020-08-30 13:01:07+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-30 13:01:08+02	2020-08-30 13:03:44+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-30 13:03:45+02	2020-08-30 13:06:34+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-30 13:06:35+02	2020-08-30 13:08:26+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-08-30 13:08:26+02	2020-08-30 13:09:29+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-30 13:09:29+02	2020-08-30 13:10:43+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-30 12:50:10+02	2020-08-30 13:10:52+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-30 13:10:44+02	2020-08-30 13:11:50+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-30 13:10:54+02	2020-08-30 13:12:06+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-08-30 13:12:07+02	2020-08-30 13:14:12+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-08-30 13:11:51+02	2020-08-30 13:14:51+02	mail	mailutils	\N	mailutils-3.10
2020-08-30 13:14:51+02	2020-08-30 13:15:56+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-30 13:15:57+02	2020-08-30 13:16:52+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-08-30 13:16:53+02	2020-08-30 13:18:46+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-08-30 13:18:47+02	2020-08-30 13:20:46+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-30 13:20:47+02	2020-08-30 13:22:08+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-30 13:22:09+02	2020-08-30 13:23:48+02	x11	zenity	\N	zenity-3.32.0
2020-08-30 13:23:49+02	2020-08-30 13:29:55+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-30 13:29:56+02	2020-08-30 13:31:49+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-08-30 13:31:50+02	2020-08-30 13:34:22+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-30 13:14:13+02	2020-08-30 13:35:29+02	net	samba410	\N	samba410-4.10.17
2020-08-30 13:34:22+02	2020-08-30 13:39:44+02	editors	emacs	full	emacs-27.1,3
2020-08-30 13:35:30+02	2020-08-30 13:43:28+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-08-30 13:43:29+02	2020-08-30 13:44:40+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-08-30 13:44:41+02	2020-08-30 13:45:10+02	net	gssdp	\N	gssdp-1.2.3
2020-08-30 13:39:45+02	2020-08-30 13:46:03+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-30 13:45:13+02	2020-08-30 13:46:07+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-08-30 13:46:04+02	2020-08-30 13:47:22+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-08-30 13:46:07+02	2020-08-30 13:49:56+02	print	fontforge	\N	fontforge-20190801_2
2020-08-30 13:49:57+02	2020-08-30 13:51:00+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-30 13:51:03+02	2020-08-30 13:53:04+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-08-30 13:47:23+02	2020-08-30 13:57:26+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-08-30 13:57:26+02	2020-08-30 13:58:59+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-30 13:59:00+02	2020-08-30 14:01:00+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-30 14:01:01+02	2020-08-30 14:02:49+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-30 14:02:49+02	2020-08-30 14:04:00+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-30 14:04:01+02	2020-08-30 14:06:15+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-30 14:06:19+02	2020-08-30 14:08:53+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-08-30 14:08:54+02	2020-08-30 14:10:40+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-08-30 14:10:41+02	2020-08-30 14:15:02+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-30 14:15:03+02	2020-08-30 14:16:32+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-30 12:35:58+02	2020-08-30 14:17:46+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-30 14:16:32+02	2020-08-30 14:18:21+02	x11-wm	marco	\N	marco-1.22.4
2020-08-30 14:17:47+02	2020-08-30 14:19:03+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-30 14:19:04+02	2020-08-30 14:20:32+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-08-30 14:18:21+02	2020-08-30 14:20:42+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-30 14:20:46+02	2020-08-30 14:23:08+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-08-30 14:20:32+02	2020-08-30 14:23:45+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-30 14:23:46+02	2020-08-30 14:28:56+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-30 14:28:58+02	2020-08-30 14:31:53+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-30 14:31:56+02	2020-08-30 14:33:24+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-08-30 13:53:04+02	2020-08-30 14:40:35+02	www	chromium	\N	chromium-84.0.4147.135
2020-08-30 14:40:40+02	2020-08-30 14:50:33+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-30 14:50:33+02	2020-08-30 14:51:57+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-30 14:51:57+02	2020-08-30 14:54:00+02	net	freerdp	\N	freerdp-2.2.0
2020-08-30 14:33:25+02	2020-08-30 14:54:24+02	math	R	\N	R-4.0.2
2020-08-30 14:54:01+02	2020-08-30 14:54:32+02	net	gupnp	\N	gupnp-1.1.1_1
2020-08-30 14:54:25+02	2020-08-30 14:55:20+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-30 14:54:32+02	2020-08-30 14:55:57+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-30 14:55:21+02	2020-08-30 14:56:16+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-30 14:56:17+02	2020-08-30 14:56:48+02	print	libspectre	\N	libspectre-0.2.9
2020-08-30 14:56:48+02	2020-08-30 14:59:10+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-30 14:55:58+02	2020-08-30 15:00:04+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-30 14:59:11+02	2020-08-30 15:06:05+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-30 15:00:04+02	2020-08-30 15:06:41+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-30 15:06:42+02	2020-08-30 15:07:46+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-30 15:06:06+02	2020-08-30 15:09:27+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-30 15:07:47+02	2020-08-30 15:10:09+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-30 15:10:09+02	2020-08-30 15:10:51+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-08-30 15:09:27+02	2020-08-30 15:11:41+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-30 15:10:52+02	2020-08-30 15:12:04+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-30 15:11:42+02	2020-08-30 15:12:21+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-30 15:12:05+02	2020-08-30 15:13:25+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-30 15:12:21+02	2020-08-30 15:13:31+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-30 15:13:26+02	2020-08-30 15:14:17+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-30 15:13:32+02	2020-08-30 15:14:34+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-30 15:14:19+02	2020-08-30 15:15:31+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-30 15:15:32+02	2020-08-30 15:18:30+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-30 15:18:31+02	2020-08-30 15:19:12+02	audio	faudio	\N	FAudio-20.08
2020-08-30 14:23:08+02	2020-08-30 15:19:25+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-30 15:19:13+02	2020-08-30 15:20:15+02	audio	mate-media	\N	mate-media-1.24.1
2020-08-30 15:14:35+02	2020-08-30 15:20:22+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-30 15:19:26+02	2020-08-30 15:20:31+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-30 15:20:22+02	2020-08-30 15:21:05+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-30 15:20:31+02	2020-08-30 15:21:16+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-30 15:21:06+02	2020-08-30 15:22:39+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-30 15:20:16+02	2020-08-30 15:22:39+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-30 15:22:40+02	2020-08-30 15:25:40+02	editors	pluma	\N	pluma-1.24.0
2020-08-30 15:25:41+02	2020-08-30 15:27:26+02	graphics	atril	\N	atril-1.22.3_8
2020-08-30 15:29:01+02	2020-08-30 15:30:46+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-30 15:30:46+02	2020-08-30 15:31:58+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-30 15:31:59+02	2020-08-30 15:33:57+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-08-30 15:21:17+02	2020-08-30 15:41:50+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-30 15:33:59+02	2020-08-30 15:42:14+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-30 15:42:14+02	2020-08-30 15:43:47+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-30 15:22:39+02	2020-08-30 15:44:38+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-08-30 15:43:48+02	2020-08-30 15:45:16+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-30 15:44:40+02	2020-08-30 15:46:35+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-30 15:46:36+02	2020-08-30 15:47:35+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-30 15:41:51+02	2020-08-30 15:47:59+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-30 15:47:35+02	2020-08-30 15:48:28+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-30 15:48:01+02	2020-08-30 15:48:55+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-30 15:48:29+02	2020-08-30 15:49:13+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-30 15:48:56+02	2020-08-30 15:49:40+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-30 15:45:17+02	2020-08-30 15:49:49+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-30 15:49:13+02	2020-08-30 15:49:52+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-30 15:49:40+02	2020-08-30 15:50:17+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-30 15:49:49+02	2020-08-30 15:50:40+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-30 15:49:52+02	2020-08-30 15:51:02+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-30 15:50:17+02	2020-08-30 15:51:06+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-30 15:50:40+02	2020-08-30 15:51:46+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-30 15:51:02+02	2020-08-30 15:51:57+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-30 15:51:07+02	2020-08-30 15:56:40+02	print	lilypond	\N	lilypond-2.20.0
2020-08-30 15:51:58+02	2020-08-30 15:57:48+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-30 15:51:47+02	2020-08-30 15:58:09+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-30 15:56:41+02	2020-08-30 15:58:22+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-30 15:58:23+02	2020-08-30 15:58:45+02	print	transfig	\N	transfig-3.2.5e_4
2020-08-30 15:58:45+02	2020-08-30 15:59:29+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-30 15:59:30+02	2020-08-30 16:00:57+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-30 16:00:58+02	2020-08-30 16:02:33+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-30 16:02:33+02	2020-08-30 16:03:22+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-30 16:03:23+02	2020-08-30 16:04:03+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-08-30 15:57:48+02	2020-08-30 16:04:37+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-08-30 16:04:04+02	2020-08-30 16:04:57+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-30 16:04:37+02	2020-08-30 16:05:39+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-30 16:05:39+02	2020-08-30 16:06:58+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-30 16:06:58+02	2020-08-30 16:08:32+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-30 16:08:33+02	2020-08-30 16:10:20+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-30 16:04:58+02	2020-08-30 16:10:24+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-30 16:10:24+02	2020-08-30 16:11:20+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-30 16:10:21+02	2020-08-30 16:11:30+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-30 16:11:20+02	2020-08-30 16:11:52+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-30 16:11:31+02	2020-08-30 16:12:49+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-30 16:12:50+02	2020-08-30 16:18:25+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-30 15:58:10+02	2020-08-30 16:21:17+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-30 16:11:53+02	2020-08-30 16:25:57+02	astro	stellarium	\N	stellarium-0.20.2
2020-08-30 16:21:18+02	2020-08-30 16:27:28+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-30 16:25:58+02	2020-08-30 16:28:47+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-30 16:28:48+02	2020-08-30 16:31:22+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-30 16:31:23+02	2020-08-30 16:33:25+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-30 16:33:27+02	2020-08-30 16:37:48+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-30 16:18:27+02	2020-08-30 16:39:05+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-30 16:39:06+02	2020-08-30 16:39:51+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-30 16:37:49+02	2020-08-30 16:39:51+02	deskutils	calibre	\N	calibre-4.23.0
2020-08-30 16:39:51+02	2020-08-30 16:53:20+02	devel	codeblocks	\N	codeblocks-20.03
2020-08-30 16:53:20+02	2020-08-30 16:54:37+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-30 16:39:51+02	2020-08-30 17:09:39+02	devel	RStudio	\N	RStudio-1.3.1073
2020-08-30 17:09:40+02	2020-08-30 17:22:42+02	editors	codelite	\N	codelite-12.0_4
2020-08-30 17:22:43+02	2020-08-30 17:25:36+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-08-30 17:25:38+02	2020-08-30 17:28:24+02	editors	vim	\N	vim-8.2.1382
2020-08-30 17:28:25+02	2020-08-30 17:41:10+02	emulators	qemu	\N	qemu-4.2.1
2020-08-30 16:54:37+02	2020-08-30 17:42:48+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-30 17:42:52+02	2020-08-30 18:05:11+02	emulators	wine	\N	wine-5.0.2,1
2020-08-30 18:05:12+02	2020-08-30 18:24:42+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-30 18:24:43+02	2020-08-30 18:25:46+02	games	xcowsay	\N	xcowsay-1.5
2020-08-30 18:25:47+02	2020-08-30 18:38:17+02	graphics	blender	\N	blender-2.83.2_1
2020-08-30 16:27:29+02	2020-08-30 18:40:26+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-30 18:38:18+02	2020-08-30 18:40:54+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-30 18:40:28+02	2020-08-30 18:41:43+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-30 18:40:54+02	2020-08-30 18:48:04+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-30 18:48:11+02	2020-08-30 18:52:43+02	graphics	wings	\N	wings-2.2.5
2020-08-30 18:52:44+02	2020-08-30 18:53:34+02	graphics	xpdf	\N	xpdf-4.02,1
2020-08-30 18:41:43+02	2020-08-30 18:53:34+02	graphics	krita	\N	krita-4.3.0_4
2020-08-30 18:53:34+02	2020-08-30 18:54:25+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-30 17:41:12+02	2020-08-30 18:55:20+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-08-30 18:55:21+02	2020-08-30 18:59:52+02	math	maxima	\N	maxima-5.44.0_3
2020-08-30 18:59:52+02	2020-08-30 19:00:43+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-30 19:00:44+02	2020-08-30 19:02:48+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-30 19:02:49+02	2020-08-30 19:04:56+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-30 19:04:57+02	2020-08-30 19:07:38+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-30 19:07:38+02	2020-08-30 19:08:58+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-30 19:08:59+02	2020-08-30 19:13:03+02	net	wireshark	\N	wireshark-3.2.6
2020-08-30 19:13:04+02	2020-08-30 19:13:47+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-30 19:13:47+02	2020-08-30 19:14:30+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-30 19:14:30+02	2020-08-30 19:15:13+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-30 19:15:15+02	2020-08-30 19:15:58+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-30 19:15:59+02	2020-08-30 19:16:47+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-30 19:16:47+02	2020-08-30 19:17:33+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-30 19:17:34+02	2020-08-30 19:18:16+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-30 18:53:35+02	2020-08-30 19:18:46+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-08-30 18:54:26+02	2020-08-30 19:18:46+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-30 19:18:16+02	2020-08-30 19:18:55+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-08-30 19:18:47+02	2020-08-30 19:19:28+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-30 19:18:55+02	2020-08-30 19:19:33+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-30 19:18:47+02	2020-08-30 19:19:47+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-30 19:19:34+02	2020-08-30 19:19:56+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-08-30 19:19:47+02	2020-08-30 19:20:15+02	print	gv	\N	gv-3.7.4_5
2020-08-30 19:19:29+02	2020-08-30 19:21:34+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-30 19:21:35+02	2020-08-30 19:22:44+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-08-30 19:22:46+02	2020-08-30 19:23:54+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-30 19:23:55+02	2020-08-30 19:24:40+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-30 19:20:15+02	2020-08-30 19:25:03+02	print	texlive-full	\N	texlive-full-20150521
2020-08-30 19:24:40+02	2020-08-30 19:25:35+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-30 19:25:04+02	2020-08-30 19:25:50+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-30 19:19:56+02	2020-08-30 19:26:10+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-08-30 19:25:51+02	2020-08-30 19:27:07+02	x11	mate	\N	mate-1.22.2
2020-08-30 19:26:11+02	2020-08-30 19:27:37+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-30 19:27:38+02	2020-08-30 19:28:23+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-30 19:27:08+02	2020-08-30 19:32:02+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-30 19:32:03+02	2020-08-30 19:32:59+02	graphics	xfig	\N	xfig-3.2.7a
2020-08-30 19:28:23+02	2020-08-30 19:55:04+02	editors	vscode	\N	vscode-1.46.1
2020-08-30 19:25:35+02	2020-08-30 19:59:49+02	www	firefox	\N	firefox-80.0_4,1
2020-08-30 19:59:50+02	2020-08-30 20:17:21+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-08-30 20:17:37+02	2020-08-30 20:18:48+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-30 20:17:37+02	2020-08-30 20:27:44+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-30 20:27:45+02	2020-08-30 20:27:52+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-30 20:35:28+02	2020-08-30 20:35:44+02	www	npm	\N	npm-6.12.1_1
2020-08-30 21:30:56+02	2020-08-30 21:32:03+02	x11	pixman	\N	pixman-0.40.0_1
2020-08-30 21:32:04+02	2020-08-30 21:35:12+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_4,1
2020-08-30 21:32:04+02	2020-08-30 21:35:14+02	graphics	cairo	\N	cairo-1.16.0,2
2020-08-30 21:35:12+02	2020-08-30 21:36:19+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-08-30 21:35:14+02	2020-08-30 21:38:14+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-08-30 21:36:19+02	2020-08-30 21:39:30+02	graphics	poppler	\N	poppler-20.08.0_1
2020-08-30 21:39:31+02	2020-08-30 21:41:29+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-08-30 21:38:14+02	2020-08-30 21:42:20+02	graphics	graphviz	\N	graphviz-2.44.1_1
2020-08-30 21:42:21+02	2020-08-30 21:45:32+02	lang	vala	\N	vala-0.40.15,1
2020-08-30 21:45:33+02	2020-08-30 21:46:36+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-08-30 21:41:33+02	2020-08-30 21:47:16+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-08-30 21:47:17+02	2020-08-30 21:50:29+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-08-30 21:50:30+02	2020-08-30 21:53:39+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-08-30 21:46:37+02	2020-08-30 21:54:43+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-08-30 21:54:44+02	2020-08-30 21:56:31+02	graphics	frei0r	\N	frei0r-1.6.1
2020-08-30 21:56:32+02	2020-08-30 21:57:55+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-08-30 21:57:55+02	2020-08-30 21:58:22+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-08-30 21:53:39+02	2020-08-30 21:58:41+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-30 21:58:23+02	2020-08-30 21:59:11+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-08-30 21:59:11+02	2020-08-30 22:00:58+02	graphics	graphene	\N	graphene-1.10.0
2020-08-30 22:00:59+02	2020-08-30 22:03:41+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-08-30 21:58:42+02	2020-08-30 22:24:09+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-08-30 22:24:10+02	2020-08-30 22:28:14+02	print	tex-formats	\N	tex-formats-20150521_2
2020-08-30 22:28:15+02	2020-08-30 22:36:48+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-08-30 22:36:49+02	2020-08-30 22:41:42+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-08-30 22:41:45+02	2020-08-30 22:49:06+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-08-30 22:49:06+02	2020-08-30 22:49:53+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-08-30 22:49:53+02	2020-08-30 22:50:51+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-08-30 22:50:52+02	2020-08-30 22:52:28+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-08-30 22:52:29+02	2020-08-30 22:53:39+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-08-30 22:53:40+02	2020-08-30 22:55:06+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-08-30 22:55:07+02	2020-08-30 22:57:02+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-08-30 22:57:03+02	2020-08-30 23:01:01+02	graphics	poppler-glib	\N	poppler-glib-20.08.0_1
2020-08-30 23:01:02+02	2020-08-30 23:02:26+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-08-30 23:02:27+02	2020-08-30 23:03:43+02	net	remmina	\N	remmina-1.3.9
2020-08-30 23:03:44+02	2020-08-30 23:06:03+02	security	gcr	\N	gcr-3.28.0_1
2020-08-30 23:06:04+02	2020-08-30 23:06:38+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-08-30 22:03:42+02	2020-08-30 23:07:05+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-08-30 23:06:38+02	2020-08-30 23:07:23+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-08-30 23:07:24+02	2020-08-30 23:08:36+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-08-30 23:08:39+02	2020-08-30 23:09:21+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-30 23:09:22+02	2020-08-30 23:12:19+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-08-30 23:12:19+02	2020-08-30 23:13:33+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-08-30 23:07:06+02	2020-08-30 23:13:46+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-08-30 23:13:48+02	2020-08-30 23:15:49+02	math	gnuplot	\N	gnuplot-5.4.0
2020-08-30 23:13:34+02	2020-08-30 23:17:16+02	net	libmateweather	\N	libmateweather-1.24.0
2020-08-30 23:17:17+02	2020-08-30 23:18:36+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-08-30 23:18:37+02	2020-08-30 23:19:27+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-08-30 23:19:28+02	2020-08-30 23:23:06+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-08-30 23:23:07+02	2020-08-30 23:24:16+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-08-30 23:24:17+02	2020-08-30 23:25:32+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-08-30 23:25:33+02	2020-08-30 23:26:42+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-08-30 23:26:43+02	2020-08-30 23:27:54+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-08-30 23:27:55+02	2020-08-30 23:30:04+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-08-30 23:30:05+02	2020-08-30 23:31:16+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-08-30 23:31:17+02	2020-08-30 23:32:27+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-08-30 23:32:27+02	2020-08-30 23:33:05+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-08-30 23:33:06+02	2020-08-30 23:41:10+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-08-30 23:41:11+02	2020-08-31 00:04:47+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-30 23:15:50+02	2020-08-31 00:05:46+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-31 00:04:48+02	2020-08-31 00:06:10+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-08-31 00:06:11+02	2020-08-31 00:07:06+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-08-31 00:05:47+02	2020-08-31 00:07:37+02	graphics	poppler-utils	\N	poppler-utils-20.08.0_1
2020-08-31 00:07:37+02	2020-08-31 00:07:47+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-08-31 00:07:08+02	2020-08-31 00:08:17+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-08-31 00:07:48+02	2020-08-31 00:08:48+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-08-31 00:08:18+02	2020-08-31 00:10:10+02	x11-fm	caja	\N	caja-1.22.3_1
2020-08-31 00:08:48+02	2020-08-31 00:10:33+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_4,1
2020-08-31 00:10:33+02	2020-08-31 00:11:32+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-08-31 00:10:11+02	2020-08-31 00:12:09+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-08-31 00:11:33+02	2020-08-31 00:12:52+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-08-31 00:12:10+02	2020-08-31 00:13:17+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-08-31 00:13:17+02	2020-08-31 00:13:38+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-08-31 00:12:53+02	2020-08-31 00:14:24+02	graphics	gegl	\N	gegl-0.4.24_4
2020-08-31 00:14:25+02	2020-08-31 00:16:07+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-08-31 00:16:08+02	2020-08-31 00:17:05+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-08-31 00:17:05+02	2020-08-31 00:18:41+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-08-31 00:13:39+02	2020-08-31 00:18:52+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-08-31 00:18:53+02	2020-08-31 00:19:48+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-08-31 00:18:42+02	2020-08-31 00:20:04+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-08-31 00:20:05+02	2020-08-31 00:21:11+02	x11	zenity	\N	zenity-3.32.0
2020-08-31 00:19:48+02	2020-08-31 00:22:53+02	x11	xorg	\N	xorg-7.7_3
2020-08-31 00:22:54+02	2020-08-31 00:25:18+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-08-31 00:21:11+02	2020-08-31 00:25:20+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-08-31 00:25:21+02	2020-08-31 00:25:57+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-08-31 00:25:19+02	2020-08-31 00:26:25+02	devel	libpeas	\N	libpeas-1.22.0
2020-08-31 00:26:26+02	2020-08-31 00:26:48+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-08-31 00:25:57+02	2020-08-31 00:31:00+02	editors	emacs	full	emacs-27.1,3
2020-08-31 00:26:48+02	2020-08-31 00:33:53+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-08-31 00:33:54+02	2020-08-31 00:34:59+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-08-31 00:34:59+02	2020-08-31 00:35:53+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-08-31 00:31:01+02	2020-08-31 00:36:46+02	graphics	opencv	\N	opencv-3.4.1_35
2020-08-31 00:35:54+02	2020-08-31 00:39:10+02	print	fontforge	\N	fontforge-20190801_2
2020-08-31 00:39:10+02	2020-08-31 00:40:07+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-08-31 00:40:07+02	2020-08-31 00:41:48+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-08-31 00:36:47+02	2020-08-31 00:44:55+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-08-31 00:44:56+02	2020-08-31 00:46:08+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-08-31 00:46:09+02	2020-08-31 00:47:36+02	x11	yelp	\N	yelp-3.28.1_2
2020-08-31 00:47:36+02	2020-08-31 00:48:49+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-08-31 00:48:51+02	2020-08-31 00:49:56+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-08-31 00:49:57+02	2020-08-31 00:51:31+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-08-31 00:51:32+02	2020-08-31 00:55:09+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-08-31 00:55:10+02	2020-08-31 00:56:31+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-08-31 00:56:32+02	2020-08-31 00:58:22+02	x11-wm	marco	\N	marco-1.22.4
2020-08-31 00:58:23+02	2020-08-31 00:59:33+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-08-31 00:59:33+02	2020-08-31 01:01:50+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-08-31 01:01:51+02	2020-08-31 01:03:02+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-08-31 01:03:03+02	2020-08-31 01:06:15+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-08-31 01:06:15+02	2020-08-31 01:08:48+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-08-31 01:08:49+02	2020-08-31 01:16:36+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-08-31 01:16:38+02	2020-08-31 01:17:31+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-08-31 01:17:32+02	2020-08-31 01:19:22+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-31 01:19:23+02	2020-08-31 01:19:51+02	graphics	libgxps	\N	libgxps-0.3.1
2020-08-31 01:19:53+02	2020-08-31 01:21:10+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-08-31 01:21:12+02	2020-08-31 01:21:43+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-08-31 00:41:49+02	2020-08-31 01:22:24+02	www	chromium	\N	chromium-84.0.4147.135
2020-08-31 01:22:28+02	2020-08-31 01:30:12+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-08-31 01:30:13+02	2020-08-31 01:31:36+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-31 01:31:37+02	2020-08-31 01:32:25+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-08-31 01:32:25+02	2020-08-31 01:33:56+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-31 01:33:56+02	2020-08-31 01:34:46+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-08-31 01:21:44+02	2020-08-31 01:38:12+02	math	R	\N	R-4.0.2
2020-08-31 01:34:47+02	2020-08-31 01:38:29+02	print	gutenprint	\N	gutenprint-5.3.3
2020-08-31 01:38:12+02	2020-08-31 01:38:36+02	print	libspectre	\N	libspectre-0.2.9
2020-08-31 01:38:29+02	2020-08-31 01:40:50+02	print	mftrace	\N	mftrace-1.2.18_2
2020-08-31 01:38:36+02	2020-08-31 01:44:51+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-08-31 01:40:51+02	2020-08-31 01:46:53+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-08-31 01:44:51+02	2020-08-31 01:47:43+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-08-31 01:46:54+02	2020-08-31 01:47:52+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-08-31 01:47:43+02	2020-08-31 01:49:37+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-08-31 01:47:53+02	2020-08-31 01:49:55+02	textproc	dblatex	\N	dblatex-0.3.11
2020-08-31 01:49:37+02	2020-08-31 01:50:37+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-31 01:49:56+02	2020-08-31 01:51:07+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-08-31 01:51:08+02	2020-08-31 01:52:22+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-08-31 01:50:38+02	2020-08-31 01:52:24+02	x11-servers	xephyr	\N	xephyr-1.20.8_4,1
2020-08-31 01:52:23+02	2020-08-31 01:53:11+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-08-31 01:52:24+02	2020-08-31 01:53:22+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-08-31 01:53:11+02	2020-08-31 01:54:18+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-08-31 01:54:18+02	2020-08-31 01:56:54+02	archivers	engrampa	\N	engrampa-1.24.0
2020-08-31 01:56:55+02	2020-08-31 01:58:10+02	audio	mate-media	\N	mate-media-1.24.1
2020-08-31 01:53:22+02	2020-08-31 01:59:08+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.3
2020-08-31 01:58:11+02	2020-08-31 01:59:12+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-08-31 01:59:08+02	2020-08-31 02:00:20+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-08-31 02:00:21+02	2020-08-31 02:01:21+02	deskutils	mozo	\N	mozo-1.24.0
2020-08-31 01:59:12+02	2020-08-31 02:02:08+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-08-31 02:01:22+02	2020-08-31 02:02:15+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-08-31 02:02:08+02	2020-08-31 02:25:52+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-08-31 02:02:16+02	2020-08-31 02:26:09+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-08-31 02:26:11+02	2020-08-31 02:28:39+02	graphics	atril	\N	atril-1.22.3_8
2020-08-31 02:25:53+02	2020-08-31 02:29:58+02	editors	pluma	\N	pluma-1.24.0
2020-08-31 02:28:40+02	2020-08-31 02:30:34+02	graphics	eom	\N	eom-1.24.0
2020-08-31 02:30:35+02	2020-08-31 02:31:33+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-08-31 02:29:59+02	2020-08-31 02:31:39+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-31 02:31:34+02	2020-08-31 02:33:10+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-08-31 02:33:11+02	2020-08-31 02:39:57+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17,1
2020-08-31 02:31:40+02	2020-08-31 02:40:21+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-08-31 02:40:22+02	2020-08-31 02:41:20+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-08-31 02:39:57+02	2020-08-31 02:41:22+02	math	mate-calc	\N	mate-calc-1.24.0
2020-08-31 02:41:20+02	2020-08-31 02:43:07+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-31 02:43:08+02	2020-08-31 02:44:17+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-31 02:44:18+02	2020-08-31 02:45:24+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-08-31 02:45:25+02	2020-08-31 02:46:06+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-08-31 02:41:23+02	2020-08-31 02:46:27+02	multimedia	vlc	\N	vlc-3.0.11_3,4
2020-08-31 02:46:07+02	2020-08-31 02:46:46+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-08-31 02:46:28+02	2020-08-31 02:47:11+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-08-31 02:46:47+02	2020-08-31 02:47:24+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-08-31 02:47:11+02	2020-08-31 02:47:54+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-08-31 02:47:24+02	2020-08-31 02:48:28+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-08-31 02:47:54+02	2020-08-31 02:49:07+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-08-31 02:48:29+02	2020-08-31 02:49:12+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-08-31 02:49:12+02	2020-08-31 02:50:05+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-08-31 02:49:07+02	2020-08-31 02:50:07+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-30 21:30:56+02	2020-08-31 02:53:26+02	lang	rust	\N	rust-1.46.0
2020-08-31 02:50:06+02	2020-08-31 02:55:20+02	print	lilypond	\N	lilypond-2.20.0
2020-08-31 02:50:07+02	2020-08-31 02:56:03+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-08-31 02:56:04+02	2020-08-31 02:57:56+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-08-31 02:53:29+02	2020-08-31 02:58:21+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.4_1
2020-08-31 02:55:21+02	2020-08-31 03:00:27+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-08-31 03:00:28+02	2020-08-31 03:01:08+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-08-31 03:01:08+02	2020-08-31 03:02:30+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-08-31 03:02:30+02	2020-08-31 03:04:05+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-08-31 02:57:57+02	2020-08-31 03:04:16+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-08-31 03:04:05+02	2020-08-31 03:05:00+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-08-31 03:05:01+02	2020-08-31 03:06:26+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-31 03:04:16+02	2020-08-31 03:11:12+02	textproc	ripgrep	\N	ripgrep-12.1.1_5
2020-08-31 03:11:13+02	2020-08-31 03:12:16+02	x11	mate-base	\N	mate-base-1.24.0
2020-08-31 03:06:26+02	2020-08-31 03:12:42+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-08-31 03:12:17+02	2020-08-31 03:13:17+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-08-31 03:12:42+02	2020-08-31 03:14:14+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-08-31 03:13:17+02	2020-08-31 03:14:33+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-08-31 03:14:34+02	2020-08-31 03:15:08+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-08-31 03:14:15+02	2020-08-31 03:15:09+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-08-31 03:15:09+02	2020-08-31 03:16:50+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-08-31 02:58:21+02	2020-08-31 03:18:53+02	print	texlive-docs	\N	texlive-docs-20150523
2020-08-31 03:15:09+02	2020-08-31 03:19:05+02	audio	rosegarden	\N	rosegarden-20.06
2020-08-31 03:19:06+02	2020-08-31 03:21:38+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-31 03:18:56+02	2020-08-31 03:24:24+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-31 03:24:25+02	2020-08-31 03:26:26+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-31 03:26:27+02	2020-08-31 03:28:10+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-31 03:28:11+02	2020-08-31 03:32:02+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-08-31 03:32:02+02	2020-08-31 03:33:46+02	deskutils	calibre	\N	calibre-4.23.0
2020-08-31 03:16:51+02	2020-08-31 03:33:51+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-31 03:33:47+02	2020-08-31 03:34:27+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-08-31 03:34:28+02	2020-08-31 03:45:06+02	devel	codeblocks	\N	codeblocks-20.03
2020-08-31 03:45:07+02	2020-08-31 03:46:16+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-08-31 03:33:52+02	2020-08-31 03:56:18+02	devel	RStudio	\N	RStudio-1.3.1073
2020-08-31 03:56:19+02	2020-08-31 04:06:58+02	editors	codelite	\N	codelite-12.0_4
2020-08-31 04:06:59+02	2020-08-31 04:09:14+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-08-31 04:09:15+02	2020-08-31 04:11:20+02	editors	vim	\N	vim-8.2.1382
2020-08-31 04:11:21+02	2020-08-31 04:22:07+02	emulators	qemu	\N	qemu-4.2.1
2020-08-31 03:46:16+02	2020-08-31 04:24:47+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-31 04:24:51+02	2020-08-31 04:38:53+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-08-31 04:38:53+02	2020-08-31 04:39:26+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-08-31 04:39:26+02	2020-08-31 04:40:07+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-08-31 04:40:08+02	2020-08-31 04:41:02+02	games	xboard	\N	xboard-4.9.1_1
2020-08-31 04:41:02+02	2020-08-31 04:41:46+02	games	xcowsay	\N	xcowsay-1.5
2020-08-31 04:41:46+02	2020-08-31 04:48:01+02	graphics	blender	\N	blender-2.83.2_1
2020-08-31 04:48:02+02	2020-08-31 04:50:08+02	graphics	dia	\N	dia-0.97.3_1,1
2020-08-31 04:50:09+02	2020-08-31 04:51:21+02	graphics	gimp	\N	gimp-2.10.20,2
2020-08-31 04:51:22+02	2020-08-31 04:56:37+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-31 03:21:39+02	2020-08-31 04:58:40+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-31 04:58:41+02	2020-08-31 05:04:18+02	graphics	wings	\N	wings-2.2.5
2020-08-31 05:04:19+02	2020-08-31 05:05:20+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-08-31 04:56:37+02	2020-08-31 05:06:33+02	graphics	krita	\N	krita-4.3.0_4
2020-08-31 04:22:08+02	2020-08-31 05:13:02+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-08-31 05:13:03+02	2020-08-31 05:17:18+02	math	maxima	\N	maxima-5.44.0_3
2020-08-31 05:17:19+02	2020-08-31 05:18:04+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-31 05:18:04+02	2020-08-31 05:19:35+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-31 05:19:36+02	2020-08-31 05:21:36+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-31 05:05:21+02	2020-08-31 05:22:51+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-08-31 05:21:37+02	2020-08-31 05:23:39+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-31 05:22:51+02	2020-08-31 05:24:32+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-31 05:23:40+02	2020-08-31 05:24:50+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-08-31 05:24:50+02	2020-08-31 05:25:28+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-08-31 05:25:28+02	2020-08-31 05:26:23+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-08-31 05:26:24+02	2020-08-31 05:27:04+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-08-31 05:24:32+02	2020-08-31 05:27:43+02	net	wireshark	\N	wireshark-3.2.6
2020-08-31 05:27:04+02	2020-08-31 05:27:43+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-08-31 05:27:44+02	2020-08-31 05:28:22+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-08-31 05:27:44+02	2020-08-31 05:28:23+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-08-31 05:06:34+02	2020-08-31 05:28:33+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-08-31 05:28:23+02	2020-08-31 05:28:59+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-08-31 05:28:23+02	2020-08-31 05:29:00+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-10-06 08:27:03+02	2020-10-06 08:40:51+02	cad	kicad	\N	kicad-5.1.7,2
2020-08-31 05:28:33+02	2020-08-31 05:29:28+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-31 05:29:00+02	2020-08-31 05:29:38+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-08-31 05:29:00+02	2020-08-31 05:29:40+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-08-31 05:29:28+02	2020-08-31 05:30:07+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-08-31 05:29:39+02	2020-08-31 05:31:39+02	print	auctex	full	auctex-emacs27-12.2_1
2020-08-31 05:29:40+02	2020-08-31 05:33:21+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-08-31 05:30:07+02	2020-08-31 05:33:53+02	print	texlive-full	\N	texlive-full-20150521
2020-08-31 05:33:22+02	2020-08-31 05:34:16+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-08-31 05:34:16+02	2020-08-31 05:34:52+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-08-31 05:34:53+02	2020-08-31 05:35:55+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-08-31 05:33:54+02	2020-08-31 05:36:15+02	sysutils	exa	\N	exa-0.9.0_15
2020-08-31 05:35:56+02	2020-08-31 05:36:43+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-08-31 05:36:15+02	2020-08-31 05:37:25+02	sysutils	hexyl	\N	hexyl-0.8.0_4
2020-08-31 05:31:39+02	2020-08-31 05:38:53+02	shells	starship	\N	starship-0.44.0_2
2020-08-31 05:36:43+02	2020-08-31 05:44:57+02	sysutils	rsfetch	\N	rsfetch-2.0.0_7
2020-08-31 05:37:25+02	2020-08-31 05:48:48+02	textproc	bat	\N	bat-0.15.4_4
2020-08-31 05:48:49+02	2020-08-31 05:50:40+02	x11	mate	\N	mate-1.22.2
2020-08-31 05:50:41+02	2020-08-31 05:52:58+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-08-31 05:52:58+02	2020-08-31 05:58:07+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-08-31 05:58:08+02	2020-08-31 05:59:09+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-08-31 05:59:10+02	2020-08-31 05:59:37+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-08-31 05:44:57+02	2020-08-31 06:02:26+02	x11	alacritty	\N	alacritty-0.5.0_2
2020-08-31 05:38:54+02	2020-08-31 06:14:54+02	www	firefox	\N	firefox-80.0_5,1
2020-08-31 05:59:37+02	2020-08-31 06:20:10+02	editors	vscode	\N	vscode-1.46.1
2020-08-31 06:14:56+02	2020-08-31 06:32:07+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-08-31 06:32:23+02	2020-08-31 06:33:14+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-31 06:32:23+02	2020-08-31 06:33:53+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-31 06:33:54+02	2020-08-31 06:34:01+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-31 07:52:46+02	2020-08-31 07:54:54+02	sysutils	intel-pcm	\N	intel-pcm-202008_1
2020-08-31 07:54:54+02	2020-08-31 07:55:04+02	net-mgmt	nstat	\N	nstat-1.0_3
2020-08-31 07:52:46+02	2020-08-31 07:58:27+02	devel	ncurses	\N	ncurses-6.2.20200829
2020-08-31 07:58:28+02	2020-08-31 07:59:06+02	devel	adacurses	\N	adacurses-20180127
2020-08-31 07:52:46+02	2020-08-31 08:00:37+02	devel	protobuf	\N	protobuf-3.13.0,1
2020-08-31 08:00:38+02	2020-08-31 08:01:55+02	databases	mysql57-client	\N	mysql57-client-5.7.31_1
2020-08-31 08:01:55+02	2020-08-31 08:02:25+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-08-31 08:00:38+02	2020-08-31 08:03:38+02	devel	protobuf-c	\N	protobuf-c-1.3.2_6
2020-08-31 08:03:39+02	2020-08-31 08:05:48+02	dns	bind916	\N	bind916-9.16.6_1
2020-08-31 08:00:38+02	2020-08-31 08:05:54+02	graphics	opencv-core	\N	opencv-core-3.4.1_36
2020-08-31 08:05:49+02	2020-08-31 08:06:22+02	net	mosh	\N	mosh-1.3.2_14
2020-08-31 08:06:22+02	2020-08-31 08:07:30+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-08-31 08:05:55+02	2020-08-31 08:11:32+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-08-31 08:11:32+02	2020-08-31 08:12:15+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-08-31 08:12:15+02	2020-08-31 08:12:51+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-08-31 08:12:52+02	2020-08-31 08:20:13+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-08-31 08:02:26+02	2020-08-31 08:25:47+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-08-31 08:25:48+02	2020-08-31 08:33:19+02	graphics	opencv	\N	opencv-3.4.1_36
2020-08-31 08:11:32+02	2020-08-31 08:44:29+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-08-31 08:44:31+02	2020-08-31 09:06:38+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-08-31 09:06:40+02	2020-08-31 09:08:43+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-08-31 09:08:45+02	2020-08-31 09:10:28+02	multimedia	mlt	\N	mlt-6.22.1
2020-08-31 09:10:29+02	2020-08-31 09:11:52+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-08-31 09:11:54+02	2020-08-31 09:12:31+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-08-31 09:12:32+02	2020-08-31 09:13:10+02	audio	faudio	\N	FAudio-20.08
2020-08-31 09:13:12+02	2020-08-31 09:14:46+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-08-31 09:14:47+02	2020-08-31 09:16:27+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-08-31 08:33:20+02	2020-08-31 09:16:44+02	www	chromium	\N	chromium-84.0.4147.135
2020-08-31 09:16:48+02	2020-08-31 09:18:13+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-08-31 09:16:28+02	2020-08-31 09:18:30+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-08-31 09:18:31+02	2020-08-31 09:19:37+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-08-31 09:19:38+02	2020-08-31 09:20:54+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-08-31 09:20:55+02	2020-08-31 09:22:17+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-08-31 09:18:13+02	2020-08-31 09:23:35+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-08-31 09:23:35+02	2020-08-31 09:25:49+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-08-31 09:22:18+02	2020-08-31 09:27:28+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-08-31 09:27:29+02	2020-08-31 09:29:25+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-08-31 09:29:26+02	2020-08-31 09:31:11+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-08-31 08:20:14+02	2020-08-31 09:47:47+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-08-31 09:31:12+02	2020-08-31 09:56:51+02	devel	electron7	\N	electron7-7.3.1_1
2020-08-31 09:47:50+02	2020-08-31 09:57:10+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-08-31 09:56:53+02	2020-08-31 09:57:45+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-08-31 09:57:11+02	2020-08-31 09:58:25+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-08-31 09:57:46+02	2020-08-31 09:59:31+02	deskutils	calibre	\N	calibre-4.23.0
2020-08-31 09:58:26+02	2020-08-31 10:20:13+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-08-31 10:20:14+02	2020-08-31 10:22:00+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-08-31 09:59:32+02	2020-08-31 10:23:09+02	devel	RStudio	\N	RStudio-1.3.1073
2020-08-31 10:23:10+02	2020-08-31 10:27:40+02	graphics	blender	\N	blender-2.83.2_1
2020-08-31 10:22:01+02	2020-08-31 10:27:50+02	emulators	wine	\N	wine-5.0.2,1
2020-08-31 10:27:42+02	2020-08-31 10:28:24+02	multimedia	celluloid	\N	celluloid-0.19
2020-08-31 10:27:52+02	2020-08-31 10:29:16+02	multimedia	obs-studio	\N	obs-studio-25.0.8_1
2020-08-31 10:28:25+02	2020-08-31 10:30:21+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-08-31 10:29:17+02	2020-08-31 10:31:20+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-08-31 10:30:22+02	2020-08-31 10:31:26+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-08-31 10:31:26+02	2020-08-31 10:57:10+02	editors	vscode	\N	vscode-1.46.1
2020-08-31 09:25:50+02	2020-08-31 10:59:18+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-08-31 10:31:21+02	2020-08-31 11:04:58+02	www	firefox	\N	firefox-80.0_5,1
2020-08-31 11:05:00+02	2020-08-31 11:20:55+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-08-31 11:21:08+02	2020-08-31 11:22:01+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-08-31 11:21:08+02	2020-08-31 11:22:40+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-08-31 11:22:40+02	2020-08-31 11:22:47+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-08-31 23:53:30+02	2020-08-31 23:53:43+02	graphics	libpotrace	\N	libpotrace-1.16
2020-08-31 23:53:30+02	2020-08-31 23:54:24+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-08-31 23:53:44+02	2020-08-31 23:56:42+02	print	texlive-base	\N	texlive-base-20150521_54
2020-08-31 23:54:25+02	2020-08-31 23:58:02+02	graphics	inkscape	\N	inkscape-1.0_4
2020-08-31 23:58:03+02	2020-08-31 23:59:13+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-08-31 23:53:30+02	2020-09-01 00:05:00+02	net	openmpi	\N	openmpi-4.0.5
2020-08-31 23:56:42+02	2020-09-01 00:11:53+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-01 00:11:54+02	2020-09-01 00:13:29+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-01 00:11:54+02	2020-09-01 00:13:48+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-01 00:11:54+02	2020-09-01 00:16:15+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-01 00:16:15+02	2020-09-01 00:18:43+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-01 00:13:49+02	2020-09-01 00:18:59+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-01 00:18:59+02	2020-09-01 00:24:02+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-01 00:24:03+02	2020-09-01 00:29:48+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-01 00:13:30+02	2020-09-01 00:31:24+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-01 00:29:49+02	2020-09-01 00:31:49+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-01 00:31:50+02	2020-09-01 00:37:56+02	print	lilypond	\N	lilypond-2.20.0
2020-09-01 00:31:26+02	2020-09-01 00:40:47+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-01 00:37:57+02	2020-09-01 00:44:10+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-01 00:18:43+02	2020-09-01 00:44:53+02	math	vtk8	\N	vtk8-8.2.0
2020-09-01 00:44:10+02	2020-09-01 00:45:37+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-01 00:40:47+02	2020-09-01 00:46:04+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-01 00:46:05+02	2020-09-01 00:49:16+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-01 00:44:54+02	2020-09-01 00:49:22+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-01 00:49:17+02	2020-09-01 00:51:18+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-01 00:45:38+02	2020-09-01 00:51:34+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-01 00:51:35+02	2020-09-01 00:56:26+02	print	texlive-full	\N	texlive-full-20150521
2020-09-01 00:49:23+02	2020-09-01 01:03:37+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-01 01:03:38+02	2020-09-01 01:05:48+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-01 01:03:38+02	2020-09-01 01:08:08+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-01 00:51:19+02	2020-09-01 01:08:33+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-01 01:08:09+02	2020-09-01 01:09:32+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-01 01:08:34+02	2020-09-01 01:09:49+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-01 01:05:48+02	2020-09-01 01:46:44+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-01 01:46:45+02	2020-09-01 02:02:55+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-01 02:03:08+02	2020-09-01 02:04:01+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-01 02:03:08+02	2020-09-01 02:04:39+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-01 02:04:40+02	2020-09-01 02:04:47+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-01 21:40:26+02	2020-09-01 23:12:10+02	www	firefox	\N	firefox-80.0.1,1
2020-09-01 23:12:11+02	2020-09-01 23:29:21+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-01 23:29:37+02	2020-09-01 23:30:30+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-01 23:29:37+02	2020-09-01 23:31:09+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-01 23:31:10+02	2020-09-01 23:31:16+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-02 22:50:47+02	2020-09-02 22:53:51+02	devel	ncurses	\N	ncurses-6.2.20200831
2020-09-02 22:53:52+02	2020-09-02 22:54:44+02	devel	adacurses	\N	adacurses-20180127
2020-09-02 22:50:47+02	2020-09-02 22:56:37+02	lang	go	\N	go-1.15.1,1
2020-09-02 22:56:37+02	2020-09-02 22:58:27+02	games	xboard	\N	xboard-4.9.1_1
2020-09-02 22:54:45+02	2020-09-02 22:59:21+02	editors	vim	\N	vim-8.2.1558
2020-09-02 22:50:47+02	2020-09-02 23:35:12+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-02 22:59:23+02	2020-09-02 23:39:39+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p7
2020-09-02 23:39:46+02	2020-09-02 23:40:01+02	sysutils	bpytop	\N	bpytop-1.0.22
2020-09-02 23:35:38+02	2020-09-02 23:43:51+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-02 23:43:52+02	2020-09-02 23:46:44+02	graphics	wings	\N	wings-2.2.5
2020-09-02 22:58:37+02	2020-09-03 01:20:40+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200830
2020-09-03 01:20:41+02	2020-09-03 01:38:36+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-03 01:38:50+02	2020-09-03 01:39:59+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-03 01:38:50+02	2020-09-03 01:43:43+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-03 01:43:44+02	2020-09-03 01:43:51+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-03 07:43:02+02	2020-09-03 07:43:23+02	audio	faudio	\N	FAudio-20.09
2020-09-03 07:43:24+02	2020-09-03 07:47:35+02	emulators	wine	\N	wine-5.0.2,1
2020-09-03 07:47:36+02	2020-09-03 08:04:26+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-03 08:04:40+02	2020-09-03 08:05:35+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-03 08:04:40+02	2020-09-03 08:06:16+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-03 08:06:16+02	2020-09-03 08:06:23+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-03 18:06:20+02	2020-09-03 18:07:14+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-03 18:06:20+02	2020-09-03 18:07:55+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-03 18:07:55+02	2020-09-03 18:08:03+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-03 20:29:46+02	2020-09-03 20:34:07+02	games	xblast	\N	xblast-2.10.4_18
2020-09-03 20:34:09+02	2020-09-03 21:27:43+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-03 21:28:03+02	2020-09-03 21:31:03+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-03 21:28:03+02	2020-09-03 21:33:02+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-03 21:33:03+02	2020-09-03 21:33:20+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-04 22:01:45+02	2020-09-04 22:02:58+02	security	gnupg	\N	gnupg-2.2.23
2020-09-04 22:02:59+02	2020-09-04 22:03:21+02	security	gpgme	\N	gpgme-1.14.0
2020-09-04 22:03:22+02	2020-09-04 22:03:48+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-09-04 22:03:49+02	2020-09-04 22:04:46+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-09-04 22:04:47+02	2020-09-04 22:05:42+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-09-04 22:05:42+02	2020-09-04 22:07:52+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-09-04 22:07:52+02	2020-09-04 22:14:51+02	graphics	krita	\N	krita-4.3.0_4
2020-09-04 22:07:52+02	2020-09-04 22:25:41+02	editors	libreoffice	\N	libreoffice-7.0.0.3
2020-09-04 22:25:42+02	2020-09-04 22:27:02+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.0.3
2020-09-04 22:27:03+02	2020-09-04 22:44:00+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-04 22:44:12+02	2020-09-04 22:45:04+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-04 22:44:12+02	2020-09-04 22:45:48+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-04 22:45:48+02	2020-09-04 22:45:56+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-05 09:42:12+02	2020-09-05 09:42:47+02	textproc	libnumbertext	\N	libnumbertext-1.0.6
2020-09-05 09:42:12+02	2020-09-05 09:42:49+02	converters	recode	\N	recode-3.6_15
2020-09-05 09:42:12+02	2020-09-05 09:42:51+02	devel	libuv	\N	libuv-1.39.0
2020-09-05 09:42:49+02	2020-09-05 09:43:24+02	net	rdesktop	\N	rdesktop-1.9.0
2020-09-05 09:42:52+02	2020-09-05 10:08:18+02	devel	cmake	\N	cmake-3.18.2
2020-09-05 10:08:19+02	2020-09-05 10:13:43+02	dns	bind-tools	\N	bind-tools-9.16.6
2020-09-05 09:43:24+02	2020-09-05 11:38:17+02	www	node	\N	node-14.9.0
2020-09-05 11:38:18+02	2020-09-05 11:39:37+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-09-05 11:39:37+02	2020-09-05 11:41:19+02	dns	bind916	\N	bind916-9.16.6_1
2020-09-05 11:41:19+02	2020-09-05 11:42:59+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-09-05 10:13:43+02	2020-09-05 11:45:03+02	www	node12	\N	node12-12.18.3
2020-09-05 11:45:04+02	2020-09-05 11:45:16+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-09-05 11:45:03+02	2020-09-05 11:45:29+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-09-05 09:42:48+02	2020-09-05 14:14:54+02	editors	libreoffice	\N	libreoffice-7.0.1.2
2020-09-05 14:14:55+02	2020-09-05 14:16:11+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-05 14:16:12+02	2020-09-05 14:32:47+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-05 14:32:59+02	2020-09-05 14:34:06+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-05 14:32:59+02	2020-09-05 14:37:46+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-05 14:37:47+02	2020-09-05 14:37:54+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-05 16:10:34+02	2020-09-05 16:10:46+02	textproc	p5-PDF-API2-XS	\N	p5-PDF-API2-XS-1.001
2020-09-05 16:10:34+02	2020-09-05 16:10:47+02	databases	py-apsw	py37	py37-apsw-3.33.0
2020-09-05 16:10:46+02	2020-09-05 16:11:08+02	textproc	p5-PDF-API2	\N	p5-PDF-API2-2.038
2020-09-05 16:11:08+02	2020-09-05 16:11:19+02	textproc	p5-pod2pdf	\N	p5-pod2pdf-0.42_1
2020-09-05 16:10:47+02	2020-09-05 16:12:16+02	deskutils	calibre	\N	calibre-4.23.0
2020-09-05 16:12:16+02	2020-09-05 16:29:04+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-05 16:29:17+02	2020-09-05 16:30:08+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-05 16:29:17+02	2020-09-05 16:30:53+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-05 16:30:53+02	2020-09-05 16:31:00+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-05 21:12:33+02	2020-09-05 21:16:34+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1
2020-09-05 21:16:34+02	2020-09-05 21:34:01+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-05 21:34:13+02	2020-09-05 21:35:05+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-05 21:34:13+02	2020-09-05 21:35:45+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-05 21:35:45+02	2020-09-05 21:35:52+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-06 11:18:50+02	2020-09-06 11:18:56+02	misc	pciids	\N	pciids-20200819
2020-09-06 11:18:50+02	2020-09-06 11:18:56+02	misc	usbids	\N	usbids-20200826
2020-09-06 11:18:57+02	2020-09-06 11:19:04+02	devel	libpci	\N	libpci-3.7.0
2020-09-06 11:18:57+02	2020-09-06 11:19:08+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-09-06 11:18:49+02	2020-09-06 11:19:16+02	www	py-django22	py37	py37-django22-2.2.16
2020-09-06 11:19:08+02	2020-09-06 11:19:26+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-09-06 11:19:04+02	2020-09-06 11:19:52+02	sysutils	hal	\N	hal-0.5.14_34
2020-09-06 11:19:16+02	2020-09-06 11:19:56+02	devel	hwloc	\N	hwloc-1.11.11
2020-09-06 11:19:52+02	2020-09-06 11:20:10+02	multimedia	libva	\N	libva-2.8.0
2020-09-06 11:20:11+02	2020-09-06 11:25:23+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-09-06 11:19:27+02	2020-09-06 11:26:02+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-09-06 11:25:23+02	2020-09-06 11:26:27+02	x11-fonts	noto-basic	\N	noto-basic-2.0_1
2020-09-06 11:26:28+02	2020-09-06 11:27:16+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-09-06 11:19:56+02	2020-09-06 11:27:24+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-09-06 11:26:03+02	2020-09-06 11:27:58+02	graphics	cairo	\N	cairo-1.16.0,2
2020-09-06 11:27:25+02	2020-09-06 11:28:01+02	graphics	libGLU	\N	libGLU-9.0.1
2020-09-06 11:28:02+02	2020-09-06 11:29:55+02	graphics	poppler	\N	poppler-20.08.0_1
2020-09-06 11:27:58+02	2020-09-06 11:30:32+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-09-06 11:29:56+02	2020-09-06 11:31:41+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-09-06 11:31:42+02	2020-09-06 11:33:22+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-09-06 11:30:33+02	2020-09-06 11:35:17+02	graphics	graphviz	\N	graphviz-2.44.1_1
2020-09-06 11:35:17+02	2020-09-06 11:39:10+02	lang	vala	\N	vala-0.40.15,1
2020-09-06 11:39:11+02	2020-09-06 11:40:20+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-09-06 11:33:22+02	2020-09-06 11:40:42+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-06 11:40:42+02	2020-09-06 11:42:49+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-09-06 11:27:16+02	2020-09-06 11:47:02+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-09-06 11:42:50+02	2020-09-06 11:47:37+02	print	texlive-base	\N	texlive-base-20150521_54
2020-09-06 11:40:21+02	2020-09-06 11:48:44+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-09-06 11:48:45+02	2020-09-06 11:50:07+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-06 11:50:08+02	2020-09-06 11:51:04+02	graphics	frei0r	\N	frei0r-1.6.1
2020-09-06 11:51:05+02	2020-09-06 11:51:25+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-09-06 11:47:04+02	2020-09-06 11:56:56+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-09-06 11:51:25+02	2020-09-06 11:59:28+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-09-06 11:59:29+02	2020-09-06 12:01:45+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-09-06 12:01:46+02	2020-09-06 12:02:21+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-06 12:02:22+02	2020-09-06 12:04:26+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-06 12:04:27+02	2020-09-06 12:06:15+02	graphics	graphene	\N	graphene-1.10.0
2020-09-06 12:06:15+02	2020-09-06 12:06:36+02	www	libwpe	\N	libwpe-1.4.0.1
2020-09-06 12:06:36+02	2020-09-06 12:08:22+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-06 12:08:23+02	2020-09-06 12:08:48+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-09-06 11:47:38+02	2020-09-06 12:09:22+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-06 12:09:23+02	2020-09-06 12:12:52+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-06 12:12:53+02	2020-09-06 12:20:29+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-06 12:20:30+02	2020-09-06 12:21:26+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-09-06 12:21:27+02	2020-09-06 12:22:05+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-09-06 11:56:57+02	2020-09-06 12:26:20+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-09-06 12:22:06+02	2020-09-06 12:26:37+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-06 12:26:38+02	2020-09-06 12:27:36+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-09-06 12:26:20+02	2020-09-06 12:28:28+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-09-06 12:28:29+02	2020-09-06 12:30:06+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-09-06 12:27:36+02	2020-09-06 12:33:54+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-06 12:33:55+02	2020-09-06 12:34:50+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-06 12:34:51+02	2020-09-06 12:36:29+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-09-06 12:36:30+02	2020-09-06 12:37:41+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-09-06 12:37:42+02	2020-09-06 12:39:24+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-06 12:39:26+02	2020-09-06 12:40:32+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-06 12:40:32+02	2020-09-06 12:43:01+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-09-06 12:43:02+02	2020-09-06 12:44:43+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-09-06 12:30:07+02	2020-09-06 12:45:24+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-09-06 12:44:44+02	2020-09-06 12:46:23+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-09-06 12:46:23+02	2020-09-06 12:47:26+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-09-06 12:47:26+02	2020-09-06 12:49:13+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-09-06 12:45:27+02	2020-09-06 12:50:07+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-06 12:50:08+02	2020-09-06 12:51:50+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-09-06 12:49:14+02	2020-09-06 12:53:34+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_4,1
2020-09-06 12:51:52+02	2020-09-06 12:53:37+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-09-06 12:08:48+02	2020-09-06 12:59:37+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-06 12:53:35+02	2020-09-06 13:01:01+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-10-06 08:42:08+02	2020-10-06 08:42:35+02	net	gssdp	\N	gssdp-1.2.3
2020-09-06 13:01:02+02	2020-09-06 13:04:47+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-09-06 13:04:47+02	2020-09-06 13:07:06+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-09-06 13:07:11+02	2020-09-06 13:08:34+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-06 13:08:35+02	2020-09-06 13:10:03+02	graphics	poppler-glib	\N	poppler-glib-20.08.0_1
2020-09-06 12:53:38+02	2020-09-06 13:10:16+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-09-06 13:10:17+02	2020-09-06 13:11:15+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-09-06 12:59:37+02	2020-09-06 13:11:24+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-06 13:11:16+02	2020-09-06 13:12:40+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-09-06 13:10:04+02	2020-09-06 13:13:20+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-09-06 13:12:41+02	2020-09-06 13:14:01+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-09-06 13:13:21+02	2020-09-06 13:14:08+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-09-06 13:11:24+02	2020-09-06 13:14:33+02	graphics	glew	\N	glew-2.2.0
2020-09-06 13:14:02+02	2020-09-06 13:14:58+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-06 13:14:09+02	2020-09-06 13:15:08+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-06 13:15:08+02	2020-09-06 13:16:54+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-09-06 13:14:59+02	2020-09-06 13:17:29+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-09-06 13:14:33+02	2020-09-06 13:17:30+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-09-06 13:17:33+02	2020-09-06 13:17:48+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-09-06 13:17:29+02	2020-09-06 13:18:46+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-09-06 13:17:48+02	2020-09-06 13:18:58+02	net	remmina	\N	remmina-1.3.9
2020-09-06 13:16:55+02	2020-09-06 13:19:07+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-06 13:18:58+02	2020-09-06 13:20:06+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-09-06 13:18:47+02	2020-09-06 13:20:28+02	security	gcr	\N	gcr-3.28.0_1
2020-09-06 13:20:07+02	2020-09-06 13:20:45+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-09-06 13:20:29+02	2020-09-06 13:21:18+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-06 13:20:46+02	2020-09-06 13:30:58+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-09-06 13:19:08+02	2020-09-06 13:31:10+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-06 13:21:18+02	2020-09-06 13:31:24+02	net	openmpi	\N	openmpi-4.0.5
2020-09-06 13:30:59+02	2020-09-06 13:31:41+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-09-06 13:31:25+02	2020-09-06 13:32:37+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-09-06 13:31:11+02	2020-09-06 13:32:43+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-09-06 13:31:41+02	2020-09-06 13:32:48+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-06 13:32:43+02	2020-09-06 13:34:31+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-06 13:34:32+02	2020-09-06 13:35:10+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-06 13:35:11+02	2020-09-06 13:37:51+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-06 13:37:51+02	2020-09-06 13:38:25+02	print	gl2ps	\N	gl2ps-1.4.0
2020-09-06 13:38:25+02	2020-09-06 13:39:59+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-09-06 13:40:00+02	2020-09-06 13:42:23+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-09-06 13:42:24+02	2020-09-06 13:43:47+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-09-06 13:43:48+02	2020-09-06 13:48:11+02	net	libmateweather	\N	libmateweather-1.24.0
2020-09-06 13:48:12+02	2020-09-06 13:50:14+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-09-06 13:32:42+02	2020-09-06 13:54:48+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-06 13:50:14+02	2020-09-06 14:03:30+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-09-06 14:03:31+02	2020-09-06 14:04:55+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-06 13:54:49+02	2020-09-06 14:05:20+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-06 14:04:56+02	2020-09-06 14:06:07+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-09-06 14:06:07+02	2020-09-06 14:06:32+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-09-06 13:32:48+02	2020-09-06 14:07:18+02	math	vtk8	\N	vtk8-8.2.0
2020-09-06 14:06:33+02	2020-09-06 14:07:51+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-09-06 14:07:51+02	2020-09-06 14:08:33+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-06 14:08:33+02	2020-09-06 14:11:32+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-09-06 14:05:21+02	2020-09-06 14:12:21+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-06 14:12:21+02	2020-09-06 14:13:41+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-09-06 14:07:19+02	2020-09-06 14:13:54+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-06 14:13:55+02	2020-09-06 14:14:07+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-09-06 14:14:08+02	2020-09-06 14:15:28+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-09-06 14:13:42+02	2020-09-06 14:16:32+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-06 14:15:28+02	2020-09-06 14:16:41+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-09-06 14:16:32+02	2020-09-06 14:17:37+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-09-06 14:16:42+02	2020-09-06 14:17:40+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-09-06 14:17:37+02	2020-09-06 14:18:51+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-09-06 14:17:41+02	2020-09-06 14:19:29+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-09-06 14:18:51+02	2020-09-06 14:19:50+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-09-06 14:19:30+02	2020-09-06 14:20:36+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-09-06 14:19:50+02	2020-09-06 14:22:38+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-09-06 14:20:37+02	2020-09-06 14:22:39+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-09-06 14:22:44+02	2020-09-06 14:23:13+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-09-06 14:23:13+02	2020-09-06 14:26:28+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-09-06 14:26:30+02	2020-09-06 14:28:55+02	graphics	poppler-utils	\N	poppler-utils-20.08.0_1
2020-09-06 14:22:44+02	2020-09-06 14:30:21+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-06 14:11:33+02	2020-09-06 14:30:48+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-09-06 14:30:49+02	2020-09-06 14:31:09+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-09-06 14:31:09+02	2020-09-06 14:31:55+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-06 14:31:56+02	2020-09-06 14:33:08+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-09-06 14:33:09+02	2020-09-06 14:34:41+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-06 14:34:42+02	2020-09-06 14:35:18+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-09-06 14:35:19+02	2020-09-06 14:36:43+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-06 14:36:43+02	2020-09-06 14:39:53+02	x11-fm	caja	\N	caja-1.22.3_1
2020-09-06 14:39:54+02	2020-09-06 14:42:37+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_4,1
2020-09-06 14:42:38+02	2020-09-06 14:46:15+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-06 14:46:15+02	2020-09-06 14:49:34+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-09-06 14:49:35+02	2020-09-06 14:50:36+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-06 14:30:23+02	2020-09-06 14:51:31+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-06 14:50:41+02	2020-09-06 14:52:27+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-06 14:51:36+02	2020-09-06 14:53:09+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-06 14:52:27+02	2020-09-06 14:54:20+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-09-06 14:53:09+02	2020-09-06 14:56:52+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-09-06 14:54:20+02	2020-09-06 14:57:02+02	graphics	gegl	\N	gegl-0.4.24_4
2020-09-06 14:56:53+02	2020-09-06 14:57:13+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-10-06 08:44:51+02	2020-10-06 08:45:44+02	x11	yelp	\N	yelp-3.38.0
2020-09-06 14:57:14+02	2020-09-06 14:59:48+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-06 14:59:49+02	2020-09-06 15:00:14+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-09-06 14:28:58+02	2020-09-06 15:00:35+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-09-06 15:00:15+02	2020-09-06 15:01:05+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-09-06 15:00:36+02	2020-09-06 15:01:34+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-06 15:01:06+02	2020-09-06 15:02:43+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-06 14:57:02+02	2020-09-06 15:02:58+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-09-06 15:01:35+02	2020-09-06 15:03:00+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-09-06 15:02:43+02	2020-09-06 15:03:28+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-06 15:02:58+02	2020-09-06 15:03:47+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-09-06 15:03:28+02	2020-09-06 15:04:21+02	x11	zenity	\N	zenity-3.32.0
2020-09-06 15:03:48+02	2020-09-06 15:04:41+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-09-06 15:03:01+02	2020-09-06 15:05:31+02	x11	xorg	\N	xorg-7.7_3
2020-09-06 15:04:22+02	2020-09-06 15:05:34+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-06 15:05:34+02	2020-09-06 15:06:06+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-09-06 15:05:32+02	2020-09-06 15:06:36+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-06 15:04:41+02	2020-09-06 15:07:45+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-09-06 15:06:06+02	2020-09-06 15:08:30+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-09-06 15:06:37+02	2020-09-06 15:09:40+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-09-06 15:07:46+02	2020-09-06 15:10:12+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-09-06 15:09:40+02	2020-09-06 15:14:32+02	editors	emacs	full	emacs-27.1,3
2020-09-06 15:14:33+02	2020-09-06 15:15:03+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-09-06 15:08:30+02	2020-09-06 15:15:55+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-09-06 15:15:55+02	2020-09-06 15:17:04+02	graphics	glfw	\N	glfw-3.3.2
2020-09-06 15:17:05+02	2020-09-06 15:20:57+02	graphics	movit	\N	movit-1.6.2_2
2020-09-06 15:10:12+02	2020-09-06 15:21:10+02	graphics	Coin	\N	Coin-4.0.0_1
2020-09-06 15:21:11+02	2020-09-06 15:21:44+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-09-06 15:21:45+02	2020-09-06 15:23:08+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-06 15:15:03+02	2020-09-06 15:23:12+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-09-06 15:23:13+02	2020-09-06 15:24:00+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-09-06 15:23:09+02	2020-09-06 15:24:02+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-09-06 15:20:57+02	2020-09-06 15:24:36+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-06 15:24:36+02	2020-09-06 15:25:22+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-06 15:24:00+02	2020-09-06 15:26:43+02	print	fontforge	\N	fontforge-20190801_2
2020-09-06 15:25:23+02	2020-09-06 15:26:59+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-06 15:26:59+02	2020-09-06 15:27:52+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-06 15:27:53+02	2020-09-06 15:28:51+02	x11	yelp	\N	yelp-3.28.1_2
2020-09-06 15:28:52+02	2020-09-06 15:29:34+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-06 15:24:02+02	2020-09-06 15:30:13+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-06 15:29:35+02	2020-09-06 15:30:18+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-06 15:30:13+02	2020-09-06 15:31:50+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-06 15:30:19+02	2020-09-06 15:37:08+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-06 15:31:51+02	2020-09-06 15:38:03+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-06 15:37:09+02	2020-09-06 15:38:26+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-09-06 15:38:04+02	2020-09-06 15:43:35+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-09-06 15:43:36+02	2020-09-06 15:47:59+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-06 15:48:00+02	2020-09-06 15:49:37+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-06 15:49:38+02	2020-09-06 15:51:46+02	x11-wm	marco	\N	marco-1.22.4
2020-09-06 15:38:27+02	2020-09-06 15:52:01+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-09-06 15:52:02+02	2020-09-06 15:52:40+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-09-06 15:51:47+02	2020-09-06 15:52:54+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-06 15:52:54+02	2020-09-06 15:54:02+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-06 15:52:42+02	2020-09-06 15:54:45+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-06 15:54:46+02	2020-09-06 15:56:57+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-06 15:54:03+02	2020-09-06 15:57:03+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-06 15:57:04+02	2020-09-06 15:58:21+02	devel	rebar3	\N	rebar3-3.14.1,1
2020-09-06 15:58:23+02	2020-09-06 16:05:24+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-06 15:26:45+02	2020-09-06 16:05:49+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-06 16:05:25+02	2020-09-06 16:06:07+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-09-06 16:05:52+02	2020-09-06 16:06:08+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-09-06 16:06:09+02	2020-09-06 16:07:11+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-09-06 16:07:12+02	2020-09-06 16:07:36+02	graphics	libgxps	\N	libgxps-0.3.1
2020-09-06 16:06:07+02	2020-09-06 16:07:42+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-06 15:56:59+02	2020-09-06 16:07:46+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-06 16:07:37+02	2020-09-06 16:07:58+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-09-06 16:07:46+02	2020-09-06 16:08:12+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-09-06 16:07:58+02	2020-09-06 16:08:51+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-06 16:07:43+02	2020-09-06 16:09:02+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-09-06 16:08:13+02	2020-09-06 16:09:03+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-09-06 16:09:04+02	2020-09-06 16:10:01+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-09-06 16:10:01+02	2020-09-06 16:11:20+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-06 16:11:20+02	2020-09-06 16:12:17+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-06 16:12:18+02	2020-09-06 16:13:44+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-09-06 16:13:45+02	2020-09-06 16:14:29+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-09-06 16:09:03+02	2020-09-06 16:14:59+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-06 16:14:59+02	2020-09-06 16:15:20+02	print	libspectre	\N	libspectre-0.2.9
2020-09-06 16:15:20+02	2020-09-06 16:17:22+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-06 16:14:29+02	2020-09-06 16:17:47+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-06 16:08:51+02	2020-09-06 16:26:20+02	math	R	\N	R-4.0.2
2020-09-06 16:26:21+02	2020-09-06 16:26:38+02	sysutils	hal-info	\N	hal-info-20091130
2020-09-06 16:17:23+02	2020-09-06 16:27:13+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-06 16:17:48+02	2020-09-06 16:27:25+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-06 16:27:14+02	2020-09-06 16:28:04+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-06 16:26:39+02	2020-09-06 16:29:08+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-09-06 16:27:25+02	2020-09-06 16:29:19+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-09-06 16:29:09+02	2020-09-06 16:29:47+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-06 16:28:04+02	2020-09-06 16:29:56+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-06 16:29:20+02	2020-09-06 16:30:13+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-06 16:29:48+02	2020-09-06 16:30:18+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-09-06 16:29:57+02	2020-09-06 16:31:07+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-06 16:30:19+02	2020-09-06 16:31:18+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-10-06 09:06:16+02	2020-10-06 09:06:51+02	net	gupnp	\N	gupnp-1.2.4
2020-09-06 16:30:14+02	2020-09-06 16:31:41+02	x11-servers	xephyr	\N	xephyr-1.20.8_4,1
2020-09-06 16:31:07+02	2020-09-06 16:31:52+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-06 16:31:20+02	2020-09-06 16:32:10+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-06 16:31:53+02	2020-09-06 16:32:54+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-06 16:31:41+02	2020-09-06 16:33:24+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-09-06 16:33:24+02	2020-09-06 16:33:42+02	audio	faudio	\N	FAudio-20.09
2020-09-06 16:33:43+02	2020-09-06 16:35:06+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-06 16:32:54+02	2020-09-06 16:35:14+02	archivers	engrampa	\N	engrampa-1.24.0
2020-09-06 16:35:07+02	2020-09-06 16:36:05+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-09-06 16:35:14+02	2020-09-06 16:36:27+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-09-06 16:32:11+02	2020-09-06 16:36:56+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-06 16:36:28+02	2020-09-06 16:37:15+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-06 16:37:15+02	2020-09-06 16:37:36+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-09-06 16:36:57+02	2020-09-06 16:37:51+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-06 16:37:36+02	2020-09-06 16:37:55+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-09-06 16:36:05+02	2020-09-06 16:38:37+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-06 16:37:56+02	2020-09-06 16:39:10+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-09-06 16:38:38+02	2020-09-06 16:40:12+02	devel	ode	\N	ode-0.13_4,1
2020-09-06 16:39:11+02	2020-09-06 16:40:43+02	devel	py-game	py37	py37-game-1.9.6_1
2020-09-06 16:40:13+02	2020-09-06 16:41:37+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-06 16:37:52+02	2020-09-06 16:42:26+02	devel	gsoap	\N	gsoap-2.8.106
2020-09-06 16:42:27+02	2020-09-06 16:45:35+02	editors	pluma	\N	pluma-1.24.0
2020-09-06 16:45:36+02	2020-09-06 16:48:11+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-09-06 16:48:11+02	2020-09-06 16:51:17+02	graphics	atril	\N	atril-1.22.3_8
2020-09-06 16:51:18+02	2020-09-06 16:53:45+02	graphics	eom	\N	eom-1.24.0
2020-09-06 16:53:46+02	2020-09-06 16:55:58+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-06 16:55:59+02	2020-09-06 16:56:33+02	graphics	gle	\N	gle-3.1.0_7
2020-09-06 16:56:34+02	2020-09-06 16:57:03+02	graphics	gltt	\N	gltt-2.5.2_14
2020-09-06 16:57:04+02	2020-09-06 16:57:49+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-06 16:57:49+02	2020-09-06 17:00:55+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-09-06 17:00:56+02	2020-09-06 17:02:07+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-09-06 16:40:44+02	2020-09-06 17:02:55+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-06 17:02:56+02	2020-09-06 17:03:29+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-09-06 16:41:37+02	2020-09-06 17:04:59+02	editors	libreoffice	\N	libreoffice-7.0.1.2
2020-09-06 17:02:08+02	2020-09-06 17:07:54+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-09-06 17:07:54+02	2020-09-06 17:09:37+02	math	mate-calc	\N	mate-calc-1.24.0
2020-09-06 17:09:38+02	2020-09-06 17:10:34+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-09-06 17:05:01+02	2020-09-06 17:11:14+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-06 17:03:29+02	2020-09-06 17:11:38+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-06 17:11:39+02	2020-09-06 17:12:24+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-09-06 17:11:14+02	2020-09-06 17:12:36+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-09-06 17:12:24+02	2020-09-06 17:14:34+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-09-06 17:14:34+02	2020-09-06 17:15:46+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-06 17:15:46+02	2020-09-06 17:16:50+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-06 17:16:51+02	2020-09-06 17:17:51+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-06 17:17:51+02	2020-09-06 17:18:45+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-06 17:12:36+02	2020-09-06 17:18:47+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-06 17:18:48+02	2020-09-06 17:19:39+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-06 17:18:46+02	2020-09-06 17:19:42+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-06 17:19:40+02	2020-09-06 17:20:30+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-06 17:19:43+02	2020-09-06 17:20:48+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-06 17:20:48+02	2020-09-06 17:21:39+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-09-06 17:20:30+02	2020-09-06 17:21:52+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-06 17:21:39+02	2020-09-06 17:22:46+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-09-06 17:21:53+02	2020-09-06 17:22:53+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-06 17:10:34+02	2020-09-06 17:24:00+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-09-06 17:22:47+02	2020-09-06 17:27:39+02	print	lilypond	\N	lilypond-2.20.0
2020-09-06 17:22:54+02	2020-09-06 17:28:28+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-06 17:24:01+02	2020-09-06 17:29:12+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-06 17:27:40+02	2020-09-06 17:29:15+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-06 17:29:16+02	2020-09-06 17:30:02+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-06 17:30:02+02	2020-09-06 17:31:23+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-09-06 17:31:23+02	2020-09-06 17:32:50+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-09-06 17:32:50+02	2020-09-06 17:33:37+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-06 17:28:28+02	2020-09-06 17:34:14+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-06 17:33:37+02	2020-09-06 17:34:25+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-06 17:34:15+02	2020-09-06 17:34:31+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-09-06 17:34:25+02	2020-09-06 17:34:37+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-09-06 17:34:31+02	2020-09-06 17:34:45+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-09-06 17:34:37+02	2020-09-06 17:35:31+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-06 17:34:45+02	2020-09-06 17:35:48+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-06 17:35:48+02	2020-09-06 17:36:53+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-06 17:36:54+02	2020-09-06 17:38:03+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-06 17:38:03+02	2020-09-06 17:39:42+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-06 17:35:32+02	2020-09-06 17:40:03+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-06 17:39:42+02	2020-09-06 17:40:59+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-09-06 17:40:04+02	2020-09-06 17:41:29+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-09-06 17:41:00+02	2020-09-06 17:41:54+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-06 17:41:29+02	2020-09-06 17:42:00+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-06 17:41:54+02	2020-09-06 17:42:58+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-06 17:42:59+02	2020-09-06 17:47:30+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-06 17:29:13+02	2020-09-06 17:49:27+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-06 17:49:29+02	2020-09-06 17:56:26+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-06 17:56:27+02	2020-09-06 17:59:46+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-06 17:42:00+02	2020-09-06 18:06:45+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-06 18:06:45+02	2020-09-06 18:08:29+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-06 17:47:31+02	2020-09-06 18:09:11+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-06 18:08:30+02	2020-09-06 18:09:52+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-06 18:09:53+02	2020-09-06 18:12:38+02	deskutils	calibre	\N	calibre-4.23.0
2020-09-06 18:09:12+02	2020-09-06 18:13:08+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-06 18:12:38+02	2020-09-06 18:13:28+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-06 18:13:29+02	2020-09-06 18:26:45+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-06 18:26:46+02	2020-09-06 18:28:11+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-06 18:13:08+02	2020-09-06 18:40:55+02	devel	RStudio	\N	RStudio-1.3.1073
2020-09-06 18:40:56+02	2020-09-06 18:41:17+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-09-06 18:41:18+02	2020-09-06 18:54:14+02	editors	codelite	\N	codelite-12.0_4
2020-09-06 18:54:15+02	2020-09-06 18:56:22+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-06 18:28:12+02	2020-09-06 18:57:58+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-06 18:56:23+02	2020-09-06 18:58:24+02	editors	vim	\N	vim-8.2.1558
2020-09-06 18:58:03+02	2020-09-06 19:08:46+02	emulators	qemu	\N	qemu-4.2.1
2020-09-06 19:08:47+02	2020-09-06 19:19:12+02	emulators	wine	\N	wine-5.0.2,1
2020-09-06 19:19:13+02	2020-09-06 19:36:23+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-06 19:36:24+02	2020-09-06 19:36:49+02	games	KnightCap	\N	knightcap-3.6_8
2020-09-06 19:36:50+02	2020-09-06 19:37:15+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-09-06 19:37:16+02	2020-09-06 19:40:09+02	games	alienarena	\N	alienarena-2013.766_9
2020-09-06 19:40:10+02	2020-09-06 19:42:07+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-09-06 19:42:08+02	2020-09-06 19:50:49+02	games	battletanks	\N	btanks-0.9.8083_9
2020-09-06 19:50:49+02	2020-09-06 19:51:20+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-09-06 19:51:20+02	2020-09-06 19:52:10+02	games	monsterz	\N	monsterz-0.7.1_15
2020-09-06 19:52:10+02	2020-09-06 19:52:48+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-09-06 19:52:48+02	2020-09-06 19:53:38+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-09-06 19:53:38+02	2020-09-06 19:54:39+02	games	xblast	\N	xblast-2.10.4_18
2020-09-06 19:54:41+02	2020-09-06 19:55:53+02	games	xboard	\N	xboard-4.9.1_1
2020-09-06 17:59:48+02	2020-09-06 19:56:15+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-06 19:55:54+02	2020-09-06 19:56:45+02	games	xcowsay	\N	xcowsay-1.5
2020-09-06 19:56:45+02	2020-09-06 19:59:12+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-06 19:59:16+02	2020-09-06 20:00:38+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-06 19:56:16+02	2020-09-06 20:01:44+02	graphics	blender	\N	blender-2.83.2_1
2020-09-06 20:00:39+02	2020-09-06 20:04:03+02	graphics	inkscape	\N	inkscape-1.0_4
2020-09-06 18:58:24+02	2020-09-06 20:04:56+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-09-06 20:04:57+02	2020-09-06 20:07:59+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-06 20:08:00+02	2020-09-06 20:09:25+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-06 20:04:07+02	2020-09-06 20:09:42+02	graphics	wings	\N	wings-2.2.5
2020-09-06 20:01:45+02	2020-09-06 20:11:36+02	graphics	krita	\N	krita-4.3.0_4
2020-09-06 20:11:37+02	2020-09-06 20:16:29+02	math	maxima	\N	maxima-5.44.0_3
2020-09-06 20:16:30+02	2020-09-06 20:17:18+02	multimedia	celluloid	\N	celluloid-0.19
2020-09-06 20:17:19+02	2020-09-06 20:17:33+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-09-06 20:17:34+02	2020-09-06 20:18:25+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-09-06 20:18:25+02	2020-09-06 20:21:15+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p7
2020-09-06 20:21:16+02	2020-09-06 20:21:51+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-09-06 20:21:52+02	2020-09-06 20:22:08+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-09-06 20:22:08+02	2020-09-06 20:23:53+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1
2020-09-06 20:23:54+02	2020-09-06 20:26:24+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-09-06 20:26:25+02	2020-09-06 20:29:28+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-06 20:09:25+02	2020-09-06 20:29:52+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-06 20:29:29+02	2020-09-06 20:31:23+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-09-06 20:29:52+02	2020-09-06 20:31:26+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-06 20:31:27+02	2020-09-06 20:32:10+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-06 20:32:10+02	2020-09-06 20:33:17+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-09-06 20:33:18+02	2020-09-06 20:34:24+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-09-06 20:34:25+02	2020-09-06 20:35:16+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-09-06 20:31:23+02	2020-09-06 20:35:36+02	net	wireshark	\N	wireshark-3.2.6
2020-09-06 20:35:16+02	2020-09-06 20:35:59+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-09-06 20:35:36+02	2020-09-06 20:36:17+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-09-06 20:09:42+02	2020-09-06 20:36:44+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-09-06 20:35:59+02	2020-09-06 20:36:46+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-09-06 20:36:18+02	2020-09-06 20:37:07+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-09-06 20:36:46+02	2020-09-06 20:37:42+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-09-06 20:37:08+02	2020-09-06 20:38:02+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-09-06 20:36:45+02	2020-09-06 20:38:11+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-09-06 20:38:02+02	2020-09-06 20:39:11+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-09-06 20:37:42+02	2020-09-06 20:40:11+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-09-06 20:38:11+02	2020-09-06 20:41:23+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-06 20:40:11+02	2020-09-06 20:46:22+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-09-06 20:46:23+02	2020-09-06 20:47:40+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-09-06 20:41:23+02	2020-09-06 20:47:40+02	print	texlive-full	\N	texlive-full-20150521
2020-09-06 20:47:41+02	2020-09-06 20:48:06+02	sysutils	flashrom	\N	flashrom-1.2
2020-09-06 20:48:06+02	2020-09-06 20:48:43+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-06 20:47:41+02	2020-09-06 20:48:49+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-06 20:48:44+02	2020-09-06 20:49:58+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-06 20:48:49+02	2020-09-06 20:49:59+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-06 20:50:01+02	2020-09-06 20:50:23+02	sysutils	x86info	\N	x86info-1.31.s03
2020-09-06 20:49:59+02	2020-09-06 20:53:38+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-09-06 20:53:39+02	2020-09-06 20:53:59+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-09-06 20:53:59+02	2020-09-06 20:54:14+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-09-06 20:54:14+02	2020-09-06 20:54:30+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-09-06 20:54:31+02	2020-09-06 20:54:49+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-09-06 20:54:50+02	2020-09-06 20:55:06+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-09-06 20:55:06+02	2020-09-06 20:55:26+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-09-06 20:55:26+02	2020-09-06 20:55:42+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-09-06 20:55:45+02	2020-09-06 20:55:58+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-09-06 20:55:58+02	2020-09-06 20:56:12+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-09-06 20:56:12+02	2020-09-06 20:56:31+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-09-06 20:39:12+02	2020-09-06 20:56:55+02	print	lyx	\N	lyx-2.3.4.2_1
2020-09-06 20:56:32+02	2020-09-06 20:58:53+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-09-06 20:56:56+02	2020-09-06 20:59:08+02	x11	mate	\N	mate-1.22.2
2020-09-06 20:58:54+02	2020-09-06 21:00:43+02	x11	xlockmore	\N	xlockmore-5.65
2020-09-06 20:59:09+02	2020-09-06 21:01:52+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-06 21:01:53+02	2020-09-06 21:03:03+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-06 21:03:04+02	2020-09-06 21:03:20+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-09-06 21:03:20+02	2020-09-06 21:03:53+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-09-06 21:00:44+02	2020-09-06 21:07:15+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-06 21:07:16+02	2020-09-06 21:08:00+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-09-06 21:03:54+02	2020-09-06 21:30:44+02	editors	vscode	\N	vscode-1.46.1
2020-09-06 20:50:24+02	2020-09-06 21:31:53+02	www	firefox	\N	firefox-80.0.1,1
2020-09-06 21:31:55+02	2020-09-06 21:49:27+02	local	E590T-localbase	\N	E590T-localbase-2020082800
2020-09-06 21:49:44+02	2020-09-06 21:50:45+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-06 21:49:44+02	2020-09-06 21:51:29+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-06 21:51:30+02	2020-09-06 21:51:37+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-07 06:37:47+02	2020-09-07 06:37:55+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3_1
2020-09-07 06:37:55+02	2020-09-07 06:38:20+02	security	mbedtls	\N	mbedtls-2.16.8
2020-09-07 06:38:20+02	2020-09-07 06:38:33+02	textproc	py-scour	py37	py37-scour-0.38.1
2020-09-07 06:37:47+02	2020-09-07 06:38:59+02	print	texinfo	\N	texinfo-6.7_4,1
2020-09-07 06:38:34+02	2020-09-07 06:40:14+02	devel	mercurial	py37	mercurial-5.5
2020-09-07 06:40:15+02	2020-09-07 06:40:25+02	sysutils	bpytop	\N	bpytop-1.0.24
2020-09-07 06:38:59+02	2020-09-07 06:42:16+02	security	gnutls	\N	gnutls-3.6.15
2020-09-07 06:42:16+02	2020-09-07 06:42:45+02	net	glib-networking	\N	glib-networking-2.56.1_3
2020-09-07 06:42:16+02	2020-09-07 06:43:41+02	print	cups	\N	cups-2.3.3_1
2020-09-07 06:42:45+02	2020-09-07 06:44:16+02	devel	libsoup	\N	libsoup-2.62.3
2020-09-07 06:43:42+02	2020-09-07 06:47:59+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_8
2020-09-07 06:44:17+02	2020-09-07 06:50:17+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-09-07 06:48:00+02	2020-09-07 06:51:32+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-07 06:51:33+02	2020-09-07 06:52:43+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-07 06:50:17+02	2020-09-07 06:54:01+02	print	texlive-base	\N	texlive-base-20150521_54
2020-09-07 06:52:44+02	2020-09-07 07:00:45+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_1,1
2020-09-07 07:00:46+02	2020-09-07 07:02:00+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-07 07:02:01+02	2020-09-07 07:03:15+02	net	geoclue	\N	geoclue-2.5.5
2020-09-07 06:37:47+02	2020-09-07 07:08:27+02	math	openblas	\N	openblas-0.3.10,1
2020-09-07 07:08:27+02	2020-09-07 07:08:53+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-07 07:03:16+02	2020-09-07 07:11:05+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-07 07:08:54+02	2020-09-07 07:11:08+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-07 07:11:09+02	2020-09-07 07:11:40+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-07 07:11:40+02	2020-09-07 07:12:23+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-09-07 07:12:24+02	2020-09-07 07:13:36+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-07 06:54:01+02	2020-09-07 07:14:03+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-07 07:13:37+02	2020-09-07 07:14:28+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-07 07:11:06+02	2020-09-07 07:15:35+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-07 07:14:04+02	2020-09-07 07:16:40+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-07 07:16:41+02	2020-09-07 07:17:51+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-09-07 07:17:52+02	2020-09-07 07:18:50+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-07 07:18:51+02	2020-09-07 07:20:10+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-09-07 07:14:29+02	2020-09-07 07:20:30+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-07 07:20:11+02	2020-09-07 07:21:13+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-07 07:21:13+02	2020-09-07 07:22:13+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-07 07:22:13+02	2020-09-07 07:23:12+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-07 07:20:31+02	2020-09-07 07:24:00+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-07 07:23:13+02	2020-09-07 07:24:08+02	net	remmina	\N	remmina-1.3.9
2020-09-07 07:24:09+02	2020-09-07 07:25:53+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-07 07:24:01+02	2020-09-07 07:25:58+02	security	gcr	\N	gcr-3.28.0_1
2020-09-07 07:25:54+02	2020-09-07 07:26:37+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-07 07:25:59+02	2020-09-07 07:27:04+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-07 07:27:04+02	2020-09-07 07:29:04+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-07 07:26:37+02	2020-09-07 07:29:24+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-07 07:29:05+02	2020-09-07 07:29:41+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-07 07:29:25+02	2020-09-07 07:30:17+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-07 07:29:42+02	2020-09-07 07:30:41+02	security	gnupg	\N	gnupg-2.2.23
2020-09-07 07:30:41+02	2020-09-07 07:34:51+02	net	libmateweather	\N	libmateweather-1.24.0
2020-09-07 07:34:52+02	2020-09-07 07:35:16+02	security	gpgme	\N	gpgme-1.14.0
2020-09-07 07:30:17+02	2020-09-07 07:36:11+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-07 07:36:12+02	2020-09-07 07:37:23+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-07 07:37:24+02	2020-09-07 07:39:51+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-07 07:39:52+02	2020-09-07 07:41:04+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-07 07:41:04+02	2020-09-07 07:42:15+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-09-07 07:42:16+02	2020-09-07 07:42:45+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-09-07 07:35:16+02	2020-09-07 07:43:39+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-07 07:43:40+02	2020-09-07 07:46:24+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-07 07:46:25+02	2020-09-07 07:54:03+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-07 07:42:47+02	2020-09-07 08:06:40+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-09-07 08:06:42+02	2020-09-07 08:08:13+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-09-07 08:08:14+02	2020-09-07 08:09:08+02	devel	librest	\N	rest-0.7.93
2020-09-07 08:09:09+02	2020-09-07 08:10:05+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-09-07 08:10:06+02	2020-09-07 08:11:00+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-07 08:11:00+02	2020-09-07 08:12:19+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-07 07:54:04+02	2020-09-07 08:13:45+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-07 08:12:20+02	2020-09-07 08:14:07+02	x11-fm	caja	\N	caja-1.22.3_1
2020-09-07 08:13:46+02	2020-09-07 08:15:23+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-07 08:14:08+02	2020-09-07 08:15:30+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-09-07 08:15:24+02	2020-09-07 08:16:14+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-07 08:15:31+02	2020-09-07 08:16:43+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-07 08:16:15+02	2020-09-07 08:17:22+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-07 08:16:44+02	2020-09-07 08:17:50+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-09-07 08:17:51+02	2020-09-07 08:19:47+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-07 08:17:23+02	2020-09-07 08:20:26+02	mail	mailutils	\N	mailutils-3.10
2020-09-07 08:20:26+02	2020-09-07 08:21:23+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-09-07 08:21:23+02	2020-09-07 08:22:15+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-07 08:22:15+02	2020-09-07 08:23:43+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-07 08:23:44+02	2020-09-07 08:25:24+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-09-07 08:25:25+02	2020-09-07 08:26:34+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-07 08:26:35+02	2020-09-07 08:28:05+02	x11	zenity	\N	zenity-3.32.0
2020-09-07 08:28:06+02	2020-09-07 08:32:49+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-09-07 08:32:50+02	2020-09-07 08:34:57+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-07 08:34:57+02	2020-09-07 08:36:25+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-07 08:36:25+02	2020-09-07 08:42:22+02	editors	emacs	full	emacs-27.1,3
2020-09-07 08:19:48+02	2020-09-07 08:42:27+02	net	samba410	\N	samba410-4.10.17
2020-09-07 08:42:27+02	2020-09-07 08:47:16+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-07 07:15:35+02	2020-09-07 08:47:41+02	math	suitesparse	\N	suitesparse-5.8.1
2020-09-07 08:47:16+02	2020-09-07 08:48:00+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-07 08:48:00+02	2020-09-07 08:48:20+02	net	gssdp	\N	gssdp-1.2.3
2020-09-07 08:42:22+02	2020-09-07 08:48:20+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-09-07 08:48:21+02	2020-09-07 08:49:14+02	print	cups-filters	\N	cups-filters-1.27.5_2
2020-09-07 08:48:21+02	2020-09-07 08:49:22+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-09-07 08:47:42+02	2020-09-07 08:50:42+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-09-07 08:49:14+02	2020-09-07 08:51:28+02	print	fontforge	\N	fontforge-20190801_2
2020-09-07 08:50:42+02	2020-09-07 08:55:17+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-09-07 08:49:22+02	2020-09-07 08:55:24+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-07 08:55:18+02	2020-09-07 08:55:59+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-07 08:55:25+02	2020-09-07 08:56:45+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-07 08:56:46+02	2020-09-07 08:57:38+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-07 08:57:39+02	2020-09-07 08:58:49+02	x11	yelp	\N	yelp-3.28.1_2
2020-09-07 08:58:50+02	2020-09-07 08:59:39+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-07 08:59:39+02	2020-09-07 09:00:27+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-07 09:00:27+02	2020-09-07 09:01:54+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-07 09:01:55+02	2020-09-07 09:03:30+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-07 09:03:31+02	2020-09-07 09:04:58+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-07 08:51:28+02	2020-09-07 09:07:32+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-09-07 09:04:59+02	2020-09-07 09:07:33+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-07 09:07:32+02	2020-09-07 09:08:42+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-07 09:07:33+02	2020-09-07 09:08:51+02	x11-wm	marco	\N	marco-1.22.4
2020-09-07 09:08:42+02	2020-09-07 09:09:36+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-07 09:09:37+02	2020-09-07 09:10:46+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-07 09:08:54+02	2020-09-07 09:10:47+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-07 09:10:48+02	2020-09-07 09:12:47+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-07 09:10:46+02	2020-09-07 09:13:23+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-07 09:13:24+02	2020-09-07 09:17:33+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-07 09:17:34+02	2020-09-07 09:19:45+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-07 09:19:47+02	2020-09-07 09:20:51+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-07 09:12:47+02	2020-09-07 09:23:53+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-07 08:55:59+02	2020-09-07 09:26:48+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-07 09:26:52+02	2020-09-07 09:28:12+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-07 09:28:13+02	2020-09-07 09:30:26+02	net	freerdp	\N	freerdp-2.2.0
2020-09-07 09:23:56+02	2020-09-07 09:30:28+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-07 09:30:27+02	2020-09-07 09:30:52+02	net	gupnp	\N	gupnp-1.1.1_1
2020-09-07 09:30:29+02	2020-09-07 09:31:11+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-07 09:31:11+02	2020-09-07 09:31:51+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-09-07 09:30:53+02	2020-09-07 09:31:54+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-09-07 09:31:55+02	2020-09-07 09:32:11+02	print	libspectre	\N	libspectre-0.2.9
2020-09-07 09:32:12+02	2020-09-07 09:34:01+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-07 09:20:53+02	2020-09-07 09:34:11+02	math	R	\N	R-4.0.2
2020-09-07 09:31:51+02	2020-09-07 09:34:32+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-07 09:34:33+02	2020-09-07 09:35:08+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-09-07 09:35:09+02	2020-09-07 09:37:17+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-09-07 09:37:17+02	2020-09-07 09:37:58+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-07 09:34:01+02	2020-09-07 09:38:45+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-07 09:34:12+02	2020-09-07 09:38:47+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-07 09:37:58+02	2020-09-07 09:39:21+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-09-07 09:38:48+02	2020-09-07 09:39:24+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-07 09:39:24+02	2020-09-07 09:39:50+02	www	youtube_dl	\N	youtube_dl-2020.07.28
2020-09-07 09:38:46+02	2020-09-07 09:40:12+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-07 09:39:22+02	2020-09-07 09:40:13+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-07 09:39:50+02	2020-09-07 09:40:46+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-07 09:40:13+02	2020-09-07 09:40:51+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-07 09:40:13+02	2020-09-07 09:41:00+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-09-07 09:40:46+02	2020-09-07 09:41:29+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-07 09:40:51+02	2020-09-07 09:41:42+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-07 09:41:42+02	2020-09-07 09:42:45+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-07 09:41:29+02	2020-09-07 09:43:25+02	archivers	engrampa	\N	engrampa-1.24.0
2020-09-07 09:42:46+02	2020-09-07 09:44:02+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-09-07 09:44:03+02	2020-09-07 09:45:00+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-07 09:41:01+02	2020-09-07 09:45:32+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-07 09:45:02+02	2020-09-07 09:45:52+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-07 09:45:32+02	2020-09-07 09:46:09+02	devel	py-game	py37	py37-game-1.9.6_1
2020-09-07 09:43:26+02	2020-09-07 09:46:11+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-07 09:45:52+02	2020-09-07 09:47:03+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-07 09:47:03+02	2020-09-07 09:49:48+02	editors	pluma	\N	pluma-1.24.0
2020-09-07 09:49:49+02	2020-09-07 09:51:26+02	graphics	atril	\N	atril-1.22.3_8
2020-09-07 09:51:27+02	2020-09-07 09:53:13+02	graphics	eom	\N	eom-1.24.0
2020-09-07 09:53:14+02	2020-09-07 09:54:56+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-07 09:54:57+02	2020-09-07 09:55:57+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-07 09:55:57+02	2020-09-07 09:57:48+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-09-07 09:57:48+02	2020-09-07 09:58:27+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-09-07 09:46:10+02	2020-09-07 10:04:32+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-07 09:58:28+02	2020-09-07 10:05:04+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-07 09:46:11+02	2020-09-07 10:05:58+02	editors	libreoffice	\N	libreoffice-7.0.1.2
2020-09-07 10:05:59+02	2020-09-07 10:07:58+02	math	mate-calc	\N	mate-calc-1.24.0
2020-09-07 10:07:59+02	2020-09-07 10:09:02+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-09-07 10:04:33+02	2020-09-07 10:11:45+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-07 10:11:46+02	2020-09-07 10:13:24+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-09-07 10:09:02+02	2020-09-07 10:14:44+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-09-07 10:13:25+02	2020-09-07 10:15:35+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-09-07 10:15:35+02	2020-09-07 10:16:43+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-07 10:16:44+02	2020-09-07 10:17:31+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-07 10:17:32+02	2020-09-07 10:18:39+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-07 10:18:39+02	2020-09-07 10:19:33+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-06 09:52:12+02	2020-10-06 09:54:14+02	graphics	eom	\N	eom-1.24.1
2020-09-07 10:19:33+02	2020-09-07 10:20:33+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-07 10:14:45+02	2020-09-07 10:20:36+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-07 10:20:33+02	2020-09-07 10:21:19+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-07 10:20:37+02	2020-09-07 10:21:23+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-07 10:21:19+02	2020-09-07 10:22:25+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-07 10:21:24+02	2020-09-07 10:22:53+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-07 10:22:27+02	2020-09-07 10:23:21+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-09-07 10:22:54+02	2020-09-07 10:24:10+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-09-07 10:23:21+02	2020-09-07 10:24:31+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-07 10:24:11+02	2020-09-07 10:30:33+02	print	lilypond	\N	lilypond-2.20.0
2020-09-07 10:24:32+02	2020-09-07 10:31:26+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-07 10:31:27+02	2020-09-07 10:33:26+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-07 10:30:34+02	2020-09-07 10:36:45+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-07 10:33:27+02	2020-09-07 10:40:48+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-07 10:40:49+02	2020-09-07 10:41:27+02	print	transfig	\N	transfig-3.2.5e_4
2020-09-07 10:41:28+02	2020-09-07 10:42:20+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-07 10:42:20+02	2020-09-07 10:44:02+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-09-07 10:44:03+02	2020-09-07 10:46:00+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-09-07 10:46:01+02	2020-09-07 10:47:05+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-07 10:47:06+02	2020-09-07 10:47:54+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-07 10:47:54+02	2020-09-07 10:49:03+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-07 10:49:03+02	2020-09-07 10:50:21+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-07 10:50:22+02	2020-09-07 10:56:27+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-07 10:56:28+02	2020-09-07 10:57:40+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-07 10:57:41+02	2020-09-07 10:58:50+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-07 10:36:46+02	2020-09-07 10:59:28+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-07 10:58:50+02	2020-09-07 11:00:47+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-07 10:59:29+02	2020-09-07 11:00:57+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-09-07 11:00:58+02	2020-09-07 11:01:37+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-07 11:00:47+02	2020-09-07 11:01:48+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-07 11:01:38+02	2020-09-07 11:03:08+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-07 11:03:09+02	2020-09-07 11:08:44+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-07 11:01:48+02	2020-09-07 11:15:11+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-07 11:15:12+02	2020-09-07 11:20:39+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-07 11:20:40+02	2020-09-07 11:23:01+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-07 10:05:04+02	2020-09-07 11:25:21+02	lang	julia11	\N	julia11-1.1.1_5
2020-09-07 11:08:45+02	2020-09-07 11:26:00+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-07 11:25:22+02	2020-09-07 11:26:49+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-07 11:26:01+02	2020-09-07 11:27:19+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-07 11:27:20+02	2020-09-07 11:29:39+02	deskutils	calibre	\N	calibre-4.23.0
2020-09-07 11:26:50+02	2020-09-07 11:30:16+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-07 11:29:41+02	2020-09-07 11:30:29+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-07 11:30:29+02	2020-09-07 11:41:45+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-07 11:41:45+02	2020-09-07 11:42:59+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-07 11:30:17+02	2020-09-07 11:55:36+02	devel	RStudio	\N	RStudio-1.3.1073
2020-09-07 11:55:37+02	2020-09-07 12:07:07+02	editors	codelite	\N	codelite-12.0_4
2020-09-07 12:07:08+02	2020-09-07 12:08:55+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-07 11:43:00+02	2020-09-07 12:10:03+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-07 12:08:55+02	2020-09-07 12:10:40+02	editors	vim	\N	vim-8.2.1558
2020-09-07 12:10:06+02	2020-09-07 12:19:23+02	emulators	qemu	\N	qemu-4.2.1
2020-09-07 12:19:24+02	2020-09-07 12:28:18+02	emulators	wine	\N	wine-5.0.2,1
2020-09-07 12:28:20+02	2020-09-07 12:44:07+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-07 12:44:07+02	2020-09-07 12:44:48+02	games	monsterz	\N	monsterz-0.7.1_15
2020-09-07 12:44:50+02	2020-09-07 12:45:39+02	games	xcowsay	\N	xcowsay-1.5
2020-09-07 12:45:39+02	2020-09-07 12:51:21+02	graphics	blender	\N	blender-2.83.2_1
2020-09-07 12:51:22+02	2020-09-07 12:53:36+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-07 12:53:36+02	2020-09-07 12:54:48+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-07 12:54:49+02	2020-09-07 12:58:56+02	graphics	inkscape	\N	inkscape-1.0_4
2020-09-07 11:23:02+02	2020-09-07 13:07:48+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-07 12:58:57+02	2020-09-07 13:08:44+02	graphics	krita	\N	krita-4.3.0_4
2020-09-07 12:10:41+02	2020-09-07 13:09:03+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-09-07 13:08:44+02	2020-09-07 13:09:35+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-07 13:09:04+02	2020-09-07 13:09:53+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-07 13:07:49+02	2020-09-07 13:10:10+02	graphics	wings	\N	wings-2.2.5
2020-09-07 13:10:10+02	2020-09-07 13:10:38+02	lang	julia	\N	julia-1.0.3_3
2020-09-07 13:10:38+02	2020-09-07 13:15:06+02	math	maxima	\N	maxima-5.44.0_3
2020-09-07 13:15:06+02	2020-09-07 13:15:53+02	multimedia	celluloid	\N	celluloid-0.19
2020-09-07 13:15:53+02	2020-09-07 13:17:36+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1
2020-09-07 13:17:36+02	2020-09-07 13:19:40+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-09-07 13:19:40+02	2020-09-07 13:22:06+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-07 13:22:07+02	2020-09-07 13:22:32+02	net	rdesktop	\N	rdesktop-1.9.0
2020-09-07 13:22:33+02	2020-09-07 13:23:48+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-07 13:23:48+02	2020-09-07 13:27:30+02	net	wireshark	\N	wireshark-3.2.6
2020-09-07 13:09:35+02	2020-09-07 13:27:48+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-07 13:27:30+02	2020-09-07 13:28:09+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-07 13:27:49+02	2020-09-07 13:28:29+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-09-07 13:28:09+02	2020-09-07 13:28:48+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-09-07 13:28:30+02	2020-09-07 13:29:09+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-09-07 13:28:49+02	2020-09-07 13:29:28+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-09-07 13:29:10+02	2020-09-07 13:29:49+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-09-07 13:29:29+02	2020-09-07 13:30:10+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-09-07 13:29:49+02	2020-09-07 13:30:29+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-09-07 13:30:10+02	2020-09-07 13:31:12+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-09-07 13:30:30+02	2020-09-07 13:31:14+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-09-07 13:31:13+02	2020-09-07 13:31:51+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-09-07 13:31:14+02	2020-09-07 13:31:55+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-09-07 13:31:55+02	2020-09-07 13:32:25+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-10-06 10:21:52+02	2020-10-06 10:24:24+02	cad	cura	\N	Cura-4.7.1,2
2020-09-07 13:09:54+02	2020-09-07 13:32:34+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-09-07 13:32:25+02	2020-09-07 13:32:57+02	print	gv	\N	gv-3.7.4_5
2020-09-07 13:31:52+02	2020-09-07 13:33:51+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-07 13:33:52+02	2020-09-07 13:34:47+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-09-07 13:34:47+02	2020-09-07 13:35:42+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-07 13:35:42+02	2020-09-07 13:36:13+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-07 13:32:34+02	2020-09-07 13:36:23+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-09-07 13:32:58+02	2020-09-07 13:36:42+02	print	texlive-full	\N	texlive-full-20150521
2020-09-07 13:36:13+02	2020-09-07 13:37:01+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-07 13:36:24+02	2020-09-07 13:37:09+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-07 13:37:01+02	2020-09-07 13:38:12+02	x11	mate	\N	mate-1.22.2
2020-09-07 13:37:09+02	2020-09-07 13:38:34+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-07 13:38:34+02	2020-09-07 13:39:14+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-07 13:38:13+02	2020-09-07 13:42:47+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-07 13:42:48+02	2020-09-07 13:44:02+02	graphics	xfig	\N	xfig-3.2.7a
2020-09-07 13:39:14+02	2020-09-07 14:03:04+02	editors	vscode	\N	vscode-1.46.1
2020-09-07 13:36:43+02	2020-09-07 14:08:12+02	www	firefox	\N	firefox-80.0.1,1
2020-09-07 14:46:45+02	2020-09-07 14:47:40+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-07 14:46:45+02	2020-09-07 14:48:21+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-07 14:48:22+02	2020-09-07 14:48:29+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-07 19:24:12+02	2020-09-07 19:26:11+02	ports-mgmt	pkg	\N	pkg-1.15.1
2020-09-07 19:39:29+02	2020-09-07 19:39:36+02	devel	robin-map	\N	robin-map-0.6.3
2020-09-07 19:39:36+02	2020-09-07 19:39:42+02	security	base-audit	\N	base-audit-0.4
2020-09-07 19:39:43+02	2020-09-07 19:39:51+02	sysutils	bpytop	\N	bpytop-1.0.25
2020-09-07 19:39:30+02	2020-09-07 19:40:04+02	audio	opusfile	\N	opusfile-0.12
2020-09-07 19:39:30+02	2020-09-07 19:40:12+02	print	cups-filters	\N	cups-filters-1.28.1
2020-09-07 19:40:04+02	2020-09-07 19:41:16+02	audio	sox	\N	sox-14.4.2_4
2020-09-07 19:41:17+02	2020-09-07 19:42:11+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-07 19:40:12+02	2020-09-07 19:42:56+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-07 19:42:12+02	2020-09-07 19:43:02+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-07 19:42:57+02	2020-09-07 19:43:48+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-07 19:43:49+02	2020-09-07 19:44:43+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-07 19:43:02+02	2020-09-07 19:44:55+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-07 20:25:47+02	2020-09-07 20:26:40+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-07 20:25:47+02	2020-09-07 20:27:24+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-07 20:27:24+02	2020-09-07 20:27:31+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-07 21:32:32+02	2020-09-07 21:32:38+02	java	javavmwrapper	\N	javavmwrapper-2.7.6
2020-09-07 21:32:32+02	2020-09-07 21:32:50+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.6
2020-09-07 21:32:38+02	2020-09-07 22:19:12+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-09-07 22:19:13+02	2020-09-07 22:19:45+02	devel	apache-ant	\N	apache-ant-1.10.8
2020-09-07 21:32:38+02	2020-09-07 22:20:46+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-09-07 22:19:45+02	2020-09-07 22:20:54+02	devel	gradle5	\N	gradle5-5.0
2020-09-07 21:32:50+02	2020-09-07 22:20:58+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-09-07 22:20:55+02	2020-09-07 22:21:23+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-09-07 22:20:47+02	2020-09-07 22:22:01+02	textproc	fop	\N	fop-2.3
2020-09-07 22:22:02+02	2020-09-07 23:01:00+02	devel	ghidra	\N	ghidra-9.1
2020-09-07 23:39:51+02	2020-09-07 23:40:45+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-07 23:39:51+02	2020-09-07 23:41:27+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-07 23:41:27+02	2020-09-07 23:41:34+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-08 07:25:35+02	2020-09-08 07:25:55+02	devel	libfmt	\N	libfmt-7.0.3
2020-09-08 07:25:35+02	2020-09-08 07:26:28+02	print	freetype2	\N	freetype2-2.10.2_1
2020-09-08 07:26:29+02	2020-09-08 07:27:18+02	x11-fonts	fontconfig	\N	fontconfig-2.13.92_2,1
2020-09-08 07:27:19+02	2020-09-08 07:28:21+02	graphics	cairo	\N	cairo-1.16.0,2
2020-09-08 07:28:21+02	2020-09-08 07:28:33+02	x11-fonts	mkfontscale	\N	mkfontscale-1.2.1
2020-09-08 07:25:56+02	2020-09-08 07:28:35+02	lang	cython	py38	py38-cython-0.29.15
2020-09-08 07:28:36+02	2020-09-08 07:28:47+02	x11-fonts	dejavu	\N	dejavu-2.37_1
2020-09-08 07:28:48+02	2020-09-08 07:28:57+02	x11-fonts	font-bh-ttf	\N	font-bh-ttf-1.0.3_4
2020-09-08 07:28:57+02	2020-09-08 07:29:09+02	x11-fonts	font-misc-ethiopic	\N	font-misc-ethiopic-1.0.4
2020-09-08 07:25:35+02	2020-09-08 07:29:12+02	lang	php73	\N	php73-7.3.22
2020-09-08 07:29:10+02	2020-09-08 07:29:24+02	x11-fonts	font-misc-meltho	\N	font-misc-meltho-1.0.3_4
2020-09-08 07:29:12+02	2020-09-08 07:29:34+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-09-08 07:29:24+02	2020-09-08 07:29:43+02	x11-fonts	xorg-fonts-truetype	\N	xorg-fonts-truetype-7.7_1
2020-09-08 07:29:44+02	2020-09-08 07:30:27+02	graphics	gd	\N	libgd-2.3.0,1
2020-09-08 07:29:34+02	2020-09-08 07:30:55+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_8
2020-09-08 07:30:56+02	2020-09-08 07:31:36+02	graphics	poppler	\N	poppler-20.08.0_1
2020-09-08 07:28:34+02	2020-09-08 07:32:18+02	print	harfbuzz	\N	harfbuzz-2.7.2
2020-09-08 07:31:37+02	2020-09-08 07:32:30+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-09-08 07:30:28+02	2020-09-08 07:32:56+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-09-08 07:32:57+02	2020-09-08 07:33:20+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.2
2020-09-08 07:33:21+02	2020-09-08 07:34:05+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-09-08 07:32:19+02	2020-09-08 07:34:16+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-09-08 07:32:31+02	2020-09-08 07:34:23+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-09-08 07:34:23+02	2020-09-08 07:35:39+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-09-08 07:34:17+02	2020-09-08 07:36:39+02	graphics	graphviz	\N	graphviz-2.44.1_1
2020-09-08 07:34:05+02	2020-09-08 07:36:55+02	print	texlive-base	\N	texlive-base-20150521_54
2020-09-08 07:35:41+02	2020-09-08 07:37:27+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-09-08 07:36:40+02	2020-09-08 07:37:58+02	lang	vala	\N	vala-0.40.15,1
2020-09-08 07:37:59+02	2020-09-08 07:38:54+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-09-08 07:36:56+02	2020-09-08 07:40:10+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-08 07:37:28+02	2020-09-08 07:40:26+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-09-08 07:40:27+02	2020-09-08 07:43:31+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-09-08 07:38:56+02	2020-09-08 07:43:42+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-09-08 07:43:32+02	2020-09-08 07:44:12+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-09-08 07:43:43+02	2020-09-08 07:44:31+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-08 07:44:12+02	2020-09-08 07:44:32+02	graphics	frei0r	\N	frei0r-1.6.1
2020-09-08 07:44:32+02	2020-09-08 07:44:45+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-09-08 07:44:32+02	2020-09-08 07:44:51+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-09-08 07:44:45+02	2020-09-08 07:45:05+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-08 07:45:05+02	2020-09-08 07:46:05+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-08 07:46:06+02	2020-09-08 07:47:13+02	graphics	graphene	\N	graphene-1.10.0
2020-09-08 07:47:14+02	2020-09-08 07:48:20+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-08 07:44:51+02	2020-09-08 07:51:25+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_2,1
2020-09-08 07:51:26+02	2020-09-08 07:52:12+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-09-08 07:52:12+02	2020-09-08 07:52:53+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-09-08 07:52:54+02	2020-09-08 07:53:35+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-09-08 07:53:35+02	2020-09-08 07:54:00+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-08 07:54:01+02	2020-09-08 07:54:58+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-09-08 07:54:59+02	2020-09-08 07:55:15+02	print	libraqm	\N	libraqm-0.6.0
2020-09-08 07:48:21+02	2020-09-08 07:55:52+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-08 07:55:15+02	2020-09-08 07:56:04+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-09-08 07:56:05+02	2020-09-08 07:56:43+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-09-08 07:56:43+02	2020-09-08 07:57:20+02	x11-fonts	libXfont2	\N	libXfont2-2.0.4
2020-09-08 07:55:53+02	2020-09-08 07:58:10+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-08 07:40:11+02	2020-09-08 07:58:29+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-08 07:58:11+02	2020-09-08 07:58:37+02	www	php73-session	\N	php73-session-7.3.22
2020-09-08 07:58:30+02	2020-09-08 08:01:34+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-08 08:01:35+02	2020-09-08 08:01:59+02	archivers	php73-zlib	\N	php73-zlib-7.3.22
2020-09-08 07:57:20+02	2020-09-08 08:02:14+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-08 08:02:00+02	2020-09-08 08:02:19+02	databases	php73-pdo	\N	php73-pdo-7.3.22
2020-09-08 08:02:14+02	2020-09-08 08:02:34+02	devel	php73-json	\N	php73-json-7.3.22
2020-09-08 08:02:20+02	2020-09-08 08:02:52+02	converters	php73-mbstring	\N	php73-mbstring-7.3.22
2020-09-08 08:02:34+02	2020-09-08 08:02:59+02	graphics	php73-gd	\N	php73-gd-7.3.22
2020-09-08 08:02:53+02	2020-09-08 08:03:19+02	security	php73-hash	\N	php73-hash-7.3.22
2020-09-08 08:03:00+02	2020-09-08 08:03:22+02	security	php73-openssl	\N	php73-openssl-7.3.22
2020-09-08 08:03:20+02	2020-09-08 08:03:38+02	textproc	php73-xml	\N	php73-xml-7.3.22
2020-09-08 08:03:22+02	2020-09-08 08:03:55+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-08 08:03:38+02	2020-09-08 08:03:56+02	converters	php73-iconv	\N	php73-iconv-7.3.22
2020-09-08 08:03:56+02	2020-09-08 08:04:14+02	databases	pecl-memcache	php73	php73-pecl-memcache-4.0.5.2
2020-09-08 08:03:56+02	2020-09-08 08:04:18+02	databases	pecl-memcached	php73	php73-pecl-memcached-3.1.5
2020-09-08 07:58:38+02	2020-09-08 08:04:29+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-08 08:04:15+02	2020-09-08 08:04:35+02	databases	php73-pdo_pgsql	\N	php73-pdo_pgsql-7.3.22
2020-09-08 08:04:18+02	2020-09-08 08:04:38+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.22
2020-09-08 08:04:35+02	2020-09-08 08:04:56+02	databases	php73-pgsql	\N	php73-pgsql-7.3.22
2020-09-08 08:04:38+02	2020-09-08 08:04:57+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.22
2020-09-08 08:04:56+02	2020-09-08 08:05:16+02	devel	php73-gettext	\N	php73-gettext-7.3.22
2020-09-08 08:04:57+02	2020-09-08 08:05:34+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-09-08 08:05:16+02	2020-09-08 08:05:36+02	ftp	php73-curl	\N	php73-curl-7.3.22
2020-09-08 08:05:36+02	2020-09-08 08:05:55+02	math	pecl-bitset	php73	php73-pecl-bitset-3.0.1
2020-09-08 08:05:55+02	2020-09-08 08:06:16+02	math	php73-bcmath	\N	php73-bcmath-7.3.22
2020-09-08 08:05:35+02	2020-09-08 08:06:17+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-09-08 08:06:17+02	2020-09-08 08:06:40+02	net	php73-soap	\N	php73-soap-7.3.22
2020-09-08 08:06:41+02	2020-09-08 08:06:59+02	net	php73-sockets	\N	php73-sockets-7.3.22
2020-09-08 08:04:30+02	2020-09-08 08:07:03+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-08 08:07:00+02	2020-09-08 08:07:16+02	security	pecl-mcrypt	php73	php73-pecl-mcrypt-1.0.3
2020-09-08 08:07:04+02	2020-09-08 08:07:21+02	sysutils	php73-posix	\N	php73-posix-7.3.22
2020-09-08 08:07:16+02	2020-09-08 08:07:32+02	textproc	php73-ctype	\N	php73-ctype-7.3.22
2020-09-08 08:07:21+02	2020-09-08 08:07:43+02	textproc	php73-dom	\N	php73-dom-7.3.22
2020-09-08 08:07:32+02	2020-09-08 08:07:50+02	textproc	php73-simplexml	\N	php73-simplexml-7.3.22
2020-09-08 08:07:44+02	2020-09-08 08:08:14+02	www	php73-opcache	\N	php73-opcache-7.3.22
2020-09-08 08:07:51+02	2020-09-08 08:08:40+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-08 08:08:14+02	2020-09-08 08:08:51+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-08 08:06:16+02	2020-09-08 08:08:55+02	net	php73-ldap	\N	php73-ldap-7.3.22
2020-09-08 08:08:40+02	2020-09-08 08:09:18+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-09-08 08:08:51+02	2020-09-08 08:09:40+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-09-08 08:08:56+02	2020-09-08 08:09:47+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-09-08 08:09:19+02	2020-09-08 08:10:07+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-08 08:09:41+02	2020-09-08 08:10:20+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-09-08 08:09:48+02	2020-09-08 08:11:19+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_4,1
2020-09-08 08:10:08+02	2020-09-08 08:11:20+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-09-08 08:10:20+02	2020-09-08 08:11:20+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-09-08 08:11:21+02	2020-09-08 08:12:04+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-09-08 08:11:19+02	2020-09-08 08:12:07+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-08 08:12:08+02	2020-09-08 08:12:40+02	graphics	poppler-glib	\N	poppler-glib-20.08.0_1
2020-09-08 08:11:20+02	2020-09-08 08:12:40+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-09-08 08:12:05+02	2020-09-08 08:12:51+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-08 08:12:40+02	2020-09-08 08:13:07+02	x11-fonts	libXfont	\N	libXfont-1.5.4_2,2
2020-09-08 08:12:52+02	2020-09-08 08:13:36+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-09-08 08:12:41+02	2020-09-08 08:13:39+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-09-08 08:13:08+02	2020-09-08 08:13:51+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-09-08 08:13:36+02	2020-09-08 08:14:13+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-09-08 08:14:14+02	2020-09-08 08:15:08+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-09-08 08:15:09+02	2020-09-08 08:16:38+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-08 08:13:51+02	2020-09-08 08:17:14+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-09-08 08:16:38+02	2020-09-08 08:17:36+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-08 08:17:15+02	2020-09-08 08:18:03+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-09-08 08:17:38+02	2020-09-08 08:18:34+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-09-08 08:18:03+02	2020-09-08 08:19:00+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-09-08 08:18:35+02	2020-09-08 08:19:43+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-09-08 08:19:00+02	2020-09-08 08:19:59+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-08 08:19:43+02	2020-09-08 08:20:31+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-09-08 08:19:59+02	2020-09-08 08:20:57+02	net	remmina	\N	remmina-1.3.9
2020-09-08 08:20:58+02	2020-09-08 08:21:27+02	x11-fonts	font-misc-misc	\N	font-misc-misc-1.1.2_4
2020-09-08 08:20:32+02	2020-09-08 08:22:32+02	security	gcr	\N	gcr-3.28.0_1
2020-09-08 08:21:27+02	2020-09-08 08:22:34+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-09-08 08:22:32+02	2020-09-08 08:23:05+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-09-08 08:22:35+02	2020-09-08 08:23:16+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-09-08 08:23:05+02	2020-09-08 08:23:59+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-09-08 08:23:59+02	2020-09-08 08:24:43+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-09-08 08:23:16+02	2020-09-08 08:25:17+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-08 08:24:44+02	2020-09-08 08:25:29+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-09-08 08:25:17+02	2020-09-08 08:26:04+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-08 08:25:29+02	2020-09-08 08:26:14+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-09-08 08:26:04+02	2020-09-08 08:26:30+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-09-08 08:26:30+02	2020-09-08 08:27:14+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-09-08 08:26:15+02	2020-09-08 08:27:15+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-09-08 08:27:16+02	2020-09-08 08:28:01+02	x11-fonts	font-adobe-100dpi	\N	font-adobe-100dpi-1.0.3_4
2020-09-08 08:27:15+02	2020-09-08 08:28:57+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-09-08 08:28:02+02	2020-09-08 08:28:58+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-09-08 08:28:59+02	2020-09-08 08:29:39+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-09-08 08:28:58+02	2020-09-08 08:29:49+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-09-08 08:29:39+02	2020-09-08 08:30:30+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-08 08:13:39+02	2020-09-08 08:30:30+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-09-08 08:29:50+02	2020-09-08 08:31:54+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-08 08:30:31+02	2020-09-08 08:32:09+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-08 08:31:55+02	2020-09-08 08:32:25+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-08 08:32:09+02	2020-09-08 08:32:53+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-08 08:32:25+02	2020-09-08 08:33:01+02	x11	xterm	\N	xterm-359
2020-09-08 08:32:53+02	2020-09-08 08:33:20+02	x11-fonts	font-adobe-75dpi	\N	font-adobe-75dpi-1.0.3_4
2020-09-08 08:33:01+02	2020-09-08 08:33:59+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-09-08 08:33:59+02	2020-09-08 08:34:45+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-09-08 08:34:46+02	2020-09-08 08:35:07+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.9_6
2020-09-08 08:33:21+02	2020-09-08 08:35:29+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-09-08 08:35:30+02	2020-09-08 08:36:25+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-09-08 08:36:26+02	2020-09-08 08:37:29+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-09-08 08:35:08+02	2020-09-08 08:39:01+02	net	libmateweather	\N	libmateweather-1.24.0
2020-09-08 08:39:02+02	2020-09-08 08:39:26+02	x11-fonts	font-adobe-utopia-100dpi	\N	font-adobe-utopia-100dpi-1.0.4_4
2020-09-08 08:39:27+02	2020-09-08 08:39:53+02	x11-fonts	font-adobe-utopia-75dpi	\N	font-adobe-utopia-75dpi-1.0.4_4
2020-09-08 08:39:53+02	2020-09-08 08:40:06+02	x11-fonts	font-adobe-utopia-type1	\N	font-adobe-utopia-type1-1.0.4_4
2020-09-08 08:40:07+02	2020-09-08 08:40:22+02	x11-fonts	font-arabic-misc	\N	font-arabic-misc-1.0.3_4
2020-09-08 08:40:23+02	2020-09-08 08:40:58+02	x11-fonts	font-bh-100dpi	\N	font-bh-100dpi-1.0.3_4
2020-09-08 08:40:59+02	2020-09-08 08:41:34+02	x11-fonts	font-bh-75dpi	\N	font-bh-75dpi-1.0.3_4
2020-09-08 08:41:35+02	2020-09-08 08:41:59+02	x11-fonts	font-bh-lucidatypewriter-100dpi	\N	font-bh-lucidatypewriter-100dpi-1.0.3_4
2020-09-08 08:42:02+02	2020-09-08 08:42:26+02	x11-fonts	font-bh-lucidatypewriter-75dpi	\N	font-bh-lucidatypewriter-75dpi-1.0.3_4
2020-09-08 08:42:27+02	2020-09-08 08:42:44+02	x11-fonts	font-bh-type1	\N	font-bh-type1-1.0.3_4
2020-09-08 08:42:45+02	2020-09-08 08:43:03+02	x11-fonts	font-bitstream-100dpi	\N	font-bitstream-100dpi-1.0.3_4
2020-09-08 08:43:05+02	2020-09-08 08:43:22+02	x11-fonts	font-bitstream-75dpi	\N	font-bitstream-75dpi-1.0.3_4
2020-09-08 08:43:23+02	2020-09-08 08:43:36+02	x11-fonts	font-bitstream-type1	\N	font-bitstream-type1-1.0.3_4
2020-09-08 08:43:37+02	2020-09-08 08:43:47+02	x11-fonts	font-cronyx-cyrillic	\N	font-cronyx-cyrillic-1.0.3_4
2020-09-08 08:43:48+02	2020-09-08 08:44:02+02	x11-fonts	font-cursor-misc	\N	font-cursor-misc-1.0.3_4
2020-09-08 08:44:02+02	2020-09-08 08:44:14+02	x11-fonts	font-daewoo-misc	\N	font-daewoo-misc-1.0.3_4
2020-09-08 08:44:14+02	2020-09-08 08:44:26+02	x11-fonts	font-dec-misc	\N	font-dec-misc-1.0.3_4
2020-09-08 08:44:27+02	2020-09-08 08:44:38+02	x11-fonts	font-ibm-type1	\N	font-ibm-type1-1.0.3_4
2020-09-08 08:44:39+02	2020-09-08 08:44:50+02	x11-fonts	font-isas-misc	\N	font-isas-misc-1.0.3_4
2020-09-08 08:44:50+02	2020-09-08 08:45:00+02	x11-fonts	font-jis-misc	\N	font-jis-misc-1.0.3_4
2020-09-08 08:45:01+02	2020-09-08 08:45:10+02	x11-fonts	font-micro-misc	\N	font-micro-misc-1.0.3_4
2020-09-08 08:37:30+02	2020-09-08 08:45:10+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-08 08:45:10+02	2020-09-08 08:45:19+02	x11-fonts	font-misc-cyrillic	\N	font-misc-cyrillic-1.0.3_4
2020-09-08 08:45:11+02	2020-09-08 08:45:19+02	x11-fonts	font-mutt-misc	\N	font-mutt-misc-1.0.3_4
2020-09-08 08:45:20+02	2020-09-08 08:45:28+02	x11-fonts	font-screen-cyrillic	\N	font-screen-cyrillic-1.0.4_4
2020-09-08 08:45:19+02	2020-09-08 08:45:30+02	x11-fonts	font-schumacher-misc	\N	font-schumacher-misc-1.1.2_4
2020-09-08 08:45:28+02	2020-09-08 08:45:37+02	x11-fonts	font-sony-misc	\N	font-sony-misc-1.0.3_4
2020-09-08 08:45:30+02	2020-09-08 08:45:39+02	x11-fonts	font-sun-misc	\N	font-sun-misc-1.0.3_4
2020-09-08 08:45:37+02	2020-09-08 08:45:45+02	x11-fonts	font-winitzki-cyrillic	\N	font-winitzki-cyrillic-1.0.3_4
2020-09-08 08:45:39+02	2020-09-08 08:45:47+02	x11-fonts	font-xfree86-type1	\N	font-xfree86-type1-1.0.4_4
2020-09-08 08:30:31+02	2020-09-08 08:45:55+02	math	vtk8	\N	vtk8-8.2.0
2020-09-08 08:45:48+02	2020-09-08 08:46:28+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-09-08 08:45:46+02	2020-09-08 08:46:33+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-08 08:46:33+02	2020-09-08 08:47:13+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-09-08 08:46:29+02	2020-09-08 08:47:52+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-08 08:47:13+02	2020-09-08 08:47:58+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-08 08:47:53+02	2020-09-08 08:48:52+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-09-08 08:48:53+02	2020-09-08 08:49:40+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-09-08 08:45:56+02	2020-09-08 08:50:32+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-08 08:50:33+02	2020-09-08 08:50:52+02	x11	x11perf	\N	x11perf-1.6.1
2020-09-08 08:50:53+02	2020-09-08 08:51:22+02	x11	xcalc	\N	xcalc-1.1.0
2020-09-08 08:51:22+02	2020-09-08 08:51:47+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-09-08 08:49:41+02	2020-09-08 08:52:16+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-08 08:51:47+02	2020-09-08 08:52:57+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-09-08 08:52:16+02	2020-09-08 08:53:22+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-09-08 08:52:58+02	2020-09-08 08:53:59+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-09-08 08:53:22+02	2020-09-08 08:54:17+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-09-08 08:54:00+02	2020-09-08 08:55:03+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-09-08 08:54:18+02	2020-09-08 08:55:49+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-09-08 08:55:04+02	2020-09-08 08:56:02+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-09-08 08:56:02+02	2020-09-08 08:56:40+02	x11-fonts	xorg-fonts-100dpi	\N	xorg-fonts-100dpi-7.7
2020-09-08 08:55:49+02	2020-09-08 08:56:53+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-09-08 08:56:54+02	2020-09-08 08:57:10+02	x11-fonts	xorg-fonts-cyrillic	\N	xorg-fonts-cyrillic-7.7
2020-09-08 08:56:41+02	2020-09-08 08:57:13+02	x11-fonts	xorg-fonts-75dpi	\N	xorg-fonts-75dpi-7.7
2020-09-08 08:57:14+02	2020-09-08 08:57:35+02	x11-fonts	xorg-fonts-type1	\N	xorg-fonts-type1-7.7
2020-09-08 08:57:10+02	2020-09-08 08:58:01+02	x11-fonts	xorg-fonts-miscbitmaps	\N	xorg-fonts-miscbitmaps-7.7
2020-09-08 08:57:36+02	2020-09-08 08:58:25+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-09-08 08:58:26+02	2020-09-08 08:58:52+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-09-08 08:58:02+02	2020-09-08 08:58:55+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-09-08 08:58:56+02	2020-09-08 09:00:31+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-09-08 09:00:33+02	2020-09-08 09:02:31+02	graphics	poppler-utils	\N	poppler-utils-20.08.0_1
2020-09-08 08:47:58+02	2020-09-08 09:03:16+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-09-08 09:03:18+02	2020-09-08 09:03:46+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-10-06 11:46:31+02	2020-10-06 11:47:08+02	devel	tig	\N	tig-2.5.1
2020-09-08 09:03:46+02	2020-09-08 09:04:07+02	print	libotf	\N	libotf-0.9.16
2020-09-08 08:58:53+02	2020-09-08 09:04:27+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-08 09:04:08+02	2020-09-08 09:04:53+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-08 09:02:33+02	2020-09-08 09:05:09+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-09-08 09:05:10+02	2020-09-08 09:05:23+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-09-08 09:04:54+02	2020-09-08 09:05:32+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-09-08 09:05:33+02	2020-09-08 09:05:51+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-09-08 09:05:52+02	2020-09-08 09:06:02+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-09-08 09:05:24+02	2020-09-08 09:06:20+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-08 09:06:03+02	2020-09-08 09:07:05+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-08 09:06:21+02	2020-09-08 09:08:08+02	x11-fm	caja	\N	caja-1.22.3_1
2020-09-08 09:07:06+02	2020-09-08 09:09:15+02	x11-fonts	xorg-fonts	\N	xorg-fonts-7.7_1
2020-09-08 09:08:09+02	2020-09-08 09:09:49+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_4,1
2020-09-08 09:09:16+02	2020-09-08 09:11:01+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-08 09:09:49+02	2020-09-08 09:11:09+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-09-08 09:11:02+02	2020-09-08 09:11:53+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-08 09:11:10+02	2020-09-08 09:12:27+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-08 09:11:54+02	2020-09-08 09:13:07+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-08 09:12:28+02	2020-09-08 09:13:34+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-09-08 09:13:08+02	2020-09-08 09:14:11+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-09-08 09:14:11+02	2020-09-08 09:15:44+02	graphics	gegl	\N	gegl-0.4.24_4
2020-09-08 09:13:35+02	2020-09-08 09:15:47+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-09-08 09:15:45+02	2020-09-08 09:16:03+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-09-08 09:16:03+02	2020-09-08 09:17:32+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-08 09:04:27+02	2020-09-08 09:18:12+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-08 09:17:33+02	2020-09-08 09:18:18+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-09-08 09:18:12+02	2020-09-08 09:18:55+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-08 09:18:19+02	2020-09-08 09:19:35+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-08 09:15:48+02	2020-09-08 09:19:41+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-09-08 09:18:56+02	2020-09-08 09:20:06+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-09-08 09:19:35+02	2020-09-08 09:20:19+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-08 09:19:41+02	2020-09-08 09:20:26+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-09-08 09:20:27+02	2020-09-08 09:20:34+02	x11-fonts	gnu-unifont	\N	gnu-unifont-13.0.03,1
2020-09-08 09:20:19+02	2020-09-08 09:21:13+02	x11	zenity	\N	zenity-3.32.0
2020-09-08 09:20:34+02	2020-09-08 09:21:15+02	x11-fonts	noto-basic	\N	noto-basic-2.0_1
2020-09-08 09:21:16+02	2020-09-08 09:21:44+02	devel	apache-ant	\N	apache-ant-1.10.8
2020-09-08 09:21:13+02	2020-09-08 09:21:53+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-09-08 09:20:07+02	2020-09-08 09:22:39+02	x11	xorg	\N	xorg-7.7_3
2020-09-08 09:21:44+02	2020-09-08 09:22:47+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-08 09:22:47+02	2020-09-08 09:23:12+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-09-08 09:22:39+02	2020-09-08 09:23:43+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-08 09:23:13+02	2020-09-08 09:24:09+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-09-08 09:21:53+02	2020-09-08 09:24:35+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-09-08 09:23:43+02	2020-09-08 09:24:42+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-09-08 09:24:10+02	2020-09-08 09:24:55+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-09-08 09:24:56+02	2020-09-08 09:25:18+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-09-08 09:24:36+02	2020-09-08 09:25:53+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-09-08 09:25:53+02	2020-09-08 09:26:49+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-09-08 09:24:43+02	2020-09-08 09:28:29+02	editors	emacs	full	emacs-27.1,3
2020-09-08 09:28:29+02	2020-09-08 09:29:20+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-09-08 09:29:21+02	2020-09-08 09:30:10+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-08 09:25:18+02	2020-09-08 09:30:24+02	graphics	gimp-app	\N	gimp-app-2.10.20_4,1
2020-09-08 09:26:50+02	2020-09-08 09:30:34+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-08 09:30:35+02	2020-09-08 09:30:51+02	print	adobe-cmaps	\N	adobe-cmaps-20051217_4
2020-09-08 09:30:25+02	2020-09-08 09:31:06+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-09-08 09:30:52+02	2020-09-08 09:31:25+02	print	cups-filters	\N	cups-filters-1.28.1
2020-09-08 09:31:07+02	2020-09-08 09:33:16+02	print	fontforge	\N	fontforge-20190801_2
2020-09-08 09:30:10+02	2020-09-08 09:33:24+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-09-08 09:33:17+02	2020-09-08 09:34:02+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-08 09:33:24+02	2020-09-08 09:34:42+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-08 09:34:42+02	2020-09-08 09:35:32+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-08 09:35:33+02	2020-09-08 09:36:27+02	x11	yelp	\N	yelp-3.28.1_2
2020-09-08 09:36:28+02	2020-09-08 09:37:04+02	x11-fonts	cantarell-fonts	\N	cantarell-fonts-0.101_1
2020-09-08 09:31:26+02	2020-09-08 09:37:16+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-08 09:37:05+02	2020-09-08 09:37:54+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-08 09:37:17+02	2020-09-08 09:38:01+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-08 09:37:55+02	2020-09-08 09:39:23+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-08 09:39:23+02	2020-09-08 09:40:52+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-08 09:40:53+02	2020-09-08 09:42:23+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-08 09:42:24+02	2020-09-08 09:43:11+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-09-08 09:43:11+02	2020-09-08 09:44:26+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-09-08 09:44:26+02	2020-09-08 09:46:12+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-09-08 09:46:13+02	2020-09-08 09:49:01+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-08 09:49:02+02	2020-09-08 09:50:27+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-08 09:38:01+02	2020-09-08 09:50:32+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-09-08 09:50:28+02	2020-09-08 09:51:35+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-09-08 09:50:33+02	2020-09-08 09:52:03+02	x11-wm	marco	\N	marco-1.22.4
2020-09-08 09:51:35+02	2020-09-08 09:52:38+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-08 09:52:38+02	2020-09-08 09:53:47+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-08 09:52:04+02	2020-09-08 09:54:11+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-08 09:54:12+02	2020-09-08 09:56:19+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-08 09:53:47+02	2020-09-08 09:56:36+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-08 09:56:37+02	2020-09-08 10:00:41+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-08 10:00:44+02	2020-09-08 10:01:36+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-09-08 10:01:41+02	2020-09-08 10:03:36+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-08 10:03:38+02	2020-09-08 10:04:02+02	graphics	libgxps	\N	libgxps-0.3.1
2020-09-08 09:34:03+02	2020-09-08 10:05:23+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-08 10:05:30+02	2020-09-08 10:06:52+02	graphics	poppler-qt5	\N	poppler-qt5-20.08.0_1
2020-09-08 09:56:20+02	2020-09-08 10:07:00+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-08 10:06:52+02	2020-09-08 10:07:23+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-09-08 10:07:01+02	2020-09-08 10:07:51+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-10-06 13:23:01+02	2020-10-06 13:24:35+02	x11	mate	\N	mate-1.24.1
2020-09-08 10:07:23+02	2020-09-08 10:08:09+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-09-08 10:04:02+02	2020-09-08 10:08:42+02	graphics	podofo	\N	podofo-0.9.6_1
2020-09-08 10:08:43+02	2020-09-08 10:09:50+02	multimedia	libass	\N	libass-0.14.0
2020-09-08 10:09:51+02	2020-09-08 10:11:11+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-08 10:11:11+02	2020-09-08 10:12:29+02	net	freerdp	\N	freerdp-2.2.0
2020-09-08 10:12:30+02	2020-09-08 10:13:20+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-08 10:13:20+02	2020-09-08 10:14:24+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-09-08 10:08:10+02	2020-09-08 10:14:30+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-08 10:14:24+02	2020-09-08 10:15:12+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-09-08 10:15:12+02	2020-09-08 10:15:36+02	print	libspectre	\N	libspectre-0.2.9
2020-09-08 10:15:37+02	2020-09-08 10:17:32+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-08 10:14:31+02	2020-09-08 10:17:47+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-08 10:07:51+02	2020-09-08 10:20:35+02	math	R	\N	R-4.0.2
2020-09-08 10:17:32+02	2020-09-08 10:22:36+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-08 10:17:48+02	2020-09-08 10:22:37+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-08 10:20:36+02	2020-09-08 10:22:45+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-09-08 10:22:37+02	2020-09-08 10:23:17+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-08 10:23:18+02	2020-09-08 10:23:53+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-08 10:22:38+02	2020-09-08 10:24:04+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-09-08 10:22:46+02	2020-09-08 10:24:15+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-08 10:24:05+02	2020-09-08 10:24:31+02	www	youtube_dl	\N	youtube_dl-2020.09.06
2020-09-08 10:24:32+02	2020-09-08 10:24:39+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-09-08 10:23:54+02	2020-09-08 10:24:44+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-08 10:24:39+02	2020-09-08 10:24:47+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-09-08 10:24:48+02	2020-09-08 10:24:55+02	x11-fonts	gentium-basic	\N	GentiumBasic-1102_1
2020-09-08 10:24:44+02	2020-09-08 10:24:58+02	x11-fonts	droid-fonts-ttf	\N	droid-fonts-ttf-20131024_5
2020-09-08 10:24:55+02	2020-09-08 10:25:04+02	x11-fonts	liberation-fonts-ttf	\N	liberation-fonts-ttf-2.1.1,2
2020-09-08 10:24:58+02	2020-09-08 10:25:12+02	x11-fonts	linuxlibertine-g	\N	linuxlibertine-g-20120116_2
2020-09-08 10:24:15+02	2020-09-08 10:25:13+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-08 10:25:05+02	2020-09-08 10:25:16+02	x11-fonts	twemoji-color-font-ttf	\N	twemoji-color-font-ttf-12.0.1
2020-09-08 10:25:17+02	2020-09-08 10:25:56+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-08 10:25:14+02	2020-09-08 10:26:04+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-09-08 10:25:13+02	2020-09-08 10:26:29+02	x11-servers	xephyr	\N	xephyr-1.20.8_4,1
2020-09-08 10:25:57+02	2020-09-08 10:26:46+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-08 10:26:04+02	2020-09-08 10:26:59+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-09-08 10:26:29+02	2020-09-08 10:27:22+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-08 10:27:23+02	2020-09-08 10:27:37+02	audio	faudio	\N	FAudio-20.09
2020-09-08 10:27:37+02	2020-09-08 10:28:49+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-08 10:27:00+02	2020-09-08 10:28:56+02	archivers	engrampa	\N	engrampa-1.24.0
2020-09-08 10:28:50+02	2020-09-08 10:29:52+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-09-08 10:28:57+02	2020-09-08 10:30:14+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-09-08 10:30:15+02	2020-09-08 10:31:06+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-08 10:26:46+02	2020-09-08 10:31:19+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-08 10:31:19+02	2020-09-08 10:31:38+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-09-08 10:31:07+02	2020-09-08 10:31:59+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-08 10:29:52+02	2020-09-08 10:32:28+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-08 10:32:00+02	2020-09-08 10:32:42+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-09-08 10:31:39+02	2020-09-08 10:32:51+02	devel	gradle5	\N	gradle5-5.0
2020-09-08 10:32:28+02	2020-09-08 10:33:02+02	devel	py-game	py37	py37-game-1.9.6_1
2020-09-08 10:32:43+02	2020-09-08 10:33:54+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-08 10:33:54+02	2020-09-08 10:36:48+02	editors	pluma	\N	pluma-1.24.0
2020-09-08 10:36:48+02	2020-09-08 10:39:56+02	french	med	\N	fr-med-4.0.0
2020-09-08 10:39:56+02	2020-09-08 10:44:08+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-09-08 10:44:09+02	2020-09-08 10:47:09+02	graphics	atril	\N	atril-1.22.3_8
2020-09-08 10:47:10+02	2020-09-08 10:48:36+02	graphics	eom	\N	eom-1.24.0
2020-09-08 10:48:36+02	2020-09-08 10:49:47+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-08 10:49:47+02	2020-09-08 10:50:29+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-08 10:32:52+02	2020-09-08 10:54:38+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-08 10:33:03+02	2020-09-08 10:55:57+02	editors	libreoffice	\N	libreoffice-7.0.1.2
2020-09-08 10:54:39+02	2020-09-08 10:56:20+02	graphics	py-gimp	\N	py27-gimp-2.10.20_4
2020-09-08 10:55:59+02	2020-09-08 10:58:49+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-09-08 10:50:29+02	2020-09-08 11:11:01+02	graphics	openimageio	\N	openimageio-2.1.15.0_6
2020-09-08 10:56:21+02	2020-09-08 11:16:44+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-08 11:16:46+02	2020-09-08 11:21:17+02	math	mate-calc	\N	mate-calc-1.24.0
2020-09-08 11:21:17+02	2020-09-08 11:22:27+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-09-08 10:58:50+02	2020-09-08 11:25:32+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-09-08 11:22:28+02	2020-09-08 11:25:38+02	math	py-numpy	py38	py38-numpy-1.16.6,1
2020-09-08 11:11:03+02	2020-09-08 11:25:57+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-08 11:25:33+02	2020-09-08 11:26:57+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-09-08 11:25:39+02	2020-09-08 11:27:25+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-09-08 11:26:58+02	2020-09-08 11:27:56+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-08 11:27:26+02	2020-09-08 11:28:05+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-08 11:27:57+02	2020-09-08 11:28:41+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-08 11:28:05+02	2020-09-08 11:28:51+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-08 11:28:41+02	2020-09-08 11:29:31+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-08 11:28:51+02	2020-09-08 11:29:35+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-08 11:29:31+02	2020-09-08 11:30:12+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-08 11:25:57+02	2020-09-08 11:30:17+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-08 11:29:35+02	2020-09-08 11:30:26+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-08 11:30:17+02	2020-09-08 11:30:58+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-09-08 11:30:12+02	2020-09-08 11:31:18+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-08 11:30:26+02	2020-09-08 11:31:29+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-09-08 11:30:58+02	2020-09-08 11:31:56+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-08 11:31:19+02	2020-09-08 11:36:05+02	print	lilypond	\N	lilypond-2.20.0
2020-09-08 11:31:29+02	2020-09-08 11:36:49+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-08 11:31:56+02	2020-09-08 11:37:01+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-08 11:36:05+02	2020-09-08 11:37:36+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-08 11:37:37+02	2020-09-08 11:37:58+02	print	transfig	\N	transfig-3.2.5e_4
2020-09-08 11:37:58+02	2020-09-08 11:38:38+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-08 11:38:39+02	2020-09-08 11:39:57+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-09-08 11:39:57+02	2020-09-08 11:41:22+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-09-08 11:36:50+02	2020-09-08 11:42:40+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-08 11:41:22+02	2020-09-08 11:42:49+02	textproc	fop	\N	fop-2.3
2020-09-08 11:42:49+02	2020-09-08 11:43:22+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-08 11:42:41+02	2020-09-08 11:43:26+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-08 11:43:23+02	2020-09-08 11:44:13+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-08 11:43:27+02	2020-09-08 11:44:24+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-08 11:44:25+02	2020-09-08 11:45:28+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-08 11:45:29+02	2020-09-08 11:46:36+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-08 11:46:37+02	2020-09-08 11:48:13+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-08 11:44:13+02	2020-09-08 11:48:39+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-08 11:48:39+02	2020-09-08 11:48:46+02	x11-fonts	bitstream-vera	\N	bitstream-vera-1.10_8
2020-09-08 11:48:14+02	2020-09-08 11:49:27+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-09-08 11:49:28+02	2020-09-08 11:49:47+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-09-08 11:49:48+02	2020-09-08 11:50:37+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-08 11:50:37+02	2020-09-08 11:51:09+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-08 11:51:10+02	2020-09-08 11:52:12+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-08 11:37:02+02	2020-09-08 11:55:35+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-08 11:55:36+02	2020-09-08 11:58:14+02	astro	xephem	\N	xephem-3.7.7_5
2020-09-08 11:58:15+02	2020-09-08 12:02:07+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-08 11:52:13+02	2020-09-08 12:03:10+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-08 11:48:46+02	2020-09-08 12:06:59+02	x11-fonts	nerd-fonts	\N	nerd-fonts-2.1.0
2020-09-08 12:03:10+02	2020-09-08 12:08:29+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-08 12:07:00+02	2020-09-08 12:09:33+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-08 12:09:35+02	2020-09-08 12:12:03+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-08 12:12:04+02	2020-09-08 12:13:53+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-08 12:13:54+02	2020-09-08 12:14:07+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-09-08 12:14:07+02	2020-09-08 12:18:13+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-08 12:02:07+02	2020-09-08 12:19:49+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-08 12:18:13+02	2020-09-08 12:20:07+02	deskutils	calibre	\N	calibre-4.23.0
2020-09-08 12:19:51+02	2020-09-08 12:20:36+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-08 12:20:36+02	2020-09-08 12:32:43+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-08 12:32:45+02	2020-09-08 12:34:00+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-08 12:20:07+02	2020-09-08 12:46:24+02	devel	RStudio	\N	RStudio-1.3.1073
2020-09-08 12:34:01+02	2020-09-08 13:01:33+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-08 13:01:35+02	2020-09-08 13:10:31+02	editors	codelite	\N	codelite-12.0_4
2020-09-08 13:10:32+02	2020-09-08 13:12:13+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-08 13:12:14+02	2020-09-08 13:13:47+02	editors	vim	\N	vim-8.2.1558
2020-09-08 13:13:48+02	2020-09-08 13:21:18+02	emulators	qemu	\N	qemu-4.2.1
2020-09-08 12:46:25+02	2020-09-08 13:37:21+02	devel	ghidra	\N	ghidra-9.1
2020-09-08 13:37:22+02	2020-09-08 13:46:08+02	emulators	wine	\N	wine-5.0.2,1
2020-09-08 13:46:09+02	2020-09-08 13:46:54+02	emulators	x48	\N	x48-0.6.4_1
2020-09-08 12:08:30+02	2020-09-08 13:47:45+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-08 13:47:46+02	2020-09-08 13:50:06+02	games	alienarena	\N	alienarena-2013.766_9
2020-09-08 13:50:07+02	2020-09-08 13:50:51+02	games	monsterz	\N	monsterz-0.7.1_15
2020-09-08 13:50:52+02	2020-09-08 13:51:27+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-09-08 13:51:27+02	2020-09-08 13:52:14+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-09-08 13:52:15+02	2020-09-08 13:53:16+02	games	xblast	\N	xblast-2.10.4_18
2020-09-08 13:53:17+02	2020-09-08 13:54:27+02	games	xboard	\N	xboard-4.9.1_1
2020-09-08 13:54:27+02	2020-09-08 13:55:24+02	games	xcowsay	\N	xcowsay-1.5
2020-09-08 13:46:55+02	2020-09-08 14:02:35+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-08 14:02:36+02	2020-09-08 14:05:24+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-08 14:05:24+02	2020-09-08 14:06:57+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-08 14:06:58+02	2020-09-08 14:12:36+02	graphics	inkscape	\N	inkscape-1.0_4
2020-09-08 14:12:37+02	2020-09-08 14:25:30+02	graphics	krita	\N	krita-4.3.0_4
2020-09-08 13:21:19+02	2020-09-08 14:25:34+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-09-08 14:25:36+02	2020-09-08 14:26:40+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-08 14:25:31+02	2020-09-08 14:32:56+02	graphics	wings	\N	wings-2.2.5
2020-09-08 14:32:59+02	2020-09-08 14:36:09+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-08 14:26:41+02	2020-09-08 14:57:25+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-09-08 14:36:11+02	2020-09-08 15:15:30+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-08 13:55:24+02	2020-09-08 15:21:20+02	graphics	blender	\N	blender-2.90.0
2020-09-08 15:21:21+02	2020-09-08 15:27:49+02	math	maxima	\N	maxima-5.44.0_3
2020-09-08 15:27:50+02	2020-09-08 15:28:56+02	multimedia	celluloid	\N	celluloid-0.19
2020-09-08 15:28:57+02	2020-09-08 15:31:25+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1
2020-09-08 14:57:26+02	2020-09-08 15:31:37+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.3
2020-09-08 15:31:26+02	2020-09-08 15:34:04+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-09-08 15:31:38+02	2020-09-08 15:34:30+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-08 15:34:31+02	2020-09-08 15:36:03+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-08 15:36:04+02	2020-09-08 15:36:22+02	net	vnstat	\N	vnstat-2.6_1
2020-09-08 15:34:05+02	2020-09-08 15:36:26+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-09-08 15:36:26+02	2020-09-08 15:37:13+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-08 15:37:14+02	2020-09-08 15:38:09+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-09-08 15:38:09+02	2020-09-08 15:39:25+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-09-08 15:39:26+02	2020-09-08 15:40:13+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-09-08 15:36:23+02	2020-09-08 15:40:58+02	net	wireshark	\N	wireshark-3.2.6
2020-09-08 15:40:13+02	2020-09-08 15:41:02+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-09-08 15:40:59+02	2020-09-08 15:41:44+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-09-08 15:41:02+02	2020-09-08 15:41:48+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-09-08 15:41:44+02	2020-09-08 15:42:27+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-09-08 15:41:48+02	2020-09-08 15:42:56+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-09-08 15:42:27+02	2020-09-08 15:43:14+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-09-08 15:42:56+02	2020-09-08 15:43:39+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-09-08 15:43:14+02	2020-09-08 15:44:13+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-09-08 15:43:39+02	2020-09-08 15:44:28+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-09-08 15:44:29+02	2020-09-08 15:45:03+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-09-08 15:45:03+02	2020-09-08 15:45:45+02	print	gv	\N	gv-3.7.4_5
2020-09-08 15:44:13+02	2020-09-08 15:46:59+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-08 15:45:46+02	2020-09-08 15:51:57+02	print	lyx	\N	lyx-2.3.4.2_1
2020-09-08 15:47:00+02	2020-09-08 15:52:56+02	print	scribus-devel	\N	scribus-devel-1.5.5_16
2020-09-08 15:52:57+02	2020-09-08 15:53:48+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-09-08 15:51:58+02	2020-09-08 15:56:32+02	print	texlive-full	\N	texlive-full-20150521
2020-09-08 15:56:33+02	2020-09-08 15:57:30+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-08 15:57:30+02	2020-09-08 15:58:05+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-08 15:58:05+02	2020-09-08 15:58:57+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-08 15:58:57+02	2020-09-08 15:59:46+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-08 15:15:31+02	2020-09-08 15:59:59+02	lang	mono	\N	mono-5.10.1.57_3
2020-09-08 16:00:00+02	2020-09-08 16:00:18+02	www	dokuwiki	\N	dokuwiki-20200729
2020-09-08 15:59:46+02	2020-09-08 16:00:41+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-09-08 15:53:49+02	2020-09-08 16:01:12+02	shells	starship	\N	starship-0.44.0_2
2020-09-08 16:00:41+02	2020-09-08 16:01:33+02	www	links	\N	links-2.20.2,1
2020-09-08 16:01:12+02	2020-09-08 16:05:45+02	www	mod_php73	\N	mod_php73-7.3.22
2020-09-08 16:05:46+02	2020-09-08 16:07:16+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-09-08 16:07:17+02	2020-09-08 16:09:22+02	x11	mate	\N	mate-1.22.2
2020-09-08 16:09:23+02	2020-09-08 16:09:36+02	x11	sterm	\N	sterm-0.8.4
2020-09-08 16:09:37+02	2020-09-08 16:10:23+02	x11	xdm	\N	xdm-1.1.12_3
2020-09-08 16:10:23+02	2020-09-08 16:11:07+02	x11	xlockmore	\N	xlockmore-5.65
2020-09-08 16:11:08+02	2020-09-08 16:13:21+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-08 16:13:21+02	2020-09-08 16:19:09+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-08 16:19:10+02	2020-09-08 16:20:05+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-08 16:20:06+02	2020-09-08 16:20:17+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-09-08 16:20:17+02	2020-09-08 16:20:51+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-09-08 16:20:51+02	2020-09-08 16:21:11+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-09-08 16:01:33+02	2020-09-08 16:21:24+02	x11	alacritty	\N	alacritty-0.5.0_2
2020-09-08 16:21:25+02	2020-09-08 16:22:13+02	graphics	xfig	\N	xfig-3.2.7a
2020-09-08 16:22:13+02	2020-09-08 16:22:50+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-09-08 16:00:19+02	2020-09-08 16:40:07+02	www	firefox	\N	firefox-80.0.1,1
2020-09-08 16:21:12+02	2020-09-08 16:44:26+02	editors	vscode	\N	vscode-1.46.1
2020-09-08 17:20:53+02	2020-09-08 17:21:47+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-08 17:20:53+02	2020-09-08 17:22:28+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-08 17:22:29+02	2020-09-08 17:22:36+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-08 20:46:29+02	2020-09-08 20:46:37+02	sysutils	bpytop	\N	bpytop-1.0.26
2020-09-08 21:28:22+02	2020-09-08 21:29:33+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-08 21:28:22+02	2020-09-08 21:30:56+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-08 21:30:56+02	2020-09-08 21:31:04+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-09 07:48:11+02	2020-09-09 07:50:31+02	ports-mgmt	pkg	\N	pkg-1.15.3
2020-09-09 08:02:07+02	2020-09-09 08:04:23+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-09-09 08:04:24+02	2020-09-09 08:04:31+02	security	base-audit	\N	base-audit-0.4
2020-09-09 08:02:07+02	2020-09-09 09:26:57+02	graphics	blender	\N	blender-2.90.0
2020-09-09 08:02:07+02	2020-09-09 09:43:05+02	www	node	\N	node-14.10.0_1
2020-09-09 09:26:58+02	2020-09-09 09:53:48+02	local	E590T-localbase	\N	E590T-localbase-2020090900
2020-09-09 09:54:03+02	2020-09-09 09:55:18+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-09 09:54:03+02	2020-09-09 10:00:17+02	graphics	drm-current-kmod	\N	drm-current-kmod-4.16.g20200320
2020-09-09 10:00:17+02	2020-09-09 10:00:25+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-12 13:32:47+02	2020-09-12 13:34:32+02	ports-mgmt	pkg	\N	pkg-1.15.4
2020-09-12 14:12:57+02	2020-09-12 14:15:16+02	ports-mgmt	pkg	\N	pkg-1.15.4
2020-09-12 14:47:33+02	2020-09-12 14:49:21+02	ports-mgmt	pkg	\N	pkg-1.15.4
2020-09-12 14:54:38+02	2020-09-12 14:56:37+02	graphics	cairo	\N	cairo-1.17.2,2
2020-09-12 14:54:38+02	2020-09-12 14:57:19+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-09-12 14:57:20+02	2020-09-12 14:57:42+02	audio	libsndfile	\N	libsndfile-1.0.29
2020-09-12 14:56:38+02	2020-09-12 14:58:24+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-09-12 14:57:42+02	2020-09-12 14:59:12+02	graphics	poppler	\N	poppler-20.09.0
2020-09-12 14:59:13+02	2020-09-12 15:00:23+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-09-12 14:58:25+02	2020-09-12 15:01:09+02	graphics	graphviz	\N	graphviz-2.44.1_2
2020-09-12 15:01:10+02	2020-09-12 15:04:15+02	lang	vala	\N	vala-0.40.15,1
2020-09-12 15:04:15+02	2020-09-12 15:05:09+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-09-12 15:00:23+02	2020-09-12 15:05:28+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-12 15:05:29+02	2020-09-12 15:07:09+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-09-12 15:07:10+02	2020-09-12 15:09:08+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-09-12 15:05:10+02	2020-09-12 15:11:49+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-09-12 15:11:50+02	2020-09-12 15:12:16+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-09-12 15:12:17+02	2020-09-12 15:12:47+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-09-12 15:12:48+02	2020-09-12 15:13:17+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-09-12 15:13:17+02	2020-09-12 15:14:14+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-12 15:09:09+02	2020-09-12 15:14:23+02	print	texlive-base	\N	texlive-base-20150521_55
2020-09-12 15:14:15+02	2020-09-12 15:15:10+02	graphics	frei0r	\N	frei0r-1.6.1
2020-09-12 15:15:10+02	2020-09-12 15:15:27+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-09-12 15:15:27+02	2020-09-12 15:23:52+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_2,1
2020-09-12 15:23:53+02	2020-09-12 15:24:13+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-12 15:24:14+02	2020-09-12 15:25:15+02	graphics	graphene	\N	graphene-1.10.0
2020-09-12 15:25:15+02	2020-09-12 15:26:39+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-12 15:14:23+02	2020-09-12 15:29:45+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-12 15:29:46+02	2020-09-12 15:32:23+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-12 15:32:24+02	2020-09-12 15:38:32+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-12 15:38:33+02	2020-09-12 15:52:10+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-12 15:52:11+02	2020-09-12 15:52:44+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-12 15:52:44+02	2020-09-12 15:59:26+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-12 15:59:26+02	2020-09-12 16:00:10+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-09-12 16:00:11+02	2020-09-12 16:00:55+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-12 16:00:56+02	2020-09-12 16:01:51+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-09-12 16:01:52+02	2020-09-12 16:03:18+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-12 16:03:19+02	2020-09-12 16:04:08+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-12 16:04:09+02	2020-09-12 16:05:15+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-09-12 16:05:15+02	2020-09-12 16:06:50+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-09-12 16:06:50+02	2020-09-12 16:07:45+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-12 16:07:46+02	2020-09-12 16:10:19+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-09-12 16:10:20+02	2020-09-12 16:12:51+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-09-12 16:12:52+02	2020-09-12 16:14:05+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-12 16:14:06+02	2020-09-12 16:15:17+02	net	remmina	\N	remmina-1.3.9
2020-09-12 16:15:18+02	2020-09-12 16:17:47+02	security	gcr	\N	gcr-3.28.0_1
2020-09-12 16:24:38+02	2020-09-12 16:26:25+02	ports-mgmt	pkg	\N	pkg-1.15.4
2020-09-12 16:29:34+02	2020-09-12 16:29:40+02	devel	autoconf-wrapper	\N	autoconf-wrapper-20131203
2020-09-12 16:29:34+02	2020-09-12 16:29:41+02	misc	pciids	\N	pciids-20200819
2020-09-12 16:29:34+02	2020-09-12 16:29:44+02	devel	ccache	\N	ccache-3.7.1_1
2020-09-12 16:29:40+02	2020-09-12 16:29:46+02	devel	evdev-proto	\N	evdev-proto-5.3
2020-09-12 16:29:41+02	2020-09-12 16:29:47+02	sysutils	gnome_subr	\N	gnome_subr-1.0
2020-09-12 16:29:44+02	2020-09-12 16:29:50+02	print	indexinfo	\N	indexinfo-0.3.1
2020-09-12 16:29:47+02	2020-09-12 16:29:57+02	devel	pkgconf	\N	pkgconf-1.7.3,1
2020-09-12 16:29:50+02	2020-09-12 16:30:28+02	devel	gettext-runtime	\N	gettext-runtime-0.21
2020-09-12 16:30:29+02	2020-09-12 16:30:48+02	devel	gmake	\N	gmake-4.3_2
2020-09-12 16:30:49+02	2020-09-12 16:31:24+02	devel	readline	\N	readline-8.0.4
2020-09-12 16:31:25+02	2020-09-12 16:31:45+02	devel	libffi	\N	libffi-3.3_1
2020-09-12 16:29:58+02	2020-09-12 16:31:47+02	devel	libtextstyle	\N	libtextstyle-0.21
2020-09-12 16:31:45+02	2020-09-12 16:32:34+02	converters	libiconv	\N	libiconv-1.16
2020-09-12 16:32:35+02	2020-09-12 16:33:02+02	textproc	expat2	\N	expat-2.2.8
2020-09-12 16:33:02+02	2020-09-12 16:34:50+02	textproc	libxml2	\N	libxml2-2.9.10
2020-09-12 16:34:51+02	2020-09-12 16:34:58+02	devel	xorg-macros	\N	xorg-macros-1.19.2
2020-09-12 16:34:58+02	2020-09-12 16:35:22+02	security	libgpg-error	\N	libgpg-error-1.39
2020-09-12 16:35:22+02	2020-09-12 16:35:32+02	x11	xorgproto	\N	xorgproto-2020.1
2020-09-12 16:35:33+02	2020-09-12 16:36:12+02	archivers	liblz4	\N	liblz4-1.9.2_1,1
2020-09-12 16:29:48+02	2020-09-12 16:36:45+02	lang	perl5.32	\N	perl5-5.32.0
2020-09-12 16:36:46+02	2020-09-12 16:37:04+02	converters	p5-Text-Unidecode	\N	p5-Text-Unidecode-1.30
2020-09-12 16:36:13+02	2020-09-12 16:37:10+02	archivers	lzo2	\N	lzo2-2.10_1
2020-09-12 16:37:04+02	2020-09-12 16:37:17+02	textproc	p5-Unicode-EastAsianWidth	\N	p5-Unicode-EastAsianWidth-12.0
2020-09-12 16:31:47+02	2020-09-12 16:37:19+02	devel	gettext-tools	\N	gettext-tools-0.21
2020-09-12 16:37:11+02	2020-09-12 16:37:27+02	security	ca_root_nss	\N	ca_root_nss-3.56
2020-09-12 16:37:19+02	2020-09-12 16:37:34+02	devel	p5-Locale-gettext	\N	p5-Locale-gettext-1.07
2020-09-12 16:37:28+02	2020-09-12 16:37:49+02	devel	p5-Locale-libintl	\N	p5-Locale-libintl-1.31
2020-09-12 16:37:49+02	2020-09-12 16:38:09+02	misc	help2man	\N	help2man-1.47.16
2020-09-12 16:37:17+02	2020-09-12 16:38:38+02	archivers	libarchive	\N	libarchive-3.4.3,1
2020-09-12 16:38:39+02	2020-09-12 16:39:10+02	security	rhash	\N	rhash-1.3.9
2020-09-12 16:39:10+02	2020-09-12 16:39:23+02	devel	libpthread-stubs	\N	libpthread-stubs-0.4
2020-09-12 16:39:24+02	2020-09-12 16:39:40+02	x11	libXau	\N	libXau-1.0.9
2020-09-12 16:38:09+02	2020-09-12 16:39:49+02	print	texinfo	\N	texinfo-6.7_4,1
2020-09-12 16:39:41+02	2020-09-12 16:39:54+02	x11	libXdmcp	\N	libXdmcp-1.1.3
2020-09-12 16:39:50+02	2020-09-12 16:41:05+02	devel	m4	\N	m4-1.4.18_1,1
2020-09-12 16:39:55+02	2020-09-12 16:41:42+02	security	libgcrypt	\N	libgcrypt-1.8.5
2020-09-12 16:41:06+02	2020-09-12 16:41:53+02	devel	autoconf	\N	autoconf-2.69_3
2020-09-12 16:41:42+02	2020-09-12 16:42:12+02	devel	libtool	\N	libtool-2.4.6_1
2020-09-12 16:41:54+02	2020-09-12 16:42:23+02	devel	automake	\N	automake-1.16.2
2020-09-12 16:42:12+02	2020-09-12 16:42:45+02	textproc	libxslt	\N	libxslt-1.1.34_1
2020-09-12 16:42:25+02	2020-09-12 16:43:19+02	www	libnghttp2	\N	libnghttp2-1.41.0
2020-09-12 16:37:34+02	2020-09-12 16:43:28+02	lang	python37	\N	python37-3.7.9
2020-09-12 16:42:45+02	2020-09-12 16:43:33+02	devel	libuv	\N	libuv-1.39.0
2020-09-12 16:43:29+02	2020-09-12 16:43:43+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-09-12 16:43:43+02	2020-09-12 16:43:53+02	devel	py-six	py37	py37-six-1.15.0
2020-09-12 16:43:53+02	2020-09-12 16:44:05+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-09-12 16:44:05+02	2020-09-12 16:44:15+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-09-12 16:43:33+02	2020-09-12 16:44:33+02	devel	ninja	\N	ninja-1.10.1,2
2020-09-12 16:44:16+02	2020-09-12 16:44:39+02	devel	py-cffi	py37	py37-cffi-1.14.2
2020-09-12 16:44:34+02	2020-09-12 16:44:49+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-09-12 16:44:40+02	2020-09-12 16:44:54+02	dns	py-idna	py37	py37-idna-2.8
2020-09-12 16:43:19+02	2020-09-12 16:45:02+02	ftp	curl	\N	curl-7.72.0
2020-09-12 16:45:03+02	2020-09-12 16:45:12+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-09-12 16:45:13+02	2020-09-12 16:45:23+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-09-12 16:44:54+02	2020-09-12 16:45:24+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-09-12 16:44:50+02	2020-09-12 16:45:29+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-09-12 16:45:23+02	2020-09-12 16:45:33+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-09-12 16:45:24+02	2020-09-12 16:45:34+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-09-12 16:45:29+02	2020-09-12 16:45:39+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-09-12 16:45:33+02	2020-09-12 16:45:43+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-09-12 16:45:40+02	2020-09-12 16:45:50+02	net	py-urllib3	py37	py37-urllib3-1.25.10,1
2020-09-12 16:45:43+02	2020-09-12 16:45:57+02	devel	meson	\N	meson-0.54.3
2020-09-12 16:45:50+02	2020-09-12 16:46:02+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-09-12 16:46:03+02	2020-09-12 16:46:14+02	www	py-requests	py37	py37-requests-2.22.0
2020-09-12 16:46:14+02	2020-09-12 16:46:24+02	devel	py-packaging	py37	py37-packaging-20.4
2020-09-12 16:46:25+02	2020-09-12 16:46:40+02	textproc	py-pygments	py37	py37-pygments-2.5.2
2020-09-12 16:46:41+02	2020-09-12 16:46:51+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-09-12 16:45:34+02	2020-09-12 16:46:57+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-09-12 16:46:51+02	2020-09-12 16:47:02+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-09-12 16:46:57+02	2020-09-12 16:47:08+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-09-12 16:47:03+02	2020-09-12 16:47:13+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-09-12 16:47:08+02	2020-09-12 16:47:19+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-09-12 16:47:13+02	2020-09-12 16:47:24+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-09-12 16:47:19+02	2020-09-12 16:47:30+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-09-12 16:47:24+02	2020-09-12 16:47:39+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-09-12 16:45:58+02	2020-09-12 16:48:02+02	lang	cython	py37	py37-cython-0.29.15
2020-09-12 16:47:39+02	2020-09-12 16:48:15+02	devel	check	\N	check-0.15.2
2020-09-12 16:47:30+02	2020-09-12 16:48:16+02	devel	jsoncpp	\N	jsoncpp-1.9.3
2020-09-12 16:48:16+02	2020-09-12 16:48:27+02	x11	xtrans	\N	xtrans-1.4.0
2020-09-12 16:48:02+02	2020-09-12 16:48:28+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-09-12 16:48:15+02	2020-09-12 16:48:28+02	x11	xcb-proto	\N	xcb-proto-1.13_1
2020-09-12 16:48:29+02	2020-09-12 16:48:41+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-09-12 16:48:41+02	2020-09-12 16:49:14+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-09-12 16:48:29+02	2020-09-12 16:49:42+02	x11	libxcb	\N	libxcb-1.13.1
2020-09-12 16:48:28+02	2020-09-12 16:51:58+02	math	gmp	\N	gmp-6.2.0
2020-09-12 16:49:42+02	2020-09-12 16:53:25+02	x11	libX11	\N	libX11-1.6.12,1
2020-09-12 16:51:59+02	2020-09-12 16:53:39+02	math	mpfr	\N	mpfr-4.1.0
2020-09-12 16:53:26+02	2020-09-12 16:53:45+02	x11	libXext	\N	libXext-1.3.4,1
2020-09-12 16:53:39+02	2020-09-12 16:58:41+02	devel	pcre	\N	pcre-8.44
2020-09-12 16:58:41+02	2020-09-12 16:59:26+02	devel	libedit	\N	libedit-3.1.20191231,1
2020-09-12 16:59:26+02	2020-09-12 17:00:53+02	print	freetype2	\N	freetype2-2.10.2_1
2020-09-12 17:00:54+02	2020-09-12 17:03:14+02	devel	bison	\N	bison-3.6.4,1
2020-09-12 17:03:15+02	2020-09-12 17:03:45+02	devel	gperf	\N	gperf-3.1
2020-09-12 17:03:45+02	2020-09-12 17:04:43+02	x11-fonts	fontconfig	\N	fontconfig-2.13.92_2,1
2020-09-12 17:04:43+02	2020-09-12 17:05:09+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-09-12 17:05:09+02	2020-09-12 17:05:28+02	lang	lua52	\N	lua52-5.2.4
2020-09-12 17:05:28+02	2020-09-12 17:05:47+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-09-12 17:05:48+02	2020-09-12 17:06:50+02	devel	libunwind	\N	libunwind-20200331_1
2020-09-12 17:06:50+02	2020-09-12 17:07:09+02	textproc	xmlcatmgr	\N	xmlcatmgr-2.2_2
2020-09-12 17:07:13+02	2020-09-12 17:07:31+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-09-12 17:07:31+02	2020-09-12 17:07:58+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-09-12 17:07:58+02	2020-09-12 17:08:24+02	devel	py-future	py37	py37-future-0.18.2
2020-09-12 17:08:24+02	2020-09-12 17:08:32+02	textproc	iso8879	\N	iso8879-1986_3
2020-09-12 17:08:34+02	2020-09-12 17:08:42+02	textproc	xmlcharent	\N	xmlcharent-0.3_2
2020-09-12 17:08:42+02	2020-09-12 17:10:23+02	devel	swig	\N	swig-4.0.2
2020-09-12 17:10:24+02	2020-09-12 17:10:34+02	textproc	docbook-sgml	\N	docbook-sgml-4.5_1
2020-09-12 17:10:34+02	2020-09-12 17:10:47+02	textproc	docbook-xml	\N	docbook-xml-5.0_3
2020-09-12 16:49:15+02	2020-09-12 17:10:53+02	devel	cmake	\N	cmake-3.18.2
2020-09-12 17:10:48+02	2020-09-12 17:10:55+02	textproc	sdocbook-xml	\N	sdocbook-xml-1.1_2,2
2020-09-12 17:10:55+02	2020-09-12 17:11:03+02	textproc	docbook	\N	docbook-1.5
2020-09-12 17:10:54+02	2020-09-12 17:11:23+02	devel	libepoll-shim	\N	libepoll-shim-0.0.20200602
2020-09-12 17:11:03+02	2020-09-12 17:11:25+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-09-12 17:11:24+02	2020-09-12 17:11:52+02	graphics	wayland	\N	wayland-1.18.0_4
2020-09-12 17:11:25+02	2020-09-12 17:12:00+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-09-12 17:12:01+02	2020-09-12 17:12:16+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-09-12 17:12:17+02	2020-09-12 17:12:37+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-09-12 17:11:53+02	2020-09-12 17:12:48+02	textproc	docbook-xsl	\N	docbook-xsl-1.79.1_1,1
2020-09-12 17:12:37+02	2020-09-12 17:12:52+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-09-12 17:12:52+02	2020-09-12 17:13:01+02	graphics	wayland-protocols	\N	wayland-protocols-1.20
2020-09-12 17:12:48+02	2020-09-12 17:13:02+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-09-12 17:13:01+02	2020-09-12 17:13:14+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-09-12 17:13:03+02	2020-09-12 17:13:14+02	x11	libxshmfence	\N	libxshmfence-1.3
2020-09-12 17:13:14+02	2020-09-12 17:13:44+02	graphics	png	\N	png-1.6.37
2020-09-12 16:53:46+02	2020-09-12 17:15:04+02	devel	binutils	native	binutils-2.33.1_3,1
2020-09-12 17:13:15+02	2020-09-12 17:19:23+02	devel	glib20	\N	glib-2.56.3_9,1
2020-09-12 17:19:24+02	2020-09-12 17:21:30+02	x11	pixman	\N	pixman-0.40.0_1
2020-09-12 17:21:31+02	2020-09-12 17:23:14+02	devel	boost-jam	\N	boost-jam-1.72.0
2020-09-12 17:23:14+02	2020-09-12 17:23:46+02	x11	libICE	\N	libICE-1.0.10,1
2020-09-12 17:23:48+02	2020-09-12 17:24:12+02	x11	libSM	\N	libSM-1.2.3,1
2020-09-12 17:24:12+02	2020-09-12 17:25:23+02	devel	nasm	\N	nasm-2.15.03,1
2020-09-12 17:25:23+02	2020-09-12 17:25:44+02	x11-fonts	libfontenc	\N	libfontenc-1.1.4
2020-09-12 17:25:45+02	2020-09-12 17:26:21+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-09-12 17:26:22+02	2020-09-12 17:27:23+02	graphics	jpeg-turbo	\N	jpeg-turbo-2.0.4
2020-09-12 17:27:24+02	2020-09-12 17:27:39+02	textproc	itstool	\N	itstool-2.0.6
2020-09-12 17:27:40+02	2020-09-12 17:27:51+02	x11-fonts	mkfontscale	\N	mkfontscale-1.2.1
2020-09-12 17:13:44+02	2020-09-12 17:28:36+02	devel	icu	\N	icu-67.1,1
2020-09-12 17:27:52+02	2020-09-12 17:28:36+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-09-12 17:28:37+02	2020-09-12 17:29:06+02	textproc	html2text	\N	html2text-1.3.2a
2020-09-12 17:29:06+02	2020-09-12 17:29:22+02	textproc	minixmlto	\N	minixmlto-0.0.2_1
2020-09-12 17:29:23+02	2020-09-12 17:31:08+02	devel	dbus	\N	dbus-1.12.20
2020-09-12 17:31:08+02	2020-09-12 17:31:27+02	x11-fonts	bdftopcf	\N	bdftopcf-1.1_2
2020-09-12 17:31:28+02	2020-09-12 17:31:46+02	x11-fonts	font-util	\N	font-util-1.3.2
2020-09-12 17:31:46+02	2020-09-12 17:32:27+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-09-12 17:32:27+02	2020-09-12 17:33:32+02	graphics	graphite2	\N	graphite2-1.3.14
2020-09-12 17:33:33+02	2020-09-12 17:35:54+02	shells	bash	\N	bash-5.0.18_3
2020-09-12 17:35:54+02	2020-09-12 17:36:32+02	textproc	p5-XML-Parser	\N	p5-XML-Parser-2.44
2020-09-12 17:36:32+02	2020-09-12 17:36:50+02	x11-fonts	encodings	\N	encodings-1.0.5,1
2020-09-12 17:36:51+02	2020-09-12 17:37:12+02	textproc	intltool	\N	intltool-0.51.0_1
2020-09-12 17:37:13+02	2020-09-12 17:37:32+02	x11-fonts	dejavu	\N	dejavu-2.37_1
2020-09-12 17:37:33+02	2020-09-12 17:37:47+02	x11-fonts	font-bh-ttf	\N	font-bh-ttf-1.0.3_4
2020-09-12 17:37:49+02	2020-09-12 17:38:10+02	x11-fonts	font-misc-ethiopic	\N	font-misc-ethiopic-1.0.4
2020-09-12 17:38:11+02	2020-09-12 17:38:28+02	x11-fonts	font-misc-meltho	\N	font-misc-meltho-1.0.3_4
2020-09-12 17:38:28+02	2020-09-12 17:38:54+02	x11-fonts	xorg-fonts-truetype	\N	xorg-fonts-truetype-7.7_1
2020-09-12 17:38:55+02	2020-09-12 17:39:13+02	print	libpaper	\N	libpaper-1.1.24.4
2020-09-12 17:39:13+02	2020-09-12 17:39:31+02	graphics	jbigkit	\N	jbigkit-2.1_1
2020-09-12 17:39:31+02	2020-09-12 17:41:02+02	graphics	tiff	\N	tiff-4.1.0
2020-09-12 17:41:03+02	2020-09-12 17:41:47+02	x11	libXi	\N	libXi-1.7.10,1
2020-09-12 17:41:48+02	2020-09-12 17:47:11+02	devel	libunistring	\N	libunistring-0.9.10_1
2020-09-12 17:47:12+02	2020-09-12 17:48:05+02	dns	libidn2	\N	libidn2-2.3.0_1
2020-09-12 17:48:05+02	2020-09-12 17:48:46+02	emulators	tpm-emulator	\N	tpm-emulator-0.7.4_2
2020-09-12 17:48:51+02	2020-09-12 17:49:45+02	security	libtasn1	\N	libtasn1-4.16.0
2020-09-12 17:49:46+02	2020-09-12 17:51:49+02	security	nettle	\N	nettle-3.6
2020-09-12 17:51:50+02	2020-09-12 17:52:27+02	shells	bash-completion	\N	bash-completion-2.11,2
2020-09-12 17:52:27+02	2020-09-12 17:56:25+02	security	trousers	\N	trousers-0.3.14_3
2020-09-12 17:56:26+02	2020-09-12 17:57:25+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-09-12 17:57:26+02	2020-09-12 17:58:16+02	databases	gdbm	\N	gdbm-1.18.1_1
2020-09-12 17:58:17+02	2020-09-12 18:08:34+02	databases	sqlite3	\N	sqlite3-3.33.0,1
2020-09-12 18:08:35+02	2020-09-12 18:08:50+02	devel	libdaemon	\N	libdaemon-0.14_1
2020-09-12 18:08:50+02	2020-09-12 18:09:12+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-09-12 18:09:13+02	2020-09-12 18:09:26+02	devel	libltdl	\N	libltdl-2.4.6
2020-09-12 18:09:26+02	2020-09-12 18:09:42+02	graphics	giflib	\N	giflib-5.2.1
2020-09-12 18:09:43+02	2020-09-12 18:10:01+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-09-12 18:10:01+02	2020-09-12 18:10:30+02	x11	xkeyboard-config	\N	xkeyboard-config-2.30
2020-09-12 18:10:30+02	2020-09-12 18:10:39+02	misc	hicolor-icon-theme	\N	hicolor-icon-theme-0.17
2020-09-12 18:10:40+02	2020-09-12 18:11:04+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-09-12 17:28:36+02	2020-09-12 18:11:55+02	devel	boost-libs	\N	boost-libs-1.72.0_2
2020-09-12 18:11:05+02	2020-09-12 18:13:11+02	graphics	webp	\N	webp-1.1.0
2020-09-12 18:13:13+02	2020-09-12 18:13:43+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-09-12 18:13:43+02	2020-09-12 18:14:42+02	x11	libxkbcommon	\N	libxkbcommon-0.10.0_2
2020-09-12 18:14:43+02	2020-09-12 18:15:23+02	x11	libXpm	\N	libXpm-3.5.13
2020-09-12 18:15:24+02	2020-09-12 18:15:44+02	converters	fribidi	\N	fribidi-0.19.7
2020-09-12 18:12:04+02	2020-09-12 18:15:52+02	textproc	highlight	\N	highlight-3.43_8,3
2020-09-12 18:15:53+02	2020-09-12 18:16:57+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-09-12 18:15:46+02	2020-09-12 18:17:07+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-09-12 18:17:07+02	2020-09-12 18:17:54+02	devel	libatomic_ops	\N	libatomic_ops-7.6.10
2020-09-12 18:17:55+02	2020-09-12 18:18:49+02	devel	boehm-gc	\N	boehm-gc-8.0.4_1
2020-09-12 18:16:57+02	2020-09-12 18:18:53+02	security	p11-kit	\N	p11-kit-0.23.21
2020-09-12 18:18:49+02	2020-09-12 18:18:57+02	misc	getopt	\N	getopt-1.1.6
2020-09-12 18:18:57+02	2020-09-12 18:20:48+02	www	w3m	\N	w3m-0.5.3.20200507
2020-09-12 18:20:49+02	2020-09-12 18:21:29+02	textproc	xmlto	\N	xmlto-0.0.28
2020-09-12 18:21:30+02	2020-09-12 18:22:59+02	dns	libidn	\N	libidn-1.35
2020-09-12 18:22:59+02	2020-09-12 18:23:25+02	graphics	poppler-data	\N	poppler-data-0.4.9_5
2020-09-12 18:23:26+02	2020-09-12 18:23:49+02	print	psutils	\N	psutils-1.17_5
2020-09-12 18:18:54+02	2020-09-12 18:23:59+02	security	gnutls	\N	gnutls-3.6.15
2020-09-12 18:23:49+02	2020-09-12 18:24:35+02	graphics	lcms2	\N	lcms2-2.11_1
2020-09-12 18:24:00+02	2020-09-12 18:24:59+02	graphics	jbig2dec	\N	jbig2dec-0.18
2020-09-12 18:24:35+02	2020-09-12 18:25:21+02	devel	nspr	\N	nspr-4.28
2020-09-12 18:25:00+02	2020-09-12 18:25:34+02	graphics	gd	\N	libgd-2.3.0,1
2020-09-12 18:25:22+02	2020-09-12 18:27:06+02	misc	shared-mime-info	\N	shared-mime-info-2.0
2020-09-12 18:25:35+02	2020-09-12 18:30:48+02	lang	python27	\N	python27-2.7.18_1
2020-09-12 18:27:07+02	2020-09-12 18:31:34+02	devel	pcre2	\N	pcre2-10.35
2020-09-12 18:31:35+02	2020-09-12 18:31:48+02	print	gsfonts	\N	gsfonts-8.11_8
2020-09-12 18:31:48+02	2020-09-12 18:32:18+02	textproc	uchardet	\N	uchardet-0.0.7
2020-09-12 18:30:49+02	2020-09-12 18:34:26+02	lang	tcl86	\N	tcl86-8.6.10
2020-09-12 18:34:26+02	2020-09-12 18:34:41+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-09-12 18:34:41+02	2020-09-12 18:34:58+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-09-12 18:34:58+02	2020-09-12 18:35:31+02	multimedia	v4l_compat	\N	v4l_compat-1.18.0
2020-09-12 18:35:32+02	2020-09-12 18:35:47+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-09-12 18:35:48+02	2020-09-12 18:36:05+02	devel	libudev-devd	\N	libudev-devd-0.4.2
2020-09-12 18:32:18+02	2020-09-12 18:36:13+02	textproc	groff	\N	groff-1.22.4_3
2020-09-12 18:36:05+02	2020-09-12 18:36:27+02	devel	autoconf213	\N	autoconf213-2.13.000227_7
2020-09-12 18:36:14+02	2020-09-12 18:36:27+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-09-12 18:36:31+02	2020-09-12 18:36:46+02	x11	xcb-util	\N	xcb-util-0.4.0_2,1
2020-09-12 18:36:47+02	2020-09-12 18:37:27+02	textproc	libcroco	\N	libcroco-0.6.13
2020-09-12 18:37:27+02	2020-09-12 18:38:28+02	archivers	zstd	\N	zstd-1.4.5_1
2020-09-12 18:38:28+02	2020-09-12 18:38:40+02	misc	qtchooser	\N	qtchooser-66_3
2020-09-12 18:38:41+02	2020-09-12 18:39:02+02	devel	double-conversion	\N	double-conversion-3.1.5.19
2020-09-12 18:39:03+02	2020-09-12 18:43:21+02	devel	qt5-qmake	\N	qt5-qmake-5.15.0
2020-09-12 18:43:22+02	2020-09-12 18:49:04+02	devel	qt5-buildtools	\N	qt5-buildtools-5.15.0
2020-09-12 18:49:07+02	2020-09-12 18:49:24+02	x11	xprop	\N	xprop-1.2.4
2020-09-12 18:49:25+02	2020-09-12 18:49:46+02	x11	xset	\N	xset-1.2.4_3
2020-09-12 18:36:30+02	2020-09-12 19:02:52+02	lang	spidermonkey60	\N	spidermonkey60-60.9.0_4
2020-09-12 19:02:57+02	2020-09-12 19:03:38+02	x11	xcb-util-wm	\N	xcb-util-wm-0.4.1_3
2020-09-12 19:03:38+02	2020-09-12 19:04:10+02	devel	jam	\N	jam-2.6
2020-09-12 19:04:11+02	2020-09-12 19:04:42+02	devel	libevdev	\N	libevdev-1.5.9_1
2020-09-12 19:04:43+02	2020-09-12 19:05:03+02	devel	libmtdev	\N	libmtdev-1.1.5_2
2020-09-12 19:05:03+02	2020-09-12 19:05:34+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-09-12 19:05:35+02	2020-09-12 19:05:58+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-09-12 19:05:59+02	2020-09-12 19:06:15+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-09-12 18:49:46+02	2020-09-12 19:06:46+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-09-12 19:06:46+02	2020-09-12 19:08:24+02	graphics	openjpeg	\N	openjpeg-2.3.1
2020-09-12 19:08:25+02	2020-09-12 19:08:52+02	devel	vulkan-headers	\N	vulkan-headers-1.2.135.0
2020-09-12 19:08:53+02	2020-09-12 19:09:16+02	x11	xcb-util-image	\N	xcb-util-image-0.4.0_1
2020-09-12 19:09:17+02	2020-09-12 19:09:39+02	x11	xcb-util-keysyms	\N	xcb-util-keysyms-0.4.0_1
2020-09-12 19:09:39+02	2020-09-12 19:09:56+02	x11	xcb-util-renderutil	\N	xcb-util-renderutil-0.3.9_1
2020-09-12 19:06:15+02	2020-09-12 19:11:22+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-09-12 19:09:58+02	2020-09-12 19:12:51+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-09-12 19:12:52+02	2020-09-12 19:14:31+02	textproc	gsed	\N	gsed-4.8
2020-09-12 19:14:31+02	2020-09-12 19:14:51+02	audio	libogg	\N	libogg-1.3.4,4
2020-09-12 19:14:52+02	2020-09-12 19:15:48+02	misc	iso-codes	\N	iso-codes-4.2
2020-09-12 19:15:49+02	2020-09-12 19:16:33+02	audio	libvorbis	\N	libvorbis-1.3.7_2,3
2020-09-12 19:11:22+02	2020-09-12 19:17:55+02	net	qt5-network	\N	qt5-network-5.15.0
2020-09-12 19:16:33+02	2020-09-12 19:18:24+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-09-12 19:17:56+02	2020-09-12 19:19:33+02	devel	orc	\N	orc-0.4.31
2020-09-12 19:19:33+02	2020-09-12 19:21:46+02	audio	flac	\N	flac-1.3.3
2020-09-12 19:21:47+02	2020-09-12 19:22:13+02	devel	desktop-file-utils	\N	desktop-file-utils-0.24
2020-09-12 19:18:25+02	2020-09-12 19:22:54+02	security	nss	\N	nss-3.56
2020-09-12 19:22:57+02	2020-09-12 19:25:21+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-09-12 19:25:22+02	2020-09-12 19:25:51+02	archivers	zip	\N	zip-3.0_1
2020-09-12 19:25:52+02	2020-09-12 19:27:35+02	audio	opus	\N	opus-1.3.1
2020-09-12 19:22:13+02	2020-09-12 19:27:50+02	math	fftw3	\N	fftw3-3.3.8_6
2020-09-12 19:27:35+02	2020-09-12 19:28:33+02	audio	libsndfile	\N	libsndfile-1.0.29
2020-09-12 19:27:51+02	2020-09-12 19:28:40+02	devel	tex-kpathsea	\N	tex-kpathsea-6.2.1_2
2020-09-12 19:28:40+02	2020-09-12 19:29:14+02	textproc	libyaml	\N	libyaml-0.2.5
2020-09-12 19:28:34+02	2020-09-12 19:29:38+02	net	libproxy	\N	libproxy-0.4.15
2020-09-12 19:29:39+02	2020-09-12 19:30:05+02	devel	zziplib	\N	zziplib-0.13.71_1
2020-09-12 19:30:06+02	2020-09-12 19:30:44+02	graphics	openjpeg15	\N	openjpeg15-1.5.2_1
2020-09-12 19:30:44+02	2020-09-12 19:31:09+02	print	tex-ptexenc	\N	tex-ptexenc-1.3.3_2
2020-09-12 19:31:09+02	2020-09-12 19:31:43+02	math	mpc	\N	mpc-1.1.0_2
2020-09-12 19:31:43+02	2020-09-12 19:32:55+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-09-12 19:32:56+02	2020-09-12 19:33:40+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-09-12 19:33:40+02	2020-09-12 19:33:56+02	graphics	libpotrace	\N	libpotrace-1.16
2020-09-12 19:33:56+02	2020-09-12 19:34:30+02	print	texlive-tlmgr	\N	texlive-tlmgr-20150523_2
2020-09-12 19:34:30+02	2020-09-12 19:34:48+02	print	xpdfopen	\N	xpdfopen-0.86
2020-09-12 19:34:48+02	2020-09-12 19:35:58+02	textproc	teckit	\N	teckit-2.5.7
2020-09-12 19:29:15+02	2020-09-12 19:47:18+02	lang	ruby27	\N	ruby-2.7.1_1,1
2020-09-12 19:35:59+02	2020-09-12 19:47:44+02	devel	protobuf	\N	protobuf-3.13.0,1
2020-09-12 19:47:19+02	2020-09-12 19:48:02+02	textproc	flex	\N	flex-2.6.4_2
2020-09-12 19:47:45+02	2020-09-12 19:49:05+02	archivers	brotli	\N	brotli-1.0.9,1
2020-09-12 19:48:03+02	2020-09-12 19:49:50+02	textproc	hunspell	\N	hunspell-1.7.0_2
2020-09-12 19:49:05+02	2020-09-12 19:50:06+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-09-12 19:50:06+02	2020-09-12 19:50:33+02	multimedia	libva	\N	libva-2.8.0
2020-09-12 19:49:50+02	2020-09-12 19:50:46+02	devel	libevent	\N	libevent-2.1.12
2020-09-12 19:50:33+02	2020-09-12 19:50:50+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-09-12 19:50:46+02	2020-09-12 19:54:38+02	databases	db6	\N	db6-6.2.32_1
2020-09-12 19:54:39+02	2020-09-12 19:54:55+02	security	libargon2	\N	libargon2-20190702
2020-09-12 19:54:56+02	2020-09-12 19:55:08+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-09-12 19:50:50+02	2020-09-12 19:55:10+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-09-12 19:55:11+02	2020-09-12 19:55:43+02	devel	woff2	\N	woff2-1.0.2_4
2020-09-12 19:55:08+02	2020-09-12 19:55:45+02	devel	argp-standalone	\N	argp-standalone-1.3_4
2020-09-12 19:55:46+02	2020-09-12 19:55:56+02	audio	ladspa	\N	ladspa-1.15
2020-09-12 19:55:56+02	2020-09-12 19:57:19+02	multimedia	libv4l	\N	libv4l-1.18.0
2020-09-12 19:57:20+02	2020-09-12 19:57:54+02	textproc	hyphen	\N	hyphen-2.8.8
2020-09-12 19:57:55+02	2020-09-12 19:58:14+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-09-12 19:58:15+02	2020-09-12 19:58:51+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-09-12 19:58:52+02	2020-09-12 19:59:04+02	devel	opencl	\N	opencl-2.2_2
2020-09-12 19:59:05+02	2020-09-12 19:59:31+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-09-12 19:59:32+02	2020-09-12 19:59:44+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-09-12 19:55:43+02	2020-09-12 19:59:54+02	lang	php73	\N	php73-7.3.22
2020-09-12 19:59:54+02	2020-09-12 20:00:21+02	math	eigen3	\N	eigen-3.3.7
2020-09-12 20:00:22+02	2020-09-12 20:00:58+02	textproc	texi2html	\N	texi2html-5.0_2,1
2020-09-12 19:59:45+02	2020-09-12 20:01:39+02	lang	fpc	\N	fpc-3.0.4_4
2020-09-12 20:01:40+02	2020-09-12 20:01:59+02	textproc	py-rdflib	py37	py37-rdflib-5.0.0
2020-09-12 20:02:00+02	2020-09-12 20:02:21+02	audio	lv2	\N	lv2-1.18.0
2020-09-12 20:02:22+02	2020-09-12 20:03:20+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-09-12 20:03:21+02	2020-09-12 20:05:28+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-09-12 20:05:28+02	2020-09-12 20:05:56+02	multimedia	libaacs	\N	libaacs-0.9.0
2020-09-12 20:05:56+02	2020-09-12 20:06:41+02	multimedia	libbdplus	\N	libbdplus-0.1.2
2020-09-12 20:06:41+02	2020-09-12 20:07:08+02	multimedia	libtheora	\N	libtheora-1.1.1_7
2020-09-12 20:07:08+02	2020-09-12 20:07:23+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-09-12 20:07:23+02	2020-09-12 20:08:01+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-09-12 20:00:59+02	2020-09-12 20:08:46+02	multimedia	x265	\N	x265-3.2.1_4
2020-09-12 20:08:02+02	2020-09-12 20:15:47+02	graphics	opencv-core	\N	opencv-core-3.4.1_36
2020-09-12 20:15:48+02	2020-09-12 20:24:07+02	multimedia	aom	\N	aom-2.0.0_1
2020-09-12 20:24:08+02	2020-09-12 20:26:12+02	multimedia	dav1d	\N	dav1d-0.7.1
2020-09-12 20:26:13+02	2020-09-12 20:27:54+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-09-12 20:27:56+02	2020-09-12 20:34:21+02	multimedia	libvpx	\N	libvpx-1.9.0
2020-09-12 20:34:22+02	2020-09-12 20:38:57+02	multimedia	libx264	\N	libx264-0.160.3011
2020-09-12 20:38:58+02	2020-09-12 20:40:46+02	multimedia	xvid	\N	xvid-1.3.7,1
2020-09-12 20:40:46+02	2020-09-12 20:41:51+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-09-12 20:41:52+02	2020-09-12 20:45:39+02	databases	postgresql12-client	\N	postgresql12-client-12.4
2020-09-12 20:45:39+02	2020-09-12 20:47:04+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-09-12 20:47:05+02	2020-09-12 20:47:22+02	devel	py-setuptools	py27	py27-setuptools-44.0.0
2020-09-12 20:47:22+02	2020-09-12 20:48:03+02	graphics	jasper	\N	jasper-2.0.16_1
2020-09-12 20:48:04+02	2020-09-12 20:48:25+02	devel	libinotify	\N	libinotify-20180201_2
2020-09-12 20:48:26+02	2020-09-12 20:49:29+02	audio	alsa-lib	\N	alsa-lib-1.1.2_2
2020-09-12 20:49:30+02	2020-09-12 20:50:49+02	security	cyrus-sasl2	\N	cyrus-sasl-2.1.27_1
2020-09-12 20:50:50+02	2020-09-12 20:51:32+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-09-12 20:51:33+02	2020-09-12 20:51:58+02	archivers	libmspack	\N	libmspack-0.10.1
2020-09-12 20:51:58+02	2020-09-12 20:53:31+02	graphics	libraw	\N	libraw-0.19.5
2020-09-12 20:53:32+02	2020-09-12 20:54:11+02	graphics	aalib	\N	aalib-1.4.r5_13
2020-09-12 20:54:12+02	2020-09-12 20:54:32+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-09-12 20:54:32+02	2020-09-12 20:57:04+02	net	openldap24-client	\N	openldap-client-2.4.51
2020-09-12 20:57:05+02	2020-09-12 20:57:38+02	devel	json-c	\N	json-c-0.15
2020-09-12 20:57:38+02	2020-09-12 20:58:04+02	science	szip	\N	hdf-szip-2.1.1
2020-09-12 20:58:05+02	2020-09-12 20:58:26+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-09-12 20:58:26+02	2020-09-12 20:59:14+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-09-12 20:59:15+02	2020-09-12 20:59:35+02	net	p5-URI	\N	p5-URI-1.76
2020-09-12 20:59:36+02	2020-09-12 21:24:27+02	science	hdf5	\N	hdf5-1.10.6,1
2020-09-12 21:24:27+02	2020-09-12 21:24:41+02	textproc	py-toml	py37	py37-toml-0.10.1
2020-09-12 21:24:41+02	2020-09-12 21:24:57+02	x11	libXres	\N	libXres-1.2.0_2
2020-09-12 21:24:58+02	2020-09-12 21:25:21+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-09-12 21:25:21+02	2020-09-12 21:25:50+02	audio	speexdsp	\N	speexdsp-1.2.0
2020-09-12 21:25:51+02	2020-09-12 21:26:05+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3_1
2020-09-12 21:26:06+02	2020-09-12 21:27:47+02	textproc	librevenge	\N	librevenge-0.0.4_13
2020-09-12 21:27:47+02	2020-09-12 21:34:19+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-09-12 21:34:20+02	2020-09-12 21:34:38+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-09-12 21:34:39+02	2020-09-12 21:35:16+02	x11-fonts	libXfont2	\N	libXfont2-2.0.4
2020-09-12 21:35:17+02	2020-09-12 21:36:47+02	audio	portaudio	\N	portaudio-19.6.0_5,1
2020-09-12 21:36:48+02	2020-09-12 21:37:12+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.9_7
2020-09-12 21:37:12+02	2020-09-12 21:38:11+02	devel	oniguruma	\N	oniguruma-6.9.5.r1_1
2020-09-12 21:38:12+02	2020-09-12 21:38:57+02	graphics	lcms	\N	lcms-1.19_6,1
2020-09-12 21:38:57+02	2020-09-12 21:39:17+02	lang	fpc-rtl-objpas	\N	fpc-rtl-objpas-3.0.4_5
2020-09-12 21:39:18+02	2020-09-12 21:39:51+02	www	php73-session	\N	php73-session-7.3.22
2020-09-12 21:39:52+02	2020-09-12 21:40:23+02	archivers	php73-zlib	\N	php73-zlib-7.3.22
2020-09-12 21:40:23+02	2020-09-12 21:41:01+02	audio	espeak	\N	espeak-1.48.04_7
2020-09-12 21:41:02+02	2020-09-12 21:41:30+02	databases	php73-pdo	\N	php73-pdo-7.3.22
2020-09-12 21:41:31+02	2020-09-12 21:41:45+02	devel	dotconf	\N	dotconf-1.3_1
2020-09-12 21:41:46+02	2020-09-12 21:42:04+02	devel	fpc-fcl-res	\N	fpc-fcl-res-3.0.4_4
2020-09-12 21:42:05+02	2020-09-12 21:42:38+02	devel	php73-json	\N	php73-json-7.3.22
2020-09-12 21:42:40+02	2020-09-12 21:43:06+02	graphics	libexif	\N	libexif-0.6.21_5
2020-09-12 21:43:07+02	2020-09-12 21:43:45+02	graphics	libmng	\N	libmng-1.0.10_3
2020-09-12 21:43:46+02	2020-09-12 21:48:55+02	math	fftw3-float	\N	fftw3-float-3.3.8_6
2020-09-12 21:48:55+02	2020-09-12 21:49:19+02	security	libassuan	\N	libassuan-2.5.3
2020-09-12 21:49:20+02	2020-09-12 21:52:53+02	security	libsodium	\N	libsodium-1.0.18
2020-09-12 21:52:53+02	2020-09-12 21:53:42+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-09-12 21:53:43+02	2020-09-12 21:54:07+02	archivers	snappy	\N	snappy-1.1.8
2020-09-12 21:54:09+02	2020-09-12 21:54:59+02	converters	php73-mbstring	\N	php73-mbstring-7.3.22
2020-09-12 21:54:59+02	2020-09-12 22:10:49+02	databases	libmemcached	\N	libmemcached-1.0.18_7
2020-09-12 22:10:49+02	2020-09-12 22:11:18+02	devel	ctags	\N	ctags-5.8
2020-09-12 22:11:19+02	2020-09-12 22:11:39+02	devel	fpc-fcl-base	\N	fpc-fcl-base-3.0.4_4
2020-09-12 22:11:40+02	2020-09-12 22:12:26+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-09-12 22:12:26+02	2020-09-12 22:12:49+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-09-12 22:12:50+02	2020-09-12 22:13:26+02	graphics	php73-gd	\N	php73-gd-7.3.22
2020-09-12 22:13:26+02	2020-09-12 22:14:38+02	math	metis	\N	metis-5.1.0_8
2020-09-12 22:14:39+02	2020-09-12 22:14:54+02	multimedia	libdvdcss	\N	libdvdcss-1.4.2
2020-09-12 22:14:55+02	2020-09-12 22:15:44+02	security	libmcrypt	\N	libmcrypt-2.5.8_3
2020-09-12 22:15:45+02	2020-09-12 22:16:23+02	security	php73-hash	\N	php73-hash-7.3.22
2020-09-12 22:16:23+02	2020-09-12 22:16:59+02	security	php73-openssl	\N	php73-openssl-7.3.22
2020-09-12 22:17:01+02	2020-09-12 22:17:33+02	textproc	php73-xml	\N	php73-xml-7.3.22
2020-09-12 22:17:34+02	2020-09-12 22:17:50+02	audio	alsa-plugins	\N	alsa-plugins-1.1.1_6
2020-09-12 22:17:50+02	2020-09-12 22:18:21+02	converters	php73-iconv	\N	php73-iconv-7.3.22
2020-09-12 22:18:22+02	2020-09-12 22:18:53+02	databases	pecl-memcache	php73	php73-pecl-memcache-4.0.5.2
2020-09-12 22:18:54+02	2020-09-12 22:19:30+02	databases	pecl-memcached	php73	php73-pecl-memcached-3.1.5
2020-09-12 22:19:30+02	2020-09-12 22:20:03+02	databases	php73-pdo_pgsql	\N	php73-pdo_pgsql-7.3.22
2020-09-12 22:20:04+02	2020-09-12 22:20:40+02	databases	php73-pdo_sqlite	\N	php73-pdo_sqlite-7.3.22
2020-09-12 22:20:41+02	2020-09-12 22:21:19+02	databases	php73-pgsql	\N	php73-pgsql-7.3.22
2020-09-12 22:21:20+02	2020-09-12 22:21:54+02	databases	php73-sqlite3	\N	php73-sqlite3-7.3.22
2020-09-12 22:21:55+02	2020-09-12 22:22:08+02	devel	libpci	\N	libpci-3.7.0
2020-09-12 22:22:08+02	2020-09-12 22:22:37+02	devel	php73-gettext	\N	php73-gettext-7.3.22
2020-09-12 22:22:38+02	2020-09-12 22:23:15+02	ftp	php73-curl	\N	php73-curl-7.3.22
2020-09-12 22:23:16+02	2020-09-12 22:23:41+02	math	pecl-bitset	php73	php73-pecl-bitset-3.0.1
2020-09-12 22:23:43+02	2020-09-12 22:24:19+02	math	php73-bcmath	\N	php73-bcmath-7.3.22
2020-09-12 22:24:20+02	2020-09-12 22:24:44+02	multimedia	libdvdread	\N	libdvdread-6.1.0
2020-09-12 22:24:44+02	2020-09-12 22:25:10+02	net	p5-Socket6	\N	p5-Socket6-0.29
2020-09-12 22:25:11+02	2020-09-12 22:28:02+02	net	php73-ldap	\N	php73-ldap-7.3.22
2020-09-12 22:28:03+02	2020-09-12 22:28:35+02	net	php73-soap	\N	php73-soap-7.3.22
2020-09-12 22:28:36+02	2020-09-12 22:29:12+02	net	php73-sockets	\N	php73-sockets-7.3.22
2020-09-12 22:29:13+02	2020-09-12 22:29:39+02	security	pecl-mcrypt	php73	php73-pecl-mcrypt-1.0.3
2020-09-12 22:29:39+02	2020-09-12 22:29:53+02	sysutils	dmidecode	\N	dmidecode-3.2
2020-09-12 22:29:54+02	2020-09-12 22:30:27+02	sysutils	php73-posix	\N	php73-posix-7.3.22
2020-09-12 22:30:28+02	2020-09-12 22:31:30+02	sysutils	policykit	\N	policykit-0.9_10
2020-09-12 22:31:31+02	2020-09-12 22:31:59+02	textproc	php73-ctype	\N	php73-ctype-7.3.22
2020-09-12 22:32:00+02	2020-09-12 22:32:33+02	textproc	php73-dom	\N	php73-dom-7.3.22
2020-09-12 22:32:33+02	2020-09-12 22:33:09+02	textproc	php73-simplexml	\N	php73-simplexml-7.3.22
2020-09-12 22:33:09+02	2020-09-12 22:33:54+02	www	php73-opcache	\N	php73-opcache-7.3.22
2020-09-12 22:33:54+02	2020-09-12 22:34:33+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-09-12 22:34:34+02	2020-09-12 22:34:49+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-09-12 22:34:50+02	2020-09-12 22:35:06+02	devel	libvolume_id	\N	libvolume_id-0.81.1
2020-09-12 22:35:06+02	2020-09-12 22:35:24+02	net	p5-IO-Socket-INET6	\N	p5-IO-Socket-INET6-2.72_1
2020-09-12 22:35:25+02	2020-09-12 22:35:57+02	security	libssh2	\N	libssh2-1.9.0_3,3
2020-09-12 22:35:57+02	2020-09-12 22:36:14+02	security	p5-Digest-HMAC	\N	p5-Digest-HMAC-1.03_1
2020-09-12 22:36:15+02	2020-09-12 22:36:54+02	security	p5-Net-SSLeay	\N	p5-Net-SSLeay-1.88
2020-09-12 22:36:54+02	2020-09-12 22:37:09+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-09-12 22:37:10+02	2020-09-12 22:44:18+02	textproc	source-highlight	\N	source-highlight-3.1.9_1
2020-09-12 22:44:19+02	2020-09-12 22:44:34+02	www	p5-HTML-Tagset	\N	p5-HTML-Tagset-3.20_1
2020-09-12 22:44:35+02	2020-09-12 22:44:52+02	www	p5-Mozilla-CA	\N	p5-Mozilla-CA-20200520
2020-09-12 17:15:05+02	2020-09-12 22:53:12+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-09-12 20:08:47+02	2020-09-12 22:53:43+02	lang	gcc9	\N	gcc9-9.3.0_1
2020-09-12 22:53:14+02	2020-09-12 23:00:21+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-09-12 22:44:53+02	2020-09-12 23:02:33+02	devel	gdb	py37	gdb-9.2_1
2020-09-12 23:00:22+02	2020-09-12 23:03:08+02	graphics	cairo	\N	cairo-1.17.2,2
2020-09-12 23:02:33+02	2020-09-12 23:03:17+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-09-12 23:03:18+02	2020-09-12 23:04:11+02	graphics	libGLU	\N	libGLU-9.0.1
2020-09-12 23:03:08+02	2020-09-12 23:04:36+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-09-12 23:04:13+02	2020-09-12 23:07:07+02	graphics	poppler	\N	poppler-20.09.0
2020-09-12 23:07:07+02	2020-09-12 23:08:09+02	net	avahi-app	\N	avahi-app-0.7_3
2020-09-12 23:04:36+02	2020-09-12 23:09:01+02	print	harfbuzz	\N	harfbuzz-2.7.2
2020-09-12 23:08:10+02	2020-09-12 23:09:38+02	print	cups	\N	cups-2.3.3_1
2020-09-12 23:09:39+02	2020-09-12 23:10:09+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-09-12 23:09:02+02	2020-09-12 23:11:43+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-09-12 23:11:44+02	2020-09-12 23:13:03+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-09-12 23:13:04+02	2020-09-12 23:13:33+02	accessibility	atk	\N	atk-2.36.0
2020-09-12 23:10:10+02	2020-09-12 23:13:57+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-09-12 23:13:34+02	2020-09-12 23:14:00+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-09-12 23:14:01+02	2020-09-12 23:14:57+02	sysutils	polkit	\N	polkit-0.116
2020-09-12 23:14:57+02	2020-09-12 23:16:00+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-09-12 23:16:01+02	2020-09-12 23:16:22+02	devel	libgudev	\N	libgudev-233
2020-09-12 23:16:23+02	2020-09-12 23:16:41+02	x11	libwacom	\N	libwacom-1.4.1
2020-09-12 23:13:58+02	2020-09-12 23:18:35+02	graphics	graphviz	\N	graphviz-2.44.1_2
2020-09-12 23:16:43+02	2020-09-12 23:19:14+02	graphics	colord	\N	colord-1.3.5
2020-09-12 23:19:14+02	2020-09-12 23:19:48+02	x11	libinput	\N	libinput-1.15.6
2020-09-12 23:19:49+02	2020-09-12 23:20:33+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-09-12 23:18:36+02	2020-09-12 23:22:02+02	lang	vala	\N	vala-0.40.15,1
2020-09-12 23:20:34+02	2020-09-12 23:22:09+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-09-12 23:22:03+02	2020-09-12 23:23:10+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-09-12 22:53:44+02	2020-09-12 23:27:27+02	math	openblas	\N	openblas-0.3.10,1
2020-09-12 23:27:28+02	2020-09-12 23:28:04+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.2
2020-09-12 23:23:10+02	2020-09-12 23:35:53+02	x11-toolkits	gtk30	\N	gtk3-3.24.20
2020-09-12 23:28:04+02	2020-09-12 23:37:17+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-12 23:37:18+02	2020-09-12 23:37:45+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-09-12 23:35:55+02	2020-09-12 23:38:14+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-09-12 23:37:46+02	2020-09-12 23:38:17+02	net	glib-networking	\N	glib-networking-2.56.1_3
2020-09-12 23:38:17+02	2020-09-12 23:39:08+02	devel	dconf	\N	dconf-0.36.0
2020-09-12 23:22:09+02	2020-09-12 23:39:20+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-09-12 23:39:09+02	2020-09-12 23:40:26+02	devel	libsoup	\N	libsoup-2.62.3
2020-09-12 23:38:15+02	2020-09-12 23:40:31+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-09-12 23:40:26+02	2020-09-12 23:40:53+02	devel	json-glib	\N	json-glib-1.4.4
2020-09-12 23:40:31+02	2020-09-12 23:43:06+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-09-12 23:40:53+02	2020-09-12 23:44:05+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-09-12 23:43:06+02	2020-09-12 23:45:13+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-09-12 23:45:14+02	2020-09-12 23:45:37+02	devel	libnotify	\N	libnotify-0.7.8
2020-09-12 23:45:37+02	2020-09-12 23:46:48+02	security	libsecret	\N	libsecret-0.18.6_1
2020-09-12 23:39:21+02	2020-09-12 23:48:19+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-09-12 23:46:49+02	2020-09-12 23:48:22+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-12 23:48:22+02	2020-09-12 23:50:08+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-09-12 23:44:06+02	2020-09-12 23:51:00+02	print	texlive-base	\N	texlive-base-20150521_55
2020-09-12 23:50:09+02	2020-09-12 23:52:17+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-12 23:52:18+02	2020-09-12 23:53:31+02	graphics	frei0r	\N	frei0r-1.6.1
2020-09-12 23:53:31+02	2020-09-12 23:53:53+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-09-12 23:53:53+02	2020-09-12 23:54:11+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-09-12 23:54:11+02	2020-09-13 00:10:49+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_2,1
2020-09-13 00:10:50+02	2020-09-13 00:11:18+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-12 23:48:19+02	2020-09-13 00:11:19+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-09-12 23:51:01+02	2020-09-13 00:11:30+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-13 00:11:18+02	2020-09-13 00:12:32+02	graphics	graphene	\N	graphene-1.10.0
2020-09-13 00:11:20+02	2020-09-13 00:13:00+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-09-13 00:13:01+02	2020-09-13 00:13:14+02	www	libwpe	\N	libwpe-1.4.0.1
2020-09-13 00:11:31+02	2020-09-13 00:13:50+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-13 00:13:15+02	2020-09-13 00:14:48+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-13 00:13:50+02	2020-09-13 00:14:50+02	net	geoclue	\N	geoclue-2.5.5
2020-09-13 00:14:48+02	2020-09-13 00:15:06+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-09-13 00:14:50+02	2020-09-13 00:15:41+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.73.0
2020-09-13 00:15:41+02	2020-09-13 00:16:17+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-09-13 00:16:17+02	2020-09-13 00:16:50+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-09-13 00:16:50+02	2020-09-13 00:17:31+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-09-13 00:12:33+02	2020-09-13 00:17:37+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-13 00:17:31+02	2020-09-13 00:18:00+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-13 00:18:00+02	2020-09-13 00:19:29+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-09-13 00:19:30+02	2020-09-13 00:19:46+02	print	libraqm	\N	libraqm-0.6.0
2020-09-13 00:19:47+02	2020-09-13 00:20:50+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-09-13 00:20:51+02	2020-09-13 00:26:13+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-13 00:17:38+02	2020-09-13 00:26:45+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-13 00:26:14+02	2020-09-13 00:27:10+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-09-13 00:26:46+02	2020-09-13 00:35:49+02	devel	tbb	\N	tbb-2020.3
2020-09-13 00:35:50+02	2020-09-13 00:38:01+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.73.0
2020-09-13 00:38:01+02	2020-09-13 00:40:59+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-13 00:41:00+02	2020-09-13 00:41:52+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-13 00:27:10+02	2020-09-13 00:42:26+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-09-13 00:42:26+02	2020-09-13 00:43:23+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-09-13 00:43:24+02	2020-09-13 00:44:27+02	graphics	pecl-imagick-im7	php73	php73-pecl-imagick-im7-3.4.4
2020-09-13 00:41:53+02	2020-09-13 00:44:55+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-13 00:44:56+02	2020-09-13 00:46:59+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-13 00:46:59+02	2020-09-13 00:48:09+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-13 00:48:09+02	2020-09-13 00:50:06+02	devel	kf5-kconfig	\N	kf5-kconfig-5.73.0
2020-09-13 00:50:08+02	2020-09-13 00:51:34+02	net-mgmt	icingaweb2	php73	icingaweb2-php73-2.8.2
2020-09-13 00:51:34+02	2020-09-13 00:52:44+02	archivers	kf5-karchive	\N	kf5-karchive-5.73.0
2020-09-13 00:52:45+02	2020-09-13 00:54:21+02	devel	kf5-ki18n	\N	kf5-ki18n-5.73.0
2020-09-13 00:54:21+02	2020-09-13 00:55:06+02	devel	libsigc++20	\N	libsigc++-2.10.3
2020-09-13 00:55:07+02	2020-09-13 00:56:05+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-09-13 00:56:05+02	2020-09-13 00:58:09+02	graphics	ilmbase	\N	ilmbase-2.5.3
2020-09-13 00:58:10+02	2020-09-13 00:58:16+02	java	javavmwrapper	\N	javavmwrapper-2.7.6
2020-09-13 00:58:17+02	2020-09-13 00:59:01+02	misc	e2fsprogs-libuuid	\N	e2fsprogs-libuuid-1.45.6
2020-09-13 00:59:02+02	2020-09-13 00:59:26+02	multimedia	libdvdnav	\N	libdvdnav-6.1.0
2020-09-13 00:59:27+02	2020-09-13 00:59:47+02	security	p5-IO-Socket-SSL	\N	p5-IO-Socket-SSL-2.068
2020-09-13 00:59:48+02	2020-09-13 01:01:16+02	sysutils	hal	\N	hal-0.5.14_34
2020-09-13 01:01:16+02	2020-09-13 01:01:42+02	www	p5-HTML-Parser	\N	p5-HTML-Parser-3.72
2020-09-13 01:01:42+02	2020-09-13 01:02:31+02	www	py-django22	py37	py37-django22-2.2.16
2020-09-13 01:02:32+02	2020-09-13 01:03:43+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-09-13 01:03:43+02	2020-09-13 01:07:47+02	x11-servers	xorg-server	\N	xorg-server-1.20.8_4,1
2020-09-13 01:07:48+02	2020-09-13 01:09:58+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-09-13 01:09:59+02	2020-09-13 01:11:34+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.73.0
2020-09-13 01:11:35+02	2020-09-13 01:13:08+02	devel	libgit2	\N	libgit2-1.0.1
2020-09-13 01:13:09+02	2020-09-13 01:13:16+02	devel	libsysinfo	\N	libsysinfo-0.0.3_2
2020-09-13 01:13:17+02	2020-09-13 01:21:55+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-09-13 01:21:56+02	2020-09-13 01:23:18+02	devel	re2	\N	re2-20200401
2020-09-13 01:23:19+02	2020-09-13 01:26:12+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-09-13 01:26:13+02	2020-09-13 01:26:55+02	devel	yasm	\N	yasm-1.3.0
2020-09-13 01:26:56+02	2020-09-13 01:28:02+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-09-13 01:28:03+02	2020-09-13 01:28:22+02	lang	fpc-rtl-extra	\N	fpc-rtl-extra-3.0.4_4
2020-09-13 01:28:23+02	2020-09-13 01:28:44+02	textproc	p5-XML-NamespaceSupport	\N	p5-XML-NamespaceSupport-1.12
2020-09-13 01:28:45+02	2020-09-13 01:29:06+02	textproc	p5-XML-SAX-Base	\N	p5-XML-SAX-Base-1.09
2020-09-13 01:29:06+02	2020-09-13 01:30:25+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.73.0
2020-09-13 01:30:25+02	2020-09-13 01:30:37+02	x11	xbitmaps	\N	xbitmaps-1.1.2
2020-09-13 01:30:37+02	2020-09-13 01:31:41+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-13 01:31:42+02	2020-09-13 01:32:04+02	audio	liba52	\N	liba52-0.7.4_3
2020-09-13 01:32:04+02	2020-09-13 01:32:33+02	audio	libsoxr	\N	libsoxr-0.1.3_2
2020-09-13 01:32:34+02	2020-09-13 01:37:10+02	audio	webrtc-audio-processing	\N	webrtc-audio-processing-0.3.1_2
2020-09-13 01:37:11+02	2020-09-13 01:37:28+02	converters	fpc-iconvenc	\N	fpc-iconvenc-3.0.4_4
2020-09-13 01:37:28+02	2020-09-13 01:45:25+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-09-13 00:15:06+02	2020-09-13 01:48:48+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-13 01:45:26+02	2020-09-13 01:50:15+02	graphics	openexr	\N	openexr-2.5.3
2020-09-13 01:50:16+02	2020-09-13 01:53:45+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-09-13 01:53:46+02	2020-09-13 01:55:44+02	java	bootstrap-openjdk8	\N	bootstrap-openjdk8-r450802_1
2020-09-13 01:55:45+02	2020-09-13 01:55:53+02	java	java-zoneinfo	\N	java-zoneinfo-2020.a
2020-09-13 01:55:53+02	2020-09-13 01:56:19+02	multimedia	libdca	\N	libdca-0.0.6_1
2020-09-13 01:56:19+02	2020-09-13 01:56:27+02	net	avahi-header	\N	avahi-header-0.7
2020-09-13 01:56:27+02	2020-09-13 01:57:43+02	security	libssh	\N	libssh-0.9.4
2020-09-13 01:57:43+02	2020-09-13 01:58:11+02	security	p5-GSSAPI	\N	p5-GSSAPI-0.28_1
2020-09-13 01:58:12+02	2020-09-13 01:58:33+02	textproc	p5-XML-SAX	\N	p5-XML-SAX-1.02
2020-09-13 01:58:33+02	2020-09-13 01:58:49+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-09-13 01:58:50+02	2020-09-13 01:59:09+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-09-13 01:59:09+02	2020-09-13 01:59:43+02	x11-fonts	libXfont	\N	libXfont-1.5.4_2,2
2020-09-13 01:59:43+02	2020-09-13 02:02:04+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.73.0
2020-09-13 02:02:05+02	2020-09-13 02:05:41+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-09-13 02:05:42+02	2020-09-13 02:05:54+02	devel	cvsps	\N	cvsps-2.1_2
2020-09-13 02:05:54+02	2020-09-13 02:06:18+02	devel	fpc-fcl-xml	\N	fpc-fcl-xml-3.0.4_4
2020-09-13 02:06:18+02	2020-09-13 02:07:24+02	devel	hwloc	\N	hwloc-1.11.11
2020-09-13 02:07:25+02	2020-09-13 02:08:27+02	devel	kf5-kcrash	\N	kf5-kcrash-5.73.0
2020-09-13 01:48:49+02	2020-09-13 02:09:26+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-13 02:08:28+02	2020-09-13 02:09:27+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.73.0
2020-09-13 02:09:27+02	2020-09-13 02:09:36+02	devel	py-six	py27	py27-six-1.15.0
2020-09-13 02:09:28+02	2020-09-13 02:11:16+02	dns	c-ares	\N	c-ares-1.16.1
2020-09-13 02:09:36+02	2020-09-13 02:13:00+02	graphics	glew	\N	glew-2.2.0
2020-09-13 02:13:01+02	2020-09-13 02:13:25+02	lang	p5-Error	\N	p5-Error-0.17029
2020-09-13 02:13:26+02	2020-09-13 02:15:36+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-09-13 02:15:36+02	2020-09-13 02:16:09+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-09-13 02:16:09+02	2020-09-13 02:17:06+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-09-13 02:17:07+02	2020-09-13 02:18:26+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-13 02:18:27+02	2020-09-13 02:20:50+02	net	openldap24-sasl-client	\N	openldap-sasl-client-2.4.51
2020-09-13 02:20:51+02	2020-09-13 02:22:31+02	security	munge	\N	munge-0.5.14_1
2020-09-13 02:22:32+02	2020-09-13 02:22:57+02	security	p5-Authen-SASL	\N	p5-Authen-SASL-2.16_1
2020-09-13 02:22:58+02	2020-09-13 02:24:36+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-13 02:24:37+02	2020-09-13 02:26:05+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.73.0
2020-09-13 02:26:06+02	2020-09-13 02:26:26+02	textproc	man2html	\N	man2html-3.0.1_2
2020-09-13 02:26:27+02	2020-09-13 02:26:45+02	textproc	p5-XML-SAX-Expat	\N	p5-XML-SAX-Expat-0.51_1
2020-09-13 02:26:47+02	2020-09-13 02:27:11+02	www	p5-CGI	\N	p5-CGI-4.50
2020-09-13 00:44:28+02	2020-09-13 02:31:55+02	math	suitesparse	\N	suitesparse-5.8.1
2020-09-13 02:27:12+02	2020-09-13 02:31:58+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-13 02:11:16+02	2020-09-13 02:32:18+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-09-13 02:31:58+02	2020-09-13 02:32:24+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-09-13 02:32:19+02	2020-09-13 02:33:05+02	audio	libmikmod	\N	libmikmod-3.3.11.1_1
2020-09-13 02:32:24+02	2020-09-13 02:33:23+02	audio	mpg123	\N	mpg123-1.26.3
2020-09-13 02:33:06+02	2020-09-13 02:33:25+02	audio	timidity	\N	timidity-0.2i_1
2020-09-13 02:33:23+02	2020-09-13 02:34:39+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-09-13 02:31:56+02	2020-09-13 02:35:27+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-09-13 02:35:28+02	2020-09-13 02:35:46+02	devel	jansson	\N	jansson-2.13.1
2020-09-13 02:33:26+02	2020-09-13 02:36:44+02	databases	mysql57-client	\N	mysql57-client-5.7.31_1
2020-09-13 02:35:46+02	2020-09-13 02:36:50+02	devel	kf5-kauth	\N	kf5-kauth-5.73.0
2020-09-13 02:36:50+02	2020-09-13 02:37:03+02	devel	py-pytz	py27	py27-pytz-2020.1,1
2020-09-13 02:36:44+02	2020-09-13 02:37:56+02	devel	kf5-kservice	\N	kf5-kservice-5.73.0
2020-09-13 02:37:04+02	2020-09-13 02:38:02+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-13 02:37:57+02	2020-09-13 02:38:05+02	devel	tradcpp	\N	tradcpp-0.5.3
2020-09-13 02:38:06+02	2020-09-13 02:38:20+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-09-13 02:38:03+02	2020-09-13 02:38:37+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-09-13 02:38:20+02	2020-09-13 02:38:44+02	math	glm	\N	glm-0.9.8.5_4,1
2020-09-13 02:38:38+02	2020-09-13 02:39:40+02	net	remmina	\N	remmina-1.3.9
2020-09-13 02:39:41+02	2020-09-13 02:39:53+02	security	pinentry-tty	\N	pinentry-tty-1.1.0
2020-09-13 02:38:45+02	2020-09-13 02:41:00+02	security	gcr	\N	gcr-3.28.0_1
2020-09-13 02:41:01+02	2020-09-13 02:43:14+02	textproc	aspell	\N	aspell-0.60.8,1
2020-09-13 02:43:15+02	2020-09-13 02:44:53+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-09-13 02:44:54+02	2020-09-13 02:45:15+02	textproc	p5-XML-Simple	\N	p5-XML-Simple-2.25
2020-09-13 02:45:15+02	2020-09-13 02:46:03+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-09-13 02:46:03+02	2020-09-13 02:46:20+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-09-13 02:46:21+02	2020-09-13 02:46:34+02	x11	xrdb	\N	xrdb-1.2.0
2020-09-13 02:46:35+02	2020-09-13 02:47:03+02	x11-fonts	font-misc-misc	\N	font-misc-misc-1.1.2_4
2020-09-13 02:39:53+02	2020-09-13 02:47:03+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-09-13 02:47:04+02	2020-09-13 02:47:25+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-09-13 02:47:26+02	2020-09-13 02:47:51+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-09-13 02:47:03+02	2020-09-13 02:47:59+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.73.0
2020-09-13 02:47:59+02	2020-09-13 02:48:26+02	audio	libmysofa	\N	libmysofa-1.1
2020-09-13 02:47:52+02	2020-09-13 02:48:27+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-09-13 02:48:27+02	2020-09-13 02:48:36+02	devel	gccmakedep	\N	gccmakedep-1.0.3
2020-09-13 02:48:27+02	2020-09-13 02:48:48+02	audio	speex	\N	speex-1.2.0,1
2020-09-13 02:48:48+02	2020-09-13 02:49:00+02	devel	makedepend	\N	makedepend-1.0.6,1
2020-09-13 02:49:01+02	2020-09-13 02:49:15+02	devel	npth	\N	npth-1.6
2020-09-13 02:48:37+02	2020-09-13 02:49:34+02	devel	libgtop	\N	libgtop-2.40.0
2020-09-13 02:49:16+02	2020-09-13 02:49:51+02	devel	py-babel	py27	py27-Babel-2.8.0
2020-09-13 02:49:51+02	2020-09-13 02:50:31+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-09-13 02:49:35+02	2020-09-13 02:53:29+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-13 02:50:32+02	2020-09-13 02:53:41+02	graphics	exiv2	\N	exiv2-0.27.3,1
2020-09-13 02:53:30+02	2020-09-13 02:53:59+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-09-13 02:53:59+02	2020-09-13 02:54:14+02	lang	lua53	\N	lua53-5.3.5_3
2020-09-13 02:53:41+02	2020-09-13 02:54:14+02	graphics	libspiro	\N	libspiro-20190731,1
2020-09-13 02:54:14+02	2020-09-13 02:54:57+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-09-13 02:54:14+02	2020-09-13 02:54:59+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-13 02:54:58+02	2020-09-13 02:55:21+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-09-13 02:55:00+02	2020-09-13 02:55:43+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-09-13 02:55:21+02	2020-09-13 02:55:54+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-09-13 02:55:44+02	2020-09-13 02:57:45+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-09-13 02:57:45+02	2020-09-13 02:58:46+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-13 02:58:46+02	2020-09-13 02:59:26+02	security	libksba	\N	libksba-1.4.0
2020-09-13 02:59:26+02	2020-09-13 02:59:33+02	security	pinentry	\N	pinentry-1.1.0_6
2020-09-13 02:59:34+02	2020-09-13 02:59:43+02	textproc	py-MarkupSafe	py27	py27-MarkupSafe-1.1.1
2020-09-13 02:59:44+02	2020-09-13 03:00:00+02	x11	luit	\N	luit-20190106_1
2020-09-13 03:00:02+02	2020-09-13 03:00:16+02	x11	xorg-cf-files	\N	xorg-cf-files-1.0.6
2020-09-13 03:00:16+02	2020-09-13 03:00:54+02	x11-fonts	font-adobe-100dpi	\N	font-adobe-100dpi-1.0.3_4
2020-09-13 03:00:54+02	2020-09-13 03:01:23+02	x11-themes	icon-naming-utils	\N	icon-naming-utils-0.8.90_1
2020-09-13 03:01:23+02	2020-09-13 03:04:19+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.73.0
2020-09-13 03:04:19+02	2020-09-13 03:05:41+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-09-13 03:05:42+02	2020-09-13 03:07:07+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.73.0
2020-09-13 03:07:08+02	2020-09-13 03:08:14+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.73.0
2020-09-13 03:08:16+02	2020-09-13 03:10:04+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-13 03:10:04+02	2020-09-13 03:13:07+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-13 03:13:08+02	2020-09-13 03:14:17+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-09-13 02:55:54+02	2020-09-13 03:14:42+02	net	openmpi	\N	openmpi-4.0.5
2020-09-13 03:14:17+02	2020-09-13 03:16:13+02	audio	openal-soft	\N	openal-soft-1.20.1_2
2020-09-13 03:16:13+02	2020-09-13 03:16:47+02	devel	imake	\N	imake-1.0.8,1
2020-09-13 03:14:42+02	2020-09-13 03:17:00+02	devel	apr1	\N	apr-1.7.0.1.6.1_1
2020-09-13 03:16:48+02	2020-09-13 03:17:11+02	devel	p5-TimeDate	\N	p5-TimeDate-2.33,1
2020-09-13 03:17:04+02	2020-09-13 03:17:21+02	devel	py-Jinja2	py27	py27-Jinja2-2.10.1
2020-09-13 03:17:11+02	2020-09-13 03:17:27+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-09-13 03:17:21+02	2020-09-13 03:17:33+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-09-13 03:17:28+02	2020-09-13 03:17:40+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-09-13 03:17:33+02	2020-09-13 03:17:56+02	devel	yajl	\N	yajl-2.1.0
2020-09-13 03:17:56+02	2020-09-13 03:18:11+02	lang	erlang-man	\N	erlang-man-21.3_2
2020-09-13 03:18:11+02	2020-09-13 03:21:14+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-13 03:17:42+02	2020-09-13 03:23:55+02	graphics	freeimage	\N	freeimage-3.18.0_1
2020-09-13 03:23:56+02	2020-09-13 03:24:37+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-13 03:24:37+02	2020-09-13 03:25:00+02	print	gl2ps	\N	gl2ps-1.4.0
2020-09-13 03:25:01+02	2020-09-13 03:28:26+02	security	gnupg	\N	gnupg-2.2.23
2020-09-13 03:28:27+02	2020-09-13 03:28:59+02	textproc	yelp-xsl	\N	yelp-xsl-3.28.0
2020-09-13 03:28:59+02	2020-09-13 03:29:18+02	x11	xauth	\N	xauth-1.1
2020-09-13 03:29:18+02	2020-09-13 03:29:58+02	x11	xterm	\N	xterm-359
2020-09-13 03:30:03+02	2020-09-13 03:30:52+02	x11-fonts	font-adobe-75dpi	\N	font-adobe-75dpi-1.0.3_4
2020-09-13 03:30:53+02	2020-09-13 03:31:08+02	x11-fonts	font-alias	\N	font-alias-1.0.4
2020-09-13 03:31:09+02	2020-09-13 03:32:48+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.73.0
2020-09-13 03:21:15+02	2020-09-13 03:35:05+02	math	vtk8	\N	vtk8-8.2.0
2020-09-13 03:32:49+02	2020-09-13 03:35:19+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-09-13 03:35:19+02	2020-09-13 03:35:36+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.9_6
2020-09-13 03:35:37+02	2020-09-13 03:36:19+02	archivers	minizip	\N	minizip-1.2.11
2020-09-13 03:35:06+02	2020-09-13 03:36:27+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-09-13 03:36:20+02	2020-09-13 03:36:44+02	audio	libcddb	\N	libcddb-1.3.2_4
2020-09-13 03:36:27+02	2020-09-13 03:46:50+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-13 03:36:45+02	2020-09-13 03:54:22+02	databases	firebird25-client	\N	firebird25-client-2.5.8_6
2020-09-13 03:54:23+02	2020-09-13 03:54:39+02	devel	popt	\N	popt-1.18_1
2020-09-13 03:54:39+02	2020-09-13 03:54:52+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-09-13 03:54:54+02	2020-09-13 03:56:19+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-09-13 03:56:19+02	2020-09-13 03:59:01+02	graphics	netpbm	\N	netpbm-10.91.01
2020-09-13 03:59:02+02	2020-09-13 04:29:08+02	lang	erlang	\N	erlang-21.3.8.17,4
2020-09-13 04:29:09+02	2020-09-13 04:29:16+02	multimedia	gmp-api	\N	gmp-api-58
2020-09-13 04:29:17+02	2020-09-13 04:32:31+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-09-13 04:32:32+02	2020-09-13 04:38:18+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-09-13 04:38:18+02	2020-09-13 04:42:49+02	net	libmateweather	\N	libmateweather-1.24.0
2020-09-13 04:42:50+02	2020-09-13 04:43:35+02	security	gpgme	\N	gpgme-1.14.0
2020-09-13 04:43:36+02	2020-09-13 04:44:22+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-09-13 04:44:23+02	2020-09-13 04:48:23+02	textproc	exempi	\N	exempi-2.2.2_1
2020-09-13 04:48:24+02	2020-09-13 04:49:37+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.73.0
2020-09-13 04:49:37+02	2020-09-13 04:51:29+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-09-13 04:51:30+02	2020-09-13 04:51:51+02	www	p5-HTTP-Date	\N	p5-HTTP-Date-6.05
2020-09-13 04:51:51+02	2020-09-13 05:16:20+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-13 05:16:21+02	2020-09-13 05:16:36+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-09-13 05:16:36+02	2020-09-13 05:16:56+02	x11-fonts	font-adobe-utopia-100dpi	\N	font-adobe-utopia-100dpi-1.0.4_4
2020-09-13 05:16:57+02	2020-09-13 05:17:16+02	x11-fonts	font-adobe-utopia-75dpi	\N	font-adobe-utopia-75dpi-1.0.4_4
2020-09-13 05:17:16+02	2020-09-13 05:17:28+02	x11-fonts	font-adobe-utopia-type1	\N	font-adobe-utopia-type1-1.0.4_4
2020-09-13 05:17:28+02	2020-09-13 05:17:39+02	x11-fonts	font-arabic-misc	\N	font-arabic-misc-1.0.3_4
2020-09-13 05:17:39+02	2020-09-13 05:18:14+02	x11-fonts	font-bh-100dpi	\N	font-bh-100dpi-1.0.3_4
2020-09-13 05:18:14+02	2020-09-13 05:18:43+02	x11-fonts	font-bh-75dpi	\N	font-bh-75dpi-1.0.3_4
2020-09-13 05:18:43+02	2020-09-13 05:19:02+02	x11-fonts	font-bh-lucidatypewriter-100dpi	\N	font-bh-lucidatypewriter-100dpi-1.0.3_4
2020-09-13 05:19:02+02	2020-09-13 05:19:19+02	x11-fonts	font-bh-lucidatypewriter-75dpi	\N	font-bh-lucidatypewriter-75dpi-1.0.3_4
2020-09-13 05:19:19+02	2020-09-13 05:19:32+02	x11-fonts	font-bh-type1	\N	font-bh-type1-1.0.3_4
2020-09-13 05:19:32+02	2020-09-13 05:19:43+02	x11-fonts	font-bitstream-100dpi	\N	font-bitstream-100dpi-1.0.3_4
2020-09-13 05:19:44+02	2020-09-13 05:19:54+02	x11-fonts	font-bitstream-75dpi	\N	font-bitstream-75dpi-1.0.3_4
2020-09-13 05:19:55+02	2020-09-13 05:20:07+02	x11-fonts	font-bitstream-type1	\N	font-bitstream-type1-1.0.3_4
2020-09-13 05:20:08+02	2020-09-13 05:20:18+02	x11-fonts	font-cronyx-cyrillic	\N	font-cronyx-cyrillic-1.0.3_4
2020-09-13 05:20:19+02	2020-09-13 05:20:30+02	x11-fonts	font-cursor-misc	\N	font-cursor-misc-1.0.3_4
2020-09-13 05:20:31+02	2020-09-13 05:20:42+02	x11-fonts	font-daewoo-misc	\N	font-daewoo-misc-1.0.3_4
2020-09-13 05:20:43+02	2020-09-13 05:20:53+02	x11-fonts	font-dec-misc	\N	font-dec-misc-1.0.3_4
2020-09-13 05:20:54+02	2020-09-13 05:21:06+02	x11-fonts	font-ibm-type1	\N	font-ibm-type1-1.0.3_4
2020-09-13 05:21:07+02	2020-09-13 05:21:18+02	x11-fonts	font-isas-misc	\N	font-isas-misc-1.0.3_4
2020-09-13 05:21:19+02	2020-09-13 05:21:31+02	x11-fonts	font-jis-misc	\N	font-jis-misc-1.0.3_4
2020-09-13 05:21:32+02	2020-09-13 05:21:42+02	x11-fonts	font-micro-misc	\N	font-micro-misc-1.0.3_4
2020-09-13 05:21:43+02	2020-09-13 05:21:53+02	x11-fonts	font-misc-cyrillic	\N	font-misc-cyrillic-1.0.3_4
2020-09-13 05:21:54+02	2020-09-13 05:22:05+02	x11-fonts	font-mutt-misc	\N	font-mutt-misc-1.0.3_4
2020-09-13 05:22:06+02	2020-09-13 05:22:19+02	x11-fonts	font-schumacher-misc	\N	font-schumacher-misc-1.1.2_4
2020-09-13 05:22:20+02	2020-09-13 05:22:32+02	x11-fonts	font-screen-cyrillic	\N	font-screen-cyrillic-1.0.4_4
2020-09-13 05:22:33+02	2020-09-13 05:22:43+02	x11-fonts	font-sony-misc	\N	font-sony-misc-1.0.3_4
2020-09-13 05:22:44+02	2020-09-13 05:22:54+02	x11-fonts	font-sun-misc	\N	font-sun-misc-1.0.3_4
2020-09-13 05:22:55+02	2020-09-13 05:23:06+02	x11-fonts	font-winitzki-cyrillic	\N	font-winitzki-cyrillic-1.0.3_4
2020-09-13 05:23:06+02	2020-09-13 05:23:18+02	x11-fonts	font-xfree86-type1	\N	font-xfree86-type1-1.0.4_4
2020-09-13 05:23:18+02	2020-09-13 05:24:41+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-13 05:24:42+02	2020-09-13 05:25:46+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.73.0
2020-09-13 05:25:47+02	2020-09-13 05:26:17+02	audio	celt	\N	celt-0.11.3_3
2020-09-13 05:26:18+02	2020-09-13 05:26:46+02	audio	libmad	\N	libmad-0.15.1b_7
2020-09-13 05:26:47+02	2020-09-13 05:27:23+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-09-13 05:27:24+02	2020-09-13 05:35:23+02	cad	ngspice_rework	shlib	ngspice_rework-shlib-32
2020-09-13 05:35:23+02	2020-09-13 05:35:42+02	converters	p5-Encode-Locale	\N	p5-Encode-Locale-1.05
2020-09-13 05:35:43+02	2020-09-13 05:35:59+02	databases	fpc-dblib	\N	fpc-dblib-3.0.4_4
2020-09-13 05:35:59+02	2020-09-13 05:36:18+02	databases	fpc-ibase	\N	fpc-ibase-3.0.4_5
2020-09-13 05:36:19+02	2020-09-13 05:36:53+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-09-13 05:36:54+02	2020-09-13 05:37:10+02	databases	fpc-odbc	\N	fpc-odbc-3.0.4_4
2020-09-13 05:37:11+02	2020-09-13 05:37:28+02	databases	fpc-oracle	\N	fpc-oracle-3.0.4_4
2020-09-13 05:37:28+02	2020-09-13 05:37:45+02	databases	fpc-postgres	\N	fpc-postgres-3.0.4_6
2020-09-13 05:37:46+02	2020-09-13 05:38:02+02	databases	fpc-pxlib	\N	fpc-pxlib-3.0.4_4
2020-09-13 05:38:03+02	2020-09-13 05:38:21+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-09-13 05:38:22+02	2020-09-13 05:38:47+02	devel	autoconf-archive	\N	autoconf-archive-0.2019.01.06
2020-09-13 05:38:47+02	2020-09-13 05:39:03+02	devel	fpc-fcl-async	\N	fpc-fcl-async-3.0.4_4
2020-09-13 05:39:04+02	2020-09-13 05:39:24+02	devel	fpc-fcl-json	\N	fpc-fcl-json-3.0.4_5
2020-09-13 05:39:24+02	2020-09-13 05:39:45+02	devel	fpc-fcl-passrc	\N	fpc-fcl-passrc-3.0.4_5
2020-09-13 05:39:46+02	2020-09-13 05:40:15+02	devel	gamin	\N	gamin-0.1.10_10
2020-09-13 05:40:16+02	2020-09-13 05:41:12+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-09-13 05:41:13+02	2020-09-13 05:42:06+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-13 05:42:07+02	2020-09-13 05:43:17+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-09-13 05:43:18+02	2020-09-13 05:43:36+02	devel	netsurf-buildsystem	\N	netsurf-buildsystem-1.8
2020-09-13 05:43:37+02	2020-09-13 05:43:56+02	devel	p5-Clone	\N	p5-Clone-0.45
2020-09-13 05:43:57+02	2020-09-13 05:44:17+02	devel	p5-IO-HTML	\N	p5-IO-HTML-1.001_1
2020-09-13 05:44:18+02	2020-09-13 05:44:31+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-09-13 05:44:31+02	2020-09-13 05:44:43+02	devel	py-pycparser	py27	py27-pycparser-2.20
2020-09-13 05:44:44+02	2020-09-13 05:46:31+02	devel	talloc	\N	talloc-2.3.1
2020-09-13 05:46:31+02	2020-09-13 05:52:59+02	lang	go	\N	go-1.15.2,1
2020-09-13 05:52:59+02	2020-09-13 06:07:37+02	multimedia	assimp	\N	assimp-5.0.1
2020-09-13 06:07:37+02	2020-09-13 06:11:32+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-09-13 06:11:33+02	2020-09-13 06:12:28+02	multimedia	openh264	\N	openh264-2.1.1,2
2020-09-13 06:12:28+02	2020-09-13 06:13:33+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-09-13 06:13:34+02	2020-09-13 06:14:11+02	net	norm	\N	norm-1.5r6_1
2020-09-13 06:14:12+02	2020-09-13 06:15:29+02	net	openpgm	\N	openpgm-5.2.122_6
2020-09-13 06:15:30+02	2020-09-13 06:16:28+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-13 06:16:29+02	2020-09-13 06:16:45+02	security	fpc-openssl	\N	fpc-openssl-3.0.4_4
2020-09-13 06:16:46+02	2020-09-13 06:17:53+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-09-13 06:17:53+02	2020-09-13 06:18:11+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7_1
2020-09-13 06:18:12+02	2020-09-13 06:19:15+02	sysutils	upower	\N	upower-0.99.11
2020-09-13 06:19:15+02	2020-09-13 06:20:21+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-09-13 06:20:22+02	2020-09-13 06:20:43+02	www	p5-LWP-MediaTypes	\N	p5-LWP-MediaTypes-6.04
2020-09-13 06:20:44+02	2020-09-13 06:43:49+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-09-13 06:43:52+02	2020-09-13 06:44:07+02	x11	appres	\N	appres-1.0.5
2020-09-13 06:44:07+02	2020-09-13 06:44:26+02	x11	bitmap	\N	bitmap-1.0.9
2020-09-13 06:44:26+02	2020-09-13 06:44:40+02	x11	iceauth	\N	iceauth-1.0.8_2
2020-09-13 06:44:40+02	2020-09-13 06:45:50+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.73.0
2020-09-13 06:45:51+02	2020-09-13 06:46:28+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-09-13 06:46:28+02	2020-09-13 06:50:57+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-13 06:50:58+02	2020-09-13 06:51:13+02	x11	sessreg	\N	sessreg-1.1.2
2020-09-13 06:51:13+02	2020-09-13 06:51:27+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-09-13 06:51:28+02	2020-09-13 06:51:41+02	x11	smproxy	\N	smproxy-1.0.6
2020-09-13 06:51:42+02	2020-09-13 06:51:57+02	x11	x11perf	\N	x11perf-1.6.1
2020-09-13 06:51:57+02	2020-09-13 06:52:11+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-09-13 06:52:12+02	2020-09-13 06:52:35+02	x11	xcalc	\N	xcalc-1.1.0
2020-09-13 06:52:51+02	2020-09-13 06:53:11+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-09-13 06:53:11+02	2020-09-13 06:53:25+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-09-13 06:53:26+02	2020-09-13 06:53:40+02	x11	xev	\N	xev-1.2.4
2020-09-13 06:53:40+02	2020-09-13 06:53:55+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-09-13 06:53:55+02	2020-09-13 06:54:09+02	x11	xgamma	\N	xgamma-1.0.6
2020-09-13 06:54:10+02	2020-09-13 06:54:30+02	x11	xgc	\N	xgc-1.0.5
2020-09-13 06:54:30+02	2020-09-13 06:54:46+02	x11	xhost	\N	xhost-1.0.8
2020-09-13 06:54:46+02	2020-09-13 06:55:01+02	x11	xinit	\N	xinit-1.4.1,1
2020-09-13 06:55:02+02	2020-09-13 06:55:18+02	x11	xinput	\N	xinput-1.6.3
2020-09-13 06:55:18+02	2020-09-13 06:55:32+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-09-13 06:55:33+02	2020-09-13 06:55:48+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-09-13 06:55:49+02	2020-09-13 06:56:05+02	x11	xkill	\N	xkill-1.0.5
2020-09-13 06:56:05+02	2020-09-13 06:56:18+02	x11	xlsatoms	\N	xlsatoms-1.1.3
2020-09-13 06:56:19+02	2020-09-13 06:56:33+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-09-13 06:56:34+02	2020-09-13 06:56:48+02	x11	xmessage	\N	xmessage-1.0.5
2020-09-13 06:56:48+02	2020-09-13 06:57:04+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-09-13 06:57:04+02	2020-09-13 06:57:22+02	x11	xpr	\N	xpr-1.0.5
2020-09-13 06:57:22+02	2020-09-13 06:57:42+02	x11	xrandr	\N	xrandr-1.5.1
2020-09-13 06:57:43+02	2020-09-13 06:57:55+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-09-13 06:57:58+02	2020-09-13 06:58:14+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-09-13 06:58:14+02	2020-09-13 06:58:27+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-09-13 06:58:28+02	2020-09-13 06:58:42+02	x11	xwd	\N	xwd-1.0.7
2020-09-13 06:58:43+02	2020-09-13 06:58:58+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-09-13 06:58:59+02	2020-09-13 06:59:13+02	x11	xwud	\N	xwud-1.0.5
2020-09-13 06:59:14+02	2020-09-13 06:59:30+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-09-13 06:59:30+02	2020-09-13 07:00:52+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-09-13 07:00:53+02	2020-09-13 07:01:56+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-09-13 07:01:57+02	2020-09-13 07:03:05+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-09-13 07:03:05+02	2020-09-13 07:04:15+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-09-13 07:04:15+02	2020-09-13 07:05:23+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-09-13 07:05:24+02	2020-09-13 07:07:08+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-09-13 07:07:09+02	2020-09-13 07:08:09+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-09-13 07:08:09+02	2020-09-13 07:09:10+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.4.0_3
2020-09-13 07:09:11+02	2020-09-13 07:09:33+02	x11-fonts	libFS	\N	libFS-1.0.8
2020-09-13 07:09:33+02	2020-09-13 07:10:08+02	x11-fonts	xorg-fonts-100dpi	\N	xorg-fonts-100dpi-7.7
2020-09-13 07:10:08+02	2020-09-13 07:10:44+02	x11-fonts	xorg-fonts-75dpi	\N	xorg-fonts-75dpi-7.7
2020-09-13 07:10:44+02	2020-09-13 07:11:01+02	x11-fonts	xorg-fonts-cyrillic	\N	xorg-fonts-cyrillic-7.7
2020-09-13 07:11:02+02	2020-09-13 07:11:46+02	x11-fonts	xorg-fonts-miscbitmaps	\N	xorg-fonts-miscbitmaps-7.7
2020-09-13 07:11:47+02	2020-09-13 07:12:07+02	x11-fonts	xorg-fonts-type1	\N	xorg-fonts-type1-7.7
2020-09-13 07:12:07+02	2020-09-13 07:13:08+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.73.0
2020-09-13 07:13:08+02	2020-09-13 07:14:27+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.73.0
2020-09-13 07:14:28+02	2020-09-13 07:14:59+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-09-13 07:14:59+02	2020-09-13 07:22:07+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-13 07:22:08+02	2020-09-13 07:22:33+02	x11-wm	twm	\N	twm-1.0.11
2020-09-13 07:22:34+02	2020-09-13 07:23:39+02	archivers	gcab	\N	gcab-1.3
2020-09-13 07:23:39+02	2020-09-13 07:24:51+02	audio	jack	\N	jackit-0.125.0_10
2020-09-13 07:24:51+02	2020-09-13 07:25:58+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-09-13 07:25:59+02	2020-09-13 07:48:28+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-13 07:48:29+02	2020-09-13 07:48:42+02	converters	py-webencodings	py27	py27-webencodings-0.5.1
2020-09-13 07:48:42+02	2020-09-13 07:51:43+02	databases	tdb	\N	tdb-1.4.3,1
2020-09-13 07:51:43+02	2020-09-13 07:52:24+02	devel	boehm-gc-threaded	\N	boehm-gc-threaded-8.0.4_1
2020-09-13 07:52:24+02	2020-09-13 07:54:22+02	devel	cppunit	\N	cppunit-1.14.0_8
2020-09-13 07:54:23+02	2020-09-13 07:55:21+02	devel	fpc-fcl-db	\N	fpc-fcl-db-3.0.4_6
2020-09-13 07:55:22+02	2020-09-13 07:55:38+02	devel	fpc-fcl-fpcunit	\N	fpc-fcl-fpcunit-3.0.4_4
2020-09-13 07:55:38+02	2020-09-13 07:55:54+02	devel	fpc-fcl-net	\N	fpc-fcl-net-3.0.4_4
2020-09-13 07:55:55+02	2020-09-13 07:57:02+02	devel	gflags	\N	gflags-2.2.2_2
2020-09-13 07:57:03+02	2020-09-13 07:57:12+02	devel	gnome-common	\N	gnome-common-3.18.0
2020-09-13 07:57:12+02	2020-09-13 07:58:37+02	devel	kf5-knotifications	\N	kf5-knotifications-5.73.0
2020-09-13 07:58:38+02	2020-09-13 07:59:28+02	devel	librest	\N	rest-0.7.93
2020-09-13 07:59:29+02	2020-09-13 08:00:14+02	devel	m17n-db	\N	m17n-db-1.8.0
2020-09-13 08:00:15+02	2020-09-13 08:00:38+02	devel	p5-Parse-Yapp	\N	p5-Parse-Yapp-1.21
2020-09-13 08:00:39+02	2020-09-13 08:00:58+02	devel	py-cffi	py27	py27-cffi-1.14.2
2020-09-13 08:00:59+02	2020-09-13 08:01:12+02	devel	py-iso8601	py37	py37-iso8601-0.1.12
2020-09-13 08:01:13+02	2020-09-13 08:01:31+02	devel	scons	py37	scons-py37-3.1.2
2020-09-13 08:01:32+02	2020-09-13 08:03:16+02	devel	tevent	\N	tevent-0.10.2
2020-09-13 08:03:17+02	2020-09-13 08:03:34+02	graphics	fpc-pasjpeg	\N	fpc-pasjpeg-3.0.4_4
2020-09-13 08:03:35+02	2020-09-13 08:09:27+02	graphics	leptonica	\N	leptonica-1.76.0_1
2020-09-13 08:09:29+02	2020-09-13 08:09:48+02	graphics	libnsgif	\N	libnsgif-0.2.1
2020-09-13 08:09:48+02	2020-09-13 08:13:25+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-09-13 08:13:25+02	2020-09-13 08:15:59+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-09-13 08:15:59+02	2020-09-13 08:16:13+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-09-13 02:38:17+02	2020-09-13 08:33:28+02	lang	rust	\N	rust-1.46.0
2020-09-13 08:16:13+02	2020-09-13 08:33:33+02	graphics	tesseract-data	\N	tesseract-data-4.0.0
2020-09-13 08:33:33+02	2020-09-13 08:35:54+02	net	libzmq4	\N	libzmq4-4.3.1_1
2020-09-13 08:35:57+02	2020-09-13 08:36:43+02	print	enscript-a4	\N	enscript-a4-1.6.6_3
2020-09-13 08:36:44+02	2020-09-13 08:36:56+02	print	libotf	\N	libotf-0.9.16
2020-09-13 08:33:30+02	2020-09-13 08:37:51+02	lang	python38	\N	python38-3.8.5
2020-09-13 08:36:56+02	2020-09-13 08:38:36+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-09-13 08:37:51+02	2020-09-13 08:38:40+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-13 08:38:37+02	2020-09-13 08:39:21+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-09-13 08:38:41+02	2020-09-13 08:39:39+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-09-13 08:39:22+02	2020-09-13 08:39:47+02	sysutils	cmocka	\N	cmocka-1.1.5
2020-09-13 08:39:40+02	2020-09-13 08:39:49+02	sysutils	libsunacl	\N	libsunacl-1.0.1
2020-09-13 08:39:50+02	2020-09-13 08:40:02+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-09-13 08:40:03+02	2020-09-13 08:40:22+02	textproc	tinyxml	\N	tinyxml-2.6.2_4
2020-09-13 08:40:22+02	2020-09-13 08:40:42+02	www	fpc-fastcgi	\N	fpc-fastcgi-3.0.4_4
2020-09-13 08:40:42+02	2020-09-13 08:40:59+02	www	fpc-httpd22	\N	fpc-httpd22-3.0.4_4
2020-09-13 08:40:59+02	2020-09-13 08:41:17+02	www	fpc-httpd24	\N	fpc-httpd24-3.0.4_4
2020-09-13 08:39:48+02	2020-09-13 08:41:52+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-09-13 08:41:52+02	2020-09-13 08:42:19+02	www	p5-HTTP-Message	\N	p5-HTTP-Message-6.25
2020-09-13 08:42:20+02	2020-09-13 08:43:16+02	x11	babl	\N	babl-0.1.78
2020-09-13 08:43:17+02	2020-09-13 08:43:34+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-09-13 08:41:17+02	2020-09-13 08:44:23+02	www	node	\N	node-14.10.0_1
2020-09-13 08:43:34+02	2020-09-13 08:44:42+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-13 08:44:24+02	2020-09-13 08:44:47+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-09-13 08:44:42+02	2020-09-13 08:44:54+02	x11	xorg-docs	\N	xorg-docs-1.7.1,1
2020-09-13 08:44:47+02	2020-09-13 08:44:58+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-09-13 08:44:54+02	2020-09-13 08:45:58+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-13 08:44:59+02	2020-09-13 08:48:01+02	x11-fm	caja	\N	caja-1.22.3_1
2020-09-13 08:45:59+02	2020-09-13 08:48:10+02	x11-fonts	xorg-fonts	\N	xorg-fonts-7.7_1
2020-09-13 08:48:10+02	2020-09-13 08:48:24+02	x11-themes	xcursor-themes	\N	xcursor-themes-1.0.6
2020-09-13 08:48:01+02	2020-09-13 08:49:33+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.8_4,1
2020-09-13 08:49:34+02	2020-09-13 08:51:04+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.73.0
2020-09-13 08:51:05+02	2020-09-13 08:52:09+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-13 08:52:09+02	2020-09-13 08:53:37+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-13 08:53:37+02	2020-09-13 08:53:55+02	archivers	unzip	\N	unzip-6.0_8
2020-09-13 08:53:55+02	2020-09-13 08:54:09+02	audio	gsm	\N	gsm-1.0.13_2
2020-09-13 08:54:10+02	2020-09-13 08:55:03+02	audio	lame	\N	lame-3.100_2
2020-09-13 08:55:04+02	2020-09-13 08:55:21+02	audio	libao	\N	libao-1.2.0_5
2020-09-13 08:55:21+02	2020-09-13 08:55:38+02	audio	libid3tag	\N	libid3tag-0.15.1b_2
2020-09-13 08:55:41+02	2020-09-13 08:56:38+02	audio	opusfile	\N	opusfile-0.12
2020-09-13 08:56:39+02	2020-09-13 08:57:58+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-13 08:48:24+02	2020-09-13 08:58:13+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-13 08:58:13+02	2020-09-13 08:58:30+02	devel	fpc-fcl-image	\N	fpc-fcl-image-3.0.4_4
2020-09-13 08:58:30+02	2020-09-13 08:59:10+02	devel	fpc-fcl-web	\N	fpc-fcl-web-3.0.4_7
2020-09-13 08:59:10+02	2020-09-13 09:00:34+02	devel	glog	\N	glog-0.4.0_1
2020-09-13 08:57:59+02	2020-09-13 09:03:32+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-09-13 09:00:34+02	2020-09-13 09:04:03+02	devel	glslang	\N	glslang-8.13.3743
2020-09-13 09:03:32+02	2020-09-13 09:04:38+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.73.0
2020-09-13 09:04:03+02	2020-09-13 09:04:59+02	devel	kf5-solid	\N	kf5-solid-5.73.0
2020-09-13 09:04:39+02	2020-09-13 09:05:04+02	devel	liblangtag	\N	liblangtag-0.6.2
2020-09-13 09:05:04+02	2020-09-13 09:05:11+02	devel	mdds	\N	mdds-1.5.0
2020-09-13 09:05:11+02	2020-09-13 09:05:24+02	devel	p5-IO-String	\N	p5-IO-String-1.08_1
2020-09-13 09:05:25+02	2020-09-13 09:05:34+02	devel	py-click	py37	py37-click-7.0
2020-09-13 09:05:00+02	2020-09-13 09:05:41+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-09-13 09:05:35+02	2020-09-13 09:05:44+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-09-13 09:05:42+02	2020-09-13 09:05:51+02	devel	py-olefile	py37	py37-olefile-0.46
2020-09-13 09:05:44+02	2020-09-13 09:05:54+02	devel	py-ply	py37	py37-ply-3.11
2020-09-13 09:05:51+02	2020-09-13 09:05:59+02	devel	py-ply	py27	py27-ply-3.11
2020-09-13 09:05:54+02	2020-09-13 09:06:04+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-09-13 09:05:59+02	2020-09-13 09:06:11+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-09-13 09:06:05+02	2020-09-13 09:06:14+02	devel	py-zipp	py37	py37-zipp-3.1.0
2020-09-13 09:06:14+02	2020-09-13 09:06:52+02	devel	tex-libtexlua	\N	tex-libtexlua-5.2.4
2020-09-13 09:06:53+02	2020-09-13 09:09:19+02	devel	tex-libtexluajit	\N	tex-libtexluajit-2.0.3
2020-09-13 09:09:20+02	2020-09-13 09:10:02+02	devel	yaml-cpp03	\N	yaml-cpp03-0.3.0_2
2020-09-13 09:10:03+02	2020-09-13 09:11:36+02	graphics	gegl	\N	gegl-0.4.24_5
2020-09-13 09:11:37+02	2020-09-13 09:13:05+02	graphics	libcdr01	\N	libcdr01-0.1.6
2020-09-13 09:13:06+02	2020-09-13 09:13:26+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-09-13 09:13:27+02	2020-09-13 09:14:04+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-09-13 09:14:05+02	2020-09-13 09:14:28+02	graphics	mypaint-brushes	\N	mypaint-brushes-1.3.1
2020-09-13 09:14:28+02	2020-09-13 09:14:53+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-09-13 09:14:54+02	2020-09-13 09:15:20+02	graphics	simage	\N	simage-1.7.0_7
2020-09-13 03:46:51+02	2020-09-13 09:19:57+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-09-13 09:15:20+02	2020-09-13 09:20:27+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-09-13 09:06:11+02	2020-09-13 09:20:28+02	devel	spirv-tools	\N	spirv-tools-2020.2
2020-09-13 09:20:28+02	2020-09-13 09:20:35+02	misc	compat12x	\N	compat12x-amd64-12.1.1201000.20200220
2020-09-13 09:20:35+02	2020-09-13 09:20:52+02	misc	fpc-chm	\N	fpc-chm-3.0.4_4
2020-09-13 09:20:53+02	2020-09-13 09:21:00+02	misc	usbids	\N	usbids-20200826
2020-09-13 09:20:28+02	2020-09-13 09:21:09+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0,1
2020-09-13 09:21:04+02	2020-09-13 09:21:14+02	multimedia	librtmp	\N	librtmp-2.4.20190330
2020-09-13 09:21:14+02	2020-09-13 09:21:28+02	net	liboauth	\N	liboauth-1.0.3_4
2020-09-13 09:19:58+02	2020-09-13 09:22:13+02	mail	mailutils	\N	mailutils-3.10
2020-09-13 09:22:13+02	2020-09-13 09:22:25+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-09-13 09:21:10+02	2020-09-13 09:22:47+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-13 09:22:25+02	2020-09-13 09:23:17+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-09-13 09:23:18+02	2020-09-13 09:24:51+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-13 09:22:47+02	2020-09-13 09:25:42+02	print	qpdf	\N	qpdf-10.0.1
2020-09-13 09:24:52+02	2020-09-13 09:25:45+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-09-13 09:25:43+02	2020-09-13 09:26:19+02	sysutils	flock	\N	flock-2.30.1
2020-09-13 09:25:45+02	2020-09-13 09:26:25+02	sysutils	fusefs-libs	\N	fusefs-libs-2.9.9_1
2020-09-13 09:26:25+02	2020-09-13 09:27:12+02	sysutils	libcdio	\N	libcdio-2.1.0
2020-09-13 09:26:19+02	2020-09-13 09:27:52+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.73.0
2020-09-13 09:27:12+02	2020-09-13 09:28:16+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-13 09:27:53+02	2020-09-13 09:28:38+02	textproc	libuninameslist	\N	libuninameslist-20190701
2020-09-13 09:28:39+02	2020-09-13 09:28:51+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-09-13 09:28:51+02	2020-09-13 09:29:01+02	textproc	utf8proc	\N	utf8proc-2.5.0
2020-09-13 09:29:02+02	2020-09-13 09:29:24+02	textproc	yelp-tools	\N	yelp-tools-3.28.0
2020-09-13 09:29:25+02	2020-09-13 09:29:39+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-09-13 09:29:40+02	2020-09-13 09:29:52+02	www	py-html5lib	py27	py27-html5lib-1.0.1
2020-09-13 09:29:53+02	2020-09-13 09:30:07+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-09-13 09:30:08+02	2020-09-13 09:30:27+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-09-13 09:28:17+02	2020-09-13 09:30:42+02	textproc	libvisio01	\N	libvisio01-0.1.7_2
2020-09-13 09:30:27+02	2020-09-13 09:31:13+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-09-13 09:30:42+02	2020-09-13 09:31:40+02	x11	kf5-kded	\N	kf5-kded-5.73.0
2020-09-13 09:31:41+02	2020-09-13 09:32:54+02	x11	zenity	\N	zenity-3.32.0
2020-09-13 09:32:54+02	2020-09-13 09:33:02+02	x11-fonts	gnu-unifont	\N	gnu-unifont-13.0.03,1
2020-09-13 09:33:02+02	2020-09-13 09:33:54+02	x11-fonts	noto-basic	\N	noto-basic-2.0_1
2020-09-13 09:31:13+02	2020-09-13 09:34:15+02	x11	xorg	\N	xorg-7.7_3
2020-09-13 09:33:55+02	2020-09-13 09:34:50+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.73.0
2020-09-13 09:34:16+02	2020-09-13 09:35:26+02	archivers	gtar	\N	gtar-1.32
2020-09-13 09:34:51+02	2020-09-13 09:35:32+02	audio	libmodplug	\N	libmodplug-0.8.9.0
2020-09-13 09:35:33+02	2020-09-13 09:35:45+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-09-13 09:35:46+02	2020-09-13 09:35:58+02	databases	py-redis	py37	py37-redis-3.5.3
2020-09-13 09:35:59+02	2020-09-13 09:36:50+02	devel	apache-ant	\N	apache-ant-1.10.8
2020-09-13 09:35:27+02	2020-09-13 09:37:01+02	audio	sox	\N	sox-14.4.2_4
2020-09-13 09:37:02+02	2020-09-13 09:37:15+02	devel	fpc-regexpr	\N	fpc-regexpr-3.0.4_4
2020-09-13 09:36:50+02	2020-09-13 09:37:20+02	devel	fpc-fppkg	\N	fpc-fppkg-3.0.4_6
2020-09-13 09:37:20+02	2020-09-13 09:39:03+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-13 09:39:04+02	2020-09-13 09:40:54+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-13 09:40:56+02	2020-09-13 09:41:21+02	devel	physfs	\N	physfs-3.0.2
2020-09-13 09:41:22+02	2020-09-13 09:41:36+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-09-13 09:21:28+02	2020-09-13 09:41:42+02	net	samba410	\N	samba410-4.10.17
2020-09-13 09:41:37+02	2020-09-13 09:41:49+02	devel	py-appdirs	py37	py37-appdirs-1.4.4
2020-09-13 09:41:42+02	2020-09-13 09:41:52+02	devel	py-asn1crypto	py27	py27-asn1crypto-1.3.0
2020-09-13 09:41:49+02	2020-09-13 09:41:59+02	devel	py-backports	py37	py37-backports-1
2020-09-13 09:41:53+02	2020-09-13 09:42:04+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-09-13 09:42:00+02	2020-09-13 09:42:09+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-09-13 09:37:15+02	2020-09-13 09:42:10+02	devel	kf5-kio	\N	kf5-kio-5.73.0
2020-09-13 09:42:04+02	2020-09-13 09:42:12+02	devel	py-enum34	py27	py27-enum34-1.1.6
2020-09-13 09:42:11+02	2020-09-13 09:42:20+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-1.7.0
2020-09-13 09:42:13+02	2020-09-13 09:42:21+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-09-13 09:42:09+02	2020-09-13 09:42:29+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-09-13 09:42:21+02	2020-09-13 09:42:30+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-09-13 09:42:22+02	2020-09-13 09:42:32+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-09-13 09:42:30+02	2020-09-13 09:42:38+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-09-13 09:42:31+02	2020-09-13 09:42:39+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-09-13 09:42:32+02	2020-09-13 09:42:40+02	devel	py-wheel	py27	py27-wheel-0.30.0_1
2020-09-13 09:42:38+02	2020-09-13 09:42:49+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-09-13 09:42:40+02	2020-09-13 09:43:43+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-09-13 09:42:49+02	2020-09-13 09:43:46+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-09-13 09:42:40+02	2020-09-13 09:43:47+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-09-13 09:43:47+02	2020-09-13 09:43:56+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-09-13 09:43:48+02	2020-09-13 09:43:59+02	devel	xxhash	\N	xxhash-0.8.0
2020-09-13 09:43:43+02	2020-09-13 09:46:13+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-09-13 09:46:13+02	2020-09-13 09:46:29+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-09-13 09:43:57+02	2020-09-13 09:47:18+02	dns	bind-tools	\N	bind-tools-9.16.6
2020-09-13 09:47:19+02	2020-09-13 09:47:35+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-09-13 09:43:59+02	2020-09-13 09:48:23+02	editors	emacs	full	emacs-27.1,3
2020-09-13 09:48:23+02	2020-09-13 09:49:21+02	graphics	glfw	\N	glfw-3.3.2
2020-09-13 09:49:21+02	2020-09-13 09:50:15+02	graphics	libfreehand	\N	libfreehand-0.1.2_18
2020-09-13 09:50:16+02	2020-09-13 09:51:33+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-09-13 09:51:34+02	2020-09-13 09:55:07+02	graphics	movit	\N	movit-1.6.2_2
2020-09-13 09:46:29+02	2020-09-13 09:56:20+02	graphics	Coin	\N	Coin-4.0.0_1
2020-09-13 09:55:07+02	2020-09-13 09:58:49+02	graphics	opencolorio	\N	opencolorio-1.1.1_1
2020-09-13 09:58:50+02	2020-09-13 09:58:59+02	graphics	potrace	\N	potrace-1.16
2020-09-13 09:58:59+02	2020-09-13 09:59:33+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-09-13 09:59:34+02	2020-09-13 10:00:15+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-09-13 10:00:16+02	2020-09-13 10:01:15+02	graphics	shaderc	\N	shaderc-2020.0
2020-09-13 10:01:16+02	2020-09-13 10:01:43+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-09-13 09:47:35+02	2020-09-13 10:03:59+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-13 10:01:44+02	2020-09-13 10:06:27+02	java	bootstrap-openjdk11	\N	bootstrap-openjdk11-11.0.5.10.1
2020-09-13 10:04:02+02	2020-09-13 10:08:49+02	lang	cython	py27	py27-cython-0.29.15
2020-09-13 09:56:21+02	2020-09-13 10:13:27+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-13 10:13:28+02	2020-09-13 10:14:52+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-13 10:14:52+02	2020-09-13 10:20:33+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-09-13 10:20:34+02	2020-09-13 10:22:26+02	multimedia	gavl	\N	gavl-1.4.0_4
2020-09-13 10:22:26+02	2020-09-13 10:26:19+02	multimedia	libde265	\N	libde265-1.0.2_5
2020-09-13 10:26:20+02	2020-09-13 10:26:32+02	multimedia	rtmpdump	\N	rtmpdump-2.4.20190330
2020-09-13 10:26:33+02	2020-09-13 10:26:53+02	multimedia	vid.stab	\N	vid.stab-0.98.2
2020-09-13 10:26:54+02	2020-09-13 10:27:27+02	net	gssdp	\N	gssdp-1.2.3
2020-09-13 10:27:27+02	2020-09-13 10:28:26+02	net	libvncserver	\N	libvncserver-0.9.12_2
2020-09-13 10:28:27+02	2020-09-13 10:28:42+02	net	py-ipaddress	py27	py27-ipaddress-1.0.23
2020-09-13 10:28:43+02	2020-09-13 10:29:51+02	net-mgmt	icingaweb2-module-ipl	php73	icingaweb2-module-ipl-php73-0.5.0
2020-09-13 10:29:52+02	2020-09-13 10:30:24+02	print	adobe-cmaps	\N	adobe-cmaps-20051217_4
2020-09-13 10:30:24+02	2020-09-13 10:31:51+02	print	cups-filters	\N	cups-filters-1.28.1_1
2020-09-13 10:31:52+02	2020-09-13 10:38:14+02	print	fontforge	\N	fontforge-20190801_2
2020-09-13 10:38:15+02	2020-09-13 10:38:55+02	print	libmspub01	\N	libmspub01-0.1.4_16
2020-09-13 10:38:55+02	2020-09-13 10:39:26+02	print	libpagemaker	\N	libpagemaker-0.0.4_10
2020-09-13 10:39:27+02	2020-09-13 10:39:37+02	print	t1utils	\N	t1utils-1.32
2020-09-13 10:06:28+02	2020-09-13 10:40:05+02	lang	gcc6-aux	\N	gcc6-aux-20180516_1,1
2020-09-13 10:39:38+02	2020-09-13 10:52:58+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-13 10:52:59+02	2020-09-13 10:53:35+02	security	mbedtls	\N	mbedtls-2.16.8
2020-09-13 10:53:36+02	2020-09-13 10:53:49+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-09-13 10:53:50+02	2020-09-13 10:54:57+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-13 10:54:57+02	2020-09-13 10:55:37+02	sysutils	libcdio-paranoia	\N	libcdio-paranoia-10.2+2.0.1
2020-09-13 10:55:37+02	2020-09-13 10:55:55+02	sysutils	lsof	\N	lsof-4.93.2_12,8
2020-09-13 10:55:55+02	2020-09-13 10:58:45+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-13 10:58:46+02	2020-09-13 10:59:04+02	sysutils	py-psutil	py37	py37-psutil-5.7.2
2020-09-13 10:59:05+02	2020-09-13 11:00:31+02	textproc	UCD	\N	UCD-13.0.0
2020-09-13 11:00:31+02	2020-09-13 11:01:03+02	textproc	libebml	\N	libebml-1.4.0
2020-09-13 11:01:06+02	2020-09-13 11:01:31+02	textproc	mythes	\N	mythes-1.2.4_7
2020-09-13 11:01:33+02	2020-09-13 11:01:52+02	textproc	p5-PDF-API2-XS	\N	p5-PDF-API2-XS-1.001
2020-09-13 11:01:52+02	2020-09-13 11:02:07+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-09-13 10:40:07+02	2020-09-13 11:02:17+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-09-13 11:02:08+02	2020-09-13 11:02:21+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-09-13 11:02:20+02	2020-09-13 11:02:36+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-09-13 11:02:22+02	2020-09-13 11:03:35+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-09-13 11:02:36+02	2020-09-13 11:07:24+02	www	apache24	\N	apache24-2.4.46
2020-09-13 11:07:25+02	2020-09-13 12:52:30+02	www	node12	\N	node12-12.18.3
2020-09-13 12:52:32+02	2020-09-13 12:52:53+02	www	py-h2	py37	py37-h2-3.2.0
2020-09-13 12:52:53+02	2020-09-13 12:53:07+02	www	py-hyperlink	py37	py37-hyperlink-20.0.1
2020-09-13 12:53:08+02	2020-09-13 12:53:26+02	www	py-priority	py37	py37-priority-1.3.0
2020-09-13 12:53:26+02	2020-09-13 12:53:58+02	www	serf	\N	serf-1.3.9_5
2020-09-13 10:08:49+02	2020-09-13 12:55:11+02	lang	ghc	\N	ghc-8.10.2
2020-09-13 12:53:58+02	2020-09-13 12:55:57+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-13 12:55:58+02	2020-09-13 12:56:50+02	x11-fonts	cantarell-fonts	\N	cantarell-fonts-0.101_1
2020-09-13 12:55:11+02	2020-09-13 12:57:05+02	x11	yelp	\N	yelp-3.28.1_2
2020-09-13 12:56:50+02	2020-09-13 12:57:10+02	x11-fonts	p5-Font-TTF	\N	p5-Font-TTF-1.06
2020-09-13 12:57:11+02	2020-09-13 12:58:16+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-13 12:57:06+02	2020-09-13 12:58:34+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-13 12:58:17+02	2020-09-13 13:00:08+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-13 13:00:09+02	2020-09-13 13:04:33+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-13 12:58:35+02	2020-09-13 13:07:00+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-09-13 13:07:02+02	2020-09-13 13:08:19+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-09-13 13:04:33+02	2020-09-13 13:08:19+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-13 13:08:20+02	2020-09-13 13:11:43+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-09-13 13:08:20+02	2020-09-13 13:15:58+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-09-13 13:16:01+02	2020-09-13 13:17:31+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-13 13:17:31+02	2020-09-13 13:18:33+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-09-13 13:11:44+02	2020-09-13 13:21:19+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-13 13:21:19+02	2020-09-13 13:21:40+02	archivers	c-blosc	\N	c-blosc-1.15.0_2
2020-09-13 13:18:33+02	2020-09-13 13:21:41+02	x11-wm	marco	\N	marco-1.22.4
2020-09-13 13:21:42+02	2020-09-13 13:22:00+02	archivers	p5-Archive-Zip	\N	p5-Archive-Zip-1.68
2020-09-13 13:21:40+02	2020-09-13 13:22:42+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-13 13:22:43+02	2020-09-13 13:23:08+02	audio	freealut	\N	freealut-1.1.0_4
2020-09-13 13:22:01+02	2020-09-13 13:23:13+02	audio	faad	\N	faad2-2.8.8,1
2020-09-13 13:23:08+02	2020-09-13 13:23:20+02	audio	libdssialsacompat	\N	libdssialsacompat-1.0.8a_2
2020-09-13 13:23:14+02	2020-09-13 13:23:52+02	audio	liblo	\N	liblo-0.31_1
2020-09-13 13:23:53+02	2020-09-13 13:24:27+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-09-13 13:23:20+02	2020-09-13 13:25:06+02	audio	libopenshot-audio	\N	libopenshot-audio-0.2.0.10
2020-09-13 13:25:07+02	2020-09-13 13:25:29+02	audio	twolame	\N	twolame-0.4.0
2020-09-13 13:25:30+02	2020-09-13 13:26:06+02	databases	p5-DBI	\N	p5-DBI-1.643
2020-09-13 13:24:27+02	2020-09-13 13:26:20+02	audio	taglib	\N	taglib-1.12.b.1_1
2020-09-13 13:26:06+02	2020-09-13 13:27:40+02	databases	unixODBC	\N	unixODBC-2.3.7
2020-09-13 13:26:21+02	2020-09-13 13:28:42+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-13 13:27:41+02	2020-09-13 13:28:56+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-13 13:28:57+02	2020-09-13 13:31:06+02	devel	binutils	aarch64	aarch64-binutils-2.33.1_3,1
2020-09-13 13:28:43+02	2020-09-13 13:32:46+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-13 13:31:06+02	2020-09-13 13:33:03+02	devel	binutils	mips64	mips64-binutils-2.33.1_3,1
2020-09-13 13:32:47+02	2020-09-13 13:34:19+02	devel	binutils	powerpc64	powerpc64-binutils-2.33.1_3,1
2020-09-13 13:33:04+02	2020-09-13 13:34:33+02	devel	binutils	riscv64	riscv64-binutils-2.33.1_3,1
2020-09-13 13:34:34+02	2020-09-13 13:34:41+02	devel	cl-asdf	\N	cl-asdf-3.3.1.7,2
2020-09-13 13:34:42+02	2020-09-13 13:35:06+02	devel	cscope	\N	cscope-15.9
2020-09-13 13:34:20+02	2020-09-13 13:35:37+02	devel	binutils	sparc64	sparc64-binutils-2.33.1_3,1
2020-09-13 13:35:06+02	2020-09-13 13:35:40+02	devel	dmake	\N	dmake-4.12.3,1
2020-09-13 13:35:38+02	2020-09-13 13:35:55+02	devel	fpc-fcl-registry	\N	fpc-fcl-registry-3.0.4_4
2020-09-13 13:35:41+02	2020-09-13 13:38:15+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-13 13:38:18+02	2020-09-13 13:42:05+02	devel	kBuild	\N	kBuild-0.1.9998_13
2020-09-13 13:42:06+02	2020-09-13 13:42:35+02	devel	libfmt	\N	libfmt-7.0.3
2020-09-13 13:35:55+02	2020-09-13 13:48:52+02	devel	hs-cabal-install	\N	hs-cabal-install-3.2.0.0
2020-09-13 13:48:52+02	2020-09-13 13:49:14+02	devel	libsigsegv	\N	libsigsegv-2.12
2020-09-13 13:42:36+02	2020-09-13 13:49:59+02	devel	liborcus	\N	liborcus-0.15.4
2020-09-13 13:49:14+02	2020-09-13 13:52:26+02	devel	libslang2	\N	libslang2-2.3.2_2
2020-09-13 13:52:26+02	2020-09-13 13:53:05+02	devel	ocl-icd	\N	ocl-icd-2.2.12
2020-09-13 13:53:06+02	2020-09-13 13:53:25+02	devel	p5-Getopt-ArgvFile	\N	p5-Getopt-ArgvFile-1.11_1
2020-09-13 13:53:28+02	2020-09-13 13:53:40+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-09-13 13:53:40+02	2020-09-13 13:53:56+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-09-13 13:53:56+02	2020-09-13 13:54:10+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-09-13 13:54:10+02	2020-09-13 13:54:23+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-09-13 13:54:23+02	2020-09-13 13:54:36+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-09-13 13:54:37+02	2020-09-13 13:54:50+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-09-13 13:54:51+02	2020-09-13 13:55:02+02	devel	py-pyasn1	py27	py27-pyasn1-0.4.7
2020-09-13 13:55:02+02	2020-09-13 13:56:05+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-09-13 13:56:06+02	2020-09-13 13:56:30+02	devel	pydbus-common	\N	pydbus-common-1.2.16
2020-09-13 13:50:00+02	2020-09-13 13:56:46+02	devel	ncurses	\N	ncurses-6.2.20200907
2020-09-13 13:56:46+02	2020-09-13 13:57:08+02	devel	rebar	\N	rebar-2.6.4,1
2020-09-13 13:57:08+02	2020-09-13 13:58:16+02	devel	rebar3	\N	rebar3-3.14.1,1
2020-09-13 13:58:17+02	2020-09-13 13:58:23+02	devel	robin-map	\N	robin-map-0.6.3
2020-09-13 13:58:24+02	2020-09-13 13:58:36+02	devel	ucpp	\N	ucpp-1.3.2,1
2020-09-13 13:58:37+02	2020-09-13 13:59:45+02	devel	unittest-cpp	\N	unittest-cpp-2.0.0
2020-09-13 13:59:45+02	2020-09-13 14:03:06+02	dns	ldns	\N	ldns-1.7.1_1
2020-09-13 14:03:07+02	2020-09-13 14:11:18+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-13 14:11:18+02	2020-09-13 14:46:30+02	graphics	dcmtk	\N	dcmtk-3.6.5_2
2020-09-13 14:46:31+02	2020-09-13 14:48:13+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-09-13 14:48:13+02	2020-09-13 14:48:37+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-09-13 14:48:37+02	2020-09-13 14:48:55+02	graphics	frei0r-plugins-gavl	\N	frei0r-plugins-gavl-1.6.1
2020-09-13 14:48:55+02	2020-09-13 14:51:22+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-13 14:51:22+02	2020-09-13 15:05:35+02	graphics	libetonyek01	\N	libetonyek01-0.1.9_6,1
2020-09-13 15:05:36+02	2020-09-13 15:07:22+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-09-13 15:07:23+02	2020-09-13 15:08:06+02	graphics	libgxps	\N	libgxps-0.3.1
2020-09-13 15:08:07+02	2020-09-13 15:10:47+02	graphics	libheif	\N	libheif-1.6.2
2020-09-13 15:10:48+02	2020-09-13 15:11:32+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-09-13 13:56:31+02	2020-09-13 15:12:32+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-13 15:11:35+02	2020-09-13 15:12:33+02	graphics	libzmf	\N	libzmf-0.0.2_21
2020-09-13 15:12:33+02	2020-09-13 15:17:44+02	graphics	podofo	\N	podofo-0.9.6_1
2020-09-13 15:17:45+02	2020-09-13 15:18:25+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-09-13 15:12:33+02	2020-09-13 15:19:20+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-09-13 15:18:25+02	2020-09-13 15:19:28+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-13 15:19:21+02	2020-09-13 15:19:40+02	graphics	qr-code-generator	\N	qr-code-generator-1.6.0
2020-09-13 15:19:28+02	2020-09-13 15:20:32+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-09-13 15:20:32+02	2020-09-13 15:20:59+02	graphics	squish	\N	squish-1.10_2
2020-09-13 15:21:00+02	2020-09-13 15:22:30+02	graphics	vigra	\N	vigra-1.11.1_12
2020-09-13 15:19:41+02	2020-09-13 15:24:05+02	graphics	sane-backends	\N	sane-backends-1.0.31_1
2020-09-13 15:24:06+02	2020-09-13 15:24:20+02	lang	fpc-rtl-console	\N	fpc-rtl-console-3.0.4_4
2020-09-13 15:22:31+02	2020-09-13 15:25:16+02	lang	cython	py38	py38-cython-0.29.15
2020-09-13 15:25:16+02	2020-09-13 15:26:26+02	lang	fpc-utils	\N	fpc-utils-3.0.4_6
2020-09-13 15:24:21+02	2020-09-13 15:27:02+02	lang	fpc-source	\N	fpc-source-3.0.4_2
2020-09-13 15:27:02+02	2020-09-13 15:27:23+02	lang	p5-Try-Tiny	\N	p5-Try-Tiny-0.30
2020-09-13 15:26:27+02	2020-09-13 15:30:10+02	lang	guile1	\N	guile1-1.8.8
2020-09-13 15:30:11+02	2020-09-13 15:32:46+02	math	cryptominisat	\N	cryptominisat-5.7.1
2020-09-13 15:32:47+02	2020-09-13 15:33:05+02	math	fpc-numlib	\N	fpc-numlib-3.0.4_4
2020-09-13 15:33:06+02	2020-09-13 15:38:10+02	math	gsl	\N	gsl-2.6
2020-09-13 15:38:11+02	2020-09-13 15:41:08+02	math	lp_solve	\N	lp_solve-5.5.2.5
2020-09-13 15:41:08+02	2020-09-13 15:41:23+02	math	minisat	\N	minisat-2.2.1_2
2020-09-13 15:41:23+02	2020-09-13 15:41:45+02	math	openlibm	\N	openlibm-0.6.0_1
2020-09-13 15:41:45+02	2020-09-13 15:42:15+02	misc	box2d	\N	Box2D-2.3.1
2020-09-13 15:27:24+02	2020-09-13 15:50:15+02	math	R	\N	R-4.0.2
2020-09-13 15:42:16+02	2020-09-13 15:50:29+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-13 15:50:16+02	2020-09-13 15:51:23+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-09-13 15:50:30+02	2020-09-13 15:51:32+02	multimedia	libass	\N	libass-0.14.0
2020-09-13 15:51:23+02	2020-09-13 15:51:50+02	multimedia	libdvbpsi	\N	libdvbpsi-1.3.3
2020-09-13 15:51:33+02	2020-09-13 15:52:08+02	multimedia	libmatroska	\N	libmatroska-1.6.2
2020-09-13 15:51:51+02	2020-09-13 15:52:32+02	multimedia	libmpeg2	\N	libmpeg2-0.5.1_6
2020-09-13 15:52:33+02	2020-09-13 15:52:47+02	net	cppzmq	\N	cppzmq-4.6.0
2020-09-13 15:52:48+02	2020-09-13 15:53:12+02	net	fping	\N	fping-5.0_1
2020-09-13 15:52:09+02	2020-09-13 15:54:40+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-13 15:54:41+02	2020-09-13 15:55:21+02	net	gupnp	\N	gupnp-1.1.1_1
2020-09-13 15:53:13+02	2020-09-13 15:56:32+02	net	freerdp	\N	freerdp-2.2.0
2020-09-13 15:56:33+02	2020-09-13 15:56:49+02	net	libmaxminddb	\N	libmaxminddb-1.4.3
2020-09-13 15:56:49+02	2020-09-13 15:58:16+02	net	liveMedia	\N	liveMedia-2020.08.19,2
2020-09-13 15:58:17+02	2020-09-13 15:58:40+02	net	p5-Net-HTTP	\N	p5-Net-HTTP-6.19
2020-09-13 15:58:41+02	2020-09-13 15:59:03+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-09-13 15:59:03+02	2020-09-13 15:59:17+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-09-13 15:59:21+02	2020-09-13 16:00:25+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-13 16:00:25+02	2020-09-13 16:01:21+02	net	rsync	\N	rsync-3.2.3
2020-09-13 16:01:21+02	2020-09-13 16:02:30+02	net-mgmt	icingaweb2-module-reactbundle	php73	icingaweb2-module-reactbundle-php73-0.7.0_1
2020-09-13 16:02:31+02	2020-09-13 16:03:04+02	print	freetype	\N	freetype-1.3.1_5
2020-09-13 15:55:22+02	2020-09-13 16:08:29+02	net	libcmis	\N	libcmis-0.5.2_1
2020-09-13 16:03:05+02	2020-09-13 16:08:35+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-13 16:08:30+02	2020-09-13 16:09:04+02	print	libspectre	\N	libspectre-0.2.9
2020-09-13 16:08:36+02	2020-09-13 16:11:26+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-13 16:09:04+02	2020-09-13 16:16:55+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-13 16:16:56+02	2020-09-13 16:17:51+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-09-13 16:17:52+02	2020-09-13 16:18:22+02	security	mhash	\N	mhash-0.9.9.9_5
2020-09-13 16:11:27+02	2020-09-13 16:18:23+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-13 16:18:24+02	2020-09-13 16:18:34+02	security	pam_helper	\N	pam_helper-1.1
2020-09-13 16:18:22+02	2020-09-13 16:18:40+02	security	p5-Authen-NTLM	\N	p5-Authen-NTLM-1.09_1
2020-09-13 16:18:34+02	2020-09-13 16:18:48+02	security	py-bcrypt	py27	py27-bcrypt-3.1.7_1
2020-09-13 16:18:41+02	2020-09-13 16:19:32+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-09-13 16:18:49+02	2020-09-13 16:19:34+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-09-13 16:19:34+02	2020-09-13 16:19:52+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-09-13 16:19:35+02	2020-09-13 16:19:52+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-09-13 16:19:56+02	2020-09-13 16:21:12+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-09-13 16:19:53+02	2020-09-13 16:21:21+02	security	sudo	\N	sudo-1.9.2
2020-09-13 16:21:13+02	2020-09-13 16:21:32+02	sysutils	hal-info	\N	hal-info-20091130
2020-09-13 16:21:32+02	2020-09-13 16:22:31+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-13 16:22:32+02	2020-09-13 16:22:53+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.04_1
2020-09-13 16:22:53+02	2020-09-13 16:23:33+02	sysutils	patchelf	\N	patchelf-0.11
2020-09-13 16:23:35+02	2020-09-13 16:23:43+02	sysutils	sysctlbyname-improved-kmod	\N	sysctlbyname-improved-kmod-20191124
2020-09-13 16:23:44+02	2020-09-13 16:23:52+02	sysutils	sysctlinfo-kmod	\N	sysctlinfo-kmod-20191005
2020-09-13 16:21:22+02	2020-09-13 16:25:25+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-09-13 16:25:26+02	2020-09-13 16:25:35+02	sysutils	vpnc-scripts	\N	vpnc-scripts-20190611
2020-09-13 16:25:35+02	2020-09-13 16:28:42+02	textproc	clucene	\N	clucene-2.3.3.4_19
2020-09-13 16:23:52+02	2020-09-13 16:28:44+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-09-13 16:28:44+02	2020-09-13 16:29:24+02	textproc	enchant	\N	enchant-1.6.0_9
2020-09-13 16:29:25+02	2020-09-13 16:30:51+02	textproc	libabw	\N	libabw-0.1.3
2020-09-13 16:28:43+02	2020-09-13 16:31:00+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-13 16:30:52+02	2020-09-13 16:31:02+02	textproc	libdiff	\N	libdiff-0.1.0
2020-09-13 16:31:02+02	2020-09-13 16:32:18+02	textproc	libepubgen	\N	libepubgen-0.1.0_9
2020-09-13 16:32:19+02	2020-09-13 16:32:30+02	textproc	libexttextcat	\N	libexttextcat-3.4.5
2020-09-13 16:31:02+02	2020-09-13 16:36:43+02	textproc	libe-book	\N	libe-book-0.1.3_18
2020-09-13 16:36:45+02	2020-09-13 16:37:47+02	textproc	libnumbertext	\N	libnumbertext-1.0.6
2020-09-13 16:37:47+02	2020-09-13 16:39:39+02	textproc	libodfgen01	\N	libodfgen01-0.1.7_3
2020-09-13 16:39:42+02	2020-09-13 16:41:25+02	textproc	libqxp	\N	libqxp-0.0.0_16
2020-09-13 16:32:30+02	2020-09-13 16:44:03+02	textproc	libmwaw03	\N	libmwaw03-0.3.15
2020-09-13 16:41:25+02	2020-09-13 16:45:35+02	textproc	libstaroffice	\N	libstaroffice-0.0.7
2020-09-13 16:45:36+02	2020-09-13 16:46:23+02	textproc	p5-PDF-API2	\N	p5-PDF-API2-2.038
2020-09-13 16:46:24+02	2020-09-13 16:46:45+02	textproc	p5-Pod-Escapes	\N	p5-Pod-Escapes-1.07
2020-09-13 16:46:45+02	2020-09-13 16:47:22+02	textproc	pugixml	\N	pugixml-1.10
2020-09-13 16:44:04+02	2020-09-13 16:47:37+02	textproc	libwps	\N	libwps-0.4.11
2020-09-13 16:47:22+02	2020-09-13 16:47:38+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-09-13 16:47:38+02	2020-09-13 16:47:54+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-09-13 16:47:38+02	2020-09-13 16:47:55+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-09-13 16:47:56+02	2020-09-13 16:48:13+02	www	p5-HTTP-Cookies	\N	p5-HTTP-Cookies-6.08
2020-09-13 16:48:13+02	2020-09-13 16:48:32+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.12
2020-09-13 16:47:55+02	2020-09-13 16:48:51+02	textproc	redland	\N	redland-1.0.17_4
2020-09-13 16:48:32+02	2020-09-13 16:48:55+02	www	p5-HTTP-Negotiate	\N	p5-HTTP-Negotiate-6.01_1
2020-09-13 16:48:52+02	2020-09-13 16:49:11+02	www	p5-WWW-RobotRules	\N	p5-WWW-RobotRules-6.02_1
2020-09-13 16:48:56+02	2020-09-13 16:49:23+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-09-13 16:49:11+02	2020-09-13 16:50:05+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-13 16:50:07+02	2020-09-13 16:50:22+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-09-13 16:49:23+02	2020-09-13 16:51:05+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-13 16:50:23+02	2020-09-13 16:51:15+02	www	youtube_dl	\N	youtube_dl-2020.09.06
2020-09-13 16:51:15+02	2020-09-13 16:51:25+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-09-13 16:51:25+02	2020-09-13 16:51:35+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-09-13 16:51:35+02	2020-09-13 16:51:53+02	x11-fonts	droid-fonts-ttf	\N	droid-fonts-ttf-20131024_5
2020-09-13 16:51:54+02	2020-09-13 16:52:02+02	x11-fonts	gentium-basic	\N	GentiumBasic-1102_1
2020-09-13 16:52:03+02	2020-09-13 16:52:15+02	x11-fonts	liberation-fonts-ttf	\N	liberation-fonts-ttf-2.1.1,2
2020-09-13 16:52:16+02	2020-09-13 16:52:37+02	x11-fonts	linuxlibertine-g	\N	linuxlibertine-g-20120116_2
2020-09-13 16:51:06+02	2020-09-13 16:52:39+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-13 16:52:38+02	2020-09-13 16:52:56+02	x11-fonts	twemoji-color-font-ttf	\N	twemoji-color-font-ttf-12.0.1
2020-09-13 16:52:57+02	2020-09-13 16:53:45+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-09-13 16:52:40+02	2020-09-13 16:54:56+02	x11-servers	xephyr	\N	xephyr-1.20.8_4,1
2020-09-13 16:53:45+02	2020-09-13 16:55:04+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-09-13 16:54:57+02	2020-09-13 16:55:34+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-09-13 16:55:08+02	2020-09-13 16:56:02+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-13 16:55:34+02	2020-09-13 16:58:19+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-13 16:56:03+02	2020-09-13 16:58:24+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-09-13 16:58:20+02	2020-09-13 17:01:14+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-13 17:01:14+02	2020-09-13 17:01:25+02	archivers	arc	\N	arc-5.21p
2020-09-13 17:01:26+02	2020-09-13 17:02:27+02	archivers	arj	\N	arj-3.10.22_8
2020-09-13 17:02:28+02	2020-09-13 17:06:05+02	archivers	engrampa	\N	engrampa-1.24.0
2020-09-13 17:06:06+02	2020-09-13 17:06:41+02	archivers	libunrar5	\N	libunrar5-5.9.4
2020-09-13 17:06:42+02	2020-09-13 17:08:07+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-09-13 17:08:07+02	2020-09-13 17:08:17+02	archivers	unzoo	\N	unzoo-4.4_2
2020-09-13 17:08:18+02	2020-09-13 17:08:45+02	audio	dssi	\N	dssi-1.1.1_8
2020-09-13 17:08:46+02	2020-09-13 17:09:24+02	audio	faudio	\N	FAudio-20.09
2020-09-13 17:09:25+02	2020-09-13 17:14:12+02	audio	fdk-aac	\N	fdk-aac-2.0.1
2020-09-13 17:14:12+02	2020-09-13 17:15:47+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-13 17:15:47+02	2020-09-13 17:16:15+02	audio	sndio	\N	sndio-1.7.0
2020-09-13 17:16:16+02	2020-09-13 17:17:48+02	audio	swhplugins	\N	swhplugins-0.4.15_7
2020-09-13 17:17:49+02	2020-09-13 17:18:29+02	audio	wavpack	\N	wavpack-5.3.0
2020-09-13 17:18:30+02	2020-09-13 17:18:47+02	databases	lmdb	\N	lmdb-0.9.24_2,1
2020-09-13 17:18:48+02	2020-09-13 17:19:25+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.14.2
2020-09-13 17:19:25+02	2020-09-13 17:19:49+02	databases	pgtcl	\N	pgtcl-2.1.1_2
2020-09-13 17:19:49+02	2020-09-13 17:20:14+02	databases	py-apsw	py37	py37-apsw-3.33.0
2020-09-13 17:20:15+02	2020-09-13 17:22:05+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-09-13 17:22:06+02	2020-09-13 17:24:08+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-09-13 17:24:09+02	2020-09-13 17:29:47+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-13 17:29:48+02	2020-09-13 17:31:02+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-13 16:58:25+02	2020-09-13 17:31:44+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-13 17:31:03+02	2020-09-13 17:32:00+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-13 17:32:03+02	2020-09-13 17:33:04+02	devel	adacurses	\N	adacurses-20180127
2020-09-13 17:33:05+02	2020-09-13 17:33:22+02	devel	cweb	\N	cweb-3.64a.a
2020-09-13 17:33:22+02	2020-09-13 17:33:52+02	devel	erlang-cl	\N	erlang-cl-1.2.4
2020-09-13 17:33:52+02	2020-09-13 17:34:17+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-09-13 17:34:18+02	2020-09-13 17:34:41+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-09-13 17:34:41+02	2020-09-13 17:35:04+02	devel	fpc-fv	\N	fpc-fv-3.0.4_4
2020-09-13 17:35:05+02	2020-09-13 17:35:24+02	devel	fpc-gdbint	\N	fpc-gdbint-3.0.4_4
2020-09-13 17:35:25+02	2020-09-13 17:36:15+02	devel	fstrm	\N	fstrm-0.6.0
2020-09-13 17:36:15+02	2020-09-13 17:39:41+02	devel	google-perftools	\N	google-perftools-2.8
2020-09-13 17:39:41+02	2020-09-13 17:41:37+02	devel	gradle5	\N	gradle5-5.0
2020-09-13 17:31:45+02	2020-09-13 17:44:14+02	devel	aarch64-gcc	\N	aarch64-gcc-6.4.0_8
2020-09-13 17:44:15+02	2020-09-13 17:45:30+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.73.0
2020-09-13 17:45:31+02	2020-09-13 17:47:00+02	devel	libarea	\N	libarea-20160313_13
2020-09-13 17:47:00+02	2020-09-13 17:47:46+02	devel	libstatgrab	\N	libstatgrab-0.92
2020-09-13 17:41:37+02	2020-09-13 17:47:54+02	devel	gsoap	\N	gsoap-2.8.106
2020-09-13 17:47:47+02	2020-09-13 17:47:54+02	devel	libsysctlmibinfo2	\N	libsysctlmibinfo2-2.0.0_1
2020-09-13 17:47:56+02	2020-09-13 17:48:05+02	devel	libublio	\N	libublio-20070103_2
2020-09-13 17:47:56+02	2020-09-13 17:50:04+02	devel	mercurial	py37	mercurial-5.5_1
2020-09-13 17:50:05+02	2020-09-13 17:52:17+02	devel	ode	\N	ode-0.13_4,1
2020-09-13 17:52:18+02	2020-09-13 17:52:43+02	devel	p5-Term-ReadKey	\N	p5-Term-ReadKey-2.38_1
2020-09-13 17:48:06+02	2020-09-13 18:10:43+02	devel	mips64-gcc	\N	mips64-gcc-6.4.0_8
2020-09-13 18:10:47+02	2020-09-13 18:15:10+02	devel	protobuf-c	\N	protobuf-c-1.3.2_6
2020-09-13 18:15:11+02	2020-09-13 18:15:24+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-09-13 18:15:25+02	2020-09-13 18:15:44+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-09-13 18:15:44+02	2020-09-13 18:16:07+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-09-13 18:16:08+02	2020-09-13 18:16:24+02	devel	py-funcy	py37	py37-funcy-1.12
2020-09-13 17:52:43+02	2020-09-13 18:17:04+02	devel	powerpc64-gcc	\N	powerpc64-gcc-6.4.0_8
2020-09-13 18:17:05+02	2020-09-13 18:17:21+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-09-13 18:17:21+02	2020-09-13 18:17:33+02	devel	py-iniparse	py37	py37-iniparse-0.5
2020-09-13 18:17:34+02	2020-09-13 18:17:45+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-09-13 18:17:46+02	2020-09-13 18:18:04+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-09-13 18:16:25+02	2020-09-13 18:18:04+02	devel	py-game	py37	py37-game-1.9.6_1
2020-09-13 18:18:04+02	2020-09-13 18:18:17+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-09-13 18:18:17+02	2020-09-13 18:18:29+02	devel	py-rq	py37	py37-rq-1.3.0
2020-09-13 18:18:29+02	2020-09-13 18:19:04+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-09-13 18:19:04+02	2020-09-13 18:19:23+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-09-13 18:19:24+02	2020-09-13 18:19:42+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-09-13 18:19:43+02	2020-09-13 18:19:55+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-09-13 18:18:05+02	2020-09-13 18:20:38+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-09-13 18:19:56+02	2020-09-13 18:22:27+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-13 18:22:28+02	2020-09-13 18:22:59+02	devel	rlwrap	\N	rlwrap-0.42_2
2020-09-13 18:23:03+02	2020-09-13 18:23:14+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-09-13 18:23:15+02	2020-09-13 18:23:26+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-09-13 18:23:27+02	2020-09-13 18:23:40+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-09-13 18:23:41+02	2020-09-13 18:33:34+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.4_1
2020-09-13 18:20:39+02	2020-09-13 18:45:39+02	devel	riscv64-gcc	\N	riscv64-gcc-8.3.0
2020-09-13 18:45:40+02	2020-09-13 18:46:34+02	devel	tex-synctex	\N	tex-synctex-1.17.0_1
2020-09-13 18:46:34+02	2020-09-13 18:46:50+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-09-13 18:46:50+02	2020-09-13 18:47:05+02	dns	py-dnspython	py27	py27-dnspython-1.16.0
2020-09-13 18:47:06+02	2020-09-13 18:49:23+02	dns	py-ldns	py27	py27-ldns-1.7.1_1
2020-09-13 18:33:35+02	2020-09-13 18:53:07+02	devel	sparc64-gcc	\N	sparc64-gcc-6.4.0_8
2020-09-13 18:49:24+02	2020-09-13 19:16:50+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-13 19:16:51+02	2020-09-13 19:23:28+02	editors	pluma	\N	pluma-1.24.0
2020-09-13 19:23:28+02	2020-09-13 19:27:42+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-09-13 19:27:43+02	2020-09-13 19:30:03+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-13 19:30:04+02	2020-09-13 19:36:51+02	french	med	\N	fr-med-4.0.0
2020-09-13 19:36:52+02	2020-09-13 19:38:21+02	ftp	libfilezilla	\N	libfilezilla-0.15.1_3
2020-09-13 19:38:21+02	2020-09-13 19:57:06+02	games	alienarena-data	\N	alienarena-data-2013.766
2020-09-13 19:57:07+02	2020-09-13 19:58:59+02	games	gnuchess	\N	gnuchess-6.2.7
2020-09-13 19:58:59+02	2020-09-13 20:04:40+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-09-13 20:04:41+02	2020-09-13 20:16:32+02	graphics	alembic	\N	alembic-1.7.15
2020-09-13 20:16:33+02	2020-09-13 20:21:43+02	graphics	atril	\N	atril-1.22.3_9
2020-09-13 20:21:44+02	2020-09-13 21:04:12+02	graphics	embree	\N	embree-3.10.0_1
2020-09-13 21:04:13+02	2020-09-13 21:07:45+02	graphics	eom	\N	eom-1.24.0
2020-09-13 21:07:46+02	2020-09-13 21:10:07+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-13 21:10:08+02	2020-09-13 21:10:44+02	graphics	gle	\N	gle-3.1.0_7
2020-09-13 21:10:44+02	2020-09-13 21:11:10+02	graphics	gltt	\N	gltt-2.5.2_14
2020-09-13 21:11:11+02	2020-09-13 21:13:15+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-13 21:13:15+02	2020-09-13 21:13:26+02	graphics	jgraph	\N	jgraph-8.3_1
2020-09-13 21:13:26+02	2020-09-13 21:14:11+02	graphics	libart_lgpl	\N	libart_lgpl-2.3.21_3,1
2020-09-13 21:14:12+02	2020-09-13 21:22:24+02	graphics	oidn	\N	oidn-1.1.0_3
2020-09-13 21:22:25+02	2020-09-13 21:52:57+02	graphics	opencollada	\N	opencollada-1.6.68_2
2020-09-13 21:52:58+02	2020-09-13 22:18:04+02	graphics	openimageio	\N	openimageio-2.1.15.0_6
2020-09-13 22:18:05+02	2020-09-13 22:23:48+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-09-13 22:23:50+02	2020-09-13 22:26:15+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-13 22:26:16+02	2020-09-13 22:36:14+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-09-13 22:36:16+02	2020-09-13 22:36:59+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-09-13 22:37:01+02	2020-09-13 22:37:55+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-09-13 22:37:57+02	2020-09-13 22:54:36+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-13 22:54:37+02	2020-09-13 23:37:10+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-09-13 23:37:11+02	2020-09-13 23:49:06+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-13 23:49:07+02	2020-09-14 00:58:49+02	lang	gcc48	\N	gcc48-4.8.5_11
2020-09-13 18:53:08+02	2020-09-14 02:14:42+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-09-14 02:14:45+02	2020-09-14 02:15:06+02	lang	lua51	\N	lua51-5.1.5_9
2020-09-14 02:15:06+02	2020-09-14 02:16:20+02	lang	luajit	\N	luajit-2.0.5_4
2020-09-14 02:16:20+02	2020-09-14 02:16:36+02	lang	python3	\N	python3-3_3
2020-09-14 02:16:38+02	2020-09-14 02:27:08+02	lang	sbcl	\N	sbcl-2.0.7,1
2020-09-14 02:27:08+02	2020-09-14 02:29:47+02	math	mate-calc	\N	mate-calc-1.24.0
2020-09-14 02:29:47+02	2020-09-14 02:31:12+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-09-14 02:31:13+02	2020-09-14 02:36:43+02	math	py-numpy	py38	py38-numpy-1.16.6,1
2020-09-14 02:36:44+02	2020-09-14 02:41:15+02	math	stp	\N	stp-2.3.3_6,1
2020-09-14 00:58:50+02	2020-09-14 03:04:07+02	lang	julia11	\N	julia11-1.1.1_5
2020-09-14 03:04:08+02	2020-09-14 03:04:32+02	misc	chmlib	\N	chmlib-0.40_1
2020-09-14 03:04:32+02	2020-09-14 03:04:40+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200616
2020-09-14 03:04:41+02	2020-09-14 03:12:36+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-09-14 03:12:37+02	2020-09-14 03:13:08+02	misc	ini_file_manager	\N	ini_file_manager-03_3
2020-09-14 02:41:15+02	2020-09-14 03:15:46+02	math	z3	\N	z3-4.8.8
2020-09-14 03:15:48+02	2020-09-14 03:16:13+02	misc	xosd	\N	xosd-2.2.14_3
2020-09-14 03:16:14+02	2020-09-14 03:17:31+02	multimedia	gmmlib	\N	gmmlib-20.2.5
2020-09-14 03:17:31+02	2020-09-14 03:18:29+02	multimedia	libmtp	\N	libmtp-1.1.17
2020-09-14 03:18:30+02	2020-09-14 03:27:42+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.106_1
2020-09-14 03:27:44+02	2020-09-14 03:29:23+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-09-14 03:29:24+02	2020-09-14 03:34:19+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-09-14 03:13:09+02	2020-09-14 03:37:54+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-09-14 03:37:56+02	2020-09-14 03:40:55+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-14 03:40:56+02	2020-09-14 03:43:10+02	net	ndpi	\N	ndpi-3.2.d20200721,1
2020-09-14 03:43:10+02	2020-09-14 03:43:38+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-09-14 03:43:39+02	2020-09-14 03:45:00+02	net	py-pyzmq	py37	py37-pyzmq-19.0.2
2020-09-14 03:45:02+02	2020-09-14 03:46:28+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-14 03:46:29+02	2020-09-14 03:48:09+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-14 03:48:10+02	2020-09-14 03:49:42+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-14 03:49:43+02	2020-09-14 03:51:21+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-14 03:51:23+02	2020-09-14 03:52:40+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-14 03:52:40+02	2020-09-14 03:53:55+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-14 03:34:20+02	2020-09-14 03:54:40+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-14 03:54:01+02	2020-09-14 03:55:18+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-14 03:55:19+02	2020-09-14 03:56:20+02	net	vde2	\N	vde2-2.3.2_4
2020-09-14 03:54:41+02	2020-09-14 03:56:37+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-14 03:56:21+02	2020-09-14 03:56:51+02	net	wireguard-go	\N	wireguard-go-0.0.20200320
2020-09-14 03:56:38+02	2020-09-14 03:57:48+02	net-mgmt	icingaweb2-module-director	php73	icingaweb2-module-director-php73-1.6.2_2
2020-09-14 03:56:52+02	2020-09-14 03:57:50+02	net-mgmt	libsmi	\N	libsmi-0.4.8_1
2020-09-14 03:57:51+02	2020-09-14 03:58:06+02	net-mgmt	py-ipcalc	py27	py27-ipcalc-1.99.0
2020-09-14 03:58:06+02	2020-09-14 03:58:32+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-09-14 03:58:32+02	2020-09-14 03:58:41+02	ports-mgmt	dialog4ports	\N	dialog4ports-0.1.6
2020-09-14 03:57:48+02	2020-09-14 03:59:44+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-09-14 03:58:42+02	2020-09-14 04:00:31+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-14 04:00:32+02	2020-09-14 04:10:41+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-14 04:10:42+02	2020-09-14 04:19:41+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-14 05:36:45+02	2020-09-14 05:47:51+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-14 05:47:52+02	2020-09-14 05:48:37+02	print	transfig	\N	transfig-3.2.5e_4
2020-09-14 05:48:38+02	2020-09-14 05:49:11+02	security	libntlm	\N	libntlm-1.6
2020-09-14 05:49:12+02	2020-09-14 05:49:26+02	security	libscrypt	\N	libscrypt-1.21
2020-09-14 05:49:26+02	2020-09-14 05:53:06+02	security	nmap	\N	nmap-7.80
2020-09-14 05:53:06+02	2020-09-14 05:53:50+02	security	openconnect	\N	openconnect-8.10
2020-09-14 05:53:51+02	2020-09-14 05:54:14+02	security	p5-Digest-SHA1	\N	p5-Digest-SHA1-2.13_1
2020-09-14 05:54:15+02	2020-09-14 05:54:27+02	security	pkesh	\N	pkesh-1.0
2020-09-14 05:54:28+02	2020-09-14 05:54:40+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-09-14 05:54:41+02	2020-09-14 05:54:53+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-09-14 05:54:54+02	2020-09-14 05:55:16+02	security	py-stem	py37	py37-stem-1.8.0
2020-09-14 05:55:17+02	2020-09-14 05:58:51+02	sysutils	coreutils	\N	coreutils-8.32
2020-09-14 05:58:52+02	2020-09-14 06:01:52+02	sysutils	intel-pcm	\N	intel-pcm-202008_1
2020-09-14 06:01:52+02	2020-09-14 06:02:38+02	sysutils	libcpuid	\N	libcpuid-0.5.0
2020-09-14 05:36:45+02	2020-09-14 06:02:49+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-14 06:02:39+02	2020-09-14 06:03:48+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-14 06:02:50+02	2020-09-14 06:05:21+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-09-14 06:05:23+02	2020-09-14 06:05:39+02	sysutils	smart	\N	smart-0.0.11
2020-09-14 06:05:40+02	2020-09-14 06:06:45+02	sysutils	smartmontools	\N	smartmontools-7.1_2
2020-09-14 06:03:49+02	2020-09-14 06:07:32+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-09-14 06:06:46+02	2020-09-14 06:07:44+02	sysutils	tarsnap	\N	tarsnap-1.0.39
2020-09-14 06:07:32+02	2020-09-14 06:08:35+02	textproc	diffutils	\N	diffutils-3.7
2020-09-14 06:08:37+02	2020-09-14 06:10:02+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-14 06:07:44+02	2020-09-14 06:10:03+02	textproc	fop	\N	fop-2.3
2020-09-14 06:10:03+02	2020-09-14 06:10:47+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-09-14 06:10:48+02	2020-09-14 06:11:11+02	textproc	lowdown	\N	lowdown-0.7.3
2020-09-14 06:11:12+02	2020-09-14 06:11:32+02	textproc	p5-pod2pdf	\N	p5-pod2pdf-0.42_1
2020-09-14 06:11:33+02	2020-09-14 06:13:59+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-09-14 06:14:05+02	2020-09-14 06:15:25+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-14 06:15:25+02	2020-09-14 06:16:21+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-09-14 06:16:21+02	2020-09-14 06:16:34+02	textproc	py-scour	py37	py37-scour-0.38.1
2020-09-14 06:16:36+02	2020-09-14 06:16:56+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-09-14 06:16:56+02	2020-09-14 06:25:43+02	textproc	ripgrep	\N	ripgrep-12.1.1_5
2020-09-14 06:25:44+02	2020-09-14 06:32:20+02	textproc	xerces-c3	\N	xerces-c3-3.2.3
2020-09-14 06:32:26+02	2020-09-14 06:33:35+02	www	mattermost-webapp	\N	mattermost-webapp-5.26.1
2020-09-14 06:33:36+02	2020-09-14 06:34:10+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-09-14 06:34:11+02	2020-09-14 06:34:36+02	www	p5-libwww	\N	p5-libwww-6.47
2020-09-14 06:34:37+02	2020-09-14 06:35:16+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-09-14 06:35:17+02	2020-09-14 06:35:35+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-09-14 06:35:35+02	2020-09-14 06:35:57+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-09-14 06:36:01+02	2020-09-14 06:36:20+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-09-14 06:36:21+02	2020-09-14 06:36:35+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-09-17 14:41:14+02	2020-09-17 14:43:19+02	devel	ode	\N	ode-0.13_4,1
2020-09-14 06:36:37+02	2020-09-14 06:36:58+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-09-14 06:36:59+02	2020-09-14 06:37:16+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-09-14 06:37:17+02	2020-09-14 06:39:46+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-14 06:39:47+02	2020-09-14 06:41:51+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-14 06:41:52+02	2020-09-14 06:42:08+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-09-14 06:42:09+02	2020-09-14 06:50:34+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-14 06:50:35+02	2020-09-14 06:52:22+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-14 06:52:22+02	2020-09-14 06:54:49+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-14 06:54:50+02	2020-09-14 06:59:00+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-14 06:59:02+02	2020-09-14 07:03:43+02	x11-fm	nautilus	\N	nautilus-3.28.1_3
2020-09-14 07:03:49+02	2020-09-14 07:04:05+02	x11-fonts	bitstream-vera	\N	bitstream-vera-1.10_8
2020-09-14 07:04:07+02	2020-09-14 07:40:42+02	x11-fonts	nerd-fonts	\N	nerd-fonts-2.1.0
2020-09-14 07:40:50+02	2020-09-14 07:43:06+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-09-14 08:59:49+02	2020-09-14 09:29:08+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-14 09:29:10+02	2020-09-14 09:29:27+02	archivers	pbzip2	\N	pbzip2-1.1.13
2020-09-14 09:29:29+02	2020-09-14 09:30:08+02	archivers	unrar	\N	unrar-5.91,6
2020-09-14 09:28:34+02	2020-09-14 11:10:04+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-14 11:10:06+02	2020-09-14 11:10:29+02	astro	xearth	\N	xearth-1.2
2020-09-14 11:10:31+02	2020-09-14 11:16:25+02	astro	xephem	\N	xephem-3.7.7_5
2020-09-14 11:16:28+02	2020-09-14 11:17:12+02	audio	xmix	\N	xmix-2.1_3
2020-09-14 11:17:14+02	2020-09-14 11:37:56+02	benchmarks	fio	\N	fio-3.22
2020-09-14 11:37:57+02	2020-09-14 11:38:42+02	benchmarks	iperf3	\N	iperf3-3.9
2020-09-14 18:49:58+02	2020-09-14 19:34:08+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-14 19:34:10+02	2020-09-14 19:37:33+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-14 09:30:10+02	2020-09-14 19:39:33+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-14 19:39:35+02	2020-09-14 19:40:48+02	comms	kermit	\N	kermit-9.0.304.24
2020-09-14 19:37:34+02	2020-09-14 19:41:05+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-14 19:40:49+02	2020-09-14 19:41:16+02	converters	dosunix	\N	dosunix-1.0.14
2020-09-14 19:41:18+02	2020-09-14 19:41:32+02	converters	unix2dos	\N	unix2dos-1.3
2020-09-14 19:41:33+02	2020-09-14 19:42:07+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-09-14 19:41:09+02	2020-09-14 19:43:44+02	converters	recode	\N	recode-3.6_15
2020-09-14 19:43:47+02	2020-09-14 19:44:25+02	databases	pgtop	\N	pgtop-0.05_5
2020-09-14 19:44:26+02	2020-09-14 19:56:40+02	databases	postgresql12-contrib	\N	postgresql12-contrib-12.4
2020-09-14 19:56:43+02	2020-09-14 20:12:07+02	databases	postgresql12-server	\N	postgresql12-server-12.4
2020-09-14 20:12:08+02	2020-09-14 20:12:35+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.6
2020-09-14 20:12:36+02	2020-09-14 20:13:28+02	databases	redis	\N	redis-5.0.9
2020-09-14 19:42:08+02	2020-09-14 20:16:30+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-14 20:16:31+02	2020-09-14 20:17:31+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-14 20:17:32+02	2020-09-14 20:17:47+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-09-14 20:17:47+02	2020-09-14 20:18:03+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-09-14 20:13:28+02	2020-09-14 20:18:04+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-14 20:18:03+02	2020-09-14 20:18:12+02	devel	aarch64-xtoolchain-gcc	\N	aarch64-xtoolchain-gcc-0.4_1
2020-09-14 20:18:12+02	2020-09-14 20:18:49+02	devel	cdecl	\N	cdecl-8.0
2020-09-14 20:18:04+02	2020-09-14 20:19:26+02	devel	bzr	py27	bzr-2.7.0.6622
2020-09-14 20:19:27+02	2020-09-14 20:20:08+02	devel	cvs+ipv6	\N	cvs+ipv6-1.20120905_3
2020-09-14 20:20:08+02	2020-09-14 20:21:15+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-14 20:18:49+02	2020-09-14 21:20:50+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-14 19:30:54+02	2020-09-14 21:33:23+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-14 21:20:51+02	2020-09-14 22:19:42+02	devel	ghidra	\N	ghidra-9.1
2020-09-14 22:19:43+02	2020-09-15 06:49:08+02	devel	llvm10	\N	llvm10-10.0.1_1
2020-09-15 06:49:12+02	2020-09-15 06:49:27+02	devel	mips64-xtoolchain-gcc	\N	mips64-xtoolchain-gcc-0.4_1
2020-09-15 06:49:28+02	2020-09-15 06:49:45+02	devel	powerpc64-xtoolchain-gcc	\N	powerpc64-xtoolchain-gcc-0.4_1
2020-09-15 06:49:48+02	2020-09-15 06:50:07+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-09-15 06:50:08+02	2020-09-15 06:50:24+02	devel	riscv64-xtoolchain-gcc	\N	riscv64-xtoolchain-gcc-0.4_1
2020-09-15 06:50:26+02	2020-09-15 06:50:39+02	devel	sparc64-xtoolchain-gcc	\N	sparc64-xtoolchain-gcc-0.4_1
2020-09-15 06:50:45+02	2020-09-15 07:01:28+02	devel	subversion	\N	subversion-1.14.0
2020-09-15 07:01:28+02	2020-09-15 07:02:28+02	devel	tig	\N	tig-2.5.1
2020-09-15 07:02:29+02	2020-09-15 07:03:42+02	devel	tortoisehg	py37	tortoisehg-5.5.1
2020-09-15 07:03:44+02	2020-09-15 07:13:12+02	dns	bind916	\N	bind916-9.16.6_1
2020-09-15 07:13:13+02	2020-09-15 07:13:31+02	dns	sshfp	\N	sshfp-1.2.2_5
2020-09-14 20:21:16+02	2020-09-15 08:46:35+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-15 08:46:40+02	2020-09-15 08:47:20+02	editors	fpc-ide	\N	fpc-ide-3.0.4_4
2020-09-15 07:13:32+02	2020-09-15 08:49:07+02	editors	codelite	\N	codelite-12.0_4
2020-09-15 08:47:21+02	2020-09-15 08:49:15+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-15 08:49:08+02	2020-09-15 08:52:56+02	editors	vim	\N	vim-8.2.1558
2020-09-14 21:33:24+02	2020-09-15 09:16:56+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200811
2020-09-15 08:49:16+02	2020-09-15 09:54:45+02	emulators	qemu	\N	qemu-4.2.1
2020-09-15 09:54:47+02	2020-09-15 09:55:34+02	emulators	wine-gecko	\N	wine-gecko-2.47.1
2020-09-15 09:55:34+02	2020-09-15 09:57:02+02	emulators	wine-mono	\N	wine-mono-4.9.4_1
2020-09-15 09:57:03+02	2020-09-15 09:58:01+02	emulators	x48	\N	x48-0.6.4_1
2020-09-15 08:52:57+02	2020-09-15 10:05:11+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-09-15 09:16:57+02	2020-09-15 10:05:41+02	emulators	wine	\N	wine-5.0.2,1
2020-09-15 10:05:42+02	2020-09-15 10:06:05+02	games	KnightCap	\N	knightcap-3.6_8
2020-09-15 10:06:05+02	2020-09-15 10:06:16+02	games	affenspiel	\N	affenspiel-1.0_2
2020-09-15 10:05:13+02	2020-09-15 10:06:34+02	ftp	wget	\N	wget-1.20.3
2020-09-15 10:06:17+02	2020-09-15 10:06:39+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-09-15 10:06:40+02	2020-09-15 10:08:39+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-09-15 10:06:35+02	2020-09-15 10:09:13+02	games	alienarena	\N	alienarena-2013.766_9
2020-09-15 10:09:14+02	2020-09-15 10:10:58+02	games	bsdgames	\N	bsdgames-0.75,2
2020-09-15 10:10:58+02	2020-09-15 10:11:21+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-09-15 09:58:01+02	2020-09-15 10:11:46+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-15 10:11:21+02	2020-09-15 10:11:57+02	games	monsterz	\N	monsterz-0.7.1_15
2020-09-15 10:11:46+02	2020-09-15 10:12:18+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-09-15 10:12:18+02	2020-09-15 10:12:29+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-09-15 10:11:58+02	2020-09-15 10:12:49+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-09-15 10:12:29+02	2020-09-15 10:13:29+02	games	stockfish	\N	stockfish-11
2020-09-15 10:12:50+02	2020-09-15 10:13:58+02	games	xblast	\N	xblast-2.10.4_18
2020-09-15 10:13:30+02	2020-09-15 10:14:34+02	games	xboard	\N	xboard-4.9.1_1
2020-09-15 10:08:40+02	2020-09-15 10:14:39+02	games	battletanks	\N	btanks-0.9.8083_9
2020-09-15 10:13:59+02	2020-09-15 10:14:41+02	games	xcowsay	\N	xcowsay-1.5
2020-09-15 10:14:41+02	2020-09-15 10:15:38+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-15 10:14:40+02	2020-09-15 10:18:53+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-15 10:18:55+02	2020-09-15 10:19:11+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-09-15 10:19:13+02	2020-09-15 10:24:00+02	graphics	wings	\N	wings-2.2.5
2020-09-15 10:24:01+02	2020-09-15 10:27:04+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-15 10:27:08+02	2020-09-15 10:29:46+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-09-15 10:29:47+02	2020-09-15 11:09:35+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-09-15 11:09:36+02	2020-09-15 11:11:04+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-15 10:14:35+02	2020-09-15 11:51:19+02	graphics	blender	\N	blender-2.90.0
2020-09-15 11:11:05+02	2020-09-15 11:56:59+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-15 11:57:01+02	2020-09-15 11:57:17+02	lang	gcc	\N	gcc-9_4
2020-09-15 10:15:39+02	2020-09-15 12:18:35+02	graphics	krita	\N	krita-4.3.0_5
2020-09-15 12:18:39+02	2020-09-15 12:19:19+02	lang	julia	\N	julia-1.0.3_3
2020-09-15 12:19:20+02	2020-09-15 12:19:41+02	lang	mmix	\N	mmix-20160804
2020-09-15 11:51:20+02	2020-09-15 12:37:28+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.4
2020-09-15 12:37:29+02	2020-09-15 12:37:41+02	lang	python	\N	python-3.7_3,2
2020-09-15 12:37:41+02	2020-09-15 12:37:50+02	lang	python-mode.el	\N	python-mode.el-1.0_2,1
2020-09-15 12:37:50+02	2020-09-15 12:43:05+02	mail	alpine	\N	alpine-2.23.2
2020-09-15 12:43:06+02	2020-09-15 12:43:29+02	mail	bbmail	\N	bbmail-0.9.3
2020-09-15 12:43:29+02	2020-09-15 12:50:11+02	math	maxima	\N	maxima-5.44.0_3
2020-09-15 12:50:12+02	2020-09-15 12:50:26+02	math	sc	\N	sc-7.16_3
2020-09-15 12:50:26+02	2020-09-15 12:51:11+02	misc	gnu-watch	\N	gnu-watch-3.3.16
2020-09-15 12:51:12+02	2020-09-15 12:51:24+02	misc	mbuffer	\N	mbuffer-20200505
2020-09-15 12:51:24+02	2020-09-15 12:54:14+02	misc	mc	\N	mc-nox11-4.8.24
2020-09-15 12:54:15+02	2020-09-15 12:55:04+02	misc	pinfo	\N	pinfo-0.6.10_5
2020-09-15 12:55:05+02	2020-09-15 12:56:38+02	multimedia	celluloid	\N	celluloid-0.19
2020-09-15 12:56:39+02	2020-09-15 12:57:13+02	multimedia	libva-glx	\N	libva-glx-2.8.0
2020-09-15 12:57:14+02	2020-09-15 12:58:28+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-09-15 12:19:41+02	2020-09-15 13:16:46+02	lang	mono	\N	mono-5.10.1.57_3
2020-09-15 13:16:48+02	2020-09-15 13:17:49+02	multimedia	libva-utils	\N	libva-utils-2.8.0
2020-09-15 13:17:55+02	2020-09-15 13:18:16+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-09-15 13:18:17+02	2020-09-15 13:27:54+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-09-15 12:58:29+02	2020-09-15 13:28:34+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p7
2020-09-15 13:27:55+02	2020-09-15 13:30:37+02	multimedia	openshot	py37	openshot-2.5.1.236
2020-09-15 13:30:38+02	2020-09-15 13:31:14+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-09-15 13:31:14+02	2020-09-15 13:32:40+02	net	mosh	\N	mosh-1.3.2_14
2020-09-15 13:32:41+02	2020-09-15 13:39:43+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-09-15 13:39:43+02	2020-09-15 13:40:38+02	net	rdesktop	\N	rdesktop-1.9.0
2020-09-15 13:28:36+02	2020-09-15 13:40:44+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-15 13:40:45+02	2020-09-15 13:40:54+02	net	svnup	\N	svnup-1.08
2020-09-15 13:40:55+02	2020-09-15 13:41:12+02	net	torsocks	\N	torsocks-2.3.0_1
2020-09-15 13:40:39+02	2020-09-15 13:42:15+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-15 13:41:13+02	2020-09-15 13:42:24+02	net	v2ray	\N	v2ray-4.25.0
2020-09-15 13:42:16+02	2020-09-15 13:42:35+02	net	vnstat	\N	vnstat-2.6_1
2020-09-15 13:42:24+02	2020-09-15 13:43:04+02	net	wireguard	\N	wireguard-1.0.20200827
2020-09-15 13:43:04+02	2020-09-15 13:44:10+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-15 13:42:35+02	2020-09-15 14:09:04+02	net	wireshark	\N	wireshark-3.2.6
2020-09-15 13:44:10+02	2020-09-15 14:09:28+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-09-15 14:09:05+02	2020-09-15 14:09:58+02	net-mgmt	icingaweb2-module-businessprocess	php73	icingaweb2-module-businessprocess-php73-2.2.0_2
2020-09-15 14:09:28+02	2020-09-15 14:10:21+02	net-mgmt	icingaweb2-module-cube	php73	icingaweb2-module-cube-php73-1.1.0_1
2020-09-15 14:09:59+02	2020-09-15 14:10:49+02	net-mgmt	icingaweb2-module-elasticsearch	php73	icingaweb2-module-elasticsearch-php73-0.9.0_3
2020-09-15 14:10:22+02	2020-09-15 14:11:11+02	net-mgmt	icingaweb2-module-fileshipper	php73	icingaweb2-module-fileshipper-php73-1.1.0_3
2020-09-15 14:10:50+02	2020-09-15 14:11:41+02	net-mgmt	icingaweb2-module-generictts	php73	icingaweb2-module-generictts-php73-2.0.0_3
2020-09-15 14:11:12+02	2020-09-15 14:12:02+02	net-mgmt	icingaweb2-module-grafana	php73	icingaweb2-module-grafana-php73-1.3.6_1
2020-09-15 14:11:41+02	2020-09-15 14:12:34+02	net-mgmt	icingaweb2-module-graphite	php73	icingaweb2-module-graphite-php73-1.1.0_3
2020-09-15 14:12:02+02	2020-09-15 14:12:59+02	net-mgmt	icingaweb2-module-map	php73	icingaweb2-module-map-php73-1.1.0_3
2020-09-15 14:13:00+02	2020-09-15 14:13:13+02	net-mgmt	iftop	\N	iftop-1.0.p4
2020-09-15 14:12:35+02	2020-09-15 14:13:27+02	net-mgmt	icingaweb2-module-vsphere	php73	icingaweb2-module-vsphere-php73-1.1.0_4
2020-09-15 14:13:14+02	2020-09-15 14:13:30+02	net-mgmt	ipcalc	\N	ipcalc-0.41_3
2020-09-15 14:13:31+02	2020-09-15 14:13:40+02	net-mgmt	nstat	\N	nstat-1.0_3
2020-09-15 14:13:40+02	2020-09-15 14:13:52+02	net-mgmt	whatmask	\N	whatmask-1.2_1
2020-09-15 14:13:27+02	2020-09-15 14:14:10+02	net-mgmt	ipv6calc	\N	ipv6calc-2.2.0
2020-09-15 14:14:10+02	2020-09-15 14:14:26+02	ports-mgmt	pkg_tree	\N	pkg_tree-1.1_4
2020-09-15 14:14:26+02	2020-09-15 14:15:30+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-09-15 14:15:31+02	2020-09-15 14:16:05+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-09-15 14:13:53+02	2020-09-15 14:16:14+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-09-15 14:16:05+02	2020-09-15 14:17:48+02	ports-mgmt	synth	\N	synth-2.08
2020-09-15 14:17:48+02	2020-09-15 14:18:37+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-09-15 14:16:14+02	2020-09-15 14:19:20+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-15 14:18:38+02	2020-09-15 14:19:27+02	print	gv	\N	gv-3.7.4_5
2020-09-15 14:19:23+02	2020-09-15 14:46:02+02	print	lyx	\N	lyx-2.3.4.2_1
2020-09-15 14:46:03+02	2020-09-15 14:46:57+02	security	aide	\N	aide-0.16.1
2020-09-15 14:46:58+02	2020-09-15 14:47:11+02	security	base-audit	\N	base-audit-0.4
2020-09-15 14:47:11+02	2020-09-15 14:52:04+02	security	clamav	\N	clamav-0.102.4,1
2020-09-15 14:52:04+02	2020-09-15 14:52:13+02	security	doas	\N	doas-6.3_1
2020-09-15 14:52:14+02	2020-09-15 14:52:59+02	security	ipv6toolkit	\N	ipv6toolkit-2.0
2020-09-15 14:53:01+02	2020-09-15 14:53:28+02	security	kc24	\N	kc24-2.4.2_1
2020-09-15 14:53:29+02	2020-09-15 14:59:42+02	security	klee	\N	klee-2.1_1
2020-09-15 14:59:43+02	2020-09-15 14:59:55+02	security	lynis	\N	lynis-3.0.0
2020-09-15 14:59:55+02	2020-09-15 15:00:15+02	security	nyx	\N	nyx-2.1.0_1
2020-09-15 15:00:15+02	2020-09-15 15:01:44+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-09-15 15:01:44+02	2020-09-15 15:03:25+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-09-15 15:03:26+02	2020-09-15 15:03:48+02	security	py-xkcdpass	py37	py37-xkcdpass-1.17.4
2020-09-15 15:03:49+02	2020-09-15 15:04:15+02	security	rkhunter	\N	rkhunter-1.4.6
2020-09-15 15:04:16+02	2020-09-15 15:04:36+02	security	ssl-admin	\N	ssl-admin-1.2.1
2020-09-15 15:04:36+02	2020-09-15 15:11:53+02	security	tor	\N	tor-0.4.3.6
2020-09-15 14:19:27+02	2020-09-15 15:19:52+02	print	scribus-devel	\N	scribus-devel-1.5.5_17
2020-09-15 15:11:54+02	2020-09-15 15:21:26+02	shells	fish	\N	fish-3.1.2
2020-09-15 15:21:26+02	2020-09-15 15:25:19+02	shells	zsh	\N	zsh-5.8
2020-09-15 15:25:20+02	2020-09-15 15:25:29+02	sysutils	915resolution	\N	915resolution-0.5.3_1,1
2020-09-15 15:25:29+02	2020-09-15 15:28:04+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-09-15 15:28:04+02	2020-09-15 15:28:39+02	sysutils	apachetop	\N	apachetop-0.18.4_1
2020-09-15 15:28:40+02	2020-09-15 15:28:51+02	sysutils	atop	\N	atop-2.0.2.b3_3
2020-09-15 15:28:51+02	2020-09-15 15:29:04+02	sysutils	bashtop	\N	bashtop-0.9.25_1
2020-09-15 15:29:05+02	2020-09-15 15:29:12+02	sysutils	beadm	\N	beadm-1.3.0
2020-09-15 15:29:13+02	2020-09-15 15:29:29+02	sysutils	bpytop	\N	bpytop-1.0.27
2020-09-15 15:29:30+02	2020-09-15 15:29:38+02	sysutils	bsdcrashtar	\N	bsdcrashtar-0.1.3_1
2020-09-15 15:29:39+02	2020-09-15 15:29:47+02	sysutils	bsdhwmon	\N	bsdhwmon-20151206
2020-09-15 15:29:48+02	2020-09-15 15:29:58+02	sysutils	bsdinfo	\N	bsdinfo-0.22_2
2020-09-15 15:29:58+02	2020-09-15 15:30:10+02	sysutils	bsdstats	\N	bsdstats-7.0
2020-09-15 15:19:55+02	2020-09-15 15:30:20+02	shells	starship	\N	starship-0.44.0_2
2020-09-15 15:30:21+02	2020-09-15 15:30:27+02	sysutils	cmdwatch	\N	cmdwatch-0.2.0_2
2020-09-15 15:30:28+02	2020-09-15 15:31:15+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-15 15:31:16+02	2020-09-15 15:31:23+02	sysutils	cpupdate	\N	cpupdate-g20180513_2
2020-09-15 15:31:23+02	2020-09-15 15:31:39+02	sysutils	ddrescue	\N	ddrescue-1.25
2020-09-15 15:31:39+02	2020-09-15 15:31:46+02	sysutils	devcpu-data	\N	devcpu-data-1.34
2020-09-15 15:31:47+02	2020-09-15 15:31:58+02	sysutils	dfc	\N	dfc-3.1.1
2020-09-15 15:31:59+02	2020-09-15 15:32:19+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-09-15 15:30:10+02	2020-09-15 15:33:00+02	sysutils	cdrtools	\N	cdrtools-3.01_1
2020-09-15 15:33:01+02	2020-09-15 15:33:20+02	sysutils	flashrom	\N	flashrom-1.2
2020-09-15 11:57:18+02	2020-09-15 15:33:46+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200906
2020-09-15 15:33:21+02	2020-09-15 15:34:05+02	sysutils	fusefs-ntfs	\N	fusefs-ntfs-2017.3.23
2020-09-15 15:33:51+02	2020-09-15 15:34:39+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-15 15:32:19+02	2020-09-15 15:34:39+02	sysutils	exa	\N	exa-0.9.0_15
2020-09-15 15:34:06+02	2020-09-15 15:35:01+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-15 15:35:02+02	2020-09-15 15:35:11+02	sysutils	hourglass	\N	hourglass-1.0.1
2020-09-15 15:35:12+02	2020-09-15 15:35:39+02	sysutils	htop	\N	htop-2.2.0_1
2020-09-15 15:35:39+02	2020-09-15 15:35:47+02	sysutils	ioping	\N	ioping-1.2
2020-09-15 15:35:47+02	2020-09-15 15:35:55+02	sysutils	lsop	\N	lsop-0.2
2020-09-15 15:34:40+02	2020-09-15 15:36:03+02	sysutils	hexyl	\N	hexyl-0.8.0_4
2020-09-15 15:35:55+02	2020-09-15 15:36:10+02	sysutils	memtest86+	\N	memtest86+-5.01_3
2020-09-15 15:36:04+02	2020-09-15 15:36:16+02	sysutils	most	\N	most-5.1.0
2020-09-15 15:36:11+02	2020-09-15 15:36:22+02	sysutils	ncdu	\N	ncdu-1.15.1
2020-09-15 15:36:17+02	2020-09-15 15:36:25+02	sysutils	neofetch	\N	neofetch-7.1.0
2020-09-15 15:36:23+02	2020-09-15 15:36:29+02	sysutils	obliterate	\N	obliterate-0.5
2020-09-15 15:36:25+02	2020-09-15 15:36:42+02	sysutils	panicmail	\N	panicmail-1.1
2020-09-15 15:36:30+02	2020-09-15 15:36:48+02	sysutils	parallel	\N	parallel-20200822
2020-09-15 15:36:42+02	2020-09-15 15:36:49+02	sysutils	procmap	\N	procmap-1.0
2020-09-15 15:34:40+02	2020-09-15 15:36:51+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-15 15:36:50+02	2020-09-15 15:36:57+02	sysutils	pv	\N	pv-1.6.6
2020-09-15 15:36:51+02	2020-09-15 15:36:58+02	sysutils	pwgen	\N	pwgen-2.08,2
2020-09-15 15:36:49+02	2020-09-15 15:37:00+02	sysutils	psmisc	\N	psmisc-22.16_1
2020-09-15 15:37:01+02	2020-09-15 15:37:07+02	sysutils	reptyr	\N	reptyr-0.6.3a_2
2020-09-15 15:36:59+02	2020-09-15 15:37:08+02	sysutils	py-py-cpuinfo	py37	py37-py-cpuinfo-7.0.0
2020-09-15 15:36:57+02	2020-09-15 15:37:11+02	sysutils	py-glances	py37	py37-glances-3.1.4
2020-09-15 15:37:11+02	2020-09-15 15:37:19+02	sysutils	spinner	\N	spinner-1.2.4
2020-09-15 15:37:20+02	2020-09-15 15:37:26+02	sysutils	ssd_report	smart	ssd_report-0.4
2020-09-15 15:37:09+02	2020-09-15 15:37:36+02	sysutils	screen	\N	screen-4.8.0
2020-09-15 15:37:37+02	2020-09-15 15:37:43+02	sysutils	tarsnap-periodic	\N	tarsnap-periodic-1.0.20170413
2020-09-15 15:37:43+02	2020-09-15 15:38:18+02	sysutils	tmux	\N	tmux-3.1b
2020-09-15 15:38:18+02	2020-09-15 15:38:31+02	sysutils	tree	\N	tree-1.8.0
2020-09-15 15:38:32+02	2020-09-15 15:38:51+02	sysutils	x86info	\N	x86info-1.31.s03
2020-09-15 15:38:51+02	2020-09-15 15:39:16+02	sysutils	zfs-stats	\N	zfs-stats-1.3.0
2020-09-15 15:39:16+02	2020-09-15 15:40:27+02	sysutils	zrepl	\N	zrepl-0.2.1
2020-09-15 15:37:26+02	2020-09-15 15:40:56+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-09-15 15:40:56+02	2020-09-15 15:41:14+02	textproc	colordiff	\N	colordiff-1.0.19
2020-09-15 15:41:14+02	2020-09-15 15:41:37+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-09-15 15:41:38+02	2020-09-15 15:42:03+02	www	dokuwiki	\N	dokuwiki-20200729
2020-09-15 15:37:08+02	2020-09-15 15:46:46+02	sysutils	rsfetch	\N	rsfetch-2.0.0_7
2020-09-15 15:46:46+02	2020-09-15 15:47:51+02	www	links	\N	links-2.20.2,1
2020-09-15 15:47:51+02	2020-09-15 15:50:11+02	www	lynx	\N	lynx-2.8.9.1_1,1
2020-09-15 15:50:12+02	2020-09-15 15:52:36+02	www	mattermost-server	\N	mattermost-server-5.26.1
2020-09-15 15:52:36+02	2020-09-15 15:53:03+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_2
2020-09-15 15:40:28+02	2020-09-15 15:53:20+02	textproc	bat	\N	bat-0.15.4_4
2020-09-15 15:53:20+02	2020-09-15 15:54:16+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-09-15 15:54:17+02	2020-09-15 15:54:29+02	www	py-bottle	py37	py37-bottle-0.12.15_1
2020-09-15 15:54:29+02	2020-09-15 15:54:46+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-09-15 15:54:47+02	2020-09-15 15:55:03+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-09-15 15:55:04+02	2020-09-15 15:55:21+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-09-15 15:55:22+02	2020-09-15 15:55:37+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-09-15 15:55:38+02	2020-09-15 15:55:58+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-09-15 15:55:58+02	2020-09-15 15:56:15+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-09-15 15:56:16+02	2020-09-15 15:56:34+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-09-15 15:56:34+02	2020-09-15 15:56:56+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-09-15 15:56:57+02	2020-09-15 15:57:12+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-09-15 15:57:12+02	2020-09-15 15:57:28+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-09-15 15:53:04+02	2020-09-15 15:57:42+02	www	mod_php73	\N	mod_php73-7.3.22
2020-09-15 15:57:28+02	2020-09-15 15:57:49+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-09-15 15:57:43+02	2020-09-15 15:58:03+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-09-15 15:58:03+02	2020-09-15 16:00:29+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-09-15 16:00:29+02	2020-09-15 16:02:45+02	x11	mate	\N	mate-1.22.2
2020-09-15 16:02:46+02	2020-09-15 16:02:58+02	x11	numlockx	\N	numlockx-1.2
2020-09-15 16:02:59+02	2020-09-15 16:03:14+02	x11	sterm	\N	sterm-0.8.4
2020-09-15 16:03:14+02	2020-09-15 16:03:46+02	x11	xclip	\N	xclip-0.13
2020-09-15 16:03:46+02	2020-09-15 16:04:05+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-09-15 16:04:06+02	2020-09-15 16:04:41+02	x11	xdm	\N	xdm-1.1.12_3
2020-09-15 16:04:41+02	2020-09-15 16:06:44+02	x11	xlockmore	\N	xlockmore-5.65
2020-09-15 16:06:45+02	2020-09-15 16:07:07+02	x11	xman	\N	xman-1.1.5
2020-09-15 16:07:08+02	2020-09-15 16:13:50+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-15 15:57:49+02	2020-09-15 16:18:48+02	x11	alacritty	\N	alacritty-0.5.0_2
2020-09-15 16:13:51+02	2020-09-15 16:19:43+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-15 16:19:44+02	2020-09-15 16:20:05+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-09-15 16:20:05+02	2020-09-15 16:20:23+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-09-15 16:18:49+02	2020-09-15 16:20:42+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-15 16:20:42+02	2020-09-15 16:21:10+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-09-15 16:20:24+02	2020-09-15 16:21:21+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-09-15 16:21:22+02	2020-09-15 16:21:45+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-09-15 16:21:11+02	2020-09-15 16:21:52+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-09-15 16:21:52+02	2020-09-15 16:23:12+02	graphics	xfig	\N	xfig-3.2.7a
2020-09-15 16:23:13+02	2020-09-15 16:23:48+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-09-15 20:46:14+02	2020-09-16 04:10:30+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-16 04:16:48+02	2020-09-16 04:18:02+02	net-mgmt	icingaweb2-module-pdfexport	php73	icingaweb2-module-pdfexport-php73-0.9.0_2
2020-09-16 04:16:48+02	2020-09-16 04:18:41+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-16 04:18:02+02	2020-09-16 04:19:18+02	net-mgmt	icingaweb2-module-reporting	php73	icingaweb2-module-reporting-php73-0.9.2_1
2020-09-16 04:19:19+02	2020-09-16 04:20:28+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-16 04:16:48+02	2020-09-16 04:22:14+02	print	lilypond	\N	lilypond-2.20.0
2020-09-16 04:22:15+02	2020-09-16 04:52:53+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-16 04:52:54+02	2020-09-16 04:58:22+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-16 04:18:42+02	2020-09-16 05:46:00+02	textproc	hs-pandoc	\N	hs-pandoc-2.10
2020-09-16 04:58:23+02	2020-09-16 06:02:28+02	graphics	inkscape	\N	inkscape-1.0_5
2020-09-16 06:02:30+02	2020-09-16 06:04:34+02	net-mgmt	icingaweb2-module-idoreports	php73	icingaweb2-module-idoreports-php73-0.9.0_2
2020-09-16 06:04:35+02	2020-09-16 06:11:17+02	print	texlive-full	\N	texlive-full-20150521
2020-09-16 05:46:04+02	2020-09-16 07:55:31+02	devel	RStudio	\N	RStudio-1.3.1073
2020-09-16 04:20:30+02	2020-09-16 07:57:50+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-16 07:57:53+02	2020-09-16 08:54:43+02	local	E590T-localbase	\N	E590T-localbase-2020091200
2020-09-16 08:57:46+02	2020-09-16 09:01:28+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-16 08:57:46+02	2020-09-16 09:04:37+02	graphics	gpu-firmware-kmod	\N	gpu-firmware-kmod-g20200503
2020-09-16 09:04:41+02	2020-09-16 09:20:34+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200905_2
2020-09-16 09:20:35+02	2020-09-16 09:20:55+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-16 22:19:23+02	2020-09-16 22:19:31+02	misc	qtchooser	\N	qtchooser-66_4
2020-09-16 22:19:23+02	2020-09-16 22:19:33+02	x11	xcb-proto	\N	xcb-proto-1.14
2020-09-16 22:19:22+02	2020-09-16 22:19:35+02	devel	meson	\N	meson-0.55.3
2020-09-16 22:19:36+02	2020-09-16 22:19:44+02	devel	libmtdev	\N	libmtdev-1.1.5_3
2020-09-16 22:19:45+02	2020-09-16 22:20:18+02	x11	libinput	\N	libinput-1.15.6
2020-09-16 22:19:33+02	2020-09-16 22:20:29+02	x11	libxcb	\N	libxcb-1.14_1
2020-09-16 22:20:19+02	2020-09-16 22:20:33+02	devel	desktop-file-utils	\N	desktop-file-utils-0.26
2020-09-16 22:20:34+02	2020-09-16 22:21:12+02	x11	libxkbcommon	\N	libxkbcommon-1.0.1
2020-09-16 22:21:12+02	2020-09-16 22:21:28+02	x11	xcb-util	\N	xcb-util-0.4.0_2,1
2020-09-16 22:21:29+02	2020-09-16 22:21:44+02	x11	xcb-util-wm	\N	xcb-util-wm-0.4.1_3
2020-09-16 22:21:44+02	2020-09-16 22:22:01+02	x11	xcb-util-image	\N	xcb-util-image-0.4.0_1
2020-09-16 22:22:02+02	2020-09-16 22:22:16+02	x11	xcb-util-keysyms	\N	xcb-util-keysyms-0.4.0_1
2020-09-16 22:19:31+02	2020-09-16 22:22:28+02	devel	qt5-qmake	\N	qt5-qmake-5.15.0
2020-09-16 22:22:17+02	2020-09-16 22:22:28+02	x11	xcb-util-renderutil	\N	xcb-util-renderutil-0.3.9_1
2020-09-16 22:20:30+02	2020-09-16 22:22:30+02	x11	libX11	\N	libX11-1.6.12,1
2020-09-16 22:22:30+02	2020-09-16 22:22:45+02	x11	libXext	\N	libXext-1.3.4,1
2020-09-16 22:22:28+02	2020-09-16 22:22:51+02	devel	json-glib	\N	json-glib-1.6.0
2020-09-16 22:22:45+02	2020-09-16 22:22:58+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-09-16 22:22:52+02	2020-09-16 22:23:04+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-09-16 22:22:58+02	2020-09-16 22:23:14+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-09-16 22:23:04+02	2020-09-16 22:23:18+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-09-16 22:23:14+02	2020-09-16 22:23:27+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-09-16 22:23:19+02	2020-09-16 22:24:03+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-09-16 22:24:03+02	2020-09-16 22:25:22+02	devel	dbus	\N	dbus-1.12.20
2020-09-16 22:22:28+02	2020-09-16 22:25:42+02	devel	qt5-buildtools	\N	qt5-buildtools-5.15.0
2020-09-16 22:25:23+02	2020-09-16 22:25:50+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-09-16 22:25:43+02	2020-09-16 22:26:11+02	x11	libXi	\N	libXi-1.7.10,1
2020-09-16 22:25:50+02	2020-09-16 22:26:28+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-09-16 22:26:11+02	2020-09-16 22:26:35+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-09-16 22:26:36+02	2020-09-16 22:26:52+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-09-16 22:26:52+02	2020-09-16 22:27:18+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-09-16 22:27:19+02	2020-09-16 22:27:34+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-09-16 22:26:29+02	2020-09-16 22:27:43+02	net	avahi-app	\N	avahi-app-0.7_3
2020-09-16 22:27:35+02	2020-09-16 22:28:00+02	x11	libXpm	\N	libXpm-3.5.13
2020-09-16 22:28:01+02	2020-09-16 22:29:07+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-09-16 22:27:43+02	2020-09-16 22:29:19+02	print	cups	\N	cups-2.3.3_1
2020-09-16 22:23:27+02	2020-09-16 22:29:21+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_2
2020-09-16 22:29:08+02	2020-09-16 22:29:33+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-09-16 22:29:33+02	2020-09-16 22:30:44+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-09-16 22:30:44+02	2020-09-16 22:31:00+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-09-16 22:31:01+02	2020-09-16 22:31:20+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-09-16 22:31:21+02	2020-09-16 22:31:36+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-09-16 22:29:22+02	2020-09-16 22:31:59+02	graphics	cairo	\N	cairo-1.16.0,3
2020-09-16 22:31:36+02	2020-09-16 22:32:01+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-09-16 22:32:02+02	2020-09-16 22:32:16+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-09-16 22:29:20+02	2020-09-16 22:32:55+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-09-16 22:32:16+02	2020-09-16 22:33:04+02	sysutils	polkit	\N	polkit-0.116
2020-09-16 22:33:05+02	2020-09-16 22:33:42+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-09-16 22:32:56+02	2020-09-16 22:33:46+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-09-16 22:33:42+02	2020-09-16 22:33:53+02	x11	xprop	\N	xprop-1.2.4
2020-09-16 22:32:00+02	2020-09-16 22:33:56+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-09-16 22:33:46+02	2020-09-16 22:33:57+02	x11	xset	\N	xset-1.2.4_3
2020-09-16 22:33:57+02	2020-09-16 22:34:11+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-09-16 22:34:11+02	2020-09-16 22:37:25+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-09-16 22:33:56+02	2020-09-16 22:38:45+02	graphics	graphviz	\N	graphviz-2.44.1_2
2020-09-16 22:37:25+02	2020-09-16 22:39:58+02	graphics	colord	\N	colord-1.3.5
2020-09-16 22:39:58+02	2020-09-16 22:40:41+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-09-16 22:40:42+02	2020-09-16 22:42:27+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-09-16 22:38:45+02	2020-09-16 22:42:27+02	lang	vala	\N	vala-0.40.15,1
2020-09-16 22:42:27+02	2020-09-16 22:43:05+02	graphics	libGLU	\N	libGLU-9.0.1
2020-09-16 22:33:53+02	2020-09-16 22:43:23+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-09-16 22:42:28+02	2020-09-16 22:43:28+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-09-16 22:43:24+02	2020-09-16 22:45:30+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-09-16 22:43:29+02	2020-09-16 22:47:57+02	net	qt5-network	\N	qt5-network-5.15.0
2020-09-16 22:43:06+02	2020-09-16 22:51:10+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-16 22:51:11+02	2020-09-16 22:52:35+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-09-16 22:52:35+02	2020-09-16 22:54:07+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-09-16 22:54:08+02	2020-09-16 22:56:55+02	graphics	poppler	\N	poppler-20.09.0
2020-09-16 22:45:31+02	2020-09-16 22:57:07+02	x11-toolkits	gtk30	\N	gtk3-3.24.23
2020-09-16 22:56:56+02	2020-09-16 22:57:23+02	net	glib-networking	\N	glib-networking-2.56.1_3
2020-09-16 22:57:08+02	2020-09-16 22:57:56+02	devel	dconf	\N	dconf-0.38.0
2020-09-16 22:57:24+02	2020-09-16 22:58:57+02	devel	libsoup	\N	libsoup-2.62.3
2020-09-16 22:58:58+02	2020-09-16 23:00:06+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-09-16 22:57:57+02	2020-09-16 23:00:27+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-09-16 23:00:07+02	2020-09-16 23:00:43+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-09-16 23:00:28+02	2020-09-16 23:00:45+02	print	xpdfopen	\N	xpdfopen-0.86
2020-09-16 23:00:43+02	2020-09-16 23:02:45+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-09-16 23:02:46+02	2020-09-16 23:03:14+02	devel	libnotify	\N	libnotify-0.7.9
2020-09-16 23:03:14+02	2020-09-16 23:04:06+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-09-16 23:04:07+02	2020-09-16 23:04:32+02	multimedia	libva	\N	libva-2.9.0
2020-09-16 23:04:32+02	2020-09-16 23:04:49+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-09-16 22:47:58+02	2020-09-16 23:05:40+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-09-16 23:00:46+02	2020-09-16 23:07:43+02	print	texlive-base	\N	texlive-base-20150521_55
2020-09-16 23:04:49+02	2020-09-16 23:09:16+02	lang	php74	\N	php74-7.4.10
2020-09-16 23:09:16+02	2020-09-16 23:09:39+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-09-16 23:09:39+02	2020-09-16 23:10:58+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-16 23:10:59+02	2020-09-16 23:11:16+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-09-16 23:11:16+02	2020-09-16 23:11:52+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-16 23:11:52+02	2020-09-16 23:13:16+02	graphics	frei0r	\N	frei0r-1.6.1
2020-09-16 23:05:41+02	2020-09-16 23:14:36+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-09-16 23:07:48+02	2020-09-16 23:29:20+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-16 23:13:17+02	2020-09-16 23:31:16+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_2,1
2020-09-16 23:29:21+02	2020-09-16 23:32:13+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-16 23:31:17+02	2020-09-16 23:33:11+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-09-16 23:33:11+02	2020-09-16 23:35:06+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-16 23:35:06+02	2020-09-16 23:35:48+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-09-16 23:35:49+02	2020-09-16 23:37:14+02	graphics	graphene	\N	graphene-1.10.0
2020-09-16 23:37:15+02	2020-09-16 23:37:32+02	www	libwpe	\N	libwpe-1.4.0.1
2020-09-16 23:32:14+02	2020-09-16 23:38:32+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-16 23:38:33+02	2020-09-16 23:39:45+02	net	geoclue	\N	geoclue-2.5.5
2020-09-16 23:37:33+02	2020-09-16 23:39:48+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-16 23:39:46+02	2020-09-16 23:40:08+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-09-16 23:39:48+02	2020-09-16 23:40:23+02	graphics	jasper	\N	jasper-2.0.20
2020-09-16 23:14:37+02	2020-09-16 23:41:02+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-09-16 23:40:24+02	2020-09-16 23:41:05+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-09-16 23:41:02+02	2020-09-16 23:42:59+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-09-16 23:43:00+02	2020-09-16 23:44:07+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.74.0
2020-09-16 23:44:07+02	2020-09-16 23:44:41+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-09-16 23:44:42+02	2020-09-16 23:45:28+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-09-16 23:45:28+02	2020-09-16 23:46:09+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-09-16 23:46:09+02	2020-09-16 23:46:41+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-16 23:46:41+02	2020-09-16 23:48:33+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-09-16 23:48:34+02	2020-09-16 23:48:48+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-09-16 23:48:49+02	2020-09-16 23:49:48+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-09-16 23:49:49+02	2020-09-16 23:50:47+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-09-16 23:50:48+02	2020-09-16 23:51:01+02	x11	libXres	\N	libXres-1.2.0_2
2020-09-16 23:51:01+02	2020-09-16 23:51:16+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-09-16 23:41:05+02	2020-09-16 23:56:08+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-16 23:56:09+02	2020-09-16 23:57:42+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-09-16 23:51:17+02	2020-09-16 23:59:15+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-16 23:57:43+02	2020-09-17 00:03:00+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-09-17 00:03:01+02	2020-09-17 00:03:18+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-09-17 00:03:19+02	2020-09-17 00:04:14+02	devel	re2c	\N	re2c-0.14.3
2020-09-17 00:04:14+02	2020-09-17 00:04:51+02	www	php74-session	\N	php74-session-7.4.10
2020-09-17 00:04:52+02	2020-09-17 00:05:22+02	archivers	php74-zlib	\N	php74-zlib-7.4.10
2020-09-17 00:05:23+02	2020-09-17 00:05:55+02	databases	php74-pdo	\N	php74-pdo-7.4.10
2020-09-17 00:05:56+02	2020-09-17 00:06:27+02	devel	php74-json	\N	php74-json-7.4.10
2020-09-17 00:06:28+02	2020-09-17 00:07:36+02	sysutils	policykit	\N	policykit-0.9_10
2020-09-17 00:07:37+02	2020-09-17 00:08:32+02	converters	php74-mbstring	\N	php74-mbstring-7.4.10
2020-09-17 00:08:32+02	2020-09-17 00:10:51+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.74.0
2020-09-16 23:59:16+02	2020-09-17 00:14:34+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-09-17 00:14:35+02	2020-09-17 00:15:10+02	graphics	php74-gd	\N	php74-gd-7.4.10
2020-09-17 00:15:11+02	2020-09-17 00:15:43+02	security	php74-openssl	\N	php74-openssl-7.4.10
2020-09-17 00:10:52+02	2020-09-17 00:15:58+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-17 00:15:44+02	2020-09-17 00:16:07+02	textproc	php74-xml	\N	php74-xml-7.4.10
2020-09-17 00:16:08+02	2020-09-17 00:16:33+02	converters	php74-iconv	\N	php74-iconv-7.4.10
2020-09-17 00:15:59+02	2020-09-17 00:16:44+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-17 00:16:34+02	2020-09-17 00:16:58+02	databases	pecl-memcache	php74	php74-pecl-memcache-4.0.5.2
2020-09-17 00:16:44+02	2020-09-17 00:17:11+02	databases	pecl-memcached	php74	php74-pecl-memcached-3.1.5
2020-09-17 00:16:58+02	2020-09-17 00:17:23+02	databases	php74-pdo_pgsql	\N	php74-pdo_pgsql-7.4.10
2020-09-17 00:17:12+02	2020-09-17 00:17:37+02	databases	php74-pdo_sqlite	\N	php74-pdo_sqlite-7.4.10
2020-09-17 00:17:23+02	2020-09-17 00:17:51+02	databases	php74-pgsql	\N	php74-pgsql-7.4.10
2020-09-17 00:17:38+02	2020-09-17 00:18:02+02	databases	php74-sqlite3	\N	php74-sqlite3-7.4.10
2020-09-17 00:17:51+02	2020-09-17 00:18:16+02	devel	php74-gettext	\N	php74-gettext-7.4.10
2020-09-17 00:18:16+02	2020-09-17 00:19:34+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-09-17 00:19:34+02	2020-09-17 00:20:08+02	ftp	php74-curl	\N	php74-curl-7.4.10
2020-09-17 00:20:09+02	2020-09-17 00:21:15+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-09-17 00:21:16+02	2020-09-17 00:21:41+02	math	pecl-bitset	php74	php74-pecl-bitset-3.0.1
2020-09-17 00:21:41+02	2020-09-17 00:22:18+02	math	php74-bcmath	\N	php74-bcmath-7.4.10
2020-09-17 00:18:03+02	2020-09-17 00:24:54+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-17 00:22:18+02	2020-09-17 00:25:08+02	net	php74-ldap	\N	php74-ldap-7.4.10
2020-09-17 00:24:55+02	2020-09-17 00:25:25+02	net	php74-soap	\N	php74-soap-7.4.10
2020-09-17 00:25:09+02	2020-09-17 00:25:34+02	net	php74-sockets	\N	php74-sockets-7.4.10
2020-09-17 00:25:26+02	2020-09-17 00:25:48+02	security	pecl-mcrypt	php74	php74-pecl-mcrypt-1.0.3
2020-09-17 00:25:49+02	2020-09-17 00:26:14+02	sysutils	php74-posix	\N	php74-posix-7.4.10
2020-09-17 00:25:35+02	2020-09-17 00:26:34+02	sysutils	hal	\N	hal-0.5.14_34
2020-09-17 00:26:15+02	2020-09-17 00:26:40+02	textproc	php74-ctype	\N	php74-ctype-7.4.10
2020-09-17 00:26:40+02	2020-09-17 00:27:06+02	textproc	php74-simplexml	\N	php74-simplexml-7.4.10
2020-09-17 00:26:34+02	2020-09-17 00:27:07+02	textproc	php74-dom	\N	php74-dom-7.4.10
2020-09-17 00:27:06+02	2020-09-17 00:27:50+02	www	php74-opcache	\N	php74-opcache-7.4.10
2020-09-17 00:27:07+02	2020-09-17 00:28:32+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-17 00:27:51+02	2020-09-17 00:28:43+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-17 00:28:33+02	2020-09-17 00:29:02+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-09-17 00:29:02+02	2020-09-17 00:30:14+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-09-17 00:28:44+02	2020-09-17 00:30:37+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-09-17 00:30:14+02	2020-09-17 00:31:12+02	archivers	kf5-karchive	\N	kf5-karchive-5.74.0
2020-09-17 00:31:12+02	2020-09-17 00:31:54+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-09-17 00:30:37+02	2020-09-17 00:32:12+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-09-17 00:31:55+02	2020-09-17 00:33:08+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-09-17 00:33:08+02	2020-09-17 00:35:10+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-09-17 00:32:12+02	2020-09-17 00:36:33+02	x11-servers	xorg-server	\N	xorg-server-1.20.9,1
2020-09-17 00:35:10+02	2020-09-17 00:36:39+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-09-17 00:36:39+02	2020-09-17 00:39:34+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-09-17 00:39:34+02	2020-09-17 00:40:52+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-09-17 00:40:53+02	2020-09-17 00:42:48+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-09-17 00:42:49+02	2020-09-17 00:43:55+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-17 00:43:56+02	2020-09-17 00:47:12+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-09-21 07:08:22+02	2020-09-21 07:11:57+02	lang	php74	\N	php74-7.4.10
2020-09-17 00:36:34+02	2020-09-17 00:50:40+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-09-17 00:47:13+02	2020-09-17 00:50:41+02	security	gcr	\N	gcr-3.28.0_1
2020-09-17 00:50:42+02	2020-09-17 00:50:55+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-09-17 00:50:41+02	2020-09-17 00:51:31+02	security	libssh	\N	libssh-0.9.5
2020-09-17 00:50:56+02	2020-09-17 00:54:21+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-09-17 00:51:31+02	2020-09-17 00:54:35+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-09-17 00:54:21+02	2020-09-17 00:55:12+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-09-17 00:54:36+02	2020-09-17 00:55:44+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-09-17 00:55:12+02	2020-09-17 00:58:36+02	graphics	glew	\N	glew-2.2.0
2020-09-17 00:58:37+02	2020-09-17 00:59:30+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-09-17 00:59:30+02	2020-09-17 01:00:47+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-17 01:00:47+02	2020-09-17 01:02:07+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-17 01:02:08+02	2020-09-17 01:03:27+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.74.0
2020-09-17 00:55:45+02	2020-09-17 01:17:23+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-09-17 01:17:24+02	2020-09-17 01:20:16+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-09-17 01:20:17+02	2020-09-17 01:21:31+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-09-17 01:21:32+02	2020-09-17 01:23:33+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-09-17 01:23:34+02	2020-09-17 01:25:49+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-17 01:25:49+02	2020-09-17 01:27:00+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-09-17 01:03:27+02	2020-09-17 01:27:17+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-17 01:27:01+02	2020-09-17 01:27:23+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-09-17 01:27:24+02	2020-09-17 01:27:38+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-09-17 01:27:38+02	2020-09-17 01:27:50+02	x11	xrdb	\N	xrdb-1.2.0
2020-09-17 01:27:17+02	2020-09-17 01:28:28+02	net	remmina	\N	remmina-1.3.9
2020-09-17 01:27:50+02	2020-09-17 01:29:00+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-09-17 01:29:01+02	2020-09-17 01:29:46+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-09-17 01:29:46+02	2020-09-17 01:30:45+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-17 01:30:46+02	2020-09-17 01:40:19+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-09-17 01:28:29+02	2020-09-17 01:42:16+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-17 01:42:17+02	2020-09-17 01:42:33+02	x11	luit	\N	luit-20190106_1
2020-09-17 01:40:20+02	2020-09-17 01:42:56+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-17 01:42:57+02	2020-09-17 01:43:50+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-09-17 01:42:34+02	2020-09-17 01:44:28+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.74.0
2020-09-17 01:43:51+02	2020-09-17 01:45:19+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-09-17 01:44:29+02	2020-09-17 01:45:43+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-09-17 01:45:19+02	2020-09-17 01:46:32+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-17 01:46:33+02	2020-09-17 01:47:24+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-09-16 23:40:09+02	2020-09-17 01:47:53+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-17 01:45:44+02	2020-09-17 01:52:54+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-17 01:52:55+02	2020-09-17 01:53:35+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-17 01:53:36+02	2020-09-17 01:54:00+02	print	gl2ps	\N	gl2ps-1.4.0
2020-09-17 01:54:01+02	2020-09-17 01:54:52+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-09-17 01:54:53+02	2020-09-17 01:55:09+02	x11	xauth	\N	xauth-1.1
2020-09-17 01:55:10+02	2020-09-17 01:56:03+02	x11	xterm	\N	xterm-359
2020-09-17 01:56:04+02	2020-09-17 01:57:36+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-09-17 01:57:37+02	2020-09-17 02:00:07+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-09-17 02:00:07+02	2020-09-17 02:02:16+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-09-17 02:02:17+02	2020-09-17 02:02:38+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.9_6
2020-09-17 02:02:38+02	2020-09-17 02:03:23+02	devel	librest	\N	rest-0.7.93
2020-09-17 02:03:24+02	2020-09-17 02:06:10+02	graphics	netpbm	\N	netpbm-10.91.01
2020-09-17 02:06:11+02	2020-09-17 02:11:44+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-09-17 02:11:45+02	2020-09-17 02:16:14+02	net	libmateweather	\N	libmateweather-1.24.0
2020-09-17 02:16:14+02	2020-09-17 02:18:14+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-09-17 01:47:55+02	2020-09-17 02:19:06+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-17 02:19:07+02	2020-09-17 02:22:03+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-17 02:18:14+02	2020-09-17 02:28:09+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-09-17 02:28:10+02	2020-09-17 02:28:24+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-09-17 02:28:25+02	2020-09-17 02:29:46+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-17 02:29:47+02	2020-09-17 02:31:10+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-09-17 02:31:11+02	2020-09-17 02:31:43+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-09-17 02:31:44+02	2020-09-17 02:32:59+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-09-17 02:33:00+02	2020-09-17 02:33:51+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-17 02:33:52+02	2020-09-17 02:37:47+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-09-17 02:37:48+02	2020-09-17 02:40:26+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-09-17 02:40:27+02	2020-09-17 02:43:08+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-17 02:43:09+02	2020-09-17 02:44:17+02	net	uhttpmock	\N	uhttpmock-0.5.2
2020-09-17 02:44:17+02	2020-09-17 02:45:39+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-17 02:45:40+02	2020-09-17 02:46:53+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-17 02:46:54+02	2020-09-17 02:47:52+02	sysutils	upower	\N	upower-0.99.11
2020-09-17 01:47:25+02	2020-09-17 03:20:43+02	math	vtk8	\N	vtk8-8.2.0
2020-09-17 03:20:44+02	2020-09-17 05:44:41+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-17 05:44:43+02	2020-09-17 05:45:05+02	x11	appres	\N	appres-1.0.5
2020-09-17 05:45:05+02	2020-09-17 05:45:28+02	x11	bitmap	\N	bitmap-1.0.9
2020-09-17 05:45:29+02	2020-09-17 05:47:38+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-09-17 05:47:39+02	2020-09-17 05:48:24+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-09-17 05:48:24+02	2020-09-17 05:54:12+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-17 05:54:13+02	2020-09-17 05:54:32+02	x11	sessreg	\N	sessreg-1.1.2
2020-09-17 05:54:33+02	2020-09-17 05:54:48+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-09-17 05:54:49+02	2020-09-17 05:55:07+02	x11	smproxy	\N	smproxy-1.0.6
2020-09-17 05:55:08+02	2020-09-17 05:55:31+02	x11	x11perf	\N	x11perf-1.6.1
2020-09-17 05:55:31+02	2020-09-17 05:55:50+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-09-17 05:55:51+02	2020-09-17 05:56:23+02	x11	xcalc	\N	xcalc-1.1.0
2020-09-17 05:56:24+02	2020-09-17 05:56:43+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-09-17 05:56:43+02	2020-09-17 05:57:04+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-09-17 05:57:05+02	2020-09-17 05:57:27+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-09-17 05:57:28+02	2020-09-17 05:57:46+02	x11	xev	\N	xev-1.2.4
2020-09-17 05:57:47+02	2020-09-17 05:58:06+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-09-17 05:58:06+02	2020-09-17 05:58:22+02	x11	xgamma	\N	xgamma-1.0.6
2020-09-17 05:58:23+02	2020-09-17 05:58:48+02	x11	xgc	\N	xgc-1.0.5
2020-09-17 05:58:49+02	2020-09-17 05:59:08+02	x11	xhost	\N	xhost-1.0.8
2020-09-17 05:59:08+02	2020-09-17 05:59:29+02	x11	xinit	\N	xinit-1.4.1,1
2020-09-17 05:59:29+02	2020-09-17 05:59:50+02	x11	xinput	\N	xinput-1.6.3
2020-09-17 05:59:51+02	2020-09-17 06:00:12+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-09-17 06:00:13+02	2020-09-17 06:00:30+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-09-17 06:00:31+02	2020-09-17 06:00:49+02	x11	xkill	\N	xkill-1.0.5
2020-09-17 06:00:51+02	2020-09-17 06:01:12+02	x11	xlsatoms	\N	xlsatoms-1.1.3
2020-09-17 06:01:13+02	2020-09-17 06:01:33+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-09-17 06:01:33+02	2020-09-17 06:01:52+02	x11	xmessage	\N	xmessage-1.0.5
2020-09-17 06:01:52+02	2020-09-17 06:02:10+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-09-17 06:02:10+02	2020-09-17 06:02:28+02	x11	xpr	\N	xpr-1.0.5
2020-09-17 06:02:29+02	2020-09-17 06:02:48+02	x11	xrandr	\N	xrandr-1.5.1
2020-09-17 06:02:49+02	2020-09-17 06:03:06+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-09-17 02:22:03+02	2020-09-17 06:03:08+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-17 06:03:07+02	2020-09-17 06:03:20+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-09-17 06:03:10+02	2020-09-17 06:03:22+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-09-17 06:03:20+02	2020-09-17 06:03:33+02	x11	xwd	\N	xwd-1.0.7
2020-09-17 06:03:23+02	2020-09-17 06:03:36+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-09-17 06:03:34+02	2020-09-17 06:03:46+02	x11	xwud	\N	xwud-1.0.5
2020-09-17 06:03:36+02	2020-09-17 06:03:51+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-09-17 06:03:52+02	2020-09-17 06:04:53+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-09-17 06:03:47+02	2020-09-17 06:04:56+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-09-17 06:04:54+02	2020-09-17 06:05:49+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-09-17 06:04:56+02	2020-09-17 06:05:51+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-09-17 06:05:50+02	2020-09-17 06:06:49+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-09-17 06:06:50+02	2020-09-17 06:07:50+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-09-17 06:05:51+02	2020-09-17 06:08:04+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-09-17 06:07:51+02	2020-09-17 06:08:43+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.5.0
2020-09-17 06:08:05+02	2020-09-17 06:10:16+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-09-17 06:08:43+02	2020-09-17 06:10:24+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-09-17 06:10:17+02	2020-09-17 06:10:48+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-09-17 06:10:48+02	2020-09-17 06:11:06+02	x11-wm	twm	\N	twm-1.0.11
2020-09-17 06:11:07+02	2020-09-17 06:15:43+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-09-17 06:15:46+02	2020-09-17 06:20:31+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-09-17 06:20:33+02	2020-09-17 06:31:57+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-17 06:32:00+02	2020-09-17 06:37:45+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-09-17 06:10:25+02	2020-09-17 06:38:05+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-17 06:37:46+02	2020-09-17 07:19:51+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-09-17 07:19:55+02	2020-09-17 07:21:09+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-09-17 07:21:09+02	2020-09-17 07:21:51+02	print	libotf	\N	libotf-0.9.16
2020-09-17 07:21:52+02	2020-09-17 07:26:45+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-09-17 07:26:51+02	2020-09-17 07:29:20+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-17 07:29:20+02	2020-09-17 07:30:26+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-09-17 07:30:27+02	2020-09-17 07:31:45+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-09-17 07:31:46+02	2020-09-17 07:32:56+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-17 07:32:56+02	2020-09-17 07:35:48+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-09-17 06:38:07+02	2020-09-17 08:37:11+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-17 08:37:12+02	2020-09-17 08:37:33+02	www	p5-HTTP-Message	\N	p5-HTTP-Message-6.26
2020-09-17 08:37:34+02	2020-09-17 08:37:54+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-09-17 08:37:55+02	2020-09-17 08:39:22+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-17 08:39:23+02	2020-09-17 08:39:51+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-09-17 08:39:52+02	2020-09-17 08:40:05+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-09-17 08:40:06+02	2020-09-17 08:41:16+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-17 08:41:17+02	2020-09-17 08:45:32+02	x11-fm	caja	\N	caja-1.22.3_1
2020-09-17 08:45:33+02	2020-09-17 08:49:39+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.9,1
2020-09-17 08:49:40+02	2020-09-17 09:01:57+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-17 09:01:57+02	2020-09-17 09:04:49+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-09-17 09:04:49+02	2020-09-17 09:05:54+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-17 09:05:55+02	2020-09-17 09:07:23+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-17 09:07:23+02	2020-09-17 09:09:02+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-17 09:09:03+02	2020-09-17 09:12:34+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-17 09:12:35+02	2020-09-17 09:14:18+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-09-17 07:35:49+02	2020-09-17 09:16:47+02	www	node	\N	node-14.10.1
2020-09-17 09:14:19+02	2020-09-17 09:16:55+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-09-17 09:16:55+02	2020-09-17 09:17:06+02	devel	rubygem-mustache	\N	rubygem-mustache-1.1.1
2020-09-17 09:16:48+02	2020-09-17 09:18:42+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-09-17 09:18:42+02	2020-09-17 09:19:27+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-09-17 09:19:27+02	2020-09-17 09:19:55+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-09-17 09:19:55+02	2020-09-17 09:20:20+02	graphics	simage	\N	simage-1.7.0_7
2020-09-17 09:17:07+02	2020-09-17 09:20:23+02	graphics	gegl	\N	gegl-0.4.24_5
2020-09-17 09:20:24+02	2020-09-17 09:20:38+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-09-17 09:20:38+02	2020-09-17 09:23:38+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-17 09:23:39+02	2020-09-17 09:24:33+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-09-17 09:24:33+02	2020-09-17 09:26:53+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-09-17 09:26:53+02	2020-09-17 09:27:15+02	textproc	rubygem-rdiscount	\N	rubygem-rdiscount-2.2.0.1
2020-09-17 09:27:16+02	2020-09-17 09:28:13+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-09-17 09:28:14+02	2020-09-17 09:29:14+02	www	rubygem-hpricot	\N	rubygem-hpricot-0.8.6
2020-09-17 09:29:15+02	2020-09-17 09:30:16+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-09-17 09:20:21+02	2020-09-17 09:32:09+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-09-17 09:30:17+02	2020-09-17 09:33:12+02	x11	xorg	\N	xorg-7.7_3
2020-09-17 09:32:10+02	2020-09-17 09:33:15+02	x11	zenity	\N	zenity-3.32.0
2020-09-17 09:33:15+02	2020-09-17 09:33:53+02	devel	apache-ant	\N	apache-ant-1.10.8
2020-09-17 09:33:12+02	2020-09-17 09:34:25+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-09-17 09:33:53+02	2020-09-17 09:35:03+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-17 09:35:04+02	2020-09-17 09:35:41+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-09-17 09:35:41+02	2020-09-17 09:38:21+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-09-17 09:38:22+02	2020-09-17 09:41:25+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-09-17 09:41:26+02	2020-09-17 09:44:07+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-09-17 09:34:25+02	2020-09-17 09:46:16+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-09-17 09:44:07+02	2020-09-17 09:51:59+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-09-17 09:46:17+02	2020-09-17 09:53:16+02	editors	emacs	full	emacs-27.1,3
2020-09-17 09:53:17+02	2020-09-17 09:53:36+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-09-17 09:51:59+02	2020-09-17 10:02:54+02	graphics	Coin	\N	Coin-4.0.0_1
2020-09-17 10:02:55+02	2020-09-17 10:04:19+02	graphics	glfw	\N	glfw-3.3.2
2020-09-17 10:04:20+02	2020-09-17 10:07:52+02	graphics	movit	\N	movit-1.6.2_2
2020-09-17 09:53:37+02	2020-09-17 10:11:41+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-17 10:11:42+02	2020-09-17 10:12:51+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-09-17 10:12:52+02	2020-09-17 10:13:34+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-09-17 10:13:35+02	2020-09-17 10:14:04+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-09-17 10:14:04+02	2020-09-17 10:16:19+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-17 10:16:20+02	2020-09-17 10:22:48+02	math	py-matplotlib	py37	py37-matplotlib-2.2.4_2
2020-09-17 10:22:49+02	2020-09-17 10:23:21+02	net	gssdp	\N	gssdp-1.2.3
2020-09-17 10:23:22+02	2020-09-17 10:24:25+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-09-17 10:24:25+02	2020-09-17 10:26:33+02	print	cups-filters	\N	cups-filters-1.28.1_1
2020-09-17 10:26:34+02	2020-09-17 10:39:23+02	print	fontforge	\N	fontforge-20190801_2
2020-09-17 10:07:53+02	2020-09-17 10:44:05+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-17 10:44:06+02	2020-09-17 10:46:12+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-17 10:46:13+02	2020-09-17 10:46:24+02	textproc	rubygem-ronn	\N	rubygem-ronn-0.7.3
2020-09-17 10:39:24+02	2020-09-17 10:58:41+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-17 10:58:42+02	2020-09-17 11:00:27+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-17 11:00:28+02	2020-09-17 11:02:35+02	x11	yelp	\N	yelp-3.28.1_2
2020-09-17 11:02:35+02	2020-09-17 11:04:00+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-17 11:04:01+02	2020-09-17 11:05:06+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-17 11:05:07+02	2020-09-17 11:07:22+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-17 11:07:23+02	2020-09-17 11:19:45+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-09-17 11:19:45+02	2020-09-17 11:26:37+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-17 11:26:38+02	2020-09-17 11:33:08+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-17 11:33:09+02	2020-09-17 11:34:25+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-09-17 11:34:26+02	2020-09-17 11:39:19+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-09-17 11:39:20+02	2020-09-17 11:51:34+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-09-17 11:51:35+02	2020-09-17 12:06:49+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-17 12:06:50+02	2020-09-17 12:08:11+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-17 12:08:12+02	2020-09-17 12:09:25+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-09-17 12:09:26+02	2020-09-17 12:12:39+02	x11-wm	marco	\N	marco-1.22.4
2020-09-17 12:12:39+02	2020-09-17 12:13:48+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-17 12:13:49+02	2020-09-17 12:15:49+02	audio	libopenshot-audio	\N	libopenshot-audio-0.2.0.14
2020-09-17 12:15:50+02	2020-09-17 12:16:34+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-09-17 12:16:35+02	2020-09-17 12:19:11+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-17 12:19:12+02	2020-09-17 12:20:35+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-17 12:20:36+02	2020-09-17 12:25:38+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-17 12:25:39+02	2020-09-17 12:26:46+02	devel	git-lfs	\N	git-lfs-2.12.0
2020-09-17 12:26:47+02	2020-09-17 12:32:05+02	devel	ispc	\N	ispc-1.14.1_2
2020-09-17 12:32:06+02	2020-09-17 12:41:34+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-17 02:47:52+02	2020-09-17 12:42:11+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_2
2020-09-17 12:41:35+02	2020-09-17 12:43:10+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-09-17 12:43:11+02	2020-09-17 12:43:29+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-09-17 12:43:30+02	2020-09-17 12:45:34+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-17 12:45:34+02	2020-09-17 12:47:00+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-09-17 12:47:01+02	2020-09-17 12:47:36+02	graphics	libgxps	\N	libgxps-0.3.1
2020-09-17 12:47:37+02	2020-09-17 12:48:12+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-09-17 12:48:12+02	2020-09-17 12:54:40+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-09-17 12:54:41+02	2020-09-17 12:55:15+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-09-17 12:55:16+02	2020-09-17 12:56:21+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-17 12:56:22+02	2020-09-17 12:57:26+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-09-17 12:57:26+02	2020-09-17 13:19:47+02	math	R	\N	R-4.0.2
2020-09-17 13:19:49+02	2020-09-17 13:20:13+02	math	openlibm	\N	openlibm-0.6.0_2
2020-09-17 13:20:13+02	2020-09-17 13:29:14+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-17 13:29:14+02	2020-09-17 13:30:31+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-09-17 13:30:32+02	2020-09-17 13:33:14+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-17 13:33:15+02	2020-09-17 13:36:38+02	net	freerdp	\N	freerdp-2.2.0
2020-09-17 13:36:39+02	2020-09-17 13:37:16+02	net	gupnp	\N	gupnp-1.1.1_1
2020-09-17 13:37:16+02	2020-09-17 13:38:11+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-17 13:38:11+02	2020-09-17 13:39:09+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-09-17 13:39:10+02	2020-09-17 13:44:10+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-17 13:44:10+02	2020-09-17 13:44:45+02	print	libspectre	\N	libspectre-0.2.9
2020-09-17 13:44:45+02	2020-09-17 13:47:13+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-17 13:47:14+02	2020-09-17 13:55:23+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-17 13:55:24+02	2020-09-17 14:02:10+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-17 14:02:11+02	2020-09-17 14:02:32+02	sysutils	hal-info	\N	hal-info-20091130
2020-09-17 14:02:32+02	2020-09-17 14:07:07+02	sysutils	mate-control-center	\N	mate-control-center-1.24.0
2020-09-17 12:42:14+02	2020-09-17 14:07:41+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-17 14:07:08+02	2020-09-17 14:08:00+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-17 14:08:00+02	2020-09-17 14:08:09+02	sysutils	vpnc-scripts	\N	vpnc-scripts-20200226
2020-09-17 14:08:09+02	2020-09-17 14:10:10+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-17 14:10:11+02	2020-09-17 14:10:27+02	www	p5-HTTP-Cookies	\N	p5-HTTP-Cookies-6.08
2020-09-17 14:10:27+02	2020-09-17 14:10:43+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.12
2020-09-17 14:10:44+02	2020-09-17 14:11:00+02	www	p5-HTTP-Negotiate	\N	p5-HTTP-Negotiate-6.01_1
2020-09-17 14:07:42+02	2020-09-17 14:11:19+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-09-17 14:11:00+02	2020-09-17 14:11:45+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-17 14:11:46+02	2020-09-17 14:12:24+02	www	youtube_dl	\N	youtube_dl-2020.09.06
2020-09-17 14:11:19+02	2020-09-17 14:12:43+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-17 14:12:44+02	2020-09-17 14:12:52+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-09-17 14:12:53+02	2020-09-17 14:13:02+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-09-17 14:12:25+02	2020-09-17 14:13:47+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-17 14:13:48+02	2020-09-17 14:15:01+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-09-17 14:13:03+02	2020-09-17 14:15:05+02	x11-servers	xephyr	\N	xephyr-1.20.9,1
2020-09-17 14:15:02+02	2020-09-17 14:15:31+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-09-17 14:15:06+02	2020-09-17 14:15:54+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-17 14:15:32+02	2020-09-17 14:17:41+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-17 14:15:54+02	2020-09-17 14:17:55+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-09-17 14:17:42+02	2020-09-17 14:20:23+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-17 14:20:23+02	2020-09-17 14:23:28+02	archivers	engrampa	\N	engrampa-1.24.1
2020-09-17 14:23:29+02	2020-09-17 14:24:39+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-09-17 14:24:40+02	2020-09-17 14:25:03+02	audio	dssi	\N	dssi-1.1.1_8
2020-09-17 14:25:04+02	2020-09-17 14:25:35+02	audio	faudio	\N	FAudio-20.09
2020-09-17 14:25:35+02	2020-09-17 14:26:56+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-17 14:26:58+02	2020-09-17 14:28:39+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-09-17 14:28:40+02	2020-09-17 14:30:25+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-09-17 14:30:25+02	2020-09-17 14:35:06+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-17 14:35:07+02	2020-09-17 14:36:08+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-17 14:36:09+02	2020-09-17 14:37:10+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-17 14:37:11+02	2020-09-17 14:37:37+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-09-17 14:37:37+02	2020-09-17 14:38:02+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-09-17 14:38:02+02	2020-09-17 14:39:40+02	devel	gradle5	\N	gradle5-5.0
2020-09-17 14:39:41+02	2020-09-17 14:41:14+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-09-17 14:43:20+02	2020-09-17 14:43:53+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-09-17 14:43:54+02	2020-09-17 14:46:07+02	devel	py-game	py37	py37-game-1.9.6_1
2020-09-17 14:17:56+02	2020-09-17 14:47:41+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-17 14:46:08+02	2020-09-17 14:48:39+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-09-17 14:47:42+02	2020-09-17 14:49:29+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-17 10:46:25+02	2020-09-17 14:51:50+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-17 14:51:53+02	2020-09-17 14:52:48+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-09-17 14:52:48+02	2020-09-17 14:56:16+02	editors	pluma	\N	pluma-1.24.0
2020-09-17 14:56:16+02	2020-09-17 14:58:59+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-09-17 14:58:59+02	2020-09-17 15:02:31+02	french	med	\N	fr-med-4.0.0
2020-09-17 15:02:31+02	2020-09-17 15:05:47+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-09-17 15:05:47+02	2020-09-17 15:08:55+02	graphics	atril	\N	atril-1.22.3_9
2020-09-17 14:48:39+02	2020-09-17 15:09:23+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-17 15:08:56+02	2020-09-17 15:11:02+02	graphics	eom	\N	eom-1.24.0
2020-09-17 15:09:23+02	2020-09-17 15:11:04+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-17 15:11:04+02	2020-09-17 15:11:25+02	graphics	gltt	\N	gltt-2.5.2_14
2020-09-17 15:11:03+02	2020-09-17 15:11:27+02	graphics	gle	\N	gle-3.1.0_7
2020-09-17 15:11:26+02	2020-09-17 15:12:47+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-17 15:12:47+02	2020-09-17 15:16:09+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-09-17 15:16:10+02	2020-09-17 15:17:40+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-17 15:17:41+02	2020-09-17 15:25:19+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-09-17 15:25:20+02	2020-09-17 15:26:02+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-09-17 15:11:28+02	2020-09-17 15:26:59+02	graphics	oidn	\N	oidn-1.2.3
2020-09-17 15:26:02+02	2020-09-17 15:51:17+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-17 15:27:00+02	2020-09-17 15:53:43+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-09-17 15:51:18+02	2020-09-17 15:59:35+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-17 15:59:36+02	2020-09-17 16:09:05+02	lang	sbcl	\N	sbcl-2.0.8,1
2020-09-17 16:09:05+02	2020-09-17 16:11:27+02	math	mate-calc	\N	mate-calc-1.24.0
2020-09-17 16:11:28+02	2020-09-17 16:12:37+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-09-17 16:12:38+02	2020-09-17 16:12:46+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200912
2020-09-17 16:12:46+02	2020-09-17 16:20:24+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-09-17 16:20:25+02	2020-09-17 16:20:45+02	misc	xosd	\N	xosd-2.2.14_3
2020-09-17 16:20:46+02	2020-09-17 16:21:44+02	multimedia	gmmlib	\N	gmmlib-20.3.2
2020-09-17 16:21:44+02	2020-09-17 16:27:49+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-09-17 16:27:50+02	2020-09-17 16:28:51+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-09-17 16:28:52+02	2020-09-17 16:31:59+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-09-17 16:32:00+02	2020-09-17 16:45:05+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-17 16:45:05+02	2020-09-17 16:46:40+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-17 16:46:41+02	2020-09-17 16:47:29+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-17 16:47:30+02	2020-09-17 16:48:23+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-17 16:48:24+02	2020-09-17 16:49:12+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-17 16:49:12+02	2020-09-17 16:50:05+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-17 16:50:05+02	2020-09-17 16:50:50+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-17 16:50:51+02	2020-09-17 16:51:38+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-17 16:51:39+02	2020-09-17 16:52:38+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-17 16:52:39+02	2020-09-17 16:54:01+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-17 16:54:02+02	2020-09-17 16:54:52+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-09-17 16:54:53+02	2020-09-17 16:56:08+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-09-17 16:56:09+02	2020-09-17 16:57:23+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-17 15:53:44+02	2020-09-17 16:57:57+02	lang	julia11	\N	julia11-1.1.1_5
2020-09-17 16:57:23+02	2020-09-17 17:03:19+02	print	lilypond	\N	lilypond-2.20.0
2020-09-17 16:57:58+02	2020-09-17 17:05:01+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-17 17:05:02+02	2020-09-17 17:06:54+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-17 17:03:20+02	2020-09-17 17:09:08+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-17 17:06:55+02	2020-09-17 17:15:19+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-17 17:15:20+02	2020-09-17 17:15:49+02	print	transfig	\N	transfig-3.2.5e_4
2020-09-17 17:15:50+02	2020-09-17 17:16:15+02	security	openconnect	\N	openconnect-8.10
2020-09-17 17:16:16+02	2020-09-17 17:17:11+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-17 17:17:12+02	2020-09-17 17:19:11+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.1
2020-09-17 17:19:11+02	2020-09-17 17:21:35+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.0
2020-09-17 17:21:36+02	2020-09-17 17:23:19+02	textproc	fop	\N	fop-2.3
2020-09-17 17:23:19+02	2020-09-17 17:24:19+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-17 17:24:19+02	2020-09-17 17:24:42+02	textproc	lowdown	\N	lowdown-0.7.4
2020-09-17 17:24:42+02	2020-09-17 17:26:07+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-09-17 17:26:08+02	2020-09-17 17:26:58+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-17 17:26:59+02	2020-09-17 17:27:17+02	www	p5-libwww	\N	p5-libwww-6.47
2020-09-17 17:27:18+02	2020-09-17 17:28:49+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-17 17:09:09+02	2020-09-17 17:29:40+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-17 17:28:50+02	2020-09-17 17:30:15+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-17 17:30:16+02	2020-09-17 17:31:31+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-17 17:31:32+02	2020-09-17 17:33:06+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-17 17:33:07+02	2020-09-17 17:35:09+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-17 17:29:41+02	2020-09-17 17:35:35+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-17 17:35:36+02	2020-09-17 17:36:43+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-09-17 17:36:44+02	2020-09-17 17:38:03+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-17 17:35:10+02	2020-09-17 17:38:09+02	x11-fm	nautilus	\N	nautilus-3.28.1_4
2020-09-17 17:38:04+02	2020-09-17 17:38:40+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-17 17:38:10+02	2020-09-17 17:47:22+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-17 17:47:23+02	2020-09-17 17:47:34+02	astro	xearth	\N	xearth-1.2
2020-09-17 17:47:34+02	2020-09-17 17:49:47+02	astro	xephem	\N	xephem-3.7.7_5
2020-09-17 17:49:48+02	2020-09-17 17:53:45+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-17 17:53:46+02	2020-09-17 17:54:03+02	audio	xmix	\N	xmix-2.1_3
2020-09-17 17:38:41+02	2020-09-17 17:55:21+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-17 17:55:22+02	2020-09-17 18:01:04+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-17 18:01:05+02	2020-09-17 18:03:47+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-17 17:54:03+02	2020-09-17 18:13:42+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-17 18:13:43+02	2020-09-17 18:15:34+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-17 18:15:35+02	2020-09-17 18:17:12+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-17 18:17:12+02	2020-09-17 18:17:24+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-09-17 18:17:25+02	2020-09-17 18:32:11+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-17 18:32:12+02	2020-09-17 18:35:08+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-17 18:35:09+02	2020-09-17 18:36:10+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-17 18:36:11+02	2020-09-17 18:36:23+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-09-17 18:36:24+02	2020-09-17 18:36:41+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-09-17 18:36:42+02	2020-09-17 19:01:37+02	devel	RStudio	\N	RStudio-1.3.1073
2020-09-17 14:49:30+02	2020-09-17 19:11:53+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-09-17 19:11:54+02	2020-09-17 19:12:54+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-17 19:01:38+02	2020-09-17 19:25:10+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-17 18:03:48+02	2020-09-17 19:33:04+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-17 19:33:05+02	2020-09-17 19:33:47+02	devel	tortoisehg	py37	tortoisehg-5.5.1
2020-09-17 19:33:47+02	2020-09-17 19:53:47+02	editors	codelite	\N	codelite-12.0_4
2020-09-17 19:53:48+02	2020-09-17 19:55:52+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-17 19:55:53+02	2020-09-17 19:59:26+02	editors	vim	\N	vim-8.2.1558
2020-09-17 19:25:11+02	2020-09-17 20:19:51+02	devel	ghidra	\N	ghidra-9.1
2020-09-17 19:59:27+02	2020-09-17 20:43:47+02	emulators	qemu	\N	qemu-4.2.1
2020-09-17 20:43:48+02	2020-09-17 21:38:42+02	emulators	wine	\N	wine-5.0.2,1
2020-09-17 21:38:44+02	2020-09-17 21:39:39+02	emulators	x48	\N	x48-0.6.4_1
2020-09-17 21:39:39+02	2020-09-17 21:57:27+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-17 21:57:28+02	2020-09-17 21:57:46+02	games	KnightCap	\N	knightcap-3.6_8
2020-09-17 21:57:46+02	2020-09-17 21:57:55+02	games	affenspiel	\N	affenspiel-1.0_2
2020-09-17 21:57:55+02	2020-09-17 21:58:11+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-09-17 21:58:12+02	2020-09-17 22:00:18+02	games	alienarena	\N	alienarena-2013.766_9
2020-09-17 22:00:18+02	2020-09-17 22:00:48+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-09-17 20:19:53+02	2020-09-17 22:02:04+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-09-17 22:02:05+02	2020-09-17 22:02:26+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-09-17 22:00:48+02	2020-09-17 22:02:56+02	games	battletanks	\N	btanks-0.9.8083_9
2020-09-17 22:02:26+02	2020-09-17 22:03:02+02	games	monsterz	\N	monsterz-0.7.1_15
2020-09-17 22:02:56+02	2020-09-17 22:03:29+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-09-17 22:03:03+02	2020-09-17 22:03:39+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-09-17 22:03:29+02	2020-09-17 22:04:18+02	games	xblast	\N	xblast-2.10.4_18
2020-09-17 22:03:40+02	2020-09-17 22:04:34+02	games	xboard	\N	xboard-4.9.1_1
2020-09-17 22:04:18+02	2020-09-17 22:05:04+02	games	xcowsay	\N	xcowsay-1.5
2020-09-17 22:05:05+02	2020-09-17 22:11:05+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-17 22:11:06+02	2020-09-17 22:12:34+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-17 22:04:35+02	2020-09-17 22:33:12+02	graphics	blender	\N	blender-2.90.0
2020-09-17 22:12:35+02	2020-09-17 23:15:17+02	graphics	inkscape	\N	inkscape-1.0_5
2020-09-17 23:15:18+02	2020-09-17 23:20:00+02	graphics	wings	\N	wings-2.2.5
2020-09-17 23:20:04+02	2020-09-17 23:22:13+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-17 23:22:14+02	2020-09-17 23:58:28+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-09-17 23:58:30+02	2020-09-17 23:59:50+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-17 22:33:14+02	2020-09-18 00:27:12+02	graphics	krita	\N	krita-4.3.0_5
2020-09-17 23:59:51+02	2020-09-18 00:33:31+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-17 19:12:55+02	2020-09-18 01:01:57+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-18 01:02:00+02	2020-09-18 01:02:38+02	lang	julia	\N	julia-1.0.3_3
2020-09-18 00:27:14+02	2020-09-18 01:04:41+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.4
2020-09-18 01:04:42+02	2020-09-18 01:04:55+02	mail	bbmail	\N	bbmail-0.9.3
2020-09-18 01:04:56+02	2020-09-18 01:11:08+02	math	maxima	\N	maxima-5.44.0_4
2020-09-18 01:11:09+02	2020-09-18 01:12:28+02	multimedia	celluloid	\N	celluloid-0.19
2020-09-18 01:12:29+02	2020-09-18 01:12:48+02	multimedia	libva-glx	\N	libva-glx-2.9.0
2020-09-18 01:12:49+02	2020-09-18 01:13:47+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-09-18 01:13:48+02	2020-09-18 01:41:35+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p8
2020-09-18 01:41:37+02	2020-09-18 01:42:20+02	multimedia	libva-utils	\N	libva-utils-2.9.0
2020-09-18 01:42:21+02	2020-09-18 01:42:37+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-09-18 01:42:38+02	2020-09-18 01:47:50+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-09-18 01:02:39+02	2020-09-18 01:47:53+02	lang	mono	\N	mono-5.10.1.57_3
2020-09-18 01:47:50+02	2020-09-18 01:54:24+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-09-18 01:47:54+02	2020-09-18 01:54:47+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-18 01:54:48+02	2020-09-18 01:55:32+02	net	rdesktop	\N	rdesktop-1.9.0
2020-09-18 01:55:32+02	2020-09-18 01:57:11+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-18 01:54:27+02	2020-09-18 01:58:14+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-09-18 01:58:15+02	2020-09-18 01:59:25+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-18 01:59:25+02	2020-09-18 02:00:31+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-09-18 02:00:31+02	2020-09-18 02:01:33+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-09-18 02:01:33+02	2020-09-18 02:02:31+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-09-18 02:02:32+02	2020-09-18 02:03:28+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-09-18 02:03:29+02	2020-09-18 02:04:27+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-09-18 02:04:28+02	2020-09-18 02:05:26+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-09-18 02:05:29+02	2020-09-18 02:06:37+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-09-18 02:06:38+02	2020-09-18 02:08:12+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-09-18 02:08:13+02	2020-09-18 02:09:16+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-09-18 02:09:16+02	2020-09-18 02:10:14+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-09-18 02:10:15+02	2020-09-18 02:12:08+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-09-18 02:12:09+02	2020-09-18 02:13:03+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-09-18 01:57:12+02	2020-09-18 02:13:20+02	net	wireshark	\N	wireshark-3.2.6
2020-09-18 02:13:04+02	2020-09-18 02:13:21+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-09-18 02:13:22+02	2020-09-18 02:14:01+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-09-18 02:14:02+02	2020-09-18 02:14:43+02	print	gv	\N	gv-3.7.4_5
2020-09-18 02:13:21+02	2020-09-18 02:15:57+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-18 02:14:44+02	2020-09-18 02:29:17+02	print	lyx	\N	lyx-2.3.4.2_1
2020-09-18 02:29:18+02	2020-09-18 02:35:02+02	print	texlive-full	\N	texlive-full-20150521
2020-09-18 02:35:03+02	2020-09-18 02:39:32+02	security	clamav	\N	clamav-0.103.0,1
2020-09-18 02:39:33+02	2020-09-18 02:40:54+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-09-18 02:40:55+02	2020-09-18 02:48:39+02	security	tor	\N	tor-0.4.4.5
2020-09-18 02:48:40+02	2020-09-18 02:48:57+02	sysutils	bpytop	\N	bpytop-1.0.33
2020-09-18 02:48:58+02	2020-09-18 02:50:14+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-18 02:50:15+02	2020-09-18 02:51:19+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-18 02:51:20+02	2020-09-18 02:52:59+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-18 02:15:58+02	2020-09-18 02:55:10+02	print	scribus-devel	\N	scribus-devel-1.5.5_18
2020-09-18 02:52:59+02	2020-09-18 02:55:23+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-18 02:55:11+02	2020-09-18 02:55:46+02	sysutils	htop	\N	htop-3.0.2
2020-09-18 02:55:47+02	2020-09-18 02:56:11+02	www	dokuwiki	\N	dokuwiki-20200729
2020-09-18 02:55:23+02	2020-09-18 02:57:55+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-09-18 02:57:56+02	2020-09-18 02:58:55+02	www	links	\N	links-2.20.2,1
2020-09-18 02:58:56+02	2020-09-18 03:05:18+02	www	mod_php74	\N	mod_php74-7.4.10
2020-09-18 03:05:19+02	2020-09-18 03:31:16+02	x11	alacritty	\N	alacritty-0.5.0_2
2020-09-18 03:31:17+02	2020-09-18 03:33:33+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-09-18 03:33:33+02	2020-09-18 03:36:04+02	x11	mate	\N	mate-1.22.2
2020-09-18 03:36:05+02	2020-09-18 03:36:21+02	x11	numlockx	\N	numlockx-1.2
2020-09-18 03:36:22+02	2020-09-18 03:36:34+02	x11	sterm	\N	sterm-0.8.4
2020-09-18 03:36:35+02	2020-09-18 03:37:12+02	x11	xclip	\N	xclip-0.13
2020-09-18 03:37:12+02	2020-09-18 03:37:33+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-09-18 03:37:34+02	2020-09-18 03:38:07+02	x11	xdm	\N	xdm-1.1.12_3
2020-09-18 03:38:13+02	2020-09-18 03:39:58+02	x11	xlockmore	\N	xlockmore-5.65
2020-09-18 03:39:59+02	2020-09-18 03:40:22+02	x11	xman	\N	xman-1.1.5
2020-09-18 03:40:23+02	2020-09-18 03:47:40+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-18 03:47:41+02	2020-09-18 03:54:18+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-18 03:54:19+02	2020-09-18 03:56:04+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-18 03:56:04+02	2020-09-18 03:56:21+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-09-18 03:56:21+02	2020-09-18 03:56:32+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-09-18 03:56:33+02	2020-09-18 03:57:18+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-09-18 03:57:18+02	2020-09-18 03:57:37+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-09-18 03:57:37+02	2020-09-18 03:58:08+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-09-18 00:33:32+02	2020-09-18 03:58:12+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200913
2020-09-18 03:58:09+02	2020-09-18 03:58:25+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-09-18 03:58:25+02	2020-09-18 03:59:25+02	graphics	xfig	\N	xfig-3.2.7a
2020-09-18 03:59:25+02	2020-09-18 04:00:00+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-09-18 03:58:13+02	2020-09-18 04:23:06+02	editors	vscode	\N	vscode-1.46.1
2020-09-18 02:56:11+02	2020-09-18 04:40:43+02	www	firefox	\N	firefox-80.0.1_1,2
2020-09-18 04:40:45+02	2020-09-18 04:56:49+02	local	E590T-localbase	\N	E590T-localbase-2020091600
2020-09-18 04:57:03+02	2020-09-18 04:57:39+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-18 04:57:03+02	2020-09-18 04:59:09+02	graphics	gpu-firmware-kmod	\N	gpu-firmware-kmod-g20200914
2020-09-18 04:59:10+02	2020-09-18 05:05:17+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-18 05:05:18+02	2020-09-18 05:05:25+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-19 13:27:55+02	2020-09-19 13:29:37+02	devel	nspr	\N	nspr-4.29
2020-09-19 13:27:56+02	2020-09-19 13:30:20+02	devel	nasm	\N	nasm-2.15.05,1
2020-09-19 13:27:56+02	2020-09-19 13:31:39+02	graphics	graphene	\N	graphene-1.10.2
2020-09-19 13:30:21+02	2020-09-19 13:36:39+02	security	nss	\N	nss-3.56
2020-09-19 13:36:40+02	2020-09-19 13:40:02+02	graphics	poppler	\N	poppler-20.09.0
2020-09-19 13:31:40+02	2020-09-19 13:41:05+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-19 13:41:08+02	2020-09-19 13:44:00+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-09-19 13:44:05+02	2020-09-19 13:45:42+02	textproc	yelp-xsl	\N	yelp-xsl-3.38.0
2020-09-19 13:45:45+02	2020-09-19 13:51:19+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-09-19 13:29:40+02	2020-09-19 14:06:30+02	lang	spidermonkey60	\N	spidermonkey60-60.9.0_4
2020-09-19 14:06:34+02	2020-09-19 14:08:38+02	sysutils	polkit	\N	polkit-0.116
2020-09-19 13:40:07+02	2020-09-19 14:10:14+02	print	texlive-base	\N	texlive-base-20150521_55
2020-09-19 14:08:39+02	2020-09-19 14:10:51+02	graphics	colord	\N	colord-1.3.5
2020-09-19 14:10:51+02	2020-09-19 14:20:21+02	x11-toolkits	gtk30	\N	gtk3-3.24.23
2020-09-19 14:20:23+02	2020-09-19 14:24:02+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-19 13:51:22+02	2020-09-19 14:36:29+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_3
2020-09-19 14:36:32+02	2020-09-19 14:38:48+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-09-19 14:38:49+02	2020-09-19 14:42:24+02	sysutils	hal	\N	hal-0.5.14_34
2020-09-19 14:10:14+02	2020-09-19 14:45:34+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-19 14:42:25+02	2020-09-19 14:46:50+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-19 14:45:36+02	2020-09-19 14:51:20+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-19 14:51:22+02	2020-09-19 14:53:25+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-19 14:24:03+02	2020-09-19 14:56:19+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-19 14:53:28+02	2020-09-19 14:57:22+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-09-19 14:46:52+02	2020-09-19 15:00:22+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-19 14:57:25+02	2020-09-19 15:07:00+02	security	gcr	\N	gcr-3.28.0_1
2020-09-19 14:56:20+02	2020-09-19 15:12:47+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-19 15:00:24+02	2020-09-19 15:13:31+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-19 15:13:34+02	2020-09-19 15:15:23+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-19 15:12:50+02	2020-09-19 15:16:09+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-19 15:15:26+02	2020-09-19 15:17:24+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-09-19 15:07:08+02	2020-09-19 15:17:34+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-09-19 15:16:10+02	2020-09-19 15:18:27+02	net	remmina	\N	remmina-1.3.9
2020-09-19 15:17:25+02	2020-09-19 15:20:02+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-09-19 15:17:35+02	2020-09-19 15:20:59+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-19 15:20:04+02	2020-09-19 15:22:50+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-09-19 15:18:28+02	2020-09-19 15:23:30+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-19 15:21:00+02	2020-09-19 15:24:40+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-09-19 15:23:32+02	2020-09-19 15:25:56+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-19 15:25:57+02	2020-09-19 15:28:42+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-19 15:24:41+02	2020-09-19 15:29:43+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-19 15:28:45+02	2020-09-19 15:30:51+02	sysutils	upower	\N	upower-0.99.11
2020-09-19 15:22:54+02	2020-09-19 15:31:39+02	net	libmateweather	\N	libmateweather-1.24.0
2020-09-19 15:29:44+02	2020-09-19 15:32:21+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-09-19 15:30:55+02	2020-09-19 15:33:21+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-09-19 15:33:22+02	2020-09-19 15:39:14+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-09-19 15:31:40+02	2020-09-19 15:42:34+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-19 15:32:23+02	2020-09-19 15:42:59+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-19 15:39:18+02	2020-09-19 15:43:24+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-09-19 15:43:25+02	2020-09-19 15:44:44+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-09-19 15:44:45+02	2020-09-19 15:46:36+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-19 15:42:37+02	2020-09-19 15:47:31+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-19 15:46:37+02	2020-09-19 15:49:19+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-19 15:49:20+02	2020-09-19 15:52:03+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-19 15:52:04+02	2020-09-19 15:53:37+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-19 15:53:38+02	2020-09-19 15:56:52+02	x11-fm	caja	\N	caja-1.22.3_1
2020-09-19 15:56:53+02	2020-09-19 16:01:58+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-19 16:02:02+02	2020-09-19 16:07:22+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-09-19 16:07:24+02	2020-09-19 16:11:32+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-19 16:11:34+02	2020-09-19 16:20:27+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-19 16:20:30+02	2020-09-19 16:23:38+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-19 15:43:05+02	2020-09-19 16:31:45+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-19 16:23:42+02	2020-09-19 16:33:39+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-19 16:31:49+02	2020-09-19 16:35:21+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-09-19 16:33:41+02	2020-09-19 16:43:23+02	graphics	gegl	\N	gegl-0.4.26_5
2020-09-19 16:43:24+02	2020-09-19 16:46:05+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-09-19 16:35:23+02	2020-09-19 16:47:47+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-19 16:47:51+02	2020-09-19 16:48:49+02	textproc	yelp-tools	\N	yelp-tools-3.38.0
2020-09-19 16:46:07+02	2020-09-19 16:51:02+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-09-19 16:48:55+02	2020-09-19 16:55:08+02	x11	xorg	\N	xorg-7.7_3
2020-09-19 16:51:04+02	2020-09-19 16:55:43+02	x11	zenity	\N	zenity-3.32.0
2020-09-19 16:55:45+02	2020-09-19 17:02:12+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-19 16:55:10+02	2020-09-19 17:07:56+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-09-19 17:02:15+02	2020-09-19 17:18:51+02	editors	emacs	full	emacs-27.1,3
2020-09-19 17:18:52+02	2020-09-19 17:20:08+02	print	cups-filters	\N	cups-filters-1.28.1_1
2020-09-19 17:20:09+02	2020-09-19 17:25:05+02	print	fontforge	\N	fontforge-20190801_2
2020-09-19 17:08:02+02	2020-09-19 17:35:21+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-19 17:25:07+02	2020-09-19 17:36:44+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-19 17:35:23+02	2020-09-19 17:37:18+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-19 17:37:18+02	2020-09-19 17:38:49+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-19 17:38:52+02	2020-09-19 17:41:29+02	x11	yelp	\N	yelp-3.38.0
2020-09-19 17:41:31+02	2020-09-19 17:43:23+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-19 17:43:24+02	2020-09-19 17:47:33+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-19 17:47:33+02	2020-09-19 17:49:13+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-19 17:49:14+02	2020-09-19 17:51:15+02	x11-wm	marco	\N	marco-1.22.4
2020-09-19 17:51:16+02	2020-09-19 17:52:30+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-19 17:52:31+02	2020-09-19 17:55:07+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-19 17:55:08+02	2020-09-19 17:56:32+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-19 17:56:33+02	2020-09-19 17:59:59+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-19 17:59:59+02	2020-09-19 18:15:24+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-19 15:47:33+02	2020-09-19 18:15:50+02	www	node	\N	node-14.11.0
2020-09-19 18:15:25+02	2020-09-19 18:17:00+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-09-19 18:17:03+02	2020-09-19 18:27:22+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-19 18:27:26+02	2020-09-19 18:28:47+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-19 18:28:48+02	2020-09-19 18:34:48+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-19 18:34:49+02	2020-09-19 18:38:36+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-19 18:38:37+02	2020-09-19 18:49:38+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-19 18:49:39+02	2020-09-19 19:00:24+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-19 19:00:25+02	2020-09-19 19:02:24+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-09-19 19:02:27+02	2020-09-19 19:02:58+02	sysutils	hal-info	\N	hal-info-20091130
2020-09-19 19:02:59+02	2020-09-19 19:10:11+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-09-19 19:10:14+02	2020-09-19 19:11:47+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-19 17:36:45+02	2020-09-19 19:12:04+02	www	node12	\N	node12-12.18.4
2020-09-19 19:12:05+02	2020-09-19 19:13:27+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-19 19:11:47+02	2020-09-19 19:13:57+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-19 19:13:28+02	2020-09-19 19:14:36+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-19 18:15:51+02	2020-09-19 19:16:30+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-19 19:14:37+02	2020-09-19 19:17:24+02	archivers	engrampa	\N	engrampa-1.24.1
2020-09-19 19:16:36+02	2020-09-19 19:18:18+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-09-19 19:17:24+02	2020-09-19 19:18:45+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-19 19:18:19+02	2020-09-19 19:19:39+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-09-19 19:13:59+02	2020-09-19 19:19:42+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-19 19:19:39+02	2020-09-19 19:20:35+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-19 19:19:42+02	2020-09-19 19:21:41+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-19 19:18:46+02	2020-09-19 19:21:48+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-19 19:21:48+02	2020-09-19 19:24:06+02	graphics	atril	\N	atril-1.22.3_9
2020-09-19 19:21:41+02	2020-09-19 19:25:51+02	editors	pluma	\N	pluma-1.24.1
2020-09-19 19:24:07+02	2020-09-19 19:26:26+02	graphics	eom	\N	eom-1.24.1
2020-09-19 19:25:52+02	2020-09-19 19:27:07+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-19 19:26:27+02	2020-09-19 19:28:46+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-19 19:28:48+02	2020-09-19 19:41:23+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-19 19:27:08+02	2020-09-19 19:43:27+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-19 19:43:28+02	2020-09-19 19:43:34+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200918
2020-09-19 19:41:24+02	2020-09-19 19:43:42+02	math	mate-calc	\N	mate-calc-1.24.1
2020-09-19 19:43:35+02	2020-09-19 19:44:17+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-19 19:43:43+02	2020-09-19 19:44:28+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-19 19:44:17+02	2020-09-19 19:44:59+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-19 19:20:37+02	2020-09-19 19:45:02+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-09-19 19:44:28+02	2020-09-19 19:45:13+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-19 19:44:59+02	2020-09-19 19:45:38+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-19 19:45:03+02	2020-09-19 19:45:42+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-19 19:45:13+02	2020-09-19 19:46:01+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-19 19:45:38+02	2020-09-19 19:46:47+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-19 19:45:43+02	2020-09-19 19:46:51+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-09-19 19:46:02+02	2020-09-19 19:47:05+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-19 19:46:47+02	2020-09-19 19:52:06+02	print	lilypond	\N	lilypond-2.20.0
2020-09-19 19:47:06+02	2020-09-19 19:52:46+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-19 19:46:52+02	2020-09-19 19:53:18+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-19 19:52:07+02	2020-09-19 19:53:44+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-19 19:53:44+02	2020-09-19 19:54:27+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-19 19:54:27+02	2020-09-19 19:55:57+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-09-19 19:55:57+02	2020-09-19 19:57:33+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-09-19 19:57:34+02	2020-09-19 19:58:22+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-19 19:58:23+02	2020-09-19 19:58:46+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-09-19 19:52:46+02	2020-09-19 19:59:24+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-19 19:59:25+02	2020-09-19 19:59:37+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-09-19 19:58:46+02	2020-09-19 19:59:40+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-19 19:59:41+02	2020-09-19 20:00:41+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-19 20:00:42+02	2020-09-19 20:01:59+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-19 20:02:00+02	2020-09-19 20:03:45+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-19 19:59:38+02	2020-09-19 20:04:29+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-19 20:03:45+02	2020-09-19 20:05:11+02	x11-fm	nautilus	\N	nautilus-3.28.1_4
2020-09-19 20:04:30+02	2020-09-19 20:05:22+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-19 20:05:12+02	2020-09-19 20:08:50+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-19 19:53:19+02	2020-09-19 20:14:12+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-19 20:08:51+02	2020-09-19 20:15:05+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-19 20:14:17+02	2020-09-19 20:17:01+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-19 20:17:02+02	2020-09-19 20:19:16+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-19 20:19:16+02	2020-09-19 20:20:50+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-19 20:05:23+02	2020-09-19 20:23:44+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-19 20:20:51+02	2020-09-19 20:24:44+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-19 20:24:45+02	2020-09-19 20:25:30+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-19 20:23:46+02	2020-09-19 20:26:00+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-19 20:26:01+02	2020-09-19 20:39:14+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-19 20:39:15+02	2020-09-19 20:40:42+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-19 20:25:30+02	2020-09-19 20:53:30+02	devel	RStudio	\N	RStudio-1.3.1073
2020-09-19 20:53:32+02	2020-09-19 21:06:28+02	editors	codelite	\N	codelite-12.0_4
2020-09-19 21:06:29+02	2020-09-19 21:09:08+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-19 21:09:09+02	2020-09-19 21:11:34+02	editors	vim	\N	vim-8.2.1558
2020-09-19 21:11:35+02	2020-09-19 21:36:00+02	emulators	qemu	\N	qemu-4.2.1
2020-09-19 21:36:04+02	2020-09-19 21:56:11+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-19 21:56:11+02	2020-09-19 21:57:10+02	games	xcowsay	\N	xcowsay-1.5
2020-09-19 21:57:10+02	2020-09-19 21:58:50+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-19 21:58:50+02	2020-09-19 22:04:19+02	graphics	inkscape	\N	inkscape-1.0_5
2020-09-19 22:04:20+02	2020-09-19 22:19:31+02	graphics	krita	\N	krita-4.3.0_5
2020-09-19 22:19:32+02	2020-09-19 22:24:23+02	graphics	wings	\N	wings-2.2.5
2020-09-19 20:15:06+02	2020-09-19 22:25:08+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-19 22:24:23+02	2020-09-19 22:25:44+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-19 22:25:11+02	2020-09-19 22:54:55+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-19 22:54:56+02	2020-09-19 23:02:11+02	math	maxima	\N	maxima-5.44.0_4
2020-09-19 23:02:12+02	2020-09-19 23:03:18+02	multimedia	celluloid	\N	celluloid-0.19
2020-09-19 22:25:44+02	2020-09-19 23:04:04+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.4
2020-09-19 23:04:05+02	2020-09-19 23:06:07+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-19 23:03:19+02	2020-09-19 23:06:08+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-09-19 23:06:08+02	2020-09-19 23:06:59+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-19 23:07:00+02	2020-09-19 23:07:18+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-09-19 23:06:09+02	2020-09-19 23:07:48+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-09-19 23:07:19+02	2020-09-19 23:11:07+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-19 23:11:10+02	2020-09-19 23:16:58+02	print	texlive-full	\N	texlive-full-20150521
2020-09-19 23:07:49+02	2020-09-19 23:17:19+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-09-19 23:16:59+02	2020-09-19 23:17:51+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-19 23:17:19+02	2020-09-19 23:18:27+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-19 23:17:51+02	2020-09-19 23:19:00+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-19 20:40:43+02	2020-09-19 23:19:06+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-19 23:19:01+02	2020-09-19 23:20:13+02	x11	mate	\N	mate-1.22.2
2020-09-19 23:19:08+02	2020-09-19 23:45:45+02	editors	vscode	\N	vscode-1.46.1
2020-09-19 23:18:28+02	2020-09-20 01:16:21+02	www	firefox	\N	firefox-81.0,2
2020-09-20 01:16:23+02	2020-09-20 01:33:50+02	local	E590T-localbase	\N	E590T-localbase-2020091900
2020-09-20 01:34:04+02	2020-09-20 01:35:10+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-20 01:34:04+02	2020-09-20 01:40:57+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-20 01:40:57+02	2020-09-20 01:41:05+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-20 11:36:46+02	2020-09-20 11:37:56+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-20 11:36:46+02	2020-09-20 11:42:48+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-20 11:42:49+02	2020-09-20 11:42:56+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-20 21:46:52+02	2020-09-20 21:49:12+02	ports-mgmt	pkg	\N	pkg-1.15.4
2020-09-20 21:51:07+02	2020-09-20 21:52:55+02	ports-mgmt	pkg	\N	pkg-1.15.4
2020-09-20 21:56:04+02	2020-09-20 21:56:10+02	devel	autoconf-wrapper	\N	autoconf-wrapper-20131203
2020-09-20 21:56:04+02	2020-09-20 21:56:10+02	misc	pciids	\N	pciids-20200819
2020-09-20 21:56:04+02	2020-09-20 21:56:13+02	devel	ccache	\N	ccache-3.7.1_1
2020-09-20 21:56:10+02	2020-09-20 21:56:16+02	devel	evdev-proto	\N	evdev-proto-5.3
2020-09-20 21:56:10+02	2020-09-20 21:56:16+02	sysutils	gnome_subr	\N	gnome_subr-1.0
2020-09-20 21:56:14+02	2020-09-20 21:56:19+02	print	indexinfo	\N	indexinfo-0.3.1
2020-09-20 21:56:17+02	2020-09-20 21:56:27+02	devel	pkgconf	\N	pkgconf-1.7.3,1
2020-09-20 21:56:20+02	2020-09-20 21:56:56+02	devel	gettext-runtime	\N	gettext-runtime-0.21
2020-09-20 21:56:57+02	2020-09-20 21:57:16+02	devel	gmake	\N	gmake-4.3_2
2020-09-20 21:57:17+02	2020-09-20 21:57:46+02	devel	readline	\N	readline-8.0.4
2020-09-20 21:57:47+02	2020-09-20 21:58:05+02	devel	libffi	\N	libffi-3.3_1
2020-09-20 21:56:28+02	2020-09-20 21:58:08+02	devel	libtextstyle	\N	libtextstyle-0.21
2020-09-20 21:58:06+02	2020-09-20 21:58:45+02	converters	libiconv	\N	libiconv-1.16
2020-09-20 21:58:45+02	2020-09-20 21:59:08+02	textproc	expat2	\N	expat-2.2.8
2020-09-20 21:59:09+02	2020-09-20 22:00:50+02	textproc	libxml2	\N	libxml2-2.9.10
2020-09-20 22:00:51+02	2020-09-20 22:00:58+02	devel	xorg-macros	\N	xorg-macros-1.19.2
2020-09-20 22:00:59+02	2020-09-20 22:01:09+02	x11	xorgproto	\N	xorgproto-2020.1
2020-09-20 22:01:10+02	2020-09-20 22:01:49+02	archivers	liblz4	\N	liblz4-1.9.2_1,1
2020-09-20 22:01:49+02	2020-09-20 22:02:38+02	archivers	lzo2	\N	lzo2-2.10_1
2020-09-20 21:56:17+02	2020-09-20 22:02:47+02	lang	perl5.32	\N	perl5-5.32.0
2020-09-20 22:02:47+02	2020-09-20 22:03:03+02	converters	p5-Text-Unidecode	\N	p5-Text-Unidecode-1.30
2020-09-20 22:03:03+02	2020-09-20 22:03:19+02	textproc	p5-Unicode-EastAsianWidth	\N	p5-Unicode-EastAsianWidth-12.0
2020-09-20 21:58:09+02	2020-09-20 22:03:22+02	devel	gettext-tools	\N	gettext-tools-0.21
2020-09-20 22:03:19+02	2020-09-20 22:03:41+02	security	ca_root_nss	\N	ca_root_nss-3.56
2020-09-20 22:02:38+02	2020-09-20 22:03:49+02	archivers	libarchive	\N	libarchive-3.4.3,1
2020-09-20 22:03:41+02	2020-09-20 22:03:57+02	devel	p5-Locale-gettext	\N	p5-Locale-gettext-1.07
2020-09-20 22:03:50+02	2020-09-20 22:04:13+02	devel	p5-Locale-libintl	\N	p5-Locale-libintl-1.31
2020-09-20 22:03:57+02	2020-09-20 22:04:16+02	misc	help2man	\N	help2man-1.47.16
2020-09-20 22:04:14+02	2020-09-20 22:04:36+02	security	rhash	\N	rhash-1.3.9
2020-09-20 22:04:37+02	2020-09-20 22:04:45+02	devel	libpthread-stubs	\N	libpthread-stubs-0.4
2020-09-20 22:04:46+02	2020-09-20 22:04:57+02	x11	libXau	\N	libXau-1.0.9
2020-09-20 22:04:57+02	2020-09-20 22:05:09+02	x11	libXdmcp	\N	libXdmcp-1.1.3
2020-09-20 22:05:10+02	2020-09-20 22:05:20+02	x11	xtrans	\N	xtrans-1.4.0
2020-09-20 22:04:17+02	2020-09-20 22:05:25+02	print	texinfo	\N	texinfo-6.7_4,1
2020-09-20 22:05:21+02	2020-09-20 22:05:40+02	security	libgpg-error	\N	libgpg-error-1.39
2020-09-20 22:05:25+02	2020-09-20 22:06:07+02	devel	m4	\N	m4-1.4.18_1,1
2020-09-20 22:06:07+02	2020-09-20 22:06:44+02	devel	autoconf	\N	autoconf-2.69_3
2020-09-20 22:06:45+02	2020-09-20 22:07:20+02	devel	automake	\N	automake-1.16.2
2020-09-20 22:05:40+02	2020-09-20 22:07:38+02	math	gmp	\N	gmp-6.2.0
2020-09-20 22:07:20+02	2020-09-20 22:07:43+02	devel	libtool	\N	libtool-2.4.6_1
2020-09-20 22:03:22+02	2020-09-20 22:07:43+02	lang	python37	\N	python37-3.7.9
2020-09-20 22:07:44+02	2020-09-20 22:07:57+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-09-20 22:07:58+02	2020-09-20 22:08:09+02	devel	py-six	py37	py37-six-1.15.0
2020-09-20 22:08:10+02	2020-09-20 22:08:22+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-09-20 22:07:39+02	2020-09-20 22:08:31+02	math	mpfr	\N	mpfr-4.1.0
2020-09-20 22:07:43+02	2020-09-20 22:08:33+02	www	libnghttp2	\N	libnghttp2-1.41.0
2020-09-20 22:08:23+02	2020-09-20 22:08:34+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-09-20 22:08:32+02	2020-09-20 22:08:41+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-09-21 12:11:55+02	2020-09-21 12:17:11+02	lang	go	\N	go-1.15.2,1
2020-09-20 22:08:33+02	2020-09-20 22:08:43+02	dns	py-idna	py37	py37-idna-2.8
2020-09-20 22:08:35+02	2020-09-20 22:08:49+02	devel	py-cffi	py37	py37-cffi-1.14.3
2020-09-20 22:08:43+02	2020-09-20 22:09:11+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-09-20 22:09:11+02	2020-09-20 22:09:21+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-09-20 22:08:49+02	2020-09-20 22:09:26+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-09-20 22:09:21+02	2020-09-20 22:09:36+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-09-20 22:08:42+02	2020-09-20 22:09:39+02	devel	ninja	\N	ninja-1.10.1,2
2020-09-20 22:09:26+02	2020-09-20 22:09:41+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-09-20 22:09:37+02	2020-09-20 22:09:46+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-09-20 22:09:41+02	2020-09-20 22:09:50+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-09-20 22:09:39+02	2020-09-20 22:09:53+02	devel	meson	\N	meson-0.55.3
2020-09-20 22:09:46+02	2020-09-20 22:09:56+02	net	py-urllib3	py37	py37-urllib3-1.25.10,1
2020-09-20 22:09:51+02	2020-09-20 22:10:00+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-09-20 22:09:57+02	2020-09-20 22:10:09+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-09-20 22:10:09+02	2020-09-20 22:10:20+02	www	py-requests	py37	py37-requests-2.22.0
2020-09-20 22:10:20+02	2020-09-20 22:10:30+02	devel	py-packaging	py37	py37-packaging-20.4
2020-09-20 22:10:31+02	2020-09-20 22:10:45+02	textproc	py-pygments	py37	py37-pygments-2.6.1
2020-09-20 22:09:53+02	2020-09-20 22:11:13+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-09-20 22:11:13+02	2020-09-20 22:11:22+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-09-20 22:11:23+02	2020-09-20 22:11:32+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-09-20 22:11:32+02	2020-09-20 22:11:42+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-09-20 22:10:01+02	2020-09-20 22:11:45+02	lang	cython	py37	py37-cython-0.29.15
2020-09-20 22:11:43+02	2020-09-20 22:11:53+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-09-20 22:11:54+02	2020-09-20 22:12:05+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-09-20 22:11:45+02	2020-09-20 22:12:06+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-09-20 22:10:46+02	2020-09-20 22:12:11+02	ftp	curl	\N	curl-7.72.0
2020-09-20 22:12:06+02	2020-09-20 22:12:15+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-09-20 22:12:07+02	2020-09-20 22:12:17+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-09-20 22:12:11+02	2020-09-20 22:12:20+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-09-20 22:12:15+02	2020-09-20 22:12:24+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-09-20 22:12:24+02	2020-09-20 22:12:47+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-09-20 22:12:20+02	2020-09-20 22:12:53+02	devel	jsoncpp	\N	jsoncpp-1.9.3
2020-09-20 22:12:17+02	2020-09-20 22:12:54+02	devel	libuv	\N	libuv-1.39.0
2020-09-20 22:12:48+02	2020-09-20 22:12:59+02	x11	xcb-proto	\N	xcb-proto-1.14
2020-09-20 22:13:00+02	2020-09-20 22:14:14+02	x11	libxcb	\N	libxcb-1.14_1
2020-09-20 22:12:53+02	2020-09-20 22:14:37+02	security	libgcrypt	\N	libgcrypt-1.8.5
2020-09-20 22:14:37+02	2020-09-20 22:15:07+02	devel	libedit	\N	libedit-3.1.20191231,1
2020-09-20 22:15:07+02	2020-09-20 22:15:47+02	textproc	libxslt	\N	libxslt-1.1.34_1
2020-09-20 22:14:15+02	2020-09-20 22:16:52+02	x11	libX11	\N	libX11-1.6.12,1
2020-09-20 22:16:53+02	2020-09-20 22:17:16+02	x11	libXext	\N	libXext-1.3.4,1
2020-09-20 22:17:16+02	2020-09-20 22:21:11+02	devel	pcre	\N	pcre-8.44
2020-09-20 22:21:11+02	2020-09-20 22:22:43+02	print	freetype2	\N	freetype2-2.10.2_1
2020-09-20 22:22:43+02	2020-09-20 22:24:31+02	devel	bison	\N	bison-3.6.4,1
2020-09-20 22:24:32+02	2020-09-20 22:24:54+02	devel	gperf	\N	gperf-3.1
2020-09-20 22:24:54+02	2020-09-20 22:25:51+02	x11-fonts	fontconfig	\N	fontconfig-2.13.92_2,1
2020-09-20 22:25:52+02	2020-09-20 22:26:13+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-09-20 22:26:14+02	2020-09-20 22:26:32+02	lang	lua52	\N	lua52-5.2.4
2020-09-20 22:26:32+02	2020-09-20 22:27:21+02	devel	libunwind	\N	libunwind-20200331_1
2020-09-20 22:27:21+02	2020-09-20 22:27:37+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-09-20 22:27:38+02	2020-09-20 22:27:50+02	textproc	xmlcatmgr	\N	xmlcatmgr-2.2_2
2020-09-20 22:27:50+02	2020-09-20 22:28:08+02	devel	py-future	py37	py37-future-0.18.2
2020-09-20 22:28:09+02	2020-09-20 22:28:26+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-09-20 22:28:27+02	2020-09-20 22:28:48+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-09-20 22:28:48+02	2020-09-20 22:28:56+02	textproc	iso8879	\N	iso8879-1986_3
2020-09-20 22:28:56+02	2020-09-20 22:29:15+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-09-20 22:29:15+02	2020-09-20 22:29:23+02	textproc	xmlcharent	\N	xmlcharent-0.3_2
2020-09-20 22:29:23+02	2020-09-20 22:30:54+02	devel	swig	\N	swig-4.0.2
2020-09-20 22:30:55+02	2020-09-20 22:31:04+02	textproc	docbook-sgml	\N	docbook-sgml-4.5_1
2020-09-20 22:31:05+02	2020-09-20 22:31:14+02	textproc	docbook-xml	\N	docbook-xml-5.0_3
2020-09-20 22:31:15+02	2020-09-20 22:31:32+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-09-20 22:31:33+02	2020-09-20 22:31:40+02	textproc	sdocbook-xml	\N	sdocbook-xml-1.1_2,2
2020-09-20 22:31:40+02	2020-09-20 22:31:47+02	textproc	docbook	\N	docbook-1.5
2020-09-20 22:31:48+02	2020-09-20 22:32:14+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-09-20 22:32:14+02	2020-09-20 22:32:59+02	textproc	docbook-xsl	\N	docbook-xsl-1.79.1_1,1
2020-09-20 22:15:48+02	2020-09-20 22:33:11+02	devel	binutils	native	binutils-2.33.1_3,1
2020-09-20 22:32:59+02	2020-09-20 22:33:14+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-09-20 22:33:13+02	2020-09-20 22:33:25+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-09-20 22:33:15+02	2020-09-20 22:33:27+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-09-20 22:33:25+02	2020-09-20 22:33:37+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-09-20 22:33:27+02	2020-09-20 22:33:39+02	x11	libxshmfence	\N	libxshmfence-1.3
2020-09-20 22:33:38+02	2020-09-20 22:34:10+02	graphics	png	\N	png-1.6.37
2020-09-20 22:12:55+02	2020-09-20 22:35:52+02	devel	cmake	\N	cmake-3.18.2
2020-09-20 22:35:53+02	2020-09-20 22:36:16+02	devel	libepoll-shim	\N	libepoll-shim-0.0.20200602
2020-09-20 22:36:17+02	2020-09-20 22:36:42+02	graphics	wayland	\N	wayland-1.18.0_4
2020-09-20 22:33:39+02	2020-09-20 22:38:22+02	devel	glib20	\N	glib-2.56.3_9,1
2020-09-20 22:38:23+02	2020-09-20 22:38:32+02	graphics	wayland-protocols	\N	wayland-protocols-1.20
2020-09-20 22:38:33+02	2020-09-20 22:39:50+02	devel	boost-jam	\N	boost-jam-1.72.0
2020-09-20 22:39:50+02	2020-09-20 22:41:22+02	x11	pixman	\N	pixman-0.40.0_1
2020-09-20 22:41:22+02	2020-09-20 22:41:44+02	x11	libICE	\N	libICE-1.0.10,1
2020-09-20 22:41:45+02	2020-09-20 22:42:45+02	devel	nasm	\N	nasm-2.15.05,1
2020-09-20 22:42:46+02	2020-09-20 22:43:03+02	x11	libSM	\N	libSM-1.2.3,1
2020-09-20 22:43:03+02	2020-09-20 22:43:18+02	x11-fonts	libfontenc	\N	libfontenc-1.1.4
2020-09-20 22:43:19+02	2020-09-20 22:43:51+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-09-20 22:43:51+02	2020-09-20 22:44:40+02	graphics	jpeg-turbo	\N	jpeg-turbo-2.0.4
2020-09-20 22:44:40+02	2020-09-20 22:44:52+02	textproc	itstool	\N	itstool-2.0.6
2020-09-20 22:44:52+02	2020-09-20 22:45:05+02	x11-fonts	mkfontscale	\N	mkfontscale-1.2.1
2020-09-20 22:34:10+02	2020-09-20 22:45:12+02	devel	icu	\N	icu-67.1,1
2020-09-20 22:45:05+02	2020-09-20 22:45:42+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-09-20 22:45:43+02	2020-09-20 22:46:06+02	textproc	html2text	\N	html2text-1.3.2a
2020-09-20 22:46:07+02	2020-09-20 22:46:17+02	textproc	minixmlto	\N	minixmlto-0.0.2_1
2020-09-20 22:46:18+02	2020-09-20 22:47:43+02	devel	dbus	\N	dbus-1.12.20
2020-09-20 22:47:44+02	2020-09-20 22:47:57+02	x11-fonts	bdftopcf	\N	bdftopcf-1.1_2
2020-09-20 22:47:58+02	2020-09-20 22:48:12+02	x11-fonts	font-util	\N	font-util-1.3.2
2020-09-20 22:48:13+02	2020-09-20 22:48:43+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-09-20 22:48:43+02	2020-09-20 22:50:20+02	shells	bash	\N	bash-5.0.18_3
2020-09-20 22:50:20+02	2020-09-20 22:51:04+02	graphics	graphite2	\N	graphite2-1.3.14
2020-09-20 22:51:04+02	2020-09-20 22:51:29+02	textproc	p5-XML-Parser	\N	p5-XML-Parser-2.44
2020-09-20 22:51:29+02	2020-09-20 22:51:42+02	x11-fonts	encodings	\N	encodings-1.0.5,1
2020-09-20 22:51:43+02	2020-09-20 22:51:58+02	x11-fonts	dejavu	\N	dejavu-2.37_1
2020-09-20 22:51:59+02	2020-09-20 22:52:19+02	textproc	intltool	\N	intltool-0.51.0_1
2020-09-20 22:52:20+02	2020-09-20 22:52:31+02	x11-fonts	font-bh-ttf	\N	font-bh-ttf-1.0.3_4
2020-09-20 22:52:31+02	2020-09-20 22:52:44+02	x11-fonts	font-misc-ethiopic	\N	font-misc-ethiopic-1.0.4
2020-09-20 22:52:44+02	2020-09-20 22:52:58+02	x11-fonts	font-misc-meltho	\N	font-misc-meltho-1.0.3_4
2020-09-20 22:52:59+02	2020-09-20 22:53:19+02	x11-fonts	xorg-fonts-truetype	\N	xorg-fonts-truetype-7.7_1
2020-09-20 22:53:19+02	2020-09-20 22:53:30+02	print	libpaper	\N	libpaper-1.1.24.4
2020-09-20 22:53:31+02	2020-09-20 22:53:43+02	graphics	jbigkit	\N	jbigkit-2.1_1
2020-09-20 22:53:43+02	2020-09-20 22:54:52+02	graphics	tiff	\N	tiff-4.1.0
2020-09-20 22:54:52+02	2020-09-20 22:55:25+02	x11	libXi	\N	libXi-1.7.10,1
2020-09-20 22:55:26+02	2020-09-20 22:59:14+02	devel	libunistring	\N	libunistring-0.9.10_1
2020-09-20 22:59:14+02	2020-09-20 22:59:51+02	dns	libidn2	\N	libidn2-2.3.0_1
2020-09-20 22:59:52+02	2020-09-20 23:00:20+02	emulators	tpm-emulator	\N	tpm-emulator-0.7.4_2
2020-09-20 23:00:21+02	2020-09-20 23:00:56+02	security	libtasn1	\N	libtasn1-4.16.0
2020-09-20 23:00:57+02	2020-09-20 23:02:24+02	security	nettle	\N	nettle-3.6
2020-09-20 23:02:24+02	2020-09-20 23:02:53+02	shells	bash-completion	\N	bash-completion-2.11,2
2020-09-20 23:02:54+02	2020-09-20 23:05:27+02	security	trousers	\N	trousers-0.3.14_3
2020-09-20 23:05:27+02	2020-09-20 23:06:09+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-09-20 23:06:10+02	2020-09-20 23:06:43+02	databases	gdbm	\N	gdbm-1.18.1_1
2020-09-20 23:06:44+02	2020-09-20 23:14:10+02	databases	sqlite3	\N	sqlite3-3.33.0,1
2020-09-20 23:14:11+02	2020-09-20 23:14:24+02	devel	libdaemon	\N	libdaemon-0.14_1
2020-09-20 23:14:24+02	2020-09-20 23:14:38+02	devel	libltdl	\N	libltdl-2.4.6
2020-09-20 23:14:39+02	2020-09-20 23:14:58+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-09-20 23:14:59+02	2020-09-20 23:15:09+02	graphics	giflib	\N	giflib-5.2.1
2020-09-20 23:15:10+02	2020-09-20 23:15:25+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-09-20 23:15:25+02	2020-09-20 23:15:44+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-09-20 22:45:13+02	2020-09-20 23:16:02+02	devel	boost-libs	\N	boost-libs-1.72.0_2
2020-09-20 23:15:44+02	2020-09-20 23:16:10+02	x11	xkeyboard-config	\N	xkeyboard-config-2.30
2020-09-20 23:16:11+02	2020-09-20 23:16:19+02	misc	hicolor-icon-theme	\N	hicolor-icon-theme-0.17
2020-09-20 23:16:20+02	2020-09-20 23:17:58+02	graphics	webp	\N	webp-1.1.0
2020-09-20 23:17:58+02	2020-09-20 23:18:12+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-09-20 23:16:03+02	2020-09-20 23:18:24+02	textproc	highlight	\N	highlight-3.43_8,3
2020-09-20 23:18:12+02	2020-09-20 23:18:47+02	x11	libxkbcommon	\N	libxkbcommon-1.0.1
2020-09-20 23:18:25+02	2020-09-20 23:19:05+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-09-20 23:18:48+02	2020-09-20 23:19:12+02	x11	libXpm	\N	libXpm-3.5.13
2020-09-20 23:19:13+02	2020-09-20 23:19:26+02	converters	fribidi	\N	fribidi-0.19.7
2020-09-20 23:19:27+02	2020-09-20 23:20:00+02	devel	libatomic_ops	\N	libatomic_ops-7.6.10
2020-09-20 23:19:05+02	2020-09-20 23:20:17+02	security	p11-kit	\N	p11-kit-0.23.21
2020-09-20 23:20:01+02	2020-09-20 23:20:31+02	devel	boehm-gc	\N	boehm-gc-8.0.4_1
2020-09-20 23:20:32+02	2020-09-20 23:21:21+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-09-20 23:21:22+02	2020-09-20 23:21:29+02	misc	getopt	\N	getopt-1.1.6
2020-09-20 23:21:30+02	2020-09-20 23:22:55+02	www	w3m	\N	w3m-0.5.3.20200507
2020-09-20 23:22:56+02	2020-09-20 23:23:22+02	textproc	xmlto	\N	xmlto-0.0.28
2020-09-20 23:20:18+02	2020-09-20 23:23:56+02	security	gnutls	\N	gnutls-3.6.15
2020-09-20 23:23:56+02	2020-09-20 23:24:14+02	graphics	poppler-data	\N	poppler-data-0.4.9_5
2020-09-20 23:23:23+02	2020-09-20 23:24:14+02	dns	libidn	\N	libidn-1.35
2020-09-20 23:24:15+02	2020-09-20 23:24:33+02	print	psutils	\N	psutils-1.17_5
2020-09-20 23:24:15+02	2020-09-20 23:24:46+02	graphics	lcms2	\N	lcms2-2.11_1
2020-09-20 23:24:33+02	2020-09-20 23:25:13+02	graphics	jbig2dec	\N	jbig2dec-0.19
2020-09-20 23:24:47+02	2020-09-20 23:25:18+02	devel	nspr	\N	nspr-4.29
2020-09-20 23:25:14+02	2020-09-20 23:25:43+02	graphics	gd	\N	libgd-2.3.0,1
2020-09-20 23:25:18+02	2020-09-20 23:26:30+02	misc	shared-mime-info	\N	shared-mime-info-2.0
2020-09-20 23:25:43+02	2020-09-20 23:28:46+02	devel	pcre2	\N	pcre2-10.35
2020-09-20 23:28:46+02	2020-09-20 23:28:56+02	print	gsfonts	\N	gsfonts-8.11_8
2020-09-20 23:28:57+02	2020-09-20 23:29:16+02	textproc	uchardet	\N	uchardet-0.0.7
2020-09-20 23:26:31+02	2020-09-20 23:29:20+02	lang	tcl86	\N	tcl86-8.6.10
2020-09-20 23:29:21+02	2020-09-20 23:29:33+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-09-20 23:29:33+02	2020-09-20 23:29:48+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-09-20 23:29:49+02	2020-09-20 23:30:12+02	multimedia	v4l_compat	\N	v4l_compat-1.18.0
2020-09-20 23:30:13+02	2020-09-20 23:30:29+02	devel	ctags	\N	ctags-5.8
2020-09-20 23:30:30+02	2020-09-20 23:31:00+02	security	libssh2	\N	libssh2-1.9.0_3,3
2020-09-20 23:29:16+02	2020-09-20 23:32:27+02	textproc	groff	\N	groff-1.22.4_3
2020-09-20 23:32:28+02	2020-09-20 23:32:43+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-09-20 23:32:43+02	2020-09-20 23:33:55+02	devel	libgit2	\N	libgit2-1.0.1
2020-09-20 23:31:00+02	2020-09-20 23:37:08+02	textproc	source-highlight	\N	source-highlight-3.1.9_1
2020-09-20 23:37:09+02	2020-09-20 23:57:51+02	devel	gdb	py37	gdb-9.2_1
2020-09-20 22:36:42+02	2020-09-21 04:05:24+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-09-21 04:05:25+02	2020-09-21 04:14:46+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_3
2020-09-21 04:14:46+02	2020-09-21 04:18:59+02	graphics	cairo	\N	cairo-1.16.0,3
2020-09-21 04:19:01+02	2020-09-21 04:21:06+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-09-21 04:21:07+02	2020-09-21 04:27:26+02	print	harfbuzz	\N	harfbuzz-2.7.2
2020-09-21 04:27:26+02	2020-09-21 04:29:25+02	net	avahi-app	\N	avahi-app-0.7_3
2020-09-21 04:29:25+02	2020-09-21 04:31:23+02	print	cups	\N	cups-2.3.3_1
2020-09-21 04:31:24+02	2020-09-21 04:34:25+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-09-21 04:34:26+02	2020-09-21 04:35:05+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-09-21 04:35:06+02	2020-09-21 04:40:27+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-09-21 04:40:27+02	2020-09-21 04:41:46+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-09-21 04:41:47+02	2020-09-21 04:47:56+02	graphics	graphviz	\N	graphviz-2.44.1_2
2020-09-21 04:47:57+02	2020-09-21 04:48:31+02	accessibility	atk	\N	atk-2.36.0
2020-09-21 04:48:32+02	2020-09-21 04:49:02+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-09-21 04:49:03+02	2020-09-21 04:49:18+02	devel	libudev-devd	\N	libudev-devd-0.4.2
2020-09-21 04:49:19+02	2020-09-21 04:54:23+02	lang	vala	\N	vala-0.40.15,1
2020-09-21 04:54:24+02	2020-09-21 04:54:49+02	devel	autoconf213	\N	autoconf213-2.13.000227_7
2020-09-21 04:54:50+02	2020-09-21 04:55:06+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-09-21 04:55:07+02	2020-09-21 04:55:22+02	x11	xcb-util	\N	xcb-util-0.4.0_2,1
2020-09-21 04:55:22+02	2020-09-21 04:56:49+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-09-21 04:56:50+02	2020-09-21 04:57:44+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-09-21 04:57:45+02	2020-09-21 04:58:29+02	textproc	libcroco	\N	libcroco-0.6.13
2020-09-21 04:58:30+02	2020-09-21 04:59:51+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-09-21 04:59:51+02	2020-09-21 05:00:54+02	archivers	zstd	\N	zstd-1.4.5_1
2020-09-21 05:00:54+02	2020-09-21 05:01:07+02	misc	qtchooser	\N	qtchooser-66_4
2020-09-21 05:01:08+02	2020-09-21 05:01:28+02	devel	double-conversion	\N	double-conversion-3.1.5.19
2020-09-21 05:01:29+02	2020-09-21 05:05:20+02	devel	qt5-qmake	\N	qt5-qmake-5.15.0
2020-09-21 05:05:21+02	2020-09-21 05:09:45+02	devel	qt5-buildtools	\N	qt5-buildtools-5.15.0
2020-09-21 05:09:46+02	2020-09-21 05:10:01+02	x11	xprop	\N	xprop-1.2.4
2020-09-21 05:10:02+02	2020-09-21 05:10:17+02	x11	xset	\N	xset-1.2.4_3
2020-09-21 05:10:18+02	2020-09-21 05:22:40+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-09-21 05:22:41+02	2020-09-21 05:23:37+02	graphics	openjpeg	\N	openjpeg-2.3.1
2020-09-21 05:23:37+02	2020-09-21 05:23:54+02	x11	xcb-util-wm	\N	xcb-util-wm-0.4.1_3
2020-09-21 05:23:55+02	2020-09-21 05:24:16+02	devel	jam	\N	jam-2.6
2020-09-21 05:24:16+02	2020-09-21 05:24:38+02	devel	libevdev	\N	libevdev-1.5.9_1
2020-09-21 05:24:39+02	2020-09-21 05:24:58+02	devel	libgudev	\N	libgudev-233
2020-09-21 05:24:59+02	2020-09-21 05:25:13+02	devel	libmtdev	\N	libmtdev-1.1.5_3
2020-09-21 05:25:13+02	2020-09-21 05:25:35+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-09-21 05:25:36+02	2020-09-21 05:25:50+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-09-21 05:25:51+02	2020-09-21 05:26:05+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-09-21 05:26:05+02	2020-09-21 05:30:28+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-09-21 05:30:29+02	2020-09-21 05:30:49+02	x11	libwacom	\N	libwacom-1.4.1
2020-09-21 05:30:50+02	2020-09-21 05:31:24+02	x11	libinput	\N	libinput-1.15.6
2020-09-21 05:31:25+02	2020-09-21 05:31:43+02	devel	vulkan-headers	\N	vulkan-headers-1.2.135.0
2020-09-21 05:31:44+02	2020-09-21 05:32:02+02	x11	xcb-util-image	\N	xcb-util-image-0.4.0_1
2020-09-21 05:32:02+02	2020-09-21 05:32:17+02	x11	xcb-util-keysyms	\N	xcb-util-keysyms-0.4.0_1
2020-09-21 05:32:18+02	2020-09-21 05:32:33+02	x11	xcb-util-renderutil	\N	xcb-util-renderutil-0.3.9_1
2020-09-21 05:32:33+02	2020-09-21 05:33:18+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-09-21 05:33:19+02	2020-09-21 05:35:11+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-09-21 05:35:12+02	2020-09-21 05:37:25+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-09-21 05:37:25+02	2020-09-21 05:42:57+02	net	qt5-network	\N	qt5-network-5.15.0
2020-09-20 23:33:56+02	2020-09-21 05:52:57+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-09-21 05:52:59+02	2020-09-21 05:54:04+02	textproc	gsed	\N	gsed-4.8
2020-09-21 05:54:05+02	2020-09-21 05:54:18+02	audio	libogg	\N	libogg-1.3.4,4
2020-09-21 05:54:19+02	2020-09-21 05:55:05+02	misc	iso-codes	\N	iso-codes-4.2
2020-09-21 05:55:06+02	2020-09-21 05:55:43+02	audio	libvorbis	\N	libvorbis-1.3.7_2,3
2020-09-21 05:55:43+02	2020-09-21 05:56:18+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.2
2020-09-21 05:56:19+02	2020-09-21 05:57:12+02	graphics	libGLU	\N	libGLU-9.0.1
2020-09-21 05:57:13+02	2020-09-21 05:58:27+02	devel	orc	\N	orc-0.4.31
2020-09-21 05:58:27+02	2020-09-21 05:59:49+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-09-21 05:42:57+02	2020-09-21 06:01:25+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-09-21 05:59:50+02	2020-09-21 06:02:45+02	security	nss	\N	nss-3.56
2020-09-21 06:02:45+02	2020-09-21 06:04:10+02	audio	flac	\N	flac-1.3.3
2020-09-21 06:04:11+02	2020-09-21 06:04:29+02	devel	desktop-file-utils	\N	desktop-file-utils-0.26
2020-09-21 06:04:30+02	2020-09-21 06:07:40+02	math	fftw3	\N	fftw3-3.3.8_6
2020-09-21 06:01:25+02	2020-09-21 06:11:16+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-09-21 06:11:17+02	2020-09-21 06:13:03+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-09-21 06:13:06+02	2020-09-21 06:13:26+02	archivers	zip	\N	zip-3.0_1
2020-09-21 06:13:26+02	2020-09-21 06:15:28+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-09-21 06:07:40+02	2020-09-21 06:15:32+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-21 06:15:33+02	2020-09-21 06:16:16+02	audio	opus	\N	opus-1.3.1
2020-09-21 06:16:17+02	2020-09-21 06:19:50+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-09-21 06:19:51+02	2020-09-21 06:20:10+02	textproc	libyaml	\N	libyaml-0.2.5
2020-09-21 06:20:10+02	2020-09-21 06:20:47+02	audio	libsndfile	\N	libsndfile-1.0.29
2020-09-21 06:20:48+02	2020-09-21 06:23:18+02	graphics	poppler	\N	poppler-20.09.0
2020-09-21 06:23:18+02	2020-09-21 06:23:44+02	devel	tex-kpathsea	\N	tex-kpathsea-6.2.1_2
2020-09-21 06:23:45+02	2020-09-21 06:24:10+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-09-20 23:57:51+02	2020-09-21 06:25:09+02	lang	rust	\N	rust-1.46.0
2020-09-21 06:25:11+02	2020-09-21 06:31:35+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.5
2020-09-21 06:24:11+02	2020-09-21 06:39:36+02	lang	ruby27	\N	ruby-2.7.1_1,1
2020-09-21 06:39:37+02	2020-09-21 06:40:27+02	net	libproxy	\N	libproxy-0.4.15
2020-09-21 06:40:27+02	2020-09-21 06:41:00+02	net	glib-networking	\N	glib-networking-2.56.1_3
2020-09-21 06:41:02+02	2020-09-21 06:41:57+02	devel	dconf	\N	dconf-0.38.0
2020-09-21 06:15:29+02	2020-09-21 06:42:51+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-09-21 06:42:52+02	2020-09-21 06:43:15+02	devel	zziplib	\N	zziplib-0.13.71_1
2020-09-21 06:41:58+02	2020-09-21 06:43:29+02	devel	libsoup	\N	libsoup-2.62.3
2020-09-21 06:43:16+02	2020-09-21 06:43:41+02	graphics	openjpeg15	\N	openjpeg15-1.5.2_1
2020-09-21 06:43:30+02	2020-09-21 06:43:50+02	print	tex-ptexenc	\N	tex-ptexenc-1.3.3_2
2020-09-21 06:43:41+02	2020-09-21 06:44:08+02	devel	json-glib	\N	json-glib-1.6.0
2020-09-21 06:44:09+02	2020-09-21 06:44:31+02	math	mpc	\N	mpc-1.1.0_2
2020-09-21 06:44:31+02	2020-09-21 06:45:32+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-09-21 06:31:36+02	2020-09-21 06:45:39+02	lang	spidermonkey78	\N	spidermonkey78-78.2.0
2020-09-21 06:45:33+02	2020-09-21 06:46:03+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-09-21 06:43:50+02	2020-09-21 06:46:03+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-09-21 06:46:04+02	2020-09-21 06:46:18+02	graphics	libpotrace	\N	libpotrace-1.16
2020-09-21 06:45:41+02	2020-09-21 06:46:27+02	sysutils	polkit	\N	polkit-0.118
2020-09-21 06:46:04+02	2020-09-21 06:46:29+02	print	texlive-tlmgr	\N	texlive-tlmgr-20150523_2
2020-09-21 06:46:18+02	2020-09-21 06:46:32+02	print	xpdfopen	\N	xpdfopen-0.86
2020-09-21 06:46:29+02	2020-09-21 06:47:25+02	textproc	teckit	\N	teckit-2.5.7
2020-09-21 06:46:33+02	2020-09-21 06:48:06+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-09-21 06:46:28+02	2020-09-21 06:48:16+02	graphics	colord	\N	colord-1.3.5
2020-09-21 06:48:06+02	2020-09-21 06:48:57+02	archivers	brotli	\N	brotli-1.0.9,1
2020-09-21 06:47:26+02	2020-09-21 06:55:05+02	print	texlive-base	\N	texlive-base-20150521_55
2020-09-21 06:48:16+02	2020-09-21 06:58:56+02	x11-toolkits	gtk30	\N	gtk3-3.24.23
2020-09-21 06:48:58+02	2020-09-21 07:00:03+02	devel	protobuf	\N	protobuf-3.13.0,1
2020-09-21 07:00:04+02	2020-09-21 07:00:40+02	textproc	flex	\N	flex-2.6.4_2
2020-09-21 07:00:40+02	2020-09-21 07:01:05+02	devel	libnotify	\N	libnotify-0.7.9
2020-09-21 06:58:57+02	2020-09-21 07:01:24+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-09-21 07:01:24+02	2020-09-21 07:02:11+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-09-21 07:01:06+02	2020-09-21 07:02:17+02	textproc	hunspell	\N	hunspell-1.7.0_2
2020-09-21 07:02:11+02	2020-09-21 07:02:50+02	devel	libevent	\N	libevent-2.1.12
2020-09-21 07:02:50+02	2020-09-21 07:03:12+02	multimedia	libva	\N	libva-2.9.0
2020-09-21 07:03:13+02	2020-09-21 07:03:24+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-09-21 07:02:18+02	2020-09-21 07:05:07+02	lang	python27	\N	python27-2.7.18_1
2020-09-21 07:03:24+02	2020-09-21 07:06:10+02	databases	db6	\N	db6-6.2.32_1
2020-09-21 07:06:10+02	2020-09-21 07:06:32+02	devel	woff2	\N	woff2-1.0.2_4
2020-09-21 07:06:32+02	2020-09-21 07:06:43+02	security	libargon2	\N	libargon2-20190702
2020-09-21 07:06:43+02	2020-09-21 07:07:34+02	security	libsecret	\N	libsecret-0.18.6_1
2020-09-21 07:05:07+02	2020-09-21 07:07:45+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-09-21 07:07:35+02	2020-09-21 07:07:51+02	textproc	hyphen	\N	hyphen-2.8.8
2020-09-21 07:07:46+02	2020-09-21 07:07:54+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-09-21 07:07:51+02	2020-09-21 07:08:14+02	devel	argp-standalone	\N	argp-standalone-1.3_4
2020-09-21 07:08:14+02	2020-09-21 07:08:21+02	audio	ladspa	\N	ladspa-1.15
2020-09-21 07:07:55+02	2020-09-21 07:09:31+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-09-21 07:09:32+02	2020-09-21 07:10:36+02	multimedia	libv4l	\N	libv4l-1.18.0
2020-09-21 07:10:36+02	2020-09-21 07:10:48+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-09-21 07:10:48+02	2020-09-21 07:11:17+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-09-21 07:11:18+02	2020-09-21 07:11:42+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-09-21 07:11:43+02	2020-09-21 07:11:52+02	devel	opencl	\N	opencl-2.2_2
2020-09-21 07:11:52+02	2020-09-21 07:12:02+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-09-21 06:55:06+02	2020-09-21 07:12:13+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-21 07:12:02+02	2020-09-21 07:12:21+02	math	eigen3	\N	eigen-3.3.7
2020-09-21 07:11:58+02	2020-09-21 07:13:24+02	lang	fpc	\N	fpc-3.0.4_4
2020-09-21 07:13:24+02	2020-09-21 07:13:48+02	textproc	texi2html	\N	texi2html-5.0_2,1
2020-09-21 07:12:13+02	2020-09-21 07:14:22+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-21 07:14:23+02	2020-09-21 07:14:41+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-09-21 07:13:48+02	2020-09-21 07:14:56+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-09-21 07:14:56+02	2020-09-21 07:15:11+02	textproc	py-rdflib	py37	py37-rdflib-5.0.0
2020-09-21 07:15:11+02	2020-09-21 07:15:28+02	audio	lv2	\N	lv2-1.18.0
2020-09-21 07:15:28+02	2020-09-21 07:16:09+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-09-21 07:16:10+02	2020-09-21 07:17:35+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-21 07:17:35+02	2020-09-21 07:17:54+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-09-21 07:12:22+02	2020-09-21 07:18:21+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-21 07:18:22+02	2020-09-21 07:18:42+02	multimedia	libaacs	\N	libaacs-0.9.0
2020-09-21 07:18:42+02	2020-09-21 07:19:25+02	multimedia	libbdplus	\N	libbdplus-0.1.2
2020-09-21 07:19:26+02	2020-09-21 07:19:53+02	multimedia	libtheora	\N	libtheora-1.1.1_7
2020-09-21 07:17:55+02	2020-09-21 07:19:58+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-09-21 07:19:53+02	2020-09-21 07:20:06+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-09-21 07:20:07+02	2020-09-21 07:20:35+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-09-21 07:14:42+02	2020-09-21 07:20:45+02	multimedia	x265	\N	x265-3.2.1_4
2020-09-21 07:20:35+02	2020-09-21 07:21:00+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-21 07:19:59+02	2020-09-21 07:21:49+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-21 07:20:46+02	2020-09-21 07:22:02+02	graphics	frei0r	\N	frei0r-1.6.1
2020-09-21 07:21:02+02	2020-09-21 07:28:44+02	graphics	opencv-core	\N	opencv-core-3.4.1_36
2020-09-21 07:22:02+02	2020-09-21 07:29:13+02	multimedia	aom	\N	aom-2.0.0_1
2020-09-21 07:29:14+02	2020-09-21 07:29:45+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-09-21 07:28:45+02	2020-09-21 07:30:14+02	multimedia	dav1d	\N	dav1d-0.7.1
2020-09-21 07:30:14+02	2020-09-21 07:32:53+02	multimedia	libx264	\N	libx264-0.160.3011
2020-09-21 07:29:46+02	2020-09-21 07:33:40+02	multimedia	libvpx	\N	libvpx-1.9.0
2020-09-21 07:32:54+02	2020-09-21 07:33:44+02	multimedia	xvid	\N	xvid-1.3.7,1
2020-09-21 07:33:40+02	2020-09-21 07:34:42+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-09-21 07:34:42+02	2020-09-21 07:35:30+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-09-21 07:35:30+02	2020-09-21 07:37:04+02	graphics	graphene	\N	graphene-1.10.2
2020-09-21 07:37:05+02	2020-09-21 07:37:23+02	www	libwpe	\N	libwpe-1.4.0.1
2020-09-21 07:37:23+02	2020-09-21 07:40:53+02	databases	postgresql12-client	\N	postgresql12-client-12.4
2020-09-21 07:40:54+02	2020-09-21 07:41:13+02	devel	libinotify	\N	libinotify-20180201_2
2020-09-21 07:41:13+02	2020-09-21 07:44:23+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-21 07:44:25+02	2020-09-21 07:45:53+02	net	geoclue	\N	geoclue-2.5.5
2020-09-21 07:45:54+02	2020-09-21 07:46:18+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-09-21 07:46:18+02	2020-09-21 07:46:46+02	graphics	jasper	\N	jasper-2.0.20
2020-09-21 07:33:44+02	2020-09-21 07:52:03+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_2,1
2020-09-21 07:52:04+02	2020-09-21 07:52:56+02	audio	alsa-lib	\N	alsa-lib-1.1.2_2
2020-09-21 07:52:57+02	2020-09-21 07:54:14+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.74.0
2020-09-21 07:54:14+02	2020-09-21 07:55:02+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-09-21 07:55:02+02	2020-09-21 08:12:25+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-21 08:12:26+02	2020-09-21 08:12:41+02	devel	py-setuptools	py27	py27-setuptools-44.0.0
2020-09-21 08:12:42+02	2020-09-21 08:13:57+02	security	cyrus-sasl2	\N	cyrus-sasl-2.1.27_1
2020-09-21 08:13:57+02	2020-09-21 08:14:35+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-09-21 08:14:36+02	2020-09-21 08:15:36+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-09-21 08:15:37+02	2020-09-21 08:16:03+02	archivers	libmspack	\N	libmspack-0.10.1
2020-09-21 08:16:03+02	2020-09-21 08:17:36+02	graphics	libraw	\N	libraw-0.19.5
2020-09-21 08:17:36+02	2020-09-21 08:18:28+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-09-21 08:18:29+02	2020-09-21 08:27:22+02	devel	tbb	\N	tbb-2020.3
2020-09-21 08:27:24+02	2020-09-21 08:28:03+02	graphics	aalib	\N	aalib-1.4.r5_13
2020-09-21 08:28:04+02	2020-09-21 08:30:22+02	net	openldap24-client	\N	openldap-client-2.4.51
2020-09-21 08:30:22+02	2020-09-21 08:31:03+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-21 08:31:03+02	2020-09-21 08:31:27+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-09-21 08:31:28+02	2020-09-21 08:33:46+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-09-21 08:33:47+02	2020-09-21 08:34:09+02	print	libraqm	\N	libraqm-0.6.0
2020-09-21 08:34:10+02	2020-09-21 08:34:30+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-09-21 08:34:31+02	2020-09-21 08:35:02+02	devel	json-c	\N	json-c-0.15
2020-09-21 08:35:03+02	2020-09-21 08:35:28+02	science	szip	\N	hdf-szip-2.1.1
2020-09-21 08:35:28+02	2020-09-21 08:35:42+02	textproc	py-toml	py37	py37-toml-0.10.1
2020-09-21 08:35:42+02	2020-09-21 08:35:59+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-09-21 08:36:00+02	2020-09-21 08:37:13+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-09-21 08:37:14+02	2020-09-21 08:37:27+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3_1
2020-09-21 08:37:28+02	2020-09-21 08:38:41+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-09-21 08:38:41+02	2020-09-21 08:39:02+02	net	p5-URI	\N	p5-URI-1.76
2020-09-21 08:39:03+02	2020-09-21 09:01:47+02	science	hdf5	\N	hdf5-1.10.6,1
2020-09-21 09:01:48+02	2020-09-21 09:02:05+02	x11	libXres	\N	libXres-1.2.0_2
2020-09-21 09:02:06+02	2020-09-21 09:02:24+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-09-21 09:02:25+02	2020-09-21 09:02:49+02	audio	speexdsp	\N	speexdsp-1.2.0
2020-09-21 09:02:50+02	2020-09-21 09:13:10+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-21 09:13:11+02	2020-09-21 09:14:44+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-09-21 09:14:45+02	2020-09-21 09:16:20+02	textproc	librevenge	\N	librevenge-0.0.4_13
2020-09-21 09:16:20+02	2020-09-21 09:22:06+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-09-21 09:22:07+02	2020-09-21 09:22:31+02	graphics	libexif	\N	libexif-0.6.21_5
2020-09-21 09:22:31+02	2020-09-21 09:37:38+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-09-21 09:37:39+02	2020-09-21 09:37:52+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-09-21 09:37:53+02	2020-09-21 09:38:26+02	x11-fonts	libXfont2	\N	libXfont2-2.0.4
2020-09-21 09:38:27+02	2020-09-21 09:39:38+02	audio	portaudio	\N	portaudio-19.6.0_5,1
2020-09-21 09:39:39+02	2020-09-21 09:40:03+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.9_7
2020-09-21 09:40:04+02	2020-09-21 09:40:55+02	devel	oniguruma	\N	oniguruma-6.9.5.r1_1
2020-09-21 09:40:55+02	2020-09-21 09:41:36+02	devel	re2c	\N	re2c-0.14.3
2020-09-21 09:41:37+02	2020-09-21 09:42:19+02	graphics	lcms	\N	lcms-1.19_6,1
2020-09-21 09:42:19+02	2020-09-21 09:42:39+02	lang	fpc-rtl-objpas	\N	fpc-rtl-objpas-3.0.4_5
2020-09-21 09:42:39+02	2020-09-21 09:43:06+02	net	p5-Socket6	\N	p5-Socket6-0.29
2020-09-21 09:43:07+02	2020-09-21 09:43:42+02	www	php74-session	\N	php74-session-7.4.10
2020-09-21 09:43:43+02	2020-09-21 09:44:14+02	archivers	php74-zlib	\N	php74-zlib-7.4.10
2020-09-21 09:44:15+02	2020-09-21 09:44:48+02	audio	espeak	\N	espeak-1.48.04_7
2020-09-21 09:44:48+02	2020-09-21 09:45:21+02	databases	php74-pdo	\N	php74-pdo-7.4.10
2020-09-21 09:45:22+02	2020-09-21 09:45:37+02	devel	dotconf	\N	dotconf-1.3_1
2020-09-21 09:45:38+02	2020-09-21 09:45:56+02	devel	fpc-fcl-res	\N	fpc-fcl-res-3.0.4_4
2020-09-21 09:45:57+02	2020-09-21 09:46:28+02	devel	php74-json	\N	php74-json-7.4.10
2020-09-21 09:46:29+02	2020-09-21 09:47:07+02	graphics	libmng	\N	libmng-1.0.10_3
2020-09-21 09:47:07+02	2020-09-21 09:50:25+02	math	fftw3-float	\N	fftw3-float-3.3.8_6
2020-09-21 09:50:26+02	2020-09-21 09:50:41+02	net	p5-IO-Socket-INET6	\N	p5-IO-Socket-INET6-2.72_1
2020-09-21 09:50:42+02	2020-09-21 09:50:55+02	security	libassuan	\N	libassuan-2.5.3
2020-09-21 07:21:49+02	2020-09-21 09:51:05+02	lang	gcc9	\N	gcc9-9.3.0_1
2020-09-21 09:50:56+02	2020-09-21 09:53:59+02	security	libsodium	\N	libsodium-1.0.18
2020-09-21 09:54:00+02	2020-09-21 09:54:19+02	security	p5-Digest-HMAC	\N	p5-Digest-HMAC-1.03_1
2020-09-21 09:54:20+02	2020-09-21 09:55:10+02	security	p5-Net-SSLeay	\N	p5-Net-SSLeay-1.88
2020-09-21 09:55:11+02	2020-09-21 09:55:23+02	sysutils	dmidecode	\N	dmidecode-3.2
2020-09-21 09:55:24+02	2020-09-21 09:56:25+02	sysutils	policykit	\N	policykit-0.9_10
2020-09-21 09:56:26+02	2020-09-21 09:56:45+02	www	p5-HTML-Tagset	\N	p5-HTML-Tagset-3.20_1
2020-09-21 09:56:46+02	2020-09-21 09:57:06+02	www	p5-Mozilla-CA	\N	p5-Mozilla-CA-20200520
2020-09-21 09:57:06+02	2020-09-21 09:57:58+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-09-21 09:57:59+02	2020-09-21 09:58:21+02	archivers	snappy	\N	snappy-1.1.8
2020-09-21 09:58:22+02	2020-09-21 09:59:16+02	converters	php74-mbstring	\N	php74-mbstring-7.4.10
2020-09-21 07:46:47+02	2020-09-21 10:05:36+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-21 09:59:16+02	2020-09-21 10:13:04+02	databases	libmemcached	\N	libmemcached-1.0.18_7
2020-09-21 10:13:05+02	2020-09-21 10:13:26+02	devel	fpc-fcl-base	\N	fpc-fcl-base-3.0.4_4
2020-09-21 10:13:27+02	2020-09-21 10:15:24+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.74.0
2020-09-21 10:15:25+02	2020-09-21 10:15:38+02	devel	libvolume_id	\N	libvolume_id-0.81.1
2020-09-21 10:15:38+02	2020-09-21 10:16:19+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-09-21 10:16:20+02	2020-09-21 10:21:49+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-21 10:21:50+02	2020-09-21 10:22:24+02	graphics	php74-gd	\N	php74-gd-7.4.10
2020-09-21 10:22:24+02	2020-09-21 10:23:22+02	math	metis	\N	metis-5.1.0_8
2020-09-21 10:23:22+02	2020-09-21 10:23:38+02	multimedia	libdvdcss	\N	libdvdcss-1.4.2
2020-09-21 10:23:39+02	2020-09-21 10:24:21+02	security	libmcrypt	\N	libmcrypt-2.5.8_3
2020-09-21 10:24:22+02	2020-09-21 10:24:39+02	security	p5-IO-Socket-SSL	\N	p5-IO-Socket-SSL-2.068
2020-09-21 10:24:40+02	2020-09-21 10:25:06+02	security	php74-openssl	\N	php74-openssl-7.4.10
2020-09-21 09:51:06+02	2020-09-21 10:25:13+02	math	openblas	\N	openblas-0.3.10,1
2020-09-21 10:25:06+02	2020-09-21 10:25:30+02	textproc	php74-xml	\N	php74-xml-7.4.10
2020-09-21 10:25:14+02	2020-09-21 10:25:32+02	www	p5-HTML-Parser	\N	p5-HTML-Parser-3.72
2020-09-21 10:25:30+02	2020-09-21 10:25:43+02	audio	alsa-plugins	\N	alsa-plugins-1.1.1_6
2020-09-21 10:25:44+02	2020-09-21 10:26:06+02	converters	php74-iconv	\N	php74-iconv-7.4.10
2020-09-21 10:25:33+02	2020-09-21 10:26:15+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-21 10:26:07+02	2020-09-21 10:26:31+02	databases	pecl-memcache	php74	php74-pecl-memcache-4.0.5.2
2020-09-21 10:26:15+02	2020-09-21 10:26:42+02	databases	pecl-memcached	php74	php74-pecl-memcached-3.1.5
2020-09-21 10:26:31+02	2020-09-21 10:26:56+02	databases	php74-pdo_pgsql	\N	php74-pdo_pgsql-7.4.10
2020-09-21 10:26:43+02	2020-09-21 10:27:08+02	databases	php74-pdo_sqlite	\N	php74-pdo_sqlite-7.4.10
2020-09-21 10:26:56+02	2020-09-21 10:27:26+02	databases	php74-pgsql	\N	php74-pgsql-7.4.10
2020-09-21 10:27:08+02	2020-09-21 10:27:33+02	databases	php74-sqlite3	\N	php74-sqlite3-7.4.10
2020-09-21 10:27:27+02	2020-09-21 10:27:34+02	devel	libpci	\N	libpci-3.7.0
2020-09-21 10:27:33+02	2020-09-21 10:27:56+02	devel	php74-gettext	\N	php74-gettext-7.4.10
2020-09-21 10:27:57+02	2020-09-21 10:29:17+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-09-21 10:29:17+02	2020-09-21 10:29:51+02	ftp	php74-curl	\N	php74-curl-7.4.10
2020-09-21 10:29:51+02	2020-09-21 10:30:57+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-09-21 10:30:57+02	2020-09-21 10:31:23+02	math	pecl-bitset	php74	php74-pecl-bitset-3.0.1
2020-09-21 10:31:24+02	2020-09-21 10:31:59+02	math	php74-bcmath	\N	php74-bcmath-7.4.10
2020-09-21 10:27:34+02	2020-09-21 10:35:20+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-21 10:35:20+02	2020-09-21 10:35:38+02	multimedia	libdvdread	\N	libdvdread-6.1.0
2020-09-21 10:05:37+02	2020-09-21 10:35:46+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-21 10:35:47+02	2020-09-21 10:36:13+02	net	php74-soap	\N	php74-soap-7.4.10
2020-09-21 10:36:14+02	2020-09-21 10:36:36+02	net	php74-sockets	\N	php74-sockets-7.4.10
2020-09-21 10:36:37+02	2020-09-21 10:36:55+02	security	pecl-mcrypt	php74	php74-pecl-mcrypt-1.0.3
2020-09-21 10:36:56+02	2020-09-21 10:37:49+02	sysutils	hal	\N	hal-0.5.14_34
2020-09-21 10:37:50+02	2020-09-21 10:38:11+02	sysutils	php74-posix	\N	php74-posix-7.4.10
2020-09-21 10:35:38+02	2020-09-21 10:38:22+02	net	php74-ldap	\N	php74-ldap-7.4.10
2020-09-21 10:38:12+02	2020-09-21 10:38:33+02	textproc	php74-ctype	\N	php74-ctype-7.4.10
2020-09-21 10:38:23+02	2020-09-21 10:38:55+02	textproc	php74-dom	\N	php74-dom-7.4.10
2020-09-21 10:38:34+02	2020-09-21 10:39:02+02	textproc	php74-simplexml	\N	php74-simplexml-7.4.10
2020-09-21 10:38:55+02	2020-09-21 10:39:42+02	www	php74-opcache	\N	php74-opcache-7.4.10
2020-09-21 10:39:03+02	2020-09-21 10:40:31+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-21 10:39:43+02	2020-09-21 10:40:36+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-21 10:40:36+02	2020-09-21 10:40:48+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-09-21 10:40:32+02	2020-09-21 10:41:03+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-09-21 10:41:03+02	2020-09-21 10:42:14+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-09-21 10:42:15+02	2020-09-21 10:42:37+02	security	p5-GSSAPI	\N	p5-GSSAPI-0.28_1
2020-09-21 10:40:48+02	2020-09-21 10:42:38+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-09-21 10:42:38+02	2020-09-21 10:42:50+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-09-21 10:42:38+02	2020-09-21 10:42:54+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-09-21 10:42:54+02	2020-09-21 10:43:03+02	devel	cvsps	\N	cvsps-2.1_2
2020-09-21 10:42:50+02	2020-09-21 10:43:42+02	archivers	kf5-karchive	\N	kf5-karchive-5.74.0
2020-09-21 10:43:42+02	2020-09-21 10:44:18+02	devel	libsigc++20	\N	libsigc++-2.10.3
2020-09-21 10:43:04+02	2020-09-21 10:44:32+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-09-21 10:44:18+02	2020-09-21 10:44:59+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-09-21 10:45:00+02	2020-09-21 10:45:08+02	java	javavmwrapper	\N	javavmwrapper-2.7.6
2020-09-21 10:45:08+02	2020-09-21 10:45:27+02	lang	p5-Error	\N	p5-Error-0.17029
2020-09-21 10:45:28+02	2020-09-21 10:46:04+02	misc	e2fsprogs-libuuid	\N	e2fsprogs-libuuid-1.45.6
2020-09-21 10:44:32+02	2020-09-21 10:46:06+02	graphics	ilmbase	\N	ilmbase-2.5.3
2020-09-21 10:46:05+02	2020-09-21 10:46:19+02	multimedia	libdvdnav	\N	libdvdnav-6.1.0
2020-09-21 10:46:06+02	2020-09-21 10:46:23+02	security	p5-Authen-SASL	\N	p5-Authen-SASL-2.16_1
2020-09-21 10:46:19+02	2020-09-21 10:46:35+02	www	p5-CGI	\N	p5-CGI-4.50
2020-09-21 10:46:23+02	2020-09-21 10:47:00+02	www	py-django22	py37	py37-django22-2.2.16
2020-09-21 10:46:35+02	2020-09-21 10:47:50+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-09-21 10:47:51+02	2020-09-21 10:49:49+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-09-21 10:47:00+02	2020-09-21 10:51:38+02	x11-servers	xorg-server	\N	xorg-server-1.20.9,1
2020-09-21 10:51:38+02	2020-09-21 10:53:18+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-09-21 10:53:19+02	2020-09-21 10:53:27+02	devel	libsysinfo	\N	libsysinfo-0.0.3_2
2020-09-21 10:49:49+02	2020-09-21 10:54:17+02	devel	git	\N	git-2.28.0
2020-09-21 10:54:18+02	2020-09-21 10:55:32+02	devel	re2	\N	re2-20200401
2020-09-21 10:55:33+02	2020-09-21 10:58:49+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-09-21 10:58:50+02	2020-09-21 10:59:28+02	devel	yasm	\N	yasm-1.3.0
2020-09-21 10:59:29+02	2020-09-21 11:00:51+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-09-21 11:00:51+02	2020-09-21 11:01:10+02	lang	fpc-rtl-extra	\N	fpc-rtl-extra-3.0.4_4
2020-09-21 11:01:10+02	2020-09-21 11:01:30+02	textproc	p5-XML-NamespaceSupport	\N	p5-XML-NamespaceSupport-1.12
2020-09-21 11:01:30+02	2020-09-21 11:01:50+02	textproc	p5-XML-SAX-Base	\N	p5-XML-SAX-Base-1.09
2020-09-21 11:01:51+02	2020-09-21 11:03:45+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-09-21 11:03:45+02	2020-09-21 11:03:57+02	x11	xbitmaps	\N	xbitmaps-1.1.2
2020-09-21 11:03:58+02	2020-09-21 11:05:07+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-21 11:05:07+02	2020-09-21 11:05:25+02	audio	liba52	\N	liba52-0.7.4_3
2020-09-21 11:05:26+02	2020-09-21 11:05:52+02	audio	libsoxr	\N	libsoxr-0.1.3_2
2020-09-21 10:53:28+02	2020-09-21 11:07:26+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-09-21 11:07:27+02	2020-09-21 11:07:42+02	converters	fpc-iconvenc	\N	fpc-iconvenc-3.0.4_4
2020-09-21 11:05:53+02	2020-09-21 11:09:11+02	audio	webrtc-audio-processing	\N	webrtc-audio-processing-0.3.1_2
2020-09-21 11:09:12+02	2020-09-21 11:09:30+02	devel	jansson	\N	jansson-2.13.1
2020-09-21 11:09:30+02	2020-09-21 11:14:24+02	graphics	openexr	\N	openexr-2.5.3
2020-09-21 11:07:43+02	2020-09-21 11:14:27+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-09-21 11:14:28+02	2020-09-21 11:16:17+02	java	bootstrap-openjdk8	\N	bootstrap-openjdk8-r450802_1
2020-09-21 11:16:17+02	2020-09-21 11:16:25+02	java	java-zoneinfo	\N	java-zoneinfo-2020.a
2020-09-21 11:16:25+02	2020-09-21 11:16:49+02	multimedia	libdca	\N	libdca-0.0.6_1
2020-09-21 11:16:49+02	2020-09-21 11:16:57+02	net	avahi-header	\N	avahi-header-0.7
2020-09-21 11:14:25+02	2020-09-21 11:17:03+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-09-21 11:17:04+02	2020-09-21 11:17:52+02	security	libssh	\N	libssh-0.9.5
2020-09-21 11:17:53+02	2020-09-21 11:18:13+02	textproc	p5-XML-SAX	\N	p5-XML-SAX-1.02
2020-09-21 11:18:13+02	2020-09-21 11:18:30+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-09-21 11:18:30+02	2020-09-21 11:19:14+02	x11-fonts	libXfont	\N	libXfont-1.5.4_2,2
2020-09-21 11:16:57+02	2020-09-21 11:19:39+02	security	gcr	\N	gcr-3.28.0_1
2020-09-21 11:19:14+02	2020-09-21 11:22:35+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-09-21 11:19:39+02	2020-09-21 11:22:46+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-09-21 11:22:36+02	2020-09-21 11:22:54+02	devel	fpc-fcl-xml	\N	fpc-fcl-xml-3.0.4_4
2020-09-21 11:22:46+02	2020-09-21 11:23:32+02	devel	hwloc	\N	hwloc-1.11.11
2020-09-21 11:22:55+02	2020-09-21 11:23:50+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-09-21 11:23:33+02	2020-09-21 11:24:38+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-09-21 11:23:50+02	2020-09-21 11:25:23+02	dns	c-ares	\N	c-ares-1.16.1
2020-09-21 11:24:38+02	2020-09-21 11:27:52+02	graphics	glew	\N	glew-2.2.0
2020-09-21 11:27:53+02	2020-09-21 11:32:10+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-09-21 11:32:10+02	2020-09-21 11:32:53+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-09-21 11:32:53+02	2020-09-21 11:33:48+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-09-21 11:33:49+02	2020-09-21 11:35:06+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-21 11:35:07+02	2020-09-21 11:38:20+02	net	openldap24-sasl-client	\N	openldap-sasl-client-2.4.51
2020-09-21 11:38:21+02	2020-09-21 11:39:27+02	security	munge	\N	munge-0.5.14_1
2020-09-21 11:39:27+02	2020-09-21 11:40:50+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-21 11:40:51+02	2020-09-21 11:42:09+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.74.0
2020-09-21 11:42:10+02	2020-09-21 11:42:31+02	textproc	man2html	\N	man2html-3.0.1_2
2020-09-21 11:42:32+02	2020-09-21 11:42:54+02	textproc	p5-XML-SAX-Expat	\N	p5-XML-SAX-Expat-0.51_1
2020-09-21 11:25:23+02	2020-09-21 11:45:56+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-09-21 11:45:57+02	2020-09-21 11:46:51+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-09-21 11:46:52+02	2020-09-21 11:47:36+02	audio	libmikmod	\N	libmikmod-3.3.11.1_1
2020-09-21 11:47:37+02	2020-09-21 11:48:33+02	audio	mpg123	\N	mpg123-1.26.3
2020-09-21 11:48:34+02	2020-09-21 11:48:53+02	audio	timidity	\N	timidity-0.2i_1
2020-09-21 11:48:53+02	2020-09-21 11:52:03+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-09-21 11:52:04+02	2020-09-21 11:55:41+02	databases	mysql57-client	\N	mysql57-client-5.7.31_1
2020-09-21 11:55:42+02	2020-09-21 11:57:06+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-09-21 11:57:07+02	2020-09-21 11:59:16+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-09-21 11:59:17+02	2020-09-21 11:59:27+02	devel	py-pytz	py27	py27-pytz-2020.1,1
2020-09-21 11:59:28+02	2020-09-21 11:59:39+02	devel	py-six	py27	py27-six-1.15.0
2020-09-21 11:59:39+02	2020-09-21 11:59:50+02	devel	py-zipp	py37	py37-zipp-3.1.0
2020-09-21 11:59:51+02	2020-09-21 12:01:49+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-21 12:01:50+02	2020-09-21 12:01:58+02	devel	tradcpp	\N	tradcpp-0.5.3
2020-09-21 10:32:00+02	2020-09-21 12:02:09+02	math	suitesparse	\N	suitesparse-5.8.1
2020-09-21 12:01:58+02	2020-09-21 12:02:54+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-09-21 12:02:54+02	2020-09-21 12:03:26+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-09-21 12:03:26+02	2020-09-21 12:03:49+02	math	glm	\N	glm-0.9.8.5_4,1
2020-09-21 12:03:49+02	2020-09-21 12:04:45+02	net	remmina	\N	remmina-1.3.9
2020-09-21 11:42:55+02	2020-09-21 12:04:51+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-21 12:04:46+02	2020-09-21 12:04:56+02	security	pinentry-tty	\N	pinentry-tty-1.1.0
2020-09-21 12:02:09+02	2020-09-21 12:05:08+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-09-21 12:04:56+02	2020-09-21 12:06:46+02	textproc	aspell	\N	aspell-0.60.8,1
2020-09-21 12:05:09+02	2020-09-21 12:06:47+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-09-21 12:06:46+02	2020-09-21 12:07:02+02	textproc	p5-XML-Simple	\N	p5-XML-Simple-2.25
2020-09-21 12:07:03+02	2020-09-21 12:07:15+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-09-21 12:06:48+02	2020-09-21 12:07:17+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-09-21 12:07:15+02	2020-09-21 12:07:25+02	x11	xrdb	\N	xrdb-1.2.0
2020-09-21 12:07:17+02	2020-09-21 12:07:41+02	x11-fonts	font-misc-misc	\N	font-misc-misc-1.1.2_4
2020-09-21 12:07:41+02	2020-09-21 12:07:57+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-09-21 12:07:26+02	2020-09-21 12:08:21+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-09-21 12:07:58+02	2020-09-21 12:08:25+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-09-21 12:08:26+02	2020-09-21 12:08:39+02	audio	libcddb	\N	libcddb-1.3.2_4
2020-09-21 12:08:22+02	2020-09-21 12:08:56+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-09-21 12:08:40+02	2020-09-21 12:09:05+02	audio	libmysofa	\N	libmysofa-1.1
2020-09-21 12:09:05+02	2020-09-21 12:09:16+02	devel	gccmakedep	\N	gccmakedep-1.0.3
2020-09-21 12:08:57+02	2020-09-21 12:09:17+02	audio	speex	\N	speex-1.2.0,1
2020-09-21 12:09:17+02	2020-09-21 12:09:28+02	devel	makedepend	\N	makedepend-1.0.6,1
2020-09-21 12:09:28+02	2020-09-21 12:09:37+02	devel	npth	\N	npth-1.6
2020-09-21 12:09:37+02	2020-09-21 12:09:50+02	devel	popt	\N	popt-1.18_1
2020-09-21 12:09:16+02	2020-09-21 12:09:56+02	devel	libgtop	\N	libgtop-2.40.0
2020-09-21 12:04:52+02	2020-09-21 12:10:01+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-09-21 12:09:56+02	2020-09-21 12:10:06+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-1.7.0
2020-09-21 12:09:51+02	2020-09-21 12:10:16+02	devel	py-babel	py27	py27-Babel-2.8.0
2020-09-21 12:10:06+02	2020-09-21 12:10:36+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-09-21 12:10:36+02	2020-09-21 12:11:14+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-09-21 12:11:15+02	2020-09-21 12:11:55+02	graphics	libspiro	\N	libspiro-20190731,1
2020-09-21 12:10:17+02	2020-09-21 12:12:57+02	graphics	exiv2	\N	exiv2-0.27.3,1
2020-09-21 12:12:58+02	2020-09-21 12:13:16+02	lang	lua53	\N	lua53-5.3.5_3
2020-09-21 12:13:16+02	2020-09-21 12:14:23+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-21 12:14:23+02	2020-09-21 12:15:03+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-09-21 12:15:04+02	2020-09-21 12:15:25+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-09-21 12:15:26+02	2020-09-21 12:16:11+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-09-21 12:16:11+02	2020-09-21 12:16:44+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-09-21 12:10:01+02	2020-09-21 12:21:30+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-21 12:21:31+02	2020-09-21 12:24:07+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-21 12:16:44+02	2020-09-21 12:24:43+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-09-21 12:24:08+02	2020-09-21 12:24:44+02	security	libksba	\N	libksba-1.4.0
2020-09-21 12:24:44+02	2020-09-21 12:24:50+02	security	pinentry	\N	pinentry-1.1.0_6
2020-09-21 12:24:44+02	2020-09-21 12:24:53+02	textproc	py-MarkupSafe	py27	py27-MarkupSafe-1.1.1
2020-09-21 12:24:50+02	2020-09-21 12:25:02+02	x11	luit	\N	luit-20190106_1
2020-09-21 12:24:53+02	2020-09-21 12:25:04+02	x11	xorg-cf-files	\N	xorg-cf-files-1.0.6
2020-09-21 12:25:05+02	2020-09-21 12:25:24+02	x11-themes	icon-naming-utils	\N	icon-naming-utils-0.8.90_1
2020-09-21 12:25:03+02	2020-09-21 12:25:34+02	x11-fonts	font-adobe-100dpi	\N	font-adobe-100dpi-1.0.3_4
2020-09-21 12:25:35+02	2020-09-21 12:26:26+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-09-21 12:25:24+02	2020-09-21 12:27:09+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.74.0
2020-09-21 12:26:26+02	2020-09-21 12:27:52+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-09-21 12:27:09+02	2020-09-21 12:28:25+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-09-21 12:27:53+02	2020-09-21 12:29:10+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-21 12:29:10+02	2020-09-21 12:30:13+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-09-21 12:30:13+02	2020-09-21 12:31:42+02	audio	openal-soft	\N	openal-soft-1.20.1_2
2020-09-21 12:17:12+02	2020-09-21 12:31:58+02	net	openmpi	\N	openmpi-4.0.5
2020-09-21 12:31:59+02	2020-09-21 12:32:19+02	devel	autoconf-archive	\N	autoconf-archive-0.2019.01.06
2020-09-21 12:32:19+02	2020-09-21 12:32:40+02	devel	gamin	\N	gamin-0.1.10_10
2020-09-21 12:32:41+02	2020-09-21 12:33:05+02	devel	imake	\N	imake-1.0.8,1
2020-09-21 12:31:43+02	2020-09-21 12:33:16+02	devel	apr1	\N	apr-1.7.0.1.6.1_1
2020-09-21 12:33:17+02	2020-09-21 12:33:33+02	devel	p5-TimeDate	\N	p5-TimeDate-2.33,1
2020-09-21 12:33:34+02	2020-09-21 12:33:53+02	devel	py-Jinja2	py27	py27-Jinja2-2.10.1
2020-09-21 12:33:05+02	2020-09-21 12:33:59+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-09-21 12:33:53+02	2020-09-21 12:34:04+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-09-21 12:33:59+02	2020-09-21 12:34:10+02	devel	py-olefile	py37	py37-olefile-0.46
2020-09-21 12:34:05+02	2020-09-21 12:34:16+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-09-21 12:34:10+02	2020-09-21 12:34:22+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-09-21 12:34:22+02	2020-09-21 12:34:37+02	devel	yajl	\N	yajl-2.1.0
2020-09-21 12:28:25+02	2020-09-21 12:35:24+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-21 12:34:16+02	2020-09-21 12:35:34+02	devel	talloc	\N	talloc-2.3.1
2020-09-21 12:35:24+02	2020-09-21 12:35:36+02	lang	erlang-man	\N	erlang-man-21.3_2
2020-09-21 12:35:34+02	2020-09-21 12:38:01+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-21 12:34:37+02	2020-09-21 12:38:30+02	graphics	freeimage	\N	freeimage-3.18.0_1
2020-09-21 12:38:02+02	2020-09-21 12:38:35+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-21 12:38:30+02	2020-09-21 12:38:46+02	print	gl2ps	\N	gl2ps-1.4.0
2020-09-21 12:38:46+02	2020-09-21 12:39:26+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-09-21 12:39:26+02	2020-09-21 12:39:43+02	textproc	yelp-xsl	\N	yelp-xsl-3.38.0
2020-09-21 12:39:44+02	2020-09-21 12:39:59+02	x11	xauth	\N	xauth-1.1
2020-09-21 12:39:59+02	2020-09-21 12:40:45+02	x11	xterm	\N	xterm-359
2020-09-21 12:38:35+02	2020-09-21 12:40:53+02	security	gnupg	\N	gnupg-2.2.23
2020-09-21 12:40:54+02	2020-09-21 12:41:06+02	x11-fonts	font-alias	\N	font-alias-1.0.4
2020-09-21 12:40:46+02	2020-09-21 12:41:16+02	x11-fonts	font-adobe-75dpi	\N	font-adobe-75dpi-1.0.3_4
2020-09-21 12:41:07+02	2020-09-21 12:42:26+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-09-21 12:41:16+02	2020-09-21 12:43:15+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-09-21 12:43:16+02	2020-09-21 12:43:38+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.9_6
2020-09-21 12:42:27+02	2020-09-21 12:44:08+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-09-21 12:43:39+02	2020-09-21 12:44:11+02	archivers	minizip	\N	minizip-1.2.11
2020-09-21 12:44:12+02	2020-09-21 12:46:30+02	databases	tdb	\N	tdb-1.4.3,1
2020-09-21 12:46:30+02	2020-09-21 12:46:39+02	devel	gnome-common	\N	gnome-common-3.18.0
2020-09-21 12:46:39+02	2020-09-21 12:47:16+02	devel	librest	\N	rest-0.7.93
2020-09-21 12:47:17+02	2020-09-21 12:47:32+02	devel	p5-Parse-Yapp	\N	p5-Parse-Yapp-1.21
2020-09-21 12:47:33+02	2020-09-21 12:47:44+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-09-21 12:47:45+02	2020-09-21 12:48:57+02	devel	tevent	\N	tevent-0.10.2
2020-09-21 12:48:57+02	2020-09-21 12:50:09+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-09-21 12:50:09+02	2020-09-21 12:52:04+02	graphics	netpbm	\N	netpbm-10.91.01
2020-09-21 12:52:05+02	2020-09-21 12:52:58+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-09-21 12:44:09+02	2020-09-21 12:59:27+02	databases	firebird25-client	\N	firebird25-client-2.5.8_6
2020-09-21 12:59:28+02	2020-09-21 12:59:34+02	multimedia	gmp-api	\N	gmp-api-58
2020-09-21 12:59:34+02	2020-09-21 13:02:00+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-09-21 13:02:01+02	2020-09-21 13:06:30+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-09-21 13:06:31+02	2020-09-21 13:10:21+02	net	libmateweather	\N	libmateweather-1.24.0
2020-09-21 13:10:22+02	2020-09-21 13:10:58+02	security	gpgme	\N	gpgme-1.14.0
2020-09-21 13:10:58+02	2020-09-21 13:11:23+02	sysutils	cmocka	\N	cmocka-1.1.5
2020-09-21 13:11:24+02	2020-09-21 13:11:32+02	sysutils	libsunacl	\N	libsunacl-1.0.1
2020-09-21 13:11:32+02	2020-09-21 13:14:29+02	textproc	exempi	\N	exempi-2.2.2_1
2020-09-21 13:14:29+02	2020-09-21 13:16:15+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-09-21 12:52:58+02	2020-09-21 13:17:41+02	lang	erlang	\N	erlang-21.3.8.17,4
2020-09-21 13:17:42+02	2020-09-21 13:18:02+02	www	p5-HTTP-Date	\N	p5-HTTP-Date-6.05
2020-09-21 13:16:16+02	2020-09-21 13:26:54+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-09-21 13:26:55+02	2020-09-21 13:27:11+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-09-21 13:27:11+02	2020-09-21 13:27:31+02	x11-fonts	font-adobe-utopia-100dpi	\N	font-adobe-utopia-100dpi-1.0.4_4
2020-09-21 13:27:31+02	2020-09-21 13:27:50+02	x11-fonts	font-adobe-utopia-75dpi	\N	font-adobe-utopia-75dpi-1.0.4_4
2020-09-21 13:27:51+02	2020-09-21 13:28:03+02	x11-fonts	font-adobe-utopia-type1	\N	font-adobe-utopia-type1-1.0.4_4
2020-09-21 13:28:03+02	2020-09-21 13:28:14+02	x11-fonts	font-arabic-misc	\N	font-arabic-misc-1.0.3_4
2020-09-21 13:28:14+02	2020-09-21 13:28:46+02	x11-fonts	font-bh-100dpi	\N	font-bh-100dpi-1.0.3_4
2020-09-21 13:28:46+02	2020-09-21 13:29:15+02	x11-fonts	font-bh-75dpi	\N	font-bh-75dpi-1.0.3_4
2020-09-21 13:29:16+02	2020-09-21 13:29:34+02	x11-fonts	font-bh-lucidatypewriter-100dpi	\N	font-bh-lucidatypewriter-100dpi-1.0.3_4
2020-09-21 13:29:34+02	2020-09-21 13:29:51+02	x11-fonts	font-bh-lucidatypewriter-75dpi	\N	font-bh-lucidatypewriter-75dpi-1.0.3_4
2020-09-21 13:29:52+02	2020-09-21 13:30:05+02	x11-fonts	font-bh-type1	\N	font-bh-type1-1.0.3_4
2020-09-21 13:30:05+02	2020-09-21 13:30:16+02	x11-fonts	font-bitstream-100dpi	\N	font-bitstream-100dpi-1.0.3_4
2020-09-21 13:30:17+02	2020-09-21 13:30:28+02	x11-fonts	font-bitstream-75dpi	\N	font-bitstream-75dpi-1.0.3_4
2020-09-21 13:30:29+02	2020-09-21 13:30:39+02	x11-fonts	font-bitstream-type1	\N	font-bitstream-type1-1.0.3_4
2020-09-21 13:30:40+02	2020-09-21 13:30:51+02	x11-fonts	font-cronyx-cyrillic	\N	font-cronyx-cyrillic-1.0.3_4
2020-09-21 13:30:52+02	2020-09-21 13:31:03+02	x11-fonts	font-cursor-misc	\N	font-cursor-misc-1.0.3_4
2020-09-21 13:31:04+02	2020-09-21 13:31:17+02	x11-fonts	font-daewoo-misc	\N	font-daewoo-misc-1.0.3_4
2020-09-21 13:31:17+02	2020-09-21 13:31:28+02	x11-fonts	font-dec-misc	\N	font-dec-misc-1.0.3_4
2020-09-21 13:31:28+02	2020-09-21 13:31:40+02	x11-fonts	font-ibm-type1	\N	font-ibm-type1-1.0.3_4
2020-09-21 13:31:40+02	2020-09-21 13:31:52+02	x11-fonts	font-isas-misc	\N	font-isas-misc-1.0.3_4
2020-09-24 04:54:20+02	2020-09-24 04:54:28+02	sysutils	pv	\N	pv-1.6.6
2020-09-21 13:31:52+02	2020-09-21 13:32:03+02	x11-fonts	font-jis-misc	\N	font-jis-misc-1.0.3_4
2020-09-21 13:32:04+02	2020-09-21 13:32:15+02	x11-fonts	font-micro-misc	\N	font-micro-misc-1.0.3_4
2020-09-21 13:32:15+02	2020-09-21 13:32:26+02	x11-fonts	font-misc-cyrillic	\N	font-misc-cyrillic-1.0.3_4
2020-09-21 13:32:27+02	2020-09-21 13:32:38+02	x11-fonts	font-mutt-misc	\N	font-mutt-misc-1.0.3_4
2020-09-21 13:32:39+02	2020-09-21 13:32:52+02	x11-fonts	font-schumacher-misc	\N	font-schumacher-misc-1.1.2_4
2020-09-21 13:32:52+02	2020-09-21 13:33:03+02	x11-fonts	font-screen-cyrillic	\N	font-screen-cyrillic-1.0.4_4
2020-09-21 13:33:04+02	2020-09-21 13:33:14+02	x11-fonts	font-sony-misc	\N	font-sony-misc-1.0.3_4
2020-09-21 13:33:15+02	2020-09-21 13:33:26+02	x11-fonts	font-sun-misc	\N	font-sun-misc-1.0.3_4
2020-09-21 13:33:26+02	2020-09-21 13:33:37+02	x11-fonts	font-winitzki-cyrillic	\N	font-winitzki-cyrillic-1.0.3_4
2020-09-21 13:33:37+02	2020-09-21 13:33:48+02	x11-fonts	font-xfree86-type1	\N	font-xfree86-type1-1.0.4_4
2020-09-21 13:33:48+02	2020-09-21 13:35:13+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-21 13:35:13+02	2020-09-21 13:36:38+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-09-21 13:36:38+02	2020-09-21 13:37:07+02	audio	celt	\N	celt-0.11.3_3
2020-09-21 13:37:09+02	2020-09-21 13:37:38+02	audio	libmad	\N	libmad-0.15.1b_7
2020-09-21 13:37:38+02	2020-09-21 13:38:16+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-09-21 13:38:17+02	2020-09-21 13:46:18+02	cad	ngspice_rework	shlib	ngspice_rework-shlib-32
2020-09-21 13:46:19+02	2020-09-21 13:46:38+02	converters	p5-Encode-Locale	\N	p5-Encode-Locale-1.05
2020-09-21 13:46:39+02	2020-09-21 13:46:54+02	databases	fpc-dblib	\N	fpc-dblib-3.0.4_4
2020-09-21 13:46:55+02	2020-09-21 13:47:14+02	databases	fpc-ibase	\N	fpc-ibase-3.0.4_5
2020-09-21 13:47:15+02	2020-09-21 13:47:50+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-09-21 13:47:50+02	2020-09-21 13:48:06+02	databases	fpc-odbc	\N	fpc-odbc-3.0.4_4
2020-09-21 13:48:07+02	2020-09-21 13:48:24+02	databases	fpc-oracle	\N	fpc-oracle-3.0.4_4
2020-09-21 13:48:25+02	2020-09-21 13:48:42+02	databases	fpc-postgres	\N	fpc-postgres-3.0.4_6
2020-09-21 13:48:42+02	2020-09-21 13:48:58+02	databases	fpc-pxlib	\N	fpc-pxlib-3.0.4_4
2020-09-21 13:48:59+02	2020-09-21 13:49:18+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-09-21 13:49:18+02	2020-09-21 13:49:34+02	devel	fpc-fcl-async	\N	fpc-fcl-async-3.0.4_4
2020-09-21 13:49:35+02	2020-09-21 13:49:53+02	devel	fpc-fcl-json	\N	fpc-fcl-json-3.0.4_5
2020-09-21 13:49:53+02	2020-09-21 13:50:16+02	devel	fpc-fcl-passrc	\N	fpc-fcl-passrc-3.0.4_5
2020-09-21 13:50:17+02	2020-09-21 13:51:36+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-09-21 13:51:36+02	2020-09-21 13:52:23+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-21 13:52:23+02	2020-09-21 13:52:38+02	devel	netsurf-buildsystem	\N	netsurf-buildsystem-1.8
2020-09-21 13:52:39+02	2020-09-21 13:52:54+02	devel	p5-Clone	\N	p5-Clone-0.45
2020-09-21 13:52:55+02	2020-09-21 13:53:10+02	devel	p5-IO-HTML	\N	p5-IO-HTML-1.001_1
2020-09-21 13:53:10+02	2020-09-21 13:53:21+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-09-21 13:53:22+02	2020-09-21 13:53:33+02	devel	py-pycparser	py27	py27-pycparser-2.20
2020-09-21 12:35:36+02	2020-09-21 13:53:37+02	math	vtk8	\N	vtk8-8.2.0
2020-09-21 13:53:34+02	2020-09-21 14:08:11+02	multimedia	assimp	\N	assimp-5.0.1
2020-09-21 14:08:12+02	2020-09-21 14:12:35+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-09-21 14:12:35+02	2020-09-21 14:13:38+02	multimedia	openh264	\N	openh264-2.1.1,2
2020-09-21 14:13:39+02	2020-09-21 14:16:22+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-09-21 14:16:23+02	2020-09-21 14:19:48+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-21 14:19:49+02	2020-09-21 14:20:10+02	net	liboauth	\N	liboauth-1.0.3_4
2020-09-21 14:20:10+02	2020-09-21 14:20:49+02	net	norm	\N	norm-1.5r6_1
2020-09-21 14:20:49+02	2020-09-21 14:22:11+02	net	openpgm	\N	openpgm-5.2.122_6
2020-09-21 14:22:11+02	2020-09-21 14:54:39+02	net	samba410	\N	samba410-4.10.18
2020-09-21 14:54:40+02	2020-09-21 14:55:56+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-09-21 14:55:56+02	2020-09-21 14:57:23+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-21 14:57:23+02	2020-09-21 14:57:41+02	security	fpc-openssl	\N	fpc-openssl-3.0.4_4
2020-09-21 14:57:42+02	2020-09-21 14:58:44+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-09-21 14:58:45+02	2020-09-21 14:59:04+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7_1
2020-09-21 14:59:05+02	2020-09-21 15:00:03+02	sysutils	fusefs-libs	\N	fusefs-libs-2.9.9_1
2020-09-21 15:00:04+02	2020-09-21 15:01:11+02	sysutils	libcdio	\N	libcdio-2.1.0
2020-09-21 15:01:11+02	2020-09-21 15:02:26+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-21 15:02:26+02	2020-09-21 15:03:28+02	sysutils	upower	\N	upower-0.99.11
2020-09-21 15:03:28+02	2020-09-21 15:04:28+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-09-21 15:04:28+02	2020-09-21 15:04:48+02	www	p5-LWP-MediaTypes	\N	p5-LWP-MediaTypes-6.04
2020-09-21 13:53:38+02	2020-09-21 16:10:26+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-21 16:10:27+02	2020-09-21 16:10:43+02	x11	appres	\N	appres-1.0.5
2020-09-21 16:10:43+02	2020-09-21 16:11:02+02	x11	bitmap	\N	bitmap-1.0.9
2020-09-21 16:11:02+02	2020-09-21 16:11:15+02	x11	iceauth	\N	iceauth-1.0.8_2
2020-09-21 16:11:16+02	2020-09-21 16:13:08+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-09-21 16:13:09+02	2020-09-21 16:13:42+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-09-21 16:13:43+02	2020-09-21 16:18:20+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-21 16:18:21+02	2020-09-21 16:18:35+02	x11	sessreg	\N	sessreg-1.1.2
2020-09-21 16:18:36+02	2020-09-21 16:18:50+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-09-21 16:18:51+02	2020-09-21 16:19:05+02	x11	smproxy	\N	smproxy-1.0.6
2020-09-21 16:19:06+02	2020-09-21 16:19:25+02	x11	x11perf	\N	x11perf-1.6.1
2020-09-21 16:19:25+02	2020-09-21 16:19:40+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-09-21 16:19:41+02	2020-09-21 16:20:07+02	x11	xcalc	\N	xcalc-1.1.0
2020-09-21 16:20:07+02	2020-09-21 16:20:23+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-09-21 16:20:24+02	2020-09-21 16:20:39+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-09-21 16:20:39+02	2020-09-21 16:20:54+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-09-21 16:20:55+02	2020-09-21 16:21:11+02	x11	xev	\N	xev-1.2.4
2020-09-21 16:21:12+02	2020-09-21 16:21:24+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-09-21 16:21:25+02	2020-09-21 16:21:38+02	x11	xgamma	\N	xgamma-1.0.6
2020-09-21 16:21:38+02	2020-09-21 16:21:57+02	x11	xgc	\N	xgc-1.0.5
2020-09-21 16:21:58+02	2020-09-21 16:22:13+02	x11	xhost	\N	xhost-1.0.8
2020-09-21 16:22:14+02	2020-09-21 16:22:29+02	x11	xinit	\N	xinit-1.4.1,1
2020-09-21 16:22:29+02	2020-09-21 16:22:46+02	x11	xinput	\N	xinput-1.6.3
2020-09-21 16:22:48+02	2020-09-21 16:23:04+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-09-21 16:23:05+02	2020-09-21 16:23:22+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-09-21 16:23:22+02	2020-09-21 16:23:36+02	x11	xkill	\N	xkill-1.0.5
2020-09-21 16:23:37+02	2020-09-21 16:23:50+02	x11	xlsatoms	\N	xlsatoms-1.1.3
2020-09-21 16:23:50+02	2020-09-21 16:24:07+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-09-21 16:24:08+02	2020-09-21 16:24:22+02	x11	xmessage	\N	xmessage-1.0.5
2020-09-21 16:24:23+02	2020-09-21 16:24:40+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-09-21 16:24:41+02	2020-09-21 16:24:57+02	x11	xpr	\N	xpr-1.0.5
2020-09-21 16:24:57+02	2020-09-21 16:25:17+02	x11	xrandr	\N	xrandr-1.5.1
2020-09-21 16:25:18+02	2020-09-21 16:25:31+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-09-21 16:25:32+02	2020-09-21 16:25:46+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-09-21 16:25:47+02	2020-09-21 16:26:00+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-09-21 16:26:01+02	2020-09-21 16:26:16+02	x11	xwd	\N	xwd-1.0.7
2020-09-21 16:26:16+02	2020-09-21 16:26:32+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-09-21 16:26:33+02	2020-09-21 16:26:50+02	x11	xwud	\N	xwud-1.0.5
2020-09-21 16:26:51+02	2020-09-21 16:27:07+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-09-21 16:27:08+02	2020-09-21 16:28:26+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-09-21 16:28:27+02	2020-09-21 16:29:33+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-09-25 04:06:30+02	2020-09-25 04:06:43+02	x11	xgc	\N	xgc-1.0.5
2020-09-21 16:29:33+02	2020-09-21 16:30:47+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-09-21 16:30:48+02	2020-09-21 16:31:56+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-09-21 16:31:57+02	2020-09-21 16:33:14+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-09-21 16:33:15+02	2020-09-21 16:36:25+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-09-21 16:36:26+02	2020-09-21 16:37:34+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-09-21 16:37:34+02	2020-09-21 16:38:46+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.5.0
2020-09-21 16:38:47+02	2020-09-21 16:39:10+02	x11-fonts	libFS	\N	libFS-1.0.8
2020-09-21 16:39:11+02	2020-09-21 16:39:48+02	x11-fonts	xorg-fonts-100dpi	\N	xorg-fonts-100dpi-7.7
2020-09-21 16:39:49+02	2020-09-21 16:40:23+02	x11-fonts	xorg-fonts-75dpi	\N	xorg-fonts-75dpi-7.7
2020-09-21 16:40:24+02	2020-09-21 16:40:41+02	x11-fonts	xorg-fonts-cyrillic	\N	xorg-fonts-cyrillic-7.7
2020-09-21 16:40:42+02	2020-09-21 16:41:28+02	x11-fonts	xorg-fonts-miscbitmaps	\N	xorg-fonts-miscbitmaps-7.7
2020-09-21 16:41:29+02	2020-09-21 16:41:50+02	x11-fonts	xorg-fonts-type1	\N	xorg-fonts-type1-7.7
2020-09-21 13:18:02+02	2020-09-21 16:43:10+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-21 16:41:50+02	2020-09-21 16:44:05+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-09-21 16:44:06+02	2020-09-21 16:44:39+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-09-21 16:43:12+02	2020-09-21 16:44:43+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-09-21 16:44:44+02	2020-09-21 16:45:00+02	x11-wm	twm	\N	twm-1.0.11
2020-09-21 16:45:01+02	2020-09-21 16:46:15+02	archivers	gcab	\N	gcab-1.3
2020-09-21 16:46:18+02	2020-09-21 16:53:10+02	audio	jack	\N	jackit-0.125.0_10
2020-09-21 16:53:13+02	2020-09-21 16:58:37+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-09-21 16:58:39+02	2020-09-21 16:59:09+02	converters	py-webencodings	py27	py27-webencodings-0.5.1
2020-09-21 16:59:12+02	2020-09-21 17:03:04+02	devel	boehm-gc-threaded	\N	boehm-gc-threaded-8.0.4_1
2020-09-21 17:03:07+02	2020-09-21 17:08:30+02	devel	cppunit	\N	cppunit-1.14.0_8
2020-09-21 17:08:33+02	2020-09-21 17:10:21+02	devel	fpc-fcl-db	\N	fpc-fcl-db-3.0.4_6
2020-09-21 17:10:24+02	2020-09-21 17:11:04+02	devel	fpc-fcl-fpcunit	\N	fpc-fcl-fpcunit-3.0.4_4
2020-09-21 17:11:07+02	2020-09-21 17:11:30+02	devel	fpc-fcl-net	\N	fpc-fcl-net-3.0.4_4
2020-09-21 16:44:40+02	2020-09-21 17:12:20+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-21 17:11:30+02	2020-09-21 17:12:21+02	devel	gflags	\N	gflags-2.2.2_2
2020-09-21 17:12:22+02	2020-09-21 17:14:18+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-09-21 17:14:19+02	2020-09-21 17:18:36+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-21 17:18:37+02	2020-09-21 17:19:22+02	devel	m17n-db	\N	m17n-db-1.8.0
2020-09-21 17:19:22+02	2020-09-21 17:19:42+02	devel	py-cffi	py27	py27-cffi-1.14.3
2020-09-21 17:19:43+02	2020-09-21 17:20:01+02	devel	scons	py37	scons-py37-3.1.2
2020-09-21 17:20:02+02	2020-09-21 17:20:19+02	graphics	fpc-pasjpeg	\N	fpc-pasjpeg-3.0.4_4
2020-09-21 17:20:19+02	2020-09-21 17:26:28+02	graphics	leptonica	\N	leptonica-1.76.0_1
2020-09-21 17:26:29+02	2020-09-21 17:28:00+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-09-21 17:28:01+02	2020-09-21 17:28:21+02	graphics	libnsgif	\N	libnsgif-0.2.1
2020-09-21 17:28:21+02	2020-09-21 17:32:02+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-09-21 17:32:02+02	2020-09-21 18:06:14+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-09-21 18:06:14+02	2020-09-21 18:06:38+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-09-21 18:06:39+02	2020-09-21 18:25:34+02	graphics	tesseract-data	\N	tesseract-data-4.0.0
2020-09-21 18:25:35+02	2020-09-21 18:30:50+02	lang	python38	\N	python38-3.8.5
2020-09-21 18:30:51+02	2020-09-21 18:33:20+02	net	libzmq4	\N	libzmq4-4.3.1_1
2020-09-21 18:33:20+02	2020-09-21 18:34:06+02	print	enscript-a4	\N	enscript-a4-1.6.6_3
2020-09-21 18:34:07+02	2020-09-21 18:34:34+02	print	libotf	\N	libotf-0.9.16
2020-09-21 18:34:34+02	2020-09-21 18:36:49+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-09-21 18:36:49+02	2020-09-21 18:37:47+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-21 18:37:47+02	2020-09-21 18:38:47+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-09-21 18:38:48+02	2020-09-21 18:40:03+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-09-21 18:40:04+02	2020-09-21 18:41:13+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-21 18:41:14+02	2020-09-21 18:41:55+02	sysutils	libcdio-paranoia	\N	libcdio-paranoia-10.2+2.0.1
2020-09-21 18:41:55+02	2020-09-21 18:44:50+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-09-21 18:44:50+02	2020-09-21 18:45:04+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-09-21 18:45:05+02	2020-09-21 18:45:28+02	textproc	tinyxml	\N	tinyxml-2.6.2_4
2020-09-21 18:45:29+02	2020-09-21 18:45:44+02	www	fpc-fastcgi	\N	fpc-fastcgi-3.0.4_4
2020-09-21 18:45:45+02	2020-09-21 18:46:04+02	www	fpc-httpd22	\N	fpc-httpd22-3.0.4_4
2020-09-21 18:46:04+02	2020-09-21 18:46:21+02	www	fpc-httpd24	\N	fpc-httpd24-3.0.4_4
2020-09-21 17:12:21+02	2020-09-21 18:55:38+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-21 18:55:39+02	2020-09-21 18:56:00+02	www	p5-HTTP-Message	\N	p5-HTTP-Message-6.26
2020-09-21 18:56:02+02	2020-09-21 18:57:08+02	x11	babl	\N	babl-0.1.78
2020-09-21 18:57:09+02	2020-09-21 18:57:30+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-09-21 18:57:30+02	2020-09-21 18:59:03+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-21 18:59:04+02	2020-09-21 18:59:34+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-09-21 18:59:35+02	2020-09-21 18:59:49+02	x11	xorg-docs	\N	xorg-docs-1.7.1,1
2020-09-21 18:59:50+02	2020-09-21 19:00:02+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-09-21 19:00:03+02	2020-09-21 19:01:14+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-21 19:01:14+02	2020-09-21 19:05:44+02	x11-fm	caja	\N	caja-1.22.3_1
2020-09-21 19:05:45+02	2020-09-21 19:08:05+02	x11-fonts	xorg-fonts	\N	xorg-fonts-7.7_1
2020-09-21 19:08:06+02	2020-09-21 19:12:34+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.9,1
2020-09-21 19:12:35+02	2020-09-21 19:12:51+02	x11-themes	xcursor-themes	\N	xcursor-themes-1.0.6
2020-09-21 19:12:52+02	2020-09-21 19:26:06+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-21 19:26:07+02	2020-09-21 19:29:07+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-09-21 19:29:07+02	2020-09-21 19:30:15+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-21 19:30:16+02	2020-09-21 19:31:47+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-21 19:31:48+02	2020-09-21 19:32:09+02	archivers	unzip	\N	unzip-6.0_8
2020-09-21 19:32:10+02	2020-09-21 19:32:24+02	audio	gsm	\N	gsm-1.0.13_2
2020-09-21 19:32:24+02	2020-09-21 19:33:21+02	audio	lame	\N	lame-3.100_2
2020-09-21 19:33:21+02	2020-09-21 19:33:41+02	audio	libao	\N	libao-1.2.0_5
2020-09-21 19:33:41+02	2020-09-21 19:33:59+02	audio	libid3tag	\N	libid3tag-0.15.1b_2
2020-09-21 19:34:00+02	2020-09-21 19:34:56+02	audio	opusfile	\N	opusfile-0.12
2020-09-21 19:34:56+02	2020-09-21 19:36:49+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-21 19:36:49+02	2020-09-21 19:43:04+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-09-21 19:43:05+02	2020-09-21 19:43:26+02	devel	fpc-fcl-image	\N	fpc-fcl-image-3.0.4_4
2020-09-21 19:43:26+02	2020-09-21 19:44:14+02	devel	fpc-fcl-web	\N	fpc-fcl-web-3.0.4_7
2020-09-21 19:44:15+02	2020-09-21 19:45:41+02	devel	glog	\N	glog-0.4.0_1
2020-09-21 19:45:41+02	2020-09-21 19:49:27+02	devel	glslang	\N	glslang-8.13.3743
2020-09-21 19:49:27+02	2020-09-21 19:53:18+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-21 19:53:20+02	2020-09-21 19:55:10+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-09-21 19:55:10+02	2020-09-21 19:58:31+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-09-21 19:58:32+02	2020-09-21 19:59:06+02	devel	liblangtag	\N	liblangtag-0.6.2
2020-09-21 19:59:07+02	2020-09-21 20:01:37+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-09-21 20:01:37+02	2020-09-21 20:01:45+02	devel	mdds	\N	mdds-1.5.0
2020-09-21 20:01:45+02	2020-09-21 20:02:06+02	devel	p5-IO-String	\N	p5-IO-String-1.08_1
2020-09-21 20:02:06+02	2020-09-21 20:02:20+02	devel	py-click	py37	py37-click-7.0
2020-09-25 04:06:40+02	2020-09-25 04:06:52+02	x11	xhost	\N	xhost-1.0.8
2020-09-21 20:02:20+02	2020-09-21 20:02:35+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-09-21 20:02:36+02	2020-09-21 20:02:50+02	devel	py-pluggy	py37	py37-pluggy-0.13.1
2020-09-21 20:02:51+02	2020-09-21 20:03:06+02	devel	py-ply	py37	py37-ply-3.11
2020-09-21 20:03:07+02	2020-09-21 20:03:19+02	devel	py-ply	py27	py27-ply-3.11
2020-09-21 20:03:19+02	2020-09-21 20:03:34+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-09-21 20:03:34+02	2020-09-21 20:03:53+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-09-21 20:03:53+02	2020-09-21 20:04:06+02	devel	rubygem-mustache	\N	rubygem-mustache-1.1.1
2020-09-21 20:04:07+02	2020-09-21 20:25:51+02	devel	spirv-tools	\N	spirv-tools-2020.2
2020-09-21 20:25:52+02	2020-09-21 20:26:43+02	devel	tex-libtexlua	\N	tex-libtexlua-5.2.4
2020-09-21 20:26:43+02	2020-09-21 20:28:38+02	devel	tex-libtexluajit	\N	tex-libtexluajit-2.0.3
2020-09-21 20:28:38+02	2020-09-21 20:29:38+02	devel	yaml-cpp03	\N	yaml-cpp03-0.3.0_2
2020-09-21 18:46:21+02	2020-09-21 20:30:49+02	www	node	\N	node-14.11.0
2020-09-21 20:30:50+02	2020-09-21 20:33:15+02	graphics	libcdr01	\N	libcdr01-0.1.6
2020-09-21 20:33:17+02	2020-09-21 20:33:46+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-09-21 20:29:39+02	2020-09-21 20:33:53+02	graphics	gegl	\N	gegl-0.4.26_5
2020-09-21 20:33:53+02	2020-09-21 20:34:17+02	graphics	mypaint-brushes	\N	mypaint-brushes-1.3.1
2020-09-21 20:33:46+02	2020-09-21 20:34:26+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-09-21 20:34:18+02	2020-09-21 20:34:49+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-09-21 20:34:27+02	2020-09-21 20:34:59+02	graphics	simage	\N	simage-1.7.0_7
2020-09-21 20:34:59+02	2020-09-21 20:39:36+02	mail	mailutils	\N	mailutils-3.10
2020-09-21 20:39:37+02	2020-09-21 20:40:38+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0,1
2020-09-21 20:40:38+02	2020-09-21 20:42:36+02	math	qhull	\N	qhull-7.2.1,1
2020-09-21 20:42:37+02	2020-09-21 20:42:49+02	misc	compat12x	\N	compat12x-amd64-12.1.1201000.20200220
2020-09-21 20:42:50+02	2020-09-21 20:43:15+02	misc	fpc-chm	\N	fpc-chm-3.0.4_4
2020-09-21 20:43:16+02	2020-09-21 20:43:25+02	misc	usbids	\N	usbids-20200826
2020-09-21 20:43:25+02	2020-09-21 20:43:41+02	multimedia	librtmp	\N	librtmp-2.4.20190330
2020-09-21 20:43:41+02	2020-09-21 20:44:03+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-09-21 20:44:03+02	2020-09-21 20:48:54+02	print	qpdf	\N	qpdf-10.0.1
2020-09-21 20:34:50+02	2020-09-21 20:51:21+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-09-21 20:48:55+02	2020-09-21 20:51:52+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-21 20:51:21+02	2020-09-21 20:52:19+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-09-21 20:51:53+02	2020-09-21 20:52:34+02	sysutils	flock	\N	flock-2.30.1
2020-09-21 20:52:35+02	2020-09-21 20:53:20+02	textproc	libuninameslist	\N	libuninameslist-20190701
2020-09-21 20:52:19+02	2020-09-21 20:54:28+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-09-21 20:54:29+02	2020-09-21 20:54:43+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-09-21 20:54:44+02	2020-09-21 20:55:10+02	textproc	rubygem-rdiscount	\N	rubygem-rdiscount-2.2.0.1
2020-09-21 20:55:10+02	2020-09-21 20:55:20+02	textproc	utf8proc	\N	utf8proc-2.5.0
2020-09-21 20:55:21+02	2020-09-21 20:55:38+02	textproc	yelp-tools	\N	yelp-tools-3.38.0
2020-09-21 20:55:38+02	2020-09-21 20:55:51+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-09-21 20:53:21+02	2020-09-21 20:56:04+02	textproc	libvisio01	\N	libvisio01-0.1.7_2
2020-09-21 20:55:52+02	2020-09-21 20:56:05+02	www	py-html5lib	py27	py27-html5lib-1.0.1
2020-09-21 20:56:05+02	2020-09-21 20:56:15+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-09-21 20:56:05+02	2020-09-21 20:56:19+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-09-21 20:56:16+02	2020-09-21 20:57:08+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-09-21 20:56:20+02	2020-09-21 20:57:19+02	www	rubygem-hpricot	\N	rubygem-hpricot-0.8.6
2020-09-21 20:57:09+02	2020-09-21 20:58:13+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-09-21 20:58:13+02	2020-09-21 20:59:22+02	x11	zenity	\N	zenity-3.32.0
2020-09-21 20:59:22+02	2020-09-21 20:59:29+02	x11-fonts	gnu-unifont	\N	gnu-unifont-13.0.03,1
2020-09-21 20:57:20+02	2020-09-21 21:00:19+02	x11	xorg	\N	xorg-7.7_3
2020-09-21 20:59:30+02	2020-09-21 21:00:22+02	x11-fonts	noto-basic	\N	noto-basic-2.0_1
2020-09-21 21:00:22+02	2020-09-21 21:01:35+02	archivers	gtar	\N	gtar-1.32
2020-09-21 21:00:20+02	2020-09-21 21:01:38+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-09-21 21:01:36+02	2020-09-21 21:02:04+02	audio	libmodplug	\N	libmodplug-0.8.9.0
2020-09-21 21:02:05+02	2020-09-21 21:02:17+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-09-21 21:02:17+02	2020-09-21 21:02:35+02	databases	py-redis	py37	py37-redis-3.5.3
2020-09-21 21:01:39+02	2020-09-21 21:03:24+02	audio	sox	\N	sox-14.4.2_4
2020-09-21 21:02:35+02	2020-09-21 21:03:26+02	devel	apache-ant	\N	apache-ant-1.10.8
2020-09-21 21:03:27+02	2020-09-21 21:03:40+02	devel	fpc-regexpr	\N	fpc-regexpr-3.0.4_4
2020-09-21 21:03:25+02	2020-09-21 21:03:58+02	devel	fpc-fppkg	\N	fpc-fppkg-3.0.4_6
2020-09-21 21:03:58+02	2020-09-21 21:05:18+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-21 21:05:19+02	2020-09-21 21:05:51+02	devel	physfs	\N	physfs-3.0.2
2020-09-21 21:05:52+02	2020-09-21 21:06:08+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-09-21 21:06:09+02	2020-09-21 21:06:22+02	devel	py-appdirs	py37	py37-appdirs-1.4.4
2020-09-21 21:06:22+02	2020-09-21 21:06:36+02	devel	py-asn1crypto	py27	py27-asn1crypto-1.3.0
2020-09-21 21:06:36+02	2020-09-21 21:06:49+02	devel	py-backports	py37	py37-backports-1
2020-09-21 21:06:50+02	2020-09-21 21:07:06+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-09-21 21:07:07+02	2020-09-21 21:07:23+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-09-21 21:07:24+02	2020-09-21 21:07:35+02	devel	py-enum34	py27	py27-enum34-1.1.6
2020-09-21 21:07:37+02	2020-09-21 21:08:32+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-09-21 21:08:32+02	2020-09-21 21:08:45+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-09-21 21:08:45+02	2020-09-21 21:09:01+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-09-21 21:09:02+02	2020-09-21 21:09:18+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-09-21 21:09:18+02	2020-09-21 21:09:30+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-09-21 21:09:31+02	2020-09-21 21:09:51+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-09-21 21:09:51+02	2020-09-21 21:10:02+02	devel	py-wheel	py27	py27-wheel-0.30.0_1
2020-09-21 21:10:03+02	2020-09-21 21:10:19+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-09-21 21:10:20+02	2020-09-21 21:13:28+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-09-21 21:03:41+02	2020-09-21 21:16:37+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-09-21 21:13:29+02	2020-09-21 21:16:54+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-09-21 21:16:38+02	2020-09-21 21:19:15+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-09-21 21:19:15+02	2020-09-21 21:19:31+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-09-21 21:19:31+02	2020-09-21 21:19:50+02	devel	xxhash	\N	xxhash-0.8.0
2020-09-21 21:19:51+02	2020-09-21 21:25:30+02	dns	bind-tools	\N	bind-tools-9.16.6
2020-09-21 21:16:54+02	2020-09-21 21:25:43+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-09-21 21:25:44+02	2020-09-21 21:26:01+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-09-21 21:25:31+02	2020-09-21 21:33:05+02	editors	emacs	full	emacs-27.1,3
2020-09-21 21:33:06+02	2020-09-21 21:33:34+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-09-21 21:26:01+02	2020-09-21 21:38:06+02	graphics	Coin	\N	Coin-4.0.0_1
2020-09-21 21:38:06+02	2020-09-21 21:39:51+02	graphics	glfw	\N	glfw-3.3.2
2020-09-21 21:39:53+02	2020-09-21 21:41:09+02	graphics	libfreehand	\N	libfreehand-0.1.2_18
2020-09-21 21:41:10+02	2020-09-21 21:45:47+02	graphics	movit	\N	movit-1.6.2_2
2020-09-21 21:45:48+02	2020-09-21 21:51:23+02	graphics	opencolorio	\N	opencolorio-1.1.1_1
2020-09-21 21:33:36+02	2020-09-21 21:57:03+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-21 21:57:04+02	2020-09-21 21:57:18+02	graphics	potrace	\N	potrace-1.16
2020-09-21 21:57:19+02	2020-09-21 21:58:07+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-09-21 21:58:07+02	2020-09-21 21:59:16+02	graphics	shaderc	\N	shaderc-2020.0
2020-09-21 21:59:17+02	2020-09-21 21:59:50+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-09-21 21:59:50+02	2020-09-21 22:03:41+02	java	bootstrap-openjdk11	\N	bootstrap-openjdk11-11.0.5.10.1
2020-09-21 21:51:24+02	2020-09-21 22:33:55+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-21 22:03:41+02	2020-09-21 22:39:13+02	lang	gcc6-aux	\N	gcc6-aux-20180516_1,1
2020-09-21 22:39:14+02	2020-09-21 22:41:48+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-21 22:41:49+02	2020-09-21 22:49:10+02	math	py-matplotlib	py37	py37-matplotlib-3.3.1
2020-09-21 22:49:10+02	2020-09-21 22:50:54+02	multimedia	gavl	\N	gavl-1.4.0_4
2020-09-21 22:50:55+02	2020-09-21 22:55:01+02	multimedia	libde265	\N	libde265-1.0.2_5
2020-09-21 22:55:02+02	2020-09-21 22:55:11+02	multimedia	rtmpdump	\N	rtmpdump-2.4.20190330
2020-09-21 22:55:12+02	2020-09-21 22:55:36+02	multimedia	vid.stab	\N	vid.stab-0.98.2
2020-09-21 22:55:36+02	2020-09-21 22:56:10+02	net	gssdp	\N	gssdp-1.2.3
2020-09-21 22:56:11+02	2020-09-21 22:57:11+02	net	libvncserver	\N	libvncserver-0.9.12_2
2020-09-21 22:57:11+02	2020-09-21 22:57:23+02	net	py-ipaddress	py27	py27-ipaddress-1.0.23
2020-09-21 22:57:23+02	2020-09-21 22:58:33+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-09-21 22:58:33+02	2020-09-21 22:59:00+02	print	adobe-cmaps	\N	adobe-cmaps-20051217_4
2020-09-21 22:59:00+02	2020-09-21 23:01:11+02	print	cups-filters	\N	cups-filters-1.28.1_1
2020-09-21 23:01:12+02	2020-09-21 23:12:20+02	print	fontforge	\N	fontforge-20190801_2
2020-09-21 23:12:21+02	2020-09-21 23:13:31+02	print	libmspub01	\N	libmspub01-0.1.4_16
2020-09-21 23:13:32+02	2020-09-21 23:14:23+02	print	libpagemaker	\N	libpagemaker-0.0.4_10
2020-09-21 23:14:24+02	2020-09-21 23:14:37+02	print	t1utils	\N	t1utils-1.32
2020-09-21 23:14:38+02	2020-09-21 23:38:12+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-21 23:38:13+02	2020-09-22 00:05:15+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-09-22 00:05:16+02	2020-09-22 00:06:04+02	security	mbedtls	\N	mbedtls-2.16.8
2020-09-22 00:06:05+02	2020-09-22 00:06:23+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-09-22 00:06:24+02	2020-09-22 00:06:46+02	sysutils	lsof	\N	lsof-4.93.2_12,8
2020-09-22 00:06:47+02	2020-09-22 00:09:51+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-22 00:09:52+02	2020-09-22 00:10:17+02	sysutils	py-psutil	py37	py37-psutil-5.7.2
2020-09-22 00:10:18+02	2020-09-22 00:11:50+02	textproc	UCD	\N	UCD-13.0.0
2020-09-22 00:11:51+02	2020-09-22 00:12:26+02	textproc	libebml	\N	libebml-1.4.0
2020-09-22 00:12:26+02	2020-09-22 00:12:57+02	textproc	mythes	\N	mythes-1.2.4_7
2020-09-22 00:12:58+02	2020-09-22 00:13:23+02	textproc	p5-PDF-API2-XS	\N	p5-PDF-API2-XS-1.001
2020-09-22 00:13:24+02	2020-09-22 00:13:40+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-09-22 00:13:41+02	2020-09-22 00:13:57+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-09-22 00:13:58+02	2020-09-22 00:14:13+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-09-22 00:14:14+02	2020-09-22 00:15:45+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-09-22 00:15:46+02	2020-09-22 00:15:59+02	textproc	rubygem-ronn	\N	rubygem-ronn-0.7.3
2020-09-22 00:16:00+02	2020-09-22 00:21:43+02	www	apache24	\N	apache24-2.4.46
2020-09-21 22:33:56+02	2020-09-22 00:59:44+02	lang	ghc	\N	ghc-8.10.2
2020-09-21 15:04:49+02	2020-09-22 01:02:00+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_3
2020-09-22 01:02:02+02	2020-09-22 01:02:15+02	www	py-h2	py37	py37-h2-3.2.0
2020-09-22 01:02:16+02	2020-09-22 01:02:30+02	www	py-hyperlink	py37	py37-hyperlink-20.0.1
2020-09-22 01:02:31+02	2020-09-22 01:02:49+02	www	py-priority	py37	py37-priority-1.3.0
2020-09-22 01:02:49+02	2020-09-22 01:03:19+02	www	serf	\N	serf-1.3.9_5
2020-09-22 01:03:19+02	2020-09-22 01:05:13+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-22 01:05:13+02	2020-09-22 01:07:26+02	x11	yelp	\N	yelp-3.38.0
2020-09-22 01:07:26+02	2020-09-22 01:08:18+02	x11-fonts	cantarell-fonts	\N	cantarell-fonts-0.101_1
2020-09-22 01:08:18+02	2020-09-22 01:08:42+02	x11-fonts	p5-Font-TTF	\N	p5-Font-TTF-1.06
2020-09-22 01:08:43+02	2020-09-22 01:10:16+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-22 01:10:16+02	2020-09-22 01:11:18+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-22 01:11:19+02	2020-09-22 01:13:46+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-22 01:13:46+02	2020-09-22 01:27:24+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-09-22 01:27:25+02	2020-09-22 01:35:11+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-22 01:35:12+02	2020-09-22 01:42:13+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-22 01:42:14+02	2020-09-22 01:43:38+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-09-22 01:43:38+02	2020-09-22 01:48:51+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-09-22 01:48:52+02	2020-09-22 02:02:15+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-09-22 02:02:16+02	2020-09-22 02:16:46+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-22 02:16:46+02	2020-09-22 02:18:11+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-22 02:18:12+02	2020-09-22 02:19:30+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-09-22 02:19:31+02	2020-09-22 02:22:06+02	x11-wm	marco	\N	marco-1.22.4
2020-09-22 00:59:44+02	2020-09-22 02:22:08+02	www	node12	\N	node12-12.18.4
2020-09-22 02:22:07+02	2020-09-22 02:22:27+02	archivers	c-blosc	\N	c-blosc-1.15.0_2
2020-09-22 02:22:27+02	2020-09-22 02:22:44+02	archivers	p5-Archive-Zip	\N	p5-Archive-Zip-1.68
2020-09-22 02:22:10+02	2020-09-22 02:23:10+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-22 02:23:11+02	2020-09-22 02:23:33+02	audio	freealut	\N	freealut-1.1.0_4
2020-09-22 02:23:34+02	2020-09-22 02:23:47+02	audio	libdssialsacompat	\N	libdssialsacompat-1.0.8a_2
2020-09-22 02:22:46+02	2020-09-22 02:23:52+02	audio	faad	\N	faad2-2.8.8,1
2020-09-22 02:23:48+02	2020-09-22 02:24:24+02	audio	liblo	\N	liblo-0.31_1
2020-09-22 02:24:25+02	2020-09-22 02:24:58+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-09-22 02:23:52+02	2020-09-22 02:25:36+02	audio	libopenshot-audio	\N	libopenshot-audio-0.2.0.14
2020-09-22 02:25:36+02	2020-09-22 02:26:00+02	audio	twolame	\N	twolame-0.4.0
2020-09-22 02:26:01+02	2020-09-22 02:26:36+02	databases	p5-DBI	\N	p5-DBI-1.643
2020-09-22 02:24:59+02	2020-09-22 02:26:46+02	audio	taglib	\N	taglib-1.12.b.1_1
2020-09-22 02:26:36+02	2020-09-22 02:27:59+02	databases	unixODBC	\N	unixODBC-2.3.7
2020-09-22 02:26:47+02	2020-09-22 02:28:58+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-22 02:27:59+02	2020-09-22 02:29:12+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-22 02:29:12+02	2020-09-22 02:31:05+02	devel	binutils	aarch64	aarch64-binutils-2.33.1_3,1
2020-09-22 02:31:06+02	2020-09-22 02:33:02+02	devel	binutils	mips64	mips64-binutils-2.33.1_3,1
2020-09-22 02:29:00+02	2020-09-22 02:33:45+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-22 02:33:03+02	2020-09-22 02:34:41+02	devel	binutils	powerpc64	powerpc64-binutils-2.33.1_3,1
2020-09-22 02:33:46+02	2020-09-22 02:35:08+02	devel	binutils	riscv64	riscv64-binutils-2.33.1_3,1
2020-09-22 02:35:08+02	2020-09-22 02:35:17+02	devel	cl-asdf	\N	cl-asdf-3.3.1.7,2
2020-09-22 02:35:19+02	2020-09-22 02:35:41+02	devel	cscope	\N	cscope-15.9
2020-09-22 02:34:42+02	2020-09-22 02:35:56+02	devel	binutils	sparc64	sparc64-binutils-2.33.1_3,1
2020-09-22 02:35:42+02	2020-09-22 02:36:12+02	devel	dmake	\N	dmake-4.12.3,1
2020-09-22 02:35:57+02	2020-09-22 02:36:14+02	devel	fpc-fcl-registry	\N	fpc-fcl-registry-3.0.4_4
2020-09-22 02:36:13+02	2020-09-22 02:37:11+02	devel	git-lfs	\N	git-lfs-2.12.0
2020-09-22 02:37:12+02	2020-09-22 02:40:26+02	devel	ispc	\N	ispc-1.14.1_2
2020-09-22 02:40:27+02	2020-09-22 02:44:05+02	devel	kBuild	\N	kBuild-0.1.9998_13
2020-09-22 02:44:06+02	2020-09-22 02:44:32+02	devel	libfmt	\N	libfmt-7.0.3
2020-09-22 02:36:14+02	2020-09-22 02:47:38+02	devel	hs-cabal-install	\N	hs-cabal-install-3.2.0.0
2020-09-22 02:47:38+02	2020-09-22 02:47:54+02	devel	libsigsegv	\N	libsigsegv-2.12
2020-09-22 02:44:33+02	2020-09-22 02:50:52+02	devel	liborcus	\N	liborcus-0.15.4
2020-09-22 02:47:54+02	2020-09-22 02:51:06+02	devel	libslang2	\N	libslang2-2.3.2_2
2020-09-22 02:51:07+02	2020-09-22 02:51:40+02	devel	ocl-icd	\N	ocl-icd-2.2.12
2020-09-22 02:51:40+02	2020-09-22 02:51:56+02	devel	p5-Getopt-ArgvFile	\N	p5-Getopt-ArgvFile-1.11_1
2020-09-22 02:51:57+02	2020-09-22 02:52:08+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-09-22 02:52:09+02	2020-09-22 02:52:22+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-09-22 02:52:23+02	2020-09-22 02:52:36+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-09-22 02:52:37+02	2020-09-22 02:52:48+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-09-22 02:52:48+02	2020-09-22 02:53:01+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-09-22 02:53:02+02	2020-09-22 02:53:15+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-09-22 02:53:15+02	2020-09-22 02:53:26+02	devel	py-pyasn1	py27	py27-pyasn1-0.4.7
2020-09-22 02:53:26+02	2020-09-22 02:54:20+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-09-22 02:54:20+02	2020-09-22 02:54:45+02	devel	pydbus-common	\N	pydbus-common-1.2.16
2020-09-22 02:50:53+02	2020-09-22 02:57:15+02	devel	ncurses	\N	ncurses-6.2.20200907
2020-09-22 02:57:16+02	2020-09-22 02:57:40+02	devel	rebar	\N	rebar-2.6.4,1
2020-09-22 02:57:40+02	2020-09-22 02:58:55+02	devel	rebar3	\N	rebar3-3.14.1,1
2020-09-22 02:58:56+02	2020-09-22 02:59:04+02	devel	robin-map	\N	robin-map-0.6.3
2020-09-22 02:59:04+02	2020-09-22 02:59:15+02	devel	ucpp	\N	ucpp-1.3.2,1
2020-09-22 02:59:16+02	2020-09-22 03:00:19+02	devel	unittest-cpp	\N	unittest-cpp-2.0.0
2020-09-22 03:00:20+02	2020-09-22 03:13:21+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-22 03:13:22+02	2020-09-22 03:47:39+02	graphics	dcmtk	\N	dcmtk-3.6.5_2
2020-09-22 03:47:40+02	2020-09-22 03:50:02+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-09-22 03:50:03+02	2020-09-22 03:50:33+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-09-22 03:50:34+02	2020-09-22 03:50:58+02	graphics	frei0r-plugins-gavl	\N	frei0r-plugins-gavl-1.6.1
2020-09-22 03:50:59+02	2020-09-22 03:53:33+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-22 03:53:35+02	2020-09-22 04:09:17+02	graphics	libetonyek01	\N	libetonyek01-0.1.9_6,1
2020-09-22 04:09:17+02	2020-09-22 04:11:19+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-09-22 04:11:22+02	2020-09-22 04:12:14+02	graphics	libgxps	\N	libgxps-0.3.1
2020-09-22 04:12:16+02	2020-09-22 04:15:27+02	graphics	libheif	\N	libheif-1.6.2
2020-09-22 04:15:27+02	2020-09-22 04:16:18+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-09-22 04:16:19+02	2020-09-22 04:17:49+02	graphics	libzmf	\N	libzmf-0.0.2_21
2020-09-22 04:17:49+02	2020-09-22 04:26:28+02	graphics	podofo	\N	podofo-0.9.6_1
2020-09-22 04:26:29+02	2020-09-22 04:35:32+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-09-22 04:35:33+02	2020-09-22 04:36:19+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-09-22 04:36:19+02	2020-09-22 04:37:43+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-22 04:37:44+02	2020-09-22 04:38:07+02	graphics	qr-code-generator	\N	qr-code-generator-1.6.0
2020-09-22 04:38:08+02	2020-09-22 04:39:39+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-09-22 04:39:40+02	2020-09-22 04:46:33+02	graphics	sane-backends	\N	sane-backends-1.0.31_1
2020-09-22 04:46:36+02	2020-09-22 04:47:03+02	graphics	squish	\N	squish-1.10_2
2020-09-22 04:47:03+02	2020-09-22 04:48:59+02	graphics	vigra	\N	vigra-1.11.1_12
2020-09-22 04:49:00+02	2020-09-22 04:53:01+02	lang	cython	py38	py38-cython-0.29.15
2020-09-22 04:53:01+02	2020-09-22 04:53:34+02	lang	fpc-rtl-console	\N	fpc-rtl-console-3.0.4_4
2020-09-22 02:54:45+02	2020-09-22 04:53:58+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-22 04:54:01+02	2020-09-22 04:55:53+02	lang	fpc-utils	\N	fpc-utils-3.0.4_6
2020-09-22 04:53:35+02	2020-09-22 04:57:57+02	lang	fpc-source	\N	fpc-source-3.0.4_2
2020-09-22 04:57:57+02	2020-09-22 04:58:35+02	lang	p5-Try-Tiny	\N	p5-Try-Tiny-0.30
2020-09-22 04:55:54+02	2020-09-22 05:01:47+02	lang	guile1	\N	guile1-1.8.8
2020-09-22 05:01:48+02	2020-09-22 05:04:31+02	math	cryptominisat	\N	cryptominisat-5.7.1
2020-09-22 05:04:32+02	2020-09-22 05:04:54+02	math	fpc-numlib	\N	fpc-numlib-3.0.4_4
2020-09-22 05:04:54+02	2020-09-22 05:10:18+02	math	gsl	\N	gsl-2.6
2020-09-22 05:10:19+02	2020-09-22 05:13:12+02	math	lp_solve	\N	lp_solve-5.5.2.5
2020-09-22 05:13:12+02	2020-09-22 05:13:33+02	math	minisat	\N	minisat-2.2.1_2
2020-09-22 05:13:33+02	2020-09-22 05:13:55+02	math	openlibm	\N	openlibm-0.6.0_2
2020-09-22 05:13:55+02	2020-09-22 05:14:23+02	misc	box2d	\N	Box2D-2.3.1
2020-09-22 05:14:23+02	2020-09-22 05:22:25+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-22 04:58:38+02	2020-09-22 05:22:54+02	math	R	\N	R-4.0.2
2020-09-22 05:22:26+02	2020-09-22 05:23:30+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-09-22 05:22:55+02	2020-09-22 05:24:06+02	multimedia	libass	\N	libass-0.14.0
2020-09-22 05:23:31+02	2020-09-22 05:24:07+02	multimedia	libdvbpsi	\N	libdvbpsi-1.3.3
2020-09-22 05:24:07+02	2020-09-22 05:24:41+02	multimedia	libmatroska	\N	libmatroska-1.6.2
2020-09-22 05:24:07+02	2020-09-22 05:24:50+02	multimedia	libmpeg2	\N	libmpeg2-0.5.1_6
2020-09-22 05:24:50+02	2020-09-22 05:25:04+02	net	cppzmq	\N	cppzmq-4.6.0
2020-09-22 05:25:05+02	2020-09-22 05:25:30+02	net	fping	\N	fping-5.0_1
2020-09-22 05:24:42+02	2020-09-22 05:27:24+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-22 05:27:24+02	2020-09-22 05:28:07+02	net	gupnp	\N	gupnp-1.1.1_1
2020-09-22 05:25:30+02	2020-09-22 05:28:58+02	net	freerdp	\N	freerdp-2.2.0
2020-09-22 05:28:59+02	2020-09-22 05:29:13+02	net	libmaxminddb	\N	libmaxminddb-1.4.3
2020-09-22 05:29:13+02	2020-09-22 05:30:41+02	net	liveMedia	\N	liveMedia-2020.08.19,2
2020-09-22 05:30:42+02	2020-09-22 05:31:03+02	net	p5-Net-HTTP	\N	p5-Net-HTTP-6.19
2020-09-22 05:31:04+02	2020-09-22 05:31:23+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-09-22 05:31:24+02	2020-09-22 05:31:42+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-09-22 05:31:42+02	2020-09-22 05:32:43+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-22 05:32:43+02	2020-09-22 05:33:43+02	net	rsync	\N	rsync-3.2.3
2020-09-22 05:33:44+02	2020-09-22 05:34:59+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-09-22 05:35:00+02	2020-09-22 05:35:38+02	print	freetype	\N	freetype-1.3.1_5
2020-09-22 05:28:07+02	2020-09-22 05:40:57+02	net	libcmis	\N	libcmis-0.5.2_1
2020-09-22 05:35:38+02	2020-09-22 05:41:09+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-22 05:40:58+02	2020-09-22 05:41:27+02	print	libspectre	\N	libspectre-0.2.9
2020-09-22 05:41:09+02	2020-09-22 05:43:36+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-22 05:41:28+02	2020-09-22 05:48:39+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-22 05:48:40+02	2020-09-22 05:49:26+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-09-22 05:49:27+02	2020-09-22 05:49:59+02	security	mhash	\N	mhash-0.9.9.9_5
2020-09-22 05:43:37+02	2020-09-22 05:50:01+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-22 05:50:05+02	2020-09-22 05:50:13+02	security	pam_helper	\N	pam_helper-1.1
2020-09-22 05:49:59+02	2020-09-22 05:50:18+02	security	p5-Authen-NTLM	\N	p5-Authen-NTLM-1.09_1
2020-09-22 05:50:13+02	2020-09-22 05:50:27+02	security	py-bcrypt	py27	py27-bcrypt-3.1.7_1
2020-09-22 05:50:18+02	2020-09-22 05:51:08+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-09-22 05:50:28+02	2020-09-22 05:51:11+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-09-22 05:51:09+02	2020-09-22 05:51:26+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-09-22 05:51:11+02	2020-09-22 05:51:27+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-09-22 05:51:27+02	2020-09-22 05:52:47+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-09-22 05:51:27+02	2020-09-22 05:52:57+02	security	sudo	\N	sudo-1.9.2
2020-09-22 05:52:48+02	2020-09-22 05:53:09+02	sysutils	hal-info	\N	hal-info-20091130
2020-09-22 05:53:09+02	2020-09-22 05:54:05+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-22 05:54:06+02	2020-09-22 05:54:25+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.04_1
2020-09-22 05:54:26+02	2020-09-22 05:55:05+02	sysutils	patchelf	\N	patchelf-0.11
2020-09-22 05:55:05+02	2020-09-22 05:55:16+02	sysutils	sysctlbyname-improved-kmod	\N	sysctlbyname-improved-kmod-20191124
2020-09-22 05:55:16+02	2020-09-22 05:55:24+02	sysutils	sysctlinfo-kmod	\N	sysctlinfo-kmod-20191005
2020-09-22 05:52:57+02	2020-09-22 05:56:49+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-09-22 05:56:50+02	2020-09-22 05:57:00+02	sysutils	vpnc-scripts	\N	vpnc-scripts-20200226
2020-09-22 05:57:00+02	2020-09-22 06:00:05+02	textproc	clucene	\N	clucene-2.3.3.4_19
2020-09-22 05:55:25+02	2020-09-22 06:00:28+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-09-22 06:00:29+02	2020-09-22 06:01:49+02	textproc	libabw	\N	libabw-0.1.3
2020-09-22 06:01:50+02	2020-09-22 06:01:59+02	textproc	libdiff	\N	libdiff-0.1.0
2020-09-22 06:00:07+02	2020-09-22 06:02:18+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-22 06:02:18+02	2020-09-22 06:03:29+02	textproc	libepubgen	\N	libepubgen-0.1.0_9
2020-09-22 06:03:29+02	2020-09-22 06:03:41+02	textproc	libexttextcat	\N	libexttextcat-3.4.5
2020-09-22 06:01:59+02	2020-09-22 06:06:56+02	textproc	libe-book	\N	libe-book-0.1.3_18
2020-09-22 06:06:57+02	2020-09-22 06:07:51+02	textproc	libnumbertext	\N	libnumbertext-1.0.6
2020-09-22 06:07:51+02	2020-09-22 06:09:35+02	textproc	libodfgen01	\N	libodfgen01-0.1.7_3
2020-09-22 06:09:36+02	2020-09-22 06:11:16+02	textproc	libqxp	\N	libqxp-0.0.0_16
2020-09-22 06:03:41+02	2020-09-22 06:14:12+02	textproc	libmwaw03	\N	libmwaw03-0.3.15
2020-09-22 06:11:16+02	2020-09-22 06:15:06+02	textproc	libstaroffice	\N	libstaroffice-0.0.7
2020-09-22 06:15:06+02	2020-09-22 06:15:54+02	textproc	p5-PDF-API2	\N	p5-PDF-API2-2.038
2020-09-22 06:15:54+02	2020-09-22 06:16:15+02	textproc	p5-Pod-Escapes	\N	p5-Pod-Escapes-1.07
2020-09-22 06:16:16+02	2020-09-22 06:16:52+02	textproc	pugixml	\N	pugixml-1.10
2020-09-22 06:16:53+02	2020-09-22 06:17:12+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-09-22 06:17:12+02	2020-09-22 06:17:25+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-09-22 06:14:13+02	2020-09-22 06:17:28+02	textproc	libwps	\N	libwps-0.4.11
2020-09-22 06:17:25+02	2020-09-22 06:17:37+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-09-22 06:17:40+02	2020-09-22 06:17:58+02	www	p5-HTTP-Cookies	\N	p5-HTTP-Cookies-6.08
2020-09-22 06:17:58+02	2020-09-22 06:18:22+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.12
2020-09-22 06:17:28+02	2020-09-22 06:18:25+02	textproc	redland	\N	redland-1.0.17_4
2020-09-22 06:18:22+02	2020-09-22 06:18:38+02	www	p5-HTTP-Negotiate	\N	p5-HTTP-Negotiate-6.01_1
2020-09-22 06:18:26+02	2020-09-22 06:18:41+02	www	p5-WWW-RobotRules	\N	p5-WWW-RobotRules-6.02_1
2020-09-22 06:18:39+02	2020-09-22 06:19:04+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-09-22 06:18:42+02	2020-09-22 06:19:31+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-22 06:19:33+02	2020-09-22 06:19:44+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-09-22 06:19:45+02	2020-09-22 06:20:24+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-09-22 06:19:05+02	2020-09-22 06:20:41+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-22 06:20:41+02	2020-09-22 06:20:53+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-09-22 06:20:54+02	2020-09-22 06:21:03+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-09-22 06:21:03+02	2020-09-22 06:21:21+02	x11-fonts	droid-fonts-ttf	\N	droid-fonts-ttf-20131024_5
2020-09-22 06:21:22+02	2020-09-22 06:21:33+02	x11-fonts	gentium-basic	\N	GentiumBasic-1102_1
2020-09-22 06:21:34+02	2020-09-22 06:21:45+02	x11-fonts	liberation-fonts-ttf	\N	liberation-fonts-ttf-2.1.1,2
2020-09-22 06:20:25+02	2020-09-22 06:21:57+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-22 06:21:45+02	2020-09-22 06:22:03+02	x11-fonts	linuxlibertine-g	\N	linuxlibertine-g-20120116_2
2020-09-22 06:21:57+02	2020-09-22 06:22:13+02	x11-fonts	twemoji-color-font-ttf	\N	twemoji-color-font-ttf-12.0.1
2020-09-22 06:22:13+02	2020-09-22 06:22:56+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-09-22 06:22:57+02	2020-09-22 06:24:15+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-09-22 06:22:04+02	2020-09-22 06:24:19+02	x11-servers	xephyr	\N	xephyr-1.20.9,1
2020-09-22 06:24:16+02	2020-09-22 06:24:48+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-09-22 06:24:20+02	2020-09-22 06:25:11+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-22 06:24:49+02	2020-09-22 06:27:17+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-22 06:25:12+02	2020-09-22 06:27:33+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-09-22 06:27:18+02	2020-09-22 06:30:27+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-22 06:30:28+02	2020-09-22 06:30:37+02	archivers	arc	\N	arc-5.21p
2020-09-22 06:30:38+02	2020-09-22 06:31:39+02	archivers	arj	\N	arj-3.10.22_8
2020-09-22 06:31:40+02	2020-09-22 06:35:03+02	archivers	engrampa	\N	engrampa-1.24.1
2020-09-22 06:35:04+02	2020-09-22 06:35:39+02	archivers	libunrar5	\N	libunrar5-5.9.4
2020-09-22 06:35:40+02	2020-09-22 06:37:03+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-09-22 06:37:04+02	2020-09-22 06:37:13+02	archivers	unzoo	\N	unzoo-4.4_2
2020-09-22 06:37:14+02	2020-09-22 06:37:37+02	audio	dssi	\N	dssi-1.1.1_8
2020-09-22 06:37:38+02	2020-09-22 06:38:13+02	audio	faudio	\N	FAudio-20.09
2020-09-22 06:38:13+02	2020-09-22 06:42:58+02	audio	fdk-aac	\N	fdk-aac-2.0.1
2020-09-22 06:42:59+02	2020-09-22 06:44:33+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-22 06:44:33+02	2020-09-22 06:44:56+02	audio	sndio	\N	sndio-1.7.0
2020-09-22 06:44:57+02	2020-09-22 06:45:55+02	audio	swhplugins	\N	swhplugins-0.4.15_7
2020-09-22 06:45:56+02	2020-09-22 06:46:39+02	audio	wavpack	\N	wavpack-5.3.0
2020-09-22 06:46:39+02	2020-09-22 06:46:56+02	databases	lmdb	\N	lmdb-0.9.24_2,1
2020-09-22 06:46:56+02	2020-09-22 06:47:33+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.14.2
2020-09-22 06:47:34+02	2020-09-22 06:47:51+02	databases	pgtcl	\N	pgtcl-2.1.1_2
2020-09-22 06:47:51+02	2020-09-22 06:48:16+02	databases	py-apsw	py37	py37-apsw-3.33.0
2020-09-22 06:48:16+02	2020-09-22 06:50:13+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-09-22 06:50:14+02	2020-09-22 06:52:07+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-09-22 06:52:07+02	2020-09-22 06:57:28+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-22 06:57:29+02	2020-09-22 06:58:32+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-22 06:58:33+02	2020-09-22 06:59:41+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-22 06:27:34+02	2020-09-22 07:01:59+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-22 07:02:00+02	2020-09-22 07:02:49+02	devel	adacurses	\N	adacurses-20180127
2020-09-22 07:02:50+02	2020-09-22 07:03:07+02	devel	cweb	\N	cweb-3.64a.a
2020-09-22 07:03:08+02	2020-09-22 07:03:34+02	devel	erlang-cl	\N	erlang-cl-1.2.4
2020-09-22 07:03:34+02	2020-09-22 07:04:00+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-09-22 07:04:01+02	2020-09-22 07:04:27+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-09-22 07:04:28+02	2020-09-22 07:04:50+02	devel	fpc-fv	\N	fpc-fv-3.0.4_4
2020-09-22 07:04:50+02	2020-09-22 07:05:14+02	devel	fpc-gdbint	\N	fpc-gdbint-3.0.4_4
2020-09-22 07:05:15+02	2020-09-22 07:06:09+02	devel	fstrm	\N	fstrm-0.6.0
2020-09-22 07:06:11+02	2020-09-22 07:09:29+02	devel	google-perftools	\N	google-perftools-2.8
2020-09-22 07:09:29+02	2020-09-22 07:11:21+02	devel	gradle5	\N	gradle5-5.0
2020-09-22 06:59:42+02	2020-09-22 07:12:27+02	devel	aarch64-gcc	\N	aarch64-gcc-6.4.0_8
2020-09-22 07:12:28+02	2020-09-22 07:13:59+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-09-22 07:13:59+02	2020-09-22 07:15:36+02	devel	libarea	\N	libarea-20160313_13
2020-09-22 07:15:37+02	2020-09-22 07:16:35+02	devel	libstatgrab	\N	libstatgrab-0.92
2020-09-22 07:16:35+02	2020-09-22 07:16:42+02	devel	libsysctlmibinfo2	\N	libsysctlmibinfo2-2.0.0_1
2020-09-22 07:16:43+02	2020-09-22 07:16:52+02	devel	libublio	\N	libublio-20070103_2
2020-09-22 07:11:21+02	2020-09-22 07:18:06+02	devel	gsoap	\N	gsoap-2.8.106
2020-09-22 07:16:53+02	2020-09-22 07:18:59+02	devel	mercurial	py37	mercurial-5.5_1
2020-09-22 07:18:59+02	2020-09-22 07:21:27+02	devel	ode	\N	ode-0.13_4,1
2020-09-22 07:21:27+02	2020-09-22 07:21:50+02	devel	p5-Term-ReadKey	\N	p5-Term-ReadKey-2.38_1
2020-09-22 07:18:07+02	2020-09-22 07:40:53+02	devel	mips64-gcc	\N	mips64-gcc-6.4.0_8
2020-09-25 04:06:43+02	2020-09-25 04:06:54+02	x11	xinit	\N	xinit-1.4.1,1
2020-09-22 07:40:54+02	2020-09-22 07:45:12+02	devel	protobuf-c	\N	protobuf-c-1.3.2_6
2020-09-22 07:45:13+02	2020-09-22 07:45:25+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-09-22 07:21:50+02	2020-09-22 07:45:32+02	devel	powerpc64-gcc	\N	powerpc64-gcc-6.4.0_8
2020-09-22 07:45:25+02	2020-09-22 07:45:39+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-09-22 07:45:40+02	2020-09-22 07:45:51+02	devel	py-funcy	py37	py37-funcy-1.12
2020-09-22 07:45:33+02	2020-09-22 07:45:52+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-09-22 07:45:52+02	2020-09-22 07:46:05+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-09-22 07:46:06+02	2020-09-22 07:46:17+02	devel	py-iniparse	py37	py37-iniparse-0.5
2020-09-22 07:46:18+02	2020-09-22 07:46:33+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-09-22 07:46:33+02	2020-09-22 07:46:57+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-09-22 07:46:57+02	2020-09-22 07:47:10+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-09-22 07:45:52+02	2020-09-22 07:47:29+02	devel	py-game	py37	py37-game-1.9.6_1
2020-09-22 07:47:30+02	2020-09-22 07:47:42+02	devel	py-rq	py37	py37-rq-1.3.0
2020-09-22 07:47:43+02	2020-09-22 07:48:22+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-09-22 07:48:22+02	2020-09-22 07:48:38+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-09-22 07:48:39+02	2020-09-22 07:48:52+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-09-22 07:48:52+02	2020-09-22 07:49:06+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-09-22 07:47:10+02	2020-09-22 07:49:43+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-09-22 07:49:06+02	2020-09-22 07:51:42+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-22 07:51:42+02	2020-09-22 07:52:16+02	devel	rlwrap	\N	rlwrap-0.42_2
2020-09-22 07:52:17+02	2020-09-22 07:52:29+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-09-22 07:52:31+02	2020-09-22 07:52:43+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-09-22 07:52:44+02	2020-09-22 07:52:57+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-09-22 07:49:43+02	2020-09-22 08:15:23+02	devel	riscv64-gcc	\N	riscv64-gcc-8.3.0
2020-09-22 07:52:57+02	2020-09-22 08:15:32+02	devel	sparc64-gcc	\N	sparc64-gcc-6.4.0_8
2020-09-22 08:15:33+02	2020-09-22 08:15:47+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-09-22 08:15:24+02	2020-09-22 08:16:04+02	devel	tex-synctex	\N	tex-synctex-1.17.0_1
2020-09-22 08:15:48+02	2020-09-22 08:42:29+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-22 08:42:31+02	2020-09-22 08:49:16+02	editors	pluma	\N	pluma-1.24.1
2020-09-22 08:49:20+02	2020-09-22 08:53:19+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-09-22 08:53:20+02	2020-09-22 08:55:18+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-22 08:55:19+02	2020-09-22 09:01:16+02	french	med	\N	fr-med-4.0.0
2020-09-22 09:01:17+02	2020-09-22 09:02:33+02	ftp	libfilezilla	\N	libfilezilla-0.15.1_3
2020-09-22 09:02:33+02	2020-09-22 09:17:50+02	games	alienarena-data	\N	alienarena-data-2013.766
2020-09-22 09:17:51+02	2020-09-22 09:19:28+02	games	gnuchess	\N	gnuchess-6.2.7
2020-09-22 09:19:32+02	2020-09-22 09:24:52+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-09-22 09:24:53+02	2020-09-22 09:36:06+02	graphics	alembic	\N	alembic-1.7.15
2020-09-22 09:36:09+02	2020-09-22 09:40:44+02	graphics	atril	\N	atril-1.22.3_9
2020-09-22 09:40:45+02	2020-09-22 10:18:38+02	graphics	embree	\N	embree-3.10.0_1
2020-09-22 10:18:39+02	2020-09-22 10:21:40+02	graphics	eom	\N	eom-1.24.1
2020-09-22 10:21:40+02	2020-09-22 10:23:55+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-22 10:23:55+02	2020-09-22 10:24:28+02	graphics	gle	\N	gle-3.1.0_7
2020-09-22 10:24:29+02	2020-09-22 10:24:54+02	graphics	gltt	\N	gltt-2.5.2_14
2020-09-22 10:24:55+02	2020-09-22 10:26:57+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-22 10:26:58+02	2020-09-22 10:27:11+02	graphics	jgraph	\N	jgraph-8.3_1
2020-09-22 10:27:13+02	2020-09-22 10:27:48+02	graphics	libart_lgpl	\N	libart_lgpl-2.3.21_3,1
2020-09-22 10:27:48+02	2020-09-22 10:39:31+02	graphics	oidn	\N	oidn-1.2.3
2020-09-22 10:39:35+02	2020-09-22 11:09:03+02	graphics	opencollada	\N	opencollada-1.6.68_2
2020-09-22 11:09:03+02	2020-09-22 11:33:10+02	graphics	openimageio	\N	openimageio-2.1.15.0_6
2020-09-22 11:33:11+02	2020-09-22 11:38:28+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-09-22 11:38:32+02	2020-09-22 11:40:52+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-22 11:40:53+02	2020-09-22 11:50:35+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-09-22 11:50:36+02	2020-09-22 11:51:17+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-09-22 11:51:17+02	2020-09-22 11:52:12+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-09-22 11:52:13+02	2020-09-22 12:08:00+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-22 12:08:01+02	2020-09-22 12:48:13+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-09-22 12:48:14+02	2020-09-22 12:51:34+02	lang	cython	py27	py27-cython-0.29.15
2020-09-22 12:51:34+02	2020-09-22 13:02:29+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-22 13:02:30+02	2020-09-22 14:08:29+02	lang	gcc48	\N	gcc48-4.8.5_11
2020-09-22 08:16:05+02	2020-09-22 15:08:07+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-09-22 15:08:09+02	2020-09-22 15:08:25+02	lang	lua51	\N	lua51-5.1.5_9
2020-09-22 15:08:25+02	2020-09-22 15:09:31+02	lang	luajit	\N	luajit-2.0.5_4
2020-09-22 15:09:32+02	2020-09-22 15:09:42+02	lang	python3	\N	python3-3_3
2020-09-22 15:09:43+02	2020-09-22 15:20:05+02	lang	sbcl	\N	sbcl-2.0.8,1
2020-09-22 15:20:06+02	2020-09-22 15:22:41+02	math	mate-calc	\N	mate-calc-1.24.1
2020-09-22 15:22:42+02	2020-09-22 15:24:08+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-09-22 15:24:08+02	2020-09-22 15:29:20+02	math	py-numpy	py38	py38-numpy-1.16.6,1
2020-09-22 15:29:21+02	2020-09-22 15:33:43+02	math	stp	\N	stp-2.3.3_6,1
2020-09-22 14:08:31+02	2020-09-22 16:04:07+02	lang	julia10	\N	julia10-1.0.4_5
2020-09-22 16:04:08+02	2020-09-22 16:04:22+02	misc	chmlib	\N	chmlib-0.40_1
2020-09-22 16:04:23+02	2020-09-22 16:04:32+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200918
2020-09-22 15:33:44+02	2020-09-22 16:06:53+02	math	z3	\N	z3-4.8.8
2020-09-22 16:06:53+02	2020-09-22 16:07:19+02	misc	ini_file_manager	\N	ini_file_manager-03_3
2020-09-22 16:04:33+02	2020-09-22 16:10:31+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-09-22 16:10:32+02	2020-09-22 16:10:53+02	misc	xosd	\N	xosd-2.2.14_3
2020-09-22 16:10:53+02	2020-09-22 16:11:54+02	multimedia	gmmlib	\N	gmmlib-20.3.2
2020-09-22 16:11:55+02	2020-09-22 16:12:39+02	multimedia	libmtp	\N	libmtp-1.1.17
2020-09-22 16:12:39+02	2020-09-22 16:19:28+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-09-22 16:19:29+02	2020-09-22 16:20:42+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-09-22 16:20:42+02	2020-09-22 16:24:12+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-09-22 16:07:20+02	2020-09-22 16:26:02+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-09-22 16:26:03+02	2020-09-22 16:28:12+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-22 16:28:13+02	2020-09-22 16:29:51+02	net	ndpi	\N	ndpi-3.2.d20200721,1
2020-09-22 16:29:51+02	2020-09-22 16:30:11+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-09-22 16:30:13+02	2020-09-22 16:31:06+02	net	py-pyzmq	py37	py37-pyzmq-19.0.2
2020-09-22 16:31:07+02	2020-09-22 16:32:19+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-22 16:32:20+02	2020-09-22 16:33:31+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-22 16:33:31+02	2020-09-22 16:34:40+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-22 16:34:41+02	2020-09-22 16:35:57+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-22 16:35:57+02	2020-09-22 16:37:02+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-22 16:37:03+02	2020-09-22 16:38:06+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-22 16:38:07+02	2020-09-22 16:39:16+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-22 16:24:12+02	2020-09-22 16:39:28+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-22 16:39:29+02	2020-09-22 16:40:18+02	net	vde2	\N	vde2-2.3.2_4
2020-09-25 04:06:53+02	2020-09-25 04:07:05+02	x11	xinput	\N	xinput-1.6.3
2020-09-22 16:40:19+02	2020-09-22 16:40:52+02	net	wireguard-go	\N	wireguard-go-0.0.20200320
2020-09-22 16:39:18+02	2020-09-22 16:41:07+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-22 16:40:53+02	2020-09-22 16:41:56+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-09-22 16:41:07+02	2020-09-22 16:42:04+02	net-mgmt	libsmi	\N	libsmi-0.4.8_1
2020-09-22 16:42:04+02	2020-09-22 16:42:20+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-09-22 16:42:21+02	2020-09-22 16:42:30+02	ports-mgmt	dialog4ports	\N	dialog4ports-0.1.6
2020-09-22 16:41:57+02	2020-09-22 16:43:39+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-09-22 16:42:30+02	2020-09-22 16:44:09+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-22 16:44:12+02	2020-09-22 16:54:53+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-22 16:43:40+02	2020-09-22 17:00:46+02	print	lilypond	\N	lilypond-2.20.0
2020-09-22 16:54:55+02	2020-09-22 17:02:24+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-22 17:00:47+02	2020-09-22 17:03:00+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-22 17:02:25+02	2020-09-22 17:13:09+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-22 17:13:10+02	2020-09-22 17:13:49+02	print	transfig	\N	transfig-3.2.5e_4
2020-09-22 17:13:50+02	2020-09-22 17:14:18+02	security	libntlm	\N	libntlm-1.6
2020-09-22 17:14:18+02	2020-09-22 17:14:28+02	security	libscrypt	\N	libscrypt-1.21
2020-09-22 17:14:29+02	2020-09-22 17:18:00+02	security	nmap	\N	nmap-7.80
2020-09-22 17:18:01+02	2020-09-22 17:18:39+02	security	openconnect	\N	openconnect-8.10
2020-09-22 17:18:39+02	2020-09-22 17:19:00+02	security	p5-Digest-SHA1	\N	p5-Digest-SHA1-2.13_1
2020-09-22 17:19:00+02	2020-09-22 17:19:08+02	security	pkesh	\N	pkesh-1.0
2020-09-22 17:19:08+02	2020-09-22 17:19:23+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-09-22 17:19:24+02	2020-09-22 17:19:36+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-09-22 17:19:36+02	2020-09-22 17:19:50+02	security	py-stem	py37	py37-stem-1.8.0
2020-09-22 17:19:50+02	2020-09-22 17:22:45+02	sysutils	coreutils	\N	coreutils-8.32
2020-09-22 17:22:46+02	2020-09-22 17:25:23+02	sysutils	intel-pcm	\N	intel-pcm-202008_1
2020-09-22 17:25:25+02	2020-09-22 17:25:59+02	sysutils	libcpuid	\N	libcpuid-0.5.0
2020-09-22 17:03:01+02	2020-09-22 17:26:26+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-22 17:26:02+02	2020-09-22 17:26:57+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-22 17:26:28+02	2020-09-22 17:28:42+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-09-22 17:28:43+02	2020-09-22 17:29:00+02	sysutils	smart	\N	smart-0.0.11
2020-09-22 17:29:01+02	2020-09-22 17:29:53+02	sysutils	smartmontools	\N	smartmontools-7.1_2
2020-09-22 17:26:58+02	2020-09-22 17:30:11+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-09-22 17:29:54+02	2020-09-22 17:30:39+02	sysutils	tarsnap	\N	tarsnap-1.0.39
2020-09-22 17:30:12+02	2020-09-22 17:31:08+02	textproc	diffutils	\N	diffutils-3.7
2020-09-22 17:31:08+02	2020-09-22 17:32:25+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-22 17:30:40+02	2020-09-22 17:32:43+02	textproc	fop	\N	fop-2.3
2020-09-22 17:32:43+02	2020-09-22 17:33:24+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-09-22 17:33:24+02	2020-09-22 17:33:43+02	textproc	lowdown	\N	lowdown-0.7.4
2020-09-22 17:33:43+02	2020-09-22 17:34:03+02	textproc	p5-pod2pdf	\N	p5-pod2pdf-0.42_1
2020-09-22 17:34:03+02	2020-09-22 17:36:13+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-09-22 17:36:13+02	2020-09-22 17:37:24+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-22 17:37:24+02	2020-09-22 17:38:07+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-09-22 17:38:07+02	2020-09-22 17:38:21+02	textproc	py-scour	py37	py37-scour-0.38.1
2020-09-22 17:38:22+02	2020-09-22 17:38:36+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-09-22 17:38:36+02	2020-09-22 17:46:27+02	textproc	ripgrep	\N	ripgrep-12.1.1_5
2020-09-22 17:46:28+02	2020-09-22 17:52:18+02	textproc	xerces-c3	\N	xerces-c3-3.2.3
2020-09-22 17:52:18+02	2020-09-22 17:53:25+02	www	mattermost-webapp	\N	mattermost-webapp-5.26.1
2020-09-22 17:53:26+02	2020-09-22 17:53:57+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-09-22 17:53:59+02	2020-09-22 17:54:21+02	www	p5-libwww	\N	p5-libwww-6.47
2020-09-22 17:54:21+02	2020-09-22 17:54:53+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-09-22 17:54:54+02	2020-09-22 17:55:09+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-09-22 17:55:10+02	2020-09-22 17:55:30+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-09-22 17:55:30+02	2020-09-22 17:55:44+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-09-22 17:55:46+02	2020-09-22 17:55:58+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-09-22 17:55:59+02	2020-09-22 17:56:16+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-09-22 17:56:16+02	2020-09-22 17:56:34+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-09-22 17:56:34+02	2020-09-22 17:58:46+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-22 17:58:47+02	2020-09-22 18:00:48+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-22 18:00:49+02	2020-09-22 18:01:03+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-09-22 18:01:04+02	2020-09-22 18:08:00+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-22 18:08:00+02	2020-09-22 18:09:25+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-22 18:09:26+02	2020-09-22 18:10:56+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-22 18:10:56+02	2020-09-22 18:13:17+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-22 18:13:17+02	2020-09-22 18:16:44+02	x11-fm	nautilus	\N	nautilus-3.28.1_4
2020-09-22 18:16:44+02	2020-09-22 18:16:53+02	x11-fonts	bitstream-vera	\N	bitstream-vera-1.10_8
2020-09-22 18:16:54+02	2020-09-22 18:39:15+02	x11-fonts	nerd-fonts	\N	nerd-fonts-2.1.0
2020-09-22 18:39:15+02	2020-09-22 18:41:12+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-09-22 18:41:13+02	2020-09-22 18:42:40+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-22 18:42:41+02	2020-09-22 18:43:31+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-22 18:43:31+02	2020-09-22 18:52:59+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-22 18:53:00+02	2020-09-22 18:53:17+02	archivers	pbzip2	\N	pbzip2-1.1.13
2020-09-22 18:53:18+02	2020-09-22 18:53:46+02	archivers	unrar	\N	unrar-5.91,6
2020-09-22 17:32:26+02	2020-09-22 19:10:36+02	textproc	hs-pandoc	\N	hs-pandoc-2.10
2020-09-22 19:10:38+02	2020-09-22 19:10:52+02	astro	xearth	\N	xearth-1.2
2020-09-22 19:10:53+02	2020-09-22 19:14:33+02	astro	xephem	\N	xephem-3.7.7_5
2020-09-22 18:53:49+02	2020-09-22 19:30:29+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-22 19:30:30+02	2020-09-22 19:31:00+02	audio	xmix	\N	xmix-2.1_3
2020-09-22 19:31:01+02	2020-09-22 19:33:17+02	benchmarks	fio	\N	fio-3.22
2020-09-22 19:33:18+02	2020-09-22 19:33:46+02	benchmarks	iperf3	\N	iperf3-3.9
2020-09-22 19:14:34+02	2020-09-22 19:51:49+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-22 19:51:50+02	2020-09-22 19:59:29+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-22 19:59:30+02	2020-09-22 20:03:09+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-22 20:03:11+02	2020-09-22 22:05:56+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-22 22:05:58+02	2020-09-22 22:08:40+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-22 22:08:41+02	2020-09-22 22:11:08+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-22 22:11:09+02	2020-09-22 22:12:18+02	comms	kermit	\N	kermit-9.0.304.24
2020-09-22 22:12:19+02	2020-09-22 22:12:31+02	converters	dosunix	\N	dosunix-1.0.14
2020-09-22 22:12:32+02	2020-09-22 22:13:47+02	converters	recode	\N	recode-3.6_15
2020-09-22 22:13:47+02	2020-09-22 22:13:55+02	converters	unix2dos	\N	unix2dos-1.3
2020-09-22 22:13:56+02	2020-09-22 22:14:12+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-09-22 22:14:13+02	2020-09-22 22:51:03+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-22 22:51:05+02	2020-09-22 22:51:29+02	databases	pgtop	\N	pgtop-0.05_5
2020-09-22 22:51:30+02	2020-09-22 22:56:28+02	databases	postgresql12-contrib	\N	postgresql12-contrib-12.4
2020-09-22 19:33:47+02	2020-09-22 22:59:23+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-22 22:59:24+02	2020-09-22 22:59:54+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.6
2020-09-22 22:59:55+02	2020-09-22 23:00:48+02	databases	redis	\N	redis-5.0.9
2020-09-22 23:00:49+02	2020-09-22 23:06:09+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-22 23:06:10+02	2020-09-22 23:07:22+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-22 23:07:23+02	2020-09-22 23:07:36+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-09-22 22:56:29+02	2020-09-22 23:07:50+02	databases	postgresql12-server	\N	postgresql12-server-12.4
2020-09-22 23:07:37+02	2020-09-22 23:08:07+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-09-22 23:08:07+02	2020-09-22 23:08:18+02	devel	aarch64-xtoolchain-gcc	\N	aarch64-xtoolchain-gcc-0.4_1
2020-09-22 23:08:19+02	2020-09-22 23:10:19+02	devel	bzr	py27	bzr-2.7.0.6622
2020-09-22 23:10:21+02	2020-09-22 23:12:05+02	devel	cdecl	\N	cdecl-8.0
2020-09-22 23:12:08+02	2020-09-23 00:31:01+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-23 00:31:02+02	2020-09-23 00:32:16+02	devel	cvs+ipv6	\N	cvs+ipv6-1.20120905_3
2020-09-23 00:32:17+02	2020-09-23 00:33:54+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-22 23:07:51+02	2020-09-23 00:58:14+02	devel	RStudio	\N	RStudio-1.3.1093
2020-09-23 00:58:15+02	2020-09-23 02:01:18+02	devel	ghidra	\N	ghidra-9.1
2020-09-23 00:33:55+02	2020-09-23 15:27:26+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-23 02:01:19+02	2020-09-23 16:07:57+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200811
2020-09-23 16:08:02+02	2020-09-23 16:08:27+02	devel	mips64-xtoolchain-gcc	\N	mips64-xtoolchain-gcc-0.4_1
2020-09-23 16:08:30+02	2020-09-23 16:08:50+02	devel	powerpc64-xtoolchain-gcc	\N	powerpc64-xtoolchain-gcc-0.4_1
2020-09-23 16:08:53+02	2020-09-23 16:09:23+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-09-23 16:09:29+02	2020-09-23 16:09:50+02	devel	riscv64-xtoolchain-gcc	\N	riscv64-xtoolchain-gcc-0.4_1
2020-09-23 16:09:51+02	2020-09-23 16:10:18+02	devel	sparc64-xtoolchain-gcc	\N	sparc64-xtoolchain-gcc-0.4_1
2020-09-23 16:10:21+02	2020-09-23 16:21:19+02	devel	subversion	\N	subversion-1.14.0
2020-09-23 16:21:21+02	2020-09-23 16:22:30+02	devel	tig	\N	tig-2.5.1
2020-09-23 16:22:32+02	2020-09-23 16:23:49+02	devel	tortoisehg	py37	tortoisehg-5.5.1
2020-09-23 16:23:52+02	2020-09-23 16:31:16+02	dns	bind916	\N	bind916-9.16.6_1
2020-09-23 16:31:24+02	2020-09-23 18:26:21+02	editors	codelite	\N	codelite-12.0_4
2020-09-23 18:26:22+02	2020-09-23 18:27:29+02	editors	fpc-ide	\N	fpc-ide-3.0.4_4
2020-09-23 18:27:31+02	2020-09-23 18:30:50+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-23 18:30:54+02	2020-09-23 18:37:20+02	editors	vim	\N	vim-8.2.1558
2020-09-23 18:37:21+02	2020-09-23 20:04:25+02	emulators	qemu	\N	qemu-4.2.1
2020-09-22 00:21:43+02	2020-09-23 20:10:27+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-23 20:10:31+02	2020-09-23 20:12:21+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-09-23 20:12:22+02	2020-09-23 20:14:07+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-09-23 20:14:08+02	2020-09-23 21:16:37+02	emulators	wine	\N	wine-5.0.2,1
2020-09-23 21:16:39+02	2020-09-23 21:17:32+02	emulators	wine-gecko	\N	wine-gecko-2.47.1
2020-09-23 21:17:32+02	2020-09-23 21:19:11+02	emulators	wine-mono	\N	wine-mono-4.9.4_1
2020-09-23 21:19:11+02	2020-09-23 21:20:21+02	emulators	x48	\N	x48-0.6.4_1
2020-09-23 20:04:26+02	2020-09-23 21:29:44+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-09-23 21:29:46+02	2020-09-23 21:31:56+02	ftp	wget	\N	wget-1.20.3
2020-09-23 21:31:56+02	2020-09-23 21:32:26+02	games	KnightCap	\N	knightcap-3.6_8
2020-09-23 21:32:27+02	2020-09-23 21:32:40+02	games	affenspiel	\N	affenspiel-1.0_2
2020-09-23 21:32:41+02	2020-09-23 21:33:10+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-09-23 21:33:11+02	2020-09-23 21:36:51+02	games	alienarena	\N	alienarena-2013.766_9
2020-09-23 21:20:21+02	2020-09-23 21:37:56+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-23 21:36:52+02	2020-09-23 21:38:37+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-09-23 21:38:38+02	2020-09-23 21:41:17+02	games	bsdgames	\N	bsdgames-0.75,2
2020-09-23 21:41:18+02	2020-09-23 21:41:50+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-09-23 21:41:50+02	2020-09-23 21:42:43+02	games	monsterz	\N	monsterz-0.7.1_15
2020-09-23 21:42:44+02	2020-09-23 21:43:29+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-09-23 21:43:29+02	2020-09-23 21:44:40+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-09-23 21:44:41+02	2020-09-23 21:45:01+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-09-23 21:45:01+02	2020-09-23 21:46:12+02	games	stockfish	\N	stockfish-11
2020-09-23 21:37:57+02	2020-09-23 21:46:29+02	games	battletanks	\N	btanks-0.9.8083_9
2020-09-23 21:46:13+02	2020-09-23 21:47:27+02	games	xblast	\N	xblast-2.10.4_18
2020-09-23 21:46:30+02	2020-09-23 21:48:04+02	games	xboard	\N	xboard-4.9.1_1
2020-09-23 21:47:28+02	2020-09-23 21:48:25+02	games	xcowsay	\N	xcowsay-1.5
2020-09-23 21:48:28+02	2020-09-23 21:55:00+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-23 21:55:02+02	2020-09-23 21:56:55+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-23 21:56:56+02	2020-09-23 23:22:26+02	graphics	inkscape	\N	inkscape-1.0_5
2020-09-23 21:48:05+02	2020-09-23 23:36:51+02	graphics	blender	\N	blender-2.90.0
2020-09-23 23:36:52+02	2020-09-23 23:37:07+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-09-23 23:37:08+02	2020-09-23 23:41:03+02	graphics	wings	\N	wings-2.2.5
2020-09-23 23:41:03+02	2020-09-23 23:43:37+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-23 23:43:38+02	2020-09-23 23:45:44+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-09-23 15:27:29+02	2020-09-23 23:47:03+02	devel	llvm10	\N	llvm10-10.0.1_1
2020-09-23 23:47:06+02	2020-09-23 23:50:06+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-23 23:45:45+02	2020-09-24 00:20:19+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-09-23 23:50:08+02	2020-09-24 00:44:57+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-24 00:44:57+02	2020-09-24 00:45:20+02	lang	gcc	\N	gcc-9_4
2020-09-24 00:20:20+02	2020-09-24 01:10:34+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.4_1
2020-09-24 01:11:52+02	2020-09-24 01:12:23+02	lang	mmix	\N	mmix-20160804
2020-09-23 23:22:30+02	2020-09-24 01:20:37+02	graphics	krita	\N	krita-4.3.0_5
2020-09-24 01:20:38+02	2020-09-24 01:20:56+02	lang	python	\N	python-3.7_3,2
2020-09-24 01:20:57+02	2020-09-24 01:21:05+02	lang	python-mode.el	\N	python-mode.el-1.0_2,1
2020-09-24 01:21:05+02	2020-09-24 01:27:45+02	mail	alpine	\N	alpine-2.23.2
2020-09-24 01:27:45+02	2020-09-24 01:28:09+02	mail	bbmail	\N	bbmail-0.9.3
2020-09-24 01:28:09+02	2020-09-24 01:36:22+02	math	maxima	\N	maxima-5.44.0_4
2020-09-24 01:36:22+02	2020-09-24 01:36:32+02	math	sc	\N	sc-7.16_3
2020-09-24 01:36:33+02	2020-09-24 01:37:36+02	misc	gnu-watch	\N	gnu-watch-3.3.16
2020-09-24 01:37:36+02	2020-09-24 01:37:50+02	misc	mbuffer	\N	mbuffer-20200505
2020-09-24 01:37:51+02	2020-09-24 01:41:14+02	misc	mc	\N	mc-nox11-4.8.24
2020-09-24 01:41:15+02	2020-09-24 01:42:18+02	misc	pinfo	\N	pinfo-0.6.10_5
2020-09-24 01:42:19+02	2020-09-24 01:43:49+02	multimedia	celluloid	\N	celluloid-0.20
2020-09-24 01:43:51+02	2020-09-24 01:44:08+02	multimedia	libva-glx	\N	libva-glx-2.9.0
2020-09-24 01:44:08+02	2020-09-24 01:45:25+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-09-24 01:45:25+02	2020-09-24 02:17:29+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p8
2020-09-24 02:17:31+02	2020-09-24 02:18:14+02	multimedia	libva-utils	\N	libva-utils-2.9.0
2020-09-24 02:18:15+02	2020-09-24 02:18:31+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-09-24 01:12:23+02	2020-09-24 02:19:23+02	lang	mono	\N	mono-5.10.1.57_3
2020-09-24 02:19:24+02	2020-09-24 02:22:52+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-09-24 02:18:32+02	2020-09-24 02:27:24+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-09-24 02:27:24+02	2020-09-24 02:28:04+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-09-24 02:28:05+02	2020-09-24 02:29:43+02	net	mosh	\N	mosh-1.3.2_14
2020-09-24 02:22:53+02	2020-09-24 02:36:19+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-24 02:29:44+02	2020-09-24 02:36:37+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-09-24 02:36:20+02	2020-09-24 02:37:05+02	net	rdesktop	\N	rdesktop-1.9.0
2020-09-24 02:37:05+02	2020-09-24 02:37:14+02	net	svnup	\N	svnup-1.08
2020-09-24 02:37:15+02	2020-09-24 02:37:37+02	net	torsocks	\N	torsocks-2.3.0_1
2020-09-24 02:36:37+02	2020-09-24 02:38:21+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-24 02:38:22+02	2020-09-24 02:38:39+02	net	vnstat	\N	vnstat-2.6_1
2020-09-24 02:37:37+02	2020-09-24 02:38:51+02	net	v2ray	\N	v2ray-4.25.0
2020-09-24 02:38:40+02	2020-09-24 02:39:18+02	net	wireguard	\N	wireguard-1.0.20200827
2020-09-24 02:39:19+02	2020-09-24 02:40:29+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-24 02:38:52+02	2020-09-24 03:08:41+02	net	wireshark	\N	wireshark-3.2.6
2020-09-24 02:40:30+02	2020-09-24 03:09:03+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-09-24 03:08:42+02	2020-09-24 03:09:42+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-09-24 03:09:04+02	2020-09-24 03:09:59+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-09-24 03:09:43+02	2020-09-24 03:10:43+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-09-24 03:10:00+02	2020-09-24 03:10:57+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-09-24 03:10:43+02	2020-09-24 03:11:45+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-09-24 03:10:58+02	2020-09-24 03:12:08+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-09-24 03:11:46+02	2020-09-24 03:12:53+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-09-24 03:12:09+02	2020-09-24 03:14:05+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-09-24 03:12:54+02	2020-09-24 03:14:15+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-09-24 03:14:15+02	2020-09-24 03:14:33+02	net-mgmt	iftop	\N	iftop-1.0.p4
2020-09-24 03:14:33+02	2020-09-24 03:15:00+02	net-mgmt	ipcalc	\N	ipcalc-0.41_3
2020-09-24 03:14:06+02	2020-09-24 03:15:23+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-09-24 03:15:23+02	2020-09-24 03:15:37+02	net-mgmt	nstat	\N	nstat-1.0_3
2020-09-24 03:15:38+02	2020-09-24 03:15:49+02	net-mgmt	whatmask	\N	whatmask-1.2_1
2020-09-24 03:15:01+02	2020-09-24 03:15:56+02	net-mgmt	ipv6calc	\N	ipv6calc-2.2.0
2020-09-24 03:15:56+02	2020-09-24 03:16:19+02	ports-mgmt	pkg_tree	\N	pkg_tree-1.1_4
2020-09-24 03:16:20+02	2020-09-24 03:17:40+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-09-24 03:17:41+02	2020-09-24 03:18:33+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-09-24 03:15:50+02	2020-09-24 03:19:06+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-09-24 03:18:33+02	2020-09-24 03:20:49+02	ports-mgmt	synth	\N	synth-2.08
2020-09-24 03:20:49+02	2020-09-24 03:21:51+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-09-24 03:21:54+02	2020-09-24 03:22:56+02	print	gv	\N	gv-3.7.4_5
2020-09-24 03:19:07+02	2020-09-24 03:22:56+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-24 03:22:57+02	2020-09-24 03:56:58+02	print	lyx	\N	lyx-2.3.4.2_1
2020-09-24 03:57:00+02	2020-09-24 04:04:54+02	print	texlive-full	\N	texlive-full-20150521
2020-09-24 04:04:55+02	2020-09-24 04:06:06+02	security	aide	\N	aide-0.16.1
2020-09-24 04:06:06+02	2020-09-24 04:06:16+02	security	base-audit	\N	base-audit-0.4
2020-09-24 04:06:16+02	2020-09-24 04:11:23+02	security	clamav	\N	clamav-0.103.0,1
2020-09-24 04:11:24+02	2020-09-24 04:11:35+02	security	doas	\N	doas-6.3_1
2020-09-24 04:11:36+02	2020-09-24 04:12:20+02	security	ipv6toolkit	\N	ipv6toolkit-2.0
2020-09-24 04:12:20+02	2020-09-24 04:12:51+02	security	kc24	\N	kc24-2.4.2_1
2020-09-24 04:12:51+02	2020-09-24 04:18:38+02	security	klee	\N	klee-2.1_1
2020-09-24 04:18:39+02	2020-09-24 04:18:49+02	security	lynis	\N	lynis-3.0.0
2020-09-24 04:18:50+02	2020-09-24 04:19:08+02	security	nyx	\N	nyx-2.1.0_1
2020-09-24 04:19:08+02	2020-09-24 04:20:50+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-09-24 04:20:51+02	2020-09-24 04:22:23+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-09-24 04:22:24+02	2020-09-24 04:22:46+02	security	py-xkcdpass	py37	py37-xkcdpass-1.17.4
2020-09-24 04:22:46+02	2020-09-24 04:23:11+02	security	rkhunter	\N	rkhunter-1.4.6
2020-09-24 04:23:11+02	2020-09-24 04:23:35+02	security	ssl-admin	\N	ssl-admin-1.2.1
2020-09-24 04:23:36+02	2020-09-24 04:31:32+02	security	tor	\N	tor-0.4.4.5
2020-09-24 03:22:57+02	2020-09-24 04:34:16+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-09-24 04:31:33+02	2020-09-24 04:41:05+02	shells	fish	\N	fish-3.1.2
2020-09-24 04:41:06+02	2020-09-24 04:44:22+02	shells	zsh	\N	zsh-5.8
2020-09-24 04:44:23+02	2020-09-24 04:44:31+02	sysutils	915resolution	\N	915resolution-0.5.3_1,1
2020-09-24 04:34:17+02	2020-09-24 04:45:15+02	shells	starship	\N	starship-0.44.0_2
2020-09-24 04:45:16+02	2020-09-24 04:45:54+02	sysutils	apachetop	\N	apachetop-0.18.4_1
2020-09-24 04:45:54+02	2020-09-24 04:46:05+02	sysutils	atop	\N	atop-2.0.2.b3_3
2020-09-24 04:46:06+02	2020-09-24 04:46:17+02	sysutils	bashtop	\N	bashtop-0.9.25_1
2020-09-24 04:46:18+02	2020-09-24 04:46:25+02	sysutils	beadm	\N	beadm-1.3.0
2020-09-24 04:46:26+02	2020-09-24 04:46:36+02	sysutils	bpytop	\N	bpytop-1.0.33
2020-09-24 04:46:37+02	2020-09-24 04:46:44+02	sysutils	bsdcrashtar	\N	bsdcrashtar-0.1.3_1
2020-09-24 04:46:45+02	2020-09-24 04:46:53+02	sysutils	bsdhwmon	\N	bsdhwmon-20151206
2020-09-24 04:46:53+02	2020-09-24 04:47:04+02	sysutils	bsdinfo	\N	bsdinfo-0.22_2
2020-09-24 04:44:31+02	2020-09-24 04:47:10+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-09-24 04:47:04+02	2020-09-24 04:47:11+02	sysutils	bsdstats	\N	bsdstats-7.0
2020-09-24 04:47:12+02	2020-09-24 04:47:21+02	sysutils	cmdwatch	\N	cmdwatch-0.2.0_2
2020-09-24 04:47:21+02	2020-09-24 04:48:16+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-24 04:48:16+02	2020-09-24 04:48:23+02	sysutils	cpupdate	\N	cpupdate-g20180513_2
2020-09-24 04:48:24+02	2020-09-24 04:48:39+02	sysutils	ddrescue	\N	ddrescue-1.25
2020-09-24 04:48:39+02	2020-09-24 04:48:47+02	sysutils	devcpu-data	\N	devcpu-data-1.34
2020-09-24 04:48:48+02	2020-09-24 04:49:03+02	sysutils	dfc	\N	dfc-3.1.1
2020-09-24 04:49:03+02	2020-09-24 04:49:22+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-09-24 04:47:10+02	2020-09-24 04:50:24+02	sysutils	cdrtools	\N	cdrtools-3.01_1
2020-09-24 04:50:24+02	2020-09-24 04:50:45+02	sysutils	flashrom	\N	flashrom-1.2
2020-09-24 00:45:21+02	2020-09-24 04:50:56+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200913
2020-09-24 04:50:45+02	2020-09-24 04:51:29+02	sysutils	fusefs-ntfs	\N	fusefs-ntfs-2017.3.23
2020-09-24 04:49:22+02	2020-09-24 04:51:51+02	sysutils	exa	\N	exa-0.9.0_15
2020-09-24 04:50:58+02	2020-09-24 04:51:56+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-24 04:51:30+02	2020-09-24 04:52:37+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-24 04:52:37+02	2020-09-24 04:52:49+02	sysutils	hourglass	\N	hourglass-1.0.1
2020-09-24 04:51:56+02	2020-09-24 04:53:23+02	sysutils	hexyl	\N	hexyl-0.8.0_4
2020-09-24 04:52:49+02	2020-09-24 04:53:26+02	sysutils	htop	\N	htop-3.0.2
2020-09-24 04:53:25+02	2020-09-24 04:53:32+02	sysutils	ioping	\N	ioping-1.2
2020-09-24 04:53:26+02	2020-09-24 04:53:33+02	sysutils	lsop	\N	lsop-0.2
2020-09-24 04:53:34+02	2020-09-24 04:53:50+02	sysutils	most	\N	most-5.1.0
2020-09-24 04:53:33+02	2020-09-24 04:53:55+02	sysutils	memtest86+	\N	memtest86+-5.01_3
2020-09-24 04:53:55+02	2020-09-24 04:54:02+02	sysutils	neofetch	\N	neofetch-7.1.0
2020-09-24 04:53:54+02	2020-09-24 04:54:04+02	sysutils	ncdu	\N	ncdu-1.15.1
2020-09-24 04:54:03+02	2020-09-24 04:54:09+02	sysutils	obliterate	\N	obliterate-0.5
2020-09-24 04:51:52+02	2020-09-24 04:54:13+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-24 04:54:05+02	2020-09-24 04:54:19+02	sysutils	panicmail	\N	panicmail-1.1
2020-09-24 04:54:13+02	2020-09-24 04:54:20+02	sysutils	procmap	\N	procmap-1.0
2020-09-24 04:54:10+02	2020-09-24 04:54:27+02	sysutils	parallel	\N	parallel-20200822
2020-09-24 04:54:19+02	2020-09-24 04:54:31+02	sysutils	psmisc	\N	psmisc-22.16_1
2020-09-24 04:54:27+02	2020-09-24 04:54:34+02	sysutils	pwgen	\N	pwgen-2.08,2
2020-09-24 04:54:31+02	2020-09-24 04:54:39+02	sysutils	py-py-cpuinfo	py37	py37-py-cpuinfo-7.0.0
2020-09-24 04:54:35+02	2020-09-24 04:54:41+02	sysutils	reptyr	\N	reptyr-0.6.3a_2
2020-09-24 04:54:29+02	2020-09-24 04:54:42+02	sysutils	py-glances	py37	py37-glances-3.1.4
2020-09-24 04:54:42+02	2020-09-24 04:54:50+02	sysutils	spinner	\N	spinner-1.2.4
2020-09-24 04:54:51+02	2020-09-24 04:54:57+02	sysutils	ssd_report	smart	ssd_report-0.4
2020-09-24 04:54:41+02	2020-09-24 04:55:12+02	sysutils	screen	\N	screen-4.8.0
2020-09-24 04:55:12+02	2020-09-24 04:55:18+02	sysutils	tarsnap-periodic	\N	tarsnap-periodic-1.0.20170413
2020-09-24 04:55:18+02	2020-09-24 04:55:59+02	sysutils	tmux	\N	tmux-3.1b
2020-09-24 04:56:00+02	2020-09-24 04:56:11+02	sysutils	tree	\N	tree-1.8.0
2020-09-24 04:56:11+02	2020-09-24 04:56:30+02	sysutils	x86info	\N	x86info-1.31.s03
2020-09-24 04:56:31+02	2020-09-24 04:56:52+02	sysutils	zfs-stats	\N	zfs-stats-1.3.0
2020-09-24 04:56:53+02	2020-09-24 04:58:14+02	sysutils	zrepl	\N	zrepl-0.2.1
2020-09-24 04:54:57+02	2020-09-24 04:58:36+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-09-24 04:58:37+02	2020-09-24 04:58:53+02	textproc	colordiff	\N	colordiff-1.0.19
2020-09-24 04:58:53+02	2020-09-24 04:59:16+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-09-24 04:59:17+02	2020-09-24 04:59:41+02	www	dokuwiki	\N	dokuwiki-20200729
2020-09-24 04:54:40+02	2020-09-24 05:04:52+02	sysutils	rsfetch	\N	rsfetch-2.0.0_7
2020-09-24 05:04:52+02	2020-09-24 05:06:01+02	www	links	\N	links-2.20.2,1
2020-09-24 05:06:02+02	2020-09-24 05:08:23+02	www	lynx	\N	lynx-2.8.9.1_1,1
2020-09-24 05:08:23+02	2020-09-24 05:10:44+02	www	mattermost-server	\N	mattermost-server-5.26.1
2020-09-24 05:10:45+02	2020-09-24 05:11:09+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_2
2020-09-24 04:58:14+02	2020-09-24 05:11:35+02	textproc	bat	\N	bat-0.15.4_4
2020-09-24 05:11:36+02	2020-09-24 05:12:36+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-09-24 05:12:37+02	2020-09-24 05:12:49+02	www	py-bottle	py37	py37-bottle-0.12.15_1
2020-09-24 05:12:49+02	2020-09-24 05:13:07+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-09-24 05:13:08+02	2020-09-24 05:13:24+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-09-24 05:13:25+02	2020-09-24 05:13:47+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-09-24 05:13:47+02	2020-09-24 05:14:04+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-09-24 05:14:05+02	2020-09-24 05:14:24+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-09-24 05:14:24+02	2020-09-24 05:14:39+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-09-24 05:14:40+02	2020-09-24 05:15:02+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-09-24 05:15:02+02	2020-09-24 05:15:22+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-09-24 05:15:23+02	2020-09-24 05:15:41+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-09-24 05:15:42+02	2020-09-24 05:15:57+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-09-24 05:15:58+02	2020-09-24 05:16:23+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-09-24 05:11:10+02	2020-09-24 05:16:29+02	www	mod_php74	\N	mod_php74-7.4.10
2020-09-24 05:16:24+02	2020-09-24 05:16:37+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-09-24 05:16:39+02	2020-09-24 05:19:07+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-09-24 05:19:08+02	2020-09-24 05:21:22+02	x11	mate	\N	mate-1.22.2
2020-09-24 05:21:22+02	2020-09-24 05:21:39+02	x11	numlockx	\N	numlockx-1.2
2020-09-24 05:21:39+02	2020-09-24 05:21:55+02	x11	sterm	\N	sterm-0.8.4
2020-09-24 05:21:56+02	2020-09-24 05:22:28+02	x11	xclip	\N	xclip-0.13
2020-09-24 05:22:28+02	2020-09-24 05:22:48+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-09-24 05:22:49+02	2020-09-24 05:23:25+02	x11	xdm	\N	xdm-1.1.12_3
2020-09-24 05:23:26+02	2020-09-24 05:25:32+02	x11	xlockmore	\N	xlockmore-5.65
2020-09-24 05:25:32+02	2020-09-24 05:25:50+02	x11	xman	\N	xman-1.1.5
2020-09-24 05:25:51+02	2020-09-24 05:32:46+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-24 05:16:30+02	2020-09-24 05:38:00+02	x11	alacritty	\N	alacritty-0.5.0_2
2020-09-24 05:32:47+02	2020-09-24 05:38:41+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-24 05:38:42+02	2020-09-24 05:39:06+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-09-24 05:39:06+02	2020-09-24 05:39:20+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-09-24 05:38:01+02	2020-09-24 05:39:55+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-24 05:39:21+02	2020-09-24 05:40:22+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-09-24 05:39:56+02	2020-09-24 05:40:22+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-09-24 05:40:23+02	2020-09-24 05:40:45+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-09-24 05:40:22+02	2020-09-24 05:41:10+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-09-24 05:41:10+02	2020-09-24 05:42:25+02	graphics	xfig	\N	xfig-3.2.7a
2020-09-24 05:42:26+02	2020-09-24 05:43:10+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-09-24 05:40:45+02	2020-09-24 06:10:25+02	editors	vscode	\N	vscode-1.46.1
2020-09-24 04:59:42+02	2020-09-24 07:05:46+02	www	firefox	\N	firefox-81.0,2
2020-09-24 07:17:30+02	2020-09-24 07:18:42+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-24 07:17:30+02	2020-09-24 07:19:20+02	graphics	gpu-firmware-kmod	\N	gpu-firmware-kmod-g20200914
2020-09-24 07:19:20+02	2020-09-24 07:25:35+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-24 07:25:36+02	2020-09-24 07:25:43+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-24 21:32:48+02	2020-09-24 21:37:35+02	ports-mgmt	pkg	\N	pkg-1.15.4
2020-09-24 21:52:44+02	2020-09-24 21:53:02+02	devel	autoconf-wrapper	\N	autoconf-wrapper-20131203
2020-09-24 21:52:45+02	2020-09-24 21:53:04+02	misc	pciids	\N	pciids-20200819
2020-09-24 21:53:04+02	2020-09-24 21:53:19+02	devel	evdev-proto	\N	evdev-proto-5.3
2020-09-24 21:53:05+02	2020-09-24 21:53:21+02	sysutils	gnome_subr	\N	gnome_subr-1.0
2020-09-24 21:52:44+02	2020-09-24 21:53:33+02	devel	ccache	\N	ccache-3.7.1_1
2020-09-24 21:53:22+02	2020-09-24 21:53:48+02	print	gsfonts	\N	gsfonts-8.11_8
2020-09-24 21:53:35+02	2020-09-24 21:53:52+02	print	indexinfo	\N	indexinfo-0.3.1
2020-09-24 21:53:22+02	2020-09-24 21:54:42+02	multimedia	v4l_compat	\N	v4l_compat-1.18.0
2020-09-24 21:53:49+02	2020-09-24 21:55:28+02	devel	pkgconf	\N	pkgconf-1.7.3,1
2020-09-24 21:55:30+02	2020-09-24 21:56:51+02	devel	gmake	\N	gmake-4.3_2
2020-09-24 21:53:56+02	2020-09-24 21:58:00+02	devel	gettext-runtime	\N	gettext-runtime-0.21
2020-09-24 21:54:45+02	2020-09-24 21:58:38+02	devel	libtextstyle	\N	libtextstyle-0.21
2020-09-24 21:58:02+02	2020-09-24 21:58:59+02	devel	readline	\N	readline-8.0.4
2020-09-24 21:59:01+02	2020-09-24 22:00:18+02	devel	libffi	\N	libffi-3.3_1
2020-09-24 22:00:19+02	2020-09-24 22:02:20+02	converters	libiconv	\N	libiconv-1.16
2020-09-24 22:02:22+02	2020-09-24 22:03:03+02	textproc	expat2	\N	expat-2.2.8
2020-09-24 22:03:04+02	2020-09-24 22:05:44+02	textproc	libxml2	\N	libxml2-2.9.10
2020-09-24 22:05:48+02	2020-09-24 22:06:25+02	devel	xorg-macros	\N	xorg-macros-1.19.2
2020-09-24 22:06:27+02	2020-09-24 22:08:09+02	x11	xorgproto	\N	xorgproto-2020.1
2020-09-24 22:08:11+02	2020-09-24 22:09:43+02	archivers	liblz4	\N	liblz4-1.9.2_1,1
2020-09-24 21:56:53+02	2020-09-24 22:12:12+02	lang	perl5.32	\N	perl5-5.32.0
2020-09-24 22:09:45+02	2020-09-24 22:12:27+02	archivers	lzo2	\N	lzo2-2.10_1
2020-09-24 22:12:15+02	2020-09-24 22:12:41+02	converters	p5-Text-Unidecode	\N	p5-Text-Unidecode-1.30
2020-09-24 21:58:39+02	2020-09-24 22:12:44+02	devel	gettext-tools	\N	gettext-tools-0.21
2020-09-24 22:12:28+02	2020-09-24 22:13:04+02	textproc	p5-Unicode-EastAsianWidth	\N	p5-Unicode-EastAsianWidth-12.0
2020-09-24 22:12:48+02	2020-09-24 22:13:15+02	devel	p5-Locale-gettext	\N	p5-Locale-gettext-1.07
2020-09-24 22:13:04+02	2020-09-24 22:13:26+02	devel	p5-Locale-libintl	\N	p5-Locale-libintl-1.31
2020-09-24 22:13:15+02	2020-09-24 22:13:34+02	misc	help2man	\N	help2man-1.47.16
2020-09-25 04:06:55+02	2020-09-25 04:07:07+02	x11	xkbevd	\N	xkbevd-1.1.4
2020-09-24 22:13:26+02	2020-09-24 22:13:48+02	security	ca_root_nss	\N	ca_root_nss-3.56
2020-09-24 22:13:48+02	2020-09-24 22:14:23+02	archivers	libarchive	\N	libarchive-3.4.3,1
2020-09-24 22:13:35+02	2020-09-24 22:14:27+02	print	texinfo	\N	texinfo-6.7_4,1
2020-09-24 22:14:24+02	2020-09-24 22:14:37+02	security	rhash	\N	rhash-1.3.9
2020-09-24 22:14:38+02	2020-09-24 22:14:45+02	devel	libpthread-stubs	\N	libpthread-stubs-0.4
2020-09-24 22:14:46+02	2020-09-24 22:14:59+02	x11	libXau	\N	libXau-1.0.9
2020-09-24 22:14:27+02	2020-09-24 22:14:59+02	devel	m4	\N	m4-1.4.18_1,1
2020-09-24 22:12:48+02	2020-09-24 22:15:05+02	lang	python37	\N	python37-3.7.9
2020-09-24 22:14:59+02	2020-09-24 22:15:11+02	x11	libXdmcp	\N	libXdmcp-1.1.3
2020-09-24 22:15:06+02	2020-09-24 22:15:19+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-09-24 22:15:19+02	2020-09-24 22:15:30+02	devel	py-six	py37	py37-six-1.15.0
2020-09-24 22:15:00+02	2020-09-24 22:15:31+02	devel	autoconf	\N	autoconf-2.69_3
2020-09-24 22:15:12+02	2020-09-24 22:15:34+02	devel	libtool	\N	libtool-2.4.6_1
2020-09-24 22:15:30+02	2020-09-24 22:15:41+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-09-24 22:15:35+02	2020-09-24 22:15:45+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-09-24 22:15:33+02	2020-09-24 22:15:49+02	devel	automake	\N	automake-1.16.2
2020-09-24 22:15:41+02	2020-09-24 22:15:52+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-09-24 22:15:46+02	2020-09-24 22:15:56+02	devel	py-cffi	py37	py37-cffi-1.14.3
2020-09-24 22:15:49+02	2020-09-24 22:15:59+02	dns	py-idna	py37	py37-idna-2.8
2020-09-24 22:15:57+02	2020-09-24 22:16:11+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-09-24 22:16:11+02	2020-09-24 22:16:22+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-09-24 22:15:52+02	2020-09-24 22:16:23+02	www	libnghttp2	\N	libnghttp2-1.41.0
2020-09-24 22:16:24+02	2020-09-24 22:16:33+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-09-24 22:16:34+02	2020-09-24 22:16:44+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-09-24 22:16:22+02	2020-09-24 22:17:04+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-09-24 22:16:00+02	2020-09-24 22:17:05+02	devel	ninja	\N	ninja-1.10.1,2
2020-09-24 22:16:44+02	2020-09-24 22:17:06+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-09-24 22:17:04+02	2020-09-24 22:17:14+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-09-24 22:17:06+02	2020-09-24 22:17:17+02	net	py-urllib3	py37	py37-urllib3-1.25.10,1
2020-09-24 22:17:05+02	2020-09-24 22:17:19+02	devel	meson	\N	meson-0.55.3
2020-09-24 22:17:15+02	2020-09-24 22:17:25+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-09-24 22:17:20+02	2020-09-24 22:17:31+02	devel	py-Jinja2	py37	py37-Jinja2-2.10.1
2020-09-24 22:17:32+02	2020-09-24 22:17:43+02	www	py-requests	py37	py37-requests-2.22.0
2020-09-24 22:17:43+02	2020-09-24 22:17:54+02	devel	py-packaging	py37	py37-packaging-20.4
2020-09-24 22:17:25+02	2020-09-24 22:17:54+02	lang	cython	py37	py37-cython-0.29.15
2020-09-24 22:17:55+02	2020-09-24 22:18:06+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-09-24 22:17:55+02	2020-09-24 22:18:10+02	textproc	py-pygments	py37	py37-pygments-2.6.1
2020-09-24 22:18:11+02	2020-09-24 22:18:20+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-09-24 22:18:21+02	2020-09-24 22:18:31+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-09-24 22:17:17+02	2020-09-24 22:18:37+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-09-24 22:18:31+02	2020-09-24 22:18:42+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-09-24 22:18:38+02	2020-09-24 22:18:49+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-09-24 22:18:42+02	2020-09-24 22:18:52+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-09-24 22:18:49+02	2020-09-24 22:18:59+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-09-24 22:18:52+02	2020-09-24 22:19:02+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-09-24 22:19:00+02	2020-09-24 22:19:11+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-09-24 22:19:02+02	2020-09-24 22:19:12+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-09-24 22:18:06+02	2020-09-24 22:19:14+02	ftp	curl	\N	curl-7.72.0
2020-09-24 22:19:14+02	2020-09-24 22:19:25+02	devel	jsoncpp	\N	jsoncpp-1.9.3
2020-09-24 22:19:13+02	2020-09-24 22:19:33+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-09-24 22:19:25+02	2020-09-24 22:19:37+02	x11	xcb-proto	\N	xcb-proto-1.14
2020-09-24 22:19:11+02	2020-09-24 22:19:39+02	devel	libuv	\N	libuv-1.39.0
2020-09-24 22:19:33+02	2020-09-24 22:19:43+02	x11	xtrans	\N	xtrans-1.4.0
2020-09-24 22:19:37+02	2020-09-24 22:20:13+02	x11	libxcb	\N	libxcb-1.14_1
2020-09-24 22:19:43+02	2020-09-24 22:20:59+02	math	gmp	\N	gmp-6.2.0
2020-09-24 22:20:13+02	2020-09-24 22:21:12+02	x11	libX11	\N	libX11-1.6.12,1
2020-09-24 22:20:59+02	2020-09-24 22:21:22+02	math	mpfr	\N	mpfr-4.1.0
2020-09-24 22:21:12+02	2020-09-24 22:21:25+02	security	libgpg-error	\N	libgpg-error-1.39
2020-09-24 22:21:23+02	2020-09-24 22:21:34+02	x11	libXext	\N	libXext-1.3.4,1
2020-09-24 22:21:35+02	2020-09-24 22:21:52+02	devel	libedit	\N	libedit-3.1.20191231,1
2020-09-24 22:19:40+02	2020-09-24 22:21:55+02	devel	cmake	\N	cmake-3.18.2
2020-09-24 22:21:26+02	2020-09-24 22:21:57+02	security	libgcrypt	\N	libgcrypt-1.8.5
2020-09-24 22:21:57+02	2020-09-24 22:22:14+02	textproc	libxslt	\N	libxslt-1.1.34_1
2020-09-24 22:21:56+02	2020-09-24 22:22:16+02	devel	pcre	\N	pcre-8.44
2020-09-24 22:22:14+02	2020-09-24 22:22:36+02	print	freetype2	\N	freetype2-2.10.2_1
2020-09-24 22:22:37+02	2020-09-24 22:22:54+02	devel	gperf	\N	gperf-3.1
2020-09-24 22:22:54+02	2020-09-24 22:23:23+02	x11-fonts	fontconfig	\N	fontconfig-2.13.92_2,1
2020-09-24 22:22:17+02	2020-09-24 22:23:23+02	devel	bison	\N	bison-3.6.4,1
2020-09-24 22:23:24+02	2020-09-24 22:23:31+02	lang	lua52	\N	lua52-5.2.4
2020-09-24 22:23:24+02	2020-09-24 22:23:39+02	x11	libXrender	\N	libXrender-0.9.10_2
2020-09-24 22:23:40+02	2020-09-24 22:23:53+02	x11	libXfixes	\N	libXfixes-5.0.3_2
2020-09-24 22:23:32+02	2020-09-24 22:23:55+02	devel	libunwind	\N	libunwind-20200331_1
2020-09-24 22:23:54+02	2020-09-24 22:24:06+02	textproc	xmlcatmgr	\N	xmlcatmgr-2.2_2
2020-09-24 22:23:56+02	2020-09-24 22:24:15+02	devel	py-future	py37	py37-future-0.18.2
2020-09-24 22:24:06+02	2020-09-24 22:24:19+02	devel	libepoll-shim	\N	libepoll-shim-0.0.20200602
2020-09-24 22:24:15+02	2020-09-24 22:24:25+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-09-24 22:24:20+02	2020-09-24 22:24:30+02	x11	libXrandr	\N	libXrandr-1.5.2
2020-09-24 22:24:26+02	2020-09-24 22:24:32+02	textproc	iso8879	\N	iso8879-1986_3
2020-09-24 22:24:32+02	2020-09-24 22:24:38+02	textproc	xmlcharent	\N	xmlcharent-0.3_2
2020-09-24 22:24:30+02	2020-09-24 22:24:40+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-09-24 22:24:40+02	2020-09-24 22:24:55+02	graphics	wayland	\N	wayland-1.18.0_4
2020-09-24 22:24:39+02	2020-09-24 22:25:01+02	devel	swig	\N	swig-4.0.2
2020-09-24 22:24:55+02	2020-09-24 22:25:03+02	textproc	docbook-sgml	\N	docbook-sgml-4.5_1
2020-09-24 22:25:02+02	2020-09-24 22:25:09+02	textproc	docbook-xml	\N	docbook-xml-5.0_3
2020-09-24 22:25:04+02	2020-09-24 22:25:14+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-09-24 22:25:10+02	2020-09-24 22:25:16+02	textproc	sdocbook-xml	\N	sdocbook-xml-1.1_2,2
2020-09-24 22:25:16+02	2020-09-24 22:25:23+02	textproc	docbook	\N	docbook-1.5
2020-09-24 22:25:15+02	2020-09-24 22:25:27+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-09-24 22:25:28+02	2020-09-24 22:25:38+02	x11	libXxf86vm	\N	libXxf86vm-1.1.4_3
2020-09-24 22:21:52+02	2020-09-24 22:25:40+02	devel	binutils	native	binutils-2.33.1_3,1
2020-09-24 22:25:38+02	2020-09-24 22:25:48+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-09-24 22:25:40+02	2020-09-24 22:25:50+02	x11	libXdamage	\N	libXdamage-1.1.5
2020-09-24 22:25:23+02	2020-09-24 22:25:55+02	textproc	docbook-xsl	\N	docbook-xsl-1.79.1_1,1
2020-09-24 22:25:52+02	2020-09-24 22:25:59+02	graphics	wayland-protocols	\N	wayland-protocols-1.20
2020-09-24 22:25:56+02	2020-09-24 22:26:07+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-09-24 22:26:00+02	2020-09-24 22:26:10+02	x11	libxshmfence	\N	libxshmfence-1.3
2020-09-24 22:26:07+02	2020-09-24 22:26:17+02	graphics	png	\N	png-1.6.37
2020-09-24 22:26:17+02	2020-09-24 22:27:43+02	devel	icu	\N	icu-67.1,1
2020-09-24 22:26:10+02	2020-09-24 22:27:49+02	devel	glib20	\N	glib-2.56.3_9,1
2020-09-24 22:27:49+02	2020-09-24 22:28:12+02	x11	pixman	\N	pixman-0.40.0_1
2020-09-24 22:28:13+02	2020-09-24 22:28:28+02	x11	libICE	\N	libICE-1.0.10,1
2020-09-24 22:27:43+02	2020-09-24 22:28:48+02	devel	boost-jam	\N	boost-jam-1.72.0
2020-09-24 22:28:30+02	2020-09-24 22:29:05+02	devel	nasm	\N	nasm-2.15.05,1
2020-09-24 22:29:05+02	2020-09-24 22:29:19+02	x11	libSM	\N	libSM-1.2.3,1
2020-09-24 22:29:20+02	2020-09-24 22:29:33+02	x11-fonts	libfontenc	\N	libfontenc-1.1.4
2020-09-24 22:29:33+02	2020-09-24 22:29:57+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10
2020-09-24 22:29:58+02	2020-09-24 22:30:27+02	graphics	jpeg-turbo	\N	jpeg-turbo-2.0.4
2020-09-24 22:30:27+02	2020-09-24 22:30:40+02	textproc	itstool	\N	itstool-2.0.6
2020-09-24 22:30:41+02	2020-09-24 22:30:52+02	x11-fonts	mkfontscale	\N	mkfontscale-1.2.1
2020-09-24 22:30:52+02	2020-09-24 22:31:13+02	x11-toolkits	libXt	\N	libXt-1.2.0,1
2020-09-24 22:31:13+02	2020-09-24 22:31:24+02	textproc	html2text	\N	html2text-1.3.2a
2020-09-24 22:31:25+02	2020-09-24 22:31:35+02	textproc	minixmlto	\N	minixmlto-0.0.2_1
2020-09-24 22:31:36+02	2020-09-24 22:32:32+02	devel	dbus	\N	dbus-1.12.20
2020-09-24 22:32:33+02	2020-09-24 22:32:43+02	x11-fonts	bdftopcf	\N	bdftopcf-1.1_2
2020-09-24 22:32:43+02	2020-09-24 22:32:54+02	x11-fonts	font-util	\N	font-util-1.3.2
2020-09-24 22:32:57+02	2020-09-24 22:33:14+02	x11-toolkits	libXmu	\N	libXmu-1.1.3,1
2020-09-24 22:28:49+02	2020-09-24 22:33:49+02	devel	boost-libs	\N	boost-libs-1.72.0_2
2020-09-24 22:33:14+02	2020-09-24 22:33:59+02	shells	bash	\N	bash-5.0.18_3
2020-09-24 22:33:49+02	2020-09-24 22:34:03+02	textproc	highlight	\N	highlight-3.43_8,3
2020-09-24 22:34:00+02	2020-09-24 22:34:12+02	graphics	graphite2	\N	graphite2-1.3.14
2020-09-24 22:34:12+02	2020-09-24 22:34:27+02	textproc	p5-XML-Parser	\N	p5-XML-Parser-2.44
2020-09-24 22:34:04+02	2020-09-24 22:34:34+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-09-24 22:34:28+02	2020-09-24 22:34:38+02	x11-fonts	encodings	\N	encodings-1.0.5,1
2020-09-24 22:34:35+02	2020-09-24 22:34:46+02	x11-fonts	dejavu	\N	dejavu-2.37_1
2020-09-24 22:34:38+02	2020-09-24 22:34:53+02	textproc	intltool	\N	intltool-0.51.0_1
2020-09-24 22:34:47+02	2020-09-24 22:34:56+02	x11-fonts	font-bh-ttf	\N	font-bh-ttf-1.0.3_4
2020-09-24 22:34:54+02	2020-09-24 22:35:03+02	x11-fonts	font-misc-ethiopic	\N	font-misc-ethiopic-1.0.4
2020-09-24 22:34:57+02	2020-09-24 22:35:08+02	x11-fonts	font-misc-meltho	\N	font-misc-meltho-1.0.3_4
2020-09-24 22:35:04+02	2020-09-24 22:35:11+02	print	libpaper	\N	libpaper-1.1.24.4
2020-09-24 22:35:12+02	2020-09-24 22:35:24+02	graphics	jbigkit	\N	jbigkit-2.1_1
2020-09-24 22:35:08+02	2020-09-24 22:35:30+02	x11-fonts	xorg-fonts-truetype	\N	xorg-fonts-truetype-7.7_1
2020-09-24 22:35:25+02	2020-09-24 22:35:40+02	graphics	tiff	\N	tiff-4.1.0
2020-09-24 22:35:30+02	2020-09-24 22:35:43+02	x11	libXi	\N	libXi-1.7.10,1
2020-09-24 22:35:44+02	2020-09-24 22:35:54+02	emulators	tpm-emulator	\N	tpm-emulator-0.7.4_2
2020-09-24 22:35:55+02	2020-09-24 22:36:20+02	security	libtasn1	\N	libtasn1-4.16.0
2020-09-24 22:35:41+02	2020-09-24 22:36:28+02	devel	libunistring	\N	libunistring-0.9.10_1
2020-09-24 22:36:28+02	2020-09-24 22:36:51+02	dns	libidn2	\N	libidn2-2.3.0_1
2020-09-24 22:36:20+02	2020-09-24 22:37:00+02	security	nettle	\N	nettle-3.6
2020-09-24 22:36:52+02	2020-09-24 22:37:12+02	shells	bash-completion	\N	bash-completion-2.11,2
2020-09-24 22:37:00+02	2020-09-24 22:37:52+02	security	trousers	\N	trousers-0.3.14_3
2020-09-24 22:37:13+02	2020-09-24 22:37:55+02	security	p11-kit	\N	p11-kit-0.23.21
2020-09-24 22:37:53+02	2020-09-24 22:38:10+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-09-24 22:38:10+02	2020-09-24 22:38:23+02	databases	gdbm	\N	gdbm-1.18.1_1
2020-09-24 22:38:24+02	2020-09-24 22:38:50+02	databases	sqlite3	\N	sqlite3-3.33.0,1
2020-09-24 22:38:52+02	2020-09-24 22:39:04+02	devel	libdaemon	\N	libdaemon-0.14_1
2020-09-24 22:39:05+02	2020-09-24 22:39:18+02	devel	libltdl	\N	libltdl-2.4.6
2020-09-24 22:39:19+02	2020-09-24 22:39:30+02	x11	libxkbfile	\N	libxkbfile-1.1.0
2020-09-24 22:39:30+02	2020-09-24 22:39:38+02	graphics	giflib	\N	giflib-5.2.1
2020-09-24 22:37:56+02	2020-09-24 22:39:44+02	security	gnutls	\N	gnutls-3.6.15
2020-09-24 22:39:39+02	2020-09-24 22:39:48+02	x11	xkbcomp	\N	xkbcomp-1.4.3
2020-09-24 22:39:45+02	2020-09-24 22:39:56+02	x11-fonts	libXft	\N	libXft-2.3.3
2020-09-24 22:39:56+02	2020-09-24 22:40:04+02	misc	hicolor-icon-theme	\N	hicolor-icon-theme-0.17
2020-09-24 22:39:49+02	2020-09-24 22:40:10+02	x11	xkeyboard-config	\N	xkeyboard-config-2.30
2020-09-24 22:40:10+02	2020-09-24 22:40:22+02	x11	libXtst	\N	libXtst-1.2.3_2
2020-09-24 22:40:05+02	2020-09-24 22:40:23+02	graphics	webp	\N	webp-1.1.0
2020-09-24 22:40:23+02	2020-09-24 22:40:38+02	x11	libXpm	\N	libXpm-3.5.13
2020-09-24 22:40:23+02	2020-09-24 22:40:39+02	x11	libxkbcommon	\N	libxkbcommon-1.0.1
2020-09-24 22:40:39+02	2020-09-24 22:40:48+02	converters	fribidi	\N	fribidi-0.19.7
2020-09-24 22:40:39+02	2020-09-24 22:41:06+02	devel	libatomic_ops	\N	libatomic_ops-7.6.10
2020-09-24 22:40:48+02	2020-09-24 22:41:06+02	x11-toolkits	libXaw	\N	libXaw-1.0.13_3,2
2020-09-24 22:41:07+02	2020-09-24 22:41:13+02	misc	getopt	\N	getopt-1.1.6
2020-09-24 22:41:06+02	2020-09-24 22:41:16+02	devel	boehm-gc	\N	boehm-gc-8.0.4_1
2020-09-24 22:41:17+02	2020-09-24 22:41:34+02	www	w3m	\N	w3m-0.5.3.20200507
2020-09-24 22:41:13+02	2020-09-24 22:41:47+02	dns	libidn	\N	libidn-1.35
2020-09-24 22:41:34+02	2020-09-24 22:41:53+02	textproc	xmlto	\N	xmlto-0.0.28
2020-09-24 22:41:48+02	2020-09-24 22:42:02+02	graphics	poppler-data	\N	poppler-data-0.4.9_5
2020-09-24 22:41:53+02	2020-09-24 22:42:05+02	print	psutils	\N	psutils-1.17_5
2020-09-24 22:42:03+02	2020-09-24 22:42:15+02	graphics	lcms2	\N	lcms2-2.11_1
2020-09-24 22:42:16+02	2020-09-24 22:42:25+02	devel	nspr	\N	nspr-4.29
2020-09-24 22:42:06+02	2020-09-24 22:42:31+02	graphics	jbig2dec	\N	jbig2dec-0.19
2020-09-24 22:42:26+02	2020-09-24 22:42:39+02	graphics	gd	\N	libgd-2.3.0,1
2020-09-24 22:42:40+02	2020-09-24 22:43:14+02	devel	pcre2	\N	pcre2-10.35
2020-09-24 22:42:31+02	2020-09-24 22:43:21+02	misc	shared-mime-info	\N	shared-mime-info-2.0
2020-09-24 22:43:21+02	2020-09-24 22:43:32+02	textproc	uchardet	\N	uchardet-0.0.7
2020-09-24 22:43:15+02	2020-09-24 22:43:42+02	lang	tcl86	\N	tcl86-8.6.10
2020-09-24 22:43:42+02	2020-09-24 22:43:53+02	x11	libXinerama	\N	libXinerama-1.1.4_2,1
2020-09-24 22:43:53+02	2020-09-24 22:44:08+02	x11	libXcursor	\N	libXcursor-1.2.0
2020-09-24 22:44:08+02	2020-09-24 22:44:17+02	devel	ctags	\N	ctags-5.8
2020-09-24 22:44:17+02	2020-09-24 22:44:29+02	security	libssh2	\N	libssh2-1.9.0_3,3
2020-09-24 22:43:32+02	2020-09-24 22:44:40+02	textproc	groff	\N	groff-1.22.4_3
2020-09-24 22:44:40+02	2020-09-24 22:44:51+02	x11	libXcomposite	\N	libXcomposite-0.4.5,1
2020-09-24 22:44:29+02	2020-09-24 22:44:57+02	textproc	source-highlight	\N	source-highlight-3.1.9_1
2020-09-24 22:44:52+02	2020-09-24 22:45:09+02	devel	libgit2	\N	libgit2-1.0.1
2020-09-24 22:25:48+02	2020-09-24 22:45:23+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-09-24 22:45:24+02	2020-09-24 22:46:52+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_3
2020-09-24 22:44:58+02	2020-09-24 22:47:29+02	devel	gdb	py37	gdb-9.2_1
2020-09-24 22:46:53+02	2020-09-24 22:47:51+02	graphics	cairo	\N	cairo-1.16.0,3
2020-09-24 22:47:51+02	2020-09-24 22:49:17+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-09-24 22:49:17+02	2020-09-24 22:51:45+02	print	harfbuzz	\N	harfbuzz-2.7.2
2020-09-24 22:51:46+02	2020-09-24 22:52:52+02	net	avahi-app	\N	avahi-app-0.7_3
2020-09-24 22:52:53+02	2020-09-24 22:53:51+02	print	cups	\N	cups-2.3.3_1
2020-09-24 22:53:52+02	2020-09-24 22:56:52+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-09-24 22:56:53+02	2020-09-24 22:57:19+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-09-24 22:57:20+02	2020-09-24 22:58:39+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-09-24 22:58:39+02	2020-09-24 22:59:35+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-09-24 22:59:35+02	2020-09-24 23:02:48+02	graphics	graphviz	\N	graphviz-2.44.1_2
2020-09-24 23:02:49+02	2020-09-24 23:03:15+02	accessibility	atk	\N	atk-2.36.0
2020-09-24 23:03:15+02	2020-09-24 23:03:41+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-09-24 23:03:42+02	2020-09-24 23:03:55+02	devel	libudev-devd	\N	libudev-devd-0.4.2
2020-09-24 23:03:55+02	2020-09-24 23:05:27+02	lang	vala	\N	vala-0.40.15,1
2020-09-24 23:05:27+02	2020-09-24 23:05:51+02	devel	autoconf213	\N	autoconf213-2.13.000227_7
2020-09-24 23:05:52+02	2020-09-24 23:06:11+02	x11	libXScrnSaver	\N	libXScrnSaver-1.2.3_2
2020-09-24 23:06:11+02	2020-09-24 23:06:24+02	x11	xcb-util	\N	xcb-util-0.4.0_2,1
2020-09-24 23:06:25+02	2020-09-24 23:07:08+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-09-24 23:07:08+02	2020-09-24 23:07:28+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-09-24 23:07:29+02	2020-09-24 23:07:56+02	textproc	libcroco	\N	libcroco-0.6.13
2020-09-24 23:07:57+02	2020-09-24 23:08:48+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-09-24 23:08:49+02	2020-09-24 23:09:08+02	archivers	zstd	\N	zstd-1.4.5_1
2020-09-24 23:09:09+02	2020-09-24 23:09:18+02	misc	qtchooser	\N	qtchooser-66_4
2020-09-24 23:09:18+02	2020-09-24 23:09:33+02	devel	double-conversion	\N	double-conversion-3.1.5.19
2020-09-24 23:09:34+02	2020-09-24 23:10:33+02	devel	qt5-qmake	\N	qt5-qmake-5.15.0
2020-09-24 23:10:34+02	2020-09-24 23:11:25+02	devel	qt5-buildtools	\N	qt5-buildtools-5.15.0
2020-09-24 23:11:26+02	2020-09-24 23:11:40+02	x11	xprop	\N	xprop-1.2.4
2020-09-24 23:11:41+02	2020-09-24 23:11:55+02	x11	xset	\N	xset-1.2.4_3
2020-09-24 22:45:09+02	2020-09-24 23:13:20+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-09-24 23:11:56+02	2020-09-24 23:13:24+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-09-24 23:13:25+02	2020-09-24 23:13:41+02	x11	xcb-util-wm	\N	xcb-util-wm-0.4.1_3
2020-09-24 23:13:21+02	2020-09-24 23:13:45+02	graphics	openjpeg	\N	openjpeg-2.3.1
2020-09-24 23:13:44+02	2020-09-24 23:13:58+02	devel	jam	\N	jam-2.6
2020-09-24 23:13:46+02	2020-09-24 23:14:06+02	devel	libevdev	\N	libevdev-1.5.9_1
2020-09-24 23:14:06+02	2020-09-24 23:14:21+02	devel	libmtdev	\N	libmtdev-1.1.5_3
2020-09-24 23:13:58+02	2020-09-24 23:14:22+02	devel	libgudev	\N	libgudev-233
2020-09-24 23:14:23+02	2020-09-24 23:14:35+02	devel	py-evdev	py37	py37-evdev-0.8.1_1
2020-09-24 23:14:22+02	2020-09-24 23:14:40+02	devel	xdg-utils	\N	xdg-utils-1.1.3_1
2020-09-24 23:14:36+02	2020-09-24 23:14:56+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-09-24 23:14:56+02	2020-09-24 23:15:18+02	x11	libwacom	\N	libwacom-1.4.1
2020-09-24 23:15:19+02	2020-09-24 23:15:42+02	x11	libinput	\N	libinput-1.15.6
2020-09-24 23:14:42+02	2020-09-24 23:15:55+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-09-24 23:15:42+02	2020-09-24 23:15:58+02	devel	vulkan-headers	\N	vulkan-headers-1.2.135.0
2020-09-24 23:15:56+02	2020-09-24 23:16:14+02	x11	xcb-util-image	\N	xcb-util-image-0.4.0_1
2020-09-24 23:15:59+02	2020-09-24 23:16:17+02	x11	xcb-util-keysyms	\N	xcb-util-keysyms-0.4.0_1
2020-09-24 23:16:15+02	2020-09-24 23:16:31+02	x11	xcb-util-renderutil	\N	xcb-util-renderutil-0.3.9_1
2020-09-24 23:16:18+02	2020-09-24 23:16:53+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-09-24 23:16:54+02	2020-09-24 23:17:48+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-09-24 23:16:31+02	2020-09-24 23:18:31+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-09-24 23:17:48+02	2020-09-24 23:19:06+02	net	qt5-network	\N	qt5-network-5.15.0
2020-09-24 23:18:31+02	2020-09-24 23:19:25+02	textproc	gsed	\N	gsed-4.8
2020-09-24 23:19:25+02	2020-09-24 23:19:41+02	audio	libogg	\N	libogg-1.3.4,4
2020-09-24 23:19:41+02	2020-09-24 23:20:40+02	misc	iso-codes	\N	iso-codes-4.2
2020-09-24 23:20:42+02	2020-09-24 23:21:02+02	audio	libvorbis	\N	libvorbis-1.3.7_2,3
2020-09-24 23:21:05+02	2020-09-24 23:21:42+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.2
2020-09-24 23:21:42+02	2020-09-24 23:22:06+02	graphics	libGLU	\N	libGLU-9.0.1
2020-09-24 23:19:07+02	2020-09-24 23:22:13+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-09-24 23:22:06+02	2020-09-24 23:22:51+02	devel	orc	\N	orc-0.4.31
2020-09-24 23:22:51+02	2020-09-24 23:23:54+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-09-24 23:22:13+02	2020-09-24 23:24:51+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-09-24 23:23:54+02	2020-09-24 23:25:04+02	security	nss	\N	nss-3.56
2020-09-24 23:25:05+02	2020-09-24 23:25:23+02	devel	desktop-file-utils	\N	desktop-file-utils-0.26
2020-09-24 23:24:51+02	2020-09-24 23:25:48+02	audio	flac	\N	flac-1.3.3
2020-09-24 23:25:24+02	2020-09-24 23:26:47+02	math	fftw3	\N	fftw3-3.3.8_6
2020-09-24 23:26:47+02	2020-09-24 23:27:48+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-09-24 23:27:49+02	2020-09-24 23:28:02+02	archivers	zip	\N	zip-3.0_1
2020-09-24 23:28:02+02	2020-09-24 23:29:34+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-09-24 23:25:48+02	2020-09-24 23:29:49+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-24 23:29:50+02	2020-09-24 23:30:15+02	audio	opus	\N	opus-1.3.1
2020-09-24 23:30:15+02	2020-09-24 23:32:27+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-09-24 23:32:28+02	2020-09-24 23:32:42+02	textproc	libyaml	\N	libyaml-0.2.5
2020-09-24 23:32:43+02	2020-09-24 23:33:20+02	audio	libsndfile	\N	libsndfile-1.0.29
2020-09-24 23:29:35+02	2020-09-24 23:33:43+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-09-24 23:33:20+02	2020-09-24 23:33:55+02	graphics	poppler	\N	poppler-20.09.0
2020-09-24 23:33:46+02	2020-09-24 23:34:13+02	devel	tex-kpathsea	\N	tex-kpathsea-6.2.1_2
2020-09-24 23:33:56+02	2020-09-24 23:34:26+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-09-24 23:34:27+02	2020-09-24 23:34:43+02	net	libproxy	\N	libproxy-0.4.15
2020-09-24 23:34:43+02	2020-09-24 23:35:14+02	net	glib-networking	\N	glib-networking-2.56.1_3
2020-09-24 23:35:15+02	2020-09-24 23:35:55+02	devel	dconf	\N	dconf-0.38.0
2020-09-24 23:35:56+02	2020-09-24 23:36:49+02	devel	libsoup	\N	libsoup-2.62.3
2020-09-24 23:36:49+02	2020-09-24 23:37:04+02	devel	zziplib	\N	zziplib-0.13.71_1
2020-09-24 23:37:05+02	2020-09-24 23:37:23+02	graphics	openjpeg15	\N	openjpeg15-1.5.2_1
2020-09-24 23:37:24+02	2020-09-24 23:37:42+02	print	tex-ptexenc	\N	tex-ptexenc-1.3.3_2
2020-09-24 23:37:42+02	2020-09-24 23:38:07+02	devel	json-glib	\N	json-glib-1.6.0
2020-09-24 23:38:07+02	2020-09-24 23:39:06+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-09-24 23:39:06+02	2020-09-24 23:39:27+02	math	mpc	\N	mpc-1.1.0_2
2020-09-24 23:39:27+02	2020-09-24 23:40:13+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-09-24 23:40:13+02	2020-09-24 23:40:34+02	devel	t1lib	\N	t1lib-5.1.2_5,1
2020-09-24 23:40:35+02	2020-09-24 23:40:46+02	graphics	libpotrace	\N	libpotrace-1.16
2020-09-24 23:34:14+02	2020-09-24 23:40:47+02	lang	ruby27	\N	ruby-2.7.1_1,1
2020-09-24 23:40:48+02	2020-09-24 23:41:04+02	print	xpdfopen	\N	xpdfopen-0.86
2020-09-24 23:40:47+02	2020-09-24 23:41:20+02	print	texlive-tlmgr	\N	texlive-tlmgr-20150523_2
2020-09-24 23:41:05+02	2020-09-24 23:41:44+02	textproc	teckit	\N	teckit-2.5.7
2020-09-24 23:41:20+02	2020-09-24 23:42:12+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-09-24 23:42:12+02	2020-09-24 23:42:46+02	archivers	brotli	\N	brotli-1.0.9,1
2020-09-24 23:42:47+02	2020-09-24 23:43:41+02	devel	protobuf	\N	protobuf-3.13.0,1
2020-09-24 23:43:41+02	2020-09-24 23:45:16+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-09-24 23:41:44+02	2020-09-24 23:45:44+02	print	texlive-base	\N	texlive-base-20150521_55
2020-09-24 23:45:16+02	2020-09-24 23:45:57+02	textproc	flex	\N	flex-2.6.4_2
2020-09-24 23:45:58+02	2020-09-24 23:46:29+02	devel	libnotify	\N	libnotify-0.7.9
2020-09-24 23:46:30+02	2020-09-24 23:47:05+02	textproc	hunspell	\N	hunspell-1.7.0_2
2020-09-24 23:47:06+02	2020-09-24 23:47:56+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-09-24 23:47:57+02	2020-09-24 23:48:20+02	devel	libevent	\N	libevent-2.1.12
2020-09-24 23:48:20+02	2020-09-24 23:49:55+02	lang	python27	\N	python27-2.7.18_1
2020-09-24 23:49:56+02	2020-09-24 23:50:15+02	multimedia	libva	\N	libva-2.9.0
2020-09-24 23:50:15+02	2020-09-24 23:50:28+02	x11	libXv	\N	libXv-1.0.11_2,1
2020-09-24 23:50:29+02	2020-09-24 23:51:42+02	databases	db6	\N	db6-6.2.32_1
2020-09-24 23:51:43+02	2020-09-24 23:52:12+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-09-24 23:52:12+02	2020-09-24 23:52:26+02	devel	woff2	\N	woff2-1.0.2_4
2020-09-24 23:52:27+02	2020-09-24 23:52:40+02	security	libargon2	\N	libargon2-20190702
2020-09-24 23:52:41+02	2020-09-24 23:53:38+02	security	libsecret	\N	libsecret-0.18.6_1
2020-09-24 23:53:40+02	2020-09-24 23:54:03+02	textproc	hyphen	\N	hyphen-2.8.8
2020-09-24 23:54:03+02	2020-09-24 23:54:13+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-09-24 23:54:14+02	2020-09-24 23:54:42+02	devel	argp-standalone	\N	argp-standalone-1.3_4
2020-09-24 23:54:42+02	2020-09-24 23:55:36+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-09-24 23:55:37+02	2020-09-24 23:55:44+02	audio	ladspa	\N	ladspa-1.15
2020-09-24 23:55:45+02	2020-09-24 23:57:45+02	lang	php74	\N	php74-7.4.10
2020-09-24 23:57:45+02	2020-09-24 23:58:32+02	multimedia	libv4l	\N	libv4l-1.18.0
2020-09-24 23:58:32+02	2020-09-24 23:58:44+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-09-24 23:58:45+02	2020-09-24 23:59:00+02	audio	libsamplerate	\N	libsamplerate-0.1.9
2020-09-24 23:59:00+02	2020-09-24 23:59:26+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-09-24 23:59:26+02	2020-09-24 23:59:37+02	devel	opencl	\N	opencl-2.2_2
2020-09-24 23:59:37+02	2020-09-24 23:59:49+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-09-24 23:59:49+02	2020-09-25 00:02:02+02	lang	fpc	\N	fpc-3.0.4_4
2020-09-25 00:02:02+02	2020-09-25 00:02:30+02	math	eigen3	\N	eigen-3.3.7
2020-09-25 00:02:30+02	2020-09-25 00:03:09+02	textproc	texi2html	\N	texi2html-5.0_2,1
2020-09-25 00:03:09+02	2020-09-25 00:03:47+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-09-25 00:03:48+02	2020-09-25 00:04:08+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-09-24 23:45:45+02	2020-09-25 00:08:04+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-25 00:04:08+02	2020-09-25 00:11:15+02	multimedia	x265	\N	x265-3.2.1_4
2020-09-25 00:08:05+02	2020-09-25 00:11:24+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-25 00:11:25+02	2020-09-25 00:11:45+02	textproc	py-rdflib	py37	py37-rdflib-5.0.0
2020-09-25 00:11:46+02	2020-09-25 00:12:00+02	audio	lv2	\N	lv2-1.18.0
2020-09-25 00:12:01+02	2020-09-25 00:12:15+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0
2020-09-25 00:12:16+02	2020-09-25 00:12:34+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-09-25 00:12:35+02	2020-09-25 00:13:24+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-09-25 00:13:27+02	2020-09-25 00:13:44+02	multimedia	libaacs	\N	libaacs-0.9.0
2020-09-25 00:13:45+02	2020-09-25 00:14:23+02	multimedia	libbdplus	\N	libbdplus-0.1.2
2020-09-25 00:14:23+02	2020-09-25 00:14:39+02	multimedia	libtheora	\N	libtheora-1.1.1_7
2020-09-25 00:14:39+02	2020-09-25 00:14:57+02	multimedia	libvdpau	\N	libvdpau-1.4
2020-09-25 00:14:58+02	2020-09-25 00:16:29+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-25 00:16:30+02	2020-09-25 00:16:52+02	audio	rubberband	\N	rubberband-1.8.2_1
2020-09-25 00:16:53+02	2020-09-25 00:17:19+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-25 00:17:19+02	2020-09-25 00:17:40+02	graphics	frei0r	\N	frei0r-1.6.1
2020-09-25 00:11:15+02	2020-09-25 00:18:56+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-25 00:17:43+02	2020-09-25 00:19:05+02	graphics	opencv-core	\N	opencv-core-3.4.1_36
2020-09-25 00:19:06+02	2020-09-25 00:21:58+02	multimedia	aom	\N	aom-2.0.0_1
2020-09-25 00:21:59+02	2020-09-25 00:23:51+02	multimedia	dav1d	\N	dav1d-0.7.1
2020-09-25 00:23:52+02	2020-09-25 00:24:15+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-09-25 00:24:16+02	2020-09-25 00:26:37+02	multimedia	libvpx	\N	libvpx-1.9.0
2020-09-25 00:26:37+02	2020-09-25 00:28:31+02	multimedia	libx264	\N	libx264-0.160.3011
2020-09-25 00:28:32+02	2020-09-25 00:28:44+02	multimedia	xvid	\N	xvid-1.3.7,1
2020-09-25 00:28:45+02	2020-09-25 00:39:27+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_2,1
2020-09-25 00:39:28+02	2020-09-25 00:40:38+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-09-25 00:40:39+02	2020-09-25 00:41:32+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-09-25 00:41:33+02	2020-09-25 00:43:41+02	graphics	graphene	\N	graphene-1.10.2
2020-09-25 00:43:42+02	2020-09-25 00:44:02+02	www	libwpe	\N	libwpe-1.4.0.1
2020-09-25 00:44:02+02	2020-09-25 00:45:06+02	databases	postgresql12-client	\N	postgresql12-client-12.4
2020-09-25 00:45:06+02	2020-09-25 00:45:22+02	devel	libinotify	\N	libinotify-20180201_2
2020-09-25 00:45:23+02	2020-09-25 00:46:39+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-25 00:46:40+02	2020-09-25 00:48:04+02	net	geoclue	\N	geoclue-2.5.5
2020-09-25 00:48:05+02	2020-09-25 00:48:28+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-09-25 00:48:29+02	2020-09-25 00:48:46+02	graphics	jasper	\N	jasper-2.0.20
2020-09-25 00:48:50+02	2020-09-25 00:49:17+02	audio	alsa-lib	\N	alsa-lib-1.1.2_2
2020-09-25 00:49:17+02	2020-09-25 00:50:46+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.74.0
2020-09-25 00:50:46+02	2020-09-25 00:51:41+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-09-25 00:51:42+02	2020-09-25 00:56:51+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-25 00:56:52+02	2020-09-25 00:57:13+02	devel	py-setuptools	py27	py27-setuptools-44.0.0
2020-09-25 00:57:13+02	2020-09-25 00:58:04+02	security	cyrus-sasl2	\N	cyrus-sasl-2.1.27_1
2020-09-25 00:58:05+02	2020-09-25 00:58:47+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-09-25 00:58:48+02	2020-09-25 00:59:21+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-09-25 00:59:22+02	2020-09-25 00:59:36+02	archivers	libmspack	\N	libmspack-0.10.1
2020-09-25 00:59:37+02	2020-09-25 01:00:05+02	graphics	libraw	\N	libraw-0.19.5
2020-09-25 01:00:06+02	2020-09-25 01:09:57+02	devel	tbb	\N	tbb-2020.3
2020-09-25 01:09:59+02	2020-09-25 01:10:36+02	graphics	aalib	\N	aalib-1.4.r5_13
2020-09-25 01:10:37+02	2020-09-25 01:11:56+02	net	openldap24-client	\N	openldap-client-2.4.51
2020-09-25 01:11:56+02	2020-09-25 01:12:47+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-25 01:12:48+02	2020-09-25 01:13:11+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-09-25 01:13:12+02	2020-09-25 01:14:49+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-09-25 01:14:50+02	2020-09-25 01:15:09+02	print	libraqm	\N	libraqm-0.6.0
2020-09-25 01:15:10+02	2020-09-25 01:15:33+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-09-25 01:15:33+02	2020-09-25 01:16:04+02	devel	json-c	\N	json-c-0.15
2020-09-25 01:16:04+02	2020-09-25 01:16:16+02	science	szip	\N	hdf-szip-2.1.1
2020-09-25 01:16:17+02	2020-09-25 01:16:33+02	textproc	py-toml	py37	py37-toml-0.10.1
2020-09-25 01:16:33+02	2020-09-25 01:16:50+02	x11	startup-notification	\N	startup-notification-0.12_4
2020-09-25 01:16:51+02	2020-09-25 01:17:23+02	x11-toolkits	tk86	\N	tk86-8.6.10_1
2020-09-25 01:17:24+02	2020-09-25 01:17:38+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3_1
2020-09-25 01:17:39+02	2020-09-25 01:18:21+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-09-25 01:18:22+02	2020-09-25 01:18:43+02	net	p5-URI	\N	p5-URI-1.76
2020-09-25 01:18:44+02	2020-09-25 01:21:22+02	science	hdf5	\N	hdf5-1.10.6,1
2020-09-25 01:21:22+02	2020-09-25 01:21:40+02	x11	libXres	\N	libXres-1.2.0_2
2020-09-25 01:21:41+02	2020-09-25 01:21:58+02	x11	libXvMC	\N	libXvMC-1.0.12
2020-09-25 01:21:59+02	2020-09-25 01:22:16+02	audio	speexdsp	\N	speexdsp-1.2.0
2020-09-25 01:22:17+02	2020-09-25 01:26:39+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-25 01:26:40+02	2020-09-25 01:27:37+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-09-25 01:27:38+02	2020-09-25 01:28:09+02	textproc	librevenge	\N	librevenge-0.0.4_13
2020-09-25 01:28:09+02	2020-09-25 01:28:55+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-09-25 01:28:56+02	2020-09-25 01:29:13+02	graphics	libexif	\N	libexif-0.6.21_5
2020-09-25 01:29:14+02	2020-09-25 01:32:23+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-09-25 01:32:24+02	2020-09-25 01:32:39+02	x11	libXxf86dga	\N	libXxf86dga-1.1.5
2020-09-25 01:32:42+02	2020-09-25 01:33:08+02	x11-fonts	libXfont2	\N	libXfont2-2.0.4
2020-09-25 01:33:08+02	2020-09-25 01:34:16+02	audio	portaudio	\N	portaudio-19.6.0_5,1
2020-09-25 01:34:17+02	2020-09-25 01:34:37+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.9_7
2020-09-25 01:34:37+02	2020-09-25 01:35:23+02	devel	oniguruma	\N	oniguruma-6.9.5.r1_1
2020-09-25 01:35:24+02	2020-09-25 01:35:35+02	devel	re2c	\N	re2c-0.14.3
2020-09-25 01:35:36+02	2020-09-25 01:35:59+02	graphics	lcms	\N	lcms-1.19_6,1
2020-09-25 01:35:59+02	2020-09-25 01:36:19+02	lang	fpc-rtl-objpas	\N	fpc-rtl-objpas-3.0.4_5
2020-09-25 01:36:19+02	2020-09-25 01:36:44+02	net	p5-Socket6	\N	p5-Socket6-0.29
2020-09-25 01:36:45+02	2020-09-25 01:37:20+02	www	php74-session	\N	php74-session-7.4.10
2020-09-25 01:37:21+02	2020-09-25 01:37:54+02	archivers	php74-zlib	\N	php74-zlib-7.4.10
2020-09-25 01:37:55+02	2020-09-25 01:38:07+02	audio	espeak	\N	espeak-1.48.04_7
2020-09-25 01:38:08+02	2020-09-25 01:38:39+02	databases	php74-pdo	\N	php74-pdo-7.4.10
2020-09-25 01:38:39+02	2020-09-25 01:38:56+02	devel	dotconf	\N	dotconf-1.3_1
2020-09-25 01:38:56+02	2020-09-25 01:39:16+02	devel	fpc-fcl-res	\N	fpc-fcl-res-3.0.4_4
2020-09-25 01:39:17+02	2020-09-25 01:39:49+02	devel	php74-json	\N	php74-json-7.4.10
2020-09-25 01:39:50+02	2020-09-25 01:40:12+02	graphics	libmng	\N	libmng-1.0.10_3
2020-09-25 01:40:13+02	2020-09-25 01:41:51+02	math	fftw3-float	\N	fftw3-float-3.3.8_6
2020-09-25 01:41:52+02	2020-09-25 01:42:15+02	net	p5-IO-Socket-INET6	\N	p5-IO-Socket-INET6-2.72_1
2020-09-25 01:42:16+02	2020-09-25 01:42:30+02	security	libassuan	\N	libassuan-2.5.3
2020-09-25 01:42:31+02	2020-09-25 01:43:31+02	security	libsodium	\N	libsodium-1.0.18
2020-09-25 01:43:32+02	2020-09-25 01:43:50+02	security	p5-Digest-HMAC	\N	p5-Digest-HMAC-1.03_1
2020-09-25 01:43:50+02	2020-09-25 01:44:18+02	security	p5-Net-SSLeay	\N	p5-Net-SSLeay-1.88
2020-09-25 01:44:18+02	2020-09-25 01:44:26+02	sysutils	dmidecode	\N	dmidecode-3.2
2020-09-25 01:44:27+02	2020-09-25 01:45:27+02	sysutils	policykit	\N	policykit-0.9_10
2020-09-25 01:45:27+02	2020-09-25 01:45:52+02	www	p5-HTML-Tagset	\N	p5-HTML-Tagset-3.20_1
2020-09-25 01:45:53+02	2020-09-25 01:46:15+02	www	p5-Mozilla-CA	\N	p5-Mozilla-CA-20200520
2020-09-25 01:46:15+02	2020-09-25 01:47:00+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-09-25 01:47:01+02	2020-09-25 01:47:24+02	archivers	snappy	\N	snappy-1.1.8
2020-09-25 01:47:24+02	2020-09-25 01:48:05+02	converters	php74-mbstring	\N	php74-mbstring-7.4.10
2020-09-25 01:48:06+02	2020-09-25 01:50:53+02	databases	libmemcached	\N	libmemcached-1.0.18_7
2020-09-25 01:50:55+02	2020-09-25 01:51:16+02	devel	fpc-fcl-base	\N	fpc-fcl-base-3.0.4_4
2020-09-25 01:51:17+02	2020-09-25 01:52:05+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.74.0
2020-09-25 01:52:06+02	2020-09-25 01:52:15+02	devel	libvolume_id	\N	libvolume_id-0.81.1
2020-09-25 01:52:16+02	2020-09-25 01:52:35+02	devel	py-sip	py37	py37-sip-4.19.21,1
2020-09-25 01:52:35+02	2020-09-25 01:53:56+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-25 01:53:56+02	2020-09-25 01:54:31+02	graphics	php74-gd	\N	php74-gd-7.4.10
2020-09-25 01:54:32+02	2020-09-25 01:54:56+02	math	metis	\N	metis-5.1.0_8
2020-09-25 01:54:57+02	2020-09-25 01:55:11+02	multimedia	libdvdcss	\N	libdvdcss-1.4.2
2020-09-25 01:55:11+02	2020-09-25 01:55:33+02	security	libmcrypt	\N	libmcrypt-2.5.8_3
2020-09-25 01:55:34+02	2020-09-25 01:55:52+02	security	p5-IO-Socket-SSL	\N	p5-IO-Socket-SSL-2.068
2020-09-25 01:55:53+02	2020-09-25 01:56:18+02	security	php74-openssl	\N	php74-openssl-7.4.10
2020-09-25 01:56:19+02	2020-09-25 01:56:42+02	textproc	php74-xml	\N	php74-xml-7.4.10
2020-09-25 01:56:43+02	2020-09-25 01:57:01+02	www	p5-HTML-Parser	\N	p5-HTML-Parser-3.72
2020-09-25 01:57:01+02	2020-09-25 01:57:14+02	audio	alsa-plugins	\N	alsa-plugins-1.1.1_6
2020-09-25 01:57:14+02	2020-09-25 01:57:56+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-25 01:57:56+02	2020-09-25 01:58:23+02	converters	php74-iconv	\N	php74-iconv-7.4.10
2020-09-25 01:58:24+02	2020-09-25 01:58:49+02	databases	pecl-memcache	php74	php74-pecl-memcache-4.0.5.2
2020-09-25 01:58:50+02	2020-09-25 01:59:19+02	databases	pecl-memcached	php74	php74-pecl-memcached-3.1.5
2020-09-25 01:59:19+02	2020-09-25 01:59:46+02	databases	php74-pdo_pgsql	\N	php74-pdo_pgsql-7.4.10
2020-09-25 01:59:46+02	2020-09-25 02:00:22+02	databases	php74-pdo_sqlite	\N	php74-pdo_sqlite-7.4.10
2020-09-25 02:00:23+02	2020-09-25 02:00:54+02	databases	php74-pgsql	\N	php74-pgsql-7.4.10
2020-09-25 02:00:55+02	2020-09-25 02:01:30+02	databases	php74-sqlite3	\N	php74-sqlite3-7.4.10
2020-09-25 02:01:33+02	2020-09-25 02:01:44+02	devel	libpci	\N	libpci-3.7.0
2020-09-25 02:01:44+02	2020-09-25 02:02:17+02	devel	php74-gettext	\N	php74-gettext-7.4.10
2020-09-25 02:02:18+02	2020-09-25 02:04:20+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-25 02:04:21+02	2020-09-25 02:05:13+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-09-25 02:05:14+02	2020-09-25 02:05:42+02	ftp	php74-curl	\N	php74-curl-7.4.10
2020-09-25 02:05:43+02	2020-09-25 02:06:39+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-09-25 02:06:40+02	2020-09-25 02:07:00+02	math	pecl-bitset	php74	php74-pecl-bitset-3.0.1
2020-09-25 02:07:01+02	2020-09-25 02:07:32+02	math	php74-bcmath	\N	php74-bcmath-7.4.10
2020-09-25 02:07:32+02	2020-09-25 02:07:44+02	multimedia	libdvdread	\N	libdvdread-6.1.0
2020-09-25 02:07:45+02	2020-09-25 02:11:14+02	net	php74-ldap	\N	php74-ldap-7.4.10
2020-09-25 02:11:15+02	2020-09-25 02:11:40+02	net	php74-soap	\N	php74-soap-7.4.10
2020-09-25 02:11:41+02	2020-09-25 02:12:10+02	net	php74-sockets	\N	php74-sockets-7.4.10
2020-09-25 02:12:10+02	2020-09-25 02:12:32+02	security	pecl-mcrypt	php74	php74-pecl-mcrypt-1.0.3
2020-09-25 02:12:32+02	2020-09-25 02:12:57+02	sysutils	php74-posix	\N	php74-posix-7.4.10
2020-09-25 02:12:58+02	2020-09-25 02:13:21+02	textproc	php74-ctype	\N	php74-ctype-7.4.10
2020-09-25 02:13:22+02	2020-09-25 02:13:54+02	textproc	php74-dom	\N	php74-dom-7.4.10
2020-09-25 02:13:54+02	2020-09-25 02:14:21+02	textproc	php74-simplexml	\N	php74-simplexml-7.4.10
2020-09-25 02:14:21+02	2020-09-25 02:14:52+02	www	php74-opcache	\N	php74-opcache-7.4.10
2020-09-25 02:14:52+02	2020-09-25 02:15:21+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-09-25 02:15:22+02	2020-09-25 02:15:34+02	databases	py-sqlparse	py37	py37-sqlparse-0.3.1
2020-09-25 02:15:35+02	2020-09-25 02:16:30+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-09-24 22:47:30+02	2020-09-25 02:16:53+02	lang	rust	\N	rust-1.46.0
2020-09-25 02:16:31+02	2020-09-25 02:17:47+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-09-25 02:17:47+02	2020-09-25 02:18:10+02	security	p5-GSSAPI	\N	p5-GSSAPI-0.28_1
2020-09-25 02:18:11+02	2020-09-25 02:18:35+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-09-25 02:18:36+02	2020-09-25 02:18:51+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-09-25 02:18:51+02	2020-09-25 02:19:37+02	archivers	kf5-karchive	\N	kf5-karchive-5.74.0
2020-09-25 02:19:37+02	2020-09-25 02:19:45+02	devel	cvsps	\N	cvsps-2.1_2
2020-09-25 02:19:46+02	2020-09-25 02:21:04+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-09-25 02:21:05+02	2020-09-25 02:21:40+02	devel	libsigc++20	\N	libsigc++-2.10.3
2020-09-25 02:21:40+02	2020-09-25 02:22:13+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-09-25 02:22:14+02	2020-09-25 02:22:30+02	graphics	ilmbase	\N	ilmbase-2.5.3
2020-09-25 02:22:31+02	2020-09-25 02:22:39+02	java	javavmwrapper	\N	javavmwrapper-2.7.6
2020-09-25 02:22:39+02	2020-09-25 02:23:00+02	lang	p5-Error	\N	p5-Error-0.17029
2020-09-25 02:23:02+02	2020-09-25 02:23:44+02	misc	e2fsprogs-libuuid	\N	e2fsprogs-libuuid-1.45.6
2020-09-25 02:23:45+02	2020-09-25 02:23:55+02	multimedia	libdvdnav	\N	libdvdnav-6.1.0
2020-09-25 02:16:55+02	2020-09-25 02:23:55+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.5
2020-09-25 02:23:56+02	2020-09-25 02:24:14+02	security	p5-Authen-SASL	\N	p5-Authen-SASL-2.16_1
2020-09-25 02:24:15+02	2020-09-25 02:24:32+02	www	p5-CGI	\N	p5-CGI-4.50
2020-09-25 02:24:33+02	2020-09-25 02:25:15+02	www	py-django22	py37	py37-django22-2.2.16
2020-09-25 02:25:16+02	2020-09-25 02:26:08+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-09-25 02:26:08+02	2020-09-25 02:28:42+02	x11-servers	xorg-server	\N	xorg-server-1.20.9,1
2020-09-25 02:28:43+02	2020-09-25 02:30:28+02	devel	git	\N	git-2.28.0
2020-09-25 02:30:28+02	2020-09-25 02:32:02+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-09-25 02:32:03+02	2020-09-25 02:32:12+02	devel	libsysinfo	\N	libsysinfo-0.0.3_2
2020-09-25 02:32:13+02	2020-09-25 02:34:27+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-09-25 02:34:28+02	2020-09-25 02:34:41+02	devel	re2	\N	re2-20200401
2020-09-25 02:34:42+02	2020-09-25 02:35:43+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-09-25 02:35:44+02	2020-09-25 02:36:03+02	devel	yasm	\N	yasm-1.3.0
2020-09-25 02:36:03+02	2020-09-25 02:37:08+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-09-25 02:37:09+02	2020-09-25 02:37:32+02	lang	fpc-rtl-extra	\N	fpc-rtl-extra-3.0.4_4
2020-09-25 02:37:33+02	2020-09-25 02:37:53+02	textproc	p5-XML-NamespaceSupport	\N	p5-XML-NamespaceSupport-1.12
2020-09-25 02:37:54+02	2020-09-25 02:38:17+02	textproc	p5-XML-SAX-Base	\N	p5-XML-SAX-Base-1.09
2020-09-25 02:38:18+02	2020-09-25 02:39:24+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-09-25 02:39:24+02	2020-09-25 02:39:39+02	x11	xbitmaps	\N	xbitmaps-1.1.2
2020-09-25 02:23:56+02	2020-09-25 02:40:11+02	lang	spidermonkey78	\N	spidermonkey78-78.2.0
2020-09-25 02:39:40+02	2020-09-25 02:40:27+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-25 02:40:28+02	2020-09-25 02:40:39+02	audio	liba52	\N	liba52-0.7.4_3
2020-09-25 02:40:40+02	2020-09-25 02:40:59+02	audio	libsoxr	\N	libsoxr-0.1.3_2
2020-09-25 02:40:13+02	2020-09-25 02:41:02+02	sysutils	polkit	\N	polkit-0.118
2020-09-25 02:41:00+02	2020-09-25 02:42:00+02	audio	webrtc-audio-processing	\N	webrtc-audio-processing-0.3.1_2
2020-09-25 02:41:03+02	2020-09-25 02:42:32+02	graphics	colord	\N	colord-1.3.5
2020-09-25 02:42:01+02	2020-09-25 02:43:02+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-25 02:43:03+02	2020-09-25 02:43:26+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-09-25 02:43:27+02	2020-09-25 02:44:07+02	sysutils	hal	\N	hal-0.5.14_34
2020-09-25 02:44:08+02	2020-09-25 02:44:24+02	converters	fpc-iconvenc	\N	fpc-iconvenc-3.0.4_4
2020-09-25 02:44:25+02	2020-09-25 02:45:22+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-09-25 00:18:57+02	2020-09-25 02:45:28+02	lang	gcc9	\N	gcc9-9.3.0_1
2020-09-25 02:45:22+02	2020-09-25 02:45:34+02	devel	jansson	\N	jansson-2.13.1
2020-09-25 02:45:35+02	2020-09-25 02:45:52+02	graphics	openexr	\N	openexr-2.5.3
2020-09-25 02:45:53+02	2020-09-25 02:46:28+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-09-25 02:42:33+02	2020-09-25 02:46:38+02	x11-toolkits	gtk30	\N	gtk3-3.24.23
2020-09-25 02:46:28+02	2020-09-25 02:48:10+02	java	bootstrap-openjdk8	\N	bootstrap-openjdk8-r450802_1
2020-09-25 02:48:10+02	2020-09-25 02:50:00+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-25 02:50:01+02	2020-09-25 02:51:09+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-25 02:45:29+02	2020-09-25 02:51:35+02	math	openblas	\N	openblas-0.3.10,1
2020-09-25 02:51:09+02	2020-09-25 02:52:18+02	x11-toolkits	vte3	\N	vte3-0.52.2_3
2020-09-25 02:52:19+02	2020-09-25 02:52:29+02	java	java-zoneinfo	\N	java-zoneinfo-2020.a
2020-09-25 02:52:29+02	2020-09-25 02:52:47+02	multimedia	libdca	\N	libdca-0.0.6_1
2020-09-25 02:52:48+02	2020-09-25 02:52:55+02	net	avahi-header	\N	avahi-header-0.7
2020-09-25 02:46:39+02	2020-09-25 02:54:04+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-25 02:52:56+02	2020-09-25 02:54:24+02	security	gcr	\N	gcr-3.28.0_1
2020-09-25 02:51:36+02	2020-09-25 02:54:34+02	math	suitesparse	\N	suitesparse-5.8.1
2020-09-25 02:54:24+02	2020-09-25 02:54:45+02	security	libssh	\N	libssh-0.9.5
2020-09-25 02:54:45+02	2020-09-25 02:54:58+02	textproc	p5-XML-SAX	\N	p5-XML-SAX-1.02
2020-09-25 02:54:58+02	2020-09-25 02:55:12+02	x11	libdmx	\N	libdmx-1.1.4_2
2020-09-25 02:54:35+02	2020-09-25 02:55:28+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-09-25 02:55:12+02	2020-09-25 02:55:29+02	x11-fonts	libXfont	\N	libXfont-1.5.4_2,2
2020-09-25 02:55:28+02	2020-09-25 02:56:52+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-09-25 02:56:53+02	2020-09-25 02:57:15+02	devel	fpc-fcl-xml	\N	fpc-fcl-xml-3.0.4_4
2020-09-25 02:55:30+02	2020-09-25 02:57:17+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-09-25 02:57:16+02	2020-09-25 02:57:45+02	devel	hwloc	\N	hwloc-1.11.11
2020-09-25 02:57:18+02	2020-09-25 02:58:11+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-09-25 02:57:45+02	2020-09-25 02:58:34+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-09-25 02:58:35+02	2020-09-25 02:58:46+02	graphics	glew	\N	glew-2.2.0
2020-09-25 02:54:05+02	2020-09-25 02:59:05+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-25 02:58:12+02	2020-09-25 02:59:26+02	dns	c-ares	\N	c-ares-1.16.1
2020-09-25 02:59:27+02	2020-09-25 03:00:04+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-09-25 02:59:05+02	2020-09-25 03:00:09+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-09-25 03:00:04+02	2020-09-25 03:00:18+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-09-25 03:00:19+02	2020-09-25 03:01:19+02	net	openldap24-sasl-client	\N	openldap-sasl-client-2.4.51
2020-09-25 03:00:10+02	2020-09-25 03:01:21+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-25 03:01:20+02	2020-09-25 03:02:07+02	security	munge	\N	munge-0.5.14_1
2020-09-25 03:01:22+02	2020-09-25 03:02:23+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-25 03:02:23+02	2020-09-25 03:02:45+02	textproc	man2html	\N	man2html-3.0.1_2
2020-09-25 03:02:08+02	2020-09-25 03:02:56+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.74.0
2020-09-25 03:02:45+02	2020-09-25 03:03:09+02	textproc	p5-XML-SAX-Expat	\N	p5-XML-SAX-Expat-0.51_1
2020-09-25 03:03:09+02	2020-09-25 03:03:48+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-09-25 03:03:49+02	2020-09-25 03:04:20+02	audio	libmikmod	\N	libmikmod-3.3.11.1_1
2020-09-25 03:04:21+02	2020-09-25 03:04:58+02	audio	mpg123	\N	mpg123-1.26.3
2020-09-25 03:04:59+02	2020-09-25 03:05:15+02	audio	timidity	\N	timidity-0.2i_1
2020-09-25 03:05:15+02	2020-09-25 03:06:19+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-09-25 03:02:57+02	2020-09-25 03:06:29+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-25 03:06:29+02	2020-09-25 03:07:39+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-09-25 03:06:19+02	2020-09-25 03:07:55+02	databases	mysql57-client	\N	mysql57-client-5.7.31_1
2020-09-25 03:07:57+02	2020-09-25 03:08:15+02	devel	py-pytz	py27	py27-pytz-2020.1,1
2020-09-25 03:08:16+02	2020-09-25 03:08:30+02	devel	py-six	py27	py27-six-1.15.0
2020-09-25 03:08:30+02	2020-09-25 03:08:52+02	devel	py-zipp	py37	py37-zipp-3.1.0
2020-09-25 03:07:39+02	2020-09-25 03:09:22+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-09-25 03:09:22+02	2020-09-25 03:09:34+02	devel	tradcpp	\N	tradcpp-0.5.3
2020-09-25 03:08:53+02	2020-09-25 03:10:29+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-25 03:09:34+02	2020-09-25 03:10:45+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-09-25 03:10:30+02	2020-09-25 03:10:52+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-09-25 03:10:45+02	2020-09-25 03:11:06+02	math	glm	\N	glm-0.9.8.5_4,1
2020-09-25 03:11:07+02	2020-09-25 03:11:22+02	security	pinentry-tty	\N	pinentry-tty-1.1.0
2020-09-25 03:10:53+02	2020-09-25 03:12:09+02	net	remmina	\N	remmina-1.3.9
2020-09-25 03:12:10+02	2020-09-25 03:12:50+02	textproc	aspell	\N	aspell-0.60.8,1
2020-09-25 03:12:50+02	2020-09-25 03:13:37+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-09-25 03:13:38+02	2020-09-25 03:14:04+02	textproc	p5-XML-Simple	\N	p5-XML-Simple-2.25
2020-09-25 03:11:23+02	2020-09-25 03:14:20+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-09-25 03:14:21+02	2020-09-25 03:14:41+02	x11	xdpyinfo	\N	xdpyinfo-1.3.2_3
2020-09-25 03:14:05+02	2020-09-25 03:14:51+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-09-25 03:14:42+02	2020-09-25 03:15:06+02	x11	xrdb	\N	xrdb-1.2.0
2020-09-25 03:14:52+02	2020-09-25 03:15:40+02	x11-fonts	font-misc-misc	\N	font-misc-misc-1.1.2_4
2020-09-25 03:15:40+02	2020-09-25 03:16:08+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-09-25 03:15:06+02	2020-09-25 03:16:16+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-09-25 03:16:09+02	2020-09-25 03:16:41+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-09-25 03:16:17+02	2020-09-25 03:16:49+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-09-25 03:16:41+02	2020-09-25 03:16:55+02	audio	libcddb	\N	libcddb-1.3.2_4
2020-09-25 03:16:49+02	2020-09-25 03:17:08+02	audio	libmysofa	\N	libmysofa-1.1
2020-09-25 03:16:55+02	2020-09-25 03:17:10+02	audio	speex	\N	speex-1.2.0,1
2020-09-25 03:17:09+02	2020-09-25 03:17:22+02	devel	gccmakedep	\N	gccmakedep-1.0.3
2020-09-25 03:17:22+02	2020-09-25 03:17:34+02	devel	makedepend	\N	makedepend-1.0.6,1
2020-09-25 03:17:35+02	2020-09-25 03:17:45+02	devel	npth	\N	npth-1.6
2020-09-25 03:17:10+02	2020-09-25 03:17:49+02	devel	libgtop	\N	libgtop-2.40.0
2020-09-25 03:17:46+02	2020-09-25 03:17:59+02	devel	popt	\N	popt-1.18_1
2020-09-25 03:18:00+02	2020-09-25 03:18:11+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-1.7.0
2020-09-25 03:17:50+02	2020-09-25 03:18:26+02	devel	py-babel	py27	py27-Babel-2.8.0
2020-09-25 02:58:47+02	2020-09-25 03:18:33+02	java	openjdk8	\N	openjdk8-8.265.01.1
2020-09-25 03:18:26+02	2020-09-25 03:18:54+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-09-25 03:18:34+02	2020-09-25 03:19:03+02	graphics	exiv2	\N	exiv2-0.27.3,1
2020-09-25 03:18:54+02	2020-09-25 03:19:34+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-09-25 03:19:04+02	2020-09-25 03:19:42+02	graphics	libspiro	\N	libspiro-20190731,1
2020-09-25 03:19:42+02	2020-09-25 03:19:59+02	lang	lua53	\N	lua53-5.3.5_3
2020-09-25 03:18:11+02	2020-09-25 03:20:00+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-25 03:20:00+02	2020-09-25 03:21:12+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-25 03:20:01+02	2020-09-25 03:21:12+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-09-25 03:21:13+02	2020-09-25 03:21:53+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-09-25 03:21:13+02	2020-09-25 03:22:16+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-09-25 03:21:54+02	2020-09-25 03:22:31+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-09-25 03:22:16+02	2020-09-25 03:23:46+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-09-25 03:23:46+02	2020-09-25 03:24:41+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-25 03:24:41+02	2020-09-25 03:25:07+02	security	libksba	\N	libksba-1.4.0
2020-09-25 03:25:08+02	2020-09-25 03:25:19+02	security	pinentry	\N	pinentry-1.1.0_6
2020-09-25 03:25:20+02	2020-09-25 03:25:29+02	textproc	py-MarkupSafe	py27	py27-MarkupSafe-1.1.1
2020-09-25 03:19:34+02	2020-09-25 03:25:39+02	lang	go	\N	go-1.15.2,1
2020-09-25 03:25:29+02	2020-09-25 03:25:43+02	x11	luit	\N	luit-20190106_1
2020-09-25 03:25:41+02	2020-09-25 03:25:51+02	x11	xorg-cf-files	\N	xorg-cf-files-1.0.6
2020-09-25 03:25:52+02	2020-09-25 03:26:18+02	x11-themes	icon-naming-utils	\N	icon-naming-utils-0.8.90_1
2020-09-25 03:25:43+02	2020-09-25 03:26:26+02	x11-fonts	font-adobe-100dpi	\N	font-adobe-100dpi-1.0.3_4
2020-09-25 03:26:27+02	2020-09-25 03:27:21+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-09-25 03:26:19+02	2020-09-25 03:28:31+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.74.0
2020-09-25 03:27:21+02	2020-09-25 03:28:34+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-09-25 03:22:31+02	2020-09-25 03:29:13+02	net	openmpi	\N	openmpi-4.0.5
2020-09-25 03:28:32+02	2020-09-25 03:29:14+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-09-25 03:28:35+02	2020-09-25 03:29:26+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-25 03:29:14+02	2020-09-25 03:29:34+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-09-25 03:29:26+02	2020-09-25 03:29:45+02	audio	openal-soft	\N	openal-soft-1.20.1_2
2020-09-25 03:29:45+02	2020-09-25 03:30:02+02	devel	autoconf-archive	\N	autoconf-archive-0.2019.01.06
2020-09-25 03:30:03+02	2020-09-25 03:30:23+02	devel	gamin	\N	gamin-0.1.10_10
2020-09-25 03:29:14+02	2020-09-25 03:30:35+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-25 03:29:35+02	2020-09-25 03:30:36+02	devel	apr1	\N	apr-1.7.0.1.6.1_1
2020-09-25 03:30:24+02	2020-09-25 03:30:41+02	devel	imake	\N	imake-1.0.8,1
2020-09-25 03:30:36+02	2020-09-25 03:30:48+02	devel	p5-TimeDate	\N	p5-TimeDate-2.33,1
2020-09-25 03:30:41+02	2020-09-25 03:30:51+02	devel	py-Jinja2	py27	py27-Jinja2-2.10.1
2020-09-25 03:30:48+02	2020-09-25 03:30:58+02	devel	py-attrs	py37	py37-attrs-19.3.0
2020-09-25 03:30:51+02	2020-09-25 03:31:02+02	devel	py-olefile	py37	py37-olefile-0.46
2020-09-25 03:30:35+02	2020-09-25 03:31:06+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-09-25 03:30:59+02	2020-09-25 03:31:10+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-09-25 03:31:02+02	2020-09-25 03:31:13+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-09-25 03:31:10+02	2020-09-25 03:31:20+02	devel	yajl	\N	yajl-2.1.0
2020-09-25 03:31:21+02	2020-09-25 03:31:31+02	lang	erlang-man	\N	erlang-man-21.3_2
2020-09-25 03:31:06+02	2020-09-25 03:31:40+02	devel	talloc	\N	talloc-2.3.1
2020-09-25 03:31:14+02	2020-09-25 03:31:43+02	graphics	freeimage	\N	freeimage-3.18.0_1
2020-09-25 03:31:43+02	2020-09-25 03:32:12+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-25 03:32:12+02	2020-09-25 03:32:26+02	print	gl2ps	\N	gl2ps-1.4.0
2020-09-25 03:31:32+02	2020-09-25 03:33:07+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-25 03:32:26+02	2020-09-25 03:33:08+02	security	gnupg	\N	gnupg-2.2.23
2020-09-25 03:33:09+02	2020-09-25 03:33:20+02	textproc	yelp-xsl	\N	yelp-xsl-3.38.0
2020-09-25 03:33:08+02	2020-09-25 03:33:30+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-09-25 03:33:21+02	2020-09-25 03:33:31+02	x11	xauth	\N	xauth-1.1
2020-09-25 03:33:30+02	2020-09-25 03:33:51+02	x11	xterm	\N	xterm-359
2020-09-25 03:33:31+02	2020-09-25 03:33:56+02	x11-fonts	font-adobe-75dpi	\N	font-adobe-75dpi-1.0.3_4
2020-09-25 03:33:52+02	2020-09-25 03:34:00+02	x11-fonts	font-alias	\N	font-alias-1.0.4
2020-09-25 03:33:57+02	2020-09-25 03:34:50+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-09-25 03:34:50+02	2020-09-25 03:35:33+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-09-25 03:35:33+02	2020-09-25 03:35:46+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.9_6
2020-09-25 03:34:01+02	2020-09-25 03:35:48+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-09-25 03:35:46+02	2020-09-25 03:36:07+02	archivers	minizip	\N	minizip-1.2.11
2020-09-25 03:31:41+02	2020-09-25 03:36:17+02	math	vtk8	\N	vtk8-8.2.0
2020-09-25 03:35:48+02	2020-09-25 03:36:36+02	databases	firebird25-client	\N	firebird25-client-2.5.8_6
2020-09-25 03:36:36+02	2020-09-25 03:36:44+02	devel	gnome-common	\N	gnome-common-3.18.0
2020-09-25 03:36:07+02	2020-09-25 03:36:48+02	databases	tdb	\N	tdb-1.4.3,1
2020-09-25 03:36:48+02	2020-09-25 03:37:01+02	devel	p5-Parse-Yapp	\N	p5-Parse-Yapp-1.21
2020-09-25 03:36:44+02	2020-09-25 03:37:09+02	devel	librest	\N	rest-0.7.93
2020-09-25 03:37:02+02	2020-09-25 03:37:11+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-09-25 03:37:10+02	2020-09-25 03:37:43+02	devel	tevent	\N	tevent-0.10.2
2020-09-25 03:37:11+02	2020-09-25 03:37:58+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-09-25 03:37:44+02	2020-09-25 03:38:24+02	graphics	netpbm	\N	netpbm-10.91.01
2020-09-25 03:37:58+02	2020-09-25 03:38:27+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-09-25 03:38:28+02	2020-09-25 03:38:36+02	multimedia	gmp-api	\N	gmp-api-58
2020-09-25 03:38:36+02	2020-09-25 03:39:29+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-09-25 03:39:29+02	2020-09-25 03:41:02+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-09-25 03:36:17+02	2020-09-25 03:41:13+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-25 03:41:14+02	2020-09-25 03:41:32+02	security	gpgme	\N	gpgme-1.14.0
2020-09-25 03:41:33+02	2020-09-25 03:41:51+02	sysutils	cmocka	\N	cmocka-1.1.5
2020-09-25 03:41:51+02	2020-09-25 03:41:59+02	sysutils	libsunacl	\N	libsunacl-1.0.1
2020-09-25 03:41:59+02	2020-09-25 03:42:15+02	textproc	exempi	\N	exempi-2.2.2_1
2020-09-25 03:42:15+02	2020-09-25 03:43:05+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-09-25 03:43:05+02	2020-09-25 03:43:56+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-09-25 03:41:02+02	2020-09-25 03:44:02+02	net	libmateweather	\N	libmateweather-1.24.0
2020-09-25 03:43:57+02	2020-09-25 03:44:10+02	www	p5-HTTP-Date	\N	p5-HTTP-Date-6.05
2020-09-25 03:44:11+02	2020-09-25 03:44:21+02	x11	xcursorgen	\N	xcursorgen-1.0.7
2020-09-25 03:44:21+02	2020-09-25 03:44:35+02	x11-fonts	font-adobe-utopia-100dpi	\N	font-adobe-utopia-100dpi-1.0.4_4
2020-09-25 03:44:35+02	2020-09-25 03:44:49+02	x11-fonts	font-adobe-utopia-75dpi	\N	font-adobe-utopia-75dpi-1.0.4_4
2020-09-25 03:44:49+02	2020-09-25 03:44:59+02	x11-fonts	font-adobe-utopia-type1	\N	font-adobe-utopia-type1-1.0.4_4
2020-09-25 03:44:59+02	2020-09-25 03:45:08+02	x11-fonts	font-arabic-misc	\N	font-arabic-misc-1.0.3_4
2020-09-25 03:45:09+02	2020-09-25 03:45:33+02	x11-fonts	font-bh-100dpi	\N	font-bh-100dpi-1.0.3_4
2020-09-25 03:45:34+02	2020-09-25 03:45:58+02	x11-fonts	font-bh-75dpi	\N	font-bh-75dpi-1.0.3_4
2020-09-25 03:45:59+02	2020-09-25 03:46:16+02	x11-fonts	font-bh-lucidatypewriter-100dpi	\N	font-bh-lucidatypewriter-100dpi-1.0.3_4
2020-09-25 03:46:17+02	2020-09-25 03:46:31+02	x11-fonts	font-bh-lucidatypewriter-75dpi	\N	font-bh-lucidatypewriter-75dpi-1.0.3_4
2020-09-25 03:46:32+02	2020-09-25 03:46:43+02	x11-fonts	font-bh-type1	\N	font-bh-type1-1.0.3_4
2020-09-25 03:46:43+02	2020-09-25 03:46:53+02	x11-fonts	font-bitstream-100dpi	\N	font-bitstream-100dpi-1.0.3_4
2020-09-25 03:46:53+02	2020-09-25 03:47:05+02	x11-fonts	font-bitstream-75dpi	\N	font-bitstream-75dpi-1.0.3_4
2020-09-25 03:47:05+02	2020-09-25 03:47:20+02	x11-fonts	font-bitstream-type1	\N	font-bitstream-type1-1.0.3_4
2020-09-25 03:47:21+02	2020-09-25 03:47:31+02	x11-fonts	font-cronyx-cyrillic	\N	font-cronyx-cyrillic-1.0.3_4
2020-09-25 03:47:32+02	2020-09-25 03:47:43+02	x11-fonts	font-cursor-misc	\N	font-cursor-misc-1.0.3_4
2020-09-25 03:47:44+02	2020-09-25 03:47:55+02	x11-fonts	font-daewoo-misc	\N	font-daewoo-misc-1.0.3_4
2020-09-25 03:48:01+02	2020-09-25 03:48:14+02	x11-fonts	font-dec-misc	\N	font-dec-misc-1.0.3_4
2020-09-25 03:48:14+02	2020-09-25 03:48:25+02	x11-fonts	font-ibm-type1	\N	font-ibm-type1-1.0.3_4
2020-09-25 03:48:25+02	2020-09-25 03:48:38+02	x11-fonts	font-isas-misc	\N	font-isas-misc-1.0.3_4
2020-09-25 03:48:39+02	2020-09-25 03:48:52+02	x11-fonts	font-jis-misc	\N	font-jis-misc-1.0.3_4
2020-09-25 03:48:52+02	2020-09-25 03:49:03+02	x11-fonts	font-micro-misc	\N	font-micro-misc-1.0.3_4
2020-09-25 03:49:03+02	2020-09-25 03:49:14+02	x11-fonts	font-misc-cyrillic	\N	font-misc-cyrillic-1.0.3_4
2020-09-25 03:49:14+02	2020-09-25 03:49:23+02	x11-fonts	font-mutt-misc	\N	font-mutt-misc-1.0.3_4
2020-09-25 03:49:24+02	2020-09-25 03:49:41+02	x11-fonts	font-schumacher-misc	\N	font-schumacher-misc-1.1.2_4
2020-09-25 03:49:41+02	2020-09-25 03:49:50+02	x11-fonts	font-screen-cyrillic	\N	font-screen-cyrillic-1.0.4_4
2020-09-25 03:49:51+02	2020-09-25 03:50:01+02	x11-fonts	font-sony-misc	\N	font-sony-misc-1.0.3_4
2020-09-25 03:50:02+02	2020-09-25 03:50:11+02	x11-fonts	font-sun-misc	\N	font-sun-misc-1.0.3_4
2020-09-25 03:44:03+02	2020-09-25 03:50:17+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-25 03:50:11+02	2020-09-25 03:50:20+02	x11-fonts	font-winitzki-cyrillic	\N	font-winitzki-cyrillic-1.0.3_4
2020-09-25 03:50:18+02	2020-09-25 03:50:26+02	x11-fonts	font-xfree86-type1	\N	font-xfree86-type1-1.0.4_4
2020-09-25 03:50:27+02	2020-09-25 03:51:09+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-09-25 03:50:21+02	2020-09-25 03:51:11+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-25 03:51:09+02	2020-09-25 03:51:19+02	audio	celt	\N	celt-0.11.3_3
2020-09-25 03:51:12+02	2020-09-25 03:51:22+02	audio	libmad	\N	libmad-0.15.1b_7
2020-09-25 03:51:19+02	2020-09-25 03:51:32+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-09-25 03:38:24+02	2020-09-25 03:51:37+02	lang	erlang	\N	erlang-21.3.8.17,4
2020-09-25 03:51:32+02	2020-09-25 03:51:45+02	converters	p5-Encode-Locale	\N	p5-Encode-Locale-1.05
2020-09-25 03:51:38+02	2020-09-25 03:51:49+02	databases	fpc-dblib	\N	fpc-dblib-3.0.4_4
2020-09-25 03:51:46+02	2020-09-25 03:52:00+02	databases	fpc-ibase	\N	fpc-ibase-3.0.4_5
2020-09-25 03:52:01+02	2020-09-25 03:52:16+02	databases	fpc-odbc	\N	fpc-odbc-3.0.4_4
2020-09-25 03:51:49+02	2020-09-25 03:52:16+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-09-25 03:52:17+02	2020-09-25 03:52:31+02	databases	fpc-postgres	\N	fpc-postgres-3.0.4_6
2020-09-25 03:52:16+02	2020-09-25 03:52:31+02	databases	fpc-oracle	\N	fpc-oracle-3.0.4_4
2020-09-25 03:52:32+02	2020-09-25 03:52:49+02	databases	fpc-pxlib	\N	fpc-pxlib-3.0.4_4
2020-09-25 03:52:32+02	2020-09-25 03:52:51+02	databases	fpc-sqlite	\N	fpc-sqlite-3.0.4_4
2020-09-25 03:52:49+02	2020-09-25 03:53:05+02	devel	fpc-fcl-async	\N	fpc-fcl-async-3.0.4_4
2020-09-25 03:52:51+02	2020-09-25 03:53:08+02	devel	fpc-fcl-json	\N	fpc-fcl-json-3.0.4_5
2020-09-25 03:51:22+02	2020-09-25 03:53:12+02	cad	ngspice_rework	shlib	ngspice_rework-shlib-32
2020-09-25 03:53:06+02	2020-09-25 03:53:20+02	devel	fpc-fcl-passrc	\N	fpc-fcl-passrc-3.0.4_5
2020-09-25 03:53:20+02	2020-09-25 03:53:32+02	devel	netsurf-buildsystem	\N	netsurf-buildsystem-1.8
2020-09-25 03:53:33+02	2020-09-25 03:53:44+02	devel	p5-Clone	\N	p5-Clone-0.45
2020-09-25 03:53:13+02	2020-09-25 03:53:45+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-25 03:53:08+02	2020-09-25 03:53:47+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-09-25 03:53:45+02	2020-09-25 03:53:54+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-09-25 03:53:48+02	2020-09-25 03:53:56+02	devel	py-pycparser	py27	py27-pycparser-2.20
2020-09-25 03:53:45+02	2020-09-25 03:53:57+02	devel	p5-IO-HTML	\N	p5-IO-HTML-1.001_1
2020-09-25 03:53:58+02	2020-09-25 03:54:08+02	multimedia	openh264	\N	openh264-2.1.1,2
2020-09-25 03:53:57+02	2020-09-25 03:54:49+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-09-25 03:54:08+02	2020-09-25 03:54:54+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-09-25 03:54:55+02	2020-09-25 03:55:06+02	net	liboauth	\N	liboauth-1.0.3_4
2020-09-25 03:55:06+02	2020-09-25 03:55:16+02	net	norm	\N	norm-1.5r6_1
2020-09-25 03:53:55+02	2020-09-25 03:55:28+02	multimedia	assimp	\N	assimp-5.0.1
2020-09-25 03:55:16+02	2020-09-25 03:55:48+02	net	openpgm	\N	openpgm-5.2.122_6
2020-09-25 03:54:49+02	2020-09-25 03:55:58+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-25 03:55:49+02	2020-09-25 03:56:30+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-09-25 03:55:59+02	2020-09-25 03:56:38+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-25 03:56:30+02	2020-09-25 03:56:42+02	security	fpc-openssl	\N	fpc-openssl-3.0.4_4
2020-09-25 03:56:43+02	2020-09-25 03:56:52+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7_1
2020-09-25 03:56:39+02	2020-09-25 03:56:55+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-09-25 03:56:56+02	2020-09-25 03:57:20+02	sysutils	libcdio	\N	libcdio-2.1.0
2020-09-25 03:56:53+02	2020-09-25 03:57:20+02	sysutils	fusefs-libs	\N	fusefs-libs-2.9.9_1
2020-09-25 03:57:21+02	2020-09-25 03:57:44+02	sysutils	upower	\N	upower-0.99.11
2020-09-25 03:57:20+02	2020-09-25 03:58:03+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-25 03:57:45+02	2020-09-25 03:58:08+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-09-25 03:58:06+02	2020-09-25 03:58:21+02	www	p5-LWP-MediaTypes	\N	p5-LWP-MediaTypes-6.04
2020-09-25 03:58:22+02	2020-09-25 03:58:33+02	x11	appres	\N	appres-1.0.5
2020-09-25 03:58:33+02	2020-09-25 03:58:45+02	x11	bitmap	\N	bitmap-1.0.9
2020-09-25 03:58:46+02	2020-09-25 03:58:57+02	x11	iceauth	\N	iceauth-1.0.8_2
2020-09-25 03:58:58+02	2020-09-25 04:00:05+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-09-25 04:00:06+02	2020-09-25 04:00:30+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-09-25 04:00:31+02	2020-09-25 04:04:34+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-25 04:04:35+02	2020-09-25 04:04:45+02	x11	sessreg	\N	sessreg-1.1.2
2020-09-25 04:04:46+02	2020-09-25 04:04:56+02	x11	setxkbmap	\N	setxkbmap-1.3.2
2020-09-25 04:04:57+02	2020-09-25 04:05:09+02	x11	smproxy	\N	smproxy-1.0.6
2020-09-25 04:05:09+02	2020-09-25 04:05:23+02	x11	x11perf	\N	x11perf-1.6.1
2020-09-25 04:05:23+02	2020-09-25 04:05:37+02	x11	xbacklight	\N	xbacklight-1.2.3
2020-09-25 03:55:28+02	2020-09-25 04:05:45+02	net	samba410	\N	samba410-4.10.18
2020-09-25 04:05:46+02	2020-09-25 04:05:58+02	x11	xcmsdb	\N	xcmsdb-1.0.5
2020-09-25 04:05:37+02	2020-09-25 04:05:59+02	x11	xcalc	\N	xcalc-1.1.0
2020-09-25 04:05:59+02	2020-09-25 04:06:12+02	x11	xconsole	\N	xconsole-1.0.7_1
2020-09-25 04:06:01+02	2020-09-25 04:06:13+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-09-25 04:06:13+02	2020-09-25 04:06:28+02	x11	xev	\N	xev-1.2.4
2020-09-25 04:06:14+02	2020-09-25 04:06:29+02	x11	xf86dga	\N	xf86dga-1.0.3_1
2020-09-25 04:06:29+02	2020-09-25 04:06:40+02	x11	xgamma	\N	xgamma-1.0.6
2020-09-25 04:07:06+02	2020-09-25 04:07:19+02	x11	xkbutils	\N	xkbutils-1.0.4_2
2020-09-25 04:07:19+02	2020-09-25 04:07:31+02	x11	xlsatoms	\N	xlsatoms-1.1.3
2020-09-25 04:07:19+02	2020-09-25 04:07:32+02	x11	xlsclients	\N	xlsclients-1.1.4
2020-09-25 04:07:31+02	2020-09-25 04:07:49+02	x11	xmessage	\N	xmessage-1.0.5
2020-09-25 04:07:32+02	2020-09-25 04:07:50+02	x11	xmodmap	\N	xmodmap-1.0.10
2020-09-25 04:07:50+02	2020-09-25 04:08:02+02	x11	xpr	\N	xpr-1.0.5
2020-09-25 04:07:51+02	2020-09-25 04:08:04+02	x11	xrandr	\N	xrandr-1.5.1
2020-09-25 04:08:02+02	2020-09-25 04:08:17+02	x11	xrefresh	\N	xrefresh-1.0.6
2020-09-25 04:08:04+02	2020-09-25 04:08:19+02	x11	xsetroot	\N	xsetroot-1.1.2
2020-09-25 04:08:17+02	2020-09-25 04:08:30+02	x11	xvinfo	\N	xvinfo-1.1.4
2020-09-25 04:08:19+02	2020-09-25 04:08:32+02	x11	xwd	\N	xwd-1.0.7
2020-09-25 04:08:31+02	2020-09-25 04:08:44+02	x11	xwininfo	\N	xwininfo-1.1.5
2020-09-25 04:08:33+02	2020-09-25 04:08:44+02	x11	xwud	\N	xwud-1.0.5
2020-09-25 04:08:44+02	2020-09-25 04:08:58+02	x11-clocks	xclock	\N	xclock-1.0.9
2020-09-25 04:08:45+02	2020-09-25 04:09:49+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-09-25 04:08:58+02	2020-09-25 04:09:54+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-09-25 04:09:49+02	2020-09-25 04:10:33+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-09-25 04:09:55+02	2020-09-25 04:10:39+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-09-25 04:10:34+02	2020-09-25 04:11:31+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-09-25 04:10:41+02	2020-09-25 04:11:54+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-09-25 04:11:31+02	2020-09-25 04:12:14+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-09-25 04:12:14+02	2020-09-25 04:12:25+02	x11-fonts	libFS	\N	libFS-1.0.8
2020-09-25 04:11:55+02	2020-09-25 04:12:36+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.5.0
2020-09-25 03:58:09+02	2020-09-25 04:12:46+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_3
2020-09-25 04:12:26+02	2020-09-25 04:12:52+02	x11-fonts	xorg-fonts-100dpi	\N	xorg-fonts-100dpi-7.7
2020-09-25 04:12:36+02	2020-09-25 04:13:01+02	x11-fonts	xorg-fonts-75dpi	\N	xorg-fonts-75dpi-7.7
2020-09-25 04:12:47+02	2020-09-25 04:13:02+02	x11-fonts	xorg-fonts-cyrillic	\N	xorg-fonts-cyrillic-7.7
2020-09-25 04:13:02+02	2020-09-25 04:13:19+02	x11-fonts	xorg-fonts-type1	\N	xorg-fonts-type1-7.7
2020-09-25 04:12:52+02	2020-09-25 04:13:29+02	x11-fonts	xorg-fonts-miscbitmaps	\N	xorg-fonts-miscbitmaps-7.7
2020-09-25 04:13:02+02	2020-09-25 04:13:38+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-09-25 04:13:30+02	2020-09-25 04:13:54+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-09-25 04:13:54+02	2020-09-25 04:14:06+02	x11-wm	twm	\N	twm-1.0.11
2020-09-25 04:13:19+02	2020-09-25 04:14:05+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-09-25 04:14:06+02	2020-09-25 04:15:30+02	archivers	gcab	\N	gcab-1.3
2020-09-25 04:14:06+02	2020-09-25 04:17:19+02	audio	jack	\N	jackit-0.125.0_10
2020-09-25 04:17:22+02	2020-09-25 04:17:42+02	converters	py-webencodings	py27	py27-webencodings-0.5.1
2020-09-25 04:15:33+02	2020-09-25 04:17:45+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-09-25 04:17:42+02	2020-09-25 04:17:54+02	devel	boehm-gc-threaded	\N	boehm-gc-threaded-8.0.4_1
2020-09-25 04:17:45+02	2020-09-25 04:17:59+02	devel	cppunit	\N	cppunit-1.14.0_8
2020-09-25 04:18:00+02	2020-09-25 04:18:12+02	devel	fpc-fcl-fpcunit	\N	fpc-fcl-fpcunit-3.0.4_4
2020-09-25 04:13:38+02	2020-09-25 04:18:19+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-25 04:18:13+02	2020-09-25 04:18:25+02	devel	fpc-fcl-net	\N	fpc-fcl-net-3.0.4_4
2020-09-25 04:17:55+02	2020-09-25 04:18:31+02	devel	fpc-fcl-db	\N	fpc-fcl-db-3.0.4_6
2020-09-25 04:18:26+02	2020-09-25 04:18:38+02	devel	gflags	\N	gflags-2.2.2_2
2020-09-25 04:18:31+02	2020-09-25 04:19:25+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-09-25 04:18:38+02	2020-09-25 04:19:55+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-25 04:19:25+02	2020-09-25 04:20:01+02	devel	m17n-db	\N	m17n-db-1.8.0
2020-09-25 04:19:56+02	2020-09-25 04:20:07+02	devel	py-cffi	py27	py27-cffi-1.14.3
2020-09-25 04:20:02+02	2020-09-25 04:20:17+02	devel	scons	py37	scons-py37-3.1.2
2020-09-25 04:20:07+02	2020-09-25 04:20:23+02	graphics	fpc-pasjpeg	\N	fpc-pasjpeg-3.0.4_4
2020-09-25 04:20:23+02	2020-09-25 04:21:27+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-09-25 04:21:28+02	2020-09-25 04:21:46+02	graphics	libnsgif	\N	libnsgif-0.2.1
2020-09-25 04:20:17+02	2020-09-25 04:22:04+02	graphics	leptonica	\N	leptonica-1.76.0_1
2020-09-25 04:21:46+02	2020-09-25 04:22:19+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-09-25 04:22:19+02	2020-09-25 04:22:33+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-09-25 04:22:05+02	2020-09-25 04:24:45+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-09-25 04:24:46+02	2020-09-25 04:27:06+02	lang	python38	\N	python38-3.8.5
2020-09-25 04:27:06+02	2020-09-25 04:27:44+02	net	libzmq4	\N	libzmq4-4.3.1_1
2020-09-25 04:27:44+02	2020-09-25 04:28:08+02	print	enscript-a4	\N	enscript-a4-1.6.6_3
2020-09-25 04:28:09+02	2020-09-25 04:28:22+02	print	libotf	\N	libotf-0.9.16
2020-09-25 04:28:22+02	2020-09-25 04:29:04+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-09-25 04:29:04+02	2020-09-25 04:29:52+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-25 04:29:53+02	2020-09-25 04:30:40+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-09-25 04:30:40+02	2020-09-25 04:31:13+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-09-25 04:31:13+02	2020-09-25 04:32:00+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-25 04:32:00+02	2020-09-25 04:32:18+02	sysutils	libcdio-paranoia	\N	libcdio-paranoia-10.2+2.0.1
2020-09-25 04:18:20+02	2020-09-25 04:32:34+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-25 04:32:35+02	2020-09-25 04:32:48+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-09-25 04:32:19+02	2020-09-25 04:32:49+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-09-25 04:32:49+02	2020-09-25 04:32:56+02	textproc	tinyxml	\N	tinyxml-2.6.2_4
2020-09-25 04:32:50+02	2020-09-25 04:33:00+02	www	fpc-fastcgi	\N	fpc-fastcgi-3.0.4_4
2020-09-25 04:32:57+02	2020-09-25 04:33:09+02	www	fpc-httpd22	\N	fpc-httpd22-3.0.4_4
2020-09-25 04:33:01+02	2020-09-25 04:33:12+02	www	fpc-httpd24	\N	fpc-httpd24-3.0.4_4
2020-09-25 04:33:12+02	2020-09-25 04:33:25+02	www	p5-HTTP-Message	\N	p5-HTTP-Message-6.26
2020-09-25 04:33:26+02	2020-09-25 04:34:12+02	x11	babl	\N	babl-0.1.78
2020-09-25 04:34:12+02	2020-09-25 04:34:27+02	x11	fpc-x11	\N	fpc-x11-3.0.4_6
2020-09-25 04:34:28+02	2020-09-25 04:35:18+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-25 04:33:09+02	2020-09-25 04:35:29+02	www	node	\N	node-14.11.0
2020-09-25 04:35:19+02	2020-09-25 04:35:37+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-09-25 04:35:29+02	2020-09-25 04:35:38+02	x11	xorg-docs	\N	xorg-docs-1.7.1,1
2020-09-25 04:35:37+02	2020-09-25 04:35:46+02	x11	xorg-libraries	\N	xorg-libraries-7.7_4
2020-09-25 04:35:38+02	2020-09-25 04:36:20+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-25 04:35:46+02	2020-09-25 04:37:07+02	x11-fm	caja	\N	caja-1.22.3_1
2020-09-25 04:36:21+02	2020-09-25 04:38:12+02	x11-fonts	xorg-fonts	\N	xorg-fonts-7.7_1
2020-09-25 04:37:08+02	2020-09-25 04:38:12+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.9,1
2020-09-25 04:22:34+02	2020-09-25 04:38:12+02	graphics	tesseract-data	\N	tesseract-data-4.0.0
2020-09-25 04:38:12+02	2020-09-25 04:38:24+02	x11-themes	xcursor-themes	\N	xcursor-themes-1.0.6
2020-09-25 04:38:24+02	2020-09-25 04:39:11+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-25 04:38:13+02	2020-09-25 04:39:18+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-09-25 04:39:18+02	2020-09-25 04:39:25+02	archivers	unzip	\N	unzip-6.0_8
2020-09-25 04:38:12+02	2020-09-25 04:39:26+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-25 04:39:25+02	2020-09-25 04:39:31+02	audio	gsm	\N	gsm-1.0.13_2
2020-09-25 04:39:27+02	2020-09-25 04:39:38+02	audio	lame	\N	lame-3.100_2
2020-09-25 04:39:32+02	2020-09-25 04:39:41+02	audio	libao	\N	libao-1.2.0_5
2020-09-25 04:39:39+02	2020-09-25 04:39:49+02	audio	libid3tag	\N	libid3tag-0.15.1b_2
2020-09-25 04:39:41+02	2020-09-25 04:40:06+02	audio	opusfile	\N	opusfile-0.12
2020-09-25 04:39:13+02	2020-09-25 04:40:11+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-25 04:40:11+02	2020-09-25 04:40:24+02	devel	fpc-fcl-image	\N	fpc-fcl-image-3.0.4_4
2020-09-25 04:39:50+02	2020-09-25 04:40:43+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-25 04:40:25+02	2020-09-25 04:40:53+02	devel	fpc-fcl-web	\N	fpc-fcl-web-3.0.4_7
2020-09-25 04:40:07+02	2020-09-25 04:41:03+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-09-25 04:40:53+02	2020-09-25 04:41:10+02	devel	glslang	\N	glslang-8.13.3743
2020-09-25 04:40:43+02	2020-09-25 04:41:12+02	devel	glog	\N	glog-0.4.0_1
2020-09-25 04:41:13+02	2020-09-25 04:41:59+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-09-25 04:41:11+02	2020-09-25 04:42:10+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-09-25 04:42:00+02	2020-09-25 04:42:19+02	devel	liblangtag	\N	liblangtag-0.6.2
2020-09-25 04:42:20+02	2020-09-25 04:42:26+02	devel	mdds	\N	mdds-1.5.0
2020-09-25 04:41:04+02	2020-09-25 04:42:27+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-25 04:42:27+02	2020-09-25 04:42:37+02	devel	py-click	py37	py37-click-7.0
2020-09-25 04:42:27+02	2020-09-25 04:42:39+02	devel	p5-IO-String	\N	p5-IO-String-1.08_1
2020-09-25 04:42:37+02	2020-09-25 04:42:46+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-09-25 04:42:39+02	2020-09-25 04:42:48+02	devel	py-pluggy	py37	py37-pluggy-0.13.1
2020-09-25 04:42:12+02	2020-09-25 04:42:51+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-09-25 04:42:46+02	2020-09-25 04:42:55+02	devel	py-ply	py37	py37-ply-3.11
2020-09-25 04:42:49+02	2020-09-25 04:42:57+02	devel	py-ply	py27	py27-ply-3.11
2020-09-25 04:42:51+02	2020-09-25 04:43:01+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-09-25 04:42:57+02	2020-09-25 04:43:06+02	devel	rubygem-mustache	\N	rubygem-mustache-1.1.1
2020-09-25 04:42:56+02	2020-09-25 04:43:07+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-09-25 04:43:07+02	2020-09-25 04:43:24+02	devel	tex-libtexlua	\N	tex-libtexlua-5.2.4
2020-09-25 04:43:01+02	2020-09-25 04:43:27+02	devel	spirv-tools	\N	spirv-tools-2020.2
2020-09-25 04:43:08+02	2020-09-25 04:43:28+02	devel	tex-libtexluajit	\N	tex-libtexluajit-2.0.3
2020-09-25 04:43:24+02	2020-09-25 04:43:36+02	devel	yaml-cpp03	\N	yaml-cpp03-0.3.0_2
2020-09-25 04:43:29+02	2020-09-25 04:43:48+02	graphics	libcdr01	\N	libcdr01-0.1.6
2020-09-25 04:43:36+02	2020-09-25 04:43:51+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-09-25 04:43:51+02	2020-09-25 04:44:08+02	graphics	mypaint-brushes	\N	mypaint-brushes-1.3.1
2020-09-25 04:43:50+02	2020-09-25 04:44:10+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-09-25 04:44:09+02	2020-09-25 04:44:26+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-09-25 04:44:10+02	2020-09-25 04:44:27+02	graphics	simage	\N	simage-1.7.0_7
2020-09-25 04:43:28+02	2020-09-25 04:44:31+02	graphics	gegl	\N	gegl-0.4.26_5
2020-09-25 04:44:31+02	2020-09-25 04:44:41+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0,1
2020-09-25 04:44:41+02	2020-09-25 04:44:55+02	math	qhull	\N	qhull-7.2.1,1
2020-09-25 04:44:56+02	2020-09-25 04:45:03+02	misc	compat12x	\N	compat12x-amd64-12.1.1201000.20200220
2020-09-25 04:45:04+02	2020-09-25 04:45:26+02	misc	fpc-chm	\N	fpc-chm-3.0.4_4
2020-09-25 04:45:26+02	2020-09-25 04:45:37+02	misc	usbids	\N	usbids-20200826
2020-09-25 04:45:38+02	2020-09-25 04:45:45+02	multimedia	librtmp	\N	librtmp-2.4.20190330
2020-09-25 04:44:28+02	2020-09-25 04:45:55+02	mail	mailutils	\N	mailutils-3.10
2020-09-25 04:45:45+02	2020-09-25 04:45:56+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-09-25 04:45:55+02	2020-09-25 04:46:12+02	print	qpdf	\N	qpdf-10.0.1
2020-09-25 04:46:12+02	2020-09-25 04:46:37+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-09-25 04:45:57+02	2020-09-25 04:47:10+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-25 04:46:38+02	2020-09-25 04:47:13+02	sysutils	flock	\N	flock-2.30.1
2020-09-25 04:47:13+02	2020-09-25 04:47:45+02	textproc	libuninameslist	\N	libuninameslist-20190701
2020-09-25 04:44:26+02	2020-09-25 04:48:10+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-09-25 04:47:45+02	2020-09-25 04:48:20+02	textproc	libvisio01	\N	libvisio01-0.1.7_2
2020-09-25 04:48:10+02	2020-09-25 04:48:21+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-09-25 04:47:11+02	2020-09-25 04:48:22+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-09-25 04:48:21+02	2020-09-25 04:48:28+02	textproc	utf8proc	\N	utf8proc-2.5.0
2020-09-25 04:48:22+02	2020-09-25 04:48:33+02	textproc	yelp-tools	\N	yelp-tools-3.38.0
2020-09-25 04:48:20+02	2020-09-25 04:48:34+02	textproc	rubygem-rdiscount	\N	rubygem-rdiscount-2.2.0.1
2020-09-25 04:48:28+02	2020-09-25 04:48:37+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-09-25 04:48:33+02	2020-09-25 04:48:41+02	www	py-html5lib	py27	py27-html5lib-1.0.1
2020-09-25 04:48:34+02	2020-09-25 04:48:42+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-09-25 04:48:37+02	2020-09-25 04:48:48+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-09-25 04:48:42+02	2020-09-25 04:49:01+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-09-25 04:48:43+02	2020-09-25 04:49:19+02	www	rubygem-hpricot	\N	rubygem-hpricot-0.8.6
2020-09-25 04:48:49+02	2020-09-25 04:49:35+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-09-25 04:49:35+02	2020-09-25 04:49:42+02	x11-fonts	gnu-unifont	\N	gnu-unifont-13.0.03,1
2020-09-25 04:49:20+02	2020-09-25 04:50:13+02	x11	zenity	\N	zenity-3.32.0
2020-09-25 04:49:42+02	2020-09-25 04:50:20+02	x11-fonts	noto-basic	\N	noto-basic-2.0_1
2020-09-25 04:50:21+02	2020-09-25 04:50:46+02	archivers	gtar	\N	gtar-1.32
2020-09-25 04:50:13+02	2020-09-25 04:50:52+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-09-25 04:50:46+02	2020-09-25 04:50:55+02	audio	libmodplug	\N	libmodplug-0.8.9.0
2020-09-25 04:50:55+02	2020-09-25 04:51:05+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-09-25 04:51:05+02	2020-09-25 04:51:14+02	databases	py-redis	py37	py37-redis-3.5.3
2020-09-25 04:49:02+02	2020-09-25 04:51:33+02	x11	xorg	\N	xorg-7.7_3
2020-09-25 04:50:53+02	2020-09-25 04:51:36+02	audio	sox	\N	sox-14.4.2_4
2020-09-25 04:51:36+02	2020-09-25 04:51:47+02	devel	fpc-regexpr	\N	fpc-regexpr-3.0.4_4
2020-09-25 04:51:14+02	2020-09-25 04:51:47+02	devel	apache-ant	\N	apache-ant-1.10.8
2020-09-25 04:51:34+02	2020-09-25 04:51:59+02	devel	fpc-fppkg	\N	fpc-fppkg-3.0.4_6
2020-09-25 04:51:59+02	2020-09-25 04:52:09+02	devel	physfs	\N	physfs-3.0.2
2020-09-25 04:52:10+02	2020-09-25 04:52:20+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-09-25 04:52:20+02	2020-09-25 04:52:29+02	devel	py-appdirs	py37	py37-appdirs-1.4.4
2020-09-25 04:51:48+02	2020-09-25 04:52:37+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-25 04:52:29+02	2020-09-25 04:52:38+02	devel	py-asn1crypto	py27	py27-asn1crypto-1.3.0
2020-09-25 04:52:37+02	2020-09-25 04:52:45+02	devel	py-backports	py37	py37-backports-1
2020-09-25 04:52:38+02	2020-09-25 04:52:47+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-09-25 04:52:46+02	2020-09-25 04:52:56+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-09-25 04:52:48+02	2020-09-25 04:52:57+02	devel	py-enum34	py27	py27-enum34-1.1.6
2020-09-25 04:52:57+02	2020-09-25 04:53:09+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-09-25 04:53:09+02	2020-09-25 04:53:21+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-09-25 04:52:57+02	2020-09-25 04:53:23+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-09-25 04:53:22+02	2020-09-25 04:53:33+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-09-25 04:53:23+02	2020-09-25 04:53:34+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-09-25 04:53:34+02	2020-09-25 04:53:44+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-09-25 04:53:35+02	2020-09-25 04:53:45+02	devel	py-wheel	py27	py27-wheel-0.30.0_1
2020-09-25 04:53:45+02	2020-09-25 04:54:05+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-09-25 04:51:47+02	2020-09-25 04:54:30+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-09-25 04:53:48+02	2020-09-25 04:54:41+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-09-25 04:54:05+02	2020-09-25 04:54:52+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-09-25 04:54:53+02	2020-09-25 04:55:02+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-09-25 04:54:31+02	2020-09-25 04:55:12+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-09-25 04:55:03+02	2020-09-25 04:55:15+02	devel	xxhash	\N	xxhash-0.8.0
2020-09-25 04:54:42+02	2020-09-25 04:55:59+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-09-25 04:55:12+02	2020-09-25 04:56:05+02	dns	bind-tools	\N	bind-tools-9.16.6
2020-09-25 04:55:59+02	2020-09-25 04:56:11+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-09-25 04:56:12+02	2020-09-25 04:56:28+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-09-25 04:55:15+02	2020-09-25 04:58:42+02	editors	emacs	full	emacs-27.1,3
2020-09-25 04:58:42+02	2020-09-25 04:59:04+02	graphics	glfw	\N	glfw-3.3.2
2020-09-25 04:59:05+02	2020-09-25 04:59:35+02	graphics	libfreehand	\N	libfreehand-0.1.2_18
2020-09-25 04:56:05+02	2020-09-25 04:59:36+02	graphics	Coin	\N	Coin-4.0.0_1
2020-09-25 04:59:35+02	2020-09-25 05:00:17+02	graphics	movit	\N	movit-1.6.2_2
2020-09-25 04:59:39+02	2020-09-25 05:00:25+02	graphics	opencolorio	\N	opencolorio-1.1.1_1
2020-09-25 05:00:26+02	2020-09-25 05:00:35+02	graphics	potrace	\N	potrace-1.16
2020-09-25 05:00:35+02	2020-09-25 05:00:47+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-09-25 05:00:47+02	2020-09-25 05:01:05+02	graphics	shaderc	\N	shaderc-2020.0
2020-09-25 05:01:06+02	2020-09-25 05:01:19+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-09-25 04:56:30+02	2020-09-25 05:01:40+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-25 05:00:18+02	2020-09-25 05:03:57+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-25 05:01:19+02	2020-09-25 05:04:18+02	java	bootstrap-openjdk11	\N	bootstrap-openjdk11-11.0.5.10.1
2020-09-25 05:04:18+02	2020-09-25 05:05:17+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-25 05:05:17+02	2020-09-25 05:09:51+02	math	py-matplotlib	py37	py37-matplotlib-3.3.1
2020-09-25 05:09:52+02	2020-09-25 05:10:20+02	multimedia	gavl	\N	gavl-1.4.0_4
2020-09-25 05:10:20+02	2020-09-25 05:11:20+02	multimedia	libde265	\N	libde265-1.0.2_5
2020-09-25 05:11:21+02	2020-09-25 05:11:29+02	multimedia	rtmpdump	\N	rtmpdump-2.4.20190330
2020-09-25 05:11:29+02	2020-09-25 05:11:47+02	multimedia	vid.stab	\N	vid.stab-0.98.2
2020-09-25 05:11:47+02	2020-09-25 05:12:19+02	net	gssdp	\N	gssdp-1.2.3
2020-09-25 05:12:20+02	2020-09-25 05:12:48+02	net	libvncserver	\N	libvncserver-0.9.12_2
2020-09-25 05:12:49+02	2020-09-25 05:13:00+02	net	py-ipaddress	py27	py27-ipaddress-1.0.23
2020-09-25 05:13:01+02	2020-09-25 05:14:08+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-09-25 05:14:11+02	2020-09-25 05:14:38+02	print	adobe-cmaps	\N	adobe-cmaps-20051217_4
2020-09-25 05:14:39+02	2020-09-25 05:15:46+02	print	cups-filters	\N	cups-filters-1.28.1_1
2020-09-25 05:15:47+02	2020-09-25 05:20:11+02	print	fontforge	\N	fontforge-20190801_2
2020-09-25 05:20:12+02	2020-09-25 05:20:39+02	print	libmspub01	\N	libmspub01-0.1.4_16
2020-09-25 05:20:40+02	2020-09-25 05:21:06+02	print	libpagemaker	\N	libpagemaker-0.0.4_10
2020-09-25 05:21:07+02	2020-09-25 05:21:18+02	print	t1utils	\N	t1utils-1.32
2020-09-25 05:01:40+02	2020-09-25 05:28:57+02	lang	gcc6-aux	\N	gcc6-aux-20180516_1,1
2020-09-25 05:21:18+02	2020-09-25 05:30:25+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-25 05:30:29+02	2020-09-25 05:30:41+02	security	mbedtls	\N	mbedtls-2.16.8
2020-09-25 05:30:42+02	2020-09-25 05:30:57+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-09-25 05:30:58+02	2020-09-25 05:31:05+02	sysutils	lsof	\N	lsof-4.93.2_12,8
2020-09-25 05:31:06+02	2020-09-25 05:32:48+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-25 05:32:49+02	2020-09-25 05:33:03+02	sysutils	py-psutil	py37	py37-psutil-5.7.2
2020-09-25 05:28:58+02	2020-09-25 05:33:05+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-09-25 05:33:05+02	2020-09-25 05:33:22+02	textproc	libebml	\N	libebml-1.4.0
2020-09-25 05:33:23+02	2020-09-25 05:33:43+02	textproc	mythes	\N	mythes-1.2.4_7
2020-09-25 05:33:44+02	2020-09-25 05:34:01+02	textproc	p5-PDF-API2-XS	\N	p5-PDF-API2-XS-1.001
2020-09-25 05:34:02+02	2020-09-25 05:34:14+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-09-25 05:33:04+02	2020-09-25 05:34:21+02	textproc	UCD	\N	UCD-13.0.0
2020-09-25 05:34:15+02	2020-09-25 05:34:26+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-09-25 05:34:22+02	2020-09-25 05:34:34+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-09-25 05:34:35+02	2020-09-25 05:34:47+02	textproc	rubygem-ronn	\N	rubygem-ronn-0.7.3
2020-09-25 05:34:27+02	2020-09-25 05:35:05+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-09-25 05:34:47+02	2020-09-25 05:37:28+02	www	apache24	\N	apache24-2.4.46
2020-09-25 05:37:29+02	2020-09-25 05:40:19+02	www	node12	\N	node12-12.18.4
2020-09-25 05:40:20+02	2020-09-25 05:40:40+02	www	py-h2	py37	py37-h2-3.2.0
2020-09-25 05:40:41+02	2020-09-25 05:40:54+02	www	py-hyperlink	py37	py37-hyperlink-20.0.1
2020-09-25 05:40:55+02	2020-09-25 05:41:09+02	www	py-priority	py37	py37-priority-1.3.0
2020-09-25 05:41:09+02	2020-09-25 05:41:44+02	www	serf	\N	serf-1.3.9_5
2020-09-25 05:41:44+02	2020-09-25 05:43:30+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-25 05:43:30+02	2020-09-25 05:45:13+02	x11	yelp	\N	yelp-3.38.0
2020-09-25 05:45:14+02	2020-09-25 05:46:12+02	x11-fonts	cantarell-fonts	\N	cantarell-fonts-0.101_1
2020-09-25 05:46:16+02	2020-09-25 05:46:48+02	x11-fonts	p5-Font-TTF	\N	p5-Font-TTF-1.06
2020-09-25 05:46:51+02	2020-09-25 05:48:05+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-25 05:48:06+02	2020-09-25 05:49:09+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-25 05:49:10+02	2020-09-25 05:51:00+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-25 05:51:01+02	2020-09-25 05:54:10+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-09-25 05:54:18+02	2020-09-25 05:56:20+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-25 05:56:21+02	2020-09-25 05:58:04+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-25 05:58:05+02	2020-09-25 05:59:08+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-09-25 05:59:09+02	2020-09-25 06:00:51+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-09-25 06:00:52+02	2020-09-25 06:03:09+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-09-25 06:03:10+02	2020-09-25 06:07:14+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-25 06:07:18+02	2020-09-25 06:09:13+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-25 06:09:15+02	2020-09-25 06:09:59+02	x11-wm	blackbox	\N	blackbox-0.70.1_6
2020-09-25 06:10:02+02	2020-09-25 06:11:46+02	x11-wm	marco	\N	marco-1.22.4
2020-09-25 06:11:48+02	2020-09-25 06:12:15+02	archivers	c-blosc	\N	c-blosc-1.15.0_2
2020-09-25 06:12:15+02	2020-09-25 06:13:25+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-25 06:13:28+02	2020-09-25 06:13:57+02	archivers	p5-Archive-Zip	\N	p5-Archive-Zip-1.68
2020-09-25 06:14:00+02	2020-09-25 06:14:44+02	audio	faad	\N	faad2-2.8.8,1
2020-09-25 06:14:46+02	2020-09-25 06:15:12+02	audio	freealut	\N	freealut-1.1.0_4
2020-09-25 06:15:15+02	2020-09-25 06:15:38+02	audio	libdssialsacompat	\N	libdssialsacompat-1.0.8a_2
2020-09-25 06:15:39+02	2020-09-25 06:16:05+02	audio	liblo	\N	liblo-0.31_1
2020-09-25 05:35:05+02	2020-09-25 06:16:36+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-25 06:16:42+02	2020-09-25 06:17:06+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-09-25 06:17:07+02	2020-09-25 06:17:27+02	audio	taglib	\N	taglib-1.12.b.1_1
2020-09-25 06:17:28+02	2020-09-25 06:17:44+02	audio	twolame	\N	twolame-0.4.0
2020-09-25 06:16:08+02	2020-09-25 06:17:47+02	audio	libopenshot-audio	\N	libopenshot-audio-0.2.0.14
2020-09-25 06:17:45+02	2020-09-25 06:18:08+02	databases	p5-DBI	\N	p5-DBI-1.643
2020-09-25 06:17:47+02	2020-09-25 06:18:26+02	databases	unixODBC	\N	unixODBC-2.3.7
2020-09-25 06:18:29+02	2020-09-25 06:19:34+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-25 06:18:09+02	2020-09-25 06:20:14+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-25 06:20:14+02	2020-09-25 06:21:29+02	devel	binutils	aarch64	aarch64-binutils-2.33.1_3,1
2020-09-25 06:19:35+02	2020-09-25 06:22:29+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-25 06:21:29+02	2020-09-25 06:22:53+02	devel	binutils	mips64	mips64-binutils-2.33.1_3,1
2020-09-25 06:22:30+02	2020-09-25 06:23:46+02	devel	binutils	powerpc64	powerpc64-binutils-2.33.1_3,1
2020-09-25 06:22:54+02	2020-09-25 06:24:02+02	devel	binutils	riscv64	riscv64-binutils-2.33.1_3,1
2020-09-25 06:24:04+02	2020-09-25 06:24:16+02	devel	cl-asdf	\N	cl-asdf-3.3.1.7,2
2020-09-25 06:24:16+02	2020-09-25 06:24:33+02	devel	cscope	\N	cscope-15.9
2020-09-25 06:23:47+02	2020-09-25 06:24:57+02	devel	binutils	sparc64	sparc64-binutils-2.33.1_3,1
2020-09-25 06:24:34+02	2020-09-25 06:25:02+02	devel	dmake	\N	dmake-4.12.3,1
2020-09-25 06:24:57+02	2020-09-25 06:25:19+02	devel	fpc-fcl-registry	\N	fpc-fcl-registry-3.0.4_4
2020-09-25 06:25:06+02	2020-09-25 06:26:15+02	devel	git-lfs	\N	git-lfs-2.12.0
2020-09-25 06:25:19+02	2020-09-25 06:29:16+02	devel	ispc	\N	ispc-1.14.1_2
2020-09-25 06:29:19+02	2020-09-25 06:29:53+02	devel	libfmt	\N	libfmt-7.0.3
2020-09-25 06:26:16+02	2020-09-25 06:30:15+02	devel	kBuild	\N	kBuild-0.1.9998_13
2020-09-25 06:30:15+02	2020-09-25 06:30:31+02	devel	libsigsegv	\N	libsigsegv-2.12
2020-09-25 06:29:54+02	2020-09-25 06:30:49+02	devel	liborcus	\N	liborcus-0.15.4
2020-09-25 06:30:31+02	2020-09-25 06:31:14+02	devel	libslang2	\N	libslang2-2.3.2_2
2020-09-25 06:31:14+02	2020-09-25 06:31:56+02	devel	ocl-icd	\N	ocl-icd-2.2.12
2020-09-25 06:31:57+02	2020-09-25 06:32:17+02	devel	p5-Getopt-ArgvFile	\N	p5-Getopt-ArgvFile-1.11_1
2020-09-25 06:32:18+02	2020-09-25 06:32:29+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-09-25 06:30:50+02	2020-09-25 06:32:30+02	devel	ncurses	\N	ncurses-6.2.20200907
2020-09-25 06:32:30+02	2020-09-25 06:32:45+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-09-25 06:32:30+02	2020-09-25 06:32:46+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-09-25 06:32:45+02	2020-09-25 06:32:57+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-09-25 06:32:46+02	2020-09-25 06:32:59+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-09-25 06:32:59+02	2020-09-25 06:33:12+02	devel	py-pyasn1	py27	py27-pyasn1-0.4.7
2020-09-25 06:32:57+02	2020-09-25 06:33:13+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-09-25 06:33:13+02	2020-09-25 06:33:36+02	devel	pydbus-common	\N	pydbus-common-1.2.16
2020-09-25 06:33:13+02	2020-09-25 06:34:10+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-09-25 06:34:11+02	2020-09-25 06:34:33+02	devel	rebar	\N	rebar-2.6.4,1
2020-09-25 06:34:33+02	2020-09-25 06:35:42+02	devel	rebar3	\N	rebar3-3.14.1,1
2020-09-25 06:35:43+02	2020-09-25 06:35:51+02	devel	robin-map	\N	robin-map-0.6.3
2020-09-25 06:35:52+02	2020-09-25 06:36:04+02	devel	ucpp	\N	ucpp-1.3.2,1
2020-09-25 06:36:05+02	2020-09-25 06:36:29+02	devel	unittest-cpp	\N	unittest-cpp-2.0.0
2020-09-25 06:36:29+02	2020-09-25 06:40:58+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-25 06:40:59+02	2020-09-25 06:43:16+02	graphics	dcmtk	\N	dcmtk-3.6.5_2
2020-09-25 06:43:16+02	2020-09-25 06:43:54+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-09-25 06:43:55+02	2020-09-25 06:44:13+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-09-25 06:44:14+02	2020-09-25 06:44:28+02	graphics	frei0r-plugins-gavl	\N	frei0r-plugins-gavl-1.6.1
2020-09-25 06:33:37+02	2020-09-25 06:44:59+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-25 06:44:59+02	2020-09-25 06:45:47+02	graphics	libetonyek01	\N	libetonyek01-0.1.9_6,1
2020-09-25 06:44:28+02	2020-09-25 06:46:14+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-25 06:45:50+02	2020-09-25 06:46:19+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-09-25 06:46:14+02	2020-09-25 06:46:37+02	graphics	libgxps	\N	libgxps-0.3.1
2020-09-25 06:46:37+02	2020-09-25 06:46:55+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-09-25 06:46:20+02	2020-09-25 06:47:00+02	graphics	libheif	\N	libheif-1.6.2
2020-09-25 06:46:55+02	2020-09-25 06:47:14+02	graphics	libzmf	\N	libzmf-0.0.2_21
2020-09-25 06:47:00+02	2020-09-25 06:47:24+02	graphics	podofo	\N	podofo-0.9.6_1
2020-09-25 06:47:24+02	2020-09-25 06:47:56+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-09-25 06:47:15+02	2020-09-25 06:48:30+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-09-25 06:48:31+02	2020-09-25 06:48:39+02	graphics	qr-code-generator	\N	qr-code-generator-1.6.0
2020-09-25 06:47:56+02	2020-09-25 06:48:54+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-25 06:48:39+02	2020-09-25 06:49:24+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-09-25 06:49:24+02	2020-09-25 06:49:34+02	graphics	squish	\N	squish-1.10_2
2020-09-25 06:48:54+02	2020-09-25 06:49:39+02	graphics	sane-backends	\N	sane-backends-1.0.31_1
2020-09-25 06:49:34+02	2020-09-25 06:49:57+02	graphics	vigra	\N	vigra-1.11.1_12
2020-09-25 06:49:40+02	2020-09-25 06:50:10+02	lang	cython	py38	py38-cython-0.29.15
2020-09-25 06:49:57+02	2020-09-25 06:50:10+02	lang	fpc-rtl-console	\N	fpc-rtl-console-3.0.4_4
2020-09-25 06:50:10+02	2020-09-25 06:51:07+02	lang	fpc-utils	\N	fpc-utils-3.0.4_6
2020-09-25 06:50:10+02	2020-09-25 06:52:21+02	lang	fpc-source	\N	fpc-source-3.0.4_2
2020-09-25 06:52:21+02	2020-09-25 06:52:34+02	lang	p5-Try-Tiny	\N	p5-Try-Tiny-0.30
2020-09-25 06:51:08+02	2020-09-25 06:52:43+02	lang	guile1	\N	guile1-1.8.8
2020-09-25 06:52:43+02	2020-09-25 06:53:02+02	math	cryptominisat	\N	cryptominisat-5.7.1
2020-09-25 06:53:02+02	2020-09-25 06:53:18+02	math	fpc-numlib	\N	fpc-numlib-3.0.4_4
2020-09-25 06:53:18+02	2020-09-25 06:53:53+02	math	gsl	\N	gsl-2.6
2020-09-25 06:53:54+02	2020-09-25 06:55:36+02	math	lp_solve	\N	lp_solve-5.5.2.5
2020-09-25 06:55:36+02	2020-09-25 06:55:43+02	math	minisat	\N	minisat-2.2.1_2
2020-09-25 06:55:44+02	2020-09-25 06:55:51+02	math	openlibm	\N	openlibm-0.6.0_2
2020-09-25 06:55:51+02	2020-09-25 06:56:02+02	misc	box2d	\N	Box2D-2.3.1
2020-09-25 06:56:03+02	2020-09-25 07:02:18+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-25 07:02:18+02	2020-09-25 07:02:46+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-09-25 07:02:46+02	2020-09-25 07:03:19+02	multimedia	libass	\N	libass-0.14.0
2020-09-25 07:03:19+02	2020-09-25 07:03:30+02	multimedia	libdvbpsi	\N	libdvbpsi-1.3.3
2020-09-25 07:03:31+02	2020-09-25 07:03:42+02	multimedia	libmatroska	\N	libmatroska-1.6.2
2020-09-25 07:03:43+02	2020-09-25 07:03:55+02	multimedia	libmpeg2	\N	libmpeg2-0.5.1_6
2020-09-25 07:03:55+02	2020-09-25 07:04:51+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-25 07:04:52+02	2020-09-25 07:05:03+02	net	cppzmq	\N	cppzmq-4.6.0
2020-09-25 06:52:34+02	2020-09-25 07:05:17+02	math	R	\N	R-4.0.2
2020-09-25 07:05:04+02	2020-09-25 07:05:22+02	net	fping	\N	fping-5.0_1
2020-09-25 07:05:23+02	2020-09-25 07:05:44+02	net	gupnp	\N	gupnp-1.1.1_1
2020-09-25 07:05:17+02	2020-09-25 07:06:21+02	net	freerdp	\N	freerdp-2.2.0
2020-09-25 07:06:21+02	2020-09-25 07:06:32+02	net	libmaxminddb	\N	libmaxminddb-1.4.3
2020-09-25 07:06:34+02	2020-09-25 07:06:48+02	net	liveMedia	\N	liveMedia-2020.08.19,2
2020-09-25 07:05:44+02	2020-09-25 07:06:51+02	net	libcmis	\N	libcmis-0.5.2_1
2020-09-25 05:03:58+02	2020-09-25 07:06:57+02	lang	ghc	\N	ghc-8.10.2
2020-09-25 07:06:49+02	2020-09-25 07:07:02+02	net	p5-Net-HTTP	\N	p5-Net-HTTP-6.19
2020-09-25 07:06:51+02	2020-09-25 07:07:04+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-09-25 07:07:02+02	2020-09-25 07:07:11+02	net	py-uritemplate	py37	py37-uritemplate-3.0.0
2020-09-25 07:07:12+02	2020-09-25 07:07:27+02	net	rsync	\N	rsync-3.2.3
2020-09-25 07:07:05+02	2020-09-25 07:07:45+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-25 07:07:46+02	2020-09-25 07:08:29+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-09-25 07:07:27+02	2020-09-25 07:08:34+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-09-25 07:08:29+02	2020-09-25 07:08:38+02	print	freetype	\N	freetype-1.3.1_5
2020-09-25 07:08:38+02	2020-09-25 07:08:57+02	print	libspectre	\N	libspectre-0.2.9
2020-09-25 07:08:58+02	2020-09-25 07:11:10+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-25 07:08:35+02	2020-09-25 07:12:13+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-25 07:06:58+02	2020-09-25 07:13:21+02	devel	hs-cabal-install	\N	hs-cabal-install-3.2.0.0
2020-09-25 07:13:22+02	2020-09-25 07:13:53+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-09-25 07:13:53+02	2020-09-25 07:14:05+02	security	mhash	\N	mhash-0.9.9.9_5
2020-09-25 07:14:06+02	2020-09-25 07:14:19+02	security	p5-Authen-NTLM	\N	p5-Authen-NTLM-1.09_1
2020-09-25 07:14:19+02	2020-09-25 07:14:26+02	security	pam_helper	\N	pam_helper-1.1
2020-09-25 07:14:27+02	2020-09-25 07:14:36+02	security	py-bcrypt	py27	py27-bcrypt-3.1.7_1
2020-09-25 07:14:36+02	2020-09-25 07:14:49+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-09-25 07:14:50+02	2020-09-25 07:15:13+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-09-25 07:15:14+02	2020-09-25 07:15:24+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-09-25 07:15:25+02	2020-09-25 07:15:35+02	security	py-pynacl	py27	py27-pynacl-1.4.0
2020-09-25 07:15:35+02	2020-09-25 07:15:59+02	security	sudo	\N	sudo-1.9.2
2020-09-25 07:15:59+02	2020-09-25 07:16:19+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-09-25 07:11:11+02	2020-09-25 07:16:34+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-25 07:16:19+02	2020-09-25 07:16:35+02	sysutils	hal-info	\N	hal-info-20091130
2020-09-25 07:12:14+02	2020-09-25 07:17:08+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-25 07:16:35+02	2020-09-25 07:17:17+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-25 07:17:09+02	2020-09-25 07:17:21+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.04_1
2020-09-25 07:17:21+02	2020-09-25 07:17:29+02	sysutils	sysctlbyname-improved-kmod	\N	sysctlbyname-improved-kmod-20191124
2020-09-25 07:17:29+02	2020-09-25 07:17:36+02	sysutils	sysctlinfo-kmod	\N	sysctlinfo-kmod-20191005
2020-09-25 07:17:18+02	2020-09-25 07:17:37+02	sysutils	patchelf	\N	patchelf-0.11
2020-09-25 07:17:37+02	2020-09-25 07:17:43+02	sysutils	vpnc-scripts	\N	vpnc-scripts-20200226
2020-09-25 07:17:44+02	2020-09-25 07:18:27+02	textproc	clucene	\N	clucene-2.3.3.4_19
2020-09-25 07:16:35+02	2020-09-25 07:18:50+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-09-25 07:17:37+02	2020-09-25 07:19:22+02	sysutils	tracker	\N	tracker-2.0.4_7
2020-09-25 07:18:51+02	2020-09-25 07:19:23+02	textproc	libabw	\N	libabw-0.1.3
2020-09-25 07:19:22+02	2020-09-25 07:19:30+02	textproc	libdiff	\N	libdiff-0.1.0
2020-09-25 07:19:30+02	2020-09-25 07:19:47+02	textproc	libepubgen	\N	libepubgen-0.1.0_9
2020-09-25 07:19:48+02	2020-09-25 07:19:57+02	textproc	libexttextcat	\N	libexttextcat-3.4.5
2020-09-25 07:18:27+02	2020-09-25 07:20:11+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-25 07:19:57+02	2020-09-25 07:20:19+02	textproc	libmwaw03	\N	libmwaw03-0.3.15
2020-09-25 07:20:12+02	2020-09-25 07:20:21+02	textproc	libnumbertext	\N	libnumbertext-1.0.6
2020-09-25 07:20:20+02	2020-09-25 07:20:30+02	textproc	libodfgen01	\N	libodfgen01-0.1.7_3
2020-09-25 07:20:21+02	2020-09-25 07:20:40+02	textproc	libqxp	\N	libqxp-0.0.0_16
2020-09-25 07:20:30+02	2020-09-25 07:20:49+02	textproc	libstaroffice	\N	libstaroffice-0.0.7
2020-09-25 07:20:41+02	2020-09-25 07:20:55+02	textproc	libwps	\N	libwps-0.4.11
2020-09-25 07:20:55+02	2020-09-25 07:21:08+02	textproc	p5-Pod-Escapes	\N	p5-Pod-Escapes-1.07
2020-09-25 07:20:50+02	2020-09-25 07:21:16+02	textproc	p5-PDF-API2	\N	p5-PDF-API2-2.038
2020-09-25 07:21:09+02	2020-09-25 07:21:20+02	textproc	pugixml	\N	pugixml-1.10
2020-09-25 07:19:24+02	2020-09-25 07:21:26+02	textproc	libe-book	\N	libe-book-0.1.3_18
2020-09-25 07:21:17+02	2020-09-25 07:21:28+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-09-25 07:21:20+02	2020-09-25 07:21:30+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-09-25 07:21:27+02	2020-09-25 07:21:37+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-09-25 07:21:30+02	2020-09-25 07:21:43+02	www	p5-HTTP-Cookies	\N	p5-HTTP-Cookies-6.08
2020-09-25 07:21:37+02	2020-09-25 07:21:49+02	www	p5-HTTP-Daemon	\N	p5-HTTP-Daemon-6.12
2020-09-25 07:21:29+02	2020-09-25 07:21:56+02	textproc	redland	\N	redland-1.0.17_4
2020-09-25 07:21:43+02	2020-09-25 07:21:57+02	www	p5-HTTP-Negotiate	\N	p5-HTTP-Negotiate-6.01_1
2020-09-25 07:21:50+02	2020-09-25 07:22:03+02	www	p5-WWW-RobotRules	\N	p5-WWW-RobotRules-6.02_1
2020-09-25 07:21:56+02	2020-09-25 07:22:06+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-09-25 07:22:07+02	2020-09-25 07:22:16+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-09-25 07:21:57+02	2020-09-25 07:22:31+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-25 07:22:16+02	2020-09-25 07:22:42+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-09-25 07:22:42+02	2020-09-25 07:22:49+02	x11-fonts	crosextrafonts-caladea-ttf	\N	crosextrafonts-caladea-20130214_4
2020-09-25 07:22:03+02	2020-09-25 07:22:53+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-25 07:22:50+02	2020-09-25 07:22:58+02	x11-fonts	crosextrafonts-carlito-ttf	\N	crosextrafonts-carlito-20130920_4
2020-09-25 07:22:58+02	2020-09-25 07:23:05+02	x11-fonts	gentium-basic	\N	GentiumBasic-1102_1
2020-09-25 07:22:54+02	2020-09-25 07:23:08+02	x11-fonts	droid-fonts-ttf	\N	droid-fonts-ttf-20131024_5
2020-09-25 07:23:06+02	2020-09-25 07:23:14+02	x11-fonts	liberation-fonts-ttf	\N	liberation-fonts-ttf-2.1.1,2
2020-09-25 07:23:08+02	2020-09-25 07:23:23+02	x11-fonts	linuxlibertine-g	\N	linuxlibertine-g-20120116_2
2020-09-25 07:23:14+02	2020-09-25 07:23:27+02	x11-fonts	twemoji-color-font-ttf	\N	twemoji-color-font-ttf-12.0.1
2020-09-25 07:22:31+02	2020-09-25 07:23:28+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-25 07:23:27+02	2020-09-25 07:23:58+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-09-25 07:23:59+02	2020-09-25 07:24:15+02	x11-toolkits	Xaw3d	\N	Xaw3d-1.5E_8
2020-09-25 07:23:29+02	2020-09-25 07:24:25+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-09-25 07:23:23+02	2020-09-25 07:24:43+02	x11-servers	xephyr	\N	xephyr-1.20.9,1
2020-09-25 07:24:15+02	2020-09-25 07:25:01+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-25 07:24:25+02	2020-09-25 07:25:14+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-25 07:24:43+02	2020-09-25 07:25:30+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-09-25 07:25:30+02	2020-09-25 07:25:36+02	archivers	arc	\N	arc-5.21p
2020-09-25 07:25:01+02	2020-09-25 07:25:52+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-25 07:25:37+02	2020-09-25 07:26:02+02	archivers	arj	\N	arj-3.10.22_8
2020-09-25 07:26:02+02	2020-09-25 07:26:09+02	archivers	libunrar5	\N	libunrar5-5.9.4
2020-09-25 07:26:10+02	2020-09-25 07:26:36+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-09-25 07:26:36+02	2020-09-25 07:26:44+02	archivers	unzoo	\N	unzoo-4.4_2
2020-09-25 07:26:45+02	2020-09-25 07:27:05+02	audio	dssi	\N	dssi-1.1.1_8
2020-09-25 07:27:06+02	2020-09-25 07:27:29+02	audio	faudio	\N	FAudio-20.09
2020-09-25 07:25:53+02	2020-09-25 07:28:03+02	archivers	engrampa	\N	engrampa-1.24.1
2020-09-25 07:27:30+02	2020-09-25 07:28:28+02	audio	fdk-aac	\N	fdk-aac-2.0.1
2020-09-25 07:28:29+02	2020-09-25 07:28:37+02	audio	sndio	\N	sndio-1.7.0
2020-09-25 07:28:37+02	2020-09-25 07:28:58+02	audio	swhplugins	\N	swhplugins-0.4.15_7
2020-09-25 07:28:59+02	2020-09-25 07:29:16+02	audio	wavpack	\N	wavpack-5.3.0
2020-09-25 07:28:04+02	2020-09-25 07:29:19+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-25 07:29:17+02	2020-09-25 07:29:24+02	databases	lmdb	\N	lmdb-0.9.24_2,1
2020-09-25 07:29:25+02	2020-09-25 07:29:36+02	databases	pgtcl	\N	pgtcl-2.1.1_2
2020-09-25 07:29:20+02	2020-09-25 07:29:40+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.14.2
2020-09-25 07:29:36+02	2020-09-25 07:29:48+02	databases	py-apsw	py37	py37-apsw-3.33.0
2020-09-25 07:25:14+02	2020-09-25 07:29:59+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-25 07:29:40+02	2020-09-25 07:30:23+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-09-25 07:29:48+02	2020-09-25 07:30:42+02	deskutils	caja-extensions	\N	caja-extensions-1.22.1
2020-09-25 07:30:24+02	2020-09-25 07:31:09+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-25 07:30:43+02	2020-09-25 07:31:34+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-25 07:31:35+02	2020-09-25 07:32:25+02	devel	adacurses	\N	adacurses-20180127
2020-09-25 07:32:26+02	2020-09-25 07:32:32+02	devel	cweb	\N	cweb-3.64a.a
2020-09-25 07:30:00+02	2020-09-25 07:32:40+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-25 07:32:33+02	2020-09-25 07:32:46+02	devel	erlang-cl	\N	erlang-cl-1.2.4
2020-09-25 07:32:40+02	2020-09-25 07:32:57+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-09-25 07:32:47+02	2020-09-25 07:33:03+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-09-25 07:32:58+02	2020-09-25 07:33:18+02	devel	fpc-fv	\N	fpc-fv-3.0.4_4
2020-09-25 07:33:03+02	2020-09-25 07:33:23+02	devel	fpc-gdbint	\N	fpc-gdbint-3.0.4_4
2020-09-25 07:33:23+02	2020-09-25 07:33:47+02	devel	google-perftools	\N	google-perftools-2.8
2020-09-25 07:33:18+02	2020-09-25 07:33:49+02	devel	fstrm	\N	fstrm-0.6.0
2020-09-25 07:33:49+02	2020-09-25 07:34:23+02	devel	gsoap	\N	gsoap-2.8.106
2020-09-25 07:31:10+02	2020-09-25 07:34:34+02	devel	aarch64-gcc	\N	aarch64-gcc-6.4.0_8
2020-09-25 07:34:34+02	2020-09-25 07:34:52+02	devel	libarea	\N	libarea-20160313_13
2020-09-25 07:33:48+02	2020-09-25 07:34:55+02	devel	gradle5	\N	gradle5-5.0
2020-09-25 07:34:56+02	2020-09-25 07:35:03+02	devel	libsysctlmibinfo2	\N	libsysctlmibinfo2-2.0.0_1
2020-09-25 07:34:23+02	2020-09-25 07:35:04+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-09-25 07:35:03+02	2020-09-25 07:35:10+02	devel	libublio	\N	libublio-20070103_2
2020-09-25 07:34:52+02	2020-09-25 07:35:19+02	devel	libstatgrab	\N	libstatgrab-0.92
2020-09-25 07:35:04+02	2020-09-25 07:35:33+02	devel	mercurial	py37	mercurial-5.5_1
2020-09-25 07:35:19+02	2020-09-25 07:35:39+02	devel	ode	\N	ode-0.13_4,1
2020-09-25 07:35:34+02	2020-09-25 07:35:47+02	devel	p5-Term-ReadKey	\N	p5-Term-ReadKey-2.38_1
2020-09-25 07:35:10+02	2020-09-25 07:38:26+02	devel	mips64-gcc	\N	mips64-gcc-6.4.0_8
2020-09-25 07:35:48+02	2020-09-25 07:38:34+02	devel	protobuf-c	\N	protobuf-c-1.3.2_6
2020-09-25 07:38:26+02	2020-09-25 07:38:37+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-09-25 07:38:34+02	2020-09-25 07:38:45+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-09-25 07:38:37+02	2020-09-25 07:38:53+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-09-25 07:35:40+02	2020-09-25 07:38:54+02	devel	powerpc64-gcc	\N	powerpc64-gcc-6.4.0_8
2020-09-25 07:38:46+02	2020-09-25 07:38:56+02	devel	py-funcy	py37	py37-funcy-1.12
2020-09-25 07:38:55+02	2020-09-25 07:39:04+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-09-25 07:38:56+02	2020-09-25 07:39:05+02	devel	py-iniparse	py37	py37-iniparse-0.5
2020-09-25 07:39:05+02	2020-09-25 07:39:15+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-09-25 07:39:06+02	2020-09-25 07:39:16+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-09-25 07:39:15+02	2020-09-25 07:39:26+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-09-25 07:38:53+02	2020-09-25 07:39:29+02	devel	py-game	py37	py37-game-1.9.6_1
2020-09-25 07:39:26+02	2020-09-25 07:39:36+02	devel	py-rq	py37	py37-rq-1.3.0
2020-09-25 07:39:29+02	2020-09-25 07:39:39+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-09-25 07:39:36+02	2020-09-25 07:39:45+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-09-25 07:39:40+02	2020-09-25 07:39:49+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-09-25 07:39:45+02	2020-09-25 07:39:54+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-09-25 07:39:16+02	2020-09-25 07:40:04+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-09-25 07:40:05+02	2020-09-25 07:40:22+02	devel	rlwrap	\N	rlwrap-0.42_2
2020-09-25 07:40:23+02	2020-09-25 07:40:34+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-09-25 07:40:35+02	2020-09-25 07:40:47+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-09-25 07:40:47+02	2020-09-25 07:40:57+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-09-25 07:39:49+02	2020-09-25 07:41:29+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-25 07:41:29+02	2020-09-25 07:42:02+02	devel	tex-synctex	\N	tex-synctex-1.17.0_1
2020-09-25 07:42:03+02	2020-09-25 07:42:18+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-09-25 07:39:54+02	2020-09-25 07:42:44+02	devel	riscv64-gcc	\N	riscv64-gcc-8.3.0
2020-09-25 07:40:57+02	2020-09-25 07:43:39+02	devel	sparc64-gcc	\N	sparc64-gcc-6.4.0_8
2020-09-25 07:43:40+02	2020-09-25 07:46:37+02	editors	pluma	\N	pluma-1.24.1
2020-09-25 07:46:38+02	2020-09-25 07:46:51+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-09-25 07:46:52+02	2020-09-25 07:47:40+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-25 07:47:41+02	2020-09-25 07:49:58+02	french	med	\N	fr-med-4.0.0
2020-09-25 07:49:59+02	2020-09-25 07:50:14+02	ftp	libfilezilla	\N	libfilezilla-0.15.1_3
2020-09-25 07:50:15+02	2020-09-25 08:01:37+02	games	alienarena-data	\N	alienarena-data-2013.766
2020-09-25 08:01:37+02	2020-09-25 08:02:24+02	games	gnuchess	\N	gnuchess-6.2.7
2020-09-25 07:42:19+02	2020-09-25 08:02:24+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-25 08:02:25+02	2020-09-25 08:02:48+02	graphics	alembic	\N	alembic-1.7.15
2020-09-25 08:02:24+02	2020-09-25 08:03:16+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-09-25 07:42:45+02	2020-09-25 08:03:43+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-09-25 08:03:16+02	2020-09-25 08:03:53+02	graphics	embree	\N	embree-3.10.0_1
2020-09-25 08:02:48+02	2020-09-25 08:04:47+02	graphics	atril	\N	atril-1.22.3_9
2020-09-25 08:04:48+02	2020-09-25 08:05:05+02	graphics	gle	\N	gle-3.1.0_7
2020-09-25 08:05:06+02	2020-09-25 08:05:23+02	graphics	gltt	\N	gltt-2.5.2_14
2020-09-25 08:03:45+02	2020-09-25 08:05:23+02	graphics	eom	\N	eom-1.24.1
2020-09-25 08:03:54+02	2020-09-25 08:05:28+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-25 08:05:23+02	2020-09-25 08:05:29+02	graphics	jgraph	\N	jgraph-8.3_1
2020-09-25 08:05:28+02	2020-09-25 08:05:38+02	graphics	libart_lgpl	\N	libart_lgpl-2.3.21_3,1
2020-09-25 08:05:23+02	2020-09-25 08:06:06+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-25 08:05:38+02	2020-09-25 08:06:45+02	graphics	opencollada	\N	opencollada-1.6.68_2
2020-09-25 08:05:30+02	2020-09-25 08:07:13+02	graphics	oidn	\N	oidn-1.2.3
2020-09-25 08:06:07+02	2020-09-25 08:07:22+02	graphics	openimageio	\N	openimageio-2.1.15.0_6
2020-09-25 08:07:13+02	2020-09-25 08:08:26+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-25 08:08:26+02	2020-09-25 08:08:54+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-09-25 08:06:46+02	2020-09-25 08:09:09+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-09-25 08:08:55+02	2020-09-25 08:09:20+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-09-25 08:07:23+02	2020-09-25 08:09:21+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-09-25 08:09:21+02	2020-09-25 08:09:47+02	lang	cython	py27	py27-cython-0.29.15
2020-09-25 08:09:09+02	2020-09-25 08:28:56+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-25 08:09:48+02	2020-09-25 08:30:27+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-25 08:09:21+02	2020-09-25 08:32:19+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-09-25 08:32:20+02	2020-09-25 08:32:31+02	lang	lua51	\N	lua51-5.1.5_9
2020-09-25 08:32:32+02	2020-09-25 08:32:43+02	lang	luajit	\N	luajit-2.0.5_4
2020-09-25 08:32:44+02	2020-09-25 08:32:55+02	lang	python3	\N	python3-3_3
2020-09-25 08:32:55+02	2020-09-25 08:41:07+02	lang	sbcl	\N	sbcl-2.0.8,1
2020-09-25 08:41:07+02	2020-09-25 08:43:04+02	math	mate-calc	\N	mate-calc-1.24.1
2020-09-25 08:43:04+02	2020-09-25 08:44:10+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-09-25 08:44:10+02	2020-09-25 08:45:20+02	math	py-numpy	py38	py38-numpy-1.16.6,1
2020-09-25 08:45:20+02	2020-09-25 08:46:05+02	math	stp	\N	stp-2.3.3_6,1
2020-09-25 08:46:06+02	2020-09-25 08:48:19+02	math	z3	\N	z3-4.8.8
2020-09-25 08:48:20+02	2020-09-25 08:48:30+02	misc	chmlib	\N	chmlib-0.40_1
2020-09-25 08:48:31+02	2020-09-25 08:48:37+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200918
2020-09-25 08:30:28+02	2020-09-25 08:51:20+02	lang	julia10	\N	julia10-1.0.4_5
2020-09-25 08:51:21+02	2020-09-25 08:51:46+02	misc	ini_file_manager	\N	ini_file_manager-03_3
2020-09-25 08:48:38+02	2020-09-25 08:54:07+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-09-25 08:54:08+02	2020-09-25 08:54:24+02	misc	xosd	\N	xosd-2.2.14_3
2020-09-25 08:54:24+02	2020-09-25 08:54:44+02	multimedia	gmmlib	\N	gmmlib-20.3.2
2020-09-25 08:54:44+02	2020-09-25 08:54:59+02	multimedia	libmtp	\N	libmtp-1.1.17
2020-09-25 08:55:00+02	2020-09-25 08:56:38+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-09-25 08:56:39+02	2020-09-25 08:57:15+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-09-25 08:51:46+02	2020-09-25 08:57:30+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-09-25 08:57:16+02	2020-09-25 08:59:37+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-09-25 08:59:38+02	2020-09-25 09:00:51+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-25 09:00:51+02	2020-09-25 09:01:40+02	net	ndpi	\N	ndpi-3.2.d20200721,1
2020-09-25 09:01:42+02	2020-09-25 09:01:58+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-09-25 09:01:58+02	2020-09-25 09:02:12+02	net	py-pyzmq	py37	py37-pyzmq-19.0.2
2020-09-25 09:02:12+02	2020-09-25 09:02:55+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-25 08:57:32+02	2020-09-25 09:03:24+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-25 09:02:55+02	2020-09-25 09:03:46+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-25 09:03:25+02	2020-09-25 09:04:14+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-25 09:03:47+02	2020-09-25 09:04:43+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-25 09:04:15+02	2020-09-25 09:05:00+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-25 09:04:44+02	2020-09-25 09:05:22+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-25 08:28:56+02	2020-09-25 09:05:48+02	lang	gcc48	\N	gcc48-4.8.5_11
2020-09-25 09:05:01+02	2020-09-25 09:05:49+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-25 09:05:48+02	2020-09-25 09:06:11+02	net	vde2	\N	vde2-2.3.2_4
2020-09-25 09:05:49+02	2020-09-25 09:06:12+02	net	wireguard-go	\N	wireguard-go-0.0.20200320
2020-09-25 09:05:23+02	2020-09-25 09:06:34+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-25 09:06:11+02	2020-09-25 09:06:54+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-09-25 09:06:35+02	2020-09-25 09:06:55+02	net-mgmt	libsmi	\N	libsmi-0.4.8_1
2020-09-25 09:06:55+02	2020-09-25 09:07:06+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-09-25 09:07:07+02	2020-09-25 09:07:14+02	ports-mgmt	dialog4ports	\N	dialog4ports-0.1.6
2020-09-25 09:06:12+02	2020-09-25 09:07:15+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-09-25 09:06:55+02	2020-09-25 09:07:38+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-09-25 09:07:14+02	2020-09-25 09:08:09+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-25 09:07:38+02	2020-09-25 09:13:33+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-25 09:08:10+02	2020-09-25 09:14:00+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-25 09:13:34+02	2020-09-25 09:15:42+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-25 09:07:16+02	2020-09-25 09:18:56+02	print	lilypond	\N	lilypond-2.20.0
2020-09-25 09:18:56+02	2020-09-25 09:19:17+02	print	transfig	\N	transfig-3.2.5e_4
2020-09-25 09:19:18+02	2020-09-25 09:19:33+02	security	libntlm	\N	libntlm-1.6
2020-09-25 09:19:33+02	2020-09-25 09:19:39+02	security	libscrypt	\N	libscrypt-1.21
2020-09-25 09:19:40+02	2020-09-25 09:20:29+02	security	nmap	\N	nmap-7.80
2020-09-25 09:14:01+02	2020-09-25 09:20:38+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-25 09:20:30+02	2020-09-25 09:20:44+02	security	openconnect	\N	openconnect-8.10
2020-09-25 09:20:45+02	2020-09-25 09:20:51+02	security	pkesh	\N	pkesh-1.0
2020-09-25 09:20:39+02	2020-09-25 09:20:52+02	security	p5-Digest-SHA1	\N	p5-Digest-SHA1-2.13_1
2020-09-25 09:20:52+02	2020-09-25 09:21:02+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-09-25 09:20:51+02	2020-09-25 09:21:02+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-09-25 09:21:02+02	2020-09-25 09:21:14+02	security	py-stem	py37	py37-stem-1.8.0
2020-09-25 09:21:14+02	2020-09-25 09:22:07+02	sysutils	intel-pcm	\N	intel-pcm-202008_1
2020-09-25 09:21:02+02	2020-09-25 09:22:19+02	sysutils	coreutils	\N	coreutils-8.32
2020-09-25 09:22:07+02	2020-09-25 09:22:29+02	sysutils	libcpuid	\N	libcpuid-0.5.0
2020-09-25 09:22:19+02	2020-09-25 09:22:58+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-25 09:22:30+02	2020-09-25 09:23:45+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-09-25 09:23:46+02	2020-09-25 09:23:54+02	sysutils	smart	\N	smart-0.0.11
2020-09-25 09:23:54+02	2020-09-25 09:24:06+02	sysutils	smartmontools	\N	smartmontools-7.1_2
2020-09-25 09:24:07+02	2020-09-25 09:24:23+02	sysutils	tarsnap	\N	tarsnap-1.0.39
2020-09-25 09:22:59+02	2020-09-25 09:24:25+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-09-25 09:24:24+02	2020-09-25 09:24:44+02	textproc	diffutils	\N	diffutils-3.7
2020-09-25 09:24:45+02	2020-09-25 09:25:37+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-25 09:24:25+02	2020-09-25 09:25:48+02	textproc	fop	\N	fop-2.3
2020-09-25 09:25:49+02	2020-09-25 09:26:16+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-09-25 09:26:17+02	2020-09-25 09:26:25+02	textproc	lowdown	\N	lowdown-0.7.4
2020-09-25 09:26:25+02	2020-09-25 09:26:38+02	textproc	p5-pod2pdf	\N	p5-pod2pdf-0.42_1
2020-09-25 09:26:39+02	2020-09-25 09:27:22+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-09-25 09:27:23+02	2020-09-25 09:27:57+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-25 09:27:57+02	2020-09-25 09:28:08+02	textproc	py-regex	py37	py37-regex-2020.4.4
2020-09-25 09:28:08+02	2020-09-25 09:28:18+02	textproc	py-scour	py37	py37-scour-0.38.1
2020-09-25 09:28:18+02	2020-09-25 09:28:29+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-09-25 09:28:29+02	2020-09-25 09:33:54+02	textproc	ripgrep	\N	ripgrep-12.1.1_5
2020-09-25 09:15:43+02	2020-09-25 09:34:28+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-25 09:33:55+02	2020-09-25 09:34:33+02	textproc	xerces-c3	\N	xerces-c3-3.2.3
2020-09-25 09:34:33+02	2020-09-25 09:34:55+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-09-25 09:34:55+02	2020-09-25 09:35:11+02	www	p5-libwww	\N	p5-libwww-6.47
2020-09-25 09:34:29+02	2020-09-25 09:35:15+02	www	mattermost-webapp	\N	mattermost-webapp-5.26.1
2020-09-25 09:35:15+02	2020-09-25 09:35:25+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-09-25 09:35:11+02	2020-09-25 09:35:32+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-09-25 09:35:26+02	2020-09-25 09:35:39+02	www	py-cssselect	py37	py37-cssselect-1.0.3_1
2020-09-25 09:35:33+02	2020-09-25 09:35:43+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-09-25 09:35:39+02	2020-09-25 09:35:50+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-09-25 09:35:43+02	2020-09-25 09:35:59+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-09-25 09:35:50+02	2020-09-25 09:36:04+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-09-25 09:35:59+02	2020-09-25 09:36:51+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-25 09:36:51+02	2020-09-25 09:37:01+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-09-25 09:36:05+02	2020-09-25 09:37:04+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-25 09:37:05+02	2020-09-25 09:38:04+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-25 09:38:05+02	2020-09-25 09:39:15+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-25 09:39:15+02	2020-09-25 09:40:58+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-25 09:37:02+02	2020-09-25 09:41:33+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-25 09:41:34+02	2020-09-25 09:41:41+02	x11-fonts	bitstream-vera	\N	bitstream-vera-1.10_8
2020-09-25 09:40:59+02	2020-09-25 09:42:30+02	x11-fm	nautilus	\N	nautilus-3.28.1_4
2020-09-25 09:42:31+02	2020-09-25 09:42:51+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-09-25 09:42:51+02	2020-09-25 09:43:41+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-25 09:43:41+02	2020-09-25 09:44:13+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-25 09:44:13+02	2020-09-25 09:45:17+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-25 09:45:17+02	2020-09-25 09:45:27+02	archivers	pbzip2	\N	pbzip2-1.1.13
2020-09-25 09:45:27+02	2020-09-25 09:45:34+02	archivers	unrar	\N	unrar-5.91,6
2020-09-25 09:45:34+02	2020-09-25 09:56:38+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-25 09:56:39+02	2020-09-25 09:56:46+02	astro	xearth	\N	xearth-1.2
2020-09-25 09:56:46+02	2020-09-25 09:57:15+02	astro	xephem	\N	xephem-3.7.7_5
2020-09-25 09:41:41+02	2020-09-25 09:59:17+02	x11-fonts	nerd-fonts	\N	nerd-fonts-2.1.0
2020-09-25 09:59:17+02	2020-09-25 09:59:35+02	audio	xmix	\N	xmix-2.1_3
2020-09-25 09:59:35+02	2020-09-25 10:00:33+02	benchmarks	fio	\N	fio-3.22
2020-09-25 09:57:16+02	2020-09-25 10:00:34+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-25 10:00:34+02	2020-09-25 10:00:44+02	benchmarks	iperf3	\N	iperf3-3.9
2020-09-25 10:00:45+02	2020-09-25 10:05:38+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-25 10:05:39+02	2020-09-25 10:08:08+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-25 09:25:38+02	2020-09-25 10:27:03+02	textproc	hs-pandoc	\N	hs-pandoc-2.10
2020-09-25 10:27:03+02	2020-09-25 10:29:04+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-25 10:29:05+02	2020-09-25 10:30:50+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-25 10:30:51+02	2020-09-25 10:31:03+02	comms	kermit	\N	kermit-9.0.304.24
2020-09-25 10:31:03+02	2020-09-25 10:31:11+02	converters	dosunix	\N	dosunix-1.0.14
2020-09-25 10:31:11+02	2020-09-25 10:31:33+02	converters	recode	\N	recode-3.6_15
2020-09-25 10:31:34+02	2020-09-25 10:31:40+02	converters	unix2dos	\N	unix2dos-1.3
2020-09-25 10:31:40+02	2020-09-25 10:31:50+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-09-25 10:00:35+02	2020-09-25 10:34:24+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-25 10:34:25+02	2020-09-25 10:34:41+02	databases	pgtop	\N	pgtop-0.05_5
2020-09-25 10:31:50+02	2020-09-25 10:35:21+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-25 10:35:21+02	2020-09-25 10:36:04+02	databases	postgresql12-server	\N	postgresql12-server-12.4
2020-09-25 10:36:04+02	2020-09-25 10:36:13+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.6
2020-09-25 10:36:14+02	2020-09-25 10:36:23+02	databases	redis	\N	redis-5.0.9
2020-09-25 10:34:42+02	2020-09-25 10:37:03+02	databases	postgresql12-contrib	\N	postgresql12-contrib-12.4
2020-09-25 10:37:04+02	2020-09-25 10:37:45+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-25 10:37:46+02	2020-09-25 10:37:55+02	deskutils	tpb	\N	tpb-0.6.4_13
2020-09-25 10:36:23+02	2020-09-25 10:38:08+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-25 10:37:56+02	2020-09-25 10:38:09+02	deskutils	xpostitPlus	\N	XPostitPlus-2.3_3
2020-09-25 10:38:09+02	2020-09-25 10:38:18+02	devel	aarch64-xtoolchain-gcc	\N	aarch64-xtoolchain-gcc-0.4_1
2020-09-25 10:38:19+02	2020-09-25 10:38:56+02	devel	bzr	py27	bzr-2.7.0.6622
2020-09-25 10:38:56+02	2020-09-25 10:39:21+02	devel	cdecl	\N	cdecl-8.0
2020-09-25 10:39:22+02	2020-09-25 10:50:46+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-25 10:50:47+02	2020-09-25 10:51:26+02	devel	cvs+ipv6	\N	cvs+ipv6-1.20120905_3
2020-09-25 10:51:27+02	2020-09-25 10:52:48+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-25 10:38:09+02	2020-09-25 11:01:52+02	devel	RStudio	\N	RStudio-1.3.1093
2020-09-25 10:52:48+02	2020-09-25 11:16:00+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-25 10:08:09+02	2020-09-25 11:33:45+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-25 11:01:53+02	2020-09-25 11:47:37+02	devel	ghidra	\N	ghidra-9.1
2020-09-25 11:47:38+02	2020-09-25 11:47:46+02	devel	mips64-xtoolchain-gcc	\N	mips64-xtoolchain-gcc-0.4_1
2020-09-25 11:47:46+02	2020-09-25 11:47:55+02	devel	powerpc64-xtoolchain-gcc	\N	powerpc64-xtoolchain-gcc-0.4_1
2020-09-25 11:47:55+02	2020-09-25 11:48:06+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-09-25 11:48:07+02	2020-09-25 11:48:15+02	devel	riscv64-xtoolchain-gcc	\N	riscv64-xtoolchain-gcc-0.4_1
2020-09-25 11:48:15+02	2020-09-25 11:48:23+02	devel	sparc64-xtoolchain-gcc	\N	sparc64-xtoolchain-gcc-0.4_1
2020-09-25 11:48:24+02	2020-09-25 11:49:20+02	devel	subversion	\N	subversion-1.14.0
2020-09-25 11:49:21+02	2020-09-25 11:49:37+02	devel	tig	\N	tig-2.5.1
2020-09-25 11:49:38+02	2020-09-25 11:50:14+02	devel	tortoisehg	py37	tortoisehg-5.5.1
2020-09-25 11:33:46+02	2020-09-25 11:50:21+02	devel	llvm10	\N	llvm10-10.0.1_1
2020-09-25 11:50:15+02	2020-09-25 11:51:08+02	dns	bind916	\N	bind916-9.16.6_1
2020-09-25 11:51:08+02	2020-09-25 11:51:42+02	editors	fpc-ide	\N	fpc-ide-3.0.4_4
2020-09-25 11:51:43+02	2020-09-25 11:53:59+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-25 11:53:59+02	2020-09-25 11:55:59+02	editors	vim	\N	vim-8.2.1558
2020-09-25 11:16:03+02	2020-09-25 12:04:10+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200811
2020-09-25 11:50:22+02	2020-09-25 12:04:20+02	editors	codelite	\N	codelite-12.0_4
2020-09-25 11:56:00+02	2020-09-25 12:05:08+02	emulators	qemu	\N	qemu-4.2.1
2020-09-25 12:05:08+02	2020-09-25 12:05:51+02	emulators	wine-gecko	\N	wine-gecko-2.47.1
2020-09-25 12:05:51+02	2020-09-25 12:07:04+02	emulators	wine-mono	\N	wine-mono-4.9.4_1
2020-09-25 12:07:04+02	2020-09-25 12:07:44+02	emulators	x48	\N	x48-0.6.4_1
2020-09-25 12:07:45+02	2020-09-25 12:25:07+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-25 12:25:08+02	2020-09-25 12:26:18+02	ftp	wget	\N	wget-1.20.3
2020-09-25 12:26:19+02	2020-09-25 12:26:29+02	games	KnightCap	\N	knightcap-3.6_8
2020-09-25 12:26:30+02	2020-09-25 12:26:41+02	games	affenspiel	\N	affenspiel-1.0_2
2020-09-25 12:26:42+02	2020-09-25 12:26:57+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-09-25 12:26:58+02	2020-09-25 12:29:28+02	games	alienarena	\N	alienarena-2013.766_9
2020-09-25 12:29:28+02	2020-09-25 12:29:52+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-09-25 12:29:52+02	2020-09-25 12:31:15+02	games	battletanks	\N	btanks-0.9.8083_9
2020-09-25 12:31:15+02	2020-09-25 12:32:30+02	games	bsdgames	\N	bsdgames-0.75,2
2020-09-25 12:32:31+02	2020-09-25 12:32:46+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-09-25 12:32:47+02	2020-09-25 12:33:33+02	games	monsterz	\N	monsterz-0.7.1_15
2020-09-25 12:33:34+02	2020-09-25 12:34:13+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-09-25 12:34:14+02	2020-09-25 12:34:54+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-09-25 12:34:55+02	2020-09-25 12:35:11+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-09-25 12:35:12+02	2020-09-25 12:35:44+02	games	stockfish	\N	stockfish-11
2020-09-25 12:35:45+02	2020-09-25 12:36:41+02	games	xblast	\N	xblast-2.10.4_18
2020-09-25 12:36:42+02	2020-09-25 12:37:47+02	games	xboard	\N	xboard-4.9.1_1
2020-09-25 12:37:48+02	2020-09-25 12:38:41+02	games	xcowsay	\N	xcowsay-1.5
2020-09-25 12:38:42+02	2020-09-25 12:45:39+02	graphics	blender	\N	blender-2.90.0
2020-09-25 12:04:21+02	2020-09-25 12:46:13+02	emulators	wine	\N	wine-5.0.2,1
2020-09-25 12:46:14+02	2020-09-25 12:47:32+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-25 12:45:40+02	2020-09-25 12:47:44+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-25 12:47:33+02	2020-09-25 12:52:20+02	graphics	inkscape	\N	inkscape-1.0_5
2020-09-25 12:52:21+02	2020-09-25 12:52:42+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-09-25 12:52:45+02	2020-09-25 12:57:56+02	graphics	wings	\N	wings-2.2.5
2020-09-25 12:57:57+02	2020-09-25 12:59:03+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-25 12:47:44+02	2020-09-25 12:59:04+02	graphics	krita	\N	krita-4.3.0_5
2020-09-25 12:59:03+02	2020-09-25 13:00:09+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-09-25 13:00:10+02	2020-09-25 13:02:15+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-25 12:04:11+02	2020-09-25 13:15:37+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_1
2020-09-25 12:59:05+02	2020-09-25 13:22:32+02	java	openjdk12	\N	openjdk12-12.0.2+10.4_1
2020-09-25 13:22:33+02	2020-09-25 13:22:48+02	lang	gcc	\N	gcc-9_4
2020-09-25 13:02:17+02	2020-09-25 13:36:43+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-25 13:37:46+02	2020-09-25 13:38:08+02	lang	mmix	\N	mmix-20160804
2020-09-25 13:15:40+02	2020-09-25 13:47:01+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.4_1
2020-09-25 13:47:02+02	2020-09-25 13:47:13+02	lang	python	\N	python-3.7_3,2
2020-09-25 13:47:14+02	2020-09-25 13:47:21+02	lang	python-mode.el	\N	python-mode.el-1.0_2,1
2020-09-25 13:47:22+02	2020-09-25 13:48:16+02	mail	alpine	\N	alpine-2.23.2
2020-09-25 13:48:17+02	2020-09-25 13:48:29+02	mail	bbmail	\N	bbmail-0.9.3
2020-09-25 13:48:30+02	2020-09-25 13:54:55+02	math	maxima	\N	maxima-5.44.0_4
2020-09-25 13:54:55+02	2020-09-25 13:55:04+02	math	sc	\N	sc-7.16_3
2020-09-25 13:55:05+02	2020-09-25 13:55:49+02	misc	gnu-watch	\N	gnu-watch-3.3.16
2020-09-25 13:55:49+02	2020-09-25 13:55:59+02	misc	mbuffer	\N	mbuffer-20200505
2020-09-25 13:56:00+02	2020-09-25 13:57:14+02	misc	mc	\N	mc-nox11-4.8.24
2020-09-25 13:57:14+02	2020-09-25 13:58:06+02	misc	pinfo	\N	pinfo-0.6.10_5
2020-09-25 13:58:06+02	2020-09-25 13:59:24+02	multimedia	celluloid	\N	celluloid-0.20
2020-09-25 13:59:25+02	2020-09-25 13:59:47+02	multimedia	libva-glx	\N	libva-glx-2.9.0
2020-09-25 13:59:48+02	2020-09-25 14:00:18+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-09-25 14:00:19+02	2020-09-25 14:04:22+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p8
2020-09-25 14:04:23+02	2020-09-25 14:04:47+02	multimedia	libva-utils	\N	libva-utils-2.9.0
2020-09-25 14:04:48+02	2020-09-25 14:05:07+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-09-25 14:05:08+02	2020-09-25 14:07:22+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-09-25 14:07:23+02	2020-09-25 14:09:56+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-09-25 14:09:56+02	2020-09-25 14:12:56+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-25 14:12:57+02	2020-09-25 14:13:26+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-09-25 14:13:26+02	2020-09-25 14:13:51+02	net	mosh	\N	mosh-1.3.2_14
2020-09-25 13:38:09+02	2020-09-25 14:13:50+02	lang	mono	\N	mono-5.10.1.57_3
2020-09-25 14:13:52+02	2020-09-25 14:14:24+02	net	rdesktop	\N	rdesktop-1.9.0
2020-09-25 14:13:52+02	2020-09-25 14:15:53+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-09-25 14:14:24+02	2020-09-25 14:15:59+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-25 14:15:53+02	2020-09-25 14:16:03+02	net	svnup	\N	svnup-1.08
2020-09-25 14:16:00+02	2020-09-25 14:16:14+02	net	torsocks	\N	torsocks-2.3.0_1
2020-09-25 14:16:15+02	2020-09-25 14:16:37+02	net	vnstat	\N	vnstat-2.6_1
2020-09-25 14:16:37+02	2020-09-25 14:16:45+02	net	wireguard	\N	wireguard-1.0.20200827
2020-09-25 14:16:04+02	2020-09-25 14:17:13+02	net	v2ray	\N	v2ray-4.25.0
2020-09-25 14:17:14+02	2020-09-25 14:18:03+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-25 14:18:03+02	2020-09-25 14:19:54+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-09-25 14:19:55+02	2020-09-25 14:20:50+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-09-25 14:16:46+02	2020-09-25 14:21:24+02	net	wireshark	\N	wireshark-3.2.6
2020-09-25 14:20:51+02	2020-09-25 14:21:40+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-09-25 14:21:25+02	2020-09-25 14:22:14+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-09-25 14:21:40+02	2020-09-25 14:22:32+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-09-25 14:22:14+02	2020-09-25 14:23:02+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-09-25 14:22:32+02	2020-09-25 14:23:22+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-09-25 14:23:03+02	2020-09-25 14:23:53+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-09-25 14:23:23+02	2020-09-25 14:24:44+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-09-25 14:23:53+02	2020-09-25 14:24:52+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-09-25 14:24:52+02	2020-09-25 14:25:03+02	net-mgmt	iftop	\N	iftop-1.0.p4
2020-09-25 14:25:04+02	2020-09-25 14:25:18+02	net-mgmt	ipcalc	\N	ipcalc-0.41_3
2020-09-25 14:24:44+02	2020-09-25 14:25:35+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-09-25 14:25:18+02	2020-09-25 14:25:45+02	net-mgmt	ipv6calc	\N	ipv6calc-2.2.0
2020-09-25 14:25:36+02	2020-09-25 14:25:45+02	net-mgmt	nstat	\N	nstat-1.0_3
2020-09-25 14:25:46+02	2020-09-25 14:25:54+02	net-mgmt	whatmask	\N	whatmask-1.2_1
2020-09-25 14:25:55+02	2020-09-25 14:26:10+02	ports-mgmt	pkg_tree	\N	pkg_tree-1.1_4
2020-09-25 14:25:46+02	2020-09-25 14:26:35+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-09-25 14:26:36+02	2020-09-25 14:26:47+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-09-25 14:26:10+02	2020-09-25 14:27:04+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-09-25 14:26:48+02	2020-09-25 14:28:22+02	ports-mgmt	synth	\N	synth-2.08
2020-09-25 14:28:23+02	2020-09-25 14:28:59+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-09-25 14:29:00+02	2020-09-25 14:29:34+02	print	gv	\N	gv-3.7.4_5
2020-09-25 14:27:06+02	2020-09-25 14:29:56+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-25 14:29:34+02	2020-09-25 14:31:59+02	print	lyx	\N	lyx-2.3.4.2_1
2020-09-25 14:32:02+02	2020-09-25 14:38:07+02	print	texlive-full	\N	texlive-full-20150521
2020-09-25 14:38:08+02	2020-09-25 14:38:53+02	security	aide	\N	aide-0.16.1
2020-09-25 14:38:54+02	2020-09-25 14:39:02+02	security	base-audit	\N	base-audit-0.4
2020-09-25 14:39:02+02	2020-09-25 14:40:27+02	security	clamav	\N	clamav-0.103.0,1
2020-09-25 14:40:28+02	2020-09-25 14:40:35+02	security	doas	\N	doas-6.3_1
2020-09-25 14:40:36+02	2020-09-25 14:40:53+02	security	ipv6toolkit	\N	ipv6toolkit-2.0
2020-09-25 14:40:54+02	2020-09-25 14:41:13+02	security	kc24	\N	kc24-2.4.2_1
2020-09-25 14:41:14+02	2020-09-25 14:42:57+02	security	klee	\N	klee-2.1_1
2020-09-25 14:42:58+02	2020-09-25 14:43:11+02	security	lynis	\N	lynis-3.0.0
2020-09-25 14:43:11+02	2020-09-25 14:43:25+02	security	nyx	\N	nyx-2.1.0_1
2020-09-25 14:43:26+02	2020-09-25 14:44:30+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-09-25 14:44:31+02	2020-09-25 14:45:33+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-09-25 14:45:35+02	2020-09-25 14:45:54+02	security	py-xkcdpass	py37	py37-xkcdpass-1.17.4
2020-09-25 14:45:55+02	2020-09-25 14:46:19+02	security	rkhunter	\N	rkhunter-1.4.6
2020-09-25 14:46:19+02	2020-09-25 14:46:40+02	security	ssl-admin	\N	ssl-admin-1.2.1
2020-09-25 14:46:41+02	2020-09-25 14:48:25+02	security	tor	\N	tor-0.4.4.5
2020-09-25 14:48:25+02	2020-09-25 14:50:16+02	shells	fish	\N	fish-3.1.2
2020-09-25 14:50:17+02	2020-09-25 15:02:48+02	shells	starship	\N	starship-0.44.0_2
2020-09-25 15:02:49+02	2020-09-25 15:04:26+02	shells	zsh	\N	zsh-5.8
2020-09-25 15:04:26+02	2020-09-25 15:04:34+02	sysutils	915resolution	\N	915resolution-0.5.3_1,1
2020-09-25 15:04:34+02	2020-09-25 15:07:52+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-09-25 15:07:53+02	2020-09-25 15:08:30+02	sysutils	apachetop	\N	apachetop-0.18.4_1
2020-09-25 15:08:31+02	2020-09-25 15:08:38+02	sysutils	atop	\N	atop-2.0.2.b3_3
2020-09-25 15:08:39+02	2020-09-25 15:08:53+02	sysutils	bashtop	\N	bashtop-0.9.25_1
2020-09-25 15:08:53+02	2020-09-25 15:09:01+02	sysutils	beadm	\N	beadm-1.3.0
2020-09-25 15:09:02+02	2020-09-25 15:09:17+02	sysutils	bpytop	\N	bpytop-1.0.33
2020-09-25 15:09:18+02	2020-09-25 15:09:26+02	sysutils	bsdcrashtar	\N	bsdcrashtar-0.1.3_1
2020-09-25 15:09:27+02	2020-09-25 15:09:36+02	sysutils	bsdhwmon	\N	bsdhwmon-20151206
2020-09-25 15:09:36+02	2020-09-25 15:09:44+02	sysutils	bsdinfo	\N	bsdinfo-0.22_2
2020-09-25 15:09:45+02	2020-09-25 15:09:52+02	sysutils	bsdstats	\N	bsdstats-7.0
2020-09-25 15:09:53+02	2020-09-25 15:12:19+02	sysutils	cdrtools	\N	cdrtools-3.01_1
2020-09-25 15:12:19+02	2020-09-25 15:12:27+02	sysutils	cmdwatch	\N	cmdwatch-0.2.0_2
2020-09-25 15:12:27+02	2020-09-25 15:13:43+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-25 15:13:44+02	2020-09-25 15:13:53+02	sysutils	cpupdate	\N	cpupdate-g20180513_2
2020-09-25 15:13:54+02	2020-09-25 15:14:02+02	sysutils	ddrescue	\N	ddrescue-1.25
2020-09-25 15:14:03+02	2020-09-25 15:14:15+02	sysutils	devcpu-data	\N	devcpu-data-1.34
2020-09-25 15:14:16+02	2020-09-25 15:14:34+02	sysutils	dfc	\N	dfc-3.1.1
2020-09-25 15:14:35+02	2020-09-25 15:15:09+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-09-25 15:15:10+02	2020-09-25 15:19:04+02	sysutils	exa	\N	exa-0.9.0_15
2020-09-25 15:19:05+02	2020-09-25 15:19:22+02	sysutils	flashrom	\N	flashrom-1.2
2020-09-25 15:19:23+02	2020-09-25 15:19:46+02	sysutils	fusefs-ntfs	\N	fusefs-ntfs-2017.3.23
2020-09-25 15:19:47+02	2020-09-25 15:20:31+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-25 15:20:32+02	2020-09-25 15:22:07+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-25 15:22:08+02	2020-09-25 15:23:08+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-25 14:29:57+02	2020-09-25 15:23:23+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-09-25 15:23:24+02	2020-09-25 15:23:34+02	sysutils	hourglass	\N	hourglass-1.0.1
2020-09-25 15:23:35+02	2020-09-25 15:24:06+02	sysutils	htop	\N	htop-3.0.2
2020-09-25 15:24:06+02	2020-09-25 15:24:14+02	sysutils	ioping	\N	ioping-1.2
2020-09-25 15:24:15+02	2020-09-25 15:24:23+02	sysutils	lsop	\N	lsop-0.2
2020-09-25 15:24:23+02	2020-09-25 15:24:43+02	sysutils	memtest86+	\N	memtest86+-5.01_3
2020-09-25 15:24:44+02	2020-09-25 15:24:54+02	sysutils	most	\N	most-5.1.0
2020-09-25 15:23:09+02	2020-09-25 15:24:55+02	sysutils	hexyl	\N	hexyl-0.8.0_4
2020-09-25 15:24:56+02	2020-09-25 15:25:04+02	sysutils	neofetch	\N	neofetch-7.1.0
2020-09-25 15:24:54+02	2020-09-25 15:25:04+02	sysutils	ncdu	\N	ncdu-1.15.1
2020-09-25 15:25:05+02	2020-09-25 15:25:12+02	sysutils	obliterate	\N	obliterate-0.5
2020-09-25 15:25:05+02	2020-09-25 15:25:22+02	sysutils	panicmail	\N	panicmail-1.1
2020-09-25 15:25:23+02	2020-09-25 15:25:30+02	sysutils	procmap	\N	procmap-1.0
2020-09-25 15:25:12+02	2020-09-25 15:25:36+02	sysutils	parallel	\N	parallel-20200822
2020-09-25 15:25:31+02	2020-09-25 15:25:44+02	sysutils	psmisc	\N	psmisc-22.16_1
2020-09-25 15:25:36+02	2020-09-25 15:25:45+02	sysutils	pv	\N	pv-1.6.6
2020-09-25 15:25:44+02	2020-09-25 15:25:52+02	sysutils	pwgen	\N	pwgen-2.08,2
2020-09-25 15:25:45+02	2020-09-25 15:26:03+02	sysutils	py-glances	py37	py37-glances-3.1.4
2020-09-25 15:25:53+02	2020-09-25 15:26:05+02	sysutils	py-py-cpuinfo	py37	py37-py-cpuinfo-7.0.0
2020-09-25 15:26:04+02	2020-09-25 15:26:11+02	sysutils	reptyr	\N	reptyr-0.6.3a_2
2020-09-25 15:26:11+02	2020-09-25 15:26:40+02	sysutils	screen	\N	screen-4.8.0
2020-09-25 15:26:40+02	2020-09-25 15:26:49+02	sysutils	spinner	\N	spinner-1.2.4
2020-09-25 15:26:50+02	2020-09-25 15:26:57+02	sysutils	ssd_report	smart	ssd_report-0.4
2020-09-25 15:26:58+02	2020-09-25 15:28:03+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-09-25 15:28:03+02	2020-09-25 15:28:13+02	sysutils	tarsnap-periodic	\N	tarsnap-periodic-1.0.20170413
2020-09-25 15:28:13+02	2020-09-25 15:28:40+02	sysutils	tmux	\N	tmux-3.1b
2020-09-25 15:28:41+02	2020-09-25 15:28:48+02	sysutils	tree	\N	tree-1.8.0
2020-09-25 15:28:49+02	2020-09-25 15:29:08+02	sysutils	x86info	\N	x86info-1.31.s03
2020-09-25 15:29:08+02	2020-09-25 15:29:27+02	sysutils	zfs-stats	\N	zfs-stats-1.3.0
2020-09-25 15:29:28+02	2020-09-25 15:30:44+02	sysutils	zrepl	\N	zrepl-0.2.1
2020-09-25 15:26:06+02	2020-09-25 15:37:35+02	sysutils	rsfetch	\N	rsfetch-2.0.0_7
2020-09-25 15:37:36+02	2020-09-25 15:37:53+02	textproc	colordiff	\N	colordiff-1.0.19
2020-09-25 15:37:53+02	2020-09-25 15:38:19+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-09-25 15:38:19+02	2020-09-25 15:38:49+02	www	dokuwiki	\N	dokuwiki-20200729
2020-09-25 15:30:45+02	2020-09-25 15:44:15+02	textproc	bat	\N	bat-0.15.4_4
2020-09-25 15:44:15+02	2020-09-25 15:45:06+02	www	links	\N	links-2.20.2,1
2020-09-25 15:45:07+02	2020-09-25 15:45:46+02	www	lynx	\N	lynx-2.8.9.1_1,1
2020-09-25 15:45:47+02	2020-09-25 15:48:20+02	www	mattermost-server	\N	mattermost-server-5.26.1
2020-09-25 15:48:21+02	2020-09-25 15:48:53+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_2
2020-09-25 15:48:54+02	2020-09-25 15:51:47+02	www	mod_php74	\N	mod_php74-7.4.10
2020-09-25 15:51:47+02	2020-09-25 15:52:28+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-09-25 15:52:29+02	2020-09-25 15:52:43+02	www	py-bottle	py37	py37-bottle-0.12.15_1
2020-09-25 15:52:43+02	2020-09-25 15:53:02+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-09-25 15:53:02+02	2020-09-25 15:53:18+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-09-25 15:53:19+02	2020-09-25 15:53:33+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-2.2_1
2020-09-25 15:53:34+02	2020-09-25 15:53:48+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-09-25 15:53:49+02	2020-09-25 15:54:06+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-09-25 15:54:06+02	2020-09-25 15:54:22+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-09-25 15:54:23+02	2020-09-25 15:54:43+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-09-25 15:54:44+02	2020-09-25 15:55:02+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-09-25 15:55:02+02	2020-09-25 15:55:19+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-09-25 15:55:20+02	2020-09-25 15:55:34+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-09-25 15:55:35+02	2020-09-25 15:56:01+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-09-25 15:56:02+02	2020-09-25 15:56:17+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-09-25 15:56:18+02	2020-09-25 16:20:25+02	x11	alacritty	\N	alacritty-0.5.0_2
2020-09-25 16:20:29+02	2020-09-25 16:21:33+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-09-25 16:21:33+02	2020-09-25 16:23:37+02	x11	mate	\N	mate-1.22.2
2020-09-25 16:23:38+02	2020-09-25 16:23:51+02	x11	numlockx	\N	numlockx-1.2
2020-09-25 16:23:51+02	2020-09-25 16:24:02+02	x11	sterm	\N	sterm-0.8.4
2020-09-25 16:24:03+02	2020-09-25 16:24:32+02	x11	xclip	\N	xclip-0.13
2020-09-25 16:24:32+02	2020-09-25 16:24:46+02	x11	xclipboard	\N	xclipboard-1.1.3,1
2020-09-25 16:24:47+02	2020-09-25 16:25:08+02	x11	xdm	\N	xdm-1.1.12_3
2020-09-25 16:25:09+02	2020-09-25 16:25:35+02	x11	xlockmore	\N	xlockmore-5.65
2020-09-25 16:25:36+02	2020-09-25 16:25:47+02	x11	xman	\N	xman-1.1.5
2020-09-25 16:25:48+02	2020-09-25 16:27:45+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-25 13:22:48+02	2020-09-25 16:29:26+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200913
2020-09-25 16:29:27+02	2020-09-25 16:30:18+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-25 16:30:19+02	2020-09-25 16:30:36+02	x11-fm	xcruise	\N	xcruiser-0.30_3
2020-09-25 16:30:37+02	2020-09-25 16:30:47+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-09-25 16:30:48+02	2020-09-25 16:31:08+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-09-25 16:31:08+02	2020-09-25 16:31:27+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.5_6
2020-09-25 16:31:27+02	2020-09-25 16:31:48+02	x11-wm	bbkeys	\N	bbkeys-0.9.1_1
2020-09-25 16:31:49+02	2020-09-25 16:32:01+02	x11-wm	bbpager	\N	bbpager-0.4.7_3
2020-09-25 16:27:45+02	2020-09-25 16:33:13+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-25 16:33:14+02	2020-09-25 16:34:02+02	graphics	xfig	\N	xfig-3.2.7a
2020-09-25 16:34:03+02	2020-09-25 16:34:37+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-09-25 16:32:01+02	2020-09-25 16:54:56+02	editors	vscode	\N	vscode-1.46.1
2020-09-25 15:38:49+02	2020-09-25 17:43:54+02	www	firefox	\N	firefox-81.0,2
2020-09-25 17:55:19+02	2020-09-25 17:56:28+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_1
2020-09-25 17:55:19+02	2020-09-25 17:57:08+02	graphics	gpu-firmware-kmod	\N	gpu-firmware-kmod-g20200914
2020-09-25 17:57:08+02	2020-09-25 18:03:08+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-25 18:03:08+02	2020-09-25 18:03:16+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-25 19:27:54+02	2020-09-25 19:29:57+02	ports-mgmt	pkg	\N	pkg-1.15.6
2020-09-25 19:41:19+02	2020-09-25 19:41:30+02	devel	py-Jinja2	py37	py37-Jinja2-2.11.2
2020-09-25 19:41:19+02	2020-09-25 19:41:33+02	textproc	py-pygments	py37	py37-pygments-2.7.1
2020-09-25 19:41:19+02	2020-09-25 19:41:34+02	security	ca_root_nss	\N	ca_root_nss-3.57
2020-09-25 19:41:33+02	2020-09-25 19:41:49+02	converters	fribidi	\N	fribidi-1.0.10
2020-09-25 19:41:49+02	2020-09-25 19:42:12+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-09-25 19:42:13+02	2020-09-25 19:42:27+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-09-25 19:41:34+02	2020-09-25 19:42:50+02	ftp	curl	\N	curl-7.72.0
2020-09-25 19:42:28+02	2020-09-25 19:42:52+02	www	w3m	\N	w3m-0.5.3.20200507_1
2020-09-25 19:42:52+02	2020-09-25 19:43:17+02	textproc	xmlto	\N	xmlto-0.0.28
2020-09-25 19:43:18+02	2020-09-25 19:44:44+02	security	nss	\N	nss-3.57
2020-09-25 19:44:45+02	2020-09-25 19:45:01+02	audio	lv2	\N	lv2-1.18.0
2020-09-25 19:41:30+02	2020-09-25 19:45:18+02	devel	glib20	\N	glib-2.66.0_1,1
2020-09-25 19:42:50+02	2020-09-25 19:45:20+02	devel	cmake	\N	cmake-3.18.2
2020-09-25 19:45:02+02	2020-09-25 19:45:23+02	ftp	php74-curl	\N	php74-curl-7.4.10
2020-09-25 19:45:24+02	2020-09-25 19:45:51+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-09-25 19:45:20+02	2020-09-25 19:45:58+02	security	p11-kit	\N	p11-kit-0.23.21
2020-09-25 19:45:19+02	2020-09-25 19:46:01+02	graphics	cairo	\N	cairo-1.16.0_1,3
2020-09-25 19:45:52+02	2020-09-25 19:46:51+02	misc	shared-mime-info	\N	shared-mime-info-2.0
2020-09-25 19:46:02+02	2020-09-25 19:47:13+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-09-25 19:45:59+02	2020-09-25 19:48:14+02	security	gnutls	\N	gnutls-3.6.15
2020-09-25 19:48:14+02	2020-09-25 19:48:59+02	net	avahi-app	\N	avahi-app-0.7_3
2020-09-25 19:47:14+02	2020-09-25 19:49:05+02	print	harfbuzz	\N	harfbuzz-2.7.2
2020-09-25 19:49:00+02	2020-09-25 19:49:35+02	print	cups	\N	cups-2.3.3_1
2020-09-25 19:49:35+02	2020-09-25 19:50:09+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-09-25 19:50:10+02	2020-09-25 19:51:20+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-09-25 19:49:06+02	2020-09-25 19:51:37+02	x11-toolkits	pango	\N	pango-1.42.4_4
2020-09-25 19:51:21+02	2020-09-25 19:51:49+02	accessibility	atk	\N	atk-2.36.0
2020-09-25 19:51:37+02	2020-09-25 19:52:03+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-09-25 19:52:04+02	2020-09-25 19:53:32+02	sysutils	polkit	\N	polkit-0.118
2020-09-25 19:53:33+02	2020-09-25 19:54:58+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-09-25 19:51:49+02	2020-09-25 19:55:07+02	lang	vala	\N	vala-0.48.10,1
2020-09-25 19:54:59+02	2020-09-25 19:55:37+02	textproc	libcroco	\N	libcroco-0.6.13
2020-09-25 19:55:07+02	2020-09-25 19:56:37+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-09-25 19:55:38+02	2020-09-25 19:56:52+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-09-25 19:56:38+02	2020-09-25 19:56:58+02	devel	libgudev	\N	libgudev-233
2020-09-25 19:56:58+02	2020-09-25 19:57:17+02	x11	libwacom	\N	libwacom-1.4.1
2020-09-25 19:57:17+02	2020-09-25 19:57:35+02	x11	libinput	\N	libinput-1.15.6
2020-09-25 19:57:36+02	2020-09-25 19:58:13+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-09-25 19:56:53+02	2020-09-25 19:59:32+02	graphics	colord	\N	colord-1.3.5
2020-09-25 19:58:14+02	2020-09-25 19:59:52+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-09-25 19:59:33+02	2020-09-25 20:00:20+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-09-25 19:59:53+02	2020-09-25 20:00:53+02	net	qt5-network	\N	qt5-network-5.15.0
2020-09-25 20:00:54+02	2020-09-25 20:03:46+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-09-25 20:03:47+02	2020-09-25 20:06:08+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-09-25 20:06:09+02	2020-09-25 20:07:41+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-09-25 20:07:42+02	2020-09-25 20:08:17+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.2
2020-09-25 20:08:18+02	2020-09-25 20:09:13+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-09-25 20:09:13+02	2020-09-25 20:09:32+02	devel	desktop-file-utils	\N	desktop-file-utils-0.26
2020-09-25 20:00:21+02	2020-09-25 20:12:54+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-09-25 20:12:55+02	2020-09-25 20:13:55+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-09-25 20:13:56+02	2020-09-25 20:17:14+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-09-25 20:09:33+02	2020-09-25 20:19:16+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-25 20:17:15+02	2020-09-25 20:21:09+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-09-25 20:21:12+02	2020-09-25 20:21:46+02	graphics	poppler	\N	poppler-20.09.0
2020-09-25 20:21:46+02	2020-09-25 20:22:22+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-09-25 20:22:23+02	2020-09-25 20:23:03+02	net	glib-networking	\N	glib-networking-2.56.1_3
2020-09-25 20:19:17+02	2020-09-25 20:23:04+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-09-25 20:23:04+02	2020-09-25 20:23:46+02	devel	dconf	\N	dconf-0.38.0
2020-09-25 20:23:47+02	2020-09-25 20:24:53+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-09-25 20:23:05+02	2020-09-25 20:24:59+02	devel	libsoup	\N	libsoup-2.62.3
2020-09-25 20:24:54+02	2020-09-25 20:25:28+02	devel	json-glib	\N	json-glib-1.6.0
2020-09-25 20:25:00+02	2020-09-25 20:25:50+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-09-25 20:25:50+02	2020-09-25 20:26:38+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-09-25 20:25:28+02	2020-09-25 20:29:57+02	print	texlive-base	\N	texlive-base-20150521_55
2020-09-25 20:26:39+02	2020-09-25 20:31:24+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-09-25 20:31:25+02	2020-09-25 20:32:22+02	devel	libnotify	\N	libnotify-0.7.9
2020-09-25 20:32:23+02	2020-09-25 20:33:31+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-09-25 20:33:33+02	2020-09-25 20:36:13+02	security	libsecret	\N	libsecret-0.18.6_1
2020-09-25 20:36:14+02	2020-09-25 20:37:53+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-09-25 20:37:54+02	2020-09-25 20:38:20+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-09-25 20:38:21+02	2020-09-25 20:40:01+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-09-25 20:40:02+02	2020-09-25 20:42:56+02	graphics	graphviz	\N	graphviz-2.44.1_2
2020-09-25 20:42:57+02	2020-09-25 20:43:14+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-09-25 20:43:14+02	2020-09-25 20:50:29+02	multimedia	x265	\N	x265-3.4
2020-09-25 20:50:30+02	2020-09-25 20:51:58+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-25 20:51:59+02	2020-09-25 20:52:20+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-09-25 20:52:20+02	2020-09-25 20:53:08+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-09-25 20:53:08+02	2020-09-25 20:54:23+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-25 20:29:58+02	2020-09-25 20:54:39+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-25 20:54:23+02	2020-09-25 20:54:59+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-25 20:54:40+02	2020-09-25 20:57:38+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-25 20:57:39+02	2020-09-25 20:58:03+02	graphics	frei0r	\N	frei0r-1.6.1
2020-09-25 20:54:59+02	2020-09-25 21:01:33+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-25 21:01:34+02	2020-09-25 21:02:48+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-09-25 21:02:48+02	2020-09-25 21:03:39+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-09-25 21:03:40+02	2020-09-25 21:05:26+02	graphics	graphene	\N	graphene-1.10.2
2020-09-25 20:58:03+02	2020-09-25 21:06:03+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-09-25 21:06:04+02	2020-09-25 21:07:15+02	net	geoclue	\N	geoclue-2.5.5
2020-09-25 21:07:15+02	2020-09-25 21:07:41+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-09-25 21:07:42+02	2020-09-25 21:08:10+02	graphics	jasper	\N	jasper-2.0.21
2020-09-25 21:05:27+02	2020-09-25 21:08:10+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-25 21:08:11+02	2020-09-25 21:12:43+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-25 21:12:44+02	2020-09-25 21:14:06+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.74.0
2020-09-25 21:14:07+02	2020-09-25 21:14:55+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-09-25 21:14:56+02	2020-09-25 21:15:37+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-09-25 21:15:37+02	2020-09-25 21:16:35+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-09-25 21:16:35+02	2020-09-25 21:17:18+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-25 21:17:19+02	2020-09-25 21:17:43+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-09-25 21:17:44+02	2020-09-25 21:19:11+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-09-25 21:19:12+02	2020-09-25 21:19:33+02	print	libraqm	\N	libraqm-0.6.0
2020-09-25 21:19:33+02	2020-09-25 21:23:59+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-25 21:24:00+02	2020-09-25 21:24:56+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-09-25 21:24:57+02	2020-09-25 21:25:42+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-09-25 21:25:43+02	2020-09-25 21:26:40+02	sysutils	policykit	\N	policykit-0.9_10
2020-09-25 21:26:41+02	2020-09-25 21:27:39+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-09-25 21:27:39+02	2020-09-25 21:28:31+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.74.0
2020-09-25 21:28:32+02	2020-09-25 21:29:47+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-25 21:29:48+02	2020-09-25 21:30:44+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-25 21:30:44+02	2020-09-25 21:32:03+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-25 21:32:04+02	2020-09-25 21:32:59+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-09-25 21:33:00+02	2020-09-25 21:34:06+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-09-25 21:34:07+02	2020-09-25 21:35:24+02	sysutils	hal	\N	hal-0.5.14_34
2020-09-25 21:35:25+02	2020-09-25 21:37:29+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-25 21:37:29+02	2020-09-25 21:38:33+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-25 21:38:34+02	2020-09-25 21:39:08+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-09-25 21:39:09+02	2020-09-25 21:40:09+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-09-25 21:40:09+02	2020-09-25 21:41:38+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-09-25 21:41:39+02	2020-09-25 21:42:18+02	archivers	kf5-karchive	\N	kf5-karchive-5.74.0
2020-09-25 21:42:19+02	2020-09-25 21:43:41+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-09-25 21:43:42+02	2020-09-25 21:44:40+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-09-25 21:44:40+02	2020-09-25 21:47:11+02	x11-toolkits	vte3	\N	vte3-0.62.0
2020-09-25 21:47:12+02	2020-09-25 21:48:58+02	devel	git	\N	git-2.28.0
2020-09-25 21:48:59+02	2020-09-25 21:50:23+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-09-25 21:50:24+02	2020-09-25 21:52:30+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-09-25 21:52:31+02	2020-09-25 21:53:28+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-09-25 21:53:28+02	2020-09-25 21:54:31+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-09-25 21:54:32+02	2020-09-25 21:55:23+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-25 21:55:23+02	2020-09-25 22:02:51+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-09-25 22:02:51+02	2020-09-25 22:04:03+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-09-25 22:04:04+02	2020-09-25 22:08:03+02	security	gcr	\N	gcr-3.28.0_1
2020-09-25 22:08:04+02	2020-09-25 22:09:33+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-09-25 22:09:34+02	2020-09-25 22:11:43+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-09-25 22:11:43+02	2020-09-25 22:12:44+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-09-25 22:12:44+02	2020-09-25 22:13:42+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-09-25 22:13:42+02	2020-09-25 22:18:22+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-09-25 22:18:23+02	2020-09-25 22:19:09+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-09-25 22:19:09+02	2020-09-25 22:20:32+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-25 22:20:33+02	2020-09-25 22:21:47+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-25 22:21:48+02	2020-09-25 22:22:30+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.74.0
2020-09-25 22:22:31+02	2020-09-25 22:26:14+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-25 22:26:14+02	2020-09-25 22:27:12+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-09-25 22:27:12+02	2020-09-25 22:28:20+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-09-25 22:28:21+02	2020-09-25 22:30:00+02	databases	mysql57-client	\N	mysql57-client-5.7.31_1
2020-09-25 22:30:02+02	2020-09-25 22:31:00+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-09-25 22:31:01+02	2020-09-25 22:32:17+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-09-25 22:32:18+02	2020-09-25 22:32:31+02	devel	py-zipp	py37	py37-zipp-3.2.0
2020-09-25 22:32:32+02	2020-09-25 22:33:37+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-25 22:33:38+02	2020-09-25 22:35:13+02	net	remmina	\N	remmina-1.3.9
2020-09-25 22:35:13+02	2020-09-25 22:37:09+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-09-25 22:37:09+02	2020-09-25 22:37:56+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-09-25 22:37:57+02	2020-09-25 22:38:57+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-09-25 22:38:58+02	2020-09-25 22:39:28+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-09-25 22:39:28+02	2020-09-25 22:40:09+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-09-25 22:40:10+02	2020-09-25 22:40:57+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-09-25 22:40:58+02	2020-09-25 22:42:15+02	devel	libgtop	\N	libgtop-2.40.0
2020-09-25 22:42:16+02	2020-09-25 22:42:28+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-2.0.0
2020-09-25 22:42:29+02	2020-09-25 22:44:10+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-25 22:44:11+02	2020-09-25 22:44:41+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-09-25 22:44:42+02	2020-09-25 22:45:14+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-09-25 22:45:15+02	2020-09-25 22:46:02+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-25 22:46:03+02	2020-09-25 22:46:44+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-09-25 22:46:44+02	2020-09-25 22:47:09+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-09-25 22:47:10+02	2020-09-25 22:48:14+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-09-25 22:48:14+02	2020-09-25 22:49:12+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-09-25 22:49:15+02	2020-09-25 22:53:56+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-09-25 22:53:56+02	2020-09-25 22:54:52+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-25 22:54:52+02	2020-09-25 22:57:08+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.74.0
2020-09-25 21:08:11+02	2020-09-25 22:57:32+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-25 22:57:09+02	2020-09-25 22:57:46+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-09-25 22:57:47+02	2020-09-25 22:58:40+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-09-25 22:58:41+02	2020-09-25 22:59:32+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-09-25 22:59:33+02	2020-09-25 23:00:51+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-25 19:46:51+02	2020-09-25 23:02:18+02	lang	rust	\N	rust-1.46.0
2020-09-25 23:00:52+02	2020-09-25 23:02:30+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-25 23:02:30+02	2020-09-25 23:03:31+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-09-25 23:03:31+02	2020-09-25 23:03:57+02	devel	gamin	\N	gamin-0.1.10_10
2020-09-25 23:03:58+02	2020-09-25 23:05:10+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-09-25 23:05:11+02	2020-09-25 23:05:28+02	devel	py-Jinja2	py27	py27-Jinja2-2.11.2
2020-09-25 22:57:33+02	2020-09-25 23:06:48+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-25 23:02:20+02	2020-09-25 23:08:28+02	devel	rust-cbindgen	\N	rust-cbindgen-0.14.6
2020-09-25 23:06:49+02	2020-09-25 23:08:47+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-25 23:08:29+02	2020-09-25 23:09:01+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-25 23:08:47+02	2020-09-25 23:09:38+02	security	gnupg	\N	gnupg-2.2.23
2020-09-25 23:09:01+02	2020-09-25 23:09:41+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-09-25 23:09:39+02	2020-09-25 23:10:09+02	x11	xterm	\N	xterm-360
2020-09-25 23:09:41+02	2020-09-25 23:10:33+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-09-25 23:05:29+02	2020-09-25 23:10:41+02	math	vtk8	\N	vtk8-8.2.0
2020-09-25 23:10:10+02	2020-09-25 23:11:51+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-09-25 23:10:33+02	2020-09-25 23:11:54+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-09-25 23:11:52+02	2020-09-25 23:12:27+02	devel	librest	\N	rest-0.7.93
2020-09-25 23:11:54+02	2020-09-25 23:12:51+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-09-25 23:12:30+02	2020-09-25 23:13:19+02	graphics	netpbm	\N	netpbm-10.91.01
2020-09-25 23:13:19+02	2020-09-25 23:14:58+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-09-25 23:12:52+02	2020-09-25 23:15:17+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-09-25 23:15:18+02	2020-09-25 23:15:34+02	security	gpgme	\N	gpgme-1.14.0
2020-09-25 23:10:42+02	2020-09-25 23:15:52+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-25 23:15:34+02	2020-09-25 23:16:19+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-09-25 23:15:53+02	2020-09-25 23:16:38+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-09-25 23:16:38+02	2020-09-25 23:17:32+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-25 23:14:59+02	2020-09-25 23:17:43+02	net	libmateweather	\N	libmateweather-1.24.1
2020-09-25 23:17:43+02	2020-09-25 23:18:13+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-09-25 23:17:33+02	2020-09-25 23:18:22+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-09-25 23:18:13+02	2020-09-25 23:19:05+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-09-25 23:18:23+02	2020-09-25 23:19:08+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-25 23:19:09+02	2020-09-25 23:20:11+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-09-25 23:19:05+02	2020-09-25 23:22:20+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-09-25 23:22:21+02	2020-09-25 23:22:39+02	net	liboauth	\N	liboauth-1.0.3_4
2020-09-25 23:20:12+02	2020-09-25 23:23:07+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-25 23:23:07+02	2020-09-25 23:23:56+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-09-25 23:16:20+02	2020-09-25 23:24:19+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-25 23:24:20+02	2020-09-25 23:24:37+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-09-25 23:23:56+02	2020-09-25 23:24:43+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-25 23:24:43+02	2020-09-25 23:25:16+02	sysutils	upower	\N	upower-0.99.11
2020-09-25 23:24:37+02	2020-09-25 23:25:25+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-25 23:25:17+02	2020-09-25 23:25:42+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-09-25 23:25:42+02	2020-09-25 23:26:47+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-09-25 23:26:47+02	2020-09-25 23:27:18+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-09-25 23:27:18+02	2020-09-25 23:32:16+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-25 23:32:17+02	2020-09-25 23:33:22+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-09-25 23:22:40+02	2020-09-25 23:33:22+02	net	samba410	\N	samba410-4.10.18
2020-09-25 23:33:23+02	2020-09-25 23:34:17+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-09-25 23:33:23+02	2020-09-25 23:34:23+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-09-25 23:34:24+02	2020-09-25 23:34:57+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-09-25 23:34:17+02	2020-09-25 23:35:27+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-09-25 23:35:28+02	2020-09-25 23:35:54+02	archivers	gcab	\N	gcab-1.4
2020-09-25 23:35:55+02	2020-09-25 23:39:50+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-09-25 23:34:58+02	2020-09-25 23:40:37+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-25 23:39:52+02	2020-09-25 23:40:55+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-09-25 23:25:26+02	2020-09-25 23:43:53+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_3
2020-09-25 23:40:55+02	2020-09-25 23:43:57+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-25 23:43:54+02	2020-09-25 23:44:29+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-09-25 23:44:30+02	2020-09-25 23:45:08+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-09-25 23:45:11+02	2020-09-25 23:46:09+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-25 23:43:58+02	2020-09-25 23:46:29+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-09-25 23:46:10+02	2020-09-25 23:46:59+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-09-25 23:46:30+02	2020-09-25 23:47:01+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-09-25 23:47:02+02	2020-09-25 23:47:44+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-09-25 23:47:00+02	2020-09-25 23:48:03+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-25 23:48:03+02	2020-09-25 23:48:29+02	x11	babl	\N	babl-0.1.78_1
2020-09-25 23:48:29+02	2020-09-25 23:50:04+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-25 23:50:05+02	2020-09-25 23:50:35+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-09-25 23:50:35+02	2020-09-25 23:51:47+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-25 23:51:48+02	2020-09-25 23:56:19+02	x11-fm	caja	\N	caja-1.24.0
2020-09-25 23:40:38+02	2020-09-25 23:57:04+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-25 23:57:05+02	2020-09-25 23:58:30+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-09-25 23:58:30+02	2020-09-25 23:59:38+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-25 23:59:38+02	2020-09-26 00:01:15+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-26 00:01:16+02	2020-09-26 00:02:59+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-26 00:02:59+02	2020-09-26 00:06:45+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-25 23:56:20+02	2020-09-26 00:07:02+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-26 00:06:46+02	2020-09-26 00:07:54+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-09-26 00:07:03+02	2020-09-26 00:08:03+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-09-26 00:08:03+02	2020-09-26 00:08:15+02	devel	py-pluggy	py37	py37-pluggy-0.13.1
2020-09-26 00:07:56+02	2020-09-26 00:08:55+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-09-26 00:08:56+02	2020-09-26 00:09:17+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-09-26 00:09:17+02	2020-09-26 00:09:52+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-09-26 00:09:53+02	2020-09-26 00:10:15+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-09-26 00:10:15+02	2020-09-26 00:10:37+02	graphics	simage	\N	simage-1.7.0_7
2020-09-26 00:08:16+02	2020-09-26 00:11:46+02	graphics	gegl	\N	gegl-0.4.26_5
2020-09-26 00:11:47+02	2020-09-26 00:13:48+02	mail	mailutils	\N	mailutils-3.10
2020-09-26 00:10:38+02	2020-09-26 00:16:26+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-09-26 00:13:48+02	2020-09-26 00:16:28+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-26 00:16:27+02	2020-09-26 00:17:12+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-09-26 00:17:13+02	2020-09-26 00:17:37+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-09-26 00:16:28+02	2020-09-26 00:17:48+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-09-26 00:17:38+02	2020-09-26 00:18:35+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-09-26 00:18:36+02	2020-09-26 00:19:46+02	x11	zenity	\N	zenity-3.32.0
2020-09-26 00:19:46+02	2020-09-26 00:20:37+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-09-26 00:17:49+02	2020-09-26 00:20:43+02	x11	xorg	\N	xorg-7.7_3
2020-09-26 00:20:44+02	2020-09-26 00:21:58+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-26 00:21:59+02	2020-09-26 00:22:49+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-09-26 00:22:49+02	2020-09-26 00:23:58+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-09-26 00:20:38+02	2020-09-26 00:24:13+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-09-26 00:23:59+02	2020-09-26 00:24:48+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-09-26 00:24:14+02	2020-09-26 00:24:58+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-09-26 00:24:49+02	2020-09-26 00:26:26+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-09-26 00:24:59+02	2020-09-26 00:29:02+02	dns	bind-tools	\N	bind-tools-9.16.7
2020-09-26 00:29:02+02	2020-09-26 00:29:22+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-09-26 00:26:27+02	2020-09-26 00:31:49+02	editors	emacs	full	emacs-27.1_1,3
2020-09-26 00:31:50+02	2020-09-26 00:32:12+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-09-26 00:29:22+02	2020-09-26 00:33:29+02	graphics	Coin	\N	Coin-4.0.0_1
2020-09-26 00:33:29+02	2020-09-26 00:38:24+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-26 00:38:25+02	2020-09-26 00:39:28+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-26 00:39:28+02	2020-09-26 00:44:19+02	math	py-matplotlib	py37	py37-matplotlib-3.3.1
2020-09-26 00:44:20+02	2020-09-26 00:44:49+02	net	gssdp	\N	gssdp-1.2.3
2020-09-26 00:44:50+02	2020-09-26 00:45:45+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-09-26 00:45:46+02	2020-09-26 00:46:49+02	print	cups-filters	\N	cups-filters-1.28.3
2020-09-26 00:32:13+02	2020-09-26 00:50:55+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-26 00:46:50+02	2020-09-26 00:55:39+02	print	fontforge	\N	fontforge-20190801_2
2020-09-26 00:55:39+02	2020-09-26 00:57:38+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-26 00:57:39+02	2020-09-26 00:58:11+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-09-26 00:50:55+02	2020-09-26 00:59:44+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-26 00:58:11+02	2020-09-26 01:00:33+02	www	node12	\N	node12-12.18.4
2020-09-26 00:59:45+02	2020-09-26 01:01:08+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-26 01:01:09+02	2020-09-26 01:02:25+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-26 01:02:18+02	2020-09-26 01:03:06+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-26 01:02:25+02	2020-09-26 01:04:11+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-26 01:03:06+02	2020-09-26 01:04:32+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-26 01:04:33+02	2020-09-26 01:05:19+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-09-26 01:04:11+02	2020-09-26 01:05:22+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-26 01:05:19+02	2020-09-26 01:06:26+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-09-26 01:05:22+02	2020-09-26 01:06:50+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-09-26 01:06:50+02	2020-09-26 01:07:46+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-25 23:47:44+02	2020-09-26 01:07:50+02	www	node	\N	node-14.12.0
2020-09-26 01:07:46+02	2020-09-26 01:10:06+02	x11-wm	marco	\N	marco-1.22.4
2020-09-26 01:06:26+02	2020-09-26 01:10:08+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-26 01:10:06+02	2020-09-26 01:10:49+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-26 01:10:08+02	2020-09-26 01:11:35+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-26 01:10:49+02	2020-09-26 01:11:47+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-26 01:11:47+02	2020-09-26 01:12:39+02	devel	git-lfs	\N	git-lfs-2.12.0
2020-09-26 01:11:36+02	2020-09-26 01:15:41+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-26 01:15:42+02	2020-09-26 01:15:56+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-09-26 01:15:56+02	2020-09-26 01:16:11+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-09-26 01:12:39+02	2020-09-26 01:20:00+02	devel	ncurses	\N	ncurses-6.2.20200919
2020-09-26 01:20:01+02	2020-09-26 01:23:38+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-26 01:23:39+02	2020-09-26 01:24:20+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-09-26 01:24:21+02	2020-09-26 01:26:09+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-26 01:26:09+02	2020-09-26 01:26:40+02	graphics	libgxps	\N	libgxps-0.3.1
2020-09-26 01:16:12+02	2020-09-26 01:26:50+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-26 01:26:41+02	2020-09-26 01:27:30+02	graphics	libheif	\N	libheif-1.6.2_1
2020-09-26 01:26:51+02	2020-09-26 01:27:59+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-09-26 01:28:00+02	2020-09-26 01:28:32+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-09-26 01:28:32+02	2020-09-26 01:29:25+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-26 01:27:30+02	2020-09-26 01:30:12+02	graphics	ptex	\N	ptex-2.3.2
2020-09-26 01:29:26+02	2020-09-26 01:30:20+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-09-26 01:30:21+02	2020-09-26 01:38:24+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-26 01:38:25+02	2020-09-26 01:39:21+02	multimedia	libass	\N	libass-0.14.0
2020-09-26 01:39:23+02	2020-09-26 01:40:32+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-26 01:40:33+02	2020-09-26 01:41:31+02	net	freerdp	\N	freerdp-2.2.0
2020-09-26 01:41:32+02	2020-09-26 01:42:06+02	net	gupnp	\N	gupnp-1.2.4
2020-09-26 01:07:51+02	2020-09-26 01:42:34+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-26 01:42:36+02	2020-09-26 01:42:46+02	net	py-uritemplate	py37	py37-uritemplate-3.0.1
2020-09-26 01:42:08+02	2020-09-26 01:43:06+02	net	libcmis	\N	libcmis-0.5.2_1
2020-09-26 01:42:47+02	2020-09-26 01:43:28+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-26 01:43:06+02	2020-09-26 01:44:06+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-09-26 01:43:28+02	2020-09-26 01:44:09+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-09-26 01:44:09+02	2020-09-26 01:44:25+02	print	libspectre	\N	libspectre-0.2.9
2020-09-26 01:30:13+02	2020-09-26 01:44:31+02	math	R	\N	R-4.0.2
2020-09-26 01:44:26+02	2020-09-26 01:46:09+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-26 01:44:07+02	2020-09-26 01:46:42+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-26 01:46:43+02	2020-09-26 01:47:30+02	security	sudo	\N	sudo-1.9.3p1
2020-09-26 01:47:31+02	2020-09-26 01:47:49+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-09-26 01:47:49+02	2020-09-26 01:48:03+02	sysutils	hal-info	\N	hal-info-20091130
2020-09-26 01:44:32+02	2020-09-26 01:49:17+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-26 01:49:18+02	2020-09-26 01:50:11+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-26 01:48:04+02	2020-09-26 01:50:50+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-09-26 01:46:10+02	2020-09-26 01:51:00+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-26 01:51:01+02	2020-09-26 01:51:35+02	textproc	redland	\N	redland-1.0.17_4
2020-09-26 01:50:11+02	2020-09-26 01:51:38+02	sysutils	tracker	\N	tracker-2.3.4
2020-09-26 01:51:35+02	2020-09-26 01:52:08+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-26 01:50:51+02	2020-09-26 01:52:25+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-26 01:51:39+02	2020-09-26 01:52:29+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-26 01:52:09+02	2020-09-26 01:52:35+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-09-26 01:52:29+02	2020-09-26 01:52:55+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-09-26 01:52:35+02	2020-09-26 01:53:25+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-09-26 01:52:25+02	2020-09-26 01:53:27+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-26 01:52:56+02	2020-09-26 01:53:35+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-26 01:53:27+02	2020-09-26 01:54:16+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-09-26 01:53:35+02	2020-09-26 01:54:29+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-26 01:53:25+02	2020-09-26 01:54:55+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-26 01:54:56+02	2020-09-26 01:55:17+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-09-26 01:55:18+02	2020-09-26 01:55:41+02	audio	faudio	\N	FAudio-20.09
2020-09-26 01:54:29+02	2020-09-26 01:56:48+02	archivers	engrampa	\N	engrampa-1.24.1
2020-09-26 01:56:49+02	2020-09-26 01:56:57+02	databases	lmdb	\N	lmdb-0.9.24_3,1
2020-09-26 01:55:42+02	2020-09-26 01:57:01+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-26 01:56:58+02	2020-09-26 01:58:03+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-09-26 01:57:02+02	2020-09-26 01:58:29+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-09-26 01:58:30+02	2020-09-26 01:59:24+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-26 01:59:25+02	2020-09-26 02:00:24+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-26 02:00:25+02	2020-09-26 02:01:18+02	devel	adacurses	\N	adacurses-20180127
2020-09-26 01:58:04+02	2020-09-26 02:01:56+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-26 02:01:19+02	2020-09-26 02:02:09+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-09-26 02:02:09+02	2020-09-26 02:02:20+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-09-26 02:01:56+02	2020-09-26 02:02:27+02	devel	mercurial	py37	mercurial-5.5_1
2020-09-26 02:02:21+02	2020-09-26 02:02:36+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-09-26 01:54:16+02	2020-09-26 02:02:36+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-26 02:02:36+02	2020-09-26 02:02:46+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-09-26 02:02:28+02	2020-09-26 02:03:10+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-09-26 02:02:36+02	2020-09-26 02:03:44+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-26 02:03:45+02	2020-09-26 02:07:04+02	editors	pluma	\N	pluma-1.24.1
2020-09-26 02:07:05+02	2020-09-26 02:07:54+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-26 02:07:54+02	2020-09-26 02:08:33+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-09-26 02:08:34+02	2020-09-26 02:11:55+02	graphics	atril	\N	atril-1.22.3_9
2020-09-26 02:11:56+02	2020-09-26 02:14:08+02	graphics	eom	\N	eom-1.24.1
2020-09-26 02:14:09+02	2020-09-26 02:15:40+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-26 02:15:41+02	2020-09-26 02:17:22+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-26 02:02:46+02	2020-09-26 02:23:34+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-26 02:23:35+02	2020-09-26 02:25:21+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-26 02:03:10+02	2020-09-26 02:27:45+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-09-26 02:25:21+02	2020-09-26 02:27:57+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-09-26 02:27:58+02	2020-09-26 02:34:51+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-26 02:17:23+02	2020-09-26 02:35:04+02	graphics	openimageio	\N	openimageio-2.2.6.1
2020-09-26 02:27:47+02	2020-09-26 02:36:18+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-26 02:35:05+02	2020-09-26 02:36:50+02	math	mate-calc	\N	mate-calc-1.24.1
2020-09-26 02:36:19+02	2020-09-26 02:37:23+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-09-26 02:36:50+02	2020-09-26 02:38:16+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-09-26 02:37:24+02	2020-09-26 02:39:00+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-09-26 02:39:01+02	2020-09-26 02:39:53+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-26 02:39:54+02	2020-09-26 02:40:35+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-26 02:40:35+02	2020-09-26 02:41:28+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-26 02:41:28+02	2020-09-26 02:42:10+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-26 02:38:16+02	2020-09-26 02:42:20+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-26 02:42:10+02	2020-09-26 02:42:52+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-26 02:42:20+02	2020-09-26 02:42:58+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-26 02:42:53+02	2020-09-26 02:43:30+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-26 02:42:59+02	2020-09-26 02:43:45+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-26 02:43:45+02	2020-09-26 02:44:27+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-09-26 02:43:30+02	2020-09-26 02:44:34+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-26 02:44:34+02	2020-09-26 02:45:19+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-09-26 02:44:28+02	2020-09-26 02:45:30+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-09-26 02:45:19+02	2020-09-26 02:46:18+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-26 02:34:52+02	2020-09-26 02:49:13+02	lang	julia10	\N	julia10-1.0.4_5
2020-09-26 02:45:30+02	2020-09-26 02:50:31+02	print	lilypond	\N	lilypond-2.20.0
2020-09-26 02:46:19+02	2020-09-26 02:51:29+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-26 02:50:32+02	2020-09-26 02:51:56+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-26 02:49:13+02	2020-09-26 02:53:47+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-26 02:53:47+02	2020-09-26 02:54:08+02	print	transfig	\N	transfig-3.2.5e_4
2020-09-26 02:54:08+02	2020-09-26 02:54:53+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-26 02:54:53+02	2020-09-26 02:56:20+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-09-26 02:51:30+02	2020-09-26 02:56:47+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-26 02:56:48+02	2020-09-26 02:57:44+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-26 02:57:45+02	2020-09-26 02:58:14+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-09-26 02:56:21+02	2020-09-26 02:58:15+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-09-26 02:58:15+02	2020-09-26 02:58:48+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-26 02:58:14+02	2020-09-26 02:58:54+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-09-26 02:58:49+02	2020-09-26 02:59:11+02	www	npm-node12	\N	npm-node12-6.12.1_1
2020-09-26 02:58:55+02	2020-09-26 02:59:11+02	www	p5-libwww	\N	p5-libwww-6.48
2020-09-26 02:59:11+02	2020-09-26 02:59:24+02	www	py-cssselect	py37	py37-cssselect-1.1.0
2020-09-26 02:59:11+02	2020-09-26 02:59:58+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-26 02:59:59+02	2020-09-26 03:00:10+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-09-26 02:59:24+02	2020-09-26 03:00:20+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-26 03:00:20+02	2020-09-26 03:01:21+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-26 03:01:21+02	2020-09-26 03:02:49+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-26 03:02:50+02	2020-09-26 03:04:54+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-26 03:00:10+02	2020-09-26 03:05:17+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-26 03:05:17+02	2020-09-26 03:06:27+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-26 03:06:28+02	2020-09-26 03:07:12+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-26 03:04:55+02	2020-09-26 03:07:46+02	x11-fm	nautilus	\N	nautilus-3.28.1_4
2020-09-26 02:51:57+02	2020-09-26 03:10:59+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-26 03:07:13+02	2020-09-26 03:16:17+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-26 03:11:00+02	2020-09-26 03:16:49+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-26 03:07:46+02	2020-09-26 03:21:02+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-26 03:16:50+02	2020-09-26 03:22:24+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-26 03:21:03+02	2020-09-26 03:23:43+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-26 03:23:44+02	2020-09-26 03:25:54+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-26 03:25:55+02	2020-09-26 03:27:51+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-26 03:27:52+02	2020-09-26 03:32:08+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-26 03:32:09+02	2020-09-26 03:33:54+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-26 03:16:18+02	2020-09-26 03:34:39+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-26 03:33:55+02	2020-09-26 03:34:46+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-26 03:34:47+02	2020-09-26 03:35:23+02	devel	bzr	py27	bzr-2.7.0.6622
2020-09-26 03:35:24+02	2020-09-26 03:45:55+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-26 03:45:56+02	2020-09-26 03:47:04+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-26 03:34:41+02	2020-09-26 03:56:47+02	devel	RStudio	\N	RStudio-1.3.1093
2020-09-26 03:56:48+02	2020-09-26 03:57:16+02	devel	tig	\N	tig-2.5.1
2020-09-26 03:57:16+02	2020-09-26 03:58:09+02	devel	tortoisehg	py37	tortoisehg-5.5.1_1
2020-09-26 03:58:09+02	2020-09-26 04:02:54+02	dns	bind916	\N	bind916-9.16.7_1
2020-09-26 04:02:54+02	2020-09-26 04:14:14+02	editors	codelite	\N	codelite-12.0_4
2020-09-26 04:14:15+02	2020-09-26 04:16:18+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-26 03:47:04+02	2020-09-26 04:16:36+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-26 04:16:19+02	2020-09-26 04:20:28+02	editors	vim	\N	vim-8.2.1558
2020-09-26 03:22:25+02	2020-09-26 05:07:44+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-26 05:07:45+02	2020-09-26 05:15:12+02	emulators	wine	\N	wine-5.0.2,1
2020-09-26 04:16:39+02	2020-09-26 05:15:40+02	emulators	qemu	\N	qemu-4.2.1
2020-09-26 05:15:40+02	2020-09-26 05:17:42+02	games	alienarena	\N	alienarena-2013.766_9
2020-09-26 05:17:42+02	2020-09-26 05:18:19+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-09-26 05:18:19+02	2020-09-26 05:18:48+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-09-26 05:18:49+02	2020-09-26 05:19:37+02	games	xboard	\N	xboard-4.9.1_1
2020-09-26 05:19:37+02	2020-09-26 05:20:22+02	games	xcowsay	\N	xcowsay-1.5
2020-09-26 04:20:29+02	2020-09-26 05:22:59+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_3
2020-09-26 05:15:13+02	2020-09-26 05:27:24+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-26 05:23:00+02	2020-09-26 05:28:34+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-26 05:27:25+02	2020-09-26 05:28:37+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-26 05:20:22+02	2020-09-26 05:31:36+02	graphics	blender	\N	blender-2.90.0
2020-09-26 05:31:37+02	2020-09-26 05:36:52+02	graphics	wings	\N	wings-2.2.5
2020-09-26 05:36:52+02	2020-09-26 05:37:55+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-26 05:28:37+02	2020-09-26 05:38:33+02	graphics	krita	\N	krita-4.3.0_5
2020-09-26 05:37:55+02	2020-09-26 05:39:40+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-09-26 05:38:34+02	2020-09-26 05:39:43+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-26 05:39:40+02	2020-09-26 06:03:22+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-26 05:39:43+02	2020-09-26 06:11:35+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.4_1
2020-09-26 06:12:41+02	2020-09-26 06:13:09+02	lang	mmix	\N	mmix-20160804_1
2020-09-26 05:28:35+02	2020-09-26 06:14:45+02	graphics	inkscape	\N	inkscape-1.0_5
2020-09-26 06:14:46+02	2020-09-26 06:20:44+02	math	maxima	\N	maxima-5.44.0_4
2020-09-26 06:20:45+02	2020-09-26 06:23:22+02	misc	mc	\N	mc-nox11-4.8.24
2020-09-26 06:23:23+02	2020-09-26 06:24:47+02	multimedia	celluloid	\N	celluloid-0.20
2020-09-26 06:24:47+02	2020-09-26 06:25:18+02	multimedia	libva-utils	\N	libva-utils-2.9.1
2020-09-26 06:25:19+02	2020-09-26 06:27:21+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-09-26 06:27:22+02	2020-09-26 06:29:43+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-09-26 06:29:44+02	2020-09-26 06:32:50+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-26 06:32:51+02	2020-09-26 06:33:23+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-09-26 06:33:23+02	2020-09-26 06:35:47+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-09-26 06:35:48+02	2020-09-26 06:36:14+02	net	rdesktop	\N	rdesktop-1.9.0
2020-09-26 06:36:14+02	2020-09-26 06:38:01+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-26 06:13:09+02	2020-09-26 06:47:38+02	lang	mono	\N	mono-5.10.1.57_3
2020-09-26 06:47:40+02	2020-09-26 06:48:54+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-26 06:48:55+02	2020-09-26 06:50:40+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-09-26 06:50:42+02	2020-09-26 06:51:43+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-09-26 06:51:43+02	2020-09-26 06:52:42+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-09-26 06:52:43+02	2020-09-26 06:53:43+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-09-26 06:53:43+02	2020-09-26 06:54:42+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-09-26 06:54:43+02	2020-09-26 06:55:42+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-09-26 06:55:43+02	2020-09-26 06:56:43+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-09-26 06:56:44+02	2020-09-26 06:57:43+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-09-26 06:57:44+02	2020-09-26 06:59:18+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-09-26 06:59:19+02	2020-09-26 07:00:20+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-09-26 07:00:20+02	2020-09-26 07:01:11+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-09-26 06:38:01+02	2020-09-26 07:01:43+02	net	wireshark	\N	wireshark-3.2.7
2020-09-26 07:01:11+02	2020-09-26 07:01:57+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-09-26 07:01:57+02	2020-09-26 07:02:09+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-09-26 07:01:43+02	2020-09-26 07:02:35+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-09-26 07:02:36+02	2020-09-26 07:03:07+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-09-26 07:03:08+02	2020-09-26 07:03:43+02	print	gv	\N	gv-3.7.4_5
2020-09-26 07:02:10+02	2020-09-26 07:04:40+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-26 07:03:44+02	2020-09-26 07:05:55+02	print	lyx	\N	lyx-2.3.4.2_1
2020-09-26 07:04:41+02	2020-09-26 07:09:25+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-09-26 07:09:26+02	2020-09-26 07:09:33+02	security	base-audit	\N	base-audit-0.4
2020-09-26 07:09:34+02	2020-09-26 07:10:22+02	security	clamav	\N	clamav-0.103.0,1
2020-09-26 07:05:56+02	2020-09-26 07:10:41+02	print	texlive-full	\N	texlive-full-20150521
2020-09-26 07:10:23+02	2020-09-26 07:11:13+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-09-26 07:11:14+02	2020-09-26 07:11:24+02	sysutils	bpytop	\N	bpytop-1.0.37
2020-09-26 07:11:25+02	2020-09-26 07:12:20+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-26 07:10:42+02	2020-09-26 07:12:58+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-09-26 07:12:21+02	2020-09-26 07:13:16+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-26 07:12:59+02	2020-09-26 07:14:18+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-26 07:14:19+02	2020-09-26 07:15:20+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-09-26 07:13:17+02	2020-09-26 07:15:38+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-26 07:15:39+02	2020-09-26 07:17:43+02	www	mattermost-server	\N	mattermost-server-5.26.1
2020-09-26 07:17:43+02	2020-09-26 07:18:17+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-09-26 07:18:18+02	2020-09-26 07:18:32+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-3.1
2020-09-26 07:18:33+02	2020-09-26 07:18:50+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-09-26 07:18:51+02	2020-09-26 07:19:40+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-09-26 07:19:41+02	2020-09-26 07:21:25+02	x11	mate	\N	mate-1.22.2
2020-09-26 07:21:26+02	2020-09-26 07:24:26+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-26 07:24:27+02	2020-09-26 07:30:37+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-26 07:30:39+02	2020-09-26 07:32:50+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-26 07:32:50+02	2020-09-26 07:33:56+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-09-26 07:33:57+02	2020-09-26 08:07:39+02	editors	vscode	\N	vscode-1.46.1
2020-09-26 08:07:41+02	2020-09-26 08:08:46+02	graphics	xfig	\N	xfig-3.2.7a
2020-09-26 08:08:47+02	2020-09-26 08:09:36+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-09-26 07:15:21+02	2020-09-26 08:20:26+02	www	firefox	\N	firefox-81.0_2,2
2020-09-26 06:03:22+02	2020-09-26 09:51:13+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200920
2020-09-26 20:07:59+02	2020-09-26 20:25:21+02	local	E590T-localbase	\N	E590T-localbase-2020092600
2020-09-26 20:25:34+02	2020-09-26 20:26:21+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-26 20:25:34+02	2020-09-26 20:27:50+02	graphics	gpu-firmware-kmod	\N	gpu-firmware-kmod-g20200920
2020-09-26 20:27:51+02	2020-09-26 20:34:03+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-26 20:34:03+02	2020-09-26 20:34:11+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-26 21:29:41+02	2020-09-26 21:29:50+02	devel	py-enum34	py27	py27-enum34-1.1.10
2020-09-26 21:29:50+02	2020-09-26 21:30:09+02	security	py-cryptography	py27	py27-cryptography-2.6.1
2020-09-26 21:30:09+02	2020-09-26 21:30:16+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20200926
2020-09-26 21:29:41+02	2020-09-26 21:30:24+02	textproc	libxml2	\N	libxml2-2.9.10_1
2020-09-26 21:30:17+02	2020-09-26 21:30:29+02	security	py-paramiko	py27	py27-paramiko-2.7.1
2020-09-26 21:29:41+02	2020-09-26 21:30:37+02	graphics	exiv2	\N	exiv2-0.27.3,1
2020-09-26 21:30:24+02	2020-09-26 21:30:38+02	textproc	libxslt	\N	libxslt-1.1.34_1
2020-09-26 21:30:38+02	2020-09-26 21:30:51+02	graphics	wayland	\N	wayland-1.18.0_4
2020-09-26 21:30:37+02	2020-09-26 21:30:51+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10_1
2020-09-26 21:30:51+02	2020-09-26 21:31:01+02	textproc	itstool	\N	itstool-2.0.6
2020-09-26 21:31:02+02	2020-09-26 21:31:35+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-09-26 21:31:35+02	2020-09-26 21:31:46+02	textproc	minixmlto	\N	minixmlto-0.0.2_1
2020-09-26 21:30:51+02	2020-09-26 21:31:55+02	devel	glib20	\N	glib-2.66.0_1,1
2020-09-26 21:31:46+02	2020-09-26 21:32:14+02	x11	xkeyboard-config	\N	xkeyboard-config-2.30
2020-09-26 21:32:15+02	2020-09-26 21:32:38+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-09-26 21:31:56+02	2020-09-26 21:32:46+02	security	p11-kit	\N	p11-kit-0.23.21
2020-09-26 21:32:39+02	2020-09-26 21:32:58+02	x11	libxkbcommon	\N	libxkbcommon-1.0.1
2020-09-26 21:32:59+02	2020-09-26 21:33:27+02	textproc	xmlto	\N	xmlto-0.0.28
2020-09-26 21:33:27+02	2020-09-26 21:34:58+02	misc	shared-mime-info	\N	shared-mime-info-2.0
2020-10-02 00:51:59+02	2020-10-02 00:53:05+02	print	cups	\N	cups-2.3.3_1
2020-09-26 21:32:47+02	2020-09-26 21:35:43+02	security	gnutls	\N	gnutls-3.6.15
2020-09-26 21:35:44+02	2020-09-26 21:36:54+02	lang	vala	\N	vala-0.48.10,1
2020-09-26 21:36:55+02	2020-09-26 21:37:20+02	textproc	libcroco	\N	libcroco-0.6.13
2020-09-26 21:37:21+02	2020-09-26 21:39:20+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-09-26 21:39:21+02	2020-09-26 21:40:22+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-09-26 21:40:22+02	2020-09-26 21:41:24+02	net	qt5-network	\N	qt5-network-5.15.0
2020-09-26 21:41:24+02	2020-09-26 21:42:15+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-09-26 21:42:15+02	2020-09-26 21:42:30+02	devel	desktop-file-utils	\N	desktop-file-utils-0.26
2020-09-26 21:42:30+02	2020-09-26 21:43:12+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-09-26 21:43:13+02	2020-09-26 21:43:37+02	devel	dconf	\N	dconf-0.38.0
2020-09-26 21:43:37+02	2020-09-26 21:44:18+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-09-26 21:44:19+02	2020-09-26 21:44:59+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-09-26 21:44:59+02	2020-09-26 21:45:33+02	multimedia	libva	\N	libva-2.9.0
2020-09-26 21:45:33+02	2020-09-26 21:45:55+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-09-26 21:45:56+02	2020-09-26 21:47:29+02	lang	php74	\N	php74-7.4.10
2020-09-26 21:47:29+02	2020-09-26 21:47:44+02	devel	libIDL	\N	libIDL-0.8.14_4
2020-09-26 21:47:44+02	2020-09-26 21:48:08+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-09-26 21:48:08+02	2020-09-26 21:48:20+02	audio	lv2	\N	lv2-1.18.0
2020-09-26 21:48:21+02	2020-09-26 21:48:33+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-09-26 21:48:34+02	2020-09-26 21:49:08+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-09-26 21:49:09+02	2020-09-26 21:49:41+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-09-26 21:49:41+02	2020-09-26 21:49:59+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-09-26 21:49:59+02	2020-09-26 21:50:14+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-09-26 21:50:14+02	2020-09-26 21:50:39+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-09-26 21:50:39+02	2020-09-26 21:50:57+02	www	php74-session	\N	php74-session-7.4.10
2020-09-26 21:50:58+02	2020-09-26 21:51:16+02	archivers	php74-zlib	\N	php74-zlib-7.4.10
2020-09-26 21:51:17+02	2020-09-26 21:51:35+02	databases	php74-pdo	\N	php74-pdo-7.4.10
2020-09-26 21:51:36+02	2020-09-26 21:51:54+02	devel	php74-json	\N	php74-json-7.4.10
2020-09-26 21:30:29+02	2020-09-26 21:52:22+02	devel	llvm80	\N	llvm80-8.0.1_4
2020-09-26 21:51:54+02	2020-09-26 21:52:25+02	sysutils	policykit	\N	policykit-0.9_10
2020-09-26 21:52:26+02	2020-09-26 21:52:49+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8
2020-09-26 21:52:50+02	2020-09-26 21:53:17+02	converters	php74-mbstring	\N	php74-mbstring-7.4.10
2020-09-26 21:52:23+02	2020-09-26 21:53:39+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_3
2020-09-26 21:53:18+02	2020-09-26 21:53:44+02	graphics	php74-gd	\N	php74-gd-7.4.10
2020-09-26 21:53:44+02	2020-09-26 21:53:58+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-09-26 21:53:59+02	2020-09-26 21:54:16+02	graphics	libGLU	\N	libGLU-9.0.1
2020-09-26 21:53:39+02	2020-09-26 21:54:19+02	graphics	cairo	\N	cairo-1.16.0_1,3
2020-09-26 21:54:17+02	2020-09-26 21:54:30+02	www	libwpe	\N	libwpe-1.4.0.1
2020-09-26 21:54:30+02	2020-09-26 21:54:55+02	graphics	poppler	\N	poppler-20.09.0
2020-09-26 21:54:19+02	2020-09-26 21:55:01+02	devel	gobject-introspection	\N	gobject-introspection-1.56.1_1,1
2020-09-26 21:54:56+02	2020-09-26 21:55:36+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-09-26 21:34:59+02	2020-09-26 21:55:59+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-09-26 21:55:37+02	2020-09-26 21:56:13+02	net	avahi-app	\N	avahi-app-0.7_3
2020-09-26 21:55:01+02	2020-09-26 21:56:15+02	print	harfbuzz	\N	harfbuzz-2.7.2_1
2020-09-26 21:56:00+02	2020-09-26 21:56:20+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-09-26 21:56:14+02	2020-09-26 21:56:46+02	print	cups	\N	cups-2.3.3_1
2020-09-26 21:56:20+02	2020-09-26 21:56:51+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-09-26 21:56:46+02	2020-09-26 21:57:03+02	accessibility	atk	\N	atk-2.36.0
2020-09-26 21:56:51+02	2020-09-26 21:57:10+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-09-26 21:57:10+02	2020-09-26 21:57:40+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-09-26 21:57:04+02	2020-09-26 21:57:41+02	sysutils	polkit	\N	polkit-0.118
2020-09-26 21:57:41+02	2020-09-26 21:57:54+02	devel	libgudev	\N	libgudev-233
2020-09-26 21:56:16+02	2020-09-26 21:57:55+02	x11-toolkits	pango	\N	pango-1.42.4_5
2020-09-26 21:57:55+02	2020-09-26 21:58:07+02	x11	libwacom	\N	libwacom-1.4.1
2020-09-26 21:58:08+02	2020-09-26 21:58:21+02	x11	libinput	\N	libinput-1.15.6
2020-09-26 21:57:56+02	2020-09-26 21:58:30+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-09-26 21:57:42+02	2020-09-26 21:58:40+02	graphics	colord	\N	colord-1.3.5
2020-09-26 21:58:22+02	2020-09-26 21:58:41+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-09-26 21:58:41+02	2020-09-26 21:59:35+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-09-26 21:58:30+02	2020-09-26 21:59:50+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.28.0
2020-09-26 21:59:35+02	2020-09-26 22:00:07+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.2
2020-09-26 21:58:41+02	2020-09-26 22:00:28+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0
2020-09-26 22:00:29+02	2020-09-26 22:02:37+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-09-26 22:00:07+02	2020-09-26 22:03:18+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-26 22:02:39+02	2020-09-26 22:03:37+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-09-26 21:59:50+02	2020-09-26 22:04:10+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-09-26 22:04:11+02	2020-09-26 22:04:39+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.28.1
2020-09-26 22:03:38+02	2020-09-26 22:04:50+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2
2020-09-26 22:04:39+02	2020-09-26 22:05:00+02	net	glib-networking	\N	glib-networking-2.56.1_3
2020-09-26 22:04:51+02	2020-09-26 22:05:15+02	devel	json-glib	\N	json-glib-1.6.0
2020-09-26 22:05:02+02	2020-09-26 22:05:43+02	devel	libsoup	\N	libsoup-2.62.3
2020-09-26 22:03:18+02	2020-09-26 22:05:50+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-09-26 22:05:16+02	2020-09-26 22:05:54+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-09-26 22:05:55+02	2020-09-26 22:06:15+02	devel	libnotify	\N	libnotify-0.7.9
2020-09-26 22:06:15+02	2020-09-26 22:06:48+02	security	libsecret	\N	libsecret-0.18.6_1
2020-09-26 22:05:51+02	2020-09-26 22:06:51+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-09-26 22:06:49+02	2020-09-26 22:07:29+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-09-26 22:07:30+02	2020-09-26 22:07:42+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-09-26 22:05:43+02	2020-09-26 22:08:13+02	print	texlive-base	\N	texlive-base-20150521_55
2020-09-26 22:07:43+02	2020-09-26 22:08:37+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-26 22:08:38+02	2020-09-26 22:08:53+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-09-26 22:06:51+02	2020-09-26 22:08:57+02	graphics	graphviz	\N	graphviz-2.44.1_2
2020-09-26 22:08:53+02	2020-09-26 22:09:31+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-09-26 22:09:32+02	2020-09-26 22:09:48+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-26 22:08:57+02	2020-09-26 22:09:51+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-26 22:09:49+02	2020-09-26 22:10:06+02	graphics	frei0r	\N	frei0r-1.6.1
2020-09-26 22:09:52+02	2020-09-26 22:10:57+02	graphics	graphene	\N	graphene-1.10.2
2020-09-26 22:10:58+02	2020-09-26 22:11:40+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-26 22:11:41+02	2020-09-26 22:12:19+02	net	geoclue	\N	geoclue-2.5.5
2020-09-26 22:12:20+02	2020-09-26 22:12:36+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-09-26 22:10:06+02	2020-09-26 22:16:27+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-09-26 22:16:27+02	2020-09-26 22:17:20+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.74.0
2020-09-26 22:17:20+02	2020-09-26 22:17:58+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-09-26 22:17:58+02	2020-09-26 22:18:27+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-09-26 22:12:36+02	2020-09-26 22:18:40+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-26 22:18:28+02	2020-09-26 22:18:51+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-09-26 22:18:41+02	2020-09-26 22:19:05+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-26 22:19:05+02	2020-09-26 22:19:17+02	print	libraqm	\N	libraqm-0.6.0
2020-09-26 22:19:18+02	2020-09-26 22:19:37+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-09-26 22:18:52+02	2020-09-26 22:19:44+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-09-26 22:19:45+02	2020-09-26 22:20:22+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-09-26 22:19:37+02	2020-09-26 22:22:07+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-26 22:20:23+02	2020-09-26 22:22:29+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-09-26 22:22:29+02	2020-09-26 22:22:57+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.74.0
2020-09-26 22:22:57+02	2020-09-26 22:23:56+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-26 22:23:56+02	2020-09-26 22:24:22+02	security	php74-openssl	\N	php74-openssl-7.4.10
2020-09-26 22:24:23+02	2020-09-26 22:24:47+02	textproc	php74-xml	\N	php74-xml-7.4.10
2020-09-26 22:08:14+02	2020-09-26 22:25:12+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-26 22:24:48+02	2020-09-26 22:25:31+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-26 22:22:08+02	2020-09-26 22:27:03+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-26 22:27:04+02	2020-09-26 22:27:22+02	converters	php74-iconv	\N	php74-iconv-7.4.10
2020-09-26 22:27:22+02	2020-09-26 22:27:39+02	databases	pecl-memcache	php74	php74-pecl-memcache-4.0.5.2
2020-09-26 22:25:13+02	2020-09-26 22:27:53+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-26 22:27:40+02	2020-09-26 22:27:59+02	databases	pecl-memcached	php74	php74-pecl-memcached-3.1.5
2020-09-26 22:27:53+02	2020-09-26 22:28:12+02	databases	php74-pdo_pgsql	\N	php74-pdo_pgsql-7.4.10
2020-09-26 22:27:59+02	2020-09-26 22:28:18+02	databases	php74-pdo_sqlite	\N	php74-pdo_sqlite-7.4.10
2020-09-26 22:28:12+02	2020-09-26 22:28:32+02	databases	php74-pgsql	\N	php74-pgsql-7.4.10
2020-09-26 22:28:18+02	2020-09-26 22:28:38+02	databases	php74-sqlite3	\N	php74-sqlite3-7.4.10
2020-09-26 22:28:33+02	2020-09-26 22:28:51+02	devel	php74-gettext	\N	php74-gettext-7.4.10
2020-09-26 22:28:39+02	2020-09-26 22:29:24+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-26 22:28:51+02	2020-09-26 22:29:28+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-09-26 22:29:25+02	2020-09-26 22:29:43+02	ftp	php74-curl	\N	php74-curl-7.4.10
2020-09-26 22:29:44+02	2020-09-26 22:29:59+02	math	pecl-bitset	php74	php74-pecl-bitset-3.0.1
2020-09-26 22:29:28+02	2020-09-26 22:30:08+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-09-26 22:30:00+02	2020-09-26 22:30:19+02	math	php74-bcmath	\N	php74-bcmath-7.4.10
2020-09-26 22:30:19+02	2020-09-26 22:30:37+02	net	php74-soap	\N	php74-soap-7.4.10
2020-09-26 22:25:32+02	2020-09-26 22:30:40+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-26 22:30:37+02	2020-09-26 22:30:53+02	net	php74-sockets	\N	php74-sockets-7.4.10
2020-09-26 22:30:54+02	2020-09-26 22:31:08+02	security	pecl-mcrypt	php74	php74-pecl-mcrypt-1.0.3
2020-09-26 22:31:09+02	2020-09-26 22:31:35+02	sysutils	hal	\N	hal-0.5.14_34
2020-09-26 22:31:35+02	2020-09-26 22:31:52+02	sysutils	php74-posix	\N	php74-posix-7.4.10
2020-09-26 22:31:52+02	2020-09-26 22:32:08+02	textproc	php74-ctype	\N	php74-ctype-7.4.10
2020-09-26 22:32:08+02	2020-09-26 22:32:27+02	textproc	php74-dom	\N	php74-dom-7.4.10
2020-09-26 22:32:27+02	2020-09-26 22:32:43+02	textproc	php74-simplexml	\N	php74-simplexml-7.4.10
2020-09-26 22:30:09+02	2020-09-26 22:32:46+02	net	php74-ldap	\N	php74-ldap-7.4.10
2020-09-26 22:30:40+02	2020-09-26 22:33:04+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-26 22:32:44+02	2020-09-26 22:33:06+02	www	php74-opcache	\N	php74-opcache-7.4.10
2020-09-26 22:33:07+02	2020-09-26 22:33:24+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-09-26 22:32:46+02	2020-09-26 22:33:38+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-26 22:33:05+02	2020-09-26 22:33:46+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-26 22:33:46+02	2020-09-26 22:33:58+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-09-26 22:33:24+02	2020-09-26 22:34:05+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-09-26 22:33:59+02	2020-09-26 22:34:20+02	archivers	kf5-karchive	\N	kf5-karchive-5.74.0
2020-09-26 22:33:39+02	2020-09-26 22:34:31+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-09-26 22:34:21+02	2020-09-26 22:34:36+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-09-26 22:34:06+02	2020-09-26 22:34:57+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-09-26 22:34:32+02	2020-09-26 22:35:11+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-09-26 22:34:57+02	2020-09-26 22:35:50+02	x11-toolkits	vte3	\N	vte3-0.62.0
2020-09-26 22:34:36+02	2020-09-26 22:36:06+02	x11-servers	xorg-server	\N	xorg-server-1.20.9,1
2020-09-26 22:35:12+02	2020-09-26 22:36:11+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-09-26 22:36:07+02	2020-09-26 22:36:35+02	devel	sdl20	\N	sdl2-2.0.12_1
2020-09-26 22:36:12+02	2020-09-26 22:36:51+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-09-26 22:35:51+02	2020-09-26 22:37:12+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-09-26 22:36:36+02	2020-09-26 22:37:23+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-09-26 22:36:51+02	2020-09-26 22:37:31+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-26 22:37:13+02	2020-09-26 22:37:49+02	devel	glibmm	\N	glibmm-2.56.0_2,1
2020-09-26 22:37:23+02	2020-09-26 22:37:50+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-09-26 22:37:49+02	2020-09-26 22:38:54+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-09-26 22:37:32+02	2020-09-26 22:39:01+02	security	gcr	\N	gcr-3.28.0_1
2020-09-26 22:37:50+02	2020-09-26 22:39:06+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-09-26 22:38:54+02	2020-09-26 22:39:15+02	devel	hwloc	\N	hwloc-1.11.11
2020-09-26 22:39:15+02	2020-09-26 22:39:25+02	graphics	glew	\N	glew-2.2.0
2020-09-26 22:39:02+02	2020-09-26 22:39:41+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-09-26 22:39:07+02	2020-09-26 22:39:44+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-09-26 22:39:45+02	2020-09-26 22:39:57+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-09-26 22:39:42+02	2020-09-26 22:40:02+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-09-26 22:39:25+02	2020-09-26 22:40:03+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-09-26 22:40:04+02	2020-09-26 22:40:25+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.74.0
2020-09-26 22:40:02+02	2020-09-26 22:40:41+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-26 22:39:58+02	2020-09-26 22:40:43+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-26 22:40:42+02	2020-09-26 22:41:08+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-09-26 22:40:44+02	2020-09-26 22:41:31+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-09-26 22:41:09+02	2020-09-26 22:41:55+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-09-26 22:41:32+02	2020-09-26 22:42:23+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-09-26 22:40:26+02	2020-09-26 22:42:27+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-26 22:41:56+02	2020-09-26 22:42:41+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-26 22:42:28+02	2020-09-26 22:42:41+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-09-26 22:42:23+02	2020-09-26 22:42:53+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-09-26 22:42:53+02	2020-09-26 22:43:14+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-09-26 22:42:42+02	2020-09-26 22:43:27+02	net	remmina	\N	remmina-1.3.9
2020-09-26 22:43:15+02	2020-09-26 22:43:40+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-09-26 22:43:41+02	2020-09-26 22:43:57+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-09-26 22:43:27+02	2020-09-26 22:44:09+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-09-26 22:42:42+02	2020-09-26 22:44:11+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-09-26 22:43:57+02	2020-09-26 22:44:20+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-09-26 22:44:09+02	2020-09-26 22:44:33+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-09-26 22:44:11+02	2020-09-26 22:44:39+02	devel	libgtop	\N	libgtop-2.40.0
2020-09-26 22:44:34+02	2020-09-26 22:44:55+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-09-26 22:44:40+02	2020-09-26 22:45:03+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-09-26 22:44:55+02	2020-09-26 22:45:34+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-26 22:44:20+02	2020-09-26 22:45:38+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-26 22:45:03+02	2020-09-26 22:45:40+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-09-26 22:45:35+02	2020-09-26 22:45:51+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-09-26 22:45:41+02	2020-09-26 22:46:05+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-09-26 22:45:39+02	2020-09-26 22:46:10+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-09-26 22:46:11+02	2020-09-26 22:46:59+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-26 22:45:52+02	2020-09-26 22:47:03+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-09-26 22:47:04+02	2020-09-26 22:47:46+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-09-26 22:47:00+02	2020-09-26 22:48:29+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.74.0
2020-09-26 22:47:47+02	2020-09-26 22:48:43+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-09-26 22:48:30+02	2020-09-26 22:49:17+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-09-26 22:48:43+02	2020-09-26 22:49:46+02	x11-toolkits	libwnck3	\N	libwnck3-3.24.1
2020-09-26 22:49:46+02	2020-09-26 22:50:13+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-09-26 22:50:14+02	2020-09-26 22:50:39+02	devel	gamin	\N	gamin-0.1.10_10
2020-09-26 22:49:18+02	2020-09-26 22:50:53+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-26 22:50:39+02	2020-09-26 22:51:12+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-09-26 22:46:06+02	2020-09-26 22:51:35+02	net	openmpi	\N	openmpi-4.0.5
2020-09-26 22:51:12+02	2020-09-26 22:51:42+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-26 22:51:43+02	2020-09-26 22:51:54+02	print	gl2ps	\N	gl2ps-1.4.0
2020-09-26 22:50:54+02	2020-09-26 22:52:27+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-26 22:51:55+02	2020-09-26 22:52:33+02	security	gnupg	\N	gnupg-2.2.23
2020-09-26 22:52:33+02	2020-09-26 22:52:45+02	textproc	yelp-xsl	\N	yelp-xsl-3.38.0
2020-09-26 22:52:27+02	2020-09-26 22:52:50+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-09-26 22:52:46+02	2020-09-26 22:53:35+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-09-26 22:53:36+02	2020-09-26 22:54:11+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-09-26 22:52:50+02	2020-09-26 22:54:32+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-09-26 22:54:12+02	2020-09-26 22:54:51+02	devel	librest	\N	rest-0.7.93
2020-09-26 22:54:33+02	2020-09-26 22:55:16+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-09-26 22:54:52+02	2020-09-26 22:55:42+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-09-26 22:51:36+02	2020-09-26 22:56:23+02	math	vtk8	\N	vtk8-8.2.0
2020-09-26 22:55:17+02	2020-09-26 22:56:46+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-09-26 22:56:46+02	2020-09-26 22:57:05+02	security	gpgme	\N	gpgme-1.14.0
2020-09-26 22:57:06+02	2020-09-26 22:57:52+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-09-26 22:57:53+02	2020-09-26 22:59:01+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-09-26 22:55:42+02	2020-09-26 22:59:06+02	net	libmateweather	\N	libmateweather-1.24.1
2020-09-26 22:59:06+02	2020-09-26 22:59:56+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-26 22:59:57+02	2020-09-26 23:00:48+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-09-26 23:00:48+02	2020-09-26 23:01:10+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-09-26 22:56:24+02	2020-09-26 23:01:20+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-26 23:01:11+02	2020-09-26 23:02:04+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-09-26 23:01:21+02	2020-09-26 23:02:05+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-26 23:02:05+02	2020-09-26 23:03:15+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-09-26 23:02:05+02	2020-09-26 23:03:36+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-09-26 23:03:16+02	2020-09-26 23:04:34+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-26 22:59:01+02	2020-09-26 23:04:35+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-26 23:04:37+02	2020-09-26 23:05:19+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-26 23:04:35+02	2020-09-26 23:05:20+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-09-26 23:05:20+02	2020-09-26 23:05:36+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-09-26 23:05:37+02	2020-09-26 23:06:02+02	sysutils	upower	\N	upower-0.99.11
2020-09-26 23:05:20+02	2020-09-26 23:06:02+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-26 23:06:02+02	2020-09-26 23:06:25+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-09-26 23:06:25+02	2020-09-26 23:07:29+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-09-26 23:07:30+02	2020-09-26 23:07:51+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-09-26 23:07:52+02	2020-09-26 23:10:55+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-26 23:10:55+02	2020-09-26 23:11:34+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-09-26 23:11:35+02	2020-09-26 23:13:35+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_5
2020-09-26 23:03:36+02	2020-09-26 23:14:30+02	net	samba410	\N	samba410-4.10.18
2020-09-26 23:13:36+02	2020-09-26 23:14:37+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-09-26 23:14:30+02	2020-09-26 23:15:42+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0
2020-09-26 23:14:38+02	2020-09-26 23:15:45+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-09-26 23:15:43+02	2020-09-26 23:16:43+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-09-26 23:15:45+02	2020-09-26 23:17:15+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-09-26 23:16:44+02	2020-09-26 23:17:44+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-09-26 23:17:16+02	2020-09-26 23:18:16+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.5.0
2020-09-26 23:17:45+02	2020-09-26 23:18:36+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-09-26 23:18:36+02	2020-09-26 23:19:02+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-09-26 23:18:17+02	2020-09-26 23:19:10+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-09-26 23:19:11+02	2020-09-26 23:19:35+02	archivers	gcab	\N	gcab-1.4
2020-09-26 23:19:36+02	2020-09-26 23:22:12+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-09-26 23:06:03+02	2020-09-26 23:23:57+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_3
2020-09-26 23:22:14+02	2020-09-26 23:24:25+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-09-26 23:19:02+02	2020-09-26 23:24:32+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-26 23:23:58+02	2020-09-26 23:25:20+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-26 23:24:26+02	2020-09-26 23:25:22+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-09-26 23:25:20+02	2020-09-26 23:25:45+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-09-26 23:25:46+02	2020-09-26 23:25:56+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-09-26 23:25:57+02	2020-09-26 23:26:41+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-09-26 23:26:41+02	2020-09-26 23:27:36+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-26 23:25:22+02	2020-09-26 23:27:40+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-09-26 23:27:41+02	2020-09-26 23:28:12+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-09-26 23:27:37+02	2020-09-26 23:28:24+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-09-26 23:28:25+02	2020-09-26 23:29:02+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-09-26 23:28:13+02	2020-09-26 23:29:08+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-26 23:29:03+02	2020-09-26 23:29:24+02	x11	babl	\N	babl-0.1.78_1
2020-09-26 23:29:24+02	2020-09-26 23:29:51+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-09-26 23:29:09+02	2020-09-26 23:30:18+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-26 23:29:51+02	2020-09-26 23:30:54+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-26 23:30:19+02	2020-09-26 23:32:08+02	x11-fm	caja	\N	caja-1.24.0
2020-09-26 23:30:55+02	2020-09-26 23:32:46+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.9,1
2020-09-26 23:32:09+02	2020-09-26 23:33:59+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-26 23:32:46+02	2020-09-26 23:34:09+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-09-26 23:34:00+02	2020-09-26 23:34:58+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-26 23:34:10+02	2020-09-26 23:35:29+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-26 23:34:59+02	2020-09-26 23:36:15+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-26 23:35:31+02	2020-09-26 23:37:25+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-26 23:36:15+02	2020-09-26 23:37:33+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-09-26 23:37:34+02	2020-09-26 23:37:53+02	devel	liblangtag	\N	liblangtag-0.6.2
2020-09-26 23:37:25+02	2020-09-26 23:38:23+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-09-26 23:37:53+02	2020-09-26 23:38:40+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-09-26 23:38:40+02	2020-09-26 23:38:54+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-09-26 23:24:32+02	2020-09-26 23:39:05+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-26 23:38:55+02	2020-09-26 23:39:23+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-09-26 23:39:06+02	2020-09-26 23:39:26+02	graphics	py-cairo	py27	py27-cairo-1.18.1_1
2020-09-26 23:38:24+02	2020-09-26 23:39:31+02	graphics	gegl	\N	gegl-0.4.26_5
2020-09-26 23:39:31+02	2020-09-26 23:39:44+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-09-26 23:39:26+02	2020-09-26 23:41:07+02	mail	mailutils	\N	mailutils-3.10
2020-09-26 23:39:45+02	2020-09-26 23:41:17+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-26 23:41:07+02	2020-09-26 23:41:36+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-09-26 23:41:37+02	2020-09-26 23:42:11+02	textproc	libvisio01	\N	libvisio01-0.1.7_2
2020-09-26 23:42:12+02	2020-09-26 23:42:24+02	textproc	yelp-tools	\N	yelp-tools-3.38.0
2020-09-26 23:41:18+02	2020-09-26 23:42:26+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-09-26 23:42:25+02	2020-09-26 23:42:49+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-09-26 23:39:23+02	2020-09-26 23:43:19+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-09-26 23:42:27+02	2020-09-26 23:43:19+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-09-26 23:43:19+02	2020-09-26 23:44:02+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-09-26 23:43:19+02	2020-09-26 23:44:11+02	x11	zenity	\N	zenity-3.32.0
2020-09-26 23:44:11+02	2020-09-26 23:44:58+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-26 23:44:59+02	2020-09-26 23:45:22+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-09-26 23:42:51+02	2020-09-26 23:45:24+02	x11	xorg	\N	xorg-7.7_3
2020-09-26 23:45:22+02	2020-09-26 23:46:17+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-09-26 23:45:24+02	2020-09-26 23:46:26+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-09-26 23:44:03+02	2020-09-26 23:46:43+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-09-26 23:46:18+02	2020-09-26 23:47:05+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-09-26 23:46:44+02	2020-09-26 23:47:37+02	dns	bind-tools	\N	bind-tools-9.16.7
2020-09-26 23:46:28+02	2020-09-26 23:47:43+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-09-26 23:47:43+02	2020-09-26 23:47:59+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-09-26 23:47:06+02	2020-09-26 23:50:35+02	editors	emacs	full	emacs-27.1_1,3
2020-09-26 23:50:36+02	2020-09-26 23:50:54+02	graphics	glfw	\N	glfw-3.3.2
2020-09-26 23:47:37+02	2020-09-26 23:50:58+02	graphics	Coin	\N	Coin-4.0.0_1
2020-09-26 23:50:55+02	2020-09-26 23:51:28+02	graphics	movit	\N	movit-1.6.2_2
2020-09-26 23:51:28+02	2020-09-26 23:51:41+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-09-26 23:51:42+02	2020-09-26 23:51:55+02	graphics	vulkan-loader	\N	vulkan-loader-1.2.135.0
2020-09-26 23:51:56+02	2020-09-26 23:52:40+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-26 23:48:00+02	2020-09-26 23:52:53+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-26 23:52:40+02	2020-09-26 23:52:57+02	net	gssdp	\N	gssdp-1.2.3
2020-09-26 23:52:57+02	2020-09-26 23:53:33+02	print	cups-filters	\N	cups-filters-1.28.3
2020-09-26 23:52:53+02	2020-09-26 23:53:36+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-09-26 23:50:59+02	2020-09-26 23:54:23+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-26 23:54:24+02	2020-09-26 23:55:45+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-26 23:53:34+02	2020-09-26 23:55:56+02	print	fontforge	\N	fontforge-20190801_2
2020-09-26 23:55:45+02	2020-09-26 23:56:10+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-09-26 23:55:57+02	2020-09-26 23:57:42+02	www	apache24	\N	apache24-2.4.46
2020-09-26 23:57:43+02	2020-09-26 23:58:34+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-26 23:58:35+02	2020-09-26 23:59:35+02	x11	yelp	\N	yelp-3.38.0
2020-09-26 23:53:37+02	2020-09-26 23:59:51+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-26 23:59:35+02	2020-09-27 00:00:35+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-26 23:59:52+02	2020-09-27 00:00:49+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-27 00:00:36+02	2020-09-27 00:01:56+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-27 00:00:50+02	2020-09-27 00:02:32+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-27 00:01:56+02	2020-09-27 00:03:20+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-27 00:02:32+02	2020-09-27 00:03:23+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0
2020-09-27 00:03:21+02	2020-09-27 00:04:43+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-09-27 00:03:24+02	2020-09-27 00:05:19+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-09-27 00:05:20+02	2020-09-27 00:06:46+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-27 00:04:44+02	2020-09-27 00:07:29+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-27 00:06:47+02	2020-09-27 00:08:14+02	x11-wm	marco	\N	marco-1.22.4
2020-09-27 00:07:30+02	2020-09-27 00:08:30+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-27 00:08:15+02	2020-09-27 00:08:40+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-09-27 00:08:40+02	2020-09-27 00:09:43+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-27 00:08:30+02	2020-09-27 00:10:26+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-27 00:09:44+02	2020-09-27 00:12:52+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-27 00:10:27+02	2020-09-27 00:13:58+02	devel	ispc	\N	ispc-1.14.1_2
2020-09-27 00:13:59+02	2020-09-27 00:18:12+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-27 00:18:13+02	2020-09-27 00:21:11+02	graphics	dcmtk	\N	dcmtk-3.6.5_2
2020-09-27 00:21:15+02	2020-09-27 00:22:06+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-09-27 00:22:08+02	2020-09-27 00:22:36+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-09-27 00:12:53+02	2020-09-27 00:24:04+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-27 00:22:37+02	2020-09-27 00:24:08+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-27 00:24:10+02	2020-09-27 00:24:42+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-09-27 00:24:05+02	2020-09-27 00:24:49+02	graphics	libetonyek01	\N	libetonyek01-0.1.9_6,1
2020-09-27 00:24:42+02	2020-09-27 00:25:06+02	graphics	libgxps	\N	libgxps-0.3.1
2020-09-27 00:24:49+02	2020-09-27 00:25:29+02	graphics	libheif	\N	libheif-1.6.2_1
2020-09-27 00:25:08+02	2020-09-27 00:25:30+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-09-27 00:25:32+02	2020-09-27 00:26:02+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-09-26 23:56:11+02	2020-09-27 00:26:27+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-27 00:25:30+02	2020-09-27 00:26:29+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-09-27 00:26:04+02	2020-09-27 00:26:53+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-27 00:26:31+02	2020-09-27 00:27:10+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-09-27 00:26:31+02	2020-09-27 00:27:10+02	graphics	sane-backends	\N	sane-backends-1.0.31_1
2020-09-27 00:27:10+02	2020-09-27 00:27:41+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-09-27 00:27:41+02	2020-09-27 00:28:29+02	multimedia	libass	\N	libass-0.14.0
2020-09-27 00:28:30+02	2020-09-27 00:29:42+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-27 00:29:43+02	2020-09-27 00:30:44+02	net	freerdp	\N	freerdp-2.2.0
2020-09-27 00:30:45+02	2020-09-27 00:31:09+02	net	gupnp	\N	gupnp-1.2.4
2020-09-27 00:31:09+02	2020-09-27 00:32:12+02	net	libcmis	\N	libcmis-0.5.2_1
2020-09-27 00:32:13+02	2020-09-27 00:32:51+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-27 00:27:10+02	2020-09-27 00:32:55+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-27 00:32:56+02	2020-09-27 00:33:43+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-09-27 00:32:52+02	2020-09-27 00:34:03+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-09-27 00:34:04+02	2020-09-27 00:34:22+02	print	libspectre	\N	libspectre-0.2.9
2020-09-27 00:34:22+02	2020-09-27 00:36:17+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-27 00:33:44+02	2020-09-27 00:36:37+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-27 00:26:53+02	2020-09-27 00:38:45+02	math	R	\N	R-4.0.2
2020-09-27 00:38:45+02	2020-09-27 00:39:03+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-09-27 00:39:04+02	2020-09-27 00:39:19+02	sysutils	hal-info	\N	hal-info-20091130
2020-09-27 00:36:18+02	2020-09-27 00:41:06+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-27 00:36:38+02	2020-09-27 00:41:19+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-27 00:39:19+02	2020-09-27 00:41:27+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-09-27 00:41:07+02	2020-09-27 00:41:49+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-27 00:41:19+02	2020-09-27 00:42:03+02	sysutils	tracker	\N	tracker-2.3.4
2020-09-27 00:41:49+02	2020-09-27 00:42:20+02	textproc	libabw	\N	libabw-0.1.3
2020-09-27 00:42:20+02	2020-09-27 00:42:46+02	textproc	redland	\N	redland-1.0.17_4
2020-09-27 00:41:28+02	2020-09-27 00:42:55+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-27 00:42:47+02	2020-09-27 00:42:57+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-09-27 00:42:55+02	2020-09-27 00:43:28+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-27 00:42:57+02	2020-09-27 00:43:47+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-27 00:42:03+02	2020-09-27 00:43:52+02	textproc	libe-book	\N	libe-book-0.1.3_18
2020-09-27 00:43:28+02	2020-09-27 00:43:55+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-09-27 00:43:56+02	2020-09-27 00:44:22+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-09-27 00:43:48+02	2020-09-27 00:44:47+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-27 00:43:53+02	2020-09-27 00:45:04+02	x11-servers	xephyr	\N	xephyr-1.20.9,1
2020-09-27 00:44:22+02	2020-09-27 00:45:15+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-09-27 00:44:47+02	2020-09-27 00:45:27+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-27 00:45:05+02	2020-09-27 00:45:47+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-27 00:45:15+02	2020-09-27 00:46:00+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-09-27 00:45:27+02	2020-09-27 00:46:18+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-27 00:46:19+02	2020-09-27 00:46:39+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-09-27 00:46:39+02	2020-09-27 00:46:59+02	audio	faudio	\N	FAudio-20.09
2020-09-27 00:46:00+02	2020-09-27 00:47:51+02	archivers	engrampa	\N	engrampa-1.24.1
2020-09-27 00:47:00+02	2020-09-27 00:48:11+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-27 00:47:51+02	2020-09-27 00:48:47+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-09-27 00:48:12+02	2020-09-27 00:49:20+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-09-27 00:45:47+02	2020-09-27 00:50:05+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-27 00:49:20+02	2020-09-27 00:50:12+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-27 00:50:12+02	2020-09-27 00:50:30+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-09-27 00:50:31+02	2020-09-27 00:50:45+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-09-27 00:50:06+02	2020-09-27 00:50:56+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-27 00:50:56+02	2020-09-27 00:51:16+02	devel	ode	\N	ode-0.13_4,1
2020-09-27 00:48:47+02	2020-09-27 00:51:16+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-27 00:50:45+02	2020-09-27 00:51:25+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-09-27 00:51:16+02	2020-09-27 00:51:31+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-09-27 00:51:16+02	2020-09-27 00:51:48+02	devel	py-game	py37	py37-game-1.9.6_1
2020-09-27 00:51:26+02	2020-09-27 00:52:08+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-09-27 00:51:31+02	2020-09-27 00:52:40+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-27 00:52:40+02	2020-09-27 00:55:30+02	editors	pluma	\N	pluma-1.24.1
2020-09-27 00:55:30+02	2020-09-27 00:55:44+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-09-27 00:55:44+02	2020-09-27 00:56:22+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-09-27 00:56:22+02	2020-09-27 00:58:11+02	graphics	atril	\N	atril-1.22.3_9
2020-09-27 00:58:12+02	2020-09-27 00:59:57+02	graphics	eom	\N	eom-1.24.1
2020-09-27 00:59:58+02	2020-09-27 01:01:36+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-27 01:01:36+02	2020-09-27 01:01:53+02	graphics	gle	\N	gle-3.1.0_7
2020-09-27 01:01:54+02	2020-09-27 01:02:11+02	graphics	gltt	\N	gltt-2.5.2_14
2020-09-27 01:02:11+02	2020-09-27 01:03:06+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-27 01:03:07+02	2020-09-27 01:04:35+02	graphics	opencollada	\N	opencollada-1.6.68_2
2020-09-27 01:04:36+02	2020-09-27 01:05:28+02	graphics	openimageio	\N	openimageio-2.2.6.1
2020-09-27 01:05:28+02	2020-09-27 01:07:36+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-09-27 01:07:37+02	2020-09-27 01:08:42+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-27 00:51:48+02	2020-09-27 01:09:59+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-27 01:10:00+02	2020-09-27 01:10:26+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-09-27 01:08:42+02	2020-09-27 01:10:36+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-09-27 00:52:09+02	2020-09-27 01:11:52+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-09-27 01:11:53+02	2020-09-27 01:13:24+02	math	mate-calc	\N	mate-calc-1.24.1
2020-09-27 01:13:24+02	2020-09-27 01:14:13+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-09-27 01:10:36+02	2020-09-27 01:15:53+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-27 01:10:26+02	2020-09-27 01:17:19+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-27 01:14:14+02	2020-09-27 01:18:08+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-09-27 01:17:20+02	2020-09-27 01:18:34+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-09-27 01:18:08+02	2020-09-27 01:18:37+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-09-27 01:15:54+02	2020-09-27 01:19:59+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-09-27 01:18:35+02	2020-09-27 01:20:06+02	multimedia	mpv	\N	mpv-0.32.0_4,1
2020-09-27 01:20:07+02	2020-09-27 01:20:43+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-27 01:20:01+02	2020-09-27 01:20:51+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-27 01:20:44+02	2020-09-27 01:21:32+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-27 01:20:52+02	2020-09-27 01:21:38+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-27 01:21:32+02	2020-09-27 01:22:15+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-27 01:21:39+02	2020-09-27 01:22:17+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-27 01:18:38+02	2020-09-27 01:22:33+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-27 01:22:16+02	2020-09-27 01:22:52+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-27 01:22:18+02	2020-09-27 01:23:04+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-27 01:22:53+02	2020-09-27 01:23:32+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-09-27 01:22:34+02	2020-09-27 01:23:33+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-27 04:46:28+02	2020-09-27 04:46:49+02	net	rdesktop	\N	rdesktop-1.9.0
2020-09-27 01:23:05+02	2020-09-27 01:24:02+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-09-27 01:23:33+02	2020-09-27 01:24:13+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-09-27 01:23:33+02	2020-09-27 01:24:27+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-27 01:24:03+02	2020-09-27 01:28:39+02	print	lilypond	\N	lilypond-2.20.0
2020-09-27 01:24:13+02	2020-09-27 01:29:19+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-27 01:24:28+02	2020-09-27 01:29:22+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-27 01:28:40+02	2020-09-27 01:30:08+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-27 01:30:09+02	2020-09-27 01:30:30+02	print	transfig	\N	transfig-3.2.5e_4
2020-09-27 01:30:30+02	2020-09-27 01:30:44+02	security	openconnect	\N	openconnect-8.10
2020-09-27 01:30:45+02	2020-09-27 01:31:27+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-27 01:31:27+02	2020-09-27 01:32:42+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-09-27 01:32:43+02	2020-09-27 01:34:06+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-09-27 01:29:20+02	2020-09-27 01:34:42+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-27 01:34:07+02	2020-09-27 01:34:51+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-27 01:34:43+02	2020-09-27 01:35:07+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-09-27 01:34:51+02	2020-09-27 01:35:34+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-09-27 01:35:07+02	2020-09-27 01:35:40+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-27 01:35:41+02	2020-09-27 01:35:51+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-09-27 01:35:35+02	2020-09-27 01:35:53+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.1
2020-09-27 01:35:52+02	2020-09-27 01:36:39+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-27 01:35:54+02	2020-09-27 01:36:48+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-27 01:36:49+02	2020-09-27 01:37:47+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-27 01:37:48+02	2020-09-27 01:38:53+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-27 01:38:54+02	2020-09-27 01:40:25+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-27 01:36:40+02	2020-09-27 01:40:50+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-27 01:40:51+02	2020-09-27 01:41:10+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-09-27 01:40:26+02	2020-09-27 01:41:41+02	x11-fm	nautilus	\N	nautilus-3.28.1_4
2020-09-27 01:41:10+02	2020-09-27 01:41:58+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-27 01:41:42+02	2020-09-27 01:42:13+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-27 01:41:58+02	2020-09-27 01:42:54+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-27 01:42:55+02	2020-09-27 01:46:46+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-27 01:29:23+02	2020-09-27 01:47:12+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-27 01:47:13+02	2020-09-27 01:52:15+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-27 01:42:13+02	2020-09-27 01:52:28+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-27 01:52:16+02	2020-09-27 01:54:32+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-27 01:54:33+02	2020-09-27 01:56:28+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-27 01:56:29+02	2020-09-27 01:58:15+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-27 01:58:16+02	2020-09-27 02:01:57+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-27 01:46:47+02	2020-09-27 02:03:08+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-27 02:01:58+02	2020-09-27 02:04:10+02	databases	postgresql12-contrib	\N	postgresql12-contrib-12.4
2020-09-27 02:03:09+02	2020-09-27 02:04:49+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-27 02:04:11+02	2020-09-27 02:04:52+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-27 02:04:53+02	2020-09-27 02:05:29+02	devel	bzr	py27	bzr-2.7.0.6622
2020-09-27 02:05:29+02	2020-09-27 02:15:54+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-27 02:15:54+02	2020-09-27 02:17:03+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-27 02:04:49+02	2020-09-27 02:27:17+02	devel	RStudio	\N	RStudio-1.3.1093
2020-09-27 02:17:03+02	2020-09-27 02:42:00+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-27 02:42:02+02	2020-09-27 03:00:07+02	devel	llvm10	\N	llvm10-10.0.1_1
2020-09-27 03:00:08+02	2020-09-27 03:00:47+02	devel	tortoisehg	py37	tortoisehg-5.5.1_1
2020-09-27 03:00:48+02	2020-09-27 03:01:50+02	dns	bind916	\N	bind916-9.16.7_1
2020-09-27 03:01:51+02	2020-09-27 03:11:34+02	editors	codelite	\N	codelite-12.0_4
2020-09-27 03:11:35+02	2020-09-27 03:13:28+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-27 03:13:29+02	2020-09-27 03:15:25+02	editors	vim	\N	vim-8.2.1558
2020-09-27 01:52:29+02	2020-09-27 03:19:40+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-27 02:27:18+02	2020-09-27 03:22:28+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200811
2020-09-27 03:15:25+02	2020-09-27 03:23:54+02	emulators	qemu	\N	qemu-4.2.1
2020-09-27 03:22:30+02	2020-09-27 03:31:17+02	emulators	wine	\N	wine-5.0.2,1
2020-09-27 03:31:18+02	2020-09-27 03:31:29+02	games	KnightCap	\N	knightcap-3.6_8
2020-09-27 03:31:29+02	2020-09-27 03:31:45+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-09-27 03:31:46+02	2020-09-27 03:34:06+02	games	alienarena	\N	alienarena-2013.766_9
2020-09-27 03:34:07+02	2020-09-27 03:34:30+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-09-27 03:34:31+02	2020-09-27 03:35:43+02	games	battletanks	\N	btanks-0.9.8083_9
2020-09-27 03:35:44+02	2020-09-27 03:35:59+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-09-27 03:36:00+02	2020-09-27 03:36:39+02	games	monsterz	\N	monsterz-0.7.1_15
2020-09-27 03:36:40+02	2020-09-27 03:37:11+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-09-27 03:37:11+02	2020-09-27 03:37:44+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-09-27 03:23:55+02	2020-09-27 03:37:56+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-27 03:37:44+02	2020-09-27 03:38:27+02	games	xblast	\N	xblast-2.10.4_18
2020-09-27 03:37:57+02	2020-09-27 03:38:46+02	games	xboard	\N	xboard-4.9.1_1
2020-09-27 03:38:28+02	2020-09-27 03:39:11+02	games	xcowsay	\N	xcowsay-1.5
2020-09-27 03:39:12+02	2020-09-27 03:41:08+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-27 03:41:09+02	2020-09-27 03:42:27+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-27 03:38:46+02	2020-09-27 03:43:55+02	graphics	blender	\N	blender-2.90.0
2020-09-27 03:43:56+02	2020-09-27 03:56:58+02	graphics	krita	\N	krita-4.3.0_5
2020-09-27 03:56:59+02	2020-09-27 04:01:14+02	graphics	wings	\N	wings-2.2.5
2020-09-27 04:01:15+02	2020-09-27 04:02:29+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-27 04:02:29+02	2020-09-27 04:03:40+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-09-27 04:03:40+02	2020-09-27 04:04:53+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-27 03:19:41+02	2020-09-27 04:17:26+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_3
2020-09-27 04:04:54+02	2020-09-27 04:30:17+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-27 03:42:27+02	2020-09-27 04:35:29+02	graphics	inkscape	\N	inkscape-1.0.1
2020-09-27 04:35:29+02	2020-09-27 04:39:46+02	math	maxima	\N	maxima-5.44.0_4
2020-09-27 04:39:47+02	2020-09-27 04:40:33+02	misc	mc	\N	mc-nox11-4.8.24
2020-09-27 04:40:33+02	2020-09-27 04:41:19+02	multimedia	celluloid	\N	celluloid-0.20
2020-09-27 04:41:19+02	2020-09-27 04:41:32+02	multimedia	libva-glx	\N	libva-glx-2.9.0
2020-09-27 04:41:32+02	2020-09-27 04:41:49+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-09-27 04:41:49+02	2020-09-27 04:44:18+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.p8
2020-09-27 04:17:27+02	2020-09-27 04:44:20+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.4_1
2020-09-27 04:44:20+02	2020-09-27 04:44:31+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-09-27 04:44:19+02	2020-09-27 04:44:32+02	multimedia	libva-utils	\N	libva-utils-2.9.1
2020-09-27 04:44:32+02	2020-09-27 04:45:58+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-09-27 04:30:18+02	2020-09-27 04:46:11+02	lang	julia10	\N	julia10-1.0.4_5
2020-09-27 04:44:32+02	2020-09-27 04:46:27+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-09-27 04:46:12+02	2020-09-27 04:47:46+02	net	ntopng	\N	ntopng-4.0.d20200710,1
2020-09-27 04:46:50+02	2020-09-27 04:48:05+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-27 04:45:59+02	2020-09-27 04:48:08+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-27 04:48:06+02	2020-09-27 04:48:44+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-27 04:48:08+02	2020-09-27 04:49:00+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-09-27 04:48:44+02	2020-09-27 04:49:39+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-09-27 04:49:00+02	2020-09-27 04:49:56+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-09-27 04:49:39+02	2020-09-27 04:50:17+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-09-27 04:49:57+02	2020-09-27 04:50:34+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-09-27 04:47:46+02	2020-09-27 04:50:54+02	net	wireshark	\N	wireshark-3.2.7
2020-09-27 04:50:17+02	2020-09-27 04:50:56+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-09-27 04:50:35+02	2020-09-27 04:51:13+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-09-27 04:50:54+02	2020-09-27 04:51:31+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-09-27 04:50:57+02	2020-09-27 04:51:53+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-09-27 04:51:13+02	2020-09-27 04:51:54+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-09-27 04:51:32+02	2020-09-27 04:52:10+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-09-27 04:52:10+02	2020-09-27 04:52:19+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-09-27 04:51:54+02	2020-09-27 04:52:30+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-09-27 04:51:54+02	2020-09-27 04:52:34+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-09-27 04:52:34+02	2020-09-27 04:52:56+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-09-27 04:52:19+02	2020-09-27 04:53:17+02	ports-mgmt	synth	\N	synth-2.08
2020-09-27 04:52:57+02	2020-09-27 04:53:23+02	print	gv	\N	gv-3.7.4_5
2020-09-27 04:52:30+02	2020-09-27 04:54:23+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-27 04:53:17+02	2020-09-27 04:55:04+02	print	lyx	\N	lyx-2.3.4.2_1
2020-09-27 04:55:05+02	2020-09-27 04:55:55+02	security	clamav	\N	clamav-0.103.0,1
2020-09-27 04:55:55+02	2020-09-27 04:56:06+02	security	kc24	\N	kc24-2.4.2_1
2020-09-27 04:53:24+02	2020-09-27 04:56:59+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-09-27 04:56:06+02	2020-09-27 04:56:59+02	security	klee	\N	klee-2.1_1
2020-09-27 04:57:00+02	2020-09-27 04:57:40+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-09-27 04:57:00+02	2020-09-27 04:57:43+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-27 04:54:24+02	2020-09-27 04:58:00+02	print	texlive-full	\N	texlive-full-20150521
2020-09-27 04:57:40+02	2020-09-27 04:58:08+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-27 04:57:44+02	2020-09-27 04:58:36+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-27 04:58:01+02	2020-09-27 04:58:44+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-27 04:58:08+02	2020-09-27 04:58:48+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-09-27 04:58:36+02	2020-09-27 04:58:54+02	www	dokuwiki	\N	dokuwiki-20200729
2020-09-27 04:58:48+02	2020-09-27 04:59:07+02	www	mod_evasive	\N	ap24-mod_evasive-1.10.1_2
2020-09-27 04:59:07+02	2020-09-27 04:59:28+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-09-27 04:59:29+02	2020-09-27 05:00:08+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-09-27 04:58:55+02	2020-09-27 05:00:35+02	www	mod_php74	\N	mod_php74-7.4.10
2020-09-27 05:00:36+02	2020-09-27 05:00:52+02	x11	xlockmore	\N	xlockmore-5.65
2020-09-27 05:00:08+02	2020-09-27 05:01:19+02	x11	mate	\N	mate-1.22.2
2020-09-27 05:00:52+02	2020-09-27 05:02:16+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-27 05:02:17+02	2020-09-27 05:03:10+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-27 05:03:10+02	2020-09-27 05:03:21+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-09-27 05:03:21+02	2020-09-27 05:03:41+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-09-27 05:01:19+02	2020-09-27 05:06:08+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-27 05:06:09+02	2020-09-27 05:06:53+02	graphics	xfig	\N	xfig-3.2.7a
2020-09-27 05:03:42+02	2020-09-27 05:26:09+02	editors	vscode	\N	vscode-1.46.1
2020-09-27 04:58:45+02	2020-09-27 06:38:01+02	www	firefox	\N	firefox-81.0_2,2
2020-09-27 06:38:02+02	2020-09-27 06:53:47+02	local	E590T-localbase	\N	E590T-localbase-2020092600
2020-09-27 09:19:33+02	2020-09-27 09:20:27+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-27 09:19:33+02	2020-09-27 09:21:31+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-27 09:21:32+02	2020-09-27 09:21:39+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-27 10:33:01+02	2020-09-27 10:33:10+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-09-27 10:33:11+02	2020-09-27 10:33:22+02	math	libccd	\N	libccd-2.1
2020-09-27 10:33:22+02	2020-09-27 10:33:35+02	math	py-gmpy	py37	py37-gmpy-1.17_1
2020-09-27 10:33:35+02	2020-09-27 10:33:48+02	devel	py-cloudpickle	py37	py37-cloudpickle-1.2.1
2020-09-27 10:33:01+02	2020-09-27 10:33:53+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-09-27 10:33:01+02	2020-09-27 10:36:12+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-09-27 10:33:54+02	2020-09-27 10:36:38+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-27 10:33:49+02	2020-09-27 10:36:39+02	devel	spatialindex	\N	spatialindex-1.8.5
2020-09-27 10:36:39+02	2020-09-27 10:38:50+02	math	py-PyWavelets	py37	py37-PyWavelets-0.5.2_4
2020-09-27 10:36:13+02	2020-09-27 10:38:57+02	math	octomap	\N	octomap-1.9.5.190
2020-09-27 10:38:51+02	2020-09-27 10:39:08+02	math	py-mpmath	py37	py37-mpmath-1.1.0
2020-09-27 10:39:08+02	2020-09-27 10:39:27+02	math	triangle	\N	triangle-1.6_5
2020-09-27 10:39:27+02	2020-09-27 10:39:41+02	textproc	py-numpydoc	py37	py37-numpydoc-0.9.1
2020-09-27 10:39:41+02	2020-09-27 10:40:54+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-27 10:40:55+02	2020-09-27 10:41:11+02	devel	py-colorlog	py37	py37-colorlog-4.2.1
2020-09-27 10:41:11+02	2020-09-27 10:41:26+02	devel	py-rtree	py37	py37-rtree-0.9.4
2020-09-27 10:41:28+02	2020-09-27 10:42:29+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-09-27 10:42:30+02	2020-09-27 10:42:42+02	devel	py-xxhash	py37	py37-xxhash-2.0.0
2020-09-27 10:38:58+02	2020-09-27 10:42:45+02	math	fcl05	\N	fcl05-0.5.0
2020-09-27 10:42:46+02	2020-09-27 10:43:05+02	graphics	py-pyglet	py37	py37-pyglet-1.5.7
2020-09-27 10:42:42+02	2020-09-27 10:43:31+02	graphics	py-PyOpenGL	py37	py37-PyOpenGL-3.1.5
2020-09-27 10:43:32+02	2020-09-27 10:43:45+02	graphics	py-svg.path	py37	py37-svg_path-4.0.2
2020-09-27 10:43:45+02	2020-09-27 10:45:02+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-09-27 10:36:39+02	2020-09-27 10:45:58+02	graphics	geos	\N	geos-3.8.1
2020-09-27 10:45:58+02	2020-09-27 10:46:14+02	devel	py-shapely	py37	py37-Shapely-1.6.4
2020-09-27 10:45:02+02	2020-09-27 10:46:16+02	math	py-sympy	py37	py37-sympy-1.6
2020-09-27 10:46:16+02	2020-09-27 10:46:26+02	misc	py-python-utils	py37	py37-python-utils-2.4.0
2020-09-27 10:46:14+02	2020-09-27 10:46:40+02	math	py-triangle	py37	py37-triangle-20190115.3
2020-09-27 10:46:27+02	2020-09-27 10:46:51+02	net	libarcus	\N	libArcus-4.7.1,1
2020-09-27 10:46:40+02	2020-09-27 10:46:51+02	net	py-ifaddr	py37	py37-ifaddr-0.1.7
2020-09-27 10:46:52+02	2020-09-27 10:47:51+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-27 10:46:51+02	2020-09-27 10:49:50+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.13.1_1
2020-09-27 10:49:50+02	2020-09-27 10:50:04+02	cad	fdm_materials	\N	fdm_materials-4.7.1,1
2020-09-27 10:47:52+02	2020-09-27 10:50:49+02	cad	cura-engine	\N	CuraEngine-4.7.1,1
2020-09-27 10:50:04+02	2020-09-27 10:51:09+02	cad	uranium	\N	Uranium-4.7.1,1
2020-09-27 10:50:49+02	2020-09-27 10:53:31+02	databases	postgresql13-client	\N	postgresql13-client-13.0
2020-09-27 10:51:10+02	2020-09-27 10:53:46+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-27 10:53:31+02	2020-09-27 10:53:53+02	devel	libsavitar	\N	libSavitar-4.5.0_1
2020-09-27 10:53:53+02	2020-09-27 10:54:04+02	net	libcharon	\N	libCharon-4.7.1
2020-09-27 10:54:05+02	2020-09-27 10:54:15+02	net	py-zeroconf	py37	py37-zeroconf-0.28.5
2020-09-27 10:53:46+02	2020-09-27 10:54:16+02	math	py-numpy-stl	py37	py37-numpy-stl-2.11.2
2020-09-27 10:54:15+02	2020-09-27 10:54:26+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-09-27 10:43:05+02	2020-09-27 10:54:50+02	graphics	py-scikit-image	py37	py37-scikit-image-0.14.3_1
2020-09-27 10:54:16+02	2020-09-27 10:55:44+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-09-27 10:54:51+02	2020-09-27 10:56:13+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-09-27 10:54:27+02	2020-09-27 10:56:32+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-09-27 10:56:13+02	2020-09-27 10:57:20+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-27 10:57:21+02	2020-09-27 10:57:38+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-09-27 10:56:33+02	2020-09-27 10:58:37+02	cad	cura	\N	Cura-4.7.1,2
2020-09-27 10:55:44+02	2020-09-27 11:00:13+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-27 11:00:14+02	2020-09-27 11:01:16+02	x11	mate	\N	mate-1.22.2
2020-09-27 10:57:39+02	2020-09-27 11:01:18+02	databases	postgresql13-contrib	\N	postgresql13-contrib-13.0_1
2020-09-27 11:01:19+02	2020-09-27 11:01:43+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-09-27 11:01:17+02	2020-09-27 11:19:26+02	local	E590T-localbase	\N	E590T-localbase-2020092700
2020-09-27 11:30:30+02	2020-09-27 11:31:29+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-27 11:30:30+02	2020-09-27 11:32:40+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-27 11:32:40+02	2020-09-27 11:32:48+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-27 12:44:49+02	2020-09-27 12:45:30+02	databases	postgresql12-server	\N	postgresql12-server-12.4
2020-09-27 12:45:31+02	2020-09-27 13:03:24+02	local	E590T-localbase	\N	E590T-localbase-2020092700
2020-09-27 13:10:02+02	2020-09-27 13:11:19+02	databases	postgresql13-server	\N	postgresql13-server-13.0_1
2020-09-27 13:11:31+02	2020-09-27 13:12:28+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-27 13:11:31+02	2020-09-27 13:13:38+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-27 13:13:39+02	2020-09-27 13:13:46+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-27 13:25:58+02	2020-09-27 13:26:35+02	databases	postgresql13-server	\N	postgresql13-server-13.0_1
2020-09-27 13:26:48+02	2020-09-27 13:27:44+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-27 13:26:48+02	2020-09-27 13:28:53+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-27 13:28:54+02	2020-09-27 13:29:01+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-27 14:13:54+02	2020-09-27 14:14:49+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-27 14:13:54+02	2020-09-27 14:15:58+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-27 14:15:59+02	2020-09-27 14:16:06+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-27 14:28:07+02	2020-09-27 14:28:49+02	databases	postgresql12-server	\N	postgresql12-server-12.4
2020-09-27 14:28:07+02	2020-09-27 14:28:50+02	databases	postgresql13-server	\N	postgresql13-server-13.0_1
2020-09-27 14:28:50+02	2020-09-27 14:45:47+02	local	E590T-localbase	\N	E590T-localbase-2020092700
2020-09-27 14:46:00+02	2020-09-27 14:46:56+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-27 14:46:00+02	2020-09-27 14:48:08+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-27 14:48:08+02	2020-09-27 14:48:16+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-27 20:47:42+02	2020-09-27 20:47:51+02	sysutils	bpytop	\N	bpytop-1.0.38
2020-09-27 20:47:42+02	2020-09-27 20:48:08+02	graphics	opencolorio	\N	opencolorio-1.1.1_1
2020-09-27 20:48:09+02	2020-09-27 20:48:55+02	graphics	openimageio	\N	openimageio-2.2.6.1
2020-09-27 20:48:55+02	2020-09-27 20:56:18+02	graphics	blender	\N	blender-2.90.0
2020-09-27 20:48:09+02	2020-09-27 20:58:54+02	graphics	krita	\N	krita-4.3.0_5
2020-09-27 20:58:55+02	2020-09-27 21:48:52+02	local	E590T-localbase	\N	E590T-localbase-2020092700
2020-09-27 21:49:06+02	2020-09-27 21:50:27+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-27 21:49:06+02	2020-09-27 21:56:26+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-27 21:56:26+02	2020-09-27 21:56:34+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-27 23:04:07+02	2020-09-27 23:04:20+02	www	p5-libwww	\N	p5-libwww-6.49
2020-09-27 23:04:07+02	2020-09-27 23:04:44+02	www	npm-node12	\N	npm-node12-6.14.8
2020-09-27 23:04:45+02	2020-09-27 23:05:13+02	sysutils	parallel	\N	parallel-20200922
2020-09-27 23:04:07+02	2020-09-27 23:05:28+02	devel	gobject-introspection	\N	gobject-introspection-1.66.0,1
2020-09-27 23:05:28+02	2020-09-27 23:06:01+02	net	avahi-app	\N	avahi-app-0.7_3
2020-09-27 23:06:02+02	2020-09-27 23:06:30+02	print	cups	\N	cups-2.3.3_1
2020-09-27 23:07:55+02	2020-09-27 23:09:19+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-09-27 23:06:31+02	2020-09-27 23:10:22+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-09-27 23:10:49+02	2020-09-27 23:11:11+02	devel	json-glib	\N	json-glib-1.6.0
2020-09-27 23:09:20+02	2020-09-27 23:12:03+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-09-27 23:11:11+02	2020-09-27 23:12:22+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-27 23:12:22+02	2020-09-27 23:12:51+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-09-27 23:12:03+02	2020-09-27 23:13:02+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-09-27 23:10:22+02	2020-09-27 23:13:26+02	print	texlive-base	\N	texlive-base-20150521_55
2020-09-27 23:13:03+02	2020-09-27 23:13:51+02	net	geoclue	\N	geoclue-2.5.5
2020-09-27 23:12:51+02	2020-09-27 23:14:11+02	graphics	graphene	\N	graphene-1.10.2
2020-09-27 23:13:52+02	2020-09-27 23:14:20+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-09-27 23:14:12+02	2020-09-27 23:15:00+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2
2020-09-27 23:14:20+02	2020-09-27 23:17:06+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-09-27 23:17:11+02	2020-09-27 23:18:22+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-27 23:18:22+02	2020-09-27 23:19:22+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-27 23:19:22+02	2020-09-27 23:19:57+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-09-27 23:19:57+02	2020-09-27 23:20:54+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-09-27 23:15:00+02	2020-09-27 23:21:28+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-27 23:20:54+02	2020-09-27 23:21:37+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-09-27 23:21:38+02	2020-09-27 23:22:38+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-27 23:22:38+02	2020-09-27 23:23:35+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-27 23:23:36+02	2020-09-27 23:25:33+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-27 23:25:33+02	2020-09-27 23:26:49+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-09-27 23:21:29+02	2020-09-27 23:26:53+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-27 23:26:54+02	2020-09-27 23:27:55+02	x11-toolkits	vte3	\N	vte3-0.62.0
2020-09-27 23:27:55+02	2020-09-27 23:28:26+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-09-27 23:26:50+02	2020-09-27 23:29:10+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-27 23:28:27+02	2020-09-27 23:29:55+02	security	gcr	\N	gcr-3.28.0_1
2020-09-27 23:29:11+02	2020-09-27 23:30:35+02	audio	pulseaudio	\N	pulseaudio-13.0
2020-09-27 23:29:56+02	2020-09-27 23:30:54+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-27 23:30:35+02	2020-09-27 23:31:18+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-27 23:30:54+02	2020-09-27 23:31:42+02	net	remmina	\N	remmina-1.3.9
2020-10-03 02:57:00+02	2020-10-03 02:57:18+02	print	gl2ps	\N	gl2ps-1.4.0
2020-09-27 23:31:18+02	2020-09-27 23:31:45+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-09-27 23:13:27+02	2020-09-27 23:32:10+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-09-27 23:31:45+02	2020-09-27 23:32:31+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-09-27 23:31:42+02	2020-09-27 23:33:07+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-27 23:32:11+02	2020-09-27 23:34:31+02	print	tex-formats	\N	tex-formats-20150521_2
2020-09-27 23:33:07+02	2020-09-27 23:34:41+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-27 23:34:32+02	2020-09-27 23:35:20+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-09-27 23:34:41+02	2020-09-27 23:37:59+02	net	libmateweather	\N	libmateweather-1.24.1
2020-09-27 23:32:32+02	2020-09-27 23:38:14+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-09-27 23:38:00+02	2020-09-27 23:39:16+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-27 23:39:18+02	2020-09-27 23:40:08+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-09-27 23:35:21+02	2020-09-27 23:41:14+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-27 23:40:08+02	2020-09-27 23:41:14+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_12,3
2020-09-27 23:38:15+02	2020-09-27 23:41:55+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-09-27 23:41:15+02	2020-09-27 23:42:00+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-09-27 23:41:14+02	2020-09-27 23:42:27+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-27 23:41:55+02	2020-09-27 23:42:38+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-27 23:42:00+02	2020-09-27 23:42:47+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-09-27 23:42:38+02	2020-09-27 23:48:12+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-27 23:48:12+02	2020-09-27 23:48:55+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-09-27 23:42:48+02	2020-09-27 23:49:02+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-27 23:48:56+02	2020-09-27 23:50:30+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-09-27 23:50:30+02	2020-09-27 23:52:52+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-27 23:52:56+02	2020-09-27 23:54:21+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-09-27 23:54:22+02	2020-09-27 23:55:10+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-09-27 23:55:13+02	2020-09-27 23:56:21+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-27 23:56:21+02	2020-09-27 23:57:45+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-09-27 23:57:45+02	2020-09-27 23:58:58+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-27 23:58:59+02	2020-09-28 00:01:29+02	x11-fm	caja	\N	caja-1.24.0
2020-09-27 23:42:28+02	2020-09-28 00:03:08+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_3
2020-09-28 00:01:30+02	2020-09-28 00:03:23+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-28 00:03:24+02	2020-09-28 00:04:25+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-28 00:03:09+02	2020-09-28 00:04:36+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-09-28 00:04:26+02	2020-09-28 00:05:49+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-09-28 00:04:37+02	2020-09-28 00:05:53+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-28 00:05:54+02	2020-09-28 00:06:52+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-09-28 00:05:49+02	2020-09-28 00:07:17+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-27 23:49:03+02	2020-09-28 00:07:19+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-28 00:06:53+02	2020-09-28 00:08:05+02	graphics	gegl	\N	gegl-0.4.26_5
2020-09-28 00:07:18+02	2020-09-28 00:08:05+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-28 00:07:20+02	2020-09-28 00:08:07+02	math	octomap	\N	octomap-1.9.5.190
2020-09-28 00:08:08+02	2020-09-28 00:09:08+02	x11	zenity	\N	zenity-3.32.0
2020-09-28 00:08:06+02	2020-09-28 00:09:21+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-09-28 00:08:05+02	2020-09-28 00:09:22+02	security	gnome-keyring	\N	gnome-keyring-3.28.2_3
2020-09-28 00:09:23+02	2020-09-28 00:09:43+02	devel	py-gobject	py27	py27-gobject-2.28.6_9
2020-09-28 00:09:09+02	2020-09-28 00:10:09+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-28 00:09:21+02	2020-09-28 00:12:26+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-09-28 00:12:27+02	2020-09-28 00:13:12+02	math	fcl05	\N	fcl05-0.5.0
2020-09-28 00:09:44+02	2020-09-28 00:14:04+02	editors	emacs	full	emacs-27.1_1,3
2020-09-28 00:13:13+02	2020-09-28 00:14:10+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-09-28 00:14:04+02	2020-09-28 00:14:43+02	print	cups-filters	\N	cups-filters-1.28.3
2020-09-28 00:10:09+02	2020-09-28 00:16:01+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-28 00:14:10+02	2020-09-28 00:16:43+02	print	fontforge	\N	fontforge-20190801_2
2020-09-28 00:16:01+02	2020-09-28 00:17:32+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-28 00:17:33+02	2020-09-28 00:18:26+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-28 00:18:27+02	2020-09-28 00:19:23+02	x11	yelp	\N	yelp-3.38.0
2020-09-28 00:19:24+02	2020-09-28 00:20:10+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-09-28 00:20:10+02	2020-09-28 00:20:59+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-09-28 00:14:43+02	2020-09-28 00:21:31+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-09-28 00:21:00+02	2020-09-28 00:22:27+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-28 00:21:32+02	2020-09-28 00:23:18+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-28 00:22:28+02	2020-09-28 00:23:59+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-28 00:23:59+02	2020-09-28 00:25:47+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-09-28 00:23:19+02	2020-09-28 00:26:33+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-09-28 00:25:48+02	2020-09-28 00:27:23+02	x11-wm	marco	\N	marco-1.22.4
2020-09-28 00:26:33+02	2020-09-28 00:27:33+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-28 00:27:34+02	2020-09-28 00:28:42+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-28 00:27:24+02	2020-09-28 00:29:26+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-28 00:28:43+02	2020-09-28 00:31:31+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-28 00:31:31+02	2020-09-28 00:36:27+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-09-28 00:36:28+02	2020-09-28 00:37:37+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-28 00:29:27+02	2020-09-28 00:42:31+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-28 00:42:31+02	2020-09-28 00:44:02+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-09-28 00:16:43+02	2020-09-28 00:51:17+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-28 00:44:05+02	2020-09-28 00:51:53+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-28 00:51:20+02	2020-09-28 00:52:18+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-28 00:51:53+02	2020-09-28 00:52:43+02	net	freerdp	\N	freerdp-2.2.0
2020-09-28 00:52:18+02	2020-09-28 00:53:06+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-28 00:52:43+02	2020-09-28 00:53:46+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-09-28 00:53:06+02	2020-09-28 00:53:47+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-09-28 00:53:47+02	2020-09-28 00:54:05+02	print	libspectre	\N	libspectre-0.2.9
2020-09-28 00:37:38+02	2020-09-28 00:54:25+02	math	R	\N	R-4.0.2
2020-09-28 00:54:05+02	2020-09-28 00:55:59+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-28 00:53:46+02	2020-09-28 00:56:38+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-28 00:56:38+02	2020-09-28 00:58:55+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-09-28 00:54:25+02	2020-09-28 00:59:36+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-09-28 00:58:56+02	2020-09-28 00:59:39+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-28 00:59:37+02	2020-09-28 01:00:25+02	sysutils	tracker	\N	tracker-2.3.4
2020-09-28 00:56:00+02	2020-09-28 01:00:58+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-09-28 01:00:26+02	2020-09-28 01:01:00+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-28 00:59:40+02	2020-09-28 01:01:15+02	textproc	dblatex	\N	dblatex-0.3.11
2020-09-28 01:00:59+02	2020-09-28 01:01:50+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-28 01:01:01+02	2020-09-28 01:02:01+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-28 01:01:16+02	2020-09-28 01:02:06+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-09-28 01:01:51+02	2020-09-28 01:02:30+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-09-28 01:02:01+02	2020-09-28 01:02:47+02	x11-toolkits	py-gtk2	py27	py27-gtk2-2.24.0_5
2020-09-28 01:02:07+02	2020-09-28 01:02:51+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.13.1_1
2020-09-28 01:02:30+02	2020-09-28 01:03:25+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-09-28 01:03:25+02	2020-09-28 01:04:39+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-28 01:02:52+02	2020-09-28 01:04:47+02	archivers	engrampa	\N	engrampa-1.24.1
2020-09-28 01:04:39+02	2020-09-28 01:05:56+02	cad	uranium	\N	Uranium-4.7.1,1
2020-09-28 01:04:48+02	2020-09-28 01:06:10+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-09-28 01:06:10+02	2020-09-28 01:07:08+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-28 01:02:48+02	2020-09-28 01:07:37+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-28 01:07:09+02	2020-09-28 01:08:03+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-09-28 01:07:38+02	2020-09-28 01:08:35+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-09-28 01:05:56+02	2020-09-28 01:08:44+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-28 01:08:04+02	2020-09-28 01:09:10+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-09-28 01:08:35+02	2020-09-28 01:09:46+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-28 01:09:47+02	2020-09-28 01:13:11+02	editors	pluma	\N	pluma-1.24.1
2020-09-28 01:13:11+02	2020-09-28 01:14:55+02	graphics	atril	\N	atril-1.22.3_9
2020-09-28 01:14:56+02	2020-09-28 01:17:02+02	graphics	eom	\N	eom-1.24.1
2020-09-28 01:17:05+02	2020-09-28 01:18:10+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-28 01:18:14+02	2020-09-28 01:20:13+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-28 01:20:13+02	2020-09-28 01:28:18+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-09-28 01:08:45+02	2020-09-28 01:29:08+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-09-28 01:29:09+02	2020-09-28 01:30:48+02	math	mate-calc	\N	mate-calc-1.24.1
2020-09-28 01:09:10+02	2020-09-28 01:30:52+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-09-28 01:30:53+02	2020-09-28 01:31:51+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-28 01:31:52+02	2020-09-28 01:32:32+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-28 01:32:32+02	2020-09-28 01:33:19+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-28 01:33:20+02	2020-09-28 01:34:15+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-28 01:28:19+02	2020-09-28 01:34:23+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-28 01:34:15+02	2020-09-28 01:35:01+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-28 01:34:24+02	2020-09-28 01:35:05+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-28 01:30:48+02	2020-09-28 01:35:14+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-28 01:35:01+02	2020-09-28 01:35:39+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-28 01:35:05+02	2020-09-28 01:35:53+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-28 01:35:14+02	2020-09-28 01:36:20+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-28 01:35:39+02	2020-09-28 01:36:22+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-09-28 01:35:53+02	2020-09-28 01:36:57+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-09-28 01:36:21+02	2020-09-28 01:37:16+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-28 01:36:23+02	2020-09-28 01:41:13+02	print	lilypond	\N	lilypond-2.20.0
2020-09-28 01:36:57+02	2020-09-28 01:42:29+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-09-28 01:37:17+02	2020-09-28 01:42:30+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-09-28 01:41:14+02	2020-09-28 01:42:47+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-09-28 01:42:48+02	2020-09-28 01:43:11+02	print	transfig	\N	transfig-3.2.5e_4
2020-09-28 01:43:11+02	2020-09-28 01:43:54+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-09-28 01:43:54+02	2020-09-28 01:45:15+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-09-28 01:45:15+02	2020-09-28 01:46:42+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-09-28 01:46:43+02	2020-09-28 01:47:30+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-28 01:47:30+02	2020-09-28 01:48:14+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-09-28 01:42:30+02	2020-09-28 01:48:18+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-09-28 01:48:15+02	2020-09-28 01:48:49+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-28 01:48:19+02	2020-09-28 01:49:09+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-28 01:48:49+02	2020-09-28 01:49:46+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-28 01:49:46+02	2020-09-28 01:51:00+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-28 01:51:01+02	2020-09-28 01:52:14+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-28 01:49:09+02	2020-09-28 01:53:43+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-28 01:52:14+02	2020-09-28 01:53:44+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-28 01:53:45+02	2020-09-28 01:54:34+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-28 01:53:43+02	2020-09-28 01:55:03+02	x11-fm	nautilus	\N	nautilus-3.28.1_4
2020-09-28 01:54:34+02	2020-09-28 01:55:10+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-09-28 01:55:03+02	2020-09-28 01:56:11+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-28 01:56:12+02	2020-09-28 02:01:01+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-28 01:42:30+02	2020-09-28 02:03:04+02	print	texlive-docs	\N	texlive-docs-20150523
2020-09-28 02:01:02+02	2020-09-28 02:03:28+02	cad	cura	\N	Cura-4.7.1,2
2020-09-28 01:55:10+02	2020-09-28 02:06:49+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-28 02:03:29+02	2020-09-28 02:08:39+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-28 02:06:50+02	2020-09-28 02:09:23+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-28 02:09:24+02	2020-09-28 02:11:48+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-28 02:11:49+02	2020-09-28 02:13:44+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-28 02:13:45+02	2020-09-28 02:18:21+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-28 02:18:21+02	2020-09-28 02:20:15+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-28 02:03:04+02	2020-09-28 02:20:44+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-28 02:20:16+02	2020-09-28 02:20:58+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-28 02:20:59+02	2020-09-28 02:32:47+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-28 02:32:48+02	2020-09-28 02:34:09+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-28 02:20:45+02	2020-09-28 02:47:55+02	devel	RStudio	\N	RStudio-1.3.1093
2020-09-28 02:47:56+02	2020-09-28 02:48:57+02	devel	tortoisehg	py37	tortoisehg-5.5.1_1
2020-09-28 02:48:58+02	2020-09-28 03:00:37+02	editors	codelite	\N	codelite-12.0_4
2020-09-28 03:00:38+02	2020-09-28 03:02:50+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-09-28 02:34:09+02	2020-09-28 03:02:51+02	devel	electron7	\N	electron7-7.3.1_1
2020-09-28 03:02:51+02	2020-09-28 03:05:15+02	editors	vim	\N	vim-8.2.1558
2020-09-28 03:02:53+02	2020-09-28 03:14:33+02	emulators	qemu	\N	qemu-4.2.1
2020-09-28 03:14:34+02	2020-09-28 03:34:16+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-28 03:34:17+02	2020-09-28 03:35:17+02	games	xcowsay	\N	xcowsay-1.5
2020-09-28 03:35:17+02	2020-09-28 03:37:46+02	graphics	dia	\N	dia-0.97.3_1,1
2020-09-28 03:37:47+02	2020-09-28 03:39:11+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-28 03:39:13+02	2020-09-28 03:43:54+02	graphics	inkscape	\N	inkscape-1.0.1
2020-09-28 03:43:56+02	2020-09-28 03:57:01+02	graphics	krita	\N	krita-4.3.0_5
2020-09-28 03:57:04+02	2020-09-28 04:01:09+02	graphics	wings	\N	wings-2.2.5
2020-09-28 04:01:10+02	2020-09-28 04:02:23+02	graphics	xpdf	\N	xpdf-4.02,1
2020-09-28 04:02:23+02	2020-09-28 04:03:39+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-09-28 02:08:40+02	2020-09-28 04:08:35+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-28 03:05:16+02	2020-09-28 04:12:17+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_3
2020-09-28 04:12:18+02	2020-09-28 04:17:05+02	math	maxima	\N	maxima-5.44.0_4
2020-09-28 04:17:06+02	2020-09-28 04:17:58+02	multimedia	celluloid	\N	celluloid-0.20
2020-09-28 04:17:58+02	2020-09-28 04:19:39+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-09-28 04:19:39+02	2020-09-28 04:21:51+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-09-28 04:03:40+02	2020-09-28 04:23:51+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-09-28 04:21:52+02	2020-09-28 04:24:18+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-09-28 04:23:52+02	2020-09-28 04:25:04+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-28 04:25:05+02	2020-09-28 04:25:56+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-09-28 04:25:57+02	2020-09-28 04:26:58+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-09-28 04:26:58+02	2020-09-28 04:27:39+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-09-28 04:24:19+02	2020-09-28 04:27:57+02	net	wireshark	\N	wireshark-3.2.7
2020-09-28 04:27:40+02	2020-09-28 04:28:20+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-09-28 04:27:57+02	2020-09-28 04:28:37+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-09-28 04:28:21+02	2020-09-28 04:29:01+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-09-28 04:28:38+02	2020-09-28 04:29:19+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-09-28 04:29:01+02	2020-09-28 04:29:41+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-09-28 04:29:19+02	2020-09-28 04:30:26+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-09-28 04:29:42+02	2020-09-28 04:30:30+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-09-28 04:30:26+02	2020-09-28 04:31:11+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-09-28 04:31:11+02	2020-09-28 04:31:41+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-09-28 04:31:41+02	2020-09-28 04:32:09+02	print	gv	\N	gv-3.7.4_5
2020-09-28 04:08:36+02	2020-09-28 04:32:41+02	lang	erlang-runtime23	\N	erlang-runtime23-23.0.4_1
2020-09-28 04:30:30+02	2020-09-28 04:32:42+02	print	auctex	full	auctex-emacs27-12.2_1
2020-09-28 04:32:43+02	2020-09-28 04:33:30+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-09-28 04:33:31+02	2020-09-28 04:34:12+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-09-28 04:34:13+02	2020-09-28 04:35:29+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-28 04:32:10+02	2020-09-28 04:36:06+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-09-28 04:35:29+02	2020-09-28 04:36:20+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-09-28 04:32:42+02	2020-09-28 04:36:44+02	print	texlive-full	\N	texlive-full-20150521
2020-09-28 04:36:20+02	2020-09-28 04:37:36+02	x11	mate	\N	mate-1.22.2
2020-09-28 04:36:45+02	2020-09-28 04:38:18+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-09-28 04:38:18+02	2020-09-28 04:39:02+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-09-28 04:37:37+02	2020-09-28 04:42:34+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-09-28 04:42:35+02	2020-09-28 04:43:29+02	graphics	xfig	\N	xfig-3.2.7a
2020-09-28 04:39:02+02	2020-09-28 05:04:23+02	editors	vscode	\N	vscode-1.46.1
2020-09-28 04:36:07+02	2020-09-28 05:10:23+02	www	firefox	\N	firefox-81.0_2,2
2020-09-28 07:51:28+02	2020-09-28 07:51:58+02	devel	jsoncpp	\N	jsoncpp-1.9.4
2020-09-28 07:51:28+02	2020-09-28 07:55:25+02	databases	postgresql12-server	\N	postgresql12-server-12.4
2020-09-28 07:51:58+02	2020-09-28 07:56:58+02	devel	cmake	\N	cmake-3.18.2
2020-09-28 07:55:25+02	2020-09-28 08:11:05+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_3
2020-09-28 08:11:06+02	2020-09-28 08:22:46+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-28 08:22:47+02	2020-09-28 08:24:06+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-28 07:56:58+02	2020-09-28 08:25:57+02	www	chromium	\N	chromium-84.0.4147.135
2020-09-28 08:26:03+02	2020-09-28 08:27:09+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-09-28 08:24:07+02	2020-09-28 08:27:15+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-09-28 08:27:15+02	2020-09-28 08:28:01+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-28 08:27:10+02	2020-09-28 08:28:05+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-09-28 08:28:06+02	2020-09-28 08:29:42+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-28 08:28:02+02	2020-09-28 08:46:03+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-28 08:46:04+02	2020-09-28 08:48:28+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-09-28 08:48:28+02	2020-09-28 08:49:41+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-09-28 08:29:42+02	2020-09-28 08:51:28+02	devel	RStudio	\N	RStudio-1.3.1093
2020-09-28 08:49:41+02	2020-09-28 08:52:59+02	databases	postgresql13-server	\N	postgresql13-server-13.0_2
2020-09-28 08:51:29+02	2020-09-28 09:08:17+02	local	E590T-localbase	\N	E590T-localbase-2020092700
2020-09-28 09:08:31+02	2020-09-28 09:09:26+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-28 09:08:31+02	2020-09-28 09:10:35+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-28 09:10:36+02	2020-09-28 09:10:43+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-28 18:24:35+02	2020-09-28 18:27:03+02	databases	postgresql13-server	\N	postgresql13-server-13.0_2
2020-09-28 18:27:15+02	2020-09-28 18:28:10+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-28 18:27:15+02	2020-09-28 18:29:20+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-28 18:29:20+02	2020-09-28 18:29:28+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-28 20:53:03+02	2020-09-28 20:54:10+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-28 20:53:03+02	2020-09-28 20:55:23+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-28 20:55:24+02	2020-09-28 20:55:31+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-28 21:15:34+02	2020-09-28 21:15:48+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.38.0
2020-09-28 21:15:49+02	2020-09-28 21:16:12+02	net	glib-networking	\N	glib-networking-2.66.0
2020-09-28 21:16:13+02	2020-09-28 21:16:53+02	devel	libsoup	\N	libsoup-2.62.3
2020-09-28 21:15:34+02	2020-09-28 21:17:34+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-09-28 21:16:54+02	2020-09-28 21:17:39+02	net	geoclue	\N	geoclue-2.5.5
2020-09-28 21:17:35+02	2020-09-28 21:18:17+02	net	remmina	\N	remmina-1.3.9
2020-09-28 21:18:17+02	2020-09-28 21:18:52+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-09-28 21:18:52+02	2020-09-28 21:19:48+02	devel	librest	\N	rest-0.7.93
2020-09-28 21:21:08+02	2020-09-28 21:21:55+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-09-28 21:21:56+02	2020-09-28 21:22:59+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-28 21:19:48+02	2020-09-28 21:23:14+02	net	libmateweather	\N	libmateweather-1.24.1
2020-09-28 21:17:39+02	2020-09-28 21:23:47+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-28 21:23:14+02	2020-09-28 21:25:15+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-28 21:25:15+02	2020-09-28 21:27:21+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-28 21:27:22+02	2020-09-28 21:29:15+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-28 21:23:00+02	2020-09-28 21:29:40+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-28 21:23:47+02	2020-09-28 21:29:49+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-28 21:29:40+02	2020-09-28 21:29:58+02	net	gssdp	\N	gssdp-1.2.3
2020-09-28 21:29:16+02	2020-09-28 21:30:43+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-28 21:29:50+02	2020-09-28 21:32:47+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-28 21:34:09+02	2020-09-28 21:35:15+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-28 21:29:58+02	2020-09-28 21:35:27+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-28 21:35:15+02	2020-09-28 21:36:21+02	sysutils	tracker	\N	tracker-2.3.4
2020-09-28 21:36:22+02	2020-09-28 21:38:09+02	x11	gnome-desktop	\N	gnome-desktop-3.28.2
2020-09-28 21:34:46+02	2020-09-28 21:43:08+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-28 21:43:08+02	2020-09-28 21:44:38+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-28 21:38:09+02	2020-09-28 21:45:35+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-28 21:44:39+02	2020-09-28 21:46:09+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-09-28 21:45:36+02	2020-09-28 21:49:27+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-28 21:46:09+02	2020-09-28 21:51:22+02	editors	pluma	\N	pluma-1.24.1
2020-09-28 21:49:27+02	2020-09-28 21:52:40+02	graphics	atril	\N	atril-1.22.3_9
2020-09-28 21:51:23+02	2020-09-28 21:53:31+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-28 21:53:31+02	2020-09-28 21:54:12+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-28 21:35:28+02	2020-09-28 21:54:29+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-28 21:54:12+02	2020-09-28 21:54:58+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-28 21:54:30+02	2020-09-28 21:55:16+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-28 21:54:58+02	2020-09-28 21:55:46+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-28 21:55:16+02	2020-09-28 21:55:57+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-28 21:55:47+02	2020-09-28 21:56:26+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-28 21:55:57+02	2020-09-28 21:56:49+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-28 21:56:27+02	2020-09-28 21:57:39+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-28 21:56:49+02	2020-09-28 21:57:50+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-28 21:52:42+02	2020-09-28 21:58:23+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-28 21:57:40+02	2020-09-28 21:59:08+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-09-28 21:58:23+02	2020-09-28 21:59:37+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-28 21:59:09+02	2020-09-28 22:01:23+02	x11-fm	nautilus	\N	nautilus-3.28.1_4
2020-09-28 21:59:38+02	2020-09-28 22:01:24+02	benchmarks	fio	\N	fio-3.23
2020-09-28 21:57:51+02	2020-09-28 22:02:59+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-28 22:01:25+02	2020-09-28 22:03:30+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-28 22:03:31+02	2020-09-28 22:05:01+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-28 22:01:23+02	2020-09-28 22:05:56+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-28 22:05:02+02	2020-09-28 22:06:27+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-28 22:05:56+02	2020-09-28 22:10:23+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-28 22:06:28+02	2020-09-28 22:16:34+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-28 22:10:24+02	2020-09-28 22:20:33+02	editors	codelite	\N	codelite-12.0_4
2020-09-28 22:20:34+02	2020-09-28 22:21:50+02	graphics	gimp	\N	gimp-2.10.20,2
2020-09-28 22:21:50+02	2020-09-28 22:26:21+02	graphics	inkscape	\N	inkscape-1.0.1
2020-09-28 22:16:36+02	2020-09-28 22:29:20+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-09-28 22:26:21+02	2020-09-28 22:29:30+02	graphics	wings	\N	wings-2.2.5
2020-09-28 22:29:30+02	2020-09-28 22:30:44+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-28 22:30:44+02	2020-09-28 22:30:53+02	sysutils	bpytop	\N	bpytop-1.0.40
2020-09-28 22:30:53+02	2020-09-28 22:31:59+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-09-28 22:32:00+02	2020-09-28 22:33:13+02	x11	mate	\N	mate-1.22.2
2020-09-28 22:29:21+02	2020-09-28 22:33:45+02	math	maxima	\N	maxima-5.44.0_4
2020-09-28 22:33:14+02	2020-09-28 22:45:48+02	databases	postgresql13-server	\N	postgresql13-server-13.0_2
2020-09-28 22:03:00+02	2020-09-28 23:03:59+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-29 06:39:19+02	2020-09-29 06:40:14+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-29 06:39:19+02	2020-09-29 06:41:24+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-29 06:41:24+02	2020-09-29 06:41:31+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-29 08:14:01+02	2020-09-29 08:16:29+02	databases	postgresql12-server	\N	postgresql12-server-12.4
2020-09-29 08:16:30+02	2020-09-29 08:35:27+02	local	E590T-localbase	\N	E590T-localbase-2020092700
2020-09-29 08:35:40+02	2020-09-29 08:36:35+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-09-29 08:35:40+02	2020-09-29 08:37:46+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-09-29 08:37:46+02	2020-09-29 08:37:54+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-09-29 19:53:48+02	2020-09-29 19:53:55+02	devel	evdev-proto	\N	evdev-proto-5.8
2020-09-29 19:53:48+02	2020-09-29 19:53:57+02	textproc	minixmlto	\N	minixmlto-0.0.3
2020-09-29 19:53:57+02	2020-09-29 19:54:09+02	devel	libudev-devd	\N	libudev-devd-0.4.2_1
2020-09-29 19:53:55+02	2020-09-29 19:54:14+02	multimedia	v4l_compat	\N	v4l_compat-1.18.0
2020-09-29 19:54:10+02	2020-09-29 19:54:27+02	devel	libevdev	\N	libevdev-1.5.9_2
2020-09-29 19:54:15+02	2020-09-29 19:54:32+02	devel	libgudev	\N	libgudev-234
2020-09-29 19:54:27+02	2020-09-29 19:54:39+02	devel	libmtdev	\N	libmtdev-1.1.5_4
2020-09-29 19:54:33+02	2020-09-29 19:54:46+02	devel	py-evdev	py37	py37-evdev-0.8.1_2
2020-09-29 19:54:39+02	2020-09-29 19:54:50+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-09-29 19:54:46+02	2020-09-29 19:55:03+02	x11	libwacom	\N	libwacom-1.5
2020-09-29 19:55:03+02	2020-09-29 19:55:30+02	x11	libinput	\N	libinput-1.16.1
2020-09-29 19:54:51+02	2020-09-29 19:55:46+02	textproc	dsssl-docbook-modular	\N	dsssl-docbook-modular-1.79_1,1
2020-09-29 19:55:46+02	2020-09-29 19:56:05+02	textproc	p5-SGMLSpm	\N	p5-SGMLSpm-1.03_2
2020-09-29 19:55:31+02	2020-09-29 20:02:58+02	textproc	jade	\N	jade-1.2.1_10
2020-09-29 20:02:59+02	2020-09-29 20:07:08+02	textproc	docbook-utils	\N	docbook-utils-0.6.14_13
2020-09-29 20:07:10+02	2020-09-29 20:22:36+02	graphics	colord	\N	colord-1.3.5_1
2020-09-29 19:53:48+02	2020-09-29 20:26:58+02	devel	cmake	\N	cmake-3.18.3
2020-09-29 19:56:05+02	2020-09-29 20:30:00+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0_1
2020-09-29 20:26:58+02	2020-09-29 20:30:01+02	multimedia	libv4l	\N	libv4l-1.18.0
2020-09-29 20:30:02+02	2020-09-29 20:55:03+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-09-29 20:22:36+02	2020-09-29 20:59:57+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-09-29 20:30:03+02	2020-09-29 21:00:52+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-09-29 20:59:58+02	2020-09-29 21:02:18+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-09-29 21:02:18+02	2020-09-29 21:03:03+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-09-29 21:00:52+02	2020-09-29 21:03:28+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-09-29 21:03:28+02	2020-09-29 21:04:21+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-09-29 21:03:04+02	2020-09-29 21:06:02+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2_1
2020-09-29 21:04:21+02	2020-09-29 21:06:32+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-09-29 21:06:33+02	2020-09-29 21:07:59+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-09-29 21:07:59+02	2020-09-29 21:14:32+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-09-29 21:14:33+02	2020-09-29 21:25:06+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-09-29 21:25:07+02	2020-09-29 21:31:47+02	x11-servers	xorg-server	\N	xorg-server-1.20.9,1
2020-09-29 20:55:06+02	2020-09-29 21:32:51+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-09-29 21:32:52+02	2020-09-29 21:34:30+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-09-29 21:31:48+02	2020-09-29 21:34:34+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-10-04 00:37:53+02	2020-10-04 00:42:54+02	x11	xorg	\N	xorg-7.7_3
2020-09-29 21:34:30+02	2020-09-29 21:35:45+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-09-29 21:34:35+02	2020-09-29 21:36:19+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-09-29 21:35:46+02	2020-09-29 21:36:48+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-09-29 21:36:19+02	2020-09-29 21:38:36+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-09-29 21:38:37+02	2020-09-29 21:40:35+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-09-29 21:40:35+02	2020-09-29 21:42:24+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-09-29 21:36:49+02	2020-09-29 21:53:16+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-09-29 21:53:17+02	2020-09-29 21:55:22+02	x11-toolkits	vte3	\N	vte3-0.62.0
2020-09-29 21:55:23+02	2020-09-29 21:56:59+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-09-29 21:56:59+02	2020-09-29 22:12:30+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-09-29 21:42:25+02	2020-09-29 22:12:47+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-09-29 22:12:48+02	2020-09-29 22:14:57+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-09-29 22:12:30+02	2020-09-29 22:15:31+02	devel	sdl20	\N	sdl2-2.0.12_2
2020-09-29 22:14:58+02	2020-09-29 22:16:08+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-09-29 22:16:08+02	2020-09-29 22:20:30+02	security	gcr	\N	gcr-3.28.0_1
2020-09-29 22:15:32+02	2020-09-29 22:22:28+02	devel	glibmm	\N	glibmm-2.64.2,1
2020-09-29 22:22:28+02	2020-09-29 22:23:40+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-09-29 22:20:30+02	2020-09-29 22:24:18+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-09-29 22:23:42+02	2020-09-29 22:24:59+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-09-29 22:24:19+02	2020-09-29 22:25:39+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-09-29 22:24:59+02	2020-09-29 22:26:16+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-09-29 22:26:17+02	2020-09-29 22:28:08+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-09-29 22:25:40+02	2020-09-29 22:29:27+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-09-29 22:28:09+02	2020-09-29 22:30:55+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-09-29 22:29:27+02	2020-09-29 22:32:29+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-09-29 22:30:56+02	2020-09-29 22:32:36+02	net	remmina	\N	remmina-1.3.9
2020-09-29 22:32:37+02	2020-09-29 22:34:08+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-09-29 22:32:30+02	2020-09-29 22:34:20+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-09-29 22:34:21+02	2020-09-29 22:35:07+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-09-29 22:35:08+02	2020-09-29 22:36:04+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-09-29 22:34:09+02	2020-09-29 22:44:38+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-09-29 22:44:39+02	2020-09-29 22:46:07+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-09-29 22:36:05+02	2020-09-29 22:47:09+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-09-29 22:46:07+02	2020-09-29 22:47:17+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-09-29 22:47:10+02	2020-09-29 22:48:11+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0_1
2020-09-29 22:47:17+02	2020-09-29 22:48:19+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-09-29 22:48:13+02	2020-09-29 22:49:20+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-09-29 22:49:21+02	2020-09-29 22:50:30+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-09-29 22:48:20+02	2020-09-29 22:50:54+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-09-29 22:50:31+02	2020-09-29 22:51:31+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.5.0
2020-09-29 22:50:55+02	2020-09-29 22:52:35+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-09-29 22:51:32+02	2020-09-29 22:52:56+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-09-29 22:52:35+02	2020-09-29 22:53:56+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-09-29 22:52:58+02	2020-09-29 22:54:05+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-09-29 22:54:05+02	2020-09-29 22:54:45+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-09-29 22:54:45+02	2020-09-29 22:56:03+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-09-29 22:56:04+02	2020-09-29 23:00:13+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.9,1
2020-09-29 23:00:13+02	2020-09-29 23:02:07+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-09-29 23:02:07+02	2020-09-29 23:04:41+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-09-29 23:04:41+02	2020-09-29 23:09:52+02	net	libmateweather	\N	libmateweather-1.24.1
2020-09-29 23:09:53+02	2020-09-29 23:12:07+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-09-29 23:12:08+02	2020-09-29 23:25:49+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-09-29 21:06:02+02	2020-09-29 23:48:43+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.28.4
2020-09-29 23:48:43+02	2020-09-30 00:35:47+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-09-30 00:35:47+02	2020-09-30 00:39:07+02	math	gnuplot	\N	gnuplot-5.4.0
2020-09-30 00:39:08+02	2020-09-30 00:42:58+02	x11	xorg	\N	xorg-7.7_3
2020-09-30 00:42:59+02	2020-09-30 00:44:34+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-09-30 00:44:34+02	2020-09-30 00:45:59+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-09-29 22:53:56+02	2020-09-30 00:46:48+02	math	vtk8	\N	vtk8-8.2.0
2020-09-30 00:46:00+02	2020-09-30 00:47:15+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-09-30 00:47:16+02	2020-09-30 00:49:46+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-09-30 00:49:47+02	2020-09-30 00:53:43+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.28.0_3
2020-09-30 00:53:44+02	2020-09-30 00:55:18+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-09-30 00:46:49+02	2020-09-30 03:28:27+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-09-30 03:28:29+02	2020-09-30 03:30:56+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-09-30 03:30:56+02	2020-09-30 03:36:50+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-09-29 23:25:52+02	2020-09-30 03:38:47+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-09-30 03:36:51+02	2020-09-30 03:39:56+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-09-30 03:38:48+02	2020-09-30 03:41:03+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-09-30 03:41:04+02	2020-09-30 03:46:45+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-09-30 03:46:48+02	2020-09-30 04:02:10+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-09-30 04:02:13+02	2020-09-30 04:12:22+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-09-30 03:39:57+02	2020-09-30 04:21:08+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-09-30 04:12:25+02	2020-09-30 04:58:46+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-09-30 04:58:46+02	2020-09-30 05:00:04+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-09-30 05:00:09+02	2020-09-30 05:01:36+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-09-30 05:01:36+02	2020-09-30 05:03:42+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-09-30 05:03:42+02	2020-09-30 05:10:04+02	x11-fm	caja	\N	caja-1.24.0
2020-09-30 05:10:05+02	2020-09-30 05:27:57+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-09-30 05:27:58+02	2020-09-30 05:32:03+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-09-30 05:32:04+02	2020-09-30 05:33:30+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-09-30 05:33:31+02	2020-09-30 05:35:39+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-09-30 05:35:40+02	2020-09-30 05:39:54+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-09-30 05:39:55+02	2020-09-30 05:41:57+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-09-30 05:41:58+02	2020-09-30 05:45:29+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-09-30 05:45:30+02	2020-09-30 05:50:19+02	graphics	gegl	\N	gegl-0.4.26_5
2020-09-30 05:50:19+02	2020-09-30 05:52:54+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-09-30 05:52:55+02	2020-09-30 05:56:42+02	math	octomap	\N	octomap-1.9.5.190
2020-09-30 05:56:42+02	2020-09-30 06:00:50+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-09-30 06:00:51+02	2020-09-30 06:03:33+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-09-30 06:03:34+02	2020-09-30 06:04:55+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-09-30 06:04:56+02	2020-09-30 06:06:37+02	x11	zenity	\N	zenity-3.32.0
2020-09-30 06:06:37+02	2020-09-30 06:08:14+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-09-30 06:08:15+02	2020-09-30 06:13:52+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-09-30 04:21:09+02	2020-09-30 06:24:03+02	cad	kicad	\N	kicad-5.1.6_1,2
2020-09-30 06:13:52+02	2020-09-30 06:26:10+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-09-30 06:26:11+02	2020-09-30 06:27:54+02	devel	libpeas	\N	libpeas-1.22.0
2020-09-30 06:27:55+02	2020-09-30 06:30:51+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-09-30 06:30:52+02	2020-09-30 06:34:23+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-09-30 06:34:24+02	2020-09-30 06:37:27+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-09-30 06:24:04+02	2020-09-30 06:37:39+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-09-30 06:37:40+02	2020-09-30 06:45:54+02	editors	emacs	full	emacs-27.1_1,3
2020-09-30 06:37:28+02	2020-09-30 06:47:06+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-09-30 06:45:54+02	2020-09-30 07:11:09+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-09-30 07:11:10+02	2020-09-30 07:12:19+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-09-30 07:12:20+02	2020-09-30 07:13:07+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-09-30 07:13:07+02	2020-09-30 07:18:23+02	math	fcl05	\N	fcl05-0.5.0
2020-09-30 07:18:24+02	2020-09-30 07:21:43+02	math	py-PyWavelets	py37	py37-PyWavelets-1.1.1
2020-09-30 07:21:44+02	2020-09-30 07:23:40+02	print	cups-filters	\N	cups-filters-1.28.3
2020-09-30 06:47:06+02	2020-09-30 07:27:22+02	graphics	opencv	\N	opencv-3.4.1_36
2020-09-30 07:27:23+02	2020-09-30 07:30:11+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-09-30 07:23:41+02	2020-09-30 07:33:21+02	print	fontforge	\N	fontforge-20190801_2
2020-09-30 07:33:22+02	2020-09-30 07:34:50+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-09-30 07:34:52+02	2020-09-30 07:38:45+02	x11	yelp	\N	yelp-3.38.0
2020-09-30 07:38:46+02	2020-09-30 07:41:23+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-09-30 07:41:24+02	2020-09-30 07:52:23+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-09-30 07:52:23+02	2020-09-30 08:05:12+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-09-30 08:05:15+02	2020-09-30 08:08:47+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0_1
2020-09-30 08:08:49+02	2020-09-30 08:13:04+02	x11-wm	marco	\N	marco-1.22.4
2020-09-30 08:13:05+02	2020-09-30 08:14:22+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-09-30 08:14:25+02	2020-09-30 08:17:22+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-09-30 08:17:23+02	2020-09-30 08:18:54+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-09-30 08:18:55+02	2020-09-30 08:24:44+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-09-30 08:24:45+02	2020-09-30 08:34:07+02	devel	ncurses	\N	ncurses-6.2.20200926
2020-09-30 08:34:07+02	2020-09-30 08:36:45+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-09-30 08:36:45+02	2020-09-30 08:44:18+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-09-30 08:44:19+02	2020-09-30 08:45:36+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-09-30 08:45:37+02	2020-09-30 09:05:59+02	graphics	py-scikit-image	py37	py37-scikit-image-0.17.2
2020-09-30 09:06:00+02	2020-09-30 09:07:39+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-09-30 09:07:39+02	2020-09-30 09:17:56+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-09-30 09:17:57+02	2020-09-30 09:20:47+02	multimedia	mlt	\N	mlt-6.22.1
2020-09-30 09:20:48+02	2020-09-30 09:24:46+02	net	freerdp	\N	freerdp-2.2.0_1
2020-09-30 09:24:46+02	2020-09-30 09:25:52+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-09-30 09:25:53+02	2020-09-30 09:31:47+02	print	gutenprint	\N	gutenprint-5.3.3
2020-09-30 09:31:47+02	2020-09-30 09:35:07+02	print	mftrace	\N	mftrace-1.2.18_2
2020-09-30 09:35:08+02	2020-09-30 09:40:34+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-09-30 09:40:34+02	2020-09-30 09:41:46+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-09-30 09:41:47+02	2020-09-30 09:42:46+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-09-30 09:42:47+02	2020-09-30 09:44:48+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-09-30 09:44:49+02	2020-09-30 09:45:33+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-09-30 09:45:34+02	2020-09-30 09:47:39+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-09-30 09:47:40+02	2020-09-30 09:50:54+02	x11-servers	xephyr	\N	xephyr-1.20.9,1
2020-09-30 09:50:55+02	2020-09-30 09:54:59+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.13.1_1
2020-09-30 09:54:59+02	2020-09-30 09:57:53+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-09-30 09:57:54+02	2020-09-30 10:38:49+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-09-30 10:38:50+02	2020-09-30 10:42:21+02	archivers	engrampa	\N	engrampa-1.24.1
2020-09-30 10:42:22+02	2020-09-30 10:42:58+02	audio	faudio	\N	FAudio-20.09
2020-09-30 10:42:59+02	2020-09-30 10:44:33+02	audio	mate-media	\N	mate-media-1.24.1
2020-09-30 10:44:33+02	2020-09-30 10:46:03+02	cad	uranium	\N	Uranium-4.7.1,1
2020-09-30 10:46:04+02	2020-09-30 10:48:06+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-09-30 10:48:07+02	2020-09-30 10:53:39+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-09-30 10:53:40+02	2020-09-30 10:54:50+02	deskutils	mozo	\N	mozo-1.24.0
2020-09-30 10:54:51+02	2020-09-30 10:55:56+02	devel	adacurses	\N	adacurses-20180127
2020-09-30 10:55:57+02	2020-09-30 10:57:37+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-09-30 10:57:37+02	2020-09-30 10:59:44+02	devel	py-game	py37	py37-game-1.9.6_1
2020-09-30 10:59:45+02	2020-09-30 11:02:58+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-09-30 11:02:59+02	2020-09-30 11:04:40+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-09-30 00:55:19+02	2020-09-30 12:40:32+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-09-30 12:40:33+02	2020-09-30 14:35:18+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-09-30 14:35:19+02	2020-09-30 14:37:48+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-09-30 14:37:48+02	2020-09-30 14:44:44+02	editors	pluma	\N	pluma-1.24.1
2020-09-30 14:44:44+02	2020-09-30 14:49:46+02	graphics	atril	\N	atril-1.22.3_9
2020-09-30 14:49:47+02	2020-09-30 14:52:58+02	graphics	eom	\N	eom-1.24.1
2020-09-30 14:52:59+02	2020-09-30 14:55:16+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-09-30 14:55:17+02	2020-09-30 14:57:29+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-09-30 14:57:30+02	2020-09-30 14:59:56+02	graphics	py-gimp	\N	py27-gimp-2.10.20_6
2020-09-30 14:59:57+02	2020-09-30 15:09:40+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-09-30 15:09:41+02	2020-09-30 15:21:53+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-09-30 15:21:54+02	2020-09-30 15:32:22+02	lang	sbcl	\N	sbcl-2.0.9,1
2020-09-30 15:32:23+02	2020-09-30 15:35:10+02	math	mate-calc	\N	mate-calc-1.24.1
2020-09-30 15:35:11+02	2020-09-30 15:41:36+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-09-30 15:41:37+02	2020-09-30 15:45:05+02	multimedia	mpv	\N	mpv-0.32.0_5,1
2020-09-30 15:45:05+02	2020-09-30 16:02:18+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-09-30 16:02:19+02	2020-09-30 16:04:50+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-09-30 16:04:50+02	2020-09-30 16:05:57+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-09-30 16:05:57+02	2020-09-30 16:07:10+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-09-30 16:07:11+02	2020-09-30 16:08:13+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-09-30 16:08:13+02	2020-09-30 16:09:23+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-09-30 16:09:24+02	2020-09-30 16:10:25+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-09-30 16:10:26+02	2020-09-30 16:11:29+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-09-30 16:11:30+02	2020-09-30 16:12:53+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-09-30 16:12:53+02	2020-09-30 16:14:58+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-09-30 16:14:59+02	2020-09-30 16:16:44+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-09-30 16:16:44+02	2020-09-30 16:40:16+02	print	lilypond	\N	lilypond-2.20.0
2020-09-30 16:40:19+02	2020-09-30 16:43:04+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-09-30 16:43:05+02	2020-09-30 16:46:38+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-09-30 16:46:39+02	2020-09-30 16:48:07+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-09-30 16:48:08+02	2020-09-30 16:50:34+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-09-30 16:50:38+02	2020-09-30 16:51:56+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-09-30 16:51:57+02	2020-09-30 16:54:28+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-09-30 16:54:30+02	2020-09-30 16:56:38+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-09-30 16:56:39+02	2020-09-30 17:04:50+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-09-30 17:04:51+02	2020-09-30 17:06:22+02	x11	mate-base	\N	mate-base-1.24.0
2020-09-30 17:06:23+02	2020-09-30 17:08:11+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-09-30 17:08:11+02	2020-09-30 17:10:53+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-09-30 17:10:54+02	2020-09-30 17:15:12+02	x11-fm	nautilus	\N	nautilus-3.28.1_5
2020-09-30 17:15:13+02	2020-09-30 17:16:47+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-09-30 17:16:48+02	2020-09-30 17:28:00+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-09-30 17:28:01+02	2020-09-30 18:04:10+02	astro	stellarium	\N	stellarium-0.20.2
2020-09-30 11:04:40+02	2020-09-30 18:06:31+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-09-30 18:06:32+02	2020-09-30 18:09:22+02	cad	cura	\N	Cura-4.7.1,2
2020-09-30 18:04:11+02	2020-09-30 18:39:28+02	audio	rosegarden	\N	rosegarden-20.06
2020-09-30 18:39:30+02	2020-09-30 18:46:14+02	cad	kicad-doc	\N	kicad-doc-5.1.6
2020-09-30 18:46:15+02	2020-09-30 18:49:28+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.6
2020-09-30 18:49:28+02	2020-09-30 20:42:43+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.6
2020-09-30 20:42:45+02	2020-09-30 20:45:31+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.6
2020-09-30 20:45:32+02	2020-09-30 20:47:47+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.6
2020-09-30 18:09:23+02	2020-09-30 21:15:54+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-09-30 20:47:48+02	2020-09-30 21:20:01+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-09-30 21:20:02+02	2020-09-30 21:25:10+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-09-30 21:25:11+02	2020-09-30 21:26:25+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-09-30 21:40:28+02	2020-09-30 23:02:46+02	devel	codeblocks	\N	codeblocks-20.03
2020-09-30 23:02:49+02	2020-09-30 23:06:01+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-09-30 21:26:26+02	2020-09-30 23:16:39+02	devel	RStudio	\N	RStudio-1.3.1093
2020-09-30 23:16:40+02	2020-09-30 23:17:30+02	devel	tortoisehg	py37	tortoisehg-5.5.1_1
2020-09-30 23:17:30+02	2020-10-01 00:57:57+02	editors	codelite	\N	codelite-12.0_4
2020-10-01 00:58:02+02	2020-10-01 01:01:02+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-10-01 01:01:06+02	2020-10-01 01:06:22+02	editors	vim	\N	vim-8.2.1558
2020-10-01 01:06:24+02	2020-10-01 02:18:35+02	emulators	qemu	\N	qemu-4.2.1
2020-10-01 02:18:37+02	2020-10-01 03:51:46+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_3
2020-10-01 03:51:48+02	2020-10-01 04:57:09+02	emulators	wine	\N	wine-5.0.2,1
2020-10-01 04:57:10+02	2020-10-01 05:20:27+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-01 05:20:27+02	2020-10-01 05:21:26+02	games	monsterz	\N	monsterz-0.7.1_15
2020-10-01 05:21:27+02	2020-10-01 05:22:44+02	games	xcowsay	\N	xcowsay-1.5
2020-10-01 05:22:45+02	2020-10-01 07:25:43+02	graphics	blender	\N	blender-2.90.0
2020-10-01 07:25:48+02	2020-10-01 07:28:50+02	graphics	gimp	\N	gimp-2.10.20,2
2020-10-01 07:28:51+02	2020-10-01 08:56:41+02	graphics	inkscape	\N	inkscape-1.0.1
2020-10-01 08:56:42+02	2020-10-01 11:12:09+02	graphics	krita	\N	krita-4.3.0_5
2020-10-01 11:12:10+02	2020-10-01 11:17:14+02	graphics	wings	\N	wings-2.2.5
2020-10-01 11:17:14+02	2020-10-01 11:20:08+02	graphics	xpdf	\N	xpdf-4.02,1
2020-10-01 11:20:08+02	2020-10-01 11:21:32+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-01 11:21:32+02	2020-10-01 11:29:17+02	math	maxima	\N	maxima-5.44.0_4
2020-10-01 11:29:18+02	2020-10-01 11:31:00+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-01 11:31:02+02	2020-10-01 11:40:47+02	multimedia	obs-studio	\N	obs-studio-26.0.0.r1_1
2020-10-01 11:40:49+02	2020-10-01 11:44:07+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-10-01 11:44:08+02	2020-10-01 11:57:30+02	multimedia	shotcut	\N	shotcut-20.07.11
2020-10-01 11:57:31+02	2020-10-01 11:59:30+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-09-30 23:06:02+02	2020-10-01 12:08:31+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-01 12:08:35+02	2020-10-01 12:09:51+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-01 12:09:52+02	2020-10-01 12:12:47+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-10-01 12:12:47+02	2020-10-01 12:16:40+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-01 11:59:30+02	2020-10-01 12:27:46+02	net	wireshark	\N	wireshark-3.2.7
2020-10-01 12:16:41+02	2020-10-01 12:41:20+02	print	lyx	\N	lyx-2.3.4.2_1
2020-10-01 12:41:21+02	2020-10-01 12:42:57+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-10-01 12:42:58+02	2020-10-01 12:44:14+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-01 12:44:15+02	2020-10-01 12:45:58+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-01 12:45:59+02	2020-10-01 12:49:49+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-01 12:49:51+02	2020-10-01 12:54:18+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-10-01 12:27:47+02	2020-10-01 13:30:58+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-10-01 13:30:59+02	2020-10-01 13:33:51+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-10-01 13:33:51+02	2020-10-01 13:35:57+02	x11	mate	\N	mate-1.22.2
2020-10-01 13:35:57+02	2020-10-01 13:38:23+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-10-01 14:04:09+02	2020-10-01 14:47:42+02	editors	vscode	\N	vscode-1.46.1
2020-10-01 12:54:18+02	2020-10-01 16:02:50+02	www	firefox	\N	firefox-81.0_2,2
2020-09-30 07:30:11+02	2020-10-01 20:53:23+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-01 20:53:25+02	2020-10-01 20:54:14+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-01 20:54:16+02	2020-10-01 20:55:05+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-01 20:55:06+02	2020-10-01 20:55:54+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-01 21:00:18+02	2020-10-01 21:08:20+02	databases	postgresql12-server	\N	postgresql12-server-12.4
2020-10-01 21:08:20+02	2020-10-01 21:24:45+02	local	E590T-localbase	\N	E590T-localbase-2020092700
2020-10-01 21:27:05+02	2020-10-01 21:28:03+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-01 21:27:05+02	2020-10-01 21:29:56+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-10-01 21:29:57+02	2020-10-01 21:30:04+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-02 00:49:20+02	2020-10-02 00:49:39+02	textproc	html2text	\N	html2text-1.3.2a,1
2020-10-02 00:49:39+02	2020-10-02 00:49:52+02	textproc	minixmlto	\N	minixmlto-0.0.3
2020-10-02 00:49:20+02	2020-10-02 00:50:08+02	devel	libuv	\N	libuv-1.40.0
2020-10-02 00:49:53+02	2020-10-02 00:50:10+02	devel	libevdev	\N	libevdev-1.9.1.20200928
2020-10-02 00:50:11+02	2020-10-02 00:50:22+02	devel	libmtdev	\N	libmtdev-1.1.6
2020-10-02 00:50:22+02	2020-10-02 00:50:36+02	devel	py-evdev	py37	py37-evdev-1.3.0
2020-10-02 00:49:20+02	2020-10-02 00:50:44+02	devel	gobject-introspection	\N	gobject-introspection-1.66.0_1,1
2020-10-02 00:50:36+02	2020-10-02 00:50:55+02	textproc	p5-SGMLSpm	\N	p5-SGMLSpm-1.03_2
2020-10-02 00:50:55+02	2020-10-02 00:51:26+02	x11	libinput	\N	libinput-1.16.1
2020-10-02 00:50:45+02	2020-10-02 00:51:59+02	net	avahi-app	\N	avahi-app-0.7_3
2020-10-02 00:50:08+02	2020-10-02 00:52:35+02	devel	cmake	\N	cmake-3.18.3
2020-10-02 00:51:27+02	2020-10-02 00:52:47+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.38.0
2020-10-02 00:52:47+02	2020-10-02 00:53:21+02	audio	libsndfile	\N	libsndfile-1.0.30
2020-10-02 00:52:36+02	2020-10-02 00:54:56+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0_1
2020-10-02 00:54:56+02	2020-10-02 00:57:20+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-10-02 00:53:05+02	2020-10-02 00:57:21+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-10-02 00:53:22+02	2020-10-02 00:58:08+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-10-02 00:57:21+02	2020-10-02 01:00:15+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-10-02 01:00:15+02	2020-10-02 01:04:06+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2_1
2020-10-02 01:04:07+02	2020-10-02 01:04:42+02	devel	json-glib	\N	json-glib-1.6.0
2020-10-02 00:57:21+02	2020-10-02 01:05:34+02	print	texlive-base	\N	texlive-base-20150521_55
2020-10-02 00:58:09+02	2020-10-02 01:07:10+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-10-02 01:04:42+02	2020-10-02 01:07:49+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-10-02 01:07:10+02	2020-10-02 01:07:50+02	textproc	docbook-xsl-ns	\N	docbook-xsl-ns-1.79.1
2020-10-02 01:07:50+02	2020-10-02 01:08:14+02	devel	libnotify	\N	libnotify-0.7.9_1
2020-10-02 01:07:50+02	2020-10-02 01:08:35+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-10-02 01:08:14+02	2020-10-02 01:08:37+02	audio	libsamplerate	\N	libsamplerate-0.1.9_1
2020-10-02 01:08:37+02	2020-10-02 01:09:22+02	security	libsecret	\N	libsecret-0.20.3
2020-10-02 01:08:36+02	2020-10-02 01:09:36+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-10-02 01:09:22+02	2020-10-02 01:09:37+02	audio	lv2	\N	lv2-1.18.0_1
2020-10-02 01:09:37+02	2020-10-02 01:10:13+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-10-02 01:09:37+02	2020-10-02 01:10:17+02	audio	vamp-plugin-sdk	\N	vamp-plugin-sdk-2.9.0_1
2020-10-02 01:10:17+02	2020-10-02 01:10:46+02	audio	rubberband	\N	rubberband-1.8.2_2
2020-10-02 01:10:14+02	2020-10-02 01:11:25+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-10-02 01:11:25+02	2020-10-02 01:13:02+02	graphics	graphene	\N	graphene-1.10.2
2020-10-02 01:13:02+02	2020-10-02 01:14:06+02	net	geoclue	\N	geoclue-2.5.5
2020-10-02 01:14:06+02	2020-10-02 01:14:54+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2_1
2020-10-02 01:10:46+02	2020-10-02 01:14:59+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-10-02 01:14:54+02	2020-10-02 01:15:25+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-10-02 01:15:25+02	2020-10-02 01:15:58+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-10-02 01:15:59+02	2020-10-02 01:17:03+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-10-02 01:17:04+02	2020-10-02 01:17:38+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-10-02 01:05:35+02	2020-10-02 01:25:01+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-10-02 01:17:38+02	2020-10-02 01:25:30+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-10-02 01:25:02+02	2020-10-02 01:28:00+02	print	tex-formats	\N	tex-formats-20150521_2
2020-10-02 01:25:30+02	2020-10-02 01:31:53+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-10-02 01:28:01+02	2020-10-02 01:34:07+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-10-02 01:34:08+02	2020-10-02 01:40:15+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-10-02 01:40:16+02	2020-10-02 01:42:18+02	textproc	docbook-utils	\N	docbook-utils-0.6.14_13
2020-10-02 01:42:18+02	2020-10-02 01:43:12+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-10-02 01:43:16+02	2020-10-02 01:44:06+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-10-02 01:44:07+02	2020-10-02 01:44:58+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-10-02 01:31:54+02	2020-10-02 01:45:45+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-10-02 01:44:59+02	2020-10-02 01:45:50+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-10-02 01:45:52+02	2020-10-02 01:46:35+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8_1
2020-10-02 01:45:46+02	2020-10-02 01:46:53+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-10-02 01:46:35+02	2020-10-02 01:47:36+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-10-02 01:46:54+02	2020-10-02 01:47:40+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-10-02 01:47:40+02	2020-10-02 01:48:31+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-10-02 01:47:36+02	2020-10-02 01:48:34+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-10-02 01:48:31+02	2020-10-02 01:49:51+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-10-02 01:49:53+02	2020-10-02 01:51:01+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-10-02 01:48:34+02	2020-10-02 01:51:07+02	audio	pulseaudio	\N	pulseaudio-13.0_1
2020-10-02 01:51:02+02	2020-10-02 01:53:07+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-10-02 01:53:07+02	2020-10-02 01:54:36+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-02 01:51:07+02	2020-10-02 01:54:52+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-10-02 01:54:37+02	2020-10-02 01:55:56+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-10-02 01:54:53+02	2020-10-02 01:56:16+02	devel	libsigc++20	\N	libsigc++-2.10.4
2020-10-02 01:55:56+02	2020-10-02 01:56:40+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-10-02 01:56:41+02	2020-10-02 01:57:36+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-10-02 01:56:19+02	2020-10-02 01:58:07+02	security	gcr	\N	gcr-3.28.0_1
2020-10-02 01:58:07+02	2020-10-02 01:59:23+02	x11-toolkits	vte3	\N	vte3-0.62.0
2020-10-02 01:59:24+02	2020-10-02 02:00:19+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-10-02 01:57:37+02	2020-10-02 02:00:44+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-10-02 02:00:19+02	2020-10-02 02:01:33+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-10-02 02:00:44+02	2020-10-02 02:02:29+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-10-02 02:01:34+02	2020-10-02 02:02:39+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-10-02 02:02:40+02	2020-10-02 02:03:03+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-10-02 02:02:30+02	2020-10-02 02:03:19+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-10-02 02:03:03+02	2020-10-02 02:03:33+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-10-02 02:03:19+02	2020-10-02 02:03:59+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-10-02 02:03:59+02	2020-10-02 02:04:44+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-10-02 02:04:45+02	2020-10-02 02:05:33+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-10-02 02:05:33+02	2020-10-02 02:06:34+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-10-02 02:06:35+02	2020-10-02 02:07:06+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-10-02 02:07:06+02	2020-10-02 02:08:17+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-10-02 02:08:17+02	2020-10-02 02:09:07+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-10-02 02:03:34+02	2020-10-02 02:09:19+02	devel	glibmm	\N	glibmm-2.64.2,1
2020-10-02 02:09:08+02	2020-10-02 02:09:59+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-10-02 02:09:20+02	2020-10-02 02:10:37+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-10-02 02:09:59+02	2020-10-02 02:11:02+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-10-02 02:10:37+02	2020-10-02 02:11:24+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-10-02 02:11:24+02	2020-10-02 02:12:10+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-10-02 02:12:10+02	2020-10-02 02:13:31+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-10-02 02:11:03+02	2020-10-02 02:14:00+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-10-02 02:13:32+02	2020-10-02 02:14:29+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-10-02 02:14:01+02	2020-10-02 02:14:58+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-10-02 02:14:29+02	2020-10-02 02:15:27+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-10-02 02:14:59+02	2020-10-02 02:16:06+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-10-02 02:15:27+02	2020-10-02 02:16:22+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-10-02 02:16:23+02	2020-10-02 02:16:57+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-10-02 02:16:07+02	2020-10-02 02:17:09+02	net	remmina	\N	remmina-1.3.9
2020-10-02 02:16:57+02	2020-10-02 02:17:36+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-10-02 02:17:10+02	2020-10-02 02:18:07+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-10-02 02:18:08+02	2020-10-02 02:18:16+02	devel	lua-lpeg	lua52	lua52-lpeg-1.0.2_1
2020-10-02 02:18:17+02	2020-10-02 02:19:06+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-10-02 02:17:36+02	2020-10-02 02:19:24+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-10-02 02:19:24+02	2020-10-02 02:20:39+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-02 02:19:07+02	2020-10-02 02:20:42+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-10-02 02:20:39+02	2020-10-02 02:21:33+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0_1
2020-10-02 02:20:43+02	2020-10-02 02:21:48+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-10-02 02:21:34+02	2020-10-02 02:22:27+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-10-02 02:21:49+02	2020-10-02 02:22:44+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-10-02 02:22:45+02	2020-10-02 02:22:53+02	converters	lua-json	lua52	lua52-json-1.3.4_1
2020-10-02 02:22:53+02	2020-10-02 02:23:01+02	devel	lua-bitop	lua52	lua52-bitop-1.0.2_2
2020-10-02 02:22:28+02	2020-10-02 02:23:24+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-10-02 02:23:24+02	2020-10-02 02:23:34+02	net	luasocket	lua52	lua52-luasocket-3.0.r1_5,1
2020-10-02 02:23:34+02	2020-10-02 02:24:40+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-10-02 02:24:40+02	2020-10-02 02:24:48+02	textproc	luaexpat	lua52	lua52-luaexpat-1.3.0_5
2020-10-02 02:24:48+02	2020-10-02 02:25:54+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-02 02:25:54+02	2020-10-02 02:27:07+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-10-02 02:27:08+02	2020-10-02 02:29:07+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-10-02 02:23:02+02	2020-10-02 02:29:28+02	math	vtk8	\N	vtk8-8.2.0
2020-10-02 02:29:07+02	2020-10-02 02:30:15+02	devel	libpeas	\N	libpeas-1.22.0
2020-10-02 02:30:16+02	2020-10-02 02:33:37+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-10-02 02:33:37+02	2020-10-02 02:33:49+02	multimedia	libquvi-scripts09	\N	libquvi-scripts09-0.9.20131130_1
2020-10-02 02:29:29+02	2020-10-02 02:36:02+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-02 02:36:03+02	2020-10-02 02:37:03+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-10-02 02:33:49+02	2020-10-02 02:37:35+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-02 02:37:04+02	2020-10-02 02:38:06+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-10-02 02:38:06+02	2020-10-02 02:41:28+02	x11	xorg	\N	xorg-7.7_3
2020-10-02 02:41:29+02	2020-10-02 02:43:05+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-10-02 02:43:05+02	2020-10-02 02:44:07+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-10-02 02:37:35+02	2020-10-02 02:45:31+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-10-02 02:44:08+02	2020-10-02 02:45:35+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-02 02:45:32+02	2020-10-02 02:46:23+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-10-02 02:46:24+02	2020-10-02 02:47:41+02	mail	gmime30	\N	gmime30-3.2.7
2020-10-02 02:45:36+02	2020-10-02 02:47:51+02	graphics	cogl	\N	cogl-1.22.8
2020-10-02 02:47:51+02	2020-10-02 02:48:41+02	multimedia	libquvi09	\N	libquvi09-0.9.4_4
2020-10-02 02:48:42+02	2020-10-02 02:49:50+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-10-02 02:49:51+02	2020-10-02 02:50:55+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-10-02 02:47:42+02	2020-10-02 02:51:09+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_13,3
2020-10-02 02:50:56+02	2020-10-02 02:52:02+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-10-02 02:52:03+02	2020-10-02 02:53:02+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-10-02 02:53:02+02	2020-10-02 02:56:25+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-10-02 02:56:26+02	2020-10-02 02:57:36+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-10-02 02:57:37+02	2020-10-02 02:59:07+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-10-02 02:59:08+02	2020-10-02 03:00:10+02	audio	chromaprint	\N	chromaprint-1.5.0
2020-10-02 03:00:10+02	2020-10-02 03:01:38+02	audio	jack	\N	jackit-0.125.0_11
2020-10-02 03:01:39+02	2020-10-02 03:03:17+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-10-02 03:03:18+02	2020-10-02 03:05:21+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-10-02 03:05:22+02	2020-10-02 03:05:44+02	devel	py-lazy-object-proxy	py37	py37-lazy-object-proxy-1.5.1
2020-10-02 03:05:45+02	2020-10-02 03:06:24+02	devel	py-typed-ast	py37	py37-typed-ast-1.4.1
2020-10-02 03:06:25+02	2020-10-02 03:06:41+02	devel	py-wrapt	py37	py37-wrapt-1.12.1
2020-10-02 03:06:41+02	2020-10-02 03:10:10+02	graphics	clutter	\N	clutter-1.26.2
2020-10-02 02:51:10+02	2020-10-02 03:11:22+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-02 03:10:10+02	2020-10-02 03:11:34+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-10-02 03:11:24+02	2020-10-02 03:15:32+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-10-02 03:11:35+02	2020-10-02 03:15:32+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-10-02 03:15:33+02	2020-10-02 03:16:20+02	multimedia	totem-pl-parser	\N	totem-pl-parser-3.26.5
2020-10-02 03:16:20+02	2020-10-02 03:17:43+02	net	geocode-glib	\N	geocode-glib-3.26.2
2020-10-02 03:17:43+02	2020-10-02 03:19:19+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-02 03:19:19+02	2020-10-02 03:20:22+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-10-02 03:15:33+02	2020-10-02 03:20:54+02	lang	python38	\N	python38-3.8.6
2020-10-02 03:20:22+02	2020-10-02 03:21:36+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-10-02 03:21:37+02	2020-10-02 03:23:25+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-10-02 03:23:28+02	2020-10-02 03:26:48+02	x11-fm	caja	\N	caja-1.24.0
2020-10-02 01:14:59+02	2020-10-02 03:42:51+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-02 03:26:50+02	2020-10-02 03:44:00+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-10-02 03:44:01+02	2020-10-02 03:48:43+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-02 03:42:52+02	2020-10-02 03:51:06+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-02 03:48:44+02	2020-10-02 03:51:08+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-10-02 03:51:07+02	2020-10-02 03:54:56+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-02 03:54:57+02	2020-10-02 03:57:30+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-10-02 03:51:09+02	2020-10-02 03:57:36+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-02 03:57:31+02	2020-10-02 03:58:24+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-02 03:58:24+02	2020-10-02 03:59:58+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-10-02 03:59:59+02	2020-10-02 04:01:16+02	audio	gstreamer1-plugins-chromaprint	\N	gstreamer1-plugins-chromaprint-1.16.2
2020-10-02 04:01:17+02	2020-10-02 04:01:57+02	databases	gom	\N	gom-0.4
2020-10-02 04:01:58+02	2020-10-02 04:02:50+02	devel	dee	\N	dee-1.2.7_17
2020-10-02 04:02:51+02	2020-10-02 04:05:14+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-10-02 04:05:14+02	2020-10-02 04:06:38+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-10-02 04:06:39+02	2020-10-02 04:07:56+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-10-02 04:07:57+02	2020-10-02 04:08:13+02	devel	py-astroid	py37	py37-astroid-2.4.2_1
2020-10-02 04:08:13+02	2020-10-02 04:08:27+02	devel	py-isort	py37	py37-isort-4.3.21
2020-10-02 04:08:27+02	2020-10-02 04:08:41+02	devel	py-mccabe	py37	py37-mccabe-0.6.1
2020-10-02 04:08:41+02	2020-10-02 04:09:03+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-10-02 04:09:03+02	2020-10-02 04:09:17+02	devel	py-xdg	py37	py37-xdg-0.26
2020-10-02 04:09:17+02	2020-10-02 04:09:52+02	graphics	simage	\N	simage-1.7.0_8
2020-10-02 04:09:53+02	2020-10-02 04:11:06+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-10-02 04:11:07+02	2020-10-02 04:12:22+02	mail	gmime26	\N	gmime26-2.6.23_1
2020-10-02 04:12:22+02	2020-10-02 04:13:31+02	math	octomap	\N	octomap-1.9.5.190
2020-10-02 04:13:32+02	2020-10-02 04:14:25+02	multimedia	libmediaart	\N	libmediaart-1.9.4
2020-10-02 04:14:25+02	2020-10-02 04:15:22+02	net	grilo	\N	grilo-0.3.13
2020-10-02 04:15:23+02	2020-10-02 04:17:20+02	net	libgweather	\N	libgweather-3.36.1
2020-10-02 04:17:21+02	2020-10-02 04:19:28+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-02 04:19:28+02	2020-10-02 04:21:18+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-10-02 04:21:19+02	2020-10-02 04:23:51+02	sysutils	tracker	\N	tracker-2.3.4
2020-10-02 04:23:51+02	2020-10-02 04:25:41+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-10-02 04:25:42+02	2020-10-02 04:26:53+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-10-02 04:26:54+02	2020-10-02 04:28:23+02	x11	zenity	\N	zenity-3.32.0
2020-10-02 04:28:23+02	2020-10-02 04:29:33+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-10-02 04:29:34+02	2020-10-02 04:31:01+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-10-02 04:31:02+02	2020-10-02 04:33:10+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-10-02 04:33:11+02	2020-10-02 04:35:45+02	audio	sox	\N	sox-14.4.2_5
2020-10-02 04:35:45+02	2020-10-02 04:36:38+02	devel	gstreamer1-plugins-soup	\N	gstreamer1-plugins-soup-1.16.2
2020-10-02 04:36:39+02	2020-10-02 04:41:33+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-10-02 04:41:34+02	2020-10-02 04:42:32+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-10-02 04:42:32+02	2020-10-02 04:43:49+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-02 04:43:49+02	2020-10-02 04:44:09+02	devel	pylint	py37	pylint-py37-2.6.0
2020-10-02 04:44:09+02	2020-10-02 04:45:14+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-10-02 04:45:14+02	2020-10-02 04:46:25+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-10-02 04:46:26+02	2020-10-02 04:47:29+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-10-02 04:47:30+02	2020-10-02 04:49:44+02	devel	shiboken2	py37	py37-shiboken2-5.15.0
2020-10-02 04:49:44+02	2020-10-02 04:55:05+02	dns	bind-tools	\N	bind-tools-9.16.7
2020-10-02 04:55:06+02	2020-10-02 05:00:32+02	editors	emacs	full	emacs-27.1_1,3
2020-10-02 03:20:55+02	2020-10-02 05:10:30+02	www	node	\N	node-14.13.0
2020-10-02 05:10:31+02	2020-10-02 05:11:42+02	graphics	clutter-gtk3	\N	clutter-gtk3-1.8.4
2020-10-02 05:00:33+02	2020-10-02 05:13:14+02	graphics	Coin	\N	Coin-4.0.0_1
2020-10-02 05:11:42+02	2020-10-02 05:13:19+02	graphics	gegl	\N	gegl-0.4.26_5
2020-10-02 05:13:19+02	2020-10-02 05:14:08+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-10-02 05:14:09+02	2020-10-02 05:15:05+02	math	fcl05	\N	fcl05-0.5.0
2020-10-02 05:15:05+02	2020-10-02 05:16:12+02	multimedia	clutter-gst3	\N	clutter-gst3-3.0.27
2020-10-02 05:13:15+02	2020-10-02 05:17:40+02	graphics	opencv	\N	opencv-3.4.1_36
2020-10-02 05:16:13+02	2020-10-02 05:17:54+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-02 05:17:41+02	2020-10-02 05:18:36+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-02 05:17:54+02	2020-10-02 05:18:37+02	print	cups-filters	\N	cups-filters-1.28.3
2020-10-02 05:18:36+02	2020-10-02 05:21:58+02	print	fontforge	\N	fontforge-20190801_2
2020-10-02 05:21:59+02	2020-10-02 05:23:51+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0
2020-10-02 05:23:51+02	2020-10-02 05:25:37+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-02 05:25:38+02	2020-10-02 05:27:10+02	sysutils	zeitgeist	\N	zeitgeist-1.0.2
2020-10-02 03:57:37+02	2020-10-02 05:30:42+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-02 05:18:38+02	2020-10-02 05:35:25+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-10-02 05:35:26+02	2020-10-02 05:37:08+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-10-02 05:37:09+02	2020-10-02 05:38:54+02	x11	yelp	\N	yelp-3.38.0
2020-10-02 05:38:55+02	2020-10-02 05:39:49+02	x11-themes	gnome-icon-theme-symbolic	\N	gnome-icon-theme-symbolic-3.12.0
2020-10-02 05:39:49+02	2020-10-02 05:41:31+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-10-02 05:41:32+02	2020-10-02 05:42:40+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-10-02 05:42:41+02	2020-10-02 05:44:27+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-10-02 05:44:28+02	2020-10-02 05:46:23+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-10-02 05:46:24+02	2020-10-02 05:48:06+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-10-02 05:48:07+02	2020-10-02 05:49:12+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0_1
2020-10-02 05:49:12+02	2020-10-02 06:04:19+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-10-02 06:04:20+02	2020-10-02 06:05:36+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-10-02 06:05:37+02	2020-10-02 06:07:06+02	x11-wm	marco	\N	marco-1.22.4
2020-10-02 06:07:07+02	2020-10-02 06:08:11+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-10-02 06:08:14+02	2020-10-02 06:08:42+02	audio	twolame	\N	twolame-0.4.0_1
2020-10-02 05:27:11+02	2020-10-02 06:08:45+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-02 06:08:49+02	2020-10-02 06:09:53+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-10-02 06:08:42+02	2020-10-02 06:10:37+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-02 06:09:54+02	2020-10-02 06:12:05+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-02 06:12:06+02	2020-10-02 06:25:48+02	devel	pyside2	py37	py37-pyside2-5.15.0
2020-10-02 06:25:49+02	2020-10-02 06:35:57+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-10-02 06:35:58+02	2020-10-02 06:38:09+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-10-02 06:38:10+02	2020-10-02 06:46:58+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-10-02 06:47:00+02	2020-10-02 06:48:38+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-10-02 06:48:39+02	2020-10-02 06:49:49+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-10-02 05:30:42+02	2020-10-02 06:51:07+02	www	node12	\N	node12-12.18.4
2020-10-02 06:49:49+02	2020-10-02 06:55:54+02	lang	cython	py38	py38-cython-0.29.15
2020-10-02 06:55:55+02	2020-10-02 06:58:22+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-10-02 06:58:27+02	2020-10-02 07:16:44+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-02 07:16:45+02	2020-10-02 07:17:55+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-02 07:17:56+02	2020-10-02 07:21:42+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-02 07:21:43+02	2020-10-02 07:22:54+02	net	freerdp	\N	freerdp-2.2.0_1
2020-10-02 07:22:54+02	2020-10-02 07:23:49+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-02 07:23:49+02	2020-10-02 07:25:20+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-02 07:25:21+02	2020-10-02 07:26:11+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-02 06:51:07+02	2020-10-02 07:28:20+02	math	R	\N	R-4.0.2
2020-10-02 07:28:21+02	2020-10-02 07:28:52+02	print	libspectre	\N	libspectre-0.2.9
2020-10-02 07:26:12+02	2020-10-02 07:30:04+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-02 07:28:53+02	2020-10-02 07:31:24+02	print	mftrace	\N	mftrace-1.2.18_2
2020-10-02 07:31:25+02	2020-10-02 07:34:41+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-02 07:34:41+02	2020-10-02 07:35:39+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-02 07:30:05+02	2020-10-02 07:37:17+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-10-02 07:35:39+02	2020-10-02 07:37:44+02	textproc	dblatex	\N	dblatex-0.3.11
2020-10-02 07:37:18+02	2020-10-02 07:37:59+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-10-02 07:38:00+02	2020-10-02 07:38:27+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-10-02 07:37:44+02	2020-10-02 07:38:50+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-10-02 07:38:27+02	2020-10-02 07:39:35+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-10-02 07:38:51+02	2020-10-02 07:39:45+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-10-02 07:39:35+02	2020-10-02 07:40:36+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.13.1_1
2020-10-02 07:39:46+02	2020-10-02 07:40:50+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-10-02 07:40:36+02	2020-10-02 07:43:42+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-10-02 07:43:43+02	2020-10-02 07:46:37+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-02 07:46:38+02	2020-10-02 07:47:01+02	audio	dssi	\N	dssi-1.1.1_9
2020-10-02 07:47:01+02	2020-10-02 07:47:19+02	audio	faudio	\N	FAudio-20.09
2020-10-02 07:40:51+02	2020-10-02 07:47:20+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-02 07:47:20+02	2020-10-02 07:48:22+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-02 07:47:21+02	2020-10-02 07:48:32+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-02 07:48:23+02	2020-10-02 07:49:42+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-02 07:49:42+02	2020-10-02 07:50:52+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-02 07:48:33+02	2020-10-02 07:51:51+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-02 07:50:53+02	2020-10-02 07:51:54+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-02 07:51:52+02	2020-10-02 07:52:40+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-10-02 07:51:54+02	2020-10-02 07:52:52+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-10-02 07:52:40+02	2020-10-02 07:54:07+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-02 07:52:52+02	2020-10-02 07:54:45+02	devel	pyside2-tools	py37	py37-pyside2-tools-5.15.0
2020-10-02 07:54:08+02	2020-10-02 08:20:11+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-10-02 07:54:45+02	2020-10-02 08:20:37+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-10-02 08:20:39+02	2020-10-02 08:23:24+02	graphics	atril	\N	atril-1.22.3_9
2020-10-02 08:20:14+02	2020-10-02 08:24:51+02	editors	pluma	\N	pluma-1.24.1
2020-10-02 08:24:51+02	2020-10-02 08:27:24+02	graphics	eom	\N	eom-1.24.1
2020-10-02 08:27:25+02	2020-10-02 08:29:36+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-10-02 08:29:37+02	2020-10-02 08:30:46+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-10-02 08:30:47+02	2020-10-02 08:34:21+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-10-02 08:34:22+02	2020-10-02 08:50:12+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-10-02 08:23:25+02	2020-10-02 08:53:50+02	graphics	embree	\N	embree-3.12.0
2020-10-02 08:53:51+02	2020-10-02 08:55:50+02	math	mate-calc	\N	mate-calc-1.24.1
2020-10-02 08:50:13+02	2020-10-02 08:59:58+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-10-02 08:55:50+02	2020-10-02 09:00:06+02	math	py-numpy	py38	py38-numpy-1.16.6,1
2020-10-02 08:59:58+02	2020-10-02 09:01:39+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-10-02 09:00:06+02	2020-10-02 09:02:00+02	multimedia	mpv	\N	mpv-0.32.0_5,1
2020-10-02 09:02:01+02	2020-10-02 09:03:09+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-02 09:03:10+02	2020-10-02 09:04:14+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-02 09:04:14+02	2020-10-02 09:05:20+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-02 09:05:20+02	2020-10-02 09:06:29+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-02 09:06:30+02	2020-10-02 09:07:41+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-02 09:07:41+02	2020-10-02 09:08:39+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-02 09:08:39+02	2020-10-02 09:09:41+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-02 09:09:41+02	2020-10-02 09:11:04+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-02 09:01:41+02	2020-10-02 09:12:01+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-10-02 09:11:04+02	2020-10-02 09:12:40+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-02 09:12:01+02	2020-10-02 09:12:59+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-02 09:12:41+02	2020-10-02 09:14:05+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-02 09:13:00+02	2020-10-02 09:14:24+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-10-02 09:14:06+02	2020-10-02 09:15:21+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-02 09:14:25+02	2020-10-02 09:20:49+02	print	lilypond	\N	lilypond-2.20.0
2020-10-02 09:20:50+02	2020-10-02 09:23:08+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-10-02 09:15:21+02	2020-10-02 09:24:06+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-10-02 09:23:09+02	2020-10-02 09:33:14+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-10-02 09:33:15+02	2020-10-02 09:33:51+02	print	transfig	\N	transfig-3.2.5e_4
2020-10-02 09:33:51+02	2020-10-02 09:34:51+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-02 09:34:51+02	2020-10-02 09:36:37+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-02 09:36:37+02	2020-10-02 09:39:18+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-10-02 09:39:19+02	2020-10-02 09:40:20+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-10-02 09:40:21+02	2020-10-02 09:41:17+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-10-02 09:41:18+02	2020-10-02 09:42:01+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-10-02 09:42:02+02	2020-10-02 09:42:30+02	www	npm-node12	\N	npm-node12-6.14.8
2020-10-02 09:42:30+02	2020-10-02 09:43:34+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-10-02 09:43:34+02	2020-10-02 09:44:48+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-10-02 09:44:48+02	2020-10-02 09:45:04+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-10-02 09:24:07+02	2020-10-02 09:45:54+02	print	texlive-docs	\N	texlive-docs-20150523
2020-10-02 09:45:55+02	2020-10-02 09:47:28+02	x11	mate-base	\N	mate-base-1.24.0
2020-10-02 09:47:29+02	2020-10-02 09:48:56+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-10-02 09:48:56+02	2020-10-02 09:51:02+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-10-02 09:45:04+02	2020-10-02 09:51:21+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-10-02 09:51:21+02	2020-10-02 09:52:22+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-02 09:52:23+02	2020-10-02 09:53:14+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-10-02 09:51:03+02	2020-10-02 09:55:08+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-02 09:53:15+02	2020-10-02 10:04:07+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-10-02 10:04:07+02	2020-10-02 10:09:53+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-02 10:09:54+02	2020-10-02 10:12:46+02	cad	cura	\N	Cura-4.7.1,2
2020-10-02 09:55:09+02	2020-10-02 10:29:00+02	astro	stellarium	\N	stellarium-0.20.3
2020-10-02 10:29:00+02	2020-10-02 10:35:06+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-02 10:12:47+02	2020-10-02 10:37:04+02	cad	freecad	\N	FreeCAD-0.18.4_10
2020-10-02 10:35:07+02	2020-10-02 10:37:38+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-02 10:37:39+02	2020-10-02 10:39:31+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-02 10:39:32+02	2020-10-02 10:41:20+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-02 10:41:21+02	2020-10-02 10:45:39+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-02 10:45:39+02	2020-10-02 10:48:07+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-10-02 10:48:07+02	2020-10-02 10:49:15+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-10-02 10:49:16+02	2020-10-02 11:17:21+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-02 11:17:22+02	2020-10-02 11:26:21+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-02 11:26:21+02	2020-10-02 11:27:36+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-02 11:27:36+02	2020-10-02 11:54:11+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-02 06:10:37+02	2020-10-02 12:06:08+02	devel	llvm11	\N	llvm11-11.0.0.r4
2020-10-02 10:37:05+02	2020-10-02 12:20:30+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-02 12:06:09+02	2020-10-02 12:20:49+02	databases	postgresql12-server	\N	postgresql12-server-12.4_1
2020-10-02 12:20:31+02	2020-10-02 12:21:17+02	devel	tortoisehg	py37	tortoisehg-5.5.1_1
2020-10-02 12:20:50+02	2020-10-02 12:25:35+02	dns	bind916	\N	bind916-9.16.7_1
2020-10-02 12:25:36+02	2020-10-02 12:28:25+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-10-02 12:28:26+02	2020-10-02 12:30:45+02	editors	vim	\N	vim-8.2.1558
2020-10-02 12:21:17+02	2020-10-02 12:31:03+02	editors	codelite	\N	codelite-12.0_4
2020-10-02 12:30:46+02	2020-10-02 12:41:27+02	emulators	qemu	\N	qemu-4.2.1
2020-10-02 12:41:28+02	2020-10-02 12:52:18+02	emulators	wine	\N	wine-5.0.2_1,1
2020-10-02 12:52:18+02	2020-10-02 13:10:43+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-02 13:10:44+02	2020-10-02 13:19:26+02	games	battletanks	\N	btanks-0.9.8083_9
2020-10-02 13:19:27+02	2020-10-02 13:20:24+02	games	xcowsay	\N	xcowsay-1.5.1
2020-10-02 13:20:24+02	2020-10-02 13:46:33+02	graphics	blender	\N	blender-2.90.0_1
2020-10-02 12:31:03+02	2020-10-02 13:46:43+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_3
2020-10-02 13:46:44+02	2020-10-02 13:47:56+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-02 13:46:34+02	2020-10-02 13:52:10+02	graphics	dia	\N	dia-0.97.3_1,1
2020-10-02 13:52:11+02	2020-10-02 14:06:11+02	graphics	krita	\N	krita-4.3.0_5
2020-10-02 14:06:12+02	2020-10-02 14:10:52+02	graphics	wings	\N	wings-2.2.5
2020-10-02 14:10:53+02	2020-10-02 14:12:16+02	graphics	xpdf	\N	xpdf-4.02,1
2020-10-02 14:12:17+02	2020-10-02 14:13:38+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-02 13:47:57+02	2020-10-02 14:45:44+02	graphics	inkscape	\N	inkscape-1.0.1
2020-10-02 14:13:39+02	2020-10-02 14:55:52+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.10
2020-10-02 14:45:45+02	2020-10-02 15:31:50+02	lang	erlang-runtime23	\N	erlang-runtime23-23.1
2020-10-02 15:31:51+02	2020-10-02 15:38:56+02	math	maxima	\N	maxima-5.44.0_5
2020-10-02 15:38:57+02	2020-10-02 15:40:14+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-02 15:40:14+02	2020-10-02 16:12:34+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.0
2020-10-02 16:12:34+02	2020-10-02 16:21:43+02	multimedia	obs-studio	\N	obs-studio-26.0.0
2020-10-02 16:21:44+02	2020-10-02 16:24:51+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-10-02 16:24:52+02	2020-10-02 16:34:39+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-02 16:34:40+02	2020-10-02 16:36:33+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-02 16:36:33+02	2020-10-02 16:44:45+02	net	wireshark	\N	wireshark-3.2.7
2020-10-02 16:44:46+02	2020-10-02 16:45:48+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-02 16:45:49+02	2020-10-02 17:09:21+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-10-02 17:09:22+02	2020-10-02 17:10:20+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-02 17:10:20+02	2020-10-02 17:11:22+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-02 17:11:23+02	2020-10-02 17:12:20+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-02 17:12:21+02	2020-10-02 17:13:19+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-02 17:13:19+02	2020-10-02 17:14:18+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-02 17:14:19+02	2020-10-02 17:15:22+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-02 17:15:23+02	2020-10-02 17:16:24+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-02 17:16:25+02	2020-10-02 17:18:10+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-02 17:18:11+02	2020-10-02 17:19:15+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-02 17:19:16+02	2020-10-02 17:20:17+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-02 17:20:17+02	2020-10-02 17:21:28+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-10-02 17:21:28+02	2020-10-02 17:24:53+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-02 17:24:54+02	2020-10-02 17:25:47+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-10-02 17:25:48+02	2020-10-02 17:26:47+02	print	gv	\N	gv-3.7.4_5
2020-10-02 17:26:48+02	2020-10-02 17:31:58+02	print	lyx	\N	lyx-2.3.4.2_1
2020-10-02 17:31:59+02	2020-10-02 17:43:56+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-10-02 17:43:57+02	2020-10-02 17:49:54+02	print	texlive-full	\N	texlive-full-20150521
2020-10-02 17:49:55+02	2020-10-02 17:51:05+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-10-02 17:51:06+02	2020-10-02 17:52:22+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-02 17:52:22+02	2020-10-02 17:53:33+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-10-02 17:53:34+02	2020-10-02 17:55:27+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-02 17:55:28+02	2020-10-02 17:58:27+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-02 17:58:27+02	2020-10-02 17:58:36+02	sysutils	reptyr	\N	reptyr-0.8.0
2020-10-02 17:58:37+02	2020-10-02 18:00:24+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-10-02 14:55:52+02	2020-10-02 18:54:13+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20200927
2020-10-02 18:54:14+02	2020-10-02 18:55:37+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-10-02 18:55:37+02	2020-10-02 18:57:38+02	x11	mate	\N	mate-1.22.2
2020-10-02 18:57:38+02	2020-10-02 19:05:38+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-10-02 19:05:39+02	2020-10-02 19:12:21+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-02 19:12:22+02	2020-10-02 19:14:42+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-10-02 19:14:43+02	2020-10-02 19:15:09+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.6_6
2020-10-02 19:15:10+02	2020-10-02 19:35:42+02	databases	postgresql13-server	\N	postgresql13-server-13.0_2
2020-10-02 19:35:43+02	2020-10-02 20:13:58+02	editors	vscode	\N	vscode-1.46.1
2020-10-02 20:13:59+02	2020-10-02 20:15:08+02	graphics	xfig	\N	xfig-3.2.7a
2020-10-02 18:00:25+02	2020-10-02 20:48:45+02	www	firefox	\N	firefox-81.0.1,2
2020-10-02 11:54:22+02	2020-10-02 20:48:57+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200925
2020-10-02 20:48:58+02	2020-10-02 21:05:06+02	local	E590T-localbase	\N	E590T-localbase-2020092700
2020-10-02 21:05:21+02	2020-10-02 21:06:16+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-02 21:05:21+02	2020-10-02 21:07:26+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20200914
2020-10-02 21:07:26+02	2020-10-02 21:07:33+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-03 01:15:12+02	2020-10-03 01:15:19+02	security	base-audit	\N	base-audit-0.4
2020-10-03 01:15:12+02	2020-10-03 01:15:19+02	misc	pciids	\N	pciids-20200922
2020-10-03 01:15:19+02	2020-10-03 01:15:32+02	devel	libpciaccess	\N	libpciaccess-0.16
2020-10-03 01:15:12+02	2020-10-03 01:15:42+02	x11	xterm	\N	xterm-360_1
2020-10-03 01:15:43+02	2020-10-03 01:15:54+02	devel	libpci	\N	libpci-3.7.0
2020-10-03 01:15:32+02	2020-10-03 01:15:57+02	graphics	libdrm	\N	libdrm-2.4.102,1
2020-10-03 01:15:19+02	2020-10-03 01:16:20+02	sysutils	hal	\N	hal-0.5.14_34
2020-10-03 01:15:55+02	2020-10-03 01:16:43+02	devel	hwloc	\N	hwloc-1.11.11
2020-10-03 01:16:20+02	2020-10-03 01:16:43+02	multimedia	libva	\N	libva-2.9.0
2020-10-03 01:16:43+02	2020-10-03 01:22:03+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-10-03 01:15:57+02	2020-10-03 01:22:39+02	graphics	mesa-libs	\N	mesa-libs-19.0.8_3
2020-10-03 01:22:03+02	2020-10-03 01:23:09+02	multimedia	cmrt	\N	cmrt-1.0.6_2
2020-10-03 01:23:10+02	2020-10-03 01:24:05+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-10-03 01:16:44+02	2020-10-03 01:24:45+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-10-03 01:22:40+02	2020-10-03 01:25:07+02	graphics	cairo	\N	cairo-1.16.0_1,3
2020-10-03 01:24:46+02	2020-10-03 01:25:18+02	graphics	libGLU	\N	libGLU-9.0.1
2020-10-03 01:25:08+02	2020-10-03 01:28:07+02	x11-toolkits	pango	\N	pango-1.42.4_5
2020-10-03 01:25:19+02	2020-10-03 01:28:09+02	graphics	poppler	\N	poppler-20.09.0
2020-10-03 01:28:09+02	2020-10-03 01:29:32+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-10-03 01:28:08+02	2020-10-03 01:30:32+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-10-03 01:29:32+02	2020-10-03 01:30:48+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.38.0
2020-10-03 01:24:05+02	2020-10-03 01:31:04+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0_1
2020-10-03 01:30:33+02	2020-10-03 01:32:03+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-10-03 01:30:48+02	2020-10-03 01:35:51+02	print	texlive-base	\N	texlive-base-20150521_55
2020-10-03 01:32:05+02	2020-10-03 01:36:28+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-10-03 01:31:04+02	2020-10-03 01:37:37+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-10-03 01:37:38+02	2020-10-03 01:42:21+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-10-03 01:42:22+02	2020-10-03 01:43:29+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-10-03 01:36:28+02	2020-10-03 01:46:09+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-10-03 01:43:29+02	2020-10-03 01:47:16+02	graphics	graphviz	\N	graphviz-2.44.1_2
2020-10-03 01:46:10+02	2020-10-03 01:47:25+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-10-03 01:47:17+02	2020-10-03 01:47:34+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-10-03 01:47:35+02	2020-10-03 01:48:03+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-10-03 01:47:25+02	2020-10-03 01:48:32+02	graphics	frei0r	\N	frei0r-1.6.1
2020-10-03 01:48:04+02	2020-10-03 01:49:02+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-10-03 01:49:03+02	2020-10-03 01:50:32+02	graphics	graphene	\N	graphene-1.10.2
2020-10-03 01:50:33+02	2020-10-03 01:50:53+02	www	libwpe	\N	libwpe-1.4.0.1
2020-10-03 01:50:53+02	2020-10-03 01:52:25+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2_1
2020-10-03 01:52:25+02	2020-10-03 01:52:51+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-10-03 01:52:52+02	2020-10-03 01:53:27+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-10-03 01:48:33+02	2020-10-03 01:54:45+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-10-03 01:35:52+02	2020-10-03 01:54:45+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-10-03 01:54:45+02	2020-10-03 01:55:28+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-10-03 01:54:46+02	2020-10-03 01:57:29+02	print	tex-formats	\N	tex-formats-20150521_2
2020-10-03 01:55:28+02	2020-10-03 02:01:38+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-10-03 01:57:30+02	2020-10-03 02:03:29+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-10-03 02:01:39+02	2020-10-03 02:07:36+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-10-03 02:07:36+02	2020-10-03 02:08:58+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-10-03 02:03:29+02	2020-10-03 02:09:11+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-10-03 02:09:12+02	2020-10-03 02:11:19+02	textproc	docbook-utils	\N	docbook-utils-0.6.14_13
2020-10-03 02:11:20+02	2020-10-03 02:12:36+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-10-03 02:12:37+02	2020-10-03 02:18:50+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-10-03 02:18:51+02	2020-10-03 02:20:00+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-10-03 02:20:02+02	2020-10-03 02:20:50+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-10-03 01:53:27+02	2020-10-03 02:21:36+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-03 02:08:59+02	2020-10-03 02:22:28+02	graphics	mesa-dri	\N	mesa-dri-19.0.8_9
2020-10-03 02:20:51+02	2020-10-03 02:22:51+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-10-03 02:22:28+02	2020-10-03 02:23:18+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-10-03 02:23:19+02	2020-10-03 02:24:05+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-10-03 02:22:51+02	2020-10-03 02:26:15+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-10-03 02:21:37+02	2020-10-03 02:27:17+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-03 02:26:15+02	2020-10-03 02:27:47+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-10-03 02:27:18+02	2020-10-03 02:28:21+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-10-03 02:24:06+02	2020-10-03 02:28:32+02	x11-servers	xorg-server	\N	xorg-server-1.20.9,1
2020-10-03 02:27:48+02	2020-10-03 02:28:34+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-10-03 02:28:21+02	2020-10-03 02:29:00+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-10-03 02:28:34+02	2020-10-03 02:29:30+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-10-03 02:28:33+02	2020-10-03 02:29:35+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-10-03 02:29:30+02	2020-10-03 02:30:36+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-03 02:29:36+02	2020-10-03 02:30:41+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-10-03 02:30:42+02	2020-10-03 02:31:38+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-10-03 02:30:36+02	2020-10-03 02:32:49+02	security	gcr	\N	gcr-3.28.0_1
2020-10-03 02:29:01+02	2020-10-03 02:33:29+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-10-03 02:32:49+02	2020-10-03 02:34:10+02	x11-toolkits	vte3	\N	vte3-0.62.0
2020-10-03 02:33:29+02	2020-10-03 02:34:46+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-10-03 02:34:47+02	2020-10-03 02:35:44+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-10-03 02:35:45+02	2020-10-03 02:36:35+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-10-03 02:36:35+02	2020-10-03 02:39:06+02	devel	sdl20	\N	sdl2-2.0.12_2
2020-10-03 02:34:10+02	2020-10-03 02:39:20+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-10-03 02:39:06+02	2020-10-03 02:39:33+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-10-03 02:31:39+02	2020-10-03 02:39:59+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-10-03 02:39:20+02	2020-10-03 02:40:00+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-10-03 02:39:59+02	2020-10-03 02:40:50+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-10-03 02:40:00+02	2020-10-03 02:40:54+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-10-03 02:39:33+02	2020-10-03 02:40:59+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-10-03 02:40:59+02	2020-10-03 02:41:34+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-10-03 02:40:55+02	2020-10-03 02:42:10+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-10-03 02:41:34+02	2020-10-03 02:42:12+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-10-03 02:42:12+02	2020-10-03 02:43:07+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-10-03 02:42:10+02	2020-10-03 02:43:10+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-10-03 02:40:51+02	2020-10-03 02:43:16+02	graphics	glew	\N	glew-2.2.0
2020-10-03 02:43:10+02	2020-10-03 02:44:16+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-10-03 02:43:07+02	2020-10-03 02:44:28+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-10-03 02:43:16+02	2020-10-03 02:44:34+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-10-03 02:44:29+02	2020-10-03 02:44:44+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-10-03 02:44:17+02	2020-10-03 02:45:17+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-10-03 02:44:44+02	2020-10-03 02:45:40+02	net	remmina	\N	remmina-1.3.9
2020-10-03 02:44:34+02	2020-10-03 02:45:43+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-10-03 02:45:44+02	2020-10-03 02:46:08+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-10-03 02:45:17+02	2020-10-03 02:46:09+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-10-03 02:45:41+02	2020-10-03 02:49:09+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-10-03 02:49:10+02	2020-10-03 02:49:25+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-10-03 02:46:08+02	2020-10-03 02:49:48+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-10-03 02:49:25+02	2020-10-03 02:50:18+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-03 02:49:49+02	2020-10-03 02:50:35+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-10-03 02:50:18+02	2020-10-03 02:51:11+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0_1
2020-10-03 02:50:36+02	2020-10-03 02:51:23+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-10-03 02:51:11+02	2020-10-03 02:52:04+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-10-03 02:52:04+02	2020-10-03 02:52:57+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-10-03 02:51:24+02	2020-10-03 02:53:23+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-10-03 02:52:58+02	2020-10-03 02:53:56+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.5.0
2020-10-03 02:53:24+02	2020-10-03 02:54:38+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-10-03 02:53:56+02	2020-10-03 02:54:55+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-10-03 02:54:38+02	2020-10-03 02:55:37+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-10-03 02:54:56+02	2020-10-03 02:56:44+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-03 02:55:37+02	2020-10-03 02:56:59+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-03 02:56:45+02	2020-10-03 02:57:41+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-10-03 02:57:18+02	2020-10-03 02:57:42+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-10-03 02:57:41+02	2020-10-03 02:58:25+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-03 02:46:09+02	2020-10-03 02:58:30+02	net	openmpi	\N	openmpi-4.0.5
2020-10-03 02:58:26+02	2020-10-03 02:59:31+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-10-03 02:57:42+02	2020-10-03 02:59:37+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.9,1
2020-10-03 02:59:38+02	2020-10-03 03:00:28+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-10-03 02:59:31+02	2020-10-03 03:01:09+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-10-03 03:00:29+02	2020-10-03 03:01:54+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-10-03 03:01:10+02	2020-10-03 03:02:39+02	devel	libpeas	\N	libpeas-1.22.0
2020-10-03 03:02:40+02	2020-10-03 03:04:10+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-10-03 03:04:11+02	2020-10-03 03:06:59+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-10-03 03:01:55+02	2020-10-03 03:07:17+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-03 03:07:18+02	2020-10-03 03:11:17+02	x11	xorg	\N	xorg-7.7_3
2020-10-03 03:11:17+02	2020-10-03 03:12:58+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-10-03 03:12:58+02	2020-10-03 03:14:20+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-10-03 03:14:21+02	2020-10-03 03:15:09+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-10-03 03:15:10+02	2020-10-03 03:17:07+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-03 03:17:08+02	2020-10-03 03:18:29+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-10-03 03:18:30+02	2020-10-03 03:20:58+02	graphics	cogl	\N	cogl-1.22.8
2020-10-03 03:20:59+02	2020-10-03 03:22:26+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-10-03 03:22:27+02	2020-10-03 03:23:53+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-10-03 03:23:54+02	2020-10-03 03:25:25+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-10-03 02:58:30+02	2020-10-03 03:42:30+02	math	vtk8	\N	vtk8-8.2.0
2020-10-03 03:06:59+02	2020-10-03 03:51:39+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-10-03 03:51:41+02	2020-10-03 03:53:24+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-10-03 03:53:25+02	2020-10-03 03:57:07+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-10-03 03:57:07+02	2020-10-03 03:58:25+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-10-03 03:58:26+02	2020-10-03 04:00:01+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-10-03 04:00:03+02	2020-10-03 04:00:44+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-10-03 04:00:45+02	2020-10-03 04:08:34+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-03 04:08:35+02	2020-10-03 04:09:14+02	audio	chromaprint	\N	chromaprint-1.5.0
2020-10-03 04:09:15+02	2020-10-03 04:10:44+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-10-03 04:10:44+02	2020-10-03 04:12:54+02	graphics	clutter	\N	clutter-1.26.2
2020-10-03 04:12:54+02	2020-10-03 04:13:58+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-10-03 04:13:59+02	2020-10-03 04:14:40+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-10-03 04:14:41+02	2020-10-03 04:15:25+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-10-03 03:25:26+02	2020-10-03 04:18:24+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-03 04:15:26+02	2020-10-03 04:18:48+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-10-03 04:18:25+02	2020-10-03 04:18:49+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-10-03 04:18:48+02	2020-10-03 04:19:32+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-03 04:18:50+02	2020-10-03 04:19:35+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-10-03 04:19:33+02	2020-10-03 04:20:28+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-10-03 04:19:35+02	2020-10-03 04:20:46+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-10-03 04:20:29+02	2020-10-03 04:22:26+02	x11-fm	caja	\N	caja-1.24.0
2020-10-03 04:20:46+02	2020-10-03 04:22:29+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-10-03 04:22:29+02	2020-10-03 04:23:25+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-03 04:22:26+02	2020-10-03 04:23:46+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-10-03 04:23:46+02	2020-10-03 04:24:39+02	audio	gstreamer1-plugins-chromaprint	\N	gstreamer1-plugins-chromaprint-1.16.2
2020-10-03 04:23:26+02	2020-10-03 04:24:43+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-10-03 04:24:39+02	2020-10-03 04:24:57+02	databases	gom	\N	gom-0.4
2020-10-03 04:24:44+02	2020-10-03 04:25:05+02	devel	dee	\N	dee-1.2.7_17
2020-10-03 04:25:06+02	2020-10-03 04:25:59+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-10-03 04:24:58+02	2020-10-03 04:26:17+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-10-03 03:42:32+02	2020-10-03 04:26:33+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-03 04:25:59+02	2020-10-03 04:26:48+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-10-03 04:26:48+02	2020-10-03 04:27:38+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-10-03 04:27:38+02	2020-10-03 04:28:37+02	math	octomap	\N	octomap-1.9.5.190
2020-10-03 04:28:38+02	2020-10-03 04:29:58+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-10-03 04:30:00+02	2020-10-03 04:31:35+02	net	libgweather	\N	libgweather-3.36.1
2020-10-03 04:31:36+02	2020-10-03 04:31:56+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-10-03 04:31:57+02	2020-10-03 04:34:15+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-03 04:34:16+02	2020-10-03 04:35:49+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-10-03 04:35:51+02	2020-10-03 04:36:55+02	sysutils	tracker	\N	tracker-2.3.4
2020-10-03 04:36:55+02	2020-10-03 04:38:26+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-10-03 04:26:18+02	2020-10-03 04:38:58+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-10-03 04:38:29+02	2020-10-03 04:39:27+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-10-03 04:38:58+02	2020-10-03 04:40:05+02	x11	zenity	\N	zenity-3.32.0
2020-10-03 04:39:27+02	2020-10-03 04:40:17+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-10-03 04:40:05+02	2020-10-03 04:40:57+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-10-03 04:26:34+02	2020-10-03 04:40:59+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-03 04:40:18+02	2020-10-03 04:41:26+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-10-03 04:41:00+02	2020-10-03 04:41:37+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-10-03 04:41:27+02	2020-10-03 04:42:09+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-03 04:41:37+02	2020-10-03 04:42:18+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-10-03 04:42:09+02	2020-10-03 04:43:04+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-10-03 04:42:19+02	2020-10-03 04:43:13+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-10-03 04:40:58+02	2020-10-03 04:43:53+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-10-03 04:43:14+02	2020-10-03 04:47:15+02	editors	emacs	full	emacs-27.1_1,3
2020-10-03 04:43:55+02	2020-10-03 04:47:53+02	graphics	Coin	\N	Coin-4.0.0_1
2020-10-03 04:47:16+02	2020-10-03 04:48:15+02	graphics	clutter-gtk3	\N	clutter-gtk3-1.8.4
2020-10-03 04:47:53+02	2020-10-03 04:48:16+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-10-03 04:48:17+02	2020-10-03 04:49:29+02	graphics	glfw	\N	glfw-3.3.2
2020-10-03 04:48:15+02	2020-10-03 04:49:41+02	graphics	gegl	\N	gegl-0.4.26_5
2020-10-03 04:43:05+02	2020-10-03 04:50:01+02	devel	shiboken2	py37	py37-shiboken2-5.15.1
2020-10-03 04:50:02+02	2020-10-03 04:50:45+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-10-03 04:50:46+02	2020-10-03 04:51:11+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-10-03 04:49:30+02	2020-10-03 04:51:58+02	graphics	movit	\N	movit-1.6.2_2
2020-10-03 04:51:11+02	2020-10-03 04:52:00+02	math	fcl05	\N	fcl05-0.5.0
2020-10-03 04:51:58+02	2020-10-03 04:52:38+02	multimedia	clutter-gst3	\N	clutter-gst3-3.0.27
2020-10-03 04:52:01+02	2020-10-03 04:53:02+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-03 04:49:41+02	2020-10-03 04:53:09+02	graphics	opencv	\N	opencv-3.4.1_36
2020-10-03 04:52:39+02	2020-10-03 04:53:17+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-03 04:53:02+02	2020-10-03 04:53:36+02	print	cups-filters	\N	cups-filters-1.28.3
2020-10-03 04:53:36+02	2020-10-03 04:54:39+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0
2020-10-03 04:53:09+02	2020-10-03 04:55:24+02	print	fontforge	\N	fontforge-20190801_2
2020-10-03 04:54:40+02	2020-10-03 04:55:57+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-03 04:55:25+02	2020-10-03 04:56:27+02	sysutils	zeitgeist	\N	zeitgeist-1.0.2
2020-10-03 04:56:28+02	2020-10-03 04:57:20+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-10-03 04:57:21+02	2020-10-03 04:58:19+02	x11	yelp	\N	yelp-3.38.0
2020-10-03 04:58:20+02	2020-10-03 04:58:55+02	x11-themes	gnome-icon-theme-symbolic	\N	gnome-icon-theme-symbolic-3.12.0
2020-10-03 04:53:18+02	2020-10-03 04:59:06+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-10-03 04:58:55+02	2020-10-03 04:59:42+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-10-03 04:59:07+02	2020-10-03 04:59:47+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-10-03 04:59:43+02	2020-10-03 05:01:12+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-10-03 04:59:47+02	2020-10-03 05:01:21+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-10-03 05:01:22+02	2020-10-03 05:02:11+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0_1
2020-10-03 05:01:12+02	2020-10-03 05:02:24+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-10-03 05:02:24+02	2020-10-03 05:03:55+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-10-03 05:02:11+02	2020-10-03 05:05:03+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-10-03 05:03:55+02	2020-10-03 05:05:27+02	x11-wm	marco	\N	marco-1.22.4
2020-10-03 05:05:28+02	2020-10-03 05:06:03+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-10-03 05:05:04+02	2020-10-03 05:06:04+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-10-03 05:06:05+02	2020-10-03 05:07:08+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-10-03 05:06:03+02	2020-10-03 05:07:56+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-03 05:07:09+02	2020-10-03 05:09:35+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-03 05:09:36+02	2020-10-03 05:13:39+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-10-03 05:13:40+02	2020-10-03 05:15:56+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-10-03 05:15:56+02	2020-10-03 05:16:24+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-10-03 05:16:25+02	2020-10-03 05:18:36+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-10-03 05:18:36+02	2020-10-03 05:27:02+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-10-03 05:27:03+02	2020-10-03 05:28:31+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-10-03 05:28:34+02	2020-10-03 05:29:15+02	graphics	libgxps	\N	libgxps-0.3.1
2020-10-03 04:55:58+02	2020-10-03 05:29:47+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-03 05:29:18+02	2020-10-03 05:29:54+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-10-03 05:29:54+02	2020-10-03 05:30:50+02	graphics	py-PyOpenGL	py37	py37-PyOpenGL-3.1.5
2020-10-03 05:29:52+02	2020-10-03 05:31:06+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-10-03 05:30:51+02	2020-10-03 05:31:42+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-10-03 05:31:07+02	2020-10-03 05:41:42+02	graphics	py-scikit-image	py37	py37-scikit-image-0.17.2
2020-10-03 05:41:43+02	2020-10-03 05:42:48+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-10-03 05:31:43+02	2020-10-03 05:49:07+02	math	R	\N	R-4.0.2
2020-10-03 05:49:08+02	2020-10-03 05:50:22+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-03 05:42:49+02	2020-10-03 05:50:30+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-03 05:50:30+02	2020-10-03 05:51:17+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-03 05:50:22+02	2020-10-03 05:52:34+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-03 05:51:17+02	2020-10-03 05:52:41+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-03 05:52:35+02	2020-10-03 05:53:28+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-03 05:53:29+02	2020-10-03 05:53:50+02	print	libspectre	\N	libspectre-0.2.9
2020-10-03 05:53:50+02	2020-10-03 05:55:57+02	print	mftrace	\N	mftrace-1.2.18_2
2020-10-03 05:52:41+02	2020-10-03 05:56:35+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-03 05:56:35+02	2020-10-03 05:56:53+02	sysutils	hal-info	\N	hal-info-20091130
2020-10-03 05:56:53+02	2020-10-03 05:59:57+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-03 05:59:58+02	2020-10-03 06:00:52+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-03 05:55:58+02	2020-10-03 06:02:13+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-10-03 06:00:53+02	2020-10-03 06:02:45+02	textproc	dblatex	\N	dblatex-0.3.11
2020-10-03 06:02:14+02	2020-10-03 06:02:53+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-10-03 06:02:54+02	2020-10-03 06:03:21+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-10-03 06:02:46+02	2020-10-03 06:03:53+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-10-03 06:03:54+02	2020-10-03 06:05:13+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-10-03 06:03:21+02	2020-10-03 06:05:38+02	x11-servers	xephyr	\N	xephyr-1.20.9,1
2020-10-03 06:05:13+02	2020-10-03 06:06:06+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-10-03 06:05:39+02	2020-10-03 06:06:36+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.13.1_1
2020-10-03 06:06:07+02	2020-10-03 06:07:14+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-10-03 06:06:37+02	2020-10-03 06:08:01+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-10-03 06:08:02+02	2020-10-03 06:10:40+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-03 06:10:40+02	2020-10-03 06:11:05+02	audio	faudio	\N	FAudio-20.09
2020-10-03 06:11:05+02	2020-10-03 06:12:25+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-03 06:12:26+02	2020-10-03 06:13:43+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-03 06:07:14+02	2020-10-03 06:14:25+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-03 06:13:44+02	2020-10-03 06:15:06+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-10-03 06:14:25+02	2020-10-03 06:15:40+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-03 06:15:41+02	2020-10-03 06:16:37+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-03 06:16:38+02	2020-10-03 06:17:41+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-03 06:17:41+02	2020-10-03 06:18:02+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-10-03 06:18:02+02	2020-10-03 06:18:21+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-10-03 06:15:07+02	2020-10-03 06:18:24+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-03 06:18:21+02	2020-10-03 06:19:20+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-10-03 06:18:24+02	2020-10-03 06:20:08+02	devel	ode	\N	ode-0.13_4,1
2020-10-03 06:19:21+02	2020-10-03 06:20:38+02	devel	py-game	py37	py37-game-1.9.6_1
2020-10-03 06:20:08+02	2020-10-03 06:21:15+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-10-03 06:20:38+02	2020-10-03 06:21:55+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-03 06:22:58+02	2020-10-03 06:25:51+02	editors	pluma	\N	pluma-1.24.1
2020-10-03 06:25:52+02	2020-10-03 06:28:03+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-10-03 06:28:03+02	2020-10-03 06:30:39+02	graphics	atril	\N	atril-1.22.3_9
2020-10-03 06:30:40+02	2020-10-03 06:32:33+02	graphics	eom	\N	eom-1.24.1
2020-10-03 06:32:34+02	2020-10-03 06:34:09+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-10-03 06:34:11+02	2020-10-03 06:34:37+02	graphics	gle	\N	gle-3.1.0_7
2020-10-03 06:34:38+02	2020-10-03 06:34:57+02	graphics	gltt	\N	gltt-2.5.2_14
2020-10-03 06:34:58+02	2020-10-03 06:35:59+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-10-03 06:36:00+02	2020-10-03 06:40:13+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-10-03 06:40:14+02	2020-10-03 06:43:06+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-10-03 06:43:07+02	2020-10-03 06:43:47+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-10-03 06:21:16+02	2020-10-03 06:44:32+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-10-03 06:44:32+02	2020-10-03 06:52:18+02	lang	erlang-wx	\N	erlang-wx-21.3.8.17_1,1
2020-10-03 06:43:47+02	2020-10-03 06:52:34+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-10-03 06:52:19+02	2020-10-03 06:53:39+02	math	mate-calc	\N	mate-calc-1.24.1
2020-10-03 06:53:40+02	2020-10-03 06:54:51+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-10-03 06:54:52+02	2020-10-03 06:55:40+02	multimedia	libva-intel-hybrid-driver	\N	libva-intel-hybrid-driver-1.0.2_2
2020-10-03 06:55:40+02	2020-10-03 06:57:28+02	multimedia	mpv	\N	mpv-0.32.0_5,1
2020-10-03 06:21:56+02	2020-10-03 06:59:42+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-10-03 06:59:44+02	2020-10-03 07:01:19+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-03 07:01:20+02	2020-10-03 07:02:15+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-03 07:02:16+02	2020-10-03 07:03:06+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-03 06:57:29+02	2020-10-03 07:03:07+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-10-03 07:03:06+02	2020-10-03 07:03:46+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-03 07:03:08+02	2020-10-03 07:03:51+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-03 07:03:46+02	2020-10-03 07:04:24+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-03 07:03:52+02	2020-10-03 07:04:30+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-03 07:04:25+02	2020-10-03 07:05:12+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-03 06:52:35+02	2020-10-03 07:05:13+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-10-03 07:04:30+02	2020-10-03 07:05:33+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-03 07:05:13+02	2020-10-03 07:05:53+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-03 07:05:13+02	2020-10-03 07:06:11+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-03 07:05:33+02	2020-10-03 07:06:24+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-03 07:06:25+02	2020-10-03 07:07:48+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-10-03 07:05:54+02	2020-10-03 07:10:28+02	print	lilypond	\N	lilypond-2.20.0
2020-10-03 07:06:12+02	2020-10-03 07:11:11+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-10-03 07:11:11+02	2020-10-03 07:11:51+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-03 07:11:51+02	2020-10-03 07:13:08+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-03 07:07:49+02	2020-10-03 07:13:26+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-10-03 07:13:27+02	2020-10-03 07:14:18+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-10-03 07:13:09+02	2020-10-03 07:14:31+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-10-03 07:14:19+02	2020-10-03 07:15:02+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-10-03 07:14:32+02	2020-10-03 07:15:07+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-10-03 07:15:02+02	2020-10-03 07:15:50+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-10-03 07:15:07+02	2020-10-03 07:16:04+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-10-03 07:16:05+02	2020-10-03 07:17:03+02	x11	mate-base	\N	mate-base-1.24.0
2020-10-03 07:17:03+02	2020-10-03 07:18:09+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-10-03 07:18:09+02	2020-10-03 07:19:41+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-10-03 07:15:51+02	2020-10-03 07:20:08+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-10-03 07:20:09+02	2020-10-03 07:21:23+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-10-03 07:19:41+02	2020-10-03 07:21:26+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-03 07:21:27+02	2020-10-03 07:21:57+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-10-03 07:21:23+02	2020-10-03 07:22:08+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-03 07:21:58+02	2020-10-03 07:22:56+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-10-03 07:22:57+02	2020-10-03 07:27:04+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-03 07:27:05+02	2020-10-03 07:29:28+02	cad	cura	\N	Cura-4.7.1,2
2020-10-03 07:10:29+02	2020-10-03 07:29:33+02	print	texlive-docs	\N	texlive-docs-20150523
2020-10-03 07:29:33+02	2020-10-03 07:31:21+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-03 07:22:08+02	2020-10-03 07:32:34+02	astro	stellarium	\N	stellarium-0.20.3
2020-10-03 07:29:28+02	2020-10-03 07:33:32+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-03 07:32:34+02	2020-10-03 07:34:00+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-03 07:33:32+02	2020-10-03 07:34:53+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-03 07:34:53+02	2020-10-03 07:37:14+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-10-03 07:34:01+02	2020-10-03 07:37:16+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-03 07:37:14+02	2020-10-03 07:37:56+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-10-03 07:37:56+02	2020-10-03 07:48:26+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-03 07:48:26+02	2020-10-03 07:49:35+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-03 07:37:16+02	2020-10-03 08:01:20+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-03 08:01:23+02	2020-10-03 08:02:20+02	devel	tortoisehg	py37	tortoisehg-5.5.1_1
2020-10-03 08:02:21+02	2020-10-03 08:11:40+02	editors	codelite	\N	codelite-12.0_4
2020-10-03 08:11:41+02	2020-10-03 08:13:24+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-10-03 08:13:24+02	2020-10-03 08:14:54+02	editors	vim	\N	vim-8.2.1558
2020-10-03 07:49:35+02	2020-10-03 08:15:02+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-03 08:14:54+02	2020-10-03 08:22:58+02	emulators	qemu	\N	qemu-4.2.1
2020-10-03 08:22:59+02	2020-10-03 08:30:39+02	emulators	wine	\N	wine-5.0.2_1,1
2020-10-03 08:30:39+02	2020-10-03 08:45:45+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-03 08:45:47+02	2020-10-03 08:46:07+02	games	KnightCap	\N	knightcap-3.6_8
2020-10-03 08:46:08+02	2020-10-03 08:46:28+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-10-03 08:46:28+02	2020-10-03 08:48:57+02	games	alienarena	\N	alienarena-2013.766_9
2020-10-03 08:48:58+02	2020-10-03 08:50:44+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-10-03 08:50:45+02	2020-10-03 08:51:49+02	games	battletanks	\N	btanks-0.9.8083_9
2020-10-03 08:51:50+02	2020-10-03 08:52:12+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-10-03 08:52:13+02	2020-10-03 08:52:49+02	games	monsterz	\N	monsterz-0.7.1_15
2020-10-03 08:52:49+02	2020-10-03 08:53:24+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-10-03 08:53:25+02	2020-10-03 08:54:26+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-10-03 08:54:26+02	2020-10-03 08:55:35+02	games	xblast	\N	xblast-2.10.4_18
2020-10-03 08:55:36+02	2020-10-03 08:56:53+02	games	xboard	\N	xboard-4.9.1_1
2020-10-03 08:56:54+02	2020-10-03 08:57:44+02	games	xcowsay	\N	xcowsay-1.5.1
2020-10-03 08:57:45+02	2020-10-03 09:02:52+02	graphics	blender	\N	blender-2.90.0_1
2020-10-03 09:02:53+02	2020-10-03 09:05:11+02	graphics	dia	\N	dia-0.97.3_1,1
2020-10-03 09:05:12+02	2020-10-03 09:06:21+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-03 07:31:22+02	2020-10-03 09:09:25+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-03 09:06:21+02	2020-10-03 09:09:48+02	graphics	inkscape	\N	inkscape-1.0.1
2020-10-03 08:15:06+02	2020-10-03 09:12:34+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_3
2020-10-03 09:09:48+02	2020-10-03 09:14:14+02	graphics	wings	\N	wings-2.2.5
2020-10-03 09:12:35+02	2020-10-03 09:14:14+02	graphics	xpdf	\N	xpdf-4.02,1
2020-10-03 09:14:14+02	2020-10-03 09:15:34+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-03 09:09:26+02	2020-10-03 09:17:47+02	graphics	krita	\N	krita-4.3.0_5
2020-10-03 09:14:15+02	2020-10-03 09:46:45+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.11
2020-10-03 09:46:45+02	2020-10-03 09:52:40+02	math	maxima	\N	maxima-5.44.0_5
2020-10-03 09:15:34+02	2020-10-03 09:52:43+02	lang	erlang-runtime23	\N	erlang-runtime23-23.1.1
2020-10-03 09:52:44+02	2020-10-03 09:53:01+02	multimedia	libva-glx	\N	libva-glx-2.9.0
2020-10-03 09:52:41+02	2020-10-03 09:53:43+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-03 09:53:01+02	2020-10-03 09:53:58+02	multimedia	libva-intel-driver	\N	libva-intel-driver-2.4.1
2020-10-03 09:53:58+02	2020-10-03 09:54:46+02	multimedia	libva-utils	\N	libva-utils-2.9.1
2020-10-03 09:54:46+02	2020-10-03 09:55:03+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-10-03 09:55:04+02	2020-10-03 09:56:52+02	multimedia	obs-studio	\N	obs-studio-26.0.0_1
2020-10-03 09:53:44+02	2020-10-03 09:57:07+02	multimedia	libva-intel-media-driver	\N	libva-intel-media-driver-20.3.0
2020-10-03 09:56:52+02	2020-10-03 09:59:31+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-10-03 09:57:08+02	2020-10-03 09:59:57+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-03 09:59:58+02	2020-10-03 10:01:45+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-03 09:59:31+02	2020-10-03 10:05:37+02	net	ntopng	\N	ntopng-4.0.d20200917,1
2020-10-03 10:05:38+02	2020-10-03 10:06:25+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-03 10:06:26+02	2020-10-03 10:07:19+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-03 10:01:46+02	2020-10-03 10:07:25+02	net	wireshark	\N	wireshark-3.2.7
2020-10-03 10:07:19+02	2020-10-03 10:08:07+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-03 10:07:25+02	2020-10-03 10:08:15+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-03 10:08:07+02	2020-10-03 10:08:55+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-03 10:08:15+02	2020-10-03 10:09:03+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-03 10:08:55+02	2020-10-03 10:09:44+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-03 10:09:04+02	2020-10-03 10:09:53+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-03 10:09:54+02	2020-10-03 10:10:50+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-03 10:09:44+02	2020-10-03 10:11:06+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-03 10:10:51+02	2020-10-03 10:11:45+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-03 10:11:07+02	2020-10-03 10:11:50+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-10-03 10:11:46+02	2020-10-03 10:12:39+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-10-03 10:11:51+02	2020-10-03 10:14:32+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-03 10:12:39+02	2020-10-03 10:15:12+02	print	lyx	\N	lyx-2.3.4.2_1
2020-10-03 10:15:13+02	2020-10-03 10:20:22+02	print	texlive-full	\N	texlive-full-20150521
2020-10-03 10:14:33+02	2020-10-03 10:20:47+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-10-03 10:20:23+02	2020-10-03 10:21:10+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-10-03 10:20:59+02	2020-10-03 10:21:15+02	sysutils	flashrom	\N	flashrom-1.2
2020-10-03 10:20:48+02	2020-10-03 10:21:34+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-03 10:21:11+02	2020-10-03 10:21:41+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-10-03 10:21:16+02	2020-10-03 10:22:16+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-03 10:21:34+02	2020-10-03 10:22:18+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-03 10:21:41+02	2020-10-03 10:22:22+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-10-03 10:22:17+02	2020-10-03 10:22:31+02	sysutils	x86info	\N	x86info-1.31.s03
2020-10-03 10:22:24+02	2020-10-03 10:23:03+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-10-03 10:22:31+02	2020-10-03 10:23:52+02	x11	mate	\N	mate-1.22.2
2020-10-03 10:23:03+02	2020-10-03 10:24:03+02	x11	xlockmore	\N	xlockmore-5.65
2020-10-03 10:23:53+02	2020-10-03 10:25:20+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-10-03 10:25:20+02	2020-10-03 10:26:02+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-10-03 10:26:03+02	2020-10-03 10:26:15+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-10-03 10:26:16+02	2020-10-03 10:27:05+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-10-03 10:24:04+02	2020-10-03 10:28:47+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-03 10:27:05+02	2020-10-03 10:53:03+02	editors	vscode	\N	vscode-1.46.1
2020-10-03 10:22:18+02	2020-10-03 10:56:27+02	www	firefox	\N	firefox-81.0.1,2
2020-10-03 00:59:29+02	2020-10-03 01:06:07+02	ports-mgmt	pkg	\N	pkg-1.15.8
2020-10-03 11:11:52+02	2020-10-03 11:28:07+02	local	E590T-localbase	\N	E590T-localbase-2020100300
2020-10-03 11:28:19+02	2020-10-03 11:29:03+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-03 11:28:19+02	2020-10-03 11:34:58+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201002
2020-10-03 11:34:58+02	2020-10-03 11:35:06+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-03 14:37:22+02	2020-10-03 14:40:15+02	devel	sdl20	\N	sdl2-2.0.12_3
2020-10-03 14:37:22+02	2020-10-03 14:41:04+02	mail	mailutils	\N	mailutils-3.10_1
2020-10-03 14:40:15+02	2020-10-03 14:41:33+02	graphics	gegl	\N	gegl-0.4.26_5
2020-10-03 14:41:33+02	2020-10-03 14:42:02+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-10-03 14:41:05+02	2020-10-03 14:44:40+02	editors	emacs	full	emacs-27.1_1,3
2020-10-03 14:44:41+02	2020-10-03 14:45:54+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-03 14:45:55+02	2020-10-03 14:46:15+02	audio	faudio	\N	FAudio-20.09
2020-10-03 14:42:02+02	2020-10-03 14:47:52+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-10-03 14:47:52+02	2020-10-03 14:49:11+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-03 14:49:12+02	2020-10-03 14:50:36+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-03 14:50:37+02	2020-10-03 15:00:51+02	emulators	qemu	\N	qemu-4.2.1
2020-10-03 14:37:22+02	2020-10-03 15:01:05+02	lang	erlang	\N	erlang-21.3.8.18,4
2020-10-03 15:01:06+02	2020-10-03 15:01:18+02	devel	rebar	\N	rebar-2.6.4,1
2020-10-03 15:01:19+02	2020-10-03 15:02:06+02	devel	rebar3	\N	rebar3-3.14.1,1
2020-10-03 15:02:06+02	2020-10-03 15:02:20+02	devel	erlang-cl	\N	erlang-cl-1.2.4
2020-10-03 15:02:20+02	2020-10-03 15:02:39+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-10-03 14:56:55+02	2020-10-03 15:03:20+02	emulators	wine	\N	wine-5.0.2_1,1
2020-10-03 15:03:21+02	2020-10-03 15:04:13+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-03 15:00:52+02	2020-10-03 15:04:33+02	graphics	blender	\N	blender-2.90.0_1
2020-10-03 15:04:13+02	2020-10-03 15:05:08+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-03 15:04:33+02	2020-10-03 15:05:25+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-03 15:05:09+02	2020-10-03 15:07:27+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-03 15:05:25+02	2020-10-03 15:07:36+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-03 15:02:40+02	2020-10-03 15:08:57+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-03 15:08:57+02	2020-10-03 15:10:52+02	graphics	wings	\N	wings-2.2.5
2020-10-03 15:10:52+02	2020-10-03 15:28:20+02	local	E590T-localbase	\N	E590T-localbase-2020100300
2020-10-03 15:40:27+02	2020-10-03 15:41:07+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-03 15:40:27+02	2020-10-03 15:47:11+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-03 15:47:12+02	2020-10-03 15:47:19+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-03 18:01:48+02	2020-10-03 18:02:47+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-03 18:01:48+02	2020-10-03 18:03:59+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-03 18:04:00+02	2020-10-03 18:04:07+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-03 20:54:06+02	2020-10-03 20:54:15+02	www	py-bottle	py37	py37-bottle-0.12.18
2020-10-03 20:54:06+02	2020-10-03 20:54:26+02	devel	libIDL	\N	libIDL-0.8.14_5
2020-10-03 20:54:06+02	2020-10-03 20:54:52+02	devel	dbus	\N	dbus-1.12.20_3
2020-10-03 20:54:52+02	2020-10-03 20:55:18+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-10-03 20:54:26+02	2020-10-03 20:55:49+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-10-03 20:55:33+02	2020-10-03 20:56:01+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-10-03 20:55:19+02	2020-10-03 20:56:04+02	net	avahi-app	\N	avahi-app-0.7_3
2020-10-03 20:56:02+02	2020-10-03 20:56:35+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-10-03 20:55:50+02	2020-10-03 20:56:47+02	sysutils	polkit	\N	polkit-0.118
2020-10-03 20:56:04+02	2020-10-03 20:57:03+02	print	cups	\N	cups-2.3.3_1
2020-10-03 20:56:48+02	2020-10-03 20:57:11+02	net	glib-networking	\N	glib-networking-2.66.0
2020-10-03 20:56:35+02	2020-10-03 20:58:50+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-10-03 20:57:04+02	2020-10-03 21:00:24+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-10-03 20:57:12+02	2020-10-03 21:01:05+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-10-03 20:58:50+02	2020-10-03 21:01:35+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0_1
2020-10-03 21:01:06+02	2020-10-03 21:04:10+02	devel	libsoup	\N	libsoup-2.62.3
2020-10-03 21:04:11+02	2020-10-03 21:05:52+02	devel	dconf	\N	dconf-0.38.0
2020-10-03 21:01:36+02	2020-10-03 21:06:07+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-10-03 21:05:56+02	2020-10-03 21:09:22+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-10-03 21:09:23+02	2020-10-03 21:11:17+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-10-03 21:06:08+02	2020-10-03 21:12:54+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-10-03 21:11:20+02	2020-10-03 21:14:36+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-10-03 21:12:55+02	2020-10-03 21:15:16+02	net	geoclue	\N	geoclue-2.5.5
2020-10-03 21:14:38+02	2020-10-03 21:15:49+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-10-03 21:15:17+02	2020-10-03 21:16:42+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-10-03 21:15:50+02	2020-10-03 21:18:27+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-10-03 21:16:43+02	2020-10-03 21:19:35+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-10-03 21:18:29+02	2020-10-03 21:20:25+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-10-03 21:19:36+02	2020-10-03 21:21:16+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-10-03 21:00:24+02	2020-10-03 21:23:44+02	print	texlive-base	\N	texlive-base-20150521_55
2020-10-03 21:21:17+02	2020-10-03 21:24:17+02	sysutils	policykit	\N	policykit-0.9_10
2020-10-03 21:24:18+02	2020-10-03 21:27:48+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-10-03 21:27:50+02	2020-10-03 21:30:09+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-10-03 21:30:10+02	2020-10-03 21:32:26+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-10-03 21:32:28+02	2020-10-03 21:35:53+02	sysutils	hal	\N	hal-0.5.14_34
2020-10-03 21:20:26+02	2020-10-03 21:38:01+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-10-03 21:35:56+02	2020-10-03 21:38:41+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-10-03 21:38:04+02	2020-10-03 21:40:16+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-10-03 21:38:43+02	2020-10-03 21:42:31+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-10-03 21:40:20+02	2020-10-03 21:42:44+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-10-03 21:42:48+02	2020-10-03 21:45:38+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-10-03 21:45:41+02	2020-10-03 21:51:18+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-10-03 21:51:20+02	2020-10-03 21:55:22+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-03 21:55:25+02	2020-10-03 21:59:16+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-10-03 21:23:46+02	2020-10-03 22:00:44+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-10-03 21:59:18+02	2020-10-03 22:01:46+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-10-03 22:00:49+02	2020-10-03 22:07:28+02	print	tex-formats	\N	tex-formats-20150521_2
2020-10-03 21:42:36+02	2020-10-03 22:09:55+02	audio	pulseaudio	\N	pulseaudio-13.0_1
2020-10-03 22:09:57+02	2020-10-03 22:17:16+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-10-03 22:01:49+02	2020-10-03 22:18:48+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-10-03 22:17:19+02	2020-10-03 22:20:40+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-10-03 22:07:31+02	2020-10-03 22:22:07+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-10-03 22:20:45+02	2020-10-03 22:25:16+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-10-03 22:18:51+02	2020-10-03 22:27:25+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-10-03 22:25:16+02	2020-10-03 22:28:01+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-10-03 22:27:30+02	2020-10-03 22:29:46+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-10-03 22:28:03+02	2020-10-03 22:31:22+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-10-03 22:29:51+02	2020-10-03 22:32:50+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-10-03 22:31:28+02	2020-10-03 22:33:49+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-10-03 22:32:53+02	2020-10-03 22:35:36+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-10-03 22:22:12+02	2020-10-03 22:36:45+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-10-03 22:33:50+02	2020-10-03 22:36:48+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-10-03 22:35:38+02	2020-10-03 22:38:42+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-10-03 22:36:52+02	2020-10-03 22:40:35+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-10-03 22:36:52+02	2020-10-03 22:43:22+02	textproc	docbook-utils	\N	docbook-utils-0.6.14_13
2020-10-03 22:38:45+02	2020-10-03 22:44:45+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-10-03 22:40:38+02	2020-10-03 22:46:05+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-10-03 22:44:48+02	2020-10-03 22:47:34+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-10-03 22:47:38+02	2020-10-03 22:50:24+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-10-03 22:46:07+02	2020-10-03 22:51:30+02	sysutils	upower	\N	upower-0.99.11
2020-10-03 22:50:27+02	2020-10-03 22:55:10+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-10-03 22:51:34+02	2020-10-03 22:55:45+02	devel	librest	\N	rest-0.7.93
2020-10-03 22:55:15+02	2020-10-03 22:59:54+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-10-03 22:55:47+02	2020-10-03 23:00:09+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-03 22:43:27+02	2020-10-03 23:02:32+02	graphics	colord	\N	colord-1.3.5_1
2020-10-03 23:00:12+02	2020-10-03 23:02:42+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-10-03 22:59:56+02	2020-10-03 23:03:16+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-10-03 23:03:18+02	2020-10-03 23:08:11+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-10-03 23:08:14+02	2020-10-03 23:10:01+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-10-03 23:10:01+02	2020-10-03 23:12:29+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-03 23:12:32+02	2020-10-03 23:16:28+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-10-03 23:16:28+02	2020-10-03 23:18:41+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-10-03 23:18:43+02	2020-10-03 23:21:55+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-10-03 23:02:44+02	2020-10-03 23:35:24+02	math	vtk8	\N	vtk8-8.2.0
2020-10-03 23:02:34+02	2020-10-03 23:45:59+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-10-03 23:22:00+02	2020-10-04 00:07:08+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-10-03 23:35:27+02	2020-10-04 00:08:15+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-04 00:07:11+02	2020-10-04 00:10:47+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-10-04 00:08:16+02	2020-10-04 00:13:19+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-10-04 00:13:21+02	2020-10-04 00:17:09+02	x11-toolkits	vte3	\N	vte3-0.62.0
2020-10-04 00:10:49+02	2020-10-04 00:21:26+02	security	gcr	\N	gcr-3.28.0_1
2020-10-04 00:17:10+02	2020-10-04 00:21:29+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-10-03 23:46:04+02	2020-10-04 00:23:28+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-04 00:21:27+02	2020-10-04 00:23:29+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-10-04 00:21:30+02	2020-10-04 00:23:53+02	net	remmina	\N	remmina-1.3.9
2020-10-04 00:23:55+02	2020-10-04 00:27:05+02	devel	libpeas	\N	libpeas-1.22.0
2020-10-04 00:23:31+02	2020-10-04 00:29:50+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-04 00:23:31+02	2020-10-04 00:37:36+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-04 00:29:53+02	2020-10-04 00:37:52+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-10-04 00:27:08+02	2020-10-04 00:38:09+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-04 00:38:10+02	2020-10-04 00:40:25+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-10-04 00:37:37+02	2020-10-04 00:42:00+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-04 00:40:26+02	2020-10-04 00:42:17+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-10-04 00:42:18+02	2020-10-04 00:44:09+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-10-04 00:42:01+02	2020-10-04 00:44:33+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-04 00:42:55+02	2020-10-04 00:45:04+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-10-04 00:44:10+02	2020-10-04 00:46:05+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-10-04 00:44:36+02	2020-10-04 00:47:59+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-10-04 00:46:06+02	2020-10-04 00:48:24+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-10-04 00:48:25+02	2020-10-04 00:49:29+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-10-04 00:49:29+02	2020-10-04 00:53:22+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-10-04 00:48:04+02	2020-10-04 00:56:01+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-10-04 00:56:02+02	2020-10-04 01:02:14+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-10-04 00:53:25+02	2020-10-04 01:04:08+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-04 01:02:17+02	2020-10-04 01:04:17+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-10-04 01:04:18+02	2020-10-04 01:04:49+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-10-04 01:04:50+02	2020-10-04 01:06:01+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-10-04 01:06:02+02	2020-10-04 01:09:26+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-10-04 01:09:27+02	2020-10-04 01:10:13+02	multimedia	totem-pl-parser	\N	totem-pl-parser-3.26.5
2020-10-04 01:10:13+02	2020-10-04 01:11:23+02	net	geocode-glib	\N	geocode-glib-3.26.2
2020-10-04 01:11:25+02	2020-10-04 01:12:32+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-04 01:12:32+02	2020-10-04 01:13:40+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-10-04 01:13:40+02	2020-10-04 01:14:55+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-10-04 01:14:55+02	2020-10-04 01:16:05+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-10-04 01:16:06+02	2020-10-04 01:17:19+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-10-04 00:45:05+02	2020-10-04 01:18:54+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-04 01:17:20+02	2020-10-04 01:19:11+02	x11-fm	caja	\N	caja-1.24.0
2020-10-04 01:18:56+02	2020-10-04 01:20:18+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-10-04 01:19:12+02	2020-10-04 01:20:22+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-10-04 01:04:08+02	2020-10-04 01:20:37+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-04 01:20:18+02	2020-10-04 01:20:58+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-04 01:20:23+02	2020-10-04 01:21:17+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-10-04 01:20:58+02	2020-10-04 01:21:51+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-10-04 01:20:38+02	2020-10-04 01:21:51+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-10-04 01:21:18+02	2020-10-04 01:22:07+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-10-04 01:22:08+02	2020-10-04 01:22:29+02	net	grilo	\N	grilo-0.3.13
2020-10-04 01:21:52+02	2020-10-04 01:22:37+02	math	octomap	\N	octomap-1.9.5.190
2020-10-04 01:21:52+02	2020-10-04 01:22:39+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-10-04 01:22:40+02	2020-10-04 01:23:12+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-10-04 01:22:29+02	2020-10-04 01:23:31+02	net	libgweather	\N	libgweather-3.36.1
2020-10-04 01:22:38+02	2020-10-04 01:23:46+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-04 01:23:32+02	2020-10-04 01:24:20+02	sysutils	tracker	\N	tracker-2.3.4
2020-10-04 01:23:12+02	2020-10-04 01:24:20+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-10-04 01:23:46+02	2020-10-04 01:24:47+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-10-04 01:24:20+02	2020-10-04 01:25:07+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-10-04 01:24:20+02	2020-10-04 01:25:12+02	x11	zenity	\N	zenity-3.32.0
2020-10-04 01:24:48+02	2020-10-04 01:25:31+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-10-04 01:25:07+02	2020-10-04 01:25:57+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-10-04 01:25:32+02	2020-10-04 01:26:02+02	devel	gstreamer1-plugins-soup	\N	gstreamer1-plugins-soup-1.16.2
2020-10-04 01:25:13+02	2020-10-04 01:26:25+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-10-04 01:26:03+02	2020-10-04 01:26:40+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-10-04 01:26:25+02	2020-10-04 01:27:06+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-04 01:27:07+02	2020-10-04 01:27:56+02	graphics	clutter-gtk3	\N	clutter-gtk3-1.8.4
2020-10-04 01:25:58+02	2020-10-04 01:28:55+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-10-04 01:27:56+02	2020-10-04 01:28:56+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-10-04 01:28:55+02	2020-10-04 01:29:36+02	math	fcl05	\N	fcl05-0.5.0
2020-10-04 01:28:56+02	2020-10-04 01:29:53+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-04 01:29:37+02	2020-10-04 01:29:55+02	net	gssdp	\N	gssdp-1.2.3
2020-10-04 01:26:40+02	2020-10-04 01:29:59+02	editors	emacs	full	emacs-27.1_1,3
2020-10-04 01:29:56+02	2020-10-04 01:30:25+02	print	cups-filters	\N	cups-filters-1.28.3
2020-10-04 01:29:54+02	2020-10-04 01:30:33+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-04 01:30:33+02	2020-10-04 01:31:37+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0
2020-10-04 01:30:00+02	2020-10-04 01:32:08+02	print	fontforge	\N	fontforge-20190801_2
2020-10-04 01:31:38+02	2020-10-04 01:32:56+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-04 01:32:10+02	2020-10-04 01:33:05+02	sysutils	zeitgeist	\N	zeitgeist-1.0.2
2020-10-04 01:33:05+02	2020-10-04 01:33:55+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-10-04 01:33:55+02	2020-10-04 01:34:47+02	x11	yelp	\N	yelp-3.38.0
2020-10-04 01:34:48+02	2020-10-04 01:35:29+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-10-04 01:30:25+02	2020-10-04 01:35:57+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-10-04 01:35:29+02	2020-10-04 01:36:09+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-10-04 01:35:57+02	2020-10-04 01:37:11+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-10-04 01:37:11+02	2020-10-04 01:38:44+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-10-04 01:36:09+02	2020-10-04 01:38:50+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-10-04 01:38:50+02	2020-10-04 01:39:44+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-10-04 01:38:44+02	2020-10-04 01:40:14+02	x11-wm	marco	\N	marco-1.22.4
2020-10-04 01:40:15+02	2020-10-04 01:41:31+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-10-04 01:39:44+02	2020-10-04 01:41:50+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-04 01:41:51+02	2020-10-04 01:42:41+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-10-04 01:42:41+02	2020-10-04 01:43:37+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-10-04 01:41:31+02	2020-10-04 01:43:51+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-04 01:43:37+02	2020-10-04 01:44:35+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-10-04 01:43:52+02	2020-10-04 01:45:36+02	devel	shiboken2	py37	py37-shiboken2-5.15.1
2020-10-04 01:44:36+02	2020-10-04 01:48:08+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-10-04 01:48:09+02	2020-10-04 01:49:37+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-10-04 01:49:37+02	2020-10-04 01:50:40+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-10-04 01:45:37+02	2020-10-04 01:53:11+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-10-04 01:53:11+02	2020-10-04 01:54:21+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-10-04 01:54:21+02	2020-10-04 02:02:47+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-04 02:02:48+02	2020-10-04 02:03:47+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-04 01:32:57+02	2020-10-04 02:04:27+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-04 02:04:32+02	2020-10-04 02:05:40+02	net	freerdp	\N	freerdp-2.2.0_1
2020-10-04 02:03:48+02	2020-10-04 02:05:43+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-04 02:05:40+02	2020-10-04 02:06:04+02	net	gupnp	\N	gupnp-1.2.4
2020-10-04 02:05:44+02	2020-10-04 02:06:23+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-04 01:50:41+02	2020-10-04 02:06:46+02	math	R	\N	R-4.0.2
2020-10-04 02:06:23+02	2020-10-04 02:07:02+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-04 02:06:04+02	2020-10-04 02:07:03+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-04 02:07:02+02	2020-10-04 02:07:20+02	print	libspectre	\N	libspectre-0.2.9
2020-10-04 02:07:03+02	2020-10-04 02:08:50+02	print	mftrace	\N	mftrace-1.2.18_2
2020-10-04 02:08:50+02	2020-10-04 02:09:05+02	sysutils	hal-info	\N	hal-info-20091130
2020-10-04 02:06:47+02	2020-10-04 02:09:31+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-04 02:09:32+02	2020-10-04 02:10:15+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-04 02:09:05+02	2020-10-04 02:11:12+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-04 02:10:15+02	2020-10-04 02:11:45+02	textproc	dblatex	\N	dblatex-0.3.11
2020-10-04 02:11:13+02	2020-10-04 02:11:46+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-10-04 02:07:20+02	2020-10-04 02:11:59+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-10-04 02:11:46+02	2020-10-04 02:12:34+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-10-04 02:11:45+02	2020-10-04 02:12:34+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-10-04 02:11:59+02	2020-10-04 02:12:36+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-10-04 02:12:34+02	2020-10-04 02:13:20+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.13.1_1
2020-10-04 02:12:36+02	2020-10-04 02:13:40+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-10-04 02:12:34+02	2020-10-04 02:13:47+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-10-04 02:13:20+02	2020-10-04 02:14:07+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0_1
2020-10-04 02:13:41+02	2020-10-04 02:14:30+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-10-04 02:14:31+02	2020-10-04 02:15:39+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-04 02:14:08+02	2020-10-04 02:16:05+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-04 02:15:40+02	2020-10-04 02:16:51+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-04 02:16:06+02	2020-10-04 02:17:18+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-04 02:13:47+02	2020-10-04 02:17:46+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-04 02:17:18+02	2020-10-04 02:18:04+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-04 02:17:47+02	2020-10-04 02:18:38+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-04 02:18:05+02	2020-10-04 02:18:51+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-10-04 02:16:52+02	2020-10-04 02:19:14+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-04 02:18:38+02	2020-10-04 02:19:26+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-10-04 02:18:51+02	2020-10-04 02:19:48+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-04 02:25:44+02	2020-10-04 02:30:14+02	editors	pluma	\N	pluma-1.24.1
2020-10-04 02:30:15+02	2020-10-04 02:32:53+02	graphics	atril	\N	atril-1.22.3_9
2020-10-04 02:32:53+02	2020-10-04 02:35:03+02	graphics	eom	\N	eom-1.24.1
2020-10-04 02:35:03+02	2020-10-04 02:35:44+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-10-04 02:19:26+02	2020-10-04 02:40:00+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-10-04 02:19:49+02	2020-10-04 02:42:03+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-10-04 02:35:45+02	2020-10-04 02:42:12+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-10-04 02:42:12+02	2020-10-04 02:43:26+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-10-04 02:42:04+02	2020-10-04 02:43:33+02	math	mate-calc	\N	mate-calc-1.24.1
2020-10-04 02:43:33+02	2020-10-04 02:44:26+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-04 02:44:26+02	2020-10-04 02:45:14+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-04 02:45:14+02	2020-10-04 02:46:15+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-04 02:40:00+02	2020-10-04 02:46:21+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-04 02:46:15+02	2020-10-04 02:46:59+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-04 02:46:22+02	2020-10-04 02:47:09+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-04 02:43:27+02	2020-10-04 02:47:35+02	multimedia	vlc	\N	vlc-3.0.11_4,4
2020-10-04 02:47:00+02	2020-10-04 02:47:38+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-04 02:47:09+02	2020-10-04 02:47:46+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-04 02:47:36+02	2020-10-04 02:48:22+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-04 02:47:47+02	2020-10-04 02:48:26+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-04 02:47:39+02	2020-10-04 02:48:42+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-04 02:48:27+02	2020-10-04 02:49:19+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-04 02:48:22+02	2020-10-04 02:49:22+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-04 02:49:23+02	2020-10-04 02:50:48+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-10-04 02:48:42+02	2020-10-04 02:53:16+02	print	lilypond	\N	lilypond-2.20.0
2020-10-04 02:49:20+02	2020-10-04 02:54:18+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-10-04 02:54:18+02	2020-10-04 02:54:41+02	print	transfig	\N	transfig-3.2.5e_4
2020-10-04 02:54:42+02	2020-10-04 02:55:21+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-04 02:50:49+02	2020-10-04 02:56:24+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-10-04 02:55:21+02	2020-10-04 02:56:36+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-04 02:56:37+02	2020-10-04 02:57:26+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-10-04 02:56:24+02	2020-10-04 02:57:46+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-10-04 02:57:27+02	2020-10-04 02:58:11+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-10-04 02:57:46+02	2020-10-04 02:58:19+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-10-04 02:58:12+02	2020-10-04 02:59:01+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-10-04 02:58:20+02	2020-10-04 02:59:16+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-10-04 02:59:17+02	2020-10-04 03:00:17+02	x11	mate-base	\N	mate-base-1.24.0
2020-10-04 03:00:17+02	2020-10-04 03:01:27+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-10-04 03:01:27+02	2020-10-04 03:03:28+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-10-04 02:59:01+02	2020-10-04 03:03:59+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-10-04 03:03:59+02	2020-10-04 03:04:55+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-04 03:03:29+02	2020-10-04 03:05:17+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-04 03:04:56+02	2020-10-04 03:05:34+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-10-04 03:05:17+02	2020-10-04 03:06:27+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-10-04 03:05:34+02	2020-10-04 03:06:32+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-10-04 03:06:34+02	2020-10-04 03:11:16+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-04 02:53:17+02	2020-10-04 03:13:52+02	print	texlive-docs	\N	texlive-docs-20150523
2020-10-04 03:11:19+02	2020-10-04 03:14:35+02	cad	cura	\N	Cura-4.7.1,2
2020-10-04 03:14:36+02	2020-10-04 03:17:19+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-04 03:06:27+02	2020-10-04 03:20:12+02	astro	stellarium	\N	stellarium-0.20.3
2020-10-04 03:13:54+02	2020-10-04 03:20:12+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-04 03:20:13+02	2020-10-04 03:21:53+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-04 03:20:13+02	2020-10-04 03:22:06+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-04 03:22:06+02	2020-10-04 03:24:55+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-10-04 03:24:56+02	2020-10-04 03:25:54+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-10-04 03:21:54+02	2020-10-04 03:49:33+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-04 03:25:55+02	2020-10-04 03:53:25+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-04 03:53:26+02	2020-10-04 03:54:36+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-04 03:49:33+02	2020-10-04 03:56:19+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-04 03:56:20+02	2020-10-04 03:56:56+02	devel	tortoisehg	py37	tortoisehg-5.5.1_1
2020-10-04 03:56:56+02	2020-10-04 04:05:25+02	editors	codelite	\N	codelite-12.0_4
2020-10-04 04:05:25+02	2020-10-04 04:07:53+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-10-04 04:07:54+02	2020-10-04 04:09:56+02	editors	vim	\N	vim-8.2.1558
2020-10-04 04:09:58+02	2020-10-04 04:18:40+02	emulators	qemu	\N	qemu-4.2.1
2020-10-04 03:54:36+02	2020-10-04 04:18:49+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-04 04:18:53+02	2020-10-04 04:35:22+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-04 04:35:23+02	2020-10-04 04:36:13+02	games	xcowsay	\N	xcowsay-1.5.1
2020-10-04 04:36:14+02	2020-10-04 04:38:38+02	graphics	dia	\N	dia-0.97.3_1,1
2020-10-04 04:38:39+02	2020-10-04 04:39:56+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-04 04:39:56+02	2020-10-04 04:44:17+02	graphics	inkscape	\N	inkscape-1.0.1
2020-10-04 04:44:18+02	2020-10-04 04:56:18+02	graphics	krita	\N	krita-4.3.0_5
2020-10-04 04:56:19+02	2020-10-04 04:59:29+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-10-04 04:59:30+02	2020-10-04 05:03:30+02	graphics	wings	\N	wings-2.2.5
2020-10-04 05:03:30+02	2020-10-04 05:04:39+02	graphics	xpdf	\N	xpdf-4.02,1
2020-10-04 05:04:39+02	2020-10-04 05:05:48+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-04 03:17:20+02	2020-10-04 05:06:51+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-04 04:18:43+02	2020-10-04 05:20:44+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_3
2020-10-04 05:20:45+02	2020-10-04 05:25:17+02	math	maxima	\N	maxima-5.44.0_5
2020-10-04 05:25:18+02	2020-10-04 05:26:04+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-04 05:05:50+02	2020-10-04 05:27:01+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.11
2020-10-04 05:26:05+02	2020-10-04 05:27:26+02	multimedia	obs-studio	\N	obs-studio-26.0.0_1
2020-10-04 05:27:04+02	2020-10-04 05:29:15+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-10-04 05:27:26+02	2020-10-04 05:29:39+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-04 05:29:16+02	2020-10-04 05:30:27+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-04 05:30:27+02	2020-10-04 05:31:17+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-04 05:31:18+02	2020-10-04 05:32:24+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-04 05:32:24+02	2020-10-04 05:33:03+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-04 05:29:39+02	2020-10-04 05:33:17+02	net	wireshark	\N	wireshark-3.2.7
2020-10-04 05:06:52+02	2020-10-04 05:33:20+02	lang	erlang-runtime23	\N	erlang-runtime23-23.1.1
2020-10-04 05:33:03+02	2020-10-04 05:33:41+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-04 05:33:17+02	2020-10-04 05:33:54+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-04 05:33:21+02	2020-10-04 05:33:58+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-04 05:33:42+02	2020-10-04 05:34:18+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-04 05:33:55+02	2020-10-04 05:34:31+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-04 05:33:58+02	2020-10-04 05:34:55+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-04 05:34:19+02	2020-10-04 05:35:00+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-04 05:34:31+02	2020-10-04 05:35:09+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-04 05:35:09+02	2020-10-04 05:35:30+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-10-04 05:34:55+02	2020-10-04 05:35:31+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-10-04 05:35:30+02	2020-10-04 05:35:56+02	print	gv	\N	gv-3.7.4_5
2020-10-04 05:35:00+02	2020-10-04 05:36:53+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-04 05:35:31+02	2020-10-04 05:37:24+02	print	lyx	\N	lyx-2.3.4.2_1
2020-10-04 05:37:24+02	2020-10-04 05:38:17+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-10-04 05:38:17+02	2020-10-04 05:39:24+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-04 05:39:24+02	2020-10-04 05:39:56+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-10-04 05:35:57+02	2020-10-04 05:40:31+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-10-04 05:36:53+02	2020-10-04 05:40:36+02	print	texlive-full	\N	texlive-full-20150521
2020-10-04 05:39:56+02	2020-10-04 05:41:00+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-04 05:40:32+02	2020-10-04 05:41:16+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-04 05:40:36+02	2020-10-04 05:41:17+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-10-04 05:41:17+02	2020-10-04 05:41:55+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-10-04 05:41:18+02	2020-10-04 05:42:27+02	x11	mate	\N	mate-1.22.2
2020-10-04 05:41:55+02	2020-10-04 05:43:18+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-10-04 05:43:18+02	2020-10-04 05:44:01+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-10-04 05:42:27+02	2020-10-04 05:46:59+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-04 05:47:00+02	2020-10-04 05:47:49+02	graphics	xfig	\N	xfig-3.2.7a
2020-10-04 05:44:01+02	2020-10-04 06:09:01+02	editors	vscode	\N	vscode-1.46.1
2020-10-04 05:41:01+02	2020-10-04 06:13:31+02	www	firefox	\N	firefox-81.0.1,2
2020-10-04 06:39:38+02	2020-10-04 06:39:54+02	databases	php74-pdo_pgsql	\N	php74-pdo_pgsql-7.4.10
2020-10-04 06:39:54+02	2020-10-04 06:40:12+02	databases	php74-pgsql	\N	php74-pgsql-7.4.10
2020-10-04 06:40:12+02	2020-10-04 06:40:55+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-04 06:40:55+02	2020-10-04 06:41:05+02	databases	fpc-postgres	\N	fpc-postgres-3.0.4_6
2020-10-04 06:41:05+02	2020-10-04 06:41:37+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-04 06:41:38+02	2020-10-04 06:42:25+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-04 06:42:26+02	2020-10-04 06:42:58+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-04 06:42:59+02	2020-10-04 06:43:14+02	databases	p5-DBD-Pg	\N	p5-DBD-Pg-3.14.2
2020-10-04 06:43:15+02	2020-10-04 06:43:24+02	databases	pgtcl	\N	pgtcl-2.1.1_2
2020-10-04 06:43:24+02	2020-10-04 06:43:58+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-04 06:43:58+02	2020-10-04 06:44:46+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-04 06:44:46+02	2020-10-04 06:45:21+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-10-04 06:45:22+02	2020-10-04 06:45:30+02	databases	pgaccess	\N	pgaccess-1.00.20140902_3
2020-10-04 06:45:31+02	2020-10-04 06:45:41+02	databases	pgtop	\N	pgtop-0.05_5
2020-10-04 06:45:42+02	2020-10-04 06:46:34+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-10-04 06:46:34+02	2020-10-04 06:47:07+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-04 06:47:08+02	2020-10-04 06:47:39+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-04 06:47:40+02	2020-10-04 06:48:12+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-04 06:48:12+02	2020-10-04 06:48:44+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-04 06:48:44+02	2020-10-04 06:49:16+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-04 06:49:17+02	2020-10-04 06:49:49+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-04 06:49:49+02	2020-10-04 06:50:20+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-04 06:50:21+02	2020-10-04 06:51:07+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-04 06:51:08+02	2020-10-04 06:51:43+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-04 06:51:44+02	2020-10-04 06:52:16+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-04 06:52:16+02	2020-10-04 07:07:07+02	local	E590T-localbase	\N	E590T-localbase-2020100300
2020-10-04 07:07:20+02	2020-10-04 07:08:31+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-04 07:07:20+02	2020-10-04 07:13:22+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-04 07:13:23+02	2020-10-04 07:13:30+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-04 09:51:29+02	2020-10-04 09:51:35+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20201003
2020-10-04 09:51:36+02	2020-10-04 09:53:41+02	sysutils	intel-pcm	\N	intel-pcm-202009
2020-10-04 09:53:41+02	2020-10-04 09:53:56+02	net-mgmt	nstat	\N	nstat-1.0_3
2020-10-04 09:53:57+02	2020-10-04 09:54:22+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-10-04 09:51:29+02	2020-10-04 09:55:29+02	multimedia	vlc	\N	vlc-3.0.11_5,4
2020-10-04 09:55:30+02	2020-10-04 09:57:05+02	multimedia	obs-studio	\N	obs-studio-26.0.0_1
2020-10-04 09:57:06+02	2020-10-04 10:28:29+02	local	E590T-localbase	\N	E590T-localbase-2020100400
2020-10-04 10:41:28+02	2020-10-04 10:42:28+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-04 10:41:28+02	2020-10-04 10:43:40+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-04 10:43:41+02	2020-10-04 10:43:48+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-04 21:14:10+02	2020-10-04 21:16:02+02	ports-mgmt	pkg	\N	pkg-1.15.9
2020-10-04 21:55:29+02	2020-10-04 21:56:10+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.07
2020-10-04 21:56:12+02	2020-10-04 21:57:04+02	www	p5-libwww	\N	p5-libwww-6.49
2020-10-04 21:57:06+02	2020-10-04 21:57:19+02	security	base-audit	\N	base-audit-0.4
2020-10-04 21:55:29+02	2020-10-04 22:00:20+02	devel	hwloc	\N	hwloc-1.11.13
2020-10-04 21:55:32+02	2020-10-04 22:06:35+02	devel	glib20	\N	glib-2.66.1,1
2020-10-04 22:06:40+02	2020-10-04 22:10:05+02	security	p11-kit	\N	p11-kit-0.23.21
2020-10-04 22:06:38+02	2020-10-04 22:15:32+02	graphics	cairo	\N	cairo-1.16.0_1,3
2020-10-04 22:15:33+02	2020-10-04 22:18:43+02	devel	gobject-introspection	\N	gobject-introspection-1.66.1,1
2020-10-04 22:00:20+02	2020-10-04 22:19:16+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-10-04 22:19:16+02	2020-10-04 22:19:53+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-10-04 22:10:07+02	2020-10-04 22:23:12+02	security	gnutls	\N	gnutls-3.6.15
2020-10-04 22:23:14+02	2020-10-04 22:24:19+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-10-04 22:19:56+02	2020-10-04 22:24:42+02	net	avahi-app	\N	avahi-app-0.7_3
2020-10-04 22:18:44+02	2020-10-04 22:26:25+02	print	harfbuzz	\N	harfbuzz-2.7.2_1
2020-10-04 22:24:20+02	2020-10-04 22:26:37+02	misc	shared-mime-info	\N	shared-mime-info-2.0
2020-10-04 22:26:41+02	2020-10-04 22:29:13+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-10-04 22:29:14+02	2020-10-04 22:30:45+02	graphics	poppler	\N	poppler-20.09.0
2020-10-04 22:26:28+02	2020-10-04 22:31:35+02	x11-toolkits	pango	\N	pango-1.42.4_5
2020-10-04 22:31:38+02	2020-10-04 22:32:56+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.2
2020-10-04 22:32:58+02	2020-10-04 22:34:00+02	accessibility	atk	\N	atk-2.36.0
2020-10-04 22:29:26+02	2020-10-04 22:34:28+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_9
2020-10-04 22:30:49+02	2020-10-04 22:35:03+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-10-04 22:34:30+02	2020-10-04 22:35:28+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-10-04 22:35:04+02	2020-10-04 22:39:53+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-10-04 22:34:02+02	2020-10-04 22:41:35+02	lang	vala	\N	vala-0.48.10,1
2020-10-04 22:39:56+02	2020-10-04 22:42:58+02	sysutils	polkit	\N	polkit-0.118
2020-10-04 22:41:36+02	2020-10-04 22:44:33+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-10-04 22:42:59+02	2020-10-04 22:45:10+02	textproc	libcroco	\N	libcroco-0.6.13
2020-10-04 22:45:11+02	2020-10-04 22:48:22+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-10-04 22:48:23+02	2020-10-04 22:49:54+02	devel	libgudev	\N	libgudev-234
2020-10-04 22:49:55+02	2020-10-04 22:50:42+02	x11	libwacom	\N	libwacom-1.5
2020-10-04 22:50:43+02	2020-10-04 22:51:54+02	x11	libinput	\N	libinput-1.16.1
2020-10-04 22:51:55+02	2020-10-04 22:53:22+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-10-04 22:44:36+02	2020-10-04 23:03:47+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-10-04 22:53:24+02	2020-10-04 23:05:46+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.38.0
2020-10-04 23:03:51+02	2020-10-04 23:06:17+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-10-04 22:35:31+02	2020-10-04 23:08:07+02	print	texlive-base	\N	texlive-base-20150521_55
2020-10-04 23:06:19+02	2020-10-04 23:08:28+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-10-04 23:08:28+02	2020-10-04 23:09:00+02	devel	desktop-file-utils	\N	desktop-file-utils-0.26
2020-10-04 23:09:01+02	2020-10-04 23:10:41+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-10-04 23:05:47+02	2020-10-04 23:12:03+02	net	qt5-network	\N	qt5-network-5.15.0
2020-10-04 23:12:03+02	2020-10-04 23:15:24+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0_1
2020-10-04 23:10:41+02	2020-10-04 23:16:19+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-10-04 23:15:24+02	2020-10-04 23:19:19+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-10-04 23:16:24+02	2020-10-04 23:26:40+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-10-04 23:19:20+02	2020-10-04 23:27:33+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-10-04 23:27:37+02	2020-10-04 23:28:24+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.38.0
2020-10-04 23:28:26+02	2020-10-04 23:29:28+02	net	glib-networking	\N	glib-networking-2.66.0
2020-10-04 23:29:30+02	2020-10-04 23:33:54+02	devel	libsoup	\N	libsoup-2.62.3
2020-10-04 23:33:55+02	2020-10-04 23:35:20+02	devel	json-glib	\N	json-glib-1.6.0
2020-10-04 23:35:22+02	2020-10-04 23:36:37+02	devel	dconf	\N	dconf-0.38.0
2020-10-04 23:26:42+02	2020-10-04 23:38:08+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2_1
2020-10-04 23:36:39+02	2020-10-04 23:39:05+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-10-04 23:39:09+02	2020-10-04 23:41:04+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-10-04 23:41:05+02	2020-10-04 23:42:24+02	devel	libnotify	\N	libnotify-0.7.9_1
2020-10-04 23:08:09+02	2020-10-04 23:42:51+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-10-04 23:42:26+02	2020-10-04 23:44:57+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-10-04 23:38:10+02	2020-10-04 23:48:01+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-10-04 23:42:54+02	2020-10-04 23:49:48+02	print	tex-formats	\N	tex-formats-20150521_2
2020-10-04 23:48:01+02	2020-10-04 23:51:30+02	graphics	graphviz	\N	graphviz-2.44.1_2
2020-10-04 23:51:30+02	2020-10-04 23:52:26+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-10-04 23:52:26+02	2020-10-04 23:53:05+02	security	libsecret	\N	libsecret-0.20.3
2020-10-04 23:45:00+02	2020-10-04 23:53:12+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-10-04 23:53:06+02	2020-10-04 23:53:18+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1
2020-10-04 23:53:13+02	2020-10-04 23:53:29+02	graphics	frei0r	\N	frei0r-1.6.1
2020-10-04 23:53:19+02	2020-10-04 23:53:35+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-10-04 23:53:35+02	2020-10-04 23:53:54+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-10-04 23:53:54+02	2020-10-04 23:54:12+02	devel	libIDL	\N	libIDL-0.8.14_5
2020-10-04 23:49:49+02	2020-10-04 23:54:39+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-10-04 23:54:13+02	2020-10-04 23:57:38+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-10-04 23:53:29+02	2020-10-04 23:58:03+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-10-04 23:57:38+02	2020-10-04 23:58:12+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-10-04 23:58:13+02	2020-10-04 23:59:12+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-10-04 23:58:04+02	2020-10-04 23:59:50+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-10-04 23:59:12+02	2020-10-05 00:00:11+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-10-04 23:54:40+02	2020-10-05 00:00:11+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-10-05 00:00:12+02	2020-10-05 00:00:54+02	net	geoclue	\N	geoclue-2.5.5
2020-10-04 23:59:50+02	2020-10-05 00:00:58+02	graphics	graphene	\N	graphene-1.10.2
2020-10-05 00:00:55+02	2020-10-05 00:01:10+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-10-05 00:00:12+02	2020-10-05 00:01:40+02	textproc	docbook-utils	\N	docbook-utils-0.6.14_13
2020-10-05 00:01:10+02	2020-10-05 00:01:40+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-10-05 00:00:59+02	2020-10-05 00:01:53+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2_1
2020-10-05 00:01:41+02	2020-10-05 00:02:17+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-10-05 00:01:53+02	2020-10-05 00:02:43+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.74.0
2020-10-05 00:02:17+02	2020-10-05 00:02:50+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-10-05 00:02:43+02	2020-10-05 00:03:08+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-10-05 00:03:08+02	2020-10-05 00:03:31+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-10-05 00:01:40+02	2020-10-05 00:03:41+02	graphics	colord	\N	colord-1.3.5_1
2020-10-05 00:02:50+02	2020-10-05 00:03:43+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-10-05 00:03:31+02	2020-10-05 00:03:48+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-10-05 00:03:43+02	2020-10-05 00:03:59+02	print	libraqm	\N	libraqm-0.6.0
2020-10-05 00:03:49+02	2020-10-05 00:04:26+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-10-05 00:04:26+02	2020-10-05 00:05:06+02	sysutils	policykit	\N	policykit-0.9_10
2020-10-05 00:05:07+02	2020-10-05 00:06:04+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-10-05 00:06:05+02	2020-10-05 00:07:22+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.4
2020-10-05 00:03:59+02	2020-10-05 00:08:02+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-10-05 00:08:03+02	2020-10-05 00:09:08+02	devel	py-qt5-core	py37	py37-qt5-core-5.13.1_1
2020-10-05 00:09:09+02	2020-10-05 00:09:52+02	sysutils	hal	\N	hal-0.5.14_34
2020-10-05 00:03:42+02	2020-10-05 00:10:06+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-10-05 00:09:53+02	2020-10-05 00:10:32+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8_1
2020-10-05 00:10:32+02	2020-10-05 00:11:31+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-10-05 00:07:22+02	2020-10-05 00:11:41+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-10-05 00:11:42+02	2020-10-05 00:13:07+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-10-05 00:11:31+02	2020-10-05 00:13:34+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.74.0
2020-10-05 00:13:09+02	2020-10-05 00:14:02+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-10-05 00:13:35+02	2020-10-05 00:14:28+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-10-05 00:14:03+02	2020-10-05 00:14:49+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-10-05 00:14:50+02	2020-10-05 00:16:15+02	x11	mate-desktop	\N	mate-desktop-1.24.0
2020-10-05 00:14:28+02	2020-10-05 00:16:42+02	security	gnupg	\N	gnupg-2.2.23
2020-10-05 00:16:43+02	2020-10-05 00:17:15+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-10-05 00:16:16+02	2020-10-05 00:17:30+02	audio	pulseaudio	\N	pulseaudio-13.0_1
2020-10-05 00:17:16+02	2020-10-05 00:18:05+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-10-05 00:17:30+02	2020-10-05 00:19:15+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-10-05 00:19:16+02	2020-10-05 00:20:48+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-05 00:18:05+02	2020-10-05 00:20:57+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-10-05 00:20:49+02	2020-10-05 00:21:24+02	security	gpgme	\N	gpgme-1.14.0
2020-10-05 00:20:57+02	2020-10-05 00:21:59+02	archivers	kf5-karchive	\N	kf5-karchive-5.74.0
2020-10-05 00:22:00+02	2020-10-05 00:22:42+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-10-05 00:21:24+02	2020-10-05 00:22:42+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-10-05 00:22:43+02	2020-10-05 00:23:33+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-10-05 00:22:42+02	2020-10-05 00:24:51+02	security	gcr	\N	gcr-3.28.0_1
2020-10-05 00:24:52+02	2020-10-05 00:26:20+02	x11-toolkits	vte3	\N	vte3-0.62.0
2020-10-05 00:23:33+02	2020-10-05 00:26:57+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.13.1_1
2020-10-05 00:26:21+02	2020-10-05 00:27:04+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-10-05 00:26:57+02	2020-10-05 00:28:21+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-10-05 00:27:05+02	2020-10-05 00:28:56+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-10-05 00:28:22+02	2020-10-05 00:29:18+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-10-05 00:10:06+02	2020-10-05 00:29:31+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-05 00:28:56+02	2020-10-05 00:29:33+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-10-05 00:29:18+02	2020-10-05 00:29:35+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-10-05 00:29:33+02	2020-10-05 00:29:56+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-10-05 00:29:35+02	2020-10-05 00:30:01+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-10-05 00:30:02+02	2020-10-05 00:30:27+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-10-05 00:30:27+02	2020-10-05 00:31:09+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-10-05 00:31:09+02	2020-10-05 00:32:08+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-10-05 00:32:10+02	2020-10-05 00:32:46+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-10-05 00:32:46+02	2020-10-05 00:33:57+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-10-05 00:29:56+02	2020-10-05 00:34:05+02	devel	glibmm	\N	glibmm-2.64.2,1
2020-10-05 00:33:58+02	2020-10-05 00:34:33+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-10-05 00:34:05+02	2020-10-05 00:34:49+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-10-05 00:29:32+02	2020-10-05 00:34:59+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-05 00:34:49+02	2020-10-05 00:35:29+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-10-05 00:34:33+02	2020-10-05 00:35:32+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-10-05 00:34:59+02	2020-10-05 00:35:41+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-10-05 00:35:29+02	2020-10-05 00:35:55+02	graphics	poppler-glib	\N	poppler-glib-20.09.0
2020-10-05 00:35:32+02	2020-10-05 00:36:03+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-10-05 00:35:41+02	2020-10-05 00:36:30+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-10-05 00:35:55+02	2020-10-05 00:36:39+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-10-05 00:36:04+02	2020-10-05 00:36:51+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.74.0
2020-10-05 00:36:30+02	2020-10-05 00:37:18+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-10-05 00:36:39+02	2020-10-05 00:37:25+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-10-05 00:36:51+02	2020-10-05 00:37:49+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-10-05 00:37:19+02	2020-10-05 00:38:09+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-10-05 00:37:52+02	2020-10-05 00:38:44+02	net	py-qt5-network	py37	py37-qt5-network-5.13.1_1
2020-10-05 00:37:26+02	2020-10-05 00:39:06+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-10-05 00:38:10+02	2020-10-05 00:39:08+02	net	remmina	\N	remmina-1.3.9
2020-10-05 00:38:44+02	2020-10-05 00:39:19+02	sysutils	upower	\N	upower-0.99.11
2020-10-05 00:39:10+02	2020-10-05 00:39:41+02	x11	mate-menus	\N	mate-menus-1.24.0
2020-10-05 00:39:20+02	2020-10-05 00:40:08+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-10-05 00:39:07+02	2020-10-05 00:40:10+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-10-05 00:40:11+02	2020-10-05 00:40:48+02	devel	librest	\N	rest-0.7.93
2020-10-05 00:40:09+02	2020-10-05 00:41:06+02	devel	libgtop	\N	libgtop-2.40.0
2020-10-05 00:39:41+02	2020-10-05 00:41:14+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.13.1_1
2020-10-05 00:40:49+02	2020-10-05 00:41:18+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-10-05 00:41:19+02	2020-10-05 00:42:14+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-05 00:41:07+02	2020-10-05 00:42:33+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-10-05 00:42:14+02	2020-10-05 00:43:04+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0_1
2020-10-05 00:43:04+02	2020-10-05 00:44:02+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-10-05 00:42:34+02	2020-10-05 00:44:06+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.74.0
2020-10-05 00:44:03+02	2020-10-05 00:44:48+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-10-05 00:44:07+02	2020-10-05 00:45:00+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-10-05 00:45:00+02	2020-10-05 00:45:24+02	devel	gamin	\N	gamin-0.1.10_10
2020-10-05 00:44:49+02	2020-10-05 00:45:26+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-10-05 00:45:27+02	2020-10-05 00:47:00+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-05 00:45:25+02	2020-10-05 00:47:11+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-05 00:47:11+02	2020-10-05 00:47:44+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-10-05 00:47:00+02	2020-10-05 00:47:57+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-10-05 00:47:45+02	2020-10-05 00:48:42+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-05 00:47:58+02	2020-10-05 00:49:14+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-10-05 00:49:15+02	2020-10-05 00:50:07+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-10-05 00:48:43+02	2020-10-05 00:50:56+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-10-05 00:41:14+02	2020-10-05 00:51:49+02	net	openmpi	\N	openmpi-4.0.5
2020-10-05 00:50:57+02	2020-10-05 00:52:01+02	devel	libpeas	\N	libpeas-1.22.0
2020-10-05 00:50:37+02	2020-10-05 00:52:07+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-10-05 00:52:02+02	2020-10-05 00:53:33+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-10-05 00:53:33+02	2020-10-05 00:54:20+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-10-05 00:52:08+02	2020-10-05 00:55:16+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-05 00:54:20+02	2020-10-05 00:55:17+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-10-05 00:58:40+02	2020-10-05 01:00:06+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-10-05 01:00:06+02	2020-10-05 01:01:17+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-10-05 01:01:17+02	2020-10-05 01:03:01+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-05 01:03:02+02	2020-10-05 01:04:08+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-10-05 01:04:09+02	2020-10-05 01:05:40+02	graphics	cogl	\N	cogl-1.22.8
2020-10-05 01:05:40+02	2020-10-05 01:06:41+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-10-05 01:06:42+02	2020-10-05 01:07:29+02	mail	gmime30	\N	gmime30-3.2.7
2020-10-05 00:55:17+02	2020-10-05 01:08:29+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-10-05 01:08:30+02	2020-10-05 01:09:11+02	multimedia	libquvi09	\N	libquvi09-0.9.4_4
2020-10-05 01:07:29+02	2020-10-05 01:09:42+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_13,3
2020-10-05 01:09:12+02	2020-10-05 01:10:21+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-10-05 01:10:21+02	2020-10-05 01:11:10+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.13.1_1
2020-10-05 01:11:11+02	2020-10-05 01:12:10+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-10-05 01:12:11+02	2020-10-05 01:13:18+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-10-05 01:05:39+02	2020-10-05 01:18:40+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-05 01:18:41+02	2020-10-05 01:20:01+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-10-05 01:20:02+02	2020-10-05 01:20:47+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-10-05 01:20:48+02	2020-10-05 01:24:51+02	x11	mate-panel	\N	mate-panel-1.24.0
2020-10-05 01:24:51+02	2020-10-05 01:26:03+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-10-05 01:26:04+02	2020-10-05 01:27:34+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-10-05 01:27:34+02	2020-10-05 01:28:16+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-10-05 01:28:16+02	2020-10-05 01:36:07+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-05 01:36:09+02	2020-10-05 01:37:00+02	audio	chromaprint	\N	chromaprint-1.5.0
2020-10-05 01:37:00+02	2020-10-05 01:37:28+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-10-05 01:09:43+02	2020-10-05 01:38:05+02	net	samba410	\N	samba410-4.10.18
2020-10-05 01:38:05+02	2020-10-05 01:39:04+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-10-05 01:39:04+02	2020-10-05 01:39:30+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-10-05 01:13:18+02	2020-10-05 01:40:13+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-05 01:39:31+02	2020-10-05 01:41:16+02	graphics	clutter	\N	clutter-1.26.2
2020-10-05 01:40:15+02	2020-10-05 01:41:32+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-10-05 01:41:17+02	2020-10-05 01:41:46+02	graphics	poppler-utils	\N	poppler-utils-20.09.0
2020-10-05 01:41:33+02	2020-10-05 01:42:04+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-10-05 01:42:04+02	2020-10-05 01:42:35+02	multimedia	totem-pl-parser	\N	totem-pl-parser-3.26.5
2020-10-05 01:42:35+02	2020-10-05 01:43:41+02	net	geocode-glib	\N	geocode-glib-3.26.2
2020-10-05 01:41:47+02	2020-10-05 01:44:19+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-10-05 01:44:20+02	2020-10-05 01:45:14+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-05 01:43:42+02	2020-10-05 01:45:46+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-10-05 01:45:15+02	2020-10-05 01:46:11+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-10-05 01:45:47+02	2020-10-05 01:46:17+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-10-05 01:46:11+02	2020-10-05 01:47:12+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-10-05 01:46:18+02	2020-10-05 01:48:38+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-10-05 01:47:13+02	2020-10-05 01:48:43+02	x11	libmatekbd	\N	libmatekbd-1.24.0
2020-10-05 01:48:44+02	2020-10-05 01:51:34+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-10-05 01:51:13+02	2020-10-05 01:52:14+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-10-05 01:51:35+02	2020-10-05 01:52:21+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-05 01:52:15+02	2020-10-05 01:52:53+02	audio	gstreamer1-plugins-chromaprint	\N	gstreamer1-plugins-chromaprint-1.16.2
2020-10-05 01:52:00+02	2020-10-05 01:53:03+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-10-05 01:53:04+02	2020-10-05 01:53:53+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-10-05 01:53:03+02	2020-10-05 01:54:03+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-10-05 01:52:53+02	2020-10-05 01:54:28+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-10-05 01:53:53+02	2020-10-05 01:54:30+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-10-05 01:54:30+02	2020-10-05 01:54:58+02	mail	gmime26	\N	gmime26-2.6.23_1
2020-10-05 01:54:28+02	2020-10-05 01:55:19+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.13.1_1
2020-10-05 01:55:20+02	2020-10-05 01:56:09+02	math	octomap	\N	octomap-1.9.5.190
2020-10-05 01:54:59+02	2020-10-05 01:56:25+02	mail	mailutils	\N	mailutils-3.10_1
2020-10-05 01:56:26+02	2020-10-05 01:56:55+02	multimedia	libmediaart	\N	libmediaart-1.9.4
2020-10-05 01:56:09+02	2020-10-05 01:57:00+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-10-05 01:54:04+02	2020-10-05 01:57:51+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-10-05 01:57:00+02	2020-10-05 01:58:06+02	net	libgweather	\N	libgweather-3.36.1
2020-10-05 01:57:51+02	2020-10-05 01:58:23+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-10-05 01:57:25+02	2020-10-05 01:58:44+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-05 01:58:24+02	2020-10-05 01:59:27+02	sysutils	tracker	\N	tracker-2.3.4
2020-10-05 01:58:06+02	2020-10-05 01:59:30+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-10-05 01:58:44+02	2020-10-05 01:59:31+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-10-05 01:59:31+02	2020-10-05 02:00:18+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-10-05 01:59:30+02	2020-10-05 02:00:39+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-10-05 01:59:58+02	2020-10-05 02:00:50+02	x11	zenity	\N	zenity-3.32.0
2020-10-05 02:00:19+02	2020-10-05 02:01:02+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-10-05 02:00:40+02	2020-10-05 02:01:32+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-10-05 02:01:02+02	2020-10-05 02:01:33+02	devel	gstreamer1-plugins-soup	\N	gstreamer1-plugins-soup-1.16.2
2020-10-05 02:00:51+02	2020-10-05 02:02:01+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-10-05 02:01:33+02	2020-10-05 02:02:10+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-10-05 02:02:02+02	2020-10-05 02:02:45+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-05 02:02:45+02	2020-10-05 02:03:42+02	graphics	clutter-gtk3	\N	clutter-gtk3-1.8.4
2020-10-05 02:03:42+02	2020-10-05 02:04:19+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-10-05 02:01:32+02	2020-10-05 02:04:34+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-10-05 02:04:35+02	2020-10-05 02:04:51+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-10-05 02:04:19+02	2020-10-05 02:05:40+02	graphics	gegl	\N	gegl-0.4.26_5
2020-10-05 02:02:11+02	2020-10-05 02:06:00+02	editors	emacs	full	emacs-27.1_1,3
2020-10-05 02:05:41+02	2020-10-05 02:06:18+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-10-05 02:06:18+02	2020-10-05 02:06:59+02	multimedia	clutter-gst3	\N	clutter-gst3-3.0.27
2020-10-05 02:06:42+02	2020-10-05 02:07:45+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-05 02:04:51+02	2020-10-05 02:07:55+02	graphics	opencv	\N	opencv-3.4.1_36
2020-10-05 02:07:17+02	2020-10-05 02:07:58+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-05 02:07:45+02	2020-10-05 02:08:24+02	print	cups-filters	\N	cups-filters-1.28.3
2020-10-05 02:08:24+02	2020-10-05 02:09:38+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0
2020-10-05 02:07:57+02	2020-10-05 02:10:45+02	print	fontforge	\N	fontforge-20190801_2
2020-10-05 02:09:39+02	2020-10-05 02:11:07+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-05 02:10:45+02	2020-10-05 02:11:45+02	sysutils	zeitgeist	\N	zeitgeist-1.0.2
2020-10-05 02:11:46+02	2020-10-05 02:12:36+02	x11	mate-session-manager	\N	mate-session-manager-1.24.0_1
2020-10-05 02:07:58+02	2020-10-05 02:14:03+02	print	tex-luatex	\N	tex-luatex-0.80.0_20
2020-10-05 02:13:39+02	2020-10-05 02:14:15+02	x11-themes	gnome-icon-theme-symbolic	\N	gnome-icon-theme-symbolic-3.12.0
2020-10-05 02:14:03+02	2020-10-05 02:14:55+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-10-05 02:14:15+02	2020-10-05 02:15:02+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-10-05 02:14:55+02	2020-10-05 02:16:39+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-10-05 02:15:02+02	2020-10-05 02:17:55+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-10-05 02:16:40+02	2020-10-05 02:18:06+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-10-05 02:18:07+02	2020-10-05 02:19:14+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-10-05 02:17:56+02	2020-10-05 02:19:53+02	x11-wm	marco	\N	marco-1.22.4
2020-10-05 02:19:54+02	2020-10-05 02:21:00+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-10-05 02:19:15+02	2020-10-05 02:21:18+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-05 02:21:18+02	2020-10-05 02:22:09+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-10-05 02:22:10+02	2020-10-05 02:23:17+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-10-05 02:21:01+02	2020-10-05 02:23:40+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-05 02:23:17+02	2020-10-05 02:24:18+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-10-05 02:23:41+02	2020-10-05 02:25:28+02	devel	shiboken2	py37	py37-shiboken2-5.15.1
2020-10-05 02:25:28+02	2020-10-05 02:26:26+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-10-05 02:24:19+02	2020-10-05 02:28:12+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-10-05 02:26:27+02	2020-10-05 02:28:22+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-10-05 02:28:22+02	2020-10-05 02:28:56+02	graphics	libgxps	\N	libgxps-0.3.1
2020-10-05 02:28:56+02	2020-10-05 02:31:10+02	graphics	libheif	\N	libheif-1.6.2_1
2020-10-05 02:31:11+02	2020-10-05 02:32:40+02	graphics	poppler-qt5	\N	poppler-qt5-20.09.0
2020-10-05 02:32:41+02	2020-10-05 02:33:53+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.13.1_1
2020-10-05 02:28:13+02	2020-10-05 02:39:46+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-10-05 02:33:54+02	2020-10-05 02:41:28+02	graphics	py-scikit-image	py37	py37-scikit-image-0.17.2
2020-10-05 02:41:30+02	2020-10-05 02:42:30+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-10-05 02:11:07+02	2020-10-05 02:43:43+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-05 02:43:51+02	2020-10-05 02:44:47+02	multimedia	libass	\N	libass-0.14.0
2020-10-05 02:44:47+02	2020-10-05 02:45:52+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-05 02:45:52+02	2020-10-05 02:48:32+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-05 02:48:32+02	2020-10-05 02:49:21+02	net	freerdp	\N	freerdp-2.2.0_1
2020-10-05 02:42:32+02	2020-10-05 02:49:24+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-05 02:49:24+02	2020-10-05 02:50:05+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-05 02:50:06+02	2020-10-05 02:50:46+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-05 02:49:45+02	2020-10-05 02:50:47+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-05 02:50:47+02	2020-10-05 02:51:05+02	print	libspectre	\N	libspectre-0.2.9
2020-10-05 02:51:05+02	2020-10-05 02:52:51+02	print	mftrace	\N	mftrace-1.2.18_2
2020-10-05 02:52:52+02	2020-10-05 02:53:08+02	sysutils	hal-info	\N	hal-info-20091130
2020-10-05 02:50:46+02	2020-10-05 02:53:32+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-05 02:53:33+02	2020-10-05 02:54:16+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-05 02:53:09+02	2020-10-05 02:55:29+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-05 02:54:17+02	2020-10-05 02:55:52+02	textproc	dblatex	\N	dblatex-0.3.11
2020-10-05 02:55:29+02	2020-10-05 02:56:03+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.13.1_1
2020-10-05 02:56:03+02	2020-10-05 02:56:23+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-10-05 02:55:52+02	2020-10-05 02:56:43+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.13.1_1
2020-10-05 02:56:23+02	2020-10-05 02:56:51+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.1
2020-10-05 02:52:50+02	2020-10-05 02:57:21+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-10-05 02:56:51+02	2020-10-05 02:57:29+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-10-05 02:56:43+02	2020-10-05 02:57:32+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-10-05 02:57:21+02	2020-10-05 02:58:02+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.13.1_1
2020-10-05 02:57:32+02	2020-10-05 02:58:33+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-10-05 02:57:29+02	2020-10-05 02:58:39+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-10-05 02:58:02+02	2020-10-05 02:58:51+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0_1
2020-10-05 02:58:33+02	2020-10-05 02:59:24+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-10-05 02:59:24+02	2020-10-05 03:00:37+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-10-05 02:58:52+02	2020-10-05 03:01:02+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-05 03:00:38+02	2020-10-05 03:01:07+02	audio	faudio	\N	FAudio-20.10
2020-10-05 03:01:03+02	2020-10-05 03:02:19+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-05 03:01:07+02	2020-10-05 03:02:31+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-05 03:02:20+02	2020-10-05 03:03:32+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-10-05 02:58:39+02	2020-10-05 03:03:42+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-05 03:02:31+02	2020-10-05 03:03:59+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-05 03:03:43+02	2020-10-05 03:04:33+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-05 03:04:00+02	2020-10-05 03:04:48+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-05 03:04:33+02	2020-10-05 03:05:25+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-10-05 03:04:48+02	2020-10-05 03:05:55+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.4
2020-10-05 03:03:32+02	2020-10-05 03:06:28+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-05 03:05:26+02	2020-10-05 03:06:47+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-05 03:14:56+02	2020-10-05 03:20:41+02	editors	pluma	\N	pluma-1.24.1
2020-10-05 03:20:42+02	2020-10-05 03:24:20+02	graphics	atril	\N	atril-1.22.3_9
2020-10-05 03:26:15+02	2020-10-05 03:28:30+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-10-05 03:28:31+02	2020-10-05 03:29:48+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-10-05 03:06:48+02	2020-10-05 03:36:54+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-10-05 03:06:29+02	2020-10-05 03:36:59+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-10-05 03:36:59+02	2020-10-05 03:46:06+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-05 03:36:56+02	2020-10-05 03:46:52+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-10-05 03:46:06+02	2020-10-05 03:48:04+02	math	mate-calc	\N	mate-calc-1.24.1
2020-10-05 03:46:53+02	2020-10-05 03:48:38+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-10-05 03:48:05+02	2020-10-05 03:49:59+02	multimedia	mpv	\N	mpv-0.32.0_5,1
2020-10-05 03:49:59+02	2020-10-05 03:51:10+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-05 03:29:48+02	2020-10-05 03:51:28+02	graphics	openimageio	\N	openimageio-2.2.6.1
2020-10-05 03:51:11+02	2020-10-05 03:51:54+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-05 03:51:28+02	2020-10-05 03:52:11+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-05 03:48:38+02	2020-10-05 03:52:17+02	multimedia	vlc	\N	vlc-3.0.11_5,4
2020-10-05 03:51:55+02	2020-10-05 03:52:33+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-05 03:52:11+02	2020-10-05 03:52:52+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-05 03:52:18+02	2020-10-05 03:52:55+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-05 03:52:33+02	2020-10-05 03:53:10+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-05 03:52:53+02	2020-10-05 03:53:40+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-05 03:53:11+02	2020-10-05 03:53:52+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-05 03:52:55+02	2020-10-05 03:53:59+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-05 03:53:41+02	2020-10-05 03:54:41+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-05 03:53:53+02	2020-10-05 03:54:48+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-05 03:54:48+02	2020-10-05 03:56:16+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-10-05 03:54:00+02	2020-10-05 03:58:41+02	print	lilypond	\N	lilypond-2.20.0
2020-10-05 03:54:41+02	2020-10-05 03:59:54+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-10-05 03:59:54+02	2020-10-05 04:00:14+02	print	transfig	\N	transfig-3.2.5e_4
2020-10-05 04:00:15+02	2020-10-05 04:00:55+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-05 03:56:16+02	2020-10-05 04:01:51+02	print	tex-xetex	\N	tex-xetex-0.99992_36
2020-10-05 04:00:55+02	2020-10-05 04:02:14+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-05 04:02:14+02	2020-10-05 04:03:07+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-10-05 04:01:52+02	2020-10-05 04:03:20+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-10-05 04:03:08+02	2020-10-05 04:03:52+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.13.1_1
2020-10-05 04:03:21+02	2020-10-05 04:03:56+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.13.1_1
2020-10-05 04:03:52+02	2020-10-05 04:04:41+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.12.1_1
2020-10-05 04:03:57+02	2020-10-05 04:04:52+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.13.1_1
2020-10-05 04:04:53+02	2020-10-05 04:05:51+02	x11	mate-base	\N	mate-base-1.24.0
2020-10-05 04:05:52+02	2020-10-05 04:07:04+02	x11	mate-screensaver	\N	mate-screensaver-1.24.0
2020-10-05 04:07:05+02	2020-10-05 04:08:41+02	x11	mate-terminal	\N	mate-terminal-1.24.0
2020-10-05 04:04:41+02	2020-10-05 04:09:08+02	x11	mate-applets	\N	mate-applets-1.24.0
2020-10-05 04:09:09+02	2020-10-05 04:10:06+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-05 04:08:41+02	2020-10-05 04:10:27+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-05 04:10:06+02	2020-10-05 04:10:40+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-10-05 04:10:41+02	2020-10-05 04:11:38+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-10-05 04:10:27+02	2020-10-05 04:12:10+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-10-05 04:12:10+02	2020-10-05 04:16:27+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-05 03:58:42+02	2020-10-05 04:17:49+02	print	texlive-docs	\N	texlive-docs-20150523
2020-10-05 04:18:55+02	2020-10-05 04:20:50+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-05 04:11:39+02	2020-10-05 04:22:29+02	astro	stellarium	\N	stellarium-0.20.3
2020-10-05 04:17:50+02	2020-10-05 04:22:33+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-05 04:22:33+02	2020-10-05 04:24:02+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-05 04:22:29+02	2020-10-05 04:24:09+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-05 04:24:10+02	2020-10-05 04:26:30+02	deskutils	calibre	\N	calibre-4.23.0_1
2020-10-05 04:26:30+02	2020-10-05 04:27:19+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-10-05 04:24:03+02	2020-10-05 04:27:22+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-05 04:27:22+02	2020-10-05 04:38:21+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-05 04:38:22+02	2020-10-05 04:39:40+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-05 04:27:20+02	2020-10-05 04:53:39+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-05 04:53:39+02	2020-10-05 04:54:35+02	devel	tortoisehg	py37	tortoisehg-5.5.1_1
2020-10-05 04:54:36+02	2020-10-05 05:04:59+02	editors	codelite	\N	codelite-12.0_4
2020-10-05 05:04:59+02	2020-10-05 05:06:56+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-10-05 04:39:41+02	2020-10-05 05:07:11+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-05 05:06:57+02	2020-10-05 05:08:37+02	editors	vim	\N	vim-8.2.1558
2020-10-05 05:07:16+02	2020-10-05 05:22:36+02	emulators	qemu	\N	qemu-4.2.1
2020-10-05 05:22:37+02	2020-10-05 05:31:44+02	emulators	wine	\N	wine-5.0.2_1,1
2020-10-05 05:31:45+02	2020-10-05 05:47:53+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-05 05:47:54+02	2020-10-05 05:48:27+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-10-05 05:48:28+02	2020-10-05 05:49:03+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-10-05 05:49:04+02	2020-10-05 05:49:56+02	games	xboard	\N	xboard-4.9.1_1
2020-10-05 05:49:57+02	2020-10-05 05:50:40+02	games	xcowsay	\N	xcowsay-1.5.1
2020-10-05 05:50:41+02	2020-10-05 05:55:33+02	graphics	blender	\N	blender-2.90.0_1
2020-10-05 05:55:34+02	2020-10-05 05:58:26+02	graphics	dia	\N	dia-0.97.3_1,1
2020-10-05 05:58:27+02	2020-10-05 05:59:39+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-05 04:20:51+02	2020-10-05 06:09:29+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-05 05:59:40+02	2020-10-05 06:10:28+02	graphics	inkscape	\N	inkscape-1.0.1
2020-10-05 05:08:38+02	2020-10-05 06:12:46+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_3
2020-10-05 06:10:28+02	2020-10-05 06:13:57+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-10-05 06:13:58+02	2020-10-05 06:15:34+02	graphics	xpdf	\N	xpdf-4.02,1
2020-10-05 06:12:47+02	2020-10-05 06:17:05+02	graphics	wings	\N	wings-2.2.5
2020-10-05 06:15:35+02	2020-10-05 06:17:12+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-10-05 06:17:06+02	2020-10-05 06:17:55+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-05 06:09:30+02	2020-10-05 06:18:16+02	graphics	krita	\N	krita-4.3.0_5
2020-10-05 06:18:17+02	2020-10-05 06:22:52+02	math	maxima	\N	maxima-5.44.0_5
2020-10-05 06:24:57+02	2020-10-05 06:25:52+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-05 06:25:52+02	2020-10-05 06:27:22+02	multimedia	obs-studio	\N	obs-studio-26.0.0_1
2020-10-05 06:27:22+02	2020-10-05 06:29:29+02	multimedia	openshot	py37	openshot-2.5.1.369
2020-10-05 06:29:29+02	2020-10-05 06:31:53+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-05 06:31:54+02	2020-10-05 06:33:46+02	net	ntopng	\N	ntopng-4.0.d20200917,1
2020-10-05 06:33:47+02	2020-10-05 06:35:06+02	net	rdesktop	\N	rdesktop-1.9.0
2020-10-05 06:35:07+02	2020-10-05 06:37:05+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-05 06:17:13+02	2020-10-05 06:40:08+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.11
2020-10-05 06:40:09+02	2020-10-05 06:41:11+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-05 06:41:13+02	2020-10-05 06:42:28+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-05 06:42:29+02	2020-10-05 06:43:11+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-05 06:37:05+02	2020-10-05 06:43:51+02	net	wireshark	\N	wireshark-3.2.7
2020-10-05 06:43:12+02	2020-10-05 06:43:52+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-05 06:43:52+02	2020-10-05 06:44:32+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-05 06:43:52+02	2020-10-05 06:44:33+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-05 06:44:33+02	2020-10-05 06:45:14+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-05 06:44:34+02	2020-10-05 06:45:15+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-05 06:45:15+02	2020-10-05 06:46:00+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-05 06:45:15+02	2020-10-05 06:46:19+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-05 06:17:55+02	2020-10-05 06:46:29+02	lang	erlang-runtime23	\N	erlang-runtime23-23.1.1
2020-10-05 06:46:00+02	2020-10-05 06:46:42+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-05 06:46:19+02	2020-10-05 06:46:56+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-10-05 06:46:29+02	2020-10-05 06:47:10+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-10-05 06:46:57+02	2020-10-05 06:47:22+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-10-05 06:46:43+02	2020-10-05 06:48:40+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-05 06:49:12+02	2020-10-05 06:50:05+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-10-05 06:50:06+02	2020-10-05 06:50:54+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-05 06:50:55+02	2020-10-05 06:51:33+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-10-05 06:47:42+02	2020-10-05 06:51:37+02	print	scribus-devel	\N	scribus-devel-1.5.5_19
2020-10-05 06:48:41+02	2020-10-05 06:52:18+02	print	texlive-full	\N	texlive-full-20150521
2020-10-05 06:51:37+02	2020-10-05 06:52:27+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-05 06:51:33+02	2020-10-05 06:52:38+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-05 06:52:18+02	2020-10-05 06:53:01+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-10-05 06:52:39+02	2020-10-05 06:53:18+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-10-05 06:53:18+02	2020-10-05 06:54:45+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-10-05 06:54:45+02	2020-10-05 06:55:32+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-10-05 06:55:33+02	2020-10-05 06:55:54+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-10-05 06:54:14+02	2020-10-05 06:59:03+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-05 06:59:03+02	2020-10-05 06:59:55+02	graphics	xfig	\N	xfig-3.2.7a
2020-10-05 06:55:55+02	2020-10-05 07:20:08+02	editors	vscode	\N	vscode-1.46.1
2020-10-05 06:52:28+02	2020-10-05 07:26:43+02	www	firefox	\N	firefox-81.0.1,2
2020-10-05 07:26:44+02	2020-10-05 07:42:14+02	local	E590T-localbase	\N	E590T-localbase-2020100400
2020-10-05 07:54:27+02	2020-10-05 07:55:34+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-05 07:54:27+02	2020-10-05 08:00:52+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-05 08:00:53+02	2020-10-05 08:01:01+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-05 19:24:23+02	2020-10-05 19:24:49+02	devel	py-sip	py37	py37-sip-4.19.24,1
2020-10-05 19:24:49+02	2020-10-05 19:25:18+02	x11	mate-menus	\N	mate-menus-1.24.1
2020-10-05 19:24:23+02	2020-10-05 19:25:29+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.5
2020-10-05 19:24:23+02	2020-10-05 19:25:44+02	x11	mate-desktop	\N	mate-desktop-1.24.1
2020-10-05 19:25:18+02	2020-10-05 19:26:17+02	x11	libmatekbd	\N	libmatekbd-1.24.1
2020-10-05 19:25:44+02	2020-10-05 19:29:49+02	x11	mate-panel	\N	mate-panel-1.24.1
2020-10-05 19:28:32+02	2020-10-05 19:30:30+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-05 19:29:50+02	2020-10-05 19:31:24+02	x11	mate-session-manager	\N	mate-session-manager-1.24.1
2020-10-05 19:31:25+02	2020-10-05 19:32:35+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.2
2020-10-05 19:25:29+02	2020-10-05 19:33:09+02	devel	py-qt5-core	py37	py37-qt5-core-5.15.0
2020-10-05 19:30:30+02	2020-10-05 19:33:11+02	x11-wm	marco	\N	marco-1.24.1
2020-10-05 19:32:36+02	2020-10-05 19:34:56+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-05 19:33:12+02	2020-10-05 19:36:02+02	net	py-qt5-network	py37	py37-qt5-network-5.15.0
2020-10-05 19:36:02+02	2020-10-05 19:38:28+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.15.0
2020-10-05 19:34:57+02	2020-10-05 19:38:53+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-05 19:38:29+02	2020-10-05 19:39:33+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-05 19:38:54+02	2020-10-05 19:40:08+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-05 19:40:09+02	2020-10-05 19:41:07+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-05 19:39:34+02	2020-10-05 19:42:32+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-05 19:42:32+02	2020-10-05 19:45:10+02	graphics	atril	\N	atril-1.22.3_9
2020-10-05 19:53:25+02	2020-10-05 19:54:54+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-05 19:47:09+02	2020-10-05 19:54:59+02	graphics	oidn	\N	oidn-1.2.4
2020-10-05 19:55:00+02	2020-10-05 19:55:58+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.15.0
2020-10-05 19:54:55+02	2020-10-05 19:56:22+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.15.0
2020-10-05 19:55:59+02	2020-10-05 19:56:29+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.3
2020-10-05 19:33:10+02	2020-10-05 19:56:50+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.15.0
2020-10-05 19:56:29+02	2020-10-05 19:57:27+02	x11	mate-base	\N	mate-base-1.24.1
2020-10-05 19:57:27+02	2020-10-05 19:59:26+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.15.0
2020-10-05 19:56:23+02	2020-10-05 20:03:21+02	x11	mate-applets	\N	mate-applets-1.24.1
2020-10-05 19:59:26+02	2020-10-05 20:03:35+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.15.0
2020-10-05 20:03:22+02	2020-10-05 20:04:41+02	x11	mate-screensaver	\N	mate-screensaver-1.24.1
2020-10-05 20:03:36+02	2020-10-05 20:04:49+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-05 19:56:50+02	2020-10-05 20:05:30+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.15.0
2020-10-05 20:04:42+02	2020-10-05 20:06:21+02	x11	mate-terminal	\N	mate-terminal-1.24.1
2020-10-05 20:05:30+02	2020-10-05 20:06:38+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.15.0
2020-10-05 20:06:21+02	2020-10-05 20:07:30+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.15.0
2020-10-05 20:06:39+02	2020-10-05 20:07:40+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.15.0
2020-10-05 20:04:49+02	2020-10-05 20:09:30+02	graphics	blender	\N	blender-2.90.0_1
2020-10-05 20:07:41+02	2020-10-05 20:09:46+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.15.0
2020-10-05 20:07:30+02	2020-10-05 20:09:53+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.5
2020-10-05 20:09:30+02	2020-10-05 20:10:59+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.15.0
2020-10-05 20:10:59+02	2020-10-05 20:11:39+02	devel	tortoisehg	py37	tortoisehg-5.5.1_2
2020-10-05 20:11:40+02	2020-10-05 20:11:53+02	security	rkhunter	\N	rkhunter-1.4.6
2020-10-05 20:09:53+02	2020-10-05 20:12:00+02	deskutils	calibre	\N	calibre-4.23.0_2
2020-10-05 20:11:34+02	2020-10-05 20:13:39+02	multimedia	openshot	py37	openshot-2.5.1.369_1
2020-10-05 20:13:39+02	2020-10-05 20:29:33+02	local	E590T-localbase	\N	E590T-localbase-2020100400
2020-10-05 20:41:46+02	2020-10-05 20:42:43+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-05 20:41:46+02	2020-10-05 20:43:55+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-05 20:43:56+02	2020-10-05 20:44:03+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-06 07:44:51+02	2020-10-06 07:45:05+02	graphics	libexif	\N	libexif-0.6.22
2020-10-06 07:44:51+02	2020-10-06 07:46:13+02	security	libgcrypt	\N	libgcrypt-1.8.6
2020-10-06 07:45:05+02	2020-10-06 07:46:14+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-10-06 07:46:14+02	2020-10-06 07:47:14+02	textproc	libxslt	\N	libxslt-1.1.34_1
2020-10-06 07:46:14+02	2020-10-06 07:47:29+02	devel	libsoup	\N	libsoup-2.62.3
2020-10-06 07:47:22+02	2020-10-06 07:47:34+02	textproc	minixmlto	\N	minixmlto-0.0.3
2020-10-06 07:47:15+02	2020-10-06 07:47:48+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-10-06 07:47:49+02	2020-10-06 07:48:08+02	multimedia	libaacs	\N	libaacs-0.9.0
2020-10-06 07:47:35+02	2020-10-06 07:48:09+02	security	libsecret	\N	libsecret-0.20.3
2020-10-06 07:48:09+02	2020-10-06 07:48:43+02	multimedia	libbdplus	\N	libbdplus-0.1.2
2020-10-06 07:48:09+02	2020-10-06 07:48:53+02	net	geoclue	\N	geoclue-2.5.5
2020-10-06 07:48:43+02	2020-10-06 07:49:06+02	multimedia	libbluray	\N	libbluray-1.1.1_2,1
2020-10-06 07:47:30+02	2020-10-06 07:50:17+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-10-06 07:50:54+02	2020-10-06 07:51:49+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-10-06 07:51:49+02	2020-10-06 07:52:21+02	textproc	xmlto	\N	xmlto-0.0.28
2020-10-06 07:52:21+02	2020-10-06 07:53:08+02	sysutils	hal	\N	hal-0.5.14_34
2020-10-06 07:53:09+02	2020-10-06 07:54:14+02	security	gnupg	\N	gnupg-2.2.23
2020-10-06 07:49:07+02	2020-10-06 07:55:16+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-10-06 07:54:14+02	2020-10-06 07:55:47+02	audio	pulseaudio	\N	pulseaudio-13.0_1
2020-10-06 07:55:17+02	2020-10-06 07:55:48+02	security	gpgme	\N	gpgme-1.14.0
2020-10-06 07:55:48+02	2020-10-06 07:56:03+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-10-06 07:48:53+02	2020-10-06 07:57:13+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-06 07:55:48+02	2020-10-06 07:57:24+02	security	gcr	\N	gcr-3.28.0_1
2020-10-06 07:57:24+02	2020-10-06 07:58:42+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-10-06 07:58:43+02	2020-10-06 07:59:27+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-10-06 07:59:27+02	2020-10-06 08:00:04+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-10-06 08:00:05+02	2020-10-06 08:00:50+02	devel	libsoup-gnome	\N	libsoup-gnome-2.62.3
2020-10-06 08:00:00+02	2020-10-06 08:01:16+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-10-06 08:01:20+02	2020-10-06 08:02:35+02	net	remmina	\N	remmina-1.3.9
2020-10-06 08:00:50+02	2020-10-06 08:02:51+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-10-06 07:57:14+02	2020-10-06 08:02:54+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-06 08:02:35+02	2020-10-06 08:03:19+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-10-06 08:02:52+02	2020-10-06 08:03:26+02	devel	librest	\N	rest-0.7.93
2020-10-06 08:02:54+02	2020-10-06 08:03:48+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-06 08:03:19+02	2020-10-06 08:04:11+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-10-06 08:04:12+02	2020-10-06 08:04:55+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-10-06 08:03:48+02	2020-10-06 08:04:56+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-06 08:03:26+02	2020-10-06 08:05:01+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-06 08:04:57+02	2020-10-06 08:05:08+02	textproc	yelp-xsl	\N	yelp-xsl-3.38.0
2020-10-06 08:04:55+02	2020-10-06 08:05:19+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-10-06 08:05:01+02	2020-10-06 08:05:45+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-06 08:05:08+02	2020-10-06 08:05:59+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-10-06 08:05:45+02	2020-10-06 08:07:08+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-10-06 08:05:59+02	2020-10-06 08:09:33+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-06 08:05:19+02	2020-10-06 08:10:16+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-06 08:10:17+02	2020-10-06 08:11:44+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-06 08:11:45+02	2020-10-06 08:12:14+02	mail	gmime30	\N	gmime30-3.2.7
2020-10-06 08:12:14+02	2020-10-06 08:12:41+02	multimedia	libquvi09	\N	libquvi09-0.9.4_4
2020-10-06 08:12:49+02	2020-10-06 08:13:06+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-10-06 08:07:09+02	2020-10-06 08:13:16+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-10-06 08:13:17+02	2020-10-06 08:14:05+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-10-06 08:14:05+02	2020-10-06 08:16:52+02	x11	mate-panel	\N	mate-panel-1.24.1
2020-10-06 08:16:52+02	2020-10-06 08:18:35+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-10-06 08:18:39+02	2020-10-06 08:26:12+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-06 08:26:13+02	2020-10-06 08:26:52+02	audio	chromaprint	\N	chromaprint-1.5.0
2020-10-06 08:12:42+02	2020-10-06 08:27:02+02	net	samba410	\N	samba410-4.10.18
2020-10-06 08:26:52+02	2020-10-06 08:27:40+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-10-06 08:27:41+02	2020-10-06 08:28:16+02	multimedia	totem-pl-parser	\N	totem-pl-parser-3.26.5
2020-10-06 08:28:16+02	2020-10-06 08:29:02+02	net	geocode-glib	\N	geocode-glib-3.26.2
2020-10-06 08:29:03+02	2020-10-06 08:29:45+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-10-06 08:29:45+02	2020-10-06 08:30:43+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-06 08:30:44+02	2020-10-06 08:31:42+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-10-06 08:31:43+02	2020-10-06 08:33:07+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-10-06 08:13:07+02	2020-10-06 08:33:19+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-06 08:33:07+02	2020-10-06 08:34:04+02	audio	gstreamer1-plugins-chromaprint	\N	gstreamer1-plugins-chromaprint-1.16.2
2020-10-06 08:33:20+02	2020-10-06 08:35:11+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-10-06 08:34:04+02	2020-10-06 08:35:20+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-10-06 08:35:12+02	2020-10-06 08:35:44+02	mail	gmime26	\N	gmime26-2.6.23_1
2020-10-06 08:35:45+02	2020-10-06 08:37:14+02	net	libgweather	\N	libgweather-3.36.1
2020-10-06 08:35:54+02	2020-10-06 08:37:44+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-06 08:37:14+02	2020-10-06 08:37:58+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-10-06 08:37:59+02	2020-10-06 08:39:08+02	sysutils	tracker	\N	tracker-2.3.4
2020-10-06 08:37:45+02	2020-10-06 08:39:19+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-10-06 08:39:08+02	2020-10-06 08:39:22+02	textproc	yelp-tools	\N	yelp-tools-3.38.0
2020-10-06 08:39:23+02	2020-10-06 08:39:56+02	devel	gstreamer1-plugins-soup	\N	gstreamer1-plugins-soup-1.16.2
2020-10-06 08:39:20+02	2020-10-06 08:40:13+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-10-06 08:40:52+02	2020-10-06 08:42:07+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-06 08:40:13+02	2020-10-06 08:43:21+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-10-06 08:42:35+02	2020-10-06 08:43:23+02	net	libvncserver	\N	libvncserver-0.9.12_2
2020-10-06 08:39:56+02	2020-10-06 08:43:39+02	graphics	opencv	\N	opencv-3.4.1_36
2020-10-06 08:43:40+02	2020-10-06 08:44:23+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-10-06 08:43:21+02	2020-10-06 08:44:26+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0
2020-10-06 08:43:23+02	2020-10-06 08:44:51+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-06 08:44:26+02	2020-10-06 08:45:23+02	x11	mate-session-manager	\N	mate-session-manager-1.24.1
2020-10-06 08:45:44+02	2020-10-06 08:46:26+02	devel	git-lfs	\N	git-lfs-2.12.0
2020-10-06 08:45:23+02	2020-10-06 08:47:21+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-06 08:47:22+02	2020-10-06 08:49:07+02	devel	shiboken2	py37	py37-shiboken2-5.15.1
2020-10-06 08:49:08+02	2020-10-06 08:51:20+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-10-06 08:51:20+02	2020-10-06 09:01:41+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-06 09:01:42+02	2020-10-06 09:03:09+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-06 09:03:10+02	2020-10-06 09:06:15+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-06 09:06:51+02	2020-10-06 09:08:00+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-06 09:08:01+02	2020-10-06 09:09:38+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-10-06 09:09:38+02	2020-10-06 09:10:04+02	sysutils	hal-info	\N	hal-info-20091130
2020-10-06 09:10:08+02	2020-10-06 09:14:40+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-06 09:14:41+02	2020-10-06 09:16:04+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-06 09:16:04+02	2020-10-06 09:19:16+02	textproc	dblatex	\N	dblatex-0.3.11
2020-10-06 09:19:19+02	2020-10-06 09:20:28+02	textproc	redland	\N	redland-1.0.17_4
2020-10-06 09:20:29+02	2020-10-06 09:21:50+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.15.0
2020-10-06 09:21:52+02	2020-10-06 09:22:31+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-10-06 08:44:23+02	2020-10-06 09:25:29+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-06 09:25:32+02	2020-10-06 09:27:17+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-06 09:22:32+02	2020-10-06 09:28:24+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-06 09:27:18+02	2020-10-06 09:29:32+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-06 09:28:25+02	2020-10-06 09:29:38+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-06 09:29:33+02	2020-10-06 09:30:46+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-06 09:30:47+02	2020-10-06 09:32:27+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-06 09:29:38+02	2020-10-06 09:33:03+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-06 09:42:34+02	2020-10-06 09:48:43+02	editors	pluma	\N	pluma-1.24.1
2020-10-06 09:48:44+02	2020-10-06 09:52:11+02	graphics	atril	\N	atril-1.22.3_9
2020-10-06 09:54:14+02	2020-10-06 09:55:51+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-10-06 09:33:03+02	2020-10-06 10:00:51+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-10-06 10:00:53+02	2020-10-06 10:01:24+02	multimedia	libmtp	\N	libmtp-1.1.17
2020-10-06 10:01:24+02	2020-10-06 10:03:01+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-10-06 09:55:52+02	2020-10-06 10:04:51+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-06 10:03:02+02	2020-10-06 10:04:59+02	multimedia	mpv	\N	mpv-0.32.0_5,1
2020-10-06 10:05:00+02	2020-10-06 10:06:07+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-06 10:06:08+02	2020-10-06 10:07:07+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-06 10:07:07+02	2020-10-06 10:08:09+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-06 10:08:09+02	2020-10-06 10:09:02+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-06 10:04:52+02	2020-10-06 10:09:09+02	multimedia	vlc	\N	vlc-3.0.11_5,4
2020-10-06 10:09:03+02	2020-10-06 10:09:56+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-06 10:09:10+02	2020-10-06 10:09:57+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-06 10:09:57+02	2020-10-06 10:10:46+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-06 10:09:57+02	2020-10-06 10:11:02+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-06 10:10:46+02	2020-10-06 10:12:19+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-06 10:11:02+02	2020-10-06 10:12:27+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-06 10:12:20+02	2020-10-06 10:13:10+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-06 10:13:10+02	2020-10-06 10:13:52+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-10-06 10:12:27+02	2020-10-06 10:14:15+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-06 10:13:53+02	2020-10-06 10:14:33+02	textproc	py-regex	py37	py37-regex-2020.7.14
2020-10-06 10:14:15+02	2020-10-06 10:14:43+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.3
2020-10-06 10:14:33+02	2020-10-06 10:15:39+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.15.0
2020-10-06 10:14:44+02	2020-10-06 10:16:00+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.15.0
2020-10-06 10:16:01+02	2020-10-06 10:17:19+02	x11	mate-base	\N	mate-base-1.24.1
2020-10-06 10:17:20+02	2020-10-06 10:18:47+02	x11	mate-screensaver	\N	mate-screensaver-1.24.1
2020-10-06 10:18:47+02	2020-10-06 10:21:06+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-06 10:15:40+02	2020-10-06 10:21:51+02	x11	mate-applets	\N	mate-applets-1.24.1
2020-10-06 10:21:07+02	2020-10-06 10:25:07+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-06 10:25:08+02	2020-10-06 10:27:30+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-06 10:24:25+02	2020-10-06 10:29:48+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-06 10:29:48+02	2020-10-06 10:31:47+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-06 10:31:48+02	2020-10-06 10:33:36+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-06 10:33:36+02	2020-10-06 10:38:05+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-06 10:38:05+02	2020-10-06 10:40:34+02	deskutils	calibre	\N	calibre-4.23.0_2
2020-10-06 10:40:35+02	2020-10-06 11:09:37+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-06 11:09:37+02	2020-10-06 11:18:50+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-06 11:18:51+02	2020-10-06 11:46:27+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-06 11:47:09+02	2020-10-06 11:57:02+02	editors	codelite	\N	codelite-12.0_4
2020-10-06 11:57:03+02	2020-10-06 11:59:21+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-10-06 10:27:30+02	2020-10-06 12:17:05+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-06 12:17:06+02	2020-10-06 12:35:46+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-06 12:35:48+02	2020-10-06 12:42:26+02	graphics	blender	\N	blender-2.90.0_1
2020-10-06 12:42:27+02	2020-10-06 12:45:13+02	graphics	dia	\N	dia-0.97.3_1,1
2020-10-06 12:45:14+02	2020-10-06 12:50:41+02	graphics	inkscape	\N	inkscape-1.0.1
2020-10-06 12:50:42+02	2020-10-06 13:05:00+02	graphics	krita	\N	krita-4.3.0_5
2020-10-06 13:05:03+02	2020-10-06 13:09:02+02	graphics	wings	\N	wings-2.2.5
2020-10-06 11:59:22+02	2020-10-06 13:12:37+02	emulators	virtualbox-ose	\N	virtualbox-ose-5.2.44_3
2020-10-06 13:12:38+02	2020-10-06 13:13:35+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-06 13:09:02+02	2020-10-06 13:15:05+02	math	maxima	\N	maxima-5.44.0_5
2020-10-06 13:13:36+02	2020-10-06 13:15:23+02	multimedia	obs-studio	\N	obs-studio-26.0.0_1
2020-10-06 13:15:06+02	2020-10-06 13:17:51+02	multimedia	openshot	py37	openshot-2.5.1.369_1
2020-10-06 13:15:24+02	2020-10-06 13:18:17+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-06 13:17:52+02	2020-10-06 13:19:24+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-06 13:19:25+02	2020-10-06 13:21:22+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-06 13:21:23+02	2020-10-06 13:22:47+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-06 13:18:17+02	2020-10-06 13:23:00+02	net	wireshark	\N	wireshark-3.2.7
2020-10-06 13:24:35+02	2020-10-06 13:30:19+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-06 13:30:20+02	2020-10-06 13:30:53+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-10-06 13:30:54+02	2020-10-06 14:07:24+02	editors	vscode	\N	vscode-1.46.1
2020-10-06 13:22:47+02	2020-10-06 14:12:23+02	www	firefox	\N	firefox-81.0.1,2
2020-10-06 08:46:27+02	2020-10-06 14:30:52+02	devel	llvm11	\N	llvm11-11.0.0.r5
2020-10-06 14:30:53+02	2020-10-06 14:37:15+02	databases	postgresql13-server	\N	postgresql13-server-13.0_2
2020-10-06 14:30:53+02	2020-10-06 14:38:18+02	databases	postgresql12-server	\N	postgresql12-server-12.4_1
2020-10-06 14:37:16+02	2020-10-06 14:52:50+02	local	E590T-localbase	\N	E590T-localbase-2020100400
2020-10-06 15:04:46+02	2020-10-06 15:05:48+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-06 15:04:46+02	2020-10-06 15:10:57+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-06 15:10:58+02	2020-10-06 15:11:05+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-06 18:37:41+02	2020-10-06 18:37:51+02	net	cppzmq	\N	cppzmq-4.7.1
2020-10-06 18:37:41+02	2020-10-06 18:55:07+02	lang	spidermonkey78	\N	spidermonkey78-78.3.1
2020-10-06 18:55:08+02	2020-10-06 18:56:26+02	sysutils	polkit	\N	polkit-0.118
2020-10-06 18:56:26+02	2020-10-06 19:00:46+02	graphics	colord	\N	colord-1.3.5_1
2020-10-06 18:37:41+02	2020-10-06 19:06:36+02	net	samba413	\N	samba413-4.13.0
2020-10-06 19:06:37+02	2020-10-06 19:07:51+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-10-06 19:00:47+02	2020-10-06 19:08:21+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-10-06 19:07:52+02	2020-10-06 19:08:27+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-10-06 19:08:29+02	2020-10-06 19:09:23+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-10-06 19:09:23+02	2020-10-06 19:10:12+02	sysutils	hal	\N	hal-0.5.14_34
2020-10-06 19:10:13+02	2020-10-06 19:12:01+02	x11	mate-desktop	\N	mate-desktop-1.24.1
2020-10-06 19:12:02+02	2020-10-06 19:13:43+02	audio	pulseaudio	\N	pulseaudio-13.0_1
2020-10-06 19:13:44+02	2020-10-06 19:15:54+02	security	gcr	\N	gcr-3.28.0_1
2020-10-06 19:15:55+02	2020-10-06 19:17:07+02	x11-toolkits	vte3	\N	vte3-0.62.0
2020-10-06 19:08:22+02	2020-10-06 19:17:14+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-06 19:17:07+02	2020-10-06 19:18:07+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-10-06 19:18:07+02	2020-10-06 19:19:08+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-10-06 19:19:08+02	2020-10-06 19:20:12+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-10-06 19:20:13+02	2020-10-06 19:22:34+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-10-06 19:17:15+02	2020-10-06 19:23:09+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-06 19:22:34+02	2020-10-06 19:23:32+02	net	remmina	\N	remmina-1.3.9
2020-10-06 19:23:10+02	2020-10-06 19:23:50+02	sysutils	upower	\N	upower-0.99.11
2020-10-06 19:23:33+02	2020-10-06 19:24:34+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-06 19:23:50+02	2020-10-06 19:24:56+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-10-06 19:24:34+02	2020-10-06 19:25:34+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-10-06 19:24:57+02	2020-10-06 19:26:49+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-06 19:25:35+02	2020-10-06 19:27:01+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-06 19:26:49+02	2020-10-06 19:27:45+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-06 19:27:02+02	2020-10-06 19:28:00+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-10-06 19:28:01+02	2020-10-06 19:29:14+02	devel	libpeas	\N	libpeas-1.22.0
2020-10-06 19:27:46+02	2020-10-06 19:29:23+02	devel	libgdata	\N	libgdata-0.17.9_1
2020-10-06 19:29:23+02	2020-10-06 19:32:28+02	x11	xorg	\N	xorg-7.7_3
2020-10-06 19:29:15+02	2020-10-06 19:32:57+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-06 19:32:28+02	2020-10-06 19:33:32+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-10-06 19:32:58+02	2020-10-06 19:34:08+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-06 19:33:33+02	2020-10-06 19:34:29+02	sysutils	policykit-gnome	\N	policykit-gnome-0.9.2_9
2020-10-06 19:34:30+02	2020-10-06 19:35:39+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-10-06 19:34:08+02	2020-10-06 19:36:34+02	x11	mate-panel	\N	mate-panel-1.24.1
2020-10-06 19:36:35+02	2020-10-06 19:40:56+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-10-06 19:35:40+02	2020-10-06 19:41:46+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-06 19:40:57+02	2020-10-06 19:42:08+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-10-06 19:42:09+02	2020-10-06 19:42:55+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-06 19:42:55+02	2020-10-06 19:43:58+02	sysutils	gnome-mount	\N	gnome-mount-0.8_13
2020-10-06 19:43:59+02	2020-10-06 19:45:25+02	x11	libmatekbd	\N	libmatekbd-1.24.1
2020-10-06 19:45:26+02	2020-10-06 19:47:57+02	x11-fm	caja	\N	caja-1.24.0
2020-10-06 19:47:58+02	2020-10-06 19:52:27+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-10-06 19:52:28+02	2020-10-06 19:54:06+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-10-06 19:54:07+02	2020-10-06 19:55:14+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-06 19:55:15+02	2020-10-06 19:57:00+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-10-06 19:57:01+02	2020-10-06 19:59:00+02	devel	gvfs	\N	gvfs-1.30.4_4
2020-10-06 19:41:47+02	2020-10-06 19:59:20+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-06 19:59:01+02	2020-10-06 20:00:15+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-10-06 19:59:21+02	2020-10-06 20:00:41+02	net	libgweather	\N	libgweather-3.36.1
2020-10-06 20:00:41+02	2020-10-06 20:01:21+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-10-06 20:00:15+02	2020-10-06 20:02:04+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-06 20:01:22+02	2020-10-06 20:02:59+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-10-06 20:02:04+02	2020-10-06 20:03:32+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-10-06 20:03:00+02	2020-10-06 20:04:09+02	x11	zenity	\N	zenity-3.32.0
2020-10-06 20:04:09+02	2020-10-06 20:05:07+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-06 20:03:32+02	2020-10-06 20:07:31+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-10-06 20:07:32+02	2020-10-06 20:08:28+02	graphics	clutter-gtk3	\N	clutter-gtk3-1.8.4
2020-10-06 20:08:28+02	2020-10-06 20:09:48+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-06 20:05:08+02	2020-10-06 20:10:06+02	editors	emacs	full	emacs-27.1_1,3
2020-10-06 20:09:49+02	2020-10-06 20:10:36+02	print	cups-filters	\N	cups-filters-1.28.3
2020-10-06 20:10:36+02	2020-10-06 20:12:02+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0
2020-10-06 20:12:03+02	2020-10-06 20:14:06+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-06 20:10:07+02	2020-10-06 20:14:09+02	print	fontforge	\N	fontforge-20190801_2
2020-10-06 20:14:06+02	2020-10-06 20:15:27+02	sysutils	zeitgeist	\N	zeitgeist-1.0.2
2020-10-06 20:15:28+02	2020-10-06 20:16:34+02	x11	mate-session-manager	\N	mate-session-manager-1.24.1
2020-10-06 20:16:35+02	2020-10-06 20:17:55+02	x11	yelp	\N	yelp-3.38.0
2020-10-06 20:17:56+02	2020-10-06 20:19:29+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-10-06 20:19:30+02	2020-10-06 20:23:32+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-10-06 20:23:33+02	2020-10-06 20:25:15+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-10-06 20:25:19+02	2020-10-06 20:27:08+02	x11-wm	marco	\N	marco-1.24.1
2020-10-06 20:27:09+02	2020-10-06 20:28:25+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-10-06 20:28:26+02	2020-10-06 20:30:58+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-06 20:30:58+02	2020-10-06 20:32:17+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-10-06 20:32:18+02	2020-10-06 20:35:47+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-06 20:35:48+02	2020-10-06 20:46:45+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-06 20:46:46+02	2020-10-06 20:50:08+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-06 20:50:13+02	2020-10-06 20:51:17+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-06 20:51:19+02	2020-10-06 20:55:27+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-06 20:14:10+02	2020-10-06 20:56:10+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-06 20:56:14+02	2020-10-06 20:57:45+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-06 20:55:30+02	2020-10-06 20:57:55+02	print	mftrace	\N	mftrace-1.2.18_2
2020-10-06 20:57:46+02	2020-10-06 20:58:04+02	sysutils	hal-info	\N	hal-info-20091130
2020-10-06 20:58:05+02	2020-10-06 20:59:01+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-06 20:59:01+02	2020-10-06 21:00:37+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-10-06 20:57:56+02	2020-10-06 21:01:10+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-06 21:01:10+02	2020-10-06 21:03:54+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-06 21:03:55+02	2020-10-06 21:05:19+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-06 21:00:37+02	2020-10-06 21:06:38+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-06 21:05:21+02	2020-10-06 21:06:47+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-06 21:06:48+02	2020-10-06 21:07:40+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-06 21:06:38+02	2020-10-06 21:09:55+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-06 21:09:55+02	2020-10-06 21:14:02+02	editors	pluma	\N	pluma-1.24.1
2020-10-06 21:14:03+02	2020-10-06 21:17:26+02	graphics	atril	\N	atril-1.22.3_9
2020-10-06 21:17:27+02	2020-10-06 21:20:06+02	graphics	eom	\N	eom-1.24.1
2020-10-06 21:20:07+02	2020-10-06 21:21:37+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-10-06 21:21:38+02	2020-10-06 21:31:47+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-06 21:31:47+02	2020-10-06 21:33:55+02	math	mate-calc	\N	mate-calc-1.24.1
2020-10-06 21:33:55+02	2020-10-06 21:34:48+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-06 21:07:41+02	2020-10-06 21:35:12+02	editors	libreoffice	\N	libreoffice-7.0.1.2_1
2020-10-06 21:34:48+02	2020-10-06 21:35:42+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-06 21:35:14+02	2020-10-06 21:36:06+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-06 21:35:43+02	2020-10-06 21:36:37+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-06 21:36:07+02	2020-10-06 21:36:56+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-06 21:36:38+02	2020-10-06 21:37:28+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-06 21:36:57+02	2020-10-06 21:37:59+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-06 21:37:29+02	2020-10-06 21:39:06+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-06 21:37:59+02	2020-10-06 21:39:26+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-06 21:39:07+02	2020-10-06 21:40:16+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-06 21:40:17+02	2020-10-06 21:41:13+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-06 21:41:13+02	2020-10-06 21:43:18+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-06 21:43:19+02	2020-10-06 21:45:21+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-10-06 21:39:27+02	2020-10-06 21:46:04+02	print	lilypond	\N	lilypond-2.20.0
2020-10-06 21:45:21+02	2020-10-06 21:46:27+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-10-06 21:46:28+02	2020-10-06 21:47:47+02	x11	mate-base	\N	mate-base-1.24.1
2020-10-06 21:47:48+02	2020-10-06 21:49:08+02	x11	mate-screensaver	\N	mate-screensaver-1.24.1
2020-10-06 21:49:09+02	2020-10-06 21:51:29+02	x11	mate-terminal	\N	mate-terminal-1.24.1
2020-10-06 21:46:05+02	2020-10-06 21:52:18+02	x11	mate-applets	\N	mate-applets-1.24.1
2020-10-06 21:52:18+02	2020-10-06 21:53:31+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-06 21:51:30+02	2020-10-06 21:53:34+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-06 21:53:32+02	2020-10-06 21:57:43+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-06 21:53:35+02	2020-10-06 21:59:25+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-06 21:57:44+02	2020-10-06 22:00:19+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-06 22:00:20+02	2020-10-06 22:02:39+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-06 22:02:40+02	2020-10-06 22:04:35+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-06 22:04:36+02	2020-10-06 22:08:04+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-06 18:48:45+02	2020-10-06 22:08:38+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20201004
2020-10-06 22:08:04+02	2020-10-06 22:08:47+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-10-06 22:08:47+02	2020-10-06 22:09:40+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-06 22:08:39+02	2020-10-06 22:16:25+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-06 22:16:25+02	2020-10-06 22:26:43+02	editors	codelite	\N	codelite-12.0_4
2020-10-06 22:26:43+02	2020-10-06 22:29:18+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.1.2
2020-10-06 22:29:19+02	2020-10-06 22:31:08+02	editors	vim	\N	vim-8.2.1558
2020-10-06 22:09:40+02	2020-10-06 22:35:02+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-06 22:31:09+02	2020-10-06 22:47:54+02	emulators	qemu	\N	qemu-4.2.1
2020-10-06 22:47:55+02	2020-10-06 22:48:44+02	games	xcowsay	\N	xcowsay-1.5.1
2020-10-06 22:48:46+02	2020-10-06 22:49:40+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-06 22:35:05+02	2020-10-06 22:49:43+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-06 22:49:40+02	2020-10-06 22:53:45+02	graphics	inkscape	\N	inkscape-1.0.1
2020-10-06 22:53:46+02	2020-10-06 22:58:08+02	graphics	wings	\N	wings-2.2.5
2020-10-06 22:58:09+02	2020-10-06 22:59:02+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-06 22:49:44+02	2020-10-06 22:59:14+02	graphics	krita	\N	krita-4.3.0_5
2020-10-06 22:59:03+02	2020-10-06 23:18:05+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.11
2020-10-06 23:18:05+02	2020-10-06 23:22:26+02	math	maxima	\N	maxima-5.44.0_5
2020-10-06 22:59:15+02	2020-10-06 23:23:12+02	lang	erlang-runtime23	\N	erlang-runtime23-23.1.1
2020-10-06 23:22:27+02	2020-10-06 23:23:12+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-06 23:23:13+02	2020-10-06 23:24:31+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-06 23:23:13+02	2020-10-06 23:24:37+02	multimedia	obs-studio	\N	obs-studio-26.0.0_1
2020-10-06 23:24:31+02	2020-10-06 23:25:09+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-06 23:24:37+02	2020-10-06 23:25:40+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-06 23:25:40+02	2020-10-06 23:26:26+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-06 23:25:09+02	2020-10-06 23:27:04+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-06 23:26:27+02	2020-10-06 23:27:28+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-06 23:27:05+02	2020-10-06 23:27:52+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-06 21:59:26+02	2020-10-06 23:28:11+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-06 23:27:53+02	2020-10-06 23:29:08+02	x11	mate	\N	mate-1.24.1
2020-10-06 23:28:12+02	2020-10-06 23:32:35+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-06 23:29:09+02	2020-10-06 23:52:52+02	editors	vscode	\N	vscode-1.46.1
2020-10-06 23:27:29+02	2020-10-07 00:00:02+02	www	firefox	\N	firefox-81.0.1,2
2020-10-07 00:00:04+02	2020-10-07 00:15:37+02	local	E590T-localbase	\N	E590T-localbase-2020100600
2020-10-07 00:27:45+02	2020-10-07 00:28:39+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-07 00:27:45+02	2020-10-07 00:29:50+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-07 00:29:50+02	2020-10-07 00:29:58+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-07 22:57:03+02	2020-10-07 23:03:28+02	devel	rust-cbindgen	\N	rust-cbindgen-0.15.0
2020-10-07 22:57:03+02	2020-10-07 23:03:35+02	astro	stellarium	\N	stellarium-0.20.3
2020-10-07 23:03:29+02	2020-10-07 23:03:43+02	sysutils	zfs-stats	\N	zfs-stats-1.3.0_1
2020-10-07 23:03:43+02	2020-10-07 23:22:15+02	local	E590T-localbase	\N	E590T-localbase-2020100600
2020-10-07 23:38:00+02	2020-10-07 23:39:22+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-07 23:38:00+02	2020-10-07 23:45:11+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-07 23:45:11+02	2020-10-07 23:45:19+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-08 07:48:03+02	2020-10-08 07:48:23+02	devel	json-c	\N	json-c-0.15_1
2020-10-08 07:48:23+02	2020-10-08 07:50:34+02	sysutils	slurm-wlm	\N	slurm-wlm-20.02.1_2
2020-10-08 07:50:34+02	2020-10-08 07:58:04+02	net	openmpi	\N	openmpi-4.0.5
2020-10-08 07:57:03+02	2020-10-08 08:00:41+02	dns	bind-tools	\N	bind-tools-9.16.7
2020-10-08 08:00:41+02	2020-10-08 08:01:04+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-10-08 07:58:05+02	2020-10-08 08:04:51+02	math	vtk8	\N	vtk8-8.2.0
2020-10-08 08:01:05+02	2020-10-08 08:10:46+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-10-08 08:04:52+02	2020-10-08 08:11:55+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-08 08:10:47+02	2020-10-08 08:11:55+02	net	ndpi	\N	ndpi-3.2.d20200721,1
2020-10-08 08:11:57+02	2020-10-08 08:13:00+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-10-08 08:13:00+02	2020-10-08 08:14:30+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-08 08:14:31+02	2020-10-08 08:19:36+02	dns	bind916	\N	bind916-9.16.7_1
2020-10-08 08:19:36+02	2020-10-08 08:20:58+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-08 08:20:59+02	2020-10-08 08:24:02+02	net	ntopng	\N	ntopng-4.0.d20200917,1
2020-10-08 08:24:03+02	2020-10-08 08:28:06+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-10-08 08:11:57+02	2020-10-08 08:30:04+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-08 08:28:07+02	2020-10-08 08:31:08+02	security	clamav	\N	clamav-0.103.0,1
2020-10-08 08:31:08+02	2020-10-08 08:33:38+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-13 11:43:48+02	2020-10-13 11:44:07+02	net	gssdp	\N	gssdp-1.2.3
2020-10-08 08:30:05+02	2020-10-08 08:35:53+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-08 08:35:53+02	2020-10-08 08:38:11+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-08 08:38:12+02	2020-10-08 08:39:59+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-08 07:48:03+02	2020-10-08 09:35:05+02	www	firefox	\N	firefox-81.0.1_1,2
2020-10-08 08:33:39+02	2020-10-08 09:45:48+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-08 09:45:50+02	2020-10-08 10:01:21+02	local	E590T-localbase	\N	E590T-localbase-2020100600
2020-10-08 10:13:21+02	2020-10-08 10:14:16+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-08 10:13:21+02	2020-10-08 10:15:26+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-08 10:15:26+02	2020-10-08 10:15:34+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-08 19:37:31+02	2020-10-08 19:37:49+02	databases	postgresql-jdbc	\N	postgresql-jdbc-42.2.16
2020-10-08 19:37:31+02	2020-10-08 19:38:05+02	databases	pg_qualstats	\N	pg_qualstats-2.0.2
2020-10-08 19:37:31+02	2020-10-08 19:38:05+02	databases	pg_stat_kcache	\N	pg_stat_kcache-2.1.3
2020-10-08 19:38:05+02	2020-10-08 19:38:38+02	databases	powa-archivist	\N	powa-archivist-4.0.1
2020-10-08 19:37:50+02	2020-10-08 19:38:50+02	databases	py-sqlalchemy10	py37	py37-sqlalchemy10-1.0.14
2020-10-08 19:38:38+02	2020-10-08 19:39:07+02	databases	powerarchitect	\N	powerarchitect-1.0.8_1
2020-10-08 19:38:50+02	2020-10-08 19:39:14+02	databases	powa-web	py37	py37-powa-web-4.0.2
2020-10-08 19:39:08+02	2020-10-08 19:39:33+02	databases	pspg	\N	pspg-3.1.4
2020-10-08 20:20:54+02	2020-10-08 20:21:12+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.6
2020-10-08 20:21:13+02	2020-10-08 20:22:07+02	databases	py-sqlalchemy10	py37	py37-sqlalchemy10-1.0.14
2020-10-08 20:22:08+02	2020-10-08 20:22:29+02	databases	powa-web	py37	py37-powa-web-4.0.2
2020-10-08 20:22:30+02	2020-10-08 20:41:08+02	local	E590T-localbase	\N	E590T-localbase-2020100800
2020-10-08 20:41:23+02	2020-10-08 20:42:37+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-08 20:41:23+02	2020-10-08 20:43:53+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-08 20:43:54+02	2020-10-08 20:44:02+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-09 07:19:52+02	2020-10-09 07:20:26+02	devel	py-zipp	py37	py37-zipp-3.3.0
2020-10-09 07:20:28+02	2020-10-09 07:21:03+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-2.0.0
2020-10-09 07:21:05+02	2020-10-09 07:21:34+02	devel	py-pluggy	py37	py37-pluggy-0.13.1
2020-10-09 07:21:38+02	2020-10-09 07:30:16+02	math	py-matplotlib	py37	py37-matplotlib-3.3.1
2020-10-09 07:30:17+02	2020-10-09 07:30:31+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-10-09 07:30:31+02	2020-10-09 07:31:25+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-10-09 07:19:51+02	2020-10-09 07:31:25+02	devel	ncurses	\N	ncurses-6.2.20201003
2020-10-09 07:31:26+02	2020-10-09 07:32:02+02	devel	adacurses	\N	adacurses-20180127
2020-10-09 07:32:02+02	2020-10-09 07:32:19+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-10-09 07:32:20+02	2020-10-09 07:32:38+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-10-09 07:32:17+02	2020-10-09 07:33:04+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-10-09 07:33:05+02	2020-10-09 07:33:34+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-10-09 07:31:26+02	2020-10-09 07:35:42+02	graphics	py-scikit-image	py37	py37-scikit-image-0.17.2
2020-10-09 07:35:43+02	2020-10-09 07:36:32+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-09 07:36:32+02	2020-10-09 07:37:50+02	cad	cura	\N	Cura-4.7.1,2
2020-10-09 07:37:51+02	2020-10-09 07:55:15+02	local	E590T-localbase	\N	E590T-localbase-2020100801
2020-10-09 08:07:35+02	2020-10-09 08:08:41+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-09 08:07:35+02	2020-10-09 08:14:10+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-09 08:14:11+02	2020-10-09 08:14:18+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-09 23:49:41+02	2020-10-09 23:50:52+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0_1
2020-10-09 23:49:41+02	2020-10-09 23:51:49+02	x11	mate-panel	\N	mate-panel-1.24.1_1
2020-10-09 23:51:50+02	2020-10-09 23:52:46+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-09 23:50:53+02	2020-10-09 23:52:47+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-09 23:52:47+02	2020-10-09 23:56:06+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-09 23:49:41+02	2020-10-10 00:01:29+02	lang	ruby27	\N	ruby-2.7.2,1
2020-10-10 00:01:30+02	2020-10-10 00:01:42+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-10-10 00:01:42+02	2020-10-10 00:01:53+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-10-10 00:01:54+02	2020-10-10 00:02:03+02	devel	rubygem-mustache	\N	rubygem-mustache-1.1.1
2020-10-10 00:00:59+02	2020-10-10 00:02:13+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-10 00:02:03+02	2020-10-10 00:02:22+02	textproc	rubygem-rdiscount	\N	rubygem-rdiscount-2.2.0.1
2020-10-10 00:02:22+02	2020-10-10 00:02:33+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-10-10 00:02:34+02	2020-10-10 00:02:45+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-10-10 00:02:14+02	2020-10-10 00:03:00+02	www	rubygem-hpricot	\N	rubygem-hpricot-0.8.6
2020-10-10 00:02:45+02	2020-10-10 00:03:00+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-10-10 00:03:00+02	2020-10-10 00:03:12+02	textproc	rubygem-ronn	\N	rubygem-ronn-0.7.3
2020-10-10 00:03:12+02	2020-10-10 00:04:34+02	x11	mate-base	\N	mate-base-1.24.1
2020-10-10 00:04:35+02	2020-10-10 00:07:18+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-10 00:07:18+02	2020-10-10 00:09:34+02	editors	vim	\N	vim-8.2.1558
2020-10-10 00:03:01+02	2020-10-10 00:09:46+02	x11	mate-applets	\N	mate-applets-1.24.1
2020-10-10 00:09:34+02	2020-10-10 00:09:49+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-10-10 00:09:47+02	2020-10-10 00:09:58+02	sysutils	bpytop	\N	bpytop-1.0.42
2020-10-10 00:09:49+02	2020-10-10 00:10:12+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-10-10 00:09:58+02	2020-10-10 00:11:30+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-10 00:10:13+02	2020-10-10 00:11:51+02	x11	mate	\N	mate-1.24.1
2020-10-09 23:56:06+02	2020-10-10 03:21:21+02	editors	libreoffice	\N	libreoffice-7.0.2.2_1
2020-10-10 03:21:22+02	2020-10-10 03:22:40+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.2.2
2020-10-10 03:22:41+02	2020-10-10 03:40:45+02	local	E590T-localbase	\N	E590T-localbase-2020100801
2020-10-10 03:52:44+02	2020-10-10 03:53:45+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-10 03:52:44+02	2020-10-10 03:59:01+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-10 03:59:01+02	2020-10-10 03:59:09+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-10 12:12:46+02	2020-10-10 12:13:03+02	www	p5-HTML-Parser	\N	p5-HTML-Parser-3.75
2020-10-10 12:12:46+02	2020-10-10 12:13:06+02	graphics	jasper	\N	jasper-2.0.22
2020-10-10 12:13:03+02	2020-10-10 12:13:10+02	devel	libpci	\N	libpci-3.7.0_1
2020-10-10 12:13:11+02	2020-10-10 12:13:25+02	www	p5-CGI	\N	p5-CGI-4.51
2020-10-10 12:13:06+02	2020-10-10 12:13:51+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-10-10 12:13:25+02	2020-10-10 12:15:15+02	x11-toolkits	vte3	\N	vte3-0.62.1
2020-10-10 12:12:46+02	2020-10-10 12:16:36+02	lang	php74	\N	php74-7.4.11
2020-10-10 12:15:16+02	2020-10-10 12:16:40+02	devel	git	\N	git-2.28.0
2020-10-10 12:13:51+02	2020-10-10 12:16:49+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.15.0
2020-10-10 12:16:37+02	2020-10-10 12:16:58+02	www	php74-session	\N	php74-session-7.4.11
2020-10-10 12:16:40+02	2020-10-10 12:17:00+02	archivers	php74-zlib	\N	php74-zlib-7.4.11
2020-10-10 12:16:49+02	2020-10-10 12:17:10+02	databases	php74-pdo	\N	php74-pdo-7.4.11
2020-10-10 12:16:58+02	2020-10-10 12:17:19+02	devel	php74-json	\N	php74-json-7.4.11
2020-10-10 12:17:00+02	2020-10-10 12:17:34+02	converters	php74-mbstring	\N	php74-mbstring-7.4.11
2020-10-10 12:17:11+02	2020-10-10 12:17:38+02	graphics	php74-gd	\N	php74-gd-7.4.11
2020-10-10 12:17:19+02	2020-10-10 12:17:45+02	security	php74-openssl	\N	php74-openssl-7.4.11
2020-10-15 21:54:54+02	2020-10-15 21:55:33+02	net	vde2	\N	vde2-2.3.2_4
2020-10-10 12:17:35+02	2020-10-10 12:17:54+02	textproc	php74-xml	\N	php74-xml-7.4.11
2020-10-10 12:17:38+02	2020-10-10 12:17:58+02	converters	php74-iconv	\N	php74-iconv-7.4.11
2020-10-10 12:17:45+02	2020-10-10 12:18:05+02	databases	pecl-memcache	php74	php74-pecl-memcache-4.0.5.2
2020-10-10 12:17:54+02	2020-10-10 12:18:16+02	databases	pecl-memcached	php74	php74-pecl-memcached-3.1.5
2020-10-10 12:17:58+02	2020-10-10 12:18:19+02	databases	php74-pdo_pgsql	\N	php74-pdo_pgsql-7.4.11
2020-10-10 12:18:05+02	2020-10-10 12:18:24+02	databases	php74-pdo_sqlite	\N	php74-pdo_sqlite-7.4.11
2020-10-10 12:18:19+02	2020-10-10 12:18:39+02	databases	php74-sqlite3	\N	php74-sqlite3-7.4.11
2020-10-10 12:18:17+02	2020-10-10 12:18:39+02	databases	php74-pgsql	\N	php74-pgsql-7.4.11
2020-10-10 12:18:25+02	2020-10-10 12:18:44+02	devel	php74-gettext	\N	php74-gettext-7.4.11
2020-10-10 12:18:39+02	2020-10-10 12:19:00+02	ftp	php74-curl	\N	php74-curl-7.4.11
2020-10-10 12:18:44+02	2020-10-10 12:19:01+02	math	pecl-bitset	php74	php74-pecl-bitset-3.0.1
2020-10-10 12:19:00+02	2020-10-10 12:19:22+02	math	php74-bcmath	\N	php74-bcmath-7.4.11
2020-10-10 12:19:02+02	2020-10-10 12:19:25+02	net	php74-ldap	\N	php74-ldap-7.4.11
2020-10-10 12:18:40+02	2020-10-10 12:19:26+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-10-10 12:19:26+02	2020-10-10 12:19:45+02	security	pecl-mcrypt	php74	php74-pecl-mcrypt-1.0.3
2020-10-10 12:19:26+02	2020-10-10 12:19:47+02	net	php74-sockets	\N	php74-sockets-7.4.11
2020-10-10 12:19:23+02	2020-10-10 12:19:48+02	net	php74-soap	\N	php74-soap-7.4.11
2020-10-10 12:19:45+02	2020-10-10 12:20:05+02	sysutils	php74-posix	\N	php74-posix-7.4.11
2020-10-10 12:19:47+02	2020-10-10 12:20:06+02	textproc	php74-ctype	\N	php74-ctype-7.4.11
2020-10-10 12:19:49+02	2020-10-10 12:20:15+02	textproc	php74-dom	\N	php74-dom-7.4.11
2020-10-10 12:20:05+02	2020-10-10 12:20:29+02	textproc	php74-simplexml	\N	php74-simplexml-7.4.11
2020-10-10 12:20:06+02	2020-10-10 12:20:44+02	www	php74-opcache	\N	php74-opcache-7.4.11
2020-10-10 12:20:15+02	2020-10-10 12:21:03+02	net	remmina	\N	remmina-1.3.9
2020-10-10 12:20:45+02	2020-10-10 12:22:00+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-10 12:20:29+02	2020-10-10 12:22:05+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.15.0
2020-10-10 12:21:03+02	2020-10-10 12:22:49+02	graphics	netpbm	\N	netpbm-10.91.01
2020-10-10 12:22:06+02	2020-10-10 12:22:55+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.15.0
2020-10-10 12:22:49+02	2020-10-10 12:24:52+02	graphics	gegl	\N	gegl-0.4.26_5
2020-10-10 12:24:53+02	2020-10-10 12:25:28+02	graphics	simage	\N	simage-1.7.0_8
2020-10-10 12:25:29+02	2020-10-10 12:26:30+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-10 12:22:55+02	2020-10-10 12:27:40+02	graphics	opencv	\N	opencv-3.4.1_36
2020-10-10 12:27:40+02	2020-10-10 12:28:44+02	devel	git-lfs	\N	git-lfs-2.12.0
2020-10-10 12:28:44+02	2020-10-10 12:40:03+02	graphics	Coin	\N	Coin-4.0.0_1
2020-10-10 12:22:00+02	2020-10-10 12:40:10+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-10 12:40:03+02	2020-10-10 12:42:01+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-10-10 12:42:02+02	2020-10-10 12:42:52+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.15.0
2020-10-10 12:42:52+02	2020-10-10 12:43:53+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-10 12:43:53+02	2020-10-10 12:45:02+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-10 12:45:03+02	2020-10-10 12:45:24+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.10
2020-10-10 12:45:25+02	2020-10-10 12:46:22+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.15.0
2020-10-10 12:46:22+02	2020-10-10 12:47:49+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.15.0
2020-10-10 12:47:49+02	2020-10-10 12:48:55+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.15.0
2020-10-10 12:40:11+02	2020-10-10 12:49:54+02	graphics	gimp-app	\N	gimp-app-2.10.20_6,1
2020-10-10 12:48:56+02	2020-10-10 12:50:13+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-10 12:49:55+02	2020-10-10 12:51:00+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.5
2020-10-10 12:51:00+02	2020-10-10 12:55:48+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-10-10 12:55:50+02	2020-10-10 12:58:08+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-10-10 12:58:11+02	2020-10-10 12:58:27+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20201010_1
2020-10-10 12:58:29+02	2020-10-10 12:59:22+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-10 12:59:25+02	2020-10-10 13:00:17+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-10 13:00:19+02	2020-10-10 13:01:04+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-10 13:00:40+02	2020-10-10 13:01:30+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-10 13:01:06+02	2020-10-10 13:01:51+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-10 13:01:32+02	2020-10-10 13:02:16+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-10 12:26:31+02	2020-10-10 13:02:28+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-10 13:01:52+02	2020-10-10 13:02:45+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-10 13:02:17+02	2020-10-10 13:03:26+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-10 13:02:45+02	2020-10-10 13:03:30+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-10 13:02:30+02	2020-10-10 13:03:32+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-10 13:03:30+02	2020-10-10 13:03:50+02	print	transfig	\N	transfig-3.2.5e_4
2020-10-10 13:03:51+02	2020-10-10 13:04:04+02	www	p5-libwww	\N	p5-libwww-6.49
2020-10-10 13:03:27+02	2020-10-10 13:04:20+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-10 13:03:33+02	2020-10-10 13:04:32+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-10 13:04:05+02	2020-10-10 13:04:56+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.15.0
2020-10-10 13:04:20+02	2020-10-10 13:05:19+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.15.0
2020-10-10 13:04:57+02	2020-10-10 13:05:52+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-10-10 13:04:32+02	2020-10-10 13:05:55+02	x11	mate-terminal	\N	mate-terminal-1.24.1
2020-10-10 13:05:19+02	2020-10-10 13:07:02+02	cad	cura	\N	Cura-4.7.1,2
2020-10-10 13:05:52+02	2020-10-10 13:08:01+02	deskutils	calibre	\N	calibre-4.23.0_2
2020-10-10 13:08:02+02	2020-10-10 13:08:34+02	devel	tig	\N	tig-2.5.1
2020-10-10 13:08:34+02	2020-10-10 13:09:40+02	devel	tortoisehg	py37	tortoisehg-5.5.1_2
2020-10-10 13:09:40+02	2020-10-10 13:32:32+02	emulators	qemu	\N	qemu-4.2.1
2020-10-10 13:32:33+02	2020-10-10 13:33:48+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-10 13:05:55+02	2020-10-10 13:35:11+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-10 13:33:48+02	2020-10-10 13:36:48+02	graphics	inkscape	\N	inkscape-1.0.1
2020-10-10 13:35:11+02	2020-10-10 13:37:21+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-10-10 13:07:03+02	2020-10-10 13:37:44+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-10 13:36:49+02	2020-10-10 13:38:18+02	multimedia	obs-studio	\N	obs-studio-26.0.0_1
2020-10-10 13:37:22+02	2020-10-10 13:39:32+02	multimedia	openshot	py37	openshot-2.5.1.369_1
2020-10-10 13:38:19+02	2020-10-10 13:39:36+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-10 13:37:50+02	2020-10-10 13:40:02+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-10 13:39:32+02	2020-10-10 13:40:12+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-10 13:39:37+02	2020-10-10 13:40:17+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-10 13:40:03+02	2020-10-10 13:40:40+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-10 13:40:12+02	2020-10-10 13:40:49+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-10 13:40:17+02	2020-10-10 13:40:55+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-10 13:40:41+02	2020-10-10 13:41:18+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-10 13:40:49+02	2020-10-10 13:41:27+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-10 13:40:55+02	2020-10-10 13:41:53+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-10 13:41:19+02	2020-10-10 13:42:00+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-10 13:41:27+02	2020-10-10 13:42:07+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-10 13:41:54+02	2020-10-10 13:42:07+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-10-10 13:42:07+02	2020-10-10 13:42:19+02	sysutils	flashrom	\N	flashrom-1.2
2020-10-10 13:42:01+02	2020-10-10 13:42:25+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-10-10 13:42:19+02	2020-10-10 13:42:31+02	sysutils	x86info	\N	x86info-1.31.s03
2020-10-10 13:42:25+02	2020-10-10 13:42:44+02	www	dokuwiki	\N	dokuwiki-20200729
2020-10-10 13:42:07+02	2020-10-10 13:42:51+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-10 13:42:51+02	2020-10-10 13:44:03+02	x11	mate	\N	mate-1.24.1
2020-10-10 13:44:04+02	2020-10-10 13:46:03+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-10-10 13:42:44+02	2020-10-10 13:46:35+02	www	mod_php74	\N	mod_php74-7.4.11
2020-10-10 13:46:36+02	2020-10-10 13:47:21+02	graphics	xfig	\N	xfig-3.2.7a
2020-10-10 13:46:03+02	2020-10-10 14:09:39+02	editors	vscode	\N	vscode-1.46.1
2020-10-10 13:42:31+02	2020-10-10 14:15:44+02	www	firefox	\N	firefox-81.0.1_2,2
2020-10-10 14:15:46+02	2020-10-10 14:31:27+02	local	E590T-localbase	\N	E590T-localbase-2020100801
2020-10-10 14:43:22+02	2020-10-10 14:44:26+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-10 14:43:22+02	2020-10-10 14:49:35+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-10 14:49:35+02	2020-10-10 14:49:43+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-10 15:58:17+02	2020-10-10 15:59:39+02	math	geogebra	\N	geogebra-5.0.580.0
2020-10-10 15:59:40+02	2020-10-10 15:59:53+02	math	hexcalc	\N	hexcalc-1.11_3
2020-10-10 15:58:17+02	2020-10-10 16:02:56+02	math	fricas	\N	fricas-1.3.6
2020-10-10 15:59:54+02	2020-10-10 16:07:07+02	math	wxmaxima	\N	wxmaxima-20.04.0_2
2020-10-10 16:07:08+02	2020-10-10 16:27:04+02	local	E590T-localbase	\N	E590T-localbase-2020101000
2020-10-10 16:39:26+02	2020-10-10 16:40:24+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-10 16:39:26+02	2020-10-10 16:41:35+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-10 16:41:35+02	2020-10-10 16:41:43+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-12 22:02:16+02	2020-10-12 22:09:14+02	ports-mgmt	pkg	\N	pkg-1.15.10
2020-10-13 07:11:46+02	2020-10-13 07:15:16+02	ports-mgmt	pkg	\N	pkg-1.15.10
2020-10-13 07:30:42+02	2020-10-13 07:30:55+02	dns	public_suffix_list	\N	public_suffix_list-20200930
2020-10-13 07:30:42+02	2020-10-13 07:31:06+02	dns	py-idna	py37	py37-idna-2.10
2020-10-13 07:31:08+02	2020-10-13 07:31:36+02	net	py-urllib3	py37	py37-urllib3-1.25.10,1
2020-10-13 07:31:37+02	2020-10-13 07:32:00+02	www	py-requests	py37	py37-requests-2.22.0_2
2020-10-13 07:30:56+02	2020-10-13 07:32:28+02	dns	libpsl	\N	libpsl-0.21.1
2020-10-13 07:32:03+02	2020-10-13 07:33:03+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-10-13 07:32:30+02	2020-10-13 07:35:53+02	devel	libsoup	\N	libsoup-2.72.0
2020-10-13 07:35:55+02	2020-10-13 07:36:46+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-10-13 07:36:48+02	2020-10-13 07:37:55+02	textproc	py-rdflib	py37	py37-rdflib-5.0.0
2020-10-13 07:37:57+02	2020-10-13 07:39:02+02	audio	lv2	\N	lv2-1.18.0_1
2020-10-13 07:39:06+02	2020-10-13 07:39:54+02	databases	py-sqlparse	py37	py37-sqlparse-0.4.0
2020-10-13 07:39:57+02	2020-10-13 07:42:09+02	www	py-django22	py37	py37-django22-2.2.16
2020-10-13 07:42:11+02	2020-10-13 07:45:59+02	devel	librest	\N	rest-0.7.93_1
2020-10-13 07:30:42+02	2020-10-13 07:47:11+02	graphics	mesa-libs	\N	mesa-libs-20.2.0
2020-10-13 07:46:02+02	2020-10-13 07:48:13+02	multimedia	libmtp	\N	libmtp-1.1.18
2020-10-13 07:48:14+02	2020-10-13 07:49:44+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-10-13 07:49:46+02	2020-10-13 07:52:55+02	graphics	libGLU	\N	libGLU-9.0.1
2020-10-13 07:47:14+02	2020-10-13 07:54:33+02	graphics	cairo	\N	cairo-1.16.0_1,3
2020-10-13 07:54:39+02	2020-10-13 08:02:22+02	x11-toolkits	pango	\N	pango-1.42.4_5
2020-10-13 08:02:25+02	2020-10-13 08:04:03+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1,1
2020-10-13 08:04:06+02	2020-10-13 08:08:58+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-10-13 08:09:01+02	2020-10-13 08:15:20+02	graphics	graphene	\N	graphene-1.10.2
2020-10-13 07:33:04+02	2020-10-13 08:20:34+02	devel	cmake	\N	cmake-3.18.4
2020-10-13 08:20:35+02	2020-10-13 08:21:40+02	graphics	jpeg-turbo	\N	jpeg-turbo-2.0.5
2020-10-13 08:15:22+02	2020-10-13 08:25:27+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2_1
2020-10-13 08:21:41+02	2020-10-13 08:26:10+02	graphics	tiff	\N	tiff-4.1.0
2020-10-13 07:52:56+02	2020-10-13 08:29:53+02	graphics	mesa-dri	\N	mesa-dri-20.2.0
2020-10-13 08:26:13+02	2020-10-13 08:32:26+02	graphics	webp	\N	webp-1.1.0
2020-10-13 08:29:57+02	2020-10-13 08:32:37+02	graphics	lcms2	\N	lcms2-2.11_1
2020-10-13 08:32:37+02	2020-10-13 08:33:56+02	graphics	openjpeg	\N	openjpeg-2.3.1
2020-10-13 08:33:56+02	2020-10-13 08:34:45+02	graphics	gd	\N	libgd-2.3.0,1
2020-10-13 08:32:28+02	2020-10-13 08:35:29+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_10
2020-10-13 08:34:45+02	2020-10-13 08:35:33+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-10-13 08:35:34+02	2020-10-13 08:36:05+02	graphics	openjpeg15	\N	openjpeg15-1.5.2_1
2020-10-13 08:36:05+02	2020-10-13 08:37:24+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-10-13 08:35:29+02	2020-10-13 08:38:02+02	graphics	poppler	\N	poppler-20.10.0
2020-10-13 08:37:25+02	2020-10-13 08:38:27+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-10-13 08:38:27+02	2020-10-13 08:38:50+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-10-13 08:38:50+02	2020-10-13 08:39:37+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-10-13 08:39:38+02	2020-10-13 08:40:31+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-10-13 08:25:29+02	2020-10-13 08:40:43+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0_1
2020-10-13 08:40:44+02	2020-10-13 08:42:12+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.38.0
2020-10-13 08:38:02+02	2020-10-13 08:43:01+02	print	texlive-base	\N	texlive-base-20150521_56
2020-10-13 08:40:32+02	2020-10-13 08:43:20+02	graphics	argyllcms	\N	argyllcms-1.9.2_5
2020-10-13 08:43:20+02	2020-10-13 08:49:21+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-10-13 08:49:21+02	2020-10-13 08:49:45+02	devel	libnotify	\N	libnotify-0.7.9_1
2020-10-13 08:42:12+02	2020-10-13 08:50:01+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-10-13 08:49:46+02	2020-10-13 08:50:36+02	multimedia	libv4l	\N	libv4l-1.18.0
2020-10-13 08:50:37+02	2020-10-13 08:51:41+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-10-13 08:51:42+02	2020-10-13 08:54:46+02	graphics	graphviz	\N	graphviz-2.44.1_3
2020-10-13 08:54:47+02	2020-10-13 08:56:22+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-10-13 08:56:22+02	2020-10-13 08:56:59+02	graphics	frei0r	\N	frei0r-1.6.1
2020-10-13 08:57:00+02	2020-10-13 09:02:15+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-10-13 09:02:16+02	2020-10-13 09:03:08+02	net	geoclue	\N	geoclue-2.5.5
2020-10-13 09:03:09+02	2020-10-13 09:03:25+02	www	libwpe	\N	libwpe-1.4.0.1
2020-10-13 08:43:02+02	2020-10-13 09:03:32+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-10-13 09:03:26+02	2020-10-13 09:03:48+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-10-13 09:03:33+02	2020-10-13 09:06:20+02	print	tex-formats	\N	tex-formats-20150521_2
2020-10-13 08:50:01+02	2020-10-13 09:06:25+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-10-13 09:06:25+02	2020-10-13 09:06:55+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-10-13 09:06:55+02	2020-10-13 09:07:08+02	graphics	jasper	\N	jasper-2.0.22
2020-10-13 09:07:08+02	2020-10-13 09:07:58+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-10-13 09:07:58+02	2020-10-13 09:08:34+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-10-13 09:03:48+02	2020-10-13 09:08:59+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-10-13 09:08:34+02	2020-10-13 09:09:08+02	graphics	libwmf	\N	libwmf-0.2.8.4_15
2020-10-13 09:09:09+02	2020-10-13 09:10:32+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-10-13 09:06:21+02	2020-10-13 09:10:34+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-10-13 09:10:33+02	2020-10-13 09:11:28+02	graphics	libraw	\N	libraw-0.19.5
2020-10-13 09:11:29+02	2020-10-13 09:11:53+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-10-13 09:11:53+02	2020-10-13 09:12:34+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-10-13 09:08:59+02	2020-10-13 09:13:58+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-10-13 09:13:59+02	2020-10-13 09:14:27+02	graphics	lcms	\N	lcms-1.19_6,1
2020-10-13 09:12:35+02	2020-10-13 09:14:59+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-10-13 09:14:28+02	2020-10-13 09:15:09+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-10-13 09:10:35+02	2020-10-13 09:15:22+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-10-13 09:15:00+02	2020-10-13 09:15:33+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-10-13 09:15:33+02	2020-10-13 09:15:58+02	graphics	libmng	\N	libmng-1.0.10_3
2020-10-13 09:15:09+02	2020-10-13 09:15:58+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.5
2020-10-13 09:15:58+02	2020-10-13 09:16:26+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-10-13 09:15:58+02	2020-10-13 09:16:46+02	devel	py-qt5-core	py37	py37-qt5-core-5.15.0
2020-10-13 09:16:27+02	2020-10-13 09:16:46+02	graphics	php74-gd	\N	php74-gd-7.4.11
2020-10-13 09:15:23+02	2020-10-13 09:16:49+02	textproc	docbook-utils	\N	docbook-utils-0.6.14_13
2020-10-13 09:16:47+02	2020-10-13 09:17:42+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-10-13 09:17:42+02	2020-10-13 09:18:33+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-10-13 09:16:50+02	2020-10-13 09:19:09+02	graphics	colord	\N	colord-1.3.5_1
2020-10-13 09:18:33+02	2020-10-13 09:19:23+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-10-13 09:16:46+02	2020-10-13 09:19:35+02	x11-servers	xorg-server	\N	xorg-server-1.20.9,1
2020-10-13 09:19:23+02	2020-10-13 09:20:01+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-10-13 09:19:35+02	2020-10-13 09:20:20+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-10-13 09:20:21+02	2020-10-13 09:21:37+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-13 09:21:38+02	2020-10-13 09:23:01+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-10-13 09:20:02+02	2020-10-13 09:23:24+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-10-13 09:23:02+02	2020-10-13 09:23:43+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-10-13 09:19:09+02	2020-10-13 09:23:49+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-10-13 09:23:43+02	2020-10-13 09:24:46+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-10-13 09:24:47+02	2020-10-13 09:26:26+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-10-13 09:26:26+02	2020-10-13 09:27:44+02	x11	mate-desktop	\N	mate-desktop-1.24.1
2020-10-13 09:27:45+02	2020-10-13 09:29:33+02	security	gcr	\N	gcr-3.28.0_1
2020-10-13 09:29:34+02	2020-10-13 09:30:43+02	x11-toolkits	vte3	\N	vte3-0.62.1
2020-10-13 09:30:44+02	2020-10-13 09:36:34+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-10-13 09:36:35+02	2020-10-13 09:37:40+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-10-13 09:37:41+02	2020-10-13 09:38:30+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-10-13 09:38:30+02	2020-10-13 09:40:31+02	devel	sdl20	\N	sdl2-2.0.12_3
2020-10-13 09:40:32+02	2020-10-13 09:41:26+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-10-13 09:41:26+02	2020-10-13 09:42:03+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-10-13 09:42:03+02	2020-10-13 09:42:51+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-10-13 09:42:52+02	2020-10-13 09:44:27+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-10-13 09:44:28+02	2020-10-13 09:45:21+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-10-13 09:45:21+02	2020-10-13 09:46:13+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-10-13 09:23:25+02	2020-10-13 09:47:19+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.15.0
2020-10-13 09:46:13+02	2020-10-13 09:49:42+02	graphics	glew	\N	glew-2.2.0
2020-10-13 09:49:42+02	2020-10-13 09:50:25+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-10-13 09:47:20+02	2020-10-13 09:50:29+02	graphics	poppler-glib	\N	poppler-glib-20.10.0
2020-10-13 09:50:25+02	2020-10-13 09:50:57+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-10-13 09:50:29+02	2020-10-13 09:51:34+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-10-13 09:50:57+02	2020-10-13 09:51:51+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-10-13 09:51:51+02	2020-10-13 09:52:50+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-10-13 09:51:35+02	2020-10-13 09:52:52+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-10-13 09:52:51+02	2020-10-13 09:54:08+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-10-13 09:52:53+02	2020-10-13 09:54:10+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-10-13 09:54:08+02	2020-10-13 09:54:27+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-10-13 09:54:10+02	2020-10-13 09:55:02+02	net	py-qt5-network	py37	py37-qt5-network-5.15.0
2020-10-13 09:54:27+02	2020-10-13 09:55:23+02	net	remmina	\N	remmina-1.3.9
2020-10-13 09:55:02+02	2020-10-13 09:55:55+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-10-13 09:55:56+02	2020-10-13 09:56:41+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-10-13 09:55:24+02	2020-10-13 09:57:12+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.15.0
2020-10-13 09:57:12+02	2020-10-13 09:57:24+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-10-13 09:57:25+02	2020-10-13 09:58:29+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-13 09:58:29+02	2020-10-13 09:59:25+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-10-13 09:56:41+02	2020-10-13 09:59:43+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-10-13 09:59:26+02	2020-10-13 10:00:14+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0_1
2020-10-13 09:59:44+02	2020-10-13 10:00:30+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-10-13 10:00:14+02	2020-10-13 10:01:08+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-10-13 10:01:08+02	2020-10-13 10:01:57+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-10-13 10:00:30+02	2020-10-13 10:02:43+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-10-13 10:01:58+02	2020-10-13 10:02:54+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.5.0
2020-10-13 10:02:44+02	2020-10-13 10:03:49+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-10-13 10:02:54+02	2020-10-13 10:03:49+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-10-13 10:03:50+02	2020-10-13 10:04:47+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-10-13 10:04:48+02	2020-10-13 10:05:46+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-10-13 10:05:46+02	2020-10-13 10:06:09+02	print	gl2ps	\N	gl2ps-1.4.0
2020-10-13 10:06:10+02	2020-10-13 10:06:38+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-10-13 10:06:38+02	2020-10-13 10:07:27+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-13 10:07:27+02	2020-10-13 10:10:36+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.9,1
2020-10-13 10:03:50+02	2020-10-13 10:10:42+02	math	vtk8	\N	vtk8-8.2.0
2020-10-13 10:10:37+02	2020-10-13 10:11:39+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-10-13 10:10:43+02	2020-10-13 10:12:49+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-10-13 10:11:40+02	2020-10-13 10:13:05+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-10-13 10:13:05+02	2020-10-13 10:14:12+02	devel	libpeas	\N	libpeas-1.22.0
2020-10-13 10:14:13+02	2020-10-13 10:14:58+02	graphics	netpbm	\N	netpbm-10.91.01
2020-10-13 10:14:59+02	2020-10-13 10:19:47+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-13 10:12:49+02	2020-10-13 10:20:39+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-13 10:19:48+02	2020-10-13 10:20:47+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-10-13 10:20:40+02	2020-10-13 10:22:11+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-10-13 10:22:12+02	2020-10-13 10:25:24+02	x11	xorg	\N	xorg-7.7_3
2020-10-13 10:25:24+02	2020-10-13 10:26:44+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-10-13 10:26:44+02	2020-10-13 10:27:52+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-10-13 10:27:53+02	2020-10-13 10:28:27+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-10-13 10:28:28+02	2020-10-13 10:30:08+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-13 10:30:08+02	2020-10-13 10:31:09+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-10-13 10:31:10+02	2020-10-13 10:34:13+02	graphics	cogl	\N	cogl-1.22.8
2020-10-13 10:34:14+02	2020-10-13 10:38:56+02	math	py-matplotlib	py37	py37-matplotlib-3.3.1
2020-10-13 09:23:50+02	2020-10-13 10:39:35+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-13 10:38:56+02	2020-10-13 10:40:06+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-10-13 10:40:07+02	2020-10-13 10:42:47+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-13 10:42:48+02	2020-10-13 10:45:51+02	devel	libgdata	\N	libgdata-0.17.13
2020-10-13 10:39:36+02	2020-10-13 10:46:09+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-13 10:20:48+02	2020-10-13 10:46:21+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-10-13 10:45:51+02	2020-10-13 10:46:33+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.15.0
2020-10-13 10:46:33+02	2020-10-13 10:47:17+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-10-13 10:46:09+02	2020-10-13 10:47:38+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-13 10:47:39+02	2020-10-13 10:48:26+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-10-13 10:47:17+02	2020-10-13 10:49:35+02	x11	mate-panel	\N	mate-panel-1.24.1_1
2020-10-13 10:48:26+02	2020-10-13 10:49:42+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-10-13 10:49:36+02	2020-10-13 10:50:07+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-10-13 10:50:08+02	2020-10-13 10:50:35+02	audio	chromaprint	\N	chromaprint-1.5.0
2020-10-13 10:50:36+02	2020-10-13 10:53:44+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-10-13 10:49:43+02	2020-10-13 10:55:37+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-13 10:53:47+02	2020-10-13 10:56:31+02	graphics	clutter	\N	clutter-1.26.2
2020-10-13 10:56:31+02	2020-10-13 11:02:08+02	graphics	leptonica	\N	leptonica-1.76.0_1
2020-10-13 10:46:22+02	2020-10-13 11:04:22+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-13 11:02:09+02	2020-10-13 11:05:50+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-10-13 11:04:24+02	2020-10-13 11:05:55+02	graphics	libgphoto2	\N	libgphoto2-2.5.23_1
2020-10-13 11:05:56+02	2020-10-13 11:06:25+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-10-13 11:05:51+02	2020-10-13 11:09:06+02	graphics	poppler-utils	\N	poppler-utils-20.10.0
2020-10-13 11:09:06+02	2020-10-13 11:09:28+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-10-13 11:09:28+02	2020-10-13 11:10:08+02	multimedia	totem-pl-parser	\N	totem-pl-parser-3.26.5
2020-10-13 11:10:08+02	2020-10-13 11:11:12+02	net	geocode-glib	\N	geocode-glib-3.26.2
2020-10-13 11:11:13+02	2020-10-13 11:12:22+02	net	libnfs	\N	libnfs-4.0.0_1
2020-10-13 11:12:23+02	2020-10-13 11:13:17+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-13 11:13:18+02	2020-10-13 11:14:11+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-10-13 11:14:12+02	2020-10-13 11:14:31+02	sysutils	fusefs-libs3	\N	fusefs-libs3-3.10.0
2020-10-13 10:55:38+02	2020-10-13 11:15:25+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-13 11:15:26+02	2020-10-13 11:16:11+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-10-13 11:16:12+02	2020-10-13 11:17:17+02	x11	libmatekbd	\N	libmatekbd-1.24.1
2020-10-13 11:17:18+02	2020-10-13 11:19:03+02	x11-fm	caja	\N	caja-1.24.0
2020-10-13 11:06:26+02	2020-10-13 11:19:22+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-10-13 11:14:32+02	2020-10-13 11:19:29+02	sysutils	libudisks	\N	libudisks-2.9.1
2020-10-13 11:19:29+02	2020-10-13 11:20:19+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-13 11:19:22+02	2020-10-13 11:20:27+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-10-13 11:20:28+02	2020-10-13 11:21:18+02	audio	gstreamer1-plugins-chromaprint	\N	gstreamer1-plugins-chromaprint-1.16.2
2020-10-13 11:20:20+02	2020-10-13 11:21:32+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-10-13 11:19:04+02	2020-10-13 11:21:33+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-10-13 11:21:19+02	2020-10-13 11:21:38+02	databases	gom	\N	gom-0.4
2020-10-13 11:21:32+02	2020-10-13 11:21:57+02	devel	dee	\N	dee-1.2.7_17
2020-10-13 11:21:39+02	2020-10-13 11:22:31+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-10-13 11:21:58+02	2020-10-13 11:23:06+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-10-13 11:21:34+02	2020-10-13 11:23:33+02	devel	gvfs	\N	gvfs-1.46.1
2020-10-13 11:23:33+02	2020-10-13 11:24:00+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-10-13 11:22:31+02	2020-10-13 11:24:18+02	devel	m17n-lib	\N	m17n-lib-1.8.0_2
2020-10-13 11:23:07+02	2020-10-13 11:24:22+02	graphics	libcdr01	\N	libcdr01-0.1.6
2020-10-13 11:24:18+02	2020-10-13 11:25:09+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.15.0
2020-10-13 11:24:22+02	2020-10-13 11:25:48+02	math	octomap	\N	octomap-1.9.5.190
2020-10-13 11:25:10+02	2020-10-13 11:26:14+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-10-13 11:25:48+02	2020-10-13 11:26:21+02	multimedia	libmediaart	\N	libmediaart-1.9.4
2020-10-13 11:26:15+02	2020-10-13 11:26:39+02	net	grilo	\N	grilo-0.3.13
2020-10-13 11:26:40+02	2020-10-13 11:26:52+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-10-13 11:26:21+02	2020-10-13 11:27:37+02	net	libgweather	\N	libgweather-3.36.1
2020-10-13 11:24:00+02	2020-10-13 11:29:21+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-10-13 11:27:38+02	2020-10-13 11:29:25+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-13 11:26:52+02	2020-10-13 11:29:52+02	print	qpdf	\N	qpdf-10.0.1
2020-10-13 11:29:52+02	2020-10-13 11:30:14+02	x11	babl	\N	babl-0.1.78_1
2020-10-13 11:29:25+02	2020-10-13 11:30:20+02	sysutils	tracker	\N	tracker-2.3.4
2020-10-13 11:29:22+02	2020-10-13 11:30:35+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-10-13 11:30:21+02	2020-10-13 11:31:06+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-10-13 11:30:15+02	2020-10-13 11:31:19+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-10-13 11:30:35+02	2020-10-13 11:31:27+02	x11	zenity	\N	zenity-3.32.0
2020-10-13 11:31:06+02	2020-10-13 11:31:51+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-10-13 11:31:51+02	2020-10-13 11:32:35+02	devel	gstreamer1-plugins-soup	\N	gstreamer1-plugins-soup-1.16.2
2020-10-13 11:31:20+02	2020-10-13 11:34:21+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-10-13 11:34:21+02	2020-10-13 11:35:13+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-10-13 11:35:14+02	2020-10-13 11:36:25+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-13 11:32:35+02	2020-10-13 11:36:35+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-10-13 11:36:35+02	2020-10-13 11:37:26+02	graphics	clutter-gtk3	\N	clutter-gtk3-1.8.4
2020-10-13 11:31:27+02	2020-10-13 11:37:34+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-10-13 11:37:26+02	2020-10-13 11:37:43+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-10-13 11:37:43+02	2020-10-13 11:38:36+02	graphics	glfw	\N	glfw-3.3.2
2020-10-13 11:38:36+02	2020-10-13 11:39:29+02	graphics	libfreehand	\N	libfreehand-0.1.2_18
2020-10-13 11:37:35+02	2020-10-13 11:39:32+02	graphics	gegl	\N	gegl-0.4.26_6
2020-10-13 11:36:26+02	2020-10-13 11:40:20+02	editors	emacs	full	emacs-27.1_1,3
2020-10-13 11:40:21+02	2020-10-13 11:41:11+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-10-13 11:39:30+02	2020-10-13 11:41:36+02	graphics	movit	\N	movit-1.6.2_2
2020-10-13 11:41:11+02	2020-10-13 11:41:38+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-10-13 11:41:36+02	2020-10-13 11:41:59+02	graphics	simage	\N	simage-1.7.0_8
2020-10-13 11:39:32+02	2020-10-13 11:43:08+02	graphics	opencv	\N	opencv-3.4.1_36
2020-10-13 11:41:59+02	2020-10-13 11:43:12+02	math	fcl05	\N	fcl05-0.5.0
2020-10-13 11:43:08+02	2020-10-13 11:43:48+02	multimedia	clutter-gst3	\N	clutter-gst3-3.0.27
2020-10-13 11:43:12+02	2020-10-13 11:44:13+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-13 11:44:07+02	2020-10-13 11:44:25+02	net	libvncserver	\N	libvncserver-0.9.12_2
2020-10-13 11:44:14+02	2020-10-13 11:44:54+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-13 11:44:25+02	2020-10-13 11:45:03+02	print	cups-filters	\N	cups-filters-1.28.4
2020-10-13 11:41:38+02	2020-10-13 11:47:23+02	lang	sbcl	\N	sbcl-2.0.9_1,1
2020-10-13 11:44:55+02	2020-10-13 11:47:50+02	print	fontforge	\N	fontforge-20190801_2
2020-10-13 11:47:23+02	2020-10-13 11:48:22+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0_1
2020-10-13 11:47:50+02	2020-10-13 11:49:02+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-13 11:49:02+02	2020-10-13 11:49:14+02	textproc	py-numpydoc	py37	py37-numpydoc-0.9.1
2020-10-13 11:48:22+02	2020-10-13 11:49:19+02	sysutils	zeitgeist	\N	zeitgeist-1.0.2
2020-10-13 11:49:20+02	2020-10-13 11:49:30+02	www	py-hyperlink	py37	py37-hyperlink-20.0.1
2020-10-13 11:49:30+02	2020-10-13 11:50:18+02	x11	mate-session-manager	\N	mate-session-manager-1.24.1
2020-10-13 11:50:19+02	2020-10-13 11:51:21+02	x11	yelp	\N	yelp-3.38.0
2020-10-13 11:45:04+02	2020-10-13 11:51:50+02	print	tex-luatex	\N	tex-luatex-0.80.0_21
2020-10-13 11:51:22+02	2020-10-13 11:51:55+02	x11-themes	gnome-icon-theme-symbolic	\N	gnome-icon-theme-symbolic-3.12.0
2020-10-13 11:51:55+02	2020-10-13 11:52:38+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-10-13 11:51:51+02	2020-10-13 11:52:48+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-10-13 11:52:39+02	2020-10-13 11:54:14+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-10-13 11:54:15+02	2020-10-13 11:57:04+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-10-13 11:57:04+02	2020-10-13 11:58:14+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-10-13 11:58:15+02	2020-10-13 11:59:36+02	x11-wm	marco	\N	marco-1.24.1
2020-10-13 11:59:36+02	2020-10-13 12:00:36+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-10-13 12:00:38+02	2020-10-13 12:01:11+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-10-13 12:01:11+02	2020-10-13 12:03:08+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-13 12:03:08+02	2020-10-13 12:04:12+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-10-13 11:52:48+02	2020-10-13 12:04:23+02	x11-toolkits	open-motif	\N	open-motif-2.3.8_1
2020-10-13 12:04:24+02	2020-10-13 12:04:37+02	devel	py-colorlog	py37	py37-colorlog-4.3.0
2020-10-13 12:04:38+02	2020-10-13 12:05:41+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-10-13 12:05:41+02	2020-10-13 12:06:54+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-10-13 12:04:13+02	2020-10-13 12:06:54+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-13 12:06:55+02	2020-10-13 12:08:24+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-10-13 12:06:54+02	2020-10-13 12:08:50+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-10-13 12:08:26+02	2020-10-13 12:10:46+02	devel	shiboken2	py37	py37-shiboken2-5.15.1
2020-10-13 12:10:46+02	2020-10-13 12:14:47+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-10-13 12:08:50+02	2020-10-13 12:16:04+02	graphics	Coin	\N	Coin-4.0.0_1
2020-10-13 12:16:06+02	2020-10-13 12:17:03+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-10-13 12:17:05+02	2020-10-13 12:17:30+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-10-13 12:17:31+02	2020-10-13 12:19:13+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-10-13 11:49:15+02	2020-10-13 12:19:19+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-13 12:19:23+02	2020-10-13 12:20:32+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-10-13 12:20:32+02	2020-10-13 12:21:02+02	graphics	libgxps	\N	libgxps-0.3.1
2020-10-13 12:21:02+02	2020-10-13 12:21:56+02	graphics	libheif	\N	libheif-1.6.2_1
2020-10-13 12:21:57+02	2020-10-13 12:22:34+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-10-13 12:19:16+02	2020-10-13 12:27:26+02	graphics	gimp-app	\N	gimp-app-2.10.20_7,1
2020-10-13 12:22:34+02	2020-10-13 12:28:40+02	graphics	podofo	\N	podofo-0.9.6_1
2020-10-13 12:28:40+02	2020-10-13 12:29:46+02	graphics	py-PyOpenGL	py37	py37-PyOpenGL-3.1.5
2020-10-13 12:29:47+02	2020-10-13 12:30:44+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.15.0
2020-10-13 12:27:27+02	2020-10-13 12:33:25+02	graphics	poppler-qt5	\N	poppler-qt5-20.10.0
2020-10-13 12:30:45+02	2020-10-13 12:36:36+02	graphics	py-scikit-image	py37	py37-scikit-image-0.17.2
2020-10-13 12:33:26+02	2020-10-13 12:37:30+02	graphics	sane-backends	\N	sane-backends-1.0.31_1
2020-10-13 12:36:37+02	2020-10-13 12:37:50+02	graphics	vigra	\N	vigra-1.11.1_12
2020-10-13 12:14:50+02	2020-10-13 12:38:38+02	graphics	dcmtk	\N	dcmtk-3.6.5_2
2020-10-13 12:38:38+02	2020-10-13 12:39:48+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-10-13 12:37:31+02	2020-10-13 12:41:26+02	lang	python39	\N	python39-3.9.0
2020-10-13 12:41:26+02	2020-10-13 12:42:34+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-13 12:42:35+02	2020-10-13 12:44:50+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-13 12:44:51+02	2020-10-13 12:45:20+02	net	gupnp	\N	gupnp-1.2.4
2020-10-13 12:45:20+02	2020-10-13 12:46:08+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-13 12:39:48+02	2020-10-13 12:46:09+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-13 12:46:09+02	2020-10-13 12:46:52+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-13 12:46:08+02	2020-10-13 12:47:11+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-13 12:47:12+02	2020-10-13 12:47:34+02	print	libspectre	\N	libspectre-0.2.9
2020-10-13 12:47:35+02	2020-10-13 12:49:19+02	print	mftrace	\N	mftrace-1.2.18_2
2020-10-13 12:46:52+02	2020-10-13 12:49:48+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-13 12:37:50+02	2020-10-13 12:51:47+02	math	R	\N	R-4.0.3
2020-10-13 12:49:49+02	2020-10-13 12:52:06+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-13 12:51:48+02	2020-10-13 12:52:30+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-13 12:52:30+02	2020-10-13 12:53:04+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.15.0
2020-10-13 12:52:06+02	2020-10-13 12:53:25+02	textproc	dblatex	\N	dblatex-0.3.11
2020-10-13 12:53:25+02	2020-10-13 12:53:45+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-10-13 12:49:20+02	2020-10-13 12:53:53+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-10-13 12:53:04+02	2020-10-13 12:53:54+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.15.0
2020-10-13 12:53:54+02	2020-10-13 12:54:34+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-10-13 12:53:54+02	2020-10-13 12:54:49+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-10-13 12:53:45+02	2020-10-13 12:55:03+02	x11-servers	xephyr	\N	xephyr-1.20.9,1
2020-10-13 12:54:35+02	2020-10-13 12:58:12+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.15.0
2020-10-13 12:58:12+02	2020-10-13 12:59:16+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0_1
2020-10-13 12:54:50+02	2020-10-13 12:59:47+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-10-13 12:55:04+02	2020-10-13 13:00:31+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-10-13 12:59:17+02	2020-10-13 13:00:32+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-10-13 13:00:32+02	2020-10-13 13:00:47+02	audio	faudio	\N	FAudio-20.10
2020-10-13 13:00:47+02	2020-10-13 13:01:58+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-13 13:00:31+02	2020-10-13 13:02:35+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-13 13:01:58+02	2020-10-13 13:03:11+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-13 13:02:36+02	2020-10-13 13:03:41+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-10-13 12:59:47+02	2020-10-13 13:04:16+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-13 13:03:12+02	2020-10-13 13:04:18+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-13 13:04:17+02	2020-10-13 13:05:02+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-13 13:04:18+02	2020-10-13 13:05:04+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-13 13:05:03+02	2020-10-13 13:05:23+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-10-13 13:05:04+02	2020-10-13 13:05:24+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-10-13 13:03:41+02	2020-10-13 13:06:09+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-13 13:05:23+02	2020-10-13 13:06:14+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-10-13 13:06:09+02	2020-10-13 13:06:24+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-10-13 13:05:25+02	2020-10-13 13:06:34+02	devel	ode	\N	ode-0.13_4,1
2020-10-13 13:06:34+02	2020-10-13 13:06:46+02	devel	py-setuptools	py39	py39-setuptools-44.0.0
2020-10-13 13:06:24+02	2020-10-13 13:07:12+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.5
2020-10-13 13:06:14+02	2020-10-13 13:07:16+02	devel	py-game	py37	py37-game-1.9.6_1
2020-10-13 13:06:46+02	2020-10-13 13:07:43+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-13 13:07:16+02	2020-10-13 13:31:17+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-10-13 13:07:44+02	2020-10-13 13:32:38+02	editors	libreoffice	\N	libreoffice-7.0.2.2_2
2020-10-13 13:31:17+02	2020-10-13 13:35:43+02	editors	pluma	\N	pluma-1.24.1
2020-10-13 13:32:39+02	2020-10-13 13:35:46+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-10-13 13:35:43+02	2020-10-13 13:36:35+02	graphics	GraphicsMagick	\N	GraphicsMagick-1.3.35,1
2020-10-13 13:35:47+02	2020-10-13 13:38:25+02	graphics	atril	\N	atril-1.22.3_10
2020-10-13 13:36:35+02	2020-10-13 13:38:42+02	graphics	eom	\N	eom-1.24.1
2020-10-13 13:38:42+02	2020-10-13 13:39:05+02	graphics	gle	\N	gle-3.1.0_7
2020-10-13 13:39:06+02	2020-10-13 13:39:23+02	graphics	gltt	\N	gltt-2.5.2_14
2020-10-13 13:38:26+02	2020-10-13 13:39:55+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-10-13 13:39:23+02	2020-10-13 13:40:26+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-10-13 13:39:56+02	2020-10-13 13:41:12+02	graphics	openimageio	\N	openimageio-2.2.6.1
2020-10-13 13:41:12+02	2020-10-13 13:41:47+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-10-13 13:40:26+02	2020-10-13 13:44:03+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-10-13 13:41:47+02	2020-10-13 14:02:20+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-10-13 13:44:04+02	2020-10-13 14:13:22+02	java	openjdk11	\N	openjdk11-11.0.8+10.1
2020-10-13 14:11:58+02	2020-10-13 14:13:25+02	math	mate-calc	\N	mate-calc-1.24.1
2020-10-13 14:02:23+02	2020-10-13 14:14:55+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-13 14:13:23+02	2020-10-13 14:18:57+02	math	maxima	\N	maxima-5.44.0_5
2020-10-13 14:14:55+02	2020-10-13 14:19:40+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-10-13 14:18:57+02	2020-10-13 14:20:48+02	multimedia	mpv	\N	mpv-0.32.0_5,1
2020-10-13 14:20:48+02	2020-10-13 14:22:17+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-13 14:22:18+02	2020-10-13 14:23:09+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-13 14:13:26+02	2020-10-13 14:23:22+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-10-13 14:23:10+02	2020-10-13 14:23:50+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-13 14:19:41+02	2020-10-13 14:23:59+02	multimedia	vlc	\N	vlc-3.0.11_5,4
2020-10-13 14:23:22+02	2020-10-13 14:24:02+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-13 14:23:50+02	2020-10-13 14:24:30+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-13 14:23:59+02	2020-10-13 14:24:36+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-13 14:24:02+02	2020-10-13 14:24:39+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-13 14:24:31+02	2020-10-13 14:25:18+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-13 14:24:39+02	2020-10-13 14:25:19+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-13 14:24:37+02	2020-10-13 14:25:31+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-13 14:25:19+02	2020-10-13 14:25:34+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-10-13 14:25:18+02	2020-10-13 14:26:15+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-13 14:25:32+02	2020-10-13 14:26:23+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-13 14:26:24+02	2020-10-13 14:27:51+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-10-13 14:25:34+02	2020-10-13 14:30:11+02	print	lilypond	\N	lilypond-2.20.0
2020-10-13 14:26:15+02	2020-10-13 14:31:19+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-10-13 14:31:19+02	2020-10-13 14:31:40+02	print	transfig	\N	transfig-3.2.5e_4
2020-10-13 14:31:41+02	2020-10-13 14:32:50+02	security	nmap	\N	nmap-7.91
2020-10-13 14:32:50+02	2020-10-13 14:33:31+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-13 14:27:52+02	2020-10-13 14:33:38+02	print	tex-xetex	\N	tex-xetex-0.99992_37
2020-10-13 14:33:31+02	2020-10-13 14:34:53+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-13 14:33:39+02	2020-10-13 14:35:10+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-10-13 14:34:54+02	2020-10-13 14:35:24+02	sysutils	smartmontools	\N	smartmontools-7.1_3
2020-10-13 14:35:10+02	2020-10-13 14:35:58+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-10-13 14:35:25+02	2020-10-13 14:36:09+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.15.0
2020-10-13 14:36:09+02	2020-10-13 14:36:20+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-10-13 14:36:20+02	2020-10-13 14:36:30+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-10-13 14:35:58+02	2020-10-13 14:36:33+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.15.0
2020-10-13 14:36:31+02	2020-10-13 14:36:43+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-10-13 14:36:34+02	2020-10-13 14:37:25+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.15.0
2020-10-13 14:36:44+02	2020-10-13 14:37:39+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.15.0
2020-10-13 14:37:39+02	2020-10-13 14:38:36+02	x11	mate-base	\N	mate-base-1.24.1
2020-10-13 14:38:37+02	2020-10-13 14:39:45+02	x11	mate-screensaver	\N	mate-screensaver-1.24.1
2020-10-13 14:39:45+02	2020-10-13 14:41:18+02	x11	mate-terminal	\N	mate-terminal-1.24.1
2020-10-13 14:37:25+02	2020-10-13 14:41:42+02	x11	mate-applets	\N	mate-applets-1.24.1
2020-10-13 14:41:42+02	2020-10-13 14:42:51+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-10-13 14:41:18+02	2020-10-13 14:42:59+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-13 14:42:59+02	2020-10-13 14:43:31+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-10-13 14:42:51+02	2020-10-13 14:43:39+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-13 14:43:39+02	2020-10-13 14:44:48+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-10-13 14:43:31+02	2020-10-13 14:47:17+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-10-13 14:30:12+02	2020-10-13 14:48:14+02	print	texlive-docs	\N	texlive-docs-20150523
2020-10-13 14:47:17+02	2020-10-13 14:50:04+02	astro	xephem	\N	xephem-3.7.7_5
2020-10-13 14:50:04+02	2020-10-13 14:52:10+02	cad	cura	\N	Cura-4.7.1,2
2020-10-13 14:48:15+02	2020-10-13 14:52:30+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-13 14:52:31+02	2020-10-13 14:54:49+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-13 14:52:11+02	2020-10-13 14:57:33+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-13 14:57:34+02	2020-10-13 14:58:58+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-13 14:44:48+02	2020-10-13 14:59:08+02	astro	stellarium	\N	stellarium-0.20.3
2020-10-13 14:58:59+02	2020-10-13 15:00:21+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-13 14:59:09+02	2020-10-13 15:02:24+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-13 15:00:22+02	2020-10-13 15:02:44+02	deskutils	calibre	\N	calibre-4.23.0_3
2020-10-13 15:02:25+02	2020-10-13 15:03:06+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-10-13 15:03:07+02	2020-10-13 15:15:24+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-13 15:15:25+02	2020-10-13 15:16:32+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-13 15:02:45+02	2020-10-13 15:28:39+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-13 15:16:33+02	2020-10-13 15:40:42+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-13 15:40:45+02	2020-10-13 15:40:57+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-10-13 15:40:58+02	2020-10-13 15:41:48+02	devel	tortoisehg	py37	tortoisehg-5.5.1_2
2020-10-13 15:41:48+02	2020-10-13 15:49:19+02	editors	codelite	\N	codelite-12.0_4
2020-10-13 15:49:19+02	2020-10-13 15:50:50+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.2.2
2020-10-13 15:50:51+02	2020-10-13 15:53:10+02	editors	vim	\N	vim-8.2.1558
2020-10-13 15:53:12+02	2020-10-13 16:00:46+02	emulators	qemu	\N	qemu-4.2.1
2020-10-13 15:28:40+02	2020-10-13 16:27:35+02	devel	ghidra	\N	ghidra-9.1
2020-10-13 16:08:08+02	2020-10-13 16:54:01+02	emulators	wine	\N	wine-5.0.2_1,1
2020-10-13 16:54:02+02	2020-10-13 16:54:24+02	games	KnightCap	\N	knightcap-3.6_8
2020-10-13 16:54:27+02	2020-10-13 16:54:55+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-10-13 16:27:37+02	2020-10-13 16:59:04+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-13 16:54:56+02	2020-10-13 16:59:45+02	games	alienarena	\N	alienarena-2013.766_9
2020-10-13 16:59:05+02	2020-10-13 17:02:21+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-10-13 17:02:24+02	2020-10-13 17:03:49+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-10-13 17:03:52+02	2020-10-13 17:05:47+02	games	monsterz	\N	monsterz-0.7.1_15
2020-10-13 17:05:49+02	2020-10-13 17:07:57+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-10-13 17:08:00+02	2020-10-13 17:09:58+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-10-13 16:59:47+02	2020-10-13 17:10:06+02	games	battletanks	\N	btanks-0.9.8083_9
2020-10-13 17:10:01+02	2020-10-13 17:13:16+02	games	xblast	\N	xblast-2.10.4_18
2020-10-13 17:10:08+02	2020-10-13 17:13:31+02	games	xboard	\N	xboard-4.9.1_1
2020-10-13 17:13:17+02	2020-10-13 17:15:07+02	games	xcowsay	\N	xcowsay-1.5.1
2020-10-13 17:15:08+02	2020-10-13 17:25:40+02	graphics	dia	\N	dia-0.97.3_1,1
2020-10-13 17:25:42+02	2020-10-13 17:28:38+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-13 17:28:41+02	2020-10-13 17:41:44+02	graphics	inkscape	\N	inkscape-1.0.1_1
2020-10-13 17:13:32+02	2020-10-13 17:50:18+02	graphics	blender	\N	blender-2.90.0_1
2020-10-13 14:54:50+02	2020-10-13 17:57:24+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-13 17:41:48+02	2020-10-13 18:00:52+02	graphics	krita	\N	krita-4.3.0_6
2020-10-13 17:50:19+02	2020-10-13 18:02:33+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-10-13 18:00:54+02	2020-10-13 18:03:54+02	graphics	xpdf	\N	xpdf-4.02,1
2020-10-13 18:03:57+02	2020-10-13 18:06:21+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-13 17:57:27+02	2020-10-13 18:11:57+02	graphics	wings	\N	wings-2.2.5
2020-10-13 18:06:22+02	2020-10-13 19:04:18+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.11
2020-10-13 19:04:18+02	2020-10-13 19:05:51+02	math	geogebra	\N	geogebra-5.0.580.0
2020-10-13 19:05:52+02	2020-10-13 19:07:28+02	math	wxmaxima	\N	wxmaxima-20.04.0_2
2020-10-13 18:12:01+02	2020-10-13 19:08:01+02	lang	erlang-runtime23	\N	erlang-runtime23-23.1.1
2020-10-13 19:08:01+02	2020-10-13 19:08:18+02	multimedia	libva-glx	\N	libva-glx-2.9.0
2020-10-13 19:07:29+02	2020-10-13 19:08:19+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-13 19:08:19+02	2020-10-13 19:08:35+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-10-13 19:08:35+02	2020-10-13 19:11:21+02	multimedia	openshot	py37	openshot-2.5.1.369_1
2020-10-13 19:08:20+02	2020-10-13 19:16:21+02	multimedia	obs-studio	\N	obs-studio-26.0.2
2020-10-13 19:11:22+02	2020-10-13 19:17:23+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-13 19:16:23+02	2020-10-13 19:18:46+02	net	ntopng	\N	ntopng-4.0.d20200917,1
2020-10-13 19:17:24+02	2020-10-13 19:19:02+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-13 19:18:47+02	2020-10-13 19:19:05+02	net	vnstat	\N	vnstat-2.6_1
2020-10-13 19:19:05+02	2020-10-13 19:19:55+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-13 19:19:55+02	2020-10-13 19:20:44+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-13 19:20:44+02	2020-10-13 19:21:41+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-13 19:21:42+02	2020-10-13 19:22:34+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-13 19:22:34+02	2020-10-13 19:23:29+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-13 19:23:29+02	2020-10-13 19:24:45+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-13 19:24:46+02	2020-10-13 19:25:40+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-13 19:25:41+02	2020-10-13 19:26:34+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-13 18:32:06+02	2020-10-13 19:27:31+02	lang	mono	\N	mono-5.10.1.57_3
2020-10-13 19:26:34+02	2020-10-13 19:27:52+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-13 19:27:32+02	2020-10-13 19:28:31+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-13 19:27:52+02	2020-10-13 19:29:02+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-13 19:19:03+02	2020-10-13 19:29:33+02	net	wireshark	\N	wireshark-3.2.7
2020-10-13 19:29:02+02	2020-10-13 19:30:20+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-10-13 19:28:34+02	2020-10-13 19:30:23+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-10-13 19:30:21+02	2020-10-13 19:31:12+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-10-13 19:30:23+02	2020-10-13 19:31:36+02	print	gv	\N	gv-3.7.4_5
2020-10-13 19:29:34+02	2020-10-13 19:34:03+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-13 19:34:03+02	2020-10-13 19:40:27+02	print	texlive-full	\N	texlive-full-20150521
2020-10-13 19:40:28+02	2020-10-13 19:40:35+02	security	base-audit	\N	base-audit-0.4
2020-10-13 19:40:36+02	2020-10-13 19:41:40+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-10-13 19:41:40+02	2020-10-13 19:41:58+02	security	rkhunter	\N	rkhunter-1.4.6
2020-10-13 19:31:13+02	2020-10-13 19:42:07+02	print	lyx	\N	lyx-2.3.4.2_1
2020-10-13 19:42:08+02	2020-10-13 19:42:56+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-10-13 19:41:59+02	2020-10-13 19:43:01+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-13 19:43:01+02	2020-10-13 19:44:08+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-13 19:42:57+02	2020-10-13 19:44:36+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-13 19:44:08+02	2020-10-13 19:46:35+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-10-13 19:46:36+02	2020-10-13 19:47:26+02	www	dokuwiki	\N	dokuwiki-20200729
2020-10-13 19:31:37+02	2020-10-13 19:58:48+02	print	scribus-devel	\N	scribus-devel-1.5.5_20
2020-10-13 19:44:37+02	2020-10-13 19:59:57+02	textproc	bat	\N	bat-0.16.0
2020-10-13 19:58:49+02	2020-10-13 20:00:09+02	www	links	\N	links-2.20.2,1
2020-10-13 19:59:58+02	2020-10-13 20:00:18+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-10-13 20:00:11+02	2020-10-13 20:00:25+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-10-13 20:00:18+02	2020-10-13 20:00:34+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-3.1
2020-10-13 20:00:25+02	2020-10-13 20:00:39+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-10-13 20:00:34+02	2020-10-13 20:00:48+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-10-13 20:00:39+02	2020-10-13 20:00:56+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-10-13 20:00:49+02	2020-10-13 20:01:05+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-10-13 20:00:57+02	2020-10-13 20:01:11+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-10-13 20:01:06+02	2020-10-13 20:01:20+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-10-13 20:01:12+02	2020-10-13 20:01:33+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-10-13 20:01:20+02	2020-10-13 20:02:46+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-10-13 20:01:33+02	2020-10-13 20:03:33+02	x11	mate	\N	mate-1.24.1
2020-10-13 20:02:47+02	2020-10-13 20:04:08+02	x11	xlockmore	\N	xlockmore-5.65
2020-10-13 20:03:34+02	2020-10-13 20:08:35+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-10-13 20:08:35+02	2020-10-13 20:09:58+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-10-13 20:04:09+02	2020-10-13 20:10:00+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-13 20:09:59+02	2020-10-13 20:10:12+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-10-13 20:10:02+02	2020-10-13 20:10:36+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-10-13 20:10:12+02	2020-10-13 20:10:40+02	x11-toolkits	py-tkinter	py39	py39-tkinter-3.9.0_6
2020-10-13 20:10:40+02	2020-10-13 20:11:26+02	graphics	xfig	\N	xfig-3.2.7a
2020-10-13 20:13:33+02	2020-10-13 20:14:08+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-10-13 19:47:29+02	2020-10-13 20:33:22+02	www	firefox	\N	firefox-81.0.1_3,2
2020-10-13 20:10:37+02	2020-10-13 20:35:56+02	editors	vscode	\N	vscode-1.46.1
2020-10-14 07:34:09+02	2020-10-14 07:35:40+02	graphics	mesa-libs	\N	mesa-libs-20.2.0_1
2020-10-14 07:35:41+02	2020-10-14 07:36:43+02	graphics	cairo	\N	cairo-1.16.0_1,3
2020-10-14 07:36:43+02	2020-10-14 07:38:53+02	x11-toolkits	pango	\N	pango-1.42.4_5
2020-10-14 07:38:53+02	2020-10-14 07:39:40+02	graphics	poppler	\N	poppler-20.10.0
2020-10-14 07:34:09+02	2020-10-14 07:40:09+02	lang	ruby27	\N	ruby-2.7.2_1,1
2020-10-14 07:40:09+02	2020-10-14 07:40:34+02	graphics	libepoxy	\N	libepoxy-1.5.4
2020-10-14 07:39:40+02	2020-10-14 07:40:38+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-10-14 07:40:38+02	2020-10-14 07:41:35+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-10-14 07:40:34+02	2020-10-14 07:41:38+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-10-14 07:41:38+02	2020-10-14 07:42:59+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.38.0
2020-10-14 07:41:35+02	2020-10-14 07:45:37+02	print	texlive-base	\N	texlive-base-20150521_56
2020-10-14 07:43:00+02	2020-10-14 07:45:38+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0_1
2020-10-14 07:45:39+02	2020-10-14 07:47:44+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-10-14 07:47:45+02	2020-10-14 07:48:08+02	graphics	libGLU	\N	libGLU-9.0.1
2020-10-14 07:48:09+02	2020-10-14 07:51:35+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-10-14 07:51:35+02	2020-10-14 07:56:28+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-10-14 07:56:29+02	2020-10-14 07:57:20+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-10-14 07:57:20+02	2020-10-14 08:00:19+02	graphics	graphviz	\N	graphviz-2.44.1_3
2020-10-14 08:00:19+02	2020-10-14 08:00:35+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1,1
2020-10-14 08:00:36+02	2020-10-14 08:01:52+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-10-14 08:01:53+02	2020-10-14 08:02:29+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-10-14 08:02:30+02	2020-10-14 08:02:54+02	graphics	frei0r	\N	frei0r-1.6.1
2020-10-14 08:02:55+02	2020-10-14 08:04:34+02	graphics	graphene	\N	graphene-1.10.2
2020-10-14 08:04:35+02	2020-10-14 08:04:53+02	www	libwpe	\N	libwpe-1.4.0.1
2020-10-14 08:04:53+02	2020-10-14 08:06:29+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2_1
2020-10-14 08:06:29+02	2020-10-14 08:06:50+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-10-14 07:45:38+02	2020-10-14 08:07:01+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-10-14 08:06:50+02	2020-10-14 08:07:28+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-10-14 08:07:02+02	2020-10-14 08:10:01+02	print	tex-formats	\N	tex-formats-20150521_2
2020-10-14 08:07:29+02	2020-10-14 08:14:01+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-10-14 08:10:02+02	2020-10-14 08:16:11+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-10-14 08:14:01+02	2020-10-14 08:17:31+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-10-14 08:17:32+02	2020-10-14 08:18:13+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-10-14 08:18:13+02	2020-10-14 08:19:26+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-10-14 08:19:26+02	2020-10-14 08:19:43+02	devel	ruby-gems	\N	ruby27-gems-3.0.6
2020-10-14 08:16:12+02	2020-10-14 08:23:12+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-10-14 08:19:44+02	2020-10-14 08:25:13+02	graphics	mesa-dri	\N	mesa-dri-20.2.0_1
2020-10-14 08:23:13+02	2020-10-14 08:25:27+02	textproc	docbook-utils	\N	docbook-utils-0.6.14_13
2020-10-14 08:25:14+02	2020-10-14 08:25:47+02	devel	sdl12	\N	sdl-1.2.15_15,2
2020-10-14 08:25:47+02	2020-10-14 08:26:33+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-10-14 08:26:33+02	2020-10-14 08:27:31+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-10-14 08:25:27+02	2020-10-14 08:28:32+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-10-14 08:27:31+02	2020-10-14 08:28:37+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.5
2020-10-14 08:28:32+02	2020-10-14 08:28:52+02	graphics	freeglut	\N	freeglut-3.0.0_2
2020-10-14 08:28:38+02	2020-10-14 08:29:43+02	devel	py-qt5-core	py37	py37-qt5-core-5.15.0
2020-10-14 08:29:43+02	2020-10-14 08:30:53+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-10-14 08:28:52+02	2020-10-14 08:31:16+02	x11-servers	xorg-server	\N	xorg-server-1.20.9,1
2020-10-14 08:30:54+02	2020-10-14 08:31:41+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-10-14 08:31:17+02	2020-10-14 08:32:08+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-10-14 08:32:09+02	2020-10-14 08:32:20+02	databases	py-sqlparse	py37	py37-sqlparse-0.4.1
2020-10-14 08:31:41+02	2020-10-14 08:32:28+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-10-14 08:32:21+02	2020-10-14 08:33:52+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-10-14 08:32:29+02	2020-10-14 08:34:20+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-10-14 08:34:21+02	2020-10-14 08:34:32+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-10-14 08:33:52+02	2020-10-14 08:35:09+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-14 08:34:33+02	2020-10-14 08:35:40+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-10-14 08:35:09+02	2020-10-14 08:35:56+02	www	py-django22	py37	py37-django22-2.2.16
2020-10-14 08:35:41+02	2020-10-14 08:36:29+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-10-14 08:36:30+02	2020-10-14 08:37:53+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-10-14 08:35:56+02	2020-10-14 08:39:07+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.15.0
2020-10-14 08:37:53+02	2020-10-14 08:39:47+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-10-14 08:39:07+02	2020-10-14 08:40:13+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-10-14 08:39:48+02	2020-10-14 08:40:37+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-10-14 08:40:13+02	2020-10-14 08:41:04+02	devel	sdl20	\N	sdl2-2.0.12_3
2020-10-14 08:40:39+02	2020-10-14 08:41:31+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-10-14 08:41:04+02	2020-10-14 08:42:21+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-10-14 08:41:32+02	2020-10-14 08:42:31+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-10-14 08:42:31+02	2020-10-14 08:42:43+02	devel	py-attrs	py37	py37-attrs-20.2.0
2020-10-14 08:42:43+02	2020-10-14 08:42:55+02	graphics	glew	\N	glew-2.2.0
2020-10-14 08:42:21+02	2020-10-14 08:43:12+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-10-14 08:42:56+02	2020-10-14 08:43:29+02	graphics	poppler-glib	\N	poppler-glib-20.10.0
2020-10-14 08:43:29+02	2020-10-14 08:43:47+02	multimedia	smpeg	\N	smpeg-0.4.4_15
2020-10-14 08:43:12+02	2020-10-14 08:43:54+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-10-14 08:43:47+02	2020-10-14 08:44:41+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-10-14 08:43:55+02	2020-10-14 08:45:06+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-10-14 08:45:08+02	2020-10-14 08:45:23+02	graphics	sdl_image	\N	sdl_image-1.2.12_12
2020-10-14 08:44:42+02	2020-10-14 08:45:34+02	graphics	ftgl	\N	ftgl-2.4.0,1
2020-10-14 08:45:23+02	2020-10-14 08:46:19+02	net	py-qt5-network	py37	py37-qt5-network-5.15.0
2020-10-14 08:45:34+02	2020-10-14 08:46:24+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-10-14 08:46:24+02	2020-10-14 08:46:56+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-10-14 08:46:19+02	2020-10-14 08:48:05+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.15.0
2020-10-14 08:48:06+02	2020-10-14 08:48:23+02	x11	xdriinfo	\N	xdriinfo-1.0.6_3
2020-10-14 08:46:57+02	2020-10-14 08:48:40+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-10-14 08:48:24+02	2020-10-14 08:49:14+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-10-14 08:48:41+02	2020-10-14 08:49:29+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0_1
2020-10-14 08:49:14+02	2020-10-14 08:49:59+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-10-14 08:49:30+02	2020-10-14 08:50:20+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-10-14 08:50:20+02	2020-10-14 08:51:07+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-10-14 08:50:00+02	2020-10-14 08:51:26+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-10-14 08:51:08+02	2020-10-14 08:51:58+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.5.0
2020-10-14 08:51:27+02	2020-10-14 08:52:21+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-10-14 08:52:22+02	2020-10-14 08:53:19+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-10-14 08:53:19+02	2020-10-14 08:53:34+02	print	gl2ps	\N	gl2ps-1.4.0
2020-10-14 08:53:34+02	2020-10-14 08:53:58+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-10-14 08:53:58+02	2020-10-14 08:55:56+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.9,1
2020-10-14 08:55:56+02	2020-10-14 08:58:18+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-10-14 08:51:59+02	2020-10-14 08:58:28+02	math	vtk8	\N	vtk8-8.2.0
2020-10-14 08:58:19+02	2020-10-14 08:59:55+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-10-14 08:59:56+02	2020-10-14 09:01:10+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-10-14 09:01:11+02	2020-10-14 09:02:31+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-10-14 08:58:29+02	2020-10-14 09:05:04+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-14 09:05:05+02	2020-10-14 09:06:10+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-10-14 09:06:11+02	2020-10-14 09:06:35+02	audio	sdl_mixer	\N	sdl_mixer-1.2.12_12
2020-10-14 09:06:35+02	2020-10-14 09:07:57+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-10-14 09:07:57+02	2020-10-14 09:09:46+02	graphics	cogl	\N	cogl-1.22.8
2020-10-14 09:02:31+02	2020-10-14 09:10:03+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-10-14 09:09:47+02	2020-10-14 09:10:44+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-10-14 09:10:05+02	2020-10-14 09:11:02+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.15.0
2020-10-14 09:11:03+02	2020-10-14 09:12:05+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-10-14 09:12:05+02	2020-10-14 09:12:59+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-10-14 09:13:02+02	2020-10-14 09:13:35+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-10-14 09:10:45+02	2020-10-14 09:33:56+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-14 09:33:59+02	2020-10-14 09:34:13+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-10-14 09:34:16+02	2020-10-14 09:37:35+02	graphics	clutter	\N	clutter-1.26.2
2020-10-14 09:37:37+02	2020-10-14 09:38:59+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-10-14 09:39:01+02	2020-10-14 09:39:43+02	graphics	poppler-utils	\N	poppler-utils-20.10.0
2020-10-14 09:39:43+02	2020-10-14 09:40:19+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-10-14 09:40:19+02	2020-10-14 09:43:52+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-10-14 09:43:53+02	2020-10-14 09:44:10+02	graphics	sdl_ttf	\N	sdl_ttf-2.0.11_7
2020-10-14 09:44:11+02	2020-10-14 09:45:11+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-10-14 09:45:11+02	2020-10-14 09:45:50+02	databases	gom	\N	gom-0.4
2020-10-14 09:45:50+02	2020-10-14 09:46:41+02	devel	dee	\N	dee-1.2.7_17
2020-10-14 09:46:42+02	2020-10-14 09:48:18+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-10-14 09:48:20+02	2020-10-14 09:48:33+02	devel	rubygem-mustache	\N	rubygem-mustache-1.1.1
2020-10-14 09:48:34+02	2020-10-14 09:56:19+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-10-14 09:56:20+02	2020-10-14 09:57:30+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.15.0
2020-10-14 09:57:31+02	2020-10-14 09:58:54+02	math	octomap	\N	octomap-1.9.5.190
2020-10-14 09:58:54+02	2020-10-14 10:00:20+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-10-14 10:00:21+02	2020-10-14 10:00:40+02	net	sdl_net	\N	sdl_net-1.2.8_3
2020-10-14 10:00:40+02	2020-10-14 10:02:09+02	sysutils	tracker	\N	tracker-2.3.4
2020-10-14 10:02:10+02	2020-10-14 10:02:36+02	textproc	rubygem-rdiscount	\N	rubygem-rdiscount-2.2.0.1
2020-10-14 10:02:36+02	2020-10-14 10:03:35+02	www	rubygem-hpricot	\N	rubygem-hpricot-0.8.6
2020-10-14 10:03:36+02	2020-10-14 10:04:50+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-10-14 10:04:51+02	2020-10-14 10:06:05+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-10-14 10:06:06+02	2020-10-14 10:07:40+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-10-14 10:07:41+02	2020-10-14 10:09:53+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-10-14 10:09:54+02	2020-10-14 10:10:51+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-10-14 10:10:52+02	2020-10-14 10:11:09+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-10-14 10:11:09+02	2020-10-14 10:11:40+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-10-14 10:11:43+02	2020-10-14 10:13:46+02	graphics	gegl	\N	gegl-0.4.26_6
2020-10-14 10:13:47+02	2020-10-14 10:14:39+02	graphics	glfw	\N	glfw-3.3.2
2020-10-14 10:14:40+02	2020-10-14 10:15:58+02	graphics	movit	\N	movit-1.6.2_2
2020-10-14 10:15:58+02	2020-10-14 10:22:01+02	graphics	opencv	\N	opencv-3.4.1_36
2020-10-14 10:22:01+02	2020-10-14 10:23:05+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-10-14 10:23:06+02	2020-10-14 10:23:36+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-10-14 10:23:36+02	2020-10-14 10:24:57+02	math	fcl05	\N	fcl05-0.5.0
2020-10-14 10:24:57+02	2020-10-14 10:26:06+02	multimedia	clutter-gst3	\N	clutter-gst3-3.0.27
2020-10-14 10:26:06+02	2020-10-14 10:27:17+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-14 10:27:18+02	2020-10-14 10:39:10+02	print	tex-luatex	\N	tex-luatex-0.80.0_21
2020-10-14 10:39:11+02	2020-10-14 10:39:31+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-10-14 10:39:31+02	2020-10-14 10:39:45+02	textproc	rubygem-ronn	\N	rubygem-ronn-0.7.3
2020-10-14 10:39:45+02	2020-10-14 10:40:46+02	x11-themes	gnome-icon-theme-symbolic	\N	gnome-icon-theme-symbolic-3.12.0
2020-10-14 10:40:46+02	2020-10-14 10:42:19+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-10-14 10:42:21+02	2020-10-14 10:43:20+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-10-14 10:43:21+02	2020-10-14 10:43:55+02	audio	sdl_sound	\N	sdl_sound-1.0.3_14
2020-10-14 10:43:55+02	2020-10-14 10:47:35+02	devel	ncurses	\N	ncurses-6.2.20201010
2020-10-14 10:47:35+02	2020-10-14 10:47:50+02	devel	py-colorlog	py37	py37-colorlog-4.4.0
2020-10-14 10:47:51+02	2020-10-14 10:49:05+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-10-14 10:49:05+02	2020-10-14 10:50:12+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-10-14 10:50:12+02	2020-10-14 10:51:22+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-10-14 10:51:23+02	2020-10-14 10:52:29+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-10-14 10:52:29+02	2020-10-14 10:55:26+02	devel	shiboken2	py37	py37-shiboken2-5.15.1
2020-10-14 10:55:27+02	2020-10-14 11:01:59+02	graphics	Coin	\N	Coin-4.0.0_1
2020-10-14 11:02:01+02	2020-10-14 11:06:26+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-10-14 11:06:27+02	2020-10-14 11:07:24+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-10-14 11:07:24+02	2020-10-14 11:07:53+02	graphics	fpc-opengl	\N	fpc-opengl-3.0.4_4
2020-10-14 11:07:53+02	2020-10-14 11:10:17+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-10-14 11:10:18+02	2020-10-14 11:19:02+02	graphics	gimp-app	\N	gimp-app-2.10.20_7,1
2020-10-14 11:19:03+02	2020-10-14 11:19:45+02	graphics	libgltf	\N	libgltf-0.0.2_21
2020-10-14 11:19:46+02	2020-10-14 11:20:25+02	graphics	libgxps	\N	libgxps-0.3.1
2020-10-14 11:20:26+02	2020-10-14 11:20:55+02	graphics	libplacebo	\N	libplacebo-2.72.0
2020-10-14 11:20:56+02	2020-10-14 11:22:38+02	graphics	poppler-qt5	\N	poppler-qt5-20.10.0
2020-10-14 11:22:39+02	2020-10-14 11:23:49+02	graphics	py-PyOpenGL	py37	py37-PyOpenGL-3.1.5
2020-10-14 11:23:50+02	2020-10-14 11:24:52+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.15.0
2020-10-14 11:24:52+02	2020-10-14 11:31:32+02	graphics	py-scikit-image	py37	py37-scikit-image-0.17.2
2020-10-14 11:31:33+02	2020-10-14 11:54:12+02	math	R	\N	R-4.0.3
2020-10-14 11:54:14+02	2020-10-14 11:55:49+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-10-14 11:55:50+02	2020-10-14 11:57:16+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-14 11:57:16+02	2020-10-14 11:58:24+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-14 11:58:25+02	2020-10-14 11:58:55+02	print	libspectre	\N	libspectre-0.2.9
2020-10-14 11:58:56+02	2020-10-14 12:08:27+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-10-14 12:08:28+02	2020-10-14 12:08:50+02	sysutils	p5-File-Listing	\N	p5-File-Listing-6.11
2020-10-14 12:08:51+02	2020-10-14 12:11:45+02	textproc	dblatex	\N	dblatex-0.3.11
2020-10-14 12:11:46+02	2020-10-14 12:12:39+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.15.0
2020-10-14 12:12:39+02	2020-10-14 12:14:09+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.15.0
2020-10-14 12:14:10+02	2020-10-14 12:14:29+02	x11-fonts	twemoji-color-font-ttf	\N	twemoji-color-font-ttf-13.0.1
2020-10-14 12:14:30+02	2020-10-14 12:17:25+02	x11-servers	xephyr	\N	xephyr-1.20.9,1
2020-10-14 12:17:25+02	2020-10-14 12:18:53+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-10-14 12:18:54+02	2020-10-14 12:19:55+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-10-14 12:19:56+02	2020-10-14 12:21:17+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.15.0
2020-10-14 12:21:17+02	2020-10-14 12:23:12+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-10-14 12:23:12+02	2020-10-14 12:24:49+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-10-14 12:24:50+02	2020-10-14 12:25:49+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0_1
2020-10-14 12:25:50+02	2020-10-14 12:26:18+02	audio	faudio	\N	FAudio-20.10
2020-10-14 12:26:18+02	2020-10-14 12:27:44+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-14 12:27:45+02	2020-10-14 12:29:08+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-10-14 12:29:09+02	2020-10-14 12:30:12+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-14 12:30:13+02	2020-10-14 12:31:18+02	devel	adacurses	\N	adacurses-20180127
2020-10-14 12:31:18+02	2020-10-14 12:31:42+02	devel	erlang-esdl	\N	erlang-esdl-1.3.1
2020-10-14 12:31:42+02	2020-10-14 12:32:02+02	devel	flatzebra	\N	flatzebra-0.1.7
2020-10-14 12:32:02+02	2020-10-14 12:33:09+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-10-14 12:33:10+02	2020-10-14 12:34:18+02	devel	ode	\N	ode-0.13_4,1
2020-10-14 12:34:18+02	2020-10-14 12:35:14+02	devel	py-game	py37	py37-game-1.9.6_1
2020-10-14 12:35:15+02	2020-10-14 12:36:28+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.5
2020-10-14 12:36:29+02	2020-10-14 12:36:44+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-10-14 12:36:44+02	2020-10-14 12:38:25+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-14 12:55:00+02	2020-10-14 12:55:17+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-10-14 12:55:17+02	2020-10-14 12:55:30+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-10-14 12:55:30+02	2020-10-14 12:55:44+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-10-14 12:55:45+02	2020-10-14 13:26:20+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-10-14 13:26:22+02	2020-10-14 13:27:11+02	emulators	dosbox	\N	dosbox-0.74.3_1
2020-10-14 13:27:11+02	2020-10-14 13:29:25+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-10-14 13:29:26+02	2020-10-14 13:29:48+02	graphics	gle	\N	gle-3.1.0_7
2020-10-14 13:29:49+02	2020-10-14 13:30:11+02	graphics	gltt	\N	gltt-2.5.2_14
2020-10-14 13:30:12+02	2020-10-14 13:34:31+02	graphics	opensubdiv	\N	opensubdiv-3.4.3_1
2020-10-14 13:34:32+02	2020-10-14 13:35:22+02	graphics	sdl_gfx	\N	sdl_gfx-2.0.26
2020-10-14 13:35:23+02	2020-10-14 13:48:35+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-10-14 13:48:36+02	2020-10-14 13:49:26+02	math	py-numpy-stl	py37	py37-numpy-stl-2.11.3
2020-10-14 13:49:27+02	2020-10-14 13:58:38+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-10-14 13:58:39+02	2020-10-14 14:00:49+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-10-14 14:00:50+02	2020-10-14 14:03:19+02	multimedia	mpv	\N	mpv-0.32.0_5,1
2020-10-14 14:03:20+02	2020-10-14 14:10:04+02	multimedia	vlc	\N	vlc-3.0.11_5,4
2020-10-14 14:10:05+02	2020-10-14 14:11:46+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-14 14:11:47+02	2020-10-14 14:12:56+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-14 14:12:56+02	2020-10-14 14:13:12+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-10-14 14:13:12+02	2020-10-14 14:21:23+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-10-14 14:21:26+02	2020-10-14 14:24:14+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-10-14 07:34:09+02	2020-10-14 14:29:04+02	lang	rust	\N	rust-1.47.0
2020-10-14 14:24:15+02	2020-10-14 14:32:11+02	print	tex-xetex	\N	tex-xetex-0.99992_37
2020-10-14 14:29:06+02	2020-10-14 14:34:58+02	devel	rust-cbindgen	\N	rust-cbindgen-0.15.0_1
2020-10-14 14:34:59+02	2020-10-14 14:51:24+02	lang	spidermonkey78	\N	spidermonkey78-78.3.1_1
2020-10-14 14:51:25+02	2020-10-14 14:52:23+02	sysutils	polkit	\N	polkit-0.118
2020-10-14 14:52:23+02	2020-10-14 14:55:11+02	graphics	colord	\N	colord-1.3.5_1
2020-10-14 14:32:13+02	2020-10-14 14:57:56+02	print	texlive-docs	\N	texlive-docs-20150523
2020-10-14 14:57:58+02	2020-10-14 14:59:20+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-10-14 14:59:20+02	2020-10-14 14:59:59+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-10-14 14:55:12+02	2020-10-14 15:00:22+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-10-14 15:00:00+02	2020-10-14 15:02:06+02	audio	pulseaudio	\N	pulseaudio-13.0_1
2020-10-14 15:02:06+02	2020-10-14 15:03:28+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-10-14 15:03:28+02	2020-10-14 15:04:48+02	x11	mate-desktop	\N	mate-desktop-1.24.1
2020-10-14 15:04:48+02	2020-10-14 15:06:38+02	security	gcr	\N	gcr-3.28.0_1
2020-10-14 15:06:39+02	2020-10-14 15:08:07+02	sysutils	hal	\N	hal-0.5.14_34
2020-10-14 15:08:08+02	2020-10-14 15:09:17+02	x11-toolkits	vte3	\N	vte3-0.62.1
2020-10-14 15:00:23+02	2020-10-14 15:09:36+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-14 15:09:18+02	2020-10-14 15:10:25+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-10-14 15:10:25+02	2020-10-14 15:11:21+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-10-14 15:11:21+02	2020-10-14 15:12:34+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-10-14 15:09:36+02	2020-10-14 15:16:37+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-14 15:12:35+02	2020-10-14 15:16:43+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-10-14 15:16:44+02	2020-10-14 15:17:32+02	sysutils	upower	\N	upower-0.99.11
2020-10-14 15:16:38+02	2020-10-14 15:17:38+02	net	remmina	\N	remmina-1.3.9
2020-10-14 15:17:38+02	2020-10-14 15:18:33+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-14 15:17:32+02	2020-10-14 15:19:27+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-14 15:18:33+02	2020-10-14 15:19:39+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-10-14 15:19:28+02	2020-10-14 15:20:25+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-10-14 15:20:26+02	2020-10-14 15:21:18+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-14 15:19:39+02	2020-10-14 15:21:21+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-14 15:21:19+02	2020-10-14 15:22:19+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-10-14 15:21:21+02	2020-10-14 15:22:50+02	devel	libgdata	\N	libgdata-0.17.13
2020-10-14 15:22:20+02	2020-10-14 15:23:31+02	devel	libpeas	\N	libpeas-1.22.0
2020-10-14 15:23:31+02	2020-10-14 15:26:30+02	x11	xorg	\N	xorg-7.7_3
2020-10-14 15:22:50+02	2020-10-14 15:26:30+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-14 15:26:30+02	2020-10-14 15:27:38+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-10-14 15:26:30+02	2020-10-14 15:27:41+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-14 15:27:41+02	2020-10-14 15:28:55+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-10-14 15:27:38+02	2020-10-14 15:30:21+02	x11	mate-panel	\N	mate-panel-1.24.1_1
2020-10-14 15:30:23+02	2020-10-14 15:34:30+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-10-14 15:28:55+02	2020-10-14 15:35:12+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-14 15:34:30+02	2020-10-14 15:35:30+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-10-14 15:35:31+02	2020-10-14 15:36:09+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-14 15:39:08+02	2020-10-14 15:40:20+02	x11	libmatekbd	\N	libmatekbd-1.24.1
2020-10-14 15:40:21+02	2020-10-14 15:42:21+02	x11-fm	caja	\N	caja-1.24.0
2020-10-14 15:42:22+02	2020-10-14 15:44:13+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-10-14 15:44:14+02	2020-10-14 15:45:36+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-10-14 15:45:37+02	2020-10-14 15:46:30+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-14 15:46:31+02	2020-10-14 15:47:48+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-10-14 15:47:49+02	2020-10-14 15:48:49+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-10-14 15:35:13+02	2020-10-14 15:49:16+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-14 15:48:49+02	2020-10-14 15:49:47+02	net	libgweather	\N	libgweather-3.36.1
2020-10-14 09:13:35+02	2020-10-14 15:49:53+02	devel	llvm11	\N	llvm11-11.0.0
2020-10-14 15:49:48+02	2020-10-14 15:50:27+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-10-14 15:49:17+02	2020-10-14 15:50:31+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-14 15:49:54+02	2020-10-14 15:51:01+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-10-14 15:50:32+02	2020-10-14 15:51:27+02	x11	zenity	\N	zenity-3.32.0
2020-10-14 15:50:27+02	2020-10-14 15:51:32+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-10-14 15:51:33+02	2020-10-14 15:52:13+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-14 15:51:28+02	2020-10-14 15:54:04+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-10-14 15:54:04+02	2020-10-14 15:54:49+02	graphics	clutter-gtk3	\N	clutter-gtk3-1.8.4
2020-10-14 15:52:14+02	2020-10-14 15:55:46+02	editors	emacs	full	emacs-27.1_1,3
2020-10-14 15:54:49+02	2020-10-14 15:55:51+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-14 15:55:47+02	2020-10-14 15:56:25+02	print	cups-filters	\N	cups-filters-1.28.4
2020-10-14 15:56:26+02	2020-10-14 15:57:29+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0_1
2020-10-14 15:55:51+02	2020-10-14 15:58:11+02	print	fontforge	\N	fontforge-20190801_2
2020-10-14 15:57:30+02	2020-10-14 15:58:48+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-14 15:58:12+02	2020-10-14 15:59:11+02	sysutils	zeitgeist	\N	zeitgeist-1.0.2
2020-10-14 15:59:11+02	2020-10-14 16:00:04+02	x11	mate-session-manager	\N	mate-session-manager-1.24.1
2020-10-14 15:51:02+02	2020-10-14 16:00:35+02	databases	postgresql13-server	\N	postgresql13-server-13.0_3
2020-10-14 16:00:04+02	2020-10-14 16:01:01+02	x11	yelp	\N	yelp-3.38.0
2020-10-14 16:00:36+02	2020-10-14 16:01:35+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-10-14 16:01:35+02	2020-10-14 16:02:53+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-10-14 16:01:02+02	2020-10-14 16:03:16+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-10-14 16:03:16+02	2020-10-14 16:04:19+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-10-14 16:02:54+02	2020-10-14 16:04:23+02	x11-wm	marco	\N	marco-1.24.1
2020-10-14 16:04:25+02	2020-10-14 16:05:26+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-10-14 16:04:19+02	2020-10-14 16:06:15+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-14 16:05:26+02	2020-10-14 16:08:39+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-14 16:08:40+02	2020-10-14 16:11:53+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-14 16:11:54+02	2020-10-14 16:12:59+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-14 16:06:15+02	2020-10-14 16:14:48+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-14 16:14:49+02	2020-10-14 16:17:19+02	print	mftrace	\N	mftrace-1.2.18_2
2020-10-14 16:13:00+02	2020-10-14 16:17:23+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-14 16:17:20+02	2020-10-14 16:17:41+02	sysutils	hal-info	\N	hal-info-20091130
2020-10-14 16:17:42+02	2020-10-14 16:18:46+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-14 16:18:47+02	2020-10-14 16:20:25+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-10-14 16:17:24+02	2020-10-14 16:20:55+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-14 16:20:55+02	2020-10-14 16:23:45+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-14 16:23:46+02	2020-10-14 16:25:24+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-14 16:25:25+02	2020-10-14 16:26:38+02	databases	pg_qualstats	\N	pg_qualstats-2.0.2
2020-10-14 16:20:26+02	2020-10-14 16:26:48+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-14 16:26:40+02	2020-10-14 16:27:25+02	databases	pg_stat_kcache	\N	pg_stat_kcache-2.1.3
2020-10-14 16:26:50+02	2020-10-14 16:27:54+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-14 16:27:56+02	2020-10-14 16:28:50+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-14 16:27:26+02	2020-10-14 16:30:06+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-14 15:58:49+02	2020-10-14 16:30:49+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-14 16:30:08+02	2020-10-14 16:32:02+02	graphics	atril	\N	atril-1.22.3_10
2020-10-14 16:30:52+02	2020-10-14 16:32:02+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-14 16:32:03+02	2020-10-14 16:32:43+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-10-14 16:32:02+02	2020-10-14 16:33:17+02	graphics	eom	\N	eom-1.24.1
2020-10-14 16:33:18+02	2020-10-14 16:34:45+02	math	mate-calc	\N	mate-calc-1.24.1
2020-10-14 16:34:46+02	2020-10-14 16:41:16+02	math	maxima	\N	maxima-5.44.0_5
2020-10-14 16:32:44+02	2020-10-14 16:41:20+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-14 16:41:16+02	2020-10-14 16:42:17+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-14 16:41:21+02	2020-10-14 16:42:22+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-14 16:42:18+02	2020-10-14 16:43:10+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-14 16:42:22+02	2020-10-14 16:43:13+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-14 16:43:10+02	2020-10-14 16:43:45+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-14 16:43:14+02	2020-10-14 16:43:49+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-14 16:43:46+02	2020-10-14 16:44:30+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-14 16:43:50+02	2020-10-14 16:44:41+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-14 16:44:30+02	2020-10-14 16:45:30+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-14 16:44:42+02	2020-10-14 16:45:35+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-14 16:45:36+02	2020-10-14 16:46:16+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-14 16:46:16+02	2020-10-14 16:47:34+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-14 16:47:35+02	2020-10-14 16:49:16+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-10-14 16:49:17+02	2020-10-14 16:50:12+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-10-14 16:45:30+02	2020-10-14 16:50:18+02	print	lilypond	\N	lilypond-2.20.0
2020-10-14 16:28:53+02	2020-10-14 16:50:33+02	editors	libreoffice	\N	libreoffice-7.0.2.2_2
2020-10-14 16:50:19+02	2020-10-14 16:50:56+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.15.0
2020-10-14 16:50:13+02	2020-10-14 16:50:57+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.15.0
2020-10-14 16:50:57+02	2020-10-14 16:51:09+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-10-14 16:50:56+02	2020-10-14 16:51:11+02	www	p5-libwww	\N	p5-libwww-6.49
2020-10-14 16:51:09+02	2020-10-14 16:51:22+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-10-14 16:51:12+02	2020-10-14 16:51:27+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-10-14 16:51:23+02	2020-10-14 16:52:32+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.15.0
2020-10-14 16:51:28+02	2020-10-14 16:52:44+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.15.0
2020-10-14 16:52:45+02	2020-10-14 16:54:03+02	x11	mate-base	\N	mate-base-1.24.1
2020-10-14 16:50:34+02	2020-10-14 16:54:55+02	textproc	ripgrep	\N	ripgrep-12.1.1_6
2020-10-14 16:54:04+02	2020-10-14 16:55:22+02	x11	mate-screensaver	\N	mate-screensaver-1.24.1
2020-10-14 16:55:22+02	2020-10-14 16:55:50+02	x11-toolkits	fltk	\N	fltk-1.3.5
2020-10-14 16:54:55+02	2020-10-14 16:56:59+02	x11	mate-terminal	\N	mate-terminal-1.24.1
2020-10-14 16:55:50+02	2020-10-14 16:57:05+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-14 16:56:59+02	2020-10-14 16:57:31+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-10-14 16:52:32+02	2020-10-14 16:58:00+02	x11	mate-applets	\N	mate-applets-1.24.1
2020-10-14 16:57:32+02	2020-10-14 16:58:30+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-10-14 16:57:05+02	2020-10-14 17:01:38+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-10-14 16:58:31+02	2020-10-14 17:02:36+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-14 17:01:39+02	2020-10-14 17:03:17+02	cad	cura	\N	Cura-4.7.1,2
2020-10-14 16:58:01+02	2020-10-14 17:03:50+02	astro	stellarium	\N	stellarium-0.20.3
2020-10-14 17:03:18+02	2020-10-14 17:05:05+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-14 17:05:05+02	2020-10-14 17:06:34+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-14 17:02:37+02	2020-10-14 17:06:53+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-14 17:06:34+02	2020-10-14 17:07:56+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-14 17:07:57+02	2020-10-14 17:08:45+02	databases	powa-archivist	\N	powa-archivist-4.0.1
2020-10-14 17:06:54+02	2020-10-14 17:10:17+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-14 17:08:46+02	2020-10-14 17:10:59+02	deskutils	calibre	\N	calibre-4.23.0_3
2020-10-14 17:10:18+02	2020-10-14 17:10:59+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-10-14 17:10:59+02	2020-10-14 17:21:49+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-14 17:21:50+02	2020-10-14 17:23:11+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-14 17:10:59+02	2020-10-14 17:37:43+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-14 17:37:44+02	2020-10-14 17:38:03+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-10-14 17:38:04+02	2020-10-14 17:38:52+02	devel	tortoisehg	py37	tortoisehg-5.5.1_2
2020-10-14 17:38:53+02	2020-10-14 17:48:44+02	editors	codelite	\N	codelite-12.0_4
2020-10-14 17:23:11+02	2020-10-14 17:50:20+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-14 17:48:44+02	2020-10-14 17:50:37+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.2.2
2020-10-14 17:50:23+02	2020-10-14 17:52:01+02	editors	vim	\N	vim-8.2.1558
2020-10-14 17:50:38+02	2020-10-14 17:59:59+02	emulators	qemu	\N	qemu-4.2.1
2020-10-14 17:58:32+02	2020-10-14 18:06:01+02	emulators	wine	\N	wine-5.0.2_1,1
2020-10-14 18:06:02+02	2020-10-14 18:06:14+02	games	KnightCap	\N	knightcap-3.6_8
2020-10-14 18:06:15+02	2020-10-14 18:06:32+02	games	afternoonstalker	\N	afternoonstalker-1.1.6
2020-10-14 18:06:33+02	2020-10-14 18:08:36+02	games	alienarena	\N	alienarena-2013.766_9
2020-10-14 18:08:37+02	2020-10-14 18:09:02+02	games	alienblaster	\N	alienblaster-1.1.0_7
2020-10-14 18:09:03+02	2020-10-14 18:10:16+02	games	battletanks	\N	btanks-0.9.8083_9
2020-10-14 18:10:17+02	2020-10-14 18:10:32+02	games	maelstrom	\N	maelstrom-3.0.6_14
2020-10-14 18:10:33+02	2020-10-14 18:11:14+02	games	monsterz	\N	monsterz-0.7.1_15
2020-10-14 18:11:15+02	2020-10-14 18:11:50+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-10-14 18:11:51+02	2020-10-14 18:12:31+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-10-14 18:12:32+02	2020-10-14 18:12:44+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-10-14 18:12:44+02	2020-10-14 18:13:22+02	games	xblast	\N	xblast-2.10.4_18
2020-10-14 18:00:00+02	2020-10-14 18:13:42+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-14 18:13:23+02	2020-10-14 18:14:05+02	games	xboard	\N	xboard-4.9.1_1
2020-10-14 18:13:43+02	2020-10-14 18:14:21+02	games	xcowsay	\N	xcowsay-1.5.1
2020-10-14 18:14:21+02	2020-10-14 18:16:30+02	graphics	dia	\N	dia-0.97.3_1,1
2020-10-14 18:16:30+02	2020-10-14 18:17:47+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-14 18:17:48+02	2020-10-14 18:21:45+02	graphics	inkscape	\N	inkscape-1.0.1_1
2020-10-14 18:14:05+02	2020-10-14 18:22:03+02	graphics	blender	\N	blender-2.90.0_1
2020-10-14 18:22:05+02	2020-10-14 18:24:57+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-10-14 18:25:02+02	2020-10-14 18:29:32+02	graphics	wings	\N	wings-2.2.5
2020-10-14 18:29:32+02	2020-10-14 18:30:35+02	graphics	xpdf	\N	xpdf-4.02,1
2020-10-14 18:21:46+02	2020-10-14 18:31:03+02	graphics	krita	\N	krita-4.3.0_6
2020-10-14 18:31:04+02	2020-10-14 18:32:39+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-14 17:03:51+02	2020-10-14 18:59:21+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-14 18:32:41+02	2020-10-14 19:08:52+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.11
2020-10-14 19:08:53+02	2020-10-14 19:10:41+02	math	geogebra	\N	geogebra-5.0.580.0
2020-10-14 19:10:42+02	2020-10-14 19:12:34+02	math	wxmaxima	\N	wxmaxima-20.04.0_2
2020-10-14 19:12:34+02	2020-10-14 19:13:30+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-14 19:13:31+02	2020-10-14 19:13:48+02	multimedia	libva-glx	\N	libva-glx-2.9.0
2020-10-14 19:13:49+02	2020-10-14 19:14:04+02	multimedia	libva-vdpau-driver	\N	libva-vdpau-driver-0.7.4_9
2020-10-14 19:14:04+02	2020-10-14 19:15:53+02	multimedia	obs-studio	\N	obs-studio-26.0.2
2020-10-14 19:15:53+02	2020-10-14 19:18:20+02	multimedia	openshot	py37	openshot-2.5.1.369_1
2020-10-14 18:49:56+02	2020-10-14 19:20:15+02	lang	erlang-runtime23	\N	erlang-runtime23-23.1.1
2020-10-14 19:18:21+02	2020-10-14 19:21:22+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-14 19:20:16+02	2020-10-14 19:22:28+02	net	ntopng	\N	ntopng-4.0.d20200917,1
2020-10-14 19:21:23+02	2020-10-14 19:22:51+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-14 19:22:51+02	2020-10-14 19:23:32+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-14 19:23:32+02	2020-10-14 19:24:37+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-14 19:24:38+02	2020-10-14 19:25:47+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-14 19:25:48+02	2020-10-14 19:27:08+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-14 19:27:09+02	2020-10-14 19:27:56+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-14 19:27:57+02	2020-10-14 19:28:45+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-14 19:22:28+02	2020-10-14 19:28:51+02	net	wireshark	\N	wireshark-3.2.7
2020-10-14 19:28:46+02	2020-10-14 19:29:33+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-14 19:28:52+02	2020-10-14 19:29:42+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-14 19:29:42+02	2020-10-14 19:30:36+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-14 19:29:33+02	2020-10-14 19:30:56+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-14 19:30:37+02	2020-10-14 19:31:22+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-14 19:30:56+02	2020-10-14 19:31:36+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-10-14 19:31:23+02	2020-10-14 19:32:13+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-10-14 19:31:36+02	2020-10-14 19:34:18+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-14 19:32:15+02	2020-10-14 19:37:51+02	print	lyx	\N	lyx-2.3.4.2_1
2020-10-14 19:37:51+02	2020-10-14 19:43:42+02	print	texlive-full	\N	texlive-full-20150521
2020-10-14 19:43:44+02	2020-10-14 19:44:52+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-10-14 19:34:19+02	2020-10-14 19:47:07+02	print	scribus-devel	\N	scribus-devel-1.5.5_20
2020-10-14 19:47:08+02	2020-10-14 19:48:17+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-14 19:48:17+02	2020-10-14 19:48:50+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-10-14 19:48:50+02	2020-10-14 19:52:04+02	sysutils	exa	\N	exa-0.9.0_16
2020-10-14 19:52:04+02	2020-10-14 19:52:41+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-10-14 19:52:41+02	2020-10-14 19:53:54+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-14 19:44:53+02	2020-10-14 19:53:58+02	shells	starship	\N	starship-0.44.0_3
2020-10-14 19:53:54+02	2020-10-14 19:55:54+02	sysutils	hexyl	\N	hexyl-0.8.0_5
2020-10-14 19:55:55+02	2020-10-14 19:57:01+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-10-14 19:53:59+02	2020-10-14 20:02:11+02	sysutils	rsfetch	\N	rsfetch-2.0.0_8
2020-10-14 19:57:02+02	2020-10-14 20:06:02+02	textproc	bat	\N	bat-0.16.0_1
2020-10-14 20:06:04+02	2020-10-14 20:06:19+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-10-14 20:06:20+02	2020-10-14 20:06:35+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-10-14 20:06:38+02	2020-10-14 20:06:51+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-3.1
2020-10-14 20:06:52+02	2020-10-14 20:07:05+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-10-14 20:07:06+02	2020-10-14 20:07:21+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-10-14 20:07:22+02	2020-10-14 20:07:40+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-10-14 20:07:41+02	2020-10-14 20:07:57+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-10-14 20:07:58+02	2020-10-14 20:08:15+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-10-14 20:08:16+02	2020-10-14 20:08:33+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-10-14 20:08:33+02	2020-10-14 20:08:54+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-10-14 20:08:55+02	2020-10-14 20:30:35+02	x11	alacritty	\N	alacritty-0.5.0_3
2020-10-14 20:30:35+02	2020-10-14 20:31:39+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-10-14 20:31:41+02	2020-10-14 20:32:10+02	x11	xlockmore	\N	xlockmore-5.65
2020-10-14 20:32:11+02	2020-10-14 20:34:42+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-10-14 20:34:43+02	2020-10-14 20:41:38+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-14 20:41:39+02	2020-10-14 20:43:27+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-10-14 20:43:28+02	2020-10-14 20:43:44+02	x11-fm	xdiskusage	\N	xdiskusage-1.52
2020-10-14 20:43:44+02	2020-10-14 20:44:21+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-10-14 20:44:21+02	2020-10-14 21:05:31+02	databases	postgresql12-server	\N	postgresql12-server-12.4_1
2020-10-14 21:05:32+02	2020-10-14 21:43:50+02	editors	vscode	\N	vscode-1.46.1
2020-10-14 18:59:22+02	2020-10-14 23:24:46+02	lang	gcc11-devel	\N	gcc11-devel-11.0.0.s20201011
2020-10-14 23:24:54+02	2020-10-14 23:27:57+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-10-14 20:02:12+02	2020-10-14 23:44:20+02	www	firefox	\N	firefox-81.0.2_1,2
2020-10-14 23:52:30+02	2020-10-14 23:56:12+02	sysutils	libudisks	\N	libudisks-2.9.1
2020-10-14 23:56:13+02	2020-10-14 23:57:21+02	devel	gvfs	\N	gvfs-1.46.1
2020-10-14 23:57:21+02	2020-10-15 00:00:14+02	editors	pluma	\N	pluma-1.24.1
2020-10-14 23:58:27+02	2020-10-15 00:00:21+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-15 00:08:55+02	2020-10-15 00:14:13+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-15 00:14:18+02	2020-10-15 00:17:23+02	x11	mate	\N	mate-1.24.1
2020-10-15 00:40:18+02	2020-10-15 00:41:38+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-15 00:40:18+02	2020-10-15 00:46:50+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-15 00:46:50+02	2020-10-15 00:46:57+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-15 08:12:23+02	2020-10-15 08:13:10+02	sysutils	lsof	\N	lsof-4.93.2_13,8
2020-10-15 08:13:12+02	2020-10-15 08:22:06+02	lang	python39	\N	python39-3.9.0_1
2020-10-15 08:22:08+02	2020-10-15 08:22:40+02	devel	py-setuptools	py39	py39-setuptools-44.0.0
2020-10-15 08:12:24+02	2020-10-15 08:22:48+02	lang	python38	\N	python38-3.8.6_1
2020-10-15 08:22:50+02	2020-10-15 08:23:19+02	devel	py-setuptools	py38	py38-setuptools-44.0.0
2020-10-15 08:22:41+02	2020-10-15 08:23:44+02	databases	pspg	\N	pspg-3.1.5
2020-10-15 08:23:44+02	2020-10-15 08:24:03+02	security	lynis	\N	lynis-3.0.0
2020-10-15 08:24:04+02	2020-10-15 08:24:46+02	security	rkhunter	\N	rkhunter-1.4.6
2020-10-15 08:24:47+02	2020-10-15 08:25:29+02	x11-toolkits	py-tkinter	py38	py38-tkinter-3.8.6_6
2020-10-15 08:12:23+02	2020-10-15 08:25:54+02	lang	python37	\N	python37-3.7.9_1
2020-10-15 08:25:31+02	2020-10-15 08:26:09+02	x11-toolkits	py-tkinter	py39	py39-tkinter-3.9.0_6
2020-10-15 08:25:56+02	2020-10-15 08:26:24+02	devel	py-setuptools	py37	py37-setuptools-44.0.0
2020-10-15 08:26:26+02	2020-10-15 08:26:54+02	devel	py-six	py37	py37-six-1.15.0
2020-10-15 08:26:55+02	2020-10-15 08:27:29+02	devel	py-pytz	py37	py37-pytz-2020.1,1
2020-10-15 08:23:20+02	2020-10-15 08:27:47+02	lang	cython	py38	py38-cython-0.29.15
2020-10-15 08:27:30+02	2020-10-15 08:27:59+02	devel	py-pycparser	py37	py37-pycparser-2.20
2020-10-15 08:27:48+02	2020-10-15 08:28:21+02	devel	py-asn1crypto	py37	py37-asn1crypto-1.3.0
2020-10-15 08:26:10+02	2020-10-15 08:28:44+02	devel	ninja	\N	ninja-1.10.1,2
2020-10-15 08:28:23+02	2020-10-15 08:28:49+02	dns	py-idna	py37	py37-idna-2.10
2020-10-15 08:28:01+02	2020-10-15 08:28:52+02	devel	py-cffi	py37	py37-cffi-1.14.3
2020-10-15 08:28:44+02	2020-10-15 08:29:24+02	devel	meson	\N	meson-0.55.3
2020-10-15 08:29:25+02	2020-10-15 08:29:53+02	net	py-pysocks	py37	py37-pysocks-1.7.1
2020-10-15 08:28:50+02	2020-10-15 08:30:08+02	devel	py-babel	py37	py37-Babel-2.8.0
2020-10-15 08:29:57+02	2020-10-15 08:30:22+02	security	py-certifi	py37	py37-certifi-2020.6.20
2020-10-15 08:30:09+02	2020-10-15 08:30:33+02	textproc	py-MarkupSafe	py37	py37-MarkupSafe-1.1.1
2020-10-15 08:28:54+02	2020-10-15 08:30:34+02	security	py-cryptography	py37	py37-cryptography-2.6.1
2020-10-15 08:30:24+02	2020-10-15 08:30:47+02	devel	py-pyparsing	py37	py37-pyparsing-2.4.7
2020-10-15 08:30:35+02	2020-10-15 08:30:58+02	textproc	py-chardet	py37	py37-chardet-3.0.4_3
2020-10-15 08:30:35+02	2020-10-15 08:30:58+02	security	py-openssl	py37	py37-openssl-19.0.0
2020-10-15 08:31:00+02	2020-10-15 08:31:27+02	net	py-urllib3	py37	py37-urllib3-1.25.10,1
2020-10-15 08:31:00+02	2020-10-15 08:31:28+02	devel	py-Jinja2	py37	py37-Jinja2-2.11.2
2020-10-15 08:31:30+02	2020-10-15 08:31:59+02	www	py-requests	py37	py37-requests-2.22.0_2
2020-10-15 08:32:01+02	2020-10-15 08:32:34+02	devel	py-packaging	py37	py37-packaging-20.4
2020-10-15 08:32:35+02	2020-10-15 08:33:19+02	textproc	py-pygments	py37	py37-pygments-2.7.1
2020-10-15 08:33:19+02	2020-10-15 08:33:49+02	graphics	py-imagesize	py37	py37-imagesize-1.1.0
2020-10-15 08:33:50+02	2020-10-15 08:34:17+02	textproc	py-alabaster	py37	py37-alabaster-0.7.6
2020-10-15 08:30:49+02	2020-10-15 08:34:41+02	textproc	py-docutils	py37	py37-docutils-0.15.2
2020-10-15 08:34:18+02	2020-10-15 08:34:50+02	textproc	py-sphinxcontrib-applehelp	py37	py37-sphinxcontrib-applehelp-1.0.2
2020-10-15 08:34:44+02	2020-10-15 08:35:13+02	textproc	py-sphinxcontrib-devhelp	py37	py37-sphinxcontrib-devhelp-1.0.2
2020-10-15 08:34:51+02	2020-10-15 08:35:22+02	textproc	py-sphinxcontrib-htmlhelp	py37	py37-sphinxcontrib-htmlhelp-1.0.3
2020-10-15 08:35:14+02	2020-10-15 08:35:44+02	textproc	py-sphinxcontrib-jsmath	py37	py37-sphinxcontrib-jsmath-1.0.1
2020-10-15 08:35:24+02	2020-10-15 08:35:48+02	textproc	py-sphinxcontrib-qthelp	py37	py37-sphinxcontrib-qthelp-1.0.3
2020-10-15 08:35:44+02	2020-10-15 08:36:07+02	textproc	py-sphinxcontrib-serializinghtml	py37	py37-sphinxcontrib-serializinghtml-1.1.4
2020-10-15 08:31:28+02	2020-10-15 08:36:10+02	lang	cython	py37	py37-cython-0.29.15
2020-10-15 08:35:51+02	2020-10-15 08:36:16+02	x11	xcb-proto	\N	xcb-proto-1.14.1
2020-10-15 08:36:08+02	2020-10-15 08:36:33+02	devel	py-future	py37	py37-future-0.18.2
2020-10-15 08:36:17+02	2020-10-15 08:36:39+02	www	py-beaker	py37	py37-beaker-1.11.0
2020-10-15 08:36:34+02	2020-10-15 08:36:59+02	textproc	py-CommonMark	py37	py37-CommonMark-0.9.1
2020-10-15 08:36:11+02	2020-10-15 08:36:59+02	textproc	py-pystemmer	py37	py37-pystemmer-2.0.0.1
2020-10-15 08:36:41+02	2020-10-15 08:37:06+02	textproc	py-mako	py37	py37-mako-1.0.14
2020-10-15 08:37:00+02	2020-10-15 08:37:21+02	textproc	py-snowballstemmer	py37	py37-snowballstemmer-1.2.1
2020-10-15 08:37:29+02	2020-10-15 08:38:18+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-10-15 08:37:07+02	2020-10-15 08:38:28+02	textproc	py-libxml2	py37	py37-libxml2-2.9.10_1
2020-10-15 08:38:20+02	2020-10-15 08:38:49+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-10-15 08:38:31+02	2020-10-15 08:39:04+02	textproc	itstool	\N	itstool-2.0.6
2020-10-15 08:39:06+02	2020-10-15 08:41:19+02	textproc	gtk-doc	\N	gtk-doc-1.29_1
2020-10-15 08:41:21+02	2020-10-15 08:42:47+02	x11	xkeyboard-config	\N	xkeyboard-config-2.31
2020-10-15 08:42:50+02	2020-10-15 08:44:40+02	x11	libxkbcommon	\N	libxkbcommon-1.0.1
2020-10-15 08:36:59+02	2020-10-15 08:46:24+02	devel	glib20	\N	glib-2.66.1,1
2020-10-15 08:46:28+02	2020-10-15 08:56:15+02	graphics	cairo	\N	cairo-1.16.0_1,3
2020-10-15 08:56:18+02	2020-10-15 09:01:38+02	devel	gobject-introspection	\N	gobject-introspection-1.66.1,1
2020-10-15 09:01:41+02	2020-10-15 09:10:15+02	print	harfbuzz	\N	harfbuzz-2.7.2_1
2020-10-15 09:10:16+02	2020-10-15 09:12:16+02	security	p11-kit	\N	p11-kit-0.23.21
2020-10-15 09:12:18+02	2020-10-15 09:22:55+02	security	gnutls	\N	gnutls-3.6.15
2020-10-15 09:22:59+02	2020-10-15 09:27:23+02	devel	dbus-glib	\N	dbus-glib-0.110
2020-10-15 09:27:27+02	2020-10-15 09:38:35+02	net	avahi-app	\N	avahi-app-0.7_3
2020-10-15 09:38:38+02	2020-10-15 09:45:02+02	print	cups	\N	cups-2.3.3_1
2020-10-15 09:45:06+02	2020-10-15 09:49:06+02	x11-toolkits	pango	\N	pango-1.42.4_5
2020-10-15 09:49:07+02	2020-10-15 09:49:51+02	accessibility	at-spi2-core	\N	at-spi2-core-2.36.0
2020-10-15 09:49:52+02	2020-10-15 09:51:09+02	misc	shared-mime-info	\N	shared-mime-info-2.0
2020-10-15 09:51:10+02	2020-10-15 09:54:15+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_10
2020-10-15 09:54:16+02	2020-10-15 09:55:22+02	graphics	gdk-pixbuf2	\N	gdk-pixbuf2-2.40.0
2020-10-15 09:55:22+02	2020-10-15 09:56:19+02	graphics	poppler	\N	poppler-20.10.0
2020-10-15 09:56:19+02	2020-10-15 09:57:28+02	devel	tex-web2c	\N	tex-web2c-20150521_3
2020-10-15 09:57:29+02	2020-10-15 09:58:12+02	print	harfbuzz-icu	\N	harfbuzz-icu-2.7.2
2020-10-15 09:58:13+02	2020-10-15 09:59:17+02	print	tex-basic-engines	\N	tex-basic-engines-20150521
2020-10-15 09:59:18+02	2020-10-15 10:04:08+02	print	texlive-base	\N	texlive-base-20150521_56
2020-10-15 10:04:09+02	2020-10-15 10:27:57+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-10-15 10:27:58+02	2020-10-15 10:31:27+02	print	tex-formats	\N	tex-formats-20150521_2
2020-10-15 10:31:27+02	2020-10-15 10:54:51+02	devel	gdb	py37	gdb-9.2_1
2020-10-15 10:54:51+02	2020-10-15 10:55:30+02	accessibility	atk	\N	atk-2.36.0
2020-10-15 10:55:31+02	2020-10-15 16:04:52+02	lang	rust	\N	rust-1.47.0
2020-10-15 16:04:55+02	2020-10-15 16:05:22+02	graphics	gtk-update-icon-cache	\N	gtk-update-icon-cache-3.24.20
2020-10-15 08:44:42+02	2020-10-15 16:07:01+02	devel	llvm90	\N	llvm90-9.0.1_2
2020-10-15 16:05:22+02	2020-10-15 16:08:30+02	lang	vala	\N	vala-0.48.10,1
2020-10-15 16:08:31+02	2020-10-15 16:09:23+02	sysutils	polkit	\N	polkit-0.118
2020-10-15 16:09:24+02	2020-10-15 16:10:27+02	devel	libgsf	\N	libgsf-1.14.47_1
2020-10-15 16:10:28+02	2020-10-15 16:11:03+02	textproc	libcroco	\N	libcroco-0.6.13
2020-10-15 16:07:02+02	2020-10-15 16:13:09+02	devel	rust-cbindgen	\N	rust-cbindgen-0.15.0_1
2020-10-15 16:11:03+02	2020-10-15 16:13:22+02	graphics	librsvg2	\N	librsvg2-2.40.21
2020-10-15 16:13:22+02	2020-10-15 16:13:36+02	devel	libgudev	\N	libgudev-234
2020-10-15 16:13:37+02	2020-10-15 16:19:35+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-10-15 16:13:10+02	2020-10-15 16:21:29+02	devel	qt5-core	\N	qt5-core-5.15.0_2
2020-10-15 16:21:30+02	2020-10-15 16:21:40+02	devel	py-evdev	py37	py37-evdev-1.3.0
2020-10-15 08:38:51+02	2020-10-15 16:21:48+02	devel	llvm10	\N	llvm10-10.0.1_1
2020-10-15 16:21:40+02	2020-10-15 16:21:50+02	devel	py-pyudev	py37	py37-pyudev-0.22.0
2020-10-15 16:21:49+02	2020-10-15 16:22:03+02	x11	libwacom	\N	libwacom-1.5
2020-10-15 16:22:03+02	2020-10-15 16:22:27+02	x11	libinput	\N	libinput-1.16.2
2020-10-15 16:22:27+02	2020-10-15 16:23:02+02	accessibility	at-spi2-atk	\N	at-spi2-atk-2.34.2
2020-10-15 16:21:51+02	2020-10-15 16:23:39+02	devel	qt5-dbus	\N	qt5-dbus-5.15.0
2020-10-15 16:23:02+02	2020-10-15 16:24:06+02	x11-themes	adwaita-icon-theme	\N	adwaita-icon-theme-3.38.0
2020-10-15 16:19:36+02	2020-10-15 16:24:50+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-10-15 16:24:06+02	2020-10-15 16:25:15+02	databases	qt5-sql	\N	qt5-sql-5.15.0
2020-10-15 16:25:16+02	2020-10-15 16:25:32+02	devel	desktop-file-utils	\N	desktop-file-utils-0.26
2020-10-15 16:25:33+02	2020-10-15 16:26:51+02	devel	qt5-testlib	\N	qt5-testlib-5.15.0
2020-10-15 16:24:51+02	2020-10-15 16:26:53+02	textproc	docbook-utils	\N	docbook-utils-0.6.14_13
2020-10-15 16:23:40+02	2020-10-15 16:27:29+02	net	qt5-network	\N	qt5-network-5.15.0
2020-10-15 16:26:52+02	2020-10-15 16:29:07+02	multimedia	gstreamer1	\N	gstreamer1-1.16.2
2020-10-15 16:26:56+02	2020-10-15 16:29:36+02	graphics	colord	\N	colord-1.3.5_1
2020-10-15 16:29:07+02	2020-10-15 16:32:38+02	multimedia	gstreamer1-plugins	\N	gstreamer1-plugins-1.16.2_1
2020-10-15 16:27:30+02	2020-10-15 16:35:04+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0_1
2020-10-15 16:29:37+02	2020-10-15 16:35:18+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-10-15 16:35:18+02	2020-10-15 16:35:36+02	devel	gsettings-desktop-schemas	\N	gsettings-desktop-schemas-3.38.0
2020-10-15 16:35:36+02	2020-10-15 16:36:04+02	net	glib-networking	\N	glib-networking-2.66.0
2020-10-15 16:32:38+02	2020-10-15 16:36:58+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-10-15 16:36:04+02	2020-10-15 16:37:01+02	devel	libsoup	\N	libsoup-2.72.0
2020-10-15 16:36:59+02	2020-10-15 16:37:30+02	devel	json-glib	\N	json-glib-1.6.0
2020-10-15 16:37:01+02	2020-10-15 16:38:02+02	textproc	qt5-xml	\N	qt5-xml-5.15.0
2020-10-15 16:37:30+02	2020-10-15 16:38:10+02	devel	dconf	\N	dconf-0.38.0
2020-10-15 16:38:10+02	2020-10-15 16:39:10+02	databases	qt5-sqldrivers-sqlite3	\N	qt5-sqldrivers-sqlite3-5.15.0
2020-10-15 16:35:05+02	2020-10-15 16:39:19+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-10-15 16:39:11+02	2020-10-15 16:39:35+02	devel	libnotify	\N	libnotify-0.7.9_1
2020-10-15 16:38:03+02	2020-10-15 16:41:06+02	multimedia	gstreamer1-plugins-bad	\N	gstreamer1-plugins-bad-1.16.2
2020-10-15 16:39:35+02	2020-10-15 16:43:01+02	devel	py-lxml	py37	py37-lxml-4.5.2
2020-10-15 16:43:02+02	2020-10-15 16:43:13+02	converters	py-webencodings	py37	py37-webencodings-0.5.1
2020-10-15 16:43:13+02	2020-10-15 16:44:26+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-10-15 16:44:26+02	2020-10-15 16:44:40+02	www	py-html5lib	py37	py37-html5lib-1.0.1
2020-10-15 16:44:41+02	2020-10-15 16:44:53+02	devel	py-isodate	py37	py37-isodate-0.6.0
2020-10-15 16:44:53+02	2020-10-15 16:45:48+02	security	libsecret	\N	libsecret-0.20.3
2020-10-15 16:41:06+02	2020-10-15 16:47:11+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-10-15 16:47:12+02	2020-10-15 16:47:31+02	graphics	py-cairo	py37	py37-cairo-1.18.1_1,1
2020-10-15 16:39:19+02	2020-10-15 16:48:20+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-10-15 16:48:21+02	2020-10-15 16:48:37+02	textproc	py-rdflib	py37	py37-rdflib-5.0.0
2020-10-15 16:45:48+02	2020-10-15 16:48:48+02	graphics	graphviz	\N	graphviz-2.44.1_3
2020-10-15 16:48:37+02	2020-10-15 16:48:53+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_2
2020-10-15 16:47:31+02	2020-10-15 16:48:53+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-10-15 16:48:49+02	2020-10-15 16:49:02+02	audio	lv2	\N	lv2-1.18.0_1
2020-10-15 16:48:54+02	2020-10-15 16:49:19+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_2
2020-10-15 16:48:54+02	2020-10-15 16:49:22+02	graphics	frei0r	\N	frei0r-1.6.1
2020-10-15 16:49:20+02	2020-10-15 16:50:07+02	textproc	enchant2	\N	enchant2-2.2.3_4
2020-10-15 16:50:07+02	2020-10-15 16:51:38+02	graphics	graphene	\N	graphene-1.10.2
2020-10-15 16:51:39+02	2020-10-15 16:52:42+02	net	geoclue	\N	geoclue-2.5.5
2020-10-15 16:52:42+02	2020-10-15 16:53:01+02	www	libwpe	\N	libwpe-1.4.0.1
2020-10-15 16:49:02+02	2020-10-15 16:54:59+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-10-15 16:53:02+02	2020-10-15 16:55:17+02	devel	qt5-linguisttools	\N	qt5-linguisttools-5.15.0
2020-10-15 16:55:17+02	2020-10-15 16:55:37+02	www	wpebackend-fdo	\N	wpebackend-fdo-1.4.1
2020-10-15 16:55:38+02	2020-10-15 16:56:11+02	devel	libIDL	\N	libIDL-0.8.14_5
2020-10-15 16:55:00+02	2020-10-15 16:56:39+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2_1
2020-10-15 16:56:11+02	2020-10-15 16:56:48+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-10-15 16:56:48+02	2020-10-15 16:57:46+02	devel	qt5-concurrent	\N	qt5-concurrent-5.15.0
2020-10-15 16:49:23+02	2020-10-15 16:58:19+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-10-15 16:57:46+02	2020-10-15 16:59:24+02	devel	ORBit2	\N	ORBit2-2.14.19_2
2020-10-15 16:58:19+02	2020-10-15 16:59:40+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.74.0
2020-10-15 16:59:24+02	2020-10-15 17:00:18+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-10-15 16:59:41+02	2020-10-15 17:00:18+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-10-15 17:00:18+02	2020-10-15 17:00:27+02	textproc	py-toml	py37	py37-toml-0.10.1
2020-10-15 17:00:28+02	2020-10-15 17:00:37+02	devel	py-setuptools_scm	py37	py37-setuptools_scm-3.4.3_1
2020-10-15 17:00:38+02	2020-10-15 17:01:04+02	sysutils	consolekit2	\N	consolekit2-1.2.1_1
2020-10-15 17:00:19+02	2020-10-15 17:01:28+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-10-15 17:01:04+02	2020-10-15 17:01:41+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-10-15 17:01:29+02	2020-10-15 17:01:51+02	graphics	liblqr-1	\N	liblqr-1-0.4.2
2020-10-15 17:01:41+02	2020-10-15 17:02:04+02	print	libraqm	\N	libraqm-0.6.0
2020-10-15 17:01:51+02	2020-10-15 17:02:23+02	devel	py-sip	py37	py37-sip-4.19.24,1
2020-10-15 16:56:40+02	2020-10-15 17:03:39+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-15 17:03:41+02	2020-10-15 17:04:04+02	databases	py-sqlite3	py37	py37-sqlite3-3.7.9_7
2020-10-15 17:02:24+02	2020-10-15 17:06:36+02	math	py-numpy	py37	py37-numpy-1.16.6,1
2020-10-15 17:06:37+02	2020-10-15 17:07:45+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-10-15 17:04:05+02	2020-10-15 17:08:11+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-10-15 17:08:11+02	2020-10-15 17:08:53+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-10-15 17:02:05+02	2020-10-15 17:10:32+02	graphics	mesa-dri	\N	mesa-dri-20.2.0_1
2020-10-15 17:08:54+02	2020-10-15 17:10:51+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.5
2020-10-15 17:10:52+02	2020-10-15 17:11:59+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-10-15 17:07:46+02	2020-10-15 17:13:37+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-15 17:13:39+02	2020-10-15 17:14:38+02	accessibility	speech-dispatcher	\N	speech-dispatcher-0.8.8_1
2020-10-15 17:11:59+02	2020-10-15 17:15:11+02	devel	py-qt5-core	py37	py37-qt5-core-5.15.0
2020-10-15 17:10:33+02	2020-10-15 17:15:41+02	devel	qt5-script	\N	qt5-script-5.15.0
2020-10-15 17:14:39+02	2020-10-15 17:16:15+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.74.0
2020-10-15 17:15:42+02	2020-10-15 17:16:59+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-10-15 17:16:16+02	2020-10-15 17:17:15+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-10-15 17:15:11+02	2020-10-15 17:17:30+02	x11-servers	xorg-server	\N	xorg-server-1.20.9,1
2020-10-15 17:17:00+02	2020-10-15 17:17:46+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-10-15 17:17:15+02	2020-10-15 17:17:54+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-10-15 17:17:46+02	2020-10-15 17:19:06+02	x11	mate-desktop	\N	mate-desktop-1.24.1
2020-10-15 17:17:31+02	2020-10-15 17:19:29+02	security	gnupg	\N	gnupg-2.2.23
2020-10-15 17:17:54+02	2020-10-15 17:19:32+02	audio	pulseaudio	\N	pulseaudio-13.0_1
2020-10-15 17:19:07+02	2020-10-15 17:19:33+02	comms	qt5-serialport	\N	qt5-serialport-5.15.0
2020-10-15 17:19:29+02	2020-10-15 17:19:39+02	databases	py-sqlparse	py37	py37-sqlparse-0.4.1
2020-10-15 17:19:33+02	2020-10-15 17:21:05+02	devel	kf5-kconfig	\N	kf5-kconfig-5.74.0
2020-10-15 17:21:08+02	2020-10-15 17:22:34+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-15 17:19:39+02	2020-10-15 17:22:43+02	multimedia	gstreamer1-plugins-good	\N	gstreamer1-plugins-good-1.16.2
2020-10-15 17:22:35+02	2020-10-15 17:23:11+02	security	gpgme	\N	gpgme-1.14.0
2020-10-15 17:23:12+02	2020-10-15 17:23:28+02	textproc	asciidoc	\N	asciidoc-9.0.2
2020-10-15 17:19:34+02	2020-10-15 17:23:31+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-10-15 17:22:44+02	2020-10-15 17:23:33+02	sysutils	policykit	\N	policykit-0.9_10
2020-10-15 17:23:33+02	2020-10-15 17:24:02+02	multimedia	gstreamer1-plugins-ugly	\N	gstreamer1-plugins-ugly-1.16.2
2020-10-15 17:23:28+02	2020-10-15 17:24:09+02	archivers	kf5-karchive	\N	kf5-karchive-5.74.0
2020-10-15 17:23:32+02	2020-10-15 17:24:26+02	devel	kf5-ki18n	\N	kf5-ki18n-5.74.0
2020-10-15 17:24:10+02	2020-10-15 17:24:42+02	sysutils	hal	\N	hal-0.5.14_34
2020-10-15 17:24:27+02	2020-10-15 17:24:59+02	www	py-django22	py37	py37-django22-2.2.16
2020-10-15 17:24:03+02	2020-10-15 17:25:21+02	security	gcr	\N	gcr-3.28.0_1
2020-10-15 17:24:42+02	2020-10-15 17:25:30+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-10-15 17:25:31+02	2020-10-15 17:26:15+02	audio	gstreamer1-plugins-ogg	\N	gstreamer1-plugins-ogg-1.16.2
2020-10-15 17:25:22+02	2020-10-15 17:26:24+02	x11-toolkits	vte3	\N	vte3-0.62.1
2020-10-15 17:26:25+02	2020-10-15 17:27:44+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.74.0
2020-10-15 17:27:45+02	2020-10-15 17:27:58+02	devel	py-olefile	py37	py37-olefile-0.46
2020-10-15 17:27:59+02	2020-10-15 17:28:11+02	devel	py-zipp	py37	py37-zipp-3.3.0
2020-10-15 17:26:16+02	2020-10-15 17:29:18+02	devel	git	default	git-2.28.0
2020-10-15 17:29:19+02	2020-10-15 17:30:22+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.74.0
2020-10-15 17:30:23+02	2020-10-15 17:30:42+02	x11-toolkits	py-tkinter	py37	py37-tkinter-3.7.9_6
2020-10-15 17:28:11+02	2020-10-15 17:31:01+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-10-15 17:25:00+02	2020-10-15 17:31:17+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.15.0
2020-10-15 17:31:02+02	2020-10-15 17:31:18+02	audio	gstreamer1-plugins-a52dec	\N	gstreamer1-plugins-a52dec-1.16.2
2020-10-15 17:30:42+02	2020-10-15 17:31:24+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-10-15 17:31:17+02	2020-10-15 17:31:40+02	audio	gstreamer1-plugins-mpg123	\N	gstreamer1-plugins-mpg123-1.16.2
2020-10-15 17:31:18+02	2020-10-15 17:31:46+02	audio	gstreamer1-plugins-vorbis	\N	gstreamer1-plugins-vorbis-1.16.2
2020-10-15 17:31:41+02	2020-10-15 17:31:53+02	devel	py-importlib-metadata	py37	py37-importlib-metadata-2.0.0
2020-10-15 17:31:53+02	2020-10-15 17:32:31+02	graphics	gstreamer1-plugins-png	\N	gstreamer1-plugins-png-1.16.2
2020-10-15 17:31:46+02	2020-10-15 17:32:51+02	devel	sdl20	\N	sdl2-2.0.12_3
2020-10-15 17:32:31+02	2020-10-15 17:33:11+02	graphics	py-pillow	py37	py37-pillow-7.0.0
2020-10-15 17:32:51+02	2020-10-15 17:33:26+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-10-15 17:33:26+02	2020-10-15 17:33:50+02	multimedia	gstreamer1-plugins-dvdread	\N	gstreamer1-plugins-dvdread-1.16.2_1
2020-10-15 17:33:12+02	2020-10-15 17:33:53+02	multimedia	gstreamer1-plugins-dts	\N	gstreamer1-plugins-dts-1.16.2
2020-10-15 17:33:54+02	2020-10-15 17:34:31+02	multimedia	gstreamer1-plugins-theora	\N	gstreamer1-plugins-theora-1.16.2
2020-10-15 17:33:50+02	2020-10-15 17:34:42+02	multimedia	gstreamer1-plugins-resindvd	\N	gstreamer1-plugins-resindvd-1.16.2_1
2020-10-15 17:34:32+02	2020-10-15 17:35:19+02	x11-toolkits	gstreamer1-plugins-pango	\N	gstreamer1-plugins-pango-1.16.2
2020-10-15 17:31:24+02	2020-10-15 17:36:02+02	devel	glibmm	\N	glibmm-2.64.2,1
2020-10-15 17:34:42+02	2020-10-15 17:36:09+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.74.0
2020-10-15 17:35:19+02	2020-10-15 17:36:17+02	devel	kf5-kcrash	\N	kf5-kcrash-5.74.0
2020-10-15 17:36:09+02	2020-10-15 17:36:19+02	devel	py-attrs	py37	py37-attrs-20.2.0
2020-10-15 17:36:17+02	2020-10-15 17:36:26+02	devel	py-yaml	py37	py37-yaml-5.3.1
2020-10-15 17:36:03+02	2020-10-15 17:36:45+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.74.0
2020-10-15 17:36:19+02	2020-10-15 17:36:51+02	graphics	poppler-glib	\N	poppler-glib-20.10.0
2020-10-15 17:36:26+02	2020-10-15 17:36:58+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-10-15 17:36:51+02	2020-10-15 17:37:32+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-10-15 17:36:46+02	2020-10-15 17:37:37+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-10-15 17:36:59+02	2020-10-15 17:37:44+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.74.0
2020-10-15 17:37:38+02	2020-10-15 17:38:22+02	devel	kf5-kauth	\N	kf5-kauth-5.74.0
2020-10-15 17:37:33+02	2020-10-15 17:38:37+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-10-15 17:37:45+02	2020-10-15 17:38:48+02	devel	kf5-kservice	\N	kf5-kservice-5.74.0
2020-10-15 17:38:37+02	2020-10-15 17:39:40+02	net	py-qt5-network	py37	py37-qt5-network-5.15.0
2020-10-15 17:38:48+02	2020-10-15 17:39:46+02	net	remmina	\N	remmina-1.3.9
2020-10-15 17:39:46+02	2020-10-15 17:40:28+02	sysutils	upower	\N	upower-0.99.11
2020-10-15 17:38:23+02	2020-10-15 17:40:30+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-10-15 17:39:41+02	2020-10-15 17:40:40+02	sysutils	bsdisks	\N	bsdisks-0.22
2020-10-15 17:40:31+02	2020-10-15 17:41:04+02	x11	mate-menus	\N	mate-menus-1.24.1
2020-10-15 17:40:41+02	2020-10-15 17:41:35+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.74.0
2020-10-15 17:40:29+02	2020-10-15 17:41:42+02	textproc	libxml++26	\N	libxml++-2.40.1,1
2020-10-15 17:41:42+02	2020-10-15 17:42:28+02	devel	librest	\N	rest-0.7.93_1
2020-10-15 17:41:35+02	2020-10-15 17:42:32+02	devel	libgtop	\N	libgtop-2.40.0
2020-10-15 17:42:29+02	2020-10-15 17:42:42+02	devel	py-dateutil	py37	py37-dateutil-2.8.1
2020-10-15 17:42:32+02	2020-10-15 17:43:05+02	graphics	cairomm	\N	cairomm-1.12.2_4
2020-10-15 17:41:05+02	2020-10-15 17:43:37+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.15.0
2020-10-15 17:43:38+02	2020-10-15 17:44:19+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-15 17:42:43+02	2020-10-15 17:44:28+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-15 17:44:20+02	2020-10-15 17:45:03+02	x11-drivers	xf86-input-keyboard	\N	xf86-input-keyboard-1.9.0_4
2020-10-15 17:44:29+02	2020-10-15 17:45:14+02	x11-drivers	xf86-input-libinput	\N	xf86-input-libinput-0.30.0_1
2020-10-15 17:43:06+02	2020-10-15 17:45:18+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-10-15 17:45:04+02	2020-10-15 17:45:39+02	x11-drivers	xf86-input-mouse	\N	xf86-input-mouse-1.9.3_3
2020-10-15 17:45:15+02	2020-10-15 17:45:53+02	x11-drivers	xf86-video-amdgpu	\N	xf86-video-amdgpu-19.1.0_1
2020-10-15 17:45:39+02	2020-10-15 17:46:15+02	x11-drivers	xf86-video-scfb	\N	xf86-video-scfb-0.0.5_2
2020-10-15 17:45:18+02	2020-10-15 17:46:23+02	x11-drivers	xf86-video-intel	\N	xf86-video-intel-2.99.917.909,1
2020-10-15 17:45:53+02	2020-10-15 17:46:30+02	x11-drivers	xf86-video-vesa	\N	xf86-video-vesa-2.5.0
2020-10-15 17:46:23+02	2020-10-15 17:47:18+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.74.0
2020-10-15 17:46:30+02	2020-10-15 17:47:22+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.74.0
2020-10-15 17:46:16+02	2020-10-15 17:47:42+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.74.0
2020-10-15 17:47:23+02	2020-10-15 17:48:03+02	accessibility	atkmm	\N	atkmm-2.28.0
2020-10-15 17:47:42+02	2020-10-15 17:48:05+02	devel	gamin	\N	gamin-0.1.10_10
2020-10-15 17:47:19+02	2020-10-15 17:48:08+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-10-15 17:48:04+02	2020-10-15 17:48:13+02	devel	py-pyasn1	py37	py37-pyasn1-0.4.7
2020-10-15 17:48:06+02	2020-10-15 17:49:02+02	devel	talloc	\N	talloc-2.3.1
2020-10-15 17:48:14+02	2020-10-15 17:49:28+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-15 17:49:28+02	2020-10-15 17:49:40+02	www	py-tornado	py37	py37-tornado-5.1.1
2020-10-15 17:49:02+02	2020-10-15 17:49:49+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-10-15 17:49:40+02	2020-10-15 17:50:16+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-15 17:49:49+02	2020-10-15 17:50:54+02	x11-servers	xorg-vfbserver	\N	xorg-vfbserver-1.20.9,1
2020-10-15 17:50:16+02	2020-10-15 17:51:13+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.74.0
2020-10-15 17:51:13+02	2020-10-15 17:52:11+02	x11-toolkits	pangomm	\N	pangomm-2.40.1_4
2020-10-15 17:52:11+02	2020-10-15 17:52:39+02	archivers	gcab	\N	gcab-1.4
2020-10-15 17:50:55+02	2020-10-15 17:53:07+02	x11-themes	mate-icon-theme	\N	mate-icon-theme-1.24.0
2020-10-15 17:53:07+02	2020-10-15 17:54:49+02	devel	libgdata	\N	libgdata-0.17.13
2020-10-15 17:52:40+02	2020-10-15 17:55:09+02	databases	tdb	\N	tdb-1.4.3,1
2020-10-15 17:55:10+02	2020-10-15 17:55:20+02	devel	py-cycler	py37	py37-cycler-0.10.0
2020-10-15 17:55:21+02	2020-10-15 17:55:35+02	devel	py-pathlib2	py37	py37-pathlib2-2.3.5
2020-10-15 17:55:35+02	2020-10-15 17:55:48+02	devel	py-pluggy	py37	py37-pluggy-0.13.1
2020-10-15 17:55:48+02	2020-10-15 17:56:01+02	devel	py-pytest-runner	py37	py37-pytest-runner-2.11.1
2020-10-15 17:54:50+02	2020-10-15 17:56:02+02	devel	libpeas	\N	libpeas-1.22.0
2020-10-15 17:56:02+02	2020-10-15 17:56:49+02	math	py-kiwisolver	py37	py37-kiwisolver-1.1.0,1
2020-10-15 17:56:02+02	2020-10-15 17:57:22+02	devel	tevent	\N	tevent-0.10.2
2020-10-15 17:56:49+02	2020-10-15 17:59:38+02	multimedia	gstreamer	\N	gstreamer-0.10.36_6
2020-10-15 17:57:22+02	2020-10-15 18:01:53+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-15 18:01:53+02	2020-10-15 18:03:23+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.74.0
2020-10-15 18:03:23+02	2020-10-15 18:05:10+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-10-15 17:48:08+02	2020-10-15 18:06:57+02	math	vtk8	\N	vtk8-8.2.0
2020-10-15 17:59:38+02	2020-10-15 18:21:41+02	science	py-scipy	py37	py37-scipy-1.2.2_1
2020-10-15 18:05:11+02	2020-10-15 18:24:41+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-10-15 18:21:42+02	2020-10-15 18:25:06+02	x11	xorg	\N	xorg-7.7_3
2020-10-15 18:24:42+02	2020-10-15 18:25:31+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-10-15 18:25:06+02	2020-10-15 18:25:47+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.74.0
2020-10-15 18:25:32+02	2020-10-15 18:26:22+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-15 18:25:48+02	2020-10-15 18:26:27+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-10-15 18:06:58+02	2020-10-15 18:26:29+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-15 18:26:22+02	2020-10-15 18:26:32+02	devel	py-pyrsistent	py37	py37-pyrsistent-0.14.11_1
2020-10-15 18:26:27+02	2020-10-15 18:26:36+02	devel	py-vcversioner	py37	py37-vcversioner-2.16.0.0_1
2020-10-15 18:26:33+02	2020-10-15 18:27:24+02	graphics	gexiv2	\N	gexiv2-0.12.1
2020-10-15 18:26:37+02	2020-10-15 18:27:30+02	mail	gmime30	\N	gmime30-3.2.7
2020-10-15 18:26:30+02	2020-10-15 18:27:39+02	graphics	cogl	\N	cogl-1.22.8
2020-10-15 18:27:40+02	2020-10-15 18:28:11+02	multimedia	libquvi09	\N	libquvi09-0.9.4_4
2020-10-15 18:28:11+02	2020-10-15 18:29:08+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-10-15 18:27:31+02	2020-10-15 18:29:51+02	multimedia	gstreamer-plugins	\N	gstreamer-plugins-0.10.36_13,3
2020-10-15 18:29:52+02	2020-10-15 18:30:34+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.15.0
2020-10-15 18:30:34+02	2020-10-15 18:31:16+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-10-15 18:31:17+02	2020-10-15 18:31:28+02	security	py-bcrypt	py37	py37-bcrypt-3.1.7_1
2020-10-15 18:27:25+02	2020-10-15 18:31:42+02	math	py-matplotlib	py37	py37-matplotlib-3.3.1
2020-10-15 18:31:42+02	2020-10-15 18:32:41+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.74.0
2020-10-15 18:32:42+02	2020-10-15 18:33:12+02	x11	libxklavier	\N	libxklavier-5.3_1,1
2020-10-15 18:33:14+02	2020-10-15 18:36:39+02	x11	mate-panel	\N	mate-panel-1.24.1_1
2020-10-15 18:36:40+02	2020-10-15 18:38:10+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.74.0
2020-10-15 18:38:11+02	2020-10-15 18:39:32+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.74.0
2020-10-15 18:39:33+02	2020-10-15 18:40:14+02	x11-toolkits	pangox-compat	\N	pangox-compat-0.0.2_2
2020-10-15 18:40:14+02	2020-10-15 18:47:56+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-15 18:47:57+02	2020-10-15 18:48:46+02	audio	chromaprint	\N	chromaprint-1.5.0
2020-10-15 18:48:46+02	2020-10-15 18:49:47+02	audio	libmatemixer	\N	libmatemixer-1.24.1
2020-10-15 18:29:10+02	2020-10-15 18:57:03+02	net	samba413	\N	samba413-4.13.0
2020-10-15 18:57:06+02	2020-10-15 18:57:20+02	comms	py-pyserial	py37	py37-pyserial-3.4_1
2020-10-15 18:57:21+02	2020-10-15 18:58:57+02	devel	kf5-knotifications	\N	kf5-knotifications-5.74.0
2020-10-15 18:49:47+02	2020-10-15 19:10:02+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-15 19:10:03+02	2020-10-15 19:10:43+02	devel	py-dbus	py37	py37-dbus-1.2.16
2020-10-15 19:10:43+02	2020-10-15 19:10:59+02	devel	py-decorator	py37	py37-decorator-4.4.2
2020-10-15 19:10:59+02	2020-10-15 19:11:17+02	devel	py-jsonschema	py37	py37-jsonschema-3.2.0
2020-10-15 19:11:18+02	2020-10-15 19:11:34+02	devel	py-lazy-object-proxy	py37	py37-lazy-object-proxy-1.5.1
2020-10-15 19:11:35+02	2020-10-15 19:12:28+02	devel	py-typed-ast	py37	py37-typed-ast-1.4.1
2020-10-15 19:12:28+02	2020-10-15 19:12:47+02	devel	py-wrapt	py37	py37-wrapt-1.12.1
2020-10-15 19:12:48+02	2020-10-15 19:13:07+02	devel	scons	py37	scons-py37-3.1.2
2020-10-15 19:13:09+02	2020-10-15 19:15:39+02	graphics	clutter	\N	clutter-1.26.2
2020-10-15 19:15:39+02	2020-10-15 19:17:20+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-10-15 19:17:21+02	2020-10-15 19:18:02+02	graphics	poppler-utils	\N	poppler-utils-20.10.0
2020-10-15 19:18:03+02	2020-10-15 19:18:31+02	graphics	py-pygraphviz	py37	py37-pygraphviz-1.5
2020-10-15 19:18:31+02	2020-10-15 19:22:59+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-10-15 19:22:59+02	2020-10-15 19:23:38+02	multimedia	totem-pl-parser	\N	totem-pl-parser-3.26.5
2020-10-15 19:23:38+02	2020-10-15 19:24:27+02	net	geocode-glib	\N	geocode-glib-3.26.2
2020-10-15 19:24:28+02	2020-10-15 19:25:16+02	science	py-scikit-sparse	py37	py37-scikit-sparse-0.4.4_3
2020-10-15 19:25:16+02	2020-10-15 19:27:06+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-10-15 19:27:07+02	2020-10-15 19:27:57+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-15 19:27:58+02	2020-10-15 19:28:46+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-10-15 18:58:58+02	2020-10-15 19:32:37+02	devel	llvm11	\N	llvm11-11.0.0
2020-10-15 19:32:38+02	2020-10-15 19:32:55+02	sysutils	py-psutil	py37	py37-psutil-5.7.2
2020-10-15 19:28:46+02	2020-10-15 19:33:22+02	sysutils	libudisks	\N	libudisks-2.9.1
2020-10-15 19:33:22+02	2020-10-15 19:33:37+02	textproc	py-mistune	py37	py37-mistune-0.8.4
2020-10-15 19:32:55+02	2020-10-15 19:34:43+02	textproc	libwpd010	\N	libwpd010-0.10.3_4
2020-10-15 19:33:38+02	2020-10-15 19:34:59+02	x11	libmatekbd	\N	libmatekbd-1.24.1
2020-10-15 19:34:59+02	2020-10-15 19:36:45+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-10-15 19:34:44+02	2020-10-15 19:36:51+02	x11-fm	caja	\N	caja-1.24.0
2020-10-15 19:36:51+02	2020-10-15 19:37:45+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-15 19:36:45+02	2020-10-15 19:38:02+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.74.0
2020-10-15 19:38:03+02	2020-10-15 19:38:41+02	audio	gstreamer1-plugins-chromaprint	\N	gstreamer1-plugins-chromaprint-1.16.2
2020-10-15 19:37:45+02	2020-10-15 19:38:47+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-10-15 19:38:42+02	2020-10-15 19:39:10+02	databases	gom	\N	gom-0.4
2020-10-15 19:39:11+02	2020-10-15 19:39:45+02	devel	dee	\N	dee-1.2.7_17
2020-10-15 19:39:46+02	2020-10-15 19:41:11+02	devel	gvfs	\N	gvfs-1.46.1_1
2020-10-15 18:31:29+02	2020-10-15 19:41:49+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-15 19:41:11+02	2020-10-15 19:42:21+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.74.0
2020-10-15 19:42:21+02	2020-10-15 19:42:33+02	devel	py-astroid	py37	py37-astroid-2.4.2_1
2020-10-15 19:38:48+02	2020-10-15 19:42:37+02	devel	boost-python-libs	py37	py37-boost-libs-1.72.0_1
2020-10-15 19:42:34+02	2020-10-15 19:42:48+02	devel	py-click	py37	py37-click-7.0
2020-10-15 19:41:51+02	2020-10-15 19:42:50+02	devel	kf5-solid	\N	kf5-solid-5.74.0
2020-10-15 19:42:40+02	2020-10-15 19:42:52+02	devel	py-isort	py37	py37-isort-4.3.21
2020-10-15 19:42:48+02	2020-10-15 19:42:57+02	devel	py-mccabe	py37	py37-mccabe-0.6.1
2020-10-15 19:42:51+02	2020-10-15 19:43:01+02	devel	py-ply	py37	py37-ply-3.11
2020-10-15 19:42:53+02	2020-10-15 19:43:02+02	devel	py-pyasn1-modules	py37	py37-pyasn1-modules-0.2.7
2020-10-15 19:42:58+02	2020-10-15 19:43:07+02	devel	py-xdg	py37	py37-xdg-0.26
2020-10-15 19:43:01+02	2020-10-15 19:43:24+02	graphics	libwpg03	\N	libwpg03-0.3.3_1
2020-10-15 19:43:08+02	2020-10-15 19:43:56+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.15.0
2020-10-15 19:43:25+02	2020-10-15 19:44:08+02	mail	gmime26	\N	gmime26-2.6.23_1
2020-10-15 19:44:08+02	2020-10-15 19:45:18+02	math	octomap	\N	octomap-1.9.5.190
2020-10-15 19:45:19+02	2020-10-15 19:45:39+02	math	py-gmpy	py37	py37-gmpy-1.17_1
2020-10-15 19:45:39+02	2020-10-15 19:46:37+02	math	py-networkx	py37	py37-networkx-2.1,1
2020-10-15 19:43:56+02	2020-10-15 19:46:39+02	mail	mailutils	\N	mailutils-3.10_1
2020-10-15 19:46:39+02	2020-10-15 19:47:08+02	net	grilo	\N	grilo-0.3.13
2020-10-15 19:46:38+02	2020-10-15 19:47:09+02	multimedia	libmediaart	\N	libmediaart-1.9.4
2020-10-15 19:43:02+02	2020-10-15 19:48:03+02	graphics	tesseract	\N	tesseract-4.1.1_2
2020-10-15 19:47:08+02	2020-10-15 19:48:12+02	net	libgweather	\N	libgweather-3.36.1
2020-10-15 19:47:10+02	2020-10-15 19:48:23+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-15 19:48:04+02	2020-10-15 19:48:35+02	security	libgnome-keyring	\N	libgnome-keyring-3.12.0_2
2020-10-15 19:48:13+02	2020-10-15 19:48:43+02	sysutils	accountsservice	\N	accountsservice-0.6.42
2020-10-15 19:48:43+02	2020-10-15 19:48:53+02	textproc	py-m2r	py37	py37-m2r-0.2.1_1
2020-10-15 19:48:53+02	2020-10-15 19:49:03+02	www	py-hpack	py37	py37-hpack-3.0.0
2020-10-15 19:49:04+02	2020-10-15 19:49:15+02	www	py-hyperframe	py37	py37-hyperframe-5.2.0
2020-10-15 19:48:35+02	2020-10-15 19:49:22+02	sysutils	tracker	\N	tracker-2.3.4
2020-10-15 19:48:24+02	2020-10-15 19:49:28+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.74.0
2020-10-15 19:49:22+02	2020-10-15 19:49:50+02	x11	babl	\N	babl-0.1.78_1
2020-10-15 19:49:16+02	2020-10-15 19:49:57+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-10-15 19:49:50+02	2020-10-15 19:50:34+02	x11	kf5-kded	\N	kf5-kded-5.74.0
2020-10-15 19:49:28+02	2020-10-15 19:50:38+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-10-15 19:49:58+02	2020-10-15 19:50:50+02	x11	zenity	\N	zenity-3.32.0
2020-10-15 19:50:35+02	2020-10-15 19:51:22+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.74.0
2020-10-15 19:50:39+02	2020-10-15 19:51:38+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-10-15 19:51:39+02	2020-10-15 19:52:01+02	databases	py-psycopg2	py37	py37-psycopg2-2.8.6
2020-10-15 19:52:02+02	2020-10-15 19:52:15+02	databases	py-redis	py37	py37-redis-3.5.3
2020-10-15 19:50:50+02	2020-10-15 19:52:31+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-10-15 19:52:15+02	2020-10-15 19:52:44+02	devel	gstreamer1-plugins-soup	\N	gstreamer1-plugins-soup-1.16.2
2020-10-15 19:52:44+02	2020-10-15 19:53:15+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-10-15 19:53:15+02	2020-10-15 19:53:25+02	devel	py-Automat	py37	py37-Automat-20.2.0
2020-10-15 19:53:25+02	2020-10-15 19:53:35+02	devel	py-appdirs	py37	py37-appdirs-1.4.4
2020-10-15 19:53:36+02	2020-10-15 19:53:45+02	devel	py-backports	py37	py37-backports-1
2020-10-15 19:53:45+02	2020-10-15 19:53:56+02	devel	py-cloudpickle	py37	py37-cloudpickle-1.2.1
2020-10-15 19:53:56+02	2020-10-15 19:54:08+02	devel	py-constantly	py37	py37-constantly-15.1.0
2020-10-15 19:54:08+02	2020-10-15 19:54:27+02	devel	py-defusedxml	py37	py37-defusedxml-0.5.0
2020-10-15 19:54:29+02	2020-10-15 19:54:44+02	devel	py-incremental	py37	py37-incremental-17.5.0
2020-10-15 19:54:44+02	2020-10-15 19:54:57+02	devel	py-jdcal	py37	py37-jdcal-1.4.1
2020-10-15 19:52:31+02	2020-10-15 19:55:01+02	devel	kf5-kio	\N	kf5-kio-5.74.1
2020-10-15 19:55:02+02	2020-10-15 19:55:15+02	devel	py-msgpack	py37	py37-msgpack-0.6.2
2020-10-15 19:55:16+02	2020-10-15 19:55:25+02	devel	py-wheel	py37	py37-wheel-0.30.0_1
2020-10-15 19:51:23+02	2020-10-15 19:55:28+02	databases	postgresql13-server	\N	postgresql13-server-13.0_4
2020-10-15 19:55:25+02	2020-10-15 19:55:35+02	devel	py-zope.interface	py37	py37-zope.interface-4.6.0
2020-10-15 19:54:57+02	2020-10-15 19:55:37+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-15 19:55:28+02	2020-10-15 19:55:40+02	devel	pylint	py37	pylint-py37-2.6.0
2020-10-15 19:55:35+02	2020-10-15 19:55:45+02	devel	spice-protocol	\N	spice-protocol-0.12.15
2020-10-15 19:55:46+02	2020-10-15 19:56:25+02	graphics	clutter-gtk3	\N	clutter-gtk3-1.8.4
2020-10-15 19:56:25+02	2020-10-15 19:56:42+02	graphics	fpc-cairo	\N	fpc-cairo-3.0.4_4
2020-10-15 19:56:42+02	2020-10-15 19:57:55+02	graphics	gegl	\N	gegl-0.4.26_6
2020-10-15 19:55:37+02	2020-10-15 19:58:07+02	dns	bind-tools	\N	bind-tools-9.16.7
2020-10-15 19:57:56+02	2020-10-15 19:58:12+02	graphics	libmypaint	\N	libmypaint-1.5.1_1
2020-10-15 19:58:12+02	2020-10-15 19:58:34+02	graphics	py-pycollada	py37	py37-pycollada-0.6_3
2020-10-15 19:55:40+02	2020-10-15 19:58:47+02	editors	emacs	full	emacs-27.1_1,3
2020-10-15 19:58:48+02	2020-10-15 19:59:03+02	graphics	sdl2_image	\N	sdl2_image-2.0.5
2020-10-15 19:58:34+02	2020-10-15 19:59:12+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-10-15 19:59:03+02	2020-10-15 19:59:49+02	math	fcl05	\N	fcl05-0.5.0
2020-10-15 19:59:50+02	2020-10-15 20:00:09+02	math	py-mpmath	py37	py37-mpmath-1.1.0
2020-10-15 20:00:10+02	2020-10-15 20:00:46+02	multimedia	clutter-gst3	\N	clutter-gst3-3.0.27
2020-10-15 19:59:12+02	2020-10-15 20:00:59+02	math	py-PyWavelets	py37	py37-PyWavelets-1.1.1
2020-10-15 19:58:07+02	2020-10-15 20:01:04+02	graphics	opencv	\N	opencv-3.4.1_36
2020-10-15 20:00:59+02	2020-10-15 20:01:17+02	net	gssdp	\N	gssdp-1.2.3
2020-10-15 20:00:47+02	2020-10-15 20:01:45+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-15 20:01:05+02	2020-10-15 20:01:46+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-15 20:01:17+02	2020-10-15 20:01:52+02	print	cups-filters	\N	cups-filters-1.28.4
2020-10-15 20:01:52+02	2020-10-15 20:02:04+02	security	py-service_identity	py37	py37-service_identity-18.1.0
2020-10-15 20:02:04+02	2020-10-15 20:03:07+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0_1
2020-10-15 20:01:45+02	2020-10-15 20:03:53+02	print	fontforge	\N	fontforge-20190801_2
2020-10-15 20:03:08+02	2020-10-15 20:04:25+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-15 20:04:26+02	2020-10-15 20:04:35+02	textproc	py-et_xmlfile	py37	py37-et_xmlfile-1.0.1
2020-10-15 20:04:35+02	2020-10-15 20:04:45+02	textproc	py-markdown	py37	py37-markdown-2.6.11_1
2020-10-15 20:03:53+02	2020-10-15 20:04:49+02	sysutils	zeitgeist	\N	zeitgeist-1.0.2
2020-10-15 20:04:45+02	2020-10-15 20:04:56+02	textproc	py-numpydoc	py37	py37-numpydoc-0.9.1
2020-10-15 20:04:49+02	2020-10-15 20:05:03+02	textproc	py-pyhamcrest	py37	py37-PyHamcrest-2.0.2
2020-10-15 20:05:04+02	2020-10-15 20:05:16+02	www	py-h2	py37	py37-h2-3.2.0
2020-10-15 20:05:16+02	2020-10-15 20:05:26+02	www	py-hyperlink	py37	py37-hyperlink-20.0.1
2020-10-15 20:05:26+02	2020-10-15 20:05:36+02	www	py-priority	py37	py37-priority-1.3.0
2020-10-15 20:05:36+02	2020-10-15 20:06:23+02	x11	mate-session-manager	\N	mate-session-manager-1.24.1
2020-10-15 20:06:24+02	2020-10-15 20:07:18+02	x11	yelp	\N	yelp-3.38.0
2020-10-15 20:01:47+02	2020-10-15 20:07:29+02	print	tex-luatex	\N	tex-luatex-0.80.0_21
2020-10-15 20:07:18+02	2020-10-15 20:07:51+02	x11-themes	gnome-icon-theme-symbolic	\N	gnome-icon-theme-symbolic-3.12.0
2020-10-15 20:07:29+02	2020-10-15 20:08:22+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-10-15 20:07:52+02	2020-10-15 20:08:37+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-10-15 20:08:22+02	2020-10-15 20:09:44+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-10-15 20:09:44+02	2020-10-15 20:11:10+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-10-15 20:08:38+02	2020-10-15 20:11:16+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-10-15 20:11:17+02	2020-10-15 20:12:22+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-10-15 20:11:11+02	2020-10-15 20:12:47+02	x11-wm	marco	\N	marco-1.24.1
2020-10-15 20:12:48+02	2020-10-15 20:13:54+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-10-15 20:12:23+02	2020-10-15 20:14:19+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-15 20:14:19+02	2020-10-15 20:15:17+02	devel	git-lfs	\N	git-lfs-2.12.0
2020-10-15 20:13:55+02	2020-10-15 20:16:35+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-15 20:16:36+02	2020-10-15 20:16:52+02	devel	py-backports.csv	py37	py37-backports.csv-1.0.7
2020-10-15 20:16:53+02	2020-10-15 20:17:09+02	devel	py-colorlog	py37	py37-colorlog-4.4.0
2020-10-15 20:17:09+02	2020-10-15 20:17:25+02	devel	py-coreschema	py37	py37-coreschema-0.0.4
2020-10-15 20:17:26+02	2020-10-15 20:17:42+02	devel	py-itypes	py37	py37-itypes-1.1.0
2020-10-15 20:17:42+02	2020-10-15 20:17:59+02	devel	py-odfpy	py37	py37-odfpy-1.4.1
2020-10-15 20:17:59+02	2020-10-15 20:18:15+02	devel	py-rtree	py37	py37-rtree-0.9.4
2020-10-15 20:15:18+02	2020-10-15 20:18:38+02	devel	ispc	\N	ispc-1.14.1_2
2020-10-15 20:18:16+02	2020-10-15 20:18:39+02	devel	py-shapely	py37	py37-Shapely-1.6.4
2020-10-15 20:18:40+02	2020-10-15 20:18:55+02	devel	py-xxhash	py37	py37-xxhash-2.0.0
2020-10-15 20:18:39+02	2020-10-15 20:19:34+02	devel	py-twisted	py37	py37-twisted-20.3.0_1
2020-10-15 20:18:56+02	2020-10-15 20:19:47+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-10-15 20:19:35+02	2020-10-15 20:20:28+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-10-15 20:19:47+02	2020-10-15 20:20:37+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-10-15 20:20:29+02	2020-10-15 20:23:06+02	devel	shiboken2	py37	py37-shiboken2-5.15.1
2020-10-15 20:23:07+02	2020-10-15 20:23:56+02	graphics	djvulibre	\N	djvulibre-3.5.27_2
2020-10-15 20:20:38+02	2020-10-15 20:24:13+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-10-15 20:23:57+02	2020-10-15 20:25:40+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-10-15 20:25:41+02	2020-10-15 20:26:13+02	graphics	libgxps	\N	libgxps-0.3.1
2020-10-15 20:26:13+02	2020-10-15 20:27:57+02	graphics	libheif	\N	libheif-1.6.2_1
2020-10-15 20:27:58+02	2020-10-15 20:29:42+02	graphics	poppler-qt5	\N	poppler-qt5-20.10.0
2020-10-15 20:29:45+02	2020-10-15 20:30:44+02	graphics	py-PyOpenGL	py37	py37-PyOpenGL-3.1.5
2020-10-15 20:30:46+02	2020-10-15 20:31:05+02	graphics	py-pyglet	py37	py37-pyglet-1.5.7
2020-10-15 20:24:14+02	2020-10-15 20:31:18+02	graphics	gimp-app	\N	gimp-app-2.10.20_7,1
2020-10-15 20:31:07+02	2020-10-15 20:31:54+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.15.0
2020-10-15 20:31:56+02	2020-10-15 20:32:14+02	graphics	py-svg.path	py37	py37-svg_path-4.0.2
2020-10-15 20:32:14+02	2020-10-15 20:33:09+02	graphics	vigra	\N	vigra-1.11.1_12
2020-10-15 20:04:56+02	2020-10-15 20:34:32+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-15 20:31:18+02	2020-10-15 20:35:08+02	graphics	py-scikit-image	py37	py37-scikit-image-0.17.2
2020-10-15 20:34:35+02	2020-10-15 20:35:20+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-10-15 20:35:21+02	2020-10-15 20:35:49+02	math	py-triangle	py37	py37-triangle-20190115.3
2020-10-15 20:35:09+02	2020-10-15 20:36:25+02	math	py-sympy	py37	py37-sympy-1.6
2020-10-15 20:36:25+02	2020-10-15 20:36:38+02	misc	py-python-utils	py37	py37-python-utils-2.4.0
2020-10-15 20:36:39+02	2020-10-15 20:37:33+02	multimedia	libass	\N	libass-0.14.0
2020-10-15 20:37:34+02	2020-10-15 20:38:39+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-15 20:38:39+02	2020-10-15 20:41:09+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-15 20:35:50+02	2020-10-15 20:42:23+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-15 20:42:23+02	2020-10-15 20:42:58+02	net	gupnp	\N	gupnp-1.2.4
2020-10-15 20:42:58+02	2020-10-15 20:43:30+02	net	libarcus	\N	libArcus-4.7.1,1
2020-10-15 20:41:09+02	2020-10-15 20:43:30+02	net	freerdp	\N	freerdp-2.2.0_1
2020-10-15 20:43:31+02	2020-10-15 20:43:40+02	net	py-ifaddr	py37	py37-ifaddr-0.1.7
2020-10-15 20:43:31+02	2020-10-15 20:43:44+02	net	py-netaddr	py37	py37-netaddr-0.7.19_1
2020-10-15 20:43:40+02	2020-10-15 20:43:50+02	net	py-uritemplate	py37	py37-uritemplate-3.0.1
2020-10-15 20:43:44+02	2020-10-15 20:44:21+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-15 20:43:51+02	2020-10-15 20:44:52+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-15 20:44:22+02	2020-10-15 20:45:02+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-15 20:45:02+02	2020-10-15 20:45:19+02	print	libspectre	\N	libspectre-0.2.9
2020-10-15 20:33:10+02	2020-10-15 20:46:16+02	math	R	\N	R-4.0.3
2020-10-15 20:45:19+02	2020-10-15 20:46:58+02	print	mftrace	\N	mftrace-1.2.18_2
2020-10-15 20:46:59+02	2020-10-15 20:47:29+02	security	py-pycrypto	py37	py37-pycrypto-2.6.1_3
2020-10-15 20:44:52+02	2020-10-15 20:47:34+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-15 20:47:29+02	2020-10-15 20:47:43+02	security	py-pynacl	py37	py37-pynacl-1.4.0
2020-10-15 20:47:34+02	2020-10-15 20:47:48+02	sysutils	hal-info	\N	hal-info-20091130
2020-10-15 20:47:49+02	2020-10-15 20:48:30+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-15 20:47:43+02	2020-10-15 20:49:52+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-15 20:48:30+02	2020-10-15 20:50:04+02	textproc	dblatex	\N	dblatex-0.3.11
2020-10-15 20:49:53+02	2020-10-15 20:50:05+02	textproc	py-openpyxl	py37	py37-openpyxl-2.6.4
2020-10-15 20:50:05+02	2020-10-15 20:50:16+02	textproc	py-xlrd	py37	py37-xlrd-1.2.0_1
2020-10-15 20:50:05+02	2020-10-15 20:50:17+02	textproc	py-xlwt	py37	py37-xlwt-1.3.0
2020-10-15 20:50:17+02	2020-10-15 20:50:33+02	www	py-html5-parser	py37	py37-html5-parser-0.4.9
2020-10-15 20:46:17+02	2020-10-15 20:50:47+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-10-15 20:50:17+02	2020-10-15 20:50:49+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.15.0
2020-10-15 20:50:48+02	2020-10-15 20:50:56+02	www	py-soupsieve	py37	py37-soupsieve-1.9_1
2020-10-15 20:50:50+02	2020-10-15 20:51:09+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-10-15 20:50:34+02	2020-10-15 20:51:25+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.15.0
2020-10-15 20:51:09+02	2020-10-15 20:51:52+02	x11-themes	mate-backgrounds	\N	mate-backgrounds-1.24.2
2020-10-15 20:50:57+02	2020-10-15 20:52:03+02	x11-servers	xephyr	\N	xephyr-1.20.9,1
2020-10-15 20:51:25+02	2020-10-15 20:52:16+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-10-15 20:51:53+02	2020-10-15 20:52:31+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-10-15 20:52:03+02	2020-10-15 20:52:47+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.15.0
2020-10-15 20:52:16+02	2020-10-15 20:53:18+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-10-15 20:52:31+02	2020-10-15 20:53:30+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-10-15 20:52:48+02	2020-10-15 20:53:31+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0_1
2020-10-15 20:53:19+02	2020-10-15 20:54:26+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-10-15 20:54:28+02	2020-10-15 20:56:13+02	archivers	quazip	\N	quazip-qt5-0.9.1
2020-10-15 20:56:14+02	2020-10-15 20:56:53+02	audio	faudio	\N	FAudio-20.10
2020-10-15 20:53:32+02	2020-10-15 20:56:55+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-15 20:56:53+02	2020-10-15 20:59:06+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-15 20:59:07+02	2020-10-15 21:01:44+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-15 20:56:56+02	2020-10-15 21:02:04+02	cad	cura-engine	\N	CuraEngine-4.7.1,1
2020-10-15 20:53:30+02	2020-10-15 21:02:12+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-15 21:02:12+02	2020-10-15 21:02:40+02	databases	py-apsw	py37	py37-apsw-3.33.0
2020-10-15 21:01:45+02	2020-10-15 21:03:13+02	databases	pg_qualstats	\N	pg_qualstats-2.0.2
2020-10-15 21:02:04+02	2020-10-15 21:03:26+02	databases	pg_stat_kcache	\N	pg_stat_kcache-2.1.3
2020-10-15 21:03:14+02	2020-10-15 21:04:08+02	databases	rrdtool	\N	rrdtool-1.7.2_4
2020-10-15 21:02:40+02	2020-10-15 21:04:15+02	databases	py-sqlalchemy10	py37	py37-sqlalchemy10-1.0.14
2020-10-15 21:03:27+02	2020-10-15 21:04:23+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-15 21:04:15+02	2020-10-15 21:04:57+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-15 21:04:23+02	2020-10-15 21:05:08+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-15 21:05:09+02	2020-10-15 21:05:39+02	devel	libsavitar	\N	libSavitar-4.5.0_1
2020-10-15 21:04:57+02	2020-10-15 21:06:00+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.74.0
2020-10-15 21:06:00+02	2020-10-15 21:06:13+02	devel	py-configargparse	py37	py37-configargparse-1.2.3
2020-10-15 21:06:13+02	2020-10-15 21:06:25+02	devel	py-coreapi	py37	py37-coreapi-2.3.3
2020-10-15 21:06:25+02	2020-10-15 21:06:37+02	devel	py-funcy	py37	py37-funcy-1.12
2020-10-15 21:04:08+02	2020-10-15 21:06:45+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-15 21:06:45+02	2020-10-15 21:06:59+02	devel	py-inflection	py37	py37-inflection-0.3.1
2020-10-15 21:05:39+02	2020-10-15 21:07:05+02	devel	mercurial	py37	mercurial-5.5_1
2020-10-15 21:06:59+02	2020-10-15 21:07:09+02	devel	py-iniparse	py37	py37-iniparse-0.5
2020-10-15 21:07:05+02	2020-10-15 21:07:15+02	devel	py-jmespath	py37	py37-jmespath-0.10.0
2020-10-15 21:06:38+02	2020-10-15 21:07:16+02	devel	py-game	py37	py37-game-1.9.6_1
2020-10-15 21:07:15+02	2020-10-15 21:07:25+02	devel	py-rq	py37	py37-rq-1.3.0
2020-10-15 21:07:26+02	2020-10-15 21:07:37+02	devel	py-setproctitle	py37	py37-setproctitle-1.1.10_1
2020-10-15 21:07:16+02	2020-10-15 21:07:40+02	devel	py-ruamel.yaml	py37	py37-ruamel.yaml-0.15.100
2020-10-15 21:07:37+02	2020-10-15 21:07:46+02	devel	py-swagger-spec-validator	py37	py37-swagger-spec-validator-2.5.0
2020-10-15 21:07:41+02	2020-10-15 21:07:49+02	devel	py-tabulate	py37	py37-tabulate-0.8.6
2020-10-15 21:07:10+02	2020-10-15 21:08:02+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.5
2020-10-15 21:08:03+02	2020-10-15 21:08:14+02	dns	py-dnspython	py37	py37-dnspython-1.16.0
2020-10-15 21:07:47+02	2020-10-15 21:08:52+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-15 21:20:58+02	2020-10-15 21:25:41+02	editors	pluma	\N	pluma-1.24.1
2020-10-15 21:25:42+02	2020-10-15 21:27:18+02	graphics	atril	\N	atril-1.22.3_10
2020-10-15 21:27:18+02	2020-10-15 21:28:47+02	graphics	eom	\N	eom-1.24.1
2020-10-15 21:28:47+02	2020-10-15 21:29:58+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-10-15 21:29:59+02	2020-10-15 21:30:43+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-10-15 21:08:14+02	2020-10-15 21:32:04+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-10-15 21:08:53+02	2020-10-15 21:33:57+02	editors	libreoffice	\N	libreoffice-7.0.2.2_2
2020-10-15 21:30:43+02	2020-10-15 21:40:11+02	graphics	openimageio	\N	openimageio-2.2.6.1
2020-10-15 21:40:11+02	2020-10-15 21:40:20+02	lang	python3	\N	python3-3_3
2020-10-15 21:32:05+02	2020-10-15 21:41:43+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-10-15 21:40:21+02	2020-10-15 21:41:57+02	math	mate-calc	\N	mate-calc-1.24.1
2020-10-15 21:34:00+02	2020-10-15 21:42:08+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-15 21:42:09+02	2020-10-15 21:42:37+02	math	py-numpy-stl	py37	py37-numpy-stl-2.11.3
2020-10-15 21:41:57+02	2020-10-15 21:45:58+02	math	py-numpy	py38	py38-numpy-1.16.6,1
2020-10-15 21:42:37+02	2020-10-15 21:46:00+02	math	stp	\N	stp-2.3.3_6,1
2020-10-15 21:41:44+02	2020-10-15 21:47:30+02	math	maxima	\N	maxima-5.44.0_5
2020-10-15 21:47:31+02	2020-10-15 21:48:54+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-10-15 21:48:54+02	2020-10-15 21:50:33+02	multimedia	mpv	\N	mpv-0.32.0_5,1
2020-10-15 21:45:59+02	2020-10-15 21:50:46+02	misc	gimp-help-en	\N	en-gimp-help-html-2.10.0
2020-10-15 21:46:00+02	2020-10-15 21:51:35+02	misc	openvdb	\N	openvdb-7.0.0_5
2020-10-15 21:51:36+02	2020-10-15 21:51:47+02	net	libcharon	\N	libCharon-4.7.1
2020-10-15 21:50:47+02	2020-10-15 21:51:48+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-15 21:51:47+02	2020-10-15 21:51:59+02	net	py-netifaces	py37	py37-netifaces-0.10.9
2020-10-15 21:51:59+02	2020-10-15 21:52:12+02	net	py-zeroconf	py37	py37-zeroconf-0.28.5
2020-10-15 21:51:48+02	2020-10-15 21:52:30+02	net	py-pyzmq	py37	py37-pyzmq-19.0.2
2020-10-15 21:52:13+02	2020-10-15 21:53:10+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-15 21:52:31+02	2020-10-15 21:53:30+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-15 21:53:11+02	2020-10-15 21:53:58+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-15 21:53:30+02	2020-10-15 21:54:15+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-15 21:50:33+02	2020-10-15 21:54:24+02	multimedia	vlc	\N	vlc-3.0.11_5,4
2020-10-15 21:53:58+02	2020-10-15 21:54:38+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-15 21:54:15+02	2020-10-15 21:54:54+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-15 21:54:25+02	2020-10-15 21:55:16+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-15 21:54:38+02	2020-10-15 21:55:44+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-15 21:55:16+02	2020-10-15 21:56:04+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-15 21:56:04+02	2020-10-15 21:56:17+02	net-mgmt	py-prometheus-client	py37	py37-prometheus-client-0.8.0
2020-10-15 21:55:33+02	2020-10-15 21:56:46+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-15 21:55:45+02	2020-10-15 21:56:55+02	net-mgmt	monitoring-plugins	\N	monitoring-plugins-2.2_9
2020-10-15 21:56:17+02	2020-10-15 21:57:25+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-15 21:57:26+02	2020-10-15 21:58:57+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-10-15 21:56:46+02	2020-10-15 22:02:16+02	print	lilypond	\N	lilypond-2.20.0
2020-10-15 21:56:56+02	2020-10-15 22:02:29+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-10-15 22:02:30+02	2020-10-15 22:02:53+02	print	transfig	\N	transfig-3.2.5e_4
2020-10-15 22:02:53+02	2020-10-15 22:03:04+02	security	py-paramiko	py37	py37-paramiko-2.7.1
2020-10-15 22:03:05+02	2020-10-15 22:03:16+02	security	py-stem	py37	py37-stem-1.8.0
2020-10-15 22:03:16+02	2020-10-15 22:03:57+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-15 21:58:58+02	2020-10-15 22:05:07+02	print	tex-xetex	\N	tex-xetex-0.99992_37
2020-10-15 22:03:58+02	2020-10-15 22:05:16+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-15 22:05:17+02	2020-10-15 22:06:04+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-10-15 22:05:08+02	2020-10-15 22:06:36+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-10-15 22:06:04+02	2020-10-15 22:06:57+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.15.0
2020-10-15 22:06:37+02	2020-10-15 22:07:12+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.15.0
2020-10-15 22:07:12+02	2020-10-15 22:07:22+02	textproc	py-scour	py37	py37-scour-0.38.1
2020-10-15 22:06:57+02	2020-10-15 22:07:23+02	textproc	py-regex	py37	py37-regex-2020.7.14
2020-10-15 22:07:23+02	2020-10-15 22:07:33+02	textproc	py-tablib	py37	py37-tablib-0.13.0_1
2020-10-15 22:07:23+02	2020-10-15 22:07:45+02	www	npm-node12	\N	npm-node12-6.14.8
2020-10-15 22:07:34+02	2020-10-15 22:07:53+02	www	py-beautifulsoup	py37	py37-beautifulsoup-4.9.3
2020-10-15 22:07:45+02	2020-10-15 22:07:56+02	www	py-css-parser	py37	py37-css-parser-1.0.4
2020-10-15 22:07:54+02	2020-10-15 22:08:06+02	www	py-cssselect	py37	py37-cssselect-1.1.0
2020-10-15 22:07:56+02	2020-10-15 22:08:07+02	www	py-django-js-asset	py37	py37-django-js-asset-1.2.2_1
2020-10-15 22:08:07+02	2020-10-15 22:08:16+02	www	py-django-taggit	py37	py37-django-taggit-1.2.0_1
2020-10-15 22:08:08+02	2020-10-15 22:08:19+02	www	py-djangorestframework	py37	py37-djangorestframework-3.11.0,1
2020-10-15 22:08:17+02	2020-10-15 22:08:27+02	www	py-mechanize	py37	py37-mechanize-0.4.5
2020-10-15 22:08:20+02	2020-10-15 22:09:15+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.15.0
2020-10-15 22:08:27+02	2020-10-15 22:09:29+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.15.0
2020-10-15 22:09:29+02	2020-10-15 22:10:29+02	x11	mate-base	\N	mate-base-1.24.1
2020-10-15 22:10:30+02	2020-10-15 22:11:42+02	x11	mate-screensaver	\N	mate-screensaver-1.24.1
2020-10-15 22:11:43+02	2020-10-15 22:13:20+02	x11	mate-terminal	\N	mate-terminal-1.24.1
2020-10-15 22:09:16+02	2020-10-15 22:13:40+02	x11	mate-applets	\N	mate-applets-1.24.1
2020-10-15 22:13:40+02	2020-10-15 22:14:34+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-15 22:13:21+02	2020-10-15 22:14:52+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-15 22:14:35+02	2020-10-15 22:15:08+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-10-15 22:15:09+02	2020-10-15 22:16:15+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-10-15 22:14:53+02	2020-10-15 22:16:55+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-10-15 22:02:16+02	2020-10-15 22:21:06+02	print	texlive-docs	\N	texlive-docs-20150523
2020-10-15 22:16:55+02	2020-10-15 22:22:10+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-15 22:16:15+02	2020-10-15 22:22:22+02	astro	stellarium	\N	stellarium-0.20.3
2020-10-15 22:21:07+02	2020-10-15 22:22:50+02	cad	cura	\N	Cura-4.7.1,2
2020-10-15 22:22:23+02	2020-10-15 22:24:24+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-15 22:24:25+02	2020-10-15 22:26:27+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-15 22:22:10+02	2020-10-15 22:27:06+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-15 22:26:28+02	2020-10-15 22:27:55+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-15 22:27:56+02	2020-10-15 22:28:54+02	databases	powa-archivist	\N	powa-archivist-4.0.1
2020-10-15 22:28:54+02	2020-10-15 22:29:30+02	databases	powa-web	py37	py37-powa-web-4.0.2
2020-10-15 22:27:07+02	2020-10-15 22:32:43+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-15 22:32:43+02	2020-10-15 22:33:31+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-10-15 22:29:30+02	2020-10-15 22:33:32+02	deskutils	calibre	\N	calibre-4.23.0_3
2020-10-15 22:33:33+02	2020-10-15 22:51:52+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-15 22:51:55+02	2020-10-15 22:53:38+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-15 22:33:32+02	2020-10-15 23:02:07+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-15 23:02:08+02	2020-10-15 23:03:25+02	devel	libarea	\N	libarea-20160313_13
2020-10-15 22:53:39+02	2020-10-15 23:28:39+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-15 23:28:42+02	2020-10-15 23:29:00+02	devel	py-django-rq	py37	py37-django-rq-2.3.2
2020-10-15 23:29:01+02	2020-10-15 23:29:39+02	devel	tig	\N	tig-2.5.1
2020-10-15 23:29:40+02	2020-10-15 23:30:32+02	devel	tortoisehg	py37	tortoisehg-5.5.1_2
2020-10-15 23:30:33+02	2020-10-15 23:35:22+02	dns	bind916	\N	bind916-9.16.7_1
2020-10-15 23:35:22+02	2020-10-15 23:46:52+02	editors	codelite	\N	codelite-12.0_4
2020-10-15 23:46:53+02	2020-10-15 23:49:26+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.2.2
2020-10-15 23:49:27+02	2020-10-15 23:51:44+02	editors	vim	\N	vim-8.2.1558
2020-10-15 23:51:44+02	2020-10-16 00:03:25+02	emulators	qemu	\N	qemu-4.2.1
2020-10-16 00:11:58+02	2020-10-16 00:24:00+02	emulators	wine	\N	wine-5.0.2_1,1
2020-10-15 22:22:50+02	2020-10-16 00:32:05+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-16 00:32:07+02	2020-10-16 00:32:52+02	games	monsterz	\N	monsterz-0.7.1_15
2020-10-16 00:32:53+02	2020-10-16 00:33:36+02	games	nuclearchess	\N	nuclearchess-1.0.0_10
2020-10-16 00:33:37+02	2020-10-16 00:34:24+02	games	numptyphysics	\N	numptyphysics-0.3.r20090104_11
2020-10-16 00:34:24+02	2020-10-16 00:35:35+02	games	xboard	\N	xboard-4.9.1_1
2020-10-16 00:35:36+02	2020-10-16 00:36:39+02	games	xcowsay	\N	xcowsay-1.5.1
2020-10-16 00:24:02+02	2020-10-16 00:41:39+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-16 00:41:40+02	2020-10-16 00:45:39+02	graphics	dia	\N	dia-0.97.3_1,1
2020-10-16 00:45:40+02	2020-10-16 00:47:11+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-16 00:36:40+02	2020-10-16 00:52:11+02	graphics	blender	\N	blender-2.90.0_1
2020-10-16 00:47:11+02	2020-10-16 00:58:17+02	graphics	inkscape	\N	inkscape-1.0.1_1
2020-10-16 00:58:17+02	2020-10-16 01:09:32+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-10-16 01:09:33+02	2020-10-16 01:09:46+02	graphics	py-svgwrite	py37	py37-svgwrite-1.3.1
2020-10-16 00:52:12+02	2020-10-16 01:12:25+02	graphics	krita	\N	krita-4.3.0_6
2020-10-16 01:09:46+02	2020-10-16 01:13:34+02	graphics	wings	\N	wings-2.2.5
2020-10-16 01:12:26+02	2020-10-16 01:13:48+02	graphics	xpdf	\N	xpdf-4.02,1
2020-10-16 01:13:36+02	2020-10-16 01:16:34+02	irc	irssi	\N	irssi-1.2.2_1,1
2020-10-16 01:16:36+02	2020-10-16 01:19:30+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-16 01:19:32+02	2020-10-16 02:01:33+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.11
2020-10-16 01:38:39+02	2020-10-16 02:17:36+02	lang	erlang-runtime23	\N	erlang-runtime23-23.1.1
2020-10-16 02:17:37+02	2020-10-16 02:17:49+02	lang	python	\N	python-3.7_3,2
2020-10-16 02:17:50+02	2020-10-16 02:19:55+02	math	geogebra	\N	geogebra-5.0.580.0
2020-10-16 02:19:56+02	2020-10-16 02:23:32+02	math	wxmaxima	\N	wxmaxima-20.04.0_2
2020-10-16 02:23:32+02	2020-10-16 02:26:55+02	misc	mc	\N	mc-nox11-4.8.24
2020-10-16 02:26:56+02	2020-10-16 02:28:07+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-16 02:28:07+02	2020-10-16 02:30:24+02	multimedia	obs-studio	\N	obs-studio-26.0.2
2020-10-16 02:30:25+02	2020-10-16 02:33:20+02	multimedia	openshot	py37	openshot-2.5.1.369_1
2020-10-16 02:33:22+02	2020-10-16 02:37:51+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-16 02:37:54+02	2020-10-16 02:41:01+02	net	ntopng	\N	ntopng-4.0.d20200917,1
2020-10-16 02:41:01+02	2020-10-16 02:41:53+02	net	rdesktop	\N	rdesktop-1.9.0
2020-10-16 02:41:53+02	2020-10-16 02:43:43+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-16 02:43:44+02	2020-10-16 02:52:24+02	net	wireshark	\N	wireshark-3.2.7
2020-10-16 02:52:25+02	2020-10-16 02:53:13+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-16 02:01:33+02	2020-10-16 02:54:45+02	lang	mono	\N	mono-5.10.1.57_3
2020-10-16 02:54:47+02	2020-10-16 02:55:42+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-16 02:55:43+02	2020-10-16 02:56:40+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-16 02:56:40+02	2020-10-16 02:57:36+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-16 02:57:36+02	2020-10-16 02:58:35+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-16 02:58:36+02	2020-10-16 02:59:29+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-16 02:59:29+02	2020-10-16 03:00:24+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-16 03:00:24+02	2020-10-16 03:01:22+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-16 03:01:23+02	2020-10-16 03:03:11+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-16 03:03:12+02	2020-10-16 03:04:41+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-16 03:04:41+02	2020-10-16 03:05:41+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-16 03:05:42+02	2020-10-16 03:07:03+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-10-16 03:07:03+02	2020-10-16 03:08:13+02	ports-mgmt	porttree	\N	porttree-1.1.r114_1
2020-10-16 03:08:14+02	2020-10-16 03:12:23+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-16 03:12:24+02	2020-10-16 03:13:38+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-10-16 03:13:38+02	2020-10-16 03:14:55+02	print	gv	\N	gv-3.7.4_5
2020-10-16 02:53:14+02	2020-10-16 03:17:23+02	net-mgmt	icinga2	\N	icinga2-2.12.0
2020-10-16 03:14:56+02	2020-10-16 03:24:44+02	print	lyx	\N	lyx-2.3.4.2_1
2020-10-16 03:24:45+02	2020-10-16 03:31:50+02	print	texlive-full	\N	texlive-full-20150521
2020-10-16 03:31:51+02	2020-10-16 03:39:01+02	security	klee	\N	klee-2.1_1
2020-10-16 03:17:23+02	2020-10-16 03:39:10+02	print	scribus-devel	\N	scribus-devel-1.5.5_20
2020-10-16 03:39:05+02	2020-10-16 03:39:23+02	security	nyx	\N	nyx-2.1.0_1
2020-10-16 03:39:11+02	2020-10-16 03:40:25+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-10-16 03:40:25+02	2020-10-16 03:40:57+02	security	py-xkcdpass	py37	py37-xkcdpass-1.17.4
2020-10-16 03:39:23+02	2020-10-16 03:41:17+02	security	py-pycryptodome	py37	py37-pycryptodome-3.9.8
2020-10-16 03:41:20+02	2020-10-16 03:41:40+02	sysutils	bashtop	\N	bashtop-0.9.25_1
2020-10-16 03:41:40+02	2020-10-16 03:41:56+02	sysutils	bpytop	\N	bpytop-1.0.42
2020-10-16 03:41:57+02	2020-10-16 03:43:03+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-16 03:43:04+02	2020-10-16 03:43:31+02	sysutils	dtrace-toolkit	\N	dtrace-toolkit-1.0_5
2020-10-16 03:40:57+02	2020-10-16 03:43:59+02	sysutils	ansible	py37	py37-ansible-2.9.7
2020-10-16 03:43:32+02	2020-10-16 03:44:16+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-10-16 03:44:00+02	2020-10-16 03:45:44+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-16 03:44:16+02	2020-10-16 03:45:58+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-16 03:45:59+02	2020-10-16 03:46:19+02	sysutils	panicmail	\N	panicmail-1.1
2020-10-16 03:45:45+02	2020-10-16 03:46:23+02	sysutils	htop	\N	htop-3.0.2
2020-10-16 03:46:23+02	2020-10-16 03:46:35+02	sysutils	py-py-cpuinfo	py37	py37-py-cpuinfo-7.0.0
2020-10-16 03:46:19+02	2020-10-16 03:46:38+02	sysutils	py-glances	py37	py37-glances-3.1.4
2020-10-16 03:46:38+02	2020-10-16 03:47:02+02	textproc	py-gfm	py37	py37-gfm-0.1.4
2020-10-16 03:46:36+02	2020-10-16 03:47:40+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-10-16 03:47:41+02	2020-10-16 03:47:52+02	www	py-bottle	py37	py37-bottle-0.12.18
2020-10-16 03:47:53+02	2020-10-16 03:48:06+02	www	py-django-cacheops	py37	py37-django-cacheops-4.2_1
2020-10-16 03:48:07+02	2020-10-16 03:48:23+02	www	py-django-cors-headers	py37	py37-django-cors-headers-3.3.0
2020-10-16 03:48:25+02	2020-10-16 03:48:39+02	www	py-django-debug-toolbar	py37	py37-django-debug-toolbar-3.1
2020-10-16 03:48:41+02	2020-10-16 03:48:55+02	www	py-django-filter	py37	py37-django-filter-2.2.0_1
2020-10-16 03:48:55+02	2020-10-16 03:49:09+02	www	py-django-mptt	py37	py37-django-mptt-0.11.0
2020-10-16 03:49:09+02	2020-10-16 03:49:22+02	www	py-django-pglocks	py37	py37-django-pglocks-1.0.4
2020-10-16 03:49:22+02	2020-10-16 03:49:45+02	www	py-django-prometheus	py37	py37-django-prometheus-1.1.0_2
2020-10-16 03:49:45+02	2020-10-16 03:50:00+02	www	py-django-tables2	py37	py37-django-tables2-2.2.1_1
2020-10-16 03:50:00+02	2020-10-16 03:50:12+02	www	py-django-taggit-serializer	py37	py37-django-taggit-serializer-0.1.7
2020-10-16 03:50:12+02	2020-10-16 03:50:26+02	www	py-django-timezone-field	py37	py37-django-timezone-field-4.0
2020-10-16 03:50:27+02	2020-10-16 03:50:50+02	www	py-drf-yasg	py37	py37-drf-yasg-1.17.0_1
2020-10-16 03:50:50+02	2020-10-16 03:51:09+02	www	py-gunicorn	py37	py37-gunicorn-19.9.0_1
2020-10-16 03:51:09+02	2020-10-16 03:52:15+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-10-16 03:52:16+02	2020-10-16 03:54:27+02	x11	mate	\N	mate-1.24.1
2020-10-16 03:54:27+02	2020-10-16 03:57:35+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-10-16 03:57:36+02	2020-10-16 04:04:52+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-16 04:04:53+02	2020-10-16 04:06:40+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-10-16 04:06:40+02	2020-10-16 04:07:25+02	x11-toolkits	libgdiplus	\N	libgdiplus-5.6_2
2020-10-16 04:07:25+02	2020-10-16 04:14:03+02	databases	postgresql12-server	\N	postgresql12-server-12.4_1
2020-10-16 04:14:04+02	2020-10-16 04:54:36+02	editors	vscode	\N	vscode-1.46.1
2020-10-16 04:54:38+02	2020-10-16 04:56:21+02	graphics	xfig	\N	xfig-3.2.7a
2020-10-16 05:21:16+02	2020-10-16 05:22:01+02	net-mgmt	netbox	\N	netbox-2.7.12_1
2020-10-16 03:47:03+02	2020-10-16 06:49:29+02	www	firefox	\N	firefox-82.0,2
2020-10-15 23:03:27+02	2020-10-16 07:49:16+02	devel	llvm-devel	\N	llvm-devel-12.0.d20200925
2020-10-16 20:44:46+02	2020-10-16 20:46:01+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-16 20:44:46+02	2020-10-16 20:51:50+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-16 20:51:51+02	2020-10-16 20:51:58+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-16 22:01:57+02	2020-10-16 22:02:12+02	x11-toolkits	gnome-pty-helper	\N	gnome-pty-helper-0.40.2
2020-10-16 22:01:57+02	2020-10-16 22:02:14+02	devel	pygobject3-common	\N	pygobject3-common-3.28.3_3
2020-10-16 22:02:12+02	2020-10-16 22:02:18+02	devel	geany-themes	\N	geany-themes-1.24_1
2020-10-16 22:02:14+02	2020-10-16 22:02:40+02	devel	py-gobject3	py37	py37-gobject3-3.28.3_3
2020-10-16 22:01:57+02	2020-10-16 22:03:31+02	ftp	curl	\N	curl-7.73.0
2020-10-16 22:02:19+02	2020-10-16 22:03:36+02	x11-toolkits	vte	\N	vte-0.28.2_3
2020-10-16 22:02:41+02	2020-10-16 22:04:17+02	graphics	graphene	\N	graphene-1.10.2
2020-10-16 22:04:17+02	2020-10-16 22:06:21+02	graphics	gstreamer1-plugins-gl	\N	gstreamer1-plugins-gl-1.16.2_1
2020-10-16 22:06:21+02	2020-10-16 22:06:45+02	devel	ruby-gems	\N	ruby27-gems-3.0.8
2020-10-16 22:06:47+02	2020-10-16 22:07:26+02	ftp	php74-curl	\N	php74-curl-7.4.11
2020-10-16 22:07:27+02	2020-10-16 22:07:44+02	sysutils	dmidecode	\N	dmidecode-3.3
2020-10-16 22:03:37+02	2020-10-16 22:08:06+02	devel	geany	gtk3	geany-1.36
2020-10-16 22:08:07+02	2020-10-16 22:08:19+02	textproc	rubygem-asciidoctor	\N	rubygem-asciidoctor-2.0.10
2020-10-16 22:08:20+02	2020-10-16 22:09:10+02	sysutils	hal	\N	hal-0.5.14_34
2020-10-16 22:07:45+02	2020-10-16 22:09:11+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-16 22:09:11+02	2020-10-16 22:10:05+02	x11	xterm	\N	xterm-361
2020-10-16 22:10:06+02	2020-10-16 22:10:26+02	lang	lua53	\N	lua53-5.3.6
2020-10-16 22:09:11+02	2020-10-16 22:12:02+02	devel	git	default	git-2.28.0
2020-10-16 22:12:02+02	2020-10-16 22:13:20+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-10-16 22:10:26+02	2020-10-16 22:14:05+02	multimedia	webcamd	\N	webcamd-5.7.1.1_1
2020-10-16 22:14:06+02	2020-10-16 22:14:25+02	net	liboauth	\N	liboauth-1.0.3_4
2020-10-16 22:14:25+02	2020-10-16 22:16:11+02	x11-drivers	xf86-input-evdev	\N	xf86-input-evdev-2.10.6_6
2020-10-16 22:16:11+02	2020-10-16 22:16:37+02	x11	xorg-apps	\N	xorg-apps-7.7_4
2020-10-16 22:16:38+02	2020-10-16 22:17:34+02	x11-drivers	xorg-drivers	\N	xorg-drivers-7.7_6
2020-10-16 22:13:21+02	2020-10-16 22:19:48+02	lang	go	\N	go-1.15.3,1
2020-10-16 22:17:35+02	2020-10-16 22:20:48+02	x11	xorg	\N	xorg-7.7_3
2020-10-16 22:19:51+02	2020-10-16 22:21:02+02	devel	geany-plugin-utilslib	gtk3	geany-plugin-utilslib-1.36
2020-10-16 22:20:49+02	2020-10-16 22:21:27+02	multimedia	libquvi09	\N	libquvi09-0.9.4_4
2020-10-16 22:21:27+02	2020-10-16 22:22:03+02	multimedia	totem-pl-parser	\N	totem-pl-parser-3.26.5
2020-10-16 22:22:04+02	2020-10-16 22:22:42+02	databases	gom	\N	gom-0.4
2020-10-16 22:22:43+02	2020-10-16 22:23:28+02	devel	dee	\N	dee-1.2.7_17
2020-10-16 22:23:29+02	2020-10-16 22:23:46+02	devel	rubygem-mustache	\N	rubygem-mustache-1.1.1
2020-10-16 22:23:47+02	2020-10-16 22:24:39+02	net	grilo	\N	grilo-0.3.13
2020-10-16 22:24:40+02	2020-10-16 22:25:07+02	textproc	rubygem-rdiscount	\N	rubygem-rdiscount-2.2.0.1
2020-10-16 22:25:08+02	2020-10-16 22:26:13+02	www	rubygem-hpricot	\N	rubygem-hpricot-0.8.6
2020-10-16 22:26:13+02	2020-10-16 22:27:21+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-16 22:03:31+02	2020-10-16 22:27:26+02	devel	cmake	\N	cmake-3.18.4
2020-10-16 22:27:21+02	2020-10-16 22:27:39+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-10-16 22:27:39+02	2020-10-16 22:54:17+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-16 22:54:18+02	2020-10-16 22:55:59+02	devel	kf5-extra-cmake-modules	\N	kf5-extra-cmake-modules-5.75.0
2020-10-16 22:55:59+02	2020-10-16 23:08:41+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-16 23:08:41+02	2020-10-16 23:11:10+02	devel	kf5-kcoreaddons	\N	kf5-kcoreaddons-5.75.0
2020-10-16 23:11:11+02	2020-10-16 23:13:52+02	devel	kf5-kconfig	\N	kf5-kconfig-5.75.0
2020-10-16 23:13:53+02	2020-10-16 23:15:12+02	archivers	kf5-karchive	\N	kf5-karchive-5.75.0
2020-10-16 23:15:13+02	2020-10-16 23:17:17+02	devel	kf5-ki18n	\N	kf5-ki18n-5.75.0
2020-10-16 23:17:18+02	2020-10-16 23:19:17+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.75.0
2020-10-16 23:19:17+02	2020-10-16 23:21:31+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.75.0
2020-10-16 23:21:32+02	2020-10-16 23:26:01+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.75.0
2020-10-16 23:26:02+02	2020-10-16 23:27:16+02	devel	kf5-kcrash	\N	kf5-kcrash-5.75.0
2020-10-16 23:27:16+02	2020-10-16 23:28:41+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.75.0
2020-10-16 23:28:41+02	2020-10-16 23:30:18+02	textproc	kf5-kcodecs	\N	kf5-kcodecs-5.75.0
2020-10-16 23:30:19+02	2020-10-16 23:37:15+02	databases	mysql57-client	\N	mysql57-client-5.7.31_1
2020-10-16 23:37:16+02	2020-10-16 23:39:12+02	devel	kf5-kauth	\N	kf5-kauth-5.75.0
2020-10-16 23:39:13+02	2020-10-16 23:42:20+02	devel	kf5-kservice	\N	kf5-kservice-5.75.0
2020-10-16 23:42:21+02	2020-10-16 23:43:57+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.75.0
2020-10-16 23:43:57+02	2020-10-16 23:46:57+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-16 23:46:57+02	2020-10-16 23:49:06+02	x11-themes	kf5-breeze-icons	\N	kf5-breeze-icons-5.75.0
2020-10-16 23:49:07+02	2020-10-16 23:51:23+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.75.0
2020-10-16 23:51:24+02	2020-10-16 23:53:07+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.75.0
2020-10-16 23:53:08+02	2020-10-16 23:55:55+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-16 23:55:56+02	2020-10-16 23:57:48+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.75.0
2020-10-16 23:57:49+02	2020-10-17 00:00:23+02	devel	libgdata	\N	libgdata-0.17.13
2020-10-17 00:00:24+02	2020-10-17 00:02:51+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.75.0
2020-10-17 00:02:51+02	2020-10-17 00:04:35+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.75.0
2020-10-17 00:04:35+02	2020-10-17 00:05:20+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-10-17 00:05:21+02	2020-10-17 00:06:51+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-10-17 00:06:52+02	2020-10-17 00:09:13+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.75.0
2020-10-17 00:09:14+02	2020-10-17 00:12:35+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.75.0
2020-10-17 00:12:35+02	2020-10-17 00:14:46+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.75.0
2020-10-17 00:14:47+02	2020-10-17 00:23:36+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-16 22:21:02+02	2020-10-17 00:24:28+02	www	node	\N	node-14.13.1
2020-10-17 00:24:30+02	2020-10-17 00:26:48+02	devel	kf5-knotifications	\N	kf5-knotifications-5.75.0
2020-10-17 00:26:48+02	2020-10-17 00:30:29+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.75.0
2020-10-17 00:30:32+02	2020-10-17 00:32:50+02	devel	gvfs	\N	gvfs-1.46.1_1
2020-10-17 00:32:51+02	2020-10-17 00:35:05+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.75.0
2020-10-17 00:35:06+02	2020-10-17 00:39:05+02	devel	kf5-solid	\N	kf5-solid-5.75.0
2020-10-17 00:39:06+02	2020-10-17 00:42:07+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.75.0
2020-10-17 00:42:08+02	2020-10-17 00:43:21+02	x11	kf5-kded	\N	kf5-kded-5.75.0
2020-10-17 00:23:37+02	2020-10-17 00:44:12+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-17 00:43:22+02	2020-10-17 00:44:35+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.75.0
2020-10-17 00:44:13+02	2020-10-17 00:45:39+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-17 00:45:40+02	2020-10-17 00:46:44+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-17 00:46:45+02	2020-10-17 00:48:25+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-10-17 00:48:27+02	2020-10-17 00:48:45+02	textproc	rubygem-ronn	\N	rubygem-ronn-0.7.3
2020-10-17 00:44:35+02	2020-10-17 01:00:45+02	devel	kf5-kio	\N	kf5-kio-5.75.0
2020-10-17 01:00:48+02	2020-10-17 01:02:32+02	x11	yelp	\N	yelp-3.38.0
2020-10-17 01:02:32+02	2020-10-17 01:03:52+02	devel	git-lfs	\N	git-lfs-2.12.0
2020-10-17 01:03:53+02	2020-10-17 01:26:43+02	math	R	\N	R-4.0.3
2020-10-17 01:26:44+02	2020-10-17 01:38:06+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-17 01:38:06+02	2020-10-17 01:41:28+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-17 01:41:29+02	2020-10-17 01:59:28+02	net	libcmis	\N	libcmis-0.5.2_1
2020-10-17 01:59:29+02	2020-10-17 02:01:27+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-17 02:01:28+02	2020-10-17 02:02:38+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-17 02:02:39+02	2020-10-17 02:03:05+02	sysutils	hal-info	\N	hal-info-20091130
2020-10-17 02:03:05+02	2020-10-17 02:03:46+02	textproc	ctpl	\N	ctpl-0.3.4
2020-10-17 02:03:46+02	2020-10-17 02:04:42+02	textproc	enchant	\N	enchant-1.6.0_9
2020-10-17 02:04:42+02	2020-10-17 02:06:04+02	textproc	redland	\N	redland-1.0.17_4
2020-10-17 02:06:05+02	2020-10-17 02:14:43+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-17 02:14:43+02	2020-10-17 02:15:51+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-17 02:15:51+02	2020-10-17 02:17:00+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-17 02:17:02+02	2020-10-17 02:18:49+02	devel	geany-plugin-addons	gtk3	geany-plugin-addons-1.36
2020-10-17 02:18:49+02	2020-10-17 02:20:21+02	devel	geany-plugin-autoclose	gtk3	geany-plugin-autoclose-1.36
2020-10-17 02:20:21+02	2020-10-17 02:21:51+02	devel	geany-plugin-automark	gtk3	geany-plugin-automark-1.36
2020-10-17 02:21:51+02	2020-10-17 02:23:26+02	devel	geany-plugin-codenav	gtk3	geany-plugin-codenav-1.36
2020-10-17 02:23:27+02	2020-10-17 02:24:41+02	devel	geany-plugin-commander	gtk3	geany-plugin-commander-1.36
2020-10-17 00:48:45+02	2020-10-17 02:24:43+02	www	node12	\N	node12-12.19.0
2020-10-17 02:24:41+02	2020-10-17 02:25:48+02	devel	geany-plugin-ctags	gtk3	geany-plugin-ctags-1.36
2020-10-17 02:24:44+02	2020-10-17 02:26:17+02	devel	geany-plugin-debugger	gtk3	geany-plugin-debugger-1.36
2020-10-17 02:25:48+02	2020-10-17 02:27:05+02	devel	geany-plugin-defineformat	gtk3	geany-plugin-defineformat-1.36
2020-10-17 02:26:17+02	2020-10-17 02:27:26+02	devel	geany-plugin-doc	gtk3	geany-plugin-doc-1.36
2020-10-17 02:27:05+02	2020-10-17 02:28:10+02	devel	geany-plugin-extrasel	gtk3	geany-plugin-extrasel-1.36
2020-10-17 02:27:26+02	2020-10-17 02:28:46+02	devel	geany-plugin-gendoc	gtk3	geany-plugin-gendoc-1.36
2020-10-17 02:28:11+02	2020-10-17 02:29:25+02	devel	geany-plugin-geniuspaste	gtk3	geany-plugin-geniuspaste-1.36
2020-10-17 02:28:46+02	2020-10-17 02:30:01+02	devel	geany-plugin-git-changebar	gtk3	geany-plugin-git-changebar-1.36_2
2020-10-17 02:29:26+02	2020-10-17 02:30:35+02	devel	geany-plugin-insertnum	gtk3	geany-plugin-insertnum-1.36
2020-10-17 02:30:02+02	2020-10-17 02:31:20+02	devel	geany-plugin-latex	gtk3	geany-plugin-latex-1.36
2020-10-17 02:30:35+02	2020-10-17 02:31:45+02	devel	geany-plugin-lineoperations	gtk3	geany-plugin-lineoperations-1.36
2020-10-17 02:31:21+02	2020-10-17 02:32:30+02	devel	geany-plugin-lipsum	gtk3	geany-plugin-lipsum-1.36
2020-10-17 02:31:45+02	2020-10-17 02:33:05+02	devel	geany-plugin-lua	gtk3	geany-plugin-lua-1.36
2020-10-17 02:32:31+02	2020-10-17 02:33:49+02	devel	geany-plugin-macro	gtk3	geany-plugin-macro-1.36
2020-10-17 02:33:06+02	2020-10-17 02:34:37+02	devel	geany-plugin-markdown	gtk3	geany-plugin-markdown-1.36
2020-10-17 02:33:49+02	2020-10-17 02:35:00+02	devel	geany-plugin-miniscript	gtk3	geany-plugin-miniscript-1.36
2020-10-17 02:34:38+02	2020-10-17 02:35:44+02	devel	geany-plugin-numberedbookmarks	gtk3	geany-plugin-numberedbookmarks-1.36
2020-10-17 02:35:01+02	2020-10-17 02:36:10+02	devel	geany-plugin-overview	gtk3	geany-plugin-overview-1.36
2020-10-17 02:35:44+02	2020-10-17 02:36:57+02	devel	geany-plugin-pairtaghighlighter	gtk3	geany-plugin-pairtaghighlighter-1.36
2020-10-17 02:36:11+02	2020-10-17 02:37:21+02	devel	geany-plugin-pg	gtk3	geany-plugin-pg-1.36
2020-10-17 02:36:58+02	2020-10-17 02:38:10+02	devel	geany-plugin-pohelper	gtk3	geany-plugin-pohelper-1.36
2020-10-17 02:37:22+02	2020-10-17 02:38:31+02	devel	geany-plugin-pretty-printer	gtk3	geany-plugin-pretty-printer-1.36
2020-10-17 02:38:11+02	2020-10-17 02:39:18+02	devel	geany-plugin-prj	gtk3	geany-plugin-prj-1.36
2020-10-17 02:38:32+02	2020-10-17 02:39:40+02	devel	geany-plugin-projectorganizer	gtk3	geany-plugin-projectorganizer-1.36
2020-10-17 02:39:20+02	2020-10-17 02:40:51+02	devel	geany-plugin-scope	gtk3	geany-plugin-scope-1.36
2020-10-17 02:39:40+02	2020-10-17 02:40:53+02	devel	geany-plugin-sendmail	gtk3	geany-plugin-sendmail-1.36
2020-10-17 02:40:52+02	2020-10-17 02:42:03+02	devel	geany-plugin-shiftcolumn	gtk3	geany-plugin-shiftcolumn-1.36
2020-10-17 02:40:53+02	2020-10-17 02:42:05+02	devel	geany-plugin-spellcheck	gtk3	geany-plugin-spellcheck-1.36
2020-10-17 02:42:04+02	2020-10-17 02:43:05+02	devel	geany-plugin-tableconvert	gtk3	geany-plugin-tableconvert-1.36
2020-10-17 02:42:06+02	2020-10-17 02:43:12+02	devel	geany-plugin-treebrowser	gtk3	geany-plugin-treebrowser-1.36
2020-10-17 02:43:06+02	2020-10-17 02:44:07+02	devel	geany-plugin-updatechecker	gtk3	geany-plugin-updatechecker-1.36
2020-10-17 02:43:12+02	2020-10-17 02:44:27+02	devel	geany-plugin-vc	gtk3	geany-plugin-vc-1.36
2020-10-17 02:44:08+02	2020-10-17 02:45:40+02	devel	geany-plugin-vimode	gtk3	geany-plugin-vimode-1.36
2020-10-17 02:44:28+02	2020-10-17 02:45:53+02	devel	geany-plugin-workbench	gtk3	geany-plugin-workbench-1.36_2
2020-10-17 02:45:40+02	2020-10-17 02:46:57+02	devel	geany-plugin-xmlsnippets	gtk3	geany-plugin-xmlsnippets-1.36
2020-10-17 02:45:53+02	2020-10-17 02:47:22+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.75.0
2020-10-17 02:47:23+02	2020-10-17 02:47:36+02	devel	rubygem-manpages	\N	rubygem-manpages-0.6.1
2020-10-17 02:47:36+02	2020-10-17 02:47:47+02	devel	rubygem-optimist	\N	rubygem-optimist-3.0.1
2020-10-17 02:47:47+02	2020-10-17 02:48:04+02	devel	rubygem-paint	\N	rubygem-paint-2.2.0
2020-10-17 02:59:19+02	2020-10-17 03:07:05+02	editors	pluma	\N	pluma-1.24.1
2020-10-17 03:07:05+02	2020-10-17 03:11:20+02	graphics	atril	\N	atril-1.22.3_10
2020-10-17 02:48:05+02	2020-10-17 03:28:20+02	editors	libreoffice	\N	libreoffice-7.0.2.2_2
2020-10-17 03:28:25+02	2020-10-17 03:37:53+02	graphics	seexpr	\N	seexpr-3.4.4.0
2020-10-17 03:11:23+02	2020-10-17 03:54:09+02	graphics	embree	\N	embree-3.12.1
2020-10-17 03:37:54+02	2020-10-17 03:54:30+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-17 03:54:32+02	2020-10-17 03:56:06+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-17 03:56:08+02	2020-10-17 03:57:18+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-17 03:57:18+02	2020-10-17 03:58:51+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-17 03:58:51+02	2020-10-17 03:59:34+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-10-17 03:59:35+02	2020-10-17 04:00:07+02	www	npm-node12	\N	npm-node12-6.14.8
2020-10-17 04:00:07+02	2020-10-17 04:00:20+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-10-17 03:54:09+02	2020-10-17 04:01:04+02	math	maxima	\N	maxima-5.44.0_5
2020-10-17 04:00:20+02	2020-10-17 04:01:38+02	x11	mate-base	\N	mate-base-1.24.1
2020-10-17 04:01:05+02	2020-10-17 04:03:56+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-17 04:01:38+02	2020-10-17 04:07:52+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-17 04:07:53+02	2020-10-17 04:10:34+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-17 04:03:57+02	2020-10-17 04:10:34+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-17 04:10:36+02	2020-10-17 04:12:54+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-17 04:12:55+02	2020-10-17 04:15:17+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-17 04:15:18+02	2020-10-17 04:21:06+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-16 22:27:26+02	2020-10-17 04:43:54+02	lang	rust	\N	rust-1.47.0
2020-10-17 04:43:56+02	2020-10-17 04:48:58+02	devel	rust-cbindgen	\N	rust-cbindgen-0.15.0_1
2020-10-17 04:21:06+02	2020-10-17 04:49:11+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-17 04:49:12+02	2020-10-17 04:50:05+02	devel	geany-plugins	gtk3	geany-plugins-1.36
2020-10-17 04:50:06+02	2020-10-17 04:50:26+02	devel	tig	\N	tig-2.5.1
2020-10-17 04:48:58+02	2020-10-17 04:57:57+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-17 04:50:27+02	2020-10-17 04:59:39+02	editors	codelite	\N	codelite-12.0_4
2020-10-17 04:57:58+02	2020-10-17 04:59:57+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.2.2
2020-10-17 04:59:40+02	2020-10-17 05:09:24+02	emulators	qemu	\N	qemu-4.2.1
2020-10-17 05:09:25+02	2020-10-17 05:11:53+02	games	alienarena	\N	alienarena-2013.766_9
2020-10-17 05:11:53+02	2020-10-17 05:12:06+02	games	rubygem-lolcat	\N	rubygem-lolcat-100.0.1
2020-10-17 04:59:57+02	2020-10-17 05:13:08+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-17 05:12:06+02	2020-10-17 05:20:44+02	graphics	blender	\N	blender-2.90.0_1
2020-10-17 05:20:45+02	2020-10-17 05:24:29+02	graphics	wings	\N	wings-2.2.5
2020-10-17 05:24:31+02	2020-10-17 05:29:20+02	mail	alpine	\N	alpine-2.24
2020-10-17 05:29:20+02	2020-10-17 05:31:21+02	math	wxmaxima	\N	wxmaxima-20.04.0_2
2020-10-17 05:31:21+02	2020-10-17 05:34:00+02	multimedia	obs-studio	\N	obs-studio-26.0.2
2020-10-17 05:34:01+02	2020-10-17 05:39:58+02	net	ntopng	\N	ntopng-4.0.d20200917,1
2020-10-17 05:39:59+02	2020-10-17 05:41:36+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-17 05:41:37+02	2020-10-17 05:42:37+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-17 05:42:37+02	2020-10-17 05:43:29+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-17 05:43:29+02	2020-10-17 05:44:25+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-17 05:44:25+02	2020-10-17 05:45:17+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-17 05:45:20+02	2020-10-17 05:46:11+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-17 05:46:12+02	2020-10-17 05:47:12+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-17 05:47:12+02	2020-10-17 05:48:03+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-17 05:48:04+02	2020-10-17 05:49:31+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-17 05:49:32+02	2020-10-17 05:50:28+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-17 05:50:29+02	2020-10-17 05:51:20+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-17 05:51:20+02	2020-10-17 05:54:24+02	security	clamav	\N	clamav-0.103.0,1
2020-10-17 05:54:25+02	2020-10-17 05:54:45+02	sysutils	flashrom	\N	flashrom-1.2
2020-10-17 05:54:45+02	2020-10-17 05:56:15+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-17 05:56:16+02	2020-10-17 05:57:22+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-17 04:10:36+02	2020-10-17 05:57:45+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-17 05:57:22+02	2020-10-17 05:58:18+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-10-17 05:57:48+02	2020-10-17 05:59:33+02	x11	mate	\N	mate-1.24.1
2020-10-17 05:13:09+02	2020-10-17 06:29:28+02	graphics	krita	\N	krita-4.4.0
2020-10-17 06:29:29+02	2020-10-17 06:44:45+02	local	E590T-localbase	\N	E590T-localbase-2020101600
2020-10-17 06:57:22+02	2020-10-17 06:58:19+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-17 06:57:22+02	2020-10-17 06:59:33+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-17 06:59:33+02	2020-10-17 06:59:41+02	graphics	drm-kmod	\N	drm-kmod-g20190710
2020-10-17 13:23:23+02	2020-10-17 13:23:51+02	security	ca_root_nss	\N	ca_root_nss-3.58
2020-10-17 13:23:23+02	2020-10-17 13:24:34+02	net	liveMedia	\N	liveMedia-2020.10.16,2
2020-10-17 13:24:34+02	2020-10-17 13:25:13+02	textproc	py-sphinx	py37	py37-sphinx-3.2.1,1
2020-10-17 13:25:16+02	2020-10-17 13:25:35+02	textproc	py-recommonmark	py37	py37-recommonmark-0.5.0_2
2020-10-17 13:23:52+02	2020-10-17 13:25:38+02	ftp	curl	\N	curl-7.73.0
2020-10-17 13:23:23+02	2020-10-17 13:26:56+02	security	nss	\N	nss-3.58
2020-10-17 13:25:36+02	2020-10-17 13:27:14+02	security	p11-kit	\N	p11-kit-0.23.21
2020-10-17 13:25:41+02	2020-10-17 13:28:25+02	devel	cmake	\N	cmake-3.18.4
2020-10-17 13:26:57+02	2020-10-17 13:31:37+02	net	qt5-network	\N	qt5-network-5.15.0
2020-10-17 13:28:26+02	2020-10-17 13:32:10+02	graphics	poppler	\N	poppler-20.10.0
2020-10-17 13:27:14+02	2020-10-17 13:32:41+02	security	gnutls	\N	gnutls-3.6.15
2020-10-17 13:32:42+02	2020-10-17 13:34:44+02	print	cups	\N	cups-2.3.3_1
2020-10-17 13:34:46+02	2020-10-17 13:41:10+02	print	ghostscript9-agpl-base	\N	ghostscript9-agpl-base-9.52_10
2020-10-17 13:32:10+02	2020-10-17 13:51:56+02	x11-toolkits	qt5-gui	\N	qt5-gui-5.15.0_1
2020-10-17 13:41:10+02	2020-10-17 13:53:09+02	print	texlive-base	\N	texlive-base-20150521_56
2020-10-17 13:51:57+02	2020-10-17 14:03:41+02	x11-toolkits	gtk30	\N	gtk3-3.24.23_1
2020-10-17 14:03:42+02	2020-10-17 14:11:44+02	x11-toolkits	gtk20	\N	gtk2-2.24.32
2020-10-17 14:11:45+02	2020-10-17 14:20:43+02	x11-toolkits	qt5-widgets	\N	qt5-widgets-5.15.0
2020-10-17 13:53:09+02	2020-10-17 14:22:54+02	print	texlive-texmf	\N	texlive-texmf-20150523_4
2020-10-17 14:22:55+02	2020-10-17 14:27:40+02	print	tex-formats	\N	tex-formats-20150521_2
2020-10-17 14:27:41+02	2020-10-17 14:37:34+02	print	tex-xmltex	\N	tex-xmltex-1.9_2
2020-10-17 14:20:44+02	2020-10-17 14:45:54+02	x11-toolkits	qt5-declarative	\N	qt5-declarative-5.15.0_1
2020-10-17 14:45:55+02	2020-10-17 14:46:28+02	net	glib-networking	\N	glib-networking-2.66.0
2020-10-17 14:37:35+02	2020-10-17 14:47:15+02	print	tex-jadetex	\N	tex-jadetex-3.13_3
2020-10-17 14:46:28+02	2020-10-17 14:47:46+02	devel	libsoup	\N	libsoup-2.72.0
2020-10-17 14:47:16+02	2020-10-17 14:49:48+02	textproc	docbook-utils	\N	docbook-utils-0.6.14_13
2020-10-17 14:49:49+02	2020-10-17 14:51:42+02	devel	qt5-help	\N	qt5-help-5.15.0
2020-10-17 14:51:44+02	2020-10-17 14:53:55+02	print	qt5-printsupport	\N	qt5-printsupport-5.15.0
2020-10-17 14:47:47+02	2020-10-17 14:56:40+02	print	tex-dvipsk	\N	tex-dvipsk-5.995_2
2020-10-17 14:53:56+02	2020-10-17 15:09:54+02	multimedia	ffmpeg	\N	ffmpeg-4.3.1_3,1
2020-10-17 15:09:55+02	2020-10-17 15:11:20+02	net	geoclue	\N	geoclue-2.5.5
2020-10-17 14:56:41+02	2020-10-17 15:12:22+02	devel	doxygen	\N	doxygen-1.8.15_3,2
2020-10-17 15:11:21+02	2020-10-17 15:12:39+02	x11-toolkits	vte	\N	vte-0.28.2_3
2020-10-17 15:12:39+02	2020-10-17 15:14:38+02	devel	geany	gtk3	geany-1.36
2020-10-17 15:14:39+02	2020-10-17 15:15:49+02	x11-toolkits	qt5-uiplugin	\N	qt5-uiplugin-5.15.0
2020-10-17 15:15:50+02	2020-10-17 15:17:54+02	devel	gconf2	\N	gconf2-3.2.6_5
2020-10-17 15:17:54+02	2020-10-17 15:19:01+02	x11	qt5-x11extras	\N	qt5-x11extras-5.15.0
2020-10-17 15:19:02+02	2020-10-17 15:21:03+02	graphics	qt5-opengl	\N	qt5-opengl-5.15.0
2020-10-17 15:21:04+02	2020-10-17 15:21:47+02	print	ghostscript9-agpl-x11	\N	ghostscript9-agpl-x11-9.52
2020-10-17 15:12:23+02	2020-10-17 15:30:18+02	www	webkit2-gtk3	\N	webkit2-gtk3-2.30.1
2020-10-17 15:21:47+02	2020-10-17 15:32:44+02	graphics	ImageMagick7	\N	ImageMagick7-7.0.10.24
2020-10-17 15:30:20+02	2020-10-17 15:33:05+02	graphics	qt5-svg	\N	qt5-svg-5.15.0
2020-10-17 15:33:08+02	2020-10-17 15:35:39+02	audio	libcanberra	\N	libcanberra-0.30_5
2020-10-17 15:35:39+02	2020-10-17 15:40:38+02	devel	qscintilla2-qt5	\N	qscintilla2-qt5-2.11.5
2020-10-17 15:40:39+02	2020-10-17 15:42:14+02	audio	libcanberra-gtk3	\N	libcanberra-gtk3-0.30_5
2020-10-17 15:32:46+02	2020-10-17 15:43:50+02	x11-toolkits	wxgtk30	\N	wx30-gtk3-3.0.5.1
2020-10-17 15:43:52+02	2020-10-17 15:47:11+02	security	gnupg	\N	gnupg-2.2.23
2020-10-17 15:47:12+02	2020-10-17 15:48:03+02	security	gpgme	\N	gpgme-1.14.0
2020-10-17 15:42:15+02	2020-10-17 15:49:00+02	devel	py-qt5-core	py37	py37-qt5-core-5.15.0
2020-10-17 15:48:04+02	2020-10-17 15:50:12+02	x11-toolkits	vte3	\N	vte3-0.62.1
2020-10-17 15:49:01+02	2020-10-17 15:51:37+02	devel	qt5-assistant	\N	qt5-assistant-5.15.0
2020-10-17 15:50:13+02	2020-10-17 15:51:37+02	devel	qt5-uitools	\N	qt5-uitools-5.15.0
2020-10-17 15:51:38+02	2020-10-17 15:52:12+02	ftp	php74-curl	\N	php74-curl-7.4.11
2020-10-17 15:51:38+02	2020-10-17 15:52:43+02	graphics	pecl-imagick-im7	php74	php74-pecl-imagick-im7-3.4.4
2020-10-17 15:52:13+02	2020-10-17 15:53:25+02	graphics	qt5-imageformats	\N	qt5-imageformats-5.15.0
2020-10-17 15:52:44+02	2020-10-17 15:54:42+02	x11	mate-desktop	\N	mate-desktop-1.24.1
2020-10-17 15:53:25+02	2020-10-17 15:54:43+02	devel	kf5-kconfig	\N	kf5-kconfig-5.75.0
2020-10-17 15:54:43+02	2020-10-17 15:56:21+02	net-mgmt	icingaweb2	php74	icingaweb2-php74-2.8.2
2020-10-17 15:56:21+02	2020-10-17 15:58:11+02	devel	kf5-ki18n	\N	kf5-ki18n-5.75.0
2020-10-17 15:58:11+02	2020-10-17 16:02:36+02	security	gcr	\N	gcr-3.28.0_1
2020-10-17 16:02:37+02	2020-10-17 16:04:20+02	www	qt5-webchannel	\N	qt5-webchannel-5.15.0
2020-10-17 15:54:43+02	2020-10-17 16:04:51+02	devel	qt5-designer	\N	qt5-designer-5.15.0
2020-10-17 16:04:51+02	2020-10-17 16:08:36+02	devel	git	default	git-2.28.0
2020-10-17 16:08:39+02	2020-10-17 16:10:48+02	devel	kf5-kdoctools	\N	kf5-kdoctools-5.75.0
2020-10-17 16:10:48+02	2020-10-17 16:20:59+02	devel	qt5-location	\N	qt5-location-5.15.0
2020-10-17 16:21:01+02	2020-10-17 16:22:13+02	x11	kf5-kwindowsystem	\N	kf5-kwindowsystem-5.75.0
2020-10-17 16:22:13+02	2020-10-17 16:23:23+02	accessibility	qt5-speech	\N	qt5-speech-5.15.0
2020-10-17 16:04:21+02	2020-10-17 16:23:38+02	x11-toolkits	py-qt5-gui	py37	py37-qt5-gui-5.15.0
2020-10-17 16:23:24+02	2020-10-17 16:24:14+02	multimedia	gstreamer1-libav	\N	gstreamer1-libav-1.16.2
2020-10-17 20:32:48+02	2020-10-17 20:33:37+02	devel	libglade2	\N	libglade2-2.6.4_10
2020-10-17 16:23:38+02	2020-10-17 16:25:17+02	x11-toolkits	kf5-kwidgetsaddons	\N	kf5-kwidgetsaddons-5.75.0
2020-10-17 16:24:14+02	2020-10-17 16:25:30+02	devel	kf5-kcrash	\N	kf5-kcrash-5.75.0
2020-10-17 16:25:18+02	2020-10-17 16:26:20+02	devel	kf5-kdbusaddons	\N	kf5-kdbusaddons-5.75.0
2020-10-17 16:26:25+02	2020-10-17 16:27:22+02	multimedia	gstreamer1-plugins-core	\N	gstreamer1-plugins-core-1.16
2020-10-17 16:25:30+02	2020-10-17 16:27:28+02	graphics	poppler-glib	\N	poppler-glib-20.10.0
2020-10-17 16:27:32+02	2020-10-17 16:28:54+02	sysutils	polkit-qt	\N	polkit-qt-1-0.113.0_8
2020-10-17 16:27:23+02	2020-10-17 16:28:57+02	net	avahi-gtk3	\N	avahi-gtk3-0.7_1
2020-10-17 16:28:55+02	2020-10-17 16:31:56+02	comms	qt5-sensors	\N	qt5-sensors-5.15.0
2020-10-17 16:31:57+02	2020-10-17 16:33:22+02	devel	kf5-kauth	\N	kf5-kauth-5.75.0
2020-10-17 16:33:23+02	2020-10-17 16:35:16+02	devel	kf5-kservice	\N	kf5-kservice-5.75.0
2020-10-17 16:28:58+02	2020-10-17 16:35:45+02	databases	mysql57-client	\N	mysql57-client-5.7.31_1
2020-10-17 16:35:16+02	2020-10-17 16:37:30+02	net	py-qt5-network	py37	py37-qt5-network-5.15.0
2020-10-17 16:35:46+02	2020-10-17 16:37:38+02	net	remmina	\N	remmina-1.3.9
2020-10-17 16:37:31+02	2020-10-17 16:38:07+02	textproc	raptor2	\N	raptor2-2.0.15_14
2020-10-17 16:37:38+02	2020-10-17 16:38:44+02	x11-toolkits	kf5-kguiaddons	\N	kf5-kguiaddons-5.75.0
2020-10-17 16:38:44+02	2020-10-17 16:39:59+02	devel	librest	\N	rest-0.7.93_1
2020-10-17 16:40:00+02	2020-10-17 16:42:56+02	math	gnuplot	\N	gnuplot-5.4.0
2020-10-17 16:38:07+02	2020-10-17 16:44:32+02	x11-toolkits	py-qt5-widgets	py37	py37-qt5-widgets-5.15.0
2020-10-17 16:44:33+02	2020-10-17 16:44:53+02	net	liboauth	\N	liboauth-1.0.3_4
2020-10-17 16:44:53+02	2020-10-17 16:46:43+02	x11-toolkits	kf5-kconfigwidgets	\N	kf5-kconfigwidgets-5.75.0
2020-10-17 16:46:44+02	2020-10-17 16:48:09+02	x11-toolkits	kf5-kitemviews	\N	kf5-kitemviews-5.75.0
2020-10-17 16:48:15+02	2020-10-17 16:50:05+02	x11-toolkits	libwnck3	\N	libwnck3-3.36.0
2020-10-17 16:42:57+02	2020-10-17 16:50:24+02	multimedia	qt5-multimedia	\N	qt5-multimedia-5.15.0
2020-10-17 16:50:25+02	2020-10-17 16:52:43+02	net	gnome-online-accounts	\N	gnome-online-accounts-3.36.0
2020-10-17 16:52:43+02	2020-10-17 16:54:07+02	net	uhttpmock	\N	uhttpmock-0.5.3
2020-10-17 16:54:08+02	2020-10-17 16:55:36+02	x11-themes	kf5-kiconthemes	\N	kf5-kiconthemes-5.75.0
2020-10-17 16:55:37+02	2020-10-17 16:58:07+02	devel	libgdata	\N	libgdata-0.17.13
2020-10-17 16:58:08+02	2020-10-17 17:00:35+02	devel	libpeas	\N	libpeas-1.22.0
2020-10-17 17:00:35+02	2020-10-17 17:07:19+02	net	libmateweather	\N	libmateweather-1.24.1
2020-10-17 17:07:20+02	2020-10-17 17:08:56+02	textproc	kf5-sonnet	\N	kf5-sonnet-5.75.0
2020-10-17 17:08:56+02	2020-10-17 17:16:33+02	textproc	qt5-xmlpatterns	\N	qt5-xmlpatterns-5.15.0_1
2020-10-17 16:50:06+02	2020-10-17 17:53:44+02	math	vtk8	\N	vtk8-8.2.0
2020-10-17 17:16:34+02	2020-10-17 18:37:51+02	www	qt5-webkit	\N	qt5-webkit-5.212.0.a4_3
2020-10-17 18:37:53+02	2020-10-17 18:39:09+02	x11-toolkits	gtk-layer-shell	\N	gtk-layer-shell-0.3.0
2020-10-17 18:39:09+02	2020-10-17 18:40:09+02	x11-toolkits	kf5-kcompletion	\N	kf5-kcompletion-5.75.0
2020-10-17 17:53:53+02	2020-10-17 18:40:49+02	cad	opencascade	\N	opencascade-7.4.0_7
2020-10-17 18:40:10+02	2020-10-17 18:40:53+02	databases	fpc-mysql	\N	fpc-mysql-3.0.4_4
2020-10-17 18:40:54+02	2020-10-17 18:42:17+02	devel	geany-plugin-utilslib	gtk3	geany-plugin-utilslib-1.36
2020-10-17 18:40:50+02	2020-10-17 18:42:20+02	devel	appstream-glib	\N	appstream-glib-0.7.8_2
2020-10-17 18:42:20+02	2020-10-17 18:43:05+02	mail	gmime30	\N	gmime30-3.2.7
2020-10-17 18:42:17+02	2020-10-17 18:43:21+02	devel	libdbusmenu-qt	\N	libdbusmenu-qt5-0.9.3.160420160218_10
2020-10-17 18:43:06+02	2020-10-17 18:43:38+02	multimedia	libquvi09	\N	libquvi09-0.9.4_4
2020-10-17 18:43:22+02	2020-10-17 18:44:23+02	multimedia	phonon	\N	phonon-qt5-4.11.1
2020-10-17 18:44:23+02	2020-10-17 18:45:43+02	print	py-qt5-printsupport	py37	py37-qt5-printsupport-5.15.0
2020-10-17 18:45:43+02	2020-10-17 18:46:17+02	security	gpgme-cpp	\N	gpgme-cpp-1.14.0
2020-10-17 18:43:39+02	2020-10-17 19:02:34+02	net	samba413	\N	samba413-4.13.0
2020-10-17 19:02:34+02	2020-10-17 19:04:26+02	x11	kf5-kglobalaccel	\N	kf5-kglobalaccel-5.75.0
2020-10-17 19:04:27+02	2020-10-17 19:09:30+02	x11	mate-panel	\N	mate-panel-1.24.1_1
2020-10-17 19:09:31+02	2020-10-17 19:11:16+02	x11-toolkits	kf5-attica	\N	kf5-attica-5.75.0
2020-10-17 19:11:17+02	2020-10-17 19:13:18+02	x11-toolkits	kf5-ktextwidgets	\N	kf5-ktextwidgets-5.75.0
2020-10-17 19:13:19+02	2020-10-17 19:24:39+02	x11-toolkits	py-wxPython40	py37	py37-wxPython40-4.0.7_1
2020-10-17 19:24:40+02	2020-10-17 19:25:44+02	audio	chromaprint	\N	chromaprint-1.5.0
2020-10-17 19:25:45+02	2020-10-17 19:53:38+02	cad	kicad	\N	kicad-5.1.7,2
2020-10-17 13:31:38+02	2020-10-17 19:54:47+02	lang	rust	\N	rust-1.47.0
2020-10-17 19:53:39+02	2020-10-17 19:55:33+02	devel	kf5-knotifications	\N	kf5-knotifications-5.75.0
2020-10-17 19:55:34+02	2020-10-17 19:58:07+02	graphics	libQGLViewer	\N	libQGLViewer-2.7.2
2020-10-17 19:58:07+02	2020-10-17 19:59:35+02	graphics	poppler-utils	\N	poppler-utils-20.10.0
2020-10-17 19:54:50+02	2020-10-17 20:04:09+02	devel	rust-cbindgen	\N	rust-cbindgen-0.15.0_1
2020-10-17 20:04:10+02	2020-10-17 20:05:13+02	multimedia	totem-pl-parser	\N	totem-pl-parser-3.26.5
2020-10-17 19:59:35+02	2020-10-17 20:06:34+02	graphics	qt5-3d	\N	qt5-3d-5.15.0
2020-10-17 20:05:13+02	2020-10-17 20:06:37+02	net	geocode-glib	\N	geocode-glib-3.26.2
2020-10-17 20:06:35+02	2020-10-17 20:07:32+02	security	gpgme-qt5	\N	gpgme-qt5-1.14.0
2020-10-17 20:06:37+02	2020-10-17 20:07:43+02	security	pinentry-gnome3	\N	pinentry-gnome3-1.1.0
2020-10-17 20:07:35+02	2020-10-17 20:08:43+02	security	pinentry-qt5	\N	pinentry-qt5-1.1.0_1
2020-10-17 20:08:44+02	2020-10-17 20:10:31+02	x11	libmatekbd	\N	libmatekbd-1.24.1
2020-10-17 20:07:43+02	2020-10-17 20:12:30+02	www	node	\N	node-14.13.1
2020-10-17 20:10:31+02	2020-10-17 20:13:03+02	x11-fm	caja	\N	caja-1.24.0
2020-10-17 20:12:31+02	2020-10-17 20:14:42+02	x11-toolkits	gtkmm30	\N	gtkmm30-3.24.2
2020-10-17 20:13:03+02	2020-10-17 20:14:50+02	x11-toolkits	kf5-kxmlgui	\N	kf5-kxmlgui-5.75.0
2020-10-17 20:14:43+02	2020-10-17 20:15:51+02	x11-toolkits	libunique3	\N	libunique-3.0.2_3
2020-10-17 20:14:50+02	2020-10-17 20:16:34+02	x11-toolkits	wxgtk28-common	\N	wx28-gtk2-common-2.8.12_8
2020-10-17 20:15:52+02	2020-10-17 20:17:07+02	audio	gstreamer1-plugins-chromaprint	\N	gstreamer1-plugins-chromaprint-1.16.2
2020-10-17 20:16:36+02	2020-10-17 20:18:41+02	devel	gvfs	\N	gvfs-1.46.1_1
2020-10-17 20:17:08+02	2020-10-17 20:18:48+02	devel	kf5-kbookmarks	\N	kf5-kbookmarks-5.75.0
2020-10-17 20:18:50+02	2020-10-17 20:20:06+02	lang	py-qt5-qml	py37	py37-qt5-qml-5.15.0
2020-10-17 20:18:41+02	2020-10-17 20:20:06+02	devel	kf5-solid	\N	kf5-solid-5.75.0
2020-10-17 20:20:06+02	2020-10-17 20:20:38+02	mail	gmime26	\N	gmime26-2.6.23_1
2020-10-17 20:20:41+02	2020-10-17 20:22:12+02	math	octomap	\N	octomap-1.9.5.190
2020-10-17 20:20:06+02	2020-10-17 20:22:40+02	mail	mailutils	\N	mailutils-3.10_1
2020-10-17 20:22:15+02	2020-10-17 20:23:06+02	net	grilo	\N	grilo-0.3.13
2020-10-17 20:22:40+02	2020-10-17 20:24:39+02	net	libgweather	\N	libgweather-3.36.1
2020-10-17 20:23:06+02	2020-10-17 20:25:25+02	security	gnome-keyring	\N	gnome-keyring-3.36.0
2020-10-17 20:25:26+02	2020-10-17 20:26:50+02	sysutils	tracker	\N	tracker-2.3.4
2020-10-17 20:24:40+02	2020-10-17 20:26:56+02	sysutils	kf5-kwallet	\N	kf5-kwallet-5.75.0
2020-10-17 20:26:55+02	2020-10-17 20:27:33+02	www	qt5-websockets	\N	qt5-websockets-5.15.0
2020-10-17 20:27:34+02	2020-10-17 20:28:45+02	x11	kf5-kded	\N	kf5-kded-5.75.0
2020-10-17 20:26:56+02	2020-10-17 20:28:48+02	x11	gnome-desktop	\N	gnome-desktop-3.38.0
2020-10-17 20:28:48+02	2020-10-17 20:30:06+02	x11-toolkits	kf5-kjobwidgets	\N	kf5-kjobwidgets-5.75.0
2020-10-17 20:28:45+02	2020-10-17 20:30:20+02	x11	zenity	\N	zenity-3.32.0
2020-10-17 20:30:06+02	2020-10-17 20:31:39+02	x11-toolkits	qt5-quickcontrols	\N	qt5-quickcontrols-5.15.0
2020-10-17 20:30:21+02	2020-10-17 20:32:39+02	x11-toolkits	qt5-quickcontrols2	\N	qt5-quickcontrols2-5.15.0
2020-10-17 20:31:39+02	2020-10-17 20:32:44+02	devel	gstreamer1-plugins-soup	\N	gstreamer1-plugins-soup-1.16.2
2020-10-17 20:33:38+02	2020-10-17 20:34:57+02	devel	py-libpeas	py37	py37-libpeas-1.22.0
2020-10-17 20:32:40+02	2020-10-17 20:38:05+02	devel	kf5-kio	\N	kf5-kio-5.75.0
2020-10-17 20:38:05+02	2020-10-17 20:38:30+02	ftp	fpc-libcurl	\N	fpc-libcurl-3.0.4_4
2020-10-17 20:38:32+02	2020-10-17 20:39:18+02	graphics	clutter-gtk3	\N	clutter-gtk3-1.8.4
2020-10-17 20:34:58+02	2020-10-17 20:41:11+02	editors	emacs	full	emacs-27.1_1,3
2020-10-17 20:39:19+02	2020-10-17 20:41:14+02	graphics	gegl	\N	gegl-0.4.26_6
2020-10-17 20:41:15+02	2020-10-17 20:42:00+02	graphics	qt5-graphicaleffects	\N	qt5-graphicaleffects-5.15.0
2020-10-17 20:42:00+02	2020-10-17 20:42:54+02	math	fcl05	\N	fcl05-0.5.0
2020-10-17 18:46:17+02	2020-10-17 20:42:55+02	www	qt5-webengine	\N	qt5-webengine-5.15.0_4
2020-10-17 20:42:59+02	2020-10-17 20:43:24+02	net	gssdp	\N	gssdp-1.2.3
2020-10-17 20:42:55+02	2020-10-17 20:44:11+02	net	grilo-plugins	\N	grilo-plugins-0.3.12
2020-10-17 20:43:25+02	2020-10-17 20:44:20+02	net-mgmt	icingaweb2-module-ipl	php74	icingaweb2-module-ipl-php74-0.5.0
2020-10-17 20:44:12+02	2020-10-17 20:44:53+02	print	cups-filters	\N	cups-filters-1.28.4
2020-10-17 20:41:12+02	2020-10-17 20:45:15+02	graphics	opencv	\N	opencv-3.4.1_36
2020-10-17 20:45:15+02	2020-10-17 20:46:44+02	sysutils	gnome-settings-daemon	\N	gnome-settings-daemon-3.38.0_1
2020-10-17 20:44:21+02	2020-10-17 20:47:32+02	print	fontforge	\N	fontforge-20190801_2
2020-10-17 20:46:44+02	2020-10-17 20:48:27+02	sysutils	mate-settings-daemon	\N	mate-settings-daemon-1.24.1
2020-10-17 20:47:32+02	2020-10-17 20:48:43+02	sysutils	zeitgeist	\N	zeitgeist-1.0.2
2020-10-17 20:48:43+02	2020-10-17 20:48:55+02	textproc	py-numpydoc	py37	py37-numpydoc-0.9.1
2020-10-17 20:48:28+02	2020-10-17 20:49:22+02	textproc	gtkspell3	\N	gtkspell3-3.0.10
2020-10-17 20:48:55+02	2020-10-17 20:49:28+02	textproc	rasqal	\N	rasqal-0.9.33_1
2020-10-17 20:49:28+02	2020-10-17 20:51:59+02	www	node12	\N	node12-12.19.0
2020-10-17 20:44:53+02	2020-10-17 20:52:44+02	print	tex-luatex	\N	tex-luatex-0.80.0_21
2020-10-17 20:51:59+02	2020-10-17 20:53:10+02	x11	mate-session-manager	\N	mate-session-manager-1.24.1
2020-10-17 20:52:44+02	2020-10-17 20:54:02+02	x11	yelp	\N	yelp-3.38.0
2020-10-17 20:53:11+02	2020-10-17 20:54:11+02	x11-themes	gtk-engines2	\N	gtk-engines2-2.20.2_3
2020-10-17 20:54:02+02	2020-10-17 20:55:05+02	x11-themes	gtk-murrine-engine	\N	gtk-murrine-engine-0.98.2_5
2020-10-17 20:54:11+02	2020-10-17 20:55:48+02	x11-toolkits	gtksourceview3	\N	gtksourceview3-3.24.11
2020-10-17 20:55:48+02	2020-10-17 20:57:47+02	x11-toolkits	wxgtk28-contrib-common	\N	wx28-gtk2-contrib-common-2.8.12_8
2020-10-17 20:55:05+02	2020-10-17 20:58:47+02	x11-toolkits	wxgtk28	\N	wx28-gtk2-2.8.12_8
2020-10-17 20:57:48+02	2020-10-17 20:59:55+02	x11-wm	marco	\N	marco-1.24.1
2020-10-17 20:58:49+02	2020-10-17 21:00:12+02	archivers	gnome-autoar	\N	gnome-autoar-0.2.4
2020-10-17 21:00:12+02	2020-10-17 21:01:52+02	deskutils	mate-notification-daemon	\N	mate-notification-daemon-1.24.1
2020-10-17 20:59:56+02	2020-10-17 21:02:36+02	deskutils	gucharmap	\N	gucharmap-11.0.1_1
2020-10-17 21:02:37+02	2020-10-17 21:03:57+02	devel	git-lfs	\N	git-lfs-2.12.0
2020-10-17 21:03:58+02	2020-10-17 21:05:17+02	devel	qt5-remoteobjects	\N	qt5-remoteobjects-5.15.0
2020-10-17 21:01:53+02	2020-10-17 21:05:21+02	deskutils	spice-gtk	\N	spice-gtk-0.37
2020-10-17 21:05:22+02	2020-10-17 21:06:28+02	devel	qt5-scxml	\N	qt5-scxml-5.15.0
2020-10-17 21:05:17+02	2020-10-17 21:06:31+02	devel	qt5-scripttools	\N	qt5-scripttools-5.15.0
2020-10-17 21:06:32+02	2020-10-17 21:11:48+02	graphics	ImageMagick6	\N	ImageMagick6-6.9.11.6,1
2020-10-17 21:06:28+02	2020-10-17 21:12:42+02	devel	shiboken2	py37	py37-shiboken2-5.15.1
2020-10-17 21:11:48+02	2020-10-17 21:14:29+02	graphics	frei0r-plugins-opencv	\N	frei0r-plugins-opencv-1.6.1_1
2020-10-17 21:14:31+02	2020-10-17 21:16:40+02	graphics	poppler-qt5	\N	poppler-qt5-20.10.0
2020-10-17 21:16:41+02	2020-10-17 21:18:11+02	graphics	py-qt5-svg	py37	py37-qt5-svg-5.15.0
2020-10-17 21:12:42+02	2020-10-17 21:25:37+02	graphics	gimp-app	\N	gimp-app-2.10.20_7,1
2020-10-17 20:49:26+02	2020-10-17 21:36:13+02	www	chromium	\N	chromium-84.0.4147.135
2020-10-17 21:36:16+02	2020-10-17 21:37:22+02	math	py-python-fcl	py37	py37-python-fcl-0.0.12
2020-10-17 21:18:18+02	2020-10-17 21:40:23+02	graphics	py-scikit-image	py37	py37-scikit-image-0.17.2
2020-10-17 21:40:24+02	2020-10-17 21:41:46+02	multimedia	mlt	\N	mlt-6.22.1
2020-10-17 21:41:47+02	2020-10-17 21:45:01+02	multimedia	totem	\N	totem-3.38.0_1
2020-10-17 21:45:02+02	2020-10-17 21:45:59+02	net	freerdp	\N	freerdp-2.2.0_1
2020-10-17 21:25:39+02	2020-10-17 21:46:13+02	math	R	\N	R-4.0.3
2020-10-17 21:46:00+02	2020-10-17 21:46:27+02	net	gupnp	\N	gupnp-1.2.4
2020-10-17 21:37:24+02	2020-10-17 21:46:28+02	misc	mate-user-guide	\N	mate-user-guide-1.24.0
2020-10-17 21:46:27+02	2020-10-17 21:47:22+02	net	remmina-plugin-exec	\N	remmina-plugin-exec-1.3.9
2020-10-17 21:46:13+02	2020-10-17 21:47:26+02	net	libcmis	\N	libcmis-0.5.2_1
2020-10-17 21:46:28+02	2020-10-17 21:47:51+02	net-mgmt	icingaweb2-module-pdfexport	php74	icingaweb2-module-pdfexport-php74-0.9.0_2
2020-10-17 21:47:53+02	2020-10-17 21:48:17+02	print	libspectre	\N	libspectre-0.2.9
2020-10-17 21:47:24+02	2020-10-17 21:48:18+02	net-mgmt	icingaweb2-module-reactbundle	php74	icingaweb2-module-reactbundle-php74-0.7.0_1
2020-10-17 21:48:17+02	2020-10-17 21:50:33+02	print	mftrace	\N	mftrace-1.2.18_2
2020-10-17 21:47:26+02	2020-10-17 21:51:16+02	print	gutenprint	\N	gutenprint-5.3.3
2020-10-17 21:50:34+02	2020-10-17 21:51:37+02	security	xmlsec1	\N	xmlsec1-1.2.29
2020-10-17 21:51:38+02	2020-10-17 21:52:31+02	sysutils	mate-polkit	\N	mate-polkit-1.24.0
2020-10-17 21:51:17+02	2020-10-17 21:54:27+02	sysutils	mate-control-center	\N	mate-control-center-1.24.1
2020-10-17 21:52:32+02	2020-10-17 21:54:46+02	textproc	dblatex	\N	dblatex-0.3.11
2020-10-17 21:48:18+02	2020-10-17 21:54:57+02	print	tex-dvipdfmx	\N	tex-dvipdfmx-20150315_1
2020-10-17 21:54:28+02	2020-10-17 21:55:05+02	textproc	redland	\N	redland-1.0.17_4
2020-10-17 21:54:46+02	2020-10-17 21:55:27+02	www	py-qt5-webchannel	py37	py37-qt5-webchannel-5.15.0
2020-10-17 21:55:06+02	2020-10-17 21:55:28+02	www	youtube_dl	\N	youtube_dl-2020.09.20
2020-10-17 21:54:57+02	2020-10-17 21:55:56+02	www	py-qt5-webkit	py37	py37-qt5-webkit-5.15.0
2020-10-17 21:55:29+02	2020-10-17 21:56:14+02	x11-toolkits	fpc-gtk2	\N	fpc-gtk2-3.0.4_4
2020-10-17 21:55:27+02	2020-10-17 21:56:25+02	x11-themes	mate-themes	\N	mate-themes-3.22.14
2020-10-17 21:55:57+02	2020-10-17 21:56:53+02	x11-toolkits	py-qt5-quick	py37	py37-qt5-quick-5.15.0
2020-10-17 21:56:15+02	2020-10-17 21:57:52+02	x11-toolkits	qt5-charts	\N	qt5-charts-5.15.0
2020-10-17 21:56:25+02	2020-10-17 21:57:53+02	x11-toolkits	qt5-datavis3d	\N	qt5-datavis3d-5.15.0
2020-10-17 21:56:53+02	2020-10-17 21:57:54+02	x11-toolkits	qt5-gamepad	\N	qt5-gamepad-5.15.0_1
2020-10-17 21:57:52+02	2020-10-17 21:59:06+02	x11-toolkits	wxgtk28-contrib	\N	wx28-gtk2-contrib-2.8.12_8
2020-10-17 21:57:55+02	2020-10-17 22:00:31+02	archivers	engrampa	\N	engrampa-1.24.1
2020-10-17 21:59:06+02	2020-10-17 22:00:58+02	audio	mate-media	\N	mate-media-1.24.1
2020-10-17 22:00:31+02	2020-10-17 22:02:16+02	cad	uranium	\N	Uranium-4.7.1,1
2020-10-17 22:01:02+02	2020-10-17 22:02:49+02	deskutils	caja-extensions	\N	caja-extensions-1.24.1
2020-10-17 22:02:50+02	2020-10-17 22:03:52+02	deskutils	mozo	\N	mozo-1.24.0
2020-10-17 21:57:54+02	2020-10-17 22:04:11+02	x11-toolkits	wxgtk31	\N	wx31-gtk3-3.1.4
2020-10-17 22:03:52+02	2020-10-17 22:05:10+02	devel	R-cran-rstudioapi	\N	R-cran-rstudioapi-0.11_1
2020-10-17 22:04:12+02	2020-10-17 22:05:35+02	devel	geany-plugin-addons	gtk3	geany-plugin-addons-1.36
2020-10-17 22:02:17+02	2020-10-17 22:05:55+02	deskutils	mate-utils	\N	mate-utils-1.24.0
2020-10-17 22:05:11+02	2020-10-17 22:06:15+02	devel	geany-plugin-autoclose	gtk3	geany-plugin-autoclose-1.36
2020-10-17 22:05:36+02	2020-10-17 22:06:30+02	devel	geany-plugin-automark	gtk3	geany-plugin-automark-1.36
2020-10-17 22:05:56+02	2020-10-17 22:06:53+02	devel	geany-plugin-codenav	gtk3	geany-plugin-codenav-1.36
2020-10-17 22:06:15+02	2020-10-17 22:07:09+02	devel	geany-plugin-commander	gtk3	geany-plugin-commander-1.36
2020-10-17 22:06:30+02	2020-10-17 22:07:24+02	devel	geany-plugin-ctags	gtk3	geany-plugin-ctags-1.36
2020-10-17 22:06:54+02	2020-10-17 22:07:51+02	devel	geany-plugin-debugger	gtk3	geany-plugin-debugger-1.36
2020-10-17 22:07:10+02	2020-10-17 22:08:05+02	devel	geany-plugin-defineformat	gtk3	geany-plugin-defineformat-1.36
2020-10-17 22:07:24+02	2020-10-17 22:08:24+02	devel	geany-plugin-doc	gtk3	geany-plugin-doc-1.36
2020-10-17 22:07:51+02	2020-10-17 22:08:45+02	devel	geany-plugin-extrasel	gtk3	geany-plugin-extrasel-1.36
2020-10-17 22:08:06+02	2020-10-17 22:09:03+02	devel	geany-plugin-gendoc	gtk3	geany-plugin-gendoc-1.36
2020-10-17 22:08:24+02	2020-10-17 22:09:18+02	devel	geany-plugin-geniuspaste	gtk3	geany-plugin-geniuspaste-1.36
2020-10-17 22:08:45+02	2020-10-17 22:09:39+02	devel	geany-plugin-git-changebar	gtk3	geany-plugin-git-changebar-1.36_2
2020-10-17 22:09:04+02	2020-10-17 22:09:55+02	devel	geany-plugin-insertnum	gtk3	geany-plugin-insertnum-1.36
2020-10-17 22:09:18+02	2020-10-17 22:10:13+02	devel	geany-plugin-latex	gtk3	geany-plugin-latex-1.36
2020-10-17 22:09:40+02	2020-10-17 22:10:37+02	devel	geany-plugin-lineoperations	gtk3	geany-plugin-lineoperations-1.36
2020-10-17 22:09:58+02	2020-10-17 22:10:51+02	devel	geany-plugin-lipsum	gtk3	geany-plugin-lipsum-1.36
2020-10-17 22:10:14+02	2020-10-17 22:11:12+02	devel	geany-plugin-lua	gtk3	geany-plugin-lua-1.36
2020-10-17 22:10:38+02	2020-10-17 22:11:29+02	devel	geany-plugin-macro	gtk3	geany-plugin-macro-1.36
2020-10-17 22:10:52+02	2020-10-17 22:11:55+02	devel	geany-plugin-markdown	gtk3	geany-plugin-markdown-1.36
2020-10-17 22:11:12+02	2020-10-17 22:12:08+02	devel	geany-plugin-miniscript	gtk3	geany-plugin-miniscript-1.36
2020-10-17 22:11:30+02	2020-10-17 22:12:29+02	devel	geany-plugin-numberedbookmarks	gtk3	geany-plugin-numberedbookmarks-1.36
2020-10-17 22:11:55+02	2020-10-17 22:12:49+02	devel	geany-plugin-overview	gtk3	geany-plugin-overview-1.36
2020-10-17 22:12:08+02	2020-10-17 22:13:01+02	devel	geany-plugin-pairtaghighlighter	gtk3	geany-plugin-pairtaghighlighter-1.36
2020-10-17 22:12:29+02	2020-10-17 22:13:24+02	devel	geany-plugin-pg	gtk3	geany-plugin-pg-1.36
2020-10-17 22:12:49+02	2020-10-17 22:13:43+02	devel	geany-plugin-pohelper	gtk3	geany-plugin-pohelper-1.36
2020-10-17 22:13:01+02	2020-10-17 22:13:55+02	devel	geany-plugin-pretty-printer	gtk3	geany-plugin-pretty-printer-1.36
2020-10-17 22:13:25+02	2020-10-17 22:14:20+02	devel	geany-plugin-prj	gtk3	geany-plugin-prj-1.36
2020-10-17 22:13:44+02	2020-10-17 22:14:34+02	devel	geany-plugin-projectorganizer	gtk3	geany-plugin-projectorganizer-1.36
2020-10-17 22:13:55+02	2020-10-17 22:14:57+02	devel	geany-plugin-scope	gtk3	geany-plugin-scope-1.36
2020-10-17 22:14:20+02	2020-10-17 22:15:12+02	devel	geany-plugin-sendmail	gtk3	geany-plugin-sendmail-1.36
2020-10-17 22:14:35+02	2020-10-17 22:15:30+02	devel	geany-plugin-shiftcolumn	gtk3	geany-plugin-shiftcolumn-1.36
2020-10-17 22:14:57+02	2020-10-17 22:15:49+02	devel	geany-plugin-spellcheck	gtk3	geany-plugin-spellcheck-1.36
2020-10-17 22:15:14+02	2020-10-17 22:16:05+02	devel	geany-plugin-tableconvert	gtk3	geany-plugin-tableconvert-1.36
2020-10-17 22:15:30+02	2020-10-17 22:16:24+02	devel	geany-plugin-treebrowser	gtk3	geany-plugin-treebrowser-1.36
2020-10-17 22:15:50+02	2020-10-17 22:16:43+02	devel	geany-plugin-updatechecker	gtk3	geany-plugin-updatechecker-1.36
2020-10-17 22:16:06+02	2020-10-17 22:17:00+02	devel	geany-plugin-vc	gtk3	geany-plugin-vc-1.36
2020-10-17 22:16:24+02	2020-10-17 22:17:20+02	devel	geany-plugin-vimode	gtk3	geany-plugin-vimode-1.36
2020-10-17 22:16:43+02	2020-10-17 22:17:40+02	devel	geany-plugin-workbench	gtk3	geany-plugin-workbench-1.36_2
2020-10-17 22:17:00+02	2020-10-17 22:17:56+02	devel	geany-plugin-xmlsnippets	gtk3	geany-plugin-xmlsnippets-1.36
2020-10-17 22:17:21+02	2020-10-17 22:18:04+02	devel	kf5-kitemmodels	\N	kf5-kitemmodels-5.75.0
2020-10-17 22:17:41+02	2020-10-17 22:18:12+02	devel	mercurial	py37	mercurial-5.5_1
2020-10-17 22:17:56+02	2020-10-17 22:18:47+02	devel	py-qt5-qscintilla2	py37	py37-qt5-qscintilla2-2.11.5
2020-10-17 22:18:05+02	2020-10-17 22:19:13+02	devel	py-trimesh	py37	py37-trimesh-3.5.25
2020-10-17 22:26:28+02	2020-10-17 22:32:48+02	editors	pluma	\N	pluma-1.24.1
2020-10-17 22:32:49+02	2020-10-17 22:36:43+02	graphics	atril	\N	atril-1.22.3_10
2020-10-17 22:36:44+02	2020-10-17 22:39:03+02	graphics	eom	\N	eom-1.24.1
2020-10-17 22:39:04+02	2020-10-17 22:40:17+02	graphics	frei0r-plugins	\N	frei0r-plugins-1.6.1_1
2020-10-17 22:40:17+02	2020-10-17 22:41:13+02	graphics	goocanvas2	\N	goocanvas2-2.0.4
2020-10-17 22:41:13+02	2020-10-17 22:42:09+02	graphics	seexpr	\N	seexpr-3.4.4.0
2020-10-17 22:18:47+02	2020-10-17 22:45:40+02	editors	lazarus	\N	lazarus-gtk2-2.0.8
2020-10-17 22:19:13+02	2020-10-17 22:46:47+02	editors	libreoffice	\N	libreoffice-7.0.2.2_2
2020-10-17 22:46:48+02	2020-10-17 22:48:46+02	math	mate-calc	\N	mate-calc-1.24.1
2020-10-17 22:42:10+02	2020-10-17 22:50:57+02	japanese	tex-ptex	\N	ja-tex-ptex-3.6_1
2020-10-17 22:50:57+02	2020-10-17 22:51:05+02	misc	freebsd-release-manifests	\N	freebsd-release-manifests-20201017
2020-10-17 22:51:05+02	2020-10-17 22:52:38+02	multimedia	libopenshot	py37	py37-libopenshot-0.2.5.172
2020-10-17 22:45:41+02	2020-10-17 22:53:53+02	lang	erlang-wx	\N	erlang-wx-21.3.8.18_1,1
2020-10-17 22:52:39+02	2020-10-17 22:54:29+02	multimedia	mpv	\N	mpv-0.32.0_5,1
2020-10-17 22:48:47+02	2020-10-17 22:54:30+02	math	maxima	\N	maxima-5.44.0_5
2020-10-17 22:54:31+02	2020-10-17 22:55:20+02	net	remmina-plugin-nx	\N	remmina-plugin-nx-1.3.9
2020-10-17 22:54:29+02	2020-10-17 22:55:33+02	multimedia	webvfx-qt5	\N	webvfx-qt5-0.4.4.20160823_12
2020-10-17 22:55:21+02	2020-10-17 22:56:23+02	net	remmina-plugin-rdp	\N	remmina-plugin-rdp-1.3.9
2020-10-17 22:55:34+02	2020-10-17 22:56:40+02	net	remmina-plugin-secret	\N	remmina-plugin-secret-1.3.9
2020-10-17 22:56:24+02	2020-10-17 22:57:20+02	net	remmina-plugin-spice	\N	remmina-plugin-spice-1.3.9
2020-10-17 22:56:40+02	2020-10-17 22:57:27+02	net	remmina-plugin-st	\N	remmina-plugin-st-1.3.9
2020-10-17 22:53:54+02	2020-10-17 22:57:56+02	multimedia	vlc	\N	vlc-3.0.11_5,4
2020-10-17 22:57:20+02	2020-10-17 22:58:05+02	net	remmina-plugin-vnc	\N	remmina-plugin-vnc-1.3.9
2020-10-17 22:57:28+02	2020-10-17 22:58:24+02	net	remmina-plugin-www	\N	remmina-plugin-www-1.3.9
2020-10-17 22:58:05+02	2020-10-17 22:58:53+02	net-mgmt	icingaweb2-module-director	php74	icingaweb2-module-director-php74-1.6.2_2
2020-10-17 22:57:56+02	2020-10-17 22:59:07+02	net	remmina-plugin-xdmcp	\N	remmina-plugin-xdmcp-1.3.9
2020-10-17 22:58:24+02	2020-10-17 22:59:34+02	net-mgmt	icingaweb2-module-reporting	php74	icingaweb2-module-reporting-php74-0.9.2_1
2020-10-17 22:58:54+02	2020-10-17 22:59:56+02	print	gimp-gutenprint	\N	gimp-gutenprint-5.3.3
2020-10-17 22:59:57+02	2020-10-17 23:01:53+02	print	tex-xdvik	\N	tex-xdvik-22.87_4
2020-10-17 22:59:07+02	2020-10-17 23:05:29+02	print	lilypond	\N	lilypond-2.20.0
2020-10-17 22:59:35+02	2020-10-17 23:06:12+02	print	tex-aleph	\N	tex-aleph-1.15.2.1.r.4_4
2020-10-17 23:06:13+02	2020-10-17 23:06:39+02	print	transfig	\N	transfig-3.2.5e_4
2020-10-17 23:06:40+02	2020-10-17 23:07:22+02	sysutils	libgksu	\N	libgksu-2.0.12_5
2020-10-17 23:07:23+02	2020-10-17 23:08:58+02	sysutils	mate-power-manager	\N	mate-power-manager-1.24.2
2020-10-17 23:01:54+02	2020-10-17 23:09:10+02	print	tex-xetex	\N	tex-xetex-0.99992_37
2020-10-17 23:09:11+02	2020-10-17 23:09:39+02	textproc	liblrdf	\N	liblrdf-0.6.1
2020-10-17 23:09:39+02	2020-10-17 23:10:41+02	textproc	py-qt5-xml	py37	py37-qt5-xml-5.15.0
2020-10-17 23:08:58+02	2020-10-17 23:10:49+02	sysutils	mate-system-monitor	\N	mate-system-monitor-1.24.1
2020-10-17 23:10:49+02	2020-10-17 23:11:12+02	www	npm-node12	\N	npm-node12-6.14.8
2020-10-17 23:10:42+02	2020-10-17 23:11:21+02	textproc	py-qt5-xmlpatterns	py37	py37-qt5-xmlpatterns-5.15.0
2020-10-17 23:11:13+02	2020-10-17 23:12:14+02	www	py-qt5-webengine	py37	py37-qt5-webengine-5.15.0
2020-10-17 23:12:15+02	2020-10-17 23:12:25+02	www	yarn-node12	\N	yarn-node12-1.22.4_1
2020-10-17 23:11:21+02	2020-10-17 23:12:34+02	www	py-qt5-webkitwidgets	py37	py37-qt5-webkitwidgets-5.15.0
2020-10-17 23:12:34+02	2020-10-17 23:13:37+02	x11	mate-base	\N	mate-base-1.24.1
2020-10-17 23:13:38+02	2020-10-17 23:15:10+02	x11	mate-screensaver	\N	mate-screensaver-1.24.1
2020-10-17 23:15:10+02	2020-10-17 23:17:22+02	x11	mate-terminal	\N	mate-terminal-1.24.1
2020-10-17 23:12:25+02	2020-10-17 23:18:11+02	x11	mate-applets	\N	mate-applets-1.24.1
2020-10-17 23:17:23+02	2020-10-17 23:19:18+02	x11-fm	nautilus	\N	nautilus-3.36.3
2020-10-17 23:18:11+02	2020-10-17 23:19:19+02	x11-toolkits	gdl	\N	gdl-3.28.0
2020-10-17 23:19:18+02	2020-10-17 23:19:50+02	x11-toolkits	gtkglarea2	\N	gtkglarea-2.0.1_10
2020-10-17 23:19:19+02	2020-10-17 23:21:05+02	x11-toolkits	gtkmm24	\N	gtkmm24-2.24.5_1
2020-10-17 23:19:51+02	2020-10-17 23:21:07+02	x11-toolkits	soqt	\N	SoQt-1.6.0,1
2020-10-17 23:21:07+02	2020-10-17 23:26:26+02	audio	rosegarden	\N	rosegarden-20.06_1
2020-10-17 23:21:06+02	2020-10-17 23:27:40+02	astro	stellarium	\N	stellarium-0.20.3
2020-10-17 23:05:31+02	2020-10-17 23:28:06+02	print	texlive-docs	\N	texlive-docs-20150523
2020-10-17 23:26:27+02	2020-10-17 23:28:27+02	cad	cura	\N	Cura-4.7.1,2
2020-10-17 23:28:06+02	2020-10-17 23:30:13+02	cad	kicad-library-footprints	\N	kicad-library-footprints-5.1.7
2020-10-17 23:30:14+02	2020-10-17 23:32:15+02	cad	kicad-library-symbols	\N	kicad-library-symbols-5.1.7
2020-10-17 23:27:41+02	2020-10-17 23:32:54+02	cad	kicad-doc	\N	kicad-doc-5.1.7
2020-10-17 23:32:16+02	2020-10-17 23:33:44+02	cad	kicad-library-templates	\N	kicad-library-tmpl-5.1.7
2020-10-17 23:33:45+02	2020-10-17 23:37:03+02	deskutils	calibre	\N	calibre-4.23.0_3
2020-10-17 23:32:55+02	2020-10-17 23:37:09+02	databases	pgadmin3	\N	pgadmin3-1.22.2_5
2020-10-17 23:37:04+02	2020-10-17 23:38:03+02	deskutils	sysctlview	\N	sysctlview-2.0.1
2020-10-17 23:38:03+02	2020-10-17 23:40:06+02	devel	bzr	py27	bzr-2.7.0.6622
2020-10-17 23:40:06+02	2020-10-17 23:54:35+02	devel	codeblocks	\N	codeblocks-20.03
2020-10-17 23:54:36+02	2020-10-17 23:56:25+02	devel	distel	full	distel-emacs27-4.1.1_5
2020-10-17 23:37:09+02	2020-10-18 00:08:43+02	devel	RStudio	\N	RStudio-1.3.1093
2020-10-18 00:08:45+02	2020-10-18 00:10:14+02	devel	geany-plugins	gtk3	geany-plugins-1.36
2020-10-18 00:10:15+02	2020-10-18 00:10:54+02	devel	tig	\N	tig-2.5.1
2020-10-18 00:10:56+02	2020-10-18 00:12:13+02	devel	tortoisehg	py37	tortoisehg-5.5.1_2
2020-10-18 00:12:14+02	2020-10-18 00:25:06+02	editors	codelite	\N	codelite-12.0_4
2020-10-18 00:25:09+02	2020-10-18 00:27:10+02	editors	libreoffice-nb	\N	nb-libreoffice-7.0.2.2
2020-10-18 00:27:11+02	2020-10-18 00:28:53+02	editors	vim	\N	vim-8.2.1558
2020-10-17 23:56:25+02	2020-10-18 00:28:56+02	devel	electron7	\N	electron7-7.3.1_1
2020-10-18 00:28:59+02	2020-10-18 00:37:08+02	emulators	wine	\N	wine-5.0.2_1,1
2020-10-18 00:28:54+02	2020-10-18 00:39:24+02	emulators	qemu	\N	qemu-4.2.1
2020-10-18 00:39:25+02	2020-10-18 00:41:54+02	games	alienarena	\N	alienarena-2013.766_9
2020-10-18 00:41:55+02	2020-10-18 00:42:52+02	games	xcowsay	\N	xcowsay-1.5.1
2020-10-18 00:37:09+02	2020-10-18 00:54:43+02	ftp	filezilla	\N	filezilla-3.40.0_5
2020-10-18 00:42:53+02	2020-10-18 00:55:04+02	graphics	blender	\N	blender-2.90.0_1
2020-10-18 00:55:04+02	2020-10-18 00:56:16+02	graphics	gimp	\N	gimp-2.10.20_1,2
2020-10-18 00:54:45+02	2020-10-18 00:57:00+02	graphics	dia	\N	dia-0.97.3_1,1
2020-10-18 00:56:16+02	2020-10-18 01:05:52+02	graphics	inkscape	\N	inkscape-1.0.1_1
2020-10-18 01:05:53+02	2020-10-18 01:09:05+02	graphics	py-pivy	py37	py37-pivy-0.6.5
2020-10-18 00:57:03+02	2020-10-18 01:09:59+02	graphics	krita	\N	krita-4.4.0
2020-10-18 01:10:00+02	2020-10-18 01:11:15+02	graphics	xpdf	\N	xpdf-4.02,1
2020-10-18 01:11:15+02	2020-10-18 01:12:26+02	lang	csharp-mode.el	full	csharp-mode.el-emacs27-0.9.2_2
2020-10-18 01:09:05+02	2020-10-18 01:12:27+02	graphics	wings	\N	wings-2.2.5
2020-10-17 23:28:28+02	2020-10-18 01:28:38+02	cad	kicad-library-packages3d	\N	kicad-library-packages3d-5.1.7
2020-10-18 01:12:27+02	2020-10-18 01:38:22+02	lang	erlang-runtime22	\N	erlang-runtime22-22.3.4.11
2020-10-18 01:38:23+02	2020-10-18 01:40:30+02	math	wxmaxima	\N	wxmaxima-20.04.0_2
2020-10-18 01:40:31+02	2020-10-18 01:41:34+02	multimedia	celluloid	\N	celluloid-0.20
2020-10-18 01:41:36+02	2020-10-18 01:44:25+02	multimedia	obs-studio	\N	obs-studio-26.0.2
2020-10-18 01:12:27+02	2020-10-18 01:46:28+02	lang	erlang-runtime23	\N	erlang-runtime23-23.1.1
2020-10-18 01:44:26+02	2020-10-18 01:47:00+02	multimedia	openshot	py37	openshot-2.5.1.369_1
2020-10-18 01:47:00+02	2020-10-18 01:47:30+02	net	geoipupdate	\N	geoipupdate-4.3.0_1
2020-10-18 01:47:30+02	2020-10-18 01:49:33+02	net	ntopng	\N	ntopng-4.0.d20200917,1
2020-10-18 01:46:28+02	2020-10-18 01:49:35+02	multimedia	shotcut	\N	shotcut-20.09.27
2020-10-18 01:49:34+02	2020-10-18 01:50:03+02	net	rdesktop	\N	rdesktop-1.9.0
2020-10-18 01:49:36+02	2020-10-18 01:51:12+02	net	remmina-plugins	\N	remmina-plugins-1.3.9
2020-10-18 01:51:14+02	2020-10-18 01:52:36+02	net-mgmt	etherape	\N	etherape-0.9.18
2020-10-18 01:52:37+02	2020-10-18 01:54:25+02	net-mgmt	icingaweb2-module-businessprocess	php74	icingaweb2-module-businessprocess-php74-2.2.0_2
2020-10-18 01:54:25+02	2020-10-18 01:55:34+02	net-mgmt	icingaweb2-module-cube	php74	icingaweb2-module-cube-php74-1.1.0_1
2020-10-18 01:55:35+02	2020-10-18 01:56:27+02	net-mgmt	icingaweb2-module-elasticsearch	php74	icingaweb2-module-elasticsearch-php74-0.9.0_3
2020-10-18 01:50:04+02	2020-10-18 01:57:00+02	net	wireshark	\N	wireshark-3.2.7
2020-10-18 01:56:28+02	2020-10-18 01:57:35+02	net-mgmt	icingaweb2-module-fileshipper	php74	icingaweb2-module-fileshipper-php74-1.1.0_3
2020-10-18 01:57:01+02	2020-10-18 01:58:09+02	net-mgmt	icingaweb2-module-generictts	php74	icingaweb2-module-generictts-php74-2.0.0_3
2020-10-18 01:57:36+02	2020-10-18 01:58:34+02	net-mgmt	icingaweb2-module-grafana	php74	icingaweb2-module-grafana-php74-1.3.6_1
2020-10-18 01:58:10+02	2020-10-18 01:58:59+02	net-mgmt	icingaweb2-module-graphite	php74	icingaweb2-module-graphite-php74-1.1.0_3
2020-10-18 01:59:00+02	2020-10-18 01:59:55+02	net-mgmt	icingaweb2-module-map	php74	icingaweb2-module-map-php74-1.1.0_3
2020-10-18 01:58:35+02	2020-10-18 01:59:58+02	net-mgmt	icingaweb2-module-idoreports	php74	icingaweb2-module-idoreports-php74-0.9.0_2
2020-10-18 01:59:59+02	2020-10-18 02:00:17+02	net-mgmt	ndpmon	\N	ndpmon-1.4.0_6
2020-10-18 01:59:55+02	2020-10-18 02:00:40+02	net-mgmt	icingaweb2-module-vsphere	php74	icingaweb2-module-vsphere-php74-1.1.0_4
2020-10-18 02:00:17+02	2020-10-18 02:01:02+02	ports-mgmt	octopkg	\N	octopkg-0.2.0_7
2020-10-18 02:00:40+02	2020-10-18 02:01:04+02	ports-mgmt	poudriere	\N	poudriere-3.3.4
2020-10-18 02:01:04+02	2020-10-18 02:01:32+02	print	fig2dev	\N	fig2dev-3.2.7_1
2020-10-18 02:01:32+02	2020-10-18 02:02:12+02	print	gv	\N	gv-3.7.4_5
2020-10-18 02:01:03+02	2020-10-18 02:03:21+02	print	auctex	full	auctex-emacs27-12.2_1
2020-10-18 02:02:12+02	2020-10-18 02:04:44+02	print	lyx	\N	lyx-2.3.4.2_1
2020-10-18 01:28:39+02	2020-10-18 02:05:50+02	lang	mono	\N	mono-5.10.1.57_3
2020-10-18 02:05:52+02	2020-10-18 02:06:56+02	security	clamav	\N	clamav-0.103.0,1
2020-10-18 02:06:57+02	2020-10-18 02:08:11+02	security	openconnect-gui	\N	openconnect-gui-1.4.1_4
2020-10-18 02:08:13+02	2020-10-18 02:09:27+02	sysutils	cpu-x	\N	cpu-x-4.0.1
2020-10-18 02:09:27+02	2020-10-18 02:10:01+02	sysutils	gkrellm2	\N	gkrellm2-2.3.11
2020-10-18 02:04:45+02	2020-10-18 02:10:01+02	print	texlive-full	\N	texlive-full-20150521
2020-10-18 02:03:22+02	2020-10-18 02:10:25+02	print	scribus-devel	\N	scribus-devel-1.5.5_20
2020-10-18 02:10:02+02	2020-10-18 02:10:55+02	sysutils	gsmartcontrol	\N	gsmartcontrol-1.1.3_3
2020-10-18 02:10:26+02	2020-10-18 02:11:18+02	sysutils	tarsnap-gui	\N	tarsnap-gui-1.0.2_3
2020-10-18 02:10:01+02	2020-10-18 02:11:23+02	sysutils	gksu	\N	gksu-2.0.2_7
2020-10-18 02:11:23+02	2020-10-18 02:12:27+02	www	mod_security	\N	ap24-mod_security-2.9.3
2020-10-18 02:12:28+02	2020-10-18 02:13:16+02	x11	cool-retro-term	\N	cool-retro-term-1.1.1_1
2020-10-18 02:11:19+02	2020-10-18 02:13:22+02	www	mattermost-server	\N	mattermost-server-5.26.1
2020-10-18 02:13:17+02	2020-10-18 02:14:40+02	x11	mate	\N	mate-1.24.1
2020-10-18 02:13:24+02	2020-10-18 02:15:03+02	x11	xscreensaver	\N	xscreensaver-5.44
2020-10-18 02:15:03+02	2020-10-18 02:16:01+02	x11-fm	fsv2	\N	fsv2-1.1.0_1
2020-10-18 02:14:41+02	2020-10-18 02:20:57+02	x11-fm	doublecmd	gtk2	doublecmd-gtk2-0.9.9
2020-10-18 02:30:30+02	2020-10-18 02:31:36+02	graphics	xfig	\N	xfig-3.2.7a
2020-10-18 02:16:02+02	2020-10-18 02:48:54+02	editors	vscode	\N	vscode-1.46.1
2020-10-18 02:10:55+02	2020-10-18 03:00:13+02	www	firefox	\N	firefox-82.0,2
2020-10-18 03:00:15+02	2020-10-18 03:22:30+02	local	E590T-localbase	\N	E590T-localbase-2020101700
2020-10-18 03:43:41+02	2020-10-18 03:44:59+02	emulators	virtualbox-ose-kmod	\N	virtualbox-ose-kmod-5.2.44_3
2020-10-18 03:43:41+02	2020-10-18 03:51:17+02	graphics	drm-current-kmod	\N	drm-current-kmod-5.4.62.g20201003
2020-10-18 03:51:17+02	2020-10-18 03:51:25+02	graphics	drm-kmod	\N	drm-kmod-g20190710
\.


--
-- Name: portsbuildstats portsbuildstats_pk; Type: CONSTRAINT; Schema: public; Owner: trond
--

ALTER TABLE ONLY public.portsbuildstats
    ADD CONSTRAINT portsbuildstats_pk PRIMARY KEY (start, finished, category, portname, pkgname);


--
-- PostgreSQL database dump complete
--

